html {
    font-size: 14px;
}


.grad-background {
    box-shadow: 0px 2px 4px -1px rgba(0, 0, 0, 0.2), 0px 4px 5px 0px rgba(0, 0, 0, 0.14), 0px 1px 10px 0px rgba(0, 0, 0, 0.12);
    padding-left: 10px;
    color: #fff;
    background-color: rgb(25, 118, 210);
}

.lm-Widget .lm-Panel .lm-Collapse-contents {
    padding-left: 5px;
    padding-right: 5px;
}

.bottom-shadow {
    box-shadow: 0px 2px 4px -1px rgba(0, 0, 0, 0.2), 0px 4px 5px 0px rgba(0, 0, 0, 0.14), 0px 1px 10px 0px rgba(0, 0, 0, 0.12)
}

.full-width {
    width: 100%
}