define(["font-awesome/css/font-awesome.css"],(function(__WEBPACK_EXTERNAL_MODULE__1504__){return function(e){function t(t){for(var n,i,o=t[0],a=t[1],s=0,u=[];s<o.length;s++)i=o[s],Object.prototype.hasOwnProperty.call(r,i)&&r[i]&&u.push(r[i][0]),r[i]=0;for(n in a)Object.prototype.hasOwnProperty.call(a,n)&&(e[n]=a[n]);for(l&&l(t);u.length;)u.shift()()}var n={},r={0:0};function i(t){if(n[t])return n[t].exports;var r=n[t]={i:t,l:!1,exports:{}};return e[t].call(r.exports,r,r.exports,i),r.l=!0,r.exports}i.e=function(e){var t=[],n=r[e];if(0!==n)if(n)t.push(n[2]);else{var o=new Promise((function(t,i){n=r[e]=[t,i]}));t.push(n[2]=o);var a,s=document.createElement("script");s.charset="utf-8",s.timeout=120,i.nc&&s.setAttribute("nonce",i.nc),s.src=function(e){return i.p+""+e+".index.js"}(e);var l=new Error;a=function(t){s.onerror=s.onload=null,clearTimeout(u);var n=r[e];if(0!==n){if(n){var i=t&&("load"===t.type?"missing":t.type),o=t&&t.target&&t.target.src;l.message="Loading chunk "+e+" failed.\n("+i+": "+o+")",l.name="ChunkLoadError",l.type=i,l.request=o,n[1](l)}r[e]=void 0}};var u=setTimeout((function(){a({type:"timeout",target:s})}),12e4);s.onerror=s.onload=a,document.head.appendChild(s)}return Promise.all(t)},i.m=e,i.c=n,i.d=function(e,t,n){i.o(e,t)||Object.defineProperty(e,t,{enumerable:!0,get:n})},i.r=function(e){"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})},i.t=function(e,t){if(1&t&&(e=i(e)),8&t)return e;if(4&t&&"object"==typeof e&&e&&e.__esModule)return e;var n=Object.create(null);if(i.r(n),Object.defineProperty(n,"default",{enumerable:!0,value:e}),2&t&&"string"!=typeof e)for(var r in e)i.d(n,r,function(t){return e[t]}.bind(null,r));return n},i.n=function(e){var t=e&&e.__esModule?function(){return e.default}:function(){return e};return i.d(t,"a",t),t},i.o=function(e,t){return Object.prototype.hasOwnProperty.call(e,t)},i.p="",i.oe=function(e){throw console.error(e),e};var o=window.webpackJsonp=window.webpackJsonp||[],a=o.push.bind(o);o.push=t,o=o.slice();for(var s=0;s<o.length;s++)t(o[s]);var l=a;return i(i.s=1339)}([function(e,t,n){"use strict";e.exports=n(1450)},function(e,t,n){"use strict";function r(){return(r=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e}).apply(this,arguments)}n.d(t,"a",(function(){return r}))},function(e,t,n){"use strict";n.r(t),n.d(t,"__extends",(function(){return i})),n.d(t,"__assign",(function(){return o})),n.d(t,"__rest",(function(){return a})),n.d(t,"__decorate",(function(){return s})),n.d(t,"__param",(function(){return l})),n.d(t,"__metadata",(function(){return u})),n.d(t,"__awaiter",(function(){return c})),n.d(t,"__generator",(function(){return d})),n.d(t,"__createBinding",(function(){return h})),n.d(t,"__exportStar",(function(){return f})),n.d(t,"__values",(function(){return p})),n.d(t,"__read",(function(){return g})),n.d(t,"__spread",(function(){return v})),n.d(t,"__spreadArrays",(function(){return m})),n.d(t,"__await",(function(){return y})),n.d(t,"__asyncGenerator",(function(){return b})),n.d(t,"__asyncDelegator",(function(){return C})),n.d(t,"__asyncValues",(function(){return A})),n.d(t,"__makeTemplateObject",(function(){return _})),n.d(t,"__importStar",(function(){return I})),n.d(t,"__importDefault",(function(){return x})),n.d(t,"__classPrivateFieldGet",(function(){return w})),n.d(t,"__classPrivateFieldSet",(function(){return M}));
/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
var r=function(e,t){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(e,t)};function i(e,t){function n(){this.constructor=e}r(e,t),e.prototype=null===t?Object.create(t):(n.prototype=t.prototype,new n)}var o=function(){return(o=Object.assign||function(e){for(var t,n=1,r=arguments.length;n<r;n++)for(var i in t=arguments[n])Object.prototype.hasOwnProperty.call(t,i)&&(e[i]=t[i]);return e}).apply(this,arguments)};function a(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(null!=e&&"function"==typeof Object.getOwnPropertySymbols){var i=0;for(r=Object.getOwnPropertySymbols(e);i<r.length;i++)t.indexOf(r[i])<0&&Object.prototype.propertyIsEnumerable.call(e,r[i])&&(n[r[i]]=e[r[i]])}return n}function s(e,t,n,r){var i,o=arguments.length,a=o<3?t:null===r?r=Object.getOwnPropertyDescriptor(t,n):r;if("object"==typeof Reflect&&"function"==typeof Reflect.decorate)a=Reflect.decorate(e,t,n,r);else for(var s=e.length-1;s>=0;s--)(i=e[s])&&(a=(o<3?i(a):o>3?i(t,n,a):i(t,n))||a);return o>3&&a&&Object.defineProperty(t,n,a),a}function l(e,t){return function(n,r){t(n,r,e)}}function u(e,t){if("object"==typeof Reflect&&"function"==typeof Reflect.metadata)return Reflect.metadata(e,t)}function c(e,t,n,r){return new(n||(n=Promise))((function(i,o){function a(e){try{l(r.next(e))}catch(e){o(e)}}function s(e){try{l(r.throw(e))}catch(e){o(e)}}function l(e){var t;e.done?i(e.value):(t=e.value,t instanceof n?t:new n((function(e){e(t)}))).then(a,s)}l((r=r.apply(e,t||[])).next())}))}function d(e,t){var n,r,i,o,a={label:0,sent:function(){if(1&i[0])throw i[1];return i[1]},trys:[],ops:[]};return o={next:s(0),throw:s(1),return:s(2)},"function"==typeof Symbol&&(o[Symbol.iterator]=function(){return this}),o;function s(o){return function(s){return function(o){if(n)throw new TypeError("Generator is already executing.");for(;a;)try{if(n=1,r&&(i=2&o[0]?r.return:o[0]?r.throw||((i=r.return)&&i.call(r),0):r.next)&&!(i=i.call(r,o[1])).done)return i;switch(r=0,i&&(o=[2&o[0],i.value]),o[0]){case 0:case 1:i=o;break;case 4:return a.label++,{value:o[1],done:!1};case 5:a.label++,r=o[1],o=[0];continue;case 7:o=a.ops.pop(),a.trys.pop();continue;default:if(!(i=a.trys,(i=i.length>0&&i[i.length-1])||6!==o[0]&&2!==o[0])){a=0;continue}if(3===o[0]&&(!i||o[1]>i[0]&&o[1]<i[3])){a.label=o[1];break}if(6===o[0]&&a.label<i[1]){a.label=i[1],i=o;break}if(i&&a.label<i[2]){a.label=i[2],a.ops.push(o);break}i[2]&&a.ops.pop(),a.trys.pop();continue}o=t.call(e,a)}catch(e){o=[6,e],r=0}finally{n=i=0}if(5&o[0])throw o[1];return{value:o[0]?o[1]:void 0,done:!0}}([o,s])}}}function h(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}function f(e,t){for(var n in e)"default"===n||t.hasOwnProperty(n)||(t[n]=e[n])}function p(e){var t="function"==typeof Symbol&&Symbol.iterator,n=t&&e[t],r=0;if(n)return n.call(e);if(e&&"number"==typeof e.length)return{next:function(){return e&&r>=e.length&&(e=void 0),{value:e&&e[r++],done:!e}}};throw new TypeError(t?"Object is not iterable.":"Symbol.iterator is not defined.")}function g(e,t){var n="function"==typeof Symbol&&e[Symbol.iterator];if(!n)return e;var r,i,o=n.call(e),a=[];try{for(;(void 0===t||t-- >0)&&!(r=o.next()).done;)a.push(r.value)}catch(e){i={error:e}}finally{try{r&&!r.done&&(n=o.return)&&n.call(o)}finally{if(i)throw i.error}}return a}function v(){for(var e=[],t=0;t<arguments.length;t++)e=e.concat(g(arguments[t]));return e}function m(){for(var e=0,t=0,n=arguments.length;t<n;t++)e+=arguments[t].length;var r=Array(e),i=0;for(t=0;t<n;t++)for(var o=arguments[t],a=0,s=o.length;a<s;a++,i++)r[i]=o[a];return r}function y(e){return this instanceof y?(this.v=e,this):new y(e)}function b(e,t,n){if(!Symbol.asyncIterator)throw new TypeError("Symbol.asyncIterator is not defined.");var r,i=n.apply(e,t||[]),o=[];return r={},a("next"),a("throw"),a("return"),r[Symbol.asyncIterator]=function(){return this},r;function a(e){i[e]&&(r[e]=function(t){return new Promise((function(n,r){o.push([e,t,n,r])>1||s(e,t)}))})}function s(e,t){try{(n=i[e](t)).value instanceof y?Promise.resolve(n.value.v).then(l,u):c(o[0][2],n)}catch(e){c(o[0][3],e)}var n}function l(e){s("next",e)}function u(e){s("throw",e)}function c(e,t){e(t),o.shift(),o.length&&s(o[0][0],o[0][1])}}function C(e){var t,n;return t={},r("next"),r("throw",(function(e){throw e})),r("return"),t[Symbol.iterator]=function(){return this},t;function r(r,i){t[r]=e[r]?function(t){return(n=!n)?{value:y(e[r](t)),done:"return"===r}:i?i(t):t}:i}}function A(e){if(!Symbol.asyncIterator)throw new TypeError("Symbol.asyncIterator is not defined.");var t,n=e[Symbol.asyncIterator];return n?n.call(e):(e=p(e),t={},r("next"),r("throw"),r("return"),t[Symbol.asyncIterator]=function(){return this},t);function r(n){t[n]=e[n]&&function(t){return new Promise((function(r,i){(function(e,t,n,r){Promise.resolve(r).then((function(t){e({value:t,done:n})}),t)})(r,i,(t=e[n](t)).done,t.value)}))}}}function _(e,t){return Object.defineProperty?Object.defineProperty(e,"raw",{value:t}):e.raw=t,e}function I(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)Object.hasOwnProperty.call(e,n)&&(t[n]=e[n]);return t.default=e,t}function x(e){return e&&e.__esModule?e:{default:e}}function w(e,t){if(!t.has(e))throw new TypeError("attempted to get private field on non-instance");return t.get(e)}function M(e,t,n){if(!t.has(e))throw new TypeError("attempted to set private field on non-instance");return t.set(e,n),n}},function(e,t,n){e.exports=n(1477)()},function(e,t,n){"use strict";function r(e){var t,n,i="";if("string"==typeof e||"number"==typeof e)i+=e;else if("object"==typeof e)if(Array.isArray(e))for(t=0;t<e.length;t++)e[t]&&(n=r(e[t]))&&(i&&(i+=" "),i+=n);else for(t in e)e[t]&&(i&&(i+=" "),i+=t);return i}t.a=function(){for(var e,t,n=0,i="";n<arguments.length;)(e=arguments[n++])&&(t=r(e))&&(i&&(i+=" "),i+=t);return i}},function(e,t,n){"use strict";n.d(t,"a",(function(){return i}));var r=n(92);function i(e,t){if(null==e)return{};var n,i,o=Object(r.a)(e,t);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(e,n)&&(o[n]=e[n])}return o}},function(e,t,n){"use strict";var r;function i(e){return"function"==typeof e.iter?e.iter():new p(e)}function o(e){return new g(e)}function a(e){return new v(e)}function s(e){return new m(e)}function l(e){return new y(e)}function u(e,t){for(var n,r=0,o=i(e);void 0!==(n=o.next());)if(!1===t(n,r++))return}function c(e,t){for(var n,r=0,o=i(e);void 0!==(n=o.next());)if(!t(n,r++))return!1;return!0}function d(e,t){for(var n,r=0,o=i(e);void 0!==(n=o.next());)if(t(n,r++))return!0;return!1}function h(e){for(var t,n=0,r=[],o=i(e);void 0!==(t=o.next());)r[n++]=t;return r}function f(e){for(var t,n=i(e),r={};void 0!==(t=n.next());)r[t[0]]=t[1];return r}n.r(t),n.d(t,"ArrayExt",(function(){return r})),n.d(t,"ArrayIterator",(function(){return p})),n.d(t,"ChainIterator",(function(){return C})),n.d(t,"EmptyIterator",(function(){return _})),n.d(t,"EnumerateIterator",(function(){return x})),n.d(t,"FilterIterator",(function(){return M})),n.d(t,"FnIterator",(function(){return y})),n.d(t,"ItemIterator",(function(){return m})),n.d(t,"KeyIterator",(function(){return g})),n.d(t,"MapIterator",(function(){return D})),n.d(t,"RangeIterator",(function(){return j})),n.d(t,"RepeatIterator",(function(){return B})),n.d(t,"RetroArrayIterator",(function(){return U})),n.d(t,"StrideIterator",(function(){return G})),n.d(t,"StringExt",(function(){return W})),n.d(t,"TakeIterator",(function(){return q})),n.d(t,"ValueIterator",(function(){return v})),n.d(t,"ZipIterator",(function(){return Q})),n.d(t,"chain",(function(){return b})),n.d(t,"each",(function(){return u})),n.d(t,"empty",(function(){return A})),n.d(t,"enumerate",(function(){return I})),n.d(t,"every",(function(){return c})),n.d(t,"filter",(function(){return w})),n.d(t,"find",(function(){return S})),n.d(t,"findIndex",(function(){return T})),n.d(t,"iter",(function(){return i})),n.d(t,"iterFn",(function(){return l})),n.d(t,"iterItems",(function(){return s})),n.d(t,"iterKeys",(function(){return o})),n.d(t,"iterValues",(function(){return a})),n.d(t,"map",(function(){return L})),n.d(t,"max",(function(){return O})),n.d(t,"min",(function(){return E})),n.d(t,"minmax",(function(){return k})),n.d(t,"once",(function(){return F})),n.d(t,"range",(function(){return P})),n.d(t,"reduce",(function(){return R})),n.d(t,"repeat",(function(){return z})),n.d(t,"retro",(function(){return H})),n.d(t,"some",(function(){return d})),n.d(t,"stride",(function(){return Y})),n.d(t,"take",(function(){return X})),n.d(t,"toArray",(function(){return h})),n.d(t,"toObject",(function(){return f})),n.d(t,"topologicSort",(function(){return V})),n.d(t,"zip",(function(){return Z})),function(e){function t(e,t,n,r){void 0===n&&(n=0),void 0===r&&(r=-1);var i,o=e.length;if(0===o)return-1;n=n<0?Math.max(0,n+o):Math.min(n,o-1),i=(r=r<0?Math.max(0,r+o):Math.min(r,o-1))<n?r+1+(o-n):r-n+1;for(var a=0;a<i;++a){var s=(n+a)%o;if(e[s]===t)return s}return-1}function n(e,t,n,r){void 0===n&&(n=-1),void 0===r&&(r=0);var i,o=e.length;if(0===o)return-1;i=(n=n<0?Math.max(0,n+o):Math.min(n,o-1))<(r=r<0?Math.max(0,r+o):Math.min(r,o-1))?n+1+(o-r):n-r+1;for(var a=0;a<i;++a){var s=(n-a+o)%o;if(e[s]===t)return s}return-1}function r(e,t,n,r){void 0===n&&(n=0),void 0===r&&(r=-1);var i,o=e.length;if(0===o)return-1;n=n<0?Math.max(0,n+o):Math.min(n,o-1),i=(r=r<0?Math.max(0,r+o):Math.min(r,o-1))<n?r+1+(o-n):r-n+1;for(var a=0;a<i;++a){var s=(n+a)%o;if(t(e[s],s))return s}return-1}function i(e,t,n,r){void 0===n&&(n=-1),void 0===r&&(r=0);var i,o=e.length;if(0===o)return-1;i=(n=n<0?Math.max(0,n+o):Math.min(n,o-1))<(r=r<0?Math.max(0,r+o):Math.min(r,o-1))?n+1+(o-r):n-r+1;for(var a=0;a<i;++a){var s=(n-a+o)%o;if(t(e[s],s))return s}return-1}function o(e,t,n){void 0===t&&(t=0),void 0===n&&(n=-1);var r=e.length;if(!(r<=1))for(t=t<0?Math.max(0,t+r):Math.min(t,r-1),n=n<0?Math.max(0,n+r):Math.min(n,r-1);t<n;){var i=e[t],o=e[n];e[t++]=o,e[n--]=i}}function a(e,t){var n=e.length;if(t<0&&(t+=n),!(t<0||t>=n)){for(var r=e[t],i=t+1;i<n;++i)e[i-1]=e[i];return e.length=n-1,r}}e.firstIndexOf=t,e.lastIndexOf=n,e.findFirstIndex=r,e.findLastIndex=i,e.findFirstValue=function(e,t,n,i){void 0===n&&(n=0),void 0===i&&(i=-1);var o=r(e,t,n,i);return-1!==o?e[o]:void 0},e.findLastValue=function(e,t,n,r){void 0===n&&(n=-1),void 0===r&&(r=0);var o=i(e,t,n,r);return-1!==o?e[o]:void 0},e.lowerBound=function(e,t,n,r,i){void 0===r&&(r=0),void 0===i&&(i=-1);var o=e.length;if(0===o)return 0;for(var a=r=r<0?Math.max(0,r+o):Math.min(r,o-1),s=(i=i<0?Math.max(0,i+o):Math.min(i,o-1))-r+1;s>0;){var l=s>>1,u=a+l;n(e[u],t)<0?(a=u+1,s-=l+1):s=l}return a},e.upperBound=function(e,t,n,r,i){void 0===r&&(r=0),void 0===i&&(i=-1);var o=e.length;if(0===o)return 0;for(var a=r=r<0?Math.max(0,r+o):Math.min(r,o-1),s=(i=i<0?Math.max(0,i+o):Math.min(i,o-1))-r+1;s>0;){var l=s>>1,u=a+l;n(e[u],t)>0?s=l:(a=u+1,s-=l+1)}return a},e.shallowEqual=function(e,t,n){if(e===t)return!0;if(e.length!==t.length)return!1;for(var r=0,i=e.length;r<i;++r)if(n?!n(e[r],t[r]):e[r]!==t[r])return!1;return!0},e.slice=function(e,t){void 0===t&&(t={});var n=t.start,r=t.stop,i=t.step;if(void 0===i&&(i=1),0===i)throw new Error("Slice `step` cannot be zero.");var o,a=e.length;void 0===n?n=i<0?a-1:0:n<0?n=Math.max(n+a,i<0?-1:0):n>=a&&(n=i<0?a-1:a),void 0===r?r=i<0?-1:a:r<0?r=Math.max(r+a,i<0?-1:0):r>=a&&(r=i<0?a-1:a),o=i<0&&r>=n||i>0&&n>=r?0:i<0?Math.floor((r-n+1)/i+1):Math.floor((r-n-1)/i+1);for(var s=[],l=0;l<o;++l)s[l]=e[n+l*i];return s},e.move=function(e,t,n){var r=e.length;if(!(r<=1)&&(t=t<0?Math.max(0,t+r):Math.min(t,r-1))!==(n=n<0?Math.max(0,n+r):Math.min(n,r-1))){for(var i=e[t],o=t<n?1:-1,a=t;a!==n;a+=o)e[a]=e[a+o];e[n]=i}},e.reverse=o,e.rotate=function(e,t,n,r){void 0===n&&(n=0),void 0===r&&(r=-1);var i=e.length;if(!(i<=1||(n=n<0?Math.max(0,n+i):Math.min(n,i-1))>=(r=r<0?Math.max(0,r+i):Math.min(r,i-1)))){var a=r-n+1;if(t>0?t%=a:t<0&&(t=(t%a+a)%a),0!==t){var s=n+t;o(e,n,s-1),o(e,s,r),o(e,n,r)}}},e.fill=function(e,t,n,r){void 0===n&&(n=0),void 0===r&&(r=-1);var i=e.length;if(0!==i){var o;n=n<0?Math.max(0,n+i):Math.min(n,i-1),o=(r=r<0?Math.max(0,r+i):Math.min(r,i-1))<n?r+1+(i-n):r-n+1;for(var a=0;a<o;++a)e[(n+a)%i]=t}},e.insert=function(e,t,n){var r=e.length;t=t<0?Math.max(0,t+r):Math.min(t,r);for(var i=r;i>t;--i)e[i]=e[i-1];e[t]=n},e.removeAt=a,e.removeFirstOf=function(e,n,r,i){void 0===r&&(r=0),void 0===i&&(i=-1);var o=t(e,n,r,i);return-1!==o&&a(e,o),o},e.removeLastOf=function(e,t,r,i){void 0===r&&(r=-1),void 0===i&&(i=0);var o=n(e,t,r,i);return-1!==o&&a(e,o),o},e.removeAllOf=function(e,t,n,r){void 0===n&&(n=0),void 0===r&&(r=-1);var i=e.length;if(0===i)return 0;n=n<0?Math.max(0,n+i):Math.min(n,i-1),r=r<0?Math.max(0,r+i):Math.min(r,i-1);for(var o=0,a=0;a<i;++a)n<=r&&a>=n&&a<=r&&e[a]===t||r<n&&(a<=r||a>=n)&&e[a]===t?o++:o>0&&(e[a-o]=e[a]);return o>0&&(e.length=i-o),o},e.removeFirstWhere=function(e,t,n,i){var o;void 0===n&&(n=0),void 0===i&&(i=-1);var s=r(e,t,n,i);return-1!==s&&(o=a(e,s)),{index:s,value:o}},e.removeLastWhere=function(e,t,n,r){var o;void 0===n&&(n=-1),void 0===r&&(r=0);var s=i(e,t,n,r);return-1!==s&&(o=a(e,s)),{index:s,value:o}},e.removeAllWhere=function(e,t,n,r){void 0===n&&(n=0),void 0===r&&(r=-1);var i=e.length;if(0===i)return 0;n=n<0?Math.max(0,n+i):Math.min(n,i-1),r=r<0?Math.max(0,r+i):Math.min(r,i-1);for(var o=0,a=0;a<i;++a)n<=r&&a>=n&&a<=r&&t(e[a],a)||r<n&&(a<=r||a>=n)&&t(e[a],a)?o++:o>0&&(e[a-o]=e[a]);return o>0&&(e.length=i-o),o}}(r||(r={}));var p=function(){function e(e){this._index=0,this._source=e}return e.prototype.iter=function(){return this},e.prototype.clone=function(){var t=new e(this._source);return t._index=this._index,t},e.prototype.next=function(){if(!(this._index>=this._source.length))return this._source[this._index++]},e}(),g=function(){function e(e,t){void 0===t&&(t=Object.keys(e)),this._index=0,this._source=e,this._keys=t}return e.prototype.iter=function(){return this},e.prototype.clone=function(){var t=new e(this._source,this._keys);return t._index=this._index,t},e.prototype.next=function(){if(!(this._index>=this._keys.length)){var e=this._keys[this._index++];return e in this._source?e:this.next()}},e}(),v=function(){function e(e,t){void 0===t&&(t=Object.keys(e)),this._index=0,this._source=e,this._keys=t}return e.prototype.iter=function(){return this},e.prototype.clone=function(){var t=new e(this._source,this._keys);return t._index=this._index,t},e.prototype.next=function(){if(!(this._index>=this._keys.length)){var e=this._keys[this._index++];return e in this._source?this._source[e]:this.next()}},e}(),m=function(){function e(e,t){void 0===t&&(t=Object.keys(e)),this._index=0,this._source=e,this._keys=t}return e.prototype.iter=function(){return this},e.prototype.clone=function(){var t=new e(this._source,this._keys);return t._index=this._index,t},e.prototype.next=function(){if(!(this._index>=this._keys.length)){var e=this._keys[this._index++];return e in this._source?[e,this._source[e]]:this.next()}},e}(),y=function(){function e(e){this._fn=e}return e.prototype.iter=function(){return this},e.prototype.clone=function(){throw new Error("An `FnIterator` cannot be cloned.")},e.prototype.next=function(){return this._fn.call(void 0)},e}();function b(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return new C(i(e.map(i)))}var C=function(){function e(e){this._cloned=!1,this._source=e,this._active=void 0}return e.prototype.iter=function(){return this},e.prototype.clone=function(){var t=new e(this._source.clone());return t._active=this._active&&this._active.clone(),t._cloned=!0,this._cloned=!0,t},e.prototype.next=function(){if(void 0===this._active){var e=this._source.next();if(void 0===e)return;this._active=this._cloned?e.clone():e}var t=this._active.next();return void 0!==t?t:(this._active=void 0,this.next())},e}();function A(){return new _}var _=function(){function e(){}return e.prototype.iter=function(){return this},e.prototype.clone=function(){return new e},e.prototype.next=function(){},e}();function I(e,t){return void 0===t&&(t=0),new x(i(e),t)}var x=function(){function e(e,t){this._source=e,this._index=t}return e.prototype.iter=function(){return this},e.prototype.clone=function(){return new e(this._source.clone(),this._index)},e.prototype.next=function(){var e=this._source.next();if(void 0!==e)return[this._index++,e]},e}();function w(e,t){return new M(i(e),t)}var M=function(){function e(e,t){this._index=0,this._source=e,this._fn=t}return e.prototype.iter=function(){return this},e.prototype.clone=function(){var t=new e(this._source.clone(),this._fn);return t._index=this._index,t},e.prototype.next=function(){for(var e,t=this._fn,n=this._source;void 0!==(e=n.next());)if(t(e,this._index++))return e},e}();function S(e,t){for(var n,r=0,o=i(e);void 0!==(n=o.next());)if(t(n,r++))return n}function T(e,t){for(var n,r=0,o=i(e);void 0!==(n=o.next());)if(t(n,r++))return r-1;return-1}function E(e,t){var n=i(e),r=n.next();if(void 0!==r){for(var o=r;void 0!==(r=n.next());)t(r,o)<0&&(o=r);return o}}function O(e,t){var n=i(e),r=n.next();if(void 0!==r){for(var o=r;void 0!==(r=n.next());)t(r,o)>0&&(o=r);return o}}function k(e,t){var n=i(e),r=n.next();if(void 0!==r){for(var o=r,a=r;void 0!==(r=n.next());)t(r,o)<0?o=r:t(r,a)>0&&(a=r);return[o,a]}}function L(e,t){return new D(i(e),t)}var D=function(){function e(e,t){this._index=0,this._source=e,this._fn=t}return e.prototype.iter=function(){return this},e.prototype.clone=function(){var t=new e(this._source.clone(),this._fn);return t._index=this._index,t},e.prototype.next=function(){var e=this._source.next();if(void 0!==e)return this._fn.call(void 0,e,this._index++)},e}();function P(e,t,n){return void 0===t?new j(0,e,1):new j(e,t,void 0===n?1:n)}var N,j=function(){function e(e,t,n){this._index=0,this._start=e,this._stop=t,this._step=n,this._length=N.rangeLength(e,t,n)}return e.prototype.iter=function(){return this},e.prototype.clone=function(){var t=new e(this._start,this._stop,this._step);return t._index=this._index,t},e.prototype.next=function(){if(!(this._index>=this._length))return this._start+this._step*this._index++},e}();function R(e,t,n){var r=0,o=i(e),a=o.next();if(void 0===a&&void 0===n)throw new TypeError("Reduce of empty iterable with no initial value.");if(void 0===a)return n;var s,l,u=o.next();if(void 0===u&&void 0===n)return a;if(void 0===u)return t(n,a,r++);for(s=t(void 0===n?a:t(n,a,r++),u,r++);void 0!==(l=o.next());)s=t(s,l,r++);return s}function z(e,t){return new B(e,t)}function F(e){return new B(e,1)}!function(e){e.rangeLength=function(e,t,n){return 0===n?1/0:e>t&&n>0||e<t&&n<0?0:Math.ceil((t-e)/n)}}(N||(N={}));var B=function(){function e(e,t){this._value=e,this._count=t}return e.prototype.iter=function(){return this},e.prototype.clone=function(){return new e(this._value,this._count)},e.prototype.next=function(){if(!(this._count<=0))return this._count--,this._value},e}();function H(e){return"function"==typeof e.retro?e.retro():new U(e)}var U=function(){function e(e){this._source=e,this._index=e.length-1}return e.prototype.iter=function(){return this},e.prototype.clone=function(){var t=new e(this._source);return t._index=this._index,t},e.prototype.next=function(){if(!(this._index<0||this._index>=this._source.length))return this._source[this._index--]},e}();function V(e){var t=[],n=new Set,r=new Map;return u(e,(function(e){var t=e[0],n=e[1],i=r.get(n);i?i.push(t):r.set(n,[t])})),r.forEach((function(e,t){i(t)})),t;function i(e){if(!n.has(e)){n.add(e);var o=r.get(e);o&&o.forEach(i),t.push(e)}}}function Y(e,t){return new G(i(e),t)}var W,G=function(){function e(e,t){this._source=e,this._step=t}return e.prototype.iter=function(){return this},e.prototype.clone=function(){return new e(this._source.clone(),this._step)},e.prototype.next=function(){for(var e=this._source.next(),t=this._step-1;t>0;--t)this._source.next();return e},e}();function X(e,t){return new q(i(e),t)}!function(e){function t(e,t,n){void 0===n&&(n=0);for(var r=new Array(t.length),i=0,o=n,a=t.length;i<a;++i,++o){if(-1===(o=e.indexOf(t[i],o)))return null;r[i]=o}return r}e.findIndices=t,e.matchSumOfSquares=function(e,n,r){void 0===r&&(r=0);var i=t(e,n,r);if(!i)return null;for(var o=0,a=0,s=i.length;a<s;++a){var l=i[a]-r;o+=l*l}return{score:o,indices:i}},e.matchSumOfDeltas=function(e,n,r){void 0===r&&(r=0);var i=t(e,n,r);if(!i)return null;for(var o=0,a=r-1,s=0,l=i.length;s<l;++s){var u=i[s];o+=u-a-1,a=u}return{score:o,indices:i}},e.highlight=function(e,t,n){for(var r=[],i=0,o=0,a=t.length;i<a;){for(var s=t[i],l=t[i];++i<a&&t[i]===l+1;)l++;o<s&&r.push(e.slice(o,s)),s<l+1&&r.push(n(e.slice(s,l+1))),o=l+1}return o<e.length&&r.push(e.slice(o)),r},e.cmp=function(e,t){return e<t?-1:e>t?1:0}}(W||(W={}));var q=function(){function e(e,t){this._source=e,this._count=t}return e.prototype.iter=function(){return this},e.prototype.clone=function(){return new e(this._source.clone(),this._count)},e.prototype.next=function(){if(!(this._count<=0)){var e=this._source.next();if(void 0!==e)return this._count--,e}},e}();function Z(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return new Q(e.map(i))}var Q=function(){function e(e){this._source=e}return e.prototype.iter=function(){return this},e.prototype.clone=function(){return new e(this._source.map((function(e){return e.clone()})))},e.prototype.next=function(){for(var e=new Array(this._source.length),t=0,n=this._source.length;t<n;++t){var r=this._source[t].next();if(void 0===r)return;e[t]=r}return e},e}()},function(e,t,n){"use strict";var r=n(1),i=n(2351),o=n(167);t.a=function(e,t){return Object(i.a)(e,Object(r.a)({defaultTheme:o.a},t))}},function(e,t){e.exports=function(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}},function(e,t){e.exports=function(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}},function(e,t,n){"use strict";var r=n(39),i=n(317).utcFormat,o=n(42),a=n(72),s=a.FP_SAFE,l=a.BADNUM,u=e.exports={};u.nestedProperty=n(522),u.keyedContainer=n(1647),u.relativeAttr=n(1648),u.isPlainObject=n(292),u.toLogRange=n(523),u.relinkPrivateKeys=n(1649);var c=n(249);u.isTypedArray=c.isTypedArray,u.isArrayOrTypedArray=c.isArrayOrTypedArray,u.isArray1D=c.isArray1D,u.ensureArray=c.ensureArray,u.concat=c.concat,u.maxRowLength=c.maxRowLength,u.minRowLength=c.minRowLength;var d=n(293);u.mod=d.mod,u.modHalf=d.modHalf;var h=n(1650);u.valObjectMeta=h.valObjectMeta,u.coerce=h.coerce,u.coerce2=h.coerce2,u.coerceFont=h.coerceFont,u.coerceHoverinfo=h.coerceHoverinfo,u.coerceSelectionMarkerOpacity=h.coerceSelectionMarkerOpacity,u.validate=h.validate;var f=n(1651);u.dateTime2ms=f.dateTime2ms,u.isDateTime=f.isDateTime,u.ms2DateTime=f.ms2DateTime,u.ms2DateTimeLocal=f.ms2DateTimeLocal,u.cleanDate=f.cleanDate,u.isJSDate=f.isJSDate,u.formatDate=f.formatDate,u.incrementMonth=f.incrementMonth,u.dateTick0=f.dateTick0,u.dfltRange=f.dfltRange,u.findExactDates=f.findExactDates,u.MIN_MS=f.MIN_MS,u.MAX_MS=f.MAX_MS;var p=n(538);u.findBin=p.findBin,u.sorterAsc=p.sorterAsc,u.sorterDes=p.sorterDes,u.distinctVals=p.distinctVals,u.roundUp=p.roundUp,u.sort=p.sort,u.findIndexOfMin=p.findIndexOfMin;var g=n(1664);u.aggNums=g.aggNums,u.len=g.len,u.mean=g.mean,u.median=g.median,u.midRange=g.midRange,u.variance=g.variance,u.stdev=g.stdev,u.interp=g.interp;var v=n(528);u.init2dArray=v.init2dArray,u.transposeRagged=v.transposeRagged,u.dot=v.dot,u.translationMatrix=v.translationMatrix,u.rotationMatrix=v.rotationMatrix,u.rotationXYMatrix=v.rotationXYMatrix,u.apply3DTransform=v.apply3DTransform,u.apply2DTransform=v.apply2DTransform,u.apply2DTransform2=v.apply2DTransform2,u.convertCssMatrix=v.convertCssMatrix,u.inverseTransformMatrix=v.inverseTransformMatrix;var m=n(1665);u.deg2rad=m.deg2rad,u.rad2deg=m.rad2deg,u.angleDelta=m.angleDelta,u.angleDist=m.angleDist,u.isFullCircle=m.isFullCircle,u.isAngleInsideSector=m.isAngleInsideSector,u.isPtInsideSector=m.isPtInsideSector,u.pathArc=m.pathArc,u.pathSector=m.pathSector,u.pathAnnulus=m.pathAnnulus;var y=n(1666);u.isLeftAnchor=y.isLeftAnchor,u.isCenterAnchor=y.isCenterAnchor,u.isRightAnchor=y.isRightAnchor,u.isTopAnchor=y.isTopAnchor,u.isMiddleAnchor=y.isMiddleAnchor,u.isBottomAnchor=y.isBottomAnchor;var b=n(1667);u.segmentsIntersect=b.segmentsIntersect,u.segmentDistance=b.segmentDistance,u.getTextLocation=b.getTextLocation,u.clearLocationCache=b.clearLocationCache,u.getVisibleSegment=b.getVisibleSegment,u.findPointOnPath=b.findPointOnPath;var C=n(63);u.extendFlat=C.extendFlat,u.extendDeep=C.extendDeep,u.extendDeepAll=C.extendDeepAll,u.extendDeepNoArrays=C.extendDeepNoArrays;var A=n(252);u.log=A.log,u.warn=A.warn,u.error=A.error;var _=n(294);u.counterRegex=_.counter;var I=n(539);u.throttle=I.throttle,u.throttleDone=I.done,u.clearThrottle=I.clear;var x=n(527);function w(e){var t={};for(var n in e)for(var r=e[n],i=0;i<r.length;i++)t[r[i]]=+n;return t}u.getGraphDiv=x.getGraphDiv,u.isPlotDiv=x.isPlotDiv,u.removeElement=x.removeElement,u.addStyleRule=x.addStyleRule,u.addRelatedStyleRule=x.addRelatedStyleRule,u.deleteRelatedStyleRule=x.deleteRelatedStyleRule,u.getFullTransformMatrix=x.getFullTransformMatrix,u.getElementTransformMatrix=x.getElementTransformMatrix,u.getElementAndAncestors=x.getElementAndAncestors,u.equalDomRects=x.equalDomRects,u.clearResponsive=n(1668),u.preserveDrawingBuffer=n(1669),u.makeTraceGroups=n(1670),u._=n(1671),u.notifier=n(966),u.filterUnique=n(1672),u.filterVisible=n(1673),u.pushUnique=n(967),u.increment=n(1674),u.cleanNumber=n(1675),u.ensureNumber=function(e){return o(e)?(e=Number(e))<-s||e>s?l:o(e)?Number(e):l:l},u.isIndex=function(e,t){return!(void 0!==t&&e>=t)&&(o(e)&&e>=0&&e%1==0)},u.noop=n(526),u.identity=n(977),u.repeat=function(e,t){for(var n=new Array(t),r=0;r<t;r++)n[r]=e;return n},u.swapAttrs=function(e,t,n,r){n||(n="x"),r||(r="y");for(var i=0;i<t.length;i++){var o=t[i],a=u.nestedProperty(e,o.replace("?",n)),s=u.nestedProperty(e,o.replace("?",r)),l=a.get();a.set(s.get()),s.set(l)}},u.raiseToTop=function(e){e.parentNode.appendChild(e)},u.cancelTransition=function(e){return e.transition().duration(0)},u.constrain=function(e,t,n){return t>n?Math.max(n,Math.min(t,e)):Math.max(t,Math.min(n,e))},u.bBoxIntersect=function(e,t,n){return n=n||0,e.left<=t.right+n&&t.left<=e.right+n&&e.top<=t.bottom+n&&t.top<=e.bottom+n},u.simpleMap=function(e,t,n,r,i){for(var o=e.length,a=new Array(o),s=0;s<o;s++)a[s]=t(e[s],n,r,i);return a},u.randstr=function e(t,n,r,i){if(r||(r=16),void 0===n&&(n=24),n<=0)return"0";var o,a,s=Math.log(Math.pow(2,n))/Math.log(r),l="";for(o=2;s===1/0;o*=2)s=Math.log(Math.pow(2,n/o))/Math.log(r)*o;var c=s-Math.floor(s);for(o=0;o<Math.floor(s);o++)l=Math.floor(Math.random()*r).toString(r)+l;c&&(a=Math.pow(r,c),l=Math.floor(Math.random()*a).toString(r)+l);var d=parseInt(l,r);return t&&t[l]||d!==1/0&&d>=Math.pow(2,n)?i>10?(u.warn("randstr failed uniqueness"),l):e(t,n,r,(i||0)+1):l},u.OptionControl=function(e,t){e||(e={}),t||(t="opt");var n={optionList:[],_newoption:function(r){r[t]=e,n[r.name]=r,n.optionList.push(r)}};return n["_"+t]=e,n},u.smooth=function(e,t){if((t=Math.round(t)||0)<2)return e;var n,r,i,o,a=e.length,s=2*a,l=2*t-1,u=new Array(l),c=new Array(a);for(n=0;n<l;n++)u[n]=(1-Math.cos(Math.PI*(n+1)/t))/(2*t);for(n=0;n<a;n++){for(o=0,r=0;r<l;r++)(i=n+r+1-t)<-a?i-=s*Math.round(i/s):i>=s&&(i-=s*Math.floor(i/s)),i<0?i=-1-i:i>=a&&(i=s-1-i),o+=e[i]*u[r];c[n]=o}return c},u.syncOrAsync=function(e,t,n){var r;function i(){return u.syncOrAsync(e,t,n)}for(;e.length;)if((r=(0,e.splice(0,1)[0])(t))&&r.then)return r.then(i).then(void 0,u.promiseError);return n&&n(t)},u.stripTrailingSlash=function(e){return"/"===e.substr(-1)?e.substr(0,e.length-1):e},u.noneOrAll=function(e,t,n){if(e){var r,i=!1,o=!0;for(r=0;r<n.length;r++)null!=e[n[r]]?i=!0:o=!1;if(i&&!o)for(r=0;r<n.length;r++)e[n[r]]=t[n[r]]}},u.mergeArray=function(e,t,n,r){var i="function"==typeof r;if(u.isArrayOrTypedArray(e))for(var o=Math.min(e.length,t.length),a=0;a<o;a++){var s=e[a];t[a][n]=i?r(s):s}},u.mergeArrayCastPositive=function(e,t,n){return u.mergeArray(e,t,n,(function(e){var t=+e;return isFinite(t)&&t>0?t:0}))},u.fillArray=function(e,t,n,r){if(r=r||u.identity,u.isArrayOrTypedArray(e))for(var i=0;i<t.length;i++)t[i][n]=r(e[i])},u.castOption=function(e,t,n,r){r=r||u.identity;var i=u.nestedProperty(e,n).get();return u.isArrayOrTypedArray(i)?Array.isArray(t)&&u.isArrayOrTypedArray(i[t[0]])?r(i[t[0]][t[1]]):r(i[t]):i},u.extractOption=function(e,t,n,r){if(n in e)return e[n];var i=u.nestedProperty(t,r).get();return Array.isArray(i)?void 0:i},u.tagSelected=function(e,t,n){var r,i,o=t.selectedpoints,a=t._indexToPoints;a&&(r=w(a));for(var s=0;s<o.length;s++){var l=o[s];if(u.isIndex(l)||u.isArrayOrTypedArray(l)&&u.isIndex(l[0])&&u.isIndex(l[1])){var c=r?r[l]:l,d=n?n[c]:c;void 0!==(i=d)&&i<e.length&&(e[d].selected=1)}}},u.selIndices2selPoints=function(e){var t=e.selectedpoints,n=e._indexToPoints;if(n){for(var r=w(n),i=[],o=0;o<t.length;o++){var a=t[o];if(u.isIndex(a)){var s=r[a];u.isIndex(s)&&i.push(s)}}return i}return t},u.getTargetArray=function(e,t){var n=t.target;if("string"==typeof n&&n){var r=u.nestedProperty(e,n).get();return!!Array.isArray(r)&&r}return!!Array.isArray(n)&&n},u.minExtend=function(e,t){var n={};"object"!=typeof t&&(t={});var r,i,o,a=Object.keys(e);for(r=0;r<a.length;r++)o=e[i=a[r]],"_"!==i.charAt(0)&&"function"!=typeof o&&("module"===i?n[i]=o:Array.isArray(o)?n[i]="colorscale"===i?o.slice():o.slice(0,3):u.isTypedArray(o)?n[i]=o.subarray(0,3):n[i]=o&&"object"==typeof o?u.minExtend(e[i],t[i]):o);for(a=Object.keys(t),r=0;r<a.length;r++)"object"==typeof(o=t[i=a[r]])&&i in n&&"object"==typeof n[i]||(n[i]=o);return n},u.titleCase=function(e){return e.charAt(0).toUpperCase()+e.substr(1)},u.containsAny=function(e,t){for(var n=0;n<t.length;n++)if(-1!==e.indexOf(t[n]))return!0;return!1},u.isIE=function(){return void 0!==window.navigator.msSaveBlob};var M=/MSIE [1-9]\./;u.isIE9orBelow=function(){return u.isIE()&&M.test(window.navigator.userAgent)};var S=/Version\/[\d\.]+.*Safari/;u.isSafari=function(){return S.test(window.navigator.userAgent)};var T=/iPad|iPhone|iPod/;u.isIOS=function(){return T.test(window.navigator.userAgent)},u.isD3Selection=function(e){return e&&"function"==typeof e.classed},u.ensureSingle=function(e,t,n,r){var i=e.select(t+(n?"."+n:""));if(i.size())return i;var o=e.append(t);return n&&o.classed(n,!0),r&&o.call(r),o},u.ensureSingleById=function(e,t,n,r){var i=e.select(t+"#"+n);if(i.size())return i;var o=e.append(t).attr("id",n);return r&&o.call(r),o},u.objectFromPath=function(e,t){for(var n,r=e.split("."),i=n={},o=0;o<r.length;o++){var a=r[o],s=null,l=r[o].match(/(.*)\[([0-9]+)\]/);l?(a=l[1],s=l[2],n=n[a]=[],o===r.length-1?n[s]=t:n[s]={},n=n[s]):(o===r.length-1?n[a]=t:n[a]={},n=n[a])}return i};var E=/^([^\[\.]+)\.(.+)?/,O=/^([^\.]+)\[([0-9]+)\](\.)?(.+)?/;u.expandObjectPaths=function(e){var t,n,r,i,o,a,s;if("object"==typeof e&&!Array.isArray(e))for(n in e)e.hasOwnProperty(n)&&((t=n.match(E))?(i=e[n],r=t[1],delete e[n],e[r]=u.extendDeepNoArrays(e[r]||{},u.objectFromPath(n,u.expandObjectPaths(i))[r])):(t=n.match(O))?(i=e[n],r=t[1],o=parseInt(t[2]),delete e[n],e[r]=e[r]||[],"."===t[3]?(s=t[4],a=e[r][o]=e[r][o]||{},u.extendDeepNoArrays(a,u.objectFromPath(s,u.expandObjectPaths(i)))):e[r][o]=u.expandObjectPaths(i)):e[n]=u.expandObjectPaths(e[n]));return e},u.numSeparate=function(e,t,n){if(n||(n=!1),"string"!=typeof t||0===t.length)throw new Error("Separator string required for formatting!");"number"==typeof e&&(e=String(e));var r=/(\d+)(\d{3})/,i=t.charAt(0),o=t.charAt(1),a=e.split("."),s=a[0],l=a.length>1?i+a[1]:"";if(o&&(a.length>1||s.length>4||n))for(;r.test(s);)s=s.replace(r,"$1"+o+"$2");return s+l},u.TEMPLATE_STRING_REGEX=/%{([^\s%{}:]*)([:|\|][^}]*)?}/g;var k=/^\w*$/;u.templateString=function(e,t){var n={};return e.replace(u.TEMPLATE_STRING_REGEX,(function(e,r){var i;return k.test(r)?i=t[r]:(n[r]=n[r]||u.nestedProperty(t,r).get,i=n[r]()),u.isValidTextValue(i)?i:""}))};var L={max:10,count:0,name:"hovertemplate"};u.hovertemplateString=function(){return N.apply(L,arguments)};var D={max:10,count:0,name:"texttemplate"};u.texttemplateString=function(){return N.apply(D,arguments)};var P=/^[:|\|]/;function N(e,t,n){var o=this,a=arguments;t||(t={});var s={};return e.replace(u.TEMPLATE_STRING_REGEX,(function(e,l,c){var d,h,f,p;for(f=3;f<a.length;f++)if(d=a[f]){if(d.hasOwnProperty(l)){h=d[l];break}if(k.test(l)||(h=s[l]||u.nestedProperty(d,l).get())&&(s[l]=h),void 0!==h)break}if(void 0===h&&o)return o.count<o.max&&(u.warn("Variable '"+l+"' in "+o.name+" could not be found!"),h=e),o.count===o.max&&u.warn("Too many "+o.name+" warnings - additional warnings will be suppressed"),o.count++,e;if(c){if(":"===c[0]&&(h=(p=n?n.numberFormat:r.format)(c.replace(P,""))(h)),"|"===c[0]){p=n?n.timeFormat:i;var g=u.dateTime2ms(h);h=u.formatDate(g,c.replace(P,""),!1,p)}}else t.hasOwnProperty(l+"Label")&&(h=t[l+"Label"]);return h}))}u.subplotSort=function(e,t){for(var n=Math.min(e.length,t.length)+1,r=0,i=0,o=0;o<n;o++){var a=e.charCodeAt(o)||0,s=t.charCodeAt(o)||0,l=a>=48&&a<=57,u=s>=48&&s<=57;if(l&&(r=10*r+a-48),u&&(i=10*i+s-48),!l||!u){if(r!==i)return r-i;if(a!==s)return a-s}}return i-r};var j=2e9;u.seedPseudoRandom=function(){j=2e9},u.pseudoRandom=function(){var e=j;return j=(69069*j+1)%4294967296,Math.abs(j-e)<429496729?u.pseudoRandom():j/4294967296},u.fillText=function(e,t,n){var r=Array.isArray(n)?function(e){n.push(e)}:function(e){n.text=e},i=u.extractOption(e,t,"htx","hovertext");if(u.isValidTextValue(i))return r(i);var o=u.extractOption(e,t,"tx","text");return u.isValidTextValue(o)?r(o):void 0},u.isValidTextValue=function(e){return e||0===e},u.formatPercent=function(e,t){t=t||0;for(var n=(Math.round(100*e*Math.pow(10,t))*Math.pow(.1,t)).toFixed(t)+"%",r=0;r<t;r++)-1!==n.indexOf(".")&&(n=(n=n.replace("0%","%")).replace(".%","%"));return n},u.isHidden=function(e){var t=window.getComputedStyle(e).display;return!t||"none"===t},u.strTranslate=function(e,t){return e||t?"translate("+e+","+t+")":""},u.strRotate=function(e){return e?"rotate("+e+")":""},u.strScale=function(e){return 1!==e?"scale("+e+")":""},u.getTextTransform=function(e){var t=e.noCenter,n=e.textX,r=e.textY,i=e.targetX,o=e.targetY,a=e.anchorX||0,s=e.anchorY||0,l=e.rotate,c=e.scale;return c?c>1&&(c=1):c=0,u.strTranslate(i-c*(n+a),o-c*(r+s))+u.strScale(c)+(l?"rotate("+l+(t?"":" "+n+" "+r)+")":"")},u.ensureUniformFontSize=function(e,t){var n=u.extendFlat({},t);return n.size=Math.max(t.size,e._fullLayout.uniformtext.minsize||0),n},u.join2=function(e,t,n){var r=e.length;return r>1?e.slice(0,-1).join(t)+n+e[r-1]:e.join(t)}},function(e,t,n){var r;
/*!
  Copyright (c) 2017 Jed Watson.
  Licensed under the MIT License (MIT), see
  http://jedwatson.github.io/classnames
*/!function(){"use strict";var n={}.hasOwnProperty;function i(){for(var e=[],t=0;t<arguments.length;t++){var r=arguments[t];if(r){var o=typeof r;if("string"===o||"number"===o)e.push(r);else if(Array.isArray(r)&&r.length){var a=i.apply(null,r);a&&e.push(a)}else if("object"===o)for(var s in r)n.call(r,s)&&r[s]&&e.push(s)}}return e.join(" ")}e.exports?(i.default=i,e.exports=i):void 0===(r=function(){return i}.apply(t,[]))||(e.exports=r)}()},function(e,t,n){(function(e){e.exports=function(){"use strict";var t,r;function i(){return t.apply(null,arguments)}function o(e){return e instanceof Array||"[object Array]"===Object.prototype.toString.call(e)}function a(e){return null!=e&&"[object Object]"===Object.prototype.toString.call(e)}function s(e,t){return Object.prototype.hasOwnProperty.call(e,t)}function l(e){if(Object.getOwnPropertyNames)return 0===Object.getOwnPropertyNames(e).length;var t;for(t in e)if(s(e,t))return!1;return!0}function u(e){return void 0===e}function c(e){return"number"==typeof e||"[object Number]"===Object.prototype.toString.call(e)}function d(e){return e instanceof Date||"[object Date]"===Object.prototype.toString.call(e)}function h(e,t){var n,r=[];for(n=0;n<e.length;++n)r.push(t(e[n],n));return r}function f(e,t){for(var n in t)s(t,n)&&(e[n]=t[n]);return s(t,"toString")&&(e.toString=t.toString),s(t,"valueOf")&&(e.valueOf=t.valueOf),e}function p(e,t,n,r){return wt(e,t,n,r,!0).utc()}function g(e){return null==e._pf&&(e._pf={empty:!1,unusedTokens:[],unusedInput:[],overflow:-2,charsLeftOver:0,nullInput:!1,invalidEra:null,invalidMonth:null,invalidFormat:!1,userInvalidated:!1,iso:!1,parsedDateParts:[],era:null,meridiem:null,rfc2822:!1,weekdayMismatch:!1}),e._pf}function v(e){if(null==e._isValid){var t=g(e),n=r.call(t.parsedDateParts,(function(e){return null!=e})),i=!isNaN(e._d.getTime())&&t.overflow<0&&!t.empty&&!t.invalidEra&&!t.invalidMonth&&!t.invalidWeekday&&!t.weekdayMismatch&&!t.nullInput&&!t.invalidFormat&&!t.userInvalidated&&(!t.meridiem||t.meridiem&&n);if(e._strict&&(i=i&&0===t.charsLeftOver&&0===t.unusedTokens.length&&void 0===t.bigHour),null!=Object.isFrozen&&Object.isFrozen(e))return i;e._isValid=i}return e._isValid}function m(e){var t=p(NaN);return null!=e?f(g(t),e):g(t).userInvalidated=!0,t}r=Array.prototype.some?Array.prototype.some:function(e){var t,n=Object(this),r=n.length>>>0;for(t=0;t<r;t++)if(t in n&&e.call(this,n[t],t,n))return!0;return!1};var y=i.momentProperties=[],b=!1;function C(e,t){var n,r,i;if(u(t._isAMomentObject)||(e._isAMomentObject=t._isAMomentObject),u(t._i)||(e._i=t._i),u(t._f)||(e._f=t._f),u(t._l)||(e._l=t._l),u(t._strict)||(e._strict=t._strict),u(t._tzm)||(e._tzm=t._tzm),u(t._isUTC)||(e._isUTC=t._isUTC),u(t._offset)||(e._offset=t._offset),u(t._pf)||(e._pf=g(t)),u(t._locale)||(e._locale=t._locale),y.length>0)for(n=0;n<y.length;n++)u(i=t[r=y[n]])||(e[r]=i);return e}function A(e){C(this,e),this._d=new Date(null!=e._d?e._d.getTime():NaN),this.isValid()||(this._d=new Date(NaN)),!1===b&&(b=!0,i.updateOffset(this),b=!1)}function _(e){return e instanceof A||null!=e&&null!=e._isAMomentObject}function I(e){!1===i.suppressDeprecationWarnings&&"undefined"!=typeof console&&console.warn&&console.warn("Deprecation warning: "+e)}function x(e,t){var n=!0;return f((function(){if(null!=i.deprecationHandler&&i.deprecationHandler(null,e),n){var r,o,a,l=[];for(o=0;o<arguments.length;o++){if(r="","object"==typeof arguments[o]){for(a in r+="\n["+o+"] ",arguments[0])s(arguments[0],a)&&(r+=a+": "+arguments[0][a]+", ");r=r.slice(0,-2)}else r=arguments[o];l.push(r)}I(e+"\nArguments: "+Array.prototype.slice.call(l).join("")+"\n"+(new Error).stack),n=!1}return t.apply(this,arguments)}),t)}var w,M={};function S(e,t){null!=i.deprecationHandler&&i.deprecationHandler(e,t),M[e]||(I(t),M[e]=!0)}function T(e){return"undefined"!=typeof Function&&e instanceof Function||"[object Function]"===Object.prototype.toString.call(e)}function E(e,t){var n,r=f({},e);for(n in t)s(t,n)&&(a(e[n])&&a(t[n])?(r[n]={},f(r[n],e[n]),f(r[n],t[n])):null!=t[n]?r[n]=t[n]:delete r[n]);for(n in e)s(e,n)&&!s(t,n)&&a(e[n])&&(r[n]=f({},r[n]));return r}function O(e){null!=e&&this.set(e)}function k(e,t,n){var r=""+Math.abs(e),i=t-r.length;return(e>=0?n?"+":"":"-")+Math.pow(10,Math.max(0,i)).toString().substr(1)+r}i.suppressDeprecationWarnings=!1,i.deprecationHandler=null,w=Object.keys?Object.keys:function(e){var t,n=[];for(t in e)s(e,t)&&n.push(t);return n};var L=/(\[[^\[]*\])|(\\)?([Hh]mm(ss)?|Mo|MM?M?M?|Do|DDDo|DD?D?D?|ddd?d?|do?|w[o|w]?|W[o|W]?|Qo?|N{1,5}|YYYYYY|YYYYY|YYYY|YY|y{2,4}|yo?|gg(ggg?)?|GG(GGG?)?|e|E|a|A|hh?|HH?|kk?|mm?|ss?|S{1,9}|x|X|zz?|ZZ?|.)/g,D=/(\[[^\[]*\])|(\\)?(LTS|LT|LL?L?L?|l{1,4})/g,P={},N={};function j(e,t,n,r){var i=r;"string"==typeof r&&(i=function(){return this[r]()}),e&&(N[e]=i),t&&(N[t[0]]=function(){return k(i.apply(this,arguments),t[1],t[2])}),n&&(N[n]=function(){return this.localeData().ordinal(i.apply(this,arguments),e)})}function R(e,t){return e.isValid()?(t=z(t,e.localeData()),P[t]=P[t]||function(e){var t,n,r,i=e.match(L);for(t=0,n=i.length;t<n;t++)N[i[t]]?i[t]=N[i[t]]:i[t]=(r=i[t]).match(/\[[\s\S]/)?r.replace(/^\[|\]$/g,""):r.replace(/\\/g,"");return function(t){var r,o="";for(r=0;r<n;r++)o+=T(i[r])?i[r].call(t,e):i[r];return o}}(t),P[t](e)):e.localeData().invalidDate()}function z(e,t){var n=5;function r(e){return t.longDateFormat(e)||e}for(D.lastIndex=0;n>=0&&D.test(e);)e=e.replace(D,r),D.lastIndex=0,n-=1;return e}var F={};function B(e,t){var n=e.toLowerCase();F[n]=F[n+"s"]=F[t]=e}function H(e){return"string"==typeof e?F[e]||F[e.toLowerCase()]:void 0}function U(e){var t,n,r={};for(n in e)s(e,n)&&(t=H(n))&&(r[t]=e[n]);return r}var V={};function Y(e,t){V[e]=t}function W(e){return e%4==0&&e%100!=0||e%400==0}function G(e){return e<0?Math.ceil(e)||0:Math.floor(e)}function X(e){var t=+e,n=0;return 0!==t&&isFinite(t)&&(n=G(t)),n}function q(e,t){return function(n){return null!=n?(Q(this,e,n),i.updateOffset(this,t),this):Z(this,e)}}function Z(e,t){return e.isValid()?e._d["get"+(e._isUTC?"UTC":"")+t]():NaN}function Q(e,t,n){e.isValid()&&!isNaN(n)&&("FullYear"===t&&W(e.year())&&1===e.month()&&29===e.date()?(n=X(n),e._d["set"+(e._isUTC?"UTC":"")+t](n,e.month(),_e(n,e.month()))):e._d["set"+(e._isUTC?"UTC":"")+t](n))}var K,J=/\d/,$=/\d\d/,ee=/\d{3}/,te=/\d{4}/,ne=/[+-]?\d{6}/,re=/\d\d?/,ie=/\d\d\d\d?/,oe=/\d\d\d\d\d\d?/,ae=/\d{1,3}/,se=/\d{1,4}/,le=/[+-]?\d{1,6}/,ue=/\d+/,ce=/[+-]?\d+/,de=/Z|[+-]\d\d:?\d\d/gi,he=/Z|[+-]\d\d(?::?\d\d)?/gi,fe=/[0-9]{0,256}['a-z\u00A0-\u05FF\u0700-\uD7FF\uF900-\uFDCF\uFDF0-\uFF07\uFF10-\uFFEF]{1,256}|[\u0600-\u06FF\/]{1,256}(\s*?[\u0600-\u06FF]{1,256}){1,2}/i;function pe(e,t,n){K[e]=T(t)?t:function(e,r){return e&&n?n:t}}function ge(e,t){return s(K,e)?K[e](t._strict,t._locale):new RegExp(ve(e.replace("\\","").replace(/\\(\[)|\\(\])|\[([^\]\[]*)\]|\\(.)/g,(function(e,t,n,r,i){return t||n||r||i}))))}function ve(e){return e.replace(/[-\/\\^$*+?.()|[\]{}]/g,"\\$&")}K={};var me,ye={};function be(e,t){var n,r=t;for("string"==typeof e&&(e=[e]),c(t)&&(r=function(e,n){n[t]=X(e)}),n=0;n<e.length;n++)ye[e[n]]=r}function Ce(e,t){be(e,(function(e,n,r,i){r._w=r._w||{},t(e,r._w,r,i)}))}function Ae(e,t,n){null!=t&&s(ye,e)&&ye[e](t,n._a,n,e)}function _e(e,t){if(isNaN(e)||isNaN(t))return NaN;var n,r=(t%(n=12)+n)%n;return e+=(t-r)/12,1===r?W(e)?29:28:31-r%7%2}me=Array.prototype.indexOf?Array.prototype.indexOf:function(e){var t;for(t=0;t<this.length;++t)if(this[t]===e)return t;return-1},j("M",["MM",2],"Mo",(function(){return this.month()+1})),j("MMM",0,0,(function(e){return this.localeData().monthsShort(this,e)})),j("MMMM",0,0,(function(e){return this.localeData().months(this,e)})),B("month","M"),Y("month",8),pe("M",re),pe("MM",re,$),pe("MMM",(function(e,t){return t.monthsShortRegex(e)})),pe("MMMM",(function(e,t){return t.monthsRegex(e)})),be(["M","MM"],(function(e,t){t[1]=X(e)-1})),be(["MMM","MMMM"],(function(e,t,n,r){var i=n._locale.monthsParse(e,r,n._strict);null!=i?t[1]=i:g(n).invalidMonth=e}));var Ie="January_February_March_April_May_June_July_August_September_October_November_December".split("_"),xe="Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),we=/D[oD]?(\[[^\[\]]*\]|\s)+MMMM?/,Me=fe,Se=fe;function Te(e,t,n){var r,i,o,a=e.toLocaleLowerCase();if(!this._monthsParse)for(this._monthsParse=[],this._longMonthsParse=[],this._shortMonthsParse=[],r=0;r<12;++r)o=p([2e3,r]),this._shortMonthsParse[r]=this.monthsShort(o,"").toLocaleLowerCase(),this._longMonthsParse[r]=this.months(o,"").toLocaleLowerCase();return n?"MMM"===t?-1!==(i=me.call(this._shortMonthsParse,a))?i:null:-1!==(i=me.call(this._longMonthsParse,a))?i:null:"MMM"===t?-1!==(i=me.call(this._shortMonthsParse,a))||-1!==(i=me.call(this._longMonthsParse,a))?i:null:-1!==(i=me.call(this._longMonthsParse,a))||-1!==(i=me.call(this._shortMonthsParse,a))?i:null}function Ee(e,t){var n;if(!e.isValid())return e;if("string"==typeof t)if(/^\d+$/.test(t))t=X(t);else if(!c(t=e.localeData().monthsParse(t)))return e;return n=Math.min(e.date(),_e(e.year(),t)),e._d["set"+(e._isUTC?"UTC":"")+"Month"](t,n),e}function Oe(e){return null!=e?(Ee(this,e),i.updateOffset(this,!0),this):Z(this,"Month")}function ke(){function e(e,t){return t.length-e.length}var t,n,r=[],i=[],o=[];for(t=0;t<12;t++)n=p([2e3,t]),r.push(this.monthsShort(n,"")),i.push(this.months(n,"")),o.push(this.months(n,"")),o.push(this.monthsShort(n,""));for(r.sort(e),i.sort(e),o.sort(e),t=0;t<12;t++)r[t]=ve(r[t]),i[t]=ve(i[t]);for(t=0;t<24;t++)o[t]=ve(o[t]);this._monthsRegex=new RegExp("^("+o.join("|")+")","i"),this._monthsShortRegex=this._monthsRegex,this._monthsStrictRegex=new RegExp("^("+i.join("|")+")","i"),this._monthsShortStrictRegex=new RegExp("^("+r.join("|")+")","i")}function Le(e){return W(e)?366:365}j("Y",0,0,(function(){var e=this.year();return e<=9999?k(e,4):"+"+e})),j(0,["YY",2],0,(function(){return this.year()%100})),j(0,["YYYY",4],0,"year"),j(0,["YYYYY",5],0,"year"),j(0,["YYYYYY",6,!0],0,"year"),B("year","y"),Y("year",1),pe("Y",ce),pe("YY",re,$),pe("YYYY",se,te),pe("YYYYY",le,ne),pe("YYYYYY",le,ne),be(["YYYYY","YYYYYY"],0),be("YYYY",(function(e,t){t[0]=2===e.length?i.parseTwoDigitYear(e):X(e)})),be("YY",(function(e,t){t[0]=i.parseTwoDigitYear(e)})),be("Y",(function(e,t){t[0]=parseInt(e,10)})),i.parseTwoDigitYear=function(e){return X(e)+(X(e)>68?1900:2e3)};var De=q("FullYear",!0);function Pe(e,t,n,r,i,o,a){var s;return e<100&&e>=0?(s=new Date(e+400,t,n,r,i,o,a),isFinite(s.getFullYear())&&s.setFullYear(e)):s=new Date(e,t,n,r,i,o,a),s}function Ne(e){var t,n;return e<100&&e>=0?((n=Array.prototype.slice.call(arguments))[0]=e+400,t=new Date(Date.UTC.apply(null,n)),isFinite(t.getUTCFullYear())&&t.setUTCFullYear(e)):t=new Date(Date.UTC.apply(null,arguments)),t}function je(e,t,n){var r=7+t-n;return-(7+Ne(e,0,r).getUTCDay()-t)%7+r-1}function Re(e,t,n,r,i){var o,a,s=1+7*(t-1)+(7+n-r)%7+je(e,r,i);return s<=0?a=Le(o=e-1)+s:s>Le(e)?(o=e+1,a=s-Le(e)):(o=e,a=s),{year:o,dayOfYear:a}}function ze(e,t,n){var r,i,o=je(e.year(),t,n),a=Math.floor((e.dayOfYear()-o-1)/7)+1;return a<1?r=a+Fe(i=e.year()-1,t,n):a>Fe(e.year(),t,n)?(r=a-Fe(e.year(),t,n),i=e.year()+1):(i=e.year(),r=a),{week:r,year:i}}function Fe(e,t,n){var r=je(e,t,n),i=je(e+1,t,n);return(Le(e)-r+i)/7}function Be(e,t){return e.slice(t,7).concat(e.slice(0,t))}j("w",["ww",2],"wo","week"),j("W",["WW",2],"Wo","isoWeek"),B("week","w"),B("isoWeek","W"),Y("week",5),Y("isoWeek",5),pe("w",re),pe("ww",re,$),pe("W",re),pe("WW",re,$),Ce(["w","ww","W","WW"],(function(e,t,n,r){t[r.substr(0,1)]=X(e)})),j("d",0,"do","day"),j("dd",0,0,(function(e){return this.localeData().weekdaysMin(this,e)})),j("ddd",0,0,(function(e){return this.localeData().weekdaysShort(this,e)})),j("dddd",0,0,(function(e){return this.localeData().weekdays(this,e)})),j("e",0,0,"weekday"),j("E",0,0,"isoWeekday"),B("day","d"),B("weekday","e"),B("isoWeekday","E"),Y("day",11),Y("weekday",11),Y("isoWeekday",11),pe("d",re),pe("e",re),pe("E",re),pe("dd",(function(e,t){return t.weekdaysMinRegex(e)})),pe("ddd",(function(e,t){return t.weekdaysShortRegex(e)})),pe("dddd",(function(e,t){return t.weekdaysRegex(e)})),Ce(["dd","ddd","dddd"],(function(e,t,n,r){var i=n._locale.weekdaysParse(e,r,n._strict);null!=i?t.d=i:g(n).invalidWeekday=e})),Ce(["d","e","E"],(function(e,t,n,r){t[r]=X(e)}));var He="Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),Ue="Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),Ve="Su_Mo_Tu_We_Th_Fr_Sa".split("_"),Ye=fe,We=fe,Ge=fe;function Xe(e,t,n){var r,i,o,a=e.toLocaleLowerCase();if(!this._weekdaysParse)for(this._weekdaysParse=[],this._shortWeekdaysParse=[],this._minWeekdaysParse=[],r=0;r<7;++r)o=p([2e3,1]).day(r),this._minWeekdaysParse[r]=this.weekdaysMin(o,"").toLocaleLowerCase(),this._shortWeekdaysParse[r]=this.weekdaysShort(o,"").toLocaleLowerCase(),this._weekdaysParse[r]=this.weekdays(o,"").toLocaleLowerCase();return n?"dddd"===t?-1!==(i=me.call(this._weekdaysParse,a))?i:null:"ddd"===t?-1!==(i=me.call(this._shortWeekdaysParse,a))?i:null:-1!==(i=me.call(this._minWeekdaysParse,a))?i:null:"dddd"===t?-1!==(i=me.call(this._weekdaysParse,a))||-1!==(i=me.call(this._shortWeekdaysParse,a))||-1!==(i=me.call(this._minWeekdaysParse,a))?i:null:"ddd"===t?-1!==(i=me.call(this._shortWeekdaysParse,a))||-1!==(i=me.call(this._weekdaysParse,a))||-1!==(i=me.call(this._minWeekdaysParse,a))?i:null:-1!==(i=me.call(this._minWeekdaysParse,a))||-1!==(i=me.call(this._weekdaysParse,a))||-1!==(i=me.call(this._shortWeekdaysParse,a))?i:null}function qe(){function e(e,t){return t.length-e.length}var t,n,r,i,o,a=[],s=[],l=[],u=[];for(t=0;t<7;t++)n=p([2e3,1]).day(t),r=ve(this.weekdaysMin(n,"")),i=ve(this.weekdaysShort(n,"")),o=ve(this.weekdays(n,"")),a.push(r),s.push(i),l.push(o),u.push(r),u.push(i),u.push(o);a.sort(e),s.sort(e),l.sort(e),u.sort(e),this._weekdaysRegex=new RegExp("^("+u.join("|")+")","i"),this._weekdaysShortRegex=this._weekdaysRegex,this._weekdaysMinRegex=this._weekdaysRegex,this._weekdaysStrictRegex=new RegExp("^("+l.join("|")+")","i"),this._weekdaysShortStrictRegex=new RegExp("^("+s.join("|")+")","i"),this._weekdaysMinStrictRegex=new RegExp("^("+a.join("|")+")","i")}function Ze(){return this.hours()%12||12}function Qe(e,t){j(e,0,0,(function(){return this.localeData().meridiem(this.hours(),this.minutes(),t)}))}function Ke(e,t){return t._meridiemParse}j("H",["HH",2],0,"hour"),j("h",["hh",2],0,Ze),j("k",["kk",2],0,(function(){return this.hours()||24})),j("hmm",0,0,(function(){return""+Ze.apply(this)+k(this.minutes(),2)})),j("hmmss",0,0,(function(){return""+Ze.apply(this)+k(this.minutes(),2)+k(this.seconds(),2)})),j("Hmm",0,0,(function(){return""+this.hours()+k(this.minutes(),2)})),j("Hmmss",0,0,(function(){return""+this.hours()+k(this.minutes(),2)+k(this.seconds(),2)})),Qe("a",!0),Qe("A",!1),B("hour","h"),Y("hour",13),pe("a",Ke),pe("A",Ke),pe("H",re),pe("h",re),pe("k",re),pe("HH",re,$),pe("hh",re,$),pe("kk",re,$),pe("hmm",ie),pe("hmmss",oe),pe("Hmm",ie),pe("Hmmss",oe),be(["H","HH"],3),be(["k","kk"],(function(e,t,n){var r=X(e);t[3]=24===r?0:r})),be(["a","A"],(function(e,t,n){n._isPm=n._locale.isPM(e),n._meridiem=e})),be(["h","hh"],(function(e,t,n){t[3]=X(e),g(n).bigHour=!0})),be("hmm",(function(e,t,n){var r=e.length-2;t[3]=X(e.substr(0,r)),t[4]=X(e.substr(r)),g(n).bigHour=!0})),be("hmmss",(function(e,t,n){var r=e.length-4,i=e.length-2;t[3]=X(e.substr(0,r)),t[4]=X(e.substr(r,2)),t[5]=X(e.substr(i)),g(n).bigHour=!0})),be("Hmm",(function(e,t,n){var r=e.length-2;t[3]=X(e.substr(0,r)),t[4]=X(e.substr(r))})),be("Hmmss",(function(e,t,n){var r=e.length-4,i=e.length-2;t[3]=X(e.substr(0,r)),t[4]=X(e.substr(r,2)),t[5]=X(e.substr(i))}));var Je,$e=q("Hours",!0),et={calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},longDateFormat:{LTS:"h:mm:ss A",LT:"h:mm A",L:"MM/DD/YYYY",LL:"MMMM D, YYYY",LLL:"MMMM D, YYYY h:mm A",LLLL:"dddd, MMMM D, YYYY h:mm A"},invalidDate:"Invalid date",ordinal:"%d",dayOfMonthOrdinalParse:/\d{1,2}/,relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",w:"a week",ww:"%d weeks",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},months:Ie,monthsShort:xe,week:{dow:0,doy:6},weekdays:He,weekdaysMin:Ve,weekdaysShort:Ue,meridiemParse:/[ap]\.?m?\.?/i},tt={},nt={};function rt(e,t){var n,r=Math.min(e.length,t.length);for(n=0;n<r;n+=1)if(e[n]!==t[n])return n;return r}function it(e){return e?e.toLowerCase().replace("_","-"):e}function ot(t){var r=null;if(void 0===tt[t]&&void 0!==e&&e&&e.exports)try{r=Je._abbr,n(1354)("./"+t),at(r)}catch(e){tt[t]=null}return tt[t]}function at(e,t){var n;return e&&((n=u(t)?lt(e):st(e,t))?Je=n:"undefined"!=typeof console&&console.warn&&console.warn("Locale "+e+" not found. Did you forget to load it?")),Je._abbr}function st(e,t){if(null!==t){var n,r=et;if(t.abbr=e,null!=tt[e])S("defineLocaleOverride","use moment.updateLocale(localeName, config) to change an existing locale. moment.defineLocale(localeName, config) should only be used for creating a new locale See http://momentjs.com/guides/#/warnings/define-locale/ for more info."),r=tt[e]._config;else if(null!=t.parentLocale)if(null!=tt[t.parentLocale])r=tt[t.parentLocale]._config;else{if(null==(n=ot(t.parentLocale)))return nt[t.parentLocale]||(nt[t.parentLocale]=[]),nt[t.parentLocale].push({name:e,config:t}),null;r=n._config}return tt[e]=new O(E(r,t)),nt[e]&&nt[e].forEach((function(e){st(e.name,e.config)})),at(e),tt[e]}return delete tt[e],null}function lt(e){var t;if(e&&e._locale&&e._locale._abbr&&(e=e._locale._abbr),!e)return Je;if(!o(e)){if(t=ot(e))return t;e=[e]}return function(e){for(var t,n,r,i,o=0;o<e.length;){for(t=(i=it(e[o]).split("-")).length,n=(n=it(e[o+1]))?n.split("-"):null;t>0;){if(r=ot(i.slice(0,t).join("-")))return r;if(n&&n.length>=t&&rt(i,n)>=t-1)break;t--}o++}return Je}(e)}function ut(e){var t,n=e._a;return n&&-2===g(e).overflow&&(t=n[1]<0||n[1]>11?1:n[2]<1||n[2]>_e(n[0],n[1])?2:n[3]<0||n[3]>24||24===n[3]&&(0!==n[4]||0!==n[5]||0!==n[6])?3:n[4]<0||n[4]>59?4:n[5]<0||n[5]>59?5:n[6]<0||n[6]>999?6:-1,g(e)._overflowDayOfYear&&(t<0||t>2)&&(t=2),g(e)._overflowWeeks&&-1===t&&(t=7),g(e)._overflowWeekday&&-1===t&&(t=8),g(e).overflow=t),e}var ct=/^\s*((?:[+-]\d{6}|\d{4})-(?:\d\d-\d\d|W\d\d-\d|W\d\d|\d\d\d|\d\d))(?:(T| )(\d\d(?::\d\d(?::\d\d(?:[.,]\d+)?)?)?)([+-]\d\d(?::?\d\d)?|\s*Z)?)?$/,dt=/^\s*((?:[+-]\d{6}|\d{4})(?:\d\d\d\d|W\d\d\d|W\d\d|\d\d\d|\d\d|))(?:(T| )(\d\d(?:\d\d(?:\d\d(?:[.,]\d+)?)?)?)([+-]\d\d(?::?\d\d)?|\s*Z)?)?$/,ht=/Z|[+-]\d\d(?::?\d\d)?/,ft=[["YYYYYY-MM-DD",/[+-]\d{6}-\d\d-\d\d/],["YYYY-MM-DD",/\d{4}-\d\d-\d\d/],["GGGG-[W]WW-E",/\d{4}-W\d\d-\d/],["GGGG-[W]WW",/\d{4}-W\d\d/,!1],["YYYY-DDD",/\d{4}-\d{3}/],["YYYY-MM",/\d{4}-\d\d/,!1],["YYYYYYMMDD",/[+-]\d{10}/],["YYYYMMDD",/\d{8}/],["GGGG[W]WWE",/\d{4}W\d{3}/],["GGGG[W]WW",/\d{4}W\d{2}/,!1],["YYYYDDD",/\d{7}/],["YYYYMM",/\d{6}/,!1],["YYYY",/\d{4}/,!1]],pt=[["HH:mm:ss.SSSS",/\d\d:\d\d:\d\d\.\d+/],["HH:mm:ss,SSSS",/\d\d:\d\d:\d\d,\d+/],["HH:mm:ss",/\d\d:\d\d:\d\d/],["HH:mm",/\d\d:\d\d/],["HHmmss.SSSS",/\d\d\d\d\d\d\.\d+/],["HHmmss,SSSS",/\d\d\d\d\d\d,\d+/],["HHmmss",/\d\d\d\d\d\d/],["HHmm",/\d\d\d\d/],["HH",/\d\d/]],gt=/^\/?Date\((-?\d+)/i,vt=/^(?:(Mon|Tue|Wed|Thu|Fri|Sat|Sun),?\s)?(\d{1,2})\s(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)\s(\d{2,4})\s(\d\d):(\d\d)(?::(\d\d))?\s(?:(UT|GMT|[ECMP][SD]T)|([Zz])|([+-]\d{4}))$/,mt={UT:0,GMT:0,EDT:-240,EST:-300,CDT:-300,CST:-360,MDT:-360,MST:-420,PDT:-420,PST:-480};function yt(e){var t,n,r,i,o,a,s=e._i,l=ct.exec(s)||dt.exec(s);if(l){for(g(e).iso=!0,t=0,n=ft.length;t<n;t++)if(ft[t][1].exec(l[1])){i=ft[t][0],r=!1!==ft[t][2];break}if(null==i)return void(e._isValid=!1);if(l[3]){for(t=0,n=pt.length;t<n;t++)if(pt[t][1].exec(l[3])){o=(l[2]||" ")+pt[t][0];break}if(null==o)return void(e._isValid=!1)}if(!r&&null!=o)return void(e._isValid=!1);if(l[4]){if(!ht.exec(l[4]))return void(e._isValid=!1);a="Z"}e._f=i+(o||"")+(a||""),It(e)}else e._isValid=!1}function bt(e){var t=parseInt(e,10);return t<=49?2e3+t:t<=999?1900+t:t}function Ct(e){var t,n,r,i,o,a,s,l,u=vt.exec(e._i.replace(/\([^)]*\)|[\n\t]/g," ").replace(/(\s\s+)/g," ").replace(/^\s\s*/,"").replace(/\s\s*$/,""));if(u){if(n=u[4],r=u[3],i=u[2],o=u[5],a=u[6],s=u[7],l=[bt(n),xe.indexOf(r),parseInt(i,10),parseInt(o,10),parseInt(a,10)],s&&l.push(parseInt(s,10)),t=l,!function(e,t,n){return!e||Ue.indexOf(e)===new Date(t[0],t[1],t[2]).getDay()||(g(n).weekdayMismatch=!0,n._isValid=!1,!1)}(u[1],t,e))return;e._a=t,e._tzm=function(e,t,n){if(e)return mt[e];if(t)return 0;var r=parseInt(n,10),i=r%100;return(r-i)/100*60+i}(u[8],u[9],u[10]),e._d=Ne.apply(null,e._a),e._d.setUTCMinutes(e._d.getUTCMinutes()-e._tzm),g(e).rfc2822=!0}else e._isValid=!1}function At(e,t,n){return null!=e?e:null!=t?t:n}function _t(e){var t,n,r,o,a,s=[];if(!e._d){for(r=function(e){var t=new Date(i.now());return e._useUTC?[t.getUTCFullYear(),t.getUTCMonth(),t.getUTCDate()]:[t.getFullYear(),t.getMonth(),t.getDate()]}(e),e._w&&null==e._a[2]&&null==e._a[1]&&function(e){var t,n,r,i,o,a,s,l,u;null!=(t=e._w).GG||null!=t.W||null!=t.E?(o=1,a=4,n=At(t.GG,e._a[0],ze(Mt(),1,4).year),r=At(t.W,1),((i=At(t.E,1))<1||i>7)&&(l=!0)):(o=e._locale._week.dow,a=e._locale._week.doy,u=ze(Mt(),o,a),n=At(t.gg,e._a[0],u.year),r=At(t.w,u.week),null!=t.d?((i=t.d)<0||i>6)&&(l=!0):null!=t.e?(i=t.e+o,(t.e<0||t.e>6)&&(l=!0)):i=o),r<1||r>Fe(n,o,a)?g(e)._overflowWeeks=!0:null!=l?g(e)._overflowWeekday=!0:(s=Re(n,r,i,o,a),e._a[0]=s.year,e._dayOfYear=s.dayOfYear)}(e),null!=e._dayOfYear&&(a=At(e._a[0],r[0]),(e._dayOfYear>Le(a)||0===e._dayOfYear)&&(g(e)._overflowDayOfYear=!0),n=Ne(a,0,e._dayOfYear),e._a[1]=n.getUTCMonth(),e._a[2]=n.getUTCDate()),t=0;t<3&&null==e._a[t];++t)e._a[t]=s[t]=r[t];for(;t<7;t++)e._a[t]=s[t]=null==e._a[t]?2===t?1:0:e._a[t];24===e._a[3]&&0===e._a[4]&&0===e._a[5]&&0===e._a[6]&&(e._nextDay=!0,e._a[3]=0),e._d=(e._useUTC?Ne:Pe).apply(null,s),o=e._useUTC?e._d.getUTCDay():e._d.getDay(),null!=e._tzm&&e._d.setUTCMinutes(e._d.getUTCMinutes()-e._tzm),e._nextDay&&(e._a[3]=24),e._w&&void 0!==e._w.d&&e._w.d!==o&&(g(e).weekdayMismatch=!0)}}function It(e){if(e._f!==i.ISO_8601)if(e._f!==i.RFC_2822){e._a=[],g(e).empty=!0;var t,n,r,o,a,s,l=""+e._i,u=l.length,c=0;for(r=z(e._f,e._locale).match(L)||[],t=0;t<r.length;t++)o=r[t],(n=(l.match(ge(o,e))||[])[0])&&((a=l.substr(0,l.indexOf(n))).length>0&&g(e).unusedInput.push(a),l=l.slice(l.indexOf(n)+n.length),c+=n.length),N[o]?(n?g(e).empty=!1:g(e).unusedTokens.push(o),Ae(o,n,e)):e._strict&&!n&&g(e).unusedTokens.push(o);g(e).charsLeftOver=u-c,l.length>0&&g(e).unusedInput.push(l),e._a[3]<=12&&!0===g(e).bigHour&&e._a[3]>0&&(g(e).bigHour=void 0),g(e).parsedDateParts=e._a.slice(0),g(e).meridiem=e._meridiem,e._a[3]=function(e,t,n){var r;return null==n?t:null!=e.meridiemHour?e.meridiemHour(t,n):null!=e.isPM?((r=e.isPM(n))&&t<12&&(t+=12),r||12!==t||(t=0),t):t}(e._locale,e._a[3],e._meridiem),null!==(s=g(e).era)&&(e._a[0]=e._locale.erasConvertYear(s,e._a[0])),_t(e),ut(e)}else Ct(e);else yt(e)}function xt(e){var t=e._i,n=e._f;return e._locale=e._locale||lt(e._l),null===t||void 0===n&&""===t?m({nullInput:!0}):("string"==typeof t&&(e._i=t=e._locale.preparse(t)),_(t)?new A(ut(t)):(d(t)?e._d=t:o(n)?function(e){var t,n,r,i,o,a,s=!1;if(0===e._f.length)return g(e).invalidFormat=!0,void(e._d=new Date(NaN));for(i=0;i<e._f.length;i++)o=0,a=!1,t=C({},e),null!=e._useUTC&&(t._useUTC=e._useUTC),t._f=e._f[i],It(t),v(t)&&(a=!0),o+=g(t).charsLeftOver,o+=10*g(t).unusedTokens.length,g(t).score=o,s?o<r&&(r=o,n=t):(null==r||o<r||a)&&(r=o,n=t,a&&(s=!0));f(e,n||t)}(e):n?It(e):function(e){var t=e._i;u(t)?e._d=new Date(i.now()):d(t)?e._d=new Date(t.valueOf()):"string"==typeof t?function(e){var t=gt.exec(e._i);null===t?(yt(e),!1===e._isValid&&(delete e._isValid,Ct(e),!1===e._isValid&&(delete e._isValid,e._strict?e._isValid=!1:i.createFromInputFallback(e)))):e._d=new Date(+t[1])}(e):o(t)?(e._a=h(t.slice(0),(function(e){return parseInt(e,10)})),_t(e)):a(t)?function(e){if(!e._d){var t=U(e._i),n=void 0===t.day?t.date:t.day;e._a=h([t.year,t.month,n,t.hour,t.minute,t.second,t.millisecond],(function(e){return e&&parseInt(e,10)})),_t(e)}}(e):c(t)?e._d=new Date(t):i.createFromInputFallback(e)}(e),v(e)||(e._d=null),e))}function wt(e,t,n,r,i){var s,u={};return!0!==t&&!1!==t||(r=t,t=void 0),!0!==n&&!1!==n||(r=n,n=void 0),(a(e)&&l(e)||o(e)&&0===e.length)&&(e=void 0),u._isAMomentObject=!0,u._useUTC=u._isUTC=i,u._l=n,u._i=e,u._f=t,u._strict=r,(s=new A(ut(xt(u))))._nextDay&&(s.add(1,"d"),s._nextDay=void 0),s}function Mt(e,t,n,r){return wt(e,t,n,r,!1)}i.createFromInputFallback=x("value provided is not in a recognized RFC2822 or ISO format. moment construction falls back to js Date(), which is not reliable across all browsers and versions. Non RFC2822/ISO date formats are discouraged. Please refer to http://momentjs.com/guides/#/warnings/js-date/ for more info.",(function(e){e._d=new Date(e._i+(e._useUTC?" UTC":""))})),i.ISO_8601=function(){},i.RFC_2822=function(){};var St=x("moment().min is deprecated, use moment.max instead. http://momentjs.com/guides/#/warnings/min-max/",(function(){var e=Mt.apply(null,arguments);return this.isValid()&&e.isValid()?e<this?this:e:m()})),Tt=x("moment().max is deprecated, use moment.min instead. http://momentjs.com/guides/#/warnings/min-max/",(function(){var e=Mt.apply(null,arguments);return this.isValid()&&e.isValid()?e>this?this:e:m()}));function Et(e,t){var n,r;if(1===t.length&&o(t[0])&&(t=t[0]),!t.length)return Mt();for(n=t[0],r=1;r<t.length;++r)t[r].isValid()&&!t[r][e](n)||(n=t[r]);return n}var Ot=["year","quarter","month","week","day","hour","minute","second","millisecond"];function kt(e){var t=U(e),n=t.year||0,r=t.quarter||0,i=t.month||0,o=t.week||t.isoWeek||0,a=t.day||0,l=t.hour||0,u=t.minute||0,c=t.second||0,d=t.millisecond||0;this._isValid=function(e){var t,n,r=!1;for(t in e)if(s(e,t)&&(-1===me.call(Ot,t)||null!=e[t]&&isNaN(e[t])))return!1;for(n=0;n<Ot.length;++n)if(e[Ot[n]]){if(r)return!1;parseFloat(e[Ot[n]])!==X(e[Ot[n]])&&(r=!0)}return!0}(t),this._milliseconds=+d+1e3*c+6e4*u+1e3*l*60*60,this._days=+a+7*o,this._months=+i+3*r+12*n,this._data={},this._locale=lt(),this._bubble()}function Lt(e){return e instanceof kt}function Dt(e){return e<0?-1*Math.round(-1*e):Math.round(e)}function Pt(e,t){j(e,0,0,(function(){var e=this.utcOffset(),n="+";return e<0&&(e=-e,n="-"),n+k(~~(e/60),2)+t+k(~~e%60,2)}))}Pt("Z",":"),Pt("ZZ",""),pe("Z",he),pe("ZZ",he),be(["Z","ZZ"],(function(e,t,n){n._useUTC=!0,n._tzm=jt(he,e)}));var Nt=/([\+\-]|\d\d)/gi;function jt(e,t){var n,r,i=(t||"").match(e);return null===i?null:0===(r=60*(n=((i[i.length-1]||[])+"").match(Nt)||["-",0,0])[1]+X(n[2]))?0:"+"===n[0]?r:-r}function Rt(e,t){var n,r;return t._isUTC?(n=t.clone(),r=(_(e)||d(e)?e.valueOf():Mt(e).valueOf())-n.valueOf(),n._d.setTime(n._d.valueOf()+r),i.updateOffset(n,!1),n):Mt(e).local()}function zt(e){return-Math.round(e._d.getTimezoneOffset())}function Ft(){return!!this.isValid()&&this._isUTC&&0===this._offset}i.updateOffset=function(){};var Bt=/^(-|\+)?(?:(\d*)[. ])?(\d+):(\d+)(?::(\d+)(\.\d*)?)?$/,Ht=/^(-|\+)?P(?:([-+]?[0-9,.]*)Y)?(?:([-+]?[0-9,.]*)M)?(?:([-+]?[0-9,.]*)W)?(?:([-+]?[0-9,.]*)D)?(?:T(?:([-+]?[0-9,.]*)H)?(?:([-+]?[0-9,.]*)M)?(?:([-+]?[0-9,.]*)S)?)?$/;function Ut(e,t){var n,r,i,o,a,l,u=e,d=null;return Lt(e)?u={ms:e._milliseconds,d:e._days,M:e._months}:c(e)||!isNaN(+e)?(u={},t?u[t]=+e:u.milliseconds=+e):(d=Bt.exec(e))?(n="-"===d[1]?-1:1,u={y:0,d:X(d[2])*n,h:X(d[3])*n,m:X(d[4])*n,s:X(d[5])*n,ms:X(Dt(1e3*d[6]))*n}):(d=Ht.exec(e))?(n="-"===d[1]?-1:1,u={y:Vt(d[2],n),M:Vt(d[3],n),w:Vt(d[4],n),d:Vt(d[5],n),h:Vt(d[6],n),m:Vt(d[7],n),s:Vt(d[8],n)}):null==u?u={}:"object"==typeof u&&("from"in u||"to"in u)&&(o=Mt(u.from),a=Mt(u.to),i=o.isValid()&&a.isValid()?(a=Rt(a,o),o.isBefore(a)?l=Yt(o,a):((l=Yt(a,o)).milliseconds=-l.milliseconds,l.months=-l.months),l):{milliseconds:0,months:0},(u={}).ms=i.milliseconds,u.M=i.months),r=new kt(u),Lt(e)&&s(e,"_locale")&&(r._locale=e._locale),Lt(e)&&s(e,"_isValid")&&(r._isValid=e._isValid),r}function Vt(e,t){var n=e&&parseFloat(e.replace(",","."));return(isNaN(n)?0:n)*t}function Yt(e,t){var n={};return n.months=t.month()-e.month()+12*(t.year()-e.year()),e.clone().add(n.months,"M").isAfter(t)&&--n.months,n.milliseconds=+t-+e.clone().add(n.months,"M"),n}function Wt(e,t){return function(n,r){var i;return null===r||isNaN(+r)||(S(t,"moment()."+t+"(period, number) is deprecated. Please use moment()."+t+"(number, period). See http://momentjs.com/guides/#/warnings/add-inverted-param/ for more info."),i=n,n=r,r=i),Gt(this,Ut(n,r),e),this}}function Gt(e,t,n,r){var o=t._milliseconds,a=Dt(t._days),s=Dt(t._months);e.isValid()&&(r=null==r||r,s&&Ee(e,Z(e,"Month")+s*n),a&&Q(e,"Date",Z(e,"Date")+a*n),o&&e._d.setTime(e._d.valueOf()+o*n),r&&i.updateOffset(e,a||s))}Ut.fn=kt.prototype,Ut.invalid=function(){return Ut(NaN)};var Xt=Wt(1,"add"),qt=Wt(-1,"subtract");function Zt(e){return"string"==typeof e||e instanceof String}function Qt(e){return _(e)||d(e)||Zt(e)||c(e)||function(e){var t=o(e),n=!1;return t&&(n=0===e.filter((function(t){return!c(t)&&Zt(e)})).length),t&&n}(e)||function(e){var t,n,r=a(e)&&!l(e),i=!1,o=["years","year","y","months","month","M","days","day","d","dates","date","D","hours","hour","h","minutes","minute","m","seconds","second","s","milliseconds","millisecond","ms"];for(t=0;t<o.length;t+=1)n=o[t],i=i||s(e,n);return r&&i}(e)||null==e}function Kt(e){var t,n=a(e)&&!l(e),r=!1,i=["sameDay","nextDay","lastDay","nextWeek","lastWeek","sameElse"];for(t=0;t<i.length;t+=1)r=r||s(e,i[t]);return n&&r}function Jt(e,t){if(e.date()<t.date())return-Jt(t,e);var n=12*(t.year()-e.year())+(t.month()-e.month()),r=e.clone().add(n,"months");return-(n+(t-r<0?(t-r)/(r-e.clone().add(n-1,"months")):(t-r)/(e.clone().add(n+1,"months")-r)))||0}function $t(e){var t;return void 0===e?this._locale._abbr:(null!=(t=lt(e))&&(this._locale=t),this)}i.defaultFormat="YYYY-MM-DDTHH:mm:ssZ",i.defaultFormatUtc="YYYY-MM-DDTHH:mm:ss[Z]";var en=x("moment().lang() is deprecated. Instead, use moment().localeData() to get the language configuration. Use moment().locale() to change languages.",(function(e){return void 0===e?this.localeData():this.locale(e)}));function tn(){return this._locale}function nn(e,t){return(e%t+t)%t}function rn(e,t,n){return e<100&&e>=0?new Date(e+400,t,n)-126227808e5:new Date(e,t,n).valueOf()}function on(e,t,n){return e<100&&e>=0?Date.UTC(e+400,t,n)-126227808e5:Date.UTC(e,t,n)}function an(e,t){return t.erasAbbrRegex(e)}function sn(){var e,t,n=[],r=[],i=[],o=[],a=this.eras();for(e=0,t=a.length;e<t;++e)r.push(ve(a[e].name)),n.push(ve(a[e].abbr)),i.push(ve(a[e].narrow)),o.push(ve(a[e].name)),o.push(ve(a[e].abbr)),o.push(ve(a[e].narrow));this._erasRegex=new RegExp("^("+o.join("|")+")","i"),this._erasNameRegex=new RegExp("^("+r.join("|")+")","i"),this._erasAbbrRegex=new RegExp("^("+n.join("|")+")","i"),this._erasNarrowRegex=new RegExp("^("+i.join("|")+")","i")}function ln(e,t){j(0,[e,e.length],0,t)}function un(e,t,n,r,i){var o;return null==e?ze(this,r,i).year:(t>(o=Fe(e,r,i))&&(t=o),cn.call(this,e,t,n,r,i))}function cn(e,t,n,r,i){var o=Re(e,t,n,r,i),a=Ne(o.year,0,o.dayOfYear);return this.year(a.getUTCFullYear()),this.month(a.getUTCMonth()),this.date(a.getUTCDate()),this}j("N",0,0,"eraAbbr"),j("NN",0,0,"eraAbbr"),j("NNN",0,0,"eraAbbr"),j("NNNN",0,0,"eraName"),j("NNNNN",0,0,"eraNarrow"),j("y",["y",1],"yo","eraYear"),j("y",["yy",2],0,"eraYear"),j("y",["yyy",3],0,"eraYear"),j("y",["yyyy",4],0,"eraYear"),pe("N",an),pe("NN",an),pe("NNN",an),pe("NNNN",(function(e,t){return t.erasNameRegex(e)})),pe("NNNNN",(function(e,t){return t.erasNarrowRegex(e)})),be(["N","NN","NNN","NNNN","NNNNN"],(function(e,t,n,r){var i=n._locale.erasParse(e,r,n._strict);i?g(n).era=i:g(n).invalidEra=e})),pe("y",ue),pe("yy",ue),pe("yyy",ue),pe("yyyy",ue),pe("yo",(function(e,t){return t._eraYearOrdinalRegex||ue})),be(["y","yy","yyy","yyyy"],0),be(["yo"],(function(e,t,n,r){var i;n._locale._eraYearOrdinalRegex&&(i=e.match(n._locale._eraYearOrdinalRegex)),n._locale.eraYearOrdinalParse?t[0]=n._locale.eraYearOrdinalParse(e,i):t[0]=parseInt(e,10)})),j(0,["gg",2],0,(function(){return this.weekYear()%100})),j(0,["GG",2],0,(function(){return this.isoWeekYear()%100})),ln("gggg","weekYear"),ln("ggggg","weekYear"),ln("GGGG","isoWeekYear"),ln("GGGGG","isoWeekYear"),B("weekYear","gg"),B("isoWeekYear","GG"),Y("weekYear",1),Y("isoWeekYear",1),pe("G",ce),pe("g",ce),pe("GG",re,$),pe("gg",re,$),pe("GGGG",se,te),pe("gggg",se,te),pe("GGGGG",le,ne),pe("ggggg",le,ne),Ce(["gggg","ggggg","GGGG","GGGGG"],(function(e,t,n,r){t[r.substr(0,2)]=X(e)})),Ce(["gg","GG"],(function(e,t,n,r){t[r]=i.parseTwoDigitYear(e)})),j("Q",0,"Qo","quarter"),B("quarter","Q"),Y("quarter",7),pe("Q",J),be("Q",(function(e,t){t[1]=3*(X(e)-1)})),j("D",["DD",2],"Do","date"),B("date","D"),Y("date",9),pe("D",re),pe("DD",re,$),pe("Do",(function(e,t){return e?t._dayOfMonthOrdinalParse||t._ordinalParse:t._dayOfMonthOrdinalParseLenient})),be(["D","DD"],2),be("Do",(function(e,t){t[2]=X(e.match(re)[0])}));var dn=q("Date",!0);j("DDD",["DDDD",3],"DDDo","dayOfYear"),B("dayOfYear","DDD"),Y("dayOfYear",4),pe("DDD",ae),pe("DDDD",ee),be(["DDD","DDDD"],(function(e,t,n){n._dayOfYear=X(e)})),j("m",["mm",2],0,"minute"),B("minute","m"),Y("minute",14),pe("m",re),pe("mm",re,$),be(["m","mm"],4);var hn=q("Minutes",!1);j("s",["ss",2],0,"second"),B("second","s"),Y("second",15),pe("s",re),pe("ss",re,$),be(["s","ss"],5);var fn,pn,gn=q("Seconds",!1);for(j("S",0,0,(function(){return~~(this.millisecond()/100)})),j(0,["SS",2],0,(function(){return~~(this.millisecond()/10)})),j(0,["SSS",3],0,"millisecond"),j(0,["SSSS",4],0,(function(){return 10*this.millisecond()})),j(0,["SSSSS",5],0,(function(){return 100*this.millisecond()})),j(0,["SSSSSS",6],0,(function(){return 1e3*this.millisecond()})),j(0,["SSSSSSS",7],0,(function(){return 1e4*this.millisecond()})),j(0,["SSSSSSSS",8],0,(function(){return 1e5*this.millisecond()})),j(0,["SSSSSSSSS",9],0,(function(){return 1e6*this.millisecond()})),B("millisecond","ms"),Y("millisecond",16),pe("S",ae,J),pe("SS",ae,$),pe("SSS",ae,ee),fn="SSSS";fn.length<=9;fn+="S")pe(fn,ue);function vn(e,t){t[6]=X(1e3*("0."+e))}for(fn="S";fn.length<=9;fn+="S")be(fn,vn);pn=q("Milliseconds",!1),j("z",0,0,"zoneAbbr"),j("zz",0,0,"zoneName");var mn=A.prototype;function yn(e){return e}mn.add=Xt,mn.calendar=function(e,t){1===arguments.length&&(arguments[0]?Qt(arguments[0])?(e=arguments[0],t=void 0):Kt(arguments[0])&&(t=arguments[0],e=void 0):(e=void 0,t=void 0));var n=e||Mt(),r=Rt(n,this).startOf("day"),o=i.calendarFormat(this,r)||"sameElse",a=t&&(T(t[o])?t[o].call(this,n):t[o]);return this.format(a||this.localeData().calendar(o,this,Mt(n)))},mn.clone=function(){return new A(this)},mn.diff=function(e,t,n){var r,i,o;if(!this.isValid())return NaN;if(!(r=Rt(e,this)).isValid())return NaN;switch(i=6e4*(r.utcOffset()-this.utcOffset()),t=H(t)){case"year":o=Jt(this,r)/12;break;case"month":o=Jt(this,r);break;case"quarter":o=Jt(this,r)/3;break;case"second":o=(this-r)/1e3;break;case"minute":o=(this-r)/6e4;break;case"hour":o=(this-r)/36e5;break;case"day":o=(this-r-i)/864e5;break;case"week":o=(this-r-i)/6048e5;break;default:o=this-r}return n?o:G(o)},mn.endOf=function(e){var t,n;if(void 0===(e=H(e))||"millisecond"===e||!this.isValid())return this;switch(n=this._isUTC?on:rn,e){case"year":t=n(this.year()+1,0,1)-1;break;case"quarter":t=n(this.year(),this.month()-this.month()%3+3,1)-1;break;case"month":t=n(this.year(),this.month()+1,1)-1;break;case"week":t=n(this.year(),this.month(),this.date()-this.weekday()+7)-1;break;case"isoWeek":t=n(this.year(),this.month(),this.date()-(this.isoWeekday()-1)+7)-1;break;case"day":case"date":t=n(this.year(),this.month(),this.date()+1)-1;break;case"hour":t=this._d.valueOf(),t+=36e5-nn(t+(this._isUTC?0:6e4*this.utcOffset()),36e5)-1;break;case"minute":t=this._d.valueOf(),t+=6e4-nn(t,6e4)-1;break;case"second":t=this._d.valueOf(),t+=1e3-nn(t,1e3)-1}return this._d.setTime(t),i.updateOffset(this,!0),this},mn.format=function(e){e||(e=this.isUtc()?i.defaultFormatUtc:i.defaultFormat);var t=R(this,e);return this.localeData().postformat(t)},mn.from=function(e,t){return this.isValid()&&(_(e)&&e.isValid()||Mt(e).isValid())?Ut({to:this,from:e}).locale(this.locale()).humanize(!t):this.localeData().invalidDate()},mn.fromNow=function(e){return this.from(Mt(),e)},mn.to=function(e,t){return this.isValid()&&(_(e)&&e.isValid()||Mt(e).isValid())?Ut({from:this,to:e}).locale(this.locale()).humanize(!t):this.localeData().invalidDate()},mn.toNow=function(e){return this.to(Mt(),e)},mn.get=function(e){return T(this[e=H(e)])?this[e]():this},mn.invalidAt=function(){return g(this).overflow},mn.isAfter=function(e,t){var n=_(e)?e:Mt(e);return!(!this.isValid()||!n.isValid())&&("millisecond"===(t=H(t)||"millisecond")?this.valueOf()>n.valueOf():n.valueOf()<this.clone().startOf(t).valueOf())},mn.isBefore=function(e,t){var n=_(e)?e:Mt(e);return!(!this.isValid()||!n.isValid())&&("millisecond"===(t=H(t)||"millisecond")?this.valueOf()<n.valueOf():this.clone().endOf(t).valueOf()<n.valueOf())},mn.isBetween=function(e,t,n,r){var i=_(e)?e:Mt(e),o=_(t)?t:Mt(t);return!!(this.isValid()&&i.isValid()&&o.isValid())&&(("("===(r=r||"()")[0]?this.isAfter(i,n):!this.isBefore(i,n))&&(")"===r[1]?this.isBefore(o,n):!this.isAfter(o,n)))},mn.isSame=function(e,t){var n,r=_(e)?e:Mt(e);return!(!this.isValid()||!r.isValid())&&("millisecond"===(t=H(t)||"millisecond")?this.valueOf()===r.valueOf():(n=r.valueOf(),this.clone().startOf(t).valueOf()<=n&&n<=this.clone().endOf(t).valueOf()))},mn.isSameOrAfter=function(e,t){return this.isSame(e,t)||this.isAfter(e,t)},mn.isSameOrBefore=function(e,t){return this.isSame(e,t)||this.isBefore(e,t)},mn.isValid=function(){return v(this)},mn.lang=en,mn.locale=$t,mn.localeData=tn,mn.max=Tt,mn.min=St,mn.parsingFlags=function(){return f({},g(this))},mn.set=function(e,t){if("object"==typeof e){var n,r=function(e){var t,n=[];for(t in e)s(e,t)&&n.push({unit:t,priority:V[t]});return n.sort((function(e,t){return e.priority-t.priority})),n}(e=U(e));for(n=0;n<r.length;n++)this[r[n].unit](e[r[n].unit])}else if(T(this[e=H(e)]))return this[e](t);return this},mn.startOf=function(e){var t,n;if(void 0===(e=H(e))||"millisecond"===e||!this.isValid())return this;switch(n=this._isUTC?on:rn,e){case"year":t=n(this.year(),0,1);break;case"quarter":t=n(this.year(),this.month()-this.month()%3,1);break;case"month":t=n(this.year(),this.month(),1);break;case"week":t=n(this.year(),this.month(),this.date()-this.weekday());break;case"isoWeek":t=n(this.year(),this.month(),this.date()-(this.isoWeekday()-1));break;case"day":case"date":t=n(this.year(),this.month(),this.date());break;case"hour":t=this._d.valueOf(),t-=nn(t+(this._isUTC?0:6e4*this.utcOffset()),36e5);break;case"minute":t=this._d.valueOf(),t-=nn(t,6e4);break;case"second":t=this._d.valueOf(),t-=nn(t,1e3)}return this._d.setTime(t),i.updateOffset(this,!0),this},mn.subtract=qt,mn.toArray=function(){var e=this;return[e.year(),e.month(),e.date(),e.hour(),e.minute(),e.second(),e.millisecond()]},mn.toObject=function(){var e=this;return{years:e.year(),months:e.month(),date:e.date(),hours:e.hours(),minutes:e.minutes(),seconds:e.seconds(),milliseconds:e.milliseconds()}},mn.toDate=function(){return new Date(this.valueOf())},mn.toISOString=function(e){if(!this.isValid())return null;var t=!0!==e,n=t?this.clone().utc():this;return n.year()<0||n.year()>9999?R(n,t?"YYYYYY-MM-DD[T]HH:mm:ss.SSS[Z]":"YYYYYY-MM-DD[T]HH:mm:ss.SSSZ"):T(Date.prototype.toISOString)?t?this.toDate().toISOString():new Date(this.valueOf()+60*this.utcOffset()*1e3).toISOString().replace("Z",R(n,"Z")):R(n,t?"YYYY-MM-DD[T]HH:mm:ss.SSS[Z]":"YYYY-MM-DD[T]HH:mm:ss.SSSZ")},mn.inspect=function(){if(!this.isValid())return"moment.invalid(/* "+this._i+" */)";var e,t,n,r="moment",i="";return this.isLocal()||(r=0===this.utcOffset()?"moment.utc":"moment.parseZone",i="Z"),e="["+r+'("]',t=0<=this.year()&&this.year()<=9999?"YYYY":"YYYYYY",n=i+'[")]',this.format(e+t+"-MM-DD[T]HH:mm:ss.SSS"+n)},"undefined"!=typeof Symbol&&null!=Symbol.for&&(mn[Symbol.for("nodejs.util.inspect.custom")]=function(){return"Moment<"+this.format()+">"}),mn.toJSON=function(){return this.isValid()?this.toISOString():null},mn.toString=function(){return this.clone().locale("en").format("ddd MMM DD YYYY HH:mm:ss [GMT]ZZ")},mn.unix=function(){return Math.floor(this.valueOf()/1e3)},mn.valueOf=function(){return this._d.valueOf()-6e4*(this._offset||0)},mn.creationData=function(){return{input:this._i,format:this._f,locale:this._locale,isUTC:this._isUTC,strict:this._strict}},mn.eraName=function(){var e,t,n,r=this.localeData().eras();for(e=0,t=r.length;e<t;++e){if(n=this.clone().startOf("day").valueOf(),r[e].since<=n&&n<=r[e].until)return r[e].name;if(r[e].until<=n&&n<=r[e].since)return r[e].name}return""},mn.eraNarrow=function(){var e,t,n,r=this.localeData().eras();for(e=0,t=r.length;e<t;++e){if(n=this.clone().startOf("day").valueOf(),r[e].since<=n&&n<=r[e].until)return r[e].narrow;if(r[e].until<=n&&n<=r[e].since)return r[e].narrow}return""},mn.eraAbbr=function(){var e,t,n,r=this.localeData().eras();for(e=0,t=r.length;e<t;++e){if(n=this.clone().startOf("day").valueOf(),r[e].since<=n&&n<=r[e].until)return r[e].abbr;if(r[e].until<=n&&n<=r[e].since)return r[e].abbr}return""},mn.eraYear=function(){var e,t,n,r,o=this.localeData().eras();for(e=0,t=o.length;e<t;++e)if(n=o[e].since<=o[e].until?1:-1,r=this.clone().startOf("day").valueOf(),o[e].since<=r&&r<=o[e].until||o[e].until<=r&&r<=o[e].since)return(this.year()-i(o[e].since).year())*n+o[e].offset;return this.year()},mn.year=De,mn.isLeapYear=function(){return W(this.year())},mn.weekYear=function(e){return un.call(this,e,this.week(),this.weekday(),this.localeData()._week.dow,this.localeData()._week.doy)},mn.isoWeekYear=function(e){return un.call(this,e,this.isoWeek(),this.isoWeekday(),1,4)},mn.quarter=mn.quarters=function(e){return null==e?Math.ceil((this.month()+1)/3):this.month(3*(e-1)+this.month()%3)},mn.month=Oe,mn.daysInMonth=function(){return _e(this.year(),this.month())},mn.week=mn.weeks=function(e){var t=this.localeData().week(this);return null==e?t:this.add(7*(e-t),"d")},mn.isoWeek=mn.isoWeeks=function(e){var t=ze(this,1,4).week;return null==e?t:this.add(7*(e-t),"d")},mn.weeksInYear=function(){var e=this.localeData()._week;return Fe(this.year(),e.dow,e.doy)},mn.weeksInWeekYear=function(){var e=this.localeData()._week;return Fe(this.weekYear(),e.dow,e.doy)},mn.isoWeeksInYear=function(){return Fe(this.year(),1,4)},mn.isoWeeksInISOWeekYear=function(){return Fe(this.isoWeekYear(),1,4)},mn.date=dn,mn.day=mn.days=function(e){if(!this.isValid())return null!=e?this:NaN;var t=this._isUTC?this._d.getUTCDay():this._d.getDay();return null!=e?(e=function(e,t){return"string"!=typeof e?e:isNaN(e)?"number"==typeof(e=t.weekdaysParse(e))?e:null:parseInt(e,10)}(e,this.localeData()),this.add(e-t,"d")):t},mn.weekday=function(e){if(!this.isValid())return null!=e?this:NaN;var t=(this.day()+7-this.localeData()._week.dow)%7;return null==e?t:this.add(e-t,"d")},mn.isoWeekday=function(e){if(!this.isValid())return null!=e?this:NaN;if(null!=e){var t=function(e,t){return"string"==typeof e?t.weekdaysParse(e)%7||7:isNaN(e)?null:e}(e,this.localeData());return this.day(this.day()%7?t:t-7)}return this.day()||7},mn.dayOfYear=function(e){var t=Math.round((this.clone().startOf("day")-this.clone().startOf("year"))/864e5)+1;return null==e?t:this.add(e-t,"d")},mn.hour=mn.hours=$e,mn.minute=mn.minutes=hn,mn.second=mn.seconds=gn,mn.millisecond=mn.milliseconds=pn,mn.utcOffset=function(e,t,n){var r,o=this._offset||0;if(!this.isValid())return null!=e?this:NaN;if(null!=e){if("string"==typeof e){if(null===(e=jt(he,e)))return this}else Math.abs(e)<16&&!n&&(e*=60);return!this._isUTC&&t&&(r=zt(this)),this._offset=e,this._isUTC=!0,null!=r&&this.add(r,"m"),o!==e&&(!t||this._changeInProgress?Gt(this,Ut(e-o,"m"),1,!1):this._changeInProgress||(this._changeInProgress=!0,i.updateOffset(this,!0),this._changeInProgress=null)),this}return this._isUTC?o:zt(this)},mn.utc=function(e){return this.utcOffset(0,e)},mn.local=function(e){return this._isUTC&&(this.utcOffset(0,e),this._isUTC=!1,e&&this.subtract(zt(this),"m")),this},mn.parseZone=function(){if(null!=this._tzm)this.utcOffset(this._tzm,!1,!0);else if("string"==typeof this._i){var e=jt(de,this._i);null!=e?this.utcOffset(e):this.utcOffset(0,!0)}return this},mn.hasAlignedHourOffset=function(e){return!!this.isValid()&&(e=e?Mt(e).utcOffset():0,(this.utcOffset()-e)%60==0)},mn.isDST=function(){return this.utcOffset()>this.clone().month(0).utcOffset()||this.utcOffset()>this.clone().month(5).utcOffset()},mn.isLocal=function(){return!!this.isValid()&&!this._isUTC},mn.isUtcOffset=function(){return!!this.isValid()&&this._isUTC},mn.isUtc=Ft,mn.isUTC=Ft,mn.zoneAbbr=function(){return this._isUTC?"UTC":""},mn.zoneName=function(){return this._isUTC?"Coordinated Universal Time":""},mn.dates=x("dates accessor is deprecated. Use date instead.",dn),mn.months=x("months accessor is deprecated. Use month instead",Oe),mn.years=x("years accessor is deprecated. Use year instead",De),mn.zone=x("moment().zone is deprecated, use moment().utcOffset instead. http://momentjs.com/guides/#/warnings/zone/",(function(e,t){return null!=e?("string"!=typeof e&&(e=-e),this.utcOffset(e,t),this):-this.utcOffset()})),mn.isDSTShifted=x("isDSTShifted is deprecated. See http://momentjs.com/guides/#/warnings/dst-shifted/ for more information",(function(){if(!u(this._isDSTShifted))return this._isDSTShifted;var e,t={};return C(t,this),(t=xt(t))._a?(e=t._isUTC?p(t._a):Mt(t._a),this._isDSTShifted=this.isValid()&&function(e,t,n){var r,i=Math.min(e.length,t.length),o=Math.abs(e.length-t.length),a=0;for(r=0;r<i;r++)(n&&e[r]!==t[r]||!n&&X(e[r])!==X(t[r]))&&a++;return a+o}(t._a,e.toArray())>0):this._isDSTShifted=!1,this._isDSTShifted}));var bn=O.prototype;function Cn(e,t,n,r){var i=lt(),o=p().set(r,t);return i[n](o,e)}function An(e,t,n){if(c(e)&&(t=e,e=void 0),e=e||"",null!=t)return Cn(e,t,n,"month");var r,i=[];for(r=0;r<12;r++)i[r]=Cn(e,r,n,"month");return i}function _n(e,t,n,r){"boolean"==typeof e?(c(t)&&(n=t,t=void 0),t=t||""):(n=t=e,e=!1,c(t)&&(n=t,t=void 0),t=t||"");var i,o=lt(),a=e?o._week.dow:0,s=[];if(null!=n)return Cn(t,(n+a)%7,r,"day");for(i=0;i<7;i++)s[i]=Cn(t,(i+a)%7,r,"day");return s}bn.calendar=function(e,t,n){var r=this._calendar[e]||this._calendar.sameElse;return T(r)?r.call(t,n):r},bn.longDateFormat=function(e){var t=this._longDateFormat[e],n=this._longDateFormat[e.toUpperCase()];return t||!n?t:(this._longDateFormat[e]=n.match(L).map((function(e){return"MMMM"===e||"MM"===e||"DD"===e||"dddd"===e?e.slice(1):e})).join(""),this._longDateFormat[e])},bn.invalidDate=function(){return this._invalidDate},bn.ordinal=function(e){return this._ordinal.replace("%d",e)},bn.preparse=yn,bn.postformat=yn,bn.relativeTime=function(e,t,n,r){var i=this._relativeTime[n];return T(i)?i(e,t,n,r):i.replace(/%d/i,e)},bn.pastFuture=function(e,t){var n=this._relativeTime[e>0?"future":"past"];return T(n)?n(t):n.replace(/%s/i,t)},bn.set=function(e){var t,n;for(n in e)s(e,n)&&(T(t=e[n])?this[n]=t:this["_"+n]=t);this._config=e,this._dayOfMonthOrdinalParseLenient=new RegExp((this._dayOfMonthOrdinalParse.source||this._ordinalParse.source)+"|"+/\d{1,2}/.source)},bn.eras=function(e,t){var n,r,o,a=this._eras||lt("en")._eras;for(n=0,r=a.length;n<r;++n){switch(typeof a[n].since){case"string":o=i(a[n].since).startOf("day"),a[n].since=o.valueOf()}switch(typeof a[n].until){case"undefined":a[n].until=1/0;break;case"string":o=i(a[n].until).startOf("day").valueOf(),a[n].until=o.valueOf()}}return a},bn.erasParse=function(e,t,n){var r,i,o,a,s,l=this.eras();for(e=e.toUpperCase(),r=0,i=l.length;r<i;++r)if(o=l[r].name.toUpperCase(),a=l[r].abbr.toUpperCase(),s=l[r].narrow.toUpperCase(),n)switch(t){case"N":case"NN":case"NNN":if(a===e)return l[r];break;case"NNNN":if(o===e)return l[r];break;case"NNNNN":if(s===e)return l[r]}else if([o,a,s].indexOf(e)>=0)return l[r]},bn.erasConvertYear=function(e,t){var n=e.since<=e.until?1:-1;return void 0===t?i(e.since).year():i(e.since).year()+(t-e.offset)*n},bn.erasAbbrRegex=function(e){return s(this,"_erasAbbrRegex")||sn.call(this),e?this._erasAbbrRegex:this._erasRegex},bn.erasNameRegex=function(e){return s(this,"_erasNameRegex")||sn.call(this),e?this._erasNameRegex:this._erasRegex},bn.erasNarrowRegex=function(e){return s(this,"_erasNarrowRegex")||sn.call(this),e?this._erasNarrowRegex:this._erasRegex},bn.months=function(e,t){return e?o(this._months)?this._months[e.month()]:this._months[(this._months.isFormat||we).test(t)?"format":"standalone"][e.month()]:o(this._months)?this._months:this._months.standalone},bn.monthsShort=function(e,t){return e?o(this._monthsShort)?this._monthsShort[e.month()]:this._monthsShort[we.test(t)?"format":"standalone"][e.month()]:o(this._monthsShort)?this._monthsShort:this._monthsShort.standalone},bn.monthsParse=function(e,t,n){var r,i,o;if(this._monthsParseExact)return Te.call(this,e,t,n);for(this._monthsParse||(this._monthsParse=[],this._longMonthsParse=[],this._shortMonthsParse=[]),r=0;r<12;r++){if(i=p([2e3,r]),n&&!this._longMonthsParse[r]&&(this._longMonthsParse[r]=new RegExp("^"+this.months(i,"").replace(".","")+"$","i"),this._shortMonthsParse[r]=new RegExp("^"+this.monthsShort(i,"").replace(".","")+"$","i")),n||this._monthsParse[r]||(o="^"+this.months(i,"")+"|^"+this.monthsShort(i,""),this._monthsParse[r]=new RegExp(o.replace(".",""),"i")),n&&"MMMM"===t&&this._longMonthsParse[r].test(e))return r;if(n&&"MMM"===t&&this._shortMonthsParse[r].test(e))return r;if(!n&&this._monthsParse[r].test(e))return r}},bn.monthsRegex=function(e){return this._monthsParseExact?(s(this,"_monthsRegex")||ke.call(this),e?this._monthsStrictRegex:this._monthsRegex):(s(this,"_monthsRegex")||(this._monthsRegex=Se),this._monthsStrictRegex&&e?this._monthsStrictRegex:this._monthsRegex)},bn.monthsShortRegex=function(e){return this._monthsParseExact?(s(this,"_monthsRegex")||ke.call(this),e?this._monthsShortStrictRegex:this._monthsShortRegex):(s(this,"_monthsShortRegex")||(this._monthsShortRegex=Me),this._monthsShortStrictRegex&&e?this._monthsShortStrictRegex:this._monthsShortRegex)},bn.week=function(e){return ze(e,this._week.dow,this._week.doy).week},bn.firstDayOfYear=function(){return this._week.doy},bn.firstDayOfWeek=function(){return this._week.dow},bn.weekdays=function(e,t){var n=o(this._weekdays)?this._weekdays:this._weekdays[e&&!0!==e&&this._weekdays.isFormat.test(t)?"format":"standalone"];return!0===e?Be(n,this._week.dow):e?n[e.day()]:n},bn.weekdaysMin=function(e){return!0===e?Be(this._weekdaysMin,this._week.dow):e?this._weekdaysMin[e.day()]:this._weekdaysMin},bn.weekdaysShort=function(e){return!0===e?Be(this._weekdaysShort,this._week.dow):e?this._weekdaysShort[e.day()]:this._weekdaysShort},bn.weekdaysParse=function(e,t,n){var r,i,o;if(this._weekdaysParseExact)return Xe.call(this,e,t,n);for(this._weekdaysParse||(this._weekdaysParse=[],this._minWeekdaysParse=[],this._shortWeekdaysParse=[],this._fullWeekdaysParse=[]),r=0;r<7;r++){if(i=p([2e3,1]).day(r),n&&!this._fullWeekdaysParse[r]&&(this._fullWeekdaysParse[r]=new RegExp("^"+this.weekdays(i,"").replace(".","\\.?")+"$","i"),this._shortWeekdaysParse[r]=new RegExp("^"+this.weekdaysShort(i,"").replace(".","\\.?")+"$","i"),this._minWeekdaysParse[r]=new RegExp("^"+this.weekdaysMin(i,"").replace(".","\\.?")+"$","i")),this._weekdaysParse[r]||(o="^"+this.weekdays(i,"")+"|^"+this.weekdaysShort(i,"")+"|^"+this.weekdaysMin(i,""),this._weekdaysParse[r]=new RegExp(o.replace(".",""),"i")),n&&"dddd"===t&&this._fullWeekdaysParse[r].test(e))return r;if(n&&"ddd"===t&&this._shortWeekdaysParse[r].test(e))return r;if(n&&"dd"===t&&this._minWeekdaysParse[r].test(e))return r;if(!n&&this._weekdaysParse[r].test(e))return r}},bn.weekdaysRegex=function(e){return this._weekdaysParseExact?(s(this,"_weekdaysRegex")||qe.call(this),e?this._weekdaysStrictRegex:this._weekdaysRegex):(s(this,"_weekdaysRegex")||(this._weekdaysRegex=Ye),this._weekdaysStrictRegex&&e?this._weekdaysStrictRegex:this._weekdaysRegex)},bn.weekdaysShortRegex=function(e){return this._weekdaysParseExact?(s(this,"_weekdaysRegex")||qe.call(this),e?this._weekdaysShortStrictRegex:this._weekdaysShortRegex):(s(this,"_weekdaysShortRegex")||(this._weekdaysShortRegex=We),this._weekdaysShortStrictRegex&&e?this._weekdaysShortStrictRegex:this._weekdaysShortRegex)},bn.weekdaysMinRegex=function(e){return this._weekdaysParseExact?(s(this,"_weekdaysRegex")||qe.call(this),e?this._weekdaysMinStrictRegex:this._weekdaysMinRegex):(s(this,"_weekdaysMinRegex")||(this._weekdaysMinRegex=Ge),this._weekdaysMinStrictRegex&&e?this._weekdaysMinStrictRegex:this._weekdaysMinRegex)},bn.isPM=function(e){return"p"===(e+"").toLowerCase().charAt(0)},bn.meridiem=function(e,t,n){return e>11?n?"pm":"PM":n?"am":"AM"},at("en",{eras:[{since:"0001-01-01",until:1/0,offset:1,name:"Anno Domini",narrow:"AD",abbr:"AD"},{since:"0000-12-31",until:-1/0,offset:1,name:"Before Christ",narrow:"BC",abbr:"BC"}],dayOfMonthOrdinalParse:/\d{1,2}(th|st|nd|rd)/,ordinal:function(e){var t=e%10;return e+(1===X(e%100/10)?"th":1===t?"st":2===t?"nd":3===t?"rd":"th")}}),i.lang=x("moment.lang is deprecated. Use moment.locale instead.",at),i.langData=x("moment.langData is deprecated. Use moment.localeData instead.",lt);var In=Math.abs;function xn(e,t,n,r){var i=Ut(t,n);return e._milliseconds+=r*i._milliseconds,e._days+=r*i._days,e._months+=r*i._months,e._bubble()}function wn(e){return e<0?Math.floor(e):Math.ceil(e)}function Mn(e){return 4800*e/146097}function Sn(e){return 146097*e/4800}function Tn(e){return function(){return this.as(e)}}var En=Tn("ms"),On=Tn("s"),kn=Tn("m"),Ln=Tn("h"),Dn=Tn("d"),Pn=Tn("w"),Nn=Tn("M"),jn=Tn("Q"),Rn=Tn("y");function zn(e){return function(){return this.isValid()?this._data[e]:NaN}}var Fn=zn("milliseconds"),Bn=zn("seconds"),Hn=zn("minutes"),Un=zn("hours"),Vn=zn("days"),Yn=zn("months"),Wn=zn("years"),Gn=Math.round,Xn={ss:44,s:45,m:45,h:22,d:26,w:null,M:11};function qn(e,t,n,r,i){return i.relativeTime(t||1,!!n,e,r)}var Zn=Math.abs;function Qn(e){return(e>0)-(e<0)||+e}function Kn(){if(!this.isValid())return this.localeData().invalidDate();var e,t,n,r,i,o,a,s,l=Zn(this._milliseconds)/1e3,u=Zn(this._days),c=Zn(this._months),d=this.asSeconds();return d?(e=G(l/60),t=G(e/60),l%=60,e%=60,n=G(c/12),c%=12,r=l?l.toFixed(3).replace(/\.?0+$/,""):"",i=d<0?"-":"",o=Qn(this._months)!==Qn(d)?"-":"",a=Qn(this._days)!==Qn(d)?"-":"",s=Qn(this._milliseconds)!==Qn(d)?"-":"",i+"P"+(n?o+n+"Y":"")+(c?o+c+"M":"")+(u?a+u+"D":"")+(t||e||l?"T":"")+(t?s+t+"H":"")+(e?s+e+"M":"")+(l?s+r+"S":"")):"P0D"}var Jn=kt.prototype;return Jn.isValid=function(){return this._isValid},Jn.abs=function(){var e=this._data;return this._milliseconds=In(this._milliseconds),this._days=In(this._days),this._months=In(this._months),e.milliseconds=In(e.milliseconds),e.seconds=In(e.seconds),e.minutes=In(e.minutes),e.hours=In(e.hours),e.months=In(e.months),e.years=In(e.years),this},Jn.add=function(e,t){return xn(this,e,t,1)},Jn.subtract=function(e,t){return xn(this,e,t,-1)},Jn.as=function(e){if(!this.isValid())return NaN;var t,n,r=this._milliseconds;if("month"===(e=H(e))||"quarter"===e||"year"===e)switch(t=this._days+r/864e5,n=this._months+Mn(t),e){case"month":return n;case"quarter":return n/3;case"year":return n/12}else switch(t=this._days+Math.round(Sn(this._months)),e){case"week":return t/7+r/6048e5;case"day":return t+r/864e5;case"hour":return 24*t+r/36e5;case"minute":return 1440*t+r/6e4;case"second":return 86400*t+r/1e3;case"millisecond":return Math.floor(864e5*t)+r;default:throw new Error("Unknown unit "+e)}},Jn.asMilliseconds=En,Jn.asSeconds=On,Jn.asMinutes=kn,Jn.asHours=Ln,Jn.asDays=Dn,Jn.asWeeks=Pn,Jn.asMonths=Nn,Jn.asQuarters=jn,Jn.asYears=Rn,Jn.valueOf=function(){return this.isValid()?this._milliseconds+864e5*this._days+this._months%12*2592e6+31536e6*X(this._months/12):NaN},Jn._bubble=function(){var e,t,n,r,i,o=this._milliseconds,a=this._days,s=this._months,l=this._data;return o>=0&&a>=0&&s>=0||o<=0&&a<=0&&s<=0||(o+=864e5*wn(Sn(s)+a),a=0,s=0),l.milliseconds=o%1e3,e=G(o/1e3),l.seconds=e%60,t=G(e/60),l.minutes=t%60,n=G(t/60),l.hours=n%24,a+=G(n/24),i=G(Mn(a)),s+=i,a-=wn(Sn(i)),r=G(s/12),s%=12,l.days=a,l.months=s,l.years=r,this},Jn.clone=function(){return Ut(this)},Jn.get=function(e){return e=H(e),this.isValid()?this[e+"s"]():NaN},Jn.milliseconds=Fn,Jn.seconds=Bn,Jn.minutes=Hn,Jn.hours=Un,Jn.days=Vn,Jn.weeks=function(){return G(this.days()/7)},Jn.months=Yn,Jn.years=Wn,Jn.humanize=function(e,t){if(!this.isValid())return this.localeData().invalidDate();var n,r,i=!1,o=Xn;return"object"==typeof e&&(t=e,e=!1),"boolean"==typeof e&&(i=e),"object"==typeof t&&(o=Object.assign({},Xn,t),null!=t.s&&null==t.ss&&(o.ss=t.s-1)),n=this.localeData(),r=function(e,t,n,r){var i=Ut(e).abs(),o=Gn(i.as("s")),a=Gn(i.as("m")),s=Gn(i.as("h")),l=Gn(i.as("d")),u=Gn(i.as("M")),c=Gn(i.as("w")),d=Gn(i.as("y")),h=o<=n.ss&&["s",o]||o<n.s&&["ss",o]||a<=1&&["m"]||a<n.m&&["mm",a]||s<=1&&["h"]||s<n.h&&["hh",s]||l<=1&&["d"]||l<n.d&&["dd",l];return null!=n.w&&(h=h||c<=1&&["w"]||c<n.w&&["ww",c]),(h=h||u<=1&&["M"]||u<n.M&&["MM",u]||d<=1&&["y"]||["yy",d])[2]=t,h[3]=+e>0,h[4]=r,qn.apply(null,h)}(this,!i,o,n),i&&(r=n.pastFuture(+this,r)),n.postformat(r)},Jn.toISOString=Kn,Jn.toString=Kn,Jn.toJSON=Kn,Jn.locale=$t,Jn.localeData=tn,Jn.toIsoString=x("toIsoString() is deprecated. Please use toISOString() instead (notice the capitals)",Kn),Jn.lang=en,j("X",0,0,"unix"),j("x",0,0,"valueOf"),pe("x",ce),pe("X",/[+-]?\d+(\.\d{1,3})?/),be("X",(function(e,t,n){n._d=new Date(1e3*parseFloat(e))})),be("x",(function(e,t,n){n._d=new Date(X(e))})),
//! moment.js
i.version="2.29.1",t=Mt,i.fn=mn,i.min=function(){var e=[].slice.call(arguments,0);return Et("isBefore",e)},i.max=function(){var e=[].slice.call(arguments,0);return Et("isAfter",e)},i.now=function(){return Date.now?Date.now():+new Date},i.utc=p,i.unix=function(e){return Mt(1e3*e)},i.months=function(e,t){return An(e,t,"months")},i.isDate=d,i.locale=at,i.invalid=m,i.duration=Ut,i.isMoment=_,i.weekdays=function(e,t,n){return _n(e,t,n,"weekdays")},i.parseZone=function(){return Mt.apply(null,arguments).parseZone()},i.localeData=lt,i.isDuration=Lt,i.monthsShort=function(e,t){return An(e,t,"monthsShort")},i.weekdaysMin=function(e,t,n){return _n(e,t,n,"weekdaysMin")},i.defineLocale=st,i.updateLocale=function(e,t){if(null!=t){var n,r,i=et;null!=tt[e]&&null!=tt[e].parentLocale?tt[e].set(E(tt[e]._config,t)):(null!=(r=ot(e))&&(i=r._config),t=E(i,t),null==r&&(t.abbr=e),(n=new O(t)).parentLocale=tt[e],tt[e]=n),at(e)}else null!=tt[e]&&(null!=tt[e].parentLocale?(tt[e]=tt[e].parentLocale,e===at()&&at(e)):null!=tt[e]&&delete tt[e]);return tt[e]},i.locales=function(){return w(tt)},i.weekdaysShort=function(e,t,n){return _n(e,t,n,"weekdaysShort")},i.normalizeUnits=H,i.relativeTimeRounding=function(e){return void 0===e?Gn:"function"==typeof e&&(Gn=e,!0)},i.relativeTimeThreshold=function(e,t){return void 0!==Xn[e]&&(void 0===t?Xn[e]:(Xn[e]=t,"s"===e&&(Xn.ss=t-1),!0))},i.calendarFormat=function(e,t){var n=e.diff(t,"days",!0);return n<-6?"sameElse":n<-1?"lastWeek":n<0?"lastDay":n<1?"sameDay":n<2?"nextDay":n<7?"nextWeek":"sameElse"},i.prototype=mn,i.HTML5_FMT={DATETIME_LOCAL:"YYYY-MM-DDTHH:mm",DATETIME_LOCAL_SECONDS:"YYYY-MM-DDTHH:mm:ss",DATETIME_LOCAL_MS:"YYYY-MM-DDTHH:mm:ss.SSS",DATE:"YYYY-MM-DD",TIME:"HH:mm",TIME_SECONDS:"HH:mm:ss",TIME_MS:"HH:mm:ss.SSS",WEEK:"GGGG-[W]WW",MONTH:"YYYY-MM"},i}()}).call(this,n(188)(e))},function(e,t,n){"use strict";function r(){var e=this.constructor.getDerivedStateFromProps(this.props,this.state);null!=e&&this.setState(e)}function i(e){this.setState(function(t){var n=this.constructor.getDerivedStateFromProps(e,t);return null!=n?n:null}.bind(this))}function o(e,t){try{var n=this.props,r=this.state;this.props=e,this.state=t,this.__reactInternalSnapshotFlag=!0,this.__reactInternalSnapshot=this.getSnapshotBeforeUpdate(n,r)}finally{this.props=n,this.state=r}}function a(e){var t=e.prototype;if(!t||!t.isReactComponent)throw new Error("Can only polyfill class components");if("function"!=typeof e.getDerivedStateFromProps&&"function"!=typeof t.getSnapshotBeforeUpdate)return e;var n=null,a=null,s=null;if("function"==typeof t.componentWillMount?n="componentWillMount":"function"==typeof t.UNSAFE_componentWillMount&&(n="UNSAFE_componentWillMount"),"function"==typeof t.componentWillReceiveProps?a="componentWillReceiveProps":"function"==typeof t.UNSAFE_componentWillReceiveProps&&(a="UNSAFE_componentWillReceiveProps"),"function"==typeof t.componentWillUpdate?s="componentWillUpdate":"function"==typeof t.UNSAFE_componentWillUpdate&&(s="UNSAFE_componentWillUpdate"),null!==n||null!==a||null!==s){var l=e.displayName||e.name,u="function"==typeof e.getDerivedStateFromProps?"getDerivedStateFromProps()":"getSnapshotBeforeUpdate()";throw Error("Unsafe legacy lifecycles will not be called for components using new component APIs.\n\n"+l+" uses "+u+" but also contains the following legacy lifecycles:"+(null!==n?"\n  "+n:"")+(null!==a?"\n  "+a:"")+(null!==s?"\n  "+s:"")+"\n\nThe above lifecycles should be removed. Learn more about this warning here:\nhttps://fb.me/react-async-component-lifecycle-hooks")}if("function"==typeof e.getDerivedStateFromProps&&(t.componentWillMount=r,t.componentWillReceiveProps=i),"function"==typeof t.getSnapshotBeforeUpdate){if("function"!=typeof t.componentDidUpdate)throw new Error("Cannot polyfill getSnapshotBeforeUpdate() for components that do not define componentDidUpdate() on the prototype");t.componentWillUpdate=o;var c=t.componentDidUpdate;t.componentDidUpdate=function(e,t,n){var r=this.__reactInternalSnapshotFlag?this.__reactInternalSnapshot:n;c.call(this,e,t,r)}}return e}n.r(t),n.d(t,"polyfill",(function(){return a})),r.__suppressDeprecationWarning=!0,i.__suppressDeprecationWarning=!0,o.__suppressDeprecationWarning=!0},function(e,t,n){"use strict";(function(e,r){n.d(t,"a",(function(){return u})),n.d(t,"b",(function(){return l})),n.d(t,"c",(function(){return s}));var i=n(6),o=n(1201),a=function(e,t){return(a=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(e,t)};var s,l=function(){function e(e){this.type=e}return Object.defineProperty(e.prototype,"isConflatable",{get:function(){return!1},enumerable:!0,configurable:!0}),e.prototype.conflate=function(e){return!1},e}(),u=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return function(e,t){function n(){this.constructor=e}a(e,t),e.prototype=null===t?Object.create(t):(n.prototype=t.prototype,new n)}(t,e),Object.defineProperty(t.prototype,"isConflatable",{get:function(){return!0},enumerable:!0,configurable:!0}),t.prototype.conflate=function(e){return!0},t}(l);!function(t){function n(e,t){var n=s.get(e);n&&0!==n.length?Object(i.every)(Object(i.retro)(n),(function(n){return!n||function(e,t,n){var r=!0;try{r="function"==typeof e?e(t,n):e.messageHook(t,n)}catch(e){u(e)}return r}(n,e,t)}))&&p(e,t):p(e,t)}t.sendMessage=n,t.postMessage=function(e,t){t.isConflatable&&Object(i.some)(a,(function(n){return n.handler===e&&(!!n.msg&&(n.msg.type===t.type&&(!!n.msg.isConflatable&&n.msg.conflate(t))))}))||g(e,t)},t.installMessageHook=function(e,t){var n=s.get(e);n&&-1!==n.indexOf(t)||(n?n.push(t):s.set(e,[t]))},t.removeMessageHook=function(e,t){var n=s.get(e);if(n){var r=n.indexOf(t);-1!==r&&(n[r]=null,m(n))}},t.clearData=function(e){var t=s.get(e);t&&t.length>0&&(i.ArrayExt.fill(t,null),m(t)),Object(i.each)(a,(function(t){t.handler===e&&(t.handler=null,t.msg=null)}))},t.flush=function(){d||0===c||(f(c),d=!0,v(),d=!1)},t.getExceptionHandler=function(){return u},t.setExceptionHandler=function(e){var t=u;return u=e,t};var a=new o.a,s=new WeakMap,l=new Set,u=function(e){console.error(e)},c=0,d=!1,h="function"==typeof requestAnimationFrame?requestAnimationFrame:e,f="function"==typeof cancelAnimationFrame?cancelAnimationFrame:r;function p(e,t){try{e.processMessage(t)}catch(e){u(e)}}function g(e,t){a.addLast({handler:e,msg:t}),0===c&&(c=h(v))}function v(){if(c=0,!a.isEmpty){var e={handler:null,msg:null};for(a.addLast(e);;){var t=a.removeFirst();if(t===e)return;t.handler&&t.msg&&n(t.handler,t.msg)}}}function m(e){0===l.size&&h(y),l.add(e)}function y(){l.forEach(b),l.clear()}function b(e){i.ArrayExt.removeAllWhere(e,C)}function C(e){return null===e}}(s||(s={}))}).call(this,n(234).setImmediate,n(234).clearImmediate)},function(e,t,n){"use strict";n.d(t,"a",(function(){return i}));var r=n(334);function i(e){if("string"!=typeof e)throw new Error(Object(r.a)(7));return e.charAt(0).toUpperCase()+e.slice(1)}},function(e,t,n){"use strict";n.r(t),function(e){n.d(t,"Signal",(function(){return o}));var r,i=n(6),o=function(){function e(e){this.sender=e}return e.prototype.connect=function(e,t){return r.connect(this,e,t)},e.prototype.disconnect=function(e,t){return r.disconnect(this,e,t)},e.prototype.emit=function(e){r.emit(this,e)},e}();!function(e){e.disconnectBetween=function(e,t){r.disconnectBetween(e,t)},e.disconnectSender=function(e){r.disconnectSender(e)},e.disconnectReceiver=function(e){r.disconnectReceiver(e)},e.disconnectAll=function(e){r.disconnectAll(e)},e.clearData=function(e){r.disconnectAll(e)},e.getExceptionHandler=function(){return r.exceptionHandler},e.setExceptionHandler=function(e){var t=r.exceptionHandler;return r.exceptionHandler=e,t}}(o||(o={})),function(t){function n(e){var t=o.get(e);t&&0!==t.length&&(Object(i.each)(t,(function(e){if(e.signal){var t=e.thisArg||e.slot;e.signal=null,d(a.get(t))}})),d(t))}function r(e){var t=a.get(e);t&&0!==t.length&&(Object(i.each)(t,(function(e){if(e.signal){var t=e.signal.sender;e.signal=null,d(o.get(t))}})),d(t))}t.exceptionHandler=function(e){console.error(e)},t.connect=function(e,t,n){n=n||void 0;var r=o.get(e.sender);if(r||(r=[],o.set(e.sender,r)),u(r,e,t,n))return!1;var i=n||t,s=a.get(i);s||(s=[],a.set(i,s));var l={signal:e,slot:t,thisArg:n};return r.push(l),s.push(l),!0},t.disconnect=function(e,t,n){n=n||void 0;var r=o.get(e.sender);if(!r||0===r.length)return!1;var i=u(r,e,t,n);if(!i)return!1;var s=n||t,l=a.get(s);return i.signal=null,d(r),d(l),!0},t.disconnectBetween=function(e,t){var n=o.get(e);if(n&&0!==n.length){var r=a.get(t);r&&0!==r.length&&(Object(i.each)(r,(function(t){t.signal&&t.signal.sender===e&&(t.signal=null)})),d(n),d(r))}},t.disconnectSender=n,t.disconnectReceiver=r,t.disconnectAll=function(e){n(e),r(e)},t.emit=function(e,t){var n=o.get(e.sender);if(n&&0!==n.length)for(var r=0,i=n.length;r<i;++r){var a=n[r];a.signal===e&&c(a,t)}};var o=new WeakMap,a=new WeakMap,s=new Set,l="function"==typeof requestAnimationFrame?requestAnimationFrame:e;function u(e,t,n,r){return Object(i.find)(e,(function(e){return e.signal===t&&e.slot===n&&e.thisArg===r}))}function c(e,n){var r=e.signal,i=e.slot,o=e.thisArg;try{i.call(o,r.sender,n)}catch(e){t.exceptionHandler(e)}}function d(e){0===s.size&&l(h),s.add(e)}function h(){s.forEach(f),s.clear()}function f(e){i.ArrayExt.removeAllWhere(e,p)}function p(e){return null===e.signal}}(r||(r={}))}.call(this,n(234).setImmediate)},function(e,t,n){"use strict";var r;n.r(t),n.d(t,"JSONExt",(function(){return r})),n.d(t,"MimeData",(function(){return o})),n.d(t,"PromiseDelegate",(function(){return a})),n.d(t,"Random",(function(){return i})),n.d(t,"Token",(function(){return l})),n.d(t,"UUID",(function(){return s})),function(e){function t(e){return null===e||"boolean"==typeof e||"number"==typeof e||"string"==typeof e}function n(e){return Array.isArray(e)}function r(e,i){if(e===i)return!0;if(t(e)||t(i))return!1;var o=n(e),a=n(i);return o===a&&(o&&a?function(e,t){if(e===t)return!0;if(e.length!==t.length)return!1;for(var n=0,i=e.length;n<i;++n)if(!r(e[n],t[n]))return!1;return!0}(e,i):function(e,t){if(e===t)return!0;for(var n in e)if(void 0!==e[n]&&!(n in t))return!1;for(var n in t)if(void 0!==t[n]&&!(n in e))return!1;for(var n in e){var i=e[n],o=t[n];if(void 0!==i||void 0!==o){if(void 0===i||void 0===o)return!1;if(!r(i,o))return!1}}return!0}(e,i))}function i(e){return t(e)?e:n(e)?function(e){for(var t=new Array(e.length),n=0,r=e.length;n<r;++n)t[n]=i(e[n]);return t}(e):function(e){var t={};for(var n in e){var r=e[n];void 0!==r&&(t[n]=i(r))}return t}(e)}e.emptyObject=Object.freeze({}),e.emptyArray=Object.freeze([]),e.isPrimitive=t,e.isArray=n,e.isObject=function(e){return!t(e)&&!n(e)},e.deepEqual=r,e.deepCopy=i}(r||(r={}));var i,o=function(){function e(){this._types=[],this._values=[]}return e.prototype.types=function(){return this._types.slice()},e.prototype.hasData=function(e){return-1!==this._types.indexOf(e)},e.prototype.getData=function(e){var t=this._types.indexOf(e);return-1!==t?this._values[t]:void 0},e.prototype.setData=function(e,t){this.clearData(e),this._types.push(e),this._values.push(t)},e.prototype.clearData=function(e){var t=this._types.indexOf(e);-1!==t&&(this._types.splice(t,1),this._values.splice(t,1))},e.prototype.clear=function(){this._types.length=0,this._values.length=0},e}(),a=function(){function e(){var e=this;this.promise=new Promise((function(t,n){e._resolve=t,e._reject=n}))}return e.prototype.resolve=function(e){(0,this._resolve)(e)},e.prototype.reject=function(e){(0,this._reject)(e)},e}();!function(e){e.getRandomValues=function(){var e="undefined"!=typeof window&&(window.crypto||window.msCrypto)||n(1346)||null;return e&&"function"==typeof e.getRandomValues?function(t){return e.getRandomValues(t)}:e&&"function"==typeof e.randomFillSync?function(t){return e.randomFillSync(t)}:e&&"function"==typeof e.randomBytes?function(t){for(var n=e.randomBytes(t.length),r=0,i=n.length;r<i;++r)t[r]=n[r]}:function(e){for(var t=0,n=0,r=e.length;n<r;++n)n%4==0&&(t=4294967295*Math.random()>>>0),e[n]=255&t,t>>>=8}}()}(i||(i={}));var s,l=function(e){this.name=e,this._tokenStructuralPropertyT=null};!function(e){e.uuid4=function(){for(var e=new Uint8Array(16),t=new Array(256),n=0;n<16;++n)t[n]="0"+n.toString(16);for(n=16;n<256;++n)t[n]=n.toString(16);return function(){return i.getRandomValues(e),e[6]=64|15&e[6],e[8]=128|63&e[8],t[e[0]]+t[e[1]]+t[e[2]]+t[e[3]]+"-"+t[e[4]]+t[e[5]]+"-"+t[e[6]]+t[e[7]]+"-"+t[e[8]]+t[e[9]]+"-"+t[e[10]]+t[e[11]]+t[e[12]]+t[e[13]]+t[e[14]]+t[e[15]]}}()}(s||(s={}))},function(e,t,n){"use strict";n.r(t);var r=n(6),i=n(38),o=n(14),a=n(102),s=n(16),l=n(17),u=n(86);function c(){return h.keyboardLayout}var d=function(){function e(t,n){this.name=t,this._codes=n,this._keys=e.extractKeys(n)}return e.prototype.keys=function(){return Object.keys(this._keys)},e.prototype.isValidKey=function(e){return e in this._keys},e.prototype.keyForKeydownEvent=function(e){return this._codes[e.keyCode]||""},e}();!function(e){e.extractKeys=function(e){var t=Object.create(null);for(var n in e)t[e[n]]=!0;return t}}(d||(d={}));var h,f=new d("en-us",{8:"Backspace",9:"Tab",13:"Enter",19:"Pause",27:"Escape",32:"Space",33:"PageUp",34:"PageDown",35:"End",36:"Home",37:"ArrowLeft",38:"ArrowUp",39:"ArrowRight",40:"ArrowDown",45:"Insert",46:"Delete",48:"0",49:"1",50:"2",51:"3",52:"4",53:"5",54:"6",55:"7",56:"8",57:"9",59:";",61:"=",65:"A",66:"B",67:"C",68:"D",69:"E",70:"F",71:"G",72:"H",73:"I",74:"J",75:"K",76:"L",77:"M",78:"N",79:"O",80:"P",81:"Q",82:"R",83:"S",84:"T",85:"U",86:"V",87:"W",88:"X",89:"Y",90:"Z",93:"ContextMenu",96:"0",97:"1",98:"2",99:"3",100:"4",101:"5",102:"6",103:"7",104:"8",105:"9",106:"*",107:"+",109:"-",110:".",111:"/",112:"F1",113:"F2",114:"F3",115:"F4",116:"F5",117:"F6",118:"F7",119:"F8",120:"F9",121:"F10",122:"F11",123:"F12",173:"-",186:";",187:"=",188:",",189:"-",190:".",191:"/",192:"`",219:"[",220:"\\",221:"]",222:"'"});(h||(h={})).keyboardLayout=f;var p,g=function(){function e(){this._timerID=0,this._replaying=!1,this._keystrokes=[],this._keydownEvents=[],this._keyBindings=[],this._exactKeyMatch=null,this._commands=Object.create(null),this._commandChanged=new s.Signal(this),this._commandExecuted=new s.Signal(this),this._keyBindingChanged=new s.Signal(this)}return Object.defineProperty(e.prototype,"commandChanged",{get:function(){return this._commandChanged},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"commandExecuted",{get:function(){return this._commandExecuted},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"keyBindingChanged",{get:function(){return this._keyBindingChanged},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"keyBindings",{get:function(){return this._keyBindings},enumerable:!0,configurable:!0}),e.prototype.listCommands=function(){return Object.keys(this._commands)},e.prototype.hasCommand=function(e){return e in this._commands},e.prototype.addCommand=function(e,t){var n=this;if(e in this._commands)throw new Error("Command '"+e+"' already registered.");return this._commands[e]=p.createCommand(t),this._commandChanged.emit({id:e,type:"added"}),new u.DisposableDelegate((function(){delete n._commands[e],n._commandChanged.emit({id:e,type:"removed"})}))},e.prototype.notifyCommandChanged=function(e){if(void 0!==e&&!(e in this._commands))throw new Error("Command '"+e+"' is not registered.");this._commandChanged.emit({id:e,type:e?"changed":"many-changed"})},e.prototype.label=function(e,t){void 0===t&&(t=l.JSONExt.emptyObject);var n=this._commands[e];return n?n.label.call(void 0,t):""},e.prototype.mnemonic=function(e,t){void 0===t&&(t=l.JSONExt.emptyObject);var n=this._commands[e];return n?n.mnemonic.call(void 0,t):-1},e.prototype.icon=function(e,t){void 0===t&&(t=l.JSONExt.emptyObject);var n=this._commands[e];return n?n.icon.call(void 0,t):""},e.prototype.iconClass=function(e,t){void 0===t&&(t=l.JSONExt.emptyObject);var n=this._commands[e];return n?n.iconClass.call(void 0,t):""},e.prototype.iconLabel=function(e,t){void 0===t&&(t=l.JSONExt.emptyObject);var n=this._commands[e];return n?n.iconLabel.call(void 0,t):""},e.prototype.caption=function(e,t){void 0===t&&(t=l.JSONExt.emptyObject);var n=this._commands[e];return n?n.caption.call(void 0,t):""},e.prototype.usage=function(e,t){void 0===t&&(t=l.JSONExt.emptyObject);var n=this._commands[e];return n?n.usage.call(void 0,t):""},e.prototype.className=function(e,t){void 0===t&&(t=l.JSONExt.emptyObject);var n=this._commands[e];return n?n.className.call(void 0,t):""},e.prototype.dataset=function(e,t){void 0===t&&(t=l.JSONExt.emptyObject);var n=this._commands[e];return n?n.dataset.call(void 0,t):{}},e.prototype.isEnabled=function(e,t){void 0===t&&(t=l.JSONExt.emptyObject);var n=this._commands[e];return!!n&&n.isEnabled.call(void 0,t)},e.prototype.isToggled=function(e,t){void 0===t&&(t=l.JSONExt.emptyObject);var n=this._commands[e];return!!n&&n.isToggled.call(void 0,t)},e.prototype.isToggleable=function(e,t){void 0===t&&(t=l.JSONExt.emptyObject);var n=this._commands[e];return!!n&&n.isToggleable},e.prototype.isVisible=function(e,t){void 0===t&&(t=l.JSONExt.emptyObject);var n=this._commands[e];return!!n&&n.isVisible.call(void 0,t)},e.prototype.execute=function(e,t){void 0===t&&(t=l.JSONExt.emptyObject);var n,r=this._commands[e];if(!r)return Promise.reject(new Error("Command '"+e+"' not registered."));try{n=r.execute.call(void 0,t)}catch(e){n=Promise.reject(e)}var i=Promise.resolve(n);return this._commandExecuted.emit({id:e,args:t,result:i}),i},e.prototype.addKeyBinding=function(e){var t=this,n=p.createKeyBinding(e);return this._keyBindings.push(n),this._keyBindingChanged.emit({binding:n,type:"added"}),new u.DisposableDelegate((function(){r.ArrayExt.removeFirstOf(t._keyBindings,n),t._keyBindingChanged.emit({binding:n,type:"removed"})}))},e.prototype.processKeydownEvent=function(t){if(!this._replaying){var n=e.keystrokeForKeydownEvent(t);if(!n)return this._replayKeydownEvents(),void this._clearPendingState();this._keystrokes.push(n);var r=p.matchKeyBinding(this._keyBindings,this._keystrokes,t),i=r.exact,o=r.partial;if(!i&&!o)return this._replayKeydownEvents(),void this._clearPendingState();if(t.preventDefault(),t.stopPropagation(),i&&!o)return this._executeKeyBinding(i),void this._clearPendingState();i&&(this._exactKeyMatch=i),this._keydownEvents.push(t),this._startTimer()}},e.prototype._startTimer=function(){var e=this;this._clearTimer(),this._timerID=window.setTimeout((function(){e._onPendingTimeout()}),p.CHORD_TIMEOUT)},e.prototype._clearTimer=function(){0!==this._timerID&&(clearTimeout(this._timerID),this._timerID=0)},e.prototype._replayKeydownEvents=function(){0!==this._keydownEvents.length&&(this._replaying=!0,this._keydownEvents.forEach(p.replayKeyEvent),this._replaying=!1)},e.prototype._executeKeyBinding=function(e){var t=e.command,n=e.args;if(this.hasCommand(t)&&this.isEnabled(t,n))this.execute(t,n);else{var r=this.hasCommand(t)?"enabled":"registered",i="Cannot execute key binding '"+e.keys.join(", ")+"':",o="command '"+t+"' is not "+r+".";console.warn(i+" "+o)}},e.prototype._clearPendingState=function(){this._clearTimer(),this._exactKeyMatch=null,this._keystrokes.length=0,this._keydownEvents.length=0},e.prototype._onPendingTimeout=function(){this._timerID=0,this._exactKeyMatch?this._executeKeyBinding(this._exactKeyMatch):this._replayKeydownEvents(),this._clearPendingState()},e}();!function(e){function t(e){for(var t="",n=!1,r=!1,o=!1,a=!1,s=0,l=e.split(/\s+/);s<l.length;s++){var u=l[s];"Accel"===u?i.b.IS_MAC?r=!0:o=!0:"Alt"===u?n=!0:"Cmd"===u?r=!0:"Ctrl"===u?o=!0:"Shift"===u?a=!0:u.length>0&&(t=u)}return{cmd:r,ctrl:o,alt:n,shift:a,key:t}}function n(e){var n="",r=t(e);return r.ctrl&&(n+="Ctrl "),r.alt&&(n+="Alt "),r.shift&&(n+="Shift "),r.cmd&&i.b.IS_MAC&&(n+="Cmd "),n+r.key}e.parseKeystroke=t,e.normalizeKeystroke=n,e.normalizeKeys=function(e){return(i.b.IS_WIN?e.winKeys||e.keys:i.b.IS_MAC?e.macKeys||e.keys:e.linuxKeys||e.keys).map(n)},e.formatKeystroke=function(e){var n="",r=t(e);return i.b.IS_MAC?(r.ctrl&&(n+="⌃ "),r.alt&&(n+="⌥ "),r.shift&&(n+="⇧ "),r.cmd&&(n+="⌘ ")):(r.ctrl&&(n+="Ctrl+"),r.alt&&(n+="Alt+"),r.shift&&(n+="Shift+")),n+r.key},e.keystrokeForKeydownEvent=function(e){var t=c().keyForKeydownEvent(e);if(!t)return"";var n="";return e.ctrlKey&&(n+="Ctrl "),e.altKey&&(n+="Alt "),e.shiftKey&&(n+="Shift "),e.metaKey&&i.b.IS_MAC&&(n+="Cmd "),n+t}}(g||(g={})),function(e){e.CHORD_TIMEOUT=1e3,e.createCommand=function(e){var i,l;return e.icon&&"string"!=typeof e.icon?(l=u(e.iconClass,t),i=u(e.icon,s)):i=l=u(e.iconClass||e.icon,t),{execute:e.execute,label:u(e.label,t),mnemonic:u(e.mnemonic,n),icon:i,iconClass:l,iconLabel:u(e.iconLabel,t),caption:u(e.caption,t),usage:u(e.usage,t),className:u(e.className,t),dataset:u(e.dataset,a),isEnabled:e.isEnabled||r,isToggled:e.isToggled||o,isToggleable:e.isToggleable||!!e.isToggled,isVisible:e.isVisible||r}},e.createKeyBinding=function(e){return{keys:g.normalizeKeys(e),selector:c(e),command:e.command,args:e.args||l.JSONExt.emptyObject}},e.matchKeyBinding=function(e,t,n){for(var r=null,o=!1,a=1/0,s=0,l=0,u=e.length;l<u;++l){var c=e[l],f=d(c.keys,t);if(0!==f)if(2!==f){var p=h(c.selector,n);if(!(-1===p||p>a)){var g=i.c.calculateSpecificity(c.selector);(!r||p<a||g>=s)&&(r=c,a=p,s=g)}}else o||-1===h(c.selector,n)||(o=!0)}return{exact:r,partial:o}},e.replayKeyEvent=function(e){e.target.dispatchEvent(function(e){var t=document.createEvent("Event"),n=e.bubbles||!0,r=e.cancelable||!0;return t.initEvent(e.type||"keydown",n,r),t.key=e.key||"",t.keyCode=e.keyCode||0,t.which=e.keyCode||0,t.ctrlKey=e.ctrlKey||!1,t.altKey=e.altKey||!1,t.shiftKey=e.shiftKey||!1,t.metaKey=e.metaKey||!1,t.view=e.view||window,t}(e))};var t=function(){return""},n=function(){return-1},r=function(){return!0},o=function(){return!1},a=function(){return{}},s=function(){};function u(e,t){return void 0===e?t:"function"==typeof e?e:function(){return e}}function c(e){if(-1!==e.selector.indexOf(","))throw new Error("Selector cannot contain commas: "+e.selector);if(!i.c.isValid(e.selector))throw new Error("Invalid selector: "+e.selector);return e.selector}function d(e,t){if(e.length<t.length)return 0;for(var n=0,r=t.length;n<r;++n)if(e[n]!==t[n])return 0;return e.length>t.length?2:1}function h(e,t){for(var n=t.target,r=t.currentTarget,o=0;null!==n;n=n.parentElement,++o){if(n.hasAttribute("data-lm-suppress-shortcuts"))return-1;if(n.hasAttribute("data-p-suppress-shortcuts"))return-1;if(i.c.matches(n,e))return o;if(n===r)return-1}return-1}}(p||(p={}));
/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
var v=function(e,t){return(v=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(e,t)};var m,y,b,C=function(e){this.type="text",this.content=e},A=function(e,t,n,r){this.type="element",this.tag=e,this.attrs=t,this.children=n,this.renderer=r};!function(e){function t(t,n,r){return e.call(this,t,n,[],r||void 0)||this}(function(e,t){function n(){this.constructor=e}v(e,t),e.prototype=null===t?Object.create(t):(n.prototype=t.prototype,new n)})(t,e)}(A);function _(e){for(var t,n={},r=[],i=1,o=arguments.length;i<o;++i){var a=arguments[i];"string"==typeof a?r.push(new C(a)):a instanceof C||a instanceof A?r.push(a):a instanceof Array?s(r,a):1!==i&&2!==i||!a||"object"!=typeof a||("render"in a?t=a:n=a)}return new A(e,n,r,t);function s(e,t){for(var n=0,r=t;n<r.length;n++){var i=r[n];"string"==typeof i?e.push(new C(i)):(i instanceof C||i instanceof A)&&e.push(i)}}}(m=_||(_={})).a=m.bind(void 0,"a"),m.abbr=m.bind(void 0,"abbr"),m.address=m.bind(void 0,"address"),m.area=m.bind(void 0,"area"),m.article=m.bind(void 0,"article"),m.aside=m.bind(void 0,"aside"),m.audio=m.bind(void 0,"audio"),m.b=m.bind(void 0,"b"),m.bdi=m.bind(void 0,"bdi"),m.bdo=m.bind(void 0,"bdo"),m.blockquote=m.bind(void 0,"blockquote"),m.br=m.bind(void 0,"br"),m.button=m.bind(void 0,"button"),m.canvas=m.bind(void 0,"canvas"),m.caption=m.bind(void 0,"caption"),m.cite=m.bind(void 0,"cite"),m.code=m.bind(void 0,"code"),m.col=m.bind(void 0,"col"),m.colgroup=m.bind(void 0,"colgroup"),m.data=m.bind(void 0,"data"),m.datalist=m.bind(void 0,"datalist"),m.dd=m.bind(void 0,"dd"),m.del=m.bind(void 0,"del"),m.dfn=m.bind(void 0,"dfn"),m.div=m.bind(void 0,"div"),m.dl=m.bind(void 0,"dl"),m.dt=m.bind(void 0,"dt"),m.em=m.bind(void 0,"em"),m.embed=m.bind(void 0,"embed"),m.fieldset=m.bind(void 0,"fieldset"),m.figcaption=m.bind(void 0,"figcaption"),m.figure=m.bind(void 0,"figure"),m.footer=m.bind(void 0,"footer"),m.form=m.bind(void 0,"form"),m.h1=m.bind(void 0,"h1"),m.h2=m.bind(void 0,"h2"),m.h3=m.bind(void 0,"h3"),m.h4=m.bind(void 0,"h4"),m.h5=m.bind(void 0,"h5"),m.h6=m.bind(void 0,"h6"),m.header=m.bind(void 0,"header"),m.hr=m.bind(void 0,"hr"),m.i=m.bind(void 0,"i"),m.iframe=m.bind(void 0,"iframe"),m.img=m.bind(void 0,"img"),m.input=m.bind(void 0,"input"),m.ins=m.bind(void 0,"ins"),m.kbd=m.bind(void 0,"kbd"),m.label=m.bind(void 0,"label"),m.legend=m.bind(void 0,"legend"),m.li=m.bind(void 0,"li"),m.main=m.bind(void 0,"main"),m.map=m.bind(void 0,"map"),m.mark=m.bind(void 0,"mark"),m.meter=m.bind(void 0,"meter"),m.nav=m.bind(void 0,"nav"),m.noscript=m.bind(void 0,"noscript"),m.object=m.bind(void 0,"object"),m.ol=m.bind(void 0,"ol"),m.optgroup=m.bind(void 0,"optgroup"),m.option=m.bind(void 0,"option"),m.output=m.bind(void 0,"output"),m.p=m.bind(void 0,"p"),m.param=m.bind(void 0,"param"),m.pre=m.bind(void 0,"pre"),m.progress=m.bind(void 0,"progress"),m.q=m.bind(void 0,"q"),m.rp=m.bind(void 0,"rp"),m.rt=m.bind(void 0,"rt"),m.ruby=m.bind(void 0,"ruby"),m.s=m.bind(void 0,"s"),m.samp=m.bind(void 0,"samp"),m.section=m.bind(void 0,"section"),m.select=m.bind(void 0,"select"),m.small=m.bind(void 0,"small"),m.source=m.bind(void 0,"source"),m.span=m.bind(void 0,"span"),m.strong=m.bind(void 0,"strong"),m.sub=m.bind(void 0,"sub"),m.summary=m.bind(void 0,"summary"),m.sup=m.bind(void 0,"sup"),m.table=m.bind(void 0,"table"),m.tbody=m.bind(void 0,"tbody"),m.td=m.bind(void 0,"td"),m.textarea=m.bind(void 0,"textarea"),m.tfoot=m.bind(void 0,"tfoot"),m.th=m.bind(void 0,"th"),m.thead=m.bind(void 0,"thead"),m.time=m.bind(void 0,"time"),m.title=m.bind(void 0,"title"),m.tr=m.bind(void 0,"tr"),m.track=m.bind(void 0,"track"),m.u=m.bind(void 0,"u"),m.ul=m.bind(void 0,"ul"),m.var_=m.bind(void 0,"var"),m.video=m.bind(void 0,"video"),m.wbr=m.bind(void 0,"wbr"),function(e){e.realize=function(e){return b.createDOMNode(e)},e.render=function(e,t){var n=b.hostMap.get(t)||[],r=b.asContentArray(e);b.hostMap.set(t,r),b.updateContent(t,n,r)}}(y||(y={})),function(e){function t(e){var n=arguments[1]||null,r=arguments[2]||null;if(n)n.insertBefore(t(e),r);else{if("text"===e.type)return document.createTextNode(e.content);if(i(n=document.createElement(e.tag),e.attrs),e.renderer)return e.renderer.render(n,{attrs:e.attrs,children:e.children}),n;for(var o=0,a=e.children.length;o<a;++o)t(e.children[o],n)}return n}e.hostMap=new WeakMap,e.asContentArray=function(e){return e?e instanceof Array?e:[e]:[]},e.createDOMNode=t,e.updateContent=function e(n,i,a){if(i!==a){for(var s=function(e,t){for(var n=e.firstChild,r=Object.create(null),i=0,o=t;i<o.length;i++){var a=o[i];"element"===a.type&&a.attrs.key&&(r[a.attrs.key]={vNode:a,element:n}),n=n.nextSibling}return r}(n,i),l=i.slice(),u=n.firstChild,c=a.length,d=0;d<c;++d)if(d>=l.length)t(a[d],n);else{var h=l[d],f=a[d];if(h!==f)if("text"!==h.type||"text"!==f.type)if("text"!==h.type&&"text"!==f.type)if(!h.renderer==!f.renderer){var p=f.attrs.key;if(p&&p in s){var g=s[p];g.vNode!==h&&(r.ArrayExt.move(l,l.indexOf(g.vNode,d+1),d),n.insertBefore(g.element,u),h=g.vNode,u=g.element)}if(h!==f){var v=h.attrs.key;v&&v!==p?(r.ArrayExt.insert(l,d,f),t(f,n,u)):h.tag===f.tag?(o(u,h.attrs,f.attrs),f.renderer?f.renderer.render(u,{attrs:f.attrs,children:f.children}):e(u,h.children,f.children),u=u.nextSibling):(r.ArrayExt.insert(l,d,f),t(f,n,u))}else u=u.nextSibling}else r.ArrayExt.insert(l,d,f),t(f,n,u);else r.ArrayExt.insert(l,d,f),t(f,n,u);else u.textContent=f.content,u=u.nextSibling;else u=u.nextSibling}!function e(t,n,r,i){for(var o=n.length-1;o>=r;--o){var a=n[o],s=i?t.lastChild:t.childNodes[o];"text"===a.type||(a.renderer&&a.renderer.unrender?a.renderer.unrender(s,{attrs:a.attrs,children:a.children}):e(s,a.children,0,!1)),i&&t.removeChild(s)}}(n,l,c,!0)}};var n={key:!0,className:!0,htmlFor:!0,dataset:!0,style:!0};function i(e,t){for(var r in t)r in n||("on"===r.substr(0,2)?e[r]=t[r]:e.setAttribute(r,t[r]));void 0!==t.className&&e.setAttribute("class",t.className),void 0!==t.htmlFor&&e.setAttribute("for",t.htmlFor),t.dataset&&function(e,t){for(var n in t)e.setAttribute("data-"+n,t[n])}(e,t.dataset),t.style&&function(e,t){var n,r=e.style;for(n in t)r[n]=t[n]}(e,t.style)}function o(e,t,r){if(t!==r){var i;for(i in t)i in n||i in r||("on"===i.substr(0,2)?e[i]=null:e.removeAttribute(i));for(i in r)i in n||t[i]===r[i]||("on"===i.substr(0,2)?e[i]=r[i]:e.setAttribute(i,r[i]));t.className!==r.className&&(void 0!==r.className?e.setAttribute("class",r.className):e.removeAttribute("class")),t.htmlFor!==r.htmlFor&&(void 0!==r.htmlFor?e.setAttribute("for",r.htmlFor):e.removeAttribute("for")),t.dataset!==r.dataset&&function(e,t,n){for(var r in t)r in n||e.removeAttribute("data-"+r);for(var i in n)t[i]!==n[i]&&e.setAttribute("data-"+i,n[i])}(e,t.dataset||{},r.dataset||{}),t.style!==r.style&&function(e,t,n){var r,i=e.style;for(r in t)r in n||(i[r]="");for(r in n)t[r]!==n[r]&&(i[r]=n[r])}(e,t.style||{},r.style||{})}}}(b||(b={}));var I,x=function(){function e(e){var t=this;this._onScrollFrame=function(){if(t._scrollTarget){var e=t._scrollTarget,n=e.element,r=e.edge,i=e.distance,o=I.SCROLL_EDGE_SIZE-i,a=Math.pow(o/I.SCROLL_EDGE_SIZE,2),s=Math.max(1,Math.round(a*I.SCROLL_EDGE_SIZE));switch(r){case"top":n.scrollTop-=s;break;case"left":n.scrollLeft-=s;break;case"right":n.scrollLeft+=s;break;case"bottom":n.scrollTop+=s}requestAnimationFrame(t._onScrollFrame)}},this._disposed=!1,this._dropAction="none",this._override=null,this._currentTarget=null,this._currentElement=null,this._promise=null,this._scrollTarget=null,this._resolve=null,this.mimeData=e.mimeData,this.dragImage=e.dragImage||null,this.proposedAction=e.proposedAction||"copy",this.supportedActions=e.supportedActions||"all",this.source=e.source||null}return e.prototype.dispose=function(){if(!this._disposed){if(this._disposed=!0,this._currentTarget){var e=I.createMouseEvent("mouseup",-1,-1);I.dispatchDragLeave(this,this._currentTarget,null,e)}this._finalize("none")}},Object.defineProperty(e.prototype,"isDisposed",{get:function(){return this._disposed},enumerable:!0,configurable:!0}),e.prototype.start=function(e,t){var n=this;if(this._disposed)return Promise.resolve("none");if(this._promise)return this._promise;this._addListeners(),this._attachDragImage(e,t),this._promise=new Promise((function(e,t){n._resolve=e}));var r=I.createMouseEvent("mousemove",e,t);return document.dispatchEvent(r),this._promise},e.prototype.handleEvent=function(e){switch(e.type){case"mousemove":this._evtMouseMove(e);break;case"mouseup":this._evtMouseUp(e);break;case"keydown":this._evtKeyDown(e);break;default:e.preventDefault(),e.stopPropagation()}},e.prototype.moveDragImage=function(e,t){if(this.dragImage){var n=this.dragImage.style;n.top=t+"px",n.left=e+"px"}},e.prototype._evtMouseMove=function(e){e.preventDefault(),e.stopPropagation(),this._updateCurrentTarget(e),this._updateDragScroll(e),this.moveDragImage(e.clientX,e.clientY)},e.prototype._evtMouseUp=function(e){if(e.preventDefault(),e.stopPropagation(),0===e.button)if(this._updateCurrentTarget(e),this._currentTarget){if("none"===this._dropAction)return I.dispatchDragLeave(this,this._currentTarget,null,e),void this._finalize("none");var t=I.dispatchDrop(this,this._currentTarget,e);this._finalize(t)}else this._finalize("none")},e.prototype._evtKeyDown=function(e){e.preventDefault(),e.stopPropagation(),27===e.keyCode&&this.dispose()},e.prototype._addListeners=function(){document.addEventListener("mousedown",this,!0),document.addEventListener("mousemove",this,!0),document.addEventListener("mouseup",this,!0),document.addEventListener("mouseenter",this,!0),document.addEventListener("mouseleave",this,!0),document.addEventListener("mouseover",this,!0),document.addEventListener("mouseout",this,!0),document.addEventListener("keydown",this,!0),document.addEventListener("keyup",this,!0),document.addEventListener("keypress",this,!0),document.addEventListener("contextmenu",this,!0)},e.prototype._removeListeners=function(){document.removeEventListener("mousedown",this,!0),document.removeEventListener("mousemove",this,!0),document.removeEventListener("mouseup",this,!0),document.removeEventListener("mouseenter",this,!0),document.removeEventListener("mouseleave",this,!0),document.removeEventListener("mouseover",this,!0),document.removeEventListener("mouseout",this,!0),document.removeEventListener("keydown",this,!0),document.removeEventListener("keyup",this,!0),document.removeEventListener("keypress",this,!0),document.removeEventListener("contextmenu",this,!0)},e.prototype._updateDragScroll=function(e){var t=I.findScrollTarget(e);(this._scrollTarget||t)&&(this._scrollTarget||setTimeout(this._onScrollFrame,500),this._scrollTarget=t)},e.prototype._updateCurrentTarget=function(e){var t=this._currentTarget,n=this._currentTarget,r=this._currentElement,i=document.elementFromPoint(e.clientX,e.clientY);this._currentElement=i,i!==r&&i!==n&&I.dispatchDragExit(this,n,i,e),i!==r&&i!==n&&(n=I.dispatchDragEnter(this,i,n,e)),n!==t&&(this._currentTarget=n,I.dispatchDragLeave(this,t,n,e));var o=I.dispatchDragOver(this,n,e);this._setDropAction(o)},e.prototype._attachDragImage=function(e,t){if(this.dragImage){this.dragImage.classList.add("lm-mod-drag-image"),this.dragImage.classList.add("p-mod-drag-image");var n=this.dragImage.style;n.pointerEvents="none",n.position="fixed",n.top=t+"px",n.left=e+"px",document.body.appendChild(this.dragImage)}},e.prototype._detachDragImage=function(){if(this.dragImage){var e=this.dragImage.parentNode;e&&e.removeChild(this.dragImage)}},e.prototype._setDropAction=function(t){if(t=I.validateAction(t,this.supportedActions),!this._override||this._dropAction!==t)switch(t){case"none":this._dropAction=t,this._override=e.overrideCursor("no-drop");break;case"copy":this._dropAction=t,this._override=e.overrideCursor("copy");break;case"link":this._dropAction=t,this._override=e.overrideCursor("alias");break;case"move":this._dropAction=t,this._override=e.overrideCursor("move")}},e.prototype._finalize=function(e){var t=this._resolve;this._removeListeners(),this._detachDragImage(),this._override&&(this._override.dispose(),this._override=null),this.mimeData.clear(),this._disposed=!0,this._dropAction="none",this._currentTarget=null,this._currentElement=null,this._scrollTarget=null,this._promise=null,this._resolve=null,t&&t(e)},e}();!function(e){e.overrideCursor=function(e){var n=++t;return document.body.style.cursor=e,document.body.classList.add("lm-mod-override-cursor"),document.body.classList.add("p-mod-override-cursor"),new u.DisposableDelegate((function(){n===t&&(document.body.style.cursor="",document.body.classList.remove("lm-mod-override-cursor"),document.body.classList.remove("p-mod-override-cursor"))}))};var t=0}(x||(x={})),function(e){e.SCROLL_EDGE_SIZE=20,e.validateAction=function(e,r){return t[e]&n[r]?e:"none"},e.createMouseEvent=function(e,t,n){var r=document.createEvent("MouseEvent");return r.initMouseEvent(e,!0,!0,window,0,0,0,t,n,!1,!1,!1,!1,0,null),r},e.findScrollTarget=function(t){for(var n=t.clientX,r=t.clientY,i=document.elementFromPoint(n,r);i;i=i.parentElement){var o=i.hasAttribute("data-lm-dragscroll");if(o=o||i.hasAttribute("data-p-dragscroll")){var a=0,s=0;i===document.body&&(a=window.pageXOffset,s=window.pageYOffset);var l=i.getBoundingClientRect(),u=l.top+s,c=l.left+a,d=c+l.width,h=u+l.height;if(!(n<c||n>=d||r<u||r>=h)){var f=n-c+1,p=r-u+1,g=d-n,v=h-r,m=Math.min(f,p,g,v);if(!(m>e.SCROLL_EDGE_SIZE)){var y=void 0;switch(m){case v:y="bottom";break;case p:y="top";break;case g:y="right";break;case f:y="left";break;default:throw"unreachable"}var b=i.scrollWidth-i.clientWidth,C=i.scrollHeight-i.clientHeight,A=void 0;switch(y){case"top":A=C>0&&i.scrollTop>0;break;case"left":A=b>0&&i.scrollLeft>0;break;case"right":A=b>0&&i.scrollLeft<b;break;case"bottom":A=C>0&&i.scrollTop<C;break;default:throw"unreachable"}if(A)return{element:i,edge:y,distance:m}}}}}return null},e.dispatchDragEnter=function(e,t,n,i){if(!t)return null;var o=r("lm-dragenter",e,i,n),a=!t.dispatchEvent(o);return a?t:(o=r("p-dragenter",e,i,n),(a=!t.dispatchEvent(o))?t:t===document.body?n:(o=r("lm-dragenter",e,i,n),document.body.dispatchEvent(o),o=r("p-dragenter",e,i,n),document.body.dispatchEvent(o),document.body))},e.dispatchDragExit=function(e,t,n,i){if(t){var o=r("lm-dragexit",e,i,n);t.dispatchEvent(o),o=r("p-dragexit",e,i,n),t.dispatchEvent(o)}},e.dispatchDragLeave=function(e,t,n,i){if(t){var o=r("lm-dragleave",e,i,n);t.dispatchEvent(o),o=r("p-dragleave",e,i,n),t.dispatchEvent(o)}},e.dispatchDragOver=function(e,t,n){if(!t)return"none";var i=r("lm-dragover",e,n,null),o=!t.dispatchEvent(i);return o?i.dropAction:(i=r("p-dragover",e,n,null),(o=!t.dispatchEvent(i))?i.dropAction:"none")},e.dispatchDrop=function(e,t,n){if(!t)return"none";var i=r("lm-drop",e,n,null),o=!t.dispatchEvent(i);return o?i.dropAction:(i=r("p-drop",e,n,null),(o=!t.dispatchEvent(i))?i.dropAction:"none")};var t={none:0,copy:1,link:2,move:4},n={none:t.none,copy:t.copy,link:t.link,move:t.move,"copy-link":t.copy|t.link,"copy-move":t.copy|t.move,"link-move":t.link|t.move,all:t.copy|t.link|t.move};function r(e,t,n,r){var i=document.createEvent("MouseEvent");return i.initMouseEvent(e,!0,!0,window,0,n.screenX,n.screenY,n.clientX,n.clientY,n.ctrlKey,n.altKey,n.shiftKey,n.metaKey,n.button,r),i.dropAction="none",i.mimeData=t.mimeData,i.proposedAction=t.proposedAction,i.supportedActions=t.supportedActions,i.source=t.source,i}}(I||(I={})),n.d(t,"BoxEngine",(function(){return w})),n.d(t,"BoxLayout",(function(){return B})),n.d(t,"BoxPanel",(function(){return Y})),n.d(t,"BoxSizer",(function(){return M})),n.d(t,"CommandPalette",(function(){return q})),n.d(t,"ContextMenu",(function(){return J})),n.d(t,"DockLayout",(function(){return ee})),n.d(t,"DockPanel",(function(){return ie})),n.d(t,"FocusTracker",(function(){return se})),n.d(t,"GridLayout",(function(){return le})),n.d(t,"Layout",(function(){return P})),n.d(t,"LayoutItem",(function(){return j})),n.d(t,"Menu",(function(){return Q})),n.d(t,"MenuBar",(function(){return ce})),n.d(t,"Panel",(function(){return U})),n.d(t,"PanelLayout",(function(){return F})),n.d(t,"ScrollBar",(function(){return he})),n.d(t,"SingletonLayout",(function(){return ge})),n.d(t,"SplitLayout",(function(){return ve})),n.d(t,"SplitPanel",(function(){return ye})),n.d(t,"StackedLayout",(function(){return Ce})),n.d(t,"StackedPanel",(function(){return Ae})),n.d(t,"TabBar",(function(){return ne})),n.d(t,"TabPanel",(function(){return Ie})),n.d(t,"Title",(function(){return k})),n.d(t,"Widget",(function(){return L}));var w,M=function(){this.sizeHint=0,this.minSize=0,this.maxSize=1/0,this.stretch=1,this.size=0,this.done=!1};!function(e){e.calc=function(e,t){var n=e.length;if(0===n)return t;for(var r=0,i=0,o=0,a=0,s=0,l=0;l<n;++l){var u=(m=e[l]).minSize,c=m.maxSize,d=m.sizeHint;m.done=!1,m.size=Math.max(u,Math.min(d,c)),o+=m.size,r+=u,i+=c,m.stretch>0&&(a+=m.stretch,s++)}if(t===o)return 0;if(t<=r){for(l=0;l<n;++l){(m=e[l]).size=m.minSize}return t-r}if(t>=i){for(l=0;l<n;++l){(m=e[l]).size=m.maxSize}return t-i}var h=n;if(t<o){for(var f=o-t;s>0&&f>.01;){var p=f,g=a;for(l=0;l<n;++l){if(!(m=e[l]).done&&0!==m.stretch){var v=m.stretch*p/g;m.size-v<=m.minSize?(f-=m.size-m.minSize,a-=m.stretch,m.size=m.minSize,m.done=!0,h--,s--):(f-=v,m.size-=v)}}}for(;h>0&&f>.01;)for(v=f/h,l=0;l<n;++l){(m=e[l]).done||(m.size-v<=m.minSize?(f-=m.size-m.minSize,m.size=m.minSize,m.done=!0,h--):(f-=v,m.size-=v))}}else{for(f=t-o;s>0&&f>.01;)for(p=f,g=a,l=0;l<n;++l){if(!(m=e[l]).done&&0!==m.stretch){v=m.stretch*p/g;m.size+v>=m.maxSize?(f-=m.maxSize-m.size,a-=m.stretch,m.size=m.maxSize,m.done=!0,h--,s--):(f-=v,m.size+=v)}}for(;h>0&&f>.01;)for(v=f/h,l=0;l<n;++l){var m;(m=e[l]).done||(m.size+v>=m.maxSize?(f-=m.maxSize-m.size,m.size=m.maxSize,m.done=!0,h--):(f-=v,m.size+=v))}}return 0},e.adjust=function(e,t,n){0!==e.length&&0!==n&&(n>0?function(e,t,n){for(var r=0,i=0;i<=t;++i){var o=e[i];r+=o.maxSize-o.size}for(var a=0,s=(i=t+1,e.length);i<s;++i){o=e[i];a+=o.size-o.minSize}var l=n=Math.min(n,r,a);for(i=t;i>=0&&l>0;--i){o=e[i];(c=o.maxSize-o.size)>=l?(o.sizeHint=o.size+l,l=0):(o.sizeHint=o.size+c,l-=c)}var u=n;for(i=t+1,s=e.length;i<s&&u>0;++i){var c;o=e[i];(c=o.size-o.minSize)>=u?(o.sizeHint=o.size-u,u=0):(o.sizeHint=o.size-c,u-=c)}}(e,t,n):function(e,t,n){for(var r=0,i=t+1,o=e.length;i<o;++i){var a=e[i];r+=a.maxSize-a.size}var s=0;for(i=0;i<=t;++i){a=e[i];s+=a.size-a.minSize}var l=n=Math.min(n,r,s);for(i=t+1,o=e.length;i<o&&l>0;++i){a=e[i];(c=a.maxSize-a.size)>=l?(a.sizeHint=a.size+l,l=0):(a.sizeHint=a.size+c,l-=c)}var u=n;for(i=t;i>=0&&u>0;--i){var c;a=e[i];(c=a.size-a.minSize)>=u?(a.sizeHint=a.size-u,u=0):(a.sizeHint=a.size-c,u-=c)}}(e,t,-n))}}(w||(w={}));
/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
var S=function(e,t){return(S=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(e,t)};function T(e,t){function n(){this.constructor=e}S(e,t),e.prototype=null===t?Object.create(t):(n.prototype=t.prototype,new n)}var E,O=function(){return(O=Object.assign||function(e){for(var t,n=1,r=arguments.length;n<r;n++)for(var i in t=arguments[n])Object.prototype.hasOwnProperty.call(t,i)&&(e[i]=t[i]);return e}).apply(this,arguments)},k=function(){function e(e){this._label="",this._caption="",this._mnemonic=-1,this._iconClass="",this._iconLabel="",this._className="",this._closable=!1,this._changed=new s.Signal(this),this.owner=e.owner,void 0!==e.label&&(this._label=e.label),void 0!==e.mnemonic&&(this._mnemonic=e.mnemonic),void 0!==e.icon?"string"==typeof e.icon?(this._icon=null,this._iconClass=e.icon):this._icon=e.icon:this._icon=null,void 0!==e.iconClass&&(this._iconClass=e.iconClass),void 0!==e.iconLabel&&(this._iconLabel=e.iconLabel),void 0!==e.iconRenderer&&(this._icon=e.iconRenderer),void 0!==e.caption&&(this._caption=e.caption),void 0!==e.className&&(this._className=e.className),void 0!==e.closable&&(this._closable=e.closable),this._dataset=e.dataset||{}}return Object.defineProperty(e.prototype,"changed",{get:function(){return this._changed},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"label",{get:function(){return this._label},set:function(e){this._label!==e&&(this._label=e,this._changed.emit(void 0))},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"mnemonic",{get:function(){return this._mnemonic},set:function(e){this._mnemonic!==e&&(this._mnemonic=e,this._changed.emit(void 0))},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"icon",{get:function(){return null===this._icon?this.iconClass:this._icon},set:function(e){if("string"==typeof e)this._icon=null,this.iconClass=e;else{if(this._icon===e)return;this._icon=e,this._changed.emit(void 0)}},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"iconClass",{get:function(){return this._iconClass},set:function(e){this._iconClass!==e&&(this._iconClass=e,this._changed.emit(void 0))},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"iconLabel",{get:function(){return this._iconLabel},set:function(e){this._iconLabel!==e&&(this._iconLabel=e,this._changed.emit(void 0))},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"iconRenderer",{get:function(){return this._icon||void 0},set:function(e){this.icon=e},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"caption",{get:function(){return this._caption},set:function(e){this._caption!==e&&(this._caption=e,this._changed.emit(void 0))},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"className",{get:function(){return this._className},set:function(e){this._className!==e&&(this._className=e,this._changed.emit(void 0))},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"closable",{get:function(){return this._closable},set:function(e){this._closable!==e&&(this._closable=e,this._changed.emit(void 0))},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"dataset",{get:function(){return this._dataset},set:function(e){this._dataset!==e&&(this._dataset=e,this._changed.emit(void 0))},enumerable:!0,configurable:!0}),e}(),L=function(){function e(e){void 0===e&&(e={}),this._flags=0,this._layout=null,this._parent=null,this._disposed=new s.Signal(this),this.node=E.createNode(e),this.addClass("lm-Widget"),this.addClass("p-Widget")}return e.prototype.dispose=function(){this.isDisposed||(this.setFlag(e.Flag.IsDisposed),this._disposed.emit(void 0),this.parent?this.parent=null:this.isAttached&&e.detach(this),this._layout&&(this._layout.dispose(),this._layout=null),s.Signal.clearData(this),o.c.clearData(this),a.AttachedProperty.clearData(this))},Object.defineProperty(e.prototype,"disposed",{get:function(){return this._disposed},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"isDisposed",{get:function(){return this.testFlag(e.Flag.IsDisposed)},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"isAttached",{get:function(){return this.testFlag(e.Flag.IsAttached)},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"isHidden",{get:function(){return this.testFlag(e.Flag.IsHidden)},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"isVisible",{get:function(){return this.testFlag(e.Flag.IsVisible)},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"title",{get:function(){return E.titleProperty.get(this)},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"id",{get:function(){return this.node.id},set:function(e){this.node.id=e},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"dataset",{get:function(){return this.node.dataset},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"parent",{get:function(){return this._parent},set:function(t){if(this._parent!==t){if(t&&this.contains(t))throw new Error("Invalid parent widget.");if(this._parent&&!this._parent.isDisposed){var n=new e.ChildMessage("child-removed",this);o.c.sendMessage(this._parent,n)}if(this._parent=t,this._parent&&!this._parent.isDisposed){n=new e.ChildMessage("child-added",this);o.c.sendMessage(this._parent,n)}this.isDisposed||o.c.sendMessage(this,e.Msg.ParentChanged)}},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"layout",{get:function(){return this._layout},set:function(t){if(this._layout!==t){if(this.testFlag(e.Flag.DisallowLayout))throw new Error("Cannot set widget layout.");if(this._layout)throw new Error("Cannot change widget layout.");if(t.parent)throw new Error("Cannot change layout parent.");this._layout=t,t.parent=this}},enumerable:!0,configurable:!0}),e.prototype.children=function(){return this._layout?this._layout.iter():Object(r.empty)()},e.prototype.contains=function(e){for(var t=e;t;t=t._parent)if(t===this)return!0;return!1},e.prototype.hasClass=function(e){return this.node.classList.contains(e)},e.prototype.addClass=function(e){this.node.classList.add(e)},e.prototype.removeClass=function(e){this.node.classList.remove(e)},e.prototype.toggleClass=function(e,t){return!0===t?(this.node.classList.add(e),!0):!1===t?(this.node.classList.remove(e),!1):this.node.classList.toggle(e)},e.prototype.update=function(){o.c.postMessage(this,e.Msg.UpdateRequest)},e.prototype.fit=function(){o.c.postMessage(this,e.Msg.FitRequest)},e.prototype.activate=function(){o.c.postMessage(this,e.Msg.ActivateRequest)},e.prototype.close=function(){o.c.sendMessage(this,e.Msg.CloseRequest)},e.prototype.show=function(){if(this.testFlag(e.Flag.IsHidden)&&(!this.isAttached||this.parent&&!this.parent.isVisible||o.c.sendMessage(this,e.Msg.BeforeShow),this.clearFlag(e.Flag.IsHidden),this.removeClass("lm-mod-hidden"),this.removeClass("p-mod-hidden"),!this.isAttached||this.parent&&!this.parent.isVisible||o.c.sendMessage(this,e.Msg.AfterShow),this.parent)){var t=new e.ChildMessage("child-shown",this);o.c.sendMessage(this.parent,t)}},e.prototype.hide=function(){if(!this.testFlag(e.Flag.IsHidden)&&(!this.isAttached||this.parent&&!this.parent.isVisible||o.c.sendMessage(this,e.Msg.BeforeHide),this.setFlag(e.Flag.IsHidden),this.addClass("lm-mod-hidden"),this.addClass("p-mod-hidden"),!this.isAttached||this.parent&&!this.parent.isVisible||o.c.sendMessage(this,e.Msg.AfterHide),this.parent)){var t=new e.ChildMessage("child-hidden",this);o.c.sendMessage(this.parent,t)}},e.prototype.setHidden=function(e){e?this.hide():this.show()},e.prototype.testFlag=function(e){return 0!=(this._flags&e)},e.prototype.setFlag=function(e){this._flags|=e},e.prototype.clearFlag=function(e){this._flags&=~e},e.prototype.processMessage=function(t){switch(t.type){case"resize":this.notifyLayout(t),this.onResize(t);break;case"update-request":this.notifyLayout(t),this.onUpdateRequest(t);break;case"fit-request":this.notifyLayout(t),this.onFitRequest(t);break;case"before-show":this.notifyLayout(t),this.onBeforeShow(t);break;case"after-show":this.setFlag(e.Flag.IsVisible),this.notifyLayout(t),this.onAfterShow(t);break;case"before-hide":this.notifyLayout(t),this.onBeforeHide(t);break;case"after-hide":this.clearFlag(e.Flag.IsVisible),this.notifyLayout(t),this.onAfterHide(t);break;case"before-attach":this.notifyLayout(t),this.onBeforeAttach(t);break;case"after-attach":this.isHidden||this.parent&&!this.parent.isVisible||this.setFlag(e.Flag.IsVisible),this.setFlag(e.Flag.IsAttached),this.notifyLayout(t),this.onAfterAttach(t);break;case"before-detach":this.notifyLayout(t),this.onBeforeDetach(t);break;case"after-detach":this.clearFlag(e.Flag.IsVisible),this.clearFlag(e.Flag.IsAttached),this.notifyLayout(t),this.onAfterDetach(t);break;case"activate-request":this.notifyLayout(t),this.onActivateRequest(t);break;case"close-request":this.notifyLayout(t),this.onCloseRequest(t);break;case"child-added":this.notifyLayout(t),this.onChildAdded(t);break;case"child-removed":this.notifyLayout(t),this.onChildRemoved(t);break;default:this.notifyLayout(t)}},e.prototype.notifyLayout=function(e){this._layout&&this._layout.processParentMessage(e)},e.prototype.onCloseRequest=function(t){this.parent?this.parent=null:this.isAttached&&e.detach(this)},e.prototype.onResize=function(e){},e.prototype.onUpdateRequest=function(e){},e.prototype.onFitRequest=function(e){},e.prototype.onActivateRequest=function(e){},e.prototype.onBeforeShow=function(e){},e.prototype.onAfterShow=function(e){},e.prototype.onBeforeHide=function(e){},e.prototype.onAfterHide=function(e){},e.prototype.onBeforeAttach=function(e){},e.prototype.onAfterAttach=function(e){},e.prototype.onBeforeDetach=function(e){},e.prototype.onAfterDetach=function(e){},e.prototype.onChildAdded=function(e){},e.prototype.onChildRemoved=function(e){},e}();!function(e){!function(e){e[e.IsDisposed=1]="IsDisposed",e[e.IsAttached=2]="IsAttached",e[e.IsHidden=4]="IsHidden",e[e.IsVisible=8]="IsVisible",e[e.DisallowLayout=16]="DisallowLayout"}(e.Flag||(e.Flag={})),function(e){e.BeforeShow=new o.b("before-show"),e.AfterShow=new o.b("after-show"),e.BeforeHide=new o.b("before-hide"),e.AfterHide=new o.b("after-hide"),e.BeforeAttach=new o.b("before-attach"),e.AfterAttach=new o.b("after-attach"),e.BeforeDetach=new o.b("before-detach"),e.AfterDetach=new o.b("after-detach"),e.ParentChanged=new o.b("parent-changed"),e.UpdateRequest=new o.a("update-request"),e.FitRequest=new o.a("fit-request"),e.ActivateRequest=new o.a("activate-request"),e.CloseRequest=new o.a("close-request")}(e.Msg||(e.Msg={}));var t=function(e){function t(t,n){var r=e.call(this,t)||this;return r.child=n,r}return T(t,e),t}(o.b);e.ChildMessage=t;var n=function(e){function t(t,n){var r=e.call(this,"resize")||this;return r.width=t,r.height=n,r}return T(t,e),t}(o.b);e.ResizeMessage=n,function(e){e.UnknownSize=new e(-1,-1)}(n=e.ResizeMessage||(e.ResizeMessage={})),e.attach=function(t,n,r){if(void 0===r&&(r=null),t.parent)throw new Error("Cannot attach a child widget.");if(t.isAttached||document.body.contains(t.node))throw new Error("Widget is already attached.");if(!document.body.contains(n))throw new Error("Host is not attached.");o.c.sendMessage(t,e.Msg.BeforeAttach),n.insertBefore(t.node,r),o.c.sendMessage(t,e.Msg.AfterAttach)},e.detach=function(t){if(t.parent)throw new Error("Cannot detach a child widget.");if(!t.isAttached||!document.body.contains(t.node))throw new Error("Widget is not attached.");o.c.sendMessage(t,e.Msg.BeforeDetach),t.node.parentNode.removeChild(t.node),o.c.sendMessage(t,e.Msg.AfterDetach)}}(L||(L={})),function(e){e.titleProperty=new a.AttachedProperty({name:"title",create:function(e){return new k({owner:e})}}),e.createNode=function(e){return e.node||document.createElement(e.tag||"div")}}(E||(E={}));var D,P=function(){function e(e){void 0===e&&(e={}),this._disposed=!1,this._parent=null,this._fitPolicy=e.fitPolicy||"set-min-size"}return e.prototype.dispose=function(){this._parent=null,this._disposed=!0,s.Signal.clearData(this),a.AttachedProperty.clearData(this)},Object.defineProperty(e.prototype,"isDisposed",{get:function(){return this._disposed},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"parent",{get:function(){return this._parent},set:function(e){if(this._parent!==e){if(this._parent)throw new Error("Cannot change parent widget.");if(e.layout!==this)throw new Error("Invalid parent widget.");this._parent=e,this.init()}},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"fitPolicy",{get:function(){return this._fitPolicy},set:function(e){if(this._fitPolicy!==e&&(this._fitPolicy=e,this._parent)){var t=this._parent.node.style;t.minWidth="",t.minHeight="",t.maxWidth="",t.maxHeight="",this._parent.fit()}},enumerable:!0,configurable:!0}),e.prototype.processParentMessage=function(e){switch(e.type){case"resize":this.onResize(e);break;case"update-request":this.onUpdateRequest(e);break;case"fit-request":this.onFitRequest(e);break;case"before-show":this.onBeforeShow(e);break;case"after-show":this.onAfterShow(e);break;case"before-hide":this.onBeforeHide(e);break;case"after-hide":this.onAfterHide(e);break;case"before-attach":this.onBeforeAttach(e);break;case"after-attach":this.onAfterAttach(e);break;case"before-detach":this.onBeforeDetach(e);break;case"after-detach":this.onAfterDetach(e);break;case"child-removed":this.onChildRemoved(e);break;case"child-shown":this.onChildShown(e);break;case"child-hidden":this.onChildHidden(e)}},e.prototype.init=function(){var e=this;Object(r.each)(this,(function(t){t.parent=e.parent}))},e.prototype.onResize=function(e){Object(r.each)(this,(function(e){o.c.sendMessage(e,L.ResizeMessage.UnknownSize)}))},e.prototype.onUpdateRequest=function(e){Object(r.each)(this,(function(e){o.c.sendMessage(e,L.ResizeMessage.UnknownSize)}))},e.prototype.onBeforeAttach=function(e){Object(r.each)(this,(function(t){o.c.sendMessage(t,e)}))},e.prototype.onAfterAttach=function(e){Object(r.each)(this,(function(t){o.c.sendMessage(t,e)}))},e.prototype.onBeforeDetach=function(e){Object(r.each)(this,(function(t){o.c.sendMessage(t,e)}))},e.prototype.onAfterDetach=function(e){Object(r.each)(this,(function(t){o.c.sendMessage(t,e)}))},e.prototype.onBeforeShow=function(e){Object(r.each)(this,(function(t){t.isHidden||o.c.sendMessage(t,e)}))},e.prototype.onAfterShow=function(e){Object(r.each)(this,(function(t){t.isHidden||o.c.sendMessage(t,e)}))},e.prototype.onBeforeHide=function(e){Object(r.each)(this,(function(t){t.isHidden||o.c.sendMessage(t,e)}))},e.prototype.onAfterHide=function(e){Object(r.each)(this,(function(t){t.isHidden||o.c.sendMessage(t,e)}))},e.prototype.onChildRemoved=function(e){this.removeWidget(e.child)},e.prototype.onFitRequest=function(e){},e.prototype.onChildShown=function(e){},e.prototype.onChildHidden=function(e){},e}();(D=P||(P={})).getHorizontalAlignment=function(e){return N.horizontalAlignmentProperty.get(e)},D.setHorizontalAlignment=function(e,t){N.horizontalAlignmentProperty.set(e,t)},D.getVerticalAlignment=function(e){return N.verticalAlignmentProperty.get(e)},D.setVerticalAlignment=function(e,t){N.verticalAlignmentProperty.set(e,t)};var N,j=function(){function e(e){this._top=NaN,this._left=NaN,this._width=NaN,this._height=NaN,this._minWidth=0,this._minHeight=0,this._maxWidth=1/0,this._maxHeight=1/0,this._disposed=!1,this.widget=e,this.widget.node.style.position="absolute"}return e.prototype.dispose=function(){if(!this._disposed){this._disposed=!0;var e=this.widget.node.style;e.position="",e.top="",e.left="",e.width="",e.height=""}},Object.defineProperty(e.prototype,"minWidth",{get:function(){return this._minWidth},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"minHeight",{get:function(){return this._minHeight},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"maxWidth",{get:function(){return this._maxWidth},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"maxHeight",{get:function(){return this._maxHeight},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"isDisposed",{get:function(){return this._disposed},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"isHidden",{get:function(){return this.widget.isHidden},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"isVisible",{get:function(){return this.widget.isVisible},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"isAttached",{get:function(){return this.widget.isAttached},enumerable:!0,configurable:!0}),e.prototype.fit=function(){var e=i.a.sizeLimits(this.widget.node);this._minWidth=e.minWidth,this._minHeight=e.minHeight,this._maxWidth=e.maxWidth,this._maxHeight=e.maxHeight},e.prototype.update=function(e,t,n,r){var i=Math.max(this._minWidth,Math.min(n,this._maxWidth)),a=Math.max(this._minHeight,Math.min(r,this._maxHeight));if(i<n)switch(P.getHorizontalAlignment(this.widget)){case"left":break;case"center":e+=(n-i)/2;break;case"right":e+=n-i;break;default:throw"unreachable"}if(a<r)switch(P.getVerticalAlignment(this.widget)){case"top":break;case"center":t+=(r-a)/2;break;case"bottom":t+=r-a;break;default:throw"unreachable"}var s=!1,l=this.widget.node.style;if(this._top!==t&&(this._top=t,l.top=t+"px"),this._left!==e&&(this._left=e,l.left=e+"px"),this._width!==i&&(s=!0,this._width=i,l.width=i+"px"),this._height!==a&&(s=!0,this._height=a,l.height=a+"px"),s){var u=new L.ResizeMessage(i,a);o.c.sendMessage(this.widget,u)}},e}();!function(e){function t(e){e.parent&&e.parent.layout&&e.parent.update()}e.horizontalAlignmentProperty=new a.AttachedProperty({name:"horizontalAlignment",create:function(){return"center"},changed:t}),e.verticalAlignmentProperty=new a.AttachedProperty({name:"verticalAlignment",create:function(){return"top"},changed:t})}(N||(N={}));var R,z,F=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t._widgets=[],t}return T(t,e),t.prototype.dispose=function(){for(;this._widgets.length>0;)this._widgets.pop().dispose();e.prototype.dispose.call(this)},Object.defineProperty(t.prototype,"widgets",{get:function(){return this._widgets},enumerable:!0,configurable:!0}),t.prototype.iter=function(){return Object(r.iter)(this._widgets)},t.prototype.addWidget=function(e){this.insertWidget(this._widgets.length,e)},t.prototype.insertWidget=function(e,t){t.parent=this.parent;var n=this._widgets.indexOf(t),i=Math.max(0,Math.min(e,this._widgets.length));if(-1===n)return r.ArrayExt.insert(this._widgets,i,t),void(this.parent&&this.attachWidget(i,t));i===this._widgets.length&&i--,n!==i&&(r.ArrayExt.move(this._widgets,n,i),this.parent&&this.moveWidget(n,i,t))},t.prototype.removeWidget=function(e){this.removeWidgetAt(this._widgets.indexOf(e))},t.prototype.removeWidgetAt=function(e){var t=r.ArrayExt.removeAt(this._widgets,e);t&&this.parent&&this.detachWidget(e,t)},t.prototype.init=function(){var t=this;e.prototype.init.call(this),Object(r.each)(this,(function(e,n){t.attachWidget(n,e)}))},t.prototype.attachWidget=function(e,t){var n=this.parent.node.children[e];this.parent.isAttached&&o.c.sendMessage(t,L.Msg.BeforeAttach),this.parent.node.insertBefore(t.node,n),this.parent.isAttached&&o.c.sendMessage(t,L.Msg.AfterAttach)},t.prototype.moveWidget=function(e,t,n){this.parent.isAttached&&o.c.sendMessage(n,L.Msg.BeforeDetach),this.parent.node.removeChild(n.node),this.parent.isAttached&&o.c.sendMessage(n,L.Msg.AfterDetach);var r=this.parent.node.children[t];this.parent.isAttached&&o.c.sendMessage(n,L.Msg.BeforeAttach),this.parent.node.insertBefore(n.node,r),this.parent.isAttached&&o.c.sendMessage(n,L.Msg.AfterAttach)},t.prototype.detachWidget=function(e,t){this.parent.isAttached&&o.c.sendMessage(t,L.Msg.BeforeDetach),this.parent.node.removeChild(t.node),this.parent.isAttached&&o.c.sendMessage(t,L.Msg.AfterDetach)},t}(P),B=function(e){function t(t){void 0===t&&(t={});var n=e.call(this)||this;return n._fixed=0,n._spacing=4,n._dirty=!1,n._sizers=[],n._items=[],n._box=null,n._alignment="start",n._direction="top-to-bottom",void 0!==t.direction&&(n._direction=t.direction),void 0!==t.alignment&&(n._alignment=t.alignment),void 0!==t.spacing&&(n._spacing=z.clampSpacing(t.spacing)),n}return T(t,e),t.prototype.dispose=function(){Object(r.each)(this._items,(function(e){e.dispose()})),this._box=null,this._items.length=0,this._sizers.length=0,e.prototype.dispose.call(this)},Object.defineProperty(t.prototype,"direction",{get:function(){return this._direction},set:function(e){this._direction!==e&&(this._direction=e,this.parent&&(this.parent.dataset.direction=e,this.parent.fit()))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"alignment",{get:function(){return this._alignment},set:function(e){this._alignment!==e&&(this._alignment=e,this.parent&&(this.parent.dataset.alignment=e,this.parent.update()))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"spacing",{get:function(){return this._spacing},set:function(e){e=z.clampSpacing(e),this._spacing!==e&&(this._spacing=e,this.parent&&this.parent.fit())},enumerable:!0,configurable:!0}),t.prototype.init=function(){this.parent.dataset.direction=this.direction,this.parent.dataset.alignment=this.alignment,e.prototype.init.call(this)},t.prototype.attachWidget=function(e,t){r.ArrayExt.insert(this._items,e,new j(t)),r.ArrayExt.insert(this._sizers,e,new M),this.parent.isAttached&&o.c.sendMessage(t,L.Msg.BeforeAttach),this.parent.node.appendChild(t.node),this.parent.isAttached&&o.c.sendMessage(t,L.Msg.AfterAttach),this.parent.fit()},t.prototype.moveWidget=function(e,t,n){r.ArrayExt.move(this._items,e,t),r.ArrayExt.move(this._sizers,e,t),this.parent.update()},t.prototype.detachWidget=function(e,t){var n=r.ArrayExt.removeAt(this._items,e);r.ArrayExt.removeAt(this._sizers,e),this.parent.isAttached&&o.c.sendMessage(t,L.Msg.BeforeDetach),this.parent.node.removeChild(t.node),this.parent.isAttached&&o.c.sendMessage(t,L.Msg.AfterDetach),n.dispose(),this.parent.fit()},t.prototype.onBeforeShow=function(t){e.prototype.onBeforeShow.call(this,t),this.parent.update()},t.prototype.onBeforeAttach=function(t){e.prototype.onBeforeAttach.call(this,t),this.parent.fit()},t.prototype.onChildShown=function(e){this.parent.fit()},t.prototype.onChildHidden=function(e){this.parent.fit()},t.prototype.onResize=function(e){this.parent.isVisible&&this._update(e.width,e.height)},t.prototype.onUpdateRequest=function(e){this.parent.isVisible&&this._update(-1,-1)},t.prototype.onFitRequest=function(e){this.parent.isAttached&&this._fit()},t.prototype._fit=function(){for(var e=0,n=0,r=this._items.length;n<r;++n)e+=+!this._items[n].isHidden;this._fixed=this._spacing*Math.max(0,e-1);var a=z.isHorizontal(this._direction),s=a?this._fixed:0,l=a?0:this._fixed;for(n=0,r=this._items.length;n<r;++n){var u=this._items[n],c=this._sizers[n];u.isHidden?(c.minSize=0,c.maxSize=0):(u.fit(),c.sizeHint=t.getSizeBasis(u.widget),c.stretch=t.getStretch(u.widget),a?(c.minSize=u.minWidth,c.maxSize=u.maxWidth,s+=u.minWidth,l=Math.max(l,u.minHeight)):(c.minSize=u.minHeight,c.maxSize=u.maxHeight,l+=u.minHeight,s=Math.max(s,u.minWidth)))}var d=this._box=i.a.boxSizing(this.parent.node);s+=d.horizontalSum,l+=d.verticalSum;var h=this.parent.node.style;h.minWidth=s+"px",h.minHeight=l+"px",this._dirty=!0,this.parent.parent&&o.c.sendMessage(this.parent.parent,L.Msg.FitRequest),this._dirty&&o.c.sendMessage(this.parent,L.Msg.UpdateRequest)},t.prototype._update=function(e,t){this._dirty=!1;for(var n=0,r=0,o=this._items.length;r<o;++r)n+=+!this._items[r].isHidden;if(0!==n){e<0&&(e=this.parent.node.offsetWidth),t<0&&(t=this.parent.node.offsetHeight),this._box||(this._box=i.a.boxSizing(this.parent.node));var a,s=this._box.paddingTop,l=this._box.paddingLeft,u=e-this._box.horizontalSum,c=t-this._box.verticalSum;switch(this._direction){case"left-to-right":a=w.calc(this._sizers,Math.max(0,u-this._fixed));break;case"top-to-bottom":a=w.calc(this._sizers,Math.max(0,c-this._fixed));break;case"right-to-left":a=w.calc(this._sizers,Math.max(0,u-this._fixed)),l+=u;break;case"bottom-to-top":a=w.calc(this._sizers,Math.max(0,c-this._fixed)),s+=c;break;default:throw"unreachable"}var d=0,h=0;if(a>0)switch(this._alignment){case"start":break;case"center":d=0,h=a/2;break;case"end":d=0,h=a;break;case"justify":d=a/n,h=0;break;default:throw"unreachable"}for(r=0,o=this._items.length;r<o;++r){var f=this._items[r];if(!f.isHidden){var p=this._sizers[r].size;switch(this._direction){case"left-to-right":f.update(l+h,s,p+d,c),l+=p+d+this._spacing;break;case"top-to-bottom":f.update(l,s+h,u,p+d),s+=p+d+this._spacing;break;case"right-to-left":f.update(l-h-p-d,s,p+d,c),l-=p+d+this._spacing;break;case"bottom-to-top":f.update(l,s-h-p-d,u,p+d),s-=p+d+this._spacing;break;default:throw"unreachable"}}}}},t}(F);(R=B||(B={})).getStretch=function(e){return z.stretchProperty.get(e)},R.setStretch=function(e,t){z.stretchProperty.set(e,t)},R.getSizeBasis=function(e){return z.sizeBasisProperty.get(e)},R.setSizeBasis=function(e,t){z.sizeBasisProperty.set(e,t)},function(e){function t(e){e.parent&&e.parent.layout instanceof B&&e.parent.fit()}e.stretchProperty=new a.AttachedProperty({name:"stretch",create:function(){return 0},coerce:function(e,t){return Math.max(0,Math.floor(t))},changed:t}),e.sizeBasisProperty=new a.AttachedProperty({name:"sizeBasis",create:function(){return 0},coerce:function(e,t){return Math.max(0,Math.floor(t))},changed:t}),e.isHorizontal=function(e){return"left-to-right"===e||"right-to-left"===e},e.clampSpacing=function(e){return Math.max(0,Math.floor(e))}}(z||(z={}));var H,U=function(e){function t(t){void 0===t&&(t={});var n=e.call(this)||this;return n.addClass("lm-Panel"),n.addClass("p-Panel"),n.layout=H.createLayout(t),n}return T(t,e),Object.defineProperty(t.prototype,"widgets",{get:function(){return this.layout.widgets},enumerable:!0,configurable:!0}),t.prototype.addWidget=function(e){this.layout.addWidget(e)},t.prototype.insertWidget=function(e,t){this.layout.insertWidget(e,t)},t}(L);!function(e){e.createLayout=function(e){return e.layout||new F}}(H||(H={}));var V,Y=function(e){function t(t){void 0===t&&(t={});var n=e.call(this,{layout:V.createLayout(t)})||this;return n.addClass("lm-BoxPanel"),n.addClass("p-BoxPanel"),n}return T(t,e),Object.defineProperty(t.prototype,"direction",{get:function(){return this.layout.direction},set:function(e){this.layout.direction=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"alignment",{get:function(){return this.layout.alignment},set:function(e){this.layout.alignment=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"spacing",{get:function(){return this.layout.spacing},set:function(e){this.layout.spacing=e},enumerable:!0,configurable:!0}),t.prototype.onChildAdded=function(e){e.child.addClass("lm-BoxPanel-child"),e.child.addClass("p-BoxPanel-child")},t.prototype.onChildRemoved=function(e){e.child.removeClass("lm-BoxPanel-child"),e.child.removeClass("p-BoxPanel-child")},t}(U);!function(e){e.getStretch=function(e){return B.getStretch(e)},e.setStretch=function(e,t){B.setStretch(e,t)},e.getSizeBasis=function(e){return B.getSizeBasis(e)},e.setSizeBasis=function(e,t){B.setSizeBasis(e,t)}}(Y||(Y={})),function(e){e.createLayout=function(e){return e.layout||new B(e)}}(V||(V={}));var W,G,X,q=function(e){function t(n){var r=e.call(this,{node:X.createNode()})||this;return r._activeIndex=-1,r._items=[],r._results=null,r.addClass("lm-CommandPalette"),r.addClass("p-CommandPalette"),r.setFlag(L.Flag.DisallowLayout),r.commands=n.commands,r.renderer=n.renderer||t.defaultRenderer,r.commands.commandChanged.connect(r._onGenericChange,r),r.commands.keyBindingChanged.connect(r._onGenericChange,r),r}return T(t,e),t.prototype.dispose=function(){this._items.length=0,this._results=null,e.prototype.dispose.call(this)},Object.defineProperty(t.prototype,"searchNode",{get:function(){return this.node.getElementsByClassName("lm-CommandPalette-search")[0]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"inputNode",{get:function(){return this.node.getElementsByClassName("lm-CommandPalette-input")[0]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"contentNode",{get:function(){return this.node.getElementsByClassName("lm-CommandPalette-content")[0]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"items",{get:function(){return this._items},enumerable:!0,configurable:!0}),t.prototype.addItem=function(e){var t=X.createItem(this.commands,e);return this._items.push(t),this.refresh(),t},t.prototype.addItems=function(e){var t=this,n=e.map((function(e){return X.createItem(t.commands,e)}));return n.forEach((function(e){return t._items.push(e)})),this.refresh(),n},t.prototype.removeItem=function(e){this.removeItemAt(this._items.indexOf(e))},t.prototype.removeItemAt=function(e){r.ArrayExt.removeAt(this._items,e)&&this.refresh()},t.prototype.clearItems=function(){0!==this._items.length&&(this._items.length=0,this.refresh())},t.prototype.refresh=function(){(this._results=null,""!==this.inputNode.value)?this.node.getElementsByClassName("lm-close-icon")[0].style.display="inherit":this.node.getElementsByClassName("lm-close-icon")[0].style.display="none";this.update()},t.prototype.handleEvent=function(e){switch(e.type){case"click":this._evtClick(e);break;case"keydown":this._evtKeyDown(e);break;case"input":this.refresh();break;case"focus":case"blur":this._toggleFocused()}},t.prototype.onBeforeAttach=function(e){this.node.addEventListener("click",this),this.node.addEventListener("keydown",this),this.node.addEventListener("input",this),this.node.addEventListener("focus",this,!0),this.node.addEventListener("blur",this,!0)},t.prototype.onAfterDetach=function(e){this.node.removeEventListener("click",this),this.node.removeEventListener("keydown",this),this.node.removeEventListener("input",this),this.node.removeEventListener("focus",this,!0),this.node.removeEventListener("blur",this,!0)},t.prototype.onActivateRequest=function(e){if(this.isAttached){var t=this.inputNode;t.focus(),t.select()}},t.prototype.onUpdateRequest=function(e){var t=this.inputNode.value,n=this.contentNode,o=this._results;if(o||(o=this._results=X.search(this._items,t),this._activeIndex=t?r.ArrayExt.findFirstIndex(o,X.canActivate):-1),t||0!==o.length)if(t&&0===o.length){var a=this.renderer.renderEmptyMessage({query:t});y.render(a,n)}else{for(var s=this.renderer,l=this._activeIndex,u=new Array(o.length),c=0,d=o.length;c<d;++c){var h=o[c];if("header"===h.type){var f=h.indices,p=h.category;u[c]=s.renderHeader({category:p,indices:f})}else{var g=h.item,v=(f=h.indices,c===l);u[c]=s.renderItem({item:g,indices:f,active:v})}}if(y.render(u,n),l<0||l>=o.length)n.scrollTop=0;else{var m=n.children[l];i.a.scrollIntoViewIfNeeded(n,m)}}else y.render(null,n)},t.prototype._evtClick=function(e){if(0===e.button){if(e.target.classList.contains("lm-close-icon"))return this.inputNode.value="",void this.refresh();var t=r.ArrayExt.findFirstIndex(this.contentNode.children,(function(t){return t.contains(e.target)}));-1!==t&&(e.preventDefault(),e.stopPropagation(),this._execute(t))}},t.prototype._evtKeyDown=function(e){if(!(e.altKey||e.ctrlKey||e.metaKey||e.shiftKey))switch(e.keyCode){case 13:e.preventDefault(),e.stopPropagation(),this._execute(this._activeIndex);break;case 38:e.preventDefault(),e.stopPropagation(),this._activatePreviousItem();break;case 40:e.preventDefault(),e.stopPropagation(),this._activateNextItem()}},t.prototype._activateNextItem=function(){if(this._results&&0!==this._results.length){var e=this._activeIndex,t=this._results.length,n=e<t-1?e+1:0,i=0===n?t-1:n-1;this._activeIndex=r.ArrayExt.findFirstIndex(this._results,X.canActivate,n,i),this.update()}},t.prototype._activatePreviousItem=function(){if(this._results&&0!==this._results.length){var e=this._activeIndex,t=this._results.length,n=e<=0?t-1:e-1,i=n===t-1?0:n+1;this._activeIndex=r.ArrayExt.findLastIndex(this._results,X.canActivate,n,i),this.update()}},t.prototype._execute=function(e){if(this._results){var t=this._results[e];if(t){if("header"===t.type){var n=this.inputNode;return n.value=t.category.toLowerCase()+" ",n.focus(),void this.refresh()}t.item.isEnabled&&(this.commands.execute(t.item.command,t.item.args),this.inputNode.value="",this.refresh())}}},t.prototype._toggleFocused=function(){var e=document.activeElement===this.inputNode;this.toggleClass("lm-mod-focused",e),this.toggleClass("p-mod-focused",e)},t.prototype._onGenericChange=function(){this.refresh()},t}(L);W=q||(q={}),G=function(){function e(){}return e.prototype.renderHeader=function(e){var t=this.formatHeader(e);return _.li({className:"lm-CommandPalette-header p-CommandPalette-header"},t)},e.prototype.renderItem=function(e){var t=this.createItemClass(e),n=this.createItemDataset(e);return e.item.isToggleable?_.li({className:t,dataset:n,role:"checkbox","aria-checked":""+e.item.isToggled},this.renderItemIcon(e),this.renderItemContent(e),this.renderItemShortcut(e)):_.li({className:t,dataset:n},this.renderItemIcon(e),this.renderItemContent(e),this.renderItemShortcut(e))},e.prototype.renderEmptyMessage=function(e){var t=this.formatEmptyMessage(e);return _.li({className:"lm-CommandPalette-emptyMessage p-CommandPalette-emptyMessage"},t)},e.prototype.renderItemIcon=function(e){var t=this.createIconClass(e);return"string"==typeof e.item.icon?_.div({className:t},e.item.iconLabel):_.div({className:t},e.item.icon,e.item.iconLabel)},e.prototype.renderItemContent=function(e){return _.div({className:"lm-CommandPalette-itemContent p-CommandPalette-itemContent"},this.renderItemLabel(e),this.renderItemCaption(e))},e.prototype.renderItemLabel=function(e){var t=this.formatItemLabel(e);return _.div({className:"lm-CommandPalette-itemLabel p-CommandPalette-itemLabel"},t)},e.prototype.renderItemCaption=function(e){var t=this.formatItemCaption(e);return _.div({className:"lm-CommandPalette-itemCaption p-CommandPalette-itemCaption"},t)},e.prototype.renderItemShortcut=function(e){var t=this.formatItemShortcut(e);return _.div({className:"lm-CommandPalette-itemShortcut p-CommandPalette-itemShortcut"},t)},e.prototype.createItemClass=function(e){var t="lm-CommandPalette-item";t+=" p-CommandPalette-item",e.item.isEnabled||(t+=" lm-mod-disabled",t+=" p-mod-disabled"),e.item.isToggled&&(t+=" lm-mod-toggled",t+=" p-mod-toggled"),e.active&&(t+=" lm-mod-active",t+=" p-mod-active");var n=e.item.className;return n&&(t+=" "+n),t},e.prototype.createItemDataset=function(e){return O(O({},e.item.dataset),{command:e.item.command})},e.prototype.createIconClass=function(e){var t="lm-CommandPalette-itemIcon";t+=" p-CommandPalette-itemIcon";var n=e.item.iconClass;return n?t+" "+n:t},e.prototype.formatHeader=function(e){return e.indices&&0!==e.indices.length?r.StringExt.highlight(e.category,e.indices,_.mark):e.category},e.prototype.formatEmptyMessage=function(e){return"No commands found that match '"+e.query+"'"},e.prototype.formatItemShortcut=function(e){var t=e.item.keyBinding;return t?t.keys.map(g.formatKeystroke).join(", "):null},e.prototype.formatItemLabel=function(e){return e.indices&&0!==e.indices.length?r.StringExt.highlight(e.item.label,e.indices,_.mark):e.item.label},e.prototype.formatItemCaption=function(e){return e.item.caption},e}(),W.Renderer=G,W.defaultRenderer=new G,function(e){function t(e,t){for(var n=e.category.toLowerCase(),i=n+" "+e.label.toLowerCase(),o=1/0,a=null,s=/\b\w/g;;){var l=s.exec(i);if(!l)break;var u=r.StringExt.matchSumOfDeltas(i,t,l.index);if(!u)break;u&&u.score<=o&&(o=u.score,a=u.indices)}if(!a||o===1/0)return null;for(var c=n.length+1,d=r.ArrayExt.lowerBound(a,c,(function(e,t){return e-t})),h=a.slice(0,d),f=a.slice(d),p=0,g=f.length;p<g;++p)f[p]-=c;return 0===h.length?{matchType:0,categoryIndices:null,labelIndices:f,score:o,item:e}:0===f.length?{matchType:1,categoryIndices:h,labelIndices:null,score:o,item:e}:{matchType:2,categoryIndices:h,labelIndices:f,score:o,item:e}}function n(e,t){var n=e.matchType-t.matchType;if(0!==n)return n;var r=e.score-t.score;if(0!==r)return r;var i=0,o=0;switch(e.matchType){case 0:i=e.labelIndices[0],o=t.labelIndices[0];break;case 1:case 2:i=e.categoryIndices[0],o=t.categoryIndices[0]}if(i!==o)return i-o;var a=e.item.category.localeCompare(t.item.category);if(0!==a)return a;var s=e.item.rank,l=t.item.rank;return s!==l?s<l?-1:1:e.item.label.localeCompare(t.item.label)}e.createNode=function(){var e=document.createElement("div"),t=document.createElement("div"),n=document.createElement("div"),r=document.createElement("input"),i=document.createElement("ul"),o=document.createElement("button");return t.className="lm-CommandPalette-search",n.className="lm-CommandPalette-wrapper",r.className="lm-CommandPalette-input",o.className="lm-close-icon",i.className="lm-CommandPalette-content",t.classList.add("p-CommandPalette-search"),n.classList.add("p-CommandPalette-wrapper"),r.classList.add("p-CommandPalette-input"),i.classList.add("p-CommandPalette-content"),r.spellcheck=!1,n.appendChild(r),n.appendChild(o),t.appendChild(n),e.appendChild(t),e.appendChild(i),e},e.createItem=function(e,t){return new i(e,t)},e.search=function(e,i){var o=function(e,n){r=n,n=r.replace(/\s+/g,"").toLowerCase();var r;for(var i=[],o=0,a=e.length;o<a;++o){var s=e[o];if(s.isVisible)if(n){var l=t(s,n);l&&(s.isEnabled||(l.score+=1e3),i.push(l))}else i.push({matchType:3,categoryIndices:null,labelIndices:null,score:0,item:s})}return i}(e,i);return o.sort(n),function(e){var t=new Array(e.length);r.ArrayExt.fill(t,!1);for(var n=[],i=0,o=e.length;i<o;++i)if(!t[i]){var a=e[i],s=a.item,l=a.categoryIndices,u=s.category;n.push({type:"header",category:u,indices:l});for(var c=i;c<o;++c)if(!t[c]){var d=e[c],h=d.item,f=d.labelIndices;h.category===u&&(n.push({type:"item",item:h,indices:f}),t[c]=!0)}}return n}(o)},e.canActivate=function(e){return"item"===e.type&&e.item.isEnabled};var i=function(){function e(e,t){this._commands=e,this.category=t.category.trim().replace(/\s+/g," "),this.command=t.command,this.args=t.args||l.JSONExt.emptyObject,this.rank=void 0!==t.rank?t.rank:1/0}return Object.defineProperty(e.prototype,"label",{get:function(){return this._commands.label(this.command,this.args)},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"icon",{get:function(){return this._commands.icon(this.command,this.args)},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"iconClass",{get:function(){return this._commands.iconClass(this.command,this.args)},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"iconLabel",{get:function(){return this._commands.iconLabel(this.command,this.args)},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"caption",{get:function(){return this._commands.caption(this.command,this.args)},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"className",{get:function(){return this._commands.className(this.command,this.args)},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"dataset",{get:function(){return this._commands.dataset(this.command,this.args)},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"isEnabled",{get:function(){return this._commands.isEnabled(this.command,this.args)},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"isToggled",{get:function(){return this._commands.isToggled(this.command,this.args)},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"isToggleable",{get:function(){return this._commands.isToggleable(this.command,this.args)},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"isVisible",{get:function(){return this._commands.isVisible(this.command,this.args)},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"keyBinding",{get:function(){var e=this.command,t=this.args;return r.ArrayExt.findLastValue(this._commands.keyBindings,(function(n){return n.command===e&&l.JSONExt.deepEqual(n.args,t)}))||null},enumerable:!0,configurable:!0}),e}()}(X||(X={}));var Z,Q=function(e){function t(n){var r=e.call(this,{node:Z.createNode()})||this;return r._childIndex=-1,r._activeIndex=-1,r._openTimerID=0,r._closeTimerID=0,r._items=[],r._childMenu=null,r._parentMenu=null,r._aboutToClose=new s.Signal(r),r._menuRequested=new s.Signal(r),r.addClass("lm-Menu"),r.addClass("p-Menu"),r.setFlag(L.Flag.DisallowLayout),r.commands=n.commands,r.renderer=n.renderer||t.defaultRenderer,r}return T(t,e),t.prototype.dispose=function(){this.close(),this._items.length=0,e.prototype.dispose.call(this)},Object.defineProperty(t.prototype,"aboutToClose",{get:function(){return this._aboutToClose},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"menuRequested",{get:function(){return this._menuRequested},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"parentMenu",{get:function(){return this._parentMenu},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"childMenu",{get:function(){return this._childMenu},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"rootMenu",{get:function(){for(var e=this;e._parentMenu;)e=e._parentMenu;return e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"leafMenu",{get:function(){for(var e=this;e._childMenu;)e=e._childMenu;return e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"contentNode",{get:function(){return this.node.getElementsByClassName("lm-Menu-content")[0]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"activeItem",{get:function(){return this._items[this._activeIndex]||null},set:function(e){this.activeIndex=e?this._items.indexOf(e):-1},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"activeIndex",{get:function(){return this._activeIndex},set:function(e){(e<0||e>=this._items.length)&&(e=-1),-1===e||Z.canActivate(this._items[e])||(e=-1),this._activeIndex!==e&&(this._activeIndex=e,this.update())},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"items",{get:function(){return this._items},enumerable:!0,configurable:!0}),t.prototype.activateNextItem=function(){var e=this._items.length,t=this._activeIndex,n=t<e-1?t+1:0,i=0===n?e-1:n-1;this.activeIndex=r.ArrayExt.findFirstIndex(this._items,Z.canActivate,n,i)},t.prototype.activatePreviousItem=function(){var e=this._items.length,t=this._activeIndex,n=t<=0?e-1:t-1,i=n===e-1?0:n+1;this.activeIndex=r.ArrayExt.findLastIndex(this._items,Z.canActivate,n,i)},t.prototype.triggerActiveItem=function(){if(this.isAttached){var e=this.activeItem;if(e)if(this._cancelOpenTimer(),this._cancelCloseTimer(),"submenu"!==e.type){this.rootMenu.close();var t=e.command,n=e.args;this.commands.isEnabled(t,n)?this.commands.execute(t,n):console.log("Command '"+t+"' is disabled.")}else this._openChildMenu(!0)}},t.prototype.addItem=function(e){return this.insertItem(this._items.length,e)},t.prototype.insertItem=function(e,t){this.isAttached&&this.close(),this.activeIndex=-1;var n=Math.max(0,Math.min(e,this._items.length)),i=Z.createItem(this,t);return r.ArrayExt.insert(this._items,n,i),this.update(),i},t.prototype.removeItem=function(e){this.removeItemAt(this._items.indexOf(e))},t.prototype.removeItemAt=function(e){this.isAttached&&this.close(),this.activeIndex=-1,r.ArrayExt.removeAt(this._items,e)&&this.update()},t.prototype.clearItems=function(){this.isAttached&&this.close(),this.activeIndex=-1,0!==this._items.length&&(this._items.length=0,this.update())},t.prototype.open=function(e,t,n){if(void 0===n&&(n={}),!this.isAttached){var r=n.forceX||!1,i=n.forceY||!1;Z.openRootMenu(this,e,t,r,i),this.activate()}},t.prototype.handleEvent=function(e){switch(e.type){case"keydown":this._evtKeyDown(e);break;case"mouseup":this._evtMouseUp(e);break;case"mousemove":this._evtMouseMove(e);break;case"mouseenter":this._evtMouseEnter(e);break;case"mouseleave":this._evtMouseLeave(e);break;case"mousedown":this._evtMouseDown(e);break;case"contextmenu":e.preventDefault(),e.stopPropagation()}},t.prototype.onBeforeAttach=function(e){this.node.addEventListener("keydown",this),this.node.addEventListener("mouseup",this),this.node.addEventListener("mousemove",this),this.node.addEventListener("mouseenter",this),this.node.addEventListener("mouseleave",this),this.node.addEventListener("contextmenu",this),document.addEventListener("mousedown",this,!0)},t.prototype.onAfterDetach=function(e){this.node.removeEventListener("keydown",this),this.node.removeEventListener("mouseup",this),this.node.removeEventListener("mousemove",this),this.node.removeEventListener("mouseenter",this),this.node.removeEventListener("mouseleave",this),this.node.removeEventListener("contextmenu",this),document.removeEventListener("mousedown",this,!0)},t.prototype.onActivateRequest=function(e){this.isAttached&&this.node.focus()},t.prototype.onUpdateRequest=function(e){for(var t=this._items,n=this.renderer,r=this._activeIndex,i=Z.computeCollapsed(t),o=new Array(t.length),a=0,s=t.length;a<s;++a){var l=t[a],u=a===r,c=i[a];o[a]=n.renderItem({item:l,active:u,collapsed:c})}y.render(o,this.contentNode)},t.prototype.onCloseRequest=function(t){this._cancelOpenTimer(),this._cancelCloseTimer(),this.activeIndex=-1;var n=this._childMenu;n&&(this._childIndex=-1,this._childMenu=null,n._parentMenu=null,n.close());var r=this._parentMenu;r&&(this._parentMenu=null,r._childIndex=-1,r._childMenu=null,r.activate()),this.isAttached&&this._aboutToClose.emit(void 0),e.prototype.onCloseRequest.call(this,t)},t.prototype._evtKeyDown=function(e){e.preventDefault(),e.stopPropagation();var t=e.keyCode;if(13!==t)if(27!==t)if(37!==t)if(38!==t)if(39!==t)if(40!==t){var n=c().keyForKeydownEvent(e);if(n){var r=this._activeIndex+1,i=Z.findMnemonic(this._items,n,r);-1===i.index||i.multiple?-1!==i.index?this.activeIndex=i.index:-1!==i.auto&&(this.activeIndex=i.auto):(this.activeIndex=i.index,this.triggerActiveItem())}}else this.activateNextItem();else{var o=this.activeItem;o&&"submenu"===o.type?this.triggerActiveItem():this.rootMenu._menuRequested.emit("next")}else this.activatePreviousItem();else this._parentMenu?this.close():this._menuRequested.emit("previous");else this.close();else this.triggerActiveItem()},t.prototype._evtMouseUp=function(e){0===e.button&&(e.preventDefault(),e.stopPropagation(),this.triggerActiveItem())},t.prototype._evtMouseMove=function(e){var t=r.ArrayExt.findFirstIndex(this.contentNode.children,(function(t){return i.a.hitTest(t,e.clientX,e.clientY)}));if(t!==this._activeIndex){if(this.activeIndex=t,(t=this.activeIndex)===this._childIndex)return this._cancelOpenTimer(),void this._cancelCloseTimer();-1!==this._childIndex&&this._startCloseTimer(),this._cancelOpenTimer();var n=this.activeItem;n&&"submenu"===n.type&&n.submenu&&this._startOpenTimer()}},t.prototype._evtMouseEnter=function(e){for(var t=this._parentMenu;t;t=t._parentMenu)t._cancelOpenTimer(),t._cancelCloseTimer(),t.activeIndex=t._childIndex},t.prototype._evtMouseLeave=function(e){if(this._cancelOpenTimer(),this._childMenu){var t=e.clientX,n=e.clientY;i.a.hitTest(this._childMenu.node,t,n)?this._cancelCloseTimer():(this.activeIndex=-1,this._startCloseTimer())}else this.activeIndex=-1},t.prototype._evtMouseDown=function(e){this._parentMenu||(Z.hitTestMenus(this,e.clientX,e.clientY)?(e.preventDefault(),e.stopPropagation()):this.close())},t.prototype._openChildMenu=function(e){void 0===e&&(e=!1);var t=this.activeItem;if(t&&"submenu"===t.type&&t.submenu){var n=t.submenu;if(n!==this._childMenu){this._closeChildMenu(),this._childMenu=n,this._childIndex=this._activeIndex,n._parentMenu=this,o.c.sendMessage(this,L.Msg.UpdateRequest);var r=this.contentNode.children[this._activeIndex];Z.openSubmenu(n,r),e&&(n.activeIndex=-1,n.activateNextItem()),n.activate()}}else this._closeChildMenu()},t.prototype._closeChildMenu=function(){this._childMenu&&this._childMenu.close()},t.prototype._startOpenTimer=function(){var e=this;0===this._openTimerID&&(this._openTimerID=window.setTimeout((function(){e._openTimerID=0,e._openChildMenu()}),Z.TIMER_DELAY))},t.prototype._startCloseTimer=function(){var e=this;0===this._closeTimerID&&(this._closeTimerID=window.setTimeout((function(){e._closeTimerID=0,e._closeChildMenu()}),Z.TIMER_DELAY))},t.prototype._cancelOpenTimer=function(){0!==this._openTimerID&&(clearTimeout(this._openTimerID),this._openTimerID=0)},t.prototype._cancelCloseTimer=function(){0!==this._closeTimerID&&(clearTimeout(this._closeTimerID),this._closeTimerID=0)},t}(L);!function(e){var t=function(){function e(){}return e.prototype.renderItem=function(e){var t=this.createItemClass(e),n=this.createItemDataset(e),r=this.createItemARIA(e);return _.li(O({className:t,dataset:n},r),this.renderIcon(e),this.renderLabel(e),this.renderShortcut(e),this.renderSubmenu(e))},e.prototype.renderIcon=function(e){var t=this.createIconClass(e);return"string"==typeof e.item.icon?_.div({className:t},e.item.iconLabel):_.div({className:t},e.item.icon,e.item.iconLabel)},e.prototype.renderLabel=function(e){var t=this.formatLabel(e);return _.div({className:"lm-Menu-itemLabel p-Menu-itemLabel"},t)},e.prototype.renderShortcut=function(e){var t=this.formatShortcut(e);return _.div({className:"lm-Menu-itemShortcut p-Menu-itemShortcut"},t)},e.prototype.renderSubmenu=function(e){return _.div({className:"lm-Menu-itemSubmenuIcon p-Menu-itemSubmenuIcon"})},e.prototype.createItemClass=function(e){var t="lm-Menu-item";t+=" p-Menu-item",e.item.isEnabled||(t+=" lm-mod-disabled",t+=" p-mod-disabled"),e.item.isToggled&&(t+=" lm-mod-toggled",t+=" p-mod-toggled"),e.item.isVisible||(t+=" lm-mod-hidden",t+=" p-mod-hidden"),e.active&&(t+=" lm-mod-active",t+=" p-mod-active"),e.collapsed&&(t+=" lm-mod-collapsed",t+=" p-mod-collapsed");var n=e.item.className;return n&&(t+=" "+n),t},e.prototype.createItemDataset=function(e){var t=e.item,n=t.type,r=t.command,i=t.dataset;return O(O({},i),"command"===n?{type:n,command:r}:{type:n})},e.prototype.createIconClass=function(e){var t="lm-Menu-itemIcon";t+=" p-Menu-itemIcon";var n=e.item.iconClass;return n?t+" "+n:t},e.prototype.createItemARIA=function(e){var t={};switch(e.item.type){case"separator":t.role="presentation";break;case"submenu":t["aria-haspopup"]="true";break;default:t.role="menuitem"}return t},e.prototype.formatLabel=function(e){var t=e.item,n=t.label,r=t.mnemonic;if(r<0||r>=n.length)return n;var i=n.slice(0,r),o=n.slice(r+1),a=n[r];return[i,_.span({className:"lm-Menu-itemMnemonic p-Menu-itemMnemonic"},a),o]},e.prototype.formatShortcut=function(e){var t=e.item.keyBinding;return t?t.keys.map(g.formatKeystroke).join(", "):null},e}();e.Renderer=t,e.defaultRenderer=new t}(Q||(Q={})),function(e){function t(e){return"separator"!==e.type&&e.isEnabled&&e.isVisible}e.TIMER_DELAY=300,e.SUBMENU_OVERLAP=3,e.createNode=function(){var e=document.createElement("div"),t=document.createElement("ul");return t.className="lm-Menu-content",t.classList.add("p-Menu-content"),e.appendChild(t),t.setAttribute("role","menu"),e.tabIndex=-1,e},e.canActivate=t,e.createItem=function(e,t){return new n(e.commands,t)},e.hitTestMenus=function(e,t,n){for(var r=e;r;r=r.childMenu)if(i.a.hitTest(r.node,t,n))return!0;return!1},e.computeCollapsed=function(e){var t=new Array(e.length);r.ArrayExt.fill(t,!1);for(var n=0,i=e.length;n<i;++n){if((s=e[n]).isVisible){if("separator"!==s.type)break;t[n]=!0}}for(var o=i-1;o>=0;--o){if((s=e[o]).isVisible){if("separator"!==s.type)break;t[o]=!0}}for(var a=!1;++n<o;){var s;(s=e[n]).isVisible&&("separator"!==s.type?a=!1:a?t[n]=!0:a=!0)}return t},e.openRootMenu=function(e,t,n,r,i){o.c.sendMessage(e,L.Msg.UpdateRequest);var a=window.pageXOffset,s=window.pageYOffset,l=document.documentElement.clientWidth,u=document.documentElement.clientHeight,c=u-(i?n:0),d=e.node,h=d.style;h.top="",h.left="",h.width="",h.height="",h.visibility="hidden",h.maxHeight=c+"px",L.attach(e,document.body);var f=d.getBoundingClientRect(),p=f.width,g=f.height;!r&&t+p>a+l&&(t=a+l-p),!i&&n+g>s+u&&(n>s+u?n=s+u-g:n-=g),h.top=Math.max(0,n)+"px",h.left=Math.max(0,t)+"px",h.visibility=""},e.openSubmenu=function(t,n){o.c.sendMessage(t,L.Msg.UpdateRequest);var r=window.pageXOffset,a=window.pageYOffset,s=document.documentElement.clientWidth,l=document.documentElement.clientHeight,u=l,c=t.node,d=c.style;d.top="",d.left="",d.width="",d.height="",d.visibility="hidden",d.maxHeight=u+"px",L.attach(t,document.body);var h=c.getBoundingClientRect(),f=h.width,p=h.height,g=i.a.boxSizing(t.node),v=n.getBoundingClientRect(),m=v.right-e.SUBMENU_OVERLAP;m+f>r+s&&(m=v.left+e.SUBMENU_OVERLAP-f);var y=v.top-g.borderTop-g.paddingTop;y+p>a+l&&(y=v.bottom+g.borderBottom+g.paddingBottom-p),d.top=Math.max(0,y)+"px",d.left=Math.max(0,m)+"px",d.visibility=""},e.findMnemonic=function(e,n,r){for(var i=-1,o=-1,a=!1,s=n.toUpperCase(),l=0,u=e.length;l<u;++l){var c=(l+r)%u,d=e[c];if(t(d)){var h=d.label;if(0!==h.length){var f=d.mnemonic;f>=0&&f<h.length?h[f].toUpperCase()===s&&(-1===i?i=c:a=!0):-1===o&&h[0].toUpperCase()===s&&(o=c)}}}return{index:i,multiple:a,auto:o}};var n=function(){function e(e,t){this._commands=e,this.type=t.type||"command",this.command=t.command||"",this.args=t.args||l.JSONExt.emptyObject,this.submenu=t.submenu||null}return Object.defineProperty(e.prototype,"label",{get:function(){return"command"===this.type?this._commands.label(this.command,this.args):"submenu"===this.type&&this.submenu?this.submenu.title.label:""},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"mnemonic",{get:function(){return"command"===this.type?this._commands.mnemonic(this.command,this.args):"submenu"===this.type&&this.submenu?this.submenu.title.mnemonic:-1},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"icon",{get:function(){return"command"===this.type?this._commands.icon(this.command,this.args):"submenu"===this.type&&this.submenu?this.submenu.title.icon:this.iconClass},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"iconClass",{get:function(){return"command"===this.type?this._commands.iconClass(this.command,this.args):"submenu"===this.type&&this.submenu?this.submenu.title.iconClass:""},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"iconLabel",{get:function(){return"command"===this.type?this._commands.iconLabel(this.command,this.args):"submenu"===this.type&&this.submenu?this.submenu.title.iconLabel:""},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"caption",{get:function(){return"command"===this.type?this._commands.caption(this.command,this.args):"submenu"===this.type&&this.submenu?this.submenu.title.caption:""},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"className",{get:function(){return"command"===this.type?this._commands.className(this.command,this.args):"submenu"===this.type&&this.submenu?this.submenu.title.className:""},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"dataset",{get:function(){return"command"===this.type?this._commands.dataset(this.command,this.args):"submenu"===this.type&&this.submenu?this.submenu.title.dataset:{}},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"isEnabled",{get:function(){return"command"===this.type?this._commands.isEnabled(this.command,this.args):"submenu"!==this.type||null!==this.submenu},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"isToggled",{get:function(){return"command"===this.type&&this._commands.isToggled(this.command,this.args)},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"isVisible",{get:function(){return"command"===this.type?this._commands.isVisible(this.command,this.args):"submenu"!==this.type||null!==this.submenu},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"keyBinding",{get:function(){if("command"===this.type){var e=this.command,t=this.args;return r.ArrayExt.findLastValue(this._commands.keyBindings,(function(n){return n.command===e&&l.JSONExt.deepEqual(n.args,t)}))||null}return null},enumerable:!0,configurable:!0}),e}()}(Z||(Z={}));var K,J=function(){function e(e){this._idTick=0,this._items=[],this.menu=new Q(e)}return e.prototype.addItem=function(e){var t=this,n=K.createItem(e,this._idTick++);return this._items.push(n),new u.DisposableDelegate((function(){r.ArrayExt.removeFirstOf(t._items,n)}))},e.prototype.open=function(e){var t=this;if(this.menu.clearItems(),0===this._items.length)return!1;var n=K.matchItems(this._items,e);return!(!n||0===n.length)&&(Object(r.each)(n,(function(e){t.menu.addItem(e)})),this.menu.open(e.clientX,e.clientY),!0)},e}();!function(e){function t(e,t){var n=i.c.calculateSpecificity(e.selector),r=i.c.calculateSpecificity(t.selector);if(n!==r)return r-n;var o=e.rank,a=t.rank;return o!==a?o<a?-1:1:e.id-t.id}e.createItem=function(e,t){var n=function(e){if(-1!==e.indexOf(","))throw new Error("Selector cannot contain commas: "+e);if(!i.c.isValid(e))throw new Error("Invalid selector: "+e);return e}(e.selector),r=void 0!==e.rank?e.rank:1/0;return O(O({},e),{selector:n,rank:r,id:t})},e.matchItems=function(e,n){var r=n.target;if(!r)return null;var o=n.currentTarget;if(!o)return null;if(!(o.contains(r)||(r=document.elementFromPoint(n.clientX,n.clientY))&&o.contains(r)))return null;for(var a=[],s=e.slice();null!==r;){for(var l=[],u=0,c=s.length;u<c;++u){var d=s[u];d&&(i.c.matches(r,d.selector)&&(l.push(d),s[u]=null))}if(0!==l.length&&(l.sort(t),a.push.apply(a,l)),r===o)break;r=r.parentElement}return a}}(K||(K={}));var $,ee=function(e){function t(t){var n=e.call(this)||this;return n._spacing=4,n._dirty=!1,n._root=null,n._box=null,n._items=new Map,n.renderer=t.renderer,void 0!==t.spacing&&(n._spacing=$.clampSpacing(t.spacing)),n}return T(t,e),t.prototype.dispose=function(){var t=this.iter();this._items.forEach((function(e){e.dispose()})),this._box=null,this._root=null,this._items.clear(),Object(r.each)(t,(function(e){e.dispose()})),e.prototype.dispose.call(this)},Object.defineProperty(t.prototype,"spacing",{get:function(){return this._spacing},set:function(e){e=$.clampSpacing(e),this._spacing!==e&&(this._spacing=e,this.parent&&this.parent.fit())},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"isEmpty",{get:function(){return null===this._root},enumerable:!0,configurable:!0}),t.prototype.iter=function(){return this._root?this._root.iterAllWidgets():Object(r.empty)()},t.prototype.widgets=function(){return this._root?this._root.iterUserWidgets():Object(r.empty)()},t.prototype.selectedWidgets=function(){return this._root?this._root.iterSelectedWidgets():Object(r.empty)()},t.prototype.tabBars=function(){return this._root?this._root.iterTabBars():Object(r.empty)()},t.prototype.handles=function(){return this._root?this._root.iterHandles():Object(r.empty)()},t.prototype.moveHandle=function(e,t,n){var r=e.classList.contains("lm-mod-hidden");if(r=r||e.classList.contains("p-mod-hidden"),this._root&&!r){var i,o=this._root.findSplitNode(e);if(o)0!==(i="horizontal"===o.node.orientation?t-e.offsetLeft:n-e.offsetTop)&&(o.node.holdSizes(),w.adjust(o.node.sizers,o.index,i),this.parent&&this.parent.update())}},t.prototype.saveLayout=function(){return this._root?(this._root.holdAllSizes(),{main:this._root.createConfig()}):{main:null}},t.prototype.restoreLayout=function(e){var t,n=this,i=new Set;t=e.main?$.normalizeAreaConfig(e.main,i):null;var o=this.widgets(),a=this.tabBars(),s=this.handles();this._root=null,Object(r.each)(o,(function(e){i.has(e)||(e.parent=null)})),Object(r.each)(a,(function(e){e.dispose()})),Object(r.each)(s,(function(e){e.parentNode&&e.parentNode.removeChild(e)})),i.forEach((function(e){e.parent=n.parent})),this._root=t?$.realizeAreaConfig(t,{createTabBar:function(){return n._createTabBar()},createHandle:function(){return n._createHandle()}}):null,this.parent&&(i.forEach((function(e){n.attachWidget(e)})),this.parent.fit())},t.prototype.addWidget=function(e,t){void 0===t&&(t={});var n=t.ref||null,r=t.mode||"tab-after",i=null;if(this._root&&n&&(i=this._root.findTabNode(n)),n&&!i)throw new Error("Reference widget is not in the layout.");switch(e.parent=this.parent,r){case"tab-after":this._insertTab(e,n,i,!0);break;case"tab-before":this._insertTab(e,n,i,!1);break;case"split-top":this._insertSplit(e,n,i,"vertical",!1);break;case"split-left":this._insertSplit(e,n,i,"horizontal",!1);break;case"split-right":this._insertSplit(e,n,i,"horizontal",!0);break;case"split-bottom":this._insertSplit(e,n,i,"vertical",!0)}this.parent&&(this.attachWidget(e),this.parent.fit())},t.prototype.removeWidget=function(e){this._removeWidget(e),this.parent&&(this.detachWidget(e),this.parent.fit())},t.prototype.hitTestTabAreas=function(e,t){if(!this._root||!this.parent||!this.parent.isVisible)return null;this._box||(this._box=i.a.boxSizing(this.parent.node));var n=this.parent.node.getBoundingClientRect(),r=e-n.left-this._box.borderLeft,o=t-n.top-this._box.borderTop,a=this._root.hitTestTabNodes(r,o);if(!a)return null;var s=a.tabBar,l=a.top,u=a.left,c=a.width,d=a.height,h=this._box.borderLeft+this._box.borderRight,f=this._box.borderTop+this._box.borderBottom;return{tabBar:s,x:r,y:o,top:l,left:u,right:n.width-h-(u+c),bottom:n.height-f-(l+d),width:c,height:d}},t.prototype.init=function(){var t=this;e.prototype.init.call(this),Object(r.each)(this,(function(e){t.attachWidget(e)})),Object(r.each)(this.handles(),(function(e){t.parent.node.appendChild(e)})),this.parent.fit()},t.prototype.attachWidget=function(e){this.parent.node!==e.node.parentNode&&(this._items.set(e,new j(e)),this.parent.isAttached&&o.c.sendMessage(e,L.Msg.BeforeAttach),this.parent.node.appendChild(e.node),this.parent.isAttached&&o.c.sendMessage(e,L.Msg.AfterAttach))},t.prototype.detachWidget=function(e){if(this.parent.node===e.node.parentNode){this.parent.isAttached&&o.c.sendMessage(e,L.Msg.BeforeDetach),this.parent.node.removeChild(e.node),this.parent.isAttached&&o.c.sendMessage(e,L.Msg.AfterDetach);var t=this._items.get(e);t&&(this._items.delete(e),t.dispose())}},t.prototype.onBeforeShow=function(t){e.prototype.onBeforeShow.call(this,t),this.parent.update()},t.prototype.onBeforeAttach=function(t){e.prototype.onBeforeAttach.call(this,t),this.parent.fit()},t.prototype.onChildShown=function(e){this.parent.fit()},t.prototype.onChildHidden=function(e){this.parent.fit()},t.prototype.onResize=function(e){this.parent.isVisible&&this._update(e.width,e.height)},t.prototype.onUpdateRequest=function(e){this.parent.isVisible&&this._update(-1,-1)},t.prototype.onFitRequest=function(e){this.parent.isAttached&&this._fit()},t.prototype._removeWidget=function(e){if(this._root){var t=this._root.findTabNode(e);if(t)if(t.tabBar.titles.length>1)t.tabBar.removeTab(e.title);else if(t.tabBar.dispose(),this._root!==t){this._root.holdAllSizes();var n=t.parent;t.parent=null;var i=r.ArrayExt.removeFirstOf(n.children,t),o=r.ArrayExt.removeAt(n.handles,i);if(r.ArrayExt.removeAt(n.sizers,i),o.parentNode&&o.parentNode.removeChild(o),n.children.length>1)n.syncHandles();else{var a=n.parent;n.parent=null;var s=n.children[0],l=n.handles[0];if(n.children.length=0,n.handles.length=0,n.sizers.length=0,l.parentNode&&l.parentNode.removeChild(l),this._root===n)return s.parent=null,void(this._root=s);var u=a,c=u.children.indexOf(n);if(s instanceof $.TabLayoutNode)return s.parent=u,void(u.children[c]=s);var d=r.ArrayExt.removeAt(u.handles,c);r.ArrayExt.removeAt(u.children,c),r.ArrayExt.removeAt(u.sizers,c),d.parentNode&&d.parentNode.removeChild(d);for(var h=0,f=s.children.length;h<f;++h){var p=s.children[h],g=s.handles[h],v=s.sizers[h];r.ArrayExt.insert(u.children,c+h,p),r.ArrayExt.insert(u.handles,c+h,g),r.ArrayExt.insert(u.sizers,c+h,v),p.parent=u}s.children.length=0,s.handles.length=0,s.sizers.length=0,s.parent=null,u.syncHandles()}}else this._root=null}},t.prototype._insertTab=function(e,t,n,r){if(e!==t){if(!this._root){var i=new $.TabLayoutNode(this._createTabBar());return i.tabBar.addTab(e.title),void(this._root=i)}var o;n||(n=this._root.findFirstTabNode()),-1===n.tabBar.titles.indexOf(e.title)&&(this._removeWidget(e),e.hide()),o=t?n.tabBar.titles.indexOf(t.title):n.tabBar.currentIndex,n.tabBar.insertTab(o+(r?1:0),e.title)}},t.prototype._insertSplit=function(e,t,n,i,o){if(e!==t||!n||1!==n.tabBar.titles.length){this._removeWidget(e);var a=new $.TabLayoutNode(this._createTabBar());if(a.tabBar.addTab(e.title),this._root){if(!n||!n.parent){var s=this._splitRoot(i),l=o?s.children.length:0;s.normalizeSizes();var u=$.createSizer(n?1:$.GOLDEN_RATIO);return r.ArrayExt.insert(s.children,l,a),r.ArrayExt.insert(s.sizers,l,u),r.ArrayExt.insert(s.handles,l,this._createHandle()),a.parent=s,s.normalizeSizes(),void s.syncHandles()}var c=n.parent;if(c.orientation===i){var d=c.children.indexOf(n);c.normalizeSizes();var h=c.sizers[d].sizeHint/=2,f=d+(o?1:0);return r.ArrayExt.insert(c.children,f,a),r.ArrayExt.insert(c.sizers,f,$.createSizer(h)),r.ArrayExt.insert(c.handles,f,this._createHandle()),a.parent=c,void c.syncHandles()}var p=r.ArrayExt.removeFirstOf(c.children,n),g=new $.SplitLayoutNode(i);g.normalized=!0,g.children.push(n),g.sizers.push($.createSizer(.5)),g.handles.push(this._createHandle()),n.parent=g;var v=o?1:0;r.ArrayExt.insert(g.children,v,a),r.ArrayExt.insert(g.sizers,v,$.createSizer(.5)),r.ArrayExt.insert(g.handles,v,this._createHandle()),a.parent=g,g.syncHandles(),r.ArrayExt.insert(c.children,p,g),g.parent=c}else this._root=a}},t.prototype._splitRoot=function(e){var t=this._root;if(t instanceof $.SplitLayoutNode&&t.orientation===e)return t;var n=this._root=new $.SplitLayoutNode(e);return t&&(n.children.push(t),n.sizers.push($.createSizer(0)),n.handles.push(this._createHandle()),t.parent=n),n},t.prototype._fit=function(){var e=0,t=0;if(this._root){var n=this._root.fit(this._spacing,this._items);e=n.minWidth,t=n.minHeight}var r=this._box=i.a.boxSizing(this.parent.node);e+=r.horizontalSum,t+=r.verticalSum;var a=this.parent.node.style;a.minWidth=e+"px",a.minHeight=t+"px",this._dirty=!0,this.parent.parent&&o.c.sendMessage(this.parent.parent,L.Msg.FitRequest),this._dirty&&o.c.sendMessage(this.parent,L.Msg.UpdateRequest)},t.prototype._update=function(e,t){if(this._dirty=!1,this._root){e<0&&(e=this.parent.node.offsetWidth),t<0&&(t=this.parent.node.offsetHeight),this._box||(this._box=i.a.boxSizing(this.parent.node));var n=this._box.paddingTop,r=this._box.paddingLeft,o=e-this._box.horizontalSum,a=t-this._box.verticalSum;this._root.update(n,r,o,a,this._spacing,this._items)}},t.prototype._createTabBar=function(){var e=this.renderer.createTabBar();return e.orientation="horizontal",this.parent&&(e.parent=this.parent,this.attachWidget(e)),e},t.prototype._createHandle=function(){var e=this.renderer.createHandle(),t=e.style;return t.position="absolute",t.top="0",t.left="0",t.width="0",t.height="0",this.parent&&this.parent.node.appendChild(e),e},t}(P);!function(e){function t(e){var t=new M;return t.sizeHint=e,t.size=e,t}function n(e,t){return"tab-area"===e.type?function(e,t){if(0===e.widgets.length)return null;var n=[];if(Object(r.each)(e.widgets,(function(e){t.has(e)||(t.add(e),n.push(e))})),0===n.length)return null;var i=e.currentIndex;-1!==i&&(i<0||i>=n.length)&&(i=0);return{type:"tab-area",widgets:n,currentIndex:i}}(e,t):function(e,t){for(var r=e.orientation,i=[],o=[],a=0,s=e.children.length;a<s;++a){var l=n(e.children[a],t);l&&("tab-area"===l.type||l.orientation!==r?(i.push(l),o.push(Math.abs(e.sizes[a]||0))):(i.push.apply(i,l.children),o.push.apply(o,l.sizes)))}if(0===i.length)return null;if(1===i.length)return i[0];return{type:"split-area",orientation:r,children:i,sizes:o}}(e,t)}function i(e,n){return"tab-area"===e.type?function(e,t){var n=t.createTabBar();return Object(r.each)(e.widgets,(function(e){e.hide(),n.addTab(e.title)})),n.currentIndex=e.currentIndex,new o(n)}(e,n):function(e,n){var o=new a(e.orientation);return Object(r.each)(e.children,(function(r,a){var s=i(r,n),l=t(e.sizes[a]),u=n.createHandle();o.children.push(s),o.handles.push(u),o.sizers.push(l),s.parent=o})),o.syncHandles(),o.normalizeSizes(),o}(e,n)}e.GOLDEN_RATIO=.618,e.clampSpacing=function(e){return Math.max(0,Math.floor(e))},e.createSizer=t,e.normalizeAreaConfig=n,e.realizeAreaConfig=i;var o=function(){function e(e){this.parent=null,this._top=0,this._left=0,this._width=0,this._height=0;var t=new M,n=new M;t.stretch=0,n.stretch=1,this.tabBar=e,this.sizers=[t,n]}return Object.defineProperty(e.prototype,"top",{get:function(){return this._top},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"left",{get:function(){return this._left},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"width",{get:function(){return this._width},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"height",{get:function(){return this._height},enumerable:!0,configurable:!0}),e.prototype.iterAllWidgets=function(){return Object(r.chain)(Object(r.once)(this.tabBar),this.iterUserWidgets())},e.prototype.iterUserWidgets=function(){return Object(r.map)(this.tabBar.titles,(function(e){return e.owner}))},e.prototype.iterSelectedWidgets=function(){var e=this.tabBar.currentTitle;return e?Object(r.once)(e.owner):Object(r.empty)()},e.prototype.iterTabBars=function(){return Object(r.once)(this.tabBar)},e.prototype.iterHandles=function(){return Object(r.empty)()},e.prototype.findTabNode=function(e){return-1!==this.tabBar.titles.indexOf(e.title)?this:null},e.prototype.findSplitNode=function(e){return null},e.prototype.findFirstTabNode=function(){return this},e.prototype.hitTestTabNodes=function(e,t){return e<this._left||e>=this._left+this._width||t<this._top||t>=this._top+this._height?null:this},e.prototype.createConfig=function(){return{type:"tab-area",widgets:this.tabBar.titles.map((function(e){return e.owner})),currentIndex:this.tabBar.currentIndex}},e.prototype.holdAllSizes=function(){},e.prototype.fit=function(e,t){var n=0,r=0,i=t.get(this.tabBar),o=this.tabBar.currentTitle,a=o?t.get(o.owner):void 0,s=this.sizers,l=s[0],u=s[1];return i&&i.fit(),a&&a.fit(),i&&!i.isHidden?(n=Math.max(n,i.minWidth),r+=i.minHeight,l.minSize=i.minHeight,l.maxSize=i.maxHeight):(l.minSize=0,l.maxSize=0),a&&!a.isHidden?(n=Math.max(n,a.minWidth),r+=a.minHeight,u.minSize=a.minHeight,u.maxSize=1/0):(u.minSize=0,u.maxSize=1/0),{minWidth:n,minHeight:r,maxWidth:1/0,maxHeight:1/0}},e.prototype.update=function(e,t,n,r,i,o){this._top=t,this._left=e,this._width=n,this._height=r;var a=o.get(this.tabBar),s=this.tabBar.currentTitle,l=s?o.get(s.owner):void 0;if(w.calc(this.sizers,r),a&&!a.isHidden){var u=this.sizers[0].size;a.update(e,t,n,u),t+=u}if(l&&!l.isHidden){u=this.sizers[1].size;l.update(e,t,n,u)}},e}();e.TabLayoutNode=o;var a=function(){function e(e){this.parent=null,this.normalized=!1,this.children=[],this.sizers=[],this.handles=[],this.orientation=e}return e.prototype.iterAllWidgets=function(){var e=Object(r.map)(this.children,(function(e){return e.iterAllWidgets()}));return new r.ChainIterator(e)},e.prototype.iterUserWidgets=function(){var e=Object(r.map)(this.children,(function(e){return e.iterUserWidgets()}));return new r.ChainIterator(e)},e.prototype.iterSelectedWidgets=function(){var e=Object(r.map)(this.children,(function(e){return e.iterSelectedWidgets()}));return new r.ChainIterator(e)},e.prototype.iterTabBars=function(){var e=Object(r.map)(this.children,(function(e){return e.iterTabBars()}));return new r.ChainIterator(e)},e.prototype.iterHandles=function(){var e=Object(r.map)(this.children,(function(e){return e.iterHandles()}));return Object(r.chain)(this.handles,new r.ChainIterator(e))},e.prototype.findTabNode=function(e){for(var t=0,n=this.children.length;t<n;++t){var r=this.children[t].findTabNode(e);if(r)return r}return null},e.prototype.findSplitNode=function(e){var t=this.handles.indexOf(e);if(-1!==t)return{index:t,node:this};for(var n=0,r=this.children.length;n<r;++n){var i=this.children[n].findSplitNode(e);if(i)return i}return null},e.prototype.findFirstTabNode=function(){return 0===this.children.length?null:this.children[0].findFirstTabNode()},e.prototype.hitTestTabNodes=function(e,t){for(var n=0,r=this.children.length;n<r;++n){var i=this.children[n].hitTestTabNodes(e,t);if(i)return i}return null},e.prototype.createConfig=function(){var e=this.orientation,t=this.createNormalizedSizes();return{type:"split-area",orientation:e,children:this.children.map((function(e){return e.createConfig()})),sizes:t}},e.prototype.syncHandles=function(){var e=this;Object(r.each)(this.handles,(function(t,n){t.setAttribute("data-orientation",e.orientation),n===e.handles.length-1?(t.classList.add("lm-mod-hidden"),t.classList.add("p-mod-hidden")):(t.classList.remove("lm-mod-hidden"),t.classList.remove("p-mod-hidden"))}))},e.prototype.holdSizes=function(){Object(r.each)(this.sizers,(function(e){e.sizeHint=e.size}))},e.prototype.holdAllSizes=function(){Object(r.each)(this.children,(function(e){return e.holdAllSizes()})),this.holdSizes()},e.prototype.normalizeSizes=function(){var e=this.sizers.length;if(0!==e){this.holdSizes();var t=Object(r.reduce)(this.sizers,(function(e,t){return e+t.sizeHint}),0);0===t?Object(r.each)(this.sizers,(function(t){t.size=t.sizeHint=1/e})):Object(r.each)(this.sizers,(function(e){e.size=e.sizeHint/=t})),this.normalized=!0}},e.prototype.createNormalizedSizes=function(){var e=this.sizers.length;if(0===e)return[];var t=this.sizers.map((function(e){return e.size})),n=Object(r.reduce)(t,(function(e,t){return e+t}),0);return 0===n?Object(r.each)(t,(function(n,r){t[r]=1/e})):Object(r.each)(t,(function(e,r){t[r]=e/n})),t},e.prototype.fit=function(e,t){for(var n="horizontal"===this.orientation,r=Math.max(0,this.children.length-1)*e,i=n?r:0,o=n?0:r,a=0,s=this.children.length;a<s;++a){var l=this.children[a].fit(e,t);n?(o=Math.max(o,l.minHeight),i+=l.minWidth,this.sizers[a].minSize=l.minWidth):(i=Math.max(i,l.minWidth),o+=l.minHeight,this.sizers[a].minSize=l.minHeight)}return{minWidth:i,minHeight:o,maxWidth:1/0,maxHeight:1/0}},e.prototype.update=function(e,t,n,i,o,a){var s="horizontal"===this.orientation,l=Math.max(0,this.children.length-1)*o,u=Math.max(0,(s?n:i)-l);this.normalized&&(Object(r.each)(this.sizers,(function(e){e.sizeHint*=u})),this.normalized=!1),w.calc(this.sizers,u);for(var c=0,d=this.children.length;c<d;++c){var h=this.children[c],f=this.sizers[c].size,p=this.handles[c].style;s?(h.update(e,t,f,i,o,a),e+=f,p.top=t+"px",p.left=e+"px",p.width=o+"px",p.height=i+"px",e+=o):(h.update(e,t,n,f,o,a),t+=f,p.top=t+"px",p.left=e+"px",p.width=n+"px",p.height=o+"px",t+=o)}},e}();e.SplitLayoutNode=a}($||($={}));var te,ne=function(e){function t(n){void 0===n&&(n={});var r=e.call(this,{node:te.createNode()})||this;return r._currentIndex=-1,r._titles=[],r._titlesEditable=!1,r._previousTitle=null,r._dragData=null,r._tabMoved=new s.Signal(r),r._currentChanged=new s.Signal(r),r._tabCloseRequested=new s.Signal(r),r._tabDetachRequested=new s.Signal(r),r._tabActivateRequested=new s.Signal(r),r.addClass("lm-TabBar"),r.addClass("p-TabBar"),r.setFlag(L.Flag.DisallowLayout),r.tabsMovable=n.tabsMovable||!1,r.titlesEditable=n.titlesEditable||!1,r.allowDeselect=n.allowDeselect||!1,r.insertBehavior=n.insertBehavior||"select-tab-if-needed",r.removeBehavior=n.removeBehavior||"select-tab-after",r.renderer=n.renderer||t.defaultRenderer,r._orientation=n.orientation||"horizontal",r.dataset.orientation=r._orientation,r}return T(t,e),t.prototype.dispose=function(){this._releaseMouse(),this._titles.length=0,this._previousTitle=null,e.prototype.dispose.call(this)},Object.defineProperty(t.prototype,"currentChanged",{get:function(){return this._currentChanged},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"tabMoved",{get:function(){return this._tabMoved},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"tabActivateRequested",{get:function(){return this._tabActivateRequested},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"tabCloseRequested",{get:function(){return this._tabCloseRequested},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"tabDetachRequested",{get:function(){return this._tabDetachRequested},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"titlesEditable",{get:function(){return this._titlesEditable},set:function(e){this._titlesEditable=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"currentTitle",{get:function(){return this._titles[this._currentIndex]||null},set:function(e){this.currentIndex=e?this._titles.indexOf(e):-1},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"currentIndex",{get:function(){return this._currentIndex},set:function(e){if((e<0||e>=this._titles.length)&&(e=-1),this._currentIndex!==e){var t=this._currentIndex,n=this._titles[t]||null,r=e,i=this._titles[r]||null;this._currentIndex=r,this._previousTitle=n,this.update(),this._currentChanged.emit({previousIndex:t,previousTitle:n,currentIndex:r,currentTitle:i})}},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"orientation",{get:function(){return this._orientation},set:function(e){this._orientation!==e&&(this._releaseMouse(),this._orientation=e,this.dataset.orientation=e)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"titles",{get:function(){return this._titles},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"contentNode",{get:function(){return this.node.getElementsByClassName("lm-TabBar-content")[0]},enumerable:!0,configurable:!0}),t.prototype.addTab=function(e){return this.insertTab(this._titles.length,e)},t.prototype.insertTab=function(e,t){this._releaseMouse();var n=te.asTitle(t),i=this._titles.indexOf(n),o=Math.max(0,Math.min(e,this._titles.length));return-1===i?(r.ArrayExt.insert(this._titles,o,n),n.changed.connect(this._onTitleChanged,this),this.update(),this._adjustCurrentForInsert(o,n),n):(o===this._titles.length&&o--,i===o||(r.ArrayExt.move(this._titles,i,o),this.update(),this._adjustCurrentForMove(i,o)),n)},t.prototype.removeTab=function(e){this.removeTabAt(this._titles.indexOf(e))},t.prototype.removeTabAt=function(e){this._releaseMouse();var t=r.ArrayExt.removeAt(this._titles,e);t&&(t.changed.disconnect(this._onTitleChanged,this),t===this._previousTitle&&(this._previousTitle=null),this.update(),this._adjustCurrentForRemove(e,t))},t.prototype.clearTabs=function(){if(0!==this._titles.length){this._releaseMouse();for(var e=0,t=this._titles;e<t.length;e++){t[e].changed.disconnect(this._onTitleChanged,this)}var n=this.currentIndex,r=this.currentTitle;this._currentIndex=-1,this._previousTitle=null,this._titles.length=0,this.update(),-1!==n&&this._currentChanged.emit({previousIndex:n,previousTitle:r,currentIndex:-1,currentTitle:null})}},t.prototype.releaseMouse=function(){this._releaseMouse()},t.prototype.handleEvent=function(e){switch(e.type){case"mousedown":this._evtMouseDown(e);break;case"mousemove":this._evtMouseMove(e);break;case"mouseup":this._evtMouseUp(e);break;case"dblclick":this._evtDblClick(e);break;case"keydown":this._evtKeyDown(e);break;case"contextmenu":e.preventDefault(),e.stopPropagation()}},t.prototype.onBeforeAttach=function(e){this.node.addEventListener("mousedown",this),this.node.addEventListener("dblclick",this)},t.prototype.onAfterDetach=function(e){this.node.removeEventListener("mousedown",this),this.node.removeEventListener("dblclick",this),this._releaseMouse()},t.prototype.onUpdateRequest=function(e){for(var t=this._titles,n=this.renderer,r=this.currentTitle,i=new Array(t.length),o=0,a=t.length;o<a;++o){var s=t[o],l=s===r,u=l?a:a-o-1;i[o]=n.renderTab({title:s,current:l,zIndex:u})}y.render(i,this.contentNode)},t.prototype._evtDblClick=function(e){if(this.titlesEditable){var t=this.contentNode.children,n=r.ArrayExt.findFirstIndex(t,(function(t){return i.a.hitTest(t,e.clientX,e.clientY)}));if(-1!==n){var o=this.titles[n],a=t[n].querySelector(".lm-TabBar-tabLabel");if(a&&a.contains(e.target)){var s=o.label||"",l=a.innerHTML;a.innerHTML="";var u=document.createElement("input");u.classList.add("lm-TabBar-tabInput"),u.value=s,a.appendChild(u);var c=function(){u.removeEventListener("blur",c),a.innerHTML=l};u.addEventListener("dblclick",(function(e){return e.stopPropagation()})),u.addEventListener("blur",c),u.addEventListener("keydown",(function(e){"Enter"===e.key?(""!==u.value&&(o.label=o.caption=u.value),c()):"Escape"===e.key&&c()})),u.select(),u.focus(),a.children.length>0&&a.children[0].focus()}}}},t.prototype._evtKeyDown=function(e){e.preventDefault(),e.stopPropagation(),27===e.keyCode&&this._releaseMouse()},t.prototype._evtMouseDown=function(e){if((0===e.button||1===e.button)&&!this._dragData){var t=this.contentNode.children,n=r.ArrayExt.findFirstIndex(t,(function(t){return i.a.hitTest(t,e.clientX,e.clientY)}));if(-1!==n&&(e.preventDefault(),e.stopPropagation(),this._dragData={tab:t[n],index:n,pressX:e.clientX,pressY:e.clientY,tabPos:-1,tabSize:-1,tabPressPos:-1,targetIndex:-1,tabLayout:null,contentRect:null,override:null,dragActive:!1,dragAborted:!1,detachRequested:!1},document.addEventListener("mouseup",this,!0),1!==e.button)){var o=t[n].querySelector(this.renderer.closeIconSelector);o&&o.contains(e.target)||(this.tabsMovable&&(document.addEventListener("mousemove",this,!0),document.addEventListener("keydown",this,!0),document.addEventListener("contextmenu",this,!0)),this.allowDeselect&&this.currentIndex===n?this.currentIndex=-1:this.currentIndex=n,-1!==this.currentIndex&&this._tabActivateRequested.emit({index:this.currentIndex,title:this.currentTitle}))}}},t.prototype._evtMouseMove=function(e){var t=this._dragData;if(t){e.preventDefault(),e.stopPropagation();var n=this.contentNode.children;if(t.dragActive||te.dragExceeded(t,e)){if(!t.dragActive){var r=t.tab.getBoundingClientRect();"horizontal"===this._orientation?(t.tabPos=t.tab.offsetLeft,t.tabSize=r.width,t.tabPressPos=t.pressX-r.left):(t.tabPos=t.tab.offsetTop,t.tabSize=r.height,t.tabPressPos=t.pressY-r.top),t.tabLayout=te.snapTabLayout(n,this._orientation),t.contentRect=this.contentNode.getBoundingClientRect(),t.override=x.overrideCursor("default"),t.tab.classList.add("lm-mod-dragging"),this.addClass("lm-mod-dragging"),t.tab.classList.add("p-mod-dragging"),this.addClass("p-mod-dragging"),t.dragActive=!0}if(!t.detachRequested&&te.detachExceeded(t,e)){t.detachRequested=!0;var i=t.index,o=e.clientX,a=e.clientY,s=n[i],l=this._titles[i];if(this._tabDetachRequested.emit({index:i,title:l,tab:s,clientX:o,clientY:a}),t.dragAborted)return}te.layoutTabs(n,t,e,this._orientation)}}},t.prototype._evtMouseUp=function(e){var t=this;if(0===e.button||1===e.button){var n=this._dragData;if(n){if(e.preventDefault(),e.stopPropagation(),document.removeEventListener("mousemove",this,!0),document.removeEventListener("mouseup",this,!0),document.removeEventListener("keydown",this,!0),document.removeEventListener("contextmenu",this,!0),!n.dragActive){this._dragData=null;var a=this.contentNode.children,s=r.ArrayExt.findFirstIndex(a,(function(t){return i.a.hitTest(t,e.clientX,e.clientY)}));if(s!==n.index)return;var l=this._titles[s];if(!l.closable)return;if(1===e.button)return void this._tabCloseRequested.emit({index:s,title:l});var u=a[s].querySelector(this.renderer.closeIconSelector);return u&&u.contains(e.target)?void this._tabCloseRequested.emit({index:s,title:l}):void 0}if(0===e.button){te.finalizeTabPosition(n,this._orientation),n.tab.classList.remove("lm-mod-dragging"),n.tab.classList.remove("p-mod-dragging");var c=te.parseTransitionDuration(n.tab);setTimeout((function(){if(!n.dragAborted){t._dragData=null,te.resetTabPositions(t.contentNode.children,t._orientation),n.override.dispose(),t.removeClass("lm-mod-dragging"),t.removeClass("p-mod-dragging");var e=n.index,i=n.targetIndex;-1!==i&&e!==i&&(r.ArrayExt.move(t._titles,e,i),t._adjustCurrentForMove(e,i),t._tabMoved.emit({fromIndex:e,toIndex:i,title:t._titles[i]}),o.c.sendMessage(t,L.Msg.UpdateRequest))}}),c)}}}},t.prototype._releaseMouse=function(){var e=this._dragData;e&&(this._dragData=null,document.removeEventListener("mousemove",this,!0),document.removeEventListener("mouseup",this,!0),document.removeEventListener("keydown",this,!0),document.removeEventListener("contextmenu",this,!0),e.dragAborted=!0,e.dragActive&&(te.resetTabPositions(this.contentNode.children,this._orientation),e.override.dispose(),e.tab.classList.remove("lm-mod-dragging"),this.removeClass("lm-mod-dragging"),e.tab.classList.remove("p-mod-dragging"),this.removeClass("p-mod-dragging")))},t.prototype._adjustCurrentForInsert=function(e,t){var n=this.currentTitle,r=this._currentIndex,i=this.insertBehavior;if("select-tab"===i||"select-tab-if-needed"===i&&-1===r)return this._currentIndex=e,this._previousTitle=n,void this._currentChanged.emit({previousIndex:r,previousTitle:n,currentIndex:e,currentTitle:t});r>=e&&this._currentIndex++},t.prototype._adjustCurrentForMove=function(e,t){this._currentIndex===e?this._currentIndex=t:this._currentIndex<e&&this._currentIndex>=t?this._currentIndex++:this._currentIndex>e&&this._currentIndex<=t&&this._currentIndex--},t.prototype._adjustCurrentForRemove=function(e,t){var n=this._currentIndex,r=this.removeBehavior;if(n===e){if(0===this._titles.length)return this._currentIndex=-1,void this._currentChanged.emit({previousIndex:e,previousTitle:t,currentIndex:-1,currentTitle:null});if("select-tab-after"===r)return this._currentIndex=Math.min(e,this._titles.length-1),void this._currentChanged.emit({previousIndex:e,previousTitle:t,currentIndex:this._currentIndex,currentTitle:this.currentTitle});if("select-tab-before"===r)return this._currentIndex=Math.max(0,e-1),void this._currentChanged.emit({previousIndex:e,previousTitle:t,currentIndex:this._currentIndex,currentTitle:this.currentTitle});if("select-previous-tab"===r)return this._previousTitle?(this._currentIndex=this._titles.indexOf(this._previousTitle),this._previousTitle=null):this._currentIndex=Math.min(e,this._titles.length-1),void this._currentChanged.emit({previousIndex:e,previousTitle:t,currentIndex:this._currentIndex,currentTitle:this.currentTitle});this._currentIndex=-1,this._currentChanged.emit({previousIndex:e,previousTitle:t,currentIndex:-1,currentTitle:null})}else n>e&&this._currentIndex--},t.prototype._onTitleChanged=function(e){this.update()},t}(L);!function(e){var t=function(){function e(){this.closeIconSelector=".lm-TabBar-tabCloseIcon",this._tabID=0,this._tabKeys=new WeakMap}return e.prototype.renderTab=function(e){var t=e.title.caption,n=this.createTabKey(e),r=this.createTabStyle(e),i=this.createTabClass(e),o=this.createTabDataset(e);return _.li({key:n,className:i,title:t,style:r,dataset:o},this.renderIcon(e),this.renderLabel(e),this.renderCloseIcon(e))},e.prototype.renderIcon=function(e){var t=e.title,n=this.createIconClass(e);return"string"==typeof t.icon?_.div({className:n},t.iconLabel):_.div({className:n},t.icon,t.iconLabel)},e.prototype.renderLabel=function(e){return _.div({className:"lm-TabBar-tabLabel p-TabBar-tabLabel"},e.title.label)},e.prototype.renderCloseIcon=function(e){return _.div({className:"lm-TabBar-tabCloseIcon p-TabBar-tabCloseIcon"})},e.prototype.createTabKey=function(e){var t=this._tabKeys.get(e.title);return void 0===t&&(t="tab-key-"+this._tabID++,this._tabKeys.set(e.title,t)),t},e.prototype.createTabStyle=function(e){return{zIndex:""+e.zIndex}},e.prototype.createTabClass=function(e){var t="lm-TabBar-tab";return t+=" p-TabBar-tab",e.title.className&&(t+=" "+e.title.className),e.title.closable&&(t+=" lm-mod-closable",t+=" p-mod-closable"),e.current&&(t+=" lm-mod-current",t+=" p-mod-current"),t},e.prototype.createTabDataset=function(e){return e.title.dataset},e.prototype.createIconClass=function(e){var t="lm-TabBar-tabIcon";t+=" p-TabBar-tabIcon";var n=e.title.iconClass;return n?t+" "+n:t},e}();e.Renderer=t,e.defaultRenderer=new t}(ne||(ne={})),function(e){e.DRAG_THRESHOLD=5,e.DETACH_THRESHOLD=20,e.createNode=function(){var e=document.createElement("div"),t=document.createElement("ul");return t.className="lm-TabBar-content",t.classList.add("p-TabBar-content"),e.appendChild(t),e},e.asTitle=function(e){return e instanceof k?e:new k(e)},e.parseTransitionDuration=function(e){var t=window.getComputedStyle(e);return 1e3*(parseFloat(t.transitionDuration)||0)},e.snapTabLayout=function(e,t){for(var n=new Array(e.length),r=0,i=e.length;r<i;++r){var o=e[r],a=window.getComputedStyle(o);n[r]="horizontal"===t?{pos:o.offsetLeft,size:o.offsetWidth,margin:parseFloat(a.marginLeft)||0}:{pos:o.offsetTop,size:o.offsetHeight,margin:parseFloat(a.marginTop)||0}}return n},e.dragExceeded=function(t,n){var r=Math.abs(n.clientX-t.pressX),i=Math.abs(n.clientY-t.pressY);return r>=e.DRAG_THRESHOLD||i>=e.DRAG_THRESHOLD},e.detachExceeded=function(t,n){var r=t.contentRect;return n.clientX<r.left-e.DETACH_THRESHOLD||n.clientX>=r.right+e.DETACH_THRESHOLD||n.clientY<r.top-e.DETACH_THRESHOLD||n.clientY>=r.bottom+e.DETACH_THRESHOLD},e.layoutTabs=function(e,t,n,r){var i,o,a,s;"horizontal"===r?(i=t.pressX,o=n.clientX-t.contentRect.left,a=n.clientX,s=t.contentRect.width):(i=t.pressY,o=n.clientY-t.contentRect.top,a=n.clientY,s=t.contentRect.height);for(var l=t.index,u=o-t.tabPressPos,c=u+t.tabSize,d=0,h=e.length;d<h;++d){var f=void 0,p=t.tabLayout[d],g=p.pos+(p.size>>1);if(d<t.index&&u<g)f=t.tabSize+t.tabLayout[d+1].margin+"px",l=Math.min(l,d);else if(d>t.index&&c>g)f=-t.tabSize-p.margin+"px",l=Math.max(l,d);else if(d===t.index){var v=a-i,m=s-(t.tabPos+t.tabSize);f=Math.max(-t.tabPos,Math.min(v,m))+"px"}else f="";"horizontal"===r?e[d].style.left=f:e[d].style.top=f}t.targetIndex=l},e.finalizeTabPosition=function(e,t){var n,r;if(n="horizontal"===t?e.contentRect.width:e.contentRect.height,e.targetIndex===e.index)r=0;else if(e.targetIndex>e.index){r=(i=e.tabLayout[e.targetIndex]).pos+i.size-e.tabSize-e.tabPos}else{var i;r=(i=e.tabLayout[e.targetIndex]).pos-e.tabPos}var o=n-(e.tabPos+e.tabSize),a=Math.max(-e.tabPos,Math.min(r,o));"horizontal"===t?e.tab.style.left=a+"px":e.tab.style.top=a+"px"},e.resetTabPositions=function(e,t){Object(r.each)(e,(function(e){"horizontal"===t?e.style.left="":e.style.top=""}))}}(te||(te={}));var re,ie=function(e){function t(n){void 0===n&&(n={});var r=e.call(this)||this;r._drag=null,r._tabsMovable=!0,r._tabsConstrained=!1,r._pressData=null,r._layoutModified=new s.Signal(r),r.addClass("lm-DockPanel"),r.addClass("p-DockPanel"),r._mode=n.mode||"multiple-document",r._renderer=n.renderer||t.defaultRenderer,r._edges=n.edges||re.DEFAULT_EDGES,void 0!==n.tabsMovable&&(r._tabsMovable=n.tabsMovable),void 0!==n.tabsConstrained&&(r._tabsConstrained=n.tabsConstrained),r.dataset.mode=r._mode;var i={createTabBar:function(){return r._createTabBar()},createHandle:function(){return r._createHandle()}};return r.layout=new ee({renderer:i,spacing:n.spacing}),r.overlay=n.overlay||new t.Overlay,r.node.appendChild(r.overlay.node),r}return T(t,e),t.prototype.dispose=function(){this._releaseMouse(),this.overlay.hide(0),this._drag&&this._drag.dispose(),e.prototype.dispose.call(this)},Object.defineProperty(t.prototype,"layoutModified",{get:function(){return this._layoutModified},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"renderer",{get:function(){return this.layout.renderer},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"spacing",{get:function(){return this.layout.spacing},set:function(e){this.layout.spacing=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"mode",{get:function(){return this._mode},set:function(e){if(this._mode!==e){this._mode=e,this.dataset.mode=e;var t=this.layout;switch(e){case"multiple-document":Object(r.each)(t.tabBars(),(function(e){e.show()}));break;case"single-document":t.restoreLayout(re.createSingleDocumentConfig(this));break;default:throw"unreachable"}o.c.postMessage(this,re.LayoutModified)}},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"tabsMovable",{get:function(){return this._tabsMovable},set:function(e){this._tabsMovable=e,Object(r.each)(this.tabBars(),(function(t){t.tabsMovable=e}))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"tabsConstrained",{get:function(){return this._tabsConstrained},set:function(e){this._tabsConstrained=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"isEmpty",{get:function(){return this.layout.isEmpty},enumerable:!0,configurable:!0}),t.prototype.widgets=function(){return this.layout.widgets()},t.prototype.selectedWidgets=function(){return this.layout.selectedWidgets()},t.prototype.tabBars=function(){return this.layout.tabBars()},t.prototype.handles=function(){return this.layout.handles()},t.prototype.selectWidget=function(e){var t=Object(r.find)(this.tabBars(),(function(t){return-1!==t.titles.indexOf(e.title)}));if(!t)throw new Error("Widget is not contained in the dock panel.");t.currentTitle=e.title},t.prototype.activateWidget=function(e){this.selectWidget(e),e.activate()},t.prototype.saveLayout=function(){return this.layout.saveLayout()},t.prototype.restoreLayout=function(e){this._mode="multiple-document",this.layout.restoreLayout(e),(i.b.IS_EDGE||i.b.IS_IE)&&o.c.flush(),o.c.postMessage(this,re.LayoutModified)},t.prototype.addWidget=function(e,t){void 0===t&&(t={}),"single-document"===this._mode?this.layout.addWidget(e):this.layout.addWidget(e,t),o.c.postMessage(this,re.LayoutModified)},t.prototype.processMessage=function(t){"layout-modified"===t.type?this._layoutModified.emit(void 0):e.prototype.processMessage.call(this,t)},t.prototype.handleEvent=function(e){switch(e.type){case"lm-dragenter":this._evtDragEnter(e);break;case"lm-dragleave":this._evtDragLeave(e);break;case"lm-dragover":this._evtDragOver(e);break;case"lm-drop":this._evtDrop(e);break;case"mousedown":this._evtMouseDown(e);break;case"mousemove":this._evtMouseMove(e);break;case"mouseup":this._evtMouseUp(e);break;case"keydown":this._evtKeyDown(e);break;case"contextmenu":e.preventDefault(),e.stopPropagation()}},t.prototype.onBeforeAttach=function(e){this.node.addEventListener("lm-dragenter",this),this.node.addEventListener("lm-dragleave",this),this.node.addEventListener("lm-dragover",this),this.node.addEventListener("lm-drop",this),this.node.addEventListener("mousedown",this)},t.prototype.onAfterDetach=function(e){this.node.removeEventListener("lm-dragenter",this),this.node.removeEventListener("lm-dragleave",this),this.node.removeEventListener("lm-dragover",this),this.node.removeEventListener("lm-drop",this),this.node.removeEventListener("mousedown",this),this._releaseMouse()},t.prototype.onChildAdded=function(e){re.isGeneratedTabBarProperty.get(e.child)||(e.child.addClass("lm-DockPanel-widget"),e.child.addClass("p-DockPanel-widget"))},t.prototype.onChildRemoved=function(e){re.isGeneratedTabBarProperty.get(e.child)||(e.child.removeClass("lm-DockPanel-widget"),e.child.removeClass("p-DockPanel-widget"),o.c.postMessage(this,re.LayoutModified))},t.prototype._evtDragEnter=function(e){e.mimeData.hasData("application/vnd.lumino.widget-factory")&&(e.preventDefault(),e.stopPropagation())},t.prototype._evtDragLeave=function(e){e.preventDefault(),e.stopPropagation(),this.overlay.hide(1)},t.prototype._evtDragOver=function(e){e.preventDefault(),e.stopPropagation(),this._tabsConstrained&&e.source!==this||"invalid"===this._showOverlay(e.clientX,e.clientY)?e.dropAction="none":e.dropAction=e.proposedAction},t.prototype._evtDrop=function(e){if(e.preventDefault(),e.stopPropagation(),this.overlay.hide(0),"none"!==e.proposedAction){var t=e.clientX,n=e.clientY,r=re.findDropTarget(this,t,n,this._edges),i=r.zone,o=r.target;if("invalid"!==i){var a=e.mimeData.getData("application/vnd.lumino.widget-factory");if("function"==typeof a){var s=a();if(s instanceof L)if(s.contains(this))e.dropAction="none";else{var l=o?re.getDropRef(o.tabBar):null;switch(i){case"root-all":this.addWidget(s);break;case"root-top":this.addWidget(s,{mode:"split-top"});break;case"root-left":this.addWidget(s,{mode:"split-left"});break;case"root-right":this.addWidget(s,{mode:"split-right"});break;case"root-bottom":this.addWidget(s,{mode:"split-bottom"});break;case"widget-all":this.addWidget(s,{mode:"tab-after",ref:l});break;case"widget-top":this.addWidget(s,{mode:"split-top",ref:l});break;case"widget-left":this.addWidget(s,{mode:"split-left",ref:l});break;case"widget-right":this.addWidget(s,{mode:"split-right",ref:l});break;case"widget-bottom":this.addWidget(s,{mode:"split-bottom",ref:l});break;case"widget-tab":this.addWidget(s,{mode:"tab-after",ref:l});break;default:throw"unreachable"}e.dropAction=e.proposedAction,this.activateWidget(s)}else e.dropAction="none"}else e.dropAction="none"}else e.dropAction="none"}else e.dropAction="none"},t.prototype._evtKeyDown=function(e){e.preventDefault(),e.stopPropagation(),27===e.keyCode&&(this._releaseMouse(),o.c.postMessage(this,re.LayoutModified))},t.prototype._evtMouseDown=function(e){if(0===e.button){var t=this.layout,n=e.target,i=Object(r.find)(t.handles(),(function(e){return e.contains(n)}));if(i){e.preventDefault(),e.stopPropagation(),document.addEventListener("keydown",this,!0),document.addEventListener("mouseup",this,!0),document.addEventListener("mousemove",this,!0),document.addEventListener("contextmenu",this,!0);var o=i.getBoundingClientRect(),a=e.clientX-o.left,s=e.clientY-o.top,l=window.getComputedStyle(i),u=x.overrideCursor(l.cursor);this._pressData={handle:i,deltaX:a,deltaY:s,override:u}}}},t.prototype._evtMouseMove=function(e){if(this._pressData){e.preventDefault(),e.stopPropagation();var t=this.node.getBoundingClientRect(),n=e.clientX-t.left-this._pressData.deltaX,r=e.clientY-t.top-this._pressData.deltaY;this.layout.moveHandle(this._pressData.handle,n,r)}},t.prototype._evtMouseUp=function(e){0===e.button&&(e.preventDefault(),e.stopPropagation(),this._releaseMouse(),o.c.postMessage(this,re.LayoutModified))},t.prototype._releaseMouse=function(){this._pressData&&(this._pressData.override.dispose(),this._pressData=null,document.removeEventListener("keydown",this,!0),document.removeEventListener("mouseup",this,!0),document.removeEventListener("mousemove",this,!0),document.removeEventListener("contextmenu",this,!0))},t.prototype._showOverlay=function(e,t){var n,r,o,a,s=re.findDropTarget(this,e,t,this._edges),l=s.zone,u=s.target;if("invalid"===l)return this.overlay.hide(100),l;var c=i.a.boxSizing(this.node),d=this.node.getBoundingClientRect();switch(l){case"root-all":n=c.paddingTop,r=c.paddingLeft,o=c.paddingRight,a=c.paddingBottom;break;case"root-top":n=c.paddingTop,r=c.paddingLeft,o=c.paddingRight,a=d.height*re.GOLDEN_RATIO;break;case"root-left":n=c.paddingTop,r=c.paddingLeft,o=d.width*re.GOLDEN_RATIO,a=c.paddingBottom;break;case"root-right":n=c.paddingTop,r=d.width*re.GOLDEN_RATIO,o=c.paddingRight,a=c.paddingBottom;break;case"root-bottom":n=d.height*re.GOLDEN_RATIO,r=c.paddingLeft,o=c.paddingRight,a=c.paddingBottom;break;case"widget-all":n=u.top,r=u.left,o=u.right,a=u.bottom;break;case"widget-top":n=u.top,r=u.left,o=u.right,a=u.bottom+u.height/2;break;case"widget-left":n=u.top,r=u.left,o=u.right+u.width/2,a=u.bottom;break;case"widget-right":n=u.top,r=u.left+u.width/2,o=u.right,a=u.bottom;break;case"widget-bottom":n=u.top+u.height/2,r=u.left,o=u.right,a=u.bottom;break;case"widget-tab":var h=u.tabBar.node.getBoundingClientRect().height;n=u.top,r=u.left,o=u.right,a=u.bottom+u.height-h;break;default:throw"unreachable"}return this.overlay.show({top:n,left:r,right:o,bottom:a}),l},t.prototype._createTabBar=function(){var e=this._renderer.createTabBar();return re.isGeneratedTabBarProperty.set(e,!0),"single-document"===this._mode&&e.hide(),e.tabsMovable=this._tabsMovable,e.allowDeselect=!1,e.removeBehavior="select-previous-tab",e.insertBehavior="select-tab-if-needed",e.tabMoved.connect(this._onTabMoved,this),e.currentChanged.connect(this._onCurrentChanged,this),e.tabCloseRequested.connect(this._onTabCloseRequested,this),e.tabDetachRequested.connect(this._onTabDetachRequested,this),e.tabActivateRequested.connect(this._onTabActivateRequested,this),e},t.prototype._createHandle=function(){return this._renderer.createHandle()},t.prototype._onTabMoved=function(){o.c.postMessage(this,re.LayoutModified)},t.prototype._onCurrentChanged=function(e,t){var n=t.previousTitle,r=t.currentTitle;n&&n.owner.hide(),r&&r.owner.show(),(i.b.IS_EDGE||i.b.IS_IE)&&o.c.flush(),o.c.postMessage(this,re.LayoutModified)},t.prototype._onTabActivateRequested=function(e,t){t.title.owner.activate()},t.prototype._onTabCloseRequested=function(e,t){t.title.owner.close()},t.prototype._onTabDetachRequested=function(e,t){var n=this;if(!this._drag){e.releaseMouse();var r=t.title,i=t.tab,o=t.clientX,a=t.clientY,s=new l.MimeData;s.setData("application/vnd.lumino.widget-factory",(function(){return r.owner}));var u=i.cloneNode(!0);this._drag=new x({mimeData:s,dragImage:u,proposedAction:"move",supportedActions:"move",source:this}),i.classList.add("lm-mod-hidden"),i.classList.add("p-mod-hidden");this._drag.start(o,a).then((function(){n._drag=null,i.classList.remove("lm-mod-hidden"),i.classList.remove("p-mod-hidden")}))}},t}(L);!function(e){var t=function(){function e(){this._timer=-1,this._hidden=!0,this.node=document.createElement("div"),this.node.classList.add("lm-DockPanel-overlay"),this.node.classList.add("lm-mod-hidden"),this.node.classList.add("p-DockPanel-overlay"),this.node.classList.add("p-mod-hidden"),this.node.style.position="absolute"}return e.prototype.show=function(e){var t=this.node.style;t.top=e.top+"px",t.left=e.left+"px",t.right=e.right+"px",t.bottom=e.bottom+"px",clearTimeout(this._timer),this._timer=-1,this._hidden&&(this._hidden=!1,this.node.classList.remove("lm-mod-hidden"),this.node.classList.remove("p-mod-hidden"))},e.prototype.hide=function(e){var t=this;if(!this._hidden)return e<=0?(clearTimeout(this._timer),this._timer=-1,this._hidden=!0,this.node.classList.add("lm-mod-hidden"),void this.node.classList.add("p-mod-hidden")):void(-1===this._timer&&(this._timer=window.setTimeout((function(){t._timer=-1,t._hidden=!0,t.node.classList.add("lm-mod-hidden"),t.node.classList.add("p-mod-hidden")}),e)))},e}();e.Overlay=t;var n=function(){function e(){}return e.prototype.createTabBar=function(){var e=new ne;return e.addClass("lm-DockPanel-tabBar"),e.addClass("p-DockPanel-tabBar"),e},e.prototype.createHandle=function(){var e=document.createElement("div");return e.className="lm-DockPanel-handle",e.classList.add("p-DockPanel-handle"),e},e}();e.Renderer=n,e.defaultRenderer=new n}(ie||(ie={})),function(e){e.GOLDEN_RATIO=.618,e.DEFAULT_EDGES={top:12,right:40,bottom:40,left:40},e.LayoutModified=new o.a("layout-modified"),e.isGeneratedTabBarProperty=new a.AttachedProperty({name:"isGeneratedTabBar",create:function(){return!1}}),e.createSingleDocumentConfig=function(e){if(e.isEmpty)return{main:null};var t=Object(r.toArray)(e.widgets()),n=e.selectedWidgets().next(),i=n?t.indexOf(n):-1;return{main:{type:"tab-area",widgets:t,currentIndex:i}}},e.findDropTarget=function(e,t,n,r){if(!i.a.hitTest(e.node,t,n))return{zone:"invalid",target:null};var o=e.layout;if(o.isEmpty)return{zone:"root-all",target:null};if("multiple-document"===e.mode){var a=e.node.getBoundingClientRect(),s=t-a.left+1,l=n-a.top+1,u=a.right-t,c=a.bottom-n;switch(Math.min(l,u,c,s)){case l:if(l<r.top)return{zone:"root-top",target:null};break;case u:if(u<r.right)return{zone:"root-right",target:null};break;case c:if(c<r.bottom)return{zone:"root-bottom",target:null};break;case s:if(s<r.left)return{zone:"root-left",target:null};break;default:throw"unreachable"}}var d=o.hitTestTabAreas(t,n);if(!d)return{zone:"invalid",target:null};if("single-document"===e.mode)return{zone:"widget-all",target:d};var h=d.x-d.left+1,f=d.y-d.top+1,p=d.left+d.width-d.x,g=d.top+d.height-d.y;if(f<d.tabBar.node.getBoundingClientRect().height)return{zone:"widget-tab",target:d};var v,m=Math.round(d.width/3),y=Math.round(d.height/3);if(h>m&&p>m&&f>y&&g>y)return{zone:"widget-all",target:d};switch(h/=m,f/=y,p/=m,g/=y,Math.min(h,f,p,g)){case h:v="widget-left";break;case f:v="widget-top";break;case p:v="widget-right";break;case g:v="widget-bottom";break;default:throw"unreachable"}return{zone:v,target:d}},e.getDropRef=function(e){return 0===e.titles.length?null:e.currentTitle?e.currentTitle.owner:e.titles[e.titles.length-1].owner}}(re||(re={}));var oe,ae,se=function(){function e(){this._counter=0,this._widgets=[],this._activeWidget=null,this._currentWidget=null,this._numbers=new Map,this._nodes=new Map,this._activeChanged=new s.Signal(this),this._currentChanged=new s.Signal(this)}return e.prototype.dispose=function(){var e=this;this._counter<0||(this._counter=-1,s.Signal.clearData(this),Object(r.each)(this._widgets,(function(t){t.node.removeEventListener("focus",e,!0),t.node.removeEventListener("blur",e,!0)})),this._activeWidget=null,this._currentWidget=null,this._nodes.clear(),this._numbers.clear(),this._widgets.length=0)},Object.defineProperty(e.prototype,"currentChanged",{get:function(){return this._currentChanged},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"activeChanged",{get:function(){return this._activeChanged},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"isDisposed",{get:function(){return this._counter<0},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"currentWidget",{get:function(){return this._currentWidget},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"activeWidget",{get:function(){return this._activeWidget},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"widgets",{get:function(){return this._widgets},enumerable:!0,configurable:!0}),e.prototype.focusNumber=function(e){var t=this._numbers.get(e);return void 0===t?-1:t},e.prototype.has=function(e){return this._numbers.has(e)},e.prototype.add=function(e){if(!this._numbers.has(e)){var t=e.node.contains(document.activeElement),n=t?this._counter++:-1;this._widgets.push(e),this._numbers.set(e,n),this._nodes.set(e.node,e),e.node.addEventListener("focus",this,!0),e.node.addEventListener("blur",this,!0),e.disposed.connect(this._onWidgetDisposed,this),t&&this._setWidgets(e,e)}},e.prototype.remove=function(e){var t=this;if(this._numbers.has(e)&&(e.disposed.disconnect(this._onWidgetDisposed,this),e.node.removeEventListener("focus",this,!0),e.node.removeEventListener("blur",this,!0),r.ArrayExt.removeFirstOf(this._widgets,e),this._nodes.delete(e.node),this._numbers.delete(e),this._currentWidget===e)){var n=Object(r.filter)(this._widgets,(function(e){return-1!==t._numbers.get(e)})),i=Object(r.max)(n,(function(e,n){return t._numbers.get(e)-t._numbers.get(n)}))||null;this._setWidgets(i,null)}},e.prototype.handleEvent=function(e){switch(e.type){case"focus":this._evtFocus(e);break;case"blur":this._evtBlur(e)}},e.prototype._setWidgets=function(e,t){var n=this._currentWidget;this._currentWidget=e;var r=this._activeWidget;this._activeWidget=t,n!==e&&this._currentChanged.emit({oldValue:n,newValue:e}),r!==t&&this._activeChanged.emit({oldValue:r,newValue:t})},e.prototype._evtFocus=function(e){var t=this._nodes.get(e.currentTarget);t!==this._currentWidget&&this._numbers.set(t,this._counter++),this._setWidgets(t,t)},e.prototype._evtBlur=function(e){var t=this._nodes.get(e.currentTarget),n=e.relatedTarget;n&&(t.node.contains(n)||Object(r.find)(this._widgets,(function(e){return e.node.contains(n)})))||this._setWidgets(this._currentWidget,null)},e.prototype._onWidgetDisposed=function(e){this.remove(e)},e}(),le=function(e){function t(t){void 0===t&&(t={});var n=e.call(this,t)||this;return n._dirty=!1,n._rowSpacing=4,n._columnSpacing=4,n._items=[],n._rowStarts=[],n._columnStarts=[],n._rowSizers=[new M],n._columnSizers=[new M],n._box=null,void 0!==t.rowCount&&ae.reallocSizers(n._rowSizers,t.rowCount),void 0!==t.columnCount&&ae.reallocSizers(n._columnSizers,t.columnCount),void 0!==t.rowSpacing&&(n._rowSpacing=ae.clampValue(t.rowSpacing)),void 0!==t.columnSpacing&&(n._columnSpacing=ae.clampValue(t.columnSpacing)),n}return T(t,e),t.prototype.dispose=function(){Object(r.each)(this._items,(function(e){var t=e.widget;e.dispose(),t.dispose()})),this._box=null,this._items.length=0,this._rowStarts.length=0,this._rowSizers.length=0,this._columnStarts.length=0,this._columnSizers.length=0,e.prototype.dispose.call(this)},Object.defineProperty(t.prototype,"rowCount",{get:function(){return this._rowSizers.length},set:function(e){e!==this.rowCount&&(ae.reallocSizers(this._rowSizers,e),this.parent&&this.parent.fit())},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"columnCount",{get:function(){return this._columnSizers.length},set:function(e){e!==this.columnCount&&(ae.reallocSizers(this._columnSizers,e),this.parent&&this.parent.fit())},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"rowSpacing",{get:function(){return this._rowSpacing},set:function(e){e=ae.clampValue(e),this._rowSpacing!==e&&(this._rowSpacing=e,this.parent&&this.parent.fit())},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"columnSpacing",{get:function(){return this._columnSpacing},set:function(e){e=ae.clampValue(e),this._columnSpacing!==e&&(this._columnSpacing=e,this.parent&&this.parent.fit())},enumerable:!0,configurable:!0}),t.prototype.rowStretch=function(e){var t=this._rowSizers[e];return t?t.stretch:-1},t.prototype.setRowStretch=function(e,t){var n=this._rowSizers[e];n&&(t=ae.clampValue(t),n.stretch!==t&&(n.stretch=t,this.parent&&this.parent.update()))},t.prototype.columnStretch=function(e){var t=this._columnSizers[e];return t?t.stretch:-1},t.prototype.setColumnStretch=function(e,t){var n=this._columnSizers[e];n&&(t=ae.clampValue(t),n.stretch!==t&&(n.stretch=t,this.parent&&this.parent.update()))},t.prototype.iter=function(){return Object(r.map)(this._items,(function(e){return e.widget}))},t.prototype.addWidget=function(e){-1===r.ArrayExt.findFirstIndex(this._items,(function(t){return t.widget===e}))&&(this._items.push(new j(e)),this.parent&&this.attachWidget(e))},t.prototype.removeWidget=function(e){var t=r.ArrayExt.findFirstIndex(this._items,(function(t){return t.widget===e}));if(-1!==t){var n=r.ArrayExt.removeAt(this._items,t);this.parent&&this.detachWidget(e),n.dispose()}},t.prototype.init=function(){var t=this;e.prototype.init.call(this),Object(r.each)(this,(function(e){t.attachWidget(e)}))},t.prototype.attachWidget=function(e){this.parent.isAttached&&o.c.sendMessage(e,L.Msg.BeforeAttach),this.parent.node.appendChild(e.node),this.parent.isAttached&&o.c.sendMessage(e,L.Msg.AfterAttach),this.parent.fit()},t.prototype.detachWidget=function(e){this.parent.isAttached&&o.c.sendMessage(e,L.Msg.BeforeDetach),this.parent.node.removeChild(e.node),this.parent.isAttached&&o.c.sendMessage(e,L.Msg.AfterDetach),this.parent.fit()},t.prototype.onBeforeShow=function(t){e.prototype.onBeforeShow.call(this,t),this.parent.update()},t.prototype.onBeforeAttach=function(t){e.prototype.onBeforeAttach.call(this,t),this.parent.fit()},t.prototype.onChildShown=function(e){this.parent.fit()},t.prototype.onChildHidden=function(e){this.parent.fit()},t.prototype.onResize=function(e){this.parent.isVisible&&this._update(e.width,e.height)},t.prototype.onUpdateRequest=function(e){this.parent.isVisible&&this._update(-1,-1)},t.prototype.onFitRequest=function(e){this.parent.isAttached&&this._fit()},t.prototype._fit=function(){for(var e=0,n=this.rowCount;e<n;++e)this._rowSizers[e].minSize=0;for(e=0,n=this.columnCount;e<n;++e)this._columnSizers[e].minSize=0;var r=this._items.filter((function(e){return!e.isHidden}));for(e=0,n=r.length;e<n;++e)r[e].fit();var a=this.rowCount-1,s=this.columnCount-1;r.sort(ae.rowSpanCmp);for(e=0,n=r.length;e<n;++e){var l=r[e],u=t.getCellConfig(l.widget),c=Math.min(u.row,a),d=Math.min(u.row+u.rowSpan-1,a);ae.distributeMin(this._rowSizers,c,d,l.minHeight)}r.sort(ae.columnSpanCmp);for(e=0,n=r.length;e<n;++e){l=r[e],u=t.getCellConfig(l.widget);var h=Math.min(u.column,s),f=Math.min(u.column+u.columnSpan-1,s);ae.distributeMin(this._columnSizers,h,f,l.minWidth)}if("set-no-constraint"!==this.fitPolicy){var p=a*this._rowSpacing,g=s*this._columnSpacing;for(e=0,n=this.rowCount;e<n;++e)p+=this._rowSizers[e].minSize;for(e=0,n=this.columnCount;e<n;++e)g+=this._columnSizers[e].minSize;var v=this._box=i.a.boxSizing(this.parent.node);g+=v.horizontalSum,p+=v.verticalSum;var m=this.parent.node.style;m.minWidth=g+"px",m.minHeight=p+"px",this._dirty=!0,this.parent.parent&&o.c.sendMessage(this.parent.parent,L.Msg.FitRequest),this._dirty&&o.c.sendMessage(this.parent,L.Msg.UpdateRequest)}else o.c.sendMessage(this.parent,L.Msg.UpdateRequest)},t.prototype._update=function(e,n){this._dirty=!1,e<0&&(e=this.parent.node.offsetWidth),n<0&&(n=this.parent.node.offsetHeight),this._box||(this._box=i.a.boxSizing(this.parent.node));var r=this._box.paddingTop,o=this._box.paddingLeft,a=e-this._box.horizontalSum,s=n-this._box.verticalSum,l=this.rowCount-1,u=this.columnCount-1,c=l*this._rowSpacing,d=u*this._columnSpacing;w.calc(this._rowSizers,Math.max(0,s-c)),w.calc(this._columnSizers,Math.max(0,a-d));for(var h=0,f=r,p=this.rowCount;h<p;++h)this._rowStarts[h]=f,f+=this._rowSizers[h].size+this._rowSpacing;for(h=0,f=o,p=this.columnCount;h<p;++h)this._columnStarts[h]=f,f+=this._columnSizers[h].size+this._columnSpacing;for(h=0,p=this._items.length;h<p;++h){var g=this._items[h];if(!g.isHidden){var v=t.getCellConfig(g.widget),m=Math.min(v.row,l),y=Math.min(v.column,u),b=Math.min(v.row+v.rowSpan-1,l),C=Math.min(v.column+v.columnSpan-1,u),A=this._columnStarts[y],_=this._rowStarts[m],I=this._columnStarts[C]+this._columnSizers[C].size-A,x=this._rowStarts[b]+this._rowSizers[b].size-_;g.update(A,_,I,x)}}},t}(P);(oe=le||(le={})).getCellConfig=function(e){return ae.cellConfigProperty.get(e)},oe.setCellConfig=function(e,t){ae.cellConfigProperty.set(e,ae.normalizeConfig(t))},function(e){e.cellConfigProperty=new a.AttachedProperty({name:"cellConfig",create:function(){return{row:0,column:0,rowSpan:1,columnSpan:1}},changed:function(e){e.parent&&e.parent.layout instanceof le&&e.parent.fit()}}),e.normalizeConfig=function(e){return{row:Math.max(0,Math.floor(e.row||0)),column:Math.max(0,Math.floor(e.column||0)),rowSpan:Math.max(1,Math.floor(e.rowSpan||0)),columnSpan:Math.max(1,Math.floor(e.columnSpan||0))}},e.clampValue=function(e){return Math.max(0,Math.floor(e))},e.rowSpanCmp=function(t,n){var r=e.cellConfigProperty.get(t.widget),i=e.cellConfigProperty.get(n.widget);return r.rowSpan-i.rowSpan},e.columnSpanCmp=function(t,n){var r=e.cellConfigProperty.get(t.widget),i=e.cellConfigProperty.get(n.widget);return r.columnSpan-i.columnSpan},e.reallocSizers=function(e,t){for(t=Math.max(1,Math.floor(t));e.length<t;)e.push(new M);e.length>t&&(e.length=t)},e.distributeMin=function(e,t,n,r){if(!(n<t))if(t!==n){for(var i=0,o=t;o<=n;++o)i+=e[o].minSize;if(!(i>=r)){var a=(r-i)/(n-t+1);for(o=t;o<=n;++o)e[o].minSize+=a}}else{var s=e[t];s.minSize=Math.max(s.minSize,r)}}}(ae||(ae={}));var ue,ce=function(e){function t(n){void 0===n&&(n={});var r=e.call(this,{node:ue.createNode()})||this;return r._activeIndex=-1,r._menus=[],r._childMenu=null,r.addClass("lm-MenuBar"),r.addClass("p-MenuBar"),r.setFlag(L.Flag.DisallowLayout),r.renderer=n.renderer||t.defaultRenderer,r}return T(t,e),t.prototype.dispose=function(){this._closeChildMenu(),this._menus.length=0,e.prototype.dispose.call(this)},Object.defineProperty(t.prototype,"childMenu",{get:function(){return this._childMenu},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"contentNode",{get:function(){return this.node.getElementsByClassName("lm-MenuBar-content")[0]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"activeMenu",{get:function(){return this._menus[this._activeIndex]||null},set:function(e){this.activeIndex=e?this._menus.indexOf(e):-1},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"activeIndex",{get:function(){return this._activeIndex},set:function(e){(e<0||e>=this._menus.length)&&(e=-1),this._activeIndex!==e&&(this._activeIndex=e,this.update())},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"menus",{get:function(){return this._menus},enumerable:!0,configurable:!0}),t.prototype.openActiveMenu=function(){-1!==this._activeIndex&&(this._openChildMenu(),this._childMenu&&(this._childMenu.activeIndex=-1,this._childMenu.activateNextItem()))},t.prototype.addMenu=function(e){this.insertMenu(this._menus.length,e)},t.prototype.insertMenu=function(e,t){this._closeChildMenu();var n=this._menus.indexOf(t),i=Math.max(0,Math.min(e,this._menus.length));if(-1===n)return r.ArrayExt.insert(this._menus,i,t),t.addClass("lm-MenuBar-menu"),t.addClass("p-MenuBar-menu"),t.aboutToClose.connect(this._onMenuAboutToClose,this),t.menuRequested.connect(this._onMenuMenuRequested,this),t.title.changed.connect(this._onTitleChanged,this),void this.update();i===this._menus.length&&i--,n!==i&&(r.ArrayExt.move(this._menus,n,i),this.update())},t.prototype.removeMenu=function(e){this.removeMenuAt(this._menus.indexOf(e))},t.prototype.removeMenuAt=function(e){this._closeChildMenu();var t=r.ArrayExt.removeAt(this._menus,e);t&&(t.aboutToClose.disconnect(this._onMenuAboutToClose,this),t.menuRequested.disconnect(this._onMenuMenuRequested,this),t.title.changed.disconnect(this._onTitleChanged,this),t.removeClass("lm-MenuBar-menu"),t.removeClass("p-MenuBar-menu"),this.update())},t.prototype.clearMenus=function(){if(0!==this._menus.length){this._closeChildMenu();for(var e=0,t=this._menus;e<t.length;e++){var n=t[e];n.aboutToClose.disconnect(this._onMenuAboutToClose,this),n.menuRequested.disconnect(this._onMenuMenuRequested,this),n.title.changed.disconnect(this._onTitleChanged,this),n.removeClass("lm-MenuBar-menu"),n.removeClass("p-MenuBar-menu")}this._menus.length=0,this.update()}},t.prototype.handleEvent=function(e){switch(e.type){case"keydown":this._evtKeyDown(e);break;case"mousedown":this._evtMouseDown(e);break;case"mousemove":this._evtMouseMove(e);break;case"mouseleave":this._evtMouseLeave(e);break;case"contextmenu":e.preventDefault(),e.stopPropagation()}},t.prototype.onBeforeAttach=function(e){this.node.addEventListener("keydown",this),this.node.addEventListener("mousedown",this),this.node.addEventListener("mousemove",this),this.node.addEventListener("mouseleave",this),this.node.addEventListener("contextmenu",this)},t.prototype.onAfterDetach=function(e){this.node.removeEventListener("keydown",this),this.node.removeEventListener("mousedown",this),this.node.removeEventListener("mousemove",this),this.node.removeEventListener("mouseleave",this),this.node.removeEventListener("contextmenu",this),this._closeChildMenu()},t.prototype.onActivateRequest=function(e){this.isAttached&&this.node.focus()},t.prototype.onUpdateRequest=function(e){for(var t=this._menus,n=this.renderer,r=this._activeIndex,i=new Array(t.length),o=0,a=t.length;o<a;++o){var s=t[o].title,l=o===r;i[o]=n.renderItem({title:s,active:l})}y.render(i,this.contentNode)},t.prototype._evtKeyDown=function(e){e.preventDefault(),e.stopPropagation();var t=e.keyCode;if(13!==t&&38!==t&&40!==t){if(27===t)return this._closeChildMenu(),this.activeIndex=-1,void this.node.blur();if(37!==t)if(39!==t){var n=c().keyForKeydownEvent(e);if(n){var r=this._activeIndex+1,i=ue.findMnemonic(this._menus,n,r);-1===i.index||i.multiple?-1!==i.index?this.activeIndex=i.index:-1!==i.auto&&(this.activeIndex=i.auto):(this.activeIndex=i.index,this.openActiveMenu())}}else{o=this._activeIndex,a=this._menus.length;this.activeIndex=o===a-1?0:o+1}else{var o=this._activeIndex,a=this._menus.length;this.activeIndex=0===o?a-1:o-1}}else this.openActiveMenu()},t.prototype._evtMouseDown=function(e){if(i.a.hitTest(this.node,e.clientX,e.clientY)){e.preventDefault(),e.stopPropagation(),e.stopImmediatePropagation();var t=r.ArrayExt.findFirstIndex(this.contentNode.children,(function(t){return i.a.hitTest(t,e.clientX,e.clientY)}));-1!==t?0===e.button&&(this._childMenu?(this._closeChildMenu(),this.activeIndex=t):(this.activeIndex=t,this._openChildMenu())):this._closeChildMenu()}},t.prototype._evtMouseMove=function(e){var t=r.ArrayExt.findFirstIndex(this.contentNode.children,(function(t){return i.a.hitTest(t,e.clientX,e.clientY)}));t!==this._activeIndex&&(-1===t&&this._childMenu||(this.activeIndex=t,this._childMenu&&this._openChildMenu()))},t.prototype._evtMouseLeave=function(e){this._childMenu||(this.activeIndex=-1)},t.prototype._openChildMenu=function(){var e=this.activeMenu;if(e){var t=this._childMenu;if(t!==e){this._childMenu=e,t?t.close():(this.addClass("lm-mod-active"),this.addClass("p-mod-active"),document.addEventListener("mousedown",this,!0)),o.c.sendMessage(this,L.Msg.UpdateRequest);var n=this.contentNode.children[this._activeIndex].getBoundingClientRect(),r=n.left,i=n.bottom;e.open(r,i,{forceX:!0,forceY:!0})}}else this._closeChildMenu()},t.prototype._closeChildMenu=function(){if(this._childMenu){this.removeClass("lm-mod-active"),this.removeClass("p-mod-active"),document.removeEventListener("mousedown",this,!0);var e=this._childMenu;this._childMenu=null,e.close(),this.activeIndex=-1}},t.prototype._onMenuAboutToClose=function(e){e===this._childMenu&&(this.removeClass("lm-mod-active"),this.removeClass("p-mod-active"),document.removeEventListener("mousedown",this,!0),this._childMenu=null,this.activeIndex=-1)},t.prototype._onMenuMenuRequested=function(e,t){if(e===this._childMenu){var n=this._activeIndex,r=this._menus.length;switch(t){case"next":this.activeIndex=n===r-1?0:n+1;break;case"previous":this.activeIndex=0===n?r-1:n-1}this.openActiveMenu()}},t.prototype._onTitleChanged=function(){this.update()},t}(L);!function(e){var t=function(){function e(){}return e.prototype.renderItem=function(e){var t=this.createItemClass(e),n=this.createItemDataset(e),r=this.createItemARIA(e);return _.li(O({className:t,dataset:n},r),this.renderIcon(e),this.renderLabel(e))},e.prototype.renderIcon=function(e){var t=this.createIconClass(e);return"string"==typeof e.title.icon?_.div({className:t},e.title.iconLabel):_.div({className:t},e.title.icon,e.title.iconLabel)},e.prototype.renderLabel=function(e){var t=this.formatLabel(e);return _.div({className:"lm-MenuBar-itemLabel p-MenuBar-itemLabel"},t)},e.prototype.createItemClass=function(e){var t="lm-MenuBar-item";return t+=" p-MenuBar-item",e.title.className&&(t+=" "+e.title.className),e.active&&(t+=" lm-mod-active",t+=" p-mod-active"),t},e.prototype.createItemDataset=function(e){return e.title.dataset},e.prototype.createItemARIA=function(e){return{role:"menuitem","aria-haspopup":"true"}},e.prototype.createIconClass=function(e){var t="lm-MenuBar-itemIcon";t+=" p-MenuBar-itemIcon";var n=e.title.iconClass;return n?t+" "+n:t},e.prototype.formatLabel=function(e){var t=e.title,n=t.label,r=t.mnemonic;if(r<0||r>=n.length)return n;var i=n.slice(0,r),o=n.slice(r+1),a=n[r];return[i,_.span({className:"lm-MenuBar-itemMnemonic p-MenuBar-itemMnemonic"},a),o]},e}();e.Renderer=t,e.defaultRenderer=new t}(ce||(ce={})),function(e){e.createNode=function(){var e=document.createElement("div"),t=document.createElement("ul");return t.className="lm-MenuBar-content",t.classList.add("p-MenuBar-content"),e.appendChild(t),t.setAttribute("role","menubar"),e.tabIndex=-1,e},e.findMnemonic=function(e,t,n){for(var r=-1,i=-1,o=!1,a=t.toUpperCase(),s=0,l=e.length;s<l;++s){var u=(s+n)%l,c=e[u].title;if(0!==c.label.length){var d=c.mnemonic;d>=0&&d<c.label.length?c.label[d].toUpperCase()===a&&(-1===r?r=u:o=!0):-1===i&&c.label[0].toUpperCase()===a&&(i=u)}}return{index:r,multiple:o,auto:i}}}(ue||(ue={}));var de,he=function(e){function t(t){void 0===t&&(t={});var n=e.call(this,{node:de.createNode()})||this;return n._onRepeat=function(){if(n._repeatTimer=-1,n._pressData){var e=n._pressData.part;if("thumb"!==e){n._repeatTimer=window.setTimeout(n._onRepeat,20);var t=n._pressData.mouseX,r=n._pressData.mouseY;if("decrement"!==e)if("increment"!==e){if("track"===e){if(!i.a.hitTest(n.trackNode,t,r))return;var o=n.thumbNode;if(i.a.hitTest(o,t,r))return;var a=o.getBoundingClientRect(),s=void 0;return s="horizontal"===n._orientation?t<a.left?"decrement":"increment":r<a.top?"decrement":"increment",void n._pageRequested.emit(s)}}else{if(!i.a.hitTest(n.incrementNode,t,r))return;n._stepRequested.emit("increment")}else{if(!i.a.hitTest(n.decrementNode,t,r))return;n._stepRequested.emit("decrement")}}}},n._value=0,n._page=10,n._maximum=100,n._repeatTimer=-1,n._pressData=null,n._thumbMoved=new s.Signal(n),n._stepRequested=new s.Signal(n),n._pageRequested=new s.Signal(n),n.addClass("lm-ScrollBar"),n.addClass("p-ScrollBar"),n.setFlag(L.Flag.DisallowLayout),n._orientation=t.orientation||"vertical",n.dataset.orientation=n._orientation,void 0!==t.maximum&&(n._maximum=Math.max(0,t.maximum)),void 0!==t.page&&(n._page=Math.max(0,t.page)),void 0!==t.value&&(n._value=Math.max(0,Math.min(t.value,n._maximum))),n}return T(t,e),Object.defineProperty(t.prototype,"thumbMoved",{get:function(){return this._thumbMoved},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"stepRequested",{get:function(){return this._stepRequested},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"pageRequested",{get:function(){return this._pageRequested},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"orientation",{get:function(){return this._orientation},set:function(e){this._orientation!==e&&(this._releaseMouse(),this._orientation=e,this.dataset.orientation=e,this.update())},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"value",{get:function(){return this._value},set:function(e){e=Math.max(0,Math.min(e,this._maximum)),this._value!==e&&(this._value=e,this.update())},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"page",{get:function(){return this._page},set:function(e){e=Math.max(0,e),this._page!==e&&(this._page=e,this.update())},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"maximum",{get:function(){return this._maximum},set:function(e){e=Math.max(0,e),this._maximum!==e&&(this._maximum=e,this._value=Math.min(this._value,e),this.update())},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"decrementNode",{get:function(){return this.node.getElementsByClassName("lm-ScrollBar-button")[0]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"incrementNode",{get:function(){return this.node.getElementsByClassName("lm-ScrollBar-button")[1]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"trackNode",{get:function(){return this.node.getElementsByClassName("lm-ScrollBar-track")[0]},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"thumbNode",{get:function(){return this.node.getElementsByClassName("lm-ScrollBar-thumb")[0]},enumerable:!0,configurable:!0}),t.prototype.handleEvent=function(e){switch(e.type){case"mousedown":this._evtMouseDown(e);break;case"mousemove":this._evtMouseMove(e);break;case"mouseup":this._evtMouseUp(e);break;case"keydown":this._evtKeyDown(e);break;case"contextmenu":e.preventDefault(),e.stopPropagation()}},t.prototype.onBeforeAttach=function(e){this.node.addEventListener("mousedown",this),this.update()},t.prototype.onAfterDetach=function(e){this.node.removeEventListener("mousedown",this),this._releaseMouse()},t.prototype.onUpdateRequest=function(e){var t=100*this._value/this._maximum,n=100*this._page/(this._page+this._maximum);t=Math.max(0,Math.min(t,100)),n=Math.max(0,Math.min(n,100));var r=this.thumbNode.style;"horizontal"===this._orientation?(r.top="",r.height="",r.left=t+"%",r.width=n+"%",r.transform="translate("+-t+"%, 0%)"):(r.left="",r.width="",r.top=t+"%",r.height=n+"%",r.transform="translate(0%, "+-t+"%)")},t.prototype._evtKeyDown=function(e){if(e.preventDefault(),e.stopPropagation(),27===e.keyCode){var t=this._pressData?this._pressData.value:-1;this._releaseMouse(),-1!==t&&this._moveThumb(t)}},t.prototype._evtMouseDown=function(e){if(0===e.button&&(this.activate(),!this._pressData)){var t=de.findPart(this,e.target);if(t){e.preventDefault(),e.stopPropagation();var n=x.overrideCursor("default");if(this._pressData={part:t,override:n,delta:-1,value:-1,mouseX:e.clientX,mouseY:e.clientY},document.addEventListener("mousemove",this,!0),document.addEventListener("mouseup",this,!0),document.addEventListener("keydown",this,!0),document.addEventListener("contextmenu",this,!0),"thumb"===t){var r=this.thumbNode,i=r.getBoundingClientRect();return"horizontal"===this._orientation?this._pressData.delta=e.clientX-i.left:this._pressData.delta=e.clientY-i.top,r.classList.add("lm-mod-active"),r.classList.add("p-mod-active"),void(this._pressData.value=this._value)}if("track"===t){i=this.thumbNode.getBoundingClientRect();var o=void 0;return o="horizontal"===this._orientation?e.clientX<i.left?"decrement":"increment":e.clientY<i.top?"decrement":"increment",this._repeatTimer=window.setTimeout(this._onRepeat,350),void this._pageRequested.emit(o)}return"decrement"===t?(this.decrementNode.classList.add("lm-mod-active"),this.decrementNode.classList.add("p-mod-active"),this._repeatTimer=window.setTimeout(this._onRepeat,350),void this._stepRequested.emit("decrement")):"increment"===t?(this.incrementNode.classList.add("lm-mod-active"),this.incrementNode.classList.add("p-mod-active"),this._repeatTimer=window.setTimeout(this._onRepeat,350),void this._stepRequested.emit("increment")):void 0}}},t.prototype._evtMouseMove=function(e){if(this._pressData&&(e.preventDefault(),e.stopPropagation(),this._pressData.mouseX=e.clientX,this._pressData.mouseY=e.clientY,"thumb"===this._pressData.part)){var t,n,r=this.thumbNode.getBoundingClientRect(),i=this.trackNode.getBoundingClientRect();"horizontal"===this._orientation?(t=e.clientX-i.left-this._pressData.delta,n=i.width-r.width):(t=e.clientY-i.top-this._pressData.delta,n=i.height-r.height);var o=0===n?0:t*this._maximum/n;this._moveThumb(o)}},t.prototype._evtMouseUp=function(e){0===e.button&&(e.preventDefault(),e.stopPropagation(),this._releaseMouse())},t.prototype._releaseMouse=function(){this._pressData&&(clearTimeout(this._repeatTimer),this._repeatTimer=-1,this._pressData.override.dispose(),this._pressData=null,document.removeEventListener("mousemove",this,!0),document.removeEventListener("mouseup",this,!0),document.removeEventListener("keydown",this,!0),document.removeEventListener("contextmenu",this,!0),this.thumbNode.classList.remove("lm-mod-active"),this.decrementNode.classList.remove("lm-mod-active"),this.incrementNode.classList.remove("lm-mod-active"),this.thumbNode.classList.remove("p-mod-active"),this.decrementNode.classList.remove("p-mod-active"),this.incrementNode.classList.remove("p-mod-active"))},t.prototype._moveThumb=function(e){e=Math.max(0,Math.min(e,this._maximum)),this._value!==e&&(this._value=e,this.update(),this._thumbMoved.emit(e))},t}(L);!function(e){e.createNode=function(){var e=document.createElement("div"),t=document.createElement("div"),n=document.createElement("div"),r=document.createElement("div"),i=document.createElement("div");return t.className="lm-ScrollBar-button",n.className="lm-ScrollBar-button",t.dataset.action="decrement",n.dataset.action="increment",r.className="lm-ScrollBar-track",i.className="lm-ScrollBar-thumb",t.classList.add("p-ScrollBar-button"),n.classList.add("p-ScrollBar-button"),r.classList.add("p-ScrollBar-track"),i.classList.add("p-ScrollBar-thumb"),r.appendChild(i),e.appendChild(t),e.appendChild(r),e.appendChild(n),e},e.findPart=function(e,t){return e.thumbNode.contains(t)?"thumb":e.trackNode.contains(t)?"track":e.decrementNode.contains(t)?"decrement":e.incrementNode.contains(t)?"increment":null}}(de||(de={}));var fe,pe,ge=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t._widget=null,t}return T(t,e),t.prototype.dispose=function(){if(this._widget){var t=this._widget;this._widget=null,t.dispose()}e.prototype.dispose.call(this)},Object.defineProperty(t.prototype,"widget",{get:function(){return this._widget},set:function(e){e&&(e.parent=this.parent),this._widget!==e&&(this._widget&&this._widget.dispose(),this._widget=e,this.parent&&e&&this.attachWidget(e))},enumerable:!0,configurable:!0}),t.prototype.iter=function(){return this._widget?Object(r.once)(this._widget):Object(r.empty)()},t.prototype.removeWidget=function(e){this._widget===e&&(this._widget=null,this.parent&&this.detachWidget(e))},t.prototype.init=function(){var t=this;e.prototype.init.call(this),Object(r.each)(this,(function(e){t.attachWidget(e)}))},t.prototype.attachWidget=function(e){this.parent.isAttached&&o.c.sendMessage(e,L.Msg.BeforeAttach),this.parent.node.appendChild(e.node),this.parent.isAttached&&o.c.sendMessage(e,L.Msg.AfterAttach)},t.prototype.detachWidget=function(e){this.parent.isAttached&&o.c.sendMessage(e,L.Msg.BeforeDetach),this.parent.node.removeChild(e.node),this.parent.isAttached&&o.c.sendMessage(e,L.Msg.AfterDetach)},t}(P),ve=function(e){function t(t){var n=e.call(this)||this;return n._fixed=0,n._spacing=4,n._dirty=!1,n._hasNormedSizes=!1,n._sizers=[],n._items=[],n._handles=[],n._box=null,n._alignment="start",n._orientation="horizontal",n.renderer=t.renderer,void 0!==t.orientation&&(n._orientation=t.orientation),void 0!==t.alignment&&(n._alignment=t.alignment),void 0!==t.spacing&&(n._spacing=pe.clampSpacing(t.spacing)),n}return T(t,e),t.prototype.dispose=function(){Object(r.each)(this._items,(function(e){e.dispose()})),this._box=null,this._items.length=0,this._sizers.length=0,this._handles.length=0,e.prototype.dispose.call(this)},Object.defineProperty(t.prototype,"orientation",{get:function(){return this._orientation},set:function(e){this._orientation!==e&&(this._orientation=e,this.parent&&(this.parent.dataset.orientation=e,this.parent.fit()))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"alignment",{get:function(){return this._alignment},set:function(e){this._alignment!==e&&(this._alignment=e,this.parent&&(this.parent.dataset.alignment=e,this.parent.update()))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"spacing",{get:function(){return this._spacing},set:function(e){e=pe.clampSpacing(e),this._spacing!==e&&(this._spacing=e,this.parent&&this.parent.fit())},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"handles",{get:function(){return this._handles},enumerable:!0,configurable:!0}),t.prototype.relativeSizes=function(){return pe.normalize(this._sizers.map((function(e){return e.size})))},t.prototype.setRelativeSizes=function(e){for(var t=this._sizers.length,n=e.slice(0,t);n.length<t;)n.push(0);for(var r=pe.normalize(n),i=0;i<t;++i){var o=this._sizers[i];o.sizeHint=r[i],o.size=r[i]}this._hasNormedSizes=!0,this.parent&&this.parent.update()},t.prototype.moveHandle=function(e,t){var n,r=this._handles[e];if(r&&!r.classList.contains("lm-mod-hidden")&&0!==(n="horizontal"===this._orientation?t-r.offsetLeft:t-r.offsetTop)){for(var i=0,o=this._sizers;i<o.length;i++){var a=o[i];a.size>0&&(a.sizeHint=a.size)}w.adjust(this._sizers,e,n),this.parent&&this.parent.update()}},t.prototype.init=function(){this.parent.dataset.orientation=this.orientation,this.parent.dataset.alignment=this.alignment,e.prototype.init.call(this)},t.prototype.attachWidget=function(e,t){var n=new j(t),i=pe.createHandle(this.renderer),a=pe.averageSize(this._sizers),s=pe.createSizer(a);r.ArrayExt.insert(this._items,e,n),r.ArrayExt.insert(this._sizers,e,s),r.ArrayExt.insert(this._handles,e,i),this.parent.isAttached&&o.c.sendMessage(t,L.Msg.BeforeAttach),this.parent.node.appendChild(t.node),this.parent.node.appendChild(i),this.parent.isAttached&&o.c.sendMessage(t,L.Msg.AfterAttach),this.parent.fit()},t.prototype.moveWidget=function(e,t,n){r.ArrayExt.move(this._items,e,t),r.ArrayExt.move(this._sizers,e,t),r.ArrayExt.move(this._handles,e,t),this.parent.fit()},t.prototype.detachWidget=function(e,t){var n=r.ArrayExt.removeAt(this._items,e),i=r.ArrayExt.removeAt(this._handles,e);r.ArrayExt.removeAt(this._sizers,e),this.parent.isAttached&&o.c.sendMessage(t,L.Msg.BeforeDetach),this.parent.node.removeChild(t.node),this.parent.node.removeChild(i),this.parent.isAttached&&o.c.sendMessage(t,L.Msg.AfterDetach),n.dispose(),this.parent.fit()},t.prototype.onBeforeShow=function(t){e.prototype.onBeforeShow.call(this,t),this.parent.update()},t.prototype.onBeforeAttach=function(t){e.prototype.onBeforeAttach.call(this,t),this.parent.fit()},t.prototype.onChildShown=function(e){this.parent.fit()},t.prototype.onChildHidden=function(e){this.parent.fit()},t.prototype.onResize=function(e){this.parent.isVisible&&this._update(e.width,e.height)},t.prototype.onUpdateRequest=function(e){this.parent.isVisible&&this._update(-1,-1)},t.prototype.onFitRequest=function(e){this.parent.isAttached&&this._fit()},t.prototype._fit=function(){for(var e=0,n=-1,r=0,a=this._items.length;r<a;++r)this._items[r].isHidden?(this._handles[r].classList.add("lm-mod-hidden"),this._handles[r].classList.add("p-mod-hidden")):(this._handles[r].classList.remove("lm-mod-hidden"),this._handles[r].classList.remove("p-mod-hidden"),n=r,e++);-1!==n&&(this._handles[n].classList.add("lm-mod-hidden"),this._handles[n].classList.add("p-mod-hidden")),this._fixed=this._spacing*Math.max(0,e-1);var s="horizontal"===this._orientation,l=s?this._fixed:0,u=s?0:this._fixed;for(r=0,a=this._items.length;r<a;++r){var c=this._items[r],d=this._sizers[r];d.size>0&&(d.sizeHint=d.size),c.isHidden?(d.minSize=0,d.maxSize=0):(c.fit(),d.stretch=t.getStretch(c.widget),s?(d.minSize=c.minWidth,d.maxSize=c.maxWidth,l+=c.minWidth,u=Math.max(u,c.minHeight)):(d.minSize=c.minHeight,d.maxSize=c.maxHeight,u+=c.minHeight,l=Math.max(l,c.minWidth)))}var h=this._box=i.a.boxSizing(this.parent.node);l+=h.horizontalSum,u+=h.verticalSum;var f=this.parent.node.style;f.minWidth=l+"px",f.minHeight=u+"px",this._dirty=!0,this.parent.parent&&o.c.sendMessage(this.parent.parent,L.Msg.FitRequest),this._dirty&&o.c.sendMessage(this.parent,L.Msg.UpdateRequest)},t.prototype._update=function(e,t){this._dirty=!1;for(var n=0,r=0,o=this._items.length;r<o;++r)n+=+!this._items[r].isHidden;if(0!==n){e<0&&(e=this.parent.node.offsetWidth),t<0&&(t=this.parent.node.offsetHeight),this._box||(this._box=i.a.boxSizing(this.parent.node));var a,s=this._box.paddingTop,l=this._box.paddingLeft,u=e-this._box.horizontalSum,c=t-this._box.verticalSum,d="horizontal"===this._orientation;if(a=d?Math.max(0,u-this._fixed):Math.max(0,c-this._fixed),this._hasNormedSizes){for(var h=0,f=this._sizers;h<f.length;h++){f[h].sizeHint*=a}this._hasNormedSizes=!1}var p=w.calc(this._sizers,a),g=0,v=0;if(p>0)switch(this._alignment){case"start":break;case"center":g=0,v=p/2;break;case"end":g=0,v=p;break;case"justify":g=p/n,v=0;break;default:throw"unreachable"}for(r=0,o=this._items.length;r<o;++r){var m=this._items[r];if(!m.isHidden){var y=this._sizers[r].size,b=this._handles[r].style;d?(m.update(l+v,s,y+g,c),l+=y+g,b.top=s+"px",b.left=l+v+"px",b.width=this._spacing+"px",b.height=c+"px",l+=this._spacing):(m.update(l,s+v,u,y+g),s+=y+g,b.top=s+v+"px",b.left=l+"px",b.width=u+"px",b.height=this._spacing+"px",s+=this._spacing)}}}},t}(F);(fe=ve||(ve={})).getStretch=function(e){return pe.stretchProperty.get(e)},fe.setStretch=function(e,t){pe.stretchProperty.set(e,t)},function(e){e.stretchProperty=new a.AttachedProperty({name:"stretch",create:function(){return 0},coerce:function(e,t){return Math.max(0,Math.floor(t))},changed:function(e){e.parent&&e.parent.layout instanceof ve&&e.parent.fit()}}),e.createSizer=function(e){var t=new M;return t.sizeHint=Math.floor(e),t},e.createHandle=function(e){var t=e.createHandle();return t.style.position="absolute",t},e.clampSpacing=function(e){return Math.max(0,Math.floor(e))},e.averageSize=function(e){return e.reduce((function(e,t){return e+t.size}),0)/e.length||0},e.normalize=function(e){var t=e.length;if(0===t)return[];var n=e.reduce((function(e,t){return e+Math.abs(t)}),0);return 0===n?e.map((function(e){return 1/t})):e.map((function(e){return e/n}))}}(pe||(pe={}));var me,ye=function(e){function t(t){void 0===t&&(t={});var n=e.call(this,{layout:me.createLayout(t)})||this;return n._pressData=null,n.addClass("lm-SplitPanel"),n.addClass("p-SplitPanel"),n}return T(t,e),t.prototype.dispose=function(){this._releaseMouse(),e.prototype.dispose.call(this)},Object.defineProperty(t.prototype,"orientation",{get:function(){return this.layout.orientation},set:function(e){this.layout.orientation=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"alignment",{get:function(){return this.layout.alignment},set:function(e){this.layout.alignment=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"spacing",{get:function(){return this.layout.spacing},set:function(e){this.layout.spacing=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"renderer",{get:function(){return this.layout.renderer},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"handles",{get:function(){return this.layout.handles},enumerable:!0,configurable:!0}),t.prototype.relativeSizes=function(){return this.layout.relativeSizes()},t.prototype.setRelativeSizes=function(e){this.layout.setRelativeSizes(e)},t.prototype.handleEvent=function(e){switch(e.type){case"mousedown":this._evtMouseDown(e);break;case"mousemove":this._evtMouseMove(e);break;case"mouseup":this._evtMouseUp(e);break;case"keydown":this._evtKeyDown(e);break;case"contextmenu":e.preventDefault(),e.stopPropagation()}},t.prototype.onBeforeAttach=function(e){this.node.addEventListener("mousedown",this)},t.prototype.onAfterDetach=function(e){this.node.removeEventListener("mousedown",this),this._releaseMouse()},t.prototype.onChildAdded=function(e){e.child.addClass("lm-SplitPanel-child"),e.child.addClass("p-SplitPanel-child"),this._releaseMouse()},t.prototype.onChildRemoved=function(e){e.child.removeClass("lm-SplitPanel-child"),e.child.removeClass("p-SplitPanel-child"),this._releaseMouse()},t.prototype._evtKeyDown=function(e){e.preventDefault(),e.stopPropagation(),27===e.keyCode&&this._releaseMouse()},t.prototype._evtMouseDown=function(e){if(0===e.button){var t=this.layout,n=r.ArrayExt.findFirstIndex(t.handles,(function(t){return t.contains(e.target)}));if(-1!==n){var i;e.preventDefault(),e.stopPropagation(),document.addEventListener("mouseup",this,!0),document.addEventListener("mousemove",this,!0),document.addEventListener("keydown",this,!0),document.addEventListener("contextmenu",this,!0);var o=t.handles[n],a=o.getBoundingClientRect();i="horizontal"===t.orientation?e.clientX-a.left:e.clientY-a.top;var s=window.getComputedStyle(o),l=x.overrideCursor(s.cursor);this._pressData={index:n,delta:i,override:l}}}},t.prototype._evtMouseMove=function(e){var t;e.preventDefault(),e.stopPropagation();var n=this.layout,r=this.node.getBoundingClientRect();t="horizontal"===n.orientation?e.clientX-r.left-this._pressData.delta:e.clientY-r.top-this._pressData.delta,n.moveHandle(this._pressData.index,t)},t.prototype._evtMouseUp=function(e){0===e.button&&(e.preventDefault(),e.stopPropagation(),this._releaseMouse())},t.prototype._releaseMouse=function(){this._pressData&&(this._pressData.override.dispose(),this._pressData=null,document.removeEventListener("mouseup",this,!0),document.removeEventListener("mousemove",this,!0),document.removeEventListener("keydown",this,!0),document.removeEventListener("contextmenu",this,!0))},t}(U);!function(e){var t=function(){function e(){}return e.prototype.createHandle=function(){var e=document.createElement("div");return e.className="lm-SplitPanel-handle",e.classList.add("p-SplitPanel-handle"),e},e}();e.Renderer=t,e.defaultRenderer=new t,e.getStretch=function(e){return ve.getStretch(e)},e.setStretch=function(e,t){ve.setStretch(e,t)}}(ye||(ye={})),function(e){e.createLayout=function(e){return e.layout||new ve({renderer:e.renderer||ye.defaultRenderer,orientation:e.orientation,alignment:e.alignment,spacing:e.spacing})}}(me||(me={}));var be,Ce=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t._dirty=!1,t._items=[],t._box=null,t}return T(t,e),t.prototype.dispose=function(){Object(r.each)(this._items,(function(e){e.dispose()})),this._box=null,this._items.length=0,e.prototype.dispose.call(this)},t.prototype.attachWidget=function(e,t){r.ArrayExt.insert(this._items,e,new j(t)),this.parent.isAttached&&o.c.sendMessage(t,L.Msg.BeforeAttach),this.parent.node.appendChild(t.node),this.parent.isAttached&&o.c.sendMessage(t,L.Msg.AfterAttach),this.parent.fit()},t.prototype.moveWidget=function(e,t,n){r.ArrayExt.move(this._items,e,t),this.parent.update()},t.prototype.detachWidget=function(e,t){var n=r.ArrayExt.removeAt(this._items,e);this.parent.isAttached&&o.c.sendMessage(t,L.Msg.BeforeDetach),this.parent.node.removeChild(t.node),this.parent.isAttached&&o.c.sendMessage(t,L.Msg.AfterDetach),n.widget.node.style.zIndex="",n.dispose(),this.parent.fit()},t.prototype.onBeforeShow=function(t){e.prototype.onBeforeShow.call(this,t),this.parent.update()},t.prototype.onBeforeAttach=function(t){e.prototype.onBeforeAttach.call(this,t),this.parent.fit()},t.prototype.onChildShown=function(e){this.parent.fit()},t.prototype.onChildHidden=function(e){this.parent.fit()},t.prototype.onResize=function(e){this.parent.isVisible&&this._update(e.width,e.height)},t.prototype.onUpdateRequest=function(e){this.parent.isVisible&&this._update(-1,-1)},t.prototype.onFitRequest=function(e){this.parent.isAttached&&this._fit()},t.prototype._fit=function(){for(var e=0,t=0,n=0,r=this._items.length;n<r;++n){var a=this._items[n];a.isHidden||(a.fit(),e=Math.max(e,a.minWidth),t=Math.max(t,a.minHeight))}var s=this._box=i.a.boxSizing(this.parent.node);e+=s.horizontalSum,t+=s.verticalSum;var l=this.parent.node.style;l.minWidth=e+"px",l.minHeight=t+"px",this._dirty=!0,this.parent.parent&&o.c.sendMessage(this.parent.parent,L.Msg.FitRequest),this._dirty&&o.c.sendMessage(this.parent,L.Msg.UpdateRequest)},t.prototype._update=function(e,t){this._dirty=!1;for(var n=0,r=0,o=this._items.length;r<o;++r)n+=+!this._items[r].isHidden;if(0!==n){e<0&&(e=this.parent.node.offsetWidth),t<0&&(t=this.parent.node.offsetHeight),this._box||(this._box=i.a.boxSizing(this.parent.node));var a=this._box.paddingTop,s=this._box.paddingLeft,l=e-this._box.horizontalSum,u=t-this._box.verticalSum;for(r=0,o=this._items.length;r<o;++r){var c=this._items[r];c.isHidden||(c.widget.node.style.zIndex=""+r,c.update(s,a,l,u))}}},t}(F),Ae=function(e){function t(t){void 0===t&&(t={});var n=e.call(this,{layout:be.createLayout(t)})||this;return n._widgetRemoved=new s.Signal(n),n.addClass("lm-StackedPanel"),n.addClass("p-StackedPanel"),n}return T(t,e),Object.defineProperty(t.prototype,"widgetRemoved",{get:function(){return this._widgetRemoved},enumerable:!0,configurable:!0}),t.prototype.onChildAdded=function(e){e.child.addClass("lm-StackedPanel-child"),e.child.addClass("p-StackedPanel-child")},t.prototype.onChildRemoved=function(e){e.child.removeClass("lm-StackedPanel-child"),e.child.removeClass("p-StackedPanel-child"),this._widgetRemoved.emit(e.child)},t}(U);!function(e){e.createLayout=function(e){return e.layout||new Ce}}(be||(be={}));var _e,Ie=function(e){function t(t){void 0===t&&(t={});var n=e.call(this)||this;n._currentChanged=new s.Signal(n),n.addClass("lm-TabPanel"),n.addClass("p-TabPanel"),n.tabBar=new ne(t),n.tabBar.addClass("lm-TabPanel-tabBar"),n.stackedPanel=new Ae,n.stackedPanel.addClass("lm-TabPanel-stackedPanel"),n.tabBar.addClass("p-TabPanel-tabBar"),n.stackedPanel.addClass("p-TabPanel-stackedPanel"),n.tabBar.tabMoved.connect(n._onTabMoved,n),n.tabBar.currentChanged.connect(n._onCurrentChanged,n),n.tabBar.tabCloseRequested.connect(n._onTabCloseRequested,n),n.tabBar.tabActivateRequested.connect(n._onTabActivateRequested,n),n.stackedPanel.widgetRemoved.connect(n._onWidgetRemoved,n),n._tabPlacement=t.tabPlacement||"top";var r=_e.directionFromPlacement(n._tabPlacement),i=_e.orientationFromPlacement(n._tabPlacement);n.tabBar.orientation=i,n.tabBar.dataset.placement=n._tabPlacement;var o=new B({direction:r,spacing:0});return B.setStretch(n.tabBar,0),B.setStretch(n.stackedPanel,1),o.addWidget(n.tabBar),o.addWidget(n.stackedPanel),n.layout=o,n}return T(t,e),Object.defineProperty(t.prototype,"currentChanged",{get:function(){return this._currentChanged},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"currentIndex",{get:function(){return this.tabBar.currentIndex},set:function(e){this.tabBar.currentIndex=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"currentWidget",{get:function(){var e=this.tabBar.currentTitle;return e?e.owner:null},set:function(e){this.tabBar.currentTitle=e?e.title:null},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"tabsMovable",{get:function(){return this.tabBar.tabsMovable},set:function(e){this.tabBar.tabsMovable=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"tabPlacement",{get:function(){return this._tabPlacement},set:function(e){if(this._tabPlacement!==e){this._tabPlacement=e;var t=_e.directionFromPlacement(e),n=_e.orientationFromPlacement(e);this.tabBar.orientation=n,this.tabBar.dataset.placement=e,this.layout.direction=t}},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"widgets",{get:function(){return this.stackedPanel.widgets},enumerable:!0,configurable:!0}),t.prototype.addWidget=function(e){this.insertWidget(this.widgets.length,e)},t.prototype.insertWidget=function(e,t){t!==this.currentWidget&&t.hide(),this.stackedPanel.insertWidget(e,t),this.tabBar.insertTab(e,t.title)},t.prototype._onCurrentChanged=function(e,t){var n=t.previousIndex,r=t.previousTitle,a=t.currentIndex,s=t.currentTitle,l=r?r.owner:null,u=s?s.owner:null;l&&l.hide(),u&&u.show(),this._currentChanged.emit({previousIndex:n,previousWidget:l,currentIndex:a,currentWidget:u}),(i.b.IS_EDGE||i.b.IS_IE)&&o.c.flush()},t.prototype._onTabActivateRequested=function(e,t){t.title.owner.activate()},t.prototype._onTabCloseRequested=function(e,t){t.title.owner.close()},t.prototype._onTabMoved=function(e,t){this.stackedPanel.insertWidget(t.toIndex,t.title.owner)},t.prototype._onWidgetRemoved=function(e,t){this.tabBar.removeTab(t.title)},t}(L);!function(e){e.orientationFromPlacement=function(e){return t[e]},e.directionFromPlacement=function(e){return n[e]};var t={top:"horizontal",left:"vertical",right:"vertical",bottom:"horizontal"},n={top:"top-to-bottom",left:"left-to-right",right:"right-to-left",bottom:"bottom-to-top"}}(_e||(_e={}))},function(e,t,n){"use strict";e.exports=function(e,t,n,r,i,o,a,s){if(!e){var l;if(void 0===t)l=new Error("Minified exception occurred; use the non-minified dev environment for the full error message and additional helpful warnings.");else{var u=[n,r,i,o,a,s],c=0;(l=new Error(t.replace(/%s/g,(function(){return u[c++]})))).name="Invariant Violation"}throw l.framesToPop=1,l}}},function(e,t,n){"use strict";n.r(t),n.d(t,"ACESFilmicToneMapping",(function(){return oe})),n.d(t,"AddEquation",(function(){return S})),n.d(t,"AddOperation",(function(){return $})),n.d(t,"AdditiveAnimationBlendMode",(function(){return Zt})),n.d(t,"AdditiveBlending",(function(){return I})),n.d(t,"AlphaFormat",(function(){return Fe})),n.d(t,"AlwaysDepth",(function(){return Y})),n.d(t,"AlwaysStencilFunc",(function(){return Sn})),n.d(t,"AmbientLight",(function(){return _d})),n.d(t,"AmbientLightProbe",(function(){return Wd})),n.d(t,"AnimationClip",(function(){return Ec})),n.d(t,"AnimationLoader",(function(){return Rc})),n.d(t,"AnimationMixer",(function(){return Ah})),n.d(t,"AnimationObjectGroup",(function(){return bh})),n.d(t,"AnimationUtils",(function(){return vc})),n.d(t,"ArcCurve",(function(){return Wc})),n.d(t,"ArrayCamera",(function(){return Os})),n.d(t,"ArrowHelper",(function(){return cf})),n.d(t,"Audio",(function(){return th})),n.d(t,"AudioAnalyser",(function(){return sh})),n.d(t,"AudioContext",(function(){return Ud})),n.d(t,"AudioListener",(function(){return eh})),n.d(t,"AudioLoader",(function(){return Vd})),n.d(t,"AxesHelper",(function(){return df})),n.d(t,"AxisHelper",(function(){return ap})),n.d(t,"BackSide",(function(){return m})),n.d(t,"BasicDepthPacking",(function(){return ln})),n.d(t,"BasicShadowMap",(function(){return h})),n.d(t,"BinaryTextureLoader",(function(){return dp})),n.d(t,"Bone",(function(){return gl})),n.d(t,"BooleanKeyframeTrack",(function(){return _c})),n.d(t,"BoundingBoxHelper",(function(){return sp})),n.d(t,"Box2",(function(){return Oh})),n.d(t,"Box3",(function(){return Br})),n.d(t,"Box3Helper",(function(){return of})),n.d(t,"BoxBufferGeometry",(function(){return yo})),n.d(t,"BoxGeometry",(function(){return mo})),n.d(t,"BoxHelper",(function(){return rf})),n.d(t,"BufferAttribute",(function(){return wi})),n.d(t,"BufferGeometry",(function(){return Yi})),n.d(t,"BufferGeometryLoader",(function(){return Od})),n.d(t,"ByteType",(function(){return Te})),n.d(t,"Cache",(function(){return kc})),n.d(t,"Camera",(function(){return Io})),n.d(t,"CameraHelper",(function(){return ef})),n.d(t,"CanvasRenderer",(function(){return pp})),n.d(t,"CanvasTexture",(function(){return Hl})),n.d(t,"CatmullRomCurve3",(function(){return Kc})),n.d(t,"CineonToneMapping",(function(){return ie})),n.d(t,"CircleBufferGeometry",(function(){return rc})),n.d(t,"CircleGeometry",(function(){return nc})),n.d(t,"ClampToEdgeWrapping",(function(){return ge})),n.d(t,"Clock",(function(){return Zd})),n.d(t,"ClosedSplineCurve3",(function(){return rp})),n.d(t,"Color",(function(){return vi})),n.d(t,"ColorKeyframeTrack",(function(){return Ic})),n.d(t,"CompressedTexture",(function(){return Bl})),n.d(t,"CompressedTextureLoader",(function(){return zc})),n.d(t,"ConeBufferGeometry",(function(){return tc})),n.d(t,"ConeGeometry",(function(){return ec})),n.d(t,"CubeCamera",(function(){return wo})),n.d(t,"CubeGeometry",(function(){return mo})),n.d(t,"CubeReflectionMapping",(function(){return se})),n.d(t,"CubeRefractionMapping",(function(){return le})),n.d(t,"CubeTexture",(function(){return Zo})),n.d(t,"CubeTextureLoader",(function(){return Hc})),n.d(t,"CubeUVReflectionMapping",(function(){return he})),n.d(t,"CubeUVRefractionMapping",(function(){return fe})),n.d(t,"CubicBezierCurve",(function(){return td})),n.d(t,"CubicBezierCurve3",(function(){return nd})),n.d(t,"CubicInterpolant",(function(){return yc})),n.d(t,"CullFaceBack",(function(){return s})),n.d(t,"CullFaceFront",(function(){return l})),n.d(t,"CullFaceFrontBack",(function(){return u})),n.d(t,"CullFaceNone",(function(){return a})),n.d(t,"Curve",(function(){return Vc})),n.d(t,"CurvePath",(function(){return ud})),n.d(t,"CustomBlending",(function(){return M})),n.d(t,"CylinderBufferGeometry",(function(){return $u})),n.d(t,"CylinderGeometry",(function(){return Ju})),n.d(t,"Cylindrical",(function(){return Th})),n.d(t,"DataTexture",(function(){return So})),n.d(t,"DataTexture2DArray",(function(){return Qo})),n.d(t,"DataTexture3D",(function(){return Ko})),n.d(t,"DataTextureLoader",(function(){return Fc})),n.d(t,"DecrementStencilOp",(function(){return vn})),n.d(t,"DecrementWrapStencilOp",(function(){return yn})),n.d(t,"DefaultLoadingManager",(function(){return Dc})),n.d(t,"DepthFormat",(function(){return We})),n.d(t,"DepthStencilFormat",(function(){return Ge})),n.d(t,"DepthTexture",(function(){return Ul})),n.d(t,"DirectionalLight",(function(){return Ad})),n.d(t,"DirectionalLightHelper",(function(){return Kh})),n.d(t,"DirectionalLightShadow",(function(){return Cd})),n.d(t,"DiscreteInterpolant",(function(){return Cc})),n.d(t,"DodecahedronBufferGeometry",(function(){return tu})),n.d(t,"DodecahedronGeometry",(function(){return eu})),n.d(t,"DoubleSide",(function(){return y})),n.d(t,"DstAlphaFactor",(function(){return z})),n.d(t,"DstColorFactor",(function(){return B})),n.d(t,"DynamicBufferAttribute",(function(){return Xf})),n.d(t,"DynamicCopyUsage",(function(){return Nn})),n.d(t,"DynamicDrawUsage",(function(){return En})),n.d(t,"DynamicReadUsage",(function(){return Ln})),n.d(t,"EdgesGeometry",(function(){return Ku})),n.d(t,"EdgesHelper",(function(){return lp})),n.d(t,"EllipseCurve",(function(){return Yc})),n.d(t,"EqualDepth",(function(){return X})),n.d(t,"EqualStencilFunc",(function(){return _n})),n.d(t,"EquirectangularReflectionMapping",(function(){return ue})),n.d(t,"EquirectangularRefractionMapping",(function(){return ce})),n.d(t,"Euler",(function(){return cr})),n.d(t,"EventDispatcher",(function(){return Rn})),n.d(t,"ExtrudeBufferGeometry",(function(){return Ru})),n.d(t,"ExtrudeGeometry",(function(){return ju})),n.d(t,"Face3",(function(){return Ci})),n.d(t,"Face4",(function(){return Lf})),n.d(t,"FaceColors",(function(){return jf})),n.d(t,"FileLoader",(function(){return jc})),n.d(t,"FlatShading",(function(){return b})),n.d(t,"Float32Attribute",(function(){return tp})),n.d(t,"Float32BufferAttribute",(function(){return Di})),n.d(t,"Float64Attribute",(function(){return np})),n.d(t,"Float64BufferAttribute",(function(){return Pi})),n.d(t,"FloatType",(function(){return De})),n.d(t,"Fog",(function(){return Rs})),n.d(t,"FogExp2",(function(){return js})),n.d(t,"Font",(function(){return Fd})),n.d(t,"FontLoader",(function(){return Hd})),n.d(t,"FrontFaceDirectionCCW",(function(){return d})),n.d(t,"FrontFaceDirectionCW",(function(){return c})),n.d(t,"FrontSide",(function(){return v})),n.d(t,"Frustum",(function(){return Oo})),n.d(t,"GammaEncoding",(function(){return tn})),n.d(t,"Geometry",(function(){return vo})),n.d(t,"GeometryUtils",(function(){return fp})),n.d(t,"GreaterDepth",(function(){return Z})),n.d(t,"GreaterEqualDepth",(function(){return q})),n.d(t,"GreaterEqualStencilFunc",(function(){return Mn})),n.d(t,"GreaterStencilFunc",(function(){return xn})),n.d(t,"GridHelper",(function(){return Gh})),n.d(t,"Group",(function(){return ks})),n.d(t,"HalfFloatType",(function(){return Pe})),n.d(t,"HemisphereLight",(function(){return fd})),n.d(t,"HemisphereLightHelper",(function(){return Wh})),n.d(t,"HemisphereLightProbe",(function(){return Yd})),n.d(t,"IcosahedronBufferGeometry",(function(){return $l})),n.d(t,"IcosahedronGeometry",(function(){return Jl})),n.d(t,"ImageBitmapLoader",(function(){return Rd})),n.d(t,"ImageLoader",(function(){return Bc})),n.d(t,"ImageUtils",(function(){return Yn})),n.d(t,"ImmediateRenderObject",(function(){return Ph})),n.d(t,"IncrementStencilOp",(function(){return gn})),n.d(t,"IncrementWrapStencilOp",(function(){return mn})),n.d(t,"InstancedBufferAttribute",(function(){return Ed})),n.d(t,"InstancedBufferGeometry",(function(){return Td})),n.d(t,"InstancedInterleavedBuffer",(function(){return Ih})),n.d(t,"InstancedMesh",(function(){return Cl})),n.d(t,"Int16Attribute",(function(){return Kf})),n.d(t,"Int16BufferAttribute",(function(){return Ei})),n.d(t,"Int32Attribute",(function(){return $f})),n.d(t,"Int32BufferAttribute",(function(){return ki})),n.d(t,"Int8Attribute",(function(){return qf})),n.d(t,"Int8BufferAttribute",(function(){return Mi})),n.d(t,"IntType",(function(){return ke})),n.d(t,"InterleavedBuffer",(function(){return zs})),n.d(t,"InterleavedBufferAttribute",(function(){return Hs})),n.d(t,"Interpolant",(function(){return mc})),n.d(t,"InterpolateDiscrete",(function(){return Ut})),n.d(t,"InterpolateLinear",(function(){return Vt})),n.d(t,"InterpolateSmooth",(function(){return Yt})),n.d(t,"InvertStencilOp",(function(){return bn})),n.d(t,"JSONLoader",(function(){return gp})),n.d(t,"KeepStencilOp",(function(){return fn})),n.d(t,"KeyframeTrack",(function(){return Ac})),n.d(t,"LOD",(function(){return cl})),n.d(t,"LatheBufferGeometry",(function(){return Xu})),n.d(t,"LatheGeometry",(function(){return Gu})),n.d(t,"Layers",(function(){return dr})),n.d(t,"LensFlare",(function(){return mp})),n.d(t,"LessDepth",(function(){return W})),n.d(t,"LessEqualDepth",(function(){return G})),n.d(t,"LessEqualStencilFunc",(function(){return In})),n.d(t,"LessStencilFunc",(function(){return An})),n.d(t,"Light",(function(){return hd})),n.d(t,"LightProbe",(function(){return wd})),n.d(t,"LightShadow",(function(){return pd})),n.d(t,"Line",(function(){return Sl})),n.d(t,"Line3",(function(){return Dh})),n.d(t,"LineBasicMaterial",(function(){return Al})),n.d(t,"LineCurve",(function(){return rd})),n.d(t,"LineCurve3",(function(){return id})),n.d(t,"LineDashedMaterial",(function(){return pc})),n.d(t,"LineLoop",(function(){return kl})),n.d(t,"LinePieces",(function(){return Pf})),n.d(t,"LineSegments",(function(){return Ol})),n.d(t,"LineStrip",(function(){return Df})),n.d(t,"LinearEncoding",(function(){return $t})),n.d(t,"LinearFilter",(function(){return _e})),n.d(t,"LinearInterpolant",(function(){return bc})),n.d(t,"LinearMipMapLinearFilter",(function(){return Me})),n.d(t,"LinearMipMapNearestFilter",(function(){return xe})),n.d(t,"LinearMipmapLinearFilter",(function(){return we})),n.d(t,"LinearMipmapNearestFilter",(function(){return Ie})),n.d(t,"LinearToneMapping",(function(){return te})),n.d(t,"Loader",(function(){return Pc})),n.d(t,"LoaderUtils",(function(){return Sd})),n.d(t,"LoadingManager",(function(){return Lc})),n.d(t,"LogLuvEncoding",(function(){return rn})),n.d(t,"LoopOnce",(function(){return Ft})),n.d(t,"LoopPingPong",(function(){return Ht})),n.d(t,"LoopRepeat",(function(){return Bt})),n.d(t,"LuminanceAlphaFormat",(function(){return Ve})),n.d(t,"LuminanceFormat",(function(){return Ue})),n.d(t,"MOUSE",(function(){return i})),n.d(t,"Material",(function(){return _i})),n.d(t,"MaterialLoader",(function(){return Md})),n.d(t,"Math",(function(){return Hn})),n.d(t,"MathUtils",(function(){return Hn})),n.d(t,"Matrix3",(function(){return Vn})),n.d(t,"Matrix4",(function(){return sr})),n.d(t,"MaxEquation",(function(){return k})),n.d(t,"Mesh",(function(){return lo})),n.d(t,"MeshBasicMaterial",(function(){return Ii})),n.d(t,"MeshDepthMaterial",(function(){return xs})),n.d(t,"MeshDistanceMaterial",(function(){return ws})),n.d(t,"MeshFaceMaterial",(function(){return zf})),n.d(t,"MeshLambertMaterial",(function(){return hc})),n.d(t,"MeshMatcapMaterial",(function(){return fc})),n.d(t,"MeshNormalMaterial",(function(){return dc})),n.d(t,"MeshPhongMaterial",(function(){return uc})),n.d(t,"MeshPhysicalMaterial",(function(){return lc})),n.d(t,"MeshStandardMaterial",(function(){return sc})),n.d(t,"MeshToonMaterial",(function(){return cc})),n.d(t,"MinEquation",(function(){return O})),n.d(t,"MirroredRepeatWrapping",(function(){return ve})),n.d(t,"MixOperation",(function(){return J})),n.d(t,"MultiMaterial",(function(){return Ff})),n.d(t,"MultiplyBlending",(function(){return w})),n.d(t,"MultiplyOperation",(function(){return K})),n.d(t,"NearestFilter",(function(){return me})),n.d(t,"NearestMipMapLinearFilter",(function(){return Ae})),n.d(t,"NearestMipMapNearestFilter",(function(){return be})),n.d(t,"NearestMipmapLinearFilter",(function(){return Ce})),n.d(t,"NearestMipmapNearestFilter",(function(){return ye})),n.d(t,"NeverDepth",(function(){return V})),n.d(t,"NeverStencilFunc",(function(){return Cn})),n.d(t,"NoBlending",(function(){return A})),n.d(t,"NoColors",(function(){return Nf})),n.d(t,"NoToneMapping",(function(){return ee})),n.d(t,"NormalAnimationBlendMode",(function(){return qt})),n.d(t,"NormalBlending",(function(){return _})),n.d(t,"NotEqualDepth",(function(){return Q})),n.d(t,"NotEqualStencilFunc",(function(){return wn})),n.d(t,"NumberKeyframeTrack",(function(){return xc})),n.d(t,"Object3D",(function(){return wr})),n.d(t,"ObjectLoader",(function(){return Ld})),n.d(t,"ObjectSpaceNormalMap",(function(){return dn})),n.d(t,"OctahedronBufferGeometry",(function(){return Kl})),n.d(t,"OctahedronGeometry",(function(){return Ql})),n.d(t,"OneFactor",(function(){return D})),n.d(t,"OneMinusDstAlphaFactor",(function(){return F})),n.d(t,"OneMinusDstColorFactor",(function(){return H})),n.d(t,"OneMinusSrcAlphaFactor",(function(){return R})),n.d(t,"OneMinusSrcColorFactor",(function(){return N})),n.d(t,"OrthographicCamera",(function(){return bd})),n.d(t,"PCFShadowMap",(function(){return f})),n.d(t,"PCFSoftShadowMap",(function(){return p})),n.d(t,"PMREMGenerator",(function(){return xf})),n.d(t,"ParametricBufferGeometry",(function(){return Wl})),n.d(t,"ParametricGeometry",(function(){return Yl})),n.d(t,"Particle",(function(){return Hf})),n.d(t,"ParticleBasicMaterial",(function(){return Yf})),n.d(t,"ParticleSystem",(function(){return Uf})),n.d(t,"ParticleSystemMaterial",(function(){return Wf})),n.d(t,"Path",(function(){return cd})),n.d(t,"PerspectiveCamera",(function(){return xo})),n.d(t,"Plane",(function(){return ti})),n.d(t,"PlaneBufferGeometry",(function(){return No})),n.d(t,"PlaneGeometry",(function(){return Po})),n.d(t,"PlaneHelper",(function(){return af})),n.d(t,"PointCloud",(function(){return Bf})),n.d(t,"PointCloudMaterial",(function(){return Vf})),n.d(t,"PointLight",(function(){return yd})),n.d(t,"PointLightHelper",(function(){return Hh})),n.d(t,"Points",(function(){return Rl})),n.d(t,"PointsMaterial",(function(){return Ll})),n.d(t,"PolarGridHelper",(function(){return Xh})),n.d(t,"PolyhedronBufferGeometry",(function(){return Xl})),n.d(t,"PolyhedronGeometry",(function(){return Gl})),n.d(t,"PositionalAudio",(function(){return ah})),n.d(t,"PropertyBinding",(function(){return yh})),n.d(t,"PropertyMixer",(function(){return lh})),n.d(t,"QuadraticBezierCurve",(function(){return od})),n.d(t,"QuadraticBezierCurve3",(function(){return ad})),n.d(t,"Quaternion",(function(){return Qn})),n.d(t,"QuaternionKeyframeTrack",(function(){return Mc})),n.d(t,"QuaternionLinearInterpolant",(function(){return wc})),n.d(t,"REVISION",(function(){return r})),n.d(t,"RGBADepthPacking",(function(){return un})),n.d(t,"RGBAFormat",(function(){return He})),n.d(t,"RGBAIntegerFormat",(function(){return Je})),n.d(t,"RGBA_ASTC_10x10_Format",(function(){return At})),n.d(t,"RGBA_ASTC_10x5_Format",(function(){return yt})),n.d(t,"RGBA_ASTC_10x6_Format",(function(){return bt})),n.d(t,"RGBA_ASTC_10x8_Format",(function(){return Ct})),n.d(t,"RGBA_ASTC_12x10_Format",(function(){return _t})),n.d(t,"RGBA_ASTC_12x12_Format",(function(){return It})),n.d(t,"RGBA_ASTC_4x4_Format",(function(){return ct})),n.d(t,"RGBA_ASTC_5x4_Format",(function(){return dt})),n.d(t,"RGBA_ASTC_5x5_Format",(function(){return ht})),n.d(t,"RGBA_ASTC_6x5_Format",(function(){return ft})),n.d(t,"RGBA_ASTC_6x6_Format",(function(){return pt})),n.d(t,"RGBA_ASTC_8x5_Format",(function(){return gt})),n.d(t,"RGBA_ASTC_8x6_Format",(function(){return vt})),n.d(t,"RGBA_ASTC_8x8_Format",(function(){return mt})),n.d(t,"RGBA_BPTC_Format",(function(){return xt})),n.d(t,"RGBA_ETC2_EAC_Format",(function(){return ut})),n.d(t,"RGBA_PVRTC_2BPPV1_Format",(function(){return at})),n.d(t,"RGBA_PVRTC_4BPPV1_Format",(function(){return ot})),n.d(t,"RGBA_S3TC_DXT1_Format",(function(){return et})),n.d(t,"RGBA_S3TC_DXT3_Format",(function(){return tt})),n.d(t,"RGBA_S3TC_DXT5_Format",(function(){return nt})),n.d(t,"RGBDEncoding",(function(){return sn})),n.d(t,"RGBEEncoding",(function(){return nn})),n.d(t,"RGBEFormat",(function(){return Ye})),n.d(t,"RGBFormat",(function(){return Be})),n.d(t,"RGBIntegerFormat",(function(){return Ke})),n.d(t,"RGBM16Encoding",(function(){return an})),n.d(t,"RGBM7Encoding",(function(){return on})),n.d(t,"RGB_ETC1_Format",(function(){return st})),n.d(t,"RGB_ETC2_Format",(function(){return lt})),n.d(t,"RGB_PVRTC_2BPPV1_Format",(function(){return it})),n.d(t,"RGB_PVRTC_4BPPV1_Format",(function(){return rt})),n.d(t,"RGB_S3TC_DXT1_Format",(function(){return $e})),n.d(t,"RGFormat",(function(){return Ze})),n.d(t,"RGIntegerFormat",(function(){return Qe})),n.d(t,"RawShaderMaterial",(function(){return ac})),n.d(t,"Ray",(function(){return Kr})),n.d(t,"Raycaster",(function(){return xh})),n.d(t,"RectAreaLight",(function(){return Id})),n.d(t,"RedFormat",(function(){return Xe})),n.d(t,"RedIntegerFormat",(function(){return qe})),n.d(t,"ReinhardToneMapping",(function(){return ne})),n.d(t,"RepeatWrapping",(function(){return pe})),n.d(t,"ReplaceStencilOp",(function(){return pn})),n.d(t,"ReverseSubtractEquation",(function(){return E})),n.d(t,"RingBufferGeometry",(function(){return Wu})),n.d(t,"RingGeometry",(function(){return Yu})),n.d(t,"SRGB8_ALPHA8_ASTC_10x10_Format",(function(){return jt})),n.d(t,"SRGB8_ALPHA8_ASTC_10x5_Format",(function(){return Dt})),n.d(t,"SRGB8_ALPHA8_ASTC_10x6_Format",(function(){return Pt})),n.d(t,"SRGB8_ALPHA8_ASTC_10x8_Format",(function(){return Nt})),n.d(t,"SRGB8_ALPHA8_ASTC_12x10_Format",(function(){return Rt})),n.d(t,"SRGB8_ALPHA8_ASTC_12x12_Format",(function(){return zt})),n.d(t,"SRGB8_ALPHA8_ASTC_4x4_Format",(function(){return wt})),n.d(t,"SRGB8_ALPHA8_ASTC_5x4_Format",(function(){return Mt})),n.d(t,"SRGB8_ALPHA8_ASTC_5x5_Format",(function(){return St})),n.d(t,"SRGB8_ALPHA8_ASTC_6x5_Format",(function(){return Tt})),n.d(t,"SRGB8_ALPHA8_ASTC_6x6_Format",(function(){return Et})),n.d(t,"SRGB8_ALPHA8_ASTC_8x5_Format",(function(){return Ot})),n.d(t,"SRGB8_ALPHA8_ASTC_8x6_Format",(function(){return kt})),n.d(t,"SRGB8_ALPHA8_ASTC_8x8_Format",(function(){return Lt})),n.d(t,"Scene",(function(){return Mr})),n.d(t,"SceneUtils",(function(){return vp})),n.d(t,"ShaderChunk",(function(){return jo})),n.d(t,"ShaderLib",(function(){return Ro})),n.d(t,"ShaderMaterial",(function(){return _o})),n.d(t,"ShadowMaterial",(function(){return oc})),n.d(t,"Shape",(function(){return dd})),n.d(t,"ShapeBufferGeometry",(function(){return Zu})),n.d(t,"ShapeGeometry",(function(){return qu})),n.d(t,"ShapePath",(function(){return zd})),n.d(t,"ShapeUtils",(function(){return Du})),n.d(t,"ShortType",(function(){return Ee})),n.d(t,"Skeleton",(function(){return pl})),n.d(t,"SkeletonHelper",(function(){return Bh})),n.d(t,"SkinnedMesh",(function(){return dl})),n.d(t,"SmoothShading",(function(){return C})),n.d(t,"Sphere",(function(){return Vr})),n.d(t,"SphereBufferGeometry",(function(){return Vu})),n.d(t,"SphereGeometry",(function(){return Uu})),n.d(t,"Spherical",(function(){return Sh})),n.d(t,"SphericalHarmonics3",(function(){return xd})),n.d(t,"SphericalReflectionMapping",(function(){return de})),n.d(t,"Spline",(function(){return op})),n.d(t,"SplineCurve",(function(){return sd})),n.d(t,"SplineCurve3",(function(){return ip})),n.d(t,"SpotLight",(function(){return vd})),n.d(t,"SpotLightHelper",(function(){return jh})),n.d(t,"SpotLightShadow",(function(){return gd})),n.d(t,"Sprite",(function(){return tl})),n.d(t,"SpriteMaterial",(function(){return Us})),n.d(t,"SrcAlphaFactor",(function(){return j})),n.d(t,"SrcAlphaSaturateFactor",(function(){return U})),n.d(t,"SrcColorFactor",(function(){return P})),n.d(t,"StaticCopyUsage",(function(){return Pn})),n.d(t,"StaticDrawUsage",(function(){return Tn})),n.d(t,"StaticReadUsage",(function(){return kn})),n.d(t,"StereoCamera",(function(){return qd})),n.d(t,"StreamCopyUsage",(function(){return jn})),n.d(t,"StreamDrawUsage",(function(){return On})),n.d(t,"StreamReadUsage",(function(){return Dn})),n.d(t,"StringKeyframeTrack",(function(){return Sc})),n.d(t,"SubtractEquation",(function(){return T})),n.d(t,"SubtractiveBlending",(function(){return x})),n.d(t,"TOUCH",(function(){return o})),n.d(t,"TangentSpaceNormalMap",(function(){return cn})),n.d(t,"TetrahedronBufferGeometry",(function(){return Zl})),n.d(t,"TetrahedronGeometry",(function(){return ql})),n.d(t,"TextBufferGeometry",(function(){return Hu})),n.d(t,"TextGeometry",(function(){return Bu})),n.d(t,"Texture",(function(){return Gn})),n.d(t,"TextureLoader",(function(){return Uc})),n.d(t,"TorusBufferGeometry",(function(){return su})),n.d(t,"TorusGeometry",(function(){return au})),n.d(t,"TorusKnotBufferGeometry",(function(){return ou})),n.d(t,"TorusKnotGeometry",(function(){return iu})),n.d(t,"Triangle",(function(){return hi})),n.d(t,"TriangleFanDrawMode",(function(){return Jt})),n.d(t,"TriangleStripDrawMode",(function(){return Kt})),n.d(t,"TrianglesDrawMode",(function(){return Qt})),n.d(t,"TubeBufferGeometry",(function(){return ru})),n.d(t,"TubeGeometry",(function(){return nu})),n.d(t,"UVMapping",(function(){return ae})),n.d(t,"Uint16Attribute",(function(){return Jf})),n.d(t,"Uint16BufferAttribute",(function(){return Oi})),n.d(t,"Uint32Attribute",(function(){return ep})),n.d(t,"Uint32BufferAttribute",(function(){return Li})),n.d(t,"Uint8Attribute",(function(){return Zf})),n.d(t,"Uint8BufferAttribute",(function(){return Si})),n.d(t,"Uint8ClampedAttribute",(function(){return Qf})),n.d(t,"Uint8ClampedBufferAttribute",(function(){return Ti})),n.d(t,"Uncharted2ToneMapping",(function(){return re})),n.d(t,"Uniform",(function(){return _h})),n.d(t,"UniformsLib",(function(){return ko})),n.d(t,"UniformsUtils",(function(){return Ao})),n.d(t,"UnsignedByteType",(function(){return Se})),n.d(t,"UnsignedInt248Type",(function(){return ze})),n.d(t,"UnsignedIntType",(function(){return Le})),n.d(t,"UnsignedShort4444Type",(function(){return Ne})),n.d(t,"UnsignedShort5551Type",(function(){return je})),n.d(t,"UnsignedShort565Type",(function(){return Re})),n.d(t,"UnsignedShortType",(function(){return Oe})),n.d(t,"VSMShadowMap",(function(){return g})),n.d(t,"Vector2",(function(){return Un})),n.d(t,"Vector3",(function(){return $n})),n.d(t,"Vector4",(function(){return Xn})),n.d(t,"VectorKeyframeTrack",(function(){return Tc})),n.d(t,"Vertex",(function(){return Gf})),n.d(t,"VertexColors",(function(){return Rf})),n.d(t,"VideoTexture",(function(){return Fl})),n.d(t,"WebGLCubeRenderTarget",(function(){return Mo})),n.d(t,"WebGLMultisampleRenderTarget",(function(){return Zn})),n.d(t,"WebGLRenderTarget",(function(){return qn})),n.d(t,"WebGLRenderTargetCube",(function(){return hp})),n.d(t,"WebGLRenderer",(function(){return Ns})),n.d(t,"WebGLUtils",(function(){return Es})),n.d(t,"WireframeGeometry",(function(){return Vl})),n.d(t,"WireframeHelper",(function(){return up})),n.d(t,"WrapAroundEnding",(function(){return Xt})),n.d(t,"XHRLoader",(function(){return cp})),n.d(t,"ZeroCurvatureEnding",(function(){return Wt})),n.d(t,"ZeroFactor",(function(){return L})),n.d(t,"ZeroSlopeEnding",(function(){return Gt})),n.d(t,"ZeroStencilOp",(function(){return hn})),n.d(t,"sRGBEncoding",(function(){return en})),void 0===Number.EPSILON&&(Number.EPSILON=Math.pow(2,-52)),void 0===Number.isInteger&&(Number.isInteger=function(e){return"number"==typeof e&&isFinite(e)&&Math.floor(e)===e}),void 0===Math.sign&&(Math.sign=function(e){return e<0?-1:e>0?1:+e}),"name"in Function.prototype==!1&&Object.defineProperty(Function.prototype,"name",{get:function(){return this.toString().match(/^\s*function\s*([^\(\s]*)/)[1]}}),void 0===Object.assign&&(Object.assign=function(e){if(null==e)throw new TypeError("Cannot convert undefined or null to object");const t=Object(e);for(let e=1;e<arguments.length;e++){const n=arguments[e];if(null!=n)for(const e in n)Object.prototype.hasOwnProperty.call(n,e)&&(t[e]=n[e])}return t});const r="117",i={LEFT:0,MIDDLE:1,RIGHT:2,ROTATE:0,DOLLY:1,PAN:2},o={ROTATE:0,PAN:1,DOLLY_PAN:2,DOLLY_ROTATE:3},a=0,s=1,l=2,u=3,c=0,d=1,h=0,f=1,p=2,g=3,v=0,m=1,y=2,b=1,C=2,A=0,_=1,I=2,x=3,w=4,M=5,S=100,T=101,E=102,O=103,k=104,L=200,D=201,P=202,N=203,j=204,R=205,z=206,F=207,B=208,H=209,U=210,V=0,Y=1,W=2,G=3,X=4,q=5,Z=6,Q=7,K=0,J=1,$=2,ee=0,te=1,ne=2,re=3,ie=4,oe=5,ae=300,se=301,le=302,ue=303,ce=304,de=305,he=306,fe=307,pe=1e3,ge=1001,ve=1002,me=1003,ye=1004,be=1004,Ce=1005,Ae=1005,_e=1006,Ie=1007,xe=1007,we=1008,Me=1008,Se=1009,Te=1010,Ee=1011,Oe=1012,ke=1013,Le=1014,De=1015,Pe=1016,Ne=1017,je=1018,Re=1019,ze=1020,Fe=1021,Be=1022,He=1023,Ue=1024,Ve=1025,Ye=He,We=1026,Ge=1027,Xe=1028,qe=1029,Ze=1030,Qe=1031,Ke=1032,Je=1033,$e=33776,et=33777,tt=33778,nt=33779,rt=35840,it=35841,ot=35842,at=35843,st=36196,lt=37492,ut=37496,ct=37808,dt=37809,ht=37810,ft=37811,pt=37812,gt=37813,vt=37814,mt=37815,yt=37816,bt=37817,Ct=37818,At=37819,_t=37820,It=37821,xt=36492,wt=37840,Mt=37841,St=37842,Tt=37843,Et=37844,Ot=37845,kt=37846,Lt=37847,Dt=37848,Pt=37849,Nt=37850,jt=37851,Rt=37852,zt=37853,Ft=2200,Bt=2201,Ht=2202,Ut=2300,Vt=2301,Yt=2302,Wt=2400,Gt=2401,Xt=2402,qt=2500,Zt=2501,Qt=0,Kt=1,Jt=2,$t=3e3,en=3001,tn=3007,nn=3002,rn=3003,on=3004,an=3005,sn=3006,ln=3200,un=3201,cn=0,dn=1,hn=0,fn=7680,pn=7681,gn=7682,vn=7683,mn=34055,yn=34056,bn=5386,Cn=512,An=513,_n=514,In=515,xn=516,wn=517,Mn=518,Sn=519,Tn=35044,En=35048,On=35040,kn=35045,Ln=35049,Dn=35041,Pn=35046,Nn=35050,jn=35042;function Rn(){}Object.assign(Rn.prototype,{addEventListener:function(e,t){void 0===this._listeners&&(this._listeners={});var n=this._listeners;void 0===n[e]&&(n[e]=[]),-1===n[e].indexOf(t)&&n[e].push(t)},hasEventListener:function(e,t){if(void 0===this._listeners)return!1;var n=this._listeners;return void 0!==n[e]&&-1!==n[e].indexOf(t)},removeEventListener:function(e,t){if(void 0!==this._listeners){var n=this._listeners[e];if(void 0!==n){var r=n.indexOf(t);-1!==r&&n.splice(r,1)}}},dispatchEvent:function(e){if(void 0!==this._listeners){var t=this._listeners[e.type];if(void 0!==t){e.target=this;for(var n=t.slice(0),r=0,i=n.length;r<i;r++)n[r].call(this,e)}}}});for(var zn=[],Fn=0;Fn<256;Fn++)zn[Fn]=(Fn<16?"0":"")+Fn.toString(16);var Bn,Hn={DEG2RAD:Math.PI/180,RAD2DEG:180/Math.PI,generateUUID:function(){var e=4294967295*Math.random()|0,t=4294967295*Math.random()|0,n=4294967295*Math.random()|0,r=4294967295*Math.random()|0;return(zn[255&e]+zn[e>>8&255]+zn[e>>16&255]+zn[e>>24&255]+"-"+zn[255&t]+zn[t>>8&255]+"-"+zn[t>>16&15|64]+zn[t>>24&255]+"-"+zn[63&n|128]+zn[n>>8&255]+"-"+zn[n>>16&255]+zn[n>>24&255]+zn[255&r]+zn[r>>8&255]+zn[r>>16&255]+zn[r>>24&255]).toUpperCase()},clamp:function(e,t,n){return Math.max(t,Math.min(n,e))},euclideanModulo:function(e,t){return(e%t+t)%t},mapLinear:function(e,t,n,r,i){return r+(e-t)*(i-r)/(n-t)},lerp:function(e,t,n){return(1-n)*e+n*t},smoothstep:function(e,t,n){return e<=t?0:e>=n?1:(e=(e-t)/(n-t))*e*(3-2*e)},smootherstep:function(e,t,n){return e<=t?0:e>=n?1:(e=(e-t)/(n-t))*e*e*(e*(6*e-15)+10)},randInt:function(e,t){return e+Math.floor(Math.random()*(t-e+1))},randFloat:function(e,t){return e+Math.random()*(t-e)},randFloatSpread:function(e){return e*(.5-Math.random())},degToRad:function(e){return e*Hn.DEG2RAD},radToDeg:function(e){return e*Hn.RAD2DEG},isPowerOfTwo:function(e){return 0==(e&e-1)&&0!==e},ceilPowerOfTwo:function(e){return Math.pow(2,Math.ceil(Math.log(e)/Math.LN2))},floorPowerOfTwo:function(e){return Math.pow(2,Math.floor(Math.log(e)/Math.LN2))},setQuaternionFromProperEuler:function(e,t,n,r,i){var o=Math.cos,a=Math.sin,s=o(n/2),l=a(n/2),u=o((t+r)/2),c=a((t+r)/2),d=o((t-r)/2),h=a((t-r)/2),f=o((r-t)/2),p=a((r-t)/2);switch(i){case"XYX":e.set(s*c,l*d,l*h,s*u);break;case"YZY":e.set(l*h,s*c,l*d,s*u);break;case"ZXZ":e.set(l*d,l*h,s*c,s*u);break;case"XZX":e.set(s*c,l*p,l*f,s*u);break;case"YXY":e.set(l*f,s*c,l*p,s*u);break;case"ZYZ":e.set(l*p,l*f,s*c,s*u);break;default:console.warn("THREE.MathUtils: .setQuaternionFromProperEuler() encountered an unknown order: "+i)}}};function Un(e,t){this.x=e||0,this.y=t||0}function Vn(){this.elements=[1,0,0,0,1,0,0,0,1],arguments.length>0&&console.error("THREE.Matrix3: the constructor no longer reads arguments. use .set() instead.")}Object.defineProperties(Un.prototype,{width:{get:function(){return this.x},set:function(e){this.x=e}},height:{get:function(){return this.y},set:function(e){this.y=e}}}),Object.assign(Un.prototype,{isVector2:!0,set:function(e,t){return this.x=e,this.y=t,this},setScalar:function(e){return this.x=e,this.y=e,this},setX:function(e){return this.x=e,this},setY:function(e){return this.y=e,this},setComponent:function(e,t){switch(e){case 0:this.x=t;break;case 1:this.y=t;break;default:throw new Error("index is out of range: "+e)}return this},getComponent:function(e){switch(e){case 0:return this.x;case 1:return this.y;default:throw new Error("index is out of range: "+e)}},clone:function(){return new this.constructor(this.x,this.y)},copy:function(e){return this.x=e.x,this.y=e.y,this},add:function(e,t){return void 0!==t?(console.warn("THREE.Vector2: .add() now only accepts one argument. Use .addVectors( a, b ) instead."),this.addVectors(e,t)):(this.x+=e.x,this.y+=e.y,this)},addScalar:function(e){return this.x+=e,this.y+=e,this},addVectors:function(e,t){return this.x=e.x+t.x,this.y=e.y+t.y,this},addScaledVector:function(e,t){return this.x+=e.x*t,this.y+=e.y*t,this},sub:function(e,t){return void 0!==t?(console.warn("THREE.Vector2: .sub() now only accepts one argument. Use .subVectors( a, b ) instead."),this.subVectors(e,t)):(this.x-=e.x,this.y-=e.y,this)},subScalar:function(e){return this.x-=e,this.y-=e,this},subVectors:function(e,t){return this.x=e.x-t.x,this.y=e.y-t.y,this},multiply:function(e){return this.x*=e.x,this.y*=e.y,this},multiplyScalar:function(e){return this.x*=e,this.y*=e,this},divide:function(e){return this.x/=e.x,this.y/=e.y,this},divideScalar:function(e){return this.multiplyScalar(1/e)},applyMatrix3:function(e){var t=this.x,n=this.y,r=e.elements;return this.x=r[0]*t+r[3]*n+r[6],this.y=r[1]*t+r[4]*n+r[7],this},min:function(e){return this.x=Math.min(this.x,e.x),this.y=Math.min(this.y,e.y),this},max:function(e){return this.x=Math.max(this.x,e.x),this.y=Math.max(this.y,e.y),this},clamp:function(e,t){return this.x=Math.max(e.x,Math.min(t.x,this.x)),this.y=Math.max(e.y,Math.min(t.y,this.y)),this},clampScalar:function(e,t){return this.x=Math.max(e,Math.min(t,this.x)),this.y=Math.max(e,Math.min(t,this.y)),this},clampLength:function(e,t){var n=this.length();return this.divideScalar(n||1).multiplyScalar(Math.max(e,Math.min(t,n)))},floor:function(){return this.x=Math.floor(this.x),this.y=Math.floor(this.y),this},ceil:function(){return this.x=Math.ceil(this.x),this.y=Math.ceil(this.y),this},round:function(){return this.x=Math.round(this.x),this.y=Math.round(this.y),this},roundToZero:function(){return this.x=this.x<0?Math.ceil(this.x):Math.floor(this.x),this.y=this.y<0?Math.ceil(this.y):Math.floor(this.y),this},negate:function(){return this.x=-this.x,this.y=-this.y,this},dot:function(e){return this.x*e.x+this.y*e.y},cross:function(e){return this.x*e.y-this.y*e.x},lengthSq:function(){return this.x*this.x+this.y*this.y},length:function(){return Math.sqrt(this.x*this.x+this.y*this.y)},manhattanLength:function(){return Math.abs(this.x)+Math.abs(this.y)},normalize:function(){return this.divideScalar(this.length()||1)},angle:function(){return Math.atan2(-this.y,-this.x)+Math.PI},distanceTo:function(e){return Math.sqrt(this.distanceToSquared(e))},distanceToSquared:function(e){var t=this.x-e.x,n=this.y-e.y;return t*t+n*n},manhattanDistanceTo:function(e){return Math.abs(this.x-e.x)+Math.abs(this.y-e.y)},setLength:function(e){return this.normalize().multiplyScalar(e)},lerp:function(e,t){return this.x+=(e.x-this.x)*t,this.y+=(e.y-this.y)*t,this},lerpVectors:function(e,t,n){return this.x=e.x+(t.x-e.x)*n,this.y=e.y+(t.y-e.y)*n,this},equals:function(e){return e.x===this.x&&e.y===this.y},fromArray:function(e,t){return void 0===t&&(t=0),this.x=e[t],this.y=e[t+1],this},toArray:function(e,t){return void 0===e&&(e=[]),void 0===t&&(t=0),e[t]=this.x,e[t+1]=this.y,e},fromBufferAttribute:function(e,t,n){return void 0!==n&&console.warn("THREE.Vector2: offset has been removed from .fromBufferAttribute()."),this.x=e.getX(t),this.y=e.getY(t),this},rotateAround:function(e,t){var n=Math.cos(t),r=Math.sin(t),i=this.x-e.x,o=this.y-e.y;return this.x=i*n-o*r+e.x,this.y=i*r+o*n+e.y,this},random:function(){return this.x=Math.random(),this.y=Math.random(),this}}),Object.assign(Vn.prototype,{isMatrix3:!0,set:function(e,t,n,r,i,o,a,s,l){var u=this.elements;return u[0]=e,u[1]=r,u[2]=a,u[3]=t,u[4]=i,u[5]=s,u[6]=n,u[7]=o,u[8]=l,this},identity:function(){return this.set(1,0,0,0,1,0,0,0,1),this},clone:function(){return(new this.constructor).fromArray(this.elements)},copy:function(e){var t=this.elements,n=e.elements;return t[0]=n[0],t[1]=n[1],t[2]=n[2],t[3]=n[3],t[4]=n[4],t[5]=n[5],t[6]=n[6],t[7]=n[7],t[8]=n[8],this},extractBasis:function(e,t,n){return e.setFromMatrix3Column(this,0),t.setFromMatrix3Column(this,1),n.setFromMatrix3Column(this,2),this},setFromMatrix4:function(e){var t=e.elements;return this.set(t[0],t[4],t[8],t[1],t[5],t[9],t[2],t[6],t[10]),this},multiply:function(e){return this.multiplyMatrices(this,e)},premultiply:function(e){return this.multiplyMatrices(e,this)},multiplyMatrices:function(e,t){var n=e.elements,r=t.elements,i=this.elements,o=n[0],a=n[3],s=n[6],l=n[1],u=n[4],c=n[7],d=n[2],h=n[5],f=n[8],p=r[0],g=r[3],v=r[6],m=r[1],y=r[4],b=r[7],C=r[2],A=r[5],_=r[8];return i[0]=o*p+a*m+s*C,i[3]=o*g+a*y+s*A,i[6]=o*v+a*b+s*_,i[1]=l*p+u*m+c*C,i[4]=l*g+u*y+c*A,i[7]=l*v+u*b+c*_,i[2]=d*p+h*m+f*C,i[5]=d*g+h*y+f*A,i[8]=d*v+h*b+f*_,this},multiplyScalar:function(e){var t=this.elements;return t[0]*=e,t[3]*=e,t[6]*=e,t[1]*=e,t[4]*=e,t[7]*=e,t[2]*=e,t[5]*=e,t[8]*=e,this},determinant:function(){var e=this.elements,t=e[0],n=e[1],r=e[2],i=e[3],o=e[4],a=e[5],s=e[6],l=e[7],u=e[8];return t*o*u-t*a*l-n*i*u+n*a*s+r*i*l-r*o*s},getInverse:function(e,t){void 0!==t&&console.warn("THREE.Matrix3: .getInverse() can no longer be configured to throw on degenerate.");var n=e.elements,r=this.elements,i=n[0],o=n[1],a=n[2],s=n[3],l=n[4],u=n[5],c=n[6],d=n[7],h=n[8],f=h*l-u*d,p=u*c-h*s,g=d*s-l*c,v=i*f+o*p+a*g;if(0===v)return this.set(0,0,0,0,0,0,0,0,0);var m=1/v;return r[0]=f*m,r[1]=(a*d-h*o)*m,r[2]=(u*o-a*l)*m,r[3]=p*m,r[4]=(h*i-a*c)*m,r[5]=(a*s-u*i)*m,r[6]=g*m,r[7]=(o*c-d*i)*m,r[8]=(l*i-o*s)*m,this},transpose:function(){var e,t=this.elements;return e=t[1],t[1]=t[3],t[3]=e,e=t[2],t[2]=t[6],t[6]=e,e=t[5],t[5]=t[7],t[7]=e,this},getNormalMatrix:function(e){return this.setFromMatrix4(e).getInverse(this).transpose()},transposeIntoArray:function(e){var t=this.elements;return e[0]=t[0],e[1]=t[3],e[2]=t[6],e[3]=t[1],e[4]=t[4],e[5]=t[7],e[6]=t[2],e[7]=t[5],e[8]=t[8],this},setUvTransform:function(e,t,n,r,i,o,a){var s=Math.cos(i),l=Math.sin(i);this.set(n*s,n*l,-n*(s*o+l*a)+o+e,-r*l,r*s,-r*(-l*o+s*a)+a+t,0,0,1)},scale:function(e,t){var n=this.elements;return n[0]*=e,n[3]*=e,n[6]*=e,n[1]*=t,n[4]*=t,n[7]*=t,this},rotate:function(e){var t=Math.cos(e),n=Math.sin(e),r=this.elements,i=r[0],o=r[3],a=r[6],s=r[1],l=r[4],u=r[7];return r[0]=t*i+n*s,r[3]=t*o+n*l,r[6]=t*a+n*u,r[1]=-n*i+t*s,r[4]=-n*o+t*l,r[7]=-n*a+t*u,this},translate:function(e,t){var n=this.elements;return n[0]+=e*n[2],n[3]+=e*n[5],n[6]+=e*n[8],n[1]+=t*n[2],n[4]+=t*n[5],n[7]+=t*n[8],this},equals:function(e){for(var t=this.elements,n=e.elements,r=0;r<9;r++)if(t[r]!==n[r])return!1;return!0},fromArray:function(e,t){void 0===t&&(t=0);for(var n=0;n<9;n++)this.elements[n]=e[n+t];return this},toArray:function(e,t){void 0===e&&(e=[]),void 0===t&&(t=0);var n=this.elements;return e[t]=n[0],e[t+1]=n[1],e[t+2]=n[2],e[t+3]=n[3],e[t+4]=n[4],e[t+5]=n[5],e[t+6]=n[6],e[t+7]=n[7],e[t+8]=n[8],e}});var Yn={getDataURL:function(e){var t;if("undefined"==typeof HTMLCanvasElement)return e.src;if(e instanceof HTMLCanvasElement)t=e;else{void 0===Bn&&(Bn=document.createElementNS("http://www.w3.org/1999/xhtml","canvas")),Bn.width=e.width,Bn.height=e.height;var n=Bn.getContext("2d");e instanceof ImageData?n.putImageData(e,0,0):n.drawImage(e,0,0,e.width,e.height),t=Bn}return t.width>2048||t.height>2048?t.toDataURL("image/jpeg",.6):t.toDataURL("image/png")}},Wn=0;function Gn(e,t,n,r,i,o,a,s,l,u){Object.defineProperty(this,"id",{value:Wn++}),this.uuid=Hn.generateUUID(),this.name="",this.image=void 0!==e?e:Gn.DEFAULT_IMAGE,this.mipmaps=[],this.mapping=void 0!==t?t:Gn.DEFAULT_MAPPING,this.wrapS=void 0!==n?n:ge,this.wrapT=void 0!==r?r:ge,this.magFilter=void 0!==i?i:_e,this.minFilter=void 0!==o?o:we,this.anisotropy=void 0!==l?l:1,this.format=void 0!==a?a:He,this.internalFormat=null,this.type=void 0!==s?s:Se,this.offset=new Un(0,0),this.repeat=new Un(1,1),this.center=new Un(0,0),this.rotation=0,this.matrixAutoUpdate=!0,this.matrix=new Vn,this.generateMipmaps=!0,this.premultiplyAlpha=!1,this.flipY=!0,this.unpackAlignment=4,this.encoding=void 0!==u?u:$t,this.version=0,this.onUpdate=null}function Xn(e,t,n,r){this.x=e||0,this.y=t||0,this.z=n||0,this.w=void 0!==r?r:1}function qn(e,t,n){this.width=e,this.height=t,this.scissor=new Xn(0,0,e,t),this.scissorTest=!1,this.viewport=new Xn(0,0,e,t),n=n||{},this.texture=new Gn(void 0,n.mapping,n.wrapS,n.wrapT,n.magFilter,n.minFilter,n.format,n.type,n.anisotropy,n.encoding),this.texture.image={},this.texture.image.width=e,this.texture.image.height=t,this.texture.generateMipmaps=void 0!==n.generateMipmaps&&n.generateMipmaps,this.texture.minFilter=void 0!==n.minFilter?n.minFilter:_e,this.depthBuffer=void 0===n.depthBuffer||n.depthBuffer,this.stencilBuffer=void 0===n.stencilBuffer||n.stencilBuffer,this.depthTexture=void 0!==n.depthTexture?n.depthTexture:null}function Zn(e,t,n){qn.call(this,e,t,n),this.samples=4}function Qn(e,t,n,r){this._x=e||0,this._y=t||0,this._z=n||0,this._w=void 0!==r?r:1}Gn.DEFAULT_IMAGE=void 0,Gn.DEFAULT_MAPPING=ae,Gn.prototype=Object.assign(Object.create(Rn.prototype),{constructor:Gn,isTexture:!0,updateMatrix:function(){this.matrix.setUvTransform(this.offset.x,this.offset.y,this.repeat.x,this.repeat.y,this.rotation,this.center.x,this.center.y)},clone:function(){return(new this.constructor).copy(this)},copy:function(e){return this.name=e.name,this.image=e.image,this.mipmaps=e.mipmaps.slice(0),this.mapping=e.mapping,this.wrapS=e.wrapS,this.wrapT=e.wrapT,this.magFilter=e.magFilter,this.minFilter=e.minFilter,this.anisotropy=e.anisotropy,this.format=e.format,this.internalFormat=e.internalFormat,this.type=e.type,this.offset.copy(e.offset),this.repeat.copy(e.repeat),this.center.copy(e.center),this.rotation=e.rotation,this.matrixAutoUpdate=e.matrixAutoUpdate,this.matrix.copy(e.matrix),this.generateMipmaps=e.generateMipmaps,this.premultiplyAlpha=e.premultiplyAlpha,this.flipY=e.flipY,this.unpackAlignment=e.unpackAlignment,this.encoding=e.encoding,this},toJSON:function(e){var t=void 0===e||"string"==typeof e;if(!t&&void 0!==e.textures[this.uuid])return e.textures[this.uuid];var n={metadata:{version:4.5,type:"Texture",generator:"Texture.toJSON"},uuid:this.uuid,name:this.name,mapping:this.mapping,repeat:[this.repeat.x,this.repeat.y],offset:[this.offset.x,this.offset.y],center:[this.center.x,this.center.y],rotation:this.rotation,wrap:[this.wrapS,this.wrapT],format:this.format,type:this.type,encoding:this.encoding,minFilter:this.minFilter,magFilter:this.magFilter,anisotropy:this.anisotropy,flipY:this.flipY,premultiplyAlpha:this.premultiplyAlpha,unpackAlignment:this.unpackAlignment};if(void 0!==this.image){var r=this.image;if(void 0===r.uuid&&(r.uuid=Hn.generateUUID()),!t&&void 0===e.images[r.uuid]){var i;if(Array.isArray(r)){i=[];for(var o=0,a=r.length;o<a;o++)i.push(Yn.getDataURL(r[o]))}else i=Yn.getDataURL(r);e.images[r.uuid]={uuid:r.uuid,url:i}}n.image=r.uuid}return t||(e.textures[this.uuid]=n),n},dispose:function(){this.dispatchEvent({type:"dispose"})},transformUv:function(e){if(this.mapping!==ae)return e;if(e.applyMatrix3(this.matrix),e.x<0||e.x>1)switch(this.wrapS){case pe:e.x=e.x-Math.floor(e.x);break;case ge:e.x=e.x<0?0:1;break;case ve:1===Math.abs(Math.floor(e.x)%2)?e.x=Math.ceil(e.x)-e.x:e.x=e.x-Math.floor(e.x)}if(e.y<0||e.y>1)switch(this.wrapT){case pe:e.y=e.y-Math.floor(e.y);break;case ge:e.y=e.y<0?0:1;break;case ve:1===Math.abs(Math.floor(e.y)%2)?e.y=Math.ceil(e.y)-e.y:e.y=e.y-Math.floor(e.y)}return this.flipY&&(e.y=1-e.y),e}}),Object.defineProperty(Gn.prototype,"needsUpdate",{set:function(e){!0===e&&this.version++}}),Object.defineProperties(Xn.prototype,{width:{get:function(){return this.z},set:function(e){this.z=e}},height:{get:function(){return this.w},set:function(e){this.w=e}}}),Object.assign(Xn.prototype,{isVector4:!0,set:function(e,t,n,r){return this.x=e,this.y=t,this.z=n,this.w=r,this},setScalar:function(e){return this.x=e,this.y=e,this.z=e,this.w=e,this},setX:function(e){return this.x=e,this},setY:function(e){return this.y=e,this},setZ:function(e){return this.z=e,this},setW:function(e){return this.w=e,this},setComponent:function(e,t){switch(e){case 0:this.x=t;break;case 1:this.y=t;break;case 2:this.z=t;break;case 3:this.w=t;break;default:throw new Error("index is out of range: "+e)}return this},getComponent:function(e){switch(e){case 0:return this.x;case 1:return this.y;case 2:return this.z;case 3:return this.w;default:throw new Error("index is out of range: "+e)}},clone:function(){return new this.constructor(this.x,this.y,this.z,this.w)},copy:function(e){return this.x=e.x,this.y=e.y,this.z=e.z,this.w=void 0!==e.w?e.w:1,this},add:function(e,t){return void 0!==t?(console.warn("THREE.Vector4: .add() now only accepts one argument. Use .addVectors( a, b ) instead."),this.addVectors(e,t)):(this.x+=e.x,this.y+=e.y,this.z+=e.z,this.w+=e.w,this)},addScalar:function(e){return this.x+=e,this.y+=e,this.z+=e,this.w+=e,this},addVectors:function(e,t){return this.x=e.x+t.x,this.y=e.y+t.y,this.z=e.z+t.z,this.w=e.w+t.w,this},addScaledVector:function(e,t){return this.x+=e.x*t,this.y+=e.y*t,this.z+=e.z*t,this.w+=e.w*t,this},sub:function(e,t){return void 0!==t?(console.warn("THREE.Vector4: .sub() now only accepts one argument. Use .subVectors( a, b ) instead."),this.subVectors(e,t)):(this.x-=e.x,this.y-=e.y,this.z-=e.z,this.w-=e.w,this)},subScalar:function(e){return this.x-=e,this.y-=e,this.z-=e,this.w-=e,this},subVectors:function(e,t){return this.x=e.x-t.x,this.y=e.y-t.y,this.z=e.z-t.z,this.w=e.w-t.w,this},multiplyScalar:function(e){return this.x*=e,this.y*=e,this.z*=e,this.w*=e,this},applyMatrix4:function(e){var t=this.x,n=this.y,r=this.z,i=this.w,o=e.elements;return this.x=o[0]*t+o[4]*n+o[8]*r+o[12]*i,this.y=o[1]*t+o[5]*n+o[9]*r+o[13]*i,this.z=o[2]*t+o[6]*n+o[10]*r+o[14]*i,this.w=o[3]*t+o[7]*n+o[11]*r+o[15]*i,this},divideScalar:function(e){return this.multiplyScalar(1/e)},setAxisAngleFromQuaternion:function(e){this.w=2*Math.acos(e.w);var t=Math.sqrt(1-e.w*e.w);return t<1e-4?(this.x=1,this.y=0,this.z=0):(this.x=e.x/t,this.y=e.y/t,this.z=e.z/t),this},setAxisAngleFromRotationMatrix:function(e){var t,n,r,i,o=e.elements,a=o[0],s=o[4],l=o[8],u=o[1],c=o[5],d=o[9],h=o[2],f=o[6],p=o[10];if(Math.abs(s-u)<.01&&Math.abs(l-h)<.01&&Math.abs(d-f)<.01){if(Math.abs(s+u)<.1&&Math.abs(l+h)<.1&&Math.abs(d+f)<.1&&Math.abs(a+c+p-3)<.1)return this.set(1,0,0,0),this;t=Math.PI;var g=(a+1)/2,v=(c+1)/2,m=(p+1)/2,y=(s+u)/4,b=(l+h)/4,C=(d+f)/4;return g>v&&g>m?g<.01?(n=0,r=.707106781,i=.707106781):(r=y/(n=Math.sqrt(g)),i=b/n):v>m?v<.01?(n=.707106781,r=0,i=.707106781):(n=y/(r=Math.sqrt(v)),i=C/r):m<.01?(n=.707106781,r=.707106781,i=0):(n=b/(i=Math.sqrt(m)),r=C/i),this.set(n,r,i,t),this}var A=Math.sqrt((f-d)*(f-d)+(l-h)*(l-h)+(u-s)*(u-s));return Math.abs(A)<.001&&(A=1),this.x=(f-d)/A,this.y=(l-h)/A,this.z=(u-s)/A,this.w=Math.acos((a+c+p-1)/2),this},min:function(e){return this.x=Math.min(this.x,e.x),this.y=Math.min(this.y,e.y),this.z=Math.min(this.z,e.z),this.w=Math.min(this.w,e.w),this},max:function(e){return this.x=Math.max(this.x,e.x),this.y=Math.max(this.y,e.y),this.z=Math.max(this.z,e.z),this.w=Math.max(this.w,e.w),this},clamp:function(e,t){return this.x=Math.max(e.x,Math.min(t.x,this.x)),this.y=Math.max(e.y,Math.min(t.y,this.y)),this.z=Math.max(e.z,Math.min(t.z,this.z)),this.w=Math.max(e.w,Math.min(t.w,this.w)),this},clampScalar:function(e,t){return this.x=Math.max(e,Math.min(t,this.x)),this.y=Math.max(e,Math.min(t,this.y)),this.z=Math.max(e,Math.min(t,this.z)),this.w=Math.max(e,Math.min(t,this.w)),this},clampLength:function(e,t){var n=this.length();return this.divideScalar(n||1).multiplyScalar(Math.max(e,Math.min(t,n)))},floor:function(){return this.x=Math.floor(this.x),this.y=Math.floor(this.y),this.z=Math.floor(this.z),this.w=Math.floor(this.w),this},ceil:function(){return this.x=Math.ceil(this.x),this.y=Math.ceil(this.y),this.z=Math.ceil(this.z),this.w=Math.ceil(this.w),this},round:function(){return this.x=Math.round(this.x),this.y=Math.round(this.y),this.z=Math.round(this.z),this.w=Math.round(this.w),this},roundToZero:function(){return this.x=this.x<0?Math.ceil(this.x):Math.floor(this.x),this.y=this.y<0?Math.ceil(this.y):Math.floor(this.y),this.z=this.z<0?Math.ceil(this.z):Math.floor(this.z),this.w=this.w<0?Math.ceil(this.w):Math.floor(this.w),this},negate:function(){return this.x=-this.x,this.y=-this.y,this.z=-this.z,this.w=-this.w,this},dot:function(e){return this.x*e.x+this.y*e.y+this.z*e.z+this.w*e.w},lengthSq:function(){return this.x*this.x+this.y*this.y+this.z*this.z+this.w*this.w},length:function(){return Math.sqrt(this.x*this.x+this.y*this.y+this.z*this.z+this.w*this.w)},manhattanLength:function(){return Math.abs(this.x)+Math.abs(this.y)+Math.abs(this.z)+Math.abs(this.w)},normalize:function(){return this.divideScalar(this.length()||1)},setLength:function(e){return this.normalize().multiplyScalar(e)},lerp:function(e,t){return this.x+=(e.x-this.x)*t,this.y+=(e.y-this.y)*t,this.z+=(e.z-this.z)*t,this.w+=(e.w-this.w)*t,this},lerpVectors:function(e,t,n){return this.x=e.x+(t.x-e.x)*n,this.y=e.y+(t.y-e.y)*n,this.z=e.z+(t.z-e.z)*n,this.w=e.w+(t.w-e.w)*n,this},equals:function(e){return e.x===this.x&&e.y===this.y&&e.z===this.z&&e.w===this.w},fromArray:function(e,t){return void 0===t&&(t=0),this.x=e[t],this.y=e[t+1],this.z=e[t+2],this.w=e[t+3],this},toArray:function(e,t){return void 0===e&&(e=[]),void 0===t&&(t=0),e[t]=this.x,e[t+1]=this.y,e[t+2]=this.z,e[t+3]=this.w,e},fromBufferAttribute:function(e,t,n){return void 0!==n&&console.warn("THREE.Vector4: offset has been removed from .fromBufferAttribute()."),this.x=e.getX(t),this.y=e.getY(t),this.z=e.getZ(t),this.w=e.getW(t),this},random:function(){return this.x=Math.random(),this.y=Math.random(),this.z=Math.random(),this.w=Math.random(),this}}),qn.prototype=Object.assign(Object.create(Rn.prototype),{constructor:qn,isWebGLRenderTarget:!0,setSize:function(e,t){this.width===e&&this.height===t||(this.width=e,this.height=t,this.texture.image.width=e,this.texture.image.height=t,this.dispose()),this.viewport.set(0,0,e,t),this.scissor.set(0,0,e,t)},clone:function(){return(new this.constructor).copy(this)},copy:function(e){return this.width=e.width,this.height=e.height,this.viewport.copy(e.viewport),this.texture=e.texture.clone(),this.depthBuffer=e.depthBuffer,this.stencilBuffer=e.stencilBuffer,this.depthTexture=e.depthTexture,this},dispose:function(){this.dispatchEvent({type:"dispose"})}}),Zn.prototype=Object.assign(Object.create(qn.prototype),{constructor:Zn,isWebGLMultisampleRenderTarget:!0,copy:function(e){return qn.prototype.copy.call(this,e),this.samples=e.samples,this}}),Object.assign(Qn,{slerp:function(e,t,n,r){return n.copy(e).slerp(t,r)},slerpFlat:function(e,t,n,r,i,o,a){var s=n[r+0],l=n[r+1],u=n[r+2],c=n[r+3],d=i[o+0],h=i[o+1],f=i[o+2],p=i[o+3];if(c!==p||s!==d||l!==h||u!==f){var g=1-a,v=s*d+l*h+u*f+c*p,m=v>=0?1:-1,y=1-v*v;if(y>Number.EPSILON){var b=Math.sqrt(y),C=Math.atan2(b,v*m);g=Math.sin(g*C)/b,a=Math.sin(a*C)/b}var A=a*m;if(s=s*g+d*A,l=l*g+h*A,u=u*g+f*A,c=c*g+p*A,g===1-a){var _=1/Math.sqrt(s*s+l*l+u*u+c*c);s*=_,l*=_,u*=_,c*=_}}e[t]=s,e[t+1]=l,e[t+2]=u,e[t+3]=c},multiplyQuaternionsFlat:function(e,t,n,r,i,o){var a=n[r],s=n[r+1],l=n[r+2],u=n[r+3],c=i[o],d=i[o+1],h=i[o+2],f=i[o+3];return e[t]=a*f+u*c+s*h-l*d,e[t+1]=s*f+u*d+l*c-a*h,e[t+2]=l*f+u*h+a*d-s*c,e[t+3]=u*f-a*c-s*d-l*h,e}}),Object.defineProperties(Qn.prototype,{x:{get:function(){return this._x},set:function(e){this._x=e,this._onChangeCallback()}},y:{get:function(){return this._y},set:function(e){this._y=e,this._onChangeCallback()}},z:{get:function(){return this._z},set:function(e){this._z=e,this._onChangeCallback()}},w:{get:function(){return this._w},set:function(e){this._w=e,this._onChangeCallback()}}}),Object.assign(Qn.prototype,{isQuaternion:!0,set:function(e,t,n,r){return this._x=e,this._y=t,this._z=n,this._w=r,this._onChangeCallback(),this},clone:function(){return new this.constructor(this._x,this._y,this._z,this._w)},copy:function(e){return this._x=e.x,this._y=e.y,this._z=e.z,this._w=e.w,this._onChangeCallback(),this},setFromEuler:function(e,t){if(!e||!e.isEuler)throw new Error("THREE.Quaternion: .setFromEuler() now expects an Euler rotation rather than a Vector3 and order.");var n=e._x,r=e._y,i=e._z,o=e.order,a=Math.cos,s=Math.sin,l=a(n/2),u=a(r/2),c=a(i/2),d=s(n/2),h=s(r/2),f=s(i/2);switch(o){case"XYZ":this._x=d*u*c+l*h*f,this._y=l*h*c-d*u*f,this._z=l*u*f+d*h*c,this._w=l*u*c-d*h*f;break;case"YXZ":this._x=d*u*c+l*h*f,this._y=l*h*c-d*u*f,this._z=l*u*f-d*h*c,this._w=l*u*c+d*h*f;break;case"ZXY":this._x=d*u*c-l*h*f,this._y=l*h*c+d*u*f,this._z=l*u*f+d*h*c,this._w=l*u*c-d*h*f;break;case"ZYX":this._x=d*u*c-l*h*f,this._y=l*h*c+d*u*f,this._z=l*u*f-d*h*c,this._w=l*u*c+d*h*f;break;case"YZX":this._x=d*u*c+l*h*f,this._y=l*h*c+d*u*f,this._z=l*u*f-d*h*c,this._w=l*u*c-d*h*f;break;case"XZY":this._x=d*u*c-l*h*f,this._y=l*h*c-d*u*f,this._z=l*u*f+d*h*c,this._w=l*u*c+d*h*f;break;default:console.warn("THREE.Quaternion: .setFromEuler() encountered an unknown order: "+o)}return!1!==t&&this._onChangeCallback(),this},setFromAxisAngle:function(e,t){var n=t/2,r=Math.sin(n);return this._x=e.x*r,this._y=e.y*r,this._z=e.z*r,this._w=Math.cos(n),this._onChangeCallback(),this},setFromRotationMatrix:function(e){var t,n=e.elements,r=n[0],i=n[4],o=n[8],a=n[1],s=n[5],l=n[9],u=n[2],c=n[6],d=n[10],h=r+s+d;return h>0?(t=.5/Math.sqrt(h+1),this._w=.25/t,this._x=(c-l)*t,this._y=(o-u)*t,this._z=(a-i)*t):r>s&&r>d?(t=2*Math.sqrt(1+r-s-d),this._w=(c-l)/t,this._x=.25*t,this._y=(i+a)/t,this._z=(o+u)/t):s>d?(t=2*Math.sqrt(1+s-r-d),this._w=(o-u)/t,this._x=(i+a)/t,this._y=.25*t,this._z=(l+c)/t):(t=2*Math.sqrt(1+d-r-s),this._w=(a-i)/t,this._x=(o+u)/t,this._y=(l+c)/t,this._z=.25*t),this._onChangeCallback(),this},setFromUnitVectors:function(e,t){var n=e.dot(t)+1;return n<1e-6?(n=0,Math.abs(e.x)>Math.abs(e.z)?(this._x=-e.y,this._y=e.x,this._z=0,this._w=n):(this._x=0,this._y=-e.z,this._z=e.y,this._w=n)):(this._x=e.y*t.z-e.z*t.y,this._y=e.z*t.x-e.x*t.z,this._z=e.x*t.y-e.y*t.x,this._w=n),this.normalize()},angleTo:function(e){return 2*Math.acos(Math.abs(Hn.clamp(this.dot(e),-1,1)))},rotateTowards:function(e,t){var n=this.angleTo(e);if(0===n)return this;var r=Math.min(1,t/n);return this.slerp(e,r),this},inverse:function(){return this.conjugate()},conjugate:function(){return this._x*=-1,this._y*=-1,this._z*=-1,this._onChangeCallback(),this},dot:function(e){return this._x*e._x+this._y*e._y+this._z*e._z+this._w*e._w},lengthSq:function(){return this._x*this._x+this._y*this._y+this._z*this._z+this._w*this._w},length:function(){return Math.sqrt(this._x*this._x+this._y*this._y+this._z*this._z+this._w*this._w)},normalize:function(){var e=this.length();return 0===e?(this._x=0,this._y=0,this._z=0,this._w=1):(e=1/e,this._x=this._x*e,this._y=this._y*e,this._z=this._z*e,this._w=this._w*e),this._onChangeCallback(),this},multiply:function(e,t){return void 0!==t?(console.warn("THREE.Quaternion: .multiply() now only accepts one argument. Use .multiplyQuaternions( a, b ) instead."),this.multiplyQuaternions(e,t)):this.multiplyQuaternions(this,e)},premultiply:function(e){return this.multiplyQuaternions(e,this)},multiplyQuaternions:function(e,t){var n=e._x,r=e._y,i=e._z,o=e._w,a=t._x,s=t._y,l=t._z,u=t._w;return this._x=n*u+o*a+r*l-i*s,this._y=r*u+o*s+i*a-n*l,this._z=i*u+o*l+n*s-r*a,this._w=o*u-n*a-r*s-i*l,this._onChangeCallback(),this},slerp:function(e,t){if(0===t)return this;if(1===t)return this.copy(e);var n=this._x,r=this._y,i=this._z,o=this._w,a=o*e._w+n*e._x+r*e._y+i*e._z;if(a<0?(this._w=-e._w,this._x=-e._x,this._y=-e._y,this._z=-e._z,a=-a):this.copy(e),a>=1)return this._w=o,this._x=n,this._y=r,this._z=i,this;var s=1-a*a;if(s<=Number.EPSILON){var l=1-t;return this._w=l*o+t*this._w,this._x=l*n+t*this._x,this._y=l*r+t*this._y,this._z=l*i+t*this._z,this.normalize(),this._onChangeCallback(),this}var u=Math.sqrt(s),c=Math.atan2(u,a),d=Math.sin((1-t)*c)/u,h=Math.sin(t*c)/u;return this._w=o*d+this._w*h,this._x=n*d+this._x*h,this._y=r*d+this._y*h,this._z=i*d+this._z*h,this._onChangeCallback(),this},equals:function(e){return e._x===this._x&&e._y===this._y&&e._z===this._z&&e._w===this._w},fromArray:function(e,t){return void 0===t&&(t=0),this._x=e[t],this._y=e[t+1],this._z=e[t+2],this._w=e[t+3],this._onChangeCallback(),this},toArray:function(e,t){return void 0===e&&(e=[]),void 0===t&&(t=0),e[t]=this._x,e[t+1]=this._y,e[t+2]=this._z,e[t+3]=this._w,e},fromBufferAttribute:function(e,t){return this._x=e.getX(t),this._y=e.getY(t),this._z=e.getZ(t),this._w=e.getW(t),this},_onChange:function(e){return this._onChangeCallback=e,this},_onChangeCallback:function(){}});var Kn=new $n,Jn=new Qn;function $n(e,t,n){this.x=e||0,this.y=t||0,this.z=n||0}Object.assign($n.prototype,{isVector3:!0,set:function(e,t,n){return this.x=e,this.y=t,this.z=n,this},setScalar:function(e){return this.x=e,this.y=e,this.z=e,this},setX:function(e){return this.x=e,this},setY:function(e){return this.y=e,this},setZ:function(e){return this.z=e,this},setComponent:function(e,t){switch(e){case 0:this.x=t;break;case 1:this.y=t;break;case 2:this.z=t;break;default:throw new Error("index is out of range: "+e)}return this},getComponent:function(e){switch(e){case 0:return this.x;case 1:return this.y;case 2:return this.z;default:throw new Error("index is out of range: "+e)}},clone:function(){return new this.constructor(this.x,this.y,this.z)},copy:function(e){return this.x=e.x,this.y=e.y,this.z=e.z,this},add:function(e,t){return void 0!==t?(console.warn("THREE.Vector3: .add() now only accepts one argument. Use .addVectors( a, b ) instead."),this.addVectors(e,t)):(this.x+=e.x,this.y+=e.y,this.z+=e.z,this)},addScalar:function(e){return this.x+=e,this.y+=e,this.z+=e,this},addVectors:function(e,t){return this.x=e.x+t.x,this.y=e.y+t.y,this.z=e.z+t.z,this},addScaledVector:function(e,t){return this.x+=e.x*t,this.y+=e.y*t,this.z+=e.z*t,this},sub:function(e,t){return void 0!==t?(console.warn("THREE.Vector3: .sub() now only accepts one argument. Use .subVectors( a, b ) instead."),this.subVectors(e,t)):(this.x-=e.x,this.y-=e.y,this.z-=e.z,this)},subScalar:function(e){return this.x-=e,this.y-=e,this.z-=e,this},subVectors:function(e,t){return this.x=e.x-t.x,this.y=e.y-t.y,this.z=e.z-t.z,this},multiply:function(e,t){return void 0!==t?(console.warn("THREE.Vector3: .multiply() now only accepts one argument. Use .multiplyVectors( a, b ) instead."),this.multiplyVectors(e,t)):(this.x*=e.x,this.y*=e.y,this.z*=e.z,this)},multiplyScalar:function(e){return this.x*=e,this.y*=e,this.z*=e,this},multiplyVectors:function(e,t){return this.x=e.x*t.x,this.y=e.y*t.y,this.z=e.z*t.z,this},applyEuler:function(e){return e&&e.isEuler||console.error("THREE.Vector3: .applyEuler() now expects an Euler rotation rather than a Vector3 and order."),this.applyQuaternion(Jn.setFromEuler(e))},applyAxisAngle:function(e,t){return this.applyQuaternion(Jn.setFromAxisAngle(e,t))},applyMatrix3:function(e){var t=this.x,n=this.y,r=this.z,i=e.elements;return this.x=i[0]*t+i[3]*n+i[6]*r,this.y=i[1]*t+i[4]*n+i[7]*r,this.z=i[2]*t+i[5]*n+i[8]*r,this},applyNormalMatrix:function(e){return this.applyMatrix3(e).normalize()},applyMatrix4:function(e){var t=this.x,n=this.y,r=this.z,i=e.elements,o=1/(i[3]*t+i[7]*n+i[11]*r+i[15]);return this.x=(i[0]*t+i[4]*n+i[8]*r+i[12])*o,this.y=(i[1]*t+i[5]*n+i[9]*r+i[13])*o,this.z=(i[2]*t+i[6]*n+i[10]*r+i[14])*o,this},applyQuaternion:function(e){var t=this.x,n=this.y,r=this.z,i=e.x,o=e.y,a=e.z,s=e.w,l=s*t+o*r-a*n,u=s*n+a*t-i*r,c=s*r+i*n-o*t,d=-i*t-o*n-a*r;return this.x=l*s+d*-i+u*-a-c*-o,this.y=u*s+d*-o+c*-i-l*-a,this.z=c*s+d*-a+l*-o-u*-i,this},project:function(e){return this.applyMatrix4(e.matrixWorldInverse).applyMatrix4(e.projectionMatrix)},unproject:function(e){return this.applyMatrix4(e.projectionMatrixInverse).applyMatrix4(e.matrixWorld)},transformDirection:function(e){var t=this.x,n=this.y,r=this.z,i=e.elements;return this.x=i[0]*t+i[4]*n+i[8]*r,this.y=i[1]*t+i[5]*n+i[9]*r,this.z=i[2]*t+i[6]*n+i[10]*r,this.normalize()},divide:function(e){return this.x/=e.x,this.y/=e.y,this.z/=e.z,this},divideScalar:function(e){return this.multiplyScalar(1/e)},min:function(e){return this.x=Math.min(this.x,e.x),this.y=Math.min(this.y,e.y),this.z=Math.min(this.z,e.z),this},max:function(e){return this.x=Math.max(this.x,e.x),this.y=Math.max(this.y,e.y),this.z=Math.max(this.z,e.z),this},clamp:function(e,t){return this.x=Math.max(e.x,Math.min(t.x,this.x)),this.y=Math.max(e.y,Math.min(t.y,this.y)),this.z=Math.max(e.z,Math.min(t.z,this.z)),this},clampScalar:function(e,t){return this.x=Math.max(e,Math.min(t,this.x)),this.y=Math.max(e,Math.min(t,this.y)),this.z=Math.max(e,Math.min(t,this.z)),this},clampLength:function(e,t){var n=this.length();return this.divideScalar(n||1).multiplyScalar(Math.max(e,Math.min(t,n)))},floor:function(){return this.x=Math.floor(this.x),this.y=Math.floor(this.y),this.z=Math.floor(this.z),this},ceil:function(){return this.x=Math.ceil(this.x),this.y=Math.ceil(this.y),this.z=Math.ceil(this.z),this},round:function(){return this.x=Math.round(this.x),this.y=Math.round(this.y),this.z=Math.round(this.z),this},roundToZero:function(){return this.x=this.x<0?Math.ceil(this.x):Math.floor(this.x),this.y=this.y<0?Math.ceil(this.y):Math.floor(this.y),this.z=this.z<0?Math.ceil(this.z):Math.floor(this.z),this},negate:function(){return this.x=-this.x,this.y=-this.y,this.z=-this.z,this},dot:function(e){return this.x*e.x+this.y*e.y+this.z*e.z},lengthSq:function(){return this.x*this.x+this.y*this.y+this.z*this.z},length:function(){return Math.sqrt(this.x*this.x+this.y*this.y+this.z*this.z)},manhattanLength:function(){return Math.abs(this.x)+Math.abs(this.y)+Math.abs(this.z)},normalize:function(){return this.divideScalar(this.length()||1)},setLength:function(e){return this.normalize().multiplyScalar(e)},lerp:function(e,t){return this.x+=(e.x-this.x)*t,this.y+=(e.y-this.y)*t,this.z+=(e.z-this.z)*t,this},lerpVectors:function(e,t,n){return this.x=e.x+(t.x-e.x)*n,this.y=e.y+(t.y-e.y)*n,this.z=e.z+(t.z-e.z)*n,this},cross:function(e,t){return void 0!==t?(console.warn("THREE.Vector3: .cross() now only accepts one argument. Use .crossVectors( a, b ) instead."),this.crossVectors(e,t)):this.crossVectors(this,e)},crossVectors:function(e,t){var n=e.x,r=e.y,i=e.z,o=t.x,a=t.y,s=t.z;return this.x=r*s-i*a,this.y=i*o-n*s,this.z=n*a-r*o,this},projectOnVector:function(e){var t=e.lengthSq();if(0===t)return this.set(0,0,0);var n=e.dot(this)/t;return this.copy(e).multiplyScalar(n)},projectOnPlane:function(e){return Kn.copy(this).projectOnVector(e),this.sub(Kn)},reflect:function(e){return this.sub(Kn.copy(e).multiplyScalar(2*this.dot(e)))},angleTo:function(e){var t=Math.sqrt(this.lengthSq()*e.lengthSq());if(0===t)return Math.PI/2;var n=this.dot(e)/t;return Math.acos(Hn.clamp(n,-1,1))},distanceTo:function(e){return Math.sqrt(this.distanceToSquared(e))},distanceToSquared:function(e){var t=this.x-e.x,n=this.y-e.y,r=this.z-e.z;return t*t+n*n+r*r},manhattanDistanceTo:function(e){return Math.abs(this.x-e.x)+Math.abs(this.y-e.y)+Math.abs(this.z-e.z)},setFromSpherical:function(e){return this.setFromSphericalCoords(e.radius,e.phi,e.theta)},setFromSphericalCoords:function(e,t,n){var r=Math.sin(t)*e;return this.x=r*Math.sin(n),this.y=Math.cos(t)*e,this.z=r*Math.cos(n),this},setFromCylindrical:function(e){return this.setFromCylindricalCoords(e.radius,e.theta,e.y)},setFromCylindricalCoords:function(e,t,n){return this.x=e*Math.sin(t),this.y=n,this.z=e*Math.cos(t),this},setFromMatrixPosition:function(e){var t=e.elements;return this.x=t[12],this.y=t[13],this.z=t[14],this},setFromMatrixScale:function(e){var t=this.setFromMatrixColumn(e,0).length(),n=this.setFromMatrixColumn(e,1).length(),r=this.setFromMatrixColumn(e,2).length();return this.x=t,this.y=n,this.z=r,this},setFromMatrixColumn:function(e,t){return this.fromArray(e.elements,4*t)},setFromMatrix3Column:function(e,t){return this.fromArray(e.elements,3*t)},equals:function(e){return e.x===this.x&&e.y===this.y&&e.z===this.z},fromArray:function(e,t){return void 0===t&&(t=0),this.x=e[t],this.y=e[t+1],this.z=e[t+2],this},toArray:function(e,t){return void 0===e&&(e=[]),void 0===t&&(t=0),e[t]=this.x,e[t+1]=this.y,e[t+2]=this.z,e},fromBufferAttribute:function(e,t,n){return void 0!==n&&console.warn("THREE.Vector3: offset has been removed from .fromBufferAttribute()."),this.x=e.getX(t),this.y=e.getY(t),this.z=e.getZ(t),this},random:function(){return this.x=Math.random(),this.y=Math.random(),this.z=Math.random(),this}});var er=new $n,tr=new sr,nr=new $n(0,0,0),rr=new $n(1,1,1),ir=new $n,or=new $n,ar=new $n;function sr(){this.elements=[1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1],arguments.length>0&&console.error("THREE.Matrix4: the constructor no longer reads arguments. use .set() instead.")}Object.assign(sr.prototype,{isMatrix4:!0,set:function(e,t,n,r,i,o,a,s,l,u,c,d,h,f,p,g){var v=this.elements;return v[0]=e,v[4]=t,v[8]=n,v[12]=r,v[1]=i,v[5]=o,v[9]=a,v[13]=s,v[2]=l,v[6]=u,v[10]=c,v[14]=d,v[3]=h,v[7]=f,v[11]=p,v[15]=g,this},identity:function(){return this.set(1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1),this},clone:function(){return(new sr).fromArray(this.elements)},copy:function(e){var t=this.elements,n=e.elements;return t[0]=n[0],t[1]=n[1],t[2]=n[2],t[3]=n[3],t[4]=n[4],t[5]=n[5],t[6]=n[6],t[7]=n[7],t[8]=n[8],t[9]=n[9],t[10]=n[10],t[11]=n[11],t[12]=n[12],t[13]=n[13],t[14]=n[14],t[15]=n[15],this},copyPosition:function(e){var t=this.elements,n=e.elements;return t[12]=n[12],t[13]=n[13],t[14]=n[14],this},extractBasis:function(e,t,n){return e.setFromMatrixColumn(this,0),t.setFromMatrixColumn(this,1),n.setFromMatrixColumn(this,2),this},makeBasis:function(e,t,n){return this.set(e.x,t.x,n.x,0,e.y,t.y,n.y,0,e.z,t.z,n.z,0,0,0,0,1),this},extractRotation:function(e){var t=this.elements,n=e.elements,r=1/er.setFromMatrixColumn(e,0).length(),i=1/er.setFromMatrixColumn(e,1).length(),o=1/er.setFromMatrixColumn(e,2).length();return t[0]=n[0]*r,t[1]=n[1]*r,t[2]=n[2]*r,t[3]=0,t[4]=n[4]*i,t[5]=n[5]*i,t[6]=n[6]*i,t[7]=0,t[8]=n[8]*o,t[9]=n[9]*o,t[10]=n[10]*o,t[11]=0,t[12]=0,t[13]=0,t[14]=0,t[15]=1,this},makeRotationFromEuler:function(e){e&&e.isEuler||console.error("THREE.Matrix4: .makeRotationFromEuler() now expects a Euler rotation rather than a Vector3 and order.");var t=this.elements,n=e.x,r=e.y,i=e.z,o=Math.cos(n),a=Math.sin(n),s=Math.cos(r),l=Math.sin(r),u=Math.cos(i),c=Math.sin(i);if("XYZ"===e.order){var d=o*u,h=o*c,f=a*u,p=a*c;t[0]=s*u,t[4]=-s*c,t[8]=l,t[1]=h+f*l,t[5]=d-p*l,t[9]=-a*s,t[2]=p-d*l,t[6]=f+h*l,t[10]=o*s}else if("YXZ"===e.order){var g=s*u,v=s*c,m=l*u,y=l*c;t[0]=g+y*a,t[4]=m*a-v,t[8]=o*l,t[1]=o*c,t[5]=o*u,t[9]=-a,t[2]=v*a-m,t[6]=y+g*a,t[10]=o*s}else if("ZXY"===e.order){g=s*u,v=s*c,m=l*u,y=l*c;t[0]=g-y*a,t[4]=-o*c,t[8]=m+v*a,t[1]=v+m*a,t[5]=o*u,t[9]=y-g*a,t[2]=-o*l,t[6]=a,t[10]=o*s}else if("ZYX"===e.order){d=o*u,h=o*c,f=a*u,p=a*c;t[0]=s*u,t[4]=f*l-h,t[8]=d*l+p,t[1]=s*c,t[5]=p*l+d,t[9]=h*l-f,t[2]=-l,t[6]=a*s,t[10]=o*s}else if("YZX"===e.order){var b=o*s,C=o*l,A=a*s,_=a*l;t[0]=s*u,t[4]=_-b*c,t[8]=A*c+C,t[1]=c,t[5]=o*u,t[9]=-a*u,t[2]=-l*u,t[6]=C*c+A,t[10]=b-_*c}else if("XZY"===e.order){b=o*s,C=o*l,A=a*s,_=a*l;t[0]=s*u,t[4]=-c,t[8]=l*u,t[1]=b*c+_,t[5]=o*u,t[9]=C*c-A,t[2]=A*c-C,t[6]=a*u,t[10]=_*c+b}return t[3]=0,t[7]=0,t[11]=0,t[12]=0,t[13]=0,t[14]=0,t[15]=1,this},makeRotationFromQuaternion:function(e){return this.compose(nr,e,rr)},lookAt:function(e,t,n){var r=this.elements;return ar.subVectors(e,t),0===ar.lengthSq()&&(ar.z=1),ar.normalize(),ir.crossVectors(n,ar),0===ir.lengthSq()&&(1===Math.abs(n.z)?ar.x+=1e-4:ar.z+=1e-4,ar.normalize(),ir.crossVectors(n,ar)),ir.normalize(),or.crossVectors(ar,ir),r[0]=ir.x,r[4]=or.x,r[8]=ar.x,r[1]=ir.y,r[5]=or.y,r[9]=ar.y,r[2]=ir.z,r[6]=or.z,r[10]=ar.z,this},multiply:function(e,t){return void 0!==t?(console.warn("THREE.Matrix4: .multiply() now only accepts one argument. Use .multiplyMatrices( a, b ) instead."),this.multiplyMatrices(e,t)):this.multiplyMatrices(this,e)},premultiply:function(e){return this.multiplyMatrices(e,this)},multiplyMatrices:function(e,t){var n=e.elements,r=t.elements,i=this.elements,o=n[0],a=n[4],s=n[8],l=n[12],u=n[1],c=n[5],d=n[9],h=n[13],f=n[2],p=n[6],g=n[10],v=n[14],m=n[3],y=n[7],b=n[11],C=n[15],A=r[0],_=r[4],I=r[8],x=r[12],w=r[1],M=r[5],S=r[9],T=r[13],E=r[2],O=r[6],k=r[10],L=r[14],D=r[3],P=r[7],N=r[11],j=r[15];return i[0]=o*A+a*w+s*E+l*D,i[4]=o*_+a*M+s*O+l*P,i[8]=o*I+a*S+s*k+l*N,i[12]=o*x+a*T+s*L+l*j,i[1]=u*A+c*w+d*E+h*D,i[5]=u*_+c*M+d*O+h*P,i[9]=u*I+c*S+d*k+h*N,i[13]=u*x+c*T+d*L+h*j,i[2]=f*A+p*w+g*E+v*D,i[6]=f*_+p*M+g*O+v*P,i[10]=f*I+p*S+g*k+v*N,i[14]=f*x+p*T+g*L+v*j,i[3]=m*A+y*w+b*E+C*D,i[7]=m*_+y*M+b*O+C*P,i[11]=m*I+y*S+b*k+C*N,i[15]=m*x+y*T+b*L+C*j,this},multiplyScalar:function(e){var t=this.elements;return t[0]*=e,t[4]*=e,t[8]*=e,t[12]*=e,t[1]*=e,t[5]*=e,t[9]*=e,t[13]*=e,t[2]*=e,t[6]*=e,t[10]*=e,t[14]*=e,t[3]*=e,t[7]*=e,t[11]*=e,t[15]*=e,this},determinant:function(){var e=this.elements,t=e[0],n=e[4],r=e[8],i=e[12],o=e[1],a=e[5],s=e[9],l=e[13],u=e[2],c=e[6],d=e[10],h=e[14];return e[3]*(+i*s*c-r*l*c-i*a*d+n*l*d+r*a*h-n*s*h)+e[7]*(+t*s*h-t*l*d+i*o*d-r*o*h+r*l*u-i*s*u)+e[11]*(+t*l*c-t*a*h-i*o*c+n*o*h+i*a*u-n*l*u)+e[15]*(-r*a*u-t*s*c+t*a*d+r*o*c-n*o*d+n*s*u)},transpose:function(){var e,t=this.elements;return e=t[1],t[1]=t[4],t[4]=e,e=t[2],t[2]=t[8],t[8]=e,e=t[6],t[6]=t[9],t[9]=e,e=t[3],t[3]=t[12],t[12]=e,e=t[7],t[7]=t[13],t[13]=e,e=t[11],t[11]=t[14],t[14]=e,this},setPosition:function(e,t,n){var r=this.elements;return e.isVector3?(r[12]=e.x,r[13]=e.y,r[14]=e.z):(r[12]=e,r[13]=t,r[14]=n),this},getInverse:function(e,t){void 0!==t&&console.warn("THREE.Matrix4: .getInverse() can no longer be configured to throw on degenerate.");var n=this.elements,r=e.elements,i=r[0],o=r[1],a=r[2],s=r[3],l=r[4],u=r[5],c=r[6],d=r[7],h=r[8],f=r[9],p=r[10],g=r[11],v=r[12],m=r[13],y=r[14],b=r[15],C=f*y*d-m*p*d+m*c*g-u*y*g-f*c*b+u*p*b,A=v*p*d-h*y*d-v*c*g+l*y*g+h*c*b-l*p*b,_=h*m*d-v*f*d+v*u*g-l*m*g-h*u*b+l*f*b,I=v*f*c-h*m*c-v*u*p+l*m*p+h*u*y-l*f*y,x=i*C+o*A+a*_+s*I;if(0===x)return this.set(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0);var w=1/x;return n[0]=C*w,n[1]=(m*p*s-f*y*s-m*a*g+o*y*g+f*a*b-o*p*b)*w,n[2]=(u*y*s-m*c*s+m*a*d-o*y*d-u*a*b+o*c*b)*w,n[3]=(f*c*s-u*p*s-f*a*d+o*p*d+u*a*g-o*c*g)*w,n[4]=A*w,n[5]=(h*y*s-v*p*s+v*a*g-i*y*g-h*a*b+i*p*b)*w,n[6]=(v*c*s-l*y*s-v*a*d+i*y*d+l*a*b-i*c*b)*w,n[7]=(l*p*s-h*c*s+h*a*d-i*p*d-l*a*g+i*c*g)*w,n[8]=_*w,n[9]=(v*f*s-h*m*s-v*o*g+i*m*g+h*o*b-i*f*b)*w,n[10]=(l*m*s-v*u*s+v*o*d-i*m*d-l*o*b+i*u*b)*w,n[11]=(h*u*s-l*f*s-h*o*d+i*f*d+l*o*g-i*u*g)*w,n[12]=I*w,n[13]=(h*m*a-v*f*a+v*o*p-i*m*p-h*o*y+i*f*y)*w,n[14]=(v*u*a-l*m*a-v*o*c+i*m*c+l*o*y-i*u*y)*w,n[15]=(l*f*a-h*u*a+h*o*c-i*f*c-l*o*p+i*u*p)*w,this},scale:function(e){var t=this.elements,n=e.x,r=e.y,i=e.z;return t[0]*=n,t[4]*=r,t[8]*=i,t[1]*=n,t[5]*=r,t[9]*=i,t[2]*=n,t[6]*=r,t[10]*=i,t[3]*=n,t[7]*=r,t[11]*=i,this},getMaxScaleOnAxis:function(){var e=this.elements,t=e[0]*e[0]+e[1]*e[1]+e[2]*e[2],n=e[4]*e[4]+e[5]*e[5]+e[6]*e[6],r=e[8]*e[8]+e[9]*e[9]+e[10]*e[10];return Math.sqrt(Math.max(t,n,r))},makeTranslation:function(e,t,n){return this.set(1,0,0,e,0,1,0,t,0,0,1,n,0,0,0,1),this},makeRotationX:function(e){var t=Math.cos(e),n=Math.sin(e);return this.set(1,0,0,0,0,t,-n,0,0,n,t,0,0,0,0,1),this},makeRotationY:function(e){var t=Math.cos(e),n=Math.sin(e);return this.set(t,0,n,0,0,1,0,0,-n,0,t,0,0,0,0,1),this},makeRotationZ:function(e){var t=Math.cos(e),n=Math.sin(e);return this.set(t,-n,0,0,n,t,0,0,0,0,1,0,0,0,0,1),this},makeRotationAxis:function(e,t){var n=Math.cos(t),r=Math.sin(t),i=1-n,o=e.x,a=e.y,s=e.z,l=i*o,u=i*a;return this.set(l*o+n,l*a-r*s,l*s+r*a,0,l*a+r*s,u*a+n,u*s-r*o,0,l*s-r*a,u*s+r*o,i*s*s+n,0,0,0,0,1),this},makeScale:function(e,t,n){return this.set(e,0,0,0,0,t,0,0,0,0,n,0,0,0,0,1),this},makeShear:function(e,t,n){return this.set(1,t,n,0,e,1,n,0,e,t,1,0,0,0,0,1),this},compose:function(e,t,n){var r=this.elements,i=t._x,o=t._y,a=t._z,s=t._w,l=i+i,u=o+o,c=a+a,d=i*l,h=i*u,f=i*c,p=o*u,g=o*c,v=a*c,m=s*l,y=s*u,b=s*c,C=n.x,A=n.y,_=n.z;return r[0]=(1-(p+v))*C,r[1]=(h+b)*C,r[2]=(f-y)*C,r[3]=0,r[4]=(h-b)*A,r[5]=(1-(d+v))*A,r[6]=(g+m)*A,r[7]=0,r[8]=(f+y)*_,r[9]=(g-m)*_,r[10]=(1-(d+p))*_,r[11]=0,r[12]=e.x,r[13]=e.y,r[14]=e.z,r[15]=1,this},decompose:function(e,t,n){var r=this.elements,i=er.set(r[0],r[1],r[2]).length(),o=er.set(r[4],r[5],r[6]).length(),a=er.set(r[8],r[9],r[10]).length();this.determinant()<0&&(i=-i),e.x=r[12],e.y=r[13],e.z=r[14],tr.copy(this);var s=1/i,l=1/o,u=1/a;return tr.elements[0]*=s,tr.elements[1]*=s,tr.elements[2]*=s,tr.elements[4]*=l,tr.elements[5]*=l,tr.elements[6]*=l,tr.elements[8]*=u,tr.elements[9]*=u,tr.elements[10]*=u,t.setFromRotationMatrix(tr),n.x=i,n.y=o,n.z=a,this},makePerspective:function(e,t,n,r,i,o){void 0===o&&console.warn("THREE.Matrix4: .makePerspective() has been redefined and has a new signature. Please check the docs.");var a=this.elements,s=2*i/(t-e),l=2*i/(n-r),u=(t+e)/(t-e),c=(n+r)/(n-r),d=-(o+i)/(o-i),h=-2*o*i/(o-i);return a[0]=s,a[4]=0,a[8]=u,a[12]=0,a[1]=0,a[5]=l,a[9]=c,a[13]=0,a[2]=0,a[6]=0,a[10]=d,a[14]=h,a[3]=0,a[7]=0,a[11]=-1,a[15]=0,this},makeOrthographic:function(e,t,n,r,i,o){var a=this.elements,s=1/(t-e),l=1/(n-r),u=1/(o-i),c=(t+e)*s,d=(n+r)*l,h=(o+i)*u;return a[0]=2*s,a[4]=0,a[8]=0,a[12]=-c,a[1]=0,a[5]=2*l,a[9]=0,a[13]=-d,a[2]=0,a[6]=0,a[10]=-2*u,a[14]=-h,a[3]=0,a[7]=0,a[11]=0,a[15]=1,this},equals:function(e){for(var t=this.elements,n=e.elements,r=0;r<16;r++)if(t[r]!==n[r])return!1;return!0},fromArray:function(e,t){void 0===t&&(t=0);for(var n=0;n<16;n++)this.elements[n]=e[n+t];return this},toArray:function(e,t){void 0===e&&(e=[]),void 0===t&&(t=0);var n=this.elements;return e[t]=n[0],e[t+1]=n[1],e[t+2]=n[2],e[t+3]=n[3],e[t+4]=n[4],e[t+5]=n[5],e[t+6]=n[6],e[t+7]=n[7],e[t+8]=n[8],e[t+9]=n[9],e[t+10]=n[10],e[t+11]=n[11],e[t+12]=n[12],e[t+13]=n[13],e[t+14]=n[14],e[t+15]=n[15],e}});var lr=new sr,ur=new Qn;function cr(e,t,n,r){this._x=e||0,this._y=t||0,this._z=n||0,this._order=r||cr.DefaultOrder}function dr(){this.mask=1}cr.RotationOrders=["XYZ","YZX","ZXY","XZY","YXZ","ZYX"],cr.DefaultOrder="XYZ",Object.defineProperties(cr.prototype,{x:{get:function(){return this._x},set:function(e){this._x=e,this._onChangeCallback()}},y:{get:function(){return this._y},set:function(e){this._y=e,this._onChangeCallback()}},z:{get:function(){return this._z},set:function(e){this._z=e,this._onChangeCallback()}},order:{get:function(){return this._order},set:function(e){this._order=e,this._onChangeCallback()}}}),Object.assign(cr.prototype,{isEuler:!0,set:function(e,t,n,r){return this._x=e,this._y=t,this._z=n,this._order=r||this._order,this._onChangeCallback(),this},clone:function(){return new this.constructor(this._x,this._y,this._z,this._order)},copy:function(e){return this._x=e._x,this._y=e._y,this._z=e._z,this._order=e._order,this._onChangeCallback(),this},setFromRotationMatrix:function(e,t,n){var r=Hn.clamp,i=e.elements,o=i[0],a=i[4],s=i[8],l=i[1],u=i[5],c=i[9],d=i[2],h=i[6],f=i[10];switch(t=t||this._order){case"XYZ":this._y=Math.asin(r(s,-1,1)),Math.abs(s)<.9999999?(this._x=Math.atan2(-c,f),this._z=Math.atan2(-a,o)):(this._x=Math.atan2(h,u),this._z=0);break;case"YXZ":this._x=Math.asin(-r(c,-1,1)),Math.abs(c)<.9999999?(this._y=Math.atan2(s,f),this._z=Math.atan2(l,u)):(this._y=Math.atan2(-d,o),this._z=0);break;case"ZXY":this._x=Math.asin(r(h,-1,1)),Math.abs(h)<.9999999?(this._y=Math.atan2(-d,f),this._z=Math.atan2(-a,u)):(this._y=0,this._z=Math.atan2(l,o));break;case"ZYX":this._y=Math.asin(-r(d,-1,1)),Math.abs(d)<.9999999?(this._x=Math.atan2(h,f),this._z=Math.atan2(l,o)):(this._x=0,this._z=Math.atan2(-a,u));break;case"YZX":this._z=Math.asin(r(l,-1,1)),Math.abs(l)<.9999999?(this._x=Math.atan2(-c,u),this._y=Math.atan2(-d,o)):(this._x=0,this._y=Math.atan2(s,f));break;case"XZY":this._z=Math.asin(-r(a,-1,1)),Math.abs(a)<.9999999?(this._x=Math.atan2(h,u),this._y=Math.atan2(s,o)):(this._x=Math.atan2(-c,f),this._y=0);break;default:console.warn("THREE.Euler: .setFromRotationMatrix() encountered an unknown order: "+t)}return this._order=t,!1!==n&&this._onChangeCallback(),this},setFromQuaternion:function(e,t,n){return lr.makeRotationFromQuaternion(e),this.setFromRotationMatrix(lr,t,n)},setFromVector3:function(e,t){return this.set(e.x,e.y,e.z,t||this._order)},reorder:function(e){return ur.setFromEuler(this),this.setFromQuaternion(ur,e)},equals:function(e){return e._x===this._x&&e._y===this._y&&e._z===this._z&&e._order===this._order},fromArray:function(e){return this._x=e[0],this._y=e[1],this._z=e[2],void 0!==e[3]&&(this._order=e[3]),this._onChangeCallback(),this},toArray:function(e,t){return void 0===e&&(e=[]),void 0===t&&(t=0),e[t]=this._x,e[t+1]=this._y,e[t+2]=this._z,e[t+3]=this._order,e},toVector3:function(e){return e?e.set(this._x,this._y,this._z):new $n(this._x,this._y,this._z)},_onChange:function(e){return this._onChangeCallback=e,this},_onChangeCallback:function(){}}),Object.assign(dr.prototype,{set:function(e){this.mask=1<<e|0},enable:function(e){this.mask|=1<<e|0},enableAll:function(){this.mask=-1},toggle:function(e){this.mask^=1<<e|0},disable:function(e){this.mask&=~(1<<e|0)},disableAll:function(){this.mask=0},test:function(e){return 0!=(this.mask&e.mask)}});let hr=0;const fr=new $n,pr=new Qn,gr=new sr,vr=new $n,mr=new $n,yr=new $n,br=new Qn,Cr=new $n(1,0,0),Ar=new $n(0,1,0),_r=new $n(0,0,1),Ir={type:"added"},xr={type:"removed"};function wr(){Object.defineProperty(this,"id",{value:hr++}),this.uuid=Hn.generateUUID(),this.name="",this.type="Object3D",this.parent=null,this.children=[],this.up=wr.DefaultUp.clone();var e=new $n,t=new cr,n=new Qn,r=new $n(1,1,1);t._onChange((function(){n.setFromEuler(t,!1)})),n._onChange((function(){t.setFromQuaternion(n,void 0,!1)})),Object.defineProperties(this,{position:{configurable:!0,enumerable:!0,value:e},rotation:{configurable:!0,enumerable:!0,value:t},quaternion:{configurable:!0,enumerable:!0,value:n},scale:{configurable:!0,enumerable:!0,value:r},modelViewMatrix:{value:new sr},normalMatrix:{value:new Vn}}),this.matrix=new sr,this.matrixWorld=new sr,this.matrixAutoUpdate=wr.DefaultMatrixAutoUpdate,this.matrixWorldNeedsUpdate=!1,this.layers=new dr,this.visible=!0,this.castShadow=!1,this.receiveShadow=!1,this.frustumCulled=!0,this.renderOrder=0,this.userData={}}function Mr(){wr.call(this),this.type="Scene",this.background=null,this.environment=null,this.fog=null,this.overrideMaterial=null,this.autoUpdate=!0,"undefined"!=typeof __THREE_DEVTOOLS__&&__THREE_DEVTOOLS__.dispatchEvent(new CustomEvent("observe",{detail:this}))}wr.DefaultUp=new $n(0,1,0),wr.DefaultMatrixAutoUpdate=!0,wr.prototype=Object.assign(Object.create(Rn.prototype),{constructor:wr,isObject3D:!0,onBeforeRender:function(){},onAfterRender:function(){},applyMatrix4:function(e){this.matrixAutoUpdate&&this.updateMatrix(),this.matrix.premultiply(e),this.matrix.decompose(this.position,this.quaternion,this.scale)},applyQuaternion:function(e){return this.quaternion.premultiply(e),this},setRotationFromAxisAngle:function(e,t){this.quaternion.setFromAxisAngle(e,t)},setRotationFromEuler:function(e){this.quaternion.setFromEuler(e,!0)},setRotationFromMatrix:function(e){this.quaternion.setFromRotationMatrix(e)},setRotationFromQuaternion:function(e){this.quaternion.copy(e)},rotateOnAxis:function(e,t){return pr.setFromAxisAngle(e,t),this.quaternion.multiply(pr),this},rotateOnWorldAxis:function(e,t){return pr.setFromAxisAngle(e,t),this.quaternion.premultiply(pr),this},rotateX:function(e){return this.rotateOnAxis(Cr,e)},rotateY:function(e){return this.rotateOnAxis(Ar,e)},rotateZ:function(e){return this.rotateOnAxis(_r,e)},translateOnAxis:function(e,t){return fr.copy(e).applyQuaternion(this.quaternion),this.position.add(fr.multiplyScalar(t)),this},translateX:function(e){return this.translateOnAxis(Cr,e)},translateY:function(e){return this.translateOnAxis(Ar,e)},translateZ:function(e){return this.translateOnAxis(_r,e)},localToWorld:function(e){return e.applyMatrix4(this.matrixWorld)},worldToLocal:function(e){return e.applyMatrix4(gr.getInverse(this.matrixWorld))},lookAt:function(e,t,n){e.isVector3?vr.copy(e):vr.set(e,t,n);var r=this.parent;this.updateWorldMatrix(!0,!1),mr.setFromMatrixPosition(this.matrixWorld),this.isCamera||this.isLight?gr.lookAt(mr,vr,this.up):gr.lookAt(vr,mr,this.up),this.quaternion.setFromRotationMatrix(gr),r&&(gr.extractRotation(r.matrixWorld),pr.setFromRotationMatrix(gr),this.quaternion.premultiply(pr.inverse()))},add:function(e){if(arguments.length>1){for(var t=0;t<arguments.length;t++)this.add(arguments[t]);return this}return e===this?(console.error("THREE.Object3D.add: object can't be added as a child of itself.",e),this):(e&&e.isObject3D?(null!==e.parent&&e.parent.remove(e),e.parent=this,this.children.push(e),e.dispatchEvent(Ir)):console.error("THREE.Object3D.add: object not an instance of THREE.Object3D.",e),this)},remove:function(e){if(arguments.length>1){for(var t=0;t<arguments.length;t++)this.remove(arguments[t]);return this}var n=this.children.indexOf(e);return-1!==n&&(e.parent=null,this.children.splice(n,1),e.dispatchEvent(xr)),this},attach:function(e){return this.updateWorldMatrix(!0,!1),gr.getInverse(this.matrixWorld),null!==e.parent&&(e.parent.updateWorldMatrix(!0,!1),gr.multiply(e.parent.matrixWorld)),e.applyMatrix4(gr),e.updateWorldMatrix(!1,!1),this.add(e),this},getObjectById:function(e){return this.getObjectByProperty("id",e)},getObjectByName:function(e){return this.getObjectByProperty("name",e)},getObjectByProperty:function(e,t){if(this[e]===t)return this;for(var n=0,r=this.children.length;n<r;n++){var i=this.children[n].getObjectByProperty(e,t);if(void 0!==i)return i}},getWorldPosition:function(e){return void 0===e&&(console.warn("THREE.Object3D: .getWorldPosition() target is now required"),e=new $n),this.updateMatrixWorld(!0),e.setFromMatrixPosition(this.matrixWorld)},getWorldQuaternion:function(e){return void 0===e&&(console.warn("THREE.Object3D: .getWorldQuaternion() target is now required"),e=new Qn),this.updateMatrixWorld(!0),this.matrixWorld.decompose(mr,e,yr),e},getWorldScale:function(e){return void 0===e&&(console.warn("THREE.Object3D: .getWorldScale() target is now required"),e=new $n),this.updateMatrixWorld(!0),this.matrixWorld.decompose(mr,br,e),e},getWorldDirection:function(e){void 0===e&&(console.warn("THREE.Object3D: .getWorldDirection() target is now required"),e=new $n),this.updateMatrixWorld(!0);var t=this.matrixWorld.elements;return e.set(t[8],t[9],t[10]).normalize()},raycast:function(){},traverse:function(e){e(this);for(var t=this.children,n=0,r=t.length;n<r;n++)t[n].traverse(e)},traverseVisible:function(e){if(!1!==this.visible){e(this);for(var t=this.children,n=0,r=t.length;n<r;n++)t[n].traverseVisible(e)}},traverseAncestors:function(e){var t=this.parent;null!==t&&(e(t),t.traverseAncestors(e))},updateMatrix:function(){this.matrix.compose(this.position,this.quaternion,this.scale),this.matrixWorldNeedsUpdate=!0},updateMatrixWorld:function(e){this.matrixAutoUpdate&&this.updateMatrix(),(this.matrixWorldNeedsUpdate||e)&&(null===this.parent?this.matrixWorld.copy(this.matrix):this.matrixWorld.multiplyMatrices(this.parent.matrixWorld,this.matrix),this.matrixWorldNeedsUpdate=!1,e=!0);for(var t=this.children,n=0,r=t.length;n<r;n++)t[n].updateMatrixWorld(e)},updateWorldMatrix:function(e,t){var n=this.parent;if(!0===e&&null!==n&&n.updateWorldMatrix(!0,!1),this.matrixAutoUpdate&&this.updateMatrix(),null===this.parent?this.matrixWorld.copy(this.matrix):this.matrixWorld.multiplyMatrices(this.parent.matrixWorld,this.matrix),!0===t)for(var r=this.children,i=0,o=r.length;i<o;i++)r[i].updateWorldMatrix(!1,!0)},toJSON:function(e){var t=void 0===e||"string"==typeof e,n={};t&&(e={geometries:{},materials:{},textures:{},images:{},shapes:{}},n.metadata={version:4.5,type:"Object",generator:"Object3D.toJSON"});var r={};function i(t,n){return void 0===t[n.uuid]&&(t[n.uuid]=n.toJSON(e)),n.uuid}if(r.uuid=this.uuid,r.type=this.type,""!==this.name&&(r.name=this.name),!0===this.castShadow&&(r.castShadow=!0),!0===this.receiveShadow&&(r.receiveShadow=!0),!1===this.visible&&(r.visible=!1),!1===this.frustumCulled&&(r.frustumCulled=!1),0!==this.renderOrder&&(r.renderOrder=this.renderOrder),"{}"!==JSON.stringify(this.userData)&&(r.userData=this.userData),r.layers=this.layers.mask,r.matrix=this.matrix.toArray(),!1===this.matrixAutoUpdate&&(r.matrixAutoUpdate=!1),this.isInstancedMesh&&(r.type="InstancedMesh",r.count=this.count,r.instanceMatrix=this.instanceMatrix.toJSON()),this.isMesh||this.isLine||this.isPoints){r.geometry=i(e.geometries,this.geometry);var o=this.geometry.parameters;if(void 0!==o&&void 0!==o.shapes){var a=o.shapes;if(Array.isArray(a))for(var s=0,l=a.length;s<l;s++){var u=a[s];i(e.shapes,u)}else i(e.shapes,a)}}if(void 0!==this.material)if(Array.isArray(this.material)){var c=[];for(s=0,l=this.material.length;s<l;s++)c.push(i(e.materials,this.material[s]));r.material=c}else r.material=i(e.materials,this.material);if(this.children.length>0){r.children=[];for(s=0;s<this.children.length;s++)r.children.push(this.children[s].toJSON(e).object)}if(t){var d=g(e.geometries),h=g(e.materials),f=g(e.textures),p=g(e.images);a=g(e.shapes);d.length>0&&(n.geometries=d),h.length>0&&(n.materials=h),f.length>0&&(n.textures=f),p.length>0&&(n.images=p),a.length>0&&(n.shapes=a)}return n.object=r,n;function g(e){var t=[];for(var n in e){var r=e[n];delete r.metadata,t.push(r)}return t}},clone:function(e){return(new this.constructor).copy(this,e)},copy:function(e,t){if(void 0===t&&(t=!0),this.name=e.name,this.up.copy(e.up),this.position.copy(e.position),this.quaternion.copy(e.quaternion),this.scale.copy(e.scale),this.matrix.copy(e.matrix),this.matrixWorld.copy(e.matrixWorld),this.matrixAutoUpdate=e.matrixAutoUpdate,this.matrixWorldNeedsUpdate=e.matrixWorldNeedsUpdate,this.layers.mask=e.layers.mask,this.visible=e.visible,this.castShadow=e.castShadow,this.receiveShadow=e.receiveShadow,this.frustumCulled=e.frustumCulled,this.renderOrder=e.renderOrder,this.userData=JSON.parse(JSON.stringify(e.userData)),!0===t)for(var n=0;n<e.children.length;n++){var r=e.children[n];this.add(r.clone())}return this}}),Mr.prototype=Object.assign(Object.create(wr.prototype),{constructor:Mr,isScene:!0,copy:function(e,t){return wr.prototype.copy.call(this,e,t),null!==e.background&&(this.background=e.background.clone()),null!==e.environment&&(this.environment=e.environment.clone()),null!==e.fog&&(this.fog=e.fog.clone()),null!==e.overrideMaterial&&(this.overrideMaterial=e.overrideMaterial.clone()),this.autoUpdate=e.autoUpdate,this.matrixAutoUpdate=e.matrixAutoUpdate,this},toJSON:function(e){var t=wr.prototype.toJSON.call(this,e);return null!==this.background&&(t.object.background=this.background.toJSON(e)),null!==this.environment&&(t.object.environment=this.environment.toJSON(e)),null!==this.fog&&(t.object.fog=this.fog.toJSON()),t},dispose:function(){this.dispatchEvent({type:"dispose"})}});var Sr=[new $n,new $n,new $n,new $n,new $n,new $n,new $n,new $n],Tr=new $n,Er=new Br,Or=new $n,kr=new $n,Lr=new $n,Dr=new $n,Pr=new $n,Nr=new $n,jr=new $n,Rr=new $n,zr=new $n,Fr=new $n;function Br(e,t){this.min=void 0!==e?e:new $n(1/0,1/0,1/0),this.max=void 0!==t?t:new $n(-1/0,-1/0,-1/0)}function Hr(e,t,n,r,i){var o,a;for(o=0,a=e.length-3;o<=a;o+=3){Fr.fromArray(e,o);var s=i.x*Math.abs(Fr.x)+i.y*Math.abs(Fr.y)+i.z*Math.abs(Fr.z),l=t.dot(Fr),u=n.dot(Fr),c=r.dot(Fr);if(Math.max(-Math.max(l,u,c),Math.min(l,u,c))>s)return!1}return!0}Object.assign(Br.prototype,{isBox3:!0,set:function(e,t){return this.min.copy(e),this.max.copy(t),this},setFromArray:function(e){for(var t=1/0,n=1/0,r=1/0,i=-1/0,o=-1/0,a=-1/0,s=0,l=e.length;s<l;s+=3){var u=e[s],c=e[s+1],d=e[s+2];u<t&&(t=u),c<n&&(n=c),d<r&&(r=d),u>i&&(i=u),c>o&&(o=c),d>a&&(a=d)}return this.min.set(t,n,r),this.max.set(i,o,a),this},setFromBufferAttribute:function(e){for(var t=1/0,n=1/0,r=1/0,i=-1/0,o=-1/0,a=-1/0,s=0,l=e.count;s<l;s++){var u=e.getX(s),c=e.getY(s),d=e.getZ(s);u<t&&(t=u),c<n&&(n=c),d<r&&(r=d),u>i&&(i=u),c>o&&(o=c),d>a&&(a=d)}return this.min.set(t,n,r),this.max.set(i,o,a),this},setFromPoints:function(e){this.makeEmpty();for(var t=0,n=e.length;t<n;t++)this.expandByPoint(e[t]);return this},setFromCenterAndSize:function(e,t){var n=Tr.copy(t).multiplyScalar(.5);return this.min.copy(e).sub(n),this.max.copy(e).add(n),this},setFromObject:function(e){return this.makeEmpty(),this.expandByObject(e)},clone:function(){return(new this.constructor).copy(this)},copy:function(e){return this.min.copy(e.min),this.max.copy(e.max),this},makeEmpty:function(){return this.min.x=this.min.y=this.min.z=1/0,this.max.x=this.max.y=this.max.z=-1/0,this},isEmpty:function(){return this.max.x<this.min.x||this.max.y<this.min.y||this.max.z<this.min.z},getCenter:function(e){return void 0===e&&(console.warn("THREE.Box3: .getCenter() target is now required"),e=new $n),this.isEmpty()?e.set(0,0,0):e.addVectors(this.min,this.max).multiplyScalar(.5)},getSize:function(e){return void 0===e&&(console.warn("THREE.Box3: .getSize() target is now required"),e=new $n),this.isEmpty()?e.set(0,0,0):e.subVectors(this.max,this.min)},expandByPoint:function(e){return this.min.min(e),this.max.max(e),this},expandByVector:function(e){return this.min.sub(e),this.max.add(e),this},expandByScalar:function(e){return this.min.addScalar(-e),this.max.addScalar(e),this},expandByObject:function(e){e.updateWorldMatrix(!1,!1);var t=e.geometry;void 0!==t&&(null===t.boundingBox&&t.computeBoundingBox(),Er.copy(t.boundingBox),Er.applyMatrix4(e.matrixWorld),this.union(Er));for(var n=e.children,r=0,i=n.length;r<i;r++)this.expandByObject(n[r]);return this},containsPoint:function(e){return!(e.x<this.min.x||e.x>this.max.x||e.y<this.min.y||e.y>this.max.y||e.z<this.min.z||e.z>this.max.z)},containsBox:function(e){return this.min.x<=e.min.x&&e.max.x<=this.max.x&&this.min.y<=e.min.y&&e.max.y<=this.max.y&&this.min.z<=e.min.z&&e.max.z<=this.max.z},getParameter:function(e,t){return void 0===t&&(console.warn("THREE.Box3: .getParameter() target is now required"),t=new $n),t.set((e.x-this.min.x)/(this.max.x-this.min.x),(e.y-this.min.y)/(this.max.y-this.min.y),(e.z-this.min.z)/(this.max.z-this.min.z))},intersectsBox:function(e){return!(e.max.x<this.min.x||e.min.x>this.max.x||e.max.y<this.min.y||e.min.y>this.max.y||e.max.z<this.min.z||e.min.z>this.max.z)},intersectsSphere:function(e){return this.clampPoint(e.center,Tr),Tr.distanceToSquared(e.center)<=e.radius*e.radius},intersectsPlane:function(e){var t,n;return e.normal.x>0?(t=e.normal.x*this.min.x,n=e.normal.x*this.max.x):(t=e.normal.x*this.max.x,n=e.normal.x*this.min.x),e.normal.y>0?(t+=e.normal.y*this.min.y,n+=e.normal.y*this.max.y):(t+=e.normal.y*this.max.y,n+=e.normal.y*this.min.y),e.normal.z>0?(t+=e.normal.z*this.min.z,n+=e.normal.z*this.max.z):(t+=e.normal.z*this.max.z,n+=e.normal.z*this.min.z),t<=-e.constant&&n>=-e.constant},intersectsTriangle:function(e){if(this.isEmpty())return!1;this.getCenter(jr),Rr.subVectors(this.max,jr),Or.subVectors(e.a,jr),kr.subVectors(e.b,jr),Lr.subVectors(e.c,jr),Dr.subVectors(kr,Or),Pr.subVectors(Lr,kr),Nr.subVectors(Or,Lr);var t=[0,-Dr.z,Dr.y,0,-Pr.z,Pr.y,0,-Nr.z,Nr.y,Dr.z,0,-Dr.x,Pr.z,0,-Pr.x,Nr.z,0,-Nr.x,-Dr.y,Dr.x,0,-Pr.y,Pr.x,0,-Nr.y,Nr.x,0];return!!Hr(t,Or,kr,Lr,Rr)&&(!!Hr(t=[1,0,0,0,1,0,0,0,1],Or,kr,Lr,Rr)&&(zr.crossVectors(Dr,Pr),Hr(t=[zr.x,zr.y,zr.z],Or,kr,Lr,Rr)))},clampPoint:function(e,t){return void 0===t&&(console.warn("THREE.Box3: .clampPoint() target is now required"),t=new $n),t.copy(e).clamp(this.min,this.max)},distanceToPoint:function(e){return Tr.copy(e).clamp(this.min,this.max).sub(e).length()},getBoundingSphere:function(e){return void 0===e&&console.error("THREE.Box3: .getBoundingSphere() target is now required"),this.getCenter(e.center),e.radius=.5*this.getSize(Tr).length(),e},intersect:function(e){return this.min.max(e.min),this.max.min(e.max),this.isEmpty()&&this.makeEmpty(),this},union:function(e){return this.min.min(e.min),this.max.max(e.max),this},applyMatrix4:function(e){return this.isEmpty()||(Sr[0].set(this.min.x,this.min.y,this.min.z).applyMatrix4(e),Sr[1].set(this.min.x,this.min.y,this.max.z).applyMatrix4(e),Sr[2].set(this.min.x,this.max.y,this.min.z).applyMatrix4(e),Sr[3].set(this.min.x,this.max.y,this.max.z).applyMatrix4(e),Sr[4].set(this.max.x,this.min.y,this.min.z).applyMatrix4(e),Sr[5].set(this.max.x,this.min.y,this.max.z).applyMatrix4(e),Sr[6].set(this.max.x,this.max.y,this.min.z).applyMatrix4(e),Sr[7].set(this.max.x,this.max.y,this.max.z).applyMatrix4(e),this.setFromPoints(Sr)),this},translate:function(e){return this.min.add(e),this.max.add(e),this},equals:function(e){return e.min.equals(this.min)&&e.max.equals(this.max)}});var Ur=new Br;function Vr(e,t){this.center=void 0!==e?e:new $n,this.radius=void 0!==t?t:-1}Object.assign(Vr.prototype,{set:function(e,t){return this.center.copy(e),this.radius=t,this},setFromPoints:function(e,t){var n=this.center;void 0!==t?n.copy(t):Ur.setFromPoints(e).getCenter(n);for(var r=0,i=0,o=e.length;i<o;i++)r=Math.max(r,n.distanceToSquared(e[i]));return this.radius=Math.sqrt(r),this},clone:function(){return(new this.constructor).copy(this)},copy:function(e){return this.center.copy(e.center),this.radius=e.radius,this},isEmpty:function(){return this.radius<0},makeEmpty:function(){return this.center.set(0,0,0),this.radius=-1,this},containsPoint:function(e){return e.distanceToSquared(this.center)<=this.radius*this.radius},distanceToPoint:function(e){return e.distanceTo(this.center)-this.radius},intersectsSphere:function(e){var t=this.radius+e.radius;return e.center.distanceToSquared(this.center)<=t*t},intersectsBox:function(e){return e.intersectsSphere(this)},intersectsPlane:function(e){return Math.abs(e.distanceToPoint(this.center))<=this.radius},clampPoint:function(e,t){var n=this.center.distanceToSquared(e);return void 0===t&&(console.warn("THREE.Sphere: .clampPoint() target is now required"),t=new $n),t.copy(e),n>this.radius*this.radius&&(t.sub(this.center).normalize(),t.multiplyScalar(this.radius).add(this.center)),t},getBoundingBox:function(e){return void 0===e&&(console.warn("THREE.Sphere: .getBoundingBox() target is now required"),e=new Br),this.isEmpty()?(e.makeEmpty(),e):(e.set(this.center,this.center),e.expandByScalar(this.radius),e)},applyMatrix4:function(e){return this.center.applyMatrix4(e),this.radius=this.radius*e.getMaxScaleOnAxis(),this},translate:function(e){return this.center.add(e),this},equals:function(e){return e.center.equals(this.center)&&e.radius===this.radius}});var Yr=new $n,Wr=new $n,Gr=new $n,Xr=new $n,qr=new $n,Zr=new $n,Qr=new $n;function Kr(e,t){this.origin=void 0!==e?e:new $n,this.direction=void 0!==t?t:new $n(0,0,-1)}Object.assign(Kr.prototype,{set:function(e,t){return this.origin.copy(e),this.direction.copy(t),this},clone:function(){return(new this.constructor).copy(this)},copy:function(e){return this.origin.copy(e.origin),this.direction.copy(e.direction),this},at:function(e,t){return void 0===t&&(console.warn("THREE.Ray: .at() target is now required"),t=new $n),t.copy(this.direction).multiplyScalar(e).add(this.origin)},lookAt:function(e){return this.direction.copy(e).sub(this.origin).normalize(),this},recast:function(e){return this.origin.copy(this.at(e,Yr)),this},closestPointToPoint:function(e,t){void 0===t&&(console.warn("THREE.Ray: .closestPointToPoint() target is now required"),t=new $n),t.subVectors(e,this.origin);var n=t.dot(this.direction);return n<0?t.copy(this.origin):t.copy(this.direction).multiplyScalar(n).add(this.origin)},distanceToPoint:function(e){return Math.sqrt(this.distanceSqToPoint(e))},distanceSqToPoint:function(e){var t=Yr.subVectors(e,this.origin).dot(this.direction);return t<0?this.origin.distanceToSquared(e):(Yr.copy(this.direction).multiplyScalar(t).add(this.origin),Yr.distanceToSquared(e))},distanceSqToSegment:function(e,t,n,r){Wr.copy(e).add(t).multiplyScalar(.5),Gr.copy(t).sub(e).normalize(),Xr.copy(this.origin).sub(Wr);var i,o,a,s,l=.5*e.distanceTo(t),u=-this.direction.dot(Gr),c=Xr.dot(this.direction),d=-Xr.dot(Gr),h=Xr.lengthSq(),f=Math.abs(1-u*u);if(f>0)if(o=u*c-d,s=l*f,(i=u*d-c)>=0)if(o>=-s)if(o<=s){var p=1/f;a=(i*=p)*(i+u*(o*=p)+2*c)+o*(u*i+o+2*d)+h}else o=l,a=-(i=Math.max(0,-(u*o+c)))*i+o*(o+2*d)+h;else o=-l,a=-(i=Math.max(0,-(u*o+c)))*i+o*(o+2*d)+h;else o<=-s?a=-(i=Math.max(0,-(-u*l+c)))*i+(o=i>0?-l:Math.min(Math.max(-l,-d),l))*(o+2*d)+h:o<=s?(i=0,a=(o=Math.min(Math.max(-l,-d),l))*(o+2*d)+h):a=-(i=Math.max(0,-(u*l+c)))*i+(o=i>0?l:Math.min(Math.max(-l,-d),l))*(o+2*d)+h;else o=u>0?-l:l,a=-(i=Math.max(0,-(u*o+c)))*i+o*(o+2*d)+h;return n&&n.copy(this.direction).multiplyScalar(i).add(this.origin),r&&r.copy(Gr).multiplyScalar(o).add(Wr),a},intersectSphere:function(e,t){Yr.subVectors(e.center,this.origin);var n=Yr.dot(this.direction),r=Yr.dot(Yr)-n*n,i=e.radius*e.radius;if(r>i)return null;var o=Math.sqrt(i-r),a=n-o,s=n+o;return a<0&&s<0?null:a<0?this.at(s,t):this.at(a,t)},intersectsSphere:function(e){return this.distanceSqToPoint(e.center)<=e.radius*e.radius},distanceToPlane:function(e){var t=e.normal.dot(this.direction);if(0===t)return 0===e.distanceToPoint(this.origin)?0:null;var n=-(this.origin.dot(e.normal)+e.constant)/t;return n>=0?n:null},intersectPlane:function(e,t){var n=this.distanceToPlane(e);return null===n?null:this.at(n,t)},intersectsPlane:function(e){var t=e.distanceToPoint(this.origin);return 0===t||e.normal.dot(this.direction)*t<0},intersectBox:function(e,t){var n,r,i,o,a,s,l=1/this.direction.x,u=1/this.direction.y,c=1/this.direction.z,d=this.origin;return l>=0?(n=(e.min.x-d.x)*l,r=(e.max.x-d.x)*l):(n=(e.max.x-d.x)*l,r=(e.min.x-d.x)*l),u>=0?(i=(e.min.y-d.y)*u,o=(e.max.y-d.y)*u):(i=(e.max.y-d.y)*u,o=(e.min.y-d.y)*u),n>o||i>r?null:((i>n||n!=n)&&(n=i),(o<r||r!=r)&&(r=o),c>=0?(a=(e.min.z-d.z)*c,s=(e.max.z-d.z)*c):(a=(e.max.z-d.z)*c,s=(e.min.z-d.z)*c),n>s||a>r?null:((a>n||n!=n)&&(n=a),(s<r||r!=r)&&(r=s),r<0?null:this.at(n>=0?n:r,t)))},intersectsBox:function(e){return null!==this.intersectBox(e,Yr)},intersectTriangle:function(e,t,n,r,i){qr.subVectors(t,e),Zr.subVectors(n,e),Qr.crossVectors(qr,Zr);var o,a=this.direction.dot(Qr);if(a>0){if(r)return null;o=1}else{if(!(a<0))return null;o=-1,a=-a}Xr.subVectors(this.origin,e);var s=o*this.direction.dot(Zr.crossVectors(Xr,Zr));if(s<0)return null;var l=o*this.direction.dot(qr.cross(Xr));if(l<0)return null;if(s+l>a)return null;var u=-o*Xr.dot(Qr);return u<0?null:this.at(u/a,i)},applyMatrix4:function(e){return this.origin.applyMatrix4(e),this.direction.transformDirection(e),this},equals:function(e){return e.origin.equals(this.origin)&&e.direction.equals(this.direction)}});var Jr=new $n,$r=new $n,ei=new Vn;function ti(e,t){this.normal=void 0!==e?e:new $n(1,0,0),this.constant=void 0!==t?t:0}Object.assign(ti.prototype,{isPlane:!0,set:function(e,t){return this.normal.copy(e),this.constant=t,this},setComponents:function(e,t,n,r){return this.normal.set(e,t,n),this.constant=r,this},setFromNormalAndCoplanarPoint:function(e,t){return this.normal.copy(e),this.constant=-t.dot(this.normal),this},setFromCoplanarPoints:function(e,t,n){var r=Jr.subVectors(n,t).cross($r.subVectors(e,t)).normalize();return this.setFromNormalAndCoplanarPoint(r,e),this},clone:function(){return(new this.constructor).copy(this)},copy:function(e){return this.normal.copy(e.normal),this.constant=e.constant,this},normalize:function(){var e=1/this.normal.length();return this.normal.multiplyScalar(e),this.constant*=e,this},negate:function(){return this.constant*=-1,this.normal.negate(),this},distanceToPoint:function(e){return this.normal.dot(e)+this.constant},distanceToSphere:function(e){return this.distanceToPoint(e.center)-e.radius},projectPoint:function(e,t){return void 0===t&&(console.warn("THREE.Plane: .projectPoint() target is now required"),t=new $n),t.copy(this.normal).multiplyScalar(-this.distanceToPoint(e)).add(e)},intersectLine:function(e,t){void 0===t&&(console.warn("THREE.Plane: .intersectLine() target is now required"),t=new $n);var n=e.delta(Jr),r=this.normal.dot(n);if(0===r)return 0===this.distanceToPoint(e.start)?t.copy(e.start):void 0;var i=-(e.start.dot(this.normal)+this.constant)/r;return i<0||i>1?void 0:t.copy(n).multiplyScalar(i).add(e.start)},intersectsLine:function(e){var t=this.distanceToPoint(e.start),n=this.distanceToPoint(e.end);return t<0&&n>0||n<0&&t>0},intersectsBox:function(e){return e.intersectsPlane(this)},intersectsSphere:function(e){return e.intersectsPlane(this)},coplanarPoint:function(e){return void 0===e&&(console.warn("THREE.Plane: .coplanarPoint() target is now required"),e=new $n),e.copy(this.normal).multiplyScalar(-this.constant)},applyMatrix4:function(e,t){var n=t||ei.getNormalMatrix(e),r=this.coplanarPoint(Jr).applyMatrix4(e),i=this.normal.applyMatrix3(n).normalize();return this.constant=-r.dot(i),this},translate:function(e){return this.constant-=e.dot(this.normal),this},equals:function(e){return e.normal.equals(this.normal)&&e.constant===this.constant}});var ni=new $n,ri=new $n,ii=new $n,oi=new $n,ai=new $n,si=new $n,li=new $n,ui=new $n,ci=new $n,di=new $n;function hi(e,t,n){this.a=void 0!==e?e:new $n,this.b=void 0!==t?t:new $n,this.c=void 0!==n?n:new $n}Object.assign(hi,{getNormal:function(e,t,n,r){void 0===r&&(console.warn("THREE.Triangle: .getNormal() target is now required"),r=new $n),r.subVectors(n,t),ni.subVectors(e,t),r.cross(ni);var i=r.lengthSq();return i>0?r.multiplyScalar(1/Math.sqrt(i)):r.set(0,0,0)},getBarycoord:function(e,t,n,r,i){ni.subVectors(r,t),ri.subVectors(n,t),ii.subVectors(e,t);var o=ni.dot(ni),a=ni.dot(ri),s=ni.dot(ii),l=ri.dot(ri),u=ri.dot(ii),c=o*l-a*a;if(void 0===i&&(console.warn("THREE.Triangle: .getBarycoord() target is now required"),i=new $n),0===c)return i.set(-2,-1,-1);var d=1/c,h=(l*s-a*u)*d,f=(o*u-a*s)*d;return i.set(1-h-f,f,h)},containsPoint:function(e,t,n,r){return hi.getBarycoord(e,t,n,r,oi),oi.x>=0&&oi.y>=0&&oi.x+oi.y<=1},getUV:function(e,t,n,r,i,o,a,s){return this.getBarycoord(e,t,n,r,oi),s.set(0,0),s.addScaledVector(i,oi.x),s.addScaledVector(o,oi.y),s.addScaledVector(a,oi.z),s},isFrontFacing:function(e,t,n,r){return ni.subVectors(n,t),ri.subVectors(e,t),ni.cross(ri).dot(r)<0}}),Object.assign(hi.prototype,{set:function(e,t,n){return this.a.copy(e),this.b.copy(t),this.c.copy(n),this},setFromPointsAndIndices:function(e,t,n,r){return this.a.copy(e[t]),this.b.copy(e[n]),this.c.copy(e[r]),this},clone:function(){return(new this.constructor).copy(this)},copy:function(e){return this.a.copy(e.a),this.b.copy(e.b),this.c.copy(e.c),this},getArea:function(){return ni.subVectors(this.c,this.b),ri.subVectors(this.a,this.b),.5*ni.cross(ri).length()},getMidpoint:function(e){return void 0===e&&(console.warn("THREE.Triangle: .getMidpoint() target is now required"),e=new $n),e.addVectors(this.a,this.b).add(this.c).multiplyScalar(1/3)},getNormal:function(e){return hi.getNormal(this.a,this.b,this.c,e)},getPlane:function(e){return void 0===e&&(console.warn("THREE.Triangle: .getPlane() target is now required"),e=new ti),e.setFromCoplanarPoints(this.a,this.b,this.c)},getBarycoord:function(e,t){return hi.getBarycoord(e,this.a,this.b,this.c,t)},getUV:function(e,t,n,r,i){return hi.getUV(e,this.a,this.b,this.c,t,n,r,i)},containsPoint:function(e){return hi.containsPoint(e,this.a,this.b,this.c)},isFrontFacing:function(e){return hi.isFrontFacing(this.a,this.b,this.c,e)},intersectsBox:function(e){return e.intersectsTriangle(this)},closestPointToPoint:function(e,t){void 0===t&&(console.warn("THREE.Triangle: .closestPointToPoint() target is now required"),t=new $n);var n,r,i=this.a,o=this.b,a=this.c;ai.subVectors(o,i),si.subVectors(a,i),ui.subVectors(e,i);var s=ai.dot(ui),l=si.dot(ui);if(s<=0&&l<=0)return t.copy(i);ci.subVectors(e,o);var u=ai.dot(ci),c=si.dot(ci);if(u>=0&&c<=u)return t.copy(o);var d=s*c-u*l;if(d<=0&&s>=0&&u<=0)return n=s/(s-u),t.copy(i).addScaledVector(ai,n);di.subVectors(e,a);var h=ai.dot(di),f=si.dot(di);if(f>=0&&h<=f)return t.copy(a);var p=h*l-s*f;if(p<=0&&l>=0&&f<=0)return r=l/(l-f),t.copy(i).addScaledVector(si,r);var g=u*f-h*c;if(g<=0&&c-u>=0&&h-f>=0)return li.subVectors(a,o),r=(c-u)/(c-u+(h-f)),t.copy(o).addScaledVector(li,r);var v=1/(g+p+d);return n=p*v,r=d*v,t.copy(i).addScaledVector(ai,n).addScaledVector(si,r)},equals:function(e){return e.a.equals(this.a)&&e.b.equals(this.b)&&e.c.equals(this.c)}});var fi={aliceblue:15792383,antiquewhite:16444375,aqua:65535,aquamarine:8388564,azure:15794175,beige:16119260,bisque:16770244,black:0,blanchedalmond:16772045,blue:255,blueviolet:9055202,brown:10824234,burlywood:14596231,cadetblue:6266528,chartreuse:8388352,chocolate:13789470,coral:16744272,cornflowerblue:6591981,cornsilk:16775388,crimson:14423100,cyan:65535,darkblue:139,darkcyan:35723,darkgoldenrod:12092939,darkgray:11119017,darkgreen:25600,darkgrey:11119017,darkkhaki:12433259,darkmagenta:9109643,darkolivegreen:5597999,darkorange:16747520,darkorchid:10040012,darkred:9109504,darksalmon:15308410,darkseagreen:9419919,darkslateblue:4734347,darkslategray:3100495,darkslategrey:3100495,darkturquoise:52945,darkviolet:9699539,deeppink:16716947,deepskyblue:49151,dimgray:6908265,dimgrey:6908265,dodgerblue:2003199,firebrick:11674146,floralwhite:16775920,forestgreen:2263842,fuchsia:16711935,gainsboro:14474460,ghostwhite:16316671,gold:16766720,goldenrod:14329120,gray:8421504,green:32768,greenyellow:11403055,grey:8421504,honeydew:15794160,hotpink:16738740,indianred:13458524,indigo:4915330,ivory:16777200,khaki:15787660,lavender:15132410,lavenderblush:16773365,lawngreen:8190976,lemonchiffon:16775885,lightblue:11393254,lightcoral:15761536,lightcyan:14745599,lightgoldenrodyellow:16448210,lightgray:13882323,lightgreen:9498256,lightgrey:13882323,lightpink:16758465,lightsalmon:16752762,lightseagreen:2142890,lightskyblue:8900346,lightslategray:7833753,lightslategrey:7833753,lightsteelblue:11584734,lightyellow:16777184,lime:65280,limegreen:3329330,linen:16445670,magenta:16711935,maroon:8388608,mediumaquamarine:6737322,mediumblue:205,mediumorchid:12211667,mediumpurple:9662683,mediumseagreen:3978097,mediumslateblue:8087790,mediumspringgreen:64154,mediumturquoise:4772300,mediumvioletred:13047173,midnightblue:1644912,mintcream:16121850,mistyrose:16770273,moccasin:16770229,navajowhite:16768685,navy:128,oldlace:16643558,olive:8421376,olivedrab:7048739,orange:16753920,orangered:16729344,orchid:14315734,palegoldenrod:15657130,palegreen:10025880,paleturquoise:11529966,palevioletred:14381203,papayawhip:16773077,peachpuff:16767673,peru:13468991,pink:16761035,plum:14524637,powderblue:11591910,purple:8388736,rebeccapurple:6697881,red:16711680,rosybrown:12357519,royalblue:4286945,saddlebrown:9127187,salmon:16416882,sandybrown:16032864,seagreen:3050327,seashell:16774638,sienna:10506797,silver:12632256,skyblue:8900331,slateblue:6970061,slategray:7372944,slategrey:7372944,snow:16775930,springgreen:65407,steelblue:4620980,tan:13808780,teal:32896,thistle:14204888,tomato:16737095,turquoise:4251856,violet:15631086,wheat:16113331,white:16777215,whitesmoke:16119285,yellow:16776960,yellowgreen:10145074},pi={h:0,s:0,l:0},gi={h:0,s:0,l:0};function vi(e,t,n){return void 0===t&&void 0===n?this.set(e):this.setRGB(e,t,n)}function mi(e,t,n){return n<0&&(n+=1),n>1&&(n-=1),n<1/6?e+6*(t-e)*n:n<.5?t:n<2/3?e+6*(t-e)*(2/3-n):e}function yi(e){return e<.04045?.0773993808*e:Math.pow(.9478672986*e+.0521327014,2.4)}function bi(e){return e<.0031308?12.92*e:1.055*Math.pow(e,.41666)-.055}function Ci(e,t,n,r,i,o){this.a=e,this.b=t,this.c=n,this.normal=r&&r.isVector3?r:new $n,this.vertexNormals=Array.isArray(r)?r:[],this.color=i&&i.isColor?i:new vi,this.vertexColors=Array.isArray(i)?i:[],this.materialIndex=void 0!==o?o:0}Object.assign(vi.prototype,{isColor:!0,r:1,g:1,b:1,set:function(e){return e&&e.isColor?this.copy(e):"number"==typeof e?this.setHex(e):"string"==typeof e&&this.setStyle(e),this},setScalar:function(e){return this.r=e,this.g=e,this.b=e,this},setHex:function(e){return e=Math.floor(e),this.r=(e>>16&255)/255,this.g=(e>>8&255)/255,this.b=(255&e)/255,this},setRGB:function(e,t,n){return this.r=e,this.g=t,this.b=n,this},setHSL:function(e,t,n){if(e=Hn.euclideanModulo(e,1),t=Hn.clamp(t,0,1),n=Hn.clamp(n,0,1),0===t)this.r=this.g=this.b=n;else{var r=n<=.5?n*(1+t):n+t-n*t,i=2*n-r;this.r=mi(i,r,e+1/3),this.g=mi(i,r,e),this.b=mi(i,r,e-1/3)}return this},setStyle:function(e){function t(t){void 0!==t&&parseFloat(t)<1&&console.warn("THREE.Color: Alpha component of "+e+" will be ignored.")}var n;if(n=/^((?:rgb|hsl)a?)\(\s*([^\)]*)\)/.exec(e)){var r,i=n[1],o=n[2];switch(i){case"rgb":case"rgba":if(r=/^(\d+)\s*,\s*(\d+)\s*,\s*(\d+)\s*(,\s*([0-9]*\.?[0-9]+)\s*)?$/.exec(o))return this.r=Math.min(255,parseInt(r[1],10))/255,this.g=Math.min(255,parseInt(r[2],10))/255,this.b=Math.min(255,parseInt(r[3],10))/255,t(r[5]),this;if(r=/^(\d+)\%\s*,\s*(\d+)\%\s*,\s*(\d+)\%\s*(,\s*([0-9]*\.?[0-9]+)\s*)?$/.exec(o))return this.r=Math.min(100,parseInt(r[1],10))/100,this.g=Math.min(100,parseInt(r[2],10))/100,this.b=Math.min(100,parseInt(r[3],10))/100,t(r[5]),this;break;case"hsl":case"hsla":if(r=/^([0-9]*\.?[0-9]+)\s*,\s*(\d+)\%\s*,\s*(\d+)\%\s*(,\s*([0-9]*\.?[0-9]+)\s*)?$/.exec(o)){var a=parseFloat(r[1])/360,s=parseInt(r[2],10)/100,l=parseInt(r[3],10)/100;return t(r[5]),this.setHSL(a,s,l)}}}else if(n=/^\#([A-Fa-f0-9]+)$/.exec(e)){var u=n[1],c=u.length;if(3===c)return this.r=parseInt(u.charAt(0)+u.charAt(0),16)/255,this.g=parseInt(u.charAt(1)+u.charAt(1),16)/255,this.b=parseInt(u.charAt(2)+u.charAt(2),16)/255,this;if(6===c)return this.r=parseInt(u.charAt(0)+u.charAt(1),16)/255,this.g=parseInt(u.charAt(2)+u.charAt(3),16)/255,this.b=parseInt(u.charAt(4)+u.charAt(5),16)/255,this}return e&&e.length>0?this.setColorName(e):this},setColorName:function(e){var t=fi[e];return void 0!==t?this.setHex(t):console.warn("THREE.Color: Unknown color "+e),this},clone:function(){return new this.constructor(this.r,this.g,this.b)},copy:function(e){return this.r=e.r,this.g=e.g,this.b=e.b,this},copyGammaToLinear:function(e,t){return void 0===t&&(t=2),this.r=Math.pow(e.r,t),this.g=Math.pow(e.g,t),this.b=Math.pow(e.b,t),this},copyLinearToGamma:function(e,t){void 0===t&&(t=2);var n=t>0?1/t:1;return this.r=Math.pow(e.r,n),this.g=Math.pow(e.g,n),this.b=Math.pow(e.b,n),this},convertGammaToLinear:function(e){return this.copyGammaToLinear(this,e),this},convertLinearToGamma:function(e){return this.copyLinearToGamma(this,e),this},copySRGBToLinear:function(e){return this.r=yi(e.r),this.g=yi(e.g),this.b=yi(e.b),this},copyLinearToSRGB:function(e){return this.r=bi(e.r),this.g=bi(e.g),this.b=bi(e.b),this},convertSRGBToLinear:function(){return this.copySRGBToLinear(this),this},convertLinearToSRGB:function(){return this.copyLinearToSRGB(this),this},getHex:function(){return 255*this.r<<16^255*this.g<<8^255*this.b<<0},getHexString:function(){return("000000"+this.getHex().toString(16)).slice(-6)},getHSL:function(e){void 0===e&&(console.warn("THREE.Color: .getHSL() target is now required"),e={h:0,s:0,l:0});var t,n,r=this.r,i=this.g,o=this.b,a=Math.max(r,i,o),s=Math.min(r,i,o),l=(s+a)/2;if(s===a)t=0,n=0;else{var u=a-s;switch(n=l<=.5?u/(a+s):u/(2-a-s),a){case r:t=(i-o)/u+(i<o?6:0);break;case i:t=(o-r)/u+2;break;case o:t=(r-i)/u+4}t/=6}return e.h=t,e.s=n,e.l=l,e},getStyle:function(){return"rgb("+(255*this.r|0)+","+(255*this.g|0)+","+(255*this.b|0)+")"},offsetHSL:function(e,t,n){return this.getHSL(pi),pi.h+=e,pi.s+=t,pi.l+=n,this.setHSL(pi.h,pi.s,pi.l),this},add:function(e){return this.r+=e.r,this.g+=e.g,this.b+=e.b,this},addColors:function(e,t){return this.r=e.r+t.r,this.g=e.g+t.g,this.b=e.b+t.b,this},addScalar:function(e){return this.r+=e,this.g+=e,this.b+=e,this},sub:function(e){return this.r=Math.max(0,this.r-e.r),this.g=Math.max(0,this.g-e.g),this.b=Math.max(0,this.b-e.b),this},multiply:function(e){return this.r*=e.r,this.g*=e.g,this.b*=e.b,this},multiplyScalar:function(e){return this.r*=e,this.g*=e,this.b*=e,this},lerp:function(e,t){return this.r+=(e.r-this.r)*t,this.g+=(e.g-this.g)*t,this.b+=(e.b-this.b)*t,this},lerpHSL:function(e,t){this.getHSL(pi),e.getHSL(gi);var n=Hn.lerp(pi.h,gi.h,t),r=Hn.lerp(pi.s,gi.s,t),i=Hn.lerp(pi.l,gi.l,t);return this.setHSL(n,r,i),this},equals:function(e){return e.r===this.r&&e.g===this.g&&e.b===this.b},fromArray:function(e,t){return void 0===t&&(t=0),this.r=e[t],this.g=e[t+1],this.b=e[t+2],this},toArray:function(e,t){return void 0===e&&(e=[]),void 0===t&&(t=0),e[t]=this.r,e[t+1]=this.g,e[t+2]=this.b,e},toJSON:function(){return this.getHex()}}),vi.NAMES=fi,Object.assign(Ci.prototype,{clone:function(){return(new this.constructor).copy(this)},copy:function(e){this.a=e.a,this.b=e.b,this.c=e.c,this.normal.copy(e.normal),this.color.copy(e.color),this.materialIndex=e.materialIndex;for(var t=0,n=e.vertexNormals.length;t<n;t++)this.vertexNormals[t]=e.vertexNormals[t].clone();for(t=0,n=e.vertexColors.length;t<n;t++)this.vertexColors[t]=e.vertexColors[t].clone();return this}});var Ai=0;function _i(){Object.defineProperty(this,"id",{value:Ai++}),this.uuid=Hn.generateUUID(),this.name="",this.type="Material",this.fog=!0,this.blending=_,this.side=v,this.flatShading=!1,this.vertexColors=!1,this.opacity=1,this.transparent=!1,this.blendSrc=j,this.blendDst=R,this.blendEquation=S,this.blendSrcAlpha=null,this.blendDstAlpha=null,this.blendEquationAlpha=null,this.depthFunc=G,this.depthTest=!0,this.depthWrite=!0,this.stencilWriteMask=255,this.stencilFunc=Sn,this.stencilRef=0,this.stencilFuncMask=255,this.stencilFail=fn,this.stencilZFail=fn,this.stencilZPass=fn,this.stencilWrite=!1,this.clippingPlanes=null,this.clipIntersection=!1,this.clipShadows=!1,this.shadowSide=null,this.colorWrite=!0,this.precision=null,this.polygonOffset=!1,this.polygonOffsetFactor=0,this.polygonOffsetUnits=0,this.dithering=!1,this.alphaTest=0,this.premultipliedAlpha=!1,this.visible=!0,this.toneMapped=!0,this.userData={},this.version=0}function Ii(e){_i.call(this),this.type="MeshBasicMaterial",this.color=new vi(16777215),this.map=null,this.lightMap=null,this.lightMapIntensity=1,this.aoMap=null,this.aoMapIntensity=1,this.specularMap=null,this.alphaMap=null,this.envMap=null,this.combine=K,this.reflectivity=1,this.refractionRatio=.98,this.wireframe=!1,this.wireframeLinewidth=1,this.wireframeLinecap="round",this.wireframeLinejoin="round",this.skinning=!1,this.morphTargets=!1,this.setValues(e)}_i.prototype=Object.assign(Object.create(Rn.prototype),{constructor:_i,isMaterial:!0,onBeforeCompile:function(){},setValues:function(e){if(void 0!==e)for(var t in e){var n=e[t];if(void 0!==n)if("shading"!==t){var r=this[t];void 0!==r?r&&r.isColor?r.set(n):r&&r.isVector3&&n&&n.isVector3?r.copy(n):this[t]=n:console.warn("THREE."+this.type+": '"+t+"' is not a property of this material.")}else console.warn("THREE."+this.type+": .shading has been removed. Use the boolean .flatShading instead."),this.flatShading=n===b;else console.warn("THREE.Material: '"+t+"' parameter is undefined.")}},toJSON:function(e){var t=void 0===e||"string"==typeof e;t&&(e={textures:{},images:{}});var n={metadata:{version:4.5,type:"Material",generator:"Material.toJSON"}};function r(e){var t=[];for(var n in e){var r=e[n];delete r.metadata,t.push(r)}return t}if(n.uuid=this.uuid,n.type=this.type,""!==this.name&&(n.name=this.name),this.color&&this.color.isColor&&(n.color=this.color.getHex()),void 0!==this.roughness&&(n.roughness=this.roughness),void 0!==this.metalness&&(n.metalness=this.metalness),this.sheen&&this.sheen.isColor&&(n.sheen=this.sheen.getHex()),this.emissive&&this.emissive.isColor&&(n.emissive=this.emissive.getHex()),this.emissiveIntensity&&1!==this.emissiveIntensity&&(n.emissiveIntensity=this.emissiveIntensity),this.specular&&this.specular.isColor&&(n.specular=this.specular.getHex()),void 0!==this.shininess&&(n.shininess=this.shininess),void 0!==this.clearcoat&&(n.clearcoat=this.clearcoat),void 0!==this.clearcoatRoughness&&(n.clearcoatRoughness=this.clearcoatRoughness),this.clearcoatMap&&this.clearcoatMap.isTexture&&(n.clearcoatMap=this.clearcoatMap.toJSON(e).uuid),this.clearcoatRoughnessMap&&this.clearcoatRoughnessMap.isTexture&&(n.clearcoatRoughnessMap=this.clearcoatRoughnessMap.toJSON(e).uuid),this.clearcoatNormalMap&&this.clearcoatNormalMap.isTexture&&(n.clearcoatNormalMap=this.clearcoatNormalMap.toJSON(e).uuid,n.clearcoatNormalScale=this.clearcoatNormalScale.toArray()),this.map&&this.map.isTexture&&(n.map=this.map.toJSON(e).uuid),this.matcap&&this.matcap.isTexture&&(n.matcap=this.matcap.toJSON(e).uuid),this.alphaMap&&this.alphaMap.isTexture&&(n.alphaMap=this.alphaMap.toJSON(e).uuid),this.lightMap&&this.lightMap.isTexture&&(n.lightMap=this.lightMap.toJSON(e).uuid),this.aoMap&&this.aoMap.isTexture&&(n.aoMap=this.aoMap.toJSON(e).uuid,n.aoMapIntensity=this.aoMapIntensity),this.bumpMap&&this.bumpMap.isTexture&&(n.bumpMap=this.bumpMap.toJSON(e).uuid,n.bumpScale=this.bumpScale),this.normalMap&&this.normalMap.isTexture&&(n.normalMap=this.normalMap.toJSON(e).uuid,n.normalMapType=this.normalMapType,n.normalScale=this.normalScale.toArray()),this.displacementMap&&this.displacementMap.isTexture&&(n.displacementMap=this.displacementMap.toJSON(e).uuid,n.displacementScale=this.displacementScale,n.displacementBias=this.displacementBias),this.roughnessMap&&this.roughnessMap.isTexture&&(n.roughnessMap=this.roughnessMap.toJSON(e).uuid),this.metalnessMap&&this.metalnessMap.isTexture&&(n.metalnessMap=this.metalnessMap.toJSON(e).uuid),this.emissiveMap&&this.emissiveMap.isTexture&&(n.emissiveMap=this.emissiveMap.toJSON(e).uuid),this.specularMap&&this.specularMap.isTexture&&(n.specularMap=this.specularMap.toJSON(e).uuid),this.envMap&&this.envMap.isTexture&&(n.envMap=this.envMap.toJSON(e).uuid,n.reflectivity=this.reflectivity,n.refractionRatio=this.refractionRatio,void 0!==this.combine&&(n.combine=this.combine),void 0!==this.envMapIntensity&&(n.envMapIntensity=this.envMapIntensity)),this.gradientMap&&this.gradientMap.isTexture&&(n.gradientMap=this.gradientMap.toJSON(e).uuid),void 0!==this.size&&(n.size=this.size),void 0!==this.sizeAttenuation&&(n.sizeAttenuation=this.sizeAttenuation),this.blending!==_&&(n.blending=this.blending),!0===this.flatShading&&(n.flatShading=this.flatShading),this.side!==v&&(n.side=this.side),this.vertexColors&&(n.vertexColors=!0),this.opacity<1&&(n.opacity=this.opacity),!0===this.transparent&&(n.transparent=this.transparent),n.depthFunc=this.depthFunc,n.depthTest=this.depthTest,n.depthWrite=this.depthWrite,n.stencilWrite=this.stencilWrite,n.stencilWriteMask=this.stencilWriteMask,n.stencilFunc=this.stencilFunc,n.stencilRef=this.stencilRef,n.stencilFuncMask=this.stencilFuncMask,n.stencilFail=this.stencilFail,n.stencilZFail=this.stencilZFail,n.stencilZPass=this.stencilZPass,this.rotation&&0!==this.rotation&&(n.rotation=this.rotation),!0===this.polygonOffset&&(n.polygonOffset=!0),0!==this.polygonOffsetFactor&&(n.polygonOffsetFactor=this.polygonOffsetFactor),0!==this.polygonOffsetUnits&&(n.polygonOffsetUnits=this.polygonOffsetUnits),this.linewidth&&1!==this.linewidth&&(n.linewidth=this.linewidth),void 0!==this.dashSize&&(n.dashSize=this.dashSize),void 0!==this.gapSize&&(n.gapSize=this.gapSize),void 0!==this.scale&&(n.scale=this.scale),!0===this.dithering&&(n.dithering=!0),this.alphaTest>0&&(n.alphaTest=this.alphaTest),!0===this.premultipliedAlpha&&(n.premultipliedAlpha=this.premultipliedAlpha),!0===this.wireframe&&(n.wireframe=this.wireframe),this.wireframeLinewidth>1&&(n.wireframeLinewidth=this.wireframeLinewidth),"round"!==this.wireframeLinecap&&(n.wireframeLinecap=this.wireframeLinecap),"round"!==this.wireframeLinejoin&&(n.wireframeLinejoin=this.wireframeLinejoin),!0===this.morphTargets&&(n.morphTargets=!0),!0===this.morphNormals&&(n.morphNormals=!0),!0===this.skinning&&(n.skinning=!0),!1===this.visible&&(n.visible=!1),!1===this.toneMapped&&(n.toneMapped=!1),"{}"!==JSON.stringify(this.userData)&&(n.userData=this.userData),t){var i=r(e.textures),o=r(e.images);i.length>0&&(n.textures=i),o.length>0&&(n.images=o)}return n},clone:function(){return(new this.constructor).copy(this)},copy:function(e){this.name=e.name,this.fog=e.fog,this.blending=e.blending,this.side=e.side,this.flatShading=e.flatShading,this.vertexColors=e.vertexColors,this.opacity=e.opacity,this.transparent=e.transparent,this.blendSrc=e.blendSrc,this.blendDst=e.blendDst,this.blendEquation=e.blendEquation,this.blendSrcAlpha=e.blendSrcAlpha,this.blendDstAlpha=e.blendDstAlpha,this.blendEquationAlpha=e.blendEquationAlpha,this.depthFunc=e.depthFunc,this.depthTest=e.depthTest,this.depthWrite=e.depthWrite,this.stencilWriteMask=e.stencilWriteMask,this.stencilFunc=e.stencilFunc,this.stencilRef=e.stencilRef,this.stencilFuncMask=e.stencilFuncMask,this.stencilFail=e.stencilFail,this.stencilZFail=e.stencilZFail,this.stencilZPass=e.stencilZPass,this.stencilWrite=e.stencilWrite;var t=e.clippingPlanes,n=null;if(null!==t){var r=t.length;n=new Array(r);for(var i=0;i!==r;++i)n[i]=t[i].clone()}return this.clippingPlanes=n,this.clipIntersection=e.clipIntersection,this.clipShadows=e.clipShadows,this.shadowSide=e.shadowSide,this.colorWrite=e.colorWrite,this.precision=e.precision,this.polygonOffset=e.polygonOffset,this.polygonOffsetFactor=e.polygonOffsetFactor,this.polygonOffsetUnits=e.polygonOffsetUnits,this.dithering=e.dithering,this.alphaTest=e.alphaTest,this.premultipliedAlpha=e.premultipliedAlpha,this.visible=e.visible,this.toneMapped=e.toneMapped,this.userData=JSON.parse(JSON.stringify(e.userData)),this},dispose:function(){this.dispatchEvent({type:"dispose"})}}),Object.defineProperty(_i.prototype,"needsUpdate",{set:function(e){!0===e&&this.version++}}),Ii.prototype=Object.create(_i.prototype),Ii.prototype.constructor=Ii,Ii.prototype.isMeshBasicMaterial=!0,Ii.prototype.copy=function(e){return _i.prototype.copy.call(this,e),this.color.copy(e.color),this.map=e.map,this.lightMap=e.lightMap,this.lightMapIntensity=e.lightMapIntensity,this.aoMap=e.aoMap,this.aoMapIntensity=e.aoMapIntensity,this.specularMap=e.specularMap,this.alphaMap=e.alphaMap,this.envMap=e.envMap,this.combine=e.combine,this.reflectivity=e.reflectivity,this.refractionRatio=e.refractionRatio,this.wireframe=e.wireframe,this.wireframeLinewidth=e.wireframeLinewidth,this.wireframeLinecap=e.wireframeLinecap,this.wireframeLinejoin=e.wireframeLinejoin,this.skinning=e.skinning,this.morphTargets=e.morphTargets,this};var xi=new $n;function wi(e,t,n){if(Array.isArray(e))throw new TypeError("THREE.BufferAttribute: array should be a Typed Array.");this.name="",this.array=e,this.itemSize=t,this.count=void 0!==e?e.length/t:0,this.normalized=!0===n,this.usage=Tn,this.updateRange={offset:0,count:-1},this.version=0}function Mi(e,t,n){wi.call(this,new Int8Array(e),t,n)}function Si(e,t,n){wi.call(this,new Uint8Array(e),t,n)}function Ti(e,t,n){wi.call(this,new Uint8ClampedArray(e),t,n)}function Ei(e,t,n){wi.call(this,new Int16Array(e),t,n)}function Oi(e,t,n){wi.call(this,new Uint16Array(e),t,n)}function ki(e,t,n){wi.call(this,new Int32Array(e),t,n)}function Li(e,t,n){wi.call(this,new Uint32Array(e),t,n)}function Di(e,t,n){wi.call(this,new Float32Array(e),t,n)}function Pi(e,t,n){wi.call(this,new Float64Array(e),t,n)}function Ni(){this.vertices=[],this.normals=[],this.colors=[],this.uvs=[],this.uvs2=[],this.groups=[],this.morphTargets={},this.skinWeights=[],this.skinIndices=[],this.boundingBox=null,this.boundingSphere=null,this.verticesNeedUpdate=!1,this.normalsNeedUpdate=!1,this.colorsNeedUpdate=!1,this.uvsNeedUpdate=!1,this.groupsNeedUpdate=!1}function ji(e){if(0===e.length)return-1/0;let t=e[0];for(let n=1,r=e.length;n<r;++n)e[n]>t&&(t=e[n]);return t}Object.defineProperty(wi.prototype,"needsUpdate",{set:function(e){!0===e&&this.version++}}),Object.assign(wi.prototype,{isBufferAttribute:!0,onUploadCallback:function(){},setUsage:function(e){return this.usage=e,this},copy:function(e){return this.name=e.name,this.array=new e.array.constructor(e.array),this.itemSize=e.itemSize,this.count=e.count,this.normalized=e.normalized,this.usage=e.usage,this},copyAt:function(e,t,n){e*=this.itemSize,n*=t.itemSize;for(var r=0,i=this.itemSize;r<i;r++)this.array[e+r]=t.array[n+r];return this},copyArray:function(e){return this.array.set(e),this},copyColorsArray:function(e){for(var t=this.array,n=0,r=0,i=e.length;r<i;r++){var o=e[r];void 0===o&&(console.warn("THREE.BufferAttribute.copyColorsArray(): color is undefined",r),o=new vi),t[n++]=o.r,t[n++]=o.g,t[n++]=o.b}return this},copyVector2sArray:function(e){for(var t=this.array,n=0,r=0,i=e.length;r<i;r++){var o=e[r];void 0===o&&(console.warn("THREE.BufferAttribute.copyVector2sArray(): vector is undefined",r),o=new Un),t[n++]=o.x,t[n++]=o.y}return this},copyVector3sArray:function(e){for(var t=this.array,n=0,r=0,i=e.length;r<i;r++){var o=e[r];void 0===o&&(console.warn("THREE.BufferAttribute.copyVector3sArray(): vector is undefined",r),o=new $n),t[n++]=o.x,t[n++]=o.y,t[n++]=o.z}return this},copyVector4sArray:function(e){for(var t=this.array,n=0,r=0,i=e.length;r<i;r++){var o=e[r];void 0===o&&(console.warn("THREE.BufferAttribute.copyVector4sArray(): vector is undefined",r),o=new Xn),t[n++]=o.x,t[n++]=o.y,t[n++]=o.z,t[n++]=o.w}return this},applyMatrix3:function(e){for(var t=0,n=this.count;t<n;t++)xi.x=this.getX(t),xi.y=this.getY(t),xi.z=this.getZ(t),xi.applyMatrix3(e),this.setXYZ(t,xi.x,xi.y,xi.z);return this},applyMatrix4:function(e){for(var t=0,n=this.count;t<n;t++)xi.x=this.getX(t),xi.y=this.getY(t),xi.z=this.getZ(t),xi.applyMatrix4(e),this.setXYZ(t,xi.x,xi.y,xi.z);return this},applyNormalMatrix:function(e){for(var t=0,n=this.count;t<n;t++)xi.x=this.getX(t),xi.y=this.getY(t),xi.z=this.getZ(t),xi.applyNormalMatrix(e),this.setXYZ(t,xi.x,xi.y,xi.z);return this},transformDirection:function(e){for(var t=0,n=this.count;t<n;t++)xi.x=this.getX(t),xi.y=this.getY(t),xi.z=this.getZ(t),xi.transformDirection(e),this.setXYZ(t,xi.x,xi.y,xi.z);return this},set:function(e,t){return void 0===t&&(t=0),this.array.set(e,t),this},getX:function(e){return this.array[e*this.itemSize]},setX:function(e,t){return this.array[e*this.itemSize]=t,this},getY:function(e){return this.array[e*this.itemSize+1]},setY:function(e,t){return this.array[e*this.itemSize+1]=t,this},getZ:function(e){return this.array[e*this.itemSize+2]},setZ:function(e,t){return this.array[e*this.itemSize+2]=t,this},getW:function(e){return this.array[e*this.itemSize+3]},setW:function(e,t){return this.array[e*this.itemSize+3]=t,this},setXY:function(e,t,n){return e*=this.itemSize,this.array[e+0]=t,this.array[e+1]=n,this},setXYZ:function(e,t,n,r){return e*=this.itemSize,this.array[e+0]=t,this.array[e+1]=n,this.array[e+2]=r,this},setXYZW:function(e,t,n,r,i){return e*=this.itemSize,this.array[e+0]=t,this.array[e+1]=n,this.array[e+2]=r,this.array[e+3]=i,this},onUpload:function(e){return this.onUploadCallback=e,this},clone:function(){return new this.constructor(this.array,this.itemSize).copy(this)},toJSON:function(){return{itemSize:this.itemSize,type:this.array.constructor.name,array:Array.prototype.slice.call(this.array),normalized:this.normalized}}}),Mi.prototype=Object.create(wi.prototype),Mi.prototype.constructor=Mi,Si.prototype=Object.create(wi.prototype),Si.prototype.constructor=Si,Ti.prototype=Object.create(wi.prototype),Ti.prototype.constructor=Ti,Ei.prototype=Object.create(wi.prototype),Ei.prototype.constructor=Ei,Oi.prototype=Object.create(wi.prototype),Oi.prototype.constructor=Oi,ki.prototype=Object.create(wi.prototype),ki.prototype.constructor=ki,Li.prototype=Object.create(wi.prototype),Li.prototype.constructor=Li,Di.prototype=Object.create(wi.prototype),Di.prototype.constructor=Di,Pi.prototype=Object.create(wi.prototype),Pi.prototype.constructor=Pi,Object.assign(Ni.prototype,{computeGroups:function(e){for(var t,n=[],r=void 0,i=e.faces,o=0;o<i.length;o++){var a=i[o];a.materialIndex!==r&&(r=a.materialIndex,void 0!==t&&(t.count=3*o-t.start,n.push(t)),t={start:3*o,materialIndex:r})}void 0!==t&&(t.count=3*o-t.start,n.push(t)),this.groups=n},fromGeometry:function(e){var t,n=e.faces,r=e.vertices,i=e.faceVertexUvs,o=i[0]&&i[0].length>0,a=i[1]&&i[1].length>0,s=e.morphTargets,l=s.length;if(l>0){t=[];for(var u=0;u<l;u++)t[u]={name:s[u].name,data:[]};this.morphTargets.position=t}var c,d=e.morphNormals,h=d.length;if(h>0){c=[];for(u=0;u<h;u++)c[u]={name:d[u].name,data:[]};this.morphTargets.normal=c}var f=e.skinIndices,p=e.skinWeights,g=f.length===r.length,v=p.length===r.length;r.length>0&&0===n.length&&console.error("THREE.DirectGeometry: Faceless geometries are not supported.");for(u=0;u<n.length;u++){var m=n[u];this.vertices.push(r[m.a],r[m.b],r[m.c]);var y=m.vertexNormals;if(3===y.length)this.normals.push(y[0],y[1],y[2]);else{var b=m.normal;this.normals.push(b,b,b)}var C,A=m.vertexColors;if(3===A.length)this.colors.push(A[0],A[1],A[2]);else{var _=m.color;this.colors.push(_,_,_)}if(!0===o)void 0!==(C=i[0][u])?this.uvs.push(C[0],C[1],C[2]):(console.warn("THREE.DirectGeometry.fromGeometry(): Undefined vertexUv ",u),this.uvs.push(new Un,new Un,new Un));if(!0===a)void 0!==(C=i[1][u])?this.uvs2.push(C[0],C[1],C[2]):(console.warn("THREE.DirectGeometry.fromGeometry(): Undefined vertexUv2 ",u),this.uvs2.push(new Un,new Un,new Un));for(var I=0;I<l;I++){var x=s[I].vertices;t[I].data.push(x[m.a],x[m.b],x[m.c])}for(I=0;I<h;I++){var w=d[I].vertexNormals[u];c[I].data.push(w.a,w.b,w.c)}g&&this.skinIndices.push(f[m.a],f[m.b],f[m.c]),v&&this.skinWeights.push(p[m.a],p[m.b],p[m.c])}return this.computeGroups(e),this.verticesNeedUpdate=e.verticesNeedUpdate,this.normalsNeedUpdate=e.normalsNeedUpdate,this.colorsNeedUpdate=e.colorsNeedUpdate,this.uvsNeedUpdate=e.uvsNeedUpdate,this.groupsNeedUpdate=e.groupsNeedUpdate,null!==e.boundingSphere&&(this.boundingSphere=e.boundingSphere.clone()),null!==e.boundingBox&&(this.boundingBox=e.boundingBox.clone()),this}});var Ri=1,zi=new sr,Fi=new wr,Bi=new $n,Hi=new Br,Ui=new Br,Vi=new $n;function Yi(){Object.defineProperty(this,"id",{value:Ri+=2}),this.uuid=Hn.generateUUID(),this.name="",this.type="BufferGeometry",this.index=null,this.attributes={},this.morphAttributes={},this.morphTargetsRelative=!1,this.groups=[],this.boundingBox=null,this.boundingSphere=null,this.drawRange={start:0,count:1/0},this.userData={}}Yi.prototype=Object.assign(Object.create(Rn.prototype),{constructor:Yi,isBufferGeometry:!0,getIndex:function(){return this.index},setIndex:function(e){Array.isArray(e)?this.index=new(ji(e)>65535?Li:Oi)(e,1):this.index=e},getAttribute:function(e){return this.attributes[e]},setAttribute:function(e,t){return this.attributes[e]=t,this},deleteAttribute:function(e){return delete this.attributes[e],this},addGroup:function(e,t,n){this.groups.push({start:e,count:t,materialIndex:void 0!==n?n:0})},clearGroups:function(){this.groups=[]},setDrawRange:function(e,t){this.drawRange.start=e,this.drawRange.count=t},applyMatrix4:function(e){var t=this.attributes.position;void 0!==t&&(t.applyMatrix4(e),t.needsUpdate=!0);var n=this.attributes.normal;if(void 0!==n){var r=(new Vn).getNormalMatrix(e);n.applyNormalMatrix(r),n.needsUpdate=!0}var i=this.attributes.tangent;return void 0!==i&&(i.transformDirection(e),i.needsUpdate=!0),null!==this.boundingBox&&this.computeBoundingBox(),null!==this.boundingSphere&&this.computeBoundingSphere(),this},rotateX:function(e){return zi.makeRotationX(e),this.applyMatrix4(zi),this},rotateY:function(e){return zi.makeRotationY(e),this.applyMatrix4(zi),this},rotateZ:function(e){return zi.makeRotationZ(e),this.applyMatrix4(zi),this},translate:function(e,t,n){return zi.makeTranslation(e,t,n),this.applyMatrix4(zi),this},scale:function(e,t,n){return zi.makeScale(e,t,n),this.applyMatrix4(zi),this},lookAt:function(e){return Fi.lookAt(e),Fi.updateMatrix(),this.applyMatrix4(Fi.matrix),this},center:function(){return this.computeBoundingBox(),this.boundingBox.getCenter(Bi).negate(),this.translate(Bi.x,Bi.y,Bi.z),this},setFromObject:function(e){var t=e.geometry;if(e.isPoints||e.isLine){var n=new Di(3*t.vertices.length,3),r=new Di(3*t.colors.length,3);if(this.setAttribute("position",n.copyVector3sArray(t.vertices)),this.setAttribute("color",r.copyColorsArray(t.colors)),t.lineDistances&&t.lineDistances.length===t.vertices.length){var i=new Di(t.lineDistances.length,1);this.setAttribute("lineDistance",i.copyArray(t.lineDistances))}null!==t.boundingSphere&&(this.boundingSphere=t.boundingSphere.clone()),null!==t.boundingBox&&(this.boundingBox=t.boundingBox.clone())}else e.isMesh&&t&&t.isGeometry&&this.fromGeometry(t);return this},setFromPoints:function(e){for(var t=[],n=0,r=e.length;n<r;n++){var i=e[n];t.push(i.x,i.y,i.z||0)}return this.setAttribute("position",new Di(t,3)),this},updateFromObject:function(e){var t,n=e.geometry;if(e.isMesh){var r=n.__directGeometry;if(!0===n.elementsNeedUpdate&&(r=void 0,n.elementsNeedUpdate=!1),void 0===r)return this.fromGeometry(n);r.verticesNeedUpdate=n.verticesNeedUpdate,r.normalsNeedUpdate=n.normalsNeedUpdate,r.colorsNeedUpdate=n.colorsNeedUpdate,r.uvsNeedUpdate=n.uvsNeedUpdate,r.groupsNeedUpdate=n.groupsNeedUpdate,n.verticesNeedUpdate=!1,n.normalsNeedUpdate=!1,n.colorsNeedUpdate=!1,n.uvsNeedUpdate=!1,n.groupsNeedUpdate=!1,n=r}return!0===n.verticesNeedUpdate&&(void 0!==(t=this.attributes.position)&&(t.copyVector3sArray(n.vertices),t.needsUpdate=!0),n.verticesNeedUpdate=!1),!0===n.normalsNeedUpdate&&(void 0!==(t=this.attributes.normal)&&(t.copyVector3sArray(n.normals),t.needsUpdate=!0),n.normalsNeedUpdate=!1),!0===n.colorsNeedUpdate&&(void 0!==(t=this.attributes.color)&&(t.copyColorsArray(n.colors),t.needsUpdate=!0),n.colorsNeedUpdate=!1),n.uvsNeedUpdate&&(void 0!==(t=this.attributes.uv)&&(t.copyVector2sArray(n.uvs),t.needsUpdate=!0),n.uvsNeedUpdate=!1),n.lineDistancesNeedUpdate&&(void 0!==(t=this.attributes.lineDistance)&&(t.copyArray(n.lineDistances),t.needsUpdate=!0),n.lineDistancesNeedUpdate=!1),n.groupsNeedUpdate&&(n.computeGroups(e.geometry),this.groups=n.groups,n.groupsNeedUpdate=!1),this},fromGeometry:function(e){return e.__directGeometry=(new Ni).fromGeometry(e),this.fromDirectGeometry(e.__directGeometry)},fromDirectGeometry:function(e){var t=new Float32Array(3*e.vertices.length);if(this.setAttribute("position",new wi(t,3).copyVector3sArray(e.vertices)),e.normals.length>0){var n=new Float32Array(3*e.normals.length);this.setAttribute("normal",new wi(n,3).copyVector3sArray(e.normals))}if(e.colors.length>0){var r=new Float32Array(3*e.colors.length);this.setAttribute("color",new wi(r,3).copyColorsArray(e.colors))}if(e.uvs.length>0){var i=new Float32Array(2*e.uvs.length);this.setAttribute("uv",new wi(i,2).copyVector2sArray(e.uvs))}if(e.uvs2.length>0){var o=new Float32Array(2*e.uvs2.length);this.setAttribute("uv2",new wi(o,2).copyVector2sArray(e.uvs2))}for(var a in this.groups=e.groups,e.morphTargets){for(var s=[],l=e.morphTargets[a],u=0,c=l.length;u<c;u++){var d=l[u],h=new Di(3*d.data.length,3);h.name=d.name,s.push(h.copyVector3sArray(d.data))}this.morphAttributes[a]=s}if(e.skinIndices.length>0){var f=new Di(4*e.skinIndices.length,4);this.setAttribute("skinIndex",f.copyVector4sArray(e.skinIndices))}if(e.skinWeights.length>0){var p=new Di(4*e.skinWeights.length,4);this.setAttribute("skinWeight",p.copyVector4sArray(e.skinWeights))}return null!==e.boundingSphere&&(this.boundingSphere=e.boundingSphere.clone()),null!==e.boundingBox&&(this.boundingBox=e.boundingBox.clone()),this},computeBoundingBox:function(){null===this.boundingBox&&(this.boundingBox=new Br);var e=this.attributes.position,t=this.morphAttributes.position;if(void 0!==e){if(this.boundingBox.setFromBufferAttribute(e),t)for(var n=0,r=t.length;n<r;n++){var i=t[n];Hi.setFromBufferAttribute(i),this.morphTargetsRelative?(Vi.addVectors(this.boundingBox.min,Hi.min),this.boundingBox.expandByPoint(Vi),Vi.addVectors(this.boundingBox.max,Hi.max),this.boundingBox.expandByPoint(Vi)):(this.boundingBox.expandByPoint(Hi.min),this.boundingBox.expandByPoint(Hi.max))}}else this.boundingBox.makeEmpty();(isNaN(this.boundingBox.min.x)||isNaN(this.boundingBox.min.y)||isNaN(this.boundingBox.min.z))&&console.error('THREE.BufferGeometry.computeBoundingBox: Computed min/max have NaN values. The "position" attribute is likely to have NaN values.',this)},computeBoundingSphere:function(){null===this.boundingSphere&&(this.boundingSphere=new Vr);var e=this.attributes.position,t=this.morphAttributes.position;if(e){var n=this.boundingSphere.center;if(Hi.setFromBufferAttribute(e),t)for(var r=0,i=t.length;r<i;r++){var o=t[r];Ui.setFromBufferAttribute(o),this.morphTargetsRelative?(Vi.addVectors(Hi.min,Ui.min),Hi.expandByPoint(Vi),Vi.addVectors(Hi.max,Ui.max),Hi.expandByPoint(Vi)):(Hi.expandByPoint(Ui.min),Hi.expandByPoint(Ui.max))}Hi.getCenter(n);var a=0;for(r=0,i=e.count;r<i;r++)Vi.fromBufferAttribute(e,r),a=Math.max(a,n.distanceToSquared(Vi));if(t)for(r=0,i=t.length;r<i;r++){o=t[r];for(var s=this.morphTargetsRelative,l=0,u=o.count;l<u;l++)Vi.fromBufferAttribute(o,l),s&&(Bi.fromBufferAttribute(e,l),Vi.add(Bi)),a=Math.max(a,n.distanceToSquared(Vi))}this.boundingSphere.radius=Math.sqrt(a),isNaN(this.boundingSphere.radius)&&console.error('THREE.BufferGeometry.computeBoundingSphere(): Computed radius is NaN. The "position" attribute is likely to have NaN values.',this)}},computeFaceNormals:function(){},computeVertexNormals:function(){var e=this.index,t=this.attributes;if(t.position){var n=t.position.array;if(void 0===t.normal)this.setAttribute("normal",new wi(new Float32Array(n.length),3));else for(var r=t.normal.array,i=0,o=r.length;i<o;i++)r[i]=0;var a,s,l,u=t.normal.array,c=new $n,d=new $n,h=new $n,f=new $n,p=new $n;if(e){var g=e.array;for(i=0,o=e.count;i<o;i+=3)a=3*g[i+0],s=3*g[i+1],l=3*g[i+2],c.fromArray(n,a),d.fromArray(n,s),h.fromArray(n,l),f.subVectors(h,d),p.subVectors(c,d),f.cross(p),u[a]+=f.x,u[a+1]+=f.y,u[a+2]+=f.z,u[s]+=f.x,u[s+1]+=f.y,u[s+2]+=f.z,u[l]+=f.x,u[l+1]+=f.y,u[l+2]+=f.z}else for(i=0,o=n.length;i<o;i+=9)c.fromArray(n,i),d.fromArray(n,i+3),h.fromArray(n,i+6),f.subVectors(h,d),p.subVectors(c,d),f.cross(p),u[i]=f.x,u[i+1]=f.y,u[i+2]=f.z,u[i+3]=f.x,u[i+4]=f.y,u[i+5]=f.z,u[i+6]=f.x,u[i+7]=f.y,u[i+8]=f.z;this.normalizeNormals(),t.normal.needsUpdate=!0}},merge:function(e,t){if(e&&e.isBufferGeometry){void 0===t&&(t=0,console.warn("THREE.BufferGeometry.merge(): Overwriting original geometry, starting at offset=0. Use BufferGeometryUtils.mergeBufferGeometries() for lossless merge."));var n=this.attributes;for(var r in n)if(void 0!==e.attributes[r])for(var i=n[r].array,o=e.attributes[r],a=o.array,s=o.itemSize*t,l=Math.min(a.length,i.length-s),u=0,c=s;u<l;u++,c++)i[c]=a[u];return this}console.error("THREE.BufferGeometry.merge(): geometry not an instance of THREE.BufferGeometry.",e)},normalizeNormals:function(){for(var e=this.attributes.normal,t=0,n=e.count;t<n;t++)Vi.x=e.getX(t),Vi.y=e.getY(t),Vi.z=e.getZ(t),Vi.normalize(),e.setXYZ(t,Vi.x,Vi.y,Vi.z)},toNonIndexed:function(){function e(e,t){for(var n=e.array,r=e.itemSize,i=e.normalized,o=new n.constructor(t.length*r),a=0,s=0,l=0,u=t.length;l<u;l++){a=t[l]*r;for(var c=0;c<r;c++)o[s++]=n[a++]}return new wi(o,r,i)}if(null===this.index)return console.warn("THREE.BufferGeometry.toNonIndexed(): Geometry is already non-indexed."),this;var t=new Yi,n=this.index.array,r=this.attributes;for(var i in r){var o=e(r[i],n);t.setAttribute(i,o)}var a=this.morphAttributes;for(i in a){for(var s=[],l=a[i],u=0,c=l.length;u<c;u++){o=e(l[u],n);s.push(o)}t.morphAttributes[i]=s}t.morphTargetsRelative=this.morphTargetsRelative;for(var d=this.groups,h=(u=0,d.length);u<h;u++){var f=d[u];t.addGroup(f.start,f.count,f.materialIndex)}return t},toJSON:function(){var e={metadata:{version:4.5,type:"BufferGeometry",generator:"BufferGeometry.toJSON"}};if(e.uuid=this.uuid,e.type=this.type,""!==this.name&&(e.name=this.name),Object.keys(this.userData).length>0&&(e.userData=this.userData),void 0!==this.parameters){var t=this.parameters;for(var n in t)void 0!==t[n]&&(e[n]=t[n]);return e}e.data={attributes:{}};var r=this.index;null!==r&&(e.data.index={type:r.array.constructor.name,array:Array.prototype.slice.call(r.array)});var i=this.attributes;for(var n in i){var o=(h=i[n]).toJSON();""!==h.name&&(o.name=h.name),e.data.attributes[n]=o}var a={},s=!1;for(var n in this.morphAttributes){for(var l=this.morphAttributes[n],u=[],c=0,d=l.length;c<d;c++){var h;o=(h=l[c]).toJSON();""!==h.name&&(o.name=h.name),u.push(o)}u.length>0&&(a[n]=u,s=!0)}s&&(e.data.morphAttributes=a,e.data.morphTargetsRelative=this.morphTargetsRelative);var f=this.groups;f.length>0&&(e.data.groups=JSON.parse(JSON.stringify(f)));var p=this.boundingSphere;return null!==p&&(e.data.boundingSphere={center:p.center.toArray(),radius:p.radius}),e},clone:function(){return(new Yi).copy(this)},copy:function(e){var t,n,r;this.index=null,this.attributes={},this.morphAttributes={},this.groups=[],this.boundingBox=null,this.boundingSphere=null,this.name=e.name;var i=e.index;null!==i&&this.setIndex(i.clone());var o=e.attributes;for(t in o){var a=o[t];this.setAttribute(t,a.clone())}var s=e.morphAttributes;for(t in s){var l=[],u=s[t];for(n=0,r=u.length;n<r;n++)l.push(u[n].clone());this.morphAttributes[t]=l}this.morphTargetsRelative=e.morphTargetsRelative;var c=e.groups;for(n=0,r=c.length;n<r;n++){var d=c[n];this.addGroup(d.start,d.count,d.materialIndex)}var h=e.boundingBox;null!==h&&(this.boundingBox=h.clone());var f=e.boundingSphere;return null!==f&&(this.boundingSphere=f.clone()),this.drawRange.start=e.drawRange.start,this.drawRange.count=e.drawRange.count,this.userData=e.userData,this},dispose:function(){this.dispatchEvent({type:"dispose"})}});var Wi=new sr,Gi=new Kr,Xi=new Vr,qi=new $n,Zi=new $n,Qi=new $n,Ki=new $n,Ji=new $n,$i=new $n,eo=new $n,to=new $n,no=new $n,ro=new Un,io=new Un,oo=new Un,ao=new $n,so=new $n;function lo(e,t){wr.call(this),this.type="Mesh",this.geometry=void 0!==e?e:new Yi,this.material=void 0!==t?t:new Ii,this.updateMorphTargets()}function uo(e,t,n,r,i,o,a,s){if(null===(t.side===m?r.intersectTriangle(a,o,i,!0,s):r.intersectTriangle(i,o,a,t.side!==y,s)))return null;so.copy(s),so.applyMatrix4(e.matrixWorld);var l=n.ray.origin.distanceTo(so);return l<n.near||l>n.far?null:{distance:l,point:so.clone(),object:e}}function co(e,t,n,r,i,o,a,s,l,u,c,d){qi.fromBufferAttribute(i,u),Zi.fromBufferAttribute(i,c),Qi.fromBufferAttribute(i,d);var h=e.morphTargetInfluences;if(t.morphTargets&&o&&h){eo.set(0,0,0),to.set(0,0,0),no.set(0,0,0);for(var f=0,p=o.length;f<p;f++){var g=h[f],v=o[f];0!==g&&(Ki.fromBufferAttribute(v,u),Ji.fromBufferAttribute(v,c),$i.fromBufferAttribute(v,d),a?(eo.addScaledVector(Ki,g),to.addScaledVector(Ji,g),no.addScaledVector($i,g)):(eo.addScaledVector(Ki.sub(qi),g),to.addScaledVector(Ji.sub(Zi),g),no.addScaledVector($i.sub(Qi),g)))}qi.add(eo),Zi.add(to),Qi.add(no)}e.isSkinnedMesh&&(e.boneTransform(u,qi),e.boneTransform(c,Zi),e.boneTransform(d,Qi));var m=uo(e,t,n,r,qi,Zi,Qi,ao);if(m){s&&(ro.fromBufferAttribute(s,u),io.fromBufferAttribute(s,c),oo.fromBufferAttribute(s,d),m.uv=hi.getUV(ao,qi,Zi,Qi,ro,io,oo,new Un)),l&&(ro.fromBufferAttribute(l,u),io.fromBufferAttribute(l,c),oo.fromBufferAttribute(l,d),m.uv2=hi.getUV(ao,qi,Zi,Qi,ro,io,oo,new Un));var y=new Ci(u,c,d);hi.getNormal(qi,Zi,Qi,y.normal),m.face=y}return m}lo.prototype=Object.assign(Object.create(wr.prototype),{constructor:lo,isMesh:!0,copy:function(e){return wr.prototype.copy.call(this,e),void 0!==e.morphTargetInfluences&&(this.morphTargetInfluences=e.morphTargetInfluences.slice()),void 0!==e.morphTargetDictionary&&(this.morphTargetDictionary=Object.assign({},e.morphTargetDictionary)),this},updateMorphTargets:function(){var e,t,n,r=this.geometry;if(r.isBufferGeometry){var i=r.morphAttributes,o=Object.keys(i);if(o.length>0){var a=i[o[0]];if(void 0!==a)for(this.morphTargetInfluences=[],this.morphTargetDictionary={},e=0,t=a.length;e<t;e++)n=a[e].name||String(e),this.morphTargetInfluences.push(0),this.morphTargetDictionary[n]=e}}else{var s=r.morphTargets;void 0!==s&&s.length>0&&console.error("THREE.Mesh.updateMorphTargets() no longer supports THREE.Geometry. Use THREE.BufferGeometry instead.")}},raycast:function(e,t){var n,r=this.geometry,i=this.material,o=this.matrixWorld;if(void 0!==i&&(null===r.boundingSphere&&r.computeBoundingSphere(),Xi.copy(r.boundingSphere),Xi.applyMatrix4(o),!1!==e.ray.intersectsSphere(Xi)&&(Wi.getInverse(o),Gi.copy(e.ray).applyMatrix4(Wi),null===r.boundingBox||!1!==Gi.intersectsBox(r.boundingBox))))if(r.isBufferGeometry){var a,s,l,u,c,d,h,f,p,g=r.index,v=r.attributes.position,m=r.morphAttributes.position,y=r.morphTargetsRelative,b=r.attributes.uv,C=r.attributes.uv2,A=r.groups,_=r.drawRange;if(null!==g)if(Array.isArray(i))for(u=0,d=A.length;u<d;u++)for(p=i[(f=A[u]).materialIndex],c=Math.max(f.start,_.start),h=Math.min(f.start+f.count,_.start+_.count);c<h;c+=3)a=g.getX(c),s=g.getX(c+1),l=g.getX(c+2),(n=co(this,p,e,Gi,v,m,y,b,C,a,s,l))&&(n.faceIndex=Math.floor(c/3),n.face.materialIndex=f.materialIndex,t.push(n));else for(u=Math.max(0,_.start),d=Math.min(g.count,_.start+_.count);u<d;u+=3)a=g.getX(u),s=g.getX(u+1),l=g.getX(u+2),(n=co(this,i,e,Gi,v,m,y,b,C,a,s,l))&&(n.faceIndex=Math.floor(u/3),t.push(n));else if(void 0!==v)if(Array.isArray(i))for(u=0,d=A.length;u<d;u++)for(p=i[(f=A[u]).materialIndex],c=Math.max(f.start,_.start),h=Math.min(f.start+f.count,_.start+_.count);c<h;c+=3)(n=co(this,p,e,Gi,v,m,y,b,C,a=c,s=c+1,l=c+2))&&(n.faceIndex=Math.floor(c/3),n.face.materialIndex=f.materialIndex,t.push(n));else for(u=Math.max(0,_.start),d=Math.min(v.count,_.start+_.count);u<d;u+=3)(n=co(this,i,e,Gi,v,m,y,b,C,a=u,s=u+1,l=u+2))&&(n.faceIndex=Math.floor(u/3),t.push(n))}else if(r.isGeometry){var I,x,w,M,S=Array.isArray(i),T=r.vertices,E=r.faces,O=r.faceVertexUvs[0];O.length>0&&(M=O);for(var k=0,L=E.length;k<L;k++){var D=E[k],P=S?i[D.materialIndex]:i;if(void 0!==P&&(I=T[D.a],x=T[D.b],w=T[D.c],n=uo(this,P,e,Gi,I,x,w,ao))){if(M&&M[k]){var N=M[k];ro.copy(N[0]),io.copy(N[1]),oo.copy(N[2]),n.uv=hi.getUV(ao,I,x,w,ro,io,oo,new Un)}n.face=D,n.faceIndex=k,t.push(n)}}}},clone:function(){return new this.constructor(this.geometry,this.material).copy(this)}});var ho=0,fo=new sr,po=new wr,go=new $n;function vo(){Object.defineProperty(this,"id",{value:ho+=2}),this.uuid=Hn.generateUUID(),this.name="",this.type="Geometry",this.vertices=[],this.colors=[],this.faces=[],this.faceVertexUvs=[[]],this.morphTargets=[],this.morphNormals=[],this.skinWeights=[],this.skinIndices=[],this.lineDistances=[],this.boundingBox=null,this.boundingSphere=null,this.elementsNeedUpdate=!1,this.verticesNeedUpdate=!1,this.uvsNeedUpdate=!1,this.normalsNeedUpdate=!1,this.colorsNeedUpdate=!1,this.lineDistancesNeedUpdate=!1,this.groupsNeedUpdate=!1}vo.prototype=Object.assign(Object.create(Rn.prototype),{constructor:vo,isGeometry:!0,applyMatrix4:function(e){for(var t=(new Vn).getNormalMatrix(e),n=0,r=this.vertices.length;n<r;n++){this.vertices[n].applyMatrix4(e)}for(n=0,r=this.faces.length;n<r;n++){var i=this.faces[n];i.normal.applyMatrix3(t).normalize();for(var o=0,a=i.vertexNormals.length;o<a;o++)i.vertexNormals[o].applyMatrix3(t).normalize()}return null!==this.boundingBox&&this.computeBoundingBox(),null!==this.boundingSphere&&this.computeBoundingSphere(),this.verticesNeedUpdate=!0,this.normalsNeedUpdate=!0,this},rotateX:function(e){return fo.makeRotationX(e),this.applyMatrix4(fo),this},rotateY:function(e){return fo.makeRotationY(e),this.applyMatrix4(fo),this},rotateZ:function(e){return fo.makeRotationZ(e),this.applyMatrix4(fo),this},translate:function(e,t,n){return fo.makeTranslation(e,t,n),this.applyMatrix4(fo),this},scale:function(e,t,n){return fo.makeScale(e,t,n),this.applyMatrix4(fo),this},lookAt:function(e){return po.lookAt(e),po.updateMatrix(),this.applyMatrix4(po.matrix),this},fromBufferGeometry:function(e){var t=this,n=null!==e.index?e.index.array:void 0,r=e.attributes;if(void 0===r.position)return console.error("THREE.Geometry.fromBufferGeometry(): Position attribute required for conversion."),this;var i=r.position.array,o=void 0!==r.normal?r.normal.array:void 0,a=void 0!==r.color?r.color.array:void 0,s=void 0!==r.uv?r.uv.array:void 0,l=void 0!==r.uv2?r.uv2.array:void 0;void 0!==l&&(this.faceVertexUvs[1]=[]);for(var u=0;u<i.length;u+=3)t.vertices.push((new $n).fromArray(i,u)),void 0!==a&&t.colors.push((new vi).fromArray(a,u));function c(e,n,r,i){var u=void 0===a?[]:[t.colors[e].clone(),t.colors[n].clone(),t.colors[r].clone()],c=new Ci(e,n,r,void 0===o?[]:[(new $n).fromArray(o,3*e),(new $n).fromArray(o,3*n),(new $n).fromArray(o,3*r)],u,i);t.faces.push(c),void 0!==s&&t.faceVertexUvs[0].push([(new Un).fromArray(s,2*e),(new Un).fromArray(s,2*n),(new Un).fromArray(s,2*r)]),void 0!==l&&t.faceVertexUvs[1].push([(new Un).fromArray(l,2*e),(new Un).fromArray(l,2*n),(new Un).fromArray(l,2*r)])}var d=e.groups;if(d.length>0)for(u=0;u<d.length;u++)for(var h=d[u],f=h.start,p=f,g=f+h.count;p<g;p+=3)void 0!==n?c(n[p],n[p+1],n[p+2],h.materialIndex):c(p,p+1,p+2,h.materialIndex);else if(void 0!==n)for(u=0;u<n.length;u+=3)c(n[u],n[u+1],n[u+2]);else for(u=0;u<i.length/3;u+=3)c(u,u+1,u+2);return this.computeFaceNormals(),null!==e.boundingBox&&(this.boundingBox=e.boundingBox.clone()),null!==e.boundingSphere&&(this.boundingSphere=e.boundingSphere.clone()),this},center:function(){return this.computeBoundingBox(),this.boundingBox.getCenter(go).negate(),this.translate(go.x,go.y,go.z),this},normalize:function(){this.computeBoundingSphere();var e=this.boundingSphere.center,t=this.boundingSphere.radius,n=0===t?1:1/t,r=new sr;return r.set(n,0,0,-n*e.x,0,n,0,-n*e.y,0,0,n,-n*e.z,0,0,0,1),this.applyMatrix4(r),this},computeFaceNormals:function(){for(var e=new $n,t=new $n,n=0,r=this.faces.length;n<r;n++){var i=this.faces[n],o=this.vertices[i.a],a=this.vertices[i.b],s=this.vertices[i.c];e.subVectors(s,a),t.subVectors(o,a),e.cross(t),e.normalize(),i.normal.copy(e)}},computeVertexNormals:function(e){var t,n,r,i,o,a;for(void 0===e&&(e=!0),a=new Array(this.vertices.length),t=0,n=this.vertices.length;t<n;t++)a[t]=new $n;if(e){var s,l,u,c=new $n,d=new $n;for(r=0,i=this.faces.length;r<i;r++)o=this.faces[r],s=this.vertices[o.a],l=this.vertices[o.b],u=this.vertices[o.c],c.subVectors(u,l),d.subVectors(s,l),c.cross(d),a[o.a].add(c),a[o.b].add(c),a[o.c].add(c)}else for(this.computeFaceNormals(),r=0,i=this.faces.length;r<i;r++)a[(o=this.faces[r]).a].add(o.normal),a[o.b].add(o.normal),a[o.c].add(o.normal);for(t=0,n=this.vertices.length;t<n;t++)a[t].normalize();for(r=0,i=this.faces.length;r<i;r++){var h=(o=this.faces[r]).vertexNormals;3===h.length?(h[0].copy(a[o.a]),h[1].copy(a[o.b]),h[2].copy(a[o.c])):(h[0]=a[o.a].clone(),h[1]=a[o.b].clone(),h[2]=a[o.c].clone())}this.faces.length>0&&(this.normalsNeedUpdate=!0)},computeFlatVertexNormals:function(){var e,t,n;for(this.computeFaceNormals(),e=0,t=this.faces.length;e<t;e++){var r=(n=this.faces[e]).vertexNormals;3===r.length?(r[0].copy(n.normal),r[1].copy(n.normal),r[2].copy(n.normal)):(r[0]=n.normal.clone(),r[1]=n.normal.clone(),r[2]=n.normal.clone())}this.faces.length>0&&(this.normalsNeedUpdate=!0)},computeMorphNormals:function(){var e,t,n,r,i;for(n=0,r=this.faces.length;n<r;n++)for((i=this.faces[n]).__originalFaceNormal?i.__originalFaceNormal.copy(i.normal):i.__originalFaceNormal=i.normal.clone(),i.__originalVertexNormals||(i.__originalVertexNormals=[]),e=0,t=i.vertexNormals.length;e<t;e++)i.__originalVertexNormals[e]?i.__originalVertexNormals[e].copy(i.vertexNormals[e]):i.__originalVertexNormals[e]=i.vertexNormals[e].clone();var o=new vo;for(o.faces=this.faces,e=0,t=this.morphTargets.length;e<t;e++){if(!this.morphNormals[e]){this.morphNormals[e]={},this.morphNormals[e].faceNormals=[],this.morphNormals[e].vertexNormals=[];var a=this.morphNormals[e].faceNormals,s=this.morphNormals[e].vertexNormals;for(n=0,r=this.faces.length;n<r;n++)l=new $n,u={a:new $n,b:new $n,c:new $n},a.push(l),s.push(u)}var l,u,c=this.morphNormals[e];for(o.vertices=this.morphTargets[e].vertices,o.computeFaceNormals(),o.computeVertexNormals(),n=0,r=this.faces.length;n<r;n++)i=this.faces[n],l=c.faceNormals[n],u=c.vertexNormals[n],l.copy(i.normal),u.a.copy(i.vertexNormals[0]),u.b.copy(i.vertexNormals[1]),u.c.copy(i.vertexNormals[2])}for(n=0,r=this.faces.length;n<r;n++)(i=this.faces[n]).normal=i.__originalFaceNormal,i.vertexNormals=i.__originalVertexNormals},computeBoundingBox:function(){null===this.boundingBox&&(this.boundingBox=new Br),this.boundingBox.setFromPoints(this.vertices)},computeBoundingSphere:function(){null===this.boundingSphere&&(this.boundingSphere=new Vr),this.boundingSphere.setFromPoints(this.vertices)},merge:function(e,t,n){if(e&&e.isGeometry){var r,i=this.vertices.length,o=this.vertices,a=e.vertices,s=this.faces,l=e.faces,u=this.colors,c=e.colors;void 0===n&&(n=0),void 0!==t&&(r=(new Vn).getNormalMatrix(t));for(var d=0,h=a.length;d<h;d++){var f=a[d].clone();void 0!==t&&f.applyMatrix4(t),o.push(f)}for(d=0,h=c.length;d<h;d++)u.push(c[d].clone());for(d=0,h=l.length;d<h;d++){var p,g,v,m=l[d],y=m.vertexNormals,b=m.vertexColors;(p=new Ci(m.a+i,m.b+i,m.c+i)).normal.copy(m.normal),void 0!==r&&p.normal.applyMatrix3(r).normalize();for(var C=0,A=y.length;C<A;C++)g=y[C].clone(),void 0!==r&&g.applyMatrix3(r).normalize(),p.vertexNormals.push(g);p.color.copy(m.color);for(C=0,A=b.length;C<A;C++)v=b[C],p.vertexColors.push(v.clone());p.materialIndex=m.materialIndex+n,s.push(p)}for(d=0,h=e.faceVertexUvs.length;d<h;d++){var _=e.faceVertexUvs[d];void 0===this.faceVertexUvs[d]&&(this.faceVertexUvs[d]=[]);for(C=0,A=_.length;C<A;C++){for(var I=_[C],x=[],w=0,M=I.length;w<M;w++)x.push(I[w].clone());this.faceVertexUvs[d].push(x)}}}else console.error("THREE.Geometry.merge(): geometry not an instance of THREE.Geometry.",e)},mergeMesh:function(e){e&&e.isMesh?(e.matrixAutoUpdate&&e.updateMatrix(),this.merge(e.geometry,e.matrix)):console.error("THREE.Geometry.mergeMesh(): mesh not an instance of THREE.Mesh.",e)},mergeVertices:function(){var e,t,n,r,i,o,a,s,l={},u=[],c=[],d=Math.pow(10,4);for(n=0,r=this.vertices.length;n<r;n++)e=this.vertices[n],void 0===l[t=Math.round(e.x*d)+"_"+Math.round(e.y*d)+"_"+Math.round(e.z*d)]?(l[t]=n,u.push(this.vertices[n]),c[n]=u.length-1):c[n]=c[l[t]];var h=[];for(n=0,r=this.faces.length;n<r;n++){(i=this.faces[n]).a=c[i.a],i.b=c[i.b],i.c=c[i.c],o=[i.a,i.b,i.c];for(var f=0;f<3;f++)if(o[f]===o[(f+1)%3]){h.push(n);break}}for(n=h.length-1;n>=0;n--){var p=h[n];for(this.faces.splice(p,1),a=0,s=this.faceVertexUvs.length;a<s;a++)this.faceVertexUvs[a].splice(p,1)}var g=this.vertices.length-u.length;return this.vertices=u,g},setFromPoints:function(e){this.vertices=[];for(var t=0,n=e.length;t<n;t++){var r=e[t];this.vertices.push(new $n(r.x,r.y,r.z||0))}return this},sortFacesByMaterialIndex:function(){for(var e=this.faces,t=e.length,n=0;n<t;n++)e[n]._id=n;e.sort((function(e,t){return e.materialIndex-t.materialIndex}));var r,i,o=this.faceVertexUvs[0],a=this.faceVertexUvs[1];o&&o.length===t&&(r=[]),a&&a.length===t&&(i=[]);for(n=0;n<t;n++){var s=e[n]._id;r&&r.push(o[s]),i&&i.push(a[s])}r&&(this.faceVertexUvs[0]=r),i&&(this.faceVertexUvs[1]=i)},toJSON:function(){var e={metadata:{version:4.5,type:"Geometry",generator:"Geometry.toJSON"}};if(e.uuid=this.uuid,e.type=this.type,""!==this.name&&(e.name=this.name),void 0!==this.parameters){var t=this.parameters;for(var n in t)void 0!==t[n]&&(e[n]=t[n]);return e}for(var r=[],i=0;i<this.vertices.length;i++){var o=this.vertices[i];r.push(o.x,o.y,o.z)}var a=[],s=[],l={},u=[],c={},d=[],h={};for(i=0;i<this.faces.length;i++){var f=this.faces[i],p=void 0!==this.faceVertexUvs[0][i],g=f.normal.length()>0,v=f.vertexNormals.length>0,m=1!==f.color.r||1!==f.color.g||1!==f.color.b,y=f.vertexColors.length>0,b=0;if(b=I(b,0,0),b=I(b,1,!0),b=I(b,2,!1),b=I(b,3,p),b=I(b,4,g),b=I(b,5,v),b=I(b,6,m),b=I(b,7,y),a.push(b),a.push(f.a,f.b,f.c),a.push(f.materialIndex),p){var C=this.faceVertexUvs[0][i];a.push(M(C[0]),M(C[1]),M(C[2]))}if(g&&a.push(x(f.normal)),v){var A=f.vertexNormals;a.push(x(A[0]),x(A[1]),x(A[2]))}if(m&&a.push(w(f.color)),y){var _=f.vertexColors;a.push(w(_[0]),w(_[1]),w(_[2]))}}function I(e,t,n){return n?e|1<<t:e&~(1<<t)}function x(e){var t=e.x.toString()+e.y.toString()+e.z.toString();return void 0!==l[t]||(l[t]=s.length/3,s.push(e.x,e.y,e.z)),l[t]}function w(e){var t=e.r.toString()+e.g.toString()+e.b.toString();return void 0!==c[t]||(c[t]=u.length,u.push(e.getHex())),c[t]}function M(e){var t=e.x.toString()+e.y.toString();return void 0!==h[t]||(h[t]=d.length/2,d.push(e.x,e.y)),h[t]}return e.data={},e.data.vertices=r,e.data.normals=s,u.length>0&&(e.data.colors=u),d.length>0&&(e.data.uvs=[d]),e.data.faces=a,e},clone:function(){return(new vo).copy(this)},copy:function(e){var t,n,r,i,o,a;this.vertices=[],this.colors=[],this.faces=[],this.faceVertexUvs=[[]],this.morphTargets=[],this.morphNormals=[],this.skinWeights=[],this.skinIndices=[],this.lineDistances=[],this.boundingBox=null,this.boundingSphere=null,this.name=e.name;var s=e.vertices;for(t=0,n=s.length;t<n;t++)this.vertices.push(s[t].clone());var l=e.colors;for(t=0,n=l.length;t<n;t++)this.colors.push(l[t].clone());var u=e.faces;for(t=0,n=u.length;t<n;t++)this.faces.push(u[t].clone());for(t=0,n=e.faceVertexUvs.length;t<n;t++){var c=e.faceVertexUvs[t];for(void 0===this.faceVertexUvs[t]&&(this.faceVertexUvs[t]=[]),r=0,i=c.length;r<i;r++){var d=c[r],h=[];for(o=0,a=d.length;o<a;o++){var f=d[o];h.push(f.clone())}this.faceVertexUvs[t].push(h)}}var p=e.morphTargets;for(t=0,n=p.length;t<n;t++){var g={};if(g.name=p[t].name,void 0!==p[t].vertices)for(g.vertices=[],r=0,i=p[t].vertices.length;r<i;r++)g.vertices.push(p[t].vertices[r].clone());if(void 0!==p[t].normals)for(g.normals=[],r=0,i=p[t].normals.length;r<i;r++)g.normals.push(p[t].normals[r].clone());this.morphTargets.push(g)}var v=e.morphNormals;for(t=0,n=v.length;t<n;t++){var m={};if(void 0!==v[t].vertexNormals)for(m.vertexNormals=[],r=0,i=v[t].vertexNormals.length;r<i;r++){var y=v[t].vertexNormals[r],b={};b.a=y.a.clone(),b.b=y.b.clone(),b.c=y.c.clone(),m.vertexNormals.push(b)}if(void 0!==v[t].faceNormals)for(m.faceNormals=[],r=0,i=v[t].faceNormals.length;r<i;r++)m.faceNormals.push(v[t].faceNormals[r].clone());this.morphNormals.push(m)}var C=e.skinWeights;for(t=0,n=C.length;t<n;t++)this.skinWeights.push(C[t].clone());var A=e.skinIndices;for(t=0,n=A.length;t<n;t++)this.skinIndices.push(A[t].clone());var _=e.lineDistances;for(t=0,n=_.length;t<n;t++)this.lineDistances.push(_[t]);var I=e.boundingBox;null!==I&&(this.boundingBox=I.clone());var x=e.boundingSphere;return null!==x&&(this.boundingSphere=x.clone()),this.elementsNeedUpdate=e.elementsNeedUpdate,this.verticesNeedUpdate=e.verticesNeedUpdate,this.uvsNeedUpdate=e.uvsNeedUpdate,this.normalsNeedUpdate=e.normalsNeedUpdate,this.colorsNeedUpdate=e.colorsNeedUpdate,this.lineDistancesNeedUpdate=e.lineDistancesNeedUpdate,this.groupsNeedUpdate=e.groupsNeedUpdate,this},dispose:function(){this.dispatchEvent({type:"dispose"})}});class mo extends vo{constructor(e,t,n,r,i,o){super(),this.type="BoxGeometry",this.parameters={width:e,height:t,depth:n,widthSegments:r,heightSegments:i,depthSegments:o},this.fromBufferGeometry(new yo(e,t,n,r,i,o)),this.mergeVertices()}}class yo extends Yi{constructor(e,t,n,r,i,o){super(),this.type="BoxBufferGeometry",this.parameters={width:e,height:t,depth:n,widthSegments:r,heightSegments:i,depthSegments:o};const a=this;e=e||1,t=t||1,n=n||1,r=Math.floor(r)||1,i=Math.floor(i)||1,o=Math.floor(o)||1;const s=[],l=[],u=[],c=[];let d=0,h=0;function f(e,t,n,r,i,o,f,p,g,v,m){const y=o/g,b=f/v,C=o/2,A=f/2,_=p/2,I=g+1,x=v+1;let w=0,M=0;const S=new $n;for(let o=0;o<x;o++){const a=o*b-A;for(let s=0;s<I;s++){const d=s*y-C;S[e]=d*r,S[t]=a*i,S[n]=_,l.push(S.x,S.y,S.z),S[e]=0,S[t]=0,S[n]=p>0?1:-1,u.push(S.x,S.y,S.z),c.push(s/g),c.push(1-o/v),w+=1}}for(let e=0;e<v;e++)for(let t=0;t<g;t++){const n=d+t+I*e,r=d+t+I*(e+1),i=d+(t+1)+I*(e+1),o=d+(t+1)+I*e;s.push(n,r,o),s.push(r,i,o),M+=6}a.addGroup(h,M,m),h+=M,d+=w}f("z","y","x",-1,-1,n,t,e,o,i,0),f("z","y","x",1,-1,n,t,-e,o,i,1),f("x","z","y",1,1,e,n,t,r,o,2),f("x","z","y",1,-1,e,n,-t,r,o,3),f("x","y","z",1,-1,e,t,n,r,i,4),f("x","y","z",-1,-1,e,t,-n,r,i,5),this.setIndex(s),this.setAttribute("position",new Di(l,3)),this.setAttribute("normal",new Di(u,3)),this.setAttribute("uv",new Di(c,2))}}function bo(e){var t={};for(var n in e)for(var r in t[n]={},e[n]){var i=e[n][r];i&&(i.isColor||i.isMatrix3||i.isMatrix4||i.isVector2||i.isVector3||i.isVector4||i.isTexture)?t[n][r]=i.clone():Array.isArray(i)?t[n][r]=i.slice():t[n][r]=i}return t}function Co(e){for(var t={},n=0;n<e.length;n++){var r=bo(e[n]);for(var i in r)t[i]=r[i]}return t}var Ao={clone:bo,merge:Co};function _o(e){_i.call(this),this.type="ShaderMaterial",this.defines={},this.uniforms={},this.vertexShader="void main() {\n\tgl_Position = projectionMatrix * modelViewMatrix * vec4( position, 1.0 );\n}",this.fragmentShader="void main() {\n\tgl_FragColor = vec4( 1.0, 0.0, 0.0, 1.0 );\n}",this.linewidth=1,this.wireframe=!1,this.wireframeLinewidth=1,this.fog=!1,this.lights=!1,this.clipping=!1,this.skinning=!1,this.morphTargets=!1,this.morphNormals=!1,this.extensions={derivatives:!1,fragDepth:!1,drawBuffers:!1,shaderTextureLOD:!1},this.defaultAttributeValues={color:[1,1,1],uv:[0,0],uv2:[0,0]},this.index0AttributeName=void 0,this.uniformsNeedUpdate=!1,void 0!==e&&(void 0!==e.attributes&&console.error("THREE.ShaderMaterial: attributes should now be defined in THREE.BufferGeometry instead."),this.setValues(e))}function Io(){wr.call(this),this.type="Camera",this.matrixWorldInverse=new sr,this.projectionMatrix=new sr,this.projectionMatrixInverse=new sr}function xo(e,t,n,r){Io.call(this),this.type="PerspectiveCamera",this.fov=void 0!==e?e:50,this.zoom=1,this.near=void 0!==n?n:.1,this.far=void 0!==r?r:2e3,this.focus=10,this.aspect=void 0!==t?t:1,this.view=null,this.filmGauge=35,this.filmOffset=0,this.updateProjectionMatrix()}_o.prototype=Object.create(_i.prototype),_o.prototype.constructor=_o,_o.prototype.isShaderMaterial=!0,_o.prototype.copy=function(e){return _i.prototype.copy.call(this,e),this.fragmentShader=e.fragmentShader,this.vertexShader=e.vertexShader,this.uniforms=bo(e.uniforms),this.defines=Object.assign({},e.defines),this.wireframe=e.wireframe,this.wireframeLinewidth=e.wireframeLinewidth,this.lights=e.lights,this.clipping=e.clipping,this.skinning=e.skinning,this.morphTargets=e.morphTargets,this.morphNormals=e.morphNormals,this.extensions=Object.assign({},e.extensions),this},_o.prototype.toJSON=function(e){var t=_i.prototype.toJSON.call(this,e);for(var n in t.uniforms={},this.uniforms){var r=this.uniforms[n].value;r&&r.isTexture?t.uniforms[n]={type:"t",value:r.toJSON(e).uuid}:r&&r.isColor?t.uniforms[n]={type:"c",value:r.getHex()}:r&&r.isVector2?t.uniforms[n]={type:"v2",value:r.toArray()}:r&&r.isVector3?t.uniforms[n]={type:"v3",value:r.toArray()}:r&&r.isVector4?t.uniforms[n]={type:"v4",value:r.toArray()}:r&&r.isMatrix3?t.uniforms[n]={type:"m3",value:r.toArray()}:r&&r.isMatrix4?t.uniforms[n]={type:"m4",value:r.toArray()}:t.uniforms[n]={value:r}}Object.keys(this.defines).length>0&&(t.defines=this.defines),t.vertexShader=this.vertexShader,t.fragmentShader=this.fragmentShader;var i={};for(var o in this.extensions)!0===this.extensions[o]&&(i[o]=!0);return Object.keys(i).length>0&&(t.extensions=i),t},Io.prototype=Object.assign(Object.create(wr.prototype),{constructor:Io,isCamera:!0,copy:function(e,t){return wr.prototype.copy.call(this,e,t),this.matrixWorldInverse.copy(e.matrixWorldInverse),this.projectionMatrix.copy(e.projectionMatrix),this.projectionMatrixInverse.copy(e.projectionMatrixInverse),this},getWorldDirection:function(e){void 0===e&&(console.warn("THREE.Camera: .getWorldDirection() target is now required"),e=new $n),this.updateMatrixWorld(!0);var t=this.matrixWorld.elements;return e.set(-t[8],-t[9],-t[10]).normalize()},updateMatrixWorld:function(e){wr.prototype.updateMatrixWorld.call(this,e),this.matrixWorldInverse.getInverse(this.matrixWorld)},updateWorldMatrix:function(e,t){wr.prototype.updateWorldMatrix.call(this,e,t),this.matrixWorldInverse.getInverse(this.matrixWorld)},clone:function(){return(new this.constructor).copy(this)}}),xo.prototype=Object.assign(Object.create(Io.prototype),{constructor:xo,isPerspectiveCamera:!0,copy:function(e,t){return Io.prototype.copy.call(this,e,t),this.fov=e.fov,this.zoom=e.zoom,this.near=e.near,this.far=e.far,this.focus=e.focus,this.aspect=e.aspect,this.view=null===e.view?null:Object.assign({},e.view),this.filmGauge=e.filmGauge,this.filmOffset=e.filmOffset,this},setFocalLength:function(e){var t=.5*this.getFilmHeight()/e;this.fov=2*Hn.RAD2DEG*Math.atan(t),this.updateProjectionMatrix()},getFocalLength:function(){var e=Math.tan(.5*Hn.DEG2RAD*this.fov);return.5*this.getFilmHeight()/e},getEffectiveFOV:function(){return 2*Hn.RAD2DEG*Math.atan(Math.tan(.5*Hn.DEG2RAD*this.fov)/this.zoom)},getFilmWidth:function(){return this.filmGauge*Math.min(this.aspect,1)},getFilmHeight:function(){return this.filmGauge/Math.max(this.aspect,1)},setViewOffset:function(e,t,n,r,i,o){this.aspect=e/t,null===this.view&&(this.view={enabled:!0,fullWidth:1,fullHeight:1,offsetX:0,offsetY:0,width:1,height:1}),this.view.enabled=!0,this.view.fullWidth=e,this.view.fullHeight=t,this.view.offsetX=n,this.view.offsetY=r,this.view.width=i,this.view.height=o,this.updateProjectionMatrix()},clearViewOffset:function(){null!==this.view&&(this.view.enabled=!1),this.updateProjectionMatrix()},updateProjectionMatrix:function(){var e=this.near,t=e*Math.tan(.5*Hn.DEG2RAD*this.fov)/this.zoom,n=2*t,r=this.aspect*n,i=-.5*r,o=this.view;if(null!==this.view&&this.view.enabled){var a=o.fullWidth,s=o.fullHeight;i+=o.offsetX*r/a,t-=o.offsetY*n/s,r*=o.width/a,n*=o.height/s}var l=this.filmOffset;0!==l&&(i+=e*l/this.getFilmWidth()),this.projectionMatrix.makePerspective(i,i+r,t,t-n,e,this.far),this.projectionMatrixInverse.getInverse(this.projectionMatrix)},toJSON:function(e){var t=wr.prototype.toJSON.call(this,e);return t.object.fov=this.fov,t.object.zoom=this.zoom,t.object.near=this.near,t.object.far=this.far,t.object.focus=this.focus,t.object.aspect=this.aspect,null!==this.view&&(t.object.view=Object.assign({},this.view)),t.object.filmGauge=this.filmGauge,t.object.filmOffset=this.filmOffset,t}});function wo(e,t,n){if(wr.call(this),this.type="CubeCamera",!0===n.isWebGLCubeRenderTarget){this.renderTarget=n;var r=new xo(90,1,e,t);r.layers=this.layers,r.up.set(0,-1,0),r.lookAt(new $n(1,0,0)),this.add(r);var i=new xo(90,1,e,t);i.layers=this.layers,i.up.set(0,-1,0),i.lookAt(new $n(-1,0,0)),this.add(i);var o=new xo(90,1,e,t);o.layers=this.layers,o.up.set(0,0,1),o.lookAt(new $n(0,1,0)),this.add(o);var a=new xo(90,1,e,t);a.layers=this.layers,a.up.set(0,0,-1),a.lookAt(new $n(0,-1,0)),this.add(a);var s=new xo(90,1,e,t);s.layers=this.layers,s.up.set(0,-1,0),s.lookAt(new $n(0,0,1)),this.add(s);var l=new xo(90,1,e,t);l.layers=this.layers,l.up.set(0,-1,0),l.lookAt(new $n(0,0,-1)),this.add(l),this.update=function(e,t){null===this.parent&&this.updateMatrixWorld();var u=e.xr.enabled,c=e.getRenderTarget();e.xr.enabled=!1;var d=n.texture.generateMipmaps;n.texture.generateMipmaps=!1,e.setRenderTarget(n,0),e.render(t,r),e.setRenderTarget(n,1),e.render(t,i),e.setRenderTarget(n,2),e.render(t,o),e.setRenderTarget(n,3),e.render(t,a),e.setRenderTarget(n,4),e.render(t,s),n.texture.generateMipmaps=d,e.setRenderTarget(n,5),e.render(t,l),e.setRenderTarget(c),e.xr.enabled=u},this.clear=function(e,t,r,i){for(var o=e.getRenderTarget(),a=0;a<6;a++)e.setRenderTarget(n,a),e.clear(t,r,i);e.setRenderTarget(o)}}else console.error("THREE.CubeCamera: The constructor now expects an instance of WebGLCubeRenderTarget as third parameter.")}function Mo(e,t,n){Number.isInteger(t)&&(console.warn("THREE.WebGLCubeRenderTarget: constructor signature is now WebGLCubeRenderTarget( size, options )"),t=n),qn.call(this,e,e,t)}function So(e,t,n,r,i,o,a,s,l,u,c,d){Gn.call(this,null,o,a,s,l,u,r,i,c,d),this.image={data:e||null,width:t||1,height:n||1},this.magFilter=void 0!==l?l:me,this.minFilter=void 0!==u?u:me,this.generateMipmaps=!1,this.flipY=!1,this.unpackAlignment=1,this.needsUpdate=!0}wo.prototype=Object.create(wr.prototype),wo.prototype.constructor=wo,Mo.prototype=Object.create(qn.prototype),Mo.prototype.constructor=Mo,Mo.prototype.isWebGLCubeRenderTarget=!0,Mo.prototype.fromEquirectangularTexture=function(e,t){this.texture.type=t.type,this.texture.format=t.format,this.texture.encoding=t.encoding;var n=new Mr,r={uniforms:{tEquirect:{value:null}},vertexShader:["varying vec3 vWorldDirection;","vec3 transformDirection( in vec3 dir, in mat4 matrix ) {","\treturn normalize( ( matrix * vec4( dir, 0.0 ) ).xyz );","}","void main() {","\tvWorldDirection = transformDirection( position, modelMatrix );","\t#include <begin_vertex>","\t#include <project_vertex>","}"].join("\n"),fragmentShader:["uniform sampler2D tEquirect;","varying vec3 vWorldDirection;","#include <common>","void main() {","\tvec3 direction = normalize( vWorldDirection );","\tvec2 sampleUV = equirectUv( direction );","\tgl_FragColor = texture2D( tEquirect, sampleUV );","}"].join("\n")},i=new _o({type:"CubemapFromEquirect",uniforms:bo(r.uniforms),vertexShader:r.vertexShader,fragmentShader:r.fragmentShader,side:m,blending:A});i.uniforms.tEquirect.value=t;var o=new lo(new yo(5,5,5),i);return n.add(o),new wo(1,10,this).update(e,n),o.geometry.dispose(),o.material.dispose(),this},So.prototype=Object.create(Gn.prototype),So.prototype.constructor=So,So.prototype.isDataTexture=!0;var To=new Vr,Eo=new $n;function Oo(e,t,n,r,i,o){this.planes=[void 0!==e?e:new ti,void 0!==t?t:new ti,void 0!==n?n:new ti,void 0!==r?r:new ti,void 0!==i?i:new ti,void 0!==o?o:new ti]}Object.assign(Oo.prototype,{set:function(e,t,n,r,i,o){var a=this.planes;return a[0].copy(e),a[1].copy(t),a[2].copy(n),a[3].copy(r),a[4].copy(i),a[5].copy(o),this},clone:function(){return(new this.constructor).copy(this)},copy:function(e){for(var t=this.planes,n=0;n<6;n++)t[n].copy(e.planes[n]);return this},setFromProjectionMatrix:function(e){var t=this.planes,n=e.elements,r=n[0],i=n[1],o=n[2],a=n[3],s=n[4],l=n[5],u=n[6],c=n[7],d=n[8],h=n[9],f=n[10],p=n[11],g=n[12],v=n[13],m=n[14],y=n[15];return t[0].setComponents(a-r,c-s,p-d,y-g).normalize(),t[1].setComponents(a+r,c+s,p+d,y+g).normalize(),t[2].setComponents(a+i,c+l,p+h,y+v).normalize(),t[3].setComponents(a-i,c-l,p-h,y-v).normalize(),t[4].setComponents(a-o,c-u,p-f,y-m).normalize(),t[5].setComponents(a+o,c+u,p+f,y+m).normalize(),this},intersectsObject:function(e){var t=e.geometry;return null===t.boundingSphere&&t.computeBoundingSphere(),To.copy(t.boundingSphere).applyMatrix4(e.matrixWorld),this.intersectsSphere(To)},intersectsSprite:function(e){return To.center.set(0,0,0),To.radius=.7071067811865476,To.applyMatrix4(e.matrixWorld),this.intersectsSphere(To)},intersectsSphere:function(e){for(var t=this.planes,n=e.center,r=-e.radius,i=0;i<6;i++){if(t[i].distanceToPoint(n)<r)return!1}return!0},intersectsBox:function(e){for(var t=this.planes,n=0;n<6;n++){var r=t[n];if(Eo.x=r.normal.x>0?e.max.x:e.min.x,Eo.y=r.normal.y>0?e.max.y:e.min.y,Eo.z=r.normal.z>0?e.max.z:e.min.z,r.distanceToPoint(Eo)<0)return!1}return!0},containsPoint:function(e){for(var t=this.planes,n=0;n<6;n++)if(t[n].distanceToPoint(e)<0)return!1;return!0}});var ko={common:{diffuse:{value:new vi(15658734)},opacity:{value:1},map:{value:null},uvTransform:{value:new Vn},uv2Transform:{value:new Vn},alphaMap:{value:null}},specularmap:{specularMap:{value:null}},envmap:{envMap:{value:null},flipEnvMap:{value:-1},reflectivity:{value:1},refractionRatio:{value:.98},maxMipLevel:{value:0}},aomap:{aoMap:{value:null},aoMapIntensity:{value:1}},lightmap:{lightMap:{value:null},lightMapIntensity:{value:1}},emissivemap:{emissiveMap:{value:null}},bumpmap:{bumpMap:{value:null},bumpScale:{value:1}},normalmap:{normalMap:{value:null},normalScale:{value:new Un(1,1)}},displacementmap:{displacementMap:{value:null},displacementScale:{value:1},displacementBias:{value:0}},roughnessmap:{roughnessMap:{value:null}},metalnessmap:{metalnessMap:{value:null}},gradientmap:{gradientMap:{value:null}},fog:{fogDensity:{value:25e-5},fogNear:{value:1},fogFar:{value:2e3},fogColor:{value:new vi(16777215)}},lights:{ambientLightColor:{value:[]},lightProbe:{value:[]},directionalLights:{value:[],properties:{direction:{},color:{}}},directionalLightShadows:{value:[],properties:{shadowBias:{},shadowRadius:{},shadowMapSize:{}}},directionalShadowMap:{value:[]},directionalShadowMatrix:{value:[]},spotLights:{value:[],properties:{color:{},position:{},direction:{},distance:{},coneCos:{},penumbraCos:{},decay:{}}},spotLightShadows:{value:[],properties:{shadowBias:{},shadowRadius:{},shadowMapSize:{}}},spotShadowMap:{value:[]},spotShadowMatrix:{value:[]},pointLights:{value:[],properties:{color:{},position:{},decay:{},distance:{}}},pointLightShadows:{value:[],properties:{shadowBias:{},shadowRadius:{},shadowMapSize:{},shadowCameraNear:{},shadowCameraFar:{}}},pointShadowMap:{value:[]},pointShadowMatrix:{value:[]},hemisphereLights:{value:[],properties:{direction:{},skyColor:{},groundColor:{}}},rectAreaLights:{value:[],properties:{color:{},position:{},width:{},height:{}}}},points:{diffuse:{value:new vi(15658734)},opacity:{value:1},size:{value:1},scale:{value:1},map:{value:null},alphaMap:{value:null},uvTransform:{value:new Vn}},sprite:{diffuse:{value:new vi(15658734)},opacity:{value:1},center:{value:new Un(.5,.5)},rotation:{value:0},map:{value:null},alphaMap:{value:null},uvTransform:{value:new Vn}}};function Lo(){var e=null,t=!1,n=null;function r(i,o){!1!==t&&(n(i,o),e.requestAnimationFrame(r))}return{start:function(){!0!==t&&null!==n&&(e.requestAnimationFrame(r),t=!0)},stop:function(){t=!1},setAnimationLoop:function(e){n=e},setContext:function(t){e=t}}}function Do(e,t){var n=t.isWebGL2,r=new WeakMap;return{get:function(e){return e.isInterleavedBufferAttribute&&(e=e.data),r.get(e)},remove:function(t){t.isInterleavedBufferAttribute&&(t=t.data);var n=r.get(t);n&&(e.deleteBuffer(n.buffer),r.delete(t))},update:function(t,i){t.isInterleavedBufferAttribute&&(t=t.data);var o=r.get(t);void 0===o?r.set(t,function(t,n){var r=t.array,i=t.usage,o=e.createBuffer();e.bindBuffer(n,o),e.bufferData(n,r,i),t.onUploadCallback();var a=5126;return r instanceof Float32Array?a=5126:r instanceof Float64Array?console.warn("THREE.WebGLAttributes: Unsupported data buffer format: Float64Array."):r instanceof Uint16Array?a=5123:r instanceof Int16Array?a=5122:r instanceof Uint32Array?a=5125:r instanceof Int32Array?a=5124:r instanceof Int8Array?a=5120:r instanceof Uint8Array&&(a=5121),{buffer:o,type:a,bytesPerElement:r.BYTES_PER_ELEMENT,version:t.version}}(t,i)):o.version<t.version&&(!function(t,r,i){var o=r.array,a=r.updateRange;e.bindBuffer(i,t),-1===a.count?e.bufferSubData(i,0,o):(n?e.bufferSubData(i,a.offset*o.BYTES_PER_ELEMENT,o,a.offset,a.count):e.bufferSubData(i,a.offset*o.BYTES_PER_ELEMENT,o.subarray(a.offset,a.offset+a.count)),a.count=-1)}(o.buffer,t,i),o.version=t.version)}}}function Po(e,t,n,r){vo.call(this),this.type="PlaneGeometry",this.parameters={width:e,height:t,widthSegments:n,heightSegments:r},this.fromBufferGeometry(new No(e,t,n,r)),this.mergeVertices()}function No(e,t,n,r){Yi.call(this),this.type="PlaneBufferGeometry",this.parameters={width:e,height:t,widthSegments:n,heightSegments:r};var i,o,a=(e=e||1)/2,s=(t=t||1)/2,l=Math.floor(n)||1,u=Math.floor(r)||1,c=l+1,d=u+1,h=e/l,f=t/u,p=[],g=[],v=[],m=[];for(o=0;o<d;o++){var y=o*f-s;for(i=0;i<c;i++){var b=i*h-a;g.push(b,-y,0),v.push(0,0,1),m.push(i/l),m.push(1-o/u)}}for(o=0;o<u;o++)for(i=0;i<l;i++){var C=i+c*o,A=i+c*(o+1),_=i+1+c*(o+1),I=i+1+c*o;p.push(C,A,I),p.push(A,_,I)}this.setIndex(p),this.setAttribute("position",new Di(g,3)),this.setAttribute("normal",new Di(v,3)),this.setAttribute("uv",new Di(m,2))}Po.prototype=Object.create(vo.prototype),Po.prototype.constructor=Po,No.prototype=Object.create(Yi.prototype),No.prototype.constructor=No;var jo={alphamap_fragment:"#ifdef USE_ALPHAMAP\n\tdiffuseColor.a *= texture2D( alphaMap, vUv ).g;\n#endif",alphamap_pars_fragment:"#ifdef USE_ALPHAMAP\n\tuniform sampler2D alphaMap;\n#endif",alphatest_fragment:"#ifdef ALPHATEST\n\tif ( diffuseColor.a < ALPHATEST ) discard;\n#endif",aomap_fragment:"#ifdef USE_AOMAP\n\tfloat ambientOcclusion = ( texture2D( aoMap, vUv2 ).r - 1.0 ) * aoMapIntensity + 1.0;\n\treflectedLight.indirectDiffuse *= ambientOcclusion;\n\t#if defined( USE_ENVMAP ) && defined( STANDARD )\n\t\tfloat dotNV = saturate( dot( geometry.normal, geometry.viewDir ) );\n\t\treflectedLight.indirectSpecular *= computeSpecularOcclusion( dotNV, ambientOcclusion, material.specularRoughness );\n\t#endif\n#endif",aomap_pars_fragment:"#ifdef USE_AOMAP\n\tuniform sampler2D aoMap;\n\tuniform float aoMapIntensity;\n#endif",begin_vertex:"vec3 transformed = vec3( position );",beginnormal_vertex:"vec3 objectNormal = vec3( normal );\n#ifdef USE_TANGENT\n\tvec3 objectTangent = vec3( tangent.xyz );\n#endif",bsdfs:"vec2 integrateSpecularBRDF( const in float dotNV, const in float roughness ) {\n\tconst vec4 c0 = vec4( - 1, - 0.0275, - 0.572, 0.022 );\n\tconst vec4 c1 = vec4( 1, 0.0425, 1.04, - 0.04 );\n\tvec4 r = roughness * c0 + c1;\n\tfloat a004 = min( r.x * r.x, exp2( - 9.28 * dotNV ) ) * r.x + r.y;\n\treturn vec2( -1.04, 1.04 ) * a004 + r.zw;\n}\nfloat punctualLightIntensityToIrradianceFactor( const in float lightDistance, const in float cutoffDistance, const in float decayExponent ) {\n#if defined ( PHYSICALLY_CORRECT_LIGHTS )\n\tfloat distanceFalloff = 1.0 / max( pow( lightDistance, decayExponent ), 0.01 );\n\tif( cutoffDistance > 0.0 ) {\n\t\tdistanceFalloff *= pow2( saturate( 1.0 - pow4( lightDistance / cutoffDistance ) ) );\n\t}\n\treturn distanceFalloff;\n#else\n\tif( cutoffDistance > 0.0 && decayExponent > 0.0 ) {\n\t\treturn pow( saturate( -lightDistance / cutoffDistance + 1.0 ), decayExponent );\n\t}\n\treturn 1.0;\n#endif\n}\nvec3 BRDF_Diffuse_Lambert( const in vec3 diffuseColor ) {\n\treturn RECIPROCAL_PI * diffuseColor;\n}\nvec3 F_Schlick( const in vec3 specularColor, const in float dotLH ) {\n\tfloat fresnel = exp2( ( -5.55473 * dotLH - 6.98316 ) * dotLH );\n\treturn ( 1.0 - specularColor ) * fresnel + specularColor;\n}\nvec3 F_Schlick_RoughnessDependent( const in vec3 F0, const in float dotNV, const in float roughness ) {\n\tfloat fresnel = exp2( ( -5.55473 * dotNV - 6.98316 ) * dotNV );\n\tvec3 Fr = max( vec3( 1.0 - roughness ), F0 ) - F0;\n\treturn Fr * fresnel + F0;\n}\nfloat G_GGX_Smith( const in float alpha, const in float dotNL, const in float dotNV ) {\n\tfloat a2 = pow2( alpha );\n\tfloat gl = dotNL + sqrt( a2 + ( 1.0 - a2 ) * pow2( dotNL ) );\n\tfloat gv = dotNV + sqrt( a2 + ( 1.0 - a2 ) * pow2( dotNV ) );\n\treturn 1.0 / ( gl * gv );\n}\nfloat G_GGX_SmithCorrelated( const in float alpha, const in float dotNL, const in float dotNV ) {\n\tfloat a2 = pow2( alpha );\n\tfloat gv = dotNL * sqrt( a2 + ( 1.0 - a2 ) * pow2( dotNV ) );\n\tfloat gl = dotNV * sqrt( a2 + ( 1.0 - a2 ) * pow2( dotNL ) );\n\treturn 0.5 / max( gv + gl, EPSILON );\n}\nfloat D_GGX( const in float alpha, const in float dotNH ) {\n\tfloat a2 = pow2( alpha );\n\tfloat denom = pow2( dotNH ) * ( a2 - 1.0 ) + 1.0;\n\treturn RECIPROCAL_PI * a2 / pow2( denom );\n}\nvec3 BRDF_Specular_GGX( const in IncidentLight incidentLight, const in vec3 viewDir, const in vec3 normal, const in vec3 specularColor, const in float roughness ) {\n\tfloat alpha = pow2( roughness );\n\tvec3 halfDir = normalize( incidentLight.direction + viewDir );\n\tfloat dotNL = saturate( dot( normal, incidentLight.direction ) );\n\tfloat dotNV = saturate( dot( normal, viewDir ) );\n\tfloat dotNH = saturate( dot( normal, halfDir ) );\n\tfloat dotLH = saturate( dot( incidentLight.direction, halfDir ) );\n\tvec3 F = F_Schlick( specularColor, dotLH );\n\tfloat G = G_GGX_SmithCorrelated( alpha, dotNL, dotNV );\n\tfloat D = D_GGX( alpha, dotNH );\n\treturn F * ( G * D );\n}\nvec2 LTC_Uv( const in vec3 N, const in vec3 V, const in float roughness ) {\n\tconst float LUT_SIZE  = 64.0;\n\tconst float LUT_SCALE = ( LUT_SIZE - 1.0 ) / LUT_SIZE;\n\tconst float LUT_BIAS  = 0.5 / LUT_SIZE;\n\tfloat dotNV = saturate( dot( N, V ) );\n\tvec2 uv = vec2( roughness, sqrt( 1.0 - dotNV ) );\n\tuv = uv * LUT_SCALE + LUT_BIAS;\n\treturn uv;\n}\nfloat LTC_ClippedSphereFormFactor( const in vec3 f ) {\n\tfloat l = length( f );\n\treturn max( ( l * l + f.z ) / ( l + 1.0 ), 0.0 );\n}\nvec3 LTC_EdgeVectorFormFactor( const in vec3 v1, const in vec3 v2 ) {\n\tfloat x = dot( v1, v2 );\n\tfloat y = abs( x );\n\tfloat a = 0.8543985 + ( 0.4965155 + 0.0145206 * y ) * y;\n\tfloat b = 3.4175940 + ( 4.1616724 + y ) * y;\n\tfloat v = a / b;\n\tfloat theta_sintheta = ( x > 0.0 ) ? v : 0.5 * inversesqrt( max( 1.0 - x * x, 1e-7 ) ) - v;\n\treturn cross( v1, v2 ) * theta_sintheta;\n}\nvec3 LTC_Evaluate( const in vec3 N, const in vec3 V, const in vec3 P, const in mat3 mInv, const in vec3 rectCoords[ 4 ] ) {\n\tvec3 v1 = rectCoords[ 1 ] - rectCoords[ 0 ];\n\tvec3 v2 = rectCoords[ 3 ] - rectCoords[ 0 ];\n\tvec3 lightNormal = cross( v1, v2 );\n\tif( dot( lightNormal, P - rectCoords[ 0 ] ) < 0.0 ) return vec3( 0.0 );\n\tvec3 T1, T2;\n\tT1 = normalize( V - N * dot( V, N ) );\n\tT2 = - cross( N, T1 );\n\tmat3 mat = mInv * transposeMat3( mat3( T1, T2, N ) );\n\tvec3 coords[ 4 ];\n\tcoords[ 0 ] = mat * ( rectCoords[ 0 ] - P );\n\tcoords[ 1 ] = mat * ( rectCoords[ 1 ] - P );\n\tcoords[ 2 ] = mat * ( rectCoords[ 2 ] - P );\n\tcoords[ 3 ] = mat * ( rectCoords[ 3 ] - P );\n\tcoords[ 0 ] = normalize( coords[ 0 ] );\n\tcoords[ 1 ] = normalize( coords[ 1 ] );\n\tcoords[ 2 ] = normalize( coords[ 2 ] );\n\tcoords[ 3 ] = normalize( coords[ 3 ] );\n\tvec3 vectorFormFactor = vec3( 0.0 );\n\tvectorFormFactor += LTC_EdgeVectorFormFactor( coords[ 0 ], coords[ 1 ] );\n\tvectorFormFactor += LTC_EdgeVectorFormFactor( coords[ 1 ], coords[ 2 ] );\n\tvectorFormFactor += LTC_EdgeVectorFormFactor( coords[ 2 ], coords[ 3 ] );\n\tvectorFormFactor += LTC_EdgeVectorFormFactor( coords[ 3 ], coords[ 0 ] );\n\tfloat result = LTC_ClippedSphereFormFactor( vectorFormFactor );\n\treturn vec3( result );\n}\nvec3 BRDF_Specular_GGX_Environment( const in vec3 viewDir, const in vec3 normal, const in vec3 specularColor, const in float roughness ) {\n\tfloat dotNV = saturate( dot( normal, viewDir ) );\n\tvec2 brdf = integrateSpecularBRDF( dotNV, roughness );\n\treturn specularColor * brdf.x + brdf.y;\n}\nvoid BRDF_Specular_Multiscattering_Environment( const in GeometricContext geometry, const in vec3 specularColor, const in float roughness, inout vec3 singleScatter, inout vec3 multiScatter ) {\n\tfloat dotNV = saturate( dot( geometry.normal, geometry.viewDir ) );\n\tvec3 F = F_Schlick_RoughnessDependent( specularColor, dotNV, roughness );\n\tvec2 brdf = integrateSpecularBRDF( dotNV, roughness );\n\tvec3 FssEss = F * brdf.x + brdf.y;\n\tfloat Ess = brdf.x + brdf.y;\n\tfloat Ems = 1.0 - Ess;\n\tvec3 Favg = specularColor + ( 1.0 - specularColor ) * 0.047619;\tvec3 Fms = FssEss * Favg / ( 1.0 - Ems * Favg );\n\tsingleScatter += FssEss;\n\tmultiScatter += Fms * Ems;\n}\nfloat G_BlinnPhong_Implicit( ) {\n\treturn 0.25;\n}\nfloat D_BlinnPhong( const in float shininess, const in float dotNH ) {\n\treturn RECIPROCAL_PI * ( shininess * 0.5 + 1.0 ) * pow( dotNH, shininess );\n}\nvec3 BRDF_Specular_BlinnPhong( const in IncidentLight incidentLight, const in GeometricContext geometry, const in vec3 specularColor, const in float shininess ) {\n\tvec3 halfDir = normalize( incidentLight.direction + geometry.viewDir );\n\tfloat dotNH = saturate( dot( geometry.normal, halfDir ) );\n\tfloat dotLH = saturate( dot( incidentLight.direction, halfDir ) );\n\tvec3 F = F_Schlick( specularColor, dotLH );\n\tfloat G = G_BlinnPhong_Implicit( );\n\tfloat D = D_BlinnPhong( shininess, dotNH );\n\treturn F * ( G * D );\n}\nfloat GGXRoughnessToBlinnExponent( const in float ggxRoughness ) {\n\treturn ( 2.0 / pow2( ggxRoughness + 0.0001 ) - 2.0 );\n}\nfloat BlinnExponentToGGXRoughness( const in float blinnExponent ) {\n\treturn sqrt( 2.0 / ( blinnExponent + 2.0 ) );\n}\n#if defined( USE_SHEEN )\nfloat D_Charlie(float roughness, float NoH) {\n\tfloat invAlpha  = 1.0 / roughness;\n\tfloat cos2h = NoH * NoH;\n\tfloat sin2h = max(1.0 - cos2h, 0.0078125);\treturn (2.0 + invAlpha) * pow(sin2h, invAlpha * 0.5) / (2.0 * PI);\n}\nfloat V_Neubelt(float NoV, float NoL) {\n\treturn saturate(1.0 / (4.0 * (NoL + NoV - NoL * NoV)));\n}\nvec3 BRDF_Specular_Sheen( const in float roughness, const in vec3 L, const in GeometricContext geometry, vec3 specularColor ) {\n\tvec3 N = geometry.normal;\n\tvec3 V = geometry.viewDir;\n\tvec3 H = normalize( V + L );\n\tfloat dotNH = saturate( dot( N, H ) );\n\treturn specularColor * D_Charlie( roughness, dotNH ) * V_Neubelt( dot(N, V), dot(N, L) );\n}\n#endif",bumpmap_pars_fragment:"#ifdef USE_BUMPMAP\n\tuniform sampler2D bumpMap;\n\tuniform float bumpScale;\n\tvec2 dHdxy_fwd() {\n\t\tvec2 dSTdx = dFdx( vUv );\n\t\tvec2 dSTdy = dFdy( vUv );\n\t\tfloat Hll = bumpScale * texture2D( bumpMap, vUv ).x;\n\t\tfloat dBx = bumpScale * texture2D( bumpMap, vUv + dSTdx ).x - Hll;\n\t\tfloat dBy = bumpScale * texture2D( bumpMap, vUv + dSTdy ).x - Hll;\n\t\treturn vec2( dBx, dBy );\n\t}\n\tvec3 perturbNormalArb( vec3 surf_pos, vec3 surf_norm, vec2 dHdxy ) {\n\t\tvec3 vSigmaX = vec3( dFdx( surf_pos.x ), dFdx( surf_pos.y ), dFdx( surf_pos.z ) );\n\t\tvec3 vSigmaY = vec3( dFdy( surf_pos.x ), dFdy( surf_pos.y ), dFdy( surf_pos.z ) );\n\t\tvec3 vN = surf_norm;\n\t\tvec3 R1 = cross( vSigmaY, vN );\n\t\tvec3 R2 = cross( vN, vSigmaX );\n\t\tfloat fDet = dot( vSigmaX, R1 );\n\t\tfDet *= ( float( gl_FrontFacing ) * 2.0 - 1.0 );\n\t\tvec3 vGrad = sign( fDet ) * ( dHdxy.x * R1 + dHdxy.y * R2 );\n\t\treturn normalize( abs( fDet ) * surf_norm - vGrad );\n\t}\n#endif",clipping_planes_fragment:"#if NUM_CLIPPING_PLANES > 0\n\tvec4 plane;\n\t#pragma unroll_loop_start\n\tfor ( int i = 0; i < UNION_CLIPPING_PLANES; i ++ ) {\n\t\tplane = clippingPlanes[ i ];\n\t\tif ( dot( vClipPosition, plane.xyz ) > plane.w ) discard;\n\t}\n\t#pragma unroll_loop_end\n\t#if UNION_CLIPPING_PLANES < NUM_CLIPPING_PLANES\n\t\tbool clipped = true;\n\t\t#pragma unroll_loop_start\n\t\tfor ( int i = UNION_CLIPPING_PLANES; i < NUM_CLIPPING_PLANES; i ++ ) {\n\t\t\tplane = clippingPlanes[ i ];\n\t\t\tclipped = ( dot( vClipPosition, plane.xyz ) > plane.w ) && clipped;\n\t\t}\n\t\t#pragma unroll_loop_end\n\t\tif ( clipped ) discard;\n\t#endif\n#endif",clipping_planes_pars_fragment:"#if NUM_CLIPPING_PLANES > 0\n\tvarying vec3 vClipPosition;\n\tuniform vec4 clippingPlanes[ NUM_CLIPPING_PLANES ];\n#endif",clipping_planes_pars_vertex:"#if NUM_CLIPPING_PLANES > 0\n\tvarying vec3 vClipPosition;\n#endif",clipping_planes_vertex:"#if NUM_CLIPPING_PLANES > 0\n\tvClipPosition = - mvPosition.xyz;\n#endif",color_fragment:"#ifdef USE_COLOR\n\tdiffuseColor.rgb *= vColor;\n#endif",color_pars_fragment:"#ifdef USE_COLOR\n\tvarying vec3 vColor;\n#endif",color_pars_vertex:"#ifdef USE_COLOR\n\tvarying vec3 vColor;\n#endif",color_vertex:"#ifdef USE_COLOR\n\tvColor.xyz = color.xyz;\n#endif",common:"#define PI 3.14159265359\n#define PI2 6.28318530718\n#define PI_HALF 1.5707963267949\n#define RECIPROCAL_PI 0.31830988618\n#define RECIPROCAL_PI2 0.15915494\n#define LOG2 1.442695\n#define EPSILON 1e-6\n#ifndef saturate\n#define saturate(a) clamp( a, 0.0, 1.0 )\n#endif\n#define whiteComplement(a) ( 1.0 - saturate( a ) )\nfloat pow2( const in float x ) { return x*x; }\nfloat pow3( const in float x ) { return x*x*x; }\nfloat pow4( const in float x ) { float x2 = x*x; return x2*x2; }\nfloat average( const in vec3 color ) { return dot( color, vec3( 0.3333 ) ); }\nhighp float rand( const in vec2 uv ) {\n\tconst highp float a = 12.9898, b = 78.233, c = 43758.5453;\n\thighp float dt = dot( uv.xy, vec2( a,b ) ), sn = mod( dt, PI );\n\treturn fract(sin(sn) * c);\n}\n#ifdef HIGH_PRECISION\n\tfloat precisionSafeLength( vec3 v ) { return length( v ); }\n#else\n\tfloat max3( vec3 v ) { return max( max( v.x, v.y ), v.z ); }\n\tfloat precisionSafeLength( vec3 v ) {\n\t\tfloat maxComponent = max3( abs( v ) );\n\t\treturn length( v / maxComponent ) * maxComponent;\n\t}\n#endif\nstruct IncidentLight {\n\tvec3 color;\n\tvec3 direction;\n\tbool visible;\n};\nstruct ReflectedLight {\n\tvec3 directDiffuse;\n\tvec3 directSpecular;\n\tvec3 indirectDiffuse;\n\tvec3 indirectSpecular;\n};\nstruct GeometricContext {\n\tvec3 position;\n\tvec3 normal;\n\tvec3 viewDir;\n#ifdef CLEARCOAT\n\tvec3 clearcoatNormal;\n#endif\n};\nvec3 transformDirection( in vec3 dir, in mat4 matrix ) {\n\treturn normalize( ( matrix * vec4( dir, 0.0 ) ).xyz );\n}\nvec3 inverseTransformDirection( in vec3 dir, in mat4 matrix ) {\n\treturn normalize( ( vec4( dir, 0.0 ) * matrix ).xyz );\n}\nvec3 projectOnPlane(in vec3 point, in vec3 pointOnPlane, in vec3 planeNormal ) {\n\tfloat distance = dot( planeNormal, point - pointOnPlane );\n\treturn - distance * planeNormal + point;\n}\nfloat sideOfPlane( in vec3 point, in vec3 pointOnPlane, in vec3 planeNormal ) {\n\treturn sign( dot( point - pointOnPlane, planeNormal ) );\n}\nvec3 linePlaneIntersect( in vec3 pointOnLine, in vec3 lineDirection, in vec3 pointOnPlane, in vec3 planeNormal ) {\n\treturn lineDirection * ( dot( planeNormal, pointOnPlane - pointOnLine ) / dot( planeNormal, lineDirection ) ) + pointOnLine;\n}\nmat3 transposeMat3( const in mat3 m ) {\n\tmat3 tmp;\n\ttmp[ 0 ] = vec3( m[ 0 ].x, m[ 1 ].x, m[ 2 ].x );\n\ttmp[ 1 ] = vec3( m[ 0 ].y, m[ 1 ].y, m[ 2 ].y );\n\ttmp[ 2 ] = vec3( m[ 0 ].z, m[ 1 ].z, m[ 2 ].z );\n\treturn tmp;\n}\nfloat linearToRelativeLuminance( const in vec3 color ) {\n\tvec3 weights = vec3( 0.2126, 0.7152, 0.0722 );\n\treturn dot( weights, color.rgb );\n}\nbool isPerspectiveMatrix( mat4 m ) {\n  return m[ 2 ][ 3 ] == - 1.0;\n}\nvec2 equirectUv( in vec3 dir ) {\n\tfloat u = atan( dir.z, dir.x ) * RECIPROCAL_PI2 + 0.5;\n\tfloat v = asin( clamp( dir.y, - 1.0, 1.0 ) ) * RECIPROCAL_PI + 0.5;\n\treturn vec2( u, v );\n}",cube_uv_reflection_fragment:"#ifdef ENVMAP_TYPE_CUBE_UV\n#define cubeUV_maxMipLevel 8.0\n#define cubeUV_minMipLevel 4.0\n#define cubeUV_maxTileSize 256.0\n#define cubeUV_minTileSize 16.0\nfloat getFace(vec3 direction) {\n    vec3 absDirection = abs(direction);\n    float face = -1.0;\n    if (absDirection.x > absDirection.z) {\n      if (absDirection.x > absDirection.y)\n        face = direction.x > 0.0 ? 0.0 : 3.0;\n      else\n        face = direction.y > 0.0 ? 1.0 : 4.0;\n    } else {\n      if (absDirection.z > absDirection.y)\n        face = direction.z > 0.0 ? 2.0 : 5.0;\n      else\n        face = direction.y > 0.0 ? 1.0 : 4.0;\n    }\n    return face;\n}\nvec2 getUV(vec3 direction, float face) {\n    vec2 uv;\n    if (face == 0.0) {\n      uv = vec2(direction.z, direction.y) / abs(direction.x);    } else if (face == 1.0) {\n      uv = vec2(-direction.x, -direction.z) / abs(direction.y);    } else if (face == 2.0) {\n      uv = vec2(-direction.x, direction.y) / abs(direction.z);    } else if (face == 3.0) {\n      uv = vec2(-direction.z, direction.y) / abs(direction.x);    } else if (face == 4.0) {\n      uv = vec2(-direction.x, direction.z) / abs(direction.y);    } else {\n      uv = vec2(direction.x, direction.y) / abs(direction.z);    }\n    return 0.5 * (uv + 1.0);\n}\nvec3 bilinearCubeUV(sampler2D envMap, vec3 direction, float mipInt) {\n  float face = getFace(direction);\n  float filterInt = max(cubeUV_minMipLevel - mipInt, 0.0);\n  mipInt = max(mipInt, cubeUV_minMipLevel);\n  float faceSize = exp2(mipInt);\n  float texelSize = 1.0 / (3.0 * cubeUV_maxTileSize);\n  vec2 uv = getUV(direction, face) * (faceSize - 1.0);\n  vec2 f = fract(uv);\n  uv += 0.5 - f;\n  if (face > 2.0) {\n    uv.y += faceSize;\n    face -= 3.0;\n  }\n  uv.x += face * faceSize;\n  if(mipInt < cubeUV_maxMipLevel){\n    uv.y += 2.0 * cubeUV_maxTileSize;\n  }\n  uv.y += filterInt * 2.0 * cubeUV_minTileSize;\n  uv.x += 3.0 * max(0.0, cubeUV_maxTileSize - 2.0 * faceSize);\n  uv *= texelSize;\n  vec3 tl = envMapTexelToLinear(texture2D(envMap, uv)).rgb;\n  uv.x += texelSize;\n  vec3 tr = envMapTexelToLinear(texture2D(envMap, uv)).rgb;\n  uv.y += texelSize;\n  vec3 br = envMapTexelToLinear(texture2D(envMap, uv)).rgb;\n  uv.x -= texelSize;\n  vec3 bl = envMapTexelToLinear(texture2D(envMap, uv)).rgb;\n  vec3 tm = mix(tl, tr, f.x);\n  vec3 bm = mix(bl, br, f.x);\n  return mix(tm, bm, f.y);\n}\n#define r0 1.0\n#define v0 0.339\n#define m0 -2.0\n#define r1 0.8\n#define v1 0.276\n#define m1 -1.0\n#define r4 0.4\n#define v4 0.046\n#define m4 2.0\n#define r5 0.305\n#define v5 0.016\n#define m5 3.0\n#define r6 0.21\n#define v6 0.0038\n#define m6 4.0\nfloat roughnessToMip(float roughness) {\n  float mip = 0.0;\n  if (roughness >= r1) {\n    mip = (r0 - roughness) * (m1 - m0) / (r0 - r1) + m0;\n  } else if (roughness >= r4) {\n    mip = (r1 - roughness) * (m4 - m1) / (r1 - r4) + m1;\n  } else if (roughness >= r5) {\n    mip = (r4 - roughness) * (m5 - m4) / (r4 - r5) + m4;\n  } else if (roughness >= r6) {\n    mip = (r5 - roughness) * (m6 - m5) / (r5 - r6) + m5;\n  } else {\n    mip = -2.0 * log2(1.16 * roughness);  }\n  return mip;\n}\nvec4 textureCubeUV(sampler2D envMap, vec3 sampleDir, float roughness) {\n  float mip = clamp(roughnessToMip(roughness), m0, cubeUV_maxMipLevel);\n  float mipF = fract(mip);\n  float mipInt = floor(mip);\n  vec3 color0 = bilinearCubeUV(envMap, sampleDir, mipInt);\n  if (mipF == 0.0) {\n    return vec4(color0, 1.0);\n  } else {\n    vec3 color1 = bilinearCubeUV(envMap, sampleDir, mipInt + 1.0);\n    return vec4(mix(color0, color1, mipF), 1.0);\n  }\n}\n#endif",defaultnormal_vertex:"vec3 transformedNormal = objectNormal;\n#ifdef USE_INSTANCING\n\tmat3 m = mat3( instanceMatrix );\n\ttransformedNormal /= vec3( dot( m[ 0 ], m[ 0 ] ), dot( m[ 1 ], m[ 1 ] ), dot( m[ 2 ], m[ 2 ] ) );\n\ttransformedNormal = m * transformedNormal;\n#endif\ntransformedNormal = normalMatrix * transformedNormal;\n#ifdef FLIP_SIDED\n\ttransformedNormal = - transformedNormal;\n#endif\n#ifdef USE_TANGENT\n\tvec3 transformedTangent = ( modelViewMatrix * vec4( objectTangent, 0.0 ) ).xyz;\n\t#ifdef FLIP_SIDED\n\t\ttransformedTangent = - transformedTangent;\n\t#endif\n#endif",displacementmap_pars_vertex:"#ifdef USE_DISPLACEMENTMAP\n\tuniform sampler2D displacementMap;\n\tuniform float displacementScale;\n\tuniform float displacementBias;\n#endif",displacementmap_vertex:"#ifdef USE_DISPLACEMENTMAP\n\ttransformed += normalize( objectNormal ) * ( texture2D( displacementMap, vUv ).x * displacementScale + displacementBias );\n#endif",emissivemap_fragment:"#ifdef USE_EMISSIVEMAP\n\tvec4 emissiveColor = texture2D( emissiveMap, vUv );\n\temissiveColor.rgb = emissiveMapTexelToLinear( emissiveColor ).rgb;\n\ttotalEmissiveRadiance *= emissiveColor.rgb;\n#endif",emissivemap_pars_fragment:"#ifdef USE_EMISSIVEMAP\n\tuniform sampler2D emissiveMap;\n#endif",encodings_fragment:"gl_FragColor = linearToOutputTexel( gl_FragColor );",encodings_pars_fragment:"\nvec4 LinearToLinear( in vec4 value ) {\n\treturn value;\n}\nvec4 GammaToLinear( in vec4 value, in float gammaFactor ) {\n\treturn vec4( pow( value.rgb, vec3( gammaFactor ) ), value.a );\n}\nvec4 LinearToGamma( in vec4 value, in float gammaFactor ) {\n\treturn vec4( pow( value.rgb, vec3( 1.0 / gammaFactor ) ), value.a );\n}\nvec4 sRGBToLinear( in vec4 value ) {\n\treturn vec4( mix( pow( value.rgb * 0.9478672986 + vec3( 0.0521327014 ), vec3( 2.4 ) ), value.rgb * 0.0773993808, vec3( lessThanEqual( value.rgb, vec3( 0.04045 ) ) ) ), value.a );\n}\nvec4 LinearTosRGB( in vec4 value ) {\n\treturn vec4( mix( pow( value.rgb, vec3( 0.41666 ) ) * 1.055 - vec3( 0.055 ), value.rgb * 12.92, vec3( lessThanEqual( value.rgb, vec3( 0.0031308 ) ) ) ), value.a );\n}\nvec4 RGBEToLinear( in vec4 value ) {\n\treturn vec4( value.rgb * exp2( value.a * 255.0 - 128.0 ), 1.0 );\n}\nvec4 LinearToRGBE( in vec4 value ) {\n\tfloat maxComponent = max( max( value.r, value.g ), value.b );\n\tfloat fExp = clamp( ceil( log2( maxComponent ) ), -128.0, 127.0 );\n\treturn vec4( value.rgb / exp2( fExp ), ( fExp + 128.0 ) / 255.0 );\n}\nvec4 RGBMToLinear( in vec4 value, in float maxRange ) {\n\treturn vec4( value.rgb * value.a * maxRange, 1.0 );\n}\nvec4 LinearToRGBM( in vec4 value, in float maxRange ) {\n\tfloat maxRGB = max( value.r, max( value.g, value.b ) );\n\tfloat M = clamp( maxRGB / maxRange, 0.0, 1.0 );\n\tM = ceil( M * 255.0 ) / 255.0;\n\treturn vec4( value.rgb / ( M * maxRange ), M );\n}\nvec4 RGBDToLinear( in vec4 value, in float maxRange ) {\n\treturn vec4( value.rgb * ( ( maxRange / 255.0 ) / value.a ), 1.0 );\n}\nvec4 LinearToRGBD( in vec4 value, in float maxRange ) {\n\tfloat maxRGB = max( value.r, max( value.g, value.b ) );\n\tfloat D = max( maxRange / maxRGB, 1.0 );\n\tD = clamp( floor( D ) / 255.0, 0.0, 1.0 );\n\treturn vec4( value.rgb * ( D * ( 255.0 / maxRange ) ), D );\n}\nconst mat3 cLogLuvM = mat3( 0.2209, 0.3390, 0.4184, 0.1138, 0.6780, 0.7319, 0.0102, 0.1130, 0.2969 );\nvec4 LinearToLogLuv( in vec4 value )  {\n\tvec3 Xp_Y_XYZp = cLogLuvM * value.rgb;\n\tXp_Y_XYZp = max( Xp_Y_XYZp, vec3( 1e-6, 1e-6, 1e-6 ) );\n\tvec4 vResult;\n\tvResult.xy = Xp_Y_XYZp.xy / Xp_Y_XYZp.z;\n\tfloat Le = 2.0 * log2(Xp_Y_XYZp.y) + 127.0;\n\tvResult.w = fract( Le );\n\tvResult.z = ( Le - ( floor( vResult.w * 255.0 ) ) / 255.0 ) / 255.0;\n\treturn vResult;\n}\nconst mat3 cLogLuvInverseM = mat3( 6.0014, -2.7008, -1.7996, -1.3320, 3.1029, -5.7721, 0.3008, -1.0882, 5.6268 );\nvec4 LogLuvToLinear( in vec4 value ) {\n\tfloat Le = value.z * 255.0 + value.w;\n\tvec3 Xp_Y_XYZp;\n\tXp_Y_XYZp.y = exp2( ( Le - 127.0 ) / 2.0 );\n\tXp_Y_XYZp.z = Xp_Y_XYZp.y / value.y;\n\tXp_Y_XYZp.x = value.x * Xp_Y_XYZp.z;\n\tvec3 vRGB = cLogLuvInverseM * Xp_Y_XYZp.rgb;\n\treturn vec4( max( vRGB, 0.0 ), 1.0 );\n}",envmap_fragment:"#ifdef USE_ENVMAP\n\t#ifdef ENV_WORLDPOS\n\t\tvec3 cameraToFrag;\n\t\t\n\t\tif ( isOrthographic ) {\n\t\t\tcameraToFrag = normalize( vec3( - viewMatrix[ 0 ][ 2 ], - viewMatrix[ 1 ][ 2 ], - viewMatrix[ 2 ][ 2 ] ) );\n\t\t}  else {\n\t\t\tcameraToFrag = normalize( vWorldPosition - cameraPosition );\n\t\t}\n\t\tvec3 worldNormal = inverseTransformDirection( normal, viewMatrix );\n\t\t#ifdef ENVMAP_MODE_REFLECTION\n\t\t\tvec3 reflectVec = reflect( cameraToFrag, worldNormal );\n\t\t#else\n\t\t\tvec3 reflectVec = refract( cameraToFrag, worldNormal, refractionRatio );\n\t\t#endif\n\t#else\n\t\tvec3 reflectVec = vReflect;\n\t#endif\n\t#ifdef ENVMAP_TYPE_CUBE\n\t\tvec4 envColor = textureCube( envMap, vec3( flipEnvMap * reflectVec.x, reflectVec.yz ) );\n\t#elif defined( ENVMAP_TYPE_CUBE_UV )\n\t\tvec4 envColor = textureCubeUV( envMap, reflectVec, 0.0 );\n\t#elif defined( ENVMAP_TYPE_EQUIREC )\n\t\treflectVec = normalize( reflectVec );\n\t\tvec2 sampleUV = equirectUv( reflectVec );\n\t\tvec4 envColor = texture2D( envMap, sampleUV );\n\t#elif defined( ENVMAP_TYPE_SPHERE )\n\t\treflectVec = normalize( reflectVec );\n\t\tvec3 reflectView = normalize( ( viewMatrix * vec4( reflectVec, 0.0 ) ).xyz + vec3( 0.0, 0.0, 1.0 ) );\n\t\tvec4 envColor = texture2D( envMap, reflectView.xy * 0.5 + 0.5 );\n\t#else\n\t\tvec4 envColor = vec4( 0.0 );\n\t#endif\n\t#ifndef ENVMAP_TYPE_CUBE_UV\n\t\tenvColor = envMapTexelToLinear( envColor );\n\t#endif\n\t#ifdef ENVMAP_BLENDING_MULTIPLY\n\t\toutgoingLight = mix( outgoingLight, outgoingLight * envColor.xyz, specularStrength * reflectivity );\n\t#elif defined( ENVMAP_BLENDING_MIX )\n\t\toutgoingLight = mix( outgoingLight, envColor.xyz, specularStrength * reflectivity );\n\t#elif defined( ENVMAP_BLENDING_ADD )\n\t\toutgoingLight += envColor.xyz * specularStrength * reflectivity;\n\t#endif\n#endif",envmap_common_pars_fragment:"#ifdef USE_ENVMAP\n\tuniform float envMapIntensity;\n\tuniform float flipEnvMap;\n\tuniform int maxMipLevel;\n\t#ifdef ENVMAP_TYPE_CUBE\n\t\tuniform samplerCube envMap;\n\t#else\n\t\tuniform sampler2D envMap;\n\t#endif\n\t\n#endif",envmap_pars_fragment:"#ifdef USE_ENVMAP\n\tuniform float reflectivity;\n\t#if defined( USE_BUMPMAP ) || defined( USE_NORMALMAP ) || defined( PHONG )\n\t\t#define ENV_WORLDPOS\n\t#endif\n\t#ifdef ENV_WORLDPOS\n\t\tvarying vec3 vWorldPosition;\n\t\tuniform float refractionRatio;\n\t#else\n\t\tvarying vec3 vReflect;\n\t#endif\n#endif",envmap_pars_vertex:"#ifdef USE_ENVMAP\n\t#if defined( USE_BUMPMAP ) || defined( USE_NORMALMAP ) ||defined( PHONG )\n\t\t#define ENV_WORLDPOS\n\t#endif\n\t#ifdef ENV_WORLDPOS\n\t\t\n\t\tvarying vec3 vWorldPosition;\n\t#else\n\t\tvarying vec3 vReflect;\n\t\tuniform float refractionRatio;\n\t#endif\n#endif",envmap_physical_pars_fragment:"#if defined( USE_ENVMAP )\n\t#ifdef ENVMAP_MODE_REFRACTION\n\t\tuniform float refractionRatio;\n\t#endif\n\tvec3 getLightProbeIndirectIrradiance( const in GeometricContext geometry, const in int maxMIPLevel ) {\n\t\tvec3 worldNormal = inverseTransformDirection( geometry.normal, viewMatrix );\n\t\t#ifdef ENVMAP_TYPE_CUBE\n\t\t\tvec3 queryVec = vec3( flipEnvMap * worldNormal.x, worldNormal.yz );\n\t\t\t#ifdef TEXTURE_LOD_EXT\n\t\t\t\tvec4 envMapColor = textureCubeLodEXT( envMap, queryVec, float( maxMIPLevel ) );\n\t\t\t#else\n\t\t\t\tvec4 envMapColor = textureCube( envMap, queryVec, float( maxMIPLevel ) );\n\t\t\t#endif\n\t\t\tenvMapColor.rgb = envMapTexelToLinear( envMapColor ).rgb;\n\t\t#elif defined( ENVMAP_TYPE_CUBE_UV )\n\t\t\tvec4 envMapColor = textureCubeUV( envMap, worldNormal, 1.0 );\n\t\t#else\n\t\t\tvec4 envMapColor = vec4( 0.0 );\n\t\t#endif\n\t\treturn PI * envMapColor.rgb * envMapIntensity;\n\t}\n\tfloat getSpecularMIPLevel( const in float roughness, const in int maxMIPLevel ) {\n\t\tfloat maxMIPLevelScalar = float( maxMIPLevel );\n\t\tfloat sigma = PI * roughness * roughness / ( 1.0 + roughness );\n\t\tfloat desiredMIPLevel = maxMIPLevelScalar + log2( sigma );\n\t\treturn clamp( desiredMIPLevel, 0.0, maxMIPLevelScalar );\n\t}\n\tvec3 getLightProbeIndirectRadiance( const in vec3 viewDir, const in vec3 normal, const in float roughness, const in int maxMIPLevel ) {\n\t\t#ifdef ENVMAP_MODE_REFLECTION\n\t\t  vec3 reflectVec = reflect( -viewDir, normal );\n\t\t  reflectVec = normalize( mix( reflectVec, normal, roughness * roughness) );\n\t\t#else\n\t\t  vec3 reflectVec = refract( -viewDir, normal, refractionRatio );\n\t\t#endif\n\t\treflectVec = inverseTransformDirection( reflectVec, viewMatrix );\n\t\tfloat specularMIPLevel = getSpecularMIPLevel( roughness, maxMIPLevel );\n\t\t#ifdef ENVMAP_TYPE_CUBE\n\t\t\tvec3 queryReflectVec = vec3( flipEnvMap * reflectVec.x, reflectVec.yz );\n\t\t\t#ifdef TEXTURE_LOD_EXT\n\t\t\t\tvec4 envMapColor = textureCubeLodEXT( envMap, queryReflectVec, specularMIPLevel );\n\t\t\t#else\n\t\t\t\tvec4 envMapColor = textureCube( envMap, queryReflectVec, specularMIPLevel );\n\t\t\t#endif\n\t\t\tenvMapColor.rgb = envMapTexelToLinear( envMapColor ).rgb;\n\t\t#elif defined( ENVMAP_TYPE_CUBE_UV )\n\t\t\tvec4 envMapColor = textureCubeUV( envMap, reflectVec, roughness );\n\t\t#elif defined( ENVMAP_TYPE_EQUIREC )\n\t\t\tvec2 sampleUV = equirectUv( reflectVec );\n\t\t\t#ifdef TEXTURE_LOD_EXT\n\t\t\t\tvec4 envMapColor = texture2DLodEXT( envMap, sampleUV, specularMIPLevel );\n\t\t\t#else\n\t\t\t\tvec4 envMapColor = texture2D( envMap, sampleUV, specularMIPLevel );\n\t\t\t#endif\n\t\t\tenvMapColor.rgb = envMapTexelToLinear( envMapColor ).rgb;\n\t\t#elif defined( ENVMAP_TYPE_SPHERE )\n\t\t\tvec3 reflectView = normalize( ( viewMatrix * vec4( reflectVec, 0.0 ) ).xyz + vec3( 0.0,0.0,1.0 ) );\n\t\t\t#ifdef TEXTURE_LOD_EXT\n\t\t\t\tvec4 envMapColor = texture2DLodEXT( envMap, reflectView.xy * 0.5 + 0.5, specularMIPLevel );\n\t\t\t#else\n\t\t\t\tvec4 envMapColor = texture2D( envMap, reflectView.xy * 0.5 + 0.5, specularMIPLevel );\n\t\t\t#endif\n\t\t\tenvMapColor.rgb = envMapTexelToLinear( envMapColor ).rgb;\n\t\t#endif\n\t\treturn envMapColor.rgb * envMapIntensity;\n\t}\n#endif",envmap_vertex:"#ifdef USE_ENVMAP\n\t#ifdef ENV_WORLDPOS\n\t\tvWorldPosition = worldPosition.xyz;\n\t#else\n\t\tvec3 cameraToVertex;\n\t\tif ( isOrthographic ) { \n\t\t\tcameraToVertex = normalize( vec3( - viewMatrix[ 0 ][ 2 ], - viewMatrix[ 1 ][ 2 ], - viewMatrix[ 2 ][ 2 ] ) );\n\t\t} else {\n\t\t\tcameraToVertex = normalize( worldPosition.xyz - cameraPosition );\n\t\t}\n\t\tvec3 worldNormal = inverseTransformDirection( transformedNormal, viewMatrix );\n\t\t#ifdef ENVMAP_MODE_REFLECTION\n\t\t\tvReflect = reflect( cameraToVertex, worldNormal );\n\t\t#else\n\t\t\tvReflect = refract( cameraToVertex, worldNormal, refractionRatio );\n\t\t#endif\n\t#endif\n#endif",fog_vertex:"#ifdef USE_FOG\n\tfogDepth = -mvPosition.z;\n#endif",fog_pars_vertex:"#ifdef USE_FOG\n\tvarying float fogDepth;\n#endif",fog_fragment:"#ifdef USE_FOG\n\t#ifdef FOG_EXP2\n\t\tfloat fogFactor = 1.0 - exp( - fogDensity * fogDensity * fogDepth * fogDepth );\n\t#else\n\t\tfloat fogFactor = smoothstep( fogNear, fogFar, fogDepth );\n\t#endif\n\tgl_FragColor.rgb = mix( gl_FragColor.rgb, fogColor, fogFactor );\n#endif",fog_pars_fragment:"#ifdef USE_FOG\n\tuniform vec3 fogColor;\n\tvarying float fogDepth;\n\t#ifdef FOG_EXP2\n\t\tuniform float fogDensity;\n\t#else\n\t\tuniform float fogNear;\n\t\tuniform float fogFar;\n\t#endif\n#endif",gradientmap_pars_fragment:"#ifdef USE_GRADIENTMAP\n\tuniform sampler2D gradientMap;\n#endif\nvec3 getGradientIrradiance( vec3 normal, vec3 lightDirection ) {\n\tfloat dotNL = dot( normal, lightDirection );\n\tvec2 coord = vec2( dotNL * 0.5 + 0.5, 0.0 );\n\t#ifdef USE_GRADIENTMAP\n\t\treturn texture2D( gradientMap, coord ).rgb;\n\t#else\n\t\treturn ( coord.x < 0.7 ) ? vec3( 0.7 ) : vec3( 1.0 );\n\t#endif\n}",lightmap_fragment:"#ifdef USE_LIGHTMAP\n\tvec4 lightMapTexel= texture2D( lightMap, vUv2 );\n\treflectedLight.indirectDiffuse += PI * lightMapTexelToLinear( lightMapTexel ).rgb * lightMapIntensity;\n#endif",lightmap_pars_fragment:"#ifdef USE_LIGHTMAP\n\tuniform sampler2D lightMap;\n\tuniform float lightMapIntensity;\n#endif",lights_lambert_vertex:"vec3 diffuse = vec3( 1.0 );\nGeometricContext geometry;\ngeometry.position = mvPosition.xyz;\ngeometry.normal = normalize( transformedNormal );\ngeometry.viewDir = ( isOrthographic ) ? vec3( 0, 0, 1 ) : normalize( -mvPosition.xyz );\nGeometricContext backGeometry;\nbackGeometry.position = geometry.position;\nbackGeometry.normal = -geometry.normal;\nbackGeometry.viewDir = geometry.viewDir;\nvLightFront = vec3( 0.0 );\nvIndirectFront = vec3( 0.0 );\n#ifdef DOUBLE_SIDED\n\tvLightBack = vec3( 0.0 );\n\tvIndirectBack = vec3( 0.0 );\n#endif\nIncidentLight directLight;\nfloat dotNL;\nvec3 directLightColor_Diffuse;\nvIndirectFront += getAmbientLightIrradiance( ambientLightColor );\nvIndirectFront += getLightProbeIrradiance( lightProbe, geometry );\n#ifdef DOUBLE_SIDED\n\tvIndirectBack += getAmbientLightIrradiance( ambientLightColor );\n\tvIndirectBack += getLightProbeIrradiance( lightProbe, backGeometry );\n#endif\n#if NUM_POINT_LIGHTS > 0\n\t#pragma unroll_loop_start\n\tfor ( int i = 0; i < NUM_POINT_LIGHTS; i ++ ) {\n\t\tgetPointDirectLightIrradiance( pointLights[ i ], geometry, directLight );\n\t\tdotNL = dot( geometry.normal, directLight.direction );\n\t\tdirectLightColor_Diffuse = PI * directLight.color;\n\t\tvLightFront += saturate( dotNL ) * directLightColor_Diffuse;\n\t\t#ifdef DOUBLE_SIDED\n\t\t\tvLightBack += saturate( -dotNL ) * directLightColor_Diffuse;\n\t\t#endif\n\t}\n\t#pragma unroll_loop_end\n#endif\n#if NUM_SPOT_LIGHTS > 0\n\t#pragma unroll_loop_start\n\tfor ( int i = 0; i < NUM_SPOT_LIGHTS; i ++ ) {\n\t\tgetSpotDirectLightIrradiance( spotLights[ i ], geometry, directLight );\n\t\tdotNL = dot( geometry.normal, directLight.direction );\n\t\tdirectLightColor_Diffuse = PI * directLight.color;\n\t\tvLightFront += saturate( dotNL ) * directLightColor_Diffuse;\n\t\t#ifdef DOUBLE_SIDED\n\t\t\tvLightBack += saturate( -dotNL ) * directLightColor_Diffuse;\n\t\t#endif\n\t}\n\t#pragma unroll_loop_end\n#endif\n#if NUM_DIR_LIGHTS > 0\n\t#pragma unroll_loop_start\n\tfor ( int i = 0; i < NUM_DIR_LIGHTS; i ++ ) {\n\t\tgetDirectionalDirectLightIrradiance( directionalLights[ i ], geometry, directLight );\n\t\tdotNL = dot( geometry.normal, directLight.direction );\n\t\tdirectLightColor_Diffuse = PI * directLight.color;\n\t\tvLightFront += saturate( dotNL ) * directLightColor_Diffuse;\n\t\t#ifdef DOUBLE_SIDED\n\t\t\tvLightBack += saturate( -dotNL ) * directLightColor_Diffuse;\n\t\t#endif\n\t}\n\t#pragma unroll_loop_end\n#endif\n#if NUM_HEMI_LIGHTS > 0\n\t#pragma unroll_loop_start\n\tfor ( int i = 0; i < NUM_HEMI_LIGHTS; i ++ ) {\n\t\tvIndirectFront += getHemisphereLightIrradiance( hemisphereLights[ i ], geometry );\n\t\t#ifdef DOUBLE_SIDED\n\t\t\tvIndirectBack += getHemisphereLightIrradiance( hemisphereLights[ i ], backGeometry );\n\t\t#endif\n\t}\n\t#pragma unroll_loop_end\n#endif",lights_pars_begin:"uniform bool receiveShadow;\nuniform vec3 ambientLightColor;\nuniform vec3 lightProbe[ 9 ];\nvec3 shGetIrradianceAt( in vec3 normal, in vec3 shCoefficients[ 9 ] ) {\n\tfloat x = normal.x, y = normal.y, z = normal.z;\n\tvec3 result = shCoefficients[ 0 ] * 0.886227;\n\tresult += shCoefficients[ 1 ] * 2.0 * 0.511664 * y;\n\tresult += shCoefficients[ 2 ] * 2.0 * 0.511664 * z;\n\tresult += shCoefficients[ 3 ] * 2.0 * 0.511664 * x;\n\tresult += shCoefficients[ 4 ] * 2.0 * 0.429043 * x * y;\n\tresult += shCoefficients[ 5 ] * 2.0 * 0.429043 * y * z;\n\tresult += shCoefficients[ 6 ] * ( 0.743125 * z * z - 0.247708 );\n\tresult += shCoefficients[ 7 ] * 2.0 * 0.429043 * x * z;\n\tresult += shCoefficients[ 8 ] * 0.429043 * ( x * x - y * y );\n\treturn result;\n}\nvec3 getLightProbeIrradiance( const in vec3 lightProbe[ 9 ], const in GeometricContext geometry ) {\n\tvec3 worldNormal = inverseTransformDirection( geometry.normal, viewMatrix );\n\tvec3 irradiance = shGetIrradianceAt( worldNormal, lightProbe );\n\treturn irradiance;\n}\nvec3 getAmbientLightIrradiance( const in vec3 ambientLightColor ) {\n\tvec3 irradiance = ambientLightColor;\n\t#ifndef PHYSICALLY_CORRECT_LIGHTS\n\t\tirradiance *= PI;\n\t#endif\n\treturn irradiance;\n}\n#if NUM_DIR_LIGHTS > 0\n\tstruct DirectionalLight {\n\t\tvec3 direction;\n\t\tvec3 color;\n\t};\n\tuniform DirectionalLight directionalLights[ NUM_DIR_LIGHTS ];\n\t#if defined( USE_SHADOWMAP ) && NUM_DIR_LIGHT_SHADOWS > 0\n\t\tstruct DirectionalLightShadow {\n\t\t\tfloat shadowBias;\n\t\t\tfloat shadowRadius;\n\t\t\tvec2 shadowMapSize;\n\t\t};\n\t\tuniform DirectionalLightShadow directionalLightShadows[ NUM_DIR_LIGHT_SHADOWS ];\n\t#endif\n\tvoid getDirectionalDirectLightIrradiance( const in DirectionalLight directionalLight, const in GeometricContext geometry, out IncidentLight directLight ) {\n\t\tdirectLight.color = directionalLight.color;\n\t\tdirectLight.direction = directionalLight.direction;\n\t\tdirectLight.visible = true;\n\t}\n#endif\n#if NUM_POINT_LIGHTS > 0\n\tstruct PointLight {\n\t\tvec3 position;\n\t\tvec3 color;\n\t\tfloat distance;\n\t\tfloat decay;\n\t};\n\tuniform PointLight pointLights[ NUM_POINT_LIGHTS ];\n\t#if defined( USE_SHADOWMAP ) && NUM_POINT_LIGHT_SHADOWS > 0\n\t\tstruct PointLightShadow {\n\t\t\tfloat shadowBias;\n\t\t\tfloat shadowRadius;\n\t\t\tvec2 shadowMapSize;\n\t\t\tfloat shadowCameraNear;\n\t\t\tfloat shadowCameraFar;\n\t\t};\n\t\tuniform PointLightShadow pointLightShadows[ NUM_POINT_LIGHT_SHADOWS ];\n\t#endif\n\tvoid getPointDirectLightIrradiance( const in PointLight pointLight, const in GeometricContext geometry, out IncidentLight directLight ) {\n\t\tvec3 lVector = pointLight.position - geometry.position;\n\t\tdirectLight.direction = normalize( lVector );\n\t\tfloat lightDistance = length( lVector );\n\t\tdirectLight.color = pointLight.color;\n\t\tdirectLight.color *= punctualLightIntensityToIrradianceFactor( lightDistance, pointLight.distance, pointLight.decay );\n\t\tdirectLight.visible = ( directLight.color != vec3( 0.0 ) );\n\t}\n#endif\n#if NUM_SPOT_LIGHTS > 0\n\tstruct SpotLight {\n\t\tvec3 position;\n\t\tvec3 direction;\n\t\tvec3 color;\n\t\tfloat distance;\n\t\tfloat decay;\n\t\tfloat coneCos;\n\t\tfloat penumbraCos;\n\t};\n\tuniform SpotLight spotLights[ NUM_SPOT_LIGHTS ];\n\t#if defined( USE_SHADOWMAP ) && NUM_SPOT_LIGHT_SHADOWS > 0\n\t\tstruct SpotLightShadow {\n\t\t\tfloat shadowBias;\n\t\t\tfloat shadowRadius;\n\t\t\tvec2 shadowMapSize;\n\t\t};\n\t\tuniform SpotLightShadow spotLightShadows[ NUM_SPOT_LIGHT_SHADOWS ];\n\t#endif\n\tvoid getSpotDirectLightIrradiance( const in SpotLight spotLight, const in GeometricContext geometry, out IncidentLight directLight  ) {\n\t\tvec3 lVector = spotLight.position - geometry.position;\n\t\tdirectLight.direction = normalize( lVector );\n\t\tfloat lightDistance = length( lVector );\n\t\tfloat angleCos = dot( directLight.direction, spotLight.direction );\n\t\tif ( angleCos > spotLight.coneCos ) {\n\t\t\tfloat spotEffect = smoothstep( spotLight.coneCos, spotLight.penumbraCos, angleCos );\n\t\t\tdirectLight.color = spotLight.color;\n\t\t\tdirectLight.color *= spotEffect * punctualLightIntensityToIrradianceFactor( lightDistance, spotLight.distance, spotLight.decay );\n\t\t\tdirectLight.visible = true;\n\t\t} else {\n\t\t\tdirectLight.color = vec3( 0.0 );\n\t\t\tdirectLight.visible = false;\n\t\t}\n\t}\n#endif\n#if NUM_RECT_AREA_LIGHTS > 0\n\tstruct RectAreaLight {\n\t\tvec3 color;\n\t\tvec3 position;\n\t\tvec3 halfWidth;\n\t\tvec3 halfHeight;\n\t};\n\tuniform sampler2D ltc_1;\tuniform sampler2D ltc_2;\n\tuniform RectAreaLight rectAreaLights[ NUM_RECT_AREA_LIGHTS ];\n#endif\n#if NUM_HEMI_LIGHTS > 0\n\tstruct HemisphereLight {\n\t\tvec3 direction;\n\t\tvec3 skyColor;\n\t\tvec3 groundColor;\n\t};\n\tuniform HemisphereLight hemisphereLights[ NUM_HEMI_LIGHTS ];\n\tvec3 getHemisphereLightIrradiance( const in HemisphereLight hemiLight, const in GeometricContext geometry ) {\n\t\tfloat dotNL = dot( geometry.normal, hemiLight.direction );\n\t\tfloat hemiDiffuseWeight = 0.5 * dotNL + 0.5;\n\t\tvec3 irradiance = mix( hemiLight.groundColor, hemiLight.skyColor, hemiDiffuseWeight );\n\t\t#ifndef PHYSICALLY_CORRECT_LIGHTS\n\t\t\tirradiance *= PI;\n\t\t#endif\n\t\treturn irradiance;\n\t}\n#endif",lights_toon_fragment:"ToonMaterial material;\nmaterial.diffuseColor = diffuseColor.rgb;\nmaterial.specularColor = specular;\nmaterial.specularShininess = shininess;\nmaterial.specularStrength = specularStrength;",lights_toon_pars_fragment:"varying vec3 vViewPosition;\n#ifndef FLAT_SHADED\n\tvarying vec3 vNormal;\n#endif\nstruct ToonMaterial {\n\tvec3\tdiffuseColor;\n\tvec3\tspecularColor;\n\tfloat\tspecularShininess;\n\tfloat\tspecularStrength;\n};\nvoid RE_Direct_Toon( const in IncidentLight directLight, const in GeometricContext geometry, const in ToonMaterial material, inout ReflectedLight reflectedLight ) {\n\tvec3 irradiance = getGradientIrradiance( geometry.normal, directLight.direction ) * directLight.color;\n\t#ifndef PHYSICALLY_CORRECT_LIGHTS\n\t\tirradiance *= PI;\n\t#endif\n\treflectedLight.directDiffuse += irradiance * BRDF_Diffuse_Lambert( material.diffuseColor );\n\treflectedLight.directSpecular += irradiance * BRDF_Specular_BlinnPhong( directLight, geometry, material.specularColor, material.specularShininess ) * material.specularStrength;\n}\nvoid RE_IndirectDiffuse_Toon( const in vec3 irradiance, const in GeometricContext geometry, const in ToonMaterial material, inout ReflectedLight reflectedLight ) {\n\treflectedLight.indirectDiffuse += irradiance * BRDF_Diffuse_Lambert( material.diffuseColor );\n}\n#define RE_Direct\t\t\t\tRE_Direct_Toon\n#define RE_IndirectDiffuse\t\tRE_IndirectDiffuse_Toon\n#define Material_LightProbeLOD( material )\t(0)",lights_phong_fragment:"BlinnPhongMaterial material;\nmaterial.diffuseColor = diffuseColor.rgb;\nmaterial.specularColor = specular;\nmaterial.specularShininess = shininess;\nmaterial.specularStrength = specularStrength;",lights_phong_pars_fragment:"varying vec3 vViewPosition;\n#ifndef FLAT_SHADED\n\tvarying vec3 vNormal;\n#endif\nstruct BlinnPhongMaterial {\n\tvec3\tdiffuseColor;\n\tvec3\tspecularColor;\n\tfloat\tspecularShininess;\n\tfloat\tspecularStrength;\n};\nvoid RE_Direct_BlinnPhong( const in IncidentLight directLight, const in GeometricContext geometry, const in BlinnPhongMaterial material, inout ReflectedLight reflectedLight ) {\n\tfloat dotNL = saturate( dot( geometry.normal, directLight.direction ) );\n\tvec3 irradiance = dotNL * directLight.color;\n\t#ifndef PHYSICALLY_CORRECT_LIGHTS\n\t\tirradiance *= PI;\n\t#endif\n\treflectedLight.directDiffuse += irradiance * BRDF_Diffuse_Lambert( material.diffuseColor );\n\treflectedLight.directSpecular += irradiance * BRDF_Specular_BlinnPhong( directLight, geometry, material.specularColor, material.specularShininess ) * material.specularStrength;\n}\nvoid RE_IndirectDiffuse_BlinnPhong( const in vec3 irradiance, const in GeometricContext geometry, const in BlinnPhongMaterial material, inout ReflectedLight reflectedLight ) {\n\treflectedLight.indirectDiffuse += irradiance * BRDF_Diffuse_Lambert( material.diffuseColor );\n}\n#define RE_Direct\t\t\t\tRE_Direct_BlinnPhong\n#define RE_IndirectDiffuse\t\tRE_IndirectDiffuse_BlinnPhong\n#define Material_LightProbeLOD( material )\t(0)",lights_physical_fragment:"PhysicalMaterial material;\nmaterial.diffuseColor = diffuseColor.rgb * ( 1.0 - metalnessFactor );\nvec3 dxy = max( abs( dFdx( geometryNormal ) ), abs( dFdy( geometryNormal ) ) );\nfloat geometryRoughness = max( max( dxy.x, dxy.y ), dxy.z );\nmaterial.specularRoughness = max( roughnessFactor, 0.0525 );material.specularRoughness += geometryRoughness;\nmaterial.specularRoughness = min( material.specularRoughness, 1.0 );\n#ifdef REFLECTIVITY\n\tmaterial.specularColor = mix( vec3( MAXIMUM_SPECULAR_COEFFICIENT * pow2( reflectivity ) ), diffuseColor.rgb, metalnessFactor );\n#else\n\tmaterial.specularColor = mix( vec3( DEFAULT_SPECULAR_COEFFICIENT ), diffuseColor.rgb, metalnessFactor );\n#endif\n#ifdef CLEARCOAT\n\tmaterial.clearcoat = clearcoat;\n\tmaterial.clearcoatRoughness = clearcoatRoughness;\n\t#ifdef USE_CLEARCOATMAP\n\t\tmaterial.clearcoat *= texture2D( clearcoatMap, vUv ).x;\n\t#endif\n\t#ifdef USE_CLEARCOAT_ROUGHNESSMAP\n\t\tmaterial.clearcoatRoughness *= texture2D( clearcoatRoughnessMap, vUv ).y;\n\t#endif\n\tmaterial.clearcoat = saturate( material.clearcoat );\tmaterial.clearcoatRoughness = max( material.clearcoatRoughness, 0.0525 );\n\tmaterial.clearcoatRoughness += geometryRoughness;\n\tmaterial.clearcoatRoughness = min( material.clearcoatRoughness, 1.0 );\n#endif\n#ifdef USE_SHEEN\n\tmaterial.sheenColor = sheen;\n#endif",lights_physical_pars_fragment:"struct PhysicalMaterial {\n\tvec3\tdiffuseColor;\n\tfloat\tspecularRoughness;\n\tvec3\tspecularColor;\n#ifdef CLEARCOAT\n\tfloat clearcoat;\n\tfloat clearcoatRoughness;\n#endif\n#ifdef USE_SHEEN\n\tvec3 sheenColor;\n#endif\n};\n#define MAXIMUM_SPECULAR_COEFFICIENT 0.16\n#define DEFAULT_SPECULAR_COEFFICIENT 0.04\nfloat clearcoatDHRApprox( const in float roughness, const in float dotNL ) {\n\treturn DEFAULT_SPECULAR_COEFFICIENT + ( 1.0 - DEFAULT_SPECULAR_COEFFICIENT ) * ( pow( 1.0 - dotNL, 5.0 ) * pow( 1.0 - roughness, 2.0 ) );\n}\n#if NUM_RECT_AREA_LIGHTS > 0\n\tvoid RE_Direct_RectArea_Physical( const in RectAreaLight rectAreaLight, const in GeometricContext geometry, const in PhysicalMaterial material, inout ReflectedLight reflectedLight ) {\n\t\tvec3 normal = geometry.normal;\n\t\tvec3 viewDir = geometry.viewDir;\n\t\tvec3 position = geometry.position;\n\t\tvec3 lightPos = rectAreaLight.position;\n\t\tvec3 halfWidth = rectAreaLight.halfWidth;\n\t\tvec3 halfHeight = rectAreaLight.halfHeight;\n\t\tvec3 lightColor = rectAreaLight.color;\n\t\tfloat roughness = material.specularRoughness;\n\t\tvec3 rectCoords[ 4 ];\n\t\trectCoords[ 0 ] = lightPos + halfWidth - halfHeight;\t\trectCoords[ 1 ] = lightPos - halfWidth - halfHeight;\n\t\trectCoords[ 2 ] = lightPos - halfWidth + halfHeight;\n\t\trectCoords[ 3 ] = lightPos + halfWidth + halfHeight;\n\t\tvec2 uv = LTC_Uv( normal, viewDir, roughness );\n\t\tvec4 t1 = texture2D( ltc_1, uv );\n\t\tvec4 t2 = texture2D( ltc_2, uv );\n\t\tmat3 mInv = mat3(\n\t\t\tvec3( t1.x, 0, t1.y ),\n\t\t\tvec3(    0, 1,    0 ),\n\t\t\tvec3( t1.z, 0, t1.w )\n\t\t);\n\t\tvec3 fresnel = ( material.specularColor * t2.x + ( vec3( 1.0 ) - material.specularColor ) * t2.y );\n\t\treflectedLight.directSpecular += lightColor * fresnel * LTC_Evaluate( normal, viewDir, position, mInv, rectCoords );\n\t\treflectedLight.directDiffuse += lightColor * material.diffuseColor * LTC_Evaluate( normal, viewDir, position, mat3( 1.0 ), rectCoords );\n\t}\n#endif\nvoid RE_Direct_Physical( const in IncidentLight directLight, const in GeometricContext geometry, const in PhysicalMaterial material, inout ReflectedLight reflectedLight ) {\n\tfloat dotNL = saturate( dot( geometry.normal, directLight.direction ) );\n\tvec3 irradiance = dotNL * directLight.color;\n\t#ifndef PHYSICALLY_CORRECT_LIGHTS\n\t\tirradiance *= PI;\n\t#endif\n\t#ifdef CLEARCOAT\n\t\tfloat ccDotNL = saturate( dot( geometry.clearcoatNormal, directLight.direction ) );\n\t\tvec3 ccIrradiance = ccDotNL * directLight.color;\n\t\t#ifndef PHYSICALLY_CORRECT_LIGHTS\n\t\t\tccIrradiance *= PI;\n\t\t#endif\n\t\tfloat clearcoatDHR = material.clearcoat * clearcoatDHRApprox( material.clearcoatRoughness, ccDotNL );\n\t\treflectedLight.directSpecular += ccIrradiance * material.clearcoat * BRDF_Specular_GGX( directLight, geometry.viewDir, geometry.clearcoatNormal, vec3( DEFAULT_SPECULAR_COEFFICIENT ), material.clearcoatRoughness );\n\t#else\n\t\tfloat clearcoatDHR = 0.0;\n\t#endif\n\t#ifdef USE_SHEEN\n\t\treflectedLight.directSpecular += ( 1.0 - clearcoatDHR ) * irradiance * BRDF_Specular_Sheen(\n\t\t\tmaterial.specularRoughness,\n\t\t\tdirectLight.direction,\n\t\t\tgeometry,\n\t\t\tmaterial.sheenColor\n\t\t);\n\t#else\n\t\treflectedLight.directSpecular += ( 1.0 - clearcoatDHR ) * irradiance * BRDF_Specular_GGX( directLight, geometry.viewDir, geometry.normal, material.specularColor, material.specularRoughness);\n\t#endif\n\treflectedLight.directDiffuse += ( 1.0 - clearcoatDHR ) * irradiance * BRDF_Diffuse_Lambert( material.diffuseColor );\n}\nvoid RE_IndirectDiffuse_Physical( const in vec3 irradiance, const in GeometricContext geometry, const in PhysicalMaterial material, inout ReflectedLight reflectedLight ) {\n\treflectedLight.indirectDiffuse += irradiance * BRDF_Diffuse_Lambert( material.diffuseColor );\n}\nvoid RE_IndirectSpecular_Physical( const in vec3 radiance, const in vec3 irradiance, const in vec3 clearcoatRadiance, const in GeometricContext geometry, const in PhysicalMaterial material, inout ReflectedLight reflectedLight) {\n\t#ifdef CLEARCOAT\n\t\tfloat ccDotNV = saturate( dot( geometry.clearcoatNormal, geometry.viewDir ) );\n\t\treflectedLight.indirectSpecular += clearcoatRadiance * material.clearcoat * BRDF_Specular_GGX_Environment( geometry.viewDir, geometry.clearcoatNormal, vec3( DEFAULT_SPECULAR_COEFFICIENT ), material.clearcoatRoughness );\n\t\tfloat ccDotNL = ccDotNV;\n\t\tfloat clearcoatDHR = material.clearcoat * clearcoatDHRApprox( material.clearcoatRoughness, ccDotNL );\n\t#else\n\t\tfloat clearcoatDHR = 0.0;\n\t#endif\n\tfloat clearcoatInv = 1.0 - clearcoatDHR;\n\tvec3 singleScattering = vec3( 0.0 );\n\tvec3 multiScattering = vec3( 0.0 );\n\tvec3 cosineWeightedIrradiance = irradiance * RECIPROCAL_PI;\n\tBRDF_Specular_Multiscattering_Environment( geometry, material.specularColor, material.specularRoughness, singleScattering, multiScattering );\n\tvec3 diffuse = material.diffuseColor * ( 1.0 - ( singleScattering + multiScattering ) );\n\treflectedLight.indirectSpecular += clearcoatInv * radiance * singleScattering;\n\treflectedLight.indirectSpecular += multiScattering * cosineWeightedIrradiance;\n\treflectedLight.indirectDiffuse += diffuse * cosineWeightedIrradiance;\n}\n#define RE_Direct\t\t\t\tRE_Direct_Physical\n#define RE_Direct_RectArea\t\tRE_Direct_RectArea_Physical\n#define RE_IndirectDiffuse\t\tRE_IndirectDiffuse_Physical\n#define RE_IndirectSpecular\t\tRE_IndirectSpecular_Physical\nfloat computeSpecularOcclusion( const in float dotNV, const in float ambientOcclusion, const in float roughness ) {\n\treturn saturate( pow( dotNV + ambientOcclusion, exp2( - 16.0 * roughness - 1.0 ) ) - 1.0 + ambientOcclusion );\n}",lights_fragment_begin:"\nGeometricContext geometry;\ngeometry.position = - vViewPosition;\ngeometry.normal = normal;\ngeometry.viewDir = ( isOrthographic ) ? vec3( 0, 0, 1 ) : normalize( vViewPosition );\n#ifdef CLEARCOAT\n\tgeometry.clearcoatNormal = clearcoatNormal;\n#endif\nIncidentLight directLight;\n#if ( NUM_POINT_LIGHTS > 0 ) && defined( RE_Direct )\n\tPointLight pointLight;\n\t#if defined( USE_SHADOWMAP ) && NUM_POINT_LIGHT_SHADOWS > 0\n\tPointLightShadow pointLightShadow;\n\t#endif\n\t#pragma unroll_loop_start\n\tfor ( int i = 0; i < NUM_POINT_LIGHTS; i ++ ) {\n\t\tpointLight = pointLights[ i ];\n\t\tgetPointDirectLightIrradiance( pointLight, geometry, directLight );\n\t\t#if defined( USE_SHADOWMAP ) && ( UNROLLED_LOOP_INDEX < NUM_POINT_LIGHT_SHADOWS )\n\t\tpointLightShadow = pointLightShadows[ i ];\n\t\tdirectLight.color *= all( bvec2( directLight.visible, receiveShadow ) ) ? getPointShadow( pointShadowMap[ i ], pointLightShadow.shadowMapSize, pointLightShadow.shadowBias, pointLightShadow.shadowRadius, vPointShadowCoord[ i ], pointLightShadow.shadowCameraNear, pointLightShadow.shadowCameraFar ) : 1.0;\n\t\t#endif\n\t\tRE_Direct( directLight, geometry, material, reflectedLight );\n\t}\n\t#pragma unroll_loop_end\n#endif\n#if ( NUM_SPOT_LIGHTS > 0 ) && defined( RE_Direct )\n\tSpotLight spotLight;\n\t#if defined( USE_SHADOWMAP ) && NUM_SPOT_LIGHT_SHADOWS > 0\n\tSpotLightShadow spotLightShadow;\n\t#endif\n\t#pragma unroll_loop_start\n\tfor ( int i = 0; i < NUM_SPOT_LIGHTS; i ++ ) {\n\t\tspotLight = spotLights[ i ];\n\t\tgetSpotDirectLightIrradiance( spotLight, geometry, directLight );\n\t\t#if defined( USE_SHADOWMAP ) && ( UNROLLED_LOOP_INDEX < NUM_SPOT_LIGHT_SHADOWS )\n\t\tspotLightShadow = spotLightShadows[ i ];\n\t\tdirectLight.color *= all( bvec2( directLight.visible, receiveShadow ) ) ? getShadow( spotShadowMap[ i ], spotLightShadow.shadowMapSize, spotLightShadow.shadowBias, spotLightShadow.shadowRadius, vSpotShadowCoord[ i ] ) : 1.0;\n\t\t#endif\n\t\tRE_Direct( directLight, geometry, material, reflectedLight );\n\t}\n\t#pragma unroll_loop_end\n#endif\n#if ( NUM_DIR_LIGHTS > 0 ) && defined( RE_Direct )\n\tDirectionalLight directionalLight;\n\t#if defined( USE_SHADOWMAP ) && NUM_DIR_LIGHT_SHADOWS > 0\n\tDirectionalLightShadow directionalLightShadow;\n\t#endif\n\t#pragma unroll_loop_start\n\tfor ( int i = 0; i < NUM_DIR_LIGHTS; i ++ ) {\n\t\tdirectionalLight = directionalLights[ i ];\n\t\tgetDirectionalDirectLightIrradiance( directionalLight, geometry, directLight );\n\t\t#if defined( USE_SHADOWMAP ) && ( UNROLLED_LOOP_INDEX < NUM_DIR_LIGHT_SHADOWS )\n\t\tdirectionalLightShadow = directionalLightShadows[ i ];\n\t\tdirectLight.color *= all( bvec2( directLight.visible, receiveShadow ) ) ? getShadow( directionalShadowMap[ i ], directionalLightShadow.shadowMapSize, directionalLightShadow.shadowBias, directionalLightShadow.shadowRadius, vDirectionalShadowCoord[ i ] ) : 1.0;\n\t\t#endif\n\t\tRE_Direct( directLight, geometry, material, reflectedLight );\n\t}\n\t#pragma unroll_loop_end\n#endif\n#if ( NUM_RECT_AREA_LIGHTS > 0 ) && defined( RE_Direct_RectArea )\n\tRectAreaLight rectAreaLight;\n\t#pragma unroll_loop_start\n\tfor ( int i = 0; i < NUM_RECT_AREA_LIGHTS; i ++ ) {\n\t\trectAreaLight = rectAreaLights[ i ];\n\t\tRE_Direct_RectArea( rectAreaLight, geometry, material, reflectedLight );\n\t}\n\t#pragma unroll_loop_end\n#endif\n#if defined( RE_IndirectDiffuse )\n\tvec3 iblIrradiance = vec3( 0.0 );\n\tvec3 irradiance = getAmbientLightIrradiance( ambientLightColor );\n\tirradiance += getLightProbeIrradiance( lightProbe, geometry );\n\t#if ( NUM_HEMI_LIGHTS > 0 )\n\t\t#pragma unroll_loop_start\n\t\tfor ( int i = 0; i < NUM_HEMI_LIGHTS; i ++ ) {\n\t\t\tirradiance += getHemisphereLightIrradiance( hemisphereLights[ i ], geometry );\n\t\t}\n\t\t#pragma unroll_loop_end\n\t#endif\n#endif\n#if defined( RE_IndirectSpecular )\n\tvec3 radiance = vec3( 0.0 );\n\tvec3 clearcoatRadiance = vec3( 0.0 );\n#endif",lights_fragment_maps:"#if defined( RE_IndirectDiffuse )\n\t#ifdef USE_LIGHTMAP\n\t\tvec4 lightMapTexel= texture2D( lightMap, vUv2 );\n\t\tvec3 lightMapIrradiance = lightMapTexelToLinear( lightMapTexel ).rgb * lightMapIntensity;\n\t\t#ifndef PHYSICALLY_CORRECT_LIGHTS\n\t\t\tlightMapIrradiance *= PI;\n\t\t#endif\n\t\tirradiance += lightMapIrradiance;\n\t#endif\n\t#if defined( USE_ENVMAP ) && defined( STANDARD ) && defined( ENVMAP_TYPE_CUBE_UV )\n\t\tiblIrradiance += getLightProbeIndirectIrradiance( geometry, maxMipLevel );\n\t#endif\n#endif\n#if defined( USE_ENVMAP ) && defined( RE_IndirectSpecular )\n\tradiance += getLightProbeIndirectRadiance( geometry.viewDir, geometry.normal, material.specularRoughness, maxMipLevel );\n\t#ifdef CLEARCOAT\n\t\tclearcoatRadiance += getLightProbeIndirectRadiance( geometry.viewDir, geometry.clearcoatNormal, material.clearcoatRoughness, maxMipLevel );\n\t#endif\n#endif",lights_fragment_end:"#if defined( RE_IndirectDiffuse )\n\tRE_IndirectDiffuse( irradiance, geometry, material, reflectedLight );\n#endif\n#if defined( RE_IndirectSpecular )\n\tRE_IndirectSpecular( radiance, iblIrradiance, clearcoatRadiance, geometry, material, reflectedLight );\n#endif",logdepthbuf_fragment:"#if defined( USE_LOGDEPTHBUF ) && defined( USE_LOGDEPTHBUF_EXT )\n\tgl_FragDepthEXT = vIsPerspective == 0.0 ? gl_FragCoord.z : log2( vFragDepth ) * logDepthBufFC * 0.5;\n#endif",logdepthbuf_pars_fragment:"#if defined( USE_LOGDEPTHBUF ) && defined( USE_LOGDEPTHBUF_EXT )\n\tuniform float logDepthBufFC;\n\tvarying float vFragDepth;\n\tvarying float vIsPerspective;\n#endif",logdepthbuf_pars_vertex:"#ifdef USE_LOGDEPTHBUF\n\t#ifdef USE_LOGDEPTHBUF_EXT\n\t\tvarying float vFragDepth;\n\t\tvarying float vIsPerspective;\n\t#else\n\t\tuniform float logDepthBufFC;\n\t#endif\n#endif",logdepthbuf_vertex:"#ifdef USE_LOGDEPTHBUF\n\t#ifdef USE_LOGDEPTHBUF_EXT\n\t\tvFragDepth = 1.0 + gl_Position.w;\n\t\tvIsPerspective = float( isPerspectiveMatrix( projectionMatrix ) );\n\t#else\n\t\tif ( isPerspectiveMatrix( projectionMatrix ) ) {\n\t\t\tgl_Position.z = log2( max( EPSILON, gl_Position.w + 1.0 ) ) * logDepthBufFC - 1.0;\n\t\t\tgl_Position.z *= gl_Position.w;\n\t\t}\n\t#endif\n#endif",map_fragment:"#ifdef USE_MAP\n\tvec4 texelColor = texture2D( map, vUv );\n\ttexelColor = mapTexelToLinear( texelColor );\n\tdiffuseColor *= texelColor;\n#endif",map_pars_fragment:"#ifdef USE_MAP\n\tuniform sampler2D map;\n#endif",map_particle_fragment:"#if defined( USE_MAP ) || defined( USE_ALPHAMAP )\n\tvec2 uv = ( uvTransform * vec3( gl_PointCoord.x, 1.0 - gl_PointCoord.y, 1 ) ).xy;\n#endif\n#ifdef USE_MAP\n\tvec4 mapTexel = texture2D( map, uv );\n\tdiffuseColor *= mapTexelToLinear( mapTexel );\n#endif\n#ifdef USE_ALPHAMAP\n\tdiffuseColor.a *= texture2D( alphaMap, uv ).g;\n#endif",map_particle_pars_fragment:"#if defined( USE_MAP ) || defined( USE_ALPHAMAP )\n\tuniform mat3 uvTransform;\n#endif\n#ifdef USE_MAP\n\tuniform sampler2D map;\n#endif\n#ifdef USE_ALPHAMAP\n\tuniform sampler2D alphaMap;\n#endif",metalnessmap_fragment:"float metalnessFactor = metalness;\n#ifdef USE_METALNESSMAP\n\tvec4 texelMetalness = texture2D( metalnessMap, vUv );\n\tmetalnessFactor *= texelMetalness.b;\n#endif",metalnessmap_pars_fragment:"#ifdef USE_METALNESSMAP\n\tuniform sampler2D metalnessMap;\n#endif",morphnormal_vertex:"#ifdef USE_MORPHNORMALS\n\tobjectNormal *= morphTargetBaseInfluence;\n\tobjectNormal += morphNormal0 * morphTargetInfluences[ 0 ];\n\tobjectNormal += morphNormal1 * morphTargetInfluences[ 1 ];\n\tobjectNormal += morphNormal2 * morphTargetInfluences[ 2 ];\n\tobjectNormal += morphNormal3 * morphTargetInfluences[ 3 ];\n#endif",morphtarget_pars_vertex:"#ifdef USE_MORPHTARGETS\n\tuniform float morphTargetBaseInfluence;\n\t#ifndef USE_MORPHNORMALS\n\tuniform float morphTargetInfluences[ 8 ];\n\t#else\n\tuniform float morphTargetInfluences[ 4 ];\n\t#endif\n#endif",morphtarget_vertex:"#ifdef USE_MORPHTARGETS\n\ttransformed *= morphTargetBaseInfluence;\n\ttransformed += morphTarget0 * morphTargetInfluences[ 0 ];\n\ttransformed += morphTarget1 * morphTargetInfluences[ 1 ];\n\ttransformed += morphTarget2 * morphTargetInfluences[ 2 ];\n\ttransformed += morphTarget3 * morphTargetInfluences[ 3 ];\n\t#ifndef USE_MORPHNORMALS\n\ttransformed += morphTarget4 * morphTargetInfluences[ 4 ];\n\ttransformed += morphTarget5 * morphTargetInfluences[ 5 ];\n\ttransformed += morphTarget6 * morphTargetInfluences[ 6 ];\n\ttransformed += morphTarget7 * morphTargetInfluences[ 7 ];\n\t#endif\n#endif",normal_fragment_begin:"#ifdef FLAT_SHADED\n\tvec3 fdx = vec3( dFdx( vViewPosition.x ), dFdx( vViewPosition.y ), dFdx( vViewPosition.z ) );\n\tvec3 fdy = vec3( dFdy( vViewPosition.x ), dFdy( vViewPosition.y ), dFdy( vViewPosition.z ) );\n\tvec3 normal = normalize( cross( fdx, fdy ) );\n#else\n\tvec3 normal = normalize( vNormal );\n\t#ifdef DOUBLE_SIDED\n\t\tnormal = normal * ( float( gl_FrontFacing ) * 2.0 - 1.0 );\n\t#endif\n\t#ifdef USE_TANGENT\n\t\tvec3 tangent = normalize( vTangent );\n\t\tvec3 bitangent = normalize( vBitangent );\n\t\t#ifdef DOUBLE_SIDED\n\t\t\ttangent = tangent * ( float( gl_FrontFacing ) * 2.0 - 1.0 );\n\t\t\tbitangent = bitangent * ( float( gl_FrontFacing ) * 2.0 - 1.0 );\n\t\t#endif\n\t\t#if defined( TANGENTSPACE_NORMALMAP ) || defined( USE_CLEARCOAT_NORMALMAP )\n\t\t\tmat3 vTBN = mat3( tangent, bitangent, normal );\n\t\t#endif\n\t#endif\n#endif\nvec3 geometryNormal = normal;",normal_fragment_maps:"#ifdef OBJECTSPACE_NORMALMAP\n\tnormal = texture2D( normalMap, vUv ).xyz * 2.0 - 1.0;\n\t#ifdef FLIP_SIDED\n\t\tnormal = - normal;\n\t#endif\n\t#ifdef DOUBLE_SIDED\n\t\tnormal = normal * ( float( gl_FrontFacing ) * 2.0 - 1.0 );\n\t#endif\n\tnormal = normalize( normalMatrix * normal );\n#elif defined( TANGENTSPACE_NORMALMAP )\n\tvec3 mapN = texture2D( normalMap, vUv ).xyz * 2.0 - 1.0;\n\tmapN.xy *= normalScale;\n\t#ifdef USE_TANGENT\n\t\tnormal = normalize( vTBN * mapN );\n\t#else\n\t\tnormal = perturbNormal2Arb( -vViewPosition, normal, mapN );\n\t#endif\n#elif defined( USE_BUMPMAP )\n\tnormal = perturbNormalArb( -vViewPosition, normal, dHdxy_fwd() );\n#endif",normalmap_pars_fragment:"#ifdef USE_NORMALMAP\n\tuniform sampler2D normalMap;\n\tuniform vec2 normalScale;\n#endif\n#ifdef OBJECTSPACE_NORMALMAP\n\tuniform mat3 normalMatrix;\n#endif\n#if ! defined ( USE_TANGENT ) && ( defined ( TANGENTSPACE_NORMALMAP ) || defined ( USE_CLEARCOAT_NORMALMAP ) )\n\tvec3 perturbNormal2Arb( vec3 eye_pos, vec3 surf_norm, vec3 mapN ) {\n\t\tvec3 q0 = vec3( dFdx( eye_pos.x ), dFdx( eye_pos.y ), dFdx( eye_pos.z ) );\n\t\tvec3 q1 = vec3( dFdy( eye_pos.x ), dFdy( eye_pos.y ), dFdy( eye_pos.z ) );\n\t\tvec2 st0 = dFdx( vUv.st );\n\t\tvec2 st1 = dFdy( vUv.st );\n\t\tfloat scale = sign( st1.t * st0.s - st0.t * st1.s );\n\t\tvec3 S = normalize( ( q0 * st1.t - q1 * st0.t ) * scale );\n\t\tvec3 T = normalize( ( - q0 * st1.s + q1 * st0.s ) * scale );\n\t\tvec3 N = normalize( surf_norm );\n\t\tmat3 tsn = mat3( S, T, N );\n\t\tmapN.xy *= ( float( gl_FrontFacing ) * 2.0 - 1.0 );\n\t\treturn normalize( tsn * mapN );\n\t}\n#endif",clearcoat_normal_fragment_begin:"#ifdef CLEARCOAT\n\tvec3 clearcoatNormal = geometryNormal;\n#endif",clearcoat_normal_fragment_maps:"#ifdef USE_CLEARCOAT_NORMALMAP\n\tvec3 clearcoatMapN = texture2D( clearcoatNormalMap, vUv ).xyz * 2.0 - 1.0;\n\tclearcoatMapN.xy *= clearcoatNormalScale;\n\t#ifdef USE_TANGENT\n\t\tclearcoatNormal = normalize( vTBN * clearcoatMapN );\n\t#else\n\t\tclearcoatNormal = perturbNormal2Arb( - vViewPosition, clearcoatNormal, clearcoatMapN );\n\t#endif\n#endif",clearcoat_pars_fragment:"#ifdef USE_CLEARCOATMAP\n\tuniform sampler2D clearcoatMap;\n#endif\n#ifdef USE_CLEARCOAT_ROUGHNESSMAP\n\tuniform sampler2D clearcoatRoughnessMap;\n#endif\n#ifdef USE_CLEARCOAT_NORMALMAP\n\tuniform sampler2D clearcoatNormalMap;\n\tuniform vec2 clearcoatNormalScale;\n#endif",packing:"vec3 packNormalToRGB( const in vec3 normal ) {\n\treturn normalize( normal ) * 0.5 + 0.5;\n}\nvec3 unpackRGBToNormal( const in vec3 rgb ) {\n\treturn 2.0 * rgb.xyz - 1.0;\n}\nconst float PackUpscale = 256. / 255.;const float UnpackDownscale = 255. / 256.;\nconst vec3 PackFactors = vec3( 256. * 256. * 256., 256. * 256.,  256. );\nconst vec4 UnpackFactors = UnpackDownscale / vec4( PackFactors, 1. );\nconst float ShiftRight8 = 1. / 256.;\nvec4 packDepthToRGBA( const in float v ) {\n\tvec4 r = vec4( fract( v * PackFactors ), v );\n\tr.yzw -= r.xyz * ShiftRight8;\treturn r * PackUpscale;\n}\nfloat unpackRGBAToDepth( const in vec4 v ) {\n\treturn dot( v, UnpackFactors );\n}\nvec4 pack2HalfToRGBA( vec2 v ) {\n\tvec4 r = vec4( v.x, fract( v.x * 255.0 ), v.y, fract( v.y * 255.0 ));\n\treturn vec4( r.x - r.y / 255.0, r.y, r.z - r.w / 255.0, r.w);\n}\nvec2 unpackRGBATo2Half( vec4 v ) {\n\treturn vec2( v.x + ( v.y / 255.0 ), v.z + ( v.w / 255.0 ) );\n}\nfloat viewZToOrthographicDepth( const in float viewZ, const in float near, const in float far ) {\n\treturn ( viewZ + near ) / ( near - far );\n}\nfloat orthographicDepthToViewZ( const in float linearClipZ, const in float near, const in float far ) {\n\treturn linearClipZ * ( near - far ) - near;\n}\nfloat viewZToPerspectiveDepth( const in float viewZ, const in float near, const in float far ) {\n\treturn (( near + viewZ ) * far ) / (( far - near ) * viewZ );\n}\nfloat perspectiveDepthToViewZ( const in float invClipZ, const in float near, const in float far ) {\n\treturn ( near * far ) / ( ( far - near ) * invClipZ - far );\n}",premultiplied_alpha_fragment:"#ifdef PREMULTIPLIED_ALPHA\n\tgl_FragColor.rgb *= gl_FragColor.a;\n#endif",project_vertex:"vec4 mvPosition = vec4( transformed, 1.0 );\n#ifdef USE_INSTANCING\n\tmvPosition = instanceMatrix * mvPosition;\n#endif\nmvPosition = modelViewMatrix * mvPosition;\ngl_Position = projectionMatrix * mvPosition;",dithering_fragment:"#ifdef DITHERING\n\tgl_FragColor.rgb = dithering( gl_FragColor.rgb );\n#endif",dithering_pars_fragment:"#ifdef DITHERING\n\tvec3 dithering( vec3 color ) {\n\t\tfloat grid_position = rand( gl_FragCoord.xy );\n\t\tvec3 dither_shift_RGB = vec3( 0.25 / 255.0, -0.25 / 255.0, 0.25 / 255.0 );\n\t\tdither_shift_RGB = mix( 2.0 * dither_shift_RGB, -2.0 * dither_shift_RGB, grid_position );\n\t\treturn color + dither_shift_RGB;\n\t}\n#endif",roughnessmap_fragment:"float roughnessFactor = roughness;\n#ifdef USE_ROUGHNESSMAP\n\tvec4 texelRoughness = texture2D( roughnessMap, vUv );\n\troughnessFactor *= texelRoughness.g;\n#endif",roughnessmap_pars_fragment:"#ifdef USE_ROUGHNESSMAP\n\tuniform sampler2D roughnessMap;\n#endif",shadowmap_pars_fragment:"#ifdef USE_SHADOWMAP\n\t#if NUM_DIR_LIGHT_SHADOWS > 0\n\t\tuniform sampler2D directionalShadowMap[ NUM_DIR_LIGHT_SHADOWS ];\n\t\tvarying vec4 vDirectionalShadowCoord[ NUM_DIR_LIGHT_SHADOWS ];\n\t#endif\n\t#if NUM_SPOT_LIGHT_SHADOWS > 0\n\t\tuniform sampler2D spotShadowMap[ NUM_SPOT_LIGHT_SHADOWS ];\n\t\tvarying vec4 vSpotShadowCoord[ NUM_SPOT_LIGHT_SHADOWS ];\n\t#endif\n\t#if NUM_POINT_LIGHT_SHADOWS > 0\n\t\tuniform sampler2D pointShadowMap[ NUM_POINT_LIGHT_SHADOWS ];\n\t\tvarying vec4 vPointShadowCoord[ NUM_POINT_LIGHT_SHADOWS ];\n\t#endif\n\tfloat texture2DCompare( sampler2D depths, vec2 uv, float compare ) {\n\t\treturn step( compare, unpackRGBAToDepth( texture2D( depths, uv ) ) );\n\t}\n\tvec2 texture2DDistribution( sampler2D shadow, vec2 uv ) {\n\t\treturn unpackRGBATo2Half( texture2D( shadow, uv ) );\n\t}\n\tfloat VSMShadow (sampler2D shadow, vec2 uv, float compare ){\n\t\tfloat occlusion = 1.0;\n\t\tvec2 distribution = texture2DDistribution( shadow, uv );\n\t\tfloat hard_shadow = step( compare , distribution.x );\n\t\tif (hard_shadow != 1.0 ) {\n\t\t\tfloat distance = compare - distribution.x ;\n\t\t\tfloat variance = max( 0.00000, distribution.y * distribution.y );\n\t\t\tfloat softness_probability = variance / (variance + distance * distance );\t\t\tsoftness_probability = clamp( ( softness_probability - 0.3 ) / ( 0.95 - 0.3 ), 0.0, 1.0 );\t\t\tocclusion = clamp( max( hard_shadow, softness_probability ), 0.0, 1.0 );\n\t\t}\n\t\treturn occlusion;\n\t}\n\tfloat getShadow( sampler2D shadowMap, vec2 shadowMapSize, float shadowBias, float shadowRadius, vec4 shadowCoord ) {\n\t\tfloat shadow = 1.0;\n\t\tshadowCoord.xyz /= shadowCoord.w;\n\t\tshadowCoord.z += shadowBias;\n\t\tbvec4 inFrustumVec = bvec4 ( shadowCoord.x >= 0.0, shadowCoord.x <= 1.0, shadowCoord.y >= 0.0, shadowCoord.y <= 1.0 );\n\t\tbool inFrustum = all( inFrustumVec );\n\t\tbvec2 frustumTestVec = bvec2( inFrustum, shadowCoord.z <= 1.0 );\n\t\tbool frustumTest = all( frustumTestVec );\n\t\tif ( frustumTest ) {\n\t\t#if defined( SHADOWMAP_TYPE_PCF )\n\t\t\tvec2 texelSize = vec2( 1.0 ) / shadowMapSize;\n\t\t\tfloat dx0 = - texelSize.x * shadowRadius;\n\t\t\tfloat dy0 = - texelSize.y * shadowRadius;\n\t\t\tfloat dx1 = + texelSize.x * shadowRadius;\n\t\t\tfloat dy1 = + texelSize.y * shadowRadius;\n\t\t\tfloat dx2 = dx0 / 2.0;\n\t\t\tfloat dy2 = dy0 / 2.0;\n\t\t\tfloat dx3 = dx1 / 2.0;\n\t\t\tfloat dy3 = dy1 / 2.0;\n\t\t\tshadow = (\n\t\t\t\ttexture2DCompare( shadowMap, shadowCoord.xy + vec2( dx0, dy0 ), shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, shadowCoord.xy + vec2( 0.0, dy0 ), shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, shadowCoord.xy + vec2( dx1, dy0 ), shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, shadowCoord.xy + vec2( dx2, dy2 ), shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, shadowCoord.xy + vec2( 0.0, dy2 ), shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, shadowCoord.xy + vec2( dx3, dy2 ), shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, shadowCoord.xy + vec2( dx0, 0.0 ), shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, shadowCoord.xy + vec2( dx2, 0.0 ), shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, shadowCoord.xy, shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, shadowCoord.xy + vec2( dx3, 0.0 ), shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, shadowCoord.xy + vec2( dx1, 0.0 ), shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, shadowCoord.xy + vec2( dx2, dy3 ), shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, shadowCoord.xy + vec2( 0.0, dy3 ), shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, shadowCoord.xy + vec2( dx3, dy3 ), shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, shadowCoord.xy + vec2( dx0, dy1 ), shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, shadowCoord.xy + vec2( 0.0, dy1 ), shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, shadowCoord.xy + vec2( dx1, dy1 ), shadowCoord.z )\n\t\t\t) * ( 1.0 / 17.0 );\n\t\t#elif defined( SHADOWMAP_TYPE_PCF_SOFT )\n\t\t\tvec2 texelSize = vec2( 1.0 ) / shadowMapSize;\n\t\t\tfloat dx = texelSize.x;\n\t\t\tfloat dy = texelSize.y;\n\t\t\tvec2 uv = shadowCoord.xy;\n\t\t\tvec2 f = fract( uv * shadowMapSize + 0.5 );\n\t\t\tuv -= f * texelSize;\n\t\t\tshadow = (\n\t\t\t\ttexture2DCompare( shadowMap, uv, shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, uv + vec2( dx, 0.0 ), shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, uv + vec2( 0.0, dy ), shadowCoord.z ) +\n\t\t\t\ttexture2DCompare( shadowMap, uv + texelSize, shadowCoord.z ) +\n\t\t\t\tmix( texture2DCompare( shadowMap, uv + vec2( -dx, 0.0 ), shadowCoord.z ), \n\t\t\t\t\t texture2DCompare( shadowMap, uv + vec2( 2.0 * dx, 0.0 ), shadowCoord.z ),\n\t\t\t\t\t f.x ) +\n\t\t\t\tmix( texture2DCompare( shadowMap, uv + vec2( -dx, dy ), shadowCoord.z ), \n\t\t\t\t\t texture2DCompare( shadowMap, uv + vec2( 2.0 * dx, dy ), shadowCoord.z ),\n\t\t\t\t\t f.x ) +\n\t\t\t\tmix( texture2DCompare( shadowMap, uv + vec2( 0.0, -dy ), shadowCoord.z ), \n\t\t\t\t\t texture2DCompare( shadowMap, uv + vec2( 0.0, 2.0 * dy ), shadowCoord.z ),\n\t\t\t\t\t f.y ) +\n\t\t\t\tmix( texture2DCompare( shadowMap, uv + vec2( dx, -dy ), shadowCoord.z ), \n\t\t\t\t\t texture2DCompare( shadowMap, uv + vec2( dx, 2.0 * dy ), shadowCoord.z ),\n\t\t\t\t\t f.y ) +\n\t\t\t\tmix( mix( texture2DCompare( shadowMap, uv + vec2( -dx, -dy ), shadowCoord.z ), \n\t\t\t\t\t\t  texture2DCompare( shadowMap, uv + vec2( 2.0 * dx, -dy ), shadowCoord.z ),\n\t\t\t\t\t\t  f.x ),\n\t\t\t\t\t mix( texture2DCompare( shadowMap, uv + vec2( -dx, 2.0 * dy ), shadowCoord.z ), \n\t\t\t\t\t\t  texture2DCompare( shadowMap, uv + vec2( 2.0 * dx, 2.0 * dy ), shadowCoord.z ),\n\t\t\t\t\t\t  f.x ),\n\t\t\t\t\t f.y )\n\t\t\t) * ( 1.0 / 9.0 );\n\t\t#elif defined( SHADOWMAP_TYPE_VSM )\n\t\t\tshadow = VSMShadow( shadowMap, shadowCoord.xy, shadowCoord.z );\n\t\t#else\n\t\t\tshadow = texture2DCompare( shadowMap, shadowCoord.xy, shadowCoord.z );\n\t\t#endif\n\t\t}\n\t\treturn shadow;\n\t}\n\tvec2 cubeToUV( vec3 v, float texelSizeY ) {\n\t\tvec3 absV = abs( v );\n\t\tfloat scaleToCube = 1.0 / max( absV.x, max( absV.y, absV.z ) );\n\t\tabsV *= scaleToCube;\n\t\tv *= scaleToCube * ( 1.0 - 2.0 * texelSizeY );\n\t\tvec2 planar = v.xy;\n\t\tfloat almostATexel = 1.5 * texelSizeY;\n\t\tfloat almostOne = 1.0 - almostATexel;\n\t\tif ( absV.z >= almostOne ) {\n\t\t\tif ( v.z > 0.0 )\n\t\t\t\tplanar.x = 4.0 - v.x;\n\t\t} else if ( absV.x >= almostOne ) {\n\t\t\tfloat signX = sign( v.x );\n\t\t\tplanar.x = v.z * signX + 2.0 * signX;\n\t\t} else if ( absV.y >= almostOne ) {\n\t\t\tfloat signY = sign( v.y );\n\t\t\tplanar.x = v.x + 2.0 * signY + 2.0;\n\t\t\tplanar.y = v.z * signY - 2.0;\n\t\t}\n\t\treturn vec2( 0.125, 0.25 ) * planar + vec2( 0.375, 0.75 );\n\t}\n\tfloat getPointShadow( sampler2D shadowMap, vec2 shadowMapSize, float shadowBias, float shadowRadius, vec4 shadowCoord, float shadowCameraNear, float shadowCameraFar ) {\n\t\tvec2 texelSize = vec2( 1.0 ) / ( shadowMapSize * vec2( 4.0, 2.0 ) );\n\t\tvec3 lightToPosition = shadowCoord.xyz;\n\t\tfloat dp = ( length( lightToPosition ) - shadowCameraNear ) / ( shadowCameraFar - shadowCameraNear );\t\tdp += shadowBias;\n\t\tvec3 bd3D = normalize( lightToPosition );\n\t\t#if defined( SHADOWMAP_TYPE_PCF ) || defined( SHADOWMAP_TYPE_PCF_SOFT ) || defined( SHADOWMAP_TYPE_VSM )\n\t\t\tvec2 offset = vec2( - 1, 1 ) * shadowRadius * texelSize.y;\n\t\t\treturn (\n\t\t\t\ttexture2DCompare( shadowMap, cubeToUV( bd3D + offset.xyy, texelSize.y ), dp ) +\n\t\t\t\ttexture2DCompare( shadowMap, cubeToUV( bd3D + offset.yyy, texelSize.y ), dp ) +\n\t\t\t\ttexture2DCompare( shadowMap, cubeToUV( bd3D + offset.xyx, texelSize.y ), dp ) +\n\t\t\t\ttexture2DCompare( shadowMap, cubeToUV( bd3D + offset.yyx, texelSize.y ), dp ) +\n\t\t\t\ttexture2DCompare( shadowMap, cubeToUV( bd3D, texelSize.y ), dp ) +\n\t\t\t\ttexture2DCompare( shadowMap, cubeToUV( bd3D + offset.xxy, texelSize.y ), dp ) +\n\t\t\t\ttexture2DCompare( shadowMap, cubeToUV( bd3D + offset.yxy, texelSize.y ), dp ) +\n\t\t\t\ttexture2DCompare( shadowMap, cubeToUV( bd3D + offset.xxx, texelSize.y ), dp ) +\n\t\t\t\ttexture2DCompare( shadowMap, cubeToUV( bd3D + offset.yxx, texelSize.y ), dp )\n\t\t\t) * ( 1.0 / 9.0 );\n\t\t#else\n\t\t\treturn texture2DCompare( shadowMap, cubeToUV( bd3D, texelSize.y ), dp );\n\t\t#endif\n\t}\n#endif",shadowmap_pars_vertex:"#ifdef USE_SHADOWMAP\n\t#if NUM_DIR_LIGHT_SHADOWS > 0\n\t\tuniform mat4 directionalShadowMatrix[ NUM_DIR_LIGHT_SHADOWS ];\n\t\tvarying vec4 vDirectionalShadowCoord[ NUM_DIR_LIGHT_SHADOWS ];\n\t#endif\n\t#if NUM_SPOT_LIGHT_SHADOWS > 0\n\t\tuniform mat4 spotShadowMatrix[ NUM_SPOT_LIGHT_SHADOWS ];\n\t\tvarying vec4 vSpotShadowCoord[ NUM_SPOT_LIGHT_SHADOWS ];\n\t#endif\n\t#if NUM_POINT_LIGHT_SHADOWS > 0\n\t\tuniform mat4 pointShadowMatrix[ NUM_POINT_LIGHT_SHADOWS ];\n\t\tvarying vec4 vPointShadowCoord[ NUM_POINT_LIGHT_SHADOWS ];\n\t#endif\n#endif",shadowmap_vertex:"#ifdef USE_SHADOWMAP\n\t#if NUM_DIR_LIGHT_SHADOWS > 0\n\t#pragma unroll_loop_start\n\tfor ( int i = 0; i < NUM_DIR_LIGHT_SHADOWS; i ++ ) {\n\t\tvDirectionalShadowCoord[ i ] = directionalShadowMatrix[ i ] * worldPosition;\n\t}\n\t#pragma unroll_loop_end\n\t#endif\n\t#if NUM_SPOT_LIGHT_SHADOWS > 0\n\t#pragma unroll_loop_start\n\tfor ( int i = 0; i < NUM_SPOT_LIGHT_SHADOWS; i ++ ) {\n\t\tvSpotShadowCoord[ i ] = spotShadowMatrix[ i ] * worldPosition;\n\t}\n\t#pragma unroll_loop_end\n\t#endif\n\t#if NUM_POINT_LIGHT_SHADOWS > 0\n\t#pragma unroll_loop_start\n\tfor ( int i = 0; i < NUM_POINT_LIGHT_SHADOWS; i ++ ) {\n\t\tvPointShadowCoord[ i ] = pointShadowMatrix[ i ] * worldPosition;\n\t}\n\t#pragma unroll_loop_end\n\t#endif\n#endif",shadowmask_pars_fragment:"float getShadowMask() {\n\tfloat shadow = 1.0;\n\t#ifdef USE_SHADOWMAP\n\t#if NUM_DIR_LIGHT_SHADOWS > 0\n\tDirectionalLightShadow directionalLight;\n\t#pragma unroll_loop_start\n\tfor ( int i = 0; i < NUM_DIR_LIGHT_SHADOWS; i ++ ) {\n\t\tdirectionalLight = directionalLightShadows[ i ];\n\t\tshadow *= receiveShadow ? getShadow( directionalShadowMap[ i ], directionalLight.shadowMapSize, directionalLight.shadowBias, directionalLight.shadowRadius, vDirectionalShadowCoord[ i ] ) : 1.0;\n\t}\n\t#pragma unroll_loop_end\n\t#endif\n\t#if NUM_SPOT_LIGHT_SHADOWS > 0\n\tSpotLightShadow spotLight;\n\t#pragma unroll_loop_start\n\tfor ( int i = 0; i < NUM_SPOT_LIGHT_SHADOWS; i ++ ) {\n\t\tspotLight = spotLightShadows[ i ];\n\t\tshadow *= receiveShadow ? getShadow( spotShadowMap[ i ], spotLight.shadowMapSize, spotLight.shadowBias, spotLight.shadowRadius, vSpotShadowCoord[ i ] ) : 1.0;\n\t}\n\t#pragma unroll_loop_end\n\t#endif\n\t#if NUM_POINT_LIGHT_SHADOWS > 0\n\tPointLightShadow pointLight;\n\t#pragma unroll_loop_start\n\tfor ( int i = 0; i < NUM_POINT_LIGHT_SHADOWS; i ++ ) {\n\t\tpointLight = pointLightShadows[ i ];\n\t\tshadow *= receiveShadow ? getPointShadow( pointShadowMap[ i ], pointLight.shadowMapSize, pointLight.shadowBias, pointLight.shadowRadius, vPointShadowCoord[ i ], pointLight.shadowCameraNear, pointLight.shadowCameraFar ) : 1.0;\n\t}\n\t#pragma unroll_loop_end\n\t#endif\n\t#endif\n\treturn shadow;\n}",skinbase_vertex:"#ifdef USE_SKINNING\n\tmat4 boneMatX = getBoneMatrix( skinIndex.x );\n\tmat4 boneMatY = getBoneMatrix( skinIndex.y );\n\tmat4 boneMatZ = getBoneMatrix( skinIndex.z );\n\tmat4 boneMatW = getBoneMatrix( skinIndex.w );\n#endif",skinning_pars_vertex:"#ifdef USE_SKINNING\n\tuniform mat4 bindMatrix;\n\tuniform mat4 bindMatrixInverse;\n\t#ifdef BONE_TEXTURE\n\t\tuniform highp sampler2D boneTexture;\n\t\tuniform int boneTextureSize;\n\t\tmat4 getBoneMatrix( const in float i ) {\n\t\t\tfloat j = i * 4.0;\n\t\t\tfloat x = mod( j, float( boneTextureSize ) );\n\t\t\tfloat y = floor( j / float( boneTextureSize ) );\n\t\t\tfloat dx = 1.0 / float( boneTextureSize );\n\t\t\tfloat dy = 1.0 / float( boneTextureSize );\n\t\t\ty = dy * ( y + 0.5 );\n\t\t\tvec4 v1 = texture2D( boneTexture, vec2( dx * ( x + 0.5 ), y ) );\n\t\t\tvec4 v2 = texture2D( boneTexture, vec2( dx * ( x + 1.5 ), y ) );\n\t\t\tvec4 v3 = texture2D( boneTexture, vec2( dx * ( x + 2.5 ), y ) );\n\t\t\tvec4 v4 = texture2D( boneTexture, vec2( dx * ( x + 3.5 ), y ) );\n\t\t\tmat4 bone = mat4( v1, v2, v3, v4 );\n\t\t\treturn bone;\n\t\t}\n\t#else\n\t\tuniform mat4 boneMatrices[ MAX_BONES ];\n\t\tmat4 getBoneMatrix( const in float i ) {\n\t\t\tmat4 bone = boneMatrices[ int(i) ];\n\t\t\treturn bone;\n\t\t}\n\t#endif\n#endif",skinning_vertex:"#ifdef USE_SKINNING\n\tvec4 skinVertex = bindMatrix * vec4( transformed, 1.0 );\n\tvec4 skinned = vec4( 0.0 );\n\tskinned += boneMatX * skinVertex * skinWeight.x;\n\tskinned += boneMatY * skinVertex * skinWeight.y;\n\tskinned += boneMatZ * skinVertex * skinWeight.z;\n\tskinned += boneMatW * skinVertex * skinWeight.w;\n\ttransformed = ( bindMatrixInverse * skinned ).xyz;\n#endif",skinnormal_vertex:"#ifdef USE_SKINNING\n\tmat4 skinMatrix = mat4( 0.0 );\n\tskinMatrix += skinWeight.x * boneMatX;\n\tskinMatrix += skinWeight.y * boneMatY;\n\tskinMatrix += skinWeight.z * boneMatZ;\n\tskinMatrix += skinWeight.w * boneMatW;\n\tskinMatrix  = bindMatrixInverse * skinMatrix * bindMatrix;\n\tobjectNormal = vec4( skinMatrix * vec4( objectNormal, 0.0 ) ).xyz;\n\t#ifdef USE_TANGENT\n\t\tobjectTangent = vec4( skinMatrix * vec4( objectTangent, 0.0 ) ).xyz;\n\t#endif\n#endif",specularmap_fragment:"float specularStrength;\n#ifdef USE_SPECULARMAP\n\tvec4 texelSpecular = texture2D( specularMap, vUv );\n\tspecularStrength = texelSpecular.r;\n#else\n\tspecularStrength = 1.0;\n#endif",specularmap_pars_fragment:"#ifdef USE_SPECULARMAP\n\tuniform sampler2D specularMap;\n#endif",tonemapping_fragment:"#if defined( TONE_MAPPING )\n\tgl_FragColor.rgb = toneMapping( gl_FragColor.rgb );\n#endif",tonemapping_pars_fragment:"#ifndef saturate\n#define saturate(a) clamp( a, 0.0, 1.0 )\n#endif\nuniform float toneMappingExposure;\nuniform float toneMappingWhitePoint;\nvec3 LinearToneMapping( vec3 color ) {\n\treturn toneMappingExposure * color;\n}\nvec3 ReinhardToneMapping( vec3 color ) {\n\tcolor *= toneMappingExposure;\n\treturn saturate( color / ( vec3( 1.0 ) + color ) );\n}\n#define Uncharted2Helper( x ) max( ( ( x * ( 0.15 * x + 0.10 * 0.50 ) + 0.20 * 0.02 ) / ( x * ( 0.15 * x + 0.50 ) + 0.20 * 0.30 ) ) - 0.02 / 0.30, vec3( 0.0 ) )\nvec3 Uncharted2ToneMapping( vec3 color ) {\n\tcolor *= toneMappingExposure;\n\treturn saturate( Uncharted2Helper( color ) / Uncharted2Helper( vec3( toneMappingWhitePoint ) ) );\n}\nvec3 OptimizedCineonToneMapping( vec3 color ) {\n\tcolor *= toneMappingExposure;\n\tcolor = max( vec3( 0.0 ), color - 0.004 );\n\treturn pow( ( color * ( 6.2 * color + 0.5 ) ) / ( color * ( 6.2 * color + 1.7 ) + 0.06 ), vec3( 2.2 ) );\n}\nvec3 ACESFilmicToneMapping( vec3 color ) {\n\tcolor *= toneMappingExposure;\n\treturn saturate( ( color * ( 2.51 * color + 0.03 ) ) / ( color * ( 2.43 * color + 0.59 ) + 0.14 ) );\n}",uv_pars_fragment:"#if ( defined( USE_UV ) && ! defined( UVS_VERTEX_ONLY ) )\n\tvarying vec2 vUv;\n#endif",uv_pars_vertex:"#ifdef USE_UV\n\t#ifdef UVS_VERTEX_ONLY\n\t\tvec2 vUv;\n\t#else\n\t\tvarying vec2 vUv;\n\t#endif\n\tuniform mat3 uvTransform;\n#endif",uv_vertex:"#ifdef USE_UV\n\tvUv = ( uvTransform * vec3( uv, 1 ) ).xy;\n#endif",uv2_pars_fragment:"#if defined( USE_LIGHTMAP ) || defined( USE_AOMAP )\n\tvarying vec2 vUv2;\n#endif",uv2_pars_vertex:"#if defined( USE_LIGHTMAP ) || defined( USE_AOMAP )\n\tattribute vec2 uv2;\n\tvarying vec2 vUv2;\n\tuniform mat3 uv2Transform;\n#endif",uv2_vertex:"#if defined( USE_LIGHTMAP ) || defined( USE_AOMAP )\n\tvUv2 = ( uv2Transform * vec3( uv2, 1 ) ).xy;\n#endif",worldpos_vertex:"#if defined( USE_ENVMAP ) || defined( DISTANCE ) || defined ( USE_SHADOWMAP )\n\tvec4 worldPosition = vec4( transformed, 1.0 );\n\t#ifdef USE_INSTANCING\n\t\tworldPosition = instanceMatrix * worldPosition;\n\t#endif\n\tworldPosition = modelMatrix * worldPosition;\n#endif",background_frag:"uniform sampler2D t2D;\nvarying vec2 vUv;\nvoid main() {\n\tvec4 texColor = texture2D( t2D, vUv );\n\tgl_FragColor = mapTexelToLinear( texColor );\n\t#include <tonemapping_fragment>\n\t#include <encodings_fragment>\n}",background_vert:"varying vec2 vUv;\nuniform mat3 uvTransform;\nvoid main() {\n\tvUv = ( uvTransform * vec3( uv, 1 ) ).xy;\n\tgl_Position = vec4( position.xy, 1.0, 1.0 );\n}",cube_frag:"#include <envmap_common_pars_fragment>\nuniform float opacity;\nvarying vec3 vWorldDirection;\n#include <cube_uv_reflection_fragment>\nvoid main() {\n\tvec3 vReflect = vWorldDirection;\n\t#include <envmap_fragment>\n\tgl_FragColor = envColor;\n\tgl_FragColor.a *= opacity;\n\t#include <tonemapping_fragment>\n\t#include <encodings_fragment>\n}",cube_vert:"varying vec3 vWorldDirection;\n#include <common>\nvoid main() {\n\tvWorldDirection = transformDirection( position, modelMatrix );\n\t#include <begin_vertex>\n\t#include <project_vertex>\n\tgl_Position.z = gl_Position.w;\n}",depth_frag:"#if DEPTH_PACKING == 3200\n\tuniform float opacity;\n#endif\n#include <common>\n#include <packing>\n#include <uv_pars_fragment>\n#include <map_pars_fragment>\n#include <alphamap_pars_fragment>\n#include <logdepthbuf_pars_fragment>\n#include <clipping_planes_pars_fragment>\nvarying vec2 vHighPrecisionZW;\nvoid main() {\n\t#include <clipping_planes_fragment>\n\tvec4 diffuseColor = vec4( 1.0 );\n\t#if DEPTH_PACKING == 3200\n\t\tdiffuseColor.a = opacity;\n\t#endif\n\t#include <map_fragment>\n\t#include <alphamap_fragment>\n\t#include <alphatest_fragment>\n\t#include <logdepthbuf_fragment>\n\tfloat fragCoordZ = 0.5 * vHighPrecisionZW[0] / vHighPrecisionZW[1] + 0.5;\n\t#if DEPTH_PACKING == 3200\n\t\tgl_FragColor = vec4( vec3( 1.0 - fragCoordZ ), opacity );\n\t#elif DEPTH_PACKING == 3201\n\t\tgl_FragColor = packDepthToRGBA( fragCoordZ );\n\t#endif\n}",depth_vert:"#include <common>\n#include <uv_pars_vertex>\n#include <displacementmap_pars_vertex>\n#include <morphtarget_pars_vertex>\n#include <skinning_pars_vertex>\n#include <logdepthbuf_pars_vertex>\n#include <clipping_planes_pars_vertex>\nvarying vec2 vHighPrecisionZW;\nvoid main() {\n\t#include <uv_vertex>\n\t#include <skinbase_vertex>\n\t#ifdef USE_DISPLACEMENTMAP\n\t\t#include <beginnormal_vertex>\n\t\t#include <morphnormal_vertex>\n\t\t#include <skinnormal_vertex>\n\t#endif\n\t#include <begin_vertex>\n\t#include <morphtarget_vertex>\n\t#include <skinning_vertex>\n\t#include <displacementmap_vertex>\n\t#include <project_vertex>\n\t#include <logdepthbuf_vertex>\n\t#include <clipping_planes_vertex>\n\tvHighPrecisionZW = gl_Position.zw;\n}",distanceRGBA_frag:"#define DISTANCE\nuniform vec3 referencePosition;\nuniform float nearDistance;\nuniform float farDistance;\nvarying vec3 vWorldPosition;\n#include <common>\n#include <packing>\n#include <uv_pars_fragment>\n#include <map_pars_fragment>\n#include <alphamap_pars_fragment>\n#include <clipping_planes_pars_fragment>\nvoid main () {\n\t#include <clipping_planes_fragment>\n\tvec4 diffuseColor = vec4( 1.0 );\n\t#include <map_fragment>\n\t#include <alphamap_fragment>\n\t#include <alphatest_fragment>\n\tfloat dist = length( vWorldPosition - referencePosition );\n\tdist = ( dist - nearDistance ) / ( farDistance - nearDistance );\n\tdist = saturate( dist );\n\tgl_FragColor = packDepthToRGBA( dist );\n}",distanceRGBA_vert:"#define DISTANCE\nvarying vec3 vWorldPosition;\n#include <common>\n#include <uv_pars_vertex>\n#include <displacementmap_pars_vertex>\n#include <morphtarget_pars_vertex>\n#include <skinning_pars_vertex>\n#include <clipping_planes_pars_vertex>\nvoid main() {\n\t#include <uv_vertex>\n\t#include <skinbase_vertex>\n\t#ifdef USE_DISPLACEMENTMAP\n\t\t#include <beginnormal_vertex>\n\t\t#include <morphnormal_vertex>\n\t\t#include <skinnormal_vertex>\n\t#endif\n\t#include <begin_vertex>\n\t#include <morphtarget_vertex>\n\t#include <skinning_vertex>\n\t#include <displacementmap_vertex>\n\t#include <project_vertex>\n\t#include <worldpos_vertex>\n\t#include <clipping_planes_vertex>\n\tvWorldPosition = worldPosition.xyz;\n}",equirect_frag:"uniform sampler2D tEquirect;\nvarying vec3 vWorldDirection;\n#include <common>\nvoid main() {\n\tvec3 direction = normalize( vWorldDirection );\n\tvec2 sampleUV = equirectUv( direction );\n\tvec4 texColor = texture2D( tEquirect, sampleUV );\n\tgl_FragColor = mapTexelToLinear( texColor );\n\t#include <tonemapping_fragment>\n\t#include <encodings_fragment>\n}",equirect_vert:"varying vec3 vWorldDirection;\n#include <common>\nvoid main() {\n\tvWorldDirection = transformDirection( position, modelMatrix );\n\t#include <begin_vertex>\n\t#include <project_vertex>\n}",linedashed_frag:"uniform vec3 diffuse;\nuniform float opacity;\nuniform float dashSize;\nuniform float totalSize;\nvarying float vLineDistance;\n#include <common>\n#include <color_pars_fragment>\n#include <fog_pars_fragment>\n#include <logdepthbuf_pars_fragment>\n#include <clipping_planes_pars_fragment>\nvoid main() {\n\t#include <clipping_planes_fragment>\n\tif ( mod( vLineDistance, totalSize ) > dashSize ) {\n\t\tdiscard;\n\t}\n\tvec3 outgoingLight = vec3( 0.0 );\n\tvec4 diffuseColor = vec4( diffuse, opacity );\n\t#include <logdepthbuf_fragment>\n\t#include <color_fragment>\n\toutgoingLight = diffuseColor.rgb;\n\tgl_FragColor = vec4( outgoingLight, diffuseColor.a );\n\t#include <tonemapping_fragment>\n\t#include <encodings_fragment>\n\t#include <fog_fragment>\n\t#include <premultiplied_alpha_fragment>\n}",linedashed_vert:"uniform float scale;\nattribute float lineDistance;\nvarying float vLineDistance;\n#include <common>\n#include <color_pars_vertex>\n#include <fog_pars_vertex>\n#include <morphtarget_pars_vertex>\n#include <logdepthbuf_pars_vertex>\n#include <clipping_planes_pars_vertex>\nvoid main() {\n\tvLineDistance = scale * lineDistance;\n\t#include <color_vertex>\n\t#include <begin_vertex>\n\t#include <morphtarget_vertex>\n\t#include <project_vertex>\n\t#include <logdepthbuf_vertex>\n\t#include <clipping_planes_vertex>\n\t#include <fog_vertex>\n}",meshbasic_frag:"uniform vec3 diffuse;\nuniform float opacity;\n#ifndef FLAT_SHADED\n\tvarying vec3 vNormal;\n#endif\n#include <common>\n#include <dithering_pars_fragment>\n#include <color_pars_fragment>\n#include <uv_pars_fragment>\n#include <uv2_pars_fragment>\n#include <map_pars_fragment>\n#include <alphamap_pars_fragment>\n#include <aomap_pars_fragment>\n#include <lightmap_pars_fragment>\n#include <envmap_common_pars_fragment>\n#include <envmap_pars_fragment>\n#include <cube_uv_reflection_fragment>\n#include <fog_pars_fragment>\n#include <specularmap_pars_fragment>\n#include <logdepthbuf_pars_fragment>\n#include <clipping_planes_pars_fragment>\nvoid main() {\n\t#include <clipping_planes_fragment>\n\tvec4 diffuseColor = vec4( diffuse, opacity );\n\t#include <logdepthbuf_fragment>\n\t#include <map_fragment>\n\t#include <color_fragment>\n\t#include <alphamap_fragment>\n\t#include <alphatest_fragment>\n\t#include <specularmap_fragment>\n\tReflectedLight reflectedLight = ReflectedLight( vec3( 0.0 ), vec3( 0.0 ), vec3( 0.0 ), vec3( 0.0 ) );\n\t#ifdef USE_LIGHTMAP\n\t\n\t\tvec4 lightMapTexel= texture2D( lightMap, vUv2 );\n\t\treflectedLight.indirectDiffuse += lightMapTexelToLinear( lightMapTexel ).rgb * lightMapIntensity;\n\t#else\n\t\treflectedLight.indirectDiffuse += vec3( 1.0 );\n\t#endif\n\t#include <aomap_fragment>\n\treflectedLight.indirectDiffuse *= diffuseColor.rgb;\n\tvec3 outgoingLight = reflectedLight.indirectDiffuse;\n\t#include <envmap_fragment>\n\tgl_FragColor = vec4( outgoingLight, diffuseColor.a );\n\t#include <tonemapping_fragment>\n\t#include <encodings_fragment>\n\t#include <fog_fragment>\n\t#include <premultiplied_alpha_fragment>\n\t#include <dithering_fragment>\n}",meshbasic_vert:"#include <common>\n#include <uv_pars_vertex>\n#include <uv2_pars_vertex>\n#include <envmap_pars_vertex>\n#include <color_pars_vertex>\n#include <fog_pars_vertex>\n#include <morphtarget_pars_vertex>\n#include <skinning_pars_vertex>\n#include <logdepthbuf_pars_vertex>\n#include <clipping_planes_pars_vertex>\nvoid main() {\n\t#include <uv_vertex>\n\t#include <uv2_vertex>\n\t#include <color_vertex>\n\t#include <skinbase_vertex>\n\t#ifdef USE_ENVMAP\n\t#include <beginnormal_vertex>\n\t#include <morphnormal_vertex>\n\t#include <skinnormal_vertex>\n\t#include <defaultnormal_vertex>\n\t#endif\n\t#include <begin_vertex>\n\t#include <morphtarget_vertex>\n\t#include <skinning_vertex>\n\t#include <project_vertex>\n\t#include <logdepthbuf_vertex>\n\t#include <worldpos_vertex>\n\t#include <clipping_planes_vertex>\n\t#include <envmap_vertex>\n\t#include <fog_vertex>\n}",meshlambert_frag:"uniform vec3 diffuse;\nuniform vec3 emissive;\nuniform float opacity;\nvarying vec3 vLightFront;\nvarying vec3 vIndirectFront;\n#ifdef DOUBLE_SIDED\n\tvarying vec3 vLightBack;\n\tvarying vec3 vIndirectBack;\n#endif\n#include <common>\n#include <packing>\n#include <dithering_pars_fragment>\n#include <color_pars_fragment>\n#include <uv_pars_fragment>\n#include <uv2_pars_fragment>\n#include <map_pars_fragment>\n#include <alphamap_pars_fragment>\n#include <aomap_pars_fragment>\n#include <lightmap_pars_fragment>\n#include <emissivemap_pars_fragment>\n#include <envmap_common_pars_fragment>\n#include <envmap_pars_fragment>\n#include <cube_uv_reflection_fragment>\n#include <bsdfs>\n#include <lights_pars_begin>\n#include <fog_pars_fragment>\n#include <shadowmap_pars_fragment>\n#include <shadowmask_pars_fragment>\n#include <specularmap_pars_fragment>\n#include <logdepthbuf_pars_fragment>\n#include <clipping_planes_pars_fragment>\nvoid main() {\n\t#include <clipping_planes_fragment>\n\tvec4 diffuseColor = vec4( diffuse, opacity );\n\tReflectedLight reflectedLight = ReflectedLight( vec3( 0.0 ), vec3( 0.0 ), vec3( 0.0 ), vec3( 0.0 ) );\n\tvec3 totalEmissiveRadiance = emissive;\n\t#include <logdepthbuf_fragment>\n\t#include <map_fragment>\n\t#include <color_fragment>\n\t#include <alphamap_fragment>\n\t#include <alphatest_fragment>\n\t#include <specularmap_fragment>\n\t#include <emissivemap_fragment>\n\t#ifdef DOUBLE_SIDED\n\t\treflectedLight.indirectDiffuse += ( gl_FrontFacing ) ? vIndirectFront : vIndirectBack;\n\t#else\n\t\treflectedLight.indirectDiffuse += vIndirectFront;\n\t#endif\n\t#include <lightmap_fragment>\n\treflectedLight.indirectDiffuse *= BRDF_Diffuse_Lambert( diffuseColor.rgb );\n\t#ifdef DOUBLE_SIDED\n\t\treflectedLight.directDiffuse = ( gl_FrontFacing ) ? vLightFront : vLightBack;\n\t#else\n\t\treflectedLight.directDiffuse = vLightFront;\n\t#endif\n\treflectedLight.directDiffuse *= BRDF_Diffuse_Lambert( diffuseColor.rgb ) * getShadowMask();\n\t#include <aomap_fragment>\n\tvec3 outgoingLight = reflectedLight.directDiffuse + reflectedLight.indirectDiffuse + totalEmissiveRadiance;\n\t#include <envmap_fragment>\n\tgl_FragColor = vec4( outgoingLight, diffuseColor.a );\n\t#include <tonemapping_fragment>\n\t#include <encodings_fragment>\n\t#include <fog_fragment>\n\t#include <premultiplied_alpha_fragment>\n\t#include <dithering_fragment>\n}",meshlambert_vert:"#define LAMBERT\nvarying vec3 vLightFront;\nvarying vec3 vIndirectFront;\n#ifdef DOUBLE_SIDED\n\tvarying vec3 vLightBack;\n\tvarying vec3 vIndirectBack;\n#endif\n#include <common>\n#include <uv_pars_vertex>\n#include <uv2_pars_vertex>\n#include <envmap_pars_vertex>\n#include <bsdfs>\n#include <lights_pars_begin>\n#include <color_pars_vertex>\n#include <fog_pars_vertex>\n#include <morphtarget_pars_vertex>\n#include <skinning_pars_vertex>\n#include <shadowmap_pars_vertex>\n#include <logdepthbuf_pars_vertex>\n#include <clipping_planes_pars_vertex>\nvoid main() {\n\t#include <uv_vertex>\n\t#include <uv2_vertex>\n\t#include <color_vertex>\n\t#include <beginnormal_vertex>\n\t#include <morphnormal_vertex>\n\t#include <skinbase_vertex>\n\t#include <skinnormal_vertex>\n\t#include <defaultnormal_vertex>\n\t#include <begin_vertex>\n\t#include <morphtarget_vertex>\n\t#include <skinning_vertex>\n\t#include <project_vertex>\n\t#include <logdepthbuf_vertex>\n\t#include <clipping_planes_vertex>\n\t#include <worldpos_vertex>\n\t#include <envmap_vertex>\n\t#include <lights_lambert_vertex>\n\t#include <shadowmap_vertex>\n\t#include <fog_vertex>\n}",meshmatcap_frag:"#define MATCAP\nuniform vec3 diffuse;\nuniform float opacity;\nuniform sampler2D matcap;\nvarying vec3 vViewPosition;\n#ifndef FLAT_SHADED\n\tvarying vec3 vNormal;\n#endif\n#include <common>\n#include <dithering_pars_fragment>\n#include <color_pars_fragment>\n#include <uv_pars_fragment>\n#include <map_pars_fragment>\n#include <alphamap_pars_fragment>\n#include <fog_pars_fragment>\n#include <bumpmap_pars_fragment>\n#include <normalmap_pars_fragment>\n#include <logdepthbuf_pars_fragment>\n#include <clipping_planes_pars_fragment>\nvoid main() {\n\t#include <clipping_planes_fragment>\n\tvec4 diffuseColor = vec4( diffuse, opacity );\n\t#include <logdepthbuf_fragment>\n\t#include <map_fragment>\n\t#include <color_fragment>\n\t#include <alphamap_fragment>\n\t#include <alphatest_fragment>\n\t#include <normal_fragment_begin>\n\t#include <normal_fragment_maps>\n\tvec3 viewDir = normalize( vViewPosition );\n\tvec3 x = normalize( vec3( viewDir.z, 0.0, - viewDir.x ) );\n\tvec3 y = cross( viewDir, x );\n\tvec2 uv = vec2( dot( x, normal ), dot( y, normal ) ) * 0.495 + 0.5;\n\t#ifdef USE_MATCAP\n\t\tvec4 matcapColor = texture2D( matcap, uv );\n\t\tmatcapColor = matcapTexelToLinear( matcapColor );\n\t#else\n\t\tvec4 matcapColor = vec4( 1.0 );\n\t#endif\n\tvec3 outgoingLight = diffuseColor.rgb * matcapColor.rgb;\n\tgl_FragColor = vec4( outgoingLight, diffuseColor.a );\n\t#include <tonemapping_fragment>\n\t#include <encodings_fragment>\n\t#include <fog_fragment>\n\t#include <premultiplied_alpha_fragment>\n\t#include <dithering_fragment>\n}",meshmatcap_vert:"#define MATCAP\nvarying vec3 vViewPosition;\n#ifndef FLAT_SHADED\n\tvarying vec3 vNormal;\n#endif\n#include <common>\n#include <uv_pars_vertex>\n#include <color_pars_vertex>\n#include <displacementmap_pars_vertex>\n#include <fog_pars_vertex>\n#include <morphtarget_pars_vertex>\n#include <skinning_pars_vertex>\n#include <logdepthbuf_pars_vertex>\n#include <clipping_planes_pars_vertex>\nvoid main() {\n\t#include <uv_vertex>\n\t#include <color_vertex>\n\t#include <beginnormal_vertex>\n\t#include <morphnormal_vertex>\n\t#include <skinbase_vertex>\n\t#include <skinnormal_vertex>\n\t#include <defaultnormal_vertex>\n\t#ifndef FLAT_SHADED\n\t\tvNormal = normalize( transformedNormal );\n\t#endif\n\t#include <begin_vertex>\n\t#include <morphtarget_vertex>\n\t#include <skinning_vertex>\n\t#include <displacementmap_vertex>\n\t#include <project_vertex>\n\t#include <logdepthbuf_vertex>\n\t#include <clipping_planes_vertex>\n\t#include <fog_vertex>\n\tvViewPosition = - mvPosition.xyz;\n}",meshtoon_frag:"#define TOON\nuniform vec3 diffuse;\nuniform vec3 emissive;\nuniform vec3 specular;\nuniform float shininess;\nuniform float opacity;\n#include <common>\n#include <packing>\n#include <dithering_pars_fragment>\n#include <color_pars_fragment>\n#include <uv_pars_fragment>\n#include <uv2_pars_fragment>\n#include <map_pars_fragment>\n#include <alphamap_pars_fragment>\n#include <aomap_pars_fragment>\n#include <lightmap_pars_fragment>\n#include <emissivemap_pars_fragment>\n#include <gradientmap_pars_fragment>\n#include <fog_pars_fragment>\n#include <bsdfs>\n#include <lights_pars_begin>\n#include <lights_toon_pars_fragment>\n#include <shadowmap_pars_fragment>\n#include <bumpmap_pars_fragment>\n#include <normalmap_pars_fragment>\n#include <specularmap_pars_fragment>\n#include <logdepthbuf_pars_fragment>\n#include <clipping_planes_pars_fragment>\nvoid main() {\n\t#include <clipping_planes_fragment>\n\tvec4 diffuseColor = vec4( diffuse, opacity );\n\tReflectedLight reflectedLight = ReflectedLight( vec3( 0.0 ), vec3( 0.0 ), vec3( 0.0 ), vec3( 0.0 ) );\n\tvec3 totalEmissiveRadiance = emissive;\n\t#include <logdepthbuf_fragment>\n\t#include <map_fragment>\n\t#include <color_fragment>\n\t#include <alphamap_fragment>\n\t#include <alphatest_fragment>\n\t#include <specularmap_fragment>\n\t#include <normal_fragment_begin>\n\t#include <normal_fragment_maps>\n\t#include <emissivemap_fragment>\n\t#include <lights_toon_fragment>\n\t#include <lights_fragment_begin>\n\t#include <lights_fragment_maps>\n\t#include <lights_fragment_end>\n\t#include <aomap_fragment>\n\tvec3 outgoingLight = reflectedLight.directDiffuse + reflectedLight.indirectDiffuse + reflectedLight.directSpecular + reflectedLight.indirectSpecular + totalEmissiveRadiance;\n\tgl_FragColor = vec4( outgoingLight, diffuseColor.a );\n\t#include <tonemapping_fragment>\n\t#include <encodings_fragment>\n\t#include <fog_fragment>\n\t#include <premultiplied_alpha_fragment>\n\t#include <dithering_fragment>\n}",meshtoon_vert:"#define TOON\nvarying vec3 vViewPosition;\n#ifndef FLAT_SHADED\n\tvarying vec3 vNormal;\n#endif\n#include <common>\n#include <uv_pars_vertex>\n#include <uv2_pars_vertex>\n#include <displacementmap_pars_vertex>\n#include <color_pars_vertex>\n#include <fog_pars_vertex>\n#include <morphtarget_pars_vertex>\n#include <skinning_pars_vertex>\n#include <shadowmap_pars_vertex>\n#include <logdepthbuf_pars_vertex>\n#include <clipping_planes_pars_vertex>\nvoid main() {\n\t#include <uv_vertex>\n\t#include <uv2_vertex>\n\t#include <color_vertex>\n\t#include <beginnormal_vertex>\n\t#include <morphnormal_vertex>\n\t#include <skinbase_vertex>\n\t#include <skinnormal_vertex>\n\t#include <defaultnormal_vertex>\n#ifndef FLAT_SHADED\n\tvNormal = normalize( transformedNormal );\n#endif\n\t#include <begin_vertex>\n\t#include <morphtarget_vertex>\n\t#include <skinning_vertex>\n\t#include <displacementmap_vertex>\n\t#include <project_vertex>\n\t#include <logdepthbuf_vertex>\n\t#include <clipping_planes_vertex>\n\tvViewPosition = - mvPosition.xyz;\n\t#include <worldpos_vertex>\n\t#include <shadowmap_vertex>\n\t#include <fog_vertex>\n}",meshphong_frag:"#define PHONG\nuniform vec3 diffuse;\nuniform vec3 emissive;\nuniform vec3 specular;\nuniform float shininess;\nuniform float opacity;\n#include <common>\n#include <packing>\n#include <dithering_pars_fragment>\n#include <color_pars_fragment>\n#include <uv_pars_fragment>\n#include <uv2_pars_fragment>\n#include <map_pars_fragment>\n#include <alphamap_pars_fragment>\n#include <aomap_pars_fragment>\n#include <lightmap_pars_fragment>\n#include <emissivemap_pars_fragment>\n#include <envmap_common_pars_fragment>\n#include <envmap_pars_fragment>\n#include <cube_uv_reflection_fragment>\n#include <fog_pars_fragment>\n#include <bsdfs>\n#include <lights_pars_begin>\n#include <lights_phong_pars_fragment>\n#include <shadowmap_pars_fragment>\n#include <bumpmap_pars_fragment>\n#include <normalmap_pars_fragment>\n#include <specularmap_pars_fragment>\n#include <logdepthbuf_pars_fragment>\n#include <clipping_planes_pars_fragment>\nvoid main() {\n\t#include <clipping_planes_fragment>\n\tvec4 diffuseColor = vec4( diffuse, opacity );\n\tReflectedLight reflectedLight = ReflectedLight( vec3( 0.0 ), vec3( 0.0 ), vec3( 0.0 ), vec3( 0.0 ) );\n\tvec3 totalEmissiveRadiance = emissive;\n\t#include <logdepthbuf_fragment>\n\t#include <map_fragment>\n\t#include <color_fragment>\n\t#include <alphamap_fragment>\n\t#include <alphatest_fragment>\n\t#include <specularmap_fragment>\n\t#include <normal_fragment_begin>\n\t#include <normal_fragment_maps>\n\t#include <emissivemap_fragment>\n\t#include <lights_phong_fragment>\n\t#include <lights_fragment_begin>\n\t#include <lights_fragment_maps>\n\t#include <lights_fragment_end>\n\t#include <aomap_fragment>\n\tvec3 outgoingLight = reflectedLight.directDiffuse + reflectedLight.indirectDiffuse + reflectedLight.directSpecular + reflectedLight.indirectSpecular + totalEmissiveRadiance;\n\t#include <envmap_fragment>\n\tgl_FragColor = vec4( outgoingLight, diffuseColor.a );\n\t#include <tonemapping_fragment>\n\t#include <encodings_fragment>\n\t#include <fog_fragment>\n\t#include <premultiplied_alpha_fragment>\n\t#include <dithering_fragment>\n}",meshphong_vert:"#define PHONG\nvarying vec3 vViewPosition;\n#ifndef FLAT_SHADED\n\tvarying vec3 vNormal;\n#endif\n#include <common>\n#include <uv_pars_vertex>\n#include <uv2_pars_vertex>\n#include <displacementmap_pars_vertex>\n#include <envmap_pars_vertex>\n#include <color_pars_vertex>\n#include <fog_pars_vertex>\n#include <morphtarget_pars_vertex>\n#include <skinning_pars_vertex>\n#include <shadowmap_pars_vertex>\n#include <logdepthbuf_pars_vertex>\n#include <clipping_planes_pars_vertex>\nvoid main() {\n\t#include <uv_vertex>\n\t#include <uv2_vertex>\n\t#include <color_vertex>\n\t#include <beginnormal_vertex>\n\t#include <morphnormal_vertex>\n\t#include <skinbase_vertex>\n\t#include <skinnormal_vertex>\n\t#include <defaultnormal_vertex>\n#ifndef FLAT_SHADED\n\tvNormal = normalize( transformedNormal );\n#endif\n\t#include <begin_vertex>\n\t#include <morphtarget_vertex>\n\t#include <skinning_vertex>\n\t#include <displacementmap_vertex>\n\t#include <project_vertex>\n\t#include <logdepthbuf_vertex>\n\t#include <clipping_planes_vertex>\n\tvViewPosition = - mvPosition.xyz;\n\t#include <worldpos_vertex>\n\t#include <envmap_vertex>\n\t#include <shadowmap_vertex>\n\t#include <fog_vertex>\n}",meshphysical_frag:"#define STANDARD\n#ifdef PHYSICAL\n\t#define REFLECTIVITY\n\t#define CLEARCOAT\n\t#define TRANSPARENCY\n#endif\nuniform vec3 diffuse;\nuniform vec3 emissive;\nuniform float roughness;\nuniform float metalness;\nuniform float opacity;\n#ifdef TRANSPARENCY\n\tuniform float transparency;\n#endif\n#ifdef REFLECTIVITY\n\tuniform float reflectivity;\n#endif\n#ifdef CLEARCOAT\n\tuniform float clearcoat;\n\tuniform float clearcoatRoughness;\n#endif\n#ifdef USE_SHEEN\n\tuniform vec3 sheen;\n#endif\nvarying vec3 vViewPosition;\n#ifndef FLAT_SHADED\n\tvarying vec3 vNormal;\n\t#ifdef USE_TANGENT\n\t\tvarying vec3 vTangent;\n\t\tvarying vec3 vBitangent;\n\t#endif\n#endif\n#include <common>\n#include <packing>\n#include <dithering_pars_fragment>\n#include <color_pars_fragment>\n#include <uv_pars_fragment>\n#include <uv2_pars_fragment>\n#include <map_pars_fragment>\n#include <alphamap_pars_fragment>\n#include <aomap_pars_fragment>\n#include <lightmap_pars_fragment>\n#include <emissivemap_pars_fragment>\n#include <bsdfs>\n#include <cube_uv_reflection_fragment>\n#include <envmap_common_pars_fragment>\n#include <envmap_physical_pars_fragment>\n#include <fog_pars_fragment>\n#include <lights_pars_begin>\n#include <lights_physical_pars_fragment>\n#include <shadowmap_pars_fragment>\n#include <bumpmap_pars_fragment>\n#include <normalmap_pars_fragment>\n#include <clearcoat_pars_fragment>\n#include <roughnessmap_pars_fragment>\n#include <metalnessmap_pars_fragment>\n#include <logdepthbuf_pars_fragment>\n#include <clipping_planes_pars_fragment>\nvoid main() {\n\t#include <clipping_planes_fragment>\n\tvec4 diffuseColor = vec4( diffuse, opacity );\n\tReflectedLight reflectedLight = ReflectedLight( vec3( 0.0 ), vec3( 0.0 ), vec3( 0.0 ), vec3( 0.0 ) );\n\tvec3 totalEmissiveRadiance = emissive;\n\t#include <logdepthbuf_fragment>\n\t#include <map_fragment>\n\t#include <color_fragment>\n\t#include <alphamap_fragment>\n\t#include <alphatest_fragment>\n\t#include <roughnessmap_fragment>\n\t#include <metalnessmap_fragment>\n\t#include <normal_fragment_begin>\n\t#include <normal_fragment_maps>\n\t#include <clearcoat_normal_fragment_begin>\n\t#include <clearcoat_normal_fragment_maps>\n\t#include <emissivemap_fragment>\n\t#include <lights_physical_fragment>\n\t#include <lights_fragment_begin>\n\t#include <lights_fragment_maps>\n\t#include <lights_fragment_end>\n\t#include <aomap_fragment>\n\tvec3 outgoingLight = reflectedLight.directDiffuse + reflectedLight.indirectDiffuse + reflectedLight.directSpecular + reflectedLight.indirectSpecular + totalEmissiveRadiance;\n\t#ifdef TRANSPARENCY\n\t\tdiffuseColor.a *= saturate( 1. - transparency + linearToRelativeLuminance( reflectedLight.directSpecular + reflectedLight.indirectSpecular ) );\n\t#endif\n\tgl_FragColor = vec4( outgoingLight, diffuseColor.a );\n\t#include <tonemapping_fragment>\n\t#include <encodings_fragment>\n\t#include <fog_fragment>\n\t#include <premultiplied_alpha_fragment>\n\t#include <dithering_fragment>\n}",meshphysical_vert:"#define STANDARD\nvarying vec3 vViewPosition;\n#ifndef FLAT_SHADED\n\tvarying vec3 vNormal;\n\t#ifdef USE_TANGENT\n\t\tvarying vec3 vTangent;\n\t\tvarying vec3 vBitangent;\n\t#endif\n#endif\n#include <common>\n#include <uv_pars_vertex>\n#include <uv2_pars_vertex>\n#include <displacementmap_pars_vertex>\n#include <color_pars_vertex>\n#include <fog_pars_vertex>\n#include <morphtarget_pars_vertex>\n#include <skinning_pars_vertex>\n#include <shadowmap_pars_vertex>\n#include <logdepthbuf_pars_vertex>\n#include <clipping_planes_pars_vertex>\nvoid main() {\n\t#include <uv_vertex>\n\t#include <uv2_vertex>\n\t#include <color_vertex>\n\t#include <beginnormal_vertex>\n\t#include <morphnormal_vertex>\n\t#include <skinbase_vertex>\n\t#include <skinnormal_vertex>\n\t#include <defaultnormal_vertex>\n#ifndef FLAT_SHADED\n\tvNormal = normalize( transformedNormal );\n\t#ifdef USE_TANGENT\n\t\tvTangent = normalize( transformedTangent );\n\t\tvBitangent = normalize( cross( vNormal, vTangent ) * tangent.w );\n\t#endif\n#endif\n\t#include <begin_vertex>\n\t#include <morphtarget_vertex>\n\t#include <skinning_vertex>\n\t#include <displacementmap_vertex>\n\t#include <project_vertex>\n\t#include <logdepthbuf_vertex>\n\t#include <clipping_planes_vertex>\n\tvViewPosition = - mvPosition.xyz;\n\t#include <worldpos_vertex>\n\t#include <shadowmap_vertex>\n\t#include <fog_vertex>\n}",normal_frag:"#define NORMAL\nuniform float opacity;\n#if defined( FLAT_SHADED ) || defined( USE_BUMPMAP ) || defined( TANGENTSPACE_NORMALMAP )\n\tvarying vec3 vViewPosition;\n#endif\n#ifndef FLAT_SHADED\n\tvarying vec3 vNormal;\n\t#ifdef USE_TANGENT\n\t\tvarying vec3 vTangent;\n\t\tvarying vec3 vBitangent;\n\t#endif\n#endif\n#include <packing>\n#include <uv_pars_fragment>\n#include <bumpmap_pars_fragment>\n#include <normalmap_pars_fragment>\n#include <logdepthbuf_pars_fragment>\n#include <clipping_planes_pars_fragment>\nvoid main() {\n\t#include <clipping_planes_fragment>\n\t#include <logdepthbuf_fragment>\n\t#include <normal_fragment_begin>\n\t#include <normal_fragment_maps>\n\tgl_FragColor = vec4( packNormalToRGB( normal ), opacity );\n}",normal_vert:"#define NORMAL\n#if defined( FLAT_SHADED ) || defined( USE_BUMPMAP ) || defined( TANGENTSPACE_NORMALMAP )\n\tvarying vec3 vViewPosition;\n#endif\n#ifndef FLAT_SHADED\n\tvarying vec3 vNormal;\n\t#ifdef USE_TANGENT\n\t\tvarying vec3 vTangent;\n\t\tvarying vec3 vBitangent;\n\t#endif\n#endif\n#include <common>\n#include <uv_pars_vertex>\n#include <displacementmap_pars_vertex>\n#include <morphtarget_pars_vertex>\n#include <skinning_pars_vertex>\n#include <logdepthbuf_pars_vertex>\n#include <clipping_planes_pars_vertex>\nvoid main() {\n\t#include <uv_vertex>\n\t#include <beginnormal_vertex>\n\t#include <morphnormal_vertex>\n\t#include <skinbase_vertex>\n\t#include <skinnormal_vertex>\n\t#include <defaultnormal_vertex>\n#ifndef FLAT_SHADED\n\tvNormal = normalize( transformedNormal );\n\t#ifdef USE_TANGENT\n\t\tvTangent = normalize( transformedTangent );\n\t\tvBitangent = normalize( cross( vNormal, vTangent ) * tangent.w );\n\t#endif\n#endif\n\t#include <begin_vertex>\n\t#include <morphtarget_vertex>\n\t#include <skinning_vertex>\n\t#include <displacementmap_vertex>\n\t#include <project_vertex>\n\t#include <logdepthbuf_vertex>\n\t#include <clipping_planes_vertex>\n#if defined( FLAT_SHADED ) || defined( USE_BUMPMAP ) || defined( TANGENTSPACE_NORMALMAP )\n\tvViewPosition = - mvPosition.xyz;\n#endif\n}",points_frag:"uniform vec3 diffuse;\nuniform float opacity;\n#include <common>\n#include <color_pars_fragment>\n#include <map_particle_pars_fragment>\n#include <fog_pars_fragment>\n#include <logdepthbuf_pars_fragment>\n#include <clipping_planes_pars_fragment>\nvoid main() {\n\t#include <clipping_planes_fragment>\n\tvec3 outgoingLight = vec3( 0.0 );\n\tvec4 diffuseColor = vec4( diffuse, opacity );\n\t#include <logdepthbuf_fragment>\n\t#include <map_particle_fragment>\n\t#include <color_fragment>\n\t#include <alphatest_fragment>\n\toutgoingLight = diffuseColor.rgb;\n\tgl_FragColor = vec4( outgoingLight, diffuseColor.a );\n\t#include <tonemapping_fragment>\n\t#include <encodings_fragment>\n\t#include <fog_fragment>\n\t#include <premultiplied_alpha_fragment>\n}",points_vert:"uniform float size;\nuniform float scale;\n#include <common>\n#include <color_pars_vertex>\n#include <fog_pars_vertex>\n#include <morphtarget_pars_vertex>\n#include <logdepthbuf_pars_vertex>\n#include <clipping_planes_pars_vertex>\nvoid main() {\n\t#include <color_vertex>\n\t#include <begin_vertex>\n\t#include <morphtarget_vertex>\n\t#include <project_vertex>\n\tgl_PointSize = size;\n\t#ifdef USE_SIZEATTENUATION\n\t\tbool isPerspective = isPerspectiveMatrix( projectionMatrix );\n\t\tif ( isPerspective ) gl_PointSize *= ( scale / - mvPosition.z );\n\t#endif\n\t#include <logdepthbuf_vertex>\n\t#include <clipping_planes_vertex>\n\t#include <worldpos_vertex>\n\t#include <fog_vertex>\n}",shadow_frag:"uniform vec3 color;\nuniform float opacity;\n#include <common>\n#include <packing>\n#include <fog_pars_fragment>\n#include <bsdfs>\n#include <lights_pars_begin>\n#include <shadowmap_pars_fragment>\n#include <shadowmask_pars_fragment>\nvoid main() {\n\tgl_FragColor = vec4( color, opacity * ( 1.0 - getShadowMask() ) );\n\t#include <tonemapping_fragment>\n\t#include <encodings_fragment>\n\t#include <fog_fragment>\n}",shadow_vert:"#include <fog_pars_vertex>\n#include <shadowmap_pars_vertex>\nvoid main() {\n\t#include <begin_vertex>\n\t#include <project_vertex>\n\t#include <worldpos_vertex>\n\t#include <shadowmap_vertex>\n\t#include <fog_vertex>\n}",sprite_frag:"uniform vec3 diffuse;\nuniform float opacity;\n#include <common>\n#include <uv_pars_fragment>\n#include <map_pars_fragment>\n#include <alphamap_pars_fragment>\n#include <fog_pars_fragment>\n#include <logdepthbuf_pars_fragment>\n#include <clipping_planes_pars_fragment>\nvoid main() {\n\t#include <clipping_planes_fragment>\n\tvec3 outgoingLight = vec3( 0.0 );\n\tvec4 diffuseColor = vec4( diffuse, opacity );\n\t#include <logdepthbuf_fragment>\n\t#include <map_fragment>\n\t#include <alphamap_fragment>\n\t#include <alphatest_fragment>\n\toutgoingLight = diffuseColor.rgb;\n\tgl_FragColor = vec4( outgoingLight, diffuseColor.a );\n\t#include <tonemapping_fragment>\n\t#include <encodings_fragment>\n\t#include <fog_fragment>\n}",sprite_vert:"uniform float rotation;\nuniform vec2 center;\n#include <common>\n#include <uv_pars_vertex>\n#include <fog_pars_vertex>\n#include <logdepthbuf_pars_vertex>\n#include <clipping_planes_pars_vertex>\nvoid main() {\n\t#include <uv_vertex>\n\tvec4 mvPosition = modelViewMatrix * vec4( 0.0, 0.0, 0.0, 1.0 );\n\tvec2 scale;\n\tscale.x = length( vec3( modelMatrix[ 0 ].x, modelMatrix[ 0 ].y, modelMatrix[ 0 ].z ) );\n\tscale.y = length( vec3( modelMatrix[ 1 ].x, modelMatrix[ 1 ].y, modelMatrix[ 1 ].z ) );\n\t#ifndef USE_SIZEATTENUATION\n\t\tbool isPerspective = isPerspectiveMatrix( projectionMatrix );\n\t\tif ( isPerspective ) scale *= - mvPosition.z;\n\t#endif\n\tvec2 alignedPosition = ( position.xy - ( center - vec2( 0.5 ) ) ) * scale;\n\tvec2 rotatedPosition;\n\trotatedPosition.x = cos( rotation ) * alignedPosition.x - sin( rotation ) * alignedPosition.y;\n\trotatedPosition.y = sin( rotation ) * alignedPosition.x + cos( rotation ) * alignedPosition.y;\n\tmvPosition.xy += rotatedPosition;\n\tgl_Position = projectionMatrix * mvPosition;\n\t#include <logdepthbuf_vertex>\n\t#include <clipping_planes_vertex>\n\t#include <fog_vertex>\n}"},Ro={basic:{uniforms:Co([ko.common,ko.specularmap,ko.envmap,ko.aomap,ko.lightmap,ko.fog]),vertexShader:jo.meshbasic_vert,fragmentShader:jo.meshbasic_frag},lambert:{uniforms:Co([ko.common,ko.specularmap,ko.envmap,ko.aomap,ko.lightmap,ko.emissivemap,ko.fog,ko.lights,{emissive:{value:new vi(0)}}]),vertexShader:jo.meshlambert_vert,fragmentShader:jo.meshlambert_frag},phong:{uniforms:Co([ko.common,ko.specularmap,ko.envmap,ko.aomap,ko.lightmap,ko.emissivemap,ko.bumpmap,ko.normalmap,ko.displacementmap,ko.fog,ko.lights,{emissive:{value:new vi(0)},specular:{value:new vi(1118481)},shininess:{value:30}}]),vertexShader:jo.meshphong_vert,fragmentShader:jo.meshphong_frag},standard:{uniforms:Co([ko.common,ko.envmap,ko.aomap,ko.lightmap,ko.emissivemap,ko.bumpmap,ko.normalmap,ko.displacementmap,ko.roughnessmap,ko.metalnessmap,ko.fog,ko.lights,{emissive:{value:new vi(0)},roughness:{value:1},metalness:{value:0},envMapIntensity:{value:1}}]),vertexShader:jo.meshphysical_vert,fragmentShader:jo.meshphysical_frag},toon:{uniforms:Co([ko.common,ko.specularmap,ko.aomap,ko.lightmap,ko.emissivemap,ko.bumpmap,ko.normalmap,ko.displacementmap,ko.gradientmap,ko.fog,ko.lights,{emissive:{value:new vi(0)},specular:{value:new vi(1118481)},shininess:{value:30}}]),vertexShader:jo.meshtoon_vert,fragmentShader:jo.meshtoon_frag},matcap:{uniforms:Co([ko.common,ko.bumpmap,ko.normalmap,ko.displacementmap,ko.fog,{matcap:{value:null}}]),vertexShader:jo.meshmatcap_vert,fragmentShader:jo.meshmatcap_frag},points:{uniforms:Co([ko.points,ko.fog]),vertexShader:jo.points_vert,fragmentShader:jo.points_frag},dashed:{uniforms:Co([ko.common,ko.fog,{scale:{value:1},dashSize:{value:1},totalSize:{value:2}}]),vertexShader:jo.linedashed_vert,fragmentShader:jo.linedashed_frag},depth:{uniforms:Co([ko.common,ko.displacementmap]),vertexShader:jo.depth_vert,fragmentShader:jo.depth_frag},normal:{uniforms:Co([ko.common,ko.bumpmap,ko.normalmap,ko.displacementmap,{opacity:{value:1}}]),vertexShader:jo.normal_vert,fragmentShader:jo.normal_frag},sprite:{uniforms:Co([ko.sprite,ko.fog]),vertexShader:jo.sprite_vert,fragmentShader:jo.sprite_frag},background:{uniforms:{uvTransform:{value:new Vn},t2D:{value:null}},vertexShader:jo.background_vert,fragmentShader:jo.background_frag},cube:{uniforms:Co([ko.envmap,{opacity:{value:1}}]),vertexShader:jo.cube_vert,fragmentShader:jo.cube_frag},equirect:{uniforms:{tEquirect:{value:null}},vertexShader:jo.equirect_vert,fragmentShader:jo.equirect_frag},distanceRGBA:{uniforms:Co([ko.common,ko.displacementmap,{referencePosition:{value:new $n},nearDistance:{value:1},farDistance:{value:1e3}}]),vertexShader:jo.distanceRGBA_vert,fragmentShader:jo.distanceRGBA_frag},shadow:{uniforms:Co([ko.lights,ko.fog,{color:{value:new vi(0)},opacity:{value:1}}]),vertexShader:jo.shadow_vert,fragmentShader:jo.shadow_frag}};function zo(e,t,n,r){var i,o,a=new vi(0),s=0,l=null,u=0,c=null;function d(e,n){t.buffers.color.setClear(e.r,e.g,e.b,n,r)}return{getClearColor:function(){return a},setClearColor:function(e,t){a.set(e),d(a,s=void 0!==t?t:1)},getClearAlpha:function(){return s},setClearAlpha:function(e){d(a,s=e)},render:function(t,r,h,f){var p=r.background,g=e.xr,y=g.getSession&&g.getSession();if(y&&"additive"===y.environmentBlendMode&&(p=null),null===p?d(a,s):p&&p.isColor&&(d(p,1),f=!0),(e.autoClear||f)&&e.clear(e.autoClearColor,e.autoClearDepth,e.autoClearStencil),p&&(p.isCubeTexture||p.isWebGLCubeRenderTarget||p.mapping===he)){void 0===o&&((o=new lo(new yo(1,1,1),new _o({type:"BackgroundCubeMaterial",uniforms:bo(Ro.cube.uniforms),vertexShader:Ro.cube.vertexShader,fragmentShader:Ro.cube.fragmentShader,side:m,depthTest:!1,depthWrite:!1,fog:!1}))).geometry.deleteAttribute("normal"),o.geometry.deleteAttribute("uv"),o.onBeforeRender=function(e,t,n){this.matrixWorld.copyPosition(n.matrixWorld)},Object.defineProperty(o.material,"envMap",{get:function(){return this.uniforms.envMap.value}}),n.update(o));var b=p.isWebGLCubeRenderTarget?p.texture:p;o.material.uniforms.envMap.value=b,o.material.uniforms.flipEnvMap.value=b.isCubeTexture?-1:1,l===p&&u===b.version&&c===e.toneMapping||(o.material.needsUpdate=!0,l=p,u=b.version,c=e.toneMapping),t.unshift(o,o.geometry,o.material,0,0,null)}else p&&p.isTexture&&(void 0===i&&((i=new lo(new No(2,2),new _o({type:"BackgroundMaterial",uniforms:bo(Ro.background.uniforms),vertexShader:Ro.background.vertexShader,fragmentShader:Ro.background.fragmentShader,side:v,depthTest:!1,depthWrite:!1,fog:!1}))).geometry.deleteAttribute("normal"),Object.defineProperty(i.material,"map",{get:function(){return this.uniforms.t2D.value}}),n.update(i)),i.material.uniforms.t2D.value=p,!0===p.matrixAutoUpdate&&p.updateMatrix(),i.material.uniforms.uvTransform.value.copy(p.matrix),l===p&&u===p.version&&c===e.toneMapping||(i.material.needsUpdate=!0,l=p,u=p.version,c=e.toneMapping),t.unshift(i,i.geometry,i.material,0,0,null))}}}function Fo(e,t,n,r){var i,o=r.isWebGL2;this.setMode=function(e){i=e},this.render=function(t,r){e.drawArrays(i,t,r),n.update(r,i)},this.renderInstances=function(r,a,s,l){if(0!==l){var u,c;if(o)u=e,c="drawArraysInstanced";else if(c="drawArraysInstancedANGLE",null===(u=t.get("ANGLE_instanced_arrays")))return void console.error("THREE.WebGLBufferRenderer: using THREE.InstancedBufferGeometry but hardware does not support extension ANGLE_instanced_arrays.");u[c](i,a,s,l),n.update(s,i,l)}}}function Bo(e,t,n){var r;function i(t){if("highp"===t){if(e.getShaderPrecisionFormat(35633,36338).precision>0&&e.getShaderPrecisionFormat(35632,36338).precision>0)return"highp";t="mediump"}return"mediump"===t&&e.getShaderPrecisionFormat(35633,36337).precision>0&&e.getShaderPrecisionFormat(35632,36337).precision>0?"mediump":"lowp"}var o="undefined"!=typeof WebGL2RenderingContext&&e instanceof WebGL2RenderingContext||"undefined"!=typeof WebGL2ComputeRenderingContext&&e instanceof WebGL2ComputeRenderingContext,a=void 0!==n.precision?n.precision:"highp",s=i(a);s!==a&&(console.warn("THREE.WebGLRenderer:",a,"not supported, using",s,"instead."),a=s);var l=!0===n.logarithmicDepthBuffer,u=e.getParameter(34930),c=e.getParameter(35660),d=e.getParameter(3379),h=e.getParameter(34076),f=e.getParameter(34921),p=e.getParameter(36347),g=e.getParameter(36348),v=e.getParameter(36349),m=c>0,y=o||!!t.get("OES_texture_float");return{isWebGL2:o,getMaxAnisotropy:function(){if(void 0!==r)return r;var n=t.get("EXT_texture_filter_anisotropic");return r=null!==n?e.getParameter(n.MAX_TEXTURE_MAX_ANISOTROPY_EXT):0},getMaxPrecision:i,precision:a,logarithmicDepthBuffer:l,maxTextures:u,maxVertexTextures:c,maxTextureSize:d,maxCubemapSize:h,maxAttributes:f,maxVertexUniforms:p,maxVaryings:g,maxFragmentUniforms:v,vertexTextures:m,floatFragmentTextures:y,floatVertexTextures:m&&y,maxSamples:o?e.getParameter(36183):0}}function Ho(){var e=this,t=null,n=0,r=!1,i=!1,o=new ti,a=new Vn,s={value:null,needsUpdate:!1};function l(){s.value!==t&&(s.value=t,s.needsUpdate=n>0),e.numPlanes=n,e.numIntersection=0}function u(t,n,r,i){var l=null!==t?t.length:0,u=null;if(0!==l){if(u=s.value,!0!==i||null===u){var c=r+4*l,d=n.matrixWorldInverse;a.getNormalMatrix(d),(null===u||u.length<c)&&(u=new Float32Array(c));for(var h=0,f=r;h!==l;++h,f+=4)o.copy(t[h]).applyMatrix4(d,a),o.normal.toArray(u,f),u[f+3]=o.constant}s.value=u,s.needsUpdate=!0}return e.numPlanes=l,e.numIntersection=0,u}this.uniform=s,this.numPlanes=0,this.numIntersection=0,this.init=function(e,i,o){var a=0!==e.length||i||0!==n||r;return r=i,t=u(e,o,0),n=e.length,a},this.beginShadows=function(){i=!0,u(null)},this.endShadows=function(){i=!1,l()},this.setState=function(e,o,a,c,d,h){if(!r||null===e||0===e.length||i&&!a)i?u(null):l();else{var f=i?0:n,p=4*f,g=d.clippingState||null;s.value=g,g=u(e,c,p,h);for(var v=0;v!==p;++v)g[v]=t[v];d.clippingState=g,this.numIntersection=o?this.numPlanes:0,this.numPlanes+=f}}}function Uo(e){var t={};return{get:function(n){if(void 0!==t[n])return t[n];var r;switch(n){case"WEBGL_depth_texture":r=e.getExtension("WEBGL_depth_texture")||e.getExtension("MOZ_WEBGL_depth_texture")||e.getExtension("WEBKIT_WEBGL_depth_texture");break;case"EXT_texture_filter_anisotropic":r=e.getExtension("EXT_texture_filter_anisotropic")||e.getExtension("MOZ_EXT_texture_filter_anisotropic")||e.getExtension("WEBKIT_EXT_texture_filter_anisotropic");break;case"WEBGL_compressed_texture_s3tc":r=e.getExtension("WEBGL_compressed_texture_s3tc")||e.getExtension("MOZ_WEBGL_compressed_texture_s3tc")||e.getExtension("WEBKIT_WEBGL_compressed_texture_s3tc");break;case"WEBGL_compressed_texture_pvrtc":r=e.getExtension("WEBGL_compressed_texture_pvrtc")||e.getExtension("WEBKIT_WEBGL_compressed_texture_pvrtc");break;default:r=e.getExtension(n)}return null===r&&console.warn("THREE.WebGLRenderer: "+n+" extension not supported."),t[n]=r,r}}}function Vo(e,t,n){var r=new WeakMap,i=new WeakMap;function o(e){var a=e.target,s=r.get(a);for(var l in null!==s.index&&t.remove(s.index),s.attributes)t.remove(s.attributes[l]);a.removeEventListener("dispose",o),r.delete(a);var u=i.get(s);u&&(t.remove(u),i.delete(s)),n.memory.geometries--}function a(e){var n=[],r=e.index,o=e.attributes.position,a=0;if(null!==r){var s=r.array;a=r.version;for(var l=0,u=s.length;l<u;l+=3){var c=s[l+0],d=s[l+1],h=s[l+2];n.push(c,d,d,h,h,c)}}else{s=o.array;a=o.version;for(l=0,u=s.length/3-1;l<u;l+=3){c=l+0,d=l+1,h=l+2;n.push(c,d,d,h,h,c)}}var f=new(ji(n)>65535?Li:Oi)(n,1);f.version=a,t.update(f,34963);var p=i.get(e);p&&t.remove(p),i.set(e,f)}return{get:function(e,t){var i=r.get(t);return i||(t.addEventListener("dispose",o),t.isBufferGeometry?i=t:t.isGeometry&&(void 0===t._bufferGeometry&&(t._bufferGeometry=(new Yi).setFromObject(e)),i=t._bufferGeometry),r.set(t,i),n.memory.geometries++,i)},update:function(e){var n=e.index,r=e.attributes;for(var i in null!==n&&t.update(n,34963),r)t.update(r[i],34962);var o=e.morphAttributes;for(var i in o)for(var a=o[i],s=0,l=a.length;s<l;s++)t.update(a[s],34962)},getWireframeAttribute:function(e){var t=i.get(e);if(t){var n=e.index;null!==n&&t.version<n.version&&a(e)}else a(e);return i.get(e)}}}function Yo(e,t,n,r){var i,o,a,s=r.isWebGL2;this.setMode=function(e){i=e},this.setIndex=function(e){o=e.type,a=e.bytesPerElement},this.render=function(t,r){e.drawElements(i,r,o,t*a),n.update(r,i)},this.renderInstances=function(r,l,u,c){if(0!==c){var d,h;if(s)d=e,h="drawElementsInstanced";else if(h="drawElementsInstancedANGLE",null===(d=t.get("ANGLE_instanced_arrays")))return void console.error("THREE.WebGLIndexedBufferRenderer: using THREE.InstancedBufferGeometry but hardware does not support extension ANGLE_instanced_arrays.");d[h](i,u,o,l*a,c),n.update(u,i,c)}}}function Wo(e){var t={frame:0,calls:0,triangles:0,points:0,lines:0};return{memory:{geometries:0,textures:0},render:t,programs:null,autoReset:!0,reset:function(){t.frame++,t.calls=0,t.triangles=0,t.points=0,t.lines=0},update:function(e,n,r){switch(r=r||1,t.calls++,n){case 4:t.triangles+=r*(e/3);break;case 1:t.lines+=r*(e/2);break;case 3:t.lines+=r*(e-1);break;case 2:t.lines+=r*e;break;case 0:t.points+=r*e;break;default:console.error("THREE.WebGLInfo: Unknown draw mode:",n)}}}}function Go(e,t){return Math.abs(t[1])-Math.abs(e[1])}function Xo(e){var t={},n=new Float32Array(8);return{update:function(r,i,o,a){var s=r.morphTargetInfluences,l=void 0===s?0:s.length,u=t[i.id];if(void 0===u){u=[];for(var c=0;c<l;c++)u[c]=[c,0];t[i.id]=u}var d=o.morphTargets&&i.morphAttributes.position,h=o.morphNormals&&i.morphAttributes.normal;for(c=0;c<l;c++){0!==(p=u[c])[1]&&(d&&i.deleteAttribute("morphTarget"+c),h&&i.deleteAttribute("morphNormal"+c))}for(c=0;c<l;c++){(p=u[c])[0]=c,p[1]=s[c]}u.sort(Go);var f=0;for(c=0;c<8;c++){var p;if(p=u[c]){var g=p[0],v=p[1];if(v){d&&i.setAttribute("morphTarget"+c,d[g]),h&&i.setAttribute("morphNormal"+c,h[g]),n[c]=v,f+=v;continue}}n[c]=0}var m=i.morphTargetsRelative?1:1-f;a.getUniforms().setValue(e,"morphTargetBaseInfluence",m),a.getUniforms().setValue(e,"morphTargetInfluences",n)}}}function qo(e,t,n,r){var i=new WeakMap;return{update:function(e){var o=r.render.frame,a=e.geometry,s=t.get(e,a);return i.get(s)!==o&&(a.isGeometry&&s.updateFromObject(e),t.update(s),i.set(s,o)),e.isInstancedMesh&&n.update(e.instanceMatrix,34962),s},dispose:function(){i=new WeakMap}}}function Zo(e,t,n,r,i,o,a,s,l,u){e=void 0!==e?e:[],t=void 0!==t?t:se,a=void 0!==a?a:Be,Gn.call(this,e,t,n,r,i,o,a,s,l,u),this.flipY=!1}function Qo(e,t,n,r){Gn.call(this,null),this.image={data:e||null,width:t||1,height:n||1,depth:r||1},this.magFilter=me,this.minFilter=me,this.wrapR=ge,this.generateMipmaps=!1,this.flipY=!1,this.needsUpdate=!0}function Ko(e,t,n,r){Gn.call(this,null),this.image={data:e||null,width:t||1,height:n||1,depth:r||1},this.magFilter=me,this.minFilter=me,this.wrapR=ge,this.generateMipmaps=!1,this.flipY=!1,this.needsUpdate=!0}Ro.physical={uniforms:Co([Ro.standard.uniforms,{clearcoat:{value:0},clearcoatMap:{value:null},clearcoatRoughness:{value:0},clearcoatRoughnessMap:{value:null},clearcoatNormalScale:{value:new Un(1,1)},clearcoatNormalMap:{value:null},sheen:{value:new vi(0)},transparency:{value:0}}]),vertexShader:jo.meshphysical_vert,fragmentShader:jo.meshphysical_frag},Zo.prototype=Object.create(Gn.prototype),Zo.prototype.constructor=Zo,Zo.prototype.isCubeTexture=!0,Object.defineProperty(Zo.prototype,"images",{get:function(){return this.image},set:function(e){this.image=e}}),Qo.prototype=Object.create(Gn.prototype),Qo.prototype.constructor=Qo,Qo.prototype.isDataTexture2DArray=!0,Ko.prototype=Object.create(Gn.prototype),Ko.prototype.constructor=Ko,Ko.prototype.isDataTexture3D=!0;var Jo=new Gn,$o=new Qo,ea=new Ko,ta=new Zo,na=[],ra=[],ia=new Float32Array(16),oa=new Float32Array(9),aa=new Float32Array(4);function sa(e,t,n){var r=e[0];if(r<=0||r>0)return e;var i=t*n,o=na[i];if(void 0===o&&(o=new Float32Array(i),na[i]=o),0!==t){r.toArray(o,0);for(var a=1,s=0;a!==t;++a)s+=n,e[a].toArray(o,s)}return o}function la(e,t){if(e.length!==t.length)return!1;for(var n=0,r=e.length;n<r;n++)if(e[n]!==t[n])return!1;return!0}function ua(e,t){for(var n=0,r=t.length;n<r;n++)e[n]=t[n]}function ca(e,t){var n=ra[t];void 0===n&&(n=new Int32Array(t),ra[t]=n);for(var r=0;r!==t;++r)n[r]=e.allocateTextureUnit();return n}function da(e,t){var n=this.cache;n[0]!==t&&(e.uniform1f(this.addr,t),n[0]=t)}function ha(e,t){var n=this.cache;if(void 0!==t.x)n[0]===t.x&&n[1]===t.y||(e.uniform2f(this.addr,t.x,t.y),n[0]=t.x,n[1]=t.y);else{if(la(n,t))return;e.uniform2fv(this.addr,t),ua(n,t)}}function fa(e,t){var n=this.cache;if(void 0!==t.x)n[0]===t.x&&n[1]===t.y&&n[2]===t.z||(e.uniform3f(this.addr,t.x,t.y,t.z),n[0]=t.x,n[1]=t.y,n[2]=t.z);else if(void 0!==t.r)n[0]===t.r&&n[1]===t.g&&n[2]===t.b||(e.uniform3f(this.addr,t.r,t.g,t.b),n[0]=t.r,n[1]=t.g,n[2]=t.b);else{if(la(n,t))return;e.uniform3fv(this.addr,t),ua(n,t)}}function pa(e,t){var n=this.cache;if(void 0!==t.x)n[0]===t.x&&n[1]===t.y&&n[2]===t.z&&n[3]===t.w||(e.uniform4f(this.addr,t.x,t.y,t.z,t.w),n[0]=t.x,n[1]=t.y,n[2]=t.z,n[3]=t.w);else{if(la(n,t))return;e.uniform4fv(this.addr,t),ua(n,t)}}function ga(e,t){var n=this.cache,r=t.elements;if(void 0===r){if(la(n,t))return;e.uniformMatrix2fv(this.addr,!1,t),ua(n,t)}else{if(la(n,r))return;aa.set(r),e.uniformMatrix2fv(this.addr,!1,aa),ua(n,r)}}function va(e,t){var n=this.cache,r=t.elements;if(void 0===r){if(la(n,t))return;e.uniformMatrix3fv(this.addr,!1,t),ua(n,t)}else{if(la(n,r))return;oa.set(r),e.uniformMatrix3fv(this.addr,!1,oa),ua(n,r)}}function ma(e,t){var n=this.cache,r=t.elements;if(void 0===r){if(la(n,t))return;e.uniformMatrix4fv(this.addr,!1,t),ua(n,t)}else{if(la(n,r))return;ia.set(r),e.uniformMatrix4fv(this.addr,!1,ia),ua(n,r)}}function ya(e,t,n){var r=this.cache,i=n.allocateTextureUnit();r[0]!==i&&(e.uniform1i(this.addr,i),r[0]=i),n.safeSetTexture2D(t||Jo,i)}function ba(e,t,n){var r=this.cache,i=n.allocateTextureUnit();r[0]!==i&&(e.uniform1i(this.addr,i),r[0]=i),n.setTexture2DArray(t||$o,i)}function Ca(e,t,n){var r=this.cache,i=n.allocateTextureUnit();r[0]!==i&&(e.uniform1i(this.addr,i),r[0]=i),n.setTexture3D(t||ea,i)}function Aa(e,t,n){var r=this.cache,i=n.allocateTextureUnit();r[0]!==i&&(e.uniform1i(this.addr,i),r[0]=i),n.safeSetTextureCube(t||ta,i)}function _a(e,t){var n=this.cache;n[0]!==t&&(e.uniform1i(this.addr,t),n[0]=t)}function Ia(e,t){var n=this.cache;la(n,t)||(e.uniform2iv(this.addr,t),ua(n,t))}function xa(e,t){var n=this.cache;la(n,t)||(e.uniform3iv(this.addr,t),ua(n,t))}function wa(e,t){var n=this.cache;la(n,t)||(e.uniform4iv(this.addr,t),ua(n,t))}function Ma(e,t){var n=this.cache;n[0]!==t&&(e.uniform1ui(this.addr,t),n[0]=t)}function Sa(e,t){e.uniform1fv(this.addr,t)}function Ta(e,t){e.uniform1iv(this.addr,t)}function Ea(e,t){e.uniform2iv(this.addr,t)}function Oa(e,t){e.uniform3iv(this.addr,t)}function ka(e,t){e.uniform4iv(this.addr,t)}function La(e,t){var n=sa(t,this.size,2);e.uniform2fv(this.addr,n)}function Da(e,t){var n=sa(t,this.size,3);e.uniform3fv(this.addr,n)}function Pa(e,t){var n=sa(t,this.size,4);e.uniform4fv(this.addr,n)}function Na(e,t){var n=sa(t,this.size,4);e.uniformMatrix2fv(this.addr,!1,n)}function ja(e,t){var n=sa(t,this.size,9);e.uniformMatrix3fv(this.addr,!1,n)}function Ra(e,t){var n=sa(t,this.size,16);e.uniformMatrix4fv(this.addr,!1,n)}function za(e,t,n){var r=t.length,i=ca(n,r);e.uniform1iv(this.addr,i);for(var o=0;o!==r;++o)n.safeSetTexture2D(t[o]||Jo,i[o])}function Fa(e,t,n){var r=t.length,i=ca(n,r);e.uniform1iv(this.addr,i);for(var o=0;o!==r;++o)n.safeSetTextureCube(t[o]||ta,i[o])}function Ba(e,t,n){this.id=e,this.addr=n,this.cache=[],this.setValue=function(e){switch(e){case 5126:return da;case 35664:return ha;case 35665:return fa;case 35666:return pa;case 35674:return ga;case 35675:return va;case 35676:return ma;case 5124:case 35670:return _a;case 35667:case 35671:return Ia;case 35668:case 35672:return xa;case 35669:case 35673:return wa;case 5125:return Ma;case 35678:case 36198:case 36298:case 36306:case 35682:return ya;case 35679:case 36299:case 36307:return Ca;case 35680:case 36300:case 36308:case 36293:return Aa;case 36289:case 36303:case 36311:case 36292:return ba}}(t.type)}function Ha(e,t,n){this.id=e,this.addr=n,this.cache=[],this.size=t.size,this.setValue=function(e){switch(e){case 5126:return Sa;case 35664:return La;case 35665:return Da;case 35666:return Pa;case 35674:return Na;case 35675:return ja;case 35676:return Ra;case 5124:case 35670:return Ta;case 35667:case 35671:return Ea;case 35668:case 35672:return Oa;case 35669:case 35673:return ka;case 35678:case 36198:case 36298:case 36306:case 35682:return za;case 35680:case 36300:case 36308:case 36293:return Fa}}(t.type)}function Ua(e){this.id=e,this.seq=[],this.map={}}Ha.prototype.updateCache=function(e){var t=this.cache;e instanceof Float32Array&&t.length!==e.length&&(this.cache=new Float32Array(e.length)),ua(t,e)},Ua.prototype.setValue=function(e,t,n){for(var r=this.seq,i=0,o=r.length;i!==o;++i){var a=r[i];a.setValue(e,t[a.id],n)}};var Va=/([\w\d_]+)(\])?(\[|\.)?/g;function Ya(e,t){e.seq.push(t),e.map[t.id]=t}function Wa(e,t,n){var r=e.name,i=r.length;for(Va.lastIndex=0;;){var o=Va.exec(r),a=Va.lastIndex,s=o[1],l="]"===o[2],u=o[3];if(l&&(s|=0),void 0===u||"["===u&&a+2===i){Ya(n,void 0===u?new Ba(s,e,t):new Ha(s,e,t));break}var c=n.map[s];void 0===c&&Ya(n,c=new Ua(s)),n=c}}function Ga(e,t){this.seq=[],this.map={};for(var n=e.getProgramParameter(t,35718),r=0;r<n;++r){var i=e.getActiveUniform(t,r);Wa(i,e.getUniformLocation(t,i.name),this)}}function Xa(e,t,n){var r=e.createShader(t);return e.shaderSource(r,n),e.compileShader(r),r}Ga.prototype.setValue=function(e,t,n,r){var i=this.map[t];void 0!==i&&i.setValue(e,n,r)},Ga.prototype.setOptional=function(e,t,n){var r=t[n];void 0!==r&&this.setValue(e,n,r)},Ga.upload=function(e,t,n,r){for(var i=0,o=t.length;i!==o;++i){var a=t[i],s=n[a.id];!1!==s.needsUpdate&&a.setValue(e,s.value,r)}},Ga.seqWithValue=function(e,t){for(var n=[],r=0,i=e.length;r!==i;++r){var o=e[r];o.id in t&&n.push(o)}return n};var qa=0;function Za(e){switch(e){case $t:return["Linear","( value )"];case en:return["sRGB","( value )"];case nn:return["RGBE","( value )"];case on:return["RGBM","( value, 7.0 )"];case an:return["RGBM","( value, 16.0 )"];case sn:return["RGBD","( value, 256.0 )"];case tn:return["Gamma","( value, float( GAMMA_FACTOR ) )"];case rn:return["LogLuv","( value )"];default:throw new Error("unsupported encoding: "+e)}}function Qa(e,t,n){var r=e.getShaderParameter(t,35713),i=e.getShaderInfoLog(t).trim();return r&&""===i?"":"THREE.WebGLShader: gl.getShaderInfoLog() "+n+"\n"+i+function(e){for(var t=e.split("\n"),n=0;n<t.length;n++)t[n]=n+1+": "+t[n];return t.join("\n")}(e.getShaderSource(t))}function Ka(e,t){var n=Za(t);return"vec4 "+e+"( vec4 value ) { return "+n[0]+"ToLinear"+n[1]+"; }"}function Ja(e,t){var n;switch(t){case te:n="Linear";break;case ne:n="Reinhard";break;case re:n="Uncharted2";break;case ie:n="OptimizedCineon";break;case oe:n="ACESFilmic";break;default:throw new Error("unsupported toneMapping: "+t)}return"vec3 "+e+"( vec3 color ) { return "+n+"ToneMapping( color ); }"}function $a(e){return""!==e}function es(e,t){return e.replace(/NUM_DIR_LIGHTS/g,t.numDirLights).replace(/NUM_SPOT_LIGHTS/g,t.numSpotLights).replace(/NUM_RECT_AREA_LIGHTS/g,t.numRectAreaLights).replace(/NUM_POINT_LIGHTS/g,t.numPointLights).replace(/NUM_HEMI_LIGHTS/g,t.numHemiLights).replace(/NUM_DIR_LIGHT_SHADOWS/g,t.numDirLightShadows).replace(/NUM_SPOT_LIGHT_SHADOWS/g,t.numSpotLightShadows).replace(/NUM_POINT_LIGHT_SHADOWS/g,t.numPointLightShadows)}function ts(e,t){return e.replace(/NUM_CLIPPING_PLANES/g,t.numClippingPlanes).replace(/UNION_CLIPPING_PLANES/g,t.numClippingPlanes-t.numClipIntersection)}var ns=/^[ \t]*#include +<([\w\d./]+)>/gm;function rs(e){return e.replace(ns,is)}function is(e,t){var n=jo[t];if(void 0===n)throw new Error("Can not resolve #include <"+t+">");return rs(n)}var os=/#pragma unroll_loop[\s]+?for \( int i \= (\d+)\; i < (\d+)\; i \+\+ \) \{([\s\S]+?)(?=\})\}/g,as=/#pragma unroll_loop_start[\s]+?for \( int i \= (\d+)\; i < (\d+)\; i \+\+ \) \{([\s\S]+?)(?=\})\}[\s]+?#pragma unroll_loop_end/g;function ss(e){return e.replace(as,us).replace(os,ls)}function ls(e,t,n,r){return console.warn("WebGLProgram: #pragma unroll_loop shader syntax is deprecated. Please use #pragma unroll_loop_start syntax instead."),us(e,t,n,r)}function us(e,t,n,r){for(var i="",o=parseInt(t);o<parseInt(n);o++)i+=r.replace(/\[ i \]/g,"[ "+o+" ]").replace(/UNROLLED_LOOP_INDEX/g,o);return i}function cs(e){var t="precision "+e.precision+" float;\nprecision "+e.precision+" int;";return"highp"===e.precision?t+="\n#define HIGH_PRECISION":"mediump"===e.precision?t+="\n#define MEDIUM_PRECISION":"lowp"===e.precision&&(t+="\n#define LOW_PRECISION"),t}function ds(e,t,n){var r,i,o,a,s,l=e.getContext(),u=n.defines,c=n.vertexShader,d=n.fragmentShader,h=function(e){var t="SHADOWMAP_TYPE_BASIC";return e.shadowMapType===f?t="SHADOWMAP_TYPE_PCF":e.shadowMapType===p?t="SHADOWMAP_TYPE_PCF_SOFT":e.shadowMapType===g&&(t="SHADOWMAP_TYPE_VSM"),t}(n),v=function(e){var t="ENVMAP_TYPE_CUBE";if(e.envMap)switch(e.envMapMode){case se:case le:t="ENVMAP_TYPE_CUBE";break;case he:case fe:t="ENVMAP_TYPE_CUBE_UV";break;case ue:case ce:t="ENVMAP_TYPE_EQUIREC";break;case de:t="ENVMAP_TYPE_SPHERE"}return t}(n),m=function(e){var t="ENVMAP_MODE_REFLECTION";if(e.envMap)switch(e.envMapMode){case le:case ce:t="ENVMAP_MODE_REFRACTION"}return t}(n),y=function(e){var t="ENVMAP_BLENDING_NONE";if(e.envMap)switch(e.combine){case K:t="ENVMAP_BLENDING_MULTIPLY";break;case J:t="ENVMAP_BLENDING_MIX";break;case $:t="ENVMAP_BLENDING_ADD"}return t}(n),b=e.gammaFactor>0?e.gammaFactor:1,C=n.isWebGL2?"":function(e){return[e.extensionDerivatives||e.envMapCubeUV||e.bumpMap||e.tangentSpaceNormalMap||e.clearcoatNormalMap||e.flatShading||"physical"===e.shaderID?"#extension GL_OES_standard_derivatives : enable":"",(e.extensionFragDepth||e.logarithmicDepthBuffer)&&e.rendererExtensionFragDepth?"#extension GL_EXT_frag_depth : enable":"",e.extensionDrawBuffers&&e.rendererExtensionDrawBuffers?"#extension GL_EXT_draw_buffers : require":"",(e.extensionShaderTextureLOD||e.envMap)&&e.rendererExtensionShaderTextureLod?"#extension GL_EXT_shader_texture_lod : enable":""].filter($a).join("\n")}(n),A=function(e){var t=[];for(var n in e){var r=e[n];!1!==r&&t.push("#define "+n+" "+r)}return t.join("\n")}(u),_=l.createProgram();if(n.isRawShaderMaterial?((r=[A].filter($a).join("\n")).length>0&&(r+="\n"),(i=[C,A].filter($a).join("\n")).length>0&&(i+="\n")):(r=[cs(n),"#define SHADER_NAME "+n.shaderName,A,n.instancing?"#define USE_INSTANCING":"",n.supportsVertexTextures?"#define VERTEX_TEXTURES":"","#define GAMMA_FACTOR "+b,"#define MAX_BONES "+n.maxBones,n.useFog&&n.fog?"#define USE_FOG":"",n.useFog&&n.fogExp2?"#define FOG_EXP2":"",n.map?"#define USE_MAP":"",n.envMap?"#define USE_ENVMAP":"",n.envMap?"#define "+m:"",n.lightMap?"#define USE_LIGHTMAP":"",n.aoMap?"#define USE_AOMAP":"",n.emissiveMap?"#define USE_EMISSIVEMAP":"",n.bumpMap?"#define USE_BUMPMAP":"",n.normalMap?"#define USE_NORMALMAP":"",n.normalMap&&n.objectSpaceNormalMap?"#define OBJECTSPACE_NORMALMAP":"",n.normalMap&&n.tangentSpaceNormalMap?"#define TANGENTSPACE_NORMALMAP":"",n.clearcoatMap?"#define USE_CLEARCOATMAP":"",n.clearcoatRoughnessMap?"#define USE_CLEARCOAT_ROUGHNESSMAP":"",n.clearcoatNormalMap?"#define USE_CLEARCOAT_NORMALMAP":"",n.displacementMap&&n.supportsVertexTextures?"#define USE_DISPLACEMENTMAP":"",n.specularMap?"#define USE_SPECULARMAP":"",n.roughnessMap?"#define USE_ROUGHNESSMAP":"",n.metalnessMap?"#define USE_METALNESSMAP":"",n.alphaMap?"#define USE_ALPHAMAP":"",n.vertexTangents?"#define USE_TANGENT":"",n.vertexColors?"#define USE_COLOR":"",n.vertexUvs?"#define USE_UV":"",n.uvsVertexOnly?"#define UVS_VERTEX_ONLY":"",n.flatShading?"#define FLAT_SHADED":"",n.skinning?"#define USE_SKINNING":"",n.useVertexTexture?"#define BONE_TEXTURE":"",n.morphTargets?"#define USE_MORPHTARGETS":"",n.morphNormals&&!1===n.flatShading?"#define USE_MORPHNORMALS":"",n.doubleSided?"#define DOUBLE_SIDED":"",n.flipSided?"#define FLIP_SIDED":"",n.shadowMapEnabled?"#define USE_SHADOWMAP":"",n.shadowMapEnabled?"#define "+h:"",n.sizeAttenuation?"#define USE_SIZEATTENUATION":"",n.logarithmicDepthBuffer?"#define USE_LOGDEPTHBUF":"",n.logarithmicDepthBuffer&&n.rendererExtensionFragDepth?"#define USE_LOGDEPTHBUF_EXT":"","uniform mat4 modelMatrix;","uniform mat4 modelViewMatrix;","uniform mat4 projectionMatrix;","uniform mat4 viewMatrix;","uniform mat3 normalMatrix;","uniform vec3 cameraPosition;","uniform bool isOrthographic;","#ifdef USE_INSTANCING"," attribute mat4 instanceMatrix;","#endif","attribute vec3 position;","attribute vec3 normal;","attribute vec2 uv;","#ifdef USE_TANGENT","\tattribute vec4 tangent;","#endif","#ifdef USE_COLOR","\tattribute vec3 color;","#endif","#ifdef USE_MORPHTARGETS","\tattribute vec3 morphTarget0;","\tattribute vec3 morphTarget1;","\tattribute vec3 morphTarget2;","\tattribute vec3 morphTarget3;","\t#ifdef USE_MORPHNORMALS","\t\tattribute vec3 morphNormal0;","\t\tattribute vec3 morphNormal1;","\t\tattribute vec3 morphNormal2;","\t\tattribute vec3 morphNormal3;","\t#else","\t\tattribute vec3 morphTarget4;","\t\tattribute vec3 morphTarget5;","\t\tattribute vec3 morphTarget6;","\t\tattribute vec3 morphTarget7;","\t#endif","#endif","#ifdef USE_SKINNING","\tattribute vec4 skinIndex;","\tattribute vec4 skinWeight;","#endif","\n"].filter($a).join("\n"),i=[C,cs(n),"#define SHADER_NAME "+n.shaderName,A,n.alphaTest?"#define ALPHATEST "+n.alphaTest+(n.alphaTest%1?"":".0"):"","#define GAMMA_FACTOR "+b,n.useFog&&n.fog?"#define USE_FOG":"",n.useFog&&n.fogExp2?"#define FOG_EXP2":"",n.map?"#define USE_MAP":"",n.matcap?"#define USE_MATCAP":"",n.envMap?"#define USE_ENVMAP":"",n.envMap?"#define "+v:"",n.envMap?"#define "+m:"",n.envMap?"#define "+y:"",n.lightMap?"#define USE_LIGHTMAP":"",n.aoMap?"#define USE_AOMAP":"",n.emissiveMap?"#define USE_EMISSIVEMAP":"",n.bumpMap?"#define USE_BUMPMAP":"",n.normalMap?"#define USE_NORMALMAP":"",n.normalMap&&n.objectSpaceNormalMap?"#define OBJECTSPACE_NORMALMAP":"",n.normalMap&&n.tangentSpaceNormalMap?"#define TANGENTSPACE_NORMALMAP":"",n.clearcoatMap?"#define USE_CLEARCOATMAP":"",n.clearcoatRoughnessMap?"#define USE_CLEARCOAT_ROUGHNESSMAP":"",n.clearcoatNormalMap?"#define USE_CLEARCOAT_NORMALMAP":"",n.specularMap?"#define USE_SPECULARMAP":"",n.roughnessMap?"#define USE_ROUGHNESSMAP":"",n.metalnessMap?"#define USE_METALNESSMAP":"",n.alphaMap?"#define USE_ALPHAMAP":"",n.sheen?"#define USE_SHEEN":"",n.vertexTangents?"#define USE_TANGENT":"",n.vertexColors?"#define USE_COLOR":"",n.vertexUvs?"#define USE_UV":"",n.uvsVertexOnly?"#define UVS_VERTEX_ONLY":"",n.gradientMap?"#define USE_GRADIENTMAP":"",n.flatShading?"#define FLAT_SHADED":"",n.doubleSided?"#define DOUBLE_SIDED":"",n.flipSided?"#define FLIP_SIDED":"",n.shadowMapEnabled?"#define USE_SHADOWMAP":"",n.shadowMapEnabled?"#define "+h:"",n.premultipliedAlpha?"#define PREMULTIPLIED_ALPHA":"",n.physicallyCorrectLights?"#define PHYSICALLY_CORRECT_LIGHTS":"",n.logarithmicDepthBuffer?"#define USE_LOGDEPTHBUF":"",n.logarithmicDepthBuffer&&n.rendererExtensionFragDepth?"#define USE_LOGDEPTHBUF_EXT":"",(n.extensionShaderTextureLOD||n.envMap)&&n.rendererExtensionShaderTextureLod?"#define TEXTURE_LOD_EXT":"","uniform mat4 viewMatrix;","uniform vec3 cameraPosition;","uniform bool isOrthographic;",n.toneMapping!==ee?"#define TONE_MAPPING":"",n.toneMapping!==ee?jo.tonemapping_pars_fragment:"",n.toneMapping!==ee?Ja("toneMapping",n.toneMapping):"",n.dithering?"#define DITHERING":"",n.outputEncoding||n.mapEncoding||n.matcapEncoding||n.envMapEncoding||n.emissiveMapEncoding||n.lightMapEncoding?jo.encodings_pars_fragment:"",n.mapEncoding?Ka("mapTexelToLinear",n.mapEncoding):"",n.matcapEncoding?Ka("matcapTexelToLinear",n.matcapEncoding):"",n.envMapEncoding?Ka("envMapTexelToLinear",n.envMapEncoding):"",n.emissiveMapEncoding?Ka("emissiveMapTexelToLinear",n.emissiveMapEncoding):"",n.lightMapEncoding?Ka("lightMapTexelToLinear",n.lightMapEncoding):"",n.outputEncoding?(o="linearToOutputTexel",a=n.outputEncoding,s=Za(a),"vec4 "+o+"( vec4 value ) { return LinearTo"+s[0]+s[1]+"; }"):"",n.depthPacking?"#define DEPTH_PACKING "+n.depthPacking:"","\n"].filter($a).join("\n")),c=ts(c=es(c=rs(c),n),n),d=ts(d=es(d=rs(d),n),n),c=ss(c),d=ss(d),n.isWebGL2&&!n.isRawShaderMaterial){var I=!1,x=/^\s*#version\s+300\s+es\s*\n/;n.isShaderMaterial&&null!==c.match(x)&&null!==d.match(x)&&(I=!0,c=c.replace(x,""),d=d.replace(x,"")),r=["#version 300 es\n","#define attribute in","#define varying out","#define texture2D texture"].join("\n")+"\n"+r,i=["#version 300 es\n","#define varying in",I?"":"out highp vec4 pc_fragColor;",I?"":"#define gl_FragColor pc_fragColor","#define gl_FragDepthEXT gl_FragDepth","#define texture2D texture","#define textureCube texture","#define texture2DProj textureProj","#define texture2DLodEXT textureLod","#define texture2DProjLodEXT textureProjLod","#define textureCubeLodEXT textureLod","#define texture2DGradEXT textureGrad","#define texture2DProjGradEXT textureProjGrad","#define textureCubeGradEXT textureGrad"].join("\n")+"\n"+i}var w,M,S=i+d,T=Xa(l,35633,r+c),E=Xa(l,35632,S);if(l.attachShader(_,T),l.attachShader(_,E),void 0!==n.index0AttributeName?l.bindAttribLocation(_,0,n.index0AttributeName):!0===n.morphTargets&&l.bindAttribLocation(_,0,"position"),l.linkProgram(_),e.debug.checkShaderErrors){var O=l.getProgramInfoLog(_).trim(),k=l.getShaderInfoLog(T).trim(),L=l.getShaderInfoLog(E).trim(),D=!0,P=!0;if(!1===l.getProgramParameter(_,35714)){D=!1;var N=Qa(l,T,"vertex"),j=Qa(l,E,"fragment");console.error("THREE.WebGLProgram: shader error: ",l.getError(),"35715",l.getProgramParameter(_,35715),"gl.getProgramInfoLog",O,N,j)}else""!==O?console.warn("THREE.WebGLProgram: gl.getProgramInfoLog()",O):""!==k&&""!==L||(P=!1);P&&(this.diagnostics={runnable:D,programLog:O,vertexShader:{log:k,prefix:r},fragmentShader:{log:L,prefix:i}})}return l.deleteShader(T),l.deleteShader(E),this.getUniforms=function(){return void 0===w&&(w=new Ga(l,_)),w},this.getAttributes=function(){return void 0===M&&(M=function(e,t){for(var n={},r=e.getProgramParameter(t,35721),i=0;i<r;i++){var o=e.getActiveAttrib(t,i).name;n[o]=e.getAttribLocation(t,o)}return n}(l,_)),M},this.destroy=function(){l.deleteProgram(_),this.program=void 0},this.name=n.shaderName,this.id=qa++,this.cacheKey=t,this.usedTimes=1,this.program=_,this.vertexShader=T,this.fragmentShader=E,this}function hs(e,t,n){var r=[],i=n.isWebGL2,o=n.logarithmicDepthBuffer,a=n.floatVertexTextures,s=n.precision,l=n.maxVertexUniforms,u=n.vertexTextures,c={MeshDepthMaterial:"depth",MeshDistanceMaterial:"distanceRGBA",MeshNormalMaterial:"normal",MeshBasicMaterial:"basic",MeshLambertMaterial:"lambert",MeshPhongMaterial:"phong",MeshToonMaterial:"toon",MeshStandardMaterial:"physical",MeshPhysicalMaterial:"physical",MeshMatcapMaterial:"matcap",LineBasicMaterial:"basic",LineDashedMaterial:"dashed",PointsMaterial:"points",ShadowMaterial:"shadow",SpriteMaterial:"sprite"},d=["precision","isWebGL2","supportsVertexTextures","outputEncoding","instancing","map","mapEncoding","matcap","matcapEncoding","envMap","envMapMode","envMapEncoding","envMapCubeUV","lightMap","lightMapEncoding","aoMap","emissiveMap","emissiveMapEncoding","bumpMap","normalMap","objectSpaceNormalMap","tangentSpaceNormalMap","clearcoatMap","clearcoatRoughnessMap","clearcoatNormalMap","displacementMap","specularMap","roughnessMap","metalnessMap","gradientMap","alphaMap","combine","vertexColors","vertexTangents","vertexUvs","uvsVertexOnly","fog","useFog","fogExp2","flatShading","sizeAttenuation","logarithmicDepthBuffer","skinning","maxBones","useVertexTexture","morphTargets","morphNormals","maxMorphTargets","maxMorphNormals","premultipliedAlpha","numDirLights","numPointLights","numSpotLights","numHemiLights","numRectAreaLights","numDirLightShadows","numPointLightShadows","numSpotLightShadows","shadowMapEnabled","shadowMapType","toneMapping","physicallyCorrectLights","alphaTest","doubleSided","flipSided","numClippingPlanes","numClipIntersection","depthPacking","dithering","sheen"];function h(e){var t;return e?e.isTexture?t=e.encoding:e.isWebGLRenderTarget&&(console.warn("THREE.WebGLPrograms.getTextureEncodingFromMap: don't use render targets as textures. Use their .texture property instead."),t=e.texture.encoding):t=$t,t}this.getParameters=function(r,d,f,p,g,v,b){var C=p.fog,A=r.isMeshStandardMaterial?p.environment:null,_=r.envMap||A,I=c[r.type],x=b.isSkinnedMesh?function(e){var t=e.skeleton.bones;if(a)return 1024;var n=l,r=Math.floor((n-20)/4),i=Math.min(r,t.length);return i<t.length?(console.warn("THREE.WebGLRenderer: Skeleton has "+t.length+" bones. This GPU supports "+i+"."),0):i}(b):0;null!==r.precision&&(s=n.getMaxPrecision(r.precision))!==r.precision&&console.warn("THREE.WebGLProgram.getParameters:",r.precision,"not supported, using",s,"instead.");var w=function(e,t){var n;if(t){var r=Ro[t];n={name:e.type,uniforms:Ao.clone(r.uniforms),vertexShader:r.vertexShader,fragmentShader:r.fragmentShader}}else n={name:e.type,uniforms:e.uniforms,vertexShader:e.vertexShader,fragmentShader:e.fragmentShader};return n}(r,I);r.onBeforeCompile(w,e);var M=e.getRenderTarget();return{isWebGL2:i,shaderID:I,shaderName:w.name,uniforms:w.uniforms,vertexShader:w.vertexShader,fragmentShader:w.fragmentShader,defines:r.defines,isRawShaderMaterial:r.isRawShaderMaterial,isShaderMaterial:r.isShaderMaterial,precision:s,instancing:!0===b.isInstancedMesh,supportsVertexTextures:u,outputEncoding:null!==M?h(M.texture):e.outputEncoding,map:!!r.map,mapEncoding:h(r.map),matcap:!!r.matcap,matcapEncoding:h(r.matcap),envMap:!!_,envMapMode:_&&_.mapping,envMapEncoding:h(_),envMapCubeUV:!!_&&(_.mapping===he||_.mapping===fe),lightMap:!!r.lightMap,lightMapEncoding:h(r.lightMap),aoMap:!!r.aoMap,emissiveMap:!!r.emissiveMap,emissiveMapEncoding:h(r.emissiveMap),bumpMap:!!r.bumpMap,normalMap:!!r.normalMap,objectSpaceNormalMap:r.normalMapType===dn,tangentSpaceNormalMap:r.normalMapType===cn,clearcoatMap:!!r.clearcoatMap,clearcoatRoughnessMap:!!r.clearcoatRoughnessMap,clearcoatNormalMap:!!r.clearcoatNormalMap,displacementMap:!!r.displacementMap,roughnessMap:!!r.roughnessMap,metalnessMap:!!r.metalnessMap,specularMap:!!r.specularMap,alphaMap:!!r.alphaMap,gradientMap:!!r.gradientMap,sheen:!!r.sheen,combine:r.combine,vertexTangents:r.normalMap&&r.vertexTangents,vertexColors:r.vertexColors,vertexUvs:!!(r.map||r.bumpMap||r.normalMap||r.specularMap||r.alphaMap||r.emissiveMap||r.roughnessMap||r.metalnessMap||r.clearcoatMap||r.clearcoatRoughnessMap||r.clearcoatNormalMap||r.displacementMap),uvsVertexOnly:!(r.map||r.bumpMap||r.normalMap||r.specularMap||r.alphaMap||r.emissiveMap||r.roughnessMap||r.metalnessMap||r.clearcoatNormalMap||!r.displacementMap),fog:!!C,useFog:r.fog,fogExp2:C&&C.isFogExp2,flatShading:r.flatShading,sizeAttenuation:r.sizeAttenuation,logarithmicDepthBuffer:o,skinning:r.skinning&&x>0,maxBones:x,useVertexTexture:a,morphTargets:r.morphTargets,morphNormals:r.morphNormals,maxMorphTargets:e.maxMorphTargets,maxMorphNormals:e.maxMorphNormals,numDirLights:d.directional.length,numPointLights:d.point.length,numSpotLights:d.spot.length,numRectAreaLights:d.rectArea.length,numHemiLights:d.hemi.length,numDirLightShadows:d.directionalShadowMap.length,numPointLightShadows:d.pointShadowMap.length,numSpotLightShadows:d.spotShadowMap.length,numClippingPlanes:g,numClipIntersection:v,dithering:r.dithering,shadowMapEnabled:e.shadowMap.enabled&&f.length>0,shadowMapType:e.shadowMap.type,toneMapping:r.toneMapped?e.toneMapping:ee,physicallyCorrectLights:e.physicallyCorrectLights,premultipliedAlpha:r.premultipliedAlpha,alphaTest:r.alphaTest,doubleSided:r.side===y,flipSided:r.side===m,depthPacking:void 0!==r.depthPacking&&r.depthPacking,index0AttributeName:r.index0AttributeName,extensionDerivatives:r.extensions&&r.extensions.derivatives,extensionFragDepth:r.extensions&&r.extensions.fragDepth,extensionDrawBuffers:r.extensions&&r.extensions.drawBuffers,extensionShaderTextureLOD:r.extensions&&r.extensions.shaderTextureLOD,rendererExtensionFragDepth:i||null!==t.get("EXT_frag_depth"),rendererExtensionDrawBuffers:i||null!==t.get("WEBGL_draw_buffers"),rendererExtensionShaderTextureLod:i||null!==t.get("EXT_shader_texture_lod"),onBeforeCompile:r.onBeforeCompile}},this.getProgramCacheKey=function(t){var n=[];if(t.shaderID?n.push(t.shaderID):(n.push(t.fragmentShader),n.push(t.vertexShader)),void 0!==t.defines)for(var r in t.defines)n.push(r),n.push(t.defines[r]);if(void 0===t.isRawShaderMaterial){for(var i=0;i<d.length;i++)n.push(t[d[i]]);n.push(e.outputEncoding),n.push(e.gammaFactor)}return n.push(t.onBeforeCompile.toString()),n.join()},this.acquireProgram=function(t,n){for(var i,o=0,a=r.length;o<a;o++){var s=r[o];if(s.cacheKey===n){++(i=s).usedTimes;break}}return void 0===i&&(i=new ds(e,n,t),r.push(i)),i},this.releaseProgram=function(e){if(0==--e.usedTimes){var t=r.indexOf(e);r[t]=r[r.length-1],r.pop(),e.destroy()}},this.programs=r}function fs(){var e=new WeakMap;return{get:function(t){var n=e.get(t);return void 0===n&&(n={},e.set(t,n)),n},remove:function(t){e.delete(t)},update:function(t,n,r){e.get(t)[n]=r},dispose:function(){e=new WeakMap}}}function ps(e,t){return e.groupOrder!==t.groupOrder?e.groupOrder-t.groupOrder:e.renderOrder!==t.renderOrder?e.renderOrder-t.renderOrder:e.program!==t.program?e.program.id-t.program.id:e.material.id!==t.material.id?e.material.id-t.material.id:e.z!==t.z?e.z-t.z:e.id-t.id}function gs(e,t){return e.groupOrder!==t.groupOrder?e.groupOrder-t.groupOrder:e.renderOrder!==t.renderOrder?e.renderOrder-t.renderOrder:e.z!==t.z?t.z-e.z:e.id-t.id}function vs(){var e=[],t=0,n=[],r=[],i={id:-1};function o(n,r,o,a,s,l){var u=e[t];return void 0===u?(u={id:n.id,object:n,geometry:r,material:o,program:o.program||i,groupOrder:a,renderOrder:n.renderOrder,z:s,group:l},e[t]=u):(u.id=n.id,u.object=n,u.geometry=r,u.material=o,u.program=o.program||i,u.groupOrder=a,u.renderOrder=n.renderOrder,u.z=s,u.group=l),t++,u}return{opaque:n,transparent:r,init:function(){t=0,n.length=0,r.length=0},push:function(e,t,i,a,s,l){var u=o(e,t,i,a,s,l);(!0===i.transparent?r:n).push(u)},unshift:function(e,t,i,a,s,l){var u=o(e,t,i,a,s,l);(!0===i.transparent?r:n).unshift(u)},finish:function(){for(var n=t,r=e.length;n<r;n++){var i=e[n];if(null===i.id)break;i.id=null,i.object=null,i.geometry=null,i.material=null,i.program=null,i.group=null}},sort:function(e,t){n.length>1&&n.sort(e||ps),r.length>1&&r.sort(t||gs)}}}function ms(){var e=new WeakMap;function t(n){var r=n.target;r.removeEventListener("dispose",t),e.delete(r)}return{get:function(n,r){var i,o=e.get(n);return void 0===o?(i=new vs,e.set(n,new WeakMap),e.get(n).set(r,i),n.addEventListener("dispose",t)):void 0===(i=o.get(r))&&(i=new vs,o.set(r,i)),i},dispose:function(){e=new WeakMap}}}function ys(){var e={};return{get:function(t){if(void 0!==e[t.id])return e[t.id];var n;switch(t.type){case"DirectionalLight":n={direction:new $n,color:new vi};break;case"SpotLight":n={position:new $n,direction:new $n,color:new vi,distance:0,coneCos:0,penumbraCos:0,decay:0};break;case"PointLight":n={position:new $n,color:new vi,distance:0,decay:0};break;case"HemisphereLight":n={direction:new $n,skyColor:new vi,groundColor:new vi};break;case"RectAreaLight":n={color:new vi,position:new $n,halfWidth:new $n,halfHeight:new $n}}return e[t.id]=n,n}}}var bs=0;function Cs(e,t){return(t.castShadow?1:0)-(e.castShadow?1:0)}function As(){for(var e,t=new ys,n=(e={},{get:function(t){if(void 0!==e[t.id])return e[t.id];var n;switch(t.type){case"DirectionalLight":case"SpotLight":n={shadowBias:0,shadowRadius:1,shadowMapSize:new Un};break;case"PointLight":n={shadowBias:0,shadowRadius:1,shadowMapSize:new Un,shadowCameraNear:1,shadowCameraFar:1e3}}return e[t.id]=n,n}}),r={version:0,hash:{directionalLength:-1,pointLength:-1,spotLength:-1,rectAreaLength:-1,hemiLength:-1,numDirectionalShadows:-1,numPointShadows:-1,numSpotShadows:-1},ambient:[0,0,0],probe:[],directional:[],directionalShadow:[],directionalShadowMap:[],directionalShadowMatrix:[],spot:[],spotShadow:[],spotShadowMap:[],spotShadowMatrix:[],rectArea:[],point:[],pointShadow:[],pointShadowMap:[],pointShadowMatrix:[],hemi:[]},i=0;i<9;i++)r.probe.push(new $n);var o=new $n,a=new sr,s=new sr;return{setup:function(e,i,l){for(var u=0,c=0,d=0,h=0;h<9;h++)r.probe[h].set(0,0,0);var f=0,p=0,g=0,v=0,m=0,y=0,b=0,C=0,A=l.matrixWorldInverse;e.sort(Cs),h=0;for(var _=e.length;h<_;h++){var I=e[h],x=I.color,w=I.intensity,M=I.distance,S=I.shadow&&I.shadow.map?I.shadow.map.texture:null;if(I.isAmbientLight)u+=x.r*w,c+=x.g*w,d+=x.b*w;else if(I.isLightProbe)for(var T=0;T<9;T++)r.probe[T].addScaledVector(I.sh.coefficients[T],w);else if(I.isDirectionalLight){if((k=t.get(I)).color.copy(I.color).multiplyScalar(I.intensity),k.direction.setFromMatrixPosition(I.matrixWorld),o.setFromMatrixPosition(I.target.matrixWorld),k.direction.sub(o),k.direction.transformDirection(A),I.castShadow){var E=I.shadow;(O=n.get(I)).shadowBias=E.bias,O.shadowRadius=E.radius,O.shadowMapSize=E.mapSize,r.directionalShadow[f]=O,r.directionalShadowMap[f]=S,r.directionalShadowMatrix[f]=I.shadow.matrix,y++}r.directional[f]=k,f++}else if(I.isSpotLight){if((k=t.get(I)).position.setFromMatrixPosition(I.matrixWorld),k.position.applyMatrix4(A),k.color.copy(x).multiplyScalar(w),k.distance=M,k.direction.setFromMatrixPosition(I.matrixWorld),o.setFromMatrixPosition(I.target.matrixWorld),k.direction.sub(o),k.direction.transformDirection(A),k.coneCos=Math.cos(I.angle),k.penumbraCos=Math.cos(I.angle*(1-I.penumbra)),k.decay=I.decay,I.castShadow){E=I.shadow;(O=n.get(I)).shadowBias=E.bias,O.shadowRadius=E.radius,O.shadowMapSize=E.mapSize,r.spotShadow[g]=O,r.spotShadowMap[g]=S,r.spotShadowMatrix[g]=I.shadow.matrix,C++}r.spot[g]=k,g++}else if(I.isRectAreaLight){(k=t.get(I)).color.copy(x).multiplyScalar(w),k.position.setFromMatrixPosition(I.matrixWorld),k.position.applyMatrix4(A),s.identity(),a.copy(I.matrixWorld),a.premultiply(A),s.extractRotation(a),k.halfWidth.set(.5*I.width,0,0),k.halfHeight.set(0,.5*I.height,0),k.halfWidth.applyMatrix4(s),k.halfHeight.applyMatrix4(s),r.rectArea[v]=k,v++}else if(I.isPointLight){if((k=t.get(I)).position.setFromMatrixPosition(I.matrixWorld),k.position.applyMatrix4(A),k.color.copy(I.color).multiplyScalar(I.intensity),k.distance=I.distance,k.decay=I.decay,I.castShadow){var O;E=I.shadow;(O=n.get(I)).shadowBias=E.bias,O.shadowRadius=E.radius,O.shadowMapSize=E.mapSize,O.shadowCameraNear=E.camera.near,O.shadowCameraFar=E.camera.far,r.pointShadow[p]=O,r.pointShadowMap[p]=S,r.pointShadowMatrix[p]=I.shadow.matrix,b++}r.point[p]=k,p++}else if(I.isHemisphereLight){var k;(k=t.get(I)).direction.setFromMatrixPosition(I.matrixWorld),k.direction.transformDirection(A),k.direction.normalize(),k.skyColor.copy(I.color).multiplyScalar(w),k.groundColor.copy(I.groundColor).multiplyScalar(w),r.hemi[m]=k,m++}}r.ambient[0]=u,r.ambient[1]=c,r.ambient[2]=d;var L=r.hash;L.directionalLength===f&&L.pointLength===p&&L.spotLength===g&&L.rectAreaLength===v&&L.hemiLength===m&&L.numDirectionalShadows===y&&L.numPointShadows===b&&L.numSpotShadows===C||(r.directional.length=f,r.spot.length=g,r.rectArea.length=v,r.point.length=p,r.hemi.length=m,r.directionalShadow.length=y,r.directionalShadowMap.length=y,r.pointShadow.length=b,r.pointShadowMap.length=b,r.spotShadow.length=C,r.spotShadowMap.length=C,r.directionalShadowMatrix.length=y,r.pointShadowMatrix.length=b,r.spotShadowMatrix.length=C,L.directionalLength=f,L.pointLength=p,L.spotLength=g,L.rectAreaLength=v,L.hemiLength=m,L.numDirectionalShadows=y,L.numPointShadows=b,L.numSpotShadows=C,r.version=bs++)},state:r}}function _s(){var e=new As,t=[],n=[];return{init:function(){t.length=0,n.length=0},state:{lightsArray:t,shadowsArray:n,lights:e},setupLights:function(r){e.setup(t,n,r)},pushLight:function(e){t.push(e)},pushShadow:function(e){n.push(e)}}}function Is(){var e=new WeakMap;function t(n){var r=n.target;r.removeEventListener("dispose",t),e.delete(r)}return{get:function(n,r){var i;return!1===e.has(n)?(i=new _s,e.set(n,new WeakMap),e.get(n).set(r,i),n.addEventListener("dispose",t)):!1===e.get(n).has(r)?(i=new _s,e.get(n).set(r,i)):i=e.get(n).get(r),i},dispose:function(){e=new WeakMap}}}function xs(e){_i.call(this),this.type="MeshDepthMaterial",this.depthPacking=ln,this.skinning=!1,this.morphTargets=!1,this.map=null,this.alphaMap=null,this.displacementMap=null,this.displacementScale=1,this.displacementBias=0,this.wireframe=!1,this.wireframeLinewidth=1,this.fog=!1,this.setValues(e)}function ws(e){_i.call(this),this.type="MeshDistanceMaterial",this.referencePosition=new $n,this.nearDistance=1,this.farDistance=1e3,this.skinning=!1,this.morphTargets=!1,this.map=null,this.alphaMap=null,this.displacementMap=null,this.displacementScale=1,this.displacementBias=0,this.fog=!1,this.setValues(e)}xs.prototype=Object.create(_i.prototype),xs.prototype.constructor=xs,xs.prototype.isMeshDepthMaterial=!0,xs.prototype.copy=function(e){return _i.prototype.copy.call(this,e),this.depthPacking=e.depthPacking,this.skinning=e.skinning,this.morphTargets=e.morphTargets,this.map=e.map,this.alphaMap=e.alphaMap,this.displacementMap=e.displacementMap,this.displacementScale=e.displacementScale,this.displacementBias=e.displacementBias,this.wireframe=e.wireframe,this.wireframeLinewidth=e.wireframeLinewidth,this},ws.prototype=Object.create(_i.prototype),ws.prototype.constructor=ws,ws.prototype.isMeshDistanceMaterial=!0,ws.prototype.copy=function(e){return _i.prototype.copy.call(this,e),this.referencePosition.copy(e.referencePosition),this.nearDistance=e.nearDistance,this.farDistance=e.farDistance,this.skinning=e.skinning,this.morphTargets=e.morphTargets,this.map=e.map,this.alphaMap=e.alphaMap,this.displacementMap=e.displacementMap,this.displacementScale=e.displacementScale,this.displacementBias=e.displacementBias,this};function Ms(e,t,n){var r=new Oo,i=new Un,o=new Un,a=new Xn,s=[],l=[],u={},c={0:m,1:v,2:y},d=new _o({defines:{SAMPLE_RATE:2/8,HALF_SAMPLE_RATE:1/8},uniforms:{shadow_pass:{value:null},resolution:{value:new Un},radius:{value:4}},vertexShader:"void main() {\n\tgl_Position = vec4( position, 1.0 );\n}",fragmentShader:"uniform sampler2D shadow_pass;\nuniform vec2 resolution;\nuniform float radius;\n#include <packing>\nvoid main() {\n  float mean = 0.0;\n  float squared_mean = 0.0;\n\tfloat depth = unpackRGBAToDepth( texture2D( shadow_pass, ( gl_FragCoord.xy  ) / resolution ) );\n  for ( float i = -1.0; i < 1.0 ; i += SAMPLE_RATE) {\n    #ifdef HORIZONAL_PASS\n      vec2 distribution = unpackRGBATo2Half( texture2D( shadow_pass, ( gl_FragCoord.xy + vec2( i, 0.0 ) * radius ) / resolution ) );\n      mean += distribution.x;\n      squared_mean += distribution.y * distribution.y + distribution.x * distribution.x;\n    #else\n      float depth = unpackRGBAToDepth( texture2D( shadow_pass, ( gl_FragCoord.xy + vec2( 0.0,  i )  * radius ) / resolution ) );\n      mean += depth;\n      squared_mean += depth * depth;\n    #endif\n  }\n  mean = mean * HALF_SAMPLE_RATE;\n  squared_mean = squared_mean * HALF_SAMPLE_RATE;\n  float std_dev = sqrt( squared_mean - mean * mean );\n  gl_FragColor = pack2HalfToRGBA( vec2( mean, std_dev ) );\n}"}),h=d.clone();h.defines.HORIZONAL_PASS=1;var p=new Yi;p.setAttribute("position",new wi(new Float32Array([-1,-1,.5,3,-1,.5,-1,3,.5]),3));var b=new lo(p,d),C=this;function _(n,r){var i=t.update(b);d.uniforms.shadow_pass.value=n.map.texture,d.uniforms.resolution.value=n.mapSize,d.uniforms.radius.value=n.radius,e.setRenderTarget(n.mapPass),e.clear(),e.renderBufferDirect(r,null,i,d,b,null),h.uniforms.shadow_pass.value=n.mapPass.texture,h.uniforms.resolution.value=n.mapSize,h.uniforms.radius.value=n.radius,e.setRenderTarget(n.map),e.clear(),e.renderBufferDirect(r,null,i,h,b,null)}function I(e,t,n){var r=e<<0|t<<1|n<<2,i=s[r];return void 0===i&&(i=new xs({depthPacking:un,morphTargets:e,skinning:t}),s[r]=i),i}function x(e,t,n){var r=e<<0|t<<1|n<<2,i=l[r];return void 0===i&&(i=new ws({morphTargets:e,skinning:t}),l[r]=i),i}function w(t,n,r,i,o,a,s){var l=null,d=I,h=t.customDepthMaterial;if(!0===i.isPointLight&&(d=x,h=t.customDistanceMaterial),void 0===h){var f=!1;!0===r.morphTargets&&(f=n.morphAttributes&&n.morphAttributes.position&&n.morphAttributes.position.length>0);var p=!1;!0===t.isSkinnedMesh&&(!0===r.skinning?p=!0:console.warn("THREE.WebGLShadowMap: THREE.SkinnedMesh with material.skinning set to false:",t)),l=d(f,p,!0===t.isInstancedMesh)}else l=h;if(e.localClippingEnabled&&!0===r.clipShadows&&0!==r.clippingPlanes.length){var v=l.uuid,m=r.uuid,y=u[v];void 0===y&&(y={},u[v]=y);var b=y[m];void 0===b&&(b=l.clone(),y[m]=b),l=b}return l.visible=r.visible,l.wireframe=r.wireframe,l.side=s===g?null!==r.shadowSide?r.shadowSide:r.side:null!==r.shadowSide?r.shadowSide:c[r.side],l.clipShadows=r.clipShadows,l.clippingPlanes=r.clippingPlanes,l.clipIntersection=r.clipIntersection,l.wireframeLinewidth=r.wireframeLinewidth,l.linewidth=r.linewidth,!0===i.isPointLight&&!0===l.isMeshDistanceMaterial&&(l.referencePosition.setFromMatrixPosition(i.matrixWorld),l.nearDistance=o,l.farDistance=a),l}function M(n,i,o,a,s){if(!1!==n.visible){if(n.layers.test(i.layers)&&(n.isMesh||n.isLine||n.isPoints)&&(n.castShadow||n.receiveShadow&&s===g)&&(!n.frustumCulled||r.intersectsObject(n))){n.modelViewMatrix.multiplyMatrices(o.matrixWorldInverse,n.matrixWorld);var l=t.update(n),u=n.material;if(Array.isArray(u))for(var c=l.groups,d=0,h=c.length;d<h;d++){var f=c[d],p=u[f.materialIndex];if(p&&p.visible){var v=w(n,l,p,a,o.near,o.far,s);e.renderBufferDirect(o,null,l,v,n,f)}}else if(u.visible){v=w(n,l,u,a,o.near,o.far,s);e.renderBufferDirect(o,null,l,v,n,null)}}for(var m=n.children,y=0,b=m.length;y<b;y++)M(m[y],i,o,a,s)}}this.enabled=!1,this.autoUpdate=!0,this.needsUpdate=!1,this.type=f,this.render=function(t,s,l){if(!1!==C.enabled&&(!1!==C.autoUpdate||!1!==C.needsUpdate)&&0!==t.length){var u=e.getRenderTarget(),c=e.getActiveCubeFace(),d=e.getActiveMipmapLevel(),h=e.state;h.setBlending(A),h.buffers.color.setClear(1,1,1,1),h.buffers.depth.setTest(!0),h.setScissorTest(!1);for(var f=0,p=t.length;f<p;f++){var v=t[f],m=v.shadow;if(void 0!==m){i.copy(m.mapSize);var y=m.getFrameExtents();if(i.multiply(y),o.copy(m.mapSize),(i.x>n||i.y>n)&&(i.x>n&&(o.x=Math.floor(n/y.x),i.x=o.x*y.x,m.mapSize.x=o.x),i.y>n&&(o.y=Math.floor(n/y.y),i.y=o.y*y.y,m.mapSize.y=o.y)),null===m.map&&!m.isPointLightShadow&&this.type===g){var b={minFilter:_e,magFilter:_e,format:He};m.map=new qn(i.x,i.y,b),m.map.texture.name=v.name+".shadowMap",m.mapPass=new qn(i.x,i.y,b),m.camera.updateProjectionMatrix()}if(null===m.map){b={minFilter:me,magFilter:me,format:He};m.map=new qn(i.x,i.y,b),m.map.texture.name=v.name+".shadowMap",m.camera.updateProjectionMatrix()}e.setRenderTarget(m.map),e.clear();for(var I=m.getViewportCount(),x=0;x<I;x++){var w=m.getViewport(x);a.set(o.x*w.x,o.y*w.y,o.x*w.z,o.y*w.w),h.viewport(a),m.updateMatrices(v,x),r=m.getFrustum(),M(s,l,m.camera,v,this.type)}m.isPointLightShadow||this.type!==g||_(m,l)}else console.warn("THREE.WebGLShadowMap:",v,"has no shadow.")}C.needsUpdate=!1,e.setRenderTarget(u,c,d)}}}function Ss(e,t,n){var r=n.isWebGL2;var i=new function(){var t=!1,n=new Xn,r=null,i=new Xn(0,0,0,0);return{setMask:function(n){r===n||t||(e.colorMask(n,n,n,n),r=n)},setLocked:function(e){t=e},setClear:function(t,r,o,a,s){!0===s&&(t*=a,r*=a,o*=a),n.set(t,r,o,a),!1===i.equals(n)&&(e.clearColor(t,r,o,a),i.copy(n))},reset:function(){t=!1,r=null,i.set(-1,0,0,0)}}},o=new function(){var t=!1,n=null,r=null,i=null;return{setTest:function(e){e?be(2929):Ce(2929)},setMask:function(r){n===r||t||(e.depthMask(r),n=r)},setFunc:function(t){if(r!==t){if(t)switch(t){case V:e.depthFunc(512);break;case Y:e.depthFunc(519);break;case W:e.depthFunc(513);break;case G:e.depthFunc(515);break;case X:e.depthFunc(514);break;case q:e.depthFunc(518);break;case Z:e.depthFunc(516);break;case Q:e.depthFunc(517);break;default:e.depthFunc(515)}else e.depthFunc(515);r=t}},setLocked:function(e){t=e},setClear:function(t){i!==t&&(e.clearDepth(t),i=t)},reset:function(){t=!1,n=null,r=null,i=null}}},u=new function(){var t=!1,n=null,r=null,i=null,o=null,a=null,s=null,l=null,u=null;return{setTest:function(e){t||(e?be(2960):Ce(2960))},setMask:function(r){n===r||t||(e.stencilMask(r),n=r)},setFunc:function(t,n,a){r===t&&i===n&&o===a||(e.stencilFunc(t,n,a),r=t,i=n,o=a)},setOp:function(t,n,r){a===t&&s===n&&l===r||(e.stencilOp(t,n,r),a=t,s=n,l=r)},setLocked:function(e){t=e},setClear:function(t){u!==t&&(e.clearStencil(t),u=t)},reset:function(){t=!1,n=null,r=null,i=null,o=null,a=null,s=null,l=null,u=null}}},c=e.getParameter(34921),d=new Uint8Array(c),h=new Uint8Array(c),f=new Uint8Array(c),p={},g=null,v=null,b=null,C=null,K=null,J=null,$=null,ee=null,te=null,ne=!1,re=null,ie=null,oe=null,ae=null,se=null,le=e.getParameter(35661),ue=!1,ce=0,de=e.getParameter(7938);-1!==de.indexOf("WebGL")?(ce=parseFloat(/^WebGL\ ([0-9])/.exec(de)[1]),ue=ce>=1):-1!==de.indexOf("OpenGL ES")&&(ce=parseFloat(/^OpenGL\ ES\ ([0-9])/.exec(de)[1]),ue=ce>=2);var he=null,fe={},pe=new Xn,ge=new Xn;function ve(t,n,r){var i=new Uint8Array(4),o=e.createTexture();e.bindTexture(t,o),e.texParameteri(t,10241,9728),e.texParameteri(t,10240,9728);for(var a=0;a<r;a++)e.texImage2D(n+a,0,6408,1,1,0,6408,5121,i);return o}var me={};function ye(n,i){(d[n]=1,0===h[n]&&(e.enableVertexAttribArray(n),h[n]=1),f[n]!==i)&&((r?e:t.get("ANGLE_instanced_arrays"))[r?"vertexAttribDivisor":"vertexAttribDivisorANGLE"](n,i),f[n]=i)}function be(t){!0!==p[t]&&(e.enable(t),p[t]=!0)}function Ce(t){!1!==p[t]&&(e.disable(t),p[t]=!1)}me[3553]=ve(3553,3553,1),me[34067]=ve(34067,34069,6),i.setClear(0,0,0,1),o.setClear(1),u.setClear(0),be(2929),o.setFunc(G),we(!1),Me(s),be(2884),xe(A);var Ae={[S]:32774,[T]:32778,[E]:32779};if(r)Ae[O]=32775,Ae[k]=32776;else{var _e=t.get("EXT_blend_minmax");null!==_e&&(Ae[O]=_e.MIN_EXT,Ae[k]=_e.MAX_EXT)}var Ie={[L]:0,[D]:1,[P]:768,[j]:770,[U]:776,[B]:774,[z]:772,[N]:769,[R]:771,[H]:775,[F]:773};function xe(t,n,r,i,o,a,s,l){if(t!==A){if(v||(be(3042),v=!0),t===M)o=o||n,a=a||r,s=s||i,n===C&&o===$||(e.blendEquationSeparate(Ae[n],Ae[o]),C=n,$=o),r===K&&i===J&&a===ee&&s===te||(e.blendFuncSeparate(Ie[r],Ie[i],Ie[a],Ie[s]),K=r,J=i,ee=a,te=s),b=t,ne=null;else if(t!==b||l!==ne){if(C===S&&$===S||(e.blendEquation(32774),C=S,$=S),l)switch(t){case _:e.blendFuncSeparate(1,771,1,771);break;case I:e.blendFunc(1,1);break;case x:e.blendFuncSeparate(0,0,769,771);break;case w:e.blendFuncSeparate(0,768,0,770);break;default:console.error("THREE.WebGLState: Invalid blending: ",t)}else switch(t){case _:e.blendFuncSeparate(770,771,1,771);break;case I:e.blendFunc(770,1);break;case x:e.blendFunc(0,769);break;case w:e.blendFunc(0,768);break;default:console.error("THREE.WebGLState: Invalid blending: ",t)}K=null,J=null,ee=null,te=null,b=t,ne=l}}else v&&(Ce(3042),v=!1)}function we(t){re!==t&&(t?e.frontFace(2304):e.frontFace(2305),re=t)}function Me(t){t!==a?(be(2884),t!==ie&&(t===s?e.cullFace(1029):t===l?e.cullFace(1028):e.cullFace(1032))):Ce(2884),ie=t}function Se(t,n,r){t?(be(32823),ae===n&&se===r||(e.polygonOffset(n,r),ae=n,se=r)):Ce(32823)}function Te(t){void 0===t&&(t=33984+le-1),he!==t&&(e.activeTexture(t),he=t)}return{buffers:{color:i,depth:o,stencil:u},initAttributes:function(){for(var e=0,t=d.length;e<t;e++)d[e]=0},enableAttribute:function(e){ye(e,0)},enableAttributeAndDivisor:ye,disableUnusedAttributes:function(){for(var t=0,n=h.length;t!==n;++t)h[t]!==d[t]&&(e.disableVertexAttribArray(t),h[t]=0)},vertexAttribPointer:function(t,n,i,o,a,s){!0!==r||5124!==i&&5125!==i?e.vertexAttribPointer(t,n,i,o,a,s):e.vertexAttribIPointer(t,n,i,o,a,s)},enable:be,disable:Ce,useProgram:function(t){return g!==t&&(e.useProgram(t),g=t,!0)},setBlending:xe,setMaterial:function(e,t){e.side===y?Ce(2884):be(2884);var n=e.side===m;t&&(n=!n),we(n),e.blending===_&&!1===e.transparent?xe(A):xe(e.blending,e.blendEquation,e.blendSrc,e.blendDst,e.blendEquationAlpha,e.blendSrcAlpha,e.blendDstAlpha,e.premultipliedAlpha),o.setFunc(e.depthFunc),o.setTest(e.depthTest),o.setMask(e.depthWrite),i.setMask(e.colorWrite);var r=e.stencilWrite;u.setTest(r),r&&(u.setMask(e.stencilWriteMask),u.setFunc(e.stencilFunc,e.stencilRef,e.stencilFuncMask),u.setOp(e.stencilFail,e.stencilZFail,e.stencilZPass)),Se(e.polygonOffset,e.polygonOffsetFactor,e.polygonOffsetUnits)},setFlipSided:we,setCullFace:Me,setLineWidth:function(t){t!==oe&&(ue&&e.lineWidth(t),oe=t)},setPolygonOffset:Se,setScissorTest:function(e){e?be(3089):Ce(3089)},activeTexture:Te,bindTexture:function(t,n){null===he&&Te();var r=fe[he];void 0===r&&(r={type:void 0,texture:void 0},fe[he]=r),r.type===t&&r.texture===n||(e.bindTexture(t,n||me[t]),r.type=t,r.texture=n)},unbindTexture:function(){var t=fe[he];void 0!==t&&void 0!==t.type&&(e.bindTexture(t.type,null),t.type=void 0,t.texture=void 0)},compressedTexImage2D:function(){try{e.compressedTexImage2D.apply(e,arguments)}catch(e){console.error("THREE.WebGLState:",e)}},texImage2D:function(){try{e.texImage2D.apply(e,arguments)}catch(e){console.error("THREE.WebGLState:",e)}},texImage3D:function(){try{e.texImage3D.apply(e,arguments)}catch(e){console.error("THREE.WebGLState:",e)}},scissor:function(t){!1===pe.equals(t)&&(e.scissor(t.x,t.y,t.z,t.w),pe.copy(t))},viewport:function(t){!1===ge.equals(t)&&(e.viewport(t.x,t.y,t.z,t.w),ge.copy(t))},reset:function(){for(var t=0;t<h.length;t++)1===h[t]&&(e.disableVertexAttribArray(t),h[t]=0);p={},he=null,fe={},g=null,b=null,re=null,ie=null,i.reset(),o.reset(),u.reset()}}}function Ts(e,t,n,r,i,o,a){var s,l=i.isWebGL2,u=i.maxTextures,c=i.maxCubemapSize,d=i.maxTextureSize,h=i.maxSamples,f=new WeakMap,p=!1;try{p="undefined"!=typeof OffscreenCanvas&&null!==new OffscreenCanvas(1,1).getContext("2d")}catch(e){}function g(e,t){return p?new OffscreenCanvas(e,t):document.createElementNS("http://www.w3.org/1999/xhtml","canvas")}function v(e,t,n,r){var i=1;if((e.width>r||e.height>r)&&(i=r/Math.max(e.width,e.height)),i<1||!0===t){if("undefined"!=typeof HTMLImageElement&&e instanceof HTMLImageElement||"undefined"!=typeof HTMLCanvasElement&&e instanceof HTMLCanvasElement||"undefined"!=typeof ImageBitmap&&e instanceof ImageBitmap){var o=t?Hn.floorPowerOfTwo:Math.floor,a=o(i*e.width),l=o(i*e.height);void 0===s&&(s=g(a,l));var u=n?g(a,l):s;return u.width=a,u.height=l,u.getContext("2d").drawImage(e,0,0,a,l),console.warn("THREE.WebGLRenderer: Texture has been resized from ("+e.width+"x"+e.height+") to ("+a+"x"+l+")."),u}return"data"in e&&console.warn("THREE.WebGLRenderer: Image in DataTexture is too big ("+e.width+"x"+e.height+")."),e}return e}function m(e){return Hn.isPowerOfTwo(e.width)&&Hn.isPowerOfTwo(e.height)}function y(e,t){return e.generateMipmaps&&t&&e.minFilter!==me&&e.minFilter!==_e}function b(t,n,i,o){e.generateMipmap(t),r.get(n).__maxMipLevel=Math.log(Math.max(i,o))*Math.LOG2E}function C(n,r,i){if(!1===l)return r;if(null!==n){if(void 0!==e[n])return e[n];console.warn("THREE.WebGLRenderer: Attempt to use non-existing WebGL internal format '"+n+"'")}var o=r;return 6403===r&&(5126===i&&(o=33326),5131===i&&(o=33325),5121===i&&(o=33321)),6407===r&&(5126===i&&(o=34837),5131===i&&(o=34843),5121===i&&(o=32849)),6408===r&&(5126===i&&(o=34836),5131===i&&(o=34842),5121===i&&(o=32856)),33325!==o&&33326!==o&&34842!==o&&34836!==o||t.get("EXT_color_buffer_float"),o}function A(e){return e===me||e===ye||e===Ce?9728:9729}function _(t){var n=t.target;n.removeEventListener("dispose",_),function(t){var n=r.get(t);if(void 0===n.__webglInit)return;e.deleteTexture(n.__webglTexture),r.remove(t)}(n),n.isVideoTexture&&f.delete(n),a.memory.textures--}function I(t){var n=t.target;n.removeEventListener("dispose",I),function(t){var n=r.get(t),i=r.get(t.texture);if(!t)return;void 0!==i.__webglTexture&&e.deleteTexture(i.__webglTexture);t.depthTexture&&t.depthTexture.dispose();if(t.isWebGLCubeRenderTarget)for(var o=0;o<6;o++)e.deleteFramebuffer(n.__webglFramebuffer[o]),n.__webglDepthbuffer&&e.deleteRenderbuffer(n.__webglDepthbuffer[o]);else e.deleteFramebuffer(n.__webglFramebuffer),n.__webglDepthbuffer&&e.deleteRenderbuffer(n.__webglDepthbuffer),n.__webglMultisampledFramebuffer&&e.deleteFramebuffer(n.__webglMultisampledFramebuffer),n.__webglColorRenderbuffer&&e.deleteRenderbuffer(n.__webglColorRenderbuffer),n.__webglDepthRenderbuffer&&e.deleteRenderbuffer(n.__webglDepthRenderbuffer);r.remove(t.texture),r.remove(t)}(n),a.memory.textures--}var x=0;function w(e,t){var i=r.get(e);if(e.isVideoTexture&&function(e){var t=a.render.frame;f.get(e)!==t&&(f.set(e,t),e.update())}(e),e.version>0&&i.__version!==e.version){var o=e.image;if(void 0===o)console.warn("THREE.WebGLRenderer: Texture marked for update but image is undefined");else{if(!1!==o.complete)return void L(i,e,t);console.warn("THREE.WebGLRenderer: Texture marked for update but image is incomplete")}}n.activeTexture(33984+t),n.bindTexture(3553,i.__webglTexture)}function M(t,i){if(6===t.image.length){var a=r.get(t);if(t.version>0&&a.__version!==t.version){k(a,t),n.activeTexture(33984+i),n.bindTexture(34067,a.__webglTexture),e.pixelStorei(37440,t.flipY);for(var s=t&&(t.isCompressedTexture||t.image[0].isCompressedTexture),u=t.image[0]&&t.image[0].isDataTexture,d=[],h=0;h<6;h++)d[h]=s||u?u?t.image[h].image:t.image[h]:v(t.image[h],!1,!0,c);var f,p=d[0],g=m(p)||l,A=o.convert(t.format),_=o.convert(t.type),I=C(t.internalFormat,A,_);if(O(34067,t,g),s){for(h=0;h<6;h++){f=d[h].mipmaps;for(var x=0;x<f.length;x++){var w=f[x];t.format!==He&&t.format!==Be?null!==A?n.compressedTexImage2D(34069+h,x,I,w.width,w.height,0,w.data):console.warn("THREE.WebGLRenderer: Attempt to load unsupported compressed texture format in .setTextureCube()"):n.texImage2D(34069+h,x,I,w.width,w.height,0,A,_,w.data)}}a.__maxMipLevel=f.length-1}else{f=t.mipmaps;for(h=0;h<6;h++)if(u){n.texImage2D(34069+h,0,I,d[h].width,d[h].height,0,A,_,d[h].data);for(x=0;x<f.length;x++){var M=(w=f[x]).image[h].image;n.texImage2D(34069+h,x+1,I,M.width,M.height,0,A,_,M.data)}}else{n.texImage2D(34069+h,0,I,A,_,d[h]);for(x=0;x<f.length;x++){w=f[x];n.texImage2D(34069+h,x+1,I,A,_,w.image[h])}}a.__maxMipLevel=f.length}y(t,g)&&b(34067,t,p.width,p.height),a.__version=t.version,t.onUpdate&&t.onUpdate(t)}else n.activeTexture(33984+i),n.bindTexture(34067,a.__webglTexture)}}function S(e,t){n.activeTexture(33984+t),n.bindTexture(34067,r.get(e).__webglTexture)}var T={[pe]:10497,[ge]:33071,[ve]:33648},E={[me]:9728,[ye]:9984,[Ce]:9986,[_e]:9729,[Ie]:9985,[we]:9987};function O(n,o,a){a?(e.texParameteri(n,10242,T[o.wrapS]),e.texParameteri(n,10243,T[o.wrapT]),32879!==n&&35866!==n||e.texParameteri(n,32882,T[o.wrapR]),e.texParameteri(n,10240,E[o.magFilter]),e.texParameteri(n,10241,E[o.minFilter])):(e.texParameteri(n,10242,33071),e.texParameteri(n,10243,33071),32879!==n&&35866!==n||e.texParameteri(n,32882,33071),o.wrapS===ge&&o.wrapT===ge||console.warn("THREE.WebGLRenderer: Texture is not power of two. Texture.wrapS and Texture.wrapT should be set to THREE.ClampToEdgeWrapping."),e.texParameteri(n,10240,A(o.magFilter)),e.texParameteri(n,10241,A(o.minFilter)),o.minFilter!==me&&o.minFilter!==_e&&console.warn("THREE.WebGLRenderer: Texture is not power of two. Texture.minFilter should be set to THREE.NearestFilter or THREE.LinearFilter."));var s=t.get("EXT_texture_filter_anisotropic");if(s){if(o.type===De&&null===t.get("OES_texture_float_linear"))return;if(o.type===Pe&&null===(l||t.get("OES_texture_half_float_linear")))return;(o.anisotropy>1||r.get(o).__currentAnisotropy)&&(e.texParameterf(n,s.TEXTURE_MAX_ANISOTROPY_EXT,Math.min(o.anisotropy,i.getMaxAnisotropy())),r.get(o).__currentAnisotropy=o.anisotropy)}}function k(t,n){void 0===t.__webglInit&&(t.__webglInit=!0,n.addEventListener("dispose",_),t.__webglTexture=e.createTexture(),a.memory.textures++)}function L(t,r,i){var a=3553;r.isDataTexture2DArray&&(a=35866),r.isDataTexture3D&&(a=32879),k(t,r),n.activeTexture(33984+i),n.bindTexture(a,t.__webglTexture),e.pixelStorei(37440,r.flipY),e.pixelStorei(37441,r.premultiplyAlpha),e.pixelStorei(3317,r.unpackAlignment);var s=function(e){return!l&&(e.wrapS!==ge||e.wrapT!==ge||e.minFilter!==me&&e.minFilter!==_e)}(r)&&!1===m(r.image),u=v(r.image,s,!1,d),c=m(u)||l,h=o.convert(r.format),f=o.convert(r.type),p=C(r.internalFormat,h,f);O(a,r,c);var g,A=r.mipmaps;if(r.isDepthTexture)p=6402,l?p=r.type===De?36012:r.type===Le?33190:r.type===ze?35056:33189:r.type===De&&console.error("WebGLRenderer: Floating point depth texture requires WebGL2."),r.format===We&&6402===p&&r.type!==Oe&&r.type!==Le&&(console.warn("THREE.WebGLRenderer: Use UnsignedShortType or UnsignedIntType for DepthFormat DepthTexture."),r.type=Oe,f=o.convert(r.type)),r.format===Ge&&6402===p&&(p=34041,r.type!==ze&&(console.warn("THREE.WebGLRenderer: Use UnsignedInt248Type for DepthStencilFormat DepthTexture."),r.type=ze,f=o.convert(r.type))),n.texImage2D(3553,0,p,u.width,u.height,0,h,f,null);else if(r.isDataTexture)if(A.length>0&&c){for(var _=0,I=A.length;_<I;_++)g=A[_],n.texImage2D(3553,_,p,g.width,g.height,0,h,f,g.data);r.generateMipmaps=!1,t.__maxMipLevel=A.length-1}else n.texImage2D(3553,0,p,u.width,u.height,0,h,f,u.data),t.__maxMipLevel=0;else if(r.isCompressedTexture){for(_=0,I=A.length;_<I;_++)g=A[_],r.format!==He&&r.format!==Be?null!==h?n.compressedTexImage2D(3553,_,p,g.width,g.height,0,g.data):console.warn("THREE.WebGLRenderer: Attempt to load unsupported compressed texture format in .uploadTexture()"):n.texImage2D(3553,_,p,g.width,g.height,0,h,f,g.data);t.__maxMipLevel=A.length-1}else if(r.isDataTexture2DArray)n.texImage3D(35866,0,p,u.width,u.height,u.depth,0,h,f,u.data),t.__maxMipLevel=0;else if(r.isDataTexture3D)n.texImage3D(32879,0,p,u.width,u.height,u.depth,0,h,f,u.data),t.__maxMipLevel=0;else if(A.length>0&&c){for(_=0,I=A.length;_<I;_++)g=A[_],n.texImage2D(3553,_,p,h,f,g);r.generateMipmaps=!1,t.__maxMipLevel=A.length-1}else n.texImage2D(3553,0,p,h,f,u),t.__maxMipLevel=0;y(r,c)&&b(a,r,u.width,u.height),t.__version=r.version,r.onUpdate&&r.onUpdate(r)}function D(t,i,a,s){var l=o.convert(i.texture.format),u=o.convert(i.texture.type),c=C(i.texture.internalFormat,l,u);n.texImage2D(s,0,c,i.width,i.height,0,l,u,null),e.bindFramebuffer(36160,t),e.framebufferTexture2D(36160,a,s,r.get(i.texture).__webglTexture,0),e.bindFramebuffer(36160,null)}function P(t,n,r){if(e.bindRenderbuffer(36161,t),n.depthBuffer&&!n.stencilBuffer){var i=33189;if(r){var a=n.depthTexture;a&&a.isDepthTexture&&(a.type===De?i=36012:a.type===Le&&(i=33190));var s=j(n);e.renderbufferStorageMultisample(36161,s,i,n.width,n.height)}else e.renderbufferStorage(36161,i,n.width,n.height);e.framebufferRenderbuffer(36160,36096,36161,t)}else if(n.depthBuffer&&n.stencilBuffer){if(r){s=j(n);e.renderbufferStorageMultisample(36161,s,35056,n.width,n.height)}else e.renderbufferStorage(36161,34041,n.width,n.height);e.framebufferRenderbuffer(36160,33306,36161,t)}else{var l=o.convert(n.texture.format),u=o.convert(n.texture.type);i=C(n.texture.internalFormat,l,u);if(r){s=j(n);e.renderbufferStorageMultisample(36161,s,i,n.width,n.height)}else e.renderbufferStorage(36161,i,n.width,n.height)}e.bindRenderbuffer(36161,null)}function N(t){var n=r.get(t),i=!0===t.isWebGLCubeRenderTarget;if(t.depthTexture){if(i)throw new Error("target.depthTexture not supported in Cube render targets");!function(t,n){if(n&&n.isWebGLCubeRenderTarget)throw new Error("Depth Texture with cube render targets is not supported");if(e.bindFramebuffer(36160,t),!n.depthTexture||!n.depthTexture.isDepthTexture)throw new Error("renderTarget.depthTexture must be an instance of THREE.DepthTexture");r.get(n.depthTexture).__webglTexture&&n.depthTexture.image.width===n.width&&n.depthTexture.image.height===n.height||(n.depthTexture.image.width=n.width,n.depthTexture.image.height=n.height,n.depthTexture.needsUpdate=!0),w(n.depthTexture,0);var i=r.get(n.depthTexture).__webglTexture;if(n.depthTexture.format===We)e.framebufferTexture2D(36160,36096,3553,i,0);else{if(n.depthTexture.format!==Ge)throw new Error("Unknown depthTexture format");e.framebufferTexture2D(36160,33306,3553,i,0)}}(n.__webglFramebuffer,t)}else if(i){n.__webglDepthbuffer=[];for(var o=0;o<6;o++)e.bindFramebuffer(36160,n.__webglFramebuffer[o]),n.__webglDepthbuffer[o]=e.createRenderbuffer(),P(n.__webglDepthbuffer[o],t,!1)}else e.bindFramebuffer(36160,n.__webglFramebuffer),n.__webglDepthbuffer=e.createRenderbuffer(),P(n.__webglDepthbuffer,t,!1);e.bindFramebuffer(36160,null)}function j(e){return l&&e.isWebGLMultisampleRenderTarget?Math.min(h,e.samples):0}var R=!1,z=!1;this.allocateTextureUnit=function(){var e=x;return e>=u&&console.warn("THREE.WebGLTextures: Trying to use "+e+" texture units while this GPU supports only "+u),x+=1,e},this.resetTextureUnits=function(){x=0},this.setTexture2D=w,this.setTexture2DArray=function(e,t){var i=r.get(e);e.version>0&&i.__version!==e.version?L(i,e,t):(n.activeTexture(33984+t),n.bindTexture(35866,i.__webglTexture))},this.setTexture3D=function(e,t){var i=r.get(e);e.version>0&&i.__version!==e.version?L(i,e,t):(n.activeTexture(33984+t),n.bindTexture(32879,i.__webglTexture))},this.setTextureCube=M,this.setTextureCubeDynamic=S,this.setupRenderTarget=function(t){var i=r.get(t),s=r.get(t.texture);t.addEventListener("dispose",I),s.__webglTexture=e.createTexture(),a.memory.textures++;var u=!0===t.isWebGLCubeRenderTarget,c=!0===t.isWebGLMultisampleRenderTarget,d=m(t)||l;if(!l||t.texture.format!==Be||t.texture.type!==De&&t.texture.type!==Pe||(t.texture.format=He,console.warn("THREE.WebGLRenderer: Rendering to textures with RGB format is not supported. Using RGBA format instead.")),u){i.__webglFramebuffer=[];for(var h=0;h<6;h++)i.__webglFramebuffer[h]=e.createFramebuffer()}else if(i.__webglFramebuffer=e.createFramebuffer(),c)if(l){i.__webglMultisampledFramebuffer=e.createFramebuffer(),i.__webglColorRenderbuffer=e.createRenderbuffer(),e.bindRenderbuffer(36161,i.__webglColorRenderbuffer);var f=o.convert(t.texture.format),p=o.convert(t.texture.type),g=C(t.texture.internalFormat,f,p),v=j(t);e.renderbufferStorageMultisample(36161,v,g,t.width,t.height),e.bindFramebuffer(36160,i.__webglMultisampledFramebuffer),e.framebufferRenderbuffer(36160,36064,36161,i.__webglColorRenderbuffer),e.bindRenderbuffer(36161,null),t.depthBuffer&&(i.__webglDepthRenderbuffer=e.createRenderbuffer(),P(i.__webglDepthRenderbuffer,t,!0)),e.bindFramebuffer(36160,null)}else console.warn("THREE.WebGLRenderer: WebGLMultisampleRenderTarget can only be used with WebGL2.");if(u){n.bindTexture(34067,s.__webglTexture),O(34067,t.texture,d);for(h=0;h<6;h++)D(i.__webglFramebuffer[h],t,36064,34069+h);y(t.texture,d)&&b(34067,t.texture,t.width,t.height),n.bindTexture(34067,null)}else n.bindTexture(3553,s.__webglTexture),O(3553,t.texture,d),D(i.__webglFramebuffer,t,36064,3553),y(t.texture,d)&&b(3553,t.texture,t.width,t.height),n.bindTexture(3553,null);t.depthBuffer&&N(t)},this.updateRenderTargetMipmap=function(e){var t=e.texture;if(y(t,m(e)||l)){var i=e.isWebGLCubeRenderTarget?34067:3553,o=r.get(t).__webglTexture;n.bindTexture(i,o),b(i,t,e.width,e.height),n.bindTexture(i,null)}},this.updateMultisampleRenderTarget=function(t){if(t.isWebGLMultisampleRenderTarget)if(l){var n=r.get(t);e.bindFramebuffer(36008,n.__webglMultisampledFramebuffer),e.bindFramebuffer(36009,n.__webglFramebuffer);var i=t.width,o=t.height,a=16384;t.depthBuffer&&(a|=256),t.stencilBuffer&&(a|=1024),e.blitFramebuffer(0,0,i,o,0,0,i,o,a,9728),e.bindFramebuffer(36160,n.__webglMultisampledFramebuffer)}else console.warn("THREE.WebGLRenderer: WebGLMultisampleRenderTarget can only be used with WebGL2.")},this.safeSetTexture2D=function(e,t){e&&e.isWebGLRenderTarget&&(!1===R&&(console.warn("THREE.WebGLTextures.safeSetTexture2D: don't use render targets as textures. Use their .texture property instead."),R=!0),e=e.texture),w(e,t)},this.safeSetTextureCube=function(e,t){e&&e.isWebGLCubeRenderTarget&&(!1===z&&(console.warn("THREE.WebGLTextures.safeSetTextureCube: don't use cube render targets as textures. Use their .texture property instead."),z=!0),e=e.texture),e&&e.isCubeTexture||Array.isArray(e.image)&&6===e.image.length?M(e,t):S(e,t)}}function Es(e,t,n){var r=n.isWebGL2;return{convert:function(e){var n;if(e===Se)return 5121;if(e===Ne)return 32819;if(e===je)return 32820;if(e===Re)return 33635;if(e===Te)return 5120;if(e===Ee)return 5122;if(e===Oe)return 5123;if(e===ke)return 5124;if(e===Le)return 5125;if(e===De)return 5126;if(e===Pe)return r?5131:null!==(n=t.get("OES_texture_half_float"))?n.HALF_FLOAT_OES:null;if(e===Fe)return 6406;if(e===Be)return 6407;if(e===He)return 6408;if(e===Ue)return 6409;if(e===Ve)return 6410;if(e===We)return 6402;if(e===Ge)return 34041;if(e===Xe)return 6403;if(e===qe)return 36244;if(e===Ze)return 33319;if(e===Qe)return 33320;if(e===Ke)return 36248;if(e===Je)return 36249;if(e===$e||e===et||e===tt||e===nt){if(null===(n=t.get("WEBGL_compressed_texture_s3tc")))return null;if(e===$e)return n.COMPRESSED_RGB_S3TC_DXT1_EXT;if(e===et)return n.COMPRESSED_RGBA_S3TC_DXT1_EXT;if(e===tt)return n.COMPRESSED_RGBA_S3TC_DXT3_EXT;if(e===nt)return n.COMPRESSED_RGBA_S3TC_DXT5_EXT}if(e===rt||e===it||e===ot||e===at){if(null===(n=t.get("WEBGL_compressed_texture_pvrtc")))return null;if(e===rt)return n.COMPRESSED_RGB_PVRTC_4BPPV1_IMG;if(e===it)return n.COMPRESSED_RGB_PVRTC_2BPPV1_IMG;if(e===ot)return n.COMPRESSED_RGBA_PVRTC_4BPPV1_IMG;if(e===at)return n.COMPRESSED_RGBA_PVRTC_2BPPV1_IMG}if(e===st)return null!==(n=t.get("WEBGL_compressed_texture_etc1"))?n.COMPRESSED_RGB_ETC1_WEBGL:null;if((e===lt||e===ut)&&null!==(n=t.get("WEBGL_compressed_texture_etc"))){if(e===lt)return n.COMPRESSED_RGB8_ETC2;if(e===ut)return n.COMPRESSED_RGBA8_ETC2_EAC}return e===ct||e===dt||e===ht||e===ft||e===pt||e===gt||e===vt||e===mt||e===yt||e===bt||e===Ct||e===At||e===_t||e===It||e===wt||e===Mt||e===St||e===Tt||e===Et||e===Ot||e===kt||e===Lt||e===Dt||e===Pt||e===Nt||e===jt||e===Rt||e===zt?null!==(n=t.get("WEBGL_compressed_texture_astc"))?e:null:e===xt?null!==(n=t.get("EXT_texture_compression_bptc"))?e:null:e===ze?r?34042:null!==(n=t.get("WEBGL_depth_texture"))?n.UNSIGNED_INT_24_8_WEBGL:null:void 0}}}function Os(e){xo.call(this),this.cameras=e||[]}function ks(){wr.call(this),this.type="Group"}function Ls(){this._targetRay=null,this._grip=null}function Ds(e,t){var n=this,r=null,i=1,o=null,a="local-floor",s=null,l=[],u=new Map,c=new xo;c.layers.enable(1),c.viewport=new Xn;var d=new xo;d.layers.enable(2),d.viewport=new Xn;var h=[c,d],f=new Os;f.layers.enable(1),f.layers.enable(2);var p=null,g=null;function v(e){var t=u.get(e.inputSource);t&&t.dispatchEvent({type:e.type})}function m(){u.forEach((function(e,t){e.disconnect(t)})),u.clear(),e.setFramebuffer(null),e.setRenderTarget(e.getRenderTarget()),x.stop(),n.isPresenting=!1,n.dispatchEvent({type:"sessionend"})}function y(e){o=e,x.setContext(r),x.start(),n.isPresenting=!0,n.dispatchEvent({type:"sessionstart"})}function b(e){for(var t=r.inputSources,n=0;n<l.length;n++)u.set(t[n],l[n]);for(n=0;n<e.removed.length;n++){var i=e.removed[n];(o=u.get(i))&&(o.dispatchEvent({type:"disconnected",data:i}),u.delete(i))}for(n=0;n<e.added.length;n++){var o;i=e.added[n];(o=u.get(i))&&o.dispatchEvent({type:"connected",data:i})}}this.enabled=!1,this.isPresenting=!1,this.getController=function(e){var t=l[e];return void 0===t&&(t=new Ls,l[e]=t),t.getTargetRaySpace()},this.getControllerGrip=function(e){var t=l[e];return void 0===t&&(t=new Ls,l[e]=t),t.getGripSpace()},this.setFramebufferScaleFactor=function(e){i=e,!0===n.isPresenting&&console.warn("THREE.WebXRManager: Cannot change framebuffer scale while presenting.")},this.setReferenceSpaceType=function(e){a=e,!0===n.isPresenting&&console.warn("THREE.WebXRManager: Cannot change reference space type while presenting.")},this.getReferenceSpace=function(){return o},this.getSession=function(){return r},this.setSession=function(e){if(null!==(r=e)){r.addEventListener("select",v),r.addEventListener("selectstart",v),r.addEventListener("selectend",v),r.addEventListener("squeeze",v),r.addEventListener("squeezestart",v),r.addEventListener("squeezeend",v),r.addEventListener("end",m);var n=t.getContextAttributes();!0!==n.xrCompatible&&t.makeXRCompatible();var o={antialias:n.antialias,alpha:n.alpha,depth:n.depth,stencil:n.stencil,framebufferScaleFactor:i},s=new XRWebGLLayer(r,t,o);r.updateRenderState({baseLayer:s}),r.requestReferenceSpace(a).then(y),r.addEventListener("inputsourceschange",b)}};var C=new $n,A=new $n;function _(e,t){null===t?e.matrixWorld.copy(e.matrix):e.matrixWorld.multiplyMatrices(t.matrixWorld,e.matrix),e.matrixWorldInverse.getInverse(e.matrixWorld)}this.getCamera=function(e){f.near=d.near=c.near=e.near,f.far=d.far=c.far=e.far,p===f.near&&g===f.far||(r.updateRenderState({depthNear:f.near,depthFar:f.far}),p=f.near,g=f.far);var t=e.parent,n=f.cameras;_(f,t);for(var i=0;i<n.length;i++)_(n[i],t);e.matrixWorld.copy(f.matrixWorld);for(var o=e.children,a=(i=0,o.length);i<a;i++)o[i].updateMatrixWorld(!0);return 2===n.length?function(e,t,n){C.setFromMatrixPosition(t.matrixWorld),A.setFromMatrixPosition(n.matrixWorld);var r=C.distanceTo(A),i=t.projectionMatrix.elements,o=n.projectionMatrix.elements,a=i[14]/(i[10]-1),s=i[14]/(i[10]+1),l=(i[9]+1)/i[5],u=(i[9]-1)/i[5],c=(i[8]-1)/i[0],d=(o[8]+1)/o[0],h=a*c,f=a*d,p=r/(-c+d),g=p*-c;t.matrixWorld.decompose(e.position,e.quaternion,e.scale),e.translateX(g),e.translateZ(p),e.matrixWorld.compose(e.position,e.quaternion,e.scale),e.matrixWorldInverse.getInverse(e.matrixWorld);var v=a+p,m=s+p,y=h-g,b=f+(r-g),_=l*s/m*v,I=u*s/m*v;e.projectionMatrix.makePerspective(y,b,_,I,v,m)}(f,c,d):f.projectionMatrix.copy(c.projectionMatrix),f};var I=null;var x=new Lo;x.setAnimationLoop((function(t,n){if(null!==(s=n.getViewerPose(o))){var i=s.views,a=r.renderState.baseLayer;e.setFramebuffer(a.framebuffer);var u=!1;i.length!==f.cameras.length&&(f.cameras.length=0,u=!0);for(var c=0;c<i.length;c++){var d=i[c],p=a.getViewport(d),g=h[c];g.matrix.fromArray(d.transform.matrix),g.projectionMatrix.fromArray(d.projectionMatrix),g.viewport.set(p.x,p.y,p.width,p.height),0===c&&f.matrix.copy(g.matrix),!0===u&&f.cameras.push(g)}}var v=r.inputSources;for(c=0;c<l.length;c++){var m=l[c],y=v[c];m.update(y,n,o)}I&&I(t,n)})),this.setAnimationLoop=function(e){I=e},this.dispose=function(){}}function Ps(e){function t(t,n,r){t.opacity.value=n.opacity,n.color&&t.diffuse.value.copy(n.color),n.emissive&&t.emissive.value.copy(n.emissive).multiplyScalar(n.emissiveIntensity),n.map&&(t.map.value=n.map),n.alphaMap&&(t.alphaMap.value=n.alphaMap),n.specularMap&&(t.specularMap.value=n.specularMap);var i,o,a=n.envMap||r;a&&(t.envMap.value=a,t.flipEnvMap.value=a.isCubeTexture?-1:1,t.reflectivity.value=n.reflectivity,t.refractionRatio.value=n.refractionRatio,t.maxMipLevel.value=e.get(a).__maxMipLevel),n.lightMap&&(t.lightMap.value=n.lightMap,t.lightMapIntensity.value=n.lightMapIntensity),n.aoMap&&(t.aoMap.value=n.aoMap,t.aoMapIntensity.value=n.aoMapIntensity),n.map?i=n.map:n.specularMap?i=n.specularMap:n.displacementMap?i=n.displacementMap:n.normalMap?i=n.normalMap:n.bumpMap?i=n.bumpMap:n.roughnessMap?i=n.roughnessMap:n.metalnessMap?i=n.metalnessMap:n.alphaMap?i=n.alphaMap:n.emissiveMap&&(i=n.emissiveMap),void 0!==i&&(i.isWebGLRenderTarget&&(i=i.texture),!0===i.matrixAutoUpdate&&i.updateMatrix(),t.uvTransform.value.copy(i.matrix)),n.aoMap?o=n.aoMap:n.lightMap&&(o=n.lightMap),void 0!==o&&(o.isWebGLRenderTarget&&(o=o.texture),!0===o.matrixAutoUpdate&&o.updateMatrix(),t.uv2Transform.value.copy(o.matrix))}function n(e,t,n){e.roughness.value=t.roughness,e.metalness.value=t.metalness,t.roughnessMap&&(e.roughnessMap.value=t.roughnessMap),t.metalnessMap&&(e.metalnessMap.value=t.metalnessMap),t.emissiveMap&&(e.emissiveMap.value=t.emissiveMap),t.bumpMap&&(e.bumpMap.value=t.bumpMap,e.bumpScale.value=t.bumpScale,t.side===m&&(e.bumpScale.value*=-1)),t.normalMap&&(e.normalMap.value=t.normalMap,e.normalScale.value.copy(t.normalScale),t.side===m&&e.normalScale.value.negate()),t.displacementMap&&(e.displacementMap.value=t.displacementMap,e.displacementScale.value=t.displacementScale,e.displacementBias.value=t.displacementBias),(t.envMap||n)&&(e.envMapIntensity.value=t.envMapIntensity)}return{refreshFogUniforms:function(e,t){e.fogColor.value.copy(t.color),t.isFog?(e.fogNear.value=t.near,e.fogFar.value=t.far):t.isFogExp2&&(e.fogDensity.value=t.density)},refreshMaterialUniforms:function(e,r,i,o,a){r.isMeshBasicMaterial?t(e,r):r.isMeshLambertMaterial?(t(e,r),function(e,t){t.emissiveMap&&(e.emissiveMap.value=t.emissiveMap)}(e,r)):r.isMeshToonMaterial?(t(e,r),function(e,t){e.specular.value.copy(t.specular),e.shininess.value=Math.max(t.shininess,1e-4),t.gradientMap&&(e.gradientMap.value=t.gradientMap);t.emissiveMap&&(e.emissiveMap.value=t.emissiveMap);t.bumpMap&&(e.bumpMap.value=t.bumpMap,e.bumpScale.value=t.bumpScale,t.side===m&&(e.bumpScale.value*=-1));t.normalMap&&(e.normalMap.value=t.normalMap,e.normalScale.value.copy(t.normalScale),t.side===m&&e.normalScale.value.negate());t.displacementMap&&(e.displacementMap.value=t.displacementMap,e.displacementScale.value=t.displacementScale,e.displacementBias.value=t.displacementBias)}(e,r)):r.isMeshPhongMaterial?(t(e,r),function(e,t){e.specular.value.copy(t.specular),e.shininess.value=Math.max(t.shininess,1e-4),t.emissiveMap&&(e.emissiveMap.value=t.emissiveMap);t.bumpMap&&(e.bumpMap.value=t.bumpMap,e.bumpScale.value=t.bumpScale,t.side===m&&(e.bumpScale.value*=-1));t.normalMap&&(e.normalMap.value=t.normalMap,e.normalScale.value.copy(t.normalScale),t.side===m&&e.normalScale.value.negate());t.displacementMap&&(e.displacementMap.value=t.displacementMap,e.displacementScale.value=t.displacementScale,e.displacementBias.value=t.displacementBias)}(e,r)):r.isMeshStandardMaterial?(t(e,r,i),r.isMeshPhysicalMaterial?function(e,t,r){n(e,t,r),e.reflectivity.value=t.reflectivity,e.clearcoat.value=t.clearcoat,e.clearcoatRoughness.value=t.clearcoatRoughness,t.sheen&&e.sheen.value.copy(t.sheen);t.clearcoatMap&&(e.clearcoatMap.value=t.clearcoatMap);t.clearcoatRoughnessMap&&(e.clearcoatRoughnessMap.value=t.clearcoatRoughnessMap);t.clearcoatNormalMap&&(e.clearcoatNormalScale.value.copy(t.clearcoatNormalScale),e.clearcoatNormalMap.value=t.clearcoatNormalMap,t.side===m&&e.clearcoatNormalScale.value.negate());e.transparency.value=t.transparency}(e,r,i):n(e,r,i)):r.isMeshMatcapMaterial?(t(e,r),function(e,t){t.matcap&&(e.matcap.value=t.matcap);t.bumpMap&&(e.bumpMap.value=t.bumpMap,e.bumpScale.value=t.bumpScale,t.side===m&&(e.bumpScale.value*=-1));t.normalMap&&(e.normalMap.value=t.normalMap,e.normalScale.value.copy(t.normalScale),t.side===m&&e.normalScale.value.negate());t.displacementMap&&(e.displacementMap.value=t.displacementMap,e.displacementScale.value=t.displacementScale,e.displacementBias.value=t.displacementBias)}(e,r)):r.isMeshDepthMaterial?(t(e,r),function(e,t){t.displacementMap&&(e.displacementMap.value=t.displacementMap,e.displacementScale.value=t.displacementScale,e.displacementBias.value=t.displacementBias)}(e,r)):r.isMeshDistanceMaterial?(t(e,r),function(e,t){t.displacementMap&&(e.displacementMap.value=t.displacementMap,e.displacementScale.value=t.displacementScale,e.displacementBias.value=t.displacementBias);e.referencePosition.value.copy(t.referencePosition),e.nearDistance.value=t.nearDistance,e.farDistance.value=t.farDistance}(e,r)):r.isMeshNormalMaterial?(t(e,r),function(e,t){t.bumpMap&&(e.bumpMap.value=t.bumpMap,e.bumpScale.value=t.bumpScale,t.side===m&&(e.bumpScale.value*=-1));t.normalMap&&(e.normalMap.value=t.normalMap,e.normalScale.value.copy(t.normalScale),t.side===m&&e.normalScale.value.negate());t.displacementMap&&(e.displacementMap.value=t.displacementMap,e.displacementScale.value=t.displacementScale,e.displacementBias.value=t.displacementBias)}(e,r)):r.isLineBasicMaterial?(function(e,t){e.diffuse.value.copy(t.color),e.opacity.value=t.opacity}(e,r),r.isLineDashedMaterial&&function(e,t){e.dashSize.value=t.dashSize,e.totalSize.value=t.dashSize+t.gapSize,e.scale.value=t.scale}(e,r)):r.isPointsMaterial?function(e,t,n,r){e.diffuse.value.copy(t.color),e.opacity.value=t.opacity,e.size.value=t.size*n,e.scale.value=.5*r,t.map&&(e.map.value=t.map);t.alphaMap&&(e.alphaMap.value=t.alphaMap);var i;t.map?i=t.map:t.alphaMap&&(i=t.alphaMap);void 0!==i&&(!0===i.matrixAutoUpdate&&i.updateMatrix(),e.uvTransform.value.copy(i.matrix))}(e,r,o,a):r.isSpriteMaterial?function(e,t){e.diffuse.value.copy(t.color),e.opacity.value=t.opacity,e.rotation.value=t.rotation,t.map&&(e.map.value=t.map);t.alphaMap&&(e.alphaMap.value=t.alphaMap);var n;t.map?n=t.map:t.alphaMap&&(n=t.alphaMap);void 0!==n&&(!0===n.matrixAutoUpdate&&n.updateMatrix(),e.uvTransform.value.copy(n.matrix))}(e,r):r.isShadowMaterial?(e.color.value.copy(r.color),e.opacity.value=r.opacity):r.isShaderMaterial&&(r.uniformsNeedUpdate=!1)}}}function Ns(e){var t=void 0!==(e=e||{}).canvas?e.canvas:document.createElementNS("http://www.w3.org/1999/xhtml","canvas"),n=void 0!==e.context?e.context:null,r=void 0!==e.alpha&&e.alpha,i=void 0===e.depth||e.depth,o=void 0===e.stencil||e.stencil,a=void 0!==e.antialias&&e.antialias,s=void 0===e.premultipliedAlpha||e.premultipliedAlpha,l=void 0!==e.preserveDrawingBuffer&&e.preserveDrawingBuffer,u=void 0!==e.powerPreference?e.powerPreference:"default",c=void 0!==e.failIfMajorPerformanceCaveat&&e.failIfMajorPerformanceCaveat,d=null,h=null;this.domElement=t,this.debug={checkShaderErrors:!0},this.autoClear=!0,this.autoClearColor=!0,this.autoClearDepth=!0,this.autoClearStencil=!0,this.sortObjects=!0,this.clippingPlanes=[],this.localClippingEnabled=!1,this.gammaFactor=2,this.outputEncoding=$t,this.physicallyCorrectLights=!1,this.toneMapping=ee,this.toneMappingExposure=1,this.toneMappingWhitePoint=1,this.maxMorphTargets=8,this.maxMorphNormals=4;var f,p,g,v,m,y,b,C,A,_,I,x,w,M,S,T,E,O,k,L=this,D=!1,P=null,N=0,j=0,R=null,z=null,F=-1,B={geometry:null,program:null,wireframe:!1},H=null,U=null,V=new Xn,Y=new Xn,W=null,G=t.width,X=t.height,q=1,Z=null,Q=null,K=new Xn(0,0,G,X),J=new Xn(0,0,G,X),$=!1,te=new Oo,ne=new Ho,re=!1,ie=!1,oe=new sr,ae=new $n;function se(){return null===R?q:1}try{var le={alpha:r,depth:i,stencil:o,antialias:a,premultipliedAlpha:s,preserveDrawingBuffer:l,powerPreference:u,failIfMajorPerformanceCaveat:c};if(t.addEventListener("webglcontextlost",he,!1),t.addEventListener("webglcontextrestored",fe,!1),null===(f=n||t.getContext("webgl",le)||t.getContext("experimental-webgl",le)))throw null!==t.getContext("webgl")?new Error("Error creating WebGL context with your selected attributes."):new Error("Error creating WebGL context.");void 0===f.getShaderPrecisionFormat&&(f.getShaderPrecisionFormat=function(){return{rangeMin:1,rangeMax:1,precision:1}})}catch(e){throw console.error("THREE.WebGLRenderer: "+e.message),e}function ue(){p=new Uo(f),!1===(g=new Bo(f,p,e)).isWebGL2&&(p.get("WEBGL_depth_texture"),p.get("OES_texture_float"),p.get("OES_texture_half_float"),p.get("OES_texture_half_float_linear"),p.get("OES_standard_derivatives"),p.get("OES_element_index_uint"),p.get("ANGLE_instanced_arrays")),p.get("OES_texture_float_linear"),k=new Es(f,p,g),(v=new Ss(f,p,g)).scissor(Y.copy(J).multiplyScalar(q).floor()),v.viewport(V.copy(K).multiplyScalar(q).floor()),m=new Wo(f),y=new fs,b=new Ts(f,p,v,y,g,k,m),C=new Do(f,g),A=new Vo(f,C,m),_=new qo(f,A,C,m),T=new Xo(f),I=new hs(L,p,g),x=new Ps(y),w=new ms,M=new Is,S=new zo(L,v,_,s),E=new Fo(f,p,m,g),O=new Yo(f,p,m,g),m.programs=I.programs,L.capabilities=g,L.extensions=p,L.properties=y,L.renderLists=w,L.state=v,L.info=m}ue();var ce=new Ds(L,f);this.xr=ce;var de=new Ms(L,_,g.maxTextureSize);function he(e){e.preventDefault(),console.log("THREE.WebGLRenderer: Context Lost."),D=!0}function fe(){console.log("THREE.WebGLRenderer: Context Restored."),D=!1,ue()}function pe(e){var t=e.target;t.removeEventListener("dispose",pe),function(e){ge(e),y.remove(e)}(t)}function ge(e){var t=y.get(e).program;e.program=void 0,void 0!==t&&I.releaseProgram(t)}this.shadowMap=de,this.getContext=function(){return f},this.getContextAttributes=function(){return f.getContextAttributes()},this.forceContextLoss=function(){var e=p.get("WEBGL_lose_context");e&&e.loseContext()},this.forceContextRestore=function(){var e=p.get("WEBGL_lose_context");e&&e.restoreContext()},this.getPixelRatio=function(){return q},this.setPixelRatio=function(e){void 0!==e&&(q=e,this.setSize(G,X,!1))},this.getSize=function(e){return void 0===e&&(console.warn("WebGLRenderer: .getsize() now requires a Vector2 as an argument"),e=new Un),e.set(G,X)},this.setSize=function(e,n,r){ce.isPresenting?console.warn("THREE.WebGLRenderer: Can't change size while VR device is presenting."):(G=e,X=n,t.width=Math.floor(e*q),t.height=Math.floor(n*q),!1!==r&&(t.style.width=e+"px",t.style.height=n+"px"),this.setViewport(0,0,e,n))},this.getDrawingBufferSize=function(e){return void 0===e&&(console.warn("WebGLRenderer: .getdrawingBufferSize() now requires a Vector2 as an argument"),e=new Un),e.set(G*q,X*q).floor()},this.setDrawingBufferSize=function(e,n,r){G=e,X=n,q=r,t.width=Math.floor(e*r),t.height=Math.floor(n*r),this.setViewport(0,0,e,n)},this.getCurrentViewport=function(e){return void 0===e&&(console.warn("WebGLRenderer: .getCurrentViewport() now requires a Vector4 as an argument"),e=new Xn),e.copy(V)},this.getViewport=function(e){return e.copy(K)},this.setViewport=function(e,t,n,r){e.isVector4?K.set(e.x,e.y,e.z,e.w):K.set(e,t,n,r),v.viewport(V.copy(K).multiplyScalar(q).floor())},this.getScissor=function(e){return e.copy(J)},this.setScissor=function(e,t,n,r){e.isVector4?J.set(e.x,e.y,e.z,e.w):J.set(e,t,n,r),v.scissor(Y.copy(J).multiplyScalar(q).floor())},this.getScissorTest=function(){return $},this.setScissorTest=function(e){v.setScissorTest($=e)},this.setOpaqueSort=function(e){Z=e},this.setTransparentSort=function(e){Q=e},this.getClearColor=function(){return S.getClearColor()},this.setClearColor=function(){S.setClearColor.apply(S,arguments)},this.getClearAlpha=function(){return S.getClearAlpha()},this.setClearAlpha=function(){S.setClearAlpha.apply(S,arguments)},this.clear=function(e,t,n){var r=0;(void 0===e||e)&&(r|=16384),(void 0===t||t)&&(r|=256),(void 0===n||n)&&(r|=1024),f.clear(r)},this.clearColor=function(){this.clear(!0,!1,!1)},this.clearDepth=function(){this.clear(!1,!0,!1)},this.clearStencil=function(){this.clear(!1,!1,!0)},this.dispose=function(){t.removeEventListener("webglcontextlost",he,!1),t.removeEventListener("webglcontextrestored",fe,!1),w.dispose(),M.dispose(),y.dispose(),_.dispose(),ce.dispose(),ye.stop()},this.renderBufferImmediate=function(e,t){v.initAttributes();var n=y.get(e);e.hasPositions&&!n.position&&(n.position=f.createBuffer()),e.hasNormals&&!n.normal&&(n.normal=f.createBuffer()),e.hasUvs&&!n.uv&&(n.uv=f.createBuffer()),e.hasColors&&!n.color&&(n.color=f.createBuffer());var r=t.getAttributes();e.hasPositions&&(f.bindBuffer(34962,n.position),f.bufferData(34962,e.positionArray,35048),v.enableAttribute(r.position),f.vertexAttribPointer(r.position,3,5126,!1,0,0)),e.hasNormals&&(f.bindBuffer(34962,n.normal),f.bufferData(34962,e.normalArray,35048),v.enableAttribute(r.normal),f.vertexAttribPointer(r.normal,3,5126,!1,0,0)),e.hasUvs&&(f.bindBuffer(34962,n.uv),f.bufferData(34962,e.uvArray,35048),v.enableAttribute(r.uv),f.vertexAttribPointer(r.uv,2,5126,!1,0,0)),e.hasColors&&(f.bindBuffer(34962,n.color),f.bufferData(34962,e.colorArray,35048),v.enableAttribute(r.color),f.vertexAttribPointer(r.color,3,5126,!1,0,0)),v.disableUnusedAttributes(),f.drawArrays(4,0,e.count),e.count=0};var ve=new Mr;this.renderBufferDirect=function(e,t,n,r,i,o){null===t&&(t=ve);var a=i.isMesh&&i.matrixWorld.determinant()<0,s=Ie(e,t,r,i);v.setMaterial(r,a);var l=!1;B.geometry===n.id&&B.program===s.id&&B.wireframe===(!0===r.wireframe)||(B.geometry=n.id,B.program=s.id,B.wireframe=!0===r.wireframe,l=!0),(r.morphTargets||r.morphNormals)&&(T.update(i,n,r,s),l=!0),!0===i.isInstancedMesh&&(l=!0);var u=n.index,c=n.attributes.position;if(null===u){if(void 0===c||0===c.count)return}else if(0===u.count)return;var d,h=1;!0===r.wireframe&&(u=A.getWireframeAttribute(n),h=2);var m=E;null!==u&&(d=C.get(u),(m=O).setIndex(d)),l&&(!function(e,t,n,r){if(!1===g.isWebGL2&&(e.isInstancedMesh||t.isInstancedBufferGeometry)&&null===p.get("ANGLE_instanced_arrays"))return;v.initAttributes();var i=t.attributes,o=r.getAttributes(),a=n.defaultAttributeValues;for(var s in o){var l=o[s];if(l>=0){var u=i[s];if(void 0!==u){var c=u.normalized,d=u.itemSize;if(void 0===(I=C.get(u)))continue;var h=I.buffer,m=I.type,y=I.bytesPerElement;if(u.isInterleavedBufferAttribute){var b=u.data,A=b.stride,_=u.offset;b&&b.isInstancedInterleavedBuffer?(v.enableAttributeAndDivisor(l,b.meshPerAttribute),void 0===t._maxInstanceCount&&(t._maxInstanceCount=b.meshPerAttribute*b.count)):v.enableAttribute(l),f.bindBuffer(34962,h),v.vertexAttribPointer(l,d,m,c,A*y,_*y)}else u.isInstancedBufferAttribute?(v.enableAttributeAndDivisor(l,u.meshPerAttribute),void 0===t._maxInstanceCount&&(t._maxInstanceCount=u.meshPerAttribute*u.count)):v.enableAttribute(l),f.bindBuffer(34962,h),v.vertexAttribPointer(l,d,m,c,0,0)}else if("instanceMatrix"===s){var I;if(void 0===(I=C.get(e.instanceMatrix)))continue;h=I.buffer,m=I.type;v.enableAttributeAndDivisor(l+0,1),v.enableAttributeAndDivisor(l+1,1),v.enableAttributeAndDivisor(l+2,1),v.enableAttributeAndDivisor(l+3,1),f.bindBuffer(34962,h),f.vertexAttribPointer(l+0,4,m,!1,64,0),f.vertexAttribPointer(l+1,4,m,!1,64,16),f.vertexAttribPointer(l+2,4,m,!1,64,32),f.vertexAttribPointer(l+3,4,m,!1,64,48)}else if(void 0!==a){var x=a[s];if(void 0!==x)switch(x.length){case 2:f.vertexAttrib2fv(l,x);break;case 3:f.vertexAttrib3fv(l,x);break;case 4:f.vertexAttrib4fv(l,x);break;default:f.vertexAttrib1fv(l,x)}}}}v.disableUnusedAttributes()}(i,n,r,s),null!==u&&f.bindBuffer(34963,d.buffer));var y=null!==u?u.count:c.count,b=n.drawRange.start*h,_=n.drawRange.count*h,I=null!==o?o.start*h:0,x=null!==o?o.count*h:1/0,w=Math.max(b,I),M=Math.min(y,b+_,I+x)-1,S=Math.max(0,M-w+1);if(0!==S){if(i.isMesh)!0===r.wireframe?(v.setLineWidth(r.wireframeLinewidth*se()),m.setMode(1)):m.setMode(4);else if(i.isLine){var k=r.linewidth;void 0===k&&(k=1),v.setLineWidth(k*se()),i.isLineSegments?m.setMode(1):i.isLineLoop?m.setMode(2):m.setMode(3)}else i.isPoints?m.setMode(0):i.isSprite&&m.setMode(4);if(i.isInstancedMesh)m.renderInstances(n,w,S,i.count);else if(n.isInstancedBufferGeometry){var L=Math.min(n.instanceCount,n._maxInstanceCount);m.renderInstances(n,w,S,L)}else m.render(w,S)}},this.compile=function(e,t){(h=M.get(e,t)).init(),e.traverse((function(e){e.isLight&&(h.pushLight(e),e.castShadow&&h.pushShadow(e))})),h.setupLights(t);const n={};e.traverse((function(t){let r=t.material;if(r)if(Array.isArray(r))for(let i=0;i<r.length;i++){let o=r[i];o.uuid in n==!1&&(_e(o,e,t),n[o.uuid]=!0)}else r.uuid in n==!1&&(_e(r,e,t),n[r.uuid]=!0)}))};var me=null;var ye=new Lo;function be(e,t,n,r){if(!1!==e.visible){if(e.layers.test(t.layers))if(e.isGroup)n=e.renderOrder;else if(e.isLOD)!0===e.autoUpdate&&e.update(t);else if(e.isLight)h.pushLight(e),e.castShadow&&h.pushShadow(e);else if(e.isSprite){if(!e.frustumCulled||te.intersectsSprite(e)){r&&ae.setFromMatrixPosition(e.matrixWorld).applyMatrix4(oe);var i=_.update(e);(o=e.material).visible&&d.push(e,i,o,n,ae.z,null)}}else if(e.isImmediateRenderObject)r&&ae.setFromMatrixPosition(e.matrixWorld).applyMatrix4(oe),d.push(e,null,e.material,n,ae.z,null);else if((e.isMesh||e.isLine||e.isPoints)&&(e.isSkinnedMesh&&e.skeleton.frame!==m.render.frame&&(e.skeleton.update(),e.skeleton.frame=m.render.frame),!e.frustumCulled||te.intersectsObject(e))){r&&ae.setFromMatrixPosition(e.matrixWorld).applyMatrix4(oe);i=_.update(e);var o=e.material;if(Array.isArray(o))for(var a=i.groups,s=0,l=a.length;s<l;s++){var u=a[s],c=o[u.materialIndex];c&&c.visible&&d.push(e,i,c,n,ae.z,u)}else o.visible&&d.push(e,i,o,n,ae.z,null)}var f=e.children;for(s=0,l=f.length;s<l;s++)be(f[s],t,n,r)}}function Ce(e,t,n,r){for(var i=0,o=e.length;i<o;i++){var a=e[i],s=a.object,l=a.geometry,u=void 0===r?a.material:r,c=a.group;if(n.isArrayCamera){U=n;for(var d=n.cameras,f=0,p=d.length;f<p;f++){var g=d[f];s.layers.test(g.layers)&&(v.viewport(V.copy(g.viewport)),h.setupLights(g),Ae(s,t,g,l,u,c))}}else U=null,Ae(s,t,n,l,u,c)}}function Ae(e,t,n,r,i,o){if(e.onBeforeRender(L,t,n,r,i,o),h=M.get(t,U||n),e.modelViewMatrix.multiplyMatrices(n.matrixWorldInverse,e.matrixWorld),e.normalMatrix.getNormalMatrix(e.modelViewMatrix),e.isImmediateRenderObject){var a=Ie(n,t,i,e);v.setMaterial(i),B.geometry=null,B.program=null,B.wireframe=!1,function(e,t){e.render((function(e){L.renderBufferImmediate(e,t)}))}(e,a)}else L.renderBufferDirect(n,t,r,i,e,o);e.onAfterRender(L,t,n,r,i,o),h=M.get(t,U||n)}function _e(e,t,n){var r=y.get(e),i=h.state.lights,o=h.state.shadowsArray,a=i.state.version,s=I.getParameters(e,i.state,o,t,ne.numPlanes,ne.numIntersection,n),l=I.getProgramCacheKey(s),u=r.program,c=!0;if(void 0===u)e.addEventListener("dispose",pe);else if(u.cacheKey!==l)ge(e);else if(r.lightsStateVersion!==a)r.lightsStateVersion=a,c=!1;else{if(void 0!==s.shaderID)return;c=!1}c&&(u=I.acquireProgram(s,l),r.program=u,r.uniforms=s.uniforms,r.outputEncoding=s.outputEncoding,e.program=u);var d=u.getAttributes();if(e.morphTargets){e.numSupportedMorphTargets=0;for(var f=0;f<L.maxMorphTargets;f++)d["morphTarget"+f]>=0&&e.numSupportedMorphTargets++}if(e.morphNormals){e.numSupportedMorphNormals=0;for(f=0;f<L.maxMorphNormals;f++)d["morphNormal"+f]>=0&&e.numSupportedMorphNormals++}var p=r.uniforms;(e.isShaderMaterial||e.isRawShaderMaterial)&&!0!==e.clipping||(r.numClippingPlanes=ne.numPlanes,r.numIntersection=ne.numIntersection,p.clippingPlanes=ne.uniform),r.environment=e.isMeshStandardMaterial?t.environment:null,r.fog=t.fog,r.needsLights=function(e){return e.isMeshLambertMaterial||e.isMeshToonMaterial||e.isMeshPhongMaterial||e.isMeshStandardMaterial||e.isShadowMaterial||e.isShaderMaterial&&!0===e.lights}(e),r.lightsStateVersion=a,r.needsLights&&(p.ambientLightColor.value=i.state.ambient,p.lightProbe.value=i.state.probe,p.directionalLights.value=i.state.directional,p.directionalLightShadows.value=i.state.directionalShadow,p.spotLights.value=i.state.spot,p.spotLightShadows.value=i.state.spotShadow,p.rectAreaLights.value=i.state.rectArea,p.pointLights.value=i.state.point,p.pointLightShadows.value=i.state.pointShadow,p.hemisphereLights.value=i.state.hemi,p.directionalShadowMap.value=i.state.directionalShadowMap,p.directionalShadowMatrix.value=i.state.directionalShadowMatrix,p.spotShadowMap.value=i.state.spotShadowMap,p.spotShadowMatrix.value=i.state.spotShadowMatrix,p.pointShadowMap.value=i.state.pointShadowMap,p.pointShadowMatrix.value=i.state.pointShadowMatrix);var g=r.program.getUniforms(),v=Ga.seqWithValue(g.seq,p);r.uniformsList=v}function Ie(e,t,n,r){b.resetTextureUnits();var i=t.fog,o=n.isMeshStandardMaterial?t.environment:null,a=null===R?L.outputEncoding:R.texture.encoding,s=y.get(n),l=h.state.lights;if(re&&(ie||e!==H)){var u=e===H&&n.id===F;ne.setState(n.clippingPlanes,n.clipIntersection,n.clipShadows,e,s,u)}n.version===s.__version?void 0===s.program||n.fog&&s.fog!==i||s.environment!==o||s.needsLights&&s.lightsStateVersion!==l.state.version?_e(n,t,r):void 0===s.numClippingPlanes||s.numClippingPlanes===ne.numPlanes&&s.numIntersection===ne.numIntersection?s.outputEncoding!==a&&_e(n,t,r):_e(n,t,r):(_e(n,t,r),s.__version=n.version);var c,d,p=!1,m=!1,C=!1,A=s.program,_=A.getUniforms(),I=s.uniforms;if(v.useProgram(A.program)&&(p=!0,m=!0,C=!0),n.id!==F&&(F=n.id,m=!0),p||H!==e){if(_.setValue(f,"projectionMatrix",e.projectionMatrix),g.logarithmicDepthBuffer&&_.setValue(f,"logDepthBufFC",2/(Math.log(e.far+1)/Math.LN2)),H!==e&&(H=e,m=!0,C=!0),n.isShaderMaterial||n.isMeshPhongMaterial||n.isMeshToonMaterial||n.isMeshStandardMaterial||n.envMap){var w=_.map.cameraPosition;void 0!==w&&w.setValue(f,ae.setFromMatrixPosition(e.matrixWorld))}(n.isMeshPhongMaterial||n.isMeshToonMaterial||n.isMeshLambertMaterial||n.isMeshBasicMaterial||n.isMeshStandardMaterial||n.isShaderMaterial)&&_.setValue(f,"isOrthographic",!0===e.isOrthographicCamera),(n.isMeshPhongMaterial||n.isMeshToonMaterial||n.isMeshLambertMaterial||n.isMeshBasicMaterial||n.isMeshStandardMaterial||n.isShaderMaterial||n.skinning)&&_.setValue(f,"viewMatrix",e.matrixWorldInverse)}if(n.skinning){_.setOptional(f,r,"bindMatrix"),_.setOptional(f,r,"bindMatrixInverse");var M=r.skeleton;if(M){var S=M.bones;if(g.floatVertexTextures){if(void 0===M.boneTexture){var T=Math.sqrt(4*S.length);T=Hn.ceilPowerOfTwo(T),T=Math.max(T,4);var E=new Float32Array(T*T*4);E.set(M.boneMatrices);var O=new So(E,T,T,He,De);M.boneMatrices=E,M.boneTexture=O,M.boneTextureSize=T}_.setValue(f,"boneTexture",M.boneTexture,b),_.setValue(f,"boneTextureSize",M.boneTextureSize)}else _.setOptional(f,M,"boneMatrices")}}return(m||s.receiveShadow!==r.receiveShadow)&&(s.receiveShadow=r.receiveShadow,_.setValue(f,"receiveShadow",r.receiveShadow)),m&&(_.setValue(f,"toneMappingExposure",L.toneMappingExposure),_.setValue(f,"toneMappingWhitePoint",L.toneMappingWhitePoint),s.needsLights&&(d=C,(c=I).ambientLightColor.needsUpdate=d,c.lightProbe.needsUpdate=d,c.directionalLights.needsUpdate=d,c.directionalLightShadows.needsUpdate=d,c.pointLights.needsUpdate=d,c.pointLightShadows.needsUpdate=d,c.spotLights.needsUpdate=d,c.spotLightShadows.needsUpdate=d,c.rectAreaLights.needsUpdate=d,c.hemisphereLights.needsUpdate=d),i&&n.fog&&x.refreshFogUniforms(I,i),x.refreshMaterialUniforms(I,n,o,q,X),void 0!==I.ltc_1&&(I.ltc_1.value=ko.LTC_1),void 0!==I.ltc_2&&(I.ltc_2.value=ko.LTC_2),Ga.upload(f,s.uniformsList,I,b)),n.isShaderMaterial&&!0===n.uniformsNeedUpdate&&(Ga.upload(f,s.uniformsList,I,b),n.uniformsNeedUpdate=!1),n.isSpriteMaterial&&_.setValue(f,"center",r.center),_.setValue(f,"modelViewMatrix",r.modelViewMatrix),_.setValue(f,"normalMatrix",r.normalMatrix),_.setValue(f,"modelMatrix",r.matrixWorld),A}ye.setAnimationLoop((function(e){ce.isPresenting||me&&me(e)})),"undefined"!=typeof window&&ye.setContext(window),this.setAnimationLoop=function(e){me=e,ce.setAnimationLoop(e),ye.start()},this.render=function(e,t){var n,r;if(void 0!==arguments[2]&&(console.warn("THREE.WebGLRenderer.render(): the renderTarget argument has been removed. Use .setRenderTarget() instead."),n=arguments[2]),void 0!==arguments[3]&&(console.warn("THREE.WebGLRenderer.render(): the forceClear argument has been removed. Use .clear() instead."),r=arguments[3]),t&&t.isCamera){if(!D){B.geometry=null,B.program=null,B.wireframe=!1,F=-1,H=null,!0===e.autoUpdate&&e.updateMatrixWorld(),null===t.parent&&t.updateMatrixWorld(),ce.enabled&&ce.isPresenting&&(t=ce.getCamera(t)),e.onBeforeRender(L,e,t,n||R),(h=M.get(e,t)).init(),oe.multiplyMatrices(t.projectionMatrix,t.matrixWorldInverse),te.setFromProjectionMatrix(oe),ie=this.localClippingEnabled,re=ne.init(this.clippingPlanes,ie,t),(d=w.get(e,t)).init(),be(e,t,0,L.sortObjects),d.finish(),!0===L.sortObjects&&d.sort(Z,Q),re&&ne.beginShadows();var i=h.state.shadowsArray;de.render(i,e,t),h.setupLights(t),re&&ne.endShadows(),this.info.autoReset&&this.info.reset(),void 0!==n&&this.setRenderTarget(n),S.render(d,e,t,r);var o=d.opaque,a=d.transparent;if(e.overrideMaterial){var s=e.overrideMaterial;o.length&&Ce(o,e,t,s),a.length&&Ce(a,e,t,s)}else o.length&&Ce(o,e,t),a.length&&Ce(a,e,t);e.onAfterRender(L,e,t),null!==R&&(b.updateRenderTargetMipmap(R),b.updateMultisampleRenderTarget(R)),v.buffers.depth.setTest(!0),v.buffers.depth.setMask(!0),v.buffers.color.setMask(!0),v.setPolygonOffset(!1),d=null,h=null}}else console.error("THREE.WebGLRenderer.render: camera is not an instance of THREE.Camera.")},this.setFramebuffer=function(e){P!==e&&null===R&&f.bindFramebuffer(36160,e),P=e},this.getActiveCubeFace=function(){return N},this.getActiveMipmapLevel=function(){return j},this.getRenderTarget=function(){return R},this.setRenderTarget=function(e,t,n){R=e,N=t,j=n,e&&void 0===y.get(e).__webglFramebuffer&&b.setupRenderTarget(e);var r=P,i=!1;if(e){var o=y.get(e).__webglFramebuffer;e.isWebGLCubeRenderTarget?(r=o[t||0],i=!0):r=e.isWebGLMultisampleRenderTarget?y.get(e).__webglMultisampledFramebuffer:o,V.copy(e.viewport),Y.copy(e.scissor),W=e.scissorTest}else V.copy(K).multiplyScalar(q).floor(),Y.copy(J).multiplyScalar(q).floor(),W=$;if(z!==r&&(f.bindFramebuffer(36160,r),z=r),v.viewport(V),v.scissor(Y),v.setScissorTest(W),i){var a=y.get(e.texture);f.framebufferTexture2D(36160,36064,34069+(t||0),a.__webglTexture,n||0)}},this.readRenderTargetPixels=function(e,t,n,r,i,o,a){if(e&&e.isWebGLRenderTarget){var s=y.get(e).__webglFramebuffer;if(e.isWebGLCubeRenderTarget&&void 0!==a&&(s=s[a]),s){var l=!1;s!==z&&(f.bindFramebuffer(36160,s),l=!0);try{var u=e.texture,c=u.format,d=u.type;if(c!==He&&k.convert(c)!==f.getParameter(35739))return void console.error("THREE.WebGLRenderer.readRenderTargetPixels: renderTarget is not in RGBA or implementation defined format.");if(!(d===Se||k.convert(d)===f.getParameter(35738)||d===De&&(g.isWebGL2||p.get("OES_texture_float")||p.get("WEBGL_color_buffer_float"))||d===Pe&&(g.isWebGL2?p.get("EXT_color_buffer_float"):p.get("EXT_color_buffer_half_float"))))return void console.error("THREE.WebGLRenderer.readRenderTargetPixels: renderTarget is not in UnsignedByteType or implementation defined type.");36053===f.checkFramebufferStatus(36160)?t>=0&&t<=e.width-r&&n>=0&&n<=e.height-i&&f.readPixels(t,n,r,i,k.convert(c),k.convert(d),o):console.error("THREE.WebGLRenderer.readRenderTargetPixels: readPixels from renderTarget failed. Framebuffer not complete.")}finally{l&&f.bindFramebuffer(36160,z)}}}else console.error("THREE.WebGLRenderer.readRenderTargetPixels: renderTarget is not THREE.WebGLRenderTarget.")},this.copyFramebufferToTexture=function(e,t,n){void 0===n&&(n=0);var r=Math.pow(2,-n),i=Math.floor(t.image.width*r),o=Math.floor(t.image.height*r),a=k.convert(t.format);b.setTexture2D(t,0),f.copyTexImage2D(3553,n,a,e.x,e.y,i,o,0),v.unbindTexture()},this.copyTextureToTexture=function(e,t,n,r){void 0===r&&(r=0);var i=t.image.width,o=t.image.height,a=k.convert(n.format),s=k.convert(n.type);b.setTexture2D(n,0),t.isDataTexture?f.texSubImage2D(3553,r,e.x,e.y,i,o,a,s,t.image.data):t.isCompressedTexture?f.compressedTexSubImage2D(3553,r,e.x,e.y,t.mipmaps[0].width,t.mipmaps[0].height,a,t.mipmaps[0].data):f.texSubImage2D(3553,r,e.x,e.y,a,s,t.image),0===r&&n.generateMipmaps&&f.generateMipmap(3553),v.unbindTexture()},this.initTexture=function(e){b.setTexture2D(e,0),v.unbindTexture()},"undefined"!=typeof __THREE_DEVTOOLS__&&__THREE_DEVTOOLS__.dispatchEvent(new CustomEvent("observe",{detail:this}))}function js(e,t){this.name="",this.color=new vi(e),this.density=void 0!==t?t:25e-5}function Rs(e,t,n){this.name="",this.color=new vi(e),this.near=void 0!==t?t:1,this.far=void 0!==n?n:1e3}function zs(e,t){this.array=e,this.stride=t,this.count=void 0!==e?e.length/t:0,this.usage=Tn,this.updateRange={offset:0,count:-1},this.version=0}Os.prototype=Object.assign(Object.create(xo.prototype),{constructor:Os,isArrayCamera:!0}),ks.prototype=Object.assign(Object.create(wr.prototype),{constructor:ks,isGroup:!0}),Object.assign(Ls.prototype,{constructor:Ls,getTargetRaySpace:function(){return null===this._targetRay&&(this._targetRay=new ks,this._targetRay.matrixAutoUpdate=!1,this._targetRay.visible=!1),this._targetRay},getGripSpace:function(){return null===this._grip&&(this._grip=new ks,this._grip.matrixAutoUpdate=!1,this._grip.visible=!1),this._grip},dispatchEvent:function(e){return null!==this._targetRay&&this._targetRay.dispatchEvent(e),null!==this._grip&&this._grip.dispatchEvent(e),this},disconnect:function(e){return this.dispatchEvent({type:"disconnected",data:e}),null!==this._targetRay&&(this._targetRay.visible=!1),null!==this._grip&&(this._grip.visible=!1),this},update:function(e,t,n){var r=null,i=null,o=this._targetRay,a=this._grip;return e&&(null!==o&&null!==(r=t.getPose(e.targetRaySpace,n))&&(o.matrix.fromArray(r.transform.matrix),o.matrix.decompose(o.position,o.rotation,o.scale)),null!==a&&e.gripSpace&&null!==(i=t.getPose(e.gripSpace,n))&&(a.matrix.fromArray(i.transform.matrix),a.matrix.decompose(a.position,a.rotation,a.scale))),null!==o&&(o.visible=null!==r),null!==a&&(a.visible=null!==i),this}}),Object.assign(Ds.prototype,Rn.prototype),Object.assign(js.prototype,{isFogExp2:!0,clone:function(){return new js(this.color,this.density)},toJSON:function(){return{type:"FogExp2",color:this.color.getHex(),density:this.density}}}),Object.assign(Rs.prototype,{isFog:!0,clone:function(){return new Rs(this.color,this.near,this.far)},toJSON:function(){return{type:"Fog",color:this.color.getHex(),near:this.near,far:this.far}}}),Object.defineProperty(zs.prototype,"needsUpdate",{set:function(e){!0===e&&this.version++}}),Object.assign(zs.prototype,{isInterleavedBuffer:!0,onUploadCallback:function(){},setUsage:function(e){return this.usage=e,this},copy:function(e){return this.array=new e.array.constructor(e.array),this.count=e.count,this.stride=e.stride,this.usage=e.usage,this},copyAt:function(e,t,n){e*=this.stride,n*=t.stride;for(var r=0,i=this.stride;r<i;r++)this.array[e+r]=t.array[n+r];return this},set:function(e,t){return void 0===t&&(t=0),this.array.set(e,t),this},clone:function(){return(new this.constructor).copy(this)},onUpload:function(e){return this.onUploadCallback=e,this}});var Fs,Bs=new $n;function Hs(e,t,n,r){this.name="",this.data=e,this.itemSize=t,this.offset=n,this.normalized=!0===r}function Us(e){_i.call(this),this.type="SpriteMaterial",this.color=new vi(16777215),this.map=null,this.alphaMap=null,this.rotation=0,this.sizeAttenuation=!0,this.transparent=!0,this.setValues(e)}Object.defineProperties(Hs.prototype,{count:{get:function(){return this.data.count}},array:{get:function(){return this.data.array}}}),Object.assign(Hs.prototype,{isInterleavedBufferAttribute:!0,applyMatrix4:function(e){for(var t=0,n=this.data.count;t<n;t++)Bs.x=this.getX(t),Bs.y=this.getY(t),Bs.z=this.getZ(t),Bs.applyMatrix4(e),this.setXYZ(t,Bs.x,Bs.y,Bs.z);return this},setX:function(e,t){return this.data.array[e*this.data.stride+this.offset]=t,this},setY:function(e,t){return this.data.array[e*this.data.stride+this.offset+1]=t,this},setZ:function(e,t){return this.data.array[e*this.data.stride+this.offset+2]=t,this},setW:function(e,t){return this.data.array[e*this.data.stride+this.offset+3]=t,this},getX:function(e){return this.data.array[e*this.data.stride+this.offset]},getY:function(e){return this.data.array[e*this.data.stride+this.offset+1]},getZ:function(e){return this.data.array[e*this.data.stride+this.offset+2]},getW:function(e){return this.data.array[e*this.data.stride+this.offset+3]},setXY:function(e,t,n){return e=e*this.data.stride+this.offset,this.data.array[e+0]=t,this.data.array[e+1]=n,this},setXYZ:function(e,t,n,r){return e=e*this.data.stride+this.offset,this.data.array[e+0]=t,this.data.array[e+1]=n,this.data.array[e+2]=r,this},setXYZW:function(e,t,n,r,i){return e=e*this.data.stride+this.offset,this.data.array[e+0]=t,this.data.array[e+1]=n,this.data.array[e+2]=r,this.data.array[e+3]=i,this},clone:function(){console.log("THREE.InterleavedBufferAttribute.clone(): Cloning an interlaved buffer attribute will deinterleave buffer data.");for(var e=[],t=0;t<this.count;t++)for(var n=t*this.data.stride+this.offset,r=0;r<this.itemSize;r++)e.push(this.data.array[n+r]);return new wi(new this.array.constructor(e),this.itemSize,this.normalized)},toJSON:function(){console.log("THREE.InterleavedBufferAttribute.toJSON(): Serializing an interlaved buffer attribute will deinterleave buffer data.");for(var e=[],t=0;t<this.count;t++)for(var n=t*this.data.stride+this.offset,r=0;r<this.itemSize;r++)e.push(this.data.array[n+r]);return{itemSize:this.itemSize,type:this.array.constructor.name,array:e,normalized:this.normalized}}}),Us.prototype=Object.create(_i.prototype),Us.prototype.constructor=Us,Us.prototype.isSpriteMaterial=!0,Us.prototype.copy=function(e){return _i.prototype.copy.call(this,e),this.color.copy(e.color),this.map=e.map,this.alphaMap=e.alphaMap,this.rotation=e.rotation,this.sizeAttenuation=e.sizeAttenuation,this};var Vs=new $n,Ys=new $n,Ws=new $n,Gs=new Un,Xs=new Un,qs=new sr,Zs=new $n,Qs=new $n,Ks=new $n,Js=new Un,$s=new Un,el=new Un;function tl(e){if(wr.call(this),this.type="Sprite",void 0===Fs){Fs=new Yi;var t=new zs(new Float32Array([-.5,-.5,0,0,0,.5,-.5,0,1,0,.5,.5,0,1,1,-.5,.5,0,0,1]),5);Fs.setIndex([0,1,2,0,2,3]),Fs.setAttribute("position",new Hs(t,3,0,!1)),Fs.setAttribute("uv",new Hs(t,2,3,!1))}this.geometry=Fs,this.material=void 0!==e?e:new Us,this.center=new Un(.5,.5)}function nl(e,t,n,r,i,o){Gs.subVectors(e,n).addScalar(.5).multiply(r),void 0!==i?(Xs.x=o*Gs.x-i*Gs.y,Xs.y=i*Gs.x+o*Gs.y):Xs.copy(Gs),e.copy(t),e.x+=Xs.x,e.y+=Xs.y,e.applyMatrix4(qs)}tl.prototype=Object.assign(Object.create(wr.prototype),{constructor:tl,isSprite:!0,raycast:function(e,t){null===e.camera&&console.error('THREE.Sprite: "Raycaster.camera" needs to be set in order to raycast against sprites.'),Ys.setFromMatrixScale(this.matrixWorld),qs.copy(e.camera.matrixWorld),this.modelViewMatrix.multiplyMatrices(e.camera.matrixWorldInverse,this.matrixWorld),Ws.setFromMatrixPosition(this.modelViewMatrix),e.camera.isPerspectiveCamera&&!1===this.material.sizeAttenuation&&Ys.multiplyScalar(-Ws.z);var n,r,i=this.material.rotation;0!==i&&(r=Math.cos(i),n=Math.sin(i));var o=this.center;nl(Zs.set(-.5,-.5,0),Ws,o,Ys,n,r),nl(Qs.set(.5,-.5,0),Ws,o,Ys,n,r),nl(Ks.set(.5,.5,0),Ws,o,Ys,n,r),Js.set(0,0),$s.set(1,0),el.set(1,1);var a=e.ray.intersectTriangle(Zs,Qs,Ks,!1,Vs);if(null!==a||(nl(Qs.set(-.5,.5,0),Ws,o,Ys,n,r),$s.set(0,1),null!==(a=e.ray.intersectTriangle(Zs,Ks,Qs,!1,Vs)))){var s=e.ray.origin.distanceTo(Vs);s<e.near||s>e.far||t.push({distance:s,point:Vs.clone(),uv:hi.getUV(Vs,Zs,Qs,Ks,Js,$s,el,new Un),face:null,object:this})}},clone:function(){return new this.constructor(this.material).copy(this)},copy:function(e){return wr.prototype.copy.call(this,e),void 0!==e.center&&this.center.copy(e.center),this}});var rl,il,ol,al,sl,ll=new $n,ul=new $n;function cl(){wr.call(this),this._currentLevel=0,this.type="LOD",Object.defineProperties(this,{levels:{enumerable:!0,value:[]}}),this.autoUpdate=!0}function dl(e,t){e&&e.isGeometry&&console.error("THREE.SkinnedMesh no longer supports THREE.Geometry. Use THREE.BufferGeometry instead."),lo.call(this,e,t),this.type="SkinnedMesh",this.bindMode="attached",this.bindMatrix=new sr,this.bindMatrixInverse=new sr}cl.prototype=Object.assign(Object.create(wr.prototype),{constructor:cl,isLOD:!0,copy:function(e){wr.prototype.copy.call(this,e,!1);for(var t=e.levels,n=0,r=t.length;n<r;n++){var i=t[n];this.addLevel(i.object.clone(),i.distance)}return this.autoUpdate=e.autoUpdate,this},addLevel:function(e,t){void 0===t&&(t=0),t=Math.abs(t);for(var n=this.levels,r=0;r<n.length&&!(t<n[r].distance);r++);return n.splice(r,0,{distance:t,object:e}),this.add(e),this},getCurrentLevel:function(){return this._currentLevel},getObjectForDistance:function(e){var t=this.levels;if(t.length>0){for(var n=1,r=t.length;n<r&&!(e<t[n].distance);n++);return t[n-1].object}return null},raycast:function(e,t){if(this.levels.length>0){ll.setFromMatrixPosition(this.matrixWorld);var n=e.ray.origin.distanceTo(ll);this.getObjectForDistance(n).raycast(e,t)}},update:function(e){var t=this.levels;if(t.length>1){ll.setFromMatrixPosition(e.matrixWorld),ul.setFromMatrixPosition(this.matrixWorld);var n=ll.distanceTo(ul)/e.zoom;t[0].object.visible=!0;for(var r=1,i=t.length;r<i&&n>=t[r].distance;r++)t[r-1].object.visible=!1,t[r].object.visible=!0;for(this._currentLevel=r-1;r<i;r++)t[r].object.visible=!1}},toJSON:function(e){var t=wr.prototype.toJSON.call(this,e);!1===this.autoUpdate&&(t.object.autoUpdate=!1),t.object.levels=[];for(var n=this.levels,r=0,i=n.length;r<i;r++){var o=n[r];t.object.levels.push({object:o.object.uuid,distance:o.distance})}return t}}),dl.prototype=Object.assign(Object.create(lo.prototype),{constructor:dl,isSkinnedMesh:!0,bind:function(e,t){this.skeleton=e,void 0===t&&(this.updateMatrixWorld(!0),this.skeleton.calculateInverses(),t=this.matrixWorld),this.bindMatrix.copy(t),this.bindMatrixInverse.getInverse(t)},pose:function(){this.skeleton.pose()},normalizeSkinWeights:function(){for(var e=new Xn,t=this.geometry.attributes.skinWeight,n=0,r=t.count;n<r;n++){e.x=t.getX(n),e.y=t.getY(n),e.z=t.getZ(n),e.w=t.getW(n);var i=1/e.manhattanLength();i!==1/0?e.multiplyScalar(i):e.set(1,0,0,0),t.setXYZW(n,e.x,e.y,e.z,e.w)}},updateMatrixWorld:function(e){lo.prototype.updateMatrixWorld.call(this,e),"attached"===this.bindMode?this.bindMatrixInverse.getInverse(this.matrixWorld):"detached"===this.bindMode?this.bindMatrixInverse.getInverse(this.bindMatrix):console.warn("THREE.SkinnedMesh: Unrecognized bindMode: "+this.bindMode)},clone:function(){return new this.constructor(this.geometry,this.material).copy(this)},boneTransform:(rl=new $n,il=new Xn,ol=new Xn,al=new $n,sl=new sr,function(e,t){var n=this.skeleton,r=this.geometry;il.fromBufferAttribute(r.attributes.skinIndex,e),ol.fromBufferAttribute(r.attributes.skinWeight,e),rl.fromBufferAttribute(r.attributes.position,e).applyMatrix4(this.bindMatrix),t.set(0,0,0);for(var i=0;i<4;i++){var o=ol.getComponent(i);if(0!==o){var a=il.getComponent(i);sl.multiplyMatrices(n.bones[a].matrixWorld,n.boneInverses[a]),t.addScaledVector(al.copy(rl).applyMatrix4(sl),o)}}return t.applyMatrix4(this.bindMatrixInverse)})});var hl=new sr,fl=new sr;function pl(e,t){if(e=e||[],this.bones=e.slice(0),this.boneMatrices=new Float32Array(16*this.bones.length),this.frame=-1,void 0===t)this.calculateInverses();else if(this.bones.length===t.length)this.boneInverses=t.slice(0);else{console.warn("THREE.Skeleton boneInverses is the wrong length."),this.boneInverses=[];for(var n=0,r=this.bones.length;n<r;n++)this.boneInverses.push(new sr)}}function gl(){wr.call(this),this.type="Bone"}Object.assign(pl.prototype,{calculateInverses:function(){this.boneInverses=[];for(var e=0,t=this.bones.length;e<t;e++){var n=new sr;this.bones[e]&&n.getInverse(this.bones[e].matrixWorld),this.boneInverses.push(n)}},pose:function(){var e,t,n;for(t=0,n=this.bones.length;t<n;t++)(e=this.bones[t])&&e.matrixWorld.getInverse(this.boneInverses[t]);for(t=0,n=this.bones.length;t<n;t++)(e=this.bones[t])&&(e.parent&&e.parent.isBone?(e.matrix.getInverse(e.parent.matrixWorld),e.matrix.multiply(e.matrixWorld)):e.matrix.copy(e.matrixWorld),e.matrix.decompose(e.position,e.quaternion,e.scale))},update:function(){for(var e=this.bones,t=this.boneInverses,n=this.boneMatrices,r=this.boneTexture,i=0,o=e.length;i<o;i++){var a=e[i]?e[i].matrixWorld:fl;hl.multiplyMatrices(a,t[i]),hl.toArray(n,16*i)}void 0!==r&&(r.needsUpdate=!0)},clone:function(){return new pl(this.bones,this.boneInverses)},getBoneByName:function(e){for(var t=0,n=this.bones.length;t<n;t++){var r=this.bones[t];if(r.name===e)return r}},dispose:function(){this.boneTexture&&(this.boneTexture.dispose(),this.boneTexture=void 0)}}),gl.prototype=Object.assign(Object.create(wr.prototype),{constructor:gl,isBone:!0});var vl=new sr,ml=new sr,yl=[],bl=new lo;function Cl(e,t,n){lo.call(this,e,t),this.instanceMatrix=new wi(new Float32Array(16*n),16),this.count=n,this.frustumCulled=!1}function Al(e){_i.call(this),this.type="LineBasicMaterial",this.color=new vi(16777215),this.linewidth=1,this.linecap="round",this.linejoin="round",this.morphTargets=!1,this.setValues(e)}Cl.prototype=Object.assign(Object.create(lo.prototype),{constructor:Cl,isInstancedMesh:!0,getMatrixAt:function(e,t){t.fromArray(this.instanceMatrix.array,16*e)},raycast:function(e,t){var n=this.matrixWorld,r=this.count;if(bl.geometry=this.geometry,bl.material=this.material,void 0!==bl.material)for(var i=0;i<r;i++){this.getMatrixAt(i,vl),ml.multiplyMatrices(n,vl),bl.matrixWorld=ml,bl.raycast(e,yl);for(var o=0,a=yl.length;o<a;o++){var s=yl[o];s.instanceId=i,s.object=this,t.push(s)}yl.length=0}},setMatrixAt:function(e,t){t.toArray(this.instanceMatrix.array,16*e)},updateMorphTargets:function(){}}),Al.prototype=Object.create(_i.prototype),Al.prototype.constructor=Al,Al.prototype.isLineBasicMaterial=!0,Al.prototype.copy=function(e){return _i.prototype.copy.call(this,e),this.color.copy(e.color),this.linewidth=e.linewidth,this.linecap=e.linecap,this.linejoin=e.linejoin,this.morphTargets=e.morphTargets,this};var _l=new $n,Il=new $n,xl=new sr,wl=new Kr,Ml=new Vr;function Sl(e,t,n){1===n&&console.error("THREE.Line: parameter THREE.LinePieces no longer supported. Use THREE.LineSegments instead."),wr.call(this),this.type="Line",this.geometry=void 0!==e?e:new Yi,this.material=void 0!==t?t:new Al,this.updateMorphTargets()}Sl.prototype=Object.assign(Object.create(wr.prototype),{constructor:Sl,isLine:!0,computeLineDistances:function(){var e=this.geometry;if(e.isBufferGeometry)if(null===e.index){for(var t=e.attributes.position,n=[0],r=1,i=t.count;r<i;r++)_l.fromBufferAttribute(t,r-1),Il.fromBufferAttribute(t,r),n[r]=n[r-1],n[r]+=_l.distanceTo(Il);e.setAttribute("lineDistance",new Di(n,1))}else console.warn("THREE.Line.computeLineDistances(): Computation only possible with non-indexed BufferGeometry.");else if(e.isGeometry){var o=e.vertices;(n=e.lineDistances)[0]=0;for(r=1,i=o.length;r<i;r++)n[r]=n[r-1],n[r]+=o[r-1].distanceTo(o[r])}return this},raycast:function(e,t){var n=this.geometry,r=this.matrixWorld,i=e.params.Line.threshold;if(null===n.boundingSphere&&n.computeBoundingSphere(),Ml.copy(n.boundingSphere),Ml.applyMatrix4(r),Ml.radius+=i,!1!==e.ray.intersectsSphere(Ml)){xl.getInverse(r),wl.copy(e.ray).applyMatrix4(xl);var o=i/((this.scale.x+this.scale.y+this.scale.z)/3),a=o*o,s=new $n,l=new $n,u=new $n,c=new $n,d=this&&this.isLineSegments?2:1;if(n.isBufferGeometry){var h=n.index,f=n.attributes.position.array;if(null!==h)for(var p=h.array,g=0,v=p.length-1;g<v;g+=d){var m=p[g],y=p[g+1];if(s.fromArray(f,3*m),l.fromArray(f,3*y),!(wl.distanceSqToSegment(s,l,c,u)>a))c.applyMatrix4(this.matrixWorld),(A=e.ray.origin.distanceTo(c))<e.near||A>e.far||t.push({distance:A,point:u.clone().applyMatrix4(this.matrixWorld),index:g,face:null,faceIndex:null,object:this})}else for(g=0,v=f.length/3-1;g<v;g+=d){if(s.fromArray(f,3*g),l.fromArray(f,3*g+3),!(wl.distanceSqToSegment(s,l,c,u)>a))c.applyMatrix4(this.matrixWorld),(A=e.ray.origin.distanceTo(c))<e.near||A>e.far||t.push({distance:A,point:u.clone().applyMatrix4(this.matrixWorld),index:g,face:null,faceIndex:null,object:this})}}else if(n.isGeometry){var b=n.vertices,C=b.length;for(g=0;g<C-1;g+=d){var A;if(!(wl.distanceSqToSegment(b[g],b[g+1],c,u)>a))c.applyMatrix4(this.matrixWorld),(A=e.ray.origin.distanceTo(c))<e.near||A>e.far||t.push({distance:A,point:u.clone().applyMatrix4(this.matrixWorld),index:g,face:null,faceIndex:null,object:this})}}}},updateMorphTargets:function(){var e,t,n,r=this.geometry;if(r.isBufferGeometry){var i=r.morphAttributes,o=Object.keys(i);if(o.length>0){var a=i[o[0]];if(void 0!==a)for(this.morphTargetInfluences=[],this.morphTargetDictionary={},e=0,t=a.length;e<t;e++)n=a[e].name||String(e),this.morphTargetInfluences.push(0),this.morphTargetDictionary[n]=e}}else{var s=r.morphTargets;void 0!==s&&s.length>0&&console.error("THREE.Line.updateMorphTargets() does not support THREE.Geometry. Use THREE.BufferGeometry instead.")}},clone:function(){return new this.constructor(this.geometry,this.material).copy(this)}});var Tl=new $n,El=new $n;function Ol(e,t){Sl.call(this,e,t),this.type="LineSegments"}function kl(e,t){Sl.call(this,e,t),this.type="LineLoop"}function Ll(e){_i.call(this),this.type="PointsMaterial",this.color=new vi(16777215),this.map=null,this.alphaMap=null,this.size=1,this.sizeAttenuation=!0,this.morphTargets=!1,this.setValues(e)}Ol.prototype=Object.assign(Object.create(Sl.prototype),{constructor:Ol,isLineSegments:!0,computeLineDistances:function(){var e=this.geometry;if(e.isBufferGeometry)if(null===e.index){for(var t=e.attributes.position,n=[],r=0,i=t.count;r<i;r+=2)Tl.fromBufferAttribute(t,r),El.fromBufferAttribute(t,r+1),n[r]=0===r?0:n[r-1],n[r+1]=n[r]+Tl.distanceTo(El);e.setAttribute("lineDistance",new Di(n,1))}else console.warn("THREE.LineSegments.computeLineDistances(): Computation only possible with non-indexed BufferGeometry.");else if(e.isGeometry){var o=e.vertices;for(n=e.lineDistances,r=0,i=o.length;r<i;r+=2)Tl.copy(o[r]),El.copy(o[r+1]),n[r]=0===r?0:n[r-1],n[r+1]=n[r]+Tl.distanceTo(El)}return this}}),kl.prototype=Object.assign(Object.create(Sl.prototype),{constructor:kl,isLineLoop:!0}),Ll.prototype=Object.create(_i.prototype),Ll.prototype.constructor=Ll,Ll.prototype.isPointsMaterial=!0,Ll.prototype.copy=function(e){return _i.prototype.copy.call(this,e),this.color.copy(e.color),this.map=e.map,this.alphaMap=e.alphaMap,this.size=e.size,this.sizeAttenuation=e.sizeAttenuation,this.morphTargets=e.morphTargets,this};var Dl=new sr,Pl=new Kr,Nl=new Vr,jl=new $n;function Rl(e,t){wr.call(this),this.type="Points",this.geometry=void 0!==e?e:new Yi,this.material=void 0!==t?t:new Ll,this.updateMorphTargets()}function zl(e,t,n,r,i,o,a){var s=Pl.distanceSqToPoint(e);if(s<n){var l=new $n;Pl.closestPointToPoint(e,l),l.applyMatrix4(r);var u=i.ray.origin.distanceTo(l);if(u<i.near||u>i.far)return;o.push({distance:u,distanceToRay:Math.sqrt(s),point:l,index:t,face:null,object:a})}}function Fl(e,t,n,r,i,o,a,s,l){Gn.call(this,e,t,n,r,i,o,a,s,l),this.format=void 0!==a?a:Be,this.minFilter=void 0!==o?o:_e,this.magFilter=void 0!==i?i:_e,this.generateMipmaps=!1}function Bl(e,t,n,r,i,o,a,s,l,u,c,d){Gn.call(this,null,o,a,s,l,u,r,i,c,d),this.image={width:t,height:n},this.mipmaps=e,this.flipY=!1,this.generateMipmaps=!1}function Hl(e,t,n,r,i,o,a,s,l){Gn.call(this,e,t,n,r,i,o,a,s,l),this.needsUpdate=!0}function Ul(e,t,n,r,i,o,a,s,l,u){if((u=void 0!==u?u:We)!==We&&u!==Ge)throw new Error("DepthTexture format must be either THREE.DepthFormat or THREE.DepthStencilFormat");void 0===n&&u===We&&(n=Oe),void 0===n&&u===Ge&&(n=ze),Gn.call(this,null,r,i,o,a,s,u,n,l),this.image={width:e,height:t},this.magFilter=void 0!==a?a:me,this.minFilter=void 0!==s?s:me,this.flipY=!1,this.generateMipmaps=!1}function Vl(e){Yi.call(this),this.type="WireframeGeometry";var t,n,r,i,o,a,s,l,u,c,d=[],h=[0,0],f={},p=["a","b","c"];if(e&&e.isGeometry){var g=e.faces;for(t=0,r=g.length;t<r;t++){var v=g[t];for(n=0;n<3;n++)s=v[p[n]],l=v[p[(n+1)%3]],h[0]=Math.min(s,l),h[1]=Math.max(s,l),void 0===f[u=h[0]+","+h[1]]&&(f[u]={index1:h[0],index2:h[1]})}for(u in f)a=f[u],c=e.vertices[a.index1],d.push(c.x,c.y,c.z),c=e.vertices[a.index2],d.push(c.x,c.y,c.z)}else if(e&&e.isBufferGeometry){var m,y,b,C,A,_,I;if(c=new $n,null!==e.index){for(m=e.attributes.position,y=e.index,0===(b=e.groups).length&&(b=[{start:0,count:y.count,materialIndex:0}]),i=0,o=b.length;i<o;++i)for(t=A=(C=b[i]).start,r=A+C.count;t<r;t+=3)for(n=0;n<3;n++)s=y.getX(t+n),l=y.getX(t+(n+1)%3),h[0]=Math.min(s,l),h[1]=Math.max(s,l),void 0===f[u=h[0]+","+h[1]]&&(f[u]={index1:h[0],index2:h[1]});for(u in f)a=f[u],c.fromBufferAttribute(m,a.index1),d.push(c.x,c.y,c.z),c.fromBufferAttribute(m,a.index2),d.push(c.x,c.y,c.z)}else for(t=0,r=(m=e.attributes.position).count/3;t<r;t++)for(n=0;n<3;n++)_=3*t+n,c.fromBufferAttribute(m,_),d.push(c.x,c.y,c.z),I=3*t+(n+1)%3,c.fromBufferAttribute(m,I),d.push(c.x,c.y,c.z)}this.setAttribute("position",new Di(d,3))}function Yl(e,t,n){vo.call(this),this.type="ParametricGeometry",this.parameters={func:e,slices:t,stacks:n},this.fromBufferGeometry(new Wl(e,t,n)),this.mergeVertices()}function Wl(e,t,n){Yi.call(this),this.type="ParametricBufferGeometry",this.parameters={func:e,slices:t,stacks:n};var r,i,o=[],a=[],s=[],l=[],u=1e-5,c=new $n,d=new $n,h=new $n,f=new $n,p=new $n;e.length<3&&console.error("THREE.ParametricGeometry: Function must now modify a Vector3 as third parameter.");var g=t+1;for(r=0;r<=n;r++){var v=r/n;for(i=0;i<=t;i++){var m=i/t;e(m,v,d),a.push(d.x,d.y,d.z),m-u>=0?(e(m-u,v,h),f.subVectors(d,h)):(e(m+u,v,h),f.subVectors(h,d)),v-u>=0?(e(m,v-u,h),p.subVectors(d,h)):(e(m,v+u,h),p.subVectors(h,d)),c.crossVectors(f,p).normalize(),s.push(c.x,c.y,c.z),l.push(m,v)}}for(r=0;r<n;r++)for(i=0;i<t;i++){var y=r*g+i,b=r*g+i+1,C=(r+1)*g+i+1,A=(r+1)*g+i;o.push(y,b,A),o.push(b,C,A)}this.setIndex(o),this.setAttribute("position",new Di(a,3)),this.setAttribute("normal",new Di(s,3)),this.setAttribute("uv",new Di(l,2))}function Gl(e,t,n,r){vo.call(this),this.type="PolyhedronGeometry",this.parameters={vertices:e,indices:t,radius:n,detail:r},this.fromBufferGeometry(new Xl(e,t,n,r)),this.mergeVertices()}function Xl(e,t,n,r){Yi.call(this),this.type="PolyhedronBufferGeometry",this.parameters={vertices:e,indices:t,radius:n,detail:r},n=n||1;var i=[],o=[];function a(e,t,n,r){var i,o,a=Math.pow(2,r),l=[];for(i=0;i<=a;i++){l[i]=[];var u=e.clone().lerp(n,i/a),c=t.clone().lerp(n,i/a),d=a-i;for(o=0;o<=d;o++)l[i][o]=0===o&&i===a?u:u.clone().lerp(c,o/d)}for(i=0;i<a;i++)for(o=0;o<2*(a-i)-1;o++){var h=Math.floor(o/2);o%2==0?(s(l[i][h+1]),s(l[i+1][h]),s(l[i][h])):(s(l[i][h+1]),s(l[i+1][h+1]),s(l[i+1][h]))}}function s(e){i.push(e.x,e.y,e.z)}function l(t,n){var r=3*t;n.x=e[r+0],n.y=e[r+1],n.z=e[r+2]}function u(e,t,n,r){r<0&&1===e.x&&(o[t]=e.x-1),0===n.x&&0===n.z&&(o[t]=r/2/Math.PI+.5)}function c(e){return Math.atan2(e.z,-e.x)}function d(e){return Math.atan2(-e.y,Math.sqrt(e.x*e.x+e.z*e.z))}!function(e){for(var n=new $n,r=new $n,i=new $n,o=0;o<t.length;o+=3)l(t[o+0],n),l(t[o+1],r),l(t[o+2],i),a(n,r,i,e)}(r=r||0),function(e){for(var t=new $n,n=0;n<i.length;n+=3)t.x=i[n+0],t.y=i[n+1],t.z=i[n+2],t.normalize().multiplyScalar(e),i[n+0]=t.x,i[n+1]=t.y,i[n+2]=t.z}(n),function(){for(var e=new $n,t=0;t<i.length;t+=3){e.x=i[t+0],e.y=i[t+1],e.z=i[t+2];var n=c(e)/2/Math.PI+.5,r=d(e)/Math.PI+.5;o.push(n,1-r)}(function(){for(var e=new $n,t=new $n,n=new $n,r=new $n,a=new Un,s=new Un,l=new Un,d=0,h=0;d<i.length;d+=9,h+=6){e.set(i[d+0],i[d+1],i[d+2]),t.set(i[d+3],i[d+4],i[d+5]),n.set(i[d+6],i[d+7],i[d+8]),a.set(o[h+0],o[h+1]),s.set(o[h+2],o[h+3]),l.set(o[h+4],o[h+5]),r.copy(e).add(t).add(n).divideScalar(3);var f=c(r);u(a,h+0,e,f),u(s,h+2,t,f),u(l,h+4,n,f)}})(),function(){for(var e=0;e<o.length;e+=6){var t=o[e+0],n=o[e+2],r=o[e+4],i=Math.max(t,n,r),a=Math.min(t,n,r);i>.9&&a<.1&&(t<.2&&(o[e+0]+=1),n<.2&&(o[e+2]+=1),r<.2&&(o[e+4]+=1))}}()}(),this.setAttribute("position",new Di(i,3)),this.setAttribute("normal",new Di(i.slice(),3)),this.setAttribute("uv",new Di(o,2)),0===r?this.computeVertexNormals():this.normalizeNormals()}function ql(e,t){vo.call(this),this.type="TetrahedronGeometry",this.parameters={radius:e,detail:t},this.fromBufferGeometry(new Zl(e,t)),this.mergeVertices()}function Zl(e,t){Xl.call(this,[1,1,1,-1,-1,1,-1,1,-1,1,-1,-1],[2,1,0,0,3,2,1,3,0,2,3,1],e,t),this.type="TetrahedronBufferGeometry",this.parameters={radius:e,detail:t}}function Ql(e,t){vo.call(this),this.type="OctahedronGeometry",this.parameters={radius:e,detail:t},this.fromBufferGeometry(new Kl(e,t)),this.mergeVertices()}function Kl(e,t){Xl.call(this,[1,0,0,-1,0,0,0,1,0,0,-1,0,0,0,1,0,0,-1],[0,2,4,0,4,3,0,3,5,0,5,2,1,2,5,1,5,3,1,3,4,1,4,2],e,t),this.type="OctahedronBufferGeometry",this.parameters={radius:e,detail:t}}function Jl(e,t){vo.call(this),this.type="IcosahedronGeometry",this.parameters={radius:e,detail:t},this.fromBufferGeometry(new $l(e,t)),this.mergeVertices()}function $l(e,t){var n=(1+Math.sqrt(5))/2,r=[-1,n,0,1,n,0,-1,-n,0,1,-n,0,0,-1,n,0,1,n,0,-1,-n,0,1,-n,n,0,-1,n,0,1,-n,0,-1,-n,0,1];Xl.call(this,r,[0,11,5,0,5,1,0,1,7,0,7,10,0,10,11,1,5,9,5,11,4,11,10,2,10,7,6,7,1,8,3,9,4,3,4,2,3,2,6,3,6,8,3,8,9,4,9,5,2,4,11,6,2,10,8,6,7,9,8,1],e,t),this.type="IcosahedronBufferGeometry",this.parameters={radius:e,detail:t}}function eu(e,t){vo.call(this),this.type="DodecahedronGeometry",this.parameters={radius:e,detail:t},this.fromBufferGeometry(new tu(e,t)),this.mergeVertices()}function tu(e,t){var n=(1+Math.sqrt(5))/2,r=1/n,i=[-1,-1,-1,-1,-1,1,-1,1,-1,-1,1,1,1,-1,-1,1,-1,1,1,1,-1,1,1,1,0,-r,-n,0,-r,n,0,r,-n,0,r,n,-r,-n,0,-r,n,0,r,-n,0,r,n,0,-n,0,-r,n,0,-r,-n,0,r,n,0,r];Xl.call(this,i,[3,11,7,3,7,15,3,15,13,7,19,17,7,17,6,7,6,15,17,4,8,17,8,10,17,10,6,8,0,16,8,16,2,8,2,10,0,12,1,0,1,18,0,18,16,6,10,2,6,2,13,6,13,15,2,16,18,2,18,3,2,3,13,18,1,9,18,9,11,18,11,3,4,14,12,4,12,0,4,0,8,11,9,5,11,5,19,11,19,7,19,5,14,19,14,4,19,4,17,1,12,14,1,14,5,1,5,9],e,t),this.type="DodecahedronBufferGeometry",this.parameters={radius:e,detail:t}}function nu(e,t,n,r,i,o){vo.call(this),this.type="TubeGeometry",this.parameters={path:e,tubularSegments:t,radius:n,radialSegments:r,closed:i},void 0!==o&&console.warn("THREE.TubeGeometry: taper has been removed.");var a=new ru(e,t,n,r,i);this.tangents=a.tangents,this.normals=a.normals,this.binormals=a.binormals,this.fromBufferGeometry(a),this.mergeVertices()}function ru(e,t,n,r,i){Yi.call(this),this.type="TubeBufferGeometry",this.parameters={path:e,tubularSegments:t,radius:n,radialSegments:r,closed:i},t=t||64,n=n||1,r=r||8,i=i||!1;var o=e.computeFrenetFrames(t,i);this.tangents=o.tangents,this.normals=o.normals,this.binormals=o.binormals;var a,s,l=new $n,u=new $n,c=new Un,d=new $n,h=[],f=[],p=[],g=[];function v(i){d=e.getPointAt(i/t,d);var a=o.normals[i],c=o.binormals[i];for(s=0;s<=r;s++){var p=s/r*Math.PI*2,g=Math.sin(p),v=-Math.cos(p);u.x=v*a.x+g*c.x,u.y=v*a.y+g*c.y,u.z=v*a.z+g*c.z,u.normalize(),f.push(u.x,u.y,u.z),l.x=d.x+n*u.x,l.y=d.y+n*u.y,l.z=d.z+n*u.z,h.push(l.x,l.y,l.z)}}!function(){for(a=0;a<t;a++)v(a);v(!1===i?t:0),function(){for(a=0;a<=t;a++)for(s=0;s<=r;s++)c.x=a/t,c.y=s/r,p.push(c.x,c.y)}(),function(){for(s=1;s<=t;s++)for(a=1;a<=r;a++){var e=(r+1)*(s-1)+(a-1),n=(r+1)*s+(a-1),i=(r+1)*s+a,o=(r+1)*(s-1)+a;g.push(e,n,o),g.push(n,i,o)}}()}(),this.setIndex(g),this.setAttribute("position",new Di(h,3)),this.setAttribute("normal",new Di(f,3)),this.setAttribute("uv",new Di(p,2))}function iu(e,t,n,r,i,o,a){vo.call(this),this.type="TorusKnotGeometry",this.parameters={radius:e,tube:t,tubularSegments:n,radialSegments:r,p:i,q:o},void 0!==a&&console.warn("THREE.TorusKnotGeometry: heightScale has been deprecated. Use .scale( x, y, z ) instead."),this.fromBufferGeometry(new ou(e,t,n,r,i,o)),this.mergeVertices()}function ou(e,t,n,r,i,o){Yi.call(this),this.type="TorusKnotBufferGeometry",this.parameters={radius:e,tube:t,tubularSegments:n,radialSegments:r,p:i,q:o},e=e||1,t=t||.4,n=Math.floor(n)||64,r=Math.floor(r)||8,i=i||2,o=o||3;var a,s,l=[],u=[],c=[],d=[],h=new $n,f=new $n,p=new $n,g=new $n,v=new $n,m=new $n,y=new $n;for(a=0;a<=n;++a){var b=a/n*i*Math.PI*2;for(S(b,i,o,e,p),S(b+.01,i,o,e,g),m.subVectors(g,p),y.addVectors(g,p),v.crossVectors(m,y),y.crossVectors(v,m),v.normalize(),y.normalize(),s=0;s<=r;++s){var C=s/r*Math.PI*2,A=-t*Math.cos(C),_=t*Math.sin(C);h.x=p.x+(A*y.x+_*v.x),h.y=p.y+(A*y.y+_*v.y),h.z=p.z+(A*y.z+_*v.z),u.push(h.x,h.y,h.z),f.subVectors(h,p).normalize(),c.push(f.x,f.y,f.z),d.push(a/n),d.push(s/r)}}for(s=1;s<=n;s++)for(a=1;a<=r;a++){var I=(r+1)*(s-1)+(a-1),x=(r+1)*s+(a-1),w=(r+1)*s+a,M=(r+1)*(s-1)+a;l.push(I,x,M),l.push(x,w,M)}function S(e,t,n,r,i){var o=Math.cos(e),a=Math.sin(e),s=n/t*e,l=Math.cos(s);i.x=r*(2+l)*.5*o,i.y=r*(2+l)*a*.5,i.z=r*Math.sin(s)*.5}this.setIndex(l),this.setAttribute("position",new Di(u,3)),this.setAttribute("normal",new Di(c,3)),this.setAttribute("uv",new Di(d,2))}function au(e,t,n,r,i){vo.call(this),this.type="TorusGeometry",this.parameters={radius:e,tube:t,radialSegments:n,tubularSegments:r,arc:i},this.fromBufferGeometry(new su(e,t,n,r,i)),this.mergeVertices()}function su(e,t,n,r,i){Yi.call(this),this.type="TorusBufferGeometry",this.parameters={radius:e,tube:t,radialSegments:n,tubularSegments:r,arc:i},e=e||1,t=t||.4,n=Math.floor(n)||8,r=Math.floor(r)||6,i=i||2*Math.PI;var o,a,s=[],l=[],u=[],c=[],d=new $n,h=new $n,f=new $n;for(o=0;o<=n;o++)for(a=0;a<=r;a++){var p=a/r*i,g=o/n*Math.PI*2;h.x=(e+t*Math.cos(g))*Math.cos(p),h.y=(e+t*Math.cos(g))*Math.sin(p),h.z=t*Math.sin(g),l.push(h.x,h.y,h.z),d.x=e*Math.cos(p),d.y=e*Math.sin(p),f.subVectors(h,d).normalize(),u.push(f.x,f.y,f.z),c.push(a/r),c.push(o/n)}for(o=1;o<=n;o++)for(a=1;a<=r;a++){var v=(r+1)*o+a-1,m=(r+1)*(o-1)+a-1,y=(r+1)*(o-1)+a,b=(r+1)*o+a;s.push(v,m,b),s.push(m,y,b)}this.setIndex(s),this.setAttribute("position",new Di(l,3)),this.setAttribute("normal",new Di(u,3)),this.setAttribute("uv",new Di(c,2))}Rl.prototype=Object.assign(Object.create(wr.prototype),{constructor:Rl,isPoints:!0,raycast:function(e,t){var n=this.geometry,r=this.matrixWorld,i=e.params.Points.threshold;if(null===n.boundingSphere&&n.computeBoundingSphere(),Nl.copy(n.boundingSphere),Nl.applyMatrix4(r),Nl.radius+=i,!1!==e.ray.intersectsSphere(Nl)){Dl.getInverse(r),Pl.copy(e.ray).applyMatrix4(Dl);var o=i/((this.scale.x+this.scale.y+this.scale.z)/3),a=o*o;if(n.isBufferGeometry){var s=n.index,l=n.attributes.position.array;if(null!==s)for(var u=s.array,c=0,d=u.length;c<d;c++){var h=u[c];jl.fromArray(l,3*h),zl(jl,h,a,r,e,t,this)}else{c=0;for(var f=l.length/3;c<f;c++)jl.fromArray(l,3*c),zl(jl,c,a,r,e,t,this)}}else{var p=n.vertices;for(c=0,f=p.length;c<f;c++)zl(p[c],c,a,r,e,t,this)}}},updateMorphTargets:function(){var e,t,n,r=this.geometry;if(r.isBufferGeometry){var i=r.morphAttributes,o=Object.keys(i);if(o.length>0){var a=i[o[0]];if(void 0!==a)for(this.morphTargetInfluences=[],this.morphTargetDictionary={},e=0,t=a.length;e<t;e++)n=a[e].name||String(e),this.morphTargetInfluences.push(0),this.morphTargetDictionary[n]=e}}else{var s=r.morphTargets;void 0!==s&&s.length>0&&console.error("THREE.Points.updateMorphTargets() does not support THREE.Geometry. Use THREE.BufferGeometry instead.")}},clone:function(){return new this.constructor(this.geometry,this.material).copy(this)}}),Fl.prototype=Object.assign(Object.create(Gn.prototype),{constructor:Fl,isVideoTexture:!0,update:function(){var e=this.image;e.readyState>=e.HAVE_CURRENT_DATA&&(this.needsUpdate=!0)}}),Bl.prototype=Object.create(Gn.prototype),Bl.prototype.constructor=Bl,Bl.prototype.isCompressedTexture=!0,Hl.prototype=Object.create(Gn.prototype),Hl.prototype.constructor=Hl,Hl.prototype.isCanvasTexture=!0,Ul.prototype=Object.create(Gn.prototype),Ul.prototype.constructor=Ul,Ul.prototype.isDepthTexture=!0,Vl.prototype=Object.create(Yi.prototype),Vl.prototype.constructor=Vl,Yl.prototype=Object.create(vo.prototype),Yl.prototype.constructor=Yl,Wl.prototype=Object.create(Yi.prototype),Wl.prototype.constructor=Wl,Gl.prototype=Object.create(vo.prototype),Gl.prototype.constructor=Gl,Xl.prototype=Object.create(Yi.prototype),Xl.prototype.constructor=Xl,ql.prototype=Object.create(vo.prototype),ql.prototype.constructor=ql,Zl.prototype=Object.create(Xl.prototype),Zl.prototype.constructor=Zl,Ql.prototype=Object.create(vo.prototype),Ql.prototype.constructor=Ql,Kl.prototype=Object.create(Xl.prototype),Kl.prototype.constructor=Kl,Jl.prototype=Object.create(vo.prototype),Jl.prototype.constructor=Jl,$l.prototype=Object.create(Xl.prototype),$l.prototype.constructor=$l,eu.prototype=Object.create(vo.prototype),eu.prototype.constructor=eu,tu.prototype=Object.create(Xl.prototype),tu.prototype.constructor=tu,nu.prototype=Object.create(vo.prototype),nu.prototype.constructor=nu,ru.prototype=Object.create(Yi.prototype),ru.prototype.constructor=ru,ru.prototype.toJSON=function(){var e=Yi.prototype.toJSON.call(this);return e.path=this.parameters.path.toJSON(),e},iu.prototype=Object.create(vo.prototype),iu.prototype.constructor=iu,ou.prototype=Object.create(Yi.prototype),ou.prototype.constructor=ou,au.prototype=Object.create(vo.prototype),au.prototype.constructor=au,su.prototype=Object.create(Yi.prototype),su.prototype.constructor=su;var lu=function(e,t,n){n=n||2;var r,i,o,a,s,l,u,c=t&&t.length,d=c?t[0]*n:e.length,h=uu(e,0,d,n,!0),f=[];if(!h||h.next===h.prev)return f;if(c&&(h=function(e,t,n,r){var i,o,a,s,l,u=[];for(i=0,o=t.length;i<o;i++)a=t[i]*r,s=i<o-1?t[i+1]*r:e.length,(l=uu(e,a,s,r,!1))===l.next&&(l.steiner=!0),u.push(Cu(l));for(u.sort(vu),i=0;i<u.length;i++)mu(u[i],n),n=cu(n,n.next);return n}(e,t,h,n)),e.length>80*n){r=o=e[0],i=a=e[1];for(var p=n;p<d;p+=n)(s=e[p])<r&&(r=s),(l=e[p+1])<i&&(i=l),s>o&&(o=s),l>a&&(a=l);u=0!==(u=Math.max(o-r,a-i))?1/u:0}return du(h,f,n,r,i,u),f};function uu(e,t,n,r,i){var o,a;if(i===function(e,t,n,r){for(var i=0,o=t,a=n-r;o<n;o+=r)i+=(e[a]-e[o])*(e[o+1]+e[a+1]),a=o;return i}(e,t,n,r)>0)for(o=t;o<n;o+=r)a=Ou(o,e[o],e[o+1],a);else for(o=n-r;o>=t;o-=r)a=Ou(o,e[o],e[o+1],a);return a&&xu(a,a.next)&&(ku(a),a=a.next),a}function cu(e,t){if(!e)return e;t||(t=e);var n,r=e;do{if(n=!1,r.steiner||!xu(r,r.next)&&0!==Iu(r.prev,r,r.next))r=r.next;else{if(ku(r),(r=t=r.prev)===r.next)break;n=!0}}while(n||r!==t);return t}function du(e,t,n,r,i,o,a){if(e){!a&&o&&function(e,t,n,r){var i=e;do{null===i.z&&(i.z=bu(i.x,i.y,t,n,r)),i.prevZ=i.prev,i.nextZ=i.next,i=i.next}while(i!==e);i.prevZ.nextZ=null,i.prevZ=null,function(e){var t,n,r,i,o,a,s,l,u=1;do{for(n=e,e=null,o=null,a=0;n;){for(a++,r=n,s=0,t=0;t<u&&(s++,r=r.nextZ);t++);for(l=u;s>0||l>0&&r;)0!==s&&(0===l||!r||n.z<=r.z)?(i=n,n=n.nextZ,s--):(i=r,r=r.nextZ,l--),o?o.nextZ=i:e=i,i.prevZ=o,o=i;n=r}o.nextZ=null,u*=2}while(a>1)}(i)}(e,r,i,o);for(var s,l,u=e;e.prev!==e.next;)if(s=e.prev,l=e.next,o?fu(e,r,i,o):hu(e))t.push(s.i/n),t.push(e.i/n),t.push(l.i/n),ku(e),e=l.next,u=l.next;else if((e=l)===u){a?1===a?du(e=pu(cu(e),t,n),t,n,r,i,o,2):2===a&&gu(e,t,n,r,i,o):du(cu(e),t,n,r,i,o,1);break}}}function hu(e){var t=e.prev,n=e,r=e.next;if(Iu(t,n,r)>=0)return!1;for(var i=e.next.next;i!==e.prev;){if(Au(t.x,t.y,n.x,n.y,r.x,r.y,i.x,i.y)&&Iu(i.prev,i,i.next)>=0)return!1;i=i.next}return!0}function fu(e,t,n,r){var i=e.prev,o=e,a=e.next;if(Iu(i,o,a)>=0)return!1;for(var s=i.x<o.x?i.x<a.x?i.x:a.x:o.x<a.x?o.x:a.x,l=i.y<o.y?i.y<a.y?i.y:a.y:o.y<a.y?o.y:a.y,u=i.x>o.x?i.x>a.x?i.x:a.x:o.x>a.x?o.x:a.x,c=i.y>o.y?i.y>a.y?i.y:a.y:o.y>a.y?o.y:a.y,d=bu(s,l,t,n,r),h=bu(u,c,t,n,r),f=e.prevZ,p=e.nextZ;f&&f.z>=d&&p&&p.z<=h;){if(f!==e.prev&&f!==e.next&&Au(i.x,i.y,o.x,o.y,a.x,a.y,f.x,f.y)&&Iu(f.prev,f,f.next)>=0)return!1;if(f=f.prevZ,p!==e.prev&&p!==e.next&&Au(i.x,i.y,o.x,o.y,a.x,a.y,p.x,p.y)&&Iu(p.prev,p,p.next)>=0)return!1;p=p.nextZ}for(;f&&f.z>=d;){if(f!==e.prev&&f!==e.next&&Au(i.x,i.y,o.x,o.y,a.x,a.y,f.x,f.y)&&Iu(f.prev,f,f.next)>=0)return!1;f=f.prevZ}for(;p&&p.z<=h;){if(p!==e.prev&&p!==e.next&&Au(i.x,i.y,o.x,o.y,a.x,a.y,p.x,p.y)&&Iu(p.prev,p,p.next)>=0)return!1;p=p.nextZ}return!0}function pu(e,t,n){var r=e;do{var i=r.prev,o=r.next.next;!xu(i,o)&&wu(i,r,r.next,o)&&Tu(i,o)&&Tu(o,i)&&(t.push(i.i/n),t.push(r.i/n),t.push(o.i/n),ku(r),ku(r.next),r=e=o),r=r.next}while(r!==e);return cu(r)}function gu(e,t,n,r,i,o){var a=e;do{for(var s=a.next.next;s!==a.prev;){if(a.i!==s.i&&_u(a,s)){var l=Eu(a,s);return a=cu(a,a.next),l=cu(l,l.next),du(a,t,n,r,i,o),void du(l,t,n,r,i,o)}s=s.next}a=a.next}while(a!==e)}function vu(e,t){return e.x-t.x}function mu(e,t){if(t=function(e,t){var n,r=t,i=e.x,o=e.y,a=-1/0;do{if(o<=r.y&&o>=r.next.y&&r.next.y!==r.y){var s=r.x+(o-r.y)*(r.next.x-r.x)/(r.next.y-r.y);if(s<=i&&s>a){if(a=s,s===i){if(o===r.y)return r;if(o===r.next.y)return r.next}n=r.x<r.next.x?r:r.next}}r=r.next}while(r!==t);if(!n)return null;if(i===a)return n;var l,u=n,c=n.x,d=n.y,h=1/0;r=n;do{i>=r.x&&r.x>=c&&i!==r.x&&Au(o<d?i:a,o,c,d,o<d?a:i,o,r.x,r.y)&&(l=Math.abs(o-r.y)/(i-r.x),Tu(r,e)&&(l<h||l===h&&(r.x>n.x||r.x===n.x&&yu(n,r)))&&(n=r,h=l)),r=r.next}while(r!==u);return n}(e,t)){var n=Eu(t,e);cu(t,t.next),cu(n,n.next)}}function yu(e,t){return Iu(e.prev,e,t.prev)<0&&Iu(t.next,e,e.next)<0}function bu(e,t,n,r,i){return(e=1431655765&((e=858993459&((e=252645135&((e=16711935&((e=32767*(e-n)*i)|e<<8))|e<<4))|e<<2))|e<<1))|(t=1431655765&((t=858993459&((t=252645135&((t=16711935&((t=32767*(t-r)*i)|t<<8))|t<<4))|t<<2))|t<<1))<<1}function Cu(e){var t=e,n=e;do{(t.x<n.x||t.x===n.x&&t.y<n.y)&&(n=t),t=t.next}while(t!==e);return n}function Au(e,t,n,r,i,o,a,s){return(i-a)*(t-s)-(e-a)*(o-s)>=0&&(e-a)*(r-s)-(n-a)*(t-s)>=0&&(n-a)*(o-s)-(i-a)*(r-s)>=0}function _u(e,t){return e.next.i!==t.i&&e.prev.i!==t.i&&!function(e,t){var n=e;do{if(n.i!==e.i&&n.next.i!==e.i&&n.i!==t.i&&n.next.i!==t.i&&wu(n,n.next,e,t))return!0;n=n.next}while(n!==e);return!1}(e,t)&&(Tu(e,t)&&Tu(t,e)&&function(e,t){var n=e,r=!1,i=(e.x+t.x)/2,o=(e.y+t.y)/2;do{n.y>o!=n.next.y>o&&n.next.y!==n.y&&i<(n.next.x-n.x)*(o-n.y)/(n.next.y-n.y)+n.x&&(r=!r),n=n.next}while(n!==e);return r}(e,t)&&(Iu(e.prev,e,t.prev)||Iu(e,t.prev,t))||xu(e,t)&&Iu(e.prev,e,e.next)>0&&Iu(t.prev,t,t.next)>0)}function Iu(e,t,n){return(t.y-e.y)*(n.x-t.x)-(t.x-e.x)*(n.y-t.y)}function xu(e,t){return e.x===t.x&&e.y===t.y}function wu(e,t,n,r){var i=Su(Iu(e,t,n)),o=Su(Iu(e,t,r)),a=Su(Iu(n,r,e)),s=Su(Iu(n,r,t));return i!==o&&a!==s||(!(0!==i||!Mu(e,n,t))||(!(0!==o||!Mu(e,r,t))||(!(0!==a||!Mu(n,e,r))||!(0!==s||!Mu(n,t,r)))))}function Mu(e,t,n){return t.x<=Math.max(e.x,n.x)&&t.x>=Math.min(e.x,n.x)&&t.y<=Math.max(e.y,n.y)&&t.y>=Math.min(e.y,n.y)}function Su(e){return e>0?1:e<0?-1:0}function Tu(e,t){return Iu(e.prev,e,e.next)<0?Iu(e,t,e.next)>=0&&Iu(e,e.prev,t)>=0:Iu(e,t,e.prev)<0||Iu(e,e.next,t)<0}function Eu(e,t){var n=new Lu(e.i,e.x,e.y),r=new Lu(t.i,t.x,t.y),i=e.next,o=t.prev;return e.next=t,t.prev=e,n.next=i,i.prev=n,r.next=n,n.prev=r,o.next=r,r.prev=o,r}function Ou(e,t,n,r){var i=new Lu(e,t,n);return r?(i.next=r.next,i.prev=r,r.next.prev=i,r.next=i):(i.prev=i,i.next=i),i}function ku(e){e.next.prev=e.prev,e.prev.next=e.next,e.prevZ&&(e.prevZ.nextZ=e.nextZ),e.nextZ&&(e.nextZ.prevZ=e.prevZ)}function Lu(e,t,n){this.i=e,this.x=t,this.y=n,this.prev=null,this.next=null,this.z=null,this.prevZ=null,this.nextZ=null,this.steiner=!1}var Du={area:function(e){for(var t=e.length,n=0,r=t-1,i=0;i<t;r=i++)n+=e[r].x*e[i].y-e[i].x*e[r].y;return.5*n},isClockWise:function(e){return Du.area(e)<0},triangulateShape:function(e,t){var n=[],r=[],i=[];Pu(e),Nu(n,e);var o=e.length;t.forEach(Pu);for(var a=0;a<t.length;a++)r.push(o),o+=t[a].length,Nu(n,t[a]);var s=lu(n,r);for(a=0;a<s.length;a+=3)i.push(s.slice(a,a+3));return i}};function Pu(e){var t=e.length;t>2&&e[t-1].equals(e[0])&&e.pop()}function Nu(e,t){for(var n=0;n<t.length;n++)e.push(t[n].x),e.push(t[n].y)}function ju(e,t){vo.call(this),this.type="ExtrudeGeometry",this.parameters={shapes:e,options:t},this.fromBufferGeometry(new Ru(e,t)),this.mergeVertices()}function Ru(e,t){Yi.call(this),this.type="ExtrudeBufferGeometry",this.parameters={shapes:e,options:t},e=Array.isArray(e)?e:[e];for(var n=this,r=[],i=[],o=0,a=e.length;o<a;o++){s(e[o])}function s(e){var o=[],a=void 0!==t.curveSegments?t.curveSegments:12,s=void 0!==t.steps?t.steps:1,l=void 0!==t.depth?t.depth:100,u=void 0===t.bevelEnabled||t.bevelEnabled,c=void 0!==t.bevelThickness?t.bevelThickness:6,d=void 0!==t.bevelSize?t.bevelSize:c-2,h=void 0!==t.bevelOffset?t.bevelOffset:0,f=void 0!==t.bevelSegments?t.bevelSegments:3,p=t.extrudePath,g=void 0!==t.UVGenerator?t.UVGenerator:zu;void 0!==t.amount&&(console.warn("THREE.ExtrudeBufferGeometry: amount has been renamed to depth."),l=t.amount);var v,m,y,b,C,A,_,I,x=!1;p&&(v=p.getSpacedPoints(s),x=!0,u=!1,m=p.computeFrenetFrames(s,!1),y=new $n,b=new $n,C=new $n),u||(f=0,c=0,d=0,h=0);var w=e.extractPoints(a),M=w.shape,S=w.holes;if(!Du.isClockWise(M))for(M=M.reverse(),_=0,I=S.length;_<I;_++)A=S[_],Du.isClockWise(A)&&(S[_]=A.reverse());var T=Du.triangulateShape(M,S),E=M;for(_=0,I=S.length;_<I;_++)A=S[_],M=M.concat(A);function O(e,t,n){return t||console.error("THREE.ExtrudeGeometry: vec does not exist"),t.clone().multiplyScalar(n).add(e)}var k,L,D,P,N,j,R=M.length,z=T.length;function F(e,t,n){var r,i,o,a=e.x-t.x,s=e.y-t.y,l=n.x-e.x,u=n.y-e.y,c=a*a+s*s,d=a*u-s*l;if(Math.abs(d)>Number.EPSILON){var h=Math.sqrt(c),f=Math.sqrt(l*l+u*u),p=t.x-s/h,g=t.y+a/h,v=((n.x-u/f-p)*u-(n.y+l/f-g)*l)/(a*u-s*l),m=(r=p+a*v-e.x)*r+(i=g+s*v-e.y)*i;if(m<=2)return new Un(r,i);o=Math.sqrt(m/2)}else{var y=!1;a>Number.EPSILON?l>Number.EPSILON&&(y=!0):a<-Number.EPSILON?l<-Number.EPSILON&&(y=!0):Math.sign(s)===Math.sign(u)&&(y=!0),y?(r=-s,i=a,o=Math.sqrt(c)):(r=a,i=s,o=Math.sqrt(c/2))}return new Un(r/o,i/o)}for(var B=[],H=0,U=E.length,V=U-1,Y=H+1;H<U;H++,V++,Y++)V===U&&(V=0),Y===U&&(Y=0),B[H]=F(E[H],E[V],E[Y]);var W,G,X=[],q=B.concat();for(_=0,I=S.length;_<I;_++){for(A=S[_],W=[],H=0,V=(U=A.length)-1,Y=H+1;H<U;H++,V++,Y++)V===U&&(V=0),Y===U&&(Y=0),W[H]=F(A[H],A[V],A[Y]);X.push(W),q=q.concat(W)}for(k=0;k<f;k++){for(D=k/f,P=c*Math.cos(D*Math.PI/2),L=d*Math.sin(D*Math.PI/2)+h,H=0,U=E.length;H<U;H++)Q((N=O(E[H],B[H],L)).x,N.y,-P);for(_=0,I=S.length;_<I;_++)for(A=S[_],W=X[_],H=0,U=A.length;H<U;H++)Q((N=O(A[H],W[H],L)).x,N.y,-P)}for(L=d+h,H=0;H<R;H++)N=u?O(M[H],q[H],L):M[H],x?(b.copy(m.normals[0]).multiplyScalar(N.x),y.copy(m.binormals[0]).multiplyScalar(N.y),C.copy(v[0]).add(b).add(y),Q(C.x,C.y,C.z)):Q(N.x,N.y,0);for(G=1;G<=s;G++)for(H=0;H<R;H++)N=u?O(M[H],q[H],L):M[H],x?(b.copy(m.normals[G]).multiplyScalar(N.x),y.copy(m.binormals[G]).multiplyScalar(N.y),C.copy(v[G]).add(b).add(y),Q(C.x,C.y,C.z)):Q(N.x,N.y,l/s*G);for(k=f-1;k>=0;k--){for(D=k/f,P=c*Math.cos(D*Math.PI/2),L=d*Math.sin(D*Math.PI/2)+h,H=0,U=E.length;H<U;H++)Q((N=O(E[H],B[H],L)).x,N.y,l+P);for(_=0,I=S.length;_<I;_++)for(A=S[_],W=X[_],H=0,U=A.length;H<U;H++)N=O(A[H],W[H],L),x?Q(N.x,N.y+v[s-1].y,v[s-1].x+P):Q(N.x,N.y,l+P)}function Z(e,t){var n,r;for(H=e.length;--H>=0;){n=H,(r=H-1)<0&&(r=e.length-1);var i=0,o=s+2*f;for(i=0;i<o;i++){var a=R*i,l=R*(i+1);J(t+n+a,t+r+a,t+r+l,t+n+l)}}}function Q(e,t,n){o.push(e),o.push(t),o.push(n)}function K(e,t,i){$(e),$(t),$(i);var o=r.length/3,a=g.generateTopUV(n,r,o-3,o-2,o-1);ee(a[0]),ee(a[1]),ee(a[2])}function J(e,t,i,o){$(e),$(t),$(o),$(t),$(i),$(o);var a=r.length/3,s=g.generateSideWallUV(n,r,a-6,a-3,a-2,a-1);ee(s[0]),ee(s[1]),ee(s[3]),ee(s[1]),ee(s[2]),ee(s[3])}function $(e){r.push(o[3*e+0]),r.push(o[3*e+1]),r.push(o[3*e+2])}function ee(e){i.push(e.x),i.push(e.y)}!function(){var e=r.length/3;if(u){var t=0,i=R*t;for(H=0;H<z;H++)K((j=T[H])[2]+i,j[1]+i,j[0]+i);for(i=R*(t=s+2*f),H=0;H<z;H++)K((j=T[H])[0]+i,j[1]+i,j[2]+i)}else{for(H=0;H<z;H++)K((j=T[H])[2],j[1],j[0]);for(H=0;H<z;H++)K((j=T[H])[0]+R*s,j[1]+R*s,j[2]+R*s)}n.addGroup(e,r.length/3-e,0)}(),function(){var e=r.length/3,t=0;for(Z(E,t),t+=E.length,_=0,I=S.length;_<I;_++)Z(A=S[_],t),t+=A.length;n.addGroup(e,r.length/3-e,1)}()}this.setAttribute("position",new Di(r,3)),this.setAttribute("uv",new Di(i,2)),this.computeVertexNormals()}ju.prototype=Object.create(vo.prototype),ju.prototype.constructor=ju,ju.prototype.toJSON=function(){var e=vo.prototype.toJSON.call(this);return Fu(this.parameters.shapes,this.parameters.options,e)},Ru.prototype=Object.create(Yi.prototype),Ru.prototype.constructor=Ru,Ru.prototype.toJSON=function(){var e=Yi.prototype.toJSON.call(this);return Fu(this.parameters.shapes,this.parameters.options,e)};var zu={generateTopUV:function(e,t,n,r,i){var o=t[3*n],a=t[3*n+1],s=t[3*r],l=t[3*r+1],u=t[3*i],c=t[3*i+1];return[new Un(o,a),new Un(s,l),new Un(u,c)]},generateSideWallUV:function(e,t,n,r,i,o){var a=t[3*n],s=t[3*n+1],l=t[3*n+2],u=t[3*r],c=t[3*r+1],d=t[3*r+2],h=t[3*i],f=t[3*i+1],p=t[3*i+2],g=t[3*o],v=t[3*o+1],m=t[3*o+2];return Math.abs(s-c)<.01?[new Un(a,1-l),new Un(u,1-d),new Un(h,1-p),new Un(g,1-m)]:[new Un(s,1-l),new Un(c,1-d),new Un(f,1-p),new Un(v,1-m)]}};function Fu(e,t,n){if(n.shapes=[],Array.isArray(e))for(var r=0,i=e.length;r<i;r++){var o=e[r];n.shapes.push(o.uuid)}else n.shapes.push(e.uuid);return void 0!==t.extrudePath&&(n.options.extrudePath=t.extrudePath.toJSON()),n}function Bu(e,t){vo.call(this),this.type="TextGeometry",this.parameters={text:e,parameters:t},this.fromBufferGeometry(new Hu(e,t)),this.mergeVertices()}function Hu(e,t){var n=(t=t||{}).font;if(!n||!n.isFont)return console.error("THREE.TextGeometry: font parameter is not an instance of THREE.Font."),new vo;var r=n.generateShapes(e,t.size);t.depth=void 0!==t.height?t.height:50,void 0===t.bevelThickness&&(t.bevelThickness=10),void 0===t.bevelSize&&(t.bevelSize=8),void 0===t.bevelEnabled&&(t.bevelEnabled=!1),Ru.call(this,r,t),this.type="TextBufferGeometry"}function Uu(e,t,n,r,i,o,a){vo.call(this),this.type="SphereGeometry",this.parameters={radius:e,widthSegments:t,heightSegments:n,phiStart:r,phiLength:i,thetaStart:o,thetaLength:a},this.fromBufferGeometry(new Vu(e,t,n,r,i,o,a)),this.mergeVertices()}function Vu(e,t,n,r,i,o,a){Yi.call(this),this.type="SphereBufferGeometry",this.parameters={radius:e,widthSegments:t,heightSegments:n,phiStart:r,phiLength:i,thetaStart:o,thetaLength:a},e=e||1,t=Math.max(3,Math.floor(t)||8),n=Math.max(2,Math.floor(n)||6),r=void 0!==r?r:0,i=void 0!==i?i:2*Math.PI,o=void 0!==o?o:0,a=void 0!==a?a:Math.PI;var s,l,u=Math.min(o+a,Math.PI),c=0,d=[],h=new $n,f=new $n,p=[],g=[],v=[],m=[];for(l=0;l<=n;l++){var y=[],b=l/n,C=0;for(0==l&&0==o?C=.5/t:l==n&&u==Math.PI&&(C=-.5/t),s=0;s<=t;s++){var A=s/t;h.x=-e*Math.cos(r+A*i)*Math.sin(o+b*a),h.y=e*Math.cos(o+b*a),h.z=e*Math.sin(r+A*i)*Math.sin(o+b*a),g.push(h.x,h.y,h.z),f.copy(h).normalize(),v.push(f.x,f.y,f.z),m.push(A+C,1-b),y.push(c++)}d.push(y)}for(l=0;l<n;l++)for(s=0;s<t;s++){var _=d[l][s+1],I=d[l][s],x=d[l+1][s],w=d[l+1][s+1];(0!==l||o>0)&&p.push(_,I,w),(l!==n-1||u<Math.PI)&&p.push(I,x,w)}this.setIndex(p),this.setAttribute("position",new Di(g,3)),this.setAttribute("normal",new Di(v,3)),this.setAttribute("uv",new Di(m,2))}function Yu(e,t,n,r,i,o){vo.call(this),this.type="RingGeometry",this.parameters={innerRadius:e,outerRadius:t,thetaSegments:n,phiSegments:r,thetaStart:i,thetaLength:o},this.fromBufferGeometry(new Wu(e,t,n,r,i,o)),this.mergeVertices()}function Wu(e,t,n,r,i,o){Yi.call(this),this.type="RingBufferGeometry",this.parameters={innerRadius:e,outerRadius:t,thetaSegments:n,phiSegments:r,thetaStart:i,thetaLength:o},e=e||.5,t=t||1,i=void 0!==i?i:0,o=void 0!==o?o:2*Math.PI,n=void 0!==n?Math.max(3,n):8;var a,s,l,u=[],c=[],d=[],h=[],f=e,p=(t-e)/(r=void 0!==r?Math.max(1,r):1),g=new $n,v=new Un;for(s=0;s<=r;s++){for(l=0;l<=n;l++)a=i+l/n*o,g.x=f*Math.cos(a),g.y=f*Math.sin(a),c.push(g.x,g.y,g.z),d.push(0,0,1),v.x=(g.x/t+1)/2,v.y=(g.y/t+1)/2,h.push(v.x,v.y);f+=p}for(s=0;s<r;s++){var m=s*(n+1);for(l=0;l<n;l++){var y=a=l+m,b=a+n+1,C=a+n+2,A=a+1;u.push(y,b,A),u.push(b,C,A)}}this.setIndex(u),this.setAttribute("position",new Di(c,3)),this.setAttribute("normal",new Di(d,3)),this.setAttribute("uv",new Di(h,2))}function Gu(e,t,n,r){vo.call(this),this.type="LatheGeometry",this.parameters={points:e,segments:t,phiStart:n,phiLength:r},this.fromBufferGeometry(new Xu(e,t,n,r)),this.mergeVertices()}function Xu(e,t,n,r){Yi.call(this),this.type="LatheBufferGeometry",this.parameters={points:e,segments:t,phiStart:n,phiLength:r},t=Math.floor(t)||12,n=n||0,r=r||2*Math.PI,r=Hn.clamp(r,0,2*Math.PI);var i,o,a,s=[],l=[],u=[],c=1/t,d=new $n,h=new Un;for(o=0;o<=t;o++){var f=n+o*c*r,p=Math.sin(f),g=Math.cos(f);for(a=0;a<=e.length-1;a++)d.x=e[a].x*p,d.y=e[a].y,d.z=e[a].x*g,l.push(d.x,d.y,d.z),h.x=o/t,h.y=a/(e.length-1),u.push(h.x,h.y)}for(o=0;o<t;o++)for(a=0;a<e.length-1;a++){var v=i=a+o*e.length,m=i+e.length,y=i+e.length+1,b=i+1;s.push(v,m,b),s.push(m,y,b)}if(this.setIndex(s),this.setAttribute("position",new Di(l,3)),this.setAttribute("uv",new Di(u,2)),this.computeVertexNormals(),r===2*Math.PI){var C=this.attributes.normal.array,A=new $n,_=new $n,I=new $n;for(i=t*e.length*3,o=0,a=0;o<e.length;o++,a+=3)A.x=C[a+0],A.y=C[a+1],A.z=C[a+2],_.x=C[i+a+0],_.y=C[i+a+1],_.z=C[i+a+2],I.addVectors(A,_).normalize(),C[a+0]=C[i+a+0]=I.x,C[a+1]=C[i+a+1]=I.y,C[a+2]=C[i+a+2]=I.z}}function qu(e,t){vo.call(this),this.type="ShapeGeometry","object"==typeof t&&(console.warn("THREE.ShapeGeometry: Options parameter has been removed."),t=t.curveSegments),this.parameters={shapes:e,curveSegments:t},this.fromBufferGeometry(new Zu(e,t)),this.mergeVertices()}function Zu(e,t){Yi.call(this),this.type="ShapeBufferGeometry",this.parameters={shapes:e,curveSegments:t},t=t||12;var n=[],r=[],i=[],o=[],a=0,s=0;if(!1===Array.isArray(e))u(e);else for(var l=0;l<e.length;l++)u(e[l]),this.addGroup(a,s,l),a+=s,s=0;function u(e){var a,l,u,c=r.length/3,d=e.extractPoints(t),h=d.shape,f=d.holes;for(!1===Du.isClockWise(h)&&(h=h.reverse()),a=0,l=f.length;a<l;a++)u=f[a],!0===Du.isClockWise(u)&&(f[a]=u.reverse());var p=Du.triangulateShape(h,f);for(a=0,l=f.length;a<l;a++)u=f[a],h=h.concat(u);for(a=0,l=h.length;a<l;a++){var g=h[a];r.push(g.x,g.y,0),i.push(0,0,1),o.push(g.x,g.y)}for(a=0,l=p.length;a<l;a++){var v=p[a],m=v[0]+c,y=v[1]+c,b=v[2]+c;n.push(m,y,b),s+=3}}this.setIndex(n),this.setAttribute("position",new Di(r,3)),this.setAttribute("normal",new Di(i,3)),this.setAttribute("uv",new Di(o,2))}function Qu(e,t){if(t.shapes=[],Array.isArray(e))for(var n=0,r=e.length;n<r;n++){var i=e[n];t.shapes.push(i.uuid)}else t.shapes.push(e.uuid);return t}function Ku(e,t){Yi.call(this),this.type="EdgesGeometry",this.parameters={thresholdAngle:t},t=void 0!==t?t:1;var n,r,i,o,a=[],s=Math.cos(Hn.DEG2RAD*t),l=[0,0],u={},c=["a","b","c"];e.isBufferGeometry?(o=new vo).fromBufferGeometry(e):o=e.clone(),o.mergeVertices(),o.computeFaceNormals();for(var d=o.vertices,h=o.faces,f=0,p=h.length;f<p;f++)for(var g=h[f],v=0;v<3;v++)n=g[c[v]],r=g[c[(v+1)%3]],l[0]=Math.min(n,r),l[1]=Math.max(n,r),void 0===u[i=l[0]+","+l[1]]?u[i]={index1:l[0],index2:l[1],face1:f,face2:void 0}:u[i].face2=f;for(i in u){var m=u[i];if(void 0===m.face2||h[m.face1].normal.dot(h[m.face2].normal)<=s){var y=d[m.index1];a.push(y.x,y.y,y.z),y=d[m.index2],a.push(y.x,y.y,y.z)}}this.setAttribute("position",new Di(a,3))}function Ju(e,t,n,r,i,o,a,s){vo.call(this),this.type="CylinderGeometry",this.parameters={radiusTop:e,radiusBottom:t,height:n,radialSegments:r,heightSegments:i,openEnded:o,thetaStart:a,thetaLength:s},this.fromBufferGeometry(new $u(e,t,n,r,i,o,a,s)),this.mergeVertices()}function $u(e,t,n,r,i,o,a,s){Yi.call(this),this.type="CylinderBufferGeometry",this.parameters={radiusTop:e,radiusBottom:t,height:n,radialSegments:r,heightSegments:i,openEnded:o,thetaStart:a,thetaLength:s};var l=this;e=void 0!==e?e:1,t=void 0!==t?t:1,n=n||1,r=Math.floor(r)||8,i=Math.floor(i)||1,o=void 0!==o&&o,a=void 0!==a?a:0,s=void 0!==s?s:2*Math.PI;var u=[],c=[],d=[],h=[],f=0,p=[],g=n/2,v=0;function m(n){var i,o,p,m=new Un,y=new $n,b=0,C=!0===n?e:t,A=!0===n?1:-1;for(o=f,i=1;i<=r;i++)c.push(0,g*A,0),d.push(0,A,0),h.push(.5,.5),f++;for(p=f,i=0;i<=r;i++){var _=i/r*s+a,I=Math.cos(_),x=Math.sin(_);y.x=C*x,y.y=g*A,y.z=C*I,c.push(y.x,y.y,y.z),d.push(0,A,0),m.x=.5*I+.5,m.y=.5*x*A+.5,h.push(m.x,m.y),f++}for(i=0;i<r;i++){var w=o+i,M=p+i;!0===n?u.push(M,M+1,w):u.push(M+1,M,w),b+=3}l.addGroup(v,b,!0===n?1:2),v+=b}!function(){var o,m,y=new $n,b=new $n,C=0,A=(t-e)/n;for(m=0;m<=i;m++){var _=[],I=m/i,x=I*(t-e)+e;for(o=0;o<=r;o++){var w=o/r,M=w*s+a,S=Math.sin(M),T=Math.cos(M);b.x=x*S,b.y=-I*n+g,b.z=x*T,c.push(b.x,b.y,b.z),y.set(S,A,T).normalize(),d.push(y.x,y.y,y.z),h.push(w,1-I),_.push(f++)}p.push(_)}for(o=0;o<r;o++)for(m=0;m<i;m++){var E=p[m][o],O=p[m+1][o],k=p[m+1][o+1],L=p[m][o+1];u.push(E,O,L),u.push(O,k,L),C+=6}l.addGroup(v,C,0),v+=C}(),!1===o&&(e>0&&m(!0),t>0&&m(!1)),this.setIndex(u),this.setAttribute("position",new Di(c,3)),this.setAttribute("normal",new Di(d,3)),this.setAttribute("uv",new Di(h,2))}function ec(e,t,n,r,i,o,a){Ju.call(this,0,e,t,n,r,i,o,a),this.type="ConeGeometry",this.parameters={radius:e,height:t,radialSegments:n,heightSegments:r,openEnded:i,thetaStart:o,thetaLength:a}}function tc(e,t,n,r,i,o,a){$u.call(this,0,e,t,n,r,i,o,a),this.type="ConeBufferGeometry",this.parameters={radius:e,height:t,radialSegments:n,heightSegments:r,openEnded:i,thetaStart:o,thetaLength:a}}function nc(e,t,n,r){vo.call(this),this.type="CircleGeometry",this.parameters={radius:e,segments:t,thetaStart:n,thetaLength:r},this.fromBufferGeometry(new rc(e,t,n,r)),this.mergeVertices()}function rc(e,t,n,r){Yi.call(this),this.type="CircleBufferGeometry",this.parameters={radius:e,segments:t,thetaStart:n,thetaLength:r},e=e||1,t=void 0!==t?Math.max(3,t):8,n=void 0!==n?n:0,r=void 0!==r?r:2*Math.PI;var i,o,a=[],s=[],l=[],u=[],c=new $n,d=new Un;for(s.push(0,0,0),l.push(0,0,1),u.push(.5,.5),o=0,i=3;o<=t;o++,i+=3){var h=n+o/t*r;c.x=e*Math.cos(h),c.y=e*Math.sin(h),s.push(c.x,c.y,c.z),l.push(0,0,1),d.x=(s[i]/e+1)/2,d.y=(s[i+1]/e+1)/2,u.push(d.x,d.y)}for(i=1;i<=t;i++)a.push(i,i+1,0);this.setIndex(a),this.setAttribute("position",new Di(s,3)),this.setAttribute("normal",new Di(l,3)),this.setAttribute("uv",new Di(u,2))}Bu.prototype=Object.create(vo.prototype),Bu.prototype.constructor=Bu,Hu.prototype=Object.create(Ru.prototype),Hu.prototype.constructor=Hu,Uu.prototype=Object.create(vo.prototype),Uu.prototype.constructor=Uu,Vu.prototype=Object.create(Yi.prototype),Vu.prototype.constructor=Vu,Yu.prototype=Object.create(vo.prototype),Yu.prototype.constructor=Yu,Wu.prototype=Object.create(Yi.prototype),Wu.prototype.constructor=Wu,Gu.prototype=Object.create(vo.prototype),Gu.prototype.constructor=Gu,Xu.prototype=Object.create(Yi.prototype),Xu.prototype.constructor=Xu,qu.prototype=Object.create(vo.prototype),qu.prototype.constructor=qu,qu.prototype.toJSON=function(){var e=vo.prototype.toJSON.call(this);return Qu(this.parameters.shapes,e)},Zu.prototype=Object.create(Yi.prototype),Zu.prototype.constructor=Zu,Zu.prototype.toJSON=function(){var e=Yi.prototype.toJSON.call(this);return Qu(this.parameters.shapes,e)},Ku.prototype=Object.create(Yi.prototype),Ku.prototype.constructor=Ku,Ju.prototype=Object.create(vo.prototype),Ju.prototype.constructor=Ju,$u.prototype=Object.create(Yi.prototype),$u.prototype.constructor=$u,ec.prototype=Object.create(Ju.prototype),ec.prototype.constructor=ec,tc.prototype=Object.create($u.prototype),tc.prototype.constructor=tc,nc.prototype=Object.create(vo.prototype),nc.prototype.constructor=nc,rc.prototype=Object.create(Yi.prototype),rc.prototype.constructor=rc;var ic=Object.freeze({__proto__:null,WireframeGeometry:Vl,ParametricGeometry:Yl,ParametricBufferGeometry:Wl,TetrahedronGeometry:ql,TetrahedronBufferGeometry:Zl,OctahedronGeometry:Ql,OctahedronBufferGeometry:Kl,IcosahedronGeometry:Jl,IcosahedronBufferGeometry:$l,DodecahedronGeometry:eu,DodecahedronBufferGeometry:tu,PolyhedronGeometry:Gl,PolyhedronBufferGeometry:Xl,TubeGeometry:nu,TubeBufferGeometry:ru,TorusKnotGeometry:iu,TorusKnotBufferGeometry:ou,TorusGeometry:au,TorusBufferGeometry:su,TextGeometry:Bu,TextBufferGeometry:Hu,SphereGeometry:Uu,SphereBufferGeometry:Vu,RingGeometry:Yu,RingBufferGeometry:Wu,PlaneGeometry:Po,PlaneBufferGeometry:No,LatheGeometry:Gu,LatheBufferGeometry:Xu,ShapeGeometry:qu,ShapeBufferGeometry:Zu,ExtrudeGeometry:ju,ExtrudeBufferGeometry:Ru,EdgesGeometry:Ku,ConeGeometry:ec,ConeBufferGeometry:tc,CylinderGeometry:Ju,CylinderBufferGeometry:$u,CircleGeometry:nc,CircleBufferGeometry:rc,BoxGeometry:mo,BoxBufferGeometry:yo});function oc(e){_i.call(this),this.type="ShadowMaterial",this.color=new vi(0),this.transparent=!0,this.setValues(e)}function ac(e){_o.call(this,e),this.type="RawShaderMaterial"}function sc(e){_i.call(this),this.defines={STANDARD:""},this.type="MeshStandardMaterial",this.color=new vi(16777215),this.roughness=1,this.metalness=0,this.map=null,this.lightMap=null,this.lightMapIntensity=1,this.aoMap=null,this.aoMapIntensity=1,this.emissive=new vi(0),this.emissiveIntensity=1,this.emissiveMap=null,this.bumpMap=null,this.bumpScale=1,this.normalMap=null,this.normalMapType=cn,this.normalScale=new Un(1,1),this.displacementMap=null,this.displacementScale=1,this.displacementBias=0,this.roughnessMap=null,this.metalnessMap=null,this.alphaMap=null,this.envMap=null,this.envMapIntensity=1,this.refractionRatio=.98,this.wireframe=!1,this.wireframeLinewidth=1,this.wireframeLinecap="round",this.wireframeLinejoin="round",this.skinning=!1,this.morphTargets=!1,this.morphNormals=!1,this.vertexTangents=!1,this.setValues(e)}function lc(e){sc.call(this),this.defines={STANDARD:"",PHYSICAL:""},this.type="MeshPhysicalMaterial",this.clearcoat=0,this.clearcoatMap=null,this.clearcoatRoughness=0,this.clearcoatRoughnessMap=null,this.clearcoatNormalScale=new Un(1,1),this.clearcoatNormalMap=null,this.reflectivity=.5,this.sheen=null,this.transparency=0,this.setValues(e)}function uc(e){_i.call(this),this.type="MeshPhongMaterial",this.color=new vi(16777215),this.specular=new vi(1118481),this.shininess=30,this.map=null,this.lightMap=null,this.lightMapIntensity=1,this.aoMap=null,this.aoMapIntensity=1,this.emissive=new vi(0),this.emissiveIntensity=1,this.emissiveMap=null,this.bumpMap=null,this.bumpScale=1,this.normalMap=null,this.normalMapType=cn,this.normalScale=new Un(1,1),this.displacementMap=null,this.displacementScale=1,this.displacementBias=0,this.specularMap=null,this.alphaMap=null,this.envMap=null,this.combine=K,this.reflectivity=1,this.refractionRatio=.98,this.wireframe=!1,this.wireframeLinewidth=1,this.wireframeLinecap="round",this.wireframeLinejoin="round",this.skinning=!1,this.morphTargets=!1,this.morphNormals=!1,this.setValues(e)}function cc(e){_i.call(this),this.defines={TOON:""},this.type="MeshToonMaterial",this.color=new vi(16777215),this.specular=new vi(1118481),this.shininess=30,this.map=null,this.gradientMap=null,this.lightMap=null,this.lightMapIntensity=1,this.aoMap=null,this.aoMapIntensity=1,this.emissive=new vi(0),this.emissiveIntensity=1,this.emissiveMap=null,this.bumpMap=null,this.bumpScale=1,this.normalMap=null,this.normalMapType=cn,this.normalScale=new Un(1,1),this.displacementMap=null,this.displacementScale=1,this.displacementBias=0,this.specularMap=null,this.alphaMap=null,this.wireframe=!1,this.wireframeLinewidth=1,this.wireframeLinecap="round",this.wireframeLinejoin="round",this.skinning=!1,this.morphTargets=!1,this.morphNormals=!1,this.setValues(e)}function dc(e){_i.call(this),this.type="MeshNormalMaterial",this.bumpMap=null,this.bumpScale=1,this.normalMap=null,this.normalMapType=cn,this.normalScale=new Un(1,1),this.displacementMap=null,this.displacementScale=1,this.displacementBias=0,this.wireframe=!1,this.wireframeLinewidth=1,this.fog=!1,this.skinning=!1,this.morphTargets=!1,this.morphNormals=!1,this.setValues(e)}function hc(e){_i.call(this),this.type="MeshLambertMaterial",this.color=new vi(16777215),this.map=null,this.lightMap=null,this.lightMapIntensity=1,this.aoMap=null,this.aoMapIntensity=1,this.emissive=new vi(0),this.emissiveIntensity=1,this.emissiveMap=null,this.specularMap=null,this.alphaMap=null,this.envMap=null,this.combine=K,this.reflectivity=1,this.refractionRatio=.98,this.wireframe=!1,this.wireframeLinewidth=1,this.wireframeLinecap="round",this.wireframeLinejoin="round",this.skinning=!1,this.morphTargets=!1,this.morphNormals=!1,this.setValues(e)}function fc(e){_i.call(this),this.defines={MATCAP:""},this.type="MeshMatcapMaterial",this.color=new vi(16777215),this.matcap=null,this.map=null,this.bumpMap=null,this.bumpScale=1,this.normalMap=null,this.normalMapType=cn,this.normalScale=new Un(1,1),this.displacementMap=null,this.displacementScale=1,this.displacementBias=0,this.alphaMap=null,this.skinning=!1,this.morphTargets=!1,this.morphNormals=!1,this.setValues(e)}function pc(e){Al.call(this),this.type="LineDashedMaterial",this.scale=1,this.dashSize=3,this.gapSize=1,this.setValues(e)}oc.prototype=Object.create(_i.prototype),oc.prototype.constructor=oc,oc.prototype.isShadowMaterial=!0,oc.prototype.copy=function(e){return _i.prototype.copy.call(this,e),this.color.copy(e.color),this},ac.prototype=Object.create(_o.prototype),ac.prototype.constructor=ac,ac.prototype.isRawShaderMaterial=!0,sc.prototype=Object.create(_i.prototype),sc.prototype.constructor=sc,sc.prototype.isMeshStandardMaterial=!0,sc.prototype.copy=function(e){return _i.prototype.copy.call(this,e),this.defines={STANDARD:""},this.color.copy(e.color),this.roughness=e.roughness,this.metalness=e.metalness,this.map=e.map,this.lightMap=e.lightMap,this.lightMapIntensity=e.lightMapIntensity,this.aoMap=e.aoMap,this.aoMapIntensity=e.aoMapIntensity,this.emissive.copy(e.emissive),this.emissiveMap=e.emissiveMap,this.emissiveIntensity=e.emissiveIntensity,this.bumpMap=e.bumpMap,this.bumpScale=e.bumpScale,this.normalMap=e.normalMap,this.normalMapType=e.normalMapType,this.normalScale.copy(e.normalScale),this.displacementMap=e.displacementMap,this.displacementScale=e.displacementScale,this.displacementBias=e.displacementBias,this.roughnessMap=e.roughnessMap,this.metalnessMap=e.metalnessMap,this.alphaMap=e.alphaMap,this.envMap=e.envMap,this.envMapIntensity=e.envMapIntensity,this.refractionRatio=e.refractionRatio,this.wireframe=e.wireframe,this.wireframeLinewidth=e.wireframeLinewidth,this.wireframeLinecap=e.wireframeLinecap,this.wireframeLinejoin=e.wireframeLinejoin,this.skinning=e.skinning,this.morphTargets=e.morphTargets,this.morphNormals=e.morphNormals,this.vertexTangents=e.vertexTangents,this},lc.prototype=Object.create(sc.prototype),lc.prototype.constructor=lc,lc.prototype.isMeshPhysicalMaterial=!0,lc.prototype.copy=function(e){return sc.prototype.copy.call(this,e),this.defines={STANDARD:"",PHYSICAL:""},this.clearcoat=e.clearcoat,this.clearcoatMap=e.clearcoatMap,this.clearcoatRoughness=e.clearcoatRoughness,this.clearcoatRoughnessMap=e.clearcoatRoughnessMap,this.clearcoatNormalMap=e.clearcoatNormalMap,this.clearcoatNormalScale.copy(e.clearcoatNormalScale),this.reflectivity=e.reflectivity,e.sheen?this.sheen=(this.sheen||new vi).copy(e.sheen):this.sheen=null,this.transparency=e.transparency,this},uc.prototype=Object.create(_i.prototype),uc.prototype.constructor=uc,uc.prototype.isMeshPhongMaterial=!0,uc.prototype.copy=function(e){return _i.prototype.copy.call(this,e),this.color.copy(e.color),this.specular.copy(e.specular),this.shininess=e.shininess,this.map=e.map,this.lightMap=e.lightMap,this.lightMapIntensity=e.lightMapIntensity,this.aoMap=e.aoMap,this.aoMapIntensity=e.aoMapIntensity,this.emissive.copy(e.emissive),this.emissiveMap=e.emissiveMap,this.emissiveIntensity=e.emissiveIntensity,this.bumpMap=e.bumpMap,this.bumpScale=e.bumpScale,this.normalMap=e.normalMap,this.normalMapType=e.normalMapType,this.normalScale.copy(e.normalScale),this.displacementMap=e.displacementMap,this.displacementScale=e.displacementScale,this.displacementBias=e.displacementBias,this.specularMap=e.specularMap,this.alphaMap=e.alphaMap,this.envMap=e.envMap,this.combine=e.combine,this.reflectivity=e.reflectivity,this.refractionRatio=e.refractionRatio,this.wireframe=e.wireframe,this.wireframeLinewidth=e.wireframeLinewidth,this.wireframeLinecap=e.wireframeLinecap,this.wireframeLinejoin=e.wireframeLinejoin,this.skinning=e.skinning,this.morphTargets=e.morphTargets,this.morphNormals=e.morphNormals,this},cc.prototype=Object.create(_i.prototype),cc.prototype.constructor=cc,cc.prototype.isMeshToonMaterial=!0,cc.prototype.copy=function(e){return _i.prototype.copy.call(this,e),this.color.copy(e.color),this.specular.copy(e.specular),this.shininess=e.shininess,this.map=e.map,this.gradientMap=e.gradientMap,this.lightMap=e.lightMap,this.lightMapIntensity=e.lightMapIntensity,this.aoMap=e.aoMap,this.aoMapIntensity=e.aoMapIntensity,this.emissive.copy(e.emissive),this.emissiveMap=e.emissiveMap,this.emissiveIntensity=e.emissiveIntensity,this.bumpMap=e.bumpMap,this.bumpScale=e.bumpScale,this.normalMap=e.normalMap,this.normalMapType=e.normalMapType,this.normalScale.copy(e.normalScale),this.displacementMap=e.displacementMap,this.displacementScale=e.displacementScale,this.displacementBias=e.displacementBias,this.specularMap=e.specularMap,this.alphaMap=e.alphaMap,this.wireframe=e.wireframe,this.wireframeLinewidth=e.wireframeLinewidth,this.wireframeLinecap=e.wireframeLinecap,this.wireframeLinejoin=e.wireframeLinejoin,this.skinning=e.skinning,this.morphTargets=e.morphTargets,this.morphNormals=e.morphNormals,this},dc.prototype=Object.create(_i.prototype),dc.prototype.constructor=dc,dc.prototype.isMeshNormalMaterial=!0,dc.prototype.copy=function(e){return _i.prototype.copy.call(this,e),this.bumpMap=e.bumpMap,this.bumpScale=e.bumpScale,this.normalMap=e.normalMap,this.normalMapType=e.normalMapType,this.normalScale.copy(e.normalScale),this.displacementMap=e.displacementMap,this.displacementScale=e.displacementScale,this.displacementBias=e.displacementBias,this.wireframe=e.wireframe,this.wireframeLinewidth=e.wireframeLinewidth,this.skinning=e.skinning,this.morphTargets=e.morphTargets,this.morphNormals=e.morphNormals,this},hc.prototype=Object.create(_i.prototype),hc.prototype.constructor=hc,hc.prototype.isMeshLambertMaterial=!0,hc.prototype.copy=function(e){return _i.prototype.copy.call(this,e),this.color.copy(e.color),this.map=e.map,this.lightMap=e.lightMap,this.lightMapIntensity=e.lightMapIntensity,this.aoMap=e.aoMap,this.aoMapIntensity=e.aoMapIntensity,this.emissive.copy(e.emissive),this.emissiveMap=e.emissiveMap,this.emissiveIntensity=e.emissiveIntensity,this.specularMap=e.specularMap,this.alphaMap=e.alphaMap,this.envMap=e.envMap,this.combine=e.combine,this.reflectivity=e.reflectivity,this.refractionRatio=e.refractionRatio,this.wireframe=e.wireframe,this.wireframeLinewidth=e.wireframeLinewidth,this.wireframeLinecap=e.wireframeLinecap,this.wireframeLinejoin=e.wireframeLinejoin,this.skinning=e.skinning,this.morphTargets=e.morphTargets,this.morphNormals=e.morphNormals,this},fc.prototype=Object.create(_i.prototype),fc.prototype.constructor=fc,fc.prototype.isMeshMatcapMaterial=!0,fc.prototype.copy=function(e){return _i.prototype.copy.call(this,e),this.defines={MATCAP:""},this.color.copy(e.color),this.matcap=e.matcap,this.map=e.map,this.bumpMap=e.bumpMap,this.bumpScale=e.bumpScale,this.normalMap=e.normalMap,this.normalMapType=e.normalMapType,this.normalScale.copy(e.normalScale),this.displacementMap=e.displacementMap,this.displacementScale=e.displacementScale,this.displacementBias=e.displacementBias,this.alphaMap=e.alphaMap,this.skinning=e.skinning,this.morphTargets=e.morphTargets,this.morphNormals=e.morphNormals,this},pc.prototype=Object.create(Al.prototype),pc.prototype.constructor=pc,pc.prototype.isLineDashedMaterial=!0,pc.prototype.copy=function(e){return Al.prototype.copy.call(this,e),this.scale=e.scale,this.dashSize=e.dashSize,this.gapSize=e.gapSize,this};var gc=Object.freeze({__proto__:null,ShadowMaterial:oc,SpriteMaterial:Us,RawShaderMaterial:ac,ShaderMaterial:_o,PointsMaterial:Ll,MeshPhysicalMaterial:lc,MeshStandardMaterial:sc,MeshPhongMaterial:uc,MeshToonMaterial:cc,MeshNormalMaterial:dc,MeshLambertMaterial:hc,MeshDepthMaterial:xs,MeshDistanceMaterial:ws,MeshBasicMaterial:Ii,MeshMatcapMaterial:fc,LineDashedMaterial:pc,LineBasicMaterial:Al,Material:_i}),vc={arraySlice:function(e,t,n){return vc.isTypedArray(e)?new e.constructor(e.subarray(t,void 0!==n?n:e.length)):e.slice(t,n)},convertArray:function(e,t,n){return!e||!n&&e.constructor===t?e:"number"==typeof t.BYTES_PER_ELEMENT?new t(e):Array.prototype.slice.call(e)},isTypedArray:function(e){return ArrayBuffer.isView(e)&&!(e instanceof DataView)},getKeyframeOrder:function(e){for(var t=e.length,n=new Array(t),r=0;r!==t;++r)n[r]=r;return n.sort((function(t,n){return e[t]-e[n]})),n},sortedArray:function(e,t,n){for(var r=e.length,i=new e.constructor(r),o=0,a=0;a!==r;++o)for(var s=n[o]*t,l=0;l!==t;++l)i[a++]=e[s+l];return i},flattenJSON:function(e,t,n,r){for(var i=1,o=e[0];void 0!==o&&void 0===o[r];)o=e[i++];if(void 0!==o){var a=o[r];if(void 0!==a)if(Array.isArray(a))do{void 0!==(a=o[r])&&(t.push(o.time),n.push.apply(n,a)),o=e[i++]}while(void 0!==o);else if(void 0!==a.toArray)do{void 0!==(a=o[r])&&(t.push(o.time),a.toArray(n,n.length)),o=e[i++]}while(void 0!==o);else do{void 0!==(a=o[r])&&(t.push(o.time),n.push(a)),o=e[i++]}while(void 0!==o)}},subclip:function(e,t,n,r,i){i=i||30;var o=e.clone();o.name=t;for(var a=[],s=0;s<o.tracks.length;++s){for(var l=o.tracks[s],u=l.getValueSize(),c=[],d=[],h=0;h<l.times.length;++h){var f=l.times[h]*i;if(!(f<n||f>=r)){c.push(l.times[h]);for(var p=0;p<u;++p)d.push(l.values[h*u+p])}}0!==c.length&&(l.times=vc.convertArray(c,l.times.constructor),l.values=vc.convertArray(d,l.values.constructor),a.push(l))}o.tracks=a;var g=1/0;for(s=0;s<o.tracks.length;++s)g>o.tracks[s].times[0]&&(g=o.tracks[s].times[0]);for(s=0;s<o.tracks.length;++s)o.tracks[s].shift(-1*g);return o.resetDuration(),o},makeClipAdditive:function(e,t,n,r){void 0===t&&(t=0),void 0===n&&(n=e),(void 0===r||r<=0)&&(r=30);for(var i=e.tracks.length,o=t/r,a=0;a<i;++a){var s=n.tracks[a],l=s.ValueTypeName;if("bool"!==l&&"string"!==l){var u=e.tracks.find((function(e){return e.name===s.name&&e.ValueTypeName===l}));if(void 0!==u){var c,d=s.getValueSize(),h=s.times.length-1;if(o<=s.times[0])c=vc.arraySlice(s.values,0,s.valueSize);else if(o>=s.times[h]){var f=h*d;c=vc.arraySlice(s.values,f)}else{var p=s.createInterpolant();p.evaluate(o),c=p.resultBuffer}if("quaternion"===l)new Qn(c[0],c[1],c[2],c[3]).normalize().conjugate().toArray(c);for(var g=u.times.length,v=0;v<g;++v){var m=v*d;if("quaternion"===l)Qn.multiplyQuaternionsFlat(u.values,m,c,0,u.values,m);else for(var y=0;y<d;++y)u.values[m+y]-=c[y]}}}}return e.blendMode=Zt,e}};function mc(e,t,n,r){this.parameterPositions=e,this._cachedIndex=0,this.resultBuffer=void 0!==r?r:new t.constructor(n),this.sampleValues=t,this.valueSize=n}function yc(e,t,n,r){mc.call(this,e,t,n,r),this._weightPrev=-0,this._offsetPrev=-0,this._weightNext=-0,this._offsetNext=-0}function bc(e,t,n,r){mc.call(this,e,t,n,r)}function Cc(e,t,n,r){mc.call(this,e,t,n,r)}function Ac(e,t,n,r){if(void 0===e)throw new Error("THREE.KeyframeTrack: track name is undefined");if(void 0===t||0===t.length)throw new Error("THREE.KeyframeTrack: no keyframes in track named "+e);this.name=e,this.times=vc.convertArray(t,this.TimeBufferType),this.values=vc.convertArray(n,this.ValueBufferType),this.setInterpolation(r||this.DefaultInterpolation)}function _c(e,t,n){Ac.call(this,e,t,n)}function Ic(e,t,n,r){Ac.call(this,e,t,n,r)}function xc(e,t,n,r){Ac.call(this,e,t,n,r)}function wc(e,t,n,r){mc.call(this,e,t,n,r)}function Mc(e,t,n,r){Ac.call(this,e,t,n,r)}function Sc(e,t,n,r){Ac.call(this,e,t,n,r)}function Tc(e,t,n,r){Ac.call(this,e,t,n,r)}function Ec(e,t,n,r){this.name=e,this.tracks=n,this.duration=void 0!==t?t:-1,this.blendMode=void 0!==r?r:qt,this.uuid=Hn.generateUUID(),this.duration<0&&this.resetDuration()}function Oc(e){if(void 0===e.type)throw new Error("THREE.KeyframeTrack: track type undefined, can not parse");var t=function(e){switch(e.toLowerCase()){case"scalar":case"double":case"float":case"number":case"integer":return xc;case"vector":case"vector2":case"vector3":case"vector4":return Tc;case"color":return Ic;case"quaternion":return Mc;case"bool":case"boolean":return _c;case"string":return Sc}throw new Error("THREE.KeyframeTrack: Unsupported typeName: "+e)}(e.type);if(void 0===e.times){var n=[],r=[];vc.flattenJSON(e.keys,n,r,"value"),e.times=n,e.values=r}return void 0!==t.parse?t.parse(e):new t(e.name,e.times,e.values,e.interpolation)}Object.assign(mc.prototype,{evaluate:function(e){var t=this.parameterPositions,n=this._cachedIndex,r=t[n],i=t[n-1];e:{t:{var o;n:{r:if(!(e<r)){for(var a=n+2;;){if(void 0===r){if(e<i)break r;return n=t.length,this._cachedIndex=n,this.afterEnd_(n-1,e,i)}if(n===a)break;if(i=r,e<(r=t[++n]))break t}o=t.length;break n}if(e>=i)break e;var s=t[1];e<s&&(n=2,i=s);for(a=n-2;;){if(void 0===i)return this._cachedIndex=0,this.beforeStart_(0,e,r);if(n===a)break;if(r=i,e>=(i=t[--n-1]))break t}o=n,n=0}for(;n<o;){var l=n+o>>>1;e<t[l]?o=l:n=l+1}if(r=t[n],void 0===(i=t[n-1]))return this._cachedIndex=0,this.beforeStart_(0,e,r);if(void 0===r)return n=t.length,this._cachedIndex=n,this.afterEnd_(n-1,i,e)}this._cachedIndex=n,this.intervalChanged_(n,i,r)}return this.interpolate_(n,i,e,r)},settings:null,DefaultSettings_:{},getSettings_:function(){return this.settings||this.DefaultSettings_},copySampleValue_:function(e){for(var t=this.resultBuffer,n=this.sampleValues,r=this.valueSize,i=e*r,o=0;o!==r;++o)t[o]=n[i+o];return t},interpolate_:function(){throw new Error("call to abstract method")},intervalChanged_:function(){}}),Object.assign(mc.prototype,{beforeStart_:mc.prototype.copySampleValue_,afterEnd_:mc.prototype.copySampleValue_}),yc.prototype=Object.assign(Object.create(mc.prototype),{constructor:yc,DefaultSettings_:{endingStart:Wt,endingEnd:Wt},intervalChanged_:function(e,t,n){var r=this.parameterPositions,i=e-2,o=e+1,a=r[i],s=r[o];if(void 0===a)switch(this.getSettings_().endingStart){case Gt:i=e,a=2*t-n;break;case Xt:a=t+r[i=r.length-2]-r[i+1];break;default:i=e,a=n}if(void 0===s)switch(this.getSettings_().endingEnd){case Gt:o=e,s=2*n-t;break;case Xt:o=1,s=n+r[1]-r[0];break;default:o=e-1,s=t}var l=.5*(n-t),u=this.valueSize;this._weightPrev=l/(t-a),this._weightNext=l/(s-n),this._offsetPrev=i*u,this._offsetNext=o*u},interpolate_:function(e,t,n,r){for(var i=this.resultBuffer,o=this.sampleValues,a=this.valueSize,s=e*a,l=s-a,u=this._offsetPrev,c=this._offsetNext,d=this._weightPrev,h=this._weightNext,f=(n-t)/(r-t),p=f*f,g=p*f,v=-d*g+2*d*p-d*f,m=(1+d)*g+(-1.5-2*d)*p+(-.5+d)*f+1,y=(-1-h)*g+(1.5+h)*p+.5*f,b=h*g-h*p,C=0;C!==a;++C)i[C]=v*o[u+C]+m*o[l+C]+y*o[s+C]+b*o[c+C];return i}}),bc.prototype=Object.assign(Object.create(mc.prototype),{constructor:bc,interpolate_:function(e,t,n,r){for(var i=this.resultBuffer,o=this.sampleValues,a=this.valueSize,s=e*a,l=s-a,u=(n-t)/(r-t),c=1-u,d=0;d!==a;++d)i[d]=o[l+d]*c+o[s+d]*u;return i}}),Cc.prototype=Object.assign(Object.create(mc.prototype),{constructor:Cc,interpolate_:function(e){return this.copySampleValue_(e-1)}}),Object.assign(Ac,{toJSON:function(e){var t,n=e.constructor;if(void 0!==n.toJSON)t=n.toJSON(e);else{t={name:e.name,times:vc.convertArray(e.times,Array),values:vc.convertArray(e.values,Array)};var r=e.getInterpolation();r!==e.DefaultInterpolation&&(t.interpolation=r)}return t.type=e.ValueTypeName,t}}),Object.assign(Ac.prototype,{constructor:Ac,TimeBufferType:Float32Array,ValueBufferType:Float32Array,DefaultInterpolation:Vt,InterpolantFactoryMethodDiscrete:function(e){return new Cc(this.times,this.values,this.getValueSize(),e)},InterpolantFactoryMethodLinear:function(e){return new bc(this.times,this.values,this.getValueSize(),e)},InterpolantFactoryMethodSmooth:function(e){return new yc(this.times,this.values,this.getValueSize(),e)},setInterpolation:function(e){var t;switch(e){case Ut:t=this.InterpolantFactoryMethodDiscrete;break;case Vt:t=this.InterpolantFactoryMethodLinear;break;case Yt:t=this.InterpolantFactoryMethodSmooth}if(void 0===t){var n="unsupported interpolation for "+this.ValueTypeName+" keyframe track named "+this.name;if(void 0===this.createInterpolant){if(e===this.DefaultInterpolation)throw new Error(n);this.setInterpolation(this.DefaultInterpolation)}return console.warn("THREE.KeyframeTrack:",n),this}return this.createInterpolant=t,this},getInterpolation:function(){switch(this.createInterpolant){case this.InterpolantFactoryMethodDiscrete:return Ut;case this.InterpolantFactoryMethodLinear:return Vt;case this.InterpolantFactoryMethodSmooth:return Yt}},getValueSize:function(){return this.values.length/this.times.length},shift:function(e){if(0!==e)for(var t=this.times,n=0,r=t.length;n!==r;++n)t[n]+=e;return this},scale:function(e){if(1!==e)for(var t=this.times,n=0,r=t.length;n!==r;++n)t[n]*=e;return this},trim:function(e,t){for(var n=this.times,r=n.length,i=0,o=r-1;i!==r&&n[i]<e;)++i;for(;-1!==o&&n[o]>t;)--o;if(++o,0!==i||o!==r){i>=o&&(i=(o=Math.max(o,1))-1);var a=this.getValueSize();this.times=vc.arraySlice(n,i,o),this.values=vc.arraySlice(this.values,i*a,o*a)}return this},validate:function(){var e=!0,t=this.getValueSize();t-Math.floor(t)!=0&&(console.error("THREE.KeyframeTrack: Invalid value size in track.",this),e=!1);var n=this.times,r=this.values,i=n.length;0===i&&(console.error("THREE.KeyframeTrack: Track is empty.",this),e=!1);for(var o=null,a=0;a!==i;a++){var s=n[a];if("number"==typeof s&&isNaN(s)){console.error("THREE.KeyframeTrack: Time is not a valid number.",this,a,s),e=!1;break}if(null!==o&&o>s){console.error("THREE.KeyframeTrack: Out of order keys.",this,a,s,o),e=!1;break}o=s}if(void 0!==r&&vc.isTypedArray(r)){a=0;for(var l=r.length;a!==l;++a){var u=r[a];if(isNaN(u)){console.error("THREE.KeyframeTrack: Value is not a valid number.",this,a,u),e=!1;break}}}return e},optimize:function(){for(var e=vc.arraySlice(this.times),t=vc.arraySlice(this.values),n=this.getValueSize(),r=this.getInterpolation()===Yt,i=1,o=e.length-1,a=1;a<o;++a){var s=!1,l=e[a];if(l!==e[a+1]&&(1!==a||l!==l[0]))if(r)s=!0;else for(var u=a*n,c=u-n,d=u+n,h=0;h!==n;++h){var f=t[u+h];if(f!==t[c+h]||f!==t[d+h]){s=!0;break}}if(s){if(a!==i){e[i]=e[a];var p=a*n,g=i*n;for(h=0;h!==n;++h)t[g+h]=t[p+h]}++i}}if(o>0){e[i]=e[o];for(p=o*n,g=i*n,h=0;h!==n;++h)t[g+h]=t[p+h];++i}return i!==e.length?(this.times=vc.arraySlice(e,0,i),this.values=vc.arraySlice(t,0,i*n)):(this.times=e,this.values=t),this},clone:function(){var e=vc.arraySlice(this.times,0),t=vc.arraySlice(this.values,0),n=new(0,this.constructor)(this.name,e,t);return n.createInterpolant=this.createInterpolant,n}}),_c.prototype=Object.assign(Object.create(Ac.prototype),{constructor:_c,ValueTypeName:"bool",ValueBufferType:Array,DefaultInterpolation:Ut,InterpolantFactoryMethodLinear:void 0,InterpolantFactoryMethodSmooth:void 0}),Ic.prototype=Object.assign(Object.create(Ac.prototype),{constructor:Ic,ValueTypeName:"color"}),xc.prototype=Object.assign(Object.create(Ac.prototype),{constructor:xc,ValueTypeName:"number"}),wc.prototype=Object.assign(Object.create(mc.prototype),{constructor:wc,interpolate_:function(e,t,n,r){for(var i=this.resultBuffer,o=this.sampleValues,a=this.valueSize,s=e*a,l=(n-t)/(r-t),u=s+a;s!==u;s+=4)Qn.slerpFlat(i,0,o,s-a,o,s,l);return i}}),Mc.prototype=Object.assign(Object.create(Ac.prototype),{constructor:Mc,ValueTypeName:"quaternion",DefaultInterpolation:Vt,InterpolantFactoryMethodLinear:function(e){return new wc(this.times,this.values,this.getValueSize(),e)},InterpolantFactoryMethodSmooth:void 0}),Sc.prototype=Object.assign(Object.create(Ac.prototype),{constructor:Sc,ValueTypeName:"string",ValueBufferType:Array,DefaultInterpolation:Ut,InterpolantFactoryMethodLinear:void 0,InterpolantFactoryMethodSmooth:void 0}),Tc.prototype=Object.assign(Object.create(Ac.prototype),{constructor:Tc,ValueTypeName:"vector"}),Object.assign(Ec,{parse:function(e){for(var t=[],n=e.tracks,r=1/(e.fps||1),i=0,o=n.length;i!==o;++i)t.push(Oc(n[i]).scale(r));return new Ec(e.name,e.duration,t,e.blendMode)},toJSON:function(e){for(var t=[],n=e.tracks,r={name:e.name,duration:e.duration,tracks:t,uuid:e.uuid,blendMode:e.blendMode},i=0,o=n.length;i!==o;++i)t.push(Ac.toJSON(n[i]));return r},CreateFromMorphTargetSequence:function(e,t,n,r){for(var i=t.length,o=[],a=0;a<i;a++){var s=[],l=[];s.push((a+i-1)%i,a,(a+1)%i),l.push(0,1,0);var u=vc.getKeyframeOrder(s);s=vc.sortedArray(s,1,u),l=vc.sortedArray(l,1,u),r||0!==s[0]||(s.push(i),l.push(l[0])),o.push(new xc(".morphTargetInfluences["+t[a].name+"]",s,l).scale(1/n))}return new Ec(e,-1,o)},findByName:function(e,t){var n=e;if(!Array.isArray(e)){var r=e;n=r.geometry&&r.geometry.animations||r.animations}for(var i=0;i<n.length;i++)if(n[i].name===t)return n[i];return null},CreateClipsFromMorphTargetSequences:function(e,t,n){for(var r={},i=/^([\w-]*?)([\d]+)$/,o=0,a=e.length;o<a;o++){var s=e[o],l=s.name.match(i);if(l&&l.length>1){var u=r[d=l[1]];u||(r[d]=u=[]),u.push(s)}}var c=[];for(var d in r)c.push(Ec.CreateFromMorphTargetSequence(d,r[d],t,n));return c},parseAnimation:function(e,t){if(!e)return console.error("THREE.AnimationClip: No animation in JSONLoader data."),null;for(var n=function(e,t,n,r,i){if(0!==n.length){var o=[],a=[];vc.flattenJSON(n,o,a,r),0!==o.length&&i.push(new e(t,o,a))}},r=[],i=e.name||"default",o=e.length||-1,a=e.fps||30,s=e.blendMode,l=e.hierarchy||[],u=0;u<l.length;u++){var c=l[u].keys;if(c&&0!==c.length)if(c[0].morphTargets){for(var d={},h=0;h<c.length;h++)if(c[h].morphTargets)for(var f=0;f<c[h].morphTargets.length;f++)d[c[h].morphTargets[f]]=-1;for(var p in d){var g=[],v=[];for(f=0;f!==c[h].morphTargets.length;++f){var m=c[h];g.push(m.time),v.push(m.morphTarget===p?1:0)}r.push(new xc(".morphTargetInfluence["+p+"]",g,v))}o=d.length*(a||1)}else{var y=".bones["+t[u].name+"]";n(Tc,y+".position",c,"pos",r),n(Mc,y+".quaternion",c,"rot",r),n(Tc,y+".scale",c,"scl",r)}}return 0===r.length?null:new Ec(i,o,r,s)}}),Object.assign(Ec.prototype,{resetDuration:function(){for(var e=0,t=0,n=this.tracks.length;t!==n;++t){var r=this.tracks[t];e=Math.max(e,r.times[r.times.length-1])}return this.duration=e,this},trim:function(){for(var e=0;e<this.tracks.length;e++)this.tracks[e].trim(0,this.duration);return this},validate:function(){for(var e=!0,t=0;t<this.tracks.length;t++)e=e&&this.tracks[t].validate();return e},optimize:function(){for(var e=0;e<this.tracks.length;e++)this.tracks[e].optimize();return this},clone:function(){for(var e=[],t=0;t<this.tracks.length;t++)e.push(this.tracks[t].clone());return new Ec(this.name,this.duration,e,this.blendMode)}});var kc={enabled:!1,files:{},add:function(e,t){!1!==this.enabled&&(this.files[e]=t)},get:function(e){if(!1!==this.enabled)return this.files[e]},remove:function(e){delete this.files[e]},clear:function(){this.files={}}};function Lc(e,t,n){var r=this,i=!1,o=0,a=0,s=void 0,l=[];this.onStart=void 0,this.onLoad=e,this.onProgress=t,this.onError=n,this.itemStart=function(e){a++,!1===i&&void 0!==r.onStart&&r.onStart(e,o,a),i=!0},this.itemEnd=function(e){o++,void 0!==r.onProgress&&r.onProgress(e,o,a),o===a&&(i=!1,void 0!==r.onLoad&&r.onLoad())},this.itemError=function(e){void 0!==r.onError&&r.onError(e)},this.resolveURL=function(e){return s?s(e):e},this.setURLModifier=function(e){return s=e,this},this.addHandler=function(e,t){return l.push(e,t),this},this.removeHandler=function(e){var t=l.indexOf(e);return-1!==t&&l.splice(t,2),this},this.getHandler=function(e){for(var t=0,n=l.length;t<n;t+=2){var r=l[t],i=l[t+1];if(r.global&&(r.lastIndex=0),r.test(e))return i}return null}}var Dc=new Lc;function Pc(e){this.manager=void 0!==e?e:Dc,this.crossOrigin="anonymous",this.path="",this.resourcePath="",this.requestHeader={}}Object.assign(Pc.prototype,{load:function(){},loadAsync:function(e,t){var n=this;return new Promise((function(r,i){n.load(e,r,t,i)}))},parse:function(){},setCrossOrigin:function(e){return this.crossOrigin=e,this},setPath:function(e){return this.path=e,this},setResourcePath:function(e){return this.resourcePath=e,this},setRequestHeader:function(e){return this.requestHeader=e,this}});var Nc={};function jc(e){Pc.call(this,e)}function Rc(e){Pc.call(this,e)}function zc(e){Pc.call(this,e)}function Fc(e){Pc.call(this,e)}function Bc(e){Pc.call(this,e)}function Hc(e){Pc.call(this,e)}function Uc(e){Pc.call(this,e)}function Vc(){this.type="Curve",this.arcLengthDivisions=200}function Yc(e,t,n,r,i,o,a,s){Vc.call(this),this.type="EllipseCurve",this.aX=e||0,this.aY=t||0,this.xRadius=n||1,this.yRadius=r||1,this.aStartAngle=i||0,this.aEndAngle=o||2*Math.PI,this.aClockwise=a||!1,this.aRotation=s||0}function Wc(e,t,n,r,i,o){Yc.call(this,e,t,n,n,r,i,o),this.type="ArcCurve"}function Gc(){var e=0,t=0,n=0,r=0;function i(i,o,a,s){e=i,t=a,n=-3*i+3*o-2*a-s,r=2*i-2*o+a+s}return{initCatmullRom:function(e,t,n,r,o){i(t,n,o*(n-e),o*(r-t))},initNonuniformCatmullRom:function(e,t,n,r,o,a,s){var l=(t-e)/o-(n-e)/(o+a)+(n-t)/a,u=(n-t)/a-(r-t)/(a+s)+(r-n)/s;i(t,n,l*=a,u*=a)},calc:function(i){var o=i*i;return e+t*i+n*o+r*(o*i)}}}jc.prototype=Object.assign(Object.create(Pc.prototype),{constructor:jc,load:function(e,t,n,r){void 0===e&&(e=""),void 0!==this.path&&(e=this.path+e),e=this.manager.resolveURL(e);var i=this,o=kc.get(e);if(void 0!==o)return i.manager.itemStart(e),setTimeout((function(){t&&t(o),i.manager.itemEnd(e)}),0),o;if(void 0===Nc[e]){var a=e.match(/^data:(.*?)(;base64)?,(.*)$/);if(a){var s=a[1],l=!!a[2],u=a[3];u=decodeURIComponent(u),l&&(u=atob(u));try{var c,d=(this.responseType||"").toLowerCase();switch(d){case"arraybuffer":case"blob":for(var h=new Uint8Array(u.length),f=0;f<u.length;f++)h[f]=u.charCodeAt(f);c="blob"===d?new Blob([h.buffer],{type:s}):h.buffer;break;case"document":var p=new DOMParser;c=p.parseFromString(u,s);break;case"json":c=JSON.parse(u);break;default:c=u}setTimeout((function(){t&&t(c),i.manager.itemEnd(e)}),0)}catch(t){setTimeout((function(){r&&r(t),i.manager.itemError(e),i.manager.itemEnd(e)}),0)}}else{Nc[e]=[],Nc[e].push({onLoad:t,onProgress:n,onError:r});var g=new XMLHttpRequest;for(var v in g.open("GET",e,!0),g.addEventListener("load",(function(t){var n=this.response,r=Nc[e];if(delete Nc[e],200===this.status||0===this.status){0===this.status&&console.warn("THREE.FileLoader: HTTP Status 0 received."),kc.add(e,n);for(var o=0,a=r.length;o<a;o++){(s=r[o]).onLoad&&s.onLoad(n)}i.manager.itemEnd(e)}else{for(o=0,a=r.length;o<a;o++){var s;(s=r[o]).onError&&s.onError(t)}i.manager.itemError(e),i.manager.itemEnd(e)}}),!1),g.addEventListener("progress",(function(t){for(var n=Nc[e],r=0,i=n.length;r<i;r++){var o=n[r];o.onProgress&&o.onProgress(t)}}),!1),g.addEventListener("error",(function(t){var n=Nc[e];delete Nc[e];for(var r=0,o=n.length;r<o;r++){var a=n[r];a.onError&&a.onError(t)}i.manager.itemError(e),i.manager.itemEnd(e)}),!1),g.addEventListener("abort",(function(t){var n=Nc[e];delete Nc[e];for(var r=0,o=n.length;r<o;r++){var a=n[r];a.onError&&a.onError(t)}i.manager.itemError(e),i.manager.itemEnd(e)}),!1),void 0!==this.responseType&&(g.responseType=this.responseType),void 0!==this.withCredentials&&(g.withCredentials=this.withCredentials),g.overrideMimeType&&g.overrideMimeType(void 0!==this.mimeType?this.mimeType:"text/plain"),this.requestHeader)g.setRequestHeader(v,this.requestHeader[v]);g.send(null)}return i.manager.itemStart(e),g}Nc[e].push({onLoad:t,onProgress:n,onError:r})},setResponseType:function(e){return this.responseType=e,this},setWithCredentials:function(e){return this.withCredentials=e,this},setMimeType:function(e){return this.mimeType=e,this}}),Rc.prototype=Object.assign(Object.create(Pc.prototype),{constructor:Rc,load:function(e,t,n,r){var i=this,o=new jc(i.manager);o.setPath(i.path),o.load(e,(function(n){try{t(i.parse(JSON.parse(n)))}catch(t){r?r(t):console.error(t),i.manager.itemError(e)}}),n,r)},parse:function(e){for(var t=[],n=0;n<e.length;n++){var r=Ec.parse(e[n]);t.push(r)}return t}}),zc.prototype=Object.assign(Object.create(Pc.prototype),{constructor:zc,load:function(e,t,n,r){var i=this,o=[],a=new Bl;a.image=o;var s=new jc(this.manager);function l(l){s.load(e[l],(function(e){var n=i.parse(e,!0);o[l]={width:n.width,height:n.height,format:n.format,mipmaps:n.mipmaps},6===(u+=1)&&(1===n.mipmapCount&&(a.minFilter=_e),a.format=n.format,a.needsUpdate=!0,t&&t(a))}),n,r)}if(s.setPath(this.path),s.setResponseType("arraybuffer"),Array.isArray(e))for(var u=0,c=0,d=e.length;c<d;++c)l(c);else s.load(e,(function(e){var n=i.parse(e,!0);if(n.isCubemap)for(var r=n.mipmaps.length/n.mipmapCount,s=0;s<r;s++){o[s]={mipmaps:[]};for(var l=0;l<n.mipmapCount;l++)o[s].mipmaps.push(n.mipmaps[s*n.mipmapCount+l]),o[s].format=n.format,o[s].width=n.width,o[s].height=n.height}else a.image.width=n.width,a.image.height=n.height,a.mipmaps=n.mipmaps;1===n.mipmapCount&&(a.minFilter=_e),a.format=n.format,a.needsUpdate=!0,t&&t(a)}),n,r);return a}}),Fc.prototype=Object.assign(Object.create(Pc.prototype),{constructor:Fc,load:function(e,t,n,r){var i=this,o=new So,a=new jc(this.manager);return a.setResponseType("arraybuffer"),a.setPath(this.path),a.load(e,(function(e){var n=i.parse(e);n&&(void 0!==n.image?o.image=n.image:void 0!==n.data&&(o.image.width=n.width,o.image.height=n.height,o.image.data=n.data),o.wrapS=void 0!==n.wrapS?n.wrapS:ge,o.wrapT=void 0!==n.wrapT?n.wrapT:ge,o.magFilter=void 0!==n.magFilter?n.magFilter:_e,o.minFilter=void 0!==n.minFilter?n.minFilter:_e,o.anisotropy=void 0!==n.anisotropy?n.anisotropy:1,void 0!==n.format&&(o.format=n.format),void 0!==n.type&&(o.type=n.type),void 0!==n.mipmaps&&(o.mipmaps=n.mipmaps,o.minFilter=we),1===n.mipmapCount&&(o.minFilter=_e),o.needsUpdate=!0,t&&t(o,n))}),n,r),o}}),Bc.prototype=Object.assign(Object.create(Pc.prototype),{constructor:Bc,load:function(e,t,n,r){void 0!==this.path&&(e=this.path+e),e=this.manager.resolveURL(e);var i=this,o=kc.get(e);if(void 0!==o)return i.manager.itemStart(e),setTimeout((function(){t&&t(o),i.manager.itemEnd(e)}),0),o;var a=document.createElementNS("http://www.w3.org/1999/xhtml","img");function s(){a.removeEventListener("load",s,!1),a.removeEventListener("error",l,!1),kc.add(e,this),t&&t(this),i.manager.itemEnd(e)}function l(t){a.removeEventListener("load",s,!1),a.removeEventListener("error",l,!1),r&&r(t),i.manager.itemError(e),i.manager.itemEnd(e)}return a.addEventListener("load",s,!1),a.addEventListener("error",l,!1),"data:"!==e.substr(0,5)&&void 0!==this.crossOrigin&&(a.crossOrigin=this.crossOrigin),i.manager.itemStart(e),a.src=e,a}}),Hc.prototype=Object.assign(Object.create(Pc.prototype),{constructor:Hc,load:function(e,t,n,r){var i=new Zo,o=new Bc(this.manager);o.setCrossOrigin(this.crossOrigin),o.setPath(this.path);var a=0;function s(n){o.load(e[n],(function(e){i.images[n]=e,6===++a&&(i.needsUpdate=!0,t&&t(i))}),void 0,r)}for(var l=0;l<e.length;++l)s(l);return i}}),Uc.prototype=Object.assign(Object.create(Pc.prototype),{constructor:Uc,load:function(e,t,n,r){var i=new Gn,o=new Bc(this.manager);return o.setCrossOrigin(this.crossOrigin),o.setPath(this.path),o.load(e,(function(n){i.image=n;var r=e.search(/\.jpe?g($|\?)/i)>0||0===e.search(/^data\:image\/jpeg/);i.format=r?Be:He,i.needsUpdate=!0,void 0!==t&&t(i)}),n,r),i}}),Object.assign(Vc.prototype,{getPoint:function(){return console.warn("THREE.Curve: .getPoint() not implemented."),null},getPointAt:function(e,t){var n=this.getUtoTmapping(e);return this.getPoint(n,t)},getPoints:function(e){void 0===e&&(e=5);for(var t=[],n=0;n<=e;n++)t.push(this.getPoint(n/e));return t},getSpacedPoints:function(e){void 0===e&&(e=5);for(var t=[],n=0;n<=e;n++)t.push(this.getPointAt(n/e));return t},getLength:function(){var e=this.getLengths();return e[e.length-1]},getLengths:function(e){if(void 0===e&&(e=this.arcLengthDivisions),this.cacheArcLengths&&this.cacheArcLengths.length===e+1&&!this.needsUpdate)return this.cacheArcLengths;this.needsUpdate=!1;var t,n,r=[],i=this.getPoint(0),o=0;for(r.push(0),n=1;n<=e;n++)o+=(t=this.getPoint(n/e)).distanceTo(i),r.push(o),i=t;return this.cacheArcLengths=r,r},updateArcLengths:function(){this.needsUpdate=!0,this.getLengths()},getUtoTmapping:function(e,t){var n,r=this.getLengths(),i=0,o=r.length;n=t||e*r[o-1];for(var a,s=0,l=o-1;s<=l;)if((a=r[i=Math.floor(s+(l-s)/2)]-n)<0)s=i+1;else{if(!(a>0)){l=i;break}l=i-1}if(r[i=l]===n)return i/(o-1);var u=r[i];return(i+(n-u)/(r[i+1]-u))/(o-1)},getTangent:function(e,t){var n=e-1e-4,r=e+1e-4;n<0&&(n=0),r>1&&(r=1);var i=this.getPoint(n),o=this.getPoint(r),a=t||(i.isVector2?new Un:new $n);return a.copy(o).sub(i).normalize(),a},getTangentAt:function(e,t){var n=this.getUtoTmapping(e);return this.getTangent(n,t)},computeFrenetFrames:function(e,t){var n,r,i,o=new $n,a=[],s=[],l=[],u=new $n,c=new sr;for(n=0;n<=e;n++)r=n/e,a[n]=this.getTangentAt(r,new $n),a[n].normalize();s[0]=new $n,l[0]=new $n;var d=Number.MAX_VALUE,h=Math.abs(a[0].x),f=Math.abs(a[0].y),p=Math.abs(a[0].z);for(h<=d&&(d=h,o.set(1,0,0)),f<=d&&(d=f,o.set(0,1,0)),p<=d&&o.set(0,0,1),u.crossVectors(a[0],o).normalize(),s[0].crossVectors(a[0],u),l[0].crossVectors(a[0],s[0]),n=1;n<=e;n++)s[n]=s[n-1].clone(),l[n]=l[n-1].clone(),u.crossVectors(a[n-1],a[n]),u.length()>Number.EPSILON&&(u.normalize(),i=Math.acos(Hn.clamp(a[n-1].dot(a[n]),-1,1)),s[n].applyMatrix4(c.makeRotationAxis(u,i))),l[n].crossVectors(a[n],s[n]);if(!0===t)for(i=Math.acos(Hn.clamp(s[0].dot(s[e]),-1,1)),i/=e,a[0].dot(u.crossVectors(s[0],s[e]))>0&&(i=-i),n=1;n<=e;n++)s[n].applyMatrix4(c.makeRotationAxis(a[n],i*n)),l[n].crossVectors(a[n],s[n]);return{tangents:a,normals:s,binormals:l}},clone:function(){return(new this.constructor).copy(this)},copy:function(e){return this.arcLengthDivisions=e.arcLengthDivisions,this},toJSON:function(){var e={metadata:{version:4.5,type:"Curve",generator:"Curve.toJSON"}};return e.arcLengthDivisions=this.arcLengthDivisions,e.type=this.type,e},fromJSON:function(e){return this.arcLengthDivisions=e.arcLengthDivisions,this}}),Yc.prototype=Object.create(Vc.prototype),Yc.prototype.constructor=Yc,Yc.prototype.isEllipseCurve=!0,Yc.prototype.getPoint=function(e,t){for(var n=t||new Un,r=2*Math.PI,i=this.aEndAngle-this.aStartAngle,o=Math.abs(i)<Number.EPSILON;i<0;)i+=r;for(;i>r;)i-=r;i<Number.EPSILON&&(i=o?0:r),!0!==this.aClockwise||o||(i===r?i=-r:i-=r);var a=this.aStartAngle+e*i,s=this.aX+this.xRadius*Math.cos(a),l=this.aY+this.yRadius*Math.sin(a);if(0!==this.aRotation){var u=Math.cos(this.aRotation),c=Math.sin(this.aRotation),d=s-this.aX,h=l-this.aY;s=d*u-h*c+this.aX,l=d*c+h*u+this.aY}return n.set(s,l)},Yc.prototype.copy=function(e){return Vc.prototype.copy.call(this,e),this.aX=e.aX,this.aY=e.aY,this.xRadius=e.xRadius,this.yRadius=e.yRadius,this.aStartAngle=e.aStartAngle,this.aEndAngle=e.aEndAngle,this.aClockwise=e.aClockwise,this.aRotation=e.aRotation,this},Yc.prototype.toJSON=function(){var e=Vc.prototype.toJSON.call(this);return e.aX=this.aX,e.aY=this.aY,e.xRadius=this.xRadius,e.yRadius=this.yRadius,e.aStartAngle=this.aStartAngle,e.aEndAngle=this.aEndAngle,e.aClockwise=this.aClockwise,e.aRotation=this.aRotation,e},Yc.prototype.fromJSON=function(e){return Vc.prototype.fromJSON.call(this,e),this.aX=e.aX,this.aY=e.aY,this.xRadius=e.xRadius,this.yRadius=e.yRadius,this.aStartAngle=e.aStartAngle,this.aEndAngle=e.aEndAngle,this.aClockwise=e.aClockwise,this.aRotation=e.aRotation,this},Wc.prototype=Object.create(Yc.prototype),Wc.prototype.constructor=Wc,Wc.prototype.isArcCurve=!0;var Xc=new $n,qc=new Gc,Zc=new Gc,Qc=new Gc;function Kc(e,t,n,r){Vc.call(this),this.type="CatmullRomCurve3",this.points=e||[],this.closed=t||!1,this.curveType=n||"centripetal",this.tension=r||.5}function Jc(e,t,n,r,i){var o=.5*(r-t),a=.5*(i-n),s=e*e;return(2*n-2*r+o+a)*(e*s)+(-3*n+3*r-2*o-a)*s+o*e+n}function $c(e,t,n,r){return function(e,t){var n=1-e;return n*n*t}(e,t)+function(e,t){return 2*(1-e)*e*t}(e,n)+function(e,t){return e*e*t}(e,r)}function ed(e,t,n,r,i){return function(e,t){var n=1-e;return n*n*n*t}(e,t)+function(e,t){var n=1-e;return 3*n*n*e*t}(e,n)+function(e,t){return 3*(1-e)*e*e*t}(e,r)+function(e,t){return e*e*e*t}(e,i)}function td(e,t,n,r){Vc.call(this),this.type="CubicBezierCurve",this.v0=e||new Un,this.v1=t||new Un,this.v2=n||new Un,this.v3=r||new Un}function nd(e,t,n,r){Vc.call(this),this.type="CubicBezierCurve3",this.v0=e||new $n,this.v1=t||new $n,this.v2=n||new $n,this.v3=r||new $n}function rd(e,t){Vc.call(this),this.type="LineCurve",this.v1=e||new Un,this.v2=t||new Un}function id(e,t){Vc.call(this),this.type="LineCurve3",this.v1=e||new $n,this.v2=t||new $n}function od(e,t,n){Vc.call(this),this.type="QuadraticBezierCurve",this.v0=e||new Un,this.v1=t||new Un,this.v2=n||new Un}function ad(e,t,n){Vc.call(this),this.type="QuadraticBezierCurve3",this.v0=e||new $n,this.v1=t||new $n,this.v2=n||new $n}function sd(e){Vc.call(this),this.type="SplineCurve",this.points=e||[]}Kc.prototype=Object.create(Vc.prototype),Kc.prototype.constructor=Kc,Kc.prototype.isCatmullRomCurve3=!0,Kc.prototype.getPoint=function(e,t){var n,r,i,o,a=t||new $n,s=this.points,l=s.length,u=(l-(this.closed?0:1))*e,c=Math.floor(u),d=u-c;if(this.closed?c+=c>0?0:(Math.floor(Math.abs(c)/l)+1)*l:0===d&&c===l-1&&(c=l-2,d=1),this.closed||c>0?n=s[(c-1)%l]:(Xc.subVectors(s[0],s[1]).add(s[0]),n=Xc),r=s[c%l],i=s[(c+1)%l],this.closed||c+2<l?o=s[(c+2)%l]:(Xc.subVectors(s[l-1],s[l-2]).add(s[l-1]),o=Xc),"centripetal"===this.curveType||"chordal"===this.curveType){var h="chordal"===this.curveType?.5:.25,f=Math.pow(n.distanceToSquared(r),h),p=Math.pow(r.distanceToSquared(i),h),g=Math.pow(i.distanceToSquared(o),h);p<1e-4&&(p=1),f<1e-4&&(f=p),g<1e-4&&(g=p),qc.initNonuniformCatmullRom(n.x,r.x,i.x,o.x,f,p,g),Zc.initNonuniformCatmullRom(n.y,r.y,i.y,o.y,f,p,g),Qc.initNonuniformCatmullRom(n.z,r.z,i.z,o.z,f,p,g)}else"catmullrom"===this.curveType&&(qc.initCatmullRom(n.x,r.x,i.x,o.x,this.tension),Zc.initCatmullRom(n.y,r.y,i.y,o.y,this.tension),Qc.initCatmullRom(n.z,r.z,i.z,o.z,this.tension));return a.set(qc.calc(d),Zc.calc(d),Qc.calc(d)),a},Kc.prototype.copy=function(e){Vc.prototype.copy.call(this,e),this.points=[];for(var t=0,n=e.points.length;t<n;t++){var r=e.points[t];this.points.push(r.clone())}return this.closed=e.closed,this.curveType=e.curveType,this.tension=e.tension,this},Kc.prototype.toJSON=function(){var e=Vc.prototype.toJSON.call(this);e.points=[];for(var t=0,n=this.points.length;t<n;t++){var r=this.points[t];e.points.push(r.toArray())}return e.closed=this.closed,e.curveType=this.curveType,e.tension=this.tension,e},Kc.prototype.fromJSON=function(e){Vc.prototype.fromJSON.call(this,e),this.points=[];for(var t=0,n=e.points.length;t<n;t++){var r=e.points[t];this.points.push((new $n).fromArray(r))}return this.closed=e.closed,this.curveType=e.curveType,this.tension=e.tension,this},td.prototype=Object.create(Vc.prototype),td.prototype.constructor=td,td.prototype.isCubicBezierCurve=!0,td.prototype.getPoint=function(e,t){var n=t||new Un,r=this.v0,i=this.v1,o=this.v2,a=this.v3;return n.set(ed(e,r.x,i.x,o.x,a.x),ed(e,r.y,i.y,o.y,a.y)),n},td.prototype.copy=function(e){return Vc.prototype.copy.call(this,e),this.v0.copy(e.v0),this.v1.copy(e.v1),this.v2.copy(e.v2),this.v3.copy(e.v3),this},td.prototype.toJSON=function(){var e=Vc.prototype.toJSON.call(this);return e.v0=this.v0.toArray(),e.v1=this.v1.toArray(),e.v2=this.v2.toArray(),e.v3=this.v3.toArray(),e},td.prototype.fromJSON=function(e){return Vc.prototype.fromJSON.call(this,e),this.v0.fromArray(e.v0),this.v1.fromArray(e.v1),this.v2.fromArray(e.v2),this.v3.fromArray(e.v3),this},nd.prototype=Object.create(Vc.prototype),nd.prototype.constructor=nd,nd.prototype.isCubicBezierCurve3=!0,nd.prototype.getPoint=function(e,t){var n=t||new $n,r=this.v0,i=this.v1,o=this.v2,a=this.v3;return n.set(ed(e,r.x,i.x,o.x,a.x),ed(e,r.y,i.y,o.y,a.y),ed(e,r.z,i.z,o.z,a.z)),n},nd.prototype.copy=function(e){return Vc.prototype.copy.call(this,e),this.v0.copy(e.v0),this.v1.copy(e.v1),this.v2.copy(e.v2),this.v3.copy(e.v3),this},nd.prototype.toJSON=function(){var e=Vc.prototype.toJSON.call(this);return e.v0=this.v0.toArray(),e.v1=this.v1.toArray(),e.v2=this.v2.toArray(),e.v3=this.v3.toArray(),e},nd.prototype.fromJSON=function(e){return Vc.prototype.fromJSON.call(this,e),this.v0.fromArray(e.v0),this.v1.fromArray(e.v1),this.v2.fromArray(e.v2),this.v3.fromArray(e.v3),this},rd.prototype=Object.create(Vc.prototype),rd.prototype.constructor=rd,rd.prototype.isLineCurve=!0,rd.prototype.getPoint=function(e,t){var n=t||new Un;return 1===e?n.copy(this.v2):(n.copy(this.v2).sub(this.v1),n.multiplyScalar(e).add(this.v1)),n},rd.prototype.getPointAt=function(e,t){return this.getPoint(e,t)},rd.prototype.getTangent=function(e,t){return(t||new Un).copy(this.v2).sub(this.v1).normalize()},rd.prototype.copy=function(e){return Vc.prototype.copy.call(this,e),this.v1.copy(e.v1),this.v2.copy(e.v2),this},rd.prototype.toJSON=function(){var e=Vc.prototype.toJSON.call(this);return e.v1=this.v1.toArray(),e.v2=this.v2.toArray(),e},rd.prototype.fromJSON=function(e){return Vc.prototype.fromJSON.call(this,e),this.v1.fromArray(e.v1),this.v2.fromArray(e.v2),this},id.prototype=Object.create(Vc.prototype),id.prototype.constructor=id,id.prototype.isLineCurve3=!0,id.prototype.getPoint=function(e,t){var n=t||new $n;return 1===e?n.copy(this.v2):(n.copy(this.v2).sub(this.v1),n.multiplyScalar(e).add(this.v1)),n},id.prototype.getPointAt=function(e,t){return this.getPoint(e,t)},id.prototype.copy=function(e){return Vc.prototype.copy.call(this,e),this.v1.copy(e.v1),this.v2.copy(e.v2),this},id.prototype.toJSON=function(){var e=Vc.prototype.toJSON.call(this);return e.v1=this.v1.toArray(),e.v2=this.v2.toArray(),e},id.prototype.fromJSON=function(e){return Vc.prototype.fromJSON.call(this,e),this.v1.fromArray(e.v1),this.v2.fromArray(e.v2),this},od.prototype=Object.create(Vc.prototype),od.prototype.constructor=od,od.prototype.isQuadraticBezierCurve=!0,od.prototype.getPoint=function(e,t){var n=t||new Un,r=this.v0,i=this.v1,o=this.v2;return n.set($c(e,r.x,i.x,o.x),$c(e,r.y,i.y,o.y)),n},od.prototype.copy=function(e){return Vc.prototype.copy.call(this,e),this.v0.copy(e.v0),this.v1.copy(e.v1),this.v2.copy(e.v2),this},od.prototype.toJSON=function(){var e=Vc.prototype.toJSON.call(this);return e.v0=this.v0.toArray(),e.v1=this.v1.toArray(),e.v2=this.v2.toArray(),e},od.prototype.fromJSON=function(e){return Vc.prototype.fromJSON.call(this,e),this.v0.fromArray(e.v0),this.v1.fromArray(e.v1),this.v2.fromArray(e.v2),this},ad.prototype=Object.create(Vc.prototype),ad.prototype.constructor=ad,ad.prototype.isQuadraticBezierCurve3=!0,ad.prototype.getPoint=function(e,t){var n=t||new $n,r=this.v0,i=this.v1,o=this.v2;return n.set($c(e,r.x,i.x,o.x),$c(e,r.y,i.y,o.y),$c(e,r.z,i.z,o.z)),n},ad.prototype.copy=function(e){return Vc.prototype.copy.call(this,e),this.v0.copy(e.v0),this.v1.copy(e.v1),this.v2.copy(e.v2),this},ad.prototype.toJSON=function(){var e=Vc.prototype.toJSON.call(this);return e.v0=this.v0.toArray(),e.v1=this.v1.toArray(),e.v2=this.v2.toArray(),e},ad.prototype.fromJSON=function(e){return Vc.prototype.fromJSON.call(this,e),this.v0.fromArray(e.v0),this.v1.fromArray(e.v1),this.v2.fromArray(e.v2),this},sd.prototype=Object.create(Vc.prototype),sd.prototype.constructor=sd,sd.prototype.isSplineCurve=!0,sd.prototype.getPoint=function(e,t){var n=t||new Un,r=this.points,i=(r.length-1)*e,o=Math.floor(i),a=i-o,s=r[0===o?o:o-1],l=r[o],u=r[o>r.length-2?r.length-1:o+1],c=r[o>r.length-3?r.length-1:o+2];return n.set(Jc(a,s.x,l.x,u.x,c.x),Jc(a,s.y,l.y,u.y,c.y)),n},sd.prototype.copy=function(e){Vc.prototype.copy.call(this,e),this.points=[];for(var t=0,n=e.points.length;t<n;t++){var r=e.points[t];this.points.push(r.clone())}return this},sd.prototype.toJSON=function(){var e=Vc.prototype.toJSON.call(this);e.points=[];for(var t=0,n=this.points.length;t<n;t++){var r=this.points[t];e.points.push(r.toArray())}return e},sd.prototype.fromJSON=function(e){Vc.prototype.fromJSON.call(this,e),this.points=[];for(var t=0,n=e.points.length;t<n;t++){var r=e.points[t];this.points.push((new Un).fromArray(r))}return this};var ld=Object.freeze({__proto__:null,ArcCurve:Wc,CatmullRomCurve3:Kc,CubicBezierCurve:td,CubicBezierCurve3:nd,EllipseCurve:Yc,LineCurve:rd,LineCurve3:id,QuadraticBezierCurve:od,QuadraticBezierCurve3:ad,SplineCurve:sd});function ud(){Vc.call(this),this.type="CurvePath",this.curves=[],this.autoClose=!1}function cd(e){ud.call(this),this.type="Path",this.currentPoint=new Un,e&&this.setFromPoints(e)}function dd(e){cd.call(this,e),this.uuid=Hn.generateUUID(),this.type="Shape",this.holes=[]}function hd(e,t){wr.call(this),this.type="Light",this.color=new vi(e),this.intensity=void 0!==t?t:1,this.receiveShadow=void 0}function fd(e,t,n){hd.call(this,e,n),this.type="HemisphereLight",this.castShadow=void 0,this.position.copy(wr.DefaultUp),this.updateMatrix(),this.groundColor=new vi(t)}function pd(e){this.camera=e,this.bias=0,this.radius=1,this.mapSize=new Un(512,512),this.map=null,this.mapPass=null,this.matrix=new sr,this._frustum=new Oo,this._frameExtents=new Un(1,1),this._viewportCount=1,this._viewports=[new Xn(0,0,1,1)]}function gd(){pd.call(this,new xo(50,1,.5,500))}function vd(e,t,n,r,i,o){hd.call(this,e,t),this.type="SpotLight",this.position.copy(wr.DefaultUp),this.updateMatrix(),this.target=new wr,Object.defineProperty(this,"power",{get:function(){return this.intensity*Math.PI},set:function(e){this.intensity=e/Math.PI}}),this.distance=void 0!==n?n:0,this.angle=void 0!==r?r:Math.PI/3,this.penumbra=void 0!==i?i:0,this.decay=void 0!==o?o:1,this.shadow=new gd}function md(){pd.call(this,new xo(90,1,.5,500)),this._frameExtents=new Un(4,2),this._viewportCount=6,this._viewports=[new Xn(2,1,1,1),new Xn(0,1,1,1),new Xn(3,1,1,1),new Xn(1,1,1,1),new Xn(3,0,1,1),new Xn(1,0,1,1)],this._cubeDirections=[new $n(1,0,0),new $n(-1,0,0),new $n(0,0,1),new $n(0,0,-1),new $n(0,1,0),new $n(0,-1,0)],this._cubeUps=[new $n(0,1,0),new $n(0,1,0),new $n(0,1,0),new $n(0,1,0),new $n(0,0,1),new $n(0,0,-1)]}function yd(e,t,n,r){hd.call(this,e,t),this.type="PointLight",Object.defineProperty(this,"power",{get:function(){return 4*this.intensity*Math.PI},set:function(e){this.intensity=e/(4*Math.PI)}}),this.distance=void 0!==n?n:0,this.decay=void 0!==r?r:1,this.shadow=new md}function bd(e,t,n,r,i,o){Io.call(this),this.type="OrthographicCamera",this.zoom=1,this.view=null,this.left=void 0!==e?e:-1,this.right=void 0!==t?t:1,this.top=void 0!==n?n:1,this.bottom=void 0!==r?r:-1,this.near=void 0!==i?i:.1,this.far=void 0!==o?o:2e3,this.updateProjectionMatrix()}function Cd(){pd.call(this,new bd(-5,5,5,-5,.5,500))}function Ad(e,t){hd.call(this,e,t),this.type="DirectionalLight",this.position.copy(wr.DefaultUp),this.updateMatrix(),this.target=new wr,this.shadow=new Cd}function _d(e,t){hd.call(this,e,t),this.type="AmbientLight",this.castShadow=void 0}function Id(e,t,n,r){hd.call(this,e,t),this.type="RectAreaLight",this.width=void 0!==n?n:10,this.height=void 0!==r?r:10}function xd(){this.coefficients=[];for(var e=0;e<9;e++)this.coefficients.push(new $n)}function wd(e,t){hd.call(this,void 0,t),this.type="LightProbe",this.sh=void 0!==e?e:new xd}function Md(e){Pc.call(this,e),this.textures={}}ud.prototype=Object.assign(Object.create(Vc.prototype),{constructor:ud,add:function(e){this.curves.push(e)},closePath:function(){var e=this.curves[0].getPoint(0),t=this.curves[this.curves.length-1].getPoint(1);e.equals(t)||this.curves.push(new rd(t,e))},getPoint:function(e){for(var t=e*this.getLength(),n=this.getCurveLengths(),r=0;r<n.length;){if(n[r]>=t){var i=n[r]-t,o=this.curves[r],a=o.getLength(),s=0===a?0:1-i/a;return o.getPointAt(s)}r++}return null},getLength:function(){var e=this.getCurveLengths();return e[e.length-1]},updateArcLengths:function(){this.needsUpdate=!0,this.cacheLengths=null,this.getCurveLengths()},getCurveLengths:function(){if(this.cacheLengths&&this.cacheLengths.length===this.curves.length)return this.cacheLengths;for(var e=[],t=0,n=0,r=this.curves.length;n<r;n++)t+=this.curves[n].getLength(),e.push(t);return this.cacheLengths=e,e},getSpacedPoints:function(e){void 0===e&&(e=40);for(var t=[],n=0;n<=e;n++)t.push(this.getPoint(n/e));return this.autoClose&&t.push(t[0]),t},getPoints:function(e){e=e||12;for(var t,n=[],r=0,i=this.curves;r<i.length;r++)for(var o=i[r],a=o&&o.isEllipseCurve?2*e:o&&(o.isLineCurve||o.isLineCurve3)?1:o&&o.isSplineCurve?e*o.points.length:e,s=o.getPoints(a),l=0;l<s.length;l++){var u=s[l];t&&t.equals(u)||(n.push(u),t=u)}return this.autoClose&&n.length>1&&!n[n.length-1].equals(n[0])&&n.push(n[0]),n},copy:function(e){Vc.prototype.copy.call(this,e),this.curves=[];for(var t=0,n=e.curves.length;t<n;t++){var r=e.curves[t];this.curves.push(r.clone())}return this.autoClose=e.autoClose,this},toJSON:function(){var e=Vc.prototype.toJSON.call(this);e.autoClose=this.autoClose,e.curves=[];for(var t=0,n=this.curves.length;t<n;t++){var r=this.curves[t];e.curves.push(r.toJSON())}return e},fromJSON:function(e){Vc.prototype.fromJSON.call(this,e),this.autoClose=e.autoClose,this.curves=[];for(var t=0,n=e.curves.length;t<n;t++){var r=e.curves[t];this.curves.push((new ld[r.type]).fromJSON(r))}return this}}),cd.prototype=Object.assign(Object.create(ud.prototype),{constructor:cd,setFromPoints:function(e){this.moveTo(e[0].x,e[0].y);for(var t=1,n=e.length;t<n;t++)this.lineTo(e[t].x,e[t].y);return this},moveTo:function(e,t){return this.currentPoint.set(e,t),this},lineTo:function(e,t){var n=new rd(this.currentPoint.clone(),new Un(e,t));return this.curves.push(n),this.currentPoint.set(e,t),this},quadraticCurveTo:function(e,t,n,r){var i=new od(this.currentPoint.clone(),new Un(e,t),new Un(n,r));return this.curves.push(i),this.currentPoint.set(n,r),this},bezierCurveTo:function(e,t,n,r,i,o){var a=new td(this.currentPoint.clone(),new Un(e,t),new Un(n,r),new Un(i,o));return this.curves.push(a),this.currentPoint.set(i,o),this},splineThru:function(e){var t=new sd([this.currentPoint.clone()].concat(e));return this.curves.push(t),this.currentPoint.copy(e[e.length-1]),this},arc:function(e,t,n,r,i,o){var a=this.currentPoint.x,s=this.currentPoint.y;return this.absarc(e+a,t+s,n,r,i,o),this},absarc:function(e,t,n,r,i,o){return this.absellipse(e,t,n,n,r,i,o),this},ellipse:function(e,t,n,r,i,o,a,s){var l=this.currentPoint.x,u=this.currentPoint.y;return this.absellipse(e+l,t+u,n,r,i,o,a,s),this},absellipse:function(e,t,n,r,i,o,a,s){var l=new Yc(e,t,n,r,i,o,a,s);if(this.curves.length>0){var u=l.getPoint(0);u.equals(this.currentPoint)||this.lineTo(u.x,u.y)}this.curves.push(l);var c=l.getPoint(1);return this.currentPoint.copy(c),this},copy:function(e){return ud.prototype.copy.call(this,e),this.currentPoint.copy(e.currentPoint),this},toJSON:function(){var e=ud.prototype.toJSON.call(this);return e.currentPoint=this.currentPoint.toArray(),e},fromJSON:function(e){return ud.prototype.fromJSON.call(this,e),this.currentPoint.fromArray(e.currentPoint),this}}),dd.prototype=Object.assign(Object.create(cd.prototype),{constructor:dd,getPointsHoles:function(e){for(var t=[],n=0,r=this.holes.length;n<r;n++)t[n]=this.holes[n].getPoints(e);return t},extractPoints:function(e){return{shape:this.getPoints(e),holes:this.getPointsHoles(e)}},copy:function(e){cd.prototype.copy.call(this,e),this.holes=[];for(var t=0,n=e.holes.length;t<n;t++){var r=e.holes[t];this.holes.push(r.clone())}return this},toJSON:function(){var e=cd.prototype.toJSON.call(this);e.uuid=this.uuid,e.holes=[];for(var t=0,n=this.holes.length;t<n;t++){var r=this.holes[t];e.holes.push(r.toJSON())}return e},fromJSON:function(e){cd.prototype.fromJSON.call(this,e),this.uuid=e.uuid,this.holes=[];for(var t=0,n=e.holes.length;t<n;t++){var r=e.holes[t];this.holes.push((new cd).fromJSON(r))}return this}}),hd.prototype=Object.assign(Object.create(wr.prototype),{constructor:hd,isLight:!0,copy:function(e){return wr.prototype.copy.call(this,e),this.color.copy(e.color),this.intensity=e.intensity,this},toJSON:function(e){var t=wr.prototype.toJSON.call(this,e);return t.object.color=this.color.getHex(),t.object.intensity=this.intensity,void 0!==this.groundColor&&(t.object.groundColor=this.groundColor.getHex()),void 0!==this.distance&&(t.object.distance=this.distance),void 0!==this.angle&&(t.object.angle=this.angle),void 0!==this.decay&&(t.object.decay=this.decay),void 0!==this.penumbra&&(t.object.penumbra=this.penumbra),void 0!==this.shadow&&(t.object.shadow=this.shadow.toJSON()),t}}),fd.prototype=Object.assign(Object.create(hd.prototype),{constructor:fd,isHemisphereLight:!0,copy:function(e){return hd.prototype.copy.call(this,e),this.groundColor.copy(e.groundColor),this}}),Object.assign(pd.prototype,{_projScreenMatrix:new sr,_lightPositionWorld:new $n,_lookTarget:new $n,getViewportCount:function(){return this._viewportCount},getFrustum:function(){return this._frustum},updateMatrices:function(e){var t=this.camera,n=this.matrix,r=this._projScreenMatrix,i=this._lookTarget,o=this._lightPositionWorld;o.setFromMatrixPosition(e.matrixWorld),t.position.copy(o),i.setFromMatrixPosition(e.target.matrixWorld),t.lookAt(i),t.updateMatrixWorld(),r.multiplyMatrices(t.projectionMatrix,t.matrixWorldInverse),this._frustum.setFromProjectionMatrix(r),n.set(.5,0,0,.5,0,.5,0,.5,0,0,.5,.5,0,0,0,1),n.multiply(t.projectionMatrix),n.multiply(t.matrixWorldInverse)},getViewport:function(e){return this._viewports[e]},getFrameExtents:function(){return this._frameExtents},copy:function(e){return this.camera=e.camera.clone(),this.bias=e.bias,this.radius=e.radius,this.mapSize.copy(e.mapSize),this},clone:function(){return(new this.constructor).copy(this)},toJSON:function(){var e={};return 0!==this.bias&&(e.bias=this.bias),1!==this.radius&&(e.radius=this.radius),512===this.mapSize.x&&512===this.mapSize.y||(e.mapSize=this.mapSize.toArray()),e.camera=this.camera.toJSON(!1).object,delete e.camera.matrix,e}}),gd.prototype=Object.assign(Object.create(pd.prototype),{constructor:gd,isSpotLightShadow:!0,updateMatrices:function(e){var t=this.camera,n=2*Hn.RAD2DEG*e.angle,r=this.mapSize.width/this.mapSize.height,i=e.distance||t.far;n===t.fov&&r===t.aspect&&i===t.far||(t.fov=n,t.aspect=r,t.far=i,t.updateProjectionMatrix()),pd.prototype.updateMatrices.call(this,e)}}),vd.prototype=Object.assign(Object.create(hd.prototype),{constructor:vd,isSpotLight:!0,copy:function(e){return hd.prototype.copy.call(this,e),this.distance=e.distance,this.angle=e.angle,this.penumbra=e.penumbra,this.decay=e.decay,this.target=e.target.clone(),this.shadow=e.shadow.clone(),this}}),md.prototype=Object.assign(Object.create(pd.prototype),{constructor:md,isPointLightShadow:!0,updateMatrices:function(e,t){void 0===t&&(t=0);var n=this.camera,r=this.matrix,i=this._lightPositionWorld,o=this._lookTarget,a=this._projScreenMatrix;i.setFromMatrixPosition(e.matrixWorld),n.position.copy(i),o.copy(n.position),o.add(this._cubeDirections[t]),n.up.copy(this._cubeUps[t]),n.lookAt(o),n.updateMatrixWorld(),r.makeTranslation(-i.x,-i.y,-i.z),a.multiplyMatrices(n.projectionMatrix,n.matrixWorldInverse),this._frustum.setFromProjectionMatrix(a)}}),yd.prototype=Object.assign(Object.create(hd.prototype),{constructor:yd,isPointLight:!0,copy:function(e){return hd.prototype.copy.call(this,e),this.distance=e.distance,this.decay=e.decay,this.shadow=e.shadow.clone(),this}}),bd.prototype=Object.assign(Object.create(Io.prototype),{constructor:bd,isOrthographicCamera:!0,copy:function(e,t){return Io.prototype.copy.call(this,e,t),this.left=e.left,this.right=e.right,this.top=e.top,this.bottom=e.bottom,this.near=e.near,this.far=e.far,this.zoom=e.zoom,this.view=null===e.view?null:Object.assign({},e.view),this},setViewOffset:function(e,t,n,r,i,o){null===this.view&&(this.view={enabled:!0,fullWidth:1,fullHeight:1,offsetX:0,offsetY:0,width:1,height:1}),this.view.enabled=!0,this.view.fullWidth=e,this.view.fullHeight=t,this.view.offsetX=n,this.view.offsetY=r,this.view.width=i,this.view.height=o,this.updateProjectionMatrix()},clearViewOffset:function(){null!==this.view&&(this.view.enabled=!1),this.updateProjectionMatrix()},updateProjectionMatrix:function(){var e=(this.right-this.left)/(2*this.zoom),t=(this.top-this.bottom)/(2*this.zoom),n=(this.right+this.left)/2,r=(this.top+this.bottom)/2,i=n-e,o=n+e,a=r+t,s=r-t;if(null!==this.view&&this.view.enabled){var l=(this.right-this.left)/this.view.fullWidth/this.zoom,u=(this.top-this.bottom)/this.view.fullHeight/this.zoom;o=(i+=l*this.view.offsetX)+l*this.view.width,s=(a-=u*this.view.offsetY)-u*this.view.height}this.projectionMatrix.makeOrthographic(i,o,a,s,this.near,this.far),this.projectionMatrixInverse.getInverse(this.projectionMatrix)},toJSON:function(e){var t=wr.prototype.toJSON.call(this,e);return t.object.zoom=this.zoom,t.object.left=this.left,t.object.right=this.right,t.object.top=this.top,t.object.bottom=this.bottom,t.object.near=this.near,t.object.far=this.far,null!==this.view&&(t.object.view=Object.assign({},this.view)),t}}),Cd.prototype=Object.assign(Object.create(pd.prototype),{constructor:Cd,isDirectionalLightShadow:!0,updateMatrices:function(e){pd.prototype.updateMatrices.call(this,e)}}),Ad.prototype=Object.assign(Object.create(hd.prototype),{constructor:Ad,isDirectionalLight:!0,copy:function(e){return hd.prototype.copy.call(this,e),this.target=e.target.clone(),this.shadow=e.shadow.clone(),this}}),_d.prototype=Object.assign(Object.create(hd.prototype),{constructor:_d,isAmbientLight:!0}),Id.prototype=Object.assign(Object.create(hd.prototype),{constructor:Id,isRectAreaLight:!0,copy:function(e){return hd.prototype.copy.call(this,e),this.width=e.width,this.height=e.height,this},toJSON:function(e){var t=hd.prototype.toJSON.call(this,e);return t.object.width=this.width,t.object.height=this.height,t}}),Object.assign(xd.prototype,{isSphericalHarmonics3:!0,set:function(e){for(var t=0;t<9;t++)this.coefficients[t].copy(e[t]);return this},zero:function(){for(var e=0;e<9;e++)this.coefficients[e].set(0,0,0);return this},getAt:function(e,t){var n=e.x,r=e.y,i=e.z,o=this.coefficients;return t.copy(o[0]).multiplyScalar(.282095),t.addScaledVector(o[1],.488603*r),t.addScaledVector(o[2],.488603*i),t.addScaledVector(o[3],.488603*n),t.addScaledVector(o[4],n*r*1.092548),t.addScaledVector(o[5],r*i*1.092548),t.addScaledVector(o[6],.315392*(3*i*i-1)),t.addScaledVector(o[7],n*i*1.092548),t.addScaledVector(o[8],.546274*(n*n-r*r)),t},getIrradianceAt:function(e,t){var n=e.x,r=e.y,i=e.z,o=this.coefficients;return t.copy(o[0]).multiplyScalar(.886227),t.addScaledVector(o[1],1.023328*r),t.addScaledVector(o[2],1.023328*i),t.addScaledVector(o[3],1.023328*n),t.addScaledVector(o[4],.858086*n*r),t.addScaledVector(o[5],.858086*r*i),t.addScaledVector(o[6],.743125*i*i-.247708),t.addScaledVector(o[7],.858086*n*i),t.addScaledVector(o[8],.429043*(n*n-r*r)),t},add:function(e){for(var t=0;t<9;t++)this.coefficients[t].add(e.coefficients[t]);return this},addScaledSH:function(e,t){for(var n=0;n<9;n++)this.coefficients[n].addScaledVector(e.coefficients[n],t);return this},scale:function(e){for(var t=0;t<9;t++)this.coefficients[t].multiplyScalar(e);return this},lerp:function(e,t){for(var n=0;n<9;n++)this.coefficients[n].lerp(e.coefficients[n],t);return this},equals:function(e){for(var t=0;t<9;t++)if(!this.coefficients[t].equals(e.coefficients[t]))return!1;return!0},copy:function(e){return this.set(e.coefficients)},clone:function(){return(new this.constructor).copy(this)},fromArray:function(e,t){void 0===t&&(t=0);for(var n=this.coefficients,r=0;r<9;r++)n[r].fromArray(e,t+3*r);return this},toArray:function(e,t){void 0===e&&(e=[]),void 0===t&&(t=0);for(var n=this.coefficients,r=0;r<9;r++)n[r].toArray(e,t+3*r);return e}}),Object.assign(xd,{getBasisAt:function(e,t){var n=e.x,r=e.y,i=e.z;t[0]=.282095,t[1]=.488603*r,t[2]=.488603*i,t[3]=.488603*n,t[4]=1.092548*n*r,t[5]=1.092548*r*i,t[6]=.315392*(3*i*i-1),t[7]=1.092548*n*i,t[8]=.546274*(n*n-r*r)}}),wd.prototype=Object.assign(Object.create(hd.prototype),{constructor:wd,isLightProbe:!0,copy:function(e){return hd.prototype.copy.call(this,e),this.sh.copy(e.sh),this},fromJSON:function(e){return this.intensity=e.intensity,this.sh.fromArray(e.sh),this},toJSON:function(e){var t=hd.prototype.toJSON.call(this,e);return t.object.sh=this.sh.toArray(),t}}),Md.prototype=Object.assign(Object.create(Pc.prototype),{constructor:Md,load:function(e,t,n,r){var i=this,o=new jc(i.manager);o.setPath(i.path),o.load(e,(function(n){try{t(i.parse(JSON.parse(n)))}catch(t){r?r(t):console.error(t),i.manager.itemError(e)}}),n,r)},parse:function(e){var t=this.textures;function n(e){return void 0===t[e]&&console.warn("THREE.MaterialLoader: Undefined texture",e),t[e]}var r=new gc[e.type];if(void 0!==e.uuid&&(r.uuid=e.uuid),void 0!==e.name&&(r.name=e.name),void 0!==e.color&&r.color.setHex(e.color),void 0!==e.roughness&&(r.roughness=e.roughness),void 0!==e.metalness&&(r.metalness=e.metalness),void 0!==e.sheen&&(r.sheen=(new vi).setHex(e.sheen)),void 0!==e.emissive&&r.emissive.setHex(e.emissive),void 0!==e.specular&&r.specular.setHex(e.specular),void 0!==e.shininess&&(r.shininess=e.shininess),void 0!==e.clearcoat&&(r.clearcoat=e.clearcoat),void 0!==e.clearcoatRoughness&&(r.clearcoatRoughness=e.clearcoatRoughness),void 0!==e.fog&&(r.fog=e.fog),void 0!==e.flatShading&&(r.flatShading=e.flatShading),void 0!==e.blending&&(r.blending=e.blending),void 0!==e.combine&&(r.combine=e.combine),void 0!==e.side&&(r.side=e.side),void 0!==e.opacity&&(r.opacity=e.opacity),void 0!==e.transparent&&(r.transparent=e.transparent),void 0!==e.alphaTest&&(r.alphaTest=e.alphaTest),void 0!==e.depthTest&&(r.depthTest=e.depthTest),void 0!==e.depthWrite&&(r.depthWrite=e.depthWrite),void 0!==e.colorWrite&&(r.colorWrite=e.colorWrite),void 0!==e.stencilWrite&&(r.stencilWrite=e.stencilWrite),void 0!==e.stencilWriteMask&&(r.stencilWriteMask=e.stencilWriteMask),void 0!==e.stencilFunc&&(r.stencilFunc=e.stencilFunc),void 0!==e.stencilRef&&(r.stencilRef=e.stencilRef),void 0!==e.stencilFuncMask&&(r.stencilFuncMask=e.stencilFuncMask),void 0!==e.stencilFail&&(r.stencilFail=e.stencilFail),void 0!==e.stencilZFail&&(r.stencilZFail=e.stencilZFail),void 0!==e.stencilZPass&&(r.stencilZPass=e.stencilZPass),void 0!==e.wireframe&&(r.wireframe=e.wireframe),void 0!==e.wireframeLinewidth&&(r.wireframeLinewidth=e.wireframeLinewidth),void 0!==e.wireframeLinecap&&(r.wireframeLinecap=e.wireframeLinecap),void 0!==e.wireframeLinejoin&&(r.wireframeLinejoin=e.wireframeLinejoin),void 0!==e.rotation&&(r.rotation=e.rotation),1!==e.linewidth&&(r.linewidth=e.linewidth),void 0!==e.dashSize&&(r.dashSize=e.dashSize),void 0!==e.gapSize&&(r.gapSize=e.gapSize),void 0!==e.scale&&(r.scale=e.scale),void 0!==e.polygonOffset&&(r.polygonOffset=e.polygonOffset),void 0!==e.polygonOffsetFactor&&(r.polygonOffsetFactor=e.polygonOffsetFactor),void 0!==e.polygonOffsetUnits&&(r.polygonOffsetUnits=e.polygonOffsetUnits),void 0!==e.skinning&&(r.skinning=e.skinning),void 0!==e.morphTargets&&(r.morphTargets=e.morphTargets),void 0!==e.morphNormals&&(r.morphNormals=e.morphNormals),void 0!==e.dithering&&(r.dithering=e.dithering),void 0!==e.vertexTangents&&(r.vertexTangents=e.vertexTangents),void 0!==e.visible&&(r.visible=e.visible),void 0!==e.toneMapped&&(r.toneMapped=e.toneMapped),void 0!==e.userData&&(r.userData=e.userData),void 0!==e.vertexColors&&("number"==typeof e.vertexColors?r.vertexColors=e.vertexColors>0:r.vertexColors=e.vertexColors),void 0!==e.uniforms)for(var i in e.uniforms){var o=e.uniforms[i];switch(r.uniforms[i]={},o.type){case"t":r.uniforms[i].value=n(o.value);break;case"c":r.uniforms[i].value=(new vi).setHex(o.value);break;case"v2":r.uniforms[i].value=(new Un).fromArray(o.value);break;case"v3":r.uniforms[i].value=(new $n).fromArray(o.value);break;case"v4":r.uniforms[i].value=(new Xn).fromArray(o.value);break;case"m3":r.uniforms[i].value=(new Vn).fromArray(o.value);case"m4":r.uniforms[i].value=(new sr).fromArray(o.value);break;default:r.uniforms[i].value=o.value}}if(void 0!==e.defines&&(r.defines=e.defines),void 0!==e.vertexShader&&(r.vertexShader=e.vertexShader),void 0!==e.fragmentShader&&(r.fragmentShader=e.fragmentShader),void 0!==e.extensions)for(var a in e.extensions)r.extensions[a]=e.extensions[a];if(void 0!==e.shading&&(r.flatShading=1===e.shading),void 0!==e.size&&(r.size=e.size),void 0!==e.sizeAttenuation&&(r.sizeAttenuation=e.sizeAttenuation),void 0!==e.map&&(r.map=n(e.map)),void 0!==e.matcap&&(r.matcap=n(e.matcap)),void 0!==e.alphaMap&&(r.alphaMap=n(e.alphaMap)),void 0!==e.bumpMap&&(r.bumpMap=n(e.bumpMap)),void 0!==e.bumpScale&&(r.bumpScale=e.bumpScale),void 0!==e.normalMap&&(r.normalMap=n(e.normalMap)),void 0!==e.normalMapType&&(r.normalMapType=e.normalMapType),void 0!==e.normalScale){var s=e.normalScale;!1===Array.isArray(s)&&(s=[s,s]),r.normalScale=(new Un).fromArray(s)}return void 0!==e.displacementMap&&(r.displacementMap=n(e.displacementMap)),void 0!==e.displacementScale&&(r.displacementScale=e.displacementScale),void 0!==e.displacementBias&&(r.displacementBias=e.displacementBias),void 0!==e.roughnessMap&&(r.roughnessMap=n(e.roughnessMap)),void 0!==e.metalnessMap&&(r.metalnessMap=n(e.metalnessMap)),void 0!==e.emissiveMap&&(r.emissiveMap=n(e.emissiveMap)),void 0!==e.emissiveIntensity&&(r.emissiveIntensity=e.emissiveIntensity),void 0!==e.specularMap&&(r.specularMap=n(e.specularMap)),void 0!==e.envMap&&(r.envMap=n(e.envMap)),void 0!==e.envMapIntensity&&(r.envMapIntensity=e.envMapIntensity),void 0!==e.reflectivity&&(r.reflectivity=e.reflectivity),void 0!==e.refractionRatio&&(r.refractionRatio=e.refractionRatio),void 0!==e.lightMap&&(r.lightMap=n(e.lightMap)),void 0!==e.lightMapIntensity&&(r.lightMapIntensity=e.lightMapIntensity),void 0!==e.aoMap&&(r.aoMap=n(e.aoMap)),void 0!==e.aoMapIntensity&&(r.aoMapIntensity=e.aoMapIntensity),void 0!==e.gradientMap&&(r.gradientMap=n(e.gradientMap)),void 0!==e.clearcoatMap&&(r.clearcoatMap=n(e.clearcoatMap)),void 0!==e.clearcoatRoughnessMap&&(r.clearcoatRoughnessMap=n(e.clearcoatRoughnessMap)),void 0!==e.clearcoatNormalMap&&(r.clearcoatNormalMap=n(e.clearcoatNormalMap)),void 0!==e.clearcoatNormalScale&&(r.clearcoatNormalScale=(new Un).fromArray(e.clearcoatNormalScale)),r},setTextures:function(e){return this.textures=e,this}});var Sd={decodeText:function(e){if("undefined"!=typeof TextDecoder)return(new TextDecoder).decode(e);for(var t="",n=0,r=e.length;n<r;n++)t+=String.fromCharCode(e[n]);try{return decodeURIComponent(escape(t))}catch(e){return t}},extractUrlBase:function(e){var t=e.lastIndexOf("/");return-1===t?"./":e.substr(0,t+1)}};function Td(){Yi.call(this),this.type="InstancedBufferGeometry",this.instanceCount=1/0}function Ed(e,t,n,r){"number"==typeof n&&(r=n,n=!1,console.error("THREE.InstancedBufferAttribute: The constructor now expects normalized as the third argument.")),wi.call(this,e,t,n),this.meshPerAttribute=r||1}function Od(e){Pc.call(this,e)}Td.prototype=Object.assign(Object.create(Yi.prototype),{constructor:Td,isInstancedBufferGeometry:!0,copy:function(e){return Yi.prototype.copy.call(this,e),this.instanceCount=e.instanceCount,this},clone:function(){return(new this.constructor).copy(this)},toJSON:function(){var e=Yi.prototype.toJSON.call(this);return e.instanceCount=this.instanceCount,e.isInstancedBufferGeometry=!0,e}}),Ed.prototype=Object.assign(Object.create(wi.prototype),{constructor:Ed,isInstancedBufferAttribute:!0,copy:function(e){return wi.prototype.copy.call(this,e),this.meshPerAttribute=e.meshPerAttribute,this},toJSON:function(){var e=wi.prototype.toJSON.call(this);return e.meshPerAttribute=this.meshPerAttribute,e.isInstancedBufferAttribute=!0,e}}),Od.prototype=Object.assign(Object.create(Pc.prototype),{constructor:Od,load:function(e,t,n,r){var i=this,o=new jc(i.manager);o.setPath(i.path),o.load(e,(function(n){try{t(i.parse(JSON.parse(n)))}catch(t){r?r(t):console.error(t),i.manager.itemError(e)}}),n,r)},parse:function(e){var t=e.isInstancedBufferGeometry?new Td:new Yi,n=e.data.index;if(void 0!==n){var r=new kd[n.type](n.array);t.setIndex(new wi(r,1))}var i=e.data.attributes;for(var o in i){var a=i[o],s=(r=new kd[a.type](a.array),new(a.isInstancedBufferAttribute?Ed:wi)(r,a.itemSize,a.normalized));void 0!==a.name&&(s.name=a.name),t.setAttribute(o,s)}var l=e.data.morphAttributes;if(l)for(var o in l){for(var u=l[o],c=[],d=0,h=u.length;d<h;d++){a=u[d],s=new wi(r=new kd[a.type](a.array),a.itemSize,a.normalized);void 0!==a.name&&(s.name=a.name),c.push(s)}t.morphAttributes[o]=c}e.data.morphTargetsRelative&&(t.morphTargetsRelative=!0);var f=e.data.groups||e.data.drawcalls||e.data.offsets;if(void 0!==f){d=0;for(var p=f.length;d!==p;++d){var g=f[d];t.addGroup(g.start,g.count,g.materialIndex)}}var v=e.data.boundingSphere;if(void 0!==v){var m=new $n;void 0!==v.center&&m.fromArray(v.center),t.boundingSphere=new Vr(m,v.radius)}return e.name&&(t.name=e.name),e.userData&&(t.userData=e.userData),t}});var kd={Int8Array:Int8Array,Uint8Array:Uint8Array,Uint8ClampedArray:"undefined"!=typeof Uint8ClampedArray?Uint8ClampedArray:Uint8Array,Int16Array:Int16Array,Uint16Array:Uint16Array,Int32Array:Int32Array,Uint32Array:Uint32Array,Float32Array:Float32Array,Float64Array:Float64Array};function Ld(e){Pc.call(this,e)}Ld.prototype=Object.assign(Object.create(Pc.prototype),{constructor:Ld,load:function(e,t,n,r){var i=this,o=""===this.path?Sd.extractUrlBase(e):this.path;this.resourcePath=this.resourcePath||o;var a=new jc(i.manager);a.setPath(this.path),a.load(e,(function(n){var o=null;try{o=JSON.parse(n)}catch(t){return void 0!==r&&r(t),void console.error("THREE:ObjectLoader: Can't parse "+e+".",t.message)}var a=o.metadata;void 0!==a&&void 0!==a.type&&"geometry"!==a.type.toLowerCase()?i.parse(o,t):console.error("THREE.ObjectLoader: Can't load "+e)}),n,r)},parse:function(e,t){var n=this.parseShape(e.shapes),r=this.parseGeometries(e.geometries,n),i=this.parseImages(e.images,(function(){void 0!==t&&t(s)})),o=this.parseTextures(e.textures,i),a=this.parseMaterials(e.materials,o),s=this.parseObject(e.object,r,a);return e.animations&&(s.animations=this.parseAnimations(e.animations)),void 0!==e.images&&0!==e.images.length||void 0!==t&&t(s),s},parseShape:function(e){var t={};if(void 0!==e)for(var n=0,r=e.length;n<r;n++){var i=(new dd).fromJSON(e[n]);t[i.uuid]=i}return t},parseGeometries:function(e,t){var n={};if(void 0!==e)for(var r=new Od,i=0,o=e.length;i<o;i++){var a,s=e[i];switch(s.type){case"PlaneGeometry":case"PlaneBufferGeometry":a=new ic[s.type](s.width,s.height,s.widthSegments,s.heightSegments);break;case"BoxGeometry":case"BoxBufferGeometry":case"CubeGeometry":a=new ic[s.type](s.width,s.height,s.depth,s.widthSegments,s.heightSegments,s.depthSegments);break;case"CircleGeometry":case"CircleBufferGeometry":a=new ic[s.type](s.radius,s.segments,s.thetaStart,s.thetaLength);break;case"CylinderGeometry":case"CylinderBufferGeometry":a=new ic[s.type](s.radiusTop,s.radiusBottom,s.height,s.radialSegments,s.heightSegments,s.openEnded,s.thetaStart,s.thetaLength);break;case"ConeGeometry":case"ConeBufferGeometry":a=new ic[s.type](s.radius,s.height,s.radialSegments,s.heightSegments,s.openEnded,s.thetaStart,s.thetaLength);break;case"SphereGeometry":case"SphereBufferGeometry":a=new ic[s.type](s.radius,s.widthSegments,s.heightSegments,s.phiStart,s.phiLength,s.thetaStart,s.thetaLength);break;case"DodecahedronGeometry":case"DodecahedronBufferGeometry":case"IcosahedronGeometry":case"IcosahedronBufferGeometry":case"OctahedronGeometry":case"OctahedronBufferGeometry":case"TetrahedronGeometry":case"TetrahedronBufferGeometry":a=new ic[s.type](s.radius,s.detail);break;case"RingGeometry":case"RingBufferGeometry":a=new ic[s.type](s.innerRadius,s.outerRadius,s.thetaSegments,s.phiSegments,s.thetaStart,s.thetaLength);break;case"TorusGeometry":case"TorusBufferGeometry":a=new ic[s.type](s.radius,s.tube,s.radialSegments,s.tubularSegments,s.arc);break;case"TorusKnotGeometry":case"TorusKnotBufferGeometry":a=new ic[s.type](s.radius,s.tube,s.tubularSegments,s.radialSegments,s.p,s.q);break;case"TubeGeometry":case"TubeBufferGeometry":a=new ic[s.type]((new ld[s.path.type]).fromJSON(s.path),s.tubularSegments,s.radius,s.radialSegments,s.closed);break;case"LatheGeometry":case"LatheBufferGeometry":a=new ic[s.type](s.points,s.segments,s.phiStart,s.phiLength);break;case"PolyhedronGeometry":case"PolyhedronBufferGeometry":a=new ic[s.type](s.vertices,s.indices,s.radius,s.details);break;case"ShapeGeometry":case"ShapeBufferGeometry":for(var l=[],u=0,c=s.shapes.length;u<c;u++){var d=t[s.shapes[u]];l.push(d)}a=new ic[s.type](l,s.curveSegments);break;case"ExtrudeGeometry":case"ExtrudeBufferGeometry":for(l=[],u=0,c=s.shapes.length;u<c;u++){d=t[s.shapes[u]];l.push(d)}var h=s.options.extrudePath;void 0!==h&&(s.options.extrudePath=(new ld[h.type]).fromJSON(h)),a=new ic[s.type](l,s.options);break;case"BufferGeometry":case"InstancedBufferGeometry":a=r.parse(s);break;case"Geometry":console.error('THREE.ObjectLoader: Loading "Geometry" is not supported anymore.');break;default:console.warn('THREE.ObjectLoader: Unsupported geometry type "'+s.type+'"');continue}a.uuid=s.uuid,void 0!==s.name&&(a.name=s.name),!0===a.isBufferGeometry&&void 0!==s.userData&&(a.userData=s.userData),n[s.uuid]=a}return n},parseMaterials:function(e,t){var n={},r={};if(void 0!==e){var i=new Md;i.setTextures(t);for(var o=0,a=e.length;o<a;o++){var s=e[o];if("MultiMaterial"===s.type){for(var l=[],u=0;u<s.materials.length;u++){var c=s.materials[u];void 0===n[c.uuid]&&(n[c.uuid]=i.parse(c)),l.push(n[c.uuid])}r[s.uuid]=l}else void 0===n[s.uuid]&&(n[s.uuid]=i.parse(s)),r[s.uuid]=n[s.uuid]}}return r},parseAnimations:function(e){for(var t=[],n=0;n<e.length;n++){var r=e[n],i=Ec.parse(r);void 0!==r.uuid&&(i.uuid=r.uuid),t.push(i)}return t},parseImages:function(e,t){var n=this,r={};function i(e){return n.manager.itemStart(e),o.load(e,(function(){n.manager.itemEnd(e)}),void 0,(function(){n.manager.itemError(e),n.manager.itemEnd(e)}))}if(void 0!==e&&e.length>0){var o=new Bc(new Lc(t));o.setCrossOrigin(this.crossOrigin);for(var a=0,s=e.length;a<s;a++){var l=e[a],u=l.url;if(Array.isArray(u)){r[l.uuid]=[];for(var c=0,d=u.length;c<d;c++){var h=u[c],f=/^(\/\/)|([a-z]+:(\/\/)?)/i.test(h)?h:n.resourcePath+h;r[l.uuid].push(i(f))}}else{f=/^(\/\/)|([a-z]+:(\/\/)?)/i.test(l.url)?l.url:n.resourcePath+l.url;r[l.uuid]=i(f)}}}return r},parseTextures:function(e,t){function n(e,t){return"number"==typeof e?e:(console.warn("THREE.ObjectLoader.parseTexture: Constant should be in numeric form.",e),t[e])}var r={};if(void 0!==e)for(var i=0,o=e.length;i<o;i++){var a,s=e[i];void 0===s.image&&console.warn('THREE.ObjectLoader: No "image" specified for',s.uuid),void 0===t[s.image]&&console.warn("THREE.ObjectLoader: Undefined image",s.image),(a=Array.isArray(t[s.image])?new Zo(t[s.image]):new Gn(t[s.image])).needsUpdate=!0,a.uuid=s.uuid,void 0!==s.name&&(a.name=s.name),void 0!==s.mapping&&(a.mapping=n(s.mapping,Pd)),void 0!==s.offset&&a.offset.fromArray(s.offset),void 0!==s.repeat&&a.repeat.fromArray(s.repeat),void 0!==s.center&&a.center.fromArray(s.center),void 0!==s.rotation&&(a.rotation=s.rotation),void 0!==s.wrap&&(a.wrapS=n(s.wrap[0],Nd),a.wrapT=n(s.wrap[1],Nd)),void 0!==s.format&&(a.format=s.format),void 0!==s.type&&(a.type=s.type),void 0!==s.encoding&&(a.encoding=s.encoding),void 0!==s.minFilter&&(a.minFilter=n(s.minFilter,jd)),void 0!==s.magFilter&&(a.magFilter=n(s.magFilter,jd)),void 0!==s.anisotropy&&(a.anisotropy=s.anisotropy),void 0!==s.flipY&&(a.flipY=s.flipY),void 0!==s.premultiplyAlpha&&(a.premultiplyAlpha=s.premultiplyAlpha),void 0!==s.unpackAlignment&&(a.unpackAlignment=s.unpackAlignment),r[s.uuid]=a}return r},parseObject:function(e,t,n){var r;function i(e){return void 0===t[e]&&console.warn("THREE.ObjectLoader: Undefined geometry",e),t[e]}function o(e){if(void 0!==e){if(Array.isArray(e)){for(var t=[],r=0,i=e.length;r<i;r++){var o=e[r];void 0===n[o]&&console.warn("THREE.ObjectLoader: Undefined material",o),t.push(n[o])}return t}return void 0===n[e]&&console.warn("THREE.ObjectLoader: Undefined material",e),n[e]}}switch(e.type){case"Scene":r=new Mr,void 0!==e.background&&Number.isInteger(e.background)&&(r.background=new vi(e.background)),void 0!==e.fog&&("Fog"===e.fog.type?r.fog=new Rs(e.fog.color,e.fog.near,e.fog.far):"FogExp2"===e.fog.type&&(r.fog=new js(e.fog.color,e.fog.density)));break;case"PerspectiveCamera":r=new xo(e.fov,e.aspect,e.near,e.far),void 0!==e.focus&&(r.focus=e.focus),void 0!==e.zoom&&(r.zoom=e.zoom),void 0!==e.filmGauge&&(r.filmGauge=e.filmGauge),void 0!==e.filmOffset&&(r.filmOffset=e.filmOffset),void 0!==e.view&&(r.view=Object.assign({},e.view));break;case"OrthographicCamera":r=new bd(e.left,e.right,e.top,e.bottom,e.near,e.far),void 0!==e.zoom&&(r.zoom=e.zoom),void 0!==e.view&&(r.view=Object.assign({},e.view));break;case"AmbientLight":r=new _d(e.color,e.intensity);break;case"DirectionalLight":r=new Ad(e.color,e.intensity);break;case"PointLight":r=new yd(e.color,e.intensity,e.distance,e.decay);break;case"RectAreaLight":r=new Id(e.color,e.intensity,e.width,e.height);break;case"SpotLight":r=new vd(e.color,e.intensity,e.distance,e.angle,e.penumbra,e.decay);break;case"HemisphereLight":r=new fd(e.color,e.groundColor,e.intensity);break;case"LightProbe":r=(new wd).fromJSON(e);break;case"SkinnedMesh":console.warn("THREE.ObjectLoader.parseObject() does not support SkinnedMesh yet.");case"Mesh":r=new lo(a=i(e.geometry),s=o(e.material));break;case"InstancedMesh":var a=i(e.geometry),s=o(e.material),l=e.count,u=e.instanceMatrix;(r=new Cl(a,s,l)).instanceMatrix=new wi(new Float32Array(u.array),16);break;case"LOD":r=new cl;break;case"Line":r=new Sl(i(e.geometry),o(e.material),e.mode);break;case"LineLoop":r=new kl(i(e.geometry),o(e.material));break;case"LineSegments":r=new Ol(i(e.geometry),o(e.material));break;case"PointCloud":case"Points":r=new Rl(i(e.geometry),o(e.material));break;case"Sprite":r=new tl(o(e.material));break;case"Group":r=new ks;break;default:r=new wr}if(r.uuid=e.uuid,void 0!==e.name&&(r.name=e.name),void 0!==e.matrix?(r.matrix.fromArray(e.matrix),void 0!==e.matrixAutoUpdate&&(r.matrixAutoUpdate=e.matrixAutoUpdate),r.matrixAutoUpdate&&r.matrix.decompose(r.position,r.quaternion,r.scale)):(void 0!==e.position&&r.position.fromArray(e.position),void 0!==e.rotation&&r.rotation.fromArray(e.rotation),void 0!==e.quaternion&&r.quaternion.fromArray(e.quaternion),void 0!==e.scale&&r.scale.fromArray(e.scale)),void 0!==e.castShadow&&(r.castShadow=e.castShadow),void 0!==e.receiveShadow&&(r.receiveShadow=e.receiveShadow),e.shadow&&(void 0!==e.shadow.bias&&(r.shadow.bias=e.shadow.bias),void 0!==e.shadow.radius&&(r.shadow.radius=e.shadow.radius),void 0!==e.shadow.mapSize&&r.shadow.mapSize.fromArray(e.shadow.mapSize),void 0!==e.shadow.camera&&(r.shadow.camera=this.parseObject(e.shadow.camera))),void 0!==e.visible&&(r.visible=e.visible),void 0!==e.frustumCulled&&(r.frustumCulled=e.frustumCulled),void 0!==e.renderOrder&&(r.renderOrder=e.renderOrder),void 0!==e.userData&&(r.userData=e.userData),void 0!==e.layers&&(r.layers.mask=e.layers),void 0!==e.children)for(var c=e.children,d=0;d<c.length;d++)r.add(this.parseObject(c[d],t,n));if("LOD"===e.type){void 0!==e.autoUpdate&&(r.autoUpdate=e.autoUpdate);for(var h=e.levels,f=0;f<h.length;f++){var p=h[f],g=r.getObjectByProperty("uuid",p.object);void 0!==g&&r.addLevel(g,p.distance)}}return r}});var Dd,Pd={UVMapping:ae,CubeReflectionMapping:se,CubeRefractionMapping:le,EquirectangularReflectionMapping:ue,EquirectangularRefractionMapping:ce,SphericalReflectionMapping:de,CubeUVReflectionMapping:he,CubeUVRefractionMapping:fe},Nd={RepeatWrapping:pe,ClampToEdgeWrapping:ge,MirroredRepeatWrapping:ve},jd={NearestFilter:me,NearestMipmapNearestFilter:ye,NearestMipmapLinearFilter:Ce,LinearFilter:_e,LinearMipmapNearestFilter:Ie,LinearMipmapLinearFilter:we};function Rd(e){"undefined"==typeof createImageBitmap&&console.warn("THREE.ImageBitmapLoader: createImageBitmap() not supported."),"undefined"==typeof fetch&&console.warn("THREE.ImageBitmapLoader: fetch() not supported."),Pc.call(this,e),this.options=void 0}function zd(){this.type="ShapePath",this.color=new vi,this.subPaths=[],this.currentPath=null}function Fd(e){this.type="Font",this.data=e}function Bd(e,t,n,r,i){var o=i.glyphs[e]||i.glyphs["?"];if(o){var a,s,l,u,c,d,h,f,p=new zd;if(o.o)for(var g=o._cachedOutline||(o._cachedOutline=o.o.split(" ")),v=0,m=g.length;v<m;){switch(g[v++]){case"m":a=g[v++]*t+n,s=g[v++]*t+r,p.moveTo(a,s);break;case"l":a=g[v++]*t+n,s=g[v++]*t+r,p.lineTo(a,s);break;case"q":l=g[v++]*t+n,u=g[v++]*t+r,c=g[v++]*t+n,d=g[v++]*t+r,p.quadraticCurveTo(c,d,l,u);break;case"b":l=g[v++]*t+n,u=g[v++]*t+r,c=g[v++]*t+n,d=g[v++]*t+r,h=g[v++]*t+n,f=g[v++]*t+r,p.bezierCurveTo(c,d,h,f,l,u)}}return{offsetX:o.ha*t,path:p}}console.error('THREE.Font: character "'+e+'" does not exists in font family '+i.familyName+".")}function Hd(e){Pc.call(this,e)}Rd.prototype=Object.assign(Object.create(Pc.prototype),{constructor:Rd,setOptions:function(e){return this.options=e,this},load:function(e,t,n,r){void 0===e&&(e=""),void 0!==this.path&&(e=this.path+e),e=this.manager.resolveURL(e);var i=this,o=kc.get(e);if(void 0!==o)return i.manager.itemStart(e),setTimeout((function(){t&&t(o),i.manager.itemEnd(e)}),0),o;fetch(e).then((function(e){return e.blob()})).then((function(e){return void 0===i.options?createImageBitmap(e):createImageBitmap(e,i.options)})).then((function(n){kc.add(e,n),t&&t(n),i.manager.itemEnd(e)})).catch((function(t){r&&r(t),i.manager.itemError(e),i.manager.itemEnd(e)})),i.manager.itemStart(e)}}),Object.assign(zd.prototype,{moveTo:function(e,t){return this.currentPath=new cd,this.subPaths.push(this.currentPath),this.currentPath.moveTo(e,t),this},lineTo:function(e,t){return this.currentPath.lineTo(e,t),this},quadraticCurveTo:function(e,t,n,r){return this.currentPath.quadraticCurveTo(e,t,n,r),this},bezierCurveTo:function(e,t,n,r,i,o){return this.currentPath.bezierCurveTo(e,t,n,r,i,o),this},splineThru:function(e){return this.currentPath.splineThru(e),this},toShapes:function(e,t){function n(e){for(var t=[],n=0,r=e.length;n<r;n++){var i=e[n],o=new dd;o.curves=i.curves,t.push(o)}return t}function r(e,t){for(var n=t.length,r=!1,i=n-1,o=0;o<n;i=o++){var a=t[i],s=t[o],l=s.x-a.x,u=s.y-a.y;if(Math.abs(u)>Number.EPSILON){if(u<0&&(a=t[o],l=-l,s=t[i],u=-u),e.y<a.y||e.y>s.y)continue;if(e.y===a.y){if(e.x===a.x)return!0}else{var c=u*(e.x-a.x)-l*(e.y-a.y);if(0===c)return!0;if(c<0)continue;r=!r}}else{if(e.y!==a.y)continue;if(s.x<=e.x&&e.x<=a.x||a.x<=e.x&&e.x<=s.x)return!0}}return r}var i=Du.isClockWise,o=this.subPaths;if(0===o.length)return[];if(!0===t)return n(o);var a,s,l,u=[];if(1===o.length)return s=o[0],(l=new dd).curves=s.curves,u.push(l),u;var c=!i(o[0].getPoints());c=e?!c:c;var d,h,f=[],p=[],g=[],v=0;p[v]=void 0,g[v]=[];for(var m=0,y=o.length;m<y;m++)a=i(d=(s=o[m]).getPoints()),(a=e?!a:a)?(!c&&p[v]&&v++,p[v]={s:new dd,p:d},p[v].s.curves=s.curves,c&&v++,g[v]=[]):g[v].push({h:s,p:d[0]});if(!p[0])return n(o);if(p.length>1){for(var b=!1,C=[],A=0,_=p.length;A<_;A++)f[A]=[];for(A=0,_=p.length;A<_;A++)for(var I=g[A],x=0;x<I.length;x++){for(var w=I[x],M=!0,S=0;S<p.length;S++)r(w.p,p[S].p)&&(A!==S&&C.push({froms:A,tos:S,hole:x}),M?(M=!1,f[S].push(w)):b=!0);M&&f[A].push(w)}C.length>0&&(b||(g=f))}m=0;for(var T=p.length;m<T;m++){l=p[m].s,u.push(l);for(var E=0,O=(h=g[m]).length;E<O;E++)l.holes.push(h[E].h)}return u}}),Object.assign(Fd.prototype,{isFont:!0,generateShapes:function(e,t){void 0===t&&(t=100);for(var n=[],r=function(e,t,n){for(var r=Array.from?Array.from(e):String(e).split(""),i=t/n.resolution,o=(n.boundingBox.yMax-n.boundingBox.yMin+n.underlineThickness)*i,a=[],s=0,l=0,u=0;u<r.length;u++){var c=r[u];if("\n"===c)s=0,l-=o;else{var d=Bd(c,i,s,l,n);s+=d.offsetX,a.push(d.path)}}return a}(e,t,this.data),i=0,o=r.length;i<o;i++)Array.prototype.push.apply(n,r[i].toShapes());return n}}),Hd.prototype=Object.assign(Object.create(Pc.prototype),{constructor:Hd,load:function(e,t,n,r){var i=this,o=new jc(this.manager);o.setPath(this.path),o.load(e,(function(e){var n;try{n=JSON.parse(e)}catch(t){console.warn("THREE.FontLoader: typeface.js support is being deprecated. Use typeface.json instead."),n=JSON.parse(e.substring(65,e.length-2))}var r=i.parse(n);t&&t(r)}),n,r)},parse:function(e){return new Fd(e)}});var Ud={getContext:function(){return void 0===Dd&&(Dd=new(window.AudioContext||window.webkitAudioContext)),Dd},setContext:function(e){Dd=e}};function Vd(e){Pc.call(this,e)}function Yd(e,t,n){wd.call(this,void 0,n);var r=(new vi).set(e),i=(new vi).set(t),o=new $n(r.r,r.g,r.b),a=new $n(i.r,i.g,i.b),s=Math.sqrt(Math.PI),l=s*Math.sqrt(.75);this.sh.coefficients[0].copy(o).add(a).multiplyScalar(s),this.sh.coefficients[1].copy(o).sub(a).multiplyScalar(l)}function Wd(e,t){wd.call(this,void 0,t);var n=(new vi).set(e);this.sh.coefficients[0].set(n.r,n.g,n.b).multiplyScalar(2*Math.sqrt(Math.PI))}Vd.prototype=Object.assign(Object.create(Pc.prototype),{constructor:Vd,load:function(e,t,n,r){var i=this,o=new jc(i.manager);o.setResponseType("arraybuffer"),o.setPath(i.path),o.load(e,(function(n){try{var o=n.slice(0);Ud.getContext().decodeAudioData(o,(function(e){t(e)}))}catch(t){r?r(t):console.error(t),i.manager.itemError(e)}}),n,r)}}),Yd.prototype=Object.assign(Object.create(wd.prototype),{constructor:Yd,isHemisphereLightProbe:!0,copy:function(e){return wd.prototype.copy.call(this,e),this},toJSON:function(e){return wd.prototype.toJSON.call(this,e)}}),Wd.prototype=Object.assign(Object.create(wd.prototype),{constructor:Wd,isAmbientLightProbe:!0,copy:function(e){return wd.prototype.copy.call(this,e),this},toJSON:function(e){return wd.prototype.toJSON.call(this,e)}});var Gd=new sr,Xd=new sr;function qd(){this.type="StereoCamera",this.aspect=1,this.eyeSep=.064,this.cameraL=new xo,this.cameraL.layers.enable(1),this.cameraL.matrixAutoUpdate=!1,this.cameraR=new xo,this.cameraR.layers.enable(2),this.cameraR.matrixAutoUpdate=!1,this._cache={focus:null,fov:null,aspect:null,near:null,far:null,zoom:null,eyeSep:null}}function Zd(e){this.autoStart=void 0===e||e,this.startTime=0,this.oldTime=0,this.elapsedTime=0,this.running=!1}Object.assign(qd.prototype,{update:function(e){var t=this._cache;if(t.focus!==e.focus||t.fov!==e.fov||t.aspect!==e.aspect*this.aspect||t.near!==e.near||t.far!==e.far||t.zoom!==e.zoom||t.eyeSep!==this.eyeSep){t.focus=e.focus,t.fov=e.fov,t.aspect=e.aspect*this.aspect,t.near=e.near,t.far=e.far,t.zoom=e.zoom,t.eyeSep=this.eyeSep;var n,r,i=e.projectionMatrix.clone(),o=t.eyeSep/2,a=o*t.near/t.focus,s=t.near*Math.tan(Hn.DEG2RAD*t.fov*.5)/t.zoom;Xd.elements[12]=-o,Gd.elements[12]=o,n=-s*t.aspect+a,r=s*t.aspect+a,i.elements[0]=2*t.near/(r-n),i.elements[8]=(r+n)/(r-n),this.cameraL.projectionMatrix.copy(i),n=-s*t.aspect-a,r=s*t.aspect-a,i.elements[0]=2*t.near/(r-n),i.elements[8]=(r+n)/(r-n),this.cameraR.projectionMatrix.copy(i)}this.cameraL.matrixWorld.copy(e.matrixWorld).multiply(Xd),this.cameraR.matrixWorld.copy(e.matrixWorld).multiply(Gd)}}),Object.assign(Zd.prototype,{start:function(){this.startTime=("undefined"==typeof performance?Date:performance).now(),this.oldTime=this.startTime,this.elapsedTime=0,this.running=!0},stop:function(){this.getElapsedTime(),this.running=!1,this.autoStart=!1},getElapsedTime:function(){return this.getDelta(),this.elapsedTime},getDelta:function(){var e=0;if(this.autoStart&&!this.running)return this.start(),0;if(this.running){var t=("undefined"==typeof performance?Date:performance).now();e=(t-this.oldTime)/1e3,this.oldTime=t,this.elapsedTime+=e}return e}});var Qd=new $n,Kd=new Qn,Jd=new $n,$d=new $n;function eh(){wr.call(this),this.type="AudioListener",this.context=Ud.getContext(),this.gain=this.context.createGain(),this.gain.connect(this.context.destination),this.filter=null,this.timeDelta=0,this._clock=new Zd}function th(e){wr.call(this),this.type="Audio",this.listener=e,this.context=e.context,this.gain=this.context.createGain(),this.gain.connect(e.getInput()),this.autoplay=!1,this.buffer=null,this.detune=0,this.loop=!1,this.loopStart=0,this.loopEnd=0,this.offset=0,this.duration=void 0,this.playbackRate=1,this.isPlaying=!1,this.hasPlaybackControl=!0,this.sourceType="empty",this._startedAt=0,this._progress=0,this.filters=[]}eh.prototype=Object.assign(Object.create(wr.prototype),{constructor:eh,getInput:function(){return this.gain},removeFilter:function(){return null!==this.filter&&(this.gain.disconnect(this.filter),this.filter.disconnect(this.context.destination),this.gain.connect(this.context.destination),this.filter=null),this},getFilter:function(){return this.filter},setFilter:function(e){return null!==this.filter?(this.gain.disconnect(this.filter),this.filter.disconnect(this.context.destination)):this.gain.disconnect(this.context.destination),this.filter=e,this.gain.connect(this.filter),this.filter.connect(this.context.destination),this},getMasterVolume:function(){return this.gain.gain.value},setMasterVolume:function(e){return this.gain.gain.setTargetAtTime(e,this.context.currentTime,.01),this},updateMatrixWorld:function(e){wr.prototype.updateMatrixWorld.call(this,e);var t=this.context.listener,n=this.up;if(this.timeDelta=this._clock.getDelta(),this.matrixWorld.decompose(Qd,Kd,Jd),$d.set(0,0,-1).applyQuaternion(Kd),t.positionX){var r=this.context.currentTime+this.timeDelta;t.positionX.linearRampToValueAtTime(Qd.x,r),t.positionY.linearRampToValueAtTime(Qd.y,r),t.positionZ.linearRampToValueAtTime(Qd.z,r),t.forwardX.linearRampToValueAtTime($d.x,r),t.forwardY.linearRampToValueAtTime($d.y,r),t.forwardZ.linearRampToValueAtTime($d.z,r),t.upX.linearRampToValueAtTime(n.x,r),t.upY.linearRampToValueAtTime(n.y,r),t.upZ.linearRampToValueAtTime(n.z,r)}else t.setPosition(Qd.x,Qd.y,Qd.z),t.setOrientation($d.x,$d.y,$d.z,n.x,n.y,n.z)}}),th.prototype=Object.assign(Object.create(wr.prototype),{constructor:th,getOutput:function(){return this.gain},setNodeSource:function(e){return this.hasPlaybackControl=!1,this.sourceType="audioNode",this.source=e,this.connect(),this},setMediaElementSource:function(e){return this.hasPlaybackControl=!1,this.sourceType="mediaNode",this.source=this.context.createMediaElementSource(e),this.connect(),this},setMediaStreamSource:function(e){return this.hasPlaybackControl=!1,this.sourceType="mediaStreamNode",this.source=this.context.createMediaStreamSource(e),this.connect(),this},setBuffer:function(e){return this.buffer=e,this.sourceType="buffer",this.autoplay&&this.play(),this},play:function(e){if(void 0===e&&(e=0),!0!==this.isPlaying){if(!1!==this.hasPlaybackControl){this._startedAt=this.context.currentTime+e;var t=this.context.createBufferSource();return t.buffer=this.buffer,t.loop=this.loop,t.loopStart=this.loopStart,t.loopEnd=this.loopEnd,t.onended=this.onEnded.bind(this),t.start(this._startedAt,this._progress+this.offset,this.duration),this.isPlaying=!0,this.source=t,this.setDetune(this.detune),this.setPlaybackRate(this.playbackRate),this.connect()}console.warn("THREE.Audio: this Audio has no playback control.")}else console.warn("THREE.Audio: Audio is already playing.")},pause:function(){if(!1!==this.hasPlaybackControl)return!0===this.isPlaying&&(this._progress+=Math.max(this.context.currentTime-this._startedAt,0)*this.playbackRate,!0===this.loop&&(this._progress=this._progress%(this.duration||this.buffer.duration)),this.source.stop(),this.source.onended=null,this.isPlaying=!1),this;console.warn("THREE.Audio: this Audio has no playback control.")},stop:function(){if(!1!==this.hasPlaybackControl)return this._progress=0,this.source.stop(),this.source.onended=null,this.isPlaying=!1,this;console.warn("THREE.Audio: this Audio has no playback control.")},connect:function(){if(this.filters.length>0){this.source.connect(this.filters[0]);for(var e=1,t=this.filters.length;e<t;e++)this.filters[e-1].connect(this.filters[e]);this.filters[this.filters.length-1].connect(this.getOutput())}else this.source.connect(this.getOutput());return this},disconnect:function(){if(this.filters.length>0){this.source.disconnect(this.filters[0]);for(var e=1,t=this.filters.length;e<t;e++)this.filters[e-1].disconnect(this.filters[e]);this.filters[this.filters.length-1].disconnect(this.getOutput())}else this.source.disconnect(this.getOutput());return this},getFilters:function(){return this.filters},setFilters:function(e){return e||(e=[]),!0===this.isPlaying?(this.disconnect(),this.filters=e,this.connect()):this.filters=e,this},setDetune:function(e){if(this.detune=e,void 0!==this.source.detune)return!0===this.isPlaying&&this.source.detune.setTargetAtTime(this.detune,this.context.currentTime,.01),this},getDetune:function(){return this.detune},getFilter:function(){return this.getFilters()[0]},setFilter:function(e){return this.setFilters(e?[e]:[])},setPlaybackRate:function(e){if(!1!==this.hasPlaybackControl)return this.playbackRate=e,!0===this.isPlaying&&this.source.playbackRate.setTargetAtTime(this.playbackRate,this.context.currentTime,.01),this;console.warn("THREE.Audio: this Audio has no playback control.")},getPlaybackRate:function(){return this.playbackRate},onEnded:function(){this.isPlaying=!1},getLoop:function(){return!1===this.hasPlaybackControl?(console.warn("THREE.Audio: this Audio has no playback control."),!1):this.loop},setLoop:function(e){if(!1!==this.hasPlaybackControl)return this.loop=e,!0===this.isPlaying&&(this.source.loop=this.loop),this;console.warn("THREE.Audio: this Audio has no playback control.")},setLoopStart:function(e){return this.loopStart=e,this},setLoopEnd:function(e){return this.loopEnd=e,this},getVolume:function(){return this.gain.gain.value},setVolume:function(e){return this.gain.gain.setTargetAtTime(e,this.context.currentTime,.01),this}});var nh=new $n,rh=new Qn,ih=new $n,oh=new $n;function ah(e){th.call(this,e),this.panner=this.context.createPanner(),this.panner.panningModel="HRTF",this.panner.connect(this.gain)}function sh(e,t){this.analyser=e.context.createAnalyser(),this.analyser.fftSize=void 0!==t?t:2048,this.data=new Uint8Array(this.analyser.frequencyBinCount),e.getOutput().connect(this.analyser)}function lh(e,t,n){var r,i,o;switch(this.binding=e,this.valueSize=n,t){case"quaternion":r=this._slerp,i=this._slerpAdditive,o=this._setAdditiveIdentityQuaternion,this.buffer=new Float64Array(6*n),this._workIndex=5;break;case"string":case"bool":r=this._select,i=this._select,o=this._setAdditiveIdentityOther,this.buffer=new Array(5*n);break;default:r=this._lerp,i=this._lerpAdditive,o=this._setAdditiveIdentityNumeric,this.buffer=new Float64Array(5*n)}this._mixBufferRegion=r,this._mixBufferRegionAdditive=i,this._setIdentity=o,this._origIndex=3,this._addIndex=4,this.cumulativeWeight=0,this.cumulativeWeightAdditive=0,this.useCount=0,this.referenceCount=0}ah.prototype=Object.assign(Object.create(th.prototype),{constructor:ah,getOutput:function(){return this.panner},getRefDistance:function(){return this.panner.refDistance},setRefDistance:function(e){return this.panner.refDistance=e,this},getRolloffFactor:function(){return this.panner.rolloffFactor},setRolloffFactor:function(e){return this.panner.rolloffFactor=e,this},getDistanceModel:function(){return this.panner.distanceModel},setDistanceModel:function(e){return this.panner.distanceModel=e,this},getMaxDistance:function(){return this.panner.maxDistance},setMaxDistance:function(e){return this.panner.maxDistance=e,this},setDirectionalCone:function(e,t,n){return this.panner.coneInnerAngle=e,this.panner.coneOuterAngle=t,this.panner.coneOuterGain=n,this},updateMatrixWorld:function(e){if(wr.prototype.updateMatrixWorld.call(this,e),!0!==this.hasPlaybackControl||!1!==this.isPlaying){this.matrixWorld.decompose(nh,rh,ih),oh.set(0,0,1).applyQuaternion(rh);var t=this.panner;if(t.positionX){var n=this.context.currentTime+this.listener.timeDelta;t.positionX.linearRampToValueAtTime(nh.x,n),t.positionY.linearRampToValueAtTime(nh.y,n),t.positionZ.linearRampToValueAtTime(nh.z,n),t.orientationX.linearRampToValueAtTime(oh.x,n),t.orientationY.linearRampToValueAtTime(oh.y,n),t.orientationZ.linearRampToValueAtTime(oh.z,n)}else t.setPosition(nh.x,nh.y,nh.z),t.setOrientation(oh.x,oh.y,oh.z)}}}),Object.assign(sh.prototype,{getFrequencyData:function(){return this.analyser.getByteFrequencyData(this.data),this.data},getAverageFrequency:function(){for(var e=0,t=this.getFrequencyData(),n=0;n<t.length;n++)e+=t[n];return e/t.length}}),Object.assign(lh.prototype,{accumulate:function(e,t){var n=this.buffer,r=this.valueSize,i=e*r+r,o=this.cumulativeWeight;if(0===o){for(var a=0;a!==r;++a)n[i+a]=n[a];o=t}else{var s=t/(o+=t);this._mixBufferRegion(n,i,0,s,r)}this.cumulativeWeight=o},accumulateAdditive:function(e){var t=this.buffer,n=this.valueSize,r=n*this._addIndex;0===this.cumulativeWeightAdditive&&this._setIdentity(),this._mixBufferRegionAdditive(t,r,0,e,n),this.cumulativeWeightAdditive+=e},apply:function(e){var t=this.valueSize,n=this.buffer,r=e*t+t,i=this.cumulativeWeight,o=this.cumulativeWeightAdditive,a=this.binding;if(this.cumulativeWeight=0,this.cumulativeWeightAdditive=0,i<1){var s=t*this._origIndex;this._mixBufferRegion(n,r,s,1-i,t)}o>0&&this._mixBufferRegionAdditive(n,r,this._addIndex*t,1,t);for(var l=t,u=t+t;l!==u;++l)if(n[l]!==n[l+t]){a.setValue(n,r);break}},saveOriginalState:function(){var e=this.binding,t=this.buffer,n=this.valueSize,r=n*this._origIndex;e.getValue(t,r);for(var i=n,o=r;i!==o;++i)t[i]=t[r+i%n];this._setIdentity(),this.cumulativeWeight=0,this.cumulativeWeightAdditive=0},restoreOriginalState:function(){var e=3*this.valueSize;this.binding.setValue(this.buffer,e)},_setAdditiveIdentityNumeric:function(){for(var e=this._addIndex*this.valueSize,t=e+this.valueSize,n=e;n<t;n++)this.buffer[n]=0},_setAdditiveIdentityQuaternion:function(){this._setAdditiveIdentityNumeric(),this.buffer[4*this._addIndex+3]=1},_setAdditiveIdentityOther:function(){for(var e=this._origIndex*this.valueSize,t=this._addIndex*this.valueSize,n=0;n<this.valueSize;n++)this.buffer[t+n]=this.buffer[e+n]},_select:function(e,t,n,r,i){if(r>=.5)for(var o=0;o!==i;++o)e[t+o]=e[n+o]},_slerp:function(e,t,n,r){Qn.slerpFlat(e,t,e,t,e,n,r)},_slerpAdditive:function(e,t,n,r,i){var o=this._workIndex*i;Qn.multiplyQuaternionsFlat(e,o,e,t,e,n),Qn.slerpFlat(e,t,e,t,e,o,r)},_lerp:function(e,t,n,r,i){for(var o=1-r,a=0;a!==i;++a){var s=t+a;e[s]=e[s]*o+e[n+a]*r}},_lerpAdditive:function(e,t,n,r,i){for(var o=0;o!==i;++o){var a=t+o;e[a]=e[a]+e[n+o]*r}}});var uh=new RegExp("[\\[\\]\\.:\\/]","g"),ch="[^"+"\\[\\]\\.:\\/".replace("\\.","")+"]",dh=/((?:WC+[\/:])*)/.source.replace("WC","[^\\[\\]\\.:\\/]"),hh=/(WCOD+)?/.source.replace("WCOD",ch),fh=/(?:\.(WC+)(?:\[(.+)\])?)?/.source.replace("WC","[^\\[\\]\\.:\\/]"),ph=/\.(WC+)(?:\[(.+)\])?/.source.replace("WC","[^\\[\\]\\.:\\/]"),gh=new RegExp("^"+dh+hh+fh+ph+"$"),vh=["material","materials","bones"];function mh(e,t,n){var r=n||yh.parseTrackName(t);this._targetGroup=e,this._bindings=e.subscribe_(t,r)}function yh(e,t,n){this.path=t,this.parsedPath=n||yh.parseTrackName(t),this.node=yh.findNode(e,this.parsedPath.nodeName)||e,this.rootNode=e}function bh(){this.uuid=Hn.generateUUID(),this._objects=Array.prototype.slice.call(arguments),this.nCachedObjects_=0;var e={};this._indicesByUUID=e;for(var t=0,n=arguments.length;t!==n;++t)e[arguments[t].uuid]=t;this._paths=[],this._parsedPaths=[],this._bindings=[],this._bindingsIndicesByPath={};var r=this;this.stats={objects:{get total(){return r._objects.length},get inUse(){return this.total-r.nCachedObjects_}},get bindingsPerObject(){return r._bindings.length}}}function Ch(e,t,n,r){this._mixer=e,this._clip=t,this._localRoot=n||null,this.blendMode=r||t.blendMode;for(var i=t.tracks,o=i.length,a=new Array(o),s={endingStart:Wt,endingEnd:Wt},l=0;l!==o;++l){var u=i[l].createInterpolant(null);a[l]=u,u.settings=s}this._interpolantSettings=s,this._interpolants=a,this._propertyBindings=new Array(o),this._cacheIndex=null,this._byClipCacheIndex=null,this._timeScaleInterpolant=null,this._weightInterpolant=null,this.loop=Bt,this._loopCount=-1,this._startTime=null,this.time=0,this.timeScale=1,this._effectiveTimeScale=1,this.weight=1,this._effectiveWeight=1,this.repetitions=1/0,this.paused=!1,this.enabled=!0,this.clampWhenFinished=!1,this.zeroSlopeAtStart=!0,this.zeroSlopeAtEnd=!0}function Ah(e){this._root=e,this._initMemoryManager(),this._accuIndex=0,this.time=0,this.timeScale=1}function _h(e){"string"==typeof e&&(console.warn("THREE.Uniform: Type parameter is no longer needed."),e=arguments[1]),this.value=e}function Ih(e,t,n){zs.call(this,e,t),this.meshPerAttribute=n||1}function xh(e,t,n,r){this.ray=new Kr(e,t),this.near=n||0,this.far=r||1/0,this.camera=null,this.layers=new dr,this.params={Mesh:{},Line:{threshold:1},LOD:{},Points:{threshold:1},Sprite:{}},Object.defineProperties(this.params,{PointCloud:{get:function(){return console.warn("THREE.Raycaster: params.PointCloud has been renamed to params.Points."),this.Points}}})}function wh(e,t){return e.distance-t.distance}function Mh(e,t,n,r){if(e.layers.test(t.layers)&&e.raycast(t,n),!0===r)for(var i=e.children,o=0,a=i.length;o<a;o++)Mh(i[o],t,n,!0)}function Sh(e,t,n){return this.radius=void 0!==e?e:1,this.phi=void 0!==t?t:0,this.theta=void 0!==n?n:0,this}function Th(e,t,n){return this.radius=void 0!==e?e:1,this.theta=void 0!==t?t:0,this.y=void 0!==n?n:0,this}Object.assign(mh.prototype,{getValue:function(e,t){this.bind();var n=this._targetGroup.nCachedObjects_,r=this._bindings[n];void 0!==r&&r.getValue(e,t)},setValue:function(e,t){for(var n=this._bindings,r=this._targetGroup.nCachedObjects_,i=n.length;r!==i;++r)n[r].setValue(e,t)},bind:function(){for(var e=this._bindings,t=this._targetGroup.nCachedObjects_,n=e.length;t!==n;++t)e[t].bind()},unbind:function(){for(var e=this._bindings,t=this._targetGroup.nCachedObjects_,n=e.length;t!==n;++t)e[t].unbind()}}),Object.assign(yh,{Composite:mh,create:function(e,t,n){return e&&e.isAnimationObjectGroup?new yh.Composite(e,t,n):new yh(e,t,n)},sanitizeNodeName:function(e){return e.replace(/\s/g,"_").replace(uh,"")},parseTrackName:function(e){var t=gh.exec(e);if(!t)throw new Error("PropertyBinding: Cannot parse trackName: "+e);var n={nodeName:t[2],objectName:t[3],objectIndex:t[4],propertyName:t[5],propertyIndex:t[6]},r=n.nodeName&&n.nodeName.lastIndexOf(".");if(void 0!==r&&-1!==r){var i=n.nodeName.substring(r+1);-1!==vh.indexOf(i)&&(n.nodeName=n.nodeName.substring(0,r),n.objectName=i)}if(null===n.propertyName||0===n.propertyName.length)throw new Error("PropertyBinding: can not parse propertyName from trackName: "+e);return n},findNode:function(e,t){if(!t||""===t||"."===t||-1===t||t===e.name||t===e.uuid)return e;if(e.skeleton){var n=e.skeleton.getBoneByName(t);if(void 0!==n)return n}if(e.children){var r=function(e){for(var n=0;n<e.length;n++){var i=e[n];if(i.name===t||i.uuid===t)return i;var o=r(i.children);if(o)return o}return null},i=r(e.children);if(i)return i}return null}}),Object.assign(yh.prototype,{_getValue_unavailable:function(){},_setValue_unavailable:function(){},BindingType:{Direct:0,EntireArray:1,ArrayElement:2,HasFromToArray:3},Versioning:{None:0,NeedsUpdate:1,MatrixWorldNeedsUpdate:2},GetterByBindingType:[function(e,t){e[t]=this.node[this.propertyName]},function(e,t){for(var n=this.resolvedProperty,r=0,i=n.length;r!==i;++r)e[t++]=n[r]},function(e,t){e[t]=this.resolvedProperty[this.propertyIndex]},function(e,t){this.resolvedProperty.toArray(e,t)}],SetterByBindingTypeAndVersioning:[[function(e,t){this.targetObject[this.propertyName]=e[t]},function(e,t){this.targetObject[this.propertyName]=e[t],this.targetObject.needsUpdate=!0},function(e,t){this.targetObject[this.propertyName]=e[t],this.targetObject.matrixWorldNeedsUpdate=!0}],[function(e,t){for(var n=this.resolvedProperty,r=0,i=n.length;r!==i;++r)n[r]=e[t++]},function(e,t){for(var n=this.resolvedProperty,r=0,i=n.length;r!==i;++r)n[r]=e[t++];this.targetObject.needsUpdate=!0},function(e,t){for(var n=this.resolvedProperty,r=0,i=n.length;r!==i;++r)n[r]=e[t++];this.targetObject.matrixWorldNeedsUpdate=!0}],[function(e,t){this.resolvedProperty[this.propertyIndex]=e[t]},function(e,t){this.resolvedProperty[this.propertyIndex]=e[t],this.targetObject.needsUpdate=!0},function(e,t){this.resolvedProperty[this.propertyIndex]=e[t],this.targetObject.matrixWorldNeedsUpdate=!0}],[function(e,t){this.resolvedProperty.fromArray(e,t)},function(e,t){this.resolvedProperty.fromArray(e,t),this.targetObject.needsUpdate=!0},function(e,t){this.resolvedProperty.fromArray(e,t),this.targetObject.matrixWorldNeedsUpdate=!0}]],getValue:function(e,t){this.bind(),this.getValue(e,t)},setValue:function(e,t){this.bind(),this.setValue(e,t)},bind:function(){var e=this.node,t=this.parsedPath,n=t.objectName,r=t.propertyName,i=t.propertyIndex;if(e||(e=yh.findNode(this.rootNode,t.nodeName)||this.rootNode,this.node=e),this.getValue=this._getValue_unavailable,this.setValue=this._setValue_unavailable,e){if(n){var o=t.objectIndex;switch(n){case"materials":if(!e.material)return void console.error("THREE.PropertyBinding: Can not bind to material as node does not have a material.",this);if(!e.material.materials)return void console.error("THREE.PropertyBinding: Can not bind to material.materials as node.material does not have a materials array.",this);e=e.material.materials;break;case"bones":if(!e.skeleton)return void console.error("THREE.PropertyBinding: Can not bind to bones as node does not have a skeleton.",this);e=e.skeleton.bones;for(var a=0;a<e.length;a++)if(e[a].name===o){o=a;break}break;default:if(void 0===e[n])return void console.error("THREE.PropertyBinding: Can not bind to objectName of node undefined.",this);e=e[n]}if(void 0!==o){if(void 0===e[o])return void console.error("THREE.PropertyBinding: Trying to bind to objectIndex of objectName, but is undefined.",this,e);e=e[o]}}var s=e[r];if(void 0!==s){var l=this.Versioning.None;this.targetObject=e,void 0!==e.needsUpdate?l=this.Versioning.NeedsUpdate:void 0!==e.matrixWorldNeedsUpdate&&(l=this.Versioning.MatrixWorldNeedsUpdate);var u=this.BindingType.Direct;if(void 0!==i){if("morphTargetInfluences"===r){if(!e.geometry)return void console.error("THREE.PropertyBinding: Can not bind to morphTargetInfluences because node does not have a geometry.",this);if(!e.geometry.isBufferGeometry)return void console.error("THREE.PropertyBinding: Can not bind to morphTargetInfluences on THREE.Geometry. Use THREE.BufferGeometry instead.",this);if(!e.geometry.morphAttributes)return void console.error("THREE.PropertyBinding: Can not bind to morphTargetInfluences because node does not have a geometry.morphAttributes.",this);void 0!==e.morphTargetDictionary[i]&&(i=e.morphTargetDictionary[i])}u=this.BindingType.ArrayElement,this.resolvedProperty=s,this.propertyIndex=i}else void 0!==s.fromArray&&void 0!==s.toArray?(u=this.BindingType.HasFromToArray,this.resolvedProperty=s):Array.isArray(s)?(u=this.BindingType.EntireArray,this.resolvedProperty=s):this.propertyName=r;this.getValue=this.GetterByBindingType[u],this.setValue=this.SetterByBindingTypeAndVersioning[u][l]}else{var c=t.nodeName;console.error("THREE.PropertyBinding: Trying to update property for track: "+c+"."+r+" but it wasn't found.",e)}}else console.error("THREE.PropertyBinding: Trying to update node for track: "+this.path+" but it wasn't found.")},unbind:function(){this.node=null,this.getValue=this._getValue_unbound,this.setValue=this._setValue_unbound}}),Object.assign(yh.prototype,{_getValue_unbound:yh.prototype.getValue,_setValue_unbound:yh.prototype.setValue}),Object.assign(bh.prototype,{isAnimationObjectGroup:!0,add:function(){for(var e=this._objects,t=e.length,n=this.nCachedObjects_,r=this._indicesByUUID,i=this._paths,o=this._parsedPaths,a=this._bindings,s=a.length,l=void 0,u=0,c=arguments.length;u!==c;++u){var d=arguments[u],h=d.uuid,f=r[h];if(void 0===f){f=t++,r[h]=f,e.push(d);for(var p=0,g=s;p!==g;++p)a[p].push(new yh(d,i[p],o[p]))}else if(f<n){l=e[f];var v=--n,m=e[v];r[m.uuid]=f,e[f]=m,r[h]=v,e[v]=d;for(p=0,g=s;p!==g;++p){var y=a[p],b=y[v],C=y[f];y[f]=b,void 0===C&&(C=new yh(d,i[p],o[p])),y[v]=C}}else e[f]!==l&&console.error("THREE.AnimationObjectGroup: Different objects with the same UUID detected. Clean the caches or recreate your infrastructure when reloading scenes.")}this.nCachedObjects_=n},remove:function(){for(var e=this._objects,t=this.nCachedObjects_,n=this._indicesByUUID,r=this._bindings,i=r.length,o=0,a=arguments.length;o!==a;++o){var s=arguments[o],l=s.uuid,u=n[l];if(void 0!==u&&u>=t){var c=t++,d=e[c];n[d.uuid]=u,e[u]=d,n[l]=c,e[c]=s;for(var h=0,f=i;h!==f;++h){var p=r[h],g=p[c],v=p[u];p[u]=g,p[c]=v}}}this.nCachedObjects_=t},uncache:function(){for(var e=this._objects,t=e.length,n=this.nCachedObjects_,r=this._indicesByUUID,i=this._bindings,o=i.length,a=0,s=arguments.length;a!==s;++a){var l=arguments[a],u=l.uuid,c=r[u];if(void 0!==c)if(delete r[u],c<n){var d=--n,h=e[d],f=e[y=--t];r[h.uuid]=c,e[c]=h,r[f.uuid]=d,e[d]=f,e.pop();for(var p=0,g=o;p!==g;++p){var v=(b=i[p])[d],m=b[y];b[c]=v,b[d]=m,b.pop()}}else{var y;r[(f=e[y=--t]).uuid]=c,e[c]=f,e.pop();for(p=0,g=o;p!==g;++p){var b;(b=i[p])[c]=b[y],b.pop()}}}this.nCachedObjects_=n},subscribe_:function(e,t){var n=this._bindingsIndicesByPath,r=n[e],i=this._bindings;if(void 0!==r)return i[r];var o=this._paths,a=this._parsedPaths,s=this._objects,l=s.length,u=this.nCachedObjects_,c=new Array(l);r=i.length,n[e]=r,o.push(e),a.push(t),i.push(c);for(var d=u,h=s.length;d!==h;++d){var f=s[d];c[d]=new yh(f,e,t)}return c},unsubscribe_:function(e){var t=this._bindingsIndicesByPath,n=t[e];if(void 0!==n){var r=this._paths,i=this._parsedPaths,o=this._bindings,a=o.length-1,s=o[a];t[e[a]]=n,o[n]=s,o.pop(),i[n]=i[a],i.pop(),r[n]=r[a],r.pop()}}}),Object.assign(Ch.prototype,{play:function(){return this._mixer._activateAction(this),this},stop:function(){return this._mixer._deactivateAction(this),this.reset()},reset:function(){return this.paused=!1,this.enabled=!0,this.time=0,this._loopCount=-1,this._startTime=null,this.stopFading().stopWarping()},isRunning:function(){return this.enabled&&!this.paused&&0!==this.timeScale&&null===this._startTime&&this._mixer._isActiveAction(this)},isScheduled:function(){return this._mixer._isActiveAction(this)},startAt:function(e){return this._startTime=e,this},setLoop:function(e,t){return this.loop=e,this.repetitions=t,this},setEffectiveWeight:function(e){return this.weight=e,this._effectiveWeight=this.enabled?e:0,this.stopFading()},getEffectiveWeight:function(){return this._effectiveWeight},fadeIn:function(e){return this._scheduleFading(e,0,1)},fadeOut:function(e){return this._scheduleFading(e,1,0)},crossFadeFrom:function(e,t,n){if(e.fadeOut(t),this.fadeIn(t),n){var r=this._clip.duration,i=e._clip.duration,o=i/r,a=r/i;e.warp(1,o,t),this.warp(a,1,t)}return this},crossFadeTo:function(e,t,n){return e.crossFadeFrom(this,t,n)},stopFading:function(){var e=this._weightInterpolant;return null!==e&&(this._weightInterpolant=null,this._mixer._takeBackControlInterpolant(e)),this},setEffectiveTimeScale:function(e){return this.timeScale=e,this._effectiveTimeScale=this.paused?0:e,this.stopWarping()},getEffectiveTimeScale:function(){return this._effectiveTimeScale},setDuration:function(e){return this.timeScale=this._clip.duration/e,this.stopWarping()},syncWith:function(e){return this.time=e.time,this.timeScale=e.timeScale,this.stopWarping()},halt:function(e){return this.warp(this._effectiveTimeScale,0,e)},warp:function(e,t,n){var r=this._mixer,i=r.time,o=this._timeScaleInterpolant,a=this.timeScale;null===o&&(o=r._lendControlInterpolant(),this._timeScaleInterpolant=o);var s=o.parameterPositions,l=o.sampleValues;return s[0]=i,s[1]=i+n,l[0]=e/a,l[1]=t/a,this},stopWarping:function(){var e=this._timeScaleInterpolant;return null!==e&&(this._timeScaleInterpolant=null,this._mixer._takeBackControlInterpolant(e)),this},getMixer:function(){return this._mixer},getClip:function(){return this._clip},getRoot:function(){return this._localRoot||this._mixer._root},_update:function(e,t,n,r){if(this.enabled){var i=this._startTime;if(null!==i){var o=(e-i)*n;if(o<0||0===n)return;this._startTime=null,t=n*o}t*=this._updateTimeScale(e);var a=this._updateTime(t),s=this._updateWeight(e);if(s>0){var l=this._interpolants,u=this._propertyBindings;switch(this.blendMode){case Zt:for(var c=0,d=l.length;c!==d;++c)l[c].evaluate(a),u[c].accumulateAdditive(s);break;case qt:default:for(c=0,d=l.length;c!==d;++c)l[c].evaluate(a),u[c].accumulate(r,s)}}}else this._updateWeight(e)},_updateWeight:function(e){var t=0;if(this.enabled){t=this.weight;var n=this._weightInterpolant;if(null!==n){var r=n.evaluate(e)[0];t*=r,e>n.parameterPositions[1]&&(this.stopFading(),0===r&&(this.enabled=!1))}}return this._effectiveWeight=t,t},_updateTimeScale:function(e){var t=0;if(!this.paused){t=this.timeScale;var n=this._timeScaleInterpolant;if(null!==n)t*=n.evaluate(e)[0],e>n.parameterPositions[1]&&(this.stopWarping(),0===t?this.paused=!0:this.timeScale=t)}return this._effectiveTimeScale=t,t},_updateTime:function(e){var t=this.time+e,n=this._clip.duration,r=this.loop,i=this._loopCount,o=r===Ht;if(0===e)return-1===i?t:o&&1==(1&i)?n-t:t;if(r===Ft){-1===i&&(this._loopCount=0,this._setEndings(!0,!0,!1));e:{if(t>=n)t=n;else{if(!(t<0)){this.time=t;break e}t=0}this.clampWhenFinished?this.paused=!0:this.enabled=!1,this.time=t,this._mixer.dispatchEvent({type:"finished",action:this,direction:e<0?-1:1})}}else{if(-1===i&&(e>=0?(i=0,this._setEndings(!0,0===this.repetitions,o)):this._setEndings(0===this.repetitions,!0,o)),t>=n||t<0){var a=Math.floor(t/n);t-=n*a,i+=Math.abs(a);var s=this.repetitions-i;if(s<=0)this.clampWhenFinished?this.paused=!0:this.enabled=!1,t=e>0?n:0,this.time=t,this._mixer.dispatchEvent({type:"finished",action:this,direction:e>0?1:-1});else{if(1===s){var l=e<0;this._setEndings(l,!l,o)}else this._setEndings(!1,!1,o);this._loopCount=i,this.time=t,this._mixer.dispatchEvent({type:"loop",action:this,loopDelta:a})}}else this.time=t;if(o&&1==(1&i))return n-t}return t},_setEndings:function(e,t,n){var r=this._interpolantSettings;n?(r.endingStart=Gt,r.endingEnd=Gt):(r.endingStart=e?this.zeroSlopeAtStart?Gt:Wt:Xt,r.endingEnd=t?this.zeroSlopeAtEnd?Gt:Wt:Xt)},_scheduleFading:function(e,t,n){var r=this._mixer,i=r.time,o=this._weightInterpolant;null===o&&(o=r._lendControlInterpolant(),this._weightInterpolant=o);var a=o.parameterPositions,s=o.sampleValues;return a[0]=i,s[0]=t,a[1]=i+e,s[1]=n,this}}),Ah.prototype=Object.assign(Object.create(Rn.prototype),{constructor:Ah,_bindAction:function(e,t){var n=e._localRoot||this._root,r=e._clip.tracks,i=r.length,o=e._propertyBindings,a=e._interpolants,s=n.uuid,l=this._bindingsByRootAndName,u=l[s];void 0===u&&(u={},l[s]=u);for(var c=0;c!==i;++c){var d=r[c],h=d.name,f=u[h];if(void 0!==f)o[c]=f;else{if(void 0!==(f=o[c])){null===f._cacheIndex&&(++f.referenceCount,this._addInactiveBinding(f,s,h));continue}var p=t&&t._propertyBindings[c].binding.parsedPath;++(f=new lh(yh.create(n,h,p),d.ValueTypeName,d.getValueSize())).referenceCount,this._addInactiveBinding(f,s,h),o[c]=f}a[c].resultBuffer=f.buffer}},_activateAction:function(e){if(!this._isActiveAction(e)){if(null===e._cacheIndex){var t=(e._localRoot||this._root).uuid,n=e._clip.uuid,r=this._actionsByClip[n];this._bindAction(e,r&&r.knownActions[0]),this._addInactiveAction(e,n,t)}for(var i=e._propertyBindings,o=0,a=i.length;o!==a;++o){var s=i[o];0==s.useCount++&&(this._lendBinding(s),s.saveOriginalState())}this._lendAction(e)}},_deactivateAction:function(e){if(this._isActiveAction(e)){for(var t=e._propertyBindings,n=0,r=t.length;n!==r;++n){var i=t[n];0==--i.useCount&&(i.restoreOriginalState(),this._takeBackBinding(i))}this._takeBackAction(e)}},_initMemoryManager:function(){this._actions=[],this._nActiveActions=0,this._actionsByClip={},this._bindings=[],this._nActiveBindings=0,this._bindingsByRootAndName={},this._controlInterpolants=[],this._nActiveControlInterpolants=0;var e=this;this.stats={actions:{get total(){return e._actions.length},get inUse(){return e._nActiveActions}},bindings:{get total(){return e._bindings.length},get inUse(){return e._nActiveBindings}},controlInterpolants:{get total(){return e._controlInterpolants.length},get inUse(){return e._nActiveControlInterpolants}}}},_isActiveAction:function(e){var t=e._cacheIndex;return null!==t&&t<this._nActiveActions},_addInactiveAction:function(e,t,n){var r=this._actions,i=this._actionsByClip,o=i[t];if(void 0===o)o={knownActions:[e],actionByRoot:{}},e._byClipCacheIndex=0,i[t]=o;else{var a=o.knownActions;e._byClipCacheIndex=a.length,a.push(e)}e._cacheIndex=r.length,r.push(e),o.actionByRoot[n]=e},_removeInactiveAction:function(e){var t=this._actions,n=t[t.length-1],r=e._cacheIndex;n._cacheIndex=r,t[r]=n,t.pop(),e._cacheIndex=null;var i=e._clip.uuid,o=this._actionsByClip,a=o[i],s=a.knownActions,l=s[s.length-1],u=e._byClipCacheIndex;l._byClipCacheIndex=u,s[u]=l,s.pop(),e._byClipCacheIndex=null,delete a.actionByRoot[(e._localRoot||this._root).uuid],0===s.length&&delete o[i],this._removeInactiveBindingsForAction(e)},_removeInactiveBindingsForAction:function(e){for(var t=e._propertyBindings,n=0,r=t.length;n!==r;++n){var i=t[n];0==--i.referenceCount&&this._removeInactiveBinding(i)}},_lendAction:function(e){var t=this._actions,n=e._cacheIndex,r=this._nActiveActions++,i=t[r];e._cacheIndex=r,t[r]=e,i._cacheIndex=n,t[n]=i},_takeBackAction:function(e){var t=this._actions,n=e._cacheIndex,r=--this._nActiveActions,i=t[r];e._cacheIndex=r,t[r]=e,i._cacheIndex=n,t[n]=i},_addInactiveBinding:function(e,t,n){var r=this._bindingsByRootAndName,i=r[t],o=this._bindings;void 0===i&&(i={},r[t]=i),i[n]=e,e._cacheIndex=o.length,o.push(e)},_removeInactiveBinding:function(e){var t=this._bindings,n=e.binding,r=n.rootNode.uuid,i=n.path,o=this._bindingsByRootAndName,a=o[r],s=t[t.length-1],l=e._cacheIndex;s._cacheIndex=l,t[l]=s,t.pop(),delete a[i],0===Object.keys(a).length&&delete o[r]},_lendBinding:function(e){var t=this._bindings,n=e._cacheIndex,r=this._nActiveBindings++,i=t[r];e._cacheIndex=r,t[r]=e,i._cacheIndex=n,t[n]=i},_takeBackBinding:function(e){var t=this._bindings,n=e._cacheIndex,r=--this._nActiveBindings,i=t[r];e._cacheIndex=r,t[r]=e,i._cacheIndex=n,t[n]=i},_lendControlInterpolant:function(){var e=this._controlInterpolants,t=this._nActiveControlInterpolants++,n=e[t];return void 0===n&&((n=new bc(new Float32Array(2),new Float32Array(2),1,this._controlInterpolantsResultBuffer)).__cacheIndex=t,e[t]=n),n},_takeBackControlInterpolant:function(e){var t=this._controlInterpolants,n=e.__cacheIndex,r=--this._nActiveControlInterpolants,i=t[r];e.__cacheIndex=r,t[r]=e,i.__cacheIndex=n,t[n]=i},_controlInterpolantsResultBuffer:new Float32Array(1),clipAction:function(e,t,n){var r=t||this._root,i=r.uuid,o="string"==typeof e?Ec.findByName(r,e):e,a=null!==o?o.uuid:e,s=this._actionsByClip[a],l=null;if(void 0===n&&(n=null!==o?o.blendMode:qt),void 0!==s){var u=s.actionByRoot[i];if(void 0!==u&&u.blendMode===n)return u;l=s.knownActions[0],null===o&&(o=l._clip)}if(null===o)return null;var c=new Ch(this,o,t,n);return this._bindAction(c,l),this._addInactiveAction(c,a,i),c},existingAction:function(e,t){var n=t||this._root,r=n.uuid,i="string"==typeof e?Ec.findByName(n,e):e,o=i?i.uuid:e,a=this._actionsByClip[o];return void 0!==a&&a.actionByRoot[r]||null},stopAllAction:function(){for(var e=this._actions,t=this._nActiveActions-1;t>=0;--t)e[t].stop();return this},update:function(e){e*=this.timeScale;for(var t=this._actions,n=this._nActiveActions,r=this.time+=e,i=Math.sign(e),o=this._accuIndex^=1,a=0;a!==n;++a){t[a]._update(r,e,i,o)}var s=this._bindings,l=this._nActiveBindings;for(a=0;a!==l;++a)s[a].apply(o);return this},setTime:function(e){this.time=0;for(var t=0;t<this._actions.length;t++)this._actions[t].time=0;return this.update(e)},getRoot:function(){return this._root},uncacheClip:function(e){var t=this._actions,n=e.uuid,r=this._actionsByClip,i=r[n];if(void 0!==i){for(var o=i.knownActions,a=0,s=o.length;a!==s;++a){var l=o[a];this._deactivateAction(l);var u=l._cacheIndex,c=t[t.length-1];l._cacheIndex=null,l._byClipCacheIndex=null,c._cacheIndex=u,t[u]=c,t.pop(),this._removeInactiveBindingsForAction(l)}delete r[n]}},uncacheRoot:function(e){var t=e.uuid,n=this._actionsByClip;for(var r in n){var i=n[r].actionByRoot[t];void 0!==i&&(this._deactivateAction(i),this._removeInactiveAction(i))}var o=this._bindingsByRootAndName[t];if(void 0!==o)for(var a in o){var s=o[a];s.restoreOriginalState(),this._removeInactiveBinding(s)}},uncacheAction:function(e,t){var n=this.existingAction(e,t);null!==n&&(this._deactivateAction(n),this._removeInactiveAction(n))}}),_h.prototype.clone=function(){return new _h(void 0===this.value.clone?this.value:this.value.clone())},Ih.prototype=Object.assign(Object.create(zs.prototype),{constructor:Ih,isInstancedInterleavedBuffer:!0,copy:function(e){return zs.prototype.copy.call(this,e),this.meshPerAttribute=e.meshPerAttribute,this}}),Object.assign(xh.prototype,{set:function(e,t){this.ray.set(e,t)},setFromCamera:function(e,t){t&&t.isPerspectiveCamera?(this.ray.origin.setFromMatrixPosition(t.matrixWorld),this.ray.direction.set(e.x,e.y,.5).unproject(t).sub(this.ray.origin).normalize(),this.camera=t):t&&t.isOrthographicCamera?(this.ray.origin.set(e.x,e.y,(t.near+t.far)/(t.near-t.far)).unproject(t),this.ray.direction.set(0,0,-1).transformDirection(t.matrixWorld),this.camera=t):console.error("THREE.Raycaster: Unsupported camera type.")},intersectObject:function(e,t,n){var r=n||[];return Mh(e,this,r,t),r.sort(wh),r},intersectObjects:function(e,t,n){var r=n||[];if(!1===Array.isArray(e))return console.warn("THREE.Raycaster.intersectObjects: objects is not an Array."),r;for(var i=0,o=e.length;i<o;i++)Mh(e[i],this,r,t);return r.sort(wh),r}}),Object.assign(Sh.prototype,{set:function(e,t,n){return this.radius=e,this.phi=t,this.theta=n,this},clone:function(){return(new this.constructor).copy(this)},copy:function(e){return this.radius=e.radius,this.phi=e.phi,this.theta=e.theta,this},makeSafe:function(){return this.phi=Math.max(1e-6,Math.min(Math.PI-1e-6,this.phi)),this},setFromVector3:function(e){return this.setFromCartesianCoords(e.x,e.y,e.z)},setFromCartesianCoords:function(e,t,n){return this.radius=Math.sqrt(e*e+t*t+n*n),0===this.radius?(this.theta=0,this.phi=0):(this.theta=Math.atan2(e,n),this.phi=Math.acos(Hn.clamp(t/this.radius,-1,1))),this}}),Object.assign(Th.prototype,{set:function(e,t,n){return this.radius=e,this.theta=t,this.y=n,this},clone:function(){return(new this.constructor).copy(this)},copy:function(e){return this.radius=e.radius,this.theta=e.theta,this.y=e.y,this},setFromVector3:function(e){return this.setFromCartesianCoords(e.x,e.y,e.z)},setFromCartesianCoords:function(e,t,n){return this.radius=Math.sqrt(e*e+n*n),this.theta=Math.atan2(e,n),this.y=t,this}});var Eh=new Un;function Oh(e,t){this.min=void 0!==e?e:new Un(1/0,1/0),this.max=void 0!==t?t:new Un(-1/0,-1/0)}Object.assign(Oh.prototype,{set:function(e,t){return this.min.copy(e),this.max.copy(t),this},setFromPoints:function(e){this.makeEmpty();for(var t=0,n=e.length;t<n;t++)this.expandByPoint(e[t]);return this},setFromCenterAndSize:function(e,t){var n=Eh.copy(t).multiplyScalar(.5);return this.min.copy(e).sub(n),this.max.copy(e).add(n),this},clone:function(){return(new this.constructor).copy(this)},copy:function(e){return this.min.copy(e.min),this.max.copy(e.max),this},makeEmpty:function(){return this.min.x=this.min.y=1/0,this.max.x=this.max.y=-1/0,this},isEmpty:function(){return this.max.x<this.min.x||this.max.y<this.min.y},getCenter:function(e){return void 0===e&&(console.warn("THREE.Box2: .getCenter() target is now required"),e=new Un),this.isEmpty()?e.set(0,0):e.addVectors(this.min,this.max).multiplyScalar(.5)},getSize:function(e){return void 0===e&&(console.warn("THREE.Box2: .getSize() target is now required"),e=new Un),this.isEmpty()?e.set(0,0):e.subVectors(this.max,this.min)},expandByPoint:function(e){return this.min.min(e),this.max.max(e),this},expandByVector:function(e){return this.min.sub(e),this.max.add(e),this},expandByScalar:function(e){return this.min.addScalar(-e),this.max.addScalar(e),this},containsPoint:function(e){return!(e.x<this.min.x||e.x>this.max.x||e.y<this.min.y||e.y>this.max.y)},containsBox:function(e){return this.min.x<=e.min.x&&e.max.x<=this.max.x&&this.min.y<=e.min.y&&e.max.y<=this.max.y},getParameter:function(e,t){return void 0===t&&(console.warn("THREE.Box2: .getParameter() target is now required"),t=new Un),t.set((e.x-this.min.x)/(this.max.x-this.min.x),(e.y-this.min.y)/(this.max.y-this.min.y))},intersectsBox:function(e){return!(e.max.x<this.min.x||e.min.x>this.max.x||e.max.y<this.min.y||e.min.y>this.max.y)},clampPoint:function(e,t){return void 0===t&&(console.warn("THREE.Box2: .clampPoint() target is now required"),t=new Un),t.copy(e).clamp(this.min,this.max)},distanceToPoint:function(e){return Eh.copy(e).clamp(this.min,this.max).sub(e).length()},intersect:function(e){return this.min.max(e.min),this.max.min(e.max),this},union:function(e){return this.min.min(e.min),this.max.max(e.max),this},translate:function(e){return this.min.add(e),this.max.add(e),this},equals:function(e){return e.min.equals(this.min)&&e.max.equals(this.max)}});var kh=new $n,Lh=new $n;function Dh(e,t){this.start=void 0!==e?e:new $n,this.end=void 0!==t?t:new $n}function Ph(e){wr.call(this),this.material=e,this.render=function(){},this.hasPositions=!1,this.hasNormals=!1,this.hasColors=!1,this.hasUvs=!1,this.positionArray=null,this.normalArray=null,this.colorArray=null,this.uvArray=null,this.count=0}Object.assign(Dh.prototype,{set:function(e,t){return this.start.copy(e),this.end.copy(t),this},clone:function(){return(new this.constructor).copy(this)},copy:function(e){return this.start.copy(e.start),this.end.copy(e.end),this},getCenter:function(e){return void 0===e&&(console.warn("THREE.Line3: .getCenter() target is now required"),e=new $n),e.addVectors(this.start,this.end).multiplyScalar(.5)},delta:function(e){return void 0===e&&(console.warn("THREE.Line3: .delta() target is now required"),e=new $n),e.subVectors(this.end,this.start)},distanceSq:function(){return this.start.distanceToSquared(this.end)},distance:function(){return this.start.distanceTo(this.end)},at:function(e,t){return void 0===t&&(console.warn("THREE.Line3: .at() target is now required"),t=new $n),this.delta(t).multiplyScalar(e).add(this.start)},closestPointToPointParameter:function(e,t){kh.subVectors(e,this.start),Lh.subVectors(this.end,this.start);var n=Lh.dot(Lh),r=Lh.dot(kh)/n;return t&&(r=Hn.clamp(r,0,1)),r},closestPointToPoint:function(e,t,n){var r=this.closestPointToPointParameter(e,t);return void 0===n&&(console.warn("THREE.Line3: .closestPointToPoint() target is now required"),n=new $n),this.delta(n).multiplyScalar(r).add(this.start)},applyMatrix4:function(e){return this.start.applyMatrix4(e),this.end.applyMatrix4(e),this},equals:function(e){return e.start.equals(this.start)&&e.end.equals(this.end)}}),Ph.prototype=Object.create(wr.prototype),Ph.prototype.constructor=Ph,Ph.prototype.isImmediateRenderObject=!0;var Nh=new $n;function jh(e,t){wr.call(this),this.light=e,this.light.updateMatrixWorld(),this.matrix=e.matrixWorld,this.matrixAutoUpdate=!1,this.color=t;for(var n=new Yi,r=[0,0,0,0,0,1,0,0,0,1,0,1,0,0,0,-1,0,1,0,0,0,0,1,1,0,0,0,0,-1,1],i=0,o=1;i<32;i++,o++){var a=i/32*Math.PI*2,s=o/32*Math.PI*2;r.push(Math.cos(a),Math.sin(a),1,Math.cos(s),Math.sin(s),1)}n.setAttribute("position",new Di(r,3));var l=new Al({fog:!1,toneMapped:!1});this.cone=new Ol(n,l),this.add(this.cone),this.update()}jh.prototype=Object.create(wr.prototype),jh.prototype.constructor=jh,jh.prototype.dispose=function(){this.cone.geometry.dispose(),this.cone.material.dispose()},jh.prototype.update=function(){this.light.updateMatrixWorld();var e=this.light.distance?this.light.distance:1e3,t=e*Math.tan(this.light.angle);this.cone.scale.set(t,t,e),Nh.setFromMatrixPosition(this.light.target.matrixWorld),this.cone.lookAt(Nh),void 0!==this.color?this.cone.material.color.set(this.color):this.cone.material.color.copy(this.light.color)};var Rh=new $n,zh=new sr,Fh=new sr;function Bh(e){for(var t=function e(t){var n=[];t&&t.isBone&&n.push(t);for(var r=0;r<t.children.length;r++)n.push.apply(n,e(t.children[r]));return n}(e),n=new Yi,r=[],i=[],o=new vi(0,0,1),a=new vi(0,1,0),s=0;s<t.length;s++){var l=t[s];l.parent&&l.parent.isBone&&(r.push(0,0,0),r.push(0,0,0),i.push(o.r,o.g,o.b),i.push(a.r,a.g,a.b))}n.setAttribute("position",new Di(r,3)),n.setAttribute("color",new Di(i,3));var u=new Al({vertexColors:!0,depthTest:!1,depthWrite:!1,toneMapped:!1,transparent:!0});Ol.call(this,n,u),this.type="SkeletonHelper",this.root=e,this.bones=t,this.matrix=e.matrixWorld,this.matrixAutoUpdate=!1}function Hh(e,t,n){this.light=e,this.light.updateMatrixWorld(),this.color=n;var r=new Vu(t,4,2),i=new Ii({wireframe:!0,fog:!1,toneMapped:!1});lo.call(this,r,i),this.type="PointLightHelper",this.matrix=this.light.matrixWorld,this.matrixAutoUpdate=!1,this.update()}Bh.prototype=Object.create(Ol.prototype),Bh.prototype.constructor=Bh,Bh.prototype.isSkeletonHelper=!0,Bh.prototype.updateMatrixWorld=function(e){var t=this.bones,n=this.geometry,r=n.getAttribute("position");Fh.getInverse(this.root.matrixWorld);for(var i=0,o=0;i<t.length;i++){var a=t[i];a.parent&&a.parent.isBone&&(zh.multiplyMatrices(Fh,a.matrixWorld),Rh.setFromMatrixPosition(zh),r.setXYZ(o,Rh.x,Rh.y,Rh.z),zh.multiplyMatrices(Fh,a.parent.matrixWorld),Rh.setFromMatrixPosition(zh),r.setXYZ(o+1,Rh.x,Rh.y,Rh.z),o+=2)}n.getAttribute("position").needsUpdate=!0,wr.prototype.updateMatrixWorld.call(this,e)},Hh.prototype=Object.create(lo.prototype),Hh.prototype.constructor=Hh,Hh.prototype.dispose=function(){this.geometry.dispose(),this.material.dispose()},Hh.prototype.update=function(){void 0!==this.color?this.material.color.set(this.color):this.material.color.copy(this.light.color)};var Uh=new $n,Vh=new vi,Yh=new vi;function Wh(e,t,n){wr.call(this),this.light=e,this.light.updateMatrixWorld(),this.matrix=e.matrixWorld,this.matrixAutoUpdate=!1,this.color=n;var r=new Kl(t);r.rotateY(.5*Math.PI),this.material=new Ii({wireframe:!0,fog:!1,toneMapped:!1}),void 0===this.color&&(this.material.vertexColors=!0);var i=r.getAttribute("position"),o=new Float32Array(3*i.count);r.setAttribute("color",new wi(o,3)),this.add(new lo(r,this.material)),this.update()}function Gh(e,t,n,r){e=e||10,t=t||10,n=new vi(void 0!==n?n:4473924),r=new vi(void 0!==r?r:8947848);for(var i=t/2,o=e/t,a=e/2,s=[],l=[],u=0,c=0,d=-a;u<=t;u++,d+=o){s.push(-a,0,d,a,0,d),s.push(d,0,-a,d,0,a);var h=u===i?n:r;h.toArray(l,c),c+=3,h.toArray(l,c),c+=3,h.toArray(l,c),c+=3,h.toArray(l,c),c+=3}var f=new Yi;f.setAttribute("position",new Di(s,3)),f.setAttribute("color",new Di(l,3));var p=new Al({vertexColors:!0,toneMapped:!1});Ol.call(this,f,p),this.type="GridHelper"}function Xh(e,t,n,r,i,o){e=e||10,t=t||16,n=n||8,r=r||64,i=new vi(void 0!==i?i:4473924),o=new vi(void 0!==o?o:8947848);var a,s,l,u,c,d,h,f=[],p=[];for(u=0;u<=t;u++)l=u/t*(2*Math.PI),a=Math.sin(l)*e,s=Math.cos(l)*e,f.push(0,0,0),f.push(a,0,s),h=1&u?i:o,p.push(h.r,h.g,h.b),p.push(h.r,h.g,h.b);for(u=0;u<=n;u++)for(h=1&u?i:o,d=e-e/n*u,c=0;c<r;c++)l=c/r*(2*Math.PI),a=Math.sin(l)*d,s=Math.cos(l)*d,f.push(a,0,s),p.push(h.r,h.g,h.b),l=(c+1)/r*(2*Math.PI),a=Math.sin(l)*d,s=Math.cos(l)*d,f.push(a,0,s),p.push(h.r,h.g,h.b);var g=new Yi;g.setAttribute("position",new Di(f,3)),g.setAttribute("color",new Di(p,3));var v=new Al({vertexColors:!0,toneMapped:!1});Ol.call(this,g,v),this.type="PolarGridHelper"}Wh.prototype=Object.create(wr.prototype),Wh.prototype.constructor=Wh,Wh.prototype.dispose=function(){this.children[0].geometry.dispose(),this.children[0].material.dispose()},Wh.prototype.update=function(){var e=this.children[0];if(void 0!==this.color)this.material.color.set(this.color);else{var t=e.geometry.getAttribute("color");Vh.copy(this.light.color),Yh.copy(this.light.groundColor);for(var n=0,r=t.count;n<r;n++){var i=n<r/2?Vh:Yh;t.setXYZ(n,i.r,i.g,i.b)}t.needsUpdate=!0}e.lookAt(Uh.setFromMatrixPosition(this.light.matrixWorld).negate())},Gh.prototype=Object.assign(Object.create(Ol.prototype),{constructor:Gh,copy:function(e){return Ol.prototype.copy.call(this,e),this.geometry.copy(e.geometry),this.material.copy(e.material),this},clone:function(){return(new this.constructor).copy(this)}}),Xh.prototype=Object.create(Ol.prototype),Xh.prototype.constructor=Xh;var qh=new $n,Zh=new $n,Qh=new $n;function Kh(e,t,n){wr.call(this),this.light=e,this.light.updateMatrixWorld(),this.matrix=e.matrixWorld,this.matrixAutoUpdate=!1,this.color=n,void 0===t&&(t=1);var r=new Yi;r.setAttribute("position",new Di([-t,t,0,t,t,0,t,-t,0,-t,-t,0,-t,t,0],3));var i=new Al({fog:!1,toneMapped:!1});this.lightPlane=new Sl(r,i),this.add(this.lightPlane),(r=new Yi).setAttribute("position",new Di([0,0,0,0,0,1],3)),this.targetLine=new Sl(r,i),this.add(this.targetLine),this.update()}Kh.prototype=Object.create(wr.prototype),Kh.prototype.constructor=Kh,Kh.prototype.dispose=function(){this.lightPlane.geometry.dispose(),this.lightPlane.material.dispose(),this.targetLine.geometry.dispose(),this.targetLine.material.dispose()},Kh.prototype.update=function(){qh.setFromMatrixPosition(this.light.matrixWorld),Zh.setFromMatrixPosition(this.light.target.matrixWorld),Qh.subVectors(Zh,qh),this.lightPlane.lookAt(Zh),void 0!==this.color?(this.lightPlane.material.color.set(this.color),this.targetLine.material.color.set(this.color)):(this.lightPlane.material.color.copy(this.light.color),this.targetLine.material.color.copy(this.light.color)),this.targetLine.lookAt(Zh),this.targetLine.scale.z=Qh.length()};var Jh=new $n,$h=new Io;function ef(e){var t=new Yi,n=new Al({color:16777215,vertexColors:!0,toneMapped:!1}),r=[],i=[],o={},a=new vi(16755200),s=new vi(16711680),l=new vi(43775),u=new vi(16777215),c=new vi(3355443);function d(e,t,n){h(e,n),h(t,n)}function h(e,t){r.push(0,0,0),i.push(t.r,t.g,t.b),void 0===o[e]&&(o[e]=[]),o[e].push(r.length/3-1)}d("n1","n2",a),d("n2","n4",a),d("n4","n3",a),d("n3","n1",a),d("f1","f2",a),d("f2","f4",a),d("f4","f3",a),d("f3","f1",a),d("n1","f1",a),d("n2","f2",a),d("n3","f3",a),d("n4","f4",a),d("p","n1",s),d("p","n2",s),d("p","n3",s),d("p","n4",s),d("u1","u2",l),d("u2","u3",l),d("u3","u1",l),d("c","t",u),d("p","c",c),d("cn1","cn2",c),d("cn3","cn4",c),d("cf1","cf2",c),d("cf3","cf4",c),t.setAttribute("position",new Di(r,3)),t.setAttribute("color",new Di(i,3)),Ol.call(this,t,n),this.type="CameraHelper",this.camera=e,this.camera.updateProjectionMatrix&&this.camera.updateProjectionMatrix(),this.matrix=e.matrixWorld,this.matrixAutoUpdate=!1,this.pointMap=o,this.update()}function tf(e,t,n,r,i,o,a){Jh.set(i,o,a).unproject(r);var s=t[e];if(void 0!==s)for(var l=n.getAttribute("position"),u=0,c=s.length;u<c;u++)l.setXYZ(s[u],Jh.x,Jh.y,Jh.z)}ef.prototype=Object.create(Ol.prototype),ef.prototype.constructor=ef,ef.prototype.update=function(){var e=this.geometry,t=this.pointMap;$h.projectionMatrixInverse.copy(this.camera.projectionMatrixInverse),tf("c",t,e,$h,0,0,-1),tf("t",t,e,$h,0,0,1),tf("n1",t,e,$h,-1,-1,-1),tf("n2",t,e,$h,1,-1,-1),tf("n3",t,e,$h,-1,1,-1),tf("n4",t,e,$h,1,1,-1),tf("f1",t,e,$h,-1,-1,1),tf("f2",t,e,$h,1,-1,1),tf("f3",t,e,$h,-1,1,1),tf("f4",t,e,$h,1,1,1),tf("u1",t,e,$h,.7,1.1,-1),tf("u2",t,e,$h,-.7,1.1,-1),tf("u3",t,e,$h,0,2,-1),tf("cf1",t,e,$h,-1,0,1),tf("cf2",t,e,$h,1,0,1),tf("cf3",t,e,$h,0,-1,1),tf("cf4",t,e,$h,0,1,1),tf("cn1",t,e,$h,-1,0,-1),tf("cn2",t,e,$h,1,0,-1),tf("cn3",t,e,$h,0,-1,-1),tf("cn4",t,e,$h,0,1,-1),e.getAttribute("position").needsUpdate=!0};var nf=new Br;function rf(e,t){this.object=e,void 0===t&&(t=16776960);var n=new Uint16Array([0,1,1,2,2,3,3,0,4,5,5,6,6,7,7,4,0,4,1,5,2,6,3,7]),r=new Float32Array(24),i=new Yi;i.setIndex(new wi(n,1)),i.setAttribute("position",new wi(r,3)),Ol.call(this,i,new Al({color:t,toneMapped:!1})),this.type="BoxHelper",this.matrixAutoUpdate=!1,this.update()}function of(e,t){this.type="Box3Helper",this.box=e,t=t||16776960;var n=new Uint16Array([0,1,1,2,2,3,3,0,4,5,5,6,6,7,7,4,0,4,1,5,2,6,3,7]),r=new Yi;r.setIndex(new wi(n,1)),r.setAttribute("position",new Di([1,1,1,-1,1,1,-1,-1,1,1,-1,1,1,1,-1,-1,1,-1,-1,-1,-1,1,-1,-1],3)),Ol.call(this,r,new Al({color:t,toneMapped:!1})),this.type="Box3Helper",this.geometry.computeBoundingSphere()}function af(e,t,n){this.plane=e,this.size=void 0===t?1:t;var r=void 0!==n?n:16776960,i=new Yi;i.setAttribute("position",new Di([1,-1,1,-1,1,1,-1,-1,1,1,1,1,-1,1,1,-1,-1,1,1,-1,1,1,1,1,0,0,1,0,0,0],3)),i.computeBoundingSphere(),Sl.call(this,i,new Al({color:r,toneMapped:!1})),this.type="PlaneHelper";var o=new Yi;o.setAttribute("position",new Di([1,1,1,-1,1,1,-1,-1,1,1,1,1,-1,-1,1,1,-1,1],3)),o.computeBoundingSphere(),this.add(new lo(o,new Ii({color:r,opacity:.2,transparent:!0,depthWrite:!1,toneMapped:!1})))}rf.prototype=Object.create(Ol.prototype),rf.prototype.constructor=rf,rf.prototype.update=function(e){if(void 0!==e&&console.warn("THREE.BoxHelper: .update() has no longer arguments."),void 0!==this.object&&nf.setFromObject(this.object),!nf.isEmpty()){var t=nf.min,n=nf.max,r=this.geometry.attributes.position,i=r.array;i[0]=n.x,i[1]=n.y,i[2]=n.z,i[3]=t.x,i[4]=n.y,i[5]=n.z,i[6]=t.x,i[7]=t.y,i[8]=n.z,i[9]=n.x,i[10]=t.y,i[11]=n.z,i[12]=n.x,i[13]=n.y,i[14]=t.z,i[15]=t.x,i[16]=n.y,i[17]=t.z,i[18]=t.x,i[19]=t.y,i[20]=t.z,i[21]=n.x,i[22]=t.y,i[23]=t.z,r.needsUpdate=!0,this.geometry.computeBoundingSphere()}},rf.prototype.setFromObject=function(e){return this.object=e,this.update(),this},rf.prototype.copy=function(e){return Ol.prototype.copy.call(this,e),this.object=e.object,this},rf.prototype.clone=function(){return(new this.constructor).copy(this)},of.prototype=Object.create(Ol.prototype),of.prototype.constructor=of,of.prototype.updateMatrixWorld=function(e){var t=this.box;t.isEmpty()||(t.getCenter(this.position),t.getSize(this.scale),this.scale.multiplyScalar(.5),wr.prototype.updateMatrixWorld.call(this,e))},af.prototype=Object.create(Sl.prototype),af.prototype.constructor=af,af.prototype.updateMatrixWorld=function(e){var t=-this.plane.constant;Math.abs(t)<1e-8&&(t=1e-8),this.scale.set(.5*this.size,.5*this.size,t),this.children[0].material.side=t<0?m:v,this.lookAt(this.plane.normal),wr.prototype.updateMatrixWorld.call(this,e)};var sf,lf,uf=new $n;function cf(e,t,n,r,i,o){wr.call(this),this.type="ArrowHelper",void 0===e&&(e=new $n(0,0,1)),void 0===t&&(t=new $n(0,0,0)),void 0===n&&(n=1),void 0===r&&(r=16776960),void 0===i&&(i=.2*n),void 0===o&&(o=.2*i),void 0===sf&&((sf=new Yi).setAttribute("position",new Di([0,0,0,0,1,0],3)),(lf=new $u(0,.5,1,5,1)).translate(0,-.5,0)),this.position.copy(t),this.line=new Sl(sf,new Al({color:r,toneMapped:!1})),this.line.matrixAutoUpdate=!1,this.add(this.line),this.cone=new lo(lf,new Ii({color:r,toneMapped:!1})),this.cone.matrixAutoUpdate=!1,this.add(this.cone),this.setDirection(e),this.setLength(n,i,o)}function df(e){var t=[0,0,0,e=e||1,0,0,0,0,0,0,e,0,0,0,0,0,0,e],n=new Yi;n.setAttribute("position",new Di(t,3)),n.setAttribute("color",new Di([1,0,0,1,.6,0,0,1,0,.6,1,0,0,0,1,0,.6,1],3));var r=new Al({vertexColors:!0,toneMapped:!1});Ol.call(this,n,r),this.type="AxesHelper"}cf.prototype=Object.create(wr.prototype),cf.prototype.constructor=cf,cf.prototype.setDirection=function(e){if(e.y>.99999)this.quaternion.set(0,0,0,1);else if(e.y<-.99999)this.quaternion.set(1,0,0,0);else{uf.set(e.z,0,-e.x).normalize();var t=Math.acos(e.y);this.quaternion.setFromAxisAngle(uf,t)}},cf.prototype.setLength=function(e,t,n){void 0===t&&(t=.2*e),void 0===n&&(n=.2*t),this.line.scale.set(1,Math.max(1e-4,e-t),1),this.line.updateMatrix(),this.cone.scale.set(n,t,n),this.cone.position.y=e,this.cone.updateMatrix()},cf.prototype.setColor=function(e){this.line.material.color.set(e),this.cone.material.color.set(e)},cf.prototype.copy=function(e){return wr.prototype.copy.call(this,e,!1),this.line.copy(e.line),this.cone.copy(e.cone),this},cf.prototype.clone=function(){return(new this.constructor).copy(this)},df.prototype=Object.create(Ol.prototype),df.prototype.constructor=df;var hf=Math.pow(2,8),ff=[.125,.215,.35,.446,.526,.582],pf=5+ff.length,gf={[$t]:0,[en]:1,[nn]:2,[on]:3,[an]:4,[sn]:5,[tn]:6},vf=new bd,{_lodPlanes:mf,_sizeLods:yf,_sigmas:bf}=function(){for(var e=[],t=[],n=[],r=8,i=0;i<pf;i++){var o=Math.pow(2,r);t.push(o);var a=1/o;i>4?a=ff[i-8+4-1]:0==i&&(a=0),n.push(a);for(var s=1/(o-1),l=-s/2,u=1+s/2,c=[l,l,u,l,u,u,l,l,u,u,l,u],d=new Float32Array(108),h=new Float32Array(72),f=new Float32Array(36),p=0;p<6;p++){var g=p%3*2/3-1,v=p>2?0:-1,m=[g,v,0,g+2/3,v,0,g+2/3,v+1,0,g,v,0,g+2/3,v+1,0,g,v+1,0];d.set(m,18*p),h.set(c,12*p);var y=[p,p,p,p,p,p];f.set(y,6*p)}var b=new Yi;b.setAttribute("position",new wi(d,3)),b.setAttribute("uv",new wi(h,2)),b.setAttribute("faceIndex",new wi(f,1)),e.push(b),r>4&&r--}return{_lodPlanes:e,_sizeLods:t,_sigmas:n}}(),Cf=null,Af=(1+Math.sqrt(5))/2,_f=1/Af,If=[new $n(1,1,1),new $n(-1,1,1),new $n(1,1,-1),new $n(-1,1,-1),new $n(0,Af,_f),new $n(0,Af,-_f),new $n(_f,0,Af),new $n(-_f,0,Af),new $n(Af,_f,0),new $n(-Af,_f,0)];function xf(e){var t,n,r,i;this._renderer=e,this._pingPongRenderTarget=null,this._blurMaterial=(t=20,n=new Float32Array(t),r=new $n(0,1,0),(i=new ac({defines:{n:t},uniforms:{envMap:{value:null},samples:{value:1},weights:{value:n},latitudinal:{value:!1},dTheta:{value:0},mipInt:{value:0},poleAxis:{value:r},inputEncoding:{value:gf[$t]},outputEncoding:{value:gf[$t]}},vertexShader:Of(),fragmentShader:`\nprecision mediump float;\nprecision mediump int;\nvarying vec3 vOutputDirection;\nuniform sampler2D envMap;\nuniform int samples;\nuniform float weights[n];\nuniform bool latitudinal;\nuniform float dTheta;\nuniform float mipInt;\nuniform vec3 poleAxis;\n\n${kf()}\n\n#define ENVMAP_TYPE_CUBE_UV\n#include <cube_uv_reflection_fragment>\n\nvec3 getSample(float theta, vec3 axis) {\n\tfloat cosTheta = cos(theta);\n\t// Rodrigues' axis-angle rotation\n\tvec3 sampleDirection = vOutputDirection * cosTheta\n\t\t+ cross(axis, vOutputDirection) * sin(theta)\n\t\t+ axis * dot(axis, vOutputDirection) * (1.0 - cosTheta);\n\treturn bilinearCubeUV(envMap, sampleDirection, mipInt);\n}\n\nvoid main() {\n\tvec3 axis = latitudinal ? poleAxis : cross(poleAxis, vOutputDirection);\n\tif (all(equal(axis, vec3(0.0))))\n\t\taxis = vec3(vOutputDirection.z, 0.0, - vOutputDirection.x);\n\taxis = normalize(axis);\n\tgl_FragColor = vec4(0.0);\n\tgl_FragColor.rgb += weights[0] * getSample(0.0, axis);\n\tfor (int i = 1; i < n; i++) {\n\t\tif (i >= samples)\n\t\t\tbreak;\n\t\tfloat theta = dTheta * float(i);\n\t\tgl_FragColor.rgb += weights[i] * getSample(-1.0 * theta, axis);\n\t\tgl_FragColor.rgb += weights[i] * getSample(theta, axis);\n\t}\n\tgl_FragColor = linearToOutputTexel(gl_FragColor);\n}\n\t\t`,blending:A,depthTest:!1,depthWrite:!1})).type="SphericalGaussianBlur",i),this._equirectShader=null,this._cubemapShader=null,this._compileMaterial(this._blurMaterial)}function wf(e){return void 0!==e&&e.type===Se&&(e.encoding===$t||e.encoding===en||e.encoding===tn)}function Mf(e){var t=new qn(3*hf,3*hf,e);return t.texture.mapping=he,t.texture.name="PMREM.cubeUv",t.scissorTest=!0,t}function Sf(e,t,n,r,i){e.viewport.set(t,n,r,i),e.scissor.set(t,n,r,i)}function Tf(){var e=new ac({uniforms:{envMap:{value:null},texelSize:{value:new Un(1,1)},inputEncoding:{value:gf[$t]},outputEncoding:{value:gf[$t]}},vertexShader:Of(),fragmentShader:`\nprecision mediump float;\nprecision mediump int;\nvarying vec3 vOutputDirection;\nuniform sampler2D envMap;\nuniform vec2 texelSize;\n\n${kf()}\n\n#include <common>\n\nvoid main() {\n\tgl_FragColor = vec4(0.0);\n\tvec3 outputDirection = normalize(vOutputDirection);\n\tvec2 uv = equirectUv( outputDirection );\n\tvec2 f = fract(uv / texelSize - 0.5);\n\tuv -= f * texelSize;\n\tvec3 tl = envMapTexelToLinear(texture2D(envMap, uv)).rgb;\n\tuv.x += texelSize.x;\n\tvec3 tr = envMapTexelToLinear(texture2D(envMap, uv)).rgb;\n\tuv.y += texelSize.y;\n\tvec3 br = envMapTexelToLinear(texture2D(envMap, uv)).rgb;\n\tuv.x -= texelSize.x;\n\tvec3 bl = envMapTexelToLinear(texture2D(envMap, uv)).rgb;\n\tvec3 tm = mix(tl, tr, f.x);\n\tvec3 bm = mix(bl, br, f.x);\n\tgl_FragColor.rgb = mix(tm, bm, f.y);\n\tgl_FragColor = linearToOutputTexel(gl_FragColor);\n}\n\t\t`,blending:A,depthTest:!1,depthWrite:!1});return e.type="EquirectangularToCubeUV",e}function Ef(){var e=new ac({uniforms:{envMap:{value:null},inputEncoding:{value:gf[$t]},outputEncoding:{value:gf[$t]}},vertexShader:Of(),fragmentShader:`\nprecision mediump float;\nprecision mediump int;\nvarying vec3 vOutputDirection;\nuniform samplerCube envMap;\n\n${kf()}\n\nvoid main() {\n\tgl_FragColor = vec4(0.0);\n\tgl_FragColor.rgb = envMapTexelToLinear(textureCube(envMap, vec3( - vOutputDirection.x, vOutputDirection.yz ))).rgb;\n\tgl_FragColor = linearToOutputTexel(gl_FragColor);\n}\n\t\t`,blending:A,depthTest:!1,depthWrite:!1});return e.type="CubemapToCubeUV",e}function Of(){return"\nprecision mediump float;\nprecision mediump int;\nattribute vec3 position;\nattribute vec2 uv;\nattribute float faceIndex;\nvarying vec3 vOutputDirection;\n\n// RH coordinate system; PMREM face-indexing convention\nvec3 getDirection(vec2 uv, float face) {\n\tuv = 2.0 * uv - 1.0;\n\tvec3 direction = vec3(uv, 1.0);\n\tif (face == 0.0) {\n\t\tdirection = direction.zyx; // ( 1, v, u ) pos x\n\t} else if (face == 1.0) {\n\t\tdirection = direction.xzy;\n\t\tdirection.xz *= -1.0; // ( -u, 1, -v ) pos y\n\t} else if (face == 2.0) {\n\t\tdirection.x *= -1.0; // ( -u, v, 1 ) pos z\n\t} else if (face == 3.0) {\n\t\tdirection = direction.zyx;\n\t\tdirection.xz *= -1.0; // ( -1, v, -u ) neg x\n\t} else if (face == 4.0) {\n\t\tdirection = direction.xzy;\n\t\tdirection.xy *= -1.0; // ( -u, -1, v ) neg y\n\t} else if (face == 5.0) {\n\t\tdirection.z *= -1.0; // ( u, v, -1 ) neg z\n\t}\n\treturn direction;\n}\n\nvoid main() {\n\tvOutputDirection = getDirection(uv, faceIndex);\n\tgl_Position = vec4( position, 1.0 );\n}\n\t"}function kf(){return"\nuniform int inputEncoding;\nuniform int outputEncoding;\n\n#include <encodings_pars_fragment>\n\nvec4 inputTexelToLinear(vec4 value){\n\tif(inputEncoding == 0){\n\t\treturn value;\n\t}else if(inputEncoding == 1){\n\t\treturn sRGBToLinear(value);\n\t}else if(inputEncoding == 2){\n\t\treturn RGBEToLinear(value);\n\t}else if(inputEncoding == 3){\n\t\treturn RGBMToLinear(value, 7.0);\n\t}else if(inputEncoding == 4){\n\t\treturn RGBMToLinear(value, 16.0);\n\t}else if(inputEncoding == 5){\n\t\treturn RGBDToLinear(value, 256.0);\n\t}else{\n\t\treturn GammaToLinear(value, 2.2);\n\t}\n}\n\nvec4 linearToOutputTexel(vec4 value){\n\tif(outputEncoding == 0){\n\t\treturn value;\n\t}else if(outputEncoding == 1){\n\t\treturn LinearTosRGB(value);\n\t}else if(outputEncoding == 2){\n\t\treturn LinearToRGBE(value);\n\t}else if(outputEncoding == 3){\n\t\treturn LinearToRGBM(value, 7.0);\n\t}else if(outputEncoding == 4){\n\t\treturn LinearToRGBM(value, 16.0);\n\t}else if(outputEncoding == 5){\n\t\treturn LinearToRGBD(value, 256.0);\n\t}else{\n\t\treturn LinearToGamma(value, 2.2);\n\t}\n}\n\nvec4 envMapTexelToLinear(vec4 color) {\n\treturn inputTexelToLinear(color);\n}\n\t"}function Lf(e,t,n,r,i,o,a){return console.warn("THREE.Face4 has been removed. A THREE.Face3 will be created instead."),new Ci(e,t,n,i,o,a)}xf.prototype={constructor:xf,fromScene:function(e,t=0,n=.1,r=100){Cf=this._renderer.getRenderTarget();var i=this._allocateTargets();return this._sceneToCubeUV(e,n,r,i),t>0&&this._blur(i,0,0,t),this._applyPMREM(i),this._cleanup(i),i},fromEquirectangular:function(e){return e.magFilter=me,e.minFilter=me,e.generateMipmaps=!1,this.fromCubemap(e)},fromCubemap:function(e){Cf=this._renderer.getRenderTarget();var t=this._allocateTargets(e);return this._textureToCubeUV(e,t),this._applyPMREM(t),this._cleanup(t),t},compileCubemapShader:function(){null===this._cubemapShader&&(this._cubemapShader=Ef(),this._compileMaterial(this._cubemapShader))},compileEquirectangularShader:function(){null===this._equirectShader&&(this._equirectShader=Tf(),this._compileMaterial(this._equirectShader))},dispose:function(){this._blurMaterial.dispose(),null!==this._cubemapShader&&this._cubemapShader.dispose(),null!==this._equirectShader&&this._equirectShader.dispose();for(var e=0;e<mf.length;e++)mf[e].dispose()},_cleanup:function(e){this._pingPongRenderTarget.dispose(),this._renderer.setRenderTarget(Cf),e.scissorTest=!1,e.setSize(e.width,e.height)},_allocateTargets:function(e){var t={magFilter:me,minFilter:me,generateMipmaps:!1,type:Se,format:Ye,encoding:wf(e)?e.encoding:nn,depthBuffer:!1,stencilBuffer:!1},n=Mf(t);return n.depthBuffer=!e,this._pingPongRenderTarget=Mf(t),n},_compileMaterial:function(e){var t=new lo(mf[0],e);this._renderer.compile(t,vf)},_sceneToCubeUV:function(e,t,n,r){var i=new xo(90,1,t,n),o=[1,-1,1,1,1,1],a=[1,1,1,-1,-1,-1],s=this._renderer,l=s.outputEncoding,u=s.toneMapping,c=s.toneMappingExposure,d=s.getClearColor(),h=s.getClearAlpha();s.toneMapping=te,s.toneMappingExposure=1,s.outputEncoding=$t;var f=e.background;if(f&&f.isColor){f.convertSRGBToLinear();var p=Math.max(f.r,f.g,f.b),g=Math.min(Math.max(Math.ceil(Math.log2(p)),-128),127);f=f.multiplyScalar(Math.pow(2,-g));var v=(g+128)/255;s.setClearColor(f,v),e.background=null}for(var m=0;m<6;m++){var y=m%3;0==y?(i.up.set(0,o[m],0),i.lookAt(a[m],0,0)):1==y?(i.up.set(0,0,o[m]),i.lookAt(0,a[m],0)):(i.up.set(0,o[m],0),i.lookAt(0,0,a[m])),Sf(r,y*hf,m>2?hf:0,hf,hf),s.setRenderTarget(r),s.render(e,i)}s.toneMapping=u,s.toneMappingExposure=c,s.outputEncoding=l,s.setClearColor(d,h)},_textureToCubeUV:function(e,t){var n=this._renderer;e.isCubeTexture?null==this._cubemapShader&&(this._cubemapShader=Ef()):null==this._equirectShader&&(this._equirectShader=Tf());var r=e.isCubeTexture?this._cubemapShader:this._equirectShader,i=new lo(mf[0],r),o=r.uniforms;o.envMap.value=e,e.isCubeTexture||o.texelSize.value.set(1/e.image.width,1/e.image.height),o.inputEncoding.value=gf[e.encoding],o.outputEncoding.value=gf[t.texture.encoding],Sf(t,0,0,3*hf,2*hf),n.setRenderTarget(t),n.render(i,vf)},_applyPMREM:function(e){var t=this._renderer,n=t.autoClear;t.autoClear=!1;for(var r=1;r<pf;r++){var i=Math.sqrt(bf[r]*bf[r]-bf[r-1]*bf[r-1]),o=If[(r-1)%If.length];this._blur(e,r-1,r,i,o)}t.autoClear=n},_blur:function(e,t,n,r,i){var o=this._pingPongRenderTarget;this._halfBlur(e,o,t,n,r,"latitudinal",i),this._halfBlur(o,e,n,n,r,"longitudinal",i)},_halfBlur:function(e,t,n,r,i,o,a){var s=this._renderer,l=this._blurMaterial;"latitudinal"!==o&&"longitudinal"!==o&&console.error("blur direction must be either latitudinal or longitudinal!");var u=new lo(mf[r],l),c=l.uniforms,d=yf[n]-1,h=isFinite(i)?Math.PI/(2*d):2*Math.PI/39,f=i/h,p=isFinite(i)?1+Math.floor(3*f):20;p>20&&console.warn(`sigmaRadians, ${i}, is too large and will clip, as it requested ${p} samples when the maximum is set to 20`);for(var g=[],v=0,m=0;m<20;++m){var y=m/f,b=Math.exp(-y*y/2);g.push(b),0==m?v+=b:m<p&&(v+=2*b)}for(m=0;m<g.length;m++)g[m]=g[m]/v;c.envMap.value=e.texture,c.samples.value=p,c.weights.value=g,c.latitudinal.value="latitudinal"===o,a&&(c.poleAxis.value=a),c.dTheta.value=h,c.mipInt.value=8-n,c.inputEncoding.value=gf[e.texture.encoding],c.outputEncoding.value=gf[e.texture.encoding];var C=yf[r];Sf(t,y=3*Math.max(0,hf-2*C),(0===r?0:2*hf)+2*C*(r>4?r-8+4:0),3*C,2*C),s.setRenderTarget(t),s.render(u,vf)}};var Df=0,Pf=1,Nf=0,jf=1,Rf=2;function zf(e){return console.warn("THREE.MeshFaceMaterial has been removed. Use an Array instead."),e}function Ff(e){return void 0===e&&(e=[]),console.warn("THREE.MultiMaterial has been removed. Use an Array instead."),e.isMultiMaterial=!0,e.materials=e,e.clone=function(){return e.slice()},e}function Bf(e,t){return console.warn("THREE.PointCloud has been renamed to THREE.Points."),new Rl(e,t)}function Hf(e){return console.warn("THREE.Particle has been renamed to THREE.Sprite."),new tl(e)}function Uf(e,t){return console.warn("THREE.ParticleSystem has been renamed to THREE.Points."),new Rl(e,t)}function Vf(e){return console.warn("THREE.PointCloudMaterial has been renamed to THREE.PointsMaterial."),new Ll(e)}function Yf(e){return console.warn("THREE.ParticleBasicMaterial has been renamed to THREE.PointsMaterial."),new Ll(e)}function Wf(e){return console.warn("THREE.ParticleSystemMaterial has been renamed to THREE.PointsMaterial."),new Ll(e)}function Gf(e,t,n){return console.warn("THREE.Vertex has been removed. Use THREE.Vector3 instead."),new $n(e,t,n)}function Xf(e,t){return console.warn("THREE.DynamicBufferAttribute has been removed. Use new THREE.BufferAttribute().setUsage( THREE.DynamicDrawUsage ) instead."),new wi(e,t).setUsage(En)}function qf(e,t){return console.warn("THREE.Int8Attribute has been removed. Use new THREE.Int8BufferAttribute() instead."),new Mi(e,t)}function Zf(e,t){return console.warn("THREE.Uint8Attribute has been removed. Use new THREE.Uint8BufferAttribute() instead."),new Si(e,t)}function Qf(e,t){return console.warn("THREE.Uint8ClampedAttribute has been removed. Use new THREE.Uint8ClampedBufferAttribute() instead."),new Ti(e,t)}function Kf(e,t){return console.warn("THREE.Int16Attribute has been removed. Use new THREE.Int16BufferAttribute() instead."),new Ei(e,t)}function Jf(e,t){return console.warn("THREE.Uint16Attribute has been removed. Use new THREE.Uint16BufferAttribute() instead."),new Oi(e,t)}function $f(e,t){return console.warn("THREE.Int32Attribute has been removed. Use new THREE.Int32BufferAttribute() instead."),new ki(e,t)}function ep(e,t){return console.warn("THREE.Uint32Attribute has been removed. Use new THREE.Uint32BufferAttribute() instead."),new Li(e,t)}function tp(e,t){return console.warn("THREE.Float32Attribute has been removed. Use new THREE.Float32BufferAttribute() instead."),new Di(e,t)}function np(e,t){return console.warn("THREE.Float64Attribute has been removed. Use new THREE.Float64BufferAttribute() instead."),new Pi(e,t)}function rp(e){console.warn("THREE.ClosedSplineCurve3 has been deprecated. Use THREE.CatmullRomCurve3 instead."),Kc.call(this,e),this.type="catmullrom",this.closed=!0}function ip(e){console.warn("THREE.SplineCurve3 has been deprecated. Use THREE.CatmullRomCurve3 instead."),Kc.call(this,e),this.type="catmullrom"}function op(e){console.warn("THREE.Spline has been removed. Use THREE.CatmullRomCurve3 instead."),Kc.call(this,e),this.type="catmullrom"}function ap(e){return console.warn("THREE.AxisHelper has been renamed to THREE.AxesHelper."),new df(e)}function sp(e,t){return console.warn("THREE.BoundingBoxHelper has been deprecated. Creating a THREE.BoxHelper instead."),new rf(e,t)}function lp(e,t){return console.warn("THREE.EdgesHelper has been removed. Use THREE.EdgesGeometry instead."),new Ol(new Ku(e.geometry),new Al({color:void 0!==t?t:16777215}))}function up(e,t){return console.warn("THREE.WireframeHelper has been removed. Use THREE.WireframeGeometry instead."),new Ol(new Vl(e.geometry),new Al({color:void 0!==t?t:16777215}))}function cp(e){return console.warn("THREE.XHRLoader has been renamed to THREE.FileLoader."),new jc(e)}function dp(e){return console.warn("THREE.BinaryTextureLoader has been renamed to THREE.DataTextureLoader."),new Fc(e)}function hp(e,t,n){return console.warn("THREE.WebGLRenderTargetCube( width, height, options ) is now WebGLCubeRenderTarget( size, options )."),new Mo(e,n)}Vc.create=function(e,t){return console.log("THREE.Curve.create() has been deprecated"),e.prototype=Object.create(Vc.prototype),e.prototype.constructor=e,e.prototype.getPoint=t,e},Object.assign(ud.prototype,{createPointsGeometry:function(e){console.warn("THREE.CurvePath: .createPointsGeometry() has been removed. Use new THREE.Geometry().setFromPoints( points ) instead.");var t=this.getPoints(e);return this.createGeometry(t)},createSpacedPointsGeometry:function(e){console.warn("THREE.CurvePath: .createSpacedPointsGeometry() has been removed. Use new THREE.Geometry().setFromPoints( points ) instead.");var t=this.getSpacedPoints(e);return this.createGeometry(t)},createGeometry:function(e){console.warn("THREE.CurvePath: .createGeometry() has been removed. Use new THREE.Geometry().setFromPoints( points ) instead.");for(var t=new vo,n=0,r=e.length;n<r;n++){var i=e[n];t.vertices.push(new $n(i.x,i.y,i.z||0))}return t}}),Object.assign(cd.prototype,{fromPoints:function(e){return console.warn("THREE.Path: .fromPoints() has been renamed to .setFromPoints()."),this.setFromPoints(e)}}),rp.prototype=Object.create(Kc.prototype),ip.prototype=Object.create(Kc.prototype),op.prototype=Object.create(Kc.prototype),Object.assign(op.prototype,{initFromArray:function(){console.error("THREE.Spline: .initFromArray() has been removed.")},getControlPointsArray:function(){console.error("THREE.Spline: .getControlPointsArray() has been removed.")},reparametrizeByArcLength:function(){console.error("THREE.Spline: .reparametrizeByArcLength() has been removed.")}}),Gh.prototype.setColors=function(){console.error("THREE.GridHelper: setColors() has been deprecated, pass them in the constructor instead.")},Bh.prototype.update=function(){console.error("THREE.SkeletonHelper: update() no longer needs to be called.")},Object.assign(Pc.prototype,{extractUrlBase:function(e){return console.warn("THREE.Loader: .extractUrlBase() has been deprecated. Use THREE.LoaderUtils.extractUrlBase() instead."),Sd.extractUrlBase(e)}}),Pc.Handlers={add:function(){console.error("THREE.Loader: Handlers.add() has been removed. Use LoadingManager.addHandler() instead.")},get:function(){console.error("THREE.Loader: Handlers.get() has been removed. Use LoadingManager.getHandler() instead.")}},Object.assign(Ld.prototype,{setTexturePath:function(e){return console.warn("THREE.ObjectLoader: .setTexturePath() has been renamed to .setResourcePath()."),this.setResourcePath(e)}}),Object.assign(Oh.prototype,{center:function(e){return console.warn("THREE.Box2: .center() has been renamed to .getCenter()."),this.getCenter(e)},empty:function(){return console.warn("THREE.Box2: .empty() has been renamed to .isEmpty()."),this.isEmpty()},isIntersectionBox:function(e){return console.warn("THREE.Box2: .isIntersectionBox() has been renamed to .intersectsBox()."),this.intersectsBox(e)},size:function(e){return console.warn("THREE.Box2: .size() has been renamed to .getSize()."),this.getSize(e)}}),Object.assign(Br.prototype,{center:function(e){return console.warn("THREE.Box3: .center() has been renamed to .getCenter()."),this.getCenter(e)},empty:function(){return console.warn("THREE.Box3: .empty() has been renamed to .isEmpty()."),this.isEmpty()},isIntersectionBox:function(e){return console.warn("THREE.Box3: .isIntersectionBox() has been renamed to .intersectsBox()."),this.intersectsBox(e)},isIntersectionSphere:function(e){return console.warn("THREE.Box3: .isIntersectionSphere() has been renamed to .intersectsSphere()."),this.intersectsSphere(e)},size:function(e){return console.warn("THREE.Box3: .size() has been renamed to .getSize()."),this.getSize(e)}}),Object.assign(Vr.prototype,{empty:function(){return console.warn("THREE.Sphere: .empty() has been renamed to .isEmpty()."),this.isEmpty()}}),Oo.prototype.setFromMatrix=function(e){return console.warn("THREE.Frustum: .setFromMatrix() has been renamed to .setFromProjectionMatrix()."),this.setFromProjectionMatrix(e)},Dh.prototype.center=function(e){return console.warn("THREE.Line3: .center() has been renamed to .getCenter()."),this.getCenter(e)},Object.assign(Hn,{random16:function(){return console.warn("THREE.Math: .random16() has been deprecated. Use Math.random() instead."),Math.random()},nearestPowerOfTwo:function(e){return console.warn("THREE.Math: .nearestPowerOfTwo() has been renamed to .floorPowerOfTwo()."),Hn.floorPowerOfTwo(e)},nextPowerOfTwo:function(e){return console.warn("THREE.Math: .nextPowerOfTwo() has been renamed to .ceilPowerOfTwo()."),Hn.ceilPowerOfTwo(e)}}),Object.assign(Vn.prototype,{flattenToArrayOffset:function(e,t){return console.warn("THREE.Matrix3: .flattenToArrayOffset() has been deprecated. Use .toArray() instead."),this.toArray(e,t)},multiplyVector3:function(e){return console.warn("THREE.Matrix3: .multiplyVector3() has been removed. Use vector.applyMatrix3( matrix ) instead."),e.applyMatrix3(this)},multiplyVector3Array:function(){console.error("THREE.Matrix3: .multiplyVector3Array() has been removed.")},applyToBufferAttribute:function(e){return console.warn("THREE.Matrix3: .applyToBufferAttribute() has been removed. Use attribute.applyMatrix3( matrix ) instead."),e.applyMatrix3(this)},applyToVector3Array:function(){console.error("THREE.Matrix3: .applyToVector3Array() has been removed.")}}),Object.assign(sr.prototype,{extractPosition:function(e){return console.warn("THREE.Matrix4: .extractPosition() has been renamed to .copyPosition()."),this.copyPosition(e)},flattenToArrayOffset:function(e,t){return console.warn("THREE.Matrix4: .flattenToArrayOffset() has been deprecated. Use .toArray() instead."),this.toArray(e,t)},getPosition:function(){return console.warn("THREE.Matrix4: .getPosition() has been removed. Use Vector3.setFromMatrixPosition( matrix ) instead."),(new $n).setFromMatrixColumn(this,3)},setRotationFromQuaternion:function(e){return console.warn("THREE.Matrix4: .setRotationFromQuaternion() has been renamed to .makeRotationFromQuaternion()."),this.makeRotationFromQuaternion(e)},multiplyToArray:function(){console.warn("THREE.Matrix4: .multiplyToArray() has been removed.")},multiplyVector3:function(e){return console.warn("THREE.Matrix4: .multiplyVector3() has been removed. Use vector.applyMatrix4( matrix ) instead."),e.applyMatrix4(this)},multiplyVector4:function(e){return console.warn("THREE.Matrix4: .multiplyVector4() has been removed. Use vector.applyMatrix4( matrix ) instead."),e.applyMatrix4(this)},multiplyVector3Array:function(){console.error("THREE.Matrix4: .multiplyVector3Array() has been removed.")},rotateAxis:function(e){console.warn("THREE.Matrix4: .rotateAxis() has been removed. Use Vector3.transformDirection( matrix ) instead."),e.transformDirection(this)},crossVector:function(e){return console.warn("THREE.Matrix4: .crossVector() has been removed. Use vector.applyMatrix4( matrix ) instead."),e.applyMatrix4(this)},translate:function(){console.error("THREE.Matrix4: .translate() has been removed.")},rotateX:function(){console.error("THREE.Matrix4: .rotateX() has been removed.")},rotateY:function(){console.error("THREE.Matrix4: .rotateY() has been removed.")},rotateZ:function(){console.error("THREE.Matrix4: .rotateZ() has been removed.")},rotateByAxis:function(){console.error("THREE.Matrix4: .rotateByAxis() has been removed.")},applyToBufferAttribute:function(e){return console.warn("THREE.Matrix4: .applyToBufferAttribute() has been removed. Use attribute.applyMatrix4( matrix ) instead."),e.applyMatrix4(this)},applyToVector3Array:function(){console.error("THREE.Matrix4: .applyToVector3Array() has been removed.")},makeFrustum:function(e,t,n,r,i,o){return console.warn("THREE.Matrix4: .makeFrustum() has been removed. Use .makePerspective( left, right, top, bottom, near, far ) instead."),this.makePerspective(e,t,r,n,i,o)}}),ti.prototype.isIntersectionLine=function(e){return console.warn("THREE.Plane: .isIntersectionLine() has been renamed to .intersectsLine()."),this.intersectsLine(e)},Qn.prototype.multiplyVector3=function(e){return console.warn("THREE.Quaternion: .multiplyVector3() has been removed. Use is now vector.applyQuaternion( quaternion ) instead."),e.applyQuaternion(this)},Object.assign(Kr.prototype,{isIntersectionBox:function(e){return console.warn("THREE.Ray: .isIntersectionBox() has been renamed to .intersectsBox()."),this.intersectsBox(e)},isIntersectionPlane:function(e){return console.warn("THREE.Ray: .isIntersectionPlane() has been renamed to .intersectsPlane()."),this.intersectsPlane(e)},isIntersectionSphere:function(e){return console.warn("THREE.Ray: .isIntersectionSphere() has been renamed to .intersectsSphere()."),this.intersectsSphere(e)}}),Object.assign(hi.prototype,{area:function(){return console.warn("THREE.Triangle: .area() has been renamed to .getArea()."),this.getArea()},barycoordFromPoint:function(e,t){return console.warn("THREE.Triangle: .barycoordFromPoint() has been renamed to .getBarycoord()."),this.getBarycoord(e,t)},midpoint:function(e){return console.warn("THREE.Triangle: .midpoint() has been renamed to .getMidpoint()."),this.getMidpoint(e)},normal:function(e){return console.warn("THREE.Triangle: .normal() has been renamed to .getNormal()."),this.getNormal(e)},plane:function(e){return console.warn("THREE.Triangle: .plane() has been renamed to .getPlane()."),this.getPlane(e)}}),Object.assign(hi,{barycoordFromPoint:function(e,t,n,r,i){return console.warn("THREE.Triangle: .barycoordFromPoint() has been renamed to .getBarycoord()."),hi.getBarycoord(e,t,n,r,i)},normal:function(e,t,n,r){return console.warn("THREE.Triangle: .normal() has been renamed to .getNormal()."),hi.getNormal(e,t,n,r)}}),Object.assign(dd.prototype,{extractAllPoints:function(e){return console.warn("THREE.Shape: .extractAllPoints() has been removed. Use .extractPoints() instead."),this.extractPoints(e)},extrude:function(e){return console.warn("THREE.Shape: .extrude() has been removed. Use ExtrudeGeometry() instead."),new ju(this,e)},makeGeometry:function(e){return console.warn("THREE.Shape: .makeGeometry() has been removed. Use ShapeGeometry() instead."),new qu(this,e)}}),Object.assign(Un.prototype,{fromAttribute:function(e,t,n){return console.warn("THREE.Vector2: .fromAttribute() has been renamed to .fromBufferAttribute()."),this.fromBufferAttribute(e,t,n)},distanceToManhattan:function(e){return console.warn("THREE.Vector2: .distanceToManhattan() has been renamed to .manhattanDistanceTo()."),this.manhattanDistanceTo(e)},lengthManhattan:function(){return console.warn("THREE.Vector2: .lengthManhattan() has been renamed to .manhattanLength()."),this.manhattanLength()}}),Object.assign($n.prototype,{setEulerFromRotationMatrix:function(){console.error("THREE.Vector3: .setEulerFromRotationMatrix() has been removed. Use Euler.setFromRotationMatrix() instead.")},setEulerFromQuaternion:function(){console.error("THREE.Vector3: .setEulerFromQuaternion() has been removed. Use Euler.setFromQuaternion() instead.")},getPositionFromMatrix:function(e){return console.warn("THREE.Vector3: .getPositionFromMatrix() has been renamed to .setFromMatrixPosition()."),this.setFromMatrixPosition(e)},getScaleFromMatrix:function(e){return console.warn("THREE.Vector3: .getScaleFromMatrix() has been renamed to .setFromMatrixScale()."),this.setFromMatrixScale(e)},getColumnFromMatrix:function(e,t){return console.warn("THREE.Vector3: .getColumnFromMatrix() has been renamed to .setFromMatrixColumn()."),this.setFromMatrixColumn(t,e)},applyProjection:function(e){return console.warn("THREE.Vector3: .applyProjection() has been removed. Use .applyMatrix4( m ) instead."),this.applyMatrix4(e)},fromAttribute:function(e,t,n){return console.warn("THREE.Vector3: .fromAttribute() has been renamed to .fromBufferAttribute()."),this.fromBufferAttribute(e,t,n)},distanceToManhattan:function(e){return console.warn("THREE.Vector3: .distanceToManhattan() has been renamed to .manhattanDistanceTo()."),this.manhattanDistanceTo(e)},lengthManhattan:function(){return console.warn("THREE.Vector3: .lengthManhattan() has been renamed to .manhattanLength()."),this.manhattanLength()}}),Object.assign(Xn.prototype,{fromAttribute:function(e,t,n){return console.warn("THREE.Vector4: .fromAttribute() has been renamed to .fromBufferAttribute()."),this.fromBufferAttribute(e,t,n)},lengthManhattan:function(){return console.warn("THREE.Vector4: .lengthManhattan() has been renamed to .manhattanLength()."),this.manhattanLength()}}),Object.assign(vo.prototype,{computeTangents:function(){console.error("THREE.Geometry: .computeTangents() has been removed.")},computeLineDistances:function(){console.error("THREE.Geometry: .computeLineDistances() has been removed. Use THREE.Line.computeLineDistances() instead.")},applyMatrix:function(e){return console.warn("THREE.Geometry: .applyMatrix() has been renamed to .applyMatrix4()."),this.applyMatrix4(e)}}),Object.assign(wr.prototype,{getChildByName:function(e){return console.warn("THREE.Object3D: .getChildByName() has been renamed to .getObjectByName()."),this.getObjectByName(e)},renderDepth:function(){console.warn("THREE.Object3D: .renderDepth has been removed. Use .renderOrder, instead.")},translate:function(e,t){return console.warn("THREE.Object3D: .translate() has been removed. Use .translateOnAxis( axis, distance ) instead."),this.translateOnAxis(t,e)},getWorldRotation:function(){console.error("THREE.Object3D: .getWorldRotation() has been removed. Use THREE.Object3D.getWorldQuaternion( target ) instead.")},applyMatrix:function(e){return console.warn("THREE.Object3D: .applyMatrix() has been renamed to .applyMatrix4()."),this.applyMatrix4(e)}}),Object.defineProperties(wr.prototype,{eulerOrder:{get:function(){return console.warn("THREE.Object3D: .eulerOrder is now .rotation.order."),this.rotation.order},set:function(e){console.warn("THREE.Object3D: .eulerOrder is now .rotation.order."),this.rotation.order=e}},useQuaternion:{get:function(){console.warn("THREE.Object3D: .useQuaternion has been removed. The library now uses quaternions by default.")},set:function(){console.warn("THREE.Object3D: .useQuaternion has been removed. The library now uses quaternions by default.")}}}),Object.assign(lo.prototype,{setDrawMode:function(){console.error("THREE.Mesh: .setDrawMode() has been removed. The renderer now always assumes THREE.TrianglesDrawMode. Transform your geometry via BufferGeometryUtils.toTrianglesDrawMode() if necessary.")}}),Object.defineProperties(lo.prototype,{drawMode:{get:function(){return console.error("THREE.Mesh: .drawMode has been removed. The renderer now always assumes THREE.TrianglesDrawMode."),Qt},set:function(){console.error("THREE.Mesh: .drawMode has been removed. The renderer now always assumes THREE.TrianglesDrawMode. Transform your geometry via BufferGeometryUtils.toTrianglesDrawMode() if necessary.")}}}),Object.defineProperties(cl.prototype,{objects:{get:function(){return console.warn("THREE.LOD: .objects has been renamed to .levels."),this.levels}}}),Object.defineProperty(pl.prototype,"useVertexTexture",{get:function(){console.warn("THREE.Skeleton: useVertexTexture has been removed.")},set:function(){console.warn("THREE.Skeleton: useVertexTexture has been removed.")}}),dl.prototype.initBones=function(){console.error("THREE.SkinnedMesh: initBones() has been removed.")},Object.defineProperty(Vc.prototype,"__arcLengthDivisions",{get:function(){return console.warn("THREE.Curve: .__arcLengthDivisions is now .arcLengthDivisions."),this.arcLengthDivisions},set:function(e){console.warn("THREE.Curve: .__arcLengthDivisions is now .arcLengthDivisions."),this.arcLengthDivisions=e}}),xo.prototype.setLens=function(e,t){console.warn("THREE.PerspectiveCamera.setLens is deprecated. Use .setFocalLength and .filmGauge for a photographic setup."),void 0!==t&&(this.filmGauge=t),this.setFocalLength(e)},Object.defineProperties(hd.prototype,{onlyShadow:{set:function(){console.warn("THREE.Light: .onlyShadow has been removed.")}},shadowCameraFov:{set:function(e){console.warn("THREE.Light: .shadowCameraFov is now .shadow.camera.fov."),this.shadow.camera.fov=e}},shadowCameraLeft:{set:function(e){console.warn("THREE.Light: .shadowCameraLeft is now .shadow.camera.left."),this.shadow.camera.left=e}},shadowCameraRight:{set:function(e){console.warn("THREE.Light: .shadowCameraRight is now .shadow.camera.right."),this.shadow.camera.right=e}},shadowCameraTop:{set:function(e){console.warn("THREE.Light: .shadowCameraTop is now .shadow.camera.top."),this.shadow.camera.top=e}},shadowCameraBottom:{set:function(e){console.warn("THREE.Light: .shadowCameraBottom is now .shadow.camera.bottom."),this.shadow.camera.bottom=e}},shadowCameraNear:{set:function(e){console.warn("THREE.Light: .shadowCameraNear is now .shadow.camera.near."),this.shadow.camera.near=e}},shadowCameraFar:{set:function(e){console.warn("THREE.Light: .shadowCameraFar is now .shadow.camera.far."),this.shadow.camera.far=e}},shadowCameraVisible:{set:function(){console.warn("THREE.Light: .shadowCameraVisible has been removed. Use new THREE.CameraHelper( light.shadow.camera ) instead.")}},shadowBias:{set:function(e){console.warn("THREE.Light: .shadowBias is now .shadow.bias."),this.shadow.bias=e}},shadowDarkness:{set:function(){console.warn("THREE.Light: .shadowDarkness has been removed.")}},shadowMapWidth:{set:function(e){console.warn("THREE.Light: .shadowMapWidth is now .shadow.mapSize.width."),this.shadow.mapSize.width=e}},shadowMapHeight:{set:function(e){console.warn("THREE.Light: .shadowMapHeight is now .shadow.mapSize.height."),this.shadow.mapSize.height=e}}}),Object.defineProperties(wi.prototype,{length:{get:function(){return console.warn("THREE.BufferAttribute: .length has been deprecated. Use .count instead."),this.array.length}},dynamic:{get:function(){return console.warn("THREE.BufferAttribute: .dynamic has been deprecated. Use .usage instead."),this.usage===En},set:function(){console.warn("THREE.BufferAttribute: .dynamic has been deprecated. Use .usage instead."),this.setUsage(En)}}}),Object.assign(wi.prototype,{setDynamic:function(e){return console.warn("THREE.BufferAttribute: .setDynamic() has been deprecated. Use .setUsage() instead."),this.setUsage(!0===e?En:Tn),this},copyIndicesArray:function(){console.error("THREE.BufferAttribute: .copyIndicesArray() has been removed.")},setArray:function(){console.error("THREE.BufferAttribute: .setArray has been removed. Use BufferGeometry .setAttribute to replace/resize attribute buffers")}}),Object.assign(Yi.prototype,{addIndex:function(e){console.warn("THREE.BufferGeometry: .addIndex() has been renamed to .setIndex()."),this.setIndex(e)},addAttribute:function(e,t){return console.warn("THREE.BufferGeometry: .addAttribute() has been renamed to .setAttribute()."),t&&t.isBufferAttribute||t&&t.isInterleavedBufferAttribute?"index"===e?(console.warn("THREE.BufferGeometry.addAttribute: Use .setIndex() for index attribute."),this.setIndex(t),this):this.setAttribute(e,t):(console.warn("THREE.BufferGeometry: .addAttribute() now expects ( name, attribute )."),this.setAttribute(e,new wi(arguments[1],arguments[2])))},addDrawCall:function(e,t,n){void 0!==n&&console.warn("THREE.BufferGeometry: .addDrawCall() no longer supports indexOffset."),console.warn("THREE.BufferGeometry: .addDrawCall() is now .addGroup()."),this.addGroup(e,t)},clearDrawCalls:function(){console.warn("THREE.BufferGeometry: .clearDrawCalls() is now .clearGroups()."),this.clearGroups()},computeTangents:function(){console.warn("THREE.BufferGeometry: .computeTangents() has been removed.")},computeOffsets:function(){console.warn("THREE.BufferGeometry: .computeOffsets() has been removed.")},removeAttribute:function(e){return console.warn("THREE.BufferGeometry: .removeAttribute() has been renamed to .deleteAttribute()."),this.deleteAttribute(e)},applyMatrix:function(e){return console.warn("THREE.BufferGeometry: .applyMatrix() has been renamed to .applyMatrix4()."),this.applyMatrix4(e)}}),Object.defineProperties(Yi.prototype,{drawcalls:{get:function(){return console.error("THREE.BufferGeometry: .drawcalls has been renamed to .groups."),this.groups}},offsets:{get:function(){return console.warn("THREE.BufferGeometry: .offsets has been renamed to .groups."),this.groups}}}),Object.defineProperties(Td.prototype,{maxInstancedCount:{get:function(){return console.warn("THREE.InstancedBufferGeometry: .maxInstancedCount has been renamed to .instanceCount."),this.instanceCount},set:function(e){console.warn("THREE.InstancedBufferGeometry: .maxInstancedCount has been renamed to .instanceCount."),this.instanceCount=e}}}),Object.defineProperties(xh.prototype,{linePrecision:{get:function(){return console.warn("THREE.Raycaster: .linePrecision has been deprecated. Use .params.Line.threshold instead."),this.params.Line.threshold},set:function(e){console.warn("THREE.Raycaster: .linePrecision has been deprecated. Use .params.Line.threshold instead."),this.params.Line.threshold=e}}}),Object.defineProperties(zs.prototype,{dynamic:{get:function(){return console.warn("THREE.InterleavedBuffer: .length has been deprecated. Use .usage instead."),this.usage===En},set:function(e){console.warn("THREE.InterleavedBuffer: .length has been deprecated. Use .usage instead."),this.setUsage(e)}}}),Object.assign(zs.prototype,{setDynamic:function(e){return console.warn("THREE.InterleavedBuffer: .setDynamic() has been deprecated. Use .setUsage() instead."),this.setUsage(!0===e?En:Tn),this},setArray:function(){console.error("THREE.InterleavedBuffer: .setArray has been removed. Use BufferGeometry .setAttribute to replace/resize attribute buffers")}}),Object.assign(Ru.prototype,{getArrays:function(){console.error("THREE.ExtrudeBufferGeometry: .getArrays() has been removed.")},addShapeList:function(){console.error("THREE.ExtrudeBufferGeometry: .addShapeList() has been removed.")},addShape:function(){console.error("THREE.ExtrudeBufferGeometry: .addShape() has been removed.")}}),Object.defineProperties(_h.prototype,{dynamic:{set:function(){console.warn("THREE.Uniform: .dynamic has been removed. Use object.onBeforeRender() instead.")}},onUpdate:{value:function(){return console.warn("THREE.Uniform: .onUpdate() has been removed. Use object.onBeforeRender() instead."),this}}}),Object.defineProperties(_i.prototype,{wrapAround:{get:function(){console.warn("THREE.Material: .wrapAround has been removed.")},set:function(){console.warn("THREE.Material: .wrapAround has been removed.")}},overdraw:{get:function(){console.warn("THREE.Material: .overdraw has been removed.")},set:function(){console.warn("THREE.Material: .overdraw has been removed.")}},wrapRGB:{get:function(){return console.warn("THREE.Material: .wrapRGB has been removed."),new vi}},shading:{get:function(){console.error("THREE."+this.type+": .shading has been removed. Use the boolean .flatShading instead.")},set:function(e){console.warn("THREE."+this.type+": .shading has been removed. Use the boolean .flatShading instead."),this.flatShading=e===b}},stencilMask:{get:function(){return console.warn("THREE."+this.type+": .stencilMask has been removed. Use .stencilFuncMask instead."),this.stencilFuncMask},set:function(e){console.warn("THREE."+this.type+": .stencilMask has been removed. Use .stencilFuncMask instead."),this.stencilFuncMask=e}}}),Object.defineProperties(uc.prototype,{metal:{get:function(){return console.warn("THREE.MeshPhongMaterial: .metal has been removed. Use THREE.MeshStandardMaterial instead."),!1},set:function(){console.warn("THREE.MeshPhongMaterial: .metal has been removed. Use THREE.MeshStandardMaterial instead")}}}),Object.defineProperties(_o.prototype,{derivatives:{get:function(){return console.warn("THREE.ShaderMaterial: .derivatives has been moved to .extensions.derivatives."),this.extensions.derivatives},set:function(e){console.warn("THREE. ShaderMaterial: .derivatives has been moved to .extensions.derivatives."),this.extensions.derivatives=e}}}),Object.assign(Ns.prototype,{clearTarget:function(e,t,n,r){console.warn("THREE.WebGLRenderer: .clearTarget() has been deprecated. Use .setRenderTarget() and .clear() instead."),this.setRenderTarget(e),this.clear(t,n,r)},animate:function(e){console.warn("THREE.WebGLRenderer: .animate() is now .setAnimationLoop()."),this.setAnimationLoop(e)},getCurrentRenderTarget:function(){return console.warn("THREE.WebGLRenderer: .getCurrentRenderTarget() is now .getRenderTarget()."),this.getRenderTarget()},getMaxAnisotropy:function(){return console.warn("THREE.WebGLRenderer: .getMaxAnisotropy() is now .capabilities.getMaxAnisotropy()."),this.capabilities.getMaxAnisotropy()},getPrecision:function(){return console.warn("THREE.WebGLRenderer: .getPrecision() is now .capabilities.precision."),this.capabilities.precision},resetGLState:function(){return console.warn("THREE.WebGLRenderer: .resetGLState() is now .state.reset()."),this.state.reset()},supportsFloatTextures:function(){return console.warn("THREE.WebGLRenderer: .supportsFloatTextures() is now .extensions.get( 'OES_texture_float' )."),this.extensions.get("OES_texture_float")},supportsHalfFloatTextures:function(){return console.warn("THREE.WebGLRenderer: .supportsHalfFloatTextures() is now .extensions.get( 'OES_texture_half_float' )."),this.extensions.get("OES_texture_half_float")},supportsStandardDerivatives:function(){return console.warn("THREE.WebGLRenderer: .supportsStandardDerivatives() is now .extensions.get( 'OES_standard_derivatives' )."),this.extensions.get("OES_standard_derivatives")},supportsCompressedTextureS3TC:function(){return console.warn("THREE.WebGLRenderer: .supportsCompressedTextureS3TC() is now .extensions.get( 'WEBGL_compressed_texture_s3tc' )."),this.extensions.get("WEBGL_compressed_texture_s3tc")},supportsCompressedTexturePVRTC:function(){return console.warn("THREE.WebGLRenderer: .supportsCompressedTexturePVRTC() is now .extensions.get( 'WEBGL_compressed_texture_pvrtc' )."),this.extensions.get("WEBGL_compressed_texture_pvrtc")},supportsBlendMinMax:function(){return console.warn("THREE.WebGLRenderer: .supportsBlendMinMax() is now .extensions.get( 'EXT_blend_minmax' )."),this.extensions.get("EXT_blend_minmax")},supportsVertexTextures:function(){return console.warn("THREE.WebGLRenderer: .supportsVertexTextures() is now .capabilities.vertexTextures."),this.capabilities.vertexTextures},supportsInstancedArrays:function(){return console.warn("THREE.WebGLRenderer: .supportsInstancedArrays() is now .extensions.get( 'ANGLE_instanced_arrays' )."),this.extensions.get("ANGLE_instanced_arrays")},enableScissorTest:function(e){console.warn("THREE.WebGLRenderer: .enableScissorTest() is now .setScissorTest()."),this.setScissorTest(e)},initMaterial:function(){console.warn("THREE.WebGLRenderer: .initMaterial() has been removed.")},addPrePlugin:function(){console.warn("THREE.WebGLRenderer: .addPrePlugin() has been removed.")},addPostPlugin:function(){console.warn("THREE.WebGLRenderer: .addPostPlugin() has been removed.")},updateShadowMap:function(){console.warn("THREE.WebGLRenderer: .updateShadowMap() has been removed.")},setFaceCulling:function(){console.warn("THREE.WebGLRenderer: .setFaceCulling() has been removed.")},allocTextureUnit:function(){console.warn("THREE.WebGLRenderer: .allocTextureUnit() has been removed.")},setTexture:function(){console.warn("THREE.WebGLRenderer: .setTexture() has been removed.")},setTexture2D:function(){console.warn("THREE.WebGLRenderer: .setTexture2D() has been removed.")},setTextureCube:function(){console.warn("THREE.WebGLRenderer: .setTextureCube() has been removed.")},getActiveMipMapLevel:function(){return console.warn("THREE.WebGLRenderer: .getActiveMipMapLevel() is now .getActiveMipmapLevel()."),this.getActiveMipmapLevel()}}),Object.defineProperties(Ns.prototype,{shadowMapEnabled:{get:function(){return this.shadowMap.enabled},set:function(e){console.warn("THREE.WebGLRenderer: .shadowMapEnabled is now .shadowMap.enabled."),this.shadowMap.enabled=e}},shadowMapType:{get:function(){return this.shadowMap.type},set:function(e){console.warn("THREE.WebGLRenderer: .shadowMapType is now .shadowMap.type."),this.shadowMap.type=e}},shadowMapCullFace:{get:function(){console.warn("THREE.WebGLRenderer: .shadowMapCullFace has been removed. Set Material.shadowSide instead.")},set:function(){console.warn("THREE.WebGLRenderer: .shadowMapCullFace has been removed. Set Material.shadowSide instead.")}},context:{get:function(){return console.warn("THREE.WebGLRenderer: .context has been removed. Use .getContext() instead."),this.getContext()}},vr:{get:function(){return console.warn("THREE.WebGLRenderer: .vr has been renamed to .xr"),this.xr}},gammaInput:{get:function(){return console.warn("THREE.WebGLRenderer: .gammaInput has been removed. Set the encoding for textures via Texture.encoding instead."),!1},set:function(){console.warn("THREE.WebGLRenderer: .gammaInput has been removed. Set the encoding for textures via Texture.encoding instead.")}},gammaOutput:{get:function(){return console.warn("THREE.WebGLRenderer: .gammaOutput has been removed. Set WebGLRenderer.outputEncoding instead."),!1},set:function(e){console.warn("THREE.WebGLRenderer: .gammaOutput has been removed. Set WebGLRenderer.outputEncoding instead."),this.outputEncoding=!0===e?en:$t}}}),Object.defineProperties(Ms.prototype,{cullFace:{get:function(){console.warn("THREE.WebGLRenderer: .shadowMap.cullFace has been removed. Set Material.shadowSide instead.")},set:function(){console.warn("THREE.WebGLRenderer: .shadowMap.cullFace has been removed. Set Material.shadowSide instead.")}},renderReverseSided:{get:function(){console.warn("THREE.WebGLRenderer: .shadowMap.renderReverseSided has been removed. Set Material.shadowSide instead.")},set:function(){console.warn("THREE.WebGLRenderer: .shadowMap.renderReverseSided has been removed. Set Material.shadowSide instead.")}},renderSingleSided:{get:function(){console.warn("THREE.WebGLRenderer: .shadowMap.renderSingleSided has been removed. Set Material.shadowSide instead.")},set:function(){console.warn("THREE.WebGLRenderer: .shadowMap.renderSingleSided has been removed. Set Material.shadowSide instead.")}}}),Object.defineProperties(qn.prototype,{wrapS:{get:function(){return console.warn("THREE.WebGLRenderTarget: .wrapS is now .texture.wrapS."),this.texture.wrapS},set:function(e){console.warn("THREE.WebGLRenderTarget: .wrapS is now .texture.wrapS."),this.texture.wrapS=e}},wrapT:{get:function(){return console.warn("THREE.WebGLRenderTarget: .wrapT is now .texture.wrapT."),this.texture.wrapT},set:function(e){console.warn("THREE.WebGLRenderTarget: .wrapT is now .texture.wrapT."),this.texture.wrapT=e}},magFilter:{get:function(){return console.warn("THREE.WebGLRenderTarget: .magFilter is now .texture.magFilter."),this.texture.magFilter},set:function(e){console.warn("THREE.WebGLRenderTarget: .magFilter is now .texture.magFilter."),this.texture.magFilter=e}},minFilter:{get:function(){return console.warn("THREE.WebGLRenderTarget: .minFilter is now .texture.minFilter."),this.texture.minFilter},set:function(e){console.warn("THREE.WebGLRenderTarget: .minFilter is now .texture.minFilter."),this.texture.minFilter=e}},anisotropy:{get:function(){return console.warn("THREE.WebGLRenderTarget: .anisotropy is now .texture.anisotropy."),this.texture.anisotropy},set:function(e){console.warn("THREE.WebGLRenderTarget: .anisotropy is now .texture.anisotropy."),this.texture.anisotropy=e}},offset:{get:function(){return console.warn("THREE.WebGLRenderTarget: .offset is now .texture.offset."),this.texture.offset},set:function(e){console.warn("THREE.WebGLRenderTarget: .offset is now .texture.offset."),this.texture.offset=e}},repeat:{get:function(){return console.warn("THREE.WebGLRenderTarget: .repeat is now .texture.repeat."),this.texture.repeat},set:function(e){console.warn("THREE.WebGLRenderTarget: .repeat is now .texture.repeat."),this.texture.repeat=e}},format:{get:function(){return console.warn("THREE.WebGLRenderTarget: .format is now .texture.format."),this.texture.format},set:function(e){console.warn("THREE.WebGLRenderTarget: .format is now .texture.format."),this.texture.format=e}},type:{get:function(){return console.warn("THREE.WebGLRenderTarget: .type is now .texture.type."),this.texture.type},set:function(e){console.warn("THREE.WebGLRenderTarget: .type is now .texture.type."),this.texture.type=e}},generateMipmaps:{get:function(){return console.warn("THREE.WebGLRenderTarget: .generateMipmaps is now .texture.generateMipmaps."),this.texture.generateMipmaps},set:function(e){console.warn("THREE.WebGLRenderTarget: .generateMipmaps is now .texture.generateMipmaps."),this.texture.generateMipmaps=e}}}),Object.defineProperties(th.prototype,{load:{value:function(e){console.warn("THREE.Audio: .load has been deprecated. Use THREE.AudioLoader instead.");var t=this;return(new Vd).load(e,(function(e){t.setBuffer(e)})),this}},startTime:{set:function(){console.warn("THREE.Audio: .startTime is now .play( delay ).")}}}),sh.prototype.getData=function(){return console.warn("THREE.AudioAnalyser: .getData() is now .getFrequencyData()."),this.getFrequencyData()},wo.prototype.updateCubeMap=function(e,t){return console.warn("THREE.CubeCamera: .updateCubeMap() is now .update()."),this.update(e,t)};var fp={merge:function(e,t,n){var r;console.warn("THREE.GeometryUtils: .merge() has been moved to Geometry. Use geometry.merge( geometry2, matrix, materialIndexOffset ) instead."),t.isMesh&&(t.matrixAutoUpdate&&t.updateMatrix(),r=t.matrix,t=t.geometry),e.merge(t,r,n)},center:function(e){return console.warn("THREE.GeometryUtils: .center() has been moved to Geometry. Use geometry.center() instead."),e.center()}};function pp(){console.error("THREE.CanvasRenderer has been removed")}function gp(){console.error("THREE.JSONLoader has been removed.")}Yn.crossOrigin=void 0,Yn.loadTexture=function(e,t,n,r){console.warn("THREE.ImageUtils.loadTexture has been deprecated. Use THREE.TextureLoader() instead.");var i=new Uc;i.setCrossOrigin(this.crossOrigin);var o=i.load(e,n,void 0,r);return t&&(o.mapping=t),o},Yn.loadTextureCube=function(e,t,n,r){console.warn("THREE.ImageUtils.loadTextureCube has been deprecated. Use THREE.CubeTextureLoader() instead.");var i=new Hc;i.setCrossOrigin(this.crossOrigin);var o=i.load(e,n,void 0,r);return t&&(o.mapping=t),o},Yn.loadCompressedTexture=function(){console.error("THREE.ImageUtils.loadCompressedTexture has been removed. Use THREE.DDSLoader instead.")},Yn.loadCompressedTextureCube=function(){console.error("THREE.ImageUtils.loadCompressedTextureCube has been removed. Use THREE.DDSLoader instead.")};var vp={createMultiMaterialObject:function(){console.error("THREE.SceneUtils has been moved to /examples/jsm/utils/SceneUtils.js")},detach:function(){console.error("THREE.SceneUtils has been moved to /examples/jsm/utils/SceneUtils.js")},attach:function(){console.error("THREE.SceneUtils has been moved to /examples/jsm/utils/SceneUtils.js")}};function mp(){console.error("THREE.LensFlare has been moved to /examples/jsm/objects/Lensflare.js")}"undefined"!=typeof __THREE_DEVTOOLS__&&__THREE_DEVTOOLS__.dispatchEvent(new CustomEvent("register",{detail:{revision:r}}))},function(e,t,n){"use strict";function r(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}n.d(t,"a",(function(){return r}))},function(e,t,n){"use strict";!function e(){if("undefined"!=typeof __REACT_DEVTOOLS_GLOBAL_HOOK__&&"function"==typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE){0;try{__REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE(e)}catch(e){console.error(e)}}}(),e.exports=n(1451)},function(e,t,n){"use strict";var r,i=n(17),o=n(16),a=n(0),s=n.n(a),l=n(22),u=n.n(l),c=n(54),d=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(null!=e&&"function"==typeof Object.getOwnPropertySymbols){var i=0;for(r=Object.getOwnPropertySymbols(e);i<r.length;i++)t.indexOf(r[i])<0&&Object.prototype.propertyIsEnumerable.call(e,r[i])&&(n[r[i]]=e[r[i]])}return n};!function(e){const t={breadCrumb:{container:{$nest:{"&:first-child svg":{bottom:"1px",marginLeft:"0px",position:"relative"},"&:hover":{backgroundColor:"var(--jp-layout-color2)"},".jp-mod-dropTarget&":{backgroundColor:"var(--jp-brand-color2)",opacity:.7}}},element:{borderRadius:"var(--jp-border-radius)",cursor:"pointer",margin:"0px 2px",padding:"0px 2px",height:"16px",width:"16px",verticalAlign:"middle"}},commandPaletteHeader:{container:{height:"14px",margin:"0 14px 0 auto"},element:{height:"14px",width:"14px"},options:{elementPosition:"center"}},commandPaletteItem:{element:{height:"16px",width:"16px"},options:{elementPosition:"center"}},launcherCard:{container:{height:"52px",width:"52px"},element:{height:"52px",width:"52px"},options:{elementPosition:"center"}},launcherSection:{container:{boxSizing:"border-box",marginRight:"12px",height:"32px",width:"32px"},element:{height:"32px",width:"32px"},options:{elementPosition:"center"}},listing:{container:{flex:"0 0 20px",marginRight:"4px",position:"relative"},element:{height:"16px",width:"16px"},options:{elementPosition:"center"}},listingHeaderItem:{container:{display:"inline",height:"16px",width:"16px"},element:{height:"auto",margin:"-2px 0 0 0",width:"20px"},options:{elementPosition:"center"}},mainAreaTab:{container:{$nest:{".lm-DockPanel-tabBar &":{marginRight:"4px"},"#tab-manager &":{marginRight:"2px",position:"relative"}}},element:{$nest:{".lm-DockPanel-tabBar &":{height:"14px",width:"14px"},"#tab-manager &":{height:"16px",width:"16px"}}},options:{elementPosition:"center"}},menuItem:{container:{display:"inline-block",verticalAlign:"middle"},element:{height:"16px",width:"16px"},options:{elementPosition:"center"}},runningItem:{container:{margin:"0px 4px 0px 4px"},element:{height:"16px",width:"16px"},options:{elementPosition:"center"}},select:{container:{pointerEvents:"none"},element:{position:"absolute",height:"auto",width:"16px"}},settingsEditor:{container:{display:"inline-block",flex:"0 0 20px",margin:"0 3px 0 0",position:"relative",height:"20px",width:"20px"},element:{height:"16px",width:"16px"},options:{elementPosition:"center"}},sideBar:{container:{$nest:{".jp-SideBar.jp-mod-left .lm-TabBar-tab &":{transform:"rotate(90deg)"},".jp-SideBar.jp-mod-left .lm-TabBar-tab.lm-mod-current &":{transform:"rotate(90deg)\n    translate(\n      calc(-0.5 * var(--jp-border-width)),\n      calc(-0.5 * var(--jp-border-width))\n    )"},".jp-SideBar.jp-mod-right .lm-TabBar-tab &":{transform:"rotate(-90deg)"},".jp-SideBar.jp-mod-right .lm-TabBar-tab.lm-mod-current &":{transform:"rotate(-90deg)\n    translate(\n      calc(0.5 * var(--jp-border-width)),\n      calc(-0.5 * var(--jp-border-width))\n    )"}}},element:{height:"auto",width:"20px"},options:{elementPosition:"center"}},splash:{container:{animation:"0.3s fade-in linear forwards",height:"100%",width:"100%",zIndex:1},element:{width:"100px"},options:{elementPosition:"center"}},statusBar:{element:{left:"0px",top:"0px",height:"18px",width:"20px",position:"relative"}},toolbarButton:{container:{display:"inline-block",margin:"auto",verticalAlign:"middle"},element:{height:"16px",width:"16px"},options:{elementPosition:"center"}}};function n(e){return{container:{alignItems:"center",display:"flex"},element:Object.assign({display:"block"},e)}}const r={center:n({margin:"0 auto",width:"100%"}),top:n({margin:"0 0 auto 0"}),right:n({margin:"0 0 0 auto"}),bottom:n({margin:"auto 0 0 0"}),left:n({margin:"0 auto 0 0"}),"top right":n({margin:"0 0 auto auto"}),"bottom right":n({margin:"auto 0 0 auto"}),"bottom left":n({margin:"auto auto 0 0"}),"top left":n({margin:"0 auto 0 auto"})};function i(e){return{element:{height:e,width:e}}}const o={small:i("14px"),normal:i("16px"),large:i("20px"),xlarge:i("24px")};function a(e){const t=Object.assign({},...e.map(e=>e.options));return t.elementPosition&&e.unshift(r[t.elementPosition]),t.elementSize&&e.unshift(o[t.elementSize]),function(e){return{container:Object.assign({},...e.map(e=>e.container)),element:Object.assign({},...e.map(e=>e.element))}}(e)}const s=new Map;e.styleClass=function(e){if(!e||0===Object.keys(e).length)return"";let{elementPosition:n,elementSize:r,stylesheet:i,kind:o,justify:l}=e,u=d(e,["elementPosition","elementSize","stylesheet","kind","justify"]);i||(i=o),n||(n=l);const h=Object.assign(Object.assign({},n&&{elementPosition:n}),r&&{elementSize:r}),f="string"==typeof i&&0===Object.keys(u).length,p=f?[i,n,r].join(","):"";if(f&&s.has(p))return s.get(p);const g=function(e){return e?(Array.isArray(e)||(e=[e]),e.map(e=>"string"==typeof e?t[e]:e)):[]}(i);g.push({element:u,options:h});const v=function(e){var t;return Object(c.style)(Object.assign(Object.assign({},e.container),{$nest:Object.assign(Object.assign({},null===(t=e.container)||void 0===t?void 0:t.$nest),{svg:e.element})}))}(a(g));return f&&s.set(p,v),v}}(r||(r={}));var h=n(128),f=n(1204),p=n.n(f),g=n(1205),v=n.n(g),m=n(441),y=n.n(m);n.d(t,"a",(function(){return A}));var b,C=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(null!=e&&"function"==typeof Object.getOwnPropertySymbols){var i=0;for(r=Object.getOwnPropertySymbols(e);i<r.length;i++)t.indexOf(r[i])<0&&Object.prototype.propertyIsEnumerable.call(e,r[i])&&(n[r[i]]=e[r[i]])}return n};class A{constructor({name:e,svgstr:t,render:n,unrender:r,_loading:i=!1}){if(this._props={},this._svgReplaced=new o.Signal(this),this._svgElement=void 0,this._svgInnerHTML=void 0,this._svgReactAttrs=void 0,!e||!t)return console.error(`When defining a new LabIcon, name and svgstr must both be non-empty strings. name: ${e}, svgstr: ${t}`),_;if(this._loading=i,A._instances.has(e)){const n=A._instances.get(e);return this._loading?(n.svgstr=t,this._loading=!1,n):(console.warn(`Redefining previously loaded icon svgstr. name: ${e}, svgstrOld: ${n.svgstr}, svgstr: ${t}`),n.svgstr=t,n)}this.name=e,this.react=this._initReact(e),this.svgstr=t,this._initRender({render:n,unrender:r}),A._instances.set(this.name,this)}static remove(e){for(;e.firstChild;)e.firstChild.remove();return e.className="",e}static resolve({icon:e}){if(e instanceof A)return e;if("string"==typeof e){const t=A._instances.get(e);return t||(A._debug&&console.warn("Lookup failed for icon, creating loading icon. icon: "+e),new A({name:e,svgstr:y.a,_loading:!0}))}return new A(e)}static resolveElement(e){var{icon:t,iconClass:n,fallback:r}=e,i=C(e,["icon","iconClass","fallback"]);return b.isResolvable(t)?A.resolve({icon:t}).element(i):!n&&r?r.element(i):(i.className=Object(h.a)(n,i.className),b.blankElement(i))}static resolveReact(e){var{icon:t,iconClass:n,fallback:r}=e,i=C(e,["icon","iconClass","fallback"]);if(!b.isResolvable(t))return!n&&r?s.a.createElement(r.react,Object.assign({},i)):(i.className=Object(h.a)(n,i.className),s.a.createElement(b.blankReact,Object.assign({},i)));const o=A.resolve({icon:t});return s.a.createElement(o.react,Object.assign({},i))}static resolveSvg({name:e,svgstr:t}){const n=(new DOMParser).parseFromString(b.svgstrShim(t),"image/svg+xml"),r=n.querySelector("parsererror");if(r){const n=`SVG HTML was malformed for LabIcon instance.\nname: ${e}, svgstr: ${t}`;return A._debug?(console.error(n),r):(console.warn(n),null)}return n.documentElement}static toggleDebug(e){A._debug=null!=e?e:!A._debug}bindprops(e){const t=Object.create(this);return t._props=e,t.react=t._initReact(t.name+"_bind"),t}element(e={}){var t;let n=Object.assign(Object.assign({},this._props),e),{className:r,container:i,label:o,title:a,tag:s="div"}=n,l=C(n,["className","container","label","title","tag"]);const u=null==i?void 0:i.firstChild;if((null===(t=null==u?void 0:u.dataset)||void 0===t?void 0:t.iconId)===this._uuid)return u;if(!this.svgElement)return document.createElement("div");let c=!0;if(i)for(;i.firstChild;)i.firstChild.remove();else i=document.createElement(s),c=!1;null!=o&&(i.textContent=o),b.initContainer({container:i,className:r,styleProps:l,title:a});const d=this.svgElement.cloneNode(!0);return i.appendChild(d),c?d:i}render(e,t){var n;let r=null===(n=null==t?void 0:t.children)||void 0===n?void 0:n[0];"string"!=typeof r&&(r=void 0),this.element(Object.assign({container:e,label:r},null==t?void 0:t.props))}get svgElement(){return void 0===this._svgElement&&(this._svgElement=this._initSvg({uuid:this._uuid})),this._svgElement}get svgInnerHTML(){return void 0===this._svgInnerHTML&&(null===this.svgElement?this._svgInnerHTML=null:this._svgInnerHTML=this.svgElement.innerHTML),this._svgInnerHTML}get svgReactAttrs(){return void 0===this._svgReactAttrs&&(null===this.svgElement?this._svgReactAttrs=null:this._svgReactAttrs=Object(h.b)(this.svgElement,{ignore:["data-icon-id"]})),this._svgReactAttrs}get svgstr(){return this._svgstr}set svgstr(e){this._svgstr=e;const t=i.UUID.uuid4(),n=this._uuid;this._uuid=t,this._svgElement=void 0,this._svgInnerHTML=void 0,this._svgReactAttrs=void 0,document.querySelectorAll(`[data-icon-id="${n}"]`).forEach(e=>{this.svgElement&&e.replaceWith(this.svgElement.cloneNode(!0))}),this._svgReplaced.emit()}_initReact(e){const t=s.a.forwardRef((e={},t)=>{const n=Object.assign(Object.assign({},this._props),e),{className:i,container:o,label:a,title:l,tag:u="div"}=n,c=C(n,["className","container","label","title","tag"]),[,d]=s.a.useState(this._uuid);s.a.useEffect(()=>{const e=()=>{d(this._uuid)};return this._svgReplaced.connect(e),()=>{this._svgReplaced.disconnect(e)}});const f=u;if(!this.svgInnerHTML||!this.svgReactAttrs)return s.a.createElement(s.a.Fragment,null);const p=s.a.createElement("svg",Object.assign({},this.svgReactAttrs,{dangerouslySetInnerHTML:{__html:this.svgInnerHTML},ref:t}));return o?(b.initContainer({container:o,className:i,styleProps:c,title:l}),s.a.createElement(s.a.Fragment,null,p,a)):s.a.createElement(f,{className:Object(h.a)(i,r.styleClass(c))},p,a)});return t.displayName="LabIcon_"+e,t}_initRender({render:e,unrender:t}){e?(this.render=e,t&&(this.unrender=t)):t&&console.warn("In _initRender, ignoring unrender arg since render is undefined")}_initSvg({title:e,uuid:t}={}){const n=A.resolveSvg(this);return n?("parsererror"!==n.tagName&&(n.dataset.icon=this.name,t&&(n.dataset.iconId=t),e&&b.setTitleSvg(n,e)),n):n}}A._debug=!1,A._instances=new Map,function(e){function t({container:e,className:t,styleProps:n,title:i}){null!=i&&(e.title=i);const o=r.styleClass(n);if(null!=t){const n=Object(h.a)(t,o);return e.className=n,n}return o?(e.classList.add(o),o):""}e.blankElement=function(t){var{className:n="",container:r,label:i,title:o,tag:a="div"}=t,s=C(t,["className","container","label","title","tag"]);if((null==r?void 0:r.className)===n)return r;if(r)for(;r.firstChild;)r.firstChild.remove();else r=document.createElement(a);return null!=i&&(r.textContent=i),e.initContainer({container:r,className:n,styleProps:s,title:o}),r},e.blankReact=s.a.forwardRef((e,n)=>{var{className:i="",container:o,label:a,title:l,tag:u="div"}=e,c=C(e,["className","container","label","title","tag"]);const d=u;return o?(t({container:o,className:i,styleProps:c,title:l}),s.a.createElement(s.a.Fragment,null)):s.a.createElement(d,{className:Object(h.a)(i,r.styleClass(c))},n&&I.react({ref:n}),a)}),e.blankReact.displayName="BlankReact",e.initContainer=t,e.isResolvable=function(e){return!(!e||!("string"==typeof e||e.name&&e.svgstr))},e.setTitleSvg=function(e,t){const n=e.getElementsByTagName("title");if(n.length)n[0].textContent=t;else{const n=document.createElement("title");n.textContent=t,e.appendChild(n)}},e.svgstrShim=function(e,t=!0){const[,n,r]=decodeURIComponent(e).replace(/>\s*\n\s*</g,"><").replace(/\s*\n\s*/g," ").match(t?/^(?:data:.*?(;base64)?,)?(.*)/:/(?:(base64).*)?(<svg.*)/);return n?atob(r):r};class n{constructor(e,t){this._icon=e,this._rendererOptions=t}render(e,t){}}e.Renderer=n;e.ElementRenderer=class extends n{render(e,t){var n,r;let i=null===(n=null==t?void 0:t.children)||void 0===n?void 0:n[0];"string"!=typeof i&&(i=void 0),this._icon.element(Object.assign(Object.assign({container:e,label:i},null===(r=this._rendererOptions)||void 0===r?void 0:r.props),null==t?void 0:t.props))}};e.ReactRenderer=class extends n{render(e,t){var n,r;let i=null===(n=null==t?void 0:t.children)||void 0===n?void 0:n[0];"string"!=typeof i&&(i=void 0),u.a.render(s.a.createElement(this._icon.react,Object.assign({container:e,label:i},Object.assign(Object.assign({},null===(r=this._rendererOptions)||void 0===r?void 0:r.props),null==t?void 0:t.props))),e)}unrender(e){u.a.unmountComponentAtNode(e)}}}(b||(b={}));const _=new A({name:"ui-components:bad",svgstr:p.a}),I=new A({name:"ui-components:blank",svgstr:v.a})},function(e,t,n){e.exports=function(){"use strict";var e=navigator.userAgent,t=navigator.platform,n=/gecko\/\d/i.test(e),r=/MSIE \d/.test(e),i=/Trident\/(?:[7-9]|\d{2,})\..*rv:(\d+)/.exec(e),o=/Edge\/(\d+)/.exec(e),a=r||i||o,s=a&&(r?document.documentMode||6:+(o||i)[1]),l=!o&&/WebKit\//.test(e),u=l&&/Qt\/\d+\.\d+/.test(e),c=!o&&/Chrome\//.test(e),d=/Opera\//.test(e),h=/Apple Computer/.test(navigator.vendor),f=/Mac OS X 1\d\D([8-9]|\d\d)\D/.test(e),p=/PhantomJS/.test(e),g=!o&&/AppleWebKit/.test(e)&&/Mobile\/\w+/.test(e),v=/Android/.test(e),m=g||v||/webOS|BlackBerry|Opera Mini|Opera Mobi|IEMobile/i.test(e),y=g||/Mac/.test(t),b=/\bCrOS\b/.test(e),C=/win/i.test(t),A=d&&e.match(/Version\/(\d*\.\d*)/);A&&(A=Number(A[1])),A&&A>=15&&(d=!1,l=!0);var _=y&&(u||d&&(null==A||A<12.11)),I=n||a&&s>=9;function x(e){return new RegExp("(^|\\s)"+e+"(?:$|\\s)\\s*")}var w,M=function(e,t){var n=e.className,r=x(t).exec(n);if(r){var i=n.slice(r.index+r[0].length);e.className=n.slice(0,r.index)+(i?r[1]+i:"")}};function S(e){for(var t=e.childNodes.length;t>0;--t)e.removeChild(e.firstChild);return e}function T(e,t){return S(e).appendChild(t)}function E(e,t,n,r){var i=document.createElement(e);if(n&&(i.className=n),r&&(i.style.cssText=r),"string"==typeof t)i.appendChild(document.createTextNode(t));else if(t)for(var o=0;o<t.length;++o)i.appendChild(t[o]);return i}function O(e,t,n,r){var i=E(e,t,n,r);return i.setAttribute("role","presentation"),i}function k(e,t){if(3==t.nodeType&&(t=t.parentNode),e.contains)return e.contains(t);do{if(11==t.nodeType&&(t=t.host),t==e)return!0}while(t=t.parentNode)}function L(){var e;try{e=document.activeElement}catch(t){e=document.body||null}for(;e&&e.shadowRoot&&e.shadowRoot.activeElement;)e=e.shadowRoot.activeElement;return e}function D(e,t){var n=e.className;x(t).test(n)||(e.className+=(n?" ":"")+t)}function P(e,t){for(var n=e.split(" "),r=0;r<n.length;r++)n[r]&&!x(n[r]).test(t)&&(t+=" "+n[r]);return t}w=document.createRange?function(e,t,n,r){var i=document.createRange();return i.setEnd(r||e,n),i.setStart(e,t),i}:function(e,t,n){var r=document.body.createTextRange();try{r.moveToElementText(e.parentNode)}catch(e){return r}return r.collapse(!0),r.moveEnd("character",n),r.moveStart("character",t),r};var N=function(e){e.select()};function j(e){var t=Array.prototype.slice.call(arguments,1);return function(){return e.apply(null,t)}}function R(e,t,n){for(var r in t||(t={}),e)!e.hasOwnProperty(r)||!1===n&&t.hasOwnProperty(r)||(t[r]=e[r]);return t}function z(e,t,n,r,i){null==t&&-1==(t=e.search(/[^\s\u00a0]/))&&(t=e.length);for(var o=r||0,a=i||0;;){var s=e.indexOf("\t",o);if(s<0||s>=t)return a+(t-o);a+=s-o,a+=n-a%n,o=s+1}}g?N=function(e){e.selectionStart=0,e.selectionEnd=e.value.length}:a&&(N=function(e){try{e.select()}catch(e){}});var F=function(){this.id=null,this.f=null,this.time=0,this.handler=j(this.onTimeout,this)};function B(e,t){for(var n=0;n<e.length;++n)if(e[n]==t)return n;return-1}F.prototype.onTimeout=function(e){e.id=0,e.time<=+new Date?e.f():setTimeout(e.handler,e.time-+new Date)},F.prototype.set=function(e,t){this.f=t;var n=+new Date+e;(!this.id||n<this.time)&&(clearTimeout(this.id),this.id=setTimeout(this.handler,e),this.time=n)};var H={toString:function(){return"CodeMirror.Pass"}},U={scroll:!1},V={origin:"*mouse"},Y={origin:"+move"};function W(e,t,n){for(var r=0,i=0;;){var o=e.indexOf("\t",r);-1==o&&(o=e.length);var a=o-r;if(o==e.length||i+a>=t)return r+Math.min(a,t-i);if(i+=o-r,r=o+1,(i+=n-i%n)>=t)return r}}var G=[""];function X(e){for(;G.length<=e;)G.push(q(G)+" ");return G[e]}function q(e){return e[e.length-1]}function Z(e,t){for(var n=[],r=0;r<e.length;r++)n[r]=t(e[r],r);return n}function Q(){}function K(e,t){var n;return Object.create?n=Object.create(e):(Q.prototype=e,n=new Q),t&&R(t,n),n}var J=/[\u00df\u0587\u0590-\u05f4\u0600-\u06ff\u3040-\u309f\u30a0-\u30ff\u3400-\u4db5\u4e00-\u9fcc\uac00-\ud7af]/;function $(e){return/\w/.test(e)||e>""&&(e.toUpperCase()!=e.toLowerCase()||J.test(e))}function ee(e,t){return t?!!(t.source.indexOf("\\w")>-1&&$(e))||t.test(e):$(e)}function te(e){for(var t in e)if(e.hasOwnProperty(t)&&e[t])return!1;return!0}var ne=/[\u0300-\u036f\u0483-\u0489\u0591-\u05bd\u05bf\u05c1\u05c2\u05c4\u05c5\u05c7\u0610-\u061a\u064b-\u065e\u0670\u06d6-\u06dc\u06de-\u06e4\u06e7\u06e8\u06ea-\u06ed\u0711\u0730-\u074a\u07a6-\u07b0\u07eb-\u07f3\u0816-\u0819\u081b-\u0823\u0825-\u0827\u0829-\u082d\u0900-\u0902\u093c\u0941-\u0948\u094d\u0951-\u0955\u0962\u0963\u0981\u09bc\u09be\u09c1-\u09c4\u09cd\u09d7\u09e2\u09e3\u0a01\u0a02\u0a3c\u0a41\u0a42\u0a47\u0a48\u0a4b-\u0a4d\u0a51\u0a70\u0a71\u0a75\u0a81\u0a82\u0abc\u0ac1-\u0ac5\u0ac7\u0ac8\u0acd\u0ae2\u0ae3\u0b01\u0b3c\u0b3e\u0b3f\u0b41-\u0b44\u0b4d\u0b56\u0b57\u0b62\u0b63\u0b82\u0bbe\u0bc0\u0bcd\u0bd7\u0c3e-\u0c40\u0c46-\u0c48\u0c4a-\u0c4d\u0c55\u0c56\u0c62\u0c63\u0cbc\u0cbf\u0cc2\u0cc6\u0ccc\u0ccd\u0cd5\u0cd6\u0ce2\u0ce3\u0d3e\u0d41-\u0d44\u0d4d\u0d57\u0d62\u0d63\u0dca\u0dcf\u0dd2-\u0dd4\u0dd6\u0ddf\u0e31\u0e34-\u0e3a\u0e47-\u0e4e\u0eb1\u0eb4-\u0eb9\u0ebb\u0ebc\u0ec8-\u0ecd\u0f18\u0f19\u0f35\u0f37\u0f39\u0f71-\u0f7e\u0f80-\u0f84\u0f86\u0f87\u0f90-\u0f97\u0f99-\u0fbc\u0fc6\u102d-\u1030\u1032-\u1037\u1039\u103a\u103d\u103e\u1058\u1059\u105e-\u1060\u1071-\u1074\u1082\u1085\u1086\u108d\u109d\u135f\u1712-\u1714\u1732-\u1734\u1752\u1753\u1772\u1773\u17b7-\u17bd\u17c6\u17c9-\u17d3\u17dd\u180b-\u180d\u18a9\u1920-\u1922\u1927\u1928\u1932\u1939-\u193b\u1a17\u1a18\u1a56\u1a58-\u1a5e\u1a60\u1a62\u1a65-\u1a6c\u1a73-\u1a7c\u1a7f\u1b00-\u1b03\u1b34\u1b36-\u1b3a\u1b3c\u1b42\u1b6b-\u1b73\u1b80\u1b81\u1ba2-\u1ba5\u1ba8\u1ba9\u1c2c-\u1c33\u1c36\u1c37\u1cd0-\u1cd2\u1cd4-\u1ce0\u1ce2-\u1ce8\u1ced\u1dc0-\u1de6\u1dfd-\u1dff\u200c\u200d\u20d0-\u20f0\u2cef-\u2cf1\u2de0-\u2dff\u302a-\u302f\u3099\u309a\ua66f-\ua672\ua67c\ua67d\ua6f0\ua6f1\ua802\ua806\ua80b\ua825\ua826\ua8c4\ua8e0-\ua8f1\ua926-\ua92d\ua947-\ua951\ua980-\ua982\ua9b3\ua9b6-\ua9b9\ua9bc\uaa29-\uaa2e\uaa31\uaa32\uaa35\uaa36\uaa43\uaa4c\uaab0\uaab2-\uaab4\uaab7\uaab8\uaabe\uaabf\uaac1\uabe5\uabe8\uabed\udc00-\udfff\ufb1e\ufe00-\ufe0f\ufe20-\ufe26\uff9e\uff9f]/;function re(e){return e.charCodeAt(0)>=768&&ne.test(e)}function ie(e,t,n){for(;(n<0?t>0:t<e.length)&&re(e.charAt(t));)t+=n;return t}function oe(e,t,n){for(var r=t>n?-1:1;;){if(t==n)return t;var i=(t+n)/2,o=r<0?Math.ceil(i):Math.floor(i);if(o==t)return e(o)?t:n;e(o)?n=o:t=o+r}}var ae=null;function se(e,t,n){var r;ae=null;for(var i=0;i<e.length;++i){var o=e[i];if(o.from<t&&o.to>t)return i;o.to==t&&(o.from!=o.to&&"before"==n?r=i:ae=i),o.from==t&&(o.from!=o.to&&"before"!=n?r=i:ae=i)}return null!=r?r:ae}var le=function(){var e=/[\u0590-\u05f4\u0600-\u06ff\u0700-\u08ac]/,t=/[stwN]/,n=/[LRr]/,r=/[Lb1n]/,i=/[1n]/;function o(e,t,n){this.level=e,this.from=t,this.to=n}return function(a,s){var l="ltr"==s?"L":"R";if(0==a.length||"ltr"==s&&!e.test(a))return!1;for(var u,c=a.length,d=[],h=0;h<c;++h)d.push((u=a.charCodeAt(h))<=247?"bbbbbbbbbtstwsbbbbbbbbbbbbbbssstwNN%%%NNNNNN,N,N1111111111NNNNNNNLLLLLLLLLLLLLLLLLLLLLLLLLLNNNNNNLLLLLLLLLLLLLLLLLLLLLLLLLLNNNNbbbbbbsbbbbbbbbbbbbbbbbbbbbbbbbbb,N%%%%NNNNLNNNNN%%11NLNNN1LNNNNNLLLLLLLLLLLLLLLLLLLLLLLNLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLN".charAt(u):1424<=u&&u<=1524?"R":1536<=u&&u<=1785?"nnnnnnNNr%%r,rNNmmmmmmmmmmmrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrmmmmmmmmmmmmmmmmmmmmmnnnnnnnnnn%nnrrrmrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrmmmmmmmnNmmmmmmrrmmNmmmmrr1111111111".charAt(u-1536):1774<=u&&u<=2220?"r":8192<=u&&u<=8203?"w":8204==u?"b":"L");for(var f=0,p=l;f<c;++f){var g=d[f];"m"==g?d[f]=p:p=g}for(var v=0,m=l;v<c;++v){var y=d[v];"1"==y&&"r"==m?d[v]="n":n.test(y)&&(m=y,"r"==y&&(d[v]="R"))}for(var b=1,C=d[0];b<c-1;++b){var A=d[b];"+"==A&&"1"==C&&"1"==d[b+1]?d[b]="1":","!=A||C!=d[b+1]||"1"!=C&&"n"!=C||(d[b]=C),C=A}for(var _=0;_<c;++_){var I=d[_];if(","==I)d[_]="N";else if("%"==I){var x=void 0;for(x=_+1;x<c&&"%"==d[x];++x);for(var w=_&&"!"==d[_-1]||x<c&&"1"==d[x]?"1":"N",M=_;M<x;++M)d[M]=w;_=x-1}}for(var S=0,T=l;S<c;++S){var E=d[S];"L"==T&&"1"==E?d[S]="L":n.test(E)&&(T=E)}for(var O=0;O<c;++O)if(t.test(d[O])){var k=void 0;for(k=O+1;k<c&&t.test(d[k]);++k);for(var L="L"==(O?d[O-1]:l),D=L==("L"==(k<c?d[k]:l))?L?"L":"R":l,P=O;P<k;++P)d[P]=D;O=k-1}for(var N,j=[],R=0;R<c;)if(r.test(d[R])){var z=R;for(++R;R<c&&r.test(d[R]);++R);j.push(new o(0,z,R))}else{var F=R,B=j.length,H="rtl"==s?1:0;for(++R;R<c&&"L"!=d[R];++R);for(var U=F;U<R;)if(i.test(d[U])){F<U&&(j.splice(B,0,new o(1,F,U)),B+=H);var V=U;for(++U;U<R&&i.test(d[U]);++U);j.splice(B,0,new o(2,V,U)),B+=H,F=U}else++U;F<R&&j.splice(B,0,new o(1,F,R))}return"ltr"==s&&(1==j[0].level&&(N=a.match(/^\s+/))&&(j[0].from=N[0].length,j.unshift(new o(0,0,N[0].length))),1==q(j).level&&(N=a.match(/\s+$/))&&(q(j).to-=N[0].length,j.push(new o(0,c-N[0].length,c)))),"rtl"==s?j.reverse():j}}();function ue(e,t){var n=e.order;return null==n&&(n=e.order=le(e.text,t)),n}var ce=[],de=function(e,t,n){if(e.addEventListener)e.addEventListener(t,n,!1);else if(e.attachEvent)e.attachEvent("on"+t,n);else{var r=e._handlers||(e._handlers={});r[t]=(r[t]||ce).concat(n)}};function he(e,t){return e._handlers&&e._handlers[t]||ce}function fe(e,t,n){if(e.removeEventListener)e.removeEventListener(t,n,!1);else if(e.detachEvent)e.detachEvent("on"+t,n);else{var r=e._handlers,i=r&&r[t];if(i){var o=B(i,n);o>-1&&(r[t]=i.slice(0,o).concat(i.slice(o+1)))}}}function pe(e,t){var n=he(e,t);if(n.length)for(var r=Array.prototype.slice.call(arguments,2),i=0;i<n.length;++i)n[i].apply(null,r)}function ge(e,t,n){return"string"==typeof t&&(t={type:t,preventDefault:function(){this.defaultPrevented=!0}}),pe(e,n||t.type,e,t),Ae(t)||t.codemirrorIgnore}function ve(e){var t=e._handlers&&e._handlers.cursorActivity;if(t)for(var n=e.curOp.cursorActivityHandlers||(e.curOp.cursorActivityHandlers=[]),r=0;r<t.length;++r)-1==B(n,t[r])&&n.push(t[r])}function me(e,t){return he(e,t).length>0}function ye(e){e.prototype.on=function(e,t){de(this,e,t)},e.prototype.off=function(e,t){fe(this,e,t)}}function be(e){e.preventDefault?e.preventDefault():e.returnValue=!1}function Ce(e){e.stopPropagation?e.stopPropagation():e.cancelBubble=!0}function Ae(e){return null!=e.defaultPrevented?e.defaultPrevented:0==e.returnValue}function _e(e){be(e),Ce(e)}function Ie(e){return e.target||e.srcElement}function xe(e){var t=e.which;return null==t&&(1&e.button?t=1:2&e.button?t=3:4&e.button&&(t=2)),y&&e.ctrlKey&&1==t&&(t=3),t}var we,Me,Se=function(){if(a&&s<9)return!1;var e=E("div");return"draggable"in e||"dragDrop"in e}();function Te(e){if(null==we){var t=E("span","​");T(e,E("span",[t,document.createTextNode("x")])),0!=e.firstChild.offsetHeight&&(we=t.offsetWidth<=1&&t.offsetHeight>2&&!(a&&s<8))}var n=we?E("span","​"):E("span"," ",null,"display: inline-block; width: 1px; margin-right: -1px");return n.setAttribute("cm-text",""),n}function Ee(e){if(null!=Me)return Me;var t=T(e,document.createTextNode("AخA")),n=w(t,0,1).getBoundingClientRect(),r=w(t,1,2).getBoundingClientRect();return S(e),!(!n||n.left==n.right)&&(Me=r.right-n.right<3)}var Oe,ke=3!="\n\nb".split(/\n/).length?function(e){for(var t=0,n=[],r=e.length;t<=r;){var i=e.indexOf("\n",t);-1==i&&(i=e.length);var o=e.slice(t,"\r"==e.charAt(i-1)?i-1:i),a=o.indexOf("\r");-1!=a?(n.push(o.slice(0,a)),t+=a+1):(n.push(o),t=i+1)}return n}:function(e){return e.split(/\r\n?|\n/)},Le=window.getSelection?function(e){try{return e.selectionStart!=e.selectionEnd}catch(e){return!1}}:function(e){var t;try{t=e.ownerDocument.selection.createRange()}catch(e){}return!(!t||t.parentElement()!=e)&&0!=t.compareEndPoints("StartToEnd",t)},De="oncopy"in(Oe=E("div"))||(Oe.setAttribute("oncopy","return;"),"function"==typeof Oe.oncopy),Pe=null,Ne={},je={};function Re(e,t){arguments.length>2&&(t.dependencies=Array.prototype.slice.call(arguments,2)),Ne[e]=t}function ze(e){if("string"==typeof e&&je.hasOwnProperty(e))e=je[e];else if(e&&"string"==typeof e.name&&je.hasOwnProperty(e.name)){var t=je[e.name];"string"==typeof t&&(t={name:t}),(e=K(t,e)).name=t.name}else{if("string"==typeof e&&/^[\w\-]+\/[\w\-]+\+xml$/.test(e))return ze("application/xml");if("string"==typeof e&&/^[\w\-]+\/[\w\-]+\+json$/.test(e))return ze("application/json")}return"string"==typeof e?{name:e}:e||{name:"null"}}function Fe(e,t){t=ze(t);var n=Ne[t.name];if(!n)return Fe(e,"text/plain");var r=n(e,t);if(Be.hasOwnProperty(t.name)){var i=Be[t.name];for(var o in i)i.hasOwnProperty(o)&&(r.hasOwnProperty(o)&&(r["_"+o]=r[o]),r[o]=i[o])}if(r.name=t.name,t.helperType&&(r.helperType=t.helperType),t.modeProps)for(var a in t.modeProps)r[a]=t.modeProps[a];return r}var Be={};function He(e,t){R(t,Be.hasOwnProperty(e)?Be[e]:Be[e]={})}function Ue(e,t){if(!0===t)return t;if(e.copyState)return e.copyState(t);var n={};for(var r in t){var i=t[r];i instanceof Array&&(i=i.concat([])),n[r]=i}return n}function Ve(e,t){for(var n;e.innerMode&&(n=e.innerMode(t))&&n.mode!=e;)t=n.state,e=n.mode;return n||{mode:e,state:t}}function Ye(e,t,n){return!e.startState||e.startState(t,n)}var We=function(e,t,n){this.pos=this.start=0,this.string=e,this.tabSize=t||8,this.lastColumnPos=this.lastColumnValue=0,this.lineStart=0,this.lineOracle=n};function Ge(e,t){if((t-=e.first)<0||t>=e.size)throw new Error("There is no line "+(t+e.first)+" in the document.");for(var n=e;!n.lines;)for(var r=0;;++r){var i=n.children[r],o=i.chunkSize();if(t<o){n=i;break}t-=o}return n.lines[t]}function Xe(e,t,n){var r=[],i=t.line;return e.iter(t.line,n.line+1,(function(e){var o=e.text;i==n.line&&(o=o.slice(0,n.ch)),i==t.line&&(o=o.slice(t.ch)),r.push(o),++i})),r}function qe(e,t,n){var r=[];return e.iter(t,n,(function(e){r.push(e.text)})),r}function Ze(e,t){var n=t-e.height;if(n)for(var r=e;r;r=r.parent)r.height+=n}function Qe(e){if(null==e.parent)return null;for(var t=e.parent,n=B(t.lines,e),r=t.parent;r;t=r,r=r.parent)for(var i=0;r.children[i]!=t;++i)n+=r.children[i].chunkSize();return n+t.first}function Ke(e,t){var n=e.first;e:do{for(var r=0;r<e.children.length;++r){var i=e.children[r],o=i.height;if(t<o){e=i;continue e}t-=o,n+=i.chunkSize()}return n}while(!e.lines);for(var a=0;a<e.lines.length;++a){var s=e.lines[a].height;if(t<s)break;t-=s}return n+a}function Je(e,t){return t>=e.first&&t<e.first+e.size}function $e(e,t){return String(e.lineNumberFormatter(t+e.firstLineNumber))}function et(e,t,n){if(void 0===n&&(n=null),!(this instanceof et))return new et(e,t,n);this.line=e,this.ch=t,this.sticky=n}function tt(e,t){return e.line-t.line||e.ch-t.ch}function nt(e,t){return e.sticky==t.sticky&&0==tt(e,t)}function rt(e){return et(e.line,e.ch)}function it(e,t){return tt(e,t)<0?t:e}function ot(e,t){return tt(e,t)<0?e:t}function at(e,t){return Math.max(e.first,Math.min(t,e.first+e.size-1))}function st(e,t){if(t.line<e.first)return et(e.first,0);var n=e.first+e.size-1;return t.line>n?et(n,Ge(e,n).text.length):function(e,t){var n=e.ch;return null==n||n>t?et(e.line,t):n<0?et(e.line,0):e}(t,Ge(e,t.line).text.length)}function lt(e,t){for(var n=[],r=0;r<t.length;r++)n[r]=st(e,t[r]);return n}We.prototype.eol=function(){return this.pos>=this.string.length},We.prototype.sol=function(){return this.pos==this.lineStart},We.prototype.peek=function(){return this.string.charAt(this.pos)||void 0},We.prototype.next=function(){if(this.pos<this.string.length)return this.string.charAt(this.pos++)},We.prototype.eat=function(e){var t=this.string.charAt(this.pos);if("string"==typeof e?t==e:t&&(e.test?e.test(t):e(t)))return++this.pos,t},We.prototype.eatWhile=function(e){for(var t=this.pos;this.eat(e););return this.pos>t},We.prototype.eatSpace=function(){for(var e=this.pos;/[\s\u00a0]/.test(this.string.charAt(this.pos));)++this.pos;return this.pos>e},We.prototype.skipToEnd=function(){this.pos=this.string.length},We.prototype.skipTo=function(e){var t=this.string.indexOf(e,this.pos);if(t>-1)return this.pos=t,!0},We.prototype.backUp=function(e){this.pos-=e},We.prototype.column=function(){return this.lastColumnPos<this.start&&(this.lastColumnValue=z(this.string,this.start,this.tabSize,this.lastColumnPos,this.lastColumnValue),this.lastColumnPos=this.start),this.lastColumnValue-(this.lineStart?z(this.string,this.lineStart,this.tabSize):0)},We.prototype.indentation=function(){return z(this.string,null,this.tabSize)-(this.lineStart?z(this.string,this.lineStart,this.tabSize):0)},We.prototype.match=function(e,t,n){if("string"!=typeof e){var r=this.string.slice(this.pos).match(e);return r&&r.index>0?null:(r&&!1!==t&&(this.pos+=r[0].length),r)}var i=function(e){return n?e.toLowerCase():e};if(i(this.string.substr(this.pos,e.length))==i(e))return!1!==t&&(this.pos+=e.length),!0},We.prototype.current=function(){return this.string.slice(this.start,this.pos)},We.prototype.hideFirstChars=function(e,t){this.lineStart+=e;try{return t()}finally{this.lineStart-=e}},We.prototype.lookAhead=function(e){var t=this.lineOracle;return t&&t.lookAhead(e)},We.prototype.baseToken=function(){var e=this.lineOracle;return e&&e.baseToken(this.pos)};var ut=function(e,t){this.state=e,this.lookAhead=t},ct=function(e,t,n,r){this.state=t,this.doc=e,this.line=n,this.maxLookAhead=r||0,this.baseTokens=null,this.baseTokenPos=1};function dt(e,t,n,r){var i=[e.state.modeGen],o={};Ct(e,t.text,e.doc.mode,n,(function(e,t){return i.push(e,t)}),o,r);for(var a=n.state,s=function(r){n.baseTokens=i;var s=e.state.overlays[r],l=1,u=0;n.state=!0,Ct(e,t.text,s.mode,n,(function(e,t){for(var n=l;u<e;){var r=i[l];r>e&&i.splice(l,1,e,i[l+1],r),l+=2,u=Math.min(e,r)}if(t)if(s.opaque)i.splice(n,l-n,e,"overlay "+t),l=n+2;else for(;n<l;n+=2){var o=i[n+1];i[n+1]=(o?o+" ":"")+"overlay "+t}}),o),n.state=a,n.baseTokens=null,n.baseTokenPos=1},l=0;l<e.state.overlays.length;++l)s(l);return{styles:i,classes:o.bgClass||o.textClass?o:null}}function ht(e,t,n){if(!t.styles||t.styles[0]!=e.state.modeGen){var r=ft(e,Qe(t)),i=t.text.length>e.options.maxHighlightLength&&Ue(e.doc.mode,r.state),o=dt(e,t,r);i&&(r.state=i),t.stateAfter=r.save(!i),t.styles=o.styles,o.classes?t.styleClasses=o.classes:t.styleClasses&&(t.styleClasses=null),n===e.doc.highlightFrontier&&(e.doc.modeFrontier=Math.max(e.doc.modeFrontier,++e.doc.highlightFrontier))}return t.styles}function ft(e,t,n){var r=e.doc,i=e.display;if(!r.mode.startState)return new ct(r,!0,t);var o=function(e,t,n){for(var r,i,o=e.doc,a=n?-1:t-(e.doc.mode.innerMode?1e3:100),s=t;s>a;--s){if(s<=o.first)return o.first;var l=Ge(o,s-1),u=l.stateAfter;if(u&&(!n||s+(u instanceof ut?u.lookAhead:0)<=o.modeFrontier))return s;var c=z(l.text,null,e.options.tabSize);(null==i||r>c)&&(i=s-1,r=c)}return i}(e,t,n),a=o>r.first&&Ge(r,o-1).stateAfter,s=a?ct.fromSaved(r,a,o):new ct(r,Ye(r.mode),o);return r.iter(o,t,(function(n){pt(e,n.text,s);var r=s.line;n.stateAfter=r==t-1||r%5==0||r>=i.viewFrom&&r<i.viewTo?s.save():null,s.nextLine()})),n&&(r.modeFrontier=s.line),s}function pt(e,t,n,r){var i=e.doc.mode,o=new We(t,e.options.tabSize,n);for(o.start=o.pos=r||0,""==t&&gt(i,n.state);!o.eol();)vt(i,o,n.state),o.start=o.pos}function gt(e,t){if(e.blankLine)return e.blankLine(t);if(e.innerMode){var n=Ve(e,t);return n.mode.blankLine?n.mode.blankLine(n.state):void 0}}function vt(e,t,n,r){for(var i=0;i<10;i++){r&&(r[0]=Ve(e,n).mode);var o=e.token(t,n);if(t.pos>t.start)return o}throw new Error("Mode "+e.name+" failed to advance stream.")}ct.prototype.lookAhead=function(e){var t=this.doc.getLine(this.line+e);return null!=t&&e>this.maxLookAhead&&(this.maxLookAhead=e),t},ct.prototype.baseToken=function(e){if(!this.baseTokens)return null;for(;this.baseTokens[this.baseTokenPos]<=e;)this.baseTokenPos+=2;var t=this.baseTokens[this.baseTokenPos+1];return{type:t&&t.replace(/( |^)overlay .*/,""),size:this.baseTokens[this.baseTokenPos]-e}},ct.prototype.nextLine=function(){this.line++,this.maxLookAhead>0&&this.maxLookAhead--},ct.fromSaved=function(e,t,n){return t instanceof ut?new ct(e,Ue(e.mode,t.state),n,t.lookAhead):new ct(e,Ue(e.mode,t),n)},ct.prototype.save=function(e){var t=!1!==e?Ue(this.doc.mode,this.state):this.state;return this.maxLookAhead>0?new ut(t,this.maxLookAhead):t};var mt=function(e,t,n){this.start=e.start,this.end=e.pos,this.string=e.current(),this.type=t||null,this.state=n};function yt(e,t,n,r){var i,o,a=e.doc,s=a.mode,l=Ge(a,(t=st(a,t)).line),u=ft(e,t.line,n),c=new We(l.text,e.options.tabSize,u);for(r&&(o=[]);(r||c.pos<t.ch)&&!c.eol();)c.start=c.pos,i=vt(s,c,u.state),r&&o.push(new mt(c,i,Ue(a.mode,u.state)));return r?o:new mt(c,i,u.state)}function bt(e,t){if(e)for(;;){var n=e.match(/(?:^|\s+)line-(background-)?(\S+)/);if(!n)break;e=e.slice(0,n.index)+e.slice(n.index+n[0].length);var r=n[1]?"bgClass":"textClass";null==t[r]?t[r]=n[2]:new RegExp("(?:^|\\s)"+n[2]+"(?:$|\\s)").test(t[r])||(t[r]+=" "+n[2])}return e}function Ct(e,t,n,r,i,o,a){var s=n.flattenSpans;null==s&&(s=e.options.flattenSpans);var l,u=0,c=null,d=new We(t,e.options.tabSize,r),h=e.options.addModeClass&&[null];for(""==t&&bt(gt(n,r.state),o);!d.eol();){if(d.pos>e.options.maxHighlightLength?(s=!1,a&&pt(e,t,r,d.pos),d.pos=t.length,l=null):l=bt(vt(n,d,r.state,h),o),h){var f=h[0].name;f&&(l="m-"+(l?f+" "+l:f))}if(!s||c!=l){for(;u<d.start;)i(u=Math.min(d.start,u+5e3),c);c=l}d.start=d.pos}for(;u<d.pos;){var p=Math.min(d.pos,u+5e3);i(p,c),u=p}}var At=!1,_t=!1;function It(e,t,n){this.marker=e,this.from=t,this.to=n}function xt(e,t){if(e)for(var n=0;n<e.length;++n){var r=e[n];if(r.marker==t)return r}}function wt(e,t){for(var n,r=0;r<e.length;++r)e[r]!=t&&(n||(n=[])).push(e[r]);return n}function Mt(e,t){if(t.full)return null;var n=Je(e,t.from.line)&&Ge(e,t.from.line).markedSpans,r=Je(e,t.to.line)&&Ge(e,t.to.line).markedSpans;if(!n&&!r)return null;var i=t.from.ch,o=t.to.ch,a=0==tt(t.from,t.to),s=function(e,t,n){var r;if(e)for(var i=0;i<e.length;++i){var o=e[i],a=o.marker;if(null==o.from||(a.inclusiveLeft?o.from<=t:o.from<t)||o.from==t&&"bookmark"==a.type&&(!n||!o.marker.insertLeft)){var s=null==o.to||(a.inclusiveRight?o.to>=t:o.to>t);(r||(r=[])).push(new It(a,o.from,s?null:o.to))}}return r}(n,i,a),l=function(e,t,n){var r;if(e)for(var i=0;i<e.length;++i){var o=e[i],a=o.marker;if(null==o.to||(a.inclusiveRight?o.to>=t:o.to>t)||o.from==t&&"bookmark"==a.type&&(!n||o.marker.insertLeft)){var s=null==o.from||(a.inclusiveLeft?o.from<=t:o.from<t);(r||(r=[])).push(new It(a,s?null:o.from-t,null==o.to?null:o.to-t))}}return r}(r,o,a),u=1==t.text.length,c=q(t.text).length+(u?i:0);if(s)for(var d=0;d<s.length;++d){var h=s[d];if(null==h.to){var f=xt(l,h.marker);f?u&&(h.to=null==f.to?null:f.to+c):h.to=i}}if(l)for(var p=0;p<l.length;++p){var g=l[p];null!=g.to&&(g.to+=c),null==g.from?xt(s,g.marker)||(g.from=c,u&&(s||(s=[])).push(g)):(g.from+=c,u&&(s||(s=[])).push(g))}s&&(s=St(s)),l&&l!=s&&(l=St(l));var v=[s];if(!u){var m,y=t.text.length-2;if(y>0&&s)for(var b=0;b<s.length;++b)null==s[b].to&&(m||(m=[])).push(new It(s[b].marker,null,null));for(var C=0;C<y;++C)v.push(m);v.push(l)}return v}function St(e){for(var t=0;t<e.length;++t){var n=e[t];null!=n.from&&n.from==n.to&&!1!==n.marker.clearWhenEmpty&&e.splice(t--,1)}return e.length?e:null}function Tt(e){var t=e.markedSpans;if(t){for(var n=0;n<t.length;++n)t[n].marker.detachLine(e);e.markedSpans=null}}function Et(e,t){if(t){for(var n=0;n<t.length;++n)t[n].marker.attachLine(e);e.markedSpans=t}}function Ot(e){return e.inclusiveLeft?-1:0}function kt(e){return e.inclusiveRight?1:0}function Lt(e,t){var n=e.lines.length-t.lines.length;if(0!=n)return n;var r=e.find(),i=t.find(),o=tt(r.from,i.from)||Ot(e)-Ot(t);if(o)return-o;var a=tt(r.to,i.to)||kt(e)-kt(t);return a||t.id-e.id}function Dt(e,t){var n,r=_t&&e.markedSpans;if(r)for(var i=void 0,o=0;o<r.length;++o)(i=r[o]).marker.collapsed&&null==(t?i.from:i.to)&&(!n||Lt(n,i.marker)<0)&&(n=i.marker);return n}function Pt(e){return Dt(e,!0)}function Nt(e){return Dt(e,!1)}function jt(e,t){var n,r=_t&&e.markedSpans;if(r)for(var i=0;i<r.length;++i){var o=r[i];o.marker.collapsed&&(null==o.from||o.from<t)&&(null==o.to||o.to>t)&&(!n||Lt(n,o.marker)<0)&&(n=o.marker)}return n}function Rt(e,t,n,r,i){var o=Ge(e,t),a=_t&&o.markedSpans;if(a)for(var s=0;s<a.length;++s){var l=a[s];if(l.marker.collapsed){var u=l.marker.find(0),c=tt(u.from,n)||Ot(l.marker)-Ot(i),d=tt(u.to,r)||kt(l.marker)-kt(i);if(!(c>=0&&d<=0||c<=0&&d>=0)&&(c<=0&&(l.marker.inclusiveRight&&i.inclusiveLeft?tt(u.to,n)>=0:tt(u.to,n)>0)||c>=0&&(l.marker.inclusiveRight&&i.inclusiveLeft?tt(u.from,r)<=0:tt(u.from,r)<0)))return!0}}}function zt(e){for(var t;t=Pt(e);)e=t.find(-1,!0).line;return e}function Ft(e,t){var n=Ge(e,t),r=zt(n);return n==r?t:Qe(r)}function Bt(e,t){if(t>e.lastLine())return t;var n,r=Ge(e,t);if(!Ht(e,r))return t;for(;n=Nt(r);)r=n.find(1,!0).line;return Qe(r)+1}function Ht(e,t){var n=_t&&t.markedSpans;if(n)for(var r=void 0,i=0;i<n.length;++i)if((r=n[i]).marker.collapsed){if(null==r.from)return!0;if(!r.marker.widgetNode&&0==r.from&&r.marker.inclusiveLeft&&Ut(e,t,r))return!0}}function Ut(e,t,n){if(null==n.to){var r=n.marker.find(1,!0);return Ut(e,r.line,xt(r.line.markedSpans,n.marker))}if(n.marker.inclusiveRight&&n.to==t.text.length)return!0;for(var i=void 0,o=0;o<t.markedSpans.length;++o)if((i=t.markedSpans[o]).marker.collapsed&&!i.marker.widgetNode&&i.from==n.to&&(null==i.to||i.to!=n.from)&&(i.marker.inclusiveLeft||n.marker.inclusiveRight)&&Ut(e,t,i))return!0}function Vt(e){for(var t=0,n=(e=zt(e)).parent,r=0;r<n.lines.length;++r){var i=n.lines[r];if(i==e)break;t+=i.height}for(var o=n.parent;o;o=(n=o).parent)for(var a=0;a<o.children.length;++a){var s=o.children[a];if(s==n)break;t+=s.height}return t}function Yt(e){if(0==e.height)return 0;for(var t,n=e.text.length,r=e;t=Pt(r);){var i=t.find(0,!0);r=i.from.line,n+=i.from.ch-i.to.ch}for(r=e;t=Nt(r);){var o=t.find(0,!0);n-=r.text.length-o.from.ch,n+=(r=o.to.line).text.length-o.to.ch}return n}function Wt(e){var t=e.display,n=e.doc;t.maxLine=Ge(n,n.first),t.maxLineLength=Yt(t.maxLine),t.maxLineChanged=!0,n.iter((function(e){var n=Yt(e);n>t.maxLineLength&&(t.maxLineLength=n,t.maxLine=e)}))}var Gt=function(e,t,n){this.text=e,Et(this,t),this.height=n?n(this):1};function Xt(e){e.parent=null,Tt(e)}Gt.prototype.lineNo=function(){return Qe(this)},ye(Gt);var qt={},Zt={};function Qt(e,t){if(!e||/^\s*$/.test(e))return null;var n=t.addModeClass?Zt:qt;return n[e]||(n[e]=e.replace(/\S+/g,"cm-$&"))}function Kt(e,t){var n=O("span",null,null,l?"padding-right: .1px":null),r={pre:O("pre",[n],"CodeMirror-line"),content:n,col:0,pos:0,cm:e,trailingSpace:!1,splitSpaces:e.getOption("lineWrapping")};t.measure={};for(var i=0;i<=(t.rest?t.rest.length:0);i++){var o=i?t.rest[i-1]:t.line,a=void 0;r.pos=0,r.addToken=$t,Ee(e.display.measure)&&(a=ue(o,e.doc.direction))&&(r.addToken=en(r.addToken,a)),r.map=[],nn(o,r,ht(e,o,t!=e.display.externalMeasured&&Qe(o))),o.styleClasses&&(o.styleClasses.bgClass&&(r.bgClass=P(o.styleClasses.bgClass,r.bgClass||"")),o.styleClasses.textClass&&(r.textClass=P(o.styleClasses.textClass,r.textClass||""))),0==r.map.length&&r.map.push(0,0,r.content.appendChild(Te(e.display.measure))),0==i?(t.measure.map=r.map,t.measure.cache={}):((t.measure.maps||(t.measure.maps=[])).push(r.map),(t.measure.caches||(t.measure.caches=[])).push({}))}if(l){var s=r.content.lastChild;(/\bcm-tab\b/.test(s.className)||s.querySelector&&s.querySelector(".cm-tab"))&&(r.content.className="cm-tab-wrap-hack")}return pe(e,"renderLine",e,t.line,r.pre),r.pre.className&&(r.textClass=P(r.pre.className,r.textClass||"")),r}function Jt(e){var t=E("span","•","cm-invalidchar");return t.title="\\u"+e.charCodeAt(0).toString(16),t.setAttribute("aria-label",t.title),t}function $t(e,t,n,r,i,o,l){if(t){var u,c=e.splitSpaces?function(e,t){if(e.length>1&&!/  /.test(e))return e;for(var n=t,r="",i=0;i<e.length;i++){var o=e.charAt(i);" "!=o||!n||i!=e.length-1&&32!=e.charCodeAt(i+1)||(o=" "),r+=o,n=" "==o}return r}(t,e.trailingSpace):t,d=e.cm.state.specialChars,h=!1;if(d.test(t)){u=document.createDocumentFragment();for(var f=0;;){d.lastIndex=f;var p=d.exec(t),g=p?p.index-f:t.length-f;if(g){var v=document.createTextNode(c.slice(f,f+g));a&&s<9?u.appendChild(E("span",[v])):u.appendChild(v),e.map.push(e.pos,e.pos+g,v),e.col+=g,e.pos+=g}if(!p)break;f+=g+1;var m=void 0;if("\t"==p[0]){var y=e.cm.options.tabSize,b=y-e.col%y;(m=u.appendChild(E("span",X(b),"cm-tab"))).setAttribute("role","presentation"),m.setAttribute("cm-text","\t"),e.col+=b}else"\r"==p[0]||"\n"==p[0]?((m=u.appendChild(E("span","\r"==p[0]?"␍":"␤","cm-invalidchar"))).setAttribute("cm-text",p[0]),e.col+=1):((m=e.cm.options.specialCharPlaceholder(p[0])).setAttribute("cm-text",p[0]),a&&s<9?u.appendChild(E("span",[m])):u.appendChild(m),e.col+=1);e.map.push(e.pos,e.pos+1,m),e.pos++}}else e.col+=t.length,u=document.createTextNode(c),e.map.push(e.pos,e.pos+t.length,u),a&&s<9&&(h=!0),e.pos+=t.length;if(e.trailingSpace=32==c.charCodeAt(t.length-1),n||r||i||h||o){var C=n||"";r&&(C+=r),i&&(C+=i);var A=E("span",[u],C,o);if(l)for(var _ in l)l.hasOwnProperty(_)&&"style"!=_&&"class"!=_&&A.setAttribute(_,l[_]);return e.content.appendChild(A)}e.content.appendChild(u)}}function en(e,t){return function(n,r,i,o,a,s,l){i=i?i+" cm-force-border":"cm-force-border";for(var u=n.pos,c=u+r.length;;){for(var d=void 0,h=0;h<t.length&&!((d=t[h]).to>u&&d.from<=u);h++);if(d.to>=c)return e(n,r,i,o,a,s,l);e(n,r.slice(0,d.to-u),i,o,null,s,l),o=null,r=r.slice(d.to-u),u=d.to}}}function tn(e,t,n,r){var i=!r&&n.widgetNode;i&&e.map.push(e.pos,e.pos+t,i),!r&&e.cm.display.input.needsContentAttribute&&(i||(i=e.content.appendChild(document.createElement("span"))),i.setAttribute("cm-marker",n.id)),i&&(e.cm.display.input.setUneditable(i),e.content.appendChild(i)),e.pos+=t,e.trailingSpace=!1}function nn(e,t,n){var r=e.markedSpans,i=e.text,o=0;if(r)for(var a,s,l,u,c,d,h,f=i.length,p=0,g=1,v="",m=0;;){if(m==p){l=u=c=s="",h=null,d=null,m=1/0;for(var y=[],b=void 0,C=0;C<r.length;++C){var A=r[C],_=A.marker;if("bookmark"==_.type&&A.from==p&&_.widgetNode)y.push(_);else if(A.from<=p&&(null==A.to||A.to>p||_.collapsed&&A.to==p&&A.from==p)){if(null!=A.to&&A.to!=p&&m>A.to&&(m=A.to,u=""),_.className&&(l+=" "+_.className),_.css&&(s=(s?s+";":"")+_.css),_.startStyle&&A.from==p&&(c+=" "+_.startStyle),_.endStyle&&A.to==m&&(b||(b=[])).push(_.endStyle,A.to),_.title&&((h||(h={})).title=_.title),_.attributes)for(var I in _.attributes)(h||(h={}))[I]=_.attributes[I];_.collapsed&&(!d||Lt(d.marker,_)<0)&&(d=A)}else A.from>p&&m>A.from&&(m=A.from)}if(b)for(var x=0;x<b.length;x+=2)b[x+1]==m&&(u+=" "+b[x]);if(!d||d.from==p)for(var w=0;w<y.length;++w)tn(t,0,y[w]);if(d&&(d.from||0)==p){if(tn(t,(null==d.to?f+1:d.to)-p,d.marker,null==d.from),null==d.to)return;d.to==p&&(d=!1)}}if(p>=f)break;for(var M=Math.min(f,m);;){if(v){var S=p+v.length;if(!d){var T=S>M?v.slice(0,M-p):v;t.addToken(t,T,a?a+l:l,c,p+T.length==m?u:"",s,h)}if(S>=M){v=v.slice(M-p),p=M;break}p=S,c=""}v=i.slice(o,o=n[g++]),a=Qt(n[g++],t.cm.options)}}else for(var E=1;E<n.length;E+=2)t.addToken(t,i.slice(o,o=n[E]),Qt(n[E+1],t.cm.options))}function rn(e,t,n){this.line=t,this.rest=function(e){for(var t,n;t=Nt(e);)e=t.find(1,!0).line,(n||(n=[])).push(e);return n}(t),this.size=this.rest?Qe(q(this.rest))-n+1:1,this.node=this.text=null,this.hidden=Ht(e,t)}function on(e,t,n){for(var r,i=[],o=t;o<n;o=r){var a=new rn(e.doc,Ge(e.doc,o),o);r=o+a.size,i.push(a)}return i}var an=null,sn=null;function ln(e,t){var n=he(e,t);if(n.length){var r,i=Array.prototype.slice.call(arguments,2);an?r=an.delayedCallbacks:sn?r=sn:(r=sn=[],setTimeout(un,0));for(var o=function(e){r.push((function(){return n[e].apply(null,i)}))},a=0;a<n.length;++a)o(a)}}function un(){var e=sn;sn=null;for(var t=0;t<e.length;++t)e[t]()}function cn(e,t,n,r){for(var i=0;i<t.changes.length;i++){var o=t.changes[i];"text"==o?fn(e,t):"gutter"==o?gn(e,t,n,r):"class"==o?pn(e,t):"widget"==o&&vn(e,t,r)}t.changes=null}function dn(e){return e.node==e.text&&(e.node=E("div",null,null,"position: relative"),e.text.parentNode&&e.text.parentNode.replaceChild(e.node,e.text),e.node.appendChild(e.text),a&&s<8&&(e.node.style.zIndex=2)),e.node}function hn(e,t){var n=e.display.externalMeasured;return n&&n.line==t.line?(e.display.externalMeasured=null,t.measure=n.measure,n.built):Kt(e,t)}function fn(e,t){var n=t.text.className,r=hn(e,t);t.text==t.node&&(t.node=r.pre),t.text.parentNode.replaceChild(r.pre,t.text),t.text=r.pre,r.bgClass!=t.bgClass||r.textClass!=t.textClass?(t.bgClass=r.bgClass,t.textClass=r.textClass,pn(e,t)):n&&(t.text.className=n)}function pn(e,t){!function(e,t){var n=t.bgClass?t.bgClass+" "+(t.line.bgClass||""):t.line.bgClass;if(n&&(n+=" CodeMirror-linebackground"),t.background)n?t.background.className=n:(t.background.parentNode.removeChild(t.background),t.background=null);else if(n){var r=dn(t);t.background=r.insertBefore(E("div",null,n),r.firstChild),e.display.input.setUneditable(t.background)}}(e,t),t.line.wrapClass?dn(t).className=t.line.wrapClass:t.node!=t.text&&(t.node.className="");var n=t.textClass?t.textClass+" "+(t.line.textClass||""):t.line.textClass;t.text.className=n||""}function gn(e,t,n,r){if(t.gutter&&(t.node.removeChild(t.gutter),t.gutter=null),t.gutterBackground&&(t.node.removeChild(t.gutterBackground),t.gutterBackground=null),t.line.gutterClass){var i=dn(t);t.gutterBackground=E("div",null,"CodeMirror-gutter-background "+t.line.gutterClass,"left: "+(e.options.fixedGutter?r.fixedPos:-r.gutterTotalWidth)+"px; width: "+r.gutterTotalWidth+"px"),e.display.input.setUneditable(t.gutterBackground),i.insertBefore(t.gutterBackground,t.text)}var o=t.line.gutterMarkers;if(e.options.lineNumbers||o){var a=dn(t),s=t.gutter=E("div",null,"CodeMirror-gutter-wrapper","left: "+(e.options.fixedGutter?r.fixedPos:-r.gutterTotalWidth)+"px");if(e.display.input.setUneditable(s),a.insertBefore(s,t.text),t.line.gutterClass&&(s.className+=" "+t.line.gutterClass),!e.options.lineNumbers||o&&o["CodeMirror-linenumbers"]||(t.lineNumber=s.appendChild(E("div",$e(e.options,n),"CodeMirror-linenumber CodeMirror-gutter-elt","left: "+r.gutterLeft["CodeMirror-linenumbers"]+"px; width: "+e.display.lineNumInnerWidth+"px"))),o)for(var l=0;l<e.display.gutterSpecs.length;++l){var u=e.display.gutterSpecs[l].className,c=o.hasOwnProperty(u)&&o[u];c&&s.appendChild(E("div",[c],"CodeMirror-gutter-elt","left: "+r.gutterLeft[u]+"px; width: "+r.gutterWidth[u]+"px"))}}}function vn(e,t,n){t.alignable&&(t.alignable=null);for(var r=x("CodeMirror-linewidget"),i=t.node.firstChild,o=void 0;i;i=o)o=i.nextSibling,r.test(i.className)&&t.node.removeChild(i);yn(e,t,n)}function mn(e,t,n,r){var i=hn(e,t);return t.text=t.node=i.pre,i.bgClass&&(t.bgClass=i.bgClass),i.textClass&&(t.textClass=i.textClass),pn(e,t),gn(e,t,n,r),yn(e,t,r),t.node}function yn(e,t,n){if(bn(e,t.line,t,n,!0),t.rest)for(var r=0;r<t.rest.length;r++)bn(e,t.rest[r],t,n,!1)}function bn(e,t,n,r,i){if(t.widgets)for(var o=dn(n),a=0,s=t.widgets;a<s.length;++a){var l=s[a],u=E("div",[l.node],"CodeMirror-linewidget"+(l.className?" "+l.className:""));l.handleMouseEvents||u.setAttribute("cm-ignore-events","true"),Cn(l,u,n,r),e.display.input.setUneditable(u),i&&l.above?o.insertBefore(u,n.gutter||n.text):o.appendChild(u),ln(l,"redraw")}}function Cn(e,t,n,r){if(e.noHScroll){(n.alignable||(n.alignable=[])).push(t);var i=r.wrapperWidth;t.style.left=r.fixedPos+"px",e.coverGutter||(i-=r.gutterTotalWidth,t.style.paddingLeft=r.gutterTotalWidth+"px"),t.style.width=i+"px"}e.coverGutter&&(t.style.zIndex=5,t.style.position="relative",e.noHScroll||(t.style.marginLeft=-r.gutterTotalWidth+"px"))}function An(e){if(null!=e.height)return e.height;var t=e.doc.cm;if(!t)return 0;if(!k(document.body,e.node)){var n="position: relative;";e.coverGutter&&(n+="margin-left: -"+t.display.gutters.offsetWidth+"px;"),e.noHScroll&&(n+="width: "+t.display.wrapper.clientWidth+"px;"),T(t.display.measure,E("div",[e.node],null,n))}return e.height=e.node.parentNode.offsetHeight}function _n(e,t){for(var n=Ie(t);n!=e.wrapper;n=n.parentNode)if(!n||1==n.nodeType&&"true"==n.getAttribute("cm-ignore-events")||n.parentNode==e.sizer&&n!=e.mover)return!0}function In(e){return e.lineSpace.offsetTop}function xn(e){return e.mover.offsetHeight-e.lineSpace.offsetHeight}function wn(e){if(e.cachedPaddingH)return e.cachedPaddingH;var t=T(e.measure,E("pre","x","CodeMirror-line-like")),n=window.getComputedStyle?window.getComputedStyle(t):t.currentStyle,r={left:parseInt(n.paddingLeft),right:parseInt(n.paddingRight)};return isNaN(r.left)||isNaN(r.right)||(e.cachedPaddingH=r),r}function Mn(e){return 50-e.display.nativeBarWidth}function Sn(e){return e.display.scroller.clientWidth-Mn(e)-e.display.barWidth}function Tn(e){return e.display.scroller.clientHeight-Mn(e)-e.display.barHeight}function En(e,t,n){if(e.line==t)return{map:e.measure.map,cache:e.measure.cache};for(var r=0;r<e.rest.length;r++)if(e.rest[r]==t)return{map:e.measure.maps[r],cache:e.measure.caches[r]};for(var i=0;i<e.rest.length;i++)if(Qe(e.rest[i])>n)return{map:e.measure.maps[i],cache:e.measure.caches[i],before:!0}}function On(e,t,n,r){return Dn(e,Ln(e,t),n,r)}function kn(e,t){if(t>=e.display.viewFrom&&t<e.display.viewTo)return e.display.view[cr(e,t)];var n=e.display.externalMeasured;return n&&t>=n.lineN&&t<n.lineN+n.size?n:void 0}function Ln(e,t){var n=Qe(t),r=kn(e,n);r&&!r.text?r=null:r&&r.changes&&(cn(e,r,n,or(e)),e.curOp.forceUpdate=!0),r||(r=function(e,t){var n=Qe(t=zt(t)),r=e.display.externalMeasured=new rn(e.doc,t,n);r.lineN=n;var i=r.built=Kt(e,r);return r.text=i.pre,T(e.display.lineMeasure,i.pre),r}(e,t));var i=En(r,t,n);return{line:t,view:r,rect:null,map:i.map,cache:i.cache,before:i.before,hasHeights:!1}}function Dn(e,t,n,r,i){t.before&&(n=-1);var o,l=n+(r||"");return t.cache.hasOwnProperty(l)?o=t.cache[l]:(t.rect||(t.rect=t.view.text.getBoundingClientRect()),t.hasHeights||(function(e,t,n){var r=e.options.lineWrapping,i=r&&Sn(e);if(!t.measure.heights||r&&t.measure.width!=i){var o=t.measure.heights=[];if(r){t.measure.width=i;for(var a=t.text.firstChild.getClientRects(),s=0;s<a.length-1;s++){var l=a[s],u=a[s+1];Math.abs(l.bottom-u.bottom)>2&&o.push((l.bottom+u.top)/2-n.top)}}o.push(n.bottom-n.top)}}(e,t.view,t.rect),t.hasHeights=!0),(o=function(e,t,n,r){var i,o=jn(t.map,n,r),l=o.node,u=o.start,c=o.end,d=o.collapse;if(3==l.nodeType){for(var h=0;h<4;h++){for(;u&&re(t.line.text.charAt(o.coverStart+u));)--u;for(;o.coverStart+c<o.coverEnd&&re(t.line.text.charAt(o.coverStart+c));)++c;if((i=a&&s<9&&0==u&&c==o.coverEnd-o.coverStart?l.parentNode.getBoundingClientRect():Rn(w(l,u,c).getClientRects(),r)).left||i.right||0==u)break;c=u,u-=1,d="right"}a&&s<11&&(i=function(e,t){if(!window.screen||null==screen.logicalXDPI||screen.logicalXDPI==screen.deviceXDPI||!function(e){if(null!=Pe)return Pe;var t=T(e,E("span","x")),n=t.getBoundingClientRect(),r=w(t,0,1).getBoundingClientRect();return Pe=Math.abs(n.left-r.left)>1}(e))return t;var n=screen.logicalXDPI/screen.deviceXDPI,r=screen.logicalYDPI/screen.deviceYDPI;return{left:t.left*n,right:t.right*n,top:t.top*r,bottom:t.bottom*r}}(e.display.measure,i))}else{var f;u>0&&(d=r="right"),i=e.options.lineWrapping&&(f=l.getClientRects()).length>1?f["right"==r?f.length-1:0]:l.getBoundingClientRect()}if(a&&s<9&&!u&&(!i||!i.left&&!i.right)){var p=l.parentNode.getClientRects()[0];i=p?{left:p.left,right:p.left+ir(e.display),top:p.top,bottom:p.bottom}:Nn}for(var g=i.top-t.rect.top,v=i.bottom-t.rect.top,m=(g+v)/2,y=t.view.measure.heights,b=0;b<y.length-1&&!(m<y[b]);b++);var C=b?y[b-1]:0,A=y[b],_={left:("right"==d?i.right:i.left)-t.rect.left,right:("left"==d?i.left:i.right)-t.rect.left,top:C,bottom:A};return i.left||i.right||(_.bogus=!0),e.options.singleCursorHeightPerLine||(_.rtop=g,_.rbottom=v),_}(e,t,n,r)).bogus||(t.cache[l]=o)),{left:o.left,right:o.right,top:i?o.rtop:o.top,bottom:i?o.rbottom:o.bottom}}var Pn,Nn={left:0,right:0,top:0,bottom:0};function jn(e,t,n){for(var r,i,o,a,s,l,u=0;u<e.length;u+=3)if(s=e[u],l=e[u+1],t<s?(i=0,o=1,a="left"):t<l?o=1+(i=t-s):(u==e.length-3||t==l&&e[u+3]>t)&&(i=(o=l-s)-1,t>=l&&(a="right")),null!=i){if(r=e[u+2],s==l&&n==(r.insertLeft?"left":"right")&&(a=n),"left"==n&&0==i)for(;u&&e[u-2]==e[u-3]&&e[u-1].insertLeft;)r=e[2+(u-=3)],a="left";if("right"==n&&i==l-s)for(;u<e.length-3&&e[u+3]==e[u+4]&&!e[u+5].insertLeft;)r=e[(u+=3)+2],a="right";break}return{node:r,start:i,end:o,collapse:a,coverStart:s,coverEnd:l}}function Rn(e,t){var n=Nn;if("left"==t)for(var r=0;r<e.length&&(n=e[r]).left==n.right;r++);else for(var i=e.length-1;i>=0&&(n=e[i]).left==n.right;i--);return n}function zn(e){if(e.measure&&(e.measure.cache={},e.measure.heights=null,e.rest))for(var t=0;t<e.rest.length;t++)e.measure.caches[t]={}}function Fn(e){e.display.externalMeasure=null,S(e.display.lineMeasure);for(var t=0;t<e.display.view.length;t++)zn(e.display.view[t])}function Bn(e){Fn(e),e.display.cachedCharWidth=e.display.cachedTextHeight=e.display.cachedPaddingH=null,e.options.lineWrapping||(e.display.maxLineChanged=!0),e.display.lineNumChars=null}function Hn(){return c&&v?-(document.body.getBoundingClientRect().left-parseInt(getComputedStyle(document.body).marginLeft)):window.pageXOffset||(document.documentElement||document.body).scrollLeft}function Un(){return c&&v?-(document.body.getBoundingClientRect().top-parseInt(getComputedStyle(document.body).marginTop)):window.pageYOffset||(document.documentElement||document.body).scrollTop}function Vn(e){var t=0;if(e.widgets)for(var n=0;n<e.widgets.length;++n)e.widgets[n].above&&(t+=An(e.widgets[n]));return t}function Yn(e,t,n,r,i){if(!i){var o=Vn(t);n.top+=o,n.bottom+=o}if("line"==r)return n;r||(r="local");var a=Vt(t);if("local"==r?a+=In(e.display):a-=e.display.viewOffset,"page"==r||"window"==r){var s=e.display.lineSpace.getBoundingClientRect();a+=s.top+("window"==r?0:Un());var l=s.left+("window"==r?0:Hn());n.left+=l,n.right+=l}return n.top+=a,n.bottom+=a,n}function Wn(e,t,n){if("div"==n)return t;var r=t.left,i=t.top;if("page"==n)r-=Hn(),i-=Un();else if("local"==n||!n){var o=e.display.sizer.getBoundingClientRect();r+=o.left,i+=o.top}var a=e.display.lineSpace.getBoundingClientRect();return{left:r-a.left,top:i-a.top}}function Gn(e,t,n,r,i){return r||(r=Ge(e.doc,t.line)),Yn(e,r,On(e,r,t.ch,i),n)}function Xn(e,t,n,r,i,o){function a(t,a){var s=Dn(e,i,t,a?"right":"left",o);return a?s.left=s.right:s.right=s.left,Yn(e,r,s,n)}r=r||Ge(e.doc,t.line),i||(i=Ln(e,r));var s=ue(r,e.doc.direction),l=t.ch,u=t.sticky;if(l>=r.text.length?(l=r.text.length,u="before"):l<=0&&(l=0,u="after"),!s)return a("before"==u?l-1:l,"before"==u);function c(e,t,n){return a(n?e-1:e,1==s[t].level!=n)}var d=se(s,l,u),h=ae,f=c(l,d,"before"==u);return null!=h&&(f.other=c(l,h,"before"!=u)),f}function qn(e,t){var n=0;t=st(e.doc,t),e.options.lineWrapping||(n=ir(e.display)*t.ch);var r=Ge(e.doc,t.line),i=Vt(r)+In(e.display);return{left:n,right:n,top:i,bottom:i+r.height}}function Zn(e,t,n,r,i){var o=et(e,t,n);return o.xRel=i,r&&(o.outside=r),o}function Qn(e,t,n){var r=e.doc;if((n+=e.display.viewOffset)<0)return Zn(r.first,0,null,-1,-1);var i=Ke(r,n),o=r.first+r.size-1;if(i>o)return Zn(r.first+r.size-1,Ge(r,o).text.length,null,1,1);t<0&&(t=0);for(var a=Ge(r,i);;){var s=er(e,a,i,t,n),l=jt(a,s.ch+(s.xRel>0||s.outside>0?1:0));if(!l)return s;var u=l.find(1);if(u.line==i)return u;a=Ge(r,i=u.line)}}function Kn(e,t,n,r){r-=Vn(t);var i=t.text.length,o=oe((function(t){return Dn(e,n,t-1).bottom<=r}),i,0);return{begin:o,end:i=oe((function(t){return Dn(e,n,t).top>r}),o,i)}}function Jn(e,t,n,r){return n||(n=Ln(e,t)),Kn(e,t,n,Yn(e,t,Dn(e,n,r),"line").top)}function $n(e,t,n,r){return!(e.bottom<=n)&&(e.top>n||(r?e.left:e.right)>t)}function er(e,t,n,r,i){i-=Vt(t);var o=Ln(e,t),a=Vn(t),s=0,l=t.text.length,u=!0,c=ue(t,e.doc.direction);if(c){var d=(e.options.lineWrapping?nr:tr)(e,t,n,o,c,r,i);s=(u=1!=d.level)?d.from:d.to-1,l=u?d.to:d.from-1}var h,f,p=null,g=null,v=oe((function(t){var n=Dn(e,o,t);return n.top+=a,n.bottom+=a,!!$n(n,r,i,!1)&&(n.top<=i&&n.left<=r&&(p=t,g=n),!0)}),s,l),m=!1;if(g){var y=r-g.left<g.right-r,b=y==u;v=p+(b?0:1),f=b?"after":"before",h=y?g.left:g.right}else{u||v!=l&&v!=s||v++,f=0==v?"after":v==t.text.length?"before":Dn(e,o,v-(u?1:0)).bottom+a<=i==u?"after":"before";var C=Xn(e,et(n,v,f),"line",t,o);h=C.left,m=i<C.top?-1:i>=C.bottom?1:0}return Zn(n,v=ie(t.text,v,1),f,m,r-h)}function tr(e,t,n,r,i,o,a){var s=oe((function(s){var l=i[s],u=1!=l.level;return $n(Xn(e,et(n,u?l.to:l.from,u?"before":"after"),"line",t,r),o,a,!0)}),0,i.length-1),l=i[s];if(s>0){var u=1!=l.level,c=Xn(e,et(n,u?l.from:l.to,u?"after":"before"),"line",t,r);$n(c,o,a,!0)&&c.top>a&&(l=i[s-1])}return l}function nr(e,t,n,r,i,o,a){var s=Kn(e,t,r,a),l=s.begin,u=s.end;/\s/.test(t.text.charAt(u-1))&&u--;for(var c=null,d=null,h=0;h<i.length;h++){var f=i[h];if(!(f.from>=u||f.to<=l)){var p=Dn(e,r,1!=f.level?Math.min(u,f.to)-1:Math.max(l,f.from)).right,g=p<o?o-p+1e9:p-o;(!c||d>g)&&(c=f,d=g)}}return c||(c=i[i.length-1]),c.from<l&&(c={from:l,to:c.to,level:c.level}),c.to>u&&(c={from:c.from,to:u,level:c.level}),c}function rr(e){if(null!=e.cachedTextHeight)return e.cachedTextHeight;if(null==Pn){Pn=E("pre",null,"CodeMirror-line-like");for(var t=0;t<49;++t)Pn.appendChild(document.createTextNode("x")),Pn.appendChild(E("br"));Pn.appendChild(document.createTextNode("x"))}T(e.measure,Pn);var n=Pn.offsetHeight/50;return n>3&&(e.cachedTextHeight=n),S(e.measure),n||1}function ir(e){if(null!=e.cachedCharWidth)return e.cachedCharWidth;var t=E("span","xxxxxxxxxx"),n=E("pre",[t],"CodeMirror-line-like");T(e.measure,n);var r=t.getBoundingClientRect(),i=(r.right-r.left)/10;return i>2&&(e.cachedCharWidth=i),i||10}function or(e){for(var t=e.display,n={},r={},i=t.gutters.clientLeft,o=t.gutters.firstChild,a=0;o;o=o.nextSibling,++a){var s=e.display.gutterSpecs[a].className;n[s]=o.offsetLeft+o.clientLeft+i,r[s]=o.clientWidth}return{fixedPos:ar(t),gutterTotalWidth:t.gutters.offsetWidth,gutterLeft:n,gutterWidth:r,wrapperWidth:t.wrapper.clientWidth}}function ar(e){return e.scroller.getBoundingClientRect().left-e.sizer.getBoundingClientRect().left}function sr(e){var t=rr(e.display),n=e.options.lineWrapping,r=n&&Math.max(5,e.display.scroller.clientWidth/ir(e.display)-3);return function(i){if(Ht(e.doc,i))return 0;var o=0;if(i.widgets)for(var a=0;a<i.widgets.length;a++)i.widgets[a].height&&(o+=i.widgets[a].height);return n?o+(Math.ceil(i.text.length/r)||1)*t:o+t}}function lr(e){var t=e.doc,n=sr(e);t.iter((function(e){var t=n(e);t!=e.height&&Ze(e,t)}))}function ur(e,t,n,r){var i=e.display;if(!n&&"true"==Ie(t).getAttribute("cm-not-content"))return null;var o,a,s=i.lineSpace.getBoundingClientRect();try{o=t.clientX-s.left,a=t.clientY-s.top}catch(e){return null}var l,u=Qn(e,o,a);if(r&&u.xRel>0&&(l=Ge(e.doc,u.line).text).length==u.ch){var c=z(l,l.length,e.options.tabSize)-l.length;u=et(u.line,Math.max(0,Math.round((o-wn(e.display).left)/ir(e.display))-c))}return u}function cr(e,t){if(t>=e.display.viewTo)return null;if((t-=e.display.viewFrom)<0)return null;for(var n=e.display.view,r=0;r<n.length;r++)if((t-=n[r].size)<0)return r}function dr(e,t,n,r){null==t&&(t=e.doc.first),null==n&&(n=e.doc.first+e.doc.size),r||(r=0);var i=e.display;if(r&&n<i.viewTo&&(null==i.updateLineNumbers||i.updateLineNumbers>t)&&(i.updateLineNumbers=t),e.curOp.viewChanged=!0,t>=i.viewTo)_t&&Ft(e.doc,t)<i.viewTo&&fr(e);else if(n<=i.viewFrom)_t&&Bt(e.doc,n+r)>i.viewFrom?fr(e):(i.viewFrom+=r,i.viewTo+=r);else if(t<=i.viewFrom&&n>=i.viewTo)fr(e);else if(t<=i.viewFrom){var o=pr(e,n,n+r,1);o?(i.view=i.view.slice(o.index),i.viewFrom=o.lineN,i.viewTo+=r):fr(e)}else if(n>=i.viewTo){var a=pr(e,t,t,-1);a?(i.view=i.view.slice(0,a.index),i.viewTo=a.lineN):fr(e)}else{var s=pr(e,t,t,-1),l=pr(e,n,n+r,1);s&&l?(i.view=i.view.slice(0,s.index).concat(on(e,s.lineN,l.lineN)).concat(i.view.slice(l.index)),i.viewTo+=r):fr(e)}var u=i.externalMeasured;u&&(n<u.lineN?u.lineN+=r:t<u.lineN+u.size&&(i.externalMeasured=null))}function hr(e,t,n){e.curOp.viewChanged=!0;var r=e.display,i=e.display.externalMeasured;if(i&&t>=i.lineN&&t<i.lineN+i.size&&(r.externalMeasured=null),!(t<r.viewFrom||t>=r.viewTo)){var o=r.view[cr(e,t)];if(null!=o.node){var a=o.changes||(o.changes=[]);-1==B(a,n)&&a.push(n)}}}function fr(e){e.display.viewFrom=e.display.viewTo=e.doc.first,e.display.view=[],e.display.viewOffset=0}function pr(e,t,n,r){var i,o=cr(e,t),a=e.display.view;if(!_t||n==e.doc.first+e.doc.size)return{index:o,lineN:n};for(var s=e.display.viewFrom,l=0;l<o;l++)s+=a[l].size;if(s!=t){if(r>0){if(o==a.length-1)return null;i=s+a[o].size-t,o++}else i=s-t;t+=i,n+=i}for(;Ft(e.doc,n)!=n;){if(o==(r<0?0:a.length-1))return null;n+=r*a[o-(r<0?1:0)].size,o+=r}return{index:o,lineN:n}}function gr(e){for(var t=e.display.view,n=0,r=0;r<t.length;r++){var i=t[r];i.hidden||i.node&&!i.changes||++n}return n}function vr(e){e.display.input.showSelection(e.display.input.prepareSelection())}function mr(e,t){void 0===t&&(t=!0);for(var n=e.doc,r={},i=r.cursors=document.createDocumentFragment(),o=r.selection=document.createDocumentFragment(),a=0;a<n.sel.ranges.length;a++)if(t||a!=n.sel.primIndex){var s=n.sel.ranges[a];if(!(s.from().line>=e.display.viewTo||s.to().line<e.display.viewFrom)){var l=s.empty();(l||e.options.showCursorWhenSelecting)&&yr(e,s.head,i),l||Cr(e,s,o)}}return r}function yr(e,t,n){var r=Xn(e,t,"div",null,null,!e.options.singleCursorHeightPerLine),i=n.appendChild(E("div"," ","CodeMirror-cursor"));if(i.style.left=r.left+"px",i.style.top=r.top+"px",i.style.height=Math.max(0,r.bottom-r.top)*e.options.cursorHeight+"px",r.other){var o=n.appendChild(E("div"," ","CodeMirror-cursor CodeMirror-secondarycursor"));o.style.display="",o.style.left=r.other.left+"px",o.style.top=r.other.top+"px",o.style.height=.85*(r.other.bottom-r.other.top)+"px"}}function br(e,t){return e.top-t.top||e.left-t.left}function Cr(e,t,n){var r=e.display,i=e.doc,o=document.createDocumentFragment(),a=wn(e.display),s=a.left,l=Math.max(r.sizerWidth,Sn(e)-r.sizer.offsetLeft)-a.right,u="ltr"==i.direction;function c(e,t,n,r){t<0&&(t=0),t=Math.round(t),r=Math.round(r),o.appendChild(E("div",null,"CodeMirror-selected","position: absolute; left: "+e+"px;\n                             top: "+t+"px; width: "+(null==n?l-e:n)+"px;\n                             height: "+(r-t)+"px"))}function d(t,n,r){var o,a,d=Ge(i,t),h=d.text.length;function f(n,r){return Gn(e,et(t,n),"div",d,r)}function p(t,n,r){var i=Jn(e,d,null,t),o="ltr"==n==("after"==r)?"left":"right";return f("after"==r?i.begin:i.end-(/\s/.test(d.text.charAt(i.end-1))?2:1),o)[o]}var g=ue(d,i.direction);return function(e,t,n,r){if(!e)return r(t,n,"ltr",0);for(var i=!1,o=0;o<e.length;++o){var a=e[o];(a.from<n&&a.to>t||t==n&&a.to==t)&&(r(Math.max(a.from,t),Math.min(a.to,n),1==a.level?"rtl":"ltr",o),i=!0)}i||r(t,n,"ltr")}(g,n||0,null==r?h:r,(function(e,t,i,d){var v="ltr"==i,m=f(e,v?"left":"right"),y=f(t-1,v?"right":"left"),b=null==n&&0==e,C=null==r&&t==h,A=0==d,_=!g||d==g.length-1;if(y.top-m.top<=3){var I=(u?C:b)&&_,x=(u?b:C)&&A?s:(v?m:y).left,w=I?l:(v?y:m).right;c(x,m.top,w-x,m.bottom)}else{var M,S,T,E;v?(M=u&&b&&A?s:m.left,S=u?l:p(e,i,"before"),T=u?s:p(t,i,"after"),E=u&&C&&_?l:y.right):(M=u?p(e,i,"before"):s,S=!u&&b&&A?l:m.right,T=!u&&C&&_?s:y.left,E=u?p(t,i,"after"):l),c(M,m.top,S-M,m.bottom),m.bottom<y.top&&c(s,m.bottom,null,y.top),c(T,y.top,E-T,y.bottom)}(!o||br(m,o)<0)&&(o=m),br(y,o)<0&&(o=y),(!a||br(m,a)<0)&&(a=m),br(y,a)<0&&(a=y)})),{start:o,end:a}}var h=t.from(),f=t.to();if(h.line==f.line)d(h.line,h.ch,f.ch);else{var p=Ge(i,h.line),g=Ge(i,f.line),v=zt(p)==zt(g),m=d(h.line,h.ch,v?p.text.length+1:null).end,y=d(f.line,v?0:null,f.ch).start;v&&(m.top<y.top-2?(c(m.right,m.top,null,m.bottom),c(s,y.top,y.left,y.bottom)):c(m.right,m.top,y.left-m.right,m.bottom)),m.bottom<y.top&&c(s,m.bottom,null,y.top)}n.appendChild(o)}function Ar(e){if(e.state.focused){var t=e.display;clearInterval(t.blinker);var n=!0;t.cursorDiv.style.visibility="",e.options.cursorBlinkRate>0?t.blinker=setInterval((function(){return t.cursorDiv.style.visibility=(n=!n)?"":"hidden"}),e.options.cursorBlinkRate):e.options.cursorBlinkRate<0&&(t.cursorDiv.style.visibility="hidden")}}function _r(e){e.state.focused||(e.display.input.focus(),xr(e))}function Ir(e){e.state.delayingBlurEvent=!0,setTimeout((function(){e.state.delayingBlurEvent&&(e.state.delayingBlurEvent=!1,wr(e))}),100)}function xr(e,t){e.state.delayingBlurEvent&&(e.state.delayingBlurEvent=!1),"nocursor"!=e.options.readOnly&&(e.state.focused||(pe(e,"focus",e,t),e.state.focused=!0,D(e.display.wrapper,"CodeMirror-focused"),e.curOp||e.display.selForContextMenu==e.doc.sel||(e.display.input.reset(),l&&setTimeout((function(){return e.display.input.reset(!0)}),20)),e.display.input.receivedFocus()),Ar(e))}function wr(e,t){e.state.delayingBlurEvent||(e.state.focused&&(pe(e,"blur",e,t),e.state.focused=!1,M(e.display.wrapper,"CodeMirror-focused")),clearInterval(e.display.blinker),setTimeout((function(){e.state.focused||(e.display.shift=!1)}),150))}function Mr(e){for(var t=e.display,n=t.lineDiv.offsetTop,r=0;r<t.view.length;r++){var i=t.view[r],o=e.options.lineWrapping,l=void 0,u=0;if(!i.hidden){if(a&&s<8){var c=i.node.offsetTop+i.node.offsetHeight;l=c-n,n=c}else{var d=i.node.getBoundingClientRect();l=d.bottom-d.top,!o&&i.text.firstChild&&(u=i.text.firstChild.getBoundingClientRect().right-d.left-1)}var h=i.line.height-l;if((h>.005||h<-.005)&&(Ze(i.line,l),Sr(i.line),i.rest))for(var f=0;f<i.rest.length;f++)Sr(i.rest[f]);if(u>e.display.sizerWidth){var p=Math.ceil(u/ir(e.display));p>e.display.maxLineLength&&(e.display.maxLineLength=p,e.display.maxLine=i.line,e.display.maxLineChanged=!0)}}}}function Sr(e){if(e.widgets)for(var t=0;t<e.widgets.length;++t){var n=e.widgets[t],r=n.node.parentNode;r&&(n.height=r.offsetHeight)}}function Tr(e,t,n){var r=n&&null!=n.top?Math.max(0,n.top):e.scroller.scrollTop;r=Math.floor(r-In(e));var i=n&&null!=n.bottom?n.bottom:r+e.wrapper.clientHeight,o=Ke(t,r),a=Ke(t,i);if(n&&n.ensure){var s=n.ensure.from.line,l=n.ensure.to.line;s<o?(o=s,a=Ke(t,Vt(Ge(t,s))+e.wrapper.clientHeight)):Math.min(l,t.lastLine())>=a&&(o=Ke(t,Vt(Ge(t,l))-e.wrapper.clientHeight),a=l)}return{from:o,to:Math.max(a,o+1)}}function Er(e,t){var n=e.display,r=rr(e.display);t.top<0&&(t.top=0);var i=e.curOp&&null!=e.curOp.scrollTop?e.curOp.scrollTop:n.scroller.scrollTop,o=Tn(e),a={};t.bottom-t.top>o&&(t.bottom=t.top+o);var s=e.doc.height+xn(n),l=t.top<r,u=t.bottom>s-r;if(t.top<i)a.scrollTop=l?0:t.top;else if(t.bottom>i+o){var c=Math.min(t.top,(u?s:t.bottom)-o);c!=i&&(a.scrollTop=c)}var d=e.curOp&&null!=e.curOp.scrollLeft?e.curOp.scrollLeft:n.scroller.scrollLeft,h=Sn(e)-(e.options.fixedGutter?n.gutters.offsetWidth:0),f=t.right-t.left>h;return f&&(t.right=t.left+h),t.left<10?a.scrollLeft=0:t.left<d?a.scrollLeft=Math.max(0,t.left-(f?0:10)):t.right>h+d-3&&(a.scrollLeft=t.right+(f?0:10)-h),a}function Or(e,t){null!=t&&(Dr(e),e.curOp.scrollTop=(null==e.curOp.scrollTop?e.doc.scrollTop:e.curOp.scrollTop)+t)}function kr(e){Dr(e);var t=e.getCursor();e.curOp.scrollToPos={from:t,to:t,margin:e.options.cursorScrollMargin}}function Lr(e,t,n){null==t&&null==n||Dr(e),null!=t&&(e.curOp.scrollLeft=t),null!=n&&(e.curOp.scrollTop=n)}function Dr(e){var t=e.curOp.scrollToPos;t&&(e.curOp.scrollToPos=null,Pr(e,qn(e,t.from),qn(e,t.to),t.margin))}function Pr(e,t,n,r){var i=Er(e,{left:Math.min(t.left,n.left),top:Math.min(t.top,n.top)-r,right:Math.max(t.right,n.right),bottom:Math.max(t.bottom,n.bottom)+r});Lr(e,i.scrollLeft,i.scrollTop)}function Nr(e,t){Math.abs(e.doc.scrollTop-t)<2||(n||li(e,{top:t}),jr(e,t,!0),n&&li(e),ri(e,100))}function jr(e,t,n){t=Math.max(0,Math.min(e.display.scroller.scrollHeight-e.display.scroller.clientHeight,t)),(e.display.scroller.scrollTop!=t||n)&&(e.doc.scrollTop=t,e.display.scrollbars.setScrollTop(t),e.display.scroller.scrollTop!=t&&(e.display.scroller.scrollTop=t))}function Rr(e,t,n,r){t=Math.max(0,Math.min(t,e.display.scroller.scrollWidth-e.display.scroller.clientWidth)),(n?t==e.doc.scrollLeft:Math.abs(e.doc.scrollLeft-t)<2)&&!r||(e.doc.scrollLeft=t,di(e),e.display.scroller.scrollLeft!=t&&(e.display.scroller.scrollLeft=t),e.display.scrollbars.setScrollLeft(t))}function zr(e){var t=e.display,n=t.gutters.offsetWidth,r=Math.round(e.doc.height+xn(e.display));return{clientHeight:t.scroller.clientHeight,viewHeight:t.wrapper.clientHeight,scrollWidth:t.scroller.scrollWidth,clientWidth:t.scroller.clientWidth,viewWidth:t.wrapper.clientWidth,barLeft:e.options.fixedGutter?n:0,docHeight:r,scrollHeight:r+Mn(e)+t.barHeight,nativeBarWidth:t.nativeBarWidth,gutterWidth:n}}var Fr=function(e,t,n){this.cm=n;var r=this.vert=E("div",[E("div",null,null,"min-width: 1px")],"CodeMirror-vscrollbar"),i=this.horiz=E("div",[E("div",null,null,"height: 100%; min-height: 1px")],"CodeMirror-hscrollbar");r.tabIndex=i.tabIndex=-1,e(r),e(i),de(r,"scroll",(function(){r.clientHeight&&t(r.scrollTop,"vertical")})),de(i,"scroll",(function(){i.clientWidth&&t(i.scrollLeft,"horizontal")})),this.checkedZeroWidth=!1,a&&s<8&&(this.horiz.style.minHeight=this.vert.style.minWidth="18px")};Fr.prototype.update=function(e){var t=e.scrollWidth>e.clientWidth+1,n=e.scrollHeight>e.clientHeight+1,r=e.nativeBarWidth;if(n){this.vert.style.display="block",this.vert.style.bottom=t?r+"px":"0";var i=e.viewHeight-(t?r:0);this.vert.firstChild.style.height=Math.max(0,e.scrollHeight-e.clientHeight+i)+"px"}else this.vert.style.display="",this.vert.firstChild.style.height="0";if(t){this.horiz.style.display="block",this.horiz.style.right=n?r+"px":"0",this.horiz.style.left=e.barLeft+"px";var o=e.viewWidth-e.barLeft-(n?r:0);this.horiz.firstChild.style.width=Math.max(0,e.scrollWidth-e.clientWidth+o)+"px"}else this.horiz.style.display="",this.horiz.firstChild.style.width="0";return!this.checkedZeroWidth&&e.clientHeight>0&&(0==r&&this.zeroWidthHack(),this.checkedZeroWidth=!0),{right:n?r:0,bottom:t?r:0}},Fr.prototype.setScrollLeft=function(e){this.horiz.scrollLeft!=e&&(this.horiz.scrollLeft=e),this.disableHoriz&&this.enableZeroWidthBar(this.horiz,this.disableHoriz,"horiz")},Fr.prototype.setScrollTop=function(e){this.vert.scrollTop!=e&&(this.vert.scrollTop=e),this.disableVert&&this.enableZeroWidthBar(this.vert,this.disableVert,"vert")},Fr.prototype.zeroWidthHack=function(){var e=y&&!f?"12px":"18px";this.horiz.style.height=this.vert.style.width=e,this.horiz.style.pointerEvents=this.vert.style.pointerEvents="none",this.disableHoriz=new F,this.disableVert=new F},Fr.prototype.enableZeroWidthBar=function(e,t,n){e.style.pointerEvents="auto",t.set(1e3,(function r(){var i=e.getBoundingClientRect();("vert"==n?document.elementFromPoint(i.right-1,(i.top+i.bottom)/2):document.elementFromPoint((i.right+i.left)/2,i.bottom-1))!=e?e.style.pointerEvents="none":t.set(1e3,r)}))},Fr.prototype.clear=function(){var e=this.horiz.parentNode;e.removeChild(this.horiz),e.removeChild(this.vert)};var Br=function(){};function Hr(e,t){t||(t=zr(e));var n=e.display.barWidth,r=e.display.barHeight;Ur(e,t);for(var i=0;i<4&&n!=e.display.barWidth||r!=e.display.barHeight;i++)n!=e.display.barWidth&&e.options.lineWrapping&&Mr(e),Ur(e,zr(e)),n=e.display.barWidth,r=e.display.barHeight}function Ur(e,t){var n=e.display,r=n.scrollbars.update(t);n.sizer.style.paddingRight=(n.barWidth=r.right)+"px",n.sizer.style.paddingBottom=(n.barHeight=r.bottom)+"px",n.heightForcer.style.borderBottom=r.bottom+"px solid transparent",r.right&&r.bottom?(n.scrollbarFiller.style.display="block",n.scrollbarFiller.style.height=r.bottom+"px",n.scrollbarFiller.style.width=r.right+"px"):n.scrollbarFiller.style.display="",r.bottom&&e.options.coverGutterNextToScrollbar&&e.options.fixedGutter?(n.gutterFiller.style.display="block",n.gutterFiller.style.height=r.bottom+"px",n.gutterFiller.style.width=t.gutterWidth+"px"):n.gutterFiller.style.display=""}Br.prototype.update=function(){return{bottom:0,right:0}},Br.prototype.setScrollLeft=function(){},Br.prototype.setScrollTop=function(){},Br.prototype.clear=function(){};var Vr={native:Fr,null:Br};function Yr(e){e.display.scrollbars&&(e.display.scrollbars.clear(),e.display.scrollbars.addClass&&M(e.display.wrapper,e.display.scrollbars.addClass)),e.display.scrollbars=new Vr[e.options.scrollbarStyle]((function(t){e.display.wrapper.insertBefore(t,e.display.scrollbarFiller),de(t,"mousedown",(function(){e.state.focused&&setTimeout((function(){return e.display.input.focus()}),0)})),t.setAttribute("cm-not-content","true")}),(function(t,n){"horizontal"==n?Rr(e,t):Nr(e,t)}),e),e.display.scrollbars.addClass&&D(e.display.wrapper,e.display.scrollbars.addClass)}var Wr=0;function Gr(e){var t;e.curOp={cm:e,viewChanged:!1,startHeight:e.doc.height,forceUpdate:!1,updateInput:0,typing:!1,changeObjs:null,cursorActivityHandlers:null,cursorActivityCalled:0,selectionChanged:!1,updateMaxLine:!1,scrollLeft:null,scrollTop:null,scrollToPos:null,focus:!1,id:++Wr},t=e.curOp,an?an.ops.push(t):t.ownsGroup=an={ops:[t],delayedCallbacks:[]}}function Xr(e){var t=e.curOp;t&&function(e,t){var n=e.ownsGroup;if(n)try{!function(e){var t=e.delayedCallbacks,n=0;do{for(;n<t.length;n++)t[n].call(null);for(var r=0;r<e.ops.length;r++){var i=e.ops[r];if(i.cursorActivityHandlers)for(;i.cursorActivityCalled<i.cursorActivityHandlers.length;)i.cursorActivityHandlers[i.cursorActivityCalled++].call(null,i.cm)}}while(n<t.length)}(n)}finally{an=null,t(n)}}(t,(function(e){for(var t=0;t<e.ops.length;t++)e.ops[t].cm.curOp=null;!function(e){for(var t=e.ops,n=0;n<t.length;n++)qr(t[n]);for(var r=0;r<t.length;r++)Zr(t[r]);for(var i=0;i<t.length;i++)Qr(t[i]);for(var o=0;o<t.length;o++)Kr(t[o]);for(var a=0;a<t.length;a++)Jr(t[a])}(e)}))}function qr(e){var t=e.cm,n=t.display;!function(e){var t=e.display;!t.scrollbarsClipped&&t.scroller.offsetWidth&&(t.nativeBarWidth=t.scroller.offsetWidth-t.scroller.clientWidth,t.heightForcer.style.height=Mn(e)+"px",t.sizer.style.marginBottom=-t.nativeBarWidth+"px",t.sizer.style.borderRightWidth=Mn(e)+"px",t.scrollbarsClipped=!0)}(t),e.updateMaxLine&&Wt(t),e.mustUpdate=e.viewChanged||e.forceUpdate||null!=e.scrollTop||e.scrollToPos&&(e.scrollToPos.from.line<n.viewFrom||e.scrollToPos.to.line>=n.viewTo)||n.maxLineChanged&&t.options.lineWrapping,e.update=e.mustUpdate&&new oi(t,e.mustUpdate&&{top:e.scrollTop,ensure:e.scrollToPos},e.forceUpdate)}function Zr(e){e.updatedDisplay=e.mustUpdate&&ai(e.cm,e.update)}function Qr(e){var t=e.cm,n=t.display;e.updatedDisplay&&Mr(t),e.barMeasure=zr(t),n.maxLineChanged&&!t.options.lineWrapping&&(e.adjustWidthTo=On(t,n.maxLine,n.maxLine.text.length).left+3,t.display.sizerWidth=e.adjustWidthTo,e.barMeasure.scrollWidth=Math.max(n.scroller.clientWidth,n.sizer.offsetLeft+e.adjustWidthTo+Mn(t)+t.display.barWidth),e.maxScrollLeft=Math.max(0,n.sizer.offsetLeft+e.adjustWidthTo-Sn(t))),(e.updatedDisplay||e.selectionChanged)&&(e.preparedSelection=n.input.prepareSelection())}function Kr(e){var t=e.cm;null!=e.adjustWidthTo&&(t.display.sizer.style.minWidth=e.adjustWidthTo+"px",e.maxScrollLeft<t.doc.scrollLeft&&Rr(t,Math.min(t.display.scroller.scrollLeft,e.maxScrollLeft),!0),t.display.maxLineChanged=!1);var n=e.focus&&e.focus==L();e.preparedSelection&&t.display.input.showSelection(e.preparedSelection,n),(e.updatedDisplay||e.startHeight!=t.doc.height)&&Hr(t,e.barMeasure),e.updatedDisplay&&ci(t,e.barMeasure),e.selectionChanged&&Ar(t),t.state.focused&&e.updateInput&&t.display.input.reset(e.typing),n&&_r(e.cm)}function Jr(e){var t=e.cm,n=t.display,r=t.doc;e.updatedDisplay&&si(t,e.update),null==n.wheelStartX||null==e.scrollTop&&null==e.scrollLeft&&!e.scrollToPos||(n.wheelStartX=n.wheelStartY=null),null!=e.scrollTop&&jr(t,e.scrollTop,e.forceScroll),null!=e.scrollLeft&&Rr(t,e.scrollLeft,!0,!0),e.scrollToPos&&function(e,t){if(!ge(e,"scrollCursorIntoView")){var n=e.display,r=n.sizer.getBoundingClientRect(),i=null;if(t.top+r.top<0?i=!0:t.bottom+r.top>(window.innerHeight||document.documentElement.clientHeight)&&(i=!1),null!=i&&!p){var o=E("div","​",null,"position: absolute;\n                         top: "+(t.top-n.viewOffset-In(e.display))+"px;\n                         height: "+(t.bottom-t.top+Mn(e)+n.barHeight)+"px;\n                         left: "+t.left+"px; width: "+Math.max(2,t.right-t.left)+"px;");e.display.lineSpace.appendChild(o),o.scrollIntoView(i),e.display.lineSpace.removeChild(o)}}}(t,function(e,t,n,r){var i;null==r&&(r=0),e.options.lineWrapping||t!=n||(n="before"==(t=t.ch?et(t.line,"before"==t.sticky?t.ch-1:t.ch,"after"):t).sticky?et(t.line,t.ch+1,"before"):t);for(var o=0;o<5;o++){var a=!1,s=Xn(e,t),l=n&&n!=t?Xn(e,n):s,u=Er(e,i={left:Math.min(s.left,l.left),top:Math.min(s.top,l.top)-r,right:Math.max(s.left,l.left),bottom:Math.max(s.bottom,l.bottom)+r}),c=e.doc.scrollTop,d=e.doc.scrollLeft;if(null!=u.scrollTop&&(Nr(e,u.scrollTop),Math.abs(e.doc.scrollTop-c)>1&&(a=!0)),null!=u.scrollLeft&&(Rr(e,u.scrollLeft),Math.abs(e.doc.scrollLeft-d)>1&&(a=!0)),!a)break}return i}(t,st(r,e.scrollToPos.from),st(r,e.scrollToPos.to),e.scrollToPos.margin));var i=e.maybeHiddenMarkers,o=e.maybeUnhiddenMarkers;if(i)for(var a=0;a<i.length;++a)i[a].lines.length||pe(i[a],"hide");if(o)for(var s=0;s<o.length;++s)o[s].lines.length&&pe(o[s],"unhide");n.wrapper.offsetHeight&&(r.scrollTop=t.display.scroller.scrollTop),e.changeObjs&&pe(t,"changes",t,e.changeObjs),e.update&&e.update.finish()}function $r(e,t){if(e.curOp)return t();Gr(e);try{return t()}finally{Xr(e)}}function ei(e,t){return function(){if(e.curOp)return t.apply(e,arguments);Gr(e);try{return t.apply(e,arguments)}finally{Xr(e)}}}function ti(e){return function(){if(this.curOp)return e.apply(this,arguments);Gr(this);try{return e.apply(this,arguments)}finally{Xr(this)}}}function ni(e){return function(){var t=this.cm;if(!t||t.curOp)return e.apply(this,arguments);Gr(t);try{return e.apply(this,arguments)}finally{Xr(t)}}}function ri(e,t){e.doc.highlightFrontier<e.display.viewTo&&e.state.highlight.set(t,j(ii,e))}function ii(e){var t=e.doc;if(!(t.highlightFrontier>=e.display.viewTo)){var n=+new Date+e.options.workTime,r=ft(e,t.highlightFrontier),i=[];t.iter(r.line,Math.min(t.first+t.size,e.display.viewTo+500),(function(o){if(r.line>=e.display.viewFrom){var a=o.styles,s=o.text.length>e.options.maxHighlightLength?Ue(t.mode,r.state):null,l=dt(e,o,r,!0);s&&(r.state=s),o.styles=l.styles;var u=o.styleClasses,c=l.classes;c?o.styleClasses=c:u&&(o.styleClasses=null);for(var d=!a||a.length!=o.styles.length||u!=c&&(!u||!c||u.bgClass!=c.bgClass||u.textClass!=c.textClass),h=0;!d&&h<a.length;++h)d=a[h]!=o.styles[h];d&&i.push(r.line),o.stateAfter=r.save(),r.nextLine()}else o.text.length<=e.options.maxHighlightLength&&pt(e,o.text,r),o.stateAfter=r.line%5==0?r.save():null,r.nextLine();if(+new Date>n)return ri(e,e.options.workDelay),!0})),t.highlightFrontier=r.line,t.modeFrontier=Math.max(t.modeFrontier,r.line),i.length&&$r(e,(function(){for(var t=0;t<i.length;t++)hr(e,i[t],"text")}))}}var oi=function(e,t,n){var r=e.display;this.viewport=t,this.visible=Tr(r,e.doc,t),this.editorIsHidden=!r.wrapper.offsetWidth,this.wrapperHeight=r.wrapper.clientHeight,this.wrapperWidth=r.wrapper.clientWidth,this.oldDisplayWidth=Sn(e),this.force=n,this.dims=or(e),this.events=[]};function ai(e,t){var n=e.display,r=e.doc;if(t.editorIsHidden)return fr(e),!1;if(!t.force&&t.visible.from>=n.viewFrom&&t.visible.to<=n.viewTo&&(null==n.updateLineNumbers||n.updateLineNumbers>=n.viewTo)&&n.renderedView==n.view&&0==gr(e))return!1;hi(e)&&(fr(e),t.dims=or(e));var i=r.first+r.size,o=Math.max(t.visible.from-e.options.viewportMargin,r.first),a=Math.min(i,t.visible.to+e.options.viewportMargin);n.viewFrom<o&&o-n.viewFrom<20&&(o=Math.max(r.first,n.viewFrom)),n.viewTo>a&&n.viewTo-a<20&&(a=Math.min(i,n.viewTo)),_t&&(o=Ft(e.doc,o),a=Bt(e.doc,a));var s=o!=n.viewFrom||a!=n.viewTo||n.lastWrapHeight!=t.wrapperHeight||n.lastWrapWidth!=t.wrapperWidth;!function(e,t,n){var r=e.display;0==r.view.length||t>=r.viewTo||n<=r.viewFrom?(r.view=on(e,t,n),r.viewFrom=t):(r.viewFrom>t?r.view=on(e,t,r.viewFrom).concat(r.view):r.viewFrom<t&&(r.view=r.view.slice(cr(e,t))),r.viewFrom=t,r.viewTo<n?r.view=r.view.concat(on(e,r.viewTo,n)):r.viewTo>n&&(r.view=r.view.slice(0,cr(e,n)))),r.viewTo=n}(e,o,a),n.viewOffset=Vt(Ge(e.doc,n.viewFrom)),e.display.mover.style.top=n.viewOffset+"px";var u=gr(e);if(!s&&0==u&&!t.force&&n.renderedView==n.view&&(null==n.updateLineNumbers||n.updateLineNumbers>=n.viewTo))return!1;var c=function(e){if(e.hasFocus())return null;var t=L();if(!t||!k(e.display.lineDiv,t))return null;var n={activeElt:t};if(window.getSelection){var r=window.getSelection();r.anchorNode&&r.extend&&k(e.display.lineDiv,r.anchorNode)&&(n.anchorNode=r.anchorNode,n.anchorOffset=r.anchorOffset,n.focusNode=r.focusNode,n.focusOffset=r.focusOffset)}return n}(e);return u>4&&(n.lineDiv.style.display="none"),function(e,t,n){var r=e.display,i=e.options.lineNumbers,o=r.lineDiv,a=o.firstChild;function s(t){var n=t.nextSibling;return l&&y&&e.display.currentWheelTarget==t?t.style.display="none":t.parentNode.removeChild(t),n}for(var u=r.view,c=r.viewFrom,d=0;d<u.length;d++){var h=u[d];if(h.hidden);else if(h.node&&h.node.parentNode==o){for(;a!=h.node;)a=s(a);var f=i&&null!=t&&t<=c&&h.lineNumber;h.changes&&(B(h.changes,"gutter")>-1&&(f=!1),cn(e,h,c,n)),f&&(S(h.lineNumber),h.lineNumber.appendChild(document.createTextNode($e(e.options,c)))),a=h.node.nextSibling}else{var p=mn(e,h,c,n);o.insertBefore(p,a)}c+=h.size}for(;a;)a=s(a)}(e,n.updateLineNumbers,t.dims),u>4&&(n.lineDiv.style.display=""),n.renderedView=n.view,function(e){if(e&&e.activeElt&&e.activeElt!=L()&&(e.activeElt.focus(),!/^(INPUT|TEXTAREA)$/.test(e.activeElt.nodeName)&&e.anchorNode&&k(document.body,e.anchorNode)&&k(document.body,e.focusNode))){var t=window.getSelection(),n=document.createRange();n.setEnd(e.anchorNode,e.anchorOffset),n.collapse(!1),t.removeAllRanges(),t.addRange(n),t.extend(e.focusNode,e.focusOffset)}}(c),S(n.cursorDiv),S(n.selectionDiv),n.gutters.style.height=n.sizer.style.minHeight=0,s&&(n.lastWrapHeight=t.wrapperHeight,n.lastWrapWidth=t.wrapperWidth,ri(e,400)),n.updateLineNumbers=null,!0}function si(e,t){for(var n=t.viewport,r=!0;;r=!1){if(r&&e.options.lineWrapping&&t.oldDisplayWidth!=Sn(e))r&&(t.visible=Tr(e.display,e.doc,n));else if(n&&null!=n.top&&(n={top:Math.min(e.doc.height+xn(e.display)-Tn(e),n.top)}),t.visible=Tr(e.display,e.doc,n),t.visible.from>=e.display.viewFrom&&t.visible.to<=e.display.viewTo)break;if(!ai(e,t))break;Mr(e);var i=zr(e);vr(e),Hr(e,i),ci(e,i),t.force=!1}t.signal(e,"update",e),e.display.viewFrom==e.display.reportedViewFrom&&e.display.viewTo==e.display.reportedViewTo||(t.signal(e,"viewportChange",e,e.display.viewFrom,e.display.viewTo),e.display.reportedViewFrom=e.display.viewFrom,e.display.reportedViewTo=e.display.viewTo)}function li(e,t){var n=new oi(e,t);if(ai(e,n)){Mr(e),si(e,n);var r=zr(e);vr(e),Hr(e,r),ci(e,r),n.finish()}}function ui(e){var t=e.gutters.offsetWidth;e.sizer.style.marginLeft=t+"px"}function ci(e,t){e.display.sizer.style.minHeight=t.docHeight+"px",e.display.heightForcer.style.top=t.docHeight+"px",e.display.gutters.style.height=t.docHeight+e.display.barHeight+Mn(e)+"px"}function di(e){var t=e.display,n=t.view;if(t.alignWidgets||t.gutters.firstChild&&e.options.fixedGutter){for(var r=ar(t)-t.scroller.scrollLeft+e.doc.scrollLeft,i=t.gutters.offsetWidth,o=r+"px",a=0;a<n.length;a++)if(!n[a].hidden){e.options.fixedGutter&&(n[a].gutter&&(n[a].gutter.style.left=o),n[a].gutterBackground&&(n[a].gutterBackground.style.left=o));var s=n[a].alignable;if(s)for(var l=0;l<s.length;l++)s[l].style.left=o}e.options.fixedGutter&&(t.gutters.style.left=r+i+"px")}}function hi(e){if(!e.options.lineNumbers)return!1;var t=e.doc,n=$e(e.options,t.first+t.size-1),r=e.display;if(n.length!=r.lineNumChars){var i=r.measure.appendChild(E("div",[E("div",n)],"CodeMirror-linenumber CodeMirror-gutter-elt")),o=i.firstChild.offsetWidth,a=i.offsetWidth-o;return r.lineGutter.style.width="",r.lineNumInnerWidth=Math.max(o,r.lineGutter.offsetWidth-a)+1,r.lineNumWidth=r.lineNumInnerWidth+a,r.lineNumChars=r.lineNumInnerWidth?n.length:-1,r.lineGutter.style.width=r.lineNumWidth+"px",ui(e.display),!0}return!1}function fi(e,t){for(var n=[],r=!1,i=0;i<e.length;i++){var o=e[i],a=null;if("string"!=typeof o&&(a=o.style,o=o.className),"CodeMirror-linenumbers"==o){if(!t)continue;r=!0}n.push({className:o,style:a})}return t&&!r&&n.push({className:"CodeMirror-linenumbers",style:null}),n}function pi(e){var t=e.gutters,n=e.gutterSpecs;S(t),e.lineGutter=null;for(var r=0;r<n.length;++r){var i=n[r],o=i.className,a=i.style,s=t.appendChild(E("div",null,"CodeMirror-gutter "+o));a&&(s.style.cssText=a),"CodeMirror-linenumbers"==o&&(e.lineGutter=s,s.style.width=(e.lineNumWidth||1)+"px")}t.style.display=n.length?"":"none",ui(e)}function gi(e){pi(e.display),dr(e),di(e)}function vi(e,t,r,i){var o=this;this.input=r,o.scrollbarFiller=E("div",null,"CodeMirror-scrollbar-filler"),o.scrollbarFiller.setAttribute("cm-not-content","true"),o.gutterFiller=E("div",null,"CodeMirror-gutter-filler"),o.gutterFiller.setAttribute("cm-not-content","true"),o.lineDiv=O("div",null,"CodeMirror-code"),o.selectionDiv=E("div",null,null,"position: relative; z-index: 1"),o.cursorDiv=E("div",null,"CodeMirror-cursors"),o.measure=E("div",null,"CodeMirror-measure"),o.lineMeasure=E("div",null,"CodeMirror-measure"),o.lineSpace=O("div",[o.measure,o.lineMeasure,o.selectionDiv,o.cursorDiv,o.lineDiv],null,"position: relative; outline: none");var u=O("div",[o.lineSpace],"CodeMirror-lines");o.mover=E("div",[u],null,"position: relative"),o.sizer=E("div",[o.mover],"CodeMirror-sizer"),o.sizerWidth=null,o.heightForcer=E("div",null,null,"position: absolute; height: 50px; width: 1px;"),o.gutters=E("div",null,"CodeMirror-gutters"),o.lineGutter=null,o.scroller=E("div",[o.sizer,o.heightForcer,o.gutters],"CodeMirror-scroll"),o.scroller.setAttribute("tabIndex","-1"),o.wrapper=E("div",[o.scrollbarFiller,o.gutterFiller,o.scroller],"CodeMirror"),a&&s<8&&(o.gutters.style.zIndex=-1,o.scroller.style.paddingRight=0),l||n&&m||(o.scroller.draggable=!0),e&&(e.appendChild?e.appendChild(o.wrapper):e(o.wrapper)),o.viewFrom=o.viewTo=t.first,o.reportedViewFrom=o.reportedViewTo=t.first,o.view=[],o.renderedView=null,o.externalMeasured=null,o.viewOffset=0,o.lastWrapHeight=o.lastWrapWidth=0,o.updateLineNumbers=null,o.nativeBarWidth=o.barHeight=o.barWidth=0,o.scrollbarsClipped=!1,o.lineNumWidth=o.lineNumInnerWidth=o.lineNumChars=null,o.alignWidgets=!1,o.cachedCharWidth=o.cachedTextHeight=o.cachedPaddingH=null,o.maxLine=null,o.maxLineLength=0,o.maxLineChanged=!1,o.wheelDX=o.wheelDY=o.wheelStartX=o.wheelStartY=null,o.shift=!1,o.selForContextMenu=null,o.activeTouch=null,o.gutterSpecs=fi(i.gutters,i.lineNumbers),pi(o),r.init(o)}oi.prototype.signal=function(e,t){me(e,t)&&this.events.push(arguments)},oi.prototype.finish=function(){for(var e=0;e<this.events.length;e++)pe.apply(null,this.events[e])};var mi=0,yi=null;function bi(e){var t=e.wheelDeltaX,n=e.wheelDeltaY;return null==t&&e.detail&&e.axis==e.HORIZONTAL_AXIS&&(t=e.detail),null==n&&e.detail&&e.axis==e.VERTICAL_AXIS?n=e.detail:null==n&&(n=e.wheelDelta),{x:t,y:n}}function Ci(e){var t=bi(e);return t.x*=yi,t.y*=yi,t}function Ai(e,t){var r=bi(t),i=r.x,o=r.y,a=e.display,s=a.scroller,u=s.scrollWidth>s.clientWidth,c=s.scrollHeight>s.clientHeight;if(i&&u||o&&c){if(o&&y&&l)e:for(var h=t.target,f=a.view;h!=s;h=h.parentNode)for(var p=0;p<f.length;p++)if(f[p].node==h){e.display.currentWheelTarget=h;break e}if(i&&!n&&!d&&null!=yi)return o&&c&&Nr(e,Math.max(0,s.scrollTop+o*yi)),Rr(e,Math.max(0,s.scrollLeft+i*yi)),(!o||o&&c)&&be(t),void(a.wheelStartX=null);if(o&&null!=yi){var g=o*yi,v=e.doc.scrollTop,m=v+a.wrapper.clientHeight;g<0?v=Math.max(0,v+g-50):m=Math.min(e.doc.height,m+g+50),li(e,{top:v,bottom:m})}mi<20&&(null==a.wheelStartX?(a.wheelStartX=s.scrollLeft,a.wheelStartY=s.scrollTop,a.wheelDX=i,a.wheelDY=o,setTimeout((function(){if(null!=a.wheelStartX){var e=s.scrollLeft-a.wheelStartX,t=s.scrollTop-a.wheelStartY,n=t&&a.wheelDY&&t/a.wheelDY||e&&a.wheelDX&&e/a.wheelDX;a.wheelStartX=a.wheelStartY=null,n&&(yi=(yi*mi+n)/(mi+1),++mi)}}),200)):(a.wheelDX+=i,a.wheelDY+=o))}}a?yi=-.53:n?yi=15:c?yi=-.7:h&&(yi=-1/3);var _i=function(e,t){this.ranges=e,this.primIndex=t};_i.prototype.primary=function(){return this.ranges[this.primIndex]},_i.prototype.equals=function(e){if(e==this)return!0;if(e.primIndex!=this.primIndex||e.ranges.length!=this.ranges.length)return!1;for(var t=0;t<this.ranges.length;t++){var n=this.ranges[t],r=e.ranges[t];if(!nt(n.anchor,r.anchor)||!nt(n.head,r.head))return!1}return!0},_i.prototype.deepCopy=function(){for(var e=[],t=0;t<this.ranges.length;t++)e[t]=new Ii(rt(this.ranges[t].anchor),rt(this.ranges[t].head));return new _i(e,this.primIndex)},_i.prototype.somethingSelected=function(){for(var e=0;e<this.ranges.length;e++)if(!this.ranges[e].empty())return!0;return!1},_i.prototype.contains=function(e,t){t||(t=e);for(var n=0;n<this.ranges.length;n++){var r=this.ranges[n];if(tt(t,r.from())>=0&&tt(e,r.to())<=0)return n}return-1};var Ii=function(e,t){this.anchor=e,this.head=t};function xi(e,t,n){var r=e&&e.options.selectionsMayTouch,i=t[n];t.sort((function(e,t){return tt(e.from(),t.from())})),n=B(t,i);for(var o=1;o<t.length;o++){var a=t[o],s=t[o-1],l=tt(s.to(),a.from());if(r&&!a.empty()?l>0:l>=0){var u=ot(s.from(),a.from()),c=it(s.to(),a.to()),d=s.empty()?a.from()==a.head:s.from()==s.head;o<=n&&--n,t.splice(--o,2,new Ii(d?c:u,d?u:c))}}return new _i(t,n)}function wi(e,t){return new _i([new Ii(e,t||e)],0)}function Mi(e){return e.text?et(e.from.line+e.text.length-1,q(e.text).length+(1==e.text.length?e.from.ch:0)):e.to}function Si(e,t){if(tt(e,t.from)<0)return e;if(tt(e,t.to)<=0)return Mi(t);var n=e.line+t.text.length-(t.to.line-t.from.line)-1,r=e.ch;return e.line==t.to.line&&(r+=Mi(t).ch-t.to.ch),et(n,r)}function Ti(e,t){for(var n=[],r=0;r<e.sel.ranges.length;r++){var i=e.sel.ranges[r];n.push(new Ii(Si(i.anchor,t),Si(i.head,t)))}return xi(e.cm,n,e.sel.primIndex)}function Ei(e,t,n){return e.line==t.line?et(n.line,e.ch-t.ch+n.ch):et(n.line+(e.line-t.line),e.ch)}function Oi(e){e.doc.mode=Fe(e.options,e.doc.modeOption),ki(e)}function ki(e){e.doc.iter((function(e){e.stateAfter&&(e.stateAfter=null),e.styles&&(e.styles=null)})),e.doc.modeFrontier=e.doc.highlightFrontier=e.doc.first,ri(e,100),e.state.modeGen++,e.curOp&&dr(e)}function Li(e,t){return 0==t.from.ch&&0==t.to.ch&&""==q(t.text)&&(!e.cm||e.cm.options.wholeLineUpdateBefore)}function Di(e,t,n,r){function i(e){return n?n[e]:null}function o(e,n,i){!function(e,t,n,r){e.text=t,e.stateAfter&&(e.stateAfter=null),e.styles&&(e.styles=null),null!=e.order&&(e.order=null),Tt(e),Et(e,n);var i=r?r(e):1;i!=e.height&&Ze(e,i)}(e,n,i,r),ln(e,"change",e,t)}function a(e,t){for(var n=[],o=e;o<t;++o)n.push(new Gt(u[o],i(o),r));return n}var s=t.from,l=t.to,u=t.text,c=Ge(e,s.line),d=Ge(e,l.line),h=q(u),f=i(u.length-1),p=l.line-s.line;if(t.full)e.insert(0,a(0,u.length)),e.remove(u.length,e.size-u.length);else if(Li(e,t)){var g=a(0,u.length-1);o(d,d.text,f),p&&e.remove(s.line,p),g.length&&e.insert(s.line,g)}else if(c==d)if(1==u.length)o(c,c.text.slice(0,s.ch)+h+c.text.slice(l.ch),f);else{var v=a(1,u.length-1);v.push(new Gt(h+c.text.slice(l.ch),f,r)),o(c,c.text.slice(0,s.ch)+u[0],i(0)),e.insert(s.line+1,v)}else if(1==u.length)o(c,c.text.slice(0,s.ch)+u[0]+d.text.slice(l.ch),i(0)),e.remove(s.line+1,p);else{o(c,c.text.slice(0,s.ch)+u[0],i(0)),o(d,h+d.text.slice(l.ch),f);var m=a(1,u.length-1);p>1&&e.remove(s.line+1,p-1),e.insert(s.line+1,m)}ln(e,"change",e,t)}function Pi(e,t,n){!function e(r,i,o){if(r.linked)for(var a=0;a<r.linked.length;++a){var s=r.linked[a];if(s.doc!=i){var l=o&&s.sharedHist;n&&!l||(t(s.doc,l),e(s.doc,r,l))}}}(e,null,!0)}function Ni(e,t){if(t.cm)throw new Error("This document is already in use.");e.doc=t,t.cm=e,lr(e),Oi(e),ji(e),e.options.lineWrapping||Wt(e),e.options.mode=t.modeOption,dr(e)}function ji(e){("rtl"==e.doc.direction?D:M)(e.display.lineDiv,"CodeMirror-rtl")}function Ri(e){this.done=[],this.undone=[],this.undoDepth=1/0,this.lastModTime=this.lastSelTime=0,this.lastOp=this.lastSelOp=null,this.lastOrigin=this.lastSelOrigin=null,this.generation=this.maxGeneration=e||1}function zi(e,t){var n={from:rt(t.from),to:Mi(t),text:Xe(e,t.from,t.to)};return Vi(e,n,t.from.line,t.to.line+1),Pi(e,(function(e){return Vi(e,n,t.from.line,t.to.line+1)}),!0),n}function Fi(e){for(;e.length&&q(e).ranges;)e.pop()}function Bi(e,t,n,r){var i=e.history;i.undone.length=0;var o,a,s=+new Date;if((i.lastOp==r||i.lastOrigin==t.origin&&t.origin&&("+"==t.origin.charAt(0)&&i.lastModTime>s-(e.cm?e.cm.options.historyEventDelay:500)||"*"==t.origin.charAt(0)))&&(o=function(e,t){return t?(Fi(e.done),q(e.done)):e.done.length&&!q(e.done).ranges?q(e.done):e.done.length>1&&!e.done[e.done.length-2].ranges?(e.done.pop(),q(e.done)):void 0}(i,i.lastOp==r)))a=q(o.changes),0==tt(t.from,t.to)&&0==tt(t.from,a.to)?a.to=Mi(t):o.changes.push(zi(e,t));else{var l=q(i.done);for(l&&l.ranges||Ui(e.sel,i.done),o={changes:[zi(e,t)],generation:i.generation},i.done.push(o);i.done.length>i.undoDepth;)i.done.shift(),i.done[0].ranges||i.done.shift()}i.done.push(n),i.generation=++i.maxGeneration,i.lastModTime=i.lastSelTime=s,i.lastOp=i.lastSelOp=r,i.lastOrigin=i.lastSelOrigin=t.origin,a||pe(e,"historyAdded")}function Hi(e,t,n,r){var i=e.history,o=r&&r.origin;n==i.lastSelOp||o&&i.lastSelOrigin==o&&(i.lastModTime==i.lastSelTime&&i.lastOrigin==o||function(e,t,n,r){var i=t.charAt(0);return"*"==i||"+"==i&&n.ranges.length==r.ranges.length&&n.somethingSelected()==r.somethingSelected()&&new Date-e.history.lastSelTime<=(e.cm?e.cm.options.historyEventDelay:500)}(e,o,q(i.done),t))?i.done[i.done.length-1]=t:Ui(t,i.done),i.lastSelTime=+new Date,i.lastSelOrigin=o,i.lastSelOp=n,r&&!1!==r.clearRedo&&Fi(i.undone)}function Ui(e,t){var n=q(t);n&&n.ranges&&n.equals(e)||t.push(e)}function Vi(e,t,n,r){var i=t["spans_"+e.id],o=0;e.iter(Math.max(e.first,n),Math.min(e.first+e.size,r),(function(n){n.markedSpans&&((i||(i=t["spans_"+e.id]={}))[o]=n.markedSpans),++o}))}function Yi(e){if(!e)return null;for(var t,n=0;n<e.length;++n)e[n].marker.explicitlyCleared?t||(t=e.slice(0,n)):t&&t.push(e[n]);return t?t.length?t:null:e}function Wi(e,t){var n=function(e,t){var n=t["spans_"+e.id];if(!n)return null;for(var r=[],i=0;i<t.text.length;++i)r.push(Yi(n[i]));return r}(e,t),r=Mt(e,t);if(!n)return r;if(!r)return n;for(var i=0;i<n.length;++i){var o=n[i],a=r[i];if(o&&a)e:for(var s=0;s<a.length;++s){for(var l=a[s],u=0;u<o.length;++u)if(o[u].marker==l.marker)continue e;o.push(l)}else a&&(n[i]=a)}return n}function Gi(e,t,n){for(var r=[],i=0;i<e.length;++i){var o=e[i];if(o.ranges)r.push(n?_i.prototype.deepCopy.call(o):o);else{var a=o.changes,s=[];r.push({changes:s});for(var l=0;l<a.length;++l){var u=a[l],c=void 0;if(s.push({from:u.from,to:u.to,text:u.text}),t)for(var d in u)(c=d.match(/^spans_(\d+)$/))&&B(t,Number(c[1]))>-1&&(q(s)[d]=u[d],delete u[d])}}}return r}function Xi(e,t,n,r){if(r){var i=e.anchor;if(n){var o=tt(t,i)<0;o!=tt(n,i)<0?(i=t,t=n):o!=tt(t,n)<0&&(t=n)}return new Ii(i,t)}return new Ii(n||t,t)}function qi(e,t,n,r,i){null==i&&(i=e.cm&&(e.cm.display.shift||e.extend)),$i(e,new _i([Xi(e.sel.primary(),t,n,i)],0),r)}function Zi(e,t,n){for(var r=[],i=e.cm&&(e.cm.display.shift||e.extend),o=0;o<e.sel.ranges.length;o++)r[o]=Xi(e.sel.ranges[o],t[o],null,i);$i(e,xi(e.cm,r,e.sel.primIndex),n)}function Qi(e,t,n,r){var i=e.sel.ranges.slice(0);i[t]=n,$i(e,xi(e.cm,i,e.sel.primIndex),r)}function Ki(e,t,n,r){$i(e,wi(t,n),r)}function Ji(e,t,n){var r=e.history.done,i=q(r);i&&i.ranges?(r[r.length-1]=t,eo(e,t,n)):$i(e,t,n)}function $i(e,t,n){eo(e,t,n),Hi(e,e.sel,e.cm?e.cm.curOp.id:NaN,n)}function eo(e,t,n){(me(e,"beforeSelectionChange")||e.cm&&me(e.cm,"beforeSelectionChange"))&&(t=function(e,t,n){var r={ranges:t.ranges,update:function(t){this.ranges=[];for(var n=0;n<t.length;n++)this.ranges[n]=new Ii(st(e,t[n].anchor),st(e,t[n].head))},origin:n&&n.origin};return pe(e,"beforeSelectionChange",e,r),e.cm&&pe(e.cm,"beforeSelectionChange",e.cm,r),r.ranges!=t.ranges?xi(e.cm,r.ranges,r.ranges.length-1):t}(e,t,n));var r=n&&n.bias||(tt(t.primary().head,e.sel.primary().head)<0?-1:1);to(e,ro(e,t,r,!0)),n&&!1===n.scroll||!e.cm||kr(e.cm)}function to(e,t){t.equals(e.sel)||(e.sel=t,e.cm&&(e.cm.curOp.updateInput=1,e.cm.curOp.selectionChanged=!0,ve(e.cm)),ln(e,"cursorActivity",e))}function no(e){to(e,ro(e,e.sel,null,!1))}function ro(e,t,n,r){for(var i,o=0;o<t.ranges.length;o++){var a=t.ranges[o],s=t.ranges.length==e.sel.ranges.length&&e.sel.ranges[o],l=oo(e,a.anchor,s&&s.anchor,n,r),u=oo(e,a.head,s&&s.head,n,r);(i||l!=a.anchor||u!=a.head)&&(i||(i=t.ranges.slice(0,o)),i[o]=new Ii(l,u))}return i?xi(e.cm,i,t.primIndex):t}function io(e,t,n,r,i){var o=Ge(e,t.line);if(o.markedSpans)for(var a=0;a<o.markedSpans.length;++a){var s=o.markedSpans[a],l=s.marker,u="selectLeft"in l?!l.selectLeft:l.inclusiveLeft,c="selectRight"in l?!l.selectRight:l.inclusiveRight;if((null==s.from||(u?s.from<=t.ch:s.from<t.ch))&&(null==s.to||(c?s.to>=t.ch:s.to>t.ch))){if(i&&(pe(l,"beforeCursorEnter"),l.explicitlyCleared)){if(o.markedSpans){--a;continue}break}if(!l.atomic)continue;if(n){var d=l.find(r<0?1:-1),h=void 0;if((r<0?c:u)&&(d=ao(e,d,-r,d&&d.line==t.line?o:null)),d&&d.line==t.line&&(h=tt(d,n))&&(r<0?h<0:h>0))return io(e,d,t,r,i)}var f=l.find(r<0?-1:1);return(r<0?u:c)&&(f=ao(e,f,r,f.line==t.line?o:null)),f?io(e,f,t,r,i):null}}return t}function oo(e,t,n,r,i){var o=r||1,a=io(e,t,n,o,i)||!i&&io(e,t,n,o,!0)||io(e,t,n,-o,i)||!i&&io(e,t,n,-o,!0);return a||(e.cantEdit=!0,et(e.first,0))}function ao(e,t,n,r){return n<0&&0==t.ch?t.line>e.first?st(e,et(t.line-1)):null:n>0&&t.ch==(r||Ge(e,t.line)).text.length?t.line<e.first+e.size-1?et(t.line+1,0):null:new et(t.line,t.ch+n)}function so(e){e.setSelection(et(e.firstLine(),0),et(e.lastLine()),U)}function lo(e,t,n){var r={canceled:!1,from:t.from,to:t.to,text:t.text,origin:t.origin,cancel:function(){return r.canceled=!0}};return n&&(r.update=function(t,n,i,o){t&&(r.from=st(e,t)),n&&(r.to=st(e,n)),i&&(r.text=i),void 0!==o&&(r.origin=o)}),pe(e,"beforeChange",e,r),e.cm&&pe(e.cm,"beforeChange",e.cm,r),r.canceled?(e.cm&&(e.cm.curOp.updateInput=2),null):{from:r.from,to:r.to,text:r.text,origin:r.origin}}function uo(e,t,n){if(e.cm){if(!e.cm.curOp)return ei(e.cm,uo)(e,t,n);if(e.cm.state.suppressEdits)return}if(!(me(e,"beforeChange")||e.cm&&me(e.cm,"beforeChange"))||(t=lo(e,t,!0))){var r=At&&!n&&function(e,t,n){var r=null;if(e.iter(t.line,n.line+1,(function(e){if(e.markedSpans)for(var t=0;t<e.markedSpans.length;++t){var n=e.markedSpans[t].marker;!n.readOnly||r&&-1!=B(r,n)||(r||(r=[])).push(n)}})),!r)return null;for(var i=[{from:t,to:n}],o=0;o<r.length;++o)for(var a=r[o],s=a.find(0),l=0;l<i.length;++l){var u=i[l];if(!(tt(u.to,s.from)<0||tt(u.from,s.to)>0)){var c=[l,1],d=tt(u.from,s.from),h=tt(u.to,s.to);(d<0||!a.inclusiveLeft&&!d)&&c.push({from:u.from,to:s.from}),(h>0||!a.inclusiveRight&&!h)&&c.push({from:s.to,to:u.to}),i.splice.apply(i,c),l+=c.length-3}}return i}(e,t.from,t.to);if(r)for(var i=r.length-1;i>=0;--i)co(e,{from:r[i].from,to:r[i].to,text:i?[""]:t.text,origin:t.origin});else co(e,t)}}function co(e,t){if(1!=t.text.length||""!=t.text[0]||0!=tt(t.from,t.to)){var n=Ti(e,t);Bi(e,t,n,e.cm?e.cm.curOp.id:NaN),po(e,t,n,Mt(e,t));var r=[];Pi(e,(function(e,n){n||-1!=B(r,e.history)||(yo(e.history,t),r.push(e.history)),po(e,t,null,Mt(e,t))}))}}function ho(e,t,n){var r=e.cm&&e.cm.state.suppressEdits;if(!r||n){for(var i,o=e.history,a=e.sel,s="undo"==t?o.done:o.undone,l="undo"==t?o.undone:o.done,u=0;u<s.length&&(i=s[u],n?!i.ranges||i.equals(e.sel):i.ranges);u++);if(u!=s.length){for(o.lastOrigin=o.lastSelOrigin=null;;){if(!(i=s.pop()).ranges){if(r)return void s.push(i);break}if(Ui(i,l),n&&!i.equals(e.sel))return void $i(e,i,{clearRedo:!1});a=i}var c=[];Ui(a,l),l.push({changes:c,generation:o.generation}),o.generation=i.generation||++o.maxGeneration;for(var d=me(e,"beforeChange")||e.cm&&me(e.cm,"beforeChange"),h=function(n){var r=i.changes[n];if(r.origin=t,d&&!lo(e,r,!1))return s.length=0,{};c.push(zi(e,r));var o=n?Ti(e,r):q(s);po(e,r,o,Wi(e,r)),!n&&e.cm&&e.cm.scrollIntoView({from:r.from,to:Mi(r)});var a=[];Pi(e,(function(e,t){t||-1!=B(a,e.history)||(yo(e.history,r),a.push(e.history)),po(e,r,null,Wi(e,r))}))},f=i.changes.length-1;f>=0;--f){var p=h(f);if(p)return p.v}}}}function fo(e,t){if(0!=t&&(e.first+=t,e.sel=new _i(Z(e.sel.ranges,(function(e){return new Ii(et(e.anchor.line+t,e.anchor.ch),et(e.head.line+t,e.head.ch))})),e.sel.primIndex),e.cm)){dr(e.cm,e.first,e.first-t,t);for(var n=e.cm.display,r=n.viewFrom;r<n.viewTo;r++)hr(e.cm,r,"gutter")}}function po(e,t,n,r){if(e.cm&&!e.cm.curOp)return ei(e.cm,po)(e,t,n,r);if(t.to.line<e.first)fo(e,t.text.length-1-(t.to.line-t.from.line));else if(!(t.from.line>e.lastLine())){if(t.from.line<e.first){var i=t.text.length-1-(e.first-t.from.line);fo(e,i),t={from:et(e.first,0),to:et(t.to.line+i,t.to.ch),text:[q(t.text)],origin:t.origin}}var o=e.lastLine();t.to.line>o&&(t={from:t.from,to:et(o,Ge(e,o).text.length),text:[t.text[0]],origin:t.origin}),t.removed=Xe(e,t.from,t.to),n||(n=Ti(e,t)),e.cm?function(e,t,n){var r=e.doc,i=e.display,o=t.from,a=t.to,s=!1,l=o.line;e.options.lineWrapping||(l=Qe(zt(Ge(r,o.line))),r.iter(l,a.line+1,(function(e){if(e==i.maxLine)return s=!0,!0}))),r.sel.contains(t.from,t.to)>-1&&ve(e),Di(r,t,n,sr(e)),e.options.lineWrapping||(r.iter(l,o.line+t.text.length,(function(e){var t=Yt(e);t>i.maxLineLength&&(i.maxLine=e,i.maxLineLength=t,i.maxLineChanged=!0,s=!1)})),s&&(e.curOp.updateMaxLine=!0)),function(e,t){if(e.modeFrontier=Math.min(e.modeFrontier,t),!(e.highlightFrontier<t-10)){for(var n=e.first,r=t-1;r>n;r--){var i=Ge(e,r).stateAfter;if(i&&(!(i instanceof ut)||r+i.lookAhead<t)){n=r+1;break}}e.highlightFrontier=Math.min(e.highlightFrontier,n)}}(r,o.line),ri(e,400);var u=t.text.length-(a.line-o.line)-1;t.full?dr(e):o.line!=a.line||1!=t.text.length||Li(e.doc,t)?dr(e,o.line,a.line+1,u):hr(e,o.line,"text");var c=me(e,"changes"),d=me(e,"change");if(d||c){var h={from:o,to:a,text:t.text,removed:t.removed,origin:t.origin};d&&ln(e,"change",e,h),c&&(e.curOp.changeObjs||(e.curOp.changeObjs=[])).push(h)}e.display.selForContextMenu=null}(e.cm,t,r):Di(e,t,r),eo(e,n,U),e.cantEdit&&oo(e,et(e.firstLine(),0))&&(e.cantEdit=!1)}}function go(e,t,n,r,i){var o;r||(r=n),tt(r,n)<0&&(n=(o=[r,n])[0],r=o[1]),"string"==typeof t&&(t=e.splitLines(t)),uo(e,{from:n,to:r,text:t,origin:i})}function vo(e,t,n,r){n<e.line?e.line+=r:t<e.line&&(e.line=t,e.ch=0)}function mo(e,t,n,r){for(var i=0;i<e.length;++i){var o=e[i],a=!0;if(o.ranges){o.copied||((o=e[i]=o.deepCopy()).copied=!0);for(var s=0;s<o.ranges.length;s++)vo(o.ranges[s].anchor,t,n,r),vo(o.ranges[s].head,t,n,r)}else{for(var l=0;l<o.changes.length;++l){var u=o.changes[l];if(n<u.from.line)u.from=et(u.from.line+r,u.from.ch),u.to=et(u.to.line+r,u.to.ch);else if(t<=u.to.line){a=!1;break}}a||(e.splice(0,i+1),i=0)}}}function yo(e,t){var n=t.from.line,r=t.to.line,i=t.text.length-(r-n)-1;mo(e.done,n,r,i),mo(e.undone,n,r,i)}function bo(e,t,n,r){var i=t,o=t;return"number"==typeof t?o=Ge(e,at(e,t)):i=Qe(t),null==i?null:(r(o,i)&&e.cm&&hr(e.cm,i,n),o)}function Co(e){this.lines=e,this.parent=null;for(var t=0,n=0;n<e.length;++n)e[n].parent=this,t+=e[n].height;this.height=t}function Ao(e){this.children=e;for(var t=0,n=0,r=0;r<e.length;++r){var i=e[r];t+=i.chunkSize(),n+=i.height,i.parent=this}this.size=t,this.height=n,this.parent=null}Ii.prototype.from=function(){return ot(this.anchor,this.head)},Ii.prototype.to=function(){return it(this.anchor,this.head)},Ii.prototype.empty=function(){return this.head.line==this.anchor.line&&this.head.ch==this.anchor.ch},Co.prototype={chunkSize:function(){return this.lines.length},removeInner:function(e,t){for(var n=e,r=e+t;n<r;++n){var i=this.lines[n];this.height-=i.height,Xt(i),ln(i,"delete")}this.lines.splice(e,t)},collapse:function(e){e.push.apply(e,this.lines)},insertInner:function(e,t,n){this.height+=n,this.lines=this.lines.slice(0,e).concat(t).concat(this.lines.slice(e));for(var r=0;r<t.length;++r)t[r].parent=this},iterN:function(e,t,n){for(var r=e+t;e<r;++e)if(n(this.lines[e]))return!0}},Ao.prototype={chunkSize:function(){return this.size},removeInner:function(e,t){this.size-=t;for(var n=0;n<this.children.length;++n){var r=this.children[n],i=r.chunkSize();if(e<i){var o=Math.min(t,i-e),a=r.height;if(r.removeInner(e,o),this.height-=a-r.height,i==o&&(this.children.splice(n--,1),r.parent=null),0==(t-=o))break;e=0}else e-=i}if(this.size-t<25&&(this.children.length>1||!(this.children[0]instanceof Co))){var s=[];this.collapse(s),this.children=[new Co(s)],this.children[0].parent=this}},collapse:function(e){for(var t=0;t<this.children.length;++t)this.children[t].collapse(e)},insertInner:function(e,t,n){this.size+=t.length,this.height+=n;for(var r=0;r<this.children.length;++r){var i=this.children[r],o=i.chunkSize();if(e<=o){if(i.insertInner(e,t,n),i.lines&&i.lines.length>50){for(var a=i.lines.length%25+25,s=a;s<i.lines.length;){var l=new Co(i.lines.slice(s,s+=25));i.height-=l.height,this.children.splice(++r,0,l),l.parent=this}i.lines=i.lines.slice(0,a),this.maybeSpill()}break}e-=o}},maybeSpill:function(){if(!(this.children.length<=10)){var e=this;do{var t=new Ao(e.children.splice(e.children.length-5,5));if(e.parent){e.size-=t.size,e.height-=t.height;var n=B(e.parent.children,e);e.parent.children.splice(n+1,0,t)}else{var r=new Ao(e.children);r.parent=e,e.children=[r,t],e=r}t.parent=e.parent}while(e.children.length>10);e.parent.maybeSpill()}},iterN:function(e,t,n){for(var r=0;r<this.children.length;++r){var i=this.children[r],o=i.chunkSize();if(e<o){var a=Math.min(t,o-e);if(i.iterN(e,a,n))return!0;if(0==(t-=a))break;e=0}else e-=o}}};var _o=function(e,t,n){if(n)for(var r in n)n.hasOwnProperty(r)&&(this[r]=n[r]);this.doc=e,this.node=t};function Io(e,t,n){Vt(t)<(e.curOp&&e.curOp.scrollTop||e.doc.scrollTop)&&Or(e,n)}_o.prototype.clear=function(){var e=this.doc.cm,t=this.line.widgets,n=this.line,r=Qe(n);if(null!=r&&t){for(var i=0;i<t.length;++i)t[i]==this&&t.splice(i--,1);t.length||(n.widgets=null);var o=An(this);Ze(n,Math.max(0,n.height-o)),e&&($r(e,(function(){Io(e,n,-o),hr(e,r,"widget")})),ln(e,"lineWidgetCleared",e,this,r))}},_o.prototype.changed=function(){var e=this,t=this.height,n=this.doc.cm,r=this.line;this.height=null;var i=An(this)-t;i&&(Ht(this.doc,r)||Ze(r,r.height+i),n&&$r(n,(function(){n.curOp.forceUpdate=!0,Io(n,r,i),ln(n,"lineWidgetChanged",n,e,Qe(r))})))},ye(_o);var xo=0,wo=function(e,t){this.lines=[],this.type=t,this.doc=e,this.id=++xo};function Mo(e,t,n,r,i){if(r&&r.shared)return function(e,t,n,r,i){(r=R(r)).shared=!1;var o=[Mo(e,t,n,r,i)],a=o[0],s=r.widgetNode;return Pi(e,(function(e){s&&(r.widgetNode=s.cloneNode(!0)),o.push(Mo(e,st(e,t),st(e,n),r,i));for(var l=0;l<e.linked.length;++l)if(e.linked[l].isParent)return;a=q(o)})),new So(o,a)}(e,t,n,r,i);if(e.cm&&!e.cm.curOp)return ei(e.cm,Mo)(e,t,n,r,i);var o=new wo(e,i),a=tt(t,n);if(r&&R(r,o,!1),a>0||0==a&&!1!==o.clearWhenEmpty)return o;if(o.replacedWith&&(o.collapsed=!0,o.widgetNode=O("span",[o.replacedWith],"CodeMirror-widget"),r.handleMouseEvents||o.widgetNode.setAttribute("cm-ignore-events","true"),r.insertLeft&&(o.widgetNode.insertLeft=!0)),o.collapsed){if(Rt(e,t.line,t,n,o)||t.line!=n.line&&Rt(e,n.line,t,n,o))throw new Error("Inserting collapsed marker partially overlapping an existing one");_t=!0}o.addToHistory&&Bi(e,{from:t,to:n,origin:"markText"},e.sel,NaN);var s,l=t.line,u=e.cm;if(e.iter(l,n.line+1,(function(e){u&&o.collapsed&&!u.options.lineWrapping&&zt(e)==u.display.maxLine&&(s=!0),o.collapsed&&l!=t.line&&Ze(e,0),function(e,t){e.markedSpans=e.markedSpans?e.markedSpans.concat([t]):[t],t.marker.attachLine(e)}(e,new It(o,l==t.line?t.ch:null,l==n.line?n.ch:null)),++l})),o.collapsed&&e.iter(t.line,n.line+1,(function(t){Ht(e,t)&&Ze(t,0)})),o.clearOnEnter&&de(o,"beforeCursorEnter",(function(){return o.clear()})),o.readOnly&&(At=!0,(e.history.done.length||e.history.undone.length)&&e.clearHistory()),o.collapsed&&(o.id=++xo,o.atomic=!0),u){if(s&&(u.curOp.updateMaxLine=!0),o.collapsed)dr(u,t.line,n.line+1);else if(o.className||o.startStyle||o.endStyle||o.css||o.attributes||o.title)for(var c=t.line;c<=n.line;c++)hr(u,c,"text");o.atomic&&no(u.doc),ln(u,"markerAdded",u,o)}return o}wo.prototype.clear=function(){if(!this.explicitlyCleared){var e=this.doc.cm,t=e&&!e.curOp;if(t&&Gr(e),me(this,"clear")){var n=this.find();n&&ln(this,"clear",n.from,n.to)}for(var r=null,i=null,o=0;o<this.lines.length;++o){var a=this.lines[o],s=xt(a.markedSpans,this);e&&!this.collapsed?hr(e,Qe(a),"text"):e&&(null!=s.to&&(i=Qe(a)),null!=s.from&&(r=Qe(a))),a.markedSpans=wt(a.markedSpans,s),null==s.from&&this.collapsed&&!Ht(this.doc,a)&&e&&Ze(a,rr(e.display))}if(e&&this.collapsed&&!e.options.lineWrapping)for(var l=0;l<this.lines.length;++l){var u=zt(this.lines[l]),c=Yt(u);c>e.display.maxLineLength&&(e.display.maxLine=u,e.display.maxLineLength=c,e.display.maxLineChanged=!0)}null!=r&&e&&this.collapsed&&dr(e,r,i+1),this.lines.length=0,this.explicitlyCleared=!0,this.atomic&&this.doc.cantEdit&&(this.doc.cantEdit=!1,e&&no(e.doc)),e&&ln(e,"markerCleared",e,this,r,i),t&&Xr(e),this.parent&&this.parent.clear()}},wo.prototype.find=function(e,t){var n,r;null==e&&"bookmark"==this.type&&(e=1);for(var i=0;i<this.lines.length;++i){var o=this.lines[i],a=xt(o.markedSpans,this);if(null!=a.from&&(n=et(t?o:Qe(o),a.from),-1==e))return n;if(null!=a.to&&(r=et(t?o:Qe(o),a.to),1==e))return r}return n&&{from:n,to:r}},wo.prototype.changed=function(){var e=this,t=this.find(-1,!0),n=this,r=this.doc.cm;t&&r&&$r(r,(function(){var i=t.line,o=Qe(t.line),a=kn(r,o);if(a&&(zn(a),r.curOp.selectionChanged=r.curOp.forceUpdate=!0),r.curOp.updateMaxLine=!0,!Ht(n.doc,i)&&null!=n.height){var s=n.height;n.height=null;var l=An(n)-s;l&&Ze(i,i.height+l)}ln(r,"markerChanged",r,e)}))},wo.prototype.attachLine=function(e){if(!this.lines.length&&this.doc.cm){var t=this.doc.cm.curOp;t.maybeHiddenMarkers&&-1!=B(t.maybeHiddenMarkers,this)||(t.maybeUnhiddenMarkers||(t.maybeUnhiddenMarkers=[])).push(this)}this.lines.push(e)},wo.prototype.detachLine=function(e){if(this.lines.splice(B(this.lines,e),1),!this.lines.length&&this.doc.cm){var t=this.doc.cm.curOp;(t.maybeHiddenMarkers||(t.maybeHiddenMarkers=[])).push(this)}},ye(wo);var So=function(e,t){this.markers=e,this.primary=t;for(var n=0;n<e.length;++n)e[n].parent=this};function To(e){return e.findMarks(et(e.first,0),e.clipPos(et(e.lastLine())),(function(e){return e.parent}))}function Eo(e){for(var t=function(t){var n=e[t],r=[n.primary.doc];Pi(n.primary.doc,(function(e){return r.push(e)}));for(var i=0;i<n.markers.length;i++){var o=n.markers[i];-1==B(r,o.doc)&&(o.parent=null,n.markers.splice(i--,1))}},n=0;n<e.length;n++)t(n)}So.prototype.clear=function(){if(!this.explicitlyCleared){this.explicitlyCleared=!0;for(var e=0;e<this.markers.length;++e)this.markers[e].clear();ln(this,"clear")}},So.prototype.find=function(e,t){return this.primary.find(e,t)},ye(So);var Oo=0,ko=function(e,t,n,r,i){if(!(this instanceof ko))return new ko(e,t,n,r,i);null==n&&(n=0),Ao.call(this,[new Co([new Gt("",null)])]),this.first=n,this.scrollTop=this.scrollLeft=0,this.cantEdit=!1,this.cleanGeneration=1,this.modeFrontier=this.highlightFrontier=n;var o=et(n,0);this.sel=wi(o),this.history=new Ri(null),this.id=++Oo,this.modeOption=t,this.lineSep=r,this.direction="rtl"==i?"rtl":"ltr",this.extend=!1,"string"==typeof e&&(e=this.splitLines(e)),Di(this,{from:o,to:o,text:e}),$i(this,wi(o),U)};ko.prototype=K(Ao.prototype,{constructor:ko,iter:function(e,t,n){n?this.iterN(e-this.first,t-e,n):this.iterN(this.first,this.first+this.size,e)},insert:function(e,t){for(var n=0,r=0;r<t.length;++r)n+=t[r].height;this.insertInner(e-this.first,t,n)},remove:function(e,t){this.removeInner(e-this.first,t)},getValue:function(e){var t=qe(this,this.first,this.first+this.size);return!1===e?t:t.join(e||this.lineSeparator())},setValue:ni((function(e){var t=et(this.first,0),n=this.first+this.size-1;uo(this,{from:t,to:et(n,Ge(this,n).text.length),text:this.splitLines(e),origin:"setValue",full:!0},!0),this.cm&&Lr(this.cm,0,0),$i(this,wi(t),U)})),replaceRange:function(e,t,n,r){go(this,e,t=st(this,t),n=n?st(this,n):t,r)},getRange:function(e,t,n){var r=Xe(this,st(this,e),st(this,t));return!1===n?r:r.join(n||this.lineSeparator())},getLine:function(e){var t=this.getLineHandle(e);return t&&t.text},getLineHandle:function(e){if(Je(this,e))return Ge(this,e)},getLineNumber:function(e){return Qe(e)},getLineHandleVisualStart:function(e){return"number"==typeof e&&(e=Ge(this,e)),zt(e)},lineCount:function(){return this.size},firstLine:function(){return this.first},lastLine:function(){return this.first+this.size-1},clipPos:function(e){return st(this,e)},getCursor:function(e){var t=this.sel.primary();return null==e||"head"==e?t.head:"anchor"==e?t.anchor:"end"==e||"to"==e||!1===e?t.to():t.from()},listSelections:function(){return this.sel.ranges},somethingSelected:function(){return this.sel.somethingSelected()},setCursor:ni((function(e,t,n){Ki(this,st(this,"number"==typeof e?et(e,t||0):e),null,n)})),setSelection:ni((function(e,t,n){Ki(this,st(this,e),st(this,t||e),n)})),extendSelection:ni((function(e,t,n){qi(this,st(this,e),t&&st(this,t),n)})),extendSelections:ni((function(e,t){Zi(this,lt(this,e),t)})),extendSelectionsBy:ni((function(e,t){Zi(this,lt(this,Z(this.sel.ranges,e)),t)})),setSelections:ni((function(e,t,n){if(e.length){for(var r=[],i=0;i<e.length;i++)r[i]=new Ii(st(this,e[i].anchor),st(this,e[i].head));null==t&&(t=Math.min(e.length-1,this.sel.primIndex)),$i(this,xi(this.cm,r,t),n)}})),addSelection:ni((function(e,t,n){var r=this.sel.ranges.slice(0);r.push(new Ii(st(this,e),st(this,t||e))),$i(this,xi(this.cm,r,r.length-1),n)})),getSelection:function(e){for(var t,n=this.sel.ranges,r=0;r<n.length;r++){var i=Xe(this,n[r].from(),n[r].to());t=t?t.concat(i):i}return!1===e?t:t.join(e||this.lineSeparator())},getSelections:function(e){for(var t=[],n=this.sel.ranges,r=0;r<n.length;r++){var i=Xe(this,n[r].from(),n[r].to());!1!==e&&(i=i.join(e||this.lineSeparator())),t[r]=i}return t},replaceSelection:function(e,t,n){for(var r=[],i=0;i<this.sel.ranges.length;i++)r[i]=e;this.replaceSelections(r,t,n||"+input")},replaceSelections:ni((function(e,t,n){for(var r=[],i=this.sel,o=0;o<i.ranges.length;o++){var a=i.ranges[o];r[o]={from:a.from(),to:a.to(),text:this.splitLines(e[o]),origin:n}}for(var s=t&&"end"!=t&&function(e,t,n){for(var r=[],i=et(e.first,0),o=i,a=0;a<t.length;a++){var s=t[a],l=Ei(s.from,i,o),u=Ei(Mi(s),i,o);if(i=s.to,o=u,"around"==n){var c=e.sel.ranges[a],d=tt(c.head,c.anchor)<0;r[a]=new Ii(d?u:l,d?l:u)}else r[a]=new Ii(l,l)}return new _i(r,e.sel.primIndex)}(this,r,t),l=r.length-1;l>=0;l--)uo(this,r[l]);s?Ji(this,s):this.cm&&kr(this.cm)})),undo:ni((function(){ho(this,"undo")})),redo:ni((function(){ho(this,"redo")})),undoSelection:ni((function(){ho(this,"undo",!0)})),redoSelection:ni((function(){ho(this,"redo",!0)})),setExtending:function(e){this.extend=e},getExtending:function(){return this.extend},historySize:function(){for(var e=this.history,t=0,n=0,r=0;r<e.done.length;r++)e.done[r].ranges||++t;for(var i=0;i<e.undone.length;i++)e.undone[i].ranges||++n;return{undo:t,redo:n}},clearHistory:function(){var e=this;this.history=new Ri(this.history.maxGeneration),Pi(this,(function(t){return t.history=e.history}),!0)},markClean:function(){this.cleanGeneration=this.changeGeneration(!0)},changeGeneration:function(e){return e&&(this.history.lastOp=this.history.lastSelOp=this.history.lastOrigin=null),this.history.generation},isClean:function(e){return this.history.generation==(e||this.cleanGeneration)},getHistory:function(){return{done:Gi(this.history.done),undone:Gi(this.history.undone)}},setHistory:function(e){var t=this.history=new Ri(this.history.maxGeneration);t.done=Gi(e.done.slice(0),null,!0),t.undone=Gi(e.undone.slice(0),null,!0)},setGutterMarker:ni((function(e,t,n){return bo(this,e,"gutter",(function(e){var r=e.gutterMarkers||(e.gutterMarkers={});return r[t]=n,!n&&te(r)&&(e.gutterMarkers=null),!0}))})),clearGutter:ni((function(e){var t=this;this.iter((function(n){n.gutterMarkers&&n.gutterMarkers[e]&&bo(t,n,"gutter",(function(){return n.gutterMarkers[e]=null,te(n.gutterMarkers)&&(n.gutterMarkers=null),!0}))}))})),lineInfo:function(e){var t;if("number"==typeof e){if(!Je(this,e))return null;if(t=e,!(e=Ge(this,e)))return null}else if(null==(t=Qe(e)))return null;return{line:t,handle:e,text:e.text,gutterMarkers:e.gutterMarkers,textClass:e.textClass,bgClass:e.bgClass,wrapClass:e.wrapClass,widgets:e.widgets}},addLineClass:ni((function(e,t,n){return bo(this,e,"gutter"==t?"gutter":"class",(function(e){var r="text"==t?"textClass":"background"==t?"bgClass":"gutter"==t?"gutterClass":"wrapClass";if(e[r]){if(x(n).test(e[r]))return!1;e[r]+=" "+n}else e[r]=n;return!0}))})),removeLineClass:ni((function(e,t,n){return bo(this,e,"gutter"==t?"gutter":"class",(function(e){var r="text"==t?"textClass":"background"==t?"bgClass":"gutter"==t?"gutterClass":"wrapClass",i=e[r];if(!i)return!1;if(null==n)e[r]=null;else{var o=i.match(x(n));if(!o)return!1;var a=o.index+o[0].length;e[r]=i.slice(0,o.index)+(o.index&&a!=i.length?" ":"")+i.slice(a)||null}return!0}))})),addLineWidget:ni((function(e,t,n){return function(e,t,n,r){var i=new _o(e,n,r),o=e.cm;return o&&i.noHScroll&&(o.display.alignWidgets=!0),bo(e,t,"widget",(function(t){var n=t.widgets||(t.widgets=[]);if(null==i.insertAt?n.push(i):n.splice(Math.min(n.length-1,Math.max(0,i.insertAt)),0,i),i.line=t,o&&!Ht(e,t)){var r=Vt(t)<e.scrollTop;Ze(t,t.height+An(i)),r&&Or(o,i.height),o.curOp.forceUpdate=!0}return!0})),o&&ln(o,"lineWidgetAdded",o,i,"number"==typeof t?t:Qe(t)),i}(this,e,t,n)})),removeLineWidget:function(e){e.clear()},markText:function(e,t,n){return Mo(this,st(this,e),st(this,t),n,n&&n.type||"range")},setBookmark:function(e,t){var n={replacedWith:t&&(null==t.nodeType?t.widget:t),insertLeft:t&&t.insertLeft,clearWhenEmpty:!1,shared:t&&t.shared,handleMouseEvents:t&&t.handleMouseEvents};return Mo(this,e=st(this,e),e,n,"bookmark")},findMarksAt:function(e){var t=[],n=Ge(this,(e=st(this,e)).line).markedSpans;if(n)for(var r=0;r<n.length;++r){var i=n[r];(null==i.from||i.from<=e.ch)&&(null==i.to||i.to>=e.ch)&&t.push(i.marker.parent||i.marker)}return t},findMarks:function(e,t,n){e=st(this,e),t=st(this,t);var r=[],i=e.line;return this.iter(e.line,t.line+1,(function(o){var a=o.markedSpans;if(a)for(var s=0;s<a.length;s++){var l=a[s];null!=l.to&&i==e.line&&e.ch>=l.to||null==l.from&&i!=e.line||null!=l.from&&i==t.line&&l.from>=t.ch||n&&!n(l.marker)||r.push(l.marker.parent||l.marker)}++i})),r},getAllMarks:function(){var e=[];return this.iter((function(t){var n=t.markedSpans;if(n)for(var r=0;r<n.length;++r)null!=n[r].from&&e.push(n[r].marker)})),e},posFromIndex:function(e){var t,n=this.first,r=this.lineSeparator().length;return this.iter((function(i){var o=i.text.length+r;if(o>e)return t=e,!0;e-=o,++n})),st(this,et(n,t))},indexFromPos:function(e){var t=(e=st(this,e)).ch;if(e.line<this.first||e.ch<0)return 0;var n=this.lineSeparator().length;return this.iter(this.first,e.line,(function(e){t+=e.text.length+n})),t},copy:function(e){var t=new ko(qe(this,this.first,this.first+this.size),this.modeOption,this.first,this.lineSep,this.direction);return t.scrollTop=this.scrollTop,t.scrollLeft=this.scrollLeft,t.sel=this.sel,t.extend=!1,e&&(t.history.undoDepth=this.history.undoDepth,t.setHistory(this.getHistory())),t},linkedDoc:function(e){e||(e={});var t=this.first,n=this.first+this.size;null!=e.from&&e.from>t&&(t=e.from),null!=e.to&&e.to<n&&(n=e.to);var r=new ko(qe(this,t,n),e.mode||this.modeOption,t,this.lineSep,this.direction);return e.sharedHist&&(r.history=this.history),(this.linked||(this.linked=[])).push({doc:r,sharedHist:e.sharedHist}),r.linked=[{doc:this,isParent:!0,sharedHist:e.sharedHist}],function(e,t){for(var n=0;n<t.length;n++){var r=t[n],i=r.find(),o=e.clipPos(i.from),a=e.clipPos(i.to);if(tt(o,a)){var s=Mo(e,o,a,r.primary,r.primary.type);r.markers.push(s),s.parent=r}}}(r,To(this)),r},unlinkDoc:function(e){if(e instanceof Sa&&(e=e.doc),this.linked)for(var t=0;t<this.linked.length;++t)if(this.linked[t].doc==e){this.linked.splice(t,1),e.unlinkDoc(this),Eo(To(this));break}if(e.history==this.history){var n=[e.id];Pi(e,(function(e){return n.push(e.id)}),!0),e.history=new Ri(null),e.history.done=Gi(this.history.done,n),e.history.undone=Gi(this.history.undone,n)}},iterLinkedDocs:function(e){Pi(this,e)},getMode:function(){return this.mode},getEditor:function(){return this.cm},splitLines:function(e){return this.lineSep?e.split(this.lineSep):ke(e)},lineSeparator:function(){return this.lineSep||"\n"},setDirection:ni((function(e){var t;"rtl"!=e&&(e="ltr"),e!=this.direction&&(this.direction=e,this.iter((function(e){return e.order=null})),this.cm&&$r(t=this.cm,(function(){ji(t),dr(t)})))}))}),ko.prototype.eachLine=ko.prototype.iter;var Lo=0;function Do(e){var t=this;if(Po(t),!ge(t,e)&&!_n(t.display,e)){be(e),a&&(Lo=+new Date);var n=ur(t,e,!0),r=e.dataTransfer.files;if(n&&!t.isReadOnly())if(r&&r.length&&window.FileReader&&window.File)for(var i=r.length,o=Array(i),s=0,l=function(){++s==i&&ei(t,(function(){var e={from:n=st(t.doc,n),to:n,text:t.doc.splitLines(o.filter((function(e){return null!=e})).join(t.doc.lineSeparator())),origin:"paste"};uo(t.doc,e),Ji(t.doc,wi(st(t.doc,n),st(t.doc,Mi(e))))}))()},u=function(e,n){if(t.options.allowDropFileTypes&&-1==B(t.options.allowDropFileTypes,e.type))l();else{var r=new FileReader;r.onerror=function(){return l()},r.onload=function(){var e=r.result;/[\x00-\x08\x0e-\x1f]{2}/.test(e)||(o[n]=e),l()},r.readAsText(e)}},c=0;c<r.length;c++)u(r[c],c);else{if(t.state.draggingText&&t.doc.sel.contains(n)>-1)return t.state.draggingText(e),void setTimeout((function(){return t.display.input.focus()}),20);try{var d=e.dataTransfer.getData("Text");if(d){var h;if(t.state.draggingText&&!t.state.draggingText.copy&&(h=t.listSelections()),eo(t.doc,wi(n,n)),h)for(var f=0;f<h.length;++f)go(t.doc,"",h[f].anchor,h[f].head,"drag");t.replaceSelection(d,"around","paste"),t.display.input.focus()}}catch(e){}}}}function Po(e){e.display.dragCursor&&(e.display.lineSpace.removeChild(e.display.dragCursor),e.display.dragCursor=null)}function No(e){if(document.getElementsByClassName){for(var t=document.getElementsByClassName("CodeMirror"),n=[],r=0;r<t.length;r++){var i=t[r].CodeMirror;i&&n.push(i)}n.length&&n[0].operation((function(){for(var t=0;t<n.length;t++)e(n[t])}))}}var jo=!1;function Ro(){var e;jo||(de(window,"resize",(function(){null==e&&(e=setTimeout((function(){e=null,No(zo)}),100))})),de(window,"blur",(function(){return No(wr)})),jo=!0)}function zo(e){var t=e.display;t.cachedCharWidth=t.cachedTextHeight=t.cachedPaddingH=null,t.scrollbarsClipped=!1,e.setSize()}for(var Fo={3:"Pause",8:"Backspace",9:"Tab",13:"Enter",16:"Shift",17:"Ctrl",18:"Alt",19:"Pause",20:"CapsLock",27:"Esc",32:"Space",33:"PageUp",34:"PageDown",35:"End",36:"Home",37:"Left",38:"Up",39:"Right",40:"Down",44:"PrintScrn",45:"Insert",46:"Delete",59:";",61:"=",91:"Mod",92:"Mod",93:"Mod",106:"*",107:"=",109:"-",110:".",111:"/",145:"ScrollLock",173:"-",186:";",187:"=",188:",",189:"-",190:".",191:"/",192:"`",219:"[",220:"\\",221:"]",222:"'",224:"Mod",63232:"Up",63233:"Down",63234:"Left",63235:"Right",63272:"Delete",63273:"Home",63275:"End",63276:"PageUp",63277:"PageDown",63302:"Insert"},Bo=0;Bo<10;Bo++)Fo[Bo+48]=Fo[Bo+96]=String(Bo);for(var Ho=65;Ho<=90;Ho++)Fo[Ho]=String.fromCharCode(Ho);for(var Uo=1;Uo<=12;Uo++)Fo[Uo+111]=Fo[Uo+63235]="F"+Uo;var Vo={};function Yo(e){var t,n,r,i,o=e.split(/-(?!$)/);e=o[o.length-1];for(var a=0;a<o.length-1;a++){var s=o[a];if(/^(cmd|meta|m)$/i.test(s))i=!0;else if(/^a(lt)?$/i.test(s))t=!0;else if(/^(c|ctrl|control)$/i.test(s))n=!0;else{if(!/^s(hift)?$/i.test(s))throw new Error("Unrecognized modifier name: "+s);r=!0}}return t&&(e="Alt-"+e),n&&(e="Ctrl-"+e),i&&(e="Cmd-"+e),r&&(e="Shift-"+e),e}function Wo(e){var t={};for(var n in e)if(e.hasOwnProperty(n)){var r=e[n];if(/^(name|fallthrough|(de|at)tach)$/.test(n))continue;if("..."==r){delete e[n];continue}for(var i=Z(n.split(" "),Yo),o=0;o<i.length;o++){var a=void 0,s=void 0;o==i.length-1?(s=i.join(" "),a=r):(s=i.slice(0,o+1).join(" "),a="...");var l=t[s];if(l){if(l!=a)throw new Error("Inconsistent bindings for "+s)}else t[s]=a}delete e[n]}for(var u in t)e[u]=t[u];return e}function Go(e,t,n,r){var i=(t=Qo(t)).call?t.call(e,r):t[e];if(!1===i)return"nothing";if("..."===i)return"multi";if(null!=i&&n(i))return"handled";if(t.fallthrough){if("[object Array]"!=Object.prototype.toString.call(t.fallthrough))return Go(e,t.fallthrough,n,r);for(var o=0;o<t.fallthrough.length;o++){var a=Go(e,t.fallthrough[o],n,r);if(a)return a}}}function Xo(e){var t="string"==typeof e?e:Fo[e.keyCode];return"Ctrl"==t||"Alt"==t||"Shift"==t||"Mod"==t}function qo(e,t,n){var r=e;return t.altKey&&"Alt"!=r&&(e="Alt-"+e),(_?t.metaKey:t.ctrlKey)&&"Ctrl"!=r&&(e="Ctrl-"+e),(_?t.ctrlKey:t.metaKey)&&"Mod"!=r&&(e="Cmd-"+e),!n&&t.shiftKey&&"Shift"!=r&&(e="Shift-"+e),e}function Zo(e,t){if(d&&34==e.keyCode&&e.char)return!1;var n=Fo[e.keyCode];return null!=n&&!e.altGraphKey&&(3==e.keyCode&&e.code&&(n=e.code),qo(n,e,t))}function Qo(e){return"string"==typeof e?Vo[e]:e}function Ko(e,t){for(var n=e.doc.sel.ranges,r=[],i=0;i<n.length;i++){for(var o=t(n[i]);r.length&&tt(o.from,q(r).to)<=0;){var a=r.pop();if(tt(a.from,o.from)<0){o.from=a.from;break}}r.push(o)}$r(e,(function(){for(var t=r.length-1;t>=0;t--)go(e.doc,"",r[t].from,r[t].to,"+delete");kr(e)}))}function Jo(e,t,n){var r=ie(e.text,t+n,n);return r<0||r>e.text.length?null:r}function $o(e,t,n){var r=Jo(e,t.ch,n);return null==r?null:new et(t.line,r,n<0?"after":"before")}function ea(e,t,n,r,i){if(e){"rtl"==t.doc.direction&&(i=-i);var o=ue(n,t.doc.direction);if(o){var a,s=i<0?q(o):o[0],l=i<0==(1==s.level)?"after":"before";if(s.level>0||"rtl"==t.doc.direction){var u=Ln(t,n);a=i<0?n.text.length-1:0;var c=Dn(t,u,a).top;a=oe((function(e){return Dn(t,u,e).top==c}),i<0==(1==s.level)?s.from:s.to-1,a),"before"==l&&(a=Jo(n,a,1))}else a=i<0?s.to:s.from;return new et(r,a,l)}}return new et(r,i<0?n.text.length:0,i<0?"before":"after")}Vo.basic={Left:"goCharLeft",Right:"goCharRight",Up:"goLineUp",Down:"goLineDown",End:"goLineEnd",Home:"goLineStartSmart",PageUp:"goPageUp",PageDown:"goPageDown",Delete:"delCharAfter",Backspace:"delCharBefore","Shift-Backspace":"delCharBefore",Tab:"defaultTab","Shift-Tab":"indentAuto",Enter:"newlineAndIndent",Insert:"toggleOverwrite",Esc:"singleSelection"},Vo.pcDefault={"Ctrl-A":"selectAll","Ctrl-D":"deleteLine","Ctrl-Z":"undo","Shift-Ctrl-Z":"redo","Ctrl-Y":"redo","Ctrl-Home":"goDocStart","Ctrl-End":"goDocEnd","Ctrl-Up":"goLineUp","Ctrl-Down":"goLineDown","Ctrl-Left":"goGroupLeft","Ctrl-Right":"goGroupRight","Alt-Left":"goLineStart","Alt-Right":"goLineEnd","Ctrl-Backspace":"delGroupBefore","Ctrl-Delete":"delGroupAfter","Ctrl-S":"save","Ctrl-F":"find","Ctrl-G":"findNext","Shift-Ctrl-G":"findPrev","Shift-Ctrl-F":"replace","Shift-Ctrl-R":"replaceAll","Ctrl-[":"indentLess","Ctrl-]":"indentMore","Ctrl-U":"undoSelection","Shift-Ctrl-U":"redoSelection","Alt-U":"redoSelection",fallthrough:"basic"},Vo.emacsy={"Ctrl-F":"goCharRight","Ctrl-B":"goCharLeft","Ctrl-P":"goLineUp","Ctrl-N":"goLineDown","Alt-F":"goWordRight","Alt-B":"goWordLeft","Ctrl-A":"goLineStart","Ctrl-E":"goLineEnd","Ctrl-V":"goPageDown","Shift-Ctrl-V":"goPageUp","Ctrl-D":"delCharAfter","Ctrl-H":"delCharBefore","Alt-D":"delWordAfter","Alt-Backspace":"delWordBefore","Ctrl-K":"killLine","Ctrl-T":"transposeChars","Ctrl-O":"openLine"},Vo.macDefault={"Cmd-A":"selectAll","Cmd-D":"deleteLine","Cmd-Z":"undo","Shift-Cmd-Z":"redo","Cmd-Y":"redo","Cmd-Home":"goDocStart","Cmd-Up":"goDocStart","Cmd-End":"goDocEnd","Cmd-Down":"goDocEnd","Alt-Left":"goGroupLeft","Alt-Right":"goGroupRight","Cmd-Left":"goLineLeft","Cmd-Right":"goLineRight","Alt-Backspace":"delGroupBefore","Ctrl-Alt-Backspace":"delGroupAfter","Alt-Delete":"delGroupAfter","Cmd-S":"save","Cmd-F":"find","Cmd-G":"findNext","Shift-Cmd-G":"findPrev","Cmd-Alt-F":"replace","Shift-Cmd-Alt-F":"replaceAll","Cmd-[":"indentLess","Cmd-]":"indentMore","Cmd-Backspace":"delWrappedLineLeft","Cmd-Delete":"delWrappedLineRight","Cmd-U":"undoSelection","Shift-Cmd-U":"redoSelection","Ctrl-Up":"goDocStart","Ctrl-Down":"goDocEnd",fallthrough:["basic","emacsy"]},Vo.default=y?Vo.macDefault:Vo.pcDefault;var ta={selectAll:so,singleSelection:function(e){return e.setSelection(e.getCursor("anchor"),e.getCursor("head"),U)},killLine:function(e){return Ko(e,(function(t){if(t.empty()){var n=Ge(e.doc,t.head.line).text.length;return t.head.ch==n&&t.head.line<e.lastLine()?{from:t.head,to:et(t.head.line+1,0)}:{from:t.head,to:et(t.head.line,n)}}return{from:t.from(),to:t.to()}}))},deleteLine:function(e){return Ko(e,(function(t){return{from:et(t.from().line,0),to:st(e.doc,et(t.to().line+1,0))}}))},delLineLeft:function(e){return Ko(e,(function(e){return{from:et(e.from().line,0),to:e.from()}}))},delWrappedLineLeft:function(e){return Ko(e,(function(t){var n=e.charCoords(t.head,"div").top+5;return{from:e.coordsChar({left:0,top:n},"div"),to:t.from()}}))},delWrappedLineRight:function(e){return Ko(e,(function(t){var n=e.charCoords(t.head,"div").top+5,r=e.coordsChar({left:e.display.lineDiv.offsetWidth+100,top:n},"div");return{from:t.from(),to:r}}))},undo:function(e){return e.undo()},redo:function(e){return e.redo()},undoSelection:function(e){return e.undoSelection()},redoSelection:function(e){return e.redoSelection()},goDocStart:function(e){return e.extendSelection(et(e.firstLine(),0))},goDocEnd:function(e){return e.extendSelection(et(e.lastLine()))},goLineStart:function(e){return e.extendSelectionsBy((function(t){return na(e,t.head.line)}),{origin:"+move",bias:1})},goLineStartSmart:function(e){return e.extendSelectionsBy((function(t){return ra(e,t.head)}),{origin:"+move",bias:1})},goLineEnd:function(e){return e.extendSelectionsBy((function(t){return function(e,t){var n=Ge(e.doc,t),r=function(e){for(var t;t=Nt(e);)e=t.find(1,!0).line;return e}(n);return r!=n&&(t=Qe(r)),ea(!0,e,n,t,-1)}(e,t.head.line)}),{origin:"+move",bias:-1})},goLineRight:function(e){return e.extendSelectionsBy((function(t){var n=e.cursorCoords(t.head,"div").top+5;return e.coordsChar({left:e.display.lineDiv.offsetWidth+100,top:n},"div")}),Y)},goLineLeft:function(e){return e.extendSelectionsBy((function(t){var n=e.cursorCoords(t.head,"div").top+5;return e.coordsChar({left:0,top:n},"div")}),Y)},goLineLeftSmart:function(e){return e.extendSelectionsBy((function(t){var n=e.cursorCoords(t.head,"div").top+5,r=e.coordsChar({left:0,top:n},"div");return r.ch<e.getLine(r.line).search(/\S/)?ra(e,t.head):r}),Y)},goLineUp:function(e){return e.moveV(-1,"line")},goLineDown:function(e){return e.moveV(1,"line")},goPageUp:function(e){return e.moveV(-1,"page")},goPageDown:function(e){return e.moveV(1,"page")},goCharLeft:function(e){return e.moveH(-1,"char")},goCharRight:function(e){return e.moveH(1,"char")},goColumnLeft:function(e){return e.moveH(-1,"column")},goColumnRight:function(e){return e.moveH(1,"column")},goWordLeft:function(e){return e.moveH(-1,"word")},goGroupRight:function(e){return e.moveH(1,"group")},goGroupLeft:function(e){return e.moveH(-1,"group")},goWordRight:function(e){return e.moveH(1,"word")},delCharBefore:function(e){return e.deleteH(-1,"char")},delCharAfter:function(e){return e.deleteH(1,"char")},delWordBefore:function(e){return e.deleteH(-1,"word")},delWordAfter:function(e){return e.deleteH(1,"word")},delGroupBefore:function(e){return e.deleteH(-1,"group")},delGroupAfter:function(e){return e.deleteH(1,"group")},indentAuto:function(e){return e.indentSelection("smart")},indentMore:function(e){return e.indentSelection("add")},indentLess:function(e){return e.indentSelection("subtract")},insertTab:function(e){return e.replaceSelection("\t")},insertSoftTab:function(e){for(var t=[],n=e.listSelections(),r=e.options.tabSize,i=0;i<n.length;i++){var o=n[i].from(),a=z(e.getLine(o.line),o.ch,r);t.push(X(r-a%r))}e.replaceSelections(t)},defaultTab:function(e){e.somethingSelected()?e.indentSelection("add"):e.execCommand("insertTab")},transposeChars:function(e){return $r(e,(function(){for(var t=e.listSelections(),n=[],r=0;r<t.length;r++)if(t[r].empty()){var i=t[r].head,o=Ge(e.doc,i.line).text;if(o)if(i.ch==o.length&&(i=new et(i.line,i.ch-1)),i.ch>0)i=new et(i.line,i.ch+1),e.replaceRange(o.charAt(i.ch-1)+o.charAt(i.ch-2),et(i.line,i.ch-2),i,"+transpose");else if(i.line>e.doc.first){var a=Ge(e.doc,i.line-1).text;a&&(i=new et(i.line,1),e.replaceRange(o.charAt(0)+e.doc.lineSeparator()+a.charAt(a.length-1),et(i.line-1,a.length-1),i,"+transpose"))}n.push(new Ii(i,i))}e.setSelections(n)}))},newlineAndIndent:function(e){return $r(e,(function(){for(var t=e.listSelections(),n=t.length-1;n>=0;n--)e.replaceRange(e.doc.lineSeparator(),t[n].anchor,t[n].head,"+input");t=e.listSelections();for(var r=0;r<t.length;r++)e.indentLine(t[r].from().line,null,!0);kr(e)}))},openLine:function(e){return e.replaceSelection("\n","start")},toggleOverwrite:function(e){return e.toggleOverwrite()}};function na(e,t){var n=Ge(e.doc,t),r=zt(n);return r!=n&&(t=Qe(r)),ea(!0,e,r,t,1)}function ra(e,t){var n=na(e,t.line),r=Ge(e.doc,n.line),i=ue(r,e.doc.direction);if(!i||0==i[0].level){var o=Math.max(n.ch,r.text.search(/\S/)),a=t.line==n.line&&t.ch<=o&&t.ch;return et(n.line,a?0:o,n.sticky)}return n}function ia(e,t,n){if("string"==typeof t&&!(t=ta[t]))return!1;e.display.input.ensurePolled();var r=e.display.shift,i=!1;try{e.isReadOnly()&&(e.state.suppressEdits=!0),n&&(e.display.shift=!1),i=t(e)!=H}finally{e.display.shift=r,e.state.suppressEdits=!1}return i}var oa=new F;function aa(e,t,n,r){var i=e.state.keySeq;if(i){if(Xo(t))return"handled";if(/\'$/.test(t)?e.state.keySeq=null:oa.set(50,(function(){e.state.keySeq==i&&(e.state.keySeq=null,e.display.input.reset())})),sa(e,i+" "+t,n,r))return!0}return sa(e,t,n,r)}function sa(e,t,n,r){var i=function(e,t,n){for(var r=0;r<e.state.keyMaps.length;r++){var i=Go(t,e.state.keyMaps[r],n,e);if(i)return i}return e.options.extraKeys&&Go(t,e.options.extraKeys,n,e)||Go(t,e.options.keyMap,n,e)}(e,t,r);return"multi"==i&&(e.state.keySeq=t),"handled"==i&&ln(e,"keyHandled",e,t,n),"handled"!=i&&"multi"!=i||(be(n),Ar(e)),!!i}function la(e,t){var n=Zo(t,!0);return!!n&&(t.shiftKey&&!e.state.keySeq?aa(e,"Shift-"+n,t,(function(t){return ia(e,t,!0)}))||aa(e,n,t,(function(t){if("string"==typeof t?/^go[A-Z]/.test(t):t.motion)return ia(e,t)})):aa(e,n,t,(function(t){return ia(e,t)})))}var ua=null;function ca(e){var t=this;if(!(e.target&&e.target!=t.display.input.getField()||(t.curOp.focus=L(),ge(t,e)))){a&&s<11&&27==e.keyCode&&(e.returnValue=!1);var r=e.keyCode;t.display.shift=16==r||e.shiftKey;var i=la(t,e);d&&(ua=i?r:null,i||88!=r||De||!(y?e.metaKey:e.ctrlKey)||t.replaceSelection("",null,"cut")),n&&!y&&!i&&46==r&&e.shiftKey&&!e.ctrlKey&&document.execCommand&&document.execCommand("cut"),18!=r||/\bCodeMirror-crosshair\b/.test(t.display.lineDiv.className)||function(e){var t=e.display.lineDiv;function n(e){18!=e.keyCode&&e.altKey||(M(t,"CodeMirror-crosshair"),fe(document,"keyup",n),fe(document,"mouseover",n))}D(t,"CodeMirror-crosshair"),de(document,"keyup",n),de(document,"mouseover",n)}(t)}}function da(e){16==e.keyCode&&(this.doc.sel.shift=!1),ge(this,e)}function ha(e){var t=this;if(!(e.target&&e.target!=t.display.input.getField()||_n(t.display,e)||ge(t,e)||e.ctrlKey&&!e.altKey||y&&e.metaKey)){var n=e.keyCode,r=e.charCode;if(d&&n==ua)return ua=null,void be(e);if(!d||e.which&&!(e.which<10)||!la(t,e)){var i=String.fromCharCode(null==r?n:r);"\b"!=i&&(function(e,t,n){return aa(e,"'"+n+"'",t,(function(t){return ia(e,t,!0)}))}(t,e,i)||t.display.input.onKeyPress(e))}}}var fa,pa,ga=function(e,t,n){this.time=e,this.pos=t,this.button=n};function va(e){var t=this,n=t.display;if(!(ge(t,e)||n.activeTouch&&n.input.supportsTouch()))if(n.input.ensurePolled(),n.shift=e.shiftKey,_n(n,e))l||(n.scroller.draggable=!1,setTimeout((function(){return n.scroller.draggable=!0}),100));else if(!ba(t,e)){var r=ur(t,e),i=xe(e),o=r?function(e,t){var n=+new Date;return pa&&pa.compare(n,e,t)?(fa=pa=null,"triple"):fa&&fa.compare(n,e,t)?(pa=new ga(n,e,t),fa=null,"double"):(fa=new ga(n,e,t),pa=null,"single")}(r,i):"single";window.focus(),1==i&&t.state.selectingText&&t.state.selectingText(e),r&&function(e,t,n,r,i){var o="Click";return"double"==r?o="Double"+o:"triple"==r&&(o="Triple"+o),aa(e,qo(o=(1==t?"Left":2==t?"Middle":"Right")+o,i),i,(function(t){if("string"==typeof t&&(t=ta[t]),!t)return!1;var r=!1;try{e.isReadOnly()&&(e.state.suppressEdits=!0),r=t(e,n)!=H}finally{e.state.suppressEdits=!1}return r}))}(t,i,r,o,e)||(1==i?r?function(e,t,n,r){a?setTimeout(j(_r,e),0):e.curOp.focus=L();var i,o=function(e,t,n){var r=e.getOption("configureMouse"),i=r?r(e,t,n):{};if(null==i.unit){var o=b?n.shiftKey&&n.metaKey:n.altKey;i.unit=o?"rectangle":"single"==t?"char":"double"==t?"word":"line"}return(null==i.extend||e.doc.extend)&&(i.extend=e.doc.extend||n.shiftKey),null==i.addNew&&(i.addNew=y?n.metaKey:n.ctrlKey),null==i.moveOnDrag&&(i.moveOnDrag=!(y?n.altKey:n.ctrlKey)),i}(e,n,r),u=e.doc.sel;e.options.dragDrop&&Se&&!e.isReadOnly()&&"single"==n&&(i=u.contains(t))>-1&&(tt((i=u.ranges[i]).from(),t)<0||t.xRel>0)&&(tt(i.to(),t)>0||t.xRel<0)?function(e,t,n,r){var i=e.display,o=!1,u=ei(e,(function(t){l&&(i.scroller.draggable=!1),e.state.draggingText=!1,fe(i.wrapper.ownerDocument,"mouseup",u),fe(i.wrapper.ownerDocument,"mousemove",c),fe(i.scroller,"dragstart",d),fe(i.scroller,"drop",u),o||(be(t),r.addNew||qi(e.doc,n,null,null,r.extend),l&&!h||a&&9==s?setTimeout((function(){i.wrapper.ownerDocument.body.focus({preventScroll:!0}),i.input.focus()}),20):i.input.focus())})),c=function(e){o=o||Math.abs(t.clientX-e.clientX)+Math.abs(t.clientY-e.clientY)>=10},d=function(){return o=!0};l&&(i.scroller.draggable=!0),e.state.draggingText=u,u.copy=!r.moveOnDrag,i.scroller.dragDrop&&i.scroller.dragDrop(),de(i.wrapper.ownerDocument,"mouseup",u),de(i.wrapper.ownerDocument,"mousemove",c),de(i.scroller,"dragstart",d),de(i.scroller,"drop",u),Ir(e),setTimeout((function(){return i.input.focus()}),20)}(e,r,t,o):function(e,t,n,r){var i=e.display,o=e.doc;be(t);var a,s,l=o.sel,u=l.ranges;if(r.addNew&&!r.extend?(s=o.sel.contains(n),a=s>-1?u[s]:new Ii(n,n)):(a=o.sel.primary(),s=o.sel.primIndex),"rectangle"==r.unit)r.addNew||(a=new Ii(n,n)),n=ur(e,t,!0,!0),s=-1;else{var c=ma(e,n,r.unit);a=r.extend?Xi(a,c.anchor,c.head,r.extend):c}r.addNew?-1==s?(s=u.length,$i(o,xi(e,u.concat([a]),s),{scroll:!1,origin:"*mouse"})):u.length>1&&u[s].empty()&&"char"==r.unit&&!r.extend?($i(o,xi(e,u.slice(0,s).concat(u.slice(s+1)),0),{scroll:!1,origin:"*mouse"}),l=o.sel):Qi(o,s,a,V):(s=0,$i(o,new _i([a],0),V),l=o.sel);var d=n;function h(t){if(0!=tt(d,t))if(d=t,"rectangle"==r.unit){for(var i=[],u=e.options.tabSize,c=z(Ge(o,n.line).text,n.ch,u),h=z(Ge(o,t.line).text,t.ch,u),f=Math.min(c,h),p=Math.max(c,h),g=Math.min(n.line,t.line),v=Math.min(e.lastLine(),Math.max(n.line,t.line));g<=v;g++){var m=Ge(o,g).text,y=W(m,f,u);f==p?i.push(new Ii(et(g,y),et(g,y))):m.length>y&&i.push(new Ii(et(g,y),et(g,W(m,p,u))))}i.length||i.push(new Ii(n,n)),$i(o,xi(e,l.ranges.slice(0,s).concat(i),s),{origin:"*mouse",scroll:!1}),e.scrollIntoView(t)}else{var b,C=a,A=ma(e,t,r.unit),_=C.anchor;tt(A.anchor,_)>0?(b=A.head,_=ot(C.from(),A.anchor)):(b=A.anchor,_=it(C.to(),A.head));var I=l.ranges.slice(0);I[s]=function(e,t){var n=t.anchor,r=t.head,i=Ge(e.doc,n.line);if(0==tt(n,r)&&n.sticky==r.sticky)return t;var o=ue(i);if(!o)return t;var a=se(o,n.ch,n.sticky),s=o[a];if(s.from!=n.ch&&s.to!=n.ch)return t;var l,u=a+(s.from==n.ch==(1!=s.level)?0:1);if(0==u||u==o.length)return t;if(r.line!=n.line)l=(r.line-n.line)*("ltr"==e.doc.direction?1:-1)>0;else{var c=se(o,r.ch,r.sticky),d=c-a||(r.ch-n.ch)*(1==s.level?-1:1);l=c==u-1||c==u?d<0:d>0}var h=o[u+(l?-1:0)],f=l==(1==h.level),p=f?h.from:h.to,g=f?"after":"before";return n.ch==p&&n.sticky==g?t:new Ii(new et(n.line,p,g),r)}(e,new Ii(st(o,_),b)),$i(o,xi(e,I,s),V)}}var f=i.wrapper.getBoundingClientRect(),p=0;function g(t){e.state.selectingText=!1,p=1/0,t&&(be(t),i.input.focus()),fe(i.wrapper.ownerDocument,"mousemove",v),fe(i.wrapper.ownerDocument,"mouseup",m),o.history.lastSelOrigin=null}var v=ei(e,(function(t){0!==t.buttons&&xe(t)?function t(n){var a=++p,s=ur(e,n,!0,"rectangle"==r.unit);if(s)if(0!=tt(s,d)){e.curOp.focus=L(),h(s);var l=Tr(i,o);(s.line>=l.to||s.line<l.from)&&setTimeout(ei(e,(function(){p==a&&t(n)})),150)}else{var u=n.clientY<f.top?-20:n.clientY>f.bottom?20:0;u&&setTimeout(ei(e,(function(){p==a&&(i.scroller.scrollTop+=u,t(n))})),50)}}(t):g(t)})),m=ei(e,g);e.state.selectingText=m,de(i.wrapper.ownerDocument,"mousemove",v),de(i.wrapper.ownerDocument,"mouseup",m)}(e,r,t,o)}(t,r,o,e):Ie(e)==n.scroller&&be(e):2==i?(r&&qi(t.doc,r),setTimeout((function(){return n.input.focus()}),20)):3==i&&(I?t.display.input.onContextMenu(e):Ir(t)))}}function ma(e,t,n){if("char"==n)return new Ii(t,t);if("word"==n)return e.findWordAt(t);if("line"==n)return new Ii(et(t.line,0),st(e.doc,et(t.line+1,0)));var r=n(e,t);return new Ii(r.from,r.to)}function ya(e,t,n,r){var i,o;if(t.touches)i=t.touches[0].clientX,o=t.touches[0].clientY;else try{i=t.clientX,o=t.clientY}catch(e){return!1}if(i>=Math.floor(e.display.gutters.getBoundingClientRect().right))return!1;r&&be(t);var a=e.display,s=a.lineDiv.getBoundingClientRect();if(o>s.bottom||!me(e,n))return Ae(t);o-=s.top-a.viewOffset;for(var l=0;l<e.display.gutterSpecs.length;++l){var u=a.gutters.childNodes[l];if(u&&u.getBoundingClientRect().right>=i)return pe(e,n,e,Ke(e.doc,o),e.display.gutterSpecs[l].className,t),Ae(t)}}function ba(e,t){return ya(e,t,"gutterClick",!0)}function Ca(e,t){_n(e.display,t)||function(e,t){return!!me(e,"gutterContextMenu")&&ya(e,t,"gutterContextMenu",!1)}(e,t)||ge(e,t,"contextmenu")||I||e.display.input.onContextMenu(t)}function Aa(e){e.display.wrapper.className=e.display.wrapper.className.replace(/\s*cm-s-\S+/g,"")+e.options.theme.replace(/(^|\s)\s*/g," cm-s-"),Bn(e)}ga.prototype.compare=function(e,t,n){return this.time+400>e&&0==tt(t,this.pos)&&n==this.button};var _a={toString:function(){return"CodeMirror.Init"}},Ia={},xa={};function wa(e,t,n){if(!t!=!(n&&n!=_a)){var r=e.display.dragFunctions,i=t?de:fe;i(e.display.scroller,"dragstart",r.start),i(e.display.scroller,"dragenter",r.enter),i(e.display.scroller,"dragover",r.over),i(e.display.scroller,"dragleave",r.leave),i(e.display.scroller,"drop",r.drop)}}function Ma(e){e.options.lineWrapping?(D(e.display.wrapper,"CodeMirror-wrap"),e.display.sizer.style.minWidth="",e.display.sizerWidth=null):(M(e.display.wrapper,"CodeMirror-wrap"),Wt(e)),lr(e),dr(e),Bn(e),setTimeout((function(){return Hr(e)}),100)}function Sa(e,t){var n=this;if(!(this instanceof Sa))return new Sa(e,t);this.options=t=t?R(t):{},R(Ia,t,!1);var r=t.value;"string"==typeof r?r=new ko(r,t.mode,null,t.lineSeparator,t.direction):t.mode&&(r.modeOption=t.mode),this.doc=r;var i=new Sa.inputStyles[t.inputStyle](this),o=this.display=new vi(e,r,i,t);for(var u in o.wrapper.CodeMirror=this,Aa(this),t.lineWrapping&&(this.display.wrapper.className+=" CodeMirror-wrap"),Yr(this),this.state={keyMaps:[],overlays:[],modeGen:0,overwrite:!1,delayingBlurEvent:!1,focused:!1,suppressEdits:!1,pasteIncoming:-1,cutIncoming:-1,selectingText:!1,draggingText:!1,highlight:new F,keySeq:null,specialChars:null},t.autofocus&&!m&&o.input.focus(),a&&s<11&&setTimeout((function(){return n.display.input.reset(!0)}),20),function(e){var t=e.display;de(t.scroller,"mousedown",ei(e,va)),de(t.scroller,"dblclick",a&&s<11?ei(e,(function(t){if(!ge(e,t)){var n=ur(e,t);if(n&&!ba(e,t)&&!_n(e.display,t)){be(t);var r=e.findWordAt(n);qi(e.doc,r.anchor,r.head)}}})):function(t){return ge(e,t)||be(t)}),de(t.scroller,"contextmenu",(function(t){return Ca(e,t)})),de(t.input.getField(),"contextmenu",(function(n){t.scroller.contains(n.target)||Ca(e,n)}));var n,r={end:0};function i(){t.activeTouch&&(n=setTimeout((function(){return t.activeTouch=null}),1e3),(r=t.activeTouch).end=+new Date)}function o(e,t){if(null==t.left)return!0;var n=t.left-e.left,r=t.top-e.top;return n*n+r*r>400}de(t.scroller,"touchstart",(function(i){if(!ge(e,i)&&!function(e){if(1!=e.touches.length)return!1;var t=e.touches[0];return t.radiusX<=1&&t.radiusY<=1}(i)&&!ba(e,i)){t.input.ensurePolled(),clearTimeout(n);var o=+new Date;t.activeTouch={start:o,moved:!1,prev:o-r.end<=300?r:null},1==i.touches.length&&(t.activeTouch.left=i.touches[0].pageX,t.activeTouch.top=i.touches[0].pageY)}})),de(t.scroller,"touchmove",(function(){t.activeTouch&&(t.activeTouch.moved=!0)})),de(t.scroller,"touchend",(function(n){var r=t.activeTouch;if(r&&!_n(t,n)&&null!=r.left&&!r.moved&&new Date-r.start<300){var a,s=e.coordsChar(t.activeTouch,"page");a=!r.prev||o(r,r.prev)?new Ii(s,s):!r.prev.prev||o(r,r.prev.prev)?e.findWordAt(s):new Ii(et(s.line,0),st(e.doc,et(s.line+1,0))),e.setSelection(a.anchor,a.head),e.focus(),be(n)}i()})),de(t.scroller,"touchcancel",i),de(t.scroller,"scroll",(function(){t.scroller.clientHeight&&(Nr(e,t.scroller.scrollTop),Rr(e,t.scroller.scrollLeft,!0),pe(e,"scroll",e))})),de(t.scroller,"mousewheel",(function(t){return Ai(e,t)})),de(t.scroller,"DOMMouseScroll",(function(t){return Ai(e,t)})),de(t.wrapper,"scroll",(function(){return t.wrapper.scrollTop=t.wrapper.scrollLeft=0})),t.dragFunctions={enter:function(t){ge(e,t)||_e(t)},over:function(t){ge(e,t)||(function(e,t){var n=ur(e,t);if(n){var r=document.createDocumentFragment();yr(e,n,r),e.display.dragCursor||(e.display.dragCursor=E("div",null,"CodeMirror-cursors CodeMirror-dragcursors"),e.display.lineSpace.insertBefore(e.display.dragCursor,e.display.cursorDiv)),T(e.display.dragCursor,r)}}(e,t),_e(t))},start:function(t){return function(e,t){if(a&&(!e.state.draggingText||+new Date-Lo<100))_e(t);else if(!ge(e,t)&&!_n(e.display,t)&&(t.dataTransfer.setData("Text",e.getSelection()),t.dataTransfer.effectAllowed="copyMove",t.dataTransfer.setDragImage&&!h)){var n=E("img",null,null,"position: fixed; left: 0; top: 0;");n.src="data:image/gif;base64,R0lGODlhAQABAAAAACH5BAEKAAEALAAAAAABAAEAAAICTAEAOw==",d&&(n.width=n.height=1,e.display.wrapper.appendChild(n),n._top=n.offsetTop),t.dataTransfer.setDragImage(n,0,0),d&&n.parentNode.removeChild(n)}}(e,t)},drop:ei(e,Do),leave:function(t){ge(e,t)||Po(e)}};var l=t.input.getField();de(l,"keyup",(function(t){return da.call(e,t)})),de(l,"keydown",ei(e,ca)),de(l,"keypress",ei(e,ha)),de(l,"focus",(function(t){return xr(e,t)})),de(l,"blur",(function(t){return wr(e,t)}))}(this),Ro(),Gr(this),this.curOp.forceUpdate=!0,Ni(this,r),t.autofocus&&!m||this.hasFocus()?setTimeout(j(xr,this),20):wr(this),xa)xa.hasOwnProperty(u)&&xa[u](this,t[u],_a);hi(this),t.finishInit&&t.finishInit(this);for(var c=0;c<Ta.length;++c)Ta[c](this);Xr(this),l&&t.lineWrapping&&"optimizelegibility"==getComputedStyle(o.lineDiv).textRendering&&(o.lineDiv.style.textRendering="auto")}Sa.defaults=Ia,Sa.optionHandlers=xa;var Ta=[];function Ea(e,t,n,r){var i,o=e.doc;null==n&&(n="add"),"smart"==n&&(o.mode.indent?i=ft(e,t).state:n="prev");var a=e.options.tabSize,s=Ge(o,t),l=z(s.text,null,a);s.stateAfter&&(s.stateAfter=null);var u,c=s.text.match(/^\s*/)[0];if(r||/\S/.test(s.text)){if("smart"==n&&((u=o.mode.indent(i,s.text.slice(c.length),s.text))==H||u>150)){if(!r)return;n="prev"}}else u=0,n="not";"prev"==n?u=t>o.first?z(Ge(o,t-1).text,null,a):0:"add"==n?u=l+e.options.indentUnit:"subtract"==n?u=l-e.options.indentUnit:"number"==typeof n&&(u=l+n),u=Math.max(0,u);var d="",h=0;if(e.options.indentWithTabs)for(var f=Math.floor(u/a);f;--f)h+=a,d+="\t";if(h<u&&(d+=X(u-h)),d!=c)return go(o,d,et(t,0),et(t,c.length),"+input"),s.stateAfter=null,!0;for(var p=0;p<o.sel.ranges.length;p++){var g=o.sel.ranges[p];if(g.head.line==t&&g.head.ch<c.length){var v=et(t,c.length);Qi(o,p,new Ii(v,v));break}}}Sa.defineInitHook=function(e){return Ta.push(e)};var Oa=null;function ka(e){Oa=e}function La(e,t,n,r,i){var o=e.doc;e.display.shift=!1,r||(r=o.sel);var a=+new Date-200,s="paste"==i||e.state.pasteIncoming>a,l=ke(t),u=null;if(s&&r.ranges.length>1)if(Oa&&Oa.text.join("\n")==t){if(r.ranges.length%Oa.text.length==0){u=[];for(var c=0;c<Oa.text.length;c++)u.push(o.splitLines(Oa.text[c]))}}else l.length==r.ranges.length&&e.options.pasteLinesPerSelection&&(u=Z(l,(function(e){return[e]})));for(var d=e.curOp.updateInput,h=r.ranges.length-1;h>=0;h--){var f=r.ranges[h],p=f.from(),g=f.to();f.empty()&&(n&&n>0?p=et(p.line,p.ch-n):e.state.overwrite&&!s?g=et(g.line,Math.min(Ge(o,g.line).text.length,g.ch+q(l).length)):s&&Oa&&Oa.lineWise&&Oa.text.join("\n")==l.join("\n")&&(p=g=et(p.line,0)));var v={from:p,to:g,text:u?u[h%u.length]:l,origin:i||(s?"paste":e.state.cutIncoming>a?"cut":"+input")};uo(e.doc,v),ln(e,"inputRead",e,v)}t&&!s&&Pa(e,t),kr(e),e.curOp.updateInput<2&&(e.curOp.updateInput=d),e.curOp.typing=!0,e.state.pasteIncoming=e.state.cutIncoming=-1}function Da(e,t){var n=e.clipboardData&&e.clipboardData.getData("Text");if(n)return e.preventDefault(),t.isReadOnly()||t.options.disableInput||$r(t,(function(){return La(t,n,0,null,"paste")})),!0}function Pa(e,t){if(e.options.electricChars&&e.options.smartIndent)for(var n=e.doc.sel,r=n.ranges.length-1;r>=0;r--){var i=n.ranges[r];if(!(i.head.ch>100||r&&n.ranges[r-1].head.line==i.head.line)){var o=e.getModeAt(i.head),a=!1;if(o.electricChars){for(var s=0;s<o.electricChars.length;s++)if(t.indexOf(o.electricChars.charAt(s))>-1){a=Ea(e,i.head.line,"smart");break}}else o.electricInput&&o.electricInput.test(Ge(e.doc,i.head.line).text.slice(0,i.head.ch))&&(a=Ea(e,i.head.line,"smart"));a&&ln(e,"electricInput",e,i.head.line)}}}function Na(e){for(var t=[],n=[],r=0;r<e.doc.sel.ranges.length;r++){var i=e.doc.sel.ranges[r].head.line,o={anchor:et(i,0),head:et(i+1,0)};n.push(o),t.push(e.getRange(o.anchor,o.head))}return{text:t,ranges:n}}function ja(e,t,n,r){e.setAttribute("autocorrect",n?"":"off"),e.setAttribute("autocapitalize",r?"":"off"),e.setAttribute("spellcheck",!!t)}function Ra(){var e=E("textarea",null,null,"position: absolute; bottom: -1em; padding: 0; width: 1px; height: 1em; outline: none"),t=E("div",[e],null,"overflow: hidden; position: relative; width: 3px; height: 0px;");return l?e.style.width="1000px":e.setAttribute("wrap","off"),g&&(e.style.border="1px solid black"),ja(e),t}function za(e,t,n,r,i){var o=t,a=n,s=Ge(e,t.line),l=i&&"rtl"==e.direction?-n:n;function u(r){var o,a;if(null==(o=i?function(e,t,n,r){var i=ue(t,e.doc.direction);if(!i)return $o(t,n,r);n.ch>=t.text.length?(n.ch=t.text.length,n.sticky="before"):n.ch<=0&&(n.ch=0,n.sticky="after");var o=se(i,n.ch,n.sticky),a=i[o];if("ltr"==e.doc.direction&&a.level%2==0&&(r>0?a.to>n.ch:a.from<n.ch))return $o(t,n,r);var s,l=function(e,n){return Jo(t,e instanceof et?e.ch:e,n)},u=function(n){return e.options.lineWrapping?(s=s||Ln(e,t),Jn(e,t,s,n)):{begin:0,end:t.text.length}},c=u("before"==n.sticky?l(n,-1):n.ch);if("rtl"==e.doc.direction||1==a.level){var d=1==a.level==r<0,h=l(n,d?1:-1);if(null!=h&&(d?h<=a.to&&h<=c.end:h>=a.from&&h>=c.begin)){var f=d?"before":"after";return new et(n.line,h,f)}}var p=function(e,t,r){for(var o=function(e,t){return t?new et(n.line,l(e,1),"before"):new et(n.line,e,"after")};e>=0&&e<i.length;e+=t){var a=i[e],s=t>0==(1!=a.level),u=s?r.begin:l(r.end,-1);if(a.from<=u&&u<a.to)return o(u,s);if(u=s?a.from:l(a.to,-1),r.begin<=u&&u<r.end)return o(u,s)}},g=p(o+r,r,c);if(g)return g;var v=r>0?c.end:l(c.begin,-1);return null==v||r>0&&v==t.text.length||!(g=p(r>0?0:i.length-1,r,u(v)))?null:g}(e.cm,s,t,n):$o(s,t,n))){if(r||(a=t.line+l)<e.first||a>=e.first+e.size||(t=new et(a,t.ch,t.sticky),!(s=Ge(e,a))))return!1;t=ea(i,e.cm,s,t.line,l)}else t=o;return!0}if("char"==r)u();else if("column"==r)u(!0);else if("word"==r||"group"==r)for(var c=null,d="group"==r,h=e.cm&&e.cm.getHelper(t,"wordChars"),f=!0;!(n<0)||u(!f);f=!1){var p=s.text.charAt(t.ch)||"\n",g=ee(p,h)?"w":d&&"\n"==p?"n":!d||/\s/.test(p)?null:"p";if(!d||f||g||(g="s"),c&&c!=g){n<0&&(n=1,u(),t.sticky="after");break}if(g&&(c=g),n>0&&!u(!f))break}var v=oo(e,t,o,a,!0);return nt(o,v)&&(v.hitSide=!0),v}function Fa(e,t,n,r){var i,o,a=e.doc,s=t.left;if("page"==r){var l=Math.min(e.display.wrapper.clientHeight,window.innerHeight||document.documentElement.clientHeight),u=Math.max(l-.5*rr(e.display),3);i=(n>0?t.bottom:t.top)+n*u}else"line"==r&&(i=n>0?t.bottom+3:t.top-3);for(;(o=Qn(e,s,i)).outside;){if(n<0?i<=0:i>=a.height){o.hitSide=!0;break}i+=5*n}return o}var Ba=function(e){this.cm=e,this.lastAnchorNode=this.lastAnchorOffset=this.lastFocusNode=this.lastFocusOffset=null,this.polling=new F,this.composing=null,this.gracePeriod=!1,this.readDOMTimeout=null};function Ha(e,t){var n=kn(e,t.line);if(!n||n.hidden)return null;var r=Ge(e.doc,t.line),i=En(n,r,t.line),o=ue(r,e.doc.direction),a="left";o&&(a=se(o,t.ch)%2?"right":"left");var s=jn(i.map,t.ch,a);return s.offset="right"==s.collapse?s.end:s.start,s}function Ua(e,t){return t&&(e.bad=!0),e}function Va(e,t,n){var r;if(t==e.display.lineDiv){if(!(r=e.display.lineDiv.childNodes[n]))return Ua(e.clipPos(et(e.display.viewTo-1)),!0);t=null,n=0}else for(r=t;;r=r.parentNode){if(!r||r==e.display.lineDiv)return null;if(r.parentNode&&r.parentNode==e.display.lineDiv)break}for(var i=0;i<e.display.view.length;i++){var o=e.display.view[i];if(o.node==r)return Ya(o,t,n)}}function Ya(e,t,n){var r=e.text.firstChild,i=!1;if(!t||!k(r,t))return Ua(et(Qe(e.line),0),!0);if(t==r&&(i=!0,t=r.childNodes[n],n=0,!t)){var o=e.rest?q(e.rest):e.line;return Ua(et(Qe(o),o.text.length),i)}var a=3==t.nodeType?t:null,s=t;for(a||1!=t.childNodes.length||3!=t.firstChild.nodeType||(a=t.firstChild,n&&(n=a.nodeValue.length));s.parentNode!=r;)s=s.parentNode;var l=e.measure,u=l.maps;function c(t,n,r){for(var i=-1;i<(u?u.length:0);i++)for(var o=i<0?l.map:u[i],a=0;a<o.length;a+=3){var s=o[a+2];if(s==t||s==n){var c=Qe(i<0?e.line:e.rest[i]),d=o[a]+r;return(r<0||s!=t)&&(d=o[a+(r?1:0)]),et(c,d)}}}var d=c(a,s,n);if(d)return Ua(d,i);for(var h=s.nextSibling,f=a?a.nodeValue.length-n:0;h;h=h.nextSibling){if(d=c(h,h.firstChild,0))return Ua(et(d.line,d.ch-f),i);f+=h.textContent.length}for(var p=s.previousSibling,g=n;p;p=p.previousSibling){if(d=c(p,p.firstChild,-1))return Ua(et(d.line,d.ch+g),i);g+=p.textContent.length}}Ba.prototype.init=function(e){var t=this,n=this,r=n.cm,i=n.div=e.lineDiv;function o(e){for(var t=e.target;t;t=t.parentNode){if(t==i)return!0;if(/\bCodeMirror-(?:line)?widget\b/.test(t.className))break}return!1}function a(e){if(o(e)&&!ge(r,e)){if(r.somethingSelected())ka({lineWise:!1,text:r.getSelections()}),"cut"==e.type&&r.replaceSelection("",null,"cut");else{if(!r.options.lineWiseCopyCut)return;var t=Na(r);ka({lineWise:!0,text:t.text}),"cut"==e.type&&r.operation((function(){r.setSelections(t.ranges,0,U),r.replaceSelection("",null,"cut")}))}if(e.clipboardData){e.clipboardData.clearData();var a=Oa.text.join("\n");if(e.clipboardData.setData("Text",a),e.clipboardData.getData("Text")==a)return void e.preventDefault()}var s=Ra(),l=s.firstChild;r.display.lineSpace.insertBefore(s,r.display.lineSpace.firstChild),l.value=Oa.text.join("\n");var u=document.activeElement;N(l),setTimeout((function(){r.display.lineSpace.removeChild(s),u.focus(),u==i&&n.showPrimarySelection()}),50)}}ja(i,r.options.spellcheck,r.options.autocorrect,r.options.autocapitalize),de(i,"paste",(function(e){!o(e)||ge(r,e)||Da(e,r)||s<=11&&setTimeout(ei(r,(function(){return t.updateFromDOM()})),20)})),de(i,"compositionstart",(function(e){t.composing={data:e.data,done:!1}})),de(i,"compositionupdate",(function(e){t.composing||(t.composing={data:e.data,done:!1})})),de(i,"compositionend",(function(e){t.composing&&(e.data!=t.composing.data&&t.readFromDOMSoon(),t.composing.done=!0)})),de(i,"touchstart",(function(){return n.forceCompositionEnd()})),de(i,"input",(function(){t.composing||t.readFromDOMSoon()})),de(i,"copy",a),de(i,"cut",a)},Ba.prototype.screenReaderLabelChanged=function(e){e?this.div.setAttribute("aria-label",e):this.div.removeAttribute("aria-label")},Ba.prototype.prepareSelection=function(){var e=mr(this.cm,!1);return e.focus=document.activeElement==this.div,e},Ba.prototype.showSelection=function(e,t){e&&this.cm.display.view.length&&((e.focus||t)&&this.showPrimarySelection(),this.showMultipleSelections(e))},Ba.prototype.getSelection=function(){return this.cm.display.wrapper.ownerDocument.getSelection()},Ba.prototype.showPrimarySelection=function(){var e=this.getSelection(),t=this.cm,r=t.doc.sel.primary(),i=r.from(),o=r.to();if(t.display.viewTo==t.display.viewFrom||i.line>=t.display.viewTo||o.line<t.display.viewFrom)e.removeAllRanges();else{var a=Va(t,e.anchorNode,e.anchorOffset),s=Va(t,e.focusNode,e.focusOffset);if(!a||a.bad||!s||s.bad||0!=tt(ot(a,s),i)||0!=tt(it(a,s),o)){var l=t.display.view,u=i.line>=t.display.viewFrom&&Ha(t,i)||{node:l[0].measure.map[2],offset:0},c=o.line<t.display.viewTo&&Ha(t,o);if(!c){var d=l[l.length-1].measure,h=d.maps?d.maps[d.maps.length-1]:d.map;c={node:h[h.length-1],offset:h[h.length-2]-h[h.length-3]}}if(u&&c){var f,p=e.rangeCount&&e.getRangeAt(0);try{f=w(u.node,u.offset,c.offset,c.node)}catch(e){}f&&(!n&&t.state.focused?(e.collapse(u.node,u.offset),f.collapsed||(e.removeAllRanges(),e.addRange(f))):(e.removeAllRanges(),e.addRange(f)),p&&null==e.anchorNode?e.addRange(p):n&&this.startGracePeriod()),this.rememberSelection()}else e.removeAllRanges()}}},Ba.prototype.startGracePeriod=function(){var e=this;clearTimeout(this.gracePeriod),this.gracePeriod=setTimeout((function(){e.gracePeriod=!1,e.selectionChanged()&&e.cm.operation((function(){return e.cm.curOp.selectionChanged=!0}))}),20)},Ba.prototype.showMultipleSelections=function(e){T(this.cm.display.cursorDiv,e.cursors),T(this.cm.display.selectionDiv,e.selection)},Ba.prototype.rememberSelection=function(){var e=this.getSelection();this.lastAnchorNode=e.anchorNode,this.lastAnchorOffset=e.anchorOffset,this.lastFocusNode=e.focusNode,this.lastFocusOffset=e.focusOffset},Ba.prototype.selectionInEditor=function(){var e=this.getSelection();if(!e.rangeCount)return!1;var t=e.getRangeAt(0).commonAncestorContainer;return k(this.div,t)},Ba.prototype.focus=function(){"nocursor"!=this.cm.options.readOnly&&(this.selectionInEditor()&&document.activeElement==this.div||this.showSelection(this.prepareSelection(),!0),this.div.focus())},Ba.prototype.blur=function(){this.div.blur()},Ba.prototype.getField=function(){return this.div},Ba.prototype.supportsTouch=function(){return!0},Ba.prototype.receivedFocus=function(){var e=this;this.selectionInEditor()?this.pollSelection():$r(this.cm,(function(){return e.cm.curOp.selectionChanged=!0})),this.polling.set(this.cm.options.pollInterval,(function t(){e.cm.state.focused&&(e.pollSelection(),e.polling.set(e.cm.options.pollInterval,t))}))},Ba.prototype.selectionChanged=function(){var e=this.getSelection();return e.anchorNode!=this.lastAnchorNode||e.anchorOffset!=this.lastAnchorOffset||e.focusNode!=this.lastFocusNode||e.focusOffset!=this.lastFocusOffset},Ba.prototype.pollSelection=function(){if(null==this.readDOMTimeout&&!this.gracePeriod&&this.selectionChanged()){var e=this.getSelection(),t=this.cm;if(v&&c&&this.cm.display.gutterSpecs.length&&function(e){for(var t=e;t;t=t.parentNode)if(/CodeMirror-gutter-wrapper/.test(t.className))return!0;return!1}(e.anchorNode))return this.cm.triggerOnKeyDown({type:"keydown",keyCode:8,preventDefault:Math.abs}),this.blur(),void this.focus();if(!this.composing){this.rememberSelection();var n=Va(t,e.anchorNode,e.anchorOffset),r=Va(t,e.focusNode,e.focusOffset);n&&r&&$r(t,(function(){$i(t.doc,wi(n,r),U),(n.bad||r.bad)&&(t.curOp.selectionChanged=!0)}))}}},Ba.prototype.pollContent=function(){null!=this.readDOMTimeout&&(clearTimeout(this.readDOMTimeout),this.readDOMTimeout=null);var e,t,n,r=this.cm,i=r.display,o=r.doc.sel.primary(),a=o.from(),s=o.to();if(0==a.ch&&a.line>r.firstLine()&&(a=et(a.line-1,Ge(r.doc,a.line-1).length)),s.ch==Ge(r.doc,s.line).text.length&&s.line<r.lastLine()&&(s=et(s.line+1,0)),a.line<i.viewFrom||s.line>i.viewTo-1)return!1;a.line==i.viewFrom||0==(e=cr(r,a.line))?(t=Qe(i.view[0].line),n=i.view[0].node):(t=Qe(i.view[e].line),n=i.view[e-1].node.nextSibling);var l,u,c=cr(r,s.line);if(c==i.view.length-1?(l=i.viewTo-1,u=i.lineDiv.lastChild):(l=Qe(i.view[c+1].line)-1,u=i.view[c+1].node.previousSibling),!n)return!1;for(var d=r.doc.splitLines(function(e,t,n,r,i){var o="",a=!1,s=e.doc.lineSeparator(),l=!1;function u(){a&&(o+=s,l&&(o+=s),a=l=!1)}function c(e){e&&(u(),o+=e)}function d(t){if(1==t.nodeType){var n=t.getAttribute("cm-text");if(n)return void c(n);var o,h=t.getAttribute("cm-marker");if(h){var f=e.findMarks(et(r,0),et(i+1,0),(v=+h,function(e){return e.id==v}));return void(f.length&&(o=f[0].find(0))&&c(Xe(e.doc,o.from,o.to).join(s)))}if("false"==t.getAttribute("contenteditable"))return;var p=/^(pre|div|p|li|table|br)$/i.test(t.nodeName);if(!/^br$/i.test(t.nodeName)&&0==t.textContent.length)return;p&&u();for(var g=0;g<t.childNodes.length;g++)d(t.childNodes[g]);/^(pre|p)$/i.test(t.nodeName)&&(l=!0),p&&(a=!0)}else 3==t.nodeType&&c(t.nodeValue.replace(/\u200b/g,"").replace(/\u00a0/g," "));var v}for(;d(t),t!=n;)t=t.nextSibling,l=!1;return o}(r,n,u,t,l)),h=Xe(r.doc,et(t,0),et(l,Ge(r.doc,l).text.length));d.length>1&&h.length>1;)if(q(d)==q(h))d.pop(),h.pop(),l--;else{if(d[0]!=h[0])break;d.shift(),h.shift(),t++}for(var f=0,p=0,g=d[0],v=h[0],m=Math.min(g.length,v.length);f<m&&g.charCodeAt(f)==v.charCodeAt(f);)++f;for(var y=q(d),b=q(h),C=Math.min(y.length-(1==d.length?f:0),b.length-(1==h.length?f:0));p<C&&y.charCodeAt(y.length-p-1)==b.charCodeAt(b.length-p-1);)++p;if(1==d.length&&1==h.length&&t==a.line)for(;f&&f>a.ch&&y.charCodeAt(y.length-p-1)==b.charCodeAt(b.length-p-1);)f--,p++;d[d.length-1]=y.slice(0,y.length-p).replace(/^\u200b+/,""),d[0]=d[0].slice(f).replace(/\u200b+$/,"");var A=et(t,f),_=et(l,h.length?q(h).length-p:0);return d.length>1||d[0]||tt(A,_)?(go(r.doc,d,A,_,"+input"),!0):void 0},Ba.prototype.ensurePolled=function(){this.forceCompositionEnd()},Ba.prototype.reset=function(){this.forceCompositionEnd()},Ba.prototype.forceCompositionEnd=function(){this.composing&&(clearTimeout(this.readDOMTimeout),this.composing=null,this.updateFromDOM(),this.div.blur(),this.div.focus())},Ba.prototype.readFromDOMSoon=function(){var e=this;null==this.readDOMTimeout&&(this.readDOMTimeout=setTimeout((function(){if(e.readDOMTimeout=null,e.composing){if(!e.composing.done)return;e.composing=null}e.updateFromDOM()}),80))},Ba.prototype.updateFromDOM=function(){var e=this;!this.cm.isReadOnly()&&this.pollContent()||$r(this.cm,(function(){return dr(e.cm)}))},Ba.prototype.setUneditable=function(e){e.contentEditable="false"},Ba.prototype.onKeyPress=function(e){0==e.charCode||this.composing||(e.preventDefault(),this.cm.isReadOnly()||ei(this.cm,La)(this.cm,String.fromCharCode(null==e.charCode?e.keyCode:e.charCode),0))},Ba.prototype.readOnlyChanged=function(e){this.div.contentEditable=String("nocursor"!=e)},Ba.prototype.onContextMenu=function(){},Ba.prototype.resetPosition=function(){},Ba.prototype.needsContentAttribute=!0;var Wa=function(e){this.cm=e,this.prevInput="",this.pollingFast=!1,this.polling=new F,this.hasSelection=!1,this.composing=null};Wa.prototype.init=function(e){var t=this,n=this,r=this.cm;this.createField(e);var i=this.textarea;function o(e){if(!ge(r,e)){if(r.somethingSelected())ka({lineWise:!1,text:r.getSelections()});else{if(!r.options.lineWiseCopyCut)return;var t=Na(r);ka({lineWise:!0,text:t.text}),"cut"==e.type?r.setSelections(t.ranges,null,U):(n.prevInput="",i.value=t.text.join("\n"),N(i))}"cut"==e.type&&(r.state.cutIncoming=+new Date)}}e.wrapper.insertBefore(this.wrapper,e.wrapper.firstChild),g&&(i.style.width="0px"),de(i,"input",(function(){a&&s>=9&&t.hasSelection&&(t.hasSelection=null),n.poll()})),de(i,"paste",(function(e){ge(r,e)||Da(e,r)||(r.state.pasteIncoming=+new Date,n.fastPoll())})),de(i,"cut",o),de(i,"copy",o),de(e.scroller,"paste",(function(t){if(!_n(e,t)&&!ge(r,t)){if(!i.dispatchEvent)return r.state.pasteIncoming=+new Date,void n.focus();var o=new Event("paste");o.clipboardData=t.clipboardData,i.dispatchEvent(o)}})),de(e.lineSpace,"selectstart",(function(t){_n(e,t)||be(t)})),de(i,"compositionstart",(function(){var e=r.getCursor("from");n.composing&&n.composing.range.clear(),n.composing={start:e,range:r.markText(e,r.getCursor("to"),{className:"CodeMirror-composing"})}})),de(i,"compositionend",(function(){n.composing&&(n.poll(),n.composing.range.clear(),n.composing=null)}))},Wa.prototype.createField=function(e){this.wrapper=Ra(),this.textarea=this.wrapper.firstChild},Wa.prototype.screenReaderLabelChanged=function(e){e?this.textarea.setAttribute("aria-label",e):this.textarea.removeAttribute("aria-label")},Wa.prototype.prepareSelection=function(){var e=this.cm,t=e.display,n=e.doc,r=mr(e);if(e.options.moveInputWithCursor){var i=Xn(e,n.sel.primary().head,"div"),o=t.wrapper.getBoundingClientRect(),a=t.lineDiv.getBoundingClientRect();r.teTop=Math.max(0,Math.min(t.wrapper.clientHeight-10,i.top+a.top-o.top)),r.teLeft=Math.max(0,Math.min(t.wrapper.clientWidth-10,i.left+a.left-o.left))}return r},Wa.prototype.showSelection=function(e){var t=this.cm.display;T(t.cursorDiv,e.cursors),T(t.selectionDiv,e.selection),null!=e.teTop&&(this.wrapper.style.top=e.teTop+"px",this.wrapper.style.left=e.teLeft+"px")},Wa.prototype.reset=function(e){if(!this.contextMenuPending&&!this.composing){var t=this.cm;if(t.somethingSelected()){this.prevInput="";var n=t.getSelection();this.textarea.value=n,t.state.focused&&N(this.textarea),a&&s>=9&&(this.hasSelection=n)}else e||(this.prevInput=this.textarea.value="",a&&s>=9&&(this.hasSelection=null))}},Wa.prototype.getField=function(){return this.textarea},Wa.prototype.supportsTouch=function(){return!1},Wa.prototype.focus=function(){if("nocursor"!=this.cm.options.readOnly&&(!m||L()!=this.textarea))try{this.textarea.focus()}catch(e){}},Wa.prototype.blur=function(){this.textarea.blur()},Wa.prototype.resetPosition=function(){this.wrapper.style.top=this.wrapper.style.left=0},Wa.prototype.receivedFocus=function(){this.slowPoll()},Wa.prototype.slowPoll=function(){var e=this;this.pollingFast||this.polling.set(this.cm.options.pollInterval,(function(){e.poll(),e.cm.state.focused&&e.slowPoll()}))},Wa.prototype.fastPoll=function(){var e=!1,t=this;t.pollingFast=!0,t.polling.set(20,(function n(){t.poll()||e?(t.pollingFast=!1,t.slowPoll()):(e=!0,t.polling.set(60,n))}))},Wa.prototype.poll=function(){var e=this,t=this.cm,n=this.textarea,r=this.prevInput;if(this.contextMenuPending||!t.state.focused||Le(n)&&!r&&!this.composing||t.isReadOnly()||t.options.disableInput||t.state.keySeq)return!1;var i=n.value;if(i==r&&!t.somethingSelected())return!1;if(a&&s>=9&&this.hasSelection===i||y&&/[\uf700-\uf7ff]/.test(i))return t.display.input.reset(),!1;if(t.doc.sel==t.display.selForContextMenu){var o=i.charCodeAt(0);if(8203!=o||r||(r="​"),8666==o)return this.reset(),this.cm.execCommand("undo")}for(var l=0,u=Math.min(r.length,i.length);l<u&&r.charCodeAt(l)==i.charCodeAt(l);)++l;return $r(t,(function(){La(t,i.slice(l),r.length-l,null,e.composing?"*compose":null),i.length>1e3||i.indexOf("\n")>-1?n.value=e.prevInput="":e.prevInput=i,e.composing&&(e.composing.range.clear(),e.composing.range=t.markText(e.composing.start,t.getCursor("to"),{className:"CodeMirror-composing"}))})),!0},Wa.prototype.ensurePolled=function(){this.pollingFast&&this.poll()&&(this.pollingFast=!1)},Wa.prototype.onKeyPress=function(){a&&s>=9&&(this.hasSelection=null),this.fastPoll()},Wa.prototype.onContextMenu=function(e){var t=this,n=t.cm,r=n.display,i=t.textarea;t.contextMenuPending&&t.contextMenuPending();var o=ur(n,e),u=r.scroller.scrollTop;if(o&&!d){n.options.resetSelectionOnContextMenu&&-1==n.doc.sel.contains(o)&&ei(n,$i)(n.doc,wi(o),U);var c,h=i.style.cssText,f=t.wrapper.style.cssText,p=t.wrapper.offsetParent.getBoundingClientRect();if(t.wrapper.style.cssText="position: static",i.style.cssText="position: absolute; width: 30px; height: 30px;\n      top: "+(e.clientY-p.top-5)+"px; left: "+(e.clientX-p.left-5)+"px;\n      z-index: 1000; background: "+(a?"rgba(255, 255, 255, .05)":"transparent")+";\n      outline: none; border-width: 0; outline: none; overflow: hidden; opacity: .05; filter: alpha(opacity=5);",l&&(c=window.scrollY),r.input.focus(),l&&window.scrollTo(null,c),r.input.reset(),n.somethingSelected()||(i.value=t.prevInput=" "),t.contextMenuPending=m,r.selForContextMenu=n.doc.sel,clearTimeout(r.detectingSelectAll),a&&s>=9&&v(),I){_e(e);var g=function(){fe(window,"mouseup",g),setTimeout(m,20)};de(window,"mouseup",g)}else setTimeout(m,50)}function v(){if(null!=i.selectionStart){var e=n.somethingSelected(),o="​"+(e?i.value:"");i.value="⇚",i.value=o,t.prevInput=e?"":"​",i.selectionStart=1,i.selectionEnd=o.length,r.selForContextMenu=n.doc.sel}}function m(){if(t.contextMenuPending==m&&(t.contextMenuPending=!1,t.wrapper.style.cssText=f,i.style.cssText=h,a&&s<9&&r.scrollbars.setScrollTop(r.scroller.scrollTop=u),null!=i.selectionStart)){(!a||a&&s<9)&&v();var e=0,o=function(){r.selForContextMenu==n.doc.sel&&0==i.selectionStart&&i.selectionEnd>0&&"​"==t.prevInput?ei(n,so)(n):e++<10?r.detectingSelectAll=setTimeout(o,500):(r.selForContextMenu=null,r.input.reset())};r.detectingSelectAll=setTimeout(o,200)}}},Wa.prototype.readOnlyChanged=function(e){e||this.reset(),this.textarea.disabled="nocursor"==e},Wa.prototype.setUneditable=function(){},Wa.prototype.needsContentAttribute=!1,function(e){var t=e.optionHandlers;function n(n,r,i,o){e.defaults[n]=r,i&&(t[n]=o?function(e,t,n){n!=_a&&i(e,t,n)}:i)}e.defineOption=n,e.Init=_a,n("value","",(function(e,t){return e.setValue(t)}),!0),n("mode",null,(function(e,t){e.doc.modeOption=t,Oi(e)}),!0),n("indentUnit",2,Oi,!0),n("indentWithTabs",!1),n("smartIndent",!0),n("tabSize",4,(function(e){ki(e),Bn(e),dr(e)}),!0),n("lineSeparator",null,(function(e,t){if(e.doc.lineSep=t,t){var n=[],r=e.doc.first;e.doc.iter((function(e){for(var i=0;;){var o=e.text.indexOf(t,i);if(-1==o)break;i=o+t.length,n.push(et(r,o))}r++}));for(var i=n.length-1;i>=0;i--)go(e.doc,t,n[i],et(n[i].line,n[i].ch+t.length))}})),n("specialChars",/[\u0000-\u001f\u007f-\u009f\u00ad\u061c\u200b-\u200c\u200e\u200f\u2028\u2029\ufeff\ufff9-\ufffc]/g,(function(e,t,n){e.state.specialChars=new RegExp(t.source+(t.test("\t")?"":"|\t"),"g"),n!=_a&&e.refresh()})),n("specialCharPlaceholder",Jt,(function(e){return e.refresh()}),!0),n("electricChars",!0),n("inputStyle",m?"contenteditable":"textarea",(function(){throw new Error("inputStyle can not (yet) be changed in a running editor")}),!0),n("spellcheck",!1,(function(e,t){return e.getInputField().spellcheck=t}),!0),n("autocorrect",!1,(function(e,t){return e.getInputField().autocorrect=t}),!0),n("autocapitalize",!1,(function(e,t){return e.getInputField().autocapitalize=t}),!0),n("rtlMoveVisually",!C),n("wholeLineUpdateBefore",!0),n("theme","default",(function(e){Aa(e),gi(e)}),!0),n("keyMap","default",(function(e,t,n){var r=Qo(t),i=n!=_a&&Qo(n);i&&i.detach&&i.detach(e,r),r.attach&&r.attach(e,i||null)})),n("extraKeys",null),n("configureMouse",null),n("lineWrapping",!1,Ma,!0),n("gutters",[],(function(e,t){e.display.gutterSpecs=fi(t,e.options.lineNumbers),gi(e)}),!0),n("fixedGutter",!0,(function(e,t){e.display.gutters.style.left=t?ar(e.display)+"px":"0",e.refresh()}),!0),n("coverGutterNextToScrollbar",!1,(function(e){return Hr(e)}),!0),n("scrollbarStyle","native",(function(e){Yr(e),Hr(e),e.display.scrollbars.setScrollTop(e.doc.scrollTop),e.display.scrollbars.setScrollLeft(e.doc.scrollLeft)}),!0),n("lineNumbers",!1,(function(e,t){e.display.gutterSpecs=fi(e.options.gutters,t),gi(e)}),!0),n("firstLineNumber",1,gi,!0),n("lineNumberFormatter",(function(e){return e}),gi,!0),n("showCursorWhenSelecting",!1,vr,!0),n("resetSelectionOnContextMenu",!0),n("lineWiseCopyCut",!0),n("pasteLinesPerSelection",!0),n("selectionsMayTouch",!1),n("readOnly",!1,(function(e,t){"nocursor"==t&&(wr(e),e.display.input.blur()),e.display.input.readOnlyChanged(t)})),n("screenReaderLabel",null,(function(e,t){t=""===t?null:t,e.display.input.screenReaderLabelChanged(t)})),n("disableInput",!1,(function(e,t){t||e.display.input.reset()}),!0),n("dragDrop",!0,wa),n("allowDropFileTypes",null),n("cursorBlinkRate",530),n("cursorScrollMargin",0),n("cursorHeight",1,vr,!0),n("singleCursorHeightPerLine",!0,vr,!0),n("workTime",100),n("workDelay",100),n("flattenSpans",!0,ki,!0),n("addModeClass",!1,ki,!0),n("pollInterval",100),n("undoDepth",200,(function(e,t){return e.doc.history.undoDepth=t})),n("historyEventDelay",1250),n("viewportMargin",10,(function(e){return e.refresh()}),!0),n("maxHighlightLength",1e4,ki,!0),n("moveInputWithCursor",!0,(function(e,t){t||e.display.input.resetPosition()})),n("tabindex",null,(function(e,t){return e.display.input.getField().tabIndex=t||""})),n("autofocus",null),n("direction","ltr",(function(e,t){return e.doc.setDirection(t)}),!0),n("phrases",null)}(Sa),function(e){var t=e.optionHandlers,n=e.helpers={};e.prototype={constructor:e,focus:function(){window.focus(),this.display.input.focus()},setOption:function(e,n){var r=this.options,i=r[e];r[e]==n&&"mode"!=e||(r[e]=n,t.hasOwnProperty(e)&&ei(this,t[e])(this,n,i),pe(this,"optionChange",this,e))},getOption:function(e){return this.options[e]},getDoc:function(){return this.doc},addKeyMap:function(e,t){this.state.keyMaps[t?"push":"unshift"](Qo(e))},removeKeyMap:function(e){for(var t=this.state.keyMaps,n=0;n<t.length;++n)if(t[n]==e||t[n].name==e)return t.splice(n,1),!0},addOverlay:ti((function(t,n){var r=t.token?t:e.getMode(this.options,t);if(r.startState)throw new Error("Overlays may not be stateful.");!function(e,t,n){for(var r=0,i=n(t);r<e.length&&n(e[r])<=i;)r++;e.splice(r,0,t)}(this.state.overlays,{mode:r,modeSpec:t,opaque:n&&n.opaque,priority:n&&n.priority||0},(function(e){return e.priority})),this.state.modeGen++,dr(this)})),removeOverlay:ti((function(e){for(var t=this.state.overlays,n=0;n<t.length;++n){var r=t[n].modeSpec;if(r==e||"string"==typeof e&&r.name==e)return t.splice(n,1),this.state.modeGen++,void dr(this)}})),indentLine:ti((function(e,t,n){"string"!=typeof t&&"number"!=typeof t&&(t=null==t?this.options.smartIndent?"smart":"prev":t?"add":"subtract"),Je(this.doc,e)&&Ea(this,e,t,n)})),indentSelection:ti((function(e){for(var t=this.doc.sel.ranges,n=-1,r=0;r<t.length;r++){var i=t[r];if(i.empty())i.head.line>n&&(Ea(this,i.head.line,e,!0),n=i.head.line,r==this.doc.sel.primIndex&&kr(this));else{var o=i.from(),a=i.to(),s=Math.max(n,o.line);n=Math.min(this.lastLine(),a.line-(a.ch?0:1))+1;for(var l=s;l<n;++l)Ea(this,l,e);var u=this.doc.sel.ranges;0==o.ch&&t.length==u.length&&u[r].from().ch>0&&Qi(this.doc,r,new Ii(o,u[r].to()),U)}}})),getTokenAt:function(e,t){return yt(this,e,t)},getLineTokens:function(e,t){return yt(this,et(e),t,!0)},getTokenTypeAt:function(e){e=st(this.doc,e);var t,n=ht(this,Ge(this.doc,e.line)),r=0,i=(n.length-1)/2,o=e.ch;if(0==o)t=n[2];else for(;;){var a=r+i>>1;if((a?n[2*a-1]:0)>=o)i=a;else{if(!(n[2*a+1]<o)){t=n[2*a+2];break}r=a+1}}var s=t?t.indexOf("overlay "):-1;return s<0?t:0==s?null:t.slice(0,s-1)},getModeAt:function(t){var n=this.doc.mode;return n.innerMode?e.innerMode(n,this.getTokenAt(t).state).mode:n},getHelper:function(e,t){return this.getHelpers(e,t)[0]},getHelpers:function(e,t){var r=[];if(!n.hasOwnProperty(t))return r;var i=n[t],o=this.getModeAt(e);if("string"==typeof o[t])i[o[t]]&&r.push(i[o[t]]);else if(o[t])for(var a=0;a<o[t].length;a++){var s=i[o[t][a]];s&&r.push(s)}else o.helperType&&i[o.helperType]?r.push(i[o.helperType]):i[o.name]&&r.push(i[o.name]);for(var l=0;l<i._global.length;l++){var u=i._global[l];u.pred(o,this)&&-1==B(r,u.val)&&r.push(u.val)}return r},getStateAfter:function(e,t){var n=this.doc;return ft(this,(e=at(n,null==e?n.first+n.size-1:e))+1,t).state},cursorCoords:function(e,t){var n=this.doc.sel.primary();return Xn(this,null==e?n.head:"object"==typeof e?st(this.doc,e):e?n.from():n.to(),t||"page")},charCoords:function(e,t){return Gn(this,st(this.doc,e),t||"page")},coordsChar:function(e,t){return Qn(this,(e=Wn(this,e,t||"page")).left,e.top)},lineAtHeight:function(e,t){return e=Wn(this,{top:e,left:0},t||"page").top,Ke(this.doc,e+this.display.viewOffset)},heightAtLine:function(e,t,n){var r,i=!1;if("number"==typeof e){var o=this.doc.first+this.doc.size-1;e<this.doc.first?e=this.doc.first:e>o&&(e=o,i=!0),r=Ge(this.doc,e)}else r=e;return Yn(this,r,{top:0,left:0},t||"page",n||i).top+(i?this.doc.height-Vt(r):0)},defaultTextHeight:function(){return rr(this.display)},defaultCharWidth:function(){return ir(this.display)},getViewport:function(){return{from:this.display.viewFrom,to:this.display.viewTo}},addWidget:function(e,t,n,r,i){var o,a,s,l=this.display,u=(e=Xn(this,st(this.doc,e))).bottom,c=e.left;if(t.style.position="absolute",t.setAttribute("cm-ignore-events","true"),this.display.input.setUneditable(t),l.sizer.appendChild(t),"over"==r)u=e.top;else if("above"==r||"near"==r){var d=Math.max(l.wrapper.clientHeight,this.doc.height),h=Math.max(l.sizer.clientWidth,l.lineSpace.clientWidth);("above"==r||e.bottom+t.offsetHeight>d)&&e.top>t.offsetHeight?u=e.top-t.offsetHeight:e.bottom+t.offsetHeight<=d&&(u=e.bottom),c+t.offsetWidth>h&&(c=h-t.offsetWidth)}t.style.top=u+"px",t.style.left=t.style.right="","right"==i?(c=l.sizer.clientWidth-t.offsetWidth,t.style.right="0px"):("left"==i?c=0:"middle"==i&&(c=(l.sizer.clientWidth-t.offsetWidth)/2),t.style.left=c+"px"),n&&(o=this,a={left:c,top:u,right:c+t.offsetWidth,bottom:u+t.offsetHeight},null!=(s=Er(o,a)).scrollTop&&Nr(o,s.scrollTop),null!=s.scrollLeft&&Rr(o,s.scrollLeft))},triggerOnKeyDown:ti(ca),triggerOnKeyPress:ti(ha),triggerOnKeyUp:da,triggerOnMouseDown:ti(va),execCommand:function(e){if(ta.hasOwnProperty(e))return ta[e].call(null,this)},triggerElectric:ti((function(e){Pa(this,e)})),findPosH:function(e,t,n,r){var i=1;t<0&&(i=-1,t=-t);for(var o=st(this.doc,e),a=0;a<t&&!(o=za(this.doc,o,i,n,r)).hitSide;++a);return o},moveH:ti((function(e,t){var n=this;this.extendSelectionsBy((function(r){return n.display.shift||n.doc.extend||r.empty()?za(n.doc,r.head,e,t,n.options.rtlMoveVisually):e<0?r.from():r.to()}),Y)})),deleteH:ti((function(e,t){var n=this.doc.sel,r=this.doc;n.somethingSelected()?r.replaceSelection("",null,"+delete"):Ko(this,(function(n){var i=za(r,n.head,e,t,!1);return e<0?{from:i,to:n.head}:{from:n.head,to:i}}))})),findPosV:function(e,t,n,r){var i=1,o=r;t<0&&(i=-1,t=-t);for(var a=st(this.doc,e),s=0;s<t;++s){var l=Xn(this,a,"div");if(null==o?o=l.left:l.left=o,(a=Fa(this,l,i,n)).hitSide)break}return a},moveV:ti((function(e,t){var n=this,r=this.doc,i=[],o=!this.display.shift&&!r.extend&&r.sel.somethingSelected();if(r.extendSelectionsBy((function(a){if(o)return e<0?a.from():a.to();var s=Xn(n,a.head,"div");null!=a.goalColumn&&(s.left=a.goalColumn),i.push(s.left);var l=Fa(n,s,e,t);return"page"==t&&a==r.sel.primary()&&Or(n,Gn(n,l,"div").top-s.top),l}),Y),i.length)for(var a=0;a<r.sel.ranges.length;a++)r.sel.ranges[a].goalColumn=i[a]})),findWordAt:function(e){var t=Ge(this.doc,e.line).text,n=e.ch,r=e.ch;if(t){var i=this.getHelper(e,"wordChars");"before"!=e.sticky&&r!=t.length||!n?++r:--n;for(var o=t.charAt(n),a=ee(o,i)?function(e){return ee(e,i)}:/\s/.test(o)?function(e){return/\s/.test(e)}:function(e){return!/\s/.test(e)&&!ee(e)};n>0&&a(t.charAt(n-1));)--n;for(;r<t.length&&a(t.charAt(r));)++r}return new Ii(et(e.line,n),et(e.line,r))},toggleOverwrite:function(e){null!=e&&e==this.state.overwrite||((this.state.overwrite=!this.state.overwrite)?D(this.display.cursorDiv,"CodeMirror-overwrite"):M(this.display.cursorDiv,"CodeMirror-overwrite"),pe(this,"overwriteToggle",this,this.state.overwrite))},hasFocus:function(){return this.display.input.getField()==L()},isReadOnly:function(){return!(!this.options.readOnly&&!this.doc.cantEdit)},scrollTo:ti((function(e,t){Lr(this,e,t)})),getScrollInfo:function(){var e=this.display.scroller;return{left:e.scrollLeft,top:e.scrollTop,height:e.scrollHeight-Mn(this)-this.display.barHeight,width:e.scrollWidth-Mn(this)-this.display.barWidth,clientHeight:Tn(this),clientWidth:Sn(this)}},scrollIntoView:ti((function(e,t){null==e?(e={from:this.doc.sel.primary().head,to:null},null==t&&(t=this.options.cursorScrollMargin)):"number"==typeof e?e={from:et(e,0),to:null}:null==e.from&&(e={from:e,to:null}),e.to||(e.to=e.from),e.margin=t||0,null!=e.from.line?function(e,t){Dr(e),e.curOp.scrollToPos=t}(this,e):Pr(this,e.from,e.to,e.margin)})),setSize:ti((function(e,t){var n=this,r=function(e){return"number"==typeof e||/^\d+$/.test(String(e))?e+"px":e};null!=e&&(this.display.wrapper.style.width=r(e)),null!=t&&(this.display.wrapper.style.height=r(t)),this.options.lineWrapping&&Fn(this);var i=this.display.viewFrom;this.doc.iter(i,this.display.viewTo,(function(e){if(e.widgets)for(var t=0;t<e.widgets.length;t++)if(e.widgets[t].noHScroll){hr(n,i,"widget");break}++i})),this.curOp.forceUpdate=!0,pe(this,"refresh",this)})),operation:function(e){return $r(this,e)},startOperation:function(){return Gr(this)},endOperation:function(){return Xr(this)},refresh:ti((function(){var e=this.display.cachedTextHeight;dr(this),this.curOp.forceUpdate=!0,Bn(this),Lr(this,this.doc.scrollLeft,this.doc.scrollTop),ui(this.display),(null==e||Math.abs(e-rr(this.display))>.5||this.options.lineWrapping)&&lr(this),pe(this,"refresh",this)})),swapDoc:ti((function(e){var t=this.doc;return t.cm=null,this.state.selectingText&&this.state.selectingText(),Ni(this,e),Bn(this),this.display.input.reset(),Lr(this,e.scrollLeft,e.scrollTop),this.curOp.forceScroll=!0,ln(this,"swapDoc",this,t),t})),phrase:function(e){var t=this.options.phrases;return t&&Object.prototype.hasOwnProperty.call(t,e)?t[e]:e},getInputField:function(){return this.display.input.getField()},getWrapperElement:function(){return this.display.wrapper},getScrollerElement:function(){return this.display.scroller},getGutterElement:function(){return this.display.gutters}},ye(e),e.registerHelper=function(t,r,i){n.hasOwnProperty(t)||(n[t]=e[t]={_global:[]}),n[t][r]=i},e.registerGlobalHelper=function(t,r,i,o){e.registerHelper(t,r,o),n[t]._global.push({pred:i,val:o})}}(Sa);var Ga="iter insert remove copy getEditor constructor".split(" ");for(var Xa in ko.prototype)ko.prototype.hasOwnProperty(Xa)&&B(Ga,Xa)<0&&(Sa.prototype[Xa]=function(e){return function(){return e.apply(this.doc,arguments)}}(ko.prototype[Xa]));return ye(ko),Sa.inputStyles={textarea:Wa,contenteditable:Ba},Sa.defineMode=function(e){Sa.defaults.mode||"null"==e||(Sa.defaults.mode=e),Re.apply(this,arguments)},Sa.defineMIME=function(e,t){je[e]=t},Sa.defineMode("null",(function(){return{token:function(e){return e.skipToEnd()}}})),Sa.defineMIME("text/plain","null"),Sa.defineExtension=function(e,t){Sa.prototype[e]=t},Sa.defineDocExtension=function(e,t){ko.prototype[e]=t},Sa.fromTextArea=function(e,t){if((t=t?R(t):{}).value=e.value,!t.tabindex&&e.tabIndex&&(t.tabindex=e.tabIndex),!t.placeholder&&e.placeholder&&(t.placeholder=e.placeholder),null==t.autofocus){var n=L();t.autofocus=n==e||null!=e.getAttribute("autofocus")&&n==document.body}function r(){e.value=s.getValue()}var i;if(e.form&&(de(e.form,"submit",r),!t.leaveSubmitMethodAlone)){var o=e.form;i=o.submit;try{var a=o.submit=function(){r(),o.submit=i,o.submit(),o.submit=a}}catch(e){}}t.finishInit=function(n){n.save=r,n.getTextArea=function(){return e},n.toTextArea=function(){n.toTextArea=isNaN,r(),e.parentNode.removeChild(n.getWrapperElement()),e.style.display="",e.form&&(fe(e.form,"submit",r),t.leaveSubmitMethodAlone||"function"!=typeof e.form.submit||(e.form.submit=i))}},e.style.display="none";var s=Sa((function(t){return e.parentNode.insertBefore(t,e.nextSibling)}),t);return s},function(e){e.off=fe,e.on=de,e.wheelEventPixels=Ci,e.Doc=ko,e.splitLines=ke,e.countColumn=z,e.findColumn=W,e.isWordChar=$,e.Pass=H,e.signal=pe,e.Line=Gt,e.changeEnd=Mi,e.scrollbarModel=Vr,e.Pos=et,e.cmpPos=tt,e.modes=Ne,e.mimeModes=je,e.resolveMode=ze,e.getMode=Fe,e.modeExtensions=Be,e.extendMode=He,e.copyState=Ue,e.startState=Ye,e.innerMode=Ve,e.commands=ta,e.keyMap=Vo,e.keyName=Zo,e.isModifierKey=Xo,e.lookupKey=Go,e.normalizeKeyMap=Wo,e.StringStream=We,e.SharedTextMarker=So,e.TextMarker=wo,e.LineWidget=_o,e.e_preventDefault=be,e.e_stopPropagation=Ce,e.e_stop=_e,e.addClass=D,e.contains=k,e.rmClass=M,e.keyNames=Fo}(Sa),Sa.version="5.57.0",Sa}()},function(e,t,n){"use strict";var r=n(21),i=n(264);function o(e,t){return t&&"string"==typeof t?t.split(".").reduce((function(e,t){return e&&e[t]?e[t]:null}),e):null}t.a=function(e){var t=e.prop,n=e.cssProperty,a=void 0===n?e.prop:n,s=e.themeKey,l=e.transform,u=function(e){if(null==e[t])return null;var n=e[t],u=o(e.theme,s)||{};return Object(i.b)(e,n,(function(e){var t;return"function"==typeof u?t=u(e):Array.isArray(u)?t=u[e]||e:(t=o(u,e)||e,l&&(t=l(t))),!1===a?t:Object(r.a)({},a,t)}))};return u.propTypes={},u.filterProps=[t],u}},function(e,t,n){"use strict";n.d(t,"g",(function(){return o})),n.d(t,"k",(function(){return a})),n.d(t,"h",(function(){return s})),n.d(t,"b",(function(){return l})),n.d(t,"j",(function(){return u})),n.d(t,"e",(function(){return c})),n.d(t,"f",(function(){return d})),n.d(t,"c",(function(){return h})),n.d(t,"d",(function(){return f})),n.d(t,"a",(function(){return p})),n.d(t,"i",(function(){return g}));var r=n(334);function i(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:0,n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:1;return Math.min(Math.max(t,e),n)}function o(e){e=e.substr(1);var t=new RegExp(".{1,".concat(e.length>=6?2:1,"}"),"g"),n=e.match(t);return n&&1===n[0].length&&(n=n.map((function(e){return e+e}))),n?"rgb".concat(4===n.length?"a":"","(").concat(n.map((function(e,t){return t<3?parseInt(e,16):Math.round(parseInt(e,16)/255*1e3)/1e3})).join(", "),")"):""}function a(e){if(0===e.indexOf("#"))return e;var t=l(e).values;return"#".concat(t.map((function(e){return 1===(t=e.toString(16)).length?"0".concat(t):t;var t})).join(""))}function s(e){var t=(e=l(e)).values,n=t[0],r=t[1]/100,i=t[2]/100,o=r*Math.min(i,1-i),a=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:(e+n/30)%12;return i-o*Math.max(Math.min(t-3,9-t,1),-1)},s="rgb",c=[Math.round(255*a(0)),Math.round(255*a(8)),Math.round(255*a(4))];return"hsla"===e.type&&(s+="a",c.push(t[3])),u({type:s,values:c})}function l(e){if(e.type)return e;if("#"===e.charAt(0))return l(o(e));var t=e.indexOf("("),n=e.substring(0,t);if(-1===["rgb","rgba","hsl","hsla"].indexOf(n))throw new Error(Object(r.a)(3,e));var i=e.substring(t+1,e.length-1).split(",");return{type:n,values:i=i.map((function(e){return parseFloat(e)}))}}function u(e){var t=e.type,n=e.values;return-1!==t.indexOf("rgb")?n=n.map((function(e,t){return t<3?parseInt(e,10):e})):-1!==t.indexOf("hsl")&&(n[1]="".concat(n[1],"%"),n[2]="".concat(n[2],"%")),"".concat(t,"(").concat(n.join(", "),")")}function c(e,t){var n=d(e),r=d(t);return(Math.max(n,r)+.05)/(Math.min(n,r)+.05)}function d(e){var t="hsl"===(e=l(e)).type?l(s(e)).values:e.values;return t=t.map((function(e){return(e/=255)<=.03928?e/12.92:Math.pow((e+.055)/1.055,2.4)})),Number((.2126*t[0]+.7152*t[1]+.0722*t[2]).toFixed(3))}function h(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:.15;return d(e)>.5?p(e,t):g(e,t)}function f(e,t){return e=l(e),t=i(t),"rgb"!==e.type&&"hsl"!==e.type||(e.type+="a"),e.values[3]=t,u(e)}function p(e,t){if(e=l(e),t=i(t),-1!==e.type.indexOf("hsl"))e.values[2]*=1-t;else if(-1!==e.type.indexOf("rgb"))for(var n=0;n<3;n+=1)e.values[n]*=1-t;return u(e)}function g(e,t){if(e=l(e),t=i(t),-1!==e.type.indexOf("hsl"))e.values[2]+=(100-e.values[2])*t;else if(-1!==e.type.indexOf("rgb"))for(var n=0;n<3;n+=1)e.values[n]+=(255-e.values[n])*t;return u(e)}},function(e,t,n){"use strict";var r=n(252),i=n(526),o=n(967),a=n(292),s=n(527).addStyleRule,l=n(63),u=n(160),c=n(295),d=l.extendFlat,h=l.extendDeepAll;function f(e){var n=e.name,i=e.categories,o=e.meta;if(t.modules[n])r.log("Type "+n+" already registered");else{t.subplotsRegistry[e.basePlotModule.name]||function(e){var n=e.name;if(t.subplotsRegistry[n])return void r.log("Plot type "+n+" already registered.");for(var i in m(e),t.subplotsRegistry[n]=e,t.componentsRegistry)C(i,e.name)}(e.basePlotModule);for(var a={},l=0;l<i.length;l++)a[i[l]]=!0,t.allCategories[i[l]]=!0;for(var u in t.modules[n]={_module:e,categories:a},o&&Object.keys(o).length&&(t.modules[n].meta=o),t.allTypes.push(n),t.componentsRegistry)y(u,n);e.layoutAttributes&&d(t.traceLayoutAttributes,e.layoutAttributes);var c=e.basePlotModule,h=c.name;if("mapbox"===h){var f=c.constants.styleRules;for(var p in f)s(".js-plotly-plot .plotly .mapboxgl-"+p,f[p])}"geo"!==h&&"mapbox"!==h||void 0===typeof window||void 0!==window.PlotlyGeoAssets||(window.PlotlyGeoAssets={topojson:{}})}}function p(e){if("string"!=typeof e.name)throw new Error("Component module *name* must be a string.");var n=e.name;for(var r in t.componentsRegistry[n]=e,e.layoutAttributes&&(e.layoutAttributes._isLinkedToArray&&o(t.layoutArrayContainers,n),m(e)),t.modules)y(n,r);for(var i in t.subplotsRegistry)C(n,i);for(var a in t.transformsRegistry)b(n,a);e.schema&&e.schema.layout&&h(c,e.schema.layout)}function g(e){if("string"!=typeof e.name)throw new Error("Transform module *name* must be a string.");var n="Transform module "+e.name,i="function"==typeof e.transform,o="function"==typeof e.calcTransform;if(!i&&!o)throw new Error(n+" is missing a *transform* or *calcTransform* method.");for(var s in i&&o&&r.log([n+" has both a *transform* and *calcTransform* methods.","Please note that all *transform* methods are executed","before all *calcTransform* methods."].join(" ")),a(e.attributes)||r.log(n+" registered without an *attributes* object."),"function"!=typeof e.supplyDefaults&&r.log(n+" registered without a *supplyDefaults* method."),t.transformsRegistry[e.name]=e,t.componentsRegistry)b(s,e.name)}function v(e){var n=e.name,r=n.split("-")[0],i=e.dictionary,o=e.format,a=i&&Object.keys(i).length,s=o&&Object.keys(o).length,l=t.localeRegistry,u=l[n];if(u||(l[n]=u={}),r!==n){var c=l[r];c||(l[r]=c={}),a&&c.dictionary===u.dictionary&&(c.dictionary=i),s&&c.format===u.format&&(c.format=o)}a&&(u.dictionary=i),s&&(u.format=o)}function m(e){if(e.layoutAttributes){var n=e.layoutAttributes._arrayAttrRegexps;if(n)for(var r=0;r<n.length;r++)o(t.layoutArrayRegexes,n[r])}}function y(e,n){var r=t.componentsRegistry[e].schema;if(r&&r.traces){var i=r.traces[n];i&&h(t.modules[n]._module.attributes,i)}}function b(e,n){var r=t.componentsRegistry[e].schema;if(r&&r.transforms){var i=r.transforms[n];i&&h(t.transformsRegistry[n].attributes,i)}}function C(e,n){var r=t.componentsRegistry[e].schema;if(r&&r.subplots){var i=t.subplotsRegistry[n],o=i.layoutAttributes,a="subplot"===i.attr?i.name:i.attr;Array.isArray(a)&&(a=a[0]);var s=r.subplots[a];o&&s&&h(o,s)}}function A(e){return"object"==typeof e&&(e=e.type),e}t.modules={},t.allCategories={},t.allTypes=[],t.subplotsRegistry={},t.transformsRegistry={},t.componentsRegistry={},t.layoutArrayContainers=[],t.layoutArrayRegexes=[],t.traceLayoutAttributes={},t.localeRegistry={},t.apiMethodRegistry={},t.collectableSubplotTypes=null,t.register=function(e){if(t.collectableSubplotTypes=null,!e)throw new Error("No argument passed to Plotly.register.");e&&!Array.isArray(e)&&(e=[e]);for(var n=0;n<e.length;n++){var r=e[n];if(!r)throw new Error("Invalid module was attempted to be registered!");switch(r.moduleType){case"trace":f(r);break;case"transform":g(r);break;case"component":p(r);break;case"locale":v(r);break;case"apiMethod":var i=r.name;t.apiMethodRegistry[i]=r.fn;break;default:throw new Error("Invalid module was attempted to be registered!")}}},t.getModule=function(e){var n=t.modules[A(e)];return!!n&&n._module},t.traceIs=function(e,n){if("various"===(e=A(e)))return!1;var i=t.modules[e];return i||(e&&"area"!==e&&r.log("Unrecognized trace type "+e+"."),i=t.modules[u.type.dflt]),!!i.categories[n]},t.getTransformIndices=function(e,t){for(var n=[],r=e.transforms||[],i=0;i<r.length;i++)r[i].type===t&&n.push(i);return n},t.hasTransform=function(e,t){for(var n=e.transforms||[],r=0;r<n.length;r++)if(n[r].type===t)return!0;return!1},t.getComponentMethod=function(e,n){var r=t.componentsRegistry[e];return r&&r[n]||i},t.call=function(){var e=arguments[0],n=[].slice.call(arguments,1);return t.apiMethodRegistry[e].apply(null,n)}},function(e,t,n){"use strict";n.r(t);var r={};n.r(r),n.d(r,"VERSION",(function(){return i.e})),n.d(r,"restArguments",(function(){return o})),n.d(r,"isObject",(function(){return a})),n.d(r,"isNull",(function(){return s})),n.d(r,"isUndefined",(function(){return l})),n.d(r,"isBoolean",(function(){return u})),n.d(r,"isElement",(function(){return c})),n.d(r,"isString",(function(){return h})),n.d(r,"isNumber",(function(){return f})),n.d(r,"isDate",(function(){return p})),n.d(r,"isRegExp",(function(){return g})),n.d(r,"isError",(function(){return v})),n.d(r,"isSymbol",(function(){return m})),n.d(r,"isArrayBuffer",(function(){return y})),n.d(r,"isDataView",(function(){return M})),n.d(r,"isArray",(function(){return S})),n.d(r,"isFunction",(function(){return A})),n.d(r,"isArguments",(function(){return O})),n.d(r,"isFinite",(function(){return k})),n.d(r,"isNaN",(function(){return L})),n.d(r,"isTypedArray",(function(){return F})),n.d(r,"isEmpty",(function(){return V})),n.d(r,"isMatch",(function(){return Y})),n.d(r,"isEqual",(function(){return q})),n.d(r,"isMap",(function(){return re})),n.d(r,"isWeakMap",(function(){return ie})),n.d(r,"isSet",(function(){return oe})),n.d(r,"isWeakSet",(function(){return ae})),n.d(r,"keys",(function(){return U})),n.d(r,"allKeys",(function(){return Z})),n.d(r,"values",(function(){return se})),n.d(r,"pairs",(function(){return le})),n.d(r,"invert",(function(){return ue})),n.d(r,"functions",(function(){return ce})),n.d(r,"methods",(function(){return ce})),n.d(r,"extend",(function(){return he})),n.d(r,"extendOwn",(function(){return fe})),n.d(r,"assign",(function(){return fe})),n.d(r,"defaults",(function(){return pe})),n.d(r,"create",(function(){return ve})),n.d(r,"clone",(function(){return me})),n.d(r,"tap",(function(){return ye})),n.d(r,"get",(function(){return _e})),n.d(r,"has",(function(){return Ie})),n.d(r,"mapObject",(function(){return ke})),n.d(r,"identity",(function(){return xe})),n.d(r,"constant",(function(){return D})),n.d(r,"noop",(function(){return Le})),n.d(r,"toPath",(function(){return be})),n.d(r,"property",(function(){return Me})),n.d(r,"propertyOf",(function(){return De})),n.d(r,"matcher",(function(){return we})),n.d(r,"matches",(function(){return we})),n.d(r,"times",(function(){return Pe})),n.d(r,"random",(function(){return Ne})),n.d(r,"now",(function(){return je})),n.d(r,"escape",(function(){return Fe})),n.d(r,"unescape",(function(){return Be})),n.d(r,"templateSettings",(function(){return He})),n.d(r,"template",(function(){return Ge})),n.d(r,"result",(function(){return Xe})),n.d(r,"uniqueId",(function(){return Ze})),n.d(r,"chain",(function(){return Qe})),n.d(r,"iteratee",(function(){return Ee})),n.d(r,"partial",(function(){return $e})),n.d(r,"bind",(function(){return et})),n.d(r,"bindAll",(function(){return rt})),n.d(r,"memoize",(function(){return it})),n.d(r,"delay",(function(){return ot})),n.d(r,"defer",(function(){return at})),n.d(r,"throttle",(function(){return st})),n.d(r,"debounce",(function(){return lt})),n.d(r,"wrap",(function(){return ut})),n.d(r,"negate",(function(){return ct})),n.d(r,"compose",(function(){return dt})),n.d(r,"after",(function(){return ht})),n.d(r,"before",(function(){return ft})),n.d(r,"once",(function(){return pt})),n.d(r,"findKey",(function(){return gt})),n.d(r,"findIndex",(function(){return mt})),n.d(r,"findLastIndex",(function(){return yt})),n.d(r,"sortedIndex",(function(){return bt})),n.d(r,"indexOf",(function(){return At})),n.d(r,"lastIndexOf",(function(){return _t})),n.d(r,"find",(function(){return It})),n.d(r,"detect",(function(){return It})),n.d(r,"findWhere",(function(){return xt})),n.d(r,"each",(function(){return wt})),n.d(r,"forEach",(function(){return wt})),n.d(r,"map",(function(){return Mt})),n.d(r,"collect",(function(){return Mt})),n.d(r,"reduce",(function(){return Tt})),n.d(r,"foldl",(function(){return Tt})),n.d(r,"inject",(function(){return Tt})),n.d(r,"reduceRight",(function(){return Et})),n.d(r,"foldr",(function(){return Et})),n.d(r,"filter",(function(){return Ot})),n.d(r,"select",(function(){return Ot})),n.d(r,"reject",(function(){return kt})),n.d(r,"every",(function(){return Lt})),n.d(r,"all",(function(){return Lt})),n.d(r,"some",(function(){return Dt})),n.d(r,"any",(function(){return Dt})),n.d(r,"contains",(function(){return Pt})),n.d(r,"includes",(function(){return Pt})),n.d(r,"include",(function(){return Pt})),n.d(r,"invoke",(function(){return Nt})),n.d(r,"pluck",(function(){return jt})),n.d(r,"where",(function(){return Rt})),n.d(r,"max",(function(){return zt})),n.d(r,"min",(function(){return Ft})),n.d(r,"shuffle",(function(){return Ht})),n.d(r,"sample",(function(){return Bt})),n.d(r,"sortBy",(function(){return Ut})),n.d(r,"groupBy",(function(){return Yt})),n.d(r,"indexBy",(function(){return Wt})),n.d(r,"countBy",(function(){return Gt})),n.d(r,"partition",(function(){return Xt})),n.d(r,"toArray",(function(){return Zt})),n.d(r,"size",(function(){return Qt})),n.d(r,"pick",(function(){return Jt})),n.d(r,"omit",(function(){return $t})),n.d(r,"first",(function(){return tn})),n.d(r,"head",(function(){return tn})),n.d(r,"take",(function(){return tn})),n.d(r,"initial",(function(){return en})),n.d(r,"last",(function(){return rn})),n.d(r,"rest",(function(){return nn})),n.d(r,"tail",(function(){return nn})),n.d(r,"drop",(function(){return nn})),n.d(r,"compact",(function(){return on})),n.d(r,"flatten",(function(){return an})),n.d(r,"without",(function(){return ln})),n.d(r,"uniq",(function(){return un})),n.d(r,"unique",(function(){return un})),n.d(r,"union",(function(){return cn})),n.d(r,"intersection",(function(){return dn})),n.d(r,"difference",(function(){return sn})),n.d(r,"unzip",(function(){return hn})),n.d(r,"transpose",(function(){return hn})),n.d(r,"zip",(function(){return fn})),n.d(r,"object",(function(){return pn})),n.d(r,"range",(function(){return gn})),n.d(r,"chunk",(function(){return vn})),n.d(r,"mixin",(function(){return yn})),n.d(r,"default",(function(){return bn}));var i=n(32);function o(e,t){return t=null==t?e.length-1:+t,function(){for(var n=Math.max(arguments.length-t,0),r=Array(n),i=0;i<n;i++)r[i]=arguments[i+t];switch(t){case 0:return e.call(this,r);case 1:return e.call(this,arguments[0],r);case 2:return e.call(this,arguments[0],arguments[1],r)}var o=Array(t+1);for(i=0;i<t;i++)o[i]=arguments[i];return o[t]=r,e.apply(this,o)}}function a(e){var t=typeof e;return"function"===t||"object"===t&&!!e}function s(e){return null===e}function l(e){return void 0===e}function u(e){return!0===e||!1===e||"[object Boolean]"===i.t.call(e)}function c(e){return!(!e||1!==e.nodeType)}function d(e){var t="[object "+e+"]";return function(e){return i.t.call(e)===t}}var h=d("String"),f=d("Number"),p=d("Date"),g=d("RegExp"),v=d("Error"),m=d("Symbol"),y=d("ArrayBuffer"),b=d("Function"),C=i.p.document&&i.p.document.childNodes;"object"!=typeof Int8Array&&"function"!=typeof C&&(b=function(e){return"function"==typeof e||!1});var A=b,_=d("Object"),I=i.s&&_(new DataView(new ArrayBuffer(8))),x="undefined"!=typeof Map&&_(new Map),w=d("DataView");var M=I?function(e){return null!=e&&A(e.getInt8)&&y(e.buffer)}:w,S=i.k||d("Array");function T(e,t){return null!=e&&i.i.call(e,t)}var E=d("Arguments");!function(){E(arguments)||(E=function(e){return T(e,"callee")})}();var O=E;function k(e){return!m(e)&&Object(i.f)(e)&&!isNaN(parseFloat(e))}function L(e){return f(e)&&Object(i.g)(e)}function D(e){return function(){return e}}function P(e){return function(t){var n=e(t);return"number"==typeof n&&n>=0&&n<=i.b}}function N(e){return function(t){return null==t?void 0:t[e]}}var j=N("byteLength"),R=P(j),z=/\[object ((I|Ui)nt(8|16|32)|Float(32|64)|Uint8Clamped|Big(I|Ui)nt64)Array\]/;var F=i.r?function(e){return i.l?Object(i.l)(e)&&!M(e):R(e)&&z.test(i.t.call(e))}:D(!1),B=N("length");function H(e,t){t=function(e){for(var t={},n=e.length,r=0;r<n;++r)t[e[r]]=!0;return{contains:function(e){return t[e]},push:function(n){return t[n]=!0,e.push(n)}}}(t);var n=i.n.length,r=e.constructor,o=A(r)&&r.prototype||i.c,a="constructor";for(T(e,a)&&!t.contains(a)&&t.push(a);n--;)(a=i.n[n])in e&&e[a]!==o[a]&&!t.contains(a)&&t.push(a)}function U(e){if(!a(e))return[];if(i.m)return Object(i.m)(e);var t=[];for(var n in e)T(e,n)&&t.push(n);return i.h&&H(e,t),t}function V(e){if(null==e)return!0;var t=B(e);return"number"==typeof t&&(S(e)||h(e)||O(e))?0===t:0===B(U(e))}function Y(e,t){var n=U(t),r=n.length;if(null==e)return!r;for(var i=Object(e),o=0;o<r;o++){var a=n[o];if(t[a]!==i[a]||!(a in i))return!1}return!0}function W(e){return e instanceof W?e:this instanceof W?void(this._wrapped=e):new W(e)}function G(e){return new Uint8Array(e.buffer||e,e.byteOffset||0,j(e))}W.VERSION=i.e,W.prototype.value=function(){return this._wrapped},W.prototype.valueOf=W.prototype.toJSON=W.prototype.value,W.prototype.toString=function(){return String(this._wrapped)};function X(e,t,n,r){if(e===t)return 0!==e||1/e==1/t;if(null==e||null==t)return!1;if(e!=e)return t!=t;var o=typeof e;return("function"===o||"object"===o||"object"==typeof t)&&function e(t,n,r,o){t instanceof W&&(t=t._wrapped);n instanceof W&&(n=n._wrapped);var a=i.t.call(t);if(a!==i.t.call(n))return!1;if(I&&"[object Object]"==a&&M(t)){if(!M(n))return!1;a="[object DataView]"}switch(a){case"[object RegExp]":case"[object String]":return""+t==""+n;case"[object Number]":return+t!=+t?+n!=+n:0==+t?1/+t==1/n:+t==+n;case"[object Date]":case"[object Boolean]":return+t==+n;case"[object Symbol]":return i.d.valueOf.call(t)===i.d.valueOf.call(n);case"[object ArrayBuffer]":case"[object DataView]":return e(G(t),G(n),r,o)}var s="[object Array]"===a;if(!s&&F(t)){if(j(t)!==j(n))return!1;if(t.buffer===n.buffer&&t.byteOffset===n.byteOffset)return!0;s=!0}if(!s){if("object"!=typeof t||"object"!=typeof n)return!1;var l=t.constructor,u=n.constructor;if(l!==u&&!(A(l)&&l instanceof l&&A(u)&&u instanceof u)&&"constructor"in t&&"constructor"in n)return!1}o=o||[];var c=(r=r||[]).length;for(;c--;)if(r[c]===t)return o[c]===n;if(r.push(t),o.push(n),s){if((c=t.length)!==n.length)return!1;for(;c--;)if(!X(t[c],n[c],r,o))return!1}else{var d,h=U(t);if(c=h.length,U(n).length!==c)return!1;for(;c--;)if(d=h[c],!T(n,d)||!X(t[d],n[d],r,o))return!1}return r.pop(),o.pop(),!0}(e,t,n,r)}function q(e,t){return X(e,t)}function Z(e){if(!a(e))return[];var t=[];for(var n in e)t.push(n);return i.h&&H(e,t),t}function Q(e){var t=B(e);return function(n){if(null==n)return!1;var r=Z(n);if(B(r))return!1;for(var i=0;i<t;i++)if(!A(n[e[i]]))return!1;return e!==te||!A(n[K])}}var K="forEach",J=["clear","delete"],$=["get","has","set"],ee=J.concat(K,$),te=J.concat($),ne=["add"].concat(J,K,"has"),re=x?Q(ee):d("Map"),ie=x?Q(te):d("WeakMap"),oe=x?Q(ne):d("Set"),ae=d("WeakSet");function se(e){for(var t=U(e),n=t.length,r=Array(n),i=0;i<n;i++)r[i]=e[t[i]];return r}function le(e){for(var t=U(e),n=t.length,r=Array(n),i=0;i<n;i++)r[i]=[t[i],e[t[i]]];return r}function ue(e){for(var t={},n=U(e),r=0,i=n.length;r<i;r++)t[e[n[r]]]=n[r];return t}function ce(e){var t=[];for(var n in e)A(e[n])&&t.push(n);return t.sort()}function de(e,t){return function(n){var r=arguments.length;if(t&&(n=Object(n)),r<2||null==n)return n;for(var i=1;i<r;i++)for(var o=arguments[i],a=e(o),s=a.length,l=0;l<s;l++){var u=a[l];t&&void 0!==n[u]||(n[u]=o[u])}return n}}var he=de(Z),fe=de(U),pe=de(Z,!0);function ge(e){if(!a(e))return{};if(i.j)return Object(i.j)(e);var t=function(){};t.prototype=e;var n=new t;return t.prototype=null,n}function ve(e,t){var n=ge(e);return t&&fe(n,t),n}function me(e){return a(e)?S(e)?e.slice():he({},e):e}function ye(e,t){return t(e),e}function be(e){return S(e)?e:[e]}function Ce(e){return W.toPath(e)}function Ae(e,t){for(var n=t.length,r=0;r<n;r++){if(null==e)return;e=e[t[r]]}return n?e:void 0}function _e(e,t,n){var r=Ae(e,Ce(t));return l(r)?n:r}function Ie(e,t){for(var n=(t=Ce(t)).length,r=0;r<n;r++){var i=t[r];if(!T(e,i))return!1;e=e[i]}return!!n}function xe(e){return e}function we(e){return e=fe({},e),function(t){return Y(t,e)}}function Me(e){return e=Ce(e),function(t){return Ae(t,e)}}function Se(e,t,n){if(void 0===t)return e;switch(null==n?3:n){case 1:return function(n){return e.call(t,n)};case 3:return function(n,r,i){return e.call(t,n,r,i)};case 4:return function(n,r,i,o){return e.call(t,n,r,i,o)}}return function(){return e.apply(t,arguments)}}function Te(e,t,n){return null==e?xe:A(e)?Se(e,t,n):a(e)&&!S(e)?we(e):Me(e)}function Ee(e,t){return Te(e,t,1/0)}function Oe(e,t,n){return W.iteratee!==Ee?W.iteratee(e,t):Te(e,t,n)}function ke(e,t,n){t=Oe(t,n);for(var r=U(e),i=r.length,o={},a=0;a<i;a++){var s=r[a];o[s]=t(e[s],s,e)}return o}function Le(){}function De(e){return null==e?Le:function(t){return _e(e,t)}}function Pe(e,t,n){var r=Array(Math.max(0,e));t=Se(t,n,1);for(var i=0;i<e;i++)r[i]=t(i);return r}function Ne(e,t){return null==t&&(t=e,e=0),e+Math.floor(Math.random()*(t-e+1))}W.toPath=be,W.iteratee=Ee;var je=Date.now||function(){return(new Date).getTime()};function Re(e){var t=function(t){return e[t]},n="(?:"+U(e).join("|")+")",r=RegExp(n),i=RegExp(n,"g");return function(e){return e=null==e?"":""+e,r.test(e)?e.replace(i,t):e}}var ze={"&":"&amp;","<":"&lt;",">":"&gt;",'"':"&quot;","'":"&#x27;","`":"&#x60;"},Fe=Re(ze),Be=Re(ue(ze)),He=W.templateSettings={evaluate:/<%([\s\S]+?)%>/g,interpolate:/<%=([\s\S]+?)%>/g,escape:/<%-([\s\S]+?)%>/g},Ue=/(.)^/,Ve={"'":"'","\\":"\\","\r":"r","\n":"n","\u2028":"u2028","\u2029":"u2029"},Ye=/\\|'|\r|\n|\u2028|\u2029/g;function We(e){return"\\"+Ve[e]}function Ge(e,t,n){!t&&n&&(t=n),t=pe({},t,W.templateSettings);var r,i=RegExp([(t.escape||Ue).source,(t.interpolate||Ue).source,(t.evaluate||Ue).source].join("|")+"|$","g"),o=0,a="__p+='";e.replace(i,(function(t,n,r,i,s){return a+=e.slice(o,s).replace(Ye,We),o=s+t.length,n?a+="'+\n((__t=("+n+"))==null?'':_.escape(__t))+\n'":r?a+="'+\n((__t=("+r+"))==null?'':__t)+\n'":i&&(a+="';\n"+i+"\n__p+='"),t})),a+="';\n",t.variable||(a="with(obj||{}){\n"+a+"}\n"),a="var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};\n"+a+"return __p;\n";try{r=new Function(t.variable||"obj","_",a)}catch(e){throw e.source=a,e}var s=function(e){return r.call(this,e,W)},l=t.variable||"obj";return s.source="function("+l+"){\n"+a+"}",s}function Xe(e,t,n){var r=(t=Ce(t)).length;if(!r)return A(n)?n.call(e):n;for(var i=0;i<r;i++){var o=null==e?void 0:e[t[i]];void 0===o&&(o=n,i=r),e=A(o)?o.call(e):o}return e}var qe=0;function Ze(e){var t=++qe+"";return e?e+t:t}function Qe(e){var t=W(e);return t._chain=!0,t}function Ke(e,t,n,r,i){if(!(r instanceof t))return e.apply(n,i);var o=ge(e.prototype),s=e.apply(o,i);return a(s)?s:o}var Je=o((function(e,t){var n=Je.placeholder,r=function(){for(var i=0,o=t.length,a=Array(o),s=0;s<o;s++)a[s]=t[s]===n?arguments[i++]:t[s];for(;i<arguments.length;)a.push(arguments[i++]);return Ke(e,r,this,this,a)};return r}));Je.placeholder=W;var $e=Je,et=o((function(e,t,n){if(!A(e))throw new TypeError("Bind must be called on a function");var r=o((function(i){return Ke(e,r,t,this,n.concat(i))}));return r})),tt=P(B);function nt(e,t,n,r){if(r=r||[],t||0===t){if(t<=0)return r.concat(e)}else t=1/0;for(var i=r.length,o=0,a=B(e);o<a;o++){var s=e[o];if(tt(s)&&(S(s)||O(s)))if(t>1)nt(s,t-1,n,r),i=r.length;else for(var l=0,u=s.length;l<u;)r[i++]=s[l++];else n||(r[i++]=s)}return r}var rt=o((function(e,t){var n=(t=nt(t,!1,!1)).length;if(n<1)throw new Error("bindAll must be passed function names");for(;n--;){var r=t[n];e[r]=et(e[r],e)}return e}));function it(e,t){var n=function(r){var i=n.cache,o=""+(t?t.apply(this,arguments):r);return T(i,o)||(i[o]=e.apply(this,arguments)),i[o]};return n.cache={},n}var ot=o((function(e,t,n){return setTimeout((function(){return e.apply(null,n)}),t)})),at=$e(ot,W,1);function st(e,t,n){var r,i,o,a,s=0;n||(n={});var l=function(){s=!1===n.leading?0:je(),r=null,a=e.apply(i,o),r||(i=o=null)},u=function(){var u=je();s||!1!==n.leading||(s=u);var c=t-(u-s);return i=this,o=arguments,c<=0||c>t?(r&&(clearTimeout(r),r=null),s=u,a=e.apply(i,o),r||(i=o=null)):r||!1===n.trailing||(r=setTimeout(l,c)),a};return u.cancel=function(){clearTimeout(r),s=0,r=i=o=null},u}function lt(e,t,n){var r,i,a=function(t,n){r=null,n&&(i=e.apply(t,n))},s=o((function(o){if(r&&clearTimeout(r),n){var s=!r;r=setTimeout(a,t),s&&(i=e.apply(this,o))}else r=ot(a,t,this,o);return i}));return s.cancel=function(){clearTimeout(r),r=null},s}function ut(e,t){return $e(t,e)}function ct(e){return function(){return!e.apply(this,arguments)}}function dt(){var e=arguments,t=e.length-1;return function(){for(var n=t,r=e[t].apply(this,arguments);n--;)r=e[n].call(this,r);return r}}function ht(e,t){return function(){if(--e<1)return t.apply(this,arguments)}}function ft(e,t){var n;return function(){return--e>0&&(n=t.apply(this,arguments)),e<=1&&(t=null),n}}var pt=$e(ft,2);function gt(e,t,n){t=Oe(t,n);for(var r,i=U(e),o=0,a=i.length;o<a;o++)if(t(e[r=i[o]],r,e))return r}function vt(e){return function(t,n,r){n=Oe(n,r);for(var i=B(t),o=e>0?0:i-1;o>=0&&o<i;o+=e)if(n(t[o],o,t))return o;return-1}}var mt=vt(1),yt=vt(-1);function bt(e,t,n,r){for(var i=(n=Oe(n,r,1))(t),o=0,a=B(e);o<a;){var s=Math.floor((o+a)/2);n(e[s])<i?o=s+1:a=s}return o}function Ct(e,t,n){return function(r,o,a){var s=0,l=B(r);if("number"==typeof a)e>0?s=a>=0?a:Math.max(a+l,s):l=a>=0?Math.min(a+1,l):a+l+1;else if(n&&a&&l)return r[a=n(r,o)]===o?a:-1;if(o!=o)return(a=t(i.q.call(r,s,l),L))>=0?a+s:-1;for(a=e>0?s:l-1;a>=0&&a<l;a+=e)if(r[a]===o)return a;return-1}}var At=Ct(1,mt,bt),_t=Ct(-1,yt);function It(e,t,n){var r=(tt(e)?mt:gt)(e,t,n);if(void 0!==r&&-1!==r)return e[r]}function xt(e,t){return It(e,we(t))}function wt(e,t,n){var r,i;if(t=Se(t,n),tt(e))for(r=0,i=e.length;r<i;r++)t(e[r],r,e);else{var o=U(e);for(r=0,i=o.length;r<i;r++)t(e[o[r]],o[r],e)}return e}function Mt(e,t,n){t=Oe(t,n);for(var r=!tt(e)&&U(e),i=(r||e).length,o=Array(i),a=0;a<i;a++){var s=r?r[a]:a;o[a]=t(e[s],s,e)}return o}function St(e){var t=function(t,n,r,i){var o=!tt(t)&&U(t),a=(o||t).length,s=e>0?0:a-1;for(i||(r=t[o?o[s]:s],s+=e);s>=0&&s<a;s+=e){var l=o?o[s]:s;r=n(r,t[l],l,t)}return r};return function(e,n,r,i){var o=arguments.length>=3;return t(e,Se(n,i,4),r,o)}}var Tt=St(1),Et=St(-1);function Ot(e,t,n){var r=[];return t=Oe(t,n),wt(e,(function(e,n,i){t(e,n,i)&&r.push(e)})),r}function kt(e,t,n){return Ot(e,ct(Oe(t)),n)}function Lt(e,t,n){t=Oe(t,n);for(var r=!tt(e)&&U(e),i=(r||e).length,o=0;o<i;o++){var a=r?r[o]:o;if(!t(e[a],a,e))return!1}return!0}function Dt(e,t,n){t=Oe(t,n);for(var r=!tt(e)&&U(e),i=(r||e).length,o=0;o<i;o++){var a=r?r[o]:o;if(t(e[a],a,e))return!0}return!1}function Pt(e,t,n,r){return tt(e)||(e=se(e)),("number"!=typeof n||r)&&(n=0),At(e,t,n)>=0}var Nt=o((function(e,t,n){var r,i;return A(t)?i=t:(t=Ce(t),r=t.slice(0,-1),t=t[t.length-1]),Mt(e,(function(e){var o=i;if(!o){if(r&&r.length&&(e=Ae(e,r)),null==e)return;o=e[t]}return null==o?o:o.apply(e,n)}))}));function jt(e,t){return Mt(e,Me(t))}function Rt(e,t){return Ot(e,we(t))}function zt(e,t,n){var r,i,o=-1/0,a=-1/0;if(null==t||"number"==typeof t&&"object"!=typeof e[0]&&null!=e)for(var s=0,l=(e=tt(e)?e:se(e)).length;s<l;s++)null!=(r=e[s])&&r>o&&(o=r);else t=Oe(t,n),wt(e,(function(e,n,r){((i=t(e,n,r))>a||i===-1/0&&o===-1/0)&&(o=e,a=i)}));return o}function Ft(e,t,n){var r,i,o=1/0,a=1/0;if(null==t||"number"==typeof t&&"object"!=typeof e[0]&&null!=e)for(var s=0,l=(e=tt(e)?e:se(e)).length;s<l;s++)null!=(r=e[s])&&r<o&&(o=r);else t=Oe(t,n),wt(e,(function(e,n,r){((i=t(e,n,r))<a||i===1/0&&o===1/0)&&(o=e,a=i)}));return o}function Bt(e,t,n){if(null==t||n)return tt(e)||(e=se(e)),e[Ne(e.length-1)];var r=tt(e)?me(e):se(e),i=B(r);t=Math.max(Math.min(t,i),0);for(var o=i-1,a=0;a<t;a++){var s=Ne(a,o),l=r[a];r[a]=r[s],r[s]=l}return r.slice(0,t)}function Ht(e){return Bt(e,1/0)}function Ut(e,t,n){var r=0;return t=Oe(t,n),jt(Mt(e,(function(e,n,i){return{value:e,index:r++,criteria:t(e,n,i)}})).sort((function(e,t){var n=e.criteria,r=t.criteria;if(n!==r){if(n>r||void 0===n)return 1;if(n<r||void 0===r)return-1}return e.index-t.index})),"value")}function Vt(e,t){return function(n,r,i){var o=t?[[],[]]:{};return r=Oe(r,i),wt(n,(function(t,i){var a=r(t,i,n);e(o,t,a)})),o}}var Yt=Vt((function(e,t,n){T(e,n)?e[n].push(t):e[n]=[t]})),Wt=Vt((function(e,t,n){e[n]=t})),Gt=Vt((function(e,t,n){T(e,n)?e[n]++:e[n]=1})),Xt=Vt((function(e,t,n){e[n?0:1].push(t)}),!0),qt=/[^\ud800-\udfff]|[\ud800-\udbff][\udc00-\udfff]|[\ud800-\udfff]/g;function Zt(e){return e?S(e)?i.q.call(e):h(e)?e.match(qt):tt(e)?Mt(e,xe):se(e):[]}function Qt(e){return null==e?0:tt(e)?e.length:U(e).length}function Kt(e,t,n){return t in n}var Jt=o((function(e,t){var n={},r=t[0];if(null==e)return n;A(r)?(t.length>1&&(r=Se(r,t[1])),t=Z(e)):(r=Kt,t=nt(t,!1,!1),e=Object(e));for(var i=0,o=t.length;i<o;i++){var a=t[i],s=e[a];r(s,a,e)&&(n[a]=s)}return n})),$t=o((function(e,t){var n,r=t[0];return A(r)?(r=ct(r),t.length>1&&(n=t[1])):(t=Mt(nt(t,!1,!1),String),r=function(e,n){return!Pt(t,n)}),Jt(e,r,n)}));function en(e,t,n){return i.q.call(e,0,Math.max(0,e.length-(null==t||n?1:t)))}function tn(e,t,n){return null==e||e.length<1?null==t||n?void 0:[]:null==t||n?e[0]:en(e,e.length-t)}function nn(e,t,n){return i.q.call(e,null==t||n?1:t)}function rn(e,t,n){return null==e||e.length<1?null==t||n?void 0:[]:null==t||n?e[e.length-1]:nn(e,Math.max(0,e.length-t))}function on(e){return Ot(e,Boolean)}function an(e,t){return nt(e,t,!1)}var sn=o((function(e,t){return t=nt(t,!0,!0),Ot(e,(function(e){return!Pt(t,e)}))})),ln=o((function(e,t){return sn(e,t)}));function un(e,t,n,r){u(t)||(r=n,n=t,t=!1),null!=n&&(n=Oe(n,r));for(var i=[],o=[],a=0,s=B(e);a<s;a++){var l=e[a],c=n?n(l,a,e):l;t&&!n?(a&&o===c||i.push(l),o=c):n?Pt(o,c)||(o.push(c),i.push(l)):Pt(i,l)||i.push(l)}return i}var cn=o((function(e){return un(nt(e,!0,!0))}));function dn(e){for(var t=[],n=arguments.length,r=0,i=B(e);r<i;r++){var o=e[r];if(!Pt(t,o)){var a;for(a=1;a<n&&Pt(arguments[a],o);a++);a===n&&t.push(o)}}return t}function hn(e){for(var t=e&&zt(e,B).length||0,n=Array(t),r=0;r<t;r++)n[r]=jt(e,r);return n}var fn=o(hn);function pn(e,t){for(var n={},r=0,i=B(e);r<i;r++)t?n[e[r]]=t[r]:n[e[r][0]]=e[r][1];return n}function gn(e,t,n){null==t&&(t=e||0,e=0),n||(n=t<e?-1:1);for(var r=Math.max(Math.ceil((t-e)/n),0),i=Array(r),o=0;o<r;o++,e+=n)i[o]=e;return i}function vn(e,t){if(null==t||t<1)return[];for(var n=[],r=0,o=e.length;r<o;)n.push(i.q.call(e,r,r+=t));return n}function mn(e,t){return e._chain?W(t).chain():t}function yn(e){return wt(ce(e),(function(t){var n=W[t]=e[t];W.prototype[t]=function(){var e=[this._wrapped];return i.o.apply(e,arguments),mn(this,n.apply(W,e))}})),W}wt(["pop","push","reverse","shift","sort","splice","unshift"],(function(e){var t=i.a[e];W.prototype[e]=function(){var n=this._wrapped;return null!=n&&(t.apply(n,arguments),"shift"!==e&&"splice"!==e||0!==n.length||delete n[0]),mn(this,n)}})),wt(["concat","join","slice"],(function(e){var t=i.a[e];W.prototype[e]=function(){var e=this._wrapped;return null!=e&&(e=t.apply(e,arguments)),mn(this,e)}}));var bn=W,Cn=yn(r);Cn._=Cn;var An=Cn;n.d(t,"default",(function(){return An})),n.d(t,"VERSION",(function(){return i.e})),n.d(t,"restArguments",(function(){return o})),n.d(t,"isObject",(function(){return a})),n.d(t,"isNull",(function(){return s})),n.d(t,"isUndefined",(function(){return l})),n.d(t,"isBoolean",(function(){return u})),n.d(t,"isElement",(function(){return c})),n.d(t,"isString",(function(){return h})),n.d(t,"isNumber",(function(){return f})),n.d(t,"isDate",(function(){return p})),n.d(t,"isRegExp",(function(){return g})),n.d(t,"isError",(function(){return v})),n.d(t,"isSymbol",(function(){return m})),n.d(t,"isArrayBuffer",(function(){return y})),n.d(t,"isDataView",(function(){return M})),n.d(t,"isArray",(function(){return S})),n.d(t,"isFunction",(function(){return A})),n.d(t,"isArguments",(function(){return O})),n.d(t,"isFinite",(function(){return k})),n.d(t,"isNaN",(function(){return L})),n.d(t,"isTypedArray",(function(){return F})),n.d(t,"isEmpty",(function(){return V})),n.d(t,"isMatch",(function(){return Y})),n.d(t,"isEqual",(function(){return q})),n.d(t,"isMap",(function(){return re})),n.d(t,"isWeakMap",(function(){return ie})),n.d(t,"isSet",(function(){return oe})),n.d(t,"isWeakSet",(function(){return ae})),n.d(t,"keys",(function(){return U})),n.d(t,"allKeys",(function(){return Z})),n.d(t,"values",(function(){return se})),n.d(t,"pairs",(function(){return le})),n.d(t,"invert",(function(){return ue})),n.d(t,"functions",(function(){return ce})),n.d(t,"methods",(function(){return ce})),n.d(t,"extend",(function(){return he})),n.d(t,"extendOwn",(function(){return fe})),n.d(t,"assign",(function(){return fe})),n.d(t,"defaults",(function(){return pe})),n.d(t,"create",(function(){return ve})),n.d(t,"clone",(function(){return me})),n.d(t,"tap",(function(){return ye})),n.d(t,"get",(function(){return _e})),n.d(t,"has",(function(){return Ie})),n.d(t,"mapObject",(function(){return ke})),n.d(t,"identity",(function(){return xe})),n.d(t,"constant",(function(){return D})),n.d(t,"noop",(function(){return Le})),n.d(t,"toPath",(function(){return be})),n.d(t,"property",(function(){return Me})),n.d(t,"propertyOf",(function(){return De})),n.d(t,"matcher",(function(){return we})),n.d(t,"matches",(function(){return we})),n.d(t,"times",(function(){return Pe})),n.d(t,"random",(function(){return Ne})),n.d(t,"now",(function(){return je})),n.d(t,"escape",(function(){return Fe})),n.d(t,"unescape",(function(){return Be})),n.d(t,"templateSettings",(function(){return He})),n.d(t,"template",(function(){return Ge})),n.d(t,"result",(function(){return Xe})),n.d(t,"uniqueId",(function(){return Ze})),n.d(t,"chain",(function(){return Qe})),n.d(t,"iteratee",(function(){return Ee})),n.d(t,"partial",(function(){return $e})),n.d(t,"bind",(function(){return et})),n.d(t,"bindAll",(function(){return rt})),n.d(t,"memoize",(function(){return it})),n.d(t,"delay",(function(){return ot})),n.d(t,"defer",(function(){return at})),n.d(t,"throttle",(function(){return st})),n.d(t,"debounce",(function(){return lt})),n.d(t,"wrap",(function(){return ut})),n.d(t,"negate",(function(){return ct})),n.d(t,"compose",(function(){return dt})),n.d(t,"after",(function(){return ht})),n.d(t,"before",(function(){return ft})),n.d(t,"once",(function(){return pt})),n.d(t,"findKey",(function(){return gt})),n.d(t,"findIndex",(function(){return mt})),n.d(t,"findLastIndex",(function(){return yt})),n.d(t,"sortedIndex",(function(){return bt})),n.d(t,"indexOf",(function(){return At})),n.d(t,"lastIndexOf",(function(){return _t})),n.d(t,"find",(function(){return It})),n.d(t,"detect",(function(){return It})),n.d(t,"findWhere",(function(){return xt})),n.d(t,"each",(function(){return wt})),n.d(t,"forEach",(function(){return wt})),n.d(t,"map",(function(){return Mt})),n.d(t,"collect",(function(){return Mt})),n.d(t,"reduce",(function(){return Tt})),n.d(t,"foldl",(function(){return Tt})),n.d(t,"inject",(function(){return Tt})),n.d(t,"reduceRight",(function(){return Et})),n.d(t,"foldr",(function(){return Et})),n.d(t,"filter",(function(){return Ot})),n.d(t,"select",(function(){return Ot})),n.d(t,"reject",(function(){return kt})),n.d(t,"every",(function(){return Lt})),n.d(t,"all",(function(){return Lt})),n.d(t,"some",(function(){return Dt})),n.d(t,"any",(function(){return Dt})),n.d(t,"contains",(function(){return Pt})),n.d(t,"includes",(function(){return Pt})),n.d(t,"include",(function(){return Pt})),n.d(t,"invoke",(function(){return Nt})),n.d(t,"pluck",(function(){return jt})),n.d(t,"where",(function(){return Rt})),n.d(t,"max",(function(){return zt})),n.d(t,"min",(function(){return Ft})),n.d(t,"shuffle",(function(){return Ht})),n.d(t,"sample",(function(){return Bt})),n.d(t,"sortBy",(function(){return Ut})),n.d(t,"groupBy",(function(){return Yt})),n.d(t,"indexBy",(function(){return Wt})),n.d(t,"countBy",(function(){return Gt})),n.d(t,"partition",(function(){return Xt})),n.d(t,"toArray",(function(){return Zt})),n.d(t,"size",(function(){return Qt})),n.d(t,"pick",(function(){return Jt})),n.d(t,"omit",(function(){return $t})),n.d(t,"first",(function(){return tn})),n.d(t,"head",(function(){return tn})),n.d(t,"take",(function(){return tn})),n.d(t,"initial",(function(){return en})),n.d(t,"last",(function(){return rn})),n.d(t,"rest",(function(){return nn})),n.d(t,"tail",(function(){return nn})),n.d(t,"drop",(function(){return nn})),n.d(t,"compact",(function(){return on})),n.d(t,"flatten",(function(){return an})),n.d(t,"without",(function(){return ln})),n.d(t,"uniq",(function(){return un})),n.d(t,"unique",(function(){return un})),n.d(t,"union",(function(){return cn})),n.d(t,"intersection",(function(){return dn})),n.d(t,"difference",(function(){return sn})),n.d(t,"unzip",(function(){return hn})),n.d(t,"transpose",(function(){return hn})),n.d(t,"zip",(function(){return fn})),n.d(t,"object",(function(){return pn})),n.d(t,"range",(function(){return gn})),n.d(t,"chunk",(function(){return vn})),n.d(t,"mixin",(function(){return yn}))},function(e,t,n){"use strict";n.d(t,"d",(function(){return i})),n.d(t,"c",(function(){return o})),n.d(t,"a",(function(){return a})),n.d(t,"b",(function(){return s})),n.d(t,"e",(function(){return l})),n.d(t,"g",(function(){return u})),n.d(t,"f",(function(){return c})),n.d(t,"l",(function(){return d})),n.d(t,"n",(function(){return h})),n.d(t,"m",(function(){return f})),n.d(t,"o",(function(){return p})),n.d(t,"w",(function(){return g})),n.d(t,"u",(function(){return v})),n.d(t,"s",(function(){return m})),n.d(t,"v",(function(){return y})),n.d(t,"t",(function(){return b})),n.d(t,"x",(function(){return C})),n.d(t,"r",(function(){return A})),n.d(t,"z",(function(){return _})),n.d(t,"A",(function(){return I})),n.d(t,"y",(function(){return x})),n.d(t,"C",(function(){return w})),n.d(t,"B",(function(){return M})),n.d(t,"H",(function(){return S})),n.d(t,"D",(function(){return T})),n.d(t,"E",(function(){return E})),n.d(t,"F",(function(){return O})),n.d(t,"G",(function(){return k})),n.d(t,"I",(function(){return L})),n.d(t,"J",(function(){return D})),n.d(t,"K",(function(){return P})),n.d(t,"N",(function(){return N})),n.d(t,"M",(function(){return j})),n.d(t,"L",(function(){return R})),n.d(t,"p",(function(){return z})),n.d(t,"q",(function(){return F})),n.d(t,"O",(function(){return B})),n.d(t,"P",(function(){return H})),n.d(t,"R",(function(){return U})),n.d(t,"i",(function(){return V})),n.d(t,"h",(function(){return Y})),n.d(t,"k",(function(){return W})),n.d(t,"j",(function(){return G})),n.d(t,"Q",(function(){return X}));var r="[Blueprint]",i=r+" clamp: max cannot be less than min",o=r+" <Alert> cancelButtonText and onCancel should be set together.",a=r+" <Alert> canEscapeKeyCancel enabled without onCancel or onClose handler.",s=r+" <Alert> canOutsideClickCancel enbaled without onCancel or onClose handler.",l=r+" <CollapsibleList> children must be <MenuItem>s",u=r+" @ContextMenuTarget-decorated class should implement renderContextMenu.",c=r+' "@ContextMenuTarget-decorated components must return a single JSX.Element or an empty render.',d=r+" <Hotkeys> only accepts <Hotkey> children.",h=r+" @HotkeysTarget-decorated class should implement renderHotkeys.",f=r+' "@HotkeysTarget-decorated components must return a single JSX.Element or an empty render.',p=r+" <InputGroup> leftElement and leftIcon prop are mutually exclusive, with leftElement taking priority.",g=r+" <NumericInput> requires min to be no greater than max if both are defined.",v=r+" <NumericInput> requires minorStepSize to be no greater than stepSize.",m=r+" <NumericInput> requires stepSize to be no greater than majorStepSize.",y=r+" <NumericInput> requires minorStepSize to be strictly greater than zero.",b=r+" <NumericInput> requires majorStepSize to be strictly greater than zero.",C=r+" <NumericInput> requires stepSize to be strictly greater than zero.",A=r+" <NumericInput> controlled value prop does not adhere to stepSize, min, and/or max constraints.",_=r+" <PanelStack> requires exactly one of initialPanel and stack prop",I=r+" <PanelStack> requires at least one panel in the stack",x=r+" <OverflowList> does not support changing observeParents after mounting.",w=r+" <Popover> requires target prop or at least one child element.",M=r+" <Popover hasBackdrop={true}> requires interactionKind={PopoverInteractionKind.CLICK}.",S=r+" <Popover> supports one or two children; additional children are ignored. First child is the target, second child is the content. You may instead supply these two as props.",T=r+" <Popover> with two children ignores content prop; use either prop or children.",E=r+" <Popover> with children ignores target prop; use either prop or children.",O=r+" Disabling <Popover> with empty/whitespace content...",k=r+" <Popover usePortal={false}> ignores hasBackdrop",L=r+" <Popover> onInteraction is ignored when uncontrolled.",D=r+" <Portal> context blueprintPortalClassName must be string",P=r+" <RadioGroup> children and options prop are mutually exclusive, with options taking priority.",N=r+" <Slider> stepSize must be greater than zero.",j=r+" <Slider> labelStepSize must be greater than zero.",R=r+" <RangeSlider> value prop must be an array of two non-null numbers.",z=r+" <MultiSlider> children must be <SliderHandle>s or <SliderTrackStop>s",F=r+" <MultiSlider> labelStepSize and labelValues prop are mutually exclusive, with labelStepSize taking priority.",B=r+" <Spinner> Classes.SMALL/LARGE are ignored if size prop is set.",H=r+" Toaster.create() is not supported inside React lifecycle methods in React 16. See usage example on the docs site.",U=r+" Toaster.create() ignores inline prop as it always creates a new element.",V=r+" <Dialog> iconName is ignored if title is omitted.",Y=r+" <Dialog> isCloseButtonShown prop is ignored if title is omitted.",W=r+" <Drawer> vertical is ignored if position is defined",G=r+" <Drawer> all angle positions are casted into pure position (TOP, BOTTOM, LEFT or RIGHT)",X=r+" <Toaster> maxToasts is set to an invalid number, must be greater than 0"},function(e,t,n){"use strict";n.r(t),function(e){n.d(t,"ACTIVE",(function(){return l})),n.d(t,"ALIGN_LEFT",(function(){return u})),n.d(t,"ALIGN_RIGHT",(function(){return c})),n.d(t,"DARK",(function(){return d})),n.d(t,"DISABLED",(function(){return h})),n.d(t,"FILL",(function(){return f})),n.d(t,"FIXED",(function(){return p})),n.d(t,"FIXED_TOP",(function(){return g})),n.d(t,"INLINE",(function(){return v})),n.d(t,"INTERACTIVE",(function(){return m})),n.d(t,"LARGE",(function(){return y})),n.d(t,"LOADING",(function(){return b})),n.d(t,"MINIMAL",(function(){return C})),n.d(t,"OUTLINED",(function(){return A})),n.d(t,"MULTILINE",(function(){return _})),n.d(t,"ROUND",(function(){return I})),n.d(t,"SMALL",(function(){return x})),n.d(t,"VERTICAL",(function(){return w})),n.d(t,"POSITION_TOP",(function(){return M})),n.d(t,"POSITION_BOTTOM",(function(){return S})),n.d(t,"POSITION_LEFT",(function(){return T})),n.d(t,"POSITION_RIGHT",(function(){return E})),n.d(t,"ELEVATION_0",(function(){return O})),n.d(t,"ELEVATION_1",(function(){return k})),n.d(t,"ELEVATION_2",(function(){return L})),n.d(t,"ELEVATION_3",(function(){return D})),n.d(t,"ELEVATION_4",(function(){return P})),n.d(t,"INTENT_PRIMARY",(function(){return N})),n.d(t,"INTENT_SUCCESS",(function(){return j})),n.d(t,"INTENT_WARNING",(function(){return R})),n.d(t,"INTENT_DANGER",(function(){return z})),n.d(t,"FOCUS_DISABLED",(function(){return F})),n.d(t,"UI_TEXT",(function(){return B})),n.d(t,"RUNNING_TEXT",(function(){return H})),n.d(t,"MONOSPACE_TEXT",(function(){return U})),n.d(t,"TEXT_LARGE",(function(){return V})),n.d(t,"TEXT_SMALL",(function(){return Y})),n.d(t,"TEXT_MUTED",(function(){return W})),n.d(t,"TEXT_DISABLED",(function(){return G})),n.d(t,"TEXT_OVERFLOW_ELLIPSIS",(function(){return X})),n.d(t,"BLOCKQUOTE",(function(){return q})),n.d(t,"CODE",(function(){return Z})),n.d(t,"CODE_BLOCK",(function(){return Q})),n.d(t,"HEADING",(function(){return K})),n.d(t,"LIST",(function(){return J})),n.d(t,"LIST_UNSTYLED",(function(){return $})),n.d(t,"RTL",(function(){return ee})),n.d(t,"ALERT",(function(){return te})),n.d(t,"ALERT_BODY",(function(){return ne})),n.d(t,"ALERT_CONTENTS",(function(){return re})),n.d(t,"ALERT_FOOTER",(function(){return ie})),n.d(t,"BREADCRUMB",(function(){return oe})),n.d(t,"BREADCRUMB_CURRENT",(function(){return ae})),n.d(t,"BREADCRUMBS",(function(){return se})),n.d(t,"BREADCRUMBS_COLLAPSED",(function(){return le})),n.d(t,"BUTTON",(function(){return ue})),n.d(t,"BUTTON_GROUP",(function(){return ce})),n.d(t,"BUTTON_SPINNER",(function(){return de})),n.d(t,"BUTTON_TEXT",(function(){return he})),n.d(t,"CALLOUT",(function(){return fe})),n.d(t,"CALLOUT_ICON",(function(){return pe})),n.d(t,"CARD",(function(){return ge})),n.d(t,"COLLAPSE",(function(){return ve})),n.d(t,"COLLAPSE_BODY",(function(){return me})),n.d(t,"COLLAPSIBLE_LIST",(function(){return ye})),n.d(t,"CONTEXT_MENU",(function(){return be})),n.d(t,"CONTEXT_MENU_POPOVER_TARGET",(function(){return Ce})),n.d(t,"CONTROL_GROUP",(function(){return Ae})),n.d(t,"DIALOG",(function(){return _e})),n.d(t,"DIALOG_CONTAINER",(function(){return Ie})),n.d(t,"DIALOG_BODY",(function(){return xe})),n.d(t,"DIALOG_CLOSE_BUTTON",(function(){return we})),n.d(t,"DIALOG_FOOTER",(function(){return Me})),n.d(t,"DIALOG_FOOTER_ACTIONS",(function(){return Se})),n.d(t,"DIALOG_HEADER",(function(){return Te})),n.d(t,"DIALOG_STEP",(function(){return Ee})),n.d(t,"DIALOG_STEP_CONTAINER",(function(){return Oe})),n.d(t,"DIALOG_STEP_TITLE",(function(){return ke})),n.d(t,"DIALOG_STEP_ICON",(function(){return Le})),n.d(t,"DIVIDER",(function(){return De})),n.d(t,"DRAWER",(function(){return Pe})),n.d(t,"DRAWER_BODY",(function(){return Ne})),n.d(t,"DRAWER_FOOTER",(function(){return je})),n.d(t,"DRAWER_HEADER",(function(){return Re})),n.d(t,"EDITABLE_TEXT",(function(){return ze})),n.d(t,"EDITABLE_TEXT_CONTENT",(function(){return Fe})),n.d(t,"EDITABLE_TEXT_EDITING",(function(){return Be})),n.d(t,"EDITABLE_TEXT_INPUT",(function(){return He})),n.d(t,"EDITABLE_TEXT_PLACEHOLDER",(function(){return Ue})),n.d(t,"FLEX_EXPANDER",(function(){return Ve})),n.d(t,"HTML_SELECT",(function(){return Ye})),n.d(t,"SELECT",(function(){return We})),n.d(t,"HTML_TABLE",(function(){return Ge})),n.d(t,"HTML_TABLE_BORDERED",(function(){return Xe})),n.d(t,"HTML_TABLE_CONDENSED",(function(){return qe})),n.d(t,"HTML_TABLE_STRIPED",(function(){return Ze})),n.d(t,"INPUT",(function(){return Qe})),n.d(t,"INPUT_GHOST",(function(){return Ke})),n.d(t,"INPUT_GROUP",(function(){return Je})),n.d(t,"INPUT_LEFT_CONTAINER",(function(){return $e})),n.d(t,"INPUT_ACTION",(function(){return et})),n.d(t,"CONTROL",(function(){return tt})),n.d(t,"CONTROL_INDICATOR",(function(){return nt})),n.d(t,"CONTROL_INDICATOR_CHILD",(function(){return rt})),n.d(t,"CHECKBOX",(function(){return it})),n.d(t,"RADIO",(function(){return ot})),n.d(t,"SWITCH",(function(){return at})),n.d(t,"SWITCH_INNER_TEXT",(function(){return st})),n.d(t,"FILE_INPUT",(function(){return lt})),n.d(t,"FILE_INPUT_HAS_SELECTION",(function(){return ut})),n.d(t,"FILE_UPLOAD_INPUT",(function(){return ct})),n.d(t,"FILE_UPLOAD_INPUT_CUSTOM_TEXT",(function(){return dt})),n.d(t,"KEY",(function(){return ht})),n.d(t,"KEY_COMBO",(function(){return ft})),n.d(t,"MODIFIER_KEY",(function(){return pt})),n.d(t,"HOTKEY",(function(){return gt})),n.d(t,"HOTKEY_LABEL",(function(){return vt})),n.d(t,"HOTKEY_COLUMN",(function(){return mt})),n.d(t,"HOTKEY_DIALOG",(function(){return yt})),n.d(t,"LABEL",(function(){return bt})),n.d(t,"FORM_GROUP",(function(){return Ct})),n.d(t,"FORM_CONTENT",(function(){return At})),n.d(t,"FORM_HELPER_TEXT",(function(){return _t})),n.d(t,"MENU",(function(){return It})),n.d(t,"MENU_ITEM",(function(){return xt})),n.d(t,"MENU_ITEM_LABEL",(function(){return wt})),n.d(t,"MENU_SUBMENU",(function(){return Mt})),n.d(t,"MENU_DIVIDER",(function(){return St})),n.d(t,"MENU_HEADER",(function(){return Tt})),n.d(t,"MULTISTEP_DIALOG",(function(){return Et})),n.d(t,"MULTISTEP_DIALOG_PANELS",(function(){return Ot})),n.d(t,"MULTISTEP_DIALOG_LEFT_PANEL",(function(){return kt})),n.d(t,"MULTISTEP_DIALOG_RIGHT_PANEL",(function(){return Lt})),n.d(t,"MULTISTEP_DIALOG_FOOTER",(function(){return Dt})),n.d(t,"NAVBAR",(function(){return Pt})),n.d(t,"NAVBAR_GROUP",(function(){return Nt})),n.d(t,"NAVBAR_HEADING",(function(){return jt})),n.d(t,"NAVBAR_DIVIDER",(function(){return Rt})),n.d(t,"NON_IDEAL_STATE",(function(){return zt})),n.d(t,"NON_IDEAL_STATE_VISUAL",(function(){return Ft})),n.d(t,"NUMERIC_INPUT",(function(){return Bt})),n.d(t,"OVERFLOW_LIST",(function(){return Ht})),n.d(t,"OVERFLOW_LIST_SPACER",(function(){return Ut})),n.d(t,"OVERLAY",(function(){return Vt})),n.d(t,"OVERLAY_BACKDROP",(function(){return Yt})),n.d(t,"OVERLAY_CONTAINER",(function(){return Wt})),n.d(t,"OVERLAY_CONTENT",(function(){return Gt})),n.d(t,"OVERLAY_INLINE",(function(){return Xt})),n.d(t,"OVERLAY_OPEN",(function(){return qt})),n.d(t,"OVERLAY_SCROLL_CONTAINER",(function(){return Zt})),n.d(t,"PANEL_STACK",(function(){return Qt})),n.d(t,"PANEL_STACK_HEADER",(function(){return Kt})),n.d(t,"PANEL_STACK_HEADER_BACK",(function(){return Jt})),n.d(t,"PANEL_STACK_VIEW",(function(){return $t})),n.d(t,"POPOVER",(function(){return en})),n.d(t,"POPOVER_ARROW",(function(){return tn})),n.d(t,"POPOVER_BACKDROP",(function(){return nn})),n.d(t,"POPOVER_CAPTURING_DISMISS",(function(){return rn})),n.d(t,"POPOVER_CONTENT",(function(){return on})),n.d(t,"POPOVER_CONTENT_SIZING",(function(){return an})),n.d(t,"POPOVER_DISMISS",(function(){return sn})),n.d(t,"POPOVER_DISMISS_OVERRIDE",(function(){return ln})),n.d(t,"POPOVER_OPEN",(function(){return un})),n.d(t,"POPOVER_TARGET",(function(){return cn})),n.d(t,"POPOVER_WRAPPER",(function(){return dn})),n.d(t,"TRANSITION_CONTAINER",(function(){return hn})),n.d(t,"PROGRESS_BAR",(function(){return fn})),n.d(t,"PROGRESS_METER",(function(){return pn})),n.d(t,"PROGRESS_NO_STRIPES",(function(){return gn})),n.d(t,"PROGRESS_NO_ANIMATION",(function(){return vn})),n.d(t,"PORTAL",(function(){return mn})),n.d(t,"SKELETON",(function(){return yn})),n.d(t,"SLIDER",(function(){return bn})),n.d(t,"SLIDER_AXIS",(function(){return Cn})),n.d(t,"SLIDER_HANDLE",(function(){return An})),n.d(t,"SLIDER_LABEL",(function(){return _n})),n.d(t,"SLIDER_TRACK",(function(){return In})),n.d(t,"SLIDER_PROGRESS",(function(){return xn})),n.d(t,"START",(function(){return wn})),n.d(t,"END",(function(){return Mn})),n.d(t,"SPINNER",(function(){return Sn})),n.d(t,"SPINNER_ANIMATION",(function(){return Tn})),n.d(t,"SPINNER_HEAD",(function(){return En})),n.d(t,"SPINNER_NO_SPIN",(function(){return On})),n.d(t,"SPINNER_TRACK",(function(){return kn})),n.d(t,"TAB",(function(){return Ln})),n.d(t,"TAB_INDICATOR",(function(){return Dn})),n.d(t,"TAB_INDICATOR_WRAPPER",(function(){return Pn})),n.d(t,"TAB_LIST",(function(){return Nn})),n.d(t,"TAB_PANEL",(function(){return jn})),n.d(t,"TABS",(function(){return Rn})),n.d(t,"TAG",(function(){return zn})),n.d(t,"TAG_REMOVE",(function(){return Fn})),n.d(t,"TAG_INPUT",(function(){return Bn})),n.d(t,"TAG_INPUT_ICON",(function(){return Hn})),n.d(t,"TAG_INPUT_VALUES",(function(){return Un})),n.d(t,"TOAST",(function(){return Vn})),n.d(t,"TOAST_CONTAINER",(function(){return Yn})),n.d(t,"TOAST_MESSAGE",(function(){return Wn})),n.d(t,"TOOLTIP",(function(){return Gn})),n.d(t,"TOOLTIP_INDICATOR",(function(){return Xn})),n.d(t,"TREE",(function(){return qn})),n.d(t,"TREE_NODE",(function(){return Zn})),n.d(t,"TREE_NODE_CARET",(function(){return Qn})),n.d(t,"TREE_NODE_CARET_CLOSED",(function(){return Kn})),n.d(t,"TREE_NODE_CARET_NONE",(function(){return Jn})),n.d(t,"TREE_NODE_CARET_OPEN",(function(){return $n})),n.d(t,"TREE_NODE_CONTENT",(function(){return er})),n.d(t,"TREE_NODE_EXPANDED",(function(){return tr})),n.d(t,"TREE_NODE_ICON",(function(){return nr})),n.d(t,"TREE_NODE_LABEL",(function(){return rr})),n.d(t,"TREE_NODE_LIST",(function(){return ir})),n.d(t,"TREE_NODE_SECONDARY_LABEL",(function(){return or})),n.d(t,"TREE_NODE_SELECTED",(function(){return ar})),n.d(t,"TREE_ROOT",(function(){return sr})),n.d(t,"ICON",(function(){return lr})),n.d(t,"ICON_STANDARD",(function(){return ur})),n.d(t,"ICON_LARGE",(function(){return cr})),n.d(t,"getClassNamespace",(function(){return dr})),n.d(t,"alignmentClass",(function(){return hr})),n.d(t,"elevationClass",(function(){return fr})),n.d(t,"iconClass",(function(){return pr})),n.d(t,"intentClass",(function(){return gr})),n.d(t,"positionClass",(function(){return vr}));var r=n(328),i=n(205),o=n(206),a=n(60),s=e.env.BLUEPRINT_NAMESPACE||e.env.REACT_APP_BLUEPRINT_NAMESPACE||"bp3",l=s+"-active",u=s+"-align-left",c=s+"-align-right",d=s+"-dark",h=s+"-disabled",f=s+"-fill",p=s+"-fixed",g=s+"-fixed-top",v=s+"-inline",m=s+"-interactive",y=s+"-large",b=s+"-loading",C=s+"-minimal",A=s+"-outlined",_=s+"-multiline",I=s+"-round",x=s+"-small",w=s+"-vertical",M=vr(a.a.TOP),S=vr(a.a.BOTTOM),T=vr(a.a.LEFT),E=vr(a.a.RIGHT),O=fr(i.a.ZERO),k=fr(i.a.ONE),L=fr(i.a.TWO),D=fr(i.a.THREE),P=fr(i.a.FOUR),N=gr(o.a.PRIMARY),j=gr(o.a.SUCCESS),R=gr(o.a.WARNING),z=gr(o.a.DANGER),F=s+"-focus-disabled",B=s+"-ui-text",H=s+"-running-text",U=s+"-monospace-text",V=s+"-text-large",Y=s+"-text-small",W=s+"-text-muted",G=s+"-text-disabled",X=s+"-text-overflow-ellipsis",q=s+"-blockquote",Z=s+"-code",Q=s+"-code-block",K=s+"-heading",J=s+"-list",$=s+"-list-unstyled",ee=s+"-rtl",te=s+"-alert",ne=te+"-body",re=te+"-contents",ie=te+"-footer",oe=s+"-breadcrumb",ae=oe+"-current",se=oe+"s",le=oe+"s-collapsed",ue=s+"-button",ce=ue+"-group",de=ue+"-spinner",he=ue+"-text",fe=s+"-callout",pe=fe+"-icon",ge=s+"-card",ve=s+"-collapse",me=ve+"-body",ye=s+"-collapse-list",be=s+"-context-menu",Ce=be+"-popover-target",Ae=s+"-control-group",_e=s+"-dialog",Ie=_e+"-container",xe=_e+"-body",we=_e+"-close-button",Me=_e+"-footer",Se=_e+"-footer-actions",Te=_e+"-header",Ee=s+"-dialog-step",Oe=Ee+"-container",ke=Ee+"-title",Le=Ee+"-icon",De=s+"-divider",Pe=s+"-drawer",Ne=Pe+"-body",je=Pe+"-footer",Re=Pe+"-header",ze=s+"-editable-text",Fe=ze+"-content",Be=ze+"-editing",He=ze+"-input",Ue=ze+"-placeholder",Ve=s+"-flex-expander",Ye=s+"-html-select",We=s+"-select",Ge=s+"-html-table",Xe=Ge+"-bordered",qe=Ge+"-condensed",Ze=Ge+"-striped",Qe=s+"-input",Ke=Qe+"-ghost",Je=Qe+"-group",$e=Qe+"-left-container",et=Qe+"-action",tt=s+"-control",nt=tt+"-indicator",rt=nt+"-child",it=s+"-checkbox",ot=s+"-radio",at=s+"-switch",st=at+"-inner-text",lt=s+"-file-input",ut=s+"-file-input-has-selection",ct=s+"-file-upload-input",dt=s+"-file-upload-input-custom-text",ht=s+"-key",ft=ht+"-combo",pt=s+"-modifier-key",gt=s+"-hotkey",vt=gt+"-label",mt=gt+"-column",yt=gt+"-dialog",bt=s+"-label",Ct=s+"-form-group",At=s+"-form-content",_t=s+"-form-helper-text",It=s+"-menu",xt=It+"-item",wt=xt+"-label",Mt=s+"-submenu",St=It+"-divider",Tt=It+"-header",Et=s+"-multistep-dialog",Ot=Et+"-panels",kt=Et+"-left-panel",Lt=Et+"-right-panel",Dt=Et+"-footer",Pt=s+"-navbar",Nt=Pt+"-group",jt=Pt+"-heading",Rt=Pt+"-divider",zt=s+"-non-ideal-state",Ft=zt+"-visual",Bt=s+"-numeric-input",Ht=s+"-overflow-list",Ut=Ht+"-spacer",Vt=s+"-overlay",Yt=Vt+"-backdrop",Wt=Vt+"-container",Gt=Vt+"-content",Xt=Vt+"-inline",qt=Vt+"-open",Zt=Vt+"-scroll-container",Qt=s+"-panel-stack",Kt=Qt+"-header",Jt=Qt+"-header-back",$t=Qt+"-view",en=s+"-popover",tn=en+"-arrow",nn=en+"-backdrop",rn=en+"-capturing-dismiss",on=en+"-content",an=on+"-sizing",sn=en+"-dismiss",ln=sn+"-override",un=en+"-open",cn=en+"-target",dn=en+"-wrapper",hn=s+"-transition-container",fn=s+"-progress-bar",pn=s+"-progress-meter",gn=s+"-no-stripes",vn=s+"-no-animation",mn=s+"-portal",yn=s+"-skeleton",bn=s+"-slider",Cn=bn+"-axis",An=bn+"-handle",_n=bn+"-label",In=bn+"-track",xn=bn+"-progress",wn=s+"-start",Mn=s+"-end",Sn=s+"-spinner",Tn=Sn+"-animation",En=Sn+"-head",On=s+"-no-spin",kn=Sn+"-track",Ln=s+"-tab",Dn=Ln+"-indicator",Pn=Dn+"-wrapper",Nn=Ln+"-list",jn=Ln+"-panel",Rn=Ln+"s",zn=s+"-tag",Fn=zn+"-remove",Bn=s+"-tag-input",Hn=Bn+"-icon",Un=Bn+"-values",Vn=s+"-toast",Yn=Vn+"-container",Wn=Vn+"-message",Gn=s+"-tooltip",Xn=Gn+"-indicator",qn=s+"-tree",Zn=s+"-tree-node",Qn=Zn+"-caret",Kn=Qn+"-closed",Jn=Qn+"-none",$n=Qn+"-open",er=Zn+"-content",tr=Zn+"-expanded",nr=Zn+"-icon",rr=Zn+"-label",ir=Zn+"-list",or=Zn+"-secondary-label",ar=Zn+"-selected",sr=s+"-tree-root",lr=s+"-icon",ur=lr+"-standard",cr=lr+"-large";function dr(){return s}function hr(e){switch(e){case r.a.LEFT:return u;case r.a.RIGHT:return c;default:return}}function fr(e){if(void 0!==e)return s+"-elevation-"+e}function pr(e){if(null!=e)return 0===e.indexOf(s+"-icon-")?e:s+"-icon-"+e}function gr(e){if(null!=e&&e!==o.a.NONE)return s+"-intent-"+e.toLowerCase()}function vr(e){if(void 0!==e)return s+"-position-"+e}}.call(this,n(145))},function(e,t,n){"use strict";function r(e,t){return function(){return null}}n.r(t);var i=n(442),o=n(3),a=n.n(o);var s=(a.a.element,function(){return null});s.isRequired=(a.a.element.isRequired,function(){return null});var l=s;var u=(o.elementType,function(){return null});n(21),n(1);function c(e){return e}var d=n(334),h=n(140),f=n(116),p=/^\s*function(?:\s|\s*\/\*.*\*\/\s*)+([^(\s/]*)\s*/;function g(e){var t="".concat(e).match(p);return t&&t[1]||""}function v(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:"";return e.displayName||e.name||g(e)||t}function m(e,t,n){var r=v(t);return e.displayName||(""!==r?"".concat(n,"(").concat(r,")"):n)}function y(e){if(null!=e){if("string"==typeof e)return e;if("function"==typeof e)return v(e,"Component");if("object"===Object(h.a)(e))switch(e.$$typeof){case f.ForwardRef:return m(e,e.render,"ForwardRef");case f.Memo:return m(e,e.type,"memo");default:return}}}function b(e,t,n,r,i){return null}var C="undefined"!=typeof window&&window.Math==Math?window:"undefined"!=typeof self&&self.Math==Math?self:Function("return this")(),A=a.a.oneOfType([a.a.func,a.a.object]);n.d(t,"chainPropTypes",(function(){return r})),n.d(t,"deepmerge",(function(){return i.a})),n.d(t,"elementAcceptingRef",(function(){return l})),n.d(t,"elementTypeAcceptingRef",(function(){return u})),n.d(t,"exactProp",(function(){return c})),n.d(t,"formatMuiErrorMessage",(function(){return d.a})),n.d(t,"getDisplayName",(function(){return y})),n.d(t,"HTMLElementType",(function(){return b})),n.d(t,"ponyfillGlobal",(function(){return C})),n.d(t,"refType",(function(){return A}))},function(e,t,n){"use strict";(function(e){n.d(t,"e",(function(){return r})),n.d(t,"p",(function(){return i})),n.d(t,"a",(function(){return o})),n.d(t,"c",(function(){return a})),n.d(t,"d",(function(){return s})),n.d(t,"o",(function(){return l})),n.d(t,"q",(function(){return u})),n.d(t,"t",(function(){return c})),n.d(t,"i",(function(){return d})),n.d(t,"r",(function(){return h})),n.d(t,"s",(function(){return f})),n.d(t,"k",(function(){return p})),n.d(t,"m",(function(){return g})),n.d(t,"j",(function(){return v})),n.d(t,"l",(function(){return m})),n.d(t,"g",(function(){return y})),n.d(t,"f",(function(){return b})),n.d(t,"h",(function(){return C})),n.d(t,"n",(function(){return A})),n.d(t,"b",(function(){return _}));var r="1.12.0",i="object"==typeof self&&self.self===self&&self||"object"==typeof e&&e.global===e&&e||Function("return this")()||{},o=Array.prototype,a=Object.prototype,s="undefined"!=typeof Symbol?Symbol.prototype:null,l=o.push,u=o.slice,c=a.toString,d=a.hasOwnProperty,h="undefined"!=typeof ArrayBuffer,f="undefined"!=typeof DataView,p=Array.isArray,g=Object.keys,v=Object.create,m=h&&ArrayBuffer.isView,y=isNaN,b=isFinite,C=!{toString:null}.propertyIsEnumerable("toString"),A=["valueOf","isPrototypeOf","toString","propertyIsEnumerable","hasOwnProperty","toLocaleString"],_=Math.pow(2,53)-1}).call(this,n(61))},function(e,t){e.exports=function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}},function(e,t){function n(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}e.exports=function(e,t,r){return t&&n(e.prototype,t),r&&n(e,r),e}},function(e,t,n){"use strict";n.r(t);var r,i=n(437),o=n(17),a=n(1200),s=n.n(a),l=n(623),u=n.n(l),c=(r=function(e,t){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(e,t)},function(e,t){function n(){this.constructor=e}r(e,t),e.prototype=null===t?Object.create(t):(n.prototype=t.prototype,new n)}),d=function(){return(d=Object.assign||function(e){for(var t,n=1,r=arguments.length;n<r;n++)for(var i in t=arguments[n])Object.prototype.hasOwnProperty.call(t,i)&&(e[i]=t[i]);return e}).apply(this,arguments)};function h(e,t){return e.filter((function(e){return-1===t.indexOf(e)}))}function f(e,t){return s()(e,t)}var p=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e};function g(){return o.UUID.uuid4()}var v=function(e){function t(n,r){var i=e.call(this,n)||this;return console.warn("WrappedError has been deprecated!"),i.error_stack=r instanceof t?r.error_stack:[r],i.error_stack.push(i),i}return c(t,e),t}(Error);function m(e){var t=Object.keys(e),n=[];return t.forEach((function(t){n.push(e[t])})),Promise.all(n).then((function(e){for(var n={},r=0;r<t.length;r++)n[t[r]]=e[r];return n}))}function y(e,t){return function(n){throw t&&console.error(new Error(e)),n}}function b(e,t,n){for(var r=0;r<t.length;r++){for(var i=t[r],o=e,a=0;a<i.length-1;a++)o=o[i[a]];o[i[i.length-1]]=n[r]}}function C(e){var t=[],n=[];return{state:function e(r,i){if(r.toJSON&&(r=r.toJSON()),Array.isArray(r))for(var o=!1,a=0;a<r.length;a++){if(l=r[a])if(l instanceof ArrayBuffer||ArrayBuffer.isView(l))o||(r=r.slice(),o=!0),t.push(ArrayBuffer.isView(l)?l.buffer:l),n.push(i.concat([a])),r[a]=null;else(c=e(l,i.concat([a])))!==l&&(o||(r=r.slice(),o=!0),r[a]=c)}else if(u()(r))for(var s in r){var l,c;o=!1;if(r.hasOwnProperty(s))if(l=r[s])if(l instanceof ArrayBuffer||ArrayBuffer.isView(l))o||(r=d({},r),o=!0),t.push(ArrayBuffer.isView(l)?l.buffer:l),n.push(i.concat([s])),delete r[s];else(c=e(l,i.concat([s])))!==l&&(o||(r=d({},r),o=!0),r[s]=c)}return r}(e,[]),buffers:t,buffer_paths:n}}var A=["00","01","02","03","04","05","06","07","08","09","0A","0B","0C","0D","0E","0F","10","11","12","13","14","15","16","17","18","19","1A","1B","1C","1D","1E","1F","20","21","22","23","24","25","26","27","28","29","2A","2B","2C","2D","2E","2F","30","31","32","33","34","35","36","37","38","39","3A","3B","3C","3D","3E","3F","40","41","42","43","44","45","46","47","48","49","4A","4B","4C","4D","4E","4F","50","51","52","53","54","55","56","57","58","59","5A","5B","5C","5D","5E","5F","60","61","62","63","64","65","66","67","68","69","6A","6B","6C","6D","6E","6F","70","71","72","73","74","75","76","77","78","79","7A","7B","7C","7D","7E","7F","80","81","82","83","84","85","86","87","88","89","8A","8B","8C","8D","8E","8F","90","91","92","93","94","95","96","97","98","99","9A","9B","9C","9D","9E","9F","A0","A1","A2","A3","A4","A5","A6","A7","A8","A9","AA","AB","AC","AD","AE","AF","B0","B1","B2","B3","B4","B5","B6","B7","B8","B9","BA","BB","BC","BD","BE","BF","C0","C1","C2","C3","C4","C5","C6","C7","C8","C9","CA","CB","CC","CD","CE","CF","D0","D1","D2","D3","D4","D5","D6","D7","D8","D9","DA","DB","DC","DD","DE","DF","E0","E1","E2","E3","E4","E5","E6","E7","E8","E9","EA","EB","EC","ED","EE","EF","F0","F1","F2","F3","F4","F5","F6","F7","F8","F9","FA","FB","FC","FD","FE","FF"];function _(e){for(var t=new Uint8Array(e),n=[],r=0;r<t.length;r++)n.push(A[t[r]]);return n.join("")}function I(e){for(var t=new Uint8Array(e.length/2),n=0;n<e.length;n+=2)t[n/2]=parseInt(e.slice(n,n+2),16);return t.buffer}function x(e){return Object(i.fromByteArray)(new Uint8Array(e))}function w(e){return Object(i.toByteArray)(e).buffer}var M=function(){return(M=Object.assign||function(e){for(var t,n=1,r=arguments.length;n<r;n++)for(var i in t=arguments[n])Object.prototype.hasOwnProperty.call(t,i)&&(e[i]=t[i]);return e}).apply(this,arguments)},S=function(e,t,n,r){return new(n||(n=Promise))((function(i,o){function a(e){try{l(r.next(e))}catch(e){o(e)}}function s(e){try{l(r.throw(e))}catch(e){o(e)}}function l(e){var t;e.done?i(e.value):(t=e.value,t instanceof n?t:new n((function(e){e(t)}))).then(a,s)}l((r=r.apply(e,t||[])).next())}))},T=function(e,t){var n,r,i,o,a={label:0,sent:function(){if(1&i[0])throw i[1];return i[1]},trys:[],ops:[]};return o={next:s(0),throw:s(1),return:s(2)},"function"==typeof Symbol&&(o[Symbol.iterator]=function(){return this}),o;function s(o){return function(s){return function(o){if(n)throw new TypeError("Generator is already executing.");for(;a;)try{if(n=1,r&&(i=2&o[0]?r.return:o[0]?r.throw||((i=r.return)&&i.call(r),0):r.next)&&!(i=i.call(r,o[1])).done)return i;switch(r=0,i&&(o=[2&o[0],i.value]),o[0]){case 0:case 1:i=o;break;case 4:return a.label++,{value:o[1],done:!1};case 5:a.label++,r=o[1],o=[0];continue;case 7:o=a.ops.pop(),a.trys.pop();continue;default:if(!(i=a.trys,(i=i.length>0&&i[i.length-1])||6!==o[0]&&2!==o[0])){a=0;continue}if(3===o[0]&&(!i||o[1]>i[0]&&o[1]<i[3])){a.label=o[1];break}if(6===o[0]&&a.label<i[1]){a.label=i[1],i=o;break}if(i&&a.label<i[2]){a.label=i[2],a.ops.push(o);break}i[2]&&a.ops.pop(),a.trys.pop();continue}o=t.call(e,a)}catch(e){o=[6,e],r=0}finally{n=i=0}if(5&o[0])throw o[1];return{value:o[0]?o[1]:void 0,done:!0}}([o,s])}}},E="2.0.0".split(".",1)[0],O=function(){function e(){this.comm_target_name="jupyter.widget",this._models=Object.create(null)}return e.prototype.display_model=function(e,t,n){var r=this;return void 0===n&&(n={}),this.create_view(t,n).then((function(t){return r.display_view(e,t,n)})).catch(y("Could not create view",!0))},e.prototype.setViewOptions=function(e){return void 0===e&&(e={}),e},e.prototype.create_view=function(e,t){var n=this;void 0===t&&(t={});var r=e.state_change=e.state_change.then((function(){return n.loadClass(e.get("_view_name"),e.get("_view_module"),e.get("_view_module_version")).then((function(r){var i=new r({model:e,options:n.setViewOptions(t)});return i.listenTo(e,"destroy",i.remove),Promise.resolve(i.render()).then((function(){return i}))})).catch(y("Could not create a view for model id "+e.model_id,!0))})),i=g();return e.views[i]=r,r.then((function(e){e.once("remove",(function(){delete e.model.views[i]}),n)})),e.state_change},e.prototype.callbacks=function(e){return{}},e.prototype.get_model=function(e){return this._models[e]},e.prototype.handle_comm_open=function(e,t){var n=(t.metadata||{}).version||"";if(n.split(".",1)[0]!==E){var r="Wrong widget protocol version: received protocol version '"+n+"', but was expecting major version '"+E+"'";return console.error(r),Promise.reject(r)}var i=t.content.data,o=i.buffer_paths||[],a=(t.buffers||[]).map((function(e){return e instanceof DataView?e:new DataView(e instanceof ArrayBuffer?e:e.buffer)}));return b(i.state,o,a),this.new_model({model_name:i.state._model_name,model_module:i.state._model_module,model_module_version:i.state._model_module_version,comm:e},i.state).catch(y("Could not create a model.",!0))},e.prototype.new_widget=function(e,t){var n,r=this;if(void 0===t&&(t={}),void 0===e.view_name||void 0===e.view_module||void 0===e.view_module_version)return Promise.reject("new_widget(...) must be given view information in the options.");n=e.comm?Promise.resolve(e.comm):this._create_comm(this.comm_target_name,e.model_id,{state:{_model_module:e.model_module,_model_module_version:e.model_module_version,_model_name:e.model_name,_view_module:e.view_module,_view_module_version:e.view_module_version,_view_name:e.view_name}},{version:"2.0.0"});var i=M({},e);return n.then((function(e){return i.comm=e,r.new_model(i,t).then((function(e){return e.sync("create",e),e}))}),(function(){return i.model_id||(i.model_id=g()),r.new_model(i,t)}))},e.prototype.register_model=function(e,t){var n=this;this._models[e]=t,t.then((function(t){t.once("comm:close",(function(){delete n._models[e]}))}))},e.prototype.new_model=function(e,t){return void 0===t&&(t={}),S(this,void 0,void 0,(function(){var n,r;return T(this,(function(i){switch(i.label){case 0:if(e.model_id)n=e.model_id;else{if(!e.comm)throw new Error("Neither comm nor model_id provided in options object. At least one must exist.");n=e.model_id=e.comm.comm_id}return r=this._make_model(e,t),this.register_model(n,r),[4,r];case 1:return[2,i.sent()]}}))}))},e.prototype._make_model=function(e,t){return void 0===t&&(t={}),S(this,void 0,void 0,(function(){var n,r,i,o,a,s,l;return T(this,(function(u){switch(u.label){case 0:n=e.model_id,r=this.loadClass(e.model_name,e.model_module,e.model_module_version),u.label=1;case 1:return u.trys.push([1,3,,4]),[4,r];case 2:return i=u.sent(),[3,4];case 3:throw o=u.sent(),console.error("Could not instantiate widget"),o;case 4:if(!i)throw new Error("Cannot find model module "+e.model_module+"@"+e.model_module_version+", "+e.model_name);return[4,i._deserialize_state(t,this)];case 5:return a=u.sent(),s={widget_manager:this,model_id:n,comm:e.comm},(l=new i(a,s)).name=e.model_name,l.module=e.model_module,[2,l]}}))}))},e.prototype.clear_state=function(){var e=this;return m(this._models).then((function(t){Object.keys(t).forEach((function(e){return t[e].close()})),e._models=Object.create(null)}))},e.prototype.get_state=function(e){var t=this;void 0===e&&(e={});var n=Object.keys(this._models).map((function(e){return t._models[e]}));return Promise.all(n).then((function(t){return k(t,e)}))},e.prototype.set_state=function(e){var t=this;if(!(e.version_major&&e.version_major<=2))throw"Unsupported widget state format";var n=e.state;return this._get_comm_info().then((function(e){return Promise.all(Object.keys(n).map((function(r){var i={base64:w,hex:I},o=n[r],a=o.state;if(o.buffers){var s=o.buffers.map((function(e){return e.path})),l=o.buffers.map((function(e){return new DataView(i[e.encoding](e.data))}));b(o.state,s,l)}if(t._models[r])return t._models[r].then((function(e){return e.constructor._deserialize_state(a||{},t).then((function(t){return e.set_state(t),e}))}));var u={model_id:r,model_name:o.model_name,model_module:o.model_module,model_module_version:o.model_module_version};return e.hasOwnProperty(r)?t._create_comm(t.comm_target_name,r).then((function(e){return u.comm=e,t.new_model(u)})):t.new_model(u,a)})))}))},e.prototype.disconnect=function(){var e=this;Object.keys(this._models).forEach((function(t){e._models[t].then((function(e){e.comm_live=!1}))}))},e.prototype.resolveUrl=function(e){return Promise.resolve(e)},e.prototype.filterExistingModelState=function(e){var t=this,n=e.state;return n=Object.keys(n).filter((function(e){return!t._models[e]})).reduce((function(e,t){return e[t]=n[t],e}),{}),M(M({},e),{state:n})},e}();function k(e,t){void 0===t&&(t={});var n={};return e.forEach((function(e){var r=e.model_id,i=C(e.serialize(e.get_state(t.drop_defaults))),o=i.buffers.map((function(e,t){return{data:x(e),path:i.buffer_paths[t],encoding:"base64"}}));n[r]={model_name:e.name,model_module:e.module,model_module_version:e.get("_model_module_version"),state:i.state},o.length>0&&(n[r].buffers=o)})),{version_major:2,version_minor:0,state:n}}var L=function(){return(L=Object.assign||function(e){for(var t,n=1,r=arguments.length;n<r;n++)for(var i in t=arguments[n])Object.prototype.hasOwnProperty.call(t,i)&&(e[i]=t[i]);return e}).apply(this,arguments)};function D(e,t,n){if(null==e)return this;var r;if("object"==typeof e?(r=e,n=t):(r={})[e]=t,n||(n={}),!this._validate(r,n))return!1;var i=n.unset,o=n.silent,a=[],s=this._changing;this._changing=!0,s||(this._previousAttributes=L({},this.attributes),this.changed={});var l=this.attributes,u=this.changed,c=this._previousAttributes;for(var d in r)t=r[d],f(l[d],t)||a.push(d),f(c[d],t)?delete u[d]:u[d]=t,i?delete l[d]:l[d]=t;if(this.id=this.get(this.idAttribute),!o){a.length&&(this._pending=n);for(var h=0;h<a.length;h++)this.trigger("change:"+a[h],this,l[a[h]],n)}if(s)return this;if(!o)for(;this._pending;)n=this._pending,this._pending=!1,this.trigger("change",this,n);return this._pending=!1,this._changing=!1,this}var P=n(438),N=n(71),j=n.n(N),R=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),z=Element.prototype,F=z.matches||z.webkitMatchesSelector||z.mozMatchesSelector||z.msMatchesSelector||z.oMatchesSelector||function(e){for(var t=(this.document||this.ownerDocument).querySelectorAll(e),n=t.length;--n>=0&&t.item(n)!==this;);return n>-1},B=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return R(t,e),t.prototype._removeElement=function(){this.undelegateEvents(),this.el.parentNode&&this.el.parentNode.removeChild(this.el)},t.prototype._setElement=function(e){this.el=e},t.prototype._setAttributes=function(e){for(var t in e)t in this.el?this.el[t]=e[t]:this.el.setAttribute(t,e[t])},t.prototype.delegate=function(e,t,n){"string"!=typeof t&&(n=t,t=null),void 0===this._domEvents&&(this._domEvents=[]);var r=this.el,i=t?function(e){for(var i=e.target||e.srcElement;i&&i!==r;i=i.parentNode)if(F.call(i,t))return e.delegateTarget=i,n.handleEvent?n.handleEvent(e):n(e)}:n;return this.el.addEventListener(e,i,!1),this._domEvents.push({eventName:e,handler:i,listener:n,selector:t}),i},t.prototype.undelegate=function(e,t,n){if("function"==typeof t&&(n=t,t=null),this.el&&this._domEvents)for(var r=this._domEvents.slice(),i=r.length;i--;){var o=r[i];!(o.eventName!==e||n&&o.listener!==n||t&&o.selector!==t)&&(this.el.removeEventListener(o.eventName,o.handler,!1),this._domEvents.splice(i,1))}return this},t.prototype.undelegateEvents=function(){if(this.el&&this._domEvents){for(var e=this._domEvents.length,t=0;t<e;t++){var n=this._domEvents[t];this.el.removeEventListener(n.eventName,n.handler,!1)}this._domEvents.length=0}return this},t}(P.View),H=n(18),U=n(14),V=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),Y=function(){return(Y=Object.assign||function(e){for(var t,n=1,r=arguments.length;n<r;n++)for(var i in t=arguments[n])Object.prototype.hasOwnProperty.call(t,i)&&(e[i]=t[i]);return e}).apply(this,arguments)};function W(e,t){if(Array.isArray(e)){var n=[];return e.forEach((function(e,r){n.push(W(e,t))})),Promise.all(n)}if(e instanceof Object){var r={};return Object.keys(e).forEach((function(n){r[n]=W(e[n],t)})),m(r)}return"string"==typeof e&&"IPY_MODEL_"===e.slice(0,10)?t.get_model(e.slice(10,e.length)):Promise.resolve(e)}var G,X=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return V(t,e),t.prototype.defaults=function(){return{_model_module:"@jupyter-widgets/base",_model_name:"WidgetModel",_model_module_version:"1.2.0",_view_module:"@jupyter-widgets/base",_view_name:null,_view_module_version:"1.2.0",_view_count:null}},t.prototype.isNew=function(){return!1},t.prototype.initialize=function(t,n){e.prototype.initialize.call(this,t,n),this.widget_manager=n.widget_manager,this.model_id=n.model_id;var r=n.comm;this.views=Object.create(null),this.state_change=Promise.resolve(),this._closed=!1,this._state_lock=null,this._msg_buffer=null,this._msg_buffer_callbacks=null,this._pending_msgs=0,this._buffered_state_diff={},r?(this.comm=r,r.on_close(this._handle_comm_closed.bind(this)),r.on_msg(this._handle_comm_msg.bind(this)),this.comm_live=!0):this.comm_live=!1},Object.defineProperty(t.prototype,"comm_live",{get:function(){return this._comm_live},set:function(e){this._comm_live=e,this.trigger("comm_live_update")},enumerable:!0,configurable:!0}),t.prototype.send=function(e,t,n){if(void 0!==this.comm){var r={method:"custom",content:e};this.comm.send(r,t,{},n)}},t.prototype.close=function(e){var t=this;if(void 0===e&&(e=!1),!this._closed){this._closed=!0,this.comm&&!e&&this.comm.close(),this.stopListening(),this.trigger("destroy",this),this.comm&&delete this.comm;var n=Object.keys(this.views).map((function(e){return t.views[e].then((function(e){return e.remove()}))}));return delete this.views,Promise.all(n).then((function(){}))}},t.prototype._handle_comm_closed=function(e){this.trigger("comm:close"),this.close(!0)},t.prototype._handle_comm_msg=function(e){var t=this,n=e.content.data;switch(n.method){case"update":return this.state_change=this.state_change.then((function(){var r=n.state;return b(r,n.buffer_paths||[],(e.buffers||[]).map((function(e){return e instanceof DataView?e:new DataView(e instanceof ArrayBuffer?e:e.buffer)}))),t.constructor._deserialize_state(r,t.widget_manager)})).then((function(e){t.set_state(e)})).catch(y("Could not process update msg for model id: "+this.model_id,!0)),this.state_change;case"custom":return this.trigger("msg:custom",n.content,e.buffers),Promise.resolve()}},t.prototype.set_state=function(e){this._state_lock=e;try{this.set(e)}catch(e){console.error("Error setting state: "+e.message)}finally{this._state_lock=null}},t.prototype.get_state=function(e){var t=this.attributes;if(e){var n=this.defaults,r="function"==typeof n?n.call(this):n,i={};return Object.keys(t).forEach((function(e){f(t[e],r[e])||(i[e]=t[e])})),i}return Y({},t)},t.prototype._handle_status=function(e){void 0!==this.comm&&"idle"===e.content.execution_state&&(this._pending_msgs--,null!==this._msg_buffer&&this._pending_msgs<1&&(this.send_sync_message(this._msg_buffer,this._msg_buffer_callbacks),this._msg_buffer=null,this._msg_buffer_callbacks=null))},t.prototype.callbacks=function(e){return this.widget_manager.callbacks(e)},t.prototype.set=function(e,t,n){var r=D.call(this,e,t,n);if(void 0!==this._buffered_state_diff){var i=this.changedAttributes()||{};if(this._state_lock)for(var o=0,a=Object.keys(this._state_lock);o<a.length;o++){var s=a[o];i[s]===this._state_lock[s]&&delete i[s]}this._buffered_state_diff=p(this._buffered_state_diff,i)}return r},t.prototype.sync=function(e,t,n){if(void 0===n&&(n={}),void 0===this.comm)throw"Syncing error: no comm channel defined";var r="patch"===e?n.attrs:t.get_state(n.drop_defaults);if(this._state_lock)for(var i=0,o=Object.keys(this._state_lock);i<o.length;i++){var a=o[i];r[a]===this._state_lock[a]&&delete r[a]}var s=this.serialize(r);if(Object.keys(s).length>0){var l=n.callbacks||this.callbacks();if(this._pending_msgs>=1){switch(e){case"patch":this._msg_buffer=p(this._msg_buffer||{},s);break;case"update":case"create":this._msg_buffer=s;break;default:throw"unrecognized syncing method"}this._msg_buffer_callbacks=l}else this.send_sync_message(r,l)}},t.prototype.serialize=function(e){for(var t=this.constructor.serializers||{},n=0,r=Object.keys(e);n<r.length;n++){var i=r[n];try{t[i]&&t[i].serialize?e[i]=t[i].serialize(e[i],this):e[i]=JSON.parse(JSON.stringify(e[i])),e[i]&&e[i].toJSON&&(e[i]=e[i].toJSON())}catch(e){throw console.error("Error serializing widget state attribute: ",i),e}}return e},t.prototype.send_sync_message=function(e,t){var n=this;void 0===t&&(t={});try{t.iopub=t.iopub||{};var r=t.iopub.status;t.iopub.status=function(e){n._handle_status(e),r&&r(e)};var i=C(e);this.comm.send({method:"update",state:i.state,buffer_paths:i.buffer_paths},t,{},i.buffers),this._pending_msgs++}catch(e){console.error("Could not send widget sync message",e)}},t.prototype.save_changes=function(e){if(this.comm_live){var t={patch:!0};e&&(t.callbacks=e),this.save(this._buffered_state_diff,t),this._buffered_state_diff={}}},t.prototype.on_some_change=function(e,t,n){var r=this;this.on("change",(function(){e.some(r.hasChanged,r)&&t.apply(n,arguments)}),this)},t.prototype.toJSON=function(e){return"IPY_MODEL_"+this.model_id},t._deserialize_state=function(e,t){var n,r=this.serializers;if(r)for(var i in n={},e)r[i]&&r[i].deserialize?n[i]=r[i].deserialize(e[i],t):n[i]=e[i];else n=e;return m(n)},t}(P.Model),q=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return V(t,e),t.prototype.defaults=function(){return p(e.prototype.defaults.call(this),{_dom_classes:[]})},t.serializers=Y(Y({},X.serializers),{layout:{deserialize:W},style:{deserialize:W}}),t}(X),Z=function(e){function t(t){return e.call(this,t)||this}return V(t,e),t.prototype.initialize=function(e){var t=this;this.listenTo(this.model,"change",(function(){var e=Object.keys(t.model.changedAttributes()||{});"_view_count"===e[0]&&1===e.length||t.update()})),this.options=e.options,this.once("remove",(function(){"number"==typeof t.model.get("_view_count")&&(t.model.set("_view_count",t.model.get("_view_count")-1),t.model.save_changes())})),this.once("displayed",(function(){"number"==typeof t.model.get("_view_count")&&(t.model.set("_view_count",t.model.get("_view_count")+1),t.model.save_changes())})),this.displayed=new Promise((function(e,n){t.once("displayed",e)}))},t.prototype.update=function(e){},t.prototype.render=function(){},t.prototype.create_child_view=function(e,t){return void 0===t&&(t={}),t=Y({parent:this},t),this.model.widget_manager.create_view(e,t).catch(y("Could not create child view",!0))},t.prototype.callbacks=function(){return this.model.callbacks(this)},t.prototype.send=function(e,t){this.model.send(e,this.callbacks(),t)},t.prototype.touch=function(){this.model.save_changes(this.callbacks())},t.prototype.remove=function(){return e.prototype.remove.call(this),this.trigger("remove"),this},t}(B),Q=function(e){function t(t){var n=this,r=t.view;return delete t.view,(n=e.call(this,t)||this)._view=r,n}return V(t,e),t.prototype.dispose=function(){this.isDisposed||(e.prototype.dispose.call(this),this._view&&this._view.remove(),this._view=null)},t.prototype.processMessage=function(t){e.prototype.processMessage.call(this,t),this._view.processPhosphorMessage(t)},t}(H.Widget),K=function(e){function t(t){var n=this,r=t.view;return delete t.view,(n=e.call(this,t)||this)._view=r,n}return V(t,e),t.prototype.processMessage=function(t){e.prototype.processMessage.call(this,t),this._view.processPhosphorMessage(t)},t.prototype.dispose=function(){this.isDisposed||(e.prototype.dispose.call(this),this._view&&this._view.remove(),this._view=null)},t}(H.Panel),J=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return V(t,e),t.prototype.initialize=function(t){var n=this;e.prototype.initialize.call(this,t),this.listenTo(this.model,"change:_dom_classes",(function(e,t){var r=e.previous("_dom_classes");n.update_classes(r,t)})),this.layoutPromise=Promise.resolve(),this.listenTo(this.model,"change:layout",(function(e,t){n.setLayout(t,e.previous("layout"))})),this.stylePromise=Promise.resolve(),this.listenTo(this.model,"change:style",(function(e,t){n.setStyle(t,e.previous("style"))})),this.displayed.then((function(){n.update_classes([],n.model.get("_dom_classes")),n.setLayout(n.model.get("layout")),n.setStyle(n.model.get("style"))})),this._comm_live_update(),this.listenTo(this.model,"comm_live_update",(function(){n._comm_live_update()}))},t.prototype.setLayout=function(e,t){var n=this;e&&(this.layoutPromise=this.layoutPromise.then((function(t){return t&&(t.unlayout(),n.stopListening(t.model),t.remove()),n.create_child_view(e).then((function(e){return n.displayed.then((function(){return e.trigger("displayed"),n.listenTo(e.model,"change",(function(){U.c.postMessage(n.pWidget,H.Widget.ResizeMessage.UnknownSize)})),U.c.postMessage(n.pWidget,H.Widget.ResizeMessage.UnknownSize),e}))})).catch(y("Could not add LayoutView to DOMWidgetView",!0))})))},t.prototype.setStyle=function(e,t){var n=this;e&&(this.stylePromise=this.stylePromise.then((function(t){return t&&(t.unstyle(),n.stopListening(t.model),t.remove()),n.create_child_view(e).then((function(e){return n.displayed.then((function(){return e.trigger("displayed"),e}))})).catch(y("Could not add styleView to DOMWidgetView",!0))})))},t.prototype.update_classes=function(e,t,n){void 0===n&&(n=this.el),h(e,t).map((function(e){n.classList?n.classList.remove(e):n.setAttribute("class",n.getAttribute("class").replace(e,""))})),h(t,e).map((function(e){n.classList?n.classList.add(e):n.setAttribute("class",n.getAttribute("class").concat(" ",e))}))},t.prototype.update_mapped_classes=function(e,t,n){var r=this.model.previous(t),i=e[r]?e[r]:[],o=e[r=this.model.get(t)]?e[r]:[];this.update_classes(i,o,n||this.el)},t.prototype.set_mapped_classes=function(e,t,n){var r=this.model.get(t),i=e[r]?e[r]:[];this.update_classes([],i,n||this.el)},t.prototype._setElement=function(e){this.pWidget&&this.pWidget.dispose(),this.$el=e instanceof j.a?e:j()(e),this.el=this.$el[0],this.pWidget=new Q({node:e,view:this})},t.prototype.remove=function(){return this.pWidget&&this.pWidget.dispose(),e.prototype.remove.call(this)},t.prototype.processPhosphorMessage=function(e){switch(e.type){case"after-attach":this.trigger("displayed")}},t.prototype._comm_live_update=function(){this.model.comm_live?this.pWidget.removeClass("jupyter-widgets-disconnected"):this.pWidget.addClass("jupyter-widgets-disconnected")},t}(Z),$=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),ee={align_content:null,align_items:null,align_self:null,border:null,bottom:null,display:null,flex:null,flex_flow:null,height:null,justify_content:null,justify_items:null,left:null,margin:null,max_height:null,max_width:null,min_height:null,min_width:null,overflow:null,overflow_x:null,overflow_y:null,order:null,padding:null,right:null,top:null,visibility:null,width:null,object_fit:null,object_position:null,grid_auto_columns:null,grid_auto_flow:null,grid_auto_rows:null,grid_gap:null,grid_template_rows:null,grid_template_columns:null,grid_template_areas:null,grid_row:null,grid_column:null,grid_area:null},te=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return $(t,e),t.prototype.defaults=function(){return p(e.prototype.defaults.call(this),{_model_name:"LayoutModel",_view_name:"LayoutView"},ee)},t}(X),ne=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return $(t,e),t.prototype.initialize=function(t){this._traitNames=[],e.prototype.initialize.call(this,t);for(var n=0,r=Object.keys(ee);n<r.length;n++){var i=r[n];this.registerTrait(i)}},t.prototype.registerTrait=function(e){var t=this;if(this._traitNames.push(e),"overflow_x"===e||"overflow_y"===e)return this.listenTo(this.model,"change:"+e,(function(n,r){t.handleOverflowChange(e,r)})),void this.handleOverflowChange(e,this.model.get(e));this.listenTo(this.model,"change:"+e,(function(n,r){t.handleChange(e,r)})),this.handleChange(e,this.model.get(e))},t.prototype.css_name=function(e){return e.replace(/_/g,"-")},t.prototype.handleChange=function(e,t){var n=this.options.parent;n?null===t?n.el.style.removeProperty(this.css_name(e)):n.el.style[this.css_name(e)]=t:console.warn("Style not applied because a parent view does not exist")},t.prototype.handleOverflowChange=function(e,t){var n=this.options.parent;n?null===t?null===this.model.get("overflow")&&n.el.style.removeProperty(this.css_name(e)):n.el.style[this.css_name(e)]=t:console.warn("Style not applied because a parent view does not exist")},t.prototype.unlayout=function(){var e=this,t=this.options.parent;this._traitNames.forEach((function(n){t?t.el.style.removeProperty(e.css_name(n)):console.warn("Style not removed because a parent view does not exist")}),this)},t}(Z),re=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),ie=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return re(t,e),t.prototype.defaults=function(){var t=this.constructor;return p(e.prototype.defaults.call(this),{_model_name:"StyleModel",_view_name:"StyleView"},Object.keys(t.styleProperties).reduce((function(e,n){return e[n]=t.styleProperties[n].default,e}),{}))},t.styleProperties={},t}(X),oe=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return re(t,e),t.prototype.initialize=function(t){this._traitNames=[],e.prototype.initialize.call(this,t);for(var n=this.model.constructor,r=0,i=Object.keys(n.styleProperties);r<i.length;r++){var o=i[r];this.registerTrait(o)}this.style()},t.prototype.registerTrait=function(e){var t=this;this._traitNames.push(e),this.listenTo(this.model,"change:"+e,(function(n,r){t.handleChange(e,r)}))},t.prototype.handleChange=function(e,t){var n=this.options.parent;if(n){var r=this.model.constructor.styleProperties,i=r[e].attribute,o=r[e].selector,a=o?n.el.querySelectorAll(o):[n.el];if(null===t)for(var s=0;s!==a.length;++s)a[s].style.removeProperty(i);else for(s=0;s!==a.length;++s)a[s].style[i]=t}else console.warn("Style not applied because a parent view does not exist")},t.prototype.style=function(){for(var e=0,t=this._traitNames;e<t.length;e++){var n=t[e];this.handleChange(n,this.model.get(n))}},t.prototype.unstyle=function(){var e=this.options.parent,t=this.model.constructor.styleProperties;this._traitNames.forEach((function(n){if(e)for(var r=t[n].attribute,i=t[n].selector,o=i?e.el.querySelectorAll(i):[e.el],a=0;a!==o.length;++a)o[a].style.removeProperty(r);else console.warn("Style not removed because a parent view does not exist")}),this)},t}(Z),ae=function(e,t,n,r){return new(n||(n=Promise))((function(i,o){function a(e){try{l(r.next(e))}catch(e){o(e)}}function s(e){try{l(r.throw(e))}catch(e){o(e)}}function l(e){var t;e.done?i(e.value):(t=e.value,t instanceof n?t:new n((function(e){e(t)}))).then(a,s)}l((r=r.apply(e,t||[])).next())}))},se=function(e,t){var n,r,i,o,a={label:0,sent:function(){if(1&i[0])throw i[1];return i[1]},trys:[],ops:[]};return o={next:s(0),throw:s(1),return:s(2)},"function"==typeof Symbol&&(o[Symbol.iterator]=function(){return this}),o;function s(o){return function(s){return function(o){if(n)throw new TypeError("Generator is already executing.");for(;a;)try{if(n=1,r&&(i=2&o[0]?r.return:o[0]?r.throw||((i=r.return)&&i.call(r),0):r.next)&&!(i=i.call(r,o[1])).done)return i;switch(r=0,i&&(o=[2&o[0],i.value]),o[0]){case 0:case 1:i=o;break;case 4:return a.label++,{value:o[1],done:!1};case 5:a.label++,r=o[1],o=[0];continue;case 7:o=a.ops.pop(),a.trys.pop();continue;default:if(!(i=a.trys,(i=i.length>0&&i[i.length-1])||6!==o[0]&&2!==o[0])){a=0;continue}if(3===o[0]&&(!i||o[1]>i[0]&&o[1]<i[3])){a.label=o[1];break}if(6===o[0]&&a.label<i[1]){a.label=i[1],i=o;break}if(i&&a.label<i[2]){a.label=i[2],a.ops.push(o);break}i[2]&&a.ops.pop(),a.trys.pop();continue}o=t.call(e,a)}catch(e){o=[6,e],r=0}finally{n=i=0}if(5&o[0])throw o[1];return{value:o[0]?o[1]:void 0,done:!0}}([o,s])}}};!function(e){!function(e){var t=function(){function e(e){this.targets=Object.create(null),this.comms=Object.create(null),this.kernel=null,this.jsServicesKernel=null,this.init_kernel(e)}return e.prototype.init_kernel=function(e){this.kernel=e,this.jsServicesKernel=e},e.prototype.new_comm=function(e,t,r,i,o,a){return ae(this,void 0,void 0,(function(){var s,l;return se(this,(function(u){return s=this.jsServicesKernel.createComm(e,o),l=new n(s),this.register_comm(l),l.open(t,r,i,a),[2,l]}))}))},e.prototype.register_target=function(e,t){var r=this,i=this.jsServicesKernel.registerCommTarget(e,(function(e,i){var o=new n(e);r.register_comm(o);try{return t(o,i)}catch(e){o.close(),console.error(e),console.error(new Error("Exception opening new comm"))}}));this.targets[e]=i},e.prototype.unregister_target=function(e,t){this.targets[e].dispose(),delete this.targets[e]},e.prototype.register_comm=function(e){return this.comms[e.comm_id]=Promise.resolve(e),e.kernel=this.kernel,e.comm_id},e}();e.CommManager=t;var n=function(){function e(e){this.jsServicesComm=null,this.kernel=null,this.jsServicesComm=e}return Object.defineProperty(e.prototype,"comm_id",{get:function(){return this.jsServicesComm.commId},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"target_name",{get:function(){return this.jsServicesComm.targetName},enumerable:!0,configurable:!0}),e.prototype.open=function(e,t,n,r){var i=this.jsServicesComm.open(e,n,r);return this._hookupCallbacks(i,t),i.msg.header.msg_id},e.prototype.send=function(e,t,n,r){var i=this.jsServicesComm.send(e,n,r);return this._hookupCallbacks(i,t),i.msg.header.msg_id},e.prototype.close=function(e,t,n,r){var i=this.jsServicesComm.close(e,n,r);return this._hookupCallbacks(i,t),i.msg.header.msg_id},e.prototype.on_msg=function(e){this.jsServicesComm.onMsg=e.bind(this)},e.prototype.on_close=function(e){this.jsServicesComm.onClose=e.bind(this)},e.prototype._hookupCallbacks=function(e,t){t&&(e.onReply=function(e){t.shell&&t.shell.reply&&t.shell.reply(e)},e.onStdin=function(e){t.input&&t.input(e)},e.onIOPub=function(e){if(t.iopub)if(t.iopub.status&&"status"===e.header.msg_type)t.iopub.status(e);else if(t.iopub.clear_output&&"clear_output"===e.header.msg_type)t.iopub.clear_output(e);else if(t.iopub.output)switch(e.header.msg_type){case"display_data":case"execute_result":case"stream":case"error":t.iopub.output(e)}})},e}();e.Comm=n}(e.services||(e.services={}))}(G||(G={}));var le=function(){function e(e,t,n){this.initialize(e,t,n)}return e.prototype.initialize=function(e,t,n){this._handler_context=n||this,this._models=[],this.views=[],this._create_view=e,this._remove_view=t||function(e){e.remove()}},e.prototype.update=function(e,t,n,r){var i=n||this._remove_view,o=t||this._create_view;r=r||this._handler_context;for(var a=0;a<e.length&&!(a>=this._models.length||e[a]!==this._models[a]);a++);for(var s=a,l=this.views.splice(s,this.views.length-s),u=0;u<l.length;u++)l[u].then((function(e){i.call(r,e)}));for(;a<e.length;a++)this.views.push(Promise.resolve(o.call(r,e[a],a)));return this._models=e.slice(),Promise.all(this.views)},e.prototype.remove=function(){var e=this;return Promise.all(this.views).then((function(t){t.forEach((function(t){return e._remove_view.call(e._handler_context,t)})),e.views=[],e._models=[]}))},e.prototype.dispose=function(){this.views=null,this._models=null},e}(),ue=new o.Token("jupyter.extensions.jupyterWidgetRegistry");n.d(t,"ManagerBase",(function(){return O})),n.d(t,"serialize_state",(function(){return k})),n.d(t,"unpack_models",(function(){return W})),n.d(t,"WidgetModel",(function(){return X})),n.d(t,"DOMWidgetModel",(function(){return q})),n.d(t,"WidgetView",(function(){return Z})),n.d(t,"JupyterPhosphorWidget",(function(){return Q})),n.d(t,"JupyterPhosphorPanelWidget",(function(){return K})),n.d(t,"DOMWidgetView",(function(){return J})),n.d(t,"LayoutModel",(function(){return te})),n.d(t,"LayoutView",(function(){return ne})),n.d(t,"StyleModel",(function(){return ie})),n.d(t,"StyleView",(function(){return oe})),n.d(t,"shims",(function(){return G})),n.d(t,"ViewList",(function(){return le})),n.d(t,"JUPYTER_WIDGETS_VERSION",(function(){return"1.2.0"})),n.d(t,"PROTOCOL_VERSION",(function(){return"2.0.0"})),n.d(t,"difference",(function(){return h})),n.d(t,"isEqual",(function(){return f})),n.d(t,"assign",(function(){return p})),n.d(t,"uuid",(function(){return g})),n.d(t,"WrappedError",(function(){return v})),n.d(t,"resolvePromisesDict",(function(){return m})),n.d(t,"reject",(function(){return y})),n.d(t,"put_buffers",(function(){return b})),n.d(t,"remove_buffers",(function(){return C})),n.d(t,"bufferToHex",(function(){return _})),n.d(t,"hexToBuffer",(function(){return I})),n.d(t,"bufferToBase64",(function(){return x})),n.d(t,"base64ToBuffer",(function(){return w})),n.d(t,"IJupyterWidgetRegistry",(function(){return ue}))},function(e,t){e.exports=function(e){return e&&e.__esModule?e:{default:e}}},function(e,t,n){"use strict";n.d(t,"a",(function(){return o}));var r=n(0),i=n(87);function o(e,t){return r.useMemo((function(){return null==e&&null==t?null:function(n){Object(i.a)(e,n),Object(i.a)(t,n)}}),[e,t])}},function(e,t,n){"use strict";var r,i,o,a,s;n.d(t,"a",(function(){return i})),n.d(t,"b",(function(){return o})),n.d(t,"c",(function(){return a})),function(e){e.copyText=function(e){var t=document.body,n=function(r){r.preventDefault(),r.stopPropagation(),r.clipboardData.setData("text",e),t.removeEventListener("copy",n,!0)};t.addEventListener("copy",n,!0),document.execCommand("copy")}}(r||(r={})),function(e){e.boxSizing=function(e){var t=window.getComputedStyle(e),n=parseFloat(t.borderTopWidth)||0,r=parseFloat(t.borderLeftWidth)||0,i=parseFloat(t.borderRightWidth)||0,o=parseFloat(t.borderBottomWidth)||0,a=parseFloat(t.paddingTop)||0,s=parseFloat(t.paddingLeft)||0,l=parseFloat(t.paddingRight)||0,u=parseFloat(t.paddingBottom)||0;return{borderTop:n,borderLeft:r,borderRight:i,borderBottom:o,paddingTop:a,paddingLeft:s,paddingRight:l,paddingBottom:u,horizontalSum:r+s+l+i,verticalSum:n+a+u+o}},e.sizeLimits=function(e){var t=window.getComputedStyle(e),n=parseFloat(t.minWidth)||0,r=parseFloat(t.minHeight)||0,i=parseFloat(t.maxWidth)||1/0,o=parseFloat(t.maxHeight)||1/0;return{minWidth:n,minHeight:r,maxWidth:i=Math.max(n,i),maxHeight:o=Math.max(r,o)}},e.hitTest=function(e,t,n){var r=e.getBoundingClientRect();return t>=r.left&&t<r.right&&n>=r.top&&n<r.bottom},e.scrollIntoViewIfNeeded=function(e,t){var n=e.getBoundingClientRect(),r=t.getBoundingClientRect();r.top<=n.top&&r.bottom>=n.bottom||(r.top<n.top&&r.height<=n.height||r.bottom>n.bottom&&r.height>=n.height?e.scrollTop-=n.top-r.top:(r.top<n.top&&r.height>n.height||r.bottom>n.bottom&&r.height<n.height)&&(e.scrollTop-=n.bottom-r.bottom))}}(i||(i={})),function(e){e.IS_MAC=!!navigator.platform.match(/Mac/i),e.IS_WIN=!!navigator.platform.match(/Win/i),e.IS_IE=/Trident/.test(navigator.userAgent),e.IS_EDGE=/Edge/.test(navigator.userAgent),e.accelKey=function(t){return e.IS_MAC?t.metaKey:t.ctrlKey}}(o||(o={})),function(e){e.calculateSpecificity=function(e){if(e in s.specificityCache)return s.specificityCache[e];var t=s.calculateSingle(e);return s.specificityCache[e]=t},e.isValid=function(e){if(e in s.validityCache)return s.validityCache[e];var t=!0;try{s.testElem.querySelector(e)}catch(e){t=!1}return s.validityCache[e]=t},e.matches=function(e,t){return s.protoMatchFunc.call(e,t)}}(a||(a={})),function(e){var t;e.specificityCache=Object.create(null),e.validityCache=Object.create(null),e.testElem=document.createElement("div"),e.protoMatchFunc=(t=Element.prototype).matches||t.matchesSelector||t.mozMatchesSelector||t.msMatchesSelector||t.oMatchesSelector||t.webkitMatchesSelector||function(e){var t=this.ownerDocument?this.ownerDocument.querySelectorAll(e):[];return-1!==Array.prototype.indexOf.call(t,this)},e.calculateSingle=function(e){var t=0,c=0,d=0;function h(t){var n=e.match(t);return null!==n&&(e=e.slice(n[0].length),!0)}for(e=(e=e.split(",",1)[0]).replace(u," $1 ");e.length>0;)if(h(n))t++;else if(h(r))c++;else if(h(i))c++;else if(h(a))d++;else if(h(s))c++;else if(h(o))d++;else if(!h(l))return 0;return(t=Math.min(t,255))<<16|(c=Math.min(c,255))<<8|(d=Math.min(d,255))};var n=/^#[^\s\+>~#\.\[:]+/,r=/^\.[^\s\+>~#\.\[:]+/,i=/^\[[^\]]+\]/,o=/^[^\s\+>~#\.\[:]+/,a=/^(::[^\s\+>~#\.\[:]+|:first-line|:first-letter|:before|:after)/,s=/^:[^\s\+>~#\.\[:]+/,l=/^[\s\+>~\*]+/,u=/:not\(([^\)]+)\)/g}(s||(s={}))},function(e,t,n){var r,i;!function(){var o={version:"3.5.17"},a=[].slice,s=function(e){return a.call(e)},l=this.document;function u(e){return e&&(e.ownerDocument||e.document||e).documentElement}function c(e){return e&&(e.ownerDocument&&e.ownerDocument.defaultView||e.document&&e||e.defaultView)}if(l)try{s(l.documentElement.childNodes)[0].nodeType}catch(e){s=function(e){for(var t=e.length,n=new Array(t);t--;)n[t]=e[t];return n}}if(Date.now||(Date.now=function(){return+new Date}),l)try{l.createElement("DIV").style.setProperty("opacity",0,"")}catch(e){var d=this.Element.prototype,h=d.setAttribute,f=d.setAttributeNS,p=this.CSSStyleDeclaration.prototype,g=p.setProperty;d.setAttribute=function(e,t){h.call(this,e,t+"")},d.setAttributeNS=function(e,t,n){f.call(this,e,t,n+"")},p.setProperty=function(e,t,n){g.call(this,e,t+"",n)}}function v(e,t){return e<t?-1:e>t?1:e>=t?0:NaN}function m(e){return null===e?NaN:+e}function y(e){return!isNaN(e)}function b(e){return{left:function(t,n,r,i){for(arguments.length<3&&(r=0),arguments.length<4&&(i=t.length);r<i;){var o=r+i>>>1;e(t[o],n)<0?r=o+1:i=o}return r},right:function(t,n,r,i){for(arguments.length<3&&(r=0),arguments.length<4&&(i=t.length);r<i;){var o=r+i>>>1;e(t[o],n)>0?i=o:r=o+1}return r}}}o.ascending=v,o.descending=function(e,t){return t<e?-1:t>e?1:t>=e?0:NaN},o.min=function(e,t){var n,r,i=-1,o=e.length;if(1===arguments.length){for(;++i<o;)if(null!=(r=e[i])&&r>=r){n=r;break}for(;++i<o;)null!=(r=e[i])&&n>r&&(n=r)}else{for(;++i<o;)if(null!=(r=t.call(e,e[i],i))&&r>=r){n=r;break}for(;++i<o;)null!=(r=t.call(e,e[i],i))&&n>r&&(n=r)}return n},o.max=function(e,t){var n,r,i=-1,o=e.length;if(1===arguments.length){for(;++i<o;)if(null!=(r=e[i])&&r>=r){n=r;break}for(;++i<o;)null!=(r=e[i])&&r>n&&(n=r)}else{for(;++i<o;)if(null!=(r=t.call(e,e[i],i))&&r>=r){n=r;break}for(;++i<o;)null!=(r=t.call(e,e[i],i))&&r>n&&(n=r)}return n},o.extent=function(e,t){var n,r,i,o=-1,a=e.length;if(1===arguments.length){for(;++o<a;)if(null!=(r=e[o])&&r>=r){n=i=r;break}for(;++o<a;)null!=(r=e[o])&&(n>r&&(n=r),i<r&&(i=r))}else{for(;++o<a;)if(null!=(r=t.call(e,e[o],o))&&r>=r){n=i=r;break}for(;++o<a;)null!=(r=t.call(e,e[o],o))&&(n>r&&(n=r),i<r&&(i=r))}return[n,i]},o.sum=function(e,t){var n,r=0,i=e.length,o=-1;if(1===arguments.length)for(;++o<i;)y(n=+e[o])&&(r+=n);else for(;++o<i;)y(n=+t.call(e,e[o],o))&&(r+=n);return r},o.mean=function(e,t){var n,r=0,i=e.length,o=-1,a=i;if(1===arguments.length)for(;++o<i;)y(n=m(e[o]))?r+=n:--a;else for(;++o<i;)y(n=m(t.call(e,e[o],o)))?r+=n:--a;if(a)return r/a},o.quantile=function(e,t){var n=(e.length-1)*t+1,r=Math.floor(n),i=+e[r-1],o=n-r;return o?i+o*(e[r]-i):i},o.median=function(e,t){var n,r=[],i=e.length,a=-1;if(1===arguments.length)for(;++a<i;)y(n=m(e[a]))&&r.push(n);else for(;++a<i;)y(n=m(t.call(e,e[a],a)))&&r.push(n);if(r.length)return o.quantile(r.sort(v),.5)},o.variance=function(e,t){var n,r,i=e.length,o=0,a=0,s=-1,l=0;if(1===arguments.length)for(;++s<i;)y(n=m(e[s]))&&(a+=(r=n-o)*(n-(o+=r/++l)));else for(;++s<i;)y(n=m(t.call(e,e[s],s)))&&(a+=(r=n-o)*(n-(o+=r/++l)));if(l>1)return a/(l-1)},o.deviation=function(){var e=o.variance.apply(this,arguments);return e?Math.sqrt(e):e};var C=b(v);function A(e){return e.length}o.bisectLeft=C.left,o.bisect=o.bisectRight=C.right,o.bisector=function(e){return b(1===e.length?function(t,n){return v(e(t),n)}:e)},o.shuffle=function(e,t,n){(o=arguments.length)<3&&(n=e.length,o<2&&(t=0));for(var r,i,o=n-t;o;)i=Math.random()*o--|0,r=e[o+t],e[o+t]=e[i+t],e[i+t]=r;return e},o.permute=function(e,t){for(var n=t.length,r=new Array(n);n--;)r[n]=e[t[n]];return r},o.pairs=function(e){for(var t=0,n=e.length-1,r=e[0],i=new Array(n<0?0:n);t<n;)i[t]=[r,r=e[++t]];return i},o.transpose=function(e){if(!(i=e.length))return[];for(var t=-1,n=o.min(e,A),r=new Array(n);++t<n;)for(var i,a=-1,s=r[t]=new Array(i);++a<i;)s[a]=e[a][t];return r},o.zip=function(){return o.transpose(arguments)},o.keys=function(e){var t=[];for(var n in e)t.push(n);return t},o.values=function(e){var t=[];for(var n in e)t.push(e[n]);return t},o.entries=function(e){var t=[];for(var n in e)t.push({key:n,value:e[n]});return t},o.merge=function(e){for(var t,n,r,i=e.length,o=-1,a=0;++o<i;)a+=e[o].length;for(n=new Array(a);--i>=0;)for(t=(r=e[i]).length;--t>=0;)n[--a]=r[t];return n};var _=Math.abs;function I(e){for(var t=1;e*t%1;)t*=10;return t}function x(e,t){for(var n in t)Object.defineProperty(e.prototype,n,{value:t[n],enumerable:!1})}function w(){this._=Object.create(null)}o.range=function(e,t,n){if(arguments.length<3&&(n=1,arguments.length<2&&(t=e,e=0)),(t-e)/n==1/0)throw new Error("infinite range");var r,i=[],o=I(_(n)),a=-1;if(e*=o,t*=o,(n*=o)<0)for(;(r=e+n*++a)>t;)i.push(r/o);else for(;(r=e+n*++a)<t;)i.push(r/o);return i},o.map=function(e,t){var n=new w;if(e instanceof w)e.forEach((function(e,t){n.set(e,t)}));else if(Array.isArray(e)){var r,i=-1,o=e.length;if(1===arguments.length)for(;++i<o;)n.set(i,e[i]);else for(;++i<o;)n.set(t.call(e,r=e[i],i),r)}else for(var a in e)n.set(a,e[a]);return n};function M(e){return"__proto__"==(e+="")||"\0"===e[0]?"\0"+e:e}function S(e){return"\0"===(e+="")[0]?e.slice(1):e}function T(e){return M(e)in this._}function E(e){return(e=M(e))in this._&&delete this._[e]}function O(){var e=[];for(var t in this._)e.push(S(t));return e}function k(){var e=0;for(var t in this._)++e;return e}function L(){for(var e in this._)return!1;return!0}function D(){this._=Object.create(null)}function P(e){return e}function N(e,t,n){return function(){var r=n.apply(t,arguments);return r===t?e:r}}function j(e,t){if(t in e)return t;t=t.charAt(0).toUpperCase()+t.slice(1);for(var n=0,r=R.length;n<r;++n){var i=R[n]+t;if(i in e)return i}}x(w,{has:T,get:function(e){return this._[M(e)]},set:function(e,t){return this._[M(e)]=t},remove:E,keys:O,values:function(){var e=[];for(var t in this._)e.push(this._[t]);return e},entries:function(){var e=[];for(var t in this._)e.push({key:S(t),value:this._[t]});return e},size:k,empty:L,forEach:function(e){for(var t in this._)e.call(this,S(t),this._[t])}}),o.nest=function(){var e,t,n={},r=[],i=[];function a(i,o,s){if(s>=r.length)return t?t.call(n,o):e?o.sort(e):o;for(var l,u,c,d,h=-1,f=o.length,p=r[s++],g=new w;++h<f;)(d=g.get(l=p(u=o[h])))?d.push(u):g.set(l,[u]);return i?(u=i(),c=function(e,t){u.set(e,a(i,t,s))}):(u={},c=function(e,t){u[e]=a(i,t,s)}),g.forEach(c),u}return n.map=function(e,t){return a(t,e,0)},n.entries=function(e){return function e(t,n){if(n>=r.length)return t;var o=[],a=i[n++];return t.forEach((function(t,r){o.push({key:t,values:e(r,n)})})),a?o.sort((function(e,t){return a(e.key,t.key)})):o}(a(o.map,e,0),0)},n.key=function(e){return r.push(e),n},n.sortKeys=function(e){return i[r.length-1]=e,n},n.sortValues=function(t){return e=t,n},n.rollup=function(e){return t=e,n},n},o.set=function(e){var t=new D;if(e)for(var n=0,r=e.length;n<r;++n)t.add(e[n]);return t},x(D,{has:T,add:function(e){return this._[M(e+="")]=!0,e},remove:E,values:O,size:k,empty:L,forEach:function(e){for(var t in this._)e.call(this,S(t))}}),o.behavior={},o.rebind=function(e,t){for(var n,r=1,i=arguments.length;++r<i;)e[n=arguments[r]]=N(e,t,t[n]);return e};var R=["webkit","ms","moz","Moz","o","O"];function z(){}function F(){}function B(e){var t=[],n=new w;function r(){for(var n,r=t,i=-1,o=r.length;++i<o;)(n=r[i].on)&&n.apply(this,arguments);return e}return r.on=function(r,i){var o,a=n.get(r);return arguments.length<2?a&&a.on:(a&&(a.on=null,t=t.slice(0,o=t.indexOf(a)).concat(t.slice(o+1)),n.remove(r)),i&&t.push(n.set(r,{on:i})),e)},r}function H(){o.event.preventDefault()}function U(){for(var e,t=o.event;e=t.sourceEvent;)t=e;return t}function V(e){for(var t=new F,n=0,r=arguments.length;++n<r;)t[arguments[n]]=B(t);return t.of=function(n,r){return function(i){try{var a=i.sourceEvent=o.event;i.target=e,o.event=i,t[i.type].apply(n,r)}finally{o.event=a}}},t}o.dispatch=function(){for(var e=new F,t=-1,n=arguments.length;++t<n;)e[arguments[t]]=B(e);return e},F.prototype.on=function(e,t){var n=e.indexOf("."),r="";if(n>=0&&(r=e.slice(n+1),e=e.slice(0,n)),e)return arguments.length<2?this[e].on(r):this[e].on(r,t);if(2===arguments.length){if(null==t)for(e in this)this.hasOwnProperty(e)&&this[e].on(r,null);return this}},o.event=null,o.requote=function(e){return e.replace(Y,"\\$&")};var Y=/[\\\^\$\*\+\?\|\[\]\(\)\.\{\}]/g,W={}.__proto__?function(e,t){e.__proto__=t}:function(e,t){for(var n in t)e[n]=t[n]};function G(e){return W(e,Q),e}var X=function(e,t){return t.querySelector(e)},q=function(e,t){return t.querySelectorAll(e)},Z=function(e,t){var n=e.matches||e[j(e,"matchesSelector")];return(Z=function(e,t){return n.call(e,t)})(e,t)};"function"==typeof Sizzle&&(X=function(e,t){return Sizzle(e,t)[0]||null},q=Sizzle,Z=Sizzle.matchesSelector),o.selection=function(){return o.select(l.documentElement)};var Q=o.selection.prototype=[];function K(e){return"function"==typeof e?e:function(){return X(e,this)}}function J(e){return"function"==typeof e?e:function(){return q(e,this)}}Q.select=function(e){var t,n,r,i,o=[];e=K(e);for(var a=-1,s=this.length;++a<s;){o.push(t=[]),t.parentNode=(r=this[a]).parentNode;for(var l=-1,u=r.length;++l<u;)(i=r[l])?(t.push(n=e.call(i,i.__data__,l,a)),n&&"__data__"in i&&(n.__data__=i.__data__)):t.push(null)}return G(o)},Q.selectAll=function(e){var t,n,r=[];e=J(e);for(var i=-1,o=this.length;++i<o;)for(var a=this[i],l=-1,u=a.length;++l<u;)(n=a[l])&&(r.push(t=s(e.call(n,n.__data__,l,i))),t.parentNode=n);return G(r)};var $="http://www.w3.org/1999/xhtml",ee={svg:"http://www.w3.org/2000/svg",xhtml:$,xlink:"http://www.w3.org/1999/xlink",xml:"http://www.w3.org/XML/1998/namespace",xmlns:"http://www.w3.org/2000/xmlns/"};function te(e,t){return e=o.ns.qualify(e),null==t?e.local?function(){this.removeAttributeNS(e.space,e.local)}:function(){this.removeAttribute(e)}:"function"==typeof t?e.local?function(){var n=t.apply(this,arguments);null==n?this.removeAttributeNS(e.space,e.local):this.setAttributeNS(e.space,e.local,n)}:function(){var n=t.apply(this,arguments);null==n?this.removeAttribute(e):this.setAttribute(e,n)}:e.local?function(){this.setAttributeNS(e.space,e.local,t)}:function(){this.setAttribute(e,t)}}function ne(e){return e.trim().replace(/\s+/g," ")}function re(e){return new RegExp("(?:^|\\s+)"+o.requote(e)+"(?:\\s+|$)","g")}function ie(e){return(e+"").trim().split(/^|\s+/)}function oe(e,t){var n=(e=ie(e).map(ae)).length;return"function"==typeof t?function(){for(var r=-1,i=t.apply(this,arguments);++r<n;)e[r](this,i)}:function(){for(var r=-1;++r<n;)e[r](this,t)}}function ae(e){var t=re(e);return function(n,r){if(i=n.classList)return r?i.add(e):i.remove(e);var i=n.getAttribute("class")||"";r?(t.lastIndex=0,t.test(i)||n.setAttribute("class",ne(i+" "+e))):n.setAttribute("class",ne(i.replace(t," ")))}}function se(e,t,n){return null==t?function(){this.style.removeProperty(e)}:"function"==typeof t?function(){var r=t.apply(this,arguments);null==r?this.style.removeProperty(e):this.style.setProperty(e,r,n)}:function(){this.style.setProperty(e,t,n)}}function le(e,t){return null==t?function(){delete this[e]}:"function"==typeof t?function(){var n=t.apply(this,arguments);null==n?delete this[e]:this[e]=n}:function(){this[e]=t}}function ue(e){return"function"==typeof e?e:(e=o.ns.qualify(e)).local?function(){return this.ownerDocument.createElementNS(e.space,e.local)}:function(){var t=this.ownerDocument,n=this.namespaceURI;return n===$&&t.documentElement.namespaceURI===$?t.createElement(e):t.createElementNS(n,e)}}function ce(){var e=this.parentNode;e&&e.removeChild(this)}function de(e){return{__data__:e}}function he(e){return function(){return Z(this,e)}}function fe(e){return arguments.length||(e=v),function(t,n){return t&&n?e(t.__data__,n.__data__):!t-!n}}function pe(e,t){for(var n=0,r=e.length;n<r;n++)for(var i,o=e[n],a=0,s=o.length;a<s;a++)(i=o[a])&&t(i,a,n);return e}function ge(e){return W(e,ve),e}o.ns={prefix:ee,qualify:function(e){var t=e.indexOf(":"),n=e;return t>=0&&"xmlns"!==(n=e.slice(0,t))&&(e=e.slice(t+1)),ee.hasOwnProperty(n)?{space:ee[n],local:e}:e}},Q.attr=function(e,t){if(arguments.length<2){if("string"==typeof e){var n=this.node();return(e=o.ns.qualify(e)).local?n.getAttributeNS(e.space,e.local):n.getAttribute(e)}for(t in e)this.each(te(t,e[t]));return this}return this.each(te(e,t))},Q.classed=function(e,t){if(arguments.length<2){if("string"==typeof e){var n=this.node(),r=(e=ie(e)).length,i=-1;if(t=n.classList){for(;++i<r;)if(!t.contains(e[i]))return!1}else for(t=n.getAttribute("class");++i<r;)if(!re(e[i]).test(t))return!1;return!0}for(t in e)this.each(oe(t,e[t]));return this}return this.each(oe(e,t))},Q.style=function(e,t,n){var r=arguments.length;if(r<3){if("string"!=typeof e){for(n in r<2&&(t=""),e)this.each(se(n,e[n],t));return this}if(r<2){var i=this.node();return c(i).getComputedStyle(i,null).getPropertyValue(e)}n=""}return this.each(se(e,t,n))},Q.property=function(e,t){if(arguments.length<2){if("string"==typeof e)return this.node()[e];for(t in e)this.each(le(t,e[t]));return this}return this.each(le(e,t))},Q.text=function(e){return arguments.length?this.each("function"==typeof e?function(){var t=e.apply(this,arguments);this.textContent=null==t?"":t}:null==e?function(){this.textContent=""}:function(){this.textContent=e}):this.node().textContent},Q.html=function(e){return arguments.length?this.each("function"==typeof e?function(){var t=e.apply(this,arguments);this.innerHTML=null==t?"":t}:null==e?function(){this.innerHTML=""}:function(){this.innerHTML=e}):this.node().innerHTML},Q.append=function(e){return e=ue(e),this.select((function(){return this.appendChild(e.apply(this,arguments))}))},Q.insert=function(e,t){return e=ue(e),t=K(t),this.select((function(){return this.insertBefore(e.apply(this,arguments),t.apply(this,arguments)||null)}))},Q.remove=function(){return this.each(ce)},Q.data=function(e,t){var n,r,i=-1,o=this.length;if(!arguments.length){for(e=new Array(o=(n=this[0]).length);++i<o;)(r=n[i])&&(e[i]=r.__data__);return e}function a(e,n){var r,i,o,a=e.length,c=n.length,d=Math.min(a,c),h=new Array(c),f=new Array(c),p=new Array(a);if(t){var g,v=new w,m=new Array(a);for(r=-1;++r<a;)(i=e[r])&&(v.has(g=t.call(i,i.__data__,r))?p[r]=i:v.set(g,i),m[r]=g);for(r=-1;++r<c;)(i=v.get(g=t.call(n,o=n[r],r)))?!0!==i&&(h[r]=i,i.__data__=o):f[r]=de(o),v.set(g,!0);for(r=-1;++r<a;)r in m&&!0!==v.get(m[r])&&(p[r]=e[r])}else{for(r=-1;++r<d;)i=e[r],o=n[r],i?(i.__data__=o,h[r]=i):f[r]=de(o);for(;r<c;++r)f[r]=de(n[r]);for(;r<a;++r)p[r]=e[r]}f.update=h,f.parentNode=h.parentNode=p.parentNode=e.parentNode,s.push(f),l.push(h),u.push(p)}var s=ge([]),l=G([]),u=G([]);if("function"==typeof e)for(;++i<o;)a(n=this[i],e.call(n,n.parentNode.__data__,i));else for(;++i<o;)a(n=this[i],e);return l.enter=function(){return s},l.exit=function(){return u},l},Q.datum=function(e){return arguments.length?this.property("__data__",e):this.property("__data__")},Q.filter=function(e){var t,n,r,i=[];"function"!=typeof e&&(e=he(e));for(var o=0,a=this.length;o<a;o++){i.push(t=[]),t.parentNode=(n=this[o]).parentNode;for(var s=0,l=n.length;s<l;s++)(r=n[s])&&e.call(r,r.__data__,s,o)&&t.push(r)}return G(i)},Q.order=function(){for(var e=-1,t=this.length;++e<t;)for(var n,r=this[e],i=r.length-1,o=r[i];--i>=0;)(n=r[i])&&(o&&o!==n.nextSibling&&o.parentNode.insertBefore(n,o),o=n);return this},Q.sort=function(e){e=fe.apply(this,arguments);for(var t=-1,n=this.length;++t<n;)this[t].sort(e);return this.order()},Q.each=function(e){return pe(this,(function(t,n,r){e.call(t,t.__data__,n,r)}))},Q.call=function(e){var t=s(arguments);return e.apply(t[0]=this,t),this},Q.empty=function(){return!this.node()},Q.node=function(){for(var e=0,t=this.length;e<t;e++)for(var n=this[e],r=0,i=n.length;r<i;r++){var o=n[r];if(o)return o}return null},Q.size=function(){var e=0;return pe(this,(function(){++e})),e};var ve=[];function me(e){var t,n;return function(r,i,o){var a,s=e[o].update,l=s.length;for(o!=n&&(n=o,t=0),i>=t&&(t=i+1);!(a=s[t])&&++t<l;);return a}}function ye(e,t,n){var r="__on"+e,i=e.indexOf("."),a=Ce;i>0&&(e=e.slice(0,i));var l=be.get(e);function u(){var t=this[r];t&&(this.removeEventListener(e,t,t.$),delete this[r])}return l&&(e=l,a=Ae),i?t?function(){var i=a(t,s(arguments));u.call(this),this.addEventListener(e,this[r]=i,i.$=n),i._=t}:u:t?z:function(){var t,n=new RegExp("^__on([^.]+)"+o.requote(e)+"$");for(var r in this)if(t=r.match(n)){var i=this[r];this.removeEventListener(t[1],i,i.$),delete this[r]}}}o.selection.enter=ge,o.selection.enter.prototype=ve,ve.append=Q.append,ve.empty=Q.empty,ve.node=Q.node,ve.call=Q.call,ve.size=Q.size,ve.select=function(e){for(var t,n,r,i,o,a=[],s=-1,l=this.length;++s<l;){r=(i=this[s]).update,a.push(t=[]),t.parentNode=i.parentNode;for(var u=-1,c=i.length;++u<c;)(o=i[u])?(t.push(r[u]=n=e.call(i.parentNode,o.__data__,u,s)),n.__data__=o.__data__):t.push(null)}return G(a)},ve.insert=function(e,t){return arguments.length<2&&(t=me(this)),Q.insert.call(this,e,t)},o.select=function(e){var t;return"string"==typeof e?(t=[X(e,l)]).parentNode=l.documentElement:(t=[e]).parentNode=u(e),G([t])},o.selectAll=function(e){var t;return"string"==typeof e?(t=s(q(e,l))).parentNode=l.documentElement:(t=s(e)).parentNode=null,G([t])},Q.on=function(e,t,n){var r=arguments.length;if(r<3){if("string"!=typeof e){for(n in r<2&&(t=!1),e)this.each(ye(n,e[n],t));return this}if(r<2)return(r=this.node()["__on"+e])&&r._;n=!1}return this.each(ye(e,t,n))};var be=o.map({mouseenter:"mouseover",mouseleave:"mouseout"});function Ce(e,t){return function(n){var r=o.event;o.event=n,t[0]=this.__data__;try{e.apply(this,t)}finally{o.event=r}}}function Ae(e,t){var n=Ce(e,t);return function(e){var t=e.relatedTarget;t&&(t===this||8&t.compareDocumentPosition(this))||n.call(this,e)}}l&&be.forEach((function(e){"on"+e in l&&be.remove(e)}));var _e,Ie=0;function xe(e){var t=".dragsuppress-"+ ++Ie,n="click"+t,r=o.select(c(e)).on("touchmove"+t,H).on("dragstart"+t,H).on("selectstart"+t,H);if(null==_e&&(_e=!("onselectstart"in e)&&j(e.style,"userSelect")),_e){var i=u(e).style,a=i[_e];i[_e]="none"}return function(e){if(r.on(t,null),_e&&(i[_e]=a),e){var o=function(){r.on(n,null)};r.on(n,(function(){H(),o()}),!0),setTimeout(o,0)}}}o.mouse=function(e){return Me(e,U())};var we=this.navigator&&/WebKit/.test(this.navigator.userAgent)?-1:0;function Me(e,t){t.changedTouches&&(t=t.changedTouches[0]);var n=e.ownerSVGElement||e;if(n.createSVGPoint){var r=n.createSVGPoint();if(we<0){var i=c(e);if(i.scrollX||i.scrollY){var a=(n=o.select("body").append("svg").style({position:"absolute",top:0,left:0,margin:0,padding:0,border:"none"},"important"))[0][0].getScreenCTM();we=!(a.f||a.e),n.remove()}}return we?(r.x=t.pageX,r.y=t.pageY):(r.x=t.clientX,r.y=t.clientY),[(r=r.matrixTransform(e.getScreenCTM().inverse())).x,r.y]}var s=e.getBoundingClientRect();return[t.clientX-s.left-e.clientLeft,t.clientY-s.top-e.clientTop]}function Se(){return o.event.changedTouches[0].identifier}o.touch=function(e,t,n){if(arguments.length<3&&(n=t,t=U().changedTouches),t)for(var r,i=0,o=t.length;i<o;++i)if((r=t[i]).identifier===n)return Me(e,r)},o.behavior.drag=function(){var e=V(i,"drag","dragstart","dragend"),t=null,n=a(z,o.mouse,c,"mousemove","mouseup"),r=a(Se,o.touch,P,"touchmove","touchend");function i(){this.on("mousedown.drag",n).on("touchstart.drag",r)}function a(n,r,i,a,s){return function(){var l,u=this,c=o.event.target.correspondingElement||o.event.target,d=u.parentNode,h=e.of(u,arguments),f=0,p=n(),g=".drag"+(null==p?"":"-"+p),v=o.select(i(c)).on(a+g,b).on(s+g,C),m=xe(c),y=r(d,p);function b(){var e,t,n=r(d,p);n&&(e=n[0]-y[0],t=n[1]-y[1],f|=e|t,y=n,h({type:"drag",x:n[0]+l[0],y:n[1]+l[1],dx:e,dy:t}))}function C(){r(d,p)&&(v.on(a+g,null).on(s+g,null),m(f),h({type:"dragend"}))}l=t?[(l=t.apply(u,arguments)).x-y[0],l.y-y[1]]:[0,0],h({type:"dragstart"})}}return i.origin=function(e){return arguments.length?(t=e,i):t},o.rebind(i,e,"on")},o.touches=function(e,t){return arguments.length<2&&(t=U().touches),t?s(t).map((function(t){var n=Me(e,t);return n.identifier=t.identifier,n})):[]};var Te=1e-6,Ee=1e-12,Oe=Math.PI,ke=2*Oe,Le=ke-Te,De=Oe/2,Pe=Oe/180,Ne=180/Oe;function je(e){return e>0?1:e<0?-1:0}function Re(e,t,n){return(t[0]-e[0])*(n[1]-e[1])-(t[1]-e[1])*(n[0]-e[0])}function ze(e){return e>1?0:e<-1?Oe:Math.acos(e)}function Fe(e){return e>1?De:e<-1?-De:Math.asin(e)}function Be(e){return((e=Math.exp(e))+1/e)/2}function He(e){return(e=Math.sin(e/2))*e}var Ue=Math.SQRT2;o.interpolateZoom=function(e,t){var n,r,i=e[0],o=e[1],a=e[2],s=t[0],l=t[1],u=t[2],c=s-i,d=l-o,h=c*c+d*d;if(h<Ee)r=Math.log(u/a)/Ue,n=function(e){return[i+e*c,o+e*d,a*Math.exp(Ue*e*r)]};else{var f=Math.sqrt(h),p=(u*u-a*a+4*h)/(2*a*2*f),g=(u*u-a*a-4*h)/(2*u*2*f),v=Math.log(Math.sqrt(p*p+1)-p),m=Math.log(Math.sqrt(g*g+1)-g);r=(m-v)/Ue,n=function(e){var t,n=e*r,s=Be(v),l=a/(2*f)*(s*(t=Ue*n+v,((t=Math.exp(2*t))-1)/(t+1))-function(e){return((e=Math.exp(e))-1/e)/2}(v));return[i+l*c,o+l*d,a*s/Be(Ue*n+v)]}}return n.duration=1e3*r,n},o.behavior.zoom=function(){var e,t,n,r,i,a,s,u,d,h={x:0,y:0,k:1},f=[960,500],p=We,g=250,v=0,m="mousedown.zoom",y="mousemove.zoom",b="mouseup.zoom",C="touchstart.zoom",A=V(_,"zoomstart","zoom","zoomend");function _(e){e.on(m,k).on(Ye+".zoom",D).on("dblclick.zoom",P).on(C,L)}function I(e){return[(e[0]-h.x)/h.k,(e[1]-h.y)/h.k]}function x(e){h.k=Math.max(p[0],Math.min(p[1],e))}function w(e,t){t=function(e){return[e[0]*h.k+h.x,e[1]*h.k+h.y]}(t),h.x+=e[0]-t[0],h.y+=e[1]-t[1]}function M(e,n,r,i){e.__chart__={x:h.x,y:h.y,k:h.k},x(Math.pow(2,i)),w(t=n,r),e=o.select(e),g>0&&(e=e.transition().duration(g)),e.call(_.event)}function S(){s&&s.domain(a.range().map((function(e){return(e-h.x)/h.k})).map(a.invert)),d&&d.domain(u.range().map((function(e){return(e-h.y)/h.k})).map(u.invert))}function T(e){v++||e({type:"zoomstart"})}function E(e){S(),e({type:"zoom",scale:h.k,translate:[h.x,h.y]})}function O(e){--v||(e({type:"zoomend"}),t=null)}function k(){var e=this,t=A.of(e,arguments),n=0,r=o.select(c(e)).on(y,s).on(b,l),i=I(o.mouse(e)),a=xe(e);function s(){n=1,w(o.mouse(e),i),E(t)}function l(){r.on(y,null).on(b,null),a(n),O(t)}As.call(e),T(t)}function L(){var e,t=this,n=A.of(t,arguments),r={},a=0,s=".zoom-"+o.event.changedTouches[0].identifier,l="touchmove"+s,u="touchend"+s,c=[],d=o.select(t),f=xe(t);function p(){var n=o.touches(t);return e=h.k,n.forEach((function(e){e.identifier in r&&(r[e.identifier]=I(e))})),n}function g(){var e=o.event.target;o.select(e).on(l,v).on(u,y),c.push(e);for(var n=o.event.changedTouches,s=0,d=n.length;s<d;++s)r[n[s].identifier]=null;var f=p(),g=Date.now();if(1===f.length){if(g-i<500){var m=f[0];M(t,m,r[m.identifier],Math.floor(Math.log(h.k)/Math.LN2)+1),H()}i=g}else if(f.length>1){m=f[0];var b=f[1],C=m[0]-b[0],A=m[1]-b[1];a=C*C+A*A}}function v(){var s,l,u,c,d=o.touches(t);As.call(t);for(var h=0,f=d.length;h<f;++h,c=null)if(u=d[h],c=r[u.identifier]){if(l)break;s=u,l=c}if(c){var p=(p=u[0]-s[0])*p+(p=u[1]-s[1])*p,g=a&&Math.sqrt(p/a);s=[(s[0]+u[0])/2,(s[1]+u[1])/2],l=[(l[0]+c[0])/2,(l[1]+c[1])/2],x(g*e)}i=null,w(s,l),E(n)}function y(){if(o.event.touches.length){for(var e=o.event.changedTouches,t=0,i=e.length;t<i;++t)delete r[e[t].identifier];for(var a in r)return void p()}o.selectAll(c).on(s,null),d.on(m,k).on(C,L),f(),O(n)}g(),T(n),d.on(m,null).on(C,g)}function D(){var i=A.of(this,arguments);r?clearTimeout(r):(As.call(this),e=I(t=n||o.mouse(this)),T(i)),r=setTimeout((function(){r=null,O(i)}),50),H(),x(Math.pow(2,.002*Ve())*h.k),w(t,e),E(i)}function P(){var e=o.mouse(this),t=Math.log(h.k)/Math.LN2;M(this,e,I(e),o.event.shiftKey?Math.ceil(t)-1:Math.floor(t)+1)}return Ye||(Ye="onwheel"in l?(Ve=function(){return-o.event.deltaY*(o.event.deltaMode?120:1)},"wheel"):"onmousewheel"in l?(Ve=function(){return o.event.wheelDelta},"mousewheel"):(Ve=function(){return-o.event.detail},"MozMousePixelScroll")),_.event=function(e){e.each((function(){var e=A.of(this,arguments),n=h;xs?o.select(this).transition().each("start.zoom",(function(){h=this.__chart__||{x:0,y:0,k:1},T(e)})).tween("zoom:zoom",(function(){var r=f[0],i=f[1],a=t?t[0]:r/2,s=t?t[1]:i/2,l=o.interpolateZoom([(a-h.x)/h.k,(s-h.y)/h.k,r/h.k],[(a-n.x)/n.k,(s-n.y)/n.k,r/n.k]);return function(t){var n=l(t),i=r/n[2];this.__chart__=h={x:a-n[0]*i,y:s-n[1]*i,k:i},E(e)}})).each("interrupt.zoom",(function(){O(e)})).each("end.zoom",(function(){O(e)})):(this.__chart__=h,T(e),E(e),O(e))}))},_.translate=function(e){return arguments.length?(h={x:+e[0],y:+e[1],k:h.k},S(),_):[h.x,h.y]},_.scale=function(e){return arguments.length?(h={x:h.x,y:h.y,k:null},x(+e),S(),_):h.k},_.scaleExtent=function(e){return arguments.length?(p=null==e?We:[+e[0],+e[1]],_):p},_.center=function(e){return arguments.length?(n=e&&[+e[0],+e[1]],_):n},_.size=function(e){return arguments.length?(f=e&&[+e[0],+e[1]],_):f},_.duration=function(e){return arguments.length?(g=+e,_):g},_.x=function(e){return arguments.length?(s=e,a=e.copy(),h={x:0,y:0,k:1},_):s},_.y=function(e){return arguments.length?(d=e,u=e.copy(),h={x:0,y:0,k:1},_):d},o.rebind(_,A,"on")};var Ve,Ye,We=[0,1/0];function Ge(){}function Xe(e,t,n){return this instanceof Xe?(this.h=+e,this.s=+t,void(this.l=+n)):arguments.length<2?e instanceof Xe?new Xe(e.h,e.s,e.l):ht(""+e,ft,Xe):new Xe(e,t,n)}o.color=Ge,Ge.prototype.toString=function(){return this.rgb()+""},o.hsl=Xe;var qe=Xe.prototype=new Ge;function Ze(e,t,n){var r,i;function o(e){return Math.round(255*function(e){return e>360?e-=360:e<0&&(e+=360),e<60?r+(i-r)*e/60:e<180?i:e<240?r+(i-r)*(240-e)/60:r}(e))}return e=isNaN(e)?0:(e%=360)<0?e+360:e,t=isNaN(t)||t<0?0:t>1?1:t,r=2*(n=n<0?0:n>1?1:n)-(i=n<=.5?n*(1+t):n+t-n*t),new st(o(e+120),o(e),o(e-120))}function Qe(e,t,n){return this instanceof Qe?(this.h=+e,this.c=+t,void(this.l=+n)):arguments.length<2?e instanceof Qe?new Qe(e.h,e.c,e.l):rt(e instanceof $e?e.l:(e=pt((e=o.rgb(e)).r,e.g,e.b)).l,e.a,e.b):new Qe(e,t,n)}qe.brighter=function(e){return e=Math.pow(.7,arguments.length?e:1),new Xe(this.h,this.s,this.l/e)},qe.darker=function(e){return e=Math.pow(.7,arguments.length?e:1),new Xe(this.h,this.s,e*this.l)},qe.rgb=function(){return Ze(this.h,this.s,this.l)},o.hcl=Qe;var Ke=Qe.prototype=new Ge;function Je(e,t,n){return isNaN(e)&&(e=0),isNaN(t)&&(t=0),new $e(n,Math.cos(e*=Pe)*t,Math.sin(e)*t)}function $e(e,t,n){return this instanceof $e?(this.l=+e,this.a=+t,void(this.b=+n)):arguments.length<2?e instanceof $e?new $e(e.l,e.a,e.b):e instanceof Qe?Je(e.h,e.c,e.l):pt((e=st(e)).r,e.g,e.b):new $e(e,t,n)}Ke.brighter=function(e){return new Qe(this.h,this.c,Math.min(100,this.l+et*(arguments.length?e:1)))},Ke.darker=function(e){return new Qe(this.h,this.c,Math.max(0,this.l-et*(arguments.length?e:1)))},Ke.rgb=function(){return Je(this.h,this.c,this.l).rgb()},o.lab=$e;var et=18,tt=$e.prototype=new Ge;function nt(e,t,n){var r=(e+16)/116,i=r+t/500,o=r-n/200;return new st(at(3.2404542*(i=.95047*it(i))-1.5371385*(r=1*it(r))-.4985314*(o=1.08883*it(o))),at(-.969266*i+1.8760108*r+.041556*o),at(.0556434*i-.2040259*r+1.0572252*o))}function rt(e,t,n){return e>0?new Qe(Math.atan2(n,t)*Ne,Math.sqrt(t*t+n*n),e):new Qe(NaN,NaN,e)}function it(e){return e>.206893034?e*e*e:(e-4/29)/7.787037}function ot(e){return e>.008856?Math.pow(e,1/3):7.787037*e+4/29}function at(e){return Math.round(255*(e<=.00304?12.92*e:1.055*Math.pow(e,1/2.4)-.055))}function st(e,t,n){return this instanceof st?(this.r=~~e,this.g=~~t,void(this.b=~~n)):arguments.length<2?e instanceof st?new st(e.r,e.g,e.b):ht(""+e,st,Ze):new st(e,t,n)}function lt(e){return new st(e>>16,e>>8&255,255&e)}function ut(e){return lt(e)+""}tt.brighter=function(e){return new $e(Math.min(100,this.l+et*(arguments.length?e:1)),this.a,this.b)},tt.darker=function(e){return new $e(Math.max(0,this.l-et*(arguments.length?e:1)),this.a,this.b)},tt.rgb=function(){return nt(this.l,this.a,this.b)},o.rgb=st;var ct=st.prototype=new Ge;function dt(e){return e<16?"0"+Math.max(0,e).toString(16):Math.min(255,e).toString(16)}function ht(e,t,n){var r,i,o,a=0,s=0,l=0;if(r=/([a-z]+)\((.*)\)/.exec(e=e.toLowerCase()))switch(i=r[2].split(","),r[1]){case"hsl":return n(parseFloat(i[0]),parseFloat(i[1])/100,parseFloat(i[2])/100);case"rgb":return t(vt(i[0]),vt(i[1]),vt(i[2]))}return(o=mt.get(e))?t(o.r,o.g,o.b):(null==e||"#"!==e.charAt(0)||isNaN(o=parseInt(e.slice(1),16))||(4===e.length?(a=(3840&o)>>4,a|=a>>4,s=240&o,s|=s>>4,l=15&o,l|=l<<4):7===e.length&&(a=(16711680&o)>>16,s=(65280&o)>>8,l=255&o)),t(a,s,l))}function ft(e,t,n){var r,i,o=Math.min(e/=255,t/=255,n/=255),a=Math.max(e,t,n),s=a-o,l=(a+o)/2;return s?(i=l<.5?s/(a+o):s/(2-a-o),r=e==a?(t-n)/s+(t<n?6:0):t==a?(n-e)/s+2:(e-t)/s+4,r*=60):(r=NaN,i=l>0&&l<1?0:r),new Xe(r,i,l)}function pt(e,t,n){var r=ot((.4124564*(e=gt(e))+.3575761*(t=gt(t))+.1804375*(n=gt(n)))/.95047),i=ot((.2126729*e+.7151522*t+.072175*n)/1);return $e(116*i-16,500*(r-i),200*(i-ot((.0193339*e+.119192*t+.9503041*n)/1.08883)))}function gt(e){return(e/=255)<=.04045?e/12.92:Math.pow((e+.055)/1.055,2.4)}function vt(e){var t=parseFloat(e);return"%"===e.charAt(e.length-1)?Math.round(2.55*t):t}ct.brighter=function(e){e=Math.pow(.7,arguments.length?e:1);var t=this.r,n=this.g,r=this.b,i=30;return t||n||r?(t&&t<i&&(t=i),n&&n<i&&(n=i),r&&r<i&&(r=i),new st(Math.min(255,t/e),Math.min(255,n/e),Math.min(255,r/e))):new st(i,i,i)},ct.darker=function(e){return new st((e=Math.pow(.7,arguments.length?e:1))*this.r,e*this.g,e*this.b)},ct.hsl=function(){return ft(this.r,this.g,this.b)},ct.toString=function(){return"#"+dt(this.r)+dt(this.g)+dt(this.b)};var mt=o.map({aliceblue:15792383,antiquewhite:16444375,aqua:65535,aquamarine:8388564,azure:15794175,beige:16119260,bisque:16770244,black:0,blanchedalmond:16772045,blue:255,blueviolet:9055202,brown:10824234,burlywood:14596231,cadetblue:6266528,chartreuse:8388352,chocolate:13789470,coral:16744272,cornflowerblue:6591981,cornsilk:16775388,crimson:14423100,cyan:65535,darkblue:139,darkcyan:35723,darkgoldenrod:12092939,darkgray:11119017,darkgreen:25600,darkgrey:11119017,darkkhaki:12433259,darkmagenta:9109643,darkolivegreen:5597999,darkorange:16747520,darkorchid:10040012,darkred:9109504,darksalmon:15308410,darkseagreen:9419919,darkslateblue:4734347,darkslategray:3100495,darkslategrey:3100495,darkturquoise:52945,darkviolet:9699539,deeppink:16716947,deepskyblue:49151,dimgray:6908265,dimgrey:6908265,dodgerblue:2003199,firebrick:11674146,floralwhite:16775920,forestgreen:2263842,fuchsia:16711935,gainsboro:14474460,ghostwhite:16316671,gold:16766720,goldenrod:14329120,gray:8421504,green:32768,greenyellow:11403055,grey:8421504,honeydew:15794160,hotpink:16738740,indianred:13458524,indigo:4915330,ivory:16777200,khaki:15787660,lavender:15132410,lavenderblush:16773365,lawngreen:8190976,lemonchiffon:16775885,lightblue:11393254,lightcoral:15761536,lightcyan:14745599,lightgoldenrodyellow:16448210,lightgray:13882323,lightgreen:9498256,lightgrey:13882323,lightpink:16758465,lightsalmon:16752762,lightseagreen:2142890,lightskyblue:8900346,lightslategray:7833753,lightslategrey:7833753,lightsteelblue:11584734,lightyellow:16777184,lime:65280,limegreen:3329330,linen:16445670,magenta:16711935,maroon:8388608,mediumaquamarine:6737322,mediumblue:205,mediumorchid:12211667,mediumpurple:9662683,mediumseagreen:3978097,mediumslateblue:8087790,mediumspringgreen:64154,mediumturquoise:4772300,mediumvioletred:13047173,midnightblue:1644912,mintcream:16121850,mistyrose:16770273,moccasin:16770229,navajowhite:16768685,navy:128,oldlace:16643558,olive:8421376,olivedrab:7048739,orange:16753920,orangered:16729344,orchid:14315734,palegoldenrod:15657130,palegreen:10025880,paleturquoise:11529966,palevioletred:14381203,papayawhip:16773077,peachpuff:16767673,peru:13468991,pink:16761035,plum:14524637,powderblue:11591910,purple:8388736,rebeccapurple:6697881,red:16711680,rosybrown:12357519,royalblue:4286945,saddlebrown:9127187,salmon:16416882,sandybrown:16032864,seagreen:3050327,seashell:16774638,sienna:10506797,silver:12632256,skyblue:8900331,slateblue:6970061,slategray:7372944,slategrey:7372944,snow:16775930,springgreen:65407,steelblue:4620980,tan:13808780,teal:32896,thistle:14204888,tomato:16737095,turquoise:4251856,violet:15631086,wheat:16113331,white:16777215,whitesmoke:16119285,yellow:16776960,yellowgreen:10145074});function yt(e){return"function"==typeof e?e:function(){return e}}function bt(e){return function(t,n,r){return 2===arguments.length&&"function"==typeof n&&(r=n,n=null),Ct(t,n,e,r)}}function Ct(e,t,n,r){var i={},a=o.dispatch("beforesend","progress","load","error"),l={},u=new XMLHttpRequest,c=null;function d(){var e,t=u.status;if(!t&&function(e){var t=e.responseType;return t&&"text"!==t?e.response:e.responseText}(u)||t>=200&&t<300||304===t){try{e=n.call(i,u)}catch(e){return void a.error.call(i,e)}a.load.call(i,e)}else a.error.call(i,u)}return this.XDomainRequest&&!("withCredentials"in u)&&/^(http(s)?:)?\/\//.test(e)&&(u=new XDomainRequest),"onload"in u?u.onload=u.onerror=d:u.onreadystatechange=function(){u.readyState>3&&d()},u.onprogress=function(e){var t=o.event;o.event=e;try{a.progress.call(i,u)}finally{o.event=t}},i.header=function(e,t){return e=(e+"").toLowerCase(),arguments.length<2?l[e]:(null==t?delete l[e]:l[e]=t+"",i)},i.mimeType=function(e){return arguments.length?(t=null==e?null:e+"",i):t},i.responseType=function(e){return arguments.length?(c=e,i):c},i.response=function(e){return n=e,i},["get","post"].forEach((function(e){i[e]=function(){return i.send.apply(i,[e].concat(s(arguments)))}})),i.send=function(n,r,o){if(2===arguments.length&&"function"==typeof r&&(o=r,r=null),u.open(n,e,!0),null==t||"accept"in l||(l.accept=t+",*/*"),u.setRequestHeader)for(var s in l)u.setRequestHeader(s,l[s]);return null!=t&&u.overrideMimeType&&u.overrideMimeType(t),null!=c&&(u.responseType=c),null!=o&&i.on("error",o).on("load",(function(e){o(null,e)})),a.beforesend.call(i,u),u.send(null==r?null:r),i},i.abort=function(){return u.abort(),i},o.rebind(i,a,"on"),null==r?i:i.get(function(e){return 1===e.length?function(t,n){e(null==t?n:null)}:e}(r))}mt.forEach((function(e,t){mt.set(e,lt(t))})),o.functor=yt,o.xhr=bt(P),o.dsv=function(e,t){var n=new RegExp('["'+e+"\n]"),r=e.charCodeAt(0);function i(e,n,r){arguments.length<3&&(r=n,n=null);var i=Ct(e,t,null==n?o:a(n),r);return i.row=function(e){return arguments.length?i.response(null==(n=e)?o:a(e)):n},i}function o(e){return i.parse(e.responseText)}function a(e){return function(t){return i.parse(t.responseText,e)}}function s(t){return t.map(l).join(e)}function l(e){return n.test(e)?'"'+e.replace(/\"/g,'""')+'"':e}return i.parse=function(e,t){var n;return i.parseRows(e,(function(e,r){if(n)return n(e,r-1);var i=new Function("d","return {"+e.map((function(e,t){return JSON.stringify(e)+": d["+t+"]"})).join(",")+"}");n=t?function(e,n){return t(i(e),n)}:i}))},i.parseRows=function(e,t){var n,i,o={},a={},s=[],l=e.length,u=0,c=0;function d(){if(u>=l)return a;if(i)return i=!1,o;var t=u;if(34===e.charCodeAt(t)){for(var n=t;n++<l;)if(34===e.charCodeAt(n)){if(34!==e.charCodeAt(n+1))break;++n}return u=n+2,13===(s=e.charCodeAt(n+1))?(i=!0,10===e.charCodeAt(n+2)&&++u):10===s&&(i=!0),e.slice(t+1,n).replace(/""/g,'"')}for(;u<l;){var s,c=1;if(10===(s=e.charCodeAt(u++)))i=!0;else if(13===s)i=!0,10===e.charCodeAt(u)&&(++u,++c);else if(s!==r)continue;return e.slice(t,u-c)}return e.slice(t)}for(;(n=d())!==a;){for(var h=[];n!==o&&n!==a;)h.push(n),n=d();t&&null==(h=t(h,c++))||s.push(h)}return s},i.format=function(t){if(Array.isArray(t[0]))return i.formatRows(t);var n=new D,r=[];return t.forEach((function(e){for(var t in e)n.has(t)||r.push(n.add(t))})),[r.map(l).join(e)].concat(t.map((function(t){return r.map((function(e){return l(t[e])})).join(e)}))).join("\n")},i.formatRows=function(e){return e.map(s).join("\n")},i},o.csv=o.dsv(",","text/csv"),o.tsv=o.dsv("\t","text/tab-separated-values");var At,_t,It,xt,wt=this[j(this,"requestAnimationFrame")]||function(e){setTimeout(e,17)};function Mt(e,t,n){var r=arguments.length;r<2&&(t=0),r<3&&(n=Date.now());var i=n+t,o={c:e,t:i,n:null};return _t?_t.n=o:At=o,_t=o,It||(xt=clearTimeout(xt),It=1,wt(St)),o}function St(){var e=Tt(),t=Et()-e;t>24?(isFinite(t)&&(clearTimeout(xt),xt=setTimeout(St,t)),It=0):(It=1,wt(St))}function Tt(){for(var e=Date.now(),t=At;t;)e>=t.t&&t.c(e-t.t)&&(t.c=null),t=t.n;return e}function Et(){for(var e,t=At,n=1/0;t;)t.c?(t.t<n&&(n=t.t),t=(e=t).n):t=e?e.n=t.n:At=t.n;return _t=e,n}function Ot(e,t){return t-(e?Math.ceil(Math.log(e)/Math.LN10):1)}o.timer=function(){Mt.apply(this,arguments)},o.timer.flush=function(){Tt(),Et()},o.round=function(e,t){return t?Math.round(e*(t=Math.pow(10,t)))/t:Math.round(e)};var kt=["y","z","a","f","p","n","µ","m","","k","M","G","T","P","E","Z","Y"].map((function(e,t){var n=Math.pow(10,3*_(8-t));return{scale:t>8?function(e){return e/n}:function(e){return e*n},symbol:e}}));function Lt(e){var t=e.decimal,n=e.thousands,r=e.grouping,i=e.currency,a=r&&n?function(e,t){for(var i=e.length,o=[],a=0,s=r[0],l=0;i>0&&s>0&&(l+s+1>t&&(s=Math.max(1,t-l)),o.push(e.substring(i-=s,i+s)),!((l+=s+1)>t));)s=r[a=(a+1)%r.length];return o.reverse().join(n)}:P;return function(e){var n=Dt.exec(e),r=n[1]||" ",s=n[2]||">",l=n[3]||"-",u=n[4]||"",c=n[5],d=+n[6],h=n[7],f=n[8],p=n[9],g=1,v="",m="",y=!1,b=!0;switch(f&&(f=+f.substring(1)),(c||"0"===r&&"="===s)&&(c=r="0",s="="),p){case"n":h=!0,p="g";break;case"%":g=100,m="%",p="f";break;case"p":g=100,m="%",p="r";break;case"b":case"o":case"x":case"X":"#"===u&&(v="0"+p.toLowerCase());case"c":b=!1;case"d":y=!0,f=0;break;case"s":g=-1,p="r"}"$"===u&&(v=i[0],m=i[1]),"r"!=p||f||(p="g"),null!=f&&("g"==p?f=Math.max(1,Math.min(21,f)):"e"!=p&&"f"!=p||(f=Math.max(0,Math.min(20,f)))),p=Pt.get(p)||Nt;var C=c&&h;return function(e){var n=m;if(y&&e%1)return"";var i=e<0||0===e&&1/e<0?(e=-e,"-"):"-"===l?"":l;if(g<0){var u=o.formatPrefix(e,f);e=u.scale(e),n=u.symbol+m}else e*=g;var A,_,I=(e=p(e,f)).lastIndexOf(".");if(I<0){var x=b?e.lastIndexOf("e"):-1;x<0?(A=e,_=""):(A=e.substring(0,x),_=e.substring(x))}else A=e.substring(0,I),_=t+e.substring(I+1);!c&&h&&(A=a(A,1/0));var w=v.length+A.length+_.length+(C?0:i.length),M=w<d?new Array(w=d-w+1).join(r):"";return C&&(A=a(M+A,M.length?d-_.length:1/0)),i+=v,e=A+_,("<"===s?i+e+M:">"===s?M+i+e:"^"===s?M.substring(0,w>>=1)+i+e+M.substring(w):i+(C?e:M+e))+n}}}o.formatPrefix=function(e,t){var n=0;return(e=+e)&&(e<0&&(e*=-1),t&&(e=o.round(e,Ot(e,t))),n=1+Math.floor(1e-12+Math.log(e)/Math.LN10),n=Math.max(-24,Math.min(24,3*Math.floor((n-1)/3)))),kt[8+n/3]};var Dt=/(?:([^{])?([<>=^]))?([+\- ])?([$#])?(0)?(\d+)?(,)?(\.-?\d+)?([a-z%])?/i,Pt=o.map({b:function(e){return e.toString(2)},c:function(e){return String.fromCharCode(e)},o:function(e){return e.toString(8)},x:function(e){return e.toString(16)},X:function(e){return e.toString(16).toUpperCase()},g:function(e,t){return e.toPrecision(t)},e:function(e,t){return e.toExponential(t)},f:function(e,t){return e.toFixed(t)},r:function(e,t){return(e=o.round(e,Ot(e,t))).toFixed(Math.max(0,Math.min(20,Ot(e*(1+1e-15),t))))}});function Nt(e){return e+""}var jt=o.time={},Rt=Date;function zt(){this._=new Date(arguments.length>1?Date.UTC.apply(this,arguments):arguments[0])}zt.prototype={getDate:function(){return this._.getUTCDate()},getDay:function(){return this._.getUTCDay()},getFullYear:function(){return this._.getUTCFullYear()},getHours:function(){return this._.getUTCHours()},getMilliseconds:function(){return this._.getUTCMilliseconds()},getMinutes:function(){return this._.getUTCMinutes()},getMonth:function(){return this._.getUTCMonth()},getSeconds:function(){return this._.getUTCSeconds()},getTime:function(){return this._.getTime()},getTimezoneOffset:function(){return 0},valueOf:function(){return this._.valueOf()},setDate:function(){Ft.setUTCDate.apply(this._,arguments)},setDay:function(){Ft.setUTCDay.apply(this._,arguments)},setFullYear:function(){Ft.setUTCFullYear.apply(this._,arguments)},setHours:function(){Ft.setUTCHours.apply(this._,arguments)},setMilliseconds:function(){Ft.setUTCMilliseconds.apply(this._,arguments)},setMinutes:function(){Ft.setUTCMinutes.apply(this._,arguments)},setMonth:function(){Ft.setUTCMonth.apply(this._,arguments)},setSeconds:function(){Ft.setUTCSeconds.apply(this._,arguments)},setTime:function(){Ft.setTime.apply(this._,arguments)}};var Ft=Date.prototype;function Bt(e,t,n){function r(t){var n=e(t),r=o(n,1);return t-n<r-t?n:r}function i(n){return t(n=e(new Rt(n-1)),1),n}function o(e,n){return t(e=new Rt(+e),n),e}function a(e,r,o){var a=i(e),s=[];if(o>1)for(;a<r;)n(a)%o||s.push(new Date(+a)),t(a,1);else for(;a<r;)s.push(new Date(+a)),t(a,1);return s}e.floor=e,e.round=r,e.ceil=i,e.offset=o,e.range=a;var s=e.utc=Ht(e);return s.floor=s,s.round=Ht(r),s.ceil=Ht(i),s.offset=Ht(o),s.range=function(e,t,n){try{Rt=zt;var r=new zt;return r._=e,a(r,t,n)}finally{Rt=Date}},e}function Ht(e){return function(t,n){try{Rt=zt;var r=new zt;return r._=t,e(r,n)._}finally{Rt=Date}}}function Ut(e){var t=e.dateTime,n=e.date,r=e.time,i=e.periods,a=e.days,s=e.shortDays,l=e.months,u=e.shortMonths;function c(e){var t=e.length;function n(n){for(var r,i,o,a=[],s=-1,l=0;++s<t;)37===e.charCodeAt(s)&&(a.push(e.slice(l,s)),null!=(i=Vt[r=e.charAt(++s)])&&(r=e.charAt(++s)),(o=A[r])&&(r=o(n,null==i?"e"===r?" ":"0":i)),a.push(r),l=s+1);return a.push(e.slice(l,s)),a.join("")}return n.parse=function(t){var n={y:1900,m:0,d:1,H:0,M:0,S:0,L:0,Z:null};if(d(n,e,t,0)!=t.length)return null;"p"in n&&(n.H=n.H%12+12*n.p);var r=null!=n.Z&&Rt!==zt,i=new(r?zt:Rt);return"j"in n?i.setFullYear(n.y,0,n.j):"W"in n||"U"in n?("w"in n||(n.w="W"in n?1:0),i.setFullYear(n.y,0,1),i.setFullYear(n.y,0,"W"in n?(n.w+6)%7+7*n.W-(i.getDay()+5)%7:n.w+7*n.U-(i.getDay()+6)%7)):i.setFullYear(n.y,n.m,n.d),i.setHours(n.H+(n.Z/100|0),n.M+n.Z%100,n.S,n.L),r?i._:i},n.toString=function(){return e},n}function d(e,t,n,r){for(var i,o,a,s=0,l=t.length,u=n.length;s<l;){if(r>=u)return-1;if(37===(i=t.charCodeAt(s++))){if(a=t.charAt(s++),!(o=_[a in Vt?t.charAt(s++):a])||(r=o(e,n,r))<0)return-1}else if(i!=n.charCodeAt(r++))return-1}return r}c.utc=function(e){var t=c(e);function n(e){try{var n=new(Rt=zt);return n._=e,t(n)}finally{Rt=Date}}return n.parse=function(e){try{Rt=zt;var n=t.parse(e);return n&&n._}finally{Rt=Date}},n.toString=t.toString,n},c.multi=c.utc.multi=dn;var h=o.map(),f=Xt(a),p=qt(a),g=Xt(s),v=qt(s),m=Xt(l),y=qt(l),b=Xt(u),C=qt(u);i.forEach((function(e,t){h.set(e.toLowerCase(),t)}));var A={a:function(e){return s[e.getDay()]},A:function(e){return a[e.getDay()]},b:function(e){return u[e.getMonth()]},B:function(e){return l[e.getMonth()]},c:c(t),d:function(e,t){return Gt(e.getDate(),t,2)},e:function(e,t){return Gt(e.getDate(),t,2)},H:function(e,t){return Gt(e.getHours(),t,2)},I:function(e,t){return Gt(e.getHours()%12||12,t,2)},j:function(e,t){return Gt(1+jt.dayOfYear(e),t,3)},L:function(e,t){return Gt(e.getMilliseconds(),t,3)},m:function(e,t){return Gt(e.getMonth()+1,t,2)},M:function(e,t){return Gt(e.getMinutes(),t,2)},p:function(e){return i[+(e.getHours()>=12)]},S:function(e,t){return Gt(e.getSeconds(),t,2)},U:function(e,t){return Gt(jt.sundayOfYear(e),t,2)},w:function(e){return e.getDay()},W:function(e,t){return Gt(jt.mondayOfYear(e),t,2)},x:c(n),X:c(r),y:function(e,t){return Gt(e.getFullYear()%100,t,2)},Y:function(e,t){return Gt(e.getFullYear()%1e4,t,4)},Z:un,"%":function(){return"%"}},_={a:function(e,t,n){g.lastIndex=0;var r=g.exec(t.slice(n));return r?(e.w=v.get(r[0].toLowerCase()),n+r[0].length):-1},A:function(e,t,n){f.lastIndex=0;var r=f.exec(t.slice(n));return r?(e.w=p.get(r[0].toLowerCase()),n+r[0].length):-1},b:function(e,t,n){b.lastIndex=0;var r=b.exec(t.slice(n));return r?(e.m=C.get(r[0].toLowerCase()),n+r[0].length):-1},B:function(e,t,n){m.lastIndex=0;var r=m.exec(t.slice(n));return r?(e.m=y.get(r[0].toLowerCase()),n+r[0].length):-1},c:function(e,t,n){return d(e,A.c.toString(),t,n)},d:nn,e:nn,H:on,I:on,j:rn,L:ln,m:tn,M:an,p:function(e,t,n){var r=h.get(t.slice(n,n+=2).toLowerCase());return null==r?-1:(e.p=r,n)},S:sn,U:Qt,w:Zt,W:Kt,x:function(e,t,n){return d(e,A.x.toString(),t,n)},X:function(e,t,n){return d(e,A.X.toString(),t,n)},y:$t,Y:Jt,Z:en,"%":cn};return c}jt.year=Bt((function(e){return(e=jt.day(e)).setMonth(0,1),e}),(function(e,t){e.setFullYear(e.getFullYear()+t)}),(function(e){return e.getFullYear()})),jt.years=jt.year.range,jt.years.utc=jt.year.utc.range,jt.day=Bt((function(e){var t=new Rt(2e3,0);return t.setFullYear(e.getFullYear(),e.getMonth(),e.getDate()),t}),(function(e,t){e.setDate(e.getDate()+t)}),(function(e){return e.getDate()-1})),jt.days=jt.day.range,jt.days.utc=jt.day.utc.range,jt.dayOfYear=function(e){var t=jt.year(e);return Math.floor((e-t-6e4*(e.getTimezoneOffset()-t.getTimezoneOffset()))/864e5)},["sunday","monday","tuesday","wednesday","thursday","friday","saturday"].forEach((function(e,t){t=7-t;var n=jt[e]=Bt((function(e){return(e=jt.day(e)).setDate(e.getDate()-(e.getDay()+t)%7),e}),(function(e,t){e.setDate(e.getDate()+7*Math.floor(t))}),(function(e){var n=jt.year(e).getDay();return Math.floor((jt.dayOfYear(e)+(n+t)%7)/7)-(n!==t)}));jt[e+"s"]=n.range,jt[e+"s"].utc=n.utc.range,jt[e+"OfYear"]=function(e){var n=jt.year(e).getDay();return Math.floor((jt.dayOfYear(e)+(n+t)%7)/7)}})),jt.week=jt.sunday,jt.weeks=jt.sunday.range,jt.weeks.utc=jt.sunday.utc.range,jt.weekOfYear=jt.sundayOfYear;var Vt={"-":"",_:" ",0:"0"},Yt=/^\s*\d+/,Wt=/^%/;function Gt(e,t,n){var r=e<0?"-":"",i=(r?-e:e)+"",o=i.length;return r+(o<n?new Array(n-o+1).join(t)+i:i)}function Xt(e){return new RegExp("^(?:"+e.map(o.requote).join("|")+")","i")}function qt(e){for(var t=new w,n=-1,r=e.length;++n<r;)t.set(e[n].toLowerCase(),n);return t}function Zt(e,t,n){Yt.lastIndex=0;var r=Yt.exec(t.slice(n,n+1));return r?(e.w=+r[0],n+r[0].length):-1}function Qt(e,t,n){Yt.lastIndex=0;var r=Yt.exec(t.slice(n));return r?(e.U=+r[0],n+r[0].length):-1}function Kt(e,t,n){Yt.lastIndex=0;var r=Yt.exec(t.slice(n));return r?(e.W=+r[0],n+r[0].length):-1}function Jt(e,t,n){Yt.lastIndex=0;var r=Yt.exec(t.slice(n,n+4));return r?(e.y=+r[0],n+r[0].length):-1}function $t(e,t,n){Yt.lastIndex=0;var r,i=Yt.exec(t.slice(n,n+2));return i?(e.y=(r=+i[0])+(r>68?1900:2e3),n+i[0].length):-1}function en(e,t,n){return/^[+-]\d{4}$/.test(t=t.slice(n,n+5))?(e.Z=-t,n+5):-1}function tn(e,t,n){Yt.lastIndex=0;var r=Yt.exec(t.slice(n,n+2));return r?(e.m=r[0]-1,n+r[0].length):-1}function nn(e,t,n){Yt.lastIndex=0;var r=Yt.exec(t.slice(n,n+2));return r?(e.d=+r[0],n+r[0].length):-1}function rn(e,t,n){Yt.lastIndex=0;var r=Yt.exec(t.slice(n,n+3));return r?(e.j=+r[0],n+r[0].length):-1}function on(e,t,n){Yt.lastIndex=0;var r=Yt.exec(t.slice(n,n+2));return r?(e.H=+r[0],n+r[0].length):-1}function an(e,t,n){Yt.lastIndex=0;var r=Yt.exec(t.slice(n,n+2));return r?(e.M=+r[0],n+r[0].length):-1}function sn(e,t,n){Yt.lastIndex=0;var r=Yt.exec(t.slice(n,n+2));return r?(e.S=+r[0],n+r[0].length):-1}function ln(e,t,n){Yt.lastIndex=0;var r=Yt.exec(t.slice(n,n+3));return r?(e.L=+r[0],n+r[0].length):-1}function un(e){var t=e.getTimezoneOffset(),n=t>0?"-":"+",r=_(t)/60|0,i=_(t)%60;return n+Gt(r,"0",2)+Gt(i,"0",2)}function cn(e,t,n){Wt.lastIndex=0;var r=Wt.exec(t.slice(n,n+1));return r?n+r[0].length:-1}function dn(e){for(var t=e.length,n=-1;++n<t;)e[n][0]=this(e[n][0]);return function(t){for(var n=0,r=e[n];!r[1](t);)r=e[++n];return r[0](t)}}o.locale=function(e){return{numberFormat:Lt(e),timeFormat:Ut(e)}};var hn=o.locale({decimal:".",thousands:",",grouping:[3],currency:["$",""],dateTime:"%a %b %e %X %Y",date:"%m/%d/%Y",time:"%H:%M:%S",periods:["AM","PM"],days:["Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"],shortDays:["Sun","Mon","Tue","Wed","Thu","Fri","Sat"],months:["January","February","March","April","May","June","July","August","September","October","November","December"],shortMonths:["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"]});function fn(){}o.format=hn.numberFormat,o.geo={},fn.prototype={s:0,t:0,add:function(e){gn(e,this.t,pn),gn(pn.s,this.s,this),this.s?this.t+=pn.t:this.s=pn.t},reset:function(){this.s=this.t=0},valueOf:function(){return this.s}};var pn=new fn;function gn(e,t,n){var r=n.s=e+t,i=r-e,o=r-i;n.t=e-o+(t-i)}function vn(e,t){e&&yn.hasOwnProperty(e.type)&&yn[e.type](e,t)}o.geo.stream=function(e,t){e&&mn.hasOwnProperty(e.type)?mn[e.type](e,t):vn(e,t)};var mn={Feature:function(e,t){vn(e.geometry,t)},FeatureCollection:function(e,t){for(var n=e.features,r=-1,i=n.length;++r<i;)vn(n[r].geometry,t)}},yn={Sphere:function(e,t){t.sphere()},Point:function(e,t){e=e.coordinates,t.point(e[0],e[1],e[2])},MultiPoint:function(e,t){for(var n=e.coordinates,r=-1,i=n.length;++r<i;)e=n[r],t.point(e[0],e[1],e[2])},LineString:function(e,t){bn(e.coordinates,t,0)},MultiLineString:function(e,t){for(var n=e.coordinates,r=-1,i=n.length;++r<i;)bn(n[r],t,0)},Polygon:function(e,t){Cn(e.coordinates,t)},MultiPolygon:function(e,t){for(var n=e.coordinates,r=-1,i=n.length;++r<i;)Cn(n[r],t)},GeometryCollection:function(e,t){for(var n=e.geometries,r=-1,i=n.length;++r<i;)vn(n[r],t)}};function bn(e,t,n){var r,i=-1,o=e.length-n;for(t.lineStart();++i<o;)r=e[i],t.point(r[0],r[1],r[2]);t.lineEnd()}function Cn(e,t){var n=-1,r=e.length;for(t.polygonStart();++n<r;)bn(e[n],t,1);t.polygonEnd()}o.geo.area=function(e){return An=0,o.geo.stream(e,Pn),An};var An,_n,In,xn,wn,Mn,Sn,Tn,En,On,kn,Ln,Dn=new fn,Pn={sphere:function(){An+=4*Oe},point:z,lineStart:z,lineEnd:z,polygonStart:function(){Dn.reset(),Pn.lineStart=Nn},polygonEnd:function(){var e=2*Dn;An+=e<0?4*Oe+e:e,Pn.lineStart=Pn.lineEnd=Pn.point=z}};function Nn(){var e,t,n,r,i;function o(e,t){t=t*Pe/2+Oe/4;var o=(e*=Pe)-n,a=o>=0?1:-1,s=a*o,l=Math.cos(t),u=Math.sin(t),c=i*u,d=r*l+c*Math.cos(s),h=c*a*Math.sin(s);Dn.add(Math.atan2(h,d)),n=e,r=l,i=u}Pn.point=function(a,s){Pn.point=o,n=(e=a)*Pe,r=Math.cos(s=(t=s)*Pe/2+Oe/4),i=Math.sin(s)},Pn.lineEnd=function(){o(e,t)}}function jn(e){var t=e[0],n=e[1],r=Math.cos(n);return[r*Math.cos(t),r*Math.sin(t),Math.sin(n)]}function Rn(e,t){return e[0]*t[0]+e[1]*t[1]+e[2]*t[2]}function zn(e,t){return[e[1]*t[2]-e[2]*t[1],e[2]*t[0]-e[0]*t[2],e[0]*t[1]-e[1]*t[0]]}function Fn(e,t){e[0]+=t[0],e[1]+=t[1],e[2]+=t[2]}function Bn(e,t){return[e[0]*t,e[1]*t,e[2]*t]}function Hn(e){var t=Math.sqrt(e[0]*e[0]+e[1]*e[1]+e[2]*e[2]);e[0]/=t,e[1]/=t,e[2]/=t}function Un(e){return[Math.atan2(e[1],e[0]),Fe(e[2])]}function Vn(e,t){return _(e[0]-t[0])<Te&&_(e[1]-t[1])<Te}o.geo.bounds=function(){var e,t,n,r,i,a,s,l,u,c,d,h={point:f,lineStart:g,lineEnd:v,polygonStart:function(){h.point=m,h.lineStart=y,h.lineEnd=b,u=0,Pn.polygonStart()},polygonEnd:function(){Pn.polygonEnd(),h.point=f,h.lineStart=g,h.lineEnd=v,Dn<0?(e=-(n=180),t=-(r=90)):u>Te?r=90:u<-Te&&(t=-90),d[0]=e,d[1]=n}};function f(i,o){c.push(d=[e=i,n=i]),o<t&&(t=o),o>r&&(r=o)}function p(o,a){var s=jn([o*Pe,a*Pe]);if(l){var u=zn(l,s),c=zn([u[1],-u[0],0],u);Hn(c),c=Un(c);var d=o-i,h=d>0?1:-1,p=c[0]*Ne*h,g=_(d)>180;if(g^(h*i<p&&p<h*o))(v=c[1]*Ne)>r&&(r=v);else if(g^(h*i<(p=(p+360)%360-180)&&p<h*o)){var v;(v=-c[1]*Ne)<t&&(t=v)}else a<t&&(t=a),a>r&&(r=a);g?o<i?C(e,o)>C(e,n)&&(n=o):C(o,n)>C(e,n)&&(e=o):n>=e?(o<e&&(e=o),o>n&&(n=o)):o>i?C(e,o)>C(e,n)&&(n=o):C(o,n)>C(e,n)&&(e=o)}else f(o,a);l=s,i=o}function g(){h.point=p}function v(){d[0]=e,d[1]=n,h.point=f,l=null}function m(e,t){if(l){var n=e-i;u+=_(n)>180?n+(n>0?360:-360):n}else a=e,s=t;Pn.point(e,t),p(e,t)}function y(){Pn.lineStart()}function b(){m(a,s),Pn.lineEnd(),_(u)>Te&&(e=-(n=180)),d[0]=e,d[1]=n,l=null}function C(e,t){return(t-=e)<0?t+360:t}function A(e,t){return e[0]-t[0]}function I(e,t){return t[0]<=t[1]?t[0]<=e&&e<=t[1]:e<t[0]||t[1]<e}return function(i){if(r=n=-(e=t=1/0),c=[],o.geo.stream(i,h),u=c.length){c.sort(A);for(var a=1,s=[g=c[0]];a<u;++a)I((f=c[a])[0],g)||I(f[1],g)?(C(g[0],f[1])>C(g[0],g[1])&&(g[1]=f[1]),C(f[0],g[1])>C(g[0],g[1])&&(g[0]=f[0])):s.push(g=f);for(var l,u,f,p=-1/0,g=(a=0,s[u=s.length-1]);a<=u;g=f,++a)f=s[a],(l=C(g[1],f[0]))>p&&(p=l,e=f[0],n=g[1])}return c=d=null,e===1/0||t===1/0?[[NaN,NaN],[NaN,NaN]]:[[e,t],[n,r]]}}(),o.geo.centroid=function(e){_n=In=xn=wn=Mn=Sn=Tn=En=On=kn=Ln=0,o.geo.stream(e,Yn);var t=On,n=kn,r=Ln,i=t*t+n*n+r*r;return i<Ee&&(t=Sn,n=Tn,r=En,In<Te&&(t=xn,n=wn,r=Mn),(i=t*t+n*n+r*r)<Ee)?[NaN,NaN]:[Math.atan2(n,t)*Ne,Fe(r/Math.sqrt(i))*Ne]};var Yn={sphere:z,point:Wn,lineStart:Xn,lineEnd:qn,polygonStart:function(){Yn.lineStart=Zn},polygonEnd:function(){Yn.lineStart=Xn}};function Wn(e,t){e*=Pe;var n=Math.cos(t*=Pe);Gn(n*Math.cos(e),n*Math.sin(e),Math.sin(t))}function Gn(e,t,n){++_n,xn+=(e-xn)/_n,wn+=(t-wn)/_n,Mn+=(n-Mn)/_n}function Xn(){var e,t,n;function r(r,i){r*=Pe;var o=Math.cos(i*=Pe),a=o*Math.cos(r),s=o*Math.sin(r),l=Math.sin(i),u=Math.atan2(Math.sqrt((u=t*l-n*s)*u+(u=n*a-e*l)*u+(u=e*s-t*a)*u),e*a+t*s+n*l);In+=u,Sn+=u*(e+(e=a)),Tn+=u*(t+(t=s)),En+=u*(n+(n=l)),Gn(e,t,n)}Yn.point=function(i,o){i*=Pe;var a=Math.cos(o*=Pe);e=a*Math.cos(i),t=a*Math.sin(i),n=Math.sin(o),Yn.point=r,Gn(e,t,n)}}function qn(){Yn.point=Wn}function Zn(){var e,t,n,r,i;function o(e,t){e*=Pe;var o=Math.cos(t*=Pe),a=o*Math.cos(e),s=o*Math.sin(e),l=Math.sin(t),u=r*l-i*s,c=i*a-n*l,d=n*s-r*a,h=Math.sqrt(u*u+c*c+d*d),f=n*a+r*s+i*l,p=h&&-ze(f)/h,g=Math.atan2(h,f);On+=p*u,kn+=p*c,Ln+=p*d,In+=g,Sn+=g*(n+(n=a)),Tn+=g*(r+(r=s)),En+=g*(i+(i=l)),Gn(n,r,i)}Yn.point=function(a,s){e=a,t=s,Yn.point=o,a*=Pe;var l=Math.cos(s*=Pe);n=l*Math.cos(a),r=l*Math.sin(a),i=Math.sin(s),Gn(n,r,i)},Yn.lineEnd=function(){o(e,t),Yn.lineEnd=qn,Yn.point=Wn}}function Qn(e,t){function n(n,r){return n=e(n,r),t(n[0],n[1])}return e.invert&&t.invert&&(n.invert=function(n,r){return(n=t.invert(n,r))&&e.invert(n[0],n[1])}),n}function Kn(){return!0}function Jn(e,t,n,r,i){var o=[],a=[];if(e.forEach((function(e){if(!((t=e.length-1)<=0)){var t,n=e[0],r=e[t];if(Vn(n,r)){i.lineStart();for(var s=0;s<t;++s)i.point((n=e[s])[0],n[1]);i.lineEnd()}else{var l=new er(n,e,null,!0),u=new er(n,null,l,!1);l.o=u,o.push(l),a.push(u),l=new er(r,e,null,!1),u=new er(r,null,l,!0),l.o=u,o.push(l),a.push(u)}}})),a.sort(t),$n(o),$n(a),o.length){for(var s=0,l=n,u=a.length;s<u;++s)a[s].e=l=!l;for(var c,d,h=o[0];;){for(var f=h,p=!0;f.v;)if((f=f.n)===h)return;c=f.z,i.lineStart();do{if(f.v=f.o.v=!0,f.e){if(p)for(s=0,u=c.length;s<u;++s)i.point((d=c[s])[0],d[1]);else r(f.x,f.n.x,1,i);f=f.n}else{if(p)for(s=(c=f.p.z).length-1;s>=0;--s)i.point((d=c[s])[0],d[1]);else r(f.x,f.p.x,-1,i);f=f.p}c=(f=f.o).z,p=!p}while(!f.v);i.lineEnd()}}}function $n(e){if(t=e.length){for(var t,n,r=0,i=e[0];++r<t;)i.n=n=e[r],n.p=i,i=n;i.n=n=e[0],n.p=i}}function er(e,t,n,r){this.x=e,this.z=t,this.o=n,this.e=r,this.v=!1,this.n=this.p=null}function tr(e,t,n,r){return function(i,a){var s,l=t(a),u=i.invert(r[0],r[1]),c={point:d,lineStart:f,lineEnd:p,polygonStart:function(){c.point=C,c.lineStart=A,c.lineEnd=_,s=[],g=[]},polygonEnd:function(){c.point=d,c.lineStart=f,c.lineEnd=p,s=o.merge(s);var e=function(e,t){var n=e[0],r=e[1],i=[Math.sin(n),-Math.cos(n),0],o=0,a=0;Dn.reset();for(var s=0,l=t.length;s<l;++s){var u=t[s],c=u.length;if(c)for(var d=u[0],h=d[0],f=d[1]/2+Oe/4,p=Math.sin(f),g=Math.cos(f),v=1;;){v===c&&(v=0);var m=(e=u[v])[0],y=e[1]/2+Oe/4,b=Math.sin(y),C=Math.cos(y),A=m-h,_=A>=0?1:-1,I=_*A,x=I>Oe,w=p*b;if(Dn.add(Math.atan2(w*_*Math.sin(I),g*C+w*Math.cos(I))),o+=x?A+_*ke:A,x^h>=n^m>=n){var M=zn(jn(d),jn(e));Hn(M);var S=zn(i,M);Hn(S);var T=(x^A>=0?-1:1)*Fe(S[2]);(r>T||r===T&&(M[0]||M[1]))&&(a+=x^A>=0?1:-1)}if(!v++)break;h=m,p=b,g=C,d=e}}return(o<-Te||o<Te&&Dn<-Te)^1&a}(u,g);s.length?(b||(a.polygonStart(),b=!0),Jn(s,ir,e,n,a)):e&&(b||(a.polygonStart(),b=!0),a.lineStart(),n(null,null,1,a),a.lineEnd()),b&&(a.polygonEnd(),b=!1),s=g=null},sphere:function(){a.polygonStart(),a.lineStart(),n(null,null,1,a),a.lineEnd(),a.polygonEnd()}};function d(t,n){var r=i(t,n);e(t=r[0],n=r[1])&&a.point(t,n)}function h(e,t){var n=i(e,t);l.point(n[0],n[1])}function f(){c.point=h,l.lineStart()}function p(){c.point=d,l.lineEnd()}var g,v,m=rr(),y=t(m),b=!1;function C(e,t){v.push([e,t]);var n=i(e,t);y.point(n[0],n[1])}function A(){y.lineStart(),v=[]}function _(){C(v[0][0],v[0][1]),y.lineEnd();var e,t=y.clean(),n=m.buffer(),r=n.length;if(v.pop(),g.push(v),v=null,r)if(1&t){var i,o=-1;if((r=(e=n[0]).length-1)>0){for(b||(a.polygonStart(),b=!0),a.lineStart();++o<r;)a.point((i=e[o])[0],i[1]);a.lineEnd()}}else r>1&&2&t&&n.push(n.pop().concat(n.shift())),s.push(n.filter(nr))}return c}}function nr(e){return e.length>1}function rr(){var e,t=[];return{lineStart:function(){t.push(e=[])},point:function(t,n){e.push([t,n])},lineEnd:z,buffer:function(){var n=t;return t=[],e=null,n},rejoin:function(){t.length>1&&t.push(t.pop().concat(t.shift()))}}}function ir(e,t){return((e=e.x)[0]<0?e[1]-De-Te:De-e[1])-((t=t.x)[0]<0?t[1]-De-Te:De-t[1])}var or=tr(Kn,(function(e){var t,n=NaN,r=NaN,i=NaN;return{lineStart:function(){e.lineStart(),t=1},point:function(o,a){var s=o>0?Oe:-Oe,l=_(o-n);_(l-Oe)<Te?(e.point(n,r=(r+a)/2>0?De:-De),e.point(i,r),e.lineEnd(),e.lineStart(),e.point(s,r),e.point(o,r),t=0):i!==s&&l>=Oe&&(_(n-i)<Te&&(n-=i*Te),_(o-s)<Te&&(o-=s*Te),r=function(e,t,n,r){var i,o,a=Math.sin(e-n);return _(a)>Te?Math.atan((Math.sin(t)*(o=Math.cos(r))*Math.sin(n)-Math.sin(r)*(i=Math.cos(t))*Math.sin(e))/(i*o*a)):(t+r)/2}(n,r,o,a),e.point(i,r),e.lineEnd(),e.lineStart(),e.point(s,r),t=0),e.point(n=o,r=a),i=s},lineEnd:function(){e.lineEnd(),n=r=NaN},clean:function(){return 2-t}}}),(function(e,t,n,r){var i;if(null==e)i=n*De,r.point(-Oe,i),r.point(0,i),r.point(Oe,i),r.point(Oe,0),r.point(Oe,-i),r.point(0,-i),r.point(-Oe,-i),r.point(-Oe,0),r.point(-Oe,i);else if(_(e[0]-t[0])>Te){var o=e[0]<t[0]?Oe:-Oe;i=n*o/2,r.point(-o,i),r.point(0,i),r.point(o,i)}else r.point(t[0],t[1])}),[-Oe,-Oe/2]);function ar(e){var t=Math.cos(e),n=t>0,r=_(t)>Te;return tr(i,(function(e){var t,s,l,u,c;return{lineStart:function(){u=l=!1,c=1},point:function(d,h){var f,p=[d,h],g=i(d,h),v=n?g?0:a(d,h):g?a(d+(d<0?Oe:-Oe),h):0;if(!t&&(u=l=g)&&e.lineStart(),g!==l&&(f=o(t,p),(Vn(t,f)||Vn(p,f))&&(p[0]+=Te,p[1]+=Te,g=i(p[0],p[1]))),g!==l)c=0,g?(e.lineStart(),f=o(p,t),e.point(f[0],f[1])):(f=o(t,p),e.point(f[0],f[1]),e.lineEnd()),t=f;else if(r&&t&&n^g){var m;v&s||!(m=o(p,t,!0))||(c=0,n?(e.lineStart(),e.point(m[0][0],m[0][1]),e.point(m[1][0],m[1][1]),e.lineEnd()):(e.point(m[1][0],m[1][1]),e.lineEnd(),e.lineStart(),e.point(m[0][0],m[0][1])))}!g||t&&Vn(t,p)||e.point(p[0],p[1]),t=p,l=g,s=v},lineEnd:function(){l&&e.lineEnd(),t=null},clean:function(){return c|(u&&l)<<1}}}),Ur(e,6*Pe),n?[0,-e]:[-Oe,e-Oe]);function i(e,n){return Math.cos(e)*Math.cos(n)>t}function o(e,n,r){var i=[1,0,0],o=zn(jn(e),jn(n)),a=Rn(o,o),s=o[0],l=a-s*s;if(!l)return!r&&e;var u=t*a/l,c=-t*s/l,d=zn(i,o),h=Bn(i,u);Fn(h,Bn(o,c));var f=d,p=Rn(h,f),g=Rn(f,f),v=p*p-g*(Rn(h,h)-1);if(!(v<0)){var m=Math.sqrt(v),y=Bn(f,(-p-m)/g);if(Fn(y,h),y=Un(y),!r)return y;var b,C=e[0],A=n[0],I=e[1],x=n[1];A<C&&(b=C,C=A,A=b);var w=A-C,M=_(w-Oe)<Te;if(!M&&x<I&&(b=I,I=x,x=b),M||w<Te?M?I+x>0^y[1]<(_(y[0]-C)<Te?I:x):I<=y[1]&&y[1]<=x:w>Oe^(C<=y[0]&&y[0]<=A)){var S=Bn(f,(-p+m)/g);return Fn(S,h),[y,Un(S)]}}}function a(t,r){var i=n?e:Oe-e,o=0;return t<-i?o|=1:t>i&&(o|=2),r<-i?o|=4:r>i&&(o|=8),o}}function sr(e,t,n,r){return function(i){var o,a=i.a,s=i.b,l=a.x,u=a.y,c=0,d=1,h=s.x-l,f=s.y-u;if(o=e-l,h||!(o>0)){if(o/=h,h<0){if(o<c)return;o<d&&(d=o)}else if(h>0){if(o>d)return;o>c&&(c=o)}if(o=n-l,h||!(o<0)){if(o/=h,h<0){if(o>d)return;o>c&&(c=o)}else if(h>0){if(o<c)return;o<d&&(d=o)}if(o=t-u,f||!(o>0)){if(o/=f,f<0){if(o<c)return;o<d&&(d=o)}else if(f>0){if(o>d)return;o>c&&(c=o)}if(o=r-u,f||!(o<0)){if(o/=f,f<0){if(o>d)return;o>c&&(c=o)}else if(f>0){if(o<c)return;o<d&&(d=o)}return c>0&&(i.a={x:l+c*h,y:u+c*f}),d<1&&(i.b={x:l+d*h,y:u+d*f}),i}}}}}}function lr(e,t,n,r){return function(l){var u,c,d,h,f,p,g,v,m,y,b,C=l,A=rr(),_=sr(e,t,n,r),I={point:M,lineStart:function(){I.point=S,c&&c.push(d=[]);y=!0,m=!1,g=v=NaN},lineEnd:function(){u&&(S(h,f),p&&m&&A.rejoin(),u.push(A.buffer()));I.point=M,m&&l.lineEnd()},polygonStart:function(){l=A,u=[],c=[],b=!0},polygonEnd:function(){l=C,u=o.merge(u);var t=function(e){for(var t=0,n=c.length,r=e[1],i=0;i<n;++i)for(var o,a=1,s=c[i],l=s.length,u=s[0];a<l;++a)o=s[a],u[1]<=r?o[1]>r&&Re(u,o,e)>0&&++t:o[1]<=r&&Re(u,o,e)<0&&--t,u=o;return 0!==t}([e,r]),n=b&&t,i=u.length;(n||i)&&(l.polygonStart(),n&&(l.lineStart(),x(null,null,1,l),l.lineEnd()),i&&Jn(u,a,t,x,l),l.polygonEnd()),u=c=d=null}};function x(o,a,l,u){var c=0,d=0;if(null==o||(c=i(o,l))!==(d=i(a,l))||s(o,a)<0^l>0)do{u.point(0===c||3===c?e:n,c>1?r:t)}while((c=(c+l+4)%4)!==d);else u.point(a[0],a[1])}function w(i,o){return e<=i&&i<=n&&t<=o&&o<=r}function M(e,t){w(e,t)&&l.point(e,t)}function S(e,t){var n=w(e=Math.max(-1e9,Math.min(1e9,e)),t=Math.max(-1e9,Math.min(1e9,t)));if(c&&d.push([e,t]),y)h=e,f=t,p=n,y=!1,n&&(l.lineStart(),l.point(e,t));else if(n&&m)l.point(e,t);else{var r={a:{x:g,y:v},b:{x:e,y:t}};_(r)?(m||(l.lineStart(),l.point(r.a.x,r.a.y)),l.point(r.b.x,r.b.y),n||l.lineEnd(),b=!1):n&&(l.lineStart(),l.point(e,t),b=!1)}g=e,v=t,m=n}return I};function i(r,i){return _(r[0]-e)<Te?i>0?0:3:_(r[0]-n)<Te?i>0?2:1:_(r[1]-t)<Te?i>0?1:0:i>0?3:2}function a(e,t){return s(e.x,t.x)}function s(e,t){var n=i(e,1),r=i(t,1);return n!==r?n-r:0===n?t[1]-e[1]:1===n?e[0]-t[0]:2===n?e[1]-t[1]:t[0]-e[0]}}function ur(e){var t=0,n=Oe/3,r=Pr(e),i=r(t,n);return i.parallels=function(e){return arguments.length?r(t=e[0]*Oe/180,n=e[1]*Oe/180):[t/Oe*180,n/Oe*180]},i}function cr(e,t){var n=Math.sin(e),r=(n+Math.sin(t))/2,i=1+n*(2*r-n),o=Math.sqrt(i)/r;function a(e,t){var n=Math.sqrt(i-2*r*Math.sin(t))/r;return[n*Math.sin(e*=r),o-n*Math.cos(e)]}return a.invert=function(e,t){var n=o-t;return[Math.atan2(e,n)/r,Fe((i-(e*e+n*n)*r*r)/(2*r))]},a}o.geo.clipExtent=function(){var e,t,n,r,i,o,a={stream:function(e){return i&&(i.valid=!1),(i=o(e)).valid=!0,i},extent:function(s){return arguments.length?(o=lr(e=+s[0][0],t=+s[0][1],n=+s[1][0],r=+s[1][1]),i&&(i.valid=!1,i=null),a):[[e,t],[n,r]]}};return a.extent([[0,0],[960,500]])},(o.geo.conicEqualArea=function(){return ur(cr)}).raw=cr,o.geo.albers=function(){return o.geo.conicEqualArea().rotate([96,0]).center([-.6,38.7]).parallels([29.5,45.5]).scale(1070)},o.geo.albersUsa=function(){var e,t,n,r,i=o.geo.albers(),a=o.geo.conicEqualArea().rotate([154,0]).center([-2,58.5]).parallels([55,65]),s=o.geo.conicEqualArea().rotate([157,0]).center([-3,19.9]).parallels([8,18]),l={point:function(t,n){e=[t,n]}};function u(i){var o=i[0],a=i[1];return e=null,t(o,a),e||(n(o,a),e)||r(o,a),e}return u.invert=function(e){var t=i.scale(),n=i.translate(),r=(e[0]-n[0])/t,o=(e[1]-n[1])/t;return(o>=.12&&o<.234&&r>=-.425&&r<-.214?a:o>=.166&&o<.234&&r>=-.214&&r<-.115?s:i).invert(e)},u.stream=function(e){var t=i.stream(e),n=a.stream(e),r=s.stream(e);return{point:function(e,i){t.point(e,i),n.point(e,i),r.point(e,i)},sphere:function(){t.sphere(),n.sphere(),r.sphere()},lineStart:function(){t.lineStart(),n.lineStart(),r.lineStart()},lineEnd:function(){t.lineEnd(),n.lineEnd(),r.lineEnd()},polygonStart:function(){t.polygonStart(),n.polygonStart(),r.polygonStart()},polygonEnd:function(){t.polygonEnd(),n.polygonEnd(),r.polygonEnd()}}},u.precision=function(e){return arguments.length?(i.precision(e),a.precision(e),s.precision(e),u):i.precision()},u.scale=function(e){return arguments.length?(i.scale(e),a.scale(.35*e),s.scale(e),u.translate(i.translate())):i.scale()},u.translate=function(e){if(!arguments.length)return i.translate();var o=i.scale(),c=+e[0],d=+e[1];return t=i.translate(e).clipExtent([[c-.455*o,d-.238*o],[c+.455*o,d+.238*o]]).stream(l).point,n=a.translate([c-.307*o,d+.201*o]).clipExtent([[c-.425*o+Te,d+.12*o+Te],[c-.214*o-Te,d+.234*o-Te]]).stream(l).point,r=s.translate([c-.205*o,d+.212*o]).clipExtent([[c-.214*o+Te,d+.166*o+Te],[c-.115*o-Te,d+.234*o-Te]]).stream(l).point,u},u.scale(1070)};var dr,hr,fr,pr,gr,vr,mr={point:z,lineStart:z,lineEnd:z,polygonStart:function(){hr=0,mr.lineStart=yr},polygonEnd:function(){mr.lineStart=mr.lineEnd=mr.point=z,dr+=_(hr/2)}};function yr(){var e,t,n,r;function i(e,t){hr+=r*e-n*t,n=e,r=t}mr.point=function(o,a){mr.point=i,e=n=o,t=r=a},mr.lineEnd=function(){i(e,t)}}var br={point:function(e,t){e<fr&&(fr=e);e>gr&&(gr=e);t<pr&&(pr=t);t>vr&&(vr=t)},lineStart:z,lineEnd:z,polygonStart:z,polygonEnd:z};function Cr(){var e=Ar(4.5),t=[],n={point:r,lineStart:function(){n.point=i},lineEnd:a,polygonStart:function(){n.lineEnd=s},polygonEnd:function(){n.lineEnd=a,n.point=r},pointRadius:function(t){return e=Ar(t),n},result:function(){if(t.length){var e=t.join("");return t=[],e}}};function r(n,r){t.push("M",n,",",r,e)}function i(e,r){t.push("M",e,",",r),n.point=o}function o(e,n){t.push("L",e,",",n)}function a(){n.point=r}function s(){t.push("Z")}return n}function Ar(e){return"m0,"+e+"a"+e+","+e+" 0 1,1 0,"+-2*e+"a"+e+","+e+" 0 1,1 0,"+2*e+"z"}var _r,Ir={point:xr,lineStart:wr,lineEnd:Mr,polygonStart:function(){Ir.lineStart=Sr},polygonEnd:function(){Ir.point=xr,Ir.lineStart=wr,Ir.lineEnd=Mr}};function xr(e,t){xn+=e,wn+=t,++Mn}function wr(){var e,t;function n(n,r){var i=n-e,o=r-t,a=Math.sqrt(i*i+o*o);Sn+=a*(e+n)/2,Tn+=a*(t+r)/2,En+=a,xr(e=n,t=r)}Ir.point=function(r,i){Ir.point=n,xr(e=r,t=i)}}function Mr(){Ir.point=xr}function Sr(){var e,t,n,r;function i(e,t){var i=e-n,o=t-r,a=Math.sqrt(i*i+o*o);Sn+=a*(n+e)/2,Tn+=a*(r+t)/2,En+=a,On+=(a=r*e-n*t)*(n+e),kn+=a*(r+t),Ln+=3*a,xr(n=e,r=t)}Ir.point=function(o,a){Ir.point=i,xr(e=n=o,t=r=a)},Ir.lineEnd=function(){i(e,t)}}function Tr(e){var t=4.5,n={point:r,lineStart:function(){n.point=i},lineEnd:a,polygonStart:function(){n.lineEnd=s},polygonEnd:function(){n.lineEnd=a,n.point=r},pointRadius:function(e){return t=e,n},result:z};function r(n,r){e.moveTo(n+t,r),e.arc(n,r,t,0,ke)}function i(t,r){e.moveTo(t,r),n.point=o}function o(t,n){e.lineTo(t,n)}function a(){n.point=r}function s(){e.closePath()}return n}function Er(e){var t=.5,n=Math.cos(30*Pe),r=16;function i(e){return(r?a:o)(e)}function o(t){return Lr(t,(function(n,r){n=e(n,r),t.point(n[0],n[1])}))}function a(t){var n,i,o,a,l,u,c,d,h,f,p,g,v={point:m,lineStart:y,lineEnd:C,polygonStart:function(){t.polygonStart(),v.lineStart=A},polygonEnd:function(){t.polygonEnd(),v.lineStart=y}};function m(n,r){n=e(n,r),t.point(n[0],n[1])}function y(){d=NaN,v.point=b,t.lineStart()}function b(n,i){var o=jn([n,i]),a=e(n,i);s(d,h,c,f,p,g,d=a[0],h=a[1],c=n,f=o[0],p=o[1],g=o[2],r,t),t.point(d,h)}function C(){v.point=m,t.lineEnd()}function A(){y(),v.point=_,v.lineEnd=I}function _(e,t){b(n=e,t),i=d,o=h,a=f,l=p,u=g,v.point=b}function I(){s(d,h,c,f,p,g,i,o,n,a,l,u,r,t),v.lineEnd=C,C()}return v}function s(r,i,o,a,l,u,c,d,h,f,p,g,v,m){var y=c-r,b=d-i,C=y*y+b*b;if(C>4*t&&v--){var A=a+f,I=l+p,x=u+g,w=Math.sqrt(A*A+I*I+x*x),M=Math.asin(x/=w),S=_(_(x)-1)<Te||_(o-h)<Te?(o+h)/2:Math.atan2(I,A),T=e(S,M),E=T[0],O=T[1],k=E-r,L=O-i,D=b*k-y*L;(D*D/C>t||_((y*k+b*L)/C-.5)>.3||a*f+l*p+u*g<n)&&(s(r,i,o,a,l,u,E,O,S,A/=w,I/=w,x,v,m),m.point(E,O),s(E,O,S,A,I,x,c,d,h,f,p,g,v,m))}}return i.precision=function(e){return arguments.length?(r=(t=e*e)>0&&16,i):Math.sqrt(t)},i}function Or(e){var t=Er((function(t,n){return e([t*Ne,n*Ne])}));return function(e){return Nr(t(e))}}function kr(e){this.stream=e}function Lr(e,t){return{point:t,sphere:function(){e.sphere()},lineStart:function(){e.lineStart()},lineEnd:function(){e.lineEnd()},polygonStart:function(){e.polygonStart()},polygonEnd:function(){e.polygonEnd()}}}function Dr(e){return Pr((function(){return e}))()}function Pr(e){var t,n,r,i,a,s,l=Er((function(e,n){return[(e=t(e,n))[0]*u+i,a-e[1]*u]})),u=150,c=480,d=250,h=0,f=0,p=0,g=0,v=0,m=or,y=P,b=null,C=null;function A(e){return[(e=r(e[0]*Pe,e[1]*Pe))[0]*u+i,a-e[1]*u]}function _(e){return(e=r.invert((e[0]-i)/u,(a-e[1])/u))&&[e[0]*Ne,e[1]*Ne]}function I(){r=Qn(n=zr(p,g,v),t);var e=t(h,f);return i=c-e[0]*u,a=d+e[1]*u,x()}function x(){return s&&(s.valid=!1,s=null),A}return A.stream=function(e){return s&&(s.valid=!1),(s=Nr(m(n,l(y(e))))).valid=!0,s},A.clipAngle=function(e){return arguments.length?(m=null==e?(b=e,or):ar((b=+e)*Pe),x()):b},A.clipExtent=function(e){return arguments.length?(C=e,y=e?lr(e[0][0],e[0][1],e[1][0],e[1][1]):P,x()):C},A.scale=function(e){return arguments.length?(u=+e,I()):u},A.translate=function(e){return arguments.length?(c=+e[0],d=+e[1],I()):[c,d]},A.center=function(e){return arguments.length?(h=e[0]%360*Pe,f=e[1]%360*Pe,I()):[h*Ne,f*Ne]},A.rotate=function(e){return arguments.length?(p=e[0]%360*Pe,g=e[1]%360*Pe,v=e.length>2?e[2]%360*Pe:0,I()):[p*Ne,g*Ne,v*Ne]},o.rebind(A,l,"precision"),function(){return t=e.apply(this,arguments),A.invert=t.invert&&_,I()}}function Nr(e){return Lr(e,(function(t,n){e.point(t*Pe,n*Pe)}))}function jr(e,t){return[e,t]}function Rr(e,t){return[e>Oe?e-ke:e<-Oe?e+ke:e,t]}function zr(e,t,n){return e?t||n?Qn(Br(e),Hr(t,n)):Br(e):t||n?Hr(t,n):Rr}function Fr(e){return function(t,n){return[(t+=e)>Oe?t-ke:t<-Oe?t+ke:t,n]}}function Br(e){var t=Fr(e);return t.invert=Fr(-e),t}function Hr(e,t){var n=Math.cos(e),r=Math.sin(e),i=Math.cos(t),o=Math.sin(t);function a(e,t){var a=Math.cos(t),s=Math.cos(e)*a,l=Math.sin(e)*a,u=Math.sin(t),c=u*n+s*r;return[Math.atan2(l*i-c*o,s*n-u*r),Fe(c*i+l*o)]}return a.invert=function(e,t){var a=Math.cos(t),s=Math.cos(e)*a,l=Math.sin(e)*a,u=Math.sin(t),c=u*i-l*o;return[Math.atan2(l*i+u*o,s*n+c*r),Fe(c*n-s*r)]},a}function Ur(e,t){var n=Math.cos(e),r=Math.sin(e);return function(i,o,a,s){var l=a*t;null!=i?(i=Vr(n,i),o=Vr(n,o),(a>0?i<o:i>o)&&(i+=a*ke)):(i=e+a*ke,o=e-.5*l);for(var u,c=i;a>0?c>o:c<o;c-=l)s.point((u=Un([n,-r*Math.cos(c),-r*Math.sin(c)]))[0],u[1])}}function Vr(e,t){var n=jn(t);n[0]-=e,Hn(n);var r=ze(-n[1]);return((-n[2]<0?-r:r)+2*Math.PI-Te)%(2*Math.PI)}function Yr(e,t,n){var r=o.range(e,t-Te,n).concat(t);return function(e){return r.map((function(t){return[e,t]}))}}function Wr(e,t,n){var r=o.range(e,t-Te,n).concat(t);return function(e){return r.map((function(t){return[t,e]}))}}function Gr(e){return e.source}function Xr(e){return e.target}o.geo.path=function(){var e,t,n,r,i,a=4.5;function s(e){return e&&("function"==typeof a&&r.pointRadius(+a.apply(this,arguments)),i&&i.valid||(i=n(r)),o.geo.stream(e,i)),r.result()}function l(){return i=null,s}return s.area=function(e){return dr=0,o.geo.stream(e,n(mr)),dr},s.centroid=function(e){return xn=wn=Mn=Sn=Tn=En=On=kn=Ln=0,o.geo.stream(e,n(Ir)),Ln?[On/Ln,kn/Ln]:En?[Sn/En,Tn/En]:Mn?[xn/Mn,wn/Mn]:[NaN,NaN]},s.bounds=function(e){return gr=vr=-(fr=pr=1/0),o.geo.stream(e,n(br)),[[fr,pr],[gr,vr]]},s.projection=function(t){return arguments.length?(n=(e=t)?t.stream||Or(t):P,l()):e},s.context=function(e){return arguments.length?(r=null==(t=e)?new Cr:new Tr(e),"function"!=typeof a&&r.pointRadius(a),l()):t},s.pointRadius=function(e){return arguments.length?(a="function"==typeof e?e:(r.pointRadius(+e),+e),s):a},s.projection(o.geo.albersUsa()).context(null)},o.geo.transform=function(e){return{stream:function(t){var n=new kr(t);for(var r in e)n[r]=e[r];return n}}},kr.prototype={point:function(e,t){this.stream.point(e,t)},sphere:function(){this.stream.sphere()},lineStart:function(){this.stream.lineStart()},lineEnd:function(){this.stream.lineEnd()},polygonStart:function(){this.stream.polygonStart()},polygonEnd:function(){this.stream.polygonEnd()}},o.geo.projection=Dr,o.geo.projectionMutator=Pr,(o.geo.equirectangular=function(){return Dr(jr)}).raw=jr.invert=jr,o.geo.rotation=function(e){function t(t){return(t=e(t[0]*Pe,t[1]*Pe))[0]*=Ne,t[1]*=Ne,t}return e=zr(e[0]%360*Pe,e[1]*Pe,e.length>2?e[2]*Pe:0),t.invert=function(t){return(t=e.invert(t[0]*Pe,t[1]*Pe))[0]*=Ne,t[1]*=Ne,t},t},Rr.invert=jr,o.geo.circle=function(){var e,t,n=[0,0],r=6;function i(){var e="function"==typeof n?n.apply(this,arguments):n,r=zr(-e[0]*Pe,-e[1]*Pe,0).invert,i=[];return t(null,null,1,{point:function(e,t){i.push(e=r(e,t)),e[0]*=Ne,e[1]*=Ne}}),{type:"Polygon",coordinates:[i]}}return i.origin=function(e){return arguments.length?(n=e,i):n},i.angle=function(n){return arguments.length?(t=Ur((e=+n)*Pe,r*Pe),i):e},i.precision=function(n){return arguments.length?(t=Ur(e*Pe,(r=+n)*Pe),i):r},i.angle(90)},o.geo.distance=function(e,t){var n,r=(t[0]-e[0])*Pe,i=e[1]*Pe,o=t[1]*Pe,a=Math.sin(r),s=Math.cos(r),l=Math.sin(i),u=Math.cos(i),c=Math.sin(o),d=Math.cos(o);return Math.atan2(Math.sqrt((n=d*a)*n+(n=u*c-l*d*s)*n),l*c+u*d*s)},o.geo.graticule=function(){var e,t,n,r,i,a,s,l,u,c,d,h,f=10,p=f,g=90,v=360,m=2.5;function y(){return{type:"MultiLineString",coordinates:b()}}function b(){return o.range(Math.ceil(r/g)*g,n,g).map(d).concat(o.range(Math.ceil(l/v)*v,s,v).map(h)).concat(o.range(Math.ceil(t/f)*f,e,f).filter((function(e){return _(e%g)>Te})).map(u)).concat(o.range(Math.ceil(a/p)*p,i,p).filter((function(e){return _(e%v)>Te})).map(c))}return y.lines=function(){return b().map((function(e){return{type:"LineString",coordinates:e}}))},y.outline=function(){return{type:"Polygon",coordinates:[d(r).concat(h(s).slice(1),d(n).reverse().slice(1),h(l).reverse().slice(1))]}},y.extent=function(e){return arguments.length?y.majorExtent(e).minorExtent(e):y.minorExtent()},y.majorExtent=function(e){return arguments.length?(r=+e[0][0],n=+e[1][0],l=+e[0][1],s=+e[1][1],r>n&&(e=r,r=n,n=e),l>s&&(e=l,l=s,s=e),y.precision(m)):[[r,l],[n,s]]},y.minorExtent=function(n){return arguments.length?(t=+n[0][0],e=+n[1][0],a=+n[0][1],i=+n[1][1],t>e&&(n=t,t=e,e=n),a>i&&(n=a,a=i,i=n),y.precision(m)):[[t,a],[e,i]]},y.step=function(e){return arguments.length?y.majorStep(e).minorStep(e):y.minorStep()},y.majorStep=function(e){return arguments.length?(g=+e[0],v=+e[1],y):[g,v]},y.minorStep=function(e){return arguments.length?(f=+e[0],p=+e[1],y):[f,p]},y.precision=function(o){return arguments.length?(m=+o,u=Yr(a,i,90),c=Wr(t,e,m),d=Yr(l,s,90),h=Wr(r,n,m),y):m},y.majorExtent([[-180,-90+Te],[180,90-Te]]).minorExtent([[-180,-80-Te],[180,80+Te]])},o.geo.greatArc=function(){var e,t,n=Gr,r=Xr;function i(){return{type:"LineString",coordinates:[e||n.apply(this,arguments),t||r.apply(this,arguments)]}}return i.distance=function(){return o.geo.distance(e||n.apply(this,arguments),t||r.apply(this,arguments))},i.source=function(t){return arguments.length?(n=t,e="function"==typeof t?null:t,i):n},i.target=function(e){return arguments.length?(r=e,t="function"==typeof e?null:e,i):r},i.precision=function(){return arguments.length?i:0},i},o.geo.interpolate=function(e,t){return n=e[0]*Pe,r=e[1]*Pe,i=t[0]*Pe,o=t[1]*Pe,a=Math.cos(r),s=Math.sin(r),l=Math.cos(o),u=Math.sin(o),c=a*Math.cos(n),d=a*Math.sin(n),h=l*Math.cos(i),f=l*Math.sin(i),p=2*Math.asin(Math.sqrt(He(o-r)+a*l*He(i-n))),g=1/Math.sin(p),(v=p?function(e){var t=Math.sin(e*=p)*g,n=Math.sin(p-e)*g,r=n*c+t*h,i=n*d+t*f,o=n*s+t*u;return[Math.atan2(i,r)*Ne,Math.atan2(o,Math.sqrt(r*r+i*i))*Ne]}:function(){return[n*Ne,r*Ne]}).distance=p,v;var n,r,i,o,a,s,l,u,c,d,h,f,p,g,v},o.geo.length=function(e){return _r=0,o.geo.stream(e,qr),_r};var qr={sphere:z,point:z,lineStart:function(){var e,t,n;function r(r,i){var o=Math.sin(i*=Pe),a=Math.cos(i),s=_((r*=Pe)-e),l=Math.cos(s);_r+=Math.atan2(Math.sqrt((s=a*Math.sin(s))*s+(s=n*o-t*a*l)*s),t*o+n*a*l),e=r,t=o,n=a}qr.point=function(i,o){e=i*Pe,t=Math.sin(o*=Pe),n=Math.cos(o),qr.point=r},qr.lineEnd=function(){qr.point=qr.lineEnd=z}},lineEnd:z,polygonStart:z,polygonEnd:z};function Zr(e,t){function n(t,n){var r=Math.cos(t),i=Math.cos(n),o=e(r*i);return[o*i*Math.sin(t),o*Math.sin(n)]}return n.invert=function(e,n){var r=Math.sqrt(e*e+n*n),i=t(r),o=Math.sin(i),a=Math.cos(i);return[Math.atan2(e*o,r*a),Math.asin(r&&n*o/r)]},n}var Qr=Zr((function(e){return Math.sqrt(2/(1+e))}),(function(e){return 2*Math.asin(e/2)}));(o.geo.azimuthalEqualArea=function(){return Dr(Qr)}).raw=Qr;var Kr=Zr((function(e){var t=Math.acos(e);return t&&t/Math.sin(t)}),P);function Jr(e,t){var n=Math.cos(e),r=function(e){return Math.tan(Oe/4+e/2)},i=e===t?Math.sin(e):Math.log(n/Math.cos(t))/Math.log(r(t)/r(e)),o=n*Math.pow(r(e),i)/i;if(!i)return ti;function a(e,t){o>0?t<-De+Te&&(t=-De+Te):t>De-Te&&(t=De-Te);var n=o/Math.pow(r(t),i);return[n*Math.sin(i*e),o-n*Math.cos(i*e)]}return a.invert=function(e,t){var n=o-t,r=je(i)*Math.sqrt(e*e+n*n);return[Math.atan2(e,n)/i,2*Math.atan(Math.pow(o/r,1/i))-De]},a}function $r(e,t){var n=Math.cos(e),r=e===t?Math.sin(e):(n-Math.cos(t))/(t-e),i=n/r+e;if(_(r)<Te)return jr;function o(e,t){var n=i-t;return[n*Math.sin(r*e),i-n*Math.cos(r*e)]}return o.invert=function(e,t){var n=i-t;return[Math.atan2(e,n)/r,i-je(r)*Math.sqrt(e*e+n*n)]},o}(o.geo.azimuthalEquidistant=function(){return Dr(Kr)}).raw=Kr,(o.geo.conicConformal=function(){return ur(Jr)}).raw=Jr,(o.geo.conicEquidistant=function(){return ur($r)}).raw=$r;var ei=Zr((function(e){return 1/e}),Math.atan);function ti(e,t){return[e,Math.log(Math.tan(Oe/4+t/2))]}function ni(e){var t,n=Dr(e),r=n.scale,i=n.translate,o=n.clipExtent;return n.scale=function(){var e=r.apply(n,arguments);return e===n?t?n.clipExtent(null):n:e},n.translate=function(){var e=i.apply(n,arguments);return e===n?t?n.clipExtent(null):n:e},n.clipExtent=function(e){var a=o.apply(n,arguments);if(a===n){if(t=null==e){var s=Oe*r(),l=i();o([[l[0]-s,l[1]-s],[l[0]+s,l[1]+s]])}}else t&&(a=null);return a},n.clipExtent(null)}(o.geo.gnomonic=function(){return Dr(ei)}).raw=ei,ti.invert=function(e,t){return[e,2*Math.atan(Math.exp(t))-De]},(o.geo.mercator=function(){return ni(ti)}).raw=ti;var ri=Zr((function(){return 1}),Math.asin);(o.geo.orthographic=function(){return Dr(ri)}).raw=ri;var ii=Zr((function(e){return 1/(1+e)}),(function(e){return 2*Math.atan(e)}));function oi(e,t){return[Math.log(Math.tan(Oe/4+t/2)),-e]}function ai(e){return e[0]}function si(e){return e[1]}function li(e){for(var t=e.length,n=[0,1],r=2,i=2;i<t;i++){for(;r>1&&Re(e[n[r-2]],e[n[r-1]],e[i])<=0;)--r;n[r++]=i}return n.slice(0,r)}function ui(e,t){return e[0]-t[0]||e[1]-t[1]}(o.geo.stereographic=function(){return Dr(ii)}).raw=ii,oi.invert=function(e,t){return[-t,2*Math.atan(Math.exp(e))-De]},(o.geo.transverseMercator=function(){var e=ni(oi),t=e.center,n=e.rotate;return e.center=function(e){return e?t([-e[1],e[0]]):[(e=t())[1],-e[0]]},e.rotate=function(e){return e?n([e[0],e[1],e.length>2?e[2]+90:90]):[(e=n())[0],e[1],e[2]-90]},n([0,0,90])}).raw=oi,o.geom={},o.geom.hull=function(e){var t=ai,n=si;if(arguments.length)return r(e);function r(e){if(e.length<3)return[];var r,i=yt(t),o=yt(n),a=e.length,s=[],l=[];for(r=0;r<a;r++)s.push([+i.call(this,e[r],r),+o.call(this,e[r],r),r]);for(s.sort(ui),r=0;r<a;r++)l.push([s[r][0],-s[r][1]]);var u=li(s),c=li(l),d=c[0]===u[0],h=c[c.length-1]===u[u.length-1],f=[];for(r=u.length-1;r>=0;--r)f.push(e[s[u[r]][2]]);for(r=+d;r<c.length-h;++r)f.push(e[s[c[r]][2]]);return f}return r.x=function(e){return arguments.length?(t=e,r):t},r.y=function(e){return arguments.length?(n=e,r):n},r},o.geom.polygon=function(e){return W(e,ci),e};var ci=o.geom.polygon.prototype=[];function di(e,t,n){return(n[0]-t[0])*(e[1]-t[1])<(n[1]-t[1])*(e[0]-t[0])}function hi(e,t,n,r){var i=e[0],o=n[0],a=t[0]-i,s=r[0]-o,l=e[1],u=n[1],c=t[1]-l,d=r[1]-u,h=(s*(l-u)-d*(i-o))/(d*a-s*c);return[i+h*a,l+h*c]}function fi(e){var t=e[0],n=e[e.length-1];return!(t[0]-n[0]||t[1]-n[1])}ci.area=function(){for(var e,t=-1,n=this.length,r=this[n-1],i=0;++t<n;)e=r,r=this[t],i+=e[1]*r[0]-e[0]*r[1];return.5*i},ci.centroid=function(e){var t,n,r=-1,i=this.length,o=0,a=0,s=this[i-1];for(arguments.length||(e=-1/(6*this.area()));++r<i;)t=s,s=this[r],n=t[0]*s[1]-s[0]*t[1],o+=(t[0]+s[0])*n,a+=(t[1]+s[1])*n;return[o*e,a*e]},ci.clip=function(e){for(var t,n,r,i,o,a,s=fi(e),l=-1,u=this.length-fi(this),c=this[u-1];++l<u;){for(t=e.slice(),e.length=0,i=this[l],o=t[(r=t.length-s)-1],n=-1;++n<r;)di(a=t[n],c,i)?(di(o,c,i)||e.push(hi(o,a,c,i)),e.push(a)):di(o,c,i)&&e.push(hi(o,a,c,i)),o=a;s&&e.push(e[0]),c=i}return e};var pi,gi,vi,mi,yi,bi=[],Ci=[];function Ai(){Bi(this),this.edge=this.site=this.circle=null}function _i(e){var t=bi.pop()||new Ai;return t.site=e,t}function Ii(e){Li(e),vi.remove(e),bi.push(e),Bi(e)}function xi(e){var t=e.circle,n=t.x,r=t.cy,i={x:n,y:r},o=e.P,a=e.N,s=[e];Ii(e);for(var l=o;l.circle&&_(n-l.circle.x)<Te&&_(r-l.circle.cy)<Te;)o=l.P,s.unshift(l),Ii(l),l=o;s.unshift(l),Li(l);for(var u=a;u.circle&&_(n-u.circle.x)<Te&&_(r-u.circle.cy)<Te;)a=u.N,s.push(u),Ii(u),u=a;s.push(u),Li(u);var c,d=s.length;for(c=1;c<d;++c)u=s[c],l=s[c-1],Ri(u.edge,l.site,u.site,i);l=s[0],(u=s[d-1]).edge=Ni(l.site,u.site,null,i),ki(l),ki(u)}function wi(e){for(var t,n,r,i,o=e.x,a=e.y,s=vi._;s;)if((r=Mi(s,a)-o)>Te)s=s.L;else{if(!((i=o-Si(s,a))>Te)){r>-Te?(t=s.P,n=s):i>-Te?(t=s,n=s.N):t=n=s;break}if(!s.R){t=s;break}s=s.R}var l=_i(e);if(vi.insert(t,l),t||n){if(t===n)return Li(t),n=_i(t.site),vi.insert(l,n),l.edge=n.edge=Ni(t.site,l.site),ki(t),void ki(n);if(n){Li(t),Li(n);var u=t.site,c=u.x,d=u.y,h=e.x-c,f=e.y-d,p=n.site,g=p.x-c,v=p.y-d,m=2*(h*v-f*g),y=h*h+f*f,b=g*g+v*v,C={x:(v*y-f*b)/m+c,y:(h*b-g*y)/m+d};Ri(n.edge,u,p,C),l.edge=Ni(u,e,null,C),n.edge=Ni(e,p,null,C),ki(t),ki(n)}else l.edge=Ni(t.site,l.site)}}function Mi(e,t){var n=e.site,r=n.x,i=n.y,o=i-t;if(!o)return r;var a=e.P;if(!a)return-1/0;var s=(n=a.site).x,l=n.y,u=l-t;if(!u)return s;var c=s-r,d=1/o-1/u,h=c/u;return d?(-h+Math.sqrt(h*h-2*d*(c*c/(-2*u)-l+u/2+i-o/2)))/d+r:(r+s)/2}function Si(e,t){var n=e.N;if(n)return Mi(n,t);var r=e.site;return r.y===t?r.x:1/0}function Ti(e){this.site=e,this.edges=[]}function Ei(e,t){return t.angle-e.angle}function Oi(){Bi(this),this.x=this.y=this.arc=this.site=this.cy=null}function ki(e){var t=e.P,n=e.N;if(t&&n){var r=t.site,i=e.site,o=n.site;if(r!==o){var a=i.x,s=i.y,l=r.x-a,u=r.y-s,c=o.x-a,d=2*(l*(v=o.y-s)-u*c);if(!(d>=-Ee)){var h=l*l+u*u,f=c*c+v*v,p=(v*h-u*f)/d,g=(l*f-c*h)/d,v=g+s,m=Ci.pop()||new Oi;m.arc=e,m.site=i,m.x=p+a,m.y=v+Math.sqrt(p*p+g*g),m.cy=v,e.circle=m;for(var y=null,b=yi._;b;)if(m.y<b.y||m.y===b.y&&m.x<=b.x){if(!b.L){y=b.P;break}b=b.L}else{if(!b.R){y=b;break}b=b.R}yi.insert(y,m),y||(mi=m)}}}}function Li(e){var t=e.circle;t&&(t.P||(mi=t.N),yi.remove(t),Ci.push(t),Bi(t),e.circle=null)}function Di(e,t){var n=e.b;if(n)return!0;var r,i,o=e.a,a=t[0][0],s=t[1][0],l=t[0][1],u=t[1][1],c=e.l,d=e.r,h=c.x,f=c.y,p=d.x,g=d.y,v=(h+p)/2,m=(f+g)/2;if(g===f){if(v<a||v>=s)return;if(h>p){if(o){if(o.y>=u)return}else o={x:v,y:l};n={x:v,y:u}}else{if(o){if(o.y<l)return}else o={x:v,y:u};n={x:v,y:l}}}else if(i=m-(r=(h-p)/(g-f))*v,r<-1||r>1)if(h>p){if(o){if(o.y>=u)return}else o={x:(l-i)/r,y:l};n={x:(u-i)/r,y:u}}else{if(o){if(o.y<l)return}else o={x:(u-i)/r,y:u};n={x:(l-i)/r,y:l}}else if(f<g){if(o){if(o.x>=s)return}else o={x:a,y:r*a+i};n={x:s,y:r*s+i}}else{if(o){if(o.x<a)return}else o={x:s,y:r*s+i};n={x:a,y:r*a+i}}return e.a=o,e.b=n,!0}function Pi(e,t){this.l=e,this.r=t,this.a=this.b=null}function Ni(e,t,n,r){var i=new Pi(e,t);return pi.push(i),n&&Ri(i,e,t,n),r&&Ri(i,t,e,r),gi[e.i].edges.push(new zi(i,e,t)),gi[t.i].edges.push(new zi(i,t,e)),i}function ji(e,t,n){var r=new Pi(e,null);return r.a=t,r.b=n,pi.push(r),r}function Ri(e,t,n,r){e.a||e.b?e.l===n?e.b=r:e.a=r:(e.a=r,e.l=t,e.r=n)}function zi(e,t,n){var r=e.a,i=e.b;this.edge=e,this.site=t,this.angle=n?Math.atan2(n.y-t.y,n.x-t.x):e.l===t?Math.atan2(i.x-r.x,r.y-i.y):Math.atan2(r.x-i.x,i.y-r.y)}function Fi(){this._=null}function Bi(e){e.U=e.C=e.L=e.R=e.P=e.N=null}function Hi(e,t){var n=t,r=t.R,i=n.U;i?i.L===n?i.L=r:i.R=r:e._=r,r.U=i,n.U=r,n.R=r.L,n.R&&(n.R.U=n),r.L=n}function Ui(e,t){var n=t,r=t.L,i=n.U;i?i.L===n?i.L=r:i.R=r:e._=r,r.U=i,n.U=r,n.L=r.R,n.L&&(n.L.U=n),r.R=n}function Vi(e){for(;e.L;)e=e.L;return e}function Yi(e,t){var n,r,i,o=e.sort(Wi).pop();for(pi=[],gi=new Array(e.length),vi=new Fi,yi=new Fi;;)if(i=mi,o&&(!i||o.y<i.y||o.y===i.y&&o.x<i.x))o.x===n&&o.y===r||(gi[o.i]=new Ti(o),wi(o),n=o.x,r=o.y),o=e.pop();else{if(!i)break;xi(i.arc)}t&&(function(e){for(var t,n=pi,r=sr(e[0][0],e[0][1],e[1][0],e[1][1]),i=n.length;i--;)(!Di(t=n[i],e)||!r(t)||_(t.a.x-t.b.x)<Te&&_(t.a.y-t.b.y)<Te)&&(t.a=t.b=null,n.splice(i,1))}(t),function(e){for(var t,n,r,i,o,a,s,l,u,c,d=e[0][0],h=e[1][0],f=e[0][1],p=e[1][1],g=gi,v=g.length;v--;)if((o=g[v])&&o.prepare())for(l=(s=o.edges).length,a=0;a<l;)r=(c=s[a].end()).x,i=c.y,t=(u=s[++a%l].start()).x,n=u.y,(_(r-t)>Te||_(i-n)>Te)&&(s.splice(a,0,new zi(ji(o.site,c,_(r-d)<Te&&p-i>Te?{x:d,y:_(t-d)<Te?n:p}:_(i-p)<Te&&h-r>Te?{x:_(n-p)<Te?t:h,y:p}:_(r-h)<Te&&i-f>Te?{x:h,y:_(t-h)<Te?n:f}:_(i-f)<Te&&r-d>Te?{x:_(n-f)<Te?t:d,y:f}:null),o.site,null)),++l)}(t));var a={cells:gi,edges:pi};return vi=yi=pi=gi=null,a}function Wi(e,t){return t.y-e.y||t.x-e.x}Ti.prototype.prepare=function(){for(var e,t=this.edges,n=t.length;n--;)(e=t[n].edge).b&&e.a||t.splice(n,1);return t.sort(Ei),t.length},zi.prototype={start:function(){return this.edge.l===this.site?this.edge.a:this.edge.b},end:function(){return this.edge.l===this.site?this.edge.b:this.edge.a}},Fi.prototype={insert:function(e,t){var n,r,i;if(e){if(t.P=e,t.N=e.N,e.N&&(e.N.P=t),e.N=t,e.R){for(e=e.R;e.L;)e=e.L;e.L=t}else e.R=t;n=e}else this._?(e=Vi(this._),t.P=null,t.N=e,e.P=e.L=t,n=e):(t.P=t.N=null,this._=t,n=null);for(t.L=t.R=null,t.U=n,t.C=!0,e=t;n&&n.C;)n===(r=n.U).L?(i=r.R)&&i.C?(n.C=i.C=!1,r.C=!0,e=r):(e===n.R&&(Hi(this,n),n=(e=n).U),n.C=!1,r.C=!0,Ui(this,r)):(i=r.L)&&i.C?(n.C=i.C=!1,r.C=!0,e=r):(e===n.L&&(Ui(this,n),n=(e=n).U),n.C=!1,r.C=!0,Hi(this,r)),n=e.U;this._.C=!1},remove:function(e){e.N&&(e.N.P=e.P),e.P&&(e.P.N=e.N),e.N=e.P=null;var t,n,r,i=e.U,o=e.L,a=e.R;if(n=o?a?Vi(a):o:a,i?i.L===e?i.L=n:i.R=n:this._=n,o&&a?(r=n.C,n.C=e.C,n.L=o,o.U=n,n!==a?(i=n.U,n.U=e.U,e=n.R,i.L=e,n.R=a,a.U=n):(n.U=i,i=n,e=n.R)):(r=e.C,e=n),e&&(e.U=i),!r)if(e&&e.C)e.C=!1;else{do{if(e===this._)break;if(e===i.L){if((t=i.R).C&&(t.C=!1,i.C=!0,Hi(this,i),t=i.R),t.L&&t.L.C||t.R&&t.R.C){t.R&&t.R.C||(t.L.C=!1,t.C=!0,Ui(this,t),t=i.R),t.C=i.C,i.C=t.R.C=!1,Hi(this,i),e=this._;break}}else if((t=i.L).C&&(t.C=!1,i.C=!0,Ui(this,i),t=i.L),t.L&&t.L.C||t.R&&t.R.C){t.L&&t.L.C||(t.R.C=!1,t.C=!0,Hi(this,t),t=i.L),t.C=i.C,i.C=t.L.C=!1,Ui(this,i),e=this._;break}t.C=!0,e=i,i=i.U}while(!e.C);e&&(e.C=!1)}}},o.geom.voronoi=function(e){var t=ai,n=si,r=t,i=n,o=Gi;if(e)return a(e);function a(e){var t=new Array(e.length),n=o[0][0],r=o[0][1],i=o[1][0],a=o[1][1];return Yi(s(e),o).cells.forEach((function(o,s){var l=o.edges,u=o.site;(t[s]=l.length?l.map((function(e){var t=e.start();return[t.x,t.y]})):u.x>=n&&u.x<=i&&u.y>=r&&u.y<=a?[[n,a],[i,a],[i,r],[n,r]]:[]).point=e[s]})),t}function s(e){return e.map((function(e,t){return{x:Math.round(r(e,t)/Te)*Te,y:Math.round(i(e,t)/Te)*Te,i:t}}))}return a.links=function(e){return Yi(s(e)).edges.filter((function(e){return e.l&&e.r})).map((function(t){return{source:e[t.l.i],target:e[t.r.i]}}))},a.triangles=function(e){var t=[];return Yi(s(e)).cells.forEach((function(n,r){for(var i,o,a,s,l=n.site,u=n.edges.sort(Ei),c=-1,d=u.length,h=u[d-1].edge,f=h.l===l?h.r:h.l;++c<d;)h,i=f,f=(h=u[c].edge).l===l?h.r:h.l,r<i.i&&r<f.i&&(a=i,s=f,((o=l).x-s.x)*(a.y-o.y)-(o.x-a.x)*(s.y-o.y)<0)&&t.push([e[r],e[i.i],e[f.i]])})),t},a.x=function(e){return arguments.length?(r=yt(t=e),a):t},a.y=function(e){return arguments.length?(i=yt(n=e),a):n},a.clipExtent=function(e){return arguments.length?(o=null==e?Gi:e,a):o===Gi?null:o},a.size=function(e){return arguments.length?a.clipExtent(e&&[[0,0],e]):o===Gi?null:o&&o[1]},a};var Gi=[[-1e6,-1e6],[1e6,1e6]];function Xi(e){return e.x}function qi(e){return e.y}function Zi(e,t,n,r,i,o){if(!e(t,n,r,i,o)){var a=.5*(n+i),s=.5*(r+o),l=t.nodes;l[0]&&Zi(e,l[0],n,r,a,s),l[1]&&Zi(e,l[1],a,r,i,s),l[2]&&Zi(e,l[2],n,s,a,o),l[3]&&Zi(e,l[3],a,s,i,o)}}function Qi(e,t,n,r,i,o,a){var s,l=1/0;return function e(u,c,d,h,f){if(!(c>o||d>a||h<r||f<i)){if(p=u.point){var p,g=t-u.x,v=n-u.y,m=g*g+v*v;if(m<l){var y=Math.sqrt(l=m);r=t-y,i=n-y,o=t+y,a=n+y,s=p}}for(var b=u.nodes,C=.5*(c+h),A=.5*(d+f),_=(n>=A)<<1|t>=C,I=_+4;_<I;++_)if(u=b[3&_])switch(3&_){case 0:e(u,c,d,C,A);break;case 1:e(u,C,d,h,A);break;case 2:e(u,c,A,C,f);break;case 3:e(u,C,A,h,f)}}}(e,r,i,o,a),s}function Ki(e,t){e=o.rgb(e),t=o.rgb(t);var n=e.r,r=e.g,i=e.b,a=t.r-n,s=t.g-r,l=t.b-i;return function(e){return"#"+dt(Math.round(n+a*e))+dt(Math.round(r+s*e))+dt(Math.round(i+l*e))}}function Ji(e,t){var n,r={},i={};for(n in e)n in t?r[n]=ro(e[n],t[n]):i[n]=e[n];for(n in t)n in e||(i[n]=t[n]);return function(e){for(n in r)i[n]=r[n](e);return i}}function $i(e,t){return e=+e,t=+t,function(n){return e*(1-n)+t*n}}function eo(e,t){var n,r,i,o=to.lastIndex=no.lastIndex=0,a=-1,s=[],l=[];for(e+="",t+="";(n=to.exec(e))&&(r=no.exec(t));)(i=r.index)>o&&(i=t.slice(o,i),s[a]?s[a]+=i:s[++a]=i),(n=n[0])===(r=r[0])?s[a]?s[a]+=r:s[++a]=r:(s[++a]=null,l.push({i:a,x:$i(n,r)})),o=no.lastIndex;return o<t.length&&(i=t.slice(o),s[a]?s[a]+=i:s[++a]=i),s.length<2?l[0]?(t=l[0].x,function(e){return t(e)+""}):function(){return t}:(t=l.length,function(e){for(var n,r=0;r<t;++r)s[(n=l[r]).i]=n.x(e);return s.join("")})}o.geom.delaunay=function(e){return o.geom.voronoi().triangles(e)},o.geom.quadtree=function(e,t,n,r,i){var o,a=ai,s=si;if(o=arguments.length)return a=Xi,s=qi,3===o&&(i=n,r=t,n=t=0),l(e);function l(e){var l,u,c,d,h,f,p,g,v,m=yt(a),y=yt(s);if(null!=t)f=t,p=n,g=r,v=i;else if(g=v=-(f=p=1/0),u=[],c=[],h=e.length,o)for(d=0;d<h;++d)(l=e[d]).x<f&&(f=l.x),l.y<p&&(p=l.y),l.x>g&&(g=l.x),l.y>v&&(v=l.y),u.push(l.x),c.push(l.y);else for(d=0;d<h;++d){var b=+m(l=e[d],d),C=+y(l,d);b<f&&(f=b),C<p&&(p=C),b>g&&(g=b),C>v&&(v=C),u.push(b),c.push(C)}var A=g-f,I=v-p;function x(e,t,n,r,i,o,a,s){if(!isNaN(n)&&!isNaN(r))if(e.leaf){var l=e.x,u=e.y;if(null!=l)if(_(l-n)+_(u-r)<.01)w(e,t,n,r,i,o,a,s);else{var c=e.point;e.x=e.y=e.point=null,w(e,c,l,u,i,o,a,s),w(e,t,n,r,i,o,a,s)}else e.x=n,e.y=r,e.point=t}else w(e,t,n,r,i,o,a,s)}function w(e,t,n,r,i,o,a,s){var l=.5*(i+a),u=.5*(o+s),c=n>=l,d=r>=u,h=d<<1|c;e.leaf=!1,c?i=l:a=l,d?o=u:s=u,x(e=e.nodes[h]||(e.nodes[h]={leaf:!0,nodes:[],point:null,x:null,y:null}),t,n,r,i,o,a,s)}A>I?v=p+A:g=f+I;var M={leaf:!0,nodes:[],point:null,x:null,y:null,add:function(e){x(M,e,+m(e,++d),+y(e,d),f,p,g,v)},visit:function(e){Zi(e,M,f,p,g,v)},find:function(e){return Qi(M,e[0],e[1],f,p,g,v)}};if(d=-1,null==t){for(;++d<h;)x(M,e[d],u[d],c[d],f,p,g,v);--d}else e.forEach(M.add);return u=c=e=l=null,M}return l.x=function(e){return arguments.length?(a=e,l):a},l.y=function(e){return arguments.length?(s=e,l):s},l.extent=function(e){return arguments.length?(null==e?t=n=r=i=null:(t=+e[0][0],n=+e[0][1],r=+e[1][0],i=+e[1][1]),l):null==t?null:[[t,n],[r,i]]},l.size=function(e){return arguments.length?(null==e?t=n=r=i=null:(t=n=0,r=+e[0],i=+e[1]),l):null==t?null:[r-t,i-n]},l},o.interpolateRgb=Ki,o.interpolateObject=Ji,o.interpolateNumber=$i,o.interpolateString=eo;var to=/[-+]?(?:\d+\.?\d*|\.?\d+)(?:[eE][-+]?\d+)?/g,no=new RegExp(to.source,"g");function ro(e,t){for(var n,r=o.interpolators.length;--r>=0&&!(n=o.interpolators[r](e,t)););return n}function io(e,t){var n,r=[],i=[],o=e.length,a=t.length,s=Math.min(e.length,t.length);for(n=0;n<s;++n)r.push(ro(e[n],t[n]));for(;n<o;++n)i[n]=e[n];for(;n<a;++n)i[n]=t[n];return function(e){for(n=0;n<s;++n)i[n]=r[n](e);return i}}o.interpolate=ro,o.interpolators=[function(e,t){var n=typeof t;return("string"===n?mt.has(t.toLowerCase())||/^(#|rgb\(|hsl\()/i.test(t)?Ki:eo:t instanceof Ge?Ki:Array.isArray(t)?io:"object"===n&&isNaN(t)?Ji:$i)(e,t)}],o.interpolateArray=io;var oo=function(){return P},ao=o.map({linear:oo,poly:function(e){return function(t){return Math.pow(t,e)}},quad:function(){return ho},cubic:function(){return fo},sin:function(){return go},exp:function(){return vo},circle:function(){return mo},elastic:function(e,t){var n;arguments.length<2&&(t=.45);arguments.length?n=t/ke*Math.asin(1/e):(e=1,n=t/4);return function(r){return 1+e*Math.pow(2,-10*r)*Math.sin((r-n)*ke/t)}},back:function(e){e||(e=1.70158);return function(t){return t*t*((e+1)*t-e)}},bounce:function(){return yo}}),so=o.map({in:P,out:uo,"in-out":co,"out-in":function(e){return co(uo(e))}});function lo(e){return function(t){return t<=0?0:t>=1?1:e(t)}}function uo(e){return function(t){return 1-e(1-t)}}function co(e){return function(t){return.5*(t<.5?e(2*t):2-e(2-2*t))}}function ho(e){return e*e}function fo(e){return e*e*e}function po(e){if(e<=0)return 0;if(e>=1)return 1;var t=e*e,n=t*e;return 4*(e<.5?n:3*(e-t)+n-.75)}function go(e){return 1-Math.cos(e*De)}function vo(e){return Math.pow(2,10*(e-1))}function mo(e){return 1-Math.sqrt(1-e*e)}function yo(e){return e<1/2.75?7.5625*e*e:e<2/2.75?7.5625*(e-=1.5/2.75)*e+.75:e<2.5/2.75?7.5625*(e-=2.25/2.75)*e+.9375:7.5625*(e-=2.625/2.75)*e+.984375}function bo(e,t){return t-=e,function(n){return Math.round(e+t*n)}}function Co(e){var t,n,r,i=[e.a,e.b],o=[e.c,e.d],a=_o(i),s=Ao(i,o),l=_o(((t=o)[0]+=(r=-s)*(n=i)[0],t[1]+=r*n[1],t))||0;i[0]*o[1]<o[0]*i[1]&&(i[0]*=-1,i[1]*=-1,a*=-1,s*=-1),this.rotate=(a?Math.atan2(i[1],i[0]):Math.atan2(-o[0],o[1]))*Ne,this.translate=[e.e,e.f],this.scale=[a,l],this.skew=l?Math.atan2(s,l)*Ne:0}function Ao(e,t){return e[0]*t[0]+e[1]*t[1]}function _o(e){var t=Math.sqrt(Ao(e,e));return t&&(e[0]/=t,e[1]/=t),t}o.ease=function(e){var t=e.indexOf("-"),n=t>=0?e.slice(0,t):e,r=t>=0?e.slice(t+1):"in";return n=ao.get(n)||oo,lo((r=so.get(r)||P)(n.apply(null,a.call(arguments,1))))},o.interpolateHcl=function(e,t){e=o.hcl(e),t=o.hcl(t);var n=e.h,r=e.c,i=e.l,a=t.h-n,s=t.c-r,l=t.l-i;isNaN(s)&&(s=0,r=isNaN(r)?t.c:r);isNaN(a)?(a=0,n=isNaN(n)?t.h:n):a>180?a-=360:a<-180&&(a+=360);return function(e){return Je(n+a*e,r+s*e,i+l*e)+""}},o.interpolateHsl=function(e,t){e=o.hsl(e),t=o.hsl(t);var n=e.h,r=e.s,i=e.l,a=t.h-n,s=t.s-r,l=t.l-i;isNaN(s)&&(s=0,r=isNaN(r)?t.s:r);isNaN(a)?(a=0,n=isNaN(n)?t.h:n):a>180?a-=360:a<-180&&(a+=360);return function(e){return Ze(n+a*e,r+s*e,i+l*e)+""}},o.interpolateLab=function(e,t){e=o.lab(e),t=o.lab(t);var n=e.l,r=e.a,i=e.b,a=t.l-n,s=t.a-r,l=t.b-i;return function(e){return nt(n+a*e,r+s*e,i+l*e)+""}},o.interpolateRound=bo,o.transform=function(e){var t=l.createElementNS(o.ns.prefix.svg,"g");return(o.transform=function(e){if(null!=e){t.setAttribute("transform",e);var n=t.transform.baseVal.consolidate()}return new Co(n?n.matrix:Io)})(e)},Co.prototype.toString=function(){return"translate("+this.translate+")rotate("+this.rotate+")skewX("+this.skew+")scale("+this.scale+")"};var Io={a:1,b:0,c:0,d:1,e:0,f:0};function xo(e){return e.length?e.pop()+",":""}function wo(e,t){var n=[],r=[];return e=o.transform(e),t=o.transform(t),function(e,t,n,r){if(e[0]!==t[0]||e[1]!==t[1]){var i=n.push("translate(",null,",",null,")");r.push({i:i-4,x:$i(e[0],t[0])},{i:i-2,x:$i(e[1],t[1])})}else(t[0]||t[1])&&n.push("translate("+t+")")}(e.translate,t.translate,n,r),function(e,t,n,r){e!==t?(e-t>180?t+=360:t-e>180&&(e+=360),r.push({i:n.push(xo(n)+"rotate(",null,")")-2,x:$i(e,t)})):t&&n.push(xo(n)+"rotate("+t+")")}(e.rotate,t.rotate,n,r),function(e,t,n,r){e!==t?r.push({i:n.push(xo(n)+"skewX(",null,")")-2,x:$i(e,t)}):t&&n.push(xo(n)+"skewX("+t+")")}(e.skew,t.skew,n,r),function(e,t,n,r){if(e[0]!==t[0]||e[1]!==t[1]){var i=n.push(xo(n)+"scale(",null,",",null,")");r.push({i:i-4,x:$i(e[0],t[0])},{i:i-2,x:$i(e[1],t[1])})}else 1===t[0]&&1===t[1]||n.push(xo(n)+"scale("+t+")")}(e.scale,t.scale,n,r),e=t=null,function(e){for(var t,i=-1,o=r.length;++i<o;)n[(t=r[i]).i]=t.x(e);return n.join("")}}function Mo(e,t){return t=(t-=e=+e)||1/t,function(n){return(n-e)/t}}function So(e,t){return t=(t-=e=+e)||1/t,function(n){return Math.max(0,Math.min(1,(n-e)/t))}}function To(e){for(var t=e.source,n=e.target,r=function(e,t){if(e===t)return e;var n=Eo(e),r=Eo(t),i=n.pop(),o=r.pop(),a=null;for(;i===o;)a=i,i=n.pop(),o=r.pop();return a}(t,n),i=[t];t!==r;)t=t.parent,i.push(t);for(var o=i.length;n!==r;)i.splice(o,0,n),n=n.parent;return i}function Eo(e){for(var t=[],n=e.parent;null!=n;)t.push(e),e=n,n=n.parent;return t.push(e),t}function Oo(e){e.fixed|=2}function ko(e){e.fixed&=-7}function Lo(e){e.fixed|=4,e.px=e.x,e.py=e.y}function Do(e){e.fixed&=-5}o.interpolateTransform=wo,o.layout={},o.layout.bundle=function(){return function(e){for(var t=[],n=-1,r=e.length;++n<r;)t.push(To(e[n]));return t}},o.layout.chord=function(){var e,t,n,r,i,a,s,l={},u=0;function c(){var l,c,h,f,p,g={},v=[],m=o.range(r),y=[];for(e=[],t=[],l=0,f=-1;++f<r;){for(c=0,p=-1;++p<r;)c+=n[f][p];v.push(c),y.push(o.range(r)),l+=c}for(i&&m.sort((function(e,t){return i(v[e],v[t])})),a&&y.forEach((function(e,t){e.sort((function(e,r){return a(n[t][e],n[t][r])}))})),l=(ke-u*r)/l,c=0,f=-1;++f<r;){for(h=c,p=-1;++p<r;){var b=m[f],C=y[b][p],A=n[b][C],_=c,I=c+=A*l;g[b+"-"+C]={index:b,subindex:C,startAngle:_,endAngle:I,value:A}}t[b]={index:b,startAngle:h,endAngle:c,value:v[b]},c+=u}for(f=-1;++f<r;)for(p=f-1;++p<r;){var x=g[f+"-"+p],w=g[p+"-"+f];(x.value||w.value)&&e.push(x.value<w.value?{source:w,target:x}:{source:x,target:w})}s&&d()}function d(){e.sort((function(e,t){return s((e.source.value+e.target.value)/2,(t.source.value+t.target.value)/2)}))}return l.matrix=function(i){return arguments.length?(r=(n=i)&&n.length,e=t=null,l):n},l.padding=function(n){return arguments.length?(u=n,e=t=null,l):u},l.sortGroups=function(n){return arguments.length?(i=n,e=t=null,l):i},l.sortSubgroups=function(t){return arguments.length?(a=t,e=null,l):a},l.sortChords=function(t){return arguments.length?(s=t,e&&d(),l):s},l.chords=function(){return e||c(),e},l.groups=function(){return t||c(),t},l},o.layout.force=function(){var e,t,n,r,i,a,s={},l=o.dispatch("start","tick","end"),u=[1,1],c=.9,d=Po,h=No,f=-30,p=jo,g=.1,v=.64,m=[],y=[];function b(e){return function(t,n,r,i){if(t.point!==e){var o=t.cx-e.x,a=t.cy-e.y,s=i-n,l=o*o+a*a;if(s*s/v<l){if(l<p){var u=t.charge/l;e.px-=o*u,e.py-=a*u}return!0}if(t.point&&l&&l<p){u=t.pointCharge/l;e.px-=o*u,e.py-=a*u}}return!t.charge}}function C(e){e.px=o.event.x,e.py=o.event.y,s.resume()}return s.tick=function(){if((n*=.99)<.005)return e=null,l.end({type:"end",alpha:n=0}),!0;var t,s,d,h,p,v,C,A,_,I=m.length,x=y.length;for(s=0;s<x;++s)h=(d=y[s]).source,(v=(A=(p=d.target).x-h.x)*A+(_=p.y-h.y)*_)&&(A*=v=n*i[s]*((v=Math.sqrt(v))-r[s])/v,_*=v,p.x-=A*(C=h.weight+p.weight?h.weight/(h.weight+p.weight):.5),p.y-=_*C,h.x+=A*(C=1-C),h.y+=_*C);if((C=n*g)&&(A=u[0]/2,_=u[1]/2,s=-1,C))for(;++s<I;)(d=m[s]).x+=(A-d.x)*C,d.y+=(_-d.y)*C;if(f)for(!function e(t,n,r){var i=0,o=0;if(t.charge=0,!t.leaf)for(var a,s=t.nodes,l=s.length,u=-1;++u<l;)null!=(a=s[u])&&(e(a,n,r),t.charge+=a.charge,i+=a.charge*a.cx,o+=a.charge*a.cy);if(t.point){t.leaf||(t.point.x+=Math.random()-.5,t.point.y+=Math.random()-.5);var c=n*r[t.point.index];t.charge+=t.pointCharge=c,i+=c*t.point.x,o+=c*t.point.y}t.cx=i/t.charge,t.cy=o/t.charge}(t=o.geom.quadtree(m),n,a),s=-1;++s<I;)(d=m[s]).fixed||t.visit(b(d));for(s=-1;++s<I;)(d=m[s]).fixed?(d.x=d.px,d.y=d.py):(d.x-=(d.px-(d.px=d.x))*c,d.y-=(d.py-(d.py=d.y))*c);l.tick({type:"tick",alpha:n})},s.nodes=function(e){return arguments.length?(m=e,s):m},s.links=function(e){return arguments.length?(y=e,s):y},s.size=function(e){return arguments.length?(u=e,s):u},s.linkDistance=function(e){return arguments.length?(d="function"==typeof e?e:+e,s):d},s.distance=s.linkDistance,s.linkStrength=function(e){return arguments.length?(h="function"==typeof e?e:+e,s):h},s.friction=function(e){return arguments.length?(c=+e,s):c},s.charge=function(e){return arguments.length?(f="function"==typeof e?e:+e,s):f},s.chargeDistance=function(e){return arguments.length?(p=e*e,s):Math.sqrt(p)},s.gravity=function(e){return arguments.length?(g=+e,s):g},s.theta=function(e){return arguments.length?(v=e*e,s):Math.sqrt(v)},s.alpha=function(t){return arguments.length?(t=+t,n?t>0?n=t:(e.c=null,e.t=NaN,e=null,l.end({type:"end",alpha:n=0})):t>0&&(l.start({type:"start",alpha:n=t}),e=Mt(s.tick)),s):n},s.start=function(){var e,t,n,o=m.length,l=y.length,c=u[0],p=u[1];for(e=0;e<o;++e)(n=m[e]).index=e,n.weight=0;for(e=0;e<l;++e)"number"==typeof(n=y[e]).source&&(n.source=m[n.source]),"number"==typeof n.target&&(n.target=m[n.target]),++n.source.weight,++n.target.weight;for(e=0;e<o;++e)n=m[e],isNaN(n.x)&&(n.x=g("x",c)),isNaN(n.y)&&(n.y=g("y",p)),isNaN(n.px)&&(n.px=n.x),isNaN(n.py)&&(n.py=n.y);if(r=[],"function"==typeof d)for(e=0;e<l;++e)r[e]=+d.call(this,y[e],e);else for(e=0;e<l;++e)r[e]=d;if(i=[],"function"==typeof h)for(e=0;e<l;++e)i[e]=+h.call(this,y[e],e);else for(e=0;e<l;++e)i[e]=h;if(a=[],"function"==typeof f)for(e=0;e<o;++e)a[e]=+f.call(this,m[e],e);else for(e=0;e<o;++e)a[e]=f;function g(n,r){if(!t){for(t=new Array(o),u=0;u<o;++u)t[u]=[];for(u=0;u<l;++u){var i=y[u];t[i.source.index].push(i.target),t[i.target.index].push(i.source)}}for(var a,s=t[e],u=-1,c=s.length;++u<c;)if(!isNaN(a=s[u][n]))return a;return Math.random()*r}return s.resume()},s.resume=function(){return s.alpha(.1)},s.stop=function(){return s.alpha(0)},s.drag=function(){if(t||(t=o.behavior.drag().origin(P).on("dragstart.force",Oo).on("drag.force",C).on("dragend.force",ko)),!arguments.length)return t;this.on("mouseover.force",Lo).on("mouseout.force",Do).call(t)},o.rebind(s,l,"on")};var Po=20,No=1,jo=1/0;function Ro(e,t){return o.rebind(e,t,"sort","children","value"),e.nodes=e,e.links=Vo,e}function zo(e,t){for(var n=[e];null!=(e=n.pop());)if(t(e),(i=e.children)&&(r=i.length))for(var r,i;--r>=0;)n.push(i[r])}function Fo(e,t){for(var n=[e],r=[];null!=(e=n.pop());)if(r.push(e),(o=e.children)&&(i=o.length))for(var i,o,a=-1;++a<i;)n.push(o[a]);for(;null!=(e=r.pop());)t(e)}function Bo(e){return e.children}function Ho(e){return e.value}function Uo(e,t){return t.value-e.value}function Vo(e){return o.merge(e.map((function(e){return(e.children||[]).map((function(t){return{source:e,target:t}}))})))}o.layout.hierarchy=function(){var e=Uo,t=Bo,n=Ho;function r(i){var o,a=[i],s=[];for(i.depth=0;null!=(o=a.pop());)if(s.push(o),(u=t.call(r,o,o.depth))&&(l=u.length)){for(var l,u,c;--l>=0;)a.push(c=u[l]),c.parent=o,c.depth=o.depth+1;n&&(o.value=0),o.children=u}else n&&(o.value=+n.call(r,o,o.depth)||0),delete o.children;return Fo(i,(function(t){var r,i;e&&(r=t.children)&&r.sort(e),n&&(i=t.parent)&&(i.value+=t.value)})),s}return r.sort=function(t){return arguments.length?(e=t,r):e},r.children=function(e){return arguments.length?(t=e,r):t},r.value=function(e){return arguments.length?(n=e,r):n},r.revalue=function(e){return n&&(zo(e,(function(e){e.children&&(e.value=0)})),Fo(e,(function(e){var t;e.children||(e.value=+n.call(r,e,e.depth)||0),(t=e.parent)&&(t.value+=e.value)}))),e},r},o.layout.partition=function(){var e=o.layout.hierarchy(),t=[1,1];function n(n,r){var i=e.call(this,n,r);return function e(t,n,r,i){var o=t.children;if(t.x=n,t.y=t.depth*i,t.dx=r,t.dy=i,o&&(a=o.length)){var a,s,l,u=-1;for(r=t.value?r/t.value:0;++u<a;)e(s=o[u],n,l=s.value*r,i),n+=l}}(i[0],0,t[0],t[1]/function e(t){var n=t.children,r=0;if(n&&(i=n.length))for(var i,o=-1;++o<i;)r=Math.max(r,e(n[o]));return 1+r}(i[0])),i}return n.size=function(e){return arguments.length?(t=e,n):t},Ro(n,e)},o.layout.pie=function(){var e=Number,t=Yo,n=0,r=ke,i=0;function a(s){var l,u=s.length,c=s.map((function(t,n){return+e.call(a,t,n)})),d=+("function"==typeof n?n.apply(this,arguments):n),h=("function"==typeof r?r.apply(this,arguments):r)-d,f=Math.min(Math.abs(h)/u,+("function"==typeof i?i.apply(this,arguments):i)),p=f*(h<0?-1:1),g=o.sum(c),v=g?(h-u*p)/g:0,m=o.range(u),y=[];return null!=t&&m.sort(t===Yo?function(e,t){return c[t]-c[e]}:function(e,n){return t(s[e],s[n])}),m.forEach((function(e){y[e]={data:s[e],value:l=c[e],startAngle:d,endAngle:d+=l*v+p,padAngle:f}})),y}return a.value=function(t){return arguments.length?(e=t,a):e},a.sort=function(e){return arguments.length?(t=e,a):t},a.startAngle=function(e){return arguments.length?(n=e,a):n},a.endAngle=function(e){return arguments.length?(r=e,a):r},a.padAngle=function(e){return arguments.length?(i=e,a):i},a};var Yo={};function Wo(e){return e.x}function Go(e){return e.y}function Xo(e,t,n){e.y0=t,e.y=n}o.layout.stack=function(){var e=P,t=Qo,n=Ko,r=Xo,i=Wo,a=Go;function s(l,u){if(!(f=l.length))return l;var c=l.map((function(t,n){return e.call(s,t,n)})),d=c.map((function(e){return e.map((function(e,t){return[i.call(s,e,t),a.call(s,e,t)]}))})),h=t.call(s,d,u);c=o.permute(c,h),d=o.permute(d,h);var f,p,g,v,m=n.call(s,d,u),y=c[0].length;for(g=0;g<y;++g)for(r.call(s,c[0][g],v=m[g],d[0][g][1]),p=1;p<f;++p)r.call(s,c[p][g],v+=d[p-1][g][1],d[p][g][1]);return l}return s.values=function(t){return arguments.length?(e=t,s):e},s.order=function(e){return arguments.length?(t="function"==typeof e?e:qo.get(e)||Qo,s):t},s.offset=function(e){return arguments.length?(n="function"==typeof e?e:Zo.get(e)||Ko,s):n},s.x=function(e){return arguments.length?(i=e,s):i},s.y=function(e){return arguments.length?(a=e,s):a},s.out=function(e){return arguments.length?(r=e,s):r},s};var qo=o.map({"inside-out":function(e){var t,n,r=e.length,i=e.map(Jo),a=e.map($o),s=o.range(r).sort((function(e,t){return i[e]-i[t]})),l=0,u=0,c=[],d=[];for(t=0;t<r;++t)n=s[t],l<u?(l+=a[n],c.push(n)):(u+=a[n],d.push(n));return d.reverse().concat(c)},reverse:function(e){return o.range(e.length).reverse()},default:Qo}),Zo=o.map({silhouette:function(e){var t,n,r,i=e.length,o=e[0].length,a=[],s=0,l=[];for(n=0;n<o;++n){for(t=0,r=0;t<i;t++)r+=e[t][n][1];r>s&&(s=r),a.push(r)}for(n=0;n<o;++n)l[n]=(s-a[n])/2;return l},wiggle:function(e){var t,n,r,i,o,a,s,l,u,c=e.length,d=e[0],h=d.length,f=[];for(f[0]=l=u=0,n=1;n<h;++n){for(t=0,i=0;t<c;++t)i+=e[t][n][1];for(t=0,o=0,s=d[n][0]-d[n-1][0];t<c;++t){for(r=0,a=(e[t][n][1]-e[t][n-1][1])/(2*s);r<t;++r)a+=(e[r][n][1]-e[r][n-1][1])/s;o+=a*e[t][n][1]}f[n]=l-=i?o/i*s:0,l<u&&(u=l)}for(n=0;n<h;++n)f[n]-=u;return f},expand:function(e){var t,n,r,i=e.length,o=e[0].length,a=1/i,s=[];for(n=0;n<o;++n){for(t=0,r=0;t<i;t++)r+=e[t][n][1];if(r)for(t=0;t<i;t++)e[t][n][1]/=r;else for(t=0;t<i;t++)e[t][n][1]=a}for(n=0;n<o;++n)s[n]=0;return s},zero:Ko});function Qo(e){return o.range(e.length)}function Ko(e){for(var t=-1,n=e[0].length,r=[];++t<n;)r[t]=0;return r}function Jo(e){for(var t,n=1,r=0,i=e[0][1],o=e.length;n<o;++n)(t=e[n][1])>i&&(r=n,i=t);return r}function $o(e){return e.reduce(ea,0)}function ea(e,t){return e+t[1]}function ta(e,t){return na(e,Math.ceil(Math.log(t.length)/Math.LN2+1))}function na(e,t){for(var n=-1,r=+e[0],i=(e[1]-r)/t,o=[];++n<=t;)o[n]=i*n+r;return o}function ra(e){return[o.min(e),o.max(e)]}function ia(e,t){return e.value-t.value}function oa(e,t){var n=e._pack_next;e._pack_next=t,t._pack_prev=e,t._pack_next=n,n._pack_prev=t}function aa(e,t){e._pack_next=t,t._pack_prev=e}function sa(e,t){var n=t.x-e.x,r=t.y-e.y,i=e.r+t.r;return.999*i*i>n*n+r*r}function la(e){if((t=e.children)&&(l=t.length)){var t,n,r,i,o,a,s,l,u=1/0,c=-1/0,d=1/0,h=-1/0;if(t.forEach(ua),(n=t[0]).x=-n.r,n.y=0,b(n),l>1&&((r=t[1]).x=r.r,r.y=0,b(r),l>2))for(da(n,r,i=t[2]),b(i),oa(n,i),n._pack_prev=i,oa(i,r),r=n._pack_next,o=3;o<l;o++){da(n,r,i=t[o]);var f=0,p=1,g=1;for(a=r._pack_next;a!==r;a=a._pack_next,p++)if(sa(a,i)){f=1;break}if(1==f)for(s=n._pack_prev;s!==a._pack_prev&&!sa(s,i);s=s._pack_prev,g++);f?(p<g||p==g&&r.r<n.r?aa(n,r=a):aa(n=s,r),o--):(oa(n,i),r=i,b(i))}var v=(u+c)/2,m=(d+h)/2,y=0;for(o=0;o<l;o++)(i=t[o]).x-=v,i.y-=m,y=Math.max(y,i.r+Math.sqrt(i.x*i.x+i.y*i.y));e.r=y,t.forEach(ca)}function b(e){u=Math.min(e.x-e.r,u),c=Math.max(e.x+e.r,c),d=Math.min(e.y-e.r,d),h=Math.max(e.y+e.r,h)}}function ua(e){e._pack_next=e._pack_prev=e}function ca(e){delete e._pack_next,delete e._pack_prev}function da(e,t,n){var r=e.r+n.r,i=t.x-e.x,o=t.y-e.y;if(r&&(i||o)){var a=t.r+n.r,s=i*i+o*o,l=.5+((r*=r)-(a*=a))/(2*s),u=Math.sqrt(Math.max(0,2*a*(r+s)-(r-=s)*r-a*a))/(2*s);n.x=e.x+l*i+u*o,n.y=e.y+l*o-u*i}else n.x=e.x+r,n.y=e.y}function ha(e,t){return e.parent==t.parent?1:2}function fa(e){var t=e.children;return t.length?t[0]:e.t}function pa(e){var t,n=e.children;return(t=n.length)?n[t-1]:e.t}function ga(e,t,n){var r=n/(t.i-e.i);t.c-=r,t.s+=n,e.c+=r,t.z+=n,t.m+=n}function va(e,t,n){return e.a.parent===t.parent?e.a:n}function ma(e){return{x:e.x,y:e.y,dx:e.dx,dy:e.dy}}function ya(e,t){var n=e.x+t[3],r=e.y+t[0],i=e.dx-t[1]-t[3],o=e.dy-t[0]-t[2];return i<0&&(n+=i/2,i=0),o<0&&(r+=o/2,o=0),{x:n,y:r,dx:i,dy:o}}function ba(e){var t=e[0],n=e[e.length-1];return t<n?[t,n]:[n,t]}function Ca(e){return e.rangeExtent?e.rangeExtent():ba(e.range())}function Aa(e,t,n,r){var i=n(e[0],e[1]),o=r(t[0],t[1]);return function(e){return o(i(e))}}function _a(e,t){var n,r=0,i=e.length-1,o=e[r],a=e[i];return a<o&&(n=r,r=i,i=n,n=o,o=a,a=n),e[r]=t.floor(o),e[i]=t.ceil(a),e}function Ia(e){return e?{floor:function(t){return Math.floor(t/e)*e},ceil:function(t){return Math.ceil(t/e)*e}}:xa}o.layout.histogram=function(){var e=!0,t=Number,n=ra,r=ta;function i(i,a){for(var s,l,u=[],c=i.map(t,this),d=n.call(this,c,a),h=r.call(this,d,c,a),f=(a=-1,c.length),p=h.length-1,g=e?1:1/f;++a<p;)(s=u[a]=[]).dx=h[a+1]-(s.x=h[a]),s.y=0;if(p>0)for(a=-1;++a<f;)(l=c[a])>=d[0]&&l<=d[1]&&((s=u[o.bisect(h,l,1,p)-1]).y+=g,s.push(i[a]));return u}return i.value=function(e){return arguments.length?(t=e,i):t},i.range=function(e){return arguments.length?(n=yt(e),i):n},i.bins=function(e){return arguments.length?(r="number"==typeof e?function(t){return na(t,e)}:yt(e),i):r},i.frequency=function(t){return arguments.length?(e=!!t,i):e},i},o.layout.pack=function(){var e,t=o.layout.hierarchy().sort(ia),n=0,r=[1,1];function i(i,o){var a=t.call(this,i,o),s=a[0],l=r[0],u=r[1],c=null==e?Math.sqrt:"function"==typeof e?e:function(){return e};if(s.x=s.y=0,Fo(s,(function(e){e.r=+c(e.value)})),Fo(s,la),n){var d=n*(e?1:Math.max(2*s.r/l,2*s.r/u))/2;Fo(s,(function(e){e.r+=d})),Fo(s,la),Fo(s,(function(e){e.r-=d}))}return function e(t,n,r,i){var o=t.children;if(t.x=n+=i*t.x,t.y=r+=i*t.y,t.r*=i,o)for(var a=-1,s=o.length;++a<s;)e(o[a],n,r,i)}(s,l/2,u/2,e?1:1/Math.max(2*s.r/l,2*s.r/u)),a}return i.size=function(e){return arguments.length?(r=e,i):r},i.radius=function(t){return arguments.length?(e=null==t||"function"==typeof t?t:+t,i):e},i.padding=function(e){return arguments.length?(n=+e,i):n},Ro(i,t)},o.layout.tree=function(){var e=o.layout.hierarchy().sort(null).value(null),t=ha,n=[1,1],r=null;function i(i,o){var u=e.call(this,i,o),c=u[0],d=function(e){var t,n={A:null,children:[e]},r=[n];for(;null!=(t=r.pop());)for(var i,o=t.children,a=0,s=o.length;a<s;++a)r.push((o[a]=i={_:o[a],parent:t,children:(i=o[a].children)&&i.slice()||[],A:null,a:null,z:0,m:0,c:0,s:0,t:null,i:a}).a=i);return n.children[0]}(c);if(Fo(d,a),d.parent.m=-d.z,zo(d,s),r)zo(c,l);else{var h=c,f=c,p=c;zo(c,(function(e){e.x<h.x&&(h=e),e.x>f.x&&(f=e),e.depth>p.depth&&(p=e)}));var g=t(h,f)/2-h.x,v=n[0]/(f.x+t(f,h)/2+g),m=n[1]/(p.depth||1);zo(c,(function(e){e.x=(e.x+g)*v,e.y=e.depth*m}))}return u}function a(e){var n=e.children,r=e.parent.children,i=e.i?r[e.i-1]:null;if(n.length){!function(e){var t,n=0,r=0,i=e.children,o=i.length;for(;--o>=0;)(t=i[o]).z+=n,t.m+=n,n+=t.s+(r+=t.c)}(e);var o=(n[0].z+n[n.length-1].z)/2;i?(e.z=i.z+t(e._,i._),e.m=e.z-o):e.z=o}else i&&(e.z=i.z+t(e._,i._));e.parent.A=function(e,n,r){if(n){for(var i,o=e,a=e,s=n,l=o.parent.children[0],u=o.m,c=a.m,d=s.m,h=l.m;s=pa(s),o=fa(o),s&&o;)l=fa(l),(a=pa(a)).a=e,(i=s.z+d-o.z-u+t(s._,o._))>0&&(ga(va(s,e,r),e,i),u+=i,c+=i),d+=s.m,u+=o.m,h+=l.m,c+=a.m;s&&!pa(a)&&(a.t=s,a.m+=d-c),o&&!fa(l)&&(l.t=o,l.m+=u-h,r=e)}return r}(e,i,e.parent.A||r[0])}function s(e){e._.x=e.z+e.parent.m,e.m+=e.parent.m}function l(e){e.x*=n[0],e.y=e.depth*n[1]}return i.separation=function(e){return arguments.length?(t=e,i):t},i.size=function(e){return arguments.length?(r=null==(n=e)?l:null,i):r?null:n},i.nodeSize=function(e){return arguments.length?(r=null==(n=e)?null:l,i):r?n:null},Ro(i,e)},o.layout.cluster=function(){var e=o.layout.hierarchy().sort(null).value(null),t=ha,n=[1,1],r=!1;function i(i,a){var s,l=e.call(this,i,a),u=l[0],c=0;Fo(u,(function(e){var n=e.children;n&&n.length?(e.x=function(e){return e.reduce((function(e,t){return e+t.x}),0)/e.length}(n),e.y=function(e){return 1+o.max(e,(function(e){return e.y}))}(n)):(e.x=s?c+=t(e,s):0,e.y=0,s=e)}));var d=function e(t){var n=t.children;return n&&n.length?e(n[0]):t}(u),h=function e(t){var n,r=t.children;return r&&(n=r.length)?e(r[n-1]):t}(u),f=d.x-t(d,h)/2,p=h.x+t(h,d)/2;return Fo(u,r?function(e){e.x=(e.x-u.x)*n[0],e.y=(u.y-e.y)*n[1]}:function(e){e.x=(e.x-f)/(p-f)*n[0],e.y=(1-(u.y?e.y/u.y:1))*n[1]}),l}return i.separation=function(e){return arguments.length?(t=e,i):t},i.size=function(e){return arguments.length?(r=null==(n=e),i):r?null:n},i.nodeSize=function(e){return arguments.length?(r=null!=(n=e),i):r?n:null},Ro(i,e)},o.layout.treemap=function(){var e,t=o.layout.hierarchy(),n=Math.round,r=[1,1],i=null,a=ma,s=!1,l="squarify",u=.5*(1+Math.sqrt(5));function c(e,t){for(var n,r,i=-1,o=e.length;++i<o;)r=(n=e[i]).value*(t<0?0:t),n.area=isNaN(r)||r<=0?0:r}function d(e){var t=e.children;if(t&&t.length){var n,r,i,o=a(e),s=[],u=t.slice(),h=1/0,g="slice"===l?o.dx:"dice"===l?o.dy:"slice-dice"===l?1&e.depth?o.dy:o.dx:Math.min(o.dx,o.dy);for(c(u,o.dx*o.dy/e.value),s.area=0;(i=u.length)>0;)s.push(n=u[i-1]),s.area+=n.area,"squarify"!==l||(r=f(s,g))<=h?(u.pop(),h=r):(s.area-=s.pop().area,p(s,g,o,!1),g=Math.min(o.dx,o.dy),s.length=s.area=0,h=1/0);s.length&&(p(s,g,o,!0),s.length=s.area=0),t.forEach(d)}}function h(e){var t=e.children;if(t&&t.length){var n,r=a(e),i=t.slice(),o=[];for(c(i,r.dx*r.dy/e.value),o.area=0;n=i.pop();)o.push(n),o.area+=n.area,null!=n.z&&(p(o,n.z?r.dx:r.dy,r,!i.length),o.length=o.area=0);t.forEach(h)}}function f(e,t){for(var n,r=e.area,i=0,o=1/0,a=-1,s=e.length;++a<s;)(n=e[a].area)&&(n<o&&(o=n),n>i&&(i=n));return t*=t,(r*=r)?Math.max(t*i*u/r,r/(t*o*u)):1/0}function p(e,t,r,i){var o,a=-1,s=e.length,l=r.x,u=r.y,c=t?n(e.area/t):0;if(t==r.dx){for((i||c>r.dy)&&(c=r.dy);++a<s;)(o=e[a]).x=l,o.y=u,o.dy=c,l+=o.dx=Math.min(r.x+r.dx-l,c?n(o.area/c):0);o.z=!0,o.dx+=r.x+r.dx-l,r.y+=c,r.dy-=c}else{for((i||c>r.dx)&&(c=r.dx);++a<s;)(o=e[a]).x=l,o.y=u,o.dx=c,u+=o.dy=Math.min(r.y+r.dy-u,c?n(o.area/c):0);o.z=!1,o.dy+=r.y+r.dy-u,r.x+=c,r.dx-=c}}function g(n){var i=e||t(n),o=i[0];return o.x=o.y=0,o.value?(o.dx=r[0],o.dy=r[1]):o.dx=o.dy=0,e&&t.revalue(o),c([o],o.dx*o.dy/o.value),(e?h:d)(o),s&&(e=i),i}return g.size=function(e){return arguments.length?(r=e,g):r},g.padding=function(e){if(!arguments.length)return i;function t(t){var n=e.call(g,t,t.depth);return null==n?ma(t):ya(t,"number"==typeof n?[n,n,n,n]:n)}function n(t){return ya(t,e)}var r;return a=null==(i=e)?ma:"function"==(r=typeof e)?t:"number"===r?(e=[e,e,e,e],n):n,g},g.round=function(e){return arguments.length?(n=e?Math.round:Number,g):n!=Number},g.sticky=function(t){return arguments.length?(s=t,e=null,g):s},g.ratio=function(e){return arguments.length?(u=e,g):u},g.mode=function(e){return arguments.length?(l=e+"",g):l},Ro(g,t)},o.random={normal:function(e,t){var n=arguments.length;return n<2&&(t=1),n<1&&(e=0),function(){var n,r,i;do{i=(n=2*Math.random()-1)*n+(r=2*Math.random()-1)*r}while(!i||i>1);return e+t*n*Math.sqrt(-2*Math.log(i)/i)}},logNormal:function(){var e=o.random.normal.apply(o,arguments);return function(){return Math.exp(e())}},bates:function(e){var t=o.random.irwinHall(e);return function(){return t()/e}},irwinHall:function(e){return function(){for(var t=0,n=0;n<e;n++)t+=Math.random();return t}}},o.scale={};var xa={floor:P,ceil:P};function wa(e,t,n,r){var i=[],a=[],s=0,l=Math.min(e.length,t.length)-1;for(e[l]<e[0]&&(e=e.slice().reverse(),t=t.slice().reverse());++s<=l;)i.push(n(e[s-1],e[s])),a.push(r(t[s-1],t[s]));return function(t){var n=o.bisect(e,t,1,l)-1;return a[n](i[n](t))}}function Ma(e,t){return o.rebind(e,t,"range","rangeRound","interpolate","clamp")}function Sa(e,t){return _a(e,Ia(Ta(e,t)[2])),_a(e,Ia(Ta(e,t)[2])),e}function Ta(e,t){null==t&&(t=10);var n=ba(e),r=n[1]-n[0],i=Math.pow(10,Math.floor(Math.log(r/t)/Math.LN10)),o=t/r*i;return o<=.15?i*=10:o<=.35?i*=5:o<=.75&&(i*=2),n[0]=Math.ceil(n[0]/i)*i,n[1]=Math.floor(n[1]/i)*i+.5*i,n[2]=i,n}function Ea(e,t){return o.range.apply(o,Ta(e,t))}function Oa(e,t,n){var r=Ta(e,t);if(n){var i=Dt.exec(n);if(i.shift(),"s"===i[8]){var a=o.formatPrefix(Math.max(_(r[0]),_(r[1])));return i[7]||(i[7]="."+La(a.scale(r[2]))),i[8]="f",n=o.format(i.join("")),function(e){return n(a.scale(e))+a.symbol}}i[7]||(i[7]="."+function(e,t){var n=La(t[2]);return e in ka?Math.abs(n-La(Math.max(_(t[0]),_(t[1]))))+ +("e"!==e):n-2*("%"===e)}(i[8],r)),n=i.join("")}else n=",."+La(r[2])+"f";return o.format(n)}o.scale.linear=function(){return function e(t,n,r,i){var o,a;function s(){var e=Math.min(t.length,n.length)>2?wa:Aa,s=i?So:Mo;return o=e(t,n,s,r),a=e(n,t,s,ro),l}function l(e){return o(e)}return l.invert=function(e){return a(e)},l.domain=function(e){return arguments.length?(t=e.map(Number),s()):t},l.range=function(e){return arguments.length?(n=e,s()):n},l.rangeRound=function(e){return l.range(e).interpolate(bo)},l.clamp=function(e){return arguments.length?(i=e,s()):i},l.interpolate=function(e){return arguments.length?(r=e,s()):r},l.ticks=function(e){return Ea(t,e)},l.tickFormat=function(e,n){return Oa(t,e,n)},l.nice=function(e){return Sa(t,e),s()},l.copy=function(){return e(t,n,r,i)},s()}([0,1],[0,1],ro,!1)};var ka={s:1,g:1,p:1,r:1,e:1};function La(e){return-Math.floor(Math.log(e)/Math.LN10+.01)}o.scale.log=function(){return function e(t,n,r,i){function a(e){return(r?Math.log(e<0?0:e):-Math.log(e>0?0:-e))/Math.log(n)}function s(e){return r?Math.pow(n,e):-Math.pow(n,-e)}function l(e){return t(a(e))}return l.invert=function(e){return s(t.invert(e))},l.domain=function(e){return arguments.length?(r=e[0]>=0,t.domain((i=e.map(Number)).map(a)),l):i},l.base=function(e){return arguments.length?(n=+e,t.domain(i.map(a)),l):n},l.nice=function(){var e=_a(i.map(a),r?Math:Pa);return t.domain(e),i=e.map(s),l},l.ticks=function(){var e=ba(i),t=[],o=e[0],l=e[1],u=Math.floor(a(o)),c=Math.ceil(a(l)),d=n%1?2:n;if(isFinite(c-u)){if(r){for(;u<c;u++)for(var h=1;h<d;h++)t.push(s(u)*h);t.push(s(u))}else for(t.push(s(u));u++<c;)for(h=d-1;h>0;h--)t.push(s(u)*h);for(u=0;t[u]<o;u++);for(c=t.length;t[c-1]>l;c--);t=t.slice(u,c)}return t},l.tickFormat=function(e,t){if(!arguments.length)return Da;arguments.length<2?t=Da:"function"!=typeof t&&(t=o.format(t));var r=Math.max(1,n*e/l.ticks().length);return function(e){var i=e/s(Math.round(a(e)));return i*n<n-.5&&(i*=n),i<=r?t(e):""}},l.copy=function(){return e(t.copy(),n,r,i)},Ma(l,t)}(o.scale.linear().domain([0,1]),10,!0,[1,10])};var Da=o.format(".0e"),Pa={floor:function(e){return-Math.ceil(-e)},ceil:function(e){return-Math.floor(-e)}};function Na(e){return function(t){return t<0?-Math.pow(-t,e):Math.pow(t,e)}}o.scale.pow=function(){return function e(t,n,r){var i=Na(n),o=Na(1/n);function a(e){return t(i(e))}return a.invert=function(e){return o(t.invert(e))},a.domain=function(e){return arguments.length?(t.domain((r=e.map(Number)).map(i)),a):r},a.ticks=function(e){return Ea(r,e)},a.tickFormat=function(e,t){return Oa(r,e,t)},a.nice=function(e){return a.domain(Sa(r,e))},a.exponent=function(e){return arguments.length?(i=Na(n=e),o=Na(1/n),t.domain(r.map(i)),a):n},a.copy=function(){return e(t.copy(),n,r)},Ma(a,t)}(o.scale.linear(),1,[0,1])},o.scale.sqrt=function(){return o.scale.pow().exponent(.5)},o.scale.ordinal=function(){return function e(t,n){var r,i,a;function s(e){return i[((r.get(e)||("range"===n.t?r.set(e,t.push(e)):NaN))-1)%i.length]}function l(e,n){return o.range(t.length).map((function(t){return e+n*t}))}return s.domain=function(e){if(!arguments.length)return t;t=[],r=new w;for(var i,o=-1,a=e.length;++o<a;)r.has(i=e[o])||r.set(i,t.push(i));return s[n.t].apply(s,n.a)},s.range=function(e){return arguments.length?(i=e,a=0,n={t:"range",a:arguments},s):i},s.rangePoints=function(e,r){arguments.length<2&&(r=0);var o=e[0],u=e[1],c=t.length<2?(o=(o+u)/2,0):(u-o)/(t.length-1+r);return i=l(o+c*r/2,c),a=0,n={t:"rangePoints",a:arguments},s},s.rangeRoundPoints=function(e,r){arguments.length<2&&(r=0);var o=e[0],u=e[1],c=t.length<2?(o=u=Math.round((o+u)/2),0):(u-o)/(t.length-1+r)|0;return i=l(o+Math.round(c*r/2+(u-o-(t.length-1+r)*c)/2),c),a=0,n={t:"rangeRoundPoints",a:arguments},s},s.rangeBands=function(e,r,o){arguments.length<2&&(r=0),arguments.length<3&&(o=r);var u=e[1]<e[0],c=e[u-0],d=e[1-u],h=(d-c)/(t.length-r+2*o);return i=l(c+h*o,h),u&&i.reverse(),a=h*(1-r),n={t:"rangeBands",a:arguments},s},s.rangeRoundBands=function(e,r,o){arguments.length<2&&(r=0),arguments.length<3&&(o=r);var u=e[1]<e[0],c=e[u-0],d=e[1-u],h=Math.floor((d-c)/(t.length-r+2*o));return i=l(c+Math.round((d-c-(t.length-r)*h)/2),h),u&&i.reverse(),a=Math.round(h*(1-r)),n={t:"rangeRoundBands",a:arguments},s},s.rangeBand=function(){return a},s.rangeExtent=function(){return ba(n.a[0])},s.copy=function(){return e(t,n)},s.domain(t)}([],{t:"range",a:[[]]})},o.scale.category10=function(){return o.scale.ordinal().range(ja)},o.scale.category20=function(){return o.scale.ordinal().range(Ra)},o.scale.category20b=function(){return o.scale.ordinal().range(za)},o.scale.category20c=function(){return o.scale.ordinal().range(Fa)};var ja=[2062260,16744206,2924588,14034728,9725885,9197131,14907330,8355711,12369186,1556175].map(ut),Ra=[2062260,11454440,16744206,16759672,2924588,10018698,14034728,16750742,9725885,12955861,9197131,12885140,14907330,16234194,8355711,13092807,12369186,14408589,1556175,10410725].map(ut),za=[3750777,5395619,7040719,10264286,6519097,9216594,11915115,13556636,9202993,12426809,15186514,15190932,8666169,11356490,14049643,15177372,8077683,10834324,13528509,14589654].map(ut),Fa=[3244733,7057110,10406625,13032431,15095053,16616764,16625259,16634018,3253076,7652470,10607003,13101504,7695281,10394312,12369372,14342891,6513507,9868950,12434877,14277081].map(ut);function Ba(){return 0}o.scale.quantile=function(){return function e(t,n){var r;function i(){var e=0,i=n.length;for(r=[];++e<i;)r[e-1]=o.quantile(t,e/i);return a}function a(e){if(!isNaN(e=+e))return n[o.bisect(r,e)]}return a.domain=function(e){return arguments.length?(t=e.map(m).filter(y).sort(v),i()):t},a.range=function(e){return arguments.length?(n=e,i()):n},a.quantiles=function(){return r},a.invertExtent=function(e){return(e=n.indexOf(e))<0?[NaN,NaN]:[e>0?r[e-1]:t[0],e<r.length?r[e]:t[t.length-1]]},a.copy=function(){return e(t,n)},i()}([],[])},o.scale.quantize=function(){return function e(t,n,r){var i,o;function a(e){return r[Math.max(0,Math.min(o,Math.floor(i*(e-t))))]}function s(){return i=r.length/(n-t),o=r.length-1,a}return a.domain=function(e){return arguments.length?(t=+e[0],n=+e[e.length-1],s()):[t,n]},a.range=function(e){return arguments.length?(r=e,s()):r},a.invertExtent=function(e){return[e=(e=r.indexOf(e))<0?NaN:e/i+t,e+1/i]},a.copy=function(){return e(t,n,r)},s()}(0,1,[0,1])},o.scale.threshold=function(){return function e(t,n){function r(e){if(e<=e)return n[o.bisect(t,e)]}return r.domain=function(e){return arguments.length?(t=e,r):t},r.range=function(e){return arguments.length?(n=e,r):n},r.invertExtent=function(e){return e=n.indexOf(e),[t[e-1],t[e]]},r.copy=function(){return e(t,n)},r}([.5],[0,1])},o.scale.identity=function(){return function e(t){function n(e){return+e}return n.invert=n,n.domain=n.range=function(e){return arguments.length?(t=e.map(n),n):t},n.ticks=function(e){return Ea(t,e)},n.tickFormat=function(e,n){return Oa(t,e,n)},n.copy=function(){return e(t)},n}([0,1])},o.svg={},o.svg.arc=function(){var e=Ua,t=Va,n=Ba,r=Ha,i=Ya,o=Wa,a=Ga;function s(){var s=Math.max(0,+e.apply(this,arguments)),u=Math.max(0,+t.apply(this,arguments)),c=i.apply(this,arguments)-De,d=o.apply(this,arguments)-De,h=Math.abs(d-c),f=c>d?0:1;if(u<s&&(p=u,u=s,s=p),h>=Le)return l(u,f)+(s?l(s,1-f):"")+"Z";var p,g,v,m,y,b,C,A,_,I,x,w,M=0,S=0,T=[];if((m=(+a.apply(this,arguments)||0)/2)&&(v=r===Ha?Math.sqrt(s*s+u*u):+r.apply(this,arguments),f||(S*=-1),u&&(S=Fe(v/u*Math.sin(m))),s&&(M=Fe(v/s*Math.sin(m)))),u){y=u*Math.cos(c+S),b=u*Math.sin(c+S),C=u*Math.cos(d-S),A=u*Math.sin(d-S);var E=Math.abs(d-c-2*S)<=Oe?0:1;if(S&&Xa(y,b,C,A)===f^E){var O=(c+d)/2;y=u*Math.cos(O),b=u*Math.sin(O),C=A=null}}else y=b=0;if(s){_=s*Math.cos(d-M),I=s*Math.sin(d-M),x=s*Math.cos(c+M),w=s*Math.sin(c+M);var k=Math.abs(c-d+2*M)<=Oe?0:1;if(M&&Xa(_,I,x,w)===1-f^k){var L=(c+d)/2;_=s*Math.cos(L),I=s*Math.sin(L),x=w=null}}else _=I=0;if(h>Te&&(p=Math.min(Math.abs(u-s)/2,+n.apply(this,arguments)))>.001){g=s<u^f?0:1;var D=p,P=p;if(h<Oe){var N=null==x?[_,I]:null==C?[y,b]:hi([y,b],[x,w],[C,A],[_,I]),j=y-N[0],R=b-N[1],z=C-N[0],F=A-N[1],B=1/Math.sin(Math.acos((j*z+R*F)/(Math.sqrt(j*j+R*R)*Math.sqrt(z*z+F*F)))/2),H=Math.sqrt(N[0]*N[0]+N[1]*N[1]);P=Math.min(p,(s-H)/(B-1)),D=Math.min(p,(u-H)/(B+1))}if(null!=C){var U=qa(null==x?[_,I]:[x,w],[y,b],u,D,f),V=qa([C,A],[_,I],u,D,f);p===D?T.push("M",U[0],"A",D,",",D," 0 0,",g," ",U[1],"A",u,",",u," 0 ",1-f^Xa(U[1][0],U[1][1],V[1][0],V[1][1]),",",f," ",V[1],"A",D,",",D," 0 0,",g," ",V[0]):T.push("M",U[0],"A",D,",",D," 0 1,",g," ",V[0])}else T.push("M",y,",",b);if(null!=x){var Y=qa([y,b],[x,w],s,-P,f),W=qa([_,I],null==C?[y,b]:[C,A],s,-P,f);p===P?T.push("L",W[0],"A",P,",",P," 0 0,",g," ",W[1],"A",s,",",s," 0 ",f^Xa(W[1][0],W[1][1],Y[1][0],Y[1][1]),",",1-f," ",Y[1],"A",P,",",P," 0 0,",g," ",Y[0]):T.push("L",W[0],"A",P,",",P," 0 0,",g," ",Y[0])}else T.push("L",_,",",I)}else T.push("M",y,",",b),null!=C&&T.push("A",u,",",u," 0 ",E,",",f," ",C,",",A),T.push("L",_,",",I),null!=x&&T.push("A",s,",",s," 0 ",k,",",1-f," ",x,",",w);return T.push("Z"),T.join("")}function l(e,t){return"M0,"+e+"A"+e+","+e+" 0 1,"+t+" 0,"+-e+"A"+e+","+e+" 0 1,"+t+" 0,"+e}return s.innerRadius=function(t){return arguments.length?(e=yt(t),s):e},s.outerRadius=function(e){return arguments.length?(t=yt(e),s):t},s.cornerRadius=function(e){return arguments.length?(n=yt(e),s):n},s.padRadius=function(e){return arguments.length?(r=e==Ha?Ha:yt(e),s):r},s.startAngle=function(e){return arguments.length?(i=yt(e),s):i},s.endAngle=function(e){return arguments.length?(o=yt(e),s):o},s.padAngle=function(e){return arguments.length?(a=yt(e),s):a},s.centroid=function(){var n=(+e.apply(this,arguments)+ +t.apply(this,arguments))/2,r=(+i.apply(this,arguments)+ +o.apply(this,arguments))/2-De;return[Math.cos(r)*n,Math.sin(r)*n]},s};var Ha="auto";function Ua(e){return e.innerRadius}function Va(e){return e.outerRadius}function Ya(e){return e.startAngle}function Wa(e){return e.endAngle}function Ga(e){return e&&e.padAngle}function Xa(e,t,n,r){return(e-n)*t-(t-r)*e>0?0:1}function qa(e,t,n,r,i){var o=e[0]-t[0],a=e[1]-t[1],s=(i?r:-r)/Math.sqrt(o*o+a*a),l=s*a,u=-s*o,c=e[0]+l,d=e[1]+u,h=t[0]+l,f=t[1]+u,p=(c+h)/2,g=(d+f)/2,v=h-c,m=f-d,y=v*v+m*m,b=n-r,C=c*f-h*d,A=(m<0?-1:1)*Math.sqrt(Math.max(0,b*b*y-C*C)),_=(C*m-v*A)/y,I=(-C*v-m*A)/y,x=(C*m+v*A)/y,w=(-C*v+m*A)/y,M=_-p,S=I-g,T=x-p,E=w-g;return M*M+S*S>T*T+E*E&&(_=x,I=w),[[_-l,I-u],[_*n/b,I*n/b]]}function Za(e){var t=ai,n=si,r=Kn,i=Ka,o=i.key,a=.7;function s(o){var s,l=[],u=[],c=-1,d=o.length,h=yt(t),f=yt(n);function p(){l.push("M",i(e(u),a))}for(;++c<d;)r.call(this,s=o[c],c)?u.push([+h.call(this,s,c),+f.call(this,s,c)]):u.length&&(p(),u=[]);return u.length&&p(),l.length?l.join(""):null}return s.x=function(e){return arguments.length?(t=e,s):t},s.y=function(e){return arguments.length?(n=e,s):n},s.defined=function(e){return arguments.length?(r=e,s):r},s.interpolate=function(e){return arguments.length?(o="function"==typeof e?i=e:(i=Qa.get(e)||Ka).key,s):o},s.tension=function(e){return arguments.length?(a=e,s):a},s}o.svg.line=function(){return Za(P)};var Qa=o.map({linear:Ka,"linear-closed":Ja,step:function(e){var t=0,n=e.length,r=e[0],i=[r[0],",",r[1]];for(;++t<n;)i.push("H",(r[0]+(r=e[t])[0])/2,"V",r[1]);n>1&&i.push("H",r[0]);return i.join("")},"step-before":$a,"step-after":es,basis:rs,"basis-open":function(e){if(e.length<4)return Ka(e);var t,n=[],r=-1,i=e.length,o=[0],a=[0];for(;++r<3;)t=e[r],o.push(t[0]),a.push(t[1]);n.push(is(ss,o)+","+is(ss,a)),--r;for(;++r<i;)t=e[r],o.shift(),o.push(t[0]),a.shift(),a.push(t[1]),ls(n,o,a);return n.join("")},"basis-closed":function(e){var t,n,r=-1,i=e.length,o=i+4,a=[],s=[];for(;++r<4;)n=e[r%i],a.push(n[0]),s.push(n[1]);t=[is(ss,a),",",is(ss,s)],--r;for(;++r<o;)n=e[r%i],a.shift(),a.push(n[0]),s.shift(),s.push(n[1]),ls(t,a,s);return t.join("")},bundle:function(e,t){var n=e.length-1;if(n)for(var r,i,o=e[0][0],a=e[0][1],s=e[n][0]-o,l=e[n][1]-a,u=-1;++u<=n;)r=e[u],i=u/n,r[0]=t*r[0]+(1-t)*(o+i*s),r[1]=t*r[1]+(1-t)*(a+i*l);return rs(e)},cardinal:function(e,t){return e.length<3?Ka(e):e[0]+ts(e,ns(e,t))},"cardinal-open":function(e,t){return e.length<4?Ka(e):e[1]+ts(e.slice(1,-1),ns(e,t))},"cardinal-closed":function(e,t){return e.length<3?Ja(e):e[0]+ts((e.push(e[0]),e),ns([e[e.length-2]].concat(e,[e[1]]),t))},monotone:function(e){return e.length<3?Ka(e):e[0]+ts(e,function(e){var t,n,r,i,o=[],a=function(e){var t=0,n=e.length-1,r=[],i=e[0],o=e[1],a=r[0]=us(i,o);for(;++t<n;)r[t]=(a+(a=us(i=o,o=e[t+1])))/2;return r[t]=a,r}(e),s=-1,l=e.length-1;for(;++s<l;)t=us(e[s],e[s+1]),_(t)<Te?a[s]=a[s+1]=0:(n=a[s]/t,r=a[s+1]/t,(i=n*n+r*r)>9&&(i=3*t/Math.sqrt(i),a[s]=i*n,a[s+1]=i*r));s=-1;for(;++s<=l;)i=(e[Math.min(l,s+1)][0]-e[Math.max(0,s-1)][0])/(6*(1+a[s]*a[s])),o.push([i||0,a[s]*i||0]);return o}(e))}});function Ka(e){return e.length>1?e.join("L"):e+"Z"}function Ja(e){return e.join("L")+"Z"}function $a(e){for(var t=0,n=e.length,r=e[0],i=[r[0],",",r[1]];++t<n;)i.push("V",(r=e[t])[1],"H",r[0]);return i.join("")}function es(e){for(var t=0,n=e.length,r=e[0],i=[r[0],",",r[1]];++t<n;)i.push("H",(r=e[t])[0],"V",r[1]);return i.join("")}function ts(e,t){if(t.length<1||e.length!=t.length&&e.length!=t.length+2)return Ka(e);var n=e.length!=t.length,r="",i=e[0],o=e[1],a=t[0],s=a,l=1;if(n&&(r+="Q"+(o[0]-2*a[0]/3)+","+(o[1]-2*a[1]/3)+","+o[0]+","+o[1],i=e[1],l=2),t.length>1){s=t[1],o=e[l],l++,r+="C"+(i[0]+a[0])+","+(i[1]+a[1])+","+(o[0]-s[0])+","+(o[1]-s[1])+","+o[0]+","+o[1];for(var u=2;u<t.length;u++,l++)o=e[l],s=t[u],r+="S"+(o[0]-s[0])+","+(o[1]-s[1])+","+o[0]+","+o[1]}if(n){var c=e[l];r+="Q"+(o[0]+2*s[0]/3)+","+(o[1]+2*s[1]/3)+","+c[0]+","+c[1]}return r}function ns(e,t){for(var n,r=[],i=(1-t)/2,o=e[0],a=e[1],s=1,l=e.length;++s<l;)n=o,o=a,a=e[s],r.push([i*(a[0]-n[0]),i*(a[1]-n[1])]);return r}function rs(e){if(e.length<3)return Ka(e);var t=1,n=e.length,r=e[0],i=r[0],o=r[1],a=[i,i,i,(r=e[1])[0]],s=[o,o,o,r[1]],l=[i,",",o,"L",is(ss,a),",",is(ss,s)];for(e.push(e[n-1]);++t<=n;)r=e[t],a.shift(),a.push(r[0]),s.shift(),s.push(r[1]),ls(l,a,s);return e.pop(),l.push("L",r),l.join("")}function is(e,t){return e[0]*t[0]+e[1]*t[1]+e[2]*t[2]+e[3]*t[3]}Qa.forEach((function(e,t){t.key=e,t.closed=/-closed$/.test(e)}));var os=[0,2/3,1/3,0],as=[0,1/3,2/3,0],ss=[0,1/6,2/3,1/6];function ls(e,t,n){e.push("C",is(os,t),",",is(os,n),",",is(as,t),",",is(as,n),",",is(ss,t),",",is(ss,n))}function us(e,t){return(t[1]-e[1])/(t[0]-e[0])}function cs(e){for(var t,n,r,i=-1,o=e.length;++i<o;)n=(t=e[i])[0],r=t[1]-De,t[0]=n*Math.cos(r),t[1]=n*Math.sin(r);return e}function ds(e){var t=ai,n=ai,r=0,i=si,o=Kn,a=Ka,s=a.key,l=a,u="L",c=.7;function d(s){var d,h,f,p=[],g=[],v=[],m=-1,y=s.length,b=yt(t),C=yt(r),A=t===n?function(){return h}:yt(n),_=r===i?function(){return f}:yt(i);function I(){p.push("M",a(e(v),c),u,l(e(g.reverse()),c),"Z")}for(;++m<y;)o.call(this,d=s[m],m)?(g.push([h=+b.call(this,d,m),f=+C.call(this,d,m)]),v.push([+A.call(this,d,m),+_.call(this,d,m)])):g.length&&(I(),g=[],v=[]);return g.length&&I(),p.length?p.join(""):null}return d.x=function(e){return arguments.length?(t=n=e,d):n},d.x0=function(e){return arguments.length?(t=e,d):t},d.x1=function(e){return arguments.length?(n=e,d):n},d.y=function(e){return arguments.length?(r=i=e,d):i},d.y0=function(e){return arguments.length?(r=e,d):r},d.y1=function(e){return arguments.length?(i=e,d):i},d.defined=function(e){return arguments.length?(o=e,d):o},d.interpolate=function(e){return arguments.length?(s="function"==typeof e?a=e:(a=Qa.get(e)||Ka).key,l=a.reverse||a,u=a.closed?"M":"L",d):s},d.tension=function(e){return arguments.length?(c=e,d):c},d}function hs(e){return e.radius}function fs(e){return[e.x,e.y]}function ps(e){return function(){var t=e.apply(this,arguments),n=t[0],r=t[1]-De;return[n*Math.cos(r),n*Math.sin(r)]}}function gs(){return 64}function vs(){return"circle"}function ms(e){var t=Math.sqrt(e/Oe);return"M0,"+t+"A"+t+","+t+" 0 1,1 0,"+-t+"A"+t+","+t+" 0 1,1 0,"+t+"Z"}o.svg.line.radial=function(){var e=Za(cs);return e.radius=e.x,delete e.x,e.angle=e.y,delete e.y,e},$a.reverse=es,es.reverse=$a,o.svg.area=function(){return ds(P)},o.svg.area.radial=function(){var e=ds(cs);return e.radius=e.x,delete e.x,e.innerRadius=e.x0,delete e.x0,e.outerRadius=e.x1,delete e.x1,e.angle=e.y,delete e.y,e.startAngle=e.y0,delete e.y0,e.endAngle=e.y1,delete e.y1,e},o.svg.chord=function(){var e=Gr,t=Xr,n=hs,r=Ya,i=Wa;function o(n,r){var i,o,u=a(this,e,n,r),c=a(this,t,n,r);return"M"+u.p0+s(u.r,u.p1,u.a1-u.a0)+(o=c,((i=u).a0==o.a0&&i.a1==o.a1?l(u.r,u.p1,u.r,u.p0):l(u.r,u.p1,c.r,c.p0)+s(c.r,c.p1,c.a1-c.a0)+l(c.r,c.p1,u.r,u.p0))+"Z")}function a(e,t,o,a){var s=t.call(e,o,a),l=n.call(e,s,a),u=r.call(e,s,a)-De,c=i.call(e,s,a)-De;return{r:l,a0:u,a1:c,p0:[l*Math.cos(u),l*Math.sin(u)],p1:[l*Math.cos(c),l*Math.sin(c)]}}function s(e,t,n){return"A"+e+","+e+" 0 "+ +(n>Oe)+",1 "+t}function l(e,t,n,r){return"Q 0,0 "+r}return o.radius=function(e){return arguments.length?(n=yt(e),o):n},o.source=function(t){return arguments.length?(e=yt(t),o):e},o.target=function(e){return arguments.length?(t=yt(e),o):t},o.startAngle=function(e){return arguments.length?(r=yt(e),o):r},o.endAngle=function(e){return arguments.length?(i=yt(e),o):i},o},o.svg.diagonal=function(){var e=Gr,t=Xr,n=fs;function r(r,i){var o=e.call(this,r,i),a=t.call(this,r,i),s=(o.y+a.y)/2,l=[o,{x:o.x,y:s},{x:a.x,y:s},a];return"M"+(l=l.map(n))[0]+"C"+l[1]+" "+l[2]+" "+l[3]}return r.source=function(t){return arguments.length?(e=yt(t),r):e},r.target=function(e){return arguments.length?(t=yt(e),r):t},r.projection=function(e){return arguments.length?(n=e,r):n},r},o.svg.diagonal.radial=function(){var e=o.svg.diagonal(),t=fs,n=e.projection;return e.projection=function(e){return arguments.length?n(ps(t=e)):t},e},o.svg.symbol=function(){var e=vs,t=gs;function n(n,r){return(ys.get(e.call(this,n,r))||ms)(t.call(this,n,r))}return n.type=function(t){return arguments.length?(e=yt(t),n):e},n.size=function(e){return arguments.length?(t=yt(e),n):t},n};var ys=o.map({circle:ms,cross:function(e){var t=Math.sqrt(e/5)/2;return"M"+-3*t+","+-t+"H"+-t+"V"+-3*t+"H"+t+"V"+-t+"H"+3*t+"V"+t+"H"+t+"V"+3*t+"H"+-t+"V"+t+"H"+-3*t+"Z"},diamond:function(e){var t=Math.sqrt(e/(2*Cs)),n=t*Cs;return"M0,"+-t+"L"+n+",0 0,"+t+" "+-n+",0Z"},square:function(e){var t=Math.sqrt(e)/2;return"M"+-t+","+-t+"L"+t+","+-t+" "+t+","+t+" "+-t+","+t+"Z"},"triangle-down":function(e){var t=Math.sqrt(e/bs),n=t*bs/2;return"M0,"+n+"L"+t+","+-n+" "+-t+","+-n+"Z"},"triangle-up":function(e){var t=Math.sqrt(e/bs),n=t*bs/2;return"M0,"+-n+"L"+t+","+n+" "+-t+","+n+"Z"}});o.svg.symbolTypes=ys.keys();var bs=Math.sqrt(3),Cs=Math.tan(30*Pe);Q.transition=function(e){for(var t,n,r=xs||++Ss,i=Os(e),o=[],a=ws||{time:Date.now(),ease:po,delay:0,duration:250},s=-1,l=this.length;++s<l;){o.push(t=[]);for(var u=this[s],c=-1,d=u.length;++c<d;)(n=u[c])&&ks(n,c,i,r,a),t.push(n)}return Is(o,i,r)},Q.interrupt=function(e){return this.each(null==e?As:_s(Os(e)))};var As=_s(Os());function _s(e){return function(){var t,n,r;(t=this[e])&&(r=t[n=t.active])&&(r.timer.c=null,r.timer.t=NaN,--t.count?delete t[n]:delete this[e],t.active+=.5,r.event&&r.event.interrupt.call(this,this.__data__,r.index))}}function Is(e,t,n){return W(e,Ms),e.namespace=t,e.id=n,e}var xs,ws,Ms=[],Ss=0;function Ts(e,t,n,r){var i=e.id,o=e.namespace;return pe(e,"function"==typeof n?function(e,a,s){e[o][i].tween.set(t,r(n.call(e,e.__data__,a,s)))}:(n=r(n),function(e){e[o][i].tween.set(t,n)}))}function Es(e){return null==e&&(e=""),function(){this.textContent=e}}function Os(e){return null==e?"__transition__":"__transition_"+e+"__"}function ks(e,t,n,r,i){var o,a,s,l,u,c=e[n]||(e[n]={active:0,count:0}),d=c[r];function h(n){var i=c.active,h=c[i];for(var p in h&&(h.timer.c=null,h.timer.t=NaN,--c.count,delete c[i],h.event&&h.event.interrupt.call(e,e.__data__,h.index)),c)if(+p<r){var g=c[p];g.timer.c=null,g.timer.t=NaN,--c.count,delete c[p]}a.c=f,Mt((function(){return a.c&&f(n||1)&&(a.c=null,a.t=NaN),1}),0,o),c.active=r,d.event&&d.event.start.call(e,e.__data__,t),u=[],d.tween.forEach((function(n,r){(r=r.call(e,e.__data__,t))&&u.push(r)})),l=d.ease,s=d.duration}function f(i){for(var o=i/s,a=l(o),h=u.length;h>0;)u[--h].call(e,a);if(o>=1)return d.event&&d.event.end.call(e,e.__data__,t),--c.count?delete c[r]:delete e[n],1}d||(o=i.time,a=Mt((function(e){var t=d.delay;if(a.t=t+o,t<=e)return h(e-t);a.c=h}),0,o),d=c[r]={tween:new w,time:o,timer:a,delay:i.delay,duration:i.duration,ease:i.ease,index:t},i=null,++c.count)}Ms.call=Q.call,Ms.empty=Q.empty,Ms.node=Q.node,Ms.size=Q.size,o.transition=function(e,t){return e&&e.transition?xs?e.transition(t):e:o.selection().transition(e)},o.transition.prototype=Ms,Ms.select=function(e){var t,n,r,i=this.id,o=this.namespace,a=[];e=K(e);for(var s=-1,l=this.length;++s<l;){a.push(t=[]);for(var u=this[s],c=-1,d=u.length;++c<d;)(r=u[c])&&(n=e.call(r,r.__data__,c,s))?("__data__"in r&&(n.__data__=r.__data__),ks(n,c,o,i,r[o][i]),t.push(n)):t.push(null)}return Is(a,o,i)},Ms.selectAll=function(e){var t,n,r,i,o,a=this.id,s=this.namespace,l=[];e=J(e);for(var u=-1,c=this.length;++u<c;)for(var d=this[u],h=-1,f=d.length;++h<f;)if(r=d[h]){o=r[s][a],n=e.call(r,r.__data__,h,u),l.push(t=[]);for(var p=-1,g=n.length;++p<g;)(i=n[p])&&ks(i,p,s,a,o),t.push(i)}return Is(l,s,a)},Ms.filter=function(e){var t,n,r=[];"function"!=typeof e&&(e=he(e));for(var i=0,o=this.length;i<o;i++){r.push(t=[]);for(var a,s=0,l=(a=this[i]).length;s<l;s++)(n=a[s])&&e.call(n,n.__data__,s,i)&&t.push(n)}return Is(r,this.namespace,this.id)},Ms.tween=function(e,t){var n=this.id,r=this.namespace;return arguments.length<2?this.node()[r][n].tween.get(e):pe(this,null==t?function(t){t[r][n].tween.remove(e)}:function(i){i[r][n].tween.set(e,t)})},Ms.attr=function(e,t){if(arguments.length<2){for(t in e)this.attr(t,e[t]);return this}var n="transform"==e?wo:ro,r=o.ns.qualify(e);function i(){this.removeAttribute(r)}function a(){this.removeAttributeNS(r.space,r.local)}function s(e){return null==e?i:(e+="",function(){var t,i=this.getAttribute(r);return i!==e&&(t=n(i,e),function(e){this.setAttribute(r,t(e))})})}function l(e){return null==e?a:(e+="",function(){var t,i=this.getAttributeNS(r.space,r.local);return i!==e&&(t=n(i,e),function(e){this.setAttributeNS(r.space,r.local,t(e))})})}return Ts(this,"attr."+e,t,r.local?l:s)},Ms.attrTween=function(e,t){var n=o.ns.qualify(e);return this.tween("attr."+e,n.local?function(e,r){var i=t.call(this,e,r,this.getAttributeNS(n.space,n.local));return i&&function(e){this.setAttributeNS(n.space,n.local,i(e))}}:function(e,r){var i=t.call(this,e,r,this.getAttribute(n));return i&&function(e){this.setAttribute(n,i(e))}})},Ms.style=function(e,t,n){var r=arguments.length;if(r<3){if("string"!=typeof e){for(n in r<2&&(t=""),e)this.style(n,e[n],t);return this}n=""}function i(){this.style.removeProperty(e)}function o(t){return null==t?i:(t+="",function(){var r,i=c(this).getComputedStyle(this,null).getPropertyValue(e);return i!==t&&(r=ro(i,t),function(t){this.style.setProperty(e,r(t),n)})})}return Ts(this,"style."+e,t,o)},Ms.styleTween=function(e,t,n){function r(r,i){var o=t.call(this,r,i,c(this).getComputedStyle(this,null).getPropertyValue(e));return o&&function(t){this.style.setProperty(e,o(t),n)}}return arguments.length<3&&(n=""),this.tween("style."+e,r)},Ms.text=function(e){return Ts(this,"text",e,Es)},Ms.remove=function(){var e=this.namespace;return this.each("end.transition",(function(){var t;this[e].count<2&&(t=this.parentNode)&&t.removeChild(this)}))},Ms.ease=function(e){var t=this.id,n=this.namespace;return arguments.length<1?this.node()[n][t].ease:("function"!=typeof e&&(e=o.ease.apply(o,arguments)),pe(this,(function(r){r[n][t].ease=e})))},Ms.delay=function(e){var t=this.id,n=this.namespace;return arguments.length<1?this.node()[n][t].delay:pe(this,"function"==typeof e?function(r,i,o){r[n][t].delay=+e.call(r,r.__data__,i,o)}:(e=+e,function(r){r[n][t].delay=e}))},Ms.duration=function(e){var t=this.id,n=this.namespace;return arguments.length<1?this.node()[n][t].duration:pe(this,"function"==typeof e?function(r,i,o){r[n][t].duration=Math.max(1,e.call(r,r.__data__,i,o))}:(e=Math.max(1,e),function(r){r[n][t].duration=e}))},Ms.each=function(e,t){var n=this.id,r=this.namespace;if(arguments.length<2){var i=ws,a=xs;try{xs=n,pe(this,(function(t,i,o){ws=t[r][n],e.call(t,t.__data__,i,o)}))}finally{ws=i,xs=a}}else pe(this,(function(i){var a=i[r][n];(a.event||(a.event=o.dispatch("start","end","interrupt"))).on(e,t)}));return this},Ms.transition=function(){for(var e,t,n,r=this.id,i=++Ss,o=this.namespace,a=[],s=0,l=this.length;s<l;s++){a.push(e=[]);for(var u,c=0,d=(u=this[s]).length;c<d;c++)(t=u[c])&&ks(t,c,o,i,{time:(n=t[o][r]).time,ease:n.ease,delay:n.delay+n.duration,duration:n.duration}),e.push(t)}return Is(a,o,i)},o.svg.axis=function(){var e,t=o.scale.linear(),n=Ls,r=6,i=6,a=3,l=[10],u=null;function c(s){s.each((function(){var s,c=o.select(this),d=this.__chart__||t,h=this.__chart__=t.copy(),f=null==u?h.ticks?h.ticks.apply(h,l):h.domain():u,p=null==e?h.tickFormat?h.tickFormat.apply(h,l):P:e,g=c.selectAll(".tick").data(f,h),v=g.enter().insert("g",".domain").attr("class","tick").style("opacity",Te),m=o.transition(g.exit()).style("opacity",Te).remove(),y=o.transition(g.order()).style("opacity",1),b=Math.max(r,0)+a,C=Ca(h),A=c.selectAll(".domain").data([0]),_=(A.enter().append("path").attr("class","domain"),o.transition(A));v.append("line"),v.append("text");var I,x,w,M,S=v.select("line"),T=y.select("line"),E=g.select("text").text(p),O=v.select("text"),k=y.select("text"),L="top"===n||"left"===n?-1:1;if("bottom"===n||"top"===n?(s=Ps,I="x",w="y",x="x2",M="y2",E.attr("dy",L<0?"0em":".71em").style("text-anchor","middle"),_.attr("d","M"+C[0]+","+L*i+"V0H"+C[1]+"V"+L*i)):(s=Ns,I="y",w="x",x="y2",M="x2",E.attr("dy",".32em").style("text-anchor",L<0?"end":"start"),_.attr("d","M"+L*i+","+C[0]+"H0V"+C[1]+"H"+L*i)),S.attr(M,L*r),O.attr(w,L*b),T.attr(x,0).attr(M,L*r),k.attr(I,0).attr(w,L*b),h.rangeBand){var D=h,N=D.rangeBand()/2;d=h=function(e){return D(e)+N}}else d.rangeBand?d=h:m.call(s,h,d);v.call(s,d,h),y.call(s,h,h)}))}return c.scale=function(e){return arguments.length?(t=e,c):t},c.orient=function(e){return arguments.length?(n=e in Ds?e+"":Ls,c):n},c.ticks=function(){return arguments.length?(l=s(arguments),c):l},c.tickValues=function(e){return arguments.length?(u=e,c):u},c.tickFormat=function(t){return arguments.length?(e=t,c):e},c.tickSize=function(e){var t=arguments.length;return t?(r=+e,i=+arguments[t-1],c):r},c.innerTickSize=function(e){return arguments.length?(r=+e,c):r},c.outerTickSize=function(e){return arguments.length?(i=+e,c):i},c.tickPadding=function(e){return arguments.length?(a=+e,c):a},c.tickSubdivide=function(){return arguments.length&&c},c};var Ls="bottom",Ds={top:1,right:1,bottom:1,left:1};function Ps(e,t,n){e.attr("transform",(function(e){var r=t(e);return"translate("+(isFinite(r)?r:n(e))+",0)"}))}function Ns(e,t,n){e.attr("transform",(function(e){var r=t(e);return"translate(0,"+(isFinite(r)?r:n(e))+")"}))}o.svg.brush=function(){var e,t,n=V(h,"brushstart","brush","brushend"),r=null,i=null,a=[0,0],s=[0,0],l=!0,u=!0,d=Rs[0];function h(e){e.each((function(){var e=o.select(this).style("pointer-events","all").style("-webkit-tap-highlight-color","rgba(0,0,0,0)").on("mousedown.brush",v).on("touchstart.brush",v),t=e.selectAll(".background").data([0]);t.enter().append("rect").attr("class","background").style("visibility","hidden").style("cursor","crosshair"),e.selectAll(".extent").data([0]).enter().append("rect").attr("class","extent").style("cursor","move");var n=e.selectAll(".resize").data(d,P);n.exit().remove(),n.enter().append("g").attr("class",(function(e){return"resize "+e})).style("cursor",(function(e){return js[e]})).append("rect").attr("x",(function(e){return/[ew]$/.test(e)?-3:null})).attr("y",(function(e){return/^[ns]/.test(e)?-3:null})).attr("width",6).attr("height",6).style("visibility","hidden"),n.style("display",h.empty()?"none":null);var a,s=o.transition(e),l=o.transition(t);r&&(a=Ca(r),l.attr("x",a[0]).attr("width",a[1]-a[0]),p(s)),i&&(a=Ca(i),l.attr("y",a[0]).attr("height",a[1]-a[0]),g(s)),f(s)}))}function f(e){e.selectAll(".resize").attr("transform",(function(e){return"translate("+a[+/e$/.test(e)]+","+s[+/^s/.test(e)]+")"}))}function p(e){e.select(".extent").attr("x",a[0]),e.selectAll(".extent,.n>rect,.s>rect").attr("width",a[1]-a[0])}function g(e){e.select(".extent").attr("y",s[0]),e.selectAll(".extent,.e>rect,.w>rect").attr("height",s[1]-s[0])}function v(){var d,v,m=this,y=o.select(o.event.target),b=n.of(m,arguments),C=o.select(m),A=y.datum(),_=!/^(n|s)$/.test(A)&&r,I=!/^(e|w)$/.test(A)&&i,x=y.classed("extent"),w=xe(m),M=o.mouse(m),S=o.select(c(m)).on("keydown.brush",O).on("keyup.brush",k);if(o.event.changedTouches?S.on("touchmove.brush",L).on("touchend.brush",P):S.on("mousemove.brush",L).on("mouseup.brush",P),C.interrupt().selectAll("*").interrupt(),x)M[0]=a[0]-M[0],M[1]=s[0]-M[1];else if(A){var T=+/w$/.test(A),E=+/^n/.test(A);v=[a[1-T]-M[0],s[1-E]-M[1]],M[0]=a[T],M[1]=s[E]}else o.event.altKey&&(d=M.slice());function O(){32==o.event.keyCode&&(x||(d=null,M[0]-=a[1],M[1]-=s[1],x=2),H())}function k(){32==o.event.keyCode&&2==x&&(M[0]+=a[1],M[1]+=s[1],x=0,H())}function L(){var e=o.mouse(m),t=!1;v&&(e[0]+=v[0],e[1]+=v[1]),x||(o.event.altKey?(d||(d=[(a[0]+a[1])/2,(s[0]+s[1])/2]),M[0]=a[+(e[0]<d[0])],M[1]=s[+(e[1]<d[1])]):d=null),_&&D(e,r,0)&&(p(C),t=!0),I&&D(e,i,1)&&(g(C),t=!0),t&&(f(C),b({type:"brush",mode:x?"move":"resize"}))}function D(n,r,i){var o,c,h=Ca(r),f=h[0],p=h[1],g=M[i],v=i?s:a,m=v[1]-v[0];if(x&&(f-=g,p-=m+g),o=(i?u:l)?Math.max(f,Math.min(p,n[i])):n[i],x?c=(o+=g)+m:(d&&(g=Math.max(f,Math.min(p,2*d[i]-o))),g<o?(c=o,o=g):c=g),v[0]!=o||v[1]!=c)return i?t=null:e=null,v[0]=o,v[1]=c,!0}function P(){L(),C.style("pointer-events","all").selectAll(".resize").style("display",h.empty()?"none":null),o.select("body").style("cursor",null),S.on("mousemove.brush",null).on("mouseup.brush",null).on("touchmove.brush",null).on("touchend.brush",null).on("keydown.brush",null).on("keyup.brush",null),w(),b({type:"brushend"})}C.style("pointer-events","none").selectAll(".resize").style("display",null),o.select("body").style("cursor",y.style("cursor")),b({type:"brushstart"}),L()}return h.event=function(r){r.each((function(){var r=n.of(this,arguments),i={x:a,y:s,i:e,j:t},l=this.__chart__||i;this.__chart__=i,xs?o.select(this).transition().each("start.brush",(function(){e=l.i,t=l.j,a=l.x,s=l.y,r({type:"brushstart"})})).tween("brush:brush",(function(){var n=io(a,i.x),o=io(s,i.y);return e=t=null,function(e){a=i.x=n(e),s=i.y=o(e),r({type:"brush",mode:"resize"})}})).each("end.brush",(function(){e=i.i,t=i.j,r({type:"brush",mode:"resize"}),r({type:"brushend"})})):(r({type:"brushstart"}),r({type:"brush",mode:"resize"}),r({type:"brushend"}))}))},h.x=function(e){return arguments.length?(d=Rs[!(r=e)<<1|!i],h):r},h.y=function(e){return arguments.length?(d=Rs[!r<<1|!(i=e)],h):i},h.clamp=function(e){return arguments.length?(r&&i?(l=!!e[0],u=!!e[1]):r?l=!!e:i&&(u=!!e),h):r&&i?[l,u]:r?l:i?u:null},h.extent=function(n){var o,l,u,c,d;return arguments.length?(r&&(o=n[0],l=n[1],i&&(o=o[0],l=l[0]),e=[o,l],r.invert&&(o=r(o),l=r(l)),l<o&&(d=o,o=l,l=d),o==a[0]&&l==a[1]||(a=[o,l])),i&&(u=n[0],c=n[1],r&&(u=u[1],c=c[1]),t=[u,c],i.invert&&(u=i(u),c=i(c)),c<u&&(d=u,u=c,c=d),u==s[0]&&c==s[1]||(s=[u,c])),h):(r&&(e?(o=e[0],l=e[1]):(o=a[0],l=a[1],r.invert&&(o=r.invert(o),l=r.invert(l)),l<o&&(d=o,o=l,l=d))),i&&(t?(u=t[0],c=t[1]):(u=s[0],c=s[1],i.invert&&(u=i.invert(u),c=i.invert(c)),c<u&&(d=u,u=c,c=d))),r&&i?[[o,u],[l,c]]:r?[o,l]:i&&[u,c])},h.clear=function(){return h.empty()||(a=[0,0],s=[0,0],e=t=null),h},h.empty=function(){return!!r&&a[0]==a[1]||!!i&&s[0]==s[1]},o.rebind(h,n,"on")};var js={n:"ns-resize",e:"ew-resize",s:"ns-resize",w:"ew-resize",nw:"nwse-resize",ne:"nesw-resize",se:"nwse-resize",sw:"nesw-resize"},Rs=[["n","e","s","w","nw","ne","se","sw"],["e","w"],["n","s"],[]],zs=jt.format=hn.timeFormat,Fs=zs.utc,Bs=Fs("%Y-%m-%dT%H:%M:%S.%LZ");function Hs(e){return e.toISOString()}function Us(e,t,n){function r(t){return e(t)}function i(e,n){var r=(e[1]-e[0])/n,i=o.bisect(Ys,r);return i==Ys.length?[t.year,Ta(e.map((function(e){return e/31536e6})),n)[2]]:i?t[r/Ys[i-1]<Ys[i]/r?i-1:i]:[Xs,Ta(e,n)[2]]}return r.invert=function(t){return Vs(e.invert(t))},r.domain=function(t){return arguments.length?(e.domain(t),r):e.domain().map(Vs)},r.nice=function(e,t){var n=r.domain(),o=ba(n),a=null==e?i(o,10):"number"==typeof e&&i(o,e);function s(n){return!isNaN(n)&&!e.range(n,Vs(+n+1),t).length}return a&&(e=a[0],t=a[1]),r.domain(_a(n,t>1?{floor:function(t){for(;s(t=e.floor(t));)t=Vs(t-1);return t},ceil:function(t){for(;s(t=e.ceil(t));)t=Vs(+t+1);return t}}:e))},r.ticks=function(e,t){var n=ba(r.domain()),o=null==e?i(n,10):"number"==typeof e?i(n,e):!e.range&&[{range:e},t];return o&&(e=o[0],t=o[1]),e.range(n[0],Vs(+n[1]+1),t<1?1:t)},r.tickFormat=function(){return n},r.copy=function(){return Us(e.copy(),t,n)},Ma(r,e)}function Vs(e){return new Date(e)}zs.iso=Date.prototype.toISOString&&+new Date("2000-01-01T00:00:00.000Z")?Hs:Bs,Hs.parse=function(e){var t=new Date(e);return isNaN(t)?null:t},Hs.toString=Bs.toString,jt.second=Bt((function(e){return new Rt(1e3*Math.floor(e/1e3))}),(function(e,t){e.setTime(e.getTime()+1e3*Math.floor(t))}),(function(e){return e.getSeconds()})),jt.seconds=jt.second.range,jt.seconds.utc=jt.second.utc.range,jt.minute=Bt((function(e){return new Rt(6e4*Math.floor(e/6e4))}),(function(e,t){e.setTime(e.getTime()+6e4*Math.floor(t))}),(function(e){return e.getMinutes()})),jt.minutes=jt.minute.range,jt.minutes.utc=jt.minute.utc.range,jt.hour=Bt((function(e){var t=e.getTimezoneOffset()/60;return new Rt(36e5*(Math.floor(e/36e5-t)+t))}),(function(e,t){e.setTime(e.getTime()+36e5*Math.floor(t))}),(function(e){return e.getHours()})),jt.hours=jt.hour.range,jt.hours.utc=jt.hour.utc.range,jt.month=Bt((function(e){return(e=jt.day(e)).setDate(1),e}),(function(e,t){e.setMonth(e.getMonth()+t)}),(function(e){return e.getMonth()})),jt.months=jt.month.range,jt.months.utc=jt.month.utc.range;var Ys=[1e3,5e3,15e3,3e4,6e4,3e5,9e5,18e5,36e5,108e5,216e5,432e5,864e5,1728e5,6048e5,2592e6,7776e6,31536e6],Ws=[[jt.second,1],[jt.second,5],[jt.second,15],[jt.second,30],[jt.minute,1],[jt.minute,5],[jt.minute,15],[jt.minute,30],[jt.hour,1],[jt.hour,3],[jt.hour,6],[jt.hour,12],[jt.day,1],[jt.day,2],[jt.week,1],[jt.month,1],[jt.month,3],[jt.year,1]],Gs=zs.multi([[".%L",function(e){return e.getMilliseconds()}],[":%S",function(e){return e.getSeconds()}],["%I:%M",function(e){return e.getMinutes()}],["%I %p",function(e){return e.getHours()}],["%a %d",function(e){return e.getDay()&&1!=e.getDate()}],["%b %d",function(e){return 1!=e.getDate()}],["%B",function(e){return e.getMonth()}],["%Y",Kn]]),Xs={range:function(e,t,n){return o.range(Math.ceil(e/n)*n,+t,n).map(Vs)},floor:P,ceil:P};Ws.year=jt.year,jt.scale=function(){return Us(o.scale.linear(),Ws,Gs)};var qs=Ws.map((function(e){return[e[0].utc,e[1]]})),Zs=Fs.multi([[".%L",function(e){return e.getUTCMilliseconds()}],[":%S",function(e){return e.getUTCSeconds()}],["%I:%M",function(e){return e.getUTCMinutes()}],["%I %p",function(e){return e.getUTCHours()}],["%a %d",function(e){return e.getUTCDay()&&1!=e.getUTCDate()}],["%b %d",function(e){return 1!=e.getUTCDate()}],["%B",function(e){return e.getUTCMonth()}],["%Y",Kn]]);function Qs(e){return JSON.parse(e.responseText)}function Ks(e){var t=l.createRange();return t.selectNode(l.body),t.createContextualFragment(e.responseText)}qs.year=jt.year.utc,jt.scale.utc=function(){return Us(o.scale.linear(),qs,Zs)},o.text=bt((function(e){return e.responseText})),o.json=function(e,t){return Ct(e,"application/json",Qs,t)},o.html=function(e,t){return Ct(e,"text/html",Ks,t)},o.xml=bt((function(e){return e.responseXML})),this.d3=o,void 0===(i="function"==typeof(r=o)?r.call(t,n,t,e):r)||(e.exports=i)}()},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){if(t.length<e)throw new TypeError(e+" argument"+(e>1?"s":"")+" required, but only "+t.length+" present")},e.exports=t.default},function(e,t,n){"use strict";var r=n(123),i=n(42),o=e.exports={},a=n(133);o.defaults=a.defaults;var s=o.defaultLine=a.defaultLine;o.lightLine=a.lightLine;var l=o.background=a.background;function u(e){if(i(e)||"string"!=typeof e)return e;var t=e.trim();if("rgb"!==t.substr(0,3))return e;var n=t.match(/^rgba?\s*\(([^()]*)\)$/);if(!n)return e;var r=n[1].trim().split(/\s*[\s,]\s*/),o="a"===t.charAt(3)&&4===r.length;if(!o&&3!==r.length)return e;for(var a=0;a<r.length;a++){if(!r[a].length)return e;if(r[a]=Number(r[a]),!(r[a]>=0))return e;if(3===a)r[a]>1&&(r[a]=1);else if(r[a]>=1)return e}var s=Math.round(255*r[0])+", "+Math.round(255*r[1])+", "+Math.round(255*r[2]);return o?"rgba("+s+", "+r[3]+")":"rgb("+s+")"}o.tinyRGB=function(e){var t=e.toRgb();return"rgb("+Math.round(t.r)+", "+Math.round(t.g)+", "+Math.round(t.b)+")"},o.rgb=function(e){return o.tinyRGB(r(e))},o.opacity=function(e){return e?r(e).getAlpha():0},o.addOpacity=function(e,t){var n=r(e).toRgb();return"rgba("+Math.round(n.r)+", "+Math.round(n.g)+", "+Math.round(n.b)+", "+t+")"},o.combine=function(e,t){var n=r(e).toRgb();if(1===n.a)return r(e).toRgbString();var i=r(t||l).toRgb(),o=1===i.a?i:{r:255*(1-i.a)+i.r*i.a,g:255*(1-i.a)+i.g*i.a,b:255*(1-i.a)+i.b*i.a},a={r:o.r*(1-n.a)+n.r*n.a,g:o.g*(1-n.a)+n.g*n.a,b:o.b*(1-n.a)+n.b*n.a};return r(a).toRgbString()},o.contrast=function(e,t,n){var i=r(e);return 1!==i.getAlpha()&&(i=r(o.combine(e,l))),(i.isDark()?t?i.lighten(t):l:n?i.darken(n):s).toString()},o.stroke=function(e,t){var n=r(t);e.style({stroke:o.tinyRGB(n),"stroke-opacity":n.getAlpha()})},o.fill=function(e,t){var n=r(t);e.style({fill:o.tinyRGB(n),"fill-opacity":n.getAlpha()})},o.clean=function(e){if(e&&"object"==typeof e){var t,n,r,i,a=Object.keys(e);for(t=0;t<a.length;t++)if(i=e[r=a[t]],"color"===r.substr(r.length-5))if(Array.isArray(i))for(n=0;n<i.length;n++)i[n]=u(i[n]);else e[r]=u(i);else if("colorscale"===r.substr(r.length-10)&&Array.isArray(i))for(n=0;n<i.length;n++)Array.isArray(i[n])&&(i[n][1]=u(i[n][1]));else if(Array.isArray(i)){var s=i[0];if(!Array.isArray(s)&&s&&"object"==typeof s)for(n=0;n<i.length;n++)o.clean(i[n])}else i&&"object"==typeof i&&o.clean(i)}}},function(e,t,n){"use strict";var r=n(965);e.exports=function(e){var t=typeof e;if("string"===t){var n=e;if(0===(e=+e)&&r(n))return!1}else if("number"!==t)return!1;return e-e<1}},function(e,t,n){var r=n(932),i=n(9);e.exports=function(e,t){return!t||"object"!==r(t)&&"function"!=typeof t?i(e):t}},function(e,t){function n(t){return e.exports=n=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)},n(t)}e.exports=n},function(e,t,n){var r=n(877);e.exports=function(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&r(e,t)}},function(e,t,n){var r=n(932);function i(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return i=function(){return e},e}e.exports=function(e){if(e&&e.__esModule)return e;if(null===e||"object"!==r(e)&&"function"!=typeof e)return{default:e};var t=i();if(t&&t.has(e))return t.get(e);var n={},o=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var a in e)if(Object.prototype.hasOwnProperty.call(e,a)){var s=o?Object.getOwnPropertyDescriptor(e,a):null;s&&(s.get||s.set)?Object.defineProperty(n,a,s):n[a]=e[a]}return n.default=e,t&&t.set(e,n),n}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){(0,i.default)(1,arguments);var t=Object.prototype.toString.call(e);return e instanceof Date||"object"==typeof e&&"[object Date]"===t?new Date(e.getTime()):"number"==typeof e||"[object Number]"===t?new Date(e):("string"!=typeof e&&"[object String]"!==t||"undefined"==typeof console||(console.warn("Starting with v2.0.0-beta.1 date-fns doesn't accept strings as date arguments. Please use `parseISO` to parse strings. See: https://git.io/fjule"),console.warn((new Error).stack)),new Date(NaN))};var r,i=(r=n(40))&&r.__esModule?r:{default:r};e.exports=t.default},function(e,t,n){"use strict";var r=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),i=this&&this.__exportStar||function(e,t){for(var n in e)"default"===n||Object.prototype.hasOwnProperty.call(t,n)||r(t,e,n)};Object.defineProperty(t,"__esModule",{value:!0}),i(n(1341),t),i(n(1343),t),i(n(1344),t),i(n(1345),t),i(n(1351),t),i(n(1352),t),i(n(1353),t),i(n(705),t)},function(e,t,n){"use strict";var r=n(443);var i=n(273),o=n(444);function a(e,t){return Object(r.a)(e)||function(e,t){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e)){var n=[],r=!0,i=!1,o=void 0;try{for(var a,s=e[Symbol.iterator]();!(r=(a=s.next()).done)&&(n.push(a.value),!t||n.length!==t);r=!0);}catch(e){i=!0,o=e}finally{try{r||null==s.return||s.return()}finally{if(i)throw o}}return n}}(e,t)||Object(i.a)(e,t)||Object(o.a)()}n.d(t,"a",(function(){return a}))},function(e,t){function n(){return e.exports=n=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},n.apply(this,arguments)}e.exports=n},function(e,t,n){"use strict";class r{constructor(e){e=e||{},this._defaults={domain:"messages",locale:document.documentElement.getAttribute("lang")||"en",pluralFunc:function(e){return{nplurals:2,plural:1!=e?1:0}},contextDelimiter:String.fromCharCode(4),stringsPrefix:""},this._locale=(e.locale||this._defaults.locale).replace("_","-"),this._domain=e.domain||this._defaults.domain,this._contextDelimiter=e.contextDelimiter||this._defaults.contextDelimiter,this._stringsPrefix=e.stringsPrefix||this._defaults.stringsPrefix,this._pluralFuncs={},this._dictionary={},this._pluralForms={},e.messages&&(this._dictionary[this._domain]={},this._dictionary[this._domain][this._locale]=e.messages),e.pluralForms&&(this._pluralForms[this._locale]=e.pluralForms)}setContextDelimiter(e){this._contextDelimiter=e}getContextDelimiter(){return this._contextDelimiter}setLocale(e){this._locale=e.replace("_","-")}getLocale(){return this._locale}setDomain(e){this._domain=e}getDomain(){return this._domain}setStringsPrefix(e){this._stringsPrefix=e}getStringsPrefix(){return this._stringsPrefix}static strfmt(e,...t){return e.replace(/%%/g,"%% ").replace(/%(\d+)/g,(function(e,n){return t[n-1]})).replace(/%% /g,"%")}loadJSON(e,t){if(!e[""]||!e[""].language||!e[""].pluralForms)throw new Error('Wrong jsonData, it must have an empty key ("") with "language" and "pluralForms" information: '+e);let n=e[""],r=JSON.parse(JSON.stringify(e));delete r[""],this.setMessages(t||this._defaults.domain,n.language,r,n.pluralForms)}__(e,...t){return this.gettext(e,...t)}_n(e,t,n,...r){return this.ngettext(e,t,n,...r)}_p(e,t,...n){return this.pgettext(e,t,...n)}_np(e,t,n,r,...i){return this.npgettext(e,t,n,r,...i)}gettext(e,...t){return this.dcnpgettext("","",e,"",0,...t)}ngettext(e,t,n,...r){return this.dcnpgettext("","",e,t,n,...r)}pgettext(e,t,...n){return this.dcnpgettext("",e,t,"",0,...n)}npgettext(e,t,n,r,...i){return this.dcnpgettext("",e,t,n,r,...i)}dcnpgettext(e,t,n,r,i,...o){let a;e=e||this._domain;let s=t?t+this._contextDelimiter+n:n,l={pluralForm:!1},u=!1,c=this._locale,d=this.expandLocale(this._locale);for(let t in d)if(c=d[t],u=this._dictionary[e]&&this._dictionary[e][c]&&this._dictionary[e][c][s],u=r?u&&this._dictionary[e][c][s].length>1:u&&1==this._dictionary[e][c][s].length,u){l.locale=c;break}if(u?a=this._dictionary[e][c][s]:(a=[n],l.pluralFunc=this._defaults.pluralFunc),!r)return this.t(a,i,l,...o);l.pluralForm=!0;let h=u?a:[n,r];return this.t(h,i,l,...o)}expandLocale(e){let t=[e],n=e.lastIndexOf("-");for(;n>0;)e=e.slice(0,n),t.push(e),n=e.lastIndexOf("-");return t}getPluralFunc(e){if(!new RegExp("^\\s*nplurals\\s*=\\s*[0-9]+\\s*;\\s*plural\\s*=\\s*(?:\\s|[-\\?\\|&=!<>+*/%:;n0-9_()])+").test(e))throw new Error(r.strfmt('The plural form "%1" is not valid',e));return new Function("n","let plural, nplurals; "+e+" return { nplurals: nplurals, plural: (plural === true ? 1 : (plural ? plural : 0)) };")}removeContext(e){if(-1!==e.indexOf(this._contextDelimiter)){return e.split(this._contextDelimiter)[1]}return e}t(e,t,n,...i){if(!n.pluralForm)return this._stringsPrefix+r.strfmt(this.removeContext(e[0]),...i);let o;return n.pluralFunc?o=n.pluralFunc(t):(this._pluralFuncs[n.locale||""]||(this._pluralFuncs[n.locale||""]=this.getPluralFunc(this._pluralForms[n.locale||""])),o=this._pluralFuncs[n.locale||""](t)),(void 0===!o.plural||o.plural>o.nplurals||e.length<=o.plural)&&(o.plural=0),this._stringsPrefix+r.strfmt(this.removeContext(e[o.plural]),...[t].concat(i))}setMessages(e,t,n,r){r&&(this._pluralForms[t]=r),this._dictionary[e]||(this._dictionary[e]={}),this._dictionary[e][t]=n}}const i=new class{constructor(e){this._languageBundle=e}load(e){return this._languageBundle}locale(){return"en"}}(new class{__(e,...t){return this.gettext(e,...t)}_n(e,t,n,...r){return this.ngettext(e,t,n,...r)}_p(e,t,...n){return this.pgettext(e,t,...n)}_np(e,t,n,r,...i){return this.npgettext(e,t,n,r,...i)}gettext(e,...t){return r.strfmt(e,...t)}ngettext(e,t,n,...i){return r.strfmt(1==n?e:t,...[n].concat(i))}pgettext(e,t,...n){return r.strfmt(t,...n)}npgettext(e,t,n,r,...i){return this.ngettext(t,n,r,...i)}dcnpgettext(e,t,n,r,i,...o){return this.ngettext(n,r,i,...o)}});var o=n(263),a=n(17);n(48),n(84);new a.Token("@jupyterlab/translation:ITranslatorConnector");o.DataConnector;new a.Token("@jupyterlab/translation:ITranslator");n.d(t,"a",(function(){return i}))},function(e,t,n){"use strict";n.d(t,"a",(function(){return o}));var r=n(277),i=(n(0),n(167));function o(){return Object(r.a)()||i.a}},function(e,t,n){"use strict";n.d(t,"a",(function(){return o}));var r=n(0),i="undefined"!=typeof window?r.useLayoutEffect:r.useEffect;function o(e){var t=r.useRef(e);return i((function(){t.current=e})),r.useCallback((function(){return t.current.apply(void 0,arguments)}),[])}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(1471);t.TypeStyle=r.TypeStyle;var i=n(1474);t.types=i;var o=n(874);t.extend=o.extend,t.classes=o.classes,t.media=o.media;var a=new r.TypeStyle({autoGenerateTag:!0});t.setStylesTarget=a.setStylesTarget,t.cssRaw=a.cssRaw,t.cssRule=a.cssRule,t.forceRenderStyles=a.forceRenderStyles,t.fontFace=a.fontFace,t.getStyles=a.getStyles,t.keyframes=a.keyframes,t.reinit=a.reinit,t.style=a.style,t.stylesheet=a.stylesheet,t.createTypeStyle=function(e){var t=new r.TypeStyle({autoGenerateTag:!1});return e&&t.setStylesTarget(e),t}},function(e,t,n){"use strict";var r=n(39),i=n(42),o=n(82),a=n(27),s=n(10),l=s.strTranslate,u=n(96),c=n(385),d=n(41),h=n(56),f=n(134),p=n(987),g=n(72),v=g.ONEMAXYEAR,m=g.ONEAVGYEAR,y=g.ONEMINYEAR,b=g.ONEMAXQUARTER,C=g.ONEAVGQUARTER,A=g.ONEMINQUARTER,_=g.ONEMAXMONTH,I=g.ONEAVGMONTH,x=g.ONEMINMONTH,w=g.ONEWEEK,M=g.ONEDAY,S=M/2,T=g.ONEHOUR,E=g.ONEMIN,O=g.ONESEC,k=g.MINUS_SIGN,L=g.BADNUM,D=n(97),P=D.MID_SHIFT,N=D.CAP_SHIFT,j=D.LINE_SPACING,R=D.OPPOSITE_SIDE,z=e.exports={};z.setConvert=n(300);var F=n(545),B=n(83),H=B.idSort,U=B.isLinked;z.id2name=B.id2name,z.name2id=B.name2id,z.cleanId=B.cleanId,z.list=B.list,z.listIds=B.listIds,z.getFromId=B.getFromId,z.getFromTrace=B.getFromTrace;var V=n(386);z.getAutoRange=V.getAutoRange,z.findExtremes=V.findExtremes;function Y(e){var t=1e-4*(e[1]-e[0]);return[e[0]-t,e[1]+t]}z.coerceRef=function(e,t,n,r,i,o){var a=r.charAt(r.length-1),l=n._fullLayout._subplots[a+"axis"],u=r+"ref",c={};return i||(i=l[0]||("string"==typeof o?o:o[0])),o||(o=i),l=l.concat(l.map((function(e){return e+" domain"}))),c[u]={valType:"enumerated",values:l.concat(o?"string"==typeof o?[o]:o:[]),dflt:i},s.coerce(e,t,c,u)},z.getRefType=function(e){return void 0===e?e:"paper"===e?"paper":"pixel"===e?"pixel":/( domain)$/.test(e)?"domain":"range"},z.coercePosition=function(e,t,n,r,i,o){var a,l;if("range"!==z.getRefType(r))a=s.ensureNumber,l=n(i,o);else{var u=z.getFromId(t,r);l=n(i,o=u.fraction2r(o)),a=u.cleanPos}e[i]=a(l)},z.cleanPosition=function(e,t,n){return("paper"===n||"pixel"===n?s.ensureNumber:z.getFromId(t,n).cleanPos)(e)},z.redrawComponents=function(e,t){t=t||z.listIds(e);var n=e._fullLayout;function r(r,i,o,s){for(var l=a.getComponentMethod(r,i),u={},c=0;c<t.length;c++)for(var d=n[z.id2name(t[c])][o],h=0;h<d.length;h++){var f=d[h];if(!u[f]&&(l(e,f),u[f]=1,s))return}}r("annotations","drawOne","_annIndices"),r("shapes","drawOne","_shapeIndices"),r("images","draw","_imgIndices",!0)};var W=z.getDataConversions=function(e,t,n,r){var i,o="x"===n||"y"===n||"z"===n?n:r;if(Array.isArray(o)){if(i={type:F(r,void 0,{autotypenumbers:e._fullLayout.autotypenumbers}),_categories:[]},z.setConvert(i),"category"===i.type)for(var a=0;a<r.length;a++)i.d2c(r[a])}else i=z.getFromTrace(e,t,o);return i?{d2c:i.d2c,c2d:i.c2d}:"ids"===o?{d2c:X,c2d:X}:{d2c:G,c2d:G}};function G(e){return+e}function X(e){return String(e)}function q(e){return+e.substring(1)}z.getDataToCoordFunc=function(e,t,n,r){return W(e,t,n,r).d2c},z.counterLetter=function(e){var t=e.charAt(0);return"x"===t?"y":"y"===t?"x":void 0},z.minDtick=function(e,t,n,r){-1===["log","category","multicategory"].indexOf(e.type)&&r?void 0===e._minDtick?(e._minDtick=t,e._forceTick0=n):e._minDtick&&((e._minDtick/t+1e-6)%1<2e-6&&((n-e._forceTick0)/t%1+1.000001)%1<2e-6?(e._minDtick=t,e._forceTick0=n):((t/e._minDtick+1e-6)%1>2e-6||((n-e._forceTick0)/e._minDtick%1+1.000001)%1>2e-6)&&(e._minDtick=0)):e._minDtick=0},z.saveRangeInitial=function(e,t){for(var n=z.list(e,"",!0),r=!1,i=0;i<n.length;i++){var o=n[i],a=void 0===o._rangeInitial,s=a||!(o.range[0]===o._rangeInitial[0]&&o.range[1]===o._rangeInitial[1]);(a&&!1===o.autorange||t&&s)&&(o._rangeInitial=o.range.slice(),r=!0)}return r},z.saveShowSpikeInitial=function(e,t){for(var n=z.list(e,"",!0),r=!1,i="on",o=0;o<n.length;o++){var a=n[o],s=void 0===a._showSpikeInitial,l=s||!(a.showspikes===a._showspikes);(s||t&&l)&&(a._showSpikeInitial=a.showspikes,r=!0),"on"!==i||a.showspikes||(i="off")}return e._fullLayout._cartesianSpikesEnabled=i,r},z.autoBin=function(e,t,n,r,o,a){var l,u=s.aggNums(Math.min,null,e),c=s.aggNums(Math.max,null,e);if("category"===t.type||"multicategory"===t.type)return{start:u-.5,end:c+.5,size:Math.max(1,Math.round(a)||1),_dataSpan:c-u};if(o||(o=t.calendar),l="log"===t.type?{type:"linear",range:[u,c]}:{type:t.type,range:s.simpleMap([u,c],t.c2r,0,o),calendar:o},z.setConvert(l),a=a&&p.dtick(a,l.type))l.dtick=a,l.tick0=p.tick0(void 0,l.type,o);else{var d;if(n)d=(c-u)/n;else{var h=s.distinctVals(e),f=Math.pow(10,Math.floor(Math.log(h.minDiff)/Math.LN10)),g=f*s.roundUp(h.minDiff/f,[.9,1.9,4.9,9.9],!0);d=Math.max(g,2*s.stdev(e)/Math.pow(e.length,r?.25:.4)),i(d)||(d=1)}z.autoTicks(l,d)}var v,m=l.dtick,y=z.tickIncrement(z.tickFirst(l),m,"reverse",o);if("number"==typeof m)v=(y=function(e,t,n,r,o){var a=0,s=0,l=0,u=0;function c(t){return(1+100*(t-e)/n.dtick)%100<2}for(var d=0;d<t.length;d++)t[d]%1==0?l++:i(t[d])||u++,c(t[d])&&a++,c(t[d]+n.dtick/2)&&s++;var h=t.length-u;if(l===h&&"date"!==n.type)n.dtick<1?e=r-.5*n.dtick:(e-=.5)+n.dtick<r&&(e+=n.dtick);else if(s<.1*h&&(a>.3*h||c(r)||c(o))){var f=n.dtick/2;e+=e+f<r?f:-f}return e}(y,e,l,u,c))+(1+Math.floor((c-y)/m))*m;else for("M"===l.dtick.charAt(0)&&(y=function(e,t,n,r,i){var o=s.findExactDates(t,i);if(o.exactDays>.8){var a=Number(n.substr(1));o.exactYears>.8&&a%12==0?e=z.tickIncrement(e,"M6","reverse")+1.5*M:o.exactMonths>.8?e=z.tickIncrement(e,"M1","reverse")+15.5*M:e-=S;var l=z.tickIncrement(e,n);if(l<=r)return l}return e}(y,e,m,u,o)),v=y,0;v<=c;)v=z.tickIncrement(v,m,!1,o);return{start:t.c2r(y,0,o),end:t.c2r(v,0,o),size:m,_dataSpan:c-u}},z.prepTicks=function(e,t){var n=s.simpleMap(e.range,e.r2l,void 0,void 0,t);if(e._dtickInit=e.dtick,e._tick0Init=e.tick0,"auto"===e.tickmode||!e.dtick){var r,o=e.nticks;o||("category"===e.type||"multicategory"===e.type?(r=e.tickfont?1.2*(e.tickfont.size||12):15,o=e._length/r):(r="y"===e._id.charAt(0)?40:80,o=s.constrain(e._length/r,4,9)+1),"radialaxis"===e._name&&(o*=2)),"array"===e.tickmode&&(o*=100),e._roughDTick=Math.abs(n[1]-n[0])/o,z.autoTicks(e,e._roughDTick),e._minDtick>0&&e.dtick<2*e._minDtick&&(e.dtick=e._minDtick,e.tick0=e.l2r(e._forceTick0))}"period"===e.ticklabelmode&&function(e){var t;function n(){return!(i(e.dtick)||"M"!==e.dtick.charAt(0))}var r=n(),o=z.getTickFormat(e);if(o){var a=e._dtickInit!==e.dtick;/%[fLQsSMX]/.test(o)||(/%[HI]/.test(o)?(t=T,a&&!r&&e.dtick<T&&(e.dtick=T)):/%p/.test(o)?(t=S,a&&!r&&e.dtick<S&&(e.dtick=S)):/%[Aadejuwx]/.test(o)?(t=M,a&&!r&&e.dtick<M&&(e.dtick=M)):/%[UVW]/.test(o)?(t=w,a&&!r&&e.dtick<w&&(e.dtick=w)):/%[Bbm]/.test(o)?(t=I,a&&(r?q(e.dtick)<1:e.dtick<x)&&(e.dtick="M1")):/%[q]/.test(o)?(t=C,a&&(r?q(e.dtick)<3:e.dtick<A)&&(e.dtick="M3")):/%[Yy]/.test(o)&&(t=m,a&&(r?q(e.dtick)<12:e.dtick<y)&&(e.dtick="M12")))}(r=n())&&e.tick0===e._dowTick0&&(e.tick0=e._rawTick0);e._definedDelta=t}(e),e.tick0||(e.tick0="date"===e.type?"2000-01-01":0),"date"===e.type&&e.dtick<.1&&(e.dtick=.1),re(e)},z.calcTicks=function(e,t){z.prepTicks(e,t);var n=s.simpleMap(e.range,e.r2l,void 0,void 0,t);if("array"===e.tickmode)return function(e){var t=e.tickvals,n=e.ticktext,r=new Array(t.length),i=Y(s.simpleMap(e.range,e.r2l)),o=Math.min(i[0],i[1]),a=Math.max(i[0],i[1]),l=0;Array.isArray(n)||(n=[]);var u="category"===e.type?e.d2l_noadd:e.d2l;"log"===e.type&&"L"!==String(e.dtick).charAt(0)&&(e.dtick="L"+Math.pow(10,Math.floor(Math.min(e.range[0],e.range[1]))-1));for(var c=0;c<t.length;c++){var d=u(t[c]);d>o&&d<a&&(void 0===n[c]?r[l]=z.tickText(e,d):r[l]=ie(e,d,String(n[c])),l++)}l<t.length&&r.splice(l,t.length-l);e.rangebreaks&&(r=r.filter((function(t){return e.maskBreaks(t.x)!==L})));return r}(e);var r=Y(n),o=r[0],a=r[1],l=n[1]<n[0],u=Math.min(n[0],n[1]),c=Math.max(n[0],n[1]),d="log"===e.type&&!(i(e.dtick)||"L"===e.dtick.charAt(0)),h="period"===e.ticklabelmode;if(e._tmin=z.tickFirst(e,t),e._tmin<o!==l)return[];"category"!==e.type&&"multicategory"!==e.type||(a=l?Math.max(-.5,a):Math.min(e._categories.length-.5,a));var f=e._tmin;e.rangebreaks&&e._tick0Init!==e.tick0&&(f=_e(f,e),l||(f=z.tickIncrement(f,e.dtick,!l,e.calendar))),h&&(f=z.tickIncrement(f,e.dtick,!l,e.calendar));for(var p,g=Math.max(1e3,e._length||0),E=[],O=null;l?f>=a:f<=a;f=z.tickIncrement(f,e.dtick,l,e.calendar)){if(e.rangebreaks&&!l){if(f<o)continue;if(e.maskBreaks(f)===L&&_e(f,e)>=c)break}if(E.length>g||f===O)break;O=f;var k=!1;d&&f!==(0|f)&&(k=!0),E.push({minor:k,value:f})}if(h&&function(e,t,n){for(var r=0;r<e.length;r++){var i=e[r].value,o=r,a=r+1;r<e.length-1?(o=r,a=r+1):r>0?(o=r-1,a=r):(o=r,a=r);var s,l=e[o].value,u=e[a].value,c=Math.abs(u-l),d=n||c,h=0;d>=y?h=c>=y&&c<=v?c:m:n===C&&d>=A?h=c>=A&&c<=b?c:C:d>=x?h=c>=x&&c<=_?c:I:n===w&&d>=w?h=w:d>=M?h=M:n===S&&d>=S?h=S:n===T&&d>=T&&(h=T),h>=c&&(h=c,s=!0);var f=i+h;if(t.rangebreaks&&h>0){for(var p=0,g=0;g<84;g++){var E=(g+.5)/84;t.maskBreaks(i*(1-E)+E*f)!==L&&p++}(h*=p/84)||(e[r].drop=!0),s&&c>w&&(h=c)}(h>0||0===r)&&(e[r].periodX=i+h/2)}}(E,e,e._definedDelta),e.rangebreaks){var D="y"===e._id.charAt(0),P=1;"auto"===e.tickmode&&(P=e.tickfont?e.tickfont.size:12);var N=NaN;for(p=E.length-1;p>-1;p--)if(E[p].drop)E.splice(p,1);else{E[p].value=_e(E[p].value,e);var j=e.c2p(E[p].value);(D?N>j-P:N<j+P)?E.splice(l?p+1:p,1):N=j}}Ae(e)&&360===Math.abs(n[1]-n[0])&&E.pop(),e._tmax=(E[E.length-1]||{}).value,e._prevDateHead="",e._inCalcTicks=!0;var R,F,B=[];for(p=0;p<E.length;p++){var H=E[p].minor,U=E[p].value;R=z.tickText(e,U,!1,H),void 0!==(F=E[p].periodX)&&(R.periodX=F,(F>c||F<u)&&(F>c&&(R.periodX=c),F<u&&(R.periodX=u),R.text=" ",e._prevDateHead="")),B.push(R)}return e._inCalcTicks=!1,B};var Z=[2,5,10],Q=[1,2,3,6,12],K=[1,2,5,10,15,30],J=[1,2,3,7,14],$=[-.046,0,.301,.477,.602,.699,.778,.845,.903,.954,1],ee=[-.301,0,.301,.699,1],te=[15,30,45,90,180];function ne(e,t,n){return t*s.roundUp(e/t,n)}function re(e){var t=e.dtick;if(e._tickexponent=0,i(t)||"string"==typeof t||(t=1),"category"!==e.type&&"multicategory"!==e.type||(e._tickround=null),"date"===e.type){var n=e.r2l(e.tick0),r=e.l2r(n).replace(/(^-|i)/g,""),o=r.length;if("M"===String(t).charAt(0))o>10||"01-01"!==r.substr(5)?e._tickround="d":e._tickround=+t.substr(1)%12==0?"y":"m";else if(t>=M&&o<=10||t>=15*M)e._tickround="d";else if(t>=E&&o<=16||t>=T)e._tickround="M";else if(t>=O&&o<=19||t>=E)e._tickround="S";else{var a=e.l2r(n+t).replace(/^-/,"").length;e._tickround=Math.max(o,a)-20,e._tickround<0&&(e._tickround=4)}}else if(i(t)||"L"===t.charAt(0)){var s=e.range.map(e.r2d||Number);i(t)||(t=Number(t.substr(1))),e._tickround=2-Math.floor(Math.log(t)/Math.LN10+.01);var l=Math.max(Math.abs(s[0]),Math.abs(s[1])),u=Math.floor(Math.log(l)/Math.LN10+.01),c=void 0===e.minexponent?3:e.minexponent;Math.abs(u)>c&&(ae(e.exponentformat)&&!se(u)?e._tickexponent=3*Math.round((u-1)/3):e._tickexponent=u)}else e._tickround=null}function ie(e,t,n){var r=e.tickfont||{};return{x:t,dx:0,dy:0,text:n||"",fontSize:r.size,font:r.family,fontColor:r.color}}z.autoTicks=function(e,t){var n;function r(e){return Math.pow(e,Math.floor(Math.log(t)/Math.LN10))}if("date"===e.type){e.tick0=s.dateTick0(e.calendar,0);var o=2*t;if(o>m)t/=m,n=r(10),e.dtick="M"+12*ne(t,n,Z);else if(o>I)t/=I,e.dtick="M"+ne(t,1,Q);else if(o>M){e.dtick=ne(t,M,e._hasDayOfWeekBreaks?[1,2,7,14]:J);var a=z.getTickFormat(e),l="period"===e.ticklabelmode;l&&(e._rawTick0=e.tick0),/%[uVW]/.test(a)?e.tick0=s.dateTick0(e.calendar,2):e.tick0=s.dateTick0(e.calendar,1),l&&(e._dowTick0=e.tick0)}else o>T?e.dtick=ne(t,T,Q):o>E?e.dtick=ne(t,E,K):o>O?e.dtick=ne(t,O,K):(n=r(10),e.dtick=ne(t,n,Z))}else if("log"===e.type){e.tick0=0;var u=s.simpleMap(e.range,e.r2l);if(t>.7)e.dtick=Math.ceil(t);else if(Math.abs(u[1]-u[0])<1){var c=1.5*Math.abs((u[1]-u[0])/t);t=Math.abs(Math.pow(10,u[1])-Math.pow(10,u[0]))/c,n=r(10),e.dtick="L"+ne(t,n,Z)}else e.dtick=t>.3?"D2":"D1"}else"category"===e.type||"multicategory"===e.type?(e.tick0=0,e.dtick=Math.ceil(Math.max(t,1))):Ae(e)?(e.tick0=0,n=1,e.dtick=ne(t,n,te)):(e.tick0=0,n=r(10),e.dtick=ne(t,n,Z));if(0===e.dtick&&(e.dtick=1),!i(e.dtick)&&"string"!=typeof e.dtick){var d=e.dtick;throw e.dtick=1,"ax.dtick error: "+String(d)}},z.tickIncrement=function(e,t,n,o){var a=n?-1:1;if(i(t))return s.increment(e,a*t);var l=t.charAt(0),u=a*Number(t.substr(1));if("M"===l)return s.incrementMonth(e,u,o);if("L"===l)return Math.log(Math.pow(10,e)+u)/Math.LN10;if("D"===l){var c="D2"===t?ee:$,d=e+.01*a,h=s.roundUp(s.mod(d,1),c,n);return Math.floor(d)+Math.log(r.round(Math.pow(10,h),1))/Math.LN10}throw"unrecognized dtick "+String(t)},z.tickFirst=function(e,t){var n=e.r2l||Number,o=s.simpleMap(e.range,n,void 0,void 0,t),a=o[1]<o[0],l=a?Math.floor:Math.ceil,u=Y(o)[0],c=e.dtick,d=n(e.tick0);if(i(c)){var h=l((u-d)/c)*c+d;return"category"!==e.type&&"multicategory"!==e.type||(h=s.constrain(h,0,e._categories.length-1)),h}var f=c.charAt(0),p=Number(c.substr(1));if("M"===f){for(var g,v,m,y=0,b=d;y<10;){if(((g=z.tickIncrement(b,c,a,e.calendar))-u)*(b-u)<=0)return a?Math.min(b,g):Math.max(b,g);v=(u-(b+g)/2)/(g-b),m=f+(Math.abs(Math.round(v))||1)*p,b=z.tickIncrement(b,m,v<0?!a:a,e.calendar),y++}return s.error("tickFirst did not converge",e),b}if("L"===f)return Math.log(l((Math.pow(10,u)-d)/p)*p+d)/Math.LN10;if("D"===f){var C="D2"===c?ee:$,A=s.roundUp(s.mod(u,1),C,a);return Math.floor(u)+Math.log(r.round(Math.pow(10,A),1))/Math.LN10}throw"unrecognized dtick "+String(c)},z.tickText=function(e,t,n,r){var o,a=ie(e,t),l="array"===e.tickmode,u=n||l,c=e.type,d="category"===c?e.d2l_noadd:e.d2l;if(l&&Array.isArray(e.ticktext)){var h=s.simpleMap(e.range,e.r2l),f=(Math.abs(h[1]-h[0])-(e._lBreaks||0))/1e4;for(o=0;o<e.ticktext.length&&!(Math.abs(t-d(e.tickvals[o]))<f);o++);if(o<e.ticktext.length)return a.text=String(e.ticktext[o]),a}function p(r){if(void 0===r)return!0;if(n)return"none"===r;var i={first:e._tmin,last:e._tmax}[r];return"all"!==r&&t!==i}var g=n?"never":"none"!==e.exponentformat&&p(e.showexponent)?"hide":"";if("date"===c?function(e,t,n,r){var o=e._tickround,a=n&&e.hoverformat||z.getTickFormat(e);r&&(o=i(o)?4:{y:"m",m:"d",d:"M",M:"S",S:4}[o]);var l,u=s.formatDate(t.x,a,o,e._dateFormat,e.calendar,e._extraFormat),c=u.indexOf("\n");-1!==c&&(l=u.substr(c+1),u=u.substr(0,c));r&&("00:00:00"===u||"00:00"===u?(u=l,l=""):8===u.length&&(u=u.replace(/:00$/,"")));if(l)if(n)"d"===o?u+=", "+l:u=l+(u?", "+u:"");else if(e._inCalcTicks&&e._prevDateHead===l){var d=-1!==(e.ticklabelposition||"").indexOf("inside"),h=e._realSide||e.side;(!d&&"top"===h||d&&"bottom"===h)&&(u+="<br> ")}else e._prevDateHead=l,u+="<br>"+l;t.text=u}(e,a,n,u):"log"===c?function(e,t,n,r,o){var a=e.dtick,l=t.x,u=e.tickformat,c="string"==typeof a&&a.charAt(0);"never"===o&&(o="");r&&"L"!==c&&(a="L3",c="L");if(u||"L"===c)t.text=le(Math.pow(10,l),e,o,r);else if(i(a)||"D"===c&&s.mod(l+.01,1)<.1){var d=Math.round(l),h=Math.abs(d),f=e.exponentformat;"power"===f||ae(f)&&se(d)?(t.text=0===d?1:1===d?"10":"10<sup>"+(d>1?"":k)+h+"</sup>",t.fontSize*=1.25):("e"===f||"E"===f)&&h>2?t.text="1"+f+(d>0?"+":k)+h:(t.text=le(Math.pow(10,l),e,"","fakehover"),"D1"===a&&"y"===e._id.charAt(0)&&(t.dy-=t.fontSize/6))}else{if("D"!==c)throw"unrecognized dtick "+String(a);t.text=String(Math.round(Math.pow(10,s.mod(l,1)))),t.fontSize*=.75}if("D1"===e.dtick){var p=String(t.text).charAt(0);"0"!==p&&"1"!==p||("y"===e._id.charAt(0)?t.dx-=t.fontSize/4:(t.dy+=t.fontSize/2,t.dx+=(e.range[1]>e.range[0]?1:-1)*t.fontSize*(l<0?.5:.25)))}}(e,a,0,u,g):"category"===c?function(e,t){var n=e._categories[Math.round(t.x)];void 0===n&&(n="");t.text=String(n)}(e,a):"multicategory"===c?function(e,t,n){var r=Math.round(t.x),i=e._categories[r]||[],o=void 0===i[1]?"":String(i[1]),a=void 0===i[0]?"":String(i[0]);n?t.text=a+" - "+o:(t.text=o,t.text2=a)}(e,a,n):Ae(e)?function(e,t,n,r,i){if("radians"!==e.thetaunit||n)t.text=le(t.x,e,i,r);else{var o=t.x/180;if(0===o)t.text="0";else{var a=function(e){function t(e,t){return Math.abs(e-t)<=1e-6}var n=function(e){for(var n=1;!t(Math.round(e*n)/n,e);)n*=10;return n}(e),r=e*n,i=Math.abs(function e(n,r){return t(r,0)?n:e(r,n%r)}(r,n));return[Math.round(r/i),Math.round(n/i)]}(o);if(a[1]>=100)t.text=le(s.deg2rad(t.x),e,i,r);else{var l=t.x<0;1===a[1]?1===a[0]?t.text="π":t.text=a[0]+"π":t.text=["<sup>",a[0],"</sup>","⁄","<sub>",a[1],"</sub>","π"].join(""),l&&(t.text=k+t.text)}}}}(e,a,n,u,g):function(e,t,n,r,i){"never"===i?i="":"all"===e.showexponent&&Math.abs(t.x/e.dtick)<1e-6&&(i="hide");t.text=le(t.x,e,i,r)}(e,a,0,u,g),r||(e.tickprefix&&!p(e.showtickprefix)&&(a.text=e.tickprefix+a.text),e.ticksuffix&&!p(e.showticksuffix)&&(a.text+=e.ticksuffix)),"boundaries"===e.tickson||e.showdividers){var v=function(t){var n=e.l2p(t);return n>=0&&n<=e._length?t:null};a.xbnd=[v(a.x-.5),v(a.x+e.dtick-.5)]}return a},z.hoverLabelText=function(e,t,n){if(n!==L&&n!==t)return z.hoverLabelText(e,t)+" - "+z.hoverLabelText(e,n);var r="log"===e.type&&t<=0,i=z.tickText(e,e.c2l(r?-t:t),"hover").text;return r?0===t?"0":k+i:i};var oe=["f","p","n","μ","m","","k","M","G","T"];function ae(e){return"SI"===e||"B"===e}function se(e){return e>14||e<-15}function le(e,t,n,r){var o=e<0,a=t._tickround,l=n||t.exponentformat||"B",u=t._tickexponent,c=z.getTickFormat(t),d=t.separatethousands;if(r){var h={exponentformat:l,minexponent:t.minexponent,dtick:"none"===t.showexponent?t.dtick:i(e)&&Math.abs(e)||1,range:"none"===t.showexponent?t.range.map(t.r2d):[0,e||1]};re(h),a=(Number(h._tickround)||0)+4,u=h._tickexponent,t.hoverformat&&(c=t.hoverformat)}if(c)return t._numFormat(c)(e).replace(/-/g,k);var f,p=Math.pow(10,-a)/2;if("none"===l&&(u=0),(e=Math.abs(e))<p)e="0",o=!1;else{if(e+=p,u&&(e*=Math.pow(10,-u),a+=u),0===a)e=String(Math.floor(e));else if(a<0){e=(e=String(Math.round(e))).substr(0,e.length+a);for(var g=a;g<0;g++)e+="0"}else{var v=(e=String(e)).indexOf(".")+1;v&&(e=e.substr(0,v+a).replace(/\.?0+$/,""))}e=s.numSeparate(e,t._separators,d)}u&&"hide"!==l&&(ae(l)&&se(u)&&(l="power"),f=u<0?k+-u:"power"!==l?"+"+u:String(u),"e"===l||"E"===l?e+=l+f:"power"===l?e+="×10<sup>"+f+"</sup>":"B"===l&&9===u?e+="B":ae(l)&&(e+=oe[u/3+5]));return o?k+e:e}function ue(e,t){for(var n=[],r={},i=0;i<t.length;i++){var o=t[i];r[o.text2]?r[o.text2].push(o.x):r[o.text2]=[o.x]}for(var a in r)n.push(ie(e,s.interp(r[a],.5),a));return n}function ce(e){return void 0!==e.periodX?e.periodX:e.x}function de(e){return[e.text,e.x,e.axInfo,e.font,e.fontSize,e.fontColor].join("_")}function he(e){var t=e.title.font.size,n=(e.title.text.match(u.BR_TAG_ALL)||[]).length;return e.title.hasOwnProperty("standoff")?n?t*(N+n*j):t*N:n?t*(n+1)*j:t}function fe(e,t){var n=e.l2p(t);return n>1&&n<e._length-1}function pe(e){var t=r.select(e),n=t.select(".text-math-group");return n.empty()?t.select("text"):n}function ge(e){return e._id+".automargin"}function ve(e){return ge(e)+".mirror"}function me(e){return e._id+".rangeslider"}function ye(e,t){for(var n=0;n<t.length;n++)-1===e.indexOf(t[n])&&e.push(t[n])}function be(e,t,n){var r,i,o=[],a=[],l=e.layout;for(r=0;r<t.length;r++)o.push(z.getFromId(e,t[r]));for(r=0;r<n.length;r++)a.push(z.getFromId(e,n[r]));var u=Object.keys(f),c=["anchor","domain","overlaying","position","side","tickangle","editType"],d=["linear","log"];for(r=0;r<u.length;r++){var h=u[r],p=o[0][h],g=a[0][h],v=!0,m=!1,y=!1;if("_"!==h.charAt(0)&&"function"!=typeof p&&-1===c.indexOf(h)){for(i=1;i<o.length&&v;i++){var b=o[i][h];"type"===h&&-1!==d.indexOf(p)&&-1!==d.indexOf(b)&&p!==b?m=!0:b!==p&&(v=!1)}for(i=1;i<a.length&&v;i++){var C=a[i][h];"type"===h&&-1!==d.indexOf(g)&&-1!==d.indexOf(C)&&g!==C?y=!0:a[i][h]!==g&&(v=!1)}v&&(m&&(l[o[0]._name].type="linear"),y&&(l[a[0]._name].type="linear"),Ce(l,h,o,a,e._fullLayout._dfltTitle))}}for(r=0;r<e._fullLayout.annotations.length;r++){var A=e._fullLayout.annotations[r];-1!==t.indexOf(A.xref)&&-1!==n.indexOf(A.yref)&&s.swapAttrs(l.annotations[r],["?"])}}function Ce(e,t,n,r,i){var o,a=s.nestedProperty,l=a(e[n[0]._name],t).get(),u=a(e[r[0]._name],t).get();for("title"===t&&(l&&l.text===i.x&&(l.text=i.y),u&&u.text===i.y&&(u.text=i.x)),o=0;o<n.length;o++)a(e,n[o]._name+"."+t).set(u);for(o=0;o<r.length;o++)a(e,r[o]._name+"."+t).set(l)}function Ae(e){return"angularaxis"===e._id}function _e(e,t){for(var n=t._rangebreaks.length,r=0;r<n;r++){var i=t._rangebreaks[r];if(e>=i.min&&e<i.max)return i.max}return e}z.getTickFormat=function(e){var t,n,r,i,o,a,s,l;function u(e){return"string"!=typeof e?e:Number(e.replace("M",""))*I}function c(e,t){var n=["L","D"];if(typeof e==typeof t){if("number"==typeof e)return e-t;var r=n.indexOf(e.charAt(0)),i=n.indexOf(t.charAt(0));return r===i?Number(e.replace(/(L|D)/g,""))-Number(t.replace(/(L|D)/g,"")):r-i}return"number"==typeof e?1:-1}function d(e,t){var n=null===t[0],r=null===t[1],i=c(e,t[0])>=0,o=c(e,t[1])<=0;return(n||i)&&(r||o)}if(e.tickformatstops&&e.tickformatstops.length>0)switch(e.type){case"date":case"linear":for(t=0;t<e.tickformatstops.length;t++)if((r=e.tickformatstops[t]).enabled&&(i=e.dtick,o=r.dtickrange,a=void 0,s=void 0,l=void 0,a=u||function(e){return e},s=o[0],l=o[1],(!s&&"number"!=typeof s||a(s)<=a(i))&&(!l&&"number"!=typeof l||a(l)>=a(i)))){n=r;break}break;case"log":for(t=0;t<e.tickformatstops.length;t++)if((r=e.tickformatstops[t]).enabled&&d(e.dtick,r.dtickrange)){n=r;break}}return n?n.value:e.tickformat},z.getSubplots=function(e,t){var n=e._fullLayout._subplots,r=n.cartesian.concat(n.gl2d||[]),i=t?z.findSubplotsWithAxis(r,t):r;return i.sort((function(e,t){var n=e.substr(1).split("y"),r=t.substr(1).split("y");return n[0]===r[0]?+n[1]-+r[1]:+n[0]-+r[0]})),i},z.findSubplotsWithAxis=function(e,t){for(var n=new RegExp("x"===t._id.charAt(0)?"^"+t._id+"y":t._id+"$"),r=[],i=0;i<e.length;i++){var o=e[i];n.test(o)&&r.push(o)}return r},z.makeClipPaths=function(e){var t=e._fullLayout;if(!t._hasOnlyLargeSploms){var n,i,o={_offset:0,_length:t.width,_id:""},a={_offset:0,_length:t.height,_id:""},s=z.list(e,"x",!0),l=z.list(e,"y",!0),u=[];for(n=0;n<s.length;n++)for(u.push({x:s[n],y:a}),i=0;i<l.length;i++)0===n&&u.push({x:o,y:l[i]}),u.push({x:s[n],y:l[i]});var c=t._clips.selectAll(".axesclip").data(u,(function(e){return e.x._id+e.y._id}));c.enter().append("clipPath").classed("axesclip",!0).attr("id",(function(e){return"clip"+t._uid+e.x._id+e.y._id})).append("rect"),c.exit().remove(),c.each((function(e){r.select(this).select("rect").attr({x:e.x._offset||0,y:e.y._offset||0,width:e.x._length||1,height:e.y._length||1})}))}},z.draw=function(e,t,n){var r=e._fullLayout;"redraw"===t&&r._paper.selectAll("g.subplot").each((function(e){var t=e[0],n=r._plots[t];if(n){var i=n.xaxis,o=n.yaxis;n.xaxislayer.selectAll("."+i._id+"tick").remove(),n.yaxislayer.selectAll("."+o._id+"tick").remove(),n.xaxislayer.selectAll("."+i._id+"tick2").remove(),n.yaxislayer.selectAll("."+o._id+"tick2").remove(),n.xaxislayer.selectAll("."+i._id+"divider").remove(),n.yaxislayer.selectAll("."+o._id+"divider").remove(),n.gridlayer&&n.gridlayer.selectAll("path").remove(),n.zerolinelayer&&n.zerolinelayer.selectAll("path").remove(),r._infolayer.select(".g-"+i._id+"title").remove(),r._infolayer.select(".g-"+o._id+"title").remove()}}));var i=t&&"redraw"!==t?t:z.listIds(e);return s.syncOrAsync(i.map((function(t){return function(){if(t){var r=z.getFromId(e,t),i=z.drawOne(e,r,n);return r._r=r.range.slice(),r._rl=s.simpleMap(r._r,r.r2l),i}}})))},z.drawOne=function(e,t,n){var r,i,l;n=n||{},t.setScale();var u=e._fullLayout,f=t._id,p=f.charAt(0),g=z.counterLetter(f),v=u._plots[t._mainSubplot];if(v){var m=v[p+"axislayer"],y=t._mainLinePosition,b=t._mainMirrorPosition,C=t._vals=z.calcTicks(t),A=[t.mirror,y,b].join("_");for(r=0;r<C.length;r++)C[r].axInfo=A;t._selections={},t._tickAngles&&(t._prevTickAngles=t._tickAngles),t._tickAngles={},t._depth=null;var _={};if(t.visible){var I,x,w=z.makeTransTickFn(t),M=z.makeTransTickLabelFn(t),S="inside"===t.ticks,T="outside"===t.ticks;if("boundaries"===t.tickson){var E=function(e,t){var n,r=[],i=function(e,t){var n=e.xbnd[t];null!==n&&r.push(s.extendFlat({},e,{x:n}))};if(t.length){for(n=0;n<t.length;n++)i(t[n],0);i(t[n-1],1)}return r}(0,C);x=z.clipEnds(t,E),I=S?x:E}else x=z.clipEnds(t,C),I=S&&"period"!==t.ticklabelmode?x:C;var O=t._gridVals=x,k=function(e,t){var n,r,i=[],o=t.length&&t[t.length-1].x<t[0].x,a=function(e,t){var n=e.xbnd[t];null!==n&&i.push(s.extendFlat({},e,{x:n}))};if(e.showdividers&&t.length){for(n=0;n<t.length;n++){var l=t[n];l.text2!==r&&a(l,o?1:0),r=l.text2}a(t[n-1],o?0:1)}return i}(t,C);if(!u._hasOnlyLargeSploms){var L=t._subplotsWith,D={};for(r=0;r<L.length;r++){i=L[r];var P=(l=u._plots[i])[g+"axis"],N=P._mainAxis._id;if(!D[N]){D[N]=1;var F="x"===p?"M0,"+P._offset+"v"+P._length:"M"+P._offset+",0h"+P._length;z.drawGrid(e,t,{vals:O,counterAxis:P,layer:l.gridlayer.select("."+f),path:F,transFn:w}),z.drawZeroLine(e,t,{counterAxis:P,layer:l.zerolinelayer,path:F,transFn:w})}}}var B=z.getTickSigns(t),H=[];if(t.ticks){var U,V,Y,W=z.makeTickPath(t,y,B[2]);if(t._anchorAxis&&t.mirror&&!0!==t.mirror?(U=z.makeTickPath(t,b,B[3]),V=W+U):(U="",V=W),t.showdividers&&T&&"boundaries"===t.tickson){var G={};for(r=0;r<k.length;r++)G[k[r].x]=1;Y=function(e){return G[e.x]?U:V}}else Y=V;z.drawTicks(e,t,{vals:I,layer:m,path:Y,transFn:w}),"allticks"===t.mirror&&(H=Object.keys(t._linepositions||{}))}for(r=0;r<H.length;r++){i=H[r],l=u._plots[i];var X=t._linepositions[i]||[],q=z.makeTickPath(t,X[0],B[0])+z.makeTickPath(t,X[1],B[1]);z.drawTicks(e,t,{vals:I,layer:l[p+"axislayer"],path:q,transFn:w})}var Z=[];if(Z.push((function(){return z.drawLabels(e,t,{vals:C,layer:m,transFn:M,labelFns:z.makeLabelFns(t,y)})})),"multicategory"===t.type){var Q={x:2,y:10}[p];Z.push((function(){var n={x:"height",y:"width"}[p],r=J()[n]+Q+(t._tickAngles[f+"tick"]?t.tickfont.size*j:0);return z.drawLabels(e,t,{vals:ue(t,C),layer:m,cls:f+"tick2",repositionOnUpdate:!0,secondary:!0,transFn:w,labelFns:z.makeLabelFns(t,y+r*B[4])})})),Z.push((function(){return t._depth=B[4]*(J("tick2")[t.side]-y),function(e,t,n){var r=t._id+"divider",i=n.vals,o=n.layer.selectAll("path."+r).data(i,de);o.exit().remove(),o.enter().insert("path",":first-child").classed(r,1).classed("crisp",1).call(d.stroke,t.dividercolor).style("stroke-width",h.crispRound(e,t.dividerwidth,1)+"px"),o.attr("transform",n.transFn).attr("d",n.path)}(e,t,{vals:k,layer:m,path:z.makeTickPath(t,y,B[4],t._depth),transFn:w})}))}else t.title.hasOwnProperty("standoff")&&Z.push((function(){t._depth=B[4]*(J()[t.side]-y)}));var K=a.getComponentMethod("rangeslider","isVisible")(t);return Z.push((function(){var n,r,i,s,l=t.side.charAt(0),c=R[t.side].charAt(0),d=z.getPxPosition(e,t),h=T?t.ticklen:0;if((t.automargin||K)&&("multicategory"===t.type?n=J("tick2"):(n=J(),"x"===p&&"b"===l&&(t._depth=Math.max(n.width>0?n.bottom-d:0,h)))),t.automargin){r={x:0,y:0,r:0,l:0,t:0,b:0};var f=[0,1];if("x"===p){if("b"===l?r[l]=t._depth:(r[l]=t._depth=Math.max(n.width>0?d-n.top:0,h),f.reverse()),n.width>0){var v=n.right-(t._offset+t._length);v>0&&(r.xr=1,r.r=v);var m=t._offset-n.left;m>0&&(r.xl=0,r.l=m)}}else if("l"===l?r[l]=t._depth=Math.max(n.height>0?d-n.left:0,h):(r[l]=t._depth=Math.max(n.height>0?n.right-d:0,h),f.reverse()),n.height>0){var y=n.bottom-(t._offset+t._length);y>0&&(r.yb=0,r.b=y);var b=t._offset-n.top;b>0&&(r.yt=1,r.t=b)}r[g]="free"===t.anchor?t.position:t._anchorAxis.domain[f[0]],t.title.text!==u._dfltTitle[p]&&(r[l]+=he(t)+(t.title.standoff||0)),t.mirror&&"free"!==t.anchor&&((i={x:0,y:0,r:0,l:0,t:0,b:0})[c]=t.linewidth,t.mirror&&!0!==t.mirror&&(i[c]+=h),!0===t.mirror||"ticks"===t.mirror?i[g]=t._anchorAxis.domain[f[1]]:"all"!==t.mirror&&"allticks"!==t.mirror||(i[g]=[t._counterDomainMin,t._counterDomainMax][f[1]]))}K&&(s=a.getComponentMethod("rangeslider","autoMarginOpts")(e,t)),o.autoMargin(e,ge(t),r),o.autoMargin(e,ve(t),i),o.autoMargin(e,me(t),s)})),n.skipTitle||K&&"bottom"===t.side||Z.push((function(){return function(e,t){var n,r=e._fullLayout,i=t._id,o=i.charAt(0),a=t.title.font.size;if(t.title.hasOwnProperty("standoff"))n=t._depth+t.title.standoff+he(t);else{var s=-1!==(t.ticklabelposition||"").indexOf("inside");if("multicategory"===t.type)n=t._depth;else{var l=1.5*a;s&&(l=.5*a,"outside"===t.ticks&&(l+=t.ticklen)),n=10+l+(t.linewidth?t.linewidth-1:0)}s||(n+="x"===o?"top"===t.side?a*(t.showticklabels?1:0):a*(t.showticklabels?1.5:.5):"right"===t.side?a*(t.showticklabels?1:.5):a*(t.showticklabels?.5:0))}var u,d,f,p,g=z.getPxPosition(e,t);"x"===o?(d=t._offset+t._length/2,f="top"===t.side?g-n:g+n):(f=t._offset+t._length/2,d="right"===t.side?g+n:g-n,u={rotate:"-90",offset:0});if("multicategory"!==t.type){var v=t._selections[t._id+"tick"];if(p={selection:v,side:t.side},v&&v.node()&&v.node().parentNode){var m=h.getTranslate(v.node().parentNode);p.offsetLeft=m.x,p.offsetTop=m.y}t.title.hasOwnProperty("standoff")&&(p.pad=0)}return c.draw(e,i+"title",{propContainer:t,propName:t._name+".title.text",placeholder:r._dfltTitle[o],avoid:p,transform:u,attributes:{x:d,y:f,"text-anchor":"middle"}})}(e,t)})),s.syncOrAsync(Z)}}function J(e){var n=f+(e||"tick");return _[n]||(_[n]=function(e,t){var n,r,i,o;e._selections[t].size()?(n=1/0,r=-1/0,i=1/0,o=-1/0,e._selections[t].each((function(){var e=pe(this),t=h.bBox(e.node().parentNode);n=Math.min(n,t.top),r=Math.max(r,t.bottom),i=Math.min(i,t.left),o=Math.max(o,t.right)}))):(n=0,r=0,i=0,o=0);return{top:n,bottom:r,left:i,right:o,height:r-n,width:o-i}}(t,n)),_[n]}},z.getTickSigns=function(e){var t=e._id.charAt(0),n={x:"top",y:"right"}[t],r=e.side===n?1:-1,i=[-1,1,r,-r];return"inside"!==e.ticks==("x"===t)&&(i=i.map((function(e){return-e}))),e.side&&i.push({l:-1,t:-1,r:1,b:1}[e.side.charAt(0)]),i},z.makeTransTickFn=function(e){return"x"===e._id.charAt(0)?function(t){return l(e._offset+e.l2p(t.x),0)}:function(t){return l(0,e._offset+e.l2p(t.x))}},z.makeTransTickLabelFn=function(e){var t=function(e){var t=e.ticklabelposition||"",n=function(e){return-1!==t.indexOf(e)},r=n("top"),i=n("left"),o=n("right"),a=n("bottom"),s=n("inside"),l=a||i||r||o;if(!l&&!s)return[0,0];var u=e.side,c=l?(e.tickwidth||0)/2:0,d=3,h=e.tickfont?e.tickfont.size:12;(a||r)&&(c+=h*N,d+=(e.linewidth||0)/2);(i||o)&&(c+=(e.linewidth||0)/2,d+=3);s&&"top"===u&&(d-=h*(1-N));(i||r)&&(c=-c);"bottom"!==u&&"right"!==u||(d=-d);return[l?c:0,s?d:0]}(e),n=t[0],r=t[1];return"x"===e._id.charAt(0)?function(t){return l(n+e._offset+e.l2p(ce(t)),r)}:function(t){return l(r,n+e._offset+e.l2p(ce(t)))}},z.makeTickPath=function(e,t,n,r){r=void 0!==r?r:e.ticklen;var i=e._id.charAt(0),o=(e.linewidth||1)/2;return"x"===i?"M0,"+(t+o*n)+"v"+r*n:"M"+(t+o*n)+",0h"+r*n},z.makeLabelFns=function(e,t,n){var r=e.ticklabelposition||"",o=function(e){return-1!==r.indexOf(e)},a=o("top"),l=o("left"),u=o("right"),c=o("bottom")||l||a||u,d=o("inside"),h="inside"===r&&"inside"===e.ticks||!d&&"outside"===e.ticks&&"boundaries"!==e.tickson,f=0,p=0,g=h?e.ticklen:0;if(d?g*=-1:c&&(g=0),h&&(f+=g,n)){var v=s.deg2rad(n);f=g*Math.cos(v)+1,p=g*Math.sin(v)}e.showticklabels&&(h||e.showline)&&(f+=.2*e.tickfont.size);var m,y,b,C,A,_={labelStandoff:f+=(e.linewidth||1)/2*(d?-1:1),labelShift:p},I=0,x=e.side,w=e._id.charAt(0),M=e.tickangle;if("x"===w)C=(A=!d&&"bottom"===x||d&&"top"===x)?1:-1,d&&(C*=-1),m=p*C,y=t+f*C,b=A?1:-.2,90===Math.abs(M)&&(d?b+=P:b=-90===M&&"bottom"===x?N:90===M&&"top"===x?P:.5,I=P/2*(M/90)),_.xFn=function(e){return e.dx+m+I*e.fontSize},_.yFn=function(e){return e.dy+y+e.fontSize*b},_.anchorFn=function(e,t){if(c){if(l)return"end";if(u)return"start"}return i(t)&&0!==t&&180!==t?t*C<0!==d?"end":"start":"middle"},_.heightFn=function(t,n,r){return n<-60||n>60?-.5*r:"top"===e.side!==d?-r:0};else if("y"===w){if(C=(A=!d&&"left"===x||d&&"right"===x)?1:-1,d&&(C*=-1),m=f,y=p*C,b=0,d||90!==Math.abs(M)||(b=-90===M&&"left"===x||90===M&&"right"===x?N:.5),d){var S=i(M)?+M:0;if(0!==S){var T=s.deg2rad(S);I=Math.abs(Math.sin(T))*N*C,b=0}}_.xFn=function(e){return e.dx+t-(m+e.fontSize*b)*C+I*e.fontSize},_.yFn=function(e){return e.dy+y+e.fontSize*P},_.anchorFn=function(e,t){return i(t)&&90===Math.abs(t)?"middle":A?"end":"start"},_.heightFn=function(t,n,r){return"right"===e.side&&(n*=-1),n<-30?-r:n<30?-.5*r:0}}return _},z.drawTicks=function(e,t,n){n=n||{};var r=t._id+"tick",i=n.vals;"period"===t.ticklabelmode&&(i=i.slice()).shift();var o=n.layer.selectAll("path."+r).data(t.ticks?i:[],de);o.exit().remove(),o.enter().append("path").classed(r,1).classed("ticks",1).classed("crisp",!1!==n.crisp).call(d.stroke,t.tickcolor).style("stroke-width",h.crispRound(e,t.tickwidth,1)+"px").attr("d",n.path),o.attr("transform",n.transFn)},z.drawGrid=function(e,t,n){n=n||{};var r=t._id+"grid",i=n.vals,o=n.counterAxis;if(!1===t.showgrid)i=[];else if(o&&z.shouldShowZeroLine(e,t,o))for(var a="array"===t.tickmode,s=0;s<i.length;s++){var l=i[s].x;if(a?!l:Math.abs(l)<t.dtick/100){if(i=i.slice(0,s).concat(i.slice(s+1)),!a)break;s--}}var u=n.layer.selectAll("path."+r).data(i,de);u.exit().remove(),u.enter().append("path").classed(r,1).classed("crisp",!1!==n.crisp),t._gw=h.crispRound(e,t.gridwidth,1),u.attr("transform",n.transFn).attr("d",n.path).call(d.stroke,t.gridcolor||"#ddd").style("stroke-width",t._gw+"px"),"function"==typeof n.path&&u.attr("d",n.path)},z.drawZeroLine=function(e,t,n){n=n||n;var r=t._id+"zl",i=z.shouldShowZeroLine(e,t,n.counterAxis),o=n.layer.selectAll("path."+r).data(i?[{x:0,id:t._id}]:[]);o.exit().remove(),o.enter().append("path").classed(r,1).classed("zl",1).classed("crisp",!1!==n.crisp).each((function(){n.layer.selectAll("path").sort((function(e,t){return H(e.id,t.id)}))})),o.attr("transform",n.transFn).attr("d",n.path).call(d.stroke,t.zerolinecolor||d.defaultLine).style("stroke-width",h.crispRound(e,t.zerolinewidth,t._gw||1)+"px")},z.drawLabels=function(e,t,n){n=n||{};var o=e._fullLayout,a=t._id,c=a.charAt(0),d=n.cls||a+"tick",f=n.vals,p=n.labelFns,g=n.secondary?0:t.tickangle,v=(t._prevTickAngles||{})[d],m=n.layer.selectAll("g."+d).data(t.showticklabels?f:[],de),y=[];function b(e,o){var a=-1!==(t.ticklabelposition||"").indexOf("inside");e.each((function(e){var s=r.select(this),c=s.select(".text-math-group"),d=p.anchorFn(e,o),f=n.transFn.call(s.node(),e)+(i(o)&&0!=+o?" rotate("+o+","+p.xFn(e)+","+(p.yFn(e)-e.fontSize/2)+")":""),g=u.lineCount(s),v=j*e.fontSize,m=p.heightFn(e,i(o)?+o:0,(g-1)*v);if(m&&(f+=l(0,m)),c.empty()){var y=s.select("text");y.attr({transform:f,"text-anchor":d}),a&&(y.style({opacity:100}),t._hideOutOfRangeInsideTickLabels&&t._hideOutOfRangeInsideTickLabels())}else{var b=h.bBox(c.node()).width*{end:-.5,start:.5}[d];c.attr("transform",f+l(b,0))}}))}m.enter().append("g").classed(d,1).append("text").attr("text-anchor","middle").each((function(t){var n=r.select(this),i=e._promises.length;n.call(u.positionText,p.xFn(t),p.yFn(t)).call(h.font,t.font,t.fontSize,t.fontColor).text(t.text).call(u.convertToTspans,e),e._promises[i]?y.push(e._promises.pop().then((function(){b(n,g)}))):b(n,g)})),m.exit().remove(),n.repositionOnUpdate&&m.each((function(e){r.select(this).select("text").call(u.positionText,p.xFn(e),p.yFn(e))})),t._hideOutOfRangeInsideTickLabels=void 0,-1!==(t.ticklabelposition||"").indexOf("inside")&&(t._hideOutOfRangeInsideTickLabels=function(){var e=s.simpleMap(t.range,t.r2l),n=t.l2p(e[0]),i=t.l2p(e[1]),o=Math.min(n,i)+t._offset,a=Math.max(n,i)+t._offset,l="x"===t._id.charAt(0);m.each((function(e){var n=r.select(this);if(n.select(".text-math-group").empty()){var i=h.bBox(n.node()),s=!1;l?(i.right>a||i.left<o)&&(s=!0):(i.bottom>a||i.top+(t.tickangle?0:e.fontSize/4)<o)&&(s=!0),s&&n.select("text").style({opacity:0})}}))}),b(m,v+1?v:g);var C=null;t._selections&&(t._selections[d]=m);var A=[function(){return y.length&&Promise.all(y)}];t.automargin&&o._redrawFromAutoMarginCount&&90===v?(C=90,A.push((function(){b(m,v)}))):A.push((function(){if(b(m,g),f.length&&"x"===c&&!i(g)&&("log"!==t.type||"D"!==String(t.dtick).charAt(0))){C=0;var e,r=0,o=[];if(m.each((function(e){r=Math.max(r,e.fontSize);var n=t.l2p(e.x),i=pe(this),a=h.bBox(i.node());o.push({top:0,bottom:10,height:10,left:n-a.width/2,right:n+a.width/2+2,width:a.width+2})})),"boundaries"!==t.tickson&&!t.showdividers||n.secondary){var a=f.length,l=Math.abs((f[a-1].x-f[0].x)*t._m)/(a-1),u=t.ticklabelposition||"",d=function(e){return-1!==u.indexOf(e)},p=d("top"),v=d("left"),y=d("right"),A=d("bottom")||v||p||y?(t.tickwidth||0)+6:0,_=l<2.5*r||"multicategory"===t.type;for(e=0;e<o.length-1;e++)if(s.bBoxIntersect(o[e],o[e+1],A)){C=_?90:30;break}}else{var I=2;for(t.ticks&&(I+=t.tickwidth/2),e=0;e<o.length;e++){var x=f[e].xbnd,w=o[e];if(null!==x[0]&&w.left-t.l2p(x[0])<I||null!==x[1]&&t.l2p(x[1])-w.right<I){C=90;break}}}C&&b(m,C)}})),t._tickAngles&&A.push((function(){t._tickAngles[d]=null===C?i(g)?g:0:C}));var _=t._anchorAxis;_&&_.autorange&&-1!==(t.ticklabelposition||"").indexOf("inside")&&!U(o,t._id)&&(o._insideTickLabelsAutorange||(o._insideTickLabelsAutorange={}),o._insideTickLabelsAutorange[_._name+".autorange"]=_.autorange,A.push((function(){m.each((function(e,n){var r=pe(this);t._vals[n].bb=h.bBox(r.node())}))})));var I=s.syncOrAsync(A);return I&&I.then&&e._promises.push(I),I},z.getPxPosition=function(e,t){var n,r=e._fullLayout._size,i=t._id.charAt(0),o=t.side;return"free"!==t.anchor?n=t._anchorAxis:"x"===i?n={_offset:r.t+(1-(t.position||0))*r.h,_length:0}:"y"===i&&(n={_offset:r.l+(t.position||0)*r.w,_length:0}),"top"===o||"left"===o?n._offset:"bottom"===o||"right"===o?n._offset+n._length:void 0},z.shouldShowZeroLine=function(e,t,n){var r=s.simpleMap(t.range,t.r2l);return r[0]*r[1]<=0&&t.zeroline&&("linear"===t.type||"-"===t.type)&&!(t.rangebreaks&&t.maskBreaks(0)===L)&&(fe(t,0)||!function(e,t,n,r){var i=n._mainAxis;if(!i)return;var o=e._fullLayout,a=t._id.charAt(0),s=z.counterLetter(t._id),l=t._offset+(Math.abs(r[0])<Math.abs(r[1])==("x"===a)?0:t._length);function u(e){if(!e.showline||!e.linewidth)return!1;var n=Math.max((e.linewidth+t.zerolinewidth)/2,1);function r(e){return"number"==typeof e&&Math.abs(e-l)<n}if(r(e._mainLinePosition)||r(e._mainMirrorPosition))return!0;var i=e._linepositions||{};for(var o in i)if(r(i[o][0])||r(i[o][1]))return!0}var c=o._plots[n._mainSubplot];if(!(c.mainplotinfo||c).overlays.length)return u(n);for(var d=z.list(e,s),h=0;h<d.length;h++){var f=d[h];if(f._mainAxis===i&&u(f))return!0}}(e,t,n,r)||function(e,t){for(var n=e._fullData,r=t._mainSubplot,i=t._id.charAt(0),o=0;o<n.length;o++){var s=n[o];if(!0===s.visible&&s.xaxis+s.yaxis===r){if(a.traceIs(s,"bar-like")&&s.orientation==={x:"h",y:"v"}[i])return!0;if(s.fill&&s.fill.charAt(s.fill.length-1)===i)return!0}}return!1}(e,t))},z.clipEnds=function(e,t){return t.filter((function(t){return fe(e,t.x)}))},z.allowAutoMargin=function(e){for(var t=z.list(e,"",!0),n=0;n<t.length;n++){var r=t[n];r.automargin&&(o.allowAutoMargin(e,ge(r)),r.mirror&&o.allowAutoMargin(e,ve(r))),a.getComponentMethod("rangeslider","isVisible")(r)&&o.allowAutoMargin(e,me(r))}},z.swap=function(e,t){for(var n=function(e,t){var n,r,i=[];for(n=0;n<t.length;n++){var o=[],a=e._fullData[t[n]].xaxis,s=e._fullData[t[n]].yaxis;if(a&&s){for(r=0;r<i.length;r++)-1===i[r].x.indexOf(a)&&-1===i[r].y.indexOf(s)||o.push(r);if(o.length){var l,u=i[o[0]];if(o.length>1)for(r=1;r<o.length;r++)l=i[o[r]],ye(u.x,l.x),ye(u.y,l.y);ye(u.x,[a]),ye(u.y,[s])}else i.push({x:[a],y:[s]})}}return i}(e,t),r=0;r<n.length;r++)be(e,n[r].x,n[r].y)}},function(e,t,n){"use strict";var r=n(39),i=n(42),o=n(123),a=n(27),s=n(41),l=n(195),u=n(10),c=u.strTranslate,d=n(96),h=n(196),f=n(97).LINE_SPACING,p=n(525).DESELECTDIM,g=n(125),v=n(989),m=n(162).appendArrayPointValue,y=e.exports={};y.font=function(e,t,n,r){u.isPlainObject(t)&&(r=t.color,n=t.size,t=t.family),t&&e.style("font-family",t),n+1&&e.style("font-size",n+"px"),r&&e.call(s.fill,r)},y.setPosition=function(e,t,n){e.attr("x",t).attr("y",n)},y.setSize=function(e,t,n){e.attr("width",t).attr("height",n)},y.setRect=function(e,t,n,r,i){e.call(y.setPosition,t,n).call(y.setSize,r,i)},y.translatePoint=function(e,t,n,r){var o=n.c2p(e.x),a=r.c2p(e.y);return!!(i(o)&&i(a)&&t.node())&&("text"===t.node().nodeName?t.attr("x",o).attr("y",a):t.attr("transform",c(o,a)),!0)},y.translatePoints=function(e,t,n){e.each((function(e){var i=r.select(this);y.translatePoint(e,i,t,n)}))},y.hideOutsideRangePoint=function(e,t,n,r,i,o){t.attr("display",n.isPtWithinRange(e,i)&&r.isPtWithinRange(e,o)?null:"none")},y.hideOutsideRangePoints=function(e,t){if(t._hasClipOnAxisFalse){var n=t.xaxis,i=t.yaxis;e.each((function(t){var o=t[0].trace,s=o.xcalendar,l=o.ycalendar,u=a.traceIs(o,"bar-like")?".bartext":".point,.textpoint";e.selectAll(u).each((function(e){y.hideOutsideRangePoint(e,r.select(this),n,i,s,l)}))}))}},y.crispRound=function(e,t,n){return t&&i(t)?e._context.staticPlot?t:t<1?1:Math.round(t):n||0},y.singleLineStyle=function(e,t,n,r,i){t.style("fill","none");var o=(((e||[])[0]||{}).trace||{}).line||{},a=n||o.width||0,l=i||o.dash||"";s.stroke(t,r||o.color),y.dashLine(t,l,a)},y.lineGroupStyle=function(e,t,n,i){e.style("fill","none").each((function(e){var o=(((e||[])[0]||{}).trace||{}).line||{},a=t||o.width||0,l=i||o.dash||"";r.select(this).call(s.stroke,n||o.color).call(y.dashLine,l,a)}))},y.dashLine=function(e,t,n){n=+n||0,t=y.dashStyle(t,n),e.style({"stroke-dasharray":t,"stroke-width":n+"px"})},y.dashStyle=function(e,t){t=+t||1;var n=Math.max(t,3);return"solid"===e?e="":"dot"===e?e=n+"px,"+n+"px":"dash"===e?e=3*n+"px,"+3*n+"px":"longdash"===e?e=5*n+"px,"+5*n+"px":"dashdot"===e?e=3*n+"px,"+n+"px,"+n+"px,"+n+"px":"longdashdot"===e&&(e=5*n+"px,"+2*n+"px,"+n+"px,"+2*n+"px"),e},y.singleFillStyle=function(e){var t=(((r.select(e.node()).data()[0]||[])[0]||{}).trace||{}).fillcolor;t&&e.call(s.fill,t)},y.fillGroupStyle=function(e){e.style("stroke-width",0).each((function(e){var t=r.select(this);e[0].trace&&t.call(s.fill,e[0].trace.fillcolor)}))};var b=n(1681);y.symbolNames=[],y.symbolFuncs=[],y.symbolNeedLines={},y.symbolNoDot={},y.symbolNoFill={},y.symbolList=[],Object.keys(b).forEach((function(e){var t=b[e],n=t.n;y.symbolList.push(n,String(n),e,n+100,String(n+100),e+"-open"),y.symbolNames[n]=e,y.symbolFuncs[n]=t.f,t.needLine&&(y.symbolNeedLines[n]=!0),t.noDot?y.symbolNoDot[n]=!0:y.symbolList.push(n+200,String(n+200),e+"-dot",n+300,String(n+300),e+"-open-dot"),t.noFill&&(y.symbolNoFill[n]=!0)}));var C=y.symbolNames.length;function A(e,t){var n=e%100;return y.symbolFuncs[n](t)+(e>=200?"M0,0.5L0.5,0L0,-0.5L-0.5,0Z":"")}y.symbolNumber=function(e){if(i(e))e=+e;else if("string"==typeof e){var t=0;e.indexOf("-open")>0&&(t=100,e=e.replace("-open","")),e.indexOf("-dot")>0&&(t+=200,e=e.replace("-dot","")),(e=y.symbolNames.indexOf(e))>=0&&(e+=t)}return e%100>=C||e>=400?0:Math.floor(Math.max(e,0))};var _={x1:1,x2:0,y1:0,y2:0},I={x1:0,x2:0,y1:1,y2:0},x=r.format("~.1f"),w={radial:{node:"radialGradient"},radialreversed:{node:"radialGradient",reversed:!0},horizontal:{node:"linearGradient",attrs:_},horizontalreversed:{node:"linearGradient",attrs:_,reversed:!0},vertical:{node:"linearGradient",attrs:I},verticalreversed:{node:"linearGradient",attrs:I,reversed:!0}};y.gradient=function(e,t,n,i,a,l){for(var c=a.length,d=w[i],h=new Array(c),f=0;f<c;f++)d.reversed?h[c-1-f]=[x(100*(1-a[f][0])),a[f][1]]:h[f]=[x(100*a[f][0]),a[f][1]];var p=t._fullLayout,g="g"+p._uid+"-"+n,v=p._defs.select(".gradients").selectAll("#"+g).data([i+h.join(";")],u.identity);v.exit().remove(),v.enter().append(d.node).each((function(){var e=r.select(this);d.attrs&&e.attr(d.attrs),e.attr("id",g);var t=e.selectAll("stop").data(h);t.exit().remove(),t.enter().append("stop"),t.each((function(e){var t=o(e[1]);r.select(this).attr({offset:e[0]+"%","stop-color":s.tinyRGB(t),"stop-opacity":t.getAlpha()})}))})),e.style(l,P(g,t)).style(l+"-opacity",null);var m=function(e){return"."+e.attr("class").replace(/\s/g,".")},y=m(r.select(e.node().parentNode))+">"+m(e);p._gradientUrlQueryParts[y]=1},y.initGradients=function(e){var t=e._fullLayout;u.ensureSingle(t._defs,"g","gradients").selectAll("linearGradient,radialGradient").remove(),t._gradientUrlQueryParts={}},y.pointStyle=function(e,t,n){if(e.size()){var i=y.makePointStyleFns(t);e.each((function(e){y.singlePointStyle(e,r.select(this),t,i,n)}))}},y.singlePointStyle=function(e,t,n,r,i){var o=n.marker,a=o.line;if(t.style("opacity",r.selectedOpacityFn?r.selectedOpacityFn(e):void 0===e.mo?o.opacity:e.mo),r.ms2mrc){var l;l="various"===e.ms||"various"===o.size?3:r.ms2mrc(e.ms),e.mrc=l,r.selectedSizeFn&&(l=e.mrc=r.selectedSizeFn(e));var c=y.symbolNumber(e.mx||o.symbol)||0;e.om=c%200>=100,t.attr("d",A(c,l))}var d,h,f,p=!1;if(e.so)f=a.outlierwidth,h=a.outliercolor,d=o.outliercolor;else{var g=(a||{}).width;f=(e.mlw+1||g+1||(e.trace?(e.trace.marker.line||{}).width:0)+1)-1||0,h="mlc"in e?e.mlcc=r.lineScale(e.mlc):u.isArrayOrTypedArray(a.color)?s.defaultLine:a.color,u.isArrayOrTypedArray(o.color)&&(d=s.defaultLine,p=!0),d="mc"in e?e.mcc=r.markerScale(e.mc):o.color||"rgba(0,0,0,0)",r.selectedColorFn&&(d=r.selectedColorFn(e))}if(e.om)t.call(s.stroke,d).style({"stroke-width":(f||1)+"px",fill:"none"});else{t.style("stroke-width",(e.isBlank?0:f)+"px");var v=o.gradient,m=e.mgt;if(m?p=!0:m=v&&v.type,Array.isArray(m)&&(m=m[0],w[m]||(m=0)),m&&"none"!==m){var b=e.mgc;b?p=!0:b=v.color;var C=n.uid;p&&(C+="-"+e.i),y.gradient(t,i,C,m,[[0,b],[1,d]],"fill")}else s.fill(t,d);f&&s.stroke(t,h)}},y.makePointStyleFns=function(e){var t={},n=e.marker;return t.markerScale=y.tryColorscale(n,""),t.lineScale=y.tryColorscale(n,"line"),a.traceIs(e,"symbols")&&(t.ms2mrc=g.isBubble(e)?v(e):function(){return(n.size||6)/2}),e.selectedpoints&&u.extendFlat(t,y.makeSelectedPointStyleFns(e)),t},y.makeSelectedPointStyleFns=function(e){var t={},n=e.selected||{},r=e.unselected||{},i=e.marker||{},o=n.marker||{},s=r.marker||{},l=i.opacity,c=o.opacity,d=s.opacity,h=void 0!==c,f=void 0!==d;(u.isArrayOrTypedArray(l)||h||f)&&(t.selectedOpacityFn=function(e){var t=void 0===e.mo?i.opacity:e.mo;return e.selected?h?c:t:f?d:p*t});var g=i.color,v=o.color,m=s.color;(v||m)&&(t.selectedColorFn=function(e){var t=e.mcc||g;return e.selected?v||t:m||t});var y=i.size,b=o.size,C=s.size,A=void 0!==b,_=void 0!==C;return a.traceIs(e,"symbols")&&(A||_)&&(t.selectedSizeFn=function(e){var t=e.mrc||y/2;return e.selected?A?b/2:t:_?C/2:t}),t},y.makeSelectedTextStyleFns=function(e){var t={},n=e.selected||{},r=e.unselected||{},i=e.textfont||{},o=n.textfont||{},a=r.textfont||{},l=i.color,u=o.color,c=a.color;return t.selectedTextColorFn=function(e){var t=e.tc||l;return e.selected?u||t:c||(u?t:s.addOpacity(t,p))},t},y.selectedPointStyle=function(e,t){if(e.size()&&t.selectedpoints){var n=y.makeSelectedPointStyleFns(t),i=t.marker||{},o=[];n.selectedOpacityFn&&o.push((function(e,t){e.style("opacity",n.selectedOpacityFn(t))})),n.selectedColorFn&&o.push((function(e,t){s.fill(e,n.selectedColorFn(t))})),n.selectedSizeFn&&o.push((function(e,t){var r=t.mx||i.symbol||0,o=n.selectedSizeFn(t);e.attr("d",A(y.symbolNumber(r),o)),t.mrc2=o})),o.length&&e.each((function(e){for(var t=r.select(this),n=0;n<o.length;n++)o[n](t,e)}))}},y.tryColorscale=function(e,t){var n=t?u.nestedProperty(e,t).get():e;if(n){var r=n.color;if((n.colorscale||n._colorAx)&&u.isArrayOrTypedArray(r))return l.makeColorScaleFuncFromTrace(n)}return u.identity};var M={start:1,end:-1,middle:0,bottom:1,top:-1};function S(e,t,n,i){var o=r.select(e.node().parentNode),a=-1!==t.indexOf("top")?"top":-1!==t.indexOf("bottom")?"bottom":"middle",s=-1!==t.indexOf("left")?"end":-1!==t.indexOf("right")?"start":"middle",l=i?i/.8+1:0,u=(d.lineCount(e)-1)*f+1,h=M[s]*l,p=.75*n+M[a]*l+(M[a]-1)*u*n/2;e.attr("text-anchor",s),o.attr("transform",c(h,p))}function T(e,t){var n=e.ts||t.textfont.size;return i(n)&&n>0?n:0}y.textPointStyle=function(e,t,n){if(e.size()){var i;if(t.selectedpoints){var o=y.makeSelectedTextStyleFns(t);i=o.selectedTextColorFn}var a=t.texttemplate,s=n._fullLayout;e.each((function(e){var o=r.select(this),l=a?u.extractOption(e,t,"txt","texttemplate"):u.extractOption(e,t,"tx","text");if(l||0===l){if(a){var c=t._module.formatLabels?t._module.formatLabels(e,t,s):{},h={};m(h,t,e.i);var f=t._meta||{};l=u.texttemplateString(l,c,s._d3locale,h,e,f)}var p=e.tp||t.textposition,g=T(e,t),v=i?i(e):e.tc||t.textfont.color;o.call(y.font,e.tf||t.textfont.family,g,v).text(l).call(d.convertToTspans,n).call(S,p,g,e.mrc)}else o.remove()}))}},y.selectedTextStyle=function(e,t){if(e.size()&&t.selectedpoints){var n=y.makeSelectedTextStyleFns(t);e.each((function(e){var i=r.select(this),o=n.selectedTextColorFn(e),a=e.tp||t.textposition,l=T(e,t);s.fill(i,o),S(i,a,l,e.mrc2||e.mrc)}))}};function E(e,t,n,i){var o=e[0]-t[0],a=e[1]-t[1],s=n[0]-t[0],l=n[1]-t[1],u=Math.pow(o*o+a*a,.25),c=Math.pow(s*s+l*l,.25),d=(c*c*o-u*u*s)*i,h=(c*c*a-u*u*l)*i,f=3*c*(u+c),p=3*u*(u+c);return[[r.round(t[0]+(f&&d/f),2),r.round(t[1]+(f&&h/f),2)],[r.round(t[0]-(p&&d/p),2),r.round(t[1]-(p&&h/p),2)]]}y.smoothopen=function(e,t){if(e.length<3)return"M"+e.join("L");var n,r="M"+e[0],i=[];for(n=1;n<e.length-1;n++)i.push(E(e[n-1],e[n],e[n+1],t));for(r+="Q"+i[0][0]+" "+e[1],n=2;n<e.length-1;n++)r+="C"+i[n-2][1]+" "+i[n-1][0]+" "+e[n];return r+="Q"+i[e.length-3][1]+" "+e[e.length-1]},y.smoothclosed=function(e,t){if(e.length<3)return"M"+e.join("L")+"Z";var n,r="M"+e[0],i=e.length-1,o=[E(e[i],e[0],e[1],t)];for(n=1;n<i;n++)o.push(E(e[n-1],e[n],e[n+1],t));for(o.push(E(e[i-1],e[i],e[0],t)),n=1;n<=i;n++)r+="C"+o[n-1][1]+" "+o[n][0]+" "+e[n];return r+="C"+o[i][1]+" "+o[0][0]+" "+e[0]+"Z"};var O={hv:function(e,t){return"H"+r.round(t[0],2)+"V"+r.round(t[1],2)},vh:function(e,t){return"V"+r.round(t[1],2)+"H"+r.round(t[0],2)},hvh:function(e,t){return"H"+r.round((e[0]+t[0])/2,2)+"V"+r.round(t[1],2)+"H"+r.round(t[0],2)},vhv:function(e,t){return"V"+r.round((e[1]+t[1])/2,2)+"H"+r.round(t[0],2)+"V"+r.round(t[1],2)}},k=function(e,t){return"L"+r.round(t[0],2)+","+r.round(t[1],2)};y.steps=function(e){var t=O[e]||k;return function(e){for(var n="M"+r.round(e[0][0],2)+","+r.round(e[0][1],2),i=1;i<e.length;i++)n+=t(e[i-1],e[i]);return n}},y.makeTester=function(){var e=u.ensureSingleById(r.select("body"),"svg","js-plotly-tester",(function(e){e.attr(h.svgAttrs).style({position:"absolute",left:"-10000px",top:"-10000px",width:"9000px",height:"9000px","z-index":"1"})})),t=u.ensureSingle(e,"path","js-reference-point",(function(e){e.attr("d","M0,0H1V1H0Z").style({"stroke-width":0,fill:"black"})}));y.tester=e,y.testref=t},y.savedBBoxes={};var L=0;function D(e){var t=e.getAttribute("data-unformatted");if(null!==t)return t+e.getAttribute("data-math")+e.getAttribute("text-anchor")+e.getAttribute("style")}function P(e,t){if(!e)return null;var n=t._context;return"url('"+(n._exportedPlot?"":n._baseUrl||"")+"#"+e+"')"}y.bBox=function(e,t,n){var i,o,a;if(n||(n=D(e)),n){if(i=y.savedBBoxes[n])return u.extendFlat({},i)}else if(1===e.childNodes.length){var s=e.childNodes[0];if(n=D(s)){var l=+s.getAttribute("x")||0,c=+s.getAttribute("y")||0,h=s.getAttribute("transform");if(!h){var f=y.bBox(s,!1,n);return l&&(f.left+=l,f.right+=l),c&&(f.top+=c,f.bottom+=c),f}if(n+="~"+l+"~"+c+"~"+h,i=y.savedBBoxes[n])return u.extendFlat({},i)}}t?o=e:(a=y.tester.node(),o=e.cloneNode(!0),a.appendChild(o)),r.select(o).attr("transform",null).call(d.positionText,0,0);var p=o.getBoundingClientRect(),g=y.testref.node().getBoundingClientRect();t||a.removeChild(o);var v={height:p.height,width:p.width,left:p.left-g.left,top:p.top-g.top,right:p.right-g.left,bottom:p.bottom-g.top};return L>=1e4&&(y.savedBBoxes={},L=0),n&&(y.savedBBoxes[n]=v),L++,u.extendFlat({},v)},y.setClipUrl=function(e,t,n){e.attr("clip-path",P(t,n))},y.getTranslate=function(e){var t=(e[e.attr?"attr":"getAttribute"]("transform")||"").replace(/.*\btranslate\((-?\d*\.?\d*)[^-\d]*(-?\d*\.?\d*)[^\d].*/,(function(e,t,n){return[t,n].join(" ")})).split(" ");return{x:+t[0]||0,y:+t[1]||0}},y.setTranslate=function(e,t,n){var r=e.attr?"attr":"getAttribute",i=e.attr?"attr":"setAttribute",o=e[r]("transform")||"";return t=t||0,n=n||0,o=o.replace(/(\btranslate\(.*?\);?)/,"").trim(),o=(o+=c(t,n)).trim(),e[i]("transform",o),o},y.getScale=function(e){var t=(e[e.attr?"attr":"getAttribute"]("transform")||"").replace(/.*\bscale\((\d*\.?\d*)[^\d]*(\d*\.?\d*)[^\d].*/,(function(e,t,n){return[t,n].join(" ")})).split(" ");return{x:+t[0]||1,y:+t[1]||1}},y.setScale=function(e,t,n){var r=e.attr?"attr":"getAttribute",i=e.attr?"attr":"setAttribute",o=e[r]("transform")||"";return t=t||1,n=n||1,o=o.replace(/(\bscale\(.*?\);?)/,"").trim(),o=(o+="scale("+t+","+n+")").trim(),e[i]("transform",o),o};var N=/\s*sc.*/;y.setPointGroupScale=function(e,t,n){if(t=t||1,n=n||1,e){var r=1===t&&1===n?"":"scale("+t+","+n+")";e.each((function(){var e=(this.getAttribute("transform")||"").replace(N,"");e=(e+=r).trim(),this.setAttribute("transform",e)}))}};var j=/translate\([^)]*\)\s*$/;y.setTextPointsScale=function(e,t,n){e&&e.each((function(){var e,i=r.select(this),o=i.select("text");if(o.node()){var a=parseFloat(o.attr("x")||0),s=parseFloat(o.attr("y")||0),l=(i.attr("transform")||"").match(j);e=1===t&&1===n?[]:[c(a,s),"scale("+t+","+n+")",c(-a,-s)],l&&e.push(l),i.attr("transform",e.join(""))}}))}},function(e,t,n){"use strict";n.d(t,"a",(function(){return s}));var r=n(1),i=n(0),o=n.n(i),a=n(1309);function s(e,t){var n=function(t,n){return o.a.createElement(a.a,Object(r.a)({ref:n},t),e)};return n.muiName=a.a.muiName,o.a.memo(o.a.forwardRef(n))}},function(e,t,n){"use strict";n.d(t,"b",(function(){return g})),n.d(t,"c",(function(){return v})),n.d(t,"a",(function(){return m}));var r,i=n(6),o=n(17),a=n(14),s=n(18),l=n(2348),u=n(23),c=n(1307),d=n(0),h=n(203),f=n(144),p=n(434);function g(e={}){return new m(e).launch()}function v(e,t,n=[m.okButton({label:"Dismiss"})]){console.warn("Showing error:",t);const i="string"==typeof t?t:t.message,o=e+"----"+i,a=r.errorMessagePromiseCache.get(o);if(a)return a;{const t=g({title:e,body:i,buttons:n}).then(()=>{r.errorMessagePromiseCache.delete(o)},e=>{throw r.errorMessagePromiseCache.delete(o),e});return r.errorMessagePromiseCache.set(o,t),t}}class m extends s.Widget{constructor(e={}){super(),this._focusNodeSelector="",this.addClass("jp-Dialog");const t=r.handleOptions(e),n=t.renderer;this._host=t.host,this._defaultButton=t.defaultButton,this._buttons=t.buttons,this._buttonNodes=Object(i.toArray)(Object(i.map)(this._buttons,e=>n.createButtonNode(e)));const o=this.layout=new s.PanelLayout,a=new s.Panel;a.addClass("jp-Dialog-content"),o.addWidget(a),this._body=t.body;const l=n.createHeader(t.title,()=>this.reject(),e),u=n.createBody(t.body),c=n.createFooter(this._buttonNodes);a.addWidget(l),a.addWidget(u),a.addWidget(c),this._primary=this._buttonNodes[this._defaultButton],this._focusNodeSelector=e.focusNodeSelector,m.tracker.add(this)}dispose(){const e=this._promise;e&&(this._promise=null,e.reject(void 0),i.ArrayExt.removeFirstOf(r.launchQueue,e.promise)),super.dispose()}launch(){if(this._promise)return this._promise.promise;const e=this._promise=new o.PromiseDelegate,t=Promise.all(r.launchQueue);return r.launchQueue.push(this._promise.promise),t.then(()=>this._promise?(s.Widget.attach(this,this._host),e.promise):Promise.resolve({button:m.cancelButton(),value:null}))}resolve(e){this._promise&&(void 0===e&&(e=this._defaultButton),this._resolve(this._buttons[e]))}reject(){this._promise&&this._resolve(m.cancelButton())}handleEvent(e){switch(e.type){case"keydown":this._evtKeydown(e);break;case"click":this._evtClick(e);break;case"focus":this._evtFocus(e);break;case"contextmenu":e.preventDefault(),e.stopPropagation()}}onAfterAttach(e){const t=this.node;if(t.addEventListener("keydown",this,!0),t.addEventListener("contextmenu",this,!0),t.addEventListener("click",this,!0),document.addEventListener("focus",this,!0),this._first=r.findFirstFocusable(this.node),this._original=document.activeElement,this._focusNodeSelector){const e=this.node.querySelector(".jp-Dialog-body"),t=null==e?void 0:e.querySelector(this._focusNodeSelector);t&&(this._primary=t)}this._primary.focus()}onAfterDetach(e){const t=this.node;t.removeEventListener("keydown",this,!0),t.removeEventListener("contextmenu",this,!0),t.removeEventListener("click",this,!0),document.removeEventListener("focus",this,!0),this._original.focus()}onCloseRequest(e){this._promise&&this.reject(),super.onCloseRequest(e)}_evtClick(e){if(!this.node.getElementsByClassName("jp-Dialog-content")[0].contains(e.target))return e.stopPropagation(),e.preventDefault(),void this.reject();for(const t of this._buttonNodes)if(t.contains(e.target)){const e=this._buttonNodes.indexOf(t);this.resolve(e)}}_evtKeydown(e){switch(e.keyCode){case 27:e.stopPropagation(),e.preventDefault(),this.reject();break;case 9:{const t=this._buttonNodes[this._buttons.length-1];document.activeElement!==t||e.shiftKey||(e.stopPropagation(),e.preventDefault(),this._first.focus());break}case 13:e.stopPropagation(),e.preventDefault(),this.resolve()}}_evtFocus(e){const t=e.target;this.node.contains(t)||(e.stopPropagation(),this._buttonNodes[this._defaultButton].focus())}_resolve(e){const t=this._promise;if(!t)return void this.dispose();this._promise=null,i.ArrayExt.removeFirstOf(r.launchQueue,t.promise);const n=this._body;let o=null;e.accept&&n instanceof s.Widget&&"function"==typeof n.getValue&&(o=n.getValue()),this.dispose(),t.resolve({button:e,value:o})}}!function(e){function t(e){e.accept=!1!==e.accept;const t=e.accept?"OK":"Cancel";return{label:e.label||t,iconClass:e.iconClass||"",iconLabel:e.iconLabel||"",caption:e.caption||"",className:e.className||"",accept:e.accept,actions:e.actions||[],displayType:e.displayType||"default"}}e.createButton=t,e.cancelButton=function(e={}){return e.accept=!1,t(e)},e.okButton=function(e={}){return e.accept=!0,t(e)},e.warnButton=function(e={}){return e.displayType="warn",t(e)},e.flush=function(){e.tracker.forEach(e=>{e.dispose()})};class n{createHeader(e,t=(()=>{}),n={}){let r;const i=e=>{0===e.button&&(e.preventDefault(),t())},o=e=>{const{key:n}=e;"Enter"!==n&&" "!==n||t()};return r="string"==typeof e?f.a.create(d.createElement(d.Fragment,null,e,n.hasClose&&d.createElement(l.a,{className:"jp-Dialog-close-button",onMouseDown:i,onKeyDown:o,title:"Cancel",minimal:!0},d.createElement(u.a.resolveReact,{icon:c.f,iconClass:"jp-Icon",className:"jp-ToolbarButtonComponent-icon",tag:"span"})))):f.a.create(e),r.addClass("jp-Dialog-header"),h.a.styleNode(r.node),r}createBody(e){let t;return"string"==typeof e?(t=new s.Widget({node:document.createElement("span")}),t.node.textContent=e):e instanceof s.Widget?t=e:(t=f.a.create(e),a.c.sendMessage(t,s.Widget.Msg.UpdateRequest)),t.addClass("jp-Dialog-body"),h.a.styleNode(t.node),t}createFooter(e){const t=new s.Widget;return t.addClass("jp-Dialog-footer"),Object(i.each)(e,e=>{t.node.appendChild(e)}),h.a.styleNode(t.node),t}createButtonNode(e){const t=document.createElement("button");return t.className=this.createItemClass(e),t.appendChild(this.renderIcon(e)),t.appendChild(this.renderLabel(e)),t}createItemClass(e){let t="jp-Dialog-button";e.accept?t+=" jp-mod-accept":t+=" jp-mod-reject","warn"===e.displayType&&(t+=" jp-mod-warn");const n=e.className;return n&&(t+=" "+n),t}renderIcon(e){const t=document.createElement("div");return t.className=this.createIconClass(e),t.appendChild(document.createTextNode(e.iconLabel)),t}createIconClass(e){const t="jp-Dialog-buttonIcon",n=e.iconClass;return n?`${t} ${n}`:t}renderLabel(e){const t=document.createElement("div");return t.className="jp-Dialog-buttonLabel",t.title=e.caption,t.appendChild(document.createTextNode(e.label)),t}}e.Renderer=n,e.defaultRenderer=new n,e.tracker=new p.a({namespace:"@jupyterlab/apputils:Dialog"})}(m||(m={})),function(e){e.launchQueue=[],e.errorMessagePromiseCache=new Map,e.handleOptions=function(e={}){const t=e.buttons||[m.cancelButton(),m.okButton()];let n=t.length-1;return void 0!==e.defaultButton&&(n=e.defaultButton),{title:e.title||"",body:e.body||"",host:e.host||document.body,buttons:t,defaultButton:n,renderer:e.renderer||m.defaultRenderer,focusNodeSelector:e.focusNodeSelector||"",hasClose:e.hasClose||!1}},e.findFirstFocusable=function(e){const t=["input","select","a[href]","textarea","button","[tabindex]"].join(",");return e.querySelectorAll(t)[0]}}(r||(r={}))},function(e,t,n){"use strict";function r(e){return e&&e.ownerDocument||document}n.d(t,"a",(function(){return r}))},function(e,t,n){"use strict";n.d(t,"a",(function(){return r})),n.d(t,"c",(function(){return i})),n.d(t,"d",(function(){return o})),n.d(t,"b",(function(){return a}));var r={BOTTOM:"bottom",BOTTOM_LEFT:"bottom-left",BOTTOM_RIGHT:"bottom-right",LEFT:"left",LEFT_BOTTOM:"left-bottom",LEFT_TOP:"left-top",RIGHT:"right",RIGHT_BOTTOM:"right-bottom",RIGHT_TOP:"right-top",TOP:"top",TOP_LEFT:"top-left",TOP_RIGHT:"top-right"};function i(e){return e===r.TOP||e===r.TOP_LEFT||e===r.TOP_RIGHT||e===r.BOTTOM||e===r.BOTTOM_LEFT||e===r.BOTTOM_RIGHT}function o(e){return e===r.LEFT||e===r.LEFT_TOP||e===r.LEFT_BOTTOM||e===r.RIGHT||e===r.RIGHT_TOP||e===r.RIGHT_BOTTOM}function a(e){return e===r.TOP||e===r.TOP_LEFT||e===r.TOP_RIGHT?r.TOP:e===r.BOTTOM||e===r.BOTTOM_LEFT||e===r.BOTTOM_RIGHT?r.BOTTOM:e===r.LEFT||e===r.LEFT_TOP||e===r.LEFT_BOTTOM?r.LEFT:r.RIGHT}},function(e,t){var n;n=function(){return this}();try{n=n||new Function("return this")()}catch(e){"object"==typeof window&&(n=window)}e.exports=n},function(e,t,n){"use strict";n.r(t);var r=n(187);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";var r=n(292),i=Array.isArray;function o(e,t,n,a){var s,l,u,c,d,h,f=e[0],p=e.length;if(2===p&&i(f)&&i(e[1])&&0===f.length){if(function(e,t){var n,r;for(n=0;n<e.length;n++){if(null!==(r=e[n])&&"object"==typeof r)return!1;void 0!==r&&(t[n]=r)}return!0}(e[1],f))return f;f.splice(0,f.length)}for(var g=1;g<p;g++)for(l in s=e[g])u=f[l],c=s[l],a&&i(c)?f[l]=c:t&&c&&(r(c)||(d=i(c)))?(d?(d=!1,h=u&&i(u)?u:[]):h=u&&r(u)?u:{},f[l]=o([h,c],t,n,a)):(void 0!==c||n)&&(f[l]=c);return f}t.extendFlat=function(){return o(arguments,!1,!1,!1)},t.extendDeep=function(){return o(arguments,!0,!1,!1)},t.extendDeepAll=function(){return o(arguments,!0,!0,!1)},t.extendDeepNoArrays=function(){return o(arguments,!0,!1,!0)}},function(e,t,n){"use strict";var r=n(10),i=n(160),o={name:{valType:"string",editType:"none"}};function a(e){return e&&"string"==typeof e}function s(e){var t=e.length-1;return"s"!==e.charAt(t)&&r.warn("bad argument to arrayDefaultKey: "+e),e.substr(0,e.length-1)+"defaults"}o.templateitemname={valType:"string",editType:"calc"},t.templatedArray=function(e,t){return t._isLinkedToArray=e,t.name=o.name,t.templateitemname=o.templateitemname,t},t.traceTemplater=function(e){var t,n,o={};for(t in e)n=e[t],Array.isArray(n)&&n.length&&(o[t]=0);return{newTrace:function(a){var s={type:t=r.coerce(a,{},i,"type"),_template:null};if(t in o){n=e[t];var l=o[t]%n.length;o[t]++,s._template=n[l]}return s}}},t.newContainer=function(e,t,n){var i=e._template,o=i&&(i[t]||n&&i[n]);return r.isPlainObject(o)||(o=null),e[t]={_template:o}},t.arrayTemplater=function(e,t,n){var r=e._template,i=r&&r[s(t)],o=r&&r[t];Array.isArray(o)&&o.length||(o=[]);var l={};return{newItem:function(e){var t={name:e.name,_input:e},r=t.templateitemname=e.templateitemname;if(!a(r))return t._template=i,t;for(var s=0;s<o.length;s++){var u=o[s];if(u.name===r)return l[r]=1,t._template=u,t}return t[n]=e[n]||!1,t._template=!1,t},defaultItems:function(){for(var e=[],t=0;t<o.length;t++){var n=o[t],r=n.name;if(a(r)&&!l[r]){var i={_template:n,name:r,_input:{_templateitemname:r}};i.templateitemname=n.templateitemname,e.push(i),l[r]=1}}return e}}},t.arrayDefaultKey=s,t.arrayEditor=function(e,t,n){var i=(r.nestedProperty(e,t).get()||[]).length,o=n._index,a=o>=i&&(n._input||{})._templateitemname;a&&(o=i);var s,l=t+"["+o+"]";function u(){s={},a&&(s[l]={},s[l].templateitemname=a)}function c(e,t){a?r.nestedProperty(s[l],e).set(t):s[l+"."+e]=t}function d(){var e=s;return u(),e}return u(),{modifyBase:function(e,t){s[e]=t},modifyItem:c,getUpdateObj:d,applyUpdate:function(t,n){t&&c(t,n);var i=d();for(var o in i)r.nestedProperty(e,o).set(i[o])}}}},function(e,t,n){"use strict";n.d(t,"c",(function(){return i})),n.d(t,"b",(function(){return o}));var r=n(5),i={easeInOut:"cubic-bezier(0.4, 0, 0.2, 1)",easeOut:"cubic-bezier(0.0, 0, 0.2, 1)",easeIn:"cubic-bezier(0.4, 0, 1, 1)",sharp:"cubic-bezier(0.4, 0, 0.6, 1)"},o={shortest:150,shorter:200,short:250,standard:300,complex:375,enteringScreen:225,leavingScreen:195};function a(e){return"".concat(Math.round(e),"ms")}t.a={easing:i,duration:o,create:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:["all"],t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=t.duration,s=void 0===n?o.standard:n,l=t.easing,u=void 0===l?i.easeInOut:l,c=t.delay,d=void 0===c?0:c;Object(r.a)(t,["duration","easing","delay"]);return(Array.isArray(e)?e:[e]).map((function(e){return"".concat(e," ").concat("string"==typeof s?s:a(s)," ").concat(u," ").concat("string"==typeof d?d:a(d))})).join(",")},getAutoHeightDuration:function(e){if(!e)return 0;var t=e/36;return Math.round(10*(4+15*Math.pow(t,.25)+t/5))}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),Object.defineProperty(t,"default",{enumerable:!0,get:function(){return r.createSvgIcon}});var r=n(222)},function(e,t,n){(function(e,r){var i;
/**
 * @license
 * Lodash <https://lodash.com/>
 * Copyright OpenJS Foundation and other contributors <https://openjsf.org/>
 * Released under MIT license <https://lodash.com/license>
 * Based on Underscore.js 1.8.3 <http://underscorejs.org/LICENSE>
 * Copyright Jeremy Ashkenas, DocumentCloud and Investigative Reporters & Editors
 */(function(){var o="Expected a function",a="__lodash_placeholder__",s=[["ary",128],["bind",1],["bindKey",2],["curry",8],["curryRight",16],["flip",512],["partial",32],["partialRight",64],["rearg",256]],l="[object Arguments]",u="[object Array]",c="[object Boolean]",d="[object Date]",h="[object Error]",f="[object Function]",p="[object GeneratorFunction]",g="[object Map]",v="[object Number]",m="[object Object]",y="[object RegExp]",b="[object Set]",C="[object String]",A="[object Symbol]",_="[object WeakMap]",I="[object ArrayBuffer]",x="[object DataView]",w="[object Float32Array]",M="[object Float64Array]",S="[object Int8Array]",T="[object Int16Array]",E="[object Int32Array]",O="[object Uint8Array]",k="[object Uint16Array]",L="[object Uint32Array]",D=/\b__p \+= '';/g,P=/\b(__p \+=) '' \+/g,N=/(__e\(.*?\)|\b__t\)) \+\n'';/g,j=/&(?:amp|lt|gt|quot|#39);/g,R=/[&<>"']/g,z=RegExp(j.source),F=RegExp(R.source),B=/<%-([\s\S]+?)%>/g,H=/<%([\s\S]+?)%>/g,U=/<%=([\s\S]+?)%>/g,V=/\.|\[(?:[^[\]]*|(["'])(?:(?!\1)[^\\]|\\.)*?\1)\]/,Y=/^\w*$/,W=/[^.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\\]|\\.)*?)\2)\]|(?=(?:\.|\[\])(?:\.|\[\]|$))/g,G=/[\\^$.*+?()[\]{}|]/g,X=RegExp(G.source),q=/^\s+|\s+$/g,Z=/^\s+/,Q=/\s+$/,K=/\{(?:\n\/\* \[wrapped with .+\] \*\/)?\n?/,J=/\{\n\/\* \[wrapped with (.+)\] \*/,$=/,? & /,ee=/[^\x00-\x2f\x3a-\x40\x5b-\x60\x7b-\x7f]+/g,te=/\\(\\)?/g,ne=/\$\{([^\\}]*(?:\\.[^\\}]*)*)\}/g,re=/\w*$/,ie=/^[-+]0x[0-9a-f]+$/i,oe=/^0b[01]+$/i,ae=/^\[object .+?Constructor\]$/,se=/^0o[0-7]+$/i,le=/^(?:0|[1-9]\d*)$/,ue=/[\xc0-\xd6\xd8-\xf6\xf8-\xff\u0100-\u017f]/g,ce=/($^)/,de=/['\n\r\u2028\u2029\\]/g,he="\\u0300-\\u036f\\ufe20-\\ufe2f\\u20d0-\\u20ff",fe="\\xac\\xb1\\xd7\\xf7\\x00-\\x2f\\x3a-\\x40\\x5b-\\x60\\x7b-\\xbf\\u2000-\\u206f \\t\\x0b\\f\\xa0\\ufeff\\n\\r\\u2028\\u2029\\u1680\\u180e\\u2000\\u2001\\u2002\\u2003\\u2004\\u2005\\u2006\\u2007\\u2008\\u2009\\u200a\\u202f\\u205f\\u3000",pe="[\\ud800-\\udfff]",ge="["+fe+"]",ve="["+he+"]",me="\\d+",ye="[\\u2700-\\u27bf]",be="[a-z\\xdf-\\xf6\\xf8-\\xff]",Ce="[^\\ud800-\\udfff"+fe+me+"\\u2700-\\u27bfa-z\\xdf-\\xf6\\xf8-\\xffA-Z\\xc0-\\xd6\\xd8-\\xde]",Ae="\\ud83c[\\udffb-\\udfff]",_e="[^\\ud800-\\udfff]",Ie="(?:\\ud83c[\\udde6-\\uddff]){2}",xe="[\\ud800-\\udbff][\\udc00-\\udfff]",we="[A-Z\\xc0-\\xd6\\xd8-\\xde]",Me="(?:"+be+"|"+Ce+")",Se="(?:"+we+"|"+Ce+")",Te="(?:"+ve+"|"+Ae+")"+"?",Ee="[\\ufe0e\\ufe0f]?"+Te+("(?:\\u200d(?:"+[_e,Ie,xe].join("|")+")[\\ufe0e\\ufe0f]?"+Te+")*"),Oe="(?:"+[ye,Ie,xe].join("|")+")"+Ee,ke="(?:"+[_e+ve+"?",ve,Ie,xe,pe].join("|")+")",Le=RegExp("['’]","g"),De=RegExp(ve,"g"),Pe=RegExp(Ae+"(?="+Ae+")|"+ke+Ee,"g"),Ne=RegExp([we+"?"+be+"+(?:['’](?:d|ll|m|re|s|t|ve))?(?="+[ge,we,"$"].join("|")+")",Se+"+(?:['’](?:D|LL|M|RE|S|T|VE))?(?="+[ge,we+Me,"$"].join("|")+")",we+"?"+Me+"+(?:['’](?:d|ll|m|re|s|t|ve))?",we+"+(?:['’](?:D|LL|M|RE|S|T|VE))?","\\d*(?:1ST|2ND|3RD|(?![123])\\dTH)(?=\\b|[a-z_])","\\d*(?:1st|2nd|3rd|(?![123])\\dth)(?=\\b|[A-Z_])",me,Oe].join("|"),"g"),je=RegExp("[\\u200d\\ud800-\\udfff"+he+"\\ufe0e\\ufe0f]"),Re=/[a-z][A-Z]|[A-Z]{2}[a-z]|[0-9][a-zA-Z]|[a-zA-Z][0-9]|[^a-zA-Z0-9 ]/,ze=["Array","Buffer","DataView","Date","Error","Float32Array","Float64Array","Function","Int8Array","Int16Array","Int32Array","Map","Math","Object","Promise","RegExp","Set","String","Symbol","TypeError","Uint8Array","Uint8ClampedArray","Uint16Array","Uint32Array","WeakMap","_","clearTimeout","isFinite","parseInt","setTimeout"],Fe=-1,Be={};Be[w]=Be[M]=Be[S]=Be[T]=Be[E]=Be[O]=Be["[object Uint8ClampedArray]"]=Be[k]=Be[L]=!0,Be[l]=Be[u]=Be[I]=Be[c]=Be[x]=Be[d]=Be[h]=Be[f]=Be[g]=Be[v]=Be[m]=Be[y]=Be[b]=Be[C]=Be[_]=!1;var He={};He[l]=He[u]=He[I]=He[x]=He[c]=He[d]=He[w]=He[M]=He[S]=He[T]=He[E]=He[g]=He[v]=He[m]=He[y]=He[b]=He[C]=He[A]=He[O]=He["[object Uint8ClampedArray]"]=He[k]=He[L]=!0,He[h]=He[f]=He[_]=!1;var Ue={"\\":"\\","'":"'","\n":"n","\r":"r","\u2028":"u2028","\u2029":"u2029"},Ve=parseFloat,Ye=parseInt,We="object"==typeof e&&e&&e.Object===Object&&e,Ge="object"==typeof self&&self&&self.Object===Object&&self,Xe=We||Ge||Function("return this")(),qe=t&&!t.nodeType&&t,Ze=qe&&"object"==typeof r&&r&&!r.nodeType&&r,Qe=Ze&&Ze.exports===qe,Ke=Qe&&We.process,Je=function(){try{var e=Ze&&Ze.require&&Ze.require("util").types;return e||Ke&&Ke.binding&&Ke.binding("util")}catch(e){}}(),$e=Je&&Je.isArrayBuffer,et=Je&&Je.isDate,tt=Je&&Je.isMap,nt=Je&&Je.isRegExp,rt=Je&&Je.isSet,it=Je&&Je.isTypedArray;function ot(e,t,n){switch(n.length){case 0:return e.call(t);case 1:return e.call(t,n[0]);case 2:return e.call(t,n[0],n[1]);case 3:return e.call(t,n[0],n[1],n[2])}return e.apply(t,n)}function at(e,t,n,r){for(var i=-1,o=null==e?0:e.length;++i<o;){var a=e[i];t(r,a,n(a),e)}return r}function st(e,t){for(var n=-1,r=null==e?0:e.length;++n<r&&!1!==t(e[n],n,e););return e}function lt(e,t){for(var n=null==e?0:e.length;n--&&!1!==t(e[n],n,e););return e}function ut(e,t){for(var n=-1,r=null==e?0:e.length;++n<r;)if(!t(e[n],n,e))return!1;return!0}function ct(e,t){for(var n=-1,r=null==e?0:e.length,i=0,o=[];++n<r;){var a=e[n];t(a,n,e)&&(o[i++]=a)}return o}function dt(e,t){return!!(null==e?0:e.length)&&At(e,t,0)>-1}function ht(e,t,n){for(var r=-1,i=null==e?0:e.length;++r<i;)if(n(t,e[r]))return!0;return!1}function ft(e,t){for(var n=-1,r=null==e?0:e.length,i=Array(r);++n<r;)i[n]=t(e[n],n,e);return i}function pt(e,t){for(var n=-1,r=t.length,i=e.length;++n<r;)e[i+n]=t[n];return e}function gt(e,t,n,r){var i=-1,o=null==e?0:e.length;for(r&&o&&(n=e[++i]);++i<o;)n=t(n,e[i],i,e);return n}function vt(e,t,n,r){var i=null==e?0:e.length;for(r&&i&&(n=e[--i]);i--;)n=t(n,e[i],i,e);return n}function mt(e,t){for(var n=-1,r=null==e?0:e.length;++n<r;)if(t(e[n],n,e))return!0;return!1}var yt=wt("length");function bt(e,t,n){var r;return n(e,(function(e,n,i){if(t(e,n,i))return r=n,!1})),r}function Ct(e,t,n,r){for(var i=e.length,o=n+(r?1:-1);r?o--:++o<i;)if(t(e[o],o,e))return o;return-1}function At(e,t,n){return t==t?function(e,t,n){var r=n-1,i=e.length;for(;++r<i;)if(e[r]===t)return r;return-1}(e,t,n):Ct(e,It,n)}function _t(e,t,n,r){for(var i=n-1,o=e.length;++i<o;)if(r(e[i],t))return i;return-1}function It(e){return e!=e}function xt(e,t){var n=null==e?0:e.length;return n?Tt(e,t)/n:NaN}function wt(e){return function(t){return null==t?void 0:t[e]}}function Mt(e){return function(t){return null==e?void 0:e[t]}}function St(e,t,n,r,i){return i(e,(function(e,i,o){n=r?(r=!1,e):t(n,e,i,o)})),n}function Tt(e,t){for(var n,r=-1,i=e.length;++r<i;){var o=t(e[r]);void 0!==o&&(n=void 0===n?o:n+o)}return n}function Et(e,t){for(var n=-1,r=Array(e);++n<e;)r[n]=t(n);return r}function Ot(e){return function(t){return e(t)}}function kt(e,t){return ft(t,(function(t){return e[t]}))}function Lt(e,t){return e.has(t)}function Dt(e,t){for(var n=-1,r=e.length;++n<r&&At(t,e[n],0)>-1;);return n}function Pt(e,t){for(var n=e.length;n--&&At(t,e[n],0)>-1;);return n}function Nt(e,t){for(var n=e.length,r=0;n--;)e[n]===t&&++r;return r}var jt=Mt({"À":"A","Á":"A","Â":"A","Ã":"A","Ä":"A","Å":"A","à":"a","á":"a","â":"a","ã":"a","ä":"a","å":"a","Ç":"C","ç":"c","Ð":"D","ð":"d","È":"E","É":"E","Ê":"E","Ë":"E","è":"e","é":"e","ê":"e","ë":"e","Ì":"I","Í":"I","Î":"I","Ï":"I","ì":"i","í":"i","î":"i","ï":"i","Ñ":"N","ñ":"n","Ò":"O","Ó":"O","Ô":"O","Õ":"O","Ö":"O","Ø":"O","ò":"o","ó":"o","ô":"o","õ":"o","ö":"o","ø":"o","Ù":"U","Ú":"U","Û":"U","Ü":"U","ù":"u","ú":"u","û":"u","ü":"u","Ý":"Y","ý":"y","ÿ":"y","Æ":"Ae","æ":"ae","Þ":"Th","þ":"th","ß":"ss","Ā":"A","Ă":"A","Ą":"A","ā":"a","ă":"a","ą":"a","Ć":"C","Ĉ":"C","Ċ":"C","Č":"C","ć":"c","ĉ":"c","ċ":"c","č":"c","Ď":"D","Đ":"D","ď":"d","đ":"d","Ē":"E","Ĕ":"E","Ė":"E","Ę":"E","Ě":"E","ē":"e","ĕ":"e","ė":"e","ę":"e","ě":"e","Ĝ":"G","Ğ":"G","Ġ":"G","Ģ":"G","ĝ":"g","ğ":"g","ġ":"g","ģ":"g","Ĥ":"H","Ħ":"H","ĥ":"h","ħ":"h","Ĩ":"I","Ī":"I","Ĭ":"I","Į":"I","İ":"I","ĩ":"i","ī":"i","ĭ":"i","į":"i","ı":"i","Ĵ":"J","ĵ":"j","Ķ":"K","ķ":"k","ĸ":"k","Ĺ":"L","Ļ":"L","Ľ":"L","Ŀ":"L","Ł":"L","ĺ":"l","ļ":"l","ľ":"l","ŀ":"l","ł":"l","Ń":"N","Ņ":"N","Ň":"N","Ŋ":"N","ń":"n","ņ":"n","ň":"n","ŋ":"n","Ō":"O","Ŏ":"O","Ő":"O","ō":"o","ŏ":"o","ő":"o","Ŕ":"R","Ŗ":"R","Ř":"R","ŕ":"r","ŗ":"r","ř":"r","Ś":"S","Ŝ":"S","Ş":"S","Š":"S","ś":"s","ŝ":"s","ş":"s","š":"s","Ţ":"T","Ť":"T","Ŧ":"T","ţ":"t","ť":"t","ŧ":"t","Ũ":"U","Ū":"U","Ŭ":"U","Ů":"U","Ű":"U","Ų":"U","ũ":"u","ū":"u","ŭ":"u","ů":"u","ű":"u","ų":"u","Ŵ":"W","ŵ":"w","Ŷ":"Y","ŷ":"y","Ÿ":"Y","Ź":"Z","Ż":"Z","Ž":"Z","ź":"z","ż":"z","ž":"z","Ĳ":"IJ","ĳ":"ij","Œ":"Oe","œ":"oe","ŉ":"'n","ſ":"s"}),Rt=Mt({"&":"&amp;","<":"&lt;",">":"&gt;",'"':"&quot;","'":"&#39;"});function zt(e){return"\\"+Ue[e]}function Ft(e){return je.test(e)}function Bt(e){var t=-1,n=Array(e.size);return e.forEach((function(e,r){n[++t]=[r,e]})),n}function Ht(e,t){return function(n){return e(t(n))}}function Ut(e,t){for(var n=-1,r=e.length,i=0,o=[];++n<r;){var s=e[n];s!==t&&s!==a||(e[n]=a,o[i++]=n)}return o}function Vt(e){var t=-1,n=Array(e.size);return e.forEach((function(e){n[++t]=e})),n}function Yt(e){var t=-1,n=Array(e.size);return e.forEach((function(e){n[++t]=[e,e]})),n}function Wt(e){return Ft(e)?function(e){var t=Pe.lastIndex=0;for(;Pe.test(e);)++t;return t}(e):yt(e)}function Gt(e){return Ft(e)?function(e){return e.match(Pe)||[]}(e):function(e){return e.split("")}(e)}var Xt=Mt({"&amp;":"&","&lt;":"<","&gt;":">","&quot;":'"',"&#39;":"'"});var qt=function e(t){var n,r=(t=null==t?Xe:qt.defaults(Xe.Object(),t,qt.pick(Xe,ze))).Array,i=t.Date,he=t.Error,fe=t.Function,pe=t.Math,ge=t.Object,ve=t.RegExp,me=t.String,ye=t.TypeError,be=r.prototype,Ce=fe.prototype,Ae=ge.prototype,_e=t["__core-js_shared__"],Ie=Ce.toString,xe=Ae.hasOwnProperty,we=0,Me=(n=/[^.]+$/.exec(_e&&_e.keys&&_e.keys.IE_PROTO||""))?"Symbol(src)_1."+n:"",Se=Ae.toString,Te=Ie.call(ge),Ee=Xe._,Oe=ve("^"+Ie.call(xe).replace(G,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$"),ke=Qe?t.Buffer:void 0,Pe=t.Symbol,je=t.Uint8Array,Ue=ke?ke.allocUnsafe:void 0,We=Ht(ge.getPrototypeOf,ge),Ge=ge.create,qe=Ae.propertyIsEnumerable,Ze=be.splice,Ke=Pe?Pe.isConcatSpreadable:void 0,Je=Pe?Pe.iterator:void 0,yt=Pe?Pe.toStringTag:void 0,Mt=function(){try{var e=$i(ge,"defineProperty");return e({},"",{}),e}catch(e){}}(),Zt=t.clearTimeout!==Xe.clearTimeout&&t.clearTimeout,Qt=i&&i.now!==Xe.Date.now&&i.now,Kt=t.setTimeout!==Xe.setTimeout&&t.setTimeout,Jt=pe.ceil,$t=pe.floor,en=ge.getOwnPropertySymbols,tn=ke?ke.isBuffer:void 0,nn=t.isFinite,rn=be.join,on=Ht(ge.keys,ge),an=pe.max,sn=pe.min,ln=i.now,un=t.parseInt,cn=pe.random,dn=be.reverse,hn=$i(t,"DataView"),fn=$i(t,"Map"),pn=$i(t,"Promise"),gn=$i(t,"Set"),vn=$i(t,"WeakMap"),mn=$i(ge,"create"),yn=vn&&new vn,bn={},Cn=So(hn),An=So(fn),_n=So(pn),In=So(gn),xn=So(vn),wn=Pe?Pe.prototype:void 0,Mn=wn?wn.valueOf:void 0,Sn=wn?wn.toString:void 0;function Tn(e){if(Ya(e)&&!Da(e)&&!(e instanceof Ln)){if(e instanceof kn)return e;if(xe.call(e,"__wrapped__"))return To(e)}return new kn(e)}var En=function(){function e(){}return function(t){if(!Va(t))return{};if(Ge)return Ge(t);e.prototype=t;var n=new e;return e.prototype=void 0,n}}();function On(){}function kn(e,t){this.__wrapped__=e,this.__actions__=[],this.__chain__=!!t,this.__index__=0,this.__values__=void 0}function Ln(e){this.__wrapped__=e,this.__actions__=[],this.__dir__=1,this.__filtered__=!1,this.__iteratees__=[],this.__takeCount__=4294967295,this.__views__=[]}function Dn(e){var t=-1,n=null==e?0:e.length;for(this.clear();++t<n;){var r=e[t];this.set(r[0],r[1])}}function Pn(e){var t=-1,n=null==e?0:e.length;for(this.clear();++t<n;){var r=e[t];this.set(r[0],r[1])}}function Nn(e){var t=-1,n=null==e?0:e.length;for(this.clear();++t<n;){var r=e[t];this.set(r[0],r[1])}}function jn(e){var t=-1,n=null==e?0:e.length;for(this.__data__=new Nn;++t<n;)this.add(e[t])}function Rn(e){var t=this.__data__=new Pn(e);this.size=t.size}function zn(e,t){var n=Da(e),r=!n&&La(e),i=!n&&!r&&Ra(e),o=!n&&!r&&!i&&Ja(e),a=n||r||i||o,s=a?Et(e.length,me):[],l=s.length;for(var u in e)!t&&!xe.call(e,u)||a&&("length"==u||i&&("offset"==u||"parent"==u)||o&&("buffer"==u||"byteLength"==u||"byteOffset"==u)||ao(u,l))||s.push(u);return s}function Fn(e){var t=e.length;return t?e[jr(0,t-1)]:void 0}function Bn(e,t){return xo(mi(e),Zn(t,0,e.length))}function Hn(e){return xo(mi(e))}function Un(e,t,n){(void 0!==n&&!Ea(e[t],n)||void 0===n&&!(t in e))&&Xn(e,t,n)}function Vn(e,t,n){var r=e[t];xe.call(e,t)&&Ea(r,n)&&(void 0!==n||t in e)||Xn(e,t,n)}function Yn(e,t){for(var n=e.length;n--;)if(Ea(e[n][0],t))return n;return-1}function Wn(e,t,n,r){return er(e,(function(e,i,o){t(r,e,n(e),o)})),r}function Gn(e,t){return e&&yi(t,Cs(t),e)}function Xn(e,t,n){"__proto__"==t&&Mt?Mt(e,t,{configurable:!0,enumerable:!0,value:n,writable:!0}):e[t]=n}function qn(e,t){for(var n=-1,i=t.length,o=r(i),a=null==e;++n<i;)o[n]=a?void 0:gs(e,t[n]);return o}function Zn(e,t,n){return e==e&&(void 0!==n&&(e=e<=n?e:n),void 0!==t&&(e=e>=t?e:t)),e}function Qn(e,t,n,r,i,o){var a,s=1&t,u=2&t,h=4&t;if(n&&(a=i?n(e,r,i,o):n(e)),void 0!==a)return a;if(!Va(e))return e;var _=Da(e);if(_){if(a=function(e){var t=e.length,n=new e.constructor(t);t&&"string"==typeof e[0]&&xe.call(e,"index")&&(n.index=e.index,n.input=e.input);return n}(e),!s)return mi(e,a)}else{var D=no(e),P=D==f||D==p;if(Ra(e))return di(e,s);if(D==m||D==l||P&&!i){if(a=u||P?{}:io(e),!s)return u?function(e,t){return yi(e,to(e),t)}(e,function(e,t){return e&&yi(t,As(t),e)}(a,e)):function(e,t){return yi(e,eo(e),t)}(e,Gn(a,e))}else{if(!He[D])return i?e:{};a=function(e,t,n){var r=e.constructor;switch(t){case I:return hi(e);case c:case d:return new r(+e);case x:return function(e,t){var n=t?hi(e.buffer):e.buffer;return new e.constructor(n,e.byteOffset,e.byteLength)}(e,n);case w:case M:case S:case T:case E:case O:case"[object Uint8ClampedArray]":case k:case L:return fi(e,n);case g:return new r;case v:case C:return new r(e);case y:return function(e){var t=new e.constructor(e.source,re.exec(e));return t.lastIndex=e.lastIndex,t}(e);case b:return new r;case A:return i=e,Mn?ge(Mn.call(i)):{}}var i}(e,D,s)}}o||(o=new Rn);var N=o.get(e);if(N)return N;o.set(e,a),Za(e)?e.forEach((function(r){a.add(Qn(r,t,n,r,e,o))})):Wa(e)&&e.forEach((function(r,i){a.set(i,Qn(r,t,n,i,e,o))}));var j=_?void 0:(h?u?Gi:Wi:u?As:Cs)(e);return st(j||e,(function(r,i){j&&(r=e[i=r]),Vn(a,i,Qn(r,t,n,i,e,o))})),a}function Kn(e,t,n){var r=n.length;if(null==e)return!r;for(e=ge(e);r--;){var i=n[r],o=t[i],a=e[i];if(void 0===a&&!(i in e)||!o(a))return!1}return!0}function Jn(e,t,n){if("function"!=typeof e)throw new ye(o);return Co((function(){e.apply(void 0,n)}),t)}function $n(e,t,n,r){var i=-1,o=dt,a=!0,s=e.length,l=[],u=t.length;if(!s)return l;n&&(t=ft(t,Ot(n))),r?(o=ht,a=!1):t.length>=200&&(o=Lt,a=!1,t=new jn(t));e:for(;++i<s;){var c=e[i],d=null==n?c:n(c);if(c=r||0!==c?c:0,a&&d==d){for(var h=u;h--;)if(t[h]===d)continue e;l.push(c)}else o(t,d,r)||l.push(c)}return l}Tn.templateSettings={escape:B,evaluate:H,interpolate:U,variable:"",imports:{_:Tn}},Tn.prototype=On.prototype,Tn.prototype.constructor=Tn,kn.prototype=En(On.prototype),kn.prototype.constructor=kn,Ln.prototype=En(On.prototype),Ln.prototype.constructor=Ln,Dn.prototype.clear=function(){this.__data__=mn?mn(null):{},this.size=0},Dn.prototype.delete=function(e){var t=this.has(e)&&delete this.__data__[e];return this.size-=t?1:0,t},Dn.prototype.get=function(e){var t=this.__data__;if(mn){var n=t[e];return"__lodash_hash_undefined__"===n?void 0:n}return xe.call(t,e)?t[e]:void 0},Dn.prototype.has=function(e){var t=this.__data__;return mn?void 0!==t[e]:xe.call(t,e)},Dn.prototype.set=function(e,t){var n=this.__data__;return this.size+=this.has(e)?0:1,n[e]=mn&&void 0===t?"__lodash_hash_undefined__":t,this},Pn.prototype.clear=function(){this.__data__=[],this.size=0},Pn.prototype.delete=function(e){var t=this.__data__,n=Yn(t,e);return!(n<0)&&(n==t.length-1?t.pop():Ze.call(t,n,1),--this.size,!0)},Pn.prototype.get=function(e){var t=this.__data__,n=Yn(t,e);return n<0?void 0:t[n][1]},Pn.prototype.has=function(e){return Yn(this.__data__,e)>-1},Pn.prototype.set=function(e,t){var n=this.__data__,r=Yn(n,e);return r<0?(++this.size,n.push([e,t])):n[r][1]=t,this},Nn.prototype.clear=function(){this.size=0,this.__data__={hash:new Dn,map:new(fn||Pn),string:new Dn}},Nn.prototype.delete=function(e){var t=Ki(this,e).delete(e);return this.size-=t?1:0,t},Nn.prototype.get=function(e){return Ki(this,e).get(e)},Nn.prototype.has=function(e){return Ki(this,e).has(e)},Nn.prototype.set=function(e,t){var n=Ki(this,e),r=n.size;return n.set(e,t),this.size+=n.size==r?0:1,this},jn.prototype.add=jn.prototype.push=function(e){return this.__data__.set(e,"__lodash_hash_undefined__"),this},jn.prototype.has=function(e){return this.__data__.has(e)},Rn.prototype.clear=function(){this.__data__=new Pn,this.size=0},Rn.prototype.delete=function(e){var t=this.__data__,n=t.delete(e);return this.size=t.size,n},Rn.prototype.get=function(e){return this.__data__.get(e)},Rn.prototype.has=function(e){return this.__data__.has(e)},Rn.prototype.set=function(e,t){var n=this.__data__;if(n instanceof Pn){var r=n.__data__;if(!fn||r.length<199)return r.push([e,t]),this.size=++n.size,this;n=this.__data__=new Nn(r)}return n.set(e,t),this.size=n.size,this};var er=Ai(lr),tr=Ai(ur,!0);function nr(e,t){var n=!0;return er(e,(function(e,r,i){return n=!!t(e,r,i)})),n}function rr(e,t,n){for(var r=-1,i=e.length;++r<i;){var o=e[r],a=t(o);if(null!=a&&(void 0===s?a==a&&!Ka(a):n(a,s)))var s=a,l=o}return l}function ir(e,t){var n=[];return er(e,(function(e,r,i){t(e,r,i)&&n.push(e)})),n}function or(e,t,n,r,i){var o=-1,a=e.length;for(n||(n=oo),i||(i=[]);++o<a;){var s=e[o];t>0&&n(s)?t>1?or(s,t-1,n,r,i):pt(i,s):r||(i[i.length]=s)}return i}var ar=_i(),sr=_i(!0);function lr(e,t){return e&&ar(e,t,Cs)}function ur(e,t){return e&&sr(e,t,Cs)}function cr(e,t){return ct(t,(function(t){return Ba(e[t])}))}function dr(e,t){for(var n=0,r=(t=si(t,e)).length;null!=e&&n<r;)e=e[Mo(t[n++])];return n&&n==r?e:void 0}function hr(e,t,n){var r=t(e);return Da(e)?r:pt(r,n(e))}function fr(e){return null==e?void 0===e?"[object Undefined]":"[object Null]":yt&&yt in ge(e)?function(e){var t=xe.call(e,yt),n=e[yt];try{e[yt]=void 0;var r=!0}catch(e){}var i=Se.call(e);r&&(t?e[yt]=n:delete e[yt]);return i}(e):function(e){return Se.call(e)}(e)}function pr(e,t){return e>t}function gr(e,t){return null!=e&&xe.call(e,t)}function vr(e,t){return null!=e&&t in ge(e)}function mr(e,t,n){for(var i=n?ht:dt,o=e[0].length,a=e.length,s=a,l=r(a),u=1/0,c=[];s--;){var d=e[s];s&&t&&(d=ft(d,Ot(t))),u=sn(d.length,u),l[s]=!n&&(t||o>=120&&d.length>=120)?new jn(s&&d):void 0}d=e[0];var h=-1,f=l[0];e:for(;++h<o&&c.length<u;){var p=d[h],g=t?t(p):p;if(p=n||0!==p?p:0,!(f?Lt(f,g):i(c,g,n))){for(s=a;--s;){var v=l[s];if(!(v?Lt(v,g):i(e[s],g,n)))continue e}f&&f.push(g),c.push(p)}}return c}function yr(e,t,n){var r=null==(e=vo(e,t=si(t,e)))?e:e[Mo(Fo(t))];return null==r?void 0:ot(r,e,n)}function br(e){return Ya(e)&&fr(e)==l}function Cr(e,t,n,r,i){return e===t||(null==e||null==t||!Ya(e)&&!Ya(t)?e!=e&&t!=t:function(e,t,n,r,i,o){var a=Da(e),s=Da(t),f=a?u:no(e),p=s?u:no(t),_=(f=f==l?m:f)==m,w=(p=p==l?m:p)==m,M=f==p;if(M&&Ra(e)){if(!Ra(t))return!1;a=!0,_=!1}if(M&&!_)return o||(o=new Rn),a||Ja(e)?Vi(e,t,n,r,i,o):function(e,t,n,r,i,o,a){switch(n){case x:if(e.byteLength!=t.byteLength||e.byteOffset!=t.byteOffset)return!1;e=e.buffer,t=t.buffer;case I:return!(e.byteLength!=t.byteLength||!o(new je(e),new je(t)));case c:case d:case v:return Ea(+e,+t);case h:return e.name==t.name&&e.message==t.message;case y:case C:return e==t+"";case g:var s=Bt;case b:var l=1&r;if(s||(s=Vt),e.size!=t.size&&!l)return!1;var u=a.get(e);if(u)return u==t;r|=2,a.set(e,t);var f=Vi(s(e),s(t),r,i,o,a);return a.delete(e),f;case A:if(Mn)return Mn.call(e)==Mn.call(t)}return!1}(e,t,f,n,r,i,o);if(!(1&n)){var S=_&&xe.call(e,"__wrapped__"),T=w&&xe.call(t,"__wrapped__");if(S||T){var E=S?e.value():e,O=T?t.value():t;return o||(o=new Rn),i(E,O,n,r,o)}}if(!M)return!1;return o||(o=new Rn),function(e,t,n,r,i,o){var a=1&n,s=Wi(e),l=s.length,u=Wi(t).length;if(l!=u&&!a)return!1;var c=l;for(;c--;){var d=s[c];if(!(a?d in t:xe.call(t,d)))return!1}var h=o.get(e);if(h&&o.get(t))return h==t;var f=!0;o.set(e,t),o.set(t,e);var p=a;for(;++c<l;){d=s[c];var g=e[d],v=t[d];if(r)var m=a?r(v,g,d,t,e,o):r(g,v,d,e,t,o);if(!(void 0===m?g===v||i(g,v,n,r,o):m)){f=!1;break}p||(p="constructor"==d)}if(f&&!p){var y=e.constructor,b=t.constructor;y==b||!("constructor"in e)||!("constructor"in t)||"function"==typeof y&&y instanceof y&&"function"==typeof b&&b instanceof b||(f=!1)}return o.delete(e),o.delete(t),f}(e,t,n,r,i,o)}(e,t,n,r,Cr,i))}function Ar(e,t,n,r){var i=n.length,o=i,a=!r;if(null==e)return!o;for(e=ge(e);i--;){var s=n[i];if(a&&s[2]?s[1]!==e[s[0]]:!(s[0]in e))return!1}for(;++i<o;){var l=(s=n[i])[0],u=e[l],c=s[1];if(a&&s[2]){if(void 0===u&&!(l in e))return!1}else{var d=new Rn;if(r)var h=r(u,c,l,e,t,d);if(!(void 0===h?Cr(c,u,3,r,d):h))return!1}}return!0}function _r(e){return!(!Va(e)||(t=e,Me&&Me in t))&&(Ba(e)?Oe:ae).test(So(e));var t}function Ir(e){return"function"==typeof e?e:null==e?Gs:"object"==typeof e?Da(e)?Er(e[0],e[1]):Tr(e):tl(e)}function xr(e){if(!ho(e))return on(e);var t=[];for(var n in ge(e))xe.call(e,n)&&"constructor"!=n&&t.push(n);return t}function wr(e){if(!Va(e))return function(e){var t=[];if(null!=e)for(var n in ge(e))t.push(n);return t}(e);var t=ho(e),n=[];for(var r in e)("constructor"!=r||!t&&xe.call(e,r))&&n.push(r);return n}function Mr(e,t){return e<t}function Sr(e,t){var n=-1,i=Na(e)?r(e.length):[];return er(e,(function(e,r,o){i[++n]=t(e,r,o)})),i}function Tr(e){var t=Ji(e);return 1==t.length&&t[0][2]?po(t[0][0],t[0][1]):function(n){return n===e||Ar(n,e,t)}}function Er(e,t){return lo(e)&&fo(t)?po(Mo(e),t):function(n){var r=gs(n,e);return void 0===r&&r===t?vs(n,e):Cr(t,r,3)}}function Or(e,t,n,r,i){e!==t&&ar(t,(function(o,a){if(i||(i=new Rn),Va(o))!function(e,t,n,r,i,o,a){var s=yo(e,n),l=yo(t,n),u=a.get(l);if(u)return void Un(e,n,u);var c=o?o(s,l,n+"",e,t,a):void 0,d=void 0===c;if(d){var h=Da(l),f=!h&&Ra(l),p=!h&&!f&&Ja(l);c=l,h||f||p?Da(s)?c=s:ja(s)?c=mi(s):f?(d=!1,c=di(l,!0)):p?(d=!1,c=fi(l,!0)):c=[]:Xa(l)||La(l)?(c=s,La(s)?c=as(s):Va(s)&&!Ba(s)||(c=io(l))):d=!1}d&&(a.set(l,c),i(c,l,r,o,a),a.delete(l));Un(e,n,c)}(e,t,a,n,Or,r,i);else{var s=r?r(yo(e,a),o,a+"",e,t,i):void 0;void 0===s&&(s=o),Un(e,a,s)}}),As)}function kr(e,t){var n=e.length;if(n)return ao(t+=t<0?n:0,n)?e[t]:void 0}function Lr(e,t,n){var r=-1;return t=ft(t.length?t:[Gs],Ot(Qi())),function(e,t){var n=e.length;for(e.sort(t);n--;)e[n]=e[n].value;return e}(Sr(e,(function(e,n,i){return{criteria:ft(t,(function(t){return t(e)})),index:++r,value:e}})),(function(e,t){return function(e,t,n){var r=-1,i=e.criteria,o=t.criteria,a=i.length,s=n.length;for(;++r<a;){var l=pi(i[r],o[r]);if(l){if(r>=s)return l;var u=n[r];return l*("desc"==u?-1:1)}}return e.index-t.index}(e,t,n)}))}function Dr(e,t,n){for(var r=-1,i=t.length,o={};++r<i;){var a=t[r],s=dr(e,a);n(s,a)&&Hr(o,si(a,e),s)}return o}function Pr(e,t,n,r){var i=r?_t:At,o=-1,a=t.length,s=e;for(e===t&&(t=mi(t)),n&&(s=ft(e,Ot(n)));++o<a;)for(var l=0,u=t[o],c=n?n(u):u;(l=i(s,c,l,r))>-1;)s!==e&&Ze.call(s,l,1),Ze.call(e,l,1);return e}function Nr(e,t){for(var n=e?t.length:0,r=n-1;n--;){var i=t[n];if(n==r||i!==o){var o=i;ao(i)?Ze.call(e,i,1):$r(e,i)}}return e}function jr(e,t){return e+$t(cn()*(t-e+1))}function Rr(e,t){var n="";if(!e||t<1||t>9007199254740991)return n;do{t%2&&(n+=e),(t=$t(t/2))&&(e+=e)}while(t);return n}function zr(e,t){return Ao(go(e,t,Gs),e+"")}function Fr(e){return Fn(Es(e))}function Br(e,t){var n=Es(e);return xo(n,Zn(t,0,n.length))}function Hr(e,t,n,r){if(!Va(e))return e;for(var i=-1,o=(t=si(t,e)).length,a=o-1,s=e;null!=s&&++i<o;){var l=Mo(t[i]),u=n;if(i!=a){var c=s[l];void 0===(u=r?r(c,l,s):void 0)&&(u=Va(c)?c:ao(t[i+1])?[]:{})}Vn(s,l,u),s=s[l]}return e}var Ur=yn?function(e,t){return yn.set(e,t),e}:Gs,Vr=Mt?function(e,t){return Mt(e,"toString",{configurable:!0,enumerable:!1,value:Vs(t),writable:!0})}:Gs;function Yr(e){return xo(Es(e))}function Wr(e,t,n){var i=-1,o=e.length;t<0&&(t=-t>o?0:o+t),(n=n>o?o:n)<0&&(n+=o),o=t>n?0:n-t>>>0,t>>>=0;for(var a=r(o);++i<o;)a[i]=e[i+t];return a}function Gr(e,t){var n;return er(e,(function(e,r,i){return!(n=t(e,r,i))})),!!n}function Xr(e,t,n){var r=0,i=null==e?r:e.length;if("number"==typeof t&&t==t&&i<=2147483647){for(;r<i;){var o=r+i>>>1,a=e[o];null!==a&&!Ka(a)&&(n?a<=t:a<t)?r=o+1:i=o}return i}return qr(e,t,Gs,n)}function qr(e,t,n,r){t=n(t);for(var i=0,o=null==e?0:e.length,a=t!=t,s=null===t,l=Ka(t),u=void 0===t;i<o;){var c=$t((i+o)/2),d=n(e[c]),h=void 0!==d,f=null===d,p=d==d,g=Ka(d);if(a)var v=r||p;else v=u?p&&(r||h):s?p&&h&&(r||!f):l?p&&h&&!f&&(r||!g):!f&&!g&&(r?d<=t:d<t);v?i=c+1:o=c}return sn(o,4294967294)}function Zr(e,t){for(var n=-1,r=e.length,i=0,o=[];++n<r;){var a=e[n],s=t?t(a):a;if(!n||!Ea(s,l)){var l=s;o[i++]=0===a?0:a}}return o}function Qr(e){return"number"==typeof e?e:Ka(e)?NaN:+e}function Kr(e){if("string"==typeof e)return e;if(Da(e))return ft(e,Kr)+"";if(Ka(e))return Sn?Sn.call(e):"";var t=e+"";return"0"==t&&1/e==-1/0?"-0":t}function Jr(e,t,n){var r=-1,i=dt,o=e.length,a=!0,s=[],l=s;if(n)a=!1,i=ht;else if(o>=200){var u=t?null:Ri(e);if(u)return Vt(u);a=!1,i=Lt,l=new jn}else l=t?[]:s;e:for(;++r<o;){var c=e[r],d=t?t(c):c;if(c=n||0!==c?c:0,a&&d==d){for(var h=l.length;h--;)if(l[h]===d)continue e;t&&l.push(d),s.push(c)}else i(l,d,n)||(l!==s&&l.push(d),s.push(c))}return s}function $r(e,t){return null==(e=vo(e,t=si(t,e)))||delete e[Mo(Fo(t))]}function ei(e,t,n,r){return Hr(e,t,n(dr(e,t)),r)}function ti(e,t,n,r){for(var i=e.length,o=r?i:-1;(r?o--:++o<i)&&t(e[o],o,e););return n?Wr(e,r?0:o,r?o+1:i):Wr(e,r?o+1:0,r?i:o)}function ni(e,t){var n=e;return n instanceof Ln&&(n=n.value()),gt(t,(function(e,t){return t.func.apply(t.thisArg,pt([e],t.args))}),n)}function ri(e,t,n){var i=e.length;if(i<2)return i?Jr(e[0]):[];for(var o=-1,a=r(i);++o<i;)for(var s=e[o],l=-1;++l<i;)l!=o&&(a[o]=$n(a[o]||s,e[l],t,n));return Jr(or(a,1),t,n)}function ii(e,t,n){for(var r=-1,i=e.length,o=t.length,a={};++r<i;){var s=r<o?t[r]:void 0;n(a,e[r],s)}return a}function oi(e){return ja(e)?e:[]}function ai(e){return"function"==typeof e?e:Gs}function si(e,t){return Da(e)?e:lo(e,t)?[e]:wo(ss(e))}var li=zr;function ui(e,t,n){var r=e.length;return n=void 0===n?r:n,!t&&n>=r?e:Wr(e,t,n)}var ci=Zt||function(e){return Xe.clearTimeout(e)};function di(e,t){if(t)return e.slice();var n=e.length,r=Ue?Ue(n):new e.constructor(n);return e.copy(r),r}function hi(e){var t=new e.constructor(e.byteLength);return new je(t).set(new je(e)),t}function fi(e,t){var n=t?hi(e.buffer):e.buffer;return new e.constructor(n,e.byteOffset,e.length)}function pi(e,t){if(e!==t){var n=void 0!==e,r=null===e,i=e==e,o=Ka(e),a=void 0!==t,s=null===t,l=t==t,u=Ka(t);if(!s&&!u&&!o&&e>t||o&&a&&l&&!s&&!u||r&&a&&l||!n&&l||!i)return 1;if(!r&&!o&&!u&&e<t||u&&n&&i&&!r&&!o||s&&n&&i||!a&&i||!l)return-1}return 0}function gi(e,t,n,i){for(var o=-1,a=e.length,s=n.length,l=-1,u=t.length,c=an(a-s,0),d=r(u+c),h=!i;++l<u;)d[l]=t[l];for(;++o<s;)(h||o<a)&&(d[n[o]]=e[o]);for(;c--;)d[l++]=e[o++];return d}function vi(e,t,n,i){for(var o=-1,a=e.length,s=-1,l=n.length,u=-1,c=t.length,d=an(a-l,0),h=r(d+c),f=!i;++o<d;)h[o]=e[o];for(var p=o;++u<c;)h[p+u]=t[u];for(;++s<l;)(f||o<a)&&(h[p+n[s]]=e[o++]);return h}function mi(e,t){var n=-1,i=e.length;for(t||(t=r(i));++n<i;)t[n]=e[n];return t}function yi(e,t,n,r){var i=!n;n||(n={});for(var o=-1,a=t.length;++o<a;){var s=t[o],l=r?r(n[s],e[s],s,n,e):void 0;void 0===l&&(l=e[s]),i?Xn(n,s,l):Vn(n,s,l)}return n}function bi(e,t){return function(n,r){var i=Da(n)?at:Wn,o=t?t():{};return i(n,e,Qi(r,2),o)}}function Ci(e){return zr((function(t,n){var r=-1,i=n.length,o=i>1?n[i-1]:void 0,a=i>2?n[2]:void 0;for(o=e.length>3&&"function"==typeof o?(i--,o):void 0,a&&so(n[0],n[1],a)&&(o=i<3?void 0:o,i=1),t=ge(t);++r<i;){var s=n[r];s&&e(t,s,r,o)}return t}))}function Ai(e,t){return function(n,r){if(null==n)return n;if(!Na(n))return e(n,r);for(var i=n.length,o=t?i:-1,a=ge(n);(t?o--:++o<i)&&!1!==r(a[o],o,a););return n}}function _i(e){return function(t,n,r){for(var i=-1,o=ge(t),a=r(t),s=a.length;s--;){var l=a[e?s:++i];if(!1===n(o[l],l,o))break}return t}}function Ii(e){return function(t){var n=Ft(t=ss(t))?Gt(t):void 0,r=n?n[0]:t.charAt(0),i=n?ui(n,1).join(""):t.slice(1);return r[e]()+i}}function xi(e){return function(t){return gt(Bs(Ls(t).replace(Le,"")),e,"")}}function wi(e){return function(){var t=arguments;switch(t.length){case 0:return new e;case 1:return new e(t[0]);case 2:return new e(t[0],t[1]);case 3:return new e(t[0],t[1],t[2]);case 4:return new e(t[0],t[1],t[2],t[3]);case 5:return new e(t[0],t[1],t[2],t[3],t[4]);case 6:return new e(t[0],t[1],t[2],t[3],t[4],t[5]);case 7:return new e(t[0],t[1],t[2],t[3],t[4],t[5],t[6])}var n=En(e.prototype),r=e.apply(n,t);return Va(r)?r:n}}function Mi(e){return function(t,n,r){var i=ge(t);if(!Na(t)){var o=Qi(n,3);t=Cs(t),n=function(e){return o(i[e],e,i)}}var a=e(t,n,r);return a>-1?i[o?t[a]:a]:void 0}}function Si(e){return Yi((function(t){var n=t.length,r=n,i=kn.prototype.thru;for(e&&t.reverse();r--;){var a=t[r];if("function"!=typeof a)throw new ye(o);if(i&&!s&&"wrapper"==qi(a))var s=new kn([],!0)}for(r=s?r:n;++r<n;){var l=qi(a=t[r]),u="wrapper"==l?Xi(a):void 0;s=u&&uo(u[0])&&424==u[1]&&!u[4].length&&1==u[9]?s[qi(u[0])].apply(s,u[3]):1==a.length&&uo(a)?s[l]():s.thru(a)}return function(){var e=arguments,r=e[0];if(s&&1==e.length&&Da(r))return s.plant(r).value();for(var i=0,o=n?t[i].apply(this,e):r;++i<n;)o=t[i].call(this,o);return o}}))}function Ti(e,t,n,i,o,a,s,l,u,c){var d=128&t,h=1&t,f=2&t,p=24&t,g=512&t,v=f?void 0:wi(e);return function m(){for(var y=arguments.length,b=r(y),C=y;C--;)b[C]=arguments[C];if(p)var A=Zi(m),_=Nt(b,A);if(i&&(b=gi(b,i,o,p)),a&&(b=vi(b,a,s,p)),y-=_,p&&y<c){var I=Ut(b,A);return Ni(e,t,Ti,m.placeholder,n,b,I,l,u,c-y)}var x=h?n:this,w=f?x[e]:e;return y=b.length,l?b=mo(b,l):g&&y>1&&b.reverse(),d&&u<y&&(b.length=u),this&&this!==Xe&&this instanceof m&&(w=v||wi(w)),w.apply(x,b)}}function Ei(e,t){return function(n,r){return function(e,t,n,r){return lr(e,(function(e,i,o){t(r,n(e),i,o)})),r}(n,e,t(r),{})}}function Oi(e,t){return function(n,r){var i;if(void 0===n&&void 0===r)return t;if(void 0!==n&&(i=n),void 0!==r){if(void 0===i)return r;"string"==typeof n||"string"==typeof r?(n=Kr(n),r=Kr(r)):(n=Qr(n),r=Qr(r)),i=e(n,r)}return i}}function ki(e){return Yi((function(t){return t=ft(t,Ot(Qi())),zr((function(n){var r=this;return e(t,(function(e){return ot(e,r,n)}))}))}))}function Li(e,t){var n=(t=void 0===t?" ":Kr(t)).length;if(n<2)return n?Rr(t,e):t;var r=Rr(t,Jt(e/Wt(t)));return Ft(t)?ui(Gt(r),0,e).join(""):r.slice(0,e)}function Di(e){return function(t,n,i){return i&&"number"!=typeof i&&so(t,n,i)&&(n=i=void 0),t=ns(t),void 0===n?(n=t,t=0):n=ns(n),function(e,t,n,i){for(var o=-1,a=an(Jt((t-e)/(n||1)),0),s=r(a);a--;)s[i?a:++o]=e,e+=n;return s}(t,n,i=void 0===i?t<n?1:-1:ns(i),e)}}function Pi(e){return function(t,n){return"string"==typeof t&&"string"==typeof n||(t=os(t),n=os(n)),e(t,n)}}function Ni(e,t,n,r,i,o,a,s,l,u){var c=8&t;t|=c?32:64,4&(t&=~(c?64:32))||(t&=-4);var d=[e,t,i,c?o:void 0,c?a:void 0,c?void 0:o,c?void 0:a,s,l,u],h=n.apply(void 0,d);return uo(e)&&bo(h,d),h.placeholder=r,_o(h,e,t)}function ji(e){var t=pe[e];return function(e,n){if(e=os(e),(n=null==n?0:sn(rs(n),292))&&nn(e)){var r=(ss(e)+"e").split("e");return+((r=(ss(t(r[0]+"e"+(+r[1]+n)))+"e").split("e"))[0]+"e"+(+r[1]-n))}return t(e)}}var Ri=gn&&1/Vt(new gn([,-0]))[1]==1/0?function(e){return new gn(e)}:Ks;function zi(e){return function(t){var n=no(t);return n==g?Bt(t):n==b?Yt(t):function(e,t){return ft(t,(function(t){return[t,e[t]]}))}(t,e(t))}}function Fi(e,t,n,i,s,l,u,c){var d=2&t;if(!d&&"function"!=typeof e)throw new ye(o);var h=i?i.length:0;if(h||(t&=-97,i=s=void 0),u=void 0===u?u:an(rs(u),0),c=void 0===c?c:rs(c),h-=s?s.length:0,64&t){var f=i,p=s;i=s=void 0}var g=d?void 0:Xi(e),v=[e,t,n,i,s,f,p,l,u,c];if(g&&function(e,t){var n=e[1],r=t[1],i=n|r,o=i<131,s=128==r&&8==n||128==r&&256==n&&e[7].length<=t[8]||384==r&&t[7].length<=t[8]&&8==n;if(!o&&!s)return e;1&r&&(e[2]=t[2],i|=1&n?0:4);var l=t[3];if(l){var u=e[3];e[3]=u?gi(u,l,t[4]):l,e[4]=u?Ut(e[3],a):t[4]}(l=t[5])&&(u=e[5],e[5]=u?vi(u,l,t[6]):l,e[6]=u?Ut(e[5],a):t[6]);(l=t[7])&&(e[7]=l);128&r&&(e[8]=null==e[8]?t[8]:sn(e[8],t[8]));null==e[9]&&(e[9]=t[9]);e[0]=t[0],e[1]=i}(v,g),e=v[0],t=v[1],n=v[2],i=v[3],s=v[4],!(c=v[9]=void 0===v[9]?d?0:e.length:an(v[9]-h,0))&&24&t&&(t&=-25),t&&1!=t)m=8==t||16==t?function(e,t,n){var i=wi(e);return function o(){for(var a=arguments.length,s=r(a),l=a,u=Zi(o);l--;)s[l]=arguments[l];var c=a<3&&s[0]!==u&&s[a-1]!==u?[]:Ut(s,u);if((a-=c.length)<n)return Ni(e,t,Ti,o.placeholder,void 0,s,c,void 0,void 0,n-a);var d=this&&this!==Xe&&this instanceof o?i:e;return ot(d,this,s)}}(e,t,c):32!=t&&33!=t||s.length?Ti.apply(void 0,v):function(e,t,n,i){var o=1&t,a=wi(e);return function t(){for(var s=-1,l=arguments.length,u=-1,c=i.length,d=r(c+l),h=this&&this!==Xe&&this instanceof t?a:e;++u<c;)d[u]=i[u];for(;l--;)d[u++]=arguments[++s];return ot(h,o?n:this,d)}}(e,t,n,i);else var m=function(e,t,n){var r=1&t,i=wi(e);return function t(){var o=this&&this!==Xe&&this instanceof t?i:e;return o.apply(r?n:this,arguments)}}(e,t,n);return _o((g?Ur:bo)(m,v),e,t)}function Bi(e,t,n,r){return void 0===e||Ea(e,Ae[n])&&!xe.call(r,n)?t:e}function Hi(e,t,n,r,i,o){return Va(e)&&Va(t)&&(o.set(t,e),Or(e,t,void 0,Hi,o),o.delete(t)),e}function Ui(e){return Xa(e)?void 0:e}function Vi(e,t,n,r,i,o){var a=1&n,s=e.length,l=t.length;if(s!=l&&!(a&&l>s))return!1;var u=o.get(e);if(u&&o.get(t))return u==t;var c=-1,d=!0,h=2&n?new jn:void 0;for(o.set(e,t),o.set(t,e);++c<s;){var f=e[c],p=t[c];if(r)var g=a?r(p,f,c,t,e,o):r(f,p,c,e,t,o);if(void 0!==g){if(g)continue;d=!1;break}if(h){if(!mt(t,(function(e,t){if(!Lt(h,t)&&(f===e||i(f,e,n,r,o)))return h.push(t)}))){d=!1;break}}else if(f!==p&&!i(f,p,n,r,o)){d=!1;break}}return o.delete(e),o.delete(t),d}function Yi(e){return Ao(go(e,void 0,Po),e+"")}function Wi(e){return hr(e,Cs,eo)}function Gi(e){return hr(e,As,to)}var Xi=yn?function(e){return yn.get(e)}:Ks;function qi(e){for(var t=e.name+"",n=bn[t],r=xe.call(bn,t)?n.length:0;r--;){var i=n[r],o=i.func;if(null==o||o==e)return i.name}return t}function Zi(e){return(xe.call(Tn,"placeholder")?Tn:e).placeholder}function Qi(){var e=Tn.iteratee||Xs;return e=e===Xs?Ir:e,arguments.length?e(arguments[0],arguments[1]):e}function Ki(e,t){var n,r,i=e.__data__;return("string"==(r=typeof(n=t))||"number"==r||"symbol"==r||"boolean"==r?"__proto__"!==n:null===n)?i["string"==typeof t?"string":"hash"]:i.map}function Ji(e){for(var t=Cs(e),n=t.length;n--;){var r=t[n],i=e[r];t[n]=[r,i,fo(i)]}return t}function $i(e,t){var n=function(e,t){return null==e?void 0:e[t]}(e,t);return _r(n)?n:void 0}var eo=en?function(e){return null==e?[]:(e=ge(e),ct(en(e),(function(t){return qe.call(e,t)})))}:il,to=en?function(e){for(var t=[];e;)pt(t,eo(e)),e=We(e);return t}:il,no=fr;function ro(e,t,n){for(var r=-1,i=(t=si(t,e)).length,o=!1;++r<i;){var a=Mo(t[r]);if(!(o=null!=e&&n(e,a)))break;e=e[a]}return o||++r!=i?o:!!(i=null==e?0:e.length)&&Ua(i)&&ao(a,i)&&(Da(e)||La(e))}function io(e){return"function"!=typeof e.constructor||ho(e)?{}:En(We(e))}function oo(e){return Da(e)||La(e)||!!(Ke&&e&&e[Ke])}function ao(e,t){var n=typeof e;return!!(t=null==t?9007199254740991:t)&&("number"==n||"symbol"!=n&&le.test(e))&&e>-1&&e%1==0&&e<t}function so(e,t,n){if(!Va(n))return!1;var r=typeof t;return!!("number"==r?Na(n)&&ao(t,n.length):"string"==r&&t in n)&&Ea(n[t],e)}function lo(e,t){if(Da(e))return!1;var n=typeof e;return!("number"!=n&&"symbol"!=n&&"boolean"!=n&&null!=e&&!Ka(e))||(Y.test(e)||!V.test(e)||null!=t&&e in ge(t))}function uo(e){var t=qi(e),n=Tn[t];if("function"!=typeof n||!(t in Ln.prototype))return!1;if(e===n)return!0;var r=Xi(n);return!!r&&e===r[0]}(hn&&no(new hn(new ArrayBuffer(1)))!=x||fn&&no(new fn)!=g||pn&&"[object Promise]"!=no(pn.resolve())||gn&&no(new gn)!=b||vn&&no(new vn)!=_)&&(no=function(e){var t=fr(e),n=t==m?e.constructor:void 0,r=n?So(n):"";if(r)switch(r){case Cn:return x;case An:return g;case _n:return"[object Promise]";case In:return b;case xn:return _}return t});var co=_e?Ba:ol;function ho(e){var t=e&&e.constructor;return e===("function"==typeof t&&t.prototype||Ae)}function fo(e){return e==e&&!Va(e)}function po(e,t){return function(n){return null!=n&&(n[e]===t&&(void 0!==t||e in ge(n)))}}function go(e,t,n){return t=an(void 0===t?e.length-1:t,0),function(){for(var i=arguments,o=-1,a=an(i.length-t,0),s=r(a);++o<a;)s[o]=i[t+o];o=-1;for(var l=r(t+1);++o<t;)l[o]=i[o];return l[t]=n(s),ot(e,this,l)}}function vo(e,t){return t.length<2?e:dr(e,Wr(t,0,-1))}function mo(e,t){for(var n=e.length,r=sn(t.length,n),i=mi(e);r--;){var o=t[r];e[r]=ao(o,n)?i[o]:void 0}return e}function yo(e,t){if(("constructor"!==t||"function"!=typeof e[t])&&"__proto__"!=t)return e[t]}var bo=Io(Ur),Co=Kt||function(e,t){return Xe.setTimeout(e,t)},Ao=Io(Vr);function _o(e,t,n){var r=t+"";return Ao(e,function(e,t){var n=t.length;if(!n)return e;var r=n-1;return t[r]=(n>1?"& ":"")+t[r],t=t.join(n>2?", ":" "),e.replace(K,"{\n/* [wrapped with "+t+"] */\n")}(r,function(e,t){return st(s,(function(n){var r="_."+n[0];t&n[1]&&!dt(e,r)&&e.push(r)})),e.sort()}(function(e){var t=e.match(J);return t?t[1].split($):[]}(r),n)))}function Io(e){var t=0,n=0;return function(){var r=ln(),i=16-(r-n);if(n=r,i>0){if(++t>=800)return arguments[0]}else t=0;return e.apply(void 0,arguments)}}function xo(e,t){var n=-1,r=e.length,i=r-1;for(t=void 0===t?r:t;++n<t;){var o=jr(n,i),a=e[o];e[o]=e[n],e[n]=a}return e.length=t,e}var wo=function(e){var t=Ia(e,(function(e){return 500===n.size&&n.clear(),e})),n=t.cache;return t}((function(e){var t=[];return 46===e.charCodeAt(0)&&t.push(""),e.replace(W,(function(e,n,r,i){t.push(r?i.replace(te,"$1"):n||e)})),t}));function Mo(e){if("string"==typeof e||Ka(e))return e;var t=e+"";return"0"==t&&1/e==-1/0?"-0":t}function So(e){if(null!=e){try{return Ie.call(e)}catch(e){}try{return e+""}catch(e){}}return""}function To(e){if(e instanceof Ln)return e.clone();var t=new kn(e.__wrapped__,e.__chain__);return t.__actions__=mi(e.__actions__),t.__index__=e.__index__,t.__values__=e.__values__,t}var Eo=zr((function(e,t){return ja(e)?$n(e,or(t,1,ja,!0)):[]})),Oo=zr((function(e,t){var n=Fo(t);return ja(n)&&(n=void 0),ja(e)?$n(e,or(t,1,ja,!0),Qi(n,2)):[]})),ko=zr((function(e,t){var n=Fo(t);return ja(n)&&(n=void 0),ja(e)?$n(e,or(t,1,ja,!0),void 0,n):[]}));function Lo(e,t,n){var r=null==e?0:e.length;if(!r)return-1;var i=null==n?0:rs(n);return i<0&&(i=an(r+i,0)),Ct(e,Qi(t,3),i)}function Do(e,t,n){var r=null==e?0:e.length;if(!r)return-1;var i=r-1;return void 0!==n&&(i=rs(n),i=n<0?an(r+i,0):sn(i,r-1)),Ct(e,Qi(t,3),i,!0)}function Po(e){return(null==e?0:e.length)?or(e,1):[]}function No(e){return e&&e.length?e[0]:void 0}var jo=zr((function(e){var t=ft(e,oi);return t.length&&t[0]===e[0]?mr(t):[]})),Ro=zr((function(e){var t=Fo(e),n=ft(e,oi);return t===Fo(n)?t=void 0:n.pop(),n.length&&n[0]===e[0]?mr(n,Qi(t,2)):[]})),zo=zr((function(e){var t=Fo(e),n=ft(e,oi);return(t="function"==typeof t?t:void 0)&&n.pop(),n.length&&n[0]===e[0]?mr(n,void 0,t):[]}));function Fo(e){var t=null==e?0:e.length;return t?e[t-1]:void 0}var Bo=zr(Ho);function Ho(e,t){return e&&e.length&&t&&t.length?Pr(e,t):e}var Uo=Yi((function(e,t){var n=null==e?0:e.length,r=qn(e,t);return Nr(e,ft(t,(function(e){return ao(e,n)?+e:e})).sort(pi)),r}));function Vo(e){return null==e?e:dn.call(e)}var Yo=zr((function(e){return Jr(or(e,1,ja,!0))})),Wo=zr((function(e){var t=Fo(e);return ja(t)&&(t=void 0),Jr(or(e,1,ja,!0),Qi(t,2))})),Go=zr((function(e){var t=Fo(e);return t="function"==typeof t?t:void 0,Jr(or(e,1,ja,!0),void 0,t)}));function Xo(e){if(!e||!e.length)return[];var t=0;return e=ct(e,(function(e){if(ja(e))return t=an(e.length,t),!0})),Et(t,(function(t){return ft(e,wt(t))}))}function qo(e,t){if(!e||!e.length)return[];var n=Xo(e);return null==t?n:ft(n,(function(e){return ot(t,void 0,e)}))}var Zo=zr((function(e,t){return ja(e)?$n(e,t):[]})),Qo=zr((function(e){return ri(ct(e,ja))})),Ko=zr((function(e){var t=Fo(e);return ja(t)&&(t=void 0),ri(ct(e,ja),Qi(t,2))})),Jo=zr((function(e){var t=Fo(e);return t="function"==typeof t?t:void 0,ri(ct(e,ja),void 0,t)})),$o=zr(Xo);var ea=zr((function(e){var t=e.length,n=t>1?e[t-1]:void 0;return n="function"==typeof n?(e.pop(),n):void 0,qo(e,n)}));function ta(e){var t=Tn(e);return t.__chain__=!0,t}function na(e,t){return t(e)}var ra=Yi((function(e){var t=e.length,n=t?e[0]:0,r=this.__wrapped__,i=function(t){return qn(t,e)};return!(t>1||this.__actions__.length)&&r instanceof Ln&&ao(n)?((r=r.slice(n,+n+(t?1:0))).__actions__.push({func:na,args:[i],thisArg:void 0}),new kn(r,this.__chain__).thru((function(e){return t&&!e.length&&e.push(void 0),e}))):this.thru(i)}));var ia=bi((function(e,t,n){xe.call(e,n)?++e[n]:Xn(e,n,1)}));var oa=Mi(Lo),aa=Mi(Do);function sa(e,t){return(Da(e)?st:er)(e,Qi(t,3))}function la(e,t){return(Da(e)?lt:tr)(e,Qi(t,3))}var ua=bi((function(e,t,n){xe.call(e,n)?e[n].push(t):Xn(e,n,[t])}));var ca=zr((function(e,t,n){var i=-1,o="function"==typeof t,a=Na(e)?r(e.length):[];return er(e,(function(e){a[++i]=o?ot(t,e,n):yr(e,t,n)})),a})),da=bi((function(e,t,n){Xn(e,n,t)}));function ha(e,t){return(Da(e)?ft:Sr)(e,Qi(t,3))}var fa=bi((function(e,t,n){e[n?0:1].push(t)}),(function(){return[[],[]]}));var pa=zr((function(e,t){if(null==e)return[];var n=t.length;return n>1&&so(e,t[0],t[1])?t=[]:n>2&&so(t[0],t[1],t[2])&&(t=[t[0]]),Lr(e,or(t,1),[])})),ga=Qt||function(){return Xe.Date.now()};function va(e,t,n){return t=n?void 0:t,Fi(e,128,void 0,void 0,void 0,void 0,t=e&&null==t?e.length:t)}function ma(e,t){var n;if("function"!=typeof t)throw new ye(o);return e=rs(e),function(){return--e>0&&(n=t.apply(this,arguments)),e<=1&&(t=void 0),n}}var ya=zr((function(e,t,n){var r=1;if(n.length){var i=Ut(n,Zi(ya));r|=32}return Fi(e,r,t,n,i)})),ba=zr((function(e,t,n){var r=3;if(n.length){var i=Ut(n,Zi(ba));r|=32}return Fi(t,r,e,n,i)}));function Ca(e,t,n){var r,i,a,s,l,u,c=0,d=!1,h=!1,f=!0;if("function"!=typeof e)throw new ye(o);function p(t){var n=r,o=i;return r=i=void 0,c=t,s=e.apply(o,n)}function g(e){return c=e,l=Co(m,t),d?p(e):s}function v(e){var n=e-u;return void 0===u||n>=t||n<0||h&&e-c>=a}function m(){var e=ga();if(v(e))return y(e);l=Co(m,function(e){var n=t-(e-u);return h?sn(n,a-(e-c)):n}(e))}function y(e){return l=void 0,f&&r?p(e):(r=i=void 0,s)}function b(){var e=ga(),n=v(e);if(r=arguments,i=this,u=e,n){if(void 0===l)return g(u);if(h)return ci(l),l=Co(m,t),p(u)}return void 0===l&&(l=Co(m,t)),s}return t=os(t)||0,Va(n)&&(d=!!n.leading,a=(h="maxWait"in n)?an(os(n.maxWait)||0,t):a,f="trailing"in n?!!n.trailing:f),b.cancel=function(){void 0!==l&&ci(l),c=0,r=u=i=l=void 0},b.flush=function(){return void 0===l?s:y(ga())},b}var Aa=zr((function(e,t){return Jn(e,1,t)})),_a=zr((function(e,t,n){return Jn(e,os(t)||0,n)}));function Ia(e,t){if("function"!=typeof e||null!=t&&"function"!=typeof t)throw new ye(o);var n=function(){var r=arguments,i=t?t.apply(this,r):r[0],o=n.cache;if(o.has(i))return o.get(i);var a=e.apply(this,r);return n.cache=o.set(i,a)||o,a};return n.cache=new(Ia.Cache||Nn),n}function xa(e){if("function"!=typeof e)throw new ye(o);return function(){var t=arguments;switch(t.length){case 0:return!e.call(this);case 1:return!e.call(this,t[0]);case 2:return!e.call(this,t[0],t[1]);case 3:return!e.call(this,t[0],t[1],t[2])}return!e.apply(this,t)}}Ia.Cache=Nn;var wa=li((function(e,t){var n=(t=1==t.length&&Da(t[0])?ft(t[0],Ot(Qi())):ft(or(t,1),Ot(Qi()))).length;return zr((function(r){for(var i=-1,o=sn(r.length,n);++i<o;)r[i]=t[i].call(this,r[i]);return ot(e,this,r)}))})),Ma=zr((function(e,t){return Fi(e,32,void 0,t,Ut(t,Zi(Ma)))})),Sa=zr((function(e,t){return Fi(e,64,void 0,t,Ut(t,Zi(Sa)))})),Ta=Yi((function(e,t){return Fi(e,256,void 0,void 0,void 0,t)}));function Ea(e,t){return e===t||e!=e&&t!=t}var Oa=Pi(pr),ka=Pi((function(e,t){return e>=t})),La=br(function(){return arguments}())?br:function(e){return Ya(e)&&xe.call(e,"callee")&&!qe.call(e,"callee")},Da=r.isArray,Pa=$e?Ot($e):function(e){return Ya(e)&&fr(e)==I};function Na(e){return null!=e&&Ua(e.length)&&!Ba(e)}function ja(e){return Ya(e)&&Na(e)}var Ra=tn||ol,za=et?Ot(et):function(e){return Ya(e)&&fr(e)==d};function Fa(e){if(!Ya(e))return!1;var t=fr(e);return t==h||"[object DOMException]"==t||"string"==typeof e.message&&"string"==typeof e.name&&!Xa(e)}function Ba(e){if(!Va(e))return!1;var t=fr(e);return t==f||t==p||"[object AsyncFunction]"==t||"[object Proxy]"==t}function Ha(e){return"number"==typeof e&&e==rs(e)}function Ua(e){return"number"==typeof e&&e>-1&&e%1==0&&e<=9007199254740991}function Va(e){var t=typeof e;return null!=e&&("object"==t||"function"==t)}function Ya(e){return null!=e&&"object"==typeof e}var Wa=tt?Ot(tt):function(e){return Ya(e)&&no(e)==g};function Ga(e){return"number"==typeof e||Ya(e)&&fr(e)==v}function Xa(e){if(!Ya(e)||fr(e)!=m)return!1;var t=We(e);if(null===t)return!0;var n=xe.call(t,"constructor")&&t.constructor;return"function"==typeof n&&n instanceof n&&Ie.call(n)==Te}var qa=nt?Ot(nt):function(e){return Ya(e)&&fr(e)==y};var Za=rt?Ot(rt):function(e){return Ya(e)&&no(e)==b};function Qa(e){return"string"==typeof e||!Da(e)&&Ya(e)&&fr(e)==C}function Ka(e){return"symbol"==typeof e||Ya(e)&&fr(e)==A}var Ja=it?Ot(it):function(e){return Ya(e)&&Ua(e.length)&&!!Be[fr(e)]};var $a=Pi(Mr),es=Pi((function(e,t){return e<=t}));function ts(e){if(!e)return[];if(Na(e))return Qa(e)?Gt(e):mi(e);if(Je&&e[Je])return function(e){for(var t,n=[];!(t=e.next()).done;)n.push(t.value);return n}(e[Je]());var t=no(e);return(t==g?Bt:t==b?Vt:Es)(e)}function ns(e){return e?(e=os(e))===1/0||e===-1/0?17976931348623157e292*(e<0?-1:1):e==e?e:0:0===e?e:0}function rs(e){var t=ns(e),n=t%1;return t==t?n?t-n:t:0}function is(e){return e?Zn(rs(e),0,4294967295):0}function os(e){if("number"==typeof e)return e;if(Ka(e))return NaN;if(Va(e)){var t="function"==typeof e.valueOf?e.valueOf():e;e=Va(t)?t+"":t}if("string"!=typeof e)return 0===e?e:+e;e=e.replace(q,"");var n=oe.test(e);return n||se.test(e)?Ye(e.slice(2),n?2:8):ie.test(e)?NaN:+e}function as(e){return yi(e,As(e))}function ss(e){return null==e?"":Kr(e)}var ls=Ci((function(e,t){if(ho(t)||Na(t))yi(t,Cs(t),e);else for(var n in t)xe.call(t,n)&&Vn(e,n,t[n])})),us=Ci((function(e,t){yi(t,As(t),e)})),cs=Ci((function(e,t,n,r){yi(t,As(t),e,r)})),ds=Ci((function(e,t,n,r){yi(t,Cs(t),e,r)})),hs=Yi(qn);var fs=zr((function(e,t){e=ge(e);var n=-1,r=t.length,i=r>2?t[2]:void 0;for(i&&so(t[0],t[1],i)&&(r=1);++n<r;)for(var o=t[n],a=As(o),s=-1,l=a.length;++s<l;){var u=a[s],c=e[u];(void 0===c||Ea(c,Ae[u])&&!xe.call(e,u))&&(e[u]=o[u])}return e})),ps=zr((function(e){return e.push(void 0,Hi),ot(Is,void 0,e)}));function gs(e,t,n){var r=null==e?void 0:dr(e,t);return void 0===r?n:r}function vs(e,t){return null!=e&&ro(e,t,vr)}var ms=Ei((function(e,t,n){null!=t&&"function"!=typeof t.toString&&(t=Se.call(t)),e[t]=n}),Vs(Gs)),ys=Ei((function(e,t,n){null!=t&&"function"!=typeof t.toString&&(t=Se.call(t)),xe.call(e,t)?e[t].push(n):e[t]=[n]}),Qi),bs=zr(yr);function Cs(e){return Na(e)?zn(e):xr(e)}function As(e){return Na(e)?zn(e,!0):wr(e)}var _s=Ci((function(e,t,n){Or(e,t,n)})),Is=Ci((function(e,t,n,r){Or(e,t,n,r)})),xs=Yi((function(e,t){var n={};if(null==e)return n;var r=!1;t=ft(t,(function(t){return t=si(t,e),r||(r=t.length>1),t})),yi(e,Gi(e),n),r&&(n=Qn(n,7,Ui));for(var i=t.length;i--;)$r(n,t[i]);return n}));var ws=Yi((function(e,t){return null==e?{}:function(e,t){return Dr(e,t,(function(t,n){return vs(e,n)}))}(e,t)}));function Ms(e,t){if(null==e)return{};var n=ft(Gi(e),(function(e){return[e]}));return t=Qi(t),Dr(e,n,(function(e,n){return t(e,n[0])}))}var Ss=zi(Cs),Ts=zi(As);function Es(e){return null==e?[]:kt(e,Cs(e))}var Os=xi((function(e,t,n){return t=t.toLowerCase(),e+(n?ks(t):t)}));function ks(e){return Fs(ss(e).toLowerCase())}function Ls(e){return(e=ss(e))&&e.replace(ue,jt).replace(De,"")}var Ds=xi((function(e,t,n){return e+(n?"-":"")+t.toLowerCase()})),Ps=xi((function(e,t,n){return e+(n?" ":"")+t.toLowerCase()})),Ns=Ii("toLowerCase");var js=xi((function(e,t,n){return e+(n?"_":"")+t.toLowerCase()}));var Rs=xi((function(e,t,n){return e+(n?" ":"")+Fs(t)}));var zs=xi((function(e,t,n){return e+(n?" ":"")+t.toUpperCase()})),Fs=Ii("toUpperCase");function Bs(e,t,n){return e=ss(e),void 0===(t=n?void 0:t)?function(e){return Re.test(e)}(e)?function(e){return e.match(Ne)||[]}(e):function(e){return e.match(ee)||[]}(e):e.match(t)||[]}var Hs=zr((function(e,t){try{return ot(e,void 0,t)}catch(e){return Fa(e)?e:new he(e)}})),Us=Yi((function(e,t){return st(t,(function(t){t=Mo(t),Xn(e,t,ya(e[t],e))})),e}));function Vs(e){return function(){return e}}var Ys=Si(),Ws=Si(!0);function Gs(e){return e}function Xs(e){return Ir("function"==typeof e?e:Qn(e,1))}var qs=zr((function(e,t){return function(n){return yr(n,e,t)}})),Zs=zr((function(e,t){return function(n){return yr(e,n,t)}}));function Qs(e,t,n){var r=Cs(t),i=cr(t,r);null!=n||Va(t)&&(i.length||!r.length)||(n=t,t=e,e=this,i=cr(t,Cs(t)));var o=!(Va(n)&&"chain"in n&&!n.chain),a=Ba(e);return st(i,(function(n){var r=t[n];e[n]=r,a&&(e.prototype[n]=function(){var t=this.__chain__;if(o||t){var n=e(this.__wrapped__),i=n.__actions__=mi(this.__actions__);return i.push({func:r,args:arguments,thisArg:e}),n.__chain__=t,n}return r.apply(e,pt([this.value()],arguments))})})),e}function Ks(){}var Js=ki(ft),$s=ki(ut),el=ki(mt);function tl(e){return lo(e)?wt(Mo(e)):function(e){return function(t){return dr(t,e)}}(e)}var nl=Di(),rl=Di(!0);function il(){return[]}function ol(){return!1}var al=Oi((function(e,t){return e+t}),0),sl=ji("ceil"),ll=Oi((function(e,t){return e/t}),1),ul=ji("floor");var cl,dl=Oi((function(e,t){return e*t}),1),hl=ji("round"),fl=Oi((function(e,t){return e-t}),0);return Tn.after=function(e,t){if("function"!=typeof t)throw new ye(o);return e=rs(e),function(){if(--e<1)return t.apply(this,arguments)}},Tn.ary=va,Tn.assign=ls,Tn.assignIn=us,Tn.assignInWith=cs,Tn.assignWith=ds,Tn.at=hs,Tn.before=ma,Tn.bind=ya,Tn.bindAll=Us,Tn.bindKey=ba,Tn.castArray=function(){if(!arguments.length)return[];var e=arguments[0];return Da(e)?e:[e]},Tn.chain=ta,Tn.chunk=function(e,t,n){t=(n?so(e,t,n):void 0===t)?1:an(rs(t),0);var i=null==e?0:e.length;if(!i||t<1)return[];for(var o=0,a=0,s=r(Jt(i/t));o<i;)s[a++]=Wr(e,o,o+=t);return s},Tn.compact=function(e){for(var t=-1,n=null==e?0:e.length,r=0,i=[];++t<n;){var o=e[t];o&&(i[r++]=o)}return i},Tn.concat=function(){var e=arguments.length;if(!e)return[];for(var t=r(e-1),n=arguments[0],i=e;i--;)t[i-1]=arguments[i];return pt(Da(n)?mi(n):[n],or(t,1))},Tn.cond=function(e){var t=null==e?0:e.length,n=Qi();return e=t?ft(e,(function(e){if("function"!=typeof e[1])throw new ye(o);return[n(e[0]),e[1]]})):[],zr((function(n){for(var r=-1;++r<t;){var i=e[r];if(ot(i[0],this,n))return ot(i[1],this,n)}}))},Tn.conforms=function(e){return function(e){var t=Cs(e);return function(n){return Kn(n,e,t)}}(Qn(e,1))},Tn.constant=Vs,Tn.countBy=ia,Tn.create=function(e,t){var n=En(e);return null==t?n:Gn(n,t)},Tn.curry=function e(t,n,r){var i=Fi(t,8,void 0,void 0,void 0,void 0,void 0,n=r?void 0:n);return i.placeholder=e.placeholder,i},Tn.curryRight=function e(t,n,r){var i=Fi(t,16,void 0,void 0,void 0,void 0,void 0,n=r?void 0:n);return i.placeholder=e.placeholder,i},Tn.debounce=Ca,Tn.defaults=fs,Tn.defaultsDeep=ps,Tn.defer=Aa,Tn.delay=_a,Tn.difference=Eo,Tn.differenceBy=Oo,Tn.differenceWith=ko,Tn.drop=function(e,t,n){var r=null==e?0:e.length;return r?Wr(e,(t=n||void 0===t?1:rs(t))<0?0:t,r):[]},Tn.dropRight=function(e,t,n){var r=null==e?0:e.length;return r?Wr(e,0,(t=r-(t=n||void 0===t?1:rs(t)))<0?0:t):[]},Tn.dropRightWhile=function(e,t){return e&&e.length?ti(e,Qi(t,3),!0,!0):[]},Tn.dropWhile=function(e,t){return e&&e.length?ti(e,Qi(t,3),!0):[]},Tn.fill=function(e,t,n,r){var i=null==e?0:e.length;return i?(n&&"number"!=typeof n&&so(e,t,n)&&(n=0,r=i),function(e,t,n,r){var i=e.length;for((n=rs(n))<0&&(n=-n>i?0:i+n),(r=void 0===r||r>i?i:rs(r))<0&&(r+=i),r=n>r?0:is(r);n<r;)e[n++]=t;return e}(e,t,n,r)):[]},Tn.filter=function(e,t){return(Da(e)?ct:ir)(e,Qi(t,3))},Tn.flatMap=function(e,t){return or(ha(e,t),1)},Tn.flatMapDeep=function(e,t){return or(ha(e,t),1/0)},Tn.flatMapDepth=function(e,t,n){return n=void 0===n?1:rs(n),or(ha(e,t),n)},Tn.flatten=Po,Tn.flattenDeep=function(e){return(null==e?0:e.length)?or(e,1/0):[]},Tn.flattenDepth=function(e,t){return(null==e?0:e.length)?or(e,t=void 0===t?1:rs(t)):[]},Tn.flip=function(e){return Fi(e,512)},Tn.flow=Ys,Tn.flowRight=Ws,Tn.fromPairs=function(e){for(var t=-1,n=null==e?0:e.length,r={};++t<n;){var i=e[t];r[i[0]]=i[1]}return r},Tn.functions=function(e){return null==e?[]:cr(e,Cs(e))},Tn.functionsIn=function(e){return null==e?[]:cr(e,As(e))},Tn.groupBy=ua,Tn.initial=function(e){return(null==e?0:e.length)?Wr(e,0,-1):[]},Tn.intersection=jo,Tn.intersectionBy=Ro,Tn.intersectionWith=zo,Tn.invert=ms,Tn.invertBy=ys,Tn.invokeMap=ca,Tn.iteratee=Xs,Tn.keyBy=da,Tn.keys=Cs,Tn.keysIn=As,Tn.map=ha,Tn.mapKeys=function(e,t){var n={};return t=Qi(t,3),lr(e,(function(e,r,i){Xn(n,t(e,r,i),e)})),n},Tn.mapValues=function(e,t){var n={};return t=Qi(t,3),lr(e,(function(e,r,i){Xn(n,r,t(e,r,i))})),n},Tn.matches=function(e){return Tr(Qn(e,1))},Tn.matchesProperty=function(e,t){return Er(e,Qn(t,1))},Tn.memoize=Ia,Tn.merge=_s,Tn.mergeWith=Is,Tn.method=qs,Tn.methodOf=Zs,Tn.mixin=Qs,Tn.negate=xa,Tn.nthArg=function(e){return e=rs(e),zr((function(t){return kr(t,e)}))},Tn.omit=xs,Tn.omitBy=function(e,t){return Ms(e,xa(Qi(t)))},Tn.once=function(e){return ma(2,e)},Tn.orderBy=function(e,t,n,r){return null==e?[]:(Da(t)||(t=null==t?[]:[t]),Da(n=r?void 0:n)||(n=null==n?[]:[n]),Lr(e,t,n))},Tn.over=Js,Tn.overArgs=wa,Tn.overEvery=$s,Tn.overSome=el,Tn.partial=Ma,Tn.partialRight=Sa,Tn.partition=fa,Tn.pick=ws,Tn.pickBy=Ms,Tn.property=tl,Tn.propertyOf=function(e){return function(t){return null==e?void 0:dr(e,t)}},Tn.pull=Bo,Tn.pullAll=Ho,Tn.pullAllBy=function(e,t,n){return e&&e.length&&t&&t.length?Pr(e,t,Qi(n,2)):e},Tn.pullAllWith=function(e,t,n){return e&&e.length&&t&&t.length?Pr(e,t,void 0,n):e},Tn.pullAt=Uo,Tn.range=nl,Tn.rangeRight=rl,Tn.rearg=Ta,Tn.reject=function(e,t){return(Da(e)?ct:ir)(e,xa(Qi(t,3)))},Tn.remove=function(e,t){var n=[];if(!e||!e.length)return n;var r=-1,i=[],o=e.length;for(t=Qi(t,3);++r<o;){var a=e[r];t(a,r,e)&&(n.push(a),i.push(r))}return Nr(e,i),n},Tn.rest=function(e,t){if("function"!=typeof e)throw new ye(o);return zr(e,t=void 0===t?t:rs(t))},Tn.reverse=Vo,Tn.sampleSize=function(e,t,n){return t=(n?so(e,t,n):void 0===t)?1:rs(t),(Da(e)?Bn:Br)(e,t)},Tn.set=function(e,t,n){return null==e?e:Hr(e,t,n)},Tn.setWith=function(e,t,n,r){return r="function"==typeof r?r:void 0,null==e?e:Hr(e,t,n,r)},Tn.shuffle=function(e){return(Da(e)?Hn:Yr)(e)},Tn.slice=function(e,t,n){var r=null==e?0:e.length;return r?(n&&"number"!=typeof n&&so(e,t,n)?(t=0,n=r):(t=null==t?0:rs(t),n=void 0===n?r:rs(n)),Wr(e,t,n)):[]},Tn.sortBy=pa,Tn.sortedUniq=function(e){return e&&e.length?Zr(e):[]},Tn.sortedUniqBy=function(e,t){return e&&e.length?Zr(e,Qi(t,2)):[]},Tn.split=function(e,t,n){return n&&"number"!=typeof n&&so(e,t,n)&&(t=n=void 0),(n=void 0===n?4294967295:n>>>0)?(e=ss(e))&&("string"==typeof t||null!=t&&!qa(t))&&!(t=Kr(t))&&Ft(e)?ui(Gt(e),0,n):e.split(t,n):[]},Tn.spread=function(e,t){if("function"!=typeof e)throw new ye(o);return t=null==t?0:an(rs(t),0),zr((function(n){var r=n[t],i=ui(n,0,t);return r&&pt(i,r),ot(e,this,i)}))},Tn.tail=function(e){var t=null==e?0:e.length;return t?Wr(e,1,t):[]},Tn.take=function(e,t,n){return e&&e.length?Wr(e,0,(t=n||void 0===t?1:rs(t))<0?0:t):[]},Tn.takeRight=function(e,t,n){var r=null==e?0:e.length;return r?Wr(e,(t=r-(t=n||void 0===t?1:rs(t)))<0?0:t,r):[]},Tn.takeRightWhile=function(e,t){return e&&e.length?ti(e,Qi(t,3),!1,!0):[]},Tn.takeWhile=function(e,t){return e&&e.length?ti(e,Qi(t,3)):[]},Tn.tap=function(e,t){return t(e),e},Tn.throttle=function(e,t,n){var r=!0,i=!0;if("function"!=typeof e)throw new ye(o);return Va(n)&&(r="leading"in n?!!n.leading:r,i="trailing"in n?!!n.trailing:i),Ca(e,t,{leading:r,maxWait:t,trailing:i})},Tn.thru=na,Tn.toArray=ts,Tn.toPairs=Ss,Tn.toPairsIn=Ts,Tn.toPath=function(e){return Da(e)?ft(e,Mo):Ka(e)?[e]:mi(wo(ss(e)))},Tn.toPlainObject=as,Tn.transform=function(e,t,n){var r=Da(e),i=r||Ra(e)||Ja(e);if(t=Qi(t,4),null==n){var o=e&&e.constructor;n=i?r?new o:[]:Va(e)&&Ba(o)?En(We(e)):{}}return(i?st:lr)(e,(function(e,r,i){return t(n,e,r,i)})),n},Tn.unary=function(e){return va(e,1)},Tn.union=Yo,Tn.unionBy=Wo,Tn.unionWith=Go,Tn.uniq=function(e){return e&&e.length?Jr(e):[]},Tn.uniqBy=function(e,t){return e&&e.length?Jr(e,Qi(t,2)):[]},Tn.uniqWith=function(e,t){return t="function"==typeof t?t:void 0,e&&e.length?Jr(e,void 0,t):[]},Tn.unset=function(e,t){return null==e||$r(e,t)},Tn.unzip=Xo,Tn.unzipWith=qo,Tn.update=function(e,t,n){return null==e?e:ei(e,t,ai(n))},Tn.updateWith=function(e,t,n,r){return r="function"==typeof r?r:void 0,null==e?e:ei(e,t,ai(n),r)},Tn.values=Es,Tn.valuesIn=function(e){return null==e?[]:kt(e,As(e))},Tn.without=Zo,Tn.words=Bs,Tn.wrap=function(e,t){return Ma(ai(t),e)},Tn.xor=Qo,Tn.xorBy=Ko,Tn.xorWith=Jo,Tn.zip=$o,Tn.zipObject=function(e,t){return ii(e||[],t||[],Vn)},Tn.zipObjectDeep=function(e,t){return ii(e||[],t||[],Hr)},Tn.zipWith=ea,Tn.entries=Ss,Tn.entriesIn=Ts,Tn.extend=us,Tn.extendWith=cs,Qs(Tn,Tn),Tn.add=al,Tn.attempt=Hs,Tn.camelCase=Os,Tn.capitalize=ks,Tn.ceil=sl,Tn.clamp=function(e,t,n){return void 0===n&&(n=t,t=void 0),void 0!==n&&(n=(n=os(n))==n?n:0),void 0!==t&&(t=(t=os(t))==t?t:0),Zn(os(e),t,n)},Tn.clone=function(e){return Qn(e,4)},Tn.cloneDeep=function(e){return Qn(e,5)},Tn.cloneDeepWith=function(e,t){return Qn(e,5,t="function"==typeof t?t:void 0)},Tn.cloneWith=function(e,t){return Qn(e,4,t="function"==typeof t?t:void 0)},Tn.conformsTo=function(e,t){return null==t||Kn(e,t,Cs(t))},Tn.deburr=Ls,Tn.defaultTo=function(e,t){return null==e||e!=e?t:e},Tn.divide=ll,Tn.endsWith=function(e,t,n){e=ss(e),t=Kr(t);var r=e.length,i=n=void 0===n?r:Zn(rs(n),0,r);return(n-=t.length)>=0&&e.slice(n,i)==t},Tn.eq=Ea,Tn.escape=function(e){return(e=ss(e))&&F.test(e)?e.replace(R,Rt):e},Tn.escapeRegExp=function(e){return(e=ss(e))&&X.test(e)?e.replace(G,"\\$&"):e},Tn.every=function(e,t,n){var r=Da(e)?ut:nr;return n&&so(e,t,n)&&(t=void 0),r(e,Qi(t,3))},Tn.find=oa,Tn.findIndex=Lo,Tn.findKey=function(e,t){return bt(e,Qi(t,3),lr)},Tn.findLast=aa,Tn.findLastIndex=Do,Tn.findLastKey=function(e,t){return bt(e,Qi(t,3),ur)},Tn.floor=ul,Tn.forEach=sa,Tn.forEachRight=la,Tn.forIn=function(e,t){return null==e?e:ar(e,Qi(t,3),As)},Tn.forInRight=function(e,t){return null==e?e:sr(e,Qi(t,3),As)},Tn.forOwn=function(e,t){return e&&lr(e,Qi(t,3))},Tn.forOwnRight=function(e,t){return e&&ur(e,Qi(t,3))},Tn.get=gs,Tn.gt=Oa,Tn.gte=ka,Tn.has=function(e,t){return null!=e&&ro(e,t,gr)},Tn.hasIn=vs,Tn.head=No,Tn.identity=Gs,Tn.includes=function(e,t,n,r){e=Na(e)?e:Es(e),n=n&&!r?rs(n):0;var i=e.length;return n<0&&(n=an(i+n,0)),Qa(e)?n<=i&&e.indexOf(t,n)>-1:!!i&&At(e,t,n)>-1},Tn.indexOf=function(e,t,n){var r=null==e?0:e.length;if(!r)return-1;var i=null==n?0:rs(n);return i<0&&(i=an(r+i,0)),At(e,t,i)},Tn.inRange=function(e,t,n){return t=ns(t),void 0===n?(n=t,t=0):n=ns(n),function(e,t,n){return e>=sn(t,n)&&e<an(t,n)}(e=os(e),t,n)},Tn.invoke=bs,Tn.isArguments=La,Tn.isArray=Da,Tn.isArrayBuffer=Pa,Tn.isArrayLike=Na,Tn.isArrayLikeObject=ja,Tn.isBoolean=function(e){return!0===e||!1===e||Ya(e)&&fr(e)==c},Tn.isBuffer=Ra,Tn.isDate=za,Tn.isElement=function(e){return Ya(e)&&1===e.nodeType&&!Xa(e)},Tn.isEmpty=function(e){if(null==e)return!0;if(Na(e)&&(Da(e)||"string"==typeof e||"function"==typeof e.splice||Ra(e)||Ja(e)||La(e)))return!e.length;var t=no(e);if(t==g||t==b)return!e.size;if(ho(e))return!xr(e).length;for(var n in e)if(xe.call(e,n))return!1;return!0},Tn.isEqual=function(e,t){return Cr(e,t)},Tn.isEqualWith=function(e,t,n){var r=(n="function"==typeof n?n:void 0)?n(e,t):void 0;return void 0===r?Cr(e,t,void 0,n):!!r},Tn.isError=Fa,Tn.isFinite=function(e){return"number"==typeof e&&nn(e)},Tn.isFunction=Ba,Tn.isInteger=Ha,Tn.isLength=Ua,Tn.isMap=Wa,Tn.isMatch=function(e,t){return e===t||Ar(e,t,Ji(t))},Tn.isMatchWith=function(e,t,n){return n="function"==typeof n?n:void 0,Ar(e,t,Ji(t),n)},Tn.isNaN=function(e){return Ga(e)&&e!=+e},Tn.isNative=function(e){if(co(e))throw new he("Unsupported core-js use. Try https://npms.io/search?q=ponyfill.");return _r(e)},Tn.isNil=function(e){return null==e},Tn.isNull=function(e){return null===e},Tn.isNumber=Ga,Tn.isObject=Va,Tn.isObjectLike=Ya,Tn.isPlainObject=Xa,Tn.isRegExp=qa,Tn.isSafeInteger=function(e){return Ha(e)&&e>=-9007199254740991&&e<=9007199254740991},Tn.isSet=Za,Tn.isString=Qa,Tn.isSymbol=Ka,Tn.isTypedArray=Ja,Tn.isUndefined=function(e){return void 0===e},Tn.isWeakMap=function(e){return Ya(e)&&no(e)==_},Tn.isWeakSet=function(e){return Ya(e)&&"[object WeakSet]"==fr(e)},Tn.join=function(e,t){return null==e?"":rn.call(e,t)},Tn.kebabCase=Ds,Tn.last=Fo,Tn.lastIndexOf=function(e,t,n){var r=null==e?0:e.length;if(!r)return-1;var i=r;return void 0!==n&&(i=(i=rs(n))<0?an(r+i,0):sn(i,r-1)),t==t?function(e,t,n){for(var r=n+1;r--;)if(e[r]===t)return r;return r}(e,t,i):Ct(e,It,i,!0)},Tn.lowerCase=Ps,Tn.lowerFirst=Ns,Tn.lt=$a,Tn.lte=es,Tn.max=function(e){return e&&e.length?rr(e,Gs,pr):void 0},Tn.maxBy=function(e,t){return e&&e.length?rr(e,Qi(t,2),pr):void 0},Tn.mean=function(e){return xt(e,Gs)},Tn.meanBy=function(e,t){return xt(e,Qi(t,2))},Tn.min=function(e){return e&&e.length?rr(e,Gs,Mr):void 0},Tn.minBy=function(e,t){return e&&e.length?rr(e,Qi(t,2),Mr):void 0},Tn.stubArray=il,Tn.stubFalse=ol,Tn.stubObject=function(){return{}},Tn.stubString=function(){return""},Tn.stubTrue=function(){return!0},Tn.multiply=dl,Tn.nth=function(e,t){return e&&e.length?kr(e,rs(t)):void 0},Tn.noConflict=function(){return Xe._===this&&(Xe._=Ee),this},Tn.noop=Ks,Tn.now=ga,Tn.pad=function(e,t,n){e=ss(e);var r=(t=rs(t))?Wt(e):0;if(!t||r>=t)return e;var i=(t-r)/2;return Li($t(i),n)+e+Li(Jt(i),n)},Tn.padEnd=function(e,t,n){e=ss(e);var r=(t=rs(t))?Wt(e):0;return t&&r<t?e+Li(t-r,n):e},Tn.padStart=function(e,t,n){e=ss(e);var r=(t=rs(t))?Wt(e):0;return t&&r<t?Li(t-r,n)+e:e},Tn.parseInt=function(e,t,n){return n||null==t?t=0:t&&(t=+t),un(ss(e).replace(Z,""),t||0)},Tn.random=function(e,t,n){if(n&&"boolean"!=typeof n&&so(e,t,n)&&(t=n=void 0),void 0===n&&("boolean"==typeof t?(n=t,t=void 0):"boolean"==typeof e&&(n=e,e=void 0)),void 0===e&&void 0===t?(e=0,t=1):(e=ns(e),void 0===t?(t=e,e=0):t=ns(t)),e>t){var r=e;e=t,t=r}if(n||e%1||t%1){var i=cn();return sn(e+i*(t-e+Ve("1e-"+((i+"").length-1))),t)}return jr(e,t)},Tn.reduce=function(e,t,n){var r=Da(e)?gt:St,i=arguments.length<3;return r(e,Qi(t,4),n,i,er)},Tn.reduceRight=function(e,t,n){var r=Da(e)?vt:St,i=arguments.length<3;return r(e,Qi(t,4),n,i,tr)},Tn.repeat=function(e,t,n){return t=(n?so(e,t,n):void 0===t)?1:rs(t),Rr(ss(e),t)},Tn.replace=function(){var e=arguments,t=ss(e[0]);return e.length<3?t:t.replace(e[1],e[2])},Tn.result=function(e,t,n){var r=-1,i=(t=si(t,e)).length;for(i||(i=1,e=void 0);++r<i;){var o=null==e?void 0:e[Mo(t[r])];void 0===o&&(r=i,o=n),e=Ba(o)?o.call(e):o}return e},Tn.round=hl,Tn.runInContext=e,Tn.sample=function(e){return(Da(e)?Fn:Fr)(e)},Tn.size=function(e){if(null==e)return 0;if(Na(e))return Qa(e)?Wt(e):e.length;var t=no(e);return t==g||t==b?e.size:xr(e).length},Tn.snakeCase=js,Tn.some=function(e,t,n){var r=Da(e)?mt:Gr;return n&&so(e,t,n)&&(t=void 0),r(e,Qi(t,3))},Tn.sortedIndex=function(e,t){return Xr(e,t)},Tn.sortedIndexBy=function(e,t,n){return qr(e,t,Qi(n,2))},Tn.sortedIndexOf=function(e,t){var n=null==e?0:e.length;if(n){var r=Xr(e,t);if(r<n&&Ea(e[r],t))return r}return-1},Tn.sortedLastIndex=function(e,t){return Xr(e,t,!0)},Tn.sortedLastIndexBy=function(e,t,n){return qr(e,t,Qi(n,2),!0)},Tn.sortedLastIndexOf=function(e,t){if(null==e?0:e.length){var n=Xr(e,t,!0)-1;if(Ea(e[n],t))return n}return-1},Tn.startCase=Rs,Tn.startsWith=function(e,t,n){return e=ss(e),n=null==n?0:Zn(rs(n),0,e.length),t=Kr(t),e.slice(n,n+t.length)==t},Tn.subtract=fl,Tn.sum=function(e){return e&&e.length?Tt(e,Gs):0},Tn.sumBy=function(e,t){return e&&e.length?Tt(e,Qi(t,2)):0},Tn.template=function(e,t,n){var r=Tn.templateSettings;n&&so(e,t,n)&&(t=void 0),e=ss(e),t=cs({},t,r,Bi);var i,o,a=cs({},t.imports,r.imports,Bi),s=Cs(a),l=kt(a,s),u=0,c=t.interpolate||ce,d="__p += '",h=ve((t.escape||ce).source+"|"+c.source+"|"+(c===U?ne:ce).source+"|"+(t.evaluate||ce).source+"|$","g"),f="//# sourceURL="+(xe.call(t,"sourceURL")?(t.sourceURL+"").replace(/[\r\n]/g," "):"lodash.templateSources["+ ++Fe+"]")+"\n";e.replace(h,(function(t,n,r,a,s,l){return r||(r=a),d+=e.slice(u,l).replace(de,zt),n&&(i=!0,d+="' +\n__e("+n+") +\n'"),s&&(o=!0,d+="';\n"+s+";\n__p += '"),r&&(d+="' +\n((__t = ("+r+")) == null ? '' : __t) +\n'"),u=l+t.length,t})),d+="';\n";var p=xe.call(t,"variable")&&t.variable;p||(d="with (obj) {\n"+d+"\n}\n"),d=(o?d.replace(D,""):d).replace(P,"$1").replace(N,"$1;"),d="function("+(p||"obj")+") {\n"+(p?"":"obj || (obj = {});\n")+"var __t, __p = ''"+(i?", __e = _.escape":"")+(o?", __j = Array.prototype.join;\nfunction print() { __p += __j.call(arguments, '') }\n":";\n")+d+"return __p\n}";var g=Hs((function(){return fe(s,f+"return "+d).apply(void 0,l)}));if(g.source=d,Fa(g))throw g;return g},Tn.times=function(e,t){if((e=rs(e))<1||e>9007199254740991)return[];var n=4294967295,r=sn(e,4294967295);e-=4294967295;for(var i=Et(r,t=Qi(t));++n<e;)t(n);return i},Tn.toFinite=ns,Tn.toInteger=rs,Tn.toLength=is,Tn.toLower=function(e){return ss(e).toLowerCase()},Tn.toNumber=os,Tn.toSafeInteger=function(e){return e?Zn(rs(e),-9007199254740991,9007199254740991):0===e?e:0},Tn.toString=ss,Tn.toUpper=function(e){return ss(e).toUpperCase()},Tn.trim=function(e,t,n){if((e=ss(e))&&(n||void 0===t))return e.replace(q,"");if(!e||!(t=Kr(t)))return e;var r=Gt(e),i=Gt(t);return ui(r,Dt(r,i),Pt(r,i)+1).join("")},Tn.trimEnd=function(e,t,n){if((e=ss(e))&&(n||void 0===t))return e.replace(Q,"");if(!e||!(t=Kr(t)))return e;var r=Gt(e);return ui(r,0,Pt(r,Gt(t))+1).join("")},Tn.trimStart=function(e,t,n){if((e=ss(e))&&(n||void 0===t))return e.replace(Z,"");if(!e||!(t=Kr(t)))return e;var r=Gt(e);return ui(r,Dt(r,Gt(t))).join("")},Tn.truncate=function(e,t){var n=30,r="...";if(Va(t)){var i="separator"in t?t.separator:i;n="length"in t?rs(t.length):n,r="omission"in t?Kr(t.omission):r}var o=(e=ss(e)).length;if(Ft(e)){var a=Gt(e);o=a.length}if(n>=o)return e;var s=n-Wt(r);if(s<1)return r;var l=a?ui(a,0,s).join(""):e.slice(0,s);if(void 0===i)return l+r;if(a&&(s+=l.length-s),qa(i)){if(e.slice(s).search(i)){var u,c=l;for(i.global||(i=ve(i.source,ss(re.exec(i))+"g")),i.lastIndex=0;u=i.exec(c);)var d=u.index;l=l.slice(0,void 0===d?s:d)}}else if(e.indexOf(Kr(i),s)!=s){var h=l.lastIndexOf(i);h>-1&&(l=l.slice(0,h))}return l+r},Tn.unescape=function(e){return(e=ss(e))&&z.test(e)?e.replace(j,Xt):e},Tn.uniqueId=function(e){var t=++we;return ss(e)+t},Tn.upperCase=zs,Tn.upperFirst=Fs,Tn.each=sa,Tn.eachRight=la,Tn.first=No,Qs(Tn,(cl={},lr(Tn,(function(e,t){xe.call(Tn.prototype,t)||(cl[t]=e)})),cl),{chain:!1}),Tn.VERSION="4.17.15",st(["bind","bindKey","curry","curryRight","partial","partialRight"],(function(e){Tn[e].placeholder=Tn})),st(["drop","take"],(function(e,t){Ln.prototype[e]=function(n){n=void 0===n?1:an(rs(n),0);var r=this.__filtered__&&!t?new Ln(this):this.clone();return r.__filtered__?r.__takeCount__=sn(n,r.__takeCount__):r.__views__.push({size:sn(n,4294967295),type:e+(r.__dir__<0?"Right":"")}),r},Ln.prototype[e+"Right"]=function(t){return this.reverse()[e](t).reverse()}})),st(["filter","map","takeWhile"],(function(e,t){var n=t+1,r=1==n||3==n;Ln.prototype[e]=function(e){var t=this.clone();return t.__iteratees__.push({iteratee:Qi(e,3),type:n}),t.__filtered__=t.__filtered__||r,t}})),st(["head","last"],(function(e,t){var n="take"+(t?"Right":"");Ln.prototype[e]=function(){return this[n](1).value()[0]}})),st(["initial","tail"],(function(e,t){var n="drop"+(t?"":"Right");Ln.prototype[e]=function(){return this.__filtered__?new Ln(this):this[n](1)}})),Ln.prototype.compact=function(){return this.filter(Gs)},Ln.prototype.find=function(e){return this.filter(e).head()},Ln.prototype.findLast=function(e){return this.reverse().find(e)},Ln.prototype.invokeMap=zr((function(e,t){return"function"==typeof e?new Ln(this):this.map((function(n){return yr(n,e,t)}))})),Ln.prototype.reject=function(e){return this.filter(xa(Qi(e)))},Ln.prototype.slice=function(e,t){e=rs(e);var n=this;return n.__filtered__&&(e>0||t<0)?new Ln(n):(e<0?n=n.takeRight(-e):e&&(n=n.drop(e)),void 0!==t&&(n=(t=rs(t))<0?n.dropRight(-t):n.take(t-e)),n)},Ln.prototype.takeRightWhile=function(e){return this.reverse().takeWhile(e).reverse()},Ln.prototype.toArray=function(){return this.take(4294967295)},lr(Ln.prototype,(function(e,t){var n=/^(?:filter|find|map|reject)|While$/.test(t),r=/^(?:head|last)$/.test(t),i=Tn[r?"take"+("last"==t?"Right":""):t],o=r||/^find/.test(t);i&&(Tn.prototype[t]=function(){var t=this.__wrapped__,a=r?[1]:arguments,s=t instanceof Ln,l=a[0],u=s||Da(t),c=function(e){var t=i.apply(Tn,pt([e],a));return r&&d?t[0]:t};u&&n&&"function"==typeof l&&1!=l.length&&(s=u=!1);var d=this.__chain__,h=!!this.__actions__.length,f=o&&!d,p=s&&!h;if(!o&&u){t=p?t:new Ln(this);var g=e.apply(t,a);return g.__actions__.push({func:na,args:[c],thisArg:void 0}),new kn(g,d)}return f&&p?e.apply(this,a):(g=this.thru(c),f?r?g.value()[0]:g.value():g)})})),st(["pop","push","shift","sort","splice","unshift"],(function(e){var t=be[e],n=/^(?:push|sort|unshift)$/.test(e)?"tap":"thru",r=/^(?:pop|shift)$/.test(e);Tn.prototype[e]=function(){var e=arguments;if(r&&!this.__chain__){var i=this.value();return t.apply(Da(i)?i:[],e)}return this[n]((function(n){return t.apply(Da(n)?n:[],e)}))}})),lr(Ln.prototype,(function(e,t){var n=Tn[t];if(n){var r=n.name+"";xe.call(bn,r)||(bn[r]=[]),bn[r].push({name:t,func:n})}})),bn[Ti(void 0,2).name]=[{name:"wrapper",func:void 0}],Ln.prototype.clone=function(){var e=new Ln(this.__wrapped__);return e.__actions__=mi(this.__actions__),e.__dir__=this.__dir__,e.__filtered__=this.__filtered__,e.__iteratees__=mi(this.__iteratees__),e.__takeCount__=this.__takeCount__,e.__views__=mi(this.__views__),e},Ln.prototype.reverse=function(){if(this.__filtered__){var e=new Ln(this);e.__dir__=-1,e.__filtered__=!0}else(e=this.clone()).__dir__*=-1;return e},Ln.prototype.value=function(){var e=this.__wrapped__.value(),t=this.__dir__,n=Da(e),r=t<0,i=n?e.length:0,o=function(e,t,n){var r=-1,i=n.length;for(;++r<i;){var o=n[r],a=o.size;switch(o.type){case"drop":e+=a;break;case"dropRight":t-=a;break;case"take":t=sn(t,e+a);break;case"takeRight":e=an(e,t-a)}}return{start:e,end:t}}(0,i,this.__views__),a=o.start,s=o.end,l=s-a,u=r?s:a-1,c=this.__iteratees__,d=c.length,h=0,f=sn(l,this.__takeCount__);if(!n||!r&&i==l&&f==l)return ni(e,this.__actions__);var p=[];e:for(;l--&&h<f;){for(var g=-1,v=e[u+=t];++g<d;){var m=c[g],y=m.iteratee,b=m.type,C=y(v);if(2==b)v=C;else if(!C){if(1==b)continue e;break e}}p[h++]=v}return p},Tn.prototype.at=ra,Tn.prototype.chain=function(){return ta(this)},Tn.prototype.commit=function(){return new kn(this.value(),this.__chain__)},Tn.prototype.next=function(){void 0===this.__values__&&(this.__values__=ts(this.value()));var e=this.__index__>=this.__values__.length;return{done:e,value:e?void 0:this.__values__[this.__index__++]}},Tn.prototype.plant=function(e){for(var t,n=this;n instanceof On;){var r=To(n);r.__index__=0,r.__values__=void 0,t?i.__wrapped__=r:t=r;var i=r;n=n.__wrapped__}return i.__wrapped__=e,t},Tn.prototype.reverse=function(){var e=this.__wrapped__;if(e instanceof Ln){var t=e;return this.__actions__.length&&(t=new Ln(this)),(t=t.reverse()).__actions__.push({func:na,args:[Vo],thisArg:void 0}),new kn(t,this.__chain__)}return this.thru(Vo)},Tn.prototype.toJSON=Tn.prototype.valueOf=Tn.prototype.value=function(){return ni(this.__wrapped__,this.__actions__)},Tn.prototype.first=Tn.prototype.head,Je&&(Tn.prototype[Je]=function(){return this}),Tn}();Xe._=qt,void 0===(i=function(){return qt}.call(t,n,t,r))||(r.exports=i)}).call(this)}).call(this,n(61),n(188)(e))},function(e,t,n){"use strict";e.exports=function(e){var t=[];return t.toString=function(){return this.map((function(t){var n=function(e,t){var n=e[1]||"",r=e[3];if(!r)return n;if(t&&"function"==typeof btoa){var i=(a=r,"/*# sourceMappingURL=data:application/json;charset=utf-8;base64,"+btoa(unescape(encodeURIComponent(JSON.stringify(a))))+" */"),o=r.sources.map((function(e){return"/*# sourceURL="+r.sourceRoot+e+" */"}));return[n].concat(o).concat([i]).join("\n")}var a;return[n].join("\n")}(t,e);return t[2]?"@media "+t[2]+"{"+n+"}":n})).join("")},t.i=function(e,n){"string"==typeof e&&(e=[[null,e,""]]);for(var r={},i=0;i<this.length;i++){var o=this[i][0];null!=o&&(r[o]=!0)}for(i=0;i<e.length;i++){var a=e[i];null!=a[0]&&r[a[0]]||(n&&!a[2]?a[2]=n:n&&(a[2]="("+a[2]+") and ("+n+")"),t.push(a))}},t}},function(e,t,n){"use strict";var r=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),i=this&&this.__exportStar||function(e,t){for(var n in e)"default"===n||Object.prototype.hasOwnProperty.call(t,n)||r(t,e,n)};Object.defineProperty(t,"__esModule",{value:!0}),i(n(2041),t),i(n(401),t),i(n(592),t),i(n(1104),t),i(n(1105),t),i(n(1106),t),i(n(306),t),i(n(1097),t),i(n(2045),t),i(n(182),t),i(n(1100),t),i(n(593),t),i(n(594),t),i(n(1107),t),i(n(592),t),i(n(2046),t),i(n(1110),t),i(n(1108),t),i(n(1109),t),i(n(2047),t),i(n(1111),t),i(n(595),t),i(n(2048),t),i(n(403),t),i(n(404),t),i(n(1103),t),i(n(2049),t),i(n(1112),t),i(n(1113),t),i(n(1114),t),i(n(1115),t),i(n(1102),t),i(n(1101),t)},function(e,t,n){var r;try{r={cloneDeep:n(2153),constant:n(609),defaults:n(2154),each:n(1146),filter:n(1149),find:n(2155),flatten:n(1172),forEach:n(1147),forIn:n(2160),has:n(1156),isUndefined:n(1157),last:n(2161),map:n(1158),mapValues:n(2162),max:n(2163),merge:n(2165),min:n(2170),minBy:n(2171),now:n(2172),pick:n(2173),range:n(2178),reduce:n(1160),sortBy:n(2181),uniqueId:n(2186),values:n(1165),zipObject:n(2187)}}catch(e){}r||(r=window._),e.exports=r},function(e,t,n){var r;
/*!
 * jQuery JavaScript Library v3.5.1
 * https://jquery.com/
 *
 * Includes Sizzle.js
 * https://sizzlejs.com/
 *
 * Copyright JS Foundation and other contributors
 * Released under the MIT license
 * https://jquery.org/license
 *
 * Date: 2020-05-04T22:49Z
 */!function(t,n){"use strict";"object"==typeof e.exports?e.exports=t.document?n(t,!0):function(e){if(!e.document)throw new Error("jQuery requires a window with a document");return n(e)}:n(t)}("undefined"!=typeof window?window:this,(function(n,i){"use strict";var o=[],a=Object.getPrototypeOf,s=o.slice,l=o.flat?function(e){return o.flat.call(e)}:function(e){return o.concat.apply([],e)},u=o.push,c=o.indexOf,d={},h=d.toString,f=d.hasOwnProperty,p=f.toString,g=p.call(Object),v={},m=function(e){return"function"==typeof e&&"number"!=typeof e.nodeType},y=function(e){return null!=e&&e===e.window},b=n.document,C={type:!0,src:!0,nonce:!0,noModule:!0};function A(e,t,n){var r,i,o=(n=n||b).createElement("script");if(o.text=e,t)for(r in C)(i=t[r]||t.getAttribute&&t.getAttribute(r))&&o.setAttribute(r,i);n.head.appendChild(o).parentNode.removeChild(o)}function _(e){return null==e?e+"":"object"==typeof e||"function"==typeof e?d[h.call(e)]||"object":typeof e}var I=function(e,t){return new I.fn.init(e,t)};function x(e){var t=!!e&&"length"in e&&e.length,n=_(e);return!m(e)&&!y(e)&&("array"===n||0===t||"number"==typeof t&&t>0&&t-1 in e)}I.fn=I.prototype={jquery:"3.5.1",constructor:I,length:0,toArray:function(){return s.call(this)},get:function(e){return null==e?s.call(this):e<0?this[e+this.length]:this[e]},pushStack:function(e){var t=I.merge(this.constructor(),e);return t.prevObject=this,t},each:function(e){return I.each(this,e)},map:function(e){return this.pushStack(I.map(this,(function(t,n){return e.call(t,n,t)})))},slice:function(){return this.pushStack(s.apply(this,arguments))},first:function(){return this.eq(0)},last:function(){return this.eq(-1)},even:function(){return this.pushStack(I.grep(this,(function(e,t){return(t+1)%2})))},odd:function(){return this.pushStack(I.grep(this,(function(e,t){return t%2})))},eq:function(e){var t=this.length,n=+e+(e<0?t:0);return this.pushStack(n>=0&&n<t?[this[n]]:[])},end:function(){return this.prevObject||this.constructor()},push:u,sort:o.sort,splice:o.splice},I.extend=I.fn.extend=function(){var e,t,n,r,i,o,a=arguments[0]||{},s=1,l=arguments.length,u=!1;for("boolean"==typeof a&&(u=a,a=arguments[s]||{},s++),"object"==typeof a||m(a)||(a={}),s===l&&(a=this,s--);s<l;s++)if(null!=(e=arguments[s]))for(t in e)r=e[t],"__proto__"!==t&&a!==r&&(u&&r&&(I.isPlainObject(r)||(i=Array.isArray(r)))?(n=a[t],o=i&&!Array.isArray(n)?[]:i||I.isPlainObject(n)?n:{},i=!1,a[t]=I.extend(u,o,r)):void 0!==r&&(a[t]=r));return a},I.extend({expando:"jQuery"+("3.5.1"+Math.random()).replace(/\D/g,""),isReady:!0,error:function(e){throw new Error(e)},noop:function(){},isPlainObject:function(e){var t,n;return!(!e||"[object Object]"!==h.call(e))&&(!(t=a(e))||"function"==typeof(n=f.call(t,"constructor")&&t.constructor)&&p.call(n)===g)},isEmptyObject:function(e){var t;for(t in e)return!1;return!0},globalEval:function(e,t,n){A(e,{nonce:t&&t.nonce},n)},each:function(e,t){var n,r=0;if(x(e))for(n=e.length;r<n&&!1!==t.call(e[r],r,e[r]);r++);else for(r in e)if(!1===t.call(e[r],r,e[r]))break;return e},makeArray:function(e,t){var n=t||[];return null!=e&&(x(Object(e))?I.merge(n,"string"==typeof e?[e]:e):u.call(n,e)),n},inArray:function(e,t,n){return null==t?-1:c.call(t,e,n)},merge:function(e,t){for(var n=+t.length,r=0,i=e.length;r<n;r++)e[i++]=t[r];return e.length=i,e},grep:function(e,t,n){for(var r=[],i=0,o=e.length,a=!n;i<o;i++)!t(e[i],i)!==a&&r.push(e[i]);return r},map:function(e,t,n){var r,i,o=0,a=[];if(x(e))for(r=e.length;o<r;o++)null!=(i=t(e[o],o,n))&&a.push(i);else for(o in e)null!=(i=t(e[o],o,n))&&a.push(i);return l(a)},guid:1,support:v}),"function"==typeof Symbol&&(I.fn[Symbol.iterator]=o[Symbol.iterator]),I.each("Boolean Number String Function Array Date RegExp Object Error Symbol".split(" "),(function(e,t){d["[object "+t+"]"]=t.toLowerCase()}));var w=
/*!
 * Sizzle CSS Selector Engine v2.3.5
 * https://sizzlejs.com/
 *
 * Copyright JS Foundation and other contributors
 * Released under the MIT license
 * https://js.foundation/
 *
 * Date: 2020-03-14
 */
function(e){var t,n,r,i,o,a,s,l,u,c,d,h,f,p,g,v,m,y,b,C="sizzle"+1*new Date,A=e.document,_=0,I=0,x=le(),w=le(),M=le(),S=le(),T=function(e,t){return e===t&&(d=!0),0},E={}.hasOwnProperty,O=[],k=O.pop,L=O.push,D=O.push,P=O.slice,N=function(e,t){for(var n=0,r=e.length;n<r;n++)if(e[n]===t)return n;return-1},j="checked|selected|async|autofocus|autoplay|controls|defer|disabled|hidden|ismap|loop|multiple|open|readonly|required|scoped",R="[\\x20\\t\\r\\n\\f]",z="(?:\\\\[\\da-fA-F]{1,6}"+R+"?|\\\\[^\\r\\n\\f]|[\\w-]|[^\0-\\x7f])+",F="\\["+R+"*("+z+")(?:"+R+"*([*^$|!~]?=)"+R+"*(?:'((?:\\\\.|[^\\\\'])*)'|\"((?:\\\\.|[^\\\\\"])*)\"|("+z+"))|)"+R+"*\\]",B=":("+z+")(?:\\((('((?:\\\\.|[^\\\\'])*)'|\"((?:\\\\.|[^\\\\\"])*)\")|((?:\\\\.|[^\\\\()[\\]]|"+F+")*)|.*)\\)|)",H=new RegExp(R+"+","g"),U=new RegExp("^"+R+"+|((?:^|[^\\\\])(?:\\\\.)*)"+R+"+$","g"),V=new RegExp("^"+R+"*,"+R+"*"),Y=new RegExp("^"+R+"*([>+~]|"+R+")"+R+"*"),W=new RegExp(R+"|>"),G=new RegExp(B),X=new RegExp("^"+z+"$"),q={ID:new RegExp("^#("+z+")"),CLASS:new RegExp("^\\.("+z+")"),TAG:new RegExp("^("+z+"|[*])"),ATTR:new RegExp("^"+F),PSEUDO:new RegExp("^"+B),CHILD:new RegExp("^:(only|first|last|nth|nth-last)-(child|of-type)(?:\\("+R+"*(even|odd|(([+-]|)(\\d*)n|)"+R+"*(?:([+-]|)"+R+"*(\\d+)|))"+R+"*\\)|)","i"),bool:new RegExp("^(?:"+j+")$","i"),needsContext:new RegExp("^"+R+"*[>+~]|:(even|odd|eq|gt|lt|nth|first|last)(?:\\("+R+"*((?:-\\d)?\\d*)"+R+"*\\)|)(?=[^-]|$)","i")},Z=/HTML$/i,Q=/^(?:input|select|textarea|button)$/i,K=/^h\d$/i,J=/^[^{]+\{\s*\[native \w/,$=/^(?:#([\w-]+)|(\w+)|\.([\w-]+))$/,ee=/[+~]/,te=new RegExp("\\\\[\\da-fA-F]{1,6}"+R+"?|\\\\([^\\r\\n\\f])","g"),ne=function(e,t){var n="0x"+e.slice(1)-65536;return t||(n<0?String.fromCharCode(n+65536):String.fromCharCode(n>>10|55296,1023&n|56320))},re=/([\0-\x1f\x7f]|^-?\d)|^-$|[^\0-\x1f\x7f-\uFFFF\w-]/g,ie=function(e,t){return t?"\0"===e?"�":e.slice(0,-1)+"\\"+e.charCodeAt(e.length-1).toString(16)+" ":"\\"+e},oe=function(){h()},ae=Ce((function(e){return!0===e.disabled&&"fieldset"===e.nodeName.toLowerCase()}),{dir:"parentNode",next:"legend"});try{D.apply(O=P.call(A.childNodes),A.childNodes),O[A.childNodes.length].nodeType}catch(e){D={apply:O.length?function(e,t){L.apply(e,P.call(t))}:function(e,t){for(var n=e.length,r=0;e[n++]=t[r++];);e.length=n-1}}}function se(e,t,r,i){var o,s,u,c,d,p,m,y=t&&t.ownerDocument,A=t?t.nodeType:9;if(r=r||[],"string"!=typeof e||!e||1!==A&&9!==A&&11!==A)return r;if(!i&&(h(t),t=t||f,g)){if(11!==A&&(d=$.exec(e)))if(o=d[1]){if(9===A){if(!(u=t.getElementById(o)))return r;if(u.id===o)return r.push(u),r}else if(y&&(u=y.getElementById(o))&&b(t,u)&&u.id===o)return r.push(u),r}else{if(d[2])return D.apply(r,t.getElementsByTagName(e)),r;if((o=d[3])&&n.getElementsByClassName&&t.getElementsByClassName)return D.apply(r,t.getElementsByClassName(o)),r}if(n.qsa&&!S[e+" "]&&(!v||!v.test(e))&&(1!==A||"object"!==t.nodeName.toLowerCase())){if(m=e,y=t,1===A&&(W.test(e)||Y.test(e))){for((y=ee.test(e)&&me(t.parentNode)||t)===t&&n.scope||((c=t.getAttribute("id"))?c=c.replace(re,ie):t.setAttribute("id",c=C)),s=(p=a(e)).length;s--;)p[s]=(c?"#"+c:":scope")+" "+be(p[s]);m=p.join(",")}try{return D.apply(r,y.querySelectorAll(m)),r}catch(t){S(e,!0)}finally{c===C&&t.removeAttribute("id")}}}return l(e.replace(U,"$1"),t,r,i)}function le(){var e=[];return function t(n,i){return e.push(n+" ")>r.cacheLength&&delete t[e.shift()],t[n+" "]=i}}function ue(e){return e[C]=!0,e}function ce(e){var t=f.createElement("fieldset");try{return!!e(t)}catch(e){return!1}finally{t.parentNode&&t.parentNode.removeChild(t),t=null}}function de(e,t){for(var n=e.split("|"),i=n.length;i--;)r.attrHandle[n[i]]=t}function he(e,t){var n=t&&e,r=n&&1===e.nodeType&&1===t.nodeType&&e.sourceIndex-t.sourceIndex;if(r)return r;if(n)for(;n=n.nextSibling;)if(n===t)return-1;return e?1:-1}function fe(e){return function(t){return"input"===t.nodeName.toLowerCase()&&t.type===e}}function pe(e){return function(t){var n=t.nodeName.toLowerCase();return("input"===n||"button"===n)&&t.type===e}}function ge(e){return function(t){return"form"in t?t.parentNode&&!1===t.disabled?"label"in t?"label"in t.parentNode?t.parentNode.disabled===e:t.disabled===e:t.isDisabled===e||t.isDisabled!==!e&&ae(t)===e:t.disabled===e:"label"in t&&t.disabled===e}}function ve(e){return ue((function(t){return t=+t,ue((function(n,r){for(var i,o=e([],n.length,t),a=o.length;a--;)n[i=o[a]]&&(n[i]=!(r[i]=n[i]))}))}))}function me(e){return e&&void 0!==e.getElementsByTagName&&e}for(t in n=se.support={},o=se.isXML=function(e){var t=e.namespaceURI,n=(e.ownerDocument||e).documentElement;return!Z.test(t||n&&n.nodeName||"HTML")},h=se.setDocument=function(e){var t,i,a=e?e.ownerDocument||e:A;return a!=f&&9===a.nodeType&&a.documentElement?(p=(f=a).documentElement,g=!o(f),A!=f&&(i=f.defaultView)&&i.top!==i&&(i.addEventListener?i.addEventListener("unload",oe,!1):i.attachEvent&&i.attachEvent("onunload",oe)),n.scope=ce((function(e){return p.appendChild(e).appendChild(f.createElement("div")),void 0!==e.querySelectorAll&&!e.querySelectorAll(":scope fieldset div").length})),n.attributes=ce((function(e){return e.className="i",!e.getAttribute("className")})),n.getElementsByTagName=ce((function(e){return e.appendChild(f.createComment("")),!e.getElementsByTagName("*").length})),n.getElementsByClassName=J.test(f.getElementsByClassName),n.getById=ce((function(e){return p.appendChild(e).id=C,!f.getElementsByName||!f.getElementsByName(C).length})),n.getById?(r.filter.ID=function(e){var t=e.replace(te,ne);return function(e){return e.getAttribute("id")===t}},r.find.ID=function(e,t){if(void 0!==t.getElementById&&g){var n=t.getElementById(e);return n?[n]:[]}}):(r.filter.ID=function(e){var t=e.replace(te,ne);return function(e){var n=void 0!==e.getAttributeNode&&e.getAttributeNode("id");return n&&n.value===t}},r.find.ID=function(e,t){if(void 0!==t.getElementById&&g){var n,r,i,o=t.getElementById(e);if(o){if((n=o.getAttributeNode("id"))&&n.value===e)return[o];for(i=t.getElementsByName(e),r=0;o=i[r++];)if((n=o.getAttributeNode("id"))&&n.value===e)return[o]}return[]}}),r.find.TAG=n.getElementsByTagName?function(e,t){return void 0!==t.getElementsByTagName?t.getElementsByTagName(e):n.qsa?t.querySelectorAll(e):void 0}:function(e,t){var n,r=[],i=0,o=t.getElementsByTagName(e);if("*"===e){for(;n=o[i++];)1===n.nodeType&&r.push(n);return r}return o},r.find.CLASS=n.getElementsByClassName&&function(e,t){if(void 0!==t.getElementsByClassName&&g)return t.getElementsByClassName(e)},m=[],v=[],(n.qsa=J.test(f.querySelectorAll))&&(ce((function(e){var t;p.appendChild(e).innerHTML="<a id='"+C+"'></a><select id='"+C+"-\r\\' msallowcapture=''><option selected=''></option></select>",e.querySelectorAll("[msallowcapture^='']").length&&v.push("[*^$]="+R+"*(?:''|\"\")"),e.querySelectorAll("[selected]").length||v.push("\\["+R+"*(?:value|"+j+")"),e.querySelectorAll("[id~="+C+"-]").length||v.push("~="),(t=f.createElement("input")).setAttribute("name",""),e.appendChild(t),e.querySelectorAll("[name='']").length||v.push("\\["+R+"*name"+R+"*="+R+"*(?:''|\"\")"),e.querySelectorAll(":checked").length||v.push(":checked"),e.querySelectorAll("a#"+C+"+*").length||v.push(".#.+[+~]"),e.querySelectorAll("\\\f"),v.push("[\\r\\n\\f]")})),ce((function(e){e.innerHTML="<a href='' disabled='disabled'></a><select disabled='disabled'><option/></select>";var t=f.createElement("input");t.setAttribute("type","hidden"),e.appendChild(t).setAttribute("name","D"),e.querySelectorAll("[name=d]").length&&v.push("name"+R+"*[*^$|!~]?="),2!==e.querySelectorAll(":enabled").length&&v.push(":enabled",":disabled"),p.appendChild(e).disabled=!0,2!==e.querySelectorAll(":disabled").length&&v.push(":enabled",":disabled"),e.querySelectorAll("*,:x"),v.push(",.*:")}))),(n.matchesSelector=J.test(y=p.matches||p.webkitMatchesSelector||p.mozMatchesSelector||p.oMatchesSelector||p.msMatchesSelector))&&ce((function(e){n.disconnectedMatch=y.call(e,"*"),y.call(e,"[s!='']:x"),m.push("!=",B)})),v=v.length&&new RegExp(v.join("|")),m=m.length&&new RegExp(m.join("|")),t=J.test(p.compareDocumentPosition),b=t||J.test(p.contains)?function(e,t){var n=9===e.nodeType?e.documentElement:e,r=t&&t.parentNode;return e===r||!(!r||1!==r.nodeType||!(n.contains?n.contains(r):e.compareDocumentPosition&&16&e.compareDocumentPosition(r)))}:function(e,t){if(t)for(;t=t.parentNode;)if(t===e)return!0;return!1},T=t?function(e,t){if(e===t)return d=!0,0;var r=!e.compareDocumentPosition-!t.compareDocumentPosition;return r||(1&(r=(e.ownerDocument||e)==(t.ownerDocument||t)?e.compareDocumentPosition(t):1)||!n.sortDetached&&t.compareDocumentPosition(e)===r?e==f||e.ownerDocument==A&&b(A,e)?-1:t==f||t.ownerDocument==A&&b(A,t)?1:c?N(c,e)-N(c,t):0:4&r?-1:1)}:function(e,t){if(e===t)return d=!0,0;var n,r=0,i=e.parentNode,o=t.parentNode,a=[e],s=[t];if(!i||!o)return e==f?-1:t==f?1:i?-1:o?1:c?N(c,e)-N(c,t):0;if(i===o)return he(e,t);for(n=e;n=n.parentNode;)a.unshift(n);for(n=t;n=n.parentNode;)s.unshift(n);for(;a[r]===s[r];)r++;return r?he(a[r],s[r]):a[r]==A?-1:s[r]==A?1:0},f):f},se.matches=function(e,t){return se(e,null,null,t)},se.matchesSelector=function(e,t){if(h(e),n.matchesSelector&&g&&!S[t+" "]&&(!m||!m.test(t))&&(!v||!v.test(t)))try{var r=y.call(e,t);if(r||n.disconnectedMatch||e.document&&11!==e.document.nodeType)return r}catch(e){S(t,!0)}return se(t,f,null,[e]).length>0},se.contains=function(e,t){return(e.ownerDocument||e)!=f&&h(e),b(e,t)},se.attr=function(e,t){(e.ownerDocument||e)!=f&&h(e);var i=r.attrHandle[t.toLowerCase()],o=i&&E.call(r.attrHandle,t.toLowerCase())?i(e,t,!g):void 0;return void 0!==o?o:n.attributes||!g?e.getAttribute(t):(o=e.getAttributeNode(t))&&o.specified?o.value:null},se.escape=function(e){return(e+"").replace(re,ie)},se.error=function(e){throw new Error("Syntax error, unrecognized expression: "+e)},se.uniqueSort=function(e){var t,r=[],i=0,o=0;if(d=!n.detectDuplicates,c=!n.sortStable&&e.slice(0),e.sort(T),d){for(;t=e[o++];)t===e[o]&&(i=r.push(o));for(;i--;)e.splice(r[i],1)}return c=null,e},i=se.getText=function(e){var t,n="",r=0,o=e.nodeType;if(o){if(1===o||9===o||11===o){if("string"==typeof e.textContent)return e.textContent;for(e=e.firstChild;e;e=e.nextSibling)n+=i(e)}else if(3===o||4===o)return e.nodeValue}else for(;t=e[r++];)n+=i(t);return n},(r=se.selectors={cacheLength:50,createPseudo:ue,match:q,attrHandle:{},find:{},relative:{">":{dir:"parentNode",first:!0}," ":{dir:"parentNode"},"+":{dir:"previousSibling",first:!0},"~":{dir:"previousSibling"}},preFilter:{ATTR:function(e){return e[1]=e[1].replace(te,ne),e[3]=(e[3]||e[4]||e[5]||"").replace(te,ne),"~="===e[2]&&(e[3]=" "+e[3]+" "),e.slice(0,4)},CHILD:function(e){return e[1]=e[1].toLowerCase(),"nth"===e[1].slice(0,3)?(e[3]||se.error(e[0]),e[4]=+(e[4]?e[5]+(e[6]||1):2*("even"===e[3]||"odd"===e[3])),e[5]=+(e[7]+e[8]||"odd"===e[3])):e[3]&&se.error(e[0]),e},PSEUDO:function(e){var t,n=!e[6]&&e[2];return q.CHILD.test(e[0])?null:(e[3]?e[2]=e[4]||e[5]||"":n&&G.test(n)&&(t=a(n,!0))&&(t=n.indexOf(")",n.length-t)-n.length)&&(e[0]=e[0].slice(0,t),e[2]=n.slice(0,t)),e.slice(0,3))}},filter:{TAG:function(e){var t=e.replace(te,ne).toLowerCase();return"*"===e?function(){return!0}:function(e){return e.nodeName&&e.nodeName.toLowerCase()===t}},CLASS:function(e){var t=x[e+" "];return t||(t=new RegExp("(^|"+R+")"+e+"("+R+"|$)"))&&x(e,(function(e){return t.test("string"==typeof e.className&&e.className||void 0!==e.getAttribute&&e.getAttribute("class")||"")}))},ATTR:function(e,t,n){return function(r){var i=se.attr(r,e);return null==i?"!="===t:!t||(i+="","="===t?i===n:"!="===t?i!==n:"^="===t?n&&0===i.indexOf(n):"*="===t?n&&i.indexOf(n)>-1:"$="===t?n&&i.slice(-n.length)===n:"~="===t?(" "+i.replace(H," ")+" ").indexOf(n)>-1:"|="===t&&(i===n||i.slice(0,n.length+1)===n+"-"))}},CHILD:function(e,t,n,r,i){var o="nth"!==e.slice(0,3),a="last"!==e.slice(-4),s="of-type"===t;return 1===r&&0===i?function(e){return!!e.parentNode}:function(t,n,l){var u,c,d,h,f,p,g=o!==a?"nextSibling":"previousSibling",v=t.parentNode,m=s&&t.nodeName.toLowerCase(),y=!l&&!s,b=!1;if(v){if(o){for(;g;){for(h=t;h=h[g];)if(s?h.nodeName.toLowerCase()===m:1===h.nodeType)return!1;p=g="only"===e&&!p&&"nextSibling"}return!0}if(p=[a?v.firstChild:v.lastChild],a&&y){for(b=(f=(u=(c=(d=(h=v)[C]||(h[C]={}))[h.uniqueID]||(d[h.uniqueID]={}))[e]||[])[0]===_&&u[1])&&u[2],h=f&&v.childNodes[f];h=++f&&h&&h[g]||(b=f=0)||p.pop();)if(1===h.nodeType&&++b&&h===t){c[e]=[_,f,b];break}}else if(y&&(b=f=(u=(c=(d=(h=t)[C]||(h[C]={}))[h.uniqueID]||(d[h.uniqueID]={}))[e]||[])[0]===_&&u[1]),!1===b)for(;(h=++f&&h&&h[g]||(b=f=0)||p.pop())&&((s?h.nodeName.toLowerCase()!==m:1!==h.nodeType)||!++b||(y&&((c=(d=h[C]||(h[C]={}))[h.uniqueID]||(d[h.uniqueID]={}))[e]=[_,b]),h!==t)););return(b-=i)===r||b%r==0&&b/r>=0}}},PSEUDO:function(e,t){var n,i=r.pseudos[e]||r.setFilters[e.toLowerCase()]||se.error("unsupported pseudo: "+e);return i[C]?i(t):i.length>1?(n=[e,e,"",t],r.setFilters.hasOwnProperty(e.toLowerCase())?ue((function(e,n){for(var r,o=i(e,t),a=o.length;a--;)e[r=N(e,o[a])]=!(n[r]=o[a])})):function(e){return i(e,0,n)}):i}},pseudos:{not:ue((function(e){var t=[],n=[],r=s(e.replace(U,"$1"));return r[C]?ue((function(e,t,n,i){for(var o,a=r(e,null,i,[]),s=e.length;s--;)(o=a[s])&&(e[s]=!(t[s]=o))})):function(e,i,o){return t[0]=e,r(t,null,o,n),t[0]=null,!n.pop()}})),has:ue((function(e){return function(t){return se(e,t).length>0}})),contains:ue((function(e){return e=e.replace(te,ne),function(t){return(t.textContent||i(t)).indexOf(e)>-1}})),lang:ue((function(e){return X.test(e||"")||se.error("unsupported lang: "+e),e=e.replace(te,ne).toLowerCase(),function(t){var n;do{if(n=g?t.lang:t.getAttribute("xml:lang")||t.getAttribute("lang"))return(n=n.toLowerCase())===e||0===n.indexOf(e+"-")}while((t=t.parentNode)&&1===t.nodeType);return!1}})),target:function(t){var n=e.location&&e.location.hash;return n&&n.slice(1)===t.id},root:function(e){return e===p},focus:function(e){return e===f.activeElement&&(!f.hasFocus||f.hasFocus())&&!!(e.type||e.href||~e.tabIndex)},enabled:ge(!1),disabled:ge(!0),checked:function(e){var t=e.nodeName.toLowerCase();return"input"===t&&!!e.checked||"option"===t&&!!e.selected},selected:function(e){return e.parentNode&&e.parentNode.selectedIndex,!0===e.selected},empty:function(e){for(e=e.firstChild;e;e=e.nextSibling)if(e.nodeType<6)return!1;return!0},parent:function(e){return!r.pseudos.empty(e)},header:function(e){return K.test(e.nodeName)},input:function(e){return Q.test(e.nodeName)},button:function(e){var t=e.nodeName.toLowerCase();return"input"===t&&"button"===e.type||"button"===t},text:function(e){var t;return"input"===e.nodeName.toLowerCase()&&"text"===e.type&&(null==(t=e.getAttribute("type"))||"text"===t.toLowerCase())},first:ve((function(){return[0]})),last:ve((function(e,t){return[t-1]})),eq:ve((function(e,t,n){return[n<0?n+t:n]})),even:ve((function(e,t){for(var n=0;n<t;n+=2)e.push(n);return e})),odd:ve((function(e,t){for(var n=1;n<t;n+=2)e.push(n);return e})),lt:ve((function(e,t,n){for(var r=n<0?n+t:n>t?t:n;--r>=0;)e.push(r);return e})),gt:ve((function(e,t,n){for(var r=n<0?n+t:n;++r<t;)e.push(r);return e}))}}).pseudos.nth=r.pseudos.eq,{radio:!0,checkbox:!0,file:!0,password:!0,image:!0})r.pseudos[t]=fe(t);for(t in{submit:!0,reset:!0})r.pseudos[t]=pe(t);function ye(){}function be(e){for(var t=0,n=e.length,r="";t<n;t++)r+=e[t].value;return r}function Ce(e,t,n){var r=t.dir,i=t.next,o=i||r,a=n&&"parentNode"===o,s=I++;return t.first?function(t,n,i){for(;t=t[r];)if(1===t.nodeType||a)return e(t,n,i);return!1}:function(t,n,l){var u,c,d,h=[_,s];if(l){for(;t=t[r];)if((1===t.nodeType||a)&&e(t,n,l))return!0}else for(;t=t[r];)if(1===t.nodeType||a)if(c=(d=t[C]||(t[C]={}))[t.uniqueID]||(d[t.uniqueID]={}),i&&i===t.nodeName.toLowerCase())t=t[r]||t;else{if((u=c[o])&&u[0]===_&&u[1]===s)return h[2]=u[2];if(c[o]=h,h[2]=e(t,n,l))return!0}return!1}}function Ae(e){return e.length>1?function(t,n,r){for(var i=e.length;i--;)if(!e[i](t,n,r))return!1;return!0}:e[0]}function _e(e,t,n,r,i){for(var o,a=[],s=0,l=e.length,u=null!=t;s<l;s++)(o=e[s])&&(n&&!n(o,r,i)||(a.push(o),u&&t.push(s)));return a}function Ie(e,t,n,r,i,o){return r&&!r[C]&&(r=Ie(r)),i&&!i[C]&&(i=Ie(i,o)),ue((function(o,a,s,l){var u,c,d,h=[],f=[],p=a.length,g=o||function(e,t,n){for(var r=0,i=t.length;r<i;r++)se(e,t[r],n);return n}(t||"*",s.nodeType?[s]:s,[]),v=!e||!o&&t?g:_e(g,h,e,s,l),m=n?i||(o?e:p||r)?[]:a:v;if(n&&n(v,m,s,l),r)for(u=_e(m,f),r(u,[],s,l),c=u.length;c--;)(d=u[c])&&(m[f[c]]=!(v[f[c]]=d));if(o){if(i||e){if(i){for(u=[],c=m.length;c--;)(d=m[c])&&u.push(v[c]=d);i(null,m=[],u,l)}for(c=m.length;c--;)(d=m[c])&&(u=i?N(o,d):h[c])>-1&&(o[u]=!(a[u]=d))}}else m=_e(m===a?m.splice(p,m.length):m),i?i(null,a,m,l):D.apply(a,m)}))}function xe(e){for(var t,n,i,o=e.length,a=r.relative[e[0].type],s=a||r.relative[" "],l=a?1:0,c=Ce((function(e){return e===t}),s,!0),d=Ce((function(e){return N(t,e)>-1}),s,!0),h=[function(e,n,r){var i=!a&&(r||n!==u)||((t=n).nodeType?c(e,n,r):d(e,n,r));return t=null,i}];l<o;l++)if(n=r.relative[e[l].type])h=[Ce(Ae(h),n)];else{if((n=r.filter[e[l].type].apply(null,e[l].matches))[C]){for(i=++l;i<o&&!r.relative[e[i].type];i++);return Ie(l>1&&Ae(h),l>1&&be(e.slice(0,l-1).concat({value:" "===e[l-2].type?"*":""})).replace(U,"$1"),n,l<i&&xe(e.slice(l,i)),i<o&&xe(e=e.slice(i)),i<o&&be(e))}h.push(n)}return Ae(h)}return ye.prototype=r.filters=r.pseudos,r.setFilters=new ye,a=se.tokenize=function(e,t){var n,i,o,a,s,l,u,c=w[e+" "];if(c)return t?0:c.slice(0);for(s=e,l=[],u=r.preFilter;s;){for(a in n&&!(i=V.exec(s))||(i&&(s=s.slice(i[0].length)||s),l.push(o=[])),n=!1,(i=Y.exec(s))&&(n=i.shift(),o.push({value:n,type:i[0].replace(U," ")}),s=s.slice(n.length)),r.filter)!(i=q[a].exec(s))||u[a]&&!(i=u[a](i))||(n=i.shift(),o.push({value:n,type:a,matches:i}),s=s.slice(n.length));if(!n)break}return t?s.length:s?se.error(e):w(e,l).slice(0)},s=se.compile=function(e,t){var n,i=[],o=[],s=M[e+" "];if(!s){for(t||(t=a(e)),n=t.length;n--;)(s=xe(t[n]))[C]?i.push(s):o.push(s);(s=M(e,function(e,t){var n=t.length>0,i=e.length>0,o=function(o,a,s,l,c){var d,p,v,m=0,y="0",b=o&&[],C=[],A=u,I=o||i&&r.find.TAG("*",c),x=_+=null==A?1:Math.random()||.1,w=I.length;for(c&&(u=a==f||a||c);y!==w&&null!=(d=I[y]);y++){if(i&&d){for(p=0,a||d.ownerDocument==f||(h(d),s=!g);v=e[p++];)if(v(d,a||f,s)){l.push(d);break}c&&(_=x)}n&&((d=!v&&d)&&m--,o&&b.push(d))}if(m+=y,n&&y!==m){for(p=0;v=t[p++];)v(b,C,a,s);if(o){if(m>0)for(;y--;)b[y]||C[y]||(C[y]=k.call(l));C=_e(C)}D.apply(l,C),c&&!o&&C.length>0&&m+t.length>1&&se.uniqueSort(l)}return c&&(_=x,u=A),b};return n?ue(o):o}(o,i))).selector=e}return s},l=se.select=function(e,t,n,i){var o,l,u,c,d,h="function"==typeof e&&e,f=!i&&a(e=h.selector||e);if(n=n||[],1===f.length){if((l=f[0]=f[0].slice(0)).length>2&&"ID"===(u=l[0]).type&&9===t.nodeType&&g&&r.relative[l[1].type]){if(!(t=(r.find.ID(u.matches[0].replace(te,ne),t)||[])[0]))return n;h&&(t=t.parentNode),e=e.slice(l.shift().value.length)}for(o=q.needsContext.test(e)?0:l.length;o--&&(u=l[o],!r.relative[c=u.type]);)if((d=r.find[c])&&(i=d(u.matches[0].replace(te,ne),ee.test(l[0].type)&&me(t.parentNode)||t))){if(l.splice(o,1),!(e=i.length&&be(l)))return D.apply(n,i),n;break}}return(h||s(e,f))(i,t,!g,n,!t||ee.test(e)&&me(t.parentNode)||t),n},n.sortStable=C.split("").sort(T).join("")===C,n.detectDuplicates=!!d,h(),n.sortDetached=ce((function(e){return 1&e.compareDocumentPosition(f.createElement("fieldset"))})),ce((function(e){return e.innerHTML="<a href='#'></a>","#"===e.firstChild.getAttribute("href")}))||de("type|href|height|width",(function(e,t,n){if(!n)return e.getAttribute(t,"type"===t.toLowerCase()?1:2)})),n.attributes&&ce((function(e){return e.innerHTML="<input/>",e.firstChild.setAttribute("value",""),""===e.firstChild.getAttribute("value")}))||de("value",(function(e,t,n){if(!n&&"input"===e.nodeName.toLowerCase())return e.defaultValue})),ce((function(e){return null==e.getAttribute("disabled")}))||de(j,(function(e,t,n){var r;if(!n)return!0===e[t]?t.toLowerCase():(r=e.getAttributeNode(t))&&r.specified?r.value:null})),se}(n);I.find=w,I.expr=w.selectors,I.expr[":"]=I.expr.pseudos,I.uniqueSort=I.unique=w.uniqueSort,I.text=w.getText,I.isXMLDoc=w.isXML,I.contains=w.contains,I.escapeSelector=w.escape;var M=function(e,t,n){for(var r=[],i=void 0!==n;(e=e[t])&&9!==e.nodeType;)if(1===e.nodeType){if(i&&I(e).is(n))break;r.push(e)}return r},S=function(e,t){for(var n=[];e;e=e.nextSibling)1===e.nodeType&&e!==t&&n.push(e);return n},T=I.expr.match.needsContext;function E(e,t){return e.nodeName&&e.nodeName.toLowerCase()===t.toLowerCase()}var O=/^<([a-z][^\/\0>:\x20\t\r\n\f]*)[\x20\t\r\n\f]*\/?>(?:<\/\1>|)$/i;function k(e,t,n){return m(t)?I.grep(e,(function(e,r){return!!t.call(e,r,e)!==n})):t.nodeType?I.grep(e,(function(e){return e===t!==n})):"string"!=typeof t?I.grep(e,(function(e){return c.call(t,e)>-1!==n})):I.filter(t,e,n)}I.filter=function(e,t,n){var r=t[0];return n&&(e=":not("+e+")"),1===t.length&&1===r.nodeType?I.find.matchesSelector(r,e)?[r]:[]:I.find.matches(e,I.grep(t,(function(e){return 1===e.nodeType})))},I.fn.extend({find:function(e){var t,n,r=this.length,i=this;if("string"!=typeof e)return this.pushStack(I(e).filter((function(){for(t=0;t<r;t++)if(I.contains(i[t],this))return!0})));for(n=this.pushStack([]),t=0;t<r;t++)I.find(e,i[t],n);return r>1?I.uniqueSort(n):n},filter:function(e){return this.pushStack(k(this,e||[],!1))},not:function(e){return this.pushStack(k(this,e||[],!0))},is:function(e){return!!k(this,"string"==typeof e&&T.test(e)?I(e):e||[],!1).length}});var L,D=/^(?:\s*(<[\w\W]+>)[^>]*|#([\w-]+))$/;(I.fn.init=function(e,t,n){var r,i;if(!e)return this;if(n=n||L,"string"==typeof e){if(!(r="<"===e[0]&&">"===e[e.length-1]&&e.length>=3?[null,e,null]:D.exec(e))||!r[1]&&t)return!t||t.jquery?(t||n).find(e):this.constructor(t).find(e);if(r[1]){if(t=t instanceof I?t[0]:t,I.merge(this,I.parseHTML(r[1],t&&t.nodeType?t.ownerDocument||t:b,!0)),O.test(r[1])&&I.isPlainObject(t))for(r in t)m(this[r])?this[r](t[r]):this.attr(r,t[r]);return this}return(i=b.getElementById(r[2]))&&(this[0]=i,this.length=1),this}return e.nodeType?(this[0]=e,this.length=1,this):m(e)?void 0!==n.ready?n.ready(e):e(I):I.makeArray(e,this)}).prototype=I.fn,L=I(b);var P=/^(?:parents|prev(?:Until|All))/,N={children:!0,contents:!0,next:!0,prev:!0};function j(e,t){for(;(e=e[t])&&1!==e.nodeType;);return e}I.fn.extend({has:function(e){var t=I(e,this),n=t.length;return this.filter((function(){for(var e=0;e<n;e++)if(I.contains(this,t[e]))return!0}))},closest:function(e,t){var n,r=0,i=this.length,o=[],a="string"!=typeof e&&I(e);if(!T.test(e))for(;r<i;r++)for(n=this[r];n&&n!==t;n=n.parentNode)if(n.nodeType<11&&(a?a.index(n)>-1:1===n.nodeType&&I.find.matchesSelector(n,e))){o.push(n);break}return this.pushStack(o.length>1?I.uniqueSort(o):o)},index:function(e){return e?"string"==typeof e?c.call(I(e),this[0]):c.call(this,e.jquery?e[0]:e):this[0]&&this[0].parentNode?this.first().prevAll().length:-1},add:function(e,t){return this.pushStack(I.uniqueSort(I.merge(this.get(),I(e,t))))},addBack:function(e){return this.add(null==e?this.prevObject:this.prevObject.filter(e))}}),I.each({parent:function(e){var t=e.parentNode;return t&&11!==t.nodeType?t:null},parents:function(e){return M(e,"parentNode")},parentsUntil:function(e,t,n){return M(e,"parentNode",n)},next:function(e){return j(e,"nextSibling")},prev:function(e){return j(e,"previousSibling")},nextAll:function(e){return M(e,"nextSibling")},prevAll:function(e){return M(e,"previousSibling")},nextUntil:function(e,t,n){return M(e,"nextSibling",n)},prevUntil:function(e,t,n){return M(e,"previousSibling",n)},siblings:function(e){return S((e.parentNode||{}).firstChild,e)},children:function(e){return S(e.firstChild)},contents:function(e){return null!=e.contentDocument&&a(e.contentDocument)?e.contentDocument:(E(e,"template")&&(e=e.content||e),I.merge([],e.childNodes))}},(function(e,t){I.fn[e]=function(n,r){var i=I.map(this,t,n);return"Until"!==e.slice(-5)&&(r=n),r&&"string"==typeof r&&(i=I.filter(r,i)),this.length>1&&(N[e]||I.uniqueSort(i),P.test(e)&&i.reverse()),this.pushStack(i)}}));var R=/[^\x20\t\r\n\f]+/g;function z(e){return e}function F(e){throw e}function B(e,t,n,r){var i;try{e&&m(i=e.promise)?i.call(e).done(t).fail(n):e&&m(i=e.then)?i.call(e,t,n):t.apply(void 0,[e].slice(r))}catch(e){n.apply(void 0,[e])}}I.Callbacks=function(e){e="string"==typeof e?function(e){var t={};return I.each(e.match(R)||[],(function(e,n){t[n]=!0})),t}(e):I.extend({},e);var t,n,r,i,o=[],a=[],s=-1,l=function(){for(i=i||e.once,r=t=!0;a.length;s=-1)for(n=a.shift();++s<o.length;)!1===o[s].apply(n[0],n[1])&&e.stopOnFalse&&(s=o.length,n=!1);e.memory||(n=!1),t=!1,i&&(o=n?[]:"")},u={add:function(){return o&&(n&&!t&&(s=o.length-1,a.push(n)),function t(n){I.each(n,(function(n,r){m(r)?e.unique&&u.has(r)||o.push(r):r&&r.length&&"string"!==_(r)&&t(r)}))}(arguments),n&&!t&&l()),this},remove:function(){return I.each(arguments,(function(e,t){for(var n;(n=I.inArray(t,o,n))>-1;)o.splice(n,1),n<=s&&s--})),this},has:function(e){return e?I.inArray(e,o)>-1:o.length>0},empty:function(){return o&&(o=[]),this},disable:function(){return i=a=[],o=n="",this},disabled:function(){return!o},lock:function(){return i=a=[],n||t||(o=n=""),this},locked:function(){return!!i},fireWith:function(e,n){return i||(n=[e,(n=n||[]).slice?n.slice():n],a.push(n),t||l()),this},fire:function(){return u.fireWith(this,arguments),this},fired:function(){return!!r}};return u},I.extend({Deferred:function(e){var t=[["notify","progress",I.Callbacks("memory"),I.Callbacks("memory"),2],["resolve","done",I.Callbacks("once memory"),I.Callbacks("once memory"),0,"resolved"],["reject","fail",I.Callbacks("once memory"),I.Callbacks("once memory"),1,"rejected"]],r="pending",i={state:function(){return r},always:function(){return o.done(arguments).fail(arguments),this},catch:function(e){return i.then(null,e)},pipe:function(){var e=arguments;return I.Deferred((function(n){I.each(t,(function(t,r){var i=m(e[r[4]])&&e[r[4]];o[r[1]]((function(){var e=i&&i.apply(this,arguments);e&&m(e.promise)?e.promise().progress(n.notify).done(n.resolve).fail(n.reject):n[r[0]+"With"](this,i?[e]:arguments)}))})),e=null})).promise()},then:function(e,r,i){var o=0;function a(e,t,r,i){return function(){var s=this,l=arguments,u=function(){var n,u;if(!(e<o)){if((n=r.apply(s,l))===t.promise())throw new TypeError("Thenable self-resolution");u=n&&("object"==typeof n||"function"==typeof n)&&n.then,m(u)?i?u.call(n,a(o,t,z,i),a(o,t,F,i)):(o++,u.call(n,a(o,t,z,i),a(o,t,F,i),a(o,t,z,t.notifyWith))):(r!==z&&(s=void 0,l=[n]),(i||t.resolveWith)(s,l))}},c=i?u:function(){try{u()}catch(n){I.Deferred.exceptionHook&&I.Deferred.exceptionHook(n,c.stackTrace),e+1>=o&&(r!==F&&(s=void 0,l=[n]),t.rejectWith(s,l))}};e?c():(I.Deferred.getStackHook&&(c.stackTrace=I.Deferred.getStackHook()),n.setTimeout(c))}}return I.Deferred((function(n){t[0][3].add(a(0,n,m(i)?i:z,n.notifyWith)),t[1][3].add(a(0,n,m(e)?e:z)),t[2][3].add(a(0,n,m(r)?r:F))})).promise()},promise:function(e){return null!=e?I.extend(e,i):i}},o={};return I.each(t,(function(e,n){var a=n[2],s=n[5];i[n[1]]=a.add,s&&a.add((function(){r=s}),t[3-e][2].disable,t[3-e][3].disable,t[0][2].lock,t[0][3].lock),a.add(n[3].fire),o[n[0]]=function(){return o[n[0]+"With"](this===o?void 0:this,arguments),this},o[n[0]+"With"]=a.fireWith})),i.promise(o),e&&e.call(o,o),o},when:function(e){var t=arguments.length,n=t,r=Array(n),i=s.call(arguments),o=I.Deferred(),a=function(e){return function(n){r[e]=this,i[e]=arguments.length>1?s.call(arguments):n,--t||o.resolveWith(r,i)}};if(t<=1&&(B(e,o.done(a(n)).resolve,o.reject,!t),"pending"===o.state()||m(i[n]&&i[n].then)))return o.then();for(;n--;)B(i[n],a(n),o.reject);return o.promise()}});var H=/^(Eval|Internal|Range|Reference|Syntax|Type|URI)Error$/;I.Deferred.exceptionHook=function(e,t){n.console&&n.console.warn&&e&&H.test(e.name)&&n.console.warn("jQuery.Deferred exception: "+e.message,e.stack,t)},I.readyException=function(e){n.setTimeout((function(){throw e}))};var U=I.Deferred();function V(){b.removeEventListener("DOMContentLoaded",V),n.removeEventListener("load",V),I.ready()}I.fn.ready=function(e){return U.then(e).catch((function(e){I.readyException(e)})),this},I.extend({isReady:!1,readyWait:1,ready:function(e){(!0===e?--I.readyWait:I.isReady)||(I.isReady=!0,!0!==e&&--I.readyWait>0||U.resolveWith(b,[I]))}}),I.ready.then=U.then,"complete"===b.readyState||"loading"!==b.readyState&&!b.documentElement.doScroll?n.setTimeout(I.ready):(b.addEventListener("DOMContentLoaded",V),n.addEventListener("load",V));var Y=function(e,t,n,r,i,o,a){var s=0,l=e.length,u=null==n;if("object"===_(n))for(s in i=!0,n)Y(e,t,s,n[s],!0,o,a);else if(void 0!==r&&(i=!0,m(r)||(a=!0),u&&(a?(t.call(e,r),t=null):(u=t,t=function(e,t,n){return u.call(I(e),n)})),t))for(;s<l;s++)t(e[s],n,a?r:r.call(e[s],s,t(e[s],n)));return i?e:u?t.call(e):l?t(e[0],n):o},W=/^-ms-/,G=/-([a-z])/g;function X(e,t){return t.toUpperCase()}function q(e){return e.replace(W,"ms-").replace(G,X)}var Z=function(e){return 1===e.nodeType||9===e.nodeType||!+e.nodeType};function Q(){this.expando=I.expando+Q.uid++}Q.uid=1,Q.prototype={cache:function(e){var t=e[this.expando];return t||(t={},Z(e)&&(e.nodeType?e[this.expando]=t:Object.defineProperty(e,this.expando,{value:t,configurable:!0}))),t},set:function(e,t,n){var r,i=this.cache(e);if("string"==typeof t)i[q(t)]=n;else for(r in t)i[q(r)]=t[r];return i},get:function(e,t){return void 0===t?this.cache(e):e[this.expando]&&e[this.expando][q(t)]},access:function(e,t,n){return void 0===t||t&&"string"==typeof t&&void 0===n?this.get(e,t):(this.set(e,t,n),void 0!==n?n:t)},remove:function(e,t){var n,r=e[this.expando];if(void 0!==r){if(void 0!==t){n=(t=Array.isArray(t)?t.map(q):(t=q(t))in r?[t]:t.match(R)||[]).length;for(;n--;)delete r[t[n]]}(void 0===t||I.isEmptyObject(r))&&(e.nodeType?e[this.expando]=void 0:delete e[this.expando])}},hasData:function(e){var t=e[this.expando];return void 0!==t&&!I.isEmptyObject(t)}};var K=new Q,J=new Q,$=/^(?:\{[\w\W]*\}|\[[\w\W]*\])$/,ee=/[A-Z]/g;function te(e,t,n){var r;if(void 0===n&&1===e.nodeType)if(r="data-"+t.replace(ee,"-$&").toLowerCase(),"string"==typeof(n=e.getAttribute(r))){try{n=function(e){return"true"===e||"false"!==e&&("null"===e?null:e===+e+""?+e:$.test(e)?JSON.parse(e):e)}(n)}catch(e){}J.set(e,t,n)}else n=void 0;return n}I.extend({hasData:function(e){return J.hasData(e)||K.hasData(e)},data:function(e,t,n){return J.access(e,t,n)},removeData:function(e,t){J.remove(e,t)},_data:function(e,t,n){return K.access(e,t,n)},_removeData:function(e,t){K.remove(e,t)}}),I.fn.extend({data:function(e,t){var n,r,i,o=this[0],a=o&&o.attributes;if(void 0===e){if(this.length&&(i=J.get(o),1===o.nodeType&&!K.get(o,"hasDataAttrs"))){for(n=a.length;n--;)a[n]&&0===(r=a[n].name).indexOf("data-")&&(r=q(r.slice(5)),te(o,r,i[r]));K.set(o,"hasDataAttrs",!0)}return i}return"object"==typeof e?this.each((function(){J.set(this,e)})):Y(this,(function(t){var n;if(o&&void 0===t)return void 0!==(n=J.get(o,e))||void 0!==(n=te(o,e))?n:void 0;this.each((function(){J.set(this,e,t)}))}),null,t,arguments.length>1,null,!0)},removeData:function(e){return this.each((function(){J.remove(this,e)}))}}),I.extend({queue:function(e,t,n){var r;if(e)return t=(t||"fx")+"queue",r=K.get(e,t),n&&(!r||Array.isArray(n)?r=K.access(e,t,I.makeArray(n)):r.push(n)),r||[]},dequeue:function(e,t){t=t||"fx";var n=I.queue(e,t),r=n.length,i=n.shift(),o=I._queueHooks(e,t);"inprogress"===i&&(i=n.shift(),r--),i&&("fx"===t&&n.unshift("inprogress"),delete o.stop,i.call(e,(function(){I.dequeue(e,t)}),o)),!r&&o&&o.empty.fire()},_queueHooks:function(e,t){var n=t+"queueHooks";return K.get(e,n)||K.access(e,n,{empty:I.Callbacks("once memory").add((function(){K.remove(e,[t+"queue",n])}))})}}),I.fn.extend({queue:function(e,t){var n=2;return"string"!=typeof e&&(t=e,e="fx",n--),arguments.length<n?I.queue(this[0],e):void 0===t?this:this.each((function(){var n=I.queue(this,e,t);I._queueHooks(this,e),"fx"===e&&"inprogress"!==n[0]&&I.dequeue(this,e)}))},dequeue:function(e){return this.each((function(){I.dequeue(this,e)}))},clearQueue:function(e){return this.queue(e||"fx",[])},promise:function(e,t){var n,r=1,i=I.Deferred(),o=this,a=this.length,s=function(){--r||i.resolveWith(o,[o])};for("string"!=typeof e&&(t=e,e=void 0),e=e||"fx";a--;)(n=K.get(o[a],e+"queueHooks"))&&n.empty&&(r++,n.empty.add(s));return s(),i.promise(t)}});var ne=/[+-]?(?:\d*\.|)\d+(?:[eE][+-]?\d+|)/.source,re=new RegExp("^(?:([+-])=|)("+ne+")([a-z%]*)$","i"),ie=["Top","Right","Bottom","Left"],oe=b.documentElement,ae=function(e){return I.contains(e.ownerDocument,e)},se={composed:!0};oe.getRootNode&&(ae=function(e){return I.contains(e.ownerDocument,e)||e.getRootNode(se)===e.ownerDocument});var le=function(e,t){return"none"===(e=t||e).style.display||""===e.style.display&&ae(e)&&"none"===I.css(e,"display")};function ue(e,t,n,r){var i,o,a=20,s=r?function(){return r.cur()}:function(){return I.css(e,t,"")},l=s(),u=n&&n[3]||(I.cssNumber[t]?"":"px"),c=e.nodeType&&(I.cssNumber[t]||"px"!==u&&+l)&&re.exec(I.css(e,t));if(c&&c[3]!==u){for(l/=2,u=u||c[3],c=+l||1;a--;)I.style(e,t,c+u),(1-o)*(1-(o=s()/l||.5))<=0&&(a=0),c/=o;c*=2,I.style(e,t,c+u),n=n||[]}return n&&(c=+c||+l||0,i=n[1]?c+(n[1]+1)*n[2]:+n[2],r&&(r.unit=u,r.start=c,r.end=i)),i}var ce={};function de(e){var t,n=e.ownerDocument,r=e.nodeName,i=ce[r];return i||(t=n.body.appendChild(n.createElement(r)),i=I.css(t,"display"),t.parentNode.removeChild(t),"none"===i&&(i="block"),ce[r]=i,i)}function he(e,t){for(var n,r,i=[],o=0,a=e.length;o<a;o++)(r=e[o]).style&&(n=r.style.display,t?("none"===n&&(i[o]=K.get(r,"display")||null,i[o]||(r.style.display="")),""===r.style.display&&le(r)&&(i[o]=de(r))):"none"!==n&&(i[o]="none",K.set(r,"display",n)));for(o=0;o<a;o++)null!=i[o]&&(e[o].style.display=i[o]);return e}I.fn.extend({show:function(){return he(this,!0)},hide:function(){return he(this)},toggle:function(e){return"boolean"==typeof e?e?this.show():this.hide():this.each((function(){le(this)?I(this).show():I(this).hide()}))}});var fe,pe,ge=/^(?:checkbox|radio)$/i,ve=/<([a-z][^\/\0>\x20\t\r\n\f]*)/i,me=/^$|^module$|\/(?:java|ecma)script/i;fe=b.createDocumentFragment().appendChild(b.createElement("div")),(pe=b.createElement("input")).setAttribute("type","radio"),pe.setAttribute("checked","checked"),pe.setAttribute("name","t"),fe.appendChild(pe),v.checkClone=fe.cloneNode(!0).cloneNode(!0).lastChild.checked,fe.innerHTML="<textarea>x</textarea>",v.noCloneChecked=!!fe.cloneNode(!0).lastChild.defaultValue,fe.innerHTML="<option></option>",v.option=!!fe.lastChild;var ye={thead:[1,"<table>","</table>"],col:[2,"<table><colgroup>","</colgroup></table>"],tr:[2,"<table><tbody>","</tbody></table>"],td:[3,"<table><tbody><tr>","</tr></tbody></table>"],_default:[0,"",""]};function be(e,t){var n;return n=void 0!==e.getElementsByTagName?e.getElementsByTagName(t||"*"):void 0!==e.querySelectorAll?e.querySelectorAll(t||"*"):[],void 0===t||t&&E(e,t)?I.merge([e],n):n}function Ce(e,t){for(var n=0,r=e.length;n<r;n++)K.set(e[n],"globalEval",!t||K.get(t[n],"globalEval"))}ye.tbody=ye.tfoot=ye.colgroup=ye.caption=ye.thead,ye.th=ye.td,v.option||(ye.optgroup=ye.option=[1,"<select multiple='multiple'>","</select>"]);var Ae=/<|&#?\w+;/;function _e(e,t,n,r,i){for(var o,a,s,l,u,c,d=t.createDocumentFragment(),h=[],f=0,p=e.length;f<p;f++)if((o=e[f])||0===o)if("object"===_(o))I.merge(h,o.nodeType?[o]:o);else if(Ae.test(o)){for(a=a||d.appendChild(t.createElement("div")),s=(ve.exec(o)||["",""])[1].toLowerCase(),l=ye[s]||ye._default,a.innerHTML=l[1]+I.htmlPrefilter(o)+l[2],c=l[0];c--;)a=a.lastChild;I.merge(h,a.childNodes),(a=d.firstChild).textContent=""}else h.push(t.createTextNode(o));for(d.textContent="",f=0;o=h[f++];)if(r&&I.inArray(o,r)>-1)i&&i.push(o);else if(u=ae(o),a=be(d.appendChild(o),"script"),u&&Ce(a),n)for(c=0;o=a[c++];)me.test(o.type||"")&&n.push(o);return d}var Ie=/^key/,xe=/^(?:mouse|pointer|contextmenu|drag|drop)|click/,we=/^([^.]*)(?:\.(.+)|)/;function Me(){return!0}function Se(){return!1}function Te(e,t){return e===function(){try{return b.activeElement}catch(e){}}()==("focus"===t)}function Ee(e,t,n,r,i,o){var a,s;if("object"==typeof t){for(s in"string"!=typeof n&&(r=r||n,n=void 0),t)Ee(e,s,n,r,t[s],o);return e}if(null==r&&null==i?(i=n,r=n=void 0):null==i&&("string"==typeof n?(i=r,r=void 0):(i=r,r=n,n=void 0)),!1===i)i=Se;else if(!i)return e;return 1===o&&(a=i,(i=function(e){return I().off(e),a.apply(this,arguments)}).guid=a.guid||(a.guid=I.guid++)),e.each((function(){I.event.add(this,t,i,r,n)}))}function Oe(e,t,n){n?(K.set(e,t,!1),I.event.add(e,t,{namespace:!1,handler:function(e){var r,i,o=K.get(this,t);if(1&e.isTrigger&&this[t]){if(o.length)(I.event.special[t]||{}).delegateType&&e.stopPropagation();else if(o=s.call(arguments),K.set(this,t,o),r=n(this,t),this[t](),o!==(i=K.get(this,t))||r?K.set(this,t,!1):i={},o!==i)return e.stopImmediatePropagation(),e.preventDefault(),i.value}else o.length&&(K.set(this,t,{value:I.event.trigger(I.extend(o[0],I.Event.prototype),o.slice(1),this)}),e.stopImmediatePropagation())}})):void 0===K.get(e,t)&&I.event.add(e,t,Me)}I.event={global:{},add:function(e,t,n,r,i){var o,a,s,l,u,c,d,h,f,p,g,v=K.get(e);if(Z(e))for(n.handler&&(n=(o=n).handler,i=o.selector),i&&I.find.matchesSelector(oe,i),n.guid||(n.guid=I.guid++),(l=v.events)||(l=v.events=Object.create(null)),(a=v.handle)||(a=v.handle=function(t){return void 0!==I&&I.event.triggered!==t.type?I.event.dispatch.apply(e,arguments):void 0}),u=(t=(t||"").match(R)||[""]).length;u--;)f=g=(s=we.exec(t[u])||[])[1],p=(s[2]||"").split(".").sort(),f&&(d=I.event.special[f]||{},f=(i?d.delegateType:d.bindType)||f,d=I.event.special[f]||{},c=I.extend({type:f,origType:g,data:r,handler:n,guid:n.guid,selector:i,needsContext:i&&I.expr.match.needsContext.test(i),namespace:p.join(".")},o),(h=l[f])||((h=l[f]=[]).delegateCount=0,d.setup&&!1!==d.setup.call(e,r,p,a)||e.addEventListener&&e.addEventListener(f,a)),d.add&&(d.add.call(e,c),c.handler.guid||(c.handler.guid=n.guid)),i?h.splice(h.delegateCount++,0,c):h.push(c),I.event.global[f]=!0)},remove:function(e,t,n,r,i){var o,a,s,l,u,c,d,h,f,p,g,v=K.hasData(e)&&K.get(e);if(v&&(l=v.events)){for(u=(t=(t||"").match(R)||[""]).length;u--;)if(f=g=(s=we.exec(t[u])||[])[1],p=(s[2]||"").split(".").sort(),f){for(d=I.event.special[f]||{},h=l[f=(r?d.delegateType:d.bindType)||f]||[],s=s[2]&&new RegExp("(^|\\.)"+p.join("\\.(?:.*\\.|)")+"(\\.|$)"),a=o=h.length;o--;)c=h[o],!i&&g!==c.origType||n&&n.guid!==c.guid||s&&!s.test(c.namespace)||r&&r!==c.selector&&("**"!==r||!c.selector)||(h.splice(o,1),c.selector&&h.delegateCount--,d.remove&&d.remove.call(e,c));a&&!h.length&&(d.teardown&&!1!==d.teardown.call(e,p,v.handle)||I.removeEvent(e,f,v.handle),delete l[f])}else for(f in l)I.event.remove(e,f+t[u],n,r,!0);I.isEmptyObject(l)&&K.remove(e,"handle events")}},dispatch:function(e){var t,n,r,i,o,a,s=new Array(arguments.length),l=I.event.fix(e),u=(K.get(this,"events")||Object.create(null))[l.type]||[],c=I.event.special[l.type]||{};for(s[0]=l,t=1;t<arguments.length;t++)s[t]=arguments[t];if(l.delegateTarget=this,!c.preDispatch||!1!==c.preDispatch.call(this,l)){for(a=I.event.handlers.call(this,l,u),t=0;(i=a[t++])&&!l.isPropagationStopped();)for(l.currentTarget=i.elem,n=0;(o=i.handlers[n++])&&!l.isImmediatePropagationStopped();)l.rnamespace&&!1!==o.namespace&&!l.rnamespace.test(o.namespace)||(l.handleObj=o,l.data=o.data,void 0!==(r=((I.event.special[o.origType]||{}).handle||o.handler).apply(i.elem,s))&&!1===(l.result=r)&&(l.preventDefault(),l.stopPropagation()));return c.postDispatch&&c.postDispatch.call(this,l),l.result}},handlers:function(e,t){var n,r,i,o,a,s=[],l=t.delegateCount,u=e.target;if(l&&u.nodeType&&!("click"===e.type&&e.button>=1))for(;u!==this;u=u.parentNode||this)if(1===u.nodeType&&("click"!==e.type||!0!==u.disabled)){for(o=[],a={},n=0;n<l;n++)void 0===a[i=(r=t[n]).selector+" "]&&(a[i]=r.needsContext?I(i,this).index(u)>-1:I.find(i,this,null,[u]).length),a[i]&&o.push(r);o.length&&s.push({elem:u,handlers:o})}return u=this,l<t.length&&s.push({elem:u,handlers:t.slice(l)}),s},addProp:function(e,t){Object.defineProperty(I.Event.prototype,e,{enumerable:!0,configurable:!0,get:m(t)?function(){if(this.originalEvent)return t(this.originalEvent)}:function(){if(this.originalEvent)return this.originalEvent[e]},set:function(t){Object.defineProperty(this,e,{enumerable:!0,configurable:!0,writable:!0,value:t})}})},fix:function(e){return e[I.expando]?e:new I.Event(e)},special:{load:{noBubble:!0},click:{setup:function(e){var t=this||e;return ge.test(t.type)&&t.click&&E(t,"input")&&Oe(t,"click",Me),!1},trigger:function(e){var t=this||e;return ge.test(t.type)&&t.click&&E(t,"input")&&Oe(t,"click"),!0},_default:function(e){var t=e.target;return ge.test(t.type)&&t.click&&E(t,"input")&&K.get(t,"click")||E(t,"a")}},beforeunload:{postDispatch:function(e){void 0!==e.result&&e.originalEvent&&(e.originalEvent.returnValue=e.result)}}}},I.removeEvent=function(e,t,n){e.removeEventListener&&e.removeEventListener(t,n)},I.Event=function(e,t){if(!(this instanceof I.Event))return new I.Event(e,t);e&&e.type?(this.originalEvent=e,this.type=e.type,this.isDefaultPrevented=e.defaultPrevented||void 0===e.defaultPrevented&&!1===e.returnValue?Me:Se,this.target=e.target&&3===e.target.nodeType?e.target.parentNode:e.target,this.currentTarget=e.currentTarget,this.relatedTarget=e.relatedTarget):this.type=e,t&&I.extend(this,t),this.timeStamp=e&&e.timeStamp||Date.now(),this[I.expando]=!0},I.Event.prototype={constructor:I.Event,isDefaultPrevented:Se,isPropagationStopped:Se,isImmediatePropagationStopped:Se,isSimulated:!1,preventDefault:function(){var e=this.originalEvent;this.isDefaultPrevented=Me,e&&!this.isSimulated&&e.preventDefault()},stopPropagation:function(){var e=this.originalEvent;this.isPropagationStopped=Me,e&&!this.isSimulated&&e.stopPropagation()},stopImmediatePropagation:function(){var e=this.originalEvent;this.isImmediatePropagationStopped=Me,e&&!this.isSimulated&&e.stopImmediatePropagation(),this.stopPropagation()}},I.each({altKey:!0,bubbles:!0,cancelable:!0,changedTouches:!0,ctrlKey:!0,detail:!0,eventPhase:!0,metaKey:!0,pageX:!0,pageY:!0,shiftKey:!0,view:!0,char:!0,code:!0,charCode:!0,key:!0,keyCode:!0,button:!0,buttons:!0,clientX:!0,clientY:!0,offsetX:!0,offsetY:!0,pointerId:!0,pointerType:!0,screenX:!0,screenY:!0,targetTouches:!0,toElement:!0,touches:!0,which:function(e){var t=e.button;return null==e.which&&Ie.test(e.type)?null!=e.charCode?e.charCode:e.keyCode:!e.which&&void 0!==t&&xe.test(e.type)?1&t?1:2&t?3:4&t?2:0:e.which}},I.event.addProp),I.each({focus:"focusin",blur:"focusout"},(function(e,t){I.event.special[e]={setup:function(){return Oe(this,e,Te),!1},trigger:function(){return Oe(this,e),!0},delegateType:t}})),I.each({mouseenter:"mouseover",mouseleave:"mouseout",pointerenter:"pointerover",pointerleave:"pointerout"},(function(e,t){I.event.special[e]={delegateType:t,bindType:t,handle:function(e){var n,r=this,i=e.relatedTarget,o=e.handleObj;return i&&(i===r||I.contains(r,i))||(e.type=o.origType,n=o.handler.apply(this,arguments),e.type=t),n}}})),I.fn.extend({on:function(e,t,n,r){return Ee(this,e,t,n,r)},one:function(e,t,n,r){return Ee(this,e,t,n,r,1)},off:function(e,t,n){var r,i;if(e&&e.preventDefault&&e.handleObj)return r=e.handleObj,I(e.delegateTarget).off(r.namespace?r.origType+"."+r.namespace:r.origType,r.selector,r.handler),this;if("object"==typeof e){for(i in e)this.off(i,t,e[i]);return this}return!1!==t&&"function"!=typeof t||(n=t,t=void 0),!1===n&&(n=Se),this.each((function(){I.event.remove(this,e,n,t)}))}});var ke=/<script|<style|<link/i,Le=/checked\s*(?:[^=]|=\s*.checked.)/i,De=/^\s*<!(?:\[CDATA\[|--)|(?:\]\]|--)>\s*$/g;function Pe(e,t){return E(e,"table")&&E(11!==t.nodeType?t:t.firstChild,"tr")&&I(e).children("tbody")[0]||e}function Ne(e){return e.type=(null!==e.getAttribute("type"))+"/"+e.type,e}function je(e){return"true/"===(e.type||"").slice(0,5)?e.type=e.type.slice(5):e.removeAttribute("type"),e}function Re(e,t){var n,r,i,o,a,s;if(1===t.nodeType){if(K.hasData(e)&&(s=K.get(e).events))for(i in K.remove(t,"handle events"),s)for(n=0,r=s[i].length;n<r;n++)I.event.add(t,i,s[i][n]);J.hasData(e)&&(o=J.access(e),a=I.extend({},o),J.set(t,a))}}function ze(e,t){var n=t.nodeName.toLowerCase();"input"===n&&ge.test(e.type)?t.checked=e.checked:"input"!==n&&"textarea"!==n||(t.defaultValue=e.defaultValue)}function Fe(e,t,n,r){t=l(t);var i,o,a,s,u,c,d=0,h=e.length,f=h-1,p=t[0],g=m(p);if(g||h>1&&"string"==typeof p&&!v.checkClone&&Le.test(p))return e.each((function(i){var o=e.eq(i);g&&(t[0]=p.call(this,i,o.html())),Fe(o,t,n,r)}));if(h&&(o=(i=_e(t,e[0].ownerDocument,!1,e,r)).firstChild,1===i.childNodes.length&&(i=o),o||r)){for(s=(a=I.map(be(i,"script"),Ne)).length;d<h;d++)u=i,d!==f&&(u=I.clone(u,!0,!0),s&&I.merge(a,be(u,"script"))),n.call(e[d],u,d);if(s)for(c=a[a.length-1].ownerDocument,I.map(a,je),d=0;d<s;d++)u=a[d],me.test(u.type||"")&&!K.access(u,"globalEval")&&I.contains(c,u)&&(u.src&&"module"!==(u.type||"").toLowerCase()?I._evalUrl&&!u.noModule&&I._evalUrl(u.src,{nonce:u.nonce||u.getAttribute("nonce")},c):A(u.textContent.replace(De,""),u,c))}return e}function Be(e,t,n){for(var r,i=t?I.filter(t,e):e,o=0;null!=(r=i[o]);o++)n||1!==r.nodeType||I.cleanData(be(r)),r.parentNode&&(n&&ae(r)&&Ce(be(r,"script")),r.parentNode.removeChild(r));return e}I.extend({htmlPrefilter:function(e){return e},clone:function(e,t,n){var r,i,o,a,s=e.cloneNode(!0),l=ae(e);if(!(v.noCloneChecked||1!==e.nodeType&&11!==e.nodeType||I.isXMLDoc(e)))for(a=be(s),r=0,i=(o=be(e)).length;r<i;r++)ze(o[r],a[r]);if(t)if(n)for(o=o||be(e),a=a||be(s),r=0,i=o.length;r<i;r++)Re(o[r],a[r]);else Re(e,s);return(a=be(s,"script")).length>0&&Ce(a,!l&&be(e,"script")),s},cleanData:function(e){for(var t,n,r,i=I.event.special,o=0;void 0!==(n=e[o]);o++)if(Z(n)){if(t=n[K.expando]){if(t.events)for(r in t.events)i[r]?I.event.remove(n,r):I.removeEvent(n,r,t.handle);n[K.expando]=void 0}n[J.expando]&&(n[J.expando]=void 0)}}}),I.fn.extend({detach:function(e){return Be(this,e,!0)},remove:function(e){return Be(this,e)},text:function(e){return Y(this,(function(e){return void 0===e?I.text(this):this.empty().each((function(){1!==this.nodeType&&11!==this.nodeType&&9!==this.nodeType||(this.textContent=e)}))}),null,e,arguments.length)},append:function(){return Fe(this,arguments,(function(e){1!==this.nodeType&&11!==this.nodeType&&9!==this.nodeType||Pe(this,e).appendChild(e)}))},prepend:function(){return Fe(this,arguments,(function(e){if(1===this.nodeType||11===this.nodeType||9===this.nodeType){var t=Pe(this,e);t.insertBefore(e,t.firstChild)}}))},before:function(){return Fe(this,arguments,(function(e){this.parentNode&&this.parentNode.insertBefore(e,this)}))},after:function(){return Fe(this,arguments,(function(e){this.parentNode&&this.parentNode.insertBefore(e,this.nextSibling)}))},empty:function(){for(var e,t=0;null!=(e=this[t]);t++)1===e.nodeType&&(I.cleanData(be(e,!1)),e.textContent="");return this},clone:function(e,t){return e=null!=e&&e,t=null==t?e:t,this.map((function(){return I.clone(this,e,t)}))},html:function(e){return Y(this,(function(e){var t=this[0]||{},n=0,r=this.length;if(void 0===e&&1===t.nodeType)return t.innerHTML;if("string"==typeof e&&!ke.test(e)&&!ye[(ve.exec(e)||["",""])[1].toLowerCase()]){e=I.htmlPrefilter(e);try{for(;n<r;n++)1===(t=this[n]||{}).nodeType&&(I.cleanData(be(t,!1)),t.innerHTML=e);t=0}catch(e){}}t&&this.empty().append(e)}),null,e,arguments.length)},replaceWith:function(){var e=[];return Fe(this,arguments,(function(t){var n=this.parentNode;I.inArray(this,e)<0&&(I.cleanData(be(this)),n&&n.replaceChild(t,this))}),e)}}),I.each({appendTo:"append",prependTo:"prepend",insertBefore:"before",insertAfter:"after",replaceAll:"replaceWith"},(function(e,t){I.fn[e]=function(e){for(var n,r=[],i=I(e),o=i.length-1,a=0;a<=o;a++)n=a===o?this:this.clone(!0),I(i[a])[t](n),u.apply(r,n.get());return this.pushStack(r)}}));var He=new RegExp("^("+ne+")(?!px)[a-z%]+$","i"),Ue=function(e){var t=e.ownerDocument.defaultView;return t&&t.opener||(t=n),t.getComputedStyle(e)},Ve=function(e,t,n){var r,i,o={};for(i in t)o[i]=e.style[i],e.style[i]=t[i];for(i in r=n.call(e),t)e.style[i]=o[i];return r},Ye=new RegExp(ie.join("|"),"i");function We(e,t,n){var r,i,o,a,s=e.style;return(n=n||Ue(e))&&(""!==(a=n.getPropertyValue(t)||n[t])||ae(e)||(a=I.style(e,t)),!v.pixelBoxStyles()&&He.test(a)&&Ye.test(t)&&(r=s.width,i=s.minWidth,o=s.maxWidth,s.minWidth=s.maxWidth=s.width=a,a=n.width,s.width=r,s.minWidth=i,s.maxWidth=o)),void 0!==a?a+"":a}function Ge(e,t){return{get:function(){if(!e())return(this.get=t).apply(this,arguments);delete this.get}}}!function(){function e(){if(c){u.style.cssText="position:absolute;left:-11111px;width:60px;margin-top:1px;padding:0;border:0",c.style.cssText="position:relative;display:block;box-sizing:border-box;overflow:scroll;margin:auto;border:1px;padding:1px;width:60%;top:1%",oe.appendChild(u).appendChild(c);var e=n.getComputedStyle(c);r="1%"!==e.top,l=12===t(e.marginLeft),c.style.right="60%",a=36===t(e.right),i=36===t(e.width),c.style.position="absolute",o=12===t(c.offsetWidth/3),oe.removeChild(u),c=null}}function t(e){return Math.round(parseFloat(e))}var r,i,o,a,s,l,u=b.createElement("div"),c=b.createElement("div");c.style&&(c.style.backgroundClip="content-box",c.cloneNode(!0).style.backgroundClip="",v.clearCloneStyle="content-box"===c.style.backgroundClip,I.extend(v,{boxSizingReliable:function(){return e(),i},pixelBoxStyles:function(){return e(),a},pixelPosition:function(){return e(),r},reliableMarginLeft:function(){return e(),l},scrollboxSize:function(){return e(),o},reliableTrDimensions:function(){var e,t,r,i;return null==s&&(e=b.createElement("table"),t=b.createElement("tr"),r=b.createElement("div"),e.style.cssText="position:absolute;left:-11111px",t.style.height="1px",r.style.height="9px",oe.appendChild(e).appendChild(t).appendChild(r),i=n.getComputedStyle(t),s=parseInt(i.height)>3,oe.removeChild(e)),s}}))}();var Xe=["Webkit","Moz","ms"],qe=b.createElement("div").style,Ze={};function Qe(e){var t=I.cssProps[e]||Ze[e];return t||(e in qe?e:Ze[e]=function(e){for(var t=e[0].toUpperCase()+e.slice(1),n=Xe.length;n--;)if((e=Xe[n]+t)in qe)return e}(e)||e)}var Ke=/^(none|table(?!-c[ea]).+)/,Je=/^--/,$e={position:"absolute",visibility:"hidden",display:"block"},et={letterSpacing:"0",fontWeight:"400"};function tt(e,t,n){var r=re.exec(t);return r?Math.max(0,r[2]-(n||0))+(r[3]||"px"):t}function nt(e,t,n,r,i,o){var a="width"===t?1:0,s=0,l=0;if(n===(r?"border":"content"))return 0;for(;a<4;a+=2)"margin"===n&&(l+=I.css(e,n+ie[a],!0,i)),r?("content"===n&&(l-=I.css(e,"padding"+ie[a],!0,i)),"margin"!==n&&(l-=I.css(e,"border"+ie[a]+"Width",!0,i))):(l+=I.css(e,"padding"+ie[a],!0,i),"padding"!==n?l+=I.css(e,"border"+ie[a]+"Width",!0,i):s+=I.css(e,"border"+ie[a]+"Width",!0,i));return!r&&o>=0&&(l+=Math.max(0,Math.ceil(e["offset"+t[0].toUpperCase()+t.slice(1)]-o-l-s-.5))||0),l}function rt(e,t,n){var r=Ue(e),i=(!v.boxSizingReliable()||n)&&"border-box"===I.css(e,"boxSizing",!1,r),o=i,a=We(e,t,r),s="offset"+t[0].toUpperCase()+t.slice(1);if(He.test(a)){if(!n)return a;a="auto"}return(!v.boxSizingReliable()&&i||!v.reliableTrDimensions()&&E(e,"tr")||"auto"===a||!parseFloat(a)&&"inline"===I.css(e,"display",!1,r))&&e.getClientRects().length&&(i="border-box"===I.css(e,"boxSizing",!1,r),(o=s in e)&&(a=e[s])),(a=parseFloat(a)||0)+nt(e,t,n||(i?"border":"content"),o,r,a)+"px"}function it(e,t,n,r,i){return new it.prototype.init(e,t,n,r,i)}I.extend({cssHooks:{opacity:{get:function(e,t){if(t){var n=We(e,"opacity");return""===n?"1":n}}}},cssNumber:{animationIterationCount:!0,columnCount:!0,fillOpacity:!0,flexGrow:!0,flexShrink:!0,fontWeight:!0,gridArea:!0,gridColumn:!0,gridColumnEnd:!0,gridColumnStart:!0,gridRow:!0,gridRowEnd:!0,gridRowStart:!0,lineHeight:!0,opacity:!0,order:!0,orphans:!0,widows:!0,zIndex:!0,zoom:!0},cssProps:{},style:function(e,t,n,r){if(e&&3!==e.nodeType&&8!==e.nodeType&&e.style){var i,o,a,s=q(t),l=Je.test(t),u=e.style;if(l||(t=Qe(s)),a=I.cssHooks[t]||I.cssHooks[s],void 0===n)return a&&"get"in a&&void 0!==(i=a.get(e,!1,r))?i:u[t];"string"===(o=typeof n)&&(i=re.exec(n))&&i[1]&&(n=ue(e,t,i),o="number"),null!=n&&n==n&&("number"!==o||l||(n+=i&&i[3]||(I.cssNumber[s]?"":"px")),v.clearCloneStyle||""!==n||0!==t.indexOf("background")||(u[t]="inherit"),a&&"set"in a&&void 0===(n=a.set(e,n,r))||(l?u.setProperty(t,n):u[t]=n))}},css:function(e,t,n,r){var i,o,a,s=q(t);return Je.test(t)||(t=Qe(s)),(a=I.cssHooks[t]||I.cssHooks[s])&&"get"in a&&(i=a.get(e,!0,n)),void 0===i&&(i=We(e,t,r)),"normal"===i&&t in et&&(i=et[t]),""===n||n?(o=parseFloat(i),!0===n||isFinite(o)?o||0:i):i}}),I.each(["height","width"],(function(e,t){I.cssHooks[t]={get:function(e,n,r){if(n)return!Ke.test(I.css(e,"display"))||e.getClientRects().length&&e.getBoundingClientRect().width?rt(e,t,r):Ve(e,$e,(function(){return rt(e,t,r)}))},set:function(e,n,r){var i,o=Ue(e),a=!v.scrollboxSize()&&"absolute"===o.position,s=(a||r)&&"border-box"===I.css(e,"boxSizing",!1,o),l=r?nt(e,t,r,s,o):0;return s&&a&&(l-=Math.ceil(e["offset"+t[0].toUpperCase()+t.slice(1)]-parseFloat(o[t])-nt(e,t,"border",!1,o)-.5)),l&&(i=re.exec(n))&&"px"!==(i[3]||"px")&&(e.style[t]=n,n=I.css(e,t)),tt(0,n,l)}}})),I.cssHooks.marginLeft=Ge(v.reliableMarginLeft,(function(e,t){if(t)return(parseFloat(We(e,"marginLeft"))||e.getBoundingClientRect().left-Ve(e,{marginLeft:0},(function(){return e.getBoundingClientRect().left})))+"px"})),I.each({margin:"",padding:"",border:"Width"},(function(e,t){I.cssHooks[e+t]={expand:function(n){for(var r=0,i={},o="string"==typeof n?n.split(" "):[n];r<4;r++)i[e+ie[r]+t]=o[r]||o[r-2]||o[0];return i}},"margin"!==e&&(I.cssHooks[e+t].set=tt)})),I.fn.extend({css:function(e,t){return Y(this,(function(e,t,n){var r,i,o={},a=0;if(Array.isArray(t)){for(r=Ue(e),i=t.length;a<i;a++)o[t[a]]=I.css(e,t[a],!1,r);return o}return void 0!==n?I.style(e,t,n):I.css(e,t)}),e,t,arguments.length>1)}}),I.Tween=it,it.prototype={constructor:it,init:function(e,t,n,r,i,o){this.elem=e,this.prop=n,this.easing=i||I.easing._default,this.options=t,this.start=this.now=this.cur(),this.end=r,this.unit=o||(I.cssNumber[n]?"":"px")},cur:function(){var e=it.propHooks[this.prop];return e&&e.get?e.get(this):it.propHooks._default.get(this)},run:function(e){var t,n=it.propHooks[this.prop];return this.options.duration?this.pos=t=I.easing[this.easing](e,this.options.duration*e,0,1,this.options.duration):this.pos=t=e,this.now=(this.end-this.start)*t+this.start,this.options.step&&this.options.step.call(this.elem,this.now,this),n&&n.set?n.set(this):it.propHooks._default.set(this),this}},it.prototype.init.prototype=it.prototype,it.propHooks={_default:{get:function(e){var t;return 1!==e.elem.nodeType||null!=e.elem[e.prop]&&null==e.elem.style[e.prop]?e.elem[e.prop]:(t=I.css(e.elem,e.prop,""))&&"auto"!==t?t:0},set:function(e){I.fx.step[e.prop]?I.fx.step[e.prop](e):1!==e.elem.nodeType||!I.cssHooks[e.prop]&&null==e.elem.style[Qe(e.prop)]?e.elem[e.prop]=e.now:I.style(e.elem,e.prop,e.now+e.unit)}}},it.propHooks.scrollTop=it.propHooks.scrollLeft={set:function(e){e.elem.nodeType&&e.elem.parentNode&&(e.elem[e.prop]=e.now)}},I.easing={linear:function(e){return e},swing:function(e){return.5-Math.cos(e*Math.PI)/2},_default:"swing"},I.fx=it.prototype.init,I.fx.step={};var ot,at,st=/^(?:toggle|show|hide)$/,lt=/queueHooks$/;function ut(){at&&(!1===b.hidden&&n.requestAnimationFrame?n.requestAnimationFrame(ut):n.setTimeout(ut,I.fx.interval),I.fx.tick())}function ct(){return n.setTimeout((function(){ot=void 0})),ot=Date.now()}function dt(e,t){var n,r=0,i={height:e};for(t=t?1:0;r<4;r+=2-t)i["margin"+(n=ie[r])]=i["padding"+n]=e;return t&&(i.opacity=i.width=e),i}function ht(e,t,n){for(var r,i=(ft.tweeners[t]||[]).concat(ft.tweeners["*"]),o=0,a=i.length;o<a;o++)if(r=i[o].call(n,t,e))return r}function ft(e,t,n){var r,i,o=0,a=ft.prefilters.length,s=I.Deferred().always((function(){delete l.elem})),l=function(){if(i)return!1;for(var t=ot||ct(),n=Math.max(0,u.startTime+u.duration-t),r=1-(n/u.duration||0),o=0,a=u.tweens.length;o<a;o++)u.tweens[o].run(r);return s.notifyWith(e,[u,r,n]),r<1&&a?n:(a||s.notifyWith(e,[u,1,0]),s.resolveWith(e,[u]),!1)},u=s.promise({elem:e,props:I.extend({},t),opts:I.extend(!0,{specialEasing:{},easing:I.easing._default},n),originalProperties:t,originalOptions:n,startTime:ot||ct(),duration:n.duration,tweens:[],createTween:function(t,n){var r=I.Tween(e,u.opts,t,n,u.opts.specialEasing[t]||u.opts.easing);return u.tweens.push(r),r},stop:function(t){var n=0,r=t?u.tweens.length:0;if(i)return this;for(i=!0;n<r;n++)u.tweens[n].run(1);return t?(s.notifyWith(e,[u,1,0]),s.resolveWith(e,[u,t])):s.rejectWith(e,[u,t]),this}}),c=u.props;for(!function(e,t){var n,r,i,o,a;for(n in e)if(i=t[r=q(n)],o=e[n],Array.isArray(o)&&(i=o[1],o=e[n]=o[0]),n!==r&&(e[r]=o,delete e[n]),(a=I.cssHooks[r])&&"expand"in a)for(n in o=a.expand(o),delete e[r],o)n in e||(e[n]=o[n],t[n]=i);else t[r]=i}(c,u.opts.specialEasing);o<a;o++)if(r=ft.prefilters[o].call(u,e,c,u.opts))return m(r.stop)&&(I._queueHooks(u.elem,u.opts.queue).stop=r.stop.bind(r)),r;return I.map(c,ht,u),m(u.opts.start)&&u.opts.start.call(e,u),u.progress(u.opts.progress).done(u.opts.done,u.opts.complete).fail(u.opts.fail).always(u.opts.always),I.fx.timer(I.extend(l,{elem:e,anim:u,queue:u.opts.queue})),u}I.Animation=I.extend(ft,{tweeners:{"*":[function(e,t){var n=this.createTween(e,t);return ue(n.elem,e,re.exec(t),n),n}]},tweener:function(e,t){m(e)?(t=e,e=["*"]):e=e.match(R);for(var n,r=0,i=e.length;r<i;r++)n=e[r],ft.tweeners[n]=ft.tweeners[n]||[],ft.tweeners[n].unshift(t)},prefilters:[function(e,t,n){var r,i,o,a,s,l,u,c,d="width"in t||"height"in t,h=this,f={},p=e.style,g=e.nodeType&&le(e),v=K.get(e,"fxshow");for(r in n.queue||(null==(a=I._queueHooks(e,"fx")).unqueued&&(a.unqueued=0,s=a.empty.fire,a.empty.fire=function(){a.unqueued||s()}),a.unqueued++,h.always((function(){h.always((function(){a.unqueued--,I.queue(e,"fx").length||a.empty.fire()}))}))),t)if(i=t[r],st.test(i)){if(delete t[r],o=o||"toggle"===i,i===(g?"hide":"show")){if("show"!==i||!v||void 0===v[r])continue;g=!0}f[r]=v&&v[r]||I.style(e,r)}if((l=!I.isEmptyObject(t))||!I.isEmptyObject(f))for(r in d&&1===e.nodeType&&(n.overflow=[p.overflow,p.overflowX,p.overflowY],null==(u=v&&v.display)&&(u=K.get(e,"display")),"none"===(c=I.css(e,"display"))&&(u?c=u:(he([e],!0),u=e.style.display||u,c=I.css(e,"display"),he([e]))),("inline"===c||"inline-block"===c&&null!=u)&&"none"===I.css(e,"float")&&(l||(h.done((function(){p.display=u})),null==u&&(c=p.display,u="none"===c?"":c)),p.display="inline-block")),n.overflow&&(p.overflow="hidden",h.always((function(){p.overflow=n.overflow[0],p.overflowX=n.overflow[1],p.overflowY=n.overflow[2]}))),l=!1,f)l||(v?"hidden"in v&&(g=v.hidden):v=K.access(e,"fxshow",{display:u}),o&&(v.hidden=!g),g&&he([e],!0),h.done((function(){for(r in g||he([e]),K.remove(e,"fxshow"),f)I.style(e,r,f[r])}))),l=ht(g?v[r]:0,r,h),r in v||(v[r]=l.start,g&&(l.end=l.start,l.start=0))}],prefilter:function(e,t){t?ft.prefilters.unshift(e):ft.prefilters.push(e)}}),I.speed=function(e,t,n){var r=e&&"object"==typeof e?I.extend({},e):{complete:n||!n&&t||m(e)&&e,duration:e,easing:n&&t||t&&!m(t)&&t};return I.fx.off?r.duration=0:"number"!=typeof r.duration&&(r.duration in I.fx.speeds?r.duration=I.fx.speeds[r.duration]:r.duration=I.fx.speeds._default),null!=r.queue&&!0!==r.queue||(r.queue="fx"),r.old=r.complete,r.complete=function(){m(r.old)&&r.old.call(this),r.queue&&I.dequeue(this,r.queue)},r},I.fn.extend({fadeTo:function(e,t,n,r){return this.filter(le).css("opacity",0).show().end().animate({opacity:t},e,n,r)},animate:function(e,t,n,r){var i=I.isEmptyObject(e),o=I.speed(t,n,r),a=function(){var t=ft(this,I.extend({},e),o);(i||K.get(this,"finish"))&&t.stop(!0)};return a.finish=a,i||!1===o.queue?this.each(a):this.queue(o.queue,a)},stop:function(e,t,n){var r=function(e){var t=e.stop;delete e.stop,t(n)};return"string"!=typeof e&&(n=t,t=e,e=void 0),t&&this.queue(e||"fx",[]),this.each((function(){var t=!0,i=null!=e&&e+"queueHooks",o=I.timers,a=K.get(this);if(i)a[i]&&a[i].stop&&r(a[i]);else for(i in a)a[i]&&a[i].stop&&lt.test(i)&&r(a[i]);for(i=o.length;i--;)o[i].elem!==this||null!=e&&o[i].queue!==e||(o[i].anim.stop(n),t=!1,o.splice(i,1));!t&&n||I.dequeue(this,e)}))},finish:function(e){return!1!==e&&(e=e||"fx"),this.each((function(){var t,n=K.get(this),r=n[e+"queue"],i=n[e+"queueHooks"],o=I.timers,a=r?r.length:0;for(n.finish=!0,I.queue(this,e,[]),i&&i.stop&&i.stop.call(this,!0),t=o.length;t--;)o[t].elem===this&&o[t].queue===e&&(o[t].anim.stop(!0),o.splice(t,1));for(t=0;t<a;t++)r[t]&&r[t].finish&&r[t].finish.call(this);delete n.finish}))}}),I.each(["toggle","show","hide"],(function(e,t){var n=I.fn[t];I.fn[t]=function(e,r,i){return null==e||"boolean"==typeof e?n.apply(this,arguments):this.animate(dt(t,!0),e,r,i)}})),I.each({slideDown:dt("show"),slideUp:dt("hide"),slideToggle:dt("toggle"),fadeIn:{opacity:"show"},fadeOut:{opacity:"hide"},fadeToggle:{opacity:"toggle"}},(function(e,t){I.fn[e]=function(e,n,r){return this.animate(t,e,n,r)}})),I.timers=[],I.fx.tick=function(){var e,t=0,n=I.timers;for(ot=Date.now();t<n.length;t++)(e=n[t])()||n[t]!==e||n.splice(t--,1);n.length||I.fx.stop(),ot=void 0},I.fx.timer=function(e){I.timers.push(e),I.fx.start()},I.fx.interval=13,I.fx.start=function(){at||(at=!0,ut())},I.fx.stop=function(){at=null},I.fx.speeds={slow:600,fast:200,_default:400},I.fn.delay=function(e,t){return e=I.fx&&I.fx.speeds[e]||e,t=t||"fx",this.queue(t,(function(t,r){var i=n.setTimeout(t,e);r.stop=function(){n.clearTimeout(i)}}))},function(){var e=b.createElement("input"),t=b.createElement("select").appendChild(b.createElement("option"));e.type="checkbox",v.checkOn=""!==e.value,v.optSelected=t.selected,(e=b.createElement("input")).value="t",e.type="radio",v.radioValue="t"===e.value}();var pt,gt=I.expr.attrHandle;I.fn.extend({attr:function(e,t){return Y(this,I.attr,e,t,arguments.length>1)},removeAttr:function(e){return this.each((function(){I.removeAttr(this,e)}))}}),I.extend({attr:function(e,t,n){var r,i,o=e.nodeType;if(3!==o&&8!==o&&2!==o)return void 0===e.getAttribute?I.prop(e,t,n):(1===o&&I.isXMLDoc(e)||(i=I.attrHooks[t.toLowerCase()]||(I.expr.match.bool.test(t)?pt:void 0)),void 0!==n?null===n?void I.removeAttr(e,t):i&&"set"in i&&void 0!==(r=i.set(e,n,t))?r:(e.setAttribute(t,n+""),n):i&&"get"in i&&null!==(r=i.get(e,t))?r:null==(r=I.find.attr(e,t))?void 0:r)},attrHooks:{type:{set:function(e,t){if(!v.radioValue&&"radio"===t&&E(e,"input")){var n=e.value;return e.setAttribute("type",t),n&&(e.value=n),t}}}},removeAttr:function(e,t){var n,r=0,i=t&&t.match(R);if(i&&1===e.nodeType)for(;n=i[r++];)e.removeAttribute(n)}}),pt={set:function(e,t,n){return!1===t?I.removeAttr(e,n):e.setAttribute(n,n),n}},I.each(I.expr.match.bool.source.match(/\w+/g),(function(e,t){var n=gt[t]||I.find.attr;gt[t]=function(e,t,r){var i,o,a=t.toLowerCase();return r||(o=gt[a],gt[a]=i,i=null!=n(e,t,r)?a:null,gt[a]=o),i}}));var vt=/^(?:input|select|textarea|button)$/i,mt=/^(?:a|area)$/i;function yt(e){return(e.match(R)||[]).join(" ")}function bt(e){return e.getAttribute&&e.getAttribute("class")||""}function Ct(e){return Array.isArray(e)?e:"string"==typeof e&&e.match(R)||[]}I.fn.extend({prop:function(e,t){return Y(this,I.prop,e,t,arguments.length>1)},removeProp:function(e){return this.each((function(){delete this[I.propFix[e]||e]}))}}),I.extend({prop:function(e,t,n){var r,i,o=e.nodeType;if(3!==o&&8!==o&&2!==o)return 1===o&&I.isXMLDoc(e)||(t=I.propFix[t]||t,i=I.propHooks[t]),void 0!==n?i&&"set"in i&&void 0!==(r=i.set(e,n,t))?r:e[t]=n:i&&"get"in i&&null!==(r=i.get(e,t))?r:e[t]},propHooks:{tabIndex:{get:function(e){var t=I.find.attr(e,"tabindex");return t?parseInt(t,10):vt.test(e.nodeName)||mt.test(e.nodeName)&&e.href?0:-1}}},propFix:{for:"htmlFor",class:"className"}}),v.optSelected||(I.propHooks.selected={get:function(e){var t=e.parentNode;return t&&t.parentNode&&t.parentNode.selectedIndex,null},set:function(e){var t=e.parentNode;t&&(t.selectedIndex,t.parentNode&&t.parentNode.selectedIndex)}}),I.each(["tabIndex","readOnly","maxLength","cellSpacing","cellPadding","rowSpan","colSpan","useMap","frameBorder","contentEditable"],(function(){I.propFix[this.toLowerCase()]=this})),I.fn.extend({addClass:function(e){var t,n,r,i,o,a,s,l=0;if(m(e))return this.each((function(t){I(this).addClass(e.call(this,t,bt(this)))}));if((t=Ct(e)).length)for(;n=this[l++];)if(i=bt(n),r=1===n.nodeType&&" "+yt(i)+" "){for(a=0;o=t[a++];)r.indexOf(" "+o+" ")<0&&(r+=o+" ");i!==(s=yt(r))&&n.setAttribute("class",s)}return this},removeClass:function(e){var t,n,r,i,o,a,s,l=0;if(m(e))return this.each((function(t){I(this).removeClass(e.call(this,t,bt(this)))}));if(!arguments.length)return this.attr("class","");if((t=Ct(e)).length)for(;n=this[l++];)if(i=bt(n),r=1===n.nodeType&&" "+yt(i)+" "){for(a=0;o=t[a++];)for(;r.indexOf(" "+o+" ")>-1;)r=r.replace(" "+o+" "," ");i!==(s=yt(r))&&n.setAttribute("class",s)}return this},toggleClass:function(e,t){var n=typeof e,r="string"===n||Array.isArray(e);return"boolean"==typeof t&&r?t?this.addClass(e):this.removeClass(e):m(e)?this.each((function(n){I(this).toggleClass(e.call(this,n,bt(this),t),t)})):this.each((function(){var t,i,o,a;if(r)for(i=0,o=I(this),a=Ct(e);t=a[i++];)o.hasClass(t)?o.removeClass(t):o.addClass(t);else void 0!==e&&"boolean"!==n||((t=bt(this))&&K.set(this,"__className__",t),this.setAttribute&&this.setAttribute("class",t||!1===e?"":K.get(this,"__className__")||""))}))},hasClass:function(e){var t,n,r=0;for(t=" "+e+" ";n=this[r++];)if(1===n.nodeType&&(" "+yt(bt(n))+" ").indexOf(t)>-1)return!0;return!1}});var At=/\r/g;I.fn.extend({val:function(e){var t,n,r,i=this[0];return arguments.length?(r=m(e),this.each((function(n){var i;1===this.nodeType&&(null==(i=r?e.call(this,n,I(this).val()):e)?i="":"number"==typeof i?i+="":Array.isArray(i)&&(i=I.map(i,(function(e){return null==e?"":e+""}))),(t=I.valHooks[this.type]||I.valHooks[this.nodeName.toLowerCase()])&&"set"in t&&void 0!==t.set(this,i,"value")||(this.value=i))}))):i?(t=I.valHooks[i.type]||I.valHooks[i.nodeName.toLowerCase()])&&"get"in t&&void 0!==(n=t.get(i,"value"))?n:"string"==typeof(n=i.value)?n.replace(At,""):null==n?"":n:void 0}}),I.extend({valHooks:{option:{get:function(e){var t=I.find.attr(e,"value");return null!=t?t:yt(I.text(e))}},select:{get:function(e){var t,n,r,i=e.options,o=e.selectedIndex,a="select-one"===e.type,s=a?null:[],l=a?o+1:i.length;for(r=o<0?l:a?o:0;r<l;r++)if(((n=i[r]).selected||r===o)&&!n.disabled&&(!n.parentNode.disabled||!E(n.parentNode,"optgroup"))){if(t=I(n).val(),a)return t;s.push(t)}return s},set:function(e,t){for(var n,r,i=e.options,o=I.makeArray(t),a=i.length;a--;)((r=i[a]).selected=I.inArray(I.valHooks.option.get(r),o)>-1)&&(n=!0);return n||(e.selectedIndex=-1),o}}}}),I.each(["radio","checkbox"],(function(){I.valHooks[this]={set:function(e,t){if(Array.isArray(t))return e.checked=I.inArray(I(e).val(),t)>-1}},v.checkOn||(I.valHooks[this].get=function(e){return null===e.getAttribute("value")?"on":e.value})})),v.focusin="onfocusin"in n;var _t=/^(?:focusinfocus|focusoutblur)$/,It=function(e){e.stopPropagation()};I.extend(I.event,{trigger:function(e,t,r,i){var o,a,s,l,u,c,d,h,p=[r||b],g=f.call(e,"type")?e.type:e,v=f.call(e,"namespace")?e.namespace.split("."):[];if(a=h=s=r=r||b,3!==r.nodeType&&8!==r.nodeType&&!_t.test(g+I.event.triggered)&&(g.indexOf(".")>-1&&(v=g.split("."),g=v.shift(),v.sort()),u=g.indexOf(":")<0&&"on"+g,(e=e[I.expando]?e:new I.Event(g,"object"==typeof e&&e)).isTrigger=i?2:3,e.namespace=v.join("."),e.rnamespace=e.namespace?new RegExp("(^|\\.)"+v.join("\\.(?:.*\\.|)")+"(\\.|$)"):null,e.result=void 0,e.target||(e.target=r),t=null==t?[e]:I.makeArray(t,[e]),d=I.event.special[g]||{},i||!d.trigger||!1!==d.trigger.apply(r,t))){if(!i&&!d.noBubble&&!y(r)){for(l=d.delegateType||g,_t.test(l+g)||(a=a.parentNode);a;a=a.parentNode)p.push(a),s=a;s===(r.ownerDocument||b)&&p.push(s.defaultView||s.parentWindow||n)}for(o=0;(a=p[o++])&&!e.isPropagationStopped();)h=a,e.type=o>1?l:d.bindType||g,(c=(K.get(a,"events")||Object.create(null))[e.type]&&K.get(a,"handle"))&&c.apply(a,t),(c=u&&a[u])&&c.apply&&Z(a)&&(e.result=c.apply(a,t),!1===e.result&&e.preventDefault());return e.type=g,i||e.isDefaultPrevented()||d._default&&!1!==d._default.apply(p.pop(),t)||!Z(r)||u&&m(r[g])&&!y(r)&&((s=r[u])&&(r[u]=null),I.event.triggered=g,e.isPropagationStopped()&&h.addEventListener(g,It),r[g](),e.isPropagationStopped()&&h.removeEventListener(g,It),I.event.triggered=void 0,s&&(r[u]=s)),e.result}},simulate:function(e,t,n){var r=I.extend(new I.Event,n,{type:e,isSimulated:!0});I.event.trigger(r,null,t)}}),I.fn.extend({trigger:function(e,t){return this.each((function(){I.event.trigger(e,t,this)}))},triggerHandler:function(e,t){var n=this[0];if(n)return I.event.trigger(e,t,n,!0)}}),v.focusin||I.each({focus:"focusin",blur:"focusout"},(function(e,t){var n=function(e){I.event.simulate(t,e.target,I.event.fix(e))};I.event.special[t]={setup:function(){var r=this.ownerDocument||this.document||this,i=K.access(r,t);i||r.addEventListener(e,n,!0),K.access(r,t,(i||0)+1)},teardown:function(){var r=this.ownerDocument||this.document||this,i=K.access(r,t)-1;i?K.access(r,t,i):(r.removeEventListener(e,n,!0),K.remove(r,t))}}}));var xt=n.location,wt={guid:Date.now()},Mt=/\?/;I.parseXML=function(e){var t;if(!e||"string"!=typeof e)return null;try{t=(new n.DOMParser).parseFromString(e,"text/xml")}catch(e){t=void 0}return t&&!t.getElementsByTagName("parsererror").length||I.error("Invalid XML: "+e),t};var St=/\[\]$/,Tt=/\r?\n/g,Et=/^(?:submit|button|image|reset|file)$/i,Ot=/^(?:input|select|textarea|keygen)/i;function kt(e,t,n,r){var i;if(Array.isArray(t))I.each(t,(function(t,i){n||St.test(e)?r(e,i):kt(e+"["+("object"==typeof i&&null!=i?t:"")+"]",i,n,r)}));else if(n||"object"!==_(t))r(e,t);else for(i in t)kt(e+"["+i+"]",t[i],n,r)}I.param=function(e,t){var n,r=[],i=function(e,t){var n=m(t)?t():t;r[r.length]=encodeURIComponent(e)+"="+encodeURIComponent(null==n?"":n)};if(null==e)return"";if(Array.isArray(e)||e.jquery&&!I.isPlainObject(e))I.each(e,(function(){i(this.name,this.value)}));else for(n in e)kt(n,e[n],t,i);return r.join("&")},I.fn.extend({serialize:function(){return I.param(this.serializeArray())},serializeArray:function(){return this.map((function(){var e=I.prop(this,"elements");return e?I.makeArray(e):this})).filter((function(){var e=this.type;return this.name&&!I(this).is(":disabled")&&Ot.test(this.nodeName)&&!Et.test(e)&&(this.checked||!ge.test(e))})).map((function(e,t){var n=I(this).val();return null==n?null:Array.isArray(n)?I.map(n,(function(e){return{name:t.name,value:e.replace(Tt,"\r\n")}})):{name:t.name,value:n.replace(Tt,"\r\n")}})).get()}});var Lt=/%20/g,Dt=/#.*$/,Pt=/([?&])_=[^&]*/,Nt=/^(.*?):[ \t]*([^\r\n]*)$/gm,jt=/^(?:GET|HEAD)$/,Rt=/^\/\//,zt={},Ft={},Bt="*/".concat("*"),Ht=b.createElement("a");function Ut(e){return function(t,n){"string"!=typeof t&&(n=t,t="*");var r,i=0,o=t.toLowerCase().match(R)||[];if(m(n))for(;r=o[i++];)"+"===r[0]?(r=r.slice(1)||"*",(e[r]=e[r]||[]).unshift(n)):(e[r]=e[r]||[]).push(n)}}function Vt(e,t,n,r){var i={},o=e===Ft;function a(s){var l;return i[s]=!0,I.each(e[s]||[],(function(e,s){var u=s(t,n,r);return"string"!=typeof u||o||i[u]?o?!(l=u):void 0:(t.dataTypes.unshift(u),a(u),!1)})),l}return a(t.dataTypes[0])||!i["*"]&&a("*")}function Yt(e,t){var n,r,i=I.ajaxSettings.flatOptions||{};for(n in t)void 0!==t[n]&&((i[n]?e:r||(r={}))[n]=t[n]);return r&&I.extend(!0,e,r),e}Ht.href=xt.href,I.extend({active:0,lastModified:{},etag:{},ajaxSettings:{url:xt.href,type:"GET",isLocal:/^(?:about|app|app-storage|.+-extension|file|res|widget):$/.test(xt.protocol),global:!0,processData:!0,async:!0,contentType:"application/x-www-form-urlencoded; charset=UTF-8",accepts:{"*":Bt,text:"text/plain",html:"text/html",xml:"application/xml, text/xml",json:"application/json, text/javascript"},contents:{xml:/\bxml\b/,html:/\bhtml/,json:/\bjson\b/},responseFields:{xml:"responseXML",text:"responseText",json:"responseJSON"},converters:{"* text":String,"text html":!0,"text json":JSON.parse,"text xml":I.parseXML},flatOptions:{url:!0,context:!0}},ajaxSetup:function(e,t){return t?Yt(Yt(e,I.ajaxSettings),t):Yt(I.ajaxSettings,e)},ajaxPrefilter:Ut(zt),ajaxTransport:Ut(Ft),ajax:function(e,t){"object"==typeof e&&(t=e,e=void 0),t=t||{};var r,i,o,a,s,l,u,c,d,h,f=I.ajaxSetup({},t),p=f.context||f,g=f.context&&(p.nodeType||p.jquery)?I(p):I.event,v=I.Deferred(),m=I.Callbacks("once memory"),y=f.statusCode||{},C={},A={},_="canceled",x={readyState:0,getResponseHeader:function(e){var t;if(u){if(!a)for(a={};t=Nt.exec(o);)a[t[1].toLowerCase()+" "]=(a[t[1].toLowerCase()+" "]||[]).concat(t[2]);t=a[e.toLowerCase()+" "]}return null==t?null:t.join(", ")},getAllResponseHeaders:function(){return u?o:null},setRequestHeader:function(e,t){return null==u&&(e=A[e.toLowerCase()]=A[e.toLowerCase()]||e,C[e]=t),this},overrideMimeType:function(e){return null==u&&(f.mimeType=e),this},statusCode:function(e){var t;if(e)if(u)x.always(e[x.status]);else for(t in e)y[t]=[y[t],e[t]];return this},abort:function(e){var t=e||_;return r&&r.abort(t),w(0,t),this}};if(v.promise(x),f.url=((e||f.url||xt.href)+"").replace(Rt,xt.protocol+"//"),f.type=t.method||t.type||f.method||f.type,f.dataTypes=(f.dataType||"*").toLowerCase().match(R)||[""],null==f.crossDomain){l=b.createElement("a");try{l.href=f.url,l.href=l.href,f.crossDomain=Ht.protocol+"//"+Ht.host!=l.protocol+"//"+l.host}catch(e){f.crossDomain=!0}}if(f.data&&f.processData&&"string"!=typeof f.data&&(f.data=I.param(f.data,f.traditional)),Vt(zt,f,t,x),u)return x;for(d in(c=I.event&&f.global)&&0==I.active++&&I.event.trigger("ajaxStart"),f.type=f.type.toUpperCase(),f.hasContent=!jt.test(f.type),i=f.url.replace(Dt,""),f.hasContent?f.data&&f.processData&&0===(f.contentType||"").indexOf("application/x-www-form-urlencoded")&&(f.data=f.data.replace(Lt,"+")):(h=f.url.slice(i.length),f.data&&(f.processData||"string"==typeof f.data)&&(i+=(Mt.test(i)?"&":"?")+f.data,delete f.data),!1===f.cache&&(i=i.replace(Pt,"$1"),h=(Mt.test(i)?"&":"?")+"_="+wt.guid+++h),f.url=i+h),f.ifModified&&(I.lastModified[i]&&x.setRequestHeader("If-Modified-Since",I.lastModified[i]),I.etag[i]&&x.setRequestHeader("If-None-Match",I.etag[i])),(f.data&&f.hasContent&&!1!==f.contentType||t.contentType)&&x.setRequestHeader("Content-Type",f.contentType),x.setRequestHeader("Accept",f.dataTypes[0]&&f.accepts[f.dataTypes[0]]?f.accepts[f.dataTypes[0]]+("*"!==f.dataTypes[0]?", "+Bt+"; q=0.01":""):f.accepts["*"]),f.headers)x.setRequestHeader(d,f.headers[d]);if(f.beforeSend&&(!1===f.beforeSend.call(p,x,f)||u))return x.abort();if(_="abort",m.add(f.complete),x.done(f.success),x.fail(f.error),r=Vt(Ft,f,t,x)){if(x.readyState=1,c&&g.trigger("ajaxSend",[x,f]),u)return x;f.async&&f.timeout>0&&(s=n.setTimeout((function(){x.abort("timeout")}),f.timeout));try{u=!1,r.send(C,w)}catch(e){if(u)throw e;w(-1,e)}}else w(-1,"No Transport");function w(e,t,a,l){var d,h,b,C,A,_=t;u||(u=!0,s&&n.clearTimeout(s),r=void 0,o=l||"",x.readyState=e>0?4:0,d=e>=200&&e<300||304===e,a&&(C=function(e,t,n){for(var r,i,o,a,s=e.contents,l=e.dataTypes;"*"===l[0];)l.shift(),void 0===r&&(r=e.mimeType||t.getResponseHeader("Content-Type"));if(r)for(i in s)if(s[i]&&s[i].test(r)){l.unshift(i);break}if(l[0]in n)o=l[0];else{for(i in n){if(!l[0]||e.converters[i+" "+l[0]]){o=i;break}a||(a=i)}o=o||a}if(o)return o!==l[0]&&l.unshift(o),n[o]}(f,x,a)),!d&&I.inArray("script",f.dataTypes)>-1&&(f.converters["text script"]=function(){}),C=function(e,t,n,r){var i,o,a,s,l,u={},c=e.dataTypes.slice();if(c[1])for(a in e.converters)u[a.toLowerCase()]=e.converters[a];for(o=c.shift();o;)if(e.responseFields[o]&&(n[e.responseFields[o]]=t),!l&&r&&e.dataFilter&&(t=e.dataFilter(t,e.dataType)),l=o,o=c.shift())if("*"===o)o=l;else if("*"!==l&&l!==o){if(!(a=u[l+" "+o]||u["* "+o]))for(i in u)if((s=i.split(" "))[1]===o&&(a=u[l+" "+s[0]]||u["* "+s[0]])){!0===a?a=u[i]:!0!==u[i]&&(o=s[0],c.unshift(s[1]));break}if(!0!==a)if(a&&e.throws)t=a(t);else try{t=a(t)}catch(e){return{state:"parsererror",error:a?e:"No conversion from "+l+" to "+o}}}return{state:"success",data:t}}(f,C,x,d),d?(f.ifModified&&((A=x.getResponseHeader("Last-Modified"))&&(I.lastModified[i]=A),(A=x.getResponseHeader("etag"))&&(I.etag[i]=A)),204===e||"HEAD"===f.type?_="nocontent":304===e?_="notmodified":(_=C.state,h=C.data,d=!(b=C.error))):(b=_,!e&&_||(_="error",e<0&&(e=0))),x.status=e,x.statusText=(t||_)+"",d?v.resolveWith(p,[h,_,x]):v.rejectWith(p,[x,_,b]),x.statusCode(y),y=void 0,c&&g.trigger(d?"ajaxSuccess":"ajaxError",[x,f,d?h:b]),m.fireWith(p,[x,_]),c&&(g.trigger("ajaxComplete",[x,f]),--I.active||I.event.trigger("ajaxStop")))}return x},getJSON:function(e,t,n){return I.get(e,t,n,"json")},getScript:function(e,t){return I.get(e,void 0,t,"script")}}),I.each(["get","post"],(function(e,t){I[t]=function(e,n,r,i){return m(n)&&(i=i||r,r=n,n=void 0),I.ajax(I.extend({url:e,type:t,dataType:i,data:n,success:r},I.isPlainObject(e)&&e))}})),I.ajaxPrefilter((function(e){var t;for(t in e.headers)"content-type"===t.toLowerCase()&&(e.contentType=e.headers[t]||"")})),I._evalUrl=function(e,t,n){return I.ajax({url:e,type:"GET",dataType:"script",cache:!0,async:!1,global:!1,converters:{"text script":function(){}},dataFilter:function(e){I.globalEval(e,t,n)}})},I.fn.extend({wrapAll:function(e){var t;return this[0]&&(m(e)&&(e=e.call(this[0])),t=I(e,this[0].ownerDocument).eq(0).clone(!0),this[0].parentNode&&t.insertBefore(this[0]),t.map((function(){for(var e=this;e.firstElementChild;)e=e.firstElementChild;return e})).append(this)),this},wrapInner:function(e){return m(e)?this.each((function(t){I(this).wrapInner(e.call(this,t))})):this.each((function(){var t=I(this),n=t.contents();n.length?n.wrapAll(e):t.append(e)}))},wrap:function(e){var t=m(e);return this.each((function(n){I(this).wrapAll(t?e.call(this,n):e)}))},unwrap:function(e){return this.parent(e).not("body").each((function(){I(this).replaceWith(this.childNodes)})),this}}),I.expr.pseudos.hidden=function(e){return!I.expr.pseudos.visible(e)},I.expr.pseudos.visible=function(e){return!!(e.offsetWidth||e.offsetHeight||e.getClientRects().length)},I.ajaxSettings.xhr=function(){try{return new n.XMLHttpRequest}catch(e){}};var Wt={0:200,1223:204},Gt=I.ajaxSettings.xhr();v.cors=!!Gt&&"withCredentials"in Gt,v.ajax=Gt=!!Gt,I.ajaxTransport((function(e){var t,r;if(v.cors||Gt&&!e.crossDomain)return{send:function(i,o){var a,s=e.xhr();if(s.open(e.type,e.url,e.async,e.username,e.password),e.xhrFields)for(a in e.xhrFields)s[a]=e.xhrFields[a];for(a in e.mimeType&&s.overrideMimeType&&s.overrideMimeType(e.mimeType),e.crossDomain||i["X-Requested-With"]||(i["X-Requested-With"]="XMLHttpRequest"),i)s.setRequestHeader(a,i[a]);t=function(e){return function(){t&&(t=r=s.onload=s.onerror=s.onabort=s.ontimeout=s.onreadystatechange=null,"abort"===e?s.abort():"error"===e?"number"!=typeof s.status?o(0,"error"):o(s.status,s.statusText):o(Wt[s.status]||s.status,s.statusText,"text"!==(s.responseType||"text")||"string"!=typeof s.responseText?{binary:s.response}:{text:s.responseText},s.getAllResponseHeaders()))}},s.onload=t(),r=s.onerror=s.ontimeout=t("error"),void 0!==s.onabort?s.onabort=r:s.onreadystatechange=function(){4===s.readyState&&n.setTimeout((function(){t&&r()}))},t=t("abort");try{s.send(e.hasContent&&e.data||null)}catch(e){if(t)throw e}},abort:function(){t&&t()}}})),I.ajaxPrefilter((function(e){e.crossDomain&&(e.contents.script=!1)})),I.ajaxSetup({accepts:{script:"text/javascript, application/javascript, application/ecmascript, application/x-ecmascript"},contents:{script:/\b(?:java|ecma)script\b/},converters:{"text script":function(e){return I.globalEval(e),e}}}),I.ajaxPrefilter("script",(function(e){void 0===e.cache&&(e.cache=!1),e.crossDomain&&(e.type="GET")})),I.ajaxTransport("script",(function(e){var t,n;if(e.crossDomain||e.scriptAttrs)return{send:function(r,i){t=I("<script>").attr(e.scriptAttrs||{}).prop({charset:e.scriptCharset,src:e.url}).on("load error",n=function(e){t.remove(),n=null,e&&i("error"===e.type?404:200,e.type)}),b.head.appendChild(t[0])},abort:function(){n&&n()}}}));var Xt,qt=[],Zt=/(=)\?(?=&|$)|\?\?/;I.ajaxSetup({jsonp:"callback",jsonpCallback:function(){var e=qt.pop()||I.expando+"_"+wt.guid++;return this[e]=!0,e}}),I.ajaxPrefilter("json jsonp",(function(e,t,r){var i,o,a,s=!1!==e.jsonp&&(Zt.test(e.url)?"url":"string"==typeof e.data&&0===(e.contentType||"").indexOf("application/x-www-form-urlencoded")&&Zt.test(e.data)&&"data");if(s||"jsonp"===e.dataTypes[0])return i=e.jsonpCallback=m(e.jsonpCallback)?e.jsonpCallback():e.jsonpCallback,s?e[s]=e[s].replace(Zt,"$1"+i):!1!==e.jsonp&&(e.url+=(Mt.test(e.url)?"&":"?")+e.jsonp+"="+i),e.converters["script json"]=function(){return a||I.error(i+" was not called"),a[0]},e.dataTypes[0]="json",o=n[i],n[i]=function(){a=arguments},r.always((function(){void 0===o?I(n).removeProp(i):n[i]=o,e[i]&&(e.jsonpCallback=t.jsonpCallback,qt.push(i)),a&&m(o)&&o(a[0]),a=o=void 0})),"script"})),v.createHTMLDocument=((Xt=b.implementation.createHTMLDocument("").body).innerHTML="<form></form><form></form>",2===Xt.childNodes.length),I.parseHTML=function(e,t,n){return"string"!=typeof e?[]:("boolean"==typeof t&&(n=t,t=!1),t||(v.createHTMLDocument?((r=(t=b.implementation.createHTMLDocument("")).createElement("base")).href=b.location.href,t.head.appendChild(r)):t=b),o=!n&&[],(i=O.exec(e))?[t.createElement(i[1])]:(i=_e([e],t,o),o&&o.length&&I(o).remove(),I.merge([],i.childNodes)));var r,i,o},I.fn.load=function(e,t,n){var r,i,o,a=this,s=e.indexOf(" ");return s>-1&&(r=yt(e.slice(s)),e=e.slice(0,s)),m(t)?(n=t,t=void 0):t&&"object"==typeof t&&(i="POST"),a.length>0&&I.ajax({url:e,type:i||"GET",dataType:"html",data:t}).done((function(e){o=arguments,a.html(r?I("<div>").append(I.parseHTML(e)).find(r):e)})).always(n&&function(e,t){a.each((function(){n.apply(this,o||[e.responseText,t,e])}))}),this},I.expr.pseudos.animated=function(e){return I.grep(I.timers,(function(t){return e===t.elem})).length},I.offset={setOffset:function(e,t,n){var r,i,o,a,s,l,u=I.css(e,"position"),c=I(e),d={};"static"===u&&(e.style.position="relative"),s=c.offset(),o=I.css(e,"top"),l=I.css(e,"left"),("absolute"===u||"fixed"===u)&&(o+l).indexOf("auto")>-1?(a=(r=c.position()).top,i=r.left):(a=parseFloat(o)||0,i=parseFloat(l)||0),m(t)&&(t=t.call(e,n,I.extend({},s))),null!=t.top&&(d.top=t.top-s.top+a),null!=t.left&&(d.left=t.left-s.left+i),"using"in t?t.using.call(e,d):("number"==typeof d.top&&(d.top+="px"),"number"==typeof d.left&&(d.left+="px"),c.css(d))}},I.fn.extend({offset:function(e){if(arguments.length)return void 0===e?this:this.each((function(t){I.offset.setOffset(this,e,t)}));var t,n,r=this[0];return r?r.getClientRects().length?(t=r.getBoundingClientRect(),n=r.ownerDocument.defaultView,{top:t.top+n.pageYOffset,left:t.left+n.pageXOffset}):{top:0,left:0}:void 0},position:function(){if(this[0]){var e,t,n,r=this[0],i={top:0,left:0};if("fixed"===I.css(r,"position"))t=r.getBoundingClientRect();else{for(t=this.offset(),n=r.ownerDocument,e=r.offsetParent||n.documentElement;e&&(e===n.body||e===n.documentElement)&&"static"===I.css(e,"position");)e=e.parentNode;e&&e!==r&&1===e.nodeType&&((i=I(e).offset()).top+=I.css(e,"borderTopWidth",!0),i.left+=I.css(e,"borderLeftWidth",!0))}return{top:t.top-i.top-I.css(r,"marginTop",!0),left:t.left-i.left-I.css(r,"marginLeft",!0)}}},offsetParent:function(){return this.map((function(){for(var e=this.offsetParent;e&&"static"===I.css(e,"position");)e=e.offsetParent;return e||oe}))}}),I.each({scrollLeft:"pageXOffset",scrollTop:"pageYOffset"},(function(e,t){var n="pageYOffset"===t;I.fn[e]=function(r){return Y(this,(function(e,r,i){var o;if(y(e)?o=e:9===e.nodeType&&(o=e.defaultView),void 0===i)return o?o[t]:e[r];o?o.scrollTo(n?o.pageXOffset:i,n?i:o.pageYOffset):e[r]=i}),e,r,arguments.length)}})),I.each(["top","left"],(function(e,t){I.cssHooks[t]=Ge(v.pixelPosition,(function(e,n){if(n)return n=We(e,t),He.test(n)?I(e).position()[t]+"px":n}))})),I.each({Height:"height",Width:"width"},(function(e,t){I.each({padding:"inner"+e,content:t,"":"outer"+e},(function(n,r){I.fn[r]=function(i,o){var a=arguments.length&&(n||"boolean"!=typeof i),s=n||(!0===i||!0===o?"margin":"border");return Y(this,(function(t,n,i){var o;return y(t)?0===r.indexOf("outer")?t["inner"+e]:t.document.documentElement["client"+e]:9===t.nodeType?(o=t.documentElement,Math.max(t.body["scroll"+e],o["scroll"+e],t.body["offset"+e],o["offset"+e],o["client"+e])):void 0===i?I.css(t,n,s):I.style(t,n,i,s)}),t,a?i:void 0,a)}}))})),I.each(["ajaxStart","ajaxStop","ajaxComplete","ajaxError","ajaxSuccess","ajaxSend"],(function(e,t){I.fn[t]=function(e){return this.on(t,e)}})),I.fn.extend({bind:function(e,t,n){return this.on(e,null,t,n)},unbind:function(e,t){return this.off(e,null,t)},delegate:function(e,t,n,r){return this.on(t,e,n,r)},undelegate:function(e,t,n){return 1===arguments.length?this.off(e,"**"):this.off(t,e||"**",n)},hover:function(e,t){return this.mouseenter(e).mouseleave(t||e)}}),I.each("blur focus focusin focusout resize scroll click dblclick mousedown mouseup mousemove mouseover mouseout mouseenter mouseleave change select submit keydown keypress keyup contextmenu".split(" "),(function(e,t){I.fn[t]=function(e,n){return arguments.length>0?this.on(t,null,e,n):this.trigger(t)}}));var Qt=/^[\s\uFEFF\xA0]+|[\s\uFEFF\xA0]+$/g;I.proxy=function(e,t){var n,r,i;if("string"==typeof t&&(n=e[t],t=e,e=n),m(e))return r=s.call(arguments,2),(i=function(){return e.apply(t||this,r.concat(s.call(arguments)))}).guid=e.guid=e.guid||I.guid++,i},I.holdReady=function(e){e?I.readyWait++:I.ready(!0)},I.isArray=Array.isArray,I.parseJSON=JSON.parse,I.nodeName=E,I.isFunction=m,I.isWindow=y,I.camelCase=q,I.type=_,I.now=Date.now,I.isNumeric=function(e){var t=I.type(e);return("number"===t||"string"===t)&&!isNaN(e-parseFloat(e))},I.trim=function(e){return null==e?"":(e+"").replace(Qt,"")},void 0===(r=function(){return I}.apply(t,[]))||(e.exports=r);var Kt=n.jQuery,Jt=n.$;return I.noConflict=function(e){return n.$===I&&(n.$=Jt),e&&n.jQuery===I&&(n.jQuery=Kt),I},void 0===i&&(n.jQuery=n.$=I),I}))},function(e,t,n){"use strict";e.exports={BADNUM:void 0,FP_SAFE:Number.MAX_VALUE/1e4,ONEMAXYEAR:316224e5,ONEAVGYEAR:315576e5,ONEMINYEAR:31536e6,ONEMAXQUARTER:79488e5,ONEAVGQUARTER:78894e5,ONEMINQUARTER:76896e5,ONEMAXMONTH:26784e5,ONEAVGMONTH:26298e5,ONEMINMONTH:24192e5,ONEWEEK:6048e5,ONEDAY:864e5,ONEHOUR:36e5,ONEMIN:6e4,ONESEC:1e3,EPOCHJD:2440587.5,ALMOST_EQUAL:.999999,LOG_CLIP:10,MINUS_SIGN:"−"}},function(e,t,n){"use strict";n.r(t);var r=n(452);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";var r=n(335);var i=n(445),o=n(273);function a(e){return function(e){if(Array.isArray(e))return Object(r.a)(e)}(e)||Object(i.a)(e)||Object(o.a)(e)||function(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}()}n.d(t,"a",(function(){return a}))},function(e,t,n){"use strict";n.r(t);var r=n(907);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";n.r(t);var r=n(336);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){if(null===e||!0===e||!1===e)return NaN;var t=Number(e);if(isNaN(t))return t;return t<0?Math.ceil(t):Math.floor(t)},e.exports=t.default},function(e,t,n){"use strict";n.r(t);var r=n(0),i=n.n(r),o=(n(3),i.a.createContext(null));var a=function(e){e()},s={notify:function(){}};function l(){var e=a,t=null,n=null;return{clear:function(){t=null,n=null},notify:function(){e((function(){for(var e=t;e;)e.callback(),e=e.next}))},get:function(){for(var e=[],n=t;n;)e.push(n),n=n.next;return e},subscribe:function(e){var r=!0,i=n={callback:e,next:null,prev:n};return i.prev?i.prev.next=i:t=i,function(){r&&null!==t&&(r=!1,i.next?i.next.prev=i.prev:n=i.prev,i.prev?i.prev.next=i.next:t=i.next)}}}}var u=function(){function e(e,t){this.store=e,this.parentSub=t,this.unsubscribe=null,this.listeners=s,this.handleChangeWrapper=this.handleChangeWrapper.bind(this)}var t=e.prototype;return t.addNestedSub=function(e){return this.trySubscribe(),this.listeners.subscribe(e)},t.notifyNestedSubs=function(){this.listeners.notify()},t.handleChangeWrapper=function(){this.onStateChange&&this.onStateChange()},t.isSubscribed=function(){return Boolean(this.unsubscribe)},t.trySubscribe=function(){this.unsubscribe||(this.unsubscribe=this.parentSub?this.parentSub.addNestedSub(this.handleChangeWrapper):this.store.subscribe(this.handleChangeWrapper),this.listeners=l())},t.tryUnsubscribe=function(){this.unsubscribe&&(this.unsubscribe(),this.unsubscribe=null,this.listeners.clear(),this.listeners=s)},e}();var c=function(e){var t=e.store,n=e.context,a=e.children,s=Object(r.useMemo)((function(){var e=new u(t);return e.onStateChange=e.notifyNestedSubs,{store:t,subscription:e}}),[t]),l=Object(r.useMemo)((function(){return t.getState()}),[t]);Object(r.useEffect)((function(){var e=s.subscription;return e.trySubscribe(),l!==t.getState()&&e.notifyNestedSubs(),function(){e.tryUnsubscribe(),e.onStateChange=null}}),[s,l]);var c=n||o;return i.a.createElement(c.Provider,{value:s},a)},d=n(1),h=n(92),f=n(91),p=n.n(f),g=n(1288),v="undefined"!=typeof window&&void 0!==window.document&&void 0!==window.document.createElement?r.useLayoutEffect:r.useEffect,m=[],y=[null,null];function b(e,t){var n=e[1];return[t.payload,n+1]}function C(e,t,n){v((function(){return e.apply(void 0,t)}),n)}function A(e,t,n,r,i,o,a){e.current=r,t.current=i,n.current=!1,o.current&&(o.current=null,a())}function _(e,t,n,r,i,o,a,s,l,u){if(e){var c=!1,d=null,h=function(){if(!c){var e,n,h=t.getState();try{e=r(h,i.current)}catch(e){n=e,d=e}n||(d=null),e===o.current?a.current||l():(o.current=e,s.current=e,a.current=!0,u({type:"STORE_UPDATED",payload:{error:n}}))}};n.onStateChange=h,n.trySubscribe(),h();return function(){if(c=!0,n.tryUnsubscribe(),n.onStateChange=null,d)throw d}}}var I=function(){return[null,0]};function x(e,t){void 0===t&&(t={});var n=t,a=n.getDisplayName,s=void 0===a?function(e){return"ConnectAdvanced("+e+")"}:a,l=n.methodName,c=void 0===l?"connectAdvanced":l,f=n.renderCountProp,v=void 0===f?void 0:f,x=n.shouldHandleStateChanges,w=void 0===x||x,M=n.storeKey,S=void 0===M?"store":M,T=(n.withRef,n.forwardRef),E=void 0!==T&&T,O=n.context,k=void 0===O?o:O,L=Object(h.a)(n,["getDisplayName","methodName","renderCountProp","shouldHandleStateChanges","storeKey","withRef","forwardRef","context"]),D=k;return function(t){var n=t.displayName||t.name||"Component",o=s(n),a=Object(d.a)({},L,{getDisplayName:s,methodName:c,renderCountProp:v,shouldHandleStateChanges:w,storeKey:S,displayName:o,wrappedComponentName:n,WrappedComponent:t}),l=L.pure;var f=l?r.useMemo:function(e){return e()};function x(n){var o=Object(r.useMemo)((function(){var e=n.reactReduxForwardedRef,t=Object(h.a)(n,["reactReduxForwardedRef"]);return[n.context,e,t]}),[n]),s=o[0],l=o[1],c=o[2],p=Object(r.useMemo)((function(){return s&&s.Consumer&&Object(g.isContextConsumer)(i.a.createElement(s.Consumer,null))?s:D}),[s,D]),v=Object(r.useContext)(p),x=Boolean(n.store)&&Boolean(n.store.getState)&&Boolean(n.store.dispatch);Boolean(v)&&Boolean(v.store);var M=x?n.store:v.store,S=Object(r.useMemo)((function(){return function(t){return e(t.dispatch,a)}(M)}),[M]),T=Object(r.useMemo)((function(){if(!w)return y;var e=new u(M,x?null:v.subscription),t=e.notifyNestedSubs.bind(e);return[e,t]}),[M,x,v]),E=T[0],O=T[1],k=Object(r.useMemo)((function(){return x?v:Object(d.a)({},v,{subscription:E})}),[x,v,E]),L=Object(r.useReducer)(b,m,I),P=L[0][0],N=L[1];if(P&&P.error)throw P.error;var j=Object(r.useRef)(),R=Object(r.useRef)(c),z=Object(r.useRef)(),F=Object(r.useRef)(!1),B=f((function(){return z.current&&c===R.current?z.current:S(M.getState(),c)}),[M,P,c]);C(A,[R,j,F,c,B,z,O]),C(_,[w,M,E,S,R,j,F,z,O,N],[M,E,S]);var H=Object(r.useMemo)((function(){return i.a.createElement(t,Object(d.a)({},B,{ref:l}))}),[l,t,B]);return Object(r.useMemo)((function(){return w?i.a.createElement(p.Provider,{value:k},H):H}),[p,H,k])}var M=l?i.a.memo(x):x;if(M.WrappedComponent=t,M.displayName=o,E){var T=i.a.forwardRef((function(e,t){return i.a.createElement(M,Object(d.a)({},e,{reactReduxForwardedRef:t}))}));return T.displayName=o,T.WrappedComponent=t,p()(T,t)}return p()(M,t)}}function w(e,t){return e===t?0!==e||0!==t||1/e==1/t:e!=e&&t!=t}function M(e,t){if(w(e,t))return!0;if("object"!=typeof e||null===e||"object"!=typeof t||null===t)return!1;var n=Object.keys(e),r=Object.keys(t);if(n.length!==r.length)return!1;for(var i=0;i<n.length;i++)if(!Object.prototype.hasOwnProperty.call(t,n[i])||!w(e[n[i]],t[n[i]]))return!1;return!0}var S=n(118);function T(e){return function(t,n){var r=e(t,n);function i(){return r}return i.dependsOnOwnProps=!1,i}}function E(e){return null!==e.dependsOnOwnProps&&void 0!==e.dependsOnOwnProps?Boolean(e.dependsOnOwnProps):1!==e.length}function O(e,t){return function(t,n){n.displayName;var r=function(e,t){return r.dependsOnOwnProps?r.mapToProps(e,t):r.mapToProps(e)};return r.dependsOnOwnProps=!0,r.mapToProps=function(t,n){r.mapToProps=e,r.dependsOnOwnProps=E(e);var i=r(t,n);return"function"==typeof i&&(r.mapToProps=i,r.dependsOnOwnProps=E(i),i=r(t,n)),i},r}}var k=[function(e){return"function"==typeof e?O(e):void 0},function(e){return e?void 0:T((function(e){return{dispatch:e}}))},function(e){return e&&"object"==typeof e?T((function(t){return Object(S.bindActionCreators)(e,t)})):void 0}];var L=[function(e){return"function"==typeof e?O(e):void 0},function(e){return e?void 0:T((function(){return{}}))}];function D(e,t,n){return Object(d.a)({},n,e,t)}var P=[function(e){return"function"==typeof e?function(e){return function(t,n){n.displayName;var r,i=n.pure,o=n.areMergedPropsEqual,a=!1;return function(t,n,s){var l=e(t,n,s);return a?i&&o(l,r)||(r=l):(a=!0,r=l),r}}}(e):void 0},function(e){return e?void 0:function(){return D}}];function N(e,t,n,r){return function(i,o){return n(e(i,o),t(r,o),o)}}function j(e,t,n,r,i){var o,a,s,l,u,c=i.areStatesEqual,d=i.areOwnPropsEqual,h=i.areStatePropsEqual,f=!1;function p(i,f){var p,g,v=!d(f,a),m=!c(i,o);return o=i,a=f,v&&m?(s=e(o,a),t.dependsOnOwnProps&&(l=t(r,a)),u=n(s,l,a)):v?(e.dependsOnOwnProps&&(s=e(o,a)),t.dependsOnOwnProps&&(l=t(r,a)),u=n(s,l,a)):m?(p=e(o,a),g=!h(p,s),s=p,g&&(u=n(s,l,a)),u):u}return function(i,c){return f?p(i,c):(s=e(o=i,a=c),l=t(r,a),u=n(s,l,a),f=!0,u)}}function R(e,t){var n=t.initMapStateToProps,r=t.initMapDispatchToProps,i=t.initMergeProps,o=Object(h.a)(t,["initMapStateToProps","initMapDispatchToProps","initMergeProps"]),a=n(e,o),s=r(e,o),l=i(e,o);return(o.pure?j:N)(a,s,l,e,o)}function z(e,t,n){for(var r=t.length-1;r>=0;r--){var i=t[r](e);if(i)return i}return function(t,r){throw new Error("Invalid value of type "+typeof e+" for "+n+" argument when connecting component "+r.wrappedComponentName+".")}}function F(e,t){return e===t}function B(e){var t=void 0===e?{}:e,n=t.connectHOC,r=void 0===n?x:n,i=t.mapStateToPropsFactories,o=void 0===i?L:i,a=t.mapDispatchToPropsFactories,s=void 0===a?k:a,l=t.mergePropsFactories,u=void 0===l?P:l,c=t.selectorFactory,f=void 0===c?R:c;return function(e,t,n,i){void 0===i&&(i={});var a=i,l=a.pure,c=void 0===l||l,p=a.areStatesEqual,g=void 0===p?F:p,v=a.areOwnPropsEqual,m=void 0===v?M:v,y=a.areStatePropsEqual,b=void 0===y?M:y,C=a.areMergedPropsEqual,A=void 0===C?M:C,_=Object(h.a)(a,["pure","areStatesEqual","areOwnPropsEqual","areStatePropsEqual","areMergedPropsEqual"]),I=z(e,o,"mapStateToProps"),x=z(t,s,"mapDispatchToProps"),w=z(n,u,"mergeProps");return r(f,Object(d.a)({methodName:"connect",getDisplayName:function(e){return"Connect("+e+")"},shouldHandleStateChanges:Boolean(e),initMapStateToProps:I,initMapDispatchToProps:x,initMergeProps:w,pure:c,areStatesEqual:g,areOwnPropsEqual:m,areStatePropsEqual:b,areMergedPropsEqual:A},_))}}var H=B();function U(){return Object(r.useContext)(o)}function V(e){void 0===e&&(e=o);var t=e===o?U:function(){return Object(r.useContext)(e)};return function(){return t().store}}var Y=V();function W(e){void 0===e&&(e=o);var t=e===o?Y:V(e);return function(){return t().dispatch}}var G=W(),X=function(e,t){return e===t};function q(e){void 0===e&&(e=o);var t=e===o?U:function(){return Object(r.useContext)(e)};return function(e,n){void 0===n&&(n=X);var i=t(),o=function(e,t,n,i){var o,a=Object(r.useReducer)((function(e){return e+1}),0)[1],s=Object(r.useMemo)((function(){return new u(n,i)}),[n,i]),l=Object(r.useRef)(),c=Object(r.useRef)(),d=Object(r.useRef)(),h=Object(r.useRef)(),f=n.getState();try{o=e!==c.current||f!==d.current||l.current?e(f):h.current}catch(e){throw l.current&&(e.message+="\nThe error may be correlated with this previous error:\n"+l.current.stack+"\n\n"),e}return v((function(){c.current=e,d.current=f,h.current=o,l.current=void 0})),v((function(){function e(){try{var e=c.current(n.getState());if(t(e,h.current))return;h.current=e}catch(e){l.current=e}a()}return s.onStateChange=e,s.trySubscribe(),e(),function(){return s.tryUnsubscribe()}}),[n,s]),o}(e,n,i.store,i.subscription);return Object(r.useDebugValue)(o),o}}var Z,Q=q(),K=n(22);n.d(t,"Provider",(function(){return c})),n.d(t,"connectAdvanced",(function(){return x})),n.d(t,"ReactReduxContext",(function(){return o})),n.d(t,"connect",(function(){return H})),n.d(t,"batch",(function(){return K.unstable_batchedUpdates})),n.d(t,"useDispatch",(function(){return G})),n.d(t,"createDispatchHook",(function(){return W})),n.d(t,"useSelector",(function(){return Q})),n.d(t,"createSelectorHook",(function(){return q})),n.d(t,"useStore",(function(){return Y})),n.d(t,"createStoreHook",(function(){return V})),n.d(t,"shallowEqual",(function(){return M})),Z=K.unstable_batchedUpdates,a=Z},function(e,t,n){"use strict";n.d(t,"b",(function(){return r})),n.d(t,"a",(function(){return i}));var r=function(e){return e.scrollTop};function i(e,t){var n=e.timeout,r=e.style,i=void 0===r?{}:r;return{duration:i.transitionDuration||"number"==typeof n?n:n[t.mode]||0,delay:i.transitionDelay}}},function(e,t,n){"use strict";function r(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}function i(e,t,n){return t&&r(e.prototype,t),n&&r(e,n),e}n.d(t,"a",(function(){return i}))},function(e,t){var n=Array.isArray;e.exports=n},function(e,t,n){"use strict";var r=n(39),i=n(317).timeFormatLocale,o=n(42),a=n(27),s=n(297),l=n(64),u=n(10),c=n(41),d=n(72).BADNUM,h=n(83),f=n(299).clearSelect,p=n(382),g=n(983),v=n(384).getModuleCalcData,m=u.relinkPrivateKeys,y=u._,b=e.exports={};u.extendFlat(b,a),b.attributes=n(160),b.attributes.type.values=b.allTypes,b.fontAttrs=n(94),b.layoutAttributes=n(295),b.fontWeight="normal";var C=b.transformsRegistry,A=n(1683);b.executeAPICommand=A.executeAPICommand,b.computeAPICommandBindings=A.computeAPICommandBindings,b.manageCommandObserver=A.manageCommandObserver,b.hasSimpleAPICommandBindings=A.hasSimpleAPICommandBindings,b.redrawText=function(e){var t=(e=u.getGraphDiv(e))._fullLayout||{};if(!(!(t._has&&t._has("polar"))&&e.data&&e.data[0]&&e.data[0].r))return new Promise((function(t){setTimeout((function(){a.getComponentMethod("annotations","draw")(e),a.getComponentMethod("legend","draw")(e),a.getComponentMethod("colorbar","draw")(e),t(b.previousPromises(e))}),300)}))},b.resize=function(e){var t;e=u.getGraphDiv(e);var n=new Promise((function(n,r){e&&!u.isHidden(e)||r(new Error("Resize must be passed a displayed plot div element.")),e._redrawTimer&&clearTimeout(e._redrawTimer),e._resolveResize&&(t=e._resolveResize),e._resolveResize=n,e._redrawTimer=setTimeout((function(){if(!e.layout||e.layout.width&&e.layout.height||u.isHidden(e))n(e);else{delete e.layout.width,delete e.layout.height;var t=e.changed;e.autoplay=!0,a.call("relayout",e,{autosize:!0}).then((function(){e.changed=t,e._resolveResize===n&&(delete e._resolveResize,n(e))}))}}),100)}));return t&&t(n),n},b.previousPromises=function(e){if((e._promises||[]).length)return Promise.all(e._promises).then((function(){e._promises=[]}))},b.addLinks=function(e){if(e._context.showLink||e._context.showSources){var t=e._fullLayout,n=u.ensureSingle(t._paper,"text","js-plot-link-container",(function(e){e.style({"font-family":'"Open Sans", Arial, sans-serif',"font-size":"12px",fill:c.defaultLine,"pointer-events":"all"}).each((function(){var e=r.select(this);e.append("tspan").classed("js-link-to-tool",!0),e.append("tspan").classed("js-link-spacer",!0),e.append("tspan").classed("js-sourcelinks",!0)}))})),i=n.node(),o={y:t._paper.attr("height")-9};document.body.contains(i)&&i.getComputedTextLength()>=t.width-20?(o["text-anchor"]="start",o.x=5):(o["text-anchor"]="end",o.x=t._paper.attr("width")-7),n.attr(o);var a=n.select(".js-link-to-tool"),s=n.select(".js-link-spacer"),l=n.select(".js-sourcelinks");e._context.showSources&&e._context.showSources(e),e._context.showLink&&function(e,t){t.text("");var n=t.append("a").attr({"xlink:xlink:href":"#",class:"link--impt link--embedview","font-weight":"bold"}).text(e._context.linkText+" "+String.fromCharCode(187));if(e._context.sendData)n.on("click",(function(){b.sendDataToCloud(e)}));else{var r=window.location.pathname.split("/"),i=window.location.search;n.attr({"xlink:xlink:show":"new","xlink:xlink:href":"/"+r[2].split(".")[0]+"/"+r[1]+i})}}(e,a),s.text(a.text()&&l.text()?" - ":"")}},b.sendDataToCloud=function(e){var t=(window.PLOTLYENV||{}).BASE_URL||e._context.plotlyServerURL;if(t){e.emit("plotly_beforeexport");var n=r.select(e).append("div").attr("id","hiddenform").style("display","none"),i=n.append("form").attr({action:t+"/external",method:"post",target:"_blank"});return i.append("input").attr({type:"text",name:"data"}).node().value=b.graphJson(e,!1,"keepdata"),i.node().submit(),n.remove(),e.emit("plotly_afterexport"),!1}};var _=["days","shortDays","months","shortMonths","periods","dateTime","date","time","decimal","thousands","grouping","currency"],I=["year","month","dayMonth","dayMonthYear"];function x(e,t){var n=e._context.locale;n||(n="en-US");var r=!1,i={};function o(e){for(var n=!0,o=0;o<t.length;o++){var a=t[o];i[a]||(e[a]?i[a]=e[a]:n=!1)}n&&(r=!0)}for(var s=0;s<2;s++){for(var l=e._context.locales,u=0;u<2;u++){var c=(l[n]||{}).format;if(c&&(o(c),r))break;l=a.localeRegistry}var d=n.split("-")[0];if(r||d===n)break;n=d}return r||o(a.localeRegistry.en.format),i}function w(e,t){var n={_fullLayout:t},r="x"===e._id.charAt(0),i=e._mainAxis._anchorAxis,o="",a="",s="";if(i&&(s=i._mainAxis._id,o=r?e._id+s:s+e._id),!o||!t._plots[o]){o="";for(var l=e._counterAxes,u=0;u<l.length;u++){var c=l[u],d=r?e._id+c:c+e._id;a||(a=d);var f=h.getFromId(n,c);if(s&&f.overlaying===s){o=d;break}}}return o||a}function M(e){var t=e.transforms;if(Array.isArray(t)&&t.length)for(var n=0;n<t.length;n++){var r=t[n],i=r._module||C[r.type];if(i&&i.makesData)return!0}return!1}function S(e,t,n,r){for(var i=e.transforms,o=[e],a=0;a<i.length;a++){var s=i[a],l=C[s.type];l&&l.transform&&(o=l.transform(o,{transform:s,fullTrace:e,fullData:t,layout:n,fullLayout:r,transformIndex:a}))}return o}function T(e){return"string"==typeof e&&"px"===e.substr(e.length-2)&&parseFloat(e)}function E(e){var t=e.margin;if(!e._size){var n=e._size={l:Math.round(t.l),r:Math.round(t.r),t:Math.round(t.t),b:Math.round(t.b),p:Math.round(t.pad)};n.w=Math.round(e.width)-n.l-n.r,n.h=Math.round(e.height)-n.t-n.b}e._pushmargin||(e._pushmargin={}),e._pushmarginIds||(e._pushmarginIds={})}b.supplyDefaults=function(e,t){var n=t&&t.skipUpdateCalc,o=e._fullLayout||{};if(o._skipDefaults)delete o._skipDefaults;else{var s,l=e._fullLayout={},c=e.layout||{},d=e._fullData||[],h=e._fullData=[],p=e.data||[],g=e.calcdata||[],v=e._context||{};e._transitionData||b.createTransitionData(e),l._dfltTitle={plot:y(e,"Click to enter Plot title"),x:y(e,"Click to enter X axis title"),y:y(e,"Click to enter Y axis title"),colorbar:y(e,"Click to enter Colorscale title"),annotation:y(e,"new text")},l._traceWord=y(e,"trace");var C=x(e,_);if(l._mapboxAccessToken=v.mapboxAccessToken,o._initialAutoSizeIsDone){var A=o.width,w=o.height;b.supplyLayoutGlobalDefaults(c,l,C),c.width||(l.width=A),c.height||(l.height=w),b.sanitizeMargins(l)}else{b.supplyLayoutGlobalDefaults(c,l,C);var M=!c.width||!c.height,S=l.autosize,T=v.autosizable;M&&(S||T)?b.plotAutoSize(e,c,l):M&&b.sanitizeMargins(l),!S&&M&&(c.width=l.width,c.height=l.height)}l._d3locale=function(e,t){return e.decimal=t.charAt(0),e.thousands=t.charAt(1),{numberFormat:r.locale(e).numberFormat,timeFormat:i(e).utcFormat}}(C,l.separators),l._extraFormat=x(e,I),l._initialAutoSizeIsDone=!0,l._dataLength=p.length,l._modules=[],l._visibleModules=[],l._basePlotModules=[];var O=l._subplots=function(){var e,t,n=a.collectableSubplotTypes,r={};if(!n){n=[];var i=a.subplotsRegistry;for(var o in i){var s=i[o].attr;if(s&&(n.push(o),Array.isArray(s)))for(t=0;t<s.length;t++)u.pushUnique(n,s[t])}}for(e=0;e<n.length;e++)r[n[e]]=[];return r}(),k=l._splomAxes={x:{},y:{}},L=l._splomSubplots={};l._splomGridDflt={},l._scatterStackOpts={},l._firstScatter={},l._alignmentOpts={},l._colorAxes={},l._requestRangeslider={},l._traceUids=function(e,t){var n,r,i=t.length,o=[];for(n=0;n<e.length;n++){var a=e[n]._fullInput;a!==r&&o.push(a),r=a}var s=o.length,l=new Array(i),c={};function d(e,t){l[t]=e,c[e]=1}function h(e,t){if(e&&"string"==typeof e&&!c[e])return d(e,t),!0}for(n=0;n<i;n++){var f=t[n].uid;"number"==typeof f&&(f=String(f)),h(f,n)||(n<s&&h(o[n].uid,n)||d(u.randstr(c),n))}return l}(d,p),l._globalTransforms=(e._context||{}).globalTransforms,b.supplyDataDefaults(p,h,c,l);var D=Object.keys(k.x),P=Object.keys(k.y);if(D.length>1&&P.length>1){for(a.getComponentMethod("grid","sizeDefaults")(c,l),s=0;s<D.length;s++)u.pushUnique(O.xaxis,D[s]);for(s=0;s<P.length;s++)u.pushUnique(O.yaxis,P[s]);for(var N in L)u.pushUnique(O.cartesian,N)}if(l._has=b._hasPlotType.bind(l),d.length===h.length)for(s=0;s<h.length;s++)m(h[s],d[s]);b.supplyLayoutModuleDefaults(c,l,h,e._transitionData);var j=l._visibleModules,R=[];for(s=0;s<j.length;s++){var z=j[s].crossTraceDefaults;z&&u.pushUnique(R,z)}for(s=0;s<R.length;s++)R[s](h,l);l._hasOnlyLargeSploms=1===l._basePlotModules.length&&"splom"===l._basePlotModules[0].name&&D.length>15&&P.length>15&&0===l.shapes.length&&0===l.images.length,l._hasCartesian=l._has("cartesian"),l._hasGeo=l._has("geo"),l._hasGL3D=l._has("gl3d"),l._hasGL2D=l._has("gl2d"),l._hasTernary=l._has("ternary"),l._hasPie=l._has("pie"),b.linkSubplots(h,l,d,o),b.cleanPlot(h,l,d,o);var F=!(!o._has||!o._has("gl2d")),B=!(!l._has||!l._has("gl2d")),H=!(!o._has||!o._has("cartesian"))||F,U=!(!l._has||!l._has("cartesian"))||B;H&&!U?o._bgLayer.remove():U&&!H&&(l._shouldCreateBgLayer=!0),o._zoomlayer&&!e._dragging&&f({_fullLayout:o}),function(e,t){var n,r=[];t.meta&&(n=t._meta={meta:t.meta,layout:{meta:t.meta}});for(var i=0;i<e.length;i++){var o=e[i];o.meta?r[o.index]=o._meta={meta:o.meta}:t.meta&&(o._meta={meta:t.meta}),t.meta&&(o._meta.layout={meta:t.meta})}r.length&&(n||(n=t._meta={}),n.data=r)}(h,l),m(l,o),a.getComponentMethod("colorscale","crossTraceDefaults")(h,l),l._preGUI||(l._preGUI={}),l._tracePreGUI||(l._tracePreGUI={});var V,Y=l._tracePreGUI,W={};for(V in Y)W[V]="old";for(s=0;s<h.length;s++)W[V=h[s]._fullInput.uid]||(Y[V]={}),W[V]="new";for(V in W)"old"===W[V]&&delete Y[V];E(l),a.getComponentMethod("rangeslider","makeData")(l),n||g.length!==h.length||b.supplyDefaultsUpdateCalc(g,h)}},b.supplyDefaultsUpdateCalc=function(e,t){for(var n=0;n<t.length;n++){var r=t[n],i=(e[n]||[])[0];if(i&&i.trace){var o=i.trace;if(o._hasCalcTransform){var a,s,l,c=o._arrayAttrs;for(a=0;a<c.length;a++)s=c[a],l=u.nestedProperty(o,s).get().slice(),u.nestedProperty(r,s).set(l)}i.trace=r}}},b.createTransitionData=function(e){e._transitionData||(e._transitionData={}),e._transitionData._frames||(e._transitionData._frames=[]),e._transitionData._frameHash||(e._transitionData._frameHash={}),e._transitionData._counter||(e._transitionData._counter=0),e._transitionData._interruptCallbacks||(e._transitionData._interruptCallbacks=[])},b._hasPlotType=function(e){var t,n=this._basePlotModules||[];for(t=0;t<n.length;t++)if(n[t].name===e)return!0;var r=this._modules||[];for(t=0;t<r.length;t++){var i=r[t].name;if(i===e)return!0;var o=a.modules[i];if(o&&o.categories[e])return!0}return!1},b.cleanPlot=function(e,t,n,r){var i,o,a=r._basePlotModules||[];for(i=0;i<a.length;i++){var s=a[i];s.clean&&s.clean(e,t,n,r)}var l=r._has&&r._has("gl"),u=t._has&&t._has("gl");l&&!u&&void 0!==r._glcontainer&&(r._glcontainer.selectAll(".gl-canvas").remove(),r._glcontainer.selectAll(".no-webgl").remove(),r._glcanvas=null);var c=!!r._infolayer;e:for(i=0;i<n.length;i++){var d=n[i].uid;for(o=0;o<e.length;o++){if(d===e[o].uid)continue e}c&&r._infolayer.select(".cb"+d).remove()}},b.linkSubplots=function(e,t,n,r){var i,o,s=r._plots||{},l=t._plots={},c=t._subplots,d={_fullData:e,_fullLayout:t},f=c.cartesian.concat(c.gl2d||[]);for(i=0;i<f.length;i++){var p,g=f[i],v=s[g],m=h.getFromId(d,g,"x"),y=h.getFromId(d,g,"y");for(v?p=l[g]=v:(p=l[g]={}).id=g,m._counterAxes.push(y._id),y._counterAxes.push(m._id),m._subplotsWith.push(g),y._subplotsWith.push(g),p.xaxis=m,p.yaxis=y,p._hasClipOnAxisFalse=!1,o=0;o<e.length;o++){var b=e[o];if(b.xaxis===p.xaxis._id&&b.yaxis===p.yaxis._id&&!1===b.cliponaxis){p._hasClipOnAxisFalse=!0;break}}}var C,A=h.list(d,null,!0);for(i=0;i<A.length;i++){var _=null;(C=A[i]).overlaying&&(_=h.getFromId(d,C.overlaying))&&_.overlaying&&(C.overlaying=!1,_=null),C._mainAxis=_||C,_&&(C.domain=_.domain.slice()),C._anchorAxis="free"===C.anchor?null:h.getFromId(d,C.anchor)}for(i=0;i<A.length;i++)if((C=A[i])._counterAxes.sort(h.idSort),C._subplotsWith.sort(u.subplotSort),C._mainSubplot=w(C,t),C._counterAxes.length&&(C.spikemode&&-1!==C.spikemode.indexOf("across")||C.automargin&&C.mirror&&"free"!==C.anchor||a.getComponentMethod("rangeslider","isVisible")(C))){var I=1,x=0;for(o=0;o<C._counterAxes.length;o++){var M=h.getFromId(d,C._counterAxes[o]);I=Math.min(I,M.domain[0]),x=Math.max(x,M.domain[1])}I<x&&(C._counterDomainMin=I,C._counterDomainMax=x)}},b.clearExpandedTraceDefaultColors=function(e){var t,n,r;for(n=[],(t=e._module._colorAttrs)||(e._module._colorAttrs=t=[],s.crawl(e._module.attributes,(function(e,r,i,o){n[o]=r,n.length=o+1,"color"===e.valType&&void 0===e.dflt&&t.push(n.join("."))}))),r=0;r<t.length;r++){u.nestedProperty(e,"_input."+t[r]).get()||u.nestedProperty(e,t[r]).set(null)}},b.supplyDataDefaults=function(e,t,n,r){var i,o,s,c=r._modules,d=r._visibleModules,h=r._basePlotModules,f=0,p=0;function g(e){t.push(e);var n=e._module;n&&(u.pushUnique(c,n),!0===e.visible&&u.pushUnique(d,n),u.pushUnique(h,e._module.basePlotModule),f++,!1!==e._input.visible&&p++)}r._transformModules=[];var v={},y=[],C=(n.template||{}).data||{},A=l.traceTemplater(C);for(i=0;i<e.length;i++){if(s=e[i],(o=A.newTrace(s)).uid=r._traceUids[i],b.supplyTraceDefaults(s,o,p,r,i),o.index=i,o._input=s,o._expandedIndex=f,o.transforms&&o.transforms.length)for(var _=!1!==s.visible&&!1===o.visible,I=S(o,t,n,r),x=0;x<I.length;x++){var w=I[x],M={_template:o._template,type:o.type,uid:o.uid+x};_&&!1===w.visible&&delete w.visible,b.supplyTraceDefaults(w,M,f,r,i),m(M,w),M.index=i,M._input=s,M._fullInput=o,M._expandedIndex=f,M._expandedInput=w,g(M)}else o._fullInput=o,o._expandedInput=o,g(o);a.traceIs(o,"carpetAxis")&&(v[o.carpet]=o),a.traceIs(o,"carpetDependent")&&y.push(i)}for(i=0;i<y.length;i++)if((o=t[y[i]]).visible){var T=v[o.carpet];o._carpet=T,T&&T.visible?(o.xaxis=T.xaxis,o.yaxis=T.yaxis):o.visible=!1}},b.supplyAnimationDefaults=function(e){var t;e=e||{};var n={};function r(t,r){return u.coerce(e||{},n,p,t,r)}if(r("mode"),r("direction"),r("fromcurrent"),Array.isArray(e.frame))for(n.frame=[],t=0;t<e.frame.length;t++)n.frame[t]=b.supplyAnimationFrameDefaults(e.frame[t]||{});else n.frame=b.supplyAnimationFrameDefaults(e.frame||{});if(Array.isArray(e.transition))for(n.transition=[],t=0;t<e.transition.length;t++)n.transition[t]=b.supplyAnimationTransitionDefaults(e.transition[t]||{});else n.transition=b.supplyAnimationTransitionDefaults(e.transition||{});return n},b.supplyAnimationFrameDefaults=function(e){var t={};function n(n,r){return u.coerce(e||{},t,p.frame,n,r)}return n("duration"),n("redraw"),t},b.supplyAnimationTransitionDefaults=function(e){var t={};function n(n,r){return u.coerce(e||{},t,p.transition,n,r)}return n("duration"),n("easing"),t},b.supplyFrameDefaults=function(e){var t={};function n(n,r){return u.coerce(e,t,g,n,r)}return n("group"),n("name"),n("traces"),n("baseframe"),n("data"),n("layout"),t},b.supplyTraceDefaults=function(e,t,n,r,i){var o,s=r.colorway||c.defaults,l=s[n%s.length];function d(n,r){return u.coerce(e,t,b.attributes,n,r)}var h=d("visible");d("type"),d("name",r._traceWord+" "+i),d("uirevision",r.uirevision);var f=b.getModule(t);if(t._module=f,f){var p=f.basePlotModule,g=p.attr,v=p.attributes;if(g&&v){var m=r._subplots,y="";if(h||"gl2d"!==p.name){if(Array.isArray(g))for(o=0;o<g.length;o++){var C=g[o],A=u.coerce(e,t,v,C);m[C]&&u.pushUnique(m[C],A),y+=A}else y=u.coerce(e,t,v,g);m[p.name]&&u.pushUnique(m[p.name],y)}}}return h&&(d("customdata"),d("ids"),d("meta"),a.traceIs(t,"showLegend")?(u.coerce(e,t,f.attributes.showlegend?f.attributes:b.attributes,"showlegend"),d("legendgroup"),t._dfltShowLegend=!0):t._dfltShowLegend=!1,f&&f.supplyDefaults(e,t,l,r),a.traceIs(t,"noOpacity")||d("opacity"),a.traceIs(t,"notLegendIsolatable")&&(t.visible=!!t.visible),a.traceIs(t,"noHover")||(t.hovertemplate||u.coerceHoverinfo(e,t,r),"parcats"!==t.type&&a.getComponentMethod("fx","supplyDefaults")(e,t,l,r)),f&&f.selectPoints&&d("selectedpoints"),b.supplyTransformDefaults(e,t,r)),t},b.hasMakesDataTransform=M,b.supplyTransformDefaults=function(e,t,n){if(t._length||M(e)){var r=n._globalTransforms||[],i=n._transformModules||[];if(Array.isArray(e.transforms)||0!==r.length)for(var o=e.transforms||[],a=r.concat(o),s=t.transforms=[],l=0;l<a.length;l++){var c,d=a[l],h=d.type,f=C[h],p=!(d._module&&d._module===f),g=f&&"function"==typeof f.transform;f||u.warn("Unrecognized transform type "+h+"."),f&&f.supplyDefaults&&(p||g)?((c=f.supplyDefaults(d,t,n,e)).type=h,c._module=f,u.pushUnique(i,f)):c=u.extendFlat({},d),s.push(c)}}},b.supplyLayoutGlobalDefaults=function(e,t,n){function r(n,r){return u.coerce(e,t,b.layoutAttributes,n,r)}var i=e.template;u.isPlainObject(i)&&(t.template=i,t._template=i.layout,t._dataTemplate=i.data),r("autotypenumbers");var o=u.coerceFont(r,"font");r("title.text",t._dfltTitle.plot),u.coerceFont(r,"title.font",{family:o.family,size:Math.round(1.4*o.size),color:o.color}),r("title.xref"),r("title.yref"),r("title.x"),r("title.y"),r("title.xanchor"),r("title.yanchor"),r("title.pad.t"),r("title.pad.r"),r("title.pad.b"),r("title.pad.l"),r("uniformtext.mode")&&r("uniformtext.minsize"),r("autosize",!(e.width&&e.height)),r("width"),r("height"),r("margin.l"),r("margin.r"),r("margin.t"),r("margin.b"),r("margin.pad"),r("margin.autoexpand"),e.width&&e.height&&b.sanitizeMargins(t),a.getComponentMethod("grid","sizeDefaults")(e,t),r("paper_bgcolor"),r("separators",n.decimal+n.thousands),r("hidesources"),r("colorway"),r("datarevision");var s=r("uirevision");r("editrevision",s),r("selectionrevision",s),r("modebar.orientation"),r("modebar.bgcolor",c.addOpacity(t.paper_bgcolor,.5));var l=c.contrast(c.rgb(t.modebar.bgcolor));r("modebar.color",c.addOpacity(l,.3)),r("modebar.activecolor",c.addOpacity(l,.7)),r("modebar.uirevision",s),a.getComponentMethod("shapes","supplyDrawNewShapeDefaults")(e,t,r),r("meta"),u.isPlainObject(e.transition)&&(r("transition.duration"),r("transition.easing"),r("transition.ordering")),a.getComponentMethod("calendars","handleDefaults")(e,t,"calendar"),a.getComponentMethod("fx","supplyLayoutGlobalDefaults")(e,t,r)},b.plotAutoSize=function(e,t,n){var r,i,a=e._context||{},s=a.frameMargins,l=u.isPlotDiv(e);if(l&&e.emit("plotly_autosize"),a.fillFrame)r=window.innerWidth,i=window.innerHeight,document.body.style.overflow="hidden";else{var c=l?window.getComputedStyle(e):{};if(r=T(c.width)||T(c.maxWidth)||n.width,i=T(c.height)||T(c.maxHeight)||n.height,o(s)&&s>0){var d=1-2*s;r=Math.round(d*r),i=Math.round(d*i)}}var h=b.layoutAttributes.width.min,f=b.layoutAttributes.height.min;r<h&&(r=h),i<f&&(i=f);var p=!t.width&&Math.abs(n.width-r)>1,g=!t.height&&Math.abs(n.height-i)>1;(g||p)&&(p&&(n.width=r),g&&(n.height=i)),e._initialAutoSize||(e._initialAutoSize={width:r,height:i}),b.sanitizeMargins(n)},b.supplyLayoutModuleDefaults=function(e,t,n,r){var i,o,s,l=a.componentsRegistry,c=t._basePlotModules,d=a.subplotsRegistry.cartesian;for(i in l)(s=l[i]).includeBasePlot&&s.includeBasePlot(e,t);for(var h in c.length||c.push(d),t._has("cartesian")&&(a.getComponentMethod("grid","contentDefaults")(e,t),d.finalizeSubplots(e,t)),t._subplots)t._subplots[h].sort(u.subplotSort);for(o=0;o<c.length;o++)(s=c[o]).supplyLayoutDefaults&&s.supplyLayoutDefaults(e,t,n);var f=t._modules;for(o=0;o<f.length;o++)(s=f[o]).supplyLayoutDefaults&&s.supplyLayoutDefaults(e,t,n);var p=t._transformModules;for(o=0;o<p.length;o++)(s=p[o]).supplyLayoutDefaults&&s.supplyLayoutDefaults(e,t,n,r);for(i in l)(s=l[i]).supplyLayoutDefaults&&s.supplyLayoutDefaults(e,t,n)},b.purge=function(e){var t=e._fullLayout||{};void 0!==t._glcontainer&&(t._glcontainer.selectAll(".gl-canvas").remove(),t._glcontainer.remove(),t._glcanvas=null),t._modeBar&&t._modeBar.destroy(),e._transitionData&&(e._transitionData._interruptCallbacks&&(e._transitionData._interruptCallbacks.length=0),e._transitionData._animationRaf&&window.cancelAnimationFrame(e._transitionData._animationRaf)),u.clearThrottle(),u.clearResponsive(e),delete e.data,delete e.layout,delete e._fullData,delete e._fullLayout,delete e.calcdata,delete e.framework,delete e.empty,delete e.fid,delete e.undoqueue,delete e.undonum,delete e.autoplay,delete e.changed,delete e._promises,delete e._redrawTimer,delete e._hmlumcount,delete e._hmpixcount,delete e._transitionData,delete e._transitioning,delete e._initialAutoSize,delete e._transitioningWithDuration,delete e._dragging,delete e._dragged,delete e._dragdata,delete e._hoverdata,delete e._snapshotInProgress,delete e._editing,delete e._mouseDownTime,delete e._legendMouseDownTime,e.removeAllListeners&&e.removeAllListeners()},b.style=function(e){var t,n=e._fullLayout._visibleModules,r=[];for(t=0;t<n.length;t++){var i=n[t];i.style&&u.pushUnique(r,i.style)}for(t=0;t<r.length;t++)r[t](e)},b.sanitizeMargins=function(e){if(e&&e.margin){var t,n=e.width,r=e.height,i=e.margin,o=n-(i.l+i.r),a=r-(i.t+i.b);o<0&&(t=(n-1)/(i.l+i.r),i.l=Math.floor(t*i.l),i.r=Math.floor(t*i.r)),a<0&&(t=(r-1)/(i.t+i.b),i.t=Math.floor(t*i.t),i.b=Math.floor(t*i.b))}},b.clearAutoMarginIds=function(e){e._fullLayout._pushmarginIds={}},b.allowAutoMargin=function(e,t){e._fullLayout._pushmarginIds[t]=1};b.autoMargin=function(e,t,n){var r=e._fullLayout,i=r.width,o=r.height,a=r.margin,s=u.constrain(i-a.l-a.r,2,64),l=u.constrain(o-a.t-a.b,2,64),c=Math.max(0,i-s),d=Math.max(0,o-l),h=r._pushmargin,f=r._pushmarginIds;if(!1!==a.autoexpand){if(n){var p=n.pad;if(void 0===p&&(p=Math.min(12,a.l,a.r,a.t,a.b)),c){var g=(n.l+n.r)/c;g>1&&(n.l/=g,n.r/=g)}if(d){var v=(n.t+n.b)/d;v>1&&(n.t/=v,n.b/=v)}var m=void 0!==n.xl?n.xl:n.x,y=void 0!==n.xr?n.xr:n.x,C=void 0!==n.yt?n.yt:n.y,A=void 0!==n.yb?n.yb:n.y;h[t]={l:{val:m,size:n.l+p},r:{val:y,size:n.r+p},b:{val:A,size:n.b+p},t:{val:C,size:n.t+p}},f[t]=1}else delete h[t],delete f[t];if(!r._replotting)return b.doAutoMargin(e)}},b.doAutoMargin=function(e){var t=e._fullLayout,n=t.width,r=t.height;t._size||(t._size={}),E(t);var i=t._size,s=t.margin,l=u.extendFlat({},i),c=s.l,d=s.r,f=s.t,p=s.b,g=t._pushmargin,v=t._pushmarginIds;if(!1!==t.margin.autoexpand){for(var m in g)v[m]||delete g[m];for(var y in g.base={l:{val:0,size:c},r:{val:1,size:d},t:{val:1,size:f},b:{val:0,size:p}},g){var C=g[y].l||{},A=g[y].b||{},_=C.val,I=C.size,x=A.val,w=A.size;for(var M in g){if(o(I)&&g[M].r){var S=g[M].r.val,T=g[M].r.size;if(S>_){var O=(I*S+(T-n)*_)/(S-_),k=(T*(1-_)+(I-n)*(1-S))/(S-_);O+k>c+d&&(c=O,d=k)}}if(o(w)&&g[M].t){var L=g[M].t.val,D=g[M].t.size;if(L>x){var P=(w*L+(D-r)*x)/(L-x),N=(D*(1-x)+(w-r)*(1-L))/(L-x);P+N>p+f&&(p=P,f=N)}}}}}var j=u.constrain(n-s.l-s.r,2,64),R=u.constrain(r-s.t-s.b,2,64),z=Math.max(0,n-j),F=Math.max(0,r-R);if(z){var B=(c+d)/z;B>1&&(c/=B,d/=B)}if(F){var H=(p+f)/F;H>1&&(p/=H,f/=H)}if(i.l=Math.round(c),i.r=Math.round(d),i.t=Math.round(f),i.b=Math.round(p),i.p=Math.round(s.pad),i.w=Math.round(n)-i.l-i.r,i.h=Math.round(r)-i.t-i.b,!t._replotting&&b.didMarginChange(l,i)){"_redrawFromAutoMarginCount"in t?t._redrawFromAutoMarginCount++:t._redrawFromAutoMarginCount=1;var U=3*(1+Object.keys(v).length);if(t._redrawFromAutoMarginCount<U)return a.call("plot",e);t._size=l,u.warn("Too many auto-margin redraws.")}!function(e){for(var t=h.list(e,"",!0),n=0;n<t.length;n++){var r=t[n]._hideOutOfRangeInsideTickLabels;r&&r()}}(e)};var O=["l","r","t","b","p","w","h"];function k(e,t,n){var r=!1;var i=[b.previousPromises,function(){if(e._transitionData)return e._transitioning=!1,function(e){var t=Promise.resolve();if(!e)return t;for(;e.length;)t=t.then(e.shift());return t}(e._transitionData._interruptCallbacks)},n.prepareFn,b.rehover,function(){return e.emit("plotly_transitioning",[]),new Promise((function(i){e._transitioning=!0,t.duration>0&&(e._transitioningWithDuration=!0),e._transitionData._interruptCallbacks.push((function(){r=!0})),n.redraw&&e._transitionData._interruptCallbacks.push((function(){return a.call("redraw",e)})),e._transitionData._interruptCallbacks.push((function(){e.emit("plotly_transitioninterrupted",[])}));var o=0,s=0;function l(){return o++,function(){s++,r||s!==o||function(t){if(!e._transitionData)return;(function(e){if(e)for(;e.length;)e.shift()})(e._transitionData._interruptCallbacks),Promise.resolve().then((function(){if(n.redraw)return a.call("redraw",e)})).then((function(){e._transitioning=!1,e._transitioningWithDuration=!1,e.emit("plotly_transitioned",[])})).then(t)}(i)}}n.runFn(l),setTimeout(l())}))}],o=u.syncOrAsync(i,e);return o&&o.then||(o=Promise.resolve()),o.then((function(){return e}))}b.didMarginChange=function(e,t){for(var n=0;n<O.length;n++){var r=O[n],i=e[r],a=t[r];if(!o(i)||Math.abs(a-i)>1)return!0}return!1},b.graphJson=function(e,t,n,r,i,o){(i&&t&&!e._fullData||i&&!t&&!e._fullLayout)&&b.supplyDefaults(e);var a=i?e._fullData:e.data,s=i?e._fullLayout:e.layout,l=(e._transitionData||{})._frames;function c(e,t){if("function"==typeof e)return t?"_function_":null;if(u.isPlainObject(e)){var r,i={};return Object.keys(e).sort().forEach((function(o){if(-1===["_","["].indexOf(o.charAt(0)))if("function"!=typeof e[o]){if("keepdata"===n){if("src"===o.substr(o.length-3))return}else if("keepstream"===n){if("string"==typeof(r=e[o+"src"])&&r.indexOf(":")>0&&!u.isPlainObject(e.stream))return}else if("keepall"!==n&&"string"==typeof(r=e[o+"src"])&&r.indexOf(":")>0)return;i[o]=c(e[o],t)}else t&&(i[o]="_function")})),i}return Array.isArray(e)?e.map((function(e){return c(e,t)})):u.isTypedArray(e)?u.simpleMap(e,u.identity):u.isJSDate(e)?u.ms2DateTimeLocal(+e):e}var d={data:(a||[]).map((function(e){var n=c(e);return t&&delete n.fit,n}))};if(!t&&(d.layout=c(s),i)){var h=s._size;d.layout.computed={margin:{b:h.b,l:h.l,r:h.r,t:h.t}}}return e.framework&&e.framework.isPolar&&(d=e.framework.getConfig()),l&&(d.frames=c(l)),o&&(d.config=c(e._context,!0)),"object"===r?d:JSON.stringify(d)},b.modifyFrames=function(e,t){var n,r,i,o=e._transitionData._frames,a=e._transitionData._frameHash;for(n=0;n<t.length;n++)switch((r=t[n]).type){case"replace":i=r.value;var s=(o[r.index]||{}).name,l=i.name;o[r.index]=a[l]=i,l!==s&&(delete a[s],a[l]=i);break;case"insert":a[(i=r.value).name]=i,o.splice(r.index,0,i);break;case"delete":delete a[(i=o[r.index]).name],o.splice(r.index,1)}return Promise.resolve()},b.computeFrame=function(e,t){var n,r,i,o,a=e._transitionData._frameHash;if(!t)throw new Error("computeFrame must be given a string frame name");var s=a[t.toString()];if(!s)return!1;for(var l=[s],u=[s.name];s.baseframe&&(s=a[s.baseframe.toString()])&&-1===u.indexOf(s.name);)l.push(s),u.push(s.name);for(var c={};s=l.pop();)if(s.layout&&(c.layout=b.extendLayout(c.layout,s.layout)),s.data){if(c.data||(c.data=[]),!(r=s.traces))for(r=[],n=0;n<s.data.length;n++)r[n]=n;for(c.traces||(c.traces=[]),n=0;n<s.data.length;n++)null!=(i=r[n])&&(-1===(o=c.traces.indexOf(i))&&(o=c.data.length,c.traces[o]=i),c.data[o]=b.extendTrace(c.data[o],s.data[n]))}return c},b.recomputeFrameHash=function(e){for(var t=e._transitionData._frameHash={},n=e._transitionData._frames,r=0;r<n.length;r++){var i=n[r];i&&i.name&&(t[i.name]=i)}},b.extendObjectWithContainers=function(e,t,n){var r,i,o,a,s,l,c,d=u.extendDeepNoArrays({},t||{}),h=u.expandObjectPaths(d),f={};if(n&&n.length)for(o=0;o<n.length;o++)void 0===(i=(r=u.nestedProperty(h,n[o])).get())?u.nestedProperty(f,n[o]).set(null):(r.set(null),u.nestedProperty(f,n[o]).set(i));if(e=u.extendDeepNoArrays(e||{},h),n&&n.length)for(o=0;o<n.length;o++)if(l=u.nestedProperty(f,n[o]).get()){for(c=(s=u.nestedProperty(e,n[o])).get(),Array.isArray(c)||(c=[],s.set(c)),a=0;a<l.length;a++){var p=l[a];c[a]=null===p?null:b.extendObjectWithContainers(c[a],p)}s.set(c)}return e},b.dataArrayContainers=["transforms","dimensions"],b.layoutArrayContainers=a.layoutArrayContainers,b.extendTrace=function(e,t){return b.extendObjectWithContainers(e,t,b.dataArrayContainers)},b.extendLayout=function(e,t){return b.extendObjectWithContainers(e,t,b.layoutArrayContainers)},b.transition=function(e,t,n,r,i,o){var a={redraw:i.redraw},s={},l=[];return a.prepareFn=function(){for(var i=Array.isArray(t)?t.length:0,o=r.slice(0,i),a=0;a<o.length;a++){var c=o[a],d=e._fullData[c]._module;if(d){if(d.animatable){var h=d.basePlotModule.name;s[h]||(s[h]=[]),s[h].push(c)}e.data[o[a]]=b.extendTrace(e.data[o[a]],t[a])}}var f=u.expandObjectPaths(u.extendDeepNoArrays({},n)),p=/^[xy]axis[0-9]*$/;for(var g in f)p.test(g)&&delete f[g].range;b.extendLayout(e.layout,f),delete e.calcdata,b.supplyDefaults(e),b.doCalcdata(e);var v=u.expandObjectPaths(n);if(v){var m=e._fullLayout._plots;for(var y in m){var C=m[y],A=C.xaxis,_=C.yaxis,I=A.range.slice(),x=_.range.slice(),w=null,M=null,S=null,T=null;Array.isArray(v[A._name+".range"])?w=v[A._name+".range"].slice():Array.isArray((v[A._name]||{}).range)&&(w=v[A._name].range.slice()),Array.isArray(v[_._name+".range"])?M=v[_._name+".range"].slice():Array.isArray((v[_._name]||{}).range)&&(M=v[_._name].range.slice()),I&&w&&(A.r2l(I[0])!==A.r2l(w[0])||A.r2l(I[1])!==A.r2l(w[1]))&&(S={xr0:I,xr1:w}),x&&M&&(_.r2l(x[0])!==_.r2l(M[0])||_.r2l(x[1])!==_.r2l(M[1]))&&(T={yr0:x,yr1:M}),(S||T)&&l.push(u.extendFlat({plotinfo:C},S,T))}}return Promise.resolve()},a.runFn=function(t){var r,i,a=e._fullLayout._basePlotModules,c=l.length;if(n)for(i=0;i<a.length;i++)a[i].transitionAxes&&a[i].transitionAxes(e,l,o,t);for(var d in c?((r=u.extendFlat({},o)).duration=0,delete s.cartesian):r=o,s){var h=s[d];e._fullData[h[0]]._module.basePlotModule.plot(e,h,r,t)}},k(e,o,a)},b.transitionFromReact=function(e,t,n,r){var i=e._fullLayout,o=i.transition,a={},s=[];return a.prepareFn=function(){var e=i._plots;for(var o in a.redraw=!1,"some"===t.anim&&(a.redraw=!0),"some"===n.anim&&(a.redraw=!0),e){var l=e[o],c=l.xaxis,d=l.yaxis,h=r[c._name].range.slice(),f=r[d._name].range.slice(),p=c.range.slice(),g=d.range.slice();c.setScale(),d.setScale();var v=null,m=null;c.r2l(h[0])===c.r2l(p[0])&&c.r2l(h[1])===c.r2l(p[1])||(v={xr0:h,xr1:p}),d.r2l(f[0])===d.r2l(g[0])&&d.r2l(f[1])===d.r2l(g[1])||(m={yr0:f,yr1:g}),(v||m)&&s.push(u.extendFlat({plotinfo:l},v,m))}return Promise.resolve()},a.runFn=function(n){for(var r,i,a,l=e._fullData,c=e._fullLayout._basePlotModules,d=[],h=0;h<l.length;h++)d.push(h);function f(){for(var t=0;t<c.length;t++)c[t].transitionAxes&&c[t].transitionAxes(e,s,r,n)}function p(){for(var t=0;t<c.length;t++)c[t].plot(e,a,i,n)}s.length&&t.anim?"traces first"===o.ordering?(r=u.extendFlat({},o,{duration:0}),a=d,i=o,setTimeout(f,o.duration),p()):(r=o,a=null,i=u.extendFlat({},o,{duration:0}),setTimeout(p,r.duration),f()):s.length?(r=o,f()):t.anim&&(a=d,i=o,p())},k(e,o,a)},b.doCalcdata=function(e,t){var n,r,i,o,l=h.list(e),c=e._fullData,f=e._fullLayout,p=new Array(c.length),g=(e.calcdata||[]).slice();for(e.calcdata=p,f._numBoxes=0,f._numViolins=0,f._violinScaleGroupStats={},e._hmpixcount=0,e._hmlumcount=0,f._piecolormap={},f._sunburstcolormap={},f._treemapcolormap={},f._funnelareacolormap={},i=0;i<c.length;i++)Array.isArray(t)&&-1===t.indexOf(i)&&(p[i]=g[i]);for(i=0;i<c.length;i++)(n=c[i])._arrayAttrs=s.findArrayAttributes(n),n._extremes={};var v=f._subplots.polar||[];for(i=0;i<v.length;i++)l.push(f[v[i]].radialaxis,f[v[i]].angularaxis);for(var m in f._colorAxes){var y=f[m];!1!==y.cauto&&(delete y.cmin,delete y.cmax)}var b=!1;function A(t){if(n=c[t],r=n._module,!0===n.visible&&n.transforms){if(r&&r.calc){var i=r.calc(e,n);i[0]&&i[0].t&&i[0].t._scene&&delete i[0].t._scene.dirty}for(o=0;o<n.transforms.length;o++){var a=n.transforms[o];(r=C[a.type])&&r.calcTransform&&(n._hasCalcTransform=!0,b=!0,r.calcTransform(e,n,a))}}}function _(t,i){if(n=c[t],!!(r=n._module).isContainer===i){var a=[];if(!0===n.visible&&0!==n._length){delete n._indexToPoints;var s=n.transforms||[];for(o=s.length-1;o>=0;o--)if(s[o].enabled){n._indexToPoints=s[o]._indexToPoints;break}r&&r.calc&&(a=r.calc(e,n))}Array.isArray(a)&&a[0]||(a=[{x:d,y:d}]),a[0].t||(a[0].t={}),a[0].trace=n,p[t]=a}}for(D(l,c,f),i=0;i<c.length;i++)_(i,!0);for(i=0;i<c.length;i++)A(i);for(b&&D(l,c,f),i=0;i<c.length;i++)_(i,!0);for(i=0;i<c.length;i++)_(i,!1);P(e);var I=function(e,t){var n,r,i,o,s,l=[];function c(e,n,r){var i=n._id.charAt(0);if("histogram2dcontour"===e){var o=n._counterAxes[0],a=h.getFromId(t,o),s="x"===i||"x"===o&&"category"===a.type,l="y"===i||"y"===o&&"category"===a.type;return function(e,t){return 0===e||0===t||s&&e===r[t].length-1||l&&t===r.length-1?-1:("y"===i?t:e)-1}}return function(e,t){return"y"===i?t:e}}var d={min:function(e){return u.aggNums(Math.min,null,e)},max:function(e){return u.aggNums(Math.max,null,e)},sum:function(e){return u.aggNums((function(e,t){return e+t}),null,e)},total:function(e){return u.aggNums((function(e,t){return e+t}),null,e)},mean:function(e){return u.mean(e)},median:function(e){return u.median(e)}};for(n=0;n<e.length;n++){var f=e[n];if("category"===f.type){var p=f.categoryorder.match(L);if(p){var g=p[1],v=p[2],m=f._id.charAt(0),y="x"===m,b=[];for(r=0;r<f._categories.length;r++)b.push([f._categories[r],[]]);for(r=0;r<f._traceIndices.length;r++){var C=f._traceIndices[r],A=t._fullData[C];if(!0===A.visible){var _=A.type;a.traceIs(A,"histogram")&&(delete A._xautoBinFinished,delete A._yautoBinFinished);var I="splom"===_,x="scattergl"===_,w=t.calcdata[C];for(i=0;i<w.length;i++){var M,S,T=w[i];if(I){var E=A._axesDim[f._id];if(!y){var O=A._diag[E][0];O&&(f=t._fullLayout[h.id2name(O)])}var k=T.trace.dimensions[E].values;for(o=0;o<k.length;o++)for(M=f._categoriesMap[k[o]],s=0;s<T.trace.dimensions.length;s++)if(s!==E){var D=T.trace.dimensions[s];b[M][1].push(D.values[o])}}else if(x){for(o=0;o<T.t.x.length;o++)y?(M=T.t.x[o],S=T.t.y[o]):(M=T.t.y[o],S=T.t.x[o]),b[M][1].push(S);T.t&&T.t._scene&&delete T.t._scene.dirty}else if(T.hasOwnProperty("z")){S=T.z;var P=c(A.type,f,S);for(o=0;o<S.length;o++)for(s=0;s<S[o].length;s++)(M=P(s,o))+1&&b[M][1].push(S[o][s])}else for(void 0===(M=T.p)&&(M=T[m]),void 0===(S=T.s)&&(S=T.v),void 0===S&&(S=y?T.y:T.x),Array.isArray(S)||(S=void 0===S?[]:[S]),o=0;o<S.length;o++)b[M][1].push(S[o])}}}f._categoriesValue=b;var N=[];for(r=0;r<b.length;r++)N.push([b[r][0],d[g](b[r][1])]);N.sort((function(e,t){return e[1]-t[1]})),f._categoriesAggregatedValue=N,f._initialCategories=N.map((function(e){return e[0]})),"descending"===v&&f._initialCategories.reverse(),l=l.concat(f.sortByInitialCategories())}}}return l}(l,e);if(I.length){for(f._numBoxes=0,f._numViolins=0,i=0;i<I.length;i++)_(I[i],!0);for(i=0;i<I.length;i++)_(I[i],!1);P(e)}a.getComponentMethod("fx","calc")(e),a.getComponentMethod("errorbars","calc")(e)};var L=/(total|sum|min|max|mean|median) (ascending|descending)/;function D(e,t,n){var r={};function i(e){e.clearCalc(),"multicategory"===e.type&&e.setupMultiCategory(t),r[e._id]=1}u.simpleMap(e,i);for(var o=n._axisMatchGroups||[],a=0;a<o.length;a++)for(var s in o[a])r[s]||i(n[h.id2name(s)])}function P(e){var t,n,r,i=e._fullLayout,o=i._visibleModules,a={};for(n=0;n<o.length;n++){var s=o[n],l=s.crossTraceCalc;if(l){var c=s.basePlotModule.name;a[c]?u.pushUnique(a[c],l):a[c]=[l]}}for(r in a){var d=a[r],h=i._subplots[r];if(Array.isArray(h))for(t=0;t<h.length;t++){var f=h[t],p="cartesian"===r?i._plots[f]:i[f];for(n=0;n<d.length;n++)d[n](e,p,f)}else for(n=0;n<d.length;n++)d[n](e)}}b.rehover=function(e){e._fullLayout._rehover&&e._fullLayout._rehover()},b.redrag=function(e){e._fullLayout._redrag&&e._fullLayout._redrag()},b.generalUpdatePerTraceModule=function(e,t,n,r){var i,o=t.traceHash,a={};for(i=0;i<n.length;i++){var s=n[i],l=s[0].trace;l.visible&&(a[l.type]=a[l.type]||[],a[l.type].push(s))}for(var c in o)if(!a[c]){var d=o[c][0];d[0].trace.visible=!1,a[c]=[d]}for(var h in a){var f=a[h];f[0][0].trace._module.plot(e,t,u.filterVisible(f),r)}t.traceHash=a},b.plotBasePlot=function(e,t,n,r,i){var o=a.getModule(e),s=v(t.calcdata,o)[0];o.plot(t,s,r,i)},b.cleanBasePlot=function(e,t,n,r,i){var o=i._has&&i._has(e),a=n._has&&n._has(e);o&&!a&&i["_"+e+"layer"].selectAll("g.trace").remove()}},function(e,t,n){"use strict";var r=n(27),i=n(95);function o(e,t){if(t&&t.length)for(var n=0;n<t.length;n++)if(t[n][e])return!0;return!1}t.id2name=function(e){if("string"==typeof e&&e.match(i.AX_ID_PATTERN)){var t=e.split(" ")[0].substr(1);return"1"===t&&(t=""),e.charAt(0)+"axis"+t}},t.name2id=function(e){if(e.match(i.AX_NAME_PATTERN)){var t=e.substr(5);return"1"===t&&(t=""),e.charAt(0)+t}},t.cleanId=function(e,t,n){var r=/( domain)$/.test(e);if("string"==typeof e&&e.match(i.AX_ID_PATTERN)&&(!t||e.charAt(0)===t)&&(!r||n)){var o=e.split(" ")[0].substr(1).replace(/^0+/,"");return"1"===o&&(o=""),e.charAt(0)+o+(r&&n?" domain":"")}},t.list=function(e,n,r){var i=e._fullLayout;if(!i)return[];var o,a=t.listIds(e,n),s=new Array(a.length);for(o=0;o<a.length;o++){var l=a[o];s[o]=i[l.charAt(0)+"axis"+l.substr(1)]}if(!r){var u=i._subplots.gl3d||[];for(o=0;o<u.length;o++){var c=i[u[o]];n?s.push(c[n+"axis"]):s.push(c.xaxis,c.yaxis,c.zaxis)}}return s},t.listIds=function(e,t){var n=e._fullLayout;if(!n)return[];var r=n._subplots;return t?r[t+"axis"]:r.xaxis.concat(r.yaxis)},t.getFromId=function(e,n,r){var i=e._fullLayout;return n=void 0===n||"string"!=typeof n?n:n.replace(" domain",""),"x"===r?n=n.replace(/y[0-9]*/,""):"y"===r&&(n=n.replace(/x[0-9]*/,"")),i[t.id2name(n)]},t.getFromTrace=function(e,n,i){var o=e._fullLayout,a=null;if(r.traceIs(n,"gl3d")){var s=n.scene;"scene"===s.substr(0,5)&&(a=o[s][i+"axis"])}else a=t.getFromId(e,n[i+"axis"]||i);return a},t.idSort=function(e,t){var n=e.charAt(0),r=t.charAt(0);return n!==r?n>r?1:-1:+(e.substr(1)||1)-+(t.substr(1)||1)},t.ref2id=function(e){return!!/^[xyz]/.test(e)&&e.split(" ")[0]},t.isLinked=function(e,t){return o(t,e._axisMatchGroups)||o(t,e._axisConstraintGroups)}},function(e,t,n){"use strict";var r=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),i=this&&this.__exportStar||function(e,t){for(var n in e)"default"===n||Object.prototype.hasOwnProperty.call(t,n)||r(t,e,n)};Object.defineProperty(t,"__esModule",{value:!0}),i(n(1340),t),i(n(841),t),i(n(842),t),i(n(489),t),i(n(1367),t),i(n(129),t),i(n(845),t),i(n(846),t),i(n(847),t),i(n(848),t),i(n(844),t)},function(e,t,n){"use strict";n(1);var r=n(204);t.a=function(){for(var e=arguments.length,t=new Array(e),n=0;n<e;n++)t[n]=arguments[n];var i=function(e){return t.reduce((function(t,n){var i=n(e);return i?Object(r.a)(t,i):t}),{})};return i.propTypes={},i.filterProps=t.reduce((function(e,t){return e.concat(t.filterProps)}),[]),i}},function(e,t,n){"use strict";n.r(t),n.d(t,"DisposableDelegate",(function(){return s})),n.d(t,"DisposableSet",(function(){return u})),n.d(t,"ObservableDisposableDelegate",(function(){return l})),n.d(t,"ObservableDisposableSet",(function(){return c}));var r=n(6),i=n(16),o=function(e,t){return(o=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(e,t)};function a(e,t){function n(){this.constructor=e}o(e,t),e.prototype=null===t?Object.create(t):(n.prototype=t.prototype,new n)}var s=function(){function e(e){this._fn=e}return Object.defineProperty(e.prototype,"isDisposed",{get:function(){return!this._fn},enumerable:!0,configurable:!0}),e.prototype.dispose=function(){if(this._fn){var e=this._fn;this._fn=null,e()}},e}(),l=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t._disposed=new i.Signal(t),t}return a(t,e),Object.defineProperty(t.prototype,"disposed",{get:function(){return this._disposed},enumerable:!0,configurable:!0}),t.prototype.dispose=function(){this.isDisposed||(e.prototype.dispose.call(this),this._disposed.emit(void 0),i.Signal.clearData(this))},t}(s),u=function(){function e(){this._isDisposed=!1,this._items=new Set}return Object.defineProperty(e.prototype,"isDisposed",{get:function(){return this._isDisposed},enumerable:!0,configurable:!0}),e.prototype.dispose=function(){this._isDisposed||(this._isDisposed=!0,this._items.forEach((function(e){e.dispose()})),this._items.clear())},e.prototype.contains=function(e){return this._items.has(e)},e.prototype.add=function(e){this._items.add(e)},e.prototype.remove=function(e){this._items.delete(e)},e.prototype.clear=function(){this._items.clear()},e}();!function(e){e.from=function(t){var n=new e;return Object(r.each)(t,(function(e){n.add(e)})),n}}(u||(u={}));var c=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t._disposed=new i.Signal(t),t}return a(t,e),Object.defineProperty(t.prototype,"disposed",{get:function(){return this._disposed},enumerable:!0,configurable:!0}),t.prototype.dispose=function(){this.isDisposed||(e.prototype.dispose.call(this),this._disposed.emit(void 0),i.Signal.clearData(this))},t}(u);!function(e){e.from=function(t){var n=new e;return Object(r.each)(t,(function(e){n.add(e)})),n}}(c||(c={}))},function(e,t,n){"use strict";function r(e,t){"function"==typeof e?e(t):e&&(e.current=t)}n.d(t,"a",(function(){return r}))},function(e,t,n){"use strict";n.r(t);var r=n(930);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";n.d(t,"a",(function(){return X})),n.d(t,"b",(function(){return Q})),n.d(t,"c",(function(){return me})),n.d(t,"d",(function(){return d})),n.d(t,"e",(function(){return ge})),n.d(t,"f",(function(){return ve})),n.d(t,"g",(function(){return f}));var r=n(1),i=n(229),o=(n(330),n(80)),a=n(101),s=n(226),l=n(92),u={}.constructor;function c(e){if(null==e||"object"!=typeof e)return e;if(Array.isArray(e))return e.map(c);if(e.constructor!==u)return e;var t={};for(var n in e)t[n]=c(e[n]);return t}function d(e,t,n){void 0===e&&(e="unnamed");var r=n.jss,i=c(t),o=r.plugins.onCreateRule(e,i,n);return o||(e[0],null)}var h=function(e,t){for(var n="",r=0;r<e.length&&"!important"!==e[r];r++)n&&(n+=t),n+=e[r];return n},f=function(e,t){if(void 0===t&&(t=!1),!Array.isArray(e))return e;var n="";if(Array.isArray(e[0]))for(var r=0;r<e.length&&"!important"!==e[r];r++)n&&(n+=", "),n+=h(e[r]," ");else n=h(e,", ");return t||"!important"!==e[e.length-1]||(n+=" !important"),n};function p(e,t){for(var n="",r=0;r<t;r++)n+="  ";return n+e}function g(e,t,n){void 0===n&&(n={});var r="";if(!t)return r;var i=n.indent,o=void 0===i?0:i,a=t.fallbacks;if(e&&o++,a)if(Array.isArray(a))for(var s=0;s<a.length;s++){var l=a[s];for(var u in l){var c=l[u];null!=c&&(r&&(r+="\n"),r+=""+p(u+": "+f(c)+";",o))}}else for(var d in a){var h=a[d];null!=h&&(r&&(r+="\n"),r+=""+p(d+": "+f(h)+";",o))}for(var g in t){var v=t[g];null!=v&&"fallbacks"!==g&&(r&&(r+="\n"),r+=""+p(g+": "+f(v)+";",o))}return(r||n.allowEmpty)&&e?(r&&(r="\n"+r+"\n"),p(e+" {"+r,--o)+p("}",o)):r}var v=/([[\].#*$><+~=|^:(),"'`\s])/g,m="undefined"!=typeof CSS&&CSS.escape,y=function(e){return m?m(e):e.replace(v,"\\$1")},b=function(){function e(e,t,n){this.type="style",this.key=void 0,this.isProcessed=!1,this.style=void 0,this.renderer=void 0,this.renderable=void 0,this.options=void 0;var r=n.sheet,i=n.Renderer;this.key=e,this.options=n,this.style=t,r?this.renderer=r.renderer:i&&(this.renderer=new i)}return e.prototype.prop=function(e,t,n){if(void 0===t)return this.style[e];var r=!!n&&n.force;if(!r&&this.style[e]===t)return this;var i=t;n&&!1===n.process||(i=this.options.jss.plugins.onChangeValue(t,e,this));var o=null==i||!1===i,a=e in this.style;if(o&&!a&&!r)return this;var s=o&&a;if(s?delete this.style[e]:this.style[e]=i,this.renderable&&this.renderer)return s?this.renderer.removeProperty(this.renderable,e):this.renderer.setProperty(this.renderable,e,i),this;var l=this.options.sheet;return l&&l.attached,this},e}(),C=function(e){function t(t,n,r){var i;(i=e.call(this,t,n,r)||this).selectorText=void 0,i.id=void 0,i.renderable=void 0;var o=r.selector,a=r.scoped,l=r.sheet,u=r.generateId;return o?i.selectorText=o:!1!==a&&(i.id=u(Object(s.a)(Object(s.a)(i)),l),i.selectorText="."+y(i.id)),i}Object(a.a)(t,e);var n=t.prototype;return n.applyTo=function(e){var t=this.renderer;if(t){var n=this.toJSON();for(var r in n)t.setProperty(e,r,n[r])}return this},n.toJSON=function(){var e={};for(var t in this.style){var n=this.style[t];"object"!=typeof n?e[t]=n:Array.isArray(n)&&(e[t]=f(n))}return e},n.toString=function(e){var t=this.options.sheet,n=!!t&&t.options.link?Object(r.a)({},e,{allowEmpty:!0}):e;return g(this.selectorText,this.style,n)},Object(o.a)(t,[{key:"selector",set:function(e){if(e!==this.selectorText){this.selectorText=e;var t=this.renderer,n=this.renderable;if(n&&t)t.setSelector(n,e)||t.replaceRule(n,this)}},get:function(){return this.selectorText}}]),t}(b),A={onCreateRule:function(e,t,n){return"@"===e[0]||n.parent&&"keyframes"===n.parent.type?null:new C(e,t,n)}},_={indent:1,children:!0},I=/@([\w-]+)/,x=function(){function e(e,t,n){this.type="conditional",this.at=void 0,this.key=void 0,this.query=void 0,this.rules=void 0,this.options=void 0,this.isProcessed=!1,this.renderable=void 0,this.key=e;var i=e.match(I);for(var o in this.at=i?i[1]:"unknown",this.query=n.name||"@"+this.at,this.options=n,this.rules=new X(Object(r.a)({},n,{parent:this})),t)this.rules.add(o,t[o]);this.rules.process()}var t=e.prototype;return t.getRule=function(e){return this.rules.get(e)},t.indexOf=function(e){return this.rules.indexOf(e)},t.addRule=function(e,t,n){var r=this.rules.add(e,t,n);return r?(this.options.jss.plugins.onProcessRule(r),r):null},t.toString=function(e){if(void 0===e&&(e=_),null==e.indent&&(e.indent=_.indent),null==e.children&&(e.children=_.children),!1===e.children)return this.query+" {}";var t=this.rules.toString(e);return t?this.query+" {\n"+t+"\n}":""},e}(),w=/@media|@supports\s+/,M={onCreateRule:function(e,t,n){return w.test(e)?new x(e,t,n):null}},S={indent:1,children:!0},T=/@keyframes\s+([\w-]+)/,E=function(){function e(e,t,n){this.type="keyframes",this.at="@keyframes",this.key=void 0,this.name=void 0,this.id=void 0,this.rules=void 0,this.options=void 0,this.isProcessed=!1,this.renderable=void 0;var i=e.match(T);i&&i[1]?this.name=i[1]:this.name="noname",this.key=this.type+"-"+this.name,this.options=n;var o=n.scoped,a=n.sheet,s=n.generateId;for(var l in this.id=!1===o?this.name:y(s(this,a)),this.rules=new X(Object(r.a)({},n,{parent:this})),t)this.rules.add(l,t[l],Object(r.a)({},n,{parent:this}));this.rules.process()}return e.prototype.toString=function(e){if(void 0===e&&(e=S),null==e.indent&&(e.indent=S.indent),null==e.children&&(e.children=S.children),!1===e.children)return this.at+" "+this.id+" {}";var t=this.rules.toString(e);return t&&(t="\n"+t+"\n"),this.at+" "+this.id+" {"+t+"}"},e}(),O=/@keyframes\s+/,k=/\$([\w-]+)/g,L=function(e,t){return"string"==typeof e?e.replace(k,(function(e,n){return n in t?t[n]:e})):e},D=function(e,t,n){var r=e[t],i=L(r,n);i!==r&&(e[t]=i)},P={onCreateRule:function(e,t,n){return"string"==typeof e&&O.test(e)?new E(e,t,n):null},onProcessStyle:function(e,t,n){return"style"===t.type&&n?("animation-name"in e&&D(e,"animation-name",n.keyframes),"animation"in e&&D(e,"animation",n.keyframes),e):e},onChangeValue:function(e,t,n){var r=n.options.sheet;if(!r)return e;switch(t){case"animation":case"animation-name":return L(e,r.keyframes);default:return e}}},N=function(e){function t(){for(var t,n=arguments.length,r=new Array(n),i=0;i<n;i++)r[i]=arguments[i];return(t=e.call.apply(e,[this].concat(r))||this).renderable=void 0,t}return Object(a.a)(t,e),t.prototype.toString=function(e){var t=this.options.sheet,n=!!t&&t.options.link?Object(r.a)({},e,{allowEmpty:!0}):e;return g(this.key,this.style,n)},t}(b),j={onCreateRule:function(e,t,n){return n.parent&&"keyframes"===n.parent.type?new N(e,t,n):null}},R=function(){function e(e,t,n){this.type="font-face",this.at="@font-face",this.key=void 0,this.style=void 0,this.options=void 0,this.isProcessed=!1,this.renderable=void 0,this.key=e,this.style=t,this.options=n}return e.prototype.toString=function(e){if(Array.isArray(this.style)){for(var t="",n=0;n<this.style.length;n++)t+=g(this.at,this.style[n]),this.style[n+1]&&(t+="\n");return t}return g(this.at,this.style,e)},e}(),z=/@font-face/,F={onCreateRule:function(e,t,n){return z.test(e)?new R(e,t,n):null}},B=function(){function e(e,t,n){this.type="viewport",this.at="@viewport",this.key=void 0,this.style=void 0,this.options=void 0,this.isProcessed=!1,this.renderable=void 0,this.key=e,this.style=t,this.options=n}return e.prototype.toString=function(e){return g(this.key,this.style,e)},e}(),H={onCreateRule:function(e,t,n){return"@viewport"===e||"@-ms-viewport"===e?new B(e,t,n):null}},U=function(){function e(e,t,n){this.type="simple",this.key=void 0,this.value=void 0,this.options=void 0,this.isProcessed=!1,this.renderable=void 0,this.key=e,this.value=t,this.options=n}return e.prototype.toString=function(e){if(Array.isArray(this.value)){for(var t="",n=0;n<this.value.length;n++)t+=this.key+" "+this.value[n]+";",this.value[n+1]&&(t+="\n");return t}return this.key+" "+this.value+";"},e}(),V={"@charset":!0,"@import":!0,"@namespace":!0},Y=[A,M,P,j,F,H,{onCreateRule:function(e,t,n){return e in V?new U(e,t,n):null}}],W={process:!0},G={force:!0,process:!0},X=function(){function e(e){this.map={},this.raw={},this.index=[],this.counter=0,this.options=void 0,this.classes=void 0,this.keyframes=void 0,this.options=e,this.classes=e.classes,this.keyframes=e.keyframes}var t=e.prototype;return t.add=function(e,t,n){var i=this.options,o=i.parent,a=i.sheet,s=i.jss,l=i.Renderer,u=i.generateId,c=i.scoped,h=Object(r.a)({classes:this.classes,parent:o,sheet:a,jss:s,Renderer:l,generateId:u,scoped:c,name:e,keyframes:this.keyframes,selector:void 0},n),f=e;e in this.raw&&(f=e+"-d"+this.counter++),this.raw[f]=t,f in this.classes&&(h.selector="."+y(this.classes[f]));var p=d(f,t,h);if(!p)return null;this.register(p);var g=void 0===h.index?this.index.length:h.index;return this.index.splice(g,0,p),p},t.get=function(e){return this.map[e]},t.remove=function(e){this.unregister(e),delete this.raw[e.key],this.index.splice(this.index.indexOf(e),1)},t.indexOf=function(e){return this.index.indexOf(e)},t.process=function(){var e=this.options.jss.plugins;this.index.slice(0).forEach(e.onProcessRule,e)},t.register=function(e){this.map[e.key]=e,e instanceof C?(this.map[e.selector]=e,e.id&&(this.classes[e.key]=e.id)):e instanceof E&&this.keyframes&&(this.keyframes[e.name]=e.id)},t.unregister=function(e){delete this.map[e.key],e instanceof C?(delete this.map[e.selector],delete this.classes[e.key]):e instanceof E&&delete this.keyframes[e.name]},t.update=function(){var e,t,n;if("string"==typeof(arguments.length<=0?void 0:arguments[0])?(e=arguments.length<=0?void 0:arguments[0],t=arguments.length<=1?void 0:arguments[1],n=arguments.length<=2?void 0:arguments[2]):(t=arguments.length<=0?void 0:arguments[0],n=arguments.length<=1?void 0:arguments[1],e=null),e)this.updateOne(this.map[e],t,n);else for(var r=0;r<this.index.length;r++)this.updateOne(this.index[r],t,n)},t.updateOne=function(t,n,r){void 0===r&&(r=W);var i=this.options,o=i.jss.plugins,a=i.sheet;if(t.rules instanceof e)t.rules.update(n,r);else{var s=t,l=s.style;if(o.onUpdate(n,t,a,r),r.process&&l&&l!==s.style){for(var u in o.onProcessStyle(s.style,s,a),s.style){var c=s.style[u];c!==l[u]&&s.prop(u,c,G)}for(var d in l){var h=s.style[d],f=l[d];null==h&&h!==f&&s.prop(d,null,G)}}}},t.toString=function(e){for(var t="",n=this.options.sheet,r=!!n&&n.options.link,i=0;i<this.index.length;i++){var o=this.index[i].toString(e);(o||r)&&(t&&(t+="\n"),t+=o)}return t},e}(),q=function(){function e(e,t){for(var n in this.options=void 0,this.deployed=void 0,this.attached=void 0,this.rules=void 0,this.renderer=void 0,this.classes=void 0,this.keyframes=void 0,this.queue=void 0,this.attached=!1,this.deployed=!1,this.classes={},this.keyframes={},this.options=Object(r.a)({},t,{sheet:this,parent:this,classes:this.classes,keyframes:this.keyframes}),t.Renderer&&(this.renderer=new t.Renderer(this)),this.rules=new X(this.options),e)this.rules.add(n,e[n]);this.rules.process()}var t=e.prototype;return t.attach=function(){return this.attached||(this.renderer&&this.renderer.attach(),this.attached=!0,this.deployed||this.deploy()),this},t.detach=function(){return this.attached?(this.renderer&&this.renderer.detach(),this.attached=!1,this):this},t.addRule=function(e,t,n){var r=this.queue;this.attached&&!r&&(this.queue=[]);var i=this.rules.add(e,t,n);return i?(this.options.jss.plugins.onProcessRule(i),this.attached?this.deployed?(r?r.push(i):(this.insertRule(i),this.queue&&(this.queue.forEach(this.insertRule,this),this.queue=void 0)),i):i:(this.deployed=!1,i)):null},t.insertRule=function(e){this.renderer&&this.renderer.insertRule(e)},t.addRules=function(e,t){var n=[];for(var r in e){var i=this.addRule(r,e[r],t);i&&n.push(i)}return n},t.getRule=function(e){return this.rules.get(e)},t.deleteRule=function(e){var t="object"==typeof e?e:this.rules.get(e);return!(!t||this.attached&&!t.renderable)&&(this.rules.remove(t),!(this.attached&&t.renderable&&this.renderer)||this.renderer.deleteRule(t.renderable))},t.indexOf=function(e){return this.rules.indexOf(e)},t.deploy=function(){return this.renderer&&this.renderer.deploy(),this.deployed=!0,this},t.update=function(){var e;return(e=this.rules).update.apply(e,arguments),this},t.updateOne=function(e,t,n){return this.rules.updateOne(e,t,n),this},t.toString=function(e){return this.rules.toString(e)},e}(),Z=function(){function e(){this.plugins={internal:[],external:[]},this.registry=void 0}var t=e.prototype;return t.onCreateRule=function(e,t,n){for(var r=0;r<this.registry.onCreateRule.length;r++){var i=this.registry.onCreateRule[r](e,t,n);if(i)return i}return null},t.onProcessRule=function(e){if(!e.isProcessed){for(var t=e.options.sheet,n=0;n<this.registry.onProcessRule.length;n++)this.registry.onProcessRule[n](e,t);e.style&&this.onProcessStyle(e.style,e,t),e.isProcessed=!0}},t.onProcessStyle=function(e,t,n){for(var r=0;r<this.registry.onProcessStyle.length;r++)t.style=this.registry.onProcessStyle[r](t.style,t,n)},t.onProcessSheet=function(e){for(var t=0;t<this.registry.onProcessSheet.length;t++)this.registry.onProcessSheet[t](e)},t.onUpdate=function(e,t,n,r){for(var i=0;i<this.registry.onUpdate.length;i++)this.registry.onUpdate[i](e,t,n,r)},t.onChangeValue=function(e,t,n){for(var r=e,i=0;i<this.registry.onChangeValue.length;i++)r=this.registry.onChangeValue[i](r,t,n);return r},t.use=function(e,t){void 0===t&&(t={queue:"external"});var n=this.plugins[t.queue];-1===n.indexOf(e)&&(n.push(e),this.registry=[].concat(this.plugins.external,this.plugins.internal).reduce((function(e,t){for(var n in t)n in e&&e[n].push(t[n]);return e}),{onCreateRule:[],onProcessRule:[],onProcessStyle:[],onProcessSheet:[],onChangeValue:[],onUpdate:[]}))},e}(),Q=function(){function e(){this.registry=[]}var t=e.prototype;return t.add=function(e){var t=this.registry,n=e.options.index;if(-1===t.indexOf(e))if(0===t.length||n>=this.index)t.push(e);else for(var r=0;r<t.length;r++)if(t[r].options.index>n)return void t.splice(r,0,e)},t.reset=function(){this.registry=[]},t.remove=function(e){var t=this.registry.indexOf(e);this.registry.splice(t,1)},t.toString=function(e){for(var t=void 0===e?{}:e,n=t.attached,r=Object(l.a)(t,["attached"]),i="",o=0;o<this.registry.length;o++){var a=this.registry[o];null!=n&&a.attached!==n||(i&&(i+="\n"),i+=a.toString(r))}return i},Object(o.a)(e,[{key:"index",get:function(){return 0===this.registry.length?0:this.registry[this.registry.length-1].options.index}}]),e}(),K=new Q,J="undefined"!=typeof window&&window.Math==Math?window:"undefined"!=typeof self&&self.Math==Math?self:Function("return this")(),$="2f1acc6c3a606b082e5eef5e54414ffb";null==J[$]&&(J[$]=0);var ee=J[$]++,te=function(e){void 0===e&&(e={});var t=0;return function(n,r){t+=1;var i="",o="";return r&&(r.options.classNamePrefix&&(o=r.options.classNamePrefix),null!=r.options.jss.id&&(i=String(r.options.jss.id))),e.minify?""+(o||"c")+ee+i+t:o+n.key+"-"+ee+(i?"-"+i:"")+"-"+t}},ne=function(e){var t;return function(){return t||(t=e()),t}},re=function(e,t){try{return e.attributeStyleMap?e.attributeStyleMap.get(t):e.style.getPropertyValue(t)}catch(e){return""}},ie=function(e,t,n){try{var r=n;if(Array.isArray(n)&&(r=f(n,!0),"!important"===n[n.length-1]))return e.style.setProperty(t,r,"important"),!0;e.attributeStyleMap?e.attributeStyleMap.set(t,r):e.style.setProperty(t,r)}catch(e){return!1}return!0},oe=function(e,t){try{e.attributeStyleMap?e.attributeStyleMap.delete(t):e.style.removeProperty(t)}catch(e){}},ae=function(e,t){return e.selectorText=t,e.selectorText===t},se=ne((function(){return document.querySelector("head")}));function le(e){var t=K.registry;if(t.length>0){var n=function(e,t){for(var n=0;n<e.length;n++){var r=e[n];if(r.attached&&r.options.index>t.index&&r.options.insertionPoint===t.insertionPoint)return r}return null}(t,e);if(n&&n.renderer)return{parent:n.renderer.element.parentNode,node:n.renderer.element};if((n=function(e,t){for(var n=e.length-1;n>=0;n--){var r=e[n];if(r.attached&&r.options.insertionPoint===t.insertionPoint)return r}return null}(t,e))&&n.renderer)return{parent:n.renderer.element.parentNode,node:n.renderer.element.nextSibling}}var r=e.insertionPoint;if(r&&"string"==typeof r){var i=function(e){for(var t=se(),n=0;n<t.childNodes.length;n++){var r=t.childNodes[n];if(8===r.nodeType&&r.nodeValue.trim()===e)return r}return null}(r);if(i)return{parent:i.parentNode,node:i.nextSibling}}return!1}var ue=ne((function(){var e=document.querySelector('meta[property="csp-nonce"]');return e?e.getAttribute("content"):null})),ce=function(e,t,n){try{if("insertRule"in e)e.insertRule(t,n);else if("appendRule"in e){e.appendRule(t)}}catch(e){return!1}return e.cssRules[n]},de=function(e,t){var n=e.cssRules.length;return void 0===t||t>n?n:t},he=function(){function e(e){this.getPropertyValue=re,this.setProperty=ie,this.removeProperty=oe,this.setSelector=ae,this.element=void 0,this.sheet=void 0,this.hasInsertedRules=!1,this.cssRules=[],e&&K.add(e),this.sheet=e;var t,n=this.sheet?this.sheet.options:{},r=n.media,i=n.meta,o=n.element;this.element=o||((t=document.createElement("style")).textContent="\n",t),this.element.setAttribute("data-jss",""),r&&this.element.setAttribute("media",r),i&&this.element.setAttribute("data-meta",i);var a=ue();a&&this.element.setAttribute("nonce",a)}var t=e.prototype;return t.attach=function(){if(!this.element.parentNode&&this.sheet){!function(e,t){var n=t.insertionPoint,r=le(t);if(!1!==r&&r.parent)r.parent.insertBefore(e,r.node);else if(n&&"number"==typeof n.nodeType){var i=n,o=i.parentNode;o&&o.insertBefore(e,i.nextSibling)}else se().appendChild(e)}(this.element,this.sheet.options);var e=Boolean(this.sheet&&this.sheet.deployed);this.hasInsertedRules&&e&&(this.hasInsertedRules=!1,this.deploy())}},t.detach=function(){if(this.sheet){var e=this.element.parentNode;e&&e.removeChild(this.element),this.sheet.options.link&&(this.cssRules=[],this.element.textContent="\n")}},t.deploy=function(){var e=this.sheet;e&&(e.options.link?this.insertRules(e.rules):this.element.textContent="\n"+e.toString()+"\n")},t.insertRules=function(e,t){for(var n=0;n<e.index.length;n++)this.insertRule(e.index[n],n,t)},t.insertRule=function(e,t,n){if(void 0===n&&(n=this.element.sheet),e.rules){var r=e,i=n;if("conditional"===e.type||"keyframes"===e.type){var o=de(n,t);if(!1===(i=ce(n,r.toString({children:!1}),o)))return!1;this.refCssRule(e,o,i)}return this.insertRules(r.rules,i),i}var a=e.toString();if(!a)return!1;var s=de(n,t),l=ce(n,a,s);return!1!==l&&(this.hasInsertedRules=!0,this.refCssRule(e,s,l),l)},t.refCssRule=function(e,t,n){e.renderable=n,e.options.parent instanceof q&&(this.cssRules[t]=n)},t.deleteRule=function(e){var t=this.element.sheet,n=this.indexOf(e);return-1!==n&&(t.deleteRule(n),this.cssRules.splice(n,1),!0)},t.indexOf=function(e){return this.cssRules.indexOf(e)},t.replaceRule=function(e,t){var n=this.indexOf(e);return-1!==n&&(this.element.sheet.deleteRule(n),this.cssRules.splice(n,1),this.insertRule(t,n))},t.getRules=function(){return this.element.sheet.cssRules},e}(),fe=0,pe=function(){function e(e){this.id=fe++,this.version="10.5.1",this.plugins=new Z,this.options={id:{minify:!1},createGenerateId:te,Renderer:i.a?he:null,plugins:[]},this.generateId=te({minify:!1});for(var t=0;t<Y.length;t++)this.plugins.use(Y[t],{queue:"internal"});this.setup(e)}var t=e.prototype;return t.setup=function(e){return void 0===e&&(e={}),e.createGenerateId&&(this.options.createGenerateId=e.createGenerateId),e.id&&(this.options.id=Object(r.a)({},this.options.id,e.id)),(e.createGenerateId||e.id)&&(this.generateId=this.options.createGenerateId(this.options.id)),null!=e.insertionPoint&&(this.options.insertionPoint=e.insertionPoint),"Renderer"in e&&(this.options.Renderer=e.Renderer),e.plugins&&this.use.apply(this,e.plugins),this},t.createStyleSheet=function(e,t){void 0===t&&(t={});var n=t.index;"number"!=typeof n&&(n=0===K.index?0:K.index+1);var i=new q(e,Object(r.a)({},t,{jss:this,generateId:t.generateId||this.generateId,insertionPoint:this.options.insertionPoint,Renderer:this.options.Renderer,index:n}));return this.plugins.onProcessSheet(i),i},t.removeStyleSheet=function(e){return e.detach(),K.remove(e),this},t.createRule=function(e,t,n){if(void 0===t&&(t={}),void 0===n&&(n={}),"object"==typeof e)return this.createRule(void 0,e,t);var i=Object(r.a)({},n,{name:e,jss:this,Renderer:this.options.Renderer});i.generateId||(i.generateId=this.generateId),i.classes||(i.classes={}),i.keyframes||(i.keyframes={});var o=d(e,t,i);return o&&this.plugins.onProcessRule(o),o},t.use=function(){for(var e=this,t=arguments.length,n=new Array(t),r=0;r<t;r++)n[r]=arguments[r];return n.forEach((function(t){e.plugins.use(t)})),this},e}();function ge(e){var t=null;for(var n in e){var r=e[n],i=typeof r;if("function"===i)t||(t={}),t[n]=r;else if("object"===i&&null!==r&&!Array.isArray(r)){var o=ge(r);o&&(t||(t={}),t[n]=o)}}return t}var ve="object"==typeof CSS&&null!=CSS&&"number"in CSS,me=function(e){return new pe(e)};
/**
 * A better abstraction over CSS.
 *
 * @copyright Oleg Isonen (Slobodskoi) / Isonen 2014-present
 * @website https://github.com/cssinjs/jss
 * @license MIT
 */me()},function(e,t,n){"use strict";n.r(t);var r={};n.r(r),n.d(r,"common",(function(){return i.a})),n.d(r,"red",(function(){return o.a})),n.d(r,"pink",(function(){return a.a})),n.d(r,"purple",(function(){return s})),n.d(r,"deepPurple",(function(){return l})),n.d(r,"indigo",(function(){return u.a})),n.d(r,"blue",(function(){return c.a})),n.d(r,"lightBlue",(function(){return d})),n.d(r,"cyan",(function(){return h})),n.d(r,"teal",(function(){return f})),n.d(r,"green",(function(){return p.a})),n.d(r,"lightGreen",(function(){return g})),n.d(r,"lime",(function(){return v})),n.d(r,"yellow",(function(){return m})),n.d(r,"amber",(function(){return y})),n.d(r,"orange",(function(){return b.a})),n.d(r,"deepOrange",(function(){return C})),n.d(r,"brown",(function(){return A})),n.d(r,"grey",(function(){return _.a})),n.d(r,"blueGrey",(function(){return I}));var i=n(225),o=n(266),a=n(267),s={50:"#f3e5f5",100:"#e1bee7",200:"#ce93d8",300:"#ba68c8",400:"#ab47bc",500:"#9c27b0",600:"#8e24aa",700:"#7b1fa2",800:"#6a1b9a",900:"#4a148c",A100:"#ea80fc",A200:"#e040fb",A400:"#d500f9",A700:"#aa00ff"},l={50:"#ede7f6",100:"#d1c4e9",200:"#b39ddb",300:"#9575cd",400:"#7e57c2",500:"#673ab7",600:"#5e35b1",700:"#512da8",800:"#4527a0",900:"#311b92",A100:"#b388ff",A200:"#7c4dff",A400:"#651fff",A700:"#6200ea"},u=n(268),c=n(269),d={50:"#e1f5fe",100:"#b3e5fc",200:"#81d4fa",300:"#4fc3f7",400:"#29b6f6",500:"#03a9f4",600:"#039be5",700:"#0288d1",800:"#0277bd",900:"#01579b",A100:"#80d8ff",A200:"#40c4ff",A400:"#00b0ff",A700:"#0091ea"},h={50:"#e0f7fa",100:"#b2ebf2",200:"#80deea",300:"#4dd0e1",400:"#26c6da",500:"#00bcd4",600:"#00acc1",700:"#0097a7",800:"#00838f",900:"#006064",A100:"#84ffff",A200:"#18ffff",A400:"#00e5ff",A700:"#00b8d4"},f={50:"#e0f2f1",100:"#b2dfdb",200:"#80cbc4",300:"#4db6ac",400:"#26a69a",500:"#009688",600:"#00897b",700:"#00796b",800:"#00695c",900:"#004d40",A100:"#a7ffeb",A200:"#64ffda",A400:"#1de9b6",A700:"#00bfa5"},p=n(270),g={50:"#f1f8e9",100:"#dcedc8",200:"#c5e1a5",300:"#aed581",400:"#9ccc65",500:"#8bc34a",600:"#7cb342",700:"#689f38",800:"#558b2f",900:"#33691e",A100:"#ccff90",A200:"#b2ff59",A400:"#76ff03",A700:"#64dd17"},v={50:"#f9fbe7",100:"#f0f4c3",200:"#e6ee9c",300:"#dce775",400:"#d4e157",500:"#cddc39",600:"#c0ca33",700:"#afb42b",800:"#9e9d24",900:"#827717",A100:"#f4ff81",A200:"#eeff41",A400:"#c6ff00",A700:"#aeea00"},m={50:"#fffde7",100:"#fff9c4",200:"#fff59d",300:"#fff176",400:"#ffee58",500:"#ffeb3b",600:"#fdd835",700:"#fbc02d",800:"#f9a825",900:"#f57f17",A100:"#ffff8d",A200:"#ffff00",A400:"#ffea00",A700:"#ffd600"},y={50:"#fff8e1",100:"#ffecb3",200:"#ffe082",300:"#ffd54f",400:"#ffca28",500:"#ffc107",600:"#ffb300",700:"#ffa000",800:"#ff8f00",900:"#ff6f00",A100:"#ffe57f",A200:"#ffd740",A400:"#ffc400",A700:"#ffab00"},b=n(271),C={50:"#fbe9e7",100:"#ffccbc",200:"#ffab91",300:"#ff8a65",400:"#ff7043",500:"#ff5722",600:"#f4511e",700:"#e64a19",800:"#d84315",900:"#bf360c",A100:"#ff9e80",A200:"#ff6e40",A400:"#ff3d00",A700:"#dd2c00"},A={50:"#efebe9",100:"#d7ccc8",200:"#bcaaa4",300:"#a1887f",400:"#8d6e63",500:"#795548",600:"#6d4c41",700:"#5d4037",800:"#4e342e",900:"#3e2723",A100:"#d7ccc8",A200:"#bcaaa4",A400:"#8d6e63",A700:"#5d4037"},_=n(272),I={50:"#eceff1",100:"#cfd8dc",200:"#b0bec5",300:"#90a4ae",400:"#78909c",500:"#607d8b",600:"#546e7a",700:"#455a64",800:"#37474f",900:"#263238",A100:"#cfd8dc",A200:"#b0bec5",A400:"#78909c",A700:"#455a64"},x=n(104),w=n(222),M=n(904),S=n(1),T=n(5),E=n(0),O=(n(3),n(4)),k=n(7),L=E.forwardRef((function(e,t){var n=e.classes,r=e.className,i=e.disableSpacing,o=void 0!==i&&i,a=Object(T.a)(e,["classes","className","disableSpacing"]);return E.createElement("div",Object(S.a)({className:Object(O.a)(n.root,r,!o&&n.spacing),ref:t},a))})),D=Object(k.a)({root:{display:"flex",alignItems:"center",padding:8,justifyContent:"flex-end"},spacing:{"& > :not(:first-child)":{marginLeft:8}}},{name:"MuiAccordionActions"})(L),P=n(905),N=n(906),j=n(15),R=n(336),z=E.forwardRef((function(e,t){var n=e.classes,r=e.className,i=e.color,o=void 0===i?"primary":i,a=e.position,s=void 0===a?"fixed":a,l=Object(T.a)(e,["classes","className","color","position"]);return E.createElement(R.a,Object(S.a)({square:!0,component:"header",elevation:4,className:Object(O.a)(n.root,n["position".concat(Object(j.a)(s))],n["color".concat(Object(j.a)(o))],r,"fixed"===s&&"mui-fixed"),ref:t},l))})),F=Object(k.a)((function(e){var t="light"===e.palette.type?e.palette.grey[100]:e.palette.grey[900];return{root:{display:"flex",flexDirection:"column",width:"100%",boxSizing:"border-box",zIndex:e.zIndex.appBar,flexShrink:0},positionFixed:{position:"fixed",top:0,left:"auto",right:0,"@media print":{position:"absolute"}},positionAbsolute:{position:"absolute",top:0,left:"auto",right:0},positionSticky:{position:"sticky",top:0,left:"auto",right:0},positionStatic:{position:"static"},positionRelative:{position:"relative"},colorDefault:{backgroundColor:t,color:e.palette.getContrastText(t)},colorPrimary:{backgroundColor:e.palette.primary.main,color:e.palette.primary.contrastText},colorSecondary:{backgroundColor:e.palette.secondary.main,color:e.palette.secondary.contrastText},colorInherit:{color:"inherit"},colorTransparent:{backgroundColor:"transparent",color:"inherit"}}}),{name:"MuiAppBar"})(z),B=n(57),H=Object(B.a)(E.createElement("path",{d:"M12 12c2.21 0 4-1.79 4-4s-1.79-4-4-4-4 1.79-4 4 1.79 4 4 4zm0 2c-2.67 0-8 1.34-8 4v2h16v-2c0-2.66-5.33-4-8-4z"}),"Person");var U=E.forwardRef((function(e,t){var n=e.alt,r=e.children,i=e.classes,o=e.className,a=e.component,s=void 0===a?"div":a,l=e.imgProps,u=e.sizes,c=e.src,d=e.srcSet,h=e.variant,f=void 0===h?"circle":h,p=Object(T.a)(e,["alt","children","classes","className","component","imgProps","sizes","src","srcSet","variant"]),g=null,v=function(e){var t=e.src,n=e.srcSet,r=E.useState(!1),i=r[0],o=r[1];return E.useEffect((function(){if(t||n){o(!1);var e=!0,r=new Image;return r.src=t,r.srcSet=n,r.onload=function(){e&&o("loaded")},r.onerror=function(){e&&o("error")},function(){e=!1}}}),[t,n]),i}({src:c,srcSet:d}),m=c||d,y=m&&"error"!==v;return g=y?E.createElement("img",Object(S.a)({alt:n,src:c,srcSet:d,sizes:u,className:i.img},l)):null!=r?r:m&&n?n[0]:E.createElement(H,{className:i.fallback}),E.createElement(s,Object(S.a)({className:Object(O.a)(i.root,i.system,i[f],o,!y&&i.colorDefault),ref:t},p),g)})),V=Object(k.a)((function(e){return{root:{position:"relative",display:"flex",alignItems:"center",justifyContent:"center",flexShrink:0,width:40,height:40,fontFamily:e.typography.fontFamily,fontSize:e.typography.pxToRem(20),lineHeight:1,borderRadius:"50%",overflow:"hidden",userSelect:"none"},colorDefault:{color:e.palette.background.default,backgroundColor:"light"===e.palette.type?e.palette.grey[400]:e.palette.grey[600]},circle:{},circular:{},rounded:{borderRadius:e.shape.borderRadius},square:{borderRadius:0},img:{width:"100%",height:"100%",textAlign:"center",objectFit:"cover",color:"transparent",textIndent:1e4},fallback:{width:"75%",height:"75%"}}}),{name:"MuiAvatar"})(U),Y=n(2353),W=E.forwardRef((function(e,t){var n=e.anchorOrigin,r=void 0===n?{vertical:"top",horizontal:"right"}:n,i=e.badgeContent,o=e.children,a=e.classes,s=e.className,l=e.color,u=void 0===l?"default":l,c=e.component,d=void 0===c?"span":c,h=e.invisible,f=e.max,p=void 0===f?99:f,g=e.overlap,v=void 0===g?"rectangle":g,m=e.showZero,y=void 0!==m&&m,b=e.variant,C=void 0===b?"standard":b,A=Object(T.a)(e,["anchorOrigin","badgeContent","children","classes","className","color","component","invisible","max","overlap","showZero","variant"]),_=h;null==h&&(0===i&&!y||null==i&&"dot"!==C)&&(_=!0);var I="";return"dot"!==C&&(I=i>p?"".concat(p,"+"):i),E.createElement(d,Object(S.a)({className:Object(O.a)(a.root,s),ref:t},A),o,E.createElement("span",{className:Object(O.a)(a.badge,a["".concat(r.horizontal).concat(Object(j.a)(r.vertical),"}")],a["anchorOrigin".concat(Object(j.a)(r.vertical)).concat(Object(j.a)(r.horizontal)).concat(Object(j.a)(v))],"default"!==u&&a["color".concat(Object(j.a)(u))],_&&a.invisible,"dot"===C&&a.dot)},I))})),G=Object(k.a)((function(e){return{root:{position:"relative",display:"inline-flex",verticalAlign:"middle",flexShrink:0},badge:{display:"flex",flexDirection:"row",flexWrap:"wrap",justifyContent:"center",alignContent:"center",alignItems:"center",position:"absolute",boxSizing:"border-box",fontFamily:e.typography.fontFamily,fontWeight:e.typography.fontWeightMedium,fontSize:e.typography.pxToRem(12),minWidth:20,lineHeight:1,padding:"0 6px",height:20,borderRadius:10,zIndex:1,transition:e.transitions.create("transform",{easing:e.transitions.easing.easeInOut,duration:e.transitions.duration.enteringScreen})},colorPrimary:{backgroundColor:e.palette.primary.main,color:e.palette.primary.contrastText},colorSecondary:{backgroundColor:e.palette.secondary.main,color:e.palette.secondary.contrastText},colorError:{backgroundColor:e.palette.error.main,color:e.palette.error.contrastText},dot:{borderRadius:4,height:8,minWidth:8,padding:0},anchorOriginTopRightRectangle:{top:0,right:0,transform:"scale(1) translate(50%, -50%)",transformOrigin:"100% 0%","&$invisible":{transform:"scale(0) translate(50%, -50%)"}},anchorOriginBottomRightRectangle:{bottom:0,right:0,transform:"scale(1) translate(50%, 50%)",transformOrigin:"100% 100%","&$invisible":{transform:"scale(0) translate(50%, 50%)"}},anchorOriginTopLeftRectangle:{top:0,left:0,transform:"scale(1) translate(-50%, -50%)",transformOrigin:"0% 0%","&$invisible":{transform:"scale(0) translate(-50%, -50%)"}},anchorOriginBottomLeftRectangle:{bottom:0,left:0,transform:"scale(1) translate(-50%, 50%)",transformOrigin:"0% 100%","&$invisible":{transform:"scale(0) translate(-50%, 50%)"}},anchorOriginTopRightCircle:{top:"14%",right:"14%",transform:"scale(1) translate(50%, -50%)",transformOrigin:"100% 0%","&$invisible":{transform:"scale(0) translate(50%, -50%)"}},anchorOriginBottomRightCircle:{bottom:"14%",right:"14%",transform:"scale(1) translate(50%, 50%)",transformOrigin:"100% 100%","&$invisible":{transform:"scale(0) translate(50%, 50%)"}},anchorOriginTopLeftCircle:{top:"14%",left:"14%",transform:"scale(1) translate(-50%, -50%)",transformOrigin:"0% 0%","&$invisible":{transform:"scale(0) translate(-50%, -50%)"}},anchorOriginBottomLeftCircle:{bottom:"14%",left:"14%",transform:"scale(1) translate(-50%, 50%)",transformOrigin:"0% 100%","&$invisible":{transform:"scale(0) translate(-50%, 50%)"}},invisible:{transition:e.transitions.create("transform",{easing:e.transitions.easing.easeInOut,duration:e.transitions.duration.leavingScreen})}}}),{name:"MuiBadge"})(W),X=(n(116),E.forwardRef((function(e,t){var n=e.children,r=e.classes,i=e.className,o=e.component,a=void 0===o?"div":o,s=e.onChange,l=e.showLabels,u=void 0!==l&&l,c=e.value,d=Object(T.a)(e,["children","classes","className","component","onChange","showLabels","value"]);return E.createElement(a,Object(S.a)({className:Object(O.a)(r.root,i),ref:t},d),E.Children.map(n,(function(e,t){if(!E.isValidElement(e))return null;var n=void 0===e.props.value?t:e.props.value;return E.cloneElement(e,{selected:n===c,showLabel:void 0!==e.props.showLabel?e.props.showLabel:u,value:n,onChange:s})})))}))),q=Object(k.a)((function(e){return{root:{display:"flex",justifyContent:"center",height:56,backgroundColor:e.palette.background.paper}}}),{name:"MuiBottomNavigation"})(X),Z=n(231),Q=E.forwardRef((function(e,t){var n=e.classes,r=e.className,i=e.icon,o=e.label,a=e.onChange,s=e.onClick,l=e.selected,u=e.showLabel,c=e.value,d=Object(T.a)(e,["classes","className","icon","label","onChange","onClick","selected","showLabel","value"]);return E.createElement(Z.a,Object(S.a)({ref:t,className:Object(O.a)(n.root,r,l?n.selected:!u&&n.iconOnly),focusRipple:!0,onClick:function(e){a&&a(e,c),s&&s(e)}},d),E.createElement("span",{className:n.wrapper},i,E.createElement("span",{className:Object(O.a)(n.label,l?n.selected:!u&&n.iconOnly)},o)))})),K=Object(k.a)((function(e){return{root:{transition:e.transitions.create(["color","padding-top"],{duration:e.transitions.duration.short}),padding:"6px 12px 8px",minWidth:80,maxWidth:168,color:e.palette.text.secondary,flex:"1","&$iconOnly":{paddingTop:16},"&$selected":{paddingTop:6,color:e.palette.primary.main}},selected:{},iconOnly:{},wrapper:{display:"inline-flex",alignItems:"center",justifyContent:"center",width:"100%",flexDirection:"column"},label:{fontFamily:e.typography.fontFamily,fontSize:e.typography.pxToRem(12),opacity:1,transition:"font-size 0.2s, opacity 0.2s",transitionDelay:"0.1s","&$iconOnly":{opacity:0,transitionDelay:"0s"},"&$selected":{fontSize:e.typography.pxToRem(14)}}}}),{name:"MuiBottomNavigationAction"})(Q),J=n(513),$=n(368),ee=n(74),te=n(187),ne=n(26),re=Object(B.a)(E.createElement("path",{d:"M6 10c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm12 0c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm-6 0c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2z"}),"MoreHoriz");var ie=Object(k.a)((function(e){return{root:{display:"flex",marginLeft:e.spacing(.5),marginRight:e.spacing(.5),backgroundColor:e.palette.grey[100],color:e.palette.grey[700],borderRadius:2,cursor:"pointer","&:hover, &:focus":{backgroundColor:e.palette.grey[200]},"&:active":{boxShadow:e.shadows[0],backgroundColor:Object(ne.c)(e.palette.grey[200],.12)}},icon:{width:24,height:16}}}),{name:"PrivateBreadcrumbCollapsed"})((function(e){var t=e.classes,n=Object(T.a)(e,["classes"]);return E.createElement(Z.a,Object(S.a)({component:"li",className:t.root,focusRipple:!0},n),E.createElement(re,{className:t.icon}))}));var oe=E.forwardRef((function(e,t){var n=e.children,r=e.classes,i=e.className,o=e.component,a=void 0===o?"nav":o,s=e.expandText,l=void 0===s?"Show path":s,u=e.itemsAfterCollapse,c=void 0===u?1:u,d=e.itemsBeforeCollapse,h=void 0===d?1:d,f=e.maxItems,p=void 0===f?8:f,g=e.separator,v=void 0===g?"/":g,m=Object(T.a)(e,["children","classes","className","component","expandText","itemsAfterCollapse","itemsBeforeCollapse","maxItems","separator"]),y=E.useState(!1),b=y[0],C=y[1],A=E.Children.toArray(n).filter((function(e){return E.isValidElement(e)})).map((function(e,t){return E.createElement("li",{className:r.li,key:"child-".concat(t)},e)}));return E.createElement(te.a,Object(S.a)({ref:t,component:a,color:"textSecondary",className:Object(O.a)(r.root,i)},m),E.createElement("ol",{className:r.ol},function(e,t,n){return e.reduce((function(r,i,o){return o<e.length-1?r=r.concat(i,E.createElement("li",{"aria-hidden":!0,key:"separator-".concat(o),className:t},n)):r.push(i),r}),[])}(b||p&&A.length<=p?A:function(e){return h+c>=e.length?e:[].concat(Object(ee.a)(e.slice(0,h)),[E.createElement(ie,{"aria-label":l,key:"ellipsis",onClick:function(e){C(!0);var t=e.currentTarget.parentNode.querySelector("a[href],button,[tabindex]");t&&t.focus()}})],Object(ee.a)(e.slice(e.length-c,e.length)))}(A),r.separator,v)))})),ae=Object(k.a)({root:{},ol:{display:"flex",flexWrap:"wrap",alignItems:"center",padding:0,margin:0,listStyle:"none"},li:{},separator:{display:"flex",userSelect:"none",marginLeft:8,marginRight:8}},{name:"MuiBreadcrumbs"})(oe),se=n(907);se.a.styles;var le=E.forwardRef((function(e,t){var n=e.children,r=e.classes,i=e.className,o=e.color,a=void 0===o?"default":o,s=e.component,l=void 0===s?"div":s,u=e.disabled,c=void 0!==u&&u,d=e.disableElevation,h=void 0!==d&&d,f=e.disableFocusRipple,p=void 0!==f&&f,g=e.disableRipple,v=void 0!==g&&g,m=e.fullWidth,y=void 0!==m&&m,b=e.orientation,C=void 0===b?"horizontal":b,A=e.size,_=void 0===A?"medium":A,I=e.variant,x=void 0===I?"outlined":I,w=Object(T.a)(e,["children","classes","className","color","component","disabled","disableElevation","disableFocusRipple","disableRipple","fullWidth","orientation","size","variant"]),M=Object(O.a)(r.grouped,r["grouped".concat(Object(j.a)(C))],r["grouped".concat(Object(j.a)(x))],r["grouped".concat(Object(j.a)(x)).concat(Object(j.a)(C))],r["grouped".concat(Object(j.a)(x)).concat("default"!==a?Object(j.a)(a):"")],c&&r.disabled);return E.createElement(l,Object(S.a)({role:"group",className:Object(O.a)(r.root,i,y&&r.fullWidth,h&&r.disableElevation,"contained"===x&&r.contained,"vertical"===C&&r.vertical),ref:t},w),E.Children.map(n,(function(e){return E.isValidElement(e)?E.cloneElement(e,{className:Object(O.a)(M,e.props.className),color:e.props.color||a,disabled:e.props.disabled||c,disableElevation:e.props.disableElevation||h,disableFocusRipple:p,disableRipple:v,fullWidth:y,size:e.props.size||_,variant:e.props.variant||x}):null})))})),ue=Object(k.a)((function(e){return{root:{display:"inline-flex",borderRadius:e.shape.borderRadius},contained:{boxShadow:e.shadows[2]},disableElevation:{boxShadow:"none"},disabled:{},fullWidth:{width:"100%"},vertical:{flexDirection:"column"},grouped:{minWidth:40},groupedHorizontal:{"&:not(:first-child)":{borderTopLeftRadius:0,borderBottomLeftRadius:0},"&:not(:last-child)":{borderTopRightRadius:0,borderBottomRightRadius:0}},groupedVertical:{"&:not(:first-child)":{borderTopRightRadius:0,borderTopLeftRadius:0},"&:not(:last-child)":{borderBottomRightRadius:0,borderBottomLeftRadius:0}},groupedText:{},groupedTextHorizontal:{"&:not(:last-child)":{borderRight:"1px solid ".concat("light"===e.palette.type?"rgba(0, 0, 0, 0.23)":"rgba(255, 255, 255, 0.23)")}},groupedTextVertical:{"&:not(:last-child)":{borderBottom:"1px solid ".concat("light"===e.palette.type?"rgba(0, 0, 0, 0.23)":"rgba(255, 255, 255, 0.23)")}},groupedTextPrimary:{"&:not(:last-child)":{borderColor:Object(ne.d)(e.palette.primary.main,.5)}},groupedTextSecondary:{"&:not(:last-child)":{borderColor:Object(ne.d)(e.palette.secondary.main,.5)}},groupedOutlined:{},groupedOutlinedHorizontal:{"&:not(:first-child)":{marginLeft:-1},"&:not(:last-child)":{borderRightColor:"transparent"}},groupedOutlinedVertical:{"&:not(:first-child)":{marginTop:-1},"&:not(:last-child)":{borderBottomColor:"transparent"}},groupedOutlinedPrimary:{"&:hover":{borderColor:e.palette.primary.main}},groupedOutlinedSecondary:{"&:hover":{borderColor:e.palette.secondary.main}},groupedContained:{boxShadow:"none"},groupedContainedHorizontal:{"&:not(:last-child)":{borderRight:"1px solid ".concat(e.palette.grey[400]),"&$disabled":{borderRight:"1px solid ".concat(e.palette.action.disabled)}}},groupedContainedVertical:{"&:not(:last-child)":{borderBottom:"1px solid ".concat(e.palette.grey[400]),"&$disabled":{borderBottom:"1px solid ".concat(e.palette.action.disabled)}}},groupedContainedPrimary:{"&:not(:last-child)":{borderColor:e.palette.primary.dark}},groupedContainedSecondary:{"&:not(:last-child)":{borderColor:e.palette.secondary.dark}}}}),{name:"MuiButtonGroup"})(le),ce=E.forwardRef((function(e,t){var n=e.classes,r=e.className,i=e.raised,o=void 0!==i&&i,a=Object(T.a)(e,["classes","className","raised"]);return E.createElement(R.a,Object(S.a)({className:Object(O.a)(n.root,r),elevation:o?8:1,ref:t},a))})),de=Object(k.a)({root:{overflow:"hidden"}},{name:"MuiCard"})(ce),he=E.forwardRef((function(e,t){var n=e.children,r=e.classes,i=e.className,o=e.focusVisibleClassName,a=Object(T.a)(e,["children","classes","className","focusVisibleClassName"]);return E.createElement(Z.a,Object(S.a)({className:Object(O.a)(r.root,i),focusVisibleClassName:Object(O.a)(o,r.focusVisible),ref:t},a),n,E.createElement("span",{className:r.focusHighlight}))})),fe=Object(k.a)((function(e){return{root:{display:"block",textAlign:"inherit",width:"100%","&:hover $focusHighlight":{opacity:e.palette.action.hoverOpacity},"&$focusVisible $focusHighlight":{opacity:.12}},focusVisible:{},focusHighlight:{overflow:"hidden",pointerEvents:"none",position:"absolute",top:0,right:0,bottom:0,left:0,borderRadius:"inherit",opacity:0,backgroundColor:"currentcolor",transition:e.transitions.create("opacity",{duration:e.transitions.duration.short})}}}),{name:"MuiCardActionArea"})(he),pe=E.forwardRef((function(e,t){var n=e.disableSpacing,r=void 0!==n&&n,i=e.classes,o=e.className,a=Object(T.a)(e,["disableSpacing","classes","className"]);return E.createElement("div",Object(S.a)({className:Object(O.a)(i.root,o,!r&&i.spacing),ref:t},a))})),ge=Object(k.a)({root:{display:"flex",alignItems:"center",padding:8},spacing:{"& > :not(:first-child)":{marginLeft:8}}},{name:"MuiCardActions"})(pe),ve=E.forwardRef((function(e,t){var n=e.classes,r=e.className,i=e.component,o=void 0===i?"div":i,a=Object(T.a)(e,["classes","className","component"]);return E.createElement(o,Object(S.a)({className:Object(O.a)(n.root,r),ref:t},a))})),me=Object(k.a)({root:{padding:16,"&:last-child":{paddingBottom:24}}},{name:"MuiCardContent"})(ve),ye=E.forwardRef((function(e,t){var n=e.action,r=e.avatar,i=e.classes,o=e.className,a=e.component,s=void 0===a?"div":a,l=e.disableTypography,u=void 0!==l&&l,c=e.subheader,d=e.subheaderTypographyProps,h=e.title,f=e.titleTypographyProps,p=Object(T.a)(e,["action","avatar","classes","className","component","disableTypography","subheader","subheaderTypographyProps","title","titleTypographyProps"]),g=h;null==g||g.type===te.a||u||(g=E.createElement(te.a,Object(S.a)({variant:r?"body2":"h5",className:i.title,component:"span",display:"block"},f),g));var v=c;return null==v||v.type===te.a||u||(v=E.createElement(te.a,Object(S.a)({variant:r?"body2":"body1",className:i.subheader,color:"textSecondary",component:"span",display:"block"},d),v)),E.createElement(s,Object(S.a)({className:Object(O.a)(i.root,o),ref:t},p),r&&E.createElement("div",{className:i.avatar},r),E.createElement("div",{className:i.content},g,v),n&&E.createElement("div",{className:i.action},n))})),be=Object(k.a)({root:{display:"flex",alignItems:"center",padding:16},avatar:{flex:"0 0 auto",marginRight:16},action:{flex:"0 0 auto",alignSelf:"flex-start",marginTop:-8,marginRight:-8},content:{flex:"1 1 auto"},title:{},subheader:{}},{name:"MuiCardHeader"})(ye),Ce=["video","audio","picture","iframe","img"],Ae=E.forwardRef((function(e,t){var n=e.children,r=e.classes,i=e.className,o=e.component,a=void 0===o?"div":o,s=e.image,l=e.src,u=e.style,c=Object(T.a)(e,["children","classes","className","component","image","src","style"]),d=-1!==Ce.indexOf(a),h=!d&&s?Object(S.a)({backgroundImage:'url("'.concat(s,'")')},u):u;return E.createElement(a,Object(S.a)({className:Object(O.a)(r.root,i,d&&r.media,-1!=="picture img".indexOf(a)&&r.img),ref:t,style:h,src:d?s||l:void 0},c),n)})),_e=Object(k.a)({root:{display:"block",backgroundSize:"cover",backgroundRepeat:"no-repeat",backgroundPosition:"center"},media:{width:"100%"},img:{objectFit:"cover"}},{name:"MuiCardMedia"})(Ae),Ie=n(1193),xe=n(1196),we=n(908),Me=n(22),Se=n(59),Te=n(37),Ee=n(53);function Oe(e){return e.substring(2).toLowerCase()}var ke=function(e){var t=e.children,n=e.disableReactTree,r=void 0!==n&&n,i=e.mouseEvent,o=void 0===i?"onClick":i,a=e.onClickAway,s=e.touchEvent,l=void 0===s?"onTouchEnd":s,u=E.useRef(!1),c=E.useRef(null),d=E.useRef(!1),h=E.useRef(!1);E.useEffect((function(){return setTimeout((function(){d.current=!0}),0),function(){d.current=!1}}),[]);var f=E.useCallback((function(e){c.current=Me.findDOMNode(e)}),[]),p=Object(Te.a)(t.ref,f),g=Object(Ee.a)((function(e){var t=h.current;if(h.current=!1,d.current&&c.current&&!function(e){return document.documentElement.clientWidth<e.clientX||document.documentElement.clientHeight<e.clientY}(e))if(u.current)u.current=!1;else{var n;if(e.composedPath)n=e.composedPath().indexOf(c.current)>-1;else n=!Object(Se.a)(c.current).documentElement.contains(e.target)||c.current.contains(e.target);n||!r&&t||a(e)}})),v=function(e){return function(n){h.current=!0;var r=t.props[e];r&&r(n)}},m={ref:p};return!1!==l&&(m[l]=v(l)),E.useEffect((function(){if(!1!==l){var e=Oe(l),t=Object(Se.a)(c.current),n=function(){u.current=!0};return t.addEventListener(e,g),t.addEventListener("touchmove",n),function(){t.removeEventListener(e,g),t.removeEventListener("touchmove",n)}}}),[g,l]),!1!==o&&(m[o]=v(o)),E.useEffect((function(){if(!1!==o){var e=Oe(o),t=Object(Se.a)(c.current);return t.addEventListener(e,g),function(){t.removeEventListener(e,g)}}}),[g,o]),E.createElement(E.Fragment,null,E.cloneElement(t,m))},Le=n(2352),De=n(21),Pe=E.forwardRef((function(e,t){var n=e.classes,r=e.className,i=e.component,o=void 0===i?"div":i,a=e.disableGutters,s=void 0!==a&&a,l=e.fixed,u=void 0!==l&&l,c=e.maxWidth,d=void 0===c?"lg":c,h=Object(T.a)(e,["classes","className","component","disableGutters","fixed","maxWidth"]);return E.createElement(o,Object(S.a)({className:Object(O.a)(n.root,r,u&&n.fixed,s&&n.disableGutters,!1!==d&&n["maxWidth".concat(Object(j.a)(String(d)))]),ref:t},h))})),Ne=Object(k.a)((function(e){return{root:Object(De.a)({width:"100%",marginLeft:"auto",boxSizing:"border-box",marginRight:"auto",paddingLeft:e.spacing(2),paddingRight:e.spacing(2),display:"block"},e.breakpoints.up("sm"),{paddingLeft:e.spacing(3),paddingRight:e.spacing(3)}),disableGutters:{paddingLeft:0,paddingRight:0},fixed:Object.keys(e.breakpoints.values).reduce((function(t,n){var r=e.breakpoints.values[n];return 0!==r&&(t[e.breakpoints.up(n)]={maxWidth:r}),t}),{}),maxWidthXs:Object(De.a)({},e.breakpoints.up("xs"),{maxWidth:Math.max(e.breakpoints.values.xs,444)}),maxWidthSm:Object(De.a)({},e.breakpoints.up("sm"),{maxWidth:e.breakpoints.values.sm}),maxWidthMd:Object(De.a)({},e.breakpoints.up("md"),{maxWidth:e.breakpoints.values.md}),maxWidthLg:Object(De.a)({},e.breakpoints.up("lg"),{maxWidth:e.breakpoints.values.lg}),maxWidthXl:Object(De.a)({},e.breakpoints.up("xl"),{maxWidth:e.breakpoints.values.xl})}}),{name:"MuiContainer"})(Pe),je={WebkitFontSmoothing:"antialiased",MozOsxFontSmoothing:"grayscale",boxSizing:"border-box"},Re=function(e){return Object(S.a)({color:e.palette.text.primary},e.typography.body2,{backgroundColor:e.palette.background.default,"@media print":{backgroundColor:e.palette.common.white}})};var ze=Object(k.a)((function(e){return{"@global":{html:je,"*, *::before, *::after":{boxSizing:"inherit"},"strong, b":{fontWeight:e.typography.fontWeightBold},body:Object(S.a)({margin:0},Re(e),{"&::backdrop":{backgroundColor:e.palette.background.default}})}}}),{name:"MuiCssBaseline"})((function(e){var t=e.children,n=void 0===t?null:t;return e.classes,E.createElement(E.Fragment,null,n)})),Fe=n(909),Be=n(910),He=n(911),Ue=E.forwardRef((function(e,t){return E.createElement(te.a,Object(S.a)({component:"p",variant:"body1",color:"textSecondary",ref:t},e))})),Ve=Object(k.a)({root:{marginBottom:12}},{name:"MuiDialogContentText"})(Ue),Ye=n(912),We=E.forwardRef((function(e,t){var n=e.absolute,r=void 0!==n&&n,i=e.classes,o=e.className,a=e.component,s=void 0===a?"hr":a,l=e.flexItem,u=void 0!==l&&l,c=e.light,d=void 0!==c&&c,h=e.orientation,f=void 0===h?"horizontal":h,p=e.role,g=void 0===p?"hr"!==s?"separator":void 0:p,v=e.variant,m=void 0===v?"fullWidth":v,y=Object(T.a)(e,["absolute","classes","className","component","flexItem","light","orientation","role","variant"]);return E.createElement(s,Object(S.a)({className:Object(O.a)(i.root,o,"fullWidth"!==m&&i[m],r&&i.absolute,u&&i.flexItem,d&&i.light,"vertical"===f&&i.vertical),role:g,ref:t},y))})),Ge=Object(k.a)((function(e){return{root:{height:1,margin:0,border:"none",flexShrink:0,backgroundColor:e.palette.divider},absolute:{position:"absolute",bottom:0,left:0,width:"100%"},inset:{marginLeft:72},light:{backgroundColor:Object(ne.d)(e.palette.divider,.08)},middle:{marginLeft:e.spacing(2),marginRight:e.spacing(2)},vertical:{height:"100%",width:1},flexItem:{alignSelf:"stretch",height:"auto"}}}),{name:"MuiDivider"})(We),Xe=n(462),qe=n(105),Ze=n(341),Qe=n(52),Ke=n(65),Je=n(79);function $e(e,t){var n=function(e,t){var n,r=t.getBoundingClientRect();if(t.fakeTransform)n=t.fakeTransform;else{var i=window.getComputedStyle(t);n=i.getPropertyValue("-webkit-transform")||i.getPropertyValue("transform")}var o=0,a=0;if(n&&"none"!==n&&"string"==typeof n){var s=n.split("(")[1].split(")")[0].split(",");o=parseInt(s[4],10),a=parseInt(s[5],10)}return"left"===e?"translateX(".concat(window.innerWidth,"px) translateX(").concat(o-r.left,"px)"):"right"===e?"translateX(-".concat(r.left+r.width-o,"px)"):"up"===e?"translateY(".concat(window.innerHeight,"px) translateY(").concat(a-r.top,"px)"):"translateY(-".concat(r.top+r.height-a,"px)")}(e,t);n&&(t.style.webkitTransform=n,t.style.transform=n)}var et={enter:Ke.b.enteringScreen,exit:Ke.b.leavingScreen},tt=E.forwardRef((function(e,t){var n=e.children,r=e.direction,i=void 0===r?"down":r,o=e.in,a=e.onEnter,s=e.onEntered,l=e.onEntering,u=e.onExit,c=e.onExited,d=e.onExiting,h=e.style,f=e.timeout,p=void 0===f?et:f,g=e.TransitionComponent,v=void 0===g?Ze.a:g,m=Object(T.a)(e,["children","direction","in","onEnter","onEntered","onEntering","onExit","onExited","onExiting","style","timeout","TransitionComponent"]),y=Object(Qe.a)(),b=E.useRef(null),C=E.useCallback((function(e){b.current=Me.findDOMNode(e)}),[]),A=Object(Te.a)(n.ref,C),_=Object(Te.a)(A,t),I=function(e){return function(t){e&&(void 0===t?e(b.current):e(b.current,t))}},x=I((function(e,t){$e(i,e),Object(Je.b)(e),a&&a(e,t)})),w=I((function(e,t){var n=Object(Je.a)({timeout:p,style:h},{mode:"enter"});e.style.webkitTransition=y.transitions.create("-webkit-transform",Object(S.a)({},n,{easing:y.transitions.easing.easeOut})),e.style.transition=y.transitions.create("transform",Object(S.a)({},n,{easing:y.transitions.easing.easeOut})),e.style.webkitTransform="none",e.style.transform="none",l&&l(e,t)})),M=I(s),O=I(d),k=I((function(e){var t=Object(Je.a)({timeout:p,style:h},{mode:"exit"});e.style.webkitTransition=y.transitions.create("-webkit-transform",Object(S.a)({},t,{easing:y.transitions.easing.sharp})),e.style.transition=y.transitions.create("transform",Object(S.a)({},t,{easing:y.transitions.easing.sharp})),$e(i,e),u&&u(e)})),L=I((function(e){e.style.webkitTransition="",e.style.transition="",c&&c(e)})),D=E.useCallback((function(){b.current&&$e(i,b.current)}),[i]);return E.useEffect((function(){if(!o&&"down"!==i&&"right"!==i){var e=Object(qe.a)((function(){b.current&&$e(i,b.current)}));return window.addEventListener("resize",e),function(){e.clear(),window.removeEventListener("resize",e)}}}),[i,o]),E.useEffect((function(){o||D()}),[o,D]),E.createElement(v,Object(S.a)({nodeRef:b,onEnter:x,onEntered:M,onEntering:w,onExit:k,onExited:L,onExiting:O,appear:!0,in:o,timeout:p},m),(function(e,t){return E.cloneElement(n,Object(S.a)({ref:_,style:Object(S.a)({visibility:"exited"!==e||o?void 0:"hidden"},h,n.props.style)},t))}))})),nt={left:"right",right:"left",top:"down",bottom:"up"};function rt(e){return-1!==["left","right"].indexOf(e)}function it(e,t){return"rtl"===e.direction&&rt(t)?nt[t]:t}var ot={enter:Ke.b.enteringScreen,exit:Ke.b.leavingScreen},at=E.forwardRef((function(e,t){var n=e.anchor,r=void 0===n?"left":n,i=e.BackdropProps,o=e.children,a=e.classes,s=e.className,l=e.elevation,u=void 0===l?16:l,c=e.ModalProps,d=(c=void 0===c?{}:c).BackdropProps,h=Object(T.a)(c,["BackdropProps"]),f=e.onClose,p=e.open,g=void 0!==p&&p,v=e.PaperProps,m=void 0===v?{}:v,y=e.SlideProps,b=e.TransitionComponent,C=void 0===b?tt:b,A=e.transitionDuration,_=void 0===A?ot:A,I=e.variant,x=void 0===I?"temporary":I,w=Object(T.a)(e,["anchor","BackdropProps","children","classes","className","elevation","ModalProps","onClose","open","PaperProps","SlideProps","TransitionComponent","transitionDuration","variant"]),M=Object(Qe.a)(),k=E.useRef(!1);E.useEffect((function(){k.current=!0}),[]);var L=it(M,r),D=E.createElement(R.a,Object(S.a)({elevation:"temporary"===x?u:0,square:!0},m,{className:Object(O.a)(a.paper,a["paperAnchor".concat(Object(j.a)(L))],m.className,"temporary"!==x&&a["paperAnchorDocked".concat(Object(j.a)(L))])}),o);if("permanent"===x)return E.createElement("div",Object(S.a)({className:Object(O.a)(a.root,a.docked,s),ref:t},w),D);var P=E.createElement(C,Object(S.a)({in:g,direction:nt[L],timeout:_,appear:k.current},y),D);return"persistent"===x?E.createElement("div",Object(S.a)({className:Object(O.a)(a.root,a.docked,s),ref:t},w),P):E.createElement(Xe.a,Object(S.a)({BackdropProps:Object(S.a)({},i,d,{transitionDuration:_}),BackdropComponent:Y.a,className:Object(O.a)(a.root,a.modal,s),open:g,onClose:f,ref:t},w,h),P)})),st=Object(k.a)((function(e){return{root:{},docked:{flex:"0 0 auto"},paper:{overflowY:"auto",display:"flex",flexDirection:"column",height:"100%",flex:"1 0 auto",zIndex:e.zIndex.drawer,WebkitOverflowScrolling:"touch",position:"fixed",top:0,outline:0},paperAnchorLeft:{left:0,right:"auto"},paperAnchorRight:{left:"auto",right:0},paperAnchorTop:{top:0,left:0,bottom:"auto",right:0,height:"auto",maxHeight:"100%"},paperAnchorBottom:{top:"auto",left:0,bottom:0,right:0,height:"auto",maxHeight:"100%"},paperAnchorDockedLeft:{borderRight:"1px solid ".concat(e.palette.divider)},paperAnchorDockedTop:{borderBottom:"1px solid ".concat(e.palette.divider)},paperAnchorDockedRight:{borderLeft:"1px solid ".concat(e.palette.divider)},paperAnchorDockedBottom:{borderTop:"1px solid ".concat(e.palette.divider)},modal:{}}}),{name:"MuiDrawer",flip:!1})(at),lt=n(450),ut=n(49);var ct=E.createContext({}),dt=n(100),ht=E.forwardRef((function(e,t){var n=e.children,r=e.classes,i=e.className,o=e.defaultExpanded,a=void 0!==o&&o,s=e.disabled,l=void 0!==s&&s,u=e.expanded,c=e.onChange,d=e.square,h=void 0!==d&&d,f=e.TransitionComponent,p=void 0===f?Le.a:f,g=e.TransitionProps,v=Object(T.a)(e,["children","classes","className","defaultExpanded","disabled","expanded","onChange","square","TransitionComponent","TransitionProps"]),m=Object(dt.a)({controlled:u,default:a,name:"ExpansionPanel",state:"expanded"}),y=Object(ut.a)(m,2),b=y[0],C=y[1],A=E.useCallback((function(e){C(!b),c&&c(e,!b)}),[b,c,C]),_=E.Children.toArray(n),I=Object(lt.a)(_),x=I[0],w=I.slice(1),M=E.useMemo((function(){return{expanded:b,disabled:l,toggle:A}}),[b,l,A]);return E.createElement(R.a,Object(S.a)({className:Object(O.a)(r.root,i,b&&r.expanded,l&&r.disabled,!h&&r.rounded),ref:t,square:h},v),E.createElement(ct.Provider,{value:M},x),E.createElement(p,Object(S.a)({in:b,timeout:"auto"},g),E.createElement("div",{"aria-labelledby":x.props.id,id:x.props["aria-controls"],role:"region"},w)))})),ft=Object(k.a)((function(e){var t={duration:e.transitions.duration.shortest};return{root:{position:"relative",transition:e.transitions.create(["margin"],t),"&:before":{position:"absolute",left:0,top:-1,right:0,height:1,content:'""',opacity:1,backgroundColor:e.palette.divider,transition:e.transitions.create(["opacity","background-color"],t)},"&:first-child":{"&:before":{display:"none"}},"&$expanded":{margin:"16px 0","&:first-child":{marginTop:0},"&:last-child":{marginBottom:0},"&:before":{opacity:0}},"&$expanded + &":{"&:before":{display:"none"}},"&$disabled":{backgroundColor:e.palette.action.disabledBackground}},rounded:{borderRadius:0,"&:first-child":{borderTopLeftRadius:e.shape.borderRadius,borderTopRightRadius:e.shape.borderRadius},"&:last-child":{borderBottomLeftRadius:e.shape.borderRadius,borderBottomRightRadius:e.shape.borderRadius,"@supports (-ms-ime-align: auto)":{borderBottomLeftRadius:0,borderBottomRightRadius:0}}},expanded:{},disabled:{}}}),{name:"MuiExpansionPanel"})(ht),pt=E.forwardRef((function(e,t){var n=e.classes,r=e.className,i=e.disableSpacing,o=void 0!==i&&i,a=Object(T.a)(e,["classes","className","disableSpacing"]);return E.createElement("div",Object(S.a)({className:Object(O.a)(n.root,r,!o&&n.spacing),ref:t},a))})),gt=Object(k.a)({root:{display:"flex",alignItems:"center",padding:8,justifyContent:"flex-end"},spacing:{"& > :not(:first-child)":{marginLeft:8}}},{name:"MuiExpansionPanelActions"})(pt),vt=E.forwardRef((function(e,t){var n=e.classes,r=e.className,i=Object(T.a)(e,["classes","className"]);return E.createElement("div",Object(S.a)({className:Object(O.a)(n.root,r),ref:t},i))})),mt=Object(k.a)((function(e){return{root:{display:"flex",padding:e.spacing(1,2,2)}}}),{name:"MuiExpansionPanelDetails"})(vt),yt=n(452),bt=E.forwardRef((function(e,t){var n=e.children,r=e.classes,i=e.className,o=e.expandIcon,a=e.IconButtonProps,s=e.onBlur,l=e.onClick,u=e.onFocusVisible,c=Object(T.a)(e,["children","classes","className","expandIcon","IconButtonProps","onBlur","onClick","onFocusVisible"]),d=E.useState(!1),h=d[0],f=d[1],p=E.useContext(ct),g=p.disabled,v=void 0!==g&&g,m=p.expanded,y=p.toggle;return E.createElement(Z.a,Object(S.a)({focusRipple:!1,disableRipple:!0,disabled:v,component:"div","aria-expanded":m,className:Object(O.a)(r.root,i,v&&r.disabled,m&&r.expanded,h&&r.focused),onFocusVisible:function(e){f(!0),u&&u(e)},onBlur:function(e){f(!1),s&&s(e)},onClick:function(e){y&&y(e),l&&l(e)},ref:t},c),E.createElement("div",{className:Object(O.a)(r.content,m&&r.expanded)},n),o&&E.createElement(yt.a,Object(S.a)({className:Object(O.a)(r.expandIcon,m&&r.expanded),edge:"end",component:"div",tabIndex:null,role:null,"aria-hidden":!0},a),o))})),Ct=Object(k.a)((function(e){var t={duration:e.transitions.duration.shortest};return{root:{display:"flex",minHeight:48,transition:e.transitions.create(["min-height","background-color"],t),padding:e.spacing(0,2),"&:hover:not($disabled)":{cursor:"pointer"},"&$expanded":{minHeight:64},"&$focused":{backgroundColor:e.palette.action.focus},"&$disabled":{opacity:e.palette.action.disabledOpacity}},expanded:{},focused:{},disabled:{},content:{display:"flex",flexGrow:1,transition:e.transitions.create(["margin"],t),margin:"12px 0","&$expanded":{margin:"20px 0"}},expandIcon:{transform:"rotate(0deg)",transition:e.transitions.create("transform",t),"&:hover":{backgroundColor:"transparent"},"&$expanded":{transform:"rotate(180deg)"}}}}),{name:"MuiExpansionPanelSummary"})(bt),At=n(913),_t=n(1310),It=n(2355),xt=n(630),wt=n(103),Mt=n(914),St=n(915),Tt=n(2357),Et=n(631),Ot=n(916),kt=E.forwardRef((function(e,t){var n=e.cellHeight,r=void 0===n?180:n,i=e.children,o=e.classes,a=e.className,s=e.cols,l=void 0===s?2:s,u=e.component,c=void 0===u?"ul":u,d=e.spacing,h=void 0===d?4:d,f=e.style,p=Object(T.a)(e,["cellHeight","children","classes","className","cols","component","spacing","style"]);return E.createElement(c,Object(S.a)({className:Object(O.a)(o.root,a),ref:t,style:Object(S.a)({margin:-h/2},f)},p),E.Children.map(i,(function(e){if(!E.isValidElement(e))return null;var t=e.props.cols||1,n=e.props.rows||1;return E.cloneElement(e,{style:Object(S.a)({width:"".concat(100/l*t,"%"),height:"auto"===r?"auto":r*n+h,padding:h/2},e.props.style)})})))})),Lt=Object(k.a)({root:{display:"flex",flexWrap:"wrap",overflowY:"auto",listStyle:"none",padding:0,WebkitOverflowScrolling:"touch"}},{name:"MuiGridList"})(kt),Dt=n(141),Pt=function(e,t){var n,r,i,o;e&&e.complete&&(e.width/e.height>e.parentElement.offsetWidth/e.parentElement.offsetHeight?((n=e.classList).remove.apply(n,Object(ee.a)(t.imgFullWidth.split(" "))),(r=e.classList).add.apply(r,Object(ee.a)(t.imgFullHeight.split(" ")))):((i=e.classList).remove.apply(i,Object(ee.a)(t.imgFullHeight.split(" "))),(o=e.classList).add.apply(o,Object(ee.a)(t.imgFullWidth.split(" ")))))};var Nt=E.forwardRef((function(e,t){var n=e.children,r=e.classes,i=e.className,o=(e.cols,e.component),a=void 0===o?"li":o,s=(e.rows,Object(T.a)(e,["children","classes","className","cols","component","rows"])),l=E.useRef(null);return E.useEffect((function(){!function(e,t){e&&(e.complete?Pt(e,t):e.addEventListener("load",(function(){Pt(e,t)})))}(l.current,r)})),E.useEffect((function(){var e=Object(qe.a)((function(){Pt(l.current,r)}));return window.addEventListener("resize",e),function(){e.clear(),window.removeEventListener("resize",e)}}),[r]),E.createElement(a,Object(S.a)({className:Object(O.a)(r.root,i),ref:t},s),E.createElement("div",{className:r.tile},E.Children.map(n,(function(e){return E.isValidElement(e)?"img"===e.type||Object(Dt.a)(e,["Image"])?E.cloneElement(e,{ref:l}):e:null}))))})),jt=Object(k.a)({root:{boxSizing:"border-box",flexShrink:0},tile:{position:"relative",display:"block",height:"100%",overflow:"hidden"},imgFullHeight:{height:"100%",transform:"translateX(-50%)",position:"relative",left:"50%"},imgFullWidth:{width:"100%",position:"relative",transform:"translateY(-50%)",top:"50%"}},{name:"MuiGridListTile"})(Nt),Rt=E.forwardRef((function(e,t){var n=e.actionIcon,r=e.actionPosition,i=void 0===r?"right":r,o=e.classes,a=e.className,s=e.subtitle,l=e.title,u=e.titlePosition,c=void 0===u?"bottom":u,d=Object(T.a)(e,["actionIcon","actionPosition","classes","className","subtitle","title","titlePosition"]),h=n&&i;return E.createElement("div",Object(S.a)({className:Object(O.a)(o.root,a,"top"===c?o.titlePositionTop:o.titlePositionBottom,s&&o.rootSubtitle),ref:t},d),E.createElement("div",{className:Object(O.a)(o.titleWrap,{left:o.titleWrapActionPosLeft,right:o.titleWrapActionPosRight}[h])},E.createElement("div",{className:o.title},l),s?E.createElement("div",{className:o.subtitle},s):null),n?E.createElement("div",{className:Object(O.a)(o.actionIcon,"left"===h&&o.actionIconActionPosLeft)},n):null)})),zt=Object(k.a)((function(e){return{root:{position:"absolute",left:0,right:0,height:48,background:"rgba(0, 0, 0, 0.5)",display:"flex",alignItems:"center",fontFamily:e.typography.fontFamily},titlePositionBottom:{bottom:0},titlePositionTop:{top:0},rootSubtitle:{height:68},titleWrap:{flexGrow:1,marginLeft:16,marginRight:16,color:e.palette.common.white,overflow:"hidden"},titleWrapActionPosLeft:{marginLeft:0},titleWrapActionPosRight:{marginRight:0},title:{fontSize:e.typography.pxToRem(16),lineHeight:"24px",textOverflow:"ellipsis",overflow:"hidden",whiteSpace:"nowrap"},subtitle:{fontSize:e.typography.pxToRem(12),lineHeight:1,textOverflow:"ellipsis",overflow:"hidden",whiteSpace:"nowrap"},actionIcon:{},actionIconActionPosLeft:{order:-1}}}),{name:"MuiGridListTileBar"})(Rt),Ft=n(1312),Bt=n(1194),Ht=n(917),Ut=n(454),Vt=n(918),Yt=n(1311),Wt=n(632),Gt=n(919),Xt=n(166),qt=E.forwardRef((function(e,t){var n=e.classes,r=e.className,i=e.color,o=void 0===i?"primary":i,a=e.component,s=void 0===a?"a":a,l=e.onBlur,u=e.onFocus,c=e.TypographyClasses,d=e.underline,h=void 0===d?"hover":d,f=e.variant,p=void 0===f?"inherit":f,g=Object(T.a)(e,["classes","className","color","component","onBlur","onFocus","TypographyClasses","underline","variant"]),v=Object(Xt.a)(),m=v.isFocusVisible,y=v.onBlurVisible,b=v.ref,C=E.useState(!1),A=C[0],_=C[1],I=Object(Te.a)(t,b);return E.createElement(te.a,Object(S.a)({className:Object(O.a)(n.root,n["underline".concat(Object(j.a)(h))],r,A&&n.focusVisible,"button"===s&&n.button),classes:c,color:o,component:s,onBlur:function(e){A&&(y(),_(!1)),l&&l(e)},onFocus:function(e){m(e)&&_(!0),u&&u(e)},ref:I,variant:p},g))})),Zt=Object(k.a)({root:{},underlineNone:{textDecoration:"none"},underlineHover:{textDecoration:"none","&:hover":{textDecoration:"underline"}},underlineAlways:{textDecoration:"underline"},button:{position:"relative",WebkitTapHighlightColor:"transparent",backgroundColor:"transparent",outline:0,border:0,margin:0,borderRadius:0,padding:0,cursor:"pointer",userSelect:"none",verticalAlign:"middle","-moz-appearance":"none","-webkit-appearance":"none","&::-moz-focus-inner":{borderStyle:"none"},"&$focusVisible":{outline:"auto"}},focusVisible:{}},{name:"MuiLink"})(qt),Qt=n(2359),Kt=n(2360),Jt=n(143),$t=E.forwardRef((function(e,t){var n=e.classes,r=e.className,i=Object(T.a)(e,["classes","className"]),o=E.useContext(Jt.a);return E.createElement("div",Object(S.a)({className:Object(O.a)(n.root,r,"flex-start"===o.alignItems&&n.alignItemsFlexStart),ref:t},i))})),en=Object(k.a)({root:{minWidth:56,flexShrink:0},alignItemsFlexStart:{marginTop:8}},{name:"MuiListItemAvatar"})($t),tn=E.forwardRef((function(e,t){var n=e.classes,r=e.className,i=Object(T.a)(e,["classes","className"]),o=E.useContext(Jt.a);return E.createElement("div",Object(S.a)({className:Object(O.a)(n.root,r,"flex-start"===o.alignItems&&n.alignItemsFlexStart),ref:t},i))})),nn=Object(k.a)((function(e){return{root:{minWidth:56,color:e.palette.action.active,flexShrink:0,display:"inline-flex"},alignItemsFlexStart:{marginTop:8}}}),{name:"MuiListItemIcon"})(tn),rn=E.forwardRef((function(e,t){var n=e.classes,r=e.className,i=Object(T.a)(e,["classes","className"]);return E.createElement("div",Object(S.a)({className:Object(O.a)(n.root,r),ref:t},i))}));rn.muiName="ListItemSecondaryAction";var on=Object(k.a)({root:{position:"absolute",right:16,top:"50%",transform:"translateY(-50%)"}},{name:"MuiListItemSecondaryAction"})(rn),an=n(920),sn=n(2377),ln=n(436),un=n(921),cn=n(2361),dn=E.forwardRef((function(e,t){var n=e.activeStep,r=void 0===n?0:n,i=e.backButton,o=e.classes,a=e.className,s=e.LinearProgressProps,l=e.nextButton,u=e.position,c=void 0===u?"bottom":u,d=e.steps,h=e.variant,f=void 0===h?"dots":h,p=Object(T.a)(e,["activeStep","backButton","classes","className","LinearProgressProps","nextButton","position","steps","variant"]);return E.createElement(R.a,Object(S.a)({square:!0,elevation:0,className:Object(O.a)(o.root,o["position".concat(Object(j.a)(c))],a),ref:t},p),i,"text"===f&&E.createElement(E.Fragment,null,r+1," / ",d),"dots"===f&&E.createElement("div",{className:o.dots},Object(ee.a)(new Array(d)).map((function(e,t){return E.createElement("div",{key:t,className:Object(O.a)(o.dot,t===r&&o.dotActive)})}))),"progress"===f&&E.createElement(Gt.a,Object(S.a)({className:o.progress,variant:"determinate",value:Math.ceil(r/(d-1)*100)},s)),l)})),hn=Object(k.a)((function(e){return{root:{display:"flex",flexDirection:"row",justifyContent:"space-between",alignItems:"center",background:e.palette.background.default,padding:8},positionBottom:{position:"fixed",bottom:0,left:0,right:0,zIndex:e.zIndex.mobileStepper},positionTop:{position:"fixed",top:0,left:0,right:0,zIndex:e.zIndex.mobileStepper},positionStatic:{},dots:{display:"flex",flexDirection:"row"},dot:{backgroundColor:e.palette.action.disabled,borderRadius:"50%",width:8,height:8,margin:"0 2px"},dotActive:{backgroundColor:e.palette.primary.main},progress:{width:"50%"}}}),{name:"MuiMobileStepper"})(dn),fn=n(242),pn=n(1286),gn="undefined"!=typeof window?E.useLayoutEffect:E.useEffect;var vn=function(e){var t=e.children,n=e.defer,r=void 0!==n&&n,i=e.fallback,o=void 0===i?null:i,a=E.useState(!1),s=a[0],l=a[1];return gn((function(){r||l(!0)}),[r]),E.useEffect((function(){r&&l(!0)}),[r]),E.createElement(E.Fragment,null,s?t:o)},mn=n(2363),yn=n(633),bn=n(2362),Cn=n(2354),An=n(274),_n=Object(B.a)(E.createElement("path",{d:"M12 2C6.48 2 2 6.48 2 12s4.48 10 10 10 10-4.48 10-10S17.52 2 12 2zm0 18c-4.42 0-8-3.58-8-8s3.58-8 8-8 8 3.58 8 8-3.58 8-8 8z"}),"RadioButtonUnchecked"),In=Object(B.a)(E.createElement("path",{d:"M8.465 8.465C9.37 7.56 10.62 7 12 7C14.76 7 17 9.24 17 12C17 13.38 16.44 14.63 15.535 15.535C14.63 16.44 13.38 17 12 17C9.24 17 7 14.76 7 12C7 10.62 7.56 9.37 8.465 8.465Z"}),"RadioButtonChecked");var xn=Object(k.a)((function(e){return{root:{position:"relative",display:"flex","&$checked $layer":{transform:"scale(1)",transition:e.transitions.create("transform",{easing:e.transitions.easing.easeOut,duration:e.transitions.duration.shortest})}},layer:{left:0,position:"absolute",transform:"scale(0)",transition:e.transitions.create("transform",{easing:e.transitions.easing.easeIn,duration:e.transitions.duration.shortest})},checked:{}}}),{name:"PrivateRadioButtonIcon"})((function(e){var t=e.checked,n=e.classes,r=e.fontSize;return E.createElement("div",{className:Object(O.a)(n.root,t&&n.checked)},E.createElement(_n,{fontSize:r}),E.createElement(In,{fontSize:r,className:n.layer}))})),wn=n(107);var Mn=E.createContext();function Sn(){return E.useContext(Mn)}var Tn=E.createElement(xn,{checked:!0}),En=E.createElement(xn,null),On=E.forwardRef((function(e,t){var n=e.checked,r=e.classes,i=e.color,o=void 0===i?"secondary":i,a=e.name,s=e.onChange,l=e.size,u=void 0===l?"medium":l,c=Object(T.a)(e,["checked","classes","color","name","onChange","size"]),d=Sn(),h=n,f=Object(wn.a)(s,d&&d.onChange),p=a;return d&&(void 0===h&&(h=d.value===e.value),void 0===p&&(p=d.name)),E.createElement(An.a,Object(S.a)({color:o,type:"radio",icon:E.cloneElement(En,{fontSize:"small"===u?"small":"default"}),checkedIcon:E.cloneElement(Tn,{fontSize:"small"===u?"small":"default"}),classes:{root:Object(O.a)(r.root,r["color".concat(Object(j.a)(o))]),checked:r.checked,disabled:r.disabled},name:p,checked:h,onChange:f,ref:t},c))})),kn=Object(k.a)((function(e){return{root:{color:e.palette.text.secondary},checked:{},disabled:{},colorPrimary:{"&$checked":{color:e.palette.primary.main,"&:hover":{backgroundColor:Object(ne.d)(e.palette.primary.main,e.palette.action.hoverOpacity),"@media (hover: none)":{backgroundColor:"transparent"}}},"&$disabled":{color:e.palette.action.disabled}},colorSecondary:{"&$checked":{color:e.palette.secondary.main,"&:hover":{backgroundColor:Object(ne.d)(e.palette.secondary.main,e.palette.action.hoverOpacity),"@media (hover: none)":{backgroundColor:"transparent"}}},"&$disabled":{color:e.palette.action.disabled}}}}),{name:"MuiRadio"})(On),Ln=n(186),Dn=E.forwardRef((function(e,t){var n=e.actions,r=e.children,i=e.name,o=e.value,a=e.onChange,s=Object(T.a)(e,["actions","children","name","value","onChange"]),l=E.useRef(null),u=Object(dt.a)({controlled:o,default:e.defaultValue,name:"RadioGroup"}),c=Object(ut.a)(u,2),d=c[0],h=c[1];E.useImperativeHandle(n,(function(){return{focus:function(){var e=l.current.querySelector("input:not(:disabled):checked");e||(e=l.current.querySelector("input:not(:disabled)")),e&&e.focus()}}}),[]);var f=Object(Te.a)(t,l),p=Object(Ln.a)(i);return E.createElement(Mn.Provider,{value:{name:p,onChange:function(e){h(e.target.value),a&&a(e,e.target.value)},value:d}},E.createElement(St.a,Object(S.a)({role:"radiogroup",ref:f},s),r))})),Pn=n(99),Nn=n(80),jn=n(138),Rn=n(139),zn=n(127),Fn=n(87);function Bn(e){var t=function(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],(function(){}))),!0}catch(e){return!1}}();return function(){var n,r=Object(zn.a)(e);if(t){var i=Object(zn.a)(this).constructor;n=Reflect.construct(r,arguments,i)}else n=r.apply(this,arguments);return Object(Rn.a)(this,n)}}var Hn=function(e){Object(jn.a)(n,e);var t=Bn(n);function n(){return Object(Pn.a)(this,n),t.apply(this,arguments)}return Object(Nn.a)(n,[{key:"componentDidMount",value:function(){this.ref=Me.findDOMNode(this),Object(Fn.a)(this.props.rootRef,this.ref)}},{key:"componentDidUpdate",value:function(e){var t=Me.findDOMNode(this);e.rootRef===this.props.rootRef&&this.ref===t||(e.rootRef!==this.props.rootRef&&Object(Fn.a)(e.rootRef,null),this.ref=t,Object(Fn.a)(this.props.rootRef,this.ref))}},{key:"componentWillUnmount",value:function(){this.ref=null,Object(Fn.a)(this.props.rootRef,null)}},{key:"render",value:function(){return this.props.children}}]),n}(E.Component),Un=n(639),Vn=n(1197),Yn=E.forwardRef((function(e,t){var n=e.action,r=e.classes,i=e.className,o=e.message,a=e.role,s=void 0===a?"alert":a,l=Object(T.a)(e,["action","classes","className","message","role"]);return E.createElement(R.a,Object(S.a)({role:s,square:!0,elevation:6,className:Object(O.a)(r.root,i),ref:t},l),E.createElement("div",{className:r.message},o),n?E.createElement("div",{className:r.action},n):null)})),Wn=Object(k.a)((function(e){var t="light"===e.palette.type?.8:.98,n=Object(ne.c)(e.palette.background.default,t);return{root:Object(S.a)({},e.typography.body2,Object(De.a)({color:e.palette.getContrastText(n),backgroundColor:n,display:"flex",alignItems:"center",flexWrap:"wrap",padding:"6px 16px",borderRadius:e.shape.borderRadius,flexGrow:1},e.breakpoints.up("sm"),{flexGrow:"initial",minWidth:288})),message:{padding:"8px 0"},action:{display:"flex",alignItems:"center",marginLeft:"auto",paddingLeft:16,marginRight:-8}}}),{name:"MuiSnackbarContent"})(Yn),Gn=E.forwardRef((function(e,t){var n=e.action,r=e.anchorOrigin,i=(r=void 0===r?{vertical:"bottom",horizontal:"center"}:r).vertical,o=r.horizontal,a=e.autoHideDuration,s=void 0===a?null:a,l=e.children,u=e.classes,c=e.className,d=e.ClickAwayListenerProps,h=e.ContentProps,f=e.disableWindowBlurListener,p=void 0!==f&&f,g=e.message,v=e.onClose,m=e.onEnter,y=e.onEntered,b=e.onEntering,C=e.onExit,A=e.onExited,_=e.onExiting,I=e.onMouseEnter,x=e.onMouseLeave,w=e.open,M=e.resumeHideDuration,k=e.TransitionComponent,L=void 0===k?Ft.a:k,D=e.transitionDuration,P=void 0===D?{enter:Ke.b.enteringScreen,exit:Ke.b.leavingScreen}:D,N=e.TransitionProps,R=Object(T.a)(e,["action","anchorOrigin","autoHideDuration","children","classes","className","ClickAwayListenerProps","ContentProps","disableWindowBlurListener","message","onClose","onEnter","onEntered","onEntering","onExit","onExited","onExiting","onMouseEnter","onMouseLeave","open","resumeHideDuration","TransitionComponent","transitionDuration","TransitionProps"]),z=E.useRef(),F=E.useState(!0),B=F[0],H=F[1],U=Object(Ee.a)((function(){v&&v.apply(void 0,arguments)})),V=Object(Ee.a)((function(e){v&&null!=e&&(clearTimeout(z.current),z.current=setTimeout((function(){U(null,"timeout")}),e))}));E.useEffect((function(){return w&&V(s),function(){clearTimeout(z.current)}}),[w,s,V]);var Y=function(){clearTimeout(z.current)},W=E.useCallback((function(){null!=s&&V(null!=M?M:.5*s)}),[s,M,V]);return E.useEffect((function(){if(!p&&w)return window.addEventListener("focus",W),window.addEventListener("blur",Y),function(){window.removeEventListener("focus",W),window.removeEventListener("blur",Y)}}),[p,W,w]),!w&&B?null:E.createElement(ke,Object(S.a)({onClickAway:function(e){v&&v(e,"clickaway")}},d),E.createElement("div",Object(S.a)({className:Object(O.a)(u.root,u["anchorOrigin".concat(Object(j.a)(i)).concat(Object(j.a)(o))],c),onMouseEnter:function(e){I&&I(e),Y()},onMouseLeave:function(e){x&&x(e),W()},ref:t},R),E.createElement(L,Object(S.a)({appear:!0,in:w,onEnter:Object(wn.a)((function(){H(!1)}),m),onEntered:y,onEntering:b,onExit:C,onExited:Object(wn.a)((function(){H(!0)}),A),onExiting:_,timeout:P,direction:"top"===i?"down":"up"},N),l||E.createElement(Wn,Object(S.a)({message:g,action:n},h)))))})),Xn=Object(k.a)((function(e){var t={top:8},n={bottom:8},r={justifyContent:"flex-end"},i={justifyContent:"flex-start"},o={top:24},a={bottom:24},s={right:24},l={left:24},u={left:"50%",right:"auto",transform:"translateX(-50%)"};return{root:{zIndex:e.zIndex.snackbar,position:"fixed",display:"flex",left:8,right:8,justifyContent:"center",alignItems:"center"},anchorOriginTopCenter:Object(S.a)({},t,Object(De.a)({},e.breakpoints.up("sm"),Object(S.a)({},o,u))),anchorOriginBottomCenter:Object(S.a)({},n,Object(De.a)({},e.breakpoints.up("sm"),Object(S.a)({},a,u))),anchorOriginTopRight:Object(S.a)({},t,r,Object(De.a)({},e.breakpoints.up("sm"),Object(S.a)({left:"auto"},o,s))),anchorOriginBottomRight:Object(S.a)({},n,r,Object(De.a)({},e.breakpoints.up("sm"),Object(S.a)({left:"auto"},a,s))),anchorOriginTopLeft:Object(S.a)({},t,i,Object(De.a)({},e.breakpoints.up("sm"),Object(S.a)({right:"auto"},o,l))),anchorOriginBottomLeft:Object(S.a)({},n,i,Object(De.a)({},e.breakpoints.up("sm"),Object(S.a)({right:"auto"},a,l)))}}),{flip:!1,name:"MuiSnackbar"})(Gn),qn=E.forwardRef((function(e,t){var n=e.active,r=void 0!==n&&n,i=e.alternativeLabel,o=e.children,a=e.classes,s=e.className,l=e.completed,u=void 0!==l&&l,c=e.connector,d=e.disabled,h=void 0!==d&&d,f=e.expanded,p=void 0!==f&&f,g=e.index,v=e.last,m=e.orientation,y=Object(T.a)(e,["active","alternativeLabel","children","classes","className","completed","connector","disabled","expanded","index","last","orientation"]),b=c?E.cloneElement(c,{orientation:m,alternativeLabel:i,index:g,active:r,completed:u,disabled:h}):null,C=E.createElement("div",Object(S.a)({className:Object(O.a)(a.root,a[m],s,i&&a.alternativeLabel,u&&a.completed),ref:t},y),b&&i&&0!==g?b:null,E.Children.map(o,(function(e){return E.isValidElement(e)?E.cloneElement(e,Object(S.a)({active:r,alternativeLabel:i,completed:u,disabled:h,expanded:p,last:v,icon:g+1,orientation:m},e.props)):null})));return b&&!i&&0!==g?E.createElement(E.Fragment,null,b,C):C})),Zn=Object(k.a)({root:{},horizontal:{paddingLeft:8,paddingRight:8},vertical:{},alternativeLabel:{flex:1,position:"relative"},completed:{}},{name:"MuiStep"})(qn),Qn=Object(B.a)(E.createElement("path",{d:"M12 0a12 12 0 1 0 0 24 12 12 0 0 0 0-24zm-2 17l-5-5 1.4-1.4 3.6 3.6 7.6-7.6L19 8l-9 9z"}),"CheckCircle"),Kn=Object(B.a)(E.createElement("path",{d:"M1 21h22L12 2 1 21zm12-3h-2v-2h2v2zm0-4h-2v-4h2v4z"}),"Warning"),Jn=n(1309),$n=E.createElement("circle",{cx:"12",cy:"12",r:"12"}),er=E.forwardRef((function(e,t){var n=e.completed,r=void 0!==n&&n,i=e.icon,o=e.active,a=void 0!==o&&o,s=e.error,l=void 0!==s&&s,u=e.classes;if("number"==typeof i||"string"==typeof i){var c=Object(O.a)(u.root,a&&u.active,l&&u.error,r&&u.completed);return l?E.createElement(Kn,{className:c,ref:t}):r?E.createElement(Qn,{className:c,ref:t}):E.createElement(Jn.a,{className:c,ref:t},$n,E.createElement("text",{className:u.text,x:"12",y:"16",textAnchor:"middle"},i))}return i})),tr=Object(k.a)((function(e){return{root:{display:"block",color:e.palette.text.disabled,"&$completed":{color:e.palette.primary.main},"&$active":{color:e.palette.primary.main},"&$error":{color:e.palette.error.main}},text:{fill:e.palette.primary.contrastText,fontSize:e.typography.caption.fontSize,fontFamily:e.typography.fontFamily},active:{},completed:{},error:{}}}),{name:"MuiStepIcon"})(er),nr=E.forwardRef((function(e,t){var n=e.active,r=void 0!==n&&n,i=e.alternativeLabel,o=void 0!==i&&i,a=e.children,s=e.classes,l=e.className,u=e.completed,c=void 0!==u&&u,d=e.disabled,h=void 0!==d&&d,f=e.error,p=void 0!==f&&f,g=(e.expanded,e.icon),v=(e.last,e.optional),m=e.orientation,y=void 0===m?"horizontal":m,b=e.StepIconComponent,C=e.StepIconProps,A=Object(T.a)(e,["active","alternativeLabel","children","classes","className","completed","disabled","error","expanded","icon","last","optional","orientation","StepIconComponent","StepIconProps"]),_=b;return g&&!_&&(_=tr),E.createElement("span",Object(S.a)({className:Object(O.a)(s.root,s[y],l,h&&s.disabled,o&&s.alternativeLabel,p&&s.error),ref:t},A),g||_?E.createElement("span",{className:Object(O.a)(s.iconContainer,o&&s.alternativeLabel)},E.createElement(_,Object(S.a)({completed:c,active:r,error:p,icon:g},C))):null,E.createElement("span",{className:s.labelContainer},a?E.createElement(te.a,{variant:"body2",component:"span",display:"block",className:Object(O.a)(s.label,o&&s.alternativeLabel,c&&s.completed,r&&s.active,p&&s.error)},a):null,v))}));nr.muiName="StepLabel";var rr=Object(k.a)((function(e){return{root:{display:"flex",alignItems:"center","&$alternativeLabel":{flexDirection:"column"},"&$disabled":{cursor:"default"}},horizontal:{},vertical:{},label:{color:e.palette.text.secondary,"&$active":{color:e.palette.text.primary,fontWeight:500},"&$completed":{color:e.palette.text.primary,fontWeight:500},"&$alternativeLabel":{textAlign:"center",marginTop:16},"&$error":{color:e.palette.error.main}},active:{},completed:{},error:{},disabled:{},iconContainer:{flexShrink:0,display:"flex",paddingRight:8,"&$alternativeLabel":{paddingRight:0}},alternativeLabel:{},labelContainer:{width:"100%"}}}),{name:"MuiStepLabel"})(nr),ir=E.forwardRef((function(e,t){var n=e.active,r=e.alternativeLabel,i=e.children,o=e.classes,a=e.className,s=e.completed,l=e.disabled,u=(e.expanded,e.icon),c=(e.last,e.optional),d=e.orientation,h=Object(T.a)(e,["active","alternativeLabel","children","classes","className","completed","disabled","expanded","icon","last","optional","orientation"]),f={active:n,alternativeLabel:r,completed:s,disabled:l,icon:u,optional:c,orientation:d},p=Object(Dt.a)(i,["StepLabel"])?E.cloneElement(i,f):E.createElement(rr,f,i);return E.createElement(Z.a,Object(S.a)({focusRipple:!0,disabled:l,TouchRippleProps:{className:o.touchRipple},className:Object(O.a)(o.root,o[d],a),ref:t},h),p)})),or=Object(k.a)({root:{width:"100%",padding:"24px 16px",margin:"-24px -16px",boxSizing:"content-box"},horizontal:{},vertical:{justifyContent:"flex-start",padding:"8px",margin:"-8px"},touchRipple:{color:"rgba(0, 0, 0, 0.3)"}},{name:"MuiStepButton"})(ir),ar=E.forwardRef((function(e,t){var n=e.active,r=e.alternativeLabel,i=void 0!==r&&r,o=e.classes,a=e.className,s=e.completed,l=e.disabled,u=(e.index,e.orientation),c=void 0===u?"horizontal":u,d=Object(T.a)(e,["active","alternativeLabel","classes","className","completed","disabled","index","orientation"]);return E.createElement("div",Object(S.a)({className:Object(O.a)(o.root,o[c],a,i&&o.alternativeLabel,n&&o.active,s&&o.completed,l&&o.disabled),ref:t},d),E.createElement("span",{className:Object(O.a)(o.line,{horizontal:o.lineHorizontal,vertical:o.lineVertical}[c])}))})),sr=Object(k.a)((function(e){return{root:{flex:"1 1 auto"},horizontal:{},vertical:{marginLeft:12,padding:"0 0 8px"},alternativeLabel:{position:"absolute",top:12,left:"calc(-50% + 20px)",right:"calc(50% + 20px)"},active:{},completed:{},disabled:{},line:{display:"block",borderColor:"light"===e.palette.type?e.palette.grey[400]:e.palette.grey[600]},lineHorizontal:{borderTopStyle:"solid",borderTopWidth:1},lineVertical:{borderLeftStyle:"solid",borderLeftWidth:1,minHeight:24}}}),{name:"MuiStepConnector"})(ar),lr=E.forwardRef((function(e,t){var n=e.active,r=(e.alternativeLabel,e.children),i=e.classes,o=e.className,a=(e.completed,e.expanded),s=e.last,l=(e.optional,e.orientation,e.TransitionComponent),u=void 0===l?Le.a:l,c=e.transitionDuration,d=void 0===c?"auto":c,h=e.TransitionProps,f=Object(T.a)(e,["active","alternativeLabel","children","classes","className","completed","expanded","last","optional","orientation","TransitionComponent","transitionDuration","TransitionProps"]);var p=d;return"auto"!==d||u.muiSupportAuto||(p=void 0),E.createElement("div",Object(S.a)({className:Object(O.a)(i.root,o,s&&i.last),ref:t},f),E.createElement(u,Object(S.a)({in:n||a,className:i.transition,timeout:p,unmountOnExit:!0},h),r))})),ur=Object(k.a)((function(e){return{root:{marginTop:8,marginLeft:12,paddingLeft:20,paddingRight:8,borderLeft:"1px solid ".concat("light"===e.palette.type?e.palette.grey[400]:e.palette.grey[600])},last:{borderLeft:"none"},transition:{}}}),{name:"MuiStepContent"})(lr),cr=E.createElement(sr,null),dr=E.forwardRef((function(e,t){var n=e.activeStep,r=void 0===n?0:n,i=e.alternativeLabel,o=void 0!==i&&i,a=e.children,s=e.classes,l=e.className,u=e.connector,c=void 0===u?cr:u,d=e.nonLinear,h=void 0!==d&&d,f=e.orientation,p=void 0===f?"horizontal":f,g=Object(T.a)(e,["activeStep","alternativeLabel","children","classes","className","connector","nonLinear","orientation"]),v=E.isValidElement(c)?E.cloneElement(c,{orientation:p}):null,m=E.Children.toArray(a),y=m.map((function(e,t){var n={index:t,active:!1,completed:!1,disabled:!1};return r===t?n.active=!0:!h&&r>t?n.completed=!0:!h&&r<t&&(n.disabled=!0),E.cloneElement(e,Object(S.a)({alternativeLabel:o,connector:v,last:t+1===m.length,orientation:p},n,e.props))}));return E.createElement(R.a,Object(S.a)({square:!0,elevation:0,className:Object(O.a)(s.root,s[p],l,o&&s.alternativeLabel),ref:t},g),y)})),hr=Object(k.a)({root:{display:"flex",padding:24},horizontal:{flexDirection:"row",alignItems:"center"},vertical:{flexDirection:"column"},alternativeLabel:{alignItems:"flex-start"}},{name:"MuiStepper"})(dr),fr=n(653),pr=E.forwardRef((function(e,t){var n=e.anchor,r=e.classes,i=e.className,o=e.width,a=Object(T.a)(e,["anchor","classes","className","width"]);return E.createElement("div",Object(S.a)({className:Object(O.a)(r.root,r["anchor".concat(Object(j.a)(n))],i),ref:t,style:Object(De.a)({},rt(n)?"width":"height",o)},a))})),gr=Object(k.a)((function(e){return{root:{position:"fixed",top:0,left:0,bottom:0,zIndex:e.zIndex.drawer-1},anchorLeft:{right:"auto"},anchorRight:{left:"auto",right:0},anchorTop:{bottom:"auto",right:0},anchorBottom:{top:"auto",bottom:0,right:0}}}),{name:"PrivateSwipeArea"})(pr),vr=null;function mr(e,t){return"right"===e?document.body.offsetWidth-t[0].pageX:t[0].pageX}function yr(e,t){return"bottom"===e?window.innerHeight-t[0].clientY:t[0].clientY}function br(e,t){return e?t.clientWidth:t.clientHeight}function Cr(e,t,n,r){return Math.min(Math.max(n?t-e:r+t-e,0),r)}var Ar="undefined"!=typeof navigator&&/iPad|iPhone|iPod/.test(navigator.userAgent),_r={enter:Ke.b.enteringScreen,exit:Ke.b.leavingScreen},Ir="undefined"!=typeof window?E.useLayoutEffect:E.useEffect,xr=E.forwardRef((function(e,t){var n=Object(Qe.a)(),r=Object(fr.a)({name:"MuiSwipeableDrawer",props:Object(S.a)({},e),theme:n}),i=r.anchor,o=void 0===i?"left":i,a=r.disableBackdropTransition,s=void 0!==a&&a,l=r.disableDiscovery,u=void 0!==l&&l,c=r.disableSwipeToOpen,d=void 0===c?Ar:c,h=r.hideBackdrop,f=r.hysteresis,p=void 0===f?.52:f,g=r.minFlingVelocity,v=void 0===g?450:g,m=r.ModalProps,y=(m=void 0===m?{}:m).BackdropProps,b=Object(T.a)(m,["BackdropProps"]),C=r.onClose,A=r.onOpen,_=r.open,I=r.PaperProps,x=void 0===I?{}:I,w=r.SwipeAreaProps,M=r.swipeAreaWidth,O=void 0===M?20:M,k=r.transitionDuration,L=void 0===k?_r:k,D=r.variant,P=void 0===D?"temporary":D,N=Object(T.a)(r,["anchor","disableBackdropTransition","disableDiscovery","disableSwipeToOpen","hideBackdrop","hysteresis","minFlingVelocity","ModalProps","onClose","onOpen","open","PaperProps","SwipeAreaProps","swipeAreaWidth","transitionDuration","variant"]),j=E.useState(!1),R=j[0],z=j[1],F=E.useRef({isSwiping:null}),B=E.useRef(),H=E.useRef(),U=E.useRef(),V=E.useRef(!1),Y=E.useRef();Ir((function(){Y.current=null}),[_]);var W=E.useCallback((function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},r=t.mode,i=void 0===r?null:r,a=t.changeTransition,l=void 0===a||a,u=it(n,o),c=-1!==["right","bottom"].indexOf(u)?1:-1,d=rt(o),f=d?"translate(".concat(c*e,"px, 0)"):"translate(0, ".concat(c*e,"px)"),p=U.current.style;p.webkitTransform=f,p.transform=f;var g="";if(i&&(g=n.transitions.create("all",Object(Je.a)({timeout:L},{mode:i}))),l&&(p.webkitTransition=g,p.transition=g),!s&&!h){var v=H.current.style;v.opacity=1-e/br(d,U.current),l&&(v.webkitTransition=g,v.transition=g)}}),[o,s,h,n,L]),G=Object(Ee.a)((function(e){if(V.current)if(vr=null,V.current=!1,z(!1),F.current.isSwiping){F.current.isSwiping=null;var t,r=it(n,o),i=rt(o);t=i?mr(r,e.changedTouches):yr(r,e.changedTouches);var a=i?F.current.startX:F.current.startY,s=br(i,U.current),l=Cr(t,a,_,s),u=l/s;Math.abs(F.current.velocity)>v&&(Y.current=1e3*Math.abs((s-l)/F.current.velocity)),_?F.current.velocity>v||u>p?C():W(0,{mode:"exit"}):F.current.velocity<-v||1-u>p?A():W(br(i,U.current),{mode:"enter"})}else F.current.isSwiping=null})),X=Object(Ee.a)((function(e){if(U.current&&V.current&&(null==vr||vr===F.current)){var t=it(n,o),r=rt(o),i=mr(t,e.touches),a=yr(t,e.touches);if(_&&U.current.contains(e.target)&&null==vr){var s=function(e){var t=e.domTreeShapes,n=e.start,r=e.current,i=e.anchor,o={x:"scrollLeft",y:"scrollTop"},a={x:"scrollWidth",y:"scrollHeight"},s={x:"clientWidth",y:"clientHeight"};return t.some((function(e){var t=r>=n;"top"!==i&&"left"!==i||(t=!t);var l="left"===i||"right"===i?"x":"y",u=e[o[l]],c=u>0,d=u+e[s[l]]<e[a[l]];return t&&d||!t&&c?e:null}))}({domTreeShapes:function(e,t){for(var n=[];e&&e!==t;){var r=window.getComputedStyle(e);"absolute"===r.getPropertyValue("position")||"hidden"===r.getPropertyValue("overflow-x")?n=[]:(e.clientWidth>0&&e.scrollWidth>e.clientWidth||e.clientHeight>0&&e.scrollHeight>e.clientHeight)&&n.push(e),e=e.parentElement}return n}(e.target,U.current),start:r?F.current.startX:F.current.startY,current:r?i:a,anchor:o});if(s)return void(vr=s);vr=F.current}if(null==F.current.isSwiping){var l=Math.abs(i-F.current.startX),c=Math.abs(a-F.current.startY);l>c&&e.cancelable&&e.preventDefault();var d=r?l>c&&l>3:c>l&&c>3;if(!0===d||(r?c>3:l>3)){if(F.current.isSwiping=d,!d)return void G(e);F.current.startX=i,F.current.startY=a,u||_||(r?F.current.startX-=O:F.current.startY-=O)}}if(F.current.isSwiping){var h=br(r,U.current),f=r?F.current.startX:F.current.startY;_&&!F.current.paperHit&&(f=Math.min(f,h));var p=Cr(r?i:a,f,_,h);if(_)if(F.current.paperHit)0===p&&(F.current.startX=i,F.current.startY=a);else{if(!(r?i<h:a<h))return;F.current.paperHit=!0,F.current.startX=i,F.current.startY=a}null===F.current.lastTranslate&&(F.current.lastTranslate=p,F.current.lastTime=performance.now()+1);var g=(p-F.current.lastTranslate)/(performance.now()-F.current.lastTime)*1e3;F.current.velocity=.4*F.current.velocity+.6*g,F.current.lastTranslate=p,F.current.lastTime=performance.now(),e.cancelable&&e.preventDefault(),W(p)}}})),q=Object(Ee.a)((function(e){if(!e.defaultPrevented&&!e.muiHandled&&(!_||H.current.contains(e.target)||U.current.contains(e.target))){var t=it(n,o),r=rt(o),i=mr(t,e.touches),a=yr(t,e.touches);if(!_){if(d||e.target!==B.current)return;if(r){if(i>O)return}else if(a>O)return}e.muiHandled=!0,vr=null,F.current.startX=i,F.current.startY=a,z(!0),!_&&U.current&&W(br(r,U.current)+(u?20:-O),{changeTransition:!1}),F.current.velocity=0,F.current.lastTime=null,F.current.lastTranslate=null,F.current.paperHit=!1,V.current=!0}}));E.useEffect((function(){if("temporary"===P){var e=Object(Se.a)(U.current);return e.addEventListener("touchstart",q),e.addEventListener("touchmove",X,{passive:!1}),e.addEventListener("touchend",G),function(){e.removeEventListener("touchstart",q),e.removeEventListener("touchmove",X,{passive:!1}),e.removeEventListener("touchend",G)}}}),[P,q,X,G]),E.useEffect((function(){return function(){vr===F.current&&(vr=null)}}),[]),E.useEffect((function(){_||z(!1)}),[_]);var Z=E.useCallback((function(e){H.current=Me.findDOMNode(e)}),[]);return E.createElement(E.Fragment,null,E.createElement(st,Object(S.a)({open:!("temporary"!==P||!R)||_,variant:P,ModalProps:Object(S.a)({BackdropProps:Object(S.a)({},y,{ref:Z})},b),PaperProps:Object(S.a)({},x,{style:Object(S.a)({pointerEvents:"temporary"!==P||_?"":"none"},x.style),ref:U}),anchor:o,transitionDuration:Y.current||L,onClose:C,ref:t},N)),!d&&"temporary"===P&&E.createElement(vn,null,E.createElement(gr,Object(S.a)({anchor:o,ref:B,width:O},w))))})),wr=n(922),Mr=n(2378),Sr=n(923),Tr=n(924),Er=n(925),Or=E.forwardRef((function(e,t){var n=e.classes,r=e.className,i=e.component,o=void 0===i?"div":i,a=Object(T.a)(e,["classes","className","component"]);return E.createElement(o,Object(S.a)({ref:t,className:Object(O.a)(n.root,r)},a))})),kr=Object(k.a)({root:{width:"100%",overflowX:"auto"}},{name:"MuiTableContainer"})(Or),Lr=n(926),Dr=n(927),Pr=n(1198),Nr=n(929),jr=n(1199),Rr=n(2381),zr=n(2379),Fr=n(930),Br=n(2356),Hr=n(928),Ur=n(931),Vr=n(2376),Yr=n(2358);function Wr(e,t){var n=t.disableHysteresis,r=void 0!==n&&n,i=t.threshold,o=void 0===i?100:i,a=t.target,s=e.current;return a&&(e.current=void 0!==a.pageYOffset?a.pageYOffset:a.scrollTop),!(!r&&void 0!==s&&e.current<s)&&e.current>o}var Gr="undefined"!=typeof window?window:null;function Xr(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},t=e.getTrigger,n=void 0===t?Wr:t,r=e.target,i=void 0===r?Gr:r,o=Object(T.a)(e,["getTrigger","target"]),a=E.useRef(),s=E.useState((function(){return n(a,o)})),l=s[0],u=s[1];return E.useEffect((function(){var e=function(){u(n(a,Object(S.a)({target:i},o)))};return e(),i.addEventListener("scroll",e),function(){i.removeEventListener("scroll",e)}}),[i,n,JSON.stringify(o)]),l}var qr=n(369),Zr=function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};return function(t){var n=e.breakpoint,r=void 0===n?"sm":n;return Object(qr.a)()((function(e){return E.createElement(t,Object(S.a)({fullScreen:Object(qr.b)(r,e.width)},e))}))}},Qr=n(287),Kr={entering:{transform:"none"},entered:{transform:"none"}},Jr={enter:Ke.b.enteringScreen,exit:Ke.b.leavingScreen},$r=E.forwardRef((function(e,t){var n=e.children,r=e.disableStrictModeCompat,i=void 0!==r&&r,o=e.in,a=e.onEnter,s=e.onEntered,l=e.onEntering,u=e.onExit,c=e.onExited,d=e.onExiting,h=e.style,f=e.timeout,p=void 0===f?Jr:f,g=e.TransitionComponent,v=void 0===g?Ze.a:g,m=Object(T.a)(e,["children","disableStrictModeCompat","in","onEnter","onEntered","onEntering","onExit","onExited","onExiting","style","timeout","TransitionComponent"]),y=Object(Qe.a)(),b=y.unstable_strictMode&&!i,C=E.useRef(null),A=Object(Te.a)(n.ref,t),_=Object(Te.a)(b?C:void 0,A),I=function(e){return function(t,n){if(e){var r=b?[C.current,t]:[t,n],i=Object(ut.a)(r,2),o=i[0],a=i[1];void 0===a?e(o):e(o,a)}}},x=I(l),w=I((function(e,t){Object(Je.b)(e);var n=Object(Je.a)({style:h,timeout:p},{mode:"enter"});e.style.webkitTransition=y.transitions.create("transform",n),e.style.transition=y.transitions.create("transform",n),a&&a(e,t)})),M=I(s),O=I(d),k=I((function(e){var t=Object(Je.a)({style:h,timeout:p},{mode:"exit"});e.style.webkitTransition=y.transitions.create("transform",t),e.style.transition=y.transitions.create("transform",t),u&&u(e)})),L=I(c);return E.createElement(v,Object(S.a)({appear:!0,in:o,nodeRef:b?C:void 0,onEnter:w,onEntered:M,onEntering:x,onExit:k,onExited:L,onExiting:O,timeout:p},m),(function(e,t){return E.cloneElement(n,Object(S.a)({style:Object(S.a)({transform:"scale(0)",visibility:"exited"!==e||o?void 0:"hidden"},Kr[e],h,n.props.style),ref:_},t))}))}));n.d(t,"colors",(function(){return r})),n.d(t,"createMuiTheme",(function(){return x.f})),n.d(t,"unstable_createMuiStrictModeTheme",(function(){return x.y})),n.d(t,"createStyles",(function(){return x.g})),n.d(t,"makeStyles",(function(){return x.t})),n.d(t,"responsiveFontSizes",(function(){return x.v})),n.d(t,"styled",(function(){return x.x})),n.d(t,"useTheme",(function(){return x.z})),n.d(t,"withStyles",(function(){return x.A})),n.d(t,"withTheme",(function(){return x.B})),n.d(t,"createGenerateClassName",(function(){return x.e})),n.d(t,"jssPreset",(function(){return x.r})),n.d(t,"ServerStyleSheets",(function(){return x.b})),n.d(t,"StylesProvider",(function(){return x.c})),n.d(t,"MuiThemeProvider",(function(){return x.a})),n.d(t,"ThemeProvider",(function(){return x.d})),n.d(t,"hexToRgb",(function(){return x.p})),n.d(t,"rgbToHex",(function(){return x.w})),n.d(t,"hslToRgb",(function(){return x.q})),n.d(t,"decomposeColor",(function(){return x.i})),n.d(t,"recomposeColor",(function(){return x.u})),n.d(t,"getContrastRatio",(function(){return x.n})),n.d(t,"getLuminance",(function(){return x.o})),n.d(t,"emphasize",(function(){return x.l})),n.d(t,"fade",(function(){return x.m})),n.d(t,"darken",(function(){return x.h})),n.d(t,"lighten",(function(){return x.s})),n.d(t,"easing",(function(){return x.k})),n.d(t,"duration",(function(){return x.j})),n.d(t,"capitalize",(function(){return w.capitalize})),n.d(t,"createChainedFunction",(function(){return w.createChainedFunction})),n.d(t,"createSvgIcon",(function(){return w.createSvgIcon})),n.d(t,"debounce",(function(){return w.debounce})),n.d(t,"deprecatedPropType",(function(){return w.deprecatedPropType})),n.d(t,"isMuiElement",(function(){return w.isMuiElement})),n.d(t,"ownerDocument",(function(){return w.ownerDocument})),n.d(t,"ownerWindow",(function(){return w.ownerWindow})),n.d(t,"requirePropFactory",(function(){return w.requirePropFactory})),n.d(t,"setRef",(function(){return w.setRef})),n.d(t,"unsupportedProp",(function(){return w.unsupportedProp})),n.d(t,"useControlled",(function(){return w.useControlled})),n.d(t,"useEventCallback",(function(){return w.useEventCallback})),n.d(t,"useForkRef",(function(){return w.useForkRef})),n.d(t,"unstable_useId",(function(){return w.unstable_useId})),n.d(t,"useIsFocusVisible",(function(){return w.useIsFocusVisible})),n.d(t,"Accordion",(function(){return M.a})),n.d(t,"AccordionActions",(function(){return D})),n.d(t,"AccordionDetails",(function(){return P.a})),n.d(t,"AccordionSummary",(function(){return N.a})),n.d(t,"AppBar",(function(){return F})),n.d(t,"Avatar",(function(){return V})),n.d(t,"Backdrop",(function(){return Y.a})),n.d(t,"Badge",(function(){return G})),n.d(t,"BottomNavigation",(function(){return q})),n.d(t,"BottomNavigationAction",(function(){return K})),n.d(t,"Box",(function(){return J.a})),n.d(t,"styleFunction",(function(){return $.styleFunction})),n.d(t,"Breadcrumbs",(function(){return ae})),n.d(t,"Button",(function(){return se.a})),n.d(t,"ButtonBase",(function(){return Z.a})),n.d(t,"ButtonGroup",(function(){return ue})),n.d(t,"Card",(function(){return de})),n.d(t,"CardActionArea",(function(){return fe})),n.d(t,"CardActions",(function(){return ge})),n.d(t,"CardContent",(function(){return me})),n.d(t,"CardHeader",(function(){return be})),n.d(t,"CardMedia",(function(){return _e})),n.d(t,"Checkbox",(function(){return Ie.a})),n.d(t,"Chip",(function(){return xe.a})),n.d(t,"CircularProgress",(function(){return we.a})),n.d(t,"ClickAwayListener",(function(){return ke})),n.d(t,"Collapse",(function(){return Le.a})),n.d(t,"Container",(function(){return Ne})),n.d(t,"CssBaseline",(function(){return ze})),n.d(t,"Dialog",(function(){return Fe.a})),n.d(t,"DialogActions",(function(){return Be.a})),n.d(t,"DialogContent",(function(){return He.a})),n.d(t,"DialogContentText",(function(){return Ve})),n.d(t,"DialogTitle",(function(){return Ye.a})),n.d(t,"Divider",(function(){return Ge})),n.d(t,"Drawer",(function(){return st})),n.d(t,"ExpansionPanel",(function(){return ft})),n.d(t,"ExpansionPanelActions",(function(){return gt})),n.d(t,"ExpansionPanelDetails",(function(){return mt})),n.d(t,"ExpansionPanelSummary",(function(){return Ct})),n.d(t,"Fab",(function(){return At.a})),n.d(t,"Fade",(function(){return _t.a})),n.d(t,"FilledInput",(function(){return It.a})),n.d(t,"FormControl",(function(){return xt.a})),n.d(t,"useFormControl",(function(){return wt.useFormControl})),n.d(t,"FormControlLabel",(function(){return Mt.a})),n.d(t,"FormGroup",(function(){return St.a})),n.d(t,"FormHelperText",(function(){return Tt.a})),n.d(t,"FormLabel",(function(){return Et.a})),n.d(t,"Grid",(function(){return Ot.a})),n.d(t,"GridList",(function(){return Lt})),n.d(t,"GridListTile",(function(){return jt})),n.d(t,"GridListTileBar",(function(){return zt})),n.d(t,"Grow",(function(){return Ft.a})),n.d(t,"Hidden",(function(){return Bt.a})),n.d(t,"Icon",(function(){return Ht.a})),n.d(t,"IconButton",(function(){return yt.a})),n.d(t,"Input",(function(){return Ut.a})),n.d(t,"InputAdornment",(function(){return Vt.a})),n.d(t,"InputBase",(function(){return Yt.a})),n.d(t,"InputLabel",(function(){return Wt.a})),n.d(t,"LinearProgress",(function(){return Gt.a})),n.d(t,"Link",(function(){return Zt})),n.d(t,"List",(function(){return Qt.a})),n.d(t,"ListItem",(function(){return Kt.a})),n.d(t,"ListItemAvatar",(function(){return en})),n.d(t,"ListItemIcon",(function(){return nn})),n.d(t,"ListItemSecondaryAction",(function(){return on})),n.d(t,"ListItemText",(function(){return an.a})),n.d(t,"ListSubheader",(function(){return sn.a})),n.d(t,"Menu",(function(){return ln.a})),n.d(t,"MenuItem",(function(){return un.a})),n.d(t,"MenuList",(function(){return cn.a})),n.d(t,"MobileStepper",(function(){return hn})),n.d(t,"Modal",(function(){return Xe.a})),n.d(t,"ModalManager",(function(){return fn.a})),n.d(t,"NativeSelect",(function(){return pn.a})),n.d(t,"NoSsr",(function(){return vn})),n.d(t,"OutlinedInput",(function(){return mn.a})),n.d(t,"Paper",(function(){return R.a})),n.d(t,"Popover",(function(){return yn.a})),n.d(t,"Popper",(function(){return bn.a})),n.d(t,"Portal",(function(){return Cn.a})),n.d(t,"Radio",(function(){return kn})),n.d(t,"RadioGroup",(function(){return Dn})),n.d(t,"useRadioGroup",(function(){return Sn})),n.d(t,"RootRef",(function(){return Hn})),n.d(t,"Select",(function(){return Un.a})),n.d(t,"Slide",(function(){return tt})),n.d(t,"Slider",(function(){return Vn.a})),n.d(t,"Snackbar",(function(){return Xn})),n.d(t,"SnackbarContent",(function(){return Wn})),n.d(t,"Step",(function(){return Zn})),n.d(t,"StepButton",(function(){return or})),n.d(t,"StepConnector",(function(){return sr})),n.d(t,"StepContent",(function(){return ur})),n.d(t,"StepIcon",(function(){return tr})),n.d(t,"StepLabel",(function(){return rr})),n.d(t,"Stepper",(function(){return hr})),n.d(t,"SvgIcon",(function(){return Jn.a})),n.d(t,"SwipeableDrawer",(function(){return xr})),n.d(t,"Switch",(function(){return wr.a})),n.d(t,"Tab",(function(){return Mr.a})),n.d(t,"Table",(function(){return Sr.a})),n.d(t,"TableBody",(function(){return Tr.a})),n.d(t,"TableCell",(function(){return Er.a})),n.d(t,"TableContainer",(function(){return kr})),n.d(t,"TableFooter",(function(){return Lr.a})),n.d(t,"TableHead",(function(){return Dr.a})),n.d(t,"TablePagination",(function(){return Pr.a})),n.d(t,"TableRow",(function(){return Nr.a})),n.d(t,"TableSortLabel",(function(){return jr.a})),n.d(t,"Tabs",(function(){return Rr.a})),n.d(t,"TabScrollButton",(function(){return zr.a})),n.d(t,"TextField",(function(){return Fr.a})),n.d(t,"TextareaAutosize",(function(){return Br.a})),n.d(t,"Toolbar",(function(){return Hr.a})),n.d(t,"Tooltip",(function(){return Ur.a})),n.d(t,"Typography",(function(){return te.a})),n.d(t,"Unstable_TrapFocus",(function(){return Vr.a})),n.d(t,"useMediaQuery",(function(){return Yr.a})),n.d(t,"useScrollTrigger",(function(){return Xr})),n.d(t,"withMobileDialog",(function(){return Zr})),n.d(t,"withWidth",(function(){return qr.a})),n.d(t,"isWidthUp",(function(){return Qr.b})),n.d(t,"isWidthDown",(function(){return Qr.a})),n.d(t,"Zoom",(function(){return $r}))},function(e,t,n){"use strict";var r=n(1562),i={childContextTypes:!0,contextType:!0,contextTypes:!0,defaultProps:!0,displayName:!0,getDefaultProps:!0,getDerivedStateFromError:!0,getDerivedStateFromProps:!0,mixins:!0,propTypes:!0,type:!0},o={name:!0,length:!0,prototype:!0,caller:!0,callee:!0,arguments:!0,arity:!0},a={$$typeof:!0,compare:!0,defaultProps:!0,displayName:!0,propTypes:!0,type:!0},s={};function l(e){return r.isMemo(e)?a:s[e.$$typeof]||i}s[r.ForwardRef]={$$typeof:!0,render:!0,defaultProps:!0,displayName:!0,propTypes:!0},s[r.Memo]=a;var u=Object.defineProperty,c=Object.getOwnPropertyNames,d=Object.getOwnPropertySymbols,h=Object.getOwnPropertyDescriptor,f=Object.getPrototypeOf,p=Object.prototype;e.exports=function e(t,n,r){if("string"!=typeof n){if(p){var i=f(n);i&&i!==p&&e(t,i,r)}var a=c(n);d&&(a=a.concat(d(n)));for(var s=l(t),g=l(n),v=0;v<a.length;++v){var m=a[v];if(!(o[m]||r&&r[m]||g&&g[m]||s&&s[m])){var y=h(n,m);try{u(t,m,y)}catch(e){}}}}return t}},function(e,t,n){"use strict";function r(e,t){if(null==e)return{};var n,r,i={},o=Object.keys(e);for(r=0;r<o.length;r++)n=o[r],t.indexOf(n)>=0||(i[n]=e[n]);return i}n.d(t,"a",(function(){return r}))},,function(e,t,n){"use strict";e.exports=function(e){var t=e.editType,n=e.colorEditType;void 0===n&&(n=t);var r={family:{valType:"string",noBlank:!0,strict:!0,editType:t},size:{valType:"number",min:1,editType:t},color:{valType:"color",editType:n},editType:t};return e.arrayOk&&(r.family.arrayOk=!0,r.size.arrayOk=!0,r.color.arrayOk=!0),r}},function(e,t,n){"use strict";var r=n(294).counter;e.exports={idRegex:{x:r("x","( domain)?"),y:r("y","( domain)?")},attrRegex:r("[xy]axis"),xAxisMatch:r("xaxis"),yAxisMatch:r("yaxis"),AX_ID_PATTERN:/^[xyz][0-9]*( domain)?$/,AX_NAME_PATTERN:/^[xyz]axis[0-9]*$/,SUBPLOT_PATTERN:/^x([0-9]*)y([0-9]*)$/,HOUR_PATTERN:"hour",WEEKDAY_PATTERN:"day of week",MINDRAG:8,MINSELECT:12,MINZOOM:20,DRAGGERSIZE:20,BENDPX:1.5,REDRAWDELAY:50,SELECTDELAY:100,SELECTID:"-select",DFLTRANGEX:[-1,6],DFLTRANGEY:[-1,4],traceLayerClasses:["imagelayer","heatmaplayer","contourcarpetlayer","contourlayer","funnellayer","waterfalllayer","barlayer","carpetlayer","violinlayer","boxlayer","ohlclayer","scattercarpetlayer","scatterlayer"],clipOnAxisFalseQuery:[".scatterlayer",".barlayer",".funnellayer",".waterfalllayer"],layerValue2layerClass:{"above traces":"above","below traces":"below"}}},function(e,t,n){"use strict";var r=n(39),i=n(10),o=i.strTranslate,a=n(196),s=n(97).LINE_SPACING;function l(e,t){return e.node().getBoundingClientRect()[t]}var u=/([^$]*)([$]+[^$]*[$]+)([^$]*)/;t.convertToTspans=function(e,n,M){var S=e.text(),E=!e.attr("data-notex")&&"undefined"!=typeof MathJax&&S.match(u),O=r.select(e.node().parentNode);if(!O.empty()){var k=e.attr("class")?e.attr("class").split(" ")[0]:"text";return k+="-math",O.selectAll("svg."+k).remove(),O.selectAll("g."+k+"-group").remove(),e.style("display",null).attr({"data-unformatted":S,"data-math":"N"}),E?(n&&n._promises||[]).push(new Promise((function(t){e.style("display","none");var n=parseInt(e.node().style.fontSize,10),a={fontSize:n};!function(e,t,n){var o,a,s,l;MathJax.Hub.Queue((function(){return a=i.extendDeepAll({},MathJax.Hub.config),s=MathJax.Hub.processSectionDelay,void 0!==MathJax.Hub.processSectionDelay&&(MathJax.Hub.processSectionDelay=0),MathJax.Hub.Config({messageStyle:"none",tex2jax:{inlineMath:[["$","$"],["\\(","\\)"]]},displayAlign:"left"})}),(function(){if("SVG"!==(o=MathJax.Hub.config.menuSettings.renderer))return MathJax.Hub.setRenderer("SVG")}),(function(){var n="math-output-"+i.randstr({},64);return l=r.select("body").append("div").attr({id:n}).style({visibility:"hidden",position:"absolute"}).style({"font-size":t.fontSize+"px"}).text(e.replace(c,"\\lt ").replace(d,"\\gt ")),MathJax.Hub.Typeset(l.node())}),(function(){var t=r.select("body").select("#MathJax_SVG_glyphs");if(l.select(".MathJax_SVG").empty()||!l.select("svg").node())i.log("There was an error in the tex syntax.",e),n();else{var a=l.select("svg").node().getBoundingClientRect();n(l.select(".MathJax_SVG"),t,a)}if(l.remove(),"SVG"!==o)return MathJax.Hub.setRenderer(o)}),(function(){return void 0!==s&&(MathJax.Hub.processSectionDelay=s),MathJax.Hub.Config(a)}))}(E[2],a,(function(r,i,a){O.selectAll("svg."+k).remove(),O.selectAll("g."+k+"-group").remove();var s=r&&r.select("svg");if(!s||!s.node())return L(),void t();var u=O.append("g").classed(k+"-group",!0).attr({"pointer-events":"none","data-unformatted":S,"data-math":"Y"});u.node().appendChild(s.node()),i&&i.node()&&s.node().insertBefore(i.node().cloneNode(!0),s.node().firstChild),s.attr({class:k,height:a.height,preserveAspectRatio:"xMinYMin meet"}).style({overflow:"visible","pointer-events":"none"});var c=e.node().style.fill||"black",d=s.select("g");d.attr({fill:c,stroke:c});var h=l(d,"width"),f=l(d,"height"),p=+e.attr("x")-h*{start:0,middle:.5,end:1}[e.attr("text-anchor")||"start"],g=-(n||l(e,"height"))/4;"y"===k[0]?(u.attr({transform:"rotate("+[-90,+e.attr("x"),+e.attr("y")]+")"+o(-h/2,g-f/2)}),s.attr({x:+e.attr("x"),y:+e.attr("y")})):"l"===k[0]?s.attr({x:e.attr("x"),y:g-f/2}):"a"===k[0]&&0!==k.indexOf("atitle")?s.attr({x:0,y:g}):s.attr({x:p,y:+e.attr("y")+g-f/2}),M&&M.call(e,u),t(u)}))}))):L(),e}function L(){O.empty()||(k=e.attr("class")+"-math",O.select("svg."+k).remove()),e.text("").style("white-space","pre"),function(e,t){t=t.replace(v," ");var n,o=!1,l=[],u=-1;function c(){u++;var t=document.createElementNS(a.svg,"tspan");r.select(t).attr({class:"line",dy:u*s+"em"}),e.appendChild(t),n=t;var i=l;if(l=[{node:t}],i.length>1)for(var o=1;o<i.length;o++)d(i[o])}function d(e){var t,i=e.type,o={};if("a"===i){t="a";var s=e.target,u=e.href,c=e.popup;u&&(o={"xlink:xlink:show":"_blank"===s||"_"!==s.charAt(0)?"new":"replace",target:s,"xlink:xlink:href":u},c&&(o.onclick='window.open(this.href.baseVal,this.target.baseVal,"'+c+'");return false;'))}else t="tspan";e.style&&(o.style=e.style);var d=document.createElementNS(a.svg,t);if("sup"===i||"sub"===i){M(n,"​"),n.appendChild(d);var h=document.createElementNS(a.svg,"tspan");M(h,"​"),r.select(h).attr("dy",p[i]),o.dy=f[i],n.appendChild(d),n.appendChild(h)}else n.appendChild(d);r.select(d).attr(o),n=e.node=d,l.push(e)}function M(e,t){e.appendChild(document.createTextNode(t))}function S(e){if(1!==l.length){var r=l.pop();e!==r.type&&i.log("Start tag <"+r.type+"> doesnt match end tag <"+e+">. Pretending it did match.",t),n=l[l.length-1].node}else i.log("Ignoring unexpected end tag </"+e+">.",t)}b.test(t)?c():(n=e,l=[{node:e}]);for(var E=t.split(m),O=0;O<E.length;O++){var k=E[O],L=k.match(y),D=L&&L[2].toLowerCase(),P=h[D];if("br"===D)c();else if(void 0===P)M(n,T(k));else if(L[1])S(D);else{var N=L[4],j={type:D},R=x(N,C);if(R?(R=R.replace(w,"$1 fill:"),P&&(R+=";"+P)):P&&(R=P),R&&(j.style=R),"a"===D){o=!0;var z=x(N,A);if(z){var F=document.createElement("a");F.href=z,-1!==g.indexOf(F.protocol)&&(j.href=encodeURI(decodeURI(z)),j.target=x(N,_)||"_blank",j.popup=x(N,I))}}d(j)}}return o}(e.node(),S)&&e.style("pointer-events","all"),t.positionText(e),M&&M.call(e)}};var c=/(<|&lt;|&#60;)/g,d=/(>|&gt;|&#62;)/g;var h={sup:"font-size:70%",sub:"font-size:70%",b:"font-weight:bold",i:"font-style:italic",a:"cursor:pointer",span:"",em:"font-style:italic;font-weight:bold"},f={sub:"0.3em",sup:"-0.6em"},p={sub:"-0.21em",sup:"0.42em"},g=["http:","https:","mailto:","",void 0,":"],v=t.NEWLINES=/(\r\n?|\n)/g,m=/(<[^<>]*>)/,y=/<(\/?)([^ >]*)(\s+(.*))?>/i,b=/<br(\s+.*)?>/i;t.BR_TAG_ALL=/<br(\s+.*)?>/gi;var C=/(^|[\s"'])style\s*=\s*("([^"]*);?"|'([^']*);?')/i,A=/(^|[\s"'])href\s*=\s*("([^"]*)"|'([^']*)')/i,_=/(^|[\s"'])target\s*=\s*("([^"\s]*)"|'([^'\s]*)')/i,I=/(^|[\s"'])popup\s*=\s*("([\w=,]*)"|'([\w=,]*)')/i;function x(e,t){if(!e)return null;var n=e.match(t),r=n&&(n[3]||n[4]);return r&&T(r)}var w=/(^|;)\s*color:/;t.plainText=function(e,t){for(var n=void 0!==(t=t||{}).len&&-1!==t.len?t.len:1/0,r=void 0!==t.allowedTags?t.allowedTags:["br"],i="...".length,o=e.split(m),a=[],s="",l=0,u=0;u<o.length;u++){var c=o[u],d=c.match(y),h=d&&d[2].toLowerCase();if(h)-1!==r.indexOf(h)&&(a.push(c),s=h);else{var f=c.length;if(l+f<n)a.push(c),l+=f;else if(l<n){var p=n-l;s&&("br"!==s||p<=i||f<=i)&&a.pop(),n>i?a.push(c.substr(0,p-i)+"..."):a.push(c.substr(0,p));break}s=""}}return a.join("")};var M={mu:"μ",amp:"&",lt:"<",gt:">",nbsp:" ",times:"×",plusmn:"±",deg:"°"},S=/&(#\d+|#x[\da-fA-F]+|[a-z]+);/g;function T(e){return e.replace(S,(function(e,t){return("#"===t.charAt(0)?function(e){if(e>1114111)return;var t=String.fromCodePoint;if(t)return t(e);var n=String.fromCharCode;return e<=65535?n(e):n(55232+(e>>10),e%1024+56320)}("x"===t.charAt(1)?parseInt(t.substr(2),16):parseInt(t.substr(1),10)):M[t])||e}))}function E(e,t,n){var r,o,a,s=n.horizontalAlign,l=n.verticalAlign||"top",u=e.node().getBoundingClientRect(),c=t.node().getBoundingClientRect();return o="bottom"===l?function(){return u.bottom-r.height}:"middle"===l?function(){return u.top+(u.height-r.height)/2}:function(){return u.top},a="right"===s?function(){return u.right-r.width}:"center"===s?function(){return u.left+(u.width-r.width)/2}:function(){return u.left},function(){r=this.node().getBoundingClientRect();var e=a()-c.left,t=o()-c.top,s=n.gd||{};if(n.gd){s._fullLayout._calcInverseTransform(s);var l=i.apply3DTransform(s._fullLayout._invTransform)(e,t);e=l[0],t=l[1]}return this.style({top:t+"px",left:e+"px","z-index":1e3}),this}}t.convertEntities=T,t.sanitizeHTML=function(e){e=e.replace(v," ");for(var t=document.createElement("p"),n=t,i=[],o=e.split(m),a=0;a<o.length;a++){var s=o[a],l=s.match(y),u=l&&l[2].toLowerCase();if(u in h)if(l[1])i.length&&(n=i.pop());else{var c=l[4],d=x(c,C),f=d?{style:d}:{};if("a"===u){var p=x(c,A);if(p){var b=document.createElement("a");if(b.href=p,-1!==g.indexOf(b.protocol)){f.href=encodeURI(decodeURI(p));var I=x(c,_);I&&(f.target=I)}}}var w=document.createElement(u);n.appendChild(w),r.select(w).attr(f),n=w,i.push(w)}else n.appendChild(document.createTextNode(T(s)))}return t.innerHTML},t.lineCount=function(e){return e.selectAll("tspan.line").size()||1},t.positionText=function(e,t,n){return e.each((function(){var e=r.select(this);function i(t,n){return void 0===n?null===(n=e.attr(t))&&(e.attr(t,0),n=0):e.attr(t,n),n}var o=i("x",t),a=i("y",n);"text"===this.nodeName&&e.selectAll("tspan.line").attr({x:o,y:a})}))},t.makeEditable=function(e,t){var n=t.gd,i=t.delegate,o=r.dispatch("edit","input","cancel"),a=i||e;if(e.style({"pointer-events":i?"none":"all"}),1!==e.size())throw new Error("boo");function s(){!function(){var i=r.select(n).select(".svg-container"),a=i.append("div"),s=e.node().style,u=parseFloat(s.fontSize||12),c=t.text;void 0===c&&(c=e.attr("data-unformatted"));a.classed("plugin-editable editable",!0).style({position:"absolute","font-family":s.fontFamily||"Arial","font-size":u,color:t.fill||s.fill||"black",opacity:1,"background-color":t.background||"transparent",outline:"#ffffff33 1px solid",margin:[-u/8+1,0,0,-1].join("px ")+"px",padding:"0","box-sizing":"border-box"}).attr({contenteditable:!0}).text(c).call(E(e,i,t)).on("blur",(function(){n._editing=!1,e.text(this.textContent).style({opacity:1});var t,i=r.select(this).attr("class");(t=i?"."+i.split(" ")[0]+"-math-group":"[class*=-math-group]")&&r.select(e.node().parentNode).select(t).style({opacity:0});var a=this.textContent;r.select(this).transition().duration(0).remove(),r.select(document).on("mouseup",null),o.edit.call(e,a)})).on("focus",(function(){var e=this;n._editing=!0,r.select(document).on("mouseup",(function(){if(r.event.target===e)return!1;document.activeElement===a.node()&&a.node().blur()}))})).on("keyup",(function(){27===r.event.which?(n._editing=!1,e.style({opacity:1}),r.select(this).style({opacity:0}).on("blur",(function(){return!1})).transition().remove(),o.cancel.call(e,this.textContent)):(o.input.call(e,this.textContent),r.select(this).call(E(e,i,t)))})).on("keydown",(function(){13===r.event.which&&this.blur()})).call(l)}(),e.style({opacity:0});var i,s=a.attr("class");(i=s?"."+s.split(" ")[0]+"-math-group":"[class*=-math-group]")&&r.select(e.node().parentNode).select(i).style({opacity:0})}function l(e){var t=e.node(),n=document.createRange();n.selectNodeContents(t);var r=window.getSelection();r.removeAllRanges(),r.addRange(n),t.focus()}return t.immediate?s():a.on("click",s),r.rebind(e,o,"on")}},function(e,t,n){"use strict";e.exports={FROM_BL:{left:0,center:.5,right:1,bottom:0,middle:.5,top:1},FROM_TL:{left:0,center:.5,right:1,bottom:1,middle:.5,top:0},FROM_BR:{left:1,center:.5,right:0,bottom:0,middle:.5,top:1},LINE_SPACING:1.3,CAP_SHIFT:.7,MID_SHIFT:.35,OPPOSITE_SIDE:{left:"right",right:"left",top:"bottom",bottom:"top"}}},function(e,t,n){var r=n(626);e.exports=function(e,t){if(null==e)return{};var n,i,o=r(e,t);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(i=0;i<a.length;i++)n=a[i],t.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(e,n)&&(o[n]=e[n])}return o}},function(e,t,n){"use strict";function r(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}n.d(t,"a",(function(){return r}))},function(e,t,n){"use strict";n.d(t,"a",(function(){return i}));var r=n(0);function i(e){var t=e.controlled,n=e.default,i=(e.name,e.state,r.useRef(void 0!==t).current),o=r.useState(n),a=o[0],s=o[1];return[i?t:a,r.useCallback((function(e){i||s(e)}),[])]}},function(e,t,n){"use strict";n.d(t,"a",(function(){return i}));var r=n(447);function i(e,t){e.prototype=Object.create(t.prototype),e.prototype.constructor=e,Object(r.a)(e,t)}},function(e,t,n){"use strict";n.r(t),n.d(t,"AttachedProperty",(function(){return i}));var r,i=function(){function e(e){this._pid=r.nextPID(),this.name=e.name,this._create=e.create,this._coerce=e.coerce||null,this._compare=e.compare||null,this._changed=e.changed||null}return e.prototype.get=function(e){var t=r.ensureMap(e);return this._pid in t?t[this._pid]:t[this._pid]=this._createValue(e)},e.prototype.set=function(e,t){var n,i=r.ensureMap(e);n=this._pid in i?i[this._pid]:i[this._pid]=this._createValue(e);var o=this._coerceValue(e,t);this._maybeNotify(e,n,i[this._pid]=o)},e.prototype.coerce=function(e){var t,n=r.ensureMap(e);t=this._pid in n?n[this._pid]:n[this._pid]=this._createValue(e);var i=this._coerceValue(e,t);this._maybeNotify(e,t,n[this._pid]=i)},e.prototype._createValue=function(e){return(0,this._create)(e)},e.prototype._coerceValue=function(e,t){var n=this._coerce;return n?n(e,t):t},e.prototype._compareValue=function(e,t){var n=this._compare;return n?n(e,t):e===t},e.prototype._maybeNotify=function(e,t,n){var r=this._changed;r&&!this._compareValue(t,n)&&r(e,t,n)},e}();!function(e){e.clearData=function(e){r.ownerData.delete(e)}}(i||(i={})),function(e){var t;e.ownerData=new WeakMap,e.nextPID=(t=0,function(){return"pid-"+(""+Math.random()).slice(2)+"-"+t++}),e.ensureMap=function(t){var n=e.ownerData.get(t);return n||(n=Object.create(null),e.ownerData.set(t,n),n)}}(r||(r={}))},function(e,t,n){"use strict";n.r(t);var r=n(630);n.d(t,"default",(function(){return r.a}));var i=n(106);n.d(t,"useFormControl",(function(){return i.a}))},function(e,t,n){"use strict";var r=n(26),i=n(265),o=n(442);function a(e){for(var t=arguments.length,n=new Array(t>1?t-1:0),r=1;r<t;r++)n[r-1]=arguments[r];return i.a.apply(void 0,[Object(o.a)({unstable_strictMode:!0},e)].concat(n))}var s=n(902),l=n(903),u=n(1),c=n(334),d=n(21);function h(e){return String(parseFloat(e)).length===String(e).length}function f(e){return parseFloat(e)}function p(e){return function(t,n){var r=String(t).match(/[\d.\-+]*\s*(.*)/)[1]||"";if(r===n)return t;var i=f(t);if("px"!==r)if("em"===r)i=f(t)*f(e);else if("rem"===r)return i=f(t)*f(e),t;var o=i;if("px"!==n)if("em"===n)o=i/f(e);else{if("rem"!==n)return t;o=i/f(e)}return parseFloat(o.toFixed(5))+n}}function g(e){var t=e.size,n=e.grid,r=t-t%n,i=r+n;return t-r<i-t?r:i}function v(e){var t=e.lineHeight;return e.pixels/(t*e.htmlFontSize)}function m(e){var t=e.cssProperty,n=e.min,r=e.max,i=e.unit,o=void 0===i?"rem":i,a=e.breakpoints,s=void 0===a?[600,960,1280]:a,l=e.transform,u=void 0===l?null:l,c=Object(d.a)({},t,"".concat(n).concat(o)),h=(r-n)/s[s.length-1];return s.forEach((function(e){var r=n+h*e;null!==u&&(r=u(r)),c["@media (min-width:".concat(e,"px)")]=Object(d.a)({},t,"".concat(Math.round(1e4*r)/1e4).concat(o))})),c}function y(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=t.breakpoints,r=void 0===n?["sm","md","lg"]:n,i=t.disableAlign,o=void 0!==i&&i,a=t.factor,s=void 0===a?2:a,l=t.variants,d=void 0===l?["h1","h2","h3","h4","h5","h6","subtitle1","subtitle2","body1","body2","caption","button","overline"]:l,f=Object(u.a)({},e);f.typography=Object(u.a)({},f.typography);var y=f.typography,b=p(y.htmlFontSize),C=r.map((function(e){return f.breakpoints.values[e]}));return d.forEach((function(e){var t=y[e],n=parseFloat(b(t.fontSize,"rem"));if(!(n<=1)){var r=n,i=1+(r-1)/s,a=t.lineHeight;if(!h(a)&&!o)throw new Error(Object(c.a)(6));h(a)||(a=parseFloat(b(a,"rem"))/parseFloat(n));var l=null;o||(l=function(e){return g({size:e,grid:v({pixels:4,lineHeight:a,htmlFontSize:y.htmlFontSize})})}),y[e]=Object(u.a)({},t,m({cssProperty:"fontSize",min:i,max:r,unit:"rem",breakpoints:C,transform:l}))}})),f}var b=n(435),C=n(65),A=n(52),_=n(7),I=n(367),x=n(167),w=Object(I.b)({defaultTheme:x.a}),M=n(1308),S=n(1313),T=n(2374),E=n(318),O=n(2375);n.d(t,"p",(function(){return r.g})),n.d(t,"w",(function(){return r.k})),n.d(t,"q",(function(){return r.h})),n.d(t,"i",(function(){return r.b})),n.d(t,"u",(function(){return r.j})),n.d(t,"n",(function(){return r.e})),n.d(t,"o",(function(){return r.f})),n.d(t,"l",(function(){return r.c})),n.d(t,"m",(function(){return r.d})),n.d(t,"h",(function(){return r.a})),n.d(t,"s",(function(){return r.i})),n.d(t,"f",(function(){return i.a})),n.d(t,"y",(function(){return a})),n.d(t,"g",(function(){return s.a})),n.d(t,"t",(function(){return l.a})),n.d(t,"v",(function(){return y})),n.d(t,"x",(function(){return b.a})),n.d(t,"k",(function(){return C.c})),n.d(t,"j",(function(){return C.b})),n.d(t,"z",(function(){return A.a})),n.d(t,"A",(function(){return _.a})),n.d(t,"B",(function(){return w})),n.d(t,"e",(function(){return M.a})),n.d(t,"r",(function(){return S.a})),n.d(t,"b",(function(){return T.a})),n.d(t,"c",(function(){return E.b})),n.d(t,"a",(function(){return O.a})),n.d(t,"d",(function(){return O.a}))},function(e,t,n){"use strict";function r(e){var t,n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:166;function r(){for(var r=arguments.length,i=new Array(r),o=0;o<r;o++)i[o]=arguments[o];var a=this,s=function(){e.apply(a,i)};clearTimeout(t),t=setTimeout(s,n)}return r.clear=function(){clearTimeout(t)},r}n.d(t,"a",(function(){return r}))},function(e,t,n){"use strict";n.d(t,"a",(function(){return o}));var r=n(0),i=n(169);function o(){return r.useContext(i.a)}},function(e,t,n){"use strict";function r(){for(var e=arguments.length,t=new Array(e),n=0;n<e;n++)t[n]=arguments[n];return t.reduce((function(e,t){return null==t?e:function(){for(var n=arguments.length,r=new Array(n),i=0;i<n;i++)r[i]=arguments[i];e.apply(this,r),t.apply(this,r)}}),(function(){}))}n.d(t,"a",(function(){return r}))},function(e,t,n){"use strict";n.r(t);var r=n(909);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";n.r(t);var r=n(910);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";n.r(t);var r=n(911);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=function(){function e(e){this._name=e}return e.flip=function(t){return t===e.HORZ?e.VERT:e.HORZ},e.prototype.getName=function(){return this._name},e.prototype.toString=function(){return this._name},e.HORZ=new e("horz"),e.VERT=new e("vert"),e}();t.default=r},function(e,t,n){"use strict";(function(e){var r=n(215),i=n(1033),o=n(1034).Buffer;e.__TYPEDARRAY_POOL||(e.__TYPEDARRAY_POOL={UINT8:i([32,0]),UINT16:i([32,0]),UINT32:i([32,0]),BIGUINT64:i([32,0]),INT8:i([32,0]),INT16:i([32,0]),INT32:i([32,0]),BIGINT64:i([32,0]),FLOAT:i([32,0]),DOUBLE:i([32,0]),DATA:i([32,0]),UINT8C:i([32,0]),BUFFER:i([32,0])});var a="undefined"!=typeof Uint8ClampedArray,s="undefined"!=typeof BigUint64Array,l="undefined"!=typeof BigInt64Array,u=e.__TYPEDARRAY_POOL;u.UINT8C||(u.UINT8C=i([32,0])),u.BIGUINT64||(u.BIGUINT64=i([32,0])),u.BIGINT64||(u.BIGINT64=i([32,0])),u.BUFFER||(u.BUFFER=i([32,0]));var c=u.DATA,d=u.BUFFER;function h(e){if(e){var t=e.length||e.byteLength,n=r.log2(t);c[n].push(e)}}function f(e){e=r.nextPow2(e);var t=r.log2(e),n=c[t];return n.length>0?n.pop():new ArrayBuffer(e)}function p(e){return new Uint8Array(f(e),0,e)}function g(e){return new Uint16Array(f(2*e),0,e)}function v(e){return new Uint32Array(f(4*e),0,e)}function m(e){return new Int8Array(f(e),0,e)}function y(e){return new Int16Array(f(2*e),0,e)}function b(e){return new Int32Array(f(4*e),0,e)}function C(e){return new Float32Array(f(4*e),0,e)}function A(e){return new Float64Array(f(8*e),0,e)}function _(e){return a?new Uint8ClampedArray(f(e),0,e):p(e)}function I(e){return s?new BigUint64Array(f(8*e),0,e):null}function x(e){return l?new BigInt64Array(f(8*e),0,e):null}function w(e){return new DataView(f(e),0,e)}function M(e){e=r.nextPow2(e);var t=r.log2(e),n=d[t];return n.length>0?n.pop():new o(e)}t.free=function(e){if(o.isBuffer(e))d[r.log2(e.length)].push(e);else{if("[object ArrayBuffer]"!==Object.prototype.toString.call(e)&&(e=e.buffer),!e)return;var t=e.length||e.byteLength,n=0|r.log2(t);c[n].push(e)}},t.freeUint8=t.freeUint16=t.freeUint32=t.freeBigUint64=t.freeInt8=t.freeInt16=t.freeInt32=t.freeBigInt64=t.freeFloat32=t.freeFloat=t.freeFloat64=t.freeDouble=t.freeUint8Clamped=t.freeDataView=function(e){h(e.buffer)},t.freeArrayBuffer=h,t.freeBuffer=function(e){d[r.log2(e.length)].push(e)},t.malloc=function(e,t){if(void 0===t||"arraybuffer"===t)return f(e);switch(t){case"uint8":return p(e);case"uint16":return g(e);case"uint32":return v(e);case"int8":return m(e);case"int16":return y(e);case"int32":return b(e);case"float":case"float32":return C(e);case"double":case"float64":return A(e);case"uint8_clamped":return _(e);case"bigint64":return x(e);case"biguint64":return I(e);case"buffer":return M(e);case"data":case"dataview":return w(e);default:return null}return null},t.mallocArrayBuffer=f,t.mallocUint8=p,t.mallocUint16=g,t.mallocUint32=v,t.mallocInt8=m,t.mallocInt16=y,t.mallocInt32=b,t.mallocFloat32=t.mallocFloat=C,t.mallocFloat64=t.mallocDouble=A,t.mallocUint8Clamped=_,t.mallocBigUint64=I,t.mallocBigInt64=x,t.mallocDataView=w,t.mallocBuffer=M,t.clearCache=function(){for(var e=0;e<32;++e)u.UINT8[e].length=0,u.UINT16[e].length=0,u.UINT32[e].length=0,u.INT8[e].length=0,u.INT16[e].length=0,u.INT32[e].length=0,u.FLOAT[e].length=0,u.DOUBLE[e].length=0,u.BIGUINT64[e].length=0,u.BIGINT64[e].length=0,u.UINT8C[e].length=0,c[e].length=0,d[e].length=0}}).call(this,n(61))},function(e,t,n){"use strict";var r=n(70),i=n(153).Graph;function o(e,t,n,i){var o;do{o=r.uniqueId(i)}while(e.hasNode(o));return n.dummy=t,e.setNode(o,n),o}function a(e){return r.max(r.map(e.nodes(),(function(t){var n=e.node(t).rank;if(!r.isUndefined(n))return n})))}e.exports={addDummyNode:o,simplify:function(e){var t=(new i).setGraph(e.graph());return r.forEach(e.nodes(),(function(n){t.setNode(n,e.node(n))})),r.forEach(e.edges(),(function(n){var r=t.edge(n.v,n.w)||{weight:0,minlen:1},i=e.edge(n);t.setEdge(n.v,n.w,{weight:r.weight+i.weight,minlen:Math.max(r.minlen,i.minlen)})})),t},asNonCompoundGraph:function(e){var t=new i({multigraph:e.isMultigraph()}).setGraph(e.graph());return r.forEach(e.nodes(),(function(n){e.children(n).length||t.setNode(n,e.node(n))})),r.forEach(e.edges(),(function(n){t.setEdge(n,e.edge(n))})),t},successorWeights:function(e){var t=r.map(e.nodes(),(function(t){var n={};return r.forEach(e.outEdges(t),(function(t){n[t.w]=(n[t.w]||0)+e.edge(t).weight})),n}));return r.zipObject(e.nodes(),t)},predecessorWeights:function(e){var t=r.map(e.nodes(),(function(t){var n={};return r.forEach(e.inEdges(t),(function(t){n[t.v]=(n[t.v]||0)+e.edge(t).weight})),n}));return r.zipObject(e.nodes(),t)},intersectRect:function(e,t){var n,r,i=e.x,o=e.y,a=t.x-i,s=t.y-o,l=e.width/2,u=e.height/2;if(!a&&!s)throw new Error("Not possible to find intersection inside of the rectangle");Math.abs(s)*l>Math.abs(a)*u?(s<0&&(u=-u),n=u*a/s,r=u):(a<0&&(l=-l),n=l,r=l*s/a);return{x:i+n,y:o+r}},buildLayerMatrix:function(e){var t=r.map(r.range(a(e)+1),(function(){return[]}));return r.forEach(e.nodes(),(function(n){var i=e.node(n),o=i.rank;r.isUndefined(o)||(t[o][i.order]=n)})),t},normalizeRanks:function(e){var t=r.min(r.map(e.nodes(),(function(t){return e.node(t).rank})));r.forEach(e.nodes(),(function(n){var i=e.node(n);r.has(i,"rank")&&(i.rank-=t)}))},removeEmptyRanks:function(e){var t=r.min(r.map(e.nodes(),(function(t){return e.node(t).rank}))),n=[];r.forEach(e.nodes(),(function(r){var i=e.node(r).rank-t;n[i]||(n[i]=[]),n[i].push(r)}));var i=0,o=e.graph().nodeRankFactor;r.forEach(n,(function(t,n){r.isUndefined(t)&&n%o!=0?--i:i&&r.forEach(t,(function(t){e.node(t).rank+=i}))}))},addBorderNode:function(e,t,n,r){var i={width:0,height:0};arguments.length>=4&&(i.rank=n,i.order=r);return o(e,"border",i,t)},maxRank:a,partition:function(e,t){var n={lhs:[],rhs:[]};return r.forEach(e,(function(e){t(e)?n.lhs.push(e):n.rhs.push(e)})),n},time:function(e,t){var n=r.now();try{return t()}finally{console.log(e+" time: "+(r.now()-n)+"ms")}},notime:function(e,t){return t()}}},function(e,t,n){var r=n(8);e.exports=function(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?Object(arguments[t]):{},i=Object.keys(n);"function"==typeof Object.getOwnPropertySymbols&&(i=i.concat(Object.getOwnPropertySymbols(n).filter((function(e){return Object.getOwnPropertyDescriptor(n,e).enumerable})))),i.forEach((function(t){r(e,t,n[t])}))}return e}},function(e,t,n){"use strict";n.r(t);var r=n(1308),i=n(2349),o=n(653),a=n(1313),s=n(654),l=n(2373),u=n(2374),c=n(2350),d=n(318),h=n(315),f=n(2375),p=n(277),g=n(2351),v=n(367);n.d(t,"createGenerateClassName",(function(){return r.a})),n.d(t,"createStyles",(function(){return i.a})),n.d(t,"getThemeProps",(function(){return o.a})),n.d(t,"jssPreset",(function(){return a.a})),n.d(t,"makeStyles",(function(){return s.a})),n.d(t,"mergeClasses",(function(){return l.a})),n.d(t,"ServerStyleSheets",(function(){return u.a})),n.d(t,"styled",(function(){return c.a})),n.d(t,"StylesProvider",(function(){return d.b})),n.d(t,"sheetsManager",(function(){return h.b})),n.d(t,"StylesContext",(function(){return h.a})),n.d(t,"ThemeProvider",(function(){return f.a})),n.d(t,"useTheme",(function(){return p.a})),n.d(t,"withStyles",(function(){return g.a})),n.d(t,"withTheme",(function(){return v.a})),n.d(t,"withThemeCreator",(function(){return v.b}))},function(e,t,n){"use strict";e.exports=n(1564)},function(e,t){e.exports=function(e,t,n,r){var i=n?n.call(r,e,t):void 0;if(void 0!==i)return!!i;if(e===t)return!0;if("object"!=typeof e||!e||"object"!=typeof t||!t)return!1;var o=Object.keys(e),a=Object.keys(t);if(o.length!==a.length)return!1;for(var s=Object.prototype.hasOwnProperty.bind(t),l=0;l<o.length;l++){var u=o[l];if(!s(u))return!1;var c=e[u],d=t[u];if(!1===(i=n?n.call(r,c,d,u):void 0)||void 0===i&&c!==d)return!1}return!0}},function(e,t,n){"use strict";n.r(t),n.d(t,"__DO_NOT_USE__ActionTypes",(function(){return o})),n.d(t,"applyMiddleware",(function(){return v})),n.d(t,"bindActionCreators",(function(){return d})),n.d(t,"combineReducers",(function(){return u})),n.d(t,"compose",(function(){return g})),n.d(t,"createStore",(function(){return s}));var r=n(643),i=function(){return Math.random().toString(36).substring(7).split("").join(".")},o={INIT:"@@redux/INIT"+i(),REPLACE:"@@redux/REPLACE"+i(),PROBE_UNKNOWN_ACTION:function(){return"@@redux/PROBE_UNKNOWN_ACTION"+i()}};function a(e){if("object"!=typeof e||null===e)return!1;for(var t=e;null!==Object.getPrototypeOf(t);)t=Object.getPrototypeOf(t);return Object.getPrototypeOf(e)===t}function s(e,t,n){var i;if("function"==typeof t&&"function"==typeof n||"function"==typeof n&&"function"==typeof arguments[3])throw new Error("It looks like you are passing several store enhancers to createStore(). This is not supported. Instead, compose them together to a single function.");if("function"==typeof t&&void 0===n&&(n=t,t=void 0),void 0!==n){if("function"!=typeof n)throw new Error("Expected the enhancer to be a function.");return n(s)(e,t)}if("function"!=typeof e)throw new Error("Expected the reducer to be a function.");var l=e,u=t,c=[],d=c,h=!1;function f(){d===c&&(d=c.slice())}function p(){if(h)throw new Error("You may not call store.getState() while the reducer is executing. The reducer has already received the state as an argument. Pass it down from the top reducer instead of reading it from the store.");return u}function g(e){if("function"!=typeof e)throw new Error("Expected the listener to be a function.");if(h)throw new Error("You may not call store.subscribe() while the reducer is executing. If you would like to be notified after the store has been updated, subscribe from a component and invoke store.getState() in the callback to access the latest state. See https://redux.js.org/api-reference/store#subscribelistener for more details.");var t=!0;return f(),d.push(e),function(){if(t){if(h)throw new Error("You may not unsubscribe from a store listener while the reducer is executing. See https://redux.js.org/api-reference/store#subscribelistener for more details.");t=!1,f();var n=d.indexOf(e);d.splice(n,1),c=null}}}function v(e){if(!a(e))throw new Error("Actions must be plain objects. Use custom middleware for async actions.");if(void 0===e.type)throw new Error('Actions may not have an undefined "type" property. Have you misspelled a constant?');if(h)throw new Error("Reducers may not dispatch actions.");try{h=!0,u=l(u,e)}finally{h=!1}for(var t=c=d,n=0;n<t.length;n++){(0,t[n])()}return e}function m(e){if("function"!=typeof e)throw new Error("Expected the nextReducer to be a function.");l=e,v({type:o.REPLACE})}function y(){var e,t=g;return(e={subscribe:function(e){if("object"!=typeof e||null===e)throw new TypeError("Expected the observer to be an object.");function n(){e.next&&e.next(p())}return n(),{unsubscribe:t(n)}}})[r.a]=function(){return this},e}return v({type:o.INIT}),(i={dispatch:v,subscribe:g,getState:p,replaceReducer:m})[r.a]=y,i}function l(e,t){var n=t&&t.type;return"Given "+(n&&'action "'+String(n)+'"'||"an action")+', reducer "'+e+'" returned undefined. To ignore an action, you must explicitly return the previous state. If you want this reducer to hold no value, you can return null instead of undefined.'}function u(e){for(var t=Object.keys(e),n={},r=0;r<t.length;r++){var i=t[r];0,"function"==typeof e[i]&&(n[i]=e[i])}var a,s=Object.keys(n);try{!function(e){Object.keys(e).forEach((function(t){var n=e[t];if(void 0===n(void 0,{type:o.INIT}))throw new Error('Reducer "'+t+"\" returned undefined during initialization. If the state passed to the reducer is undefined, you must explicitly return the initial state. The initial state may not be undefined. If you don't want to set a value for this reducer, you can use null instead of undefined.");if(void 0===n(void 0,{type:o.PROBE_UNKNOWN_ACTION()}))throw new Error('Reducer "'+t+"\" returned undefined when probed with a random type. Don't try to handle "+o.INIT+' or other actions in "redux/*" namespace. They are considered private. Instead, you must return the current state for any unknown actions, unless it is undefined, in which case you must return the initial state, regardless of the action type. The initial state may not be undefined, but can be null.')}))}(n)}catch(e){a=e}return function(e,t){if(void 0===e&&(e={}),a)throw a;for(var r=!1,i={},o=0;o<s.length;o++){var u=s[o],c=n[u],d=e[u],h=c(d,t);if(void 0===h){var f=l(u,t);throw new Error(f)}i[u]=h,r=r||h!==d}return(r=r||s.length!==Object.keys(e).length)?i:e}}function c(e,t){return function(){return t(e.apply(this,arguments))}}function d(e,t){if("function"==typeof e)return c(e,t);if("object"!=typeof e||null===e)throw new Error("bindActionCreators expected an object or a function, instead received "+(null===e?"null":typeof e)+'. Did you write "import ActionCreators from" instead of "import * as ActionCreators from"?');var n={};for(var r in e){var i=e[r];"function"==typeof i&&(n[r]=c(i,t))}return n}function h(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function f(e,t){var n=Object.keys(e);return Object.getOwnPropertySymbols&&n.push.apply(n,Object.getOwnPropertySymbols(e)),t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),n}function p(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?f(n,!0).forEach((function(t){h(e,t,n[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):f(n).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))}))}return e}function g(){for(var e=arguments.length,t=new Array(e),n=0;n<e;n++)t[n]=arguments[n];return 0===t.length?function(e){return e}:1===t.length?t[0]:t.reduce((function(e,t){return function(){return e(t.apply(void 0,arguments))}}))}function v(){for(var e=arguments.length,t=new Array(e),n=0;n<e;n++)t[n]=arguments[n];return function(e){return function(){var n=e.apply(void 0,arguments),r=function(){throw new Error("Dispatching while constructing your middleware is not allowed. Other middleware would not be applied to this dispatch.")},i={getState:n.getState,dispatch:function(){return r.apply(void 0,arguments)}},o=t.map((function(e){return e(i)}));return p({},n,{dispatch:r=g.apply(void 0,o)(n.dispatch)})}}}},function(e,t,n){"use strict";n.d(t,"d",(function(){return r})),n.d(t,"a",(function(){return i})),n.d(t,"b",(function(){return o})),n.d(t,"e",(function(){return a})),n.d(t,"c",(function(){return s}));n(17);function r(e){return"execute_result"===e.output_type}function i(e){return"display_data"===e.output_type}function o(e){return"update_display_data"===e.output_type}function a(e){return"stream"===e.output_type}function s(e){return"error"===e.output_type}},function(e,t,n){"use strict";n.r(t);var r=n(921);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";n.r(t);var r=n(928);n.d(t,"default",(function(){return r.a}))},function(e,t,n){e.exports=function(){"use strict";var e=navigator.userAgent,t=navigator.platform,n=/gecko\/\d/i.test(e),r=/MSIE \d/.test(e),i=/Trident\/(?:[7-9]|\d{2,})\..*rv:(\d+)/.exec(e),o=/Edge\/(\d+)/.exec(e),a=r||i||o,s=a&&(r?document.documentMode||6:+(o||i)[1]),l=!o&&/WebKit\//.test(e),u=l&&/Qt\/\d+\.\d+/.test(e),c=!o&&/Chrome\//.test(e),d=/Opera\//.test(e),h=/Apple Computer/.test(navigator.vendor),f=/Mac OS X 1\d\D([8-9]|\d\d)\D/.test(e),p=/PhantomJS/.test(e),g=h&&(/Mobile\/\w+/.test(e)||navigator.maxTouchPoints>2),v=/Android/.test(e),m=g||v||/webOS|BlackBerry|Opera Mini|Opera Mobi|IEMobile/i.test(e),y=g||/Mac/.test(t),b=/\bCrOS\b/.test(e),C=/win/i.test(t),A=d&&e.match(/Version\/(\d*\.\d*)/);A&&(A=Number(A[1])),A&&A>=15&&(d=!1,l=!0);var _=y&&(u||d&&(null==A||A<12.11)),I=n||a&&s>=9;function x(e){return new RegExp("(^|\\s)"+e+"(?:$|\\s)\\s*")}var w,M=function(e,t){var n=e.className,r=x(t).exec(n);if(r){var i=n.slice(r.index+r[0].length);e.className=n.slice(0,r.index)+(i?r[1]+i:"")}};function S(e){for(var t=e.childNodes.length;t>0;--t)e.removeChild(e.firstChild);return e}function T(e,t){return S(e).appendChild(t)}function E(e,t,n,r){var i=document.createElement(e);if(n&&(i.className=n),r&&(i.style.cssText=r),"string"==typeof t)i.appendChild(document.createTextNode(t));else if(t)for(var o=0;o<t.length;++o)i.appendChild(t[o]);return i}function O(e,t,n,r){var i=E(e,t,n,r);return i.setAttribute("role","presentation"),i}function k(e,t){if(3==t.nodeType&&(t=t.parentNode),e.contains)return e.contains(t);do{if(11==t.nodeType&&(t=t.host),t==e)return!0}while(t=t.parentNode)}function L(){var e;try{e=document.activeElement}catch(t){e=document.body||null}for(;e&&e.shadowRoot&&e.shadowRoot.activeElement;)e=e.shadowRoot.activeElement;return e}function D(e,t){var n=e.className;x(t).test(n)||(e.className+=(n?" ":"")+t)}function P(e,t){for(var n=e.split(" "),r=0;r<n.length;r++)n[r]&&!x(n[r]).test(t)&&(t+=" "+n[r]);return t}w=document.createRange?function(e,t,n,r){var i=document.createRange();return i.setEnd(r||e,n),i.setStart(e,t),i}:function(e,t,n){var r=document.body.createTextRange();try{r.moveToElementText(e.parentNode)}catch(e){return r}return r.collapse(!0),r.moveEnd("character",n),r.moveStart("character",t),r};var N=function(e){e.select()};function j(e){var t=Array.prototype.slice.call(arguments,1);return function(){return e.apply(null,t)}}function R(e,t,n){for(var r in t||(t={}),e)!e.hasOwnProperty(r)||!1===n&&t.hasOwnProperty(r)||(t[r]=e[r]);return t}function z(e,t,n,r,i){null==t&&-1==(t=e.search(/[^\s\u00a0]/))&&(t=e.length);for(var o=r||0,a=i||0;;){var s=e.indexOf("\t",o);if(s<0||s>=t)return a+(t-o);a+=s-o,a+=n-a%n,o=s+1}}g?N=function(e){e.selectionStart=0,e.selectionEnd=e.value.length}:a&&(N=function(e){try{e.select()}catch(e){}});var F=function(){this.id=null,this.f=null,this.time=0,this.handler=j(this.onTimeout,this)};function B(e,t){for(var n=0;n<e.length;++n)if(e[n]==t)return n;return-1}F.prototype.onTimeout=function(e){e.id=0,e.time<=+new Date?e.f():setTimeout(e.handler,e.time-+new Date)},F.prototype.set=function(e,t){this.f=t;var n=+new Date+e;(!this.id||n<this.time)&&(clearTimeout(this.id),this.id=setTimeout(this.handler,e),this.time=n)};var H={toString:function(){return"CodeMirror.Pass"}},U={scroll:!1},V={origin:"*mouse"},Y={origin:"+move"};function W(e,t,n){for(var r=0,i=0;;){var o=e.indexOf("\t",r);-1==o&&(o=e.length);var a=o-r;if(o==e.length||i+a>=t)return r+Math.min(a,t-i);if(i+=o-r,r=o+1,(i+=n-i%n)>=t)return r}}var G=[""];function X(e){for(;G.length<=e;)G.push(q(G)+" ");return G[e]}function q(e){return e[e.length-1]}function Z(e,t){for(var n=[],r=0;r<e.length;r++)n[r]=t(e[r],r);return n}function Q(){}function K(e,t){var n;return Object.create?n=Object.create(e):(Q.prototype=e,n=new Q),t&&R(t,n),n}var J=/[\u00df\u0587\u0590-\u05f4\u0600-\u06ff\u3040-\u309f\u30a0-\u30ff\u3400-\u4db5\u4e00-\u9fcc\uac00-\ud7af]/;function $(e){return/\w/.test(e)||e>""&&(e.toUpperCase()!=e.toLowerCase()||J.test(e))}function ee(e,t){return t?!!(t.source.indexOf("\\w")>-1&&$(e))||t.test(e):$(e)}function te(e){for(var t in e)if(e.hasOwnProperty(t)&&e[t])return!1;return!0}var ne=/[\u0300-\u036f\u0483-\u0489\u0591-\u05bd\u05bf\u05c1\u05c2\u05c4\u05c5\u05c7\u0610-\u061a\u064b-\u065e\u0670\u06d6-\u06dc\u06de-\u06e4\u06e7\u06e8\u06ea-\u06ed\u0711\u0730-\u074a\u07a6-\u07b0\u07eb-\u07f3\u0816-\u0819\u081b-\u0823\u0825-\u0827\u0829-\u082d\u0900-\u0902\u093c\u0941-\u0948\u094d\u0951-\u0955\u0962\u0963\u0981\u09bc\u09be\u09c1-\u09c4\u09cd\u09d7\u09e2\u09e3\u0a01\u0a02\u0a3c\u0a41\u0a42\u0a47\u0a48\u0a4b-\u0a4d\u0a51\u0a70\u0a71\u0a75\u0a81\u0a82\u0abc\u0ac1-\u0ac5\u0ac7\u0ac8\u0acd\u0ae2\u0ae3\u0b01\u0b3c\u0b3e\u0b3f\u0b41-\u0b44\u0b4d\u0b56\u0b57\u0b62\u0b63\u0b82\u0bbe\u0bc0\u0bcd\u0bd7\u0c3e-\u0c40\u0c46-\u0c48\u0c4a-\u0c4d\u0c55\u0c56\u0c62\u0c63\u0cbc\u0cbf\u0cc2\u0cc6\u0ccc\u0ccd\u0cd5\u0cd6\u0ce2\u0ce3\u0d3e\u0d41-\u0d44\u0d4d\u0d57\u0d62\u0d63\u0dca\u0dcf\u0dd2-\u0dd4\u0dd6\u0ddf\u0e31\u0e34-\u0e3a\u0e47-\u0e4e\u0eb1\u0eb4-\u0eb9\u0ebb\u0ebc\u0ec8-\u0ecd\u0f18\u0f19\u0f35\u0f37\u0f39\u0f71-\u0f7e\u0f80-\u0f84\u0f86\u0f87\u0f90-\u0f97\u0f99-\u0fbc\u0fc6\u102d-\u1030\u1032-\u1037\u1039\u103a\u103d\u103e\u1058\u1059\u105e-\u1060\u1071-\u1074\u1082\u1085\u1086\u108d\u109d\u135f\u1712-\u1714\u1732-\u1734\u1752\u1753\u1772\u1773\u17b7-\u17bd\u17c6\u17c9-\u17d3\u17dd\u180b-\u180d\u18a9\u1920-\u1922\u1927\u1928\u1932\u1939-\u193b\u1a17\u1a18\u1a56\u1a58-\u1a5e\u1a60\u1a62\u1a65-\u1a6c\u1a73-\u1a7c\u1a7f\u1b00-\u1b03\u1b34\u1b36-\u1b3a\u1b3c\u1b42\u1b6b-\u1b73\u1b80\u1b81\u1ba2-\u1ba5\u1ba8\u1ba9\u1c2c-\u1c33\u1c36\u1c37\u1cd0-\u1cd2\u1cd4-\u1ce0\u1ce2-\u1ce8\u1ced\u1dc0-\u1de6\u1dfd-\u1dff\u200c\u200d\u20d0-\u20f0\u2cef-\u2cf1\u2de0-\u2dff\u302a-\u302f\u3099\u309a\ua66f-\ua672\ua67c\ua67d\ua6f0\ua6f1\ua802\ua806\ua80b\ua825\ua826\ua8c4\ua8e0-\ua8f1\ua926-\ua92d\ua947-\ua951\ua980-\ua982\ua9b3\ua9b6-\ua9b9\ua9bc\uaa29-\uaa2e\uaa31\uaa32\uaa35\uaa36\uaa43\uaa4c\uaab0\uaab2-\uaab4\uaab7\uaab8\uaabe\uaabf\uaac1\uabe5\uabe8\uabed\udc00-\udfff\ufb1e\ufe00-\ufe0f\ufe20-\ufe26\uff9e\uff9f]/;function re(e){return e.charCodeAt(0)>=768&&ne.test(e)}function ie(e,t,n){for(;(n<0?t>0:t<e.length)&&re(e.charAt(t));)t+=n;return t}function oe(e,t,n){for(var r=t>n?-1:1;;){if(t==n)return t;var i=(t+n)/2,o=r<0?Math.ceil(i):Math.floor(i);if(o==t)return e(o)?t:n;e(o)?n=o:t=o+r}}var ae=null;function se(e,t,n){var r;ae=null;for(var i=0;i<e.length;++i){var o=e[i];if(o.from<t&&o.to>t)return i;o.to==t&&(o.from!=o.to&&"before"==n?r=i:ae=i),o.from==t&&(o.from!=o.to&&"before"!=n?r=i:ae=i)}return null!=r?r:ae}var le=function(){var e=/[\u0590-\u05f4\u0600-\u06ff\u0700-\u08ac]/,t=/[stwN]/,n=/[LRr]/,r=/[Lb1n]/,i=/[1n]/;function o(e,t,n){this.level=e,this.from=t,this.to=n}return function(a,s){var l="ltr"==s?"L":"R";if(0==a.length||"ltr"==s&&!e.test(a))return!1;for(var u,c=a.length,d=[],h=0;h<c;++h)d.push((u=a.charCodeAt(h))<=247?"bbbbbbbbbtstwsbbbbbbbbbbbbbbssstwNN%%%NNNNNN,N,N1111111111NNNNNNNLLLLLLLLLLLLLLLLLLLLLLLLLLNNNNNNLLLLLLLLLLLLLLLLLLLLLLLLLLNNNNbbbbbbsbbbbbbbbbbbbbbbbbbbbbbbbbb,N%%%%NNNNLNNNNN%%11NLNNN1LNNNNNLLLLLLLLLLLLLLLLLLLLLLLNLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLN".charAt(u):1424<=u&&u<=1524?"R":1536<=u&&u<=1785?"nnnnnnNNr%%r,rNNmmmmmmmmmmmrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrmmmmmmmmmmmmmmmmmmmmmnnnnnnnnnn%nnrrrmrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrmmmmmmmnNmmmmmmrrmmNmmmmrr1111111111".charAt(u-1536):1774<=u&&u<=2220?"r":8192<=u&&u<=8203?"w":8204==u?"b":"L");for(var f=0,p=l;f<c;++f){var g=d[f];"m"==g?d[f]=p:p=g}for(var v=0,m=l;v<c;++v){var y=d[v];"1"==y&&"r"==m?d[v]="n":n.test(y)&&(m=y,"r"==y&&(d[v]="R"))}for(var b=1,C=d[0];b<c-1;++b){var A=d[b];"+"==A&&"1"==C&&"1"==d[b+1]?d[b]="1":","!=A||C!=d[b+1]||"1"!=C&&"n"!=C||(d[b]=C),C=A}for(var _=0;_<c;++_){var I=d[_];if(","==I)d[_]="N";else if("%"==I){var x=void 0;for(x=_+1;x<c&&"%"==d[x];++x);for(var w=_&&"!"==d[_-1]||x<c&&"1"==d[x]?"1":"N",M=_;M<x;++M)d[M]=w;_=x-1}}for(var S=0,T=l;S<c;++S){var E=d[S];"L"==T&&"1"==E?d[S]="L":n.test(E)&&(T=E)}for(var O=0;O<c;++O)if(t.test(d[O])){var k=void 0;for(k=O+1;k<c&&t.test(d[k]);++k);for(var L="L"==(O?d[O-1]:l),D=L==("L"==(k<c?d[k]:l))?L?"L":"R":l,P=O;P<k;++P)d[P]=D;O=k-1}for(var N,j=[],R=0;R<c;)if(r.test(d[R])){var z=R;for(++R;R<c&&r.test(d[R]);++R);j.push(new o(0,z,R))}else{var F=R,B=j.length,H="rtl"==s?1:0;for(++R;R<c&&"L"!=d[R];++R);for(var U=F;U<R;)if(i.test(d[U])){F<U&&(j.splice(B,0,new o(1,F,U)),B+=H);var V=U;for(++U;U<R&&i.test(d[U]);++U);j.splice(B,0,new o(2,V,U)),B+=H,F=U}else++U;F<R&&j.splice(B,0,new o(1,F,R))}return"ltr"==s&&(1==j[0].level&&(N=a.match(/^\s+/))&&(j[0].from=N[0].length,j.unshift(new o(0,0,N[0].length))),1==q(j).level&&(N=a.match(/\s+$/))&&(q(j).to-=N[0].length,j.push(new o(0,c-N[0].length,c)))),"rtl"==s?j.reverse():j}}();function ue(e,t){var n=e.order;return null==n&&(n=e.order=le(e.text,t)),n}var ce=[],de=function(e,t,n){if(e.addEventListener)e.addEventListener(t,n,!1);else if(e.attachEvent)e.attachEvent("on"+t,n);else{var r=e._handlers||(e._handlers={});r[t]=(r[t]||ce).concat(n)}};function he(e,t){return e._handlers&&e._handlers[t]||ce}function fe(e,t,n){if(e.removeEventListener)e.removeEventListener(t,n,!1);else if(e.detachEvent)e.detachEvent("on"+t,n);else{var r=e._handlers,i=r&&r[t];if(i){var o=B(i,n);o>-1&&(r[t]=i.slice(0,o).concat(i.slice(o+1)))}}}function pe(e,t){var n=he(e,t);if(n.length)for(var r=Array.prototype.slice.call(arguments,2),i=0;i<n.length;++i)n[i].apply(null,r)}function ge(e,t,n){return"string"==typeof t&&(t={type:t,preventDefault:function(){this.defaultPrevented=!0}}),pe(e,n||t.type,e,t),Ae(t)||t.codemirrorIgnore}function ve(e){var t=e._handlers&&e._handlers.cursorActivity;if(t)for(var n=e.curOp.cursorActivityHandlers||(e.curOp.cursorActivityHandlers=[]),r=0;r<t.length;++r)-1==B(n,t[r])&&n.push(t[r])}function me(e,t){return he(e,t).length>0}function ye(e){e.prototype.on=function(e,t){de(this,e,t)},e.prototype.off=function(e,t){fe(this,e,t)}}function be(e){e.preventDefault?e.preventDefault():e.returnValue=!1}function Ce(e){e.stopPropagation?e.stopPropagation():e.cancelBubble=!0}function Ae(e){return null!=e.defaultPrevented?e.defaultPrevented:0==e.returnValue}function _e(e){be(e),Ce(e)}function Ie(e){return e.target||e.srcElement}function xe(e){var t=e.which;return null==t&&(1&e.button?t=1:2&e.button?t=3:4&e.button&&(t=2)),y&&e.ctrlKey&&1==t&&(t=3),t}var we,Me,Se=function(){if(a&&s<9)return!1;var e=E("div");return"draggable"in e||"dragDrop"in e}();function Te(e){if(null==we){var t=E("span","​");T(e,E("span",[t,document.createTextNode("x")])),0!=e.firstChild.offsetHeight&&(we=t.offsetWidth<=1&&t.offsetHeight>2&&!(a&&s<8))}var n=we?E("span","​"):E("span"," ",null,"display: inline-block; width: 1px; margin-right: -1px");return n.setAttribute("cm-text",""),n}function Ee(e){if(null!=Me)return Me;var t=T(e,document.createTextNode("AخA")),n=w(t,0,1).getBoundingClientRect(),r=w(t,1,2).getBoundingClientRect();return S(e),!(!n||n.left==n.right)&&(Me=r.right-n.right<3)}var Oe,ke=3!="\n\nb".split(/\n/).length?function(e){for(var t=0,n=[],r=e.length;t<=r;){var i=e.indexOf("\n",t);-1==i&&(i=e.length);var o=e.slice(t,"\r"==e.charAt(i-1)?i-1:i),a=o.indexOf("\r");-1!=a?(n.push(o.slice(0,a)),t+=a+1):(n.push(o),t=i+1)}return n}:function(e){return e.split(/\r\n?|\n/)},Le=window.getSelection?function(e){try{return e.selectionStart!=e.selectionEnd}catch(e){return!1}}:function(e){var t;try{t=e.ownerDocument.selection.createRange()}catch(e){}return!(!t||t.parentElement()!=e)&&0!=t.compareEndPoints("StartToEnd",t)},De="oncopy"in(Oe=E("div"))||(Oe.setAttribute("oncopy","return;"),"function"==typeof Oe.oncopy),Pe=null,Ne={},je={};function Re(e,t){arguments.length>2&&(t.dependencies=Array.prototype.slice.call(arguments,2)),Ne[e]=t}function ze(e){if("string"==typeof e&&je.hasOwnProperty(e))e=je[e];else if(e&&"string"==typeof e.name&&je.hasOwnProperty(e.name)){var t=je[e.name];"string"==typeof t&&(t={name:t}),(e=K(t,e)).name=t.name}else{if("string"==typeof e&&/^[\w\-]+\/[\w\-]+\+xml$/.test(e))return ze("application/xml");if("string"==typeof e&&/^[\w\-]+\/[\w\-]+\+json$/.test(e))return ze("application/json")}return"string"==typeof e?{name:e}:e||{name:"null"}}function Fe(e,t){t=ze(t);var n=Ne[t.name];if(!n)return Fe(e,"text/plain");var r=n(e,t);if(Be.hasOwnProperty(t.name)){var i=Be[t.name];for(var o in i)i.hasOwnProperty(o)&&(r.hasOwnProperty(o)&&(r["_"+o]=r[o]),r[o]=i[o])}if(r.name=t.name,t.helperType&&(r.helperType=t.helperType),t.modeProps)for(var a in t.modeProps)r[a]=t.modeProps[a];return r}var Be={};function He(e,t){R(t,Be.hasOwnProperty(e)?Be[e]:Be[e]={})}function Ue(e,t){if(!0===t)return t;if(e.copyState)return e.copyState(t);var n={};for(var r in t){var i=t[r];i instanceof Array&&(i=i.concat([])),n[r]=i}return n}function Ve(e,t){for(var n;e.innerMode&&(n=e.innerMode(t))&&n.mode!=e;)t=n.state,e=n.mode;return n||{mode:e,state:t}}function Ye(e,t,n){return!e.startState||e.startState(t,n)}var We=function(e,t,n){this.pos=this.start=0,this.string=e,this.tabSize=t||8,this.lastColumnPos=this.lastColumnValue=0,this.lineStart=0,this.lineOracle=n};function Ge(e,t){if((t-=e.first)<0||t>=e.size)throw new Error("There is no line "+(t+e.first)+" in the document.");for(var n=e;!n.lines;)for(var r=0;;++r){var i=n.children[r],o=i.chunkSize();if(t<o){n=i;break}t-=o}return n.lines[t]}function Xe(e,t,n){var r=[],i=t.line;return e.iter(t.line,n.line+1,(function(e){var o=e.text;i==n.line&&(o=o.slice(0,n.ch)),i==t.line&&(o=o.slice(t.ch)),r.push(o),++i})),r}function qe(e,t,n){var r=[];return e.iter(t,n,(function(e){r.push(e.text)})),r}function Ze(e,t){var n=t-e.height;if(n)for(var r=e;r;r=r.parent)r.height+=n}function Qe(e){if(null==e.parent)return null;for(var t=e.parent,n=B(t.lines,e),r=t.parent;r;t=r,r=r.parent)for(var i=0;r.children[i]!=t;++i)n+=r.children[i].chunkSize();return n+t.first}function Ke(e,t){var n=e.first;e:do{for(var r=0;r<e.children.length;++r){var i=e.children[r],o=i.height;if(t<o){e=i;continue e}t-=o,n+=i.chunkSize()}return n}while(!e.lines);for(var a=0;a<e.lines.length;++a){var s=e.lines[a].height;if(t<s)break;t-=s}return n+a}function Je(e,t){return t>=e.first&&t<e.first+e.size}function $e(e,t){return String(e.lineNumberFormatter(t+e.firstLineNumber))}function et(e,t,n){if(void 0===n&&(n=null),!(this instanceof et))return new et(e,t,n);this.line=e,this.ch=t,this.sticky=n}function tt(e,t){return e.line-t.line||e.ch-t.ch}function nt(e,t){return e.sticky==t.sticky&&0==tt(e,t)}function rt(e){return et(e.line,e.ch)}function it(e,t){return tt(e,t)<0?t:e}function ot(e,t){return tt(e,t)<0?e:t}function at(e,t){return Math.max(e.first,Math.min(t,e.first+e.size-1))}function st(e,t){if(t.line<e.first)return et(e.first,0);var n=e.first+e.size-1;return t.line>n?et(n,Ge(e,n).text.length):function(e,t){var n=e.ch;return null==n||n>t?et(e.line,t):n<0?et(e.line,0):e}(t,Ge(e,t.line).text.length)}function lt(e,t){for(var n=[],r=0;r<t.length;r++)n[r]=st(e,t[r]);return n}We.prototype.eol=function(){return this.pos>=this.string.length},We.prototype.sol=function(){return this.pos==this.lineStart},We.prototype.peek=function(){return this.string.charAt(this.pos)||void 0},We.prototype.next=function(){if(this.pos<this.string.length)return this.string.charAt(this.pos++)},We.prototype.eat=function(e){var t=this.string.charAt(this.pos);if("string"==typeof e?t==e:t&&(e.test?e.test(t):e(t)))return++this.pos,t},We.prototype.eatWhile=function(e){for(var t=this.pos;this.eat(e););return this.pos>t},We.prototype.eatSpace=function(){for(var e=this.pos;/[\s\u00a0]/.test(this.string.charAt(this.pos));)++this.pos;return this.pos>e},We.prototype.skipToEnd=function(){this.pos=this.string.length},We.prototype.skipTo=function(e){var t=this.string.indexOf(e,this.pos);if(t>-1)return this.pos=t,!0},We.prototype.backUp=function(e){this.pos-=e},We.prototype.column=function(){return this.lastColumnPos<this.start&&(this.lastColumnValue=z(this.string,this.start,this.tabSize,this.lastColumnPos,this.lastColumnValue),this.lastColumnPos=this.start),this.lastColumnValue-(this.lineStart?z(this.string,this.lineStart,this.tabSize):0)},We.prototype.indentation=function(){return z(this.string,null,this.tabSize)-(this.lineStart?z(this.string,this.lineStart,this.tabSize):0)},We.prototype.match=function(e,t,n){if("string"!=typeof e){var r=this.string.slice(this.pos).match(e);return r&&r.index>0?null:(r&&!1!==t&&(this.pos+=r[0].length),r)}var i=function(e){return n?e.toLowerCase():e};if(i(this.string.substr(this.pos,e.length))==i(e))return!1!==t&&(this.pos+=e.length),!0},We.prototype.current=function(){return this.string.slice(this.start,this.pos)},We.prototype.hideFirstChars=function(e,t){this.lineStart+=e;try{return t()}finally{this.lineStart-=e}},We.prototype.lookAhead=function(e){var t=this.lineOracle;return t&&t.lookAhead(e)},We.prototype.baseToken=function(){var e=this.lineOracle;return e&&e.baseToken(this.pos)};var ut=function(e,t){this.state=e,this.lookAhead=t},ct=function(e,t,n,r){this.state=t,this.doc=e,this.line=n,this.maxLookAhead=r||0,this.baseTokens=null,this.baseTokenPos=1};function dt(e,t,n,r){var i=[e.state.modeGen],o={};Ct(e,t.text,e.doc.mode,n,(function(e,t){return i.push(e,t)}),o,r);for(var a=n.state,s=function(r){n.baseTokens=i;var s=e.state.overlays[r],l=1,u=0;n.state=!0,Ct(e,t.text,s.mode,n,(function(e,t){for(var n=l;u<e;){var r=i[l];r>e&&i.splice(l,1,e,i[l+1],r),l+=2,u=Math.min(e,r)}if(t)if(s.opaque)i.splice(n,l-n,e,"overlay "+t),l=n+2;else for(;n<l;n+=2){var o=i[n+1];i[n+1]=(o?o+" ":"")+"overlay "+t}}),o),n.state=a,n.baseTokens=null,n.baseTokenPos=1},l=0;l<e.state.overlays.length;++l)s(l);return{styles:i,classes:o.bgClass||o.textClass?o:null}}function ht(e,t,n){if(!t.styles||t.styles[0]!=e.state.modeGen){var r=ft(e,Qe(t)),i=t.text.length>e.options.maxHighlightLength&&Ue(e.doc.mode,r.state),o=dt(e,t,r);i&&(r.state=i),t.stateAfter=r.save(!i),t.styles=o.styles,o.classes?t.styleClasses=o.classes:t.styleClasses&&(t.styleClasses=null),n===e.doc.highlightFrontier&&(e.doc.modeFrontier=Math.max(e.doc.modeFrontier,++e.doc.highlightFrontier))}return t.styles}function ft(e,t,n){var r=e.doc,i=e.display;if(!r.mode.startState)return new ct(r,!0,t);var o=function(e,t,n){for(var r,i,o=e.doc,a=n?-1:t-(e.doc.mode.innerMode?1e3:100),s=t;s>a;--s){if(s<=o.first)return o.first;var l=Ge(o,s-1),u=l.stateAfter;if(u&&(!n||s+(u instanceof ut?u.lookAhead:0)<=o.modeFrontier))return s;var c=z(l.text,null,e.options.tabSize);(null==i||r>c)&&(i=s-1,r=c)}return i}(e,t,n),a=o>r.first&&Ge(r,o-1).stateAfter,s=a?ct.fromSaved(r,a,o):new ct(r,Ye(r.mode),o);return r.iter(o,t,(function(n){pt(e,n.text,s);var r=s.line;n.stateAfter=r==t-1||r%5==0||r>=i.viewFrom&&r<i.viewTo?s.save():null,s.nextLine()})),n&&(r.modeFrontier=s.line),s}function pt(e,t,n,r){var i=e.doc.mode,o=new We(t,e.options.tabSize,n);for(o.start=o.pos=r||0,""==t&&gt(i,n.state);!o.eol();)vt(i,o,n.state),o.start=o.pos}function gt(e,t){if(e.blankLine)return e.blankLine(t);if(e.innerMode){var n=Ve(e,t);return n.mode.blankLine?n.mode.blankLine(n.state):void 0}}function vt(e,t,n,r){for(var i=0;i<10;i++){r&&(r[0]=Ve(e,n).mode);var o=e.token(t,n);if(t.pos>t.start)return o}throw new Error("Mode "+e.name+" failed to advance stream.")}ct.prototype.lookAhead=function(e){var t=this.doc.getLine(this.line+e);return null!=t&&e>this.maxLookAhead&&(this.maxLookAhead=e),t},ct.prototype.baseToken=function(e){if(!this.baseTokens)return null;for(;this.baseTokens[this.baseTokenPos]<=e;)this.baseTokenPos+=2;var t=this.baseTokens[this.baseTokenPos+1];return{type:t&&t.replace(/( |^)overlay .*/,""),size:this.baseTokens[this.baseTokenPos]-e}},ct.prototype.nextLine=function(){this.line++,this.maxLookAhead>0&&this.maxLookAhead--},ct.fromSaved=function(e,t,n){return t instanceof ut?new ct(e,Ue(e.mode,t.state),n,t.lookAhead):new ct(e,Ue(e.mode,t),n)},ct.prototype.save=function(e){var t=!1!==e?Ue(this.doc.mode,this.state):this.state;return this.maxLookAhead>0?new ut(t,this.maxLookAhead):t};var mt=function(e,t,n){this.start=e.start,this.end=e.pos,this.string=e.current(),this.type=t||null,this.state=n};function yt(e,t,n,r){var i,o,a=e.doc,s=a.mode,l=Ge(a,(t=st(a,t)).line),u=ft(e,t.line,n),c=new We(l.text,e.options.tabSize,u);for(r&&(o=[]);(r||c.pos<t.ch)&&!c.eol();)c.start=c.pos,i=vt(s,c,u.state),r&&o.push(new mt(c,i,Ue(a.mode,u.state)));return r?o:new mt(c,i,u.state)}function bt(e,t){if(e)for(;;){var n=e.match(/(?:^|\s+)line-(background-)?(\S+)/);if(!n)break;e=e.slice(0,n.index)+e.slice(n.index+n[0].length);var r=n[1]?"bgClass":"textClass";null==t[r]?t[r]=n[2]:new RegExp("(?:^|\\s)"+n[2]+"(?:$|\\s)").test(t[r])||(t[r]+=" "+n[2])}return e}function Ct(e,t,n,r,i,o,a){var s=n.flattenSpans;null==s&&(s=e.options.flattenSpans);var l,u=0,c=null,d=new We(t,e.options.tabSize,r),h=e.options.addModeClass&&[null];for(""==t&&bt(gt(n,r.state),o);!d.eol();){if(d.pos>e.options.maxHighlightLength?(s=!1,a&&pt(e,t,r,d.pos),d.pos=t.length,l=null):l=bt(vt(n,d,r.state,h),o),h){var f=h[0].name;f&&(l="m-"+(l?f+" "+l:f))}if(!s||c!=l){for(;u<d.start;)i(u=Math.min(d.start,u+5e3),c);c=l}d.start=d.pos}for(;u<d.pos;){var p=Math.min(d.pos,u+5e3);i(p,c),u=p}}var At=!1,_t=!1;function It(e,t,n){this.marker=e,this.from=t,this.to=n}function xt(e,t){if(e)for(var n=0;n<e.length;++n){var r=e[n];if(r.marker==t)return r}}function wt(e,t){for(var n,r=0;r<e.length;++r)e[r]!=t&&(n||(n=[])).push(e[r]);return n}function Mt(e,t){if(t.full)return null;var n=Je(e,t.from.line)&&Ge(e,t.from.line).markedSpans,r=Je(e,t.to.line)&&Ge(e,t.to.line).markedSpans;if(!n&&!r)return null;var i=t.from.ch,o=t.to.ch,a=0==tt(t.from,t.to),s=function(e,t,n){var r;if(e)for(var i=0;i<e.length;++i){var o=e[i],a=o.marker;if(null==o.from||(a.inclusiveLeft?o.from<=t:o.from<t)||o.from==t&&"bookmark"==a.type&&(!n||!o.marker.insertLeft)){var s=null==o.to||(a.inclusiveRight?o.to>=t:o.to>t);(r||(r=[])).push(new It(a,o.from,s?null:o.to))}}return r}(n,i,a),l=function(e,t,n){var r;if(e)for(var i=0;i<e.length;++i){var o=e[i],a=o.marker;if(null==o.to||(a.inclusiveRight?o.to>=t:o.to>t)||o.from==t&&"bookmark"==a.type&&(!n||o.marker.insertLeft)){var s=null==o.from||(a.inclusiveLeft?o.from<=t:o.from<t);(r||(r=[])).push(new It(a,s?null:o.from-t,null==o.to?null:o.to-t))}}return r}(r,o,a),u=1==t.text.length,c=q(t.text).length+(u?i:0);if(s)for(var d=0;d<s.length;++d){var h=s[d];if(null==h.to){var f=xt(l,h.marker);f?u&&(h.to=null==f.to?null:f.to+c):h.to=i}}if(l)for(var p=0;p<l.length;++p){var g=l[p];null!=g.to&&(g.to+=c),null==g.from?xt(s,g.marker)||(g.from=c,u&&(s||(s=[])).push(g)):(g.from+=c,u&&(s||(s=[])).push(g))}s&&(s=St(s)),l&&l!=s&&(l=St(l));var v=[s];if(!u){var m,y=t.text.length-2;if(y>0&&s)for(var b=0;b<s.length;++b)null==s[b].to&&(m||(m=[])).push(new It(s[b].marker,null,null));for(var C=0;C<y;++C)v.push(m);v.push(l)}return v}function St(e){for(var t=0;t<e.length;++t){var n=e[t];null!=n.from&&n.from==n.to&&!1!==n.marker.clearWhenEmpty&&e.splice(t--,1)}return e.length?e:null}function Tt(e){var t=e.markedSpans;if(t){for(var n=0;n<t.length;++n)t[n].marker.detachLine(e);e.markedSpans=null}}function Et(e,t){if(t){for(var n=0;n<t.length;++n)t[n].marker.attachLine(e);e.markedSpans=t}}function Ot(e){return e.inclusiveLeft?-1:0}function kt(e){return e.inclusiveRight?1:0}function Lt(e,t){var n=e.lines.length-t.lines.length;if(0!=n)return n;var r=e.find(),i=t.find(),o=tt(r.from,i.from)||Ot(e)-Ot(t);if(o)return-o;var a=tt(r.to,i.to)||kt(e)-kt(t);return a||t.id-e.id}function Dt(e,t){var n,r=_t&&e.markedSpans;if(r)for(var i=void 0,o=0;o<r.length;++o)(i=r[o]).marker.collapsed&&null==(t?i.from:i.to)&&(!n||Lt(n,i.marker)<0)&&(n=i.marker);return n}function Pt(e){return Dt(e,!0)}function Nt(e){return Dt(e,!1)}function jt(e,t){var n,r=_t&&e.markedSpans;if(r)for(var i=0;i<r.length;++i){var o=r[i];o.marker.collapsed&&(null==o.from||o.from<t)&&(null==o.to||o.to>t)&&(!n||Lt(n,o.marker)<0)&&(n=o.marker)}return n}function Rt(e,t,n,r,i){var o=Ge(e,t),a=_t&&o.markedSpans;if(a)for(var s=0;s<a.length;++s){var l=a[s];if(l.marker.collapsed){var u=l.marker.find(0),c=tt(u.from,n)||Ot(l.marker)-Ot(i),d=tt(u.to,r)||kt(l.marker)-kt(i);if(!(c>=0&&d<=0||c<=0&&d>=0)&&(c<=0&&(l.marker.inclusiveRight&&i.inclusiveLeft?tt(u.to,n)>=0:tt(u.to,n)>0)||c>=0&&(l.marker.inclusiveRight&&i.inclusiveLeft?tt(u.from,r)<=0:tt(u.from,r)<0)))return!0}}}function zt(e){for(var t;t=Pt(e);)e=t.find(-1,!0).line;return e}function Ft(e,t){var n=Ge(e,t),r=zt(n);return n==r?t:Qe(r)}function Bt(e,t){if(t>e.lastLine())return t;var n,r=Ge(e,t);if(!Ht(e,r))return t;for(;n=Nt(r);)r=n.find(1,!0).line;return Qe(r)+1}function Ht(e,t){var n=_t&&t.markedSpans;if(n)for(var r=void 0,i=0;i<n.length;++i)if((r=n[i]).marker.collapsed){if(null==r.from)return!0;if(!r.marker.widgetNode&&0==r.from&&r.marker.inclusiveLeft&&Ut(e,t,r))return!0}}function Ut(e,t,n){if(null==n.to){var r=n.marker.find(1,!0);return Ut(e,r.line,xt(r.line.markedSpans,n.marker))}if(n.marker.inclusiveRight&&n.to==t.text.length)return!0;for(var i=void 0,o=0;o<t.markedSpans.length;++o)if((i=t.markedSpans[o]).marker.collapsed&&!i.marker.widgetNode&&i.from==n.to&&(null==i.to||i.to!=n.from)&&(i.marker.inclusiveLeft||n.marker.inclusiveRight)&&Ut(e,t,i))return!0}function Vt(e){for(var t=0,n=(e=zt(e)).parent,r=0;r<n.lines.length;++r){var i=n.lines[r];if(i==e)break;t+=i.height}for(var o=n.parent;o;o=(n=o).parent)for(var a=0;a<o.children.length;++a){var s=o.children[a];if(s==n)break;t+=s.height}return t}function Yt(e){if(0==e.height)return 0;for(var t,n=e.text.length,r=e;t=Pt(r);){var i=t.find(0,!0);r=i.from.line,n+=i.from.ch-i.to.ch}for(r=e;t=Nt(r);){var o=t.find(0,!0);n-=r.text.length-o.from.ch,n+=(r=o.to.line).text.length-o.to.ch}return n}function Wt(e){var t=e.display,n=e.doc;t.maxLine=Ge(n,n.first),t.maxLineLength=Yt(t.maxLine),t.maxLineChanged=!0,n.iter((function(e){var n=Yt(e);n>t.maxLineLength&&(t.maxLineLength=n,t.maxLine=e)}))}var Gt=function(e,t,n){this.text=e,Et(this,t),this.height=n?n(this):1};function Xt(e){e.parent=null,Tt(e)}Gt.prototype.lineNo=function(){return Qe(this)},ye(Gt);var qt={},Zt={};function Qt(e,t){if(!e||/^\s*$/.test(e))return null;var n=t.addModeClass?Zt:qt;return n[e]||(n[e]=e.replace(/\S+/g,"cm-$&"))}function Kt(e,t){var n=O("span",null,null,l?"padding-right: .1px":null),r={pre:O("pre",[n],"CodeMirror-line"),content:n,col:0,pos:0,cm:e,trailingSpace:!1,splitSpaces:e.getOption("lineWrapping")};t.measure={};for(var i=0;i<=(t.rest?t.rest.length:0);i++){var o=i?t.rest[i-1]:t.line,a=void 0;r.pos=0,r.addToken=$t,Ee(e.display.measure)&&(a=ue(o,e.doc.direction))&&(r.addToken=en(r.addToken,a)),r.map=[],nn(o,r,ht(e,o,t!=e.display.externalMeasured&&Qe(o))),o.styleClasses&&(o.styleClasses.bgClass&&(r.bgClass=P(o.styleClasses.bgClass,r.bgClass||"")),o.styleClasses.textClass&&(r.textClass=P(o.styleClasses.textClass,r.textClass||""))),0==r.map.length&&r.map.push(0,0,r.content.appendChild(Te(e.display.measure))),0==i?(t.measure.map=r.map,t.measure.cache={}):((t.measure.maps||(t.measure.maps=[])).push(r.map),(t.measure.caches||(t.measure.caches=[])).push({}))}if(l){var s=r.content.lastChild;(/\bcm-tab\b/.test(s.className)||s.querySelector&&s.querySelector(".cm-tab"))&&(r.content.className="cm-tab-wrap-hack")}return pe(e,"renderLine",e,t.line,r.pre),r.pre.className&&(r.textClass=P(r.pre.className,r.textClass||"")),r}function Jt(e){var t=E("span","•","cm-invalidchar");return t.title="\\u"+e.charCodeAt(0).toString(16),t.setAttribute("aria-label",t.title),t}function $t(e,t,n,r,i,o,l){if(t){var u,c=e.splitSpaces?function(e,t){if(e.length>1&&!/  /.test(e))return e;for(var n=t,r="",i=0;i<e.length;i++){var o=e.charAt(i);" "!=o||!n||i!=e.length-1&&32!=e.charCodeAt(i+1)||(o=" "),r+=o,n=" "==o}return r}(t,e.trailingSpace):t,d=e.cm.state.specialChars,h=!1;if(d.test(t)){u=document.createDocumentFragment();for(var f=0;;){d.lastIndex=f;var p=d.exec(t),g=p?p.index-f:t.length-f;if(g){var v=document.createTextNode(c.slice(f,f+g));a&&s<9?u.appendChild(E("span",[v])):u.appendChild(v),e.map.push(e.pos,e.pos+g,v),e.col+=g,e.pos+=g}if(!p)break;f+=g+1;var m=void 0;if("\t"==p[0]){var y=e.cm.options.tabSize,b=y-e.col%y;(m=u.appendChild(E("span",X(b),"cm-tab"))).setAttribute("role","presentation"),m.setAttribute("cm-text","\t"),e.col+=b}else"\r"==p[0]||"\n"==p[0]?((m=u.appendChild(E("span","\r"==p[0]?"␍":"␤","cm-invalidchar"))).setAttribute("cm-text",p[0]),e.col+=1):((m=e.cm.options.specialCharPlaceholder(p[0])).setAttribute("cm-text",p[0]),a&&s<9?u.appendChild(E("span",[m])):u.appendChild(m),e.col+=1);e.map.push(e.pos,e.pos+1,m),e.pos++}}else e.col+=t.length,u=document.createTextNode(c),e.map.push(e.pos,e.pos+t.length,u),a&&s<9&&(h=!0),e.pos+=t.length;if(e.trailingSpace=32==c.charCodeAt(t.length-1),n||r||i||h||o||l){var C=n||"";r&&(C+=r),i&&(C+=i);var A=E("span",[u],C,o);if(l)for(var _ in l)l.hasOwnProperty(_)&&"style"!=_&&"class"!=_&&A.setAttribute(_,l[_]);return e.content.appendChild(A)}e.content.appendChild(u)}}function en(e,t){return function(n,r,i,o,a,s,l){i=i?i+" cm-force-border":"cm-force-border";for(var u=n.pos,c=u+r.length;;){for(var d=void 0,h=0;h<t.length&&!((d=t[h]).to>u&&d.from<=u);h++);if(d.to>=c)return e(n,r,i,o,a,s,l);e(n,r.slice(0,d.to-u),i,o,null,s,l),o=null,r=r.slice(d.to-u),u=d.to}}}function tn(e,t,n,r){var i=!r&&n.widgetNode;i&&e.map.push(e.pos,e.pos+t,i),!r&&e.cm.display.input.needsContentAttribute&&(i||(i=e.content.appendChild(document.createElement("span"))),i.setAttribute("cm-marker",n.id)),i&&(e.cm.display.input.setUneditable(i),e.content.appendChild(i)),e.pos+=t,e.trailingSpace=!1}function nn(e,t,n){var r=e.markedSpans,i=e.text,o=0;if(r)for(var a,s,l,u,c,d,h,f=i.length,p=0,g=1,v="",m=0;;){if(m==p){l=u=c=s="",h=null,d=null,m=1/0;for(var y=[],b=void 0,C=0;C<r.length;++C){var A=r[C],_=A.marker;if("bookmark"==_.type&&A.from==p&&_.widgetNode)y.push(_);else if(A.from<=p&&(null==A.to||A.to>p||_.collapsed&&A.to==p&&A.from==p)){if(null!=A.to&&A.to!=p&&m>A.to&&(m=A.to,u=""),_.className&&(l+=" "+_.className),_.css&&(s=(s?s+";":"")+_.css),_.startStyle&&A.from==p&&(c+=" "+_.startStyle),_.endStyle&&A.to==m&&(b||(b=[])).push(_.endStyle,A.to),_.title&&((h||(h={})).title=_.title),_.attributes)for(var I in _.attributes)(h||(h={}))[I]=_.attributes[I];_.collapsed&&(!d||Lt(d.marker,_)<0)&&(d=A)}else A.from>p&&m>A.from&&(m=A.from)}if(b)for(var x=0;x<b.length;x+=2)b[x+1]==m&&(u+=" "+b[x]);if(!d||d.from==p)for(var w=0;w<y.length;++w)tn(t,0,y[w]);if(d&&(d.from||0)==p){if(tn(t,(null==d.to?f+1:d.to)-p,d.marker,null==d.from),null==d.to)return;d.to==p&&(d=!1)}}if(p>=f)break;for(var M=Math.min(f,m);;){if(v){var S=p+v.length;if(!d){var T=S>M?v.slice(0,M-p):v;t.addToken(t,T,a?a+l:l,c,p+T.length==m?u:"",s,h)}if(S>=M){v=v.slice(M-p),p=M;break}p=S,c=""}v=i.slice(o,o=n[g++]),a=Qt(n[g++],t.cm.options)}}else for(var E=1;E<n.length;E+=2)t.addToken(t,i.slice(o,o=n[E]),Qt(n[E+1],t.cm.options))}function rn(e,t,n){this.line=t,this.rest=function(e){for(var t,n;t=Nt(e);)e=t.find(1,!0).line,(n||(n=[])).push(e);return n}(t),this.size=this.rest?Qe(q(this.rest))-n+1:1,this.node=this.text=null,this.hidden=Ht(e,t)}function on(e,t,n){for(var r,i=[],o=t;o<n;o=r){var a=new rn(e.doc,Ge(e.doc,o),o);r=o+a.size,i.push(a)}return i}var an=null,sn=null;function ln(e,t){var n=he(e,t);if(n.length){var r,i=Array.prototype.slice.call(arguments,2);an?r=an.delayedCallbacks:sn?r=sn:(r=sn=[],setTimeout(un,0));for(var o=function(e){r.push((function(){return n[e].apply(null,i)}))},a=0;a<n.length;++a)o(a)}}function un(){var e=sn;sn=null;for(var t=0;t<e.length;++t)e[t]()}function cn(e,t,n,r){for(var i=0;i<t.changes.length;i++){var o=t.changes[i];"text"==o?fn(e,t):"gutter"==o?gn(e,t,n,r):"class"==o?pn(e,t):"widget"==o&&vn(e,t,r)}t.changes=null}function dn(e){return e.node==e.text&&(e.node=E("div",null,null,"position: relative"),e.text.parentNode&&e.text.parentNode.replaceChild(e.node,e.text),e.node.appendChild(e.text),a&&s<8&&(e.node.style.zIndex=2)),e.node}function hn(e,t){var n=e.display.externalMeasured;return n&&n.line==t.line?(e.display.externalMeasured=null,t.measure=n.measure,n.built):Kt(e,t)}function fn(e,t){var n=t.text.className,r=hn(e,t);t.text==t.node&&(t.node=r.pre),t.text.parentNode.replaceChild(r.pre,t.text),t.text=r.pre,r.bgClass!=t.bgClass||r.textClass!=t.textClass?(t.bgClass=r.bgClass,t.textClass=r.textClass,pn(e,t)):n&&(t.text.className=n)}function pn(e,t){!function(e,t){var n=t.bgClass?t.bgClass+" "+(t.line.bgClass||""):t.line.bgClass;if(n&&(n+=" CodeMirror-linebackground"),t.background)n?t.background.className=n:(t.background.parentNode.removeChild(t.background),t.background=null);else if(n){var r=dn(t);t.background=r.insertBefore(E("div",null,n),r.firstChild),e.display.input.setUneditable(t.background)}}(e,t),t.line.wrapClass?dn(t).className=t.line.wrapClass:t.node!=t.text&&(t.node.className="");var n=t.textClass?t.textClass+" "+(t.line.textClass||""):t.line.textClass;t.text.className=n||""}function gn(e,t,n,r){if(t.gutter&&(t.node.removeChild(t.gutter),t.gutter=null),t.gutterBackground&&(t.node.removeChild(t.gutterBackground),t.gutterBackground=null),t.line.gutterClass){var i=dn(t);t.gutterBackground=E("div",null,"CodeMirror-gutter-background "+t.line.gutterClass,"left: "+(e.options.fixedGutter?r.fixedPos:-r.gutterTotalWidth)+"px; width: "+r.gutterTotalWidth+"px"),e.display.input.setUneditable(t.gutterBackground),i.insertBefore(t.gutterBackground,t.text)}var o=t.line.gutterMarkers;if(e.options.lineNumbers||o){var a=dn(t),s=t.gutter=E("div",null,"CodeMirror-gutter-wrapper","left: "+(e.options.fixedGutter?r.fixedPos:-r.gutterTotalWidth)+"px");if(e.display.input.setUneditable(s),a.insertBefore(s,t.text),t.line.gutterClass&&(s.className+=" "+t.line.gutterClass),!e.options.lineNumbers||o&&o["CodeMirror-linenumbers"]||(t.lineNumber=s.appendChild(E("div",$e(e.options,n),"CodeMirror-linenumber CodeMirror-gutter-elt","left: "+r.gutterLeft["CodeMirror-linenumbers"]+"px; width: "+e.display.lineNumInnerWidth+"px"))),o)for(var l=0;l<e.display.gutterSpecs.length;++l){var u=e.display.gutterSpecs[l].className,c=o.hasOwnProperty(u)&&o[u];c&&s.appendChild(E("div",[c],"CodeMirror-gutter-elt","left: "+r.gutterLeft[u]+"px; width: "+r.gutterWidth[u]+"px"))}}}function vn(e,t,n){t.alignable&&(t.alignable=null);for(var r=x("CodeMirror-linewidget"),i=t.node.firstChild,o=void 0;i;i=o)o=i.nextSibling,r.test(i.className)&&t.node.removeChild(i);yn(e,t,n)}function mn(e,t,n,r){var i=hn(e,t);return t.text=t.node=i.pre,i.bgClass&&(t.bgClass=i.bgClass),i.textClass&&(t.textClass=i.textClass),pn(e,t),gn(e,t,n,r),yn(e,t,r),t.node}function yn(e,t,n){if(bn(e,t.line,t,n,!0),t.rest)for(var r=0;r<t.rest.length;r++)bn(e,t.rest[r],t,n,!1)}function bn(e,t,n,r,i){if(t.widgets)for(var o=dn(n),a=0,s=t.widgets;a<s.length;++a){var l=s[a],u=E("div",[l.node],"CodeMirror-linewidget"+(l.className?" "+l.className:""));l.handleMouseEvents||u.setAttribute("cm-ignore-events","true"),Cn(l,u,n,r),e.display.input.setUneditable(u),i&&l.above?o.insertBefore(u,n.gutter||n.text):o.appendChild(u),ln(l,"redraw")}}function Cn(e,t,n,r){if(e.noHScroll){(n.alignable||(n.alignable=[])).push(t);var i=r.wrapperWidth;t.style.left=r.fixedPos+"px",e.coverGutter||(i-=r.gutterTotalWidth,t.style.paddingLeft=r.gutterTotalWidth+"px"),t.style.width=i+"px"}e.coverGutter&&(t.style.zIndex=5,t.style.position="relative",e.noHScroll||(t.style.marginLeft=-r.gutterTotalWidth+"px"))}function An(e){if(null!=e.height)return e.height;var t=e.doc.cm;if(!t)return 0;if(!k(document.body,e.node)){var n="position: relative;";e.coverGutter&&(n+="margin-left: -"+t.display.gutters.offsetWidth+"px;"),e.noHScroll&&(n+="width: "+t.display.wrapper.clientWidth+"px;"),T(t.display.measure,E("div",[e.node],null,n))}return e.height=e.node.parentNode.offsetHeight}function _n(e,t){for(var n=Ie(t);n!=e.wrapper;n=n.parentNode)if(!n||1==n.nodeType&&"true"==n.getAttribute("cm-ignore-events")||n.parentNode==e.sizer&&n!=e.mover)return!0}function In(e){return e.lineSpace.offsetTop}function xn(e){return e.mover.offsetHeight-e.lineSpace.offsetHeight}function wn(e){if(e.cachedPaddingH)return e.cachedPaddingH;var t=T(e.measure,E("pre","x","CodeMirror-line-like")),n=window.getComputedStyle?window.getComputedStyle(t):t.currentStyle,r={left:parseInt(n.paddingLeft),right:parseInt(n.paddingRight)};return isNaN(r.left)||isNaN(r.right)||(e.cachedPaddingH=r),r}function Mn(e){return 50-e.display.nativeBarWidth}function Sn(e){return e.display.scroller.clientWidth-Mn(e)-e.display.barWidth}function Tn(e){return e.display.scroller.clientHeight-Mn(e)-e.display.barHeight}function En(e,t,n){if(e.line==t)return{map:e.measure.map,cache:e.measure.cache};for(var r=0;r<e.rest.length;r++)if(e.rest[r]==t)return{map:e.measure.maps[r],cache:e.measure.caches[r]};for(var i=0;i<e.rest.length;i++)if(Qe(e.rest[i])>n)return{map:e.measure.maps[i],cache:e.measure.caches[i],before:!0}}function On(e,t,n,r){return Dn(e,Ln(e,t),n,r)}function kn(e,t){if(t>=e.display.viewFrom&&t<e.display.viewTo)return e.display.view[cr(e,t)];var n=e.display.externalMeasured;return n&&t>=n.lineN&&t<n.lineN+n.size?n:void 0}function Ln(e,t){var n=Qe(t),r=kn(e,n);r&&!r.text?r=null:r&&r.changes&&(cn(e,r,n,or(e)),e.curOp.forceUpdate=!0),r||(r=function(e,t){var n=Qe(t=zt(t)),r=e.display.externalMeasured=new rn(e.doc,t,n);r.lineN=n;var i=r.built=Kt(e,r);return r.text=i.pre,T(e.display.lineMeasure,i.pre),r}(e,t));var i=En(r,t,n);return{line:t,view:r,rect:null,map:i.map,cache:i.cache,before:i.before,hasHeights:!1}}function Dn(e,t,n,r,i){t.before&&(n=-1);var o,l=n+(r||"");return t.cache.hasOwnProperty(l)?o=t.cache[l]:(t.rect||(t.rect=t.view.text.getBoundingClientRect()),t.hasHeights||(function(e,t,n){var r=e.options.lineWrapping,i=r&&Sn(e);if(!t.measure.heights||r&&t.measure.width!=i){var o=t.measure.heights=[];if(r){t.measure.width=i;for(var a=t.text.firstChild.getClientRects(),s=0;s<a.length-1;s++){var l=a[s],u=a[s+1];Math.abs(l.bottom-u.bottom)>2&&o.push((l.bottom+u.top)/2-n.top)}}o.push(n.bottom-n.top)}}(e,t.view,t.rect),t.hasHeights=!0),(o=function(e,t,n,r){var i,o=jn(t.map,n,r),l=o.node,u=o.start,c=o.end,d=o.collapse;if(3==l.nodeType){for(var h=0;h<4;h++){for(;u&&re(t.line.text.charAt(o.coverStart+u));)--u;for(;o.coverStart+c<o.coverEnd&&re(t.line.text.charAt(o.coverStart+c));)++c;if((i=a&&s<9&&0==u&&c==o.coverEnd-o.coverStart?l.parentNode.getBoundingClientRect():Rn(w(l,u,c).getClientRects(),r)).left||i.right||0==u)break;c=u,u-=1,d="right"}a&&s<11&&(i=function(e,t){if(!window.screen||null==screen.logicalXDPI||screen.logicalXDPI==screen.deviceXDPI||!function(e){if(null!=Pe)return Pe;var t=T(e,E("span","x")),n=t.getBoundingClientRect(),r=w(t,0,1).getBoundingClientRect();return Pe=Math.abs(n.left-r.left)>1}(e))return t;var n=screen.logicalXDPI/screen.deviceXDPI,r=screen.logicalYDPI/screen.deviceYDPI;return{left:t.left*n,right:t.right*n,top:t.top*r,bottom:t.bottom*r}}(e.display.measure,i))}else{var f;u>0&&(d=r="right"),i=e.options.lineWrapping&&(f=l.getClientRects()).length>1?f["right"==r?f.length-1:0]:l.getBoundingClientRect()}if(a&&s<9&&!u&&(!i||!i.left&&!i.right)){var p=l.parentNode.getClientRects()[0];i=p?{left:p.left,right:p.left+ir(e.display),top:p.top,bottom:p.bottom}:Nn}for(var g=i.top-t.rect.top,v=i.bottom-t.rect.top,m=(g+v)/2,y=t.view.measure.heights,b=0;b<y.length-1&&!(m<y[b]);b++);var C=b?y[b-1]:0,A=y[b],_={left:("right"==d?i.right:i.left)-t.rect.left,right:("left"==d?i.left:i.right)-t.rect.left,top:C,bottom:A};return i.left||i.right||(_.bogus=!0),e.options.singleCursorHeightPerLine||(_.rtop=g,_.rbottom=v),_}(e,t,n,r)).bogus||(t.cache[l]=o)),{left:o.left,right:o.right,top:i?o.rtop:o.top,bottom:i?o.rbottom:o.bottom}}var Pn,Nn={left:0,right:0,top:0,bottom:0};function jn(e,t,n){for(var r,i,o,a,s,l,u=0;u<e.length;u+=3)if(s=e[u],l=e[u+1],t<s?(i=0,o=1,a="left"):t<l?o=1+(i=t-s):(u==e.length-3||t==l&&e[u+3]>t)&&(i=(o=l-s)-1,t>=l&&(a="right")),null!=i){if(r=e[u+2],s==l&&n==(r.insertLeft?"left":"right")&&(a=n),"left"==n&&0==i)for(;u&&e[u-2]==e[u-3]&&e[u-1].insertLeft;)r=e[2+(u-=3)],a="left";if("right"==n&&i==l-s)for(;u<e.length-3&&e[u+3]==e[u+4]&&!e[u+5].insertLeft;)r=e[(u+=3)+2],a="right";break}return{node:r,start:i,end:o,collapse:a,coverStart:s,coverEnd:l}}function Rn(e,t){var n=Nn;if("left"==t)for(var r=0;r<e.length&&(n=e[r]).left==n.right;r++);else for(var i=e.length-1;i>=0&&(n=e[i]).left==n.right;i--);return n}function zn(e){if(e.measure&&(e.measure.cache={},e.measure.heights=null,e.rest))for(var t=0;t<e.rest.length;t++)e.measure.caches[t]={}}function Fn(e){e.display.externalMeasure=null,S(e.display.lineMeasure);for(var t=0;t<e.display.view.length;t++)zn(e.display.view[t])}function Bn(e){Fn(e),e.display.cachedCharWidth=e.display.cachedTextHeight=e.display.cachedPaddingH=null,e.options.lineWrapping||(e.display.maxLineChanged=!0),e.display.lineNumChars=null}function Hn(){return c&&v?-(document.body.getBoundingClientRect().left-parseInt(getComputedStyle(document.body).marginLeft)):window.pageXOffset||(document.documentElement||document.body).scrollLeft}function Un(){return c&&v?-(document.body.getBoundingClientRect().top-parseInt(getComputedStyle(document.body).marginTop)):window.pageYOffset||(document.documentElement||document.body).scrollTop}function Vn(e){var t=0;if(e.widgets)for(var n=0;n<e.widgets.length;++n)e.widgets[n].above&&(t+=An(e.widgets[n]));return t}function Yn(e,t,n,r,i){if(!i){var o=Vn(t);n.top+=o,n.bottom+=o}if("line"==r)return n;r||(r="local");var a=Vt(t);if("local"==r?a+=In(e.display):a-=e.display.viewOffset,"page"==r||"window"==r){var s=e.display.lineSpace.getBoundingClientRect();a+=s.top+("window"==r?0:Un());var l=s.left+("window"==r?0:Hn());n.left+=l,n.right+=l}return n.top+=a,n.bottom+=a,n}function Wn(e,t,n){if("div"==n)return t;var r=t.left,i=t.top;if("page"==n)r-=Hn(),i-=Un();else if("local"==n||!n){var o=e.display.sizer.getBoundingClientRect();r+=o.left,i+=o.top}var a=e.display.lineSpace.getBoundingClientRect();return{left:r-a.left,top:i-a.top}}function Gn(e,t,n,r,i){return r||(r=Ge(e.doc,t.line)),Yn(e,r,On(e,r,t.ch,i),n)}function Xn(e,t,n,r,i,o){function a(t,a){var s=Dn(e,i,t,a?"right":"left",o);return a?s.left=s.right:s.right=s.left,Yn(e,r,s,n)}r=r||Ge(e.doc,t.line),i||(i=Ln(e,r));var s=ue(r,e.doc.direction),l=t.ch,u=t.sticky;if(l>=r.text.length?(l=r.text.length,u="before"):l<=0&&(l=0,u="after"),!s)return a("before"==u?l-1:l,"before"==u);function c(e,t,n){return a(n?e-1:e,1==s[t].level!=n)}var d=se(s,l,u),h=ae,f=c(l,d,"before"==u);return null!=h&&(f.other=c(l,h,"before"!=u)),f}function qn(e,t){var n=0;t=st(e.doc,t),e.options.lineWrapping||(n=ir(e.display)*t.ch);var r=Ge(e.doc,t.line),i=Vt(r)+In(e.display);return{left:n,right:n,top:i,bottom:i+r.height}}function Zn(e,t,n,r,i){var o=et(e,t,n);return o.xRel=i,r&&(o.outside=r),o}function Qn(e,t,n){var r=e.doc;if((n+=e.display.viewOffset)<0)return Zn(r.first,0,null,-1,-1);var i=Ke(r,n),o=r.first+r.size-1;if(i>o)return Zn(r.first+r.size-1,Ge(r,o).text.length,null,1,1);t<0&&(t=0);for(var a=Ge(r,i);;){var s=er(e,a,i,t,n),l=jt(a,s.ch+(s.xRel>0||s.outside>0?1:0));if(!l)return s;var u=l.find(1);if(u.line==i)return u;a=Ge(r,i=u.line)}}function Kn(e,t,n,r){r-=Vn(t);var i=t.text.length,o=oe((function(t){return Dn(e,n,t-1).bottom<=r}),i,0);return{begin:o,end:i=oe((function(t){return Dn(e,n,t).top>r}),o,i)}}function Jn(e,t,n,r){return n||(n=Ln(e,t)),Kn(e,t,n,Yn(e,t,Dn(e,n,r),"line").top)}function $n(e,t,n,r){return!(e.bottom<=n)&&(e.top>n||(r?e.left:e.right)>t)}function er(e,t,n,r,i){i-=Vt(t);var o=Ln(e,t),a=Vn(t),s=0,l=t.text.length,u=!0,c=ue(t,e.doc.direction);if(c){var d=(e.options.lineWrapping?nr:tr)(e,t,n,o,c,r,i);s=(u=1!=d.level)?d.from:d.to-1,l=u?d.to:d.from-1}var h,f,p=null,g=null,v=oe((function(t){var n=Dn(e,o,t);return n.top+=a,n.bottom+=a,!!$n(n,r,i,!1)&&(n.top<=i&&n.left<=r&&(p=t,g=n),!0)}),s,l),m=!1;if(g){var y=r-g.left<g.right-r,b=y==u;v=p+(b?0:1),f=b?"after":"before",h=y?g.left:g.right}else{u||v!=l&&v!=s||v++,f=0==v?"after":v==t.text.length?"before":Dn(e,o,v-(u?1:0)).bottom+a<=i==u?"after":"before";var C=Xn(e,et(n,v,f),"line",t,o);h=C.left,m=i<C.top?-1:i>=C.bottom?1:0}return Zn(n,v=ie(t.text,v,1),f,m,r-h)}function tr(e,t,n,r,i,o,a){var s=oe((function(s){var l=i[s],u=1!=l.level;return $n(Xn(e,et(n,u?l.to:l.from,u?"before":"after"),"line",t,r),o,a,!0)}),0,i.length-1),l=i[s];if(s>0){var u=1!=l.level,c=Xn(e,et(n,u?l.from:l.to,u?"after":"before"),"line",t,r);$n(c,o,a,!0)&&c.top>a&&(l=i[s-1])}return l}function nr(e,t,n,r,i,o,a){var s=Kn(e,t,r,a),l=s.begin,u=s.end;/\s/.test(t.text.charAt(u-1))&&u--;for(var c=null,d=null,h=0;h<i.length;h++){var f=i[h];if(!(f.from>=u||f.to<=l)){var p=Dn(e,r,1!=f.level?Math.min(u,f.to)-1:Math.max(l,f.from)).right,g=p<o?o-p+1e9:p-o;(!c||d>g)&&(c=f,d=g)}}return c||(c=i[i.length-1]),c.from<l&&(c={from:l,to:c.to,level:c.level}),c.to>u&&(c={from:c.from,to:u,level:c.level}),c}function rr(e){if(null!=e.cachedTextHeight)return e.cachedTextHeight;if(null==Pn){Pn=E("pre",null,"CodeMirror-line-like");for(var t=0;t<49;++t)Pn.appendChild(document.createTextNode("x")),Pn.appendChild(E("br"));Pn.appendChild(document.createTextNode("x"))}T(e.measure,Pn);var n=Pn.offsetHeight/50;return n>3&&(e.cachedTextHeight=n),S(e.measure),n||1}function ir(e){if(null!=e.cachedCharWidth)return e.cachedCharWidth;var t=E("span","xxxxxxxxxx"),n=E("pre",[t],"CodeMirror-line-like");T(e.measure,n);var r=t.getBoundingClientRect(),i=(r.right-r.left)/10;return i>2&&(e.cachedCharWidth=i),i||10}function or(e){for(var t=e.display,n={},r={},i=t.gutters.clientLeft,o=t.gutters.firstChild,a=0;o;o=o.nextSibling,++a){var s=e.display.gutterSpecs[a].className;n[s]=o.offsetLeft+o.clientLeft+i,r[s]=o.clientWidth}return{fixedPos:ar(t),gutterTotalWidth:t.gutters.offsetWidth,gutterLeft:n,gutterWidth:r,wrapperWidth:t.wrapper.clientWidth}}function ar(e){return e.scroller.getBoundingClientRect().left-e.sizer.getBoundingClientRect().left}function sr(e){var t=rr(e.display),n=e.options.lineWrapping,r=n&&Math.max(5,e.display.scroller.clientWidth/ir(e.display)-3);return function(i){if(Ht(e.doc,i))return 0;var o=0;if(i.widgets)for(var a=0;a<i.widgets.length;a++)i.widgets[a].height&&(o+=i.widgets[a].height);return n?o+(Math.ceil(i.text.length/r)||1)*t:o+t}}function lr(e){var t=e.doc,n=sr(e);t.iter((function(e){var t=n(e);t!=e.height&&Ze(e,t)}))}function ur(e,t,n,r){var i=e.display;if(!n&&"true"==Ie(t).getAttribute("cm-not-content"))return null;var o,a,s=i.lineSpace.getBoundingClientRect();try{o=t.clientX-s.left,a=t.clientY-s.top}catch(e){return null}var l,u=Qn(e,o,a);if(r&&u.xRel>0&&(l=Ge(e.doc,u.line).text).length==u.ch){var c=z(l,l.length,e.options.tabSize)-l.length;u=et(u.line,Math.max(0,Math.round((o-wn(e.display).left)/ir(e.display))-c))}return u}function cr(e,t){if(t>=e.display.viewTo)return null;if((t-=e.display.viewFrom)<0)return null;for(var n=e.display.view,r=0;r<n.length;r++)if((t-=n[r].size)<0)return r}function dr(e,t,n,r){null==t&&(t=e.doc.first),null==n&&(n=e.doc.first+e.doc.size),r||(r=0);var i=e.display;if(r&&n<i.viewTo&&(null==i.updateLineNumbers||i.updateLineNumbers>t)&&(i.updateLineNumbers=t),e.curOp.viewChanged=!0,t>=i.viewTo)_t&&Ft(e.doc,t)<i.viewTo&&fr(e);else if(n<=i.viewFrom)_t&&Bt(e.doc,n+r)>i.viewFrom?fr(e):(i.viewFrom+=r,i.viewTo+=r);else if(t<=i.viewFrom&&n>=i.viewTo)fr(e);else if(t<=i.viewFrom){var o=pr(e,n,n+r,1);o?(i.view=i.view.slice(o.index),i.viewFrom=o.lineN,i.viewTo+=r):fr(e)}else if(n>=i.viewTo){var a=pr(e,t,t,-1);a?(i.view=i.view.slice(0,a.index),i.viewTo=a.lineN):fr(e)}else{var s=pr(e,t,t,-1),l=pr(e,n,n+r,1);s&&l?(i.view=i.view.slice(0,s.index).concat(on(e,s.lineN,l.lineN)).concat(i.view.slice(l.index)),i.viewTo+=r):fr(e)}var u=i.externalMeasured;u&&(n<u.lineN?u.lineN+=r:t<u.lineN+u.size&&(i.externalMeasured=null))}function hr(e,t,n){e.curOp.viewChanged=!0;var r=e.display,i=e.display.externalMeasured;if(i&&t>=i.lineN&&t<i.lineN+i.size&&(r.externalMeasured=null),!(t<r.viewFrom||t>=r.viewTo)){var o=r.view[cr(e,t)];if(null!=o.node){var a=o.changes||(o.changes=[]);-1==B(a,n)&&a.push(n)}}}function fr(e){e.display.viewFrom=e.display.viewTo=e.doc.first,e.display.view=[],e.display.viewOffset=0}function pr(e,t,n,r){var i,o=cr(e,t),a=e.display.view;if(!_t||n==e.doc.first+e.doc.size)return{index:o,lineN:n};for(var s=e.display.viewFrom,l=0;l<o;l++)s+=a[l].size;if(s!=t){if(r>0){if(o==a.length-1)return null;i=s+a[o].size-t,o++}else i=s-t;t+=i,n+=i}for(;Ft(e.doc,n)!=n;){if(o==(r<0?0:a.length-1))return null;n+=r*a[o-(r<0?1:0)].size,o+=r}return{index:o,lineN:n}}function gr(e){for(var t=e.display.view,n=0,r=0;r<t.length;r++){var i=t[r];i.hidden||i.node&&!i.changes||++n}return n}function vr(e){e.display.input.showSelection(e.display.input.prepareSelection())}function mr(e,t){void 0===t&&(t=!0);for(var n=e.doc,r={},i=r.cursors=document.createDocumentFragment(),o=r.selection=document.createDocumentFragment(),a=0;a<n.sel.ranges.length;a++)if(t||a!=n.sel.primIndex){var s=n.sel.ranges[a];if(!(s.from().line>=e.display.viewTo||s.to().line<e.display.viewFrom)){var l=s.empty();(l||e.options.showCursorWhenSelecting)&&yr(e,s.head,i),l||Cr(e,s,o)}}return r}function yr(e,t,n){var r=Xn(e,t,"div",null,null,!e.options.singleCursorHeightPerLine),i=n.appendChild(E("div"," ","CodeMirror-cursor"));if(i.style.left=r.left+"px",i.style.top=r.top+"px",i.style.height=Math.max(0,r.bottom-r.top)*e.options.cursorHeight+"px",r.other){var o=n.appendChild(E("div"," ","CodeMirror-cursor CodeMirror-secondarycursor"));o.style.display="",o.style.left=r.other.left+"px",o.style.top=r.other.top+"px",o.style.height=.85*(r.other.bottom-r.other.top)+"px"}}function br(e,t){return e.top-t.top||e.left-t.left}function Cr(e,t,n){var r=e.display,i=e.doc,o=document.createDocumentFragment(),a=wn(e.display),s=a.left,l=Math.max(r.sizerWidth,Sn(e)-r.sizer.offsetLeft)-a.right,u="ltr"==i.direction;function c(e,t,n,r){t<0&&(t=0),t=Math.round(t),r=Math.round(r),o.appendChild(E("div",null,"CodeMirror-selected","position: absolute; left: "+e+"px;\n                             top: "+t+"px; width: "+(null==n?l-e:n)+"px;\n                             height: "+(r-t)+"px"))}function d(t,n,r){var o,a,d=Ge(i,t),h=d.text.length;function f(n,r){return Gn(e,et(t,n),"div",d,r)}function p(t,n,r){var i=Jn(e,d,null,t),o="ltr"==n==("after"==r)?"left":"right";return f("after"==r?i.begin:i.end-(/\s/.test(d.text.charAt(i.end-1))?2:1),o)[o]}var g=ue(d,i.direction);return function(e,t,n,r){if(!e)return r(t,n,"ltr",0);for(var i=!1,o=0;o<e.length;++o){var a=e[o];(a.from<n&&a.to>t||t==n&&a.to==t)&&(r(Math.max(a.from,t),Math.min(a.to,n),1==a.level?"rtl":"ltr",o),i=!0)}i||r(t,n,"ltr")}(g,n||0,null==r?h:r,(function(e,t,i,d){var v="ltr"==i,m=f(e,v?"left":"right"),y=f(t-1,v?"right":"left"),b=null==n&&0==e,C=null==r&&t==h,A=0==d,_=!g||d==g.length-1;if(y.top-m.top<=3){var I=(u?C:b)&&_,x=(u?b:C)&&A?s:(v?m:y).left,w=I?l:(v?y:m).right;c(x,m.top,w-x,m.bottom)}else{var M,S,T,E;v?(M=u&&b&&A?s:m.left,S=u?l:p(e,i,"before"),T=u?s:p(t,i,"after"),E=u&&C&&_?l:y.right):(M=u?p(e,i,"before"):s,S=!u&&b&&A?l:m.right,T=!u&&C&&_?s:y.left,E=u?p(t,i,"after"):l),c(M,m.top,S-M,m.bottom),m.bottom<y.top&&c(s,m.bottom,null,y.top),c(T,y.top,E-T,y.bottom)}(!o||br(m,o)<0)&&(o=m),br(y,o)<0&&(o=y),(!a||br(m,a)<0)&&(a=m),br(y,a)<0&&(a=y)})),{start:o,end:a}}var h=t.from(),f=t.to();if(h.line==f.line)d(h.line,h.ch,f.ch);else{var p=Ge(i,h.line),g=Ge(i,f.line),v=zt(p)==zt(g),m=d(h.line,h.ch,v?p.text.length+1:null).end,y=d(f.line,v?0:null,f.ch).start;v&&(m.top<y.top-2?(c(m.right,m.top,null,m.bottom),c(s,y.top,y.left,y.bottom)):c(m.right,m.top,y.left-m.right,m.bottom)),m.bottom<y.top&&c(s,m.bottom,null,y.top)}n.appendChild(o)}function Ar(e){if(e.state.focused){var t=e.display;clearInterval(t.blinker);var n=!0;t.cursorDiv.style.visibility="",e.options.cursorBlinkRate>0?t.blinker=setInterval((function(){e.hasFocus()||wr(e),t.cursorDiv.style.visibility=(n=!n)?"":"hidden"}),e.options.cursorBlinkRate):e.options.cursorBlinkRate<0&&(t.cursorDiv.style.visibility="hidden")}}function _r(e){e.hasFocus()||(e.display.input.focus(),e.state.focused||xr(e))}function Ir(e){e.state.delayingBlurEvent=!0,setTimeout((function(){e.state.delayingBlurEvent&&(e.state.delayingBlurEvent=!1,e.state.focused&&wr(e))}),100)}function xr(e,t){e.state.delayingBlurEvent&&!e.state.draggingText&&(e.state.delayingBlurEvent=!1),"nocursor"!=e.options.readOnly&&(e.state.focused||(pe(e,"focus",e,t),e.state.focused=!0,D(e.display.wrapper,"CodeMirror-focused"),e.curOp||e.display.selForContextMenu==e.doc.sel||(e.display.input.reset(),l&&setTimeout((function(){return e.display.input.reset(!0)}),20)),e.display.input.receivedFocus()),Ar(e))}function wr(e,t){e.state.delayingBlurEvent||(e.state.focused&&(pe(e,"blur",e,t),e.state.focused=!1,M(e.display.wrapper,"CodeMirror-focused")),clearInterval(e.display.blinker),setTimeout((function(){e.state.focused||(e.display.shift=!1)}),150))}function Mr(e){for(var t=e.display,n=t.lineDiv.offsetTop,r=0;r<t.view.length;r++){var i=t.view[r],o=e.options.lineWrapping,l=void 0,u=0;if(!i.hidden){if(a&&s<8){var c=i.node.offsetTop+i.node.offsetHeight;l=c-n,n=c}else{var d=i.node.getBoundingClientRect();l=d.bottom-d.top,!o&&i.text.firstChild&&(u=i.text.firstChild.getBoundingClientRect().right-d.left-1)}var h=i.line.height-l;if((h>.005||h<-.005)&&(Ze(i.line,l),Sr(i.line),i.rest))for(var f=0;f<i.rest.length;f++)Sr(i.rest[f]);if(u>e.display.sizerWidth){var p=Math.ceil(u/ir(e.display));p>e.display.maxLineLength&&(e.display.maxLineLength=p,e.display.maxLine=i.line,e.display.maxLineChanged=!0)}}}}function Sr(e){if(e.widgets)for(var t=0;t<e.widgets.length;++t){var n=e.widgets[t],r=n.node.parentNode;r&&(n.height=r.offsetHeight)}}function Tr(e,t,n){var r=n&&null!=n.top?Math.max(0,n.top):e.scroller.scrollTop;r=Math.floor(r-In(e));var i=n&&null!=n.bottom?n.bottom:r+e.wrapper.clientHeight,o=Ke(t,r),a=Ke(t,i);if(n&&n.ensure){var s=n.ensure.from.line,l=n.ensure.to.line;s<o?(o=s,a=Ke(t,Vt(Ge(t,s))+e.wrapper.clientHeight)):Math.min(l,t.lastLine())>=a&&(o=Ke(t,Vt(Ge(t,l))-e.wrapper.clientHeight),a=l)}return{from:o,to:Math.max(a,o+1)}}function Er(e,t){var n=e.display,r=rr(e.display);t.top<0&&(t.top=0);var i=e.curOp&&null!=e.curOp.scrollTop?e.curOp.scrollTop:n.scroller.scrollTop,o=Tn(e),a={};t.bottom-t.top>o&&(t.bottom=t.top+o);var s=e.doc.height+xn(n),l=t.top<r,u=t.bottom>s-r;if(t.top<i)a.scrollTop=l?0:t.top;else if(t.bottom>i+o){var c=Math.min(t.top,(u?s:t.bottom)-o);c!=i&&(a.scrollTop=c)}var d=e.options.fixedGutter?0:n.gutters.offsetWidth,h=e.curOp&&null!=e.curOp.scrollLeft?e.curOp.scrollLeft:n.scroller.scrollLeft-d,f=Sn(e)-n.gutters.offsetWidth,p=t.right-t.left>f;return p&&(t.right=t.left+f),t.left<10?a.scrollLeft=0:t.left<h?a.scrollLeft=Math.max(0,t.left+d-(p?0:10)):t.right>f+h-3&&(a.scrollLeft=t.right+(p?0:10)-f),a}function Or(e,t){null!=t&&(Dr(e),e.curOp.scrollTop=(null==e.curOp.scrollTop?e.doc.scrollTop:e.curOp.scrollTop)+t)}function kr(e){Dr(e);var t=e.getCursor();e.curOp.scrollToPos={from:t,to:t,margin:e.options.cursorScrollMargin}}function Lr(e,t,n){null==t&&null==n||Dr(e),null!=t&&(e.curOp.scrollLeft=t),null!=n&&(e.curOp.scrollTop=n)}function Dr(e){var t=e.curOp.scrollToPos;t&&(e.curOp.scrollToPos=null,Pr(e,qn(e,t.from),qn(e,t.to),t.margin))}function Pr(e,t,n,r){var i=Er(e,{left:Math.min(t.left,n.left),top:Math.min(t.top,n.top)-r,right:Math.max(t.right,n.right),bottom:Math.max(t.bottom,n.bottom)+r});Lr(e,i.scrollLeft,i.scrollTop)}function Nr(e,t){Math.abs(e.doc.scrollTop-t)<2||(n||li(e,{top:t}),jr(e,t,!0),n&&li(e),ri(e,100))}function jr(e,t,n){t=Math.max(0,Math.min(e.display.scroller.scrollHeight-e.display.scroller.clientHeight,t)),(e.display.scroller.scrollTop!=t||n)&&(e.doc.scrollTop=t,e.display.scrollbars.setScrollTop(t),e.display.scroller.scrollTop!=t&&(e.display.scroller.scrollTop=t))}function Rr(e,t,n,r){t=Math.max(0,Math.min(t,e.display.scroller.scrollWidth-e.display.scroller.clientWidth)),(n?t==e.doc.scrollLeft:Math.abs(e.doc.scrollLeft-t)<2)&&!r||(e.doc.scrollLeft=t,di(e),e.display.scroller.scrollLeft!=t&&(e.display.scroller.scrollLeft=t),e.display.scrollbars.setScrollLeft(t))}function zr(e){var t=e.display,n=t.gutters.offsetWidth,r=Math.round(e.doc.height+xn(e.display));return{clientHeight:t.scroller.clientHeight,viewHeight:t.wrapper.clientHeight,scrollWidth:t.scroller.scrollWidth,clientWidth:t.scroller.clientWidth,viewWidth:t.wrapper.clientWidth,barLeft:e.options.fixedGutter?n:0,docHeight:r,scrollHeight:r+Mn(e)+t.barHeight,nativeBarWidth:t.nativeBarWidth,gutterWidth:n}}var Fr=function(e,t,n){this.cm=n;var r=this.vert=E("div",[E("div",null,null,"min-width: 1px")],"CodeMirror-vscrollbar"),i=this.horiz=E("div",[E("div",null,null,"height: 100%; min-height: 1px")],"CodeMirror-hscrollbar");r.tabIndex=i.tabIndex=-1,e(r),e(i),de(r,"scroll",(function(){r.clientHeight&&t(r.scrollTop,"vertical")})),de(i,"scroll",(function(){i.clientWidth&&t(i.scrollLeft,"horizontal")})),this.checkedZeroWidth=!1,a&&s<8&&(this.horiz.style.minHeight=this.vert.style.minWidth="18px")};Fr.prototype.update=function(e){var t=e.scrollWidth>e.clientWidth+1,n=e.scrollHeight>e.clientHeight+1,r=e.nativeBarWidth;if(n){this.vert.style.display="block",this.vert.style.bottom=t?r+"px":"0";var i=e.viewHeight-(t?r:0);this.vert.firstChild.style.height=Math.max(0,e.scrollHeight-e.clientHeight+i)+"px"}else this.vert.style.display="",this.vert.firstChild.style.height="0";if(t){this.horiz.style.display="block",this.horiz.style.right=n?r+"px":"0",this.horiz.style.left=e.barLeft+"px";var o=e.viewWidth-e.barLeft-(n?r:0);this.horiz.firstChild.style.width=Math.max(0,e.scrollWidth-e.clientWidth+o)+"px"}else this.horiz.style.display="",this.horiz.firstChild.style.width="0";return!this.checkedZeroWidth&&e.clientHeight>0&&(0==r&&this.zeroWidthHack(),this.checkedZeroWidth=!0),{right:n?r:0,bottom:t?r:0}},Fr.prototype.setScrollLeft=function(e){this.horiz.scrollLeft!=e&&(this.horiz.scrollLeft=e),this.disableHoriz&&this.enableZeroWidthBar(this.horiz,this.disableHoriz,"horiz")},Fr.prototype.setScrollTop=function(e){this.vert.scrollTop!=e&&(this.vert.scrollTop=e),this.disableVert&&this.enableZeroWidthBar(this.vert,this.disableVert,"vert")},Fr.prototype.zeroWidthHack=function(){var e=y&&!f?"12px":"18px";this.horiz.style.height=this.vert.style.width=e,this.horiz.style.pointerEvents=this.vert.style.pointerEvents="none",this.disableHoriz=new F,this.disableVert=new F},Fr.prototype.enableZeroWidthBar=function(e,t,n){e.style.pointerEvents="auto",t.set(1e3,(function r(){var i=e.getBoundingClientRect();("vert"==n?document.elementFromPoint(i.right-1,(i.top+i.bottom)/2):document.elementFromPoint((i.right+i.left)/2,i.bottom-1))!=e?e.style.pointerEvents="none":t.set(1e3,r)}))},Fr.prototype.clear=function(){var e=this.horiz.parentNode;e.removeChild(this.horiz),e.removeChild(this.vert)};var Br=function(){};function Hr(e,t){t||(t=zr(e));var n=e.display.barWidth,r=e.display.barHeight;Ur(e,t);for(var i=0;i<4&&n!=e.display.barWidth||r!=e.display.barHeight;i++)n!=e.display.barWidth&&e.options.lineWrapping&&Mr(e),Ur(e,zr(e)),n=e.display.barWidth,r=e.display.barHeight}function Ur(e,t){var n=e.display,r=n.scrollbars.update(t);n.sizer.style.paddingRight=(n.barWidth=r.right)+"px",n.sizer.style.paddingBottom=(n.barHeight=r.bottom)+"px",n.heightForcer.style.borderBottom=r.bottom+"px solid transparent",r.right&&r.bottom?(n.scrollbarFiller.style.display="block",n.scrollbarFiller.style.height=r.bottom+"px",n.scrollbarFiller.style.width=r.right+"px"):n.scrollbarFiller.style.display="",r.bottom&&e.options.coverGutterNextToScrollbar&&e.options.fixedGutter?(n.gutterFiller.style.display="block",n.gutterFiller.style.height=r.bottom+"px",n.gutterFiller.style.width=t.gutterWidth+"px"):n.gutterFiller.style.display=""}Br.prototype.update=function(){return{bottom:0,right:0}},Br.prototype.setScrollLeft=function(){},Br.prototype.setScrollTop=function(){},Br.prototype.clear=function(){};var Vr={native:Fr,null:Br};function Yr(e){e.display.scrollbars&&(e.display.scrollbars.clear(),e.display.scrollbars.addClass&&M(e.display.wrapper,e.display.scrollbars.addClass)),e.display.scrollbars=new Vr[e.options.scrollbarStyle]((function(t){e.display.wrapper.insertBefore(t,e.display.scrollbarFiller),de(t,"mousedown",(function(){e.state.focused&&setTimeout((function(){return e.display.input.focus()}),0)})),t.setAttribute("cm-not-content","true")}),(function(t,n){"horizontal"==n?Rr(e,t):Nr(e,t)}),e),e.display.scrollbars.addClass&&D(e.display.wrapper,e.display.scrollbars.addClass)}var Wr=0;function Gr(e){var t;e.curOp={cm:e,viewChanged:!1,startHeight:e.doc.height,forceUpdate:!1,updateInput:0,typing:!1,changeObjs:null,cursorActivityHandlers:null,cursorActivityCalled:0,selectionChanged:!1,updateMaxLine:!1,scrollLeft:null,scrollTop:null,scrollToPos:null,focus:!1,id:++Wr},t=e.curOp,an?an.ops.push(t):t.ownsGroup=an={ops:[t],delayedCallbacks:[]}}function Xr(e){var t=e.curOp;t&&function(e,t){var n=e.ownsGroup;if(n)try{!function(e){var t=e.delayedCallbacks,n=0;do{for(;n<t.length;n++)t[n].call(null);for(var r=0;r<e.ops.length;r++){var i=e.ops[r];if(i.cursorActivityHandlers)for(;i.cursorActivityCalled<i.cursorActivityHandlers.length;)i.cursorActivityHandlers[i.cursorActivityCalled++].call(null,i.cm)}}while(n<t.length)}(n)}finally{an=null,t(n)}}(t,(function(e){for(var t=0;t<e.ops.length;t++)e.ops[t].cm.curOp=null;!function(e){for(var t=e.ops,n=0;n<t.length;n++)qr(t[n]);for(var r=0;r<t.length;r++)Zr(t[r]);for(var i=0;i<t.length;i++)Qr(t[i]);for(var o=0;o<t.length;o++)Kr(t[o]);for(var a=0;a<t.length;a++)Jr(t[a])}(e)}))}function qr(e){var t=e.cm,n=t.display;!function(e){var t=e.display;!t.scrollbarsClipped&&t.scroller.offsetWidth&&(t.nativeBarWidth=t.scroller.offsetWidth-t.scroller.clientWidth,t.heightForcer.style.height=Mn(e)+"px",t.sizer.style.marginBottom=-t.nativeBarWidth+"px",t.sizer.style.borderRightWidth=Mn(e)+"px",t.scrollbarsClipped=!0)}(t),e.updateMaxLine&&Wt(t),e.mustUpdate=e.viewChanged||e.forceUpdate||null!=e.scrollTop||e.scrollToPos&&(e.scrollToPos.from.line<n.viewFrom||e.scrollToPos.to.line>=n.viewTo)||n.maxLineChanged&&t.options.lineWrapping,e.update=e.mustUpdate&&new oi(t,e.mustUpdate&&{top:e.scrollTop,ensure:e.scrollToPos},e.forceUpdate)}function Zr(e){e.updatedDisplay=e.mustUpdate&&ai(e.cm,e.update)}function Qr(e){var t=e.cm,n=t.display;e.updatedDisplay&&Mr(t),e.barMeasure=zr(t),n.maxLineChanged&&!t.options.lineWrapping&&(e.adjustWidthTo=On(t,n.maxLine,n.maxLine.text.length).left+3,t.display.sizerWidth=e.adjustWidthTo,e.barMeasure.scrollWidth=Math.max(n.scroller.clientWidth,n.sizer.offsetLeft+e.adjustWidthTo+Mn(t)+t.display.barWidth),e.maxScrollLeft=Math.max(0,n.sizer.offsetLeft+e.adjustWidthTo-Sn(t))),(e.updatedDisplay||e.selectionChanged)&&(e.preparedSelection=n.input.prepareSelection())}function Kr(e){var t=e.cm;null!=e.adjustWidthTo&&(t.display.sizer.style.minWidth=e.adjustWidthTo+"px",e.maxScrollLeft<t.doc.scrollLeft&&Rr(t,Math.min(t.display.scroller.scrollLeft,e.maxScrollLeft),!0),t.display.maxLineChanged=!1);var n=e.focus&&e.focus==L();e.preparedSelection&&t.display.input.showSelection(e.preparedSelection,n),(e.updatedDisplay||e.startHeight!=t.doc.height)&&Hr(t,e.barMeasure),e.updatedDisplay&&ci(t,e.barMeasure),e.selectionChanged&&Ar(t),t.state.focused&&e.updateInput&&t.display.input.reset(e.typing),n&&_r(e.cm)}function Jr(e){var t=e.cm,n=t.display,r=t.doc;e.updatedDisplay&&si(t,e.update),null==n.wheelStartX||null==e.scrollTop&&null==e.scrollLeft&&!e.scrollToPos||(n.wheelStartX=n.wheelStartY=null),null!=e.scrollTop&&jr(t,e.scrollTop,e.forceScroll),null!=e.scrollLeft&&Rr(t,e.scrollLeft,!0,!0),e.scrollToPos&&function(e,t){if(!ge(e,"scrollCursorIntoView")){var n=e.display,r=n.sizer.getBoundingClientRect(),i=null;if(t.top+r.top<0?i=!0:t.bottom+r.top>(window.innerHeight||document.documentElement.clientHeight)&&(i=!1),null!=i&&!p){var o=E("div","​",null,"position: absolute;\n                         top: "+(t.top-n.viewOffset-In(e.display))+"px;\n                         height: "+(t.bottom-t.top+Mn(e)+n.barHeight)+"px;\n                         left: "+t.left+"px; width: "+Math.max(2,t.right-t.left)+"px;");e.display.lineSpace.appendChild(o),o.scrollIntoView(i),e.display.lineSpace.removeChild(o)}}}(t,function(e,t,n,r){var i;null==r&&(r=0),e.options.lineWrapping||t!=n||(n="before"==(t=t.ch?et(t.line,"before"==t.sticky?t.ch-1:t.ch,"after"):t).sticky?et(t.line,t.ch+1,"before"):t);for(var o=0;o<5;o++){var a=!1,s=Xn(e,t),l=n&&n!=t?Xn(e,n):s,u=Er(e,i={left:Math.min(s.left,l.left),top:Math.min(s.top,l.top)-r,right:Math.max(s.left,l.left),bottom:Math.max(s.bottom,l.bottom)+r}),c=e.doc.scrollTop,d=e.doc.scrollLeft;if(null!=u.scrollTop&&(Nr(e,u.scrollTop),Math.abs(e.doc.scrollTop-c)>1&&(a=!0)),null!=u.scrollLeft&&(Rr(e,u.scrollLeft),Math.abs(e.doc.scrollLeft-d)>1&&(a=!0)),!a)break}return i}(t,st(r,e.scrollToPos.from),st(r,e.scrollToPos.to),e.scrollToPos.margin));var i=e.maybeHiddenMarkers,o=e.maybeUnhiddenMarkers;if(i)for(var a=0;a<i.length;++a)i[a].lines.length||pe(i[a],"hide");if(o)for(var s=0;s<o.length;++s)o[s].lines.length&&pe(o[s],"unhide");n.wrapper.offsetHeight&&(r.scrollTop=t.display.scroller.scrollTop),e.changeObjs&&pe(t,"changes",t,e.changeObjs),e.update&&e.update.finish()}function $r(e,t){if(e.curOp)return t();Gr(e);try{return t()}finally{Xr(e)}}function ei(e,t){return function(){if(e.curOp)return t.apply(e,arguments);Gr(e);try{return t.apply(e,arguments)}finally{Xr(e)}}}function ti(e){return function(){if(this.curOp)return e.apply(this,arguments);Gr(this);try{return e.apply(this,arguments)}finally{Xr(this)}}}function ni(e){return function(){var t=this.cm;if(!t||t.curOp)return e.apply(this,arguments);Gr(t);try{return e.apply(this,arguments)}finally{Xr(t)}}}function ri(e,t){e.doc.highlightFrontier<e.display.viewTo&&e.state.highlight.set(t,j(ii,e))}function ii(e){var t=e.doc;if(!(t.highlightFrontier>=e.display.viewTo)){var n=+new Date+e.options.workTime,r=ft(e,t.highlightFrontier),i=[];t.iter(r.line,Math.min(t.first+t.size,e.display.viewTo+500),(function(o){if(r.line>=e.display.viewFrom){var a=o.styles,s=o.text.length>e.options.maxHighlightLength?Ue(t.mode,r.state):null,l=dt(e,o,r,!0);s&&(r.state=s),o.styles=l.styles;var u=o.styleClasses,c=l.classes;c?o.styleClasses=c:u&&(o.styleClasses=null);for(var d=!a||a.length!=o.styles.length||u!=c&&(!u||!c||u.bgClass!=c.bgClass||u.textClass!=c.textClass),h=0;!d&&h<a.length;++h)d=a[h]!=o.styles[h];d&&i.push(r.line),o.stateAfter=r.save(),r.nextLine()}else o.text.length<=e.options.maxHighlightLength&&pt(e,o.text,r),o.stateAfter=r.line%5==0?r.save():null,r.nextLine();if(+new Date>n)return ri(e,e.options.workDelay),!0})),t.highlightFrontier=r.line,t.modeFrontier=Math.max(t.modeFrontier,r.line),i.length&&$r(e,(function(){for(var t=0;t<i.length;t++)hr(e,i[t],"text")}))}}var oi=function(e,t,n){var r=e.display;this.viewport=t,this.visible=Tr(r,e.doc,t),this.editorIsHidden=!r.wrapper.offsetWidth,this.wrapperHeight=r.wrapper.clientHeight,this.wrapperWidth=r.wrapper.clientWidth,this.oldDisplayWidth=Sn(e),this.force=n,this.dims=or(e),this.events=[]};function ai(e,t){var n=e.display,r=e.doc;if(t.editorIsHidden)return fr(e),!1;if(!t.force&&t.visible.from>=n.viewFrom&&t.visible.to<=n.viewTo&&(null==n.updateLineNumbers||n.updateLineNumbers>=n.viewTo)&&n.renderedView==n.view&&0==gr(e))return!1;hi(e)&&(fr(e),t.dims=or(e));var i=r.first+r.size,o=Math.max(t.visible.from-e.options.viewportMargin,r.first),a=Math.min(i,t.visible.to+e.options.viewportMargin);n.viewFrom<o&&o-n.viewFrom<20&&(o=Math.max(r.first,n.viewFrom)),n.viewTo>a&&n.viewTo-a<20&&(a=Math.min(i,n.viewTo)),_t&&(o=Ft(e.doc,o),a=Bt(e.doc,a));var s=o!=n.viewFrom||a!=n.viewTo||n.lastWrapHeight!=t.wrapperHeight||n.lastWrapWidth!=t.wrapperWidth;!function(e,t,n){var r=e.display;0==r.view.length||t>=r.viewTo||n<=r.viewFrom?(r.view=on(e,t,n),r.viewFrom=t):(r.viewFrom>t?r.view=on(e,t,r.viewFrom).concat(r.view):r.viewFrom<t&&(r.view=r.view.slice(cr(e,t))),r.viewFrom=t,r.viewTo<n?r.view=r.view.concat(on(e,r.viewTo,n)):r.viewTo>n&&(r.view=r.view.slice(0,cr(e,n)))),r.viewTo=n}(e,o,a),n.viewOffset=Vt(Ge(e.doc,n.viewFrom)),e.display.mover.style.top=n.viewOffset+"px";var u=gr(e);if(!s&&0==u&&!t.force&&n.renderedView==n.view&&(null==n.updateLineNumbers||n.updateLineNumbers>=n.viewTo))return!1;var c=function(e){if(e.hasFocus())return null;var t=L();if(!t||!k(e.display.lineDiv,t))return null;var n={activeElt:t};if(window.getSelection){var r=window.getSelection();r.anchorNode&&r.extend&&k(e.display.lineDiv,r.anchorNode)&&(n.anchorNode=r.anchorNode,n.anchorOffset=r.anchorOffset,n.focusNode=r.focusNode,n.focusOffset=r.focusOffset)}return n}(e);return u>4&&(n.lineDiv.style.display="none"),function(e,t,n){var r=e.display,i=e.options.lineNumbers,o=r.lineDiv,a=o.firstChild;function s(t){var n=t.nextSibling;return l&&y&&e.display.currentWheelTarget==t?t.style.display="none":t.parentNode.removeChild(t),n}for(var u=r.view,c=r.viewFrom,d=0;d<u.length;d++){var h=u[d];if(h.hidden);else if(h.node&&h.node.parentNode==o){for(;a!=h.node;)a=s(a);var f=i&&null!=t&&t<=c&&h.lineNumber;h.changes&&(B(h.changes,"gutter")>-1&&(f=!1),cn(e,h,c,n)),f&&(S(h.lineNumber),h.lineNumber.appendChild(document.createTextNode($e(e.options,c)))),a=h.node.nextSibling}else{var p=mn(e,h,c,n);o.insertBefore(p,a)}c+=h.size}for(;a;)a=s(a)}(e,n.updateLineNumbers,t.dims),u>4&&(n.lineDiv.style.display=""),n.renderedView=n.view,function(e){if(e&&e.activeElt&&e.activeElt!=L()&&(e.activeElt.focus(),!/^(INPUT|TEXTAREA)$/.test(e.activeElt.nodeName)&&e.anchorNode&&k(document.body,e.anchorNode)&&k(document.body,e.focusNode))){var t=window.getSelection(),n=document.createRange();n.setEnd(e.anchorNode,e.anchorOffset),n.collapse(!1),t.removeAllRanges(),t.addRange(n),t.extend(e.focusNode,e.focusOffset)}}(c),S(n.cursorDiv),S(n.selectionDiv),n.gutters.style.height=n.sizer.style.minHeight=0,s&&(n.lastWrapHeight=t.wrapperHeight,n.lastWrapWidth=t.wrapperWidth,ri(e,400)),n.updateLineNumbers=null,!0}function si(e,t){for(var n=t.viewport,r=!0;;r=!1){if(r&&e.options.lineWrapping&&t.oldDisplayWidth!=Sn(e))r&&(t.visible=Tr(e.display,e.doc,n));else if(n&&null!=n.top&&(n={top:Math.min(e.doc.height+xn(e.display)-Tn(e),n.top)}),t.visible=Tr(e.display,e.doc,n),t.visible.from>=e.display.viewFrom&&t.visible.to<=e.display.viewTo)break;if(!ai(e,t))break;Mr(e);var i=zr(e);vr(e),Hr(e,i),ci(e,i),t.force=!1}t.signal(e,"update",e),e.display.viewFrom==e.display.reportedViewFrom&&e.display.viewTo==e.display.reportedViewTo||(t.signal(e,"viewportChange",e,e.display.viewFrom,e.display.viewTo),e.display.reportedViewFrom=e.display.viewFrom,e.display.reportedViewTo=e.display.viewTo)}function li(e,t){var n=new oi(e,t);if(ai(e,n)){Mr(e),si(e,n);var r=zr(e);vr(e),Hr(e,r),ci(e,r),n.finish()}}function ui(e){var t=e.gutters.offsetWidth;e.sizer.style.marginLeft=t+"px"}function ci(e,t){e.display.sizer.style.minHeight=t.docHeight+"px",e.display.heightForcer.style.top=t.docHeight+"px",e.display.gutters.style.height=t.docHeight+e.display.barHeight+Mn(e)+"px"}function di(e){var t=e.display,n=t.view;if(t.alignWidgets||t.gutters.firstChild&&e.options.fixedGutter){for(var r=ar(t)-t.scroller.scrollLeft+e.doc.scrollLeft,i=t.gutters.offsetWidth,o=r+"px",a=0;a<n.length;a++)if(!n[a].hidden){e.options.fixedGutter&&(n[a].gutter&&(n[a].gutter.style.left=o),n[a].gutterBackground&&(n[a].gutterBackground.style.left=o));var s=n[a].alignable;if(s)for(var l=0;l<s.length;l++)s[l].style.left=o}e.options.fixedGutter&&(t.gutters.style.left=r+i+"px")}}function hi(e){if(!e.options.lineNumbers)return!1;var t=e.doc,n=$e(e.options,t.first+t.size-1),r=e.display;if(n.length!=r.lineNumChars){var i=r.measure.appendChild(E("div",[E("div",n)],"CodeMirror-linenumber CodeMirror-gutter-elt")),o=i.firstChild.offsetWidth,a=i.offsetWidth-o;return r.lineGutter.style.width="",r.lineNumInnerWidth=Math.max(o,r.lineGutter.offsetWidth-a)+1,r.lineNumWidth=r.lineNumInnerWidth+a,r.lineNumChars=r.lineNumInnerWidth?n.length:-1,r.lineGutter.style.width=r.lineNumWidth+"px",ui(e.display),!0}return!1}function fi(e,t){for(var n=[],r=!1,i=0;i<e.length;i++){var o=e[i],a=null;if("string"!=typeof o&&(a=o.style,o=o.className),"CodeMirror-linenumbers"==o){if(!t)continue;r=!0}n.push({className:o,style:a})}return t&&!r&&n.push({className:"CodeMirror-linenumbers",style:null}),n}function pi(e){var t=e.gutters,n=e.gutterSpecs;S(t),e.lineGutter=null;for(var r=0;r<n.length;++r){var i=n[r],o=i.className,a=i.style,s=t.appendChild(E("div",null,"CodeMirror-gutter "+o));a&&(s.style.cssText=a),"CodeMirror-linenumbers"==o&&(e.lineGutter=s,s.style.width=(e.lineNumWidth||1)+"px")}t.style.display=n.length?"":"none",ui(e)}function gi(e){pi(e.display),dr(e),di(e)}function vi(e,t,r,i){var o=this;this.input=r,o.scrollbarFiller=E("div",null,"CodeMirror-scrollbar-filler"),o.scrollbarFiller.setAttribute("cm-not-content","true"),o.gutterFiller=E("div",null,"CodeMirror-gutter-filler"),o.gutterFiller.setAttribute("cm-not-content","true"),o.lineDiv=O("div",null,"CodeMirror-code"),o.selectionDiv=E("div",null,null,"position: relative; z-index: 1"),o.cursorDiv=E("div",null,"CodeMirror-cursors"),o.measure=E("div",null,"CodeMirror-measure"),o.lineMeasure=E("div",null,"CodeMirror-measure"),o.lineSpace=O("div",[o.measure,o.lineMeasure,o.selectionDiv,o.cursorDiv,o.lineDiv],null,"position: relative; outline: none");var u=O("div",[o.lineSpace],"CodeMirror-lines");o.mover=E("div",[u],null,"position: relative"),o.sizer=E("div",[o.mover],"CodeMirror-sizer"),o.sizerWidth=null,o.heightForcer=E("div",null,null,"position: absolute; height: 50px; width: 1px;"),o.gutters=E("div",null,"CodeMirror-gutters"),o.lineGutter=null,o.scroller=E("div",[o.sizer,o.heightForcer,o.gutters],"CodeMirror-scroll"),o.scroller.setAttribute("tabIndex","-1"),o.wrapper=E("div",[o.scrollbarFiller,o.gutterFiller,o.scroller],"CodeMirror"),a&&s<8&&(o.gutters.style.zIndex=-1,o.scroller.style.paddingRight=0),l||n&&m||(o.scroller.draggable=!0),e&&(e.appendChild?e.appendChild(o.wrapper):e(o.wrapper)),o.viewFrom=o.viewTo=t.first,o.reportedViewFrom=o.reportedViewTo=t.first,o.view=[],o.renderedView=null,o.externalMeasured=null,o.viewOffset=0,o.lastWrapHeight=o.lastWrapWidth=0,o.updateLineNumbers=null,o.nativeBarWidth=o.barHeight=o.barWidth=0,o.scrollbarsClipped=!1,o.lineNumWidth=o.lineNumInnerWidth=o.lineNumChars=null,o.alignWidgets=!1,o.cachedCharWidth=o.cachedTextHeight=o.cachedPaddingH=null,o.maxLine=null,o.maxLineLength=0,o.maxLineChanged=!1,o.wheelDX=o.wheelDY=o.wheelStartX=o.wheelStartY=null,o.shift=!1,o.selForContextMenu=null,o.activeTouch=null,o.gutterSpecs=fi(i.gutters,i.lineNumbers),pi(o),r.init(o)}oi.prototype.signal=function(e,t){me(e,t)&&this.events.push(arguments)},oi.prototype.finish=function(){for(var e=0;e<this.events.length;e++)pe.apply(null,this.events[e])};var mi=0,yi=null;function bi(e){var t=e.wheelDeltaX,n=e.wheelDeltaY;return null==t&&e.detail&&e.axis==e.HORIZONTAL_AXIS&&(t=e.detail),null==n&&e.detail&&e.axis==e.VERTICAL_AXIS?n=e.detail:null==n&&(n=e.wheelDelta),{x:t,y:n}}function Ci(e){var t=bi(e);return t.x*=yi,t.y*=yi,t}function Ai(e,t){var r=bi(t),i=r.x,o=r.y,a=e.display,s=a.scroller,u=s.scrollWidth>s.clientWidth,c=s.scrollHeight>s.clientHeight;if(i&&u||o&&c){if(o&&y&&l)e:for(var h=t.target,f=a.view;h!=s;h=h.parentNode)for(var p=0;p<f.length;p++)if(f[p].node==h){e.display.currentWheelTarget=h;break e}if(i&&!n&&!d&&null!=yi)return o&&c&&Nr(e,Math.max(0,s.scrollTop+o*yi)),Rr(e,Math.max(0,s.scrollLeft+i*yi)),(!o||o&&c)&&be(t),void(a.wheelStartX=null);if(o&&null!=yi){var g=o*yi,v=e.doc.scrollTop,m=v+a.wrapper.clientHeight;g<0?v=Math.max(0,v+g-50):m=Math.min(e.doc.height,m+g+50),li(e,{top:v,bottom:m})}mi<20&&(null==a.wheelStartX?(a.wheelStartX=s.scrollLeft,a.wheelStartY=s.scrollTop,a.wheelDX=i,a.wheelDY=o,setTimeout((function(){if(null!=a.wheelStartX){var e=s.scrollLeft-a.wheelStartX,t=s.scrollTop-a.wheelStartY,n=t&&a.wheelDY&&t/a.wheelDY||e&&a.wheelDX&&e/a.wheelDX;a.wheelStartX=a.wheelStartY=null,n&&(yi=(yi*mi+n)/(mi+1),++mi)}}),200)):(a.wheelDX+=i,a.wheelDY+=o))}}a?yi=-.53:n?yi=15:c?yi=-.7:h&&(yi=-1/3);var _i=function(e,t){this.ranges=e,this.primIndex=t};_i.prototype.primary=function(){return this.ranges[this.primIndex]},_i.prototype.equals=function(e){if(e==this)return!0;if(e.primIndex!=this.primIndex||e.ranges.length!=this.ranges.length)return!1;for(var t=0;t<this.ranges.length;t++){var n=this.ranges[t],r=e.ranges[t];if(!nt(n.anchor,r.anchor)||!nt(n.head,r.head))return!1}return!0},_i.prototype.deepCopy=function(){for(var e=[],t=0;t<this.ranges.length;t++)e[t]=new Ii(rt(this.ranges[t].anchor),rt(this.ranges[t].head));return new _i(e,this.primIndex)},_i.prototype.somethingSelected=function(){for(var e=0;e<this.ranges.length;e++)if(!this.ranges[e].empty())return!0;return!1},_i.prototype.contains=function(e,t){t||(t=e);for(var n=0;n<this.ranges.length;n++){var r=this.ranges[n];if(tt(t,r.from())>=0&&tt(e,r.to())<=0)return n}return-1};var Ii=function(e,t){this.anchor=e,this.head=t};function xi(e,t,n){var r=e&&e.options.selectionsMayTouch,i=t[n];t.sort((function(e,t){return tt(e.from(),t.from())})),n=B(t,i);for(var o=1;o<t.length;o++){var a=t[o],s=t[o-1],l=tt(s.to(),a.from());if(r&&!a.empty()?l>0:l>=0){var u=ot(s.from(),a.from()),c=it(s.to(),a.to()),d=s.empty()?a.from()==a.head:s.from()==s.head;o<=n&&--n,t.splice(--o,2,new Ii(d?c:u,d?u:c))}}return new _i(t,n)}function wi(e,t){return new _i([new Ii(e,t||e)],0)}function Mi(e){return e.text?et(e.from.line+e.text.length-1,q(e.text).length+(1==e.text.length?e.from.ch:0)):e.to}function Si(e,t){if(tt(e,t.from)<0)return e;if(tt(e,t.to)<=0)return Mi(t);var n=e.line+t.text.length-(t.to.line-t.from.line)-1,r=e.ch;return e.line==t.to.line&&(r+=Mi(t).ch-t.to.ch),et(n,r)}function Ti(e,t){for(var n=[],r=0;r<e.sel.ranges.length;r++){var i=e.sel.ranges[r];n.push(new Ii(Si(i.anchor,t),Si(i.head,t)))}return xi(e.cm,n,e.sel.primIndex)}function Ei(e,t,n){return e.line==t.line?et(n.line,e.ch-t.ch+n.ch):et(n.line+(e.line-t.line),e.ch)}function Oi(e){e.doc.mode=Fe(e.options,e.doc.modeOption),ki(e)}function ki(e){e.doc.iter((function(e){e.stateAfter&&(e.stateAfter=null),e.styles&&(e.styles=null)})),e.doc.modeFrontier=e.doc.highlightFrontier=e.doc.first,ri(e,100),e.state.modeGen++,e.curOp&&dr(e)}function Li(e,t){return 0==t.from.ch&&0==t.to.ch&&""==q(t.text)&&(!e.cm||e.cm.options.wholeLineUpdateBefore)}function Di(e,t,n,r){function i(e){return n?n[e]:null}function o(e,n,i){!function(e,t,n,r){e.text=t,e.stateAfter&&(e.stateAfter=null),e.styles&&(e.styles=null),null!=e.order&&(e.order=null),Tt(e),Et(e,n);var i=r?r(e):1;i!=e.height&&Ze(e,i)}(e,n,i,r),ln(e,"change",e,t)}function a(e,t){for(var n=[],o=e;o<t;++o)n.push(new Gt(u[o],i(o),r));return n}var s=t.from,l=t.to,u=t.text,c=Ge(e,s.line),d=Ge(e,l.line),h=q(u),f=i(u.length-1),p=l.line-s.line;if(t.full)e.insert(0,a(0,u.length)),e.remove(u.length,e.size-u.length);else if(Li(e,t)){var g=a(0,u.length-1);o(d,d.text,f),p&&e.remove(s.line,p),g.length&&e.insert(s.line,g)}else if(c==d)if(1==u.length)o(c,c.text.slice(0,s.ch)+h+c.text.slice(l.ch),f);else{var v=a(1,u.length-1);v.push(new Gt(h+c.text.slice(l.ch),f,r)),o(c,c.text.slice(0,s.ch)+u[0],i(0)),e.insert(s.line+1,v)}else if(1==u.length)o(c,c.text.slice(0,s.ch)+u[0]+d.text.slice(l.ch),i(0)),e.remove(s.line+1,p);else{o(c,c.text.slice(0,s.ch)+u[0],i(0)),o(d,h+d.text.slice(l.ch),f);var m=a(1,u.length-1);p>1&&e.remove(s.line+1,p-1),e.insert(s.line+1,m)}ln(e,"change",e,t)}function Pi(e,t,n){!function e(r,i,o){if(r.linked)for(var a=0;a<r.linked.length;++a){var s=r.linked[a];if(s.doc!=i){var l=o&&s.sharedHist;n&&!l||(t(s.doc,l),e(s.doc,r,l))}}}(e,null,!0)}function Ni(e,t){if(t.cm)throw new Error("This document is already in use.");e.doc=t,t.cm=e,lr(e),Oi(e),ji(e),e.options.lineWrapping||Wt(e),e.options.mode=t.modeOption,dr(e)}function ji(e){("rtl"==e.doc.direction?D:M)(e.display.lineDiv,"CodeMirror-rtl")}function Ri(e){this.done=[],this.undone=[],this.undoDepth=e?e.undoDepth:1/0,this.lastModTime=this.lastSelTime=0,this.lastOp=this.lastSelOp=null,this.lastOrigin=this.lastSelOrigin=null,this.generation=this.maxGeneration=e?e.maxGeneration:1}function zi(e,t){var n={from:rt(t.from),to:Mi(t),text:Xe(e,t.from,t.to)};return Vi(e,n,t.from.line,t.to.line+1),Pi(e,(function(e){return Vi(e,n,t.from.line,t.to.line+1)}),!0),n}function Fi(e){for(;e.length&&q(e).ranges;)e.pop()}function Bi(e,t,n,r){var i=e.history;i.undone.length=0;var o,a,s=+new Date;if((i.lastOp==r||i.lastOrigin==t.origin&&t.origin&&("+"==t.origin.charAt(0)&&i.lastModTime>s-(e.cm?e.cm.options.historyEventDelay:500)||"*"==t.origin.charAt(0)))&&(o=function(e,t){return t?(Fi(e.done),q(e.done)):e.done.length&&!q(e.done).ranges?q(e.done):e.done.length>1&&!e.done[e.done.length-2].ranges?(e.done.pop(),q(e.done)):void 0}(i,i.lastOp==r)))a=q(o.changes),0==tt(t.from,t.to)&&0==tt(t.from,a.to)?a.to=Mi(t):o.changes.push(zi(e,t));else{var l=q(i.done);for(l&&l.ranges||Ui(e.sel,i.done),o={changes:[zi(e,t)],generation:i.generation},i.done.push(o);i.done.length>i.undoDepth;)i.done.shift(),i.done[0].ranges||i.done.shift()}i.done.push(n),i.generation=++i.maxGeneration,i.lastModTime=i.lastSelTime=s,i.lastOp=i.lastSelOp=r,i.lastOrigin=i.lastSelOrigin=t.origin,a||pe(e,"historyAdded")}function Hi(e,t,n,r){var i=e.history,o=r&&r.origin;n==i.lastSelOp||o&&i.lastSelOrigin==o&&(i.lastModTime==i.lastSelTime&&i.lastOrigin==o||function(e,t,n,r){var i=t.charAt(0);return"*"==i||"+"==i&&n.ranges.length==r.ranges.length&&n.somethingSelected()==r.somethingSelected()&&new Date-e.history.lastSelTime<=(e.cm?e.cm.options.historyEventDelay:500)}(e,o,q(i.done),t))?i.done[i.done.length-1]=t:Ui(t,i.done),i.lastSelTime=+new Date,i.lastSelOrigin=o,i.lastSelOp=n,r&&!1!==r.clearRedo&&Fi(i.undone)}function Ui(e,t){var n=q(t);n&&n.ranges&&n.equals(e)||t.push(e)}function Vi(e,t,n,r){var i=t["spans_"+e.id],o=0;e.iter(Math.max(e.first,n),Math.min(e.first+e.size,r),(function(n){n.markedSpans&&((i||(i=t["spans_"+e.id]={}))[o]=n.markedSpans),++o}))}function Yi(e){if(!e)return null;for(var t,n=0;n<e.length;++n)e[n].marker.explicitlyCleared?t||(t=e.slice(0,n)):t&&t.push(e[n]);return t?t.length?t:null:e}function Wi(e,t){var n=function(e,t){var n=t["spans_"+e.id];if(!n)return null;for(var r=[],i=0;i<t.text.length;++i)r.push(Yi(n[i]));return r}(e,t),r=Mt(e,t);if(!n)return r;if(!r)return n;for(var i=0;i<n.length;++i){var o=n[i],a=r[i];if(o&&a)e:for(var s=0;s<a.length;++s){for(var l=a[s],u=0;u<o.length;++u)if(o[u].marker==l.marker)continue e;o.push(l)}else a&&(n[i]=a)}return n}function Gi(e,t,n){for(var r=[],i=0;i<e.length;++i){var o=e[i];if(o.ranges)r.push(n?_i.prototype.deepCopy.call(o):o);else{var a=o.changes,s=[];r.push({changes:s});for(var l=0;l<a.length;++l){var u=a[l],c=void 0;if(s.push({from:u.from,to:u.to,text:u.text}),t)for(var d in u)(c=d.match(/^spans_(\d+)$/))&&B(t,Number(c[1]))>-1&&(q(s)[d]=u[d],delete u[d])}}}return r}function Xi(e,t,n,r){if(r){var i=e.anchor;if(n){var o=tt(t,i)<0;o!=tt(n,i)<0?(i=t,t=n):o!=tt(t,n)<0&&(t=n)}return new Ii(i,t)}return new Ii(n||t,t)}function qi(e,t,n,r,i){null==i&&(i=e.cm&&(e.cm.display.shift||e.extend)),$i(e,new _i([Xi(e.sel.primary(),t,n,i)],0),r)}function Zi(e,t,n){for(var r=[],i=e.cm&&(e.cm.display.shift||e.extend),o=0;o<e.sel.ranges.length;o++)r[o]=Xi(e.sel.ranges[o],t[o],null,i);$i(e,xi(e.cm,r,e.sel.primIndex),n)}function Qi(e,t,n,r){var i=e.sel.ranges.slice(0);i[t]=n,$i(e,xi(e.cm,i,e.sel.primIndex),r)}function Ki(e,t,n,r){$i(e,wi(t,n),r)}function Ji(e,t,n){var r=e.history.done,i=q(r);i&&i.ranges?(r[r.length-1]=t,eo(e,t,n)):$i(e,t,n)}function $i(e,t,n){eo(e,t,n),Hi(e,e.sel,e.cm?e.cm.curOp.id:NaN,n)}function eo(e,t,n){(me(e,"beforeSelectionChange")||e.cm&&me(e.cm,"beforeSelectionChange"))&&(t=function(e,t,n){var r={ranges:t.ranges,update:function(t){this.ranges=[];for(var n=0;n<t.length;n++)this.ranges[n]=new Ii(st(e,t[n].anchor),st(e,t[n].head))},origin:n&&n.origin};return pe(e,"beforeSelectionChange",e,r),e.cm&&pe(e.cm,"beforeSelectionChange",e.cm,r),r.ranges!=t.ranges?xi(e.cm,r.ranges,r.ranges.length-1):t}(e,t,n));var r=n&&n.bias||(tt(t.primary().head,e.sel.primary().head)<0?-1:1);to(e,ro(e,t,r,!0)),n&&!1===n.scroll||!e.cm||"nocursor"==e.cm.getOption("readOnly")||kr(e.cm)}function to(e,t){t.equals(e.sel)||(e.sel=t,e.cm&&(e.cm.curOp.updateInput=1,e.cm.curOp.selectionChanged=!0,ve(e.cm)),ln(e,"cursorActivity",e))}function no(e){to(e,ro(e,e.sel,null,!1))}function ro(e,t,n,r){for(var i,o=0;o<t.ranges.length;o++){var a=t.ranges[o],s=t.ranges.length==e.sel.ranges.length&&e.sel.ranges[o],l=oo(e,a.anchor,s&&s.anchor,n,r),u=oo(e,a.head,s&&s.head,n,r);(i||l!=a.anchor||u!=a.head)&&(i||(i=t.ranges.slice(0,o)),i[o]=new Ii(l,u))}return i?xi(e.cm,i,t.primIndex):t}function io(e,t,n,r,i){var o=Ge(e,t.line);if(o.markedSpans)for(var a=0;a<o.markedSpans.length;++a){var s=o.markedSpans[a],l=s.marker,u="selectLeft"in l?!l.selectLeft:l.inclusiveLeft,c="selectRight"in l?!l.selectRight:l.inclusiveRight;if((null==s.from||(u?s.from<=t.ch:s.from<t.ch))&&(null==s.to||(c?s.to>=t.ch:s.to>t.ch))){if(i&&(pe(l,"beforeCursorEnter"),l.explicitlyCleared)){if(o.markedSpans){--a;continue}break}if(!l.atomic)continue;if(n){var d=l.find(r<0?1:-1),h=void 0;if((r<0?c:u)&&(d=ao(e,d,-r,d&&d.line==t.line?o:null)),d&&d.line==t.line&&(h=tt(d,n))&&(r<0?h<0:h>0))return io(e,d,t,r,i)}var f=l.find(r<0?-1:1);return(r<0?u:c)&&(f=ao(e,f,r,f.line==t.line?o:null)),f?io(e,f,t,r,i):null}}return t}function oo(e,t,n,r,i){var o=r||1,a=io(e,t,n,o,i)||!i&&io(e,t,n,o,!0)||io(e,t,n,-o,i)||!i&&io(e,t,n,-o,!0);return a||(e.cantEdit=!0,et(e.first,0))}function ao(e,t,n,r){return n<0&&0==t.ch?t.line>e.first?st(e,et(t.line-1)):null:n>0&&t.ch==(r||Ge(e,t.line)).text.length?t.line<e.first+e.size-1?et(t.line+1,0):null:new et(t.line,t.ch+n)}function so(e){e.setSelection(et(e.firstLine(),0),et(e.lastLine()),U)}function lo(e,t,n){var r={canceled:!1,from:t.from,to:t.to,text:t.text,origin:t.origin,cancel:function(){return r.canceled=!0}};return n&&(r.update=function(t,n,i,o){t&&(r.from=st(e,t)),n&&(r.to=st(e,n)),i&&(r.text=i),void 0!==o&&(r.origin=o)}),pe(e,"beforeChange",e,r),e.cm&&pe(e.cm,"beforeChange",e.cm,r),r.canceled?(e.cm&&(e.cm.curOp.updateInput=2),null):{from:r.from,to:r.to,text:r.text,origin:r.origin}}function uo(e,t,n){if(e.cm){if(!e.cm.curOp)return ei(e.cm,uo)(e,t,n);if(e.cm.state.suppressEdits)return}if(!(me(e,"beforeChange")||e.cm&&me(e.cm,"beforeChange"))||(t=lo(e,t,!0))){var r=At&&!n&&function(e,t,n){var r=null;if(e.iter(t.line,n.line+1,(function(e){if(e.markedSpans)for(var t=0;t<e.markedSpans.length;++t){var n=e.markedSpans[t].marker;!n.readOnly||r&&-1!=B(r,n)||(r||(r=[])).push(n)}})),!r)return null;for(var i=[{from:t,to:n}],o=0;o<r.length;++o)for(var a=r[o],s=a.find(0),l=0;l<i.length;++l){var u=i[l];if(!(tt(u.to,s.from)<0||tt(u.from,s.to)>0)){var c=[l,1],d=tt(u.from,s.from),h=tt(u.to,s.to);(d<0||!a.inclusiveLeft&&!d)&&c.push({from:u.from,to:s.from}),(h>0||!a.inclusiveRight&&!h)&&c.push({from:s.to,to:u.to}),i.splice.apply(i,c),l+=c.length-3}}return i}(e,t.from,t.to);if(r)for(var i=r.length-1;i>=0;--i)co(e,{from:r[i].from,to:r[i].to,text:i?[""]:t.text,origin:t.origin});else co(e,t)}}function co(e,t){if(1!=t.text.length||""!=t.text[0]||0!=tt(t.from,t.to)){var n=Ti(e,t);Bi(e,t,n,e.cm?e.cm.curOp.id:NaN),po(e,t,n,Mt(e,t));var r=[];Pi(e,(function(e,n){n||-1!=B(r,e.history)||(yo(e.history,t),r.push(e.history)),po(e,t,null,Mt(e,t))}))}}function ho(e,t,n){var r=e.cm&&e.cm.state.suppressEdits;if(!r||n){for(var i,o=e.history,a=e.sel,s="undo"==t?o.done:o.undone,l="undo"==t?o.undone:o.done,u=0;u<s.length&&(i=s[u],n?!i.ranges||i.equals(e.sel):i.ranges);u++);if(u!=s.length){for(o.lastOrigin=o.lastSelOrigin=null;;){if(!(i=s.pop()).ranges){if(r)return void s.push(i);break}if(Ui(i,l),n&&!i.equals(e.sel))return void $i(e,i,{clearRedo:!1});a=i}var c=[];Ui(a,l),l.push({changes:c,generation:o.generation}),o.generation=i.generation||++o.maxGeneration;for(var d=me(e,"beforeChange")||e.cm&&me(e.cm,"beforeChange"),h=function(n){var r=i.changes[n];if(r.origin=t,d&&!lo(e,r,!1))return s.length=0,{};c.push(zi(e,r));var o=n?Ti(e,r):q(s);po(e,r,o,Wi(e,r)),!n&&e.cm&&e.cm.scrollIntoView({from:r.from,to:Mi(r)});var a=[];Pi(e,(function(e,t){t||-1!=B(a,e.history)||(yo(e.history,r),a.push(e.history)),po(e,r,null,Wi(e,r))}))},f=i.changes.length-1;f>=0;--f){var p=h(f);if(p)return p.v}}}}function fo(e,t){if(0!=t&&(e.first+=t,e.sel=new _i(Z(e.sel.ranges,(function(e){return new Ii(et(e.anchor.line+t,e.anchor.ch),et(e.head.line+t,e.head.ch))})),e.sel.primIndex),e.cm)){dr(e.cm,e.first,e.first-t,t);for(var n=e.cm.display,r=n.viewFrom;r<n.viewTo;r++)hr(e.cm,r,"gutter")}}function po(e,t,n,r){if(e.cm&&!e.cm.curOp)return ei(e.cm,po)(e,t,n,r);if(t.to.line<e.first)fo(e,t.text.length-1-(t.to.line-t.from.line));else if(!(t.from.line>e.lastLine())){if(t.from.line<e.first){var i=t.text.length-1-(e.first-t.from.line);fo(e,i),t={from:et(e.first,0),to:et(t.to.line+i,t.to.ch),text:[q(t.text)],origin:t.origin}}var o=e.lastLine();t.to.line>o&&(t={from:t.from,to:et(o,Ge(e,o).text.length),text:[t.text[0]],origin:t.origin}),t.removed=Xe(e,t.from,t.to),n||(n=Ti(e,t)),e.cm?function(e,t,n){var r=e.doc,i=e.display,o=t.from,a=t.to,s=!1,l=o.line;e.options.lineWrapping||(l=Qe(zt(Ge(r,o.line))),r.iter(l,a.line+1,(function(e){if(e==i.maxLine)return s=!0,!0}))),r.sel.contains(t.from,t.to)>-1&&ve(e),Di(r,t,n,sr(e)),e.options.lineWrapping||(r.iter(l,o.line+t.text.length,(function(e){var t=Yt(e);t>i.maxLineLength&&(i.maxLine=e,i.maxLineLength=t,i.maxLineChanged=!0,s=!1)})),s&&(e.curOp.updateMaxLine=!0)),function(e,t){if(e.modeFrontier=Math.min(e.modeFrontier,t),!(e.highlightFrontier<t-10)){for(var n=e.first,r=t-1;r>n;r--){var i=Ge(e,r).stateAfter;if(i&&(!(i instanceof ut)||r+i.lookAhead<t)){n=r+1;break}}e.highlightFrontier=Math.min(e.highlightFrontier,n)}}(r,o.line),ri(e,400);var u=t.text.length-(a.line-o.line)-1;t.full?dr(e):o.line!=a.line||1!=t.text.length||Li(e.doc,t)?dr(e,o.line,a.line+1,u):hr(e,o.line,"text");var c=me(e,"changes"),d=me(e,"change");if(d||c){var h={from:o,to:a,text:t.text,removed:t.removed,origin:t.origin};d&&ln(e,"change",e,h),c&&(e.curOp.changeObjs||(e.curOp.changeObjs=[])).push(h)}e.display.selForContextMenu=null}(e.cm,t,r):Di(e,t,r),eo(e,n,U),e.cantEdit&&oo(e,et(e.firstLine(),0))&&(e.cantEdit=!1)}}function go(e,t,n,r,i){var o;r||(r=n),tt(r,n)<0&&(n=(o=[r,n])[0],r=o[1]),"string"==typeof t&&(t=e.splitLines(t)),uo(e,{from:n,to:r,text:t,origin:i})}function vo(e,t,n,r){n<e.line?e.line+=r:t<e.line&&(e.line=t,e.ch=0)}function mo(e,t,n,r){for(var i=0;i<e.length;++i){var o=e[i],a=!0;if(o.ranges){o.copied||((o=e[i]=o.deepCopy()).copied=!0);for(var s=0;s<o.ranges.length;s++)vo(o.ranges[s].anchor,t,n,r),vo(o.ranges[s].head,t,n,r)}else{for(var l=0;l<o.changes.length;++l){var u=o.changes[l];if(n<u.from.line)u.from=et(u.from.line+r,u.from.ch),u.to=et(u.to.line+r,u.to.ch);else if(t<=u.to.line){a=!1;break}}a||(e.splice(0,i+1),i=0)}}}function yo(e,t){var n=t.from.line,r=t.to.line,i=t.text.length-(r-n)-1;mo(e.done,n,r,i),mo(e.undone,n,r,i)}function bo(e,t,n,r){var i=t,o=t;return"number"==typeof t?o=Ge(e,at(e,t)):i=Qe(t),null==i?null:(r(o,i)&&e.cm&&hr(e.cm,i,n),o)}function Co(e){this.lines=e,this.parent=null;for(var t=0,n=0;n<e.length;++n)e[n].parent=this,t+=e[n].height;this.height=t}function Ao(e){this.children=e;for(var t=0,n=0,r=0;r<e.length;++r){var i=e[r];t+=i.chunkSize(),n+=i.height,i.parent=this}this.size=t,this.height=n,this.parent=null}Ii.prototype.from=function(){return ot(this.anchor,this.head)},Ii.prototype.to=function(){return it(this.anchor,this.head)},Ii.prototype.empty=function(){return this.head.line==this.anchor.line&&this.head.ch==this.anchor.ch},Co.prototype={chunkSize:function(){return this.lines.length},removeInner:function(e,t){for(var n=e,r=e+t;n<r;++n){var i=this.lines[n];this.height-=i.height,Xt(i),ln(i,"delete")}this.lines.splice(e,t)},collapse:function(e){e.push.apply(e,this.lines)},insertInner:function(e,t,n){this.height+=n,this.lines=this.lines.slice(0,e).concat(t).concat(this.lines.slice(e));for(var r=0;r<t.length;++r)t[r].parent=this},iterN:function(e,t,n){for(var r=e+t;e<r;++e)if(n(this.lines[e]))return!0}},Ao.prototype={chunkSize:function(){return this.size},removeInner:function(e,t){this.size-=t;for(var n=0;n<this.children.length;++n){var r=this.children[n],i=r.chunkSize();if(e<i){var o=Math.min(t,i-e),a=r.height;if(r.removeInner(e,o),this.height-=a-r.height,i==o&&(this.children.splice(n--,1),r.parent=null),0==(t-=o))break;e=0}else e-=i}if(this.size-t<25&&(this.children.length>1||!(this.children[0]instanceof Co))){var s=[];this.collapse(s),this.children=[new Co(s)],this.children[0].parent=this}},collapse:function(e){for(var t=0;t<this.children.length;++t)this.children[t].collapse(e)},insertInner:function(e,t,n){this.size+=t.length,this.height+=n;for(var r=0;r<this.children.length;++r){var i=this.children[r],o=i.chunkSize();if(e<=o){if(i.insertInner(e,t,n),i.lines&&i.lines.length>50){for(var a=i.lines.length%25+25,s=a;s<i.lines.length;){var l=new Co(i.lines.slice(s,s+=25));i.height-=l.height,this.children.splice(++r,0,l),l.parent=this}i.lines=i.lines.slice(0,a),this.maybeSpill()}break}e-=o}},maybeSpill:function(){if(!(this.children.length<=10)){var e=this;do{var t=new Ao(e.children.splice(e.children.length-5,5));if(e.parent){e.size-=t.size,e.height-=t.height;var n=B(e.parent.children,e);e.parent.children.splice(n+1,0,t)}else{var r=new Ao(e.children);r.parent=e,e.children=[r,t],e=r}t.parent=e.parent}while(e.children.length>10);e.parent.maybeSpill()}},iterN:function(e,t,n){for(var r=0;r<this.children.length;++r){var i=this.children[r],o=i.chunkSize();if(e<o){var a=Math.min(t,o-e);if(i.iterN(e,a,n))return!0;if(0==(t-=a))break;e=0}else e-=o}}};var _o=function(e,t,n){if(n)for(var r in n)n.hasOwnProperty(r)&&(this[r]=n[r]);this.doc=e,this.node=t};function Io(e,t,n){Vt(t)<(e.curOp&&e.curOp.scrollTop||e.doc.scrollTop)&&Or(e,n)}_o.prototype.clear=function(){var e=this.doc.cm,t=this.line.widgets,n=this.line,r=Qe(n);if(null!=r&&t){for(var i=0;i<t.length;++i)t[i]==this&&t.splice(i--,1);t.length||(n.widgets=null);var o=An(this);Ze(n,Math.max(0,n.height-o)),e&&($r(e,(function(){Io(e,n,-o),hr(e,r,"widget")})),ln(e,"lineWidgetCleared",e,this,r))}},_o.prototype.changed=function(){var e=this,t=this.height,n=this.doc.cm,r=this.line;this.height=null;var i=An(this)-t;i&&(Ht(this.doc,r)||Ze(r,r.height+i),n&&$r(n,(function(){n.curOp.forceUpdate=!0,Io(n,r,i),ln(n,"lineWidgetChanged",n,e,Qe(r))})))},ye(_o);var xo=0,wo=function(e,t){this.lines=[],this.type=t,this.doc=e,this.id=++xo};function Mo(e,t,n,r,i){if(r&&r.shared)return function(e,t,n,r,i){(r=R(r)).shared=!1;var o=[Mo(e,t,n,r,i)],a=o[0],s=r.widgetNode;return Pi(e,(function(e){s&&(r.widgetNode=s.cloneNode(!0)),o.push(Mo(e,st(e,t),st(e,n),r,i));for(var l=0;l<e.linked.length;++l)if(e.linked[l].isParent)return;a=q(o)})),new So(o,a)}(e,t,n,r,i);if(e.cm&&!e.cm.curOp)return ei(e.cm,Mo)(e,t,n,r,i);var o=new wo(e,i),a=tt(t,n);if(r&&R(r,o,!1),a>0||0==a&&!1!==o.clearWhenEmpty)return o;if(o.replacedWith&&(o.collapsed=!0,o.widgetNode=O("span",[o.replacedWith],"CodeMirror-widget"),r.handleMouseEvents||o.widgetNode.setAttribute("cm-ignore-events","true"),r.insertLeft&&(o.widgetNode.insertLeft=!0)),o.collapsed){if(Rt(e,t.line,t,n,o)||t.line!=n.line&&Rt(e,n.line,t,n,o))throw new Error("Inserting collapsed marker partially overlapping an existing one");_t=!0}o.addToHistory&&Bi(e,{from:t,to:n,origin:"markText"},e.sel,NaN);var s,l=t.line,u=e.cm;if(e.iter(l,n.line+1,(function(e){u&&o.collapsed&&!u.options.lineWrapping&&zt(e)==u.display.maxLine&&(s=!0),o.collapsed&&l!=t.line&&Ze(e,0),function(e,t){e.markedSpans=e.markedSpans?e.markedSpans.concat([t]):[t],t.marker.attachLine(e)}(e,new It(o,l==t.line?t.ch:null,l==n.line?n.ch:null)),++l})),o.collapsed&&e.iter(t.line,n.line+1,(function(t){Ht(e,t)&&Ze(t,0)})),o.clearOnEnter&&de(o,"beforeCursorEnter",(function(){return o.clear()})),o.readOnly&&(At=!0,(e.history.done.length||e.history.undone.length)&&e.clearHistory()),o.collapsed&&(o.id=++xo,o.atomic=!0),u){if(s&&(u.curOp.updateMaxLine=!0),o.collapsed)dr(u,t.line,n.line+1);else if(o.className||o.startStyle||o.endStyle||o.css||o.attributes||o.title)for(var c=t.line;c<=n.line;c++)hr(u,c,"text");o.atomic&&no(u.doc),ln(u,"markerAdded",u,o)}return o}wo.prototype.clear=function(){if(!this.explicitlyCleared){var e=this.doc.cm,t=e&&!e.curOp;if(t&&Gr(e),me(this,"clear")){var n=this.find();n&&ln(this,"clear",n.from,n.to)}for(var r=null,i=null,o=0;o<this.lines.length;++o){var a=this.lines[o],s=xt(a.markedSpans,this);e&&!this.collapsed?hr(e,Qe(a),"text"):e&&(null!=s.to&&(i=Qe(a)),null!=s.from&&(r=Qe(a))),a.markedSpans=wt(a.markedSpans,s),null==s.from&&this.collapsed&&!Ht(this.doc,a)&&e&&Ze(a,rr(e.display))}if(e&&this.collapsed&&!e.options.lineWrapping)for(var l=0;l<this.lines.length;++l){var u=zt(this.lines[l]),c=Yt(u);c>e.display.maxLineLength&&(e.display.maxLine=u,e.display.maxLineLength=c,e.display.maxLineChanged=!0)}null!=r&&e&&this.collapsed&&dr(e,r,i+1),this.lines.length=0,this.explicitlyCleared=!0,this.atomic&&this.doc.cantEdit&&(this.doc.cantEdit=!1,e&&no(e.doc)),e&&ln(e,"markerCleared",e,this,r,i),t&&Xr(e),this.parent&&this.parent.clear()}},wo.prototype.find=function(e,t){var n,r;null==e&&"bookmark"==this.type&&(e=1);for(var i=0;i<this.lines.length;++i){var o=this.lines[i],a=xt(o.markedSpans,this);if(null!=a.from&&(n=et(t?o:Qe(o),a.from),-1==e))return n;if(null!=a.to&&(r=et(t?o:Qe(o),a.to),1==e))return r}return n&&{from:n,to:r}},wo.prototype.changed=function(){var e=this,t=this.find(-1,!0),n=this,r=this.doc.cm;t&&r&&$r(r,(function(){var i=t.line,o=Qe(t.line),a=kn(r,o);if(a&&(zn(a),r.curOp.selectionChanged=r.curOp.forceUpdate=!0),r.curOp.updateMaxLine=!0,!Ht(n.doc,i)&&null!=n.height){var s=n.height;n.height=null;var l=An(n)-s;l&&Ze(i,i.height+l)}ln(r,"markerChanged",r,e)}))},wo.prototype.attachLine=function(e){if(!this.lines.length&&this.doc.cm){var t=this.doc.cm.curOp;t.maybeHiddenMarkers&&-1!=B(t.maybeHiddenMarkers,this)||(t.maybeUnhiddenMarkers||(t.maybeUnhiddenMarkers=[])).push(this)}this.lines.push(e)},wo.prototype.detachLine=function(e){if(this.lines.splice(B(this.lines,e),1),!this.lines.length&&this.doc.cm){var t=this.doc.cm.curOp;(t.maybeHiddenMarkers||(t.maybeHiddenMarkers=[])).push(this)}},ye(wo);var So=function(e,t){this.markers=e,this.primary=t;for(var n=0;n<e.length;++n)e[n].parent=this};function To(e){return e.findMarks(et(e.first,0),e.clipPos(et(e.lastLine())),(function(e){return e.parent}))}function Eo(e){for(var t=function(t){var n=e[t],r=[n.primary.doc];Pi(n.primary.doc,(function(e){return r.push(e)}));for(var i=0;i<n.markers.length;i++){var o=n.markers[i];-1==B(r,o.doc)&&(o.parent=null,n.markers.splice(i--,1))}},n=0;n<e.length;n++)t(n)}So.prototype.clear=function(){if(!this.explicitlyCleared){this.explicitlyCleared=!0;for(var e=0;e<this.markers.length;++e)this.markers[e].clear();ln(this,"clear")}},So.prototype.find=function(e,t){return this.primary.find(e,t)},ye(So);var Oo=0,ko=function(e,t,n,r,i){if(!(this instanceof ko))return new ko(e,t,n,r,i);null==n&&(n=0),Ao.call(this,[new Co([new Gt("",null)])]),this.first=n,this.scrollTop=this.scrollLeft=0,this.cantEdit=!1,this.cleanGeneration=1,this.modeFrontier=this.highlightFrontier=n;var o=et(n,0);this.sel=wi(o),this.history=new Ri(null),this.id=++Oo,this.modeOption=t,this.lineSep=r,this.direction="rtl"==i?"rtl":"ltr",this.extend=!1,"string"==typeof e&&(e=this.splitLines(e)),Di(this,{from:o,to:o,text:e}),$i(this,wi(o),U)};ko.prototype=K(Ao.prototype,{constructor:ko,iter:function(e,t,n){n?this.iterN(e-this.first,t-e,n):this.iterN(this.first,this.first+this.size,e)},insert:function(e,t){for(var n=0,r=0;r<t.length;++r)n+=t[r].height;this.insertInner(e-this.first,t,n)},remove:function(e,t){this.removeInner(e-this.first,t)},getValue:function(e){var t=qe(this,this.first,this.first+this.size);return!1===e?t:t.join(e||this.lineSeparator())},setValue:ni((function(e){var t=et(this.first,0),n=this.first+this.size-1;uo(this,{from:t,to:et(n,Ge(this,n).text.length),text:this.splitLines(e),origin:"setValue",full:!0},!0),this.cm&&Lr(this.cm,0,0),$i(this,wi(t),U)})),replaceRange:function(e,t,n,r){go(this,e,t=st(this,t),n=n?st(this,n):t,r)},getRange:function(e,t,n){var r=Xe(this,st(this,e),st(this,t));return!1===n?r:r.join(n||this.lineSeparator())},getLine:function(e){var t=this.getLineHandle(e);return t&&t.text},getLineHandle:function(e){if(Je(this,e))return Ge(this,e)},getLineNumber:function(e){return Qe(e)},getLineHandleVisualStart:function(e){return"number"==typeof e&&(e=Ge(this,e)),zt(e)},lineCount:function(){return this.size},firstLine:function(){return this.first},lastLine:function(){return this.first+this.size-1},clipPos:function(e){return st(this,e)},getCursor:function(e){var t=this.sel.primary();return null==e||"head"==e?t.head:"anchor"==e?t.anchor:"end"==e||"to"==e||!1===e?t.to():t.from()},listSelections:function(){return this.sel.ranges},somethingSelected:function(){return this.sel.somethingSelected()},setCursor:ni((function(e,t,n){Ki(this,st(this,"number"==typeof e?et(e,t||0):e),null,n)})),setSelection:ni((function(e,t,n){Ki(this,st(this,e),st(this,t||e),n)})),extendSelection:ni((function(e,t,n){qi(this,st(this,e),t&&st(this,t),n)})),extendSelections:ni((function(e,t){Zi(this,lt(this,e),t)})),extendSelectionsBy:ni((function(e,t){Zi(this,lt(this,Z(this.sel.ranges,e)),t)})),setSelections:ni((function(e,t,n){if(e.length){for(var r=[],i=0;i<e.length;i++)r[i]=new Ii(st(this,e[i].anchor),st(this,e[i].head));null==t&&(t=Math.min(e.length-1,this.sel.primIndex)),$i(this,xi(this.cm,r,t),n)}})),addSelection:ni((function(e,t,n){var r=this.sel.ranges.slice(0);r.push(new Ii(st(this,e),st(this,t||e))),$i(this,xi(this.cm,r,r.length-1),n)})),getSelection:function(e){for(var t,n=this.sel.ranges,r=0;r<n.length;r++){var i=Xe(this,n[r].from(),n[r].to());t=t?t.concat(i):i}return!1===e?t:t.join(e||this.lineSeparator())},getSelections:function(e){for(var t=[],n=this.sel.ranges,r=0;r<n.length;r++){var i=Xe(this,n[r].from(),n[r].to());!1!==e&&(i=i.join(e||this.lineSeparator())),t[r]=i}return t},replaceSelection:function(e,t,n){for(var r=[],i=0;i<this.sel.ranges.length;i++)r[i]=e;this.replaceSelections(r,t,n||"+input")},replaceSelections:ni((function(e,t,n){for(var r=[],i=this.sel,o=0;o<i.ranges.length;o++){var a=i.ranges[o];r[o]={from:a.from(),to:a.to(),text:this.splitLines(e[o]),origin:n}}for(var s=t&&"end"!=t&&function(e,t,n){for(var r=[],i=et(e.first,0),o=i,a=0;a<t.length;a++){var s=t[a],l=Ei(s.from,i,o),u=Ei(Mi(s),i,o);if(i=s.to,o=u,"around"==n){var c=e.sel.ranges[a],d=tt(c.head,c.anchor)<0;r[a]=new Ii(d?u:l,d?l:u)}else r[a]=new Ii(l,l)}return new _i(r,e.sel.primIndex)}(this,r,t),l=r.length-1;l>=0;l--)uo(this,r[l]);s?Ji(this,s):this.cm&&kr(this.cm)})),undo:ni((function(){ho(this,"undo")})),redo:ni((function(){ho(this,"redo")})),undoSelection:ni((function(){ho(this,"undo",!0)})),redoSelection:ni((function(){ho(this,"redo",!0)})),setExtending:function(e){this.extend=e},getExtending:function(){return this.extend},historySize:function(){for(var e=this.history,t=0,n=0,r=0;r<e.done.length;r++)e.done[r].ranges||++t;for(var i=0;i<e.undone.length;i++)e.undone[i].ranges||++n;return{undo:t,redo:n}},clearHistory:function(){var e=this;this.history=new Ri(this.history),Pi(this,(function(t){return t.history=e.history}),!0)},markClean:function(){this.cleanGeneration=this.changeGeneration(!0)},changeGeneration:function(e){return e&&(this.history.lastOp=this.history.lastSelOp=this.history.lastOrigin=null),this.history.generation},isClean:function(e){return this.history.generation==(e||this.cleanGeneration)},getHistory:function(){return{done:Gi(this.history.done),undone:Gi(this.history.undone)}},setHistory:function(e){var t=this.history=new Ri(this.history);t.done=Gi(e.done.slice(0),null,!0),t.undone=Gi(e.undone.slice(0),null,!0)},setGutterMarker:ni((function(e,t,n){return bo(this,e,"gutter",(function(e){var r=e.gutterMarkers||(e.gutterMarkers={});return r[t]=n,!n&&te(r)&&(e.gutterMarkers=null),!0}))})),clearGutter:ni((function(e){var t=this;this.iter((function(n){n.gutterMarkers&&n.gutterMarkers[e]&&bo(t,n,"gutter",(function(){return n.gutterMarkers[e]=null,te(n.gutterMarkers)&&(n.gutterMarkers=null),!0}))}))})),lineInfo:function(e){var t;if("number"==typeof e){if(!Je(this,e))return null;if(t=e,!(e=Ge(this,e)))return null}else if(null==(t=Qe(e)))return null;return{line:t,handle:e,text:e.text,gutterMarkers:e.gutterMarkers,textClass:e.textClass,bgClass:e.bgClass,wrapClass:e.wrapClass,widgets:e.widgets}},addLineClass:ni((function(e,t,n){return bo(this,e,"gutter"==t?"gutter":"class",(function(e){var r="text"==t?"textClass":"background"==t?"bgClass":"gutter"==t?"gutterClass":"wrapClass";if(e[r]){if(x(n).test(e[r]))return!1;e[r]+=" "+n}else e[r]=n;return!0}))})),removeLineClass:ni((function(e,t,n){return bo(this,e,"gutter"==t?"gutter":"class",(function(e){var r="text"==t?"textClass":"background"==t?"bgClass":"gutter"==t?"gutterClass":"wrapClass",i=e[r];if(!i)return!1;if(null==n)e[r]=null;else{var o=i.match(x(n));if(!o)return!1;var a=o.index+o[0].length;e[r]=i.slice(0,o.index)+(o.index&&a!=i.length?" ":"")+i.slice(a)||null}return!0}))})),addLineWidget:ni((function(e,t,n){return function(e,t,n,r){var i=new _o(e,n,r),o=e.cm;return o&&i.noHScroll&&(o.display.alignWidgets=!0),bo(e,t,"widget",(function(t){var n=t.widgets||(t.widgets=[]);if(null==i.insertAt?n.push(i):n.splice(Math.min(n.length,Math.max(0,i.insertAt)),0,i),i.line=t,o&&!Ht(e,t)){var r=Vt(t)<e.scrollTop;Ze(t,t.height+An(i)),r&&Or(o,i.height),o.curOp.forceUpdate=!0}return!0})),o&&ln(o,"lineWidgetAdded",o,i,"number"==typeof t?t:Qe(t)),i}(this,e,t,n)})),removeLineWidget:function(e){e.clear()},markText:function(e,t,n){return Mo(this,st(this,e),st(this,t),n,n&&n.type||"range")},setBookmark:function(e,t){var n={replacedWith:t&&(null==t.nodeType?t.widget:t),insertLeft:t&&t.insertLeft,clearWhenEmpty:!1,shared:t&&t.shared,handleMouseEvents:t&&t.handleMouseEvents};return Mo(this,e=st(this,e),e,n,"bookmark")},findMarksAt:function(e){var t=[],n=Ge(this,(e=st(this,e)).line).markedSpans;if(n)for(var r=0;r<n.length;++r){var i=n[r];(null==i.from||i.from<=e.ch)&&(null==i.to||i.to>=e.ch)&&t.push(i.marker.parent||i.marker)}return t},findMarks:function(e,t,n){e=st(this,e),t=st(this,t);var r=[],i=e.line;return this.iter(e.line,t.line+1,(function(o){var a=o.markedSpans;if(a)for(var s=0;s<a.length;s++){var l=a[s];null!=l.to&&i==e.line&&e.ch>=l.to||null==l.from&&i!=e.line||null!=l.from&&i==t.line&&l.from>=t.ch||n&&!n(l.marker)||r.push(l.marker.parent||l.marker)}++i})),r},getAllMarks:function(){var e=[];return this.iter((function(t){var n=t.markedSpans;if(n)for(var r=0;r<n.length;++r)null!=n[r].from&&e.push(n[r].marker)})),e},posFromIndex:function(e){var t,n=this.first,r=this.lineSeparator().length;return this.iter((function(i){var o=i.text.length+r;if(o>e)return t=e,!0;e-=o,++n})),st(this,et(n,t))},indexFromPos:function(e){var t=(e=st(this,e)).ch;if(e.line<this.first||e.ch<0)return 0;var n=this.lineSeparator().length;return this.iter(this.first,e.line,(function(e){t+=e.text.length+n})),t},copy:function(e){var t=new ko(qe(this,this.first,this.first+this.size),this.modeOption,this.first,this.lineSep,this.direction);return t.scrollTop=this.scrollTop,t.scrollLeft=this.scrollLeft,t.sel=this.sel,t.extend=!1,e&&(t.history.undoDepth=this.history.undoDepth,t.setHistory(this.getHistory())),t},linkedDoc:function(e){e||(e={});var t=this.first,n=this.first+this.size;null!=e.from&&e.from>t&&(t=e.from),null!=e.to&&e.to<n&&(n=e.to);var r=new ko(qe(this,t,n),e.mode||this.modeOption,t,this.lineSep,this.direction);return e.sharedHist&&(r.history=this.history),(this.linked||(this.linked=[])).push({doc:r,sharedHist:e.sharedHist}),r.linked=[{doc:this,isParent:!0,sharedHist:e.sharedHist}],function(e,t){for(var n=0;n<t.length;n++){var r=t[n],i=r.find(),o=e.clipPos(i.from),a=e.clipPos(i.to);if(tt(o,a)){var s=Mo(e,o,a,r.primary,r.primary.type);r.markers.push(s),s.parent=r}}}(r,To(this)),r},unlinkDoc:function(e){if(e instanceof Sa&&(e=e.doc),this.linked)for(var t=0;t<this.linked.length;++t)if(this.linked[t].doc==e){this.linked.splice(t,1),e.unlinkDoc(this),Eo(To(this));break}if(e.history==this.history){var n=[e.id];Pi(e,(function(e){return n.push(e.id)}),!0),e.history=new Ri(null),e.history.done=Gi(this.history.done,n),e.history.undone=Gi(this.history.undone,n)}},iterLinkedDocs:function(e){Pi(this,e)},getMode:function(){return this.mode},getEditor:function(){return this.cm},splitLines:function(e){return this.lineSep?e.split(this.lineSep):ke(e)},lineSeparator:function(){return this.lineSep||"\n"},setDirection:ni((function(e){var t;"rtl"!=e&&(e="ltr"),e!=this.direction&&(this.direction=e,this.iter((function(e){return e.order=null})),this.cm&&$r(t=this.cm,(function(){ji(t),dr(t)})))}))}),ko.prototype.eachLine=ko.prototype.iter;var Lo=0;function Do(e){var t=this;if(Po(t),!ge(t,e)&&!_n(t.display,e)){be(e),a&&(Lo=+new Date);var n=ur(t,e,!0),r=e.dataTransfer.files;if(n&&!t.isReadOnly())if(r&&r.length&&window.FileReader&&window.File)for(var i=r.length,o=Array(i),s=0,l=function(){++s==i&&ei(t,(function(){var e={from:n=st(t.doc,n),to:n,text:t.doc.splitLines(o.filter((function(e){return null!=e})).join(t.doc.lineSeparator())),origin:"paste"};uo(t.doc,e),Ji(t.doc,wi(st(t.doc,n),st(t.doc,Mi(e))))}))()},u=function(e,n){if(t.options.allowDropFileTypes&&-1==B(t.options.allowDropFileTypes,e.type))l();else{var r=new FileReader;r.onerror=function(){return l()},r.onload=function(){var e=r.result;/[\x00-\x08\x0e-\x1f]{2}/.test(e)||(o[n]=e),l()},r.readAsText(e)}},c=0;c<r.length;c++)u(r[c],c);else{if(t.state.draggingText&&t.doc.sel.contains(n)>-1)return t.state.draggingText(e),void setTimeout((function(){return t.display.input.focus()}),20);try{var d=e.dataTransfer.getData("Text");if(d){var h;if(t.state.draggingText&&!t.state.draggingText.copy&&(h=t.listSelections()),eo(t.doc,wi(n,n)),h)for(var f=0;f<h.length;++f)go(t.doc,"",h[f].anchor,h[f].head,"drag");t.replaceSelection(d,"around","paste"),t.display.input.focus()}}catch(e){}}}}function Po(e){e.display.dragCursor&&(e.display.lineSpace.removeChild(e.display.dragCursor),e.display.dragCursor=null)}function No(e){if(document.getElementsByClassName){for(var t=document.getElementsByClassName("CodeMirror"),n=[],r=0;r<t.length;r++){var i=t[r].CodeMirror;i&&n.push(i)}n.length&&n[0].operation((function(){for(var t=0;t<n.length;t++)e(n[t])}))}}var jo=!1;function Ro(){var e;jo||(de(window,"resize",(function(){null==e&&(e=setTimeout((function(){e=null,No(zo)}),100))})),de(window,"blur",(function(){return No(wr)})),jo=!0)}function zo(e){var t=e.display;t.cachedCharWidth=t.cachedTextHeight=t.cachedPaddingH=null,t.scrollbarsClipped=!1,e.setSize()}for(var Fo={3:"Pause",8:"Backspace",9:"Tab",13:"Enter",16:"Shift",17:"Ctrl",18:"Alt",19:"Pause",20:"CapsLock",27:"Esc",32:"Space",33:"PageUp",34:"PageDown",35:"End",36:"Home",37:"Left",38:"Up",39:"Right",40:"Down",44:"PrintScrn",45:"Insert",46:"Delete",59:";",61:"=",91:"Mod",92:"Mod",93:"Mod",106:"*",107:"=",109:"-",110:".",111:"/",145:"ScrollLock",173:"-",186:";",187:"=",188:",",189:"-",190:".",191:"/",192:"`",219:"[",220:"\\",221:"]",222:"'",224:"Mod",63232:"Up",63233:"Down",63234:"Left",63235:"Right",63272:"Delete",63273:"Home",63275:"End",63276:"PageUp",63277:"PageDown",63302:"Insert"},Bo=0;Bo<10;Bo++)Fo[Bo+48]=Fo[Bo+96]=String(Bo);for(var Ho=65;Ho<=90;Ho++)Fo[Ho]=String.fromCharCode(Ho);for(var Uo=1;Uo<=12;Uo++)Fo[Uo+111]=Fo[Uo+63235]="F"+Uo;var Vo={};function Yo(e){var t,n,r,i,o=e.split(/-(?!$)/);e=o[o.length-1];for(var a=0;a<o.length-1;a++){var s=o[a];if(/^(cmd|meta|m)$/i.test(s))i=!0;else if(/^a(lt)?$/i.test(s))t=!0;else if(/^(c|ctrl|control)$/i.test(s))n=!0;else{if(!/^s(hift)?$/i.test(s))throw new Error("Unrecognized modifier name: "+s);r=!0}}return t&&(e="Alt-"+e),n&&(e="Ctrl-"+e),i&&(e="Cmd-"+e),r&&(e="Shift-"+e),e}function Wo(e){var t={};for(var n in e)if(e.hasOwnProperty(n)){var r=e[n];if(/^(name|fallthrough|(de|at)tach)$/.test(n))continue;if("..."==r){delete e[n];continue}for(var i=Z(n.split(" "),Yo),o=0;o<i.length;o++){var a=void 0,s=void 0;o==i.length-1?(s=i.join(" "),a=r):(s=i.slice(0,o+1).join(" "),a="...");var l=t[s];if(l){if(l!=a)throw new Error("Inconsistent bindings for "+s)}else t[s]=a}delete e[n]}for(var u in t)e[u]=t[u];return e}function Go(e,t,n,r){var i=(t=Qo(t)).call?t.call(e,r):t[e];if(!1===i)return"nothing";if("..."===i)return"multi";if(null!=i&&n(i))return"handled";if(t.fallthrough){if("[object Array]"!=Object.prototype.toString.call(t.fallthrough))return Go(e,t.fallthrough,n,r);for(var o=0;o<t.fallthrough.length;o++){var a=Go(e,t.fallthrough[o],n,r);if(a)return a}}}function Xo(e){var t="string"==typeof e?e:Fo[e.keyCode];return"Ctrl"==t||"Alt"==t||"Shift"==t||"Mod"==t}function qo(e,t,n){var r=e;return t.altKey&&"Alt"!=r&&(e="Alt-"+e),(_?t.metaKey:t.ctrlKey)&&"Ctrl"!=r&&(e="Ctrl-"+e),(_?t.ctrlKey:t.metaKey)&&"Mod"!=r&&(e="Cmd-"+e),!n&&t.shiftKey&&"Shift"!=r&&(e="Shift-"+e),e}function Zo(e,t){if(d&&34==e.keyCode&&e.char)return!1;var n=Fo[e.keyCode];return null!=n&&!e.altGraphKey&&(3==e.keyCode&&e.code&&(n=e.code),qo(n,e,t))}function Qo(e){return"string"==typeof e?Vo[e]:e}function Ko(e,t){for(var n=e.doc.sel.ranges,r=[],i=0;i<n.length;i++){for(var o=t(n[i]);r.length&&tt(o.from,q(r).to)<=0;){var a=r.pop();if(tt(a.from,o.from)<0){o.from=a.from;break}}r.push(o)}$r(e,(function(){for(var t=r.length-1;t>=0;t--)go(e.doc,"",r[t].from,r[t].to,"+delete");kr(e)}))}function Jo(e,t,n){var r=ie(e.text,t+n,n);return r<0||r>e.text.length?null:r}function $o(e,t,n){var r=Jo(e,t.ch,n);return null==r?null:new et(t.line,r,n<0?"after":"before")}function ea(e,t,n,r,i){if(e){"rtl"==t.doc.direction&&(i=-i);var o=ue(n,t.doc.direction);if(o){var a,s=i<0?q(o):o[0],l=i<0==(1==s.level)?"after":"before";if(s.level>0||"rtl"==t.doc.direction){var u=Ln(t,n);a=i<0?n.text.length-1:0;var c=Dn(t,u,a).top;a=oe((function(e){return Dn(t,u,e).top==c}),i<0==(1==s.level)?s.from:s.to-1,a),"before"==l&&(a=Jo(n,a,1))}else a=i<0?s.to:s.from;return new et(r,a,l)}}return new et(r,i<0?n.text.length:0,i<0?"before":"after")}Vo.basic={Left:"goCharLeft",Right:"goCharRight",Up:"goLineUp",Down:"goLineDown",End:"goLineEnd",Home:"goLineStartSmart",PageUp:"goPageUp",PageDown:"goPageDown",Delete:"delCharAfter",Backspace:"delCharBefore","Shift-Backspace":"delCharBefore",Tab:"defaultTab","Shift-Tab":"indentAuto",Enter:"newlineAndIndent",Insert:"toggleOverwrite",Esc:"singleSelection"},Vo.pcDefault={"Ctrl-A":"selectAll","Ctrl-D":"deleteLine","Ctrl-Z":"undo","Shift-Ctrl-Z":"redo","Ctrl-Y":"redo","Ctrl-Home":"goDocStart","Ctrl-End":"goDocEnd","Ctrl-Up":"goLineUp","Ctrl-Down":"goLineDown","Ctrl-Left":"goGroupLeft","Ctrl-Right":"goGroupRight","Alt-Left":"goLineStart","Alt-Right":"goLineEnd","Ctrl-Backspace":"delGroupBefore","Ctrl-Delete":"delGroupAfter","Ctrl-S":"save","Ctrl-F":"find","Ctrl-G":"findNext","Shift-Ctrl-G":"findPrev","Shift-Ctrl-F":"replace","Shift-Ctrl-R":"replaceAll","Ctrl-[":"indentLess","Ctrl-]":"indentMore","Ctrl-U":"undoSelection","Shift-Ctrl-U":"redoSelection","Alt-U":"redoSelection",fallthrough:"basic"},Vo.emacsy={"Ctrl-F":"goCharRight","Ctrl-B":"goCharLeft","Ctrl-P":"goLineUp","Ctrl-N":"goLineDown","Alt-F":"goWordRight","Alt-B":"goWordLeft","Ctrl-A":"goLineStart","Ctrl-E":"goLineEnd","Ctrl-V":"goPageDown","Shift-Ctrl-V":"goPageUp","Ctrl-D":"delCharAfter","Ctrl-H":"delCharBefore","Alt-D":"delWordAfter","Alt-Backspace":"delWordBefore","Ctrl-K":"killLine","Ctrl-T":"transposeChars","Ctrl-O":"openLine"},Vo.macDefault={"Cmd-A":"selectAll","Cmd-D":"deleteLine","Cmd-Z":"undo","Shift-Cmd-Z":"redo","Cmd-Y":"redo","Cmd-Home":"goDocStart","Cmd-Up":"goDocStart","Cmd-End":"goDocEnd","Cmd-Down":"goDocEnd","Alt-Left":"goGroupLeft","Alt-Right":"goGroupRight","Cmd-Left":"goLineLeft","Cmd-Right":"goLineRight","Alt-Backspace":"delGroupBefore","Ctrl-Alt-Backspace":"delGroupAfter","Alt-Delete":"delGroupAfter","Cmd-S":"save","Cmd-F":"find","Cmd-G":"findNext","Shift-Cmd-G":"findPrev","Cmd-Alt-F":"replace","Shift-Cmd-Alt-F":"replaceAll","Cmd-[":"indentLess","Cmd-]":"indentMore","Cmd-Backspace":"delWrappedLineLeft","Cmd-Delete":"delWrappedLineRight","Cmd-U":"undoSelection","Shift-Cmd-U":"redoSelection","Ctrl-Up":"goDocStart","Ctrl-Down":"goDocEnd",fallthrough:["basic","emacsy"]},Vo.default=y?Vo.macDefault:Vo.pcDefault;var ta={selectAll:so,singleSelection:function(e){return e.setSelection(e.getCursor("anchor"),e.getCursor("head"),U)},killLine:function(e){return Ko(e,(function(t){if(t.empty()){var n=Ge(e.doc,t.head.line).text.length;return t.head.ch==n&&t.head.line<e.lastLine()?{from:t.head,to:et(t.head.line+1,0)}:{from:t.head,to:et(t.head.line,n)}}return{from:t.from(),to:t.to()}}))},deleteLine:function(e){return Ko(e,(function(t){return{from:et(t.from().line,0),to:st(e.doc,et(t.to().line+1,0))}}))},delLineLeft:function(e){return Ko(e,(function(e){return{from:et(e.from().line,0),to:e.from()}}))},delWrappedLineLeft:function(e){return Ko(e,(function(t){var n=e.charCoords(t.head,"div").top+5;return{from:e.coordsChar({left:0,top:n},"div"),to:t.from()}}))},delWrappedLineRight:function(e){return Ko(e,(function(t){var n=e.charCoords(t.head,"div").top+5,r=e.coordsChar({left:e.display.lineDiv.offsetWidth+100,top:n},"div");return{from:t.from(),to:r}}))},undo:function(e){return e.undo()},redo:function(e){return e.redo()},undoSelection:function(e){return e.undoSelection()},redoSelection:function(e){return e.redoSelection()},goDocStart:function(e){return e.extendSelection(et(e.firstLine(),0))},goDocEnd:function(e){return e.extendSelection(et(e.lastLine()))},goLineStart:function(e){return e.extendSelectionsBy((function(t){return na(e,t.head.line)}),{origin:"+move",bias:1})},goLineStartSmart:function(e){return e.extendSelectionsBy((function(t){return ra(e,t.head)}),{origin:"+move",bias:1})},goLineEnd:function(e){return e.extendSelectionsBy((function(t){return function(e,t){var n=Ge(e.doc,t),r=function(e){for(var t;t=Nt(e);)e=t.find(1,!0).line;return e}(n);return r!=n&&(t=Qe(r)),ea(!0,e,n,t,-1)}(e,t.head.line)}),{origin:"+move",bias:-1})},goLineRight:function(e){return e.extendSelectionsBy((function(t){var n=e.cursorCoords(t.head,"div").top+5;return e.coordsChar({left:e.display.lineDiv.offsetWidth+100,top:n},"div")}),Y)},goLineLeft:function(e){return e.extendSelectionsBy((function(t){var n=e.cursorCoords(t.head,"div").top+5;return e.coordsChar({left:0,top:n},"div")}),Y)},goLineLeftSmart:function(e){return e.extendSelectionsBy((function(t){var n=e.cursorCoords(t.head,"div").top+5,r=e.coordsChar({left:0,top:n},"div");return r.ch<e.getLine(r.line).search(/\S/)?ra(e,t.head):r}),Y)},goLineUp:function(e){return e.moveV(-1,"line")},goLineDown:function(e){return e.moveV(1,"line")},goPageUp:function(e){return e.moveV(-1,"page")},goPageDown:function(e){return e.moveV(1,"page")},goCharLeft:function(e){return e.moveH(-1,"char")},goCharRight:function(e){return e.moveH(1,"char")},goColumnLeft:function(e){return e.moveH(-1,"column")},goColumnRight:function(e){return e.moveH(1,"column")},goWordLeft:function(e){return e.moveH(-1,"word")},goGroupRight:function(e){return e.moveH(1,"group")},goGroupLeft:function(e){return e.moveH(-1,"group")},goWordRight:function(e){return e.moveH(1,"word")},delCharBefore:function(e){return e.deleteH(-1,"codepoint")},delCharAfter:function(e){return e.deleteH(1,"char")},delWordBefore:function(e){return e.deleteH(-1,"word")},delWordAfter:function(e){return e.deleteH(1,"word")},delGroupBefore:function(e){return e.deleteH(-1,"group")},delGroupAfter:function(e){return e.deleteH(1,"group")},indentAuto:function(e){return e.indentSelection("smart")},indentMore:function(e){return e.indentSelection("add")},indentLess:function(e){return e.indentSelection("subtract")},insertTab:function(e){return e.replaceSelection("\t")},insertSoftTab:function(e){for(var t=[],n=e.listSelections(),r=e.options.tabSize,i=0;i<n.length;i++){var o=n[i].from(),a=z(e.getLine(o.line),o.ch,r);t.push(X(r-a%r))}e.replaceSelections(t)},defaultTab:function(e){e.somethingSelected()?e.indentSelection("add"):e.execCommand("insertTab")},transposeChars:function(e){return $r(e,(function(){for(var t=e.listSelections(),n=[],r=0;r<t.length;r++)if(t[r].empty()){var i=t[r].head,o=Ge(e.doc,i.line).text;if(o)if(i.ch==o.length&&(i=new et(i.line,i.ch-1)),i.ch>0)i=new et(i.line,i.ch+1),e.replaceRange(o.charAt(i.ch-1)+o.charAt(i.ch-2),et(i.line,i.ch-2),i,"+transpose");else if(i.line>e.doc.first){var a=Ge(e.doc,i.line-1).text;a&&(i=new et(i.line,1),e.replaceRange(o.charAt(0)+e.doc.lineSeparator()+a.charAt(a.length-1),et(i.line-1,a.length-1),i,"+transpose"))}n.push(new Ii(i,i))}e.setSelections(n)}))},newlineAndIndent:function(e){return $r(e,(function(){for(var t=e.listSelections(),n=t.length-1;n>=0;n--)e.replaceRange(e.doc.lineSeparator(),t[n].anchor,t[n].head,"+input");t=e.listSelections();for(var r=0;r<t.length;r++)e.indentLine(t[r].from().line,null,!0);kr(e)}))},openLine:function(e){return e.replaceSelection("\n","start")},toggleOverwrite:function(e){return e.toggleOverwrite()}};function na(e,t){var n=Ge(e.doc,t),r=zt(n);return r!=n&&(t=Qe(r)),ea(!0,e,r,t,1)}function ra(e,t){var n=na(e,t.line),r=Ge(e.doc,n.line),i=ue(r,e.doc.direction);if(!i||0==i[0].level){var o=Math.max(n.ch,r.text.search(/\S/)),a=t.line==n.line&&t.ch<=o&&t.ch;return et(n.line,a?0:o,n.sticky)}return n}function ia(e,t,n){if("string"==typeof t&&!(t=ta[t]))return!1;e.display.input.ensurePolled();var r=e.display.shift,i=!1;try{e.isReadOnly()&&(e.state.suppressEdits=!0),n&&(e.display.shift=!1),i=t(e)!=H}finally{e.display.shift=r,e.state.suppressEdits=!1}return i}var oa=new F;function aa(e,t,n,r){var i=e.state.keySeq;if(i){if(Xo(t))return"handled";if(/\'$/.test(t)?e.state.keySeq=null:oa.set(50,(function(){e.state.keySeq==i&&(e.state.keySeq=null,e.display.input.reset())})),sa(e,i+" "+t,n,r))return!0}return sa(e,t,n,r)}function sa(e,t,n,r){var i=function(e,t,n){for(var r=0;r<e.state.keyMaps.length;r++){var i=Go(t,e.state.keyMaps[r],n,e);if(i)return i}return e.options.extraKeys&&Go(t,e.options.extraKeys,n,e)||Go(t,e.options.keyMap,n,e)}(e,t,r);return"multi"==i&&(e.state.keySeq=t),"handled"==i&&ln(e,"keyHandled",e,t,n),"handled"!=i&&"multi"!=i||(be(n),Ar(e)),!!i}function la(e,t){var n=Zo(t,!0);return!!n&&(t.shiftKey&&!e.state.keySeq?aa(e,"Shift-"+n,t,(function(t){return ia(e,t,!0)}))||aa(e,n,t,(function(t){if("string"==typeof t?/^go[A-Z]/.test(t):t.motion)return ia(e,t)})):aa(e,n,t,(function(t){return ia(e,t)})))}var ua=null;function ca(e){var t=this;if(!(e.target&&e.target!=t.display.input.getField()||(t.curOp.focus=L(),ge(t,e)))){a&&s<11&&27==e.keyCode&&(e.returnValue=!1);var r=e.keyCode;t.display.shift=16==r||e.shiftKey;var i=la(t,e);d&&(ua=i?r:null,i||88!=r||De||!(y?e.metaKey:e.ctrlKey)||t.replaceSelection("",null,"cut")),n&&!y&&!i&&46==r&&e.shiftKey&&!e.ctrlKey&&document.execCommand&&document.execCommand("cut"),18!=r||/\bCodeMirror-crosshair\b/.test(t.display.lineDiv.className)||function(e){var t=e.display.lineDiv;function n(e){18!=e.keyCode&&e.altKey||(M(t,"CodeMirror-crosshair"),fe(document,"keyup",n),fe(document,"mouseover",n))}D(t,"CodeMirror-crosshair"),de(document,"keyup",n),de(document,"mouseover",n)}(t)}}function da(e){16==e.keyCode&&(this.doc.sel.shift=!1),ge(this,e)}function ha(e){var t=this;if(!(e.target&&e.target!=t.display.input.getField()||_n(t.display,e)||ge(t,e)||e.ctrlKey&&!e.altKey||y&&e.metaKey)){var n=e.keyCode,r=e.charCode;if(d&&n==ua)return ua=null,void be(e);if(!d||e.which&&!(e.which<10)||!la(t,e)){var i=String.fromCharCode(null==r?n:r);"\b"!=i&&(function(e,t,n){return aa(e,"'"+n+"'",t,(function(t){return ia(e,t,!0)}))}(t,e,i)||t.display.input.onKeyPress(e))}}}var fa,pa,ga=function(e,t,n){this.time=e,this.pos=t,this.button=n};function va(e){var t=this,n=t.display;if(!(ge(t,e)||n.activeTouch&&n.input.supportsTouch()))if(n.input.ensurePolled(),n.shift=e.shiftKey,_n(n,e))l||(n.scroller.draggable=!1,setTimeout((function(){return n.scroller.draggable=!0}),100));else if(!ba(t,e)){var r=ur(t,e),i=xe(e),o=r?function(e,t){var n=+new Date;return pa&&pa.compare(n,e,t)?(fa=pa=null,"triple"):fa&&fa.compare(n,e,t)?(pa=new ga(n,e,t),fa=null,"double"):(fa=new ga(n,e,t),pa=null,"single")}(r,i):"single";window.focus(),1==i&&t.state.selectingText&&t.state.selectingText(e),r&&function(e,t,n,r,i){var o="Click";return"double"==r?o="Double"+o:"triple"==r&&(o="Triple"+o),aa(e,qo(o=(1==t?"Left":2==t?"Middle":"Right")+o,i),i,(function(t){if("string"==typeof t&&(t=ta[t]),!t)return!1;var r=!1;try{e.isReadOnly()&&(e.state.suppressEdits=!0),r=t(e,n)!=H}finally{e.state.suppressEdits=!1}return r}))}(t,i,r,o,e)||(1==i?r?function(e,t,n,r){a?setTimeout(j(_r,e),0):e.curOp.focus=L();var i,o=function(e,t,n){var r=e.getOption("configureMouse"),i=r?r(e,t,n):{};if(null==i.unit){var o=b?n.shiftKey&&n.metaKey:n.altKey;i.unit=o?"rectangle":"single"==t?"char":"double"==t?"word":"line"}return(null==i.extend||e.doc.extend)&&(i.extend=e.doc.extend||n.shiftKey),null==i.addNew&&(i.addNew=y?n.metaKey:n.ctrlKey),null==i.moveOnDrag&&(i.moveOnDrag=!(y?n.altKey:n.ctrlKey)),i}(e,n,r),u=e.doc.sel;e.options.dragDrop&&Se&&!e.isReadOnly()&&"single"==n&&(i=u.contains(t))>-1&&(tt((i=u.ranges[i]).from(),t)<0||t.xRel>0)&&(tt(i.to(),t)>0||t.xRel<0)?function(e,t,n,r){var i=e.display,o=!1,u=ei(e,(function(t){l&&(i.scroller.draggable=!1),e.state.draggingText=!1,e.state.delayingBlurEvent&&(e.hasFocus()?e.state.delayingBlurEvent=!1:Ir(e)),fe(i.wrapper.ownerDocument,"mouseup",u),fe(i.wrapper.ownerDocument,"mousemove",c),fe(i.scroller,"dragstart",d),fe(i.scroller,"drop",u),o||(be(t),r.addNew||qi(e.doc,n,null,null,r.extend),l&&!h||a&&9==s?setTimeout((function(){i.wrapper.ownerDocument.body.focus({preventScroll:!0}),i.input.focus()}),20):i.input.focus())})),c=function(e){o=o||Math.abs(t.clientX-e.clientX)+Math.abs(t.clientY-e.clientY)>=10},d=function(){return o=!0};l&&(i.scroller.draggable=!0),e.state.draggingText=u,u.copy=!r.moveOnDrag,de(i.wrapper.ownerDocument,"mouseup",u),de(i.wrapper.ownerDocument,"mousemove",c),de(i.scroller,"dragstart",d),de(i.scroller,"drop",u),e.state.delayingBlurEvent=!0,setTimeout((function(){return i.input.focus()}),20),i.scroller.dragDrop&&i.scroller.dragDrop()}(e,r,t,o):function(e,t,n,r){a&&Ir(e);var i=e.display,o=e.doc;be(t);var s,l,u=o.sel,c=u.ranges;if(r.addNew&&!r.extend?(l=o.sel.contains(n),s=l>-1?c[l]:new Ii(n,n)):(s=o.sel.primary(),l=o.sel.primIndex),"rectangle"==r.unit)r.addNew||(s=new Ii(n,n)),n=ur(e,t,!0,!0),l=-1;else{var d=ma(e,n,r.unit);s=r.extend?Xi(s,d.anchor,d.head,r.extend):d}r.addNew?-1==l?(l=c.length,$i(o,xi(e,c.concat([s]),l),{scroll:!1,origin:"*mouse"})):c.length>1&&c[l].empty()&&"char"==r.unit&&!r.extend?($i(o,xi(e,c.slice(0,l).concat(c.slice(l+1)),0),{scroll:!1,origin:"*mouse"}),u=o.sel):Qi(o,l,s,V):(l=0,$i(o,new _i([s],0),V),u=o.sel);var h=n;function f(t){if(0!=tt(h,t))if(h=t,"rectangle"==r.unit){for(var i=[],a=e.options.tabSize,c=z(Ge(o,n.line).text,n.ch,a),d=z(Ge(o,t.line).text,t.ch,a),f=Math.min(c,d),p=Math.max(c,d),g=Math.min(n.line,t.line),v=Math.min(e.lastLine(),Math.max(n.line,t.line));g<=v;g++){var m=Ge(o,g).text,y=W(m,f,a);f==p?i.push(new Ii(et(g,y),et(g,y))):m.length>y&&i.push(new Ii(et(g,y),et(g,W(m,p,a))))}i.length||i.push(new Ii(n,n)),$i(o,xi(e,u.ranges.slice(0,l).concat(i),l),{origin:"*mouse",scroll:!1}),e.scrollIntoView(t)}else{var b,C=s,A=ma(e,t,r.unit),_=C.anchor;tt(A.anchor,_)>0?(b=A.head,_=ot(C.from(),A.anchor)):(b=A.anchor,_=it(C.to(),A.head));var I=u.ranges.slice(0);I[l]=function(e,t){var n=t.anchor,r=t.head,i=Ge(e.doc,n.line);if(0==tt(n,r)&&n.sticky==r.sticky)return t;var o=ue(i);if(!o)return t;var a=se(o,n.ch,n.sticky),s=o[a];if(s.from!=n.ch&&s.to!=n.ch)return t;var l,u=a+(s.from==n.ch==(1!=s.level)?0:1);if(0==u||u==o.length)return t;if(r.line!=n.line)l=(r.line-n.line)*("ltr"==e.doc.direction?1:-1)>0;else{var c=se(o,r.ch,r.sticky),d=c-a||(r.ch-n.ch)*(1==s.level?-1:1);l=c==u-1||c==u?d<0:d>0}var h=o[u+(l?-1:0)],f=l==(1==h.level),p=f?h.from:h.to,g=f?"after":"before";return n.ch==p&&n.sticky==g?t:new Ii(new et(n.line,p,g),r)}(e,new Ii(st(o,_),b)),$i(o,xi(e,I,l),V)}}var p=i.wrapper.getBoundingClientRect(),g=0;function v(t){e.state.selectingText=!1,g=1/0,t&&(be(t),i.input.focus()),fe(i.wrapper.ownerDocument,"mousemove",m),fe(i.wrapper.ownerDocument,"mouseup",y),o.history.lastSelOrigin=null}var m=ei(e,(function(t){0!==t.buttons&&xe(t)?function t(n){var a=++g,s=ur(e,n,!0,"rectangle"==r.unit);if(s)if(0!=tt(s,h)){e.curOp.focus=L(),f(s);var l=Tr(i,o);(s.line>=l.to||s.line<l.from)&&setTimeout(ei(e,(function(){g==a&&t(n)})),150)}else{var u=n.clientY<p.top?-20:n.clientY>p.bottom?20:0;u&&setTimeout(ei(e,(function(){g==a&&(i.scroller.scrollTop+=u,t(n))})),50)}}(t):v(t)})),y=ei(e,v);e.state.selectingText=y,de(i.wrapper.ownerDocument,"mousemove",m),de(i.wrapper.ownerDocument,"mouseup",y)}(e,r,t,o)}(t,r,o,e):Ie(e)==n.scroller&&be(e):2==i?(r&&qi(t.doc,r),setTimeout((function(){return n.input.focus()}),20)):3==i&&(I?t.display.input.onContextMenu(e):Ir(t)))}}function ma(e,t,n){if("char"==n)return new Ii(t,t);if("word"==n)return e.findWordAt(t);if("line"==n)return new Ii(et(t.line,0),st(e.doc,et(t.line+1,0)));var r=n(e,t);return new Ii(r.from,r.to)}function ya(e,t,n,r){var i,o;if(t.touches)i=t.touches[0].clientX,o=t.touches[0].clientY;else try{i=t.clientX,o=t.clientY}catch(e){return!1}if(i>=Math.floor(e.display.gutters.getBoundingClientRect().right))return!1;r&&be(t);var a=e.display,s=a.lineDiv.getBoundingClientRect();if(o>s.bottom||!me(e,n))return Ae(t);o-=s.top-a.viewOffset;for(var l=0;l<e.display.gutterSpecs.length;++l){var u=a.gutters.childNodes[l];if(u&&u.getBoundingClientRect().right>=i)return pe(e,n,e,Ke(e.doc,o),e.display.gutterSpecs[l].className,t),Ae(t)}}function ba(e,t){return ya(e,t,"gutterClick",!0)}function Ca(e,t){_n(e.display,t)||function(e,t){return!!me(e,"gutterContextMenu")&&ya(e,t,"gutterContextMenu",!1)}(e,t)||ge(e,t,"contextmenu")||I||e.display.input.onContextMenu(t)}function Aa(e){e.display.wrapper.className=e.display.wrapper.className.replace(/\s*cm-s-\S+/g,"")+e.options.theme.replace(/(^|\s)\s*/g," cm-s-"),Bn(e)}ga.prototype.compare=function(e,t,n){return this.time+400>e&&0==tt(t,this.pos)&&n==this.button};var _a={toString:function(){return"CodeMirror.Init"}},Ia={},xa={};function wa(e,t,n){if(!t!=!(n&&n!=_a)){var r=e.display.dragFunctions,i=t?de:fe;i(e.display.scroller,"dragstart",r.start),i(e.display.scroller,"dragenter",r.enter),i(e.display.scroller,"dragover",r.over),i(e.display.scroller,"dragleave",r.leave),i(e.display.scroller,"drop",r.drop)}}function Ma(e){e.options.lineWrapping?(D(e.display.wrapper,"CodeMirror-wrap"),e.display.sizer.style.minWidth="",e.display.sizerWidth=null):(M(e.display.wrapper,"CodeMirror-wrap"),Wt(e)),lr(e),dr(e),Bn(e),setTimeout((function(){return Hr(e)}),100)}function Sa(e,t){var n=this;if(!(this instanceof Sa))return new Sa(e,t);this.options=t=t?R(t):{},R(Ia,t,!1);var r=t.value;"string"==typeof r?r=new ko(r,t.mode,null,t.lineSeparator,t.direction):t.mode&&(r.modeOption=t.mode),this.doc=r;var i=new Sa.inputStyles[t.inputStyle](this),o=this.display=new vi(e,r,i,t);for(var u in o.wrapper.CodeMirror=this,Aa(this),t.lineWrapping&&(this.display.wrapper.className+=" CodeMirror-wrap"),Yr(this),this.state={keyMaps:[],overlays:[],modeGen:0,overwrite:!1,delayingBlurEvent:!1,focused:!1,suppressEdits:!1,pasteIncoming:-1,cutIncoming:-1,selectingText:!1,draggingText:!1,highlight:new F,keySeq:null,specialChars:null},t.autofocus&&!m&&o.input.focus(),a&&s<11&&setTimeout((function(){return n.display.input.reset(!0)}),20),function(e){var t=e.display;de(t.scroller,"mousedown",ei(e,va)),de(t.scroller,"dblclick",a&&s<11?ei(e,(function(t){if(!ge(e,t)){var n=ur(e,t);if(n&&!ba(e,t)&&!_n(e.display,t)){be(t);var r=e.findWordAt(n);qi(e.doc,r.anchor,r.head)}}})):function(t){return ge(e,t)||be(t)}),de(t.scroller,"contextmenu",(function(t){return Ca(e,t)})),de(t.input.getField(),"contextmenu",(function(n){t.scroller.contains(n.target)||Ca(e,n)}));var n,r={end:0};function i(){t.activeTouch&&(n=setTimeout((function(){return t.activeTouch=null}),1e3),(r=t.activeTouch).end=+new Date)}function o(e,t){if(null==t.left)return!0;var n=t.left-e.left,r=t.top-e.top;return n*n+r*r>400}de(t.scroller,"touchstart",(function(i){if(!ge(e,i)&&!function(e){if(1!=e.touches.length)return!1;var t=e.touches[0];return t.radiusX<=1&&t.radiusY<=1}(i)&&!ba(e,i)){t.input.ensurePolled(),clearTimeout(n);var o=+new Date;t.activeTouch={start:o,moved:!1,prev:o-r.end<=300?r:null},1==i.touches.length&&(t.activeTouch.left=i.touches[0].pageX,t.activeTouch.top=i.touches[0].pageY)}})),de(t.scroller,"touchmove",(function(){t.activeTouch&&(t.activeTouch.moved=!0)})),de(t.scroller,"touchend",(function(n){var r=t.activeTouch;if(r&&!_n(t,n)&&null!=r.left&&!r.moved&&new Date-r.start<300){var a,s=e.coordsChar(t.activeTouch,"page");a=!r.prev||o(r,r.prev)?new Ii(s,s):!r.prev.prev||o(r,r.prev.prev)?e.findWordAt(s):new Ii(et(s.line,0),st(e.doc,et(s.line+1,0))),e.setSelection(a.anchor,a.head),e.focus(),be(n)}i()})),de(t.scroller,"touchcancel",i),de(t.scroller,"scroll",(function(){t.scroller.clientHeight&&(Nr(e,t.scroller.scrollTop),Rr(e,t.scroller.scrollLeft,!0),pe(e,"scroll",e))})),de(t.scroller,"mousewheel",(function(t){return Ai(e,t)})),de(t.scroller,"DOMMouseScroll",(function(t){return Ai(e,t)})),de(t.wrapper,"scroll",(function(){return t.wrapper.scrollTop=t.wrapper.scrollLeft=0})),t.dragFunctions={enter:function(t){ge(e,t)||_e(t)},over:function(t){ge(e,t)||(function(e,t){var n=ur(e,t);if(n){var r=document.createDocumentFragment();yr(e,n,r),e.display.dragCursor||(e.display.dragCursor=E("div",null,"CodeMirror-cursors CodeMirror-dragcursors"),e.display.lineSpace.insertBefore(e.display.dragCursor,e.display.cursorDiv)),T(e.display.dragCursor,r)}}(e,t),_e(t))},start:function(t){return function(e,t){if(a&&(!e.state.draggingText||+new Date-Lo<100))_e(t);else if(!ge(e,t)&&!_n(e.display,t)&&(t.dataTransfer.setData("Text",e.getSelection()),t.dataTransfer.effectAllowed="copyMove",t.dataTransfer.setDragImage&&!h)){var n=E("img",null,null,"position: fixed; left: 0; top: 0;");n.src="data:image/gif;base64,R0lGODlhAQABAAAAACH5BAEKAAEALAAAAAABAAEAAAICTAEAOw==",d&&(n.width=n.height=1,e.display.wrapper.appendChild(n),n._top=n.offsetTop),t.dataTransfer.setDragImage(n,0,0),d&&n.parentNode.removeChild(n)}}(e,t)},drop:ei(e,Do),leave:function(t){ge(e,t)||Po(e)}};var l=t.input.getField();de(l,"keyup",(function(t){return da.call(e,t)})),de(l,"keydown",ei(e,ca)),de(l,"keypress",ei(e,ha)),de(l,"focus",(function(t){return xr(e,t)})),de(l,"blur",(function(t){return wr(e,t)}))}(this),Ro(),Gr(this),this.curOp.forceUpdate=!0,Ni(this,r),t.autofocus&&!m||this.hasFocus()?setTimeout((function(){n.hasFocus()&&!n.state.focused&&xr(n)}),20):wr(this),xa)xa.hasOwnProperty(u)&&xa[u](this,t[u],_a);hi(this),t.finishInit&&t.finishInit(this);for(var c=0;c<Ta.length;++c)Ta[c](this);Xr(this),l&&t.lineWrapping&&"optimizelegibility"==getComputedStyle(o.lineDiv).textRendering&&(o.lineDiv.style.textRendering="auto")}Sa.defaults=Ia,Sa.optionHandlers=xa;var Ta=[];function Ea(e,t,n,r){var i,o=e.doc;null==n&&(n="add"),"smart"==n&&(o.mode.indent?i=ft(e,t).state:n="prev");var a=e.options.tabSize,s=Ge(o,t),l=z(s.text,null,a);s.stateAfter&&(s.stateAfter=null);var u,c=s.text.match(/^\s*/)[0];if(r||/\S/.test(s.text)){if("smart"==n&&((u=o.mode.indent(i,s.text.slice(c.length),s.text))==H||u>150)){if(!r)return;n="prev"}}else u=0,n="not";"prev"==n?u=t>o.first?z(Ge(o,t-1).text,null,a):0:"add"==n?u=l+e.options.indentUnit:"subtract"==n?u=l-e.options.indentUnit:"number"==typeof n&&(u=l+n),u=Math.max(0,u);var d="",h=0;if(e.options.indentWithTabs)for(var f=Math.floor(u/a);f;--f)h+=a,d+="\t";if(h<u&&(d+=X(u-h)),d!=c)return go(o,d,et(t,0),et(t,c.length),"+input"),s.stateAfter=null,!0;for(var p=0;p<o.sel.ranges.length;p++){var g=o.sel.ranges[p];if(g.head.line==t&&g.head.ch<c.length){var v=et(t,c.length);Qi(o,p,new Ii(v,v));break}}}Sa.defineInitHook=function(e){return Ta.push(e)};var Oa=null;function ka(e){Oa=e}function La(e,t,n,r,i){var o=e.doc;e.display.shift=!1,r||(r=o.sel);var a=+new Date-200,s="paste"==i||e.state.pasteIncoming>a,l=ke(t),u=null;if(s&&r.ranges.length>1)if(Oa&&Oa.text.join("\n")==t){if(r.ranges.length%Oa.text.length==0){u=[];for(var c=0;c<Oa.text.length;c++)u.push(o.splitLines(Oa.text[c]))}}else l.length==r.ranges.length&&e.options.pasteLinesPerSelection&&(u=Z(l,(function(e){return[e]})));for(var d=e.curOp.updateInput,h=r.ranges.length-1;h>=0;h--){var f=r.ranges[h],p=f.from(),g=f.to();f.empty()&&(n&&n>0?p=et(p.line,p.ch-n):e.state.overwrite&&!s?g=et(g.line,Math.min(Ge(o,g.line).text.length,g.ch+q(l).length)):s&&Oa&&Oa.lineWise&&Oa.text.join("\n")==l.join("\n")&&(p=g=et(p.line,0)));var v={from:p,to:g,text:u?u[h%u.length]:l,origin:i||(s?"paste":e.state.cutIncoming>a?"cut":"+input")};uo(e.doc,v),ln(e,"inputRead",e,v)}t&&!s&&Pa(e,t),kr(e),e.curOp.updateInput<2&&(e.curOp.updateInput=d),e.curOp.typing=!0,e.state.pasteIncoming=e.state.cutIncoming=-1}function Da(e,t){var n=e.clipboardData&&e.clipboardData.getData("Text");if(n)return e.preventDefault(),t.isReadOnly()||t.options.disableInput||$r(t,(function(){return La(t,n,0,null,"paste")})),!0}function Pa(e,t){if(e.options.electricChars&&e.options.smartIndent)for(var n=e.doc.sel,r=n.ranges.length-1;r>=0;r--){var i=n.ranges[r];if(!(i.head.ch>100||r&&n.ranges[r-1].head.line==i.head.line)){var o=e.getModeAt(i.head),a=!1;if(o.electricChars){for(var s=0;s<o.electricChars.length;s++)if(t.indexOf(o.electricChars.charAt(s))>-1){a=Ea(e,i.head.line,"smart");break}}else o.electricInput&&o.electricInput.test(Ge(e.doc,i.head.line).text.slice(0,i.head.ch))&&(a=Ea(e,i.head.line,"smart"));a&&ln(e,"electricInput",e,i.head.line)}}}function Na(e){for(var t=[],n=[],r=0;r<e.doc.sel.ranges.length;r++){var i=e.doc.sel.ranges[r].head.line,o={anchor:et(i,0),head:et(i+1,0)};n.push(o),t.push(e.getRange(o.anchor,o.head))}return{text:t,ranges:n}}function ja(e,t,n,r){e.setAttribute("autocorrect",n?"":"off"),e.setAttribute("autocapitalize",r?"":"off"),e.setAttribute("spellcheck",!!t)}function Ra(){var e=E("textarea",null,null,"position: absolute; bottom: -1em; padding: 0; width: 1px; height: 1em; outline: none"),t=E("div",[e],null,"overflow: hidden; position: relative; width: 3px; height: 0px;");return l?e.style.width="1000px":e.setAttribute("wrap","off"),g&&(e.style.border="1px solid black"),ja(e),t}function za(e,t,n,r,i){var o=t,a=n,s=Ge(e,t.line),l=i&&"rtl"==e.direction?-n:n;function u(o){var a,u;if("codepoint"==r){var c=s.text.charCodeAt(t.ch+(n>0?0:-1));if(isNaN(c))a=null;else{var d=n>0?c>=55296&&c<56320:c>=56320&&c<57343;a=new et(t.line,Math.max(0,Math.min(s.text.length,t.ch+n*(d?2:1))),-n)}}else a=i?function(e,t,n,r){var i=ue(t,e.doc.direction);if(!i)return $o(t,n,r);n.ch>=t.text.length?(n.ch=t.text.length,n.sticky="before"):n.ch<=0&&(n.ch=0,n.sticky="after");var o=se(i,n.ch,n.sticky),a=i[o];if("ltr"==e.doc.direction&&a.level%2==0&&(r>0?a.to>n.ch:a.from<n.ch))return $o(t,n,r);var s,l=function(e,n){return Jo(t,e instanceof et?e.ch:e,n)},u=function(n){return e.options.lineWrapping?(s=s||Ln(e,t),Jn(e,t,s,n)):{begin:0,end:t.text.length}},c=u("before"==n.sticky?l(n,-1):n.ch);if("rtl"==e.doc.direction||1==a.level){var d=1==a.level==r<0,h=l(n,d?1:-1);if(null!=h&&(d?h<=a.to&&h<=c.end:h>=a.from&&h>=c.begin)){var f=d?"before":"after";return new et(n.line,h,f)}}var p=function(e,t,r){for(var o=function(e,t){return t?new et(n.line,l(e,1),"before"):new et(n.line,e,"after")};e>=0&&e<i.length;e+=t){var a=i[e],s=t>0==(1!=a.level),u=s?r.begin:l(r.end,-1);if(a.from<=u&&u<a.to)return o(u,s);if(u=s?a.from:l(a.to,-1),r.begin<=u&&u<r.end)return o(u,s)}},g=p(o+r,r,c);if(g)return g;var v=r>0?c.end:l(c.begin,-1);return null==v||r>0&&v==t.text.length||!(g=p(r>0?0:i.length-1,r,u(v)))?null:g}(e.cm,s,t,n):$o(s,t,n);if(null==a){if(o||(u=t.line+l)<e.first||u>=e.first+e.size||(t=new et(u,t.ch,t.sticky),!(s=Ge(e,u))))return!1;t=ea(i,e.cm,s,t.line,l)}else t=a;return!0}if("char"==r||"codepoint"==r)u();else if("column"==r)u(!0);else if("word"==r||"group"==r)for(var c=null,d="group"==r,h=e.cm&&e.cm.getHelper(t,"wordChars"),f=!0;!(n<0)||u(!f);f=!1){var p=s.text.charAt(t.ch)||"\n",g=ee(p,h)?"w":d&&"\n"==p?"n":!d||/\s/.test(p)?null:"p";if(!d||f||g||(g="s"),c&&c!=g){n<0&&(n=1,u(),t.sticky="after");break}if(g&&(c=g),n>0&&!u(!f))break}var v=oo(e,t,o,a,!0);return nt(o,v)&&(v.hitSide=!0),v}function Fa(e,t,n,r){var i,o,a=e.doc,s=t.left;if("page"==r){var l=Math.min(e.display.wrapper.clientHeight,window.innerHeight||document.documentElement.clientHeight),u=Math.max(l-.5*rr(e.display),3);i=(n>0?t.bottom:t.top)+n*u}else"line"==r&&(i=n>0?t.bottom+3:t.top-3);for(;(o=Qn(e,s,i)).outside;){if(n<0?i<=0:i>=a.height){o.hitSide=!0;break}i+=5*n}return o}var Ba=function(e){this.cm=e,this.lastAnchorNode=this.lastAnchorOffset=this.lastFocusNode=this.lastFocusOffset=null,this.polling=new F,this.composing=null,this.gracePeriod=!1,this.readDOMTimeout=null};function Ha(e,t){var n=kn(e,t.line);if(!n||n.hidden)return null;var r=Ge(e.doc,t.line),i=En(n,r,t.line),o=ue(r,e.doc.direction),a="left";o&&(a=se(o,t.ch)%2?"right":"left");var s=jn(i.map,t.ch,a);return s.offset="right"==s.collapse?s.end:s.start,s}function Ua(e,t){return t&&(e.bad=!0),e}function Va(e,t,n){var r;if(t==e.display.lineDiv){if(!(r=e.display.lineDiv.childNodes[n]))return Ua(e.clipPos(et(e.display.viewTo-1)),!0);t=null,n=0}else for(r=t;;r=r.parentNode){if(!r||r==e.display.lineDiv)return null;if(r.parentNode&&r.parentNode==e.display.lineDiv)break}for(var i=0;i<e.display.view.length;i++){var o=e.display.view[i];if(o.node==r)return Ya(o,t,n)}}function Ya(e,t,n){var r=e.text.firstChild,i=!1;if(!t||!k(r,t))return Ua(et(Qe(e.line),0),!0);if(t==r&&(i=!0,t=r.childNodes[n],n=0,!t)){var o=e.rest?q(e.rest):e.line;return Ua(et(Qe(o),o.text.length),i)}var a=3==t.nodeType?t:null,s=t;for(a||1!=t.childNodes.length||3!=t.firstChild.nodeType||(a=t.firstChild,n&&(n=a.nodeValue.length));s.parentNode!=r;)s=s.parentNode;var l=e.measure,u=l.maps;function c(t,n,r){for(var i=-1;i<(u?u.length:0);i++)for(var o=i<0?l.map:u[i],a=0;a<o.length;a+=3){var s=o[a+2];if(s==t||s==n){var c=Qe(i<0?e.line:e.rest[i]),d=o[a]+r;return(r<0||s!=t)&&(d=o[a+(r?1:0)]),et(c,d)}}}var d=c(a,s,n);if(d)return Ua(d,i);for(var h=s.nextSibling,f=a?a.nodeValue.length-n:0;h;h=h.nextSibling){if(d=c(h,h.firstChild,0))return Ua(et(d.line,d.ch-f),i);f+=h.textContent.length}for(var p=s.previousSibling,g=n;p;p=p.previousSibling){if(d=c(p,p.firstChild,-1))return Ua(et(d.line,d.ch+g),i);g+=p.textContent.length}}Ba.prototype.init=function(e){var t=this,n=this,r=n.cm,i=n.div=e.lineDiv;function o(e){for(var t=e.target;t;t=t.parentNode){if(t==i)return!0;if(/\bCodeMirror-(?:line)?widget\b/.test(t.className))break}return!1}function a(e){if(o(e)&&!ge(r,e)){if(r.somethingSelected())ka({lineWise:!1,text:r.getSelections()}),"cut"==e.type&&r.replaceSelection("",null,"cut");else{if(!r.options.lineWiseCopyCut)return;var t=Na(r);ka({lineWise:!0,text:t.text}),"cut"==e.type&&r.operation((function(){r.setSelections(t.ranges,0,U),r.replaceSelection("",null,"cut")}))}if(e.clipboardData){e.clipboardData.clearData();var a=Oa.text.join("\n");if(e.clipboardData.setData("Text",a),e.clipboardData.getData("Text")==a)return void e.preventDefault()}var s=Ra(),l=s.firstChild;r.display.lineSpace.insertBefore(s,r.display.lineSpace.firstChild),l.value=Oa.text.join("\n");var u=document.activeElement;N(l),setTimeout((function(){r.display.lineSpace.removeChild(s),u.focus(),u==i&&n.showPrimarySelection()}),50)}}ja(i,r.options.spellcheck,r.options.autocorrect,r.options.autocapitalize),de(i,"paste",(function(e){!o(e)||ge(r,e)||Da(e,r)||s<=11&&setTimeout(ei(r,(function(){return t.updateFromDOM()})),20)})),de(i,"compositionstart",(function(e){t.composing={data:e.data,done:!1}})),de(i,"compositionupdate",(function(e){t.composing||(t.composing={data:e.data,done:!1})})),de(i,"compositionend",(function(e){t.composing&&(e.data!=t.composing.data&&t.readFromDOMSoon(),t.composing.done=!0)})),de(i,"touchstart",(function(){return n.forceCompositionEnd()})),de(i,"input",(function(){t.composing||t.readFromDOMSoon()})),de(i,"copy",a),de(i,"cut",a)},Ba.prototype.screenReaderLabelChanged=function(e){e?this.div.setAttribute("aria-label",e):this.div.removeAttribute("aria-label")},Ba.prototype.prepareSelection=function(){var e=mr(this.cm,!1);return e.focus=document.activeElement==this.div,e},Ba.prototype.showSelection=function(e,t){e&&this.cm.display.view.length&&((e.focus||t)&&this.showPrimarySelection(),this.showMultipleSelections(e))},Ba.prototype.getSelection=function(){return this.cm.display.wrapper.ownerDocument.getSelection()},Ba.prototype.showPrimarySelection=function(){var e=this.getSelection(),t=this.cm,r=t.doc.sel.primary(),i=r.from(),o=r.to();if(t.display.viewTo==t.display.viewFrom||i.line>=t.display.viewTo||o.line<t.display.viewFrom)e.removeAllRanges();else{var a=Va(t,e.anchorNode,e.anchorOffset),s=Va(t,e.focusNode,e.focusOffset);if(!a||a.bad||!s||s.bad||0!=tt(ot(a,s),i)||0!=tt(it(a,s),o)){var l=t.display.view,u=i.line>=t.display.viewFrom&&Ha(t,i)||{node:l[0].measure.map[2],offset:0},c=o.line<t.display.viewTo&&Ha(t,o);if(!c){var d=l[l.length-1].measure,h=d.maps?d.maps[d.maps.length-1]:d.map;c={node:h[h.length-1],offset:h[h.length-2]-h[h.length-3]}}if(u&&c){var f,p=e.rangeCount&&e.getRangeAt(0);try{f=w(u.node,u.offset,c.offset,c.node)}catch(e){}f&&(!n&&t.state.focused?(e.collapse(u.node,u.offset),f.collapsed||(e.removeAllRanges(),e.addRange(f))):(e.removeAllRanges(),e.addRange(f)),p&&null==e.anchorNode?e.addRange(p):n&&this.startGracePeriod()),this.rememberSelection()}else e.removeAllRanges()}}},Ba.prototype.startGracePeriod=function(){var e=this;clearTimeout(this.gracePeriod),this.gracePeriod=setTimeout((function(){e.gracePeriod=!1,e.selectionChanged()&&e.cm.operation((function(){return e.cm.curOp.selectionChanged=!0}))}),20)},Ba.prototype.showMultipleSelections=function(e){T(this.cm.display.cursorDiv,e.cursors),T(this.cm.display.selectionDiv,e.selection)},Ba.prototype.rememberSelection=function(){var e=this.getSelection();this.lastAnchorNode=e.anchorNode,this.lastAnchorOffset=e.anchorOffset,this.lastFocusNode=e.focusNode,this.lastFocusOffset=e.focusOffset},Ba.prototype.selectionInEditor=function(){var e=this.getSelection();if(!e.rangeCount)return!1;var t=e.getRangeAt(0).commonAncestorContainer;return k(this.div,t)},Ba.prototype.focus=function(){"nocursor"!=this.cm.options.readOnly&&(this.selectionInEditor()&&document.activeElement==this.div||this.showSelection(this.prepareSelection(),!0),this.div.focus())},Ba.prototype.blur=function(){this.div.blur()},Ba.prototype.getField=function(){return this.div},Ba.prototype.supportsTouch=function(){return!0},Ba.prototype.receivedFocus=function(){var e=this;this.selectionInEditor()?this.pollSelection():$r(this.cm,(function(){return e.cm.curOp.selectionChanged=!0})),this.polling.set(this.cm.options.pollInterval,(function t(){e.cm.state.focused&&(e.pollSelection(),e.polling.set(e.cm.options.pollInterval,t))}))},Ba.prototype.selectionChanged=function(){var e=this.getSelection();return e.anchorNode!=this.lastAnchorNode||e.anchorOffset!=this.lastAnchorOffset||e.focusNode!=this.lastFocusNode||e.focusOffset!=this.lastFocusOffset},Ba.prototype.pollSelection=function(){if(null==this.readDOMTimeout&&!this.gracePeriod&&this.selectionChanged()){var e=this.getSelection(),t=this.cm;if(v&&c&&this.cm.display.gutterSpecs.length&&function(e){for(var t=e;t;t=t.parentNode)if(/CodeMirror-gutter-wrapper/.test(t.className))return!0;return!1}(e.anchorNode))return this.cm.triggerOnKeyDown({type:"keydown",keyCode:8,preventDefault:Math.abs}),this.blur(),void this.focus();if(!this.composing){this.rememberSelection();var n=Va(t,e.anchorNode,e.anchorOffset),r=Va(t,e.focusNode,e.focusOffset);n&&r&&$r(t,(function(){$i(t.doc,wi(n,r),U),(n.bad||r.bad)&&(t.curOp.selectionChanged=!0)}))}}},Ba.prototype.pollContent=function(){null!=this.readDOMTimeout&&(clearTimeout(this.readDOMTimeout),this.readDOMTimeout=null);var e,t,n,r=this.cm,i=r.display,o=r.doc.sel.primary(),a=o.from(),s=o.to();if(0==a.ch&&a.line>r.firstLine()&&(a=et(a.line-1,Ge(r.doc,a.line-1).length)),s.ch==Ge(r.doc,s.line).text.length&&s.line<r.lastLine()&&(s=et(s.line+1,0)),a.line<i.viewFrom||s.line>i.viewTo-1)return!1;a.line==i.viewFrom||0==(e=cr(r,a.line))?(t=Qe(i.view[0].line),n=i.view[0].node):(t=Qe(i.view[e].line),n=i.view[e-1].node.nextSibling);var l,u,c=cr(r,s.line);if(c==i.view.length-1?(l=i.viewTo-1,u=i.lineDiv.lastChild):(l=Qe(i.view[c+1].line)-1,u=i.view[c+1].node.previousSibling),!n)return!1;for(var d=r.doc.splitLines(function(e,t,n,r,i){var o="",a=!1,s=e.doc.lineSeparator(),l=!1;function u(){a&&(o+=s,l&&(o+=s),a=l=!1)}function c(e){e&&(u(),o+=e)}function d(t){if(1==t.nodeType){var n=t.getAttribute("cm-text");if(n)return void c(n);var o,h=t.getAttribute("cm-marker");if(h){var f=e.findMarks(et(r,0),et(i+1,0),(v=+h,function(e){return e.id==v}));return void(f.length&&(o=f[0].find(0))&&c(Xe(e.doc,o.from,o.to).join(s)))}if("false"==t.getAttribute("contenteditable"))return;var p=/^(pre|div|p|li|table|br)$/i.test(t.nodeName);if(!/^br$/i.test(t.nodeName)&&0==t.textContent.length)return;p&&u();for(var g=0;g<t.childNodes.length;g++)d(t.childNodes[g]);/^(pre|p)$/i.test(t.nodeName)&&(l=!0),p&&(a=!0)}else 3==t.nodeType&&c(t.nodeValue.replace(/\u200b/g,"").replace(/\u00a0/g," "));var v}for(;d(t),t!=n;)t=t.nextSibling,l=!1;return o}(r,n,u,t,l)),h=Xe(r.doc,et(t,0),et(l,Ge(r.doc,l).text.length));d.length>1&&h.length>1;)if(q(d)==q(h))d.pop(),h.pop(),l--;else{if(d[0]!=h[0])break;d.shift(),h.shift(),t++}for(var f=0,p=0,g=d[0],v=h[0],m=Math.min(g.length,v.length);f<m&&g.charCodeAt(f)==v.charCodeAt(f);)++f;for(var y=q(d),b=q(h),C=Math.min(y.length-(1==d.length?f:0),b.length-(1==h.length?f:0));p<C&&y.charCodeAt(y.length-p-1)==b.charCodeAt(b.length-p-1);)++p;if(1==d.length&&1==h.length&&t==a.line)for(;f&&f>a.ch&&y.charCodeAt(y.length-p-1)==b.charCodeAt(b.length-p-1);)f--,p++;d[d.length-1]=y.slice(0,y.length-p).replace(/^\u200b+/,""),d[0]=d[0].slice(f).replace(/\u200b+$/,"");var A=et(t,f),_=et(l,h.length?q(h).length-p:0);return d.length>1||d[0]||tt(A,_)?(go(r.doc,d,A,_,"+input"),!0):void 0},Ba.prototype.ensurePolled=function(){this.forceCompositionEnd()},Ba.prototype.reset=function(){this.forceCompositionEnd()},Ba.prototype.forceCompositionEnd=function(){this.composing&&(clearTimeout(this.readDOMTimeout),this.composing=null,this.updateFromDOM(),this.div.blur(),this.div.focus())},Ba.prototype.readFromDOMSoon=function(){var e=this;null==this.readDOMTimeout&&(this.readDOMTimeout=setTimeout((function(){if(e.readDOMTimeout=null,e.composing){if(!e.composing.done)return;e.composing=null}e.updateFromDOM()}),80))},Ba.prototype.updateFromDOM=function(){var e=this;!this.cm.isReadOnly()&&this.pollContent()||$r(this.cm,(function(){return dr(e.cm)}))},Ba.prototype.setUneditable=function(e){e.contentEditable="false"},Ba.prototype.onKeyPress=function(e){0==e.charCode||this.composing||(e.preventDefault(),this.cm.isReadOnly()||ei(this.cm,La)(this.cm,String.fromCharCode(null==e.charCode?e.keyCode:e.charCode),0))},Ba.prototype.readOnlyChanged=function(e){this.div.contentEditable=String("nocursor"!=e)},Ba.prototype.onContextMenu=function(){},Ba.prototype.resetPosition=function(){},Ba.prototype.needsContentAttribute=!0;var Wa=function(e){this.cm=e,this.prevInput="",this.pollingFast=!1,this.polling=new F,this.hasSelection=!1,this.composing=null};Wa.prototype.init=function(e){var t=this,n=this,r=this.cm;this.createField(e);var i=this.textarea;function o(e){if(!ge(r,e)){if(r.somethingSelected())ka({lineWise:!1,text:r.getSelections()});else{if(!r.options.lineWiseCopyCut)return;var t=Na(r);ka({lineWise:!0,text:t.text}),"cut"==e.type?r.setSelections(t.ranges,null,U):(n.prevInput="",i.value=t.text.join("\n"),N(i))}"cut"==e.type&&(r.state.cutIncoming=+new Date)}}e.wrapper.insertBefore(this.wrapper,e.wrapper.firstChild),g&&(i.style.width="0px"),de(i,"input",(function(){a&&s>=9&&t.hasSelection&&(t.hasSelection=null),n.poll()})),de(i,"paste",(function(e){ge(r,e)||Da(e,r)||(r.state.pasteIncoming=+new Date,n.fastPoll())})),de(i,"cut",o),de(i,"copy",o),de(e.scroller,"paste",(function(t){if(!_n(e,t)&&!ge(r,t)){if(!i.dispatchEvent)return r.state.pasteIncoming=+new Date,void n.focus();var o=new Event("paste");o.clipboardData=t.clipboardData,i.dispatchEvent(o)}})),de(e.lineSpace,"selectstart",(function(t){_n(e,t)||be(t)})),de(i,"compositionstart",(function(){var e=r.getCursor("from");n.composing&&n.composing.range.clear(),n.composing={start:e,range:r.markText(e,r.getCursor("to"),{className:"CodeMirror-composing"})}})),de(i,"compositionend",(function(){n.composing&&(n.poll(),n.composing.range.clear(),n.composing=null)}))},Wa.prototype.createField=function(e){this.wrapper=Ra(),this.textarea=this.wrapper.firstChild},Wa.prototype.screenReaderLabelChanged=function(e){e?this.textarea.setAttribute("aria-label",e):this.textarea.removeAttribute("aria-label")},Wa.prototype.prepareSelection=function(){var e=this.cm,t=e.display,n=e.doc,r=mr(e);if(e.options.moveInputWithCursor){var i=Xn(e,n.sel.primary().head,"div"),o=t.wrapper.getBoundingClientRect(),a=t.lineDiv.getBoundingClientRect();r.teTop=Math.max(0,Math.min(t.wrapper.clientHeight-10,i.top+a.top-o.top)),r.teLeft=Math.max(0,Math.min(t.wrapper.clientWidth-10,i.left+a.left-o.left))}return r},Wa.prototype.showSelection=function(e){var t=this.cm.display;T(t.cursorDiv,e.cursors),T(t.selectionDiv,e.selection),null!=e.teTop&&(this.wrapper.style.top=e.teTop+"px",this.wrapper.style.left=e.teLeft+"px")},Wa.prototype.reset=function(e){if(!this.contextMenuPending&&!this.composing){var t=this.cm;if(t.somethingSelected()){this.prevInput="";var n=t.getSelection();this.textarea.value=n,t.state.focused&&N(this.textarea),a&&s>=9&&(this.hasSelection=n)}else e||(this.prevInput=this.textarea.value="",a&&s>=9&&(this.hasSelection=null))}},Wa.prototype.getField=function(){return this.textarea},Wa.prototype.supportsTouch=function(){return!1},Wa.prototype.focus=function(){if("nocursor"!=this.cm.options.readOnly&&(!m||L()!=this.textarea))try{this.textarea.focus()}catch(e){}},Wa.prototype.blur=function(){this.textarea.blur()},Wa.prototype.resetPosition=function(){this.wrapper.style.top=this.wrapper.style.left=0},Wa.prototype.receivedFocus=function(){this.slowPoll()},Wa.prototype.slowPoll=function(){var e=this;this.pollingFast||this.polling.set(this.cm.options.pollInterval,(function(){e.poll(),e.cm.state.focused&&e.slowPoll()}))},Wa.prototype.fastPoll=function(){var e=!1,t=this;t.pollingFast=!0,t.polling.set(20,(function n(){t.poll()||e?(t.pollingFast=!1,t.slowPoll()):(e=!0,t.polling.set(60,n))}))},Wa.prototype.poll=function(){var e=this,t=this.cm,n=this.textarea,r=this.prevInput;if(this.contextMenuPending||!t.state.focused||Le(n)&&!r&&!this.composing||t.isReadOnly()||t.options.disableInput||t.state.keySeq)return!1;var i=n.value;if(i==r&&!t.somethingSelected())return!1;if(a&&s>=9&&this.hasSelection===i||y&&/[\uf700-\uf7ff]/.test(i))return t.display.input.reset(),!1;if(t.doc.sel==t.display.selForContextMenu){var o=i.charCodeAt(0);if(8203!=o||r||(r="​"),8666==o)return this.reset(),this.cm.execCommand("undo")}for(var l=0,u=Math.min(r.length,i.length);l<u&&r.charCodeAt(l)==i.charCodeAt(l);)++l;return $r(t,(function(){La(t,i.slice(l),r.length-l,null,e.composing?"*compose":null),i.length>1e3||i.indexOf("\n")>-1?n.value=e.prevInput="":e.prevInput=i,e.composing&&(e.composing.range.clear(),e.composing.range=t.markText(e.composing.start,t.getCursor("to"),{className:"CodeMirror-composing"}))})),!0},Wa.prototype.ensurePolled=function(){this.pollingFast&&this.poll()&&(this.pollingFast=!1)},Wa.prototype.onKeyPress=function(){a&&s>=9&&(this.hasSelection=null),this.fastPoll()},Wa.prototype.onContextMenu=function(e){var t=this,n=t.cm,r=n.display,i=t.textarea;t.contextMenuPending&&t.contextMenuPending();var o=ur(n,e),u=r.scroller.scrollTop;if(o&&!d){n.options.resetSelectionOnContextMenu&&-1==n.doc.sel.contains(o)&&ei(n,$i)(n.doc,wi(o),U);var c,h=i.style.cssText,f=t.wrapper.style.cssText,p=t.wrapper.offsetParent.getBoundingClientRect();if(t.wrapper.style.cssText="position: static",i.style.cssText="position: absolute; width: 30px; height: 30px;\n      top: "+(e.clientY-p.top-5)+"px; left: "+(e.clientX-p.left-5)+"px;\n      z-index: 1000; background: "+(a?"rgba(255, 255, 255, .05)":"transparent")+";\n      outline: none; border-width: 0; outline: none; overflow: hidden; opacity: .05; filter: alpha(opacity=5);",l&&(c=window.scrollY),r.input.focus(),l&&window.scrollTo(null,c),r.input.reset(),n.somethingSelected()||(i.value=t.prevInput=" "),t.contextMenuPending=m,r.selForContextMenu=n.doc.sel,clearTimeout(r.detectingSelectAll),a&&s>=9&&v(),I){_e(e);var g=function(){fe(window,"mouseup",g),setTimeout(m,20)};de(window,"mouseup",g)}else setTimeout(m,50)}function v(){if(null!=i.selectionStart){var e=n.somethingSelected(),o="​"+(e?i.value:"");i.value="⇚",i.value=o,t.prevInput=e?"":"​",i.selectionStart=1,i.selectionEnd=o.length,r.selForContextMenu=n.doc.sel}}function m(){if(t.contextMenuPending==m&&(t.contextMenuPending=!1,t.wrapper.style.cssText=f,i.style.cssText=h,a&&s<9&&r.scrollbars.setScrollTop(r.scroller.scrollTop=u),null!=i.selectionStart)){(!a||a&&s<9)&&v();var e=0,o=function(){r.selForContextMenu==n.doc.sel&&0==i.selectionStart&&i.selectionEnd>0&&"​"==t.prevInput?ei(n,so)(n):e++<10?r.detectingSelectAll=setTimeout(o,500):(r.selForContextMenu=null,r.input.reset())};r.detectingSelectAll=setTimeout(o,200)}}},Wa.prototype.readOnlyChanged=function(e){e||this.reset(),this.textarea.disabled="nocursor"==e,this.textarea.readOnly=!!e},Wa.prototype.setUneditable=function(){},Wa.prototype.needsContentAttribute=!1,function(e){var t=e.optionHandlers;function n(n,r,i,o){e.defaults[n]=r,i&&(t[n]=o?function(e,t,n){n!=_a&&i(e,t,n)}:i)}e.defineOption=n,e.Init=_a,n("value","",(function(e,t){return e.setValue(t)}),!0),n("mode",null,(function(e,t){e.doc.modeOption=t,Oi(e)}),!0),n("indentUnit",2,Oi,!0),n("indentWithTabs",!1),n("smartIndent",!0),n("tabSize",4,(function(e){ki(e),Bn(e),dr(e)}),!0),n("lineSeparator",null,(function(e,t){if(e.doc.lineSep=t,t){var n=[],r=e.doc.first;e.doc.iter((function(e){for(var i=0;;){var o=e.text.indexOf(t,i);if(-1==o)break;i=o+t.length,n.push(et(r,o))}r++}));for(var i=n.length-1;i>=0;i--)go(e.doc,t,n[i],et(n[i].line,n[i].ch+t.length))}})),n("specialChars",/[\u0000-\u001f\u007f-\u009f\u00ad\u061c\u200b\u200e\u200f\u2028\u2029\ufeff\ufff9-\ufffc]/g,(function(e,t,n){e.state.specialChars=new RegExp(t.source+(t.test("\t")?"":"|\t"),"g"),n!=_a&&e.refresh()})),n("specialCharPlaceholder",Jt,(function(e){return e.refresh()}),!0),n("electricChars",!0),n("inputStyle",m?"contenteditable":"textarea",(function(){throw new Error("inputStyle can not (yet) be changed in a running editor")}),!0),n("spellcheck",!1,(function(e,t){return e.getInputField().spellcheck=t}),!0),n("autocorrect",!1,(function(e,t){return e.getInputField().autocorrect=t}),!0),n("autocapitalize",!1,(function(e,t){return e.getInputField().autocapitalize=t}),!0),n("rtlMoveVisually",!C),n("wholeLineUpdateBefore",!0),n("theme","default",(function(e){Aa(e),gi(e)}),!0),n("keyMap","default",(function(e,t,n){var r=Qo(t),i=n!=_a&&Qo(n);i&&i.detach&&i.detach(e,r),r.attach&&r.attach(e,i||null)})),n("extraKeys",null),n("configureMouse",null),n("lineWrapping",!1,Ma,!0),n("gutters",[],(function(e,t){e.display.gutterSpecs=fi(t,e.options.lineNumbers),gi(e)}),!0),n("fixedGutter",!0,(function(e,t){e.display.gutters.style.left=t?ar(e.display)+"px":"0",e.refresh()}),!0),n("coverGutterNextToScrollbar",!1,(function(e){return Hr(e)}),!0),n("scrollbarStyle","native",(function(e){Yr(e),Hr(e),e.display.scrollbars.setScrollTop(e.doc.scrollTop),e.display.scrollbars.setScrollLeft(e.doc.scrollLeft)}),!0),n("lineNumbers",!1,(function(e,t){e.display.gutterSpecs=fi(e.options.gutters,t),gi(e)}),!0),n("firstLineNumber",1,gi,!0),n("lineNumberFormatter",(function(e){return e}),gi,!0),n("showCursorWhenSelecting",!1,vr,!0),n("resetSelectionOnContextMenu",!0),n("lineWiseCopyCut",!0),n("pasteLinesPerSelection",!0),n("selectionsMayTouch",!1),n("readOnly",!1,(function(e,t){"nocursor"==t&&(wr(e),e.display.input.blur()),e.display.input.readOnlyChanged(t)})),n("screenReaderLabel",null,(function(e,t){t=""===t?null:t,e.display.input.screenReaderLabelChanged(t)})),n("disableInput",!1,(function(e,t){t||e.display.input.reset()}),!0),n("dragDrop",!0,wa),n("allowDropFileTypes",null),n("cursorBlinkRate",530),n("cursorScrollMargin",0),n("cursorHeight",1,vr,!0),n("singleCursorHeightPerLine",!0,vr,!0),n("workTime",100),n("workDelay",100),n("flattenSpans",!0,ki,!0),n("addModeClass",!1,ki,!0),n("pollInterval",100),n("undoDepth",200,(function(e,t){return e.doc.history.undoDepth=t})),n("historyEventDelay",1250),n("viewportMargin",10,(function(e){return e.refresh()}),!0),n("maxHighlightLength",1e4,ki,!0),n("moveInputWithCursor",!0,(function(e,t){t||e.display.input.resetPosition()})),n("tabindex",null,(function(e,t){return e.display.input.getField().tabIndex=t||""})),n("autofocus",null),n("direction","ltr",(function(e,t){return e.doc.setDirection(t)}),!0),n("phrases",null)}(Sa),function(e){var t=e.optionHandlers,n=e.helpers={};e.prototype={constructor:e,focus:function(){window.focus(),this.display.input.focus()},setOption:function(e,n){var r=this.options,i=r[e];r[e]==n&&"mode"!=e||(r[e]=n,t.hasOwnProperty(e)&&ei(this,t[e])(this,n,i),pe(this,"optionChange",this,e))},getOption:function(e){return this.options[e]},getDoc:function(){return this.doc},addKeyMap:function(e,t){this.state.keyMaps[t?"push":"unshift"](Qo(e))},removeKeyMap:function(e){for(var t=this.state.keyMaps,n=0;n<t.length;++n)if(t[n]==e||t[n].name==e)return t.splice(n,1),!0},addOverlay:ti((function(t,n){var r=t.token?t:e.getMode(this.options,t);if(r.startState)throw new Error("Overlays may not be stateful.");!function(e,t,n){for(var r=0,i=n(t);r<e.length&&n(e[r])<=i;)r++;e.splice(r,0,t)}(this.state.overlays,{mode:r,modeSpec:t,opaque:n&&n.opaque,priority:n&&n.priority||0},(function(e){return e.priority})),this.state.modeGen++,dr(this)})),removeOverlay:ti((function(e){for(var t=this.state.overlays,n=0;n<t.length;++n){var r=t[n].modeSpec;if(r==e||"string"==typeof e&&r.name==e)return t.splice(n,1),this.state.modeGen++,void dr(this)}})),indentLine:ti((function(e,t,n){"string"!=typeof t&&"number"!=typeof t&&(t=null==t?this.options.smartIndent?"smart":"prev":t?"add":"subtract"),Je(this.doc,e)&&Ea(this,e,t,n)})),indentSelection:ti((function(e){for(var t=this.doc.sel.ranges,n=-1,r=0;r<t.length;r++){var i=t[r];if(i.empty())i.head.line>n&&(Ea(this,i.head.line,e,!0),n=i.head.line,r==this.doc.sel.primIndex&&kr(this));else{var o=i.from(),a=i.to(),s=Math.max(n,o.line);n=Math.min(this.lastLine(),a.line-(a.ch?0:1))+1;for(var l=s;l<n;++l)Ea(this,l,e);var u=this.doc.sel.ranges;0==o.ch&&t.length==u.length&&u[r].from().ch>0&&Qi(this.doc,r,new Ii(o,u[r].to()),U)}}})),getTokenAt:function(e,t){return yt(this,e,t)},getLineTokens:function(e,t){return yt(this,et(e),t,!0)},getTokenTypeAt:function(e){e=st(this.doc,e);var t,n=ht(this,Ge(this.doc,e.line)),r=0,i=(n.length-1)/2,o=e.ch;if(0==o)t=n[2];else for(;;){var a=r+i>>1;if((a?n[2*a-1]:0)>=o)i=a;else{if(!(n[2*a+1]<o)){t=n[2*a+2];break}r=a+1}}var s=t?t.indexOf("overlay "):-1;return s<0?t:0==s?null:t.slice(0,s-1)},getModeAt:function(t){var n=this.doc.mode;return n.innerMode?e.innerMode(n,this.getTokenAt(t).state).mode:n},getHelper:function(e,t){return this.getHelpers(e,t)[0]},getHelpers:function(e,t){var r=[];if(!n.hasOwnProperty(t))return r;var i=n[t],o=this.getModeAt(e);if("string"==typeof o[t])i[o[t]]&&r.push(i[o[t]]);else if(o[t])for(var a=0;a<o[t].length;a++){var s=i[o[t][a]];s&&r.push(s)}else o.helperType&&i[o.helperType]?r.push(i[o.helperType]):i[o.name]&&r.push(i[o.name]);for(var l=0;l<i._global.length;l++){var u=i._global[l];u.pred(o,this)&&-1==B(r,u.val)&&r.push(u.val)}return r},getStateAfter:function(e,t){var n=this.doc;return ft(this,(e=at(n,null==e?n.first+n.size-1:e))+1,t).state},cursorCoords:function(e,t){var n=this.doc.sel.primary();return Xn(this,null==e?n.head:"object"==typeof e?st(this.doc,e):e?n.from():n.to(),t||"page")},charCoords:function(e,t){return Gn(this,st(this.doc,e),t||"page")},coordsChar:function(e,t){return Qn(this,(e=Wn(this,e,t||"page")).left,e.top)},lineAtHeight:function(e,t){return e=Wn(this,{top:e,left:0},t||"page").top,Ke(this.doc,e+this.display.viewOffset)},heightAtLine:function(e,t,n){var r,i=!1;if("number"==typeof e){var o=this.doc.first+this.doc.size-1;e<this.doc.first?e=this.doc.first:e>o&&(e=o,i=!0),r=Ge(this.doc,e)}else r=e;return Yn(this,r,{top:0,left:0},t||"page",n||i).top+(i?this.doc.height-Vt(r):0)},defaultTextHeight:function(){return rr(this.display)},defaultCharWidth:function(){return ir(this.display)},getViewport:function(){return{from:this.display.viewFrom,to:this.display.viewTo}},addWidget:function(e,t,n,r,i){var o,a,s,l=this.display,u=(e=Xn(this,st(this.doc,e))).bottom,c=e.left;if(t.style.position="absolute",t.setAttribute("cm-ignore-events","true"),this.display.input.setUneditable(t),l.sizer.appendChild(t),"over"==r)u=e.top;else if("above"==r||"near"==r){var d=Math.max(l.wrapper.clientHeight,this.doc.height),h=Math.max(l.sizer.clientWidth,l.lineSpace.clientWidth);("above"==r||e.bottom+t.offsetHeight>d)&&e.top>t.offsetHeight?u=e.top-t.offsetHeight:e.bottom+t.offsetHeight<=d&&(u=e.bottom),c+t.offsetWidth>h&&(c=h-t.offsetWidth)}t.style.top=u+"px",t.style.left=t.style.right="","right"==i?(c=l.sizer.clientWidth-t.offsetWidth,t.style.right="0px"):("left"==i?c=0:"middle"==i&&(c=(l.sizer.clientWidth-t.offsetWidth)/2),t.style.left=c+"px"),n&&(o=this,a={left:c,top:u,right:c+t.offsetWidth,bottom:u+t.offsetHeight},null!=(s=Er(o,a)).scrollTop&&Nr(o,s.scrollTop),null!=s.scrollLeft&&Rr(o,s.scrollLeft))},triggerOnKeyDown:ti(ca),triggerOnKeyPress:ti(ha),triggerOnKeyUp:da,triggerOnMouseDown:ti(va),execCommand:function(e){if(ta.hasOwnProperty(e))return ta[e].call(null,this)},triggerElectric:ti((function(e){Pa(this,e)})),findPosH:function(e,t,n,r){var i=1;t<0&&(i=-1,t=-t);for(var o=st(this.doc,e),a=0;a<t&&!(o=za(this.doc,o,i,n,r)).hitSide;++a);return o},moveH:ti((function(e,t){var n=this;this.extendSelectionsBy((function(r){return n.display.shift||n.doc.extend||r.empty()?za(n.doc,r.head,e,t,n.options.rtlMoveVisually):e<0?r.from():r.to()}),Y)})),deleteH:ti((function(e,t){var n=this.doc.sel,r=this.doc;n.somethingSelected()?r.replaceSelection("",null,"+delete"):Ko(this,(function(n){var i=za(r,n.head,e,t,!1);return e<0?{from:i,to:n.head}:{from:n.head,to:i}}))})),findPosV:function(e,t,n,r){var i=1,o=r;t<0&&(i=-1,t=-t);for(var a=st(this.doc,e),s=0;s<t;++s){var l=Xn(this,a,"div");if(null==o?o=l.left:l.left=o,(a=Fa(this,l,i,n)).hitSide)break}return a},moveV:ti((function(e,t){var n=this,r=this.doc,i=[],o=!this.display.shift&&!r.extend&&r.sel.somethingSelected();if(r.extendSelectionsBy((function(a){if(o)return e<0?a.from():a.to();var s=Xn(n,a.head,"div");null!=a.goalColumn&&(s.left=a.goalColumn),i.push(s.left);var l=Fa(n,s,e,t);return"page"==t&&a==r.sel.primary()&&Or(n,Gn(n,l,"div").top-s.top),l}),Y),i.length)for(var a=0;a<r.sel.ranges.length;a++)r.sel.ranges[a].goalColumn=i[a]})),findWordAt:function(e){var t=Ge(this.doc,e.line).text,n=e.ch,r=e.ch;if(t){var i=this.getHelper(e,"wordChars");"before"!=e.sticky&&r!=t.length||!n?++r:--n;for(var o=t.charAt(n),a=ee(o,i)?function(e){return ee(e,i)}:/\s/.test(o)?function(e){return/\s/.test(e)}:function(e){return!/\s/.test(e)&&!ee(e)};n>0&&a(t.charAt(n-1));)--n;for(;r<t.length&&a(t.charAt(r));)++r}return new Ii(et(e.line,n),et(e.line,r))},toggleOverwrite:function(e){null!=e&&e==this.state.overwrite||((this.state.overwrite=!this.state.overwrite)?D(this.display.cursorDiv,"CodeMirror-overwrite"):M(this.display.cursorDiv,"CodeMirror-overwrite"),pe(this,"overwriteToggle",this,this.state.overwrite))},hasFocus:function(){return this.display.input.getField()==L()},isReadOnly:function(){return!(!this.options.readOnly&&!this.doc.cantEdit)},scrollTo:ti((function(e,t){Lr(this,e,t)})),getScrollInfo:function(){var e=this.display.scroller;return{left:e.scrollLeft,top:e.scrollTop,height:e.scrollHeight-Mn(this)-this.display.barHeight,width:e.scrollWidth-Mn(this)-this.display.barWidth,clientHeight:Tn(this),clientWidth:Sn(this)}},scrollIntoView:ti((function(e,t){null==e?(e={from:this.doc.sel.primary().head,to:null},null==t&&(t=this.options.cursorScrollMargin)):"number"==typeof e?e={from:et(e,0),to:null}:null==e.from&&(e={from:e,to:null}),e.to||(e.to=e.from),e.margin=t||0,null!=e.from.line?function(e,t){Dr(e),e.curOp.scrollToPos=t}(this,e):Pr(this,e.from,e.to,e.margin)})),setSize:ti((function(e,t){var n=this,r=function(e){return"number"==typeof e||/^\d+$/.test(String(e))?e+"px":e};null!=e&&(this.display.wrapper.style.width=r(e)),null!=t&&(this.display.wrapper.style.height=r(t)),this.options.lineWrapping&&Fn(this);var i=this.display.viewFrom;this.doc.iter(i,this.display.viewTo,(function(e){if(e.widgets)for(var t=0;t<e.widgets.length;t++)if(e.widgets[t].noHScroll){hr(n,i,"widget");break}++i})),this.curOp.forceUpdate=!0,pe(this,"refresh",this)})),operation:function(e){return $r(this,e)},startOperation:function(){return Gr(this)},endOperation:function(){return Xr(this)},refresh:ti((function(){var e=this.display.cachedTextHeight;dr(this),this.curOp.forceUpdate=!0,Bn(this),Lr(this,this.doc.scrollLeft,this.doc.scrollTop),ui(this.display),(null==e||Math.abs(e-rr(this.display))>.5||this.options.lineWrapping)&&lr(this),pe(this,"refresh",this)})),swapDoc:ti((function(e){var t=this.doc;return t.cm=null,this.state.selectingText&&this.state.selectingText(),Ni(this,e),Bn(this),this.display.input.reset(),Lr(this,e.scrollLeft,e.scrollTop),this.curOp.forceScroll=!0,ln(this,"swapDoc",this,t),t})),phrase:function(e){var t=this.options.phrases;return t&&Object.prototype.hasOwnProperty.call(t,e)?t[e]:e},getInputField:function(){return this.display.input.getField()},getWrapperElement:function(){return this.display.wrapper},getScrollerElement:function(){return this.display.scroller},getGutterElement:function(){return this.display.gutters}},ye(e),e.registerHelper=function(t,r,i){n.hasOwnProperty(t)||(n[t]=e[t]={_global:[]}),n[t][r]=i},e.registerGlobalHelper=function(t,r,i,o){e.registerHelper(t,r,o),n[t]._global.push({pred:i,val:o})}}(Sa);var Ga="iter insert remove copy getEditor constructor".split(" ");for(var Xa in ko.prototype)ko.prototype.hasOwnProperty(Xa)&&B(Ga,Xa)<0&&(Sa.prototype[Xa]=function(e){return function(){return e.apply(this.doc,arguments)}}(ko.prototype[Xa]));return ye(ko),Sa.inputStyles={textarea:Wa,contenteditable:Ba},Sa.defineMode=function(e){Sa.defaults.mode||"null"==e||(Sa.defaults.mode=e),Re.apply(this,arguments)},Sa.defineMIME=function(e,t){je[e]=t},Sa.defineMode("null",(function(){return{token:function(e){return e.skipToEnd()}}})),Sa.defineMIME("text/plain","null"),Sa.defineExtension=function(e,t){Sa.prototype[e]=t},Sa.defineDocExtension=function(e,t){ko.prototype[e]=t},Sa.fromTextArea=function(e,t){if((t=t?R(t):{}).value=e.value,!t.tabindex&&e.tabIndex&&(t.tabindex=e.tabIndex),!t.placeholder&&e.placeholder&&(t.placeholder=e.placeholder),null==t.autofocus){var n=L();t.autofocus=n==e||null!=e.getAttribute("autofocus")&&n==document.body}function r(){e.value=s.getValue()}var i;if(e.form&&(de(e.form,"submit",r),!t.leaveSubmitMethodAlone)){var o=e.form;i=o.submit;try{var a=o.submit=function(){r(),o.submit=i,o.submit(),o.submit=a}}catch(e){}}t.finishInit=function(n){n.save=r,n.getTextArea=function(){return e},n.toTextArea=function(){n.toTextArea=isNaN,r(),e.parentNode.removeChild(n.getWrapperElement()),e.style.display="",e.form&&(fe(e.form,"submit",r),t.leaveSubmitMethodAlone||"function"!=typeof e.form.submit||(e.form.submit=i))}},e.style.display="none";var s=Sa((function(t){return e.parentNode.insertBefore(t,e.nextSibling)}),t);return s},function(e){e.off=fe,e.on=de,e.wheelEventPixels=Ci,e.Doc=ko,e.splitLines=ke,e.countColumn=z,e.findColumn=W,e.isWordChar=$,e.Pass=H,e.signal=pe,e.Line=Gt,e.changeEnd=Mi,e.scrollbarModel=Vr,e.Pos=et,e.cmpPos=tt,e.modes=Ne,e.mimeModes=je,e.resolveMode=ze,e.getMode=Fe,e.modeExtensions=Be,e.extendMode=He,e.copyState=Ue,e.startState=Ye,e.innerMode=Ve,e.commands=ta,e.keyMap=Vo,e.keyName=Zo,e.isModifierKey=Xo,e.lookupKey=Go,e.normalizeKeyMap=Wo,e.StringStream=We,e.SharedTextMarker=So,e.TextMarker=wo,e.LineWidget=_o,e.e_preventDefault=be,e.e_stopPropagation=Ce,e.e_stop=_e,e.addClass=D,e.contains=k,e.rmClass=M,e.keyNames=Fo}(Sa),Sa.version="5.59.4",Sa}()},function(e,t,n){var r;!function(i){var o=/^\s+/,a=/\s+$/,s=0,l=i.round,u=i.min,c=i.max,d=i.random;function h(e,t){if(t=t||{},(e=e||"")instanceof h)return e;if(!(this instanceof h))return new h(e,t);var n=function(e){var t={r:0,g:0,b:0},n=1,r=null,s=null,l=null,d=!1,h=!1;"string"==typeof e&&(e=function(e){e=e.replace(o,"").replace(a,"").toLowerCase();var t,n=!1;if(O[e])e=O[e],n=!0;else if("transparent"==e)return{r:0,g:0,b:0,a:0,format:"name"};if(t=V.rgb.exec(e))return{r:t[1],g:t[2],b:t[3]};if(t=V.rgba.exec(e))return{r:t[1],g:t[2],b:t[3],a:t[4]};if(t=V.hsl.exec(e))return{h:t[1],s:t[2],l:t[3]};if(t=V.hsla.exec(e))return{h:t[1],s:t[2],l:t[3],a:t[4]};if(t=V.hsv.exec(e))return{h:t[1],s:t[2],v:t[3]};if(t=V.hsva.exec(e))return{h:t[1],s:t[2],v:t[3],a:t[4]};if(t=V.hex8.exec(e))return{r:N(t[1]),g:N(t[2]),b:N(t[3]),a:F(t[4]),format:n?"name":"hex8"};if(t=V.hex6.exec(e))return{r:N(t[1]),g:N(t[2]),b:N(t[3]),format:n?"name":"hex"};if(t=V.hex4.exec(e))return{r:N(t[1]+""+t[1]),g:N(t[2]+""+t[2]),b:N(t[3]+""+t[3]),a:F(t[4]+""+t[4]),format:n?"name":"hex8"};if(t=V.hex3.exec(e))return{r:N(t[1]+""+t[1]),g:N(t[2]+""+t[2]),b:N(t[3]+""+t[3]),format:n?"name":"hex"};return!1}(e));"object"==typeof e&&(Y(e.r)&&Y(e.g)&&Y(e.b)?(f=e.r,p=e.g,g=e.b,t={r:255*D(f,255),g:255*D(p,255),b:255*D(g,255)},d=!0,h="%"===String(e.r).substr(-1)?"prgb":"rgb"):Y(e.h)&&Y(e.s)&&Y(e.v)?(r=R(e.s),s=R(e.v),t=function(e,t,n){e=6*D(e,360),t=D(t,100),n=D(n,100);var r=i.floor(e),o=e-r,a=n*(1-t),s=n*(1-o*t),l=n*(1-(1-o)*t),u=r%6;return{r:255*[n,s,a,a,l,n][u],g:255*[l,n,n,s,a,a][u],b:255*[a,a,l,n,n,s][u]}}(e.h,r,s),d=!0,h="hsv"):Y(e.h)&&Y(e.s)&&Y(e.l)&&(r=R(e.s),l=R(e.l),t=function(e,t,n){var r,i,o;function a(e,t,n){return n<0&&(n+=1),n>1&&(n-=1),n<1/6?e+6*(t-e)*n:n<.5?t:n<2/3?e+(t-e)*(2/3-n)*6:e}if(e=D(e,360),t=D(t,100),n=D(n,100),0===t)r=i=o=n;else{var s=n<.5?n*(1+t):n+t-n*t,l=2*n-s;r=a(l,s,e+1/3),i=a(l,s,e),o=a(l,s,e-1/3)}return{r:255*r,g:255*i,b:255*o}}(e.h,r,l),d=!0,h="hsl"),e.hasOwnProperty("a")&&(n=e.a));var f,p,g;return n=L(n),{ok:d,format:e.format||h,r:u(255,c(t.r,0)),g:u(255,c(t.g,0)),b:u(255,c(t.b,0)),a:n}}(e);this._originalInput=e,this._r=n.r,this._g=n.g,this._b=n.b,this._a=n.a,this._roundA=l(100*this._a)/100,this._format=t.format||n.format,this._gradientType=t.gradientType,this._r<1&&(this._r=l(this._r)),this._g<1&&(this._g=l(this._g)),this._b<1&&(this._b=l(this._b)),this._ok=n.ok,this._tc_id=s++}function f(e,t,n){e=D(e,255),t=D(t,255),n=D(n,255);var r,i,o=c(e,t,n),a=u(e,t,n),s=(o+a)/2;if(o==a)r=i=0;else{var l=o-a;switch(i=s>.5?l/(2-o-a):l/(o+a),o){case e:r=(t-n)/l+(t<n?6:0);break;case t:r=(n-e)/l+2;break;case n:r=(e-t)/l+4}r/=6}return{h:r,s:i,l:s}}function p(e,t,n){e=D(e,255),t=D(t,255),n=D(n,255);var r,i,o=c(e,t,n),a=u(e,t,n),s=o,l=o-a;if(i=0===o?0:l/o,o==a)r=0;else{switch(o){case e:r=(t-n)/l+(t<n?6:0);break;case t:r=(n-e)/l+2;break;case n:r=(e-t)/l+4}r/=6}return{h:r,s:i,v:s}}function g(e,t,n,r){var i=[j(l(e).toString(16)),j(l(t).toString(16)),j(l(n).toString(16))];return r&&i[0].charAt(0)==i[0].charAt(1)&&i[1].charAt(0)==i[1].charAt(1)&&i[2].charAt(0)==i[2].charAt(1)?i[0].charAt(0)+i[1].charAt(0)+i[2].charAt(0):i.join("")}function v(e,t,n,r){return[j(z(r)),j(l(e).toString(16)),j(l(t).toString(16)),j(l(n).toString(16))].join("")}function m(e,t){t=0===t?0:t||10;var n=h(e).toHsl();return n.s-=t/100,n.s=P(n.s),h(n)}function y(e,t){t=0===t?0:t||10;var n=h(e).toHsl();return n.s+=t/100,n.s=P(n.s),h(n)}function b(e){return h(e).desaturate(100)}function C(e,t){t=0===t?0:t||10;var n=h(e).toHsl();return n.l+=t/100,n.l=P(n.l),h(n)}function A(e,t){t=0===t?0:t||10;var n=h(e).toRgb();return n.r=c(0,u(255,n.r-l(-t/100*255))),n.g=c(0,u(255,n.g-l(-t/100*255))),n.b=c(0,u(255,n.b-l(-t/100*255))),h(n)}function _(e,t){t=0===t?0:t||10;var n=h(e).toHsl();return n.l-=t/100,n.l=P(n.l),h(n)}function I(e,t){var n=h(e).toHsl(),r=(n.h+t)%360;return n.h=r<0?360+r:r,h(n)}function x(e){var t=h(e).toHsl();return t.h=(t.h+180)%360,h(t)}function w(e){var t=h(e).toHsl(),n=t.h;return[h(e),h({h:(n+120)%360,s:t.s,l:t.l}),h({h:(n+240)%360,s:t.s,l:t.l})]}function M(e){var t=h(e).toHsl(),n=t.h;return[h(e),h({h:(n+90)%360,s:t.s,l:t.l}),h({h:(n+180)%360,s:t.s,l:t.l}),h({h:(n+270)%360,s:t.s,l:t.l})]}function S(e){var t=h(e).toHsl(),n=t.h;return[h(e),h({h:(n+72)%360,s:t.s,l:t.l}),h({h:(n+216)%360,s:t.s,l:t.l})]}function T(e,t,n){t=t||6,n=n||30;var r=h(e).toHsl(),i=360/n,o=[h(e)];for(r.h=(r.h-(i*t>>1)+720)%360;--t;)r.h=(r.h+i)%360,o.push(h(r));return o}function E(e,t){t=t||6;for(var n=h(e).toHsv(),r=n.h,i=n.s,o=n.v,a=[],s=1/t;t--;)a.push(h({h:r,s:i,v:o})),o=(o+s)%1;return a}h.prototype={isDark:function(){return this.getBrightness()<128},isLight:function(){return!this.isDark()},isValid:function(){return this._ok},getOriginalInput:function(){return this._originalInput},getFormat:function(){return this._format},getAlpha:function(){return this._a},getBrightness:function(){var e=this.toRgb();return(299*e.r+587*e.g+114*e.b)/1e3},getLuminance:function(){var e,t,n,r=this.toRgb();return e=r.r/255,t=r.g/255,n=r.b/255,.2126*(e<=.03928?e/12.92:i.pow((e+.055)/1.055,2.4))+.7152*(t<=.03928?t/12.92:i.pow((t+.055)/1.055,2.4))+.0722*(n<=.03928?n/12.92:i.pow((n+.055)/1.055,2.4))},setAlpha:function(e){return this._a=L(e),this._roundA=l(100*this._a)/100,this},toHsv:function(){var e=p(this._r,this._g,this._b);return{h:360*e.h,s:e.s,v:e.v,a:this._a}},toHsvString:function(){var e=p(this._r,this._g,this._b),t=l(360*e.h),n=l(100*e.s),r=l(100*e.v);return 1==this._a?"hsv("+t+", "+n+"%, "+r+"%)":"hsva("+t+", "+n+"%, "+r+"%, "+this._roundA+")"},toHsl:function(){var e=f(this._r,this._g,this._b);return{h:360*e.h,s:e.s,l:e.l,a:this._a}},toHslString:function(){var e=f(this._r,this._g,this._b),t=l(360*e.h),n=l(100*e.s),r=l(100*e.l);return 1==this._a?"hsl("+t+", "+n+"%, "+r+"%)":"hsla("+t+", "+n+"%, "+r+"%, "+this._roundA+")"},toHex:function(e){return g(this._r,this._g,this._b,e)},toHexString:function(e){return"#"+this.toHex(e)},toHex8:function(e){return function(e,t,n,r,i){var o=[j(l(e).toString(16)),j(l(t).toString(16)),j(l(n).toString(16)),j(z(r))];if(i&&o[0].charAt(0)==o[0].charAt(1)&&o[1].charAt(0)==o[1].charAt(1)&&o[2].charAt(0)==o[2].charAt(1)&&o[3].charAt(0)==o[3].charAt(1))return o[0].charAt(0)+o[1].charAt(0)+o[2].charAt(0)+o[3].charAt(0);return o.join("")}(this._r,this._g,this._b,this._a,e)},toHex8String:function(e){return"#"+this.toHex8(e)},toRgb:function(){return{r:l(this._r),g:l(this._g),b:l(this._b),a:this._a}},toRgbString:function(){return 1==this._a?"rgb("+l(this._r)+", "+l(this._g)+", "+l(this._b)+")":"rgba("+l(this._r)+", "+l(this._g)+", "+l(this._b)+", "+this._roundA+")"},toPercentageRgb:function(){return{r:l(100*D(this._r,255))+"%",g:l(100*D(this._g,255))+"%",b:l(100*D(this._b,255))+"%",a:this._a}},toPercentageRgbString:function(){return 1==this._a?"rgb("+l(100*D(this._r,255))+"%, "+l(100*D(this._g,255))+"%, "+l(100*D(this._b,255))+"%)":"rgba("+l(100*D(this._r,255))+"%, "+l(100*D(this._g,255))+"%, "+l(100*D(this._b,255))+"%, "+this._roundA+")"},toName:function(){return 0===this._a?"transparent":!(this._a<1)&&(k[g(this._r,this._g,this._b,!0)]||!1)},toFilter:function(e){var t="#"+v(this._r,this._g,this._b,this._a),n=t,r=this._gradientType?"GradientType = 1, ":"";if(e){var i=h(e);n="#"+v(i._r,i._g,i._b,i._a)}return"progid:DXImageTransform.Microsoft.gradient("+r+"startColorstr="+t+",endColorstr="+n+")"},toString:function(e){var t=!!e;e=e||this._format;var n=!1,r=this._a<1&&this._a>=0;return t||!r||"hex"!==e&&"hex6"!==e&&"hex3"!==e&&"hex4"!==e&&"hex8"!==e&&"name"!==e?("rgb"===e&&(n=this.toRgbString()),"prgb"===e&&(n=this.toPercentageRgbString()),"hex"!==e&&"hex6"!==e||(n=this.toHexString()),"hex3"===e&&(n=this.toHexString(!0)),"hex4"===e&&(n=this.toHex8String(!0)),"hex8"===e&&(n=this.toHex8String()),"name"===e&&(n=this.toName()),"hsl"===e&&(n=this.toHslString()),"hsv"===e&&(n=this.toHsvString()),n||this.toHexString()):"name"===e&&0===this._a?this.toName():this.toRgbString()},clone:function(){return h(this.toString())},_applyModification:function(e,t){var n=e.apply(null,[this].concat([].slice.call(t)));return this._r=n._r,this._g=n._g,this._b=n._b,this.setAlpha(n._a),this},lighten:function(){return this._applyModification(C,arguments)},brighten:function(){return this._applyModification(A,arguments)},darken:function(){return this._applyModification(_,arguments)},desaturate:function(){return this._applyModification(m,arguments)},saturate:function(){return this._applyModification(y,arguments)},greyscale:function(){return this._applyModification(b,arguments)},spin:function(){return this._applyModification(I,arguments)},_applyCombination:function(e,t){return e.apply(null,[this].concat([].slice.call(t)))},analogous:function(){return this._applyCombination(T,arguments)},complement:function(){return this._applyCombination(x,arguments)},monochromatic:function(){return this._applyCombination(E,arguments)},splitcomplement:function(){return this._applyCombination(S,arguments)},triad:function(){return this._applyCombination(w,arguments)},tetrad:function(){return this._applyCombination(M,arguments)}},h.fromRatio=function(e,t){if("object"==typeof e){var n={};for(var r in e)e.hasOwnProperty(r)&&(n[r]="a"===r?e[r]:R(e[r]));e=n}return h(e,t)},h.equals=function(e,t){return!(!e||!t)&&h(e).toRgbString()==h(t).toRgbString()},h.random=function(){return h.fromRatio({r:d(),g:d(),b:d()})},h.mix=function(e,t,n){n=0===n?0:n||50;var r=h(e).toRgb(),i=h(t).toRgb(),o=n/100;return h({r:(i.r-r.r)*o+r.r,g:(i.g-r.g)*o+r.g,b:(i.b-r.b)*o+r.b,a:(i.a-r.a)*o+r.a})},h.readability=function(e,t){var n=h(e),r=h(t);return(i.max(n.getLuminance(),r.getLuminance())+.05)/(i.min(n.getLuminance(),r.getLuminance())+.05)},h.isReadable=function(e,t,n){var r,i,o=h.readability(e,t);switch(i=!1,(r=function(e){var t,n;t=((e=e||{level:"AA",size:"small"}).level||"AA").toUpperCase(),n=(e.size||"small").toLowerCase(),"AA"!==t&&"AAA"!==t&&(t="AA");"small"!==n&&"large"!==n&&(n="small");return{level:t,size:n}}(n)).level+r.size){case"AAsmall":case"AAAlarge":i=o>=4.5;break;case"AAlarge":i=o>=3;break;case"AAAsmall":i=o>=7}return i},h.mostReadable=function(e,t,n){var r,i,o,a,s=null,l=0;i=(n=n||{}).includeFallbackColors,o=n.level,a=n.size;for(var u=0;u<t.length;u++)(r=h.readability(e,t[u]))>l&&(l=r,s=h(t[u]));return h.isReadable(e,s,{level:o,size:a})||!i?s:(n.includeFallbackColors=!1,h.mostReadable(e,["#fff","#000"],n))};var O=h.names={aliceblue:"f0f8ff",antiquewhite:"faebd7",aqua:"0ff",aquamarine:"7fffd4",azure:"f0ffff",beige:"f5f5dc",bisque:"ffe4c4",black:"000",blanchedalmond:"ffebcd",blue:"00f",blueviolet:"8a2be2",brown:"a52a2a",burlywood:"deb887",burntsienna:"ea7e5d",cadetblue:"5f9ea0",chartreuse:"7fff00",chocolate:"d2691e",coral:"ff7f50",cornflowerblue:"6495ed",cornsilk:"fff8dc",crimson:"dc143c",cyan:"0ff",darkblue:"00008b",darkcyan:"008b8b",darkgoldenrod:"b8860b",darkgray:"a9a9a9",darkgreen:"006400",darkgrey:"a9a9a9",darkkhaki:"bdb76b",darkmagenta:"8b008b",darkolivegreen:"556b2f",darkorange:"ff8c00",darkorchid:"9932cc",darkred:"8b0000",darksalmon:"e9967a",darkseagreen:"8fbc8f",darkslateblue:"483d8b",darkslategray:"2f4f4f",darkslategrey:"2f4f4f",darkturquoise:"00ced1",darkviolet:"9400d3",deeppink:"ff1493",deepskyblue:"00bfff",dimgray:"696969",dimgrey:"696969",dodgerblue:"1e90ff",firebrick:"b22222",floralwhite:"fffaf0",forestgreen:"228b22",fuchsia:"f0f",gainsboro:"dcdcdc",ghostwhite:"f8f8ff",gold:"ffd700",goldenrod:"daa520",gray:"808080",green:"008000",greenyellow:"adff2f",grey:"808080",honeydew:"f0fff0",hotpink:"ff69b4",indianred:"cd5c5c",indigo:"4b0082",ivory:"fffff0",khaki:"f0e68c",lavender:"e6e6fa",lavenderblush:"fff0f5",lawngreen:"7cfc00",lemonchiffon:"fffacd",lightblue:"add8e6",lightcoral:"f08080",lightcyan:"e0ffff",lightgoldenrodyellow:"fafad2",lightgray:"d3d3d3",lightgreen:"90ee90",lightgrey:"d3d3d3",lightpink:"ffb6c1",lightsalmon:"ffa07a",lightseagreen:"20b2aa",lightskyblue:"87cefa",lightslategray:"789",lightslategrey:"789",lightsteelblue:"b0c4de",lightyellow:"ffffe0",lime:"0f0",limegreen:"32cd32",linen:"faf0e6",magenta:"f0f",maroon:"800000",mediumaquamarine:"66cdaa",mediumblue:"0000cd",mediumorchid:"ba55d3",mediumpurple:"9370db",mediumseagreen:"3cb371",mediumslateblue:"7b68ee",mediumspringgreen:"00fa9a",mediumturquoise:"48d1cc",mediumvioletred:"c71585",midnightblue:"191970",mintcream:"f5fffa",mistyrose:"ffe4e1",moccasin:"ffe4b5",navajowhite:"ffdead",navy:"000080",oldlace:"fdf5e6",olive:"808000",olivedrab:"6b8e23",orange:"ffa500",orangered:"ff4500",orchid:"da70d6",palegoldenrod:"eee8aa",palegreen:"98fb98",paleturquoise:"afeeee",palevioletred:"db7093",papayawhip:"ffefd5",peachpuff:"ffdab9",peru:"cd853f",pink:"ffc0cb",plum:"dda0dd",powderblue:"b0e0e6",purple:"800080",rebeccapurple:"663399",red:"f00",rosybrown:"bc8f8f",royalblue:"4169e1",saddlebrown:"8b4513",salmon:"fa8072",sandybrown:"f4a460",seagreen:"2e8b57",seashell:"fff5ee",sienna:"a0522d",silver:"c0c0c0",skyblue:"87ceeb",slateblue:"6a5acd",slategray:"708090",slategrey:"708090",snow:"fffafa",springgreen:"00ff7f",steelblue:"4682b4",tan:"d2b48c",teal:"008080",thistle:"d8bfd8",tomato:"ff6347",turquoise:"40e0d0",violet:"ee82ee",wheat:"f5deb3",white:"fff",whitesmoke:"f5f5f5",yellow:"ff0",yellowgreen:"9acd32"},k=h.hexNames=function(e){var t={};for(var n in e)e.hasOwnProperty(n)&&(t[e[n]]=n);return t}(O);function L(e){return e=parseFloat(e),(isNaN(e)||e<0||e>1)&&(e=1),e}function D(e,t){(function(e){return"string"==typeof e&&-1!=e.indexOf(".")&&1===parseFloat(e)})(e)&&(e="100%");var n=function(e){return"string"==typeof e&&-1!=e.indexOf("%")}(e);return e=u(t,c(0,parseFloat(e))),n&&(e=parseInt(e*t,10)/100),i.abs(e-t)<1e-6?1:e%t/parseFloat(t)}function P(e){return u(1,c(0,e))}function N(e){return parseInt(e,16)}function j(e){return 1==e.length?"0"+e:""+e}function R(e){return e<=1&&(e=100*e+"%"),e}function z(e){return i.round(255*parseFloat(e)).toString(16)}function F(e){return N(e)/255}var B,H,U,V=(H="[\\s|\\(]+("+(B="(?:[-\\+]?\\d*\\.\\d+%?)|(?:[-\\+]?\\d+%?)")+")[,|\\s]+("+B+")[,|\\s]+("+B+")\\s*\\)?",U="[\\s|\\(]+("+B+")[,|\\s]+("+B+")[,|\\s]+("+B+")[,|\\s]+("+B+")\\s*\\)?",{CSS_UNIT:new RegExp(B),rgb:new RegExp("rgb"+H),rgba:new RegExp("rgba"+U),hsl:new RegExp("hsl"+H),hsla:new RegExp("hsla"+U),hsv:new RegExp("hsv"+H),hsva:new RegExp("hsva"+U),hex3:/^#?([0-9a-fA-F]{1})([0-9a-fA-F]{1})([0-9a-fA-F]{1})$/,hex6:/^#?([0-9a-fA-F]{2})([0-9a-fA-F]{2})([0-9a-fA-F]{2})$/,hex4:/^#?([0-9a-fA-F]{1})([0-9a-fA-F]{1})([0-9a-fA-F]{1})([0-9a-fA-F]{1})$/,hex8:/^#?([0-9a-fA-F]{2})([0-9a-fA-F]{2})([0-9a-fA-F]{2})([0-9a-fA-F]{2})$/});function Y(e){return!!V.CSS_UNIT.exec(e)}e.exports?e.exports=h:void 0===(r=function(){return h}.call(t,n,t,e))||(e.exports=r)}(Math)},function(e,t,n){"use strict";var r=n(10),i=r.extendFlat,o=r.isPlainObject,a={valType:"flaglist",extras:["none"],flags:["calc","clearAxisTypes","plot","style","markerSize","colorbars"]},s={valType:"flaglist",extras:["none"],flags:["calc","plot","legend","ticks","axrange","layoutstyle","modebar","camera","arraydraw","colorbars"]},l=a.flags.slice().concat(["fullReplot"]),u=s.flags.slice().concat("layoutReplot");function c(e){for(var t={},n=0;n<e.length;n++)t[e[n]]=!1;return t}function d(e,t,n){var r=i({},e);for(var a in r){var s=r[a];o(s)&&(r[a]=h(s,t,n,a))}return"from-root"===n&&(r.editType=t),r}function h(e,t,n,r){if(e.valType){var o=i({},e);if(o.editType=t,Array.isArray(e.items)){o.items=new Array(e.items.length);for(var a=0;a<e.items.length;a++)o.items[a]=h(e.items[a],t,"from-root")}return o}return d(e,t,"_"===r.charAt(0)?"nested":"from-root")}e.exports={traces:a,layout:s,traceFlags:function(){return c(l)},layoutFlags:function(){return c(u)},update:function(e,t){var n=t.editType;if(n&&"none"!==n)for(var r=n.split("+"),i=0;i<r.length;i++)e[r[i]]=!0},overrideAll:d}},function(e,t,n){"use strict";var r=n(10);e.exports={hasLines:function(e){return e.visible&&e.mode&&-1!==e.mode.indexOf("lines")},hasMarkers:function(e){return e.visible&&(e.mode&&-1!==e.mode.indexOf("markers")||"splom"===e.type)},hasText:function(e){return e.visible&&e.mode&&-1!==e.mode.indexOf("text")},isBubble:function(e){return r.isPlainObject(e.marker)&&r.isArrayOrTypedArray(e.marker.size)}}},function(e,t){e.exports={Always:1,Never:2,IfAtMostOneObstacle:3,OnlyWhenNoObstacles:4}},function(e,t,n){"use strict";function r(e){return(r=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}n.d(t,"a",(function(){return r}))},function(e,t,n){"use strict";n.d(t,"a",(function(){return o})),n.d(t,"b",(function(){return a}));var r=n(48);function i(e){return e.map(e=>e&&"object"==typeof e?Object.keys(e).map(t=>!!e[t]&&t):"string"==typeof e?e.split(/\s+/):[]).reduce((e,t)=>e.concat(t),[]).filter(e=>!!e)}function o(...e){return i(e).join(" ")}function a(e,{ignore:t=[]}={}){return e.getAttributeNames().reduce((n,i)=>("style"===i||t.includes(i)||(i.startsWith("data")?n[i]=e.getAttribute(i):n[r.Text.camelCase(i)]=e.getAttribute(i)),n),{})}},function(e,t,n){"use strict";(function(e,r){var i,o,a;Object.defineProperty(t,"__esModule",{value:!0}),t.ServerConnection=void 0;const s=n(48);let l,u,c,d;if("undefined"==typeof window){const t=n(1357);l=null!==(i=e.fetch)&&void 0!==i?i:t,c=null!==(o=e.Request)&&void 0!==o?o:t.Request,u=null!==(a=e.Headers)&&void 0!==a?a:t.Headers,d=n(1358)}else l=fetch,c=Request,u=Headers,d=WebSocket;var h,f;!function(e){e.makeSettings=function(e){return f.makeSettings(e)},e.makeRequest=function(e,t,n){return f.handleRequest(e,t,n)};class t extends Error{constructor(e,t=`Invalid response: ${e.status} ${e.statusText}`,n=""){super(t),this.response=e,this.traceback=n}static async create(e){try{const n=await e.json();return n.traceback&&console.error(n.traceback),n.message?new t(e,n.message):new t(e)}catch(n){return console.debug(n),new t(e)}}}e.ResponseError=t;class n extends TypeError{constructor(e){super(e.message),this.stack=e.stack}}e.NetworkError=n}(h=t.ServerConnection||(t.ServerConnection={})),function(e){e.makeSettings=function(e={}){const t=s.PageConfig.getBaseUrl(),n=s.PageConfig.getWsUrl(),i=s.URLExt.normalize(e.baseUrl)||t;let o=e.wsUrl;return o||i!==t||(o=n),o||0!==i.indexOf("http")||(o="ws"+i.slice(4)),o=null!=o?o:n,Object.assign(Object.assign({init:{cache:"no-store",credentials:"same-origin"},fetch:l,Headers:u,Request:c,WebSocket:d,token:s.PageConfig.getToken(),appUrl:s.PageConfig.getOption("appUrl"),appendToken:"undefined"==typeof window||void 0!==r.env.JEST_WORKER_ID},e),{baseUrl:i,wsUrl:o})},e.handleRequest=function(e,t,n){var r;if(0!==e.indexOf(n.baseUrl))throw new Error("Can only be used for notebook server requests");"no-store"===(null!==(r=t.cache)&&void 0!==r?r:n.init.cache)&&(e+=(/\?/.test(e)?"&":"?")+(new Date).getTime());const i=new n.Request(e,Object.assign(Object.assign({},n.init),t));let o=!1;if(n.token&&(o=!0,i.headers.append("Authorization","token "+n.token)),"undefined"!=typeof document&&(null===document||void 0===document?void 0:document.cookie)){const e=function(e){const t=document.cookie.match("\\b"+e+"=([^;]*)\\b");return null==t?void 0:t[1]}("_xsrf");void 0!==e&&(o=!0,i.headers.append("X-XSRFToken",e))}return!i.headers.has("Content-Type")&&o&&i.headers.set("Content-Type","application/json"),n.fetch.call(null,i).catch(e=>{throw new h.NetworkError(e)})}}(f||(f={}))}).call(this,n(61),n(145))},function(e,t){e.exports=function(e){var t=typeof e;return null!=e&&("object"==t||"function"==t)}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(111),i=function(){function e(e,t,n,r){this.x=e,this.y=t,this.width=n,this.height=r}return e.empty=function(){return new e(0,0,0,0)},e.prototype.clone=function(){return new e(this.x,this.y,this.width,this.height)},e.prototype.equals=function(e){return this.x===e.x&&this.y===e.y&&this.width===e.width&&this.height===e.height},e.prototype.getBottom=function(){return this.y+this.height},e.prototype.getRight=function(){return this.x+this.width},e.prototype.positionElement=function(e){this.styleWithPosition(e.style)},e.prototype.styleWithPosition=function(e){return e.left=this.x+"px",e.top=this.y+"px",e.width=Math.max(0,this.width)+"px",e.height=Math.max(0,this.height)+"px",e.position="absolute",e},e.prototype.contains=function(e,t){return this.x<=e&&e<=this.getRight()&&this.y<=t&&t<=this.getBottom()},e.prototype.removeInsets=function(t){return new e(this.x+t.left,this.y+t.top,Math.max(0,this.width-t.left-t.right),Math.max(0,this.height-t.top-t.bottom))},e.prototype.centerInRect=function(e){this.x=(e.width-this.width)/2,this.y=(e.height-this.height)/2},e.prototype._getSize=function(e){var t=this.width;return e===r.default.VERT&&(t=this.height),t},e.prototype.toString=function(){return"(Rect: x="+this.x+", y="+this.y+", width="+this.width+", height="+this.height+")"},e}();t.default=i},function(e,t,n){"use strict";n.r(t);var r=n(639);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";t.defaults=["#1f77b4","#ff7f0e","#2ca02c","#d62728","#9467bd","#8c564b","#e377c2","#7f7f7f","#bcbd22","#17becf"],t.defaultLine="#444",t.lightLine="#eee",t.background="#fff",t.borderLine="#BEC8D9",t.lightFraction=1e3/11},function(e,t,n){"use strict";var r=n(94),i=n(133),o=n(296).dash,a=n(63).extendFlat,s=n(64).templatedArray,l=(n(194).FORMAT_LINK,n(194).DATE_FORMAT_LINK,n(72).ONEDAY),u=n(95),c=u.HOUR_PATTERN,d=u.WEEKDAY_PATTERN;e.exports={visible:{valType:"boolean",editType:"plot"},color:{valType:"color",dflt:i.defaultLine,editType:"ticks"},title:{text:{valType:"string",editType:"ticks"},font:r({editType:"ticks"}),standoff:{valType:"number",min:0,editType:"ticks"},editType:"ticks"},type:{valType:"enumerated",values:["-","linear","log","date","category","multicategory"],dflt:"-",editType:"calc",_noTemplating:!0},autotypenumbers:{valType:"enumerated",values:["convert types","strict"],dflt:"convert types",editType:"calc"},autorange:{valType:"enumerated",values:[!0,!1,"reversed"],dflt:!0,editType:"axrange",impliedEdits:{"range[0]":void 0,"range[1]":void 0}},rangemode:{valType:"enumerated",values:["normal","tozero","nonnegative"],dflt:"normal",editType:"plot"},range:{valType:"info_array",items:[{valType:"any",editType:"axrange",impliedEdits:{"^autorange":!1},anim:!0},{valType:"any",editType:"axrange",impliedEdits:{"^autorange":!1},anim:!0}],editType:"axrange",impliedEdits:{autorange:!1},anim:!0},fixedrange:{valType:"boolean",dflt:!1,editType:"calc"},scaleanchor:{valType:"enumerated",values:[u.idRegex.x.toString(),u.idRegex.y.toString()],editType:"plot"},scaleratio:{valType:"number",min:0,dflt:1,editType:"plot"},constrain:{valType:"enumerated",values:["range","domain"],editType:"plot"},constraintoward:{valType:"enumerated",values:["left","center","right","top","middle","bottom"],editType:"plot"},matches:{valType:"enumerated",values:[u.idRegex.x.toString(),u.idRegex.y.toString()],editType:"calc"},rangebreaks:s("rangebreak",{enabled:{valType:"boolean",dflt:!0,editType:"calc"},bounds:{valType:"info_array",items:[{valType:"any",editType:"calc"},{valType:"any",editType:"calc"}],editType:"calc"},pattern:{valType:"enumerated",values:[d,c,""],editType:"calc"},values:{valType:"info_array",freeLength:!0,editType:"calc",items:{valType:"any",editType:"calc"}},dvalue:{valType:"number",editType:"calc",min:0,dflt:l},editType:"calc"}),tickmode:{valType:"enumerated",values:["auto","linear","array"],editType:"ticks",impliedEdits:{tick0:void 0,dtick:void 0}},nticks:{valType:"integer",min:0,dflt:0,editType:"ticks"},tick0:{valType:"any",editType:"ticks",impliedEdits:{tickmode:"linear"}},dtick:{valType:"any",editType:"ticks",impliedEdits:{tickmode:"linear"}},tickvals:{valType:"data_array",editType:"ticks"},ticktext:{valType:"data_array",editType:"ticks"},ticks:{valType:"enumerated",values:["outside","inside",""],editType:"ticks"},tickson:{valType:"enumerated",values:["labels","boundaries"],dflt:"labels",editType:"ticks"},ticklabelmode:{valType:"enumerated",values:["instant","period"],dflt:"instant",editType:"ticks"},ticklabelposition:{valType:"enumerated",values:["outside","inside","outside top","inside top","outside left","inside left","outside right","inside right","outside bottom","inside bottom"],dflt:"outside",editType:"calc"},mirror:{valType:"enumerated",values:[!0,"ticks",!1,"all","allticks"],dflt:!1,editType:"ticks+layoutstyle"},ticklen:{valType:"number",min:0,dflt:5,editType:"ticks"},tickwidth:{valType:"number",min:0,dflt:1,editType:"ticks"},tickcolor:{valType:"color",dflt:i.defaultLine,editType:"ticks"},showticklabels:{valType:"boolean",dflt:!0,editType:"ticks"},automargin:{valType:"boolean",dflt:!1,editType:"ticks"},showspikes:{valType:"boolean",dflt:!1,editType:"modebar"},spikecolor:{valType:"color",dflt:null,editType:"none"},spikethickness:{valType:"number",dflt:3,editType:"none"},spikedash:a({},o,{dflt:"dash",editType:"none"}),spikemode:{valType:"flaglist",flags:["toaxis","across","marker"],dflt:"toaxis",editType:"none"},spikesnap:{valType:"enumerated",values:["data","cursor","hovered data"],dflt:"data",editType:"none"},tickfont:r({editType:"ticks"}),tickangle:{valType:"angle",dflt:"auto",editType:"ticks"},tickprefix:{valType:"string",dflt:"",editType:"ticks"},showtickprefix:{valType:"enumerated",values:["all","first","last","none"],dflt:"all",editType:"ticks"},ticksuffix:{valType:"string",dflt:"",editType:"ticks"},showticksuffix:{valType:"enumerated",values:["all","first","last","none"],dflt:"all",editType:"ticks"},showexponent:{valType:"enumerated",values:["all","first","last","none"],dflt:"all",editType:"ticks"},exponentformat:{valType:"enumerated",values:["none","e","E","power","SI","B"],dflt:"B",editType:"ticks"},minexponent:{valType:"number",dflt:3,min:0,editType:"ticks"},separatethousands:{valType:"boolean",dflt:!1,editType:"ticks"},tickformat:{valType:"string",dflt:"",editType:"ticks"},tickformatstops:s("tickformatstop",{enabled:{valType:"boolean",dflt:!0,editType:"ticks"},dtickrange:{valType:"info_array",items:[{valType:"any",editType:"ticks"},{valType:"any",editType:"ticks"}],editType:"ticks"},value:{valType:"string",dflt:"",editType:"ticks"},editType:"ticks"}),hoverformat:{valType:"string",dflt:"",editType:"none"},showline:{valType:"boolean",dflt:!1,editType:"ticks+layoutstyle"},linecolor:{valType:"color",dflt:i.defaultLine,editType:"layoutstyle"},linewidth:{valType:"number",min:0,dflt:1,editType:"ticks+layoutstyle"},showgrid:{valType:"boolean",editType:"ticks"},gridcolor:{valType:"color",dflt:i.lightLine,editType:"ticks"},gridwidth:{valType:"number",min:0,dflt:1,editType:"ticks"},zeroline:{valType:"boolean",editType:"ticks"},zerolinecolor:{valType:"color",dflt:i.defaultLine,editType:"ticks"},zerolinewidth:{valType:"number",dflt:1,editType:"ticks"},showdividers:{valType:"boolean",dflt:!0,editType:"ticks"},dividercolor:{valType:"color",dflt:i.defaultLine,editType:"ticks"},dividerwidth:{valType:"number",dflt:1,editType:"ticks"},anchor:{valType:"enumerated",values:["free",u.idRegex.x.toString(),u.idRegex.y.toString()],editType:"plot"},side:{valType:"enumerated",values:["top","bottom","left","right"],editType:"plot"},overlaying:{valType:"enumerated",values:["free",u.idRegex.x.toString(),u.idRegex.y.toString()],editType:"plot"},layer:{valType:"enumerated",values:["above traces","below traces"],dflt:"above traces",editType:"plot"},domain:{valType:"info_array",items:[{valType:"number",min:0,max:1,editType:"plot"},{valType:"number",min:0,max:1,editType:"plot"}],dflt:[0,1],editType:"plot"},position:{valType:"number",min:0,max:1,dflt:0,editType:"plot"},categoryorder:{valType:"enumerated",values:["trace","category ascending","category descending","array","total ascending","total descending","min ascending","min descending","max ascending","max descending","sum ascending","sum descending","mean ascending","mean descending","median ascending","median descending"],dflt:"trace",editType:"calc"},categoryarray:{valType:"data_array",editType:"calc"},uirevision:{valType:"any",editType:"none"},editType:"calc",_deprecated:{autotick:{valType:"boolean",editType:"ticks"},title:{valType:"string",editType:"ticks"},titlefont:r({editType:"ticks"})}}},function(e,t,n){var r;try{r={clone:n(2084),constant:n(609),each:n(1146),filter:n(1149),has:n(1156),isArray:n(81),isEmpty:n(2117),isFunction:n(281),isUndefined:n(1157),keys:n(189),map:n(1158),reduce:n(1160),size:n(2120),transform:n(2126),union:n(2127),values:n(1165)}}catch(e){}r||(r=window._),e.exports=r},function(e,t,n){"use strict";n.r(t);var r=n(925);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";n.r(t);var r=n(8),i=n.n(r),o=n(0),a=n(459),s=/^((children|dangerouslySetInnerHTML|key|ref|autoFocus|defaultValue|defaultChecked|innerHTML|suppressContentEditableWarning|suppressHydrationWarning|valueLink|accept|acceptCharset|accessKey|action|allow|allowUserMedia|allowPaymentRequest|allowFullScreen|allowTransparency|alt|async|autoComplete|autoPlay|capture|cellPadding|cellSpacing|challenge|charSet|checked|cite|classID|className|cols|colSpan|content|contentEditable|contextMenu|controls|controlsList|coords|crossOrigin|data|dateTime|decoding|default|defer|dir|disabled|disablePictureInPicture|download|draggable|encType|form|formAction|formEncType|formMethod|formNoValidate|formTarget|frameBorder|headers|height|hidden|high|href|hrefLang|htmlFor|httpEquiv|id|inputMode|integrity|is|keyParams|keyType|kind|label|lang|list|loading|loop|low|marginHeight|marginWidth|max|maxLength|media|mediaGroup|method|min|minLength|multiple|muted|name|nonce|noValidate|open|optimum|pattern|placeholder|playsInline|poster|preload|profile|radioGroup|readOnly|referrerPolicy|rel|required|reversed|role|rows|rowSpan|sandbox|scope|scoped|scrolling|seamless|selected|shape|size|sizes|slot|span|spellCheck|src|srcDoc|srcLang|srcSet|start|step|style|summary|tabIndex|target|title|type|useMap|value|width|wmode|wrap|about|datatype|inlist|prefix|property|resource|typeof|vocab|autoCapitalize|autoCorrect|autoSave|color|inert|itemProp|itemScope|itemType|itemID|itemRef|on|results|security|unselectable|accentHeight|accumulate|additive|alignmentBaseline|allowReorder|alphabetic|amplitude|arabicForm|ascent|attributeName|attributeType|autoReverse|azimuth|baseFrequency|baselineShift|baseProfile|bbox|begin|bias|by|calcMode|capHeight|clip|clipPathUnits|clipPath|clipRule|colorInterpolation|colorInterpolationFilters|colorProfile|colorRendering|contentScriptType|contentStyleType|cursor|cx|cy|d|decelerate|descent|diffuseConstant|direction|display|divisor|dominantBaseline|dur|dx|dy|edgeMode|elevation|enableBackground|end|exponent|externalResourcesRequired|fill|fillOpacity|fillRule|filter|filterRes|filterUnits|floodColor|floodOpacity|focusable|fontFamily|fontSize|fontSizeAdjust|fontStretch|fontStyle|fontVariant|fontWeight|format|from|fr|fx|fy|g1|g2|glyphName|glyphOrientationHorizontal|glyphOrientationVertical|glyphRef|gradientTransform|gradientUnits|hanging|horizAdvX|horizOriginX|ideographic|imageRendering|in|in2|intercept|k|k1|k2|k3|k4|kernelMatrix|kernelUnitLength|kerning|keyPoints|keySplines|keyTimes|lengthAdjust|letterSpacing|lightingColor|limitingConeAngle|local|markerEnd|markerMid|markerStart|markerHeight|markerUnits|markerWidth|mask|maskContentUnits|maskUnits|mathematical|mode|numOctaves|offset|opacity|operator|order|orient|orientation|origin|overflow|overlinePosition|overlineThickness|panose1|paintOrder|pathLength|patternContentUnits|patternTransform|patternUnits|pointerEvents|points|pointsAtX|pointsAtY|pointsAtZ|preserveAlpha|preserveAspectRatio|primitiveUnits|r|radius|refX|refY|renderingIntent|repeatCount|repeatDur|requiredExtensions|requiredFeatures|restart|result|rotate|rx|ry|scale|seed|shapeRendering|slope|spacing|specularConstant|specularExponent|speed|spreadMethod|startOffset|stdDeviation|stemh|stemv|stitchTiles|stopColor|stopOpacity|strikethroughPosition|strikethroughThickness|string|stroke|strokeDasharray|strokeDashoffset|strokeLinecap|strokeLinejoin|strokeMiterlimit|strokeOpacity|strokeWidth|surfaceScale|systemLanguage|tableValues|targetX|targetY|textAnchor|textDecoration|textRendering|textLength|to|transform|u1|u2|underlinePosition|underlineThickness|unicode|unicodeBidi|unicodeRange|unitsPerEm|vAlphabetic|vHanging|vIdeographic|vMathematical|values|vectorEffect|version|vertAdvY|vertOriginX|vertOriginY|viewBox|viewTarget|visibility|widths|wordSpacing|writingMode|x|xHeight|x1|x2|xChannelSelector|xlinkActuate|xlinkArcrole|xlinkHref|xlinkRole|xlinkShow|xlinkTitle|xlinkType|xmlBase|xmlns|xmlnsXlink|xmlLang|xmlSpace|y|y1|y2|yChannelSelector|z|zoomAndPan|for|class|autofocus)|(([Dd][Aa][Tt][Aa]|[Aa][Rr][Ii][Aa]|x)-.*))$/,l=Object(a.a)((function(e){return s.test(e)||111===e.charCodeAt(0)&&110===e.charCodeAt(1)&&e.charCodeAt(2)<91})),u=n(333),c=n(172),d=n(170),h=l,f=function(e){return"theme"!==e&&"innerRef"!==e},p=function(e){return"string"==typeof e&&e.charCodeAt(0)>96?h:f};function g(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),n.push.apply(n,r)}return n}function v(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?g(n,!0).forEach((function(t){i()(e,t,n[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):g(n).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))}))}return e}var m=function e(t,n){var r,i,a;void 0!==n&&(r=n.label,a=n.target,i=t.__emotion_forwardProp&&n.shouldForwardProp?function(e){return t.__emotion_forwardProp(e)&&n.shouldForwardProp(e)}:n.shouldForwardProp);var s=t.__emotion_real===t,l=s&&t.__emotion_base||t;"function"!=typeof i&&s&&(i=t.__emotion_forwardProp);var h=i||p(l),f=!h("as");return function(){var g=arguments,m=s&&void 0!==t.__emotion_styles?t.__emotion_styles.slice(0):[];if(void 0!==r&&m.push("label:"+r+";"),null==g[0]||void 0===g[0].raw)m.push.apply(m,g);else{0,m.push(g[0][0]);for(var y=g.length,b=1;b<y;b++)m.push(g[b],g[0][b])}var C=Object(u.withEmotionCache)((function(e,t,n){return Object(o.createElement)(u.ThemeContext.Consumer,null,(function(r){var s=f&&e.as||l,u="",g=[],v=e;if(null==e.theme){for(var y in v={},e)v[y]=e[y];v.theme=r}"string"==typeof e.className?u=Object(c.a)(t.registered,g,e.className):null!=e.className&&(u=e.className+" ");var b=Object(d.a)(m.concat(g),t.registered,v);Object(c.b)(t,b,"string"==typeof s);u+=t.key+"-"+b.name,void 0!==a&&(u+=" "+a);var C=f&&void 0===i?p(s):h,A={};for(var _ in e)f&&"as"===_||C(_)&&(A[_]=e[_]);return A.className=u,A.ref=n||e.innerRef,Object(o.createElement)(s,A)}))}));return C.displayName=void 0!==r?r:"Styled("+("string"==typeof l?l:l.displayName||l.name||"Component")+")",C.defaultProps=t.defaultProps,C.__emotion_real=C,C.__emotion_base=l,C.__emotion_styles=m,C.__emotion_forwardProp=i,Object.defineProperty(C,"toString",{value:function(){return"."+a}}),C.withComponent=function(t,r){return e(t,void 0!==r?v({},n||{},{},r):n).apply(void 0,m)},C}}.bind();["a","abbr","address","area","article","aside","audio","b","base","bdi","bdo","big","blockquote","body","br","button","canvas","caption","cite","code","col","colgroup","data","datalist","dd","del","details","dfn","dialog","div","dl","dt","em","embed","fieldset","figcaption","figure","footer","form","h1","h2","h3","h4","h5","h6","head","header","hgroup","hr","html","i","iframe","img","input","ins","kbd","keygen","label","legend","li","link","main","map","mark","marquee","menu","menuitem","meta","meter","nav","noscript","object","ol","optgroup","option","output","p","param","picture","pre","progress","q","rp","rt","ruby","s","samp","script","section","select","small","source","span","strong","style","sub","summary","sup","table","tbody","td","textarea","tfoot","th","thead","time","title","tr","track","u","ul","var","video","wbr","circle","clipPath","defs","ellipse","foreignObject","g","image","line","linearGradient","mask","path","pattern","polygon","polyline","radialGradient","rect","stop","svg","text","tspan"].forEach((function(e){m[e]=m(e)}));t.default=m},function(e,t,n){"use strict";n.d(t,"a",(function(){return i}));var r=n(447);function i(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&Object(r.a)(e,t)}},function(e,t,n){"use strict";n.d(t,"a",(function(){return o}));var r=n(140),i=n(226);function o(e,t){return!t||"object"!==Object(r.a)(t)&&"function"!=typeof t?Object(i.a)(e):t}},function(e,t,n){"use strict";function r(e){return(r="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}n.d(t,"a",(function(){return r}))},function(e,t,n){"use strict";n.d(t,"a",(function(){return i}));var r=n(0);function i(e,t){return r.isValidElement(e)&&-1!==t.indexOf(e.type.muiName)}},function(e,t,n){"use strict";function r(e){var t=e.props,n=e.states,r=e.muiFormControl;return n.reduce((function(e,n){return e[n]=t[n],r&&void 0===t[n]&&(e[n]=r[n]),e}),{})}n.d(t,"a",(function(){return r}))},function(e,t,n){"use strict";var r=n(0),i=r.createContext({});t.a=i},function(e,t,n){"use strict";n.d(t,"a",(function(){return l})),n.d(t,"d",(function(){return u})),n.d(t,"b",(function(){return c})),n.d(t,"c",(function(){return d}));var r=n(14),i=n(16),o=n(18),a=n(0),s=n(22);class l extends o.Widget{static create(e){return new class extends l{render(){return e}}}onUpdateRequest(e){this.renderPromise=this.renderDOM()}onAfterAttach(e){r.c.sendMessage(this,o.Widget.Msg.UpdateRequest)}onBeforeDetach(e){s.unmountComponentAtNode(this.node)}renderDOM(){return new Promise(e=>{const t=this.render();(Array.isArray(t)||t)&&s.render(t,this.node,e)})}}class u extends l{constructor(e){super(),this._modelChanged=new i.Signal(this),this.model=null!=e?e:null}get modelChanged(){return this._modelChanged}set model(e){this._model!==e&&(this._model&&this._model.stateChanged.disconnect(this.update,this),this._model=e,e&&e.stateChanged.connect(this.update,this),this.update(),this._modelChanged.emit(void 0))}get model(){return this._model}dispose(){this.isDisposed||(this._model=null,super.dispose())}}class c extends a.Component{constructor(e){super(e),this.slot=(e,t)=>{this.props.shouldUpdate&&!this.props.shouldUpdate(e,t)||this.setState({value:[e,t]})},this.state={value:[this.props.initialSender,this.props.initialArgs]}}componentDidMount(){this.props.signal.connect(this.slot)}componentWillUnmount(){this.props.signal.disconnect(this.slot)}render(){return this.props.children(...this.state.value)}}class d{constructor(){this.stateChanged=new i.Signal(this),this._isDisposed=!1}get isDisposed(){return this._isDisposed}dispose(){this.isDisposed||(this._isDisposed=!0,i.Signal.clearData(this))}}},function(e,t){var n,r,i=e.exports={};function o(){throw new Error("setTimeout has not been defined")}function a(){throw new Error("clearTimeout has not been defined")}function s(e){if(n===setTimeout)return setTimeout(e,0);if((n===o||!n)&&setTimeout)return n=setTimeout,setTimeout(e,0);try{return n(e,0)}catch(t){try{return n.call(null,e,0)}catch(t){return n.call(this,e,0)}}}!function(){try{n="function"==typeof setTimeout?setTimeout:o}catch(e){n=o}try{r="function"==typeof clearTimeout?clearTimeout:a}catch(e){r=a}}();var l,u=[],c=!1,d=-1;function h(){c&&l&&(c=!1,l.length?u=l.concat(u):d=-1,u.length&&f())}function f(){if(!c){var e=s(h);c=!0;for(var t=u.length;t;){for(l=u,u=[];++d<t;)l&&l[d].run();d=-1,t=u.length}l=null,c=!1,function(e){if(r===clearTimeout)return clearTimeout(e);if((r===a||!r)&&clearTimeout)return r=clearTimeout,clearTimeout(e);try{r(e)}catch(t){try{return r.call(null,e)}catch(t){return r.call(this,e)}}}(e)}}function p(e,t){this.fun=e,this.array=t}function g(){}i.nextTick=function(e){var t=new Array(arguments.length-1);if(arguments.length>1)for(var n=1;n<arguments.length;n++)t[n-1]=arguments[n];u.push(new p(e,t)),1!==u.length||c||s(f)},p.prototype.run=function(){this.fun.apply(null,this.array)},i.title="browser",i.browser=!0,i.env={},i.argv=[],i.version="",i.versions={},i.on=g,i.addListener=g,i.once=g,i.off=g,i.removeListener=g,i.removeAllListeners=g,i.emit=g,i.prependListener=g,i.prependOnceListener=g,i.listeners=function(e){return[]},i.binding=function(e){throw new Error("process.binding is not supported")},i.cwd=function(){return"/"},i.chdir=function(e){throw new Error("process.chdir is not supported")},i.umask=function(){return 0}},function(e,t,n){var r=n(850),i="object"==typeof self&&self&&self.Object===Object&&self,o=r||i||Function("return this")();e.exports=o},function(e,t,n){"use strict";var r,i={},o=function(){return void 0===r&&(r=Boolean(window&&document&&document.all&&!window.atob)),r},a=function(){var e={};return function(t){if(void 0===e[t]){var n=document.querySelector(t);if(window.HTMLIFrameElement&&n instanceof window.HTMLIFrameElement)try{n=n.contentDocument.head}catch(e){n=null}e[t]=n}return e[t]}}();function s(e,t){for(var n=[],r={},i=0;i<e.length;i++){var o=e[i],a=t.base?o[0]+t.base:o[0],s={css:o[1],media:o[2],sourceMap:o[3]};r[a]?r[a].parts.push(s):n.push(r[a]={id:a,parts:[s]})}return n}function l(e,t){for(var n=0;n<e.length;n++){var r=e[n],o=i[r.id],a=0;if(o){for(o.refs++;a<o.parts.length;a++)o.parts[a](r.parts[a]);for(;a<r.parts.length;a++)o.parts.push(v(r.parts[a],t))}else{for(var s=[];a<r.parts.length;a++)s.push(v(r.parts[a],t));i[r.id]={id:r.id,refs:1,parts:s}}}}function u(e){var t=document.createElement("style");if(void 0===e.attributes.nonce){var r=n.nc;r&&(e.attributes.nonce=r)}if(Object.keys(e.attributes).forEach((function(n){t.setAttribute(n,e.attributes[n])})),"function"==typeof e.insert)e.insert(t);else{var i=a(e.insert||"head");if(!i)throw new Error("Couldn't find a style target. This probably means that the value for the 'insert' parameter is invalid.");i.appendChild(t)}return t}var c,d=(c=[],function(e,t){return c[e]=t,c.filter(Boolean).join("\n")});function h(e,t,n,r){var i=n?"":r.css;if(e.styleSheet)e.styleSheet.cssText=d(t,i);else{var o=document.createTextNode(i),a=e.childNodes;a[t]&&e.removeChild(a[t]),a.length?e.insertBefore(o,a[t]):e.appendChild(o)}}function f(e,t,n){var r=n.css,i=n.media,o=n.sourceMap;if(i&&e.setAttribute("media",i),o&&btoa&&(r+="\n/*# sourceMappingURL=data:application/json;base64,".concat(btoa(unescape(encodeURIComponent(JSON.stringify(o))))," */")),e.styleSheet)e.styleSheet.cssText=r;else{for(;e.firstChild;)e.removeChild(e.firstChild);e.appendChild(document.createTextNode(r))}}var p=null,g=0;function v(e,t){var n,r,i;if(t.singleton){var o=g++;n=p||(p=u(t)),r=h.bind(null,n,o,!1),i=h.bind(null,n,o,!0)}else n=u(t),r=f.bind(null,n,t),i=function(){!function(e){if(null===e.parentNode)return!1;e.parentNode.removeChild(e)}(n)};return r(e),function(t){if(t){if(t.css===e.css&&t.media===e.media&&t.sourceMap===e.sourceMap)return;r(e=t)}else i()}}e.exports=function(e,t){(t=t||{}).attributes="object"==typeof t.attributes?t.attributes:{},t.singleton||"boolean"==typeof t.singleton||(t.singleton=o());var n=s(e,t);return l(n,t),function(e){for(var r=[],o=0;o<n.length;o++){var a=n[o],u=i[a.id];u&&(u.refs--,r.push(u))}e&&l(s(e,t),t);for(var c=0;c<r.length;c++){var d=r[c];if(0===d.refs){for(var h=0;h<d.parts.length;h++)d.parts[h]();delete i[d.id]}}}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.CLASSES=void 0,function(e){e.FLEXLAYOUT__BORDER="flexlayout__border",e.FLEXLAYOUT__BORDER_="flexlayout__border_",e.FLEXLAYOUT__BORDER_BUTTON="flexlayout__border_button",e.FLEXLAYOUT__BORDER_BUTTON_="flexlayout__border_button_",e.FLEXLAYOUT__BORDER_BUTTON_CONTENT="flexlayout__border_button_content",e.FLEXLAYOUT__BORDER_BUTTON_LEADING="flexlayout__border_button_leading",e.FLEXLAYOUT__BORDER_BUTTON_TRAILING="flexlayout__border_button_trailing",e.FLEXLAYOUT__BORDER_BUTTON__SELECTED="flexlayout__border_button--selected",e.FLEXLAYOUT__BORDER_BUTTON__UNSELECTED="flexlayout__border_button--unselected",e.FLEXLAYOUT__BORDER_INNER="flexlayout__border_inner",e.FLEXLAYOUT__BORDER_INNER_="flexlayout__border_inner_",e.FLEXLAYOUT__BORDER_INNER_TAB_CONTAINER="flexlayout__border_inner_tab_container",e.FLEXLAYOUT__BORDER_INNER_TAB_CONTAINER_="flexlayout__border_inner_tab_container_",e.FLEXLAYOUT__BORDER_SIZER="flexlayout__border_sizer",e.FLEXLAYOUT__BORDER_TOOLBAR="flexlayout__border_toolbar",e.FLEXLAYOUT__BORDER_TOOLBAR_="flexlayout__border_toolbar_",e.FLEXLAYOUT__BORDER_TOOLBAR_BUTTON="flexlayout__border_toolbar_button",e.FLEXLAYOUT__BORDER_TOOLBAR_BUTTON_FLOAT="flexlayout__border_toolbar_button-float",e.FLEXLAYOUT__DRAG_RECT="flexlayout__drag_rect",e.FLEXLAYOUT__EDGE_RECT="flexlayout__edge_rect",e.FLEXLAYOUT__ERROR_BOUNDARY_CONTAINER="flexlayout__error_boundary_container",e.FLEXLAYOUT__ERROR_BOUNDARY_CONTENT="flexlayout__error_boundary_content",e.FLEXLAYOUT__FLOATING_WINDOW_CONTENT="flexlayout__floating_window_content",e.FLEXLAYOUT__FLOATING_WINDOW_TAB="flexlayout__floating_window_tab",e.FLEXLAYOUT__LAYOUT="flexlayout__layout",e.FLEXLAYOUT__OUTLINE_RECT="flexlayout__outline_rect",e.FLEXLAYOUT__SPLITTER="flexlayout__splitter",e.FLEXLAYOUT__SPLITTER_="flexlayout__splitter_",e.FLEXLAYOUT__SPLITTER_BORDER="flexlayout__splitter_border",e.FLEXLAYOUT__SPLITTER_DRAG="flexlayout__splitter_drag",e.FLEXLAYOUT__TAB="flexlayout__tab",e.FLEXLAYOUT__TABSET="flexlayout__tabset",e.FLEXLAYOUT__TABSET_HEADER="flexlayout__tabset_header",e.FLEXLAYOUT__TABSET_HEADER_SIZER="flexlayout__tabset_header_sizer",e.FLEXLAYOUT__TABSET_HEADER_CONTENT="flexlayout__tabset_header_content",e.FLEXLAYOUT__TABSET_MAXIMIZED="flexlayout__tabset-maximized",e.FLEXLAYOUT__TABSET_SELECTED="flexlayout__tabset-selected",e.FLEXLAYOUT__TABSET_SIZER="flexlayout__tabset_sizer",e.FLEXLAYOUT__TABSET_TABBAR_INNER="flexlayout__tabset_tabbar_inner",e.FLEXLAYOUT__TABSET_TABBAR_INNER_="flexlayout__tabset_tabbar_inner_",e.FLEXLAYOUT__TABSET_TABBAR_INNER_TAB_CONTAINER="flexlayout__tabset_tabbar_inner_tab_container",e.FLEXLAYOUT__TABSET_TABBAR_INNER_TAB_CONTAINER_="flexlayout__tabset_tabbar_inner_tab_container_",e.FLEXLAYOUT__TABSET_TABBAR_OUTER="flexlayout__tabset_tabbar_outer",e.FLEXLAYOUT__TABSET_TABBAR_OUTER_="flexlayout__tabset_tabbar_outer_",e.FLEXLAYOUT__TAB_BORDER="flexlayout__tab_border",e.FLEXLAYOUT__TAB_BORDER_="flexlayout__tab_border_",e.FLEXLAYOUT__TAB_BUTTON="flexlayout__tab_button",e.FLEXLAYOUT__TAB_BUTTON_CONTENT="flexlayout__tab_button_content",e.FLEXLAYOUT__TAB_BUTTON_LEADING="flexlayout__tab_button_leading",e.FLEXLAYOUT__TAB_BUTTON_OVERFLOW="flexlayout__tab_button_overflow",e.FLEXLAYOUT__TAB_BUTTON_TEXTBOX="flexlayout__tab_button_textbox",e.FLEXLAYOUT__TAB_BUTTON_TRAILING="flexlayout__tab_button_trailing",e.FLEXLAYOUT__TAB_FLOATING="flexlayout__tab_floating",e.FLEXLAYOUT__TAB_FLOATING_INNER="flexlayout__tab_floating_inner",e.FLEXLAYOUT__TAB_TOOLBAR="flexlayout__tab_toolbar",e.FLEXLAYOUT__TAB_TOOLBAR_BUTTON="flexlayout__tab_toolbar_button",e.FLEXLAYOUT__TAB_TOOLBAR_BUTTON_="flexlayout__tab_toolbar_button-",e.FLEXLAYOUT__TAB_TOOLBAR_BUTTON_FLOAT="flexlayout__tab_toolbar_button-float"}(t.CLASSES||(t.CLASSES={}))},function(e,t,n){"use strict";var r=n(39),i=n(123),o=n(42),a=n(10),s=n(41),l=n(251).isValid;var u=["showscale","autocolorscale","colorscale","reversescale","colorbar"],c=["min","max","mid","auto"];function d(e){var t,n,r,i=e._colorAx,o=i||e,a={};for(n=0;n<u.length;n++)a[r=u[n]]=o[r];if(i)for(t="c",n=0;n<c.length;n++)a[r=c[n]]=o["c"+r];else{var s;for(n=0;n<c.length;n++)((s="c"+(r=c[n]))in o||(s="z"+r)in o)&&(a[r]=o[s]);t=s.charAt(0)}return a._sync=function(e,n){var r=-1!==c.indexOf(e)?t+e:e;o[r]=o["_"+r]=n},a}function h(e){for(var t=d(e),n=t.min,r=t.max,i=t.reversescale?f(t.colorscale):t.colorscale,o=i.length,a=new Array(o),s=new Array(o),l=0;l<o;l++){var u=i[l];a[l]=n+u[0]*(r-n),s[l]=u[1]}return{domain:a,range:s}}function f(e){for(var t=e.length,n=new Array(t),r=t-1,i=0;r>=0;r--,i++){var o=e[r];n[i]=[1-o[0],o[1]]}return n}function p(e,t){t=t||{};for(var n=e.domain,a=e.range,l=a.length,u=new Array(l),c=0;c<l;c++){var d=i(a[c]).toRgb();u[c]=[d.r,d.g,d.b,d.a]}var h,f=r.scale.linear().domain(n).range(u).clamp(!0),p=t.noNumericCheck,v=t.returnArray;return(h=p&&v?f:p?function(e){return g(f(e))}:v?function(e){return o(e)?f(e):i(e).isValid()?e:s.defaultLine}:function(e){return o(e)?g(f(e)):i(e).isValid()?e:s.defaultLine}).domain=f.domain,h.range=function(){return a},h}function g(e){var t={r:e[0],g:e[1],b:e[2],a:e[3]};return i(t).toRgbString()}e.exports={hasColorscale:function(e,t,n){var r=t?a.nestedProperty(e,t).get()||{}:e,i=r[n||"color"],s=!1;if(a.isArrayOrTypedArray(i))for(var u=0;u<i.length;u++)if(o(i[u])){s=!0;break}return a.isPlainObject(r)&&(s||!0===r.showscale||o(r.cmin)&&o(r.cmax)||l(r.colorscale)||a.isPlainObject(r.colorbar))},extractOpts:d,extractScale:h,flipScale:f,makeColorScaleFunc:p,makeColorScaleFuncFromTrace:function(e,t){return p(h(e),t)}}},function(e,t,n){"use strict";var r=n(257),i=n(197),o=n(396),a=n(1043);function s(e,t){for(var n=new Array(e.length-1),r=1;r<e.length;++r)for(var i=n[r-1]=new Array(e.length-1),o=0,a=0;o<e.length;++o)o!==t&&(i[a++]=e[r][o]);return n}function l(e){if(1===e.length)return e[0];if(2===e.length)return["sum(",e[0],",",e[1],")"].join("");var t=e.length>>1;return["sum(",l(e.slice(0,t)),",",l(e.slice(t)),")"].join("")}function u(e){if(2===e.length)return[["sum(prod(",e[0][0],",",e[1][1],"),prod(-",e[0][1],",",e[1][0],"))"].join("")];for(var t=[],n=0;n<e.length;++n)t.push(["scale(",l(u(s(e,n))),",",(r=n,1&r?"-":""),e[0][n],")"].join(""));return t;var r}function c(e){for(var t=[],n=[],c=function(e){for(var t=new Array(e),n=0;n<e;++n){t[n]=new Array(e);for(var r=0;r<e;++r)t[n][r]=["m",r,"[",e-n-1,"]"].join("")}return t}(e),d=[],h=0;h<e;++h)0==(1&h)?t.push.apply(t,u(s(c,h))):n.push.apply(n,u(s(c,h))),d.push("m"+h);var f=l(t),p=l(n),g="orientation"+e+"Exact",v=["function ",g,"(",d.join(),"){var p=",f,",n=",p,",d=sub(p,n);return d[d.length-1];};return ",g].join("");return new Function("sum","prod","scale","sub",v)(i,r,o,a)}var d=c(3),h=c(4),f=[function(){return 0},function(){return 0},function(e,t){return t[0]-e[0]},function(e,t,n){var r,i=(e[1]-n[1])*(t[0]-n[0]),o=(e[0]-n[0])*(t[1]-n[1]),a=i-o;if(i>0){if(o<=0)return a;r=i+o}else{if(!(i<0))return a;if(o>=0)return a;r=-(i+o)}var s=33306690738754716e-32*r;return a>=s||a<=-s?a:d(e,t,n)},function(e,t,n,r){var i=e[0]-r[0],o=t[0]-r[0],a=n[0]-r[0],s=e[1]-r[1],l=t[1]-r[1],u=n[1]-r[1],c=e[2]-r[2],d=t[2]-r[2],f=n[2]-r[2],p=o*u,g=a*l,v=a*s,m=i*u,y=i*l,b=o*s,C=c*(p-g)+d*(v-m)+f*(y-b),A=7771561172376103e-31*((Math.abs(p)+Math.abs(g))*Math.abs(c)+(Math.abs(v)+Math.abs(m))*Math.abs(d)+(Math.abs(y)+Math.abs(b))*Math.abs(f));return C>A||-C>A?C:h(e,t,n,r)}];function p(e){var t=f[e.length];return t||(t=f[e.length]=c(e.length)),t.apply(void 0,e)}!function(){for(;f.length<=5;)f.push(c(f.length));for(var t=[],n=["slow"],r=0;r<=5;++r)t.push("a"+r),n.push("o"+r);var i=["function getOrientation(",t.join(),"){switch(arguments.length){case 0:case 1:return 0;"];for(r=2;r<=5;++r)i.push("case ",r,":return o",r,"(",t.slice(0,r).join(),");");i.push("}var s=new Array(arguments.length);for(var i=0;i<arguments.length;++i){s[i]=arguments[i]};return slow(s);}return getOrientation"),n.push(i.join(""));var o=Function.apply(void 0,n);for(e.exports=o.apply(void 0,[p].concat(f)),r=0;r<=5;++r)e.exports[r]=f[r]}()},function(e,t,n){"use strict";var r=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),i=this&&this.__exportStar||function(e,t){for(var n in e)"default"===n||Object.prototype.hasOwnProperty.call(t,n)||r(t,e,n)};Object.defineProperty(t,"__esModule",{value:!0}),i(n(402),t),i(n(1098),t),i(n(591),t),i(n(1099),t),i(n(2044),t)},function(e,t,n){"use strict";var r=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),i=this&&this.__exportStar||function(e,t){for(var n in e)"default"===n||Object.prototype.hasOwnProperty.call(t,n)||r(t,e,n)};Object.defineProperty(t,"__esModule",{value:!0}),i(n(2050),t),i(n(2051),t),i(n(596),t),i(n(597),t),i(n(1117),t),i(n(599),t),i(n(1118),t),i(n(2053),t),i(n(598),t),i(n(1119),t),i(n(2054),t),i(n(405),t),i(n(1120),t),i(n(406),t),i(n(2055),t),i(n(2056),t),i(n(1122),t),i(n(1121),t),i(n(2057),t)},function(e,t,n){var r;try{r=n(2082)}catch(e){}r||(r=window.graphlib),e.exports=r},function(e,t,n){"use strict";n.r(t);var r=n(929);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";n.d(t,"b",(function(){return o})),n.d(t,"a",(function(){return a}));var r=n(1),i=n(5),o=["xs","sm","md","lg","xl"];function a(e){var t=e.values,n=void 0===t?{xs:0,sm:600,md:960,lg:1280,xl:1920}:t,a=e.unit,s=void 0===a?"px":a,l=e.step,u=void 0===l?5:l,c=Object(i.a)(e,["values","unit","step"]);function d(e){var t="number"==typeof n[e]?n[e]:e;return"@media (min-width:".concat(t).concat(s,")")}function h(e,t){var r=o.indexOf(t);return r===o.length-1?d(e):"@media (min-width:".concat("number"==typeof n[e]?n[e]:e).concat(s,") and ")+"(max-width:".concat((-1!==r&&"number"==typeof n[o[r+1]]?n[o[r+1]]:t)-u/100).concat(s,")")}return Object(r.a)({keys:o,values:n,up:d,down:function(e){var t=o.indexOf(e)+1,r=n[o[t]];return t===o.length?d("xs"):"@media (max-width:".concat(("number"==typeof r&&t>0?r:e)-u/100).concat(s,")")},between:h,only:function(e){return h(e,e)},width:function(e){return n[e]}},c)}},,,function(e,t){e.exports=function(e){return null!=e&&"object"==typeof e}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(933),i=function(){function e(){}return e.addNode=function(t,n,i,o,a){return new r.default(e.ADD_NODE,{json:t,toNode:n,location:i.getName(),index:o,select:a})},e.moveNode=function(t,n,i,o,a){return new r.default(e.MOVE_NODE,{fromNode:t,toNode:n,location:i.getName(),index:o,select:a})},e.deleteTab=function(t){return new r.default(e.DELETE_TAB,{node:t})},e.renameTab=function(t,n){return new r.default(e.RENAME_TAB,{node:t,text:n})},e.selectTab=function(t){return new r.default(e.SELECT_TAB,{tabNode:t})},e.setActiveTabset=function(t){return new r.default(e.SET_ACTIVE_TABSET,{tabsetNode:t})},e.adjustSplit=function(t){var n=t.node1Id,i=t.node2Id;return new r.default(e.ADJUST_SPLIT,{node1:n,weight1:t.weight1,pixelWidth1:t.pixelWidth1,node2:i,weight2:t.weight2,pixelWidth2:t.pixelWidth2})},e.adjustBorderSplit=function(t,n){return new r.default(e.ADJUST_BORDER_SPLIT,{node:t,pos:n})},e.maximizeToggle=function(t){return new r.default(e.MAXIMIZE_TOGGLE,{node:t})},e.updateModelAttributes=function(t){return new r.default(e.UPDATE_MODEL_ATTRIBUTES,{json:t})},e.updateNodeAttributes=function(t,n){return new r.default(e.UPDATE_NODE_ATTRIBUTES,{node:t,json:n})},e.floatTab=function(t){return new r.default(e.FLOAT_TAB,{node:t})},e.unFloatTab=function(t){return new r.default(e.UNFLOAT_TAB,{node:t})},e.ADD_NODE="FlexLayout_AddNode",e.MOVE_NODE="FlexLayout_MoveNode",e.DELETE_TAB="FlexLayout_DeleteTab",e.RENAME_TAB="FlexLayout_RenameTab",e.SELECT_TAB="FlexLayout_SelectTab",e.SET_ACTIVE_TABSET="FlexLayout_SetActiveTabset",e.ADJUST_SPLIT="FlexLayout_AdjustSplit",e.ADJUST_BORDER_SPLIT="FlexLayout_AdjustBorderSplit",e.MAXIMIZE_TOGGLE="FlexLayout_MaximizeToggle",e.UPDATE_MODEL_ATTRIBUTES="FlexLayout_UpdateModelAttributes",e.UPDATE_NODE_ATTRIBUTES="FlexLayout_UpdateNodeAttributes",e.FLOAT_TAB="FlexLayout_FloatTab",e.UNFLOAT_TAB="FlexLayout_UnFloatTab",e}();t.default=i},function(e,t,n){"use strict";var r=n(524);e.exports={type:{valType:"enumerated",values:[],dflt:"scatter",editType:"calc+clearAxisTypes",_noTemplating:!0},visible:{valType:"enumerated",values:[!0,!1,"legendonly"],dflt:!0,editType:"calc"},showlegend:{valType:"boolean",dflt:!0,editType:"style"},legendgroup:{valType:"string",dflt:"",editType:"style"},opacity:{valType:"number",min:0,max:1,dflt:1,editType:"style"},name:{valType:"string",editType:"style"},uid:{valType:"string",editType:"plot",anim:!0},ids:{valType:"data_array",editType:"calc",anim:!0},customdata:{valType:"data_array",editType:"calc"},meta:{valType:"any",arrayOk:!0,editType:"plot"},selectedpoints:{valType:"any",editType:"calc"},hoverinfo:{valType:"flaglist",flags:["x","y","z","text","name"],extras:["all","none","skip"],arrayOk:!0,dflt:"all",editType:"none"},hoverlabel:r.hoverlabel,stream:{token:{valType:"string",noBlank:!0,strict:!0,editType:"calc"},maxpoints:{valType:"number",min:0,max:1e4,dflt:500,editType:"calc"},editType:"calc"},transforms:{_isLinkedToArray:"transform",editType:"calc"},uirevision:{valType:"any",editType:"none"}}},function(e,t,n){"use strict";var r=n(541),i=n(294).counter,o=n(251).scales;Object.keys(o);function a(e){return"`"+e+"`"}e.exports=function(e,t){e=e||"";var n,s=(t=t||{}).cLetter||"c",l=("onlyIfNumerical"in t?t.onlyIfNumerical:Boolean(e),"noScale"in t?t.noScale:"marker.line"===e),u="showScaleDflt"in t?t.showScaleDflt:"z"===s,c="string"==typeof t.colorscaleDflt?o[t.colorscaleDflt]:null,d=t.editTypeOverride||"",h=e?e+".":"";"colorAttr"in t?(n=t.colorAttr,t.colorAttr):a(h+(n={z:"z",c:"color"}[s]));var f=s+"auto",p=s+"min",g=s+"max",v=s+"mid",m=(a(h+f),a(h+p),a(h+g),{});m[p]=m[g]=void 0;var y={};y[f]=!1;var b={};return"color"===n&&(b.color={valType:"color",arrayOk:!0,editType:d||"style"},t.anim&&(b.color.anim=!0)),b[f]={valType:"boolean",dflt:!0,editType:"calc",impliedEdits:m},b[p]={valType:"number",dflt:null,editType:d||"plot",impliedEdits:y},b[g]={valType:"number",dflt:null,editType:d||"plot",impliedEdits:y},b[v]={valType:"number",dflt:null,editType:"calc",impliedEdits:m},b.colorscale={valType:"colorscale",editType:"calc",dflt:c,impliedEdits:{autocolorscale:!1}},b.autocolorscale={valType:"boolean",dflt:!1!==t.autoColorDflt,editType:"calc",impliedEdits:{colorscale:void 0}},b.reversescale={valType:"boolean",dflt:!1,editType:"plot"},l||(b.showscale={valType:"boolean",dflt:u,editType:"calc"},b.colorbar=r),t.noColorAxis||(b.coloraxis={valType:"subplotid",regex:i("coloraxis"),dflt:null,editType:"calc"}),b}},function(e,t,n){"use strict";var r=n(10);t.getSubplot=function(e){return e.subplot||e.xaxis+e.yaxis||e.geo},t.isTraceInSubplots=function(e,n){if("splom"===e.type){for(var r=e.xaxes||[],i=e.yaxes||[],o=0;o<r.length;o++)for(var a=0;a<i.length;a++)if(-1!==n.indexOf(r[o]+i[a]))return!0;return!1}return-1!==n.indexOf(t.getSubplot(e))},t.flat=function(e,t){for(var n=new Array(e.length),r=0;r<e.length;r++)n[r]=t;return n},t.p2c=function(e,t){for(var n=new Array(e.length),r=0;r<e.length;r++)n[r]=e[r].p2c(t);return n},t.getDistanceFunction=function(e,n,r,i){return"closest"===e?i||t.quadrature(n,r):"x"===e.charAt(0)?n:r},t.getClosest=function(e,t,n){if(!1!==n.index)n.index>=0&&n.index<e.length?n.distance=0:n.index=!1;else for(var r=0;r<e.length;r++){var i=t(e[r]);i<=n.distance&&(n.index=r,n.distance=i)}return n},t.inbox=function(e,t,n){return e*t<0||0===e?n:1/0},t.quadrature=function(e,t){return function(n){var r=e(n),i=t(n);return Math.sqrt(r*r+i*i)}},t.makeEventData=function(e,n,r){var i="index"in e?e.index:e.pointNumber,o={data:n._input,fullData:n,curveNumber:n.index,pointNumber:i};if(n._indexToPoints){var a=n._indexToPoints[i];1===a.length?o.pointIndex=a[0]:o.pointIndices=a}else o.pointIndex=i;return n._module.eventData?o=n._module.eventData(o,e,n,r,i):("xVal"in e?o.x=e.xVal:"x"in e&&(o.x=e.x),"yVal"in e?o.y=e.yVal:"y"in e&&(o.y=e.y),e.xa&&(o.xaxis=e.xa),e.ya&&(o.yaxis=e.ya),void 0!==e.zLabelVal&&(o.z=e.zLabelVal)),t.appendArrayPointValue(o,n,i),o},t.appendArrayPointValue=function(e,t,n){var i=t._arrayAttrs;if(i)for(var s=0;s<i.length;s++){var l=i[s],u=o(l);if(void 0===e[u]){var c=a(r.nestedProperty(t,l).get(),n);void 0!==c&&(e[u]=c)}}},t.appendArrayMultiPointValues=function(e,t,n){var i=t._arrayAttrs;if(i)for(var s=0;s<i.length;s++){var l=i[s],u=o(l);if(void 0===e[u]){for(var c=r.nestedProperty(t,l).get(),d=new Array(n.length),h=0;h<n.length;h++)d[h]=a(c,n[h]);e[u]=d}}};var i={ids:"id",locations:"location",labels:"label",values:"value","marker.colors":"color",parents:"parent"};function o(e){return i[e]||e}function a(e,t){return Array.isArray(t)?Array.isArray(e)&&Array.isArray(e[t[0]])?e[t[0]][t[1]]:void 0:e[t]}var s={x:!0,y:!0},l={"x unified":!0,"y unified":!0};t.isUnifiedHover=function(e){return"string"==typeof e&&!!l[e]},t.isXYhover=function(e){return"string"==typeof e&&!!s[e]}},function(e,t,n){"use strict";var r=n(992),i=n(980),o=n(387),a=n(10).removeElement,s=n(95),l=e.exports={};l.align=n(1687),l.getCursor=n(1688);var u=n(1689);function c(){var e=document.createElement("div");e.className="dragcover";var t=e.style;return t.position="fixed",t.left=0,t.right=0,t.top=0,t.bottom=0,t.zIndex=999999999,t.background="none",document.body.appendChild(e),e}function d(e){return r(e.changedTouches?e.changedTouches[0]:e,document.body)}l.unhover=u.wrapped,l.unhoverRaw=u.raw,l.init=function(e){var t,n,r,u,h,f,p,g,v=e.gd,m=1,y=v._context.doubleClickDelay,b=e.element;v._mouseDownTime||(v._mouseDownTime=0),b.style.pointerEvents="all",b.onmousedown=A,o?(b._ontouchstart&&b.removeEventListener("touchstart",b._ontouchstart),b._ontouchstart=A,b.addEventListener("touchstart",A,{passive:!1})):b.ontouchstart=A;var C=e.clampFn||function(e,t,n){return Math.abs(e)<n&&(e=0),Math.abs(t)<n&&(t=0),[e,t]};function A(o){v._dragged=!1,v._dragging=!0;var a=d(o);t=a[0],n=a[1],p=o.target,f=o,g=2===o.buttons||o.ctrlKey,void 0===o.clientX&&void 0===o.clientY&&(o.clientX=t,o.clientY=n),(r=(new Date).getTime())-v._mouseDownTime<y?m+=1:(m=1,v._mouseDownTime=r),e.prepFn&&e.prepFn(o,t,n),i&&!g?(h=c()).style.cursor=window.getComputedStyle(b).cursor:i||(h=document,u=window.getComputedStyle(document.documentElement).cursor,document.documentElement.style.cursor=window.getComputedStyle(b).cursor),document.addEventListener("mouseup",I),document.addEventListener("touchend",I),!1!==e.dragmode&&(o.preventDefault(),document.addEventListener("mousemove",_),document.addEventListener("touchmove",_,{passive:!1}))}function _(r){r.preventDefault();var i=d(r),o=e.minDrag||s.MINDRAG,a=C(i[0]-t,i[1]-n,o),u=a[0],c=a[1];(u||c)&&(v._dragged=!0,l.unhover(v)),v._dragged&&e.moveFn&&!g&&(v._dragdata={element:b,dx:u,dy:c},e.moveFn(u,c))}function I(t){if(delete v._dragdata,!1!==e.dragmode&&(t.preventDefault(),document.removeEventListener("mousemove",_),document.removeEventListener("touchmove",_)),document.removeEventListener("mouseup",I),document.removeEventListener("touchend",I),i?a(h):u&&(h.documentElement.style.cursor=u,u=null),v._dragging){if(v._dragging=!1,(new Date).getTime()-v._mouseDownTime>y&&(m=Math.max(m-1,1)),v._dragged)e.doneFn&&e.doneFn();else if(e.clickFn&&e.clickFn(m,f),!g){var n;try{n=new MouseEvent("click",t)}catch(e){var r=d(t);(n=document.createEvent("MouseEvents")).initMouseEvent("click",t.bubbles,t.cancelable,t.view,t.detail,t.screenX,t.screenY,r[0],r[1],t.ctrlKey,t.altKey,t.shiftKey,t.metaKey,t.button,t.relatedTarget)}p.dispatchEvent(n)}v._dragging=!1,v._dragged=!1}else v._dragged=!1}},l.coverSlip=c},function(e,t,n){"use strict";n.r(t);var r=n(633);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";n.r(t);var r=n(918);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";n.d(t,"a",(function(){return p}));var r=n(0),i=n(22),o=!0,a=!1,s=null,l={text:!0,search:!0,url:!0,tel:!0,email:!0,password:!0,number:!0,date:!0,month:!0,week:!0,time:!0,datetime:!0,"datetime-local":!0};function u(e){e.metaKey||e.altKey||e.ctrlKey||(o=!0)}function c(){o=!1}function d(){"hidden"===this.visibilityState&&a&&(o=!0)}function h(e){var t,n,r,i=e.target;try{return i.matches(":focus-visible")}catch(e){}return o||(n=(t=i).type,!("INPUT"!==(r=t.tagName)||!l[n]||t.readOnly)||"TEXTAREA"===r&&!t.readOnly||!!t.isContentEditable)}function f(){a=!0,window.clearTimeout(s),s=window.setTimeout((function(){a=!1}),100)}function p(){return{isFocusVisible:h,onBlurVisible:f,ref:r.useCallback((function(e){var t,n=i.findDOMNode(e);null!=n&&((t=n.ownerDocument).addEventListener("keydown",u,!0),t.addEventListener("mousedown",c,!0),t.addEventListener("pointerdown",c,!0),t.addEventListener("touchstart",c,!0),t.addEventListener("visibilitychange",d,!0))}),[])}}},function(e,t,n){"use strict";var r=n(265),i=Object(r.a)();t.a=i},function(e,t,n){"use strict";var r=n(0),i=r.createContext();t.a=i},function(e,t,n){"use strict";n.d(t,"b",(function(){return o}));var r=n(0),i=r.createContext();function o(){return r.useContext(i)}t.a=i},function(e,t,n){"use strict";var r=function(e){for(var t,n=0,r=0,i=e.length;i>=4;++r,i-=4)t=1540483477*(65535&(t=255&e.charCodeAt(r)|(255&e.charCodeAt(++r))<<8|(255&e.charCodeAt(++r))<<16|(255&e.charCodeAt(++r))<<24))+(59797*(t>>>16)<<16),n=1540483477*(65535&(t^=t>>>24))+(59797*(t>>>16)<<16)^1540483477*(65535&n)+(59797*(n>>>16)<<16);switch(i){case 3:n^=(255&e.charCodeAt(r+2))<<16;case 2:n^=(255&e.charCodeAt(r+1))<<8;case 1:n=1540483477*(65535&(n^=255&e.charCodeAt(r)))+(59797*(n>>>16)<<16)}return(((n=1540483477*(65535&(n^=n>>>13))+(59797*(n>>>16)<<16))^n>>>15)>>>0).toString(36)},i={animationIterationCount:1,borderImageOutset:1,borderImageSlice:1,borderImageWidth:1,boxFlex:1,boxFlexGroup:1,boxOrdinalGroup:1,columnCount:1,columns:1,flex:1,flexGrow:1,flexPositive:1,flexShrink:1,flexNegative:1,flexOrder:1,gridRow:1,gridRowEnd:1,gridRowSpan:1,gridRowStart:1,gridColumn:1,gridColumnEnd:1,gridColumnSpan:1,gridColumnStart:1,msGridRow:1,msGridRowSpan:1,msGridColumn:1,msGridColumnSpan:1,fontWeight:1,lineHeight:1,opacity:1,order:1,orphans:1,tabSize:1,widows:1,zIndex:1,zoom:1,WebkitLineClamp:1,fillOpacity:1,floodOpacity:1,stopOpacity:1,strokeDasharray:1,strokeDashoffset:1,strokeMiterlimit:1,strokeOpacity:1,strokeWidth:1},o=n(459);n.d(t,"a",(function(){return g}));var a=/[A-Z]|^ms/g,s=/_EMO_([^_]+?)_([^]*?)_EMO_/g,l=function(e){return 45===e.charCodeAt(1)},u=function(e){return null!=e&&"boolean"!=typeof e},c=Object(o.a)((function(e){return l(e)?e:e.replace(a,"-$&").toLowerCase()})),d=function(e,t){switch(e){case"animation":case"animationName":if("string"==typeof t)return t.replace(s,(function(e,t,n){return f={name:t,styles:n,next:f},t}))}return 1===i[e]||l(e)||"number"!=typeof t||0===t?t:t+"px"};function h(e,t,n,r){if(null==n)return"";if(void 0!==n.__emotion_styles)return n;switch(typeof n){case"boolean":return"";case"object":if(1===n.anim)return f={name:n.name,styles:n.styles,next:f},n.name;if(void 0!==n.styles){var i=n.next;if(void 0!==i)for(;void 0!==i;)f={name:i.name,styles:i.styles,next:f},i=i.next;return n.styles+";"}return function(e,t,n){var r="";if(Array.isArray(n))for(var i=0;i<n.length;i++)r+=h(e,t,n[i],!1);else for(var o in n){var a=n[o];if("object"!=typeof a)null!=t&&void 0!==t[a]?r+=o+"{"+t[a]+"}":u(a)&&(r+=c(o)+":"+d(o,a)+";");else if(!Array.isArray(a)||"string"!=typeof a[0]||null!=t&&void 0!==t[a[0]]){var s=h(e,t,a,!1);switch(o){case"animation":case"animationName":r+=c(o)+":"+s+";";break;default:r+=o+"{"+s+"}"}}else for(var l=0;l<a.length;l++)u(a[l])&&(r+=c(o)+":"+d(o,a[l])+";")}return r}(e,t,n);case"function":if(void 0!==e){var o=f,a=n(e);return f=o,h(e,t,a,r)}break;case"string":}if(null==t)return n;var s=t[n];return void 0===s||r?n:s}var f,p=/label:\s*([^\s;\n{]+)\s*;/g;var g=function(e,t,n){if(1===e.length&&"object"==typeof e[0]&&null!==e[0]&&void 0!==e[0].styles)return e[0];var i=!0,o="";f=void 0;var a=e[0];null==a||void 0===a.raw?(i=!1,o+=h(n,t,a,!1)):o+=a[0];for(var s=1;s<e.length;s++)o+=h(n,t,e[s],46===o.charCodeAt(o.length-1)),i&&(o+=a[s]);p.lastIndex=0;for(var l,u="";null!==(l=p.exec(o));)u+="-"+l[1];return{name:r(o)+u,styles:o,next:f}}},function(e,t,n){"use strict";n.d(t,"a",(function(){return a})),n.d(t,"c",(function(){return s})),n.d(t,"b",(function(){return l})),n.d(t,"f",(function(){return u})),n.d(t,"d",(function(){return c})),n.d(t,"g",(function(){return d})),n.d(t,"h",(function(){return h})),n.d(t,"i",(function(){return f})),n.d(t,"e",(function(){return p}));var r=n(51),i=n(18),o=n(173);class a extends i.Widget{constructor(e){super(),this.mimeType=e.mimeType,this.sanitizer=e.sanitizer,this.resolver=e.resolver,this.linkHandler=e.linkHandler,this.translator=e.translator||r.a,this.latexTypesetter=e.latexTypesetter,this.node.dataset.mimeType=this.mimeType}async renderModel(e){for(;this.node.firstChild;)this.node.removeChild(this.node.firstChild);this.toggleClass("jp-mod-trusted",e.trusted),await this.render(e);const{fragment:t}=e.metadata;t&&this.setFragment(t)}setFragment(e){}}class s extends a{constructor(e){super(e),this.addClass("jp-RenderedHTMLCommon")}setFragment(e){let t;try{t=this.node.querySelector(e)}catch(e){console.warn("Unable to set URI fragment identifier.",e)}t&&t.scrollIntoView()}}class l extends s{constructor(e){super(e),this.addClass("jp-RenderedHTML")}render(e){return o.a({host:this.node,source:String(e.data[this.mimeType]),trusted:e.trusted,resolver:this.resolver,sanitizer:this.sanitizer,linkHandler:this.linkHandler,shouldTypeset:this.isAttached,latexTypesetter:this.latexTypesetter,translator:this.translator})}onAfterAttach(e){this.latexTypesetter&&this.latexTypesetter.typeset(this.node)}}class u extends a{constructor(e){super(e),this.addClass("jp-RenderedLatex")}render(e){return o.c({host:this.node,source:String(e.data[this.mimeType]),shouldTypeset:this.isAttached,latexTypesetter:this.latexTypesetter})}onAfterAttach(e){this.latexTypesetter&&this.latexTypesetter.typeset(this.node)}}class c extends a{constructor(e){super(e),this.addClass("jp-RenderedImage")}render(e){const t=e.metadata[this.mimeType];return o.b({host:this.node,mimeType:this.mimeType,source:String(e.data[this.mimeType]),width:t&&t.width,height:t&&t.height,needsBackground:e.metadata.needs_background,unconfined:t&&t.unconfined})}}class d extends s{constructor(e){super(e),this.addClass("jp-RenderedMarkdown")}render(e){return o.d({host:this.node,source:String(e.data[this.mimeType]),trusted:e.trusted,resolver:this.resolver,sanitizer:this.sanitizer,linkHandler:this.linkHandler,shouldTypeset:this.isAttached,latexTypesetter:this.latexTypesetter,translator:this.translator})}onAfterAttach(e){this.latexTypesetter&&this.latexTypesetter.typeset(this.node)}}class h extends a{constructor(e){super(e),this.addClass("jp-RenderedSVG")}render(e){const t=e.metadata[this.mimeType];return o.e({host:this.node,source:String(e.data[this.mimeType]),trusted:e.trusted,unconfined:t&&t.unconfined,translator:this.translator})}onAfterAttach(e){this.latexTypesetter&&this.latexTypesetter.typeset(this.node)}}class f extends a{constructor(e){super(e),this.addClass("jp-RenderedText")}render(e){return o.f({host:this.node,sanitizer:this.sanitizer,source:String(e.data[this.mimeType]),translator:this.translator})}}class p extends a{constructor(e){super(e),this.addClass("jp-RenderedJavaScript")}render(e){const t=this.translator.load("jupyterlab");return o.f({host:this.node,sanitizer:this.sanitizer,source:t.__("JavaScript output is disabled in JupyterLab"),translator:this.translator})}}},function(e,t,n){"use strict";n.d(t,"a",(function(){return r})),n.d(t,"b",(function(){return i}));function r(e,t,n){var r="";return n.split(" ").forEach((function(n){void 0!==e[n]?t.push(e[n]):r+=n+" "})),r}var i=function(e,t,n){var r=e.key+"-"+t.name;if(!1===n&&void 0===e.registered[r]&&(e.registered[r]=t.styles),void 0===e.inserted[t.name]){var i=t;do{e.insert("."+r,i,e.sheet,!0);i=i.next}while(void 0!==i)}}},function(e,t,n){"use strict";var r=n(440),i=n.n(r),o=n(51),a=n(24),s=n.n(a),l=n(58),u=n(16),c=n(86),d=n(17),h=n(439),f=n(463);class p{constructor(e=""){this._text="",this._isDisposed=!1,this._changed=new u.Signal(this),this._text=e}get type(){return"String"}get changed(){return this._changed}set text(e){e.length===this._text.length&&e===this._text||(this._text=e,this._changed.emit({type:"set",start:0,end:e.length,value:e}))}get text(){return this._text}insert(e,t){this._text=this._text.slice(0,e)+t+this._text.slice(e),this._changed.emit({type:"insert",start:e,end:e+t.length,value:t})}remove(e,t){const n=this._text.slice(e,t);this._text=this._text.slice(0,e)+this._text.slice(t),this._changed.emit({type:"remove",start:e,end:t,value:n})}clear(){this.text=""}get isDisposed(){return this._isDisposed}dispose(){this._isDisposed||(this._isDisposed=!0,u.Signal.clearData(this),this.clear())}}var g,v=n(6),m=n(1202);class y extends m.a{constructor(e){super(),this._inCompound=!1,this._isUndoable=!0,this._madeCompoundChange=!1,this._index=-1,this._stack=[],this._serializer=e,this.changed.connect(this._onListChanged,this)}get canRedo(){return this._index<this._stack.length-1}get canUndo(){return this._index>=0}beginCompoundOperation(e){this._inCompound=!0,this._isUndoable=!1!==e,this._madeCompoundChange=!1}endCompoundOperation(){this._inCompound=!1,this._isUndoable=!0,this._madeCompoundChange&&this._index++}undo(){if(!this.canUndo)return;const e=this._stack[this._index];this._isUndoable=!1;for(const t of e.reverse())this._undoChange(t);this._isUndoable=!0,this._index--}redo(){if(!this.canRedo)return;this._index++;const e=this._stack[this._index];this._isUndoable=!1;for(const t of e)this._redoChange(t);this._isUndoable=!0}clearUndo(){this._index=-1,this._stack=[]}_onListChanged(e,t){if(this.isDisposed||!this._isUndoable)return;this._inCompound&&this._madeCompoundChange||(this._stack=this._stack.slice(0,this._index+1));const n=this._copyChange(t);this._stack[this._index+1]?this._stack[this._index+1].push(n):this._stack.push([n]),this._inCompound?this._madeCompoundChange=!0:this._index++}_undoChange(e){let t=0;const n=this._serializer;switch(e.type){case"add":Object(v.each)(e.newValues,()=>{this.remove(e.newIndex)});break;case"set":t=e.oldIndex,Object(v.each)(e.oldValues,e=>{this.set(t++,n.fromJSON(e))});break;case"remove":t=e.oldIndex,Object(v.each)(e.oldValues,e=>{this.insert(t++,n.fromJSON(e))});break;case"move":this.move(e.newIndex,e.oldIndex);break;default:return}}_redoChange(e){let t=0;const n=this._serializer;switch(e.type){case"add":t=e.newIndex,Object(v.each)(e.newValues,e=>{this.insert(t++,n.fromJSON(e))});break;case"set":t=e.newIndex,Object(v.each)(e.newValues,t=>{this.set(e.newIndex++,n.fromJSON(t))});break;case"remove":Object(v.each)(e.oldValues,()=>{this.remove(e.oldIndex)});break;case"move":this.move(e.oldIndex,e.newIndex);break;default:return}}_copyChange(e){const t=[];Object(v.each)(e.oldValues,e=>{t.push(this._serializer.toJSON(e))});const n=[];return Object(v.each)(e.newValues,e=>{n.push(this._serializer.toJSON(e))}),{type:e.type,oldIndex:e.oldIndex,newIndex:e.newIndex,oldValues:t,newValues:n}}}(y||(y={})).IdentitySerializer=class{toJSON(e){return e}fromJSON(e){return e}};class b{constructor(e=null){this._value=null,this._changed=new u.Signal(this),this._isDisposed=!1,this._value=e}get type(){return"Value"}get isDisposed(){return this._isDisposed}get changed(){return this._changed}get(){return this._value}set(e){const t=this._value;d.JSONExt.deepEqual(t,e)||(this._value=e,this._changed.emit({oldValue:t,newValue:e}))}dispose(){this._isDisposed||(this._isDisposed=!0,u.Signal.clearData(this),this._value=null)}}(b||(b={})).IChangedArgs=class{};class C{constructor(e={}){this.isPrepopulated=!1,this.isCollaborative=!1,this.connected=Promise.resolve(void 0),this._toDispose=!1,this._isDisposed=!1,this._disposables=new c.DisposableSet,this._basePath=e.basePath||"",e.baseDB?this._db=e.baseDB:(this._db=new h.a,this._toDispose=!0)}get basePath(){return this._basePath}get isDisposed(){return this._isDisposed}get(e){return this._db.get(this._resolvePath(e))}has(e){return this._db.has(this._resolvePath(e))}createString(e){const t=new p;return this._disposables.add(t),this.set(e,t),t}createList(e){const t=new y(new y.IdentitySerializer);return this._disposables.add(t),this.set(e,t),t}createMap(e){const t=new f.a;return this._disposables.add(t),this.set(e,t),t}createValue(e){const t=new b;return this._disposables.add(t),this.set(e,t),t}getValue(e){const t=this.get(e);if(!t||"Value"!==t.type)throw Error("Can only call getValue for an ObservableValue");return t.get()}setValue(e,t){const n=this.get(e);if(!n||"Value"!==n.type)throw Error("Can only call setValue on an ObservableValue");n.set(t)}view(e){const t=new C({basePath:e,baseDB:this});return this._disposables.add(t),t}set(e,t){this._db.set(this._resolvePath(e),t)}dispose(){this.isDisposed||(this._isDisposed=!0,this._toDispose&&this._db.dispose(),this._disposables.dispose())}_resolvePath(e){return this._basePath&&(e=this._basePath+"."+e),e}}!function(e){e.defaultSelectionStyle={className:"",displayName:"",color:"black"};e.Model=class{constructor(e){this._isDisposed=!1,this._mimeTypeChanged=new u.Signal(this),(e=e||{}).modelDB?this.modelDB=e.modelDB:this.modelDB=new C;const t=this.modelDB.createString("value");t.text=t.text||e.value||"";const n=this.modelDB.createValue("mimeType");n.set(e.mimeType||"text/plain"),n.changed.connect(this._onMimeTypeChanged,this),this.modelDB.createMap("selections")}get mimeTypeChanged(){return this._mimeTypeChanged}get value(){return this.modelDB.get("value")}get selections(){return this.modelDB.get("selections")}get mimeType(){return this.modelDB.getValue("mimeType")}set mimeType(e){this.mimeType!==e&&this.modelDB.setValue("mimeType",e)}get isDisposed(){return this._isDisposed}dispose(){this._isDisposed||(this._isDisposed=!0,this.value.text="",u.Signal.clearData(this))}_onMimeTypeChanged(e,t){this._mimeTypeChanged.emit({name:"mimeType",oldValue:t.oldValue,newValue:t.newValue})}},e.defaultConfig={fontFamily:null,fontSize:null,lineHeight:null,lineNumbers:!1,lineWrap:"on",wordWrapColumn:80,readOnly:!1,tabSize:4,insertSpaces:!0,matchBrackets:!0,autoClosingBrackets:!0,handlePaste:!0,rulers:[],codeFolding:!1}}(g||(g={}));var A,_=n(261);!function(e){e.defaultMimeType="text/plain"}(A||(A={}));n(500),n(1432),n(1321);s.a.defineMode("ipython",(e,t)=>{const n={};for(const e in t)t.hasOwnProperty(e)&&(n[e]=t[e]);return n.name="python",n.singleOperators=new RegExp("^[\\+\\-\\*/%&|@\\^~<>!\\?]"),n.identifiers=new RegExp("^[_A-Za-z¡-][_A-Za-z0-9¡-]*"),s.a.getMode(e,n)},"python"),s.a.defineMIME("text/x-ipython","ipython"),s.a.modeInfo.push({ext:[],mime:"text/x-ipython",mode:"ipython",name:"ipython"});n(1322),n(1323),n(1319);s.a.defineMode("ipythongfm",(e,t)=>{const n=s.a.getMode(e,{name:"gfm",tokenTypeOverrides:{list3:"string-2"}}),r=s.a.getMode(e,{name:"stex",inMathMode:!0});return s.a.multiplexingMode(n,{open:"$$",close:"$$",mode:r,delimStyle:"delimit"},{open:"$",close:"$",mode:r,delimStyle:"delimit"},{open:"\\(",close:"\\)",mode:r,delimStyle:"delimit"},{open:"\\[",close:"\\]",mode:r,delimStyle:"delimit"})},"gfm"),s.a.defineMIME("text/x-ipythongfm","ipythongfm"),s.a.modeInfo.push({ext:[],mime:"text/x-ipythongfm",mode:"ipythongfm",name:"ipythongfm"});n(1314),n(1316),n(1324),n(1325),n(655),n(1320),n(1326),n(1327);var I,x,w=n(48);!function(e){const t=[{loader:async e=>s.a.modes.hasOwnProperty(e.mode),rank:0},{loader:function(e){return new Promise((t,r)=>{Promise.all([n.e(1),n.e(2)]).then((function(){var r=[n(2492)(`./${e.mode}/${e.mode}.js`)];(function(){t(!0)}).apply(null,r)})).catch(n.oe)})},rank:99}];function r(e){var t;const n="string"==typeof e?e:e.mode||e.name,r="string"!=typeof e?e.mime:n,o="string"!=typeof e&&null!==(t=e.ext)&&void 0!==t?t:[];return s.a.findModeByName(n||"")||s.a.findModeByMIME(r||"")||i(o)||s.a.findModeByMIME(A.defaultMimeType)||s.a.findModeByMIME("text/plain")}function i(e){if("string"==typeof e)return s.a.findModeByExtension(e);for(let t=0;t<e.length;t++){const n=s.a.findModeByExtension(e[t]);if(n)return n}return null}e.getModeInfo=function(){return s.a.modeInfo},e.run=function(e,t,n){s.a.runMode(e,t,n)},e.ensure=async function(e){const n=r(e);for(const e of t)if(await e.loader(n))return n;return null},e.addSpecLoader=function(e,n){const r={loader:e,rank:n},i=v.ArrayExt.upperBound(t,r,x.itemCmp);v.ArrayExt.insert(t,i,r)},e.findBest=r,e.findByMIME=function(e){return s.a.findModeByMIME(e)},e.findByName=function(e){return s.a.findModeByName(e)},e.findByFileName=function(e){const t=w.PathExt.basename(e);return s.a.findModeByFileName(t)},e.findByExtension=i}(I||(I={})),(x||(x={})).itemCmp=function(e,t){return e.rank-t.rank};n(1433),n(1434),n(867),n(1435),n(868),n(1436),n(1437),n(1438),n(1439),n(1440),n(1441),n(1442),n(501),n(1443),n(1444),n(1445),n(1446),n(1447),n(1315),n(1448),n(1449);var M=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(null!=e&&"function"==typeof Object.getOwnPropertySymbols){var i=0;for(r=Object.getOwnPropertySymbols(e);i<r.length;i++)t.indexOf(r[i])<0&&Object.prototype.propertyIsEnumerable.call(e,r[i])&&(n[r[i]]=e[r[i]])}return n};class S{constructor(e){this.edgeRequested=new u.Signal(this),this.selectionMarkers={},this._keydownHandlers=new Array,this._changeGuard=!1,this._uuid="",this._needsRefresh=!1,this._isDisposed=!1,this._lastChange=null;const t=this.host=e.host;this.translator=e.translator||o.a,this._trans=this.translator.load("jupyterlab"),t.classList.add("jp-CodeMirrorEditor"),t.classList.add("jp-Editor"),t.addEventListener("focus",this,!0),t.addEventListener("blur",this,!0),t.addEventListener("scroll",this,!0),this._uuid=e.uuid||d.UUID.uuid4();const n=e.selectionStyle||{};this._selectionStyle=Object.assign(Object.assign({},g.defaultSelectionStyle),n);const r=this._model=e.model,i=e.config||{},a=this._config=Object.assign(Object.assign({},S.defaultConfig),i),l=this._editor=E.createEditor(t,a);l.getDoc().setValue(r.value.text),this.clearHistory(),this._onMimeTypeChanged(),this._onCursorActivity(),this._poll=new _.Poll({factory:async()=>{this._checkSync()},frequency:{interval:3e3,backoff:!1},standby:()=>!this._lastChange||"when-hidden"}),r.value.changed.connect(this._onValueChanged,this),r.mimeTypeChanged.connect(this._onMimeTypeChanged,this),r.selections.changed.connect(this._onSelectionsChanged,this),s.a.on(l,"keydown",(e,t)=>{-1===v.ArrayExt.findFirstIndex(this._keydownHandlers,e=>!0===e(this,t)&&(t.preventDefault(),!0))&&this.onKeydown(t)}),s.a.on(l,"cursorActivity",()=>this._onCursorActivity()),s.a.on(l.getDoc(),"beforeChange",(e,t)=>{this._beforeDocChanged(e,t)}),s.a.on(l.getDoc(),"change",(e,t)=>{"setValue"===t.origin&&this.hasFocus()&&this.refresh(),this._lastChange=t}),l.on("paste",(e,t)=>{var n;null===(n=this._config.handlePaste)||void 0===n||n||(t.codemirrorIgnore=!0)}),l.getWrapperElement().addEventListener("paste",()=>{this.hasFocus()&&this.refresh()})}get uuid(){return this._uuid}set uuid(e){this._uuid=e}get selectionStyle(){return this._selectionStyle}set selectionStyle(e){this._selectionStyle=e}get editor(){return this._editor}get doc(){return this._editor.getDoc()}get lineCount(){return this.doc.lineCount()}get model(){return this._model}get lineHeight(){return this._editor.defaultTextHeight()}get charWidth(){return this._editor.defaultCharWidth()}get isDisposed(){return this._isDisposed}dispose(){this.isDisposed||(this._isDisposed=!0,this.host.removeEventListener("focus",this,!0),this.host.removeEventListener("blur",this,!0),this.host.removeEventListener("scroll",this,!0),this._keydownHandlers.length=0,this._poll.dispose(),u.Signal.clearData(this))}getOption(e){return this._config[e]}setOption(e,t){this._config[e]!==t&&(this._config[e]=t,E.setOption(this.editor,e,t,this._config))}getLine(e){return this.doc.getLine(e)}getOffsetAt(e){return this.doc.indexFromPos({ch:e.column,line:e.line})}getPositionAt(e){const{ch:t,line:n}=this.doc.posFromIndex(e);return{line:n,column:t}}undo(){this.doc.undo()}redo(){this.doc.redo()}clearHistory(){this.doc.clearHistory()}focus(){this._editor.focus()}hasFocus(){return this._editor.getWrapperElement().contains(document.activeElement)}blur(){this._editor.getInputField().blur()}refresh(){this._editor.refresh(),this._needsRefresh=!1}resizeToFit(){this.hasFocus()?this.refresh():this._needsRefresh=!0,this._clearHover()}addOverlay(e,t){this._editor.addOverlay(e,t)}removeOverlay(e){this._editor.removeOverlay(e)}getSearchCursor(e,t,n){return this._editor.getDoc().getSearchCursor(e,t,n)}getCursor(e){return this._editor.getDoc().getCursor(e)}get state(){return this._editor.state}operation(e){return this._editor.operation(e)}firstLine(){return this._editor.getDoc().firstLine()}lastLine(){return this._editor.getDoc().lastLine()}scrollIntoView(e,t){this._editor.scrollIntoView(e,t)}cursorCoords(e,t){return this._editor.cursorCoords(e,t)}getRange(e,t,n){return this._editor.getDoc().getRange(e,t,n)}addKeydownHandler(e){return this._keydownHandlers.push(e),new c.DisposableDelegate(()=>{v.ArrayExt.removeAllWhere(this._keydownHandlers,t=>t===e)})}setSize(e){e?this._editor.setSize(e.width,e.height):this._editor.setSize(null,null),this._needsRefresh=!1}revealPosition(e){const t=this._toCodeMirrorPosition(e);this._editor.scrollIntoView(t)}revealSelection(e){const t={from:this._toCodeMirrorPosition(e.start),to:this._toCodeMirrorPosition(e.end)};this._editor.scrollIntoView(t)}getCoordinateForPosition(e){const t=this._toCodeMirrorPosition(e);return this.editor.charCoords(t,"page")}getPositionForCoordinate(e){return this._toPosition(this.editor.coordsChar(e))||null}getCursorPosition(){const e=this.doc.getCursor();return this._toPosition(e)}setCursorPosition(e,t){const n=this._toCodeMirrorPosition(e);this.doc.setCursor(n,void 0,t),this.editor.hasFocus()||this.model.selections.set(this.uuid,this.getSelections())}getSelection(){return this.getSelections()[0]}setSelection(e){this.setSelections([e])}getSelections(){const e=this.doc.listSelections();if(e.length>0)return e.map(e=>this._toSelection(e));const t=this.doc.getCursor();return[this._toSelection({anchor:t,head:t})]}setSelections(e){const t=this._toCodeMirrorSelections(e);this.doc.setSelections(t,0)}replaceSelection(e){this.doc.replaceSelection(e)}getTokens(){let e=[];for(let t=0;t<this.lineCount;++t){const n=this.editor.getLineTokens(t).map(e=>({offset:this.getOffsetAt({column:e.start,line:t}),value:e.string,type:e.type||""}));e=e.concat(n)}return e}getTokenForPosition(e){var t;const n=this._toCodeMirrorPosition(e),r=this.editor.getTokenAt(n);return{offset:this.getOffsetAt({column:r.start,line:n.line}),value:r.string,type:null!==(t=r.type)&&void 0!==t?t:void 0}}newIndentedLine(){this.execCommand("newlineAndIndent")}execCommand(e){this._editor.execCommand(e)}onKeydown(e){const t=this.getCursorPosition(),{line:n,column:r}=t;if(0===n&&0===r&&38===e.keyCode)return e.shiftKey||this.edgeRequested.emit("top"),!1;if(0===n&&38===e.keyCode)return e.shiftKey||this.edgeRequested.emit("topLine"),!1;const i=this.lineCount-1,o=this.getLine(i).length;return n===i&&r===o&&40===e.keyCode&&(e.shiftKey||this.edgeRequested.emit("bottom"),!1)}_toCodeMirrorSelections(e){if(e.length>0)return e.map(e=>this._toCodeMirrorSelection(e));const t={line:0,ch:0};return[{anchor:t,head:t}]}_onMimeTypeChanged(){const e=this._model.mimeType,t=this._editor;I.ensure(e).then(e=>{var n;t.setOption("mode",null!==(n=null==e?void 0:e.mime)&&void 0!==n?n:"null")});const n=t.getOption("extraKeys")||{};"text/plain"!==e&&"text/x-ipythongfm"!==e?n.Backspace="delSpaceToPrevTabStop":delete n.Backspace,t.setOption("extraKeys",n)}_onSelectionsChanged(e,t){const n=t.key;n!==this.uuid&&(this._cleanSelections(n),"remove"!==t.type&&t.newValue&&this._markSelections(n,t.newValue))}_cleanSelections(e){const t=this.selectionMarkers[e];t&&t.forEach(e=>{e.clear()}),delete this.selectionMarkers[e]}_markSelections(e,t){const n=[];let r;e===this._hoverId&&this._clearHover(),this._model.modelDB.collaborators&&(r=this._model.modelDB.collaborators.get(e)),t.forEach(e=>{if(d.JSONExt.deepEqual(e.start,e.end)){if(r){const t=this._getCaret(r);n.push(this.doc.setBookmark(this._toCodeMirrorPosition(e.end),{widget:t}))}}else{const t=e.start.line<e.end.line||e.start.line===e.end.line&&e.start.column<=e.end.column,i=this._toCodeMirrorPosition(t?e.start:e.end),o=this._toCodeMirrorPosition(t?e.end:e.start);let a;a=r?this._toTextMarkerOptions(Object.assign(Object.assign({},e.style),{color:r.color})):this._toTextMarkerOptions(e.style),n.push(this.doc.markText(i,o,a))}}),this.selectionMarkers[e]=n}_onCursorActivity(){if(this._editor.hasFocus()){const e=this.getSelections();this.model.selections.set(this.uuid,e)}}_toSelection(e){return{uuid:this.uuid,start:this._toPosition(e.anchor),end:this._toPosition(e.head),style:this.selectionStyle}}_toTextMarkerOptions(e){const t=`background-color: rgba( ${parseInt(e.color.slice(1,3),16)}, ${parseInt(e.color.slice(3,5),16)}, ${parseInt(e.color.slice(5,7),16)}, 0.15)`;return{className:e.className,title:e.displayName,css:t}}_toCodeMirrorSelection(e){return{anchor:this._toCodeMirrorPosition(e.start),head:this._toCodeMirrorPosition(e.end)}}_toPosition(e){return{line:e.line,column:e.ch}}_toCodeMirrorPosition(e){return{line:e.line,ch:e.column}}_onValueChanged(e,t){if(this._changeGuard)return;this._changeGuard=!0;const n=this.doc;switch(t.type){case"insert":{const e=n.posFromIndex(t.start);n.replaceRange(t.value,e,e,"+input");break}case"remove":{const e=n.posFromIndex(t.start),r=n.posFromIndex(t.end);n.replaceRange("",e,r,"+input");break}case"set":n.setValue(t.value)}this._changeGuard=!1}_beforeDocChanged(e,t){if(this._changeGuard)return;this._changeGuard=!0;const n=this._model.value,r=e.indexFromPos(t.from),i=e.indexFromPos(t.to),o=t.text.join("\n");i!==r&&n.remove(r,i),o&&n.insert(r,o),this._changeGuard=!1}handleEvent(e){switch(e.type){case"focus":this._evtFocus(e);break;case"blur":this._evtBlur(e);break;case"scroll":this._evtScroll()}}_evtFocus(e){this._needsRefresh&&this.refresh(),this.host.classList.add("jp-mod-focused"),this._onCursorActivity()}_evtBlur(e){this.host.classList.remove("jp-mod-focused")}_evtScroll(){this._clearHover()}_clearHover(){this._caretHover&&(window.clearTimeout(this._hoverTimeout),document.body.removeChild(this._caretHover),this._caretHover=null)}_getCaret(e){const t=e?e.displayName:"Anonymous",n=e?e.color:this._selectionStyle.color,r=document.createElement("span");return r.className="jp-CollaboratorCursor",r.style.borderBottomColor=n,r.onmouseenter=()=>{this._clearHover(),this._hoverId=e.sessionId;const i=r.getBoundingClientRect(),o=document.createElement("div");o.className="jp-CollaboratorCursor-hover",o.style.left=String(i.left)+"px",o.style.top=String(i.bottom)+"px",o.textContent=t,o.style.backgroundColor=n,o.onmouseenter=()=>{window.clearTimeout(this._hoverTimeout)},o.onmouseleave=()=>{this._hoverTimeout=window.setTimeout(()=>{this._clearHover()},1e3)},this._caretHover=o,document.body.appendChild(o)},r.onmouseleave=()=>{this._hoverTimeout=window.setTimeout(()=>{this._clearHover()},1e3)},r}_checkSync(){const e=this._lastChange;if(!e)return;this._lastChange=null;const t=this._editor,n=t.getDoc();n.getValue()!==this._model.value.text&&(Object(l.b)({title:this._trans.__("Code Editor out of Sync"),body:this._trans.__("Please open your browser JavaScript console for bug report instructions")}),console.warn("Please paste the following to https://github.com/jupyterlab/jupyterlab/issues/2951"),console.warn(JSON.stringify({model:this._model.value.text,view:n.getValue(),selections:this.getSelections(),cursor:this.getCursorPosition(),lineSep:t.getOption("lineSeparator"),mode:t.getOption("mode"),change:e})))}}var T,E;(T=S||(S={})).defaultConfig=Object.assign(Object.assign({},g.defaultConfig),{mode:"null",theme:"jupyter",smartIndent:!0,electricChars:!0,keyMap:"default",extraKeys:null,gutters:[],fixedGutter:!0,showCursorWhenSelecting:!1,coverGutterNextToScrollbar:!1,dragDrop:!0,lineSeparator:null,scrollbarStyle:"native",lineWiseCopyCut:!0,scrollPastEnd:!1,styleActiveLine:!1,styleSelectedText:!0,selectionPointer:!1,rulers:[],foldGutter:!1,handlePaste:!0}),T.addCommand=function(e,t){s.a.commands[e]=t},function(e){function t(e,t){return e.line===t.line&&e.ch===t.ch}function n(e){const t={"CodeMirror-linenumbers":"lineNumbers","CodeMirror-foldgutter":"codeFolding"};return Object.keys(t).filter(n=>e[t[n]])}e.createEditor=function(e,t){const{autoClosingBrackets:n,fontFamily:r,fontSize:i,insertSpaces:o,lineHeight:a,lineWrap:l,wordWrapColumn:u,tabSize:c,readOnly:d}=t,h=M(t,["autoClosingBrackets","fontFamily","fontSize","insertSpaces","lineHeight","lineWrap","wordWrapColumn","tabSize","readOnly"]),f=Object.assign({autoCloseBrackets:!!n&&{},indentUnit:c,indentWithTabs:!o,lineWrapping:"off"!==l,readOnly:d},h);return s()(t=>{if(r&&(t.style.fontFamily=r),i&&(t.style.fontSize=i+"px"),a&&(t.style.lineHeight=a.toString()),d&&t.classList.add("jp-mod-readOnly"),"wordWrapColumn"===l){t.querySelector(".CodeMirror-lines").style.width=u+"ch"}if("bounded"===l){t.querySelector(".CodeMirror-lines").style.maxWidth=u+"ch"}e.appendChild(t)},f)},e.indentMoreOrinsertTab=function(e){const n=e.getDoc(),r=n.getCursor("from");if(!t(r,n.getCursor("to")))return void s.a.commands.indentMore(e);const i=n.getLine(r.line).slice(0,r.ch);/^\s*$/.test(i)?s.a.commands.indentMore(e):e.getOption("indentWithTabs")?s.a.commands.insertTab(e):s.a.commands.insertSoftTab(e)},e.delSpaceToPrevTabStop=function(e){var n;const r=e.getDoc(),i=null!==(n=e.getOption("indentUnit"))&&void 0!==n?n:2,o=r.listSelections();for(let n=o.length-1;n>=0;n--){const a=o[n].head,l=o[n].anchor;if(!t(a,l))r.replaceRange("",l,a);else{if(null!==r.getLine(a.line).substring(0,a.ch).match(/^\ +$/)){const e=(Math.ceil(a.ch/i)-1)*i,t=s.a.Pos(a.line,e);r.replaceRange("",t,a)}else{const t=e.findPosH(a,-1,"char",!1);r.replaceRange("",t,a)}}}},e.posEq=t,e.setOption=function(e,t,r,i){const o=e.getWrapperElement();switch(t){case"lineWrap":{const t="off"!==r,n=o.querySelector(".CodeMirror-lines"),a="bounded"===r?i.wordWrapColumn+"ch":null,s="wordWrapColumn"===r?i.wordWrapColumn+"ch":null;n.style.setProperty("max-width",a),n.style.setProperty("width",s),e.setOption("lineWrapping",t);break}case"wordWrapColumn":{const{lineWrap:e}=i;if("wordWrapColumn"===e||"bounded"===e){const t="wordWrapColumn"===e?"width":"maxWidth";o.querySelector(".CodeMirror-lines").style[t]=r+"ch"}break}case"tabSize":e.setOption("indentUnit",r);break;case"insertSpaces":e.setOption("indentWithTabs",!r);break;case"autoClosingBrackets":e.setOption("autoCloseBrackets",r);break;case"rulers":{const t=r;e.setOption("rulers",t.map(e=>({column:e,className:"jp-CodeMirror-ruler"})));break}case"readOnly":o.classList.toggle("jp-mod-readOnly",r),e.setOption(t,r);break;case"fontFamily":o.style.fontFamily=r;break;case"fontSize":o.style.setProperty("font-size",r?r+"px":null);break;case"lineHeight":o.style.lineHeight=r?r.toString():null;break;case"gutters":e.setOption(t,n(i));break;case"lineNumbers":e.setOption(t,r),e.setOption("gutters",n(i));break;case"codeFolding":e.setOption("foldGutter",r),e.setOption("gutters",n(i));break;default:e.setOption(t,r)}}}(E||(E={})),S.addCommand("delSpaceToPrevTabStop",E.delSpaceToPrevTabStop),S.addCommand("indentMoreOrinsertTab",E.indentMoreOrinsertTab);var O=n(0),k=n.n(O),L=n(144),D=n(54);Object(D.style)({background:"black",height:"10px",width:"100px",border:"1px solid black",borderRadius:"3px",marginLeft:"4px",overflow:"hidden"}),Object(D.style)({background:"var(--jp-brand-color2)",height:"10px"});var P="var(--jp-layout-color3)",N="var(--jp-brand-color1)",j="var(--jp-layout-color2)",R="24px",z="var(--jp-ui-font-color1)",F="white",B="2px",H="6px",U="10px",V="nowrap",Y="ellipsis";const W={fontSize:"var(--jp-ui-font-size1)",fontFamily:"var(--jp-ui-font-family)"},G=Object(D.style)(W,{lineHeight:"24px",color:z});var X=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(null!=e&&"function"==typeof Object.getOwnPropertySymbols){var i=0;for(r=Object.getOwnPropertySymbols(e);i<r.length;i++)t.indexOf(r[i])<0&&Object.prototype.propertyIsEnumerable.call(e,r[i])&&(n[r[i]]=e[r[i]])}return n};function q(e){const{title:t,source:n,className:r}=e,i=X(e,["title","source","className"]);return O.createElement("span",Object.assign({className:Object(D.classes)(G,r),title:t},i),n)}const Z={display:"flex",alignItems:"center"},Q={flexDirection:"row"};var K=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(null!=e&&"function"==typeof Object.getOwnPropertySymbols){var i=0;for(r=Object.getOwnPropertySymbols(e);i<r.length;i++)t.indexOf(r[i])<0&&Object.prototype.propertyIsEnumerable.call(e,r[i])&&(n[r[i]]=e[r[i]])}return n};const J=Object(D.style)(Z,Q);function $(e){const{spacing:t,children:n,className:r}=e,i=K(e,["spacing","children","className"]),o=O.Children.count(n);return O.createElement("div",Object.assign({className:Object(D.classes)(J,r)},i),O.Children.map(n,(e,n)=>0===n?O.createElement("div",{style:{marginRight:t+"px"}},e):n===o-1?O.createElement("div",{style:{marginLeft:t+"px"}},e):O.createElement("div",{style:{margin:`0px ${t}px`}},e)))}var ee=n(891),te=n(18);const ne={paddingLeft:H,paddingRight:H},re={$nest:{"&:hover":{backgroundColor:P}}},ie={backgroundColor:N,$nest:{["."+G]:{color:F}}},oe=(Object(D.style)({background:j,minHeight:R,justifyContent:"space-between",paddingLeft:U,paddingRight:U},Z),Object(D.style)(Z),Object(D.style)(Q),Object(D.style)({flexDirection:"row-reverse"}),Object(D.style)({maxHeight:R,marginLeft:B,marginRight:B,height:R,whiteSpace:V,textOverflow:Y,color:z},ne),Object(D.style)(ie)),ae=Object(D.style)(re),se=Object(D.style)({boxShadow:"0px 4px 4px rgba(0, 0, 0, 0.25)"});function le(e){const t=new ue(e);return t.launch(),t}class ue extends te.Widget{constructor(e){super(),this._body=e.body,this._body.addClass(se),this._anchor=e.anchor,this._align=e.align;(this.layout=new te.PanelLayout).addWidget(e.body),this._body.node.addEventListener("resize",()=>{this.update()})}launch(){this._setGeometry(),te.Widget.attach(this,document.body),this.update(),this._anchor.addClass(oe),this._anchor.removeClass(ae)}onUpdateRequest(e){this._setGeometry(),super.onUpdateRequest(e)}onAfterAttach(e){document.addEventListener("click",this,!1),this.node.addEventListener("keydown",this,!1),window.addEventListener("resize",this,!1)}onAfterDetach(e){document.removeEventListener("click",this,!1),this.node.removeEventListener("keydown",this,!1),window.removeEventListener("resize",this,!1)}onResize(){this.update()}dispose(){super.dispose(),this._anchor.removeClass(oe),this._anchor.addClass(ae)}handleEvent(e){switch(e.type){case"keydown":this._evtKeydown(e);break;case"click":this._evtClick(e);break;case"resize":this.onResize()}}_evtClick(e){!e.target||this._body.node.contains(e.target)||this._anchor.node.contains(e.target)||this.dispose()}_evtKeydown(e){switch(e.keyCode){case 27:e.stopPropagation(),e.preventDefault(),this.dispose()}}_setGeometry(){let e=0;const t=this._anchor.node.getBoundingClientRect(),n=this._body.node.getBoundingClientRect();"right"===this._align&&(e=-(n.width-t.width));const r=window.getComputedStyle(this._body.node);ee.a.setGeometry({anchor:t,host:document.body,maxHeight:500,minHeight:20,node:this._body.node,offset:{horizontal:e},privilege:"forceAbove",style:r})}}var ce=n(1307);Object(D.style)({boxShadow:"0px 4px 4px rgba(0, 0, 0, 0.25)"});const de=Object(D.style)({padding:"4px 12px",backgroundColor:"var(--jp-layout-color2)",boxShadow:"var(--jp-toolbar-box-shadow)",zIndex:2,fontSize:"var(--jp-ui-font-size1)"}),he=Object(D.style)({fontSize:"var(--jp-ui-font-size0)",lineHeight:"var(--jp-ui-font-size1)",marginTop:"4px",color:"var(--jp-ui-font-color0)"}),fe={border:"none",borderRadius:"0px",position:"absolute",backgroundSize:"16px",backgroundRepeat:"no-repeat",backgroundPosition:"center",outline:"none",top:"0px",right:"0px"},pe=Object(D.style)(fe,{top:"4px",right:"8px",height:"24px",padding:"0px 12px",width:"12px"}),ge=Object(D.style)(fe,{backgroundColor:"var(--jp-brand-color1)",height:"100%",width:"100%",boxSizing:"border-box",padding:"4px 6px"}),ve=Object(D.style)(fe,{backgroundColor:"transparent",height:"100%",width:"100%",boxSizing:"border-box"}),me=Object(D.style)({overflow:"hidden",padding:"0px 8px",border:"1px solid var(--jp-border-color0)",backgroundColor:"var(--jp-input-active-background)",height:"22px"}),ye=Object(D.style)({border:"var(--jp-border-width) solid var(--md-blue-500)",boxShadow:"inset 0 0 4px var(--md-blue-300)"}),be=Object(D.style)({background:"transparent",width:"200px",height:"100%",border:"none",outline:"none",color:"var(--jp-ui-font-color0)",lineHeight:"28px"});class Ce extends k.a.Component{constructor(e){super(e),this._handleChange=e=>{this.setState({value:e.currentTarget.value})},this._handleSubmit=e=>{e.preventDefault();const t=parseInt(this._textInput.value,10);return!isNaN(t)&&isFinite(t)&&1<=t&&t<=this.props.maxLine&&this.props.handleSubmit(t),!1},this._handleFocus=()=>{this.setState({hasFocus:!0})},this._handleBlur=()=>{this.setState({hasFocus:!1})},this._textInput=null,this.translator=e.translator||o.a,this._trans=this.translator.load("jupyterlab"),this.state={value:"",hasFocus:!1}}componentDidMount(){this._textInput.focus()}render(){return k.a.createElement("div",{className:de},k.a.createElement("form",{name:"lineColumnForm",onSubmit:this._handleSubmit,noValidate:!0},k.a.createElement("div",{className:Object(D.classes)(me,"lm-lineForm-wrapper",this.state.hasFocus?ye:void 0)},k.a.createElement("input",{type:"text",className:be,onChange:this._handleChange,onFocus:this._handleFocus,onBlur:this._handleBlur,value:this.state.value,ref:e=>{this._textInput=e}}),k.a.createElement("div",{className:pe},k.a.createElement(ce.h.react,{className:ge,elementPosition:"center"}),k.a.createElement("input",{type:"submit",className:ve,value:""}))),k.a.createElement("label",{className:he},this._trans.__("Go to line number between 1 and %1",this.props.maxLine))))}}function Ae(e){const t=(e.translator||o.a).load("jupyterlab");return k.a.createElement(q,{onClick:e.handleClick,source:t.__("Ln %1, Col %2",e.line,e.column),title:t.__("Go to line number…")})}class _e extends L.d{constructor(e){super(new _e.Model),this._popup=null,this.addClass(ae),this.translator=e||o.a}render(){return null===this.model?null:k.a.createElement(Ae,{line:this.model.line,column:this.model.column,translator:this.translator,handleClick:()=>this._handleClick()})}_handleClick(){this._popup&&this._popup.dispose();const e=L.a.create(k.a.createElement(Ce,{handleSubmit:e=>this._handleSubmit(e),currentLine:this.model.line,maxLine:this.model.editor.lineCount,translator:this.translator}));this._popup=le({body:e,anchor:this,align:"right"})}_handleSubmit(e){this.model.editor.setCursorPosition({line:e-1,column:0}),this._popup.dispose(),this.model.editor.focus()}}function Ie(e){const t=(e.translator||o.a).load("jupyterlab");let n="";return e.status&&(n=" | "+w.Text.titleCase(t.__(e.status))),k.a.createElement(q,{onClick:e.handleClick,source:`${e.kernelName}${n}`,title:t.__("Change kernel for %1",e.activityName)})}!function(e){class t extends L.c{constructor(){super(...arguments),this._onSelectionChanged=()=>{const e=this._getAllState(),t=this.editor.getCursorPosition();this._line=t.line+1,this._column=t.column+1,this._triggerChange(e,this._getAllState())},this._line=1,this._column=1,this._editor=null}get editor(){return this._editor}set editor(e){const t=this._editor;t&&t.model.selections.changed.disconnect(this._onSelectionChanged);const n=this._getAllState();if(this._editor=e,this._editor){this._editor.model.selections.changed.connect(this._onSelectionChanged);const e=this._editor.getCursorPosition();this._column=e.column+1,this._line=e.line+1}else this._column=1,this._line=1;this._triggerChange(n,this._getAllState())}get line(){return this._line}get column(){return this._column}_getAllState(){return[this._line,this._column]}_triggerChange(e,t){e[0]===t[0]&&e[1]===t[1]||this.stateChanged.emit(void 0)}}e.Model=t}(_e||(_e={}));class xe extends L.d{constructor(e,t){super(new xe.Model(t)),this.translator=t||o.a,this._handleClick=e.onClick,this.addClass(ae)}render(){return null===this.model?null:k.a.createElement(Ie,{status:this.model.status,kernelName:this.model.kernelName,activityName:this.model.activityName,handleClick:this._handleClick,translator:this.translator})}}!function(e){class t extends L.c{constructor(e){super(),this._onKernelStatusChanged=()=>{var e;this._kernelStatus=null===(e=this._sessionContext)||void 0===e?void 0:e.kernelDisplayStatus,this.stateChanged.emit(void 0)},this._onKernelChanged=(e,t)=>{var n;const r=this._getAllState();this._kernelStatus=null===(n=this._sessionContext)||void 0===n?void 0:n.kernelDisplayStatus,this._kernelName=e.kernelDisplayName,this._triggerChange(r,this._getAllState())},this._activityName="activity",this._kernelStatus="",this._sessionContext=null,e=e||o.a,this._trans=e.load("jupyterlab"),this._kernelName=this._trans.__("No Kernel!")}get kernelName(){return this._kernelName}get status(){return this._kernelStatus}get activityName(){return this._activityName}set activityName(e){this._activityName!==e&&(this._activityName=e,this.stateChanged.emit(void 0))}get sessionContext(){return this._sessionContext}set sessionContext(e){var t,n,r;null===(t=this._sessionContext)||void 0===t||t.statusChanged.disconnect(this._onKernelStatusChanged),null===(n=this._sessionContext)||void 0===n||n.kernelChanged.disconnect(this._onKernelChanged);const i=this._getAllState();this._sessionContext=e,this._kernelStatus=null==e?void 0:e.kernelDisplayStatus,this._kernelName=null!==(r=null==e?void 0:e.kernelDisplayName)&&void 0!==r?r:"No Kernel",null==e||e.statusChanged.connect(this._onKernelStatusChanged,this),null==e||e.connectionStatusChanged.connect(this._onKernelStatusChanged,this),null==e||e.kernelChanged.connect(this._onKernelChanged,this),this._triggerChange(i,this._getAllState())}_getAllState(){return[this._kernelName,this._kernelStatus,this._activityName]}_triggerChange(e,t){d.JSONExt.deepEqual(e,t)&&this.stateChanged.emit(void 0)}}e.Model=t}(xe||(xe={}));function we(e){return k.a.createElement($,{spacing:4,onClick:e.handleClick},k.a.createElement($,{spacing:4},k.a.createElement(q,{source:e.terminals}),k.a.createElement(ce.l.react,{left:"1px",top:"3px",stylesheet:"statusBar"})),k.a.createElement($,{spacing:4},k.a.createElement(q,{source:e.sessions}),k.a.createElement(ce.g.react,{top:"2px",stylesheet:"statusBar"})))}class Me extends L.d{constructor(e){super(new Me.Model),this._serviceManager=e.serviceManager,this._handleClick=e.onClick,this.translator=e.translator||o.a,this._trans=this.translator.load("jupyterload"),this._serviceManager.sessions.runningChanged.connect(this._onSessionsRunningChanged,this),this._serviceManager.terminals.runningChanged.connect(this._onTerminalsRunningChanged,this),this.addClass(ae)}render(){return this.model?(this.title.caption=this._trans.__("%1 Terminals, %2 Kernel sessions",this.model.terminals,this.model.sessions),k.a.createElement(we,{sessions:this.model.sessions,terminals:this.model.terminals,handleClick:this._handleClick})):null}dispose(){super.dispose(),this._serviceManager.sessions.runningChanged.disconnect(this._onSessionsRunningChanged,this),this._serviceManager.terminals.runningChanged.disconnect(this._onTerminalsRunningChanged,this)}_onSessionsRunningChanged(e,t){this.model.sessions=t.length}_onTerminalsRunningChanged(e,t){this.model.terminals=t.length}}!function(e){class t extends L.c{constructor(){super(...arguments),this._terminals=0,this._sessions=0}get sessions(){return this._sessions}set sessions(e){const t=this._sessions;this._sessions=e,t!==this._sessions&&this.stateChanged.emit(void 0)}get terminals(){return this._terminals}set terminals(e){const t=this._terminals;this._terminals=e,t!==this._terminals&&this.stateChanged.emit(void 0)}}e.Model=t}(Me||(Me={}));te.Widget;var Se;!function(e){e.statusItemDefaults={align:"left",rank:0,isActive:()=>!0,activeStateChanged:void 0}}(Se||(Se={}));new d.Token("@jupyterlab/statusbar:IStatusBar");function Te(e){return k.a.createElement(q,{source:e.mode,onClick:e.handleClick})}class Ee extends L.d{constructor(e){super(new Ee.Model),this._handleClick=()=>{const e=new te.Menu({commands:this._commands});this._popup&&this._popup.dispose(),I.getModeInfo().sort((e,t)=>{const n=e.name||"",r=t.name||"";return n.localeCompare(r)}).forEach(t=>{if(0===t.mode.indexOf("brainf"))return;const n={insertSpaces:!0,name:t.name};e.addItem({command:"codemirror:change-mode",args:n})}),this._popup=le({body:e,anchor:this,align:"left"})},this._popup=null,this._commands=e.commands,this.translator=e.translator||o.a;const t=this.translator.load("jupyterlab");this.addClass(ae),this.title.caption=t.__("Change text editor syntax highlighting")}render(){return this.model?k.a.createElement(Te,{mode:this.model.mode,handleClick:this._handleClick}):null}}!function(e){class t extends L.c{constructor(){super(...arguments),this._onMIMETypeChange=(e,t)=>{const n=this._mode,r=I.findByMIME(t.newValue);this._mode=r.name||r.mode,this._triggerChange(n,this._mode)},this._mode="",this._editor=null}get mode(){return this._mode}get editor(){return this._editor}set editor(e){const t=this._editor;null!==t&&t.model.mimeTypeChanged.disconnect(this._onMIMETypeChange);const n=this._mode;if(this._editor=e,null===this._editor)this._mode="";else{const e=I.findByMIME(this._editor.model.mimeType);this._mode=e.name||e.mode,this._editor.model.mimeTypeChanged.connect(this._onMIMETypeChange)}this._triggerChange(n,this._mode)}_triggerChange(e,t){e!==t&&this.stateChanged.emit(void 0)}}e.Model=t}(Ee||(Ee={}));new d.Token("@jupyterlab/codemirror:ICodeMirror"),new class{constructor(e={},t){this.newInlineEditor=e=>(e.host.dataset.type="inline",new S(Object.assign(Object.assign({},e),{config:Object.assign(Object.assign({},this.inlineCodeMirrorConfig),e.config||{}),translator:this.translator}))),this.newDocumentEditor=e=>(e.host.dataset.type="document",new S(Object.assign(Object.assign({},e),{config:Object.assign(Object.assign({},this.documentCodeMirrorConfig),e.config||{}),translator:this.translator}))),this.translator=t||o.a,this.inlineCodeMirrorConfig=Object.assign(Object.assign(Object.assign({},S.defaultConfig),{extraKeys:{"Cmd-Right":"goLineRight",End:"goLineRight","Cmd-Left":"goLineLeft",Tab:"indentMoreOrinsertTab","Shift-Tab":"indentLess","Cmd-/":"toggleComment","Ctrl-/":"toggleComment"}}),e),this.documentCodeMirrorConfig=Object.assign(Object.assign(Object.assign({},S.defaultConfig),{extraKeys:{Tab:"indentMoreOrinsertTab","Shift-Tab":"indentLess","Cmd-/":"toggleComment","Ctrl-/":"toggleComment","Shift-Enter":()=>{}},lineNumbers:!0,scrollPastEnd:!0}),e)}},new class{getMimeTypeByLanguage(e){const t=e.file_extension||"";return I.findBest(e.codemirror_mode||{mimetype:e.mimetype,name:e.name,ext:[t.split(".").slice(-1)[0]]}).mime}getMimeTypeByFilePath(e){const t=w.PathExt.extname(e);if(".ipy"===t)return"text/x-python";if(".md"===t)return"text/x-ipythongfm";return(I.findByFileName(e)||I.findBest("")).mime}};var Oe=n(1285),ke=n.n(Oe),Le=n(329);n.d(t,"a",(function(){return Ne})),n.d(t,"b",(function(){return je})),n.d(t,"c",(function(){return Re})),n.d(t,"d",(function(){return ze})),n.d(t,"e",(function(){return Fe})),n.d(t,"f",(function(){return Be}));var De,Pe=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(null!=e&&"function"==typeof Object.getOwnPropertySymbols){var i=0;for(r=Object.getOwnPropertySymbols(e);i<r.length;i++)t.indexOf(r[i])<0&&Object.prototype.propertyIsEnumerable.call(e,r[i])&&(n[r[i]]=e[r[i]])}return n};function Ne(e){let{host:t,source:n,trusted:r,sanitizer:i,resolver:a,linkHandler:s,shouldTypeset:l,latexTypesetter:u,translator:c}=e;c=c||o.a;const d=null==c?void 0:c.load("jupyterlab");let h,f=n;if(!n)return t.textContent="",Promise.resolve(void 0);if(r||(f=""+n,n=i.sanitize(n)),t.innerHTML=n,t.getElementsByTagName("script").length>0)if(r)De.evalInnerHTMLScriptTags(t);else{const e=document.createElement("div"),n=document.createElement("pre");n.textContent=d.__("This HTML output contains inline scripts. Are you sure that you want to run arbitrary Javascript within your JupyterLab session?");const r=document.createElement("button");r.textContent=d.__("Run"),r.onclick=e=>{t.innerHTML=f,De.evalInnerHTMLScriptTags(t),t.firstChild&&t.removeChild(t.firstChild)},e.appendChild(n),e.appendChild(r),t.insertBefore(e,t.firstChild)}return De.handleDefaults(t,a),h=a?De.handleUrls(t,a,s):Promise.resolve(void 0),h.then(()=>{l&&u&&u.typeset(t)})}function je(e){const{host:t,mimeType:n,source:r,width:i,height:o,needsBackground:a,unconfined:s}=e;t.textContent="";const l=document.createElement("img");return l.src=`data:${n};base64,${r}`,"number"==typeof o&&(l.height=o),"number"==typeof i&&(l.width=i),"light"===a?l.classList.add("jp-needs-light-background"):"dark"===a&&l.classList.add("jp-needs-dark-background"),!0===s&&l.classList.add("jp-mod-unconfined"),t.appendChild(l),Promise.resolve(void 0)}function Re(e){const{host:t,source:n,shouldTypeset:r,latexTypesetter:i}=e;return t.textContent=n,r&&i&&i.typeset(t),Promise.resolve(void 0)}async function ze(e){const{host:t,source:n}=e,r=Pe(e,["host","source"]);if(!n)return void(t.textContent="");const i=Object(Le.a)(n);let o=await De.renderMarked(i.text);o=Object(Le.b)(o,i.math),await Ne(Object.assign({host:t,source:o},r)),De.headerAnchors(t)}function Fe(e){let{host:t,source:n,trusted:r,unconfined:i}=e;if(!n)return t.textContent="",Promise.resolve(void 0);if(!r)return t.textContent="Cannot display an untrusted SVG. Maybe you need to run the cell?",Promise.resolve(void 0);n.search("<svg[^>]+xmlns=[^>]+svg")<0&&(n=n.replace("<svg",'<svg xmlns="http://www.w3.org/2000/svg"'));const o=new Image;return o.src="data:image/svg+xml,"+encodeURIComponent(n),t.appendChild(o),!0===i&&t.classList.add("jp-mod-unconfined"),Promise.resolve()}function Be(e){const{host:t,sanitizer:n,source:r}=e,i=n.sanitize(De.ansiSpan(r),{allowedTags:["span"]}),o=document.createElement("pre");return o.innerHTML=function(e){const t=new RegExp('(?:[a-zA-Z][a-zA-Z0-9+.-]{2,}:\\/\\/|data:|www\\.)[^\\s\\u0000-\\u0020\\u007f-\\u009f"]{2,}[^\\s\\u0000-\\u0020\\u007f-\\u009f"\'(){}\\[\\],:;.!?]',"ug");return e.replace(t,e=>{const t=e.slice(-3),n=-1!==["&gt","&lt"].indexOf(t),r=n?t:"",i=n?e.length-3:e.length;return`<a href="${e.slice(0,i)}" rel="noopener" target="_blank">${e.slice(0,i)}</a>${r}`})}(i),t.appendChild(o),Promise.resolve(void 0)}!function(e){async function t(e,t,n){const r=e.getAttribute(t)||"",i=n.isLocal?n.isLocal(r):w.URLExt.isLocal(r);if(r&&i)try{const i=await n.resolveUrl(r);let o=await n.getDownloadUrl(i);"data:"!==w.URLExt.parse(o).protocol&&(o+=(/\?/.test(o)?"&":"?")+(new Date).getTime()),e.setAttribute(t,o)}catch(n){throw e.setAttribute(t,""),n}}function n(e,t,n){let r=e.getAttribute("href")||"";const i=t.isLocal?t.isLocal(r):w.URLExt.isLocal(r);if(!r||!i)return Promise.resolve(void 0);const o=e.hash;if(o){if(o===r)return e.target="_self",Promise.resolve(void 0);r=r.replace(o,"")}return t.resolveUrl(r).then(r=>{const i=decodeURI(r);return n&&n.handleLink(e,i,o),t.getDownloadUrl(r)}).then(t=>{e.href=t+o}).catch(t=>{e.href=""})}e.evalInnerHTMLScriptTags=function(e){const t=Object(v.toArray)(e.getElementsByTagName("script"));for(const e of t){if(!e.parentNode)continue;const t=document.createElement("script"),n=e.attributes;for(let e=0,r=n.length;e<r;++e){const{name:r,value:i}=n[e];t.setAttribute(r,i)}t.textContent=e.textContent,e.parentNode.replaceChild(t,e)}},e.renderMarked=function(e){return function(){if(r)return;r=!0,i.a.setOptions({gfm:!0,sanitize:!1,langPrefix:`cm-s-${S.defaultConfig.theme} language-`,highlight:(e,t,n)=>{const r=(e,t)=>(n&&n(e,t),t);return t?(I.ensure(t).then(n=>{const i=document.createElement("div");if(!n)return console.error("No CodeMirror mode: "+t),r(null,e);try{return I.run(e,n.mime,i),r(null,i.innerHTML)}catch(n){return console.error(`Failed to highlight ${t} code`,n),r(n,e)}}).catch(n=>(console.error("No CodeMirror mode: "+t),console.error("Require CodeMirror mode error: "+n),r(null,e))),e):r(null,e)}})}(),new Promise((t,n)=>{i()(e,(e,r)=>{e?n(e):t(r)})})},e.handleDefaults=function(e,t){const n=e.getElementsByTagName("a");for(let e=0;e<n.length;e++){const r=n[e];if(!(r instanceof HTMLAnchorElement))continue;const i=r.href,o=t&&t.isLocal?t.isLocal(i):w.URLExt.isLocal(i);r.target||(r.target=o?"_self":"_blank"),o||(r.rel="noopener")}const r=e.getElementsByTagName("img");for(let e=0;e<r.length;e++)r[e].alt||(r[e].alt="Image")},e.handleUrls=function(e,r,i){const o=[],a=e.querySelectorAll("*[src]");for(let e=0;e<a.length;e++)o.push(t(a[e],"src",r));const s=e.getElementsByTagName("a");for(let e=0;e<s.length;e++)o.push(n(s[e],r,i));const l=e.getElementsByTagName("link");for(let e=0;e<l.length;e++)o.push(t(l[e],"href",r));return Promise.all(o).then(()=>{})},e.headerAnchors=function(e){var t;const n=["h1","h2","h3","h4","h5","h6"];for(const r of n){const n=e.getElementsByTagName(r);for(let e=0;e<n.length;e++){const r=n[e];r.id=(null!==(t=r.textContent)&&void 0!==t?t:"").replace(/ /g,"-");const i=document.createElement("a");i.target="_self",i.textContent="¶",i.href="#"+r.id,i.classList.add("jp-InternalAnchorLink"),r.appendChild(i)}}};let r=!1;const o=["ansi-black","ansi-red","ansi-green","ansi-yellow","ansi-blue","ansi-magenta","ansi-cyan","ansi-white","ansi-black-intense","ansi-red-intense","ansi-green-intense","ansi-yellow-intense","ansi-blue-intense","ansi-magenta-intense","ansi-cyan-intense","ansi-white-intense"];function a(e,t,n,r,i,a,s){if(e){const l=[],u=[];r&&"number"==typeof t&&0<=t&&t<8&&(t+=8),a&&([t,n]=[n,t]),"number"==typeof t?l.push(o[t]+"-fg"):t.length?u.push(`color: rgb(${t})`):a&&l.push("ansi-default-inverse-fg"),"number"==typeof n?l.push(o[n]+"-bg"):n.length?u.push(`background-color: rgb(${n})`):a&&l.push("ansi-default-inverse-bg"),r&&l.push("ansi-bold"),i&&l.push("ansi-underline"),l.length||u.length?(s.push("<span"),l.length&&s.push(` class="${l.join(" ")}"`),u.length&&s.push(` style="${u.join("; ")}"`),s.push(">"),s.push(e),s.push("</span>")):s.push(e)}}function s(e){let t,n,r;const i=e.shift();if(2===i&&e.length>=3){if(t=e.shift(),n=e.shift(),r=e.shift(),[t,n,r].some(e=>e<0||255<e))throw new RangeError("Invalid range for RGB colors")}else{if(!(5===i&&e.length>=1))throw new RangeError("Invalid extended color specification");{const i=e.shift();if(i<0)throw new RangeError("Color index must be >= 0");if(i<16)return i;if(i<232)t=Math.floor((i-16)/36),t=t>0?55+40*t:0,n=Math.floor((i-16)%36/6),n=n>0?55+40*n:0,r=(i-16)%6,r=r>0?55+40*r:0;else{if(!(i<256))throw new RangeError("Color index must be < 256");t=n=r=10*(i-232)+8}}}return[t,n,r]}e.ansiSpan=function(e){const t=/\x1b\[(.*?)([@-~])/g;let n,r=[],i=[],o=!1,l=!1,u=!1;const c=[],d=[];let h=0;for(e=ke()(e),e+="[m";n=t.exec(e);){if("m"===n[2]){const e=n[1].split(";");for(let t=0;t<e.length;t++){const n=e[t];if(""===n)d.push(0);else{if(-1===n.search(/^\d+$/)){d.length=0;break}d.push(parseInt(n,10))}}}for(a(e.substring(h,n.index),r,i,o,l,u,c),h=t.lastIndex;d.length;){const e=d.shift();switch(e){case 0:r=i=[],o=!1,l=!1,u=!1;break;case 1:case 5:o=!0;break;case 4:l=!0;break;case 7:u=!0;break;case 21:case 22:o=!1;break;case 24:l=!1;break;case 27:u=!1;break;case 30:case 31:case 32:case 33:case 34:case 35:case 36:case 37:r=e-30;break;case 38:try{r=s(d)}catch(e){d.length=0}break;case 39:r=[];break;case 40:case 41:case 42:case 43:case 44:case 45:case 46:case 47:i=e-40;break;case 48:try{i=s(d)}catch(e){d.length=0}break;case 49:i=[];break;case 90:case 91:case 92:case 93:case 94:case 95:case 96:case 97:r=e-90+8;break;case 100:case 101:case 102:case 103:case 104:case 105:case 106:case 107:i=e-100+8}}}return c.join("")}}(De||(De={}))},function(e,t,n){var r=n(281),i=n(497);e.exports=function(e){return null!=e&&i(e.length)&&!r(e)}},function(e,t,n){"use strict";n.r(t);var r=n(912);n.d(t,"default",(function(){return r.a}))},function(e,t,n){var r,i,o=this&&this.__rest||function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(null!=e&&"function"==typeof Object.getOwnPropertySymbols){var i=0;for(r=Object.getOwnPropertySymbols(e);i<r.length;i++)t.indexOf(r[i])<0&&Object.prototype.propertyIsEnumerable.call(e,r[i])&&(n[r[i]]=e[r[i]])}return n};r=[n,t,n(940),n(1191),n(519)],void 0===(i=function(e,t,n,r,i){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.dashboardUpdateComputedResult_=t.dashboardUpdateComputedResult=t.dashboardRemoveController_=t.dashboardRemoveController=t.dashboardUpdateController_=t.dashboardUpdateController=t.dashboardAddController_=t.dashboardAddController=t.dashboardSetController_=t.dashboardSetController=t.dashboardToggleRun=t.dashboardAdd3DData_=t.dashboardAdd3DData=t.archUpdateTreeData_=t.archUpdateTreeData=t.archRemoveNode_=t.archRemoveNode=t.archAddNode_=t.archAddNode=t.archSelectNode_=t.archSelectNode=t.archUpdateConnectionGraph_=t.archUpdateConnectionGraph=t.archFilterNode_=t.archFilterNode=t.archSaveGraphPosition_=t.archSaveGraphPosition=t.archSaveGraphJson_=t.archSaveGraphJson=t.serverGetData_=t.serverGetData=t.archLockGraph=t.mainSwitchPanel_=t.mainSwitchPanel=t.saveState_=t.saveState=t.resetStore=void 0,t.resetStore=function(){return{type:n.Action.RESET_STORE}},t.saveState=function(){return{type:n.Action.SAVE_STATE}},t.saveState_=function(e){const t=e.saveSignal;return Object.assign(Object.assign({},e),{saveSignal:t+1})},t.mainSwitchPanel=function(e){return{type:n.Action.SWITCH_PANEL,panelName:e}},t.mainSwitchPanel_=function(e,t){const n={panel_0:[!1,"Project_Overview"],panel_1:[!1,"System_Arc"],panel_2:[!1,"Module_Creator"],panel_3:[!1,"Dashboard"],panel_4:[!1,"User_Management"],panel_5:[!1,"Module_Library"]};for(const e in n)n.hasOwnProperty(e)&&(n[e][0]=t.panelName===n[e][1]);return Object.assign(Object.assign({},e),{mainState:n})},t.archLockGraph=function(e){return{type:n.Action.LOCK_GRAPH,status:e}},t.serverGetData=function(e){return{type:n.Action.GET_SERVER_DATA,data:e}},t.serverGetData_=function(e,t){return Object.assign(Object.assign({},e),{systemArch:Object.assign(Object.assign({},e.systemArch),{systemGraph:t.data.systemGraph,systemTree:Object.assign(Object.assign({},e.systemArch.systemTree),{nodeData:t.data.systemTree})}),dashboardState:Object.assign(Object.assign({},e.dashboardState),{variableData:t.data.variableData})})},t.archSaveGraphJson=function(e){return{type:n.Action.ARCH_SAVE_GRAPH_JSON,data:e}},t.archSaveGraphJson_=function(e,t){const n=t.data.connection,r=t.data.position,i=Object.assign({},e);for(const e in i.systemArch.systemGraph.systemGraphData)n.hasOwnProperty(e)&&(i.systemArch.systemGraph.systemGraphData[e].connections=n[e]),r.hasOwnProperty(e)&&(i.systemArch.systemGraph.systemGraphData[e].position=r[e]);const o=JSON.stringify(i,null,2),a=new Blob([o],{type:"application/json"}),s=window.URL.createObjectURL(a),l=document.createElement("a");return l.href=s,l.download="store.json",l.click(),Object.assign({},e)},t.archSaveGraphPosition=function(e){return{type:n.Action.ARCH_SAVE_GRAPH_POSITION,data:e}},t.archSaveGraphPosition_=function(e,t){return Object.assign(Object.assign({},e),{systemArch:Object.assign(Object.assign({},e.systemArch),{systemPBS:t.data})})},t.archFilterNode=function(e,t){return{type:n.Action.ARCH_FILTER_NODE,data:e,selected:t}},t.archFilterNode_=function(e,t){let n;n=JSON.parse(JSON.stringify(e.systemArch.systemPBS));const r=e.systemArch.systemPBSUpdated,i=t.selected;if(t.data.length>0){for(const e in n)n[e].visible=!1;t.data.forEach(e=>{n[e].visible=!0})}else for(const e in n)n[e].visible=!0;return Object.assign(Object.assign({},e),{systemArch:Object.assign(Object.assign({},e.systemArch),{systemTree:Object.assign(Object.assign({},e.systemArch.systemTree),{nodePath:i}),systemPBS:n,systemPBSUpdated:r+1})})},t.archUpdateConnectionGraph=function(e,t){return{type:n.Action.ARCH_UPDATE_CONNECTION_GRAPH,data:e,root:t}},t.archUpdateConnectionGraph_=function(e,t){const n=t.data,r=t.root,{systemGraphData:i,systemList:o}=Object.assign({},e.systemArch.systemGraph),a=Object.assign({},i);let s=[...o];const l=r.split(".");if(l.length>1){const e=l[l.length-1],t=r.replace("."+e,""),n=a[t].connections.filter(t=>!(t[0].includes(e)||t[1].includes(e)));a[t].connections=n}for(const[e,t]of Object.entries(n))e in i&&(a[t]=i[e],delete a[e]);s=Object.keys(a);const u=[];return e.systemArch.systemTree.nodePath.forEach(e=>{Object.keys(n).includes(e)&&u.push(n[e])}),Object.assign(Object.assign({},e),{systemArch:Object.assign(Object.assign({},e.systemArch),{systemTree:Object.assign(Object.assign({},e.systemArch.systemTree),{nodePath:u}),systemGraph:Object.assign(Object.assign({},e.systemArch.systemGraph),{systemGraphData:a,systemList:s,updateData:n})})})},t.archSelectNode=function(e){return{type:n.Action.SELECT_NODE,nodeId:e}},t.archSelectNode_=function(e,t){return Object.assign(Object.assign({},e),{systemArch:Object.assign(Object.assign({},e.systemArch),{systemTree:Object.assign(Object.assign({},e.systemArch.systemTree),{selectedNode:t.nodeId})})})},t.archAddNode=function(e,t,r){return{type:n.Action.ADD_NODE,nodeName:e,nodePath:t,nodeData:r}},t.archAddNode_=function(e,t){const n=t.nodePath,o=i.getNodeFullPath(e.systemArch.systemTree.nodeData,n),a=r.addNodeUnderParent({treeData:e.systemArch.systemTree.nodeData,parentKey:n[n.length-1],expandParent:!0,getNodeKey:e=>e.treeIndex,newNode:{title:t.nodeName}}).treeData,s=t.nodeName,l=o.concat("."+s),u=["inwards"],c=["outwards"];for(let e=0;e<t.nodeData.portList.length;e++){const n=t.nodeData.portList[e];0===n.portDirection?u.push(n.portName):c.push(n.portName)}return Object.assign(Object.assign({},e),{systemArch:Object.assign(Object.assign({},e.systemArch),{systemTree:Object.assign(Object.assign({},e.systemArch.systemTree),{nodeData:a}),systemGraph:Object.assign(Object.assign({},e.systemArch.systemGraph),{systemGraphData:Object.assign(Object.assign({},e.systemArch.systemGraph.systemGraphData),{[l]:{inPort:u,outPort:c,connections:[]}}),systemList:[...e.systemArch.systemGraph.systemList,l]}),systemData:Object.assign(Object.assign({},e.systemArch.systemData),{mainData:Object.assign(Object.assign({},e.systemArch.systemData.mainData),{[l]:{user:t.nodeData.personName,class:t.nodeData.sysClass,portList:u.concat(c)}}),portData:Object.assign({},e.systemArch.systemData.portData)})})})},t.archRemoveNode=function(e,t){return{type:n.Action.REMOVE_NODE,nodeData:e,nodePath:t}},t.archRemoveNode_=function(e,t){const n=t.nodePath,o=i.getNodeFullPath(e.systemArch.systemTree.nodeData,n),a=[],s={},l={},u=r.removeNodeAtPath({treeData:e.systemArch.systemTree.nodeData,path:n,getNodeKey:e=>e.treeIndex});return e.systemArch.systemGraph.systemList.forEach(t=>{t.startsWith(o)||(a.push(t),e.systemArch.systemGraph.systemGraphData.hasOwnProperty(t)&&(s[t]=e.systemArch.systemGraph.systemGraphData[t],l[t]=e.systemArch.systemData.mainData[t]))}),Object.assign(Object.assign({},e),{systemArch:Object.assign(Object.assign({},e.systemArch),{systemTree:Object.assign(Object.assign({},e.systemArch.systemTree),{nodeData:u}),systemGraph:Object.assign(Object.assign({},e.systemArch.systemGraph),{systemList:a,systemGraphData:s}),systemData:Object.assign(Object.assign({},e.systemArch.systemData),{mainData:l,portData:Object.assign({},e.systemArch.systemData.portData)})})})},t.archUpdateTreeData=function(e,t=!1){return{type:n.Action.UPDATE_TREE,treeData:e,updatePbs:t}},t.archUpdateTreeData_=function(e,t){let n=e.systemArch.systemPBSUpdated;return t.updatePbs&&(n+=1),Object.assign(Object.assign({},e),{systemArch:Object.assign(Object.assign({},e.systemArch),{systemTree:Object.assign(Object.assign({},e.systemArch.systemTree),{nodeData:t.treeData}),systemPBSUpdated:n})})},t.dashboardAdd3DData=function(e){return{type:n.Action.DASHBOARD_ADD_3D_DATA,data:e}},t.dashboardAdd3DData_=function(e,t){const n=["None",...t.data];return Object.assign(Object.assign({},e),{dashboardState:Object.assign(Object.assign({},e.dashboardState),{timeStepList:n})})},t.dashboardToggleRun=function(){return{type:n.Action.DASHBOARD_TOGGLE_RUN}},t.dashboardSetController=function(e){return{type:n.Action.DASHBOARD_SET_CONTROLLER,data:e}},t.dashboardSetController_=function(e,t){return Object.assign(Object.assign({},e),{dashboardState:Object.assign(Object.assign({},e.dashboardState),{selectedVariable:t.data})})},t.dashboardAddController=function(e,t){return{type:n.Action.DASHBOARD_ADD_CONTROLLER,variableName:e,value:t}},t.dashboardAddController_=function(e,t){if(Object.keys(e.dashboardState.selectedVariable).includes(t.variableName))return e;{const n=Object.assign(Object.assign({},e.dashboardState.selectedVariable),{[t.variableName]:t.value});return Object.assign(Object.assign({},e),{dashboardState:Object.assign(Object.assign({},e.dashboardState),{selectedVariable:n})})}},t.dashboardUpdateController=function(e,t){return{type:n.Action.DASHBOARD_UPDATE_CONTROLLER,variableName:e,value:t}},t.dashboardUpdateController_=function(e,t){return e.dashboardState.selectedVariable.hasOwnProperty(t.variableName)?Object.assign(Object.assign({},e),{dashboardState:Object.assign(Object.assign({},e.dashboardState),{selectedVariable:Object.assign(Object.assign({},e.dashboardState.selectedVariable),{[t.variableName]:t.value})})}):e},t.dashboardRemoveController=function(e){return{type:n.Action.DASHBOARD_REMOVE_CONTROLLER,variableName:e}},t.dashboardRemoveController_=function(e,t){const n=e.dashboardState.selectedVariable,r=t.variableName,i=(n[r],o(n,["symbol"==typeof r?r:r+""]));return Object.assign(Object.assign({},e),{dashboardState:Object.assign(Object.assign({},e.dashboardState),{selectedVariable:i})})},t.dashboardUpdateComputedResult=function(e){return{type:n.Action.DASHBOARD_UPDATE_COMPUTED_RESULT,data:e}},t.dashboardUpdateComputedResult_=function(e,t){const n=t.data.variable,r=t.data.recorder,i=t.data.driver;return Object.assign(Object.assign({},e),{dashboardState:Object.assign(Object.assign({},e.dashboardState),{computedResult:n,recorderData:r,driverData:i})})}}.apply(t,r))||(e.exports=i)},function(e,t,n){"use strict";['Variables are inserted using %{variable}, for example "y: %{y}".','Numbers are formatted using d3-format\'s syntax %{variable:d3-format}, for example "Price: %{y:$.2f}".',n(194).FORMAT_LINK,"for details on the formatting syntax.",'Dates are formatted using d3-time-format\'s syntax %{variable|d3-time-format}, for example "Day: %{2019-01-01|%A}".',n(194).DATE_FORMAT_LINK,"for details on the date formatting syntax."].join(" ");function r(e){var t=e.description?" "+e.description:"",n=e.keys||[];if(n.length>0){for(var r=[],i=0;i<n.length;i++)r[i]="`"+n[i]+"`";t+="Finally, the template string has access to ",t=1===n.length?"variable "+r[0]:"variables "+r.slice(0,-1).join(", ")+" and "+r.slice(-1)+"."}return t}t.hovertemplateAttrs=function(e,t){e=e||{};r(t=t||{});var n={valType:"string",dflt:"",editType:e.editType||"none"};return!1!==e.arrayOk&&(n.arrayOk=!0),n},t.texttemplateAttrs=function(e,t){e=e||{};r(t=t||{});var n={valType:"string",dflt:"",editType:e.editType||"calc"};return!1!==e.arrayOk&&(n.arrayOk=!0),n}},function(e,t,n){"use strict";var r=n(10),i=n(64);e.exports=function(e,t,n){var o,a,s=n.name,l=n.inclusionAttr||"visible",u=t[s],c=r.isArrayOrTypedArray(e[s])?e[s]:[],d=t[s]=[],h=i.arrayTemplater(t,s,l);for(o=0;o<c.length;o++){var f=c[o];r.isPlainObject(f)?a=h.newItem(f):(a=h.newItem({}))[l]=!1,a._index=o,!1!==a[l]&&n.handleItemDefaults(f,a,t,n),d.push(a)}var p=h.defaultItems();for(o=0;o<p.length;o++)(a=p[o])._index=d.length,n.handleItemDefaults({},a,t,n,{}),d.push(a);if(r.isArrayOrTypedArray(u)){var g=Math.min(u.length,d.length);for(o=0;o<g;o++)r.relinkPrivateKeys(d[o],u[o])}return d}},function(e,t,n){"use strict";var r=n(39),i=n(10),o=n(163),a=n(162),s=n(250),l=n(993);e.exports={moduleType:"component",name:"fx",constants:n(381),schema:{layout:s},attributes:n(524),layoutAttributes:s,supplyLayoutGlobalDefaults:n(1694),supplyDefaults:n(1695),supplyLayoutDefaults:n(1696),calc:n(1697),getDistanceFunction:a.getDistanceFunction,getClosest:a.getClosest,inbox:a.inbox,quadrature:a.quadrature,appendArrayPointValue:a.appendArrayPointValue,castHoverOption:function(e,t,n){return i.castOption(e,t,"hoverlabel."+n)},castHoverinfo:function(e,t,n){return i.castOption(e,n,"hoverinfo",(function(n){return i.coerceHoverinfo({hoverinfo:n},{_module:e._module},t)}))},hover:l.hover,unhover:o.unhover,loneHover:l.loneHover,loneUnhover:function(e){var t=i.isD3Selection(e)?e:r.select(e);t.selectAll("g.hovertext").remove(),t.selectAll(".spikeline").remove()},click:n(1698)}},function(e,t,n){"use strict";var r=n(112),i=n(1035),o=n(216),a=["uint8","uint8_clamped","uint16","uint32","int8","int16","int32","float32"];function s(e,t,n,r,i){this.gl=e,this.type=t,this.handle=n,this.length=r,this.usage=i}var l=s.prototype;function u(e,t,n,r,i,o){var a=i.length*i.BYTES_PER_ELEMENT;if(o<0)return e.bufferData(t,i,r),a;if(a+o>n)throw new Error("gl-buffer: If resizing buffer, must not specify offset");return e.bufferSubData(t,o,i),n}function c(e,t){for(var n=r.malloc(e.length,t),i=e.length,o=0;o<i;++o)n[o]=e[o];return n}l.bind=function(){this.gl.bindBuffer(this.type,this.handle)},l.unbind=function(){this.gl.bindBuffer(this.type,null)},l.dispose=function(){this.gl.deleteBuffer(this.handle)},l.update=function(e,t){if("number"!=typeof t&&(t=-1),this.bind(),"object"==typeof e&&void 0!==e.shape){var n=e.dtype;if(a.indexOf(n)<0&&(n="float32"),this.type===this.gl.ELEMENT_ARRAY_BUFFER)n=gl.getExtension("OES_element_index_uint")&&"uint16"!==n?"uint32":"uint16";if(n===e.dtype&&function(e,t){for(var n=1,r=t.length-1;r>=0;--r){if(t[r]!==n)return!1;n*=e[r]}return!0}(e.shape,e.stride))0===e.offset&&e.data.length===e.shape[0]?this.length=u(this.gl,this.type,this.length,this.usage,e.data,t):this.length=u(this.gl,this.type,this.length,this.usage,e.data.subarray(e.offset,e.shape[0]),t);else{var s=r.malloc(e.size,n),l=o(s,e.shape);i.assign(l,e),this.length=u(this.gl,this.type,this.length,this.usage,t<0?s:s.subarray(0,e.size),t),r.free(s)}}else if(Array.isArray(e)){var d;d=this.type===this.gl.ELEMENT_ARRAY_BUFFER?c(e,"uint16"):c(e,"float32"),this.length=u(this.gl,this.type,this.length,this.usage,t<0?d:d.subarray(0,e.length),t),r.free(d)}else if("object"==typeof e&&"number"==typeof e.length)this.length=u(this.gl,this.type,this.length,this.usage,e,t);else{if("number"!=typeof e&&void 0!==e)throw new Error("gl-buffer: Invalid data type");if(t>=0)throw new Error("gl-buffer: Cannot specify offset when resizing buffer");(e|=0)<=0&&(e=1),this.gl.bufferData(this.type,0|e,this.usage),this.length=e}},e.exports=function(e,t,n,r){if(n=n||e.ARRAY_BUFFER,r=r||e.DYNAMIC_DRAW,n!==e.ARRAY_BUFFER&&n!==e.ELEMENT_ARRAY_BUFFER)throw new Error("gl-buffer: Invalid type for webgl buffer, must be either gl.ARRAY_BUFFER or gl.ELEMENT_ARRAY_BUFFER");if(r!==e.DYNAMIC_DRAW&&r!==e.STATIC_DRAW&&r!==e.STREAM_DRAW)throw new Error("gl-buffer: Invalid usage for buffer, must be either gl.DYNAMIC_DRAW, gl.STATIC_DRAW or gl.STREAM_DRAW");var i=e.createBuffer(),o=new s(e,n,i,0,r);return o.update(t),o}},function(e,t,n){"use strict";var r=n(1796),i=n(1797);function o(e){this.bindVertexArrayOES=e.bindVertexArray.bind(e),this.createVertexArrayOES=e.createVertexArray.bind(e),this.deleteVertexArrayOES=e.deleteVertexArray.bind(e)}e.exports=function(e,t,n,a){var s,l=e.createVertexArray?new o(e):e.getExtension("OES_vertex_array_object");return(s=l?r(e,l):i(e)).update(t,n,a),s}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.Action=t.InputType=void 0;const r=n(401);!function(e){e.MOUSE_DOWN="mouse-down",e.MOUSE_UP="mouse-up",e.MOUSE_MOVE="mouse-move",e.MOUSE_WHEEL="mouse-wheel",e.KEY_DOWN="key-down",e.KEY_UP="key-up"}(t.InputType||(t.InputType={}));t.Action=class{constructor(e){this.options=e,this.id=r.Toolkit.UID()}setEngine(e){this.engine=e}}},function(e,t,n){var r=n(2104),i=n(2107),o=n(220),a=n(81),s=n(2114);e.exports=function(e){return"function"==typeof e?e:null==e?o:"object"==typeof e?a(e)?i(e[0],e[1]):r(e):s(e)}},function(e,t,n){"use strict";n.r(t);var r=n(908);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";n.r(t);var r=n(931);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";n.d(t,"a",(function(){return i}));var r=n(0);function i(e){var t=r.useState(e),n=t[0],i=t[1],o=e||n;return r.useEffect((function(){null==n&&i("mui-".concat(Math.round(1e5*Math.random())))}),[n]),o}},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(4)),s=n(7),l=n(15),u={h1:"h1",h2:"h2",h3:"h3",h4:"h4",h5:"h5",h6:"h6",subtitle1:"h6",subtitle2:"h6",body1:"p",body2:"p"},c=o.forwardRef((function(e,t){var n=e.align,s=void 0===n?"inherit":n,c=e.classes,d=e.className,h=e.color,f=void 0===h?"initial":h,p=e.component,g=e.display,v=void 0===g?"initial":g,m=e.gutterBottom,y=void 0!==m&&m,b=e.noWrap,C=void 0!==b&&b,A=e.paragraph,_=void 0!==A&&A,I=e.variant,x=void 0===I?"body1":I,w=e.variantMapping,M=void 0===w?u:w,S=Object(i.a)(e,["align","classes","className","color","component","display","gutterBottom","noWrap","paragraph","variant","variantMapping"]),T=p||(_?"p":M[x]||u[x])||"span";return o.createElement(T,Object(r.a)({className:Object(a.a)(c.root,d,"inherit"!==x&&c[x],"initial"!==f&&c["color".concat(Object(l.a)(f))],C&&c.noWrap,y&&c.gutterBottom,_&&c.paragraph,"inherit"!==s&&c["align".concat(Object(l.a)(s))],"initial"!==v&&c["display".concat(Object(l.a)(v))]),ref:t},S))}));t.a=Object(s.a)((function(e){return{root:{margin:0},body2:e.typography.body2,body1:e.typography.body1,caption:e.typography.caption,button:e.typography.button,h1:e.typography.h1,h2:e.typography.h2,h3:e.typography.h3,h4:e.typography.h4,h5:e.typography.h5,h6:e.typography.h6,subtitle1:e.typography.subtitle1,subtitle2:e.typography.subtitle2,overline:e.typography.overline,srOnly:{position:"absolute",height:1,width:1,overflow:"hidden"},alignLeft:{textAlign:"left"},alignCenter:{textAlign:"center"},alignRight:{textAlign:"right"},alignJustify:{textAlign:"justify"},noWrap:{overflow:"hidden",textOverflow:"ellipsis",whiteSpace:"nowrap"},gutterBottom:{marginBottom:"0.35em"},paragraph:{marginBottom:16},colorInherit:{color:"inherit"},colorPrimary:{color:e.palette.primary.main},colorSecondary:{color:e.palette.secondary.main},colorTextPrimary:{color:e.palette.text.primary},colorTextSecondary:{color:e.palette.text.secondary},colorError:{color:e.palette.error.main},displayInline:{display:"inline"},displayBlock:{display:"block"}}}),{name:"MuiTypography"})(c)},function(e,t){e.exports=function(e){return e.webpackPolyfill||(e.deprecate=function(){},e.paths=[],e.children||(e.children=[]),Object.defineProperty(e,"loaded",{enumerable:!0,get:function(){return e.l}}),Object.defineProperty(e,"id",{enumerable:!0,get:function(){return e.i}}),e.webpackPolyfill=1),e}},function(e,t,n){var r=n(860),i=n(499),o=n(174);e.exports=function(e){return o(e)?r(e):i(e)}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(111),i=n(131),o=function(){function e(t,n,r){this._name=t,this._orientation=n,this._indexPlus=r,e.values[this._name]=this}return e.getByName=function(t){return e.values[t]},e.getLocation=function(t,n,r){return n<t.x+t.width/4?e.LEFT:n>t.getRight()-t.width/4?e.RIGHT:r<t.y+t.height/4?e.TOP:r>t.getBottom()-t.height/4?e.BOTTOM:e.CENTER},e.prototype.getName=function(){return this._name},e.prototype.getOrientation=function(){return this._orientation},e.prototype.getDockRect=function(t){return this===e.TOP?new i.default(t.x,t.y,t.width,t.height/2):this===e.BOTTOM?new i.default(t.x,t.getBottom()-t.height/2,t.width,t.height/2):this===e.LEFT?new i.default(t.x,t.y,t.width/2,t.height):this===e.RIGHT?new i.default(t.getRight()-t.width/2,t.y,t.width/2,t.height):t.clone()},e.prototype.split=function(t,n){return this===e.TOP?{start:new i.default(t.x,t.y,t.width,n),end:new i.default(t.x,t.y+n,t.width,t.height-n)}:this===e.LEFT?{start:new i.default(t.x,t.y,n,t.height),end:new i.default(t.x+n,t.y,t.width-n,t.height)}:this===e.RIGHT?{start:new i.default(t.getRight()-n,t.y,n,t.height),end:new i.default(t.x,t.y,t.width-n,t.height)}:{start:new i.default(t.x,t.getBottom()-n,t.width,n),end:new i.default(t.x,t.y,t.width,t.height-n)}},e.prototype.reflect=function(){return this===e.TOP?e.BOTTOM:this===e.LEFT?e.RIGHT:this===e.RIGHT?e.LEFT:e.TOP},e.prototype.toString=function(){return"(DockLocation: name="+this._name+", orientation="+this._orientation+")"},e.values={},e.TOP=new e("top",r.default.VERT,0),e.BOTTOM=new e("bottom",r.default.VERT,1),e.LEFT=new e("left",r.default.HORZ,0),e.RIGHT=new e("right",r.default.HORZ,1),e.CENTER=new e("center",r.default.VERT,0),e}();t.default=o},function(e,t,n){"use strict";var r,i=this&&this.__extends||(r=function(e,t){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(e[n]=t[n])})(e,t)},function(e,t){function n(){this.constructor=e}r(e,t),e.prototype=null===t?Object.create(t):(n.prototype=t.prototype,new n)});Object.defineProperty(t,"__esModule",{value:!0});var o=n(289),a=n(244),s=n(190),l=n(370),u=n(111),c=n(131),d=n(245),h=n(516),f=n(290),p=n(934),g=function(e){function t(n,r){var i=e.call(this,n)||this;return t._attributeDefinitions.fromJson(r,i._attributes),n._addNode(i),i.calculatedTabBarHeight=0,i.calculatedHeaderBarHeight=0,i}return i(t,e),t._fromJson=function(e,n){var r=new t(n,e);return null!=e.children&&e.children.forEach((function(e){var t=f.default._fromJson(e,n);r._addChild(t)})),e.maximized&&!0===e.maximized&&n._setMaximizedTabset(r),e.active&&!0===e.active&&n._setActiveTabset(r),r},t._createAttributeDefinitions=function(){var e=new a.default;return e.add("type",t.TYPE,!0),e.add("id",void 0).setType(o.default.ID),e.add("weight",100),e.add("width",void 0),e.add("height",void 0),e.add("selected",0),e.add("name",void 0).setType(o.default.STRING),e.addInherited("enableDeleteWhenEmpty","tabSetEnableDeleteWhenEmpty"),e.addInherited("enableDrop","tabSetEnableDrop"),e.addInherited("enableDrag","tabSetEnableDrag"),e.addInherited("enableDivide","tabSetEnableDivide"),e.addInherited("enableMaximize","tabSetEnableMaximize"),e.addInherited("classNameTabStrip","tabSetClassNameTabStrip"),e.addInherited("classNameHeader","tabSetClassNameHeader"),e.addInherited("enableTabStrip","tabSetEnableTabStrip"),e.addInherited("borderInsets","tabSetBorderInsets"),e.addInherited("marginInsets","tabSetMarginInsets"),e.addInherited("minWidth","tabSetMinWidth"),e.addInherited("minHeight","tabSetMinHeight"),e.addInherited("headerHeight","tabSetHeaderHeight"),e.addInherited("tabStripHeight","tabSetTabStripHeight"),e.addInherited("tabLocation","tabSetTabLocation"),e.addInherited("autoSelectTab","tabSetAutoSelectTab").setType(o.default.BOOLEAN),e},t.prototype.getName=function(){return this._getAttributeAsStringOrUndefined("name")},t.prototype.getSelected=function(){var e=this._attributes.selected;return void 0!==e?e:-1},t.prototype.getSelectedNode=function(){var e=this.getSelected();if(-1!==e)return this._children[e]},t.prototype.getWeight=function(){return this._attributes.weight},t.prototype.getWidth=function(){return this._getAttributeAsNumberOrUndefined("width")},t.prototype.getMinWidth=function(){return this._getAttr("minWidth")},t.prototype.getHeight=function(){return this._getAttributeAsNumberOrUndefined("height")},t.prototype.getMinHeight=function(){return this._getAttr("minHeight")},t.prototype.getMinSize=function(e){return e===u.default.HORZ?this.getMinWidth():this.getMinHeight()},t.prototype.isMaximized=function(){return this._model.getMaximizedTabset()===this},t.prototype.isActive=function(){return this._model.getActiveTabset()===this},t.prototype.isEnableDeleteWhenEmpty=function(){return this._getAttr("enableDeleteWhenEmpty")},t.prototype.isEnableDrop=function(){return this._getAttr("enableDrop")},t.prototype.isEnableDrag=function(){return this._getAttr("enableDrag")},t.prototype.isEnableDivide=function(){return this._getAttr("enableDivide")},t.prototype.isEnableMaximize=function(){return this._getAttr("enableMaximize")},t.prototype.isEnableTabStrip=function(){return this._getAttr("enableTabStrip")},t.prototype.isAutoSelectTab=function(){return this._getAttr("autoSelectTab")},t.prototype.getClassNameTabStrip=function(){return this._getAttributeAsStringOrUndefined("classNameTabStrip")},t.prototype.getClassNameHeader=function(){return this._getAttributeAsStringOrUndefined("classNameHeader")},t.prototype.calculateHeaderBarHeight=function(e){var t=this._getAttr("headerHeight");this.calculatedHeaderBarHeight=0!==t?t:e.headerBarSize},t.prototype.calculateTabBarHeight=function(e){var t=this._getAttr("tabStripHeight");this.calculatedTabBarHeight=0!==t?t:e.tabBarSize},t.prototype.getHeaderHeight=function(){return this.calculatedHeaderBarHeight},t.prototype.getTabStripHeight=function(){return this.calculatedTabBarHeight},t.prototype.getTabLocation=function(){return this._getAttr("tabLocation")},t.prototype._setWeight=function(e){this._attributes.weight=e},t.prototype._setSelected=function(e){this._attributes.selected=e},t.prototype.canDrop=function(e,t,n){var r;if(e===this){var i=s.default.CENTER,o=this._tabHeaderRect;r=new l.default(this,o,i,-1,"flexlayout__outline_rect")}else if(this._contentRect.contains(t,n)){o=(i=s.default.getLocation(this._contentRect,t,n)).getDockRect(this._rect);r=new l.default(this,o,i,-1,"flexlayout__outline_rect")}else if(this._children.length>0&&null!=this._tabHeaderRect&&this._tabHeaderRect.contains(t,n)){for(var a=this._children[0],u=a.getTabRect(),d=u.y,h=u.height,f=this._tabHeaderRect.x,p=0,g=0;g<this._children.length;g++){if(p=(u=(a=this._children[g]).getTabRect()).x+u.width/2,t>=f&&t<p){i=s.default.CENTER,o=new c.default(u.x-2,d,3,h);r=new l.default(this,o,i,g,"flexlayout__outline_rect");break}f=p}if(null==r){i=s.default.CENTER,o=new c.default(u.getRight()-2,d,3,h);r=new l.default(this,o,i,this._children.length,"flexlayout__outline_rect")}}if(e._canDockInto(e,r))return r},t.prototype._layout=function(e,t){var n=this;this.calculateHeaderBarHeight(t),this.calculateTabBarHeight(t),this.isMaximized()&&(e=this._model.getRoot().getRect()),e=e.removeInsets(this._getAttr("marginInsets")),this._rect=e,e=e.removeInsets(this._getAttr("borderInsets"));var r=0,i=0;void 0!==this.getName()&&(r+=this.calculatedHeaderBarHeight,i+=this.calculatedHeaderBarHeight),this.isEnableTabStrip()&&("top"===this.getTabLocation()?this._tabHeaderRect=new c.default(e.x,e.y+r,e.width,this.calculatedTabBarHeight):this._tabHeaderRect=new c.default(e.x,e.y+e.height-this.calculatedTabBarHeight,e.width,this.calculatedTabBarHeight),i+=this.calculatedTabBarHeight,"top"===this.getTabLocation()&&(r+=this.calculatedTabBarHeight)),this._contentRect=new c.default(e.x,e.y+r,e.width,e.height-i),this._children.forEach((function(e,r){e._layout(n._contentRect,t),e._setVisible(r===n.getSelected())}))},t.prototype._remove=function(e){var t=this._removeChild(e);this._model._tidy(),p.adjustSelectedIndex(this,t)},t.prototype.drop=function(e,n,r,i){var o=this,a=n;if(this!==e){var l=e.getParent(),u=0;if(void 0!==l&&(u=l._removeChild(e),p.adjustSelectedIndex(l,u)),e.getType()===f.default.TYPE&&l===this&&u<r&&r>0&&r--,a===s.default.CENTER){var c=r;-1===c&&(c=this._children.length),e.getType()===f.default.TYPE?(this._addChild(e,c),(i||!1!==i&&this.isAutoSelectTab())&&this._setSelected(c)):e.getChildren().forEach((function(e,t){o._addChild(e,c),c++})),this._model._setActiveTabset(this)}else{var d=void 0;e instanceof f.default?((d=new t(this._model,{}))._addChild(e),l=d):d=e;var g=this._parent,v=g.getChildren().indexOf(this);if(g.getOrientation()===a._orientation)d._setWeight(this.getWeight()/2),this._setWeight(this.getWeight()/2),g._addChild(d,v+a._indexPlus);else{var m=new h.default(this._model,{});m._setWeight(this.getWeight()),m._addChild(this),this._setWeight(50),d._setWeight(50),m._addChild(d,a._indexPlus),g._removeChild(this),g._addChild(m,v)}this._model._setActiveTabset(d)}this._model._tidy()}},t.prototype._toJson=function(){var e={};return t._attributeDefinitions.toJson(e,this._attributes),e.children=this._children.map((function(e){return e._toJson()})),this.isActive()&&(e.active=!0),this.isMaximized()&&(e.maximized=!0),e},t.prototype._updateAttrs=function(e){t._attributeDefinitions.update(e,this._attributes)},t.prototype._getAttributeDefinitions=function(){return t._attributeDefinitions},t.prototype._getPrefSize=function(e){var t=this.getWidth();return e===u.default.VERT&&(t=this.getHeight()),t},t.TYPE="tabset",t._attributeDefinitions=t._createAttributeDefinitions(),t}(d.default);t.default=g},function(e,t,n){"use strict";n.r(t);var r=n(632);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";var r=n(177).texttemplateAttrs,i=n(177).hovertemplateAttrs,o=n(161),a=n(94),s=n(296).dash,l=n(56),u=n(544),c=n(63).extendFlat;e.exports={x:{valType:"data_array",editType:"calc+clearAxisTypes",anim:!0},x0:{valType:"any",dflt:0,editType:"calc+clearAxisTypes",anim:!0},dx:{valType:"number",dflt:1,editType:"calc",anim:!0},y:{valType:"data_array",editType:"calc+clearAxisTypes",anim:!0},y0:{valType:"any",dflt:0,editType:"calc+clearAxisTypes",anim:!0},dy:{valType:"number",dflt:1,editType:"calc",anim:!0},xperiod:{valType:"any",dflt:0,editType:"calc"},yperiod:{valType:"any",dflt:0,editType:"calc"},xperiod0:{valType:"any",editType:"calc"},yperiod0:{valType:"any",editType:"calc"},xperiodalignment:{valType:"enumerated",values:["start","middle","end"],dflt:"middle",editType:"calc"},yperiodalignment:{valType:"enumerated",values:["start","middle","end"],dflt:"middle",editType:"calc"},stackgroup:{valType:"string",dflt:"",editType:"calc"},orientation:{valType:"enumerated",values:["v","h"],editType:"calc"},groupnorm:{valType:"enumerated",values:["","fraction","percent"],dflt:"",editType:"calc"},stackgaps:{valType:"enumerated",values:["infer zero","interpolate"],dflt:"infer zero",editType:"calc"},text:{valType:"string",dflt:"",arrayOk:!0,editType:"calc"},texttemplate:r({},{}),hovertext:{valType:"string",dflt:"",arrayOk:!0,editType:"style"},mode:{valType:"flaglist",flags:["lines","markers","text"],extras:["none"],editType:"calc"},hoveron:{valType:"flaglist",flags:["points","fills"],editType:"style"},hovertemplate:i({},{keys:u.eventDataKeys}),line:{color:{valType:"color",editType:"style",anim:!0},width:{valType:"number",min:0,dflt:2,editType:"style",anim:!0},shape:{valType:"enumerated",values:["linear","spline","hv","vh","hvh","vhv"],dflt:"linear",editType:"plot"},smoothing:{valType:"number",min:0,max:1.3,dflt:1,editType:"plot"},dash:c({},s,{editType:"style"}),simplify:{valType:"boolean",dflt:!0,editType:"plot"},editType:"plot"},connectgaps:{valType:"boolean",dflt:!1,editType:"calc"},cliponaxis:{valType:"boolean",dflt:!0,editType:"plot"},fill:{valType:"enumerated",values:["none","tozeroy","tozerox","tonexty","tonextx","toself","tonext"],editType:"calc"},fillcolor:{valType:"color",editType:"style",anim:!0},marker:c({symbol:{valType:"enumerated",values:l.symbolList,dflt:"circle",arrayOk:!0,editType:"style"},opacity:{valType:"number",min:0,max:1,arrayOk:!0,editType:"style",anim:!0},size:{valType:"number",min:0,dflt:6,arrayOk:!0,editType:"calc",anim:!0},maxdisplayed:{valType:"number",min:0,dflt:0,editType:"plot"},sizeref:{valType:"number",dflt:1,editType:"calc"},sizemin:{valType:"number",min:0,dflt:0,editType:"calc"},sizemode:{valType:"enumerated",values:["diameter","area"],dflt:"diameter",editType:"calc"},line:c({width:{valType:"number",min:0,arrayOk:!0,editType:"style",anim:!0},editType:"calc"},o("marker.line",{anim:!0})),gradient:{type:{valType:"enumerated",values:["radial","horizontal","vertical","none"],arrayOk:!0,dflt:"none",editType:"calc"},color:{valType:"color",arrayOk:!0,editType:"calc"},editType:"calc"},editType:"calc"},o("marker",{anim:!0})),selected:{marker:{opacity:{valType:"number",min:0,max:1,editType:"style"},color:{valType:"color",editType:"style"},size:{valType:"number",min:0,editType:"style"},editType:"style"},textfont:{color:{valType:"color",editType:"style"},editType:"style"},editType:"style"},unselected:{marker:{opacity:{valType:"number",min:0,max:1,editType:"style"},color:{valType:"color",editType:"style"},size:{valType:"number",min:0,editType:"style"},editType:"style"},textfont:{color:{valType:"color",editType:"style"},editType:"style"},editType:"style"},textposition:{valType:"enumerated",values:["top left","top center","top right","middle left","middle center","middle right","bottom left","bottom center","bottom right"],dflt:"middle center",arrayOk:!0,editType:"calc"},textfont:a({editType:"calc",colorEditType:"style",arrayOk:!0}),r:{valType:"data_array",editType:"calc"},t:{valType:"data_array",editType:"calc"}}},function(e,t,n){"use strict";e.exports={FORMAT_LINK:"https://github.com/d3/d3-3.x-api-reference/blob/master/Formatting.md#d3_format",DATE_FORMAT_LINK:"https://github.com/d3/d3-time-format#locale_format"}},function(e,t,n){"use strict";var r=n(251),i=n(149);e.exports={moduleType:"component",name:"colorscale",attributes:n(161),layoutAttributes:n(984),supplyLayoutDefaults:n(1679),handleDefaults:n(213),crossTraceDefaults:n(1680),calc:n(298),scales:r.scales,defaultScale:r.defaultScale,getScale:r.get,isValidScale:r.isValid,hasColorscale:i.hasColorscale,extractOpts:i.extractOpts,extractScale:i.extractScale,flipScale:i.flipScale,makeColorScaleFunc:i.makeColorScaleFunc,makeColorScaleFuncFromTrace:i.makeColorScaleFuncFromTrace}},function(e,t,n){"use strict";t.xmlns="http://www.w3.org/2000/xmlns/",t.svg="http://www.w3.org/2000/svg",t.xlink="http://www.w3.org/1999/xlink",t.svgAttrs={xmlns:t.svg,"xmlns:xlink":t.xlink}},function(e,t,n){"use strict";e.exports=function(e,t){var n=0|e.length,r=0|t.length;if(1===n&&1===r)return function(e,t){var n=e+t,r=n-e,i=e-(n-r)+(t-r);if(i)return[i,n];return[n]}(e[0],t[0]);var i,o,a=new Array(n+r),s=0,l=0,u=0,c=Math.abs,d=e[l],h=c(d),f=t[u],p=c(f);h<p?(o=d,(l+=1)<n&&(d=e[l],h=c(d))):(o=f,(u+=1)<r&&(f=t[u],p=c(f)));l<n&&h<p||u>=r?(i=d,(l+=1)<n&&(d=e[l],h=c(d))):(i=f,(u+=1)<r&&(f=t[u],p=c(f)));var g,v,m=i+o,y=m-i,b=o-y,C=b,A=m;for(;l<n&&u<r;)h<p?(i=d,(l+=1)<n&&(d=e[l],h=c(d))):(i=f,(u+=1)<r&&(f=t[u],p=c(f))),(b=(o=C)-(y=(m=i+o)-i))&&(a[s++]=b),C=A-((g=A+m)-(v=g-A))+(m-v),A=g;for(;l<n;)(b=(o=C)-(y=(m=(i=d)+o)-i))&&(a[s++]=b),C=A-((g=A+m)-(v=g-A))+(m-v),A=g,(l+=1)<n&&(d=e[l]);for(;u<r;)(b=(o=C)-(y=(m=(i=f)+o)-i))&&(a[s++]=b),C=A-((g=A+m)-(v=g-A))+(m-v),A=g,(u+=1)<r&&(f=t[u]);C&&(a[s++]=C);A&&(a[s++]=A);s||(a[s++]=0);return a.length=s,a}},function(e,t,n){"use strict";var r=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),i=this&&this.__exportStar||function(e,t){for(var n in e)"default"===n||Object.prototype.hasOwnProperty.call(t,n)||r(t,e,n)};Object.defineProperty(t,"__esModule",{value:!0}),i(n(2058),t),i(n(600),t),i(n(1123),t),i(n(2059),t),i(n(601),t),i(n(1124),t),i(n(1126),t),i(n(1125),t),i(n(2060),t),i(n(1127),t),i(n(1128),t),i(n(2061),t),i(n(1129),t),i(n(602),t)},function(e,t,n){"use strict";n.r(t);var r=n(454);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";n.r(t);var r=n(1193);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";n.r(t);var r,i,o=n(17);!function(e){function t(e,t="copy"){let n=window.getSelection();const r=[];for(let e=0,t=(null==n?void 0:n.rangeCount)||0;e<t;++e)r[e]=n.getRangeAt(e).cloneRange();const i=document.createRange();if(i.selectNodeContents(e),n&&(n.removeAllRanges(),n.addRange(i)),document.execCommand(t),n=window.getSelection(),n){n.removeAllRanges();for(let e=0,t=r.length;e<t;++e)n.addRange(r[e])}}e.getInstance=function(){return i.instance},e.setInstance=function(e){i.instance=e},e.copyToSystem=function(e){const n=document.body,r=t=>{const i=t.clipboardData||window.clipboardData;"string"==typeof e?i.setData("text",e):e.types().map(t=>{i.setData(t,e.getData(t))}),t.preventDefault(),n.removeEventListener("copy",r)};n.addEventListener("copy",r),t(n)},e.generateEvent=t}(r||(r={})),(i||(i={})).instance=new o.MimeData;var a=n(16),s=n(18),l=n(1307);class u extends s.Widget{constructor(e){super(e),this._collapseChanged=new a.Signal(this);const{widget:t,collapsed:n=!0}=e;this.addClass("jp-Collapse"),this._header=new s.Widget,this._header.addClass("jp-Collapse-header"),this._content=new s.Panel,this._content.addClass("jp-Collapse-contents");const r=new s.PanelLayout;this.layout=r,r.addWidget(this._header),r.addWidget(this._content),this.widget=t,this.collapsed=n}get widget(){return this._widget}set widget(e){const t=this._widget;t&&(t.title.changed.disconnect(this._onTitleChanged,this),t.parent=null),this._widget=e,e.title.changed.connect(this._onTitleChanged,this),this._onTitleChanged(e.title),this._content.addWidget(e)}get collapsed(){return this._collapsed}set collapsed(e){e!==this._collapsed&&(e?this._collapse():this._uncollapse())}get collapseChanged(){return this._collapseChanged}toggle(){this.collapsed=!this.collapsed}dispose(){this.isDisposed||(this._header=null,this._widget=null,this._content=null,super.dispose())}handleEvent(e){switch(e.type){case"click":this._evtClick(e)}}onAfterAttach(e){this._header.node.addEventListener("click",this)}onBeforeDetach(e){this._header.node.removeEventListener("click",this)}_collapse(){this._collapsed=!0,this._content&&this._content.hide(),this._setHeader(),this._collapseChanged.emit(void 0)}_uncollapse(){this._collapsed=!1,this._content&&this._content.show(),this._setHeader(),this._collapseChanged.emit(void 0)}_evtClick(e){this.toggle()}_onTitleChanged(e){this._setHeader()}_setHeader(){(this._collapsed?l.c:l.b).element({container:this._header.node,label:this._widget.title.label,elementPosition:"right",height:"28px"})}}const c="commandlinker-command",d="commandlinker-args";class h{constructor(e){this._isDisposed=!1,this._commands=e.commands,document.body.addEventListener("click",this)}get isDisposed(){return this._isDisposed}dispose(){this.isDisposed||(this._isDisposed=!0,document.body.removeEventListener("click",this))}connectNode(e,t,n){return e.setAttribute("data-"+c,t),void 0!==n&&e.setAttribute("data-"+d,JSON.stringify(n)),e}disconnectNode(e){return e.removeAttribute("data-"+c),e.removeAttribute("data-"+d),e}handleEvent(e){switch(e.type){case"click":this._evtClick(e);break;default:return}}populateVNodeDataset(e,t){let n;return n=void 0!==t?{[d]:JSON.stringify(t),[c]:e}:{[c]:e},n}_evtClick(e){let t=e.target;for(;t&&t.parentElement;){if(t.hasAttribute("data-"+c)){e.preventDefault();const n=t.getAttribute("data-"+c);if(!n)return;const r=t.getAttribute("data-"+d);let i=o.JSONExt.emptyObject;return r&&(i=JSON.parse(r)),void this._commands.execute(n,i)}t=t.parentElement}}}const f=new o.Token("@jupyterlab/apputils:ICommandPalette");class p extends s.Panel{constructor(e){super(),this.addClass("jp-ModalCommandPalette"),this.id="modal-command-palette",this._commandPalette=e.commandPalette,this.addWidget(this._commandPalette),this._commandPalette.commands.commandExecuted.connect(()=>{this.isAttached&&this.isVisible&&this.hideAndReset()}),this.hideAndReset()}get palette(){return this._commandPalette}set palette(e){this._commandPalette=e,this.addWidget(e),this.hideAndReset()}attach(){s.Widget.attach(this,document.body)}detach(){s.Widget.detach(this)}hideAndReset(){this.hide(),this._commandPalette.inputNode.value="",this._commandPalette.refresh()}handleEvent(e){switch(e.type){case"keydown":this._evtKeydown(e);break;case"focus":const t=e.target;this.node.contains(t)||(e.stopPropagation(),this.hideAndReset());break;case"contextmenu":e.preventDefault(),e.stopPropagation()}}onAfterAttach(e){this.node.addEventListener("keydown",this,!0),this.node.addEventListener("contextmenu",this,!0)}onAfterDetach(e){this.node.removeEventListener("keydown",this,!0),this.node.removeEventListener("contextmenu",this,!0)}onBeforeHide(e){document.removeEventListener("focus",this,!0)}onAfterShow(e){document.addEventListener("focus",this,!0)}onActivateRequest(e){this.isAttached&&(this.show(),this._commandPalette.activate())}_evtKeydown(e){switch(e.keyCode){case 27:e.stopPropagation(),e.preventDefault(),this.hideAndReset()}}}var g,v,m=n(58),y=n(6),b=n(38);(v=g||(g={})).hitTestNodes=function(e,t,n){return y.ArrayExt.findFirstIndex(e,e=>b.a.hitTest(e,t,n))},v.findElement=function(e,t){return e.querySelector("."+t)},v.findElements=function(e,t){return e.getElementsByClassName(t)},v.createDomID=function(){return"id-"+o.UUID.uuid4()};var C,A=n(891);class _ extends s.Widget{constructor(e={}){super({node:C.createNode()}),this._sandbox=[],this.addClass("jp-IFrame"),this.sandbox=e.sandbox||[],this.referrerPolicy=e.referrerPolicy||"no-referrer"}get referrerPolicy(){return this._referrerPolicy}set referrerPolicy(e){if(this._referrerPolicy===e)return;this._referrerPolicy=e;this.node.querySelector("iframe").setAttribute("referrerpolicy",e)}get sandbox(){return this._sandbox.slice()}set sandbox(e){this._sandbox=e.slice();const t=this.node.querySelector("iframe"),n=e.length?e.join(" "):"";t.setAttribute("sandbox",n)}get url(){return this.node.querySelector("iframe").getAttribute("src")||""}set url(e){this.node.querySelector("iframe").setAttribute("src",e)}}!function(e){e.createNode=function(){const e=document.createElement("div"),t=document.createElement("iframe");return t.setAttribute("sandbox",""),t.style.height="100%",t.style.width="100%",e.appendChild(t),e}}(C||(C={}));var I=n(203);var x,w;(w=x||(x={})).getBoolean=function(e){return Object(m.b)(Object.assign(Object.assign({},e),{body:new S(e),buttons:[m.a.cancelButton({label:e.cancelLabel}),m.a.okButton({label:e.okLabel})],focusNodeSelector:"input"}))},w.getNumber=function(e){return Object(m.b)(Object.assign(Object.assign({},e),{body:new T(e),buttons:[m.a.cancelButton({label:e.cancelLabel}),m.a.okButton({label:e.okLabel})],focusNodeSelector:"input"}))},w.getItem=function(e){return Object(m.b)(Object.assign(Object.assign({},e),{body:new k(e),buttons:[m.a.cancelButton({label:e.cancelLabel}),m.a.okButton({label:e.okLabel})],focusNodeSelector:e.editable?"input":"select"}))},w.getText=function(e){return Object(m.b)(Object.assign(Object.assign({},e),{body:new E(e),buttons:[m.a.cancelButton({label:e.cancelLabel}),m.a.okButton({label:e.okLabel})],focusNodeSelector:"input"}))},w.getPassword=function(e){return Object(m.b)(Object.assign(Object.assign({},e),{body:new O(e),buttons:[m.a.cancelButton({label:e.cancelLabel}),m.a.okButton({label:e.okLabel})],focusNodeSelector:"input"}))};class M extends s.Widget{constructor(e){if(super(),this.addClass("jp-Input-Dialog"),void 0!==e){const t=document.createElement("label");t.textContent=e,this.node.appendChild(t)}}}class S extends M{constructor(e){super(e.label),this._input=document.createElement("input"),this._input.classList.add("jp-mod-styled"),this._input.type="checkbox",this._input.checked=!!e.value,this.node.appendChild(this._input)}getValue(){return this._input.checked}}class T extends M{constructor(e){super(e.label),this._input=document.createElement("input",{}),this._input.classList.add("jp-mod-styled"),this._input.type="number",this._input.value=e.value?e.value.toString():"0",this.node.appendChild(this._input)}getValue(){return this._input.value?Number(this._input.value):Number.NaN}}class E extends M{constructor(e){super(e.label),this._input=document.createElement("input",{}),this._input.classList.add("jp-mod-styled"),this._input.type="text",this._input.value=e.text?e.text:"",e.placeholder&&(this._input.placeholder=e.placeholder),this.node.appendChild(this._input)}getValue(){return this._input.value}}class O extends M{constructor(e){super(e.label),this._input=document.createElement("input",{}),this._input.classList.add("jp-mod-styled"),this._input.type="password",this._input.value=e.text?e.text:"",e.placeholder&&(this._input.placeholder=e.placeholder),this.node.appendChild(this._input)}getValue(){return this._input.value}}class k extends M{constructor(e){super(e.label),this._editable=e.editable||!1;let t,n=e.current||0;if("number"==typeof n&&(t=Math.max(0,Math.min(n,e.items.length-1)),n=""),this._list=document.createElement("select"),e.items.forEach((e,r)=>{const i=document.createElement("option");r===t&&(i.selected=!0,n=e),i.value=e,i.textContent=e,this._list.appendChild(i)}),e.editable){const t=document.createElement("datalist");t.id="input-dialog-items",t.appendChild(this._list),this._input=document.createElement("input",{}),this._input.classList.add("jp-mod-styled"),this._input.type="list",this._input.value=n,this._input.setAttribute("list",t.id),e.placeholder&&(this._input.placeholder=e.placeholder),this.node.appendChild(this._input),this.node.appendChild(t)}else this.node.appendChild(I.a.wrapSelect(this._list))}getValue(){return this._editable?this._input.value:this._list.value}}var L=n(14);class D extends s.Widget{constructor(){super(),this.addClass("jp-Spinner"),this.node.tabIndex=-1;const e=document.createElement("div");e.className="jp-SpinnerContent",this.node.appendChild(e)}onActivateRequest(e){this.node.focus()}}var P=n(48),N=n(2348),j=n(23),R=n(128),z=n(102),F=n(0),B=n(51);class H{constructor(e){var t,n,r,i;this._path="",this._name="",this._type="",this._prevKernelName="",this._isDisposed=!1,this._disposed=new a.Signal(this),this._session=null,this._ready=new o.PromiseDelegate,this._initializing=!1,this._initStarted=new o.PromiseDelegate,this._initPromise=new o.PromiseDelegate,this._isReady=!1,this._isTerminating=!1,this._isRestarting=!1,this._kernelChanged=new a.Signal(this),this._sessionChanged=new a.Signal(this),this._statusChanged=new a.Signal(this),this._connectionStatusChanged=new a.Signal(this),this._iopubMessage=new a.Signal(this),this._unhandledMessage=new a.Signal(this),this._propertyChanged=new a.Signal(this),this._dialog=null,this._busyDisposable=null,this._pendingKernelName="",this._pendingSessionRequest="",this.sessionManager=e.sessionManager,this.specsManager=e.specsManager,this.translator=e.translator||B.a,this._trans=this.translator.load("jupyterlab"),this._path=null!==(t=e.path)&&void 0!==t?t:o.UUID.uuid4(),this._type=null!==(n=e.type)&&void 0!==n?n:"",this._name=null!==(r=e.name)&&void 0!==r?r:"",this._setBusy=e.setBusy,this._kernelPreference=null!==(i=e.kernelPreference)&&void 0!==i?i:{}}get session(){var e;return null!==(e=this._session)&&void 0!==e?e:null}get path(){return this._path}get type(){return this._type}get name(){return this._name}get kernelChanged(){return this._kernelChanged}get sessionChanged(){return this._sessionChanged}get statusChanged(){return this._statusChanged}get connectionStatusChanged(){return this._connectionStatusChanged}get iopubMessage(){return this._iopubMessage}get unhandledMessage(){return this._unhandledMessage}get propertyChanged(){return this._propertyChanged}get kernelPreference(){return this._kernelPreference}set kernelPreference(e){this._kernelPreference=e}get isReady(){return this._isReady}get ready(){return this._ready.promise}get isTerminating(){return this._isTerminating}get isRestarting(){return this._isRestarting}get kernelDisplayName(){var e,t,n,r,i,o,a,s,l,u;const c=null===(e=this.session)||void 0===e?void 0:e.kernel;if(this._pendingKernelName===this._trans.__("No Kernel"))return this._trans.__("No Kernel");if(!c&&!this.isReady&&!1!==this.kernelPreference.canStart&&!1!==this.kernelPreference.shouldStart){let e=this._pendingKernelName||H.getDefaultKernel({specs:this.specsManager.specs,sessions:this.sessionManager.running(),preference:this.kernelPreference})||"";return e?(e=null!==(r=null===(n=null===(t=this.specsManager.specs)||void 0===t?void 0:t.kernelspecs[e])||void 0===n?void 0:n.display_name)&&void 0!==r?r:e,e):this._trans.__("No Kernel")}return this._pendingKernelName?null!==(a=null===(o=null===(i=this.specsManager.specs)||void 0===i?void 0:i.kernelspecs[this._pendingKernelName])||void 0===o?void 0:o.display_name)&&void 0!==a?a:this._pendingKernelName:c?null!==(u=null===(l=null===(s=this.specsManager.specs)||void 0===s?void 0:s.kernelspecs[c.name])||void 0===l?void 0:l.display_name)&&void 0!==u?u:c.name:this._trans.__("No Kernel")}get kernelDisplayStatus(){var e,t;const n=null===(e=this.session)||void 0===e?void 0:e.kernel;return this._isTerminating?"terminating":this._isRestarting?"restarting":this._pendingKernelName===this._trans.__("No Kernel")?"idle":!n&&this._pendingKernelName?"initializing":n||this.isReady||!1===this.kernelPreference.canStart||!1===this.kernelPreference.shouldStart?null!==(t="connected"===(null==n?void 0:n.connectionStatus)?null==n?void 0:n.status:null==n?void 0:n.connectionStatus)&&void 0!==t?t:"unknown":"initializing"}get prevKernelName(){return this._prevKernelName}get isDisposed(){return this._isDisposed}get disposed(){return this._disposed}dispose(){this._isDisposed||(this._isDisposed=!0,this._disposed.emit(),this._session&&(this.kernelPreference.shutdownOnDispose&&this.sessionManager.shutdown(this._session.id).catch(e=>{console.error("Kernel not shut down "+e)}),this._session.dispose(),this._session=null),this._dialog&&this._dialog.dispose(),this._busyDisposable&&(this._busyDisposable.dispose(),this._busyDisposable=null),a.Signal.clearData(this))}async restartKernel(){var e,t,n,r,i,o;const a=(null===(e=this.session)||void 0===e?void 0:e.kernel)||null;this._isRestarting=!0,this._isReady=!1,this._statusChanged.emit("restarting"),await(null===(n=null===(t=this.session)||void 0===t?void 0:t.kernel)||void 0===n?void 0:n.restart()),this._isRestarting=!1,this._isReady=!0,this._statusChanged.emit((null===(i=null===(r=this.session)||void 0===r?void 0:r.kernel)||void 0===i?void 0:i.status)||"unknown"),this._kernelChanged.emit({name:"kernel",oldValue:a,newValue:(null===(o=this.session)||void 0===o?void 0:o.kernel)||null})}async changeKernel(e={}){if(this.isDisposed)throw new Error("Disposed");return await this._initStarted.promise,this._changeKernel(e)}async shutdown(){if(!this.isDisposed&&this._initializing)return await this._initStarted.promise,this._pendingSessionRequest="",this._pendingKernelName=this._trans.__("No Kernel"),this._shutdownSession()}async initialize(){if(this._initializing)return this._initPromise.promise;this._initializing=!0;const e=await this._initialize();return e||(this._isReady=!0,this._ready.resolve(void 0)),this._pendingSessionRequest||this._initStarted.resolve(void 0),this._initPromise.resolve(e),e}async _initialize(){const e=this.sessionManager;await e.ready,await e.refreshRunning();const t=Object(y.find)(e.running(),e=>e.path===this._path);if(t)try{const n=e.connectTo({model:t});this._handleNewSession(n)}catch(e){return this._handleSessionError(e),Promise.reject(e)}return await this._startIfNecessary()}async _shutdownSession(){const e=this._session;this._isTerminating=!0,this._isReady=!1,this._statusChanged.emit("terminating"),await(null==e?void 0:e.shutdown()),this._isTerminating=!1,null==e||e.dispose(),this._session=null;const t=(null==e?void 0:e.kernel)||null;this._statusChanged.emit("unknown"),this._kernelChanged.emit({name:"kernel",oldValue:t,newValue:null}),this._sessionChanged.emit({name:"session",oldValue:e,newValue:null})}async _startIfNecessary(){var e;const t=this.kernelPreference;if(this.isDisposed||(null===(e=this.session)||void 0===e?void 0:e.kernel)||!1===t.shouldStart||!1===t.canStart)return!1;let n;if(t.id)n={id:t.id};else{const e=H.getDefaultKernel({specs:this.specsManager.specs,sessions:this.sessionManager.running(),preference:t});e&&(n={name:e})}if(n)try{return await this._changeKernel(n),!1}catch(e){}return!0}async _changeKernel(e={},t=!1){e.name&&(this._pendingKernelName=e.name),this._session?await this._shutdownSession():this._kernelChanged.emit({name:"kernel",oldValue:null,newValue:null}),this._pendingSessionRequest||this._initStarted.resolve(void 0);const n=P.PathExt.dirname(this._path),r=this._pendingSessionRequest=P.PathExt.join(n,o.UUID.uuid4());try{this._statusChanged.emit("starting");const t=await this.sessionManager.startNew({path:r,type:this._type,name:this._name,kernel:e});return this._pendingSessionRequest!==t.path?(await t.shutdown(),t.dispose(),null):(await t.setPath(this._path),await t.setName(this._name),this._session&&await this._shutdownSession(),this._handleNewSession(t))}catch(e){throw this._handleSessionError(e),e}}_handleNewSession(e){var t,n,r;if(this.isDisposed)throw Error("Disposed");return this._isReady||(this._isReady=!0,this._ready.resolve(void 0)),this._session&&this._session.dispose(),this._session=e,this._pendingKernelName="",e&&(this._prevKernelName=null!==(n=null===(t=e.kernel)||void 0===t?void 0:t.name)&&void 0!==n?n:"",e.disposed.connect(this._onSessionDisposed,this),e.propertyChanged.connect(this._onPropertyChanged,this),e.kernelChanged.connect(this._onKernelChanged,this),e.statusChanged.connect(this._onStatusChanged,this),e.connectionStatusChanged.connect(this._onConnectionStatusChanged,this),e.iopubMessage.connect(this._onIopubMessage,this),e.unhandledMessage.connect(this._onUnhandledMessage,this),e.path!==this._path&&this._onPropertyChanged(e,"path"),e.name!==this._name&&this._onPropertyChanged(e,"name"),e.type!==this._type&&this._onPropertyChanged(e,"type")),this._sessionChanged.emit({name:"session",oldValue:null,newValue:e}),this._kernelChanged.emit({oldValue:null,newValue:(null==e?void 0:e.kernel)||null,name:"kernel"}),this._statusChanged.emit((null===(r=null==e?void 0:e.kernel)||void 0===r?void 0:r.status)||"unknown"),(null==e?void 0:e.kernel)||null}async _handleSessionError(e){this._handleNewSession(null);let t="",n="";try{t=e.traceback,n=e.message}catch(e){}const r=F.createElement("div",null,F.createElement("pre",null,e.message),n&&F.createElement("pre",null,n),t&&F.createElement("details",{className:"jp-mod-wide"},F.createElement("pre",null,t))),i=this._dialog=new m.a({title:this._trans.__("Error Starting Kernel"),body:r,buttons:[m.a.okButton()]});await i.launch(),this._dialog=null}_onSessionDisposed(){if(this._session){const e=this._session;this._session=null;const t=this._session;this._sessionChanged.emit({name:"session",oldValue:e,newValue:t})}}_onPropertyChanged(e,t){switch(t){case"path":this._path=e.path;break;case"name":this._name=e.name;break;case"type":this._type=e.type;break;default:throw new Error("unrecognized property "+t)}this._propertyChanged.emit(t)}_onKernelChanged(e,t){this._kernelChanged.emit(t)}_onStatusChanged(e,t){this._setBusy&&("busy"===t?this._busyDisposable||(this._busyDisposable=this._setBusy()):this._busyDisposable&&(this._busyDisposable.dispose(),this._busyDisposable=null)),this._statusChanged.emit(t)}_onConnectionStatusChanged(e,t){this._connectionStatusChanged.emit(t)}_onIopubMessage(e,t){this._iopubMessage.emit(t)}_onUnhandledMessage(e,t){this._unhandledMessage.emit(t)}}(H||(H={})).getDefaultKernel=function(e){return V.getDefaultKernel(e)};const U={async selectKernel(e,t){if(e.isDisposed)return Promise.resolve();const n=(t=t||B.a).load("jupyterlab");let r=n.__("Cancel");e.kernelDisplayName===n.__("No Kernel")&&(r=n.__("No Kernel"));const i=[m.a.cancelButton({label:r}),m.a.okButton({label:n.__("Select")})],o=new m.a({title:n.__("Select Kernel"),body:new V.KernelSelector(e),buttons:i}),a=await o.launch();if(e.isDisposed||!a.button.accept)return;const s=a.value;if(null===s&&e.kernelDisplayName!==n.__("No Kernel"))return e.shutdown();s&&await e.changeKernel(s)},async restart(e,t){var n;const r=(t=t||B.a).load("jupyterlab");if(await e.initialize(),e.isDisposed)throw new Error("session already disposed");const i=null===(n=e.session)||void 0===n?void 0:n.kernel;if(!i&&e.prevKernelName)return await e.changeKernel({name:e.prevKernelName}),!0;if(!i)throw new Error("No kernel to restart");const o=m.a.warnButton({label:"Restart"}),a=await Object(m.b)({title:r.__("Restart Kernel?"),body:r.__("Do you want to restart the current kernel? All variables will be lost."),buttons:[m.a.cancelButton(),o]});return!i.isDisposed&&(!!a.button.accept&&(await e.restartKernel(),!0))}};var V;!function(e){class t extends s.Widget{constructor(e){super({node:n(e)})}getValue(){const e=this.node.querySelector("select");return JSON.parse(e.value)}}function n(e,t){const n=(t=t||B.a).load("jupyterlab"),i=document.createElement("div"),o=document.createElement("label");o.textContent=`${n.__("Select kernel for:")} "${e.name}"`,i.appendChild(o);const a=function(e){return{specs:e.specsManager.specs,sessions:e.sessionManager.running(),preference:e.kernelPreference}}(e),s=document.createElement("select");return r(s,a,t),i.appendChild(s),i}function r(e,t,n){for(;e.firstChild;)e.removeChild(e.firstChild);const{preference:r,sessions:s,specs:l}=t,{name:u,id:c,language:d,canStart:h,shouldStart:f}=r,p=(n=n||B.a).load("jupyterlab");if(!l||!1===h)return e.appendChild(o(n)),e.value="null",void(e.disabled=!0);e.disabled=!1;const g=Object.create(null),v=Object.create(null);for(const e in l.kernelspecs){const t=l.kernelspecs[e];g[e]=t.display_name,v[e]=t.language}const m=[];if(u&&u in l.kernelspecs&&m.push(u),d)for(const e in l.kernelspecs)u!==e&&v[e]===d&&m.push(e);m.length||m.push(l.default);const b=document.createElement("optgroup");b.label=p.__("Start Preferred Kernel"),m.sort((e,t)=>g[e].localeCompare(g[t]));for(const e of m)b.appendChild(i(e,g[e]));b.firstChild&&e.appendChild(b),e.appendChild(o());const C=document.createElement("optgroup");C.label=p.__("Start Other Kernel");const A=[];for(const e in l.kernelspecs)-1===m.indexOf(e)&&A.push(e);A.sort((e,t)=>g[e].localeCompare(g[t]));for(const e of A)C.appendChild(i(e,g[e]));if(A.length&&e.appendChild(C),!1===f?e.value="null":e.selectedIndex=0,!s)return;const _=[],I=[];Object(y.each)(s,e=>{var t;d&&e.kernel&&v[e.kernel.name]===d&&e.kernel.id!==c?_.push(e):(null===(t=e.kernel)||void 0===t?void 0:t.id)!==c&&I.push(e)});const x=document.createElement("optgroup");x.label=p.__("Use Kernel from Preferred Session"),e.appendChild(x),_.length&&(_.sort((e,t)=>e.path.localeCompare(t.path)),Object(y.each)(_,e=>{const t=e.kernel?g[e.kernel.name]:"";x.appendChild(a(e,t,n))}));const w=document.createElement("optgroup");w.label=p.__("Use Kernel from Other Session"),e.appendChild(w),I.length&&(I.sort((e,t)=>e.path.localeCompare(t.path)),Object(y.each)(I,e=>{const t=e.kernel?g[e.kernel.name]||e.kernel.name:"";w.appendChild(a(e,t,n))}))}function i(e,t){const n=document.createElement("option");return n.text=t,n.value=JSON.stringify({name:e}),n}function o(e){const t=(e=e||B.a).load("jupyterlab"),n=document.createElement("optgroup");n.label="Use No Kernel";const r=document.createElement("option");return r.text=t.__("No Kernel"),r.value="null",n.appendChild(r),n}function a(e,t,n){var r,i;const o=(n=n||B.a).load("jupyterlab"),a=document.createElement("option"),s=e.name||P.PathExt.basename(e.path);return a.text=s,a.value=JSON.stringify({id:null===(r=e.kernel)||void 0===r?void 0:r.id}),a.title=`${o.__("Path:")} ${e.path}\n${o.__("Name:")} ${s}\n${o.__("Kernel Name:")} ${t}\n${o.__("Kernel Id:")} ${null===(i=e.kernel)||void 0===i?void 0:i.id}`,a}e.KernelSelector=t,e.getDefaultKernel=function(e){var t;const{specs:n,preference:r}=e,{name:i,language:o,shouldStart:a,canStart:s,autoStartDefault:l}=r;if(!n||!1===a||!1===s)return null;const u=l?n.default:null;if(!i&&!o)return u;for(const e in n.kernelspecs)if(e===i)return i;if(!o)return u;const c=[];for(const e in n.kernelspecs){o===(null===(t=n.kernelspecs[e])||void 0===t?void 0:t.language)&&c.push(e)}if(1===c.length){const e=c[0];return console.warn("No exact match found for "+e+", using kernel "+e+" that matches language="+o),e}return u},e.populateKernelSelect=r}(V||(V={}));var Y=n(144);class W extends s.PanelLayout{constructor(){super(...arguments),this._dirty=!1}onFitRequest(e){super.onFitRequest(e),this.parent.isAttached&&(Object(y.some)(this.widgets,e=>!e.isHidden)?(this.parent.node.style.minHeight="var(--jp-private-toolbar-height)",this.parent.removeClass("jp-Toolbar-micro")):(this.parent.node.style.minHeight="",this.parent.addClass("jp-Toolbar-micro"))),this._dirty=!0,this.parent.parent&&L.c.sendMessage(this.parent.parent,s.Widget.Msg.FitRequest),this._dirty&&L.c.sendMessage(this.parent,s.Widget.Msg.UpdateRequest)}onUpdateRequest(e){super.onUpdateRequest(e),this.parent.isVisible&&(this._dirty=!1)}onChildShown(e){super.onChildShown(e),this.parent.fit()}onChildHidden(e){super.onChildHidden(e),this.parent.fit()}onBeforeAttach(e){super.onBeforeAttach(e),this.parent.fit()}attachWidget(e,t){super.attachWidget(e,t),this.parent.fit()}detachWidget(e,t){super.detachWidget(e,t),this.parent.fit()}}class G extends s.Widget{constructor(){super(),this.addClass("jp-Toolbar"),this.addClass("jp-scrollbar-tiny"),this.layout=new W}names(){const e=this.layout;return Object(y.map)(e.widgets,e=>q.nameProperty.get(e))}addItem(e,t){const n=this.layout;return this.insertItem(n.widgets.length,e,t)}insertItem(e,t,n){if(Object(y.find)(this.names(),e=>e===t))return!1;n.addClass("jp-Toolbar-item");return this.layout.insertWidget(e,n),q.nameProperty.set(n,t),!0}insertAfter(e,t,n){return this._insertRelative(e,1,t,n)}insertBefore(e,t,n){return this._insertRelative(e,0,t,n)}_insertRelative(e,t,n,r){const i=Object(y.map)(this.names(),(e,t)=>({name:e,index:t})),o=Object(y.find)(i,t=>t.name===e);return!!o&&this.insertItem(o.index+t,n,r)}handleEvent(e){switch(e.type){case"click":this.handleClick(e)}}handleClick(e){if(e.target instanceof HTMLLabelElement){const t=e.target.getAttribute("for");if(t&&this.node.querySelector("#"+t))return}this.node.contains(document.activeElement)||this.parent&&this.parent.activate()}onAfterAttach(e){this.node.addEventListener("click",this)}onBeforeDetach(e){this.node.removeEventListener("click",this)}}var X,q;function Z(e){var t,n;return F.createElement(N.a,{className:e.className?e.className+" jp-ToolbarButtonComponent":"jp-ToolbarButtonComponent",disabled:!1===e.enabled,onClick:null!==(t=e.actualOnClick)&&void 0!==t&&t?t=>{var n;0===t.button&&(null===(n=e.onClick)||void 0===n||n.call(e))}:void 0,onMouseDown:null!==(n=e.actualOnClick)&&void 0!==n&&n?void 0:t=>{var n;0===t.button&&(t.preventDefault(),null===(n=e.onClick)||void 0===n||n.call(e))},onKeyDown:t=>{var n;const{key:r}=t;"Enter"!==r&&" "!==r||null===(n=e.onClick)||void 0===n||n.call(e)},title:e.tooltip||e.iconLabel,minimal:!0},(e.icon||e.iconClass)&&F.createElement(j.a.resolveReact,{icon:e.icon,iconClass:Object(R.a)(e.iconClass,"jp-Icon"),className:"jp-ToolbarButtonComponent-icon",tag:"span",stylesheet:"toolbarButton"}),e.label&&F.createElement("span",{className:"jp-ToolbarButtonComponent-label"},e.label))}function Q(e){return e.addClass("jp-ToolbarButton"),e}(X=G||(G={})).createInterruptButton=function(e,t){const n=(t=t||B.a).load("jupyterlab");return new K({icon:l.k,onClick:()=>{var t,n;null===(n=null===(t=e.session)||void 0===t?void 0:t.kernel)||void 0===n||n.interrupt()},tooltip:n.__("Interrupt the kernel")})},X.createRestartButton=function(e,t,n){const r=(n=n||B.a).load("jupyterlab");return new K({icon:l.j,onClick:()=>{(null!=t?t:U).restart(e,n)},tooltip:r.__("Restart the kernel")})},X.createSpacerItem=function(){return new q.Spacer},X.createKernelNameItem=function(e,t,n){const r=Y.a.create(F.createElement(q.KernelNameComponent,{sessionContext:e,dialogs:null!=t?t:U,translator:n}));return r.addClass("jp-KernelName"),r},X.createKernelStatusItem=function(e,t){return new q.KernelStatus(e,t)};class K extends Y.a{constructor(e={}){super(),this.props=e,Q(this)}render(){return F.createElement(Z,Object.assign({},this.props))}}function J(e){return F.createElement(Y.b,{signal:e.commands.commandChanged,shouldUpdate:(t,n)=>n.id===e.id&&"changed"===n.type||"many-changed"===n.type},()=>F.createElement(Z,Object.assign({},q.propsFromCommand(e))))}function $(e){return e.addClass("jp-CommandToolbarButton"),e}class ee extends Y.a{constructor(e){super(),this.props=e,$(this)}render(){return F.createElement(J,Object.assign({},this.props))}}!function(e){e.propsFromCommand=function(e){const{commands:t,id:n,args:r}=e,i=t.iconClass(n,r),o=t.iconLabel(n,r),a=t.icon(n,r),s=a===i?void 0:a,l=t.label(n,r);let u=t.className(n,r);return t.isToggled(n,r)&&(u+=" lm-mod-toggled"),t.isVisible(n,r)||(u+=" lm-mod-hidden"),{className:u,icon:s,iconClass:i,tooltip:t.caption(n,r)||l||o,onClick:()=>{t.execute(n,r)},enabled:t.isEnabled(n,r),label:l}},e.nameProperty=new z.AttachedProperty({name:"name",create:()=>""}),e.noOp=function(){};class t extends s.Widget{constructor(){super(),this.addClass("jp-Toolbar-spacer")}}e.Spacer=t,e.KernelNameComponent=function(e){const t=e.translator||B.a,n=t.load("jupyterlab"),r=()=>{e.dialogs.selectKernel(e.sessionContext,t)};return F.createElement(Y.b,{signal:e.sessionContext.kernelChanged,initialSender:e.sessionContext},e=>F.createElement(Z,{className:"jp-Toolbar-kernelName",onClick:r,tooltip:n.__("Switch kernel"),label:null==e?void 0:e.kernelDisplayName}))};class n extends s.Widget{constructor(e,t){super(),this.translator=t||B.a,this._trans=this.translator.load("jupyterlab"),this.addClass("jp-Toolbar-kernelStatus"),this._onStatusChanged(e),e.statusChanged.connect(this._onStatusChanged,this),e.connectionStatusChanged.connect(this._onStatusChanged,this)}_onStatusChanged(e){if(this.isDisposed)return;const t=e.kernelDisplayStatus,n={container:this.node,title:this._trans.__("Kernel %1",P.Text.titleCase(t)),stylesheet:"toolbarButton",alignSelf:"normal",height:"24px"};j.a.remove(this.node),"busy"===t||"starting"===t||"terminating"===t||"restarting"===t||"initializing"===t?l.e.element(n):"connecting"===t||"disconnected"===t||"unknown"===t?l.i.element(n):l.d.element(n)}}e.KernelStatus=n}(q||(q={}));var te,ne=n(84);!function(e){function t(t){return!("object"!=typeof t||!t)&&e.symbol in t}e.symbol=Symbol("printable"),e.isPrintable=t,e.getPrintFunction=function(n){return t(n)?n[e.symbol]():null},e.printWidget=function(e){return r(e.node)};const n=ne.ServerConnection.makeSettings();async function r(e){const t="string"==typeof e,n=function(){const e=window.document.createElement("iframe");e.setAttribute("sandbox","allow-modals allow-same-origin");return e.setAttribute("style","visibility:hidden;width:0;height:0;position:absolute;z-index:-9999;bottom:0;"),e.setAttribute("width","0"),e.setAttribute("height","0"),e}(),r=window.document.body;r.appendChild(n),t?(n.srcdoc=e,await i(n)):(n.src="about:blank",await i(n),function(e,t){e.contentDocument.body.appendChild(t.cloneNode(!0)),e.contentDocument.close()}(n,e));const o=new Promise(e=>{const t=()=>{document.removeEventListener("mousemove",t,!0),document.removeEventListener("mousedown",t,!0),document.removeEventListener("keydown",t,!0),e()};document.addEventListener("mousemove",t,!0),document.addEventListener("mousedown",t,!0),document.addEventListener("keydown",t,!0)});var a;(a=n.contentWindow).document.execCommand("print",!1)||a.print(),await o,r.removeChild(n)}function i(e){return new Promise(t=>{e.onload=()=>t()})}e.printURL=async function(e){return r(await(await ne.ServerConnection.makeRequest(e,{},n)).text())}}(te||(te={}));class re extends s.Widget{constructor(e){super(e),this._changeGuard=!1,this._spinner=new D,this._isRevealed=!1,this.addClass("jp-MainAreaWidget"),this.id=g.createDomID();const t=this._content=e.content,n=this._toolbar=e.toolbar||new G,r=this._spinner,i=this.layout=new s.BoxLayout({spacing:0});i.direction="top-to-bottom",s.BoxLayout.setStretch(n,0),s.BoxLayout.setStretch(t,1),i.addWidget(n),i.addWidget(t),t.id||(t.id=g.createDomID()),t.node.tabIndex=-1,this._updateTitle(),t.title.changed.connect(this._updateTitle,this),this.title.closable=!0,this.title.changed.connect(this._updateContentTitle,this),e.reveal?(this.node.appendChild(r.node),this._revealed=e.reveal.then(()=>{if(t.isDisposed)return void this.dispose();t.disposed.connect(()=>this.dispose());const e=document.activeElement===r.node;this.node.removeChild(r.node),r.dispose(),this._isRevealed=!0,e&&this._focusContent()}).catch(e=>{const o=new s.Widget,a=document.createElement("pre");throw a.textContent=String(e),o.node.appendChild(a),s.BoxLayout.setStretch(o,1),this.node.removeChild(r.node),r.dispose(),t.dispose(),this._content=null,n.dispose(),this._toolbar=null,i.addWidget(o),this._isRevealed=!0,o})):(r.dispose(),t.disposed.connect(()=>this.dispose()),this._isRevealed=!0,this._revealed=Promise.resolve(void 0))}[te.symbol](){return this._content?te.getPrintFunction(this._content):null}get content(){return this._content}get toolbar(){return this._toolbar}get isRevealed(){return this._isRevealed}get revealed(){return this._revealed}onActivateRequest(e){this._isRevealed?this._content&&this._focusContent():this._spinner.node.focus()}onCloseRequest(e){this.dispose()}onUpdateRequest(e){this._content&&L.c.sendMessage(this._content,e)}_updateTitle(){if(this._changeGuard||!this.content)return;this._changeGuard=!0;const e=this.content;this.title.label=e.title.label,this.title.mnemonic=e.title.mnemonic,this.title.icon=e.title.icon,this.title.iconClass=e.title.iconClass,this.title.iconLabel=e.title.iconLabel,this.title.caption=e.title.caption,this.title.className=e.title.className,this.title.dataset=e.title.dataset,this._changeGuard=!1}_updateContentTitle(){if(this._changeGuard||!this.content)return;this._changeGuard=!0;const e=this.content;e.title.label=this.title.label,e.title.mnemonic=this.title.mnemonic,e.title.icon=this.title.icon,e.title.iconClass=this.title.iconClass,e.title.iconLabel=this.title.iconLabel,e.title.caption=this.title.caption,e.title.className=this.title.className,e.title.dataset=this.title.dataset,this._changeGuard=!1}_focusContent(){this.content&&(this.content.node.contains(document.activeElement)||this.content.node.focus(),this.content.activate())}}var ie=n(624);const oe=new o.Token("@jupyterlab/apputils:ISplashScreen");var ae=n(86);class se{constructor(e){this._current=null,this._links=[],this._overrides={},this._overrideProps={},this._outstanding=null,this._pending=0,this._requests={},this._themes={},this._themeChanged=new a.Signal(this);const{host:t,key:n,splash:r,url:i}=e;this.translator=e.translator||B.a,this._trans=this.translator.load("jupyterlab");const o=e.settings;this._base=i,this._host=t,this._splash=r||null,o.load(n).then(e=>{this._settings=e,this._initOverrideProps(),this._settings.changed.connect(this._loadSettings,this),this._loadSettings()})}get theme(){return this._current}get themes(){return Object.keys(this._themes)}get themeChanged(){return this._themeChanged}getCSS(e){var t;return null!==(t=this._overrides[e])&&void 0!==t?t:getComputedStyle(document.documentElement).getPropertyValue("--jp-"+e)}loadCSS(e){const t=this._base,n=P.URLExt.isLocal(e)?P.URLExt.join(t,e):e,r=this._links;return new Promise((e,t)=>{const i=document.createElement("link");i.setAttribute("rel","stylesheet"),i.setAttribute("type","text/css"),i.setAttribute("href",n),i.addEventListener("load",()=>{e(void 0)}),i.addEventListener("error",()=>{t("Stylesheet failed to load: "+n)}),document.body.appendChild(i),r.push(i),this.loadCSSOverrides()})}loadCSSOverrides(){var e;const t=null!==(e=this._settings.user.overrides)&&void 0!==e?e:{};Object.keys(Object.assign(Object.assign({},this._overrides),t)).forEach(e=>{const n=t[e];n&&this.validateCSS(e,n)?document.documentElement.style.setProperty("--jp-"+e,n):(delete t[e],document.documentElement.style.removeProperty("--jp-"+e))}),this._overrides=t}validateCSS(e,t){const n=this._overrideProps[e];return n?!!CSS.supports(n,t)||(console.warn(`CSS validation failed: invalid value.\nkey: '${e}', val: '${t}', prop: '${n}'`),!1):(console.warn(`CSS validation failed: could not find property corresponding to key.\nkey: '${e}', val: '${t}'`),!1)}register(e){const{name:t}=e,n=this._themes;if(n[t])throw new Error("Theme already registered for "+t);return n[t]=e,new ae.DisposableDelegate(()=>{delete n[t]})}setCSSOverride(e,t){return this._settings.set("overrides",Object.assign(Object.assign({},this._overrides),{[e]:t}))}setTheme(e){return this._settings.set("theme",e)}isLight(e){return this._themes[e].isLight}incrFontSize(e){return this._incrFontSize(e,!0)}decrFontSize(e){return this._incrFontSize(e,!1)}themeScrollbars(e){return!!this._settings.composite["theme-scrollbars"]&&!!this._themes[e].themeScrollbars}isToggledThemeScrollbars(){return!!this._settings.composite["theme-scrollbars"]}toggleThemeScrollbars(){return this._settings.set("theme-scrollbars",!this._settings.composite["theme-scrollbars"])}getDisplayName(e){var t,n;return null!==(n=null===(t=this._themes[e])||void 0===t?void 0:t.displayName)&&void 0!==n?n:e}_incrFontSize(e,t=!0){var n;const r=(null!==(n=this.getCSS(e))&&void 0!==n?n:"13px").split(/([a-zA-Z]+)/),i=(t?1:-1)*("em"===r[1]?.1:1);return this.setCSSOverride(e,`${Number(r[0])+i}${r[1]}`)}_initOverrideProps(){const e=this._settings.schema.definitions.cssOverrides.properties;Object.keys(e).forEach(t=>{this._overrideProps[t]=e[t].description})}_loadSettings(){const e=this._outstanding,t=this._pending,n=this._requests;t&&(window.clearTimeout(t),this._pending=0);const r=this._settings,i=this._themes,o=r.composite.theme;if(e)return e.then(()=>{this._loadSettings()}).catch(()=>{this._loadSettings()}),void(this._outstanding=null);if(n[o]=n[o]?n[o]+1:1,i[o])return this._outstanding=this._loadTheme(o),void delete n[o];if(n[o]>20){const e=r.default("theme");return delete n[o],i[e]?(console.warn(`Could not load theme ${o}, using default ${e}.`),void(this._outstanding=this._loadTheme(e))):void this._onError(this._trans.__("Neither theme %1 nor default %2 loaded.",o,e))}this._pending=window.setTimeout(()=>{this._loadSettings()},75)}_loadTheme(e){const t=this._current,n=this._links,r=this._themes,i=this._splash?this._splash.show(r[e].isLight):new ae.DisposableDelegate(()=>{});n.forEach(e=>{e.parentElement&&e.parentElement.removeChild(e)}),n.length=0;const o=t?r[t].unload():Promise.resolve();return Promise.all([o,r[e].load()]).then(()=>{this._current=e,this._themeChanged.emit({name:"theme",oldValue:t,newValue:e}),this._host.hide(),requestAnimationFrame(()=>{this._host.show(),le.fitAll(this._host),i.dispose()})}).catch(e=>{this._onError(e),i.dispose()})}_onError(e){Object(m.b)({title:this._trans.__("Error Loading Theme"),body:String(e),buttons:[m.a.okButton({label:this._trans.__("OK")})]})}}var le;!function(e){e.fitAll=function e(t){Object(y.each)(t.children(),e),t.fit()}}(le||(le={}));const ue=new o.Token("@jupyterlab/apputils:ISessionContextDialogs"),ce=new o.Token("@jupyterlab/apputils:IThemeManager");var de=n(434);const he=new o.Token("@jupyterlab/apputils:IWindowResolver");class fe{get name(){return this._name}resolve(e){return pe.resolve(e).then(e=>{this._name=e})}}var pe;!function(e){const t="@jupyterlab/statedb:StateDB",n=t+":beacon",r=Math.floor(200+300*Math.random()),i=t+":window";let a=null,s=null;const l=new o.PromiseDelegate,u={};let c=null,d=!1;function h(e){if(null===e)return;const{localStorage:t}=window;t.setItem(i,`${e}-${(new Date).getTime()}`)}function f(){d=!0,a=null,l.reject(`Window name candidate "${s}" already exists`)}e.resolve=function(e){if(d)return l.promise;if(s=e,s in u)return f(),l.promise;const{localStorage:t,setTimeout:i}=window;return i(()=>{if(!d){if(!s||s in u)return f();d=!0,a=null,l.resolve(c=s),h(c)}},r),a=`${Math.random()}-${(new Date).getTime()}`,t.setItem(n,a),l.promise},window.addEventListener("storage",e=>{const{key:t,newValue:r}=e;if(null===r)return;if(t===n&&r!==a&&null!==s)return void h(d?c:s);if(d||t!==i)return;const o=r.replace(/\-\d+$/,"");u[o]=null,s&&!(s in u)||f()})}(pe||(pe={})),n.d(t,"Clipboard",(function(){return r})),n.d(t,"Collapse",(function(){return u})),n.d(t,"CommandLinker",(function(){return h})),n.d(t,"ICommandPalette",(function(){return f})),n.d(t,"ModalCommandPalette",(function(){return p})),n.d(t,"showDialog",(function(){return m.b})),n.d(t,"showErrorMessage",(function(){return m.c})),n.d(t,"Dialog",(function(){return m.a})),n.d(t,"DOMUtils",(function(){return g})),n.d(t,"HoverBox",(function(){return A.a})),n.d(t,"IFrame",(function(){return _})),n.d(t,"InputDialog",(function(){return x})),n.d(t,"MainAreaWidget",(function(){return re})),n.d(t,"Printing",(function(){return te})),n.d(t,"defaultSanitizer",(function(){return ie.a})),n.d(t,"SessionContext",(function(){return H})),n.d(t,"sessionContextDialogs",(function(){return U})),n.d(t,"Spinner",(function(){return D})),n.d(t,"ISplashScreen",(function(){return oe})),n.d(t,"Styling",(function(){return I.a})),n.d(t,"ThemeManager",(function(){return se})),n.d(t,"ISessionContextDialogs",(function(){return ue})),n.d(t,"IThemeManager",(function(){return ce})),n.d(t,"Toolbar",(function(){return G})),n.d(t,"ToolbarButtonComponent",(function(){return Z})),n.d(t,"addToolbarButtonClass",(function(){return Q})),n.d(t,"ToolbarButton",(function(){return K})),n.d(t,"CommandToolbarButtonComponent",(function(){return J})),n.d(t,"addCommandToolbarButtonClass",(function(){return $})),n.d(t,"CommandToolbarButton",(function(){return ee})),n.d(t,"ReactWidget",(function(){return Y.a})),n.d(t,"VDomRenderer",(function(){return Y.d})),n.d(t,"UseSignal",(function(){return Y.b})),n.d(t,"VDomModel",(function(){return Y.c})),n.d(t,"WidgetTracker",(function(){return de.a})),n.d(t,"IWindowResolver",(function(){return he})),n.d(t,"WindowResolver",(function(){return fe}))},function(e,t,n){"use strict";n.d(t,"a",(function(){return i}));var r=n(59);function i(e){return Object(r.a)(e).defaultView||window}},function(e,t,n){"use strict";n.d(t,"a",(function(){return r}));var r,i,o=n(1307);!function(e){function t(e,t,r=""){e.localName===t&&e.classList.add("jp-mod-styled"),"select"===e.localName&&n(e);const i=e.getElementsByTagName(t);for(let e=0;e<i.length;e++){const o=i[e];o.classList.add("jp-mod-styled"),r&&o.classList.add(r),"select"===t&&n(o)}}function n(e){const t=document.createElement("div");return t.classList.add("jp-select-wrapper"),e.addEventListener("focus",i.onFocus),e.addEventListener("blur",i.onFocus),e.classList.add("jp-mod-styled"),e.parentElement&&e.parentElement.replaceChild(t,e),t.appendChild(e),t.appendChild(o.a.element({tag:"span",stylesheet:"select",right:"8px",top:"5px",width:"18px"})),t}e.styleNode=function(e,n=""){t(e,"select",n),t(e,"textarea",n),t(e,"input",n),t(e,"button",n)},e.styleNodeByTag=t,e.wrapSelect=n}(r||(r={})),function(e){e.onFocus=function(e){const t=e.target.parentElement;t&&("focus"===e.type?t.classList.add("jp-mod-focused"):t.classList.remove("jp-mod-focused"))}}(i||(i={}))},function(e,t,n){"use strict";var r=n(442);t.a=function(e,t){return t?Object(r.a)(e,t,{clone:!1}):e}},function(e,t,n){"use strict";n.d(t,"a",(function(){return r}));var r={ZERO:0,ONE:1,TWO:2,THREE:3,FOUR:4}},function(e,t,n){"use strict";n.d(t,"a",(function(){return r}));var r={NONE:"none",PRIMARY:"primary",SUCCESS:"success",WARNING:"warning",DANGER:"danger"}},function(e,t,n){var r=n(1393),i=n(1398);e.exports=function(e,t){var n=i(e,t);return r(n)?n:void 0}},function(e,t,n){var r=n(236),i=n(1394),o=n(1395),a=r?r.toStringTag:void 0;e.exports=function(e){return null==e?void 0===e?"[object Undefined]":"[object Null]":a&&a in Object(e)?i(e):o(e)}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.removeNonHTMLProps=t.DISPLAYNAME_PREFIX=void 0;var r=n(2);t.DISPLAYNAME_PREFIX="Blueprint3";var i=["active","alignText","asyncControl","containerRef","current","elementRef","fill","icon","inputRef","intent","inline","large","loading","leftElement","leftIcon","minimal","onRemove","outlined","panel","panelClassName","popoverProps","rightElement","rightIcon","round","small","text"];t.removeNonHTMLProps=function(e,t,n){return void 0===t&&(t=i),void 0===n&&(n=!1),n&&(t=t.concat(i)),t.reduce((function(e,t){return-1!==t.indexOf("-")||e.hasOwnProperty(t)&&delete e[t],e}),r.__assign({},e))}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.I18nLabel=void 0,function(e){e.Close_Tab="Close",e.Move_Tab="Move: ",e.Move_Tabset="Move tabset",e.Maximize="Maximize tabset",e.Restore="Restore tabset",e.Float_Tab="Show selected tab in floating window",e.Overflow_Menu_Tooltip="Hidden tabs",e.Floating_Window_Message="This panel is shown in a floating window",e.Floating_Window_Show_Window="Show window",e.Floating_Window_Dock_Window="Dock window",e.Error_rendering_component="Error rendering component"}(t.I18nLabel||(t.I18nLabel={}))},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.INIT_COORDS="dnd-core/INIT_COORDS",t.BEGIN_DRAG="dnd-core/BEGIN_DRAG",t.PUBLISH_DRAG_SOURCE="dnd-core/PUBLISH_DRAG_SOURCE",t.HOVER="dnd-core/HOVER",t.DROP="dnd-core/DROP",t.END_DRAG="dnd-core/END_DRAG"},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.isFunction=function(e){return"function"==typeof e},t.noop=function(){},t.isPlainObject=function(e){if(!function(e){return"object"==typeof e&&null!==e}(e))return!1;if(null===Object.getPrototypeOf(e))return!0;for(var t=e;null!==Object.getPrototypeOf(t);)t=Object.getPrototypeOf(t);return Object.getPrototypeOf(e)===t}},function(e,t,n){"use strict";var r=n(42),i=n(10),o=n(985),a=n(986),s=n(251).isValid,l=n(27).traceIs;function u(e,t){var n=t.slice(0,t.length-1);return t?i.nestedProperty(e,n).get()||{}:e}e.exports=function e(t,n,c,d,h){var f=h.prefix,p=h.cLetter,g="_module"in n,v=u(t,f),m=u(n,f),y=u(n._template||{},f)||{},b=function(){return delete t.coloraxis,delete n.coloraxis,e(t,n,c,d,h)};if(g){var C=c._colorAxes||{},A=d(f+"coloraxis");if(A){var _=l(n,"contour")&&i.nestedProperty(n,"contours.coloring").get()||"heatmap",I=C[A];return void(I?(I[2].push(b),I[0]!==_&&(I[0]=!1,i.warn(["Ignoring coloraxis:",A,"setting","as it is linked to incompatible colorscales."].join(" ")))):C[A]=[_,n,[b]])}}var x=v[p+"min"],w=v[p+"max"],M=r(x)&&r(w)&&x<w;d(f+p+"auto",!M)?d(f+p+"mid"):(d(f+p+"min"),d(f+p+"max"));var S,T,E=v.colorscale,O=y.colorscale;(void 0!==E&&(S=!s(E)),void 0!==O&&(S=!s(O)),d(f+"autocolorscale",S),d(f+"colorscale"),d(f+"reversescale"),"marker.line."!==f)&&(f&&g&&(T=o(v)),d(f+"showscale",T)&&(f&&y&&(m._template=y),a(v,m,c)))}},function(e,t,n){"use strict";e.exports=function(e,t){(e.attr("class")||"").split(" ").forEach((function(t){0===t.indexOf("cursor-")&&e.classed(t,!1)})),t&&e.classed("cursor-"+t,!0)}},function(e,t,n){"use strict";function r(e){var t=32;return(e&=-e)&&t--,65535&e&&(t-=16),16711935&e&&(t-=8),252645135&e&&(t-=4),858993459&e&&(t-=2),1431655765&e&&(t-=1),t}t.INT_BITS=32,t.INT_MAX=2147483647,t.INT_MIN=-1<<31,t.sign=function(e){return(e>0)-(e<0)},t.abs=function(e){var t=e>>31;return(e^t)-t},t.min=function(e,t){return t^(e^t)&-(e<t)},t.max=function(e,t){return e^(e^t)&-(e<t)},t.isPow2=function(e){return!(e&e-1||!e)},t.log2=function(e){var t,n;return t=(e>65535)<<4,t|=n=((e>>>=t)>255)<<3,t|=n=((e>>>=n)>15)<<2,(t|=n=((e>>>=n)>3)<<1)|(e>>>=n)>>1},t.log10=function(e){return e>=1e9?9:e>=1e8?8:e>=1e7?7:e>=1e6?6:e>=1e5?5:e>=1e4?4:e>=1e3?3:e>=100?2:e>=10?1:0},t.popCount=function(e){return 16843009*((e=(858993459&(e-=e>>>1&1431655765))+(e>>>2&858993459))+(e>>>4)&252645135)>>>24},t.countTrailingZeros=r,t.nextPow2=function(e){return e+=0===e,--e,e|=e>>>1,e|=e>>>2,e|=e>>>4,e|=e>>>8,(e|=e>>>16)+1},t.prevPow2=function(e){return e|=e>>>1,e|=e>>>2,e|=e>>>4,e|=e>>>8,(e|=e>>>16)-(e>>>1)},t.parity=function(e){return e^=e>>>16,e^=e>>>8,e^=e>>>4,27030>>>(e&=15)&1};var i=new Array(256);!function(e){for(var t=0;t<256;++t){var n=t,r=t,i=7;for(n>>>=1;n;n>>>=1)r<<=1,r|=1&n,--i;e[t]=r<<i&255}}(i),t.reverse=function(e){return i[255&e]<<24|i[e>>>8&255]<<16|i[e>>>16&255]<<8|i[e>>>24&255]},t.interleave2=function(e,t){return(e=1431655765&((e=858993459&((e=252645135&((e=16711935&((e&=65535)|e<<8))|e<<4))|e<<2))|e<<1))|(t=1431655765&((t=858993459&((t=252645135&((t=16711935&((t&=65535)|t<<8))|t<<4))|t<<2))|t<<1))<<1},t.deinterleave2=function(e,t){return(e=65535&((e=16711935&((e=252645135&((e=858993459&((e=e>>>t&1431655765)|e>>>1))|e>>>2))|e>>>4))|e>>>16))<<16>>16},t.interleave3=function(e,t,n){return e=1227133513&((e=3272356035&((e=251719695&((e=4278190335&((e&=1023)|e<<16))|e<<8))|e<<4))|e<<2),(e|=(t=1227133513&((t=3272356035&((t=251719695&((t=4278190335&((t&=1023)|t<<16))|t<<8))|t<<4))|t<<2))<<1)|(n=1227133513&((n=3272356035&((n=251719695&((n=4278190335&((n&=1023)|n<<16))|n<<8))|n<<4))|n<<2))<<2},t.deinterleave3=function(e,t){return(e=1023&((e=4278190335&((e=251719695&((e=3272356035&((e=e>>>t&1227133513)|e>>>2))|e>>>4))|e>>>8))|e>>>16))<<22>>22},t.nextCombination=function(e){var t=e|e-1;return t+1|(~t&-~t)-1>>>r(e)+1}},function(e,t,n){var r=n(1794),i=n(1795),o="undefined"!=typeof Float64Array;function a(e,t){return e[0]-t[0]}function s(){var e,t=this.stride,n=new Array(t.length);for(e=0;e<n.length;++e)n[e]=[Math.abs(t[e]),e];n.sort(a);var r=new Array(n.length);for(e=0;e<r.length;++e)r[e]=n[e][1];return r}function l(e,t){var n=["View",t,"d",e].join("");t<0&&(n="View_Nil"+e);var i="generic"===e;if(-1===t){var o="function "+n+"(a){this.data=a;};var proto="+n+".prototype;proto.dtype='"+e+"';proto.index=function(){return -1};proto.size=0;proto.dimension=-1;proto.shape=proto.stride=proto.order=[];proto.lo=proto.hi=proto.transpose=proto.step=function(){return new "+n+"(this.data);};proto.get=proto.set=function(){};proto.pick=function(){return null};return function construct_"+n+"(a){return new "+n+"(a);}";return new Function(o)()}if(0===t){o="function "+n+"(a,d) {this.data = a;this.offset = d};var proto="+n+".prototype;proto.dtype='"+e+"';proto.index=function(){return this.offset};proto.dimension=0;proto.size=1;proto.shape=proto.stride=proto.order=[];proto.lo=proto.hi=proto.transpose=proto.step=function "+n+"_copy() {return new "+n+"(this.data,this.offset)};proto.pick=function "+n+"_pick(){return TrivialArray(this.data);};proto.valueOf=proto.get=function "+n+"_get(){return "+(i?"this.data.get(this.offset)":"this.data[this.offset]")+"};proto.set=function "+n+"_set(v){return "+(i?"this.data.set(this.offset,v)":"this.data[this.offset]=v")+"};return function construct_"+n+"(a,b,c,d){return new "+n+"(a,d)}";return new Function("TrivialArray",o)(u[e][0])}o=["'use strict'"];var a=r(t),l=a.map((function(e){return"i"+e})),c="this.offset+"+a.map((function(e){return"this.stride["+e+"]*i"+e})).join("+"),d=a.map((function(e){return"b"+e})).join(","),h=a.map((function(e){return"c"+e})).join(",");o.push("function "+n+"(a,"+d+","+h+",d){this.data=a","this.shape=["+d+"]","this.stride=["+h+"]","this.offset=d|0}","var proto="+n+".prototype","proto.dtype='"+e+"'","proto.dimension="+t),o.push("Object.defineProperty(proto,'size',{get:function "+n+"_size(){return "+a.map((function(e){return"this.shape["+e+"]"})).join("*"),"}})"),1===t?o.push("proto.order=[0]"):(o.push("Object.defineProperty(proto,'order',{get:"),t<4?(o.push("function "+n+"_order(){"),2===t?o.push("return (Math.abs(this.stride[0])>Math.abs(this.stride[1]))?[1,0]:[0,1]}})"):3===t&&o.push("var s0=Math.abs(this.stride[0]),s1=Math.abs(this.stride[1]),s2=Math.abs(this.stride[2]);if(s0>s1){if(s1>s2){return [2,1,0];}else if(s0>s2){return [1,2,0];}else{return [1,0,2];}}else if(s0>s2){return [2,0,1];}else if(s2>s1){return [0,1,2];}else{return [0,2,1];}}})")):o.push("ORDER})")),o.push("proto.set=function "+n+"_set("+l.join(",")+",v){"),i?o.push("return this.data.set("+c+",v)}"):o.push("return this.data["+c+"]=v}"),o.push("proto.get=function "+n+"_get("+l.join(",")+"){"),i?o.push("return this.data.get("+c+")}"):o.push("return this.data["+c+"]}"),o.push("proto.index=function "+n+"_index(",l.join(),"){return "+c+"}"),o.push("proto.hi=function "+n+"_hi("+l.join(",")+"){return new "+n+"(this.data,"+a.map((function(e){return["(typeof i",e,"!=='number'||i",e,"<0)?this.shape[",e,"]:i",e,"|0"].join("")})).join(",")+","+a.map((function(e){return"this.stride["+e+"]"})).join(",")+",this.offset)}");var f=a.map((function(e){return"a"+e+"=this.shape["+e+"]"})),p=a.map((function(e){return"c"+e+"=this.stride["+e+"]"}));o.push("proto.lo=function "+n+"_lo("+l.join(",")+"){var b=this.offset,d=0,"+f.join(",")+","+p.join(","));for(var g=0;g<t;++g)o.push("if(typeof i"+g+"==='number'&&i"+g+">=0){d=i"+g+"|0;b+=c"+g+"*d;a"+g+"-=d}");o.push("return new "+n+"(this.data,"+a.map((function(e){return"a"+e})).join(",")+","+a.map((function(e){return"c"+e})).join(",")+",b)}"),o.push("proto.step=function "+n+"_step("+l.join(",")+"){var "+a.map((function(e){return"a"+e+"=this.shape["+e+"]"})).join(",")+","+a.map((function(e){return"b"+e+"=this.stride["+e+"]"})).join(",")+",c=this.offset,d=0,ceil=Math.ceil");for(g=0;g<t;++g)o.push("if(typeof i"+g+"==='number'){d=i"+g+"|0;if(d<0){c+=b"+g+"*(a"+g+"-1);a"+g+"=ceil(-a"+g+"/d)}else{a"+g+"=ceil(a"+g+"/d)}b"+g+"*=d}");o.push("return new "+n+"(this.data,"+a.map((function(e){return"a"+e})).join(",")+","+a.map((function(e){return"b"+e})).join(",")+",c)}");var v=new Array(t),m=new Array(t);for(g=0;g<t;++g)v[g]="a[i"+g+"]",m[g]="b[i"+g+"]";o.push("proto.transpose=function "+n+"_transpose("+l+"){"+l.map((function(e,t){return e+"=("+e+"===undefined?"+t+":"+e+"|0)"})).join(";"),"var a=this.shape,b=this.stride;return new "+n+"(this.data,"+v.join(",")+","+m.join(",")+",this.offset)}"),o.push("proto.pick=function "+n+"_pick("+l+"){var a=[],b=[],c=this.offset");for(g=0;g<t;++g)o.push("if(typeof i"+g+"==='number'&&i"+g+">=0){c=(c+this.stride["+g+"]*i"+g+")|0}else{a.push(this.shape["+g+"]);b.push(this.stride["+g+"])}");return o.push("var ctor=CTOR_LIST[a.length+1];return ctor(this.data,a,b,c)}"),o.push("return function construct_"+n+"(data,shape,stride,offset){return new "+n+"(data,"+a.map((function(e){return"shape["+e+"]"})).join(",")+","+a.map((function(e){return"stride["+e+"]"})).join(",")+",offset)}"),new Function("CTOR_LIST","ORDER",o.join("\n"))(u[e],s)}var u={float32:[],float64:[],int8:[],int16:[],int32:[],uint8:[],uint16:[],uint32:[],array:[],uint8_clamped:[],bigint64:[],biguint64:[],buffer:[],generic:[]};e.exports=function(e,t,n,r){if(void 0===e)return(0,u.array[0])([]);"number"==typeof e&&(e=[e]),void 0===t&&(t=[e.length]);var a=t.length;if(void 0===n){n=new Array(a);for(var s=a-1,c=1;s>=0;--s)n[s]=c,c*=t[s]}if(void 0===r){r=0;for(s=0;s<a;++s)n[s]<0&&(r-=(t[s]-1)*n[s])}for(var d=function(e){if(i(e))return"buffer";if(o)switch(Object.prototype.toString.call(e)){case"[object Float64Array]":return"float64";case"[object Float32Array]":return"float32";case"[object Int8Array]":return"int8";case"[object Int16Array]":return"int16";case"[object Int32Array]":return"int32";case"[object Uint8Array]":return"uint8";case"[object Uint16Array]":return"uint16";case"[object Uint32Array]":return"uint32";case"[object Uint8ClampedArray]":return"uint8_clamped";case"[object BigInt64Array]":return"bigint64";case"[object BigUint64Array]":return"biguint64"}return Array.isArray(e)?"array":"generic"}(e),h=u[d];h.length<=a+1;)h.push(l(d,h.length-1));return(0,h[a+1])(e,t,n,r)}},function(e,t){e.exports=function(e){"string"==typeof e&&(e=[e]);for(var t=[].slice.call(arguments,1),n=[],r=0;r<e.length-1;r++)n.push(e[r],t[r]||"");return n.push(e[r]),n.join("")}},function(e,t,n){"use strict";var r=n(1799),i=n(1800),o=n(1038),a=n(1801),s=n(1819),l=n(395);function u(e){this.gl=e,this.gl.lastAttribCount=0,this._vref=this._fref=this._relink=this.vertShader=this.fragShader=this.program=this.attributes=this.uniforms=this.types=null}var c=u.prototype;function d(e,t){return e.name<t.name?-1:1}c.bind=function(){var e;this.program||this._relink();var t=this.gl.getProgramParameter(this.program,this.gl.ACTIVE_ATTRIBUTES),n=this.gl.lastAttribCount;if(t>n)for(e=n;e<t;e++)this.gl.enableVertexAttribArray(e);else if(n>t)for(e=t;e<n;e++)this.gl.disableVertexAttribArray(e);this.gl.lastAttribCount=t,this.gl.useProgram(this.program)},c.dispose=function(){for(var e=this.gl.lastAttribCount,t=0;t<e;t++)this.gl.disableVertexAttribArray(t);this.gl.lastAttribCount=0,this._fref&&this._fref.dispose(),this._vref&&this._vref.dispose(),this.attributes=this.types=this.vertShader=this.fragShader=this.program=this._relink=this._fref=this._vref=null},c.update=function(e,t,n,u){if(!t||1===arguments.length){var c=e;e=c.vertex,t=c.fragment,n=c.uniforms,u=c.attributes}var h=this,f=h.gl,p=h._vref;h._vref=a.shader(f,f.VERTEX_SHADER,e),p&&p.dispose(),h.vertShader=h._vref.shader;var g=this._fref;if(h._fref=a.shader(f,f.FRAGMENT_SHADER,t),g&&g.dispose(),h.fragShader=h._fref.shader,!n||!u){var v=f.createProgram();if(f.attachShader(v,h.fragShader),f.attachShader(v,h.vertShader),f.linkProgram(v),!f.getProgramParameter(v,f.LINK_STATUS)){var m=f.getProgramInfoLog(v);throw new l(m,"Error linking program:"+m)}n=n||s.uniforms(f,v),u=u||s.attributes(f,v),f.deleteProgram(v)}(u=u.slice()).sort(d);var y,b=[],C=[],A=[];for(y=0;y<u.length;++y){var _=u[y];if(_.type.indexOf("mat")>=0){for(var I=0|_.type.charAt(_.type.length-1),x=new Array(I),w=0;w<I;++w)x[w]=A.length,C.push(_.name+"["+w+"]"),"number"==typeof _.location?A.push(_.location+w):Array.isArray(_.location)&&_.location.length===I&&"number"==typeof _.location[w]?A.push(0|_.location[w]):A.push(-1);b.push({name:_.name,type:_.type,locations:x})}else b.push({name:_.name,type:_.type,locations:[A.length]}),C.push(_.name),"number"==typeof _.location?A.push(0|_.location):A.push(-1)}var M=0;for(y=0;y<A.length;++y)if(A[y]<0){for(;A.indexOf(M)>=0;)M+=1;A[y]=M}var S=new Array(n.length);function T(){h.program=a.program(f,h._vref,h._fref,C,A);for(var e=0;e<n.length;++e)S[e]=f.getUniformLocation(h.program,n[e].name)}T(),h._relink=T,h.types={uniforms:o(n),attributes:o(u)},h.attributes=i(f,h,b,A),Object.defineProperty(h,"uniforms",r(f,h,n,S))},e.exports=function(e,t,n,r,i){var o=new u(e);return o.update(t,n,r,i),o}},function(e,t){function n(e){for(var t=[[e.x,e.y]];e.parent;)e=e.parent,t.push([e.x,e.y]);return t.reverse()}function r(e,t,n,r){var i,o,a,s,l,u,c=Math.abs,d=[];for(i=e<n?1:-1,o=t<r?1:-1,l=(a=c(n-e))-(s=c(r-t));d.push([e,t]),e!==n||t!==r;)(u=2*l)>-s&&(l-=s,e+=i),u<a&&(l+=a,t+=o);return d}t.backtrace=n,t.biBacktrace=function(e,t){var r=n(e),i=n(t);return r.concat(i.reverse())},t.pathLength=function(e){var t,n,r,i,o,a=0;for(t=1;t<e.length;++t)n=e[t-1],r=e[t],i=n[0]-r[0],o=n[1]-r[1],a+=Math.sqrt(i*i+o*o);return a},t.interpolate=r,t.expandPath=function(e){var t,n,i,o,a,s,l=[],u=e.length;if(u<2)return l;for(a=0;a<u-1;++a)for(t=e[a],n=e[a+1],o=(i=r(t[0],t[1],n[0],n[1])).length,s=0;s<o-1;++s)l.push(i[s]);return l.push(e[u-1]),l},t.smoothenPath=function(e,t){var n,i,o,a,s,l,u,c,d,h=t.length,f=t[0][0],p=t[0][1],g=t[h-1][0],v=t[h-1][1];for(o=[[n=f,i=p]],a=2;a<h;++a){for(u=r(n,i,(l=t[a])[0],l[1]),d=!1,s=1;s<u.length;++s)if(c=u[s],!e.isWalkableAt(c[0],c[1])){d=!0;break}d&&(lastValidCoord=t[a-1],o.push(lastValidCoord),n=lastValidCoord[0],i=lastValidCoord[1])}return o.push([g,v]),o},t.compressPath=function(e){if(e.length<3)return e;var t,n,r,i,o,a,s=[],l=e[0][0],u=e[0][1],c=e[1][0],d=e[1][1],h=c-l,f=d-u;for(h/=o=Math.sqrt(h*h+f*f),f/=o,s.push([l,u]),a=2;a<e.length;a++)t=c,n=d,r=h,i=f,h=(c=e[a][0])-t,f=(d=e[a][1])-n,f/=o=Math.sqrt(h*h+f*f),(h/=o)===r&&f===i||s.push([t,n]);return s.push([c,d]),s}},function(e,t){e.exports=function(e){return e}},function(e,t,n){"use strict";n.r(t);var r=n(917);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";n.r(t);var r=n(15),i=n(107),o=n(57),a=n(105);function s(e,t){return function(){return null}}var l=n(141),u=n(59),c=n(202),d=n(512),h=n(87),f=n(286),p=n(100),g=n(53),v=n(37),m=n(186),y=n(166);n.d(t,"capitalize",(function(){return r.a})),n.d(t,"createChainedFunction",(function(){return i.a})),n.d(t,"createSvgIcon",(function(){return o.a})),n.d(t,"debounce",(function(){return a.a})),n.d(t,"deprecatedPropType",(function(){return s})),n.d(t,"isMuiElement",(function(){return l.a})),n.d(t,"ownerDocument",(function(){return u.a})),n.d(t,"ownerWindow",(function(){return c.a})),n.d(t,"requirePropFactory",(function(){return d.a})),n.d(t,"setRef",(function(){return h.a})),n.d(t,"unsupportedProp",(function(){return f.a})),n.d(t,"useControlled",(function(){return p.a})),n.d(t,"useEventCallback",(function(){return g.a})),n.d(t,"useForkRef",(function(){return v.a})),n.d(t,"unstable_useId",(function(){return m.a})),n.d(t,"useIsFocusVisible",(function(){return y.a}))},function(e,t,n){var r=n(1580),i=n(1581),o=n(942),a=n(1582);e.exports=function(e){return r(e)||i(e)||o(e)||a()}},function(e,t,n){var r=n(877);e.exports=function(e,t){e.prototype=Object.create(t.prototype),e.prototype.constructor=e,r(e,t)}},function(e,t,n){"use strict";t.a={black:"#000",white:"#fff"}},function(e,t,n){"use strict";function r(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}n.d(t,"a",(function(){return r}))},function(e,t,n){"use strict";function r(e){return null!=e&&!(Array.isArray(e)&&0===e.length)}function i(e){var t=arguments.length>1&&void 0!==arguments[1]&&arguments[1];return e&&(r(e.value)&&""!==e.value||t&&r(e.defaultValue)&&""!==e.defaultValue)}function o(e){return e.startAdornment}n.d(t,"b",(function(){return i})),n.d(t,"a",(function(){return o}))},function(e,t,n){"use strict";var r=s(n(1480)),i=s(n(1484)),o=s(n(880)),a=s(n(878));function s(e){return e&&e.__esModule?e:{default:e}}e.exports={Transition:a.default,TransitionGroup:o.default,ReplaceTransition:i.default,CSSTransition:r.default}},function(e,t,n){"use strict";var r="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e},i="object"===("undefined"==typeof window?"undefined":r(window))&&"object"===("undefined"==typeof document?"undefined":r(document))&&9===document.nodeType;t.a=i},function(e,t,n){"use strict";n.d(t,"a",(function(){return a}));var r,i,o=n(20),a=function(){o.InstancedBufferGeometry.call(this),this.type="LineSegmentsGeometry";this.setIndex([0,2,1,2,3,1,2,4,3,4,5,3,4,6,5,6,7,5]),this.setAttribute("position",new o.Float32BufferAttribute([-1,2,0,1,2,0,-1,1,0,1,1,0,-1,0,0,1,0,0,-1,-1,0,1,-1,0],3)),this.setAttribute("uv",new o.Float32BufferAttribute([-1,2,1,2,-1,1,1,1,-1,-1,1,-1,-1,-2,1,-2],2))};a.prototype=Object.assign(Object.create(o.InstancedBufferGeometry.prototype),{constructor:a,isLineSegmentsGeometry:!0,applyMatrix4:function(e){var t=this.attributes.instanceStart,n=this.attributes.instanceEnd;return void 0!==t&&(t.applyMatrix4(e),n.applyMatrix4(e),t.data.needsUpdate=!0),null!==this.boundingBox&&this.computeBoundingBox(),null!==this.boundingSphere&&this.computeBoundingSphere(),this},setPositions:function(e){var t;e instanceof Float32Array?t=e:Array.isArray(e)&&(t=new Float32Array(e));var n=new o.InstancedInterleavedBuffer(t,6,1);return this.setAttribute("instanceStart",new o.InterleavedBufferAttribute(n,3,0)),this.setAttribute("instanceEnd",new o.InterleavedBufferAttribute(n,3,3)),this.computeBoundingBox(),this.computeBoundingSphere(),this},setColors:function(e){var t;e instanceof Float32Array?t=e:Array.isArray(e)&&(t=new Float32Array(e));var n=new o.InstancedInterleavedBuffer(t,6,1);return this.setAttribute("instanceColorStart",new o.InterleavedBufferAttribute(n,3,0)),this.setAttribute("instanceColorEnd",new o.InterleavedBufferAttribute(n,3,3)),this},fromWireframeGeometry:function(e){return this.setPositions(e.attributes.position.array),this},fromEdgesGeometry:function(e){return this.setPositions(e.attributes.position.array),this},fromMesh:function(e){return this.fromWireframeGeometry(new o.WireframeGeometry(e.geometry)),this},fromLineSegments:function(e){var t=e.geometry;return t.isGeometry?this.setPositions(t.vertices):t.isBufferGeometry&&this.setPositions(t.attributes.position.array),this},computeBoundingBox:(i=new o.Box3,function(){null===this.boundingBox&&(this.boundingBox=new o.Box3);var e=this.attributes.instanceStart,t=this.attributes.instanceEnd;void 0!==e&&void 0!==t&&(this.boundingBox.setFromBufferAttribute(e),i.setFromBufferAttribute(t),this.boundingBox.union(i))}),computeBoundingSphere:(r=new o.Vector3,function(){null===this.boundingSphere&&(this.boundingSphere=new o.Sphere),null===this.boundingBox&&this.computeBoundingBox();var e=this.attributes.instanceStart,t=this.attributes.instanceEnd;if(void 0!==e&&void 0!==t){var n=this.boundingSphere.center;this.boundingBox.getCenter(n);for(var i=0,a=0,s=e.count;a<s;a++)r.fromBufferAttribute(e,a),i=Math.max(i,n.distanceToSquared(r)),r.fromBufferAttribute(t,a),i=Math.max(i,n.distanceToSquared(r));this.boundingSphere.radius=Math.sqrt(i),isNaN(this.boundingSphere.radius)&&console.error("THREE.LineSegmentsGeometry.computeBoundingSphere(): Computed radius is NaN. The instanced position data is likely to have NaN values.",this)}}),toJSON:function(){},applyMatrix:function(e){return console.warn("THREE.LineSegmentsGeometry: applyMatrix() has been renamed to applyMatrix4()."),this.applyMatrix4(e)}})},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(22)),s=n(4),l=n(37),u=n(53),c=n(7),d=n(166),h=n(74),f=n(2382),p="undefined"==typeof window?o.useEffect:o.useLayoutEffect;var g=function(e){var t=e.classes,n=e.pulsate,r=void 0!==n&&n,i=e.rippleX,a=e.rippleY,l=e.rippleSize,c=e.in,d=e.onExited,h=void 0===d?function(){}:d,f=e.timeout,g=o.useState(!1),v=g[0],m=g[1],y=Object(s.a)(t.ripple,t.rippleVisible,r&&t.ripplePulsate),b={width:l,height:l,top:-l/2+a,left:-l/2+i},C=Object(s.a)(t.child,v&&t.childLeaving,r&&t.childPulsate),A=Object(u.a)(h);return p((function(){if(!c){m(!0);var e=setTimeout(A,f);return function(){clearTimeout(e)}}}),[A,c,f]),o.createElement("span",{className:y,style:b},o.createElement("span",{className:C}))},v=o.forwardRef((function(e,t){var n=e.center,a=void 0!==n&&n,l=e.classes,u=e.className,c=Object(i.a)(e,["center","classes","className"]),d=o.useState([]),p=d[0],v=d[1],m=o.useRef(0),y=o.useRef(null);o.useEffect((function(){y.current&&(y.current(),y.current=null)}),[p]);var b=o.useRef(!1),C=o.useRef(null),A=o.useRef(null),_=o.useRef(null);o.useEffect((function(){return function(){clearTimeout(C.current)}}),[]);var I=o.useCallback((function(e){var t=e.pulsate,n=e.rippleX,r=e.rippleY,i=e.rippleSize,a=e.cb;v((function(e){return[].concat(Object(h.a)(e),[o.createElement(g,{key:m.current,classes:l,timeout:550,pulsate:t,rippleX:n,rippleY:r,rippleSize:i})])})),m.current+=1,y.current=a}),[l]),x=o.useCallback((function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=arguments.length>2?arguments[2]:void 0,r=t.pulsate,i=void 0!==r&&r,o=t.center,s=void 0===o?a||t.pulsate:o,l=t.fakeElement,u=void 0!==l&&l;if("mousedown"===e.type&&b.current)b.current=!1;else{"touchstart"===e.type&&(b.current=!0);var c,d,h,f=u?null:_.current,p=f?f.getBoundingClientRect():{width:0,height:0,left:0,top:0};if(s||0===e.clientX&&0===e.clientY||!e.clientX&&!e.touches)c=Math.round(p.width/2),d=Math.round(p.height/2);else{var g=e.touches?e.touches[0]:e,v=g.clientX,m=g.clientY;c=Math.round(v-p.left),d=Math.round(m-p.top)}if(s)(h=Math.sqrt((2*Math.pow(p.width,2)+Math.pow(p.height,2))/3))%2==0&&(h+=1);else{var y=2*Math.max(Math.abs((f?f.clientWidth:0)-c),c)+2,x=2*Math.max(Math.abs((f?f.clientHeight:0)-d),d)+2;h=Math.sqrt(Math.pow(y,2)+Math.pow(x,2))}e.touches?null===A.current&&(A.current=function(){I({pulsate:i,rippleX:c,rippleY:d,rippleSize:h,cb:n})},C.current=setTimeout((function(){A.current&&(A.current(),A.current=null)}),80)):I({pulsate:i,rippleX:c,rippleY:d,rippleSize:h,cb:n})}}),[a,I]),w=o.useCallback((function(){x({},{pulsate:!0})}),[x]),M=o.useCallback((function(e,t){if(clearTimeout(C.current),"touchend"===e.type&&A.current)return e.persist(),A.current(),A.current=null,void(C.current=setTimeout((function(){M(e,t)})));A.current=null,v((function(e){return e.length>0?e.slice(1):e})),y.current=t}),[]);return o.useImperativeHandle(t,(function(){return{pulsate:w,start:x,stop:M}}),[w,x,M]),o.createElement("span",Object(r.a)({className:Object(s.a)(l.root,u),ref:_},c),o.createElement(f.a,{component:null,exit:!0},p))})),m=Object(c.a)((function(e){return{root:{overflow:"hidden",pointerEvents:"none",position:"absolute",zIndex:0,top:0,right:0,bottom:0,left:0,borderRadius:"inherit"},ripple:{opacity:0,position:"absolute"},rippleVisible:{opacity:.3,transform:"scale(1)",animation:"$enter ".concat(550,"ms ").concat(e.transitions.easing.easeInOut)},ripplePulsate:{animationDuration:"".concat(e.transitions.duration.shorter,"ms")},child:{opacity:1,display:"block",width:"100%",height:"100%",borderRadius:"50%",backgroundColor:"currentColor"},childLeaving:{opacity:0,animation:"$exit ".concat(550,"ms ").concat(e.transitions.easing.easeInOut)},childPulsate:{position:"absolute",left:0,top:0,animation:"$pulsate 2500ms ".concat(e.transitions.easing.easeInOut," 200ms infinite")},"@keyframes enter":{"0%":{transform:"scale(0)",opacity:.1},"100%":{transform:"scale(1)",opacity:.3}},"@keyframes exit":{"0%":{opacity:1},"100%":{opacity:0}},"@keyframes pulsate":{"0%":{transform:"scale(1)"},"50%":{transform:"scale(0.92)"},"100%":{transform:"scale(1)"}}}}),{flip:!1,name:"MuiTouchRipple"})(o.memo(v)),y=o.forwardRef((function(e,t){var n=e.action,c=e.buttonRef,h=e.centerRipple,f=void 0!==h&&h,p=e.children,g=e.classes,v=e.className,y=e.component,b=void 0===y?"button":y,C=e.disabled,A=void 0!==C&&C,_=e.disableRipple,I=void 0!==_&&_,x=e.disableTouchRipple,w=void 0!==x&&x,M=e.focusRipple,S=void 0!==M&&M,T=e.focusVisibleClassName,E=e.onBlur,O=e.onClick,k=e.onFocus,L=e.onFocusVisible,D=e.onKeyDown,P=e.onKeyUp,N=e.onMouseDown,j=e.onMouseLeave,R=e.onMouseUp,z=e.onTouchEnd,F=e.onTouchMove,B=e.onTouchStart,H=e.onDragLeave,U=e.tabIndex,V=void 0===U?0:U,Y=e.TouchRippleProps,W=e.type,G=void 0===W?"button":W,X=Object(i.a)(e,["action","buttonRef","centerRipple","children","classes","className","component","disabled","disableRipple","disableTouchRipple","focusRipple","focusVisibleClassName","onBlur","onClick","onFocus","onFocusVisible","onKeyDown","onKeyUp","onMouseDown","onMouseLeave","onMouseUp","onTouchEnd","onTouchMove","onTouchStart","onDragLeave","tabIndex","TouchRippleProps","type"]),q=o.useRef(null);var Z=o.useRef(null),Q=o.useState(!1),K=Q[0],J=Q[1];A&&K&&J(!1);var $=Object(d.a)(),ee=$.isFocusVisible,te=$.onBlurVisible,ne=$.ref;function re(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:w;return Object(u.a)((function(r){return t&&t(r),!n&&Z.current&&Z.current[e](r),!0}))}o.useImperativeHandle(n,(function(){return{focusVisible:function(){J(!0),q.current.focus()}}}),[]),o.useEffect((function(){K&&S&&!I&&Z.current.pulsate()}),[I,S,K]);var ie=re("start",N),oe=re("stop",H),ae=re("stop",R),se=re("stop",(function(e){K&&e.preventDefault(),j&&j(e)})),le=re("start",B),ue=re("stop",z),ce=re("stop",F),de=re("stop",(function(e){K&&(te(e),J(!1)),E&&E(e)}),!1),he=Object(u.a)((function(e){q.current||(q.current=e.currentTarget),ee(e)&&(J(!0),L&&L(e)),k&&k(e)})),fe=function(){var e=a.findDOMNode(q.current);return b&&"button"!==b&&!("A"===e.tagName&&e.href)},pe=o.useRef(!1),ge=Object(u.a)((function(e){S&&!pe.current&&K&&Z.current&&" "===e.key&&(pe.current=!0,e.persist(),Z.current.stop(e,(function(){Z.current.start(e)}))),e.target===e.currentTarget&&fe()&&" "===e.key&&e.preventDefault(),D&&D(e),e.target===e.currentTarget&&fe()&&"Enter"===e.key&&!A&&(e.preventDefault(),O&&O(e))})),ve=Object(u.a)((function(e){S&&" "===e.key&&Z.current&&K&&!e.defaultPrevented&&(pe.current=!1,e.persist(),Z.current.stop(e,(function(){Z.current.pulsate(e)}))),P&&P(e),O&&e.target===e.currentTarget&&fe()&&" "===e.key&&!e.defaultPrevented&&O(e)})),me=b;"button"===me&&X.href&&(me="a");var ye={};"button"===me?(ye.type=G,ye.disabled=A):("a"===me&&X.href||(ye.role="button"),ye["aria-disabled"]=A);var be=Object(l.a)(c,t),Ce=Object(l.a)(ne,q),Ae=Object(l.a)(be,Ce),_e=o.useState(!1),Ie=_e[0],xe=_e[1];o.useEffect((function(){xe(!0)}),[]);var we=Ie&&!I&&!A;return o.createElement(me,Object(r.a)({className:Object(s.a)(g.root,v,K&&[g.focusVisible,T],A&&g.disabled),onBlur:de,onClick:O,onFocus:he,onKeyDown:ge,onKeyUp:ve,onMouseDown:ie,onMouseLeave:se,onMouseUp:ae,onDragLeave:oe,onTouchEnd:ue,onTouchMove:ce,onTouchStart:le,ref:Ae,tabIndex:A?-1:V},ye,X),p,we?o.createElement(m,Object(r.a)({ref:Z,center:f},Y)):null)}));t.a=Object(c.a)({root:{display:"inline-flex",alignItems:"center",justifyContent:"center",position:"relative",WebkitTapHighlightColor:"transparent",backgroundColor:"transparent",outline:0,border:0,margin:0,borderRadius:0,padding:0,cursor:"pointer",userSelect:"none",verticalAlign:"middle","-moz-appearance":"none","-webkit-appearance":"none",textDecoration:"none",color:"inherit","&::-moz-focus-inner":{borderStyle:"none"},"&$disabled":{pointerEvents:"none",cursor:"default"},"@media print":{colorAdjust:"exact"}},disabled:{},focusVisible:{}},{name:"MuiButtonBase"})(y)},,,function(e,t,n){(function(e){var r=void 0!==e&&e||"undefined"!=typeof self&&self||window,i=Function.prototype.apply;function o(e,t){this._id=e,this._clearFn=t}t.setTimeout=function(){return new o(i.call(setTimeout,r,arguments),clearTimeout)},t.setInterval=function(){return new o(i.call(setInterval,r,arguments),clearInterval)},t.clearTimeout=t.clearInterval=function(e){e&&e.close()},o.prototype.unref=o.prototype.ref=function(){},o.prototype.close=function(){this._clearFn.call(r,this._id)},t.enroll=function(e,t){clearTimeout(e._idleTimeoutId),e._idleTimeout=t},t.unenroll=function(e){clearTimeout(e._idleTimeoutId),e._idleTimeout=-1},t._unrefActive=t.active=function(e){clearTimeout(e._idleTimeoutId);var t=e._idleTimeout;t>=0&&(e._idleTimeoutId=setTimeout((function(){e._onTimeout&&e._onTimeout()}),t))},n(1342),t.setImmediate="undefined"!=typeof self&&self.setImmediate||void 0!==e&&e.setImmediate||this&&this.setImmediate,t.clearImmediate="undefined"!=typeof self&&self.clearImmediate||void 0!==e&&e.clearImmediate||this&&this.clearImmediate}).call(this,n(61))},function(e,t){e.exports=function(e,t){return e===t||e!=e&&t!=t}},function(e,t,n){var r=n(146).Symbol;e.exports=r},function(e,t,n){(function(e){var r=n(146),i=n(1419),o=t&&!t.nodeType&&t,a=o&&"object"==typeof e&&e&&!e.nodeType&&e,s=a&&a.exports===o?r.Buffer:void 0,l=(s?s.isBuffer:void 0)||i;e.exports=l}).call(this,n(188)(e))},function(e,t,n){var r=n(1422),i=n(492),o=n(1423),a=n(862),s=n(1424),l=n(208),u=n(851),c=u(r),d=u(i),h=u(o),f=u(a),p=u(s),g=l;(r&&"[object DataView]"!=g(new r(new ArrayBuffer(1)))||i&&"[object Map]"!=g(new i)||o&&"[object Promise]"!=g(o.resolve())||a&&"[object Set]"!=g(new a)||s&&"[object WeakMap]"!=g(new s))&&(g=function(e){var t=l(e),n="[object Object]"==t?e.constructor:void 0,r=n?u(n):"";if(r)switch(r){case c:return"[object DataView]";case d:return"[object Map]";case h:return"[object Promise]";case f:return"[object Set]";case p:return"[object WeakMap]"}return t}),e.exports=g},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.Utils=t.Keys=t.Classes=void 0;var r=n(2);r.__exportStar(n(1455),t),r.__exportStar(n(1461),t),r.__exportStar(n(1462),t),r.__exportStar(n(1463),t),r.__exportStar(n(870),t),r.__exportStar(n(1464),t),r.__exportStar(n(1465),t),r.__exportStar(n(1466),t),r.__exportStar(n(871),t),r.__exportStar(n(366),t),r.__exportStar(n(872),t),r.__exportStar(n(209),t),r.__exportStar(n(502),t);var i=r.__importStar(n(1467));t.Classes=i;var o=r.__importStar(n(1468));t.Keys=o;var a=r.__importStar(n(240));t.Utils=a},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.getRef=t.setRef=void 0;var r=n(2);r.__exportStar(n(1456),t),r.__exportStar(n(1457),t),r.__exportStar(n(503),t),r.__exportStar(n(1458),t),r.__exportStar(n(1459),t),r.__exportStar(n(1460),t);var i=n(502);Object.defineProperty(t,"setRef",{enumerable:!0,get:function(){return i.setRef}}),Object.defineProperty(t,"getRef",{enumerable:!0,get:function(){return i.getRef}})},function(e,t,n){"use strict";function r(e,t,n,r){var i=r?" !== ":" === ",o=r?" || ":" && ",a=r?"!":"",s=r?"":"!";switch(e){case"null":return t+i+"null";case"array":return a+"Array.isArray("+t+")";case"object":return"("+a+t+o+"typeof "+t+i+'"object"'+o+s+"Array.isArray("+t+"))";case"integer":return"(typeof "+t+i+'"number"'+o+s+"("+t+" % 1)"+o+t+i+t+(n?o+a+"isFinite("+t+")":"")+")";case"number":return"(typeof "+t+i+'"'+e+'"'+(n?o+a+"isFinite("+t+")":"")+")";default:return"typeof "+t+i+'"'+e+'"'}}e.exports={copy:function(e,t){for(var n in t=t||{},e)t[n]=e[n];return t},checkDataType:r,checkDataTypes:function(e,t,n){switch(e.length){case 1:return r(e[0],t,n,!0);default:var i="",a=o(e);for(var s in a.array&&a.object&&(i=a.null?"(":"(!"+t+" || ",i+="typeof "+t+' !== "object")',delete a.null,delete a.array,delete a.object),a.number&&delete a.integer,a)i+=(i?" && ":"")+r(s,t,n,!0);return i}},coerceToTypes:function(e,t){if(Array.isArray(t)){for(var n=[],r=0;r<t.length;r++){var o=t[r];(i[o]||"array"===e&&"array"===o)&&(n[n.length]=o)}if(n.length)return n}else{if(i[t])return[t];if("array"===e&&"array"===t)return["array"]}},toHash:o,getProperty:l,escapeQuotes:u,equal:n(510),ucs2length:n(1525),varOccurences:function(e,t){t+="[^0-9]";var n=e.match(new RegExp(t,"g"));return n?n.length:0},varReplace:function(e,t,n){return t+="([^0-9])",n=n.replace(/\$/g,"$$$$"),e.replace(new RegExp(t,"g"),n+"$1")},schemaHasRules:function(e,t){if("boolean"==typeof e)return!e;for(var n in e)if(t[n])return!0},schemaHasRulesExcept:function(e,t,n){if("boolean"==typeof e)return!e&&"not"!=n;for(var r in e)if(r!=n&&t[r])return!0},schemaUnknownRules:function(e,t){if("boolean"==typeof e)return;for(var n in e)if(!t[n])return n},toQuotedString:c,getPathExpr:function(e,t,n,r){return f(e,n?"'/' + "+t+(r?"":".replace(/~/g, '~0').replace(/\\//g, '~1')"):r?"'[' + "+t+" + ']'":"'[\\'' + "+t+" + '\\']'")},getPath:function(e,t,n){var r=c(n?"/"+p(t):l(t));return f(e,r)},getData:function(e,t,n){var r,i,o,a;if(""===e)return"rootData";if("/"==e[0]){if(!d.test(e))throw new Error("Invalid JSON-pointer: "+e);i=e,o="rootData"}else{if(!(a=e.match(h)))throw new Error("Invalid JSON-pointer: "+e);if(r=+a[1],"#"==(i=a[2])){if(r>=t)throw new Error("Cannot access property/index "+r+" levels up, current level is "+t);return n[t-r]}if(r>t)throw new Error("Cannot access data "+r+" levels up, current level is "+t);if(o="data"+(t-r||""),!i)return o}for(var s=o,u=i.split("/"),c=0;c<u.length;c++){var f=u[c];f&&(o+=l(g(f)),s+=" && "+o)}return s},unescapeFragment:function(e){return g(decodeURIComponent(e))},unescapeJsonPointer:g,escapeFragment:function(e){return encodeURIComponent(p(e))},escapeJsonPointer:p};var i=o(["string","number","integer","boolean","null"]);function o(e){for(var t={},n=0;n<e.length;n++)t[e[n]]=!0;return t}var a=/^[a-z$_][a-z$_0-9]*$/i,s=/'|\\/g;function l(e){return"number"==typeof e?"["+e+"]":a.test(e)?"."+e:"['"+u(e)+"']"}function u(e){return e.replace(s,"\\$&").replace(/\n/g,"\\n").replace(/\r/g,"\\r").replace(/\f/g,"\\f").replace(/\t/g,"\\t")}function c(e){return"'"+u(e)+"'"}var d=/^\/(?:[^~]|~0|~1)*$/,h=/^([0-9]+)(#|\/(?:[^~]|~0|~1)*)?$/;function f(e,t){return'""'==e?t:(e+" + "+t).replace(/([^\\])' \+ '/g,"$1")}function p(e){return e.replace(/~/g,"~0").replace(/\//g,"~1")}function g(e){return e.replace(/~1/g,"/").replace(/~0/g,"~")}},function(e,t,n){"use strict";var r=n(331);n.d(t,"a",(function(){return r.b}))},function(e,t,n){"use strict";var r,i=this&&this.__extends||(r=function(e,t){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(e[n]=t[n])})(e,t)},function(e,t){function n(){this.constructor=e}r(e,t),e.prototype=null===t?Object.create(t):(n.prototype=t.prototype,new n)});Object.defineProperty(t,"__esModule",{value:!0});var o=n(289),a=n(244),s=n(190),l=n(370),u=n(111),c=n(131),d=n(245),h=n(371),f=n(290),p=n(934),g=function(e){function t(n,r,i){var o=e.call(this,i)||this;return o._adjustedSize=0,o._calculatedBorderBarSize=0,o._location=n,o._drawChildren=[],o._attributes.id="border_"+n.getName(),t._attributeDefinitions.fromJson(r,o._attributes),i._addNode(o),o}return i(t,e),t._fromJson=function(e,n){var r=new t(s.default.getByName(e.location),e,n);return e.children&&(r._children=e.children.map((function(e){var t=f.default._fromJson(e,n);return t._setParent(r),t}))),r},t._createAttributeDefinitions=function(){var e=new a.default;return e.add("type",t.TYPE,!0),e.add("selected",-1),e.add("show",!0).setType(o.default.BOOLEAN),e.addInherited("barSize","borderBarSize").setType(o.default.INT).setFrom(0),e.addInherited("enableDrop","borderEnableDrop").setType(o.default.BOOLEAN),e.addInherited("className","borderClassName").setType(o.default.STRING),e.addInherited("autoSelectTabWhenOpen","borderAutoSelectTabWhenOpen").setType(o.default.BOOLEAN),e.addInherited("autoSelectTabWhenClosed","borderAutoSelectTabWhenClosed").setType(o.default.BOOLEAN),e.addInherited("size","borderSize").setType(o.default.NUMBER),e.addInherited("minSize","borderMinSize").setType(o.default.NUMBER),e},t.prototype.getLocation=function(){return this._location},t.prototype.getTabHeaderRect=function(){return this._tabHeaderRect},t.prototype.getRect=function(){return this._tabHeaderRect},t.prototype.getContentRect=function(){return this._contentRect},t.prototype.isEnableDrop=function(){return this._getAttr("enableDrop")},t.prototype.isAutoSelectTab=function(e){return null==e&&(e=-1!==this.getSelected()),e?this._getAttr("autoSelectTabWhenOpen"):this._getAttr("autoSelectTabWhenClosed")},t.prototype.getClassName=function(){return this._getAttributeAsStringOrUndefined("className")},t.prototype.calcBorderBarSize=function(e){var t=this._getAttr("barSize");this._calculatedBorderBarSize=0!==t?t:e.borderBarSize},t.prototype.getBorderBarSize=function(){return this._calculatedBorderBarSize},t.prototype.getSize=function(){return this._getAttr("size")},t.prototype.getMinSize=function(){return this._getAttr("minSize")},t.prototype.getSelected=function(){return this._attributes.selected},t.prototype.getSelectedNode=function(){if(-1!==this.getSelected())return this._children[this.getSelected()]},t.prototype.getOrientation=function(){return this._location.getOrientation()},t.prototype.isMaximized=function(){return!1},t.prototype.isShowing=function(){return this._attributes.show},t.prototype._setSelected=function(e){this._attributes.selected=e},t.prototype._setSize=function(e){this._attributes.size=e},t.prototype._updateAttrs=function(e){t._attributeDefinitions.update(e,this._attributes)},t.prototype._getDrawChildren=function(){return this._drawChildren},t.prototype._setAdjustedSize=function(e){this._adjustedSize=e},t.prototype._getAdjustedSize=function(){return this._adjustedSize},t.prototype._layoutBorderOuter=function(e,t){this.calcBorderBarSize(t);var n=this._location.split(e,this.getBorderBarSize());return this._tabHeaderRect=n.start,n.end},t.prototype._layoutBorderInner=function(e,t){var n=this;this._drawChildren=[];var r=this._location,i=r.split(e,this._adjustedSize+this._model.getSplitterSize()),o=r.reflect().split(i.start,this._model.getSplitterSize());if(this._contentRect=o.end,this._children.forEach((function(e,r){e._layout(n._contentRect,t),e._setVisible(r===n.getSelected()),n._drawChildren.push(e)})),-1===this.getSelected())return e;var a=new h.default(this._model);return a._setParent(this),a._setRect(o.start),this._drawChildren.push(a),i.end},t.prototype._remove=function(e){var t=this._removeChild(e);-1!==this.getSelected()&&p.adjustSelectedIndex(this,t)},t.prototype.canDrop=function(e,t,n){if(e.getType()===f.default.TYPE){var r,i=s.default.CENTER;if(this._tabHeaderRect.contains(t,n)){if(this._location._orientation===u.default.VERT)if(this._children.length>0){for(var o=(v=this._children[0].getTabRect()).y,a=v.height,d=this._tabHeaderRect.x,h=0,p=0;p<this._children.length;p++){if(h=(v=this._children[p].getTabRect()).x+v.width/2,t>=d&&t<h){var g=new c.default(v.x-2,o,3,a);r=new l.default(this,g,i,p,"flexlayout__outline_rect");break}d=h}if(null==r){g=new c.default(v.getRight()-2,o,3,a);r=new l.default(this,g,i,this._children.length,"flexlayout__outline_rect")}}else{g=new c.default(this._tabHeaderRect.x+1,this._tabHeaderRect.y+2,3,18);r=new l.default(this,g,i,0,"flexlayout__outline_rect")}else if(this._children.length>0){var v,m=(v=this._children[0].getTabRect()).x,y=v.width;for(d=this._tabHeaderRect.y,h=0,p=0;p<this._children.length;p++){if(h=(v=this._children[p].getTabRect()).y+v.height/2,n>=d&&n<h){g=new c.default(m,v.y-2,y,3);r=new l.default(this,g,i,p,"flexlayout__outline_rect");break}d=h}if(null==r){g=new c.default(m,v.getBottom()-2,y,3);r=new l.default(this,g,i,this._children.length,"flexlayout__outline_rect")}}else{g=new c.default(this._tabHeaderRect.x+2,this._tabHeaderRect.y+1,18,3);r=new l.default(this,g,i,0,"flexlayout__outline_rect")}if(!e._canDockInto(e,r))return}else if(-1!==this.getSelected()&&this._contentRect.contains(t,n)){g=this._contentRect;if(r=new l.default(this,g,i,-1,"flexlayout__outline_rect"),!e._canDockInto(e,r))return}return r}},t.prototype.drop=function(e,t,n,r){var i=0,o=e.getParent();void 0!==o&&(i=o._removeChild(e),p.adjustSelectedIndex(o,i)),e.getType()===f.default.TYPE&&o===this&&i<n&&n>0&&n--;var a=n;-1===a&&(a=this._children.length),e.getType()===f.default.TYPE&&this._addChild(e,a),(r||!1!==r&&this.isAutoSelectTab())&&this._setSelected(a),this._model._tidy()},t.prototype._toJson=function(){var e={};return t._attributeDefinitions.toJson(e,this._attributes),e.location=this._location.getName(),e.children=this._children.map((function(e){return e._toJson()})),e},t.prototype._getSplitterBounds=function(e,t){void 0===t&&(t=!1);var n=[0,0],r=t?this.getMinSize():0,i=this._model._getOuterInnerRects().outer,o=this._model._getOuterInnerRects().inner;return this._location===s.default.TOP?(n[0]=i.y+r,n[1]=o.getBottom()-e.getHeight()):this._location===s.default.LEFT?(n[0]=i.x+r,n[1]=o.getRight()-e.getWidth()):this._location===s.default.BOTTOM?(n[0]=o.y,n[1]=i.getBottom()-e.getHeight()-r):this._location===s.default.RIGHT&&(n[0]=o.x,n[1]=i.getRight()-e.getWidth()-r),n},t.prototype._calculateSplit=function(e,t){var n=this._getSplitterBounds(e);return this._location===s.default.BOTTOM||this._location===s.default.RIGHT?Math.max(0,n[1]-t):Math.max(0,t-n[0])},t.prototype._getAttributeDefinitions=function(){return t._attributeDefinitions},t.TYPE="border",t._attributeDefinitions=t._createAttributeDefinitions(),t}(d.default);t.default=g},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(289),i=function(){function e(){this.attributes=[],this.nameToAttribute={}}return e.prototype.addWithAll=function(e,t,n,i){var o=new r.default(e,t,n,i);return this.attributes.push(o),this.nameToAttribute[e]=o,o},e.prototype.addInherited=function(e,t){return this.addWithAll(e,t,void 0,!1)},e.prototype.add=function(e,t,n){return this.addWithAll(e,void 0,t,n)},e.prototype.getAttributes=function(){return this.attributes},e.prototype.getModelName=function(e){var t=this.nameToAttribute[e];if(void 0!==t)return t.modelName},e.prototype.toJson=function(e,t){this.attributes.forEach((function(n){var r=t[n.name];(n.alwaysWriteJson||r!==n.defaultValue)&&(e[n.name]=r)}))},e.prototype.fromJson=function(e,t){this.attributes.forEach((function(n){var r=e[n.name];t[n.name]=void 0===r?n.defaultValue:r}))},e.prototype.update=function(e,t){this.attributes.forEach((function(n){var r=e[n.name];void 0!==r&&(t[n.name]=r)}))},e.prototype.setDefaults=function(e){this.attributes.forEach((function(t){e[t.name]=t.defaultValue}))},e}();t.default=i},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(190),i=n(111),o=n(131),a=function(){function e(e){this._dirty=!1,this._tempSize=0,this._model=e,this._attributes={},this._children=[],this._fixed=!1,this._rect=o.default.empty(),this._visible=!1,this._listeners={}}return e.prototype.getId=function(){var e=this._attributes.id;return void 0!==e||(e=this._model._nextUniqueId(),this._setId(e)),e},e.prototype.getModel=function(){return this._model},e.prototype.getType=function(){return this._attributes.type},e.prototype.getParent=function(){return this._parent},e.prototype.getChildren=function(){return this._children},e.prototype.getRect=function(){return this._rect},e.prototype.isVisible=function(){return this._visible},e.prototype.getOrientation=function(){return void 0===this._parent?i.default.HORZ:i.default.flip(this._parent.getOrientation())},e.prototype.setEventListener=function(e,t){this._listeners[e]=t},e.prototype.removeEventListener=function(e){delete this._listeners[e]},e.prototype._setId=function(e){this._attributes.id=e},e.prototype._fireEvent=function(e,t){void 0!==this._listeners[e]&&this._listeners[e](t)},e.prototype._getAttr=function(e){var t=this._attributes[e];if(void 0===t){var n=this._getAttributeDefinitions().getModelName(e);void 0!==n&&(t=this._model._getAttribute(n))}return t},e.prototype._forEachNode=function(e,t){e(this,t),t++,this._children.forEach((function(n){n._forEachNode(e,t)}))},e.prototype._setVisible=function(e){e!==this._visible&&(this._fireEvent("visibility",{visible:e}),this._visible=e)},e.prototype._getDrawChildren=function(){return this._children},e.prototype._setParent=function(e){this._parent=e},e.prototype._setRect=function(e){this._rect=e},e.prototype._setWeight=function(e){this._attributes.weight=e},e.prototype._setSelected=function(e){this._attributes.selected=e},e.prototype._isFixed=function(){return this._fixed},e.prototype._layout=function(e,t){this._rect=e},e.prototype._findDropTargetNode=function(e,t,n){var r;if(this._rect.contains(t,n)&&void 0===(r=this.canDrop(e,t,n))&&0!==this._children.length)for(var i=0,o=this._children;i<o.length;i++){if(void 0!==(r=o[i]._findDropTargetNode(e,t,n)))break}return r},e.prototype.canDrop=function(e,t,n){},e.prototype._canDockInto=function(e,t){if(null!=t){if(t.location===r.default.CENTER&&!1===t.node.isEnableDrop())return!1;if(t.location===r.default.CENTER&&"tabset"===e.getType()&&void 0!==e.getName())return!1;if(t.location!==r.default.CENTER&&!1===t.node.isEnableDivide())return!1;if(this._model._getOnAllowDrop())return this._model._getOnAllowDrop()(e,t)}return!0},e.prototype._removeChild=function(e){var t=this._children.indexOf(e);return-1!==t&&this._children.splice(t,1),this._dirty=!0,t},e.prototype._addChild=function(e,t){return null!=t?this._children.splice(t,0,e):(this._children.push(e),t=this._children.length-1),e._parent=this,this._dirty=!0,t},e.prototype._removeAll=function(){this._children=[],this._dirty=!0},e.prototype._styleWithPosition=function(e){return null==e&&(e={}),this._rect.styleWithPosition(e)},e.prototype._getTempSize=function(){return this._tempSize},e.prototype._setTempSize=function(e){this._tempSize=e},e.prototype.isEnableDivide=function(){return!0},e.prototype._toAttributeString=function(){return JSON.stringify(this._attributes,void 0,"\t")},e.prototype._getAttributeAsStringOrUndefined=function(e){var t=this._attributes[e];if(void 0!==t)return t},e.prototype._getAttributeAsNumberOrUndefined=function(e){var t=this._attributes[e];if(void 0!==t)return t},e}();t.default=a},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.get=function(e,t,n){return t.split(".").reduce((function(e,t){return e&&e[t]?e[t]:n||null}),e)},t.without=function(e,t){return e.filter((function(e){return e!==t}))},t.isString=function(e){return"string"==typeof e},t.isObject=function(e){return"object"==typeof e},t.xor=function(e,t){var n=new Map,r=function(e){return n.set(e,n.has(e)?n.get(e)+1:1)};e.forEach(r),t.forEach(r);var i=[];return n.forEach((function(e,t){1===e&&i.push(t)})),i},t.intersection=function(e,t){return e.filter((function(e){return t.indexOf(e)>-1}))}},function(e,t,n){"use strict";n.r(t);var r=n(915);n.d(t,"default",(function(){return r.a}))},function(e,t,n){var r;void 0===(r=function(e,t){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.create_UUID=t.carpet_data_factory=t.colorNameToHex=void 0;const n={aliceblue:15792383,antiquewhite:16444375,aqua:65535,aquamarine:8388564,azure:15794175,beige:16119260,bisque:16770244,black:0,blanchedalmond:16772045,blue:255,blueviolet:9055202,brown:10824234,burlywood:14596231,cadetblue:6266528,chartreuse:8388352,chocolate:13789470,coral:16744272,cornflowerblue:6591981,cornsilk:16775388,crimson:14423100,cyan:65535,darkblue:139,darkcyan:35723,darkgoldenrod:12092939,darkgray:11119017,darkgreen:25600,darkkhaki:12433259,darkmagenta:9109643,darkolivegreen:5597999,darkorange:16747520,darkorchid:10040012,darkred:9109504,darksalmon:15308410,darkseagreen:9419919,darkslateblue:4734347,darkslategray:3100495,darkturquoise:52945,darkviolet:9699539,deeppink:16716947,deepskyblue:49151,dimgray:6908265,dodgerblue:2003199,firebrick:11674146,floralwhite:16775920,forestgreen:2263842,fuchsia:16711935,gainsboro:14474460,ghostwhite:16316671,gold:16766720,goldenrod:14329120,gray:8421504,green:32768,greenyellow:11403055,honeydew:15794160,hotpink:16738740,indianred:13458524,indigo:4915330,ivory:16777200,khaki:15787660,lavender:15132410,lavenderblush:16773365,lawngreen:8190976,lemonchiffon:16775885,lightblue:11393254,lightcoral:15761536,lightcyan:14745599,lightgoldenrodyellow:16448210,lightgrey:13882323,lightgreen:9498256,lightpink:16758465,lightsalmon:16752762,lightseagreen:2142890,lightskyblue:8900346,lightslategray:7833753,lightsteelblue:11584734,lightyellow:16777184,lime:65280,limegreen:3329330,linen:16445670,magenta:16711935,maroon:8388608,mediumaquamarine:6737322,mediumblue:205,mediumorchid:12211667,mediumpurple:9662680,mediumseagreen:3978097,mediumslateblue:8087790,mediumspringgreen:64154,mediumturquoise:4772300,mediumvioletred:13047173,midnightblue:1644912,mintcream:16121850,mistyrose:16770273,moccasin:16770229,navajowhite:16768685,navy:128,oldlace:16643558,olive:8421376,olivedrab:7048739,orange:16753920,orangered:16729344,orchid:14315734,palegoldenrod:15657130,palegreen:10025880,paleturquoise:11529966,palevioletred:14184595,papayawhip:16773077,peachpuff:16767673,peru:13468991,pink:16761035,plum:14524637,powderblue:11591910,purple:8388736,rebeccapurple:6697881,red:16711680,rosybrown:12357519,royalblue:4286945,saddlebrown:9127187,salmon:16416882,sandybrown:16032864,seagreen:3050327,seashell:16774638,sienna:10506797,silver:12632256,skyblue:8900331,slateblue:6970061,slategray:7372944,snow:16775930,springgreen:65407,steelblue:4620980,tan:13808780,teal:32896,thistle:14204888,tomato:16737095,turquoise:4251856,violet:15631086,wheat:16113331,white:16777215,whitesmoke:16119285,yellow:16776960,yellowgreen:10145074};t.colorNameToHex=function(e){return"number"==typeof e?e:void 0!==n[e.toLowerCase()]?n[e.toLowerCase()]:3735794},t.carpet_data_factory=function(e,t,n){const r=[],i=[],o=[],a=[],s=[];for(let l=0;l<e.length;l++)for(let u=0;u<e[0].length;u++)r.push(u),i.push(l),o.push(e[l][u]),a.push(t[l][u]),s.push(n[l][u]);return{a:r,b:i,x:o,y:a,z:s}},t.create_UUID=function(){var e=(new Date).getTime();return"xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx".replace(/[xy]/g,(function(t){var n=(e+16*Math.random())%16|0;return e=Math.floor(e/16),("x"==t?n:3&n|8).toString(16)}))}}.apply(t,[n,t]))||(e.exports=r)},function(e,t,n){"use strict";var r=Array.isArray,i="undefined"!=typeof ArrayBuffer&&ArrayBuffer.isView?ArrayBuffer:{isView:function(){return!1}},o="undefined"==typeof DataView?function(){}:DataView;function a(e){return i.isView(e)&&!(e instanceof o)}function s(e){return r(e)||a(e)}function l(e,t,n){if(s(e)){if(s(e[0])){for(var r=n,i=0;i<e.length;i++)r=t(r,e[i].length);return r}return e.length}return 0}t.isTypedArray=a,t.isArrayOrTypedArray=s,t.isArray1D=function(e){return!s(e[0])},t.ensureArray=function(e,t){return r(e)||(e=[]),e.length=t,e},t.concat=function(){var e,t,n,i,o,a,s,l,u=[],c=!0,d=0;for(n=0;n<arguments.length;n++)(a=(i=arguments[n]).length)&&(t?u.push(i):(t=i,o=a),r(i)?e=!1:(c=!1,d?e!==i.constructor&&(e=!1):e=i.constructor),d+=a);if(!d)return[];if(!u.length)return t;if(c)return t.concat.apply(t,u);if(e){for((s=new e(d)).set(t),n=0;n<u.length;n++)i=u[n],s.set(i,o),o+=i.length;return s}for(s=new Array(d),l=0;l<t.length;l++)s[l]=t[l];for(n=0;n<u.length;n++){for(i=u[n],l=0;l<i.length;l++)s[o+l]=i[l];o+=l}return s},t.maxRowLength=function(e){return l(e,Math.max,0)},t.minRowLength=function(e){return l(e,Math.min,1/0)}},function(e,t,n){"use strict";var r=n(381),i=n(94)({editType:"none"});i.family.dflt=r.HOVERFONT,i.size.dflt=r.HOVERFONTSIZE,e.exports={clickmode:{valType:"flaglist",flags:["event","select"],dflt:"event",editType:"plot",extras:["none"]},dragmode:{valType:"enumerated",values:["zoom","pan","select","lasso","drawclosedpath","drawopenpath","drawline","drawrect","drawcircle","orbit","turntable",!1],dflt:"zoom",editType:"modebar"},hovermode:{valType:"enumerated",values:["x","y","closest",!1,"x unified","y unified"],editType:"modebar"},hoverdistance:{valType:"integer",min:-1,dflt:20,editType:"none"},spikedistance:{valType:"integer",min:-1,dflt:20,editType:"none"},hoverlabel:{bgcolor:{valType:"color",editType:"none"},bordercolor:{valType:"color",editType:"none"},font:i,align:{valType:"enumerated",values:["left","right","auto"],dflt:"auto",editType:"none"},namelength:{valType:"integer",min:-1,dflt:15,editType:"none"},editType:"none"},selectdirection:{valType:"enumerated",values:["h","v","d","any"],dflt:"any",editType:"none"}}},function(e,t,n){"use strict";var r=n(123),i={Greys:[[0,"rgb(0,0,0)"],[1,"rgb(255,255,255)"]],YlGnBu:[[0,"rgb(8,29,88)"],[.125,"rgb(37,52,148)"],[.25,"rgb(34,94,168)"],[.375,"rgb(29,145,192)"],[.5,"rgb(65,182,196)"],[.625,"rgb(127,205,187)"],[.75,"rgb(199,233,180)"],[.875,"rgb(237,248,217)"],[1,"rgb(255,255,217)"]],Greens:[[0,"rgb(0,68,27)"],[.125,"rgb(0,109,44)"],[.25,"rgb(35,139,69)"],[.375,"rgb(65,171,93)"],[.5,"rgb(116,196,118)"],[.625,"rgb(161,217,155)"],[.75,"rgb(199,233,192)"],[.875,"rgb(229,245,224)"],[1,"rgb(247,252,245)"]],YlOrRd:[[0,"rgb(128,0,38)"],[.125,"rgb(189,0,38)"],[.25,"rgb(227,26,28)"],[.375,"rgb(252,78,42)"],[.5,"rgb(253,141,60)"],[.625,"rgb(254,178,76)"],[.75,"rgb(254,217,118)"],[.875,"rgb(255,237,160)"],[1,"rgb(255,255,204)"]],Bluered:[[0,"rgb(0,0,255)"],[1,"rgb(255,0,0)"]],RdBu:[[0,"rgb(5,10,172)"],[.35,"rgb(106,137,247)"],[.5,"rgb(190,190,190)"],[.6,"rgb(220,170,132)"],[.7,"rgb(230,145,90)"],[1,"rgb(178,10,28)"]],Reds:[[0,"rgb(220,220,220)"],[.2,"rgb(245,195,157)"],[.4,"rgb(245,160,105)"],[1,"rgb(178,10,28)"]],Blues:[[0,"rgb(5,10,172)"],[.35,"rgb(40,60,190)"],[.5,"rgb(70,100,245)"],[.6,"rgb(90,120,245)"],[.7,"rgb(106,137,247)"],[1,"rgb(220,220,220)"]],Picnic:[[0,"rgb(0,0,255)"],[.1,"rgb(51,153,255)"],[.2,"rgb(102,204,255)"],[.3,"rgb(153,204,255)"],[.4,"rgb(204,204,255)"],[.5,"rgb(255,255,255)"],[.6,"rgb(255,204,255)"],[.7,"rgb(255,153,255)"],[.8,"rgb(255,102,204)"],[.9,"rgb(255,102,102)"],[1,"rgb(255,0,0)"]],Rainbow:[[0,"rgb(150,0,90)"],[.125,"rgb(0,0,200)"],[.25,"rgb(0,25,255)"],[.375,"rgb(0,152,255)"],[.5,"rgb(44,255,150)"],[.625,"rgb(151,255,0)"],[.75,"rgb(255,234,0)"],[.875,"rgb(255,111,0)"],[1,"rgb(255,0,0)"]],Portland:[[0,"rgb(12,51,131)"],[.25,"rgb(10,136,186)"],[.5,"rgb(242,211,56)"],[.75,"rgb(242,143,56)"],[1,"rgb(217,30,30)"]],Jet:[[0,"rgb(0,0,131)"],[.125,"rgb(0,60,170)"],[.375,"rgb(5,255,255)"],[.625,"rgb(255,255,0)"],[.875,"rgb(250,0,0)"],[1,"rgb(128,0,0)"]],Hot:[[0,"rgb(0,0,0)"],[.3,"rgb(230,0,0)"],[.6,"rgb(255,210,0)"],[1,"rgb(255,255,255)"]],Blackbody:[[0,"rgb(0,0,0)"],[.2,"rgb(230,0,0)"],[.4,"rgb(230,210,0)"],[.7,"rgb(255,255,255)"],[1,"rgb(160,200,255)"]],Earth:[[0,"rgb(0,0,130)"],[.1,"rgb(0,180,180)"],[.2,"rgb(40,210,40)"],[.4,"rgb(230,230,50)"],[.6,"rgb(120,70,20)"],[1,"rgb(255,255,255)"]],Electric:[[0,"rgb(0,0,0)"],[.15,"rgb(30,0,100)"],[.4,"rgb(120,0,100)"],[.6,"rgb(160,90,0)"],[.8,"rgb(230,200,0)"],[1,"rgb(255,250,220)"]],Viridis:[[0,"#440154"],[.06274509803921569,"#48186a"],[.12549019607843137,"#472d7b"],[.18823529411764706,"#424086"],[.25098039215686274,"#3b528b"],[.3137254901960784,"#33638d"],[.3764705882352941,"#2c728e"],[.4392156862745098,"#26828e"],[.5019607843137255,"#21918c"],[.5647058823529412,"#1fa088"],[.6274509803921569,"#28ae80"],[.6901960784313725,"#3fbc73"],[.7529411764705882,"#5ec962"],[.8156862745098039,"#84d44b"],[.8784313725490196,"#addc30"],[.9411764705882353,"#d8e219"],[1,"#fde725"]],Cividis:[[0,"rgb(0,32,76)"],[.058824,"rgb(0,42,102)"],[.117647,"rgb(0,52,110)"],[.176471,"rgb(39,63,108)"],[.235294,"rgb(60,74,107)"],[.294118,"rgb(76,85,107)"],[.352941,"rgb(91,95,109)"],[.411765,"rgb(104,106,112)"],[.470588,"rgb(117,117,117)"],[.529412,"rgb(131,129,120)"],[.588235,"rgb(146,140,120)"],[.647059,"rgb(161,152,118)"],[.705882,"rgb(176,165,114)"],[.764706,"rgb(192,177,109)"],[.823529,"rgb(209,191,102)"],[.882353,"rgb(225,204,92)"],[.941176,"rgb(243,219,79)"],[1,"rgb(255,233,69)"]]},o=i.RdBu;function a(e){var t=0;if(!Array.isArray(e)||e.length<2)return!1;if(!e[0]||!e[e.length-1])return!1;if(0!=+e[0][0]||1!=+e[e.length-1][0])return!1;for(var n=0;n<e.length;n++){var i=e[n];if(2!==i.length||+i[0]<t||!r(i[1]).isValid())return!1;t=+i[0]}return!0}e.exports={scales:i,defaultScale:o,get:function(e,t){if(t||(t=o),!e)return t;function n(){try{e=i[e]||JSON.parse(e)}catch(n){e=t}}return"string"==typeof e&&(n(),"string"==typeof e&&n()),a(e)?e:t},isValid:function(e){return void 0!==i[e]||a(e)}}},function(e,t,n){"use strict";var r=n(253).dfltConfig,i=n(966),o=e.exports={};function a(e,t){if(e&&e.apply)try{return void e.apply(console,t)}catch(e){}for(var n=0;n<t.length;n++)try{e(t[n])}catch(e){console.log(t[n])}}o.log=function(){var e;if(r.logging>1){var t=["LOG:"];for(e=0;e<arguments.length;e++)t.push(arguments[e]);a(console.trace||console.log,t)}if(r.notifyOnLogging>1){var n=[];for(e=0;e<arguments.length;e++)n.push(arguments[e]);i(n.join("<br>"),"long")}},o.warn=function(){var e;if(r.logging>0){var t=["WARN:"];for(e=0;e<arguments.length;e++)t.push(arguments[e]);a(console.trace||console.log,t)}if(r.notifyOnLogging>0){var n=[];for(e=0;e<arguments.length;e++)n.push(arguments[e]);i(n.join("<br>"),"stick")}},o.error=function(){var e;if(r.logging>0){var t=["ERROR:"];for(e=0;e<arguments.length;e++)t.push(arguments[e]);a(console.error,t)}if(r.notifyOnLogging>0){var n=[];for(e=0;e<arguments.length;e++)n.push(arguments[e]);i(n.join("<br>"),"stick")}}},function(e,t,n){"use strict";var r={staticPlot:{valType:"boolean",dflt:!1},plotlyServerURL:{valType:"string",dflt:""},editable:{valType:"boolean",dflt:!1},edits:{annotationPosition:{valType:"boolean",dflt:!1},annotationTail:{valType:"boolean",dflt:!1},annotationText:{valType:"boolean",dflt:!1},axisTitleText:{valType:"boolean",dflt:!1},colorbarPosition:{valType:"boolean",dflt:!1},colorbarTitleText:{valType:"boolean",dflt:!1},legendPosition:{valType:"boolean",dflt:!1},legendText:{valType:"boolean",dflt:!1},shapePosition:{valType:"boolean",dflt:!1},titleText:{valType:"boolean",dflt:!1}},autosizable:{valType:"boolean",dflt:!1},responsive:{valType:"boolean",dflt:!1},fillFrame:{valType:"boolean",dflt:!1},frameMargins:{valType:"number",dflt:0,min:0,max:.5},scrollZoom:{valType:"flaglist",flags:["cartesian","gl3d","geo","mapbox"],extras:[!0,!1],dflt:"gl3d+geo+mapbox"},doubleClick:{valType:"enumerated",values:[!1,"reset","autosize","reset+autosize"],dflt:"reset+autosize"},doubleClickDelay:{valType:"number",dflt:300,min:0},showAxisDragHandles:{valType:"boolean",dflt:!0},showAxisRangeEntryBoxes:{valType:"boolean",dflt:!0},showTips:{valType:"boolean",dflt:!0},showLink:{valType:"boolean",dflt:!1},linkText:{valType:"string",dflt:"Edit chart",noBlank:!0},sendData:{valType:"boolean",dflt:!0},showSources:{valType:"any",dflt:!1},displayModeBar:{valType:"enumerated",values:["hover",!0,!1],dflt:"hover"},showSendToCloud:{valType:"boolean",dflt:!1},showEditInChartStudio:{valType:"boolean",dflt:!1},modeBarButtonsToRemove:{valType:"any",dflt:[]},modeBarButtonsToAdd:{valType:"any",dflt:[]},modeBarButtons:{valType:"any",dflt:!1},toImageButtonOptions:{valType:"any",dflt:{}},displaylogo:{valType:"boolean",dflt:!0},watermark:{valType:"boolean",dflt:!1},plotGlPixelRatio:{valType:"number",dflt:2,min:1,max:4},setBackground:{valType:"any",dflt:"transparent"},topojsonURL:{valType:"string",noBlank:!0,dflt:"https://cdn.plot.ly/"},mapboxAccessToken:{valType:"string",dflt:null},logging:{valType:"integer",min:0,max:2,dflt:1},notifyOnLogging:{valType:"integer",min:0,max:2,dflt:0},queueLength:{valType:"integer",min:0,dflt:0},globalTransforms:{valType:"any",dflt:[]},locale:{valType:"string",dflt:"en-US"},locales:{valType:"any",dflt:{}}},i={};!function e(t,n){for(var r in t){var i=t[r];i.valType?n[r]=i.dflt:(n[r]||(n[r]={}),e(i,n[r]))}}(r,i),e.exports={configAttributes:r,dfltConfig:i}},function(e,t){e.exports=function(e,t){var n=t[0],r=t[1],i=t[2],o=t[3],a=t[4],s=t[5],l=t[6],u=t[7],c=t[8],d=t[9],h=t[10],f=t[11],p=t[12],g=t[13],v=t[14],m=t[15],y=n*s-r*a,b=n*l-i*a,C=n*u-o*a,A=r*l-i*s,_=r*u-o*s,I=i*u-o*l,x=c*g-d*p,w=c*v-h*p,M=c*m-f*p,S=d*v-h*g,T=d*m-f*g,E=h*m-f*v,O=y*E-b*T+C*S+A*M-_*w+I*x;if(!O)return null;return O=1/O,e[0]=(s*E-l*T+u*S)*O,e[1]=(i*T-r*E-o*S)*O,e[2]=(g*I-v*_+m*A)*O,e[3]=(h*_-d*I-f*A)*O,e[4]=(l*M-a*E-u*w)*O,e[5]=(n*E-i*M+o*w)*O,e[6]=(v*C-p*I-m*b)*O,e[7]=(c*I-h*C+f*b)*O,e[8]=(a*T-s*M+u*x)*O,e[9]=(r*M-n*T-o*x)*O,e[10]=(p*_-g*C+m*y)*O,e[11]=(d*C-c*_-f*y)*O,e[12]=(s*w-a*S-l*x)*O,e[13]=(n*S-r*w+i*x)*O,e[14]=(g*b-p*A-v*y)*O,e[15]=(c*A-d*b+h*y)*O,e}},function(e,t){e.exports=function(e,t,n){var r=t[0],i=t[1],o=t[2],a=t[3],s=t[4],l=t[5],u=t[6],c=t[7],d=t[8],h=t[9],f=t[10],p=t[11],g=t[12],v=t[13],m=t[14],y=t[15],b=n[0],C=n[1],A=n[2],_=n[3];return e[0]=b*r+C*s+A*d+_*g,e[1]=b*i+C*l+A*h+_*v,e[2]=b*o+C*u+A*f+_*m,e[3]=b*a+C*c+A*p+_*y,b=n[4],C=n[5],A=n[6],_=n[7],e[4]=b*r+C*s+A*d+_*g,e[5]=b*i+C*l+A*h+_*v,e[6]=b*o+C*u+A*f+_*m,e[7]=b*a+C*c+A*p+_*y,b=n[8],C=n[9],A=n[10],_=n[11],e[8]=b*r+C*s+A*d+_*g,e[9]=b*i+C*l+A*h+_*v,e[10]=b*o+C*u+A*f+_*m,e[11]=b*a+C*c+A*p+_*y,b=n[12],C=n[13],A=n[14],_=n[15],e[12]=b*r+C*s+A*d+_*g,e[13]=b*i+C*l+A*h+_*v,e[14]=b*o+C*u+A*f+_*m,e[15]=b*a+C*c+A*p+_*y,e}},function(e,t,n){"use strict";var r=n(27);t.getDelay=function(e){return e._has&&(e._has("gl3d")||e._has("gl2d")||e._has("mapbox"))?500:0},t.getRedrawFunc=function(e){return function(){var t=e._fullLayout||{};!(t._has&&t._has("polar"))&&e.data&&e.data[0]&&e.data[0].r||r.getComponentMethod("colorbar","draw")(e)}},t.encodeSVG=function(e){return"data:image/svg+xml,"+encodeURIComponent(e)},t.encodeJSON=function(e){return"data:application/json,"+encodeURIComponent(e)};var i=window.URL||window.webkitURL;t.createObjectURL=function(e){return i.createObjectURL(e)},t.revokeObjectURL=function(e){return i.revokeObjectURL(e)},t.createBlob=function(e,t){if("svg"===t)return new window.Blob([e],{type:"image/svg+xml;charset=utf-8"});if("full-json"===t)return new window.Blob([e],{type:"application/json;charset=utf-8"});var n=function(e){for(var t=e.length,n=new ArrayBuffer(t),r=new Uint8Array(n),i=0;i<t;i++)r[i]=e.charCodeAt(i);return n}(window.atob(e));return new window.Blob([n],{type:"image/"+t})},t.octetStream=function(e){document.location.href="data:application/octet-stream"+e},t.IMAGE_URL_PREFIX=/^data:image\/\w+;base64,/,t.MSG_IE_BAD_FORMAT="Sorry IE does not support downloading from canvas. Try {format:'svg'} instead."},function(e,t,n){"use strict";e.exports=function(e,t,n){var i=e*t,o=r*e,a=o-(o-e),s=e-a,l=r*t,u=l-(l-t),c=t-u,d=s*c-(i-a*u-s*u-a*c);if(n)return n[0]=d,n[1]=i,n;return[d,i]};var r=+(Math.pow(2,27)+1)},function(e,t,n){var r=n(860),i=n(2087),o=n(174);e.exports=function(e){return o(e)?r(e,!0):i(e)}},function(e,t,n){var r=n(208),i=n(158);e.exports=function(e){return"symbol"==typeof e||i(e)&&"[object Symbol]"==r(e)}},function(e,t,n){"use strict";n.r(t);var r=n(916);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";n.r(t),function(e,r){n.d(t,"Debouncer",(function(){return v})),n.d(t,"Poll",(function(){return p})),n.d(t,"RateLimiter",(function(){return g})),n.d(t,"Throttler",(function(){return m}));var i=n(17),o=n(16),a=function(e,t){return(a=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(e,t)};function s(e,t){function n(){this.constructor=e}a(e,t),e.prototype=null===t?Object.create(t):(n.prototype=t.prototype,new n)}var l=function(){return(l=Object.assign||function(e){for(var t,n=1,r=arguments.length;n<r;n++)for(var i in t=arguments[n])Object.prototype.hasOwnProperty.call(t,i)&&(e[i]=t[i]);return e}).apply(this,arguments)};function u(e,t,n,r){return new(n||(n=Promise))((function(i,o){function a(e){try{l(r.next(e))}catch(e){o(e)}}function s(e){try{l(r.throw(e))}catch(e){o(e)}}function l(e){e.done?i(e.value):new n((function(t){t(e.value)})).then(a,s)}l((r=r.apply(e,t||[])).next())}))}function c(e,t){var n,r,i,o,a={label:0,sent:function(){if(1&i[0])throw i[1];return i[1]},trys:[],ops:[]};return o={next:s(0),throw:s(1),return:s(2)},"function"==typeof Symbol&&(o[Symbol.iterator]=function(){return this}),o;function s(o){return function(s){return function(o){if(n)throw new TypeError("Generator is already executing.");for(;a;)try{if(n=1,r&&(i=2&o[0]?r.return:o[0]?r.throw||((i=r.return)&&i.call(r),0):r.next)&&!(i=i.call(r,o[1])).done)return i;switch(r=0,i&&(o=[2&o[0],i.value]),o[0]){case 0:case 1:i=o;break;case 4:return a.label++,{value:o[1],done:!1};case 5:a.label++,r=o[1],o=[0];continue;case 7:o=a.ops.pop(),a.trys.pop();continue;default:if(!(i=a.trys,(i=i.length>0&&i[i.length-1])||6!==o[0]&&2!==o[0])){a=0;continue}if(3===o[0]&&(!i||o[1]>i[0]&&o[1]<i[3])){a.label=o[1];break}if(6===o[0]&&a.label<i[1]){a.label=i[1],i=o;break}if(i&&a.label<i[2]){a.label=i[2],a.ops.push(o);break}i[2]&&a.ops.pop(),a.trys.pop();continue}o=t.call(e,a)}catch(e){o=[6,e],r=0}finally{n=i=0}if(5&o[0])throw o[1];return{value:o[0]?o[1]:void 0,done:!0}}([o,s])}}}var d,h="function"==typeof requestAnimationFrame?requestAnimationFrame:e,f="function"==typeof cancelAnimationFrame?cancelAnimationFrame:r,p=function(){function e(e){var t=this;this._disposed=new o.Signal(this),this._tick=new i.PromiseDelegate,this._ticked=new o.Signal(this),this._timeout=-1,this._factory=e.factory,this._standby=e.standby||d.DEFAULT_STANDBY,this._state=l(l({},d.DEFAULT_STATE),{timestamp:(new Date).getTime()}),this.frequency=l(l({},d.DEFAULT_FREQUENCY),e.frequency||{}),this.name=e.name||d.DEFAULT_NAME,"auto"in e&&!e.auto||h((function(){t.start()}))}return Object.defineProperty(e.prototype,"disposed",{get:function(){return this._disposed},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"frequency",{get:function(){return this._frequency},set:function(t){if(!this.isDisposed&&!i.JSONExt.deepEqual(t,this.frequency||{})){var n=t.backoff,r=t.interval,o=t.max;if(r=Math.round(r),o=Math.round(o),"number"==typeof n&&n<1)throw new Error("Poll backoff growth factor must be at least 1");if((r<0||r>o)&&r!==e.NEVER)throw new Error("Poll interval must be between 0 and max");if(o>e.MAX_INTERVAL&&o!==e.NEVER)throw new Error("Max interval must be less than "+e.MAX_INTERVAL);this._frequency={backoff:n,interval:r,max:o}}},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"isDisposed",{get:function(){return"disposed"===this.state.phase},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"standby",{get:function(){return this._standby},set:function(e){this.isDisposed||this.standby===e||(this._standby=e)},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"state",{get:function(){return this._state},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"tick",{get:function(){return this._tick.promise},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"ticked",{get:function(){return this._ticked},enumerable:!0,configurable:!0}),e.prototype.dispose=function(){this.isDisposed||(this._state=l(l({},d.DISPOSED_STATE),{timestamp:(new Date).getTime()}),this._tick.promise.catch((function(e){})),this._tick.reject(new Error("Poll ("+this.name+") is disposed.")),this._disposed.emit(void 0),o.Signal.clearData(this))},e.prototype.refresh=function(){return this.schedule({cancel:function(e){return"refreshed"===e.phase},interval:e.IMMEDIATE,phase:"refreshed"})},e.prototype.schedule=function(t){return void 0===t&&(t={}),u(this,void 0,void 0,(function(){var n,r,o,a,s,u=this;return c(this,(function(c){switch(c.label){case 0:return this.isDisposed||t.cancel&&t.cancel(this.state)?[2]:(n=this.state,r=this._tick,o=new i.PromiseDelegate,a=l({interval:this.frequency.interval,payload:null,phase:"standby",timestamp:(new Date).getTime()},t),this._state=a,this._tick=o,n.interval===e.IMMEDIATE?f(this._timeout):clearTimeout(this._timeout),this._ticked.emit(this.state),r.resolve(this),[4,r.promise]);case 1:return c.sent(),s=function(){u.isDisposed||u.tick!==o.promise||u._execute()},this._timeout=a.interval===e.IMMEDIATE?h(s):a.interval===e.NEVER?-1:setTimeout(s,a.interval),[2]}}))}))},e.prototype.start=function(){return this.schedule({cancel:function(e){var t=e.phase;return"constructed"!==t&&"standby"!==t&&"stopped"!==t},interval:e.IMMEDIATE,phase:"started"})},e.prototype.stop=function(){return this.schedule({cancel:function(e){return"stopped"===e.phase},interval:e.NEVER,phase:"stopped"})},e.prototype._execute=function(){var e=this,t="function"==typeof this.standby?this.standby():this.standby;if(t="never"!==t&&("when-hidden"===t?!("undefined"==typeof document||!document||!document.hidden):t))this.schedule();else{var n=this.tick;this._factory(this.state).then((function(t){e.isDisposed||e.tick!==n||e.schedule({payload:t,phase:"rejected"===e.state.phase?"reconnected":"resolved"})})).catch((function(t){e.isDisposed||e.tick!==n||e.schedule({interval:d.sleep(e.frequency,e.state),payload:t,phase:"rejected"})}))}},e}();!function(e){e.IMMEDIATE=0,e.MAX_INTERVAL=2147483647,e.NEVER=1/0}(p||(p={})),function(e){e.DEFAULT_BACKOFF=3,e.DEFAULT_FREQUENCY={backoff:!0,interval:1e3,max:3e4},e.DEFAULT_NAME="unknown",e.DEFAULT_STANDBY="when-hidden",e.DEFAULT_STATE={interval:p.NEVER,payload:null,phase:"constructed",timestamp:new Date(0).getTime()},e.DISPOSED_STATE={interval:p.NEVER,payload:null,phase:"disposed",timestamp:new Date(0).getTime()},e.sleep=function(t,n){var r=t.backoff,i=t.interval,o=t.max;if(i===p.NEVER)return i;var a=!0===r?e.DEFAULT_BACKOFF:!1===r?1:r,s=function(e,t){return e=Math.ceil(e),t=Math.floor(t),Math.floor(Math.random()*(t-e+1))+e}(i,n.interval*a);return Math.min(o,s)}}(d||(d={}));var g=function(){function e(e,t){var n=this;void 0===t&&(t=500),this.payload=null,this.limit=t,this.poll=new p({auto:!1,factory:function(){return u(n,void 0,void 0,(function(){return c(this,(function(t){switch(t.label){case 0:return[4,e()];case 1:return[2,t.sent()]}}))}))},frequency:{backoff:!1,interval:p.NEVER,max:p.NEVER},standby:"never"}),this.payload=new i.PromiseDelegate,this.poll.ticked.connect((function(e,t){var r=n.payload;return"resolved"===t.phase?(n.payload=new i.PromiseDelegate,void r.resolve(t.payload)):"rejected"===t.phase||"stopped"===t.phase?(n.payload=new i.PromiseDelegate,r.promise.catch((function(e){})),void r.reject(t.payload)):void 0}),this)}return Object.defineProperty(e.prototype,"isDisposed",{get:function(){return null===this.payload},enumerable:!0,configurable:!0}),e.prototype.dispose=function(){this.isDisposed||(this.payload=null,this.poll.dispose())},e.prototype.stop=function(){return u(this,void 0,void 0,(function(){return c(this,(function(e){return[2,this.poll.stop()]}))}))},e}(),v=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return s(t,e),t.prototype.invoke=function(){return this.poll.schedule({interval:this.limit,phase:"invoked"}),this.payload.promise},t}(g),m=function(e){function t(t,n){var r=e.call(this,t,"number"==typeof n?n:n&&n.limit)||this,i="leading";return"number"!=typeof n&&(i="edge"in(n=n||{})?n.edge:i),r._interval="trailing"===i?r.limit:p.IMMEDIATE,r}return s(t,e),t.prototype.invoke=function(){return"invoked"!==this.poll.state.phase&&this.poll.schedule({interval:this._interval,phase:"invoked"}),this.payload.promise},t}(g);m||(m={})}.call(this,n(234).setImmediate,n(234).clearImmediate)},function(e,t,n){"use strict";n.r(t);var r=n(35);function i(e,t){return function(n){var i=new r.WrappedError(e,n);return t&&console.error(i),Promise.reject(i)}}function o(e,t){void 0!==t&&(e.textContent=t),void 0!==window.MathJax&&MathJax.Hub.Queue(["Typeset",MathJax.Hub,e])}function a(e){var t=document.createElement("div");return t.textContent=e,t.innerHTML}var s,l=(s=function(e,t){return(s=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(e,t)},function(e,t){function n(){this.constructor=e}s(e,t),e.prototype=null===t?Object.create(t):(n.prototype=t.prototype,new n)}),u=function(){return(u=Object.assign||function(e){for(var t,n=1,r=arguments.length;n<r;n++)for(var i in t=arguments[n])Object.prototype.hasOwnProperty.call(t,i)&&(e[i]=t[i]);return e}).apply(this,arguments)},c=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return l(t,e),t.prototype.defaults=function(){return u(u({},e.prototype.defaults.call(this)),{_model_name:"DescriptionStyleModel",_model_module:"@jupyter-widgets/controls",_model_module_version:"1.5.0"})},t.styleProperties={description_width:{selector:".widget-label",attribute:"width",default:null}},t}(r.StyleModel),d=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return l(t,e),t.prototype.defaults=function(){return u(u({},e.prototype.defaults.call(this)),{_model_name:"DescriptionModel",_view_name:"DescriptionView",_view_module:"@jupyter-widgets/controls",_model_module:"@jupyter-widgets/controls",_view_module_version:"1.5.0",_model_module_version:"1.5.0",description:"",description_tooltip:null})},t}(r.DOMWidgetModel),h=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return l(t,e),t.prototype.render=function(){this.label=document.createElement("label"),this.el.appendChild(this.label),this.label.className="widget-label",this.label.style.display="none",this.listenTo(this.model,"change:description",this.updateDescription),this.listenTo(this.model,"change:description_tooltip",this.updateDescription),this.updateDescription()},t.prototype.typeset=function(e,t){this.displayed.then((function(){return o(e,t)}))},t.prototype.updateDescription=function(){var e=this.model.get("description"),t=this.model.get("description_tooltip");null===t&&(t=e),0===e.length?this.label.style.display="none":(this.label.innerHTML=e,this.typeset(this.label),this.label.style.display=""),this.label.title=t},t}(r.DOMWidgetView),f=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return l(t,e),t}(d),p=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return l(t,e),t}(h),g=n(28),v=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),m=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return v(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_model_name:"CoreWidgetModel",_view_module:"@jupyter-widgets/controls",_model_module:"@jupyter-widgets/controls",_view_module_version:"1.5.0",_model_module_version:"1.5.0"})},t}(r.WidgetModel),y=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return v(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_model_name:"CoreDOMWidgetModel",_view_module:"@jupyter-widgets/controls",_model_module:"@jupyter-widgets/controls",_view_module_version:"1.5.0",_model_module_version:"1.5.0"})},t}(r.DOMWidgetModel),b=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return v(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_model_name:"CoreDescriptionModel",_view_module:"@jupyter-widgets/controls",_model_module:"@jupyter-widgets/controls",_view_module_version:"1.5.0",_model_module_version:"1.5.0"})},t}(d),C=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),A=function(){return(A=Object.assign||function(e){for(var t,n=1,r=arguments.length;n<r;n++)for(var i in t=arguments[n])Object.prototype.hasOwnProperty.call(t,i)&&(e[i]=t[i]);return e}).apply(this,arguments)},_=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return C(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{target:void 0,source:void 0,_model_name:"DirectionalLinkModel"})},t.prototype.initialize=function(t,n){e.prototype.initialize.call(this,t,n),this.on("change",this.updateBindings,this),this.updateBindings()},t.prototype.updateValue=function(e,t,n,r){if(!this._updating){this._updating=!0;try{n&&(n.set(r,e.get(t)),n.save_changes())}finally{this._updating=!1}}},t.prototype.updateBindings=function(){var e,t,n=this;this.cleanup(),e=this.get("source")||[null,null],this.sourceModel=e[0],this.sourceAttr=e[1],t=this.get("target")||[null,null],this.targetModel=t[0],this.targetAttr=t[1],this.sourceModel&&(this.listenTo(this.sourceModel,"change:"+this.sourceAttr,(function(){n.updateValue(n.sourceModel,n.sourceAttr,n.targetModel,n.targetAttr)})),this.updateValue(this.sourceModel,this.sourceAttr,this.targetModel,this.targetAttr),this.listenToOnce(this.sourceModel,"destroy",this.cleanup)),this.targetModel&&this.listenToOnce(this.targetModel,"destroy",this.cleanup)},t.prototype.cleanup=function(){this.sourceModel&&(this.stopListening(this.sourceModel,"change:"+this.sourceAttr,null),this.stopListening(this.sourceModel,"destroy",null)),this.targetModel&&this.stopListening(this.targetModel,"destroy",null)},t.serializers=A(A({},m.serializers),{target:{deserialize:r.unpack_models},source:{deserialize:r.unpack_models}}),t}(m),I=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return C(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_model_name:"LinkModel"})},t.prototype.updateBindings=function(){var t=this;e.prototype.updateBindings.call(this),this.targetModel&&this.listenTo(this.targetModel,"change:"+this.targetAttr,(function(){t.updateValue(t.targetModel,t.targetAttr,t.sourceModel,t.sourceAttr)}))},t.prototype.cleanup=function(){e.prototype.cleanup.call(this),this.targetModel&&this.stopListening(this.targetModel,"change:"+this.targetAttr,null)},t}(_),x=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),w=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return x(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{value:!1,disabled:!1,_model_name:"BoolModel"})},t}(b),M=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return x(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{indent:!0,_view_name:"CheckboxView",_model_name:"CheckboxModel"})},t}(b),S=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return x(t,e),t.prototype.render=function(){e.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-inline-hbox"),this.el.classList.add("widget-checkbox"),this.label.innerHTML="&#8203;",this.checkboxLabel=document.createElement("label"),this.checkboxLabel.classList.add("widget-label-basic"),this.el.appendChild(this.checkboxLabel),this.checkbox=document.createElement("input"),this.checkbox.setAttribute("type","checkbox"),this.checkboxLabel.appendChild(this.checkbox),this.descriptionSpan=document.createElement("span"),this.checkboxLabel.appendChild(this.descriptionSpan),this.listenTo(this.model,"change:indent",this.updateIndent),this.update(),this.updateDescription(),this.updateIndent()},t.prototype.updateDescription=function(){if(null!=this.checkboxLabel){var e=this.model.get("description");this.descriptionSpan.innerHTML=e,this.typeset(this.descriptionSpan),this.descriptionSpan.title=e,this.checkbox.title=e}},t.prototype.updateIndent=function(){var e=this.model.get("indent");this.label.style.display=e?"":"none"},t.prototype.events=function(){return{'click input[type="checkbox"]':"_handle_click"}},t.prototype._handle_click=function(){var e=this.model.get("value");this.model.set("value",!e,{updated_view:this}),this.touch()},t.prototype.update=function(t){return this.checkbox.checked=this.model.get("value"),void 0!==t&&t.updated_view==this||(this.checkbox.disabled=this.model.get("disabled")),e.prototype.update.call(this)},t}(h),T=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return x(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_view_name:"ToggleButtonView",_model_name:"ToggleButtonModel",tooltip:"",icon:"",button_style:""})},t}(w),E=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return x(t,e),t.prototype.render=function(){e.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("jupyter-button"),this.el.classList.add("widget-toggle-button"),this.listenTo(this.model,"change:button_style",this.update_button_style),this.set_button_style(),this.update()},t.prototype.update_button_style=function(){this.update_mapped_classes(t.class_map,"button_style")},t.prototype.set_button_style=function(){this.set_mapped_classes(t.class_map,"button_style")},t.prototype.update=function(t){if(this.model.get("value")?this.el.classList.add("mod-active"):this.el.classList.remove("mod-active"),void 0===t||t.updated_view!==this){this.el.disabled=this.model.get("disabled"),this.el.setAttribute("title",this.model.get("tooltip"));var n=this.model.get("description"),r=this.model.get("icon");if(0===n.trim().length&&0===r.trim().length)this.el.innerHTML="&nbsp;";else{if(this.el.textContent="",r.trim().length){var i=document.createElement("i");this.el.appendChild(i),i.classList.add("fa"),i.classList.add("fa-"+r)}this.el.appendChild(document.createTextNode(n))}}return e.prototype.update.call(this)},t.prototype.events=function(){return{click:"_handle_click"}},t.prototype._handle_click=function(e){e.preventDefault();var t=this.model.get("value");this.model.set("value",!t,{updated_view:this}),this.touch()},Object.defineProperty(t.prototype,"tagName",{get:function(){return"button"},enumerable:!0,configurable:!0}),t.class_map={primary:["mod-primary"],success:["mod-success"],info:["mod-info"],warning:["mod-warning"],danger:["mod-danger"]},t}(r.DOMWidgetView),O=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return x(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{readout:"Invalid",_view_name:"ValidView",_model_name:"ValidModel"})},t}(w),k=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return x(t,e),t.prototype.render=function(){e.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-valid"),this.el.classList.add("widget-inline-hbox"),this.icon=document.createElement("i"),this.icon.classList.add("fa","fa-fw"),this.el.appendChild(this.icon),this.readout=document.createElement("span"),this.readout.classList.add("widget-valid-readout"),this.readout.classList.add("widget-readout"),this.el.appendChild(this.readout),this.update()},t.prototype.update=function(){this.el.classList.remove("mod-valid"),this.el.classList.remove("mod-invalid"),this.icon.classList.remove("fa-check"),this.icon.classList.remove("fa-times"),this.readout.textContent=this.model.get("readout"),this.model.get("value")?(this.el.classList.add("mod-valid"),this.icon.classList.add("fa-check")):(this.el.classList.add("mod-invalid"),this.icon.classList.add("fa-times"))},t}(h),L=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),D=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return L(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_model_name:"ButtonStyleModel",_model_module:"@jupyter-widgets/controls",_model_module_version:"1.5.0"})},t.styleProperties={button_color:{selector:"",attribute:"background-color",default:null},font_weight:{selector:"",attribute:"font-weight",default:""}},t}(r.StyleModel),P=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return L(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{description:"",tooltip:"",disabled:!1,icon:"",button_style:"",_view_name:"ButtonView",_model_name:"ButtonModel",style:null})},t}(y),N=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return L(t,e),t.prototype.render=function(){e.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("jupyter-button"),this.el.classList.add("widget-button"),this.listenTo(this.model,"change:button_style",this.update_button_style),this.set_button_style(),this.update()},t.prototype.update=function(){this.el.disabled=this.model.get("disabled"),this.el.setAttribute("title",this.model.get("tooltip"));var t=this.model.get("description"),n=this.model.get("icon");if(t.length||n.length){if(this.el.textContent="",n.length){var r=document.createElement("i");r.classList.add("fa"),r.classList.add("fa-"+n),0===t.length&&r.classList.add("center"),this.el.appendChild(r)}this.el.appendChild(document.createTextNode(t))}return e.prototype.update.call(this)},t.prototype.update_button_style=function(){this.update_mapped_classes(t.class_map,"button_style")},t.prototype.set_button_style=function(){this.set_mapped_classes(t.class_map,"button_style")},t.prototype.events=function(){return{click:"_handle_click"}},t.prototype._handle_click=function(e){e.preventDefault(),this.send({event:"click"})},Object.defineProperty(t.prototype,"tagName",{get:function(){return"button"},enumerable:!0,configurable:!0}),t.class_map={primary:["mod-primary"],success:["mod-success"],info:["mod-info"],warning:["mod-warning"],danger:["mod-danger"]},t}(r.DOMWidgetView),j=n(6),R=n(14),z=n(18),F=n(71),B=n.n(F),H=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),U=function(){return(U=Object.assign||function(e){for(var t,n=1,r=arguments.length;n<r;n++)for(var i in t=arguments[n])Object.prototype.hasOwnProperty.call(t,i)&&(e[i]=t[i]);return e}).apply(this,arguments)},V=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return H(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_view_name:"BoxView",_model_name:"BoxModel",children:[],box_style:""})},t.serializers=U(U({},y.serializers),{children:{deserialize:r.unpack_models}}),t}(y),Y=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return H(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_view_name:"HBoxView",_model_name:"HBoxModel"})},t}(V),W=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return H(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_view_name:"VBoxView",_model_name:"VBoxModel"})},t}(V),G=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return H(t,e),t.prototype._createElement=function(e){return this.pWidget=new r.JupyterPhosphorPanelWidget({view:this}),this.pWidget.node},t.prototype._setElement=function(e){if(this.el||e!==this.pWidget.node)throw new Error("Cannot reset the DOM element.");this.el=this.pWidget.node,this.$el=B()(this.pWidget.node)},t.prototype.initialize=function(t){e.prototype.initialize.call(this,t),this.children_views=new r.ViewList(this.add_child_model,null,this),this.listenTo(this.model,"change:children",this.update_children),this.listenTo(this.model,"change:box_style",this.update_box_style),this.pWidget.addClass("jupyter-widgets"),this.pWidget.addClass("widget-container"),this.pWidget.addClass("widget-box")},t.prototype.render=function(){e.prototype.render.call(this),this.update_children(),this.set_box_style()},t.prototype.update_children=function(){this.children_views.update(this.model.get("children")).then((function(e){e.forEach((function(e){R.c.postMessage(e.pWidget,z.Widget.ResizeMessage.UnknownSize)}))}))},t.prototype.update_box_style=function(){this.update_mapped_classes(t.class_map,"box_style")},t.prototype.set_box_style=function(){this.set_mapped_classes(t.class_map,"box_style")},t.prototype.add_child_model=function(e){var t=this,n=new z.Widget;return this.pWidget.addWidget(n),this.create_child_view(e).then((function(e){var r=j.ArrayExt.firstIndexOf(t.pWidget.widgets,n);return t.pWidget.insertWidget(r,e.pWidget),n.dispose(),e})).catch(i("Could not add child view to box",!0))},t.prototype.remove=function(){this.children_views=null,e.prototype.remove.call(this)},t.class_map={success:["alert","alert-success"],info:["alert","alert-info"],warning:["alert","alert-warning"],danger:["alert","alert-danger"]},t}(r.DOMWidgetView),X=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return H(t,e),t.prototype.initialize=function(t){e.prototype.initialize.call(this,t),this.pWidget.addClass("widget-hbox")},t}(G),q=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return H(t,e),t.prototype.initialize=function(t){e.prototype.initialize.call(this,t),this.pWidget.addClass("widget-vbox")},t}(G),Z=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return H(t,e),t.prototype.initialize=function(t){e.prototype.initialize.call(this,t),this.pWidget.addClass("widget-gridbox"),this.pWidget.removeClass("widget-box")},t}(G),Q=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return H(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_view_name:"GridBoxView",_model_name:"GridBoxModel"})},t}(V),K=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),J=function(){return(J=Object.assign||function(e){for(var t,n=1,r=arguments.length;n<r;n++)for(var i in t=arguments[n])Object.prototype.hasOwnProperty.call(t,i)&&(e[i]=t[i]);return e}).apply(this,arguments)},$=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return K(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_model_name:"ImageModel",_view_name:"ImageView",format:"png",width:"",height:"",value:new DataView(new ArrayBuffer(0))})},t.serializers=J(J({},y.serializers),{value:{serialize:function(e){return new DataView(e.buffer.slice(0))}}}),t}(y),ee=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return K(t,e),t.prototype.render=function(){e.prototype.render.call(this),this.pWidget.addClass("jupyter-widgets"),this.pWidget.addClass("widget-image"),this.update()},t.prototype.update=function(){var t,n=this.model.get("format"),r=this.model.get("value");if("url"!==n){var i=new Blob([r],{type:"image/"+this.model.get("format")});t=URL.createObjectURL(i)}else t=new TextDecoder("utf-8").decode(r.buffer);var o=this.el.src;this.el.src=t,o&&"string"!=typeof o&&URL.revokeObjectURL(o);var a=this.model.get("width");void 0!==a&&a.length>0?this.el.setAttribute("width",a):this.el.removeAttribute("width");var s=this.model.get("height");return void 0!==s&&s.length>0?this.el.setAttribute("height",s):this.el.removeAttribute("height"),e.prototype.update.call(this)},t.prototype.remove=function(){this.el.src&&URL.revokeObjectURL(this.el.src),e.prototype.remove.call(this)},Object.defineProperty(t.prototype,"tagName",{get:function(){return"img"},enumerable:!0,configurable:!0}),t}(r.DOMWidgetView),te=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),ne=function(){return(ne=Object.assign||function(e){for(var t,n=1,r=arguments.length;n<r;n++)for(var i in t=arguments[n])Object.prototype.hasOwnProperty.call(t,i)&&(e[i]=t[i]);return e}).apply(this,arguments)},re=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return te(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_model_name:"VideoModel",_view_name:"VideoView",format:"mp4",width:"",height:"",autoplay:!0,loop:!0,controls:!0,value:new DataView(new ArrayBuffer(0))})},t.serializers=ne(ne({},y.serializers),{value:{serialize:function(e){return new DataView(e.buffer.slice(0))}}}),t}(y),ie=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return te(t,e),t.prototype.render=function(){e.prototype.render.call(this),this.pWidget.addClass("jupyter-widgets"),this.pWidget.addClass("widget-image"),this.update()},t.prototype.update=function(){var t,n=this.model.get("format"),r=this.model.get("value");if("url"!==n){var i=new Blob([r],{type:"video/"+this.model.get("format")});t=URL.createObjectURL(i)}else t=new TextDecoder("utf-8").decode(r.buffer);var o=this.el.src;this.el.src=t,o&&"string"!=typeof o&&URL.revokeObjectURL(o);var a=this.model.get("width");void 0!==a&&a.length>0?this.el.setAttribute("width",a):this.el.removeAttribute("width");var s=this.model.get("height");return void 0!==s&&s.length>0?this.el.setAttribute("height",s):this.el.removeAttribute("height"),this.el.loop=this.model.get("loop"),this.el.autoplay=this.model.get("autoplay"),this.el.controls=this.model.get("controls"),e.prototype.update.call(this)},t.prototype.remove=function(){this.el.src&&URL.revokeObjectURL(this.el.src),e.prototype.remove.call(this)},Object.defineProperty(t.prototype,"tagName",{get:function(){return"video"},enumerable:!0,configurable:!0}),t}(r.DOMWidgetView),oe=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),ae=function(){return(ae=Object.assign||function(e){for(var t,n=1,r=arguments.length;n<r;n++)for(var i in t=arguments[n])Object.prototype.hasOwnProperty.call(t,i)&&(e[i]=t[i]);return e}).apply(this,arguments)},se=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return oe(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_model_name:"AudioModel",_view_name:"AudioView",format:"mp3",autoplay:!0,loop:!0,controls:!0,value:new DataView(new ArrayBuffer(0))})},t.serializers=ae(ae({},y.serializers),{value:{serialize:function(e){return new DataView(e.buffer.slice(0))}}}),t}(y),le=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return oe(t,e),t.prototype.render=function(){e.prototype.render.call(this),this.pWidget.addClass("jupyter-widgets"),this.update()},t.prototype.update=function(){var t,n=this.model.get("format"),r=this.model.get("value");if("url"!==n){var i=new Blob([r],{type:"audio/"+this.model.get("format")});t=URL.createObjectURL(i)}else t=new TextDecoder("utf-8").decode(r.buffer);var o=this.el.src;return this.el.src=t,o&&"string"!=typeof o&&URL.revokeObjectURL(o),this.el.loop=this.model.get("loop"),this.el.autoplay=this.model.get("autoplay"),this.el.controls=this.model.get("controls"),e.prototype.update.call(this)},t.prototype.remove=function(){this.el.src&&URL.revokeObjectURL(this.el.src),e.prototype.remove.call(this)},Object.defineProperty(t.prototype,"tagName",{get:function(){return"audio"},enumerable:!0,configurable:!0}),t}(r.DOMWidgetView),ue=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),ce=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return ue(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{value:"black",concise:!1,_model_name:"ColorPickerModel",_view_name:"ColorPickerView"})},t}(b),de=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return ue(t,e),t.prototype.render=function(){e.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-inline-hbox"),this.el.classList.add("widget-colorpicker"),this._color_container=document.createElement("div"),this._color_container.className="widget-inline-hbox widget-colorpicker-input",this.el.appendChild(this._color_container),this._textbox=document.createElement("input"),this._textbox.setAttribute("type","text"),this._textbox.id=this.label.htmlFor=Object(r.uuid)(),this._color_container.appendChild(this._textbox),this._textbox.value=this.model.get("value"),this._colorpicker=document.createElement("input"),this._colorpicker.setAttribute("type","color"),this._color_container.appendChild(this._colorpicker),this.listenTo(this.model,"change:value",this._update_value),this.listenTo(this.model,"change:concise",this._update_concise),this._update_concise(),this._update_value(),this.update()},t.prototype.update=function(t){if(void 0===t||t.updated_view!=this){var n=this.model.get("disabled");this._textbox.disabled=n,this._colorpicker.disabled=n}return e.prototype.update.call(this)},t.prototype.events=function(){return this._picker_change,this._text_change,{'change [type="color"]':"_picker_change",'change [type="text"]':"_text_change"}},t.prototype._update_value=function(){var e,t,n=this.model.get("value");this._colorpicker.value=he[(e=n).toLowerCase()]||(7===(t=e).length?t:"#"+t.charAt(1)+t.charAt(1)+t.charAt(2)+t.charAt(2)+t.charAt(3)+t.charAt(3)),this._textbox.value=n},t.prototype._update_concise=function(){this.model.get("concise")?(this.el.classList.add("concise"),this._textbox.style.display="none"):(this.el.classList.remove("concise"),this._textbox.style.display="")},t.prototype._picker_change=function(){this.model.set("value",this._colorpicker.value),this.touch()},t.prototype._text_change=function(){var e=this._validate_color(this._textbox.value,this.model.get("value"));this.model.set("value",e),this.touch()},t.prototype._validate_color=function(e,t){return e.match(/#[a-fA-F0-9]{3}(?:[a-fA-F0-9]{3})?$/)||he[e.toLowerCase()]?e:t},t}(h),he={aliceblue:"#f0f8ff",antiquewhite:"#faebd7",aqua:"#00ffff",aquamarine:"#7fffd4",azure:"#f0ffff",beige:"#f5f5dc",bisque:"#ffe4c4",black:"#000000",blanchedalmond:"#ffebcd",blue:"#0000ff",blueviolet:"#8a2be2",brown:"#a52a2a",burlywood:"#deb887",cadetblue:"#5f9ea0",chartreuse:"#7fff00",chocolate:"#d2691e",coral:"#ff7f50",cornflowerblue:"#6495ed",cornsilk:"#fff8dc",crimson:"#dc143c",cyan:"#00ffff",darkblue:"#00008b",darkcyan:"#008b8b",darkgoldenrod:"#b8860b",darkgray:"#a9a9a9",darkgrey:"#a9a9a9",darkgreen:"#006400",darkkhaki:"#bdb76b",darkmagenta:"#8b008b",darkolivegreen:"#556b2f",darkorange:"#ff8c00",darkorchid:"#9932cc",darkred:"#8b0000",darksalmon:"#e9967a",darkseagreen:"#8fbc8f",darkslateblue:"#483d8b",darkslategray:"#2f4f4f",darkslategrey:"#2f4f4f",darkturquoise:"#00ced1",darkviolet:"#9400d3",deeppink:"#ff1493",deepskyblue:"#00bfff",dimgray:"#696969",dimgrey:"#696969",dodgerblue:"#1e90ff",firebrick:"#b22222",floralwhite:"#fffaf0",forestgreen:"#228b22",fuchsia:"#ff00ff",gainsboro:"#dcdcdc",ghostwhite:"#f8f8ff",gold:"#ffd700",goldenrod:"#daa520",gray:"#808080",grey:"#808080",green:"#008000",greenyellow:"#adff2f",honeydew:"#f0fff0",hotpink:"#ff69b4",indianred:"#cd5c5c",indigo:"#4b0082",ivory:"#fffff0",khaki:"#f0e68c",lavender:"#e6e6fa",lavenderblush:"#fff0f5",lawngreen:"#7cfc00",lemonchiffon:"#fffacd",lightblue:"#add8e6",lightcoral:"#f08080",lightcyan:"#e0ffff",lightgoldenrodyellow:"#fafad2",lightgreen:"#90ee90",lightgray:"#d3d3d3",lightgrey:"#d3d3d3",lightpink:"#ffb6c1",lightsalmon:"#ffa07a",lightseagreen:"#20b2aa",lightskyblue:"#87cefa",lightslategray:"#778899",lightslategrey:"#778899",lightsteelblue:"#b0c4de",lightyellow:"#ffffe0",lime:"#00ff00",limegreen:"#32cd32",linen:"#faf0e6",magenta:"#ff00ff",maroon:"#800000",mediumaquamarine:"#66cdaa",mediumblue:"#0000cd",mediumorchid:"#ba55d3",mediumpurple:"#9370db",mediumseagreen:"#3cb371",mediumslateblue:"#7b68ee",mediumspringgreen:"#00fa9a",mediumturquoise:"#48d1cc",mediumvioletred:"#c71585",midnightblue:"#191970",mintcream:"#f5fffa",mistyrose:"#ffe4e1",moccasin:"#ffe4b5",navajowhite:"#ffdead",navy:"#000080",oldlace:"#fdf5e6",olive:"#808000",olivedrab:"#6b8e23",orange:"#ffa500",orangered:"#ff4500",orchid:"#da70d6",palegoldenrod:"#eee8aa",palegreen:"#98fb98",paleturquoise:"#afeeee",palevioletred:"#db7093",papayawhip:"#ffefd5",peachpuff:"#ffdab9",peru:"#cd853f",pink:"#ffc0cb",plum:"#dda0dd",powderblue:"#b0e0e6",purple:"#800080",red:"#ff0000",rosybrown:"#bc8f8f",royalblue:"#4169e1",saddlebrown:"#8b4513",salmon:"#fa8072",sandybrown:"#f4a460",seagreen:"#2e8b57",seashell:"#fff5ee",sienna:"#a0522d",silver:"#c0c0c0",skyblue:"#87ceeb",slateblue:"#6a5acd",slategray:"#708090",slategrey:"#708090",snow:"#fffafa",springgreen:"#00ff7f",steelblue:"#4682b4",tan:"#d2b48c",teal:"#008080",thistle:"#d8bfd8",tomato:"#ff6347",turquoise:"#40e0d0",violet:"#ee82ee",wheat:"#f5deb3",white:"#ffffff",whitesmoke:"#f5f5f5",yellow:"#ffff00",yellowgreen:"#9acd32"};var fe=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),pe=function(){return(pe=Object.assign||function(e){for(var t,n=1,r=arguments.length;n<r;n++)for(var i in t=arguments[n])Object.prototype.hasOwnProperty.call(t,i)&&(e[i]=t[i]);return e}).apply(this,arguments)};function ge(e){return null===e?null:{year:e.getUTCFullYear(),month:e.getUTCMonth(),date:e.getUTCDate()}}function ve(e){if(null===e)return null;var t=new Date;return t.setUTCFullYear(e.year,e.month,e.date),t.setUTCHours(0,0,0,0),t}var me=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return fe(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{value:null,_model_name:"DatePickerModel",_view_name:"DatePickerView"})},t.serializers=pe(pe({},b.serializers),{value:{serialize:ge,deserialize:ve}}),t}(b),ye=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return fe(t,e),t.prototype.render=function(){e.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-inline-hbox"),this.el.classList.add("widget-datepicker"),this._datepicker=document.createElement("input"),this._datepicker.setAttribute("type","date"),this._datepicker.id=this.label.htmlFor=Object(r.uuid)(),this.el.appendChild(this._datepicker),this.listenTo(this.model,"change:value",this._update_value),this._update_value(),this.update()},t.prototype.update=function(t){return void 0!==t&&t.updated_view===this||(this._datepicker.disabled=this.model.get("disabled")),e.prototype.update.call(this)},t.prototype.events=function(){return this._picker_change,this._picker_focusout,{'change [type="date"]':"_picker_change",'focusout [type="date"]':"_picker_focusout"}},t.prototype._update_value=function(){var e=this.model.get("value");this._datepicker.valueAsDate=e},t.prototype._picker_change=function(){this._datepicker.validity.badInput||(this.model.set("value",this._datepicker.valueAsDate),this.touch())},t.prototype._picker_focusout=function(){this._datepicker.validity.badInput&&(this.model.set("value",null),this.touch())},t}(h);function be(e,t){if((n=(e=t?e.toExponential(t-1):e.toExponential()).indexOf("e"))<0)return null;var n,r=e.slice(0,n);return[r.length>1?r[0]+r.slice(2):r,+e.slice(n+1)]}var Ce=/^(?:(.)?([<>=^]))?([+\-( ])?([$#])?(0)?(\d+)?(,)?(\.\d+)?(~)?([a-z%])?$/i;function Ae(e){if(!(t=Ce.exec(e)))throw new Error("invalid format: "+e);var t;return new _e({fill:t[1],align:t[2],sign:t[3],symbol:t[4],zero:t[5],width:t[6],comma:t[7],precision:t[8]&&t[8].slice(1),trim:t[9],type:t[10]})}function _e(e){this.fill=void 0===e.fill?" ":e.fill+"",this.align=void 0===e.align?">":e.align+"",this.sign=void 0===e.sign?"-":e.sign+"",this.symbol=void 0===e.symbol?"":e.symbol+"",this.zero=!!e.zero,this.width=void 0===e.width?void 0:+e.width,this.comma=!!e.comma,this.precision=void 0===e.precision?void 0:+e.precision,this.trim=!!e.trim,this.type=void 0===e.type?"":e.type+""}Ae.prototype=_e.prototype,_e.prototype.toString=function(){return this.fill+this.align+this.sign+this.symbol+(this.zero?"0":"")+(void 0===this.width?"":Math.max(1,0|this.width))+(this.comma?",":"")+(void 0===this.precision?"":"."+Math.max(0,0|this.precision))+(this.trim?"~":"")+this.type};var Ie,xe,we,Me=function(e,t){var n=be(e,t);if(!n)return e+"";var r=n[0],i=n[1];return i<0?"0."+new Array(-i).join("0")+r:r.length>i+1?r.slice(0,i+1)+"."+r.slice(i+1):r+new Array(i-r.length+2).join("0")},Se={"%":function(e,t){return(100*e).toFixed(t)},b:function(e){return Math.round(e).toString(2)},c:function(e){return e+""},d:function(e){return Math.abs(e=Math.round(e))>=1e21?e.toLocaleString("en").replace(/,/g,""):e.toString(10)},e:function(e,t){return e.toExponential(t)},f:function(e,t){return e.toFixed(t)},g:function(e,t){return e.toPrecision(t)},o:function(e){return Math.round(e).toString(8)},p:function(e,t){return Me(100*e,t)},r:Me,s:function(e,t){var n=be(e,t);if(!n)return e+"";var r=n[0],i=n[1],o=i-(Ie=3*Math.max(-8,Math.min(8,Math.floor(i/3))))+1,a=r.length;return o===a?r:o>a?r+new Array(o-a+1).join("0"):o>0?r.slice(0,o)+"."+r.slice(o):"0."+new Array(1-o).join("0")+be(e,Math.max(0,t+o-1))[0]},X:function(e){return Math.round(e).toString(16).toUpperCase()},x:function(e){return Math.round(e).toString(16)}},Te=function(e){return e},Ee=Array.prototype.map,Oe=["y","z","a","f","p","n","µ","m","","k","M","G","T","P","E","Z","Y"];xe=function(e){var t,n,r=void 0===e.grouping||void 0===e.thousands?Te:(t=Ee.call(e.grouping,Number),n=e.thousands+"",function(e,r){for(var i=e.length,o=[],a=0,s=t[0],l=0;i>0&&s>0&&(l+s+1>r&&(s=Math.max(1,r-l)),o.push(e.substring(i-=s,i+s)),!((l+=s+1)>r));)s=t[a=(a+1)%t.length];return o.reverse().join(n)}),i=void 0===e.currency?"":e.currency[0]+"",o=void 0===e.currency?"":e.currency[1]+"",a=void 0===e.decimal?".":e.decimal+"",s=void 0===e.numerals?Te:function(e){return function(t){return t.replace(/[0-9]/g,(function(t){return e[+t]}))}}(Ee.call(e.numerals,String)),l=void 0===e.percent?"%":e.percent+"",u=void 0===e.minus?"-":e.minus+"",c=void 0===e.nan?"NaN":e.nan+"";function d(e){var t=(e=Ae(e)).fill,n=e.align,d=e.sign,h=e.symbol,f=e.zero,p=e.width,g=e.comma,v=e.precision,m=e.trim,y=e.type;"n"===y?(g=!0,y="g"):Se[y]||(void 0===v&&(v=12),m=!0,y="g"),(f||"0"===t&&"="===n)&&(f=!0,t="0",n="=");var b="$"===h?i:"#"===h&&/[boxX]/.test(y)?"0"+y.toLowerCase():"",C="$"===h?o:/[%p]/.test(y)?l:"",A=Se[y],_=/[defgprs%]/.test(y);function I(e){var i,o,l,h=b,I=C;if("c"===y)I=A(e)+I,e="";else{var x=(e=+e)<0||1/e<0;if(e=isNaN(e)?c:A(Math.abs(e),v),m&&(e=function(e){e:for(var t,n=e.length,r=1,i=-1;r<n;++r)switch(e[r]){case".":i=t=r;break;case"0":0===i&&(i=r),t=r;break;default:if(!+e[r])break e;i>0&&(i=0)}return i>0?e.slice(0,i)+e.slice(t+1):e}(e)),x&&0==+e&&"+"!==d&&(x=!1),h=(x?"("===d?d:u:"-"===d||"("===d?"":d)+h,I=("s"===y?Oe[8+Ie/3]:"")+I+(x&&"("===d?")":""),_)for(i=-1,o=e.length;++i<o;)if(48>(l=e.charCodeAt(i))||l>57){I=(46===l?a+e.slice(i+1):e.slice(i))+I,e=e.slice(0,i);break}}g&&!f&&(e=r(e,1/0));var w=h.length+e.length+I.length,M=w<p?new Array(p-w+1).join(t):"";switch(g&&f&&(e=r(M+e,M.length?p-I.length:1/0),M=""),n){case"<":e=h+e+I+M;break;case"=":e=h+M+e+I;break;case"^":e=M.slice(0,w=M.length>>1)+h+e+I+M.slice(w);break;default:e=M+h+e+I}return s(e)}return v=void 0===v?6:/[gprs]/.test(y)?Math.max(1,Math.min(21,v)):Math.max(0,Math.min(20,v)),I.toString=function(){return e+""},I}return{format:d,formatPrefix:function(e,t){var n,r=d(((e=Ae(e)).type="f",e)),i=3*Math.max(-8,Math.min(8,Math.floor((n=t,((n=be(Math.abs(n)))?n[1]:NaN)/3)))),o=Math.pow(10,-i),a=Oe[8+i/3];return function(e){return r(o*e)+a}}}}({decimal:".",thousands:",",grouping:[3],currency:["$",""],minus:"-"}),we=xe.format,xe.formatPrefix;n(1425);var ke=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),Le=function(){return(Le=Object.assign||function(e){for(var t,n=1,r=arguments.length;n<r;n++)for(var i in t=arguments[n])Object.prototype.hasOwnProperty.call(t,i)&&(e[i]=t[i]);return e}).apply(this,arguments)},De=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return ke(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_model_name:"IntModel",value:0})},t}(b),Pe=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return ke(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_model_name:"BoundedIntModel",max:100,min:0})},t}(De),Ne=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return ke(t,e),t.prototype.defaults=function(){return Le(Le({},e.prototype.defaults.call(this)),{_model_name:"SliderStyleModel"})},t.styleProperties=Le(Le({},c.styleProperties),{handle_color:{selector:".ui-slider-handle",attribute:"background-color",default:null}}),t}(c),je=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return ke(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_model_name:"IntSliderModel",_view_name:"IntSliderView",step:1,orientation:"horizontal",readout:!0,readout_format:"d",continuous_update:!0,style:null,disabled:!1})},t.prototype.initialize=function(t,n){e.prototype.initialize.call(this,t,n),this.on("change:readout_format",this.update_readout_format,this),this.update_readout_format()},t.prototype.update_readout_format=function(){this.readout_formatter=we(this.get("readout_format"))},t}(Pe),Re=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return ke(t,e),t}(je),ze=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t._parse_value=parseInt,t}return ke(t,e),t.prototype.render=function(){e.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-inline-hbox"),this.el.classList.add("widget-slider"),this.el.classList.add("widget-hslider"),(this.$slider=B()("<div />")).slider({slide:this.handleSliderChange.bind(this),stop:this.handleSliderChanged.bind(this)}).addClass("slider"),this.slider_container=document.createElement("div"),this.slider_container.classList.add("slider-container"),this.slider_container.appendChild(this.$slider[0]),this.el.appendChild(this.slider_container),this.readout=document.createElement("div"),this.el.appendChild(this.readout),this.readout.classList.add("widget-readout"),this.readout.contentEditable="true",this.readout.style.display="none",this.update()},t.prototype.update=function(t){if(void 0===t||t.updated_view!==this){var n=this;n.$slider.slider({}),["step","disabled"].forEach((function(e){var t=n.model.get(e);void 0!==t&&n.$slider.slider("option",e,t)})),this.model.get("disabled")?this.readout.contentEditable="false":this.readout.contentEditable="true";var r=this.model.get("max"),i=this.model.get("min");i<=r&&(void 0!==r&&this.$slider.slider("option","max",r),void 0!==i&&this.$slider.slider("option","min",i));var o=this.model.get("orientation");this.$slider.slider("option","orientation",o),"vertical"===o?(this.el.classList.remove("widget-hslider"),this.el.classList.add("widget-vslider"),this.el.classList.remove("widget-inline-hbox"),this.el.classList.add("widget-inline-vbox")):(this.el.classList.remove("widget-vslider"),this.el.classList.add("widget-hslider"),this.el.classList.remove("widget-inline-vbox"),this.el.classList.add("widget-inline-hbox")),this.model.get("readout")?(this.readout.style.display="",this.displayed.then((function(){n.readout_overflow()?n.readout.classList.add("overflow"):n.readout.classList.remove("overflow")}))):this.readout.style.display="none"}return e.prototype.update.call(this)},t.prototype.readout_overflow=function(){return this.readout.scrollWidth>this.readout.clientWidth},t.prototype.events=function(){return{slide:"handleSliderChange",slidestop:"handleSliderChanged","blur [contentEditable=true]":"handleTextChange","keydown [contentEditable=true]":"handleKeyDown"}},t.prototype.handleKeyDown=function(e){13===e.keyCode&&(e.preventDefault(),this.handleTextChange())},t.prototype._validate_slide_value=function(e){return Math.floor(e)},t}(h),Fe=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t._range_regex=/^\s*([+-]?\d+)\s*[-:–]\s*([+-]?\d+)/,t}return ke(t,e),t.prototype.update=function(t){e.prototype.update.call(this,t),this.$slider.slider("option","range",!0);var n=this.model.get("value");this.$slider.slider("option","values",n.slice()),this.readout.textContent=this.valueToString(n),this.model.get("value")!==n&&(this.model.set("value",n,{updated_view:this}),this.touch())},t.prototype.valueToString=function(e){var t=this.model.readout_formatter;return e.map((function(e){return t(e)})).join(" – ")},t.prototype.stringToValue=function(e){var t=this._range_regex.exec(e);return t?[this._parse_value(t[1]),this._parse_value(t[2])]:null},t.prototype.handleTextChange=function(){var e=this.stringToValue(this.readout.textContent),t=this.model.get("min"),n=this.model.get("max");null===e||isNaN(e[0])||isNaN(e[1])||e[0]>e[1]?this.readout.textContent=this.valueToString(this.model.get("value")):(e=[Math.max(Math.min(e[0],n),t),Math.max(Math.min(e[1],n),t)])[0]!==this.model.get("value")[0]||e[1]!==this.model.get("value")[1]?(this.readout.textContent=this.valueToString(e),this.model.set("value",e,{updated_view:this}),this.touch()):this.readout.textContent=this.valueToString(this.model.get("value"))},t.prototype.handleSliderChange=function(e,t){var n=t.values.map(this._validate_slide_value);this.readout.textContent=this.valueToString(n),this.model.get("continuous_update")&&this.handleSliderChanged(e,t)},t.prototype.handleSliderChanged=function(e,t){var n=t.values.map(this._validate_slide_value);this.model.set("value",n,{updated_view:this}),this.touch()},t}(ze),Be=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return ke(t,e),t.prototype.update=function(t){e.prototype.update.call(this,t);var n=this.model.get("min"),r=this.model.get("max"),i=this.model.get("value");i>r?i=r:i<n&&(i=n),this.$slider.slider("option","value",i),this.readout.textContent=this.valueToString(i),this.model.get("value")!==i&&(this.model.set("value",i,{updated_view:this}),this.touch())},t.prototype.valueToString=function(e){return(0,this.model.readout_formatter)(e)},t.prototype.stringToValue=function(e){return this._parse_value(e)},t.prototype.handleTextChange=function(){var e=this.stringToValue(this.readout.textContent),t=this.model.get("min"),n=this.model.get("max");isNaN(e)?this.readout.textContent=this.valueToString(this.model.get("value")):(e=Math.max(Math.min(e,n),t))!==this.model.get("value")?(this.readout.textContent=this.valueToString(e),this.model.set("value",e,{updated_view:this}),this.touch()):this.readout.textContent=this.valueToString(this.model.get("value"))},t.prototype.handleSliderChange=function(e,t){var n=this._validate_slide_value(t.value);this.readout.textContent=this.valueToString(n),this.model.get("continuous_update")&&this.handleSliderChanged(e,t)},t.prototype.handleSliderChanged=function(e,t){var n=this._validate_slide_value(t.value);this.model.set("value",n,{updated_view:this}),this.touch()},t}(ze),He=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return ke(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_model_name:"IntTextModel",_view_name:"IntTextView",disabled:!1,continuous_update:!1})},t}(De),Ue=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return ke(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_model_name:"BoundedIntTextModel",_view_name:"IntTextView",disabled:!1,continuous_update:!1,step:1})},t}(Pe),Ve=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t._parse_value=parseInt,t._default_step="1",t}return ke(t,e),t.prototype.render=function(){e.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-inline-hbox"),this.el.classList.add("widget-text"),this.textbox=document.createElement("input"),this.textbox.type="number",this.textbox.required=!0,this.textbox.id=this.label.htmlFor=Object(r.uuid)(),this.el.appendChild(this.textbox),this.update()},t.prototype.update=function(t){if(void 0===t||t.updated_view!==this){var n=this.model.get("value");this._parse_value(this.textbox.value)!==n&&(this.textbox.value=n.toString()),void 0!==this.model.get("min")&&(this.textbox.min=this.model.get("min")),void 0!==this.model.get("max")&&(this.textbox.max=this.model.get("max")),void 0!==this.model.get("step")&&null!==this.model.get("step")?this.textbox.step=this.model.get("step"):this.textbox.step=this._default_step,this.textbox.disabled=this.model.get("disabled")}return e.prototype.update.call(this)},t.prototype.events=function(){return{"keydown input":"handleKeyDown","keypress input":"handleKeypress","keyup input":"handleKeyUp","input input":"handleChanging","change input":"handleChanged"}},t.prototype.handleKeyDown=function(e){e.stopPropagation()},t.prototype.handleKeypress=function(e){/[e,.\s]/.test(String.fromCharCode(e.keyCode))&&e.preventDefault()},t.prototype.handleKeyUp=function(e){if(!e.altKey&&!e.ctrlKey){var t=e.target,n=t.value;if((n=n.replace(/[e,.\s]/g,"")).length>=1){var r=n.substr(1);n=n[0]+r.replace(/[+-]/g,"")}t.value!=n&&(e.preventDefault(),t.value=n)}},t.prototype.handleChanging=function(e){var t=e.target.value.trim();""===t||["-","-.",".","+.","+"].indexOf(t)>=0||this.model.get("continuous_update")&&this.handleChanged(e)},t.prototype.handleChanged=function(e){var t=e.target,n=this._parse_value(t.value);if(isNaN(n))t.value=this.model.get("value");else{var r=n;void 0!==this.model.get("max")&&(r=Math.min(this.model.get("max"),r)),void 0!==this.model.get("min")&&(r=Math.max(this.model.get("min"),r)),r!==n&&(t.value=r,n=r),n!==this.model.get("value")&&(this.model.set("value",n,{updated_view:this}),this.touch())}},t}(h),Ye=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return ke(t,e),t.prototype.defaults=function(){return Le(Le({},e.prototype.defaults.call(this)),{_model_name:"ProgressStyleModel"})},t.styleProperties=Le(Le({},c.styleProperties),{bar_color:{selector:".progress-bar",attribute:"background-color",default:null}}),t}(c),We=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return ke(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_model_name:"IntProgressModel",_view_name:"ProgressView",orientation:"horizontal",bar_style:"",style:null})},t}(Pe),Ge=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return ke(t,e),t.prototype.initialize=function(t){e.prototype.initialize.call(this,t),this.listenTo(this.model,"change:bar_style",this.update_bar_style),this.pWidget.addClass("jupyter-widgets")},t.prototype.render=function(){e.prototype.render.call(this);var t="horizontal"===this.model.get("orientation")?"widget-hprogress":"widget-vprogress";this.el.classList.add(t),this.progress=document.createElement("div"),this.progress.classList.add("progress"),this.progress.style.position="relative",this.el.appendChild(this.progress),this.bar=document.createElement("div"),this.bar.classList.add("progress-bar"),this.bar.style.position="absolute",this.bar.style.bottom="0px",this.bar.style.left="0px",this.progress.appendChild(this.bar),this.update(),this.set_bar_style()},t.prototype.update=function(){var t=this.model.get("value"),n=this.model.get("max"),r=this.model.get("min"),i=100*(t-r)/(n-r);return"horizontal"===this.model.get("orientation")?(this.el.classList.remove("widget-inline-vbox"),this.el.classList.remove("widget-vprogress"),this.el.classList.add("widget-inline-hbox"),this.el.classList.add("widget-hprogress"),this.bar.style.width=i+"%",this.bar.style.height="100%"):(this.el.classList.remove("widget-inline-hbox"),this.el.classList.remove("widget-hprogress"),this.el.classList.add("widget-inline-vbox"),this.el.classList.add("widget-vprogress"),this.bar.style.width="100%",this.bar.style.height=i+"%"),e.prototype.update.call(this)},t.prototype.update_bar_style=function(){this.update_mapped_classes(t.class_map,"bar_style",this.bar)},t.prototype.set_bar_style=function(){this.set_mapped_classes(t.class_map,"bar_style",this.bar)},t.class_map={success:["progress-bar-success"],info:["progress-bar-info"],warning:["progress-bar-warning"],danger:["progress-bar-danger"]},t}(h),Xe=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return ke(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_model_name:"PlayModel",_view_name:"PlayView",_playing:!1,_repeat:!1,show_repeat:!0,interval:100,step:1,disabled:!1})},t.prototype.initialize=function(t,n){e.prototype.initialize.call(this,t,n)},t.prototype.loop=function(){if(this.get("_playing")){var e=this.get("value")+this.get("step");e<=this.get("max")?(this.set("value",e),this.schedule_next()):this.get("_repeat")?(this.set("value",this.get("min")),this.schedule_next()):this.set("_playing",!1),this.save_changes()}},t.prototype.schedule_next=function(){window.setTimeout(this.loop.bind(this),this.get("interval"))},t.prototype.stop=function(){this.set("_playing",!1),this.set("value",this.get("min")),this.save_changes()},t.prototype.pause=function(){this.set("_playing",!1),this.save_changes()},t.prototype.play=function(){this.set("_playing",!0),this.get("value")==this.get("max")?(this.set("value",this.get("min")),this.schedule_next(),this.save_changes()):this.loop()},t.prototype.repeat=function(){this.set("_repeat",!this.get("_repeat")),this.save_changes()},t}(Pe),qe=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return ke(t,e),t.prototype.render=function(){e.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-inline-hbox"),this.el.classList.add("widget-play"),this.playButton=document.createElement("button"),this.pauseButton=document.createElement("button"),this.stopButton=document.createElement("button"),this.repeatButton=document.createElement("button"),this.playButton.className="jupyter-button",this.pauseButton.className="jupyter-button",this.stopButton.className="jupyter-button",this.repeatButton.className="jupyter-button",this.el.appendChild(this.playButton),this.el.appendChild(this.pauseButton),this.el.appendChild(this.stopButton),this.el.appendChild(this.repeatButton);var t=document.createElement("i");t.className="fa fa-play",this.playButton.appendChild(t);var n=document.createElement("i");n.className="fa fa-pause",this.pauseButton.appendChild(n);var r=document.createElement("i");r.className="fa fa-stop",this.stopButton.appendChild(r);var i=document.createElement("i");i.className="fa fa-retweet",this.repeatButton.appendChild(i),this.playButton.onclick=this.model.play.bind(this.model),this.pauseButton.onclick=this.model.pause.bind(this.model),this.stopButton.onclick=this.model.stop.bind(this.model),this.repeatButton.onclick=this.model.repeat.bind(this.model),this.listenTo(this.model,"change:_playing",this.update_playing),this.listenTo(this.model,"change:_repeat",this.update_repeat),this.listenTo(this.model,"change:show_repeat",this.update_repeat),this.update_playing(),this.update_repeat(),this.update()},t.prototype.update=function(){var e=this.model.get("disabled");this.playButton.disabled=e,this.pauseButton.disabled=e,this.stopButton.disabled=e,this.repeatButton.disabled=e,this.update_playing()},t.prototype.update_playing=function(){var e=this.model.get("_playing"),t=this.model.get("disabled");e?(t||(this.pauseButton.disabled=!1),this.playButton.classList.add("mod-active")):(t||(this.pauseButton.disabled=!0),this.playButton.classList.remove("mod-active"))},t.prototype.update_repeat=function(){var e=this.model.get("_repeat");this.repeatButton.style.display=this.model.get("show_repeat")?this.playButton.style.display:"none",e?this.repeatButton.classList.add("mod-active"):this.repeatButton.classList.remove("mod-active")},t}(r.DOMWidgetView),Ze=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),Qe=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Ze(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_model_name:"FloatModel",value:0})},t}(b),Ke=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Ze(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_model_name:"BoundedFloatModel",max:100,min:0})},t}(Qe),Je=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Ze(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_model_name:"FloatSliderModel",_view_name:"FloatSliderView",step:1,orientation:"horizontal",_range:!1,readout:!0,readout_format:".2f",slider_color:null,continuous_update:!0,disabled:!1})},t.prototype.initialize=function(t,n){e.prototype.initialize.call(this,t,n),this.on("change:readout_format",this.update_readout_format,this),this.update_readout_format()},t.prototype.update_readout_format=function(){this.readout_formatter=we(this.get("readout_format"))},t}(Ke),$e=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Ze(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_model_name:"FloatLogSliderModel",_view_name:"FloatLogSliderView",step:.1,orientation:"horizontal",_range:!1,readout:!0,readout_format:".3g",slider_color:null,continuous_update:!0,disabled:!1,base:10,value:1,min:0,max:4})},t.prototype.initialize=function(t,n){e.prototype.initialize.call(this,t,n),this.on("change:readout_format",this.update_readout_format,this),this.update_readout_format()},t.prototype.update_readout_format=function(){this.readout_formatter=we(this.get("readout_format"))},t}(Ke),et=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Ze(t,e),t}(Je),tt=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t._parse_value=parseFloat,t}return Ze(t,e),t.prototype._validate_slide_value=function(e){return e},t}(Be),nt=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t._parse_value=parseFloat,t}return Ze(t,e),t.prototype.update=function(t){e.prototype.update.call(this,t);var n=this.model.get("min"),r=this.model.get("max"),i=this.model.get("value"),o=this.model.get("base"),a=Math.log(i)/Math.log(o);a>r?a=r:a<n&&(a=n),this.$slider.slider("option","value",a),this.readout.textContent=this.valueToString(i),this.model.get("value")!==i&&(this.model.set("value",i,{updated_view:this}),this.touch())},t.prototype.valueToString=function(e){return(0,this.model.readout_formatter)(e)},t.prototype.stringToValue=function(e){return this._parse_value(e)},t.prototype.handleTextChange=function(){var e=this.stringToValue(this.readout.textContent),t=this.model.get("min"),n=this.model.get("max"),r=this.model.get("base");isNaN(e)?this.readout.textContent=this.valueToString(this.model.get("value")):(e=Math.max(Math.min(e,Math.pow(r,n)),Math.pow(r,t)))!==this.model.get("value")?(this.readout.textContent=this.valueToString(e),this.model.set("value",e,{updated_view:this}),this.touch()):this.readout.textContent=this.valueToString(this.model.get("value"))},t.prototype.handleSliderChange=function(e,t){var n=this.model.get("base"),r=Math.pow(n,this._validate_slide_value(t.value));this.readout.textContent=this.valueToString(r),this.model.get("continuous_update")&&this.handleSliderChanged(e,t)},t.prototype.handleSliderChanged=function(e,t){var n=this.model.get("base"),r=Math.pow(n,this._validate_slide_value(t.value));this.model.set("value",r,{updated_view:this}),this.touch()},t.prototype._validate_slide_value=function(e){return e},t}(ze),rt=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t._parse_value=parseFloat,t._range_regex=/^\s*([+-]?(?:\d*\.?\d+|\d+\.)(?:[eE][-:]?\d+)?)\s*[-:–]\s*([+-]?(?:\d*\.?\d+|\d+\.)(?:[eE][+-]?\d+)?)/,t}return Ze(t,e),t.prototype._validate_slide_value=function(e){return e},t}(Fe),it=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Ze(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_model_name:"FloatTextModel",_view_name:"FloatTextView",disabled:!1,continuous_update:!1})},t}(Qe),ot=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Ze(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_model_name:"BoundedFloatTextModel",_view_name:"FloatTextView",disabled:!1,continuous_update:!1,step:.1})},t}(Ke),at=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t._parse_value=parseFloat,t._default_step="any",t}return Ze(t,e),t.prototype.handleKeypress=function(e){e.stopPropagation()},t.prototype.handleKeyUp=function(e){},t}(Ve),st=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Ze(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_model_name:"FloatProgressModel",_view_name:"ProgressView",orientation:"horizontal",bar_style:"",style:null})},t}(Ke),lt=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),ut=function(){return(ut=Object.assign||function(e){for(var t,n=1,r=arguments.length;n<r;n++)for(var i in t=arguments[n])Object.prototype.hasOwnProperty.call(t,i)&&(e[i]=t[i]);return e}).apply(this,arguments)},ct=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return lt(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_model_name:"ControllerButtonModel",_view_name:"ControllerButtonView",value:0,pressed:!1})},t}(y),dt=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return lt(t,e),t.prototype.render=function(){this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-controller-button"),this.el.style.width="fit-content",this.support=document.createElement("div"),this.support.style.position="relative",this.support.style.margin="1px",this.support.style.width="16px",this.support.style.height="16px",this.support.style.border="1px solid black",this.support.style.background="lightgray",this.el.appendChild(this.support),this.bar=document.createElement("div"),this.bar.style.position="absolute",this.bar.style.width="100%",this.bar.style.bottom="0px",this.bar.style.background="gray",this.support.appendChild(this.bar),this.update(),this.label=document.createElement("div"),this.label.textContent=this.model.get("description"),this.label.style.textAlign="center",this.el.appendChild(this.label)},t.prototype.update=function(){this.bar.style.height=100*this.model.get("value")+"%"},t}(r.DOMWidgetView),ht=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return lt(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_model_name:"ControllerAxisModel",_view_name:"ControllerAxisView",value:0})},t}(y),ft=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return lt(t,e),t.prototype.render=function(){this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-controller-axis"),this.el.style.width="16px",this.el.style.padding="4px",this.support=document.createElement("div"),this.support.style.position="relative",this.support.style.margin="1px",this.support.style.width="4px",this.support.style.height="64px",this.support.style.border="1px solid black",this.support.style.background="lightgray",this.bullet=document.createElement("div"),this.bullet.style.position="absolute",this.bullet.style.margin="-3px",this.bullet.style.boxSizing="unset",this.bullet.style.width="10px",this.bullet.style.height="10px",this.bullet.style.background="gray",this.label=document.createElement("div"),this.label.textContent=this.model.get("description"),this.label.style.textAlign="center",this.support.appendChild(this.bullet),this.el.appendChild(this.support),this.el.appendChild(this.label),this.update()},t.prototype.update=function(){this.bullet.style.top=50*(this.model.get("value")+1)+"%"},t}(r.DOMWidgetView),pt=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return lt(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_model_name:"ControllerModel",_view_name:"ControllerView",index:0,name:"",mapping:"",connected:!1,timestamp:0,buttons:[],axes:[]})},t.prototype.initialize=function(t,n){e.prototype.initialize.call(this,t,n),void 0===navigator.getGamepads?(this.readout="This browser does not support gamepads.",console.error(this.readout)):(this.readout="Connect gamepad and press any button.",this.get("connected")?this.update_loop():this.wait_loop())},t.prototype.wait_loop=function(){var e=this.get("index"),t=navigator.getGamepads()[e];if(t){var n=this;this.setup(t).then((function(e){n.set(e),n.save_changes(),window.requestAnimationFrame(n.update_loop.bind(n))}))}else window.requestAnimationFrame(this.wait_loop.bind(this))},t.prototype.setup=function(e){this.set({name:e.id,mapping:e.mapping,connected:e.connected,timestamp:e.timestamp});var t=this;return r.resolvePromisesDict({buttons:Promise.all(e.buttons.map((function(e,n){return t._create_button_model(n)}))),axes:Promise.all(e.axes.map((function(e,n){return t._create_axis_model(n)})))})},t.prototype.update_loop=function(){var e=this.get("index"),t=this.get("name"),n=navigator.getGamepads()[e];n&&e===n.index&&t===n.id?(this.set({timestamp:n.timestamp,connected:n.connected}),this.save_changes(),this.get("buttons").forEach((function(e,t){e.set({value:n.buttons[t].value,pressed:n.buttons[t].pressed}),e.save_changes()})),this.get("axes").forEach((function(e,t){e.set("value",n.axes[t]),e.save_changes()})),window.requestAnimationFrame(this.update_loop.bind(this))):this.reset_gamepad()},t.prototype.reset_gamepad=function(){this.get("buttons").forEach((function(e){e.close()})),this.get("axes").forEach((function(e){e.close()})),this.set({name:"",mapping:"",connected:!1,timestamp:0,buttons:[],axes:[]}),this.save_changes(),window.requestAnimationFrame(this.wait_loop.bind(this))},t.prototype._create_button_model=function(e){return this.widget_manager.new_widget({model_name:"ControllerButtonModel",model_module:"@jupyter-widgets/controls",model_module_version:this.get("_model_module_version"),view_name:"ControllerButtonView",view_module:"@jupyter-widgets/controls",view_module_version:this.get("_view_module_version")}).then((function(t){return t.set("description",e),t}))},t.prototype._create_axis_model=function(e){return this.widget_manager.new_widget({model_name:"ControllerAxisModel",model_module:"@jupyter-widgets/controls",model_module_version:this.get("_model_module_version"),view_name:"ControllerAxisView",view_module:"@jupyter-widgets/controls",view_module_version:this.get("_view_module_version")}).then((function(t){return t.set("description",e),t}))},t.serializers=ut(ut({},y.serializers),{buttons:{deserialize:r.unpack_models},axes:{deserialize:r.unpack_models}}),t}(y),gt=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return lt(t,e),t.prototype._createElement=function(e){return this.pWidget=new r.JupyterPhosphorPanelWidget({view:this}),this.pWidget.node},t.prototype._setElement=function(e){if(this.el||e!==this.pWidget.node)throw new Error("Cannot reset the DOM element.");this.el=this.pWidget.node,this.$el=B()(this.pWidget.node)},t.prototype.initialize=function(t){e.prototype.initialize.call(this,t),this.button_views=new r.ViewList(this.add_button,null,this),this.listenTo(this.model,"change:buttons",(function(e,t){this.button_views.update(t)})),this.axis_views=new r.ViewList(this.add_axis,null,this),this.listenTo(this.model,"change:axes",(function(e,t){this.axis_views.update(t)})),this.listenTo(this.model,"change:name",this.update_label)},t.prototype.render=function(){this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-controller"),this.label=document.createElement("div"),this.el.appendChild(this.label),this.axis_box=new z.Panel,this.axis_box.node.style.display="flex",this.pWidget.addWidget(this.axis_box),this.button_box=new z.Panel,this.button_box.node.style.display="flex",this.pWidget.addWidget(this.button_box),this.button_views.update(this.model.get("buttons")),this.axis_views.update(this.model.get("axes")),this.update_label()},t.prototype.update_label=function(){this.label.textContent=this.model.get("name")||this.model.readout},t.prototype.add_button=function(e){var t=this,n=new z.Widget;return this.button_box.addWidget(n),this.create_child_view(e).then((function(e){var r=j.ArrayExt.firstIndexOf(t.button_box.widgets,n);return t.button_box.insertWidget(r,e.pWidget),n.dispose(),e})).catch(i("Could not add child button view to controller",!0))},t.prototype.add_axis=function(e){var t=this,n=new z.Widget;return this.axis_box.addWidget(n),this.create_child_view(e).then((function(e){var r=j.ArrayExt.firstIndexOf(t.axis_box.widgets,n);return t.axis_box.insertWidget(r,e.pWidget),n.dispose(),e})).catch(i("Could not add child axis view to controller",!0))},t.prototype.remove=function(){e.prototype.remove.call(this),this.button_views.remove(),this.axis_views.remove()},t}(r.DOMWidgetView),vt=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),mt=function(){return(mt=Object.assign||function(e){for(var t,n=1,r=arguments.length;n<r;n++)for(var i in t=arguments[n])Object.prototype.hasOwnProperty.call(t,i)&&(e[i]=t[i]);return e}).apply(this,arguments)},yt=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return vt(t,e),t.prototype.defaults=function(){return mt(mt({},e.prototype.defaults.call(this)),{_model_name:"SelectionModel",index:"",_options_labels:[],disabled:!1})},t}(b),bt=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return vt(t,e),t.prototype.defaults=function(){return mt(mt({},e.prototype.defaults.call(this)),{_model_name:"DropdownModel",_view_name:"DropdownView",button_style:""})},t}(yt),Ct=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return vt(t,e),t.prototype.initialize=function(t){var n=this;e.prototype.initialize.call(this,t),this.listenTo(this.model,"change:_options_labels",(function(){return n._updateOptions()}))},t.prototype.render=function(){e.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-inline-hbox"),this.el.classList.add("widget-dropdown"),this.listbox=document.createElement("select"),this.listbox.id=this.label.htmlFor=Object(r.uuid)(),this.el.appendChild(this.listbox),this._updateOptions(),this.update()},t.prototype.update=function(){this.listbox.disabled=this.model.get("disabled");var t=this.model.get("index");return this.listbox.selectedIndex=null===t?-1:t,e.prototype.update.call(this)},t.prototype._updateOptions=function(){this.listbox.textContent="";for(var e=this.model.get("_options_labels"),t=0;t<e.length;t++){var n=e[t],r=document.createElement("option");r.textContent=n.replace(/ /g," "),r.setAttribute("data-value",encodeURIComponent(n)),r.value=n,this.listbox.appendChild(r)}},t.prototype.events=function(){return{"change select":"_handle_change"}},t.prototype._handle_change=function(){this.model.set("index",-1===this.listbox.selectedIndex?null:this.listbox.selectedIndex),this.touch()},t}(h),At=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return vt(t,e),t.prototype.defaults=function(){return mt(mt({},e.prototype.defaults.call(this)),{_model_name:"SelectModel",_view_name:"SelectView",rows:5})},t}(yt),_t=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return vt(t,e),t.prototype.initialize=function(t){var n=this;e.prototype.initialize.call(this,t),this.listenTo(this.model,"change:_options_labels",(function(){return n._updateOptions()})),this.listenTo(this.model,"change:index",(function(e,t,r){return n.updateSelection(r)})),this.listbox=document.createElement("select")},t.prototype.render=function(){e.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-inline-hbox"),this.el.classList.add("widget-select"),this.listbox.id=this.label.htmlFor=Object(r.uuid)(),this.el.appendChild(this.listbox),this._updateOptions(),this.update(),this.updateSelection()},t.prototype.update=function(){e.prototype.update.call(this),this.listbox.disabled=this.model.get("disabled");var t=this.model.get("rows");null===t&&(t=""),this.listbox.setAttribute("size",t)},t.prototype.updateSelection=function(e){if(void 0===e&&(e={}),e.updated_view!==this){var t=this.model.get("index");this.listbox.selectedIndex=null===t?-1:t}},t.prototype._updateOptions=function(){this.listbox.textContent="";for(var e=this.model.get("_options_labels"),t=0;t<e.length;t++){var n=e[t],r=document.createElement("option");r.textContent=n.replace(/ /g," "),r.setAttribute("data-value",encodeURIComponent(n)),r.value=n,this.listbox.appendChild(r)}},t.prototype.events=function(){return{"change select":"_handle_change"}},t.prototype._handle_change=function(){this.model.set("index",this.listbox.selectedIndex,{updated_view:this}),this.touch()},t}(h),It=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return vt(t,e),t.prototype.defaults=function(){return mt(mt({},e.prototype.defaults.call(this)),{_model_name:"RadioButtonsModel",_view_name:"RadioButtonsView",tooltips:[],icons:[],button_style:""})},t}(yt),xt=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return vt(t,e),t.prototype.render=function(){e.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-inline-hbox"),this.el.classList.add("widget-radio"),this.container=document.createElement("div"),this.el.appendChild(this.container),this.container.classList.add("widget-radio-box"),this.update()},t.prototype.update=function(t){var n=this,r=this.model.get("_options_labels"),i=g.pluck(this.container.querySelectorAll('input[type="radio"]'),"value"),o=r.length!=i.length;if(!o)for(var a=0,s=r.length;a<s;++a)if(i[a]!==r[a]){o=!0;break}return!o||void 0!==t&&t.updated_view===this||(this.container.textContent="",r.forEach((function(e,t){var r=document.createElement("label");r.textContent=e,n.container.appendChild(r);var i=document.createElement("input");i.setAttribute("type","radio"),i.value=t.toString(),i.setAttribute("data-value",encodeURIComponent(e)),r.appendChild(i)}))),r.forEach((function(e,t){var r='input[data-value="'+encodeURIComponent(e)+'"]',i=n.container.querySelectorAll(r);if(i.length>0){var o=i[0];o.checked=n.model.get("index")===t,o.disabled=n.model.get("disabled")}})),setTimeout(this.adjustPadding,0,this),e.prototype.update.call(this,t)},t.prototype.adjustPadding=function(e){var t=window.getComputedStyle(e.el),n=parseInt(t.marginTop,10)+parseInt(t.marginBottom,10),r=e.label.offsetHeight+n,i=window.getComputedStyle(e.container),o=parseInt(i.marginBottom),a=(e.el.offsetHeight+n-o)%r,s=0==a?0:r-a;e.container.style.marginBottom=s+"px"},t.prototype.events=function(){return{'click input[type="radio"]':"_handle_click"}},t.prototype._handle_click=function(e){var t=e.target;this.model.set("index",parseInt(t.value),{updated_view:this}),this.touch()},t}(h),wt=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return vt(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_model_name:"ToggleButtonsStyleModel"})},t.styleProperties=mt(mt({},c.styleProperties),{button_width:{selector:".widget-toggle-button",attribute:"width",default:null},font_weight:{selector:".widget-toggle-button",attribute:"font-weight",default:""}}),t}(c),Mt=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return vt(t,e),t.prototype.defaults=function(){return mt(mt({},e.prototype.defaults.call(this)),{_model_name:"ToggleButtonsModel",_view_name:"ToggleButtonsView"})},t}(yt),St=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return vt(t,e),t.prototype.initialize=function(t){this._css_state={},e.prototype.initialize.call(this,t),this.listenTo(this.model,"change:button_style",this.update_button_style)},t.prototype.render=function(){e.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-inline-hbox"),this.el.classList.add("widget-toggle-buttons"),this.buttongroup=document.createElement("div"),this.el.appendChild(this.buttongroup),this.update(),this.set_button_style()},t.prototype.update=function(n){for(var r=this,i=this.model.get("_options_labels"),o=this.model.get("icons")||[],s=this.model.previous("icons")||[],l=t.classMap[this.model.previous("button_style")]||"",u=r.model.get("tooltips")||[],c=this.model.get("disabled"),d=this.buttongroup.querySelectorAll("button"),h=g.pluck(d,"value"),f=!1,p=0,v=i.length;p<v;++p)if(h[p]!==i[p]||o[p]!==s[p]){f=!0;break}return!f||void 0!==n&&n.updated_view===this||(this.buttongroup.textContent="",i.forEach((function(e,t){var n;n=0===e.trim().length&&(!o[t]||0===o[t].trim().length)?"&nbsp;":a(e);var i=document.createElement("i"),s=document.createElement("button");o[t]&&(i.className="fa fa-"+o[t]),s.setAttribute("type","button"),s.className="widget-toggle-button jupyter-button",l&&s.classList.add(l),s.innerHTML=n,s.setAttribute("data-value",encodeURIComponent(e)),s.setAttribute("value",t.toString()),s.appendChild(i),s.disabled=c,u[t]&&s.setAttribute("title",u[t]),r.update_style_traits(s),r.buttongroup.appendChild(s)}))),i.forEach((function(e,t){var n='[data-value="'+encodeURIComponent(e)+'"]',i=r.buttongroup.querySelector(n);r.model.get("index")===t?i.classList.add("mod-active"):i.classList.remove("mod-active")})),this.stylePromise.then((function(e){e&&e.style()})),e.prototype.update.call(this,n)},t.prototype.update_style_traits=function(e){for(var t in this._css_state)if(this._css_state.hasOwnProperty(t))if("margin"===t)this.buttongroup.style[t]=this._css_state[t];else if("width"!==t)if(e)e.style[t]=this._css_state[t];else{var n=this.buttongroup.querySelectorAll("button");n.length&&(n[0].style[t]=this._css_state[t])}},t.prototype.update_button_style=function(){for(var e=this.buttongroup.querySelectorAll("button"),n=0;n<e.length;n++)this.update_mapped_classes(t.classMap,"button_style",e[n])},t.prototype.set_button_style=function(){for(var e=this.buttongroup.querySelectorAll("button"),n=0;n<e.length;n++)this.set_mapped_classes(t.classMap,"button_style",e[n])},t.prototype.events=function(){return{"click button":"_handle_click"}},t.prototype._handle_click=function(e){var t=e.target;this.model.set("index",parseInt(t.value,10),{updated_view:this}),this.touch(),this.send({event:"click"})},t}(h);(St||(St={})).classMap={primary:["mod-primary"],success:["mod-success"],info:["mod-info"],warning:["mod-warning"],danger:["mod-danger"]};var Tt=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return vt(t,e),t.prototype.defaults=function(){return mt(mt({},e.prototype.defaults.call(this)),{_model_name:"SelectionSliderModel",_view_name:"SelectionSliderView",orientation:"horizontal",readout:!0,continuous_update:!0})},t}(yt),Et=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return vt(t,e),t.prototype.render=function(){var t=this;e.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-inline-hbox"),this.el.classList.add("widget-hslider"),this.el.classList.add("widget-slider"),(this.$slider=B()("<div />")).slider({slide:this.handleSliderChange.bind(this),stop:this.handleSliderChanged.bind(this)}).addClass("slider"),this.slider_container=document.createElement("div"),this.slider_container.classList.add("slider-container"),this.slider_container.appendChild(this.$slider[0]),this.el.appendChild(this.slider_container),this.readout=document.createElement("div"),this.el.appendChild(this.readout),this.readout.classList.add("widget-readout"),this.readout.style.display="none",this.listenTo(this.model,"change:slider_color",(function(e,n){t.$slider.find("a").css("background",n)})),this.$slider.find("a").css("background",this.model.get("slider_color")),this.update()},t.prototype.update=function(t){if(void 0===t||t.updated_view!==this){var n=this.model.get("_options_labels").length-1;this.$slider.slider("option","step",1),this.$slider.slider("option","max",n),this.$slider.slider("option","min",0);var r=this.model.get("orientation");this.$slider.slider("option","value",0),this.$slider.slider("option","orientation",r);var i=this.model.get("disabled");this.$slider.slider("option","disabled",i),this.readout.contentEditable=i?"false":"true","vertical"===r?(this.el.classList.remove("widget-hslider"),this.el.classList.remove("widget-inline-hbox"),this.el.classList.add("widget-vslider"),this.el.classList.add("widget-inline-vbox")):(this.el.classList.remove("widget-vslider"),this.el.classList.remove("widget-inline-vbox"),this.el.classList.add("widget-hslider"),this.el.classList.add("widget-inline-hbox"));var o=this.model.get("readout");this.readout.style.display=o?"":"none",this.updateSelection()}return e.prototype.update.call(this,t)},t.prototype.events=function(){return{slide:"handleSliderChange",slidestop:"handleSliderChanged"}},t.prototype.updateSelection=function(){var e=this.model.get("index");this.$slider.slider("option","value",e),this.updateReadout(e)},t.prototype.updateReadout=function(e){var t=this.model.get("_options_labels")[e];this.readout.textContent=t},t.prototype.handleSliderChange=function(e,t){this.updateReadout(t.value),this.model.get("continuous_update")&&this.handleSliderChanged(e,t)},t.prototype.handleSliderChanged=function(e,t){this.updateReadout(t.value),this.model.set("index",t.value,{updated_view:this}),this.touch()},t}(h),Ot=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return vt(t,e),t.prototype.defaults=function(){return mt(mt({},e.prototype.defaults.call(this)),{_model_name:"MultipleSelectionModel"})},t}(yt),kt=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return vt(t,e),t.prototype.defaults=function(){return mt(mt({},e.prototype.defaults.call(this)),{_model_name:"SelectMultipleModel",_view_name:"SelectMultipleView",rows:null})},t}(Ot),Lt=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return vt(t,e),t.prototype.initialize=function(t){e.prototype.initialize.call(this,t),this.listbox.multiple=!0},t.prototype.render=function(){e.prototype.render.call(this),this.el.classList.add("widget-select-multiple")},t.prototype.updateSelection=function(e){if(void 0===e&&(e={}),e.updated_view!==this){var t=this.model.get("index")||[],n=this.listbox.options;this.listbox.selectedIndex=-1,t.forEach((function(e){n[e].selected=!0}))}},t.prototype._handle_change=function(){var e=Array.prototype.map.call(this.listbox.selectedOptions||[],(function(e){return e.index}));this.model.set("index",e,{updated_view:this}),this.touch()},t}(_t),Dt=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return vt(t,e),t.prototype.defaults=function(){return mt(mt({},e.prototype.defaults.call(this)),{_model_name:"SelectionSliderModel",_view_name:"SelectionSliderView",orientation:"horizontal",readout:!0,continuous_update:!0})},t}(Ot),Pt=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return vt(t,e),t.prototype.render=function(){e.prototype.render.call(this),this.$slider.slider("option","range",!0)},t.prototype.updateSelection=function(){var e=this.model.get("index");this.$slider.slider("option","values",e.slice()),this.updateReadout(e)},t.prototype.updateReadout=function(e){var t=this.model.get("_options_labels"),n=t[e[0]],r=t[e[1]];this.readout.textContent=n+"-"+r},t.prototype.handleSliderChange=function(e,t){this.updateReadout(t.values),this.model.get("continuous_update")&&this.handleSliderChanged(e,t)},t.prototype.handleSliderChanged=function(e,t){this.updateReadout(t.values),this.model.set("index",t.values.slice(),{updated_view:this}),this.touch()},t}(Et),Nt=n(16),jt=n(38),Rt=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),zt=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t._widgetRemoved=new Nt.Signal(t),t}return Rt(t,e),Object.defineProperty(t.prototype,"widgetRemoved",{get:function(){return this._widgetRemoved},enumerable:!0,configurable:!0}),t.prototype.onChildRemoved=function(e){this._widgetRemoved.emit(e.child)},t}(z.Panel),Ft=function(e){function t(t){void 0===t&&(t={});var n=e.call(this)||this;n._currentChanged=new Nt.Signal(n),n.addClass("p-TabPanel"),n.tabBar=new z.TabBar(t),n.tabBar.addClass("p-TabPanel-tabBar"),n.tabContents=new zt,n.tabContents.addClass("p-TabPanel-tabContents"),n.tabBar.tabMoved.connect(n._onTabMoved,n),n.tabBar.currentChanged.connect(n._onCurrentChanged,n),n.tabBar.tabCloseRequested.connect(n._onTabCloseRequested,n),n.tabBar.tabActivateRequested.connect(n._onTabActivateRequested,n),n.tabContents.widgetRemoved.connect(n._onWidgetRemoved,n);var r=new z.PanelLayout;return r.addWidget(n.tabBar),r.addWidget(n.tabContents),n.layout=r,n}return Rt(t,e),Object.defineProperty(t.prototype,"currentChanged",{get:function(){return this._currentChanged},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"currentIndex",{get:function(){var e=this.tabBar.currentIndex;return-1===e?null:e},set:function(e){this.tabBar.currentIndex=null===e?-1:e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"currentWidget",{get:function(){var e=this.tabBar.currentTitle;return e?e.owner:null},set:function(e){this.tabBar.currentTitle=e?e.title:null},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"tabsMovable",{get:function(){return this.tabBar.tabsMovable},set:function(e){this.tabBar.tabsMovable=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"widgets",{get:function(){return this.tabContents.widgets},enumerable:!0,configurable:!0}),t.prototype.addWidget=function(e){this.insertWidget(this.widgets.length,e)},t.prototype.insertWidget=function(e,t){t!==this.currentWidget&&t.hide(),this.tabContents.insertWidget(e,t),this.tabBar.insertTab(e,t.title)},t.prototype._onCurrentChanged=function(e,t){var n=t.previousIndex,r=t.previousTitle,i=t.currentIndex,o=t.currentTitle,a=r?r.owner:null,s=o?o.owner:null;a&&a.hide(),s&&s.show(),this._currentChanged.emit({previousIndex:n,previousWidget:a,currentIndex:i,currentWidget:s}),(jt.b.IS_EDGE||jt.b.IS_IE)&&R.c.flush()},t.prototype._onTabActivateRequested=function(e,t){t.title.owner.activate()},t.prototype._onTabCloseRequested=function(e,t){t.title.owner.close()},t.prototype._onTabMoved=function(e,t){this.tabContents.insertWidget(t.toIndex,t.title.owner)},t.prototype._onWidgetRemoved=function(e,t){this.tabBar.removeTab(t.title)},t}(z.Widget),Bt=function(){function e(e,t){void 0===t&&(t={}),this._array=null,this._value=null,this._previousValue=null,this._selectionChanged=new Nt.Signal(this),this._array=e,this._insertBehavior=t.insertBehavior||"select-item-if-needed",this._removeBehavior=t.removeBehavior||"select-item-after"}return Object.defineProperty(e.prototype,"selectionChanged",{get:function(){return this._selectionChanged},enumerable:!0,configurable:!0}),e.prototype.adjustSelectionForSet=function(e){var t=this.index,n=this.value;if(e===t){this._updateSelectedValue();var r=this.value;this._previousValue=null,n!==r&&this._selectionChanged.emit({previousIndex:t,previousValue:n,currentIndex:t,currentValue:r})}},Object.defineProperty(e.prototype,"value",{get:function(){return this._value},set:function(e){this.index=null===e?null:j.ArrayExt.firstIndexOf(this._array,e)},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"index",{get:function(){return this._index},set:function(e){var t;if(null!==e?((t=Math.floor(e))<0||t>=this._array.length)&&(t=null):t=null,this._index!==t){var n=this._index,r=this._value;this._index=t,this._updateSelectedValue(),this._previousValue=r,this._selectionChanged.emit({previousIndex:n,previousValue:r,currentIndex:t,currentValue:this._value})}},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"insertBehavior",{get:function(){return this._insertBehavior},set:function(e){this._insertBehavior=e},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"removeBehavior",{get:function(){return this._removeBehavior},set:function(e){this._removeBehavior=e},enumerable:!0,configurable:!0}),e.prototype.adjustSelectionForInsert=function(e,t){var n=this._value,r=this._index,i=this._insertBehavior;if("select-item"===i||"select-item-if-needed"===i&&null===r)return this._index=e,this._value=t,this._previousValue=n,void this._selectionChanged.emit({previousIndex:r,previousValue:n,currentIndex:e,currentValue:t});r>=e&&this._index++},e.prototype.adjustSelectionForMove=function(e,t){this._index===e?this._index=t:this._index<e&&this._index>=t?this._index++:this._index>e&&this._index<=t&&this._index--},e.prototype.clearSelection=function(){var e=this._index,t=this._value;this._index=null,this._value=null,this._previousValue=null,null!==e&&this._selectionChanged.emit({previousIndex:e,previousValue:t,currentIndex:this._index,currentValue:this._value})},e.prototype.adjustSelectionForRemove=function(e,t){var n=this._index,r=this._removeBehavior;if(n===e){if(0===this._array.length)return this._index=null,this._value=null,this._previousValue=null,void this._selectionChanged.emit({previousIndex:e,previousValue:t,currentIndex:this._index,currentValue:this._value});if("select-item-after"===r)return this._index=Math.min(e,this._array.length-1),this._updateSelectedValue(),this._previousValue=null,void this._selectionChanged.emit({previousIndex:e,previousValue:t,currentIndex:this._index,currentValue:this._value});if("select-item-before"===r)return this._index=Math.max(0,e-1),this._updateSelectedValue(),this._previousValue=null,void this._selectionChanged.emit({previousIndex:e,previousValue:t,currentIndex:this._index,currentValue:this._value});if("select-previous-item"===r)return this._previousValue?this.value=this._previousValue:(this._index=Math.min(e,this._array.length-1),this._updateSelectedValue()),this._previousValue=null,void this._selectionChanged.emit({previousIndex:e,previousValue:t,currentIndex:this._index,currentValue:this.value});this._index=null,this._value=null,this._previousValue=null,this._selectionChanged.emit({previousIndex:e,previousValue:t,currentIndex:this._index,currentValue:this._value})}else n>e&&this._index--},e.prototype._updateSelectedValue=function(){var e=this._index;this._value=null!==e?this._array[e]:null},e}(),Ht=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),Ut=function(e){function t(t){var n=e.call(this,t)||this;n._collapseChanged=new Nt.Signal(n),n.addClass("p-Collapse"),n._header=new z.Widget,n._header.addClass("p-Collapse-header"),n._header.node.addEventListener("click",n);var r=document.createElement("i");r.classList.add("fa","fa-fw","fa-caret-right"),n._header.node.appendChild(r),n._header.node.appendChild(document.createElement("span")),n._content=new z.Panel,n._content.addClass("p-Collapse-contents");var i=new z.PanelLayout;return n.layout=i,i.addWidget(n._header),i.addWidget(n._content),t.widget&&(n.widget=t.widget),n.collapsed=!1,n}return Ht(t,e),t.prototype.dispose=function(){this.isDisposed||(e.prototype.dispose.call(this),this._header=null,this._widget=null,this._content=null)},Object.defineProperty(t.prototype,"widget",{get:function(){return this._widget},set:function(e){var t=this._widget;t&&(t.disposed.disconnect(this._onChildDisposed,this),t.title.changed.disconnect(this._onTitleChanged,this),t.parent=null),this._widget=e,e.disposed.connect(this._onChildDisposed,this),e.title.changed.connect(this._onTitleChanged,this),this._onTitleChanged(e.title),this._content.addWidget(e)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"collapsed",{get:function(){return this._collapsed},set:function(e){e!==this._collapsed&&(e?this._collapse():this._uncollapse())},enumerable:!0,configurable:!0}),t.prototype.toggle=function(){this.collapsed=!this.collapsed},Object.defineProperty(t.prototype,"collapseChanged",{get:function(){return this._collapseChanged},enumerable:!0,configurable:!0}),t.prototype._collapse=function(){this._collapsed=!0,this._content&&this._content.hide(),this.removeClass("p-Collapse-open"),this._header.node.children[0].classList.add("fa-caret-right"),this._header.node.children[0].classList.remove("fa-caret-down"),this._collapseChanged.emit(void 0)},t.prototype._uncollapse=function(){this._collapsed=!1,this._content&&this._content.show(),this.addClass("p-Collapse-open"),this._header.node.children[0].classList.add("fa-caret-down"),this._header.node.children[0].classList.remove("fa-caret-right"),this._collapseChanged.emit(void 0)},t.prototype.handleEvent=function(e){switch(e.type){case"click":this._evtClick(e)}},t.prototype._evtClick=function(e){this.toggle()},t.prototype._onTitleChanged=function(e){this._header.node.children[1].textContent=this._widget.title.label},t.prototype._onChildDisposed=function(e){this.dispose()},t}(z.Widget),Vt=function(e){function t(t){var n=e.call(this,t)||this;return n._selection=new Bt(n.widgets),n._selection.selectionChanged.connect(n._onSelectionChanged,n),n.addClass("p-Accordion"),n}return Ht(t,e),Object.defineProperty(t.prototype,"collapseWidgets",{get:function(){return this.layout.widgets},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"selection",{get:function(){return this._selection},enumerable:!0,configurable:!0}),t.prototype.indexOf=function(e){return j.ArrayExt.findFirstIndex(this.collapseWidgets,(function(t){return t.widget===e}))},t.prototype.addWidget=function(t){var n=this._wrapWidget(t);return n.collapsed=!0,e.prototype.addWidget.call(this,n),this._selection.adjustSelectionForInsert(this.widgets.length-1,n),n},t.prototype.insertWidget=function(t,n){var r=this._wrapWidget(n);r.collapsed=!0,e.prototype.insertWidget.call(this,t,r),this._selection.adjustSelectionForInsert(t,r)},t.prototype.removeWidget=function(e){var t=this.indexOf(e);if(t>=0){var n=this.collapseWidgets[t];e.parent=null,n.dispose(),this._selection.adjustSelectionForRemove(t,null)}},t.prototype._wrapWidget=function(e){var t=new Ut({widget:e});return t.addClass("p-Accordion-child"),t.collapseChanged.connect(this._onCollapseChange,this),t},t.prototype._onCollapseChange=function(e){e.collapsed?this._selection.value===e&&e.collapsed&&(this._selection.value=null):this._selection.value=e},t.prototype._onSelectionChanged=function(e,t){var n=t.previousValue,r=t.currentValue;n&&(n.collapsed=!0,n.removeClass("p-Accordion-child-active")),r&&(r.collapsed=!1,r.addClass("p-Accordion-child-active"))},t}(z.Panel),Yt=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),Wt=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Yt(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_model_name:"SelectionContainerModel",selected_index:0,_titles:{}})},t}(V),Gt=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Yt(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_model_name:"AccordionModel",_view_name:"AccordionView"})},t}(Wt),Xt=function(e){function t(t){var n=this,r=t.view;return delete t.view,(n=e.call(this,t)||this)._view=r,n}return Yt(t,e),t.prototype.processMessage=function(t){e.prototype.processMessage.call(this,t),this._view.processPhosphorMessage(t)},t.prototype.dispose=function(){this.isDisposed||(e.prototype.dispose.call(this),this._view&&this._view.remove(),this._view=null)},t}(Vt),qt=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Yt(t,e),t.prototype._createElement=function(e){return this.pWidget=new Xt({view:this}),this.pWidget.node},t.prototype._setElement=function(e){if(this.el||e!==this.pWidget.node)throw new Error("Cannot reset the DOM element.");this.el=this.pWidget.node,this.$el=B()(this.pWidget.node)},t.prototype.initialize=function(t){var n=this;e.prototype.initialize.call(this,t),this.children_views=new r.ViewList(this.add_child_view,this.remove_child_view,this),this.listenTo(this.model,"change:children",(function(){return n.updateChildren()})),this.listenTo(this.model,"change:selected_index",(function(){return n.update_selected_index()})),this.listenTo(this.model,"change:_titles",(function(){return n.update_titles()}))},t.prototype.render=function(){var t=this;e.prototype.render.call(this);var n=this.pWidget;n.addClass("jupyter-widgets"),n.addClass("widget-accordion"),n.addClass("widget-container"),n.selection.selectionChanged.connect((function(e){t.updatingChildren||(t.model.set("selected_index",n.selection.index),t.touch())})),this.children_views.update(this.model.get("children")),this.update_titles(),this.update_selected_index()},t.prototype.updateChildren=function(){this.updatingChildren=!0,this.pWidget.selection.index=null,this.children_views.update(this.model.get("children")),this.update_selected_index(),this.updatingChildren=!1},t.prototype.update_titles=function(){for(var e=this.pWidget.collapseWidgets,t=this.model.get("_titles"),n=0;n<e.length;n++)void 0!==t[n]&&(e[n].widget.title.label=t[n])},t.prototype.update_selected_index=function(){this.pWidget.selection.index=this.model.get("selected_index")},t.prototype.remove_child_view=function(e){this.pWidget.removeWidget(e.pWidget),e.remove()},t.prototype.add_child_view=function(e,t){var n=this.pWidget,r=new z.Widget;return r.title.label=this.model.get("_titles")[t]||"",n.addWidget(r),this.create_child_view(e).then((function(e){var t=e.pWidget;return t.title.label=r.title.label,n.collapseWidgets[n.indexOf(r)].widget=t,r.dispose(),e})).catch(i("Could not add child view to box",!0))},t.prototype.remove=function(){this.children_views=null,e.prototype.remove.call(this)},t}(r.DOMWidgetView),Zt=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Yt(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_model_name:"TabModel",_view_name:"TabView"})},t}(Wt),Qt=function(e){function t(t){var n=this,r=t.view;return delete t.view,(n=e.call(this,t)||this)._view=r,R.c.installMessageHook(n.tabContents,(function(e,t){return n._view.processPhosphorMessage(t),!0})),n}return Yt(t,e),t.prototype.dispose=function(){this.isDisposed||(e.prototype.dispose.call(this),this._view&&this._view.remove(),this._view=null)},t}(Ft),Kt=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.updatingTabs=!1,t}return Yt(t,e),t.prototype._createElement=function(e){return this.pWidget=new Qt({view:this}),this.pWidget.node},t.prototype._setElement=function(e){if(this.el||e!==this.pWidget.node)throw new Error("Cannot reset the DOM element.");this.el=this.pWidget.node,this.$el=B()(this.pWidget.node)},t.prototype.initialize=function(t){var n=this;e.prototype.initialize.call(this,t),this.childrenViews=new r.ViewList(this.addChildView,(function(e){e.remove()}),this),this.listenTo(this.model,"change:children",(function(){return n.updateTabs()})),this.listenTo(this.model,"change:_titles",(function(){return n.updateTitles()}))},t.prototype.render=function(){e.prototype.render.call(this);var t=this.pWidget;t.addClass("jupyter-widgets"),t.addClass("widget-container"),t.addClass("widget-tab"),t.tabsMovable=!0,t.tabBar.insertBehavior="none",t.tabBar.currentChanged.connect(this._onTabChanged,this),t.tabBar.tabMoved.connect(this._onTabMoved,this),t.tabBar.addClass("widget-tab-bar"),t.tabContents.addClass("widget-tab-contents"),t.tabBar.tabsMovable=!1,this.updateTabs(),this.update()},t.prototype.updateTabs=function(){this.updatingTabs=!0,this.pWidget.currentIndex=null,this.childrenViews.update(this.model.get("children")),this.pWidget.currentIndex=this.model.get("selected_index"),this.updatingTabs=!1},t.prototype.addChildView=function(e,t){var n=this.model.get("_titles")[t]||"",r=this.pWidget,o=new z.Widget;return o.title.label=n,r.addWidget(o),this.create_child_view(e).then((function(e){var t=e.pWidget;t.title.label=o.title.label,t.title.closable=!1;var n=j.ArrayExt.firstIndexOf(r.widgets,o);return r.insertWidget(n+1,t),o.dispose(),e})).catch(i("Could not add child view to box",!0))},t.prototype.update=function(){return this.updateSelectedIndex(),e.prototype.update.call(this)},t.prototype.updateTitles=function(){var e=this.model.get("_titles")||{};Object(j.each)(this.pWidget.widgets,(function(t,n){t.title.label=e[n]||""}))},t.prototype.updateSelectedIndex=function(){this.pWidget.currentIndex=this.model.get("selected_index")},t.prototype.remove=function(){this.childrenViews=null,e.prototype.remove.call(this)},t.prototype._onTabChanged=function(e,t){if(!this.updatingTabs){var n=t.currentIndex;this.model.set("selected_index",-1===n?null:n),this.touch()}},t.prototype._onTabMoved=function(e,t){var n=this.model.get("children").slice();j.ArrayExt.move(n,t.fromIndex,t.toIndex),this.model.set("children",n),this.touch()},t}(r.DOMWidgetView),Jt=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),$t=function(){return($t=Object.assign||function(e){for(var t,n=1,r=arguments.length;n<r;n++)for(var i in t=arguments[n])Object.prototype.hasOwnProperty.call(t,i)&&(e[i]=t[i]);return e}).apply(this,arguments)},en=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Jt(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{value:"",disabled:!1,placeholder:"​",_model_name:"StringModel"})},t}(b),tn=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Jt(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_view_name:"HTMLView",_model_name:"HTMLModel"})},t}(en),nn=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Jt(t,e),t.prototype.render=function(){e.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-inline-hbox"),this.el.classList.add("widget-html"),this.content=document.createElement("div"),this.content.classList.add("widget-html-content"),this.el.appendChild(this.content),this.update()},t.prototype.update=function(){return this.content.innerHTML=this.model.get("value"),e.prototype.update.call(this)},t}(h),rn=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Jt(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_view_name:"HTMLMathView",_model_name:"HTMLMathModel"})},t}(en),on=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Jt(t,e),t.prototype.render=function(){e.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-inline-hbox"),this.el.classList.add("widget-htmlmath"),this.content=document.createElement("div"),this.content.classList.add("widget-htmlmath-content"),this.el.appendChild(this.content),this.update()},t.prototype.update=function(){return this.content.innerHTML=this.model.get("value"),this.typeset(this.content),e.prototype.update.call(this)},t}(h),an=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Jt(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_view_name:"LabelView",_model_name:"LabelModel"})},t}(en),sn=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Jt(t,e),t.prototype.render=function(){e.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-label"),this.update()},t.prototype.update=function(){return this.typeset(this.el,this.model.get("value")),e.prototype.update.call(this)},t}(h),ln=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Jt(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_view_name:"TextareaView",_model_name:"TextareaModel",rows:null,continuous_update:!0})},t}(en),un=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Jt(t,e),t.prototype.render=function(){var t=this;e.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-inline-hbox"),this.el.classList.add("widget-textarea"),this.textbox=document.createElement("textarea"),this.textbox.setAttribute("rows","5"),this.textbox.id=this.label.htmlFor=Object(r.uuid)(),this.el.appendChild(this.textbox),this.update(),this.listenTo(this.model,"change:placeholder",(function(e,n,r){t.update_placeholder(n)})),this.update_placeholder()},t.prototype.update_placeholder=function(e){e=e||this.model.get("placeholder"),this.textbox.setAttribute("placeholder",e.toString())},t.prototype.update=function(t){if(void 0===t||t.updated_view!=this){this.textbox.value=this.model.get("value");var n=this.model.get("rows");null===n&&(n=""),this.textbox.setAttribute("rows",n),this.textbox.disabled=this.model.get("disabled")}return e.prototype.update.call(this)},t.prototype.events=function(){return{"keydown input":"handleKeyDown","keypress input":"handleKeypress","input textarea":"handleChanging","change textarea":"handleChanged"}},t.prototype.handleKeyDown=function(e){e.stopPropagation()},t.prototype.handleKeypress=function(e){e.stopPropagation()},t.prototype.handleChanging=function(e){this.model.get("continuous_update")&&this.handleChanged(e)},t.prototype.handleChanged=function(e){var t=e.target;this.model.set("value",t.value,{updated_view:this}),this.touch()},t}(h),cn=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Jt(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_view_name:"TextView",_model_name:"TextModel",continuous_update:!0})},t}(en),dn=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.inputType="text",t}return Jt(t,e),t.prototype.render=function(){var t=this;e.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-inline-hbox"),this.el.classList.add("widget-text"),this.textbox=document.createElement("input"),this.textbox.setAttribute("type",this.inputType),this.textbox.id=this.label.htmlFor=Object(r.uuid)(),this.el.appendChild(this.textbox),this.update(),this.listenTo(this.model,"change:placeholder",(function(e,n,r){t.update_placeholder(n)})),this.listenTo(this.model,"change:description_tooltip",this.update_title),this.listenTo(this.model,"change:description",this.update_title),this.update_placeholder(),this.update_title()},t.prototype.update_placeholder=function(e){this.textbox.setAttribute("placeholder",e||this.model.get("placeholder"))},t.prototype.update_title=function(){var e=this.model.get("description_tooltip");e?0===this.model.get("description").length&&this.textbox.setAttribute("title",e):this.textbox.removeAttribute("title")},t.prototype.update=function(t){return void 0!==t&&t.updated_view===this||(this.textbox.value!==this.model.get("value")&&(this.textbox.value=this.model.get("value")),this.textbox.disabled=this.model.get("disabled")),e.prototype.update.call(this)},t.prototype.events=function(){return{"keydown input":"handleKeyDown","keypress input":"handleKeypress","input input":"handleChanging","change input":"handleChanged"}},t.prototype.handleKeyDown=function(e){e.stopPropagation()},t.prototype.handleKeypress=function(e){e.stopPropagation(),13===e.keyCode&&this.send({event:"submit"})},t.prototype.handleChanging=function(e){this.model.get("continuous_update")&&this.handleChanged(e)},t.prototype.handleChanged=function(e){var t=e.target;this.model.set("value",t.value,{updated_view:this}),this.touch()},t}(h),hn=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Jt(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_view_name:"PasswordView",_model_name:"PasswordModel"})},t}(cn),fn=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.inputType="password",t}return Jt(t,e),t}(dn),pn=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Jt(t,e),t.prototype.defaults=function(){return $t($t({},e.prototype.defaults.call(this)),{_model_name:"ComboboxModel",_view_name:"ComboboxView",options:[],ensure_options:!1})},t}(cn),gn=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.isInitialRender=!0,t}return Jt(t,e),t.prototype.render=function(){this.datalist=document.createElement("datalist"),this.datalist.id=Object(r.uuid)(),e.prototype.render.call(this),this.textbox.setAttribute("list",this.datalist.id),this.el.appendChild(this.datalist)},t.prototype.update=function(t){if(e.prototype.update.call(this,t),this.datalist){var n=this.isValid(this.model.get("value"));if(this.highlightValidState(n),!(void 0!==t&&t.updated_view||!this.model.hasChanged("options")&&!this.isInitialRender)){this.isInitialRender=!1;var r=this.model.get("options").map((function(e){return'<option value="'+e+'"></option>'}));this.datalist.innerHTML=r.join("\n")}}},t.prototype.isValid=function(e){if(!0===this.model.get("ensure_option")&&-1===this.model.get("options").indexOf(e))return!1;return!0},t.prototype.handleChanging=function(t){var n=t.target,r=this.isValid(n.value);this.highlightValidState(r),r&&e.prototype.handleChanging.call(this,t)},t.prototype.handleChanged=function(t){var n=t.target,r=this.isValid(n.value);this.highlightValidState(r),r&&e.prototype.handleChanged.call(this,t)},t.prototype.highlightValidState=function(e){this.textbox.classList.toggle("jpwidgets-invalidComboValue",!e)},t}(dn),vn=function(){var e=function(t,n){return(e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(t,n)};return function(t,n){function r(){this.constructor=t}e(t,n),t.prototype=null===n?Object.create(n):(r.prototype=n.prototype,new r)}}(),mn=function(){return(mn=Object.assign||function(e){for(var t,n=1,r=arguments.length;n<r;n++)for(var i in t=arguments[n])Object.prototype.hasOwnProperty.call(t,i)&&(e[i]=t[i]);return e}).apply(this,arguments)},yn=function(){for(var e=0,t=0,n=arguments.length;t<n;t++)e+=arguments[t].length;var r=Array(e),i=0;for(t=0;t<n;t++)for(var o=arguments[t],a=0,s=o.length;a<s;a++,i++)r[i]=o[a];return r},bn=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return vn(t,e),t.prototype.defaults=function(){return g.extend(e.prototype.defaults.call(this),{_model_name:"FileUploadModel",_view_name:"FileUploadView",_counter:0,accept:"",description:"Upload",tooltip:"",disabled:!1,icon:"upload",button_style:"",multiple:!1,metadata:[],data:[],error:"",style:null})},t.serializers=mn(mn({},y.serializers),{data:{serialize:function(e){return yn(e)}}}),t}(y),Cn=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return vn(t,e),Object.defineProperty(t.prototype,"tagName",{get:function(){return"button"},enumerable:!0,configurable:!0}),t.prototype.render=function(){var t=this;e.prototype.render.call(this),this.el.classList.add("jupyter-widgets"),this.el.classList.add("widget-upload"),this.el.classList.add("jupyter-button"),this.fileInput=document.createElement("input"),this.fileInput.type="file",this.fileInput.style.display="none",this.el.appendChild(this.fileInput),this.el.addEventListener("click",(function(){t.fileInput.click()})),this.fileInput.addEventListener("click",(function(){t.fileInput.value=""})),this.fileInput.addEventListener("change",(function(){var e=[];Array.from(t.fileInput.files).forEach((function(n){e.push(new Promise((function(e,r){var i={name:n.name,type:n.type,size:n.size,lastModified:n.lastModified};t.fileReader=new FileReader,t.fileReader.onload=function(t){var n=t.target.result;e({buffer:n,metadata:i,error:""})},t.fileReader.onerror=function(){r()},t.fileReader.onabort=t.fileReader.onerror,t.fileReader.readAsArrayBuffer(n)})))})),Promise.all(e).then((function(e){var n=[],r=[];e.forEach((function(e){n.push(e.metadata),r.push(e.buffer)}));var i=t.model.get("_counter");t.model.set({_counter:i+e.length,metadata:n,data:r,error:""}),t.touch()})).catch((function(e){console.error("error in file upload: %o",e),t.model.set({error:e}),t.touch()}))})),this.listenTo(this.model,"change:button_style",this.update_button_style),this.set_button_style(),this.update()},t.prototype.update=function(){this.el.disabled=this.model.get("disabled"),this.el.setAttribute("title",this.model.get("tooltip"));var t=this.model.get("description")+" ("+this.model.get("_counter")+")",n=this.model.get("icon");if(t.length||n.length){if(this.el.textContent="",n.length){var r=document.createElement("i");r.classList.add("fa"),r.classList.add("fa-"+n),0===t.length&&r.classList.add("center"),this.el.appendChild(r)}this.el.appendChild(document.createTextNode(t))}return this.fileInput.accept=this.model.get("accept"),this.fileInput.multiple=this.model.get("multiple"),e.prototype.update.call(this)},t.prototype.update_button_style=function(){this.update_mapped_classes(t.class_map,"button_style",this.el)},t.prototype.set_button_style=function(){this.set_mapped_classes(t.class_map,"button_style",this.el)},t.class_map={primary:["mod-primary"],success:["mod-success"],info:["mod-info"],warning:["mod-warning"],danger:["mod-danger"]},t}(r.DOMWidgetView);n.d(t,"version",(function(){return An})),n.d(t,"uuid",(function(){return r.uuid})),n.d(t,"WrappedError",(function(){return r.WrappedError})),n.d(t,"resolvePromisesDict",(function(){return r.resolvePromisesDict})),n.d(t,"reject",(function(){return i})),n.d(t,"typeset",(function(){return o})),n.d(t,"escape_html",(function(){return a})),n.d(t,"JUPYTER_CONTROLS_VERSION",(function(){return"1.5.0"})),n.d(t,"DirectionalLinkModel",(function(){return _})),n.d(t,"LinkModel",(function(){return I})),n.d(t,"BoolModel",(function(){return w})),n.d(t,"CheckboxModel",(function(){return M})),n.d(t,"CheckboxView",(function(){return S})),n.d(t,"ToggleButtonModel",(function(){return T})),n.d(t,"ToggleButtonView",(function(){return E})),n.d(t,"ValidModel",(function(){return O})),n.d(t,"ValidView",(function(){return k})),n.d(t,"ButtonStyleModel",(function(){return D})),n.d(t,"ButtonModel",(function(){return P})),n.d(t,"ButtonView",(function(){return N})),n.d(t,"BoxModel",(function(){return V})),n.d(t,"HBoxModel",(function(){return Y})),n.d(t,"VBoxModel",(function(){return W})),n.d(t,"BoxView",(function(){return G})),n.d(t,"HBoxView",(function(){return X})),n.d(t,"VBoxView",(function(){return q})),n.d(t,"GridBoxView",(function(){return Z})),n.d(t,"GridBoxModel",(function(){return Q})),n.d(t,"ImageModel",(function(){return $})),n.d(t,"ImageView",(function(){return ee})),n.d(t,"VideoModel",(function(){return re})),n.d(t,"VideoView",(function(){return ie})),n.d(t,"AudioModel",(function(){return se})),n.d(t,"AudioView",(function(){return le})),n.d(t,"ColorPickerModel",(function(){return ce})),n.d(t,"ColorPickerView",(function(){return de})),n.d(t,"serialize_date",(function(){return ge})),n.d(t,"deserialize_date",(function(){return ve})),n.d(t,"DatePickerModel",(function(){return me})),n.d(t,"DatePickerView",(function(){return ye})),n.d(t,"IntModel",(function(){return De})),n.d(t,"BoundedIntModel",(function(){return Pe})),n.d(t,"SliderStyleModel",(function(){return Ne})),n.d(t,"IntSliderModel",(function(){return je})),n.d(t,"IntRangeSliderModel",(function(){return Re})),n.d(t,"BaseIntSliderView",(function(){return ze})),n.d(t,"IntRangeSliderView",(function(){return Fe})),n.d(t,"IntSliderView",(function(){return Be})),n.d(t,"IntTextModel",(function(){return He})),n.d(t,"BoundedIntTextModel",(function(){return Ue})),n.d(t,"IntTextView",(function(){return Ve})),n.d(t,"ProgressStyleModel",(function(){return Ye})),n.d(t,"IntProgressModel",(function(){return We})),n.d(t,"ProgressView",(function(){return Ge})),n.d(t,"PlayModel",(function(){return Xe})),n.d(t,"PlayView",(function(){return qe})),n.d(t,"FloatModel",(function(){return Qe})),n.d(t,"BoundedFloatModel",(function(){return Ke})),n.d(t,"FloatSliderModel",(function(){return Je})),n.d(t,"FloatLogSliderModel",(function(){return $e})),n.d(t,"FloatRangeSliderModel",(function(){return et})),n.d(t,"FloatSliderView",(function(){return tt})),n.d(t,"FloatLogSliderView",(function(){return nt})),n.d(t,"FloatRangeSliderView",(function(){return rt})),n.d(t,"FloatTextModel",(function(){return it})),n.d(t,"BoundedFloatTextModel",(function(){return ot})),n.d(t,"FloatTextView",(function(){return at})),n.d(t,"FloatProgressModel",(function(){return st})),n.d(t,"ControllerButtonModel",(function(){return ct})),n.d(t,"ControllerButtonView",(function(){return dt})),n.d(t,"ControllerAxisModel",(function(){return ht})),n.d(t,"ControllerAxisView",(function(){return ft})),n.d(t,"ControllerModel",(function(){return pt})),n.d(t,"ControllerView",(function(){return gt})),n.d(t,"SelectionModel",(function(){return yt})),n.d(t,"DropdownModel",(function(){return bt})),n.d(t,"DropdownView",(function(){return Ct})),n.d(t,"SelectModel",(function(){return At})),n.d(t,"SelectView",(function(){return _t})),n.d(t,"RadioButtonsModel",(function(){return It})),n.d(t,"RadioButtonsView",(function(){return xt})),n.d(t,"ToggleButtonsStyleModel",(function(){return wt})),n.d(t,"ToggleButtonsModel",(function(){return Mt})),n.d(t,"ToggleButtonsView",(function(){return St})),n.d(t,"SelectionSliderModel",(function(){return Tt})),n.d(t,"SelectionSliderView",(function(){return Et})),n.d(t,"MultipleSelectionModel",(function(){return Ot})),n.d(t,"SelectMultipleModel",(function(){return kt})),n.d(t,"SelectMultipleView",(function(){return Lt})),n.d(t,"SelectionRangeSliderModel",(function(){return Dt})),n.d(t,"SelectionRangeSliderView",(function(){return Pt})),n.d(t,"SelectionContainerModel",(function(){return Wt})),n.d(t,"AccordionModel",(function(){return Gt})),n.d(t,"JupyterPhosphorAccordionWidget",(function(){return Xt})),n.d(t,"AccordionView",(function(){return qt})),n.d(t,"TabModel",(function(){return Zt})),n.d(t,"JupyterPhosphorTabPanelWidget",(function(){return Qt})),n.d(t,"TabView",(function(){return Kt})),n.d(t,"StringModel",(function(){return en})),n.d(t,"HTMLModel",(function(){return tn})),n.d(t,"HTMLView",(function(){return nn})),n.d(t,"HTMLMathModel",(function(){return rn})),n.d(t,"HTMLMathView",(function(){return on})),n.d(t,"LabelModel",(function(){return an})),n.d(t,"LabelView",(function(){return sn})),n.d(t,"TextareaModel",(function(){return ln})),n.d(t,"TextareaView",(function(){return un})),n.d(t,"TextModel",(function(){return cn})),n.d(t,"TextView",(function(){return dn})),n.d(t,"PasswordModel",(function(){return hn})),n.d(t,"PasswordView",(function(){return fn})),n.d(t,"ComboboxModel",(function(){return pn})),n.d(t,"ComboboxView",(function(){return gn})),n.d(t,"DescriptionStyleModel",(function(){return c})),n.d(t,"DescriptionModel",(function(){return d})),n.d(t,"DescriptionView",(function(){return h})),n.d(t,"LabeledDOMWidgetModel",(function(){return f})),n.d(t,"LabeledDOMWidgetView",(function(){return p})),n.d(t,"FileUploadModel",(function(){return bn})),n.d(t,"FileUploadView",(function(){return Cn}));var An=n(1429).version},function(e,t,n){"use strict";var r=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),i=this&&this.__exportStar||function(e,t){for(var n in e)"default"===n||Object.prototype.hasOwnProperty.call(t,n)||r(t,e,n)};Object.defineProperty(t,"__esModule",{value:!0}),i(n(1373),t),i(n(1374),t),i(n(1375),t),i(n(1376),t),i(n(1377),t)},function(e,t,n){"use strict";n.d(t,"b",(function(){return u}));var r=n(74),i=n(1),o=n(140),a=(n(3),n(204)),s={xs:0,sm:600,md:960,lg:1280,xl:1920},l={keys:["xs","sm","md","lg","xl"],up:function(e){return"@media (min-width:".concat(s[e],"px)")}};function u(e,t,n){if(Array.isArray(t)){var r=e.theme.breakpoints||l;return t.reduce((function(e,i,o){return e[r.up(r.keys[o])]=n(t[o]),e}),{})}if("object"===Object(o.a)(t)){var i=e.theme.breakpoints||l;return Object.keys(t).reduce((function(e,r){return e[i.up(r)]=n(t[r]),e}),{})}return n(t)}t.a=function(e){var t=function(t){var n=e(t),r=t.theme.breakpoints||l,o=r.keys.reduce((function(n,o){return t[o]&&((n=n||{})[r.up(o)]=e(Object(i.a)({theme:t.theme},t[o]))),n}),null);return Object(a.a)(n,o)};return t.propTypes={},t.filterProps=["xs","sm","md","lg","xl"].concat(Object(r.a)(e.filterProps)),t}},function(e,t,n){"use strict";var r=n(21),i=n(5),o=n(442),a=n(155),s=n(1);function l(e,t,n){var i;return Object(s.a)({gutters:function(){var n=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};return Object(s.a)({paddingLeft:t(2),paddingRight:t(2)},n,Object(r.a)({},e.up("sm"),Object(s.a)({paddingLeft:t(3),paddingRight:t(3)},n[e.up("sm")])))},toolbar:(i={minHeight:56},Object(r.a)(i,"".concat(e.up("xs")," and (orientation: landscape)"),{minHeight:48}),Object(r.a)(i,e.up("sm"),{minHeight:64}),i)},n)}var u=n(334),c=n(225),d=n(272),h=n(268),f=n(267),p=n(266),g=n(271),v=n(269),m=n(270),y=n(26),b={text:{primary:"rgba(0, 0, 0, 0.87)",secondary:"rgba(0, 0, 0, 0.54)",disabled:"rgba(0, 0, 0, 0.38)",hint:"rgba(0, 0, 0, 0.38)"},divider:"rgba(0, 0, 0, 0.12)",background:{paper:c.a.white,default:d.a[50]},action:{active:"rgba(0, 0, 0, 0.54)",hover:"rgba(0, 0, 0, 0.04)",hoverOpacity:.04,selected:"rgba(0, 0, 0, 0.08)",selectedOpacity:.08,disabled:"rgba(0, 0, 0, 0.26)",disabledBackground:"rgba(0, 0, 0, 0.12)",disabledOpacity:.38,focus:"rgba(0, 0, 0, 0.12)",focusOpacity:.12,activatedOpacity:.12}},C={text:{primary:c.a.white,secondary:"rgba(255, 255, 255, 0.7)",disabled:"rgba(255, 255, 255, 0.5)",hint:"rgba(255, 255, 255, 0.5)",icon:"rgba(255, 255, 255, 0.5)"},divider:"rgba(255, 255, 255, 0.12)",background:{paper:d.a[800],default:"#303030"},action:{active:c.a.white,hover:"rgba(255, 255, 255, 0.08)",hoverOpacity:.08,selected:"rgba(255, 255, 255, 0.16)",selectedOpacity:.16,disabled:"rgba(255, 255, 255, 0.3)",disabledBackground:"rgba(255, 255, 255, 0.12)",disabledOpacity:.38,focus:"rgba(255, 255, 255, 0.12)",focusOpacity:.12,activatedOpacity:.24}};function A(e,t,n,r){var i=r.light||r,o=r.dark||1.5*r;e[t]||(e.hasOwnProperty(n)?e[t]=e[n]:"light"===t?e.light=Object(y.i)(e.main,i):"dark"===t&&(e.dark=Object(y.a)(e.main,o)))}function _(e){var t=e.primary,n=void 0===t?{light:h.a[300],main:h.a[500],dark:h.a[700]}:t,r=e.secondary,a=void 0===r?{light:f.a.A200,main:f.a.A400,dark:f.a.A700}:r,l=e.error,_=void 0===l?{light:p.a[300],main:p.a[500],dark:p.a[700]}:l,I=e.warning,x=void 0===I?{light:g.a[300],main:g.a[500],dark:g.a[700]}:I,w=e.info,M=void 0===w?{light:v.a[300],main:v.a[500],dark:v.a[700]}:w,S=e.success,T=void 0===S?{light:m.a[300],main:m.a[500],dark:m.a[700]}:S,E=e.type,O=void 0===E?"light":E,k=e.contrastThreshold,L=void 0===k?3:k,D=e.tonalOffset,P=void 0===D?.2:D,N=Object(i.a)(e,["primary","secondary","error","warning","info","success","type","contrastThreshold","tonalOffset"]);function j(e){return Object(y.e)(e,C.text.primary)>=L?C.text.primary:b.text.primary}var R=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:500,n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:300,r=arguments.length>3&&void 0!==arguments[3]?arguments[3]:700;if(!(e=Object(s.a)({},e)).main&&e[t]&&(e.main=e[t]),!e.main)throw new Error(Object(u.a)(4,t));if("string"!=typeof e.main)throw new Error(Object(u.a)(5,JSON.stringify(e.main)));return A(e,"light",n,P),A(e,"dark",r,P),e.contrastText||(e.contrastText=j(e.main)),e},z={dark:C,light:b};return Object(o.a)(Object(s.a)({common:c.a,type:O,primary:R(n),secondary:R(a,"A400","A200","A700"),error:R(_),warning:R(x),info:R(M),success:R(T),grey:d.a,contrastThreshold:L,getContrastText:j,augmentColor:R,tonalOffset:P},z[O]),N)}function I(e){return Math.round(1e5*e)/1e5}var x={textTransform:"uppercase"};function w(e,t){var n="function"==typeof t?t(e):t,r=n.fontFamily,a=void 0===r?'"Roboto", "Helvetica", "Arial", sans-serif':r,l=n.fontSize,u=void 0===l?14:l,c=n.fontWeightLight,d=void 0===c?300:c,h=n.fontWeightRegular,f=void 0===h?400:h,p=n.fontWeightMedium,g=void 0===p?500:p,v=n.fontWeightBold,m=void 0===v?700:v,y=n.htmlFontSize,b=void 0===y?16:y,C=n.allVariants,A=n.pxToRem,_=Object(i.a)(n,["fontFamily","fontSize","fontWeightLight","fontWeightRegular","fontWeightMedium","fontWeightBold","htmlFontSize","allVariants","pxToRem"]);var w=u/14,M=A||function(e){return"".concat(e/b*w,"rem")},S=function(e,t,n,r,i){return Object(s.a)({fontFamily:a,fontWeight:e,fontSize:M(t),lineHeight:n},'"Roboto", "Helvetica", "Arial", sans-serif'===a?{letterSpacing:"".concat(I(r/t),"em")}:{},i,C)},T={h1:S(d,96,1.167,-1.5),h2:S(d,60,1.2,-.5),h3:S(f,48,1.167,0),h4:S(f,34,1.235,.25),h5:S(f,24,1.334,0),h6:S(g,20,1.6,.15),subtitle1:S(f,16,1.75,.15),subtitle2:S(g,14,1.57,.1),body1:S(f,16,1.5,.15),body2:S(f,14,1.43,.15),button:S(g,14,1.75,.4,x),caption:S(f,12,1.66,.4),overline:S(f,12,2.66,1,x)};return Object(o.a)(Object(s.a)({htmlFontSize:b,pxToRem:M,round:I,fontFamily:a,fontSize:u,fontWeightLight:d,fontWeightRegular:f,fontWeightMedium:g,fontWeightBold:m},T),_,{clone:!1})}function M(){return["".concat(arguments.length<=0?void 0:arguments[0],"px ").concat(arguments.length<=1?void 0:arguments[1],"px ").concat(arguments.length<=2?void 0:arguments[2],"px ").concat(arguments.length<=3?void 0:arguments[3],"px rgba(0,0,0,").concat(.2,")"),"".concat(arguments.length<=4?void 0:arguments[4],"px ").concat(arguments.length<=5?void 0:arguments[5],"px ").concat(arguments.length<=6?void 0:arguments[6],"px ").concat(arguments.length<=7?void 0:arguments[7],"px rgba(0,0,0,").concat(.14,")"),"".concat(arguments.length<=8?void 0:arguments[8],"px ").concat(arguments.length<=9?void 0:arguments[9],"px ").concat(arguments.length<=10?void 0:arguments[10],"px ").concat(arguments.length<=11?void 0:arguments[11],"px rgba(0,0,0,").concat(.12,")")].join(",")}var S=["none",M(0,2,1,-1,0,1,1,0,0,1,3,0),M(0,3,1,-2,0,2,2,0,0,1,5,0),M(0,3,3,-2,0,3,4,0,0,1,8,0),M(0,2,4,-1,0,4,5,0,0,1,10,0),M(0,3,5,-1,0,5,8,0,0,1,14,0),M(0,3,5,-1,0,6,10,0,0,1,18,0),M(0,4,5,-2,0,7,10,1,0,2,16,1),M(0,5,5,-3,0,8,10,1,0,3,14,2),M(0,5,6,-3,0,9,12,1,0,3,16,2),M(0,6,6,-3,0,10,14,1,0,4,18,3),M(0,6,7,-4,0,11,15,1,0,4,20,3),M(0,7,8,-4,0,12,17,2,0,5,22,4),M(0,7,8,-4,0,13,19,2,0,5,24,4),M(0,7,9,-4,0,14,21,2,0,5,26,4),M(0,8,9,-5,0,15,22,2,0,6,28,5),M(0,8,10,-5,0,16,24,2,0,6,30,5),M(0,8,11,-5,0,17,26,2,0,6,32,5),M(0,9,11,-5,0,18,28,2,0,7,34,6),M(0,9,12,-6,0,19,29,2,0,7,36,6),M(0,10,13,-6,0,20,31,3,0,8,38,7),M(0,10,13,-6,0,21,33,3,0,8,40,7),M(0,10,14,-6,0,22,35,3,0,8,42,7),M(0,11,14,-7,0,23,36,3,0,9,44,8),M(0,11,15,-7,0,24,38,3,0,9,46,8)],T={borderRadius:4},E=n(326);function O(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:8;if(e.mui)return e;var t=Object(E.a)({spacing:e}),n=function(){for(var e=arguments.length,n=new Array(e),r=0;r<e;r++)n[r]=arguments[r];return 0===n.length?t(1):1===n.length?t(n[0]):n.map((function(e){if("string"==typeof e)return e;var n=t(e);return"number"==typeof n?"".concat(n,"px"):n})).join(" ")};return Object.defineProperty(n,"unit",{get:function(){return e}}),n.mui=!0,n}var k=n(65),L=n(446);t.a=function(){for(var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},t=e.breakpoints,n=void 0===t?{}:t,r=e.mixins,s=void 0===r?{}:r,u=e.palette,c=void 0===u?{}:u,d=e.spacing,h=e.typography,f=void 0===h?{}:h,p=Object(i.a)(e,["breakpoints","mixins","palette","spacing","typography"]),g=_(c),v=Object(a.a)(n),m=O(d),y=Object(o.a)({breakpoints:v,direction:"ltr",mixins:l(v,m,s),overrides:{},palette:g,props:{},shadows:S,typography:w(g,f),spacing:m,shape:T,transitions:k.a,zIndex:L.a},p),b=arguments.length,C=new Array(b>1?b-1:0),A=1;A<b;A++)C[A-1]=arguments[A];return y=C.reduce((function(e,t){return Object(o.a)(e,t)}),y)}},function(e,t,n){"use strict";t.a={50:"#ffebee",100:"#ffcdd2",200:"#ef9a9a",300:"#e57373",400:"#ef5350",500:"#f44336",600:"#e53935",700:"#d32f2f",800:"#c62828",900:"#b71c1c",A100:"#ff8a80",A200:"#ff5252",A400:"#ff1744",A700:"#d50000"}},function(e,t,n){"use strict";t.a={50:"#fce4ec",100:"#f8bbd0",200:"#f48fb1",300:"#f06292",400:"#ec407a",500:"#e91e63",600:"#d81b60",700:"#c2185b",800:"#ad1457",900:"#880e4f",A100:"#ff80ab",A200:"#ff4081",A400:"#f50057",A700:"#c51162"}},function(e,t,n){"use strict";t.a={50:"#e8eaf6",100:"#c5cae9",200:"#9fa8da",300:"#7986cb",400:"#5c6bc0",500:"#3f51b5",600:"#3949ab",700:"#303f9f",800:"#283593",900:"#1a237e",A100:"#8c9eff",A200:"#536dfe",A400:"#3d5afe",A700:"#304ffe"}},function(e,t,n){"use strict";t.a={50:"#e3f2fd",100:"#bbdefb",200:"#90caf9",300:"#64b5f6",400:"#42a5f5",500:"#2196f3",600:"#1e88e5",700:"#1976d2",800:"#1565c0",900:"#0d47a1",A100:"#82b1ff",A200:"#448aff",A400:"#2979ff",A700:"#2962ff"}},function(e,t,n){"use strict";t.a={50:"#e8f5e9",100:"#c8e6c9",200:"#a5d6a7",300:"#81c784",400:"#66bb6a",500:"#4caf50",600:"#43a047",700:"#388e3c",800:"#2e7d32",900:"#1b5e20",A100:"#b9f6ca",A200:"#69f0ae",A400:"#00e676",A700:"#00c853"}},function(e,t,n){"use strict";t.a={50:"#fff3e0",100:"#ffe0b2",200:"#ffcc80",300:"#ffb74d",400:"#ffa726",500:"#ff9800",600:"#fb8c00",700:"#f57c00",800:"#ef6c00",900:"#e65100",A100:"#ffd180",A200:"#ffab40",A400:"#ff9100",A700:"#ff6d00"}},function(e,t,n){"use strict";t.a={50:"#fafafa",100:"#f5f5f5",200:"#eeeeee",300:"#e0e0e0",400:"#bdbdbd",500:"#9e9e9e",600:"#757575",700:"#616161",800:"#424242",900:"#212121",A100:"#d5d5d5",A200:"#aaaaaa",A400:"#303030",A700:"#616161"}},function(e,t,n){"use strict";n.d(t,"a",(function(){return i}));var r=n(335);function i(e,t){if(e){if("string"==typeof e)return Object(r.a)(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?Object(r.a)(e,t):void 0}}},function(e,t,n){"use strict";var r=n(1),i=n(49),o=n(5),a=n(0),s=(n(3),n(4)),l=n(100),u=n(106),c=n(7),d=n(452),h=a.forwardRef((function(e,t){var n=e.autoFocus,c=e.checked,h=e.checkedIcon,f=e.classes,p=e.className,g=e.defaultChecked,v=e.disabled,m=e.icon,y=e.id,b=e.inputProps,C=e.inputRef,A=e.name,_=e.onBlur,I=e.onChange,x=e.onFocus,w=e.readOnly,M=e.required,S=e.tabIndex,T=e.type,E=e.value,O=Object(o.a)(e,["autoFocus","checked","checkedIcon","classes","className","defaultChecked","disabled","icon","id","inputProps","inputRef","name","onBlur","onChange","onFocus","readOnly","required","tabIndex","type","value"]),k=Object(l.a)({controlled:c,default:Boolean(g),name:"SwitchBase",state:"checked"}),L=Object(i.a)(k,2),D=L[0],P=L[1],N=Object(u.a)(),j=v;N&&void 0===j&&(j=N.disabled);var R="checkbox"===T||"radio"===T;return a.createElement(d.a,Object(r.a)({component:"span",className:Object(s.a)(f.root,p,D&&f.checked,j&&f.disabled),disabled:j,tabIndex:null,role:void 0,onFocus:function(e){x&&x(e),N&&N.onFocus&&N.onFocus(e)},onBlur:function(e){_&&_(e),N&&N.onBlur&&N.onBlur(e)},ref:t},O),a.createElement("input",Object(r.a)({autoFocus:n,checked:c,defaultChecked:g,className:f.input,disabled:j,id:R&&y,name:A,onChange:function(e){var t=e.target.checked;P(t),I&&I(e,t)},readOnly:w,ref:C,required:M,tabIndex:S,type:T,value:E},b)),D?h:m)}));t.a=Object(c.a)({root:{padding:9},checked:{},disabled:{},input:{cursor:"inherit",position:"absolute",opacity:0,width:"100%",height:"100%",top:0,left:0,margin:0,padding:0,zIndex:1}},{name:"PrivateSwitchBase"})(h)},function(e,t,n){"use strict";var r=n(0),i=n.n(r);t.a=i.a.createContext(null)},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){(0,i.default)(2,arguments);var n=(0,r.default)(e),o=(0,r.default)(t);return n.getTime()<o.getTime()};var r=o(n(47)),i=o(n(40));function o(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";n.d(t,"a",(function(){return a}));var r=n(0),i=n.n(r),o=n(448);function a(){return i.a.useContext(o.a)}},,,,function(e,t,n){var r=n(208),i=n(130);e.exports=function(e){if(!i(e))return!1;var t=r(e);return"[object Function]"==t||"[object GeneratorFunction]"==t||"[object AsyncFunction]"==t||"[object Proxy]"==t}},function(e,t,n){var r=n(1418),i=n(158),o=Object.prototype,a=o.hasOwnProperty,s=o.propertyIsEnumerable,l=r(function(){return arguments}())?r:function(e){return i(e)&&a.call(e,"callee")&&!s.call(e,"callee")};e.exports=l},function(e,t,n){var r=n(1420),i=n(362),o=n(498),a=o&&o.isTypedArray,s=a?i(a):r;e.exports=s},function(e,t,n){var r,i,o;i=[n(71)],void 0===(o="function"==typeof(r=function(e){return e.ui=e.ui||{},e.ui.version="1.12.1"})?r.apply(t,i):r)||(e.exports=o)},function(e,t,n){"use strict";var r=n(881),i="function"==typeof Symbol&&"symbol"==typeof Symbol("foo"),o=Object.prototype.toString,a=Array.prototype.concat,s=Object.defineProperty,l=s&&function(){var e={};try{for(var t in s(e,"x",{enumerable:!1,value:e}),e)return!1;return e.x===e}catch(e){return!1}}(),u=function(e,t,n,r){var i;(!(t in e)||"function"==typeof(i=r)&&"[object Function]"===o.call(i)&&r())&&(l?s(e,t,{configurable:!0,enumerable:!1,value:n,writable:!0}):e[t]=n)},c=function(e,t){var n=arguments.length>2?arguments[2]:{},o=r(t);i&&(o=a.call(o,Object.getOwnPropertySymbols(t)));for(var s=0;s<o.length;s+=1)u(e,o[s],t[o[s]],n[o[s]])};c.supportsDescriptors=!!l,e.exports=c},function(e,t,n){"use strict";function r(e,t,n,r,i){return null}n.d(t,"a",(function(){return r}))},function(e,t,n){"use strict";var r=n(369);n.d(t,"a",(function(){return r.b})),n.d(t,"b",(function(){return r.c}))},function(e,t,n){"use strict";n.r(t);var r=n(436);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=function(){function e(e,t,n,r){this.name=e,this.modelName=t,this.defaultValue=n,this.alwaysWriteJson=r,this.type=void 0,this.values=[],this.from=-99999999,this.to=99999999}return e.prototype.setType=function(e){return this.type=e,this},e.prototype.setValues=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return this.values=e,this},e.prototype.setFrom=function(e){return this.from=e,this},e.prototype.setTo=function(e){return this.to=e,this},e.ENUM="Enum",e.INT="Int",e.NUMBER="Number",e.STRING="String",e.BOOLEAN="Boolean",e.ID="Id",e.JSON="Json",e}();t.default=r},function(e,t,n){"use strict";var r,i=this&&this.__extends||(r=function(e,t){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(e[n]=t[n])})(e,t)},function(e,t){function n(){this.constructor=e}r(e,t),e.prototype=null===t?Object.create(t):(n.prototype=t.prototype,new n)});Object.defineProperty(t,"__esModule",{value:!0});var o=n(289),a=n(244),s=function(e){function t(n,r,i){void 0===i&&(i=!0);var o=e.call(this,n)||this;return o._extra={},t._attributeDefinitions.fromJson(r,o._attributes),!0===i&&n._addNode(o),o}return i(t,e),t._fromJson=function(e,n,r){return void 0===r&&(r=!0),new t(n,e,r)},t._createAttributeDefinitions=function(){var e=new a.default;return e.add("type",t.TYPE,!0),e.add("id",void 0).setType(o.default.ID),e.add("name","[Unnamed Tab]").setType(o.default.STRING),e.add("component",void 0).setType(o.default.STRING),e.add("config",void 0).setType(o.default.JSON),e.add("floating",!1).setType(o.default.BOOLEAN),e.addInherited("enableClose","tabEnableClose").setType(o.default.BOOLEAN),e.addInherited("closeType","tabCloseType").setType(o.default.INT),e.addInherited("enableDrag","tabEnableDrag").setType(o.default.BOOLEAN),e.addInherited("enableRename","tabEnableRename").setType(o.default.BOOLEAN),e.addInherited("className","tabClassName").setType(o.default.STRING),e.addInherited("icon","tabIcon").setType(o.default.STRING),e.addInherited("enableRenderOnDemand","tabEnableRenderOnDemand").setType(o.default.BOOLEAN),e.addInherited("enableFloat","tabEnableFloat").setType(o.default.BOOLEAN),e},t.prototype.getWindow=function(){return this._window},t.prototype.getTabRect=function(){return this._tabRect},t.prototype._setTabRect=function(e){this._tabRect=e},t.prototype._setRenderedName=function(e){this._renderedName=e},t.prototype._getRenderedName=function(){return this._renderedName},t.prototype.getName=function(){return this._getAttr("name")},t.prototype.getComponent=function(){return this._getAttributeAsStringOrUndefined("component")},t.prototype.getConfig=function(){return this._attributes.config},t.prototype.getExtraData=function(){return this._extra},t.prototype.isFloating=function(){return this._getAttr("floating")},t.prototype.getIcon=function(){return this._getAttributeAsStringOrUndefined("icon")},t.prototype.isEnableClose=function(){return this._getAttr("enableClose")},t.prototype.getCloseType=function(){return this._getAttr("closeType")},t.prototype.isEnableFloat=function(){return this._getAttr("enableFloat")},t.prototype.isEnableDrag=function(){return this._getAttr("enableDrag")},t.prototype.isEnableRename=function(){return this._getAttr("enableRename")},t.prototype.getClassName=function(){return this._getAttributeAsStringOrUndefined("className")},t.prototype.isEnableRenderOnDemand=function(){return this._getAttr("enableRenderOnDemand")},t.prototype._setName=function(e){this._attributes.name=e,this._window&&this._window.document&&(this._window.document.title=e)},t.prototype._setFloating=function(e){this._attributes.floating=e},t.prototype._layout=function(e,t){e.equals(this._rect)||this._fireEvent("resize",{rect:e}),this._rect=e},t.prototype._delete=function(){this._parent._remove(this),this._fireEvent("close",{})},t.prototype._toJson=function(){var e={};return t._attributeDefinitions.toJson(e,this._attributes),e},t.prototype._updateAttrs=function(e){t._attributeDefinitions.update(e,this._attributes)},t.prototype._getAttributeDefinitions=function(){return t._attributeDefinitions},t.prototype._setWindow=function(e){this._window=e},t.TYPE="tab",t._attributeDefinitions=t._createAttributeDefinitions(),t}(n(245).default);t.default=s},function(e,t,n){"use strict";n.r(t);var r=n(631);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";e.exports=function(e){return window&&window.process&&window.process.versions?"[object Object]"===Object.prototype.toString.call(e):"[object Object]"===Object.prototype.toString.call(e)&&Object.getPrototypeOf(e)===Object.prototype}},function(e,t,n){"use strict";e.exports={mod:function(e,t){var n=e%t;return n<0?n+t:n},modHalf:function(e,t){return Math.abs(e)>t/2?e-Math.round(e/t)*t:e}}},function(e,t,n){"use strict";t.counter=function(e,t,n,r){var i=(t||"")+(n?"":"$"),o=!1===r?"":"^";return"xy"===e?new RegExp(o+"x([2-9]|[1-9][0-9]+)?y([2-9]|[1-9][0-9]+)?"+i):new RegExp(o+e+"([2-9]|[1-9][0-9]+)?"+i)}},function(e,t,n){"use strict";var r=n(94),i=n(382),o=n(133),a=n(1663),s=n(537),l=n(63).extendFlat,u=r({editType:"calc"});u.family.dflt='"Open Sans", verdana, arial, sans-serif',u.size.dflt=12,u.color.dflt=o.defaultLine,e.exports={font:u,title:{text:{valType:"string",editType:"layoutstyle"},font:r({editType:"layoutstyle"}),xref:{valType:"enumerated",dflt:"container",values:["container","paper"],editType:"layoutstyle"},yref:{valType:"enumerated",dflt:"container",values:["container","paper"],editType:"layoutstyle"},x:{valType:"number",min:0,max:1,dflt:.5,editType:"layoutstyle"},y:{valType:"number",min:0,max:1,dflt:"auto",editType:"layoutstyle"},xanchor:{valType:"enumerated",dflt:"auto",values:["auto","left","center","right"],editType:"layoutstyle"},yanchor:{valType:"enumerated",dflt:"auto",values:["auto","top","middle","bottom"],editType:"layoutstyle"},pad:l(s({editType:"layoutstyle"}),{}),editType:"layoutstyle"},uniformtext:{mode:{valType:"enumerated",values:[!1,"hide","show"],dflt:!1,editType:"plot"},minsize:{valType:"number",min:0,dflt:0,editType:"plot"},editType:"plot"},autosize:{valType:"boolean",dflt:!1,editType:"none"},width:{valType:"number",min:10,dflt:700,editType:"plot"},height:{valType:"number",min:10,dflt:450,editType:"plot"},margin:{l:{valType:"number",min:0,dflt:80,editType:"plot"},r:{valType:"number",min:0,dflt:80,editType:"plot"},t:{valType:"number",min:0,dflt:100,editType:"plot"},b:{valType:"number",min:0,dflt:80,editType:"plot"},pad:{valType:"number",min:0,dflt:0,editType:"plot"},autoexpand:{valType:"boolean",dflt:!0,editType:"plot"},editType:"plot"},computed:{valType:"any",editType:"none"},paper_bgcolor:{valType:"color",dflt:o.background,editType:"plot"},plot_bgcolor:{valType:"color",dflt:o.background,editType:"layoutstyle"},autotypenumbers:{valType:"enumerated",values:["convert types","strict"],dflt:"convert types",editType:"calc"},separators:{valType:"string",editType:"plot"},hidesources:{valType:"boolean",dflt:!1,editType:"plot"},showlegend:{valType:"boolean",editType:"legend"},colorway:{valType:"colorlist",dflt:o.defaults,editType:"calc"},datarevision:{valType:"any",editType:"calc"},uirevision:{valType:"any",editType:"none"},editrevision:{valType:"any",editType:"none"},selectionrevision:{valType:"any",editType:"none"},template:{valType:"any",editType:"calc"},modebar:{orientation:{valType:"enumerated",values:["v","h"],dflt:"h",editType:"modebar"},bgcolor:{valType:"color",editType:"modebar"},color:{valType:"color",editType:"modebar"},activecolor:{valType:"color",editType:"modebar"},uirevision:{valType:"any",editType:"none"},editType:"modebar"},newshape:a.newshape,activeshape:a.activeshape,meta:{valType:"any",arrayOk:!0,editType:"plot"},transition:l({},i.transition,{editType:"none"}),_deprecated:{title:{valType:"string",editType:"layoutstyle"},titlefont:r({editType:"layoutstyle"})}}},function(e,t,n){"use strict";t.dash={valType:"string",values:["solid","dot","dash","longdash","dashdot","longdashdot"],dflt:"solid",editType:"style"}},function(e,t,n){"use strict";var r=n(27),i=n(10),o=n(160),a=n(295),s=n(983),l=n(382),u=n(253).configAttributes,c=n(1678),d=n(1682),h=n(124),f=i.extendFlat,p=i.extendDeepAll,g=i.isPlainObject,v=i.isArrayOrTypedArray,m=i.nestedProperty,y=i.valObjectMeta,b=["_isSubplotObj","_isLinkedToArray","_arrayAttrRegexps","_deprecated"];function C(e,t,n){if(!e)return!1;if(e._isLinkedToArray)if(A(t[n]))n++;else if(n<t.length)return!1;for(;n<t.length;n++){var r=e[t[n]];if(!g(r))break;if(e=r,n===t.length-1)break;if(e._isLinkedToArray){if(!A(t[++n]))return!1}else if("info_array"===e.valType){var i=t[++n];if(!A(i))return!1;var o=e.items;if(Array.isArray(o)){if(i>=o.length)return!1;if(2===e.dimensions){if(n++,t.length===n)return e;var a=t[n];if(!A(a))return!1;e=o[i][a]}else e=o[i]}else e=o}}return e}function A(e){return e===Math.round(e)&&e>=0}function _(){var e,t,n={};for(e in p(n,a),r.subplotsRegistry){if((t=r.subplotsRegistry[e]).layoutAttributes)if(Array.isArray(t.attr))for(var i=0;i<t.attr.length;i++)w(n,t,t.attr[i]);else w(n,t,"subplot"===t.attr?t.name:t.attr)}for(e in n=function(e){return f(e,{radialaxis:d.radialaxis,angularaxis:d.angularaxis}),f(e,d.layout),e}(n),r.componentsRegistry){var o=(t=r.componentsRegistry[e]).schema;if(o&&(o.subplots||o.layout)){var s=o.subplots;if(s&&s.xaxis&&!s.yaxis)for(var l in s.xaxis)delete n.yaxis[l]}else"colorscale"===t.name?p(n,t.layoutAttributes):t.layoutAttributes&&M(n,t.layoutAttributes,t.name)}return{layoutAttributes:x(n)}}function I(){var e={frames:p({},s)};return x(e),e.frames}function x(e){return function(e){t.crawl(e,(function(e,n,r){t.isValObject(e)?"data_array"===e.valType?(e.role="data",r[n+"src"]={valType:"string",editType:"none"}):!0===e.arrayOk&&(r[n+"src"]={valType:"string",editType:"none"}):g(e)&&(e.role="object")}))}(e),function(e){t.crawl(e,(function(e,t,n){if(e){var r=e._isLinkedToArray;r&&(delete e._isLinkedToArray,n[t]={items:{}},n[t].items[r]=e,n[t].role="object")}}))}(e),function(e){!function e(t){for(var n in t)if(g(t[n]))e(t[n]);else if(Array.isArray(t[n]))for(var r=0;r<t[n].length;r++)e(t[n][r]);else t[n]instanceof RegExp&&(t[n]=t[n].toString())}(e)}(e),e}function w(e,t,n){var r=m(e,n),i=p({},t.layoutAttributes);i._isSubplotObj=!0,r.set(i)}function M(e,t,n){var r=m(e,n);r.set(p(r.get()||{},t))}t.IS_SUBPLOT_OBJ="_isSubplotObj",t.IS_LINKED_TO_ARRAY="_isLinkedToArray",t.DEPRECATED="_deprecated",t.UNDERSCORE_ATTRS=b,t.get=function(){var e={};r.allTypes.concat("area").forEach((function(n){e[n]=function(e){var n,i;"area"===e?(n={attributes:c},i={}):(n=r.modules[e]._module,i=n.basePlotModule);var a={type:null},s=p({},o),l=p({},n.attributes);t.crawl(l,(function(e,t,n,r,i){m(s,i).set(void 0),void 0===e&&m(l,i).set(void 0)})),p(a,s),r.traceIs(e,"noOpacity")&&delete a.opacity;r.traceIs(e,"showLegend")||(delete a.showlegend,delete a.legendgroup);r.traceIs(e,"noHover")&&(delete a.hoverinfo,delete a.hoverlabel);n.selectPoints||delete a.selectedpoints;p(a,l),i.attributes&&p(a,i.attributes);a.type=e;var u={meta:n.meta||{},categories:n.categories||{},animatable:Boolean(n.animatable),type:e,attributes:x(a)};if(n.layoutAttributes){var d={};p(d,n.layoutAttributes),u.layoutAttributes=x(d)}n.animatable||t.crawl(u,(function(e){t.isValObject(e)&&"anim"in e&&delete e.anim}));return u}(n)}));var n={};return Object.keys(r.transformsRegistry).forEach((function(e){n[e]=function(e){var t=r.transformsRegistry[e],n=p({},t.attributes);return Object.keys(r.componentsRegistry).forEach((function(t){var i=r.componentsRegistry[t];i.schema&&i.schema.transforms&&i.schema.transforms[e]&&Object.keys(i.schema.transforms[e]).forEach((function(t){M(n,i.schema.transforms[e][t],t)}))})),{attributes:x(n)}}(e)})),{defs:{valObjects:y,metaKeys:b.concat(["description","role","editType","impliedEdits"]),editType:{traces:h.traces,layout:h.layout},impliedEdits:{}},traces:e,layout:_(),transforms:n,frames:I(),animation:x(l),config:x(u)}},t.crawl=function(e,n,r,i){var o=r||0;i=i||"",Object.keys(e).forEach((function(r){var a=e[r];if(-1===b.indexOf(r)){var s=(i?i+".":"")+r;n(a,r,e,o,s),t.isValObject(a)||g(a)&&"impliedEdits"!==r&&t.crawl(a,n,o+1,s)}}))},t.isValObject=function(e){return e&&void 0!==e.valType},t.findArrayAttributes=function(e){var n,r,i=[],a=[],s=[];function l(e,t,o,l){a=a.slice(0,l).concat([t]),s=s.slice(0,l).concat([e&&e._isLinkedToArray]),e&&("data_array"===e.valType||!0===e.arrayOk)&&!("colorbar"===a[l-1]&&("ticktext"===t||"tickvals"===t))&&function e(t,n,o){var l=t[a[n]],u=o+a[n];if(n===a.length-1)v(l)&&i.push(r+u);else if(s[n]){if(Array.isArray(l))for(var c=0;c<l.length;c++)g(l[c])&&e(l[c],n+1,u+"["+c+"].")}else g(l)&&e(l,n+1,u+".")}(n,0,"")}n=e,r="",t.crawl(o,l),e._module&&e._module.attributes&&t.crawl(e._module.attributes,l);var u=e.transforms;if(u)for(var c=0;c<u.length;c++){var d=u[c],h=d._module;h&&(r="transforms["+c+"].",n=d,t.crawl(h.attributes,l))}return i},t.getTraceValObject=function(e,t){var n,i,a=t[0],s=1;if("transforms"===a){if(1===t.length)return o.transforms;var l=e.transforms;if(!Array.isArray(l)||!l.length)return!1;var u=t[1];if(!A(u)||u>=l.length)return!1;i=(n=(r.transformsRegistry[l[u].type]||{}).attributes)&&n[t[2]],s=3}else if("area"===e.type)i=c[a];else{var d=e._module;if(d||(d=(r.modules[e.type||o.type.dflt]||{})._module),!d)return!1;if(!(i=(n=d.attributes)&&n[a])){var h=d.basePlotModule;h&&h.attributes&&(i=h.attributes[a])}i||(i=o[a])}return C(i,t,s)},t.getLayoutValObject=function(e,t){return C(function(e,t){var n,i,o,s,l=e._basePlotModules;if(l){var u;for(n=0;n<l.length;n++){if((o=l[n]).attrRegex&&o.attrRegex.test(t)){if(o.layoutAttrOverrides)return o.layoutAttrOverrides;!u&&o.layoutAttributes&&(u=o.layoutAttributes)}var c=o.baseLayoutAttrOverrides;if(c&&t in c)return c[t]}if(u)return u}var h=e._modules;if(h)for(n=0;n<h.length;n++)if((s=h[n].layoutAttributes)&&t in s)return s[t];for(i in r.componentsRegistry){if("colorscale"===(o=r.componentsRegistry[i]).name&&0===t.indexOf("coloraxis"))return o.layoutAttributes[t];if(!o.schema&&t===o.name)return o.layoutAttributes}if(t in a)return a[t];if("radialaxis"===t||"angularaxis"===t)return d[t];return d.layout[t]||!1}(e,t[0]),t,1)}},function(e,t,n){"use strict";var r=n(42),i=n(10),o=n(149).extractOpts;e.exports=function(e,t,n){var a,s=e._fullLayout,l=n.vals,u=n.containerStr,c=u?i.nestedProperty(t,u).get():t,d=o(c),h=!1!==d.auto,f=d.min,p=d.max,g=d.mid,v=function(){return i.aggNums(Math.min,null,l)},m=function(){return i.aggNums(Math.max,null,l)};(void 0===f?f=v():h&&(f=c._colorAx&&r(f)?Math.min(f,v()):v()),void 0===p?p=m():h&&(p=c._colorAx&&r(p)?Math.max(p,m()):m()),h&&void 0!==g&&(p-g>g-f?f=g-(p-g):p-g<g-f&&(p=g+(g-f))),f===p&&(f-=.5,p+=.5),d._sync("min",f),d._sync("max",p),d.autocolorscale)&&(a=f*p<0?s.colorscale.diverging:f>=0?s.colorscale.sequential:s.colorscale.sequentialminus,d._sync("colorscale",a))}},function(e,t,n){"use strict";e.exports={clearOutlineControllers:function(e){var t=e._fullLayout._zoomlayer;t&&t.selectAll(".outline-controllers").remove()},clearSelect:function(e){var t=e._fullLayout._zoomlayer;t&&t.selectAll(".select-outline").remove(),e._fullLayout._drawing=!1}}},function(e,t,n){"use strict";var r=n(39),i=n(317).utcFormat,o=n(42),a=n(10),s=a.cleanNumber,l=a.ms2DateTime,u=a.dateTime2ms,c=a.ensureNumber,d=a.isArrayOrTypedArray,h=n(72),f=h.FP_SAFE,p=h.BADNUM,g=h.LOG_CLIP,v=h.ONEWEEK,m=h.ONEDAY,y=h.ONEHOUR,b=h.ONEMIN,C=h.ONESEC,A=n(83),_=n(95),I=_.HOUR_PATTERN,x=_.WEEKDAY_PATTERN;function w(e){return Math.pow(10,e)}function M(e){return null!=e}e.exports=function(e,t){t=t||{};var n=e._id||"x",h=n.charAt(0);function S(t,n){if(t>0)return Math.log(t)/Math.LN10;if(t<=0&&n&&e.range&&2===e.range.length){var r=e.range[0],i=e.range[1];return.5*(r+i-2*g*Math.abs(r-i))}return p}function T(t,n,r,i){if((i||{}).msUTC&&o(t))return+t;var s=u(t,r||e.calendar);if(s===p){if(!o(t))return p;t=+t;var l=Math.floor(10*a.mod(t+.05,1)),c=Math.round(t-l/10);s=u(new Date(c))+l/10}return s}function E(t,n,r){return l(t,n,r||e.calendar)}function O(t){return e._categories[Math.round(t)]}function k(t){if(M(t)){if(void 0===e._categoriesMap&&(e._categoriesMap={}),void 0!==e._categoriesMap[t])return e._categoriesMap[t];e._categories.push("number"==typeof t?String(t):t);var n=e._categories.length-1;return e._categoriesMap[t]=n,n}return p}function L(t){if(e._categoriesMap)return e._categoriesMap[t]}function D(e){var t=L(e);return void 0!==t?t:o(e)?+e:void 0}function P(e){return o(e)?+e:L(e)}function N(e,t,n){return r.round(n+t*e,2)}function j(e,t,n){return(e-n)/t}var R=function(t){return o(t)?N(t,e._m,e._b):p},z=function(t){return j(t,e._m,e._b)};if(e.rangebreaks){var F="y"===h;R=function(t){if(!o(t))return p;var n=e._rangebreaks.length;if(!n)return N(t,e._m,e._b);var r=F;e.range[0]>e.range[1]&&(r=!r);for(var i=r?-1:1,a=i*t,s=0,l=0;l<n;l++){var u=i*e._rangebreaks[l].min,c=i*e._rangebreaks[l].max;if(a<u)break;if(!(a>c)){s=a<(u+c)/2?l:l+1;break}s=l+1}var d=e._B[s]||0;return isFinite(d)?N(t,e._m2,d):0},z=function(t){var n=e._rangebreaks.length;if(!n)return j(t,e._m,e._b);for(var r=0,i=0;i<n&&!(t<e._rangebreaks[i].pmin);i++)t>e._rangebreaks[i].pmax&&(r=i+1);return j(t,e._m2,e._B[r])}}e.c2l="log"===e.type?S:c,e.l2c="log"===e.type?w:c,e.l2p=R,e.p2l=z,e.c2p="log"===e.type?function(e,t){return R(S(e,t))}:R,e.p2c="log"===e.type?function(e){return w(z(e))}:z,-1!==["linear","-"].indexOf(e.type)?(e.d2r=e.r2d=e.d2c=e.r2c=e.d2l=e.r2l=s,e.c2d=e.c2r=e.l2d=e.l2r=c,e.d2p=e.r2p=function(t){return e.l2p(s(t))},e.p2d=e.p2r=z,e.cleanPos=c):"log"===e.type?(e.d2r=e.d2l=function(e,t){return S(s(e),t)},e.r2d=e.r2c=function(e){return w(s(e))},e.d2c=e.r2l=s,e.c2d=e.l2r=c,e.c2r=S,e.l2d=w,e.d2p=function(t,n){return e.l2p(e.d2r(t,n))},e.p2d=function(e){return w(z(e))},e.r2p=function(t){return e.l2p(s(t))},e.p2r=z,e.cleanPos=c):"date"===e.type?(e.d2r=e.r2d=a.identity,e.d2c=e.r2c=e.d2l=e.r2l=T,e.c2d=e.c2r=e.l2d=e.l2r=E,e.d2p=e.r2p=function(t,n,r){return e.l2p(T(t,0,r))},e.p2d=e.p2r=function(e,t,n){return E(z(e),t,n)},e.cleanPos=function(t){return a.cleanDate(t,p,e.calendar)}):"category"===e.type?(e.d2c=e.d2l=k,e.r2d=e.c2d=e.l2d=O,e.d2r=e.d2l_noadd=D,e.r2c=function(t){var n=P(t);return void 0!==n?n:e.fraction2r(.5)},e.l2r=e.c2r=c,e.r2l=P,e.d2p=function(t){return e.l2p(e.r2c(t))},e.p2d=function(e){return O(z(e))},e.r2p=e.d2p,e.p2r=z,e.cleanPos=function(e){return"string"==typeof e&&""!==e?e:c(e)}):"multicategory"===e.type&&(e.r2d=e.c2d=e.l2d=O,e.d2r=e.d2l_noadd=D,e.r2c=function(t){var n=D(t);return void 0!==n?n:e.fraction2r(.5)},e.r2c_just_indices=L,e.l2r=e.c2r=c,e.r2l=D,e.d2p=function(t){return e.l2p(e.r2c(t))},e.p2d=function(e){return O(z(e))},e.r2p=e.d2p,e.p2r=z,e.cleanPos=function(e){return Array.isArray(e)||"string"==typeof e&&""!==e?e:c(e)},e.setupMultiCategory=function(r){var i,o,s=e._traceIndices,l=e._matchGroup;if(l&&0===e._categories.length)for(var u in l)if(u!==n){var c=t[A.id2name(u)];s=s.concat(c._traceIndices)}var f=[[0,{}],[0,{}]],p=[];for(i=0;i<s.length;i++){var g=r[s[i]];if(h in g){var v=g[h],m=g._length||a.minRowLength(v);if(d(v[0])&&d(v[1]))for(o=0;o<m;o++){var y=v[0][o],b=v[1][o];M(y)&&M(b)&&(p.push([y,b]),y in f[0][1]||(f[0][1][y]=f[0][0]++),b in f[1][1]||(f[1][1][b]=f[1][0]++))}}}for(p.sort((function(e,t){var n=f[0][1],r=n[e[0]]-n[t[0]];if(r)return r;var i=f[1][1];return i[e[1]]-i[t[1]]})),i=0;i<p.length;i++)k(p[i])}),e.fraction2r=function(t){var n=e.r2l(e.range[0]),r=e.r2l(e.range[1]);return e.l2r(n+t*(r-n))},e.r2fraction=function(t){var n=e.r2l(e.range[0]),r=e.r2l(e.range[1]);return(e.r2l(t)-n)/(r-n)},e.cleanRange=function(t,n){n||(n={}),t||(t="range");var r,i,s=a.nestedProperty(e,t).get();if(i=(i="date"===e.type?a.dfltRange(e.calendar):"y"===h?_.DFLTRANGEY:n.dfltRange||_.DFLTRANGEX).slice(),"tozero"!==e.rangemode&&"nonnegative"!==e.rangemode||(i[0]=0),s&&2===s.length)for("date"!==e.type||e.autorange||(s[0]=a.cleanDate(s[0],p,e.calendar),s[1]=a.cleanDate(s[1],p,e.calendar)),r=0;r<2;r++)if("date"===e.type){if(!a.isDateTime(s[r],e.calendar)){e[t]=i;break}if(e.r2l(s[0])===e.r2l(s[1])){var l=a.constrain(e.r2l(s[0]),a.MIN_MS+1e3,a.MAX_MS-1e3);s[0]=e.l2r(l-1e3),s[1]=e.l2r(l+1e3);break}}else{if(!o(s[r])){if(!o(s[1-r])){e[t]=i;break}s[r]=s[1-r]*(r?10:.1)}if(s[r]<-f?s[r]=-f:s[r]>f&&(s[r]=f),s[0]===s[1]){var u=Math.max(1,Math.abs(1e-6*s[0]));s[0]-=u,s[1]+=u}}else a.nestedProperty(e,t).set(i)},e.setScale=function(n){var r=t._size;if(e.overlaying){var i=A.getFromId({_fullLayout:t},e.overlaying);e.domain=i.domain}var o=n&&e._r?"_r":"range",a=e.calendar;e.cleanRange(o);var s,l,u=e.r2l(e[o][0],a),c=e.r2l(e[o][1],a),d="y"===h;if((d?(e._offset=r.t+(1-e.domain[1])*r.h,e._length=r.h*(e.domain[1]-e.domain[0]),e._m=e._length/(u-c),e._b=-e._m*c):(e._offset=r.l+e.domain[0]*r.w,e._length=r.w*(e.domain[1]-e.domain[0]),e._m=e._length/(c-u),e._b=-e._m*u),e._rangebreaks=[],e._lBreaks=0,e._m2=0,e._B=[],e.rangebreaks)&&(e._rangebreaks=e.locateBreaks(Math.min(u,c),Math.max(u,c)),e._rangebreaks.length)){for(s=0;s<e._rangebreaks.length;s++)l=e._rangebreaks[s],e._lBreaks+=Math.abs(l.max-l.min);var f=d;u>c&&(f=!f),f&&e._rangebreaks.reverse();var p=f?-1:1;for(e._m2=p*e._length/(Math.abs(c-u)-e._lBreaks),e._B.push(-e._m2*(d?c:u)),s=0;s<e._rangebreaks.length;s++)l=e._rangebreaks[s],e._B.push(e._B[e._B.length-1]-p*e._m2*(l.max-l.min));for(s=0;s<e._rangebreaks.length;s++)(l=e._rangebreaks[s]).pmin=R(l.min),l.pmax=R(l.max)}if(!isFinite(e._m)||!isFinite(e._b)||e._length<0)throw t._replotting=!1,new Error("Something went wrong with axis scaling")},e.maskBreaks=function(t){for(var n,r,i,o,l,u=e.rangebreaks||[],c=0;c<u.length;c++){var d=u[c];if(d.enabled)if(d.bounds){var h=d.pattern;switch(r=(n=a.simpleMap(d.bounds,h?s:e.d2c))[0],i=n[1],h){case x:o=(l=new Date(t)).getUTCDay(),r>i&&(i+=7,o<r&&(o+=7));break;case I:o=(l=new Date(t)).getUTCHours()+(l.getUTCMinutes()/60+l.getUTCSeconds()/3600+l.getUTCMilliseconds()/36e5),r>i&&(i+=24,o<r&&(o+=24));break;case"":o=t}if(o>=r&&o<i)return p}else for(var f=a.simpleMap(d.values,e.d2c).sort(a.sorterAsc),g=0;g<f.length;g++)if(i=(r=f[g])+d.dvalue,t>=r&&t<i)return p}return t},e.locateBreaks=function(t,n){var r,i,o,l,u=[];if(!e.rangebreaks)return u;var c=e.rangebreaks.slice().sort((function(e,t){return e.pattern===x&&t.pattern===I?-1:t.pattern===x&&e.pattern===I?1:0})),d=function(e,r){if((e=a.constrain(e,t,n))!==(r=a.constrain(r,t,n))){for(var i=!0,o=0;o<u.length;o++){var s=u[o];e<s.max&&r>=s.min&&(e<s.min&&(s.min=e),r>s.max&&(s.max=r),i=!1)}i&&u.push({min:e,max:r})}};for(r=0;r<c.length;r++){var h=c[r];if(h.enabled)if(h.bounds){var f=t,p=n;h.pattern&&(f=Math.floor(f)),o=(i=a.simpleMap(h.bounds,h.pattern?s:e.r2l))[0],l=i[1];var g,A,_=new Date(f);switch(h.pattern){case x:A=v,g=(l-o+(l<o?7:0))*m,f+=o*m-(_.getUTCDay()*m+_.getUTCHours()*y+_.getUTCMinutes()*b+_.getUTCSeconds()*C+_.getUTCMilliseconds());break;case I:A=m,g=(l-o+(l<o?24:0))*y,f+=o*y-(_.getUTCHours()*y+_.getUTCMinutes()*b+_.getUTCSeconds()*C+_.getUTCMilliseconds());break;default:f=Math.min(i[0],i[1]),g=A=(p=Math.max(i[0],i[1]))-f}for(var w=f;w<p;w+=A)d(w,w+g)}else for(var M=a.simpleMap(h.values,e.d2c),S=0;S<M.length;S++)d(o=M[S],l=o+h.dvalue)}return u.sort((function(e,t){return e.min-t.min})),u},e.makeCalcdata=function(t,n,r){var i,o,s,l,u=e.type,c="date"===u&&t[n+"calendar"];if(n in t){if(i=t[n],l=t._length||a.minRowLength(i),a.isTypedArray(i)&&("linear"===u||"log"===u)){if(l===i.length)return i;if(i.subarray)return i.subarray(0,l)}if("multicategory"===u)return function(e,t){for(var n=new Array(t),r=0;r<t;r++){var i=(e[0]||[])[r],o=(e[1]||[])[r];n[r]=L([i,o])}return n}(i,l);for(o=new Array(l),s=0;s<l;s++)o[s]=e.d2c(i[s],0,c,r)}else{var d=n+"0"in t?e.d2c(t[n+"0"],0,c):0,h=t["d"+n]?Number(t["d"+n]):1;for(i=t[{x:"y",y:"x"}[n]],l=t._length||i.length,o=new Array(l),s=0;s<l;s++)o[s]=d+s*h}if(e.rangebreaks)for(s=0;s<l;s++)o[s]=e.maskBreaks(o[s]);return o},e.isValidRange=function(t){return Array.isArray(t)&&2===t.length&&o(e.r2l(t[0]))&&o(e.r2l(t[1]))},e.isPtWithinRange=function(t,n){var r=e.c2l(t[h],null,n),i=e.r2l(e.range[0]),o=e.r2l(e.range[1]);return i<o?i<=r&&r<=o:o<=r&&r<=i},e._emptyCategories=function(){e._categories=[],e._categoriesMap={}},e.clearCalc=function(){var n=e._matchGroup;if(n){var r=null,i=null;for(var o in n){var a=t[A.id2name(o)];if(a._categories){r=a._categories,i=a._categoriesMap;break}}r&&i?(e._categories=r,e._categoriesMap=i):e._emptyCategories()}else e._emptyCategories();if(e._initialCategories)for(var s=0;s<e._initialCategories.length;s++)k(e._initialCategories[s])},e.sortByInitialCategories=function(){var r=[];if(e._emptyCategories(),e._initialCategories)for(var i=0;i<e._initialCategories.length;i++)k(e._initialCategories[i]);r=r.concat(e._traceIndices);var o=e._matchGroup;for(var a in o)if(n!==a){var s=t[A.id2name(a)];s._categories=e._categories,s._categoriesMap=e._categoriesMap,r=r.concat(s._traceIndices)}return r};var B=t._d3locale;"date"===e.type&&(e._dateFormat=B?B.timeFormat:i,e._extraFormat=t._extraFormat),e._separators=t.separators,e._numFormat=B?B.numberFormat:r.format,delete e._minDtick,delete e._forceTick0}},function(e,t,n){"use strict";var r=n(39),i=n(27),o=n(10),a=n(82),s=n(56),l=n(384).getModuleCalcData,u=n(83),c=n(95),d=n(196),h=o.ensureSingle;function f(e,t,n){return o.ensureSingle(e,t,n,(function(e){e.datum(n)}))}function p(e,t,n,o,a){for(var u,d,h,f=c.traceLayerClasses,p=e._fullLayout,g=p._modules,v=[],m=[],y=0;y<g.length;y++){var b=(u=g[y]).name,C=i.modules[b].categories;if(C.svg){var A=u.layerName||b+"layer",_=u.plot;h=(d=l(n,_))[0],n=d[1],h.length&&v.push({i:f.indexOf(A),className:A,plotMethod:_,cdModule:h}),C.zoomScale&&m.push("."+A)}}v.sort((function(e,t){return e.i-t.i}));var I=t.plot.selectAll("g.mlayer").data(v,(function(e){return e.className}));if(I.enter().append("g").attr("class",(function(e){return e.className})).classed("mlayer",!0).classed("rangeplot",t.isRangePlot),I.exit().remove(),I.order(),I.each((function(n){var i=r.select(this),l=n.className;n.plotMethod(e,t,n.cdModule,i,o,a),-1===c.clipOnAxisFalseQuery.indexOf("."+l)&&s.setClipUrl(i,t.layerClipId,e)})),p._has("scattergl")&&(u=i.getModule("scattergl"),h=l(n,u)[0],u.plot(e,t,h)),!e._context.staticPlot&&(t._hasClipOnAxisFalse&&(t.clipOnAxisFalseTraces=t.plot.selectAll(c.clipOnAxisFalseQuery.join(",")).selectAll(".trace")),m.length)){var x=t.plot.selectAll(m.join(",")).selectAll(".trace");t.zoomScalePts=x.selectAll("path.point"),t.zoomScaleTxt=x.selectAll(".textpoint")}}function g(e,t){var n=t.plotgroup,r=t.id,i=c.layerValue2layerClass[t.xaxis.layer],o=c.layerValue2layerClass[t.yaxis.layer],a=e._fullLayout._hasOnlyLargeSploms;if(t.mainplot){var s=t.mainplotinfo,l=s.plotgroup,d=r+"-x",p=r+"-y";t.gridlayer=s.gridlayer,t.zerolinelayer=s.zerolinelayer,h(s.overlinesBelow,"path",d),h(s.overlinesBelow,"path",p),h(s.overaxesBelow,"g",d),h(s.overaxesBelow,"g",p),t.plot=h(s.overplot,"g",r),h(s.overlinesAbove,"path",d),h(s.overlinesAbove,"path",p),h(s.overaxesAbove,"g",d),h(s.overaxesAbove,"g",p),t.xlines=l.select(".overlines-"+i).select("."+d),t.ylines=l.select(".overlines-"+o).select("."+p),t.xaxislayer=l.select(".overaxes-"+i).select("."+d),t.yaxislayer=l.select(".overaxes-"+o).select("."+p)}else if(a)t.xlines=h(n,"path","xlines-above"),t.ylines=h(n,"path","ylines-above"),t.xaxislayer=h(n,"g","xaxislayer-above"),t.yaxislayer=h(n,"g","yaxislayer-above");else{var g=h(n,"g","layer-subplot");t.shapelayer=h(g,"g","shapelayer"),t.imagelayer=h(g,"g","imagelayer"),t.gridlayer=h(n,"g","gridlayer"),t.zerolinelayer=h(n,"g","zerolinelayer"),h(n,"path","xlines-below"),h(n,"path","ylines-below"),t.overlinesBelow=h(n,"g","overlines-below"),h(n,"g","xaxislayer-below"),h(n,"g","yaxislayer-below"),t.overaxesBelow=h(n,"g","overaxes-below"),t.plot=h(n,"g","plot"),t.overplot=h(n,"g","overplot"),t.xlines=h(n,"path","xlines-above"),t.ylines=h(n,"path","ylines-above"),t.overlinesAbove=h(n,"g","overlines-above"),h(n,"g","xaxislayer-above"),h(n,"g","yaxislayer-above"),t.overaxesAbove=h(n,"g","overaxes-above"),t.xlines=n.select(".xlines-"+i),t.ylines=n.select(".ylines-"+o),t.xaxislayer=n.select(".xaxislayer-"+i),t.yaxislayer=n.select(".yaxislayer-"+o)}a||(f(t.gridlayer,"g",t.xaxis._id),f(t.gridlayer,"g",t.yaxis._id),t.gridlayer.selectAll("g").map((function(e){return e[0]})).sort(u.idSort)),t.xlines.style("fill","none").classed("crisp",!0),t.ylines.style("fill","none").classed("crisp",!0)}function v(e,t){if(e){var n={};for(var i in e.each((function(e){var i=e[0];r.select(this).remove(),m(i,t),n[i]=!0})),t._plots)for(var o=t._plots[i].overlays||[],a=0;a<o.length;a++){var s=o[a];n[s.id]&&s.plot.selectAll(".trace").remove()}}}function m(e,t){t._draggers.selectAll("g."+e).remove(),t._defs.select("#clip"+t._uid+e+"plot").remove()}t.name="cartesian",t.attr=["xaxis","yaxis"],t.idRoot=["x","y"],t.idRegex=c.idRegex,t.attrRegex=c.attrRegex,t.attributes=n(1733),t.layoutAttributes=n(134),t.supplyLayoutDefaults=n(1734),t.transitionAxes=n(1736),t.finalizeSubplots=function(e,t){var n,r,i,a=t._subplots,s=a.xaxis,l=a.yaxis,d=a.cartesian,h=d.concat(a.gl2d||[]),f={},p={};for(n=0;n<h.length;n++){var g=h[n].split("y");f[g[0]]=1,p["y"+g[1]]=1}for(n=0;n<s.length;n++)f[r=s[n]]||(i=(e[u.id2name(r)]||{}).anchor,c.idRegex.y.test(i)||(i="y"),d.push(r+i),h.push(r+i),p[i]||(p[i]=1,o.pushUnique(l,i)));for(n=0;n<l.length;n++)p[i=l[n]]||(r=(e[u.id2name(i)]||{}).anchor,c.idRegex.x.test(r)||(r="x"),d.push(r+i),h.push(r+i),f[r]||(f[r]=1,o.pushUnique(s,r)));if(!h.length){for(var v in r="",i="",e){if(c.attrRegex.test(v))"x"===v.charAt(0)?(!r||+v.substr(5)<+r.substr(5))&&(r=v):(!i||+v.substr(5)<+i.substr(5))&&(i=v)}r=r?u.name2id(r):"x",i=i?u.name2id(i):"y",s.push(r),l.push(i),d.push(r+i)}},t.plot=function(e,t,n,r){var i,o=e._fullLayout,a=o._subplots.cartesian,s=e.calcdata;if(!Array.isArray(t))for(t=[],i=0;i<s.length;i++)t.push(i);for(i=0;i<a.length;i++){for(var l,u=a[i],c=o._plots[u],d=[],h=0;h<s.length;h++){var f=s[h],g=f[0].trace;g.xaxis+g.yaxis===u&&((-1!==t.indexOf(g.index)||g.carpet)&&(l&&l[0].trace.xaxis+l[0].trace.yaxis===u&&-1!==["tonextx","tonexty","tonext"].indexOf(g.fill)&&-1===d.indexOf(l)&&d.push(l),d.push(f)),l=f)}p(e,c,d,n,r)}},t.clean=function(e,t,n,r){var i,o,a,s=r._plots||{},l=t._plots||{},c=r._subplots||{};if(r._hasOnlyLargeSploms&&!t._hasOnlyLargeSploms)for(a in s)(i=s[a]).plotgroup&&i.plotgroup.remove();var d=r._has&&r._has("gl"),h=t._has&&t._has("gl");if(d&&!h)for(a in s)(i=s[a])._scene&&i._scene.destroy();if(c.xaxis&&c.yaxis){var f=u.listIds({_fullLayout:r});for(o=0;o<f.length;o++){var p=f[o];t[u.id2name(p)]||r._infolayer.selectAll(".g-"+p+"title").remove()}}var g=r._has&&r._has("cartesian"),y=t._has&&t._has("cartesian");if(g&&!y)v(r._cartesianlayer.selectAll(".subplot"),r),r._defs.selectAll(".axesclip").remove(),delete r._axisConstraintGroups,delete r._axisMatchGroups;else if(c.cartesian)for(o=0;o<c.cartesian.length;o++){var b=c.cartesian[o];if(!l[b]){var C="."+b+",."+b+"-x,."+b+"-y";r._cartesianlayer.selectAll(C).remove(),m(b,r)}}},t.drawFramework=function(e){var t=e._fullLayout,n=function(e){var t,n,r,i,o,a,s=e._fullLayout,l=s._subplots.cartesian,u=l.length,c=[],d=[];for(t=0;t<u;t++){r=l[t],i=s._plots[r],o=i.xaxis,a=i.yaxis;var h=o._mainAxis,f=a._mainAxis,p=h._id+f._id,g=s._plots[p];i.overlays=[],p!==r&&g?(i.mainplot=p,i.mainplotinfo=g,d.push(r)):(i.mainplot=void 0,i.mainplotinfo=void 0,c.push(r))}for(t=0;t<d.length;t++)r=d[t],(i=s._plots[r]).mainplotinfo.overlays.push(i);var v=c.concat(d),m=new Array(u);for(t=0;t<u;t++){r=v[t],i=s._plots[r],o=i.xaxis,a=i.yaxis;var y=[r,o.layer,a.layer,o.overlaying||"",a.overlaying||""];for(n=0;n<i.overlays.length;n++)y.push(i.overlays[n].id);m[t]=y}return m}(e),i=t._cartesianlayer.selectAll(".subplot").data(n,String);i.enter().append("g").attr("class",(function(e){return"subplot "+e[0]})),i.order(),i.exit().call(v,t),i.each((function(n){var i=n[0],o=t._plots[i];o.plotgroup=r.select(this),g(e,o),o.draglayer=h(t._draggers,"g",i)}))},t.rangePlot=function(e,t,n){g(e,t),p(e,t,n),a.style(e)},t.toSVG=function(e){var t=e._fullLayout._glimages,n=r.select(e).selectAll(".svg-container");n.filter((function(e,t){return t===n.size()-1})).selectAll(".gl-canvas-context, .gl-canvas-focus").each((function(){var e=this.toDataURL("image/png");t.append("svg:image").attr({xmlns:d.svg,"xlink:href":e,preserveAspectRatio:"none",x:0,y:0,width:this.width,height:this.height})}))},t.updateFx=n(991).updateFx},function(e,t,n){"use strict";function r(e,t,n,r,i){for(var o=i+1;r<=i;){var a=r+i>>>1,s=e[a];(void 0!==n?n(s,t):s-t)>=0?(o=a,i=a-1):r=a+1}return o}function i(e,t,n,r,i){for(var o=i+1;r<=i;){var a=r+i>>>1,s=e[a];(void 0!==n?n(s,t):s-t)>0?(o=a,i=a-1):r=a+1}return o}function o(e,t,n,r,i){for(var o=r-1;r<=i;){var a=r+i>>>1,s=e[a];(void 0!==n?n(s,t):s-t)<0?(o=a,r=a+1):i=a-1}return o}function a(e,t,n,r,i){for(var o=r-1;r<=i;){var a=r+i>>>1,s=e[a];(void 0!==n?n(s,t):s-t)<=0?(o=a,r=a+1):i=a-1}return o}function s(e,t,n,r,i){for(;r<=i;){var o=r+i>>>1,a=e[o],s=void 0!==n?n(a,t):a-t;if(0===s)return o;s<=0?r=o+1:i=o-1}return-1}function l(e,t,n,r,i,o){return"function"==typeof n?o(e,t,n,void 0===r?0:0|r,void 0===i?e.length-1:0|i):o(e,t,void 0,void 0===n?0:0|n,void 0===r?e.length-1:0|r)}e.exports={ge:function(e,t,n,i,o){return l(e,t,n,i,o,r)},gt:function(e,t,n,r,o){return l(e,t,n,r,o,i)},lt:function(e,t,n,r,i){return l(e,t,n,r,i,o)},le:function(e,t,n,r,i){return l(e,t,n,r,i,a)},eq:function(e,t,n,r,i){return l(e,t,n,r,i,s)}}},function(e,t,n){"use strict";var r=n(1048),i=n(572);e.exports=function(e,t){var n=i(e),o=i(t);if(0===n)return[r(0),r(1)];if(0===o)return[r(0),r(0)];o<0&&(e=e.neg(),t=t.neg());var a=e.gcd(t);if(a.cmpn(1))return[e.div(a),t.div(a)];return[e,t]}},function(e,t,n){"use strict";var r=n(1057);e.exports=function(e){return e?r(e):[0,0,0,1]}},function(e,t,n){var r=n(2034);"string"==typeof r&&(r=[[e.i,r,""]]);var i={insert:"head",singleton:!1};n(147)(r,i);r.locals&&(e.exports=r.locals)},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.BaseObserver=void 0;const r=n(401);t.BaseObserver=class{constructor(){this.listeners={}}fireEventInternal(e,t,n){this.iterateListeners(r=>{if(!e&&!n.firing)return!1;r[t]&&r[t](n)})}fireEvent(e,t){e=Object.assign({firing:!0,stopPropagation:()=>{e.firing=!1}},e),this.fireEventInternal(!0,"eventWillFire",Object.assign(Object.assign({},e),{function:t})),this.fireEventInternal(!1,t,e),this.fireEventInternal(!0,"eventDidFire",Object.assign(Object.assign({},e),{function:t}))}iterateListeners(e){for(let t in this.listeners){if(!1===e(this.listeners[t]))return}}getListenerHandle(e){for(let t in this.listeners)if(this.listeners[t]===e)return{id:t,listener:e,deregister:()=>{delete this.listeners[t]}}}registerListener(e){const t=r.Toolkit.UID();return this.listeners[t]=e,{id:t,listener:e,deregister:()=>{delete this.listeners[t]}}}deregisterListener(e){if("object"==typeof e)return e.deregister(),!0;const t=this.getListenerHandle(e);return!!t&&(t.deregister(),!0)}}},function(e,t){e.exports={manhattan:function(e,t){return e+t},euclidean:function(e,t){return Math.sqrt(e*e+t*t)},octile:function(e,t){var n=Math.SQRT2-1;return e<t?n*e+t:n*t+e},chebyshev:function(e,t){return Math.max(e,t)}}},function(e,t,n){var r=n(409),i=n(410);e.exports=function(e,t,n,o){var a=!n;n||(n={});for(var s=-1,l=t.length;++s<l;){var u=t[s],c=o?o(n[u],e[u],u,n,e):void 0;void 0===c&&(c=e[u]),a?i(n,u,c):r(n,u,c)}return n}},function(e,t,n){var r=n(259);e.exports=function(e){if("string"==typeof e||r(e))return e;var t=e+"";return"0"==t&&1/e==-1/0?"-0":t}},function(e,t,n){"use strict";n.r(t);var r=n(923);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";n.r(t);var r=n(924);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";n.r(t);var r=n(927);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";n.r(t);var r=n(1196);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";var r=n(36);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var i=r(n(50)),o=n(115),a=r(n(2275));var s=function(e,t){return(0,o.withStyles)(e,(0,i.default)({defaultTheme:a.default},t))};t.default=s},function(e,t,n){"use strict";var r=n(318);n.d(t,"a",(function(){return r.a})),n.d(t,"b",(function(){return r.c}))},function(e,t,n){"use strict";n.r(t);var r=n(919);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";n.r(t);var r=new Date,i=new Date;function o(e,t,n,a){function s(t){return e(t=0===arguments.length?new Date:new Date(+t)),t}return s.floor=function(t){return e(t=new Date(+t)),t},s.ceil=function(n){return e(n=new Date(n-1)),t(n,1),e(n),n},s.round=function(e){var t=s(e),n=s.ceil(e);return e-t<n-e?t:n},s.offset=function(e,n){return t(e=new Date(+e),null==n?1:Math.floor(n)),e},s.range=function(n,r,i){var o,a=[];if(n=s.ceil(n),i=null==i?1:Math.floor(i),!(n<r&&i>0))return a;do{a.push(o=new Date(+n)),t(n,i),e(n)}while(o<n&&n<r);return a},s.filter=function(n){return o((function(t){if(t>=t)for(;e(t),!n(t);)t.setTime(t-1)}),(function(e,r){if(e>=e)if(r<0)for(;++r<=0;)for(;t(e,-1),!n(e););else for(;--r>=0;)for(;t(e,1),!n(e););}))},n&&(s.count=function(t,o){return r.setTime(+t),i.setTime(+o),e(r),e(i),Math.floor(n(r,i))},s.every=function(e){return e=Math.floor(e),isFinite(e)&&e>0?e>1?s.filter(a?function(t){return a(t)%e==0}:function(t){return s.count(0,t)%e==0}):s:null}),s}function a(e){return o((function(t){t.setUTCDate(t.getUTCDate()-(t.getUTCDay()+7-e)%7),t.setUTCHours(0,0,0,0)}),(function(e,t){e.setUTCDate(e.getUTCDate()+7*t)}),(function(e,t){return(t-e)/6048e5}))}var s=a(0),l=a(1),u=a(2),c=a(3),d=a(4),h=a(5),f=a(6),p=(s.range,l.range,u.range,c.range,d.range,h.range,f.range,o((function(e){e.setUTCHours(0,0,0,0)}),(function(e,t){e.setUTCDate(e.getUTCDate()+t)}),(function(e,t){return(t-e)/864e5}),(function(e){return e.getUTCDate()-1}))),g=p;p.range;function v(e){return o((function(t){t.setDate(t.getDate()-(t.getDay()+7-e)%7),t.setHours(0,0,0,0)}),(function(e,t){e.setDate(e.getDate()+7*t)}),(function(e,t){return(t-e-6e4*(t.getTimezoneOffset()-e.getTimezoneOffset()))/6048e5}))}var m=v(0),y=v(1),b=v(2),C=v(3),A=v(4),_=v(5),I=v(6),x=(m.range,y.range,b.range,C.range,A.range,_.range,I.range,o((function(e){e.setHours(0,0,0,0)}),(function(e,t){e.setDate(e.getDate()+t)}),(function(e,t){return(t-e-6e4*(t.getTimezoneOffset()-e.getTimezoneOffset()))/864e5}),(function(e){return e.getDate()-1}))),w=x,M=(x.range,o((function(e){e.setMonth(0,1),e.setHours(0,0,0,0)}),(function(e,t){e.setFullYear(e.getFullYear()+t)}),(function(e,t){return t.getFullYear()-e.getFullYear()}),(function(e){return e.getFullYear()})));M.every=function(e){return isFinite(e=Math.floor(e))&&e>0?o((function(t){t.setFullYear(Math.floor(t.getFullYear()/e)*e),t.setMonth(0,1),t.setHours(0,0,0,0)}),(function(t,n){t.setFullYear(t.getFullYear()+n*e)})):null};var S=M,T=(M.range,o((function(e){e.setUTCMonth(0,1),e.setUTCHours(0,0,0,0)}),(function(e,t){e.setUTCFullYear(e.getUTCFullYear()+t)}),(function(e,t){return t.getUTCFullYear()-e.getUTCFullYear()}),(function(e){return e.getUTCFullYear()})));T.every=function(e){return isFinite(e=Math.floor(e))&&e>0?o((function(t){t.setUTCFullYear(Math.floor(t.getUTCFullYear()/e)*e),t.setUTCMonth(0,1),t.setUTCHours(0,0,0,0)}),(function(t,n){t.setUTCFullYear(t.getUTCFullYear()+n*e)})):null};var E=T;T.range;function O(e){if(0<=e.y&&e.y<100){var t=new Date(-1,e.m,e.d,e.H,e.M,e.S,e.L);return t.setFullYear(e.y),t}return new Date(e.y,e.m,e.d,e.H,e.M,e.S,e.L)}function k(e){if(0<=e.y&&e.y<100){var t=new Date(Date.UTC(-1,e.m,e.d,e.H,e.M,e.S,e.L));return t.setUTCFullYear(e.y),t}return new Date(Date.UTC(e.y,e.m,e.d,e.H,e.M,e.S,e.L))}function L(e,t,n){return{y:e,m:t,d:n,H:0,M:0,S:0,L:0}}function D(e){var t=e.dateTime,n=e.date,r=e.time,i=e.periods,o=e.days,a=e.shortDays,s=e.months,u=e.shortMonths,c=W(i),d=G(i),h=W(o),f=G(o),p=W(a),v=G(a),m=W(s),b=G(s),C=W(u),A=G(u),_={a:function(e){return a[e.getDay()]},A:function(e){return o[e.getDay()]},b:function(e){return u[e.getMonth()]},B:function(e){return s[e.getMonth()]},c:null,d:fe,e:fe,f:ye,g:Ee,G:ke,H:pe,I:ge,j:ve,L:me,m:be,M:Ce,p:function(e){return i[+(e.getHours()>=12)]},q:function(e){return 1+~~(e.getMonth()/3)},Q:et,s:tt,S:Ae,u:_e,U:Ie,V:we,w:Me,W:Se,x:null,X:null,y:Te,Y:Oe,Z:Le,"%":$e},I={a:function(e){return a[e.getUTCDay()]},A:function(e){return o[e.getUTCDay()]},b:function(e){return u[e.getUTCMonth()]},B:function(e){return s[e.getUTCMonth()]},c:null,d:De,e:De,f:ze,g:Ze,G:Ke,H:Pe,I:Ne,j:je,L:Re,m:Fe,M:Be,p:function(e){return i[+(e.getUTCHours()>=12)]},q:function(e){return 1+~~(e.getUTCMonth()/3)},Q:et,s:tt,S:He,u:Ue,U:Ve,V:We,w:Ge,W:Xe,x:null,X:null,y:qe,Y:Qe,Z:Je,"%":$e},x={a:function(e,t,n){var r=p.exec(t.slice(n));return r?(e.w=v[r[0].toLowerCase()],n+r[0].length):-1},A:function(e,t,n){var r=h.exec(t.slice(n));return r?(e.w=f[r[0].toLowerCase()],n+r[0].length):-1},b:function(e,t,n){var r=C.exec(t.slice(n));return r?(e.m=A[r[0].toLowerCase()],n+r[0].length):-1},B:function(e,t,n){var r=m.exec(t.slice(n));return r?(e.m=b[r[0].toLowerCase()],n+r[0].length):-1},c:function(e,n,r){return T(e,t,n,r)},d:re,e:re,f:ue,g:$,G:J,H:oe,I:oe,j:ie,L:le,m:ne,M:ae,p:function(e,t,n){var r=c.exec(t.slice(n));return r?(e.p=d[r[0].toLowerCase()],n+r[0].length):-1},q:te,Q:de,s:he,S:se,u:q,U:Z,V:Q,w:X,W:K,x:function(e,t,r){return T(e,n,t,r)},X:function(e,t,n){return T(e,r,t,n)},y:$,Y:J,Z:ee,"%":ce};function M(e,t){return function(n){var r,i,o,a=[],s=-1,l=0,u=e.length;for(n instanceof Date||(n=new Date(+n));++s<u;)37===e.charCodeAt(s)&&(a.push(e.slice(l,s)),null!=(i=F[r=e.charAt(++s)])?r=e.charAt(++s):i="e"===r?" ":"0",(o=t[r])&&(r=o(n,i)),a.push(r),l=s+1);return a.push(e.slice(l,s)),a.join("")}}function S(e,t){return function(n){var r,i,o=L(1900,void 0,1);if(T(o,e,n+="",0)!=n.length)return null;if("Q"in o)return new Date(o.Q);if("s"in o)return new Date(1e3*o.s+("L"in o?o.L:0));if(t&&!("Z"in o)&&(o.Z=0),"p"in o&&(o.H=o.H%12+12*o.p),void 0===o.m&&(o.m="q"in o?o.q:0),"V"in o){if(o.V<1||o.V>53)return null;"w"in o||(o.w=1),"Z"in o?(i=(r=k(L(o.y,0,1))).getUTCDay(),r=i>4||0===i?l.ceil(r):l(r),r=g.offset(r,7*(o.V-1)),o.y=r.getUTCFullYear(),o.m=r.getUTCMonth(),o.d=r.getUTCDate()+(o.w+6)%7):(i=(r=O(L(o.y,0,1))).getDay(),r=i>4||0===i?y.ceil(r):y(r),r=w.offset(r,7*(o.V-1)),o.y=r.getFullYear(),o.m=r.getMonth(),o.d=r.getDate()+(o.w+6)%7)}else("W"in o||"U"in o)&&("w"in o||(o.w="u"in o?o.u%7:"W"in o?1:0),i="Z"in o?k(L(o.y,0,1)).getUTCDay():O(L(o.y,0,1)).getDay(),o.m=0,o.d="W"in o?(o.w+6)%7+7*o.W-(i+5)%7:o.w+7*o.U-(i+6)%7);return"Z"in o?(o.H+=o.Z/100|0,o.M+=o.Z%100,k(o)):O(o)}}function T(e,t,n,r){for(var i,o,a=0,s=t.length,l=n.length;a<s;){if(r>=l)return-1;if(37===(i=t.charCodeAt(a++))){if(i=t.charAt(a++),!(o=x[i in F?t.charAt(a++):i])||(r=o(e,n,r))<0)return-1}else if(i!=n.charCodeAt(r++))return-1}return r}return _.x=M(n,_),_.X=M(r,_),_.c=M(t,_),I.x=M(n,I),I.X=M(r,I),I.c=M(t,I),{format:function(e){var t=M(e+="",_);return t.toString=function(){return e},t},parse:function(e){var t=S(e+="",!1);return t.toString=function(){return e},t},utcFormat:function(e){var t=M(e+="",I);return t.toString=function(){return e},t},utcParse:function(e){var t=S(e+="",!0);return t.toString=function(){return e},t}}}var P,N,j,R,z,F={"-":"",_:" ",0:"0"},B=/^\s*\d+/,H=/^%/,U=/[\\^$*+?|[\]().{}]/g;function V(e,t,n){var r=e<0?"-":"",i=(r?-e:e)+"",o=i.length;return r+(o<n?new Array(n-o+1).join(t)+i:i)}function Y(e){return e.replace(U,"\\$&")}function W(e){return new RegExp("^(?:"+e.map(Y).join("|")+")","i")}function G(e){for(var t={},n=-1,r=e.length;++n<r;)t[e[n].toLowerCase()]=n;return t}function X(e,t,n){var r=B.exec(t.slice(n,n+1));return r?(e.w=+r[0],n+r[0].length):-1}function q(e,t,n){var r=B.exec(t.slice(n,n+1));return r?(e.u=+r[0],n+r[0].length):-1}function Z(e,t,n){var r=B.exec(t.slice(n,n+2));return r?(e.U=+r[0],n+r[0].length):-1}function Q(e,t,n){var r=B.exec(t.slice(n,n+2));return r?(e.V=+r[0],n+r[0].length):-1}function K(e,t,n){var r=B.exec(t.slice(n,n+2));return r?(e.W=+r[0],n+r[0].length):-1}function J(e,t,n){var r=B.exec(t.slice(n,n+4));return r?(e.y=+r[0],n+r[0].length):-1}function $(e,t,n){var r=B.exec(t.slice(n,n+2));return r?(e.y=+r[0]+(+r[0]>68?1900:2e3),n+r[0].length):-1}function ee(e,t,n){var r=/^(Z)|([+-]\d\d)(?::?(\d\d))?/.exec(t.slice(n,n+6));return r?(e.Z=r[1]?0:-(r[2]+(r[3]||"00")),n+r[0].length):-1}function te(e,t,n){var r=B.exec(t.slice(n,n+1));return r?(e.q=3*r[0]-3,n+r[0].length):-1}function ne(e,t,n){var r=B.exec(t.slice(n,n+2));return r?(e.m=r[0]-1,n+r[0].length):-1}function re(e,t,n){var r=B.exec(t.slice(n,n+2));return r?(e.d=+r[0],n+r[0].length):-1}function ie(e,t,n){var r=B.exec(t.slice(n,n+3));return r?(e.m=0,e.d=+r[0],n+r[0].length):-1}function oe(e,t,n){var r=B.exec(t.slice(n,n+2));return r?(e.H=+r[0],n+r[0].length):-1}function ae(e,t,n){var r=B.exec(t.slice(n,n+2));return r?(e.M=+r[0],n+r[0].length):-1}function se(e,t,n){var r=B.exec(t.slice(n,n+2));return r?(e.S=+r[0],n+r[0].length):-1}function le(e,t,n){var r=B.exec(t.slice(n,n+3));return r?(e.L=+r[0],n+r[0].length):-1}function ue(e,t,n){var r=B.exec(t.slice(n,n+6));return r?(e.L=Math.floor(r[0]/1e3),n+r[0].length):-1}function ce(e,t,n){var r=H.exec(t.slice(n,n+1));return r?n+r[0].length:-1}function de(e,t,n){var r=B.exec(t.slice(n));return r?(e.Q=+r[0],n+r[0].length):-1}function he(e,t,n){var r=B.exec(t.slice(n));return r?(e.s=+r[0],n+r[0].length):-1}function fe(e,t){return V(e.getDate(),t,2)}function pe(e,t){return V(e.getHours(),t,2)}function ge(e,t){return V(e.getHours()%12||12,t,2)}function ve(e,t){return V(1+w.count(S(e),e),t,3)}function me(e,t){return V(e.getMilliseconds(),t,3)}function ye(e,t){return me(e,t)+"000"}function be(e,t){return V(e.getMonth()+1,t,2)}function Ce(e,t){return V(e.getMinutes(),t,2)}function Ae(e,t){return V(e.getSeconds(),t,2)}function _e(e){var t=e.getDay();return 0===t?7:t}function Ie(e,t){return V(m.count(S(e)-1,e),t,2)}function xe(e){var t=e.getDay();return t>=4||0===t?A(e):A.ceil(e)}function we(e,t){return e=xe(e),V(A.count(S(e),e)+(4===S(e).getDay()),t,2)}function Me(e){return e.getDay()}function Se(e,t){return V(y.count(S(e)-1,e),t,2)}function Te(e,t){return V(e.getFullYear()%100,t,2)}function Ee(e,t){return V((e=xe(e)).getFullYear()%100,t,2)}function Oe(e,t){return V(e.getFullYear()%1e4,t,4)}function ke(e,t){var n=e.getDay();return V((e=n>=4||0===n?A(e):A.ceil(e)).getFullYear()%1e4,t,4)}function Le(e){var t=e.getTimezoneOffset();return(t>0?"-":(t*=-1,"+"))+V(t/60|0,"0",2)+V(t%60,"0",2)}function De(e,t){return V(e.getUTCDate(),t,2)}function Pe(e,t){return V(e.getUTCHours(),t,2)}function Ne(e,t){return V(e.getUTCHours()%12||12,t,2)}function je(e,t){return V(1+g.count(E(e),e),t,3)}function Re(e,t){return V(e.getUTCMilliseconds(),t,3)}function ze(e,t){return Re(e,t)+"000"}function Fe(e,t){return V(e.getUTCMonth()+1,t,2)}function Be(e,t){return V(e.getUTCMinutes(),t,2)}function He(e,t){return V(e.getUTCSeconds(),t,2)}function Ue(e){var t=e.getUTCDay();return 0===t?7:t}function Ve(e,t){return V(s.count(E(e)-1,e),t,2)}function Ye(e){var t=e.getUTCDay();return t>=4||0===t?d(e):d.ceil(e)}function We(e,t){return e=Ye(e),V(d.count(E(e),e)+(4===E(e).getUTCDay()),t,2)}function Ge(e){return e.getUTCDay()}function Xe(e,t){return V(l.count(E(e)-1,e),t,2)}function qe(e,t){return V(e.getUTCFullYear()%100,t,2)}function Ze(e,t){return V((e=Ye(e)).getUTCFullYear()%100,t,2)}function Qe(e,t){return V(e.getUTCFullYear()%1e4,t,4)}function Ke(e,t){var n=e.getUTCDay();return V((e=n>=4||0===n?d(e):d.ceil(e)).getUTCFullYear()%1e4,t,4)}function Je(){return"+0000"}function $e(){return"%"}function et(e){return+e}function tt(e){return Math.floor(+e/1e3)}function nt(e){return P=D(e),N=P.format,j=P.parse,R=P.utcFormat,z=P.utcParse,P}nt({dateTime:"%x, %X",date:"%-m/%-d/%Y",time:"%-I:%M:%S %p",periods:["AM","PM"],days:["Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"],shortDays:["Sun","Mon","Tue","Wed","Thu","Fri","Sat"],months:["January","February","March","April","May","June","July","August","September","October","November","December"],shortMonths:["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"]});var rt=Date.prototype.toISOString?function(e){return e.toISOString()}:R("%Y-%m-%dT%H:%M:%S.%LZ");var it=+new Date("2000-01-01T00:00:00.000Z")?function(e){var t=new Date(e);return isNaN(t)?null:t}:z("%Y-%m-%dT%H:%M:%S.%LZ");n.d(t,"timeFormatDefaultLocale",(function(){return nt})),n.d(t,"timeFormat",(function(){return N})),n.d(t,"timeParse",(function(){return j})),n.d(t,"utcFormat",(function(){return R})),n.d(t,"utcParse",(function(){return z})),n.d(t,"timeFormatLocale",(function(){return D})),n.d(t,"isoFormat",(function(){return rt})),n.d(t,"isoParse",(function(){return it}))},function(e,t,n){"use strict";n.d(t,"c",(function(){return f})),n.d(t,"a",(function(){return g})),n.d(t,"b",(function(){return v}));var r,i=n(1),o=n(5),a=n(0),s=n.n(a),l=(n(3),n(1308)),u=n(89),c=n(1313),d=Object(u.c)(Object(c.a)()),h=Object(l.a)(),f=new Map,p={disableGeneration:!1,generateClassName:h,jss:d,sheetsCache:null,sheetsManager:f,sheetsRegistry:null},g=s.a.createContext(p);function v(e){var t=e.children,n=e.injectFirst,a=void 0!==n&&n,l=e.disableGeneration,d=void 0!==l&&l,h=Object(o.a)(e,["children","injectFirst","disableGeneration"]),f=s.a.useContext(g),p=Object(i.a)({},f,{disableGeneration:d},h);if(!p.jss.options.insertionPoint&&a&&"undefined"!=typeof window){if(!r){var v=document.head;r=document.createComment("mui-inject-first"),v.insertBefore(r,v.firstChild)}p.jss=Object(u.c)({plugins:Object(c.a)().plugins,insertionPoint:r})}return s.a.createElement(g.Provider,{value:p},t)}},function(e,t,n){"use strict";n.d(t,"a",(function(){return a})),n.d(t,"g",(function(){return s})),n.d(t,"f",(function(){return l})),n.d(t,"b",(function(){return u})),n.d(t,"d",(function(){return c})),n.d(t,"c",(function(){return d})),n.d(t,"e",(function(){return h}));var r=n(25),i=n(85);function o(e){return"number"!=typeof e?e:"".concat(e,"px solid")}var a=Object(r.a)({prop:"border",themeKey:"borders",transform:o}),s=Object(r.a)({prop:"borderTop",themeKey:"borders",transform:o}),l=Object(r.a)({prop:"borderRight",themeKey:"borders",transform:o}),u=Object(r.a)({prop:"borderBottom",themeKey:"borders",transform:o}),c=Object(r.a)({prop:"borderLeft",themeKey:"borders",transform:o}),d=Object(r.a)({prop:"borderColor",themeKey:"palette"}),h=Object(r.a)({prop:"borderRadius",themeKey:"shape"}),f=Object(i.a)(a,s,l,u,c,d,h);t.h=f},function(e,t,n){"use strict";n.d(t,"f",(function(){return o})),n.d(t,"g",(function(){return a})),n.d(t,"j",(function(){return s})),n.d(t,"k",(function(){return l})),n.d(t,"b",(function(){return u})),n.d(t,"a",(function(){return c})),n.d(t,"n",(function(){return d})),n.d(t,"e",(function(){return h})),n.d(t,"h",(function(){return f})),n.d(t,"i",(function(){return p})),n.d(t,"c",(function(){return g})),n.d(t,"l",(function(){return v})),n.d(t,"m",(function(){return m}));var r=n(25),i=n(85),o=Object(r.a)({prop:"flexBasis"}),a=Object(r.a)({prop:"flexDirection"}),s=Object(r.a)({prop:"flexWrap"}),l=Object(r.a)({prop:"justifyContent"}),u=Object(r.a)({prop:"alignItems"}),c=Object(r.a)({prop:"alignContent"}),d=Object(r.a)({prop:"order"}),h=Object(r.a)({prop:"flex"}),f=Object(r.a)({prop:"flexGrow"}),p=Object(r.a)({prop:"flexShrink"}),g=Object(r.a)({prop:"alignSelf"}),v=Object(r.a)({prop:"justifyItems"}),m=Object(r.a)({prop:"justifySelf"}),y=Object(i.a)(o,a,s,l,u,c,d,h,f,p,g,v,m);t.d=y},function(e,t,n){"use strict";n.d(t,"h",(function(){return o})),n.d(t,"g",(function(){return a})),n.d(t,"j",(function(){return s})),n.d(t,"f",(function(){return l})),n.d(t,"i",(function(){return u})),n.d(t,"d",(function(){return c})),n.d(t,"c",(function(){return d})),n.d(t,"e",(function(){return h})),n.d(t,"l",(function(){return f})),n.d(t,"m",(function(){return p})),n.d(t,"k",(function(){return g})),n.d(t,"b",(function(){return v}));var r=n(25),i=n(85),o=Object(r.a)({prop:"gridGap"}),a=Object(r.a)({prop:"gridColumnGap"}),s=Object(r.a)({prop:"gridRowGap"}),l=Object(r.a)({prop:"gridColumn"}),u=Object(r.a)({prop:"gridRow"}),c=Object(r.a)({prop:"gridAutoFlow"}),d=Object(r.a)({prop:"gridAutoColumns"}),h=Object(r.a)({prop:"gridAutoRows"}),f=Object(r.a)({prop:"gridTemplateColumns"}),p=Object(r.a)({prop:"gridTemplateRows"}),g=Object(r.a)({prop:"gridTemplateAreas"}),v=Object(r.a)({prop:"gridArea"}),m=Object(i.a)(o,a,s,l,u,c,d,h,f,p,g,v);t.a=m},function(e,t,n){"use strict";n.d(t,"d",(function(){return o})),n.d(t,"g",(function(){return a})),n.d(t,"f",(function(){return s})),n.d(t,"e",(function(){return l})),n.d(t,"a",(function(){return u})),n.d(t,"c",(function(){return c}));var r=n(25),i=n(85),o=Object(r.a)({prop:"position"}),a=Object(r.a)({prop:"zIndex",themeKey:"zIndex"}),s=Object(r.a)({prop:"top"}),l=Object(r.a)({prop:"right"}),u=Object(r.a)({prop:"bottom"}),c=Object(r.a)({prop:"left"});t.b=Object(i.a)(o,a,s,l,u,c)},function(e,t,n){"use strict";n.d(t,"b",(function(){return o})),n.d(t,"a",(function(){return a}));var r=n(25),i=n(85),o=Object(r.a)({prop:"color",themeKey:"palette"}),a=Object(r.a)({prop:"bgcolor",cssProperty:"backgroundColor",themeKey:"palette"}),s=Object(i.a)(o,a);t.c=s},function(e,t,n){"use strict";n.d(t,"j",(function(){return a})),n.d(t,"e",(function(){return s})),n.d(t,"g",(function(){return l})),n.d(t,"c",(function(){return u})),n.d(t,"d",(function(){return c})),n.d(t,"f",(function(){return d})),n.d(t,"i",(function(){return h})),n.d(t,"h",(function(){return f})),n.d(t,"a",(function(){return p}));var r=n(25),i=n(85);function o(e){return e<=1?"".concat(100*e,"%"):e}var a=Object(r.a)({prop:"width",transform:o}),s=Object(r.a)({prop:"maxWidth",transform:o}),l=Object(r.a)({prop:"minWidth",transform:o}),u=Object(r.a)({prop:"height",transform:o}),c=Object(r.a)({prop:"maxHeight",transform:o}),d=Object(r.a)({prop:"minHeight",transform:o}),h=Object(r.a)({prop:"size",cssProperty:"width",transform:o}),f=Object(r.a)({prop:"size",cssProperty:"height",transform:o}),p=Object(r.a)({prop:"boxSizing"}),g=Object(i.a)(a,s,l,u,c,d,p);t.b=g},function(e,t,n){"use strict";n.d(t,"b",(function(){return o})),n.d(t,"c",(function(){return a})),n.d(t,"d",(function(){return s})),n.d(t,"e",(function(){return l})),n.d(t,"f",(function(){return u})),n.d(t,"g",(function(){return c})),n.d(t,"h",(function(){return d}));var r=n(25),i=n(85),o=Object(r.a)({prop:"fontFamily",themeKey:"typography"}),a=Object(r.a)({prop:"fontSize",themeKey:"typography"}),s=Object(r.a)({prop:"fontStyle",themeKey:"typography"}),l=Object(r.a)({prop:"fontWeight",themeKey:"typography"}),u=Object(r.a)({prop:"letterSpacing"}),c=Object(r.a)({prop:"lineHeight"}),d=Object(r.a)({prop:"textAlign"}),h=Object(i.a)(o,a,s,l,u,c,d);t.a=h},function(e,t,n){"use strict";var r=n(49),i=n(264),o=n(204);n.d(t,"a",(function(){return f}));var a,s,l={m:"margin",p:"padding"},u={t:"Top",r:"Right",b:"Bottom",l:"Left",x:["Left","Right"],y:["Top","Bottom"]},c={marginX:"mx",marginY:"my",paddingX:"px",paddingY:"py"},d=(a=function(e){if(e.length>2){if(!c[e])return[e];e=c[e]}var t=e.split(""),n=Object(r.a)(t,2),i=n[0],o=n[1],a=l[i],s=u[o]||"";return Array.isArray(s)?s.map((function(e){return a+e})):[a+s]},s={},function(e){return void 0===s[e]&&(s[e]=a(e)),s[e]}),h=["m","mt","mr","mb","ml","mx","my","p","pt","pr","pb","pl","px","py","margin","marginTop","marginRight","marginBottom","marginLeft","marginX","marginY","padding","paddingTop","paddingRight","paddingBottom","paddingLeft","paddingX","paddingY"];function f(e){var t=e.spacing||8;return"number"==typeof t?function(e){return t*e}:Array.isArray(t)?function(e){return t[e]}:"function"==typeof t?t:function(){}}function p(e,t){return function(n){return e.reduce((function(e,r){return e[r]=function(e,t){if("string"==typeof t||null==t)return t;var n=e(Math.abs(t));return t>=0?n:"number"==typeof n?-n:"-".concat(n)}(t,n),e}),{})}}function g(e){var t=f(e.theme);return Object.keys(e).map((function(n){if(-1===h.indexOf(n))return null;var r=p(d(n),t),o=e[n];return Object(i.b)(e,o,r)})).reduce(o.a,{})}g.propTypes={},g.filterProps=h;t.b=g},function(e,t,n){"use strict";n.r(t),function(e){var n=function(){if("undefined"!=typeof Map)return Map;function e(e,t){var n=-1;return e.some((function(e,r){return e[0]===t&&(n=r,!0)})),n}return function(){function t(){this.__entries__=[]}return Object.defineProperty(t.prototype,"size",{get:function(){return this.__entries__.length},enumerable:!0,configurable:!0}),t.prototype.get=function(t){var n=e(this.__entries__,t),r=this.__entries__[n];return r&&r[1]},t.prototype.set=function(t,n){var r=e(this.__entries__,t);~r?this.__entries__[r][1]=n:this.__entries__.push([t,n])},t.prototype.delete=function(t){var n=this.__entries__,r=e(n,t);~r&&n.splice(r,1)},t.prototype.has=function(t){return!!~e(this.__entries__,t)},t.prototype.clear=function(){this.__entries__.splice(0)},t.prototype.forEach=function(e,t){void 0===t&&(t=null);for(var n=0,r=this.__entries__;n<r.length;n++){var i=r[n];e.call(t,i[1],i[0])}},t}()}(),r="undefined"!=typeof window&&"undefined"!=typeof document&&window.document===document,i=void 0!==e&&e.Math===Math?e:"undefined"!=typeof self&&self.Math===Math?self:"undefined"!=typeof window&&window.Math===Math?window:Function("return this")(),o="function"==typeof requestAnimationFrame?requestAnimationFrame.bind(i):function(e){return setTimeout((function(){return e(Date.now())}),1e3/60)};var a=["top","right","bottom","left","width","height","size","weight"],s="undefined"!=typeof MutationObserver,l=function(){function e(){this.connected_=!1,this.mutationEventsAdded_=!1,this.mutationsObserver_=null,this.observers_=[],this.onTransitionEnd_=this.onTransitionEnd_.bind(this),this.refresh=function(e,t){var n=!1,r=!1,i=0;function a(){n&&(n=!1,e()),r&&l()}function s(){o(a)}function l(){var e=Date.now();if(n){if(e-i<2)return;r=!0}else n=!0,r=!1,setTimeout(s,t);i=e}return l}(this.refresh.bind(this),20)}return e.prototype.addObserver=function(e){~this.observers_.indexOf(e)||this.observers_.push(e),this.connected_||this.connect_()},e.prototype.removeObserver=function(e){var t=this.observers_,n=t.indexOf(e);~n&&t.splice(n,1),!t.length&&this.connected_&&this.disconnect_()},e.prototype.refresh=function(){this.updateObservers_()&&this.refresh()},e.prototype.updateObservers_=function(){var e=this.observers_.filter((function(e){return e.gatherActive(),e.hasActive()}));return e.forEach((function(e){return e.broadcastActive()})),e.length>0},e.prototype.connect_=function(){r&&!this.connected_&&(document.addEventListener("transitionend",this.onTransitionEnd_),window.addEventListener("resize",this.refresh),s?(this.mutationsObserver_=new MutationObserver(this.refresh),this.mutationsObserver_.observe(document,{attributes:!0,childList:!0,characterData:!0,subtree:!0})):(document.addEventListener("DOMSubtreeModified",this.refresh),this.mutationEventsAdded_=!0),this.connected_=!0)},e.prototype.disconnect_=function(){r&&this.connected_&&(document.removeEventListener("transitionend",this.onTransitionEnd_),window.removeEventListener("resize",this.refresh),this.mutationsObserver_&&this.mutationsObserver_.disconnect(),this.mutationEventsAdded_&&document.removeEventListener("DOMSubtreeModified",this.refresh),this.mutationsObserver_=null,this.mutationEventsAdded_=!1,this.connected_=!1)},e.prototype.onTransitionEnd_=function(e){var t=e.propertyName,n=void 0===t?"":t;a.some((function(e){return!!~n.indexOf(e)}))&&this.refresh()},e.getInstance=function(){return this.instance_||(this.instance_=new e),this.instance_},e.instance_=null,e}(),u=function(e,t){for(var n=0,r=Object.keys(t);n<r.length;n++){var i=r[n];Object.defineProperty(e,i,{value:t[i],enumerable:!1,writable:!1,configurable:!0})}return e},c=function(e){return e&&e.ownerDocument&&e.ownerDocument.defaultView||i},d=m(0,0,0,0);function h(e){return parseFloat(e)||0}function f(e){for(var t=[],n=1;n<arguments.length;n++)t[n-1]=arguments[n];return t.reduce((function(t,n){return t+h(e["border-"+n+"-width"])}),0)}function p(e){var t=e.clientWidth,n=e.clientHeight;if(!t&&!n)return d;var r=c(e).getComputedStyle(e),i=function(e){for(var t={},n=0,r=["top","right","bottom","left"];n<r.length;n++){var i=r[n],o=e["padding-"+i];t[i]=h(o)}return t}(r),o=i.left+i.right,a=i.top+i.bottom,s=h(r.width),l=h(r.height);if("border-box"===r.boxSizing&&(Math.round(s+o)!==t&&(s-=f(r,"left","right")+o),Math.round(l+a)!==n&&(l-=f(r,"top","bottom")+a)),!function(e){return e===c(e).document.documentElement}(e)){var u=Math.round(s+o)-t,p=Math.round(l+a)-n;1!==Math.abs(u)&&(s-=u),1!==Math.abs(p)&&(l-=p)}return m(i.left,i.top,s,l)}var g="undefined"!=typeof SVGGraphicsElement?function(e){return e instanceof c(e).SVGGraphicsElement}:function(e){return e instanceof c(e).SVGElement&&"function"==typeof e.getBBox};function v(e){return r?g(e)?function(e){var t=e.getBBox();return m(0,0,t.width,t.height)}(e):p(e):d}function m(e,t,n,r){return{x:e,y:t,width:n,height:r}}var y=function(){function e(e){this.broadcastWidth=0,this.broadcastHeight=0,this.contentRect_=m(0,0,0,0),this.target=e}return e.prototype.isActive=function(){var e=v(this.target);return this.contentRect_=e,e.width!==this.broadcastWidth||e.height!==this.broadcastHeight},e.prototype.broadcastRect=function(){var e=this.contentRect_;return this.broadcastWidth=e.width,this.broadcastHeight=e.height,e},e}(),b=function(e,t){var n,r,i,o,a,s,l,c=(r=(n=t).x,i=n.y,o=n.width,a=n.height,s="undefined"!=typeof DOMRectReadOnly?DOMRectReadOnly:Object,l=Object.create(s.prototype),u(l,{x:r,y:i,width:o,height:a,top:i,right:r+o,bottom:a+i,left:r}),l);u(this,{target:e,contentRect:c})},C=function(){function e(e,t,r){if(this.activeObservations_=[],this.observations_=new n,"function"!=typeof e)throw new TypeError("The callback provided as parameter 1 is not a function.");this.callback_=e,this.controller_=t,this.callbackCtx_=r}return e.prototype.observe=function(e){if(!arguments.length)throw new TypeError("1 argument required, but only 0 present.");if("undefined"!=typeof Element&&Element instanceof Object){if(!(e instanceof c(e).Element))throw new TypeError('parameter 1 is not of type "Element".');var t=this.observations_;t.has(e)||(t.set(e,new y(e)),this.controller_.addObserver(this),this.controller_.refresh())}},e.prototype.unobserve=function(e){if(!arguments.length)throw new TypeError("1 argument required, but only 0 present.");if("undefined"!=typeof Element&&Element instanceof Object){if(!(e instanceof c(e).Element))throw new TypeError('parameter 1 is not of type "Element".');var t=this.observations_;t.has(e)&&(t.delete(e),t.size||this.controller_.removeObserver(this))}},e.prototype.disconnect=function(){this.clearActive(),this.observations_.clear(),this.controller_.removeObserver(this)},e.prototype.gatherActive=function(){var e=this;this.clearActive(),this.observations_.forEach((function(t){t.isActive()&&e.activeObservations_.push(t)}))},e.prototype.broadcastActive=function(){if(this.hasActive()){var e=this.callbackCtx_,t=this.activeObservations_.map((function(e){return new b(e.target,e.broadcastRect())}));this.callback_.call(e,t,e),this.clearActive()}},e.prototype.clearActive=function(){this.activeObservations_.splice(0)},e.prototype.hasActive=function(){return this.activeObservations_.length>0},e}(),A="undefined"!=typeof WeakMap?new WeakMap:new n,_=function e(t){if(!(this instanceof e))throw new TypeError("Cannot call a class as a function.");if(!arguments.length)throw new TypeError("1 argument required, but only 0 present.");var n=l.getInstance(),r=new C(t,n,this);A.set(this,r)};["observe","unobserve","disconnect"].forEach((function(e){_.prototype[e]=function(){var t;return(t=A.get(this))[e].apply(t,arguments)}}));var I=void 0!==i.ResizeObserver?i.ResizeObserver:_;t.default=I}.call(this,n(61))},function(e,t,n){"use strict";n.d(t,"a",(function(){return r}));var r={CENTER:"center",LEFT:"left",RIGHT:"right"}},function(e,t,n){"use strict";n.d(t,"a",(function(){return i})),n.d(t,"b",(function(){return o}));const r=/(\$\$?|\\(?:begin|end)\{[a-z]*\*?\}|\\[{}$]|[{}]|(?:\n\s*)+|@@\d+@@|\\\\(?:\(|\)|\[|\]))/i;function i(e){const t=[];let n,i=null,o=null,s=null,l=0;/`/.test(e)?(e=e.replace(/~/g,"~T").replace(/(^|[^\\])(`+)([^\n]*?[^`\n])\2(?!`)/gm,e=>e.replace(/\$/g,"~D")),n=e=>e.replace(/~([TD])/g,(e,t)=>"T"===t?"~":"$")):n=e=>e;let u=e.replace(/\r\n?/g,"\n").split(r);for(let e=1,r=u.length;e<r;e+=2){const r=u[e];"@"===r.charAt(0)?(u[e]="@@"+t.length+"@@",t.push(r)):null!==i?r===o?l?s=e:(u=a(i,e,n,t,u),i=null,o=null,s=null):r.match(/\n.*\n/)?(null!==s&&(e=s,u=a(i,e,n,t,u)),i=null,o=null,s=null,l=0):"{"===r?l++:"}"===r&&l&&l--:"$"===r||"$$"===r?(i=e,o=r,l=0):"\\\\("===r||"\\\\["===r?(i=e,o="("===r.slice(-1)?"\\\\)":"\\\\]",l=0):"begin"===r.substr(1,5)&&(i=e,o="\\end"+r.substr(6),l=0)}return null!==i&&null!==s&&(u=a(i,s,n,t,u),i=null,o=null,s=null),{text:n(u.join("")),math:t}}function o(e,t){return e.replace(/@@(\d+)@@/g,(e,n)=>{let r=t[n];return"\\\\("===r.substr(0,3)&&"\\\\)"===r.substr(r.length-3)?r="\\("+r.substring(3,r.length-3)+"\\)":"\\\\["===r.substr(0,3)&&"\\\\]"===r.substr(r.length-3)&&(r="\\["+r.substring(3,r.length-3)+"\\]"),r})}function a(e,t,n,r,i){let o=i.slice(e,t+1).join("").replace(/&/g,"&amp;").replace(/</g,"&lt;").replace(/>/g,"&gt;");for(navigator&&"Microsoft Internet Explorer"===navigator.appName&&(o=o.replace(/(%[^\n]*)\n/g,"$1<br/>\n"));t>e;)i[t]="",t--;return i[e]="@@"+r.length+"@@",n&&(o=n(o)),r.push(o),i}},function(e,t,n){"use strict";t.a=function(e,t){}},function(e,t,n){"use strict";n.d(t,"a",(function(){return u})),n.d(t,"b",(function(){return p}));var r=n(99),i=n(80),o=n(74),a=n(453),s=n(59),l=n(202);function u(e,t){t?e.setAttribute("aria-hidden","true"):e.removeAttribute("aria-hidden")}function c(e){return parseInt(window.getComputedStyle(e)["padding-right"],10)||0}function d(e,t,n){var r=arguments.length>3&&void 0!==arguments[3]?arguments[3]:[],i=arguments.length>4?arguments[4]:void 0,a=[t,n].concat(Object(o.a)(r)),s=["TEMPLATE","SCRIPT","STYLE"];[].forEach.call(e.children,(function(e){1===e.nodeType&&-1===a.indexOf(e)&&-1===s.indexOf(e.tagName)&&u(e,i)}))}function h(e,t){var n=-1;return e.some((function(e,r){return!!t(e)&&(n=r,!0)})),n}function f(e,t){var n,r=[],i=[],o=e.container;if(!t.disableScrollLock){if(function(e){var t=Object(s.a)(e);return t.body===e?Object(l.a)(t).innerWidth>t.documentElement.clientWidth:e.scrollHeight>e.clientHeight}(o)){var u=Object(a.a)();r.push({value:o.style.paddingRight,key:"padding-right",el:o}),o.style["padding-right"]="".concat(c(o)+u,"px"),n=Object(s.a)(o).querySelectorAll(".mui-fixed"),[].forEach.call(n,(function(e){i.push(e.style.paddingRight),e.style.paddingRight="".concat(c(e)+u,"px")}))}var d=o.parentElement,h="HTML"===d.nodeName&&"scroll"===window.getComputedStyle(d)["overflow-y"]?d:o;r.push({value:h.style.overflow,key:"overflow",el:h}),h.style.overflow="hidden"}return function(){n&&[].forEach.call(n,(function(e,t){i[t]?e.style.paddingRight=i[t]:e.style.removeProperty("padding-right")})),r.forEach((function(e){var t=e.value,n=e.el,r=e.key;t?n.style.setProperty(r,t):n.style.removeProperty(r)}))}}var p=function(){function e(){Object(r.a)(this,e),this.modals=[],this.containers=[]}return Object(i.a)(e,[{key:"add",value:function(e,t){var n=this.modals.indexOf(e);if(-1!==n)return n;n=this.modals.length,this.modals.push(e),e.modalRef&&u(e.modalRef,!1);var r=function(e){var t=[];return[].forEach.call(e.children,(function(e){e.getAttribute&&"true"===e.getAttribute("aria-hidden")&&t.push(e)})),t}(t);d(t,e.mountNode,e.modalRef,r,!0);var i=h(this.containers,(function(e){return e.container===t}));return-1!==i?(this.containers[i].modals.push(e),n):(this.containers.push({modals:[e],container:t,restore:null,hiddenSiblingNodes:r}),n)}},{key:"mount",value:function(e,t){var n=h(this.containers,(function(t){return-1!==t.modals.indexOf(e)})),r=this.containers[n];r.restore||(r.restore=f(r,t))}},{key:"remove",value:function(e){var t=this.modals.indexOf(e);if(-1===t)return t;var n=h(this.containers,(function(t){return-1!==t.modals.indexOf(e)})),r=this.containers[n];if(r.modals.splice(r.modals.indexOf(e),1),this.modals.splice(t,1),0===r.modals.length)r.restore&&r.restore(),e.modalRef&&u(e.modalRef,!0),d(r.container,e.mountNode,e.modalRef,r.hiddenSiblingNodes,!1),this.containers.splice(n,1);else{var i=r.modals[r.modals.length-1];i.modalRef&&u(i.modalRef,!1)}return t}},{key:"isTopModal",value:function(e){return this.modals.length>0&&this.modals[this.modals.length-1]===e}}]),e}()},function(e,t,n){"use strict";n.r(t),n.d(t,"LineMaterial",(function(){return i}));var r=n(20);r.UniformsLib.line={linewidth:{value:1},resolution:{value:new r.Vector2(1,1)},dashScale:{value:1},dashSize:{value:1},gapSize:{value:1},opacity:{value:1}},r.ShaderLib.line={uniforms:r.UniformsUtils.merge([r.UniformsLib.common,r.UniformsLib.fog,r.UniformsLib.line]),vertexShader:"\n\t\t#include <common>\n\t\t#include <color_pars_vertex>\n\t\t#include <fog_pars_vertex>\n\t\t#include <logdepthbuf_pars_vertex>\n\t\t#include <clipping_planes_pars_vertex>\n\n\t\tuniform float linewidth;\n\t\tuniform vec2 resolution;\n\n\t\tattribute vec3 instanceStart;\n\t\tattribute vec3 instanceEnd;\n\n\t\tattribute vec3 instanceColorStart;\n\t\tattribute vec3 instanceColorEnd;\n\n\t\tvarying vec2 vUv;\n\n\t\t#ifdef USE_DASH\n\n\t\t\tuniform float dashScale;\n\t\t\tattribute float instanceDistanceStart;\n\t\t\tattribute float instanceDistanceEnd;\n\t\t\tvarying float vLineDistance;\n\n\t\t#endif\n\n\t\tvoid trimSegment( const in vec4 start, inout vec4 end ) {\n\n\t\t\t// trim end segment so it terminates between the camera plane and the near plane\n\n\t\t\t// conservative estimate of the near plane\n\t\t\tfloat a = projectionMatrix[ 2 ][ 2 ]; // 3nd entry in 3th column\n\t\t\tfloat b = projectionMatrix[ 3 ][ 2 ]; // 3nd entry in 4th column\n\t\t\tfloat nearEstimate = - 0.5 * b / a;\n\n\t\t\tfloat alpha = ( nearEstimate - start.z ) / ( end.z - start.z );\n\n\t\t\tend.xyz = mix( start.xyz, end.xyz, alpha );\n\n\t\t}\n\n\t\tvoid main() {\n\n\t\t\t#ifdef USE_COLOR\n\n\t\t\t\tvColor.xyz = ( position.y < 0.5 ) ? instanceColorStart : instanceColorEnd;\n\n\t\t\t#endif\n\n\t\t\t#ifdef USE_DASH\n\n\t\t\t\tvLineDistance = ( position.y < 0.5 ) ? dashScale * instanceDistanceStart : dashScale * instanceDistanceEnd;\n\n\t\t\t#endif\n\n\t\t\tfloat aspect = resolution.x / resolution.y;\n\n\t\t\tvUv = uv;\n\n\t\t\t// camera space\n\t\t\tvec4 start = modelViewMatrix * vec4( instanceStart, 1.0 );\n\t\t\tvec4 end = modelViewMatrix * vec4( instanceEnd, 1.0 );\n\n\t\t\t// special case for perspective projection, and segments that terminate either in, or behind, the camera plane\n\t\t\t// clearly the gpu firmware has a way of addressing this issue when projecting into ndc space\n\t\t\t// but we need to perform ndc-space calculations in the shader, so we must address this issue directly\n\t\t\t// perhaps there is a more elegant solution -- WestLangley\n\n\t\t\tbool perspective = ( projectionMatrix[ 2 ][ 3 ] == - 1.0 ); // 4th entry in the 3rd column\n\n\t\t\tif ( perspective ) {\n\n\t\t\t\tif ( start.z < 0.0 && end.z >= 0.0 ) {\n\n\t\t\t\t\ttrimSegment( start, end );\n\n\t\t\t\t} else if ( end.z < 0.0 && start.z >= 0.0 ) {\n\n\t\t\t\t\ttrimSegment( end, start );\n\n\t\t\t\t}\n\n\t\t\t}\n\n\t\t\t// clip space\n\t\t\tvec4 clipStart = projectionMatrix * start;\n\t\t\tvec4 clipEnd = projectionMatrix * end;\n\n\t\t\t// ndc space\n\t\t\tvec2 ndcStart = clipStart.xy / clipStart.w;\n\t\t\tvec2 ndcEnd = clipEnd.xy / clipEnd.w;\n\n\t\t\t// direction\n\t\t\tvec2 dir = ndcEnd - ndcStart;\n\n\t\t\t// account for clip-space aspect ratio\n\t\t\tdir.x *= aspect;\n\t\t\tdir = normalize( dir );\n\n\t\t\t// perpendicular to dir\n\t\t\tvec2 offset = vec2( dir.y, - dir.x );\n\n\t\t\t// undo aspect ratio adjustment\n\t\t\tdir.x /= aspect;\n\t\t\toffset.x /= aspect;\n\n\t\t\t// sign flip\n\t\t\tif ( position.x < 0.0 ) offset *= - 1.0;\n\n\t\t\t// endcaps\n\t\t\tif ( position.y < 0.0 ) {\n\n\t\t\t\toffset += - dir;\n\n\t\t\t} else if ( position.y > 1.0 ) {\n\n\t\t\t\toffset += dir;\n\n\t\t\t}\n\n\t\t\t// adjust for linewidth\n\t\t\toffset *= linewidth;\n\n\t\t\t// adjust for clip-space to screen-space conversion // maybe resolution should be based on viewport ...\n\t\t\toffset /= resolution.y;\n\n\t\t\t// select end\n\t\t\tvec4 clip = ( position.y < 0.5 ) ? clipStart : clipEnd;\n\n\t\t\t// back to clip space\n\t\t\toffset *= clip.w;\n\n\t\t\tclip.xy += offset;\n\n\t\t\tgl_Position = clip;\n\n\t\t\tvec4 mvPosition = ( position.y < 0.5 ) ? start : end; // this is an approximation\n\n\t\t\t#include <logdepthbuf_vertex>\n\t\t\t#include <clipping_planes_vertex>\n\t\t\t#include <fog_vertex>\n\n\t\t}\n\t\t",fragmentShader:"\n\t\tuniform vec3 diffuse;\n\t\tuniform float opacity;\n\n\t\t#ifdef USE_DASH\n\n\t\t\tuniform float dashSize;\n\t\t\tuniform float gapSize;\n\n\t\t#endif\n\n\t\tvarying float vLineDistance;\n\n\t\t#include <common>\n\t\t#include <color_pars_fragment>\n\t\t#include <fog_pars_fragment>\n\t\t#include <logdepthbuf_pars_fragment>\n\t\t#include <clipping_planes_pars_fragment>\n\n\t\tvarying vec2 vUv;\n\n\t\tvoid main() {\n\n\t\t\t#include <clipping_planes_fragment>\n\n\t\t\t#ifdef USE_DASH\n\n\t\t\t\tif ( vUv.y < - 1.0 || vUv.y > 1.0 ) discard; // discard endcaps\n\n\t\t\t\tif ( mod( vLineDistance, dashSize + gapSize ) > dashSize ) discard; // todo - FIX\n\n\t\t\t#endif\n\n\t\t\tif ( abs( vUv.y ) > 1.0 ) {\n\n\t\t\t\tfloat a = vUv.x;\n\t\t\t\tfloat b = ( vUv.y > 0.0 ) ? vUv.y - 1.0 : vUv.y + 1.0;\n\t\t\t\tfloat len2 = a * a + b * b;\n\n\t\t\t\tif ( len2 > 1.0 ) discard;\n\n\t\t\t}\n\n\t\t\tvec4 diffuseColor = vec4( diffuse, opacity );\n\n\t\t\t#include <logdepthbuf_fragment>\n\t\t\t#include <color_fragment>\n\n\t\t\tgl_FragColor = vec4( diffuseColor.rgb, diffuseColor.a );\n\n\t\t\t#include <tonemapping_fragment>\n\t\t\t#include <encodings_fragment>\n\t\t\t#include <fog_fragment>\n\t\t\t#include <premultiplied_alpha_fragment>\n\n\t\t}\n\t\t"};var i=function(e){r.ShaderMaterial.call(this,{type:"LineMaterial",uniforms:r.UniformsUtils.clone(r.ShaderLib.line.uniforms),vertexShader:r.ShaderLib.line.vertexShader,fragmentShader:r.ShaderLib.line.fragmentShader,clipping:!0}),this.dashed=!1,Object.defineProperties(this,{color:{enumerable:!0,get:function(){return this.uniforms.diffuse.value},set:function(e){this.uniforms.diffuse.value=e}},linewidth:{enumerable:!0,get:function(){return this.uniforms.linewidth.value},set:function(e){this.uniforms.linewidth.value=e}},dashScale:{enumerable:!0,get:function(){return this.uniforms.dashScale.value},set:function(e){this.uniforms.dashScale.value=e}},dashSize:{enumerable:!0,get:function(){return this.uniforms.dashSize.value},set:function(e){this.uniforms.dashSize.value=e}},gapSize:{enumerable:!0,get:function(){return this.uniforms.gapSize.value},set:function(e){this.uniforms.gapSize.value=e}},opacity:{enumerable:!0,get:function(){return this.uniforms.opacity.value},set:function(e){this.uniforms.opacity.value=e}},resolution:{enumerable:!0,get:function(){return this.uniforms.resolution.value},set:function(e){this.uniforms.resolution.value.copy(e)}}}),this.setValues(e)};i.prototype=Object.create(r.ShaderMaterial.prototype),i.prototype.constructor=i,i.prototype.isLineMaterial=!0},function(e,t,n){"use strict";n.r(t);var r=n(101),i=n(0);var o=function(){function e(e){this.isSpeedy=void 0===e.speedy||e.speedy,this.tags=[],this.ctr=0,this.nonce=e.nonce,this.key=e.key,this.container=e.container,this.before=null}var t=e.prototype;return t.insert=function(e){if(this.ctr%(this.isSpeedy?65e3:1)==0){var t,n=function(e){var t=document.createElement("style");return t.setAttribute("data-emotion",e.key),void 0!==e.nonce&&t.setAttribute("nonce",e.nonce),t.appendChild(document.createTextNode("")),t}(this);t=0===this.tags.length?this.before:this.tags[this.tags.length-1].nextSibling,this.container.insertBefore(n,t),this.tags.push(n)}var r=this.tags[this.tags.length-1];if(this.isSpeedy){var i=function(e){if(e.sheet)return e.sheet;for(var t=0;t<document.styleSheets.length;t++)if(document.styleSheets[t].ownerNode===e)return document.styleSheets[t]}(r);try{var o=105===e.charCodeAt(1)&&64===e.charCodeAt(0);i.insertRule(e,o?0:i.cssRules.length)}catch(e){0}}else r.appendChild(document.createTextNode(e));this.ctr++},t.flush=function(){this.tags.forEach((function(e){return e.parentNode.removeChild(e)})),this.tags=[],this.ctr=0},e}();var a=function(e){function t(e,t,r){var i=t.trim().split(p);t=i;var o=i.length,a=e.length;switch(a){case 0:case 1:var s=0;for(e=0===a?"":e[0]+" ";s<o;++s)t[s]=n(e,t[s],r).trim();break;default:var l=s=0;for(t=[];s<o;++s)for(var u=0;u<a;++u)t[l++]=n(e[u]+" ",i[s],r).trim()}return t}function n(e,t,n){var r=t.charCodeAt(0);switch(33>r&&(r=(t=t.trim()).charCodeAt(0)),r){case 38:return t.replace(g,"$1"+e.trim());case 58:return e.trim()+t.replace(g,"$1"+e.trim());default:if(0<1*n&&0<t.indexOf("\f"))return t.replace(g,(58===e.charCodeAt(0)?"":"$1")+e.trim())}return e+t}function r(e,t,n,o){var a=e+";",s=2*t+3*n+4*o;if(944===s){e=a.indexOf(":",9)+1;var l=a.substring(e,a.length-1).trim();return l=a.substring(0,e).trim()+l+";",1===E||2===E&&i(l,1)?"-webkit-"+l+l:l}if(0===E||2===E&&!i(a,1))return a;switch(s){case 1015:return 97===a.charCodeAt(10)?"-webkit-"+a+a:a;case 951:return 116===a.charCodeAt(3)?"-webkit-"+a+a:a;case 963:return 110===a.charCodeAt(5)?"-webkit-"+a+a:a;case 1009:if(100!==a.charCodeAt(4))break;case 969:case 942:return"-webkit-"+a+a;case 978:return"-webkit-"+a+"-moz-"+a+a;case 1019:case 983:return"-webkit-"+a+"-moz-"+a+"-ms-"+a+a;case 883:if(45===a.charCodeAt(8))return"-webkit-"+a+a;if(0<a.indexOf("image-set(",11))return a.replace(w,"$1-webkit-$2")+a;break;case 932:if(45===a.charCodeAt(4))switch(a.charCodeAt(5)){case 103:return"-webkit-box-"+a.replace("-grow","")+"-webkit-"+a+"-ms-"+a.replace("grow","positive")+a;case 115:return"-webkit-"+a+"-ms-"+a.replace("shrink","negative")+a;case 98:return"-webkit-"+a+"-ms-"+a.replace("basis","preferred-size")+a}return"-webkit-"+a+"-ms-"+a+a;case 964:return"-webkit-"+a+"-ms-flex-"+a+a;case 1023:if(99!==a.charCodeAt(8))break;return"-webkit-box-pack"+(l=a.substring(a.indexOf(":",15)).replace("flex-","").replace("space-between","justify"))+"-webkit-"+a+"-ms-flex-pack"+l+a;case 1005:return h.test(a)?a.replace(d,":-webkit-")+a.replace(d,":-moz-")+a:a;case 1e3:switch(t=(l=a.substring(13).trim()).indexOf("-")+1,l.charCodeAt(0)+l.charCodeAt(t)){case 226:l=a.replace(b,"tb");break;case 232:l=a.replace(b,"tb-rl");break;case 220:l=a.replace(b,"lr");break;default:return a}return"-webkit-"+a+"-ms-"+l+a;case 1017:if(-1===a.indexOf("sticky",9))break;case 975:switch(t=(a=e).length-10,s=(l=(33===a.charCodeAt(t)?a.substring(0,t):a).substring(e.indexOf(":",7)+1).trim()).charCodeAt(0)+(0|l.charCodeAt(7))){case 203:if(111>l.charCodeAt(8))break;case 115:a=a.replace(l,"-webkit-"+l)+";"+a;break;case 207:case 102:a=a.replace(l,"-webkit-"+(102<s?"inline-":"")+"box")+";"+a.replace(l,"-webkit-"+l)+";"+a.replace(l,"-ms-"+l+"box")+";"+a}return a+";";case 938:if(45===a.charCodeAt(5))switch(a.charCodeAt(6)){case 105:return l=a.replace("-items",""),"-webkit-"+a+"-webkit-box-"+l+"-ms-flex-"+l+a;case 115:return"-webkit-"+a+"-ms-flex-item-"+a.replace(_,"")+a;default:return"-webkit-"+a+"-ms-flex-line-pack"+a.replace("align-content","").replace(_,"")+a}break;case 973:case 989:if(45!==a.charCodeAt(3)||122===a.charCodeAt(4))break;case 931:case 953:if(!0===x.test(e))return 115===(l=e.substring(e.indexOf(":")+1)).charCodeAt(0)?r(e.replace("stretch","fill-available"),t,n,o).replace(":fill-available",":stretch"):a.replace(l,"-webkit-"+l)+a.replace(l,"-moz-"+l.replace("fill-",""))+a;break;case 962:if(a="-webkit-"+a+(102===a.charCodeAt(5)?"-ms-"+a:"")+a,211===n+o&&105===a.charCodeAt(13)&&0<a.indexOf("transform",10))return a.substring(0,a.indexOf(";",27)+1).replace(f,"$1-webkit-$2")+a}return a}function i(e,t){var n=e.indexOf(1===t?":":"{"),r=e.substring(0,3!==t?n:10);return n=e.substring(n+1,e.length-1),D(2!==t?r:r.replace(I,"$1"),n,t)}function o(e,t){var n=r(t,t.charCodeAt(0),t.charCodeAt(1),t.charCodeAt(2));return n!==t+";"?n.replace(A," or ($1)").substring(4):"("+t+")"}function a(e,t,n,r,i,o,a,s,u,c){for(var d,h=0,f=t;h<L;++h)switch(d=k[h].call(l,e,f,n,r,i,o,a,s,u,c)){case void 0:case!1:case!0:case null:break;default:f=d}if(f!==t)return f}function s(e){return void 0!==(e=e.prefix)&&(D=null,e?"function"!=typeof e?E=1:(E=2,D=e):E=0),s}function l(e,n){var s=e;if(33>s.charCodeAt(0)&&(s=s.trim()),s=[s],0<L){var l=a(-1,n,s,s,S,M,0,0,0,0);void 0!==l&&"string"==typeof l&&(n=l)}var d=function e(n,s,l,d,h){for(var f,p,g,b,A,_=0,I=0,x=0,w=0,k=0,D=0,N=g=f=0,j=0,R=0,z=0,F=0,B=l.length,H=B-1,U="",V="",Y="",W="";j<B;){if(p=l.charCodeAt(j),j===H&&0!==I+w+x+_&&(0!==I&&(p=47===I?10:47),w=x=_=0,B++,H++),0===I+w+x+_){if(j===H&&(0<R&&(U=U.replace(c,"")),0<U.trim().length)){switch(p){case 32:case 9:case 59:case 13:case 10:break;default:U+=l.charAt(j)}p=59}switch(p){case 123:for(f=(U=U.trim()).charCodeAt(0),g=1,F=++j;j<B;){switch(p=l.charCodeAt(j)){case 123:g++;break;case 125:g--;break;case 47:switch(p=l.charCodeAt(j+1)){case 42:case 47:e:{for(N=j+1;N<H;++N)switch(l.charCodeAt(N)){case 47:if(42===p&&42===l.charCodeAt(N-1)&&j+2!==N){j=N+1;break e}break;case 10:if(47===p){j=N+1;break e}}j=N}}break;case 91:p++;case 40:p++;case 34:case 39:for(;j++<H&&l.charCodeAt(j)!==p;);}if(0===g)break;j++}switch(g=l.substring(F,j),0===f&&(f=(U=U.replace(u,"").trim()).charCodeAt(0)),f){case 64:switch(0<R&&(U=U.replace(c,"")),p=U.charCodeAt(1)){case 100:case 109:case 115:case 45:R=s;break;default:R=O}if(F=(g=e(s,R,g,p,h+1)).length,0<L&&(A=a(3,g,R=t(O,U,z),s,S,M,F,p,h,d),U=R.join(""),void 0!==A&&0===(F=(g=A.trim()).length)&&(p=0,g="")),0<F)switch(p){case 115:U=U.replace(C,o);case 100:case 109:case 45:g=U+"{"+g+"}";break;case 107:g=(U=U.replace(v,"$1 $2"))+"{"+g+"}",g=1===E||2===E&&i("@"+g,3)?"@-webkit-"+g+"@"+g:"@"+g;break;default:g=U+g,112===d&&(V+=g,g="")}else g="";break;default:g=e(s,t(s,U,z),g,d,h+1)}Y+=g,g=z=R=N=f=0,U="",p=l.charCodeAt(++j);break;case 125:case 59:if(1<(F=(U=(0<R?U.replace(c,""):U).trim()).length))switch(0===N&&(f=U.charCodeAt(0),45===f||96<f&&123>f)&&(F=(U=U.replace(" ",":")).length),0<L&&void 0!==(A=a(1,U,s,n,S,M,V.length,d,h,d))&&0===(F=(U=A.trim()).length)&&(U="\0\0"),f=U.charCodeAt(0),p=U.charCodeAt(1),f){case 0:break;case 64:if(105===p||99===p){W+=U+l.charAt(j);break}default:58!==U.charCodeAt(F-1)&&(V+=r(U,f,p,U.charCodeAt(2)))}z=R=N=f=0,U="",p=l.charCodeAt(++j)}}switch(p){case 13:case 10:47===I?I=0:0===1+f&&107!==d&&0<U.length&&(R=1,U+="\0"),0<L*P&&a(0,U,s,n,S,M,V.length,d,h,d),M=1,S++;break;case 59:case 125:if(0===I+w+x+_){M++;break}default:switch(M++,b=l.charAt(j),p){case 9:case 32:if(0===w+_+I)switch(k){case 44:case 58:case 9:case 32:b="";break;default:32!==p&&(b=" ")}break;case 0:b="\\0";break;case 12:b="\\f";break;case 11:b="\\v";break;case 38:0===w+I+_&&(R=z=1,b="\f"+b);break;case 108:if(0===w+I+_+T&&0<N)switch(j-N){case 2:112===k&&58===l.charCodeAt(j-3)&&(T=k);case 8:111===D&&(T=D)}break;case 58:0===w+I+_&&(N=j);break;case 44:0===I+x+w+_&&(R=1,b+="\r");break;case 34:case 39:0===I&&(w=w===p?0:0===w?p:w);break;case 91:0===w+I+x&&_++;break;case 93:0===w+I+x&&_--;break;case 41:0===w+I+_&&x--;break;case 40:if(0===w+I+_){if(0===f)switch(2*k+3*D){case 533:break;default:f=1}x++}break;case 64:0===I+x+w+_+N+g&&(g=1);break;case 42:case 47:if(!(0<w+_+x))switch(I){case 0:switch(2*p+3*l.charCodeAt(j+1)){case 235:I=47;break;case 220:F=j,I=42}break;case 42:47===p&&42===k&&F+2!==j&&(33===l.charCodeAt(F+2)&&(V+=l.substring(F,j+1)),b="",I=0)}}0===I&&(U+=b)}D=k,k=p,j++}if(0<(F=V.length)){if(R=s,0<L&&(void 0!==(A=a(2,V,R,n,S,M,F,d,h,d))&&0===(V=A).length))return W+V+Y;if(V=R.join(",")+"{"+V+"}",0!=E*T){switch(2!==E||i(V,2)||(T=0),T){case 111:V=V.replace(y,":-moz-$1")+V;break;case 112:V=V.replace(m,"::-webkit-input-$1")+V.replace(m,"::-moz-$1")+V.replace(m,":-ms-input-$1")+V}T=0}}return W+V+Y}(O,s,n,0,0);return 0<L&&(void 0!==(l=a(-2,d,s,s,S,M,d.length,0,0,0))&&(d=l)),"",T=0,M=S=1,d}var u=/^\0+/g,c=/[\0\r\f]/g,d=/: */g,h=/zoo|gra/,f=/([,: ])(transform)/g,p=/,\r+?/g,g=/([\t\r\n ])*\f?&/g,v=/@(k\w+)\s*(\S*)\s*/,m=/::(place)/g,y=/:(read-only)/g,b=/[svh]\w+-[tblr]{2}/,C=/\(\s*(.*)\s*\)/g,A=/([\s\S]*?);/g,_=/-self|flex-/g,I=/[^]*?(:[rp][el]a[\w-]+)[^]*/,x=/stretch|:\s*\w+\-(?:conte|avail)/,w=/([^-])(image-set\()/,M=1,S=1,T=0,E=1,O=[],k=[],L=0,D=null,P=0;return l.use=function e(t){switch(t){case void 0:case null:L=k.length=0;break;default:if("function"==typeof t)k[L++]=t;else if("object"==typeof t)for(var n=0,r=t.length;n<r;++n)e(t[n]);else P=0|!!t}return e},l.set=s,void 0!==e&&s(e),l};function s(e){e&&l.current.insert(e+"}")}var l={current:null},u=function(e,t,n,r,i,o,a,u,c,d){switch(e){case 1:switch(t.charCodeAt(0)){case 64:return l.current.insert(t+";"),"";case 108:if(98===t.charCodeAt(2))return""}break;case 2:if(0===u)return t+"/*|*/";break;case 3:switch(u){case 102:case 112:return l.current.insert(n[0]+t),"";default:return t+(0===d?"/*|*/":"")}case-2:t.split("/*|*/}").forEach(s)}},c=function(e){void 0===e&&(e={});var t,n=e.key||"css";void 0!==e.prefix&&(t={prefix:e.prefix});var r=new a(t);var i,s={};i=e.container||document.head;var c,d=document.querySelectorAll("style[data-emotion-"+n+"]");Array.prototype.forEach.call(d,(function(e){e.getAttribute("data-emotion-"+n).split(" ").forEach((function(e){s[e]=!0})),e.parentNode!==i&&i.appendChild(e)})),r.use(e.stylisPlugins)(u),c=function(e,t,n,i){var o=t.name;l.current=n,r(e,t.styles),i&&(h.inserted[o]=!0)};var h={key:n,sheet:new o({key:n,container:i,nonce:e.nonce,speedy:e.speedy}),nonce:e.nonce,inserted:s,registered:{},insert:c};return h},d=(n(224),n(172)),h=n(170),f=Object.prototype.hasOwnProperty,p=Object(i.createContext)("undefined"!=typeof HTMLElement?c():null),g=Object(i.createContext)({}),v=p.Provider,m=function(e){var t=function(t,n){return Object(i.createElement)(p.Consumer,null,(function(r){return e(t,r,n)}))};return Object(i.forwardRef)(t)},y="__EMOTION_TYPE_PLEASE_DO_NOT_USE__",b=function(e,t){var n={};for(var r in t)f.call(t,r)&&(n[r]=t[r]);return n[y]=e,n},C=function(e,t,n,r){var o=null===n?t.css:t.css(n);"string"==typeof o&&void 0!==e.registered[o]&&(o=e.registered[o]);var a=t[y],s=[o],l="";"string"==typeof t.className?l=Object(d.a)(e.registered,s,t.className):null!=t.className&&(l=t.className+" ");var u=Object(h.a)(s);Object(d.b)(e,u,"string"==typeof a);l+=e.key+"-"+u.name;var c={};for(var p in t)f.call(t,p)&&"css"!==p&&p!==y&&(c[p]=t[p]);return c.ref=r,c.className=l,Object(i.createElement)(a,c)},A=m((function(e,t,n){return"function"==typeof e.css?Object(i.createElement)(g.Consumer,null,(function(r){return C(t,e,r,n)})):C(t,e,null,n)}));var _=function(){for(var e=arguments.length,t=new Array(e),n=0;n<e;n++)t[n]=arguments[n];return Object(h.a)(t)};n.d(t,"ClassNames",(function(){return E})),n.d(t,"Global",(function(){return x})),n.d(t,"createElement",(function(){return I})),n.d(t,"jsx",(function(){return I})),n.d(t,"keyframes",(function(){return M})),n.d(t,"CacheProvider",(function(){return v})),n.d(t,"ThemeContext",(function(){return g})),n.d(t,"withEmotionCache",(function(){return m})),n.d(t,"css",(function(){return _}));var I=function(e,t){var n=arguments;if(null==t||!f.call(t,"css"))return i.createElement.apply(void 0,n);var r=n.length,o=new Array(r);o[0]=A,o[1]=b(e,t);for(var a=2;a<r;a++)o[a]=n[a];return i.createElement.apply(null,o)},x=m((function(e,t){var n=e.styles;if("function"==typeof n)return Object(i.createElement)(g.Consumer,null,(function(e){var r=Object(h.a)([n(e)]);return Object(i.createElement)(w,{serialized:r,cache:t})}));var r=Object(h.a)([n]);return Object(i.createElement)(w,{serialized:r,cache:t})})),w=function(e){function t(t,n,r){return e.call(this,t,n,r)||this}Object(r.a)(t,e);var n=t.prototype;return n.componentDidMount=function(){this.sheet=new o({key:this.props.cache.key+"-global",nonce:this.props.cache.sheet.nonce,container:this.props.cache.sheet.container});var e=document.querySelector("style[data-emotion-"+this.props.cache.key+'="'+this.props.serialized.name+'"]');null!==e&&this.sheet.tags.push(e),this.props.cache.sheet.tags.length&&(this.sheet.before=this.props.cache.sheet.tags[0]),this.insertStyles()},n.componentDidUpdate=function(e){e.serialized.name!==this.props.serialized.name&&this.insertStyles()},n.insertStyles=function(){if(void 0!==this.props.serialized.next&&Object(d.b)(this.props.cache,this.props.serialized.next,!0),this.sheet.tags.length){var e=this.sheet.tags[this.sheet.tags.length-1].nextElementSibling;this.sheet.before=e,this.sheet.flush()}this.props.cache.insert("",this.props.serialized,this.sheet,!1)},n.componentWillUnmount=function(){this.sheet.flush()},n.render=function(){return null},t}(i.Component),M=function(){var e=_.apply(void 0,arguments),t="animation-"+e.name;return{name:t,styles:"@keyframes "+t+"{"+e.styles+"}",anim:1,toString:function(){return"_EMO_"+this.name+"_"+this.styles+"_EMO_"}}},S=function e(t){for(var n=t.length,r=0,i="";r<n;r++){var o=t[r];if(null!=o){var a=void 0;switch(typeof o){case"boolean":break;case"object":if(Array.isArray(o))a=e(o);else for(var s in a="",o)o[s]&&s&&(a&&(a+=" "),a+=s);break;default:a=o}a&&(i&&(i+=" "),i+=a)}}return i};function T(e,t,n){var r=[],i=Object(d.a)(e,r,n);return r.length<2?n:i+t(r)}var E=m((function(e,t){return Object(i.createElement)(g.Consumer,null,(function(n){var r=function(){for(var e=arguments.length,n=new Array(e),r=0;r<e;r++)n[r]=arguments[r];var i=Object(h.a)(n,t.registered);return Object(d.b)(t,i,!1),t.key+"-"+i.name},i={css:r,cx:function(){for(var e=arguments.length,n=new Array(e),i=0;i<e;i++)n[i]=arguments[i];return T(t.registered,r,S(n))},theme:n},o=e.children(i);return!0,o}))}))},function(e,t,n){"use strict";function r(e){for(var t="https://material-ui.com/production-error/?code="+e,n=1;n<arguments.length;n+=1)t+="&args[]="+encodeURIComponent(arguments[n]);return"Minified Material-UI error #"+e+"; visit "+t+" for the full message."}n.d(t,"a",(function(){return r}))},function(e,t,n){"use strict";function r(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,r=new Array(t);n<t;n++)r[n]=e[n];return r}n.d(t,"a",(function(){return r}))},function(e,t,n){"use strict";var r=n(5),i=n(1),o=n(0),a=(n(3),n(4)),s=n(7),l=o.forwardRef((function(e,t){var n=e.classes,s=e.className,l=e.component,u=void 0===l?"div":l,c=e.square,d=void 0!==c&&c,h=e.elevation,f=void 0===h?1:h,p=e.variant,g=void 0===p?"elevation":p,v=Object(r.a)(e,["classes","className","component","square","elevation","variant"]);return o.createElement(u,Object(i.a)({className:Object(a.a)(n.root,s,"outlined"===g?n.outlined:n["elevation".concat(f)],!d&&n.rounded),ref:t},v))}));t.a=Object(s.a)((function(e){var t={};return e.shadows.forEach((function(e,n){t["elevation".concat(n)]={boxShadow:e}})),Object(i.a)({root:{backgroundColor:e.palette.background.paper,color:e.palette.text.primary,transition:e.transitions.create("box-shadow")},rounded:{borderRadius:e.shape.borderRadius},outlined:{border:"1px solid ".concat(e.palette.divider)}},t)}),{name:"MuiPaper"})(l)},function(e,t,n){"use strict";var r=n(0),i=n(57);t.a=Object(i.a)(r.createElement("path",{d:"M8.59 16.34l4.58-4.59-4.58-4.59L10 5.75l6 6-6 6z"}),"KeyboardArrowRight")},function(e,t,n){"use strict";var r=n(0),i=n(57);t.a=Object(i.a)(r.createElement("path",{d:"M15.41 16.09l-4.58-4.59 4.58-4.59L14 5.5l-6 6 6 6z"}),"KeyboardArrowLeft")},function(e,t,n){"use strict";(function(n){var r,i,o,a;function s(e,t){var n;if("undefined"==typeof Symbol||null==e[Symbol.iterator]){if(Array.isArray(e)||(n=function(e,t){if(!e)return;if("string"==typeof e)return l(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);"Object"===n&&e.constructor&&(n=e.constructor.name);if("Map"===n||"Set"===n)return Array.from(e);if("Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n))return l(e,t)}(e))||t&&e&&"number"==typeof e.length){n&&(e=n);var r=0,i=function(){};return{s:i,n:function(){return r>=e.length?{done:!0}:{done:!1,value:e[r++]}},e:function(e){throw e},f:i}}throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}var o,a=!0,s=!1;return{s:function(){n=e[Symbol.iterator]()},n:function(){var e=n.next();return a=e.done,e},e:function(e){s=!0,o=e},f:function(){try{a||null==n.return||n.return()}finally{if(s)throw o}}}}function l(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,r=new Array(t);n<t;n++)r[n]=e[n];return r}function u(e){return(u="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}a=function(){return function e(t,n,r){function i(a,s){if(!n[a]){if(!t[a]){if(o)return o(a,!0);var l=new Error("Cannot find module '"+a+"'");throw l.code="MODULE_NOT_FOUND",l}var u=n[a]={exports:{}};t[a][0].call(u.exports,(function(e){return i(t[a][1][e]||e)}),u,u.exports,e,t,n,r)}return n[a].exports}for(var o=!1,a=0;a<r.length;a++)i(r[a]);return i}({1:[function(e,t,n){n.byteLength=function(e){var t=u(e),n=t[0],r=t[1];return 3*(n+r)/4-r},n.toByteArray=function(e){var t,n,r=u(e),a=r[0],s=r[1],l=new o(function(e,t,n){return 3*(t+n)/4-n}(0,a,s)),c=0,d=s>0?a-4:a;for(n=0;n<d;n+=4)t=i[e.charCodeAt(n)]<<18|i[e.charCodeAt(n+1)]<<12|i[e.charCodeAt(n+2)]<<6|i[e.charCodeAt(n+3)],l[c++]=t>>16&255,l[c++]=t>>8&255,l[c++]=255&t;return 2===s&&(t=i[e.charCodeAt(n)]<<2|i[e.charCodeAt(n+1)]>>4,l[c++]=255&t),1===s&&(t=i[e.charCodeAt(n)]<<10|i[e.charCodeAt(n+1)]<<4|i[e.charCodeAt(n+2)]>>2,l[c++]=t>>8&255,l[c++]=255&t),l},n.fromByteArray=function(e){for(var t,n=e.length,i=n%3,o=[],a=0,s=n-i;a<s;a+=16383)o.push(c(e,a,a+16383>s?s:a+16383));return 1===i?(t=e[n-1],o.push(r[t>>2]+r[t<<4&63]+"==")):2===i&&(t=(e[n-2]<<8)+e[n-1],o.push(r[t>>10]+r[t>>4&63]+r[t<<2&63]+"=")),o.join("")};for(var r=[],i=[],o="undefined"!=typeof Uint8Array?Uint8Array:Array,a="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/",s=0,l=a.length;s<l;++s)r[s]=a[s],i[a.charCodeAt(s)]=s;function u(e){var t=e.length;if(t%4>0)throw new Error("Invalid string. Length must be a multiple of 4");var n=e.indexOf("=");return-1===n&&(n=t),[n,n===t?0:4-n%4]}function c(e,t,n){for(var i,o,a=[],s=t;s<n;s+=3)i=(e[s]<<16&16711680)+(e[s+1]<<8&65280)+(255&e[s+2]),a.push(r[(o=i)>>18&63]+r[o>>12&63]+r[o>>6&63]+r[63&o]);return a.join("")}i["-".charCodeAt(0)]=62,i["_".charCodeAt(0)]=63},{}],2:[function(e,t,n){},{}],3:[function(e,t,n){(function(t){var r=e("base64-js"),i=e("ieee754");function o(e){if(e>2147483647)throw new RangeError('The value "'+e+'" is invalid for option "size"');var n=new Uint8Array(e);return n.__proto__=t.prototype,n}function t(e,t,n){if("number"==typeof e){if("string"==typeof t)throw new TypeError('The "string" argument must be of type string. Received type number');return l(e)}return a(e,t,n)}function a(e,n,r){if("string"==typeof e)return function(e,n){if("string"==typeof n&&""!==n||(n="utf8"),!t.isEncoding(n))throw new TypeError("Unknown encoding: "+n);var r=0|h(e,n),i=o(r),a=i.write(e,n);return a!==r&&(i=i.slice(0,a)),i}(e,n);if(ArrayBuffer.isView(e))return c(e);if(null==e)throw TypeError("The first argument must be one of type string, Buffer, ArrayBuffer, Array, or Array-like Object. Received type "+u(e));if(z(e,ArrayBuffer)||e&&z(e.buffer,ArrayBuffer))return function(e,n,r){if(n<0||e.byteLength<n)throw new RangeError('"offset" is outside of buffer bounds');if(e.byteLength<n+(r||0))throw new RangeError('"length" is outside of buffer bounds');var i;return(i=void 0===n&&void 0===r?new Uint8Array(e):void 0===r?new Uint8Array(e,n):new Uint8Array(e,n,r)).__proto__=t.prototype,i}(e,n,r);if("number"==typeof e)throw new TypeError('The "value" argument must not be of type number. Received type number');var i=e.valueOf&&e.valueOf();if(null!=i&&i!==e)return t.from(i,n,r);var a=function(e){if(t.isBuffer(e)){var n=0|d(e.length),r=o(n);return 0===r.length||e.copy(r,0,0,n),r}return void 0!==e.length?"number"!=typeof e.length||F(e.length)?o(0):c(e):"Buffer"===e.type&&Array.isArray(e.data)?c(e.data):void 0}(e);if(a)return a;if("undefined"!=typeof Symbol&&null!=Symbol.toPrimitive&&"function"==typeof e[Symbol.toPrimitive])return t.from(e[Symbol.toPrimitive]("string"),n,r);throw new TypeError("The first argument must be one of type string, Buffer, ArrayBuffer, Array, or Array-like Object. Received type "+u(e))}function s(e){if("number"!=typeof e)throw new TypeError('"size" argument must be of type number');if(e<0)throw new RangeError('The value "'+e+'" is invalid for option "size"')}function l(e){return s(e),o(e<0?0:0|d(e))}function c(e){for(var t=e.length<0?0:0|d(e.length),n=o(t),r=0;r<t;r+=1)n[r]=255&e[r];return n}function d(e){if(e>=2147483647)throw new RangeError("Attempt to allocate Buffer larger than maximum size: 0x"+2147483647..toString(16)+" bytes");return 0|e}function h(e,n){if(t.isBuffer(e))return e.length;if(ArrayBuffer.isView(e)||z(e,ArrayBuffer))return e.byteLength;if("string"!=typeof e)throw new TypeError('The "string" argument must be one of type string, Buffer, or ArrayBuffer. Received type '+u(e));var r=e.length,i=arguments.length>2&&!0===arguments[2];if(!i&&0===r)return 0;for(var o=!1;;)switch(n){case"ascii":case"latin1":case"binary":return r;case"utf8":case"utf-8":return N(e).length;case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return 2*r;case"hex":return r>>>1;case"base64":return j(e).length;default:if(o)return i?-1:N(e).length;n=(""+n).toLowerCase(),o=!0}}function f(e,t,n){var r=!1;if((void 0===t||t<0)&&(t=0),t>this.length)return"";if((void 0===n||n>this.length)&&(n=this.length),n<=0)return"";if((n>>>=0)<=(t>>>=0))return"";for(e||(e="utf8");;)switch(e){case"hex":return S(this,t,n);case"utf8":case"utf-8":return x(this,t,n);case"ascii":return w(this,t,n);case"latin1":case"binary":return M(this,t,n);case"base64":return I(this,t,n);case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return T(this,t,n);default:if(r)throw new TypeError("Unknown encoding: "+e);e=(e+"").toLowerCase(),r=!0}}function p(e,t,n){var r=e[t];e[t]=e[n],e[n]=r}function g(e,n,r,i,o){if(0===e.length)return-1;if("string"==typeof r?(i=r,r=0):r>2147483647?r=2147483647:r<-2147483648&&(r=-2147483648),F(r=+r)&&(r=o?0:e.length-1),r<0&&(r=e.length+r),r>=e.length){if(o)return-1;r=e.length-1}else if(r<0){if(!o)return-1;r=0}if("string"==typeof n&&(n=t.from(n,i)),t.isBuffer(n))return 0===n.length?-1:v(e,n,r,i,o);if("number"==typeof n)return n&=255,"function"==typeof Uint8Array.prototype.indexOf?o?Uint8Array.prototype.indexOf.call(e,n,r):Uint8Array.prototype.lastIndexOf.call(e,n,r):v(e,[n],r,i,o);throw new TypeError("val must be string, number or Buffer")}function v(e,t,n,r,i){var o,a=1,s=e.length,l=t.length;if(void 0!==r&&("ucs2"===(r=String(r).toLowerCase())||"ucs-2"===r||"utf16le"===r||"utf-16le"===r)){if(e.length<2||t.length<2)return-1;a=2,s/=2,l/=2,n/=2}function u(e,t){return 1===a?e[t]:e.readUInt16BE(t*a)}if(i){var c=-1;for(o=n;o<s;o++)if(u(e,o)===u(t,-1===c?0:o-c)){if(-1===c&&(c=o),o-c+1===l)return c*a}else-1!==c&&(o-=o-c),c=-1}else for(n+l>s&&(n=s-l),o=n;o>=0;o--){for(var d=!0,h=0;h<l;h++)if(u(e,o+h)!==u(t,h)){d=!1;break}if(d)return o}return-1}function m(e,t,n,r){n=Number(n)||0;var i=e.length-n;r?(r=Number(r))>i&&(r=i):r=i;var o=t.length;r>o/2&&(r=o/2);for(var a=0;a<r;++a){var s=parseInt(t.substr(2*a,2),16);if(F(s))return a;e[n+a]=s}return a}function y(e,t,n,r){return R(N(t,e.length-n),e,n,r)}function b(e,t,n,r){return R(function(e){for(var t=[],n=0;n<e.length;++n)t.push(255&e.charCodeAt(n));return t}(t),e,n,r)}function C(e,t,n,r){return b(e,t,n,r)}function A(e,t,n,r){return R(j(t),e,n,r)}function _(e,t,n,r){return R(function(e,t){for(var n,r,i,o=[],a=0;a<e.length&&!((t-=2)<0);++a)n=e.charCodeAt(a),r=n>>8,i=n%256,o.push(i),o.push(r);return o}(t,e.length-n),e,n,r)}function I(e,t,n){return 0===t&&n===e.length?r.fromByteArray(e):r.fromByteArray(e.slice(t,n))}function x(e,t,n){n=Math.min(e.length,n);for(var r=[],i=t;i<n;){var o,a,s,l,u=e[i],c=null,d=u>239?4:u>223?3:u>191?2:1;if(i+d<=n)switch(d){case 1:u<128&&(c=u);break;case 2:128==(192&(o=e[i+1]))&&(l=(31&u)<<6|63&o)>127&&(c=l);break;case 3:o=e[i+1],a=e[i+2],128==(192&o)&&128==(192&a)&&(l=(15&u)<<12|(63&o)<<6|63&a)>2047&&(l<55296||l>57343)&&(c=l);break;case 4:o=e[i+1],a=e[i+2],s=e[i+3],128==(192&o)&&128==(192&a)&&128==(192&s)&&(l=(15&u)<<18|(63&o)<<12|(63&a)<<6|63&s)>65535&&l<1114112&&(c=l)}null===c?(c=65533,d=1):c>65535&&(c-=65536,r.push(c>>>10&1023|55296),c=56320|1023&c),r.push(c),i+=d}return function(e){var t=e.length;if(t<=4096)return String.fromCharCode.apply(String,e);for(var n="",r=0;r<t;)n+=String.fromCharCode.apply(String,e.slice(r,r+=4096));return n}(r)}function w(e,t,n){var r="";n=Math.min(e.length,n);for(var i=t;i<n;++i)r+=String.fromCharCode(127&e[i]);return r}function M(e,t,n){var r="";n=Math.min(e.length,n);for(var i=t;i<n;++i)r+=String.fromCharCode(e[i]);return r}function S(e,t,n){var r,i=e.length;(!t||t<0)&&(t=0),(!n||n<0||n>i)&&(n=i);for(var o="",a=t;a<n;++a)o+=(r=e[a])<16?"0"+r.toString(16):r.toString(16);return o}function T(e,t,n){for(var r=e.slice(t,n),i="",o=0;o<r.length;o+=2)i+=String.fromCharCode(r[o]+256*r[o+1]);return i}function E(e,t,n){if(e%1!=0||e<0)throw new RangeError("offset is not uint");if(e+t>n)throw new RangeError("Trying to access beyond buffer length")}function O(e,n,r,i,o,a){if(!t.isBuffer(e))throw new TypeError('"buffer" argument must be a Buffer instance');if(n>o||n<a)throw new RangeError('"value" argument is out of bounds');if(r+i>e.length)throw new RangeError("Index out of range")}function k(e,t,n,r,i,o){if(n+r>e.length)throw new RangeError("Index out of range");if(n<0)throw new RangeError("Index out of range")}function L(e,t,n,r,o){return t=+t,n>>>=0,o||k(e,0,n,4),i.write(e,t,n,r,23,4),n+4}function D(e,t,n,r,o){return t=+t,n>>>=0,o||k(e,0,n,8),i.write(e,t,n,r,52,8),n+8}n.Buffer=t,n.SlowBuffer=function(e){return+e!=e&&(e=0),t.alloc(+e)},n.INSPECT_MAX_BYTES=50,n.kMaxLength=2147483647,t.TYPED_ARRAY_SUPPORT=function(){try{var e=new Uint8Array(1);return e.__proto__={__proto__:Uint8Array.prototype,foo:function(){return 42}},42===e.foo()}catch(e){return!1}}(),t.TYPED_ARRAY_SUPPORT||"undefined"==typeof console||"function"!=typeof console.error||console.error("This browser lacks typed array (Uint8Array) support which is required by `buffer` v5.x. Use `buffer` v4.x if you require old browser support."),Object.defineProperty(t.prototype,"parent",{enumerable:!0,get:function(){if(t.isBuffer(this))return this.buffer}}),Object.defineProperty(t.prototype,"offset",{enumerable:!0,get:function(){if(t.isBuffer(this))return this.byteOffset}}),"undefined"!=typeof Symbol&&null!=Symbol.species&&t[Symbol.species]===t&&Object.defineProperty(t,Symbol.species,{value:null,configurable:!0,enumerable:!1,writable:!1}),t.poolSize=8192,t.from=function(e,t,n){return a(e,t,n)},t.prototype.__proto__=Uint8Array.prototype,t.__proto__=Uint8Array,t.alloc=function(e,t,n){return function(e,t,n){return s(e),e<=0?o(e):void 0!==t?"string"==typeof n?o(e).fill(t,n):o(e).fill(t):o(e)}(e,t,n)},t.allocUnsafe=function(e){return l(e)},t.allocUnsafeSlow=function(e){return l(e)},t.isBuffer=function(e){return null!=e&&!0===e._isBuffer&&e!==t.prototype},t.compare=function(e,n){if(z(e,Uint8Array)&&(e=t.from(e,e.offset,e.byteLength)),z(n,Uint8Array)&&(n=t.from(n,n.offset,n.byteLength)),!t.isBuffer(e)||!t.isBuffer(n))throw new TypeError('The "buf1", "buf2" arguments must be one of type Buffer or Uint8Array');if(e===n)return 0;for(var r=e.length,i=n.length,o=0,a=Math.min(r,i);o<a;++o)if(e[o]!==n[o]){r=e[o],i=n[o];break}return r<i?-1:i<r?1:0},t.isEncoding=function(e){switch(String(e).toLowerCase()){case"hex":case"utf8":case"utf-8":case"ascii":case"latin1":case"binary":case"base64":case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return!0;default:return!1}},t.concat=function(e,n){if(!Array.isArray(e))throw new TypeError('"list" argument must be an Array of Buffers');if(0===e.length)return t.alloc(0);var r;if(void 0===n)for(n=0,r=0;r<e.length;++r)n+=e[r].length;var i=t.allocUnsafe(n),o=0;for(r=0;r<e.length;++r){var a=e[r];if(z(a,Uint8Array)&&(a=t.from(a)),!t.isBuffer(a))throw new TypeError('"list" argument must be an Array of Buffers');a.copy(i,o),o+=a.length}return i},t.byteLength=h,t.prototype._isBuffer=!0,t.prototype.swap16=function(){var e=this.length;if(e%2!=0)throw new RangeError("Buffer size must be a multiple of 16-bits");for(var t=0;t<e;t+=2)p(this,t,t+1);return this},t.prototype.swap32=function(){var e=this.length;if(e%4!=0)throw new RangeError("Buffer size must be a multiple of 32-bits");for(var t=0;t<e;t+=4)p(this,t,t+3),p(this,t+1,t+2);return this},t.prototype.swap64=function(){var e=this.length;if(e%8!=0)throw new RangeError("Buffer size must be a multiple of 64-bits");for(var t=0;t<e;t+=8)p(this,t,t+7),p(this,t+1,t+6),p(this,t+2,t+5),p(this,t+3,t+4);return this},t.prototype.toString=function(){var e=this.length;return 0===e?"":0===arguments.length?x(this,0,e):f.apply(this,arguments)},t.prototype.toLocaleString=t.prototype.toString,t.prototype.equals=function(e){if(!t.isBuffer(e))throw new TypeError("Argument must be a Buffer");return this===e||0===t.compare(this,e)},t.prototype.inspect=function(){var e="",t=n.INSPECT_MAX_BYTES;return e=this.toString("hex",0,t).replace(/(.{2})/g,"$1 ").trim(),this.length>t&&(e+=" ... "),"<Buffer "+e+">"},t.prototype.compare=function(e,n,r,i,o){if(z(e,Uint8Array)&&(e=t.from(e,e.offset,e.byteLength)),!t.isBuffer(e))throw new TypeError('The "target" argument must be one of type Buffer or Uint8Array. Received type '+u(e));if(void 0===n&&(n=0),void 0===r&&(r=e?e.length:0),void 0===i&&(i=0),void 0===o&&(o=this.length),n<0||r>e.length||i<0||o>this.length)throw new RangeError("out of range index");if(i>=o&&n>=r)return 0;if(i>=o)return-1;if(n>=r)return 1;if(this===e)return 0;for(var a=(o>>>=0)-(i>>>=0),s=(r>>>=0)-(n>>>=0),l=Math.min(a,s),c=this.slice(i,o),d=e.slice(n,r),h=0;h<l;++h)if(c[h]!==d[h]){a=c[h],s=d[h];break}return a<s?-1:s<a?1:0},t.prototype.includes=function(e,t,n){return-1!==this.indexOf(e,t,n)},t.prototype.indexOf=function(e,t,n){return g(this,e,t,n,!0)},t.prototype.lastIndexOf=function(e,t,n){return g(this,e,t,n,!1)},t.prototype.write=function(e,t,n,r){if(void 0===t)r="utf8",n=this.length,t=0;else if(void 0===n&&"string"==typeof t)r=t,n=this.length,t=0;else{if(!isFinite(t))throw new Error("Buffer.write(string, encoding, offset[, length]) is no longer supported");t>>>=0,isFinite(n)?(n>>>=0,void 0===r&&(r="utf8")):(r=n,n=void 0)}var i=this.length-t;if((void 0===n||n>i)&&(n=i),e.length>0&&(n<0||t<0)||t>this.length)throw new RangeError("Attempt to write outside buffer bounds");r||(r="utf8");for(var o=!1;;)switch(r){case"hex":return m(this,e,t,n);case"utf8":case"utf-8":return y(this,e,t,n);case"ascii":return b(this,e,t,n);case"latin1":case"binary":return C(this,e,t,n);case"base64":return A(this,e,t,n);case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return _(this,e,t,n);default:if(o)throw new TypeError("Unknown encoding: "+r);r=(""+r).toLowerCase(),o=!0}},t.prototype.toJSON=function(){return{type:"Buffer",data:Array.prototype.slice.call(this._arr||this,0)}},t.prototype.slice=function(e,n){var r=this.length;(e=~~e)<0?(e+=r)<0&&(e=0):e>r&&(e=r),(n=void 0===n?r:~~n)<0?(n+=r)<0&&(n=0):n>r&&(n=r),n<e&&(n=e);var i=this.subarray(e,n);return i.__proto__=t.prototype,i},t.prototype.readUIntLE=function(e,t,n){e>>>=0,t>>>=0,n||E(e,t,this.length);for(var r=this[e],i=1,o=0;++o<t&&(i*=256);)r+=this[e+o]*i;return r},t.prototype.readUIntBE=function(e,t,n){e>>>=0,t>>>=0,n||E(e,t,this.length);for(var r=this[e+--t],i=1;t>0&&(i*=256);)r+=this[e+--t]*i;return r},t.prototype.readUInt8=function(e,t){return e>>>=0,t||E(e,1,this.length),this[e]},t.prototype.readUInt16LE=function(e,t){return e>>>=0,t||E(e,2,this.length),this[e]|this[e+1]<<8},t.prototype.readUInt16BE=function(e,t){return e>>>=0,t||E(e,2,this.length),this[e]<<8|this[e+1]},t.prototype.readUInt32LE=function(e,t){return e>>>=0,t||E(e,4,this.length),(this[e]|this[e+1]<<8|this[e+2]<<16)+16777216*this[e+3]},t.prototype.readUInt32BE=function(e,t){return e>>>=0,t||E(e,4,this.length),16777216*this[e]+(this[e+1]<<16|this[e+2]<<8|this[e+3])},t.prototype.readIntLE=function(e,t,n){e>>>=0,t>>>=0,n||E(e,t,this.length);for(var r=this[e],i=1,o=0;++o<t&&(i*=256);)r+=this[e+o]*i;return r>=(i*=128)&&(r-=Math.pow(2,8*t)),r},t.prototype.readIntBE=function(e,t,n){e>>>=0,t>>>=0,n||E(e,t,this.length);for(var r=t,i=1,o=this[e+--r];r>0&&(i*=256);)o+=this[e+--r]*i;return o>=(i*=128)&&(o-=Math.pow(2,8*t)),o},t.prototype.readInt8=function(e,t){return e>>>=0,t||E(e,1,this.length),128&this[e]?-1*(255-this[e]+1):this[e]},t.prototype.readInt16LE=function(e,t){e>>>=0,t||E(e,2,this.length);var n=this[e]|this[e+1]<<8;return 32768&n?4294901760|n:n},t.prototype.readInt16BE=function(e,t){e>>>=0,t||E(e,2,this.length);var n=this[e+1]|this[e]<<8;return 32768&n?4294901760|n:n},t.prototype.readInt32LE=function(e,t){return e>>>=0,t||E(e,4,this.length),this[e]|this[e+1]<<8|this[e+2]<<16|this[e+3]<<24},t.prototype.readInt32BE=function(e,t){return e>>>=0,t||E(e,4,this.length),this[e]<<24|this[e+1]<<16|this[e+2]<<8|this[e+3]},t.prototype.readFloatLE=function(e,t){return e>>>=0,t||E(e,4,this.length),i.read(this,e,!0,23,4)},t.prototype.readFloatBE=function(e,t){return e>>>=0,t||E(e,4,this.length),i.read(this,e,!1,23,4)},t.prototype.readDoubleLE=function(e,t){return e>>>=0,t||E(e,8,this.length),i.read(this,e,!0,52,8)},t.prototype.readDoubleBE=function(e,t){return e>>>=0,t||E(e,8,this.length),i.read(this,e,!1,52,8)},t.prototype.writeUIntLE=function(e,t,n,r){e=+e,t>>>=0,n>>>=0,r||O(this,e,t,n,Math.pow(2,8*n)-1,0);var i=1,o=0;for(this[t]=255&e;++o<n&&(i*=256);)this[t+o]=e/i&255;return t+n},t.prototype.writeUIntBE=function(e,t,n,r){e=+e,t>>>=0,n>>>=0,r||O(this,e,t,n,Math.pow(2,8*n)-1,0);var i=n-1,o=1;for(this[t+i]=255&e;--i>=0&&(o*=256);)this[t+i]=e/o&255;return t+n},t.prototype.writeUInt8=function(e,t,n){return e=+e,t>>>=0,n||O(this,e,t,1,255,0),this[t]=255&e,t+1},t.prototype.writeUInt16LE=function(e,t,n){return e=+e,t>>>=0,n||O(this,e,t,2,65535,0),this[t]=255&e,this[t+1]=e>>>8,t+2},t.prototype.writeUInt16BE=function(e,t,n){return e=+e,t>>>=0,n||O(this,e,t,2,65535,0),this[t]=e>>>8,this[t+1]=255&e,t+2},t.prototype.writeUInt32LE=function(e,t,n){return e=+e,t>>>=0,n||O(this,e,t,4,4294967295,0),this[t+3]=e>>>24,this[t+2]=e>>>16,this[t+1]=e>>>8,this[t]=255&e,t+4},t.prototype.writeUInt32BE=function(e,t,n){return e=+e,t>>>=0,n||O(this,e,t,4,4294967295,0),this[t]=e>>>24,this[t+1]=e>>>16,this[t+2]=e>>>8,this[t+3]=255&e,t+4},t.prototype.writeIntLE=function(e,t,n,r){if(e=+e,t>>>=0,!r){var i=Math.pow(2,8*n-1);O(this,e,t,n,i-1,-i)}var o=0,a=1,s=0;for(this[t]=255&e;++o<n&&(a*=256);)e<0&&0===s&&0!==this[t+o-1]&&(s=1),this[t+o]=(e/a>>0)-s&255;return t+n},t.prototype.writeIntBE=function(e,t,n,r){if(e=+e,t>>>=0,!r){var i=Math.pow(2,8*n-1);O(this,e,t,n,i-1,-i)}var o=n-1,a=1,s=0;for(this[t+o]=255&e;--o>=0&&(a*=256);)e<0&&0===s&&0!==this[t+o+1]&&(s=1),this[t+o]=(e/a>>0)-s&255;return t+n},t.prototype.writeInt8=function(e,t,n){return e=+e,t>>>=0,n||O(this,e,t,1,127,-128),e<0&&(e=255+e+1),this[t]=255&e,t+1},t.prototype.writeInt16LE=function(e,t,n){return e=+e,t>>>=0,n||O(this,e,t,2,32767,-32768),this[t]=255&e,this[t+1]=e>>>8,t+2},t.prototype.writeInt16BE=function(e,t,n){return e=+e,t>>>=0,n||O(this,e,t,2,32767,-32768),this[t]=e>>>8,this[t+1]=255&e,t+2},t.prototype.writeInt32LE=function(e,t,n){return e=+e,t>>>=0,n||O(this,e,t,4,2147483647,-2147483648),this[t]=255&e,this[t+1]=e>>>8,this[t+2]=e>>>16,this[t+3]=e>>>24,t+4},t.prototype.writeInt32BE=function(e,t,n){return e=+e,t>>>=0,n||O(this,e,t,4,2147483647,-2147483648),e<0&&(e=4294967295+e+1),this[t]=e>>>24,this[t+1]=e>>>16,this[t+2]=e>>>8,this[t+3]=255&e,t+4},t.prototype.writeFloatLE=function(e,t,n){return L(this,e,t,!0,n)},t.prototype.writeFloatBE=function(e,t,n){return L(this,e,t,!1,n)},t.prototype.writeDoubleLE=function(e,t,n){return D(this,e,t,!0,n)},t.prototype.writeDoubleBE=function(e,t,n){return D(this,e,t,!1,n)},t.prototype.copy=function(e,n,r,i){if(!t.isBuffer(e))throw new TypeError("argument should be a Buffer");if(r||(r=0),i||0===i||(i=this.length),n>=e.length&&(n=e.length),n||(n=0),i>0&&i<r&&(i=r),i===r)return 0;if(0===e.length||0===this.length)return 0;if(n<0)throw new RangeError("targetStart out of bounds");if(r<0||r>=this.length)throw new RangeError("Index out of range");if(i<0)throw new RangeError("sourceEnd out of bounds");i>this.length&&(i=this.length),e.length-n<i-r&&(i=e.length-n+r);var o=i-r;if(this===e&&"function"==typeof Uint8Array.prototype.copyWithin)this.copyWithin(n,r,i);else if(this===e&&r<n&&n<i)for(var a=o-1;a>=0;--a)e[a+n]=this[a+r];else Uint8Array.prototype.set.call(e,this.subarray(r,i),n);return o},t.prototype.fill=function(e,n,r,i){if("string"==typeof e){if("string"==typeof n?(i=n,n=0,r=this.length):"string"==typeof r&&(i=r,r=this.length),void 0!==i&&"string"!=typeof i)throw new TypeError("encoding must be a string");if("string"==typeof i&&!t.isEncoding(i))throw new TypeError("Unknown encoding: "+i);if(1===e.length){var o=e.charCodeAt(0);("utf8"===i&&o<128||"latin1"===i)&&(e=o)}}else"number"==typeof e&&(e&=255);if(n<0||this.length<n||this.length<r)throw new RangeError("Out of range index");if(r<=n)return this;var a;if(n>>>=0,r=void 0===r?this.length:r>>>0,e||(e=0),"number"==typeof e)for(a=n;a<r;++a)this[a]=e;else{var s=t.isBuffer(e)?e:t.from(e,i),l=s.length;if(0===l)throw new TypeError('The value "'+e+'" is invalid for argument "value"');for(a=0;a<r-n;++a)this[a+n]=s[a%l]}return this};var P=/[^+/0-9A-Za-z-_]/g;function N(e,t){var n;t=t||1/0;for(var r=e.length,i=null,o=[],a=0;a<r;++a){if((n=e.charCodeAt(a))>55295&&n<57344){if(!i){if(n>56319){(t-=3)>-1&&o.push(239,191,189);continue}if(a+1===r){(t-=3)>-1&&o.push(239,191,189);continue}i=n;continue}if(n<56320){(t-=3)>-1&&o.push(239,191,189),i=n;continue}n=65536+(i-55296<<10|n-56320)}else i&&(t-=3)>-1&&o.push(239,191,189);if(i=null,n<128){if((t-=1)<0)break;o.push(n)}else if(n<2048){if((t-=2)<0)break;o.push(n>>6|192,63&n|128)}else if(n<65536){if((t-=3)<0)break;o.push(n>>12|224,n>>6&63|128,63&n|128)}else{if(!(n<1114112))throw new Error("Invalid code point");if((t-=4)<0)break;o.push(n>>18|240,n>>12&63|128,n>>6&63|128,63&n|128)}}return o}function j(e){return r.toByteArray(function(e){if((e=(e=e.split("=")[0]).trim().replace(P,"")).length<2)return"";for(;e.length%4!=0;)e+="=";return e}(e))}function R(e,t,n,r){for(var i=0;i<r&&!(i+n>=t.length||i>=e.length);++i)t[i+n]=e[i];return i}function z(e,t){return e instanceof t||null!=e&&null!=e.constructor&&null!=e.constructor.name&&e.constructor.name===t.name}function F(e){return e!=e}}).call(this,e("buffer").Buffer)},{"base64-js":1,buffer:3,ieee754:32}],4:[function(e,t,n){Object.defineProperty(n,"__esModule",{value:!0}),n.attributeNames=n.elementNames=void 0,n.elementNames=new Map([["altglyph","altGlyph"],["altglyphdef","altGlyphDef"],["altglyphitem","altGlyphItem"],["animatecolor","animateColor"],["animatemotion","animateMotion"],["animatetransform","animateTransform"],["clippath","clipPath"],["feblend","feBlend"],["fecolormatrix","feColorMatrix"],["fecomponenttransfer","feComponentTransfer"],["fecomposite","feComposite"],["feconvolvematrix","feConvolveMatrix"],["fediffuselighting","feDiffuseLighting"],["fedisplacementmap","feDisplacementMap"],["fedistantlight","feDistantLight"],["fedropshadow","feDropShadow"],["feflood","feFlood"],["fefunca","feFuncA"],["fefuncb","feFuncB"],["fefuncg","feFuncG"],["fefuncr","feFuncR"],["fegaussianblur","feGaussianBlur"],["feimage","feImage"],["femerge","feMerge"],["femergenode","feMergeNode"],["femorphology","feMorphology"],["feoffset","feOffset"],["fepointlight","fePointLight"],["fespecularlighting","feSpecularLighting"],["fespotlight","feSpotLight"],["fetile","feTile"],["feturbulence","feTurbulence"],["foreignobject","foreignObject"],["glyphref","glyphRef"],["lineargradient","linearGradient"],["radialgradient","radialGradient"],["textpath","textPath"]]),n.attributeNames=new Map([["definitionurl","definitionURL"],["attributename","attributeName"],["attributetype","attributeType"],["basefrequency","baseFrequency"],["baseprofile","baseProfile"],["calcmode","calcMode"],["clippathunits","clipPathUnits"],["diffuseconstant","diffuseConstant"],["edgemode","edgeMode"],["filterunits","filterUnits"],["glyphref","glyphRef"],["gradienttransform","gradientTransform"],["gradientunits","gradientUnits"],["kernelmatrix","kernelMatrix"],["kernelunitlength","kernelUnitLength"],["keypoints","keyPoints"],["keysplines","keySplines"],["keytimes","keyTimes"],["lengthadjust","lengthAdjust"],["limitingconeangle","limitingConeAngle"],["markerheight","markerHeight"],["markerunits","markerUnits"],["markerwidth","markerWidth"],["maskcontentunits","maskContentUnits"],["maskunits","maskUnits"],["numoctaves","numOctaves"],["pathlength","pathLength"],["patterncontentunits","patternContentUnits"],["patterntransform","patternTransform"],["patternunits","patternUnits"],["pointsatx","pointsAtX"],["pointsaty","pointsAtY"],["pointsatz","pointsAtZ"],["preservealpha","preserveAlpha"],["preserveaspectratio","preserveAspectRatio"],["primitiveunits","primitiveUnits"],["refx","refX"],["refy","refY"],["repeatcount","repeatCount"],["repeatdur","repeatDur"],["requiredextensions","requiredExtensions"],["requiredfeatures","requiredFeatures"],["specularconstant","specularConstant"],["specularexponent","specularExponent"],["spreadmethod","spreadMethod"],["startoffset","startOffset"],["stddeviation","stdDeviation"],["stitchtiles","stitchTiles"],["surfacescale","surfaceScale"],["systemlanguage","systemLanguage"],["tablevalues","tableValues"],["targetx","targetX"],["targety","targetY"],["textlength","textLength"],["viewbox","viewBox"],["viewtarget","viewTarget"],["xchannelselector","xChannelSelector"],["ychannelselector","yChannelSelector"],["zoomandpan","zoomAndPan"]])},{}],5:[function(e,t,n){var r=this&&this.__assign||function(){return(r=Object.assign||function(e){for(var t,n=1,r=arguments.length;n<r;n++)for(var i in t=arguments[n])Object.prototype.hasOwnProperty.call(t,i)&&(e[i]=t[i]);return e}).apply(this,arguments)},i=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),o=this&&this.__setModuleDefault||(Object.create?function(e,t){Object.defineProperty(e,"default",{enumerable:!0,value:t})}:function(e,t){e.default=t}),a=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)"default"!==n&&Object.prototype.hasOwnProperty.call(e,n)&&i(t,e,n);return o(t,e),t};Object.defineProperty(n,"__esModule",{value:!0});var s=a(e("domelementtype")),l=e("entities"),u=e("./foreignNames"),c=new Set(["style","script","xmp","iframe","noembed","noframes","plaintext","noscript"]),d=new Set(["area","base","basefont","br","col","command","embed","frame","hr","img","input","isindex","keygen","link","meta","param","source","track","wbr"]);function h(e,t){void 0===t&&(t={});for(var n=Array.isArray(e)||e.cheerio?e:[e],r="",i=0;i<n.length;i++)r+=f(n[i],t);return r}function f(e,t){switch(e.type){case"root":return h(e.children,t);case s.Directive:return"<"+e.data+">";case s.Comment:return function(e){return"\x3c!--"+e.data+"--\x3e"}(e);case s.CDATA:return function(e){return"<![CDATA["+e.children[0].data+"]]>"}(e);default:return s.isTag(e)?function(e,t){var n;"foreign"===t.xmlMode&&(e.name=null!==(n=u.elementNames.get(e.name))&&void 0!==n?n:e.name,e.parent&&p.has(e.parent.name)&&(t=r(r({},t),{xmlMode:!1}))),!t.xmlMode&&g.has(e.name)&&(t=r(r({},t),{xmlMode:"foreign"}));var i="<"+e.name,o=function(e,t){if(e)return Object.keys(e).map((function(n){var r,i,o=null!==(r=e[n])&&void 0!==r?r:"";return"foreign"===t.xmlMode&&(n=null!==(i=u.attributeNames.get(n))&&void 0!==i?i:n),t.emptyAttrs||t.xmlMode||""!==o?n+'="'+(t.decodeEntities?l.encodeXML(o):o.replace(/"/g,"&quot;"))+'"':n})).join(" ")}(e.attribs,t);return o&&(i+=" "+o),0===e.children.length&&(t.xmlMode?!1!==t.selfClosingTags:t.selfClosingTags&&d.has(e.name))?(t.xmlMode||(i+=" "),i+="/>"):(i+=">",e.children.length>0&&(i+=h(e.children,t)),!t.xmlMode&&d.has(e.name)||(i+="</"+e.name+">")),i}(e,t):function(e,t){var n=e.data||"";return!t.decodeEntities||e.parent&&c.has(e.parent.name)||(n=l.encodeXML(n)),n}(e,t)}}n.default=h;var p=new Set(["mi","mo","mn","ms","mtext","annotation-xml","foreignObject","desc","title"]),g=new Set(["svg","math"])},{"./foreignNames":4,domelementtype:6,entities:20}],6:[function(e,t,n){Object.defineProperty(n,"__esModule",{value:!0}),n.Doctype=n.CDATA=n.Tag=n.Style=n.Script=n.Comment=n.Directive=n.Text=n.isTag=void 0,n.isTag=function(e){return"tag"===e.type||"script"===e.type||"style"===e.type},n.Text="text",n.Directive="directive",n.Comment="comment",n.Script="script",n.Style="style",n.Tag="tag",n.CDATA="cdata",n.Doctype="doctype"},{}],7:[function(e,t,n){Object.defineProperty(n,"__esModule",{value:!0});var r=e("./node");n.Node=r.Node,n.Element=r.Element,n.DataNode=r.DataNode,n.NodeWithChildren=r.NodeWithChildren;var i=/\s+/g,o={normalizeWhitespace:!1,withStartIndices:!1,withEndIndices:!1},a=function(){function e(e,t,n){this.dom=[],this._done=!1,this._tagStack=[],this._lastNode=null,this._parser=null,"function"==typeof t&&(n=t,t=o),"object"===u(e)&&(t=e,e=void 0),this._callback=e||null,this._options=t||o,this._elementCB=n||null}return e.prototype.onparserinit=function(e){this._parser=e},e.prototype.onreset=function(){this.dom=[],this._done=!1,this._tagStack=[],this._lastNode=null,this._parser=this._parser||null},e.prototype.onend=function(){this._done||(this._done=!0,this._parser=null,this.handleCallback(null))},e.prototype.onerror=function(e){this.handleCallback(e)},e.prototype.onclosetag=function(){this._lastNode=null;var e=this._tagStack.pop();e&&this._parser&&(this._options.withEndIndices&&(e.endIndex=this._parser.endIndex),this._elementCB&&this._elementCB(e))},e.prototype.onopentag=function(e,t){var n=new r.Element(e,t);this.addNode(n),this._tagStack.push(n)},e.prototype.ontext=function(e){var t=this._options.normalizeWhitespace,n=this._lastNode;if(n&&"text"===n.type)t?n.data=(n.data+e).replace(i," "):n.data+=e;else{t&&(e=e.replace(i," "));var o=new r.DataNode("text",e);this.addNode(o),this._lastNode=o}},e.prototype.oncomment=function(e){if(this._lastNode&&"comment"===this._lastNode.type)this._lastNode.data+=e;else{var t=new r.DataNode("comment",e);this.addNode(t),this._lastNode=t}},e.prototype.oncommentend=function(){this._lastNode=null},e.prototype.oncdatastart=function(){var e=new r.DataNode("text",""),t=new r.NodeWithChildren("cdata",[e]);this.addNode(t),e.parent=t,this._lastNode=e},e.prototype.oncdataend=function(){this._lastNode=null},e.prototype.onprocessinginstruction=function(e,t){var n=new r.ProcessingInstruction(e,t);this.addNode(n)},e.prototype.handleCallback=function(e){if("function"==typeof this._callback)this._callback(e,this.dom);else if(e)throw e},e.prototype.addNode=function(e){var t=this._tagStack[this._tagStack.length-1],n=t?t.children:this.dom,r=n[n.length-1];this._parser&&(this._options.withStartIndices&&(e.startIndex=this._parser.startIndex),this._options.withEndIndices&&(e.endIndex=this._parser.endIndex)),n.push(e),r&&(e.prev=r,r.next=e),t&&(e.parent=t),this._lastNode=null},e.prototype.addDataNode=function(e){this.addNode(e),this._lastNode=e},e}();n.DomHandler=a,n.default=a},{"./node":8}],8:[function(e,t,n){var r,i=this&&this.__extends||(r=function(e,t){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(e,t)},function(e,t){function n(){this.constructor=e}r(e,t),e.prototype=null===t?Object.create(t):(n.prototype=t.prototype,new n)});Object.defineProperty(n,"__esModule",{value:!0});var o=new Map([["tag",1],["script",1],["style",1],["directive",1],["text",3],["cdata",4],["comment",8]]),a=function(){function e(e){this.type=e,this.parent=null,this.prev=null,this.next=null,this.startIndex=null,this.endIndex=null}return Object.defineProperty(e.prototype,"nodeType",{get:function(){return o.get(this.type)||1},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"parentNode",{get:function(){return this.parent||null},set:function(e){this.parent=e},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"previousSibling",{get:function(){return this.prev||null},set:function(e){this.prev=e},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"nextSibling",{get:function(){return this.next||null},set:function(e){this.next=e},enumerable:!0,configurable:!0}),e}();n.Node=a;var s=function(e){function t(t,n){var r=e.call(this,t)||this;return r.data=n,r}return i(t,e),Object.defineProperty(t.prototype,"nodeValue",{get:function(){return this.data},set:function(e){this.data=e},enumerable:!0,configurable:!0}),t}(a);n.DataNode=s;var l=function(e){function t(t,n){var r=e.call(this,"directive",n)||this;return r.name=t,r}return i(t,e),t}(s);n.ProcessingInstruction=l;var u=function(e){function t(t,n){var r=e.call(this,t)||this;return r.children=n,r}return i(t,e),Object.defineProperty(t.prototype,"firstChild",{get:function(){return this.children[0]||null},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"lastChild",{get:function(){return this.children[this.children.length-1]||null},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"childNodes",{get:function(){return this.children},set:function(e){this.children=e},enumerable:!0,configurable:!0}),t}(a);n.NodeWithChildren=u;var c=function(e){function t(t,n){var r=e.call(this,"script"===t?"script":"style"===t?"style":"tag",[])||this;return r.name=t,r.attribs=n,r.attribs=n,r}return i(t,e),Object.defineProperty(t.prototype,"tagName",{get:function(){return this.name},set:function(e){this.name=e},enumerable:!0,configurable:!0}),t}(u);n.Element=c},{}],9:[function(e,t,n){Object.defineProperty(n,"__esModule",{value:!0}),n.uniqueSort=n.compareDocumentPosition=n.removeSubsets=void 0;var r=e("./tagtypes");function i(e,t){var n=[],i=[];if(e===t)return 0;for(var o=r.hasChildren(e)?e:e.parent;o;)n.unshift(o),o=o.parent;for(o=r.hasChildren(t)?t:t.parent;o;)i.unshift(o),o=o.parent;for(var a=Math.min(n.length,i.length),s=0;s<a&&n[s]===i[s];)s++;if(0===s)return 1;var l=n[s-1],u=l.children,c=n[s],d=i[s];return u.indexOf(c)>u.indexOf(d)?l===t?20:4:l===e?10:2}n.removeSubsets=function(e){for(var t=e.length;--t>=0;){var n=e[t];if(t>0&&e.lastIndexOf(n,t-1)>=0)e.splice(t,1);else for(var r=n.parent;r;r=r.parent)if(e.includes(r)){e.splice(t,1);break}}return e},n.compareDocumentPosition=i,n.uniqueSort=function(e){return(e=e.filter((function(e,t,n){return!n.includes(e,t+1)}))).sort((function(e,t){var n=i(e,t);return 2&n?-1:4&n?1:0})),e}},{"./tagtypes":15}],10:[function(e,t,n){var r=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),i=this&&this.__exportStar||function(e,t){for(var n in e)"default"===n||Object.prototype.hasOwnProperty.call(t,n)||r(t,e,n)};Object.defineProperty(n,"__esModule",{value:!0}),i(e("./stringify"),n),i(e("./traversal"),n),i(e("./manipulation"),n),i(e("./querying"),n),i(e("./legacy"),n),i(e("./helpers"),n),i(e("./tagtypes"),n)},{"./helpers":9,"./legacy":11,"./manipulation":12,"./querying":13,"./stringify":14,"./tagtypes":15,"./traversal":16}],11:[function(e,t,n){Object.defineProperty(n,"__esModule",{value:!0}),n.getElementsByTagType=n.getElementsByTagName=n.getElementById=n.getElements=n.testElement=void 0;var r=e("./querying"),i=e("./tagtypes");function o(e){return"text"===e.type}var a={tag_name:function(e){return"function"==typeof e?function(t){return i.isTag(t)&&e(t.name)}:"*"===e?i.isTag:function(t){return i.isTag(t)&&t.name===e}},tag_type:function(e){return"function"==typeof e?function(t){return e(t.type)}:function(t){return t.type===e}},tag_contains:function(e){return"function"==typeof e?function(t){return o(t)&&e(t.data)}:function(t){return o(t)&&t.data===e}}};function s(e,t){return"function"==typeof t?function(n){return i.isTag(n)&&t(n.attribs[e])}:function(n){return i.isTag(n)&&n.attribs[e]===t}}function l(e,t){return function(n){return e(n)||t(n)}}function u(e){var t=Object.keys(e).map((function(t){var n=e[t];return t in a?a[t](n):s(t,n)}));return 0===t.length?null:t.reduce(l)}n.testElement=function(e,t){var n=u(e);return!n||n(t)},n.getElements=function(e,t,n,i){void 0===i&&(i=1/0);var o=u(e);return o?r.filter(o,t,n,i):[]},n.getElementById=function(e,t,n){return void 0===n&&(n=!0),Array.isArray(t)||(t=[t]),r.findOne(s("id",e),t,n)},n.getElementsByTagName=function(e,t,n,i){return void 0===i&&(i=1/0),r.filter(a.tag_name(e),t,n,i)},n.getElementsByTagType=function(e,t,n,i){return void 0===n&&(n=!0),void 0===i&&(i=1/0),r.filter(a.tag_type(e),t,n,i)}},{"./querying":13,"./tagtypes":15}],12:[function(e,t,n){function r(e){if(e.prev&&(e.prev.next=e.next),e.next&&(e.next.prev=e.prev),e.parent){var t=e.parent.children;t.splice(t.lastIndexOf(e),1)}}Object.defineProperty(n,"__esModule",{value:!0}),n.prepend=n.append=n.appendChild=n.replaceElement=n.removeElement=void 0,n.removeElement=r,n.replaceElement=function(e,t){var n=t.prev=e.prev;n&&(n.next=t);var r=t.next=e.next;r&&(r.prev=t);var i=t.parent=e.parent;if(i){var o=i.children;o[o.lastIndexOf(e)]=t}},n.appendChild=function(e,t){if(r(t),t.parent=e,1!==e.children.push(t)){var n=e.children[e.children.length-2];n.next=t,t.prev=n,t.next=null}},n.append=function(e,t){r(t);var n=e.parent,i=e.next;if(t.next=i,t.prev=e,e.next=t,t.parent=n,i){if(i.prev=t,n){var o=n.children;o.splice(o.lastIndexOf(i),0,t)}}else n&&n.children.push(t)},n.prepend=function(e,t){var n=e.parent;if(n){var r=n.children;r.splice(r.lastIndexOf(e),0,t)}e.prev&&(e.prev.next=t),t.parent=n,t.prev=e.prev,t.next=e,e.prev=t}},{}],13:[function(e,t,n){Object.defineProperty(n,"__esModule",{value:!0}),n.findAll=n.existsOne=n.findOne=n.findOneChild=n.find=n.filter=void 0;var r=e("./tagtypes");function i(e,t,n,o){for(var a=[],s=0,l=t;s<l.length;s++){var u=l[s];if(e(u)&&(a.push(u),--o<=0))break;if(n&&r.hasChildren(u)&&u.children.length>0){var c=i(e,u.children,n,o);if(a.push.apply(a,c),(o-=c.length)<=0)break}}return a}n.filter=function(e,t,n,r){return void 0===n&&(n=!0),void 0===r&&(r=1/0),Array.isArray(t)||(t=[t]),i(e,t,n,r)},n.find=i,n.findOneChild=function(e,t){return t.find(e)},n.findOne=function e(t,n,i){void 0===i&&(i=!0);for(var o=null,a=0;a<n.length&&!o;a++){var s=n[a];r.isTag(s)&&(t(s)?o=s:i&&s.children.length>0&&(o=e(t,s.children)))}return o},n.existsOne=function e(t,n){return n.some((function(n){return r.isTag(n)&&(t(n)||n.children.length>0&&e(t,n.children))}))},n.findAll=function(e,t){for(var n,i,o=[],a=t.filter(r.isTag);i=a.shift();){var s=null===(n=i.children)||void 0===n?void 0:n.filter(r.isTag);s&&s.length>0&&a.unshift.apply(a,s),e(i)&&o.push(i)}return o}},{"./tagtypes":15}],14:[function(e,t,n){var r=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};Object.defineProperty(n,"__esModule",{value:!0}),n.getText=n.getInnerHTML=n.getOuterHTML=void 0;var i=e("./tagtypes"),o=r(e("dom-serializer"));function a(e,t){return o.default(e,t)}n.getOuterHTML=a,n.getInnerHTML=function(e,t){return i.hasChildren(e)?e.children.map((function(e){return a(e,t)})).join(""):""},n.getText=function e(t){return Array.isArray(t)?t.map(e).join(""):i.isTag(t)?"br"===t.name?"\n":e(t.children):i.isCDATA(t)?e(t.children):i.isText(t)?t.data:""}},{"./tagtypes":15,"dom-serializer":5}],15:[function(e,t,n){Object.defineProperty(n,"__esModule",{value:!0}),n.hasChildren=n.isComment=n.isText=n.isCDATA=n.isTag=void 0;var r=e("domelementtype");n.isTag=function(e){return r.isTag(e)},n.isCDATA=function(e){return"cdata"===e.type},n.isText=function(e){return"text"===e.type},n.isComment=function(e){return"comment"===e.type},n.hasChildren=function(e){return Object.prototype.hasOwnProperty.call(e,"children")}},{domelementtype:6}],16:[function(e,t,n){function r(e){return e.children||null}function i(e){return e.parent||null}Object.defineProperty(n,"__esModule",{value:!0}),n.nextElementSibling=n.getName=n.hasAttrib=n.getAttributeValue=n.getSiblings=n.getParent=n.getChildren=void 0,n.getChildren=r,n.getParent=i,n.getSiblings=function(e){var t=i(e);return t?r(t):[e]},n.getAttributeValue=function(e,t){var n;return null===(n=e.attribs)||void 0===n?void 0:n[t]},n.hasAttrib=function(e,t){return!!e.attribs&&Object.prototype.hasOwnProperty.call(e.attribs,t)&&null!=e.attribs[t]},n.getName=function(e){return e.name},n.nextElementSibling=function(e){for(var t=e.next;null!==t&&"tag"!==t.type;)t=t.next;return t}},{}],17:[function(e,t,n){var r=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};Object.defineProperty(n,"__esModule",{value:!0}),n.decodeHTML=n.decodeHTMLStrict=n.decodeXML=void 0;var i=r(e("./maps/entities.json")),o=r(e("./maps/legacy.json")),a=r(e("./maps/xml.json")),s=r(e("./decode_codepoint"));function l(e){var t=Object.keys(e).join("|"),n=c(e),r=new RegExp("&(?:"+(t+="|#[xX][\\da-fA-F]+|#\\d+")+");","g");return function(e){return String(e).replace(r,n)}}n.decodeXML=l(a.default),n.decodeHTMLStrict=l(i.default);var u=function(e,t){return e<t?1:-1};function c(e){return function(t){if("#"===t.charAt(1)){var n=t.charAt(2);return"X"===n||"x"===n?s.default(parseInt(t.substr(3),16)):s.default(parseInt(t.substr(2),10))}return e[t.slice(1,-1)]}}n.decodeHTML=function(){for(var e=Object.keys(o.default).sort(u),t=Object.keys(i.default).sort(u),n=0,r=0;n<t.length;n++)e[r]===t[n]?(t[n]+=";?",r++):t[n]+=";";var a=new RegExp("&(?:"+t.join("|")+"|#[xX][\\da-fA-F]+;?|#\\d+;?)","g"),s=c(i.default);function l(e){return";"!==e.substr(-1)&&(e+=";"),s(e)}return function(e){return String(e).replace(a,l)}}()},{"./decode_codepoint":18,"./maps/entities.json":22,"./maps/legacy.json":23,"./maps/xml.json":24}],18:[function(e,t,n){var r=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};Object.defineProperty(n,"__esModule",{value:!0});var i=r(e("./maps/decode.json"));n.default=function(e){if(e>=55296&&e<=57343||e>1114111)return"�";e in i.default&&(e=i.default[e]);var t="";return e>65535&&(e-=65536,t+=String.fromCharCode(e>>>10&1023|55296),e=56320|1023&e),t+=String.fromCharCode(e)}},{"./maps/decode.json":21}],19:[function(e,t,n){var r=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};Object.defineProperty(n,"__esModule",{value:!0}),n.escape=n.encodeHTML=n.encodeXML=void 0;var i=l(r(e("./maps/xml.json")).default),o=u(i);n.encodeXML=h(i,o);var a=l(r(e("./maps/entities.json")).default),s=u(a);function l(e){return Object.keys(e).sort().reduce((function(t,n){return t[e[n]]="&"+n+";",t}),{})}function u(e){for(var t=[],n=[],r=0,i=Object.keys(e);r<i.length;r++){var o=i[r];1===o.length?t.push("\\"+o):n.push(o)}t.sort();for(var a=0;a<t.length-1;a++){for(var s=a;s<t.length-1&&t[s].charCodeAt(1)+1===t[s+1].charCodeAt(1);)s+=1;var l=1+s-a;l<3||t.splice(a,l,t[a]+"-"+t[s])}return n.unshift("["+t.join("")+"]"),new RegExp(n.join("|"),"g")}n.encodeHTML=h(a,s);var c=/(?:[\x80-\uD7FF\uE000-\uFFFF]|[\uD800-\uDBFF][\uDC00-\uDFFF]|[\uD800-\uDBFF](?![\uDC00-\uDFFF])|(?:[^\uD800-\uDBFF]|^)[\uDC00-\uDFFF])/g;function d(e){return"&#x"+e.codePointAt(0).toString(16).toUpperCase()+";"}function h(e,t){return function(n){return n.replace(t,(function(t){return e[t]})).replace(c,d)}}var f=u(i);n.escape=function(e){return e.replace(f,d).replace(c,d)}},{"./maps/entities.json":22,"./maps/xml.json":24}],20:[function(e,t,n){Object.defineProperty(n,"__esModule",{value:!0}),n.encode=n.decodeStrict=n.decode=void 0;var r=e("./decode"),i=e("./encode");n.decode=function(e,t){return(!t||t<=0?r.decodeXML:r.decodeHTML)(e)},n.decodeStrict=function(e,t){return(!t||t<=0?r.decodeXML:r.decodeHTMLStrict)(e)},n.encode=function(e,t){return(!t||t<=0?i.encodeXML:i.encodeHTML)(e)};var o=e("./encode");Object.defineProperty(n,"encodeXML",{enumerable:!0,get:function(){return o.encodeXML}}),Object.defineProperty(n,"encodeHTML",{enumerable:!0,get:function(){return o.encodeHTML}}),Object.defineProperty(n,"escape",{enumerable:!0,get:function(){return o.escape}}),Object.defineProperty(n,"encodeHTML4",{enumerable:!0,get:function(){return o.encodeHTML}}),Object.defineProperty(n,"encodeHTML5",{enumerable:!0,get:function(){return o.encodeHTML}});var a=e("./decode");Object.defineProperty(n,"decodeXML",{enumerable:!0,get:function(){return a.decodeXML}}),Object.defineProperty(n,"decodeHTML",{enumerable:!0,get:function(){return a.decodeHTML}}),Object.defineProperty(n,"decodeHTMLStrict",{enumerable:!0,get:function(){return a.decodeHTMLStrict}}),Object.defineProperty(n,"decodeHTML4",{enumerable:!0,get:function(){return a.decodeHTML}}),Object.defineProperty(n,"decodeHTML5",{enumerable:!0,get:function(){return a.decodeHTML}}),Object.defineProperty(n,"decodeHTML4Strict",{enumerable:!0,get:function(){return a.decodeHTMLStrict}}),Object.defineProperty(n,"decodeHTML5Strict",{enumerable:!0,get:function(){return a.decodeHTMLStrict}}),Object.defineProperty(n,"decodeXMLStrict",{enumerable:!0,get:function(){return a.decodeXML}})},{"./decode":17,"./encode":19}],21:[function(e,t,n){t.exports={0:65533,128:8364,130:8218,131:402,132:8222,133:8230,134:8224,135:8225,136:710,137:8240,138:352,139:8249,140:338,142:381,145:8216,146:8217,147:8220,148:8221,149:8226,150:8211,151:8212,152:732,153:8482,154:353,155:8250,156:339,158:382,159:376}},{}],22:[function(e,t,n){t.exports={Aacute:"Á",aacute:"á",Abreve:"Ă",abreve:"ă",ac:"∾",acd:"∿",acE:"∾̳",Acirc:"Â",acirc:"â",acute:"´",Acy:"А",acy:"а",AElig:"Æ",aelig:"æ",af:"⁡",Afr:"𝔄",afr:"𝔞",Agrave:"À",agrave:"à",alefsym:"ℵ",aleph:"ℵ",Alpha:"Α",alpha:"α",Amacr:"Ā",amacr:"ā",amalg:"⨿",amp:"&",AMP:"&",andand:"⩕",And:"⩓",and:"∧",andd:"⩜",andslope:"⩘",andv:"⩚",ang:"∠",ange:"⦤",angle:"∠",angmsdaa:"⦨",angmsdab:"⦩",angmsdac:"⦪",angmsdad:"⦫",angmsdae:"⦬",angmsdaf:"⦭",angmsdag:"⦮",angmsdah:"⦯",angmsd:"∡",angrt:"∟",angrtvb:"⊾",angrtvbd:"⦝",angsph:"∢",angst:"Å",angzarr:"⍼",Aogon:"Ą",aogon:"ą",Aopf:"𝔸",aopf:"𝕒",apacir:"⩯",ap:"≈",apE:"⩰",ape:"≊",apid:"≋",apos:"'",ApplyFunction:"⁡",approx:"≈",approxeq:"≊",Aring:"Å",aring:"å",Ascr:"𝒜",ascr:"𝒶",Assign:"≔",ast:"*",asymp:"≈",asympeq:"≍",Atilde:"Ã",atilde:"ã",Auml:"Ä",auml:"ä",awconint:"∳",awint:"⨑",backcong:"≌",backepsilon:"϶",backprime:"‵",backsim:"∽",backsimeq:"⋍",Backslash:"∖",Barv:"⫧",barvee:"⊽",barwed:"⌅",Barwed:"⌆",barwedge:"⌅",bbrk:"⎵",bbrktbrk:"⎶",bcong:"≌",Bcy:"Б",bcy:"б",bdquo:"„",becaus:"∵",because:"∵",Because:"∵",bemptyv:"⦰",bepsi:"϶",bernou:"ℬ",Bernoullis:"ℬ",Beta:"Β",beta:"β",beth:"ℶ",between:"≬",Bfr:"𝔅",bfr:"𝔟",bigcap:"⋂",bigcirc:"◯",bigcup:"⋃",bigodot:"⨀",bigoplus:"⨁",bigotimes:"⨂",bigsqcup:"⨆",bigstar:"★",bigtriangledown:"▽",bigtriangleup:"△",biguplus:"⨄",bigvee:"⋁",bigwedge:"⋀",bkarow:"⤍",blacklozenge:"⧫",blacksquare:"▪",blacktriangle:"▴",blacktriangledown:"▾",blacktriangleleft:"◂",blacktriangleright:"▸",blank:"␣",blk12:"▒",blk14:"░",blk34:"▓",block:"█",bne:"=⃥",bnequiv:"≡⃥",bNot:"⫭",bnot:"⌐",Bopf:"𝔹",bopf:"𝕓",bot:"⊥",bottom:"⊥",bowtie:"⋈",boxbox:"⧉",boxdl:"┐",boxdL:"╕",boxDl:"╖",boxDL:"╗",boxdr:"┌",boxdR:"╒",boxDr:"╓",boxDR:"╔",boxh:"─",boxH:"═",boxhd:"┬",boxHd:"╤",boxhD:"╥",boxHD:"╦",boxhu:"┴",boxHu:"╧",boxhU:"╨",boxHU:"╩",boxminus:"⊟",boxplus:"⊞",boxtimes:"⊠",boxul:"┘",boxuL:"╛",boxUl:"╜",boxUL:"╝",boxur:"└",boxuR:"╘",boxUr:"╙",boxUR:"╚",boxv:"│",boxV:"║",boxvh:"┼",boxvH:"╪",boxVh:"╫",boxVH:"╬",boxvl:"┤",boxvL:"╡",boxVl:"╢",boxVL:"╣",boxvr:"├",boxvR:"╞",boxVr:"╟",boxVR:"╠",bprime:"‵",breve:"˘",Breve:"˘",brvbar:"¦",bscr:"𝒷",Bscr:"ℬ",bsemi:"⁏",bsim:"∽",bsime:"⋍",bsolb:"⧅",bsol:"\\",bsolhsub:"⟈",bull:"•",bullet:"•",bump:"≎",bumpE:"⪮",bumpe:"≏",Bumpeq:"≎",bumpeq:"≏",Cacute:"Ć",cacute:"ć",capand:"⩄",capbrcup:"⩉",capcap:"⩋",cap:"∩",Cap:"⋒",capcup:"⩇",capdot:"⩀",CapitalDifferentialD:"ⅅ",caps:"∩︀",caret:"⁁",caron:"ˇ",Cayleys:"ℭ",ccaps:"⩍",Ccaron:"Č",ccaron:"č",Ccedil:"Ç",ccedil:"ç",Ccirc:"Ĉ",ccirc:"ĉ",Cconint:"∰",ccups:"⩌",ccupssm:"⩐",Cdot:"Ċ",cdot:"ċ",cedil:"¸",Cedilla:"¸",cemptyv:"⦲",cent:"¢",centerdot:"·",CenterDot:"·",cfr:"𝔠",Cfr:"ℭ",CHcy:"Ч",chcy:"ч",check:"✓",checkmark:"✓",Chi:"Χ",chi:"χ",circ:"ˆ",circeq:"≗",circlearrowleft:"↺",circlearrowright:"↻",circledast:"⊛",circledcirc:"⊚",circleddash:"⊝",CircleDot:"⊙",circledR:"®",circledS:"Ⓢ",CircleMinus:"⊖",CirclePlus:"⊕",CircleTimes:"⊗",cir:"○",cirE:"⧃",cire:"≗",cirfnint:"⨐",cirmid:"⫯",cirscir:"⧂",ClockwiseContourIntegral:"∲",CloseCurlyDoubleQuote:"”",CloseCurlyQuote:"’",clubs:"♣",clubsuit:"♣",colon:":",Colon:"∷",Colone:"⩴",colone:"≔",coloneq:"≔",comma:",",commat:"@",comp:"∁",compfn:"∘",complement:"∁",complexes:"ℂ",cong:"≅",congdot:"⩭",Congruent:"≡",conint:"∮",Conint:"∯",ContourIntegral:"∮",copf:"𝕔",Copf:"ℂ",coprod:"∐",Coproduct:"∐",copy:"©",COPY:"©",copysr:"℗",CounterClockwiseContourIntegral:"∳",crarr:"↵",cross:"✗",Cross:"⨯",Cscr:"𝒞",cscr:"𝒸",csub:"⫏",csube:"⫑",csup:"⫐",csupe:"⫒",ctdot:"⋯",cudarrl:"⤸",cudarrr:"⤵",cuepr:"⋞",cuesc:"⋟",cularr:"↶",cularrp:"⤽",cupbrcap:"⩈",cupcap:"⩆",CupCap:"≍",cup:"∪",Cup:"⋓",cupcup:"⩊",cupdot:"⊍",cupor:"⩅",cups:"∪︀",curarr:"↷",curarrm:"⤼",curlyeqprec:"⋞",curlyeqsucc:"⋟",curlyvee:"⋎",curlywedge:"⋏",curren:"¤",curvearrowleft:"↶",curvearrowright:"↷",cuvee:"⋎",cuwed:"⋏",cwconint:"∲",cwint:"∱",cylcty:"⌭",dagger:"†",Dagger:"‡",daleth:"ℸ",darr:"↓",Darr:"↡",dArr:"⇓",dash:"‐",Dashv:"⫤",dashv:"⊣",dbkarow:"⤏",dblac:"˝",Dcaron:"Ď",dcaron:"ď",Dcy:"Д",dcy:"д",ddagger:"‡",ddarr:"⇊",DD:"ⅅ",dd:"ⅆ",DDotrahd:"⤑",ddotseq:"⩷",deg:"°",Del:"∇",Delta:"Δ",delta:"δ",demptyv:"⦱",dfisht:"⥿",Dfr:"𝔇",dfr:"𝔡",dHar:"⥥",dharl:"⇃",dharr:"⇂",DiacriticalAcute:"´",DiacriticalDot:"˙",DiacriticalDoubleAcute:"˝",DiacriticalGrave:"`",DiacriticalTilde:"˜",diam:"⋄",diamond:"⋄",Diamond:"⋄",diamondsuit:"♦",diams:"♦",die:"¨",DifferentialD:"ⅆ",digamma:"ϝ",disin:"⋲",div:"÷",divide:"÷",divideontimes:"⋇",divonx:"⋇",DJcy:"Ђ",djcy:"ђ",dlcorn:"⌞",dlcrop:"⌍",dollar:"$",Dopf:"𝔻",dopf:"𝕕",Dot:"¨",dot:"˙",DotDot:"⃜",doteq:"≐",doteqdot:"≑",DotEqual:"≐",dotminus:"∸",dotplus:"∔",dotsquare:"⊡",doublebarwedge:"⌆",DoubleContourIntegral:"∯",DoubleDot:"¨",DoubleDownArrow:"⇓",DoubleLeftArrow:"⇐",DoubleLeftRightArrow:"⇔",DoubleLeftTee:"⫤",DoubleLongLeftArrow:"⟸",DoubleLongLeftRightArrow:"⟺",DoubleLongRightArrow:"⟹",DoubleRightArrow:"⇒",DoubleRightTee:"⊨",DoubleUpArrow:"⇑",DoubleUpDownArrow:"⇕",DoubleVerticalBar:"∥",DownArrowBar:"⤓",downarrow:"↓",DownArrow:"↓",Downarrow:"⇓",DownArrowUpArrow:"⇵",DownBreve:"̑",downdownarrows:"⇊",downharpoonleft:"⇃",downharpoonright:"⇂",DownLeftRightVector:"⥐",DownLeftTeeVector:"⥞",DownLeftVectorBar:"⥖",DownLeftVector:"↽",DownRightTeeVector:"⥟",DownRightVectorBar:"⥗",DownRightVector:"⇁",DownTeeArrow:"↧",DownTee:"⊤",drbkarow:"⤐",drcorn:"⌟",drcrop:"⌌",Dscr:"𝒟",dscr:"𝒹",DScy:"Ѕ",dscy:"ѕ",dsol:"⧶",Dstrok:"Đ",dstrok:"đ",dtdot:"⋱",dtri:"▿",dtrif:"▾",duarr:"⇵",duhar:"⥯",dwangle:"⦦",DZcy:"Џ",dzcy:"џ",dzigrarr:"⟿",Eacute:"É",eacute:"é",easter:"⩮",Ecaron:"Ě",ecaron:"ě",Ecirc:"Ê",ecirc:"ê",ecir:"≖",ecolon:"≕",Ecy:"Э",ecy:"э",eDDot:"⩷",Edot:"Ė",edot:"ė",eDot:"≑",ee:"ⅇ",efDot:"≒",Efr:"𝔈",efr:"𝔢",eg:"⪚",Egrave:"È",egrave:"è",egs:"⪖",egsdot:"⪘",el:"⪙",Element:"∈",elinters:"⏧",ell:"ℓ",els:"⪕",elsdot:"⪗",Emacr:"Ē",emacr:"ē",empty:"∅",emptyset:"∅",EmptySmallSquare:"◻",emptyv:"∅",EmptyVerySmallSquare:"▫",emsp13:" ",emsp14:" ",emsp:" ",ENG:"Ŋ",eng:"ŋ",ensp:" ",Eogon:"Ę",eogon:"ę",Eopf:"𝔼",eopf:"𝕖",epar:"⋕",eparsl:"⧣",eplus:"⩱",epsi:"ε",Epsilon:"Ε",epsilon:"ε",epsiv:"ϵ",eqcirc:"≖",eqcolon:"≕",eqsim:"≂",eqslantgtr:"⪖",eqslantless:"⪕",Equal:"⩵",equals:"=",EqualTilde:"≂",equest:"≟",Equilibrium:"⇌",equiv:"≡",equivDD:"⩸",eqvparsl:"⧥",erarr:"⥱",erDot:"≓",escr:"ℯ",Escr:"ℰ",esdot:"≐",Esim:"⩳",esim:"≂",Eta:"Η",eta:"η",ETH:"Ð",eth:"ð",Euml:"Ë",euml:"ë",euro:"€",excl:"!",exist:"∃",Exists:"∃",expectation:"ℰ",exponentiale:"ⅇ",ExponentialE:"ⅇ",fallingdotseq:"≒",Fcy:"Ф",fcy:"ф",female:"♀",ffilig:"ﬃ",fflig:"ﬀ",ffllig:"ﬄ",Ffr:"𝔉",ffr:"𝔣",filig:"ﬁ",FilledSmallSquare:"◼",FilledVerySmallSquare:"▪",fjlig:"fj",flat:"♭",fllig:"ﬂ",fltns:"▱",fnof:"ƒ",Fopf:"𝔽",fopf:"𝕗",forall:"∀",ForAll:"∀",fork:"⋔",forkv:"⫙",Fouriertrf:"ℱ",fpartint:"⨍",frac12:"½",frac13:"⅓",frac14:"¼",frac15:"⅕",frac16:"⅙",frac18:"⅛",frac23:"⅔",frac25:"⅖",frac34:"¾",frac35:"⅗",frac38:"⅜",frac45:"⅘",frac56:"⅚",frac58:"⅝",frac78:"⅞",frasl:"⁄",frown:"⌢",fscr:"𝒻",Fscr:"ℱ",gacute:"ǵ",Gamma:"Γ",gamma:"γ",Gammad:"Ϝ",gammad:"ϝ",gap:"⪆",Gbreve:"Ğ",gbreve:"ğ",Gcedil:"Ģ",Gcirc:"Ĝ",gcirc:"ĝ",Gcy:"Г",gcy:"г",Gdot:"Ġ",gdot:"ġ",ge:"≥",gE:"≧",gEl:"⪌",gel:"⋛",geq:"≥",geqq:"≧",geqslant:"⩾",gescc:"⪩",ges:"⩾",gesdot:"⪀",gesdoto:"⪂",gesdotol:"⪄",gesl:"⋛︀",gesles:"⪔",Gfr:"𝔊",gfr:"𝔤",gg:"≫",Gg:"⋙",ggg:"⋙",gimel:"ℷ",GJcy:"Ѓ",gjcy:"ѓ",gla:"⪥",gl:"≷",glE:"⪒",glj:"⪤",gnap:"⪊",gnapprox:"⪊",gne:"⪈",gnE:"≩",gneq:"⪈",gneqq:"≩",gnsim:"⋧",Gopf:"𝔾",gopf:"𝕘",grave:"`",GreaterEqual:"≥",GreaterEqualLess:"⋛",GreaterFullEqual:"≧",GreaterGreater:"⪢",GreaterLess:"≷",GreaterSlantEqual:"⩾",GreaterTilde:"≳",Gscr:"𝒢",gscr:"ℊ",gsim:"≳",gsime:"⪎",gsiml:"⪐",gtcc:"⪧",gtcir:"⩺",gt:">",GT:">",Gt:"≫",gtdot:"⋗",gtlPar:"⦕",gtquest:"⩼",gtrapprox:"⪆",gtrarr:"⥸",gtrdot:"⋗",gtreqless:"⋛",gtreqqless:"⪌",gtrless:"≷",gtrsim:"≳",gvertneqq:"≩︀",gvnE:"≩︀",Hacek:"ˇ",hairsp:" ",half:"½",hamilt:"ℋ",HARDcy:"Ъ",hardcy:"ъ",harrcir:"⥈",harr:"↔",hArr:"⇔",harrw:"↭",Hat:"^",hbar:"ℏ",Hcirc:"Ĥ",hcirc:"ĥ",hearts:"♥",heartsuit:"♥",hellip:"…",hercon:"⊹",hfr:"𝔥",Hfr:"ℌ",HilbertSpace:"ℋ",hksearow:"⤥",hkswarow:"⤦",hoarr:"⇿",homtht:"∻",hookleftarrow:"↩",hookrightarrow:"↪",hopf:"𝕙",Hopf:"ℍ",horbar:"―",HorizontalLine:"─",hscr:"𝒽",Hscr:"ℋ",hslash:"ℏ",Hstrok:"Ħ",hstrok:"ħ",HumpDownHump:"≎",HumpEqual:"≏",hybull:"⁃",hyphen:"‐",Iacute:"Í",iacute:"í",ic:"⁣",Icirc:"Î",icirc:"î",Icy:"И",icy:"и",Idot:"İ",IEcy:"Е",iecy:"е",iexcl:"¡",iff:"⇔",ifr:"𝔦",Ifr:"ℑ",Igrave:"Ì",igrave:"ì",ii:"ⅈ",iiiint:"⨌",iiint:"∭",iinfin:"⧜",iiota:"℩",IJlig:"Ĳ",ijlig:"ĳ",Imacr:"Ī",imacr:"ī",image:"ℑ",ImaginaryI:"ⅈ",imagline:"ℐ",imagpart:"ℑ",imath:"ı",Im:"ℑ",imof:"⊷",imped:"Ƶ",Implies:"⇒",incare:"℅",in:"∈",infin:"∞",infintie:"⧝",inodot:"ı",intcal:"⊺",int:"∫",Int:"∬",integers:"ℤ",Integral:"∫",intercal:"⊺",Intersection:"⋂",intlarhk:"⨗",intprod:"⨼",InvisibleComma:"⁣",InvisibleTimes:"⁢",IOcy:"Ё",iocy:"ё",Iogon:"Į",iogon:"į",Iopf:"𝕀",iopf:"𝕚",Iota:"Ι",iota:"ι",iprod:"⨼",iquest:"¿",iscr:"𝒾",Iscr:"ℐ",isin:"∈",isindot:"⋵",isinE:"⋹",isins:"⋴",isinsv:"⋳",isinv:"∈",it:"⁢",Itilde:"Ĩ",itilde:"ĩ",Iukcy:"І",iukcy:"і",Iuml:"Ï",iuml:"ï",Jcirc:"Ĵ",jcirc:"ĵ",Jcy:"Й",jcy:"й",Jfr:"𝔍",jfr:"𝔧",jmath:"ȷ",Jopf:"𝕁",jopf:"𝕛",Jscr:"𝒥",jscr:"𝒿",Jsercy:"Ј",jsercy:"ј",Jukcy:"Є",jukcy:"є",Kappa:"Κ",kappa:"κ",kappav:"ϰ",Kcedil:"Ķ",kcedil:"ķ",Kcy:"К",kcy:"к",Kfr:"𝔎",kfr:"𝔨",kgreen:"ĸ",KHcy:"Х",khcy:"х",KJcy:"Ќ",kjcy:"ќ",Kopf:"𝕂",kopf:"𝕜",Kscr:"𝒦",kscr:"𝓀",lAarr:"⇚",Lacute:"Ĺ",lacute:"ĺ",laemptyv:"⦴",lagran:"ℒ",Lambda:"Λ",lambda:"λ",lang:"⟨",Lang:"⟪",langd:"⦑",langle:"⟨",lap:"⪅",Laplacetrf:"ℒ",laquo:"«",larrb:"⇤",larrbfs:"⤟",larr:"←",Larr:"↞",lArr:"⇐",larrfs:"⤝",larrhk:"↩",larrlp:"↫",larrpl:"⤹",larrsim:"⥳",larrtl:"↢",latail:"⤙",lAtail:"⤛",lat:"⪫",late:"⪭",lates:"⪭︀",lbarr:"⤌",lBarr:"⤎",lbbrk:"❲",lbrace:"{",lbrack:"[",lbrke:"⦋",lbrksld:"⦏",lbrkslu:"⦍",Lcaron:"Ľ",lcaron:"ľ",Lcedil:"Ļ",lcedil:"ļ",lceil:"⌈",lcub:"{",Lcy:"Л",lcy:"л",ldca:"⤶",ldquo:"“",ldquor:"„",ldrdhar:"⥧",ldrushar:"⥋",ldsh:"↲",le:"≤",lE:"≦",LeftAngleBracket:"⟨",LeftArrowBar:"⇤",leftarrow:"←",LeftArrow:"←",Leftarrow:"⇐",LeftArrowRightArrow:"⇆",leftarrowtail:"↢",LeftCeiling:"⌈",LeftDoubleBracket:"⟦",LeftDownTeeVector:"⥡",LeftDownVectorBar:"⥙",LeftDownVector:"⇃",LeftFloor:"⌊",leftharpoondown:"↽",leftharpoonup:"↼",leftleftarrows:"⇇",leftrightarrow:"↔",LeftRightArrow:"↔",Leftrightarrow:"⇔",leftrightarrows:"⇆",leftrightharpoons:"⇋",leftrightsquigarrow:"↭",LeftRightVector:"⥎",LeftTeeArrow:"↤",LeftTee:"⊣",LeftTeeVector:"⥚",leftthreetimes:"⋋",LeftTriangleBar:"⧏",LeftTriangle:"⊲",LeftTriangleEqual:"⊴",LeftUpDownVector:"⥑",LeftUpTeeVector:"⥠",LeftUpVectorBar:"⥘",LeftUpVector:"↿",LeftVectorBar:"⥒",LeftVector:"↼",lEg:"⪋",leg:"⋚",leq:"≤",leqq:"≦",leqslant:"⩽",lescc:"⪨",les:"⩽",lesdot:"⩿",lesdoto:"⪁",lesdotor:"⪃",lesg:"⋚︀",lesges:"⪓",lessapprox:"⪅",lessdot:"⋖",lesseqgtr:"⋚",lesseqqgtr:"⪋",LessEqualGreater:"⋚",LessFullEqual:"≦",LessGreater:"≶",lessgtr:"≶",LessLess:"⪡",lesssim:"≲",LessSlantEqual:"⩽",LessTilde:"≲",lfisht:"⥼",lfloor:"⌊",Lfr:"𝔏",lfr:"𝔩",lg:"≶",lgE:"⪑",lHar:"⥢",lhard:"↽",lharu:"↼",lharul:"⥪",lhblk:"▄",LJcy:"Љ",ljcy:"љ",llarr:"⇇",ll:"≪",Ll:"⋘",llcorner:"⌞",Lleftarrow:"⇚",llhard:"⥫",lltri:"◺",Lmidot:"Ŀ",lmidot:"ŀ",lmoustache:"⎰",lmoust:"⎰",lnap:"⪉",lnapprox:"⪉",lne:"⪇",lnE:"≨",lneq:"⪇",lneqq:"≨",lnsim:"⋦",loang:"⟬",loarr:"⇽",lobrk:"⟦",longleftarrow:"⟵",LongLeftArrow:"⟵",Longleftarrow:"⟸",longleftrightarrow:"⟷",LongLeftRightArrow:"⟷",Longleftrightarrow:"⟺",longmapsto:"⟼",longrightarrow:"⟶",LongRightArrow:"⟶",Longrightarrow:"⟹",looparrowleft:"↫",looparrowright:"↬",lopar:"⦅",Lopf:"𝕃",lopf:"𝕝",loplus:"⨭",lotimes:"⨴",lowast:"∗",lowbar:"_",LowerLeftArrow:"↙",LowerRightArrow:"↘",loz:"◊",lozenge:"◊",lozf:"⧫",lpar:"(",lparlt:"⦓",lrarr:"⇆",lrcorner:"⌟",lrhar:"⇋",lrhard:"⥭",lrm:"‎",lrtri:"⊿",lsaquo:"‹",lscr:"𝓁",Lscr:"ℒ",lsh:"↰",Lsh:"↰",lsim:"≲",lsime:"⪍",lsimg:"⪏",lsqb:"[",lsquo:"‘",lsquor:"‚",Lstrok:"Ł",lstrok:"ł",ltcc:"⪦",ltcir:"⩹",lt:"<",LT:"<",Lt:"≪",ltdot:"⋖",lthree:"⋋",ltimes:"⋉",ltlarr:"⥶",ltquest:"⩻",ltri:"◃",ltrie:"⊴",ltrif:"◂",ltrPar:"⦖",lurdshar:"⥊",luruhar:"⥦",lvertneqq:"≨︀",lvnE:"≨︀",macr:"¯",male:"♂",malt:"✠",maltese:"✠",Map:"⤅",map:"↦",mapsto:"↦",mapstodown:"↧",mapstoleft:"↤",mapstoup:"↥",marker:"▮",mcomma:"⨩",Mcy:"М",mcy:"м",mdash:"—",mDDot:"∺",measuredangle:"∡",MediumSpace:" ",Mellintrf:"ℳ",Mfr:"𝔐",mfr:"𝔪",mho:"℧",micro:"µ",midast:"*",midcir:"⫰",mid:"∣",middot:"·",minusb:"⊟",minus:"−",minusd:"∸",minusdu:"⨪",MinusPlus:"∓",mlcp:"⫛",mldr:"…",mnplus:"∓",models:"⊧",Mopf:"𝕄",mopf:"𝕞",mp:"∓",mscr:"𝓂",Mscr:"ℳ",mstpos:"∾",Mu:"Μ",mu:"μ",multimap:"⊸",mumap:"⊸",nabla:"∇",Nacute:"Ń",nacute:"ń",nang:"∠⃒",nap:"≉",napE:"⩰̸",napid:"≋̸",napos:"ŉ",napprox:"≉",natural:"♮",naturals:"ℕ",natur:"♮",nbsp:" ",nbump:"≎̸",nbumpe:"≏̸",ncap:"⩃",Ncaron:"Ň",ncaron:"ň",Ncedil:"Ņ",ncedil:"ņ",ncong:"≇",ncongdot:"⩭̸",ncup:"⩂",Ncy:"Н",ncy:"н",ndash:"–",nearhk:"⤤",nearr:"↗",neArr:"⇗",nearrow:"↗",ne:"≠",nedot:"≐̸",NegativeMediumSpace:"​",NegativeThickSpace:"​",NegativeThinSpace:"​",NegativeVeryThinSpace:"​",nequiv:"≢",nesear:"⤨",nesim:"≂̸",NestedGreaterGreater:"≫",NestedLessLess:"≪",NewLine:"\n",nexist:"∄",nexists:"∄",Nfr:"𝔑",nfr:"𝔫",ngE:"≧̸",nge:"≱",ngeq:"≱",ngeqq:"≧̸",ngeqslant:"⩾̸",nges:"⩾̸",nGg:"⋙̸",ngsim:"≵",nGt:"≫⃒",ngt:"≯",ngtr:"≯",nGtv:"≫̸",nharr:"↮",nhArr:"⇎",nhpar:"⫲",ni:"∋",nis:"⋼",nisd:"⋺",niv:"∋",NJcy:"Њ",njcy:"њ",nlarr:"↚",nlArr:"⇍",nldr:"‥",nlE:"≦̸",nle:"≰",nleftarrow:"↚",nLeftarrow:"⇍",nleftrightarrow:"↮",nLeftrightarrow:"⇎",nleq:"≰",nleqq:"≦̸",nleqslant:"⩽̸",nles:"⩽̸",nless:"≮",nLl:"⋘̸",nlsim:"≴",nLt:"≪⃒",nlt:"≮",nltri:"⋪",nltrie:"⋬",nLtv:"≪̸",nmid:"∤",NoBreak:"⁠",NonBreakingSpace:" ",nopf:"𝕟",Nopf:"ℕ",Not:"⫬",not:"¬",NotCongruent:"≢",NotCupCap:"≭",NotDoubleVerticalBar:"∦",NotElement:"∉",NotEqual:"≠",NotEqualTilde:"≂̸",NotExists:"∄",NotGreater:"≯",NotGreaterEqual:"≱",NotGreaterFullEqual:"≧̸",NotGreaterGreater:"≫̸",NotGreaterLess:"≹",NotGreaterSlantEqual:"⩾̸",NotGreaterTilde:"≵",NotHumpDownHump:"≎̸",NotHumpEqual:"≏̸",notin:"∉",notindot:"⋵̸",notinE:"⋹̸",notinva:"∉",notinvb:"⋷",notinvc:"⋶",NotLeftTriangleBar:"⧏̸",NotLeftTriangle:"⋪",NotLeftTriangleEqual:"⋬",NotLess:"≮",NotLessEqual:"≰",NotLessGreater:"≸",NotLessLess:"≪̸",NotLessSlantEqual:"⩽̸",NotLessTilde:"≴",NotNestedGreaterGreater:"⪢̸",NotNestedLessLess:"⪡̸",notni:"∌",notniva:"∌",notnivb:"⋾",notnivc:"⋽",NotPrecedes:"⊀",NotPrecedesEqual:"⪯̸",NotPrecedesSlantEqual:"⋠",NotReverseElement:"∌",NotRightTriangleBar:"⧐̸",NotRightTriangle:"⋫",NotRightTriangleEqual:"⋭",NotSquareSubset:"⊏̸",NotSquareSubsetEqual:"⋢",NotSquareSuperset:"⊐̸",NotSquareSupersetEqual:"⋣",NotSubset:"⊂⃒",NotSubsetEqual:"⊈",NotSucceeds:"⊁",NotSucceedsEqual:"⪰̸",NotSucceedsSlantEqual:"⋡",NotSucceedsTilde:"≿̸",NotSuperset:"⊃⃒",NotSupersetEqual:"⊉",NotTilde:"≁",NotTildeEqual:"≄",NotTildeFullEqual:"≇",NotTildeTilde:"≉",NotVerticalBar:"∤",nparallel:"∦",npar:"∦",nparsl:"⫽⃥",npart:"∂̸",npolint:"⨔",npr:"⊀",nprcue:"⋠",nprec:"⊀",npreceq:"⪯̸",npre:"⪯̸",nrarrc:"⤳̸",nrarr:"↛",nrArr:"⇏",nrarrw:"↝̸",nrightarrow:"↛",nRightarrow:"⇏",nrtri:"⋫",nrtrie:"⋭",nsc:"⊁",nsccue:"⋡",nsce:"⪰̸",Nscr:"𝒩",nscr:"𝓃",nshortmid:"∤",nshortparallel:"∦",nsim:"≁",nsime:"≄",nsimeq:"≄",nsmid:"∤",nspar:"∦",nsqsube:"⋢",nsqsupe:"⋣",nsub:"⊄",nsubE:"⫅̸",nsube:"⊈",nsubset:"⊂⃒",nsubseteq:"⊈",nsubseteqq:"⫅̸",nsucc:"⊁",nsucceq:"⪰̸",nsup:"⊅",nsupE:"⫆̸",nsupe:"⊉",nsupset:"⊃⃒",nsupseteq:"⊉",nsupseteqq:"⫆̸",ntgl:"≹",Ntilde:"Ñ",ntilde:"ñ",ntlg:"≸",ntriangleleft:"⋪",ntrianglelefteq:"⋬",ntriangleright:"⋫",ntrianglerighteq:"⋭",Nu:"Ν",nu:"ν",num:"#",numero:"№",numsp:" ",nvap:"≍⃒",nvdash:"⊬",nvDash:"⊭",nVdash:"⊮",nVDash:"⊯",nvge:"≥⃒",nvgt:">⃒",nvHarr:"⤄",nvinfin:"⧞",nvlArr:"⤂",nvle:"≤⃒",nvlt:"<⃒",nvltrie:"⊴⃒",nvrArr:"⤃",nvrtrie:"⊵⃒",nvsim:"∼⃒",nwarhk:"⤣",nwarr:"↖",nwArr:"⇖",nwarrow:"↖",nwnear:"⤧",Oacute:"Ó",oacute:"ó",oast:"⊛",Ocirc:"Ô",ocirc:"ô",ocir:"⊚",Ocy:"О",ocy:"о",odash:"⊝",Odblac:"Ő",odblac:"ő",odiv:"⨸",odot:"⊙",odsold:"⦼",OElig:"Œ",oelig:"œ",ofcir:"⦿",Ofr:"𝔒",ofr:"𝔬",ogon:"˛",Ograve:"Ò",ograve:"ò",ogt:"⧁",ohbar:"⦵",ohm:"Ω",oint:"∮",olarr:"↺",olcir:"⦾",olcross:"⦻",oline:"‾",olt:"⧀",Omacr:"Ō",omacr:"ō",Omega:"Ω",omega:"ω",Omicron:"Ο",omicron:"ο",omid:"⦶",ominus:"⊖",Oopf:"𝕆",oopf:"𝕠",opar:"⦷",OpenCurlyDoubleQuote:"“",OpenCurlyQuote:"‘",operp:"⦹",oplus:"⊕",orarr:"↻",Or:"⩔",or:"∨",ord:"⩝",order:"ℴ",orderof:"ℴ",ordf:"ª",ordm:"º",origof:"⊶",oror:"⩖",orslope:"⩗",orv:"⩛",oS:"Ⓢ",Oscr:"𝒪",oscr:"ℴ",Oslash:"Ø",oslash:"ø",osol:"⊘",Otilde:"Õ",otilde:"õ",otimesas:"⨶",Otimes:"⨷",otimes:"⊗",Ouml:"Ö",ouml:"ö",ovbar:"⌽",OverBar:"‾",OverBrace:"⏞",OverBracket:"⎴",OverParenthesis:"⏜",para:"¶",parallel:"∥",par:"∥",parsim:"⫳",parsl:"⫽",part:"∂",PartialD:"∂",Pcy:"П",pcy:"п",percnt:"%",period:".",permil:"‰",perp:"⊥",pertenk:"‱",Pfr:"𝔓",pfr:"𝔭",Phi:"Φ",phi:"φ",phiv:"ϕ",phmmat:"ℳ",phone:"☎",Pi:"Π",pi:"π",pitchfork:"⋔",piv:"ϖ",planck:"ℏ",planckh:"ℎ",plankv:"ℏ",plusacir:"⨣",plusb:"⊞",pluscir:"⨢",plus:"+",plusdo:"∔",plusdu:"⨥",pluse:"⩲",PlusMinus:"±",plusmn:"±",plussim:"⨦",plustwo:"⨧",pm:"±",Poincareplane:"ℌ",pointint:"⨕",popf:"𝕡",Popf:"ℙ",pound:"£",prap:"⪷",Pr:"⪻",pr:"≺",prcue:"≼",precapprox:"⪷",prec:"≺",preccurlyeq:"≼",Precedes:"≺",PrecedesEqual:"⪯",PrecedesSlantEqual:"≼",PrecedesTilde:"≾",preceq:"⪯",precnapprox:"⪹",precneqq:"⪵",precnsim:"⋨",pre:"⪯",prE:"⪳",precsim:"≾",prime:"′",Prime:"″",primes:"ℙ",prnap:"⪹",prnE:"⪵",prnsim:"⋨",prod:"∏",Product:"∏",profalar:"⌮",profline:"⌒",profsurf:"⌓",prop:"∝",Proportional:"∝",Proportion:"∷",propto:"∝",prsim:"≾",prurel:"⊰",Pscr:"𝒫",pscr:"𝓅",Psi:"Ψ",psi:"ψ",puncsp:" ",Qfr:"𝔔",qfr:"𝔮",qint:"⨌",qopf:"𝕢",Qopf:"ℚ",qprime:"⁗",Qscr:"𝒬",qscr:"𝓆",quaternions:"ℍ",quatint:"⨖",quest:"?",questeq:"≟",quot:'"',QUOT:'"',rAarr:"⇛",race:"∽̱",Racute:"Ŕ",racute:"ŕ",radic:"√",raemptyv:"⦳",rang:"⟩",Rang:"⟫",rangd:"⦒",range:"⦥",rangle:"⟩",raquo:"»",rarrap:"⥵",rarrb:"⇥",rarrbfs:"⤠",rarrc:"⤳",rarr:"→",Rarr:"↠",rArr:"⇒",rarrfs:"⤞",rarrhk:"↪",rarrlp:"↬",rarrpl:"⥅",rarrsim:"⥴",Rarrtl:"⤖",rarrtl:"↣",rarrw:"↝",ratail:"⤚",rAtail:"⤜",ratio:"∶",rationals:"ℚ",rbarr:"⤍",rBarr:"⤏",RBarr:"⤐",rbbrk:"❳",rbrace:"}",rbrack:"]",rbrke:"⦌",rbrksld:"⦎",rbrkslu:"⦐",Rcaron:"Ř",rcaron:"ř",Rcedil:"Ŗ",rcedil:"ŗ",rceil:"⌉",rcub:"}",Rcy:"Р",rcy:"р",rdca:"⤷",rdldhar:"⥩",rdquo:"”",rdquor:"”",rdsh:"↳",real:"ℜ",realine:"ℛ",realpart:"ℜ",reals:"ℝ",Re:"ℜ",rect:"▭",reg:"®",REG:"®",ReverseElement:"∋",ReverseEquilibrium:"⇋",ReverseUpEquilibrium:"⥯",rfisht:"⥽",rfloor:"⌋",rfr:"𝔯",Rfr:"ℜ",rHar:"⥤",rhard:"⇁",rharu:"⇀",rharul:"⥬",Rho:"Ρ",rho:"ρ",rhov:"ϱ",RightAngleBracket:"⟩",RightArrowBar:"⇥",rightarrow:"→",RightArrow:"→",Rightarrow:"⇒",RightArrowLeftArrow:"⇄",rightarrowtail:"↣",RightCeiling:"⌉",RightDoubleBracket:"⟧",RightDownTeeVector:"⥝",RightDownVectorBar:"⥕",RightDownVector:"⇂",RightFloor:"⌋",rightharpoondown:"⇁",rightharpoonup:"⇀",rightleftarrows:"⇄",rightleftharpoons:"⇌",rightrightarrows:"⇉",rightsquigarrow:"↝",RightTeeArrow:"↦",RightTee:"⊢",RightTeeVector:"⥛",rightthreetimes:"⋌",RightTriangleBar:"⧐",RightTriangle:"⊳",RightTriangleEqual:"⊵",RightUpDownVector:"⥏",RightUpTeeVector:"⥜",RightUpVectorBar:"⥔",RightUpVector:"↾",RightVectorBar:"⥓",RightVector:"⇀",ring:"˚",risingdotseq:"≓",rlarr:"⇄",rlhar:"⇌",rlm:"‏",rmoustache:"⎱",rmoust:"⎱",rnmid:"⫮",roang:"⟭",roarr:"⇾",robrk:"⟧",ropar:"⦆",ropf:"𝕣",Ropf:"ℝ",roplus:"⨮",rotimes:"⨵",RoundImplies:"⥰",rpar:")",rpargt:"⦔",rppolint:"⨒",rrarr:"⇉",Rrightarrow:"⇛",rsaquo:"›",rscr:"𝓇",Rscr:"ℛ",rsh:"↱",Rsh:"↱",rsqb:"]",rsquo:"’",rsquor:"’",rthree:"⋌",rtimes:"⋊",rtri:"▹",rtrie:"⊵",rtrif:"▸",rtriltri:"⧎",RuleDelayed:"⧴",ruluhar:"⥨",rx:"℞",Sacute:"Ś",sacute:"ś",sbquo:"‚",scap:"⪸",Scaron:"Š",scaron:"š",Sc:"⪼",sc:"≻",sccue:"≽",sce:"⪰",scE:"⪴",Scedil:"Ş",scedil:"ş",Scirc:"Ŝ",scirc:"ŝ",scnap:"⪺",scnE:"⪶",scnsim:"⋩",scpolint:"⨓",scsim:"≿",Scy:"С",scy:"с",sdotb:"⊡",sdot:"⋅",sdote:"⩦",searhk:"⤥",searr:"↘",seArr:"⇘",searrow:"↘",sect:"§",semi:";",seswar:"⤩",setminus:"∖",setmn:"∖",sext:"✶",Sfr:"𝔖",sfr:"𝔰",sfrown:"⌢",sharp:"♯",SHCHcy:"Щ",shchcy:"щ",SHcy:"Ш",shcy:"ш",ShortDownArrow:"↓",ShortLeftArrow:"←",shortmid:"∣",shortparallel:"∥",ShortRightArrow:"→",ShortUpArrow:"↑",shy:"­",Sigma:"Σ",sigma:"σ",sigmaf:"ς",sigmav:"ς",sim:"∼",simdot:"⩪",sime:"≃",simeq:"≃",simg:"⪞",simgE:"⪠",siml:"⪝",simlE:"⪟",simne:"≆",simplus:"⨤",simrarr:"⥲",slarr:"←",SmallCircle:"∘",smallsetminus:"∖",smashp:"⨳",smeparsl:"⧤",smid:"∣",smile:"⌣",smt:"⪪",smte:"⪬",smtes:"⪬︀",SOFTcy:"Ь",softcy:"ь",solbar:"⌿",solb:"⧄",sol:"/",Sopf:"𝕊",sopf:"𝕤",spades:"♠",spadesuit:"♠",spar:"∥",sqcap:"⊓",sqcaps:"⊓︀",sqcup:"⊔",sqcups:"⊔︀",Sqrt:"√",sqsub:"⊏",sqsube:"⊑",sqsubset:"⊏",sqsubseteq:"⊑",sqsup:"⊐",sqsupe:"⊒",sqsupset:"⊐",sqsupseteq:"⊒",square:"□",Square:"□",SquareIntersection:"⊓",SquareSubset:"⊏",SquareSubsetEqual:"⊑",SquareSuperset:"⊐",SquareSupersetEqual:"⊒",SquareUnion:"⊔",squarf:"▪",squ:"□",squf:"▪",srarr:"→",Sscr:"𝒮",sscr:"𝓈",ssetmn:"∖",ssmile:"⌣",sstarf:"⋆",Star:"⋆",star:"☆",starf:"★",straightepsilon:"ϵ",straightphi:"ϕ",strns:"¯",sub:"⊂",Sub:"⋐",subdot:"⪽",subE:"⫅",sube:"⊆",subedot:"⫃",submult:"⫁",subnE:"⫋",subne:"⊊",subplus:"⪿",subrarr:"⥹",subset:"⊂",Subset:"⋐",subseteq:"⊆",subseteqq:"⫅",SubsetEqual:"⊆",subsetneq:"⊊",subsetneqq:"⫋",subsim:"⫇",subsub:"⫕",subsup:"⫓",succapprox:"⪸",succ:"≻",succcurlyeq:"≽",Succeeds:"≻",SucceedsEqual:"⪰",SucceedsSlantEqual:"≽",SucceedsTilde:"≿",succeq:"⪰",succnapprox:"⪺",succneqq:"⪶",succnsim:"⋩",succsim:"≿",SuchThat:"∋",sum:"∑",Sum:"∑",sung:"♪",sup1:"¹",sup2:"²",sup3:"³",sup:"⊃",Sup:"⋑",supdot:"⪾",supdsub:"⫘",supE:"⫆",supe:"⊇",supedot:"⫄",Superset:"⊃",SupersetEqual:"⊇",suphsol:"⟉",suphsub:"⫗",suplarr:"⥻",supmult:"⫂",supnE:"⫌",supne:"⊋",supplus:"⫀",supset:"⊃",Supset:"⋑",supseteq:"⊇",supseteqq:"⫆",supsetneq:"⊋",supsetneqq:"⫌",supsim:"⫈",supsub:"⫔",supsup:"⫖",swarhk:"⤦",swarr:"↙",swArr:"⇙",swarrow:"↙",swnwar:"⤪",szlig:"ß",Tab:"\t",target:"⌖",Tau:"Τ",tau:"τ",tbrk:"⎴",Tcaron:"Ť",tcaron:"ť",Tcedil:"Ţ",tcedil:"ţ",Tcy:"Т",tcy:"т",tdot:"⃛",telrec:"⌕",Tfr:"𝔗",tfr:"𝔱",there4:"∴",therefore:"∴",Therefore:"∴",Theta:"Θ",theta:"θ",thetasym:"ϑ",thetav:"ϑ",thickapprox:"≈",thicksim:"∼",ThickSpace:"  ",ThinSpace:" ",thinsp:" ",thkap:"≈",thksim:"∼",THORN:"Þ",thorn:"þ",tilde:"˜",Tilde:"∼",TildeEqual:"≃",TildeFullEqual:"≅",TildeTilde:"≈",timesbar:"⨱",timesb:"⊠",times:"×",timesd:"⨰",tint:"∭",toea:"⤨",topbot:"⌶",topcir:"⫱",top:"⊤",Topf:"𝕋",topf:"𝕥",topfork:"⫚",tosa:"⤩",tprime:"‴",trade:"™",TRADE:"™",triangle:"▵",triangledown:"▿",triangleleft:"◃",trianglelefteq:"⊴",triangleq:"≜",triangleright:"▹",trianglerighteq:"⊵",tridot:"◬",trie:"≜",triminus:"⨺",TripleDot:"⃛",triplus:"⨹",trisb:"⧍",tritime:"⨻",trpezium:"⏢",Tscr:"𝒯",tscr:"𝓉",TScy:"Ц",tscy:"ц",TSHcy:"Ћ",tshcy:"ћ",Tstrok:"Ŧ",tstrok:"ŧ",twixt:"≬",twoheadleftarrow:"↞",twoheadrightarrow:"↠",Uacute:"Ú",uacute:"ú",uarr:"↑",Uarr:"↟",uArr:"⇑",Uarrocir:"⥉",Ubrcy:"Ў",ubrcy:"ў",Ubreve:"Ŭ",ubreve:"ŭ",Ucirc:"Û",ucirc:"û",Ucy:"У",ucy:"у",udarr:"⇅",Udblac:"Ű",udblac:"ű",udhar:"⥮",ufisht:"⥾",Ufr:"𝔘",ufr:"𝔲",Ugrave:"Ù",ugrave:"ù",uHar:"⥣",uharl:"↿",uharr:"↾",uhblk:"▀",ulcorn:"⌜",ulcorner:"⌜",ulcrop:"⌏",ultri:"◸",Umacr:"Ū",umacr:"ū",uml:"¨",UnderBar:"_",UnderBrace:"⏟",UnderBracket:"⎵",UnderParenthesis:"⏝",Union:"⋃",UnionPlus:"⊎",Uogon:"Ų",uogon:"ų",Uopf:"𝕌",uopf:"𝕦",UpArrowBar:"⤒",uparrow:"↑",UpArrow:"↑",Uparrow:"⇑",UpArrowDownArrow:"⇅",updownarrow:"↕",UpDownArrow:"↕",Updownarrow:"⇕",UpEquilibrium:"⥮",upharpoonleft:"↿",upharpoonright:"↾",uplus:"⊎",UpperLeftArrow:"↖",UpperRightArrow:"↗",upsi:"υ",Upsi:"ϒ",upsih:"ϒ",Upsilon:"Υ",upsilon:"υ",UpTeeArrow:"↥",UpTee:"⊥",upuparrows:"⇈",urcorn:"⌝",urcorner:"⌝",urcrop:"⌎",Uring:"Ů",uring:"ů",urtri:"◹",Uscr:"𝒰",uscr:"𝓊",utdot:"⋰",Utilde:"Ũ",utilde:"ũ",utri:"▵",utrif:"▴",uuarr:"⇈",Uuml:"Ü",uuml:"ü",uwangle:"⦧",vangrt:"⦜",varepsilon:"ϵ",varkappa:"ϰ",varnothing:"∅",varphi:"ϕ",varpi:"ϖ",varpropto:"∝",varr:"↕",vArr:"⇕",varrho:"ϱ",varsigma:"ς",varsubsetneq:"⊊︀",varsubsetneqq:"⫋︀",varsupsetneq:"⊋︀",varsupsetneqq:"⫌︀",vartheta:"ϑ",vartriangleleft:"⊲",vartriangleright:"⊳",vBar:"⫨",Vbar:"⫫",vBarv:"⫩",Vcy:"В",vcy:"в",vdash:"⊢",vDash:"⊨",Vdash:"⊩",VDash:"⊫",Vdashl:"⫦",veebar:"⊻",vee:"∨",Vee:"⋁",veeeq:"≚",vellip:"⋮",verbar:"|",Verbar:"‖",vert:"|",Vert:"‖",VerticalBar:"∣",VerticalLine:"|",VerticalSeparator:"❘",VerticalTilde:"≀",VeryThinSpace:" ",Vfr:"𝔙",vfr:"𝔳",vltri:"⊲",vnsub:"⊂⃒",vnsup:"⊃⃒",Vopf:"𝕍",vopf:"𝕧",vprop:"∝",vrtri:"⊳",Vscr:"𝒱",vscr:"𝓋",vsubnE:"⫋︀",vsubne:"⊊︀",vsupnE:"⫌︀",vsupne:"⊋︀",Vvdash:"⊪",vzigzag:"⦚",Wcirc:"Ŵ",wcirc:"ŵ",wedbar:"⩟",wedge:"∧",Wedge:"⋀",wedgeq:"≙",weierp:"℘",Wfr:"𝔚",wfr:"𝔴",Wopf:"𝕎",wopf:"𝕨",wp:"℘",wr:"≀",wreath:"≀",Wscr:"𝒲",wscr:"𝓌",xcap:"⋂",xcirc:"◯",xcup:"⋃",xdtri:"▽",Xfr:"𝔛",xfr:"𝔵",xharr:"⟷",xhArr:"⟺",Xi:"Ξ",xi:"ξ",xlarr:"⟵",xlArr:"⟸",xmap:"⟼",xnis:"⋻",xodot:"⨀",Xopf:"𝕏",xopf:"𝕩",xoplus:"⨁",xotime:"⨂",xrarr:"⟶",xrArr:"⟹",Xscr:"𝒳",xscr:"𝓍",xsqcup:"⨆",xuplus:"⨄",xutri:"△",xvee:"⋁",xwedge:"⋀",Yacute:"Ý",yacute:"ý",YAcy:"Я",yacy:"я",Ycirc:"Ŷ",ycirc:"ŷ",Ycy:"Ы",ycy:"ы",yen:"¥",Yfr:"𝔜",yfr:"𝔶",YIcy:"Ї",yicy:"ї",Yopf:"𝕐",yopf:"𝕪",Yscr:"𝒴",yscr:"𝓎",YUcy:"Ю",yucy:"ю",yuml:"ÿ",Yuml:"Ÿ",Zacute:"Ź",zacute:"ź",Zcaron:"Ž",zcaron:"ž",Zcy:"З",zcy:"з",Zdot:"Ż",zdot:"ż",zeetrf:"ℨ",ZeroWidthSpace:"​",Zeta:"Ζ",zeta:"ζ",zfr:"𝔷",Zfr:"ℨ",ZHcy:"Ж",zhcy:"ж",zigrarr:"⇝",zopf:"𝕫",Zopf:"ℤ",Zscr:"𝒵",zscr:"𝓏",zwj:"‍",zwnj:"‌"}},{}],23:[function(e,t,n){t.exports={Aacute:"Á",aacute:"á",Acirc:"Â",acirc:"â",acute:"´",AElig:"Æ",aelig:"æ",Agrave:"À",agrave:"à",amp:"&",AMP:"&",Aring:"Å",aring:"å",Atilde:"Ã",atilde:"ã",Auml:"Ä",auml:"ä",brvbar:"¦",Ccedil:"Ç",ccedil:"ç",cedil:"¸",cent:"¢",copy:"©",COPY:"©",curren:"¤",deg:"°",divide:"÷",Eacute:"É",eacute:"é",Ecirc:"Ê",ecirc:"ê",Egrave:"È",egrave:"è",ETH:"Ð",eth:"ð",Euml:"Ë",euml:"ë",frac12:"½",frac14:"¼",frac34:"¾",gt:">",GT:">",Iacute:"Í",iacute:"í",Icirc:"Î",icirc:"î",iexcl:"¡",Igrave:"Ì",igrave:"ì",iquest:"¿",Iuml:"Ï",iuml:"ï",laquo:"«",lt:"<",LT:"<",macr:"¯",micro:"µ",middot:"·",nbsp:" ",not:"¬",Ntilde:"Ñ",ntilde:"ñ",Oacute:"Ó",oacute:"ó",Ocirc:"Ô",ocirc:"ô",Ograve:"Ò",ograve:"ò",ordf:"ª",ordm:"º",Oslash:"Ø",oslash:"ø",Otilde:"Õ",otilde:"õ",Ouml:"Ö",ouml:"ö",para:"¶",plusmn:"±",pound:"£",quot:'"',QUOT:'"',raquo:"»",reg:"®",REG:"®",sect:"§",shy:"­",sup1:"¹",sup2:"²",sup3:"³",szlig:"ß",THORN:"Þ",thorn:"þ",times:"×",Uacute:"Ú",uacute:"ú",Ucirc:"Û",ucirc:"û",Ugrave:"Ù",ugrave:"ù",uml:"¨",Uuml:"Ü",uuml:"ü",Yacute:"Ý",yacute:"ý",yen:"¥",yuml:"ÿ"}},{}],24:[function(e,t,n){t.exports={amp:"&",apos:"'",gt:">",lt:"<",quot:'"'}},{}],25:[function(e,t,n){var r=Object.create||function(e){var t=function(){};return t.prototype=e,new t},i=Object.keys||function(e){var t=[];for(var n in e)Object.prototype.hasOwnProperty.call(e,n)&&t.push(n);return n},o=Function.prototype.bind||function(e){var t=this;return function(){return t.apply(e,arguments)}};function a(){this._events&&Object.prototype.hasOwnProperty.call(this,"_events")||(this._events=r(null),this._eventsCount=0),this._maxListeners=this._maxListeners||void 0}t.exports=a,a.EventEmitter=a,a.prototype._events=void 0,a.prototype._maxListeners=void 0;var s,l=10;try{var c={};Object.defineProperty&&Object.defineProperty(c,"x",{value:0}),s=0===c.x}catch(e){s=!1}function d(e){return void 0===e._maxListeners?a.defaultMaxListeners:e._maxListeners}function h(e,t,n){if(t)e.call(n);else for(var r=e.length,i=_(e,r),o=0;o<r;++o)i[o].call(n)}function f(e,t,n,r){if(t)e.call(n,r);else for(var i=e.length,o=_(e,i),a=0;a<i;++a)o[a].call(n,r)}function p(e,t,n,r,i){if(t)e.call(n,r,i);else for(var o=e.length,a=_(e,o),s=0;s<o;++s)a[s].call(n,r,i)}function g(e,t,n,r,i,o){if(t)e.call(n,r,i,o);else for(var a=e.length,s=_(e,a),l=0;l<a;++l)s[l].call(n,r,i,o)}function v(e,t,n,r){if(t)e.apply(n,r);else for(var i=e.length,o=_(e,i),a=0;a<i;++a)o[a].apply(n,r)}function m(e,t,n,i){var o,a,s;if("function"!=typeof n)throw new TypeError('"listener" argument must be a function');if((a=e._events)?(a.newListener&&(e.emit("newListener",t,n.listener?n.listener:n),a=e._events),s=a[t]):(a=e._events=r(null),e._eventsCount=0),s){if("function"==typeof s?s=a[t]=i?[n,s]:[s,n]:i?s.unshift(n):s.push(n),!s.warned&&(o=d(e))&&o>0&&s.length>o){s.warned=!0;var l=new Error("Possible EventEmitter memory leak detected. "+s.length+' "'+String(t)+'" listeners added. Use emitter.setMaxListeners() to increase limit.');l.name="MaxListenersExceededWarning",l.emitter=e,l.type=t,l.count=s.length,"object"===("undefined"==typeof console?"undefined":u(console))&&console.warn&&console.warn("%s: %s",l.name,l.message)}}else s=a[t]=n,++e._eventsCount;return e}function y(){if(!this.fired)switch(this.target.removeListener(this.type,this.wrapFn),this.fired=!0,arguments.length){case 0:return this.listener.call(this.target);case 1:return this.listener.call(this.target,arguments[0]);case 2:return this.listener.call(this.target,arguments[0],arguments[1]);case 3:return this.listener.call(this.target,arguments[0],arguments[1],arguments[2]);default:for(var e=new Array(arguments.length),t=0;t<e.length;++t)e[t]=arguments[t];this.listener.apply(this.target,e)}}function b(e,t,n){var r={fired:!1,wrapFn:void 0,target:e,type:t,listener:n},i=o.call(y,r);return i.listener=n,r.wrapFn=i,i}function C(e,t,n){var r=e._events;if(!r)return[];var i=r[t];return i?"function"==typeof i?n?[i.listener||i]:[i]:n?function(e){for(var t=new Array(e.length),n=0;n<t.length;++n)t[n]=e[n].listener||e[n];return t}(i):_(i,i.length):[]}function A(e){var t=this._events;if(t){var n=t[e];if("function"==typeof n)return 1;if(n)return n.length}return 0}function _(e,t){for(var n=new Array(t),r=0;r<t;++r)n[r]=e[r];return n}s?Object.defineProperty(a,"defaultMaxListeners",{enumerable:!0,get:function(){return l},set:function(e){if("number"!=typeof e||e<0||e!=e)throw new TypeError('"defaultMaxListeners" must be a positive number');l=e}}):a.defaultMaxListeners=l,a.prototype.setMaxListeners=function(e){if("number"!=typeof e||e<0||isNaN(e))throw new TypeError('"n" argument must be a positive number');return this._maxListeners=e,this},a.prototype.getMaxListeners=function(){return d(this)},a.prototype.emit=function(e){var t,n,r,i,o,a,s="error"===e;if(a=this._events)s=s&&null==a.error;else if(!s)return!1;if(s){if(arguments.length>1&&(t=arguments[1]),t instanceof Error)throw t;var l=new Error('Unhandled "error" event. ('+t+")");throw l.context=t,l}if(!(n=a[e]))return!1;var u="function"==typeof n;switch(r=arguments.length){case 1:h(n,u,this);break;case 2:f(n,u,this,arguments[1]);break;case 3:p(n,u,this,arguments[1],arguments[2]);break;case 4:g(n,u,this,arguments[1],arguments[2],arguments[3]);break;default:for(i=new Array(r-1),o=1;o<r;o++)i[o-1]=arguments[o];v(n,u,this,i)}return!0},a.prototype.addListener=function(e,t){return m(this,e,t,!1)},a.prototype.on=a.prototype.addListener,a.prototype.prependListener=function(e,t){return m(this,e,t,!0)},a.prototype.once=function(e,t){if("function"!=typeof t)throw new TypeError('"listener" argument must be a function');return this.on(e,b(this,e,t)),this},a.prototype.prependOnceListener=function(e,t){if("function"!=typeof t)throw new TypeError('"listener" argument must be a function');return this.prependListener(e,b(this,e,t)),this},a.prototype.removeListener=function(e,t){var n,i,o,a,s;if("function"!=typeof t)throw new TypeError('"listener" argument must be a function');if(!(i=this._events))return this;if(!(n=i[e]))return this;if(n===t||n.listener===t)0==--this._eventsCount?this._events=r(null):(delete i[e],i.removeListener&&this.emit("removeListener",e,n.listener||t));else if("function"!=typeof n){for(o=-1,a=n.length-1;a>=0;a--)if(n[a]===t||n[a].listener===t){s=n[a].listener,o=a;break}if(o<0)return this;0===o?n.shift():function(e,t){for(var n=t,r=n+1,i=e.length;r<i;n+=1,r+=1)e[n]=e[r];e.pop()}(n,o),1===n.length&&(i[e]=n[0]),i.removeListener&&this.emit("removeListener",e,s||t)}return this},a.prototype.removeAllListeners=function(e){var t,n,o;if(!(n=this._events))return this;if(!n.removeListener)return 0===arguments.length?(this._events=r(null),this._eventsCount=0):n[e]&&(0==--this._eventsCount?this._events=r(null):delete n[e]),this;if(0===arguments.length){var a,s=i(n);for(o=0;o<s.length;++o)"removeListener"!==(a=s[o])&&this.removeAllListeners(a);return this.removeAllListeners("removeListener"),this._events=r(null),this._eventsCount=0,this}if("function"==typeof(t=n[e]))this.removeListener(e,t);else if(t)for(o=t.length-1;o>=0;o--)this.removeListener(e,t[o]);return this},a.prototype.listeners=function(e){return C(this,e,!0)},a.prototype.rawListeners=function(e){return C(this,e,!1)},a.listenerCount=function(e,t){return"function"==typeof e.listenerCount?e.listenerCount(t):A.call(e,t)},a.prototype.listenerCount=A,a.prototype.eventNames=function(){return this._eventsCount>0?Reflect.ownKeys(this._events):[]}},{}],26:[function(e,t,n){var r,i=this&&this.__extends||(r=function(e,t){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(e,t)},function(e,t){function n(){this.constructor=e}r(e,t),e.prototype=null===t?Object.create(t):(n.prototype=t.prototype,new n)}),o=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};Object.defineProperty(n,"__esModule",{value:!0});var a=function(e){function t(t){void 0===t&&(t={});var n=e.call(this,(function(e){for(var t,r=[],i=1;i<arguments.length;i++)r[i-1]=arguments[i];n.events.push([e].concat(r)),n._cbs[e]&&(t=n._cbs)[e].apply(t,r)}))||this;return n._cbs=t,n.events=[],n}return i(t,e),t.prototype.onreset=function(){this.events=[],this._cbs.onreset&&this._cbs.onreset()},t.prototype.restart=function(){var e;this._cbs.onreset&&this._cbs.onreset();for(var t=0;t<this.events.length;t++){var n=this.events[t],r=n[0],i=n.slice(1);this._cbs[r]&&(e=this._cbs)[r].apply(e,i)}},t}(o(e("./MultiplexHandler")).default);n.CollectingHandler=a},{"./MultiplexHandler":28}],27:[function(e,t,n){var r,i=this&&this.__extends||(r=function(e,t){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(e,t)},function(e,t){function n(){this.constructor=e}r(e,t),e.prototype=null===t?Object.create(t):(n.prototype=t.prototype,new n)}),o=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}},a=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)Object.hasOwnProperty.call(e,n)&&(t[n]=e[n]);return t.default=e,t};Object.defineProperty(n,"__esModule",{value:!0});var s=o(e("domhandler")),l=a(e("domutils")),c=e("./Parser"),d=function(e){function t(t,n){return"object"===u(t)&&null!==t&&(n=t=void 0),e.call(this,t,n)||this}return i(t,e),t.prototype.onend=function(){var e={},t=f(m,this.dom);if(t)if("feed"===t.name){var n=t.children;e.type="atom",v(e,"id","id",n),v(e,"title","title",n);var r=g("href",f("link",n));r&&(e.link=r),v(e,"description","subtitle",n),(i=p("updated",n))&&(e.updated=new Date(i)),v(e,"author","email",n,!0),e.items=h("entry",n).map((function(e){var t={},n=e.children;v(t,"id","id",n),v(t,"title","title",n);var r=g("href",f("link",n));r&&(t.link=r);var i=p("summary",n)||p("content",n);i&&(t.description=i);var o=p("updated",n);return o&&(t.pubDate=new Date(o)),t}))}else{var i;n=f("channel",t.children).children,e.type=t.name.substr(0,3),e.id="",v(e,"title","title",n),v(e,"link","link",n),v(e,"description","description",n),(i=p("lastBuildDate",n))&&(e.updated=new Date(i)),v(e,"author","managingEditor",n,!0),e.items=h("item",t.children).map((function(e){var t={},n=e.children;v(t,"id","guid",n),v(t,"title","title",n),v(t,"link","link",n),v(t,"description","description",n);var r=p("pubDate",n);return r&&(t.pubDate=new Date(r)),t}))}this.feed=e,this.handleCallback(t?null:Error("couldn't find root of feed"))},t}(s.default);function h(e,t){return l.getElementsByTagName(e,t,!0)}function f(e,t){return l.getElementsByTagName(e,t,!0,1)[0]}function p(e,t,n){return void 0===n&&(n=!1),l.getText(l.getElementsByTagName(e,t,n,1)).trim()}function g(e,t){return t?t.attribs[e]:null}function v(e,t,n,r,i){void 0===i&&(i=!1);var o=p(n,r,i);o&&(e[t]=o)}function m(e){return"rss"===e||"feed"===e||"rdf:RDF"===e}n.FeedHandler=d;var y={xmlMode:!0};n.parseFeed=function(e,t){void 0===t&&(t=y);var n=new d(t);return new c.Parser(n,t).end(e),n.feed}},{"./Parser":29,domhandler:7,domutils:10}],28:[function(e,t,n){Object.defineProperty(n,"__esModule",{value:!0});var r=function(){function e(e){this._func=e}return e.prototype.onattribute=function(e,t){this._func("onattribute",e,t)},e.prototype.oncdatastart=function(){this._func("oncdatastart")},e.prototype.oncdataend=function(){this._func("oncdataend")},e.prototype.ontext=function(e){this._func("ontext",e)},e.prototype.onprocessinginstruction=function(e,t){this._func("onprocessinginstruction",e,t)},e.prototype.oncomment=function(e){this._func("oncomment",e)},e.prototype.oncommentend=function(){this._func("oncommentend")},e.prototype.onclosetag=function(e){this._func("onclosetag",e)},e.prototype.onopentag=function(e,t){this._func("onopentag",e,t)},e.prototype.onopentagname=function(e){this._func("onopentagname",e)},e.prototype.onerror=function(e){this._func("onerror",e)},e.prototype.onend=function(){this._func("onend")},e.prototype.onparserinit=function(e){this._func("onparserinit",e)},e.prototype.onreset=function(){this._func("onreset")},e}();n.default=r},{}],29:[function(e,t,n){var r,i=this&&this.__extends||(r=function(e,t){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(e,t)},function(e,t){function n(){this.constructor=e}r(e,t),e.prototype=null===t?Object.create(t):(n.prototype=t.prototype,new n)}),o=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};Object.defineProperty(n,"__esModule",{value:!0});var a=o(e("./Tokenizer")),s=e("events"),l=new Set(["input","option","optgroup","select","button","datalist","textarea"]),u=new Set(["p"]),c={tr:new Set(["tr","th","td"]),th:new Set(["th"]),td:new Set(["thead","th","td"]),body:new Set(["head","link","script"]),li:new Set(["li"]),p:u,h1:u,h2:u,h3:u,h4:u,h5:u,h6:u,select:l,input:l,output:l,button:l,datalist:l,textarea:l,option:new Set(["option"]),optgroup:new Set(["optgroup","option"]),dd:new Set(["dt","dd"]),dt:new Set(["dt","dd"]),address:u,article:u,aside:u,blockquote:u,details:u,div:u,dl:u,fieldset:u,figcaption:u,figure:u,footer:u,form:u,header:u,hr:u,main:u,nav:u,ol:u,pre:u,section:u,table:u,ul:u,rt:new Set(["rt","rp"]),rp:new Set(["rt","rp"]),tbody:new Set(["thead","tbody"]),tfoot:new Set(["thead","tbody"])},d=new Set(["area","base","basefont","br","col","command","embed","frame","hr","img","input","isindex","keygen","link","meta","param","source","track","wbr"]),h=new Set(["math","svg"]),f=new Set(["mi","mo","mn","ms","mtext","annotation-xml","foreignObject","desc","title"]),p=/\s|\//,g=function(e){function t(n,r){var i=e.call(this)||this;return i._tagname="",i._attribname="",i._attribvalue="",i._attribs=null,i._stack=[],i._foreignContext=[],i.startIndex=0,i.endIndex=null,i.parseChunk=t.prototype.write,i.done=t.prototype.end,i._options=r||{},i._cbs=n||{},i._tagname="",i._attribname="",i._attribvalue="",i._attribs=null,i._stack=[],i._foreignContext=[],i.startIndex=0,i.endIndex=null,i._lowerCaseTagNames="lowerCaseTags"in i._options?!!i._options.lowerCaseTags:!i._options.xmlMode,i._lowerCaseAttributeNames="lowerCaseAttributeNames"in i._options?!!i._options.lowerCaseAttributeNames:!i._options.xmlMode,i._tokenizer=new(i._options.Tokenizer||a.default)(i._options,i),i._cbs.onparserinit&&i._cbs.onparserinit(i),i}return i(t,e),t.prototype._updatePosition=function(e){null===this.endIndex?this._tokenizer._sectionStart<=e?this.startIndex=0:this.startIndex=this._tokenizer._sectionStart-e:this.startIndex=this.endIndex+1,this.endIndex=this._tokenizer.getAbsoluteIndex()},t.prototype.ontext=function(e){this._updatePosition(1),this.endIndex--,this._cbs.ontext&&this._cbs.ontext(e)},t.prototype.onopentagname=function(e){if(this._lowerCaseTagNames&&(e=e.toLowerCase()),this._tagname=e,!this._options.xmlMode&&Object.prototype.hasOwnProperty.call(c,e))for(var t=void 0;c[e].has(t=this._stack[this._stack.length-1]);this.onclosetag(t));!this._options.xmlMode&&d.has(e)||(this._stack.push(e),h.has(e)?this._foreignContext.push(!0):f.has(e)&&this._foreignContext.push(!1)),this._cbs.onopentagname&&this._cbs.onopentagname(e),this._cbs.onopentag&&(this._attribs={})},t.prototype.onopentagend=function(){this._updatePosition(1),this._attribs&&(this._cbs.onopentag&&this._cbs.onopentag(this._tagname,this._attribs),this._attribs=null),!this._options.xmlMode&&this._cbs.onclosetag&&d.has(this._tagname)&&this._cbs.onclosetag(this._tagname),this._tagname=""},t.prototype.onclosetag=function(e){if(this._updatePosition(1),this._lowerCaseTagNames&&(e=e.toLowerCase()),(h.has(e)||f.has(e))&&this._foreignContext.pop(),!this._stack.length||!this._options.xmlMode&&d.has(e))this._options.xmlMode||"br"!==e&&"p"!==e||(this.onopentagname(e),this._closeCurrentTag());else{var t=this._stack.lastIndexOf(e);if(-1!==t)if(this._cbs.onclosetag)for(t=this._stack.length-t;t--;)this._cbs.onclosetag(this._stack.pop());else this._stack.length=t;else"p"!==e||this._options.xmlMode||(this.onopentagname(e),this._closeCurrentTag())}},t.prototype.onselfclosingtag=function(){this._options.xmlMode||this._options.recognizeSelfClosing||this._foreignContext[this._foreignContext.length-1]?this._closeCurrentTag():this.onopentagend()},t.prototype._closeCurrentTag=function(){var e=this._tagname;this.onopentagend(),this._stack[this._stack.length-1]===e&&(this._cbs.onclosetag&&this._cbs.onclosetag(e),this._stack.pop())},t.prototype.onattribname=function(e){this._lowerCaseAttributeNames&&(e=e.toLowerCase()),this._attribname=e},t.prototype.onattribdata=function(e){this._attribvalue+=e},t.prototype.onattribend=function(){this._cbs.onattribute&&this._cbs.onattribute(this._attribname,this._attribvalue),this._attribs&&!Object.prototype.hasOwnProperty.call(this._attribs,this._attribname)&&(this._attribs[this._attribname]=this._attribvalue),this._attribname="",this._attribvalue=""},t.prototype._getInstructionName=function(e){var t=e.search(p),n=t<0?e:e.substr(0,t);return this._lowerCaseTagNames&&(n=n.toLowerCase()),n},t.prototype.ondeclaration=function(e){if(this._cbs.onprocessinginstruction){var t=this._getInstructionName(e);this._cbs.onprocessinginstruction("!"+t,"!"+e)}},t.prototype.onprocessinginstruction=function(e){if(this._cbs.onprocessinginstruction){var t=this._getInstructionName(e);this._cbs.onprocessinginstruction("?"+t,"?"+e)}},t.prototype.oncomment=function(e){this._updatePosition(4),this._cbs.oncomment&&this._cbs.oncomment(e),this._cbs.oncommentend&&this._cbs.oncommentend()},t.prototype.oncdata=function(e){this._updatePosition(1),this._options.xmlMode||this._options.recognizeCDATA?(this._cbs.oncdatastart&&this._cbs.oncdatastart(),this._cbs.ontext&&this._cbs.ontext(e),this._cbs.oncdataend&&this._cbs.oncdataend()):this.oncomment("[CDATA["+e+"]]")},t.prototype.onerror=function(e){this._cbs.onerror&&this._cbs.onerror(e)},t.prototype.onend=function(){if(this._cbs.onclosetag)for(var e=this._stack.length;e>0;this._cbs.onclosetag(this._stack[--e]));this._cbs.onend&&this._cbs.onend()},t.prototype.reset=function(){this._cbs.onreset&&this._cbs.onreset(),this._tokenizer.reset(),this._tagname="",this._attribname="",this._attribs=null,this._stack=[],this._cbs.onparserinit&&this._cbs.onparserinit(this)},t.prototype.parseComplete=function(e){this.reset(),this.end(e)},t.prototype.write=function(e){this._tokenizer.write(e)},t.prototype.end=function(e){this._tokenizer.end(e)},t.prototype.pause=function(){this._tokenizer.pause()},t.prototype.resume=function(){this._tokenizer.resume()},t}(s.EventEmitter);n.Parser=g},{"./Tokenizer":30,events:25}],30:[function(e,t,n){var r=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};Object.defineProperty(n,"__esModule",{value:!0});var i=r(e("entities/lib/decode_codepoint")),o=r(e("entities/lib/maps/entities.json")),a=r(e("entities/lib/maps/legacy.json")),s=r(e("entities/lib/maps/xml.json"));function l(e){return" "===e||"\n"===e||"\t"===e||"\f"===e||"\r"===e}function u(e,t,n){var r=e.toLowerCase();return e===r?function(e,i){i===r?e._state=t:(e._state=n,e._index--)}:function(i,o){o===r||o===e?i._state=t:(i._state=n,i._index--)}}function c(e,t){var n=e.toLowerCase();return function(r,i){i===n||i===e?r._state=t:(r._state=3,r._index--)}}var d=u("C",23,16),h=u("D",24,16),f=u("A",25,16),p=u("T",26,16),g=u("A",27,16),v=c("R",34),m=c("I",35),y=c("P",36),b=c("T",37),C=u("R",39,1),A=u("I",40,1),_=u("P",41,1),I=u("T",42,1),x=c("Y",44),w=c("L",45),M=c("E",46),S=u("Y",48,1),T=u("L",49,1),E=u("E",50,1),O=u("#",52,53),k=u("X",55,54),L=function(){function e(e,t){this._state=1,this._buffer="",this._sectionStart=0,this._index=0,this._bufferOffset=0,this._baseState=1,this._special=1,this._running=!0,this._ended=!1,this._cbs=t,this._xmlMode=!(!e||!e.xmlMode),this._decodeEntities=!(!e||!e.decodeEntities)}return e.prototype.reset=function(){this._state=1,this._buffer="",this._sectionStart=0,this._index=0,this._bufferOffset=0,this._baseState=1,this._special=1,this._running=!0,this._ended=!1},e.prototype._stateText=function(e){"<"===e?(this._index>this._sectionStart&&this._cbs.ontext(this._getSection()),this._state=2,this._sectionStart=this._index):this._decodeEntities&&1===this._special&&"&"===e&&(this._index>this._sectionStart&&this._cbs.ontext(this._getSection()),this._baseState=1,this._state=51,this._sectionStart=this._index)},e.prototype._stateBeforeTagName=function(e){"/"===e?this._state=5:"<"===e?(this._cbs.ontext(this._getSection()),this._sectionStart=this._index):">"===e||1!==this._special||l(e)?this._state=1:"!"===e?(this._state=15,this._sectionStart=this._index+1):"?"===e?(this._state=17,this._sectionStart=this._index+1):(this._state=this._xmlMode||"s"!==e&&"S"!==e?3:31,this._sectionStart=this._index)},e.prototype._stateInTagName=function(e){("/"===e||">"===e||l(e))&&(this._emitToken("onopentagname"),this._state=8,this._index--)},e.prototype._stateBeforeClosingTagName=function(e){l(e)||(">"===e?this._state=1:1!==this._special?"s"===e||"S"===e?this._state=32:(this._state=1,this._index--):(this._state=6,this._sectionStart=this._index))},e.prototype._stateInClosingTagName=function(e){(">"===e||l(e))&&(this._emitToken("onclosetag"),this._state=7,this._index--)},e.prototype._stateAfterClosingTagName=function(e){">"===e&&(this._state=1,this._sectionStart=this._index+1)},e.prototype._stateBeforeAttributeName=function(e){">"===e?(this._cbs.onopentagend(),this._state=1,this._sectionStart=this._index+1):"/"===e?this._state=4:l(e)||(this._state=9,this._sectionStart=this._index)},e.prototype._stateInSelfClosingTag=function(e){">"===e?(this._cbs.onselfclosingtag(),this._state=1,this._sectionStart=this._index+1):l(e)||(this._state=8,this._index--)},e.prototype._stateInAttributeName=function(e){("="===e||"/"===e||">"===e||l(e))&&(this._cbs.onattribname(this._getSection()),this._sectionStart=-1,this._state=10,this._index--)},e.prototype._stateAfterAttributeName=function(e){"="===e?this._state=11:"/"===e||">"===e?(this._cbs.onattribend(),this._state=8,this._index--):l(e)||(this._cbs.onattribend(),this._state=9,this._sectionStart=this._index)},e.prototype._stateBeforeAttributeValue=function(e){'"'===e?(this._state=12,this._sectionStart=this._index+1):"'"===e?(this._state=13,this._sectionStart=this._index+1):l(e)||(this._state=14,this._sectionStart=this._index,this._index--)},e.prototype._stateInAttributeValueDoubleQuotes=function(e){'"'===e?(this._emitToken("onattribdata"),this._cbs.onattribend(),this._state=8):this._decodeEntities&&"&"===e&&(this._emitToken("onattribdata"),this._baseState=this._state,this._state=51,this._sectionStart=this._index)},e.prototype._stateInAttributeValueSingleQuotes=function(e){"'"===e?(this._emitToken("onattribdata"),this._cbs.onattribend(),this._state=8):this._decodeEntities&&"&"===e&&(this._emitToken("onattribdata"),this._baseState=this._state,this._state=51,this._sectionStart=this._index)},e.prototype._stateInAttributeValueNoQuotes=function(e){l(e)||">"===e?(this._emitToken("onattribdata"),this._cbs.onattribend(),this._state=8,this._index--):this._decodeEntities&&"&"===e&&(this._emitToken("onattribdata"),this._baseState=this._state,this._state=51,this._sectionStart=this._index)},e.prototype._stateBeforeDeclaration=function(e){this._state="["===e?22:"-"===e?18:16},e.prototype._stateInDeclaration=function(e){">"===e&&(this._cbs.ondeclaration(this._getSection()),this._state=1,this._sectionStart=this._index+1)},e.prototype._stateInProcessingInstruction=function(e){">"===e&&(this._cbs.onprocessinginstruction(this._getSection()),this._state=1,this._sectionStart=this._index+1)},e.prototype._stateBeforeComment=function(e){"-"===e?(this._state=19,this._sectionStart=this._index+1):this._state=16},e.prototype._stateInComment=function(e){"-"===e&&(this._state=20)},e.prototype._stateAfterComment1=function(e){this._state="-"===e?21:19},e.prototype._stateAfterComment2=function(e){">"===e?(this._cbs.oncomment(this._buffer.substring(this._sectionStart,this._index-2)),this._state=1,this._sectionStart=this._index+1):"-"!==e&&(this._state=19)},e.prototype._stateBeforeCdata6=function(e){"["===e?(this._state=28,this._sectionStart=this._index+1):(this._state=16,this._index--)},e.prototype._stateInCdata=function(e){"]"===e&&(this._state=29)},e.prototype._stateAfterCdata1=function(e){this._state="]"===e?30:28},e.prototype._stateAfterCdata2=function(e){">"===e?(this._cbs.oncdata(this._buffer.substring(this._sectionStart,this._index-2)),this._state=1,this._sectionStart=this._index+1):"]"!==e&&(this._state=28)},e.prototype._stateBeforeSpecial=function(e){"c"===e||"C"===e?this._state=33:"t"===e||"T"===e?this._state=43:(this._state=3,this._index--)},e.prototype._stateBeforeSpecialEnd=function(e){2!==this._special||"c"!==e&&"C"!==e?3!==this._special||"t"!==e&&"T"!==e?this._state=1:this._state=47:this._state=38},e.prototype._stateBeforeScript5=function(e){("/"===e||">"===e||l(e))&&(this._special=2),this._state=3,this._index--},e.prototype._stateAfterScript5=function(e){">"===e||l(e)?(this._special=1,this._state=6,this._sectionStart=this._index-6,this._index--):this._state=1},e.prototype._stateBeforeStyle4=function(e){("/"===e||">"===e||l(e))&&(this._special=3),this._state=3,this._index--},e.prototype._stateAfterStyle4=function(e){">"===e||l(e)?(this._special=1,this._state=6,this._sectionStart=this._index-5,this._index--):this._state=1},e.prototype._parseNamedEntityStrict=function(){if(this._sectionStart+1<this._index){var e=this._buffer.substring(this._sectionStart+1,this._index),t=this._xmlMode?s.default:o.default;Object.prototype.hasOwnProperty.call(t,e)&&(this._emitPartial(t[e]),this._sectionStart=this._index+1)}},e.prototype._parseLegacyEntity=function(){var e=this._sectionStart+1,t=this._index-e;for(t>6&&(t=6);t>=2;){var n=this._buffer.substr(e,t);if(Object.prototype.hasOwnProperty.call(a.default,n))return this._emitPartial(a.default[n]),void(this._sectionStart+=t+1);t--}},e.prototype._stateInNamedEntity=function(e){";"===e?(this._parseNamedEntityStrict(),this._sectionStart+1<this._index&&!this._xmlMode&&this._parseLegacyEntity(),this._state=this._baseState):(e<"a"||e>"z")&&(e<"A"||e>"Z")&&(e<"0"||e>"9")&&(this._xmlMode||this._sectionStart+1===this._index||(1!==this._baseState?"="!==e&&this._parseNamedEntityStrict():this._parseLegacyEntity()),this._state=this._baseState,this._index--)},e.prototype._decodeNumericEntity=function(e,t){var n=this._sectionStart+e;if(n!==this._index){var r=this._buffer.substring(n,this._index),o=parseInt(r,t);this._emitPartial(i.default(o)),this._sectionStart=this._index}else this._sectionStart--;this._state=this._baseState},e.prototype._stateInNumericEntity=function(e){";"===e?(this._decodeNumericEntity(2,10),this._sectionStart++):(e<"0"||e>"9")&&(this._xmlMode?this._state=this._baseState:this._decodeNumericEntity(2,10),this._index--)},e.prototype._stateInHexEntity=function(e){";"===e?(this._decodeNumericEntity(3,16),this._sectionStart++):(e<"a"||e>"f")&&(e<"A"||e>"F")&&(e<"0"||e>"9")&&(this._xmlMode?this._state=this._baseState:this._decodeNumericEntity(3,16),this._index--)},e.prototype._cleanup=function(){this._sectionStart<0?(this._buffer="",this._bufferOffset+=this._index,this._index=0):this._running&&(1===this._state?(this._sectionStart!==this._index&&this._cbs.ontext(this._buffer.substr(this._sectionStart)),this._buffer="",this._bufferOffset+=this._index,this._index=0):this._sectionStart===this._index?(this._buffer="",this._bufferOffset+=this._index,this._index=0):(this._buffer=this._buffer.substr(this._sectionStart),this._index-=this._sectionStart,this._bufferOffset+=this._sectionStart),this._sectionStart=0)},e.prototype.write=function(e){this._ended&&this._cbs.onerror(Error(".write() after done!")),this._buffer+=e,this._parse()},e.prototype._parse=function(){for(;this._index<this._buffer.length&&this._running;){var e=this._buffer.charAt(this._index);1===this._state?this._stateText(e):12===this._state?this._stateInAttributeValueDoubleQuotes(e):9===this._state?this._stateInAttributeName(e):19===this._state?this._stateInComment(e):8===this._state?this._stateBeforeAttributeName(e):3===this._state?this._stateInTagName(e):6===this._state?this._stateInClosingTagName(e):2===this._state?this._stateBeforeTagName(e):10===this._state?this._stateAfterAttributeName(e):13===this._state?this._stateInAttributeValueSingleQuotes(e):11===this._state?this._stateBeforeAttributeValue(e):5===this._state?this._stateBeforeClosingTagName(e):7===this._state?this._stateAfterClosingTagName(e):31===this._state?this._stateBeforeSpecial(e):20===this._state?this._stateAfterComment1(e):14===this._state?this._stateInAttributeValueNoQuotes(e):4===this._state?this._stateInSelfClosingTag(e):16===this._state?this._stateInDeclaration(e):15===this._state?this._stateBeforeDeclaration(e):21===this._state?this._stateAfterComment2(e):18===this._state?this._stateBeforeComment(e):32===this._state?this._stateBeforeSpecialEnd(e):38===this._state?C(this,e):39===this._state?A(this,e):40===this._state?_(this,e):33===this._state?v(this,e):34===this._state?m(this,e):35===this._state?y(this,e):36===this._state?b(this,e):37===this._state?this._stateBeforeScript5(e):41===this._state?I(this,e):42===this._state?this._stateAfterScript5(e):43===this._state?x(this,e):28===this._state?this._stateInCdata(e):44===this._state?w(this,e):45===this._state?M(this,e):46===this._state?this._stateBeforeStyle4(e):47===this._state?S(this,e):48===this._state?T(this,e):49===this._state?E(this,e):50===this._state?this._stateAfterStyle4(e):17===this._state?this._stateInProcessingInstruction(e):53===this._state?this._stateInNamedEntity(e):22===this._state?d(this,e):51===this._state?O(this,e):23===this._state?h(this,e):24===this._state?f(this,e):29===this._state?this._stateAfterCdata1(e):30===this._state?this._stateAfterCdata2(e):25===this._state?p(this,e):26===this._state?g(this,e):27===this._state?this._stateBeforeCdata6(e):55===this._state?this._stateInHexEntity(e):54===this._state?this._stateInNumericEntity(e):52===this._state?k(this,e):this._cbs.onerror(Error("unknown _state"),this._state),this._index++}this._cleanup()},e.prototype.pause=function(){this._running=!1},e.prototype.resume=function(){this._running=!0,this._index<this._buffer.length&&this._parse(),this._ended&&this._finish()},e.prototype.end=function(e){this._ended&&this._cbs.onerror(Error(".end() after done!")),e&&this.write(e),this._ended=!0,this._running&&this._finish()},e.prototype._finish=function(){this._sectionStart<this._index&&this._handleTrailingData(),this._cbs.onend()},e.prototype._handleTrailingData=function(){var e=this._buffer.substr(this._sectionStart);28===this._state||29===this._state||30===this._state?this._cbs.oncdata(e):19===this._state||20===this._state||21===this._state?this._cbs.oncomment(e):53!==this._state||this._xmlMode?54!==this._state||this._xmlMode?55!==this._state||this._xmlMode?3!==this._state&&8!==this._state&&11!==this._state&&10!==this._state&&9!==this._state&&13!==this._state&&12!==this._state&&14!==this._state&&6!==this._state&&this._cbs.ontext(e):(this._decodeNumericEntity(3,16),this._sectionStart<this._index&&(this._state=this._baseState,this._handleTrailingData())):(this._decodeNumericEntity(2,10),this._sectionStart<this._index&&(this._state=this._baseState,this._handleTrailingData())):(this._parseLegacyEntity(),this._sectionStart<this._index&&(this._state=this._baseState,this._handleTrailingData()))},e.prototype.getAbsoluteIndex=function(){return this._bufferOffset+this._index},e.prototype._getSection=function(){return this._buffer.substring(this._sectionStart,this._index)},e.prototype._emitToken=function(e){this._cbs[e](this._getSection()),this._sectionStart=-1},e.prototype._emitPartial=function(e){1!==this._baseState?this._cbs.onattribdata(e):this._cbs.ontext(e)},e}();n.default=L},{"entities/lib/decode_codepoint":18,"entities/lib/maps/entities.json":22,"entities/lib/maps/legacy.json":23,"entities/lib/maps/xml.json":24}],31:[function(e,t,n){function r(e){for(var t in e)n.hasOwnProperty(t)||(n[t]=e[t])}var i=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)Object.hasOwnProperty.call(e,n)&&(t[n]=e[n]);return t.default=e,t};Object.defineProperty(n,"__esModule",{value:!0});var o=e("./Parser");n.Parser=o.Parser;var a=e("domhandler");n.DomHandler=a.DomHandler,n.DefaultHandler=a.DomHandler,n.parseDOM=function(e,t){var n=new a.DomHandler(void 0,t);return new o.Parser(n,t).end(e),n.dom},n.createDomStream=function(e,t,n){var r=new a.DomHandler(e,t,n);return new o.Parser(r,t)};var s=e("./Tokenizer");n.Tokenizer=s.default;var l=i(e("domelementtype"));n.ElementType=l,n.EVENTS={attribute:2,cdatastart:0,cdataend:0,text:1,processinginstruction:2,comment:1,commentend:0,closetag:1,opentag:2,opentagname:1,error:1,end:0},r(e("./FeedHandler")),r(e("./WritableStream")),r(e("./CollectingHandler"));var u=i(e("domutils"));n.DomUtils=u;var c=e("./FeedHandler");n.RssHandler=c.FeedHandler},{"./CollectingHandler":26,"./FeedHandler":27,"./Parser":29,"./Tokenizer":30,"./WritableStream":2,domelementtype:6,domhandler:7,domutils:10}],32:[function(e,t,n){n.read=function(e,t,n,r,i){var o,a,s=8*i-r-1,l=(1<<s)-1,u=l>>1,c=-7,d=n?i-1:0,h=n?-1:1,f=e[t+d];for(d+=h,o=f&(1<<-c)-1,f>>=-c,c+=s;c>0;o=256*o+e[t+d],d+=h,c-=8);for(a=o&(1<<-c)-1,o>>=-c,c+=r;c>0;a=256*a+e[t+d],d+=h,c-=8);if(0===o)o=1-u;else{if(o===l)return a?NaN:1/0*(f?-1:1);a+=Math.pow(2,r),o-=u}return(f?-1:1)*a*Math.pow(2,o-r)},n.write=function(e,t,n,r,i,o){var a,s,l,u=8*o-i-1,c=(1<<u)-1,d=c>>1,h=23===i?Math.pow(2,-24)-Math.pow(2,-77):0,f=r?0:o-1,p=r?1:-1,g=t<0||0===t&&1/t<0?1:0;for(t=Math.abs(t),isNaN(t)||t===1/0?(s=isNaN(t)?1:0,a=c):(a=Math.floor(Math.log(t)/Math.LN2),t*(l=Math.pow(2,-a))<1&&(a--,l*=2),(t+=a+d>=1?h/l:h*Math.pow(2,1-d))*l>=2&&(a++,l/=2),a+d>=c?(s=0,a=c):a+d>=1?(s=(t*l-1)*Math.pow(2,i),a+=d):(s=t*Math.pow(2,d-1)*Math.pow(2,i),a=0));i>=8;e[n+f]=255&s,f+=p,s/=256,i-=8);for(a=a<<i|s,u+=i;u>0;e[n+f]=255&a,f+=p,a/=256,u-=8);e[n+f-p]|=128*g}},{}],33:[function(e,t,n){var r=e("./_getNative")(e("./_root"),"DataView");t.exports=r},{"./_getNative":93,"./_root":130}],34:[function(e,t,n){var r=e("./_hashClear"),i=e("./_hashDelete"),o=e("./_hashGet"),a=e("./_hashHas"),s=e("./_hashSet");function l(e){var t=-1,n=null==e?0:e.length;for(this.clear();++t<n;){var r=e[t];this.set(r[0],r[1])}}l.prototype.clear=r,l.prototype.delete=i,l.prototype.get=o,l.prototype.has=a,l.prototype.set=s,t.exports=l},{"./_hashClear":100,"./_hashDelete":101,"./_hashGet":102,"./_hashHas":103,"./_hashSet":104}],35:[function(e,t,n){var r=e("./_listCacheClear"),i=e("./_listCacheDelete"),o=e("./_listCacheGet"),a=e("./_listCacheHas"),s=e("./_listCacheSet");function l(e){var t=-1,n=null==e?0:e.length;for(this.clear();++t<n;){var r=e[t];this.set(r[0],r[1])}}l.prototype.clear=r,l.prototype.delete=i,l.prototype.get=o,l.prototype.has=a,l.prototype.set=s,t.exports=l},{"./_listCacheClear":113,"./_listCacheDelete":114,"./_listCacheGet":115,"./_listCacheHas":116,"./_listCacheSet":117}],36:[function(e,t,n){var r=e("./_getNative")(e("./_root"),"Map");t.exports=r},{"./_getNative":93,"./_root":130}],37:[function(e,t,n){var r=e("./_mapCacheClear"),i=e("./_mapCacheDelete"),o=e("./_mapCacheGet"),a=e("./_mapCacheHas"),s=e("./_mapCacheSet");function l(e){var t=-1,n=null==e?0:e.length;for(this.clear();++t<n;){var r=e[t];this.set(r[0],r[1])}}l.prototype.clear=r,l.prototype.delete=i,l.prototype.get=o,l.prototype.has=a,l.prototype.set=s,t.exports=l},{"./_mapCacheClear":118,"./_mapCacheDelete":119,"./_mapCacheGet":120,"./_mapCacheHas":121,"./_mapCacheSet":122}],38:[function(e,t,n){var r=e("./_getNative")(e("./_root"),"Promise");t.exports=r},{"./_getNative":93,"./_root":130}],39:[function(e,t,n){var r=e("./_getNative")(e("./_root"),"Set");t.exports=r},{"./_getNative":93,"./_root":130}],40:[function(e,t,n){var r=e("./_ListCache"),i=e("./_stackClear"),o=e("./_stackDelete"),a=e("./_stackGet"),s=e("./_stackHas"),l=e("./_stackSet");function u(e){var t=this.__data__=new r(e);this.size=t.size}u.prototype.clear=i,u.prototype.delete=o,u.prototype.get=a,u.prototype.has=s,u.prototype.set=l,t.exports=u},{"./_ListCache":35,"./_stackClear":134,"./_stackDelete":135,"./_stackGet":136,"./_stackHas":137,"./_stackSet":138}],41:[function(e,t,n){var r=e("./_root").Symbol;t.exports=r},{"./_root":130}],42:[function(e,t,n){var r=e("./_root").Uint8Array;t.exports=r},{"./_root":130}],43:[function(e,t,n){var r=e("./_getNative")(e("./_root"),"WeakMap");t.exports=r},{"./_getNative":93,"./_root":130}],44:[function(e,t,n){t.exports=function(e,t,n){switch(n.length){case 0:return e.call(t);case 1:return e.call(t,n[0]);case 2:return e.call(t,n[0],n[1]);case 3:return e.call(t,n[0],n[1],n[2])}return e.apply(t,n)}},{}],45:[function(e,t,n){t.exports=function(e,t){for(var n=-1,r=null==e?0:e.length;++n<r&&!1!==t(e[n],n,e););return e}},{}],46:[function(e,t,n){t.exports=function(e,t){for(var n=-1,r=null==e?0:e.length,i=0,o=[];++n<r;){var a=e[n];t(a,n,e)&&(o[i++]=a)}return o}},{}],47:[function(e,t,n){var r=e("./_baseTimes"),i=e("./isArguments"),o=e("./isArray"),a=e("./isBuffer"),s=e("./_isIndex"),l=e("./isTypedArray"),u=Object.prototype.hasOwnProperty;t.exports=function(e,t){var n=o(e),c=!n&&i(e),d=!n&&!c&&a(e),h=!n&&!c&&!d&&l(e),f=n||c||d||h,p=f?r(e.length,String):[],g=p.length;for(var v in e)!t&&!u.call(e,v)||f&&("length"==v||d&&("offset"==v||"parent"==v)||h&&("buffer"==v||"byteLength"==v||"byteOffset"==v)||s(v,g))||p.push(v);return p}},{"./_baseTimes":72,"./_isIndex":108,"./isArguments":145,"./isArray":146,"./isBuffer":149,"./isTypedArray":159}],48:[function(e,t,n){t.exports=function(e,t){for(var n=-1,r=null==e?0:e.length,i=Array(r);++n<r;)i[n]=t(e[n],n,e);return i}},{}],49:[function(e,t,n){t.exports=function(e,t){for(var n=-1,r=t.length,i=e.length;++n<r;)e[i+n]=t[n];return e}},{}],50:[function(e,t,n){var r=e("./_baseAssignValue"),i=e("./eq");t.exports=function(e,t,n){(void 0!==n&&!i(e[t],n)||void 0===n&&!(t in e))&&r(e,t,n)}},{"./_baseAssignValue":55,"./eq":142}],51:[function(e,t,n){var r=e("./_baseAssignValue"),i=e("./eq"),o=Object.prototype.hasOwnProperty;t.exports=function(e,t,n){var a=e[t];o.call(e,t)&&i(a,n)&&(void 0!==n||t in e)||r(e,t,n)}},{"./_baseAssignValue":55,"./eq":142}],52:[function(e,t,n){var r=e("./eq");t.exports=function(e,t){for(var n=e.length;n--;)if(r(e[n][0],t))return n;return-1}},{"./eq":142}],53:[function(e,t,n){var r=e("./_copyObject"),i=e("./keys");t.exports=function(e,t){return e&&r(t,i(t),e)}},{"./_copyObject":82,"./keys":160}],54:[function(e,t,n){var r=e("./_copyObject"),i=e("./keysIn");t.exports=function(e,t){return e&&r(t,i(t),e)}},{"./_copyObject":82,"./keysIn":161}],55:[function(e,t,n){var r=e("./_defineProperty");t.exports=function(e,t,n){"__proto__"==t&&r?r(e,t,{configurable:!0,enumerable:!0,value:n,writable:!0}):e[t]=n}},{"./_defineProperty":88}],56:[function(e,t,n){var r=e("./_Stack"),i=e("./_arrayEach"),o=e("./_assignValue"),a=e("./_baseAssign"),s=e("./_baseAssignIn"),l=e("./_cloneBuffer"),u=e("./_copyArray"),c=e("./_copySymbols"),d=e("./_copySymbolsIn"),h=e("./_getAllKeys"),f=e("./_getAllKeysIn"),p=e("./_getTag"),g=e("./_initCloneArray"),v=e("./_initCloneByTag"),m=e("./_initCloneObject"),y=e("./isArray"),b=e("./isBuffer"),C=e("./isMap"),A=e("./isObject"),_=e("./isSet"),I=e("./keys"),x=e("./keysIn"),w={};w["[object Arguments]"]=w["[object Array]"]=w["[object ArrayBuffer]"]=w["[object DataView]"]=w["[object Boolean]"]=w["[object Date]"]=w["[object Float32Array]"]=w["[object Float64Array]"]=w["[object Int8Array]"]=w["[object Int16Array]"]=w["[object Int32Array]"]=w["[object Map]"]=w["[object Number]"]=w["[object Object]"]=w["[object RegExp]"]=w["[object Set]"]=w["[object String]"]=w["[object Symbol]"]=w["[object Uint8Array]"]=w["[object Uint8ClampedArray]"]=w["[object Uint16Array]"]=w["[object Uint32Array]"]=!0,w["[object Error]"]=w["[object Function]"]=w["[object WeakMap]"]=!1,t.exports=function e(t,n,M,S,T,E){var O,k=1&n,L=2&n,D=4&n;if(M&&(O=T?M(t,S,T,E):M(t)),void 0!==O)return O;if(!A(t))return t;var P=y(t);if(P){if(O=g(t),!k)return u(t,O)}else{var N=p(t),j="[object Function]"==N||"[object GeneratorFunction]"==N;if(b(t))return l(t,k);if("[object Object]"==N||"[object Arguments]"==N||j&&!T){if(O=L||j?{}:m(t),!k)return L?d(t,s(O,t)):c(t,a(O,t))}else{if(!w[N])return T?t:{};O=v(t,N,k)}}E||(E=new r);var R=E.get(t);if(R)return R;E.set(t,O),_(t)?t.forEach((function(r){O.add(e(r,n,M,r,t,E))})):C(t)&&t.forEach((function(r,i){O.set(i,e(r,n,M,i,t,E))}));var z=P?void 0:(D?L?f:h:L?x:I)(t);return i(z||t,(function(r,i){z&&(r=t[i=r]),o(O,i,e(r,n,M,i,t,E))})),O}},{"./_Stack":40,"./_arrayEach":45,"./_assignValue":51,"./_baseAssign":53,"./_baseAssignIn":54,"./_cloneBuffer":76,"./_copyArray":81,"./_copySymbols":83,"./_copySymbolsIn":84,"./_getAllKeys":90,"./_getAllKeysIn":91,"./_getTag":98,"./_initCloneArray":105,"./_initCloneByTag":106,"./_initCloneObject":107,"./isArray":146,"./isBuffer":149,"./isMap":152,"./isObject":153,"./isSet":156,"./keys":160,"./keysIn":161}],57:[function(e,t,n){var r=e("./isObject"),i=Object.create,o=function(){function e(){}return function(t){if(!r(t))return{};if(i)return i(t);e.prototype=t;var n=new e;return e.prototype=void 0,n}}();t.exports=o},{"./isObject":153}],58:[function(e,t,n){var r=e("./_createBaseFor")();t.exports=r},{"./_createBaseFor":87}],59:[function(e,t,n){var r=e("./_arrayPush"),i=e("./isArray");t.exports=function(e,t,n){var o=t(e);return i(e)?o:r(o,n(e))}},{"./_arrayPush":49,"./isArray":146}],60:[function(e,t,n){var r=e("./_Symbol"),i=e("./_getRawTag"),o=e("./_objectToString"),a=r?r.toStringTag:void 0;t.exports=function(e){return null==e?void 0===e?"[object Undefined]":"[object Null]":a&&a in Object(e)?i(e):o(e)}},{"./_Symbol":41,"./_getRawTag":95,"./_objectToString":127}],61:[function(e,t,n){var r=e("./_baseGetTag"),i=e("./isObjectLike");t.exports=function(e){return i(e)&&"[object Arguments]"==r(e)}},{"./_baseGetTag":60,"./isObjectLike":154}],62:[function(e,t,n){var r=e("./_getTag"),i=e("./isObjectLike");t.exports=function(e){return i(e)&&"[object Map]"==r(e)}},{"./_getTag":98,"./isObjectLike":154}],63:[function(e,t,n){var r=e("./isFunction"),i=e("./_isMasked"),o=e("./isObject"),a=e("./_toSource"),s=/^\[object .+?Constructor\]$/,l=Function.prototype,u=Object.prototype,c=l.toString,d=u.hasOwnProperty,h=RegExp("^"+c.call(d).replace(/[\\^$.*+?()[\]{}|]/g,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$");t.exports=function(e){return!(!o(e)||i(e))&&(r(e)?h:s).test(a(e))}},{"./_isMasked":111,"./_toSource":139,"./isFunction":150,"./isObject":153}],64:[function(e,t,n){var r=e("./_getTag"),i=e("./isObjectLike");t.exports=function(e){return i(e)&&"[object Set]"==r(e)}},{"./_getTag":98,"./isObjectLike":154}],65:[function(e,t,n){var r=e("./_baseGetTag"),i=e("./isLength"),o=e("./isObjectLike"),a={};a["[object Float32Array]"]=a["[object Float64Array]"]=a["[object Int8Array]"]=a["[object Int16Array]"]=a["[object Int32Array]"]=a["[object Uint8Array]"]=a["[object Uint8ClampedArray]"]=a["[object Uint16Array]"]=a["[object Uint32Array]"]=!0,a["[object Arguments]"]=a["[object Array]"]=a["[object ArrayBuffer]"]=a["[object Boolean]"]=a["[object DataView]"]=a["[object Date]"]=a["[object Error]"]=a["[object Function]"]=a["[object Map]"]=a["[object Number]"]=a["[object Object]"]=a["[object RegExp]"]=a["[object Set]"]=a["[object String]"]=a["[object WeakMap]"]=!1,t.exports=function(e){return o(e)&&i(e.length)&&!!a[r(e)]}},{"./_baseGetTag":60,"./isLength":151,"./isObjectLike":154}],66:[function(e,t,n){var r=e("./_isPrototype"),i=e("./_nativeKeys"),o=Object.prototype.hasOwnProperty;t.exports=function(e){if(!r(e))return i(e);var t=[];for(var n in Object(e))o.call(e,n)&&"constructor"!=n&&t.push(n);return t}},{"./_isPrototype":112,"./_nativeKeys":124}],67:[function(e,t,n){var r=e("./isObject"),i=e("./_isPrototype"),o=e("./_nativeKeysIn"),a=Object.prototype.hasOwnProperty;t.exports=function(e){if(!r(e))return o(e);var t=i(e),n=[];for(var s in e)("constructor"!=s||!t&&a.call(e,s))&&n.push(s);return n}},{"./_isPrototype":112,"./_nativeKeysIn":125,"./isObject":153}],68:[function(e,t,n){var r=e("./_Stack"),i=e("./_assignMergeValue"),o=e("./_baseFor"),a=e("./_baseMergeDeep"),s=e("./isObject"),l=e("./keysIn"),u=e("./_safeGet");t.exports=function e(t,n,c,d,h){t!==n&&o(n,(function(o,l){if(h||(h=new r),s(o))a(t,n,l,c,e,d,h);else{var f=d?d(u(t,l),o,l+"",t,n,h):void 0;void 0===f&&(f=o),i(t,l,f)}}),l)}},{"./_Stack":40,"./_assignMergeValue":50,"./_baseFor":58,"./_baseMergeDeep":69,"./_safeGet":131,"./isObject":153,"./keysIn":161}],69:[function(e,t,n){var r=e("./_assignMergeValue"),i=e("./_cloneBuffer"),o=e("./_cloneTypedArray"),a=e("./_copyArray"),s=e("./_initCloneObject"),l=e("./isArguments"),u=e("./isArray"),c=e("./isArrayLikeObject"),d=e("./isBuffer"),h=e("./isFunction"),f=e("./isObject"),p=e("./isPlainObject"),g=e("./isTypedArray"),v=e("./_safeGet"),m=e("./toPlainObject");t.exports=function(e,t,n,y,b,C,A){var _=v(e,n),I=v(t,n),x=A.get(I);if(x)r(e,n,x);else{var w=C?C(_,I,n+"",e,t,A):void 0,M=void 0===w;if(M){var S=u(I),T=!S&&d(I),E=!S&&!T&&g(I);w=I,S||T||E?u(_)?w=_:c(_)?w=a(_):T?(M=!1,w=i(I,!0)):E?(M=!1,w=o(I,!0)):w=[]:p(I)||l(I)?(w=_,l(_)?w=m(_):f(_)&&!h(_)||(w=s(I))):M=!1}M&&(A.set(I,w),b(w,I,y,C,A),A.delete(I)),r(e,n,w)}}},{"./_assignMergeValue":50,"./_cloneBuffer":76,"./_cloneTypedArray":80,"./_copyArray":81,"./_initCloneObject":107,"./_safeGet":131,"./isArguments":145,"./isArray":146,"./isArrayLikeObject":148,"./isBuffer":149,"./isFunction":150,"./isObject":153,"./isPlainObject":155,"./isTypedArray":159,"./toPlainObject":165}],70:[function(e,t,n){var r=e("./identity"),i=e("./_overRest"),o=e("./_setToString");t.exports=function(e,t){return o(i(e,t,r),e+"")}},{"./_overRest":129,"./_setToString":132,"./identity":144}],71:[function(e,t,n){var r=e("./constant"),i=e("./_defineProperty"),o=e("./identity"),a=i?function(e,t){return i(e,"toString",{configurable:!0,enumerable:!1,value:r(t),writable:!0})}:o;t.exports=a},{"./_defineProperty":88,"./constant":141,"./identity":144}],72:[function(e,t,n){t.exports=function(e,t){for(var n=-1,r=Array(e);++n<e;)r[n]=t(n);return r}},{}],73:[function(e,t,n){var r=e("./_Symbol"),i=e("./_arrayMap"),o=e("./isArray"),a=e("./isSymbol"),s=r?r.prototype:void 0,l=s?s.toString:void 0;t.exports=function e(t){if("string"==typeof t)return t;if(o(t))return i(t,e)+"";if(a(t))return l?l.call(t):"";var n=t+"";return"0"==n&&1/t==-1/0?"-0":n}},{"./_Symbol":41,"./_arrayMap":48,"./isArray":146,"./isSymbol":158}],74:[function(e,t,n){t.exports=function(e){return function(t){return e(t)}}},{}],75:[function(e,t,n){var r=e("./_Uint8Array");t.exports=function(e){var t=new e.constructor(e.byteLength);return new r(t).set(new r(e)),t}},{"./_Uint8Array":42}],76:[function(e,t,n){var r=e("./_root"),i="object"==u(n)&&n&&!n.nodeType&&n,o=i&&"object"==u(t)&&t&&!t.nodeType&&t,a=o&&o.exports===i?r.Buffer:void 0,s=a?a.allocUnsafe:void 0;t.exports=function(e,t){if(t)return e.slice();var n=e.length,r=s?s(n):new e.constructor(n);return e.copy(r),r}},{"./_root":130}],77:[function(e,t,n){var r=e("./_cloneArrayBuffer");t.exports=function(e,t){var n=t?r(e.buffer):e.buffer;return new e.constructor(n,e.byteOffset,e.byteLength)}},{"./_cloneArrayBuffer":75}],78:[function(e,t,n){var r=/\w*$/;t.exports=function(e){var t=new e.constructor(e.source,r.exec(e));return t.lastIndex=e.lastIndex,t}},{}],79:[function(e,t,n){var r=e("./_Symbol"),i=r?r.prototype:void 0,o=i?i.valueOf:void 0;t.exports=function(e){return o?Object(o.call(e)):{}}},{"./_Symbol":41}],80:[function(e,t,n){var r=e("./_cloneArrayBuffer");t.exports=function(e,t){var n=t?r(e.buffer):e.buffer;return new e.constructor(n,e.byteOffset,e.length)}},{"./_cloneArrayBuffer":75}],81:[function(e,t,n){t.exports=function(e,t){var n=-1,r=e.length;for(t||(t=Array(r));++n<r;)t[n]=e[n];return t}},{}],82:[function(e,t,n){var r=e("./_assignValue"),i=e("./_baseAssignValue");t.exports=function(e,t,n,o){var a=!n;n||(n={});for(var s=-1,l=t.length;++s<l;){var u=t[s],c=o?o(n[u],e[u],u,n,e):void 0;void 0===c&&(c=e[u]),a?i(n,u,c):r(n,u,c)}return n}},{"./_assignValue":51,"./_baseAssignValue":55}],83:[function(e,t,n){var r=e("./_copyObject"),i=e("./_getSymbols");t.exports=function(e,t){return r(e,i(e),t)}},{"./_copyObject":82,"./_getSymbols":96}],84:[function(e,t,n){var r=e("./_copyObject"),i=e("./_getSymbolsIn");t.exports=function(e,t){return r(e,i(e),t)}},{"./_copyObject":82,"./_getSymbolsIn":97}],85:[function(e,t,n){var r=e("./_root")["__core-js_shared__"];t.exports=r},{"./_root":130}],86:[function(e,t,n){var r=e("./_baseRest"),i=e("./_isIterateeCall");t.exports=function(e){return r((function(t,n){var r=-1,o=n.length,a=o>1?n[o-1]:void 0,s=o>2?n[2]:void 0;for(a=e.length>3&&"function"==typeof a?(o--,a):void 0,s&&i(n[0],n[1],s)&&(a=o<3?void 0:a,o=1),t=Object(t);++r<o;){var l=n[r];l&&e(t,l,r,a)}return t}))}},{"./_baseRest":70,"./_isIterateeCall":109}],87:[function(e,t,n){t.exports=function(e){return function(t,n,r){for(var i=-1,o=Object(t),a=r(t),s=a.length;s--;){var l=a[e?s:++i];if(!1===n(o[l],l,o))break}return t}}},{}],88:[function(e,t,n){var r=e("./_getNative"),i=function(){try{var e=r(Object,"defineProperty");return e({},"",{}),e}catch(e){}}();t.exports=i},{"./_getNative":93}],89:[function(e,t,r){(function(e){var n="object"==u(e)&&e&&e.Object===Object&&e;t.exports=n}).call(this,void 0!==n?n:"undefined"!=typeof self?self:"undefined"!=typeof window?window:{})},{}],90:[function(e,t,n){var r=e("./_baseGetAllKeys"),i=e("./_getSymbols"),o=e("./keys");t.exports=function(e){return r(e,o,i)}},{"./_baseGetAllKeys":59,"./_getSymbols":96,"./keys":160}],91:[function(e,t,n){var r=e("./_baseGetAllKeys"),i=e("./_getSymbolsIn"),o=e("./keysIn");t.exports=function(e){return r(e,o,i)}},{"./_baseGetAllKeys":59,"./_getSymbolsIn":97,"./keysIn":161}],92:[function(e,t,n){var r=e("./_isKeyable");t.exports=function(e,t){var n=e.__data__;return r(t)?n["string"==typeof t?"string":"hash"]:n.map}},{"./_isKeyable":110}],93:[function(e,t,n){var r=e("./_baseIsNative"),i=e("./_getValue");t.exports=function(e,t){var n=i(e,t);return r(n)?n:void 0}},{"./_baseIsNative":63,"./_getValue":99}],94:[function(e,t,n){var r=e("./_overArg")(Object.getPrototypeOf,Object);t.exports=r},{"./_overArg":128}],95:[function(e,t,n){var r=e("./_Symbol"),i=Object.prototype,o=i.hasOwnProperty,a=i.toString,s=r?r.toStringTag:void 0;t.exports=function(e){var t=o.call(e,s),n=e[s];try{e[s]=void 0;var r=!0}catch(e){}var i=a.call(e);return r&&(t?e[s]=n:delete e[s]),i}},{"./_Symbol":41}],96:[function(e,t,n){var r=e("./_arrayFilter"),i=e("./stubArray"),o=Object.prototype.propertyIsEnumerable,a=Object.getOwnPropertySymbols,s=a?function(e){return null==e?[]:(e=Object(e),r(a(e),(function(t){return o.call(e,t)})))}:i;t.exports=s},{"./_arrayFilter":46,"./stubArray":163}],97:[function(e,t,n){var r=e("./_arrayPush"),i=e("./_getPrototype"),o=e("./_getSymbols"),a=e("./stubArray"),s=Object.getOwnPropertySymbols?function(e){for(var t=[];e;)r(t,o(e)),e=i(e);return t}:a;t.exports=s},{"./_arrayPush":49,"./_getPrototype":94,"./_getSymbols":96,"./stubArray":163}],98:[function(e,t,n){var r=e("./_DataView"),i=e("./_Map"),o=e("./_Promise"),a=e("./_Set"),s=e("./_WeakMap"),l=e("./_baseGetTag"),u=e("./_toSource"),c=u(r),d=u(i),h=u(o),f=u(a),p=u(s),g=l;(r&&"[object DataView]"!=g(new r(new ArrayBuffer(1)))||i&&"[object Map]"!=g(new i)||o&&"[object Promise]"!=g(o.resolve())||a&&"[object Set]"!=g(new a)||s&&"[object WeakMap]"!=g(new s))&&(g=function(e){var t=l(e),n="[object Object]"==t?e.constructor:void 0,r=n?u(n):"";if(r)switch(r){case c:return"[object DataView]";case d:return"[object Map]";case h:return"[object Promise]";case f:return"[object Set]";case p:return"[object WeakMap]"}return t}),t.exports=g},{"./_DataView":33,"./_Map":36,"./_Promise":38,"./_Set":39,"./_WeakMap":43,"./_baseGetTag":60,"./_toSource":139}],99:[function(e,t,n){t.exports=function(e,t){return null==e?void 0:e[t]}},{}],100:[function(e,t,n){var r=e("./_nativeCreate");t.exports=function(){this.__data__=r?r(null):{},this.size=0}},{"./_nativeCreate":123}],101:[function(e,t,n){t.exports=function(e){var t=this.has(e)&&delete this.__data__[e];return this.size-=t?1:0,t}},{}],102:[function(e,t,n){var r=e("./_nativeCreate"),i=Object.prototype.hasOwnProperty;t.exports=function(e){var t=this.__data__;if(r){var n=t[e];return"__lodash_hash_undefined__"===n?void 0:n}return i.call(t,e)?t[e]:void 0}},{"./_nativeCreate":123}],103:[function(e,t,n){var r=e("./_nativeCreate"),i=Object.prototype.hasOwnProperty;t.exports=function(e){var t=this.__data__;return r?void 0!==t[e]:i.call(t,e)}},{"./_nativeCreate":123}],104:[function(e,t,n){var r=e("./_nativeCreate");t.exports=function(e,t){var n=this.__data__;return this.size+=this.has(e)?0:1,n[e]=r&&void 0===t?"__lodash_hash_undefined__":t,this}},{"./_nativeCreate":123}],105:[function(e,t,n){var r=Object.prototype.hasOwnProperty;t.exports=function(e){var t=e.length,n=new e.constructor(t);return t&&"string"==typeof e[0]&&r.call(e,"index")&&(n.index=e.index,n.input=e.input),n}},{}],106:[function(e,t,n){var r=e("./_cloneArrayBuffer"),i=e("./_cloneDataView"),o=e("./_cloneRegExp"),a=e("./_cloneSymbol"),s=e("./_cloneTypedArray");t.exports=function(e,t,n){var l=e.constructor;switch(t){case"[object ArrayBuffer]":return r(e);case"[object Boolean]":case"[object Date]":return new l(+e);case"[object DataView]":return i(e,n);case"[object Float32Array]":case"[object Float64Array]":case"[object Int8Array]":case"[object Int16Array]":case"[object Int32Array]":case"[object Uint8Array]":case"[object Uint8ClampedArray]":case"[object Uint16Array]":case"[object Uint32Array]":return s(e,n);case"[object Map]":return new l;case"[object Number]":case"[object String]":return new l(e);case"[object RegExp]":return o(e);case"[object Set]":return new l;case"[object Symbol]":return a(e)}}},{"./_cloneArrayBuffer":75,"./_cloneDataView":77,"./_cloneRegExp":78,"./_cloneSymbol":79,"./_cloneTypedArray":80}],107:[function(e,t,n){var r=e("./_baseCreate"),i=e("./_getPrototype"),o=e("./_isPrototype");t.exports=function(e){return"function"!=typeof e.constructor||o(e)?{}:r(i(e))}},{"./_baseCreate":57,"./_getPrototype":94,"./_isPrototype":112}],108:[function(e,t,n){var r=/^(?:0|[1-9]\d*)$/;t.exports=function(e,t){var n=u(e);return!!(t=null==t?9007199254740991:t)&&("number"==n||"symbol"!=n&&r.test(e))&&e>-1&&e%1==0&&e<t}},{}],109:[function(e,t,n){var r=e("./eq"),i=e("./isArrayLike"),o=e("./_isIndex"),a=e("./isObject");t.exports=function(e,t,n){if(!a(n))return!1;var s=u(t);return!!("number"==s?i(n)&&o(t,n.length):"string"==s&&t in n)&&r(n[t],e)}},{"./_isIndex":108,"./eq":142,"./isArrayLike":147,"./isObject":153}],110:[function(e,t,n){t.exports=function(e){var t=u(e);return"string"==t||"number"==t||"symbol"==t||"boolean"==t?"__proto__"!==e:null===e}},{}],111:[function(e,t,n){var r,i=e("./_coreJsData"),o=(r=/[^.]+$/.exec(i&&i.keys&&i.keys.IE_PROTO||""))?"Symbol(src)_1."+r:"";t.exports=function(e){return!!o&&o in e}},{"./_coreJsData":85}],112:[function(e,t,n){var r=Object.prototype;t.exports=function(e){var t=e&&e.constructor;return e===("function"==typeof t&&t.prototype||r)}},{}],113:[function(e,t,n){t.exports=function(){this.__data__=[],this.size=0}},{}],114:[function(e,t,n){var r=e("./_assocIndexOf"),i=Array.prototype.splice;t.exports=function(e){var t=this.__data__,n=r(t,e);return!(n<0||(n==t.length-1?t.pop():i.call(t,n,1),--this.size,0))}},{"./_assocIndexOf":52}],115:[function(e,t,n){var r=e("./_assocIndexOf");t.exports=function(e){var t=this.__data__,n=r(t,e);return n<0?void 0:t[n][1]}},{"./_assocIndexOf":52}],116:[function(e,t,n){var r=e("./_assocIndexOf");t.exports=function(e){return r(this.__data__,e)>-1}},{"./_assocIndexOf":52}],117:[function(e,t,n){var r=e("./_assocIndexOf");t.exports=function(e,t){var n=this.__data__,i=r(n,e);return i<0?(++this.size,n.push([e,t])):n[i][1]=t,this}},{"./_assocIndexOf":52}],118:[function(e,t,n){var r=e("./_Hash"),i=e("./_ListCache"),o=e("./_Map");t.exports=function(){this.size=0,this.__data__={hash:new r,map:new(o||i),string:new r}}},{"./_Hash":34,"./_ListCache":35,"./_Map":36}],119:[function(e,t,n){var r=e("./_getMapData");t.exports=function(e){var t=r(this,e).delete(e);return this.size-=t?1:0,t}},{"./_getMapData":92}],120:[function(e,t,n){var r=e("./_getMapData");t.exports=function(e){return r(this,e).get(e)}},{"./_getMapData":92}],121:[function(e,t,n){var r=e("./_getMapData");t.exports=function(e){return r(this,e).has(e)}},{"./_getMapData":92}],122:[function(e,t,n){var r=e("./_getMapData");t.exports=function(e,t){var n=r(this,e),i=n.size;return n.set(e,t),this.size+=n.size==i?0:1,this}},{"./_getMapData":92}],123:[function(e,t,n){var r=e("./_getNative")(Object,"create");t.exports=r},{"./_getNative":93}],124:[function(e,t,n){var r=e("./_overArg")(Object.keys,Object);t.exports=r},{"./_overArg":128}],125:[function(e,t,n){t.exports=function(e){var t=[];if(null!=e)for(var n in Object(e))t.push(n);return t}},{}],126:[function(e,t,n){var r=e("./_freeGlobal"),i="object"==u(n)&&n&&!n.nodeType&&n,o=i&&"object"==u(t)&&t&&!t.nodeType&&t,a=o&&o.exports===i&&r.process,s=function(){try{var e=o&&o.require&&o.require("util").types;return e||a&&a.binding&&a.binding("util")}catch(e){}}();t.exports=s},{"./_freeGlobal":89}],127:[function(e,t,n){var r=Object.prototype.toString;t.exports=function(e){return r.call(e)}},{}],128:[function(e,t,n){t.exports=function(e,t){return function(n){return e(t(n))}}},{}],129:[function(e,t,n){var r=e("./_apply"),i=Math.max;t.exports=function(e,t,n){return t=i(void 0===t?e.length-1:t,0),function(){for(var o=arguments,a=-1,s=i(o.length-t,0),l=Array(s);++a<s;)l[a]=o[t+a];a=-1;for(var u=Array(t+1);++a<t;)u[a]=o[a];return u[t]=n(l),r(e,this,u)}}},{"./_apply":44}],130:[function(e,t,n){var r=e("./_freeGlobal"),i="object"==("undefined"==typeof self?"undefined":u(self))&&self&&self.Object===Object&&self,o=r||i||Function("return this")();t.exports=o},{"./_freeGlobal":89}],131:[function(e,t,n){t.exports=function(e,t){if(("constructor"!==t||"function"!=typeof e[t])&&"__proto__"!=t)return e[t]}},{}],132:[function(e,t,n){var r=e("./_baseSetToString"),i=e("./_shortOut")(r);t.exports=i},{"./_baseSetToString":71,"./_shortOut":133}],133:[function(e,t,n){var r=Date.now;t.exports=function(e){var t=0,n=0;return function(){var i=r(),o=16-(i-n);if(n=i,o>0){if(++t>=800)return arguments[0]}else t=0;return e.apply(void 0,arguments)}}},{}],134:[function(e,t,n){var r=e("./_ListCache");t.exports=function(){this.__data__=new r,this.size=0}},{"./_ListCache":35}],135:[function(e,t,n){t.exports=function(e){var t=this.__data__,n=t.delete(e);return this.size=t.size,n}},{}],136:[function(e,t,n){t.exports=function(e){return this.__data__.get(e)}},{}],137:[function(e,t,n){t.exports=function(e){return this.__data__.has(e)}},{}],138:[function(e,t,n){var r=e("./_ListCache"),i=e("./_Map"),o=e("./_MapCache");t.exports=function(e,t){var n=this.__data__;if(n instanceof r){var a=n.__data__;if(!i||a.length<199)return a.push([e,t]),this.size=++n.size,this;n=this.__data__=new o(a)}return n.set(e,t),this.size=n.size,this}},{"./_ListCache":35,"./_Map":36,"./_MapCache":37}],139:[function(e,t,n){var r=Function.prototype.toString;t.exports=function(e){if(null!=e){try{return r.call(e)}catch(e){}try{return e+""}catch(e){}}return""}},{}],140:[function(e,t,n){var r=e("./_baseClone");t.exports=function(e){return r(e,5)}},{"./_baseClone":56}],141:[function(e,t,n){t.exports=function(e){return function(){return e}}},{}],142:[function(e,t,n){t.exports=function(e,t){return e===t||e!=e&&t!=t}},{}],143:[function(e,t,n){var r=e("./toString"),i=/[\\^$.*+?()[\]{}|]/g,o=RegExp(i.source);t.exports=function(e){return(e=r(e))&&o.test(e)?e.replace(i,"\\$&"):e}},{"./toString":166}],144:[function(e,t,n){t.exports=function(e){return e}},{}],145:[function(e,t,n){var r=e("./_baseIsArguments"),i=e("./isObjectLike"),o=Object.prototype,a=o.hasOwnProperty,s=o.propertyIsEnumerable,l=r(function(){return arguments}())?r:function(e){return i(e)&&a.call(e,"callee")&&!s.call(e,"callee")};t.exports=l},{"./_baseIsArguments":61,"./isObjectLike":154}],146:[function(e,t,n){var r=Array.isArray;t.exports=r},{}],147:[function(e,t,n){var r=e("./isFunction"),i=e("./isLength");t.exports=function(e){return null!=e&&i(e.length)&&!r(e)}},{"./isFunction":150,"./isLength":151}],148:[function(e,t,n){var r=e("./isArrayLike"),i=e("./isObjectLike");t.exports=function(e){return i(e)&&r(e)}},{"./isArrayLike":147,"./isObjectLike":154}],149:[function(e,t,n){var r=e("./_root"),i=e("./stubFalse"),o="object"==u(n)&&n&&!n.nodeType&&n,a=o&&"object"==u(t)&&t&&!t.nodeType&&t,s=a&&a.exports===o?r.Buffer:void 0,l=(s?s.isBuffer:void 0)||i;t.exports=l},{"./_root":130,"./stubFalse":164}],150:[function(e,t,n){var r=e("./_baseGetTag"),i=e("./isObject");t.exports=function(e){if(!i(e))return!1;var t=r(e);return"[object Function]"==t||"[object GeneratorFunction]"==t||"[object AsyncFunction]"==t||"[object Proxy]"==t}},{"./_baseGetTag":60,"./isObject":153}],151:[function(e,t,n){t.exports=function(e){return"number"==typeof e&&e>-1&&e%1==0&&e<=9007199254740991}},{}],152:[function(e,t,n){var r=e("./_baseIsMap"),i=e("./_baseUnary"),o=e("./_nodeUtil"),a=o&&o.isMap,s=a?i(a):r;t.exports=s},{"./_baseIsMap":62,"./_baseUnary":74,"./_nodeUtil":126}],153:[function(e,t,n){t.exports=function(e){var t=u(e);return null!=e&&("object"==t||"function"==t)}},{}],154:[function(e,t,n){t.exports=function(e){return null!=e&&"object"==u(e)}},{}],155:[function(e,t,n){var r=e("./_baseGetTag"),i=e("./_getPrototype"),o=e("./isObjectLike"),a=Function.prototype,s=Object.prototype,l=a.toString,u=s.hasOwnProperty,c=l.call(Object);t.exports=function(e){if(!o(e)||"[object Object]"!=r(e))return!1;var t=i(e);if(null===t)return!0;var n=u.call(t,"constructor")&&t.constructor;return"function"==typeof n&&n instanceof n&&l.call(n)==c}},{"./_baseGetTag":60,"./_getPrototype":94,"./isObjectLike":154}],156:[function(e,t,n){var r=e("./_baseIsSet"),i=e("./_baseUnary"),o=e("./_nodeUtil"),a=o&&o.isSet,s=a?i(a):r;t.exports=s},{"./_baseIsSet":64,"./_baseUnary":74,"./_nodeUtil":126}],157:[function(e,t,n){var r=e("./_baseGetTag"),i=e("./isArray"),o=e("./isObjectLike");t.exports=function(e){return"string"==typeof e||!i(e)&&o(e)&&"[object String]"==r(e)}},{"./_baseGetTag":60,"./isArray":146,"./isObjectLike":154}],158:[function(e,t,n){var r=e("./_baseGetTag"),i=e("./isObjectLike");t.exports=function(e){return"symbol"==u(e)||i(e)&&"[object Symbol]"==r(e)}},{"./_baseGetTag":60,"./isObjectLike":154}],159:[function(e,t,n){var r=e("./_baseIsTypedArray"),i=e("./_baseUnary"),o=e("./_nodeUtil"),a=o&&o.isTypedArray,s=a?i(a):r;t.exports=s},{"./_baseIsTypedArray":65,"./_baseUnary":74,"./_nodeUtil":126}],160:[function(e,t,n){var r=e("./_arrayLikeKeys"),i=e("./_baseKeys"),o=e("./isArrayLike");t.exports=function(e){return o(e)?r(e):i(e)}},{"./_arrayLikeKeys":47,"./_baseKeys":66,"./isArrayLike":147}],161:[function(e,t,n){var r=e("./_arrayLikeKeys"),i=e("./_baseKeysIn"),o=e("./isArrayLike");t.exports=function(e){return o(e)?r(e,!0):i(e)}},{"./_arrayLikeKeys":47,"./_baseKeysIn":67,"./isArrayLike":147}],162:[function(e,t,n){var r=e("./_baseMerge"),i=e("./_createAssigner")((function(e,t,n,i){r(e,t,n,i)}));t.exports=i},{"./_baseMerge":68,"./_createAssigner":86}],163:[function(e,t,n){t.exports=function(){return[]}},{}],164:[function(e,t,n){t.exports=function(){return!1}},{}],165:[function(e,t,n){var r=e("./_copyObject"),i=e("./keysIn");t.exports=function(e){return r(e,i(e))}},{"./_copyObject":82,"./keysIn":161}],166:[function(e,t,n){var r=e("./_baseToString");t.exports=function(e){return null==e?"":r(e)}},{"./_baseToString":73}],167:[function(e,t,n){var r,i;r=this,i=function(){return function(e){function t(e){return" "===e||"\t"===e||"\n"===e||"\f"===e||"\r"===e}function n(t){var n,r=t.exec(e.substring(g));if(r)return n=r[0],g+=n.length,n}for(var r,i,o,a,s,l=e.length,u=/^[ \t\n\r\u000c]+/,c=/^[, \t\n\r\u000c]+/,d=/^[^ \t\n\r\u000c]+/,h=/[,]+$/,f=/^\d+$/,p=/^-?(?:[0-9]+|[0-9]*\.[0-9]+)(?:[eE][+-]?[0-9]+)?$/,g=0,v=[];;){if(n(c),g>=l)return v;r=n(d),i=[],","===r.slice(-1)?(r=r.replace(h,""),y()):m()}function m(){for(n(u),o="",a="in descriptor";;){if(s=e.charAt(g),"in descriptor"===a)if(t(s))o&&(i.push(o),o="",a="after descriptor");else{if(","===s)return g+=1,o&&i.push(o),void y();if("("===s)o+=s,a="in parens";else{if(""===s)return o&&i.push(o),void y();o+=s}}else if("in parens"===a)if(")"===s)o+=s,a="in descriptor";else{if(""===s)return i.push(o),void y();o+=s}else if("after descriptor"===a)if(t(s));else{if(""===s)return void y();a="in descriptor",g-=1}g+=1}}function y(){var t,n,o,a,s,l,u,c,d,h=!1,g={};for(a=0;a<i.length;a++)l=(s=i[a])[s.length-1],u=s.substring(0,s.length-1),c=parseInt(u,10),d=parseFloat(u),f.test(u)&&"w"===l?((t||n)&&(h=!0),0===c?h=!0:t=c):p.test(u)&&"x"===l?((t||n||o)&&(h=!0),d<0?h=!0:n=d):f.test(u)&&"h"===l?((o||n)&&(h=!0),0===c?h=!0:o=c):h=!0;h?console&&console.log&&console.log("Invalid srcset descriptor found in '"+e+"' at '"+s+"'."):(g.url=r,t&&(g.w=t),n&&(g.d=n),o&&(g.h=o),v.push(g))}}},"object"===u(t)&&t.exports?t.exports=i():r.parseSrcset=i()},{}],168:[function(e,t,n){(function(e){function t(e,t){for(var n=0,r=e.length-1;r>=0;r--){var i=e[r];"."===i?e.splice(r,1):".."===i?(e.splice(r,1),n++):n&&(e.splice(r,1),n--)}if(t)for(;n--;n)e.unshift("..");return e}function r(e,t){if(e.filter)return e.filter(t);for(var n=[],r=0;r<e.length;r++)t(e[r],r,e)&&n.push(e[r]);return n}n.resolve=function(){for(var n="",i=!1,o=arguments.length-1;o>=-1&&!i;o--){var a=o>=0?arguments[o]:e.cwd();if("string"!=typeof a)throw new TypeError("Arguments to path.resolve must be strings");a&&(n=a+"/"+n,i="/"===a.charAt(0))}return(i?"/":"")+(n=t(r(n.split("/"),(function(e){return!!e})),!i).join("/"))||"."},n.normalize=function(e){var o=n.isAbsolute(e),a="/"===i(e,-1);return(e=t(r(e.split("/"),(function(e){return!!e})),!o).join("/"))||o||(e="."),e&&a&&(e+="/"),(o?"/":"")+e},n.isAbsolute=function(e){return"/"===e.charAt(0)},n.join=function(){var e=Array.prototype.slice.call(arguments,0);return n.normalize(r(e,(function(e,t){if("string"!=typeof e)throw new TypeError("Arguments to path.join must be strings");return e})).join("/"))},n.relative=function(e,t){function r(e){for(var t=0;t<e.length&&""===e[t];t++);for(var n=e.length-1;n>=0&&""===e[n];n--);return t>n?[]:e.slice(t,n-t+1)}e=n.resolve(e).substr(1),t=n.resolve(t).substr(1);for(var i=r(e.split("/")),o=r(t.split("/")),a=Math.min(i.length,o.length),s=a,l=0;l<a;l++)if(i[l]!==o[l]){s=l;break}var u=[];for(l=s;l<i.length;l++)u.push("..");return(u=u.concat(o.slice(s))).join("/")},n.sep="/",n.delimiter=":",n.dirname=function(e){if("string"!=typeof e&&(e+=""),0===e.length)return".";for(var t=e.charCodeAt(0),n=47===t,r=-1,i=!0,o=e.length-1;o>=1;--o)if(47===(t=e.charCodeAt(o))){if(!i){r=o;break}}else i=!1;return-1===r?n?"/":".":n&&1===r?"/":e.slice(0,r)},n.basename=function(e,t){var n=function(e){"string"!=typeof e&&(e+="");var t,n=0,r=-1,i=!0;for(t=e.length-1;t>=0;--t)if(47===e.charCodeAt(t)){if(!i){n=t+1;break}}else-1===r&&(i=!1,r=t+1);return-1===r?"":e.slice(n,r)}(e);return t&&n.substr(-1*t.length)===t&&(n=n.substr(0,n.length-t.length)),n},n.extname=function(e){"string"!=typeof e&&(e+="");for(var t=-1,n=0,r=-1,i=!0,o=0,a=e.length-1;a>=0;--a){var s=e.charCodeAt(a);if(47!==s)-1===r&&(i=!1,r=a+1),46===s?-1===t?t=a:1!==o&&(o=1):-1!==t&&(o=-1);else if(!i){n=a+1;break}}return-1===t||-1===r||0===o||1===o&&t===r-1&&t===n+1?"":e.slice(t,r)};var i="b"==="ab".substr(-1)?function(e,t,n){return e.substr(t,n)}:function(e,t,n){return t<0&&(t=e.length+t),e.substr(t,n)}}).call(this,e("_process"))},{_process:193}],169:[function(e,t,n){var r;n.__esModule=!0,n.default=void 0;var i=function(e){var t,n;function r(t){var n;return(n=e.call(this,t)||this).type="atrule",n}n=e,(t=r).prototype=Object.create(n.prototype),t.prototype.constructor=t,t.__proto__=n;var i=r.prototype;return i.append=function(){var t;this.nodes||(this.nodes=[]);for(var n=arguments.length,r=new Array(n),i=0;i<n;i++)r[i]=arguments[i];return(t=e.prototype.append).call.apply(t,[this].concat(r))},i.prepend=function(){var t;this.nodes||(this.nodes=[]);for(var n=arguments.length,r=new Array(n),i=0;i<n;i++)r[i]=arguments[i];return(t=e.prototype.prepend).call.apply(t,[this].concat(r))},r}(((r=e("./container"))&&r.__esModule?r:{default:r}).default);n.default=i,t.exports=n.default},{"./container":171}],170:[function(e,t,n){var r;n.__esModule=!0,n.default=void 0;var i=function(e){var t,n;function r(t){var n;return(n=e.call(this,t)||this).type="comment",n}return n=e,(t=r).prototype=Object.create(n.prototype),t.prototype.constructor=t,t.__proto__=n,r}(((r=e("./node"))&&r.__esModule?r:{default:r}).default);n.default=i,t.exports=n.default},{"./node":178}],171:[function(e,t,n){n.__esModule=!0,n.default=void 0;var r=o(e("./declaration")),i=o(e("./comment"));function o(e){return e&&e.__esModule?e:{default:e}}function a(e,t){var n;if("undefined"==typeof Symbol||null==e[Symbol.iterator]){if(Array.isArray(e)||(n=function(e,t){if(e){if("string"==typeof e)return s(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?s(e,t):void 0}}(e))||t&&e&&"number"==typeof e.length){n&&(e=n);var r=0;return function(){return r>=e.length?{done:!0}:{done:!1,value:e[r++]}}}throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}return(n=e[Symbol.iterator]()).next.bind(n)}function s(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,r=new Array(t);n<t;n++)r[n]=e[n];return r}function l(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}var u=function(t){var n,o;function s(){return t.apply(this,arguments)||this}o=t,(n=s).prototype=Object.create(o.prototype),n.prototype.constructor=n,n.__proto__=o;var u,c,d,h=s.prototype;return h.push=function(e){return e.parent=this,this.nodes.push(e),this},h.each=function(e){this.lastEach||(this.lastEach=0),this.indexes||(this.indexes={}),this.lastEach+=1;var t=this.lastEach;if(this.indexes[t]=0,this.nodes){for(var n,r;this.indexes[t]<this.nodes.length&&(n=this.indexes[t],!1!==(r=e(this.nodes[n],n)));)this.indexes[t]+=1;return delete this.indexes[t],r}},h.walk=function(e){return this.each((function(t,n){var r;try{r=e(t,n)}catch(e){if(e.postcssNode=t,e.stack&&t.source&&/\n\s{4}at /.test(e.stack)){var i=t.source;e.stack=e.stack.replace(/\n\s{4}at /,"$&"+i.input.from+":"+i.start.line+":"+i.start.column+"$&")}throw e}return!1!==r&&t.walk&&(r=t.walk(e)),r}))},h.walkDecls=function(e,t){return t?e instanceof RegExp?this.walk((function(n,r){if("decl"===n.type&&e.test(n.prop))return t(n,r)})):this.walk((function(n,r){if("decl"===n.type&&n.prop===e)return t(n,r)})):(t=e,this.walk((function(e,n){if("decl"===e.type)return t(e,n)})))},h.walkRules=function(e,t){return t?e instanceof RegExp?this.walk((function(n,r){if("rule"===n.type&&e.test(n.selector))return t(n,r)})):this.walk((function(n,r){if("rule"===n.type&&n.selector===e)return t(n,r)})):(t=e,this.walk((function(e,n){if("rule"===e.type)return t(e,n)})))},h.walkAtRules=function(e,t){return t?e instanceof RegExp?this.walk((function(n,r){if("atrule"===n.type&&e.test(n.name))return t(n,r)})):this.walk((function(n,r){if("atrule"===n.type&&n.name===e)return t(n,r)})):(t=e,this.walk((function(e,n){if("atrule"===e.type)return t(e,n)})))},h.walkComments=function(e){return this.walk((function(t,n){if("comment"===t.type)return e(t,n)}))},h.append=function(){for(var e=arguments.length,t=new Array(e),n=0;n<e;n++)t[n]=arguments[n];for(var r=0,i=t;r<i.length;r++)for(var o,s=i[r],l=this.normalize(s,this.last),u=a(l);!(o=u()).done;){var c=o.value;this.nodes.push(c)}return this},h.prepend=function(){for(var e=arguments.length,t=new Array(e),n=0;n<e;n++)t[n]=arguments[n];for(var r,i=a(t=t.reverse());!(r=i()).done;){for(var o,s=r.value,l=this.normalize(s,this.first,"prepend").reverse(),u=a(l);!(o=u()).done;){var c=o.value;this.nodes.unshift(c)}for(var d in this.indexes)this.indexes[d]=this.indexes[d]+l.length}return this},h.cleanRaws=function(e){if(t.prototype.cleanRaws.call(this,e),this.nodes)for(var n,r=a(this.nodes);!(n=r()).done;)n.value.cleanRaws(e)},h.insertBefore=function(e,t){for(var n,r,i=0===(e=this.index(e))&&"prepend",o=this.normalize(t,this.nodes[e],i).reverse(),s=a(o);!(n=s()).done;){var l=n.value;this.nodes.splice(e,0,l)}for(var u in this.indexes)e<=(r=this.indexes[u])&&(this.indexes[u]=r+o.length);return this},h.insertAfter=function(e,t){e=this.index(e);for(var n,r,i=this.normalize(t,this.nodes[e]).reverse(),o=a(i);!(n=o()).done;){var s=n.value;this.nodes.splice(e+1,0,s)}for(var l in this.indexes)e<(r=this.indexes[l])&&(this.indexes[l]=r+i.length);return this},h.removeChild=function(e){var t;for(var n in e=this.index(e),this.nodes[e].parent=void 0,this.nodes.splice(e,1),this.indexes)(t=this.indexes[n])>=e&&(this.indexes[n]=t-1);return this},h.removeAll=function(){for(var e,t=a(this.nodes);!(e=t()).done;)e.value.parent=void 0;return this.nodes=[],this},h.replaceValues=function(e,t,n){return n||(n=t,t={}),this.walkDecls((function(r){t.props&&-1===t.props.indexOf(r.prop)||t.fast&&-1===r.value.indexOf(t.fast)||(r.value=r.value.replace(e,n))})),this},h.every=function(e){return this.nodes.every(e)},h.some=function(e){return this.nodes.some(e)},h.index=function(e){return"number"==typeof e?e:this.nodes.indexOf(e)},h.normalize=function(t,n){var o=this;if("string"==typeof t)t=function e(t){return t.map((function(t){return t.nodes&&(t.nodes=e(t.nodes)),delete t.source,t}))}(e("./parse")(t).nodes);else if(Array.isArray(t))for(var s,l=a(t=t.slice(0));!(s=l()).done;){var u=s.value;u.parent&&u.parent.removeChild(u,"ignore")}else if("root"===t.type)for(var c,d=a(t=t.nodes.slice(0));!(c=d()).done;){var h=c.value;h.parent&&h.parent.removeChild(h,"ignore")}else if(t.type)t=[t];else if(t.prop){if(void 0===t.value)throw new Error("Value field is missed in node creation");"string"!=typeof t.value&&(t.value=String(t.value)),t=[new r.default(t)]}else if(t.selector)t=[new(e("./rule"))(t)];else if(t.name)t=[new(e("./at-rule"))(t)];else{if(!t.text)throw new Error("Unknown node type in node creation");t=[new i.default(t)]}return t.map((function(e){return e.parent&&e.parent.removeChild(e),void 0===e.raws.before&&n&&void 0!==n.raws.before&&(e.raws.before=n.raws.before.replace(/[^\s]/g,"")),e.parent=o,e}))},u=s,(c=[{key:"first",get:function(){if(this.nodes)return this.nodes[0]}},{key:"last",get:function(){if(this.nodes)return this.nodes[this.nodes.length-1]}}])&&l(u.prototype,c),d&&l(u,d),s}(o(e("./node")).default);n.default=u,t.exports=n.default},{"./at-rule":169,"./comment":170,"./declaration":173,"./node":178,"./parse":179,"./rule":186}],172:[function(e,t,n){n.__esModule=!0,n.default=void 0;var r=a(e("supports-color")),i=a(e("chalk")),o=a(e("./terminal-highlight"));function a(e){return e&&e.__esModule?e:{default:e}}function s(e){var t="function"==typeof Map?new Map:void 0;return(s=function(e){if(null===e||(n=e,-1===Function.toString.call(n).indexOf("[native code]")))return e;var n;if("function"!=typeof e)throw new TypeError("Super expression must either be null or a function");if(void 0!==t){if(t.has(e))return t.get(e);t.set(e,r)}function r(){return l(e,arguments,d(this).constructor)}return r.prototype=Object.create(e.prototype,{constructor:{value:r,enumerable:!1,writable:!0,configurable:!0}}),c(r,e)})(e)}function l(e,t,n){return(l=u()?Reflect.construct:function(e,t,n){var r=[null];r.push.apply(r,t);var i=new(Function.bind.apply(e,r));return n&&c(i,n.prototype),i}).apply(null,arguments)}function u(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],(function(){}))),!0}catch(e){return!1}}function c(e,t){return(c=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function d(e){return(d=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}var h=function(e){var t,n;function a(t,n,r,i,o,s){var l;return(l=e.call(this,t)||this).name="CssSyntaxError",l.reason=t,o&&(l.file=o),i&&(l.source=i),s&&(l.plugin=s),void 0!==n&&void 0!==r&&(l.line=n,l.column=r),l.setMessage(),Error.captureStackTrace&&Error.captureStackTrace(function(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}(l),a),l}n=e,(t=a).prototype=Object.create(n.prototype),t.prototype.constructor=t,t.__proto__=n;var s=a.prototype;return s.setMessage=function(){this.message=this.plugin?this.plugin+": ":"",this.message+=this.file?this.file:"<css input>",void 0!==this.line&&(this.message+=":"+this.line+":"+this.column),this.message+=": "+this.reason},s.showSourceCode=function(e){var t=this;if(!this.source)return"";var n=this.source;o.default&&(void 0===e&&(e=r.default.stdout),e&&(n=(0,o.default)(n)));var a=n.split(/\r?\n/),s=Math.max(this.line-3,0),l=Math.min(this.line+2,a.length),u=String(l).length;function c(t){return e&&i.default.red?i.default.red.bold(t):t}function d(t){return e&&i.default.gray?i.default.gray(t):t}return a.slice(s,l).map((function(e,n){var r=s+1+n,i=" "+(" "+r).slice(-u)+" | ";if(r===t.line){var o=d(i.replace(/\d/g," "))+e.slice(0,t.column-1).replace(/[^\t]/g," ");return c(">")+d(i)+e+"\n "+o+c("^")}return" "+d(i)+e})).join("\n")},s.toString=function(){var e=this.showSourceCode();return e&&(e="\n\n"+e+"\n"),this.name+": "+this.message+e},a}(s(Error));n.default=h,t.exports=n.default},{"./terminal-highlight":2,chalk:2,"supports-color":2}],173:[function(e,t,n){var r;n.__esModule=!0,n.default=void 0;var i=function(e){var t,n;function r(t){var n;return(n=e.call(this,t)||this).type="decl",n}return n=e,(t=r).prototype=Object.create(n.prototype),t.prototype.constructor=t,t.__proto__=n,r}(((r=e("./node"))&&r.__esModule?r:{default:r}).default);n.default=i,t.exports=n.default},{"./node":178}],174:[function(e,t,n){n.__esModule=!0,n.default=void 0;var r=a(e("path")),i=a(e("./css-syntax-error")),o=a(e("./previous-map"));function a(e){return e&&e.__esModule?e:{default:e}}function s(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}var l=0,c=function(){function e(e,t){if(void 0===t&&(t={}),null==e||"object"===u(e)&&!e.toString)throw new Error("PostCSS received "+e+" instead of CSS string");this.css=e.toString(),"\ufeff"===this.css[0]||""===this.css[0]?(this.hasBOM=!0,this.css=this.css.slice(1)):this.hasBOM=!1,t.from&&(/^\w+:\/\//.test(t.from)||r.default.isAbsolute(t.from)?this.file=t.from:this.file=r.default.resolve(t.from));var n=new o.default(this.css,t);if(n.text){this.map=n;var i=n.consumer().file;!this.file&&i&&(this.file=this.mapResolve(i))}this.file||(l+=1,this.id="<input css "+l+">"),this.map&&(this.map.file=this.from)}var t,n,a,c=e.prototype;return c.error=function(e,t,n,r){var o;void 0===r&&(r={});var a=this.origin(t,n);return(o=a?new i.default(e,a.line,a.column,a.source,a.file,r.plugin):new i.default(e,t,n,this.css,this.file,r.plugin)).input={line:t,column:n,source:this.css},this.file&&(o.input.file=this.file),o},c.origin=function(e,t){if(!this.map)return!1;var n=this.map.consumer(),r=n.originalPositionFor({line:e,column:t});if(!r.source)return!1;var i={file:this.mapResolve(r.source),line:r.line,column:r.column},o=n.sourceContentFor(r.source);return o&&(i.source=o),i},c.mapResolve=function(e){return/^\w+:\/\//.test(e)?e:r.default.resolve(this.map.consumer().sourceRoot||".",e)},t=e,(n=[{key:"from",get:function(){return this.file||this.id}}])&&s(t.prototype,n),a&&s(t,a),e}();n.default=c,t.exports=n.default},{"./css-syntax-error":172,"./previous-map":182,path:168}],175:[function(e,t,n){(function(r){n.__esModule=!0,n.default=void 0;var i=c(e("./map-generator")),o=c(e("./stringify")),a=c(e("./warn-once")),s=c(e("./result")),l=c(e("./parse"));function c(e){return e&&e.__esModule?e:{default:e}}function d(e,t){var n;if("undefined"==typeof Symbol||null==e[Symbol.iterator]){if(Array.isArray(e)||(n=function(e,t){if(e){if("string"==typeof e)return h(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?h(e,t):void 0}}(e))||t&&e&&"number"==typeof e.length){n&&(e=n);var r=0;return function(){return r>=e.length?{done:!0}:{done:!1,value:e[r++]}}}throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}return(n=e[Symbol.iterator]()).next.bind(n)}function h(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,r=new Array(t);n<t;n++)r[n]=e[n];return r}function f(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}function p(e){return"object"===u(e)&&"function"==typeof e.then}var g=function(){function e(t,n,r){var i;if(this.stringified=!1,this.processed=!1,"object"===u(n)&&null!==n&&"root"===n.type)i=n;else if(n instanceof e||n instanceof s.default)i=n.root,n.map&&(void 0===r.map&&(r.map={}),r.map.inline||(r.map.inline=!1),r.map.prev=n.map);else{var o=l.default;r.syntax&&(o=r.syntax.parse),r.parser&&(o=r.parser),o.parse&&(o=o.parse);try{i=o(n,r)}catch(e){this.error=e}}this.result=new s.default(t,i,r)}var t,n,c,h=e.prototype;return h.warnings=function(){return this.sync().warnings()},h.toString=function(){return this.css},h.then=function(e,t){return"production"!==r.env.NODE_ENV&&("from"in this.opts||(0,a.default)("Without `from` option PostCSS could generate wrong source map and will not find Browserslist config. Set it to CSS file path or to `undefined` to prevent this warning.")),this.async().then(e,t)},h.catch=function(e){return this.async().catch(e)},h.finally=function(e){return this.async().then(e,e)},h.handleError=function(e,t){try{if(this.error=e,"CssSyntaxError"!==e.name||e.plugin){if(t.postcssVersion&&"production"!==r.env.NODE_ENV){var n=t.postcssPlugin,i=t.postcssVersion,o=this.result.processor.version,a=i.split("."),s=o.split(".");(a[0]!==s[0]||parseInt(a[1])>parseInt(s[1]))&&console.error("Unknown error from PostCSS plugin. Your current PostCSS version is "+o+", but "+n+" uses "+i+". Perhaps this is the source of the error below.")}}else e.plugin=t.postcssPlugin,e.setMessage()}catch(e){console&&console.error&&console.error(e)}},h.asyncTick=function(e,t){var n=this;if(this.plugin>=this.processor.plugins.length)return this.processed=!0,e();try{var r=this.processor.plugins[this.plugin],i=this.run(r);this.plugin+=1,p(i)?i.then((function(){n.asyncTick(e,t)})).catch((function(e){n.handleError(e,r),n.processed=!0,t(e)})):this.asyncTick(e,t)}catch(e){this.processed=!0,t(e)}},h.async=function(){var e=this;return this.processed?new Promise((function(t,n){e.error?n(e.error):t(e.stringify())})):(this.processing||(this.processing=new Promise((function(t,n){if(e.error)return n(e.error);e.plugin=0,e.asyncTick(t,n)})).then((function(){return e.processed=!0,e.stringify()}))),this.processing)},h.sync=function(){if(this.processed)return this.result;if(this.processed=!0,this.processing)throw new Error("Use process(css).then(cb) to work with async plugins");if(this.error)throw this.error;for(var e,t=d(this.result.processor.plugins);!(e=t()).done;){var n=e.value;if(p(this.run(n)))throw new Error("Use process(css).then(cb) to work with async plugins")}return this.result},h.run=function(e){this.result.lastPlugin=e;try{return e(this.result.root,this.result)}catch(t){throw this.handleError(t,e),t}},h.stringify=function(){if(this.stringified)return this.result;this.stringified=!0,this.sync();var e=this.result.opts,t=o.default;e.syntax&&(t=e.syntax.stringify),e.stringifier&&(t=e.stringifier),t.stringify&&(t=t.stringify);var n=new i.default(t,this.result.root,this.result.opts).generate();return this.result.css=n[0],this.result.map=n[1],this.result},t=e,(n=[{key:"processor",get:function(){return this.result.processor}},{key:"opts",get:function(){return this.result.opts}},{key:"css",get:function(){return this.stringify().css}},{key:"content",get:function(){return this.stringify().content}},{key:"map",get:function(){return this.stringify().map}},{key:"root",get:function(){return this.sync().root}},{key:"messages",get:function(){return this.sync().messages}}])&&f(t.prototype,n),c&&f(t,c),e}();n.default=g,t.exports=n.default}).call(this,e("_process"))},{"./map-generator":177,"./parse":179,"./result":184,"./stringify":188,"./warn-once":191,_process:193}],176:[function(e,t,n){n.__esModule=!0,n.default=void 0;var r={split:function(e,t,n){for(var r=[],i="",o=!1,a=0,s=!1,l=!1,u=0;u<e.length;u++){var c=e[u];s?l?l=!1:"\\"===c?l=!0:c===s&&(s=!1):'"'===c||"'"===c?s=c:"("===c?a+=1:")"===c?a>0&&(a-=1):0===a&&-1!==t.indexOf(c)&&(o=!0),o?(""!==i&&r.push(i.trim()),i="",o=!1):i+=c}return(n||""!==i)&&r.push(i.trim()),r},space:function(e){return r.split(e,[" ","\n","\t"])},comma:function(e){return r.split(e,[","],!0)}},i=r;n.default=i,t.exports=n.default},{}],177:[function(e,t,n){(function(r){n.__esModule=!0,n.default=void 0;var i=a(e("source-map")),o=a(e("path"));function a(e){return e&&e.__esModule?e:{default:e}}function s(e,t){var n;if("undefined"==typeof Symbol||null==e[Symbol.iterator]){if(Array.isArray(e)||(n=function(e,t){if(e){if("string"==typeof e)return l(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?l(e,t):void 0}}(e))||t&&e&&"number"==typeof e.length){n&&(e=n);var r=0;return function(){return r>=e.length?{done:!0}:{done:!1,value:e[r++]}}}throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}return(n=e[Symbol.iterator]()).next.bind(n)}function l(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,r=new Array(t);n<t;n++)r[n]=e[n];return r}var u=function(){function e(e,t,n){this.stringify=e,this.mapOpts=n.map||{},this.root=t,this.opts=n}var t=e.prototype;return t.isMap=function(){return void 0!==this.opts.map?!!this.opts.map:this.previous().length>0},t.previous=function(){var e=this;return this.previousMaps||(this.previousMaps=[],this.root.walk((function(t){if(t.source&&t.source.input.map){var n=t.source.input.map;-1===e.previousMaps.indexOf(n)&&e.previousMaps.push(n)}}))),this.previousMaps},t.isInline=function(){if(void 0!==this.mapOpts.inline)return this.mapOpts.inline;var e=this.mapOpts.annotation;return(void 0===e||!0===e)&&(!this.previous().length||this.previous().some((function(e){return e.inline})))},t.isSourcesContent=function(){return void 0!==this.mapOpts.sourcesContent?this.mapOpts.sourcesContent:!this.previous().length||this.previous().some((function(e){return e.withContent()}))},t.clearAnnotation=function(){if(!1!==this.mapOpts.annotation)for(var e,t=this.root.nodes.length-1;t>=0;t--)"comment"===(e=this.root.nodes[t]).type&&0===e.text.indexOf("# sourceMappingURL=")&&this.root.removeChild(t)},t.setSourcesContent=function(){var e=this,t={};this.root.walk((function(n){if(n.source){var r=n.source.input.from;if(r&&!t[r]){t[r]=!0;var i=e.relative(r);e.map.setSourceContent(i,n.source.input.css)}}}))},t.applyPrevMaps=function(){for(var e,t=s(this.previous());!(e=t()).done;){var n=e.value,r=this.relative(n.file),a=n.root||o.default.dirname(n.file),l=void 0;!1===this.mapOpts.sourcesContent?(l=new i.default.SourceMapConsumer(n.text)).sourcesContent&&(l.sourcesContent=l.sourcesContent.map((function(){return null}))):l=n.consumer(),this.map.applySourceMap(l,r,this.relative(a))}},t.isAnnotation=function(){return!!this.isInline()||(void 0!==this.mapOpts.annotation?this.mapOpts.annotation:!this.previous().length||this.previous().some((function(e){return e.annotation})))},t.toBase64=function(e){return r?r.from(e).toString("base64"):window.btoa(unescape(encodeURIComponent(e)))},t.addAnnotation=function(){var e;e=this.isInline()?"data:application/json;base64,"+this.toBase64(this.map.toString()):"string"==typeof this.mapOpts.annotation?this.mapOpts.annotation:this.outputFile()+".map";var t="\n";-1!==this.css.indexOf("\r\n")&&(t="\r\n"),this.css+=t+"/*# sourceMappingURL="+e+" */"},t.outputFile=function(){return this.opts.to?this.relative(this.opts.to):this.opts.from?this.relative(this.opts.from):"to.css"},t.generateMap=function(){return this.generateString(),this.isSourcesContent()&&this.setSourcesContent(),this.previous().length>0&&this.applyPrevMaps(),this.isAnnotation()&&this.addAnnotation(),this.isInline()?[this.css]:[this.css,this.map]},t.relative=function(e){if(0===e.indexOf("<"))return e;if(/^\w+:\/\//.test(e))return e;var t=this.opts.to?o.default.dirname(this.opts.to):".";return"string"==typeof this.mapOpts.annotation&&(t=o.default.dirname(o.default.resolve(t,this.mapOpts.annotation))),e=o.default.relative(t,e),"\\"===o.default.sep?e.replace(/\\/g,"/"):e},t.sourcePath=function(e){return this.mapOpts.from?this.mapOpts.from:this.relative(e.source.input.from)},t.generateString=function(){var e=this;this.css="",this.map=new i.default.SourceMapGenerator({file:this.outputFile()});var t,n,r=1,o=1;this.stringify(this.root,(function(i,a,s){if(e.css+=i,a&&"end"!==s&&(a.source&&a.source.start?e.map.addMapping({source:e.sourcePath(a),generated:{line:r,column:o-1},original:{line:a.source.start.line,column:a.source.start.column-1}}):e.map.addMapping({source:"<no source>",original:{line:1,column:0},generated:{line:r,column:o-1}})),(t=i.match(/\n/g))?(r+=t.length,n=i.lastIndexOf("\n"),o=i.length-n):o+=i.length,a&&"start"!==s){var l=a.parent||{raws:{}};("decl"!==a.type||a!==l.last||l.raws.semicolon)&&(a.source&&a.source.end?e.map.addMapping({source:e.sourcePath(a),generated:{line:r,column:o-2},original:{line:a.source.end.line,column:a.source.end.column-1}}):e.map.addMapping({source:"<no source>",original:{line:1,column:0},generated:{line:r,column:o-1}}))}}))},t.generate=function(){if(this.clearAnnotation(),this.isMap())return this.generateMap();var e="";return this.stringify(this.root,(function(t){e+=t})),[e]},e}();n.default=u,t.exports=n.default}).call(this,e("buffer").Buffer)},{buffer:3,path:168,"source-map":208}],178:[function(e,t,n){(function(r){n.__esModule=!0,n.default=void 0;var i=s(e("./css-syntax-error")),o=s(e("./stringifier")),a=s(e("./stringify"));function s(e){return e&&e.__esModule?e:{default:e}}var l=function(){function e(e){if(void 0===e&&(e={}),this.raws={},"production"!==r.env.NODE_ENV&&"object"!==u(e)&&void 0!==e)throw new Error("PostCSS nodes constructor accepts object, not "+JSON.stringify(e));for(var t in e)this[t]=e[t]}var t=e.prototype;return t.error=function(e,t){if(void 0===t&&(t={}),this.source){var n=this.positionBy(t);return this.source.input.error(e,n.line,n.column,t)}return new i.default(e)},t.warn=function(e,t,n){var r={node:this};for(var i in n)r[i]=n[i];return e.warn(t,r)},t.remove=function(){return this.parent&&this.parent.removeChild(this),this.parent=void 0,this},t.toString=function(e){void 0===e&&(e=a.default),e.stringify&&(e=e.stringify);var t="";return e(this,(function(e){t+=e})),t},t.clone=function(e){void 0===e&&(e={});var t=function e(t,n){var r=new t.constructor;for(var i in t)if(t.hasOwnProperty(i)){var o=t[i],a=u(o);"parent"===i&&"object"===a?n&&(r[i]=n):"source"===i?r[i]=o:o instanceof Array?r[i]=o.map((function(t){return e(t,r)})):("object"===a&&null!==o&&(o=e(o)),r[i]=o)}return r}(this);for(var n in e)t[n]=e[n];return t},t.cloneBefore=function(e){void 0===e&&(e={});var t=this.clone(e);return this.parent.insertBefore(this,t),t},t.cloneAfter=function(e){void 0===e&&(e={});var t=this.clone(e);return this.parent.insertAfter(this,t),t},t.replaceWith=function(){if(this.parent){for(var e=arguments.length,t=new Array(e),n=0;n<e;n++)t[n]=arguments[n];for(var r=0,i=t;r<i.length;r++){var o=i[r];this.parent.insertBefore(this,o)}this.remove()}return this},t.next=function(){if(this.parent){var e=this.parent.index(this);return this.parent.nodes[e+1]}},t.prev=function(){if(this.parent){var e=this.parent.index(this);return this.parent.nodes[e-1]}},t.before=function(e){return this.parent.insertBefore(this,e),this},t.after=function(e){return this.parent.insertAfter(this,e),this},t.toJSON=function(){var e={};for(var t in this)if(this.hasOwnProperty(t)&&"parent"!==t){var n=this[t];n instanceof Array?e[t]=n.map((function(e){return"object"===u(e)&&e.toJSON?e.toJSON():e})):"object"===u(n)&&n.toJSON?e[t]=n.toJSON():e[t]=n}return e},t.raw=function(e,t){return(new o.default).raw(this,e,t)},t.root=function(){for(var e=this;e.parent;)e=e.parent;return e},t.cleanRaws=function(e){delete this.raws.before,delete this.raws.after,e||delete this.raws.between},t.positionInside=function(e){for(var t=this.toString(),n=this.source.start.column,r=this.source.start.line,i=0;i<e;i++)"\n"===t[i]?(n=1,r+=1):n+=1;return{line:r,column:n}},t.positionBy=function(e){var t=this.source.start;if(e.index)t=this.positionInside(e.index);else if(e.word){var n=this.toString().indexOf(e.word);-1!==n&&(t=this.positionInside(n))}return t},e}();n.default=l,t.exports=n.default}).call(this,e("_process"))},{"./css-syntax-error":172,"./stringifier":187,"./stringify":188,_process:193}],179:[function(e,t,n){(function(r){n.__esModule=!0,n.default=void 0;var i=a(e("./parser")),o=a(e("./input"));function a(e){return e&&e.__esModule?e:{default:e}}var s=function(e,t){var n=new o.default(e,t),a=new i.default(n);try{a.parse()}catch(e){throw"production"!==r.env.NODE_ENV&&"CssSyntaxError"===e.name&&t&&t.from&&(/\.scss$/i.test(t.from)?e.message+="\nYou tried to parse SCSS with the standard CSS parser; try again with the postcss-scss parser":/\.sass/i.test(t.from)?e.message+="\nYou tried to parse Sass with the standard CSS parser; try again with the postcss-sass parser":/\.less$/i.test(t.from)&&(e.message+="\nYou tried to parse Less with the standard CSS parser; try again with the postcss-less parser")),e}return a.root};n.default=s,t.exports=n.default}).call(this,e("_process"))},{"./input":174,"./parser":180,_process:193}],180:[function(e,t,n){n.__esModule=!0,n.default=void 0;var r=u(e("./declaration")),i=u(e("./tokenize")),o=u(e("./comment")),a=u(e("./at-rule")),s=u(e("./root")),l=u(e("./rule"));function u(e){return e&&e.__esModule?e:{default:e}}var c=function(){function e(e){this.input=e,this.root=new s.default,this.current=this.root,this.spaces="",this.semicolon=!1,this.createTokenizer(),this.root.source={input:e,start:{line:1,column:1}}}var t=e.prototype;return t.createTokenizer=function(){this.tokenizer=(0,i.default)(this.input)},t.parse=function(){for(var e;!this.tokenizer.endOfFile();)switch((e=this.tokenizer.nextToken())[0]){case"space":this.spaces+=e[1];break;case";":this.freeSemicolon(e);break;case"}":this.end(e);break;case"comment":this.comment(e);break;case"at-word":this.atrule(e);break;case"{":this.emptyRule(e);break;default:this.other(e)}this.endFile()},t.comment=function(e){var t=new o.default;this.init(t,e[2],e[3]),t.source.end={line:e[4],column:e[5]};var n=e[1].slice(2,-2);if(/^\s*$/.test(n))t.text="",t.raws.left=n,t.raws.right="";else{var r=n.match(/^(\s*)([^]*[^\s])(\s*)$/);t.text=r[2],t.raws.left=r[1],t.raws.right=r[3]}},t.emptyRule=function(e){var t=new l.default;this.init(t,e[2],e[3]),t.selector="",t.raws.between="",this.current=t},t.other=function(e){for(var t=!1,n=null,r=!1,i=null,o=[],a=[],s=e;s;){if(n=s[0],a.push(s),"("===n||"["===n)i||(i=s),o.push("("===n?")":"]");else if(0===o.length){if(";"===n){if(r)return void this.decl(a);break}if("{"===n)return void this.rule(a);if("}"===n){this.tokenizer.back(a.pop()),t=!0;break}":"===n&&(r=!0)}else n===o[o.length-1]&&(o.pop(),0===o.length&&(i=null));s=this.tokenizer.nextToken()}if(this.tokenizer.endOfFile()&&(t=!0),o.length>0&&this.unclosedBracket(i),t&&r){for(;a.length&&("space"===(s=a[a.length-1][0])||"comment"===s);)this.tokenizer.back(a.pop());this.decl(a)}else this.unknownWord(a)},t.rule=function(e){e.pop();var t=new l.default;this.init(t,e[0][2],e[0][3]),t.raws.between=this.spacesAndCommentsFromEnd(e),this.raw(t,"selector",e),this.current=t},t.decl=function(e){var t=new r.default;this.init(t);var n,i=e[e.length-1];for(";"===i[0]&&(this.semicolon=!0,e.pop()),i[4]?t.source.end={line:i[4],column:i[5]}:t.source.end={line:i[2],column:i[3]};"word"!==e[0][0];)1===e.length&&this.unknownWord(e),t.raws.before+=e.shift()[1];for(t.source.start={line:e[0][2],column:e[0][3]},t.prop="";e.length;){var o=e[0][0];if(":"===o||"space"===o||"comment"===o)break;t.prop+=e.shift()[1]}for(t.raws.between="";e.length;){if(":"===(n=e.shift())[0]){t.raws.between+=n[1];break}"word"===n[0]&&/\w/.test(n[1])&&this.unknownWord([n]),t.raws.between+=n[1]}"_"!==t.prop[0]&&"*"!==t.prop[0]||(t.raws.before+=t.prop[0],t.prop=t.prop.slice(1)),t.raws.between+=this.spacesAndCommentsFromStart(e),this.precheckMissedSemicolon(e);for(var a=e.length-1;a>0;a--){if("!important"===(n=e[a])[1].toLowerCase()){t.important=!0;var s=this.stringFrom(e,a);" !important"!==(s=this.spacesFromEnd(e)+s)&&(t.raws.important=s);break}if("important"===n[1].toLowerCase()){for(var l=e.slice(0),u="",c=a;c>0;c--){var d=l[c][0];if(0===u.trim().indexOf("!")&&"space"!==d)break;u=l.pop()[1]+u}0===u.trim().indexOf("!")&&(t.important=!0,t.raws.important=u,e=l)}if("space"!==n[0]&&"comment"!==n[0])break}this.raw(t,"value",e),-1!==t.value.indexOf(":")&&this.checkMissedSemicolon(e)},t.atrule=function(e){var t,n,r=new a.default;r.name=e[1].slice(1),""===r.name&&this.unnamedAtrule(r,e),this.init(r,e[2],e[3]);for(var i=!1,o=!1,s=[];!this.tokenizer.endOfFile();){if(";"===(e=this.tokenizer.nextToken())[0]){r.source.end={line:e[2],column:e[3]},this.semicolon=!0;break}if("{"===e[0]){o=!0;break}if("}"===e[0]){if(s.length>0){for(t=s[n=s.length-1];t&&"space"===t[0];)t=s[--n];t&&(r.source.end={line:t[4],column:t[5]})}this.end(e);break}if(s.push(e),this.tokenizer.endOfFile()){i=!0;break}}r.raws.between=this.spacesAndCommentsFromEnd(s),s.length?(r.raws.afterName=this.spacesAndCommentsFromStart(s),this.raw(r,"params",s),i&&(e=s[s.length-1],r.source.end={line:e[4],column:e[5]},this.spaces=r.raws.between,r.raws.between="")):(r.raws.afterName="",r.params=""),o&&(r.nodes=[],this.current=r)},t.end=function(e){this.current.nodes&&this.current.nodes.length&&(this.current.raws.semicolon=this.semicolon),this.semicolon=!1,this.current.raws.after=(this.current.raws.after||"")+this.spaces,this.spaces="",this.current.parent?(this.current.source.end={line:e[2],column:e[3]},this.current=this.current.parent):this.unexpectedClose(e)},t.endFile=function(){this.current.parent&&this.unclosedBlock(),this.current.nodes&&this.current.nodes.length&&(this.current.raws.semicolon=this.semicolon),this.current.raws.after=(this.current.raws.after||"")+this.spaces},t.freeSemicolon=function(e){if(this.spaces+=e[1],this.current.nodes){var t=this.current.nodes[this.current.nodes.length-1];t&&"rule"===t.type&&!t.raws.ownSemicolon&&(t.raws.ownSemicolon=this.spaces,this.spaces="")}},t.init=function(e,t,n){this.current.push(e),e.source={start:{line:t,column:n},input:this.input},e.raws.before=this.spaces,this.spaces="","comment"!==e.type&&(this.semicolon=!1)},t.raw=function(e,t,n){for(var r,i,o,a,s=n.length,l="",u=!0,c=/^([.|#])?([\w])+/i,d=0;d<s;d+=1)"comment"!==(i=(r=n[d])[0])||"rule"!==e.type?"comment"===i||"space"===i&&d===s-1?u=!1:l+=r[1]:(a=n[d-1],o=n[d+1],"space"!==a[0]&&"space"!==o[0]&&c.test(a[1])&&c.test(o[1])?l+=r[1]:u=!1);if(!u){var h=n.reduce((function(e,t){return e+t[1]}),"");e.raws[t]={value:l,raw:h}}e[t]=l},t.spacesAndCommentsFromEnd=function(e){for(var t,n="";e.length&&("space"===(t=e[e.length-1][0])||"comment"===t);)n=e.pop()[1]+n;return n},t.spacesAndCommentsFromStart=function(e){for(var t,n="";e.length&&("space"===(t=e[0][0])||"comment"===t);)n+=e.shift()[1];return n},t.spacesFromEnd=function(e){for(var t="";e.length&&"space"===e[e.length-1][0];)t=e.pop()[1]+t;return t},t.stringFrom=function(e,t){for(var n="",r=t;r<e.length;r++)n+=e[r][1];return e.splice(t,e.length-t),n},t.colon=function(e){for(var t,n,r,i=0,o=0;o<e.length;o++){if("("===(n=(t=e[o])[0])&&(i+=1),")"===n&&(i-=1),0===i&&":"===n){if(r){if("word"===r[0]&&"progid"===r[1])continue;return o}this.doubleColon(t)}r=t}return!1},t.unclosedBracket=function(e){throw this.input.error("Unclosed bracket",e[2],e[3])},t.unknownWord=function(e){throw this.input.error("Unknown word",e[0][2],e[0][3])},t.unexpectedClose=function(e){throw this.input.error("Unexpected }",e[2],e[3])},t.unclosedBlock=function(){var e=this.current.source.start;throw this.input.error("Unclosed block",e.line,e.column)},t.doubleColon=function(e){throw this.input.error("Double colon",e[2],e[3])},t.unnamedAtrule=function(e,t){throw this.input.error("At-rule without name",t[2],t[3])},t.precheckMissedSemicolon=function(){},t.checkMissedSemicolon=function(e){var t=this.colon(e);if(!1!==t){for(var n,r=0,i=t-1;i>=0&&("space"===(n=e[i])[0]||2!==(r+=1));i--);throw this.input.error("Missed semicolon",n[2],n[3])}},e}();n.default=c,t.exports=n.default},{"./at-rule":169,"./comment":170,"./declaration":173,"./root":185,"./rule":186,"./tokenize":189}],181:[function(e,t,n){n.__esModule=!0,n.default=void 0;var r=f(e("./declaration")),i=f(e("./processor")),o=f(e("./stringify")),a=f(e("./comment")),s=f(e("./at-rule")),l=f(e("./vendor")),u=f(e("./parse")),c=f(e("./list")),d=f(e("./rule")),h=f(e("./root"));function f(e){return e&&e.__esModule?e:{default:e}}function p(){for(var e=arguments.length,t=new Array(e),n=0;n<e;n++)t[n]=arguments[n];return 1===t.length&&Array.isArray(t[0])&&(t=t[0]),new i.default(t)}p.plugin=function(e,t){function n(){var n=t.apply(void 0,arguments);return n.postcssPlugin=e,n.postcssVersion=(new i.default).version,n}var r;return Object.defineProperty(n,"postcss",{get:function(){return r||(r=n()),r}}),n.process=function(e,t,r){return p([n(r)]).process(e,t)},n},p.stringify=o.default,p.parse=u.default,p.vendor=l.default,p.list=c.default,p.comment=function(e){return new a.default(e)},p.atRule=function(e){return new s.default(e)},p.decl=function(e){return new r.default(e)},p.rule=function(e){return new d.default(e)},p.root=function(e){return new h.default(e)};var g=p;n.default=g,t.exports=n.default},{"./at-rule":169,"./comment":170,"./declaration":173,"./list":176,"./parse":179,"./processor":183,"./root":185,"./rule":186,"./stringify":188,"./vendor":190}],182:[function(e,t,n){(function(r){n.__esModule=!0,n.default=void 0;var i=s(e("source-map")),o=s(e("path")),a=s(e("fs"));function s(e){return e&&e.__esModule?e:{default:e}}var l=function(){function e(e,t){this.loadAnnotation(e),this.inline=this.startWith(this.annotation,"data:");var n=t.map?t.map.prev:void 0,r=this.loadMap(t.from,n);r&&(this.text=r)}var t=e.prototype;return t.consumer=function(){return this.consumerCache||(this.consumerCache=new i.default.SourceMapConsumer(this.text)),this.consumerCache},t.withContent=function(){return!!(this.consumer().sourcesContent&&this.consumer().sourcesContent.length>0)},t.startWith=function(e,t){return!!e&&e.substr(0,t.length)===t},t.getAnnotationURL=function(e){return e.match(/\/\*\s*# sourceMappingURL=(.*)\s*\*\//)[1].trim()},t.loadAnnotation=function(e){var t=e.match(/\/\*\s*# sourceMappingURL=(.*)\s*\*\//gm);if(t&&t.length>0){var n=t[t.length-1];n&&(this.annotation=this.getAnnotationURL(n))}},t.decodeInline=function(e){var t,n="data:application/json,";if(this.startWith(e,n))return decodeURIComponent(e.substr(n.length));if(/^data:application\/json;charset=utf-?8;base64,/.test(e)||/^data:application\/json;base64,/.test(e))return t=e.substr(RegExp.lastMatch.length),r?r.from(t,"base64").toString():window.atob(t);var i=e.match(/data:application\/json;([^,]+),/)[1];throw new Error("Unsupported source map encoding "+i)},t.loadMap=function(e,t){if(!1===t)return!1;if(t){if("string"==typeof t)return t;if("function"==typeof t){var n=t(e);if(n&&a.default.existsSync&&a.default.existsSync(n))return a.default.readFileSync(n,"utf-8").toString().trim();throw new Error("Unable to load previous source map: "+n.toString())}if(t instanceof i.default.SourceMapConsumer)return i.default.SourceMapGenerator.fromSourceMap(t).toString();if(t instanceof i.default.SourceMapGenerator)return t.toString();if(this.isMap(t))return JSON.stringify(t);throw new Error("Unsupported previous source map format: "+t.toString())}if(this.inline)return this.decodeInline(this.annotation);if(this.annotation){var r=this.annotation;return e&&(r=o.default.join(o.default.dirname(e),r)),this.root=o.default.dirname(r),!(!a.default.existsSync||!a.default.existsSync(r))&&a.default.readFileSync(r,"utf-8").toString().trim()}},t.isMap=function(e){return"object"===u(e)&&("string"==typeof e.mappings||"string"==typeof e._mappings)},e}();n.default=l,t.exports=n.default}).call(this,e("buffer").Buffer)},{buffer:3,fs:2,path:168,"source-map":208}],183:[function(e,t,n){(function(r){n.__esModule=!0,n.default=void 0;var i,o=(i=e("./lazy-result"))&&i.__esModule?i:{default:i};function a(e,t){var n;if("undefined"==typeof Symbol||null==e[Symbol.iterator]){if(Array.isArray(e)||(n=function(e,t){if(e){if("string"==typeof e)return s(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?s(e,t):void 0}}(e))||t&&e&&"number"==typeof e.length){n&&(e=n);var r=0;return function(){return r>=e.length?{done:!0}:{done:!1,value:e[r++]}}}throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}return(n=e[Symbol.iterator]()).next.bind(n)}function s(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,r=new Array(t);n<t;n++)r[n]=e[n];return r}var l=function(){function e(e){void 0===e&&(e=[]),this.version="7.0.34",this.plugins=this.normalize(e)}var t=e.prototype;return t.use=function(e){return this.plugins=this.plugins.concat(this.normalize([e])),this},t.process=function(e){function t(t){return e.apply(this,arguments)}return t.toString=function(){return e.toString()},t}((function(e,t){return void 0===t&&(t={}),0===this.plugins.length&&t.parser===t.stringifier&&"production"!==r.env.NODE_ENV&&"undefined"!=typeof console&&console.warn&&console.warn("You did not set any plugins, parser, or stringifier. Right now, PostCSS does nothing. Pick plugins for your case on https://www.postcss.parts/ and use them in postcss.config.js."),new o.default(this,e,t)})),t.normalize=function(e){for(var t,n=[],i=a(e);!(t=i()).done;){var o=t.value;if(!0===o.postcss){var s=o();throw new Error("PostCSS plugin "+s.postcssPlugin+" requires PostCSS 8. Update PostCSS or downgrade this plugin.")}if(o.postcss&&(o=o.postcss),"object"===u(o)&&Array.isArray(o.plugins))n=n.concat(o.plugins);else if("function"==typeof o)n.push(o);else{if("object"!==u(o)||!o.parse&&!o.stringify)throw"object"===u(o)&&o.postcssPlugin?new Error("PostCSS plugin "+o.postcssPlugin+" requires PostCSS 8. Update PostCSS or downgrade this plugin."):new Error(o+" is not a PostCSS plugin");if("production"!==r.env.NODE_ENV)throw new Error("PostCSS syntaxes cannot be used as plugins. Instead, please use one of the syntax/parser/stringifier options as outlined in your PostCSS runner documentation.")}}return n},e}();n.default=l,t.exports=n.default}).call(this,e("_process"))},{"./lazy-result":175,_process:193}],184:[function(e,t,n){n.__esModule=!0,n.default=void 0;var r,i=(r=e("./warning"))&&r.__esModule?r:{default:r};function o(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}var a=function(){function e(e,t,n){this.processor=e,this.messages=[],this.root=t,this.opts=n,this.css=void 0,this.map=void 0}var t,n,r,a=e.prototype;return a.toString=function(){return this.css},a.warn=function(e,t){void 0===t&&(t={}),t.plugin||this.lastPlugin&&this.lastPlugin.postcssPlugin&&(t.plugin=this.lastPlugin.postcssPlugin);var n=new i.default(e,t);return this.messages.push(n),n},a.warnings=function(){return this.messages.filter((function(e){return"warning"===e.type}))},t=e,(n=[{key:"content",get:function(){return this.css}}])&&o(t.prototype,n),r&&o(t,r),e}();n.default=a,t.exports=n.default},{"./warning":192}],185:[function(e,t,n){var r;function i(e,t){var n;if("undefined"==typeof Symbol||null==e[Symbol.iterator]){if(Array.isArray(e)||(n=function(e,t){if(e){if("string"==typeof e)return o(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?o(e,t):void 0}}(e))||t&&e&&"number"==typeof e.length){n&&(e=n);var r=0;return function(){return r>=e.length?{done:!0}:{done:!1,value:e[r++]}}}throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}return(n=e[Symbol.iterator]()).next.bind(n)}function o(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,r=new Array(t);n<t;n++)r[n]=e[n];return r}n.__esModule=!0,n.default=void 0;var a=function(t){var n,r;function o(e){var n;return(n=t.call(this,e)||this).type="root",n.nodes||(n.nodes=[]),n}r=t,(n=o).prototype=Object.create(r.prototype),n.prototype.constructor=n,n.__proto__=r;var a=o.prototype;return a.removeChild=function(e,n){var r=this.index(e);return!n&&0===r&&this.nodes.length>1&&(this.nodes[1].raws.before=this.nodes[r].raws.before),t.prototype.removeChild.call(this,e)},a.normalize=function(e,n,r){var o=t.prototype.normalize.call(this,e);if(n)if("prepend"===r)this.nodes.length>1?n.raws.before=this.nodes[1].raws.before:delete n.raws.before;else if(this.first!==n)for(var a,s=i(o);!(a=s()).done;)a.value.raws.before=n.raws.before;return o},a.toResult=function(t){return void 0===t&&(t={}),new(e("./lazy-result"))(new(e("./processor")),this,t).stringify()},o}(((r=e("./container"))&&r.__esModule?r:{default:r}).default);n.default=a,t.exports=n.default},{"./container":171,"./lazy-result":175,"./processor":183}],186:[function(e,t,n){n.__esModule=!0,n.default=void 0;var r=o(e("./container")),i=o(e("./list"));function o(e){return e&&e.__esModule?e:{default:e}}function a(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}var s=function(e){var t,n,r,o,s;function l(t){var n;return(n=e.call(this,t)||this).type="rule",n.nodes||(n.nodes=[]),n}return n=e,(t=l).prototype=Object.create(n.prototype),t.prototype.constructor=t,t.__proto__=n,r=l,(o=[{key:"selectors",get:function(){return i.default.comma(this.selector)},set:function(e){var t=this.selector?this.selector.match(/,\s*/):null,n=t?t[0]:","+this.raw("between","beforeOpen");this.selector=e.join(n)}}])&&a(r.prototype,o),s&&a(r,s),l}(r.default);n.default=s,t.exports=n.default},{"./container":171,"./list":176}],187:[function(e,t,n){n.__esModule=!0,n.default=void 0;var r={colon:": ",indent:"    ",beforeDecl:"\n",beforeRule:"\n",beforeOpen:" ",beforeClose:"\n",beforeComment:"\n",after:"\n",emptyBody:"",commentLeft:" ",commentRight:" ",semicolon:!1},i=function(){function e(e){this.builder=e}var t=e.prototype;return t.stringify=function(e,t){this[e.type](e,t)},t.root=function(e){this.body(e),e.raws.after&&this.builder(e.raws.after)},t.comment=function(e){var t=this.raw(e,"left","commentLeft"),n=this.raw(e,"right","commentRight");this.builder("/*"+t+e.text+n+"*/",e)},t.decl=function(e,t){var n=this.raw(e,"between","colon"),r=e.prop+n+this.rawValue(e,"value");e.important&&(r+=e.raws.important||" !important"),t&&(r+=";"),this.builder(r,e)},t.rule=function(e){this.block(e,this.rawValue(e,"selector")),e.raws.ownSemicolon&&this.builder(e.raws.ownSemicolon,e,"end")},t.atrule=function(e,t){var n="@"+e.name,r=e.params?this.rawValue(e,"params"):"";if(void 0!==e.raws.afterName?n+=e.raws.afterName:r&&(n+=" "),e.nodes)this.block(e,n+r);else{var i=(e.raws.between||"")+(t?";":"");this.builder(n+r+i,e)}},t.body=function(e){for(var t=e.nodes.length-1;t>0&&"comment"===e.nodes[t].type;)t-=1;for(var n=this.raw(e,"semicolon"),r=0;r<e.nodes.length;r++){var i=e.nodes[r],o=this.raw(i,"before");o&&this.builder(o),this.stringify(i,t!==r||n)}},t.block=function(e,t){var n,r=this.raw(e,"between","beforeOpen");this.builder(t+r+"{",e,"start"),e.nodes&&e.nodes.length?(this.body(e),n=this.raw(e,"after")):n=this.raw(e,"after","emptyBody"),n&&this.builder(n),this.builder("}",e,"end")},t.raw=function(e,t,n){var i;if(n||(n=t),t&&void 0!==(i=e.raws[t]))return i;var o=e.parent;if("before"===n&&(!o||"root"===o.type&&o.first===e))return"";if(!o)return r[n];var a=e.root();if(a.rawCache||(a.rawCache={}),void 0!==a.rawCache[n])return a.rawCache[n];if("before"===n||"after"===n)return this.beforeAfter(e,n);var s,l="raw"+((s=n)[0].toUpperCase()+s.slice(1));return this[l]?i=this[l](a,e):a.walk((function(e){if(void 0!==(i=e.raws[t]))return!1})),void 0===i&&(i=r[n]),a.rawCache[n]=i,i},t.rawSemicolon=function(e){var t;return e.walk((function(e){if(e.nodes&&e.nodes.length&&"decl"===e.last.type&&void 0!==(t=e.raws.semicolon))return!1})),t},t.rawEmptyBody=function(e){var t;return e.walk((function(e){if(e.nodes&&0===e.nodes.length&&void 0!==(t=e.raws.after))return!1})),t},t.rawIndent=function(e){return e.raws.indent?e.raws.indent:(e.walk((function(n){var r=n.parent;if(r&&r!==e&&r.parent&&r.parent===e&&void 0!==n.raws.before){var i=n.raws.before.split("\n");return t=(t=i[i.length-1]).replace(/[^\s]/g,""),!1}})),t);var t},t.rawBeforeComment=function(e,t){var n;return e.walkComments((function(e){if(void 0!==e.raws.before)return-1!==(n=e.raws.before).indexOf("\n")&&(n=n.replace(/[^\n]+$/,"")),!1})),void 0===n?n=this.raw(t,null,"beforeDecl"):n&&(n=n.replace(/[^\s]/g,"")),n},t.rawBeforeDecl=function(e,t){var n;return e.walkDecls((function(e){if(void 0!==e.raws.before)return-1!==(n=e.raws.before).indexOf("\n")&&(n=n.replace(/[^\n]+$/,"")),!1})),void 0===n?n=this.raw(t,null,"beforeRule"):n&&(n=n.replace(/[^\s]/g,"")),n},t.rawBeforeRule=function(e){var t;return e.walk((function(n){if(n.nodes&&(n.parent!==e||e.first!==n)&&void 0!==n.raws.before)return-1!==(t=n.raws.before).indexOf("\n")&&(t=t.replace(/[^\n]+$/,"")),!1})),t&&(t=t.replace(/[^\s]/g,"")),t},t.rawBeforeClose=function(e){var t;return e.walk((function(e){if(e.nodes&&e.nodes.length>0&&void 0!==e.raws.after)return-1!==(t=e.raws.after).indexOf("\n")&&(t=t.replace(/[^\n]+$/,"")),!1})),t&&(t=t.replace(/[^\s]/g,"")),t},t.rawBeforeOpen=function(e){var t;return e.walk((function(e){if("decl"!==e.type&&void 0!==(t=e.raws.between))return!1})),t},t.rawColon=function(e){var t;return e.walkDecls((function(e){if(void 0!==e.raws.between)return t=e.raws.between.replace(/[^\s:]/g,""),!1})),t},t.beforeAfter=function(e,t){var n;n="decl"===e.type?this.raw(e,null,"beforeDecl"):"comment"===e.type?this.raw(e,null,"beforeComment"):"before"===t?this.raw(e,null,"beforeRule"):this.raw(e,null,"beforeClose");for(var r=e.parent,i=0;r&&"root"!==r.type;)i+=1,r=r.parent;if(-1!==n.indexOf("\n")){var o=this.raw(e,null,"indent");if(o.length)for(var a=0;a<i;a++)n+=o}return n},t.rawValue=function(e,t){var n=e[t],r=e.raws[t];return r&&r.value===n?r.raw:n},e}();n.default=i,t.exports=n.default},{}],188:[function(e,t,n){n.__esModule=!0,n.default=void 0;var r,i=(r=e("./stringifier"))&&r.__esModule?r:{default:r},o=function(e,t){new i.default(t).stringify(e)};n.default=o,t.exports=n.default},{"./stringifier":187}],189:[function(e,t,n){n.__esModule=!0,n.default=function(e,t){void 0===t&&(t={});var n,M,S,T,E,O,k,L,D,P,N,j,R,z,F=e.css.valueOf(),B=t.ignoreErrors,H=F.length,U=-1,V=1,Y=0,W=[],G=[];function X(t){throw e.error("Unclosed "+t,V,Y-U)}return{back:function(e){G.push(e)},nextToken:function(e){if(G.length)return G.pop();if(!(Y>=H)){var t=!!e&&e.ignoreUnclosed;switch(((n=F.charCodeAt(Y))===s||n===u||n===d&&F.charCodeAt(Y+1)!==s)&&(U=Y,V+=1),n){case s:case l:case c:case d:case u:M=Y;do{M+=1,(n=F.charCodeAt(M))===s&&(U=M,V+=1)}while(n===l||n===s||n===c||n===d||n===u);z=["space",F.slice(Y,M)],Y=M-1;break;case h:case f:case v:case m:case C:case y:case g:var q=String.fromCharCode(n);z=[q,q,V,Y-U];break;case p:if(j=W.length?W.pop()[1]:"",R=F.charCodeAt(Y+1),"url"===j&&R!==r&&R!==i&&R!==l&&R!==s&&R!==c&&R!==u&&R!==d){M=Y;do{if(P=!1,-1===(M=F.indexOf(")",M+1))){if(B||t){M=Y;break}X("bracket")}for(N=M;F.charCodeAt(N-1)===o;)N-=1,P=!P}while(P);z=["brackets",F.slice(Y,M+1),V,Y-U,V,M-U],Y=M}else M=F.indexOf(")",Y+1),O=F.slice(Y,M+1),-1===M||x.test(O)?z=["(","(",V,Y-U]:(z=["brackets",O,V,Y-U,V,M-U],Y=M);break;case r:case i:S=n===r?"'":'"',M=Y;do{if(P=!1,-1===(M=F.indexOf(S,M+1))){if(B||t){M=Y+1;break}X("string")}for(N=M;F.charCodeAt(N-1)===o;)N-=1,P=!P}while(P);O=F.slice(Y,M+1),T=O.split("\n"),(E=T.length-1)>0?(L=V+E,D=M-T[E].length):(L=V,D=U),z=["string",F.slice(Y,M+1),V,Y-U,L,M-D],U=D,V=L,Y=M;break;case A:_.lastIndex=Y+1,_.test(F),M=0===_.lastIndex?F.length-1:_.lastIndex-2,z=["at-word",F.slice(Y,M+1),V,Y-U,V,M-U],Y=M;break;case o:for(M=Y,k=!0;F.charCodeAt(M+1)===o;)M+=1,k=!k;if(n=F.charCodeAt(M+1),k&&n!==a&&n!==l&&n!==s&&n!==c&&n!==d&&n!==u&&(M+=1,w.test(F.charAt(M)))){for(;w.test(F.charAt(M+1));)M+=1;F.charCodeAt(M+1)===l&&(M+=1)}z=["word",F.slice(Y,M+1),V,Y-U,V,M-U],Y=M;break;default:n===a&&F.charCodeAt(Y+1)===b?(0===(M=F.indexOf("*/",Y+2)+1)&&(B||t?M=F.length:X("comment")),O=F.slice(Y,M+1),T=O.split("\n"),(E=T.length-1)>0?(L=V+E,D=M-T[E].length):(L=V,D=U),z=["comment",O,V,Y-U,L,M-D],U=D,V=L,Y=M):(I.lastIndex=Y+1,I.test(F),M=0===I.lastIndex?F.length-1:I.lastIndex-2,z=["word",F.slice(Y,M+1),V,Y-U,V,M-U],W.push(z),Y=M)}return Y++,z}},endOfFile:function(){return 0===G.length&&Y>=H},position:function(){return Y}}};var r="'".charCodeAt(0),i='"'.charCodeAt(0),o="\\".charCodeAt(0),a="/".charCodeAt(0),s="\n".charCodeAt(0),l=" ".charCodeAt(0),u="\f".charCodeAt(0),c="\t".charCodeAt(0),d="\r".charCodeAt(0),h="[".charCodeAt(0),f="]".charCodeAt(0),p="(".charCodeAt(0),g=")".charCodeAt(0),v="{".charCodeAt(0),m="}".charCodeAt(0),y=";".charCodeAt(0),b="*".charCodeAt(0),C=":".charCodeAt(0),A="@".charCodeAt(0),_=/[ \n\t\r\f{}()'"\\;/[\]#]/g,I=/[ \n\t\r\f(){}:;@!'"\\\][#]|\/(?=\*)/g,x=/.[\\/("'\n]/,w=/[a-f0-9]/i;t.exports=n.default},{}],190:[function(e,t,n){n.__esModule=!0,n.default=void 0;var r={prefix:function(e){var t=e.match(/^(-\w+-)/);return t?t[0]:""},unprefixed:function(e){return e.replace(/^-\w+-/,"")}};n.default=r,t.exports=n.default},{}],191:[function(e,t,n){n.__esModule=!0,n.default=function(e){r[e]||(r[e]=!0,"undefined"!=typeof console&&console.warn&&console.warn(e))};var r={};t.exports=n.default},{}],192:[function(e,t,n){n.__esModule=!0,n.default=void 0;var r=function(){function e(e,t){if(void 0===t&&(t={}),this.type="warning",this.text=e,t.node&&t.node.source){var n=t.node.positionBy(t);this.line=n.line,this.column=n.column}for(var r in t)this[r]=t[r]}return e.prototype.toString=function(){return this.node?this.node.error(this.text,{plugin:this.plugin,index:this.index,word:this.word}).message:this.plugin?this.plugin+": "+this.text:this.text},e}();n.default=r,t.exports=n.default},{}],193:[function(e,t,n){var r,i,o=t.exports={};function a(){throw new Error("setTimeout has not been defined")}function s(){throw new Error("clearTimeout has not been defined")}function l(e){if(r===setTimeout)return setTimeout(e,0);if((r===a||!r)&&setTimeout)return r=setTimeout,setTimeout(e,0);try{return r(e,0)}catch(t){try{return r.call(null,e,0)}catch(t){return r.call(this,e,0)}}}!function(){try{r="function"==typeof setTimeout?setTimeout:a}catch(e){r=a}try{i="function"==typeof clearTimeout?clearTimeout:s}catch(e){i=s}}();var u,c=[],d=!1,h=-1;function f(){d&&u&&(d=!1,u.length?c=u.concat(c):h=-1,c.length&&p())}function p(){if(!d){var e=l(f);d=!0;for(var t=c.length;t;){for(u=c,c=[];++h<t;)u&&u[h].run();h=-1,t=c.length}u=null,d=!1,function(e){if(i===clearTimeout)return clearTimeout(e);if((i===s||!i)&&clearTimeout)return i=clearTimeout,clearTimeout(e);try{i(e)}catch(t){try{return i.call(null,e)}catch(t){return i.call(this,e)}}}(e)}}function g(e,t){this.fun=e,this.array=t}function v(){}o.nextTick=function(e){var t=new Array(arguments.length-1);if(arguments.length>1)for(var n=1;n<arguments.length;n++)t[n-1]=arguments[n];c.push(new g(e,t)),1!==c.length||d||l(p)},g.prototype.run=function(){this.fun.apply(null,this.array)},o.title="browser",o.browser=!0,o.env={},o.argv=[],o.version="",o.versions={},o.on=v,o.addListener=v,o.once=v,o.off=v,o.removeListener=v,o.removeAllListeners=v,o.emit=v,o.prependListener=v,o.prependOnceListener=v,o.listeners=function(e){return[]},o.binding=function(e){throw new Error("process.binding is not supported")},o.cwd=function(){return"/"},o.chdir=function(e){throw new Error("process.chdir is not supported")},o.umask=function(){return 0}},{}],194:[function(e,t,r){(function(e){!function(n){var i="object"==u(r)&&r&&!r.nodeType&&r,o="object"==u(t)&&t&&!t.nodeType&&t,a="object"==u(e)&&e;a.global!==a&&a.window!==a&&a.self!==a||(n=a);var s,l,c=2147483647,d=/^xn--/,h=/[^\x20-\x7E]/,f=/[\x2E\u3002\uFF0E\uFF61]/g,p={overflow:"Overflow: input needs wider integers to process","not-basic":"Illegal input >= 0x80 (not a basic code point)","invalid-input":"Invalid input"},g=Math.floor,v=String.fromCharCode;function m(e){throw new RangeError(p[e])}function y(e,t){for(var n=e.length,r=[];n--;)r[n]=t(e[n]);return r}function b(e,t){var n=e.split("@"),r="";return n.length>1&&(r=n[0]+"@",e=n[1]),r+y((e=e.replace(f,".")).split("."),t).join(".")}function C(e){for(var t,n,r=[],i=0,o=e.length;i<o;)(t=e.charCodeAt(i++))>=55296&&t<=56319&&i<o?56320==(64512&(n=e.charCodeAt(i++)))?r.push(((1023&t)<<10)+(1023&n)+65536):(r.push(t),i--):r.push(t);return r}function A(e){return y(e,(function(e){var t="";return e>65535&&(t+=v((e-=65536)>>>10&1023|55296),e=56320|1023&e),t+=v(e)})).join("")}function _(e,t){return e+22+75*(e<26)-((0!=t)<<5)}function I(e,t,n){var r=0;for(e=n?g(e/700):e>>1,e+=g(e/t);e>455;r+=36)e=g(e/35);return g(r+36*e/(e+38))}function x(e){var t,n,r,i,o,a,s,l,u,d,h,f=[],p=e.length,v=0,y=128,b=72;for((n=e.lastIndexOf("-"))<0&&(n=0),r=0;r<n;++r)e.charCodeAt(r)>=128&&m("not-basic"),f.push(e.charCodeAt(r));for(i=n>0?n+1:0;i<p;){for(o=v,a=1,s=36;i>=p&&m("invalid-input"),((l=(h=e.charCodeAt(i++))-48<10?h-22:h-65<26?h-65:h-97<26?h-97:36)>=36||l>g((c-v)/a))&&m("overflow"),v+=l*a,!(l<(u=s<=b?1:s>=b+26?26:s-b));s+=36)a>g(c/(d=36-u))&&m("overflow"),a*=d;b=I(v-o,t=f.length+1,0==o),g(v/t)>c-y&&m("overflow"),y+=g(v/t),v%=t,f.splice(v++,0,y)}return A(f)}function w(e){var t,n,r,i,o,a,s,l,u,d,h,f,p,y,b,A=[];for(f=(e=C(e)).length,t=128,n=0,o=72,a=0;a<f;++a)(h=e[a])<128&&A.push(v(h));for(r=i=A.length,i&&A.push("-");r<f;){for(s=c,a=0;a<f;++a)(h=e[a])>=t&&h<s&&(s=h);for(s-t>g((c-n)/(p=r+1))&&m("overflow"),n+=(s-t)*p,t=s,a=0;a<f;++a)if((h=e[a])<t&&++n>c&&m("overflow"),h==t){for(l=n,u=36;!(l<(d=u<=o?1:u>=o+26?26:u-o));u+=36)b=l-d,y=36-d,A.push(v(_(d+b%y,0))),l=g(b/y);A.push(v(_(l,0))),o=I(n,p,r==i),n=0,++r}++n,++t}return A.join("")}if(s={version:"1.4.1",ucs2:{decode:C,encode:A},decode:x,encode:w,toASCII:function(e){return b(e,(function(e){return h.test(e)?"xn--"+w(e):e}))},toUnicode:function(e){return b(e,(function(e){return d.test(e)?x(e.slice(4).toLowerCase()):e}))}},i&&o)if(t.exports==i)o.exports=s;else for(l in s)s.hasOwnProperty(l)&&(i[l]=s[l]);else n.punycode=s}(this)}).call(this,void 0!==n?n:"undefined"!=typeof self?self:"undefined"!=typeof window?window:{})},{}],195:[function(e,t,n){function r(e,t){return Object.prototype.hasOwnProperty.call(e,t)}t.exports=function(e,t,n,o){t=t||"&",n=n||"=";var a={};if("string"!=typeof e||0===e.length)return a;var s=/\+/g;e=e.split(t);var l=1e3;o&&"number"==typeof o.maxKeys&&(l=o.maxKeys);var u=e.length;l>0&&u>l&&(u=l);for(var c=0;c<u;++c){var d,h,f,p,g=e[c].replace(s,"%20"),v=g.indexOf(n);v>=0?(d=g.substr(0,v),h=g.substr(v+1)):(d=g,h=""),f=decodeURIComponent(d),p=decodeURIComponent(h),r(a,f)?i(a[f])?a[f].push(p):a[f]=[a[f],p]:a[f]=p}return a};var i=Array.isArray||function(e){return"[object Array]"===Object.prototype.toString.call(e)}},{}],196:[function(e,t,n){var r=function(e){switch(u(e)){case"string":return e;case"boolean":return e?"true":"false";case"number":return isFinite(e)?e:"";default:return""}};t.exports=function(e,t,n,s){return t=t||"&",n=n||"=",null===e&&(e=void 0),"object"===u(e)?o(a(e),(function(a){var s=encodeURIComponent(r(a))+n;return i(e[a])?o(e[a],(function(e){return s+encodeURIComponent(r(e))})).join(t):s+encodeURIComponent(r(e[a]))})).join(t):s?encodeURIComponent(r(s))+n+encodeURIComponent(r(e)):""};var i=Array.isArray||function(e){return"[object Array]"===Object.prototype.toString.call(e)};function o(e,t){if(e.map)return e.map(t);for(var n=[],r=0;r<e.length;r++)n.push(t(e[r],r));return n}var a=Object.keys||function(e){var t=[];for(var n in e)Object.prototype.hasOwnProperty.call(e,n)&&t.push(n);return t}},{}],197:[function(e,t,n){n.decode=n.parse=e("./decode"),n.encode=n.stringify=e("./encode")},{"./decode":195,"./encode":196}],198:[function(e,t,n){var r=e("./util"),i=Object.prototype.hasOwnProperty,o="undefined"!=typeof Map;function a(){this._array=[],this._set=o?new Map:Object.create(null)}a.fromArray=function(e,t){for(var n=new a,r=0,i=e.length;r<i;r++)n.add(e[r],t);return n},a.prototype.size=function(){return o?this._set.size:Object.getOwnPropertyNames(this._set).length},a.prototype.add=function(e,t){var n=o?e:r.toSetString(e),a=o?this.has(e):i.call(this._set,n),s=this._array.length;a&&!t||this._array.push(e),a||(o?this._set.set(e,s):this._set[n]=s)},a.prototype.has=function(e){if(o)return this._set.has(e);var t=r.toSetString(e);return i.call(this._set,t)},a.prototype.indexOf=function(e){if(o){var t=this._set.get(e);if(t>=0)return t}else{var n=r.toSetString(e);if(i.call(this._set,n))return this._set[n]}throw new Error('"'+e+'" is not in the set.')},a.prototype.at=function(e){if(e>=0&&e<this._array.length)return this._array[e];throw new Error("No element indexed by "+e)},a.prototype.toArray=function(){return this._array.slice()},n.ArraySet=a},{"./util":207}],199:[function(e,t,n){var r=e("./base64");n.encode=function(e){var t,n="",i=function(e){return e<0?1+(-e<<1):0+(e<<1)}(e);do{t=31&i,(i>>>=5)>0&&(t|=32),n+=r.encode(t)}while(i>0);return n},n.decode=function(e,t,n){var i,o,a,s,l=e.length,u=0,c=0;do{if(t>=l)throw new Error("Expected more digits in base 64 VLQ value.");if(-1===(o=r.decode(e.charCodeAt(t++))))throw new Error("Invalid base64 digit: "+e.charAt(t-1));i=!!(32&o),u+=(o&=31)<<c,c+=5}while(i);n.value=(s=(a=u)>>1,1==(1&a)?-s:s),n.rest=t}},{"./base64":200}],200:[function(e,t,n){var r="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/".split("");n.encode=function(e){if(0<=e&&e<r.length)return r[e];throw new TypeError("Must be between 0 and 63: "+e)},n.decode=function(e){return 65<=e&&e<=90?e-65:97<=e&&e<=122?e-97+26:48<=e&&e<=57?e-48+52:43==e?62:47==e?63:-1}},{}],201:[function(e,t,n){n.GREATEST_LOWER_BOUND=1,n.LEAST_UPPER_BOUND=2,n.search=function(e,t,r,i){if(0===t.length)return-1;var o=function e(t,r,i,o,a,s){var l=Math.floor((r-t)/2)+t,u=a(i,o[l],!0);return 0===u?l:u>0?r-l>1?e(l,r,i,o,a,s):s==n.LEAST_UPPER_BOUND?r<o.length?r:-1:l:l-t>1?e(t,l,i,o,a,s):s==n.LEAST_UPPER_BOUND?l:t<0?-1:t}(-1,t.length,e,t,r,i||n.GREATEST_LOWER_BOUND);if(o<0)return-1;for(;o-1>=0&&0===r(t[o],t[o-1],!0);)--o;return o}},{}],202:[function(e,t,n){var r=e("./util");function i(){this._array=[],this._sorted=!0,this._last={generatedLine:-1,generatedColumn:0}}i.prototype.unsortedForEach=function(e,t){this._array.forEach(e,t)},i.prototype.add=function(e){var t,n,i,o,a,s;t=this._last,n=e,i=t.generatedLine,o=n.generatedLine,a=t.generatedColumn,s=n.generatedColumn,o>i||o==i&&s>=a||r.compareByGeneratedPositionsInflated(t,n)<=0?(this._last=e,this._array.push(e)):(this._sorted=!1,this._array.push(e))},i.prototype.toArray=function(){return this._sorted||(this._array.sort(r.compareByGeneratedPositionsInflated),this._sorted=!0),this._array},n.MappingList=i},{"./util":207}],203:[function(e,t,n){function r(e,t,n){var r=e[t];e[t]=e[n],e[n]=r}function i(e,t,n,o){if(n<o){var a=n-1;r(e,(c=n,d=o,Math.round(c+Math.random()*(d-c))),o);for(var s=e[o],l=n;l<o;l++)t(e[l],s)<=0&&r(e,a+=1,l);r(e,a+1,l);var u=a+1;i(e,t,n,u-1),i(e,t,u+1,o)}var c,d}n.quickSort=function(e,t){i(e,t,0,e.length-1)}},{}],204:[function(e,t,n){var r=e("./util"),i=e("./binary-search"),o=e("./array-set").ArraySet,a=e("./base64-vlq"),s=e("./quick-sort").quickSort;function l(e,t){var n=e;return"string"==typeof e&&(n=r.parseSourceMapInput(e)),null!=n.sections?new d(n,t):new u(n,t)}function u(e,t){var n=e;"string"==typeof e&&(n=r.parseSourceMapInput(e));var i=r.getArg(n,"version"),a=r.getArg(n,"sources"),s=r.getArg(n,"names",[]),l=r.getArg(n,"sourceRoot",null),u=r.getArg(n,"sourcesContent",null),c=r.getArg(n,"mappings"),d=r.getArg(n,"file",null);if(i!=this._version)throw new Error("Unsupported version: "+i);l&&(l=r.normalize(l)),a=a.map(String).map(r.normalize).map((function(e){return l&&r.isAbsolute(l)&&r.isAbsolute(e)?r.relative(l,e):e})),this._names=o.fromArray(s.map(String),!0),this._sources=o.fromArray(a,!0),this._absoluteSources=this._sources.toArray().map((function(e){return r.computeSourceURL(l,e,t)})),this.sourceRoot=l,this.sourcesContent=u,this._mappings=c,this._sourceMapURL=t,this.file=d}function c(){this.generatedLine=0,this.generatedColumn=0,this.source=null,this.originalLine=null,this.originalColumn=null,this.name=null}function d(e,t){var n=e;"string"==typeof e&&(n=r.parseSourceMapInput(e));var i=r.getArg(n,"version"),a=r.getArg(n,"sections");if(i!=this._version)throw new Error("Unsupported version: "+i);this._sources=new o,this._names=new o;var s={line:-1,column:0};this._sections=a.map((function(e){if(e.url)throw new Error("Support for url field in sections not implemented.");var n=r.getArg(e,"offset"),i=r.getArg(n,"line"),o=r.getArg(n,"column");if(i<s.line||i===s.line&&o<s.column)throw new Error("Section offsets must be ordered and non-overlapping.");return s=n,{generatedOffset:{generatedLine:i+1,generatedColumn:o+1},consumer:new l(r.getArg(e,"map"),t)}}))}l.fromSourceMap=function(e,t){return u.fromSourceMap(e,t)},l.prototype._version=3,l.prototype.__generatedMappings=null,Object.defineProperty(l.prototype,"_generatedMappings",{configurable:!0,enumerable:!0,get:function(){return this.__generatedMappings||this._parseMappings(this._mappings,this.sourceRoot),this.__generatedMappings}}),l.prototype.__originalMappings=null,Object.defineProperty(l.prototype,"_originalMappings",{configurable:!0,enumerable:!0,get:function(){return this.__originalMappings||this._parseMappings(this._mappings,this.sourceRoot),this.__originalMappings}}),l.prototype._charIsMappingSeparator=function(e,t){var n=e.charAt(t);return";"===n||","===n},l.prototype._parseMappings=function(e,t){throw new Error("Subclasses must implement _parseMappings")},l.GENERATED_ORDER=1,l.ORIGINAL_ORDER=2,l.GREATEST_LOWER_BOUND=1,l.LEAST_UPPER_BOUND=2,l.prototype.eachMapping=function(e,t,n){var i,o=t||null;switch(n||l.GENERATED_ORDER){case l.GENERATED_ORDER:i=this._generatedMappings;break;case l.ORIGINAL_ORDER:i=this._originalMappings;break;default:throw new Error("Unknown order of iteration.")}var a=this.sourceRoot;i.map((function(e){var t=null===e.source?null:this._sources.at(e.source);return{source:t=r.computeSourceURL(a,t,this._sourceMapURL),generatedLine:e.generatedLine,generatedColumn:e.generatedColumn,originalLine:e.originalLine,originalColumn:e.originalColumn,name:null===e.name?null:this._names.at(e.name)}}),this).forEach(e,o)},l.prototype.allGeneratedPositionsFor=function(e){var t=r.getArg(e,"line"),n={source:r.getArg(e,"source"),originalLine:t,originalColumn:r.getArg(e,"column",0)};if(n.source=this._findSourceIndex(n.source),n.source<0)return[];var o=[],a=this._findMapping(n,this._originalMappings,"originalLine","originalColumn",r.compareByOriginalPositions,i.LEAST_UPPER_BOUND);if(a>=0){var s=this._originalMappings[a];if(void 0===e.column)for(var l=s.originalLine;s&&s.originalLine===l;)o.push({line:r.getArg(s,"generatedLine",null),column:r.getArg(s,"generatedColumn",null),lastColumn:r.getArg(s,"lastGeneratedColumn",null)}),s=this._originalMappings[++a];else for(var u=s.originalColumn;s&&s.originalLine===t&&s.originalColumn==u;)o.push({line:r.getArg(s,"generatedLine",null),column:r.getArg(s,"generatedColumn",null),lastColumn:r.getArg(s,"lastGeneratedColumn",null)}),s=this._originalMappings[++a]}return o},n.SourceMapConsumer=l,u.prototype=Object.create(l.prototype),u.prototype.consumer=l,u.prototype._findSourceIndex=function(e){var t,n=e;if(null!=this.sourceRoot&&(n=r.relative(this.sourceRoot,n)),this._sources.has(n))return this._sources.indexOf(n);for(t=0;t<this._absoluteSources.length;++t)if(this._absoluteSources[t]==e)return t;return-1},u.fromSourceMap=function(e,t){var n=Object.create(u.prototype),i=n._names=o.fromArray(e._names.toArray(),!0),a=n._sources=o.fromArray(e._sources.toArray(),!0);n.sourceRoot=e._sourceRoot,n.sourcesContent=e._generateSourcesContent(n._sources.toArray(),n.sourceRoot),n.file=e._file,n._sourceMapURL=t,n._absoluteSources=n._sources.toArray().map((function(e){return r.computeSourceURL(n.sourceRoot,e,t)}));for(var l=e._mappings.toArray().slice(),d=n.__generatedMappings=[],h=n.__originalMappings=[],f=0,p=l.length;f<p;f++){var g=l[f],v=new c;v.generatedLine=g.generatedLine,v.generatedColumn=g.generatedColumn,g.source&&(v.source=a.indexOf(g.source),v.originalLine=g.originalLine,v.originalColumn=g.originalColumn,g.name&&(v.name=i.indexOf(g.name)),h.push(v)),d.push(v)}return s(n.__originalMappings,r.compareByOriginalPositions),n},u.prototype._version=3,Object.defineProperty(u.prototype,"sources",{get:function(){return this._absoluteSources.slice()}}),u.prototype._parseMappings=function(e,t){for(var n,i,o,l,u,d=1,h=0,f=0,p=0,g=0,v=0,m=e.length,y=0,b={},C={},A=[],_=[];y<m;)if(";"===e.charAt(y))d++,y++,h=0;else if(","===e.charAt(y))y++;else{for((n=new c).generatedLine=d,l=y;l<m&&!this._charIsMappingSeparator(e,l);l++);if(o=b[i=e.slice(y,l)])y+=i.length;else{for(o=[];y<l;)a.decode(e,y,C),u=C.value,y=C.rest,o.push(u);if(2===o.length)throw new Error("Found a source, but no line and column");if(3===o.length)throw new Error("Found a source and line, but no column");b[i]=o}n.generatedColumn=h+o[0],h=n.generatedColumn,o.length>1&&(n.source=g+o[1],g+=o[1],n.originalLine=f+o[2],f=n.originalLine,n.originalLine+=1,n.originalColumn=p+o[3],p=n.originalColumn,o.length>4&&(n.name=v+o[4],v+=o[4])),_.push(n),"number"==typeof n.originalLine&&A.push(n)}s(_,r.compareByGeneratedPositionsDeflated),this.__generatedMappings=_,s(A,r.compareByOriginalPositions),this.__originalMappings=A},u.prototype._findMapping=function(e,t,n,r,o,a){if(e[n]<=0)throw new TypeError("Line must be greater than or equal to 1, got "+e[n]);if(e[r]<0)throw new TypeError("Column must be greater than or equal to 0, got "+e[r]);return i.search(e,t,o,a)},u.prototype.computeColumnSpans=function(){for(var e=0;e<this._generatedMappings.length;++e){var t=this._generatedMappings[e];if(e+1<this._generatedMappings.length){var n=this._generatedMappings[e+1];if(t.generatedLine===n.generatedLine){t.lastGeneratedColumn=n.generatedColumn-1;continue}}t.lastGeneratedColumn=1/0}},u.prototype.originalPositionFor=function(e){var t={generatedLine:r.getArg(e,"line"),generatedColumn:r.getArg(e,"column")},n=this._findMapping(t,this._generatedMappings,"generatedLine","generatedColumn",r.compareByGeneratedPositionsDeflated,r.getArg(e,"bias",l.GREATEST_LOWER_BOUND));if(n>=0){var i=this._generatedMappings[n];if(i.generatedLine===t.generatedLine){var o=r.getArg(i,"source",null);null!==o&&(o=this._sources.at(o),o=r.computeSourceURL(this.sourceRoot,o,this._sourceMapURL));var a=r.getArg(i,"name",null);return null!==a&&(a=this._names.at(a)),{source:o,line:r.getArg(i,"originalLine",null),column:r.getArg(i,"originalColumn",null),name:a}}}return{source:null,line:null,column:null,name:null}},u.prototype.hasContentsOfAllSources=function(){return!!this.sourcesContent&&this.sourcesContent.length>=this._sources.size()&&!this.sourcesContent.some((function(e){return null==e}))},u.prototype.sourceContentFor=function(e,t){if(!this.sourcesContent)return null;var n=this._findSourceIndex(e);if(n>=0)return this.sourcesContent[n];var i,o=e;if(null!=this.sourceRoot&&(o=r.relative(this.sourceRoot,o)),null!=this.sourceRoot&&(i=r.urlParse(this.sourceRoot))){var a=o.replace(/^file:\/\//,"");if("file"==i.scheme&&this._sources.has(a))return this.sourcesContent[this._sources.indexOf(a)];if((!i.path||"/"==i.path)&&this._sources.has("/"+o))return this.sourcesContent[this._sources.indexOf("/"+o)]}if(t)return null;throw new Error('"'+o+'" is not in the SourceMap.')},u.prototype.generatedPositionFor=function(e){var t=r.getArg(e,"source");if((t=this._findSourceIndex(t))<0)return{line:null,column:null,lastColumn:null};var n={source:t,originalLine:r.getArg(e,"line"),originalColumn:r.getArg(e,"column")},i=this._findMapping(n,this._originalMappings,"originalLine","originalColumn",r.compareByOriginalPositions,r.getArg(e,"bias",l.GREATEST_LOWER_BOUND));if(i>=0){var o=this._originalMappings[i];if(o.source===n.source)return{line:r.getArg(o,"generatedLine",null),column:r.getArg(o,"generatedColumn",null),lastColumn:r.getArg(o,"lastGeneratedColumn",null)}}return{line:null,column:null,lastColumn:null}},n.BasicSourceMapConsumer=u,d.prototype=Object.create(l.prototype),d.prototype.constructor=l,d.prototype._version=3,Object.defineProperty(d.prototype,"sources",{get:function(){for(var e=[],t=0;t<this._sections.length;t++)for(var n=0;n<this._sections[t].consumer.sources.length;n++)e.push(this._sections[t].consumer.sources[n]);return e}}),d.prototype.originalPositionFor=function(e){var t={generatedLine:r.getArg(e,"line"),generatedColumn:r.getArg(e,"column")},n=i.search(t,this._sections,(function(e,t){var n=e.generatedLine-t.generatedOffset.generatedLine;return n||e.generatedColumn-t.generatedOffset.generatedColumn})),o=this._sections[n];return o?o.consumer.originalPositionFor({line:t.generatedLine-(o.generatedOffset.generatedLine-1),column:t.generatedColumn-(o.generatedOffset.generatedLine===t.generatedLine?o.generatedOffset.generatedColumn-1:0),bias:e.bias}):{source:null,line:null,column:null,name:null}},d.prototype.hasContentsOfAllSources=function(){return this._sections.every((function(e){return e.consumer.hasContentsOfAllSources()}))},d.prototype.sourceContentFor=function(e,t){for(var n=0;n<this._sections.length;n++){var r=this._sections[n].consumer.sourceContentFor(e,!0);if(r)return r}if(t)return null;throw new Error('"'+e+'" is not in the SourceMap.')},d.prototype.generatedPositionFor=function(e){for(var t=0;t<this._sections.length;t++){var n=this._sections[t];if(-1!==n.consumer._findSourceIndex(r.getArg(e,"source"))){var i=n.consumer.generatedPositionFor(e);if(i)return{line:i.line+(n.generatedOffset.generatedLine-1),column:i.column+(n.generatedOffset.generatedLine===i.line?n.generatedOffset.generatedColumn-1:0)}}}return{line:null,column:null}},d.prototype._parseMappings=function(e,t){this.__generatedMappings=[],this.__originalMappings=[];for(var n=0;n<this._sections.length;n++)for(var i=this._sections[n],o=i.consumer._generatedMappings,a=0;a<o.length;a++){var l=o[a],u=i.consumer._sources.at(l.source);u=r.computeSourceURL(i.consumer.sourceRoot,u,this._sourceMapURL),this._sources.add(u),u=this._sources.indexOf(u);var c=null;l.name&&(c=i.consumer._names.at(l.name),this._names.add(c),c=this._names.indexOf(c));var d={source:u,generatedLine:l.generatedLine+(i.generatedOffset.generatedLine-1),generatedColumn:l.generatedColumn+(i.generatedOffset.generatedLine===l.generatedLine?i.generatedOffset.generatedColumn-1:0),originalLine:l.originalLine,originalColumn:l.originalColumn,name:c};this.__generatedMappings.push(d),"number"==typeof d.originalLine&&this.__originalMappings.push(d)}s(this.__generatedMappings,r.compareByGeneratedPositionsDeflated),s(this.__originalMappings,r.compareByOriginalPositions)},n.IndexedSourceMapConsumer=d},{"./array-set":198,"./base64-vlq":199,"./binary-search":201,"./quick-sort":203,"./util":207}],205:[function(e,t,n){var r=e("./base64-vlq"),i=e("./util"),o=e("./array-set").ArraySet,a=e("./mapping-list").MappingList;function s(e){e||(e={}),this._file=i.getArg(e,"file",null),this._sourceRoot=i.getArg(e,"sourceRoot",null),this._skipValidation=i.getArg(e,"skipValidation",!1),this._sources=new o,this._names=new o,this._mappings=new a,this._sourcesContents=null}s.prototype._version=3,s.fromSourceMap=function(e){var t=e.sourceRoot,n=new s({file:e.file,sourceRoot:t});return e.eachMapping((function(e){var r={generated:{line:e.generatedLine,column:e.generatedColumn}};null!=e.source&&(r.source=e.source,null!=t&&(r.source=i.relative(t,r.source)),r.original={line:e.originalLine,column:e.originalColumn},null!=e.name&&(r.name=e.name)),n.addMapping(r)})),e.sources.forEach((function(r){var o=r;null!==t&&(o=i.relative(t,r)),n._sources.has(o)||n._sources.add(o);var a=e.sourceContentFor(r);null!=a&&n.setSourceContent(r,a)})),n},s.prototype.addMapping=function(e){var t=i.getArg(e,"generated"),n=i.getArg(e,"original",null),r=i.getArg(e,"source",null),o=i.getArg(e,"name",null);this._skipValidation||this._validateMapping(t,n,r,o),null!=r&&(r=String(r),this._sources.has(r)||this._sources.add(r)),null!=o&&(o=String(o),this._names.has(o)||this._names.add(o)),this._mappings.add({generatedLine:t.line,generatedColumn:t.column,originalLine:null!=n&&n.line,originalColumn:null!=n&&n.column,source:r,name:o})},s.prototype.setSourceContent=function(e,t){var n=e;null!=this._sourceRoot&&(n=i.relative(this._sourceRoot,n)),null!=t?(this._sourcesContents||(this._sourcesContents=Object.create(null)),this._sourcesContents[i.toSetString(n)]=t):this._sourcesContents&&(delete this._sourcesContents[i.toSetString(n)],0===Object.keys(this._sourcesContents).length&&(this._sourcesContents=null))},s.prototype.applySourceMap=function(e,t,n){var r=t;if(null==t){if(null==e.file)throw new Error('SourceMapGenerator.prototype.applySourceMap requires either an explicit source file, or the source map\'s "file" property. Both were omitted.');r=e.file}var a=this._sourceRoot;null!=a&&(r=i.relative(a,r));var s=new o,l=new o;this._mappings.unsortedForEach((function(t){if(t.source===r&&null!=t.originalLine){var o=e.originalPositionFor({line:t.originalLine,column:t.originalColumn});null!=o.source&&(t.source=o.source,null!=n&&(t.source=i.join(n,t.source)),null!=a&&(t.source=i.relative(a,t.source)),t.originalLine=o.line,t.originalColumn=o.column,null!=o.name&&(t.name=o.name))}var u=t.source;null==u||s.has(u)||s.add(u);var c=t.name;null==c||l.has(c)||l.add(c)}),this),this._sources=s,this._names=l,e.sources.forEach((function(t){var r=e.sourceContentFor(t);null!=r&&(null!=n&&(t=i.join(n,t)),null!=a&&(t=i.relative(a,t)),this.setSourceContent(t,r))}),this)},s.prototype._validateMapping=function(e,t,n,r){if(t&&"number"!=typeof t.line&&"number"!=typeof t.column)throw new Error("original.line and original.column are not numbers -- you probably meant to omit the original mapping entirely and only map the generated position. If so, pass null for the original mapping instead of an object with empty or null values.");if((!(e&&"line"in e&&"column"in e&&e.line>0&&e.column>=0)||t||n||r)&&!(e&&"line"in e&&"column"in e&&t&&"line"in t&&"column"in t&&e.line>0&&e.column>=0&&t.line>0&&t.column>=0&&n))throw new Error("Invalid mapping: "+JSON.stringify({generated:e,source:n,original:t,name:r}))},s.prototype._serializeMappings=function(){for(var e,t,n,o,a=0,s=1,l=0,u=0,c=0,d=0,h="",f=this._mappings.toArray(),p=0,g=f.length;p<g;p++){if(e="",(t=f[p]).generatedLine!==s)for(a=0;t.generatedLine!==s;)e+=";",s++;else if(p>0){if(!i.compareByGeneratedPositionsInflated(t,f[p-1]))continue;e+=","}e+=r.encode(t.generatedColumn-a),a=t.generatedColumn,null!=t.source&&(o=this._sources.indexOf(t.source),e+=r.encode(o-d),d=o,e+=r.encode(t.originalLine-1-u),u=t.originalLine-1,e+=r.encode(t.originalColumn-l),l=t.originalColumn,null!=t.name&&(n=this._names.indexOf(t.name),e+=r.encode(n-c),c=n)),h+=e}return h},s.prototype._generateSourcesContent=function(e,t){return e.map((function(e){if(!this._sourcesContents)return null;null!=t&&(e=i.relative(t,e));var n=i.toSetString(e);return Object.prototype.hasOwnProperty.call(this._sourcesContents,n)?this._sourcesContents[n]:null}),this)},s.prototype.toJSON=function(){var e={version:this._version,sources:this._sources.toArray(),names:this._names.toArray(),mappings:this._serializeMappings()};return null!=this._file&&(e.file=this._file),null!=this._sourceRoot&&(e.sourceRoot=this._sourceRoot),this._sourcesContents&&(e.sourcesContent=this._generateSourcesContent(e.sources,e.sourceRoot)),e},s.prototype.toString=function(){return JSON.stringify(this.toJSON())},n.SourceMapGenerator=s},{"./array-set":198,"./base64-vlq":199,"./mapping-list":202,"./util":207}],206:[function(e,t,n){var r=e("./source-map-generator").SourceMapGenerator,i=e("./util"),o=/(\r?\n)/,a="$$$isSourceNode$$$";function s(e,t,n,r,i){this.children=[],this.sourceContents={},this.line=null==e?null:e,this.column=null==t?null:t,this.source=null==n?null:n,this.name=null==i?null:i,this[a]=!0,null!=r&&this.add(r)}s.fromStringWithSourceMap=function(e,t,n){var r=new s,a=e.split(o),l=0,u=function(){return e()+(e()||"");function e(){return l<a.length?a[l++]:void 0}},c=1,d=0,h=null;return t.eachMapping((function(e){if(null!==h){if(!(c<e.generatedLine)){var t=(n=a[l]||"").substr(0,e.generatedColumn-d);return a[l]=n.substr(e.generatedColumn-d),d=e.generatedColumn,f(h,t),void(h=e)}f(h,u()),c++,d=0}for(;c<e.generatedLine;)r.add(u()),c++;if(d<e.generatedColumn){var n=a[l]||"";r.add(n.substr(0,e.generatedColumn)),a[l]=n.substr(e.generatedColumn),d=e.generatedColumn}h=e}),this),l<a.length&&(h&&f(h,u()),r.add(a.splice(l).join(""))),t.sources.forEach((function(e){var o=t.sourceContentFor(e);null!=o&&(null!=n&&(e=i.join(n,e)),r.setSourceContent(e,o))})),r;function f(e,t){if(null===e||void 0===e.source)r.add(t);else{var o=n?i.join(n,e.source):e.source;r.add(new s(e.originalLine,e.originalColumn,o,t,e.name))}}},s.prototype.add=function(e){if(Array.isArray(e))e.forEach((function(e){this.add(e)}),this);else{if(!e[a]&&"string"!=typeof e)throw new TypeError("Expected a SourceNode, string, or an array of SourceNodes and strings. Got "+e);e&&this.children.push(e)}return this},s.prototype.prepend=function(e){if(Array.isArray(e))for(var t=e.length-1;t>=0;t--)this.prepend(e[t]);else{if(!e[a]&&"string"!=typeof e)throw new TypeError("Expected a SourceNode, string, or an array of SourceNodes and strings. Got "+e);this.children.unshift(e)}return this},s.prototype.walk=function(e){for(var t,n=0,r=this.children.length;n<r;n++)(t=this.children[n])[a]?t.walk(e):""!==t&&e(t,{source:this.source,line:this.line,column:this.column,name:this.name})},s.prototype.join=function(e){var t,n,r=this.children.length;if(r>0){for(t=[],n=0;n<r-1;n++)t.push(this.children[n]),t.push(e);t.push(this.children[n]),this.children=t}return this},s.prototype.replaceRight=function(e,t){var n=this.children[this.children.length-1];return n[a]?n.replaceRight(e,t):"string"==typeof n?this.children[this.children.length-1]=n.replace(e,t):this.children.push("".replace(e,t)),this},s.prototype.setSourceContent=function(e,t){this.sourceContents[i.toSetString(e)]=t},s.prototype.walkSourceContents=function(e){for(var t=0,n=this.children.length;t<n;t++)this.children[t][a]&&this.children[t].walkSourceContents(e);var r=Object.keys(this.sourceContents);for(t=0,n=r.length;t<n;t++)e(i.fromSetString(r[t]),this.sourceContents[r[t]])},s.prototype.toString=function(){var e="";return this.walk((function(t){e+=t})),e},s.prototype.toStringWithSourceMap=function(e){var t={code:"",line:1,column:0},n=new r(e),i=!1,o=null,a=null,s=null,l=null;return this.walk((function(e,r){t.code+=e,null!==r.source&&null!==r.line&&null!==r.column?(o===r.source&&a===r.line&&s===r.column&&l===r.name||n.addMapping({source:r.source,original:{line:r.line,column:r.column},generated:{line:t.line,column:t.column},name:r.name}),o=r.source,a=r.line,s=r.column,l=r.name,i=!0):i&&(n.addMapping({generated:{line:t.line,column:t.column}}),o=null,i=!1);for(var u=0,c=e.length;u<c;u++)10===e.charCodeAt(u)?(t.line++,t.column=0,u+1===c?(o=null,i=!1):i&&n.addMapping({source:r.source,original:{line:r.line,column:r.column},generated:{line:t.line,column:t.column},name:r.name})):t.column++})),this.walkSourceContents((function(e,t){n.setSourceContent(e,t)})),{code:t.code,map:n}},n.SourceNode=s},{"./source-map-generator":205,"./util":207}],207:[function(e,t,n){n.getArg=function(e,t,n){if(t in e)return e[t];if(3===arguments.length)return n;throw new Error('"'+t+'" is a required argument.')};var r=/^(?:([\w+\-.]+):)?\/\/(?:(\w+:\w+)@)?([\w.-]*)(?::(\d+))?(.*)$/,i=/^data:.+\,.+$/;function o(e){var t=e.match(r);return t?{scheme:t[1],auth:t[2],host:t[3],port:t[4],path:t[5]}:null}function a(e){var t="";return e.scheme&&(t+=e.scheme+":"),t+="//",e.auth&&(t+=e.auth+"@"),e.host&&(t+=e.host),e.port&&(t+=":"+e.port),e.path&&(t+=e.path),t}function s(e){var t=e,r=o(e);if(r){if(!r.path)return e;t=r.path}for(var i,s=n.isAbsolute(t),l=t.split(/\/+/),u=0,c=l.length-1;c>=0;c--)"."===(i=l[c])?l.splice(c,1):".."===i?u++:u>0&&(""===i?(l.splice(c+1,u),u=0):(l.splice(c,2),u--));return""===(t=l.join("/"))&&(t=s?"/":"."),r?(r.path=t,a(r)):t}function l(e,t){""===e&&(e="."),""===t&&(t=".");var n=o(t),r=o(e);if(r&&(e=r.path||"/"),n&&!n.scheme)return r&&(n.scheme=r.scheme),a(n);if(n||t.match(i))return t;if(r&&!r.host&&!r.path)return r.host=t,a(r);var l="/"===t.charAt(0)?t:s(e.replace(/\/+$/,"")+"/"+t);return r?(r.path=l,a(r)):l}n.urlParse=o,n.urlGenerate=a,n.normalize=s,n.join=l,n.isAbsolute=function(e){return"/"===e.charAt(0)||r.test(e)},n.relative=function(e,t){""===e&&(e="."),e=e.replace(/\/$/,"");for(var n=0;0!==t.indexOf(e+"/");){var r=e.lastIndexOf("/");if(r<0)return t;if((e=e.slice(0,r)).match(/^([^\/]+:\/)?\/*$/))return t;++n}return Array(n+1).join("../")+t.substr(e.length+1)};var u=!("__proto__"in Object.create(null));function c(e){return e}function d(e){if(!e)return!1;var t=e.length;if(t<9)return!1;if(95!==e.charCodeAt(t-1)||95!==e.charCodeAt(t-2)||111!==e.charCodeAt(t-3)||116!==e.charCodeAt(t-4)||111!==e.charCodeAt(t-5)||114!==e.charCodeAt(t-6)||112!==e.charCodeAt(t-7)||95!==e.charCodeAt(t-8)||95!==e.charCodeAt(t-9))return!1;for(var n=t-10;n>=0;n--)if(36!==e.charCodeAt(n))return!1;return!0}function h(e,t){return e===t?0:null===e?1:null===t?-1:e>t?1:-1}n.toSetString=u?c:function(e){return d(e)?"$"+e:e},n.fromSetString=u?c:function(e){return d(e)?e.slice(1):e},n.compareByOriginalPositions=function(e,t,n){var r=h(e.source,t.source);return 0!==r||0!=(r=e.originalLine-t.originalLine)||0!=(r=e.originalColumn-t.originalColumn)||n||0!=(r=e.generatedColumn-t.generatedColumn)||0!=(r=e.generatedLine-t.generatedLine)?r:h(e.name,t.name)},n.compareByGeneratedPositionsDeflated=function(e,t,n){var r=e.generatedLine-t.generatedLine;return 0!==r||0!=(r=e.generatedColumn-t.generatedColumn)||n||0!==(r=h(e.source,t.source))||0!=(r=e.originalLine-t.originalLine)||0!=(r=e.originalColumn-t.originalColumn)?r:h(e.name,t.name)},n.compareByGeneratedPositionsInflated=function(e,t){var n=e.generatedLine-t.generatedLine;return 0!==n||0!=(n=e.generatedColumn-t.generatedColumn)||0!==(n=h(e.source,t.source))||0!=(n=e.originalLine-t.originalLine)||0!=(n=e.originalColumn-t.originalColumn)?n:h(e.name,t.name)},n.parseSourceMapInput=function(e){return JSON.parse(e.replace(/^\)]}'[^\n]*\n/,""))},n.computeSourceURL=function(e,t,n){if(t=t||"",e&&("/"!==e[e.length-1]&&"/"!==t[0]&&(e+="/"),t=e+t),n){var r=o(n);if(!r)throw new Error("sourceMapURL could not be parsed");if(r.path){var i=r.path.lastIndexOf("/");i>=0&&(r.path=r.path.substring(0,i+1))}t=l(a(r),t)}return s(t)}},{}],208:[function(e,t,n){n.SourceMapGenerator=e("./lib/source-map-generator").SourceMapGenerator,n.SourceMapConsumer=e("./lib/source-map-consumer").SourceMapConsumer,n.SourceNode=e("./lib/source-node").SourceNode},{"./lib/source-map-consumer":204,"./lib/source-map-generator":205,"./lib/source-node":206}],209:[function(e,t,n){var r=e("punycode"),i=e("./util");function o(){this.protocol=null,this.slashes=null,this.auth=null,this.host=null,this.port=null,this.hostname=null,this.hash=null,this.search=null,this.query=null,this.pathname=null,this.path=null,this.href=null}n.parse=C,n.resolve=function(e,t){return C(e,!1,!0).resolve(t)},n.resolveObject=function(e,t){return e?C(e,!1,!0).resolveObject(t):t},n.format=function(e){return i.isString(e)&&(e=C(e)),e instanceof o?e.format():o.prototype.format.call(e)},n.Url=o;var a=/^([a-z0-9.+-]+:)/i,s=/:[0-9]*$/,l=/^(\/\/?(?!\/)[^\?\s]*)(\?[^\s]*)?$/,c=["{","}","|","\\","^","`"].concat(["<",">",'"',"`"," ","\r","\n","\t"]),d=["'"].concat(c),h=["%","/","?",";","#"].concat(d),f=["/","?","#"],p=/^[+a-z0-9A-Z_-]{0,63}$/,g=/^([+a-z0-9A-Z_-]{0,63})(.*)$/,v={javascript:!0,"javascript:":!0},m={javascript:!0,"javascript:":!0},y={http:!0,https:!0,ftp:!0,gopher:!0,file:!0,"http:":!0,"https:":!0,"ftp:":!0,"gopher:":!0,"file:":!0},b=e("querystring");function C(e,t,n){if(e&&i.isObject(e)&&e instanceof o)return e;var r=new o;return r.parse(e,t,n),r}o.prototype.parse=function(e,t,n){if(!i.isString(e))throw new TypeError("Parameter 'url' must be a string, not "+u(e));var o=e.indexOf("?"),s=-1!==o&&o<e.indexOf("#")?"?":"#",c=e.split(s);c[0]=c[0].replace(/\\/g,"/");var C=e=c.join(s);if(C=C.trim(),!n&&1===e.split("#").length){var A=l.exec(C);if(A)return this.path=C,this.href=C,this.pathname=A[1],A[2]?(this.search=A[2],this.query=t?b.parse(this.search.substr(1)):this.search.substr(1)):t&&(this.search="",this.query={}),this}var _=a.exec(C);if(_){var I=(_=_[0]).toLowerCase();this.protocol=I,C=C.substr(_.length)}if(n||_||C.match(/^\/\/[^@\/]+@[^@\/]+/)){var x="//"===C.substr(0,2);!x||_&&m[_]||(C=C.substr(2),this.slashes=!0)}if(!m[_]&&(x||_&&!y[_])){for(var w,M,S=-1,T=0;T<f.length;T++)-1!==(E=C.indexOf(f[T]))&&(-1===S||E<S)&&(S=E);for(-1!==(M=-1===S?C.lastIndexOf("@"):C.lastIndexOf("@",S))&&(w=C.slice(0,M),C=C.slice(M+1),this.auth=decodeURIComponent(w)),S=-1,T=0;T<h.length;T++){var E;-1!==(E=C.indexOf(h[T]))&&(-1===S||E<S)&&(S=E)}-1===S&&(S=C.length),this.host=C.slice(0,S),C=C.slice(S),this.parseHost(),this.hostname=this.hostname||"";var O="["===this.hostname[0]&&"]"===this.hostname[this.hostname.length-1];if(!O)for(var k=this.hostname.split(/\./),L=(T=0,k.length);T<L;T++){var D=k[T];if(D&&!D.match(p)){for(var P="",N=0,j=D.length;N<j;N++)D.charCodeAt(N)>127?P+="x":P+=D[N];if(!P.match(p)){var R=k.slice(0,T),z=k.slice(T+1),F=D.match(g);F&&(R.push(F[1]),z.unshift(F[2])),z.length&&(C="/"+z.join(".")+C),this.hostname=R.join(".");break}}}this.hostname.length>255?this.hostname="":this.hostname=this.hostname.toLowerCase(),O||(this.hostname=r.toASCII(this.hostname));var B=this.port?":"+this.port:"",H=this.hostname||"";this.host=H+B,this.href+=this.host,O&&(this.hostname=this.hostname.substr(1,this.hostname.length-2),"/"!==C[0]&&(C="/"+C))}if(!v[I])for(T=0,L=d.length;T<L;T++){var U=d[T];if(-1!==C.indexOf(U)){var V=encodeURIComponent(U);V===U&&(V=escape(U)),C=C.split(U).join(V)}}var Y=C.indexOf("#");-1!==Y&&(this.hash=C.substr(Y),C=C.slice(0,Y));var W=C.indexOf("?");if(-1!==W?(this.search=C.substr(W),this.query=C.substr(W+1),t&&(this.query=b.parse(this.query)),C=C.slice(0,W)):t&&(this.search="",this.query={}),C&&(this.pathname=C),y[I]&&this.hostname&&!this.pathname&&(this.pathname="/"),this.pathname||this.search){B=this.pathname||"";var G=this.search||"";this.path=B+G}return this.href=this.format(),this},o.prototype.format=function(){var e=this.auth||"";e&&(e=(e=encodeURIComponent(e)).replace(/%3A/i,":"),e+="@");var t=this.protocol||"",n=this.pathname||"",r=this.hash||"",o=!1,a="";this.host?o=e+this.host:this.hostname&&(o=e+(-1===this.hostname.indexOf(":")?this.hostname:"["+this.hostname+"]"),this.port&&(o+=":"+this.port)),this.query&&i.isObject(this.query)&&Object.keys(this.query).length&&(a=b.stringify(this.query));var s=this.search||a&&"?"+a||"";return t&&":"!==t.substr(-1)&&(t+=":"),this.slashes||(!t||y[t])&&!1!==o?(o="//"+(o||""),n&&"/"!==n.charAt(0)&&(n="/"+n)):o||(o=""),r&&"#"!==r.charAt(0)&&(r="#"+r),s&&"?"!==s.charAt(0)&&(s="?"+s),t+o+(n=n.replace(/[?#]/g,(function(e){return encodeURIComponent(e)})))+(s=s.replace("#","%23"))+r},o.prototype.resolve=function(e){return this.resolveObject(C(e,!1,!0)).format()},o.prototype.resolveObject=function(e){if(i.isString(e)){var t=new o;t.parse(e,!1,!0),e=t}for(var n=new o,r=Object.keys(this),a=0;a<r.length;a++){var s=r[a];n[s]=this[s]}if(n.hash=e.hash,""===e.href)return n.href=n.format(),n;if(e.slashes&&!e.protocol){for(var l=Object.keys(e),u=0;u<l.length;u++){var c=l[u];"protocol"!==c&&(n[c]=e[c])}return y[n.protocol]&&n.hostname&&!n.pathname&&(n.path=n.pathname="/"),n.href=n.format(),n}if(e.protocol&&e.protocol!==n.protocol){if(!y[e.protocol]){for(var d=Object.keys(e),h=0;h<d.length;h++){var f=d[h];n[f]=e[f]}return n.href=n.format(),n}if(n.protocol=e.protocol,e.host||m[e.protocol])n.pathname=e.pathname;else{for(var p=(e.pathname||"").split("/");p.length&&!(e.host=p.shift()););e.host||(e.host=""),e.hostname||(e.hostname=""),""!==p[0]&&p.unshift(""),p.length<2&&p.unshift(""),n.pathname=p.join("/")}if(n.search=e.search,n.query=e.query,n.host=e.host||"",n.auth=e.auth,n.hostname=e.hostname||e.host,n.port=e.port,n.pathname||n.search){var g=n.pathname||"",v=n.search||"";n.path=g+v}return n.slashes=n.slashes||e.slashes,n.href=n.format(),n}var b=n.pathname&&"/"===n.pathname.charAt(0),C=e.host||e.pathname&&"/"===e.pathname.charAt(0),A=C||b||n.host&&e.pathname,_=A,I=n.pathname&&n.pathname.split("/")||[],x=(p=e.pathname&&e.pathname.split("/")||[],n.protocol&&!y[n.protocol]);if(x&&(n.hostname="",n.port=null,n.host&&(""===I[0]?I[0]=n.host:I.unshift(n.host)),n.host="",e.protocol&&(e.hostname=null,e.port=null,e.host&&(""===p[0]?p[0]=e.host:p.unshift(e.host)),e.host=null),A=A&&(""===p[0]||""===I[0])),C)n.host=e.host||""===e.host?e.host:n.host,n.hostname=e.hostname||""===e.hostname?e.hostname:n.hostname,n.search=e.search,n.query=e.query,I=p;else if(p.length)I||(I=[]),I.pop(),I=I.concat(p),n.search=e.search,n.query=e.query;else if(!i.isNullOrUndefined(e.search))return x&&(n.hostname=n.host=I.shift(),(E=!!(n.host&&n.host.indexOf("@")>0)&&n.host.split("@"))&&(n.auth=E.shift(),n.host=n.hostname=E.shift())),n.search=e.search,n.query=e.query,i.isNull(n.pathname)&&i.isNull(n.search)||(n.path=(n.pathname?n.pathname:"")+(n.search?n.search:"")),n.href=n.format(),n;if(!I.length)return n.pathname=null,n.search?n.path="/"+n.search:n.path=null,n.href=n.format(),n;for(var w=I.slice(-1)[0],M=(n.host||e.host||I.length>1)&&("."===w||".."===w)||""===w,S=0,T=I.length;T>=0;T--)"."===(w=I[T])?I.splice(T,1):".."===w?(I.splice(T,1),S++):S&&(I.splice(T,1),S--);if(!A&&!_)for(;S--;S)I.unshift("..");!A||""===I[0]||I[0]&&"/"===I[0].charAt(0)||I.unshift(""),M&&"/"!==I.join("/").substr(-1)&&I.push("");var E,O=""===I[0]||I[0]&&"/"===I[0].charAt(0);return x&&(n.hostname=n.host=O?"":I.length?I.shift():"",(E=!!(n.host&&n.host.indexOf("@")>0)&&n.host.split("@"))&&(n.auth=E.shift(),n.host=n.hostname=E.shift())),(A=A||n.host&&I.length)&&!O&&I.unshift(""),I.length?n.pathname=I.join("/"):(n.pathname=null,n.path=null),i.isNull(n.pathname)&&i.isNull(n.search)||(n.path=(n.pathname?n.pathname:"")+(n.search?n.search:"")),n.auth=e.auth||n.auth,n.slashes=n.slashes||e.slashes,n.href=n.format(),n},o.prototype.parseHost=function(){var e=this.host,t=s.exec(e);t&&(":"!==(t=t[0])&&(this.port=t.substr(1)),e=e.substr(0,e.length-t.length)),e&&(this.hostname=e)}},{"./util":210,punycode:194,querystring:197}],210:[function(e,t,n){t.exports={isString:function(e){return"string"==typeof e},isObject:function(e){return"object"===u(e)&&null!==e},isNull:function(e){return null===e},isNullOrUndefined:function(e){return null==e}}},{}],211:[function(e,t,n){var r=e("htmlparser2"),i=e("lodash/escapeRegExp"),o=e("lodash/cloneDeep"),a=e("lodash/mergeWith"),l=e("lodash/isString"),u=e("lodash/isPlainObject"),c=e("parse-srcset"),d=e("postcss"),h=e("url"),f=["img","audio","video","picture","svg","object","map","iframe","embed"],p=["script","style"];function g(e,t){e&&Object.keys(e).forEach((function(n){t(e[n],n)}))}function v(e,t){return{}.hasOwnProperty.call(e,t)}function m(e,t){var n=[];return g(e,(function(e){t(e)&&n.push(e)})),n}t.exports=b;var y=/^[^\0\t\n\f\r /<=>]+$/;function b(e,t,n){var A="",_="";function I(e,t){var n=this;this.tag=e,this.attribs=t||{},this.tagPosition=A.length,this.text="",this.mediaChildren=[],this.updateParentNodeText=function(){O.length&&(O[O.length-1].text+=n.text)},this.updateParentNodeMediaChildren=function(){O.length&&f.indexOf(this.tag)>-1&&O[O.length-1].mediaChildren.push(this.tag)}}t?(t=Object.assign({},b.defaults,t)).parser?t.parser=Object.assign({},C,t.parser):t.parser=C:(t=b.defaults).parser=C,p.forEach((function(e){t.allowedTags&&t.allowedTags.indexOf(e)>-1&&!t.allowVulnerableTags&&console.warn("\n\n⚠️ Your `allowedTags` option includes, `".concat(e,"`, which is inherently\nvulnerable to XSS attacks. Please remove it from `allowedTags`.\nOr, to disable this warning, add the `allowVulnerableTags` option\nand ensure you are accounting for this risk.\n\n"))}));var x,w,M=t.nonTextTags||["script","style","textarea","option"];t.allowedAttributes&&(x={},w={},g(t.allowedAttributes,(function(e,t){x[t]=[];var n=[];e.forEach((function(e){l(e)&&e.indexOf("*")>=0?n.push(i(e).replace(/\\\*/g,".*")):x[t].push(e)})),w[t]=new RegExp("^("+n.join("|")+")$")})));var S={};g(t.allowedClasses,(function(e,t){x&&(v(x,t)||(x[t]=[]),x[t].push("class")),S[t]=e}));var T,E,O,k,L,D,P,N={};g(t.transformTags,(function(e,t){var n;"function"==typeof e?n=e:"string"==typeof e&&(n=b.simpleTransform(e)),"*"===t?T=n:N[t]=n}));var j=!1;z();var R=new r.Parser({onopentag:function(e,n){if(t.enforceHtmlBoundary&&"html"===e&&z(),D)P++;else{var r=new I(e,n);O.push(r);var i,l=!1,f=!!r.text;if(v(N,e)&&(i=N[e](e,n),r.attribs=n=i.attribs,void 0!==i.text&&(r.innerText=i.text),e!==i.tagName&&(r.name=e=i.tagName,L[E]=i.tagName)),T&&(i=T(e,n),r.attribs=n=i.attribs,e!==i.tagName&&(r.name=e=i.tagName,L[E]=i.tagName)),(t.allowedTags&&-1===t.allowedTags.indexOf(e)||"recursiveEscape"===t.disallowedTagsMode&&!function(e){for(var t in e)if(v(e,t))return!1;return!0}(k))&&(l=!0,k[E]=!0,"discard"===t.disallowedTagsMode&&-1!==M.indexOf(e)&&(D=!0,P=1),k[E]=!0),E++,l){if("discard"===t.disallowedTagsMode)return;_=A,A=""}A+="<"+e,(!x||v(x,e)||x["*"])&&g(n,(function(n,i){if(y.test(i)){var l,f=!1;if(!x||v(x,e)&&-1!==x[e].indexOf(i)||x["*"]&&-1!==x["*"].indexOf(i)||v(w,e)&&w[e].test(i)||w["*"]&&w["*"].test(i))f=!0;else if(x&&x[e]){var p,b=s(x[e]);try{for(b.s();!(p=b.n()).done;){var C=p.value;if(u(C)&&C.name&&C.name===i){f=!0;var _="";if(!0===C.multiple){var I,M=s(n.split(" "));try{for(M.s();!(I=M.n()).done;){var T=I.value;-1!==C.values.indexOf(T)&&(""===_?_=T:_+=" "+T)}}catch(e){M.e(e)}finally{M.f()}}else C.values.indexOf(n)>=0&&(_=n);n=_}}}catch(e){b.e(e)}finally{b.f()}}if(f){if(-1!==t.allowedSchemesAppliedToAttributes.indexOf(i)&&B(e,n))return void delete r.attribs[i];if("iframe"===e&&"src"===i){var E=!0;try{if((l=h.parse(n,!1,!0))&&null===l.host&&null===l.protocol)E=v(t,"allowIframeRelativeUrls")?t.allowIframeRelativeUrls:!t.allowedIframeHostnames&&!t.allowedIframeDomains;else if(t.allowedIframeHostnames||t.allowedIframeDomains){var O=(t.allowedIframeHostnames||[]).find((function(e){return e===l.hostname})),k=(t.allowedIframeDomains||[]).find((function(e){return l.hostname===e||l.hostname.endsWith(".".concat(e))}));E=O||k}}catch(e){E=!1}if(!E)return void delete r.attribs[i]}if("srcset"===i)try{if(g(l=c(n),(function(e){B("srcset",e.url)&&(e.evil=!0)})),!(l=m(l,(function(e){return!e.evil}))).length)return void delete r.attribs[i];n=m(l,(function(e){return!e.evil})).map((function(e){if(!e.url)throw new Error("URL missing");return e.url+(e.w?" ".concat(e.w,"w"):"")+(e.h?" ".concat(e.h,"h"):"")+(e.d?" ".concat(e.d,"x"):"")})).join(", "),r.attribs[i]=n}catch(e){return void delete r.attribs[i]}if("class"===i&&!(n=function(e,t){return t?(e=e.split(/\s+/)).filter((function(e){return-1!==t.indexOf(e)})).join(" "):e}(n,S[e])).length)return void delete r.attribs[i];if("style"===i)try{if(0===(n=function(e){return e.nodes[0].nodes.reduce((function(e,t){return e.push(t.prop+":"+t.value),e}),[]).join(";")}(function(e,t){if(!t)return e;var n,r=o(e),i=e.nodes[0];return(n=t[i.selector]&&t["*"]?a(o(t[i.selector]),t["*"],(function(e,t){if(Array.isArray(e))return e.concat(t)})):t[i.selector]||t["*"])&&(r.nodes[0].nodes=i.nodes.reduce(function(e){return function(t,n){return v(e,n.prop)&&e[n.prop].some((function(e){return e.test(n.value)}))&&t.push(n),t}}(n),[])),r}(d.parse(e+" {"+n+"}"),t.allowedStyles))).length)return void delete r.attribs[i]}catch(e){return void delete r.attribs[i]}A+=" "+i,n&&n.length&&(A+='="'+F(n,!0)+'"')}else delete r.attribs[i]}else delete r.attribs[i]})),-1!==t.selfClosing.indexOf(e)?A+=" />":(A+=">",!r.innerText||f||t.textFilter||(A+=r.innerText,j=!0)),l&&(A=_+F(A),_="")}},ontext:function(e){if(!D){var n,r=O[O.length-1];if(r&&(n=r.tag,e=void 0!==r.innerText?r.innerText:e),"discard"!==t.disallowedTagsMode||"script"!==n&&"style"!==n){var i=F(e,!1);t.textFilter&&!j?A+=t.textFilter(i,n):j||(A+=i)}else A+=e;O.length&&(O[O.length-1].text+=e)}},onclosetag:function(e){if(D){if(--P)return;D=!1}var n=O.pop();if(n){D=!!t.enforceHtmlBoundary&&"html"===e,E--;var r=k[E];if(r){if(delete k[E],"discard"===t.disallowedTagsMode)return void n.updateParentNodeText();_=A,A=""}L[E]&&(e=L[E],delete L[E]),t.exclusiveFilter&&t.exclusiveFilter(n)?A=A.substr(0,n.tagPosition):(n.updateParentNodeMediaChildren(),n.updateParentNodeText(),-1===t.selfClosing.indexOf(e)?(A+="</"+e+">",r&&(A=_+F(A),_="")):r&&(A=_,_=""))}}},t.parser);return R.write(e),R.end(),A;function z(){A="",E=0,O=[],k={},L={},D=!1,P=0}function F(e,n){return"string"!=typeof e&&(e+=""),t.parser.decodeEntities&&(e=e.replace(/&/g,"&amp;").replace(/</g,"&lt;").replace(/\>/g,"&gt;"),n&&(e=e.replace(/\"/g,"&quot;"))),e=e.replace(/&(?![a-zA-Z0-9#]{1,20};)/g,"&amp;").replace(/</g,"&lt;").replace(/\>/g,"&gt;"),n&&(e=e.replace(/\"/g,"&quot;")),e}function B(e,n){var r=(n=(n=n.replace(/[\x00-\x20]+/g,"")).replace(/<\!\-\-.*?\-\-\>/g,"")).match(/^([a-zA-Z]+)\:/);if(!r)return!!n.match(/^[\/\\]{2}/)&&!t.allowProtocolRelative;var i=r[1].toLowerCase();return v(t.allowedSchemesByTag,e)?-1===t.allowedSchemesByTag[e].indexOf(i):!t.allowedSchemes||-1===t.allowedSchemes.indexOf(i)}}var C={decodeEntities:!0};b.defaults={allowedTags:["h3","h4","h5","h6","blockquote","p","a","ul","ol","nl","li","b","i","strong","em","strike","abbr","code","hr","br","div","table","thead","caption","tbody","tr","th","td","pre","iframe"],disallowedTagsMode:"discard",allowedAttributes:{a:["href","name","target"],img:["src"]},selfClosing:["img","br","hr","area","base","basefont","input","link","meta"],allowedSchemes:["http","https","ftp","mailto"],allowedSchemesByTag:{},allowedSchemesAppliedToAttributes:["href","src","cite"],allowProtocolRelative:!0,enforceHtmlBoundary:!1},b.simpleTransform=function(e,t,n){return n=void 0===n||n,t=t||{},function(r,i){var o;if(n)for(o in t)i[o]=t[o];else i=t;return{tagName:e,attribs:i}}}},{htmlparser2:31,"lodash/cloneDeep":140,"lodash/escapeRegExp":143,"lodash/isPlainObject":155,"lodash/isString":157,"lodash/mergeWith":162,"parse-srcset":167,postcss:181,url:209}]},{},[211])(211)},"object"===u(t)&&void 0!==e?e.exports=a():(i=[],void 0===(o="function"==typeof(r=a)?r.apply(t,i):r)||(e.exports=o))}).call(this,n(61))},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.createHorizontalStrength=A,t.createVerticalStrength=_,t.createScrollingComponent=w,t.default=function(e){var t=w(e);return function(e){return r.default.createElement(u.DragDropContextConsumer,null,(function(n){var i=n.dragDropManager;return void 0===i?null:r.default.createElement(t,p({},e,{dragDropManager:i}))}))}},t.defaultVerticalStrength=t.defaultHorizontalStrength=void 0;var r=function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)if(Object.prototype.hasOwnProperty.call(e,n)){var r=Object.defineProperty&&Object.getOwnPropertyDescriptor?Object.getOwnPropertyDescriptor(e,n):{};r.get||r.set?Object.defineProperty(t,n,r):t[n]=e[n]}return t.default=e,t}(n(943)),i=h(n(3)),o=n(1587),a=h(n(1591)),s=h(n(1592)),l=h(n(1594)),u=n(1595),c=h(n(91)),d=n(1635);function h(e){return e&&e.__esModule?e:{default:e}}function f(e){return(f="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function p(){return(p=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e}).apply(this,arguments)}function g(e,t){if(null==e)return{};var n,r,i=function(e,t){if(null==e)return{};var n,r,i={},o=Object.keys(e);for(r=0;r<o.length;r++)n=o[r],t.indexOf(n)>=0||(i[n]=e[n]);return i}(e,t);if(Object.getOwnPropertySymbols){var o=Object.getOwnPropertySymbols(e);for(r=0;r<o.length;r++)n=o[r],t.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(e,n)&&(i[n]=e[n])}return i}function v(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}function m(e){return(m=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}function y(e,t){return(y=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function b(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function C(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function A(e){return function(t,n){var r=t.x,i=t.w,o=t.y,a=t.h,s=Math.min(i/2,e);if(n.x>=r&&n.x<=r+i&&n.y>=o&&n.y<=o+a){if(n.x<r+s)return(n.x-r-s)/s;if(n.x>r+i-s)return-(r+i-n.x-s)/s}return 0}}function _(e){return function(t,n){var r=t.y,i=t.h,o=t.x,a=t.w,s=Math.min(i/2,e);if(n.y>=r&&n.y<=r+i&&n.x>=o&&n.x<=o+a){if(n.y<r+s)return(n.y-r-s)/s;if(n.y>r+i-s)return-(r+i-n.y-s)/s}return 0}}var I=A(150);t.defaultHorizontalStrength=I;var x=_(150);function w(e){var t=function(t){function n(e,t){var i;return function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,n),i=function(e,t){return!t||"object"!==f(t)&&"function"!=typeof t?b(e):t}(this,m(n).call(this,e,t)),C(b(b(i)),"updateScrolling",(0,a.default)((function(e){var t=i.container.getBoundingClientRect(),n={x:t.left,y:t.top,w:t.width,h:t.height},r=(0,d.getCoords)(e),o=i.props,a=o.horizontalStrength,s=o.verticalStrength;i.scaleX=a(n,r),i.scaleY=s(n,r),i.frame||!i.scaleX&&!i.scaleY||i.startScrolling()}),100,{trailing:!1})),C(b(b(i)),"handleEvent",(function(e){i.dragging&&!i.attached&&(i.attach(),i.updateScrolling(e))})),i.wrappedInstance=r.default.createRef(),i.scaleX=0,i.scaleY=0,i.frame=null,i.attached=!1,i.dragging=!1,i}var i,l,u;return function(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&y(e,t)}(n,t),i=n,(l=[{key:"componentDidMount",value:function(){var e=this;this.container=(0,o.findDOMNode)(this.wrappedInstance.current),this.container&&"function"==typeof this.container.addEventListener&&this.container.addEventListener("dragover",this.handleEvent),window.document.body.addEventListener("touchmove",this.handleEvent);var t=this.props.dragDropManager;this.clearMonitorSubscription=t.getMonitor().subscribeToStateChange((function(){return e.handleMonitorChange()}))}},{key:"componentWillUnmount",value:function(){this.container&&"function"==typeof this.container.removeEventListener&&this.container.removeEventListener("dragover",this.handleEvent),window.document.body.removeEventListener("touchmove",this.handleEvent),this.clearMonitorSubscription(),this.stopScrolling()}},{key:"handleMonitorChange",value:function(){var e=this.props.dragDropManager.getMonitor().isDragging();!this.dragging&&e?this.dragging=!0:this.dragging&&!e&&(this.dragging=!1,this.stopScrolling())}},{key:"attach",value:function(){this.attached=!0,window.document.body.addEventListener("dragover",this.updateScrolling),window.document.body.addEventListener("touchmove",this.updateScrolling)}},{key:"detach",value:function(){this.attached=!1,window.document.body.removeEventListener("dragover",this.updateScrolling),window.document.body.removeEventListener("touchmove",this.updateScrolling)}},{key:"startScrolling",value:function(){var e=this,t=0;!function n(){var r=e.scaleX,i=e.scaleY,o=e.container,a=e.props,l=a.strengthMultiplier,u=a.onScrollChange;if(0!==l&&r+i!==0){if((t+=1)%2){var c=o.scrollLeft,h=o.scrollTop,f=o.scrollWidth,p=o.scrollHeight,g=o.clientWidth,v=o.clientHeight;u(r?o.scrollLeft=(0,d.intBetween)(0,f-g,c+r*l):c,i?o.scrollTop=(0,d.intBetween)(0,p-v,h+i*l):h)}e.frame=(0,s.default)(n)}else e.stopScrolling()}()}},{key:"stopScrolling",value:function(){this.detach(),this.scaleX=0,this.scaleY=0,this.frame&&(s.default.cancel(this.frame),this.frame=null)}},{key:"render",value:function(){var t=this.props,n=(t.strengthMultiplier,t.verticalStrength,t.horizontalStrength,t.onScrollChange,g(t,["strengthMultiplier","verticalStrength","horizontalStrength","onScrollChange"]));return r.default.createElement(e,p({ref:this.wrappedInstance},n))}}])&&v(i.prototype,l),u&&v(i,u),n}(r.Component);return C(t,"displayName","Scrolling(".concat((0,l.default)(e),")")),C(t,"propTypes",{dragDropManager:i.default.object.isRequired,onScrollChange:i.default.func,verticalStrength:i.default.func,horizontalStrength:i.default.func,strengthMultiplier:i.default.number}),C(t,"defaultProps",{onScrollChange:d.noop,verticalStrength:x,horizontalStrength:I,strengthMultiplier:30}),(0,c.default)(t,e)}t.defaultVerticalStrength=x},function(e,t,n){"use strict";var r=n(92),i=n(101),o=(n(3),n(0)),a=n.n(o),s=n(22),l=n.n(s),u=!1,c=n(275),d=function(e){function t(t,n){var r;r=e.call(this,t,n)||this;var i,o=n&&!n.isMounting?t.enter:t.appear;return r.appearStatus=null,t.in?o?(i="exited",r.appearStatus="entering"):i="entered":i=t.unmountOnExit||t.mountOnEnter?"unmounted":"exited",r.state={status:i},r.nextCallback=null,r}Object(i.a)(t,e),t.getDerivedStateFromProps=function(e,t){return e.in&&"unmounted"===t.status?{status:"exited"}:null};var n=t.prototype;return n.componentDidMount=function(){this.updateStatus(!0,this.appearStatus)},n.componentDidUpdate=function(e){var t=null;if(e!==this.props){var n=this.state.status;this.props.in?"entering"!==n&&"entered"!==n&&(t="entering"):"entering"!==n&&"entered"!==n||(t="exiting")}this.updateStatus(!1,t)},n.componentWillUnmount=function(){this.cancelNextCallback()},n.getTimeouts=function(){var e,t,n,r=this.props.timeout;return e=t=n=r,null!=r&&"number"!=typeof r&&(e=r.exit,t=r.enter,n=void 0!==r.appear?r.appear:t),{exit:e,enter:t,appear:n}},n.updateStatus=function(e,t){void 0===e&&(e=!1),null!==t?(this.cancelNextCallback(),"entering"===t?this.performEnter(e):this.performExit()):this.props.unmountOnExit&&"exited"===this.state.status&&this.setState({status:"unmounted"})},n.performEnter=function(e){var t=this,n=this.props.enter,r=this.context?this.context.isMounting:e,i=this.props.nodeRef?[r]:[l.a.findDOMNode(this),r],o=i[0],a=i[1],s=this.getTimeouts(),c=r?s.appear:s.enter;!e&&!n||u?this.safeSetState({status:"entered"},(function(){t.props.onEntered(o)})):(this.props.onEnter(o,a),this.safeSetState({status:"entering"},(function(){t.props.onEntering(o,a),t.onTransitionEnd(c,(function(){t.safeSetState({status:"entered"},(function(){t.props.onEntered(o,a)}))}))})))},n.performExit=function(){var e=this,t=this.props.exit,n=this.getTimeouts(),r=this.props.nodeRef?void 0:l.a.findDOMNode(this);t&&!u?(this.props.onExit(r),this.safeSetState({status:"exiting"},(function(){e.props.onExiting(r),e.onTransitionEnd(n.exit,(function(){e.safeSetState({status:"exited"},(function(){e.props.onExited(r)}))}))}))):this.safeSetState({status:"exited"},(function(){e.props.onExited(r)}))},n.cancelNextCallback=function(){null!==this.nextCallback&&(this.nextCallback.cancel(),this.nextCallback=null)},n.safeSetState=function(e,t){t=this.setNextCallback(t),this.setState(e,t)},n.setNextCallback=function(e){var t=this,n=!0;return this.nextCallback=function(r){n&&(n=!1,t.nextCallback=null,e(r))},this.nextCallback.cancel=function(){n=!1},this.nextCallback},n.onTransitionEnd=function(e,t){this.setNextCallback(t);var n=this.props.nodeRef?this.props.nodeRef.current:l.a.findDOMNode(this),r=null==e&&!this.props.addEndListener;if(n&&!r){if(this.props.addEndListener){var i=this.props.nodeRef?[this.nextCallback]:[n,this.nextCallback],o=i[0],a=i[1];this.props.addEndListener(o,a)}null!=e&&setTimeout(this.nextCallback,e)}else setTimeout(this.nextCallback,0)},n.render=function(){var e=this.state.status;if("unmounted"===e)return null;var t=this.props,n=t.children,i=(t.in,t.mountOnEnter,t.unmountOnExit,t.appear,t.enter,t.exit,t.timeout,t.addEndListener,t.onEnter,t.onEntering,t.onEntered,t.onExit,t.onExiting,t.onExited,t.nodeRef,Object(r.a)(t,["children","in","mountOnEnter","unmountOnExit","appear","enter","exit","timeout","addEndListener","onEnter","onEntering","onEntered","onExit","onExiting","onExited","nodeRef"]));return a.a.createElement(c.a.Provider,{value:null},"function"==typeof n?n(e,i):a.a.cloneElement(a.a.Children.only(n),i))},t}(a.a.Component);function h(){}d.contextType=c.a,d.propTypes={},d.defaultProps={in:!1,mountOnEnter:!1,unmountOnExit:!1,appear:!1,enter:!0,exit:!0,onEnter:h,onEntering:h,onEntered:h,onExit:h,onExiting:h,onExited:h},d.UNMOUNTED="unmounted",d.EXITED="exited",d.ENTERING="entering",d.ENTERED="entered",d.EXITING="exiting";t.a=d},,,,,,,,,,,function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.validateProperty=void 0,t.validateProperty=function(e,t,n,r=[]){if(!e.hasOwnProperty(t))throw Error(`Missing property '${t}'`);const i=e[t];if(void 0!==n){let e=!0;switch(n){case"array":e=Array.isArray(i);break;case"object":e=void 0!==i;break;default:e=typeof i===n}if(!e)throw new Error(`Property '${t}' is not of type '${n}'`);if(r.length>0){let e=!0;switch(n){case"string":case"number":case"boolean":e=r.includes(i);break;default:e=r.findIndex(e=>e===i)>=0}if(!e)throw new Error(`Property '${t}' is not one of the valid values ${JSON.stringify(r)}`)}}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.isInputReplyMsg=t.isInputRequestMsg=t.isDebugReplyMsg=t.isDebugRequestMsg=t.isExecuteReplyMsg=t.isInfoRequestMsg=t.isCommMsgMsg=t.isCommCloseMsg=t.isCommOpenMsg=t.isDebugEventMsg=t.isClearOutputMsg=t.isStatusMsg=t.isErrorMsg=t.isExecuteResultMsg=t.isExecuteInputMsg=t.isUpdateDisplayDataMsg=t.isDisplayDataMsg=t.isStreamMsg=t.createMessage=void 0;const r=n(17);t.createMessage=function(e){var t,n,i,o,a;return{buffers:null!==(t=e.buffers)&&void 0!==t?t:[],channel:e.channel,content:e.content,header:{date:(new Date).toISOString(),msg_id:null!==(n=e.msgId)&&void 0!==n?n:r.UUID.uuid4(),msg_type:e.msgType,session:e.session,username:null!==(i=e.username)&&void 0!==i?i:"",version:"5.2"},metadata:null!==(o=e.metadata)&&void 0!==o?o:{},parent_header:null!==(a=e.parentHeader)&&void 0!==a?a:{}}},t.isStreamMsg=function(e){return"stream"===e.header.msg_type},t.isDisplayDataMsg=function(e){return"display_data"===e.header.msg_type},t.isUpdateDisplayDataMsg=function(e){return"update_display_data"===e.header.msg_type},t.isExecuteInputMsg=function(e){return"execute_input"===e.header.msg_type},t.isExecuteResultMsg=function(e){return"execute_result"===e.header.msg_type},t.isErrorMsg=function(e){return"error"===e.header.msg_type},t.isStatusMsg=function(e){return"status"===e.header.msg_type},t.isClearOutputMsg=function(e){return"clear_output"===e.header.msg_type},t.isDebugEventMsg=function(e){return"debug_event"===e.header.msg_type},t.isCommOpenMsg=function(e){return"comm_open"===e.header.msg_type},t.isCommCloseMsg=function(e){return"comm_close"===e.header.msg_type},t.isCommMsgMsg=function(e){return"comm_msg"===e.header.msg_type},t.isInfoRequestMsg=function(e){return"kernel_info_request"===e.header.msg_type},t.isExecuteReplyMsg=function(e){return"execute_reply"===e.header.msg_type},t.isDebugRequestMsg=function(e){return"debug_request"===e.header.msg_type},t.isDebugReplyMsg=function(e){return"debug_reply"===e.header.msg_type},t.isInputRequestMsg=function(e){return"input_request"===e.header.msg_type},t.isInputReplyMsg=function(e){return"input_reply"===e.header.msg_type}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.BaseManager=void 0;const r=n(16),i=n(129);t.BaseManager=class{constructor(e){var t;this._isDisposed=!1,this._disposed=new r.Signal(this),this.serverSettings=null!==(t=e.serverSettings)&&void 0!==t?t:i.ServerConnection.makeSettings()}get disposed(){return this._disposed}get isDisposed(){return this._isDisposed}dispose(){this.isDisposed||(this._disposed.emit(void 0),r.Signal.clearData(this))}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.shutdownTerminal=t.listRunning=t.startNew=t.isAvailable=t.TERMINAL_SERVICE_URL=void 0;const r=n(48),i=n(129);function o(){return"true"===String(r.PageConfig.getOption("terminalsAvailable")).toLowerCase()}var a;t.TERMINAL_SERVICE_URL="api/terminals",t.isAvailable=o,t.startNew=async function(e=i.ServerConnection.makeSettings()){a.errorIfNotAvailable();const n=r.URLExt.join(e.baseUrl,t.TERMINAL_SERVICE_URL),o=await i.ServerConnection.makeRequest(n,{method:"POST"},e);if(200!==o.status){throw await i.ServerConnection.ResponseError.create(o)}return await o.json()},t.listRunning=async function(e=i.ServerConnection.makeSettings()){a.errorIfNotAvailable();const n=r.URLExt.join(e.baseUrl,t.TERMINAL_SERVICE_URL),o=await i.ServerConnection.makeRequest(n,{},e);if(200!==o.status){throw await i.ServerConnection.ResponseError.create(o)}const s=await o.json();if(!Array.isArray(s))throw new Error("Invalid terminal list");return s},t.shutdownTerminal=async function(e,n=i.ServerConnection.makeSettings()){var o;a.errorIfNotAvailable();const s=r.URLExt.join(n.baseUrl,t.TERMINAL_SERVICE_URL,e),l=await i.ServerConnection.makeRequest(s,{method:"DELETE"},n);if(404===l.status){const t=null!==(o=(await l.json()).message)&&void 0!==o?o:`The terminal session "${e}"" does not exist on the server`;console.warn(t)}else if(204!==l.status){throw await i.ServerConnection.ResponseError.create(l)}},function(e){e.errorIfNotAvailable=function(){if(!o())throw new Error("Terminals Unavailable")}}(a||(a={}))},function(e,t,n){var r=n(357),i=n(1388),o=n(1389),a=n(1390),s=n(1391),l=n(1392);function u(e){var t=this.__data__=new r(e);this.size=t.size}u.prototype.clear=i,u.prototype.delete=o,u.prototype.get=a,u.prototype.has=s,u.prototype.set=l,e.exports=u},function(e,t,n){var r=n(1383),i=n(1384),o=n(1385),a=n(1386),s=n(1387);function l(e){var t=-1,n=null==e?0:e.length;for(this.clear();++t<n;){var r=e[t];this.set(r[0],r[1])}}l.prototype.clear=r,l.prototype.delete=i,l.prototype.get=o,l.prototype.has=a,l.prototype.set=s,e.exports=l},function(e,t,n){var r=n(235);e.exports=function(e,t){for(var n=e.length;n--;)if(r(e[n][0],t))return n;return-1}},function(e,t,n){var r=n(207)(Object,"create");e.exports=r},function(e,t,n){var r=n(1407);e.exports=function(e,t){var n=e.__data__;return r(t)?n["string"==typeof t?"string":"hash"]:n.map}},function(e,t){var n=/^(?:0|[1-9]\d*)$/;e.exports=function(e,t){var r=typeof e;return!!(t=null==t?9007199254740991:t)&&("number"==r||"symbol"!=r&&n.test(e))&&e>-1&&e%1==0&&e<t}},function(e,t){e.exports=function(e){return function(t){return e(t)}}},function(e,t){var n=Object.prototype;e.exports=function(e){var t=e&&e.constructor;return e===("function"==typeof t&&t.prototype||n)}},function(e,t,n){var r=n(861)(Object.getPrototypeOf,Object);e.exports=r},function(e,t,n){"use strict";
/*
object-assign
(c) Sindre Sorhus
@license MIT
*/var r=Object.getOwnPropertySymbols,i=Object.prototype.hasOwnProperty,o=Object.prototype.propertyIsEnumerable;function a(e){if(null==e)throw new TypeError("Object.assign cannot be called with null or undefined");return Object(e)}e.exports=function(){try{if(!Object.assign)return!1;var e=new String("abc");if(e[5]="de","5"===Object.getOwnPropertyNames(e)[0])return!1;for(var t={},n=0;n<10;n++)t["_"+String.fromCharCode(n)]=n;if("0123456789"!==Object.getOwnPropertyNames(t).map((function(e){return t[e]})).join(""))return!1;var r={};return"abcdefghijklmnopqrst".split("").forEach((function(e){r[e]=e})),"abcdefghijklmnopqrst"===Object.keys(Object.assign({},r)).join("")}catch(e){return!1}}()?Object.assign:function(e,t){for(var n,s,l=a(e),u=1;u<arguments.length;u++){for(var c in n=Object(arguments[u]))i.call(n,c)&&(l[c]=n[c]);if(r){s=r(n);for(var d=0;d<s.length;d++)o.call(n,s[d])&&(l[s[d]]=n[s[d]])}}return l}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.Intent=void 0,t.Intent={NONE:"none",PRIMARY:"primary",SUCCESS:"success",WARNING:"warning",DANGER:"danger"}},function(e,t,n){"use strict";n.d(t,"b",(function(){return c}));var r=n(1),i=n(5),o=n(0),a=n.n(o),s=(n(3),n(91)),l=n.n(s),u=n(277);function c(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},t=e.defaultTheme,n=function(e){var n=a.a.forwardRef((function(n,o){var s=n.innerRef,l=Object(i.a)(n,["innerRef"]),c=Object(u.a)()||t;return a.a.createElement(e,Object(r.a)({theme:c,ref:s||o},l))}));return l()(n,e),n};return n}var d=c();t.a=d},function(e,t,n){"use strict";n.r(t);var r=n(513);n.d(t,"default",(function(){return r.a})),n.d(t,"styleFunction",(function(){return r.b}))},function(e,t,n){"use strict";n.d(t,"c",(function(){return h})),n.d(t,"b",(function(){return f}));var r=n(1),i=n(5),o=n(0),a=(n(3),n(653)),s=n(91),l=n.n(s),u=n(52),c=n(155),d=n(2358),h=function(e,t){var n=!(arguments.length>2&&void 0!==arguments[2])||arguments[2];return n?c.b.indexOf(e)<=c.b.indexOf(t):c.b.indexOf(e)<c.b.indexOf(t)},f=function(e,t){var n=!(arguments.length>2&&void 0!==arguments[2])||arguments[2];return n?c.b.indexOf(t)<=c.b.indexOf(e):c.b.indexOf(t)<c.b.indexOf(e)},p="undefined"==typeof window?o.useEffect:o.useLayoutEffect;t.a=function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};return function(t){var n=e.withTheme,s=void 0!==n&&n,c=e.noSSR,h=void 0!==c&&c,f=e.initialWidth;function g(e){var n=Object(u.a)(),l=e.theme||n,c=Object(a.a)({theme:l,name:"MuiWithWidth",props:Object(r.a)({},e)}),g=c.initialWidth,v=c.width,m=Object(i.a)(c,["initialWidth","width"]),y=o.useState(!1),b=y[0],C=y[1];p((function(){C(!0)}),[]);var A=l.breakpoints.keys.slice().reverse().reduce((function(e,t){var n=Object(d.a)(l.breakpoints.up(t));return!e&&n?t:e}),null),_=Object(r.a)({width:v||(b||h?A:void 0)||g||f},s?{theme:l}:{},m);return void 0===_.width?null:o.createElement(t,_)}return l()(g,t),g}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=function(e,t,n,r,i){this.node=e,this.rect=t,this.location=n,this.index=r,this.className=i};t.default=r},function(e,t,n){"use strict";var r,i=this&&this.__extends||(r=function(e,t){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(e[n]=t[n])})(e,t)},function(e,t){function n(){this.constructor=e}r(e,t),e.prototype=null===t?Object.create(t):(n.prototype=t.prototype,new n)});Object.defineProperty(t,"__esModule",{value:!0});var o=n(244),a=n(111),s=function(e){function t(n){var r=e.call(this,n)||this;return r._fixed=!0,r._attributes.type=t.TYPE,n._addNode(r),r}return i(t,e),t.prototype.getWidth=function(){return this._model.getSplitterSize()},t.prototype.getMinWidth=function(){return this.getOrientation()===a.default.VERT?this._model.getSplitterSize():0},t.prototype.getHeight=function(){return this._model.getSplitterSize()},t.prototype.getMinHeight=function(){return this.getOrientation()===a.default.HORZ?this._model.getSplitterSize():0},t.prototype.getMinSize=function(e){return e===a.default.HORZ?this.getMinWidth():this.getMinHeight()},t.prototype.getWeight=function(){return 0},t.prototype._setWeight=function(e){},t.prototype._getPrefSize=function(e){return this._model.getSplitterSize()},t.prototype._updateAttrs=function(e){},t.prototype._getAttributeDefinitions=function(){return new o.default},t.prototype._toJson=function(){},t.TYPE="splitter",t}(n(245).default);t.default=s},function(e,t,n){"use strict";var r,i=this&&this.__extends||(r=function(e,t){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(e,t)},function(e,t){function n(){this.constructor=e}r(e,t),e.prototype=null===t?Object.create(t):(n.prototype=t.prototype,new n)}),o=this&&this.__assign||function(){return(o=Object.assign||function(e){for(var t,n=1,r=arguments.length;n<r;n++)for(var i in t=arguments[n])Object.prototype.hasOwnProperty.call(t,i)&&(e[i]=t[i]);return e}).apply(this,arguments)},a=this&&this.__rest||function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(null!=e&&"function"==typeof Object.getOwnPropertySymbols){var i=0;for(r=Object.getOwnPropertySymbols(e);i<r.length;i++)t.indexOf(r[i])<0&&Object.prototype.propertyIsEnumerable.call(e,r[i])&&(n[r[i]]=e[r[i]])}return n};Object.defineProperty(t,"__esModule",{value:!0});var s=n(0),l=n(1596),u=n(375),c=n(517),d=n(19),h=n(91);function f(e,t,n){return{dragDropManager:l.createDragDropManager(e,t,n)}}t.context=s.createContext({dragDropManager:void 0}),t.Consumer=t.context.Consumer,t.Provider=t.context.Provider,t.createChildContext=f,t.DragDropContextProvider=function(e){var n=e.children,r=a(e,["children"]),i="manager"in r?{dragDropManager:r.manager}:f(r.backend,r.context,r.debugMode);return s.createElement(t.Provider,{value:i},n)},t.DragDropContext=function(e,n,r){u.default("DragDropContext","backend",e);var a=f(e,n,r);return function(e){var n=e,r=n.displayName||n.name||"Component",l=function(l){function u(){var e=null!==l&&l.apply(this,arguments)||this;return e.ref=s.createRef(),e.getManager=function(){return a.dragDropManager},e}return i(u,l),u.prototype.getDecoratedComponentInstance=function(){return d(this.ref.current,"In order to access an instance of the decorated component, it must either be a class component or use React.forwardRef()"),this.ref.current},u.prototype.render=function(){return s.createElement(t.Provider,{value:a},s.createElement(n,o({},this.props,{ref:c.isRefable(n)?this.ref:null})))},u.DecoratedComponent=e,u.displayName="DragDropContext("+r+")",u}(s.Component);return h(l,e)}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(1601),i=n(1603),o=n(1604),a=n(1605),s=n(1606);!function(e){for(var n in e)t.hasOwnProperty(n)||(t[n]=e[n])}(n(211)),t.default=function(e){return{beginDrag:r.default(e),publishDragSource:i.default(e),hover:o.default(e),drop:a.default(e),endDrag:s.default(e)}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.ADD_SOURCE="dnd-core/ADD_SOURCE",t.ADD_TARGET="dnd-core/ADD_TARGET",t.REMOVE_SOURCE="dnd-core/REMOVE_SOURCE",t.REMOVE_TARGET="dnd-core/REMOVE_TARGET",t.addSource=function(e){return{type:t.ADD_SOURCE,payload:{sourceId:e}}},t.addTarget=function(e){return{type:t.ADD_TARGET,payload:{targetId:e}}},t.removeSource=function(e){return{type:t.REMOVE_SOURCE,payload:{sourceId:e}}},t.removeTarget=function(e){return{type:t.REMOVE_TARGET,payload:{targetId:e}}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){for(var n=[],r=2;r<arguments.length;r++)n[r-2]=arguments[r]}},function(e,t,n){"use strict";n.r(t);var r=n(922);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";var r=n(36),i=n(46);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(0)),a=(0,r(n(66)).default)(o.createElement("path",{d:"M3 17.25V21h3.75L17.81 9.94l-3.75-3.75L3 17.25zM20.71 7.04c.39-.39.39-1.02 0-1.41l-2.34-2.34a.9959.9959 0 00-1.41 0l-1.83 1.83 3.75 3.75 1.83-1.83z"}),"Edit");t.default=a},function(e,t,n){"use strict";var r=n(36),i=n(46);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(0)),a=(0,r(n(66)).default)(o.createElement("path",{d:"M19.14 12.94c.04-.3.06-.61.06-.94 0-.32-.02-.64-.07-.94l2.03-1.58c.18-.14.23-.41.12-.61l-1.92-3.32c-.12-.22-.37-.29-.59-.22l-2.39.96c-.5-.38-1.03-.7-1.62-.94l-.36-2.54c-.04-.24-.24-.41-.48-.41h-3.84c-.24 0-.43.17-.47.41l-.36 2.54c-.59.24-1.13.57-1.62.94l-2.39-.96c-.22-.08-.47 0-.59.22L2.74 8.87c-.12.21-.08.47.12.61l2.03 1.58c-.05.3-.09.63-.09.94s.02.64.07.94l-2.03 1.58c-.18.14-.23.41-.12.61l1.92 3.32c.12.22.37.29.59.22l2.39-.96c.5.38 1.03.7 1.62.94l.36 2.54c.05.24.24.41.48.41h3.84c.24 0 .44-.17.47-.41l.36-2.54c.59-.24 1.13-.56 1.62-.94l2.39.96c.22.08.47 0 .59-.22l1.92-3.32c.12-.22.07-.47-.12-.61l-2.01-1.58zM12 15.6c-1.98 0-3.6-1.62-3.6-3.6s1.62-3.6 3.6-3.6 3.6 1.62 3.6 3.6-1.62 3.6-3.6 3.6z"}),"Settings");t.default=a},function(e,t,n){"use strict";var r=n(1639),i=r.default,o=r.DraggableCore;e.exports=i,e.exports.default=i,e.exports.DraggableCore=o},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.findInArray=function(e,t){for(var n=0,r=e.length;n<r;n++)if(t.apply(t,[e[n],n,e]))return e[n]},t.isFunction=function(e){return"function"==typeof e||"[object Function]"===Object.prototype.toString.call(e)},t.isNum=function(e){return"number"==typeof e&&!isNaN(e)},t.int=function(e){return parseInt(e,10)},t.dontSetMe=function(e,t,n){if(e[t])return new Error("Invalid prop ".concat(t," passed to ").concat(n," - do not set this, set it on the child."))}},function(e,t,n){"use strict";e.exports={YANGLE:60,HOVERARROWSIZE:6,HOVERTEXTPAD:3,HOVERFONTSIZE:13,HOVERFONT:"Arial, sans-serif",HOVERMINTIME:50,HOVERID:"-hover"}},function(e,t,n){"use strict";e.exports={mode:{valType:"enumerated",dflt:"afterall",values:["immediate","next","afterall"]},direction:{valType:"enumerated",values:["forward","reverse"],dflt:"forward"},fromcurrent:{valType:"boolean",dflt:!1},frame:{duration:{valType:"number",min:0,dflt:500},redraw:{valType:"boolean",dflt:!0}},transition:{duration:{valType:"number",min:0,dflt:500,editType:"none"},easing:{valType:"enumerated",dflt:"cubic-in-out",values:["linear","quad","cubic","sin","exp","circle","elastic","back","bounce","linear-in","quad-in","cubic-in","sin-in","exp-in","circle-in","elastic-in","back-in","bounce-in","linear-out","quad-out","cubic-out","sin-out","exp-out","circle-out","elastic-out","back-out","bounce-out","linear-in-out","quad-in-out","cubic-in-out","sin-in-out","exp-in-out","circle-in-out","elastic-in-out","back-in-out","bounce-in-out"],editType:"none"},ordering:{valType:"enumerated",values:["layout first","traces first"],dflt:"layout first",editType:"none"}}}},function(e,t,n){"use strict";var r=n(540).EventEmitter,i={init:function(e){if(e._ev instanceof r)return e;var t=new r,n=new r;return e._ev=t,e._internalEv=n,e.on=t.on.bind(t),e.once=t.once.bind(t),e.removeListener=t.removeListener.bind(t),e.removeAllListeners=t.removeAllListeners.bind(t),e._internalOn=n.on.bind(n),e._internalOnce=n.once.bind(n),e._removeInternalListener=n.removeListener.bind(n),e._removeAllInternalListeners=n.removeAllListeners.bind(n),e.emit=function(r,i){"undefined"!=typeof jQuery&&jQuery(e).trigger(r,i),t.emit(r,i),n.emit(r,i)},e},triggerHandler:function(e,t,n){var r,i;"undefined"!=typeof jQuery&&(r=jQuery(e).triggerHandler(t,n));var o=e._ev;if(!o)return r;var a,s=o._events[t];if(!s)return r;function l(e){return e.listener?(o.removeListener(t,e.listener),e.fired?void 0:(e.fired=!0,e.listener.apply(o,[n]))):e.apply(o,[n])}for(s=Array.isArray(s)?s:[s],a=0;a<s.length-1;a++)l(s[a]);return i=l(s[a]),void 0!==r?r:i},purge:function(e){return delete e._ev,delete e.on,delete e.once,delete e.removeListener,delete e.removeAllListeners,delete e.emit,delete e._ev,delete e._internalEv,delete e._internalOn,delete e._internalOnce,delete e._removeInternalListener,delete e._removeAllInternalListeners,e}};e.exports=i},function(e,t,n){"use strict";var r=n(27),i=n(95).SUBPLOT_PATTERN;t.getSubplotCalcData=function(e,t,n){var i=r.subplotsRegistry[t];if(!i)return[];for(var o=i.attr,a=[],s=0;s<e.length;s++){var l=e[s];l[0].trace[o]===n&&a.push(l)}return a},t.getModuleCalcData=function(e,t){var n,i=[],o=[];if(!(n="string"==typeof t?r.getModule(t).plot:"function"==typeof t?t:t.plot))return[i,e];for(var a=0;a<e.length;a++){var s=e[a],l=s[0].trace;!0===l.visible&&0!==l._length&&(l._module.plot===n?i.push(s):o.push(s))}return[i,o]},t.getSubplotData=function(e,t,n){if(!r.subplotsRegistry[t])return[];var o,a,s,l=r.subplotsRegistry[t].attr,u=[];if("gl2d"===t){var c=n.match(i);a="x"+c[1],s="y"+c[2]}for(var d=0;d<e.length;d++)o=e[d],"gl2d"===t&&r.traceIs(o,"gl2d")?o[l[0]]===a&&o[l[1]]===s&&u.push(o):o[l]===n&&u.push(o);return u}},function(e,t,n){"use strict";var r=n(39),i=n(42),o=n(82),a=n(27),s=n(10),l=s.strTranslate,u=n(56),c=n(41),d=n(96),h=n(525),f=n(97).OPPOSITE_SIDE,p=/ [XY][0-9]* /;e.exports={draw:function(e,t,n){var g,v=n.propContainer,m=n.propName,y=n.placeholder,b=n.traceIndex,C=n.avoid||{},A=n.attributes,_=n.transform,I=n.containerGroup,x=e._fullLayout,w=1,M=!1,S=v.title,T=(S&&S.text?S.text:"").trim(),E=S&&S.font?S.font:{},O=E.family,k=E.size,L=E.color;"title.text"===m?g="titleText":-1!==m.indexOf("axis")?g="axisTitleText":m.indexOf(!0)&&(g="colorbarTitleText");var D=e._context.edits[g];""===T?w=0:T.replace(p," % ")===y.replace(p," % ")&&(w=.2,M=!0,D||(T="")),n._meta?T=s.templateString(T,n._meta):x._meta&&(T=s.templateString(T,x._meta));var P=T||D;I||(I=s.ensureSingle(x._infolayer,"g","g-"+t));var N=I.selectAll("text").data(P?[0]:[]);if(N.enter().append("text"),N.text(T).attr("class",t),N.exit().remove(),!P)return I;function j(e){s.syncOrAsync([R,z],e)}function R(t){var n;return _?(n="",_.rotate&&(n+="rotate("+[_.rotate,A.x,A.y]+")"),_.offset&&(n+=l(0,_.offset))):n=null,t.attr("transform",n),t.style({"font-family":O,"font-size":r.round(k,2)+"px",fill:c.rgb(L),opacity:w*c.opacity(L),"font-weight":o.fontWeight}).attr(A).call(d.convertToTspans,e),o.previousPromises(e)}function z(e){var t=r.select(e.node().parentNode);if(C&&C.selection&&C.side&&T){t.attr("transform",null);var n=f[C.side],o="left"===C.side||"top"===C.side?-1:1,a=i(C.pad)?C.pad:2,c=u.bBox(t.node()),d={left:0,top:0,right:x.width,bottom:x.height},h=C.maxShift||o*(d[C.side]-c[C.side]),p=0;if(h<0)p=h;else{var g=C.offsetLeft||0,v=C.offsetTop||0;c.left-=g,c.right-=g,c.top-=v,c.bottom-=v,C.selection.each((function(){var e=u.bBox(this);s.bBoxIntersect(c,e,a)&&(p=Math.max(p,o*(e[C.side]-c[n])+a))})),p=Math.min(h,p)}if(p>0||h<0){var m={left:[-p,0],right:[p,0],top:[0,-p],bottom:[0,p]}[C.side];t.attr("transform",l(m[0],m[1]))}}}return N.call(j),D&&(T?N.on(".opacity",null):(w=0,M=!0,N.text(y).on("mouseover.opacity",(function(){r.select(this).transition().duration(h.SHOW_PLACEHOLDER).style("opacity",1)})).on("mouseout.opacity",(function(){r.select(this).transition().duration(h.HIDE_PLACEHOLDER).style("opacity",0)}))),N.call(d.makeEditable,{gd:e}).on("edit",(function(t){void 0!==b?a.call("_guiRestyle",e,m,t,b):a.call("_guiRelayout",e,m,t)})).on("cancel",(function(){this.text(this.attr("data-unformatted")).call(j)})).on("input",(function(e){this.text(e||" ").call(d.positionText,A.x,A.y)}))),N.classed("js-placeholder",M),I}}},function(e,t,n){"use strict";var r=n(42),i=n(10),o=n(72).FP_SAFE,a=n(27),s=n(83),l=s.getFromId,u=s.isLinked;function c(e,t){var n,r,o=[],a=e._fullLayout,s=h(a,t,0),l=h(a,t,1),u=f(e,t),c=u.min,p=u.max;if(0===c.length||0===p.length)return i.simpleMap(t.range,t.r2l);var g=c[0].val,v=p[0].val;for(n=1;n<c.length&&g===v;n++)g=Math.min(g,c[n].val);for(n=1;n<p.length&&g===v;n++)v=Math.max(v,p[n].val);var m=!1;if(t.range){var y=i.simpleMap(t.range,t.r2l);m=y[1]<y[0]}"reversed"===t.autorange&&(m=!0,t.autorange=!0);var b,C,A,_,I,x,w=t.rangemode,M="tozero"===w,S="nonnegative"===w,T=t._length,E=T/10,O=0;for(n=0;n<c.length;n++)for(b=c[n],r=0;r<p.length;r++)(x=(C=p[r]).val-b.val-d(t,b.val,C.val))>0&&((I=T-s(b)-l(C))>E?x/I>O&&(A=b,_=C,O=x/I):x/T>O&&(A={val:b.val,nopad:1},_={val:C.val,nopad:1},O=x/T));if(g===v){var k=g-1,L=g+1;if(M)if(0===g)o=[0,1];else{var D=(g>0?p:c).reduce((function(e,t){return Math.max(e,l(t))}),0),P=g/(1-Math.min(.5,D/T));o=g>0?[0,P]:[P,0]}else o=S?[Math.max(0,k),Math.max(1,L)]:[k,L]}else M?(A.val>=0&&(A={val:0,nopad:1}),_.val<=0&&(_={val:0,nopad:1})):S&&(A.val-O*s(A)<0&&(A={val:0,nopad:1}),_.val<=0&&(_={val:1,nopad:1})),O=(_.val-A.val-d(t,b.val,C.val))/(T-s(A)-l(_)),o=[A.val-O*s(A),_.val+O*l(_)];return m&&o.reverse(),i.simpleMap(o,t.l2r||Number)}function d(e,t,n){var r=0;if(e.rangebreaks)for(var i=e.locateBreaks(t,n),o=0;o<i.length;o++){var a=i[o];r+=a.max-a.min}return r}function h(e,t,n){var r=.05*t._length,o=t._anchorAxis||{};if(-1!==(t.ticklabelposition||"").indexOf("inside")||-1!==(o.ticklabelposition||"").indexOf("inside")){var a="reversed"===t.autorange;if(!a){var s=i.simpleMap(t.range,t.r2l);a=s[1]<s[0]}a&&(n=!n)}var l=0;return u(e,t._id)||(l=function(e,t){var n=0,r=e._anchorAxis||{};if(-1!==(r.ticklabelposition||"").indexOf("inside")&&(!t&&("left"===r.side||"bottom"===r.side)||t&&("top"===r.side||"right"===r.side))){var o="x"===e._id.charAt(0);if(r._vals){var a=i.deg2rad(r._tickAngles[r._id+"tick"]||0),s=Math.abs(Math.cos(a)),l=Math.abs(Math.sin(a));r._vals.forEach((function(e){if(e.bb){var t=6+e.bb.width,r=6+e.bb.height;n=Math.max(n,o?Math.max(t*s,r*l):Math.max(r*s,t*l))}}))}"inside"===r.ticks&&"inside"===r.ticklabelposition&&(n+=r.ticklen||0)}return n}(t,n)),r=Math.max(l,r),"domain"===t.constrain&&t._inputDomain&&(r*=(t._inputDomain[1]-t._inputDomain[0])/(t.domain[1]-t.domain[0])),function(e){return e.nopad?0:e.pad+(e.extrapad?r:l)}}e.exports={getAutoRange:c,makePadFn:h,doAutoRange:function(e,t,n){if(t.setScale(),t.autorange){t.range=n?n.slice():c(e,t),t._r=t.range.slice(),t._rl=i.simpleMap(t._r,t.r2l);var r=t._input,o={};o[t._attr+".range"]=t.range,o[t._attr+".autorange"]=t.autorange,a.call("_storeDirectGUIEdit",e.layout,e._fullLayout._preGUI,o),r.range=t.range.slice(),r.autorange=t.autorange}var s=t._anchorAxis;if(s&&s.rangeslider){var l=s.rangeslider[t._name];l&&"auto"===l.rangemode&&(l.range=c(e,t)),s._input.rangeslider[t._name]=i.extendFlat({},l)}},findExtremes:function(e,t,n){n||(n={});e._m||e.setScale();var i,a,s,l,u,c,d,h,f,v=[],y=[],b=t.length,C=n.padded||!1,A=n.tozero&&("linear"===e.type||"-"===e.type),_="log"===e.type,I=!1,x=n.vpadLinearized||!1;function w(e){if(Array.isArray(e))return I=!0,function(t){return Math.max(Number(e[t]||0),0)};var t=Math.max(Number(e||0),0);return function(){return t}}var M=w((e._m>0?n.ppadplus:n.ppadminus)||n.ppad||0),S=w((e._m>0?n.ppadminus:n.ppadplus)||n.ppad||0),T=w(n.vpadplus||n.vpad),E=w(n.vpadminus||n.vpad);if(!I){if(h=1/0,f=-1/0,_)for(i=0;i<b;i++)(a=t[i])<h&&a>0&&(h=a),a>f&&a<o&&(f=a);else for(i=0;i<b;i++)(a=t[i])<h&&a>-o&&(h=a),a>f&&a<o&&(f=a);t=[h,f],b=2}var O={tozero:A,extrapad:C};function k(n){s=t[n],r(s)&&(c=M(n),d=S(n),x?(l=e.c2l(s)-E(n),u=e.c2l(s)+T(n)):(h=s-E(n),f=s+T(n),_&&h<f/10&&(h=f/10),l=e.c2l(h),u=e.c2l(f)),A&&(l=Math.min(0,l),u=Math.max(0,u)),m(l)&&p(v,l,d,O),m(u)&&g(y,u,c,O))}var L=Math.min(6,b);for(i=0;i<L;i++)k(i);for(i=b-1;i>=L;i--)k(i);return{min:v,max:y,opts:n}},concatExtremes:f};function f(e,t,n){var r,i,o,a=t._id,s=e._fullData,u=e._fullLayout,c=[],d=[];function h(e,t){for(r=0;r<t.length;r++){var n=e[t[r]],s=(n._extremes||{})[a];if(!0===n.visible&&s){for(i=0;i<s.min.length;i++)o=s.min[i],p(c,o.val,o.pad,{extrapad:o.extrapad});for(i=0;i<s.max.length;i++)o=s.max[i],g(d,o.val,o.pad,{extrapad:o.extrapad})}}}if(h(s,t._traceIndices),h(u.annotations||[],t._annIndices||[]),h(u.shapes||[],t._shapeIndices||[]),t._matchGroup&&!n)for(var v in t._matchGroup)if(v!==t._id){var m=l(e,v),y=f(e,m,!0),b=t._length/m._length;for(i=0;i<y.min.length;i++)o=y.min[i],p(c,o.val,o.pad*b,{extrapad:o.extrapad});for(i=0;i<y.max.length;i++)o=y.max[i],g(d,o.val,o.pad*b,{extrapad:o.extrapad})}return{min:c,max:d}}function p(e,t,n,r){v(e,t,n,r,y)}function g(e,t,n,r){v(e,t,n,r,b)}function v(e,t,n,r,i){for(var o=r.tozero,a=r.extrapad,s=!0,l=0;l<e.length&&s;l++){var u=e[l];if(i(u.val,t)&&u.pad>=n&&(u.extrapad||!a)){s=!1;break}i(t,u.val)&&u.pad<=n&&(a||!u.extrapad)&&(e.splice(l,1),l--)}if(s){var c=o&&0===t;e.push({val:t,pad:c?0:n,extrapad:!c&&a})}}function m(e){return r(e)&&Math.abs(e)<o}function y(e,t){return e<=t}function b(e,t){return e>=t}},function(e,t,n){"use strict";var r=n(981);e.exports=r&&function(){var e=!1;try{var t=Object.defineProperty({},"passive",{get:function(){e=!0}});window.addEventListener("test",null,t),window.removeEventListener("test",null,t)}catch(t){e=!1}return e}()},function(e,t,n){"use strict";t.selectMode=function(e){return"lasso"===e||"select"===e},t.drawMode=function(e){return"drawclosedpath"===e||"drawopenpath"===e||"drawline"===e||"drawrect"===e||"drawcircle"===e},t.openMode=function(e){return"drawline"===e||"drawopenpath"===e},t.rectMode=function(e){return"select"===e||"drawline"===e||"drawrect"===e||"drawcircle"===e},t.freeMode=function(e){return"lasso"===e||"drawclosedpath"===e||"drawopenpath"===e},t.selectingOrDrawing=function(e){return t.freeMode(e)||t.rectMode(e)}},function(e,t,n){"use strict";var r=n(1704),i=n(551),o=i.CIRCLE_SIDES,a=i.SQRT2,s=n(552),l=s.p2r,u=s.r2p,c=[0,3,4,5,6,1,2],d=[0,3,4,1,2];function h(e,t){return Math.abs(e-t)<=1e-6}function f(e,t){var n=t[1]-e[1],r=t[2]-e[2];return Math.sqrt(n*n+r*r)}t.writePaths=function(e){var t=e.length;if(!t)return"M0,0Z";for(var n="",r=0;r<t;r++)for(var i=e[r].length,o=0;o<i;o++){var a=e[r][o][0];if("Z"===a)n+="Z";else for(var s=e[r][o].length,l=0;l<s;l++){var u=l;"Q"===a||"S"===a?u=d[l]:"C"===a&&(u=c[l]),n+=e[r][o][u],l>0&&l<s-1&&(n+=",")}}return n},t.readPaths=function(e,t,n,i){var a,s,c,d=r(e),h=[],f=-1,p=0,g=0,v=function(){s=p,c=g};v();for(var m=0;m<d.length;m++){var y,b,C,A,_=[],I=d[m][0],x=I;switch(I){case"M":f++,h[f]=[],p=+d[m][1],g=+d[m][2],_.push([x,p,g]),v();break;case"Q":case"S":y=+d[m][1],C=+d[m][2],p=+d[m][3],g=+d[m][4],_.push([x,p,g,y,C]);break;case"C":y=+d[m][1],C=+d[m][2],b=+d[m][3],A=+d[m][4],p=+d[m][5],g=+d[m][6],_.push([x,p,g,y,C,b,A]);break;case"T":case"L":p=+d[m][1],g=+d[m][2],_.push([x,p,g]);break;case"H":x="L",p=+d[m][1],_.push([x,p,g]);break;case"V":x="L",g=+d[m][1],_.push([x,p,g]);break;case"A":x="L";var w=+d[m][1],M=+d[m][2];+d[m][4]||(w=-w,M=-M);var S=p-w,T=g;for(a=1;a<=o/2;a++){var E=2*Math.PI*a/o;_.push([x,S+w*Math.cos(E),T+M*Math.sin(E)])}break;case"Z":p===s&&g===c||(p=s,g=c,_.push([x,p,g]))}for(var O=(n||{}).domain,k=t._fullLayout._size,L=n&&"pixel"===n.xsizemode,D=n&&"pixel"===n.ysizemode,P=!1===i,N=0;N<_.length;N++){for(a=0;a+2<7;a+=2){var j=_[N][a+1],R=_[N][a+2];void 0!==j&&void 0!==R&&(p=j,g=R,n&&(n.xaxis&&n.xaxis.p2r?(P&&(j-=n.xaxis._offset),j=L?u(n.xaxis,n.xanchor)+j:l(n.xaxis,j)):(P&&(j-=k.l),O?j=O.x[0]+j/k.w:j/=k.w),n.yaxis&&n.yaxis.p2r?(P&&(R-=n.yaxis._offset),R=D?u(n.yaxis,n.yanchor)-R:l(n.yaxis,R)):(P&&(R-=k.t),R=O?O.y[1]-R/k.h:1-R/k.h)),_[N][a+1]=j,_[N][a+2]=R)}h[f].push(_[N].slice())}}return h},t.pointsShapeRectangle=function(e){if(5!==e.length)return!1;for(var t=1;t<3;t++){if(!h(e[0][t]-e[1][t],e[3][t]-e[2][t]))return!1;if(!h(e[0][t]-e[3][t],e[1][t]-e[2][t]))return!1}return!(!h(e[0][1],e[1][1])&&!h(e[0][1],e[3][1]))&&!!(f(e[0],e[1])*f(e[0],e[3]))},t.pointsShapeEllipse=function(e){var t=e.length;if(t!==o+1)return!1;t=o;for(var n=0;n<t;n++){var r=(2*t-n)%t,i=(t/2+r)%t,a=(t/2+n)%t;if(!h(f(e[n],e[a]),f(e[r],e[i])))return!1}return!0},t.handleEllipse=function(e,n,r){if(!e)return[n,r];var i=t.ellipseOver({x0:n[0],y0:n[1],x1:r[0],y1:r[1]}),s=(i.x1+i.x0)/2,l=(i.y1+i.y0)/2,u=(i.x1-i.x0)/2,c=(i.y1-i.y0)/2;u||(u=c/=a),c||(c=u/=a);for(var d=[],h=0;h<o;h++){var f=2*h*Math.PI/o;d.push([s+u*Math.cos(f),l+c*Math.sin(f)])}return d},t.ellipseOver=function(e){var t=e.x0,n=e.y0,r=e.x1,i=e.y1,o=r-t,s=i-n,l=((t-=o)+r)/2,u=((n-=s)+i)/2;return{x0:l-(o*=a),y0:u-(s*=a),x1:l+o,y1:u+s}}},function(e,t,n){"use strict";var r=n(10),i=n(386),o=n(83).id2name,a=n(134),s=n(1004),l=n(300),u=n(72).ALMOST_EQUAL,c=n(97).FROM_BL;function d(e,t,n){var i=n.axIds,s=n.layoutOut,l=n.hasImage,u=s._axisConstraintGroups,c=s._axisMatchGroups,d=t._id,g=d.charAt(0),v=((s._splomAxes||{})[g]||{})[d]||{},m=t._id,y="x"===m.charAt(0);function b(n,i){return r.coerce(e,t,a,n,i)}t._matchGroup=null,t._constraintGroup=null,b("constrain",l?"domain":"range"),r.coerce(e,t,{constraintoward:{valType:"enumerated",values:y?["left","center","right"]:["bottom","middle","top"],dflt:y?"center":"middle"}},"constraintoward");var C,A,_=t.type,I=[];for(C=0;C<i.length;C++){if((A=i[C])!==m)s[o(A)].type===_&&I.push(A)}var x=f(u,m);if(x){var w=[];for(C=0;C<I.length;C++)x[A=I[C]]||w.push(A);I=w}var M,S,T=I.length;T&&(e.matches||v.matches)&&(M=r.coerce(e,t,{matches:{valType:"enumerated",values:I,dflt:-1!==I.indexOf(v.matches)?v.matches:void 0}},"matches"));var E=l&&!y?t.anchor:void 0;if(T&&!M&&(e.scaleanchor||E)&&(S=r.coerce(e,t,{scaleanchor:{valType:"enumerated",values:I}},"scaleanchor",E)),M){t._matchGroup=p(c,m,M,1);var O=s[o(M)],k=h(s,t)/h(s,O);y!==("x"===M.charAt(0))&&(k=(y?"x":"y")+k),p(u,m,M,k)}else e.matches&&-1!==i.indexOf(e.matches)&&r.warn("ignored "+t._name+'.matches: "'+e.matches+'" to avoid an infinite loop');if(S){var L=b("scaleratio");L||(L=t.scaleratio=1),p(u,m,S,L)}else e.scaleanchor&&-1!==i.indexOf(e.scaleanchor)&&r.warn("ignored "+t._name+'.scaleanchor: "'+e.scaleanchor+'" to avoid either an infinite loop and possibly inconsistent scaleratios, or because this axis declares a *matches* constraint.')}function h(e,t){var n=t.domain;return n||(n=e[o(t.overlaying)].domain),n[1]-n[0]}function f(e,t){for(var n=0;n<e.length;n++)if(e[n][t])return e[n];return null}function p(e,t,n,r){var i,o,a,s,l,u=f(e,t);null===u?((u={})[t]=1,l=e.length,e.push(u)):l=e.indexOf(u);var c=Object.keys(u);for(i=0;i<e.length;i++)if(a=e[i],i!==l&&a[n]){var d=a[n];for(o=0;o<c.length;o++)a[s=c[o]]=g(d,g(r,u[s]));return void e.splice(l,1)}if(1!==r)for(o=0;o<c.length;o++){var h=c[o];u[h]=g(r,u[h])}u[n]=1}function g(e,t){var n,r,i="",o="";"string"==typeof e&&(n=(i=e.match(/^[xy]*/)[0]).length,e=+e.substr(n)),"string"==typeof t&&(r=(o=t.match(/^[xy]*/)[0]).length,t=+t.substr(r));var a=e*t;return n||r?n&&r&&i.charAt(0)!==o.charAt(0)?n===r?a:(n>r?i.substr(r):o.substr(n))+a:i+o+e*t:a}function v(e,t){for(var n=t._size,r=n.h/n.w,i={},o=Object.keys(e),a=0;a<o.length;a++){var s=o[a],l=e[s];if("string"==typeof l){var u=l.match(/^[xy]*/)[0],c=u.length;l=+l.substr(c);for(var d="y"===u.charAt(0)?r:1/r,h=0;h<c;h++)l*=d}i[s]=l}return i}function m(e,t){var n=e._inputDomain,r=c[e.constraintoward],i=n[0]+(n[1]-n[0])*r;e.domain=e._input.domain=[i+(n[0]-i)/t,i+(n[1]-i)/t],e.setScale()}t.handleDefaults=function(e,t,n){var i,a,s,u,c,h,f,p,g=n.axIds,v=n.axHasImage,m=t._axisConstraintGroups=[],y=t._axisMatchGroups=[];for(i=0;i<g.length;i++)d(c=e[u=o(g[i])],h=t[u],{axIds:g,layoutOut:t,hasImage:v[u]});function b(e,n){for(i=0;i<e.length;i++)for(s in a=e[i])t[o(s)][n]=a}for(b(y,"_matchGroup"),i=0;i<m.length;i++)for(s in a=m[i])if((h=t[o(s)]).fixedrange){for(var C in a){var A=o(C);!1===(e[A]||{}).fixedrange&&r.warn("fixedrange was specified as false for axis "+A+" but was overridden because another axis in its constraint group has fixedrange true"),t[A].fixedrange=!0}break}for(i=0;i<m.length;){for(s in a=m[i]){(h=t[o(s)])._matchGroup&&Object.keys(h._matchGroup).length===Object.keys(a).length&&(m.splice(i,1),i--);break}i++}b(m,"_constraintGroup");var _=["constrain","range","autorange","rangemode","rangebreaks","categoryorder","categoryarray"],I=!1,x=!1;function w(){p=h[f],"rangebreaks"===f&&(x=h._hasDayOfWeekBreaks)}for(i=0;i<y.length;i++){a=y[i];for(var M=0;M<_.length;M++){var S;for(s in f=_[M],p=null,a)if(c=e[u=o(s)],h=t[u],f in h){if(!h.matches&&(S=h,f in c)){w();break}null===p&&f in c&&w()}if("range"===f&&p&&(I=!0),"autorange"===f&&null===p&&I&&(p=!1),null===p&&f in S&&(p=S[f]),null!==p)for(s in a)(h=t[o(s)])[f]="range"===f?p.slice():p,"rangebreaks"===f&&(h._hasDayOfWeekBreaks=x,l(h,t))}}},t.enforce=function(e){var t,n,r,a,l,c,d,h,f=e._fullLayout,p=f._axisConstraintGroups||[];for(t=0;t<p.length;t++){r=v(p[t],f);var g=Object.keys(r),y=1/0,b=0,C=1/0,A={},_={},I=!1;for(n=0;n<g.length;n++)_[a=g[n]]=l=f[o(a)],l._inputDomain?l.domain=l._inputDomain.slice():l._inputDomain=l.domain.slice(),l._inputRange||(l._inputRange=l.range.slice()),l.setScale(),A[a]=c=Math.abs(l._m)/r[a],y=Math.min(y,c),"domain"!==l.constrain&&l._constraintShrinkable||(C=Math.min(C,c)),delete l._constraintShrinkable,b=Math.max(b,c),"domain"===l.constrain&&(I=!0);if(!(y>u*b)||I)for(n=0;n<g.length;n++)if(c=A[a=g[n]],d=(l=_[a]).constrain,c!==C||"domain"===d)if(h=c/C,"range"===d)s(l,h);else{var x=l._inputDomain,w=(l.domain[1]-l.domain[0])/(x[1]-x[0]),M=(l.r2l(l.range[1])-l.r2l(l.range[0]))/(l.r2l(l._inputRange[1])-l.r2l(l._inputRange[0]));if((h/=w)*M<1){l.domain=l._input.domain=x.slice(),s(l,h);continue}if(M<1&&(l.range=l._input.range=l._inputRange.slice(),h*=M),l.autorange){var S=l.r2l(l.range[0]),T=l.r2l(l.range[1]),E=(S+T)/2,O=E,k=E,L=Math.abs(T-E),D=E-L*h*1.0001,P=E+L*h*1.0001,N=i.makePadFn(f,l,0),j=i.makePadFn(f,l,1);m(l,h);var R,z,F=Math.abs(l._m),B=i.concatExtremes(e,l),H=B.min,U=B.max;for(z=0;z<H.length;z++)(R=H[z].val-N(H[z])/F)>D&&R<O&&(O=R);for(z=0;z<U.length;z++)(R=U[z].val+j(U[z])/F)<P&&R>k&&(k=R);h/=(k-O)/(2*L),O=l.l2r(O),k=l.l2r(k),l.range=l._input.range=S<T?[O,k]:[k,O]}m(l,h)}}},t.getAxisGroup=function(e,t){for(var n=e._axisMatchGroups,r=0;r<n.length;r++){if(n[r][t])return"g"+r}return t},t.clean=function(e,t){if(t._inputDomain){for(var n=!1,r=t._id,i=e._fullLayout._axisConstraintGroups,o=0;o<i.length;o++)if(i[o][r]){n=!0;break}n&&"domain"===t.constrain||(t._input.domain=t.domain=t._inputDomain,delete t._inputDomain)}}},function(e,t,n){"use strict";var r=n(1705),i=n(27),o=n(56).dashStyle,a=n(41),s=n(179),l=n(162).makeEventData,u=n(388),c=u.freeMode,d=u.rectMode,h=u.drawMode,f=u.openMode,p=u.selectMode,g=n(1002),v=n(389).handleEllipse,m=n(1003),y=n(10),b=n(1005),C=n(539),A=n(83).getFromId,_=n(548),I=n(549).redrawReglTraces,x=n(95),w=x.MINSELECT,M=b.filter,S=b.tester,T=n(299).clearSelect,E=n(552),O=E.p2r,k=E.axValue,L=E.getTransform;function D(e,t,n,r,i,o,a){var s,l,u,c,d,h,p,v,m,y=t._hoverdata,b=t._fullLayout.clickmode.indexOf("event")>-1,C=[];if(function(e){return e&&Array.isArray(e)&&!0!==e[0].hoverOnBox}(y)){R(e,t,o);var A=function(e,t){var n,r,i=e[0],o=-1,a=[];for(r=0;r<t.length;r++)if(n=t[r],i.fullData._expandedIndex===n.cd[0].trace._expandedIndex){if(!0===i.hoverOnBox)break;void 0!==i.pointNumber?o=i.pointNumber:void 0!==i.binNumber&&(o=i.binNumber,a=i.pointNumbers);break}return{pointNumber:o,pointNumbers:a,searchInfo:n}}(y,s=F(t,n,r,i));if(A.pointNumbers.length>0?function(e,t){var n,r,i,o=[];for(i=0;i<e.length;i++)(n=e[i]).cd[0].trace.selectedpoints&&n.cd[0].trace.selectedpoints.length>0&&o.push(n);if(1===o.length&&o[0]===t.searchInfo&&(r=t.searchInfo.cd[0].trace).selectedpoints.length===t.pointNumbers.length){for(i=0;i<t.pointNumbers.length;i++)if(r.selectedpoints.indexOf(t.pointNumbers[i])<0)return!1;return!0}return!1}(s,A):function(e){var t,n,r,i=0;for(r=0;r<e.length;r++)if(t=e[r],(n=t.cd[0].trace).selectedpoints){if(n.selectedpoints.length>1)return!1;if((i+=n.selectedpoints.length)>1)return!1}return 1===i}(s)&&(h=B(A))){for(a&&a.remove(),m=0;m<s.length;m++)(l=s[m])._module.selectPoints(l,!1);H(t,s),z(o),b&&t.emit("plotly_deselect",null)}else{for(p=e.shiftKey&&(void 0!==h?h:B(A)),u=function(e,t,n){return{pointNumber:e,searchInfo:t,subtract:n}}(A.pointNumber,A.searchInfo,p),c=j(o.selectionDefs.concat([u])),m=0;m<s.length;m++)if(d=U(s[m]._module.selectPoints(s[m],c),s[m]),C.length)for(var _=0;_<d.length;_++)C.push(d[_]);else C=d;if(H(t,s,v={points:C}),u&&o&&o.selectionDefs.push(u),a){var I=o.mergedPolygons,x=f(o.dragmode);g(V(I,x),a,o)}b&&t.emit("plotly_selected",v)}}}function P(e){return"pointNumber"in e&&"searchInfo"in e}function N(e){return{xmin:0,xmax:0,ymin:0,ymax:0,pts:[],contains:function(t,n,r,i){var o=e.searchInfo.cd[0].trace._expandedIndex;return i.cd[0].trace._expandedIndex===o&&r===e.pointNumber},isRect:!1,degenerate:!1,subtract:e.subtract}}function j(e){for(var t=[],n=P(e[0])?0:e[0][0][0],r=n,i=P(e[0])?0:e[0][0][1],o=i,a=0;a<e.length;a++)if(P(e[a]))t.push(N(e[a]));else{var s=b.tester(e[a]);s.subtract=e[a].subtract,t.push(s),n=Math.min(n,s.xmin),r=Math.max(r,s.xmax),i=Math.min(i,s.ymin),o=Math.max(o,s.ymax)}return{xmin:n,xmax:r,ymin:i,ymax:o,pts:[],contains:function(e,n,r,i){for(var o=!1,a=0;a<t.length;a++)t[a].contains(e,n,r,i)&&(o=!1===t[a].subtract);return o},isRect:!1,degenerate:!1}}function R(e,t,n){t._fullLayout._drawing=!1;var r=t._fullLayout,i=n.plotinfo,o=n.dragmode,a=r._lastSelectedSubplot&&r._lastSelectedSubplot===i.id,s=(e.shiftKey||e.altKey)&&!(h(o)&&f(o));a&&s&&i.selection&&i.selection.selectionDefs&&!n.selectionDefs?(n.selectionDefs=i.selection.selectionDefs,n.mergedPolygons=i.selection.mergedPolygons):s&&i.selection||z(n),a||(T(t),r._lastSelectedSubplot=i.id)}function z(e){var t=e.dragmode,n=e.plotinfo,r=e.gd;if(r._fullLayout._activeShapeIndex>=0&&r._fullLayout._deactivateShape(r),h(t)){var o=r._fullLayout._zoomlayer.selectAll(".select-outline-"+n.id);if(o&&r._fullLayout._drawing){var a=m(o,e);a&&i.call("_guiRelayout",r,{shapes:a}),r._fullLayout._drawing=!1}}n.selection={},n.selection.selectionDefs=e.selectionDefs=[],n.selection.mergedPolygons=e.mergedPolygons=[]}function F(e,t,n,r){var i,o,a,s=[],l=t.map((function(e){return e._id})),u=n.map((function(e){return e._id}));for(a=0;a<e.calcdata.length;a++)if(!0===(o=(i=e.calcdata[a])[0].trace).visible&&o._module&&o._module.selectPoints)if(!r||o.subplot!==r&&o.geo!==r)if("splom"===o.type&&o._xaxes[l[0]]&&o._yaxes[u[0]]){var c=h(o._module,i,t[0],n[0]);c.scene=e._fullLayout._splomScenes[o.uid],s.push(c)}else if("sankey"===o.type){var d=h(o._module,i,t[0],n[0]);s.push(d)}else{if(-1===l.indexOf(o.xaxis))continue;if(-1===u.indexOf(o.yaxis))continue;s.push(h(o._module,i,A(e,o.xaxis),A(e,o.yaxis)))}else s.push(h(o._module,i,t[0],n[0]));return s;function h(e,t,n,r){return{_module:e,cd:t,xaxis:n,yaxis:r}}}function B(e){var t=e.searchInfo.cd[0].trace,n=e.pointNumber,r=e.pointNumbers,i=r.length>0?r[0]:n;return!!t.selectedpoints&&t.selectedpoints.indexOf(i)>-1}function H(e,t,n){var r,o,a,s;for(r=0;r<t.length;r++){var l=t[r].cd[0].trace._fullInput,u=e._fullLayout._tracePreGUI[l.uid]||{};void 0===u.selectedpoints&&(u.selectedpoints=l._input.selectedpoints||null)}if(n){var c=n.points||[];for(r=0;r<t.length;r++)(s=t[r].cd[0].trace)._input.selectedpoints=s._fullInput.selectedpoints=[],s._fullInput!==s&&(s.selectedpoints=[]);for(r=0;r<c.length;r++){var d=c[r],h=d.data,f=d.fullData;d.pointIndices?([].push.apply(h.selectedpoints,d.pointIndices),s._fullInput!==s&&[].push.apply(f.selectedpoints,d.pointIndices)):(h.selectedpoints.push(d.pointIndex),s._fullInput!==s&&f.selectedpoints.push(d.pointIndex))}}else for(r=0;r<t.length;r++)delete(s=t[r].cd[0].trace).selectedpoints,delete s._input.selectedpoints,s._fullInput!==s&&delete s._fullInput.selectedpoints;var p=!1;for(r=0;r<t.length;r++){s=(a=(o=t[r]).cd)[0].trace,i.traceIs(s,"regl")&&(p=!0);var g=o._module,v=g.styleOnSelect||g.style;v&&(v(e,a,a[0].node3),a[0].nodeRangePlot3&&v(e,a,a[0].nodeRangePlot3))}p&&(_(e),I(e))}function U(e,t){if(Array.isArray(e))for(var n=t.cd,r=t.cd[0].trace,i=0;i<e.length;i++)e[i]=l(e[i],r,n);return e}function V(e,t){for(var n=[],r=0;r<e.length;r++){n[r]=[];for(var i=0;i<e[r].length;i++){n[r][i]=[],n[r][i][0]=i?"L":"M";for(var o=0;o<e[r][i].length;o++)n[r][i].push(e[r][i][o])}t||n[r].push(["Z",n[r][0][1],n[r][0][2]])}return n}e.exports={prepSelect:function(e,t,n,i,l){var u=c(l),m=d(l),b=f(l),A=h(l),_=p(l),I="drawcircle"===l,T="drawline"===l||I,E=i.gd,P=E._fullLayout,N=P._zoomlayer,B=i.element.getBoundingClientRect(),Y=i.plotinfo,W=L(Y),G=t-B.left,X=n-B.top;P._calcInverseTransform(E);var q=y.apply3DTransform(P._invTransform)(G,X);G=q[0],X=q[1];var Z,Q,K,J,$,ee,te,ne=P._invScaleX,re=P._invScaleY,ie=G,oe=X,ae="M"+G+","+X,se=i.xaxes[0]._length,le=i.yaxes[0]._length,ue=i.xaxes.concat(i.yaxes),ce=e.altKey&&!(h(l)&&b);R(e,E,i),u&&(Z=M([[G,X]],x.BENDPX));var de=N.selectAll("path.select-outline-"+Y.id).data(A?[0]:[1,2]),he=P.newshape;de.enter().append("path").attr("class",(function(e){return"select-outline select-outline-"+e+" select-outline-"+Y.id})).style(A?{opacity:he.opacity/2,fill:b?void 0:he.fillcolor,stroke:he.line.color,"stroke-dasharray":o(he.line.dash,he.line.width),"stroke-width":he.line.width+"px"}:{}).attr("fill-rule",he.fillrule).classed("cursor-move",!!A).attr("transform",W).attr("d",ae+"Z");var fe,pe=N.append("path").attr("class","zoombox-corners").style({fill:a.background,stroke:a.defaultLine,"stroke-width":1}).attr("transform",W).attr("d","M0,0Z"),ge=P._uid+x.SELECTID,ve=[],me=F(E,i.xaxes,i.yaxes,i.subplot);function ye(e,t){return e-t}fe=Y.fillRangeItems?Y.fillRangeItems:m?function(e,t){var n=e.range={};for($=0;$<ue.length;$++){var r=ue[$],i=r._id.charAt(0);n[r._id]=[O(r,t[i+"min"]),O(r,t[i+"max"])].sort(ye)}}:function(e,t,n){var r=e.lassoPoints={};for($=0;$<ue.length;$++){var i=ue[$];r[i._id]=n.filtered.map(k(i))}},i.moveFn=function(e,t){ie=Math.max(0,Math.min(se,ne*e+G)),oe=Math.max(0,Math.min(le,re*t+X));var n=Math.abs(ie-G),o=Math.abs(oe-X);if(m){var a,s,l;if(_){var c=P.selectdirection;switch(a="any"===c?o<Math.min(.6*n,w)?"h":n<Math.min(.6*o,w)?"v":"d":c){case"h":s=I?le/2:0,l=le;break;case"v":s=I?se/2:0,l=se}}if(A)switch(P.newshape.drawdirection){case"vertical":a="h",s=I?le/2:0,l=le;break;case"horizontal":a="v",s=I?se/2:0,l=se;break;case"ortho":n<o?(a="h",s=X,l=oe):(a="v",s=G,l=ie);break;default:a="d"}"h"===a?((J=T?v(I,[ie,s],[ie,l]):[[G,s],[G,l],[ie,l],[ie,s]]).xmin=T?ie:Math.min(G,ie),J.xmax=T?ie:Math.max(G,ie),J.ymin=Math.min(s,l),J.ymax=Math.max(s,l),pe.attr("d","M"+J.xmin+","+(X-w)+"h-4v"+2*w+"h4ZM"+(J.xmax-1)+","+(X-w)+"h4v"+2*w+"h-4Z")):"v"===a?((J=T?v(I,[s,oe],[l,oe]):[[s,X],[s,oe],[l,oe],[l,X]]).xmin=Math.min(s,l),J.xmax=Math.max(s,l),J.ymin=T?oe:Math.min(X,oe),J.ymax=T?oe:Math.max(X,oe),pe.attr("d","M"+(G-w)+","+J.ymin+"v-4h"+2*w+"v4ZM"+(G-w)+","+(J.ymax-1)+"v4h"+2*w+"v-4Z")):"d"===a&&((J=T?v(I,[G,X],[ie,oe]):[[G,X],[G,oe],[ie,oe],[ie,X]]).xmin=Math.min(G,ie),J.xmax=Math.max(G,ie),J.ymin=Math.min(X,oe),J.ymax=Math.max(X,oe),pe.attr("d","M0,0Z"))}else u&&(Z.addPt([ie,oe]),J=Z.filtered);i.selectionDefs&&i.selectionDefs.length?(K=function(e,t,n){if(n)return r.difference({regions:e,inverted:!1},{regions:[t],inverted:!1}).regions;return r.union({regions:e,inverted:!1},{regions:[t],inverted:!1}).regions}(i.mergedPolygons,J,ce),J.subtract=ce,Q=j(i.selectionDefs.concat([J]))):(K=[J],Q=S(J)),g(V(K,b),de,i),_&&C.throttle(ge,x.SELECTDELAY,(function(){var e;ve=[];var t,n=[];for($=0;$<me.length;$++)if(t=(ee=me[$])._module.selectPoints(ee,Q),n.push(t),e=U(t,ee),ve.length)for(var r=0;r<e.length;r++)ve.push(e[r]);else ve=e;H(E,me,te={points:ve}),fe(te,J,Z),i.gd.emit("plotly_selecting",te)}))},i.clickFn=function(e,t){if(pe.remove(),E._fullLayout._activeShapeIndex>=0)E._fullLayout._deactivateShape(E);else if(!A){var n=P.clickmode;C.done(ge).then((function(){if(C.clear(ge),2===e){for(de.remove(),$=0;$<me.length;$++)(ee=me[$])._module.selectPoints(ee,!1);H(E,me),z(i),E.emit("plotly_deselect",null)}else n.indexOf("select")>-1&&D(t,E,i.xaxes,i.yaxes,i.subplot,i,de),"event"===n&&E.emit("plotly_selected",void 0);s.click(E,t)})).catch(y.error)}},i.doneFn=function(){pe.remove(),C.done(ge).then((function(){C.clear(ge),i.gd.emit("plotly_selected",te),J&&i.selectionDefs&&(J.subtract=ce,i.selectionDefs.push(J),i.mergedPolygons.length=0,[].push.apply(i.mergedPolygons,K)),i.doneFnCompleted&&i.doneFnCompleted(ve)})).catch(y.error),A&&z(i)}},clearSelect:T,clearSelectionsCache:z,selectOnClick:D}},function(e,t,n){"use strict";var r=n(1021),i=n(94),o=n(95),a=n(64).templatedArray;n(561);e.exports=a("annotation",{visible:{valType:"boolean",dflt:!0,editType:"calc+arraydraw"},text:{valType:"string",editType:"calc+arraydraw"},textangle:{valType:"angle",dflt:0,editType:"calc+arraydraw"},font:i({editType:"calc+arraydraw",colorEditType:"arraydraw"}),width:{valType:"number",min:1,dflt:null,editType:"calc+arraydraw"},height:{valType:"number",min:1,dflt:null,editType:"calc+arraydraw"},opacity:{valType:"number",min:0,max:1,dflt:1,editType:"arraydraw"},align:{valType:"enumerated",values:["left","center","right"],dflt:"center",editType:"arraydraw"},valign:{valType:"enumerated",values:["top","middle","bottom"],dflt:"middle",editType:"arraydraw"},bgcolor:{valType:"color",dflt:"rgba(0,0,0,0)",editType:"arraydraw"},bordercolor:{valType:"color",dflt:"rgba(0,0,0,0)",editType:"arraydraw"},borderpad:{valType:"number",min:0,dflt:1,editType:"calc+arraydraw"},borderwidth:{valType:"number",min:0,dflt:1,editType:"calc+arraydraw"},showarrow:{valType:"boolean",dflt:!0,editType:"calc+arraydraw"},arrowcolor:{valType:"color",editType:"arraydraw"},arrowhead:{valType:"integer",min:0,max:r.length,dflt:1,editType:"arraydraw"},startarrowhead:{valType:"integer",min:0,max:r.length,dflt:1,editType:"arraydraw"},arrowside:{valType:"flaglist",flags:["end","start"],extras:["none"],dflt:"end",editType:"arraydraw"},arrowsize:{valType:"number",min:.3,dflt:1,editType:"calc+arraydraw"},startarrowsize:{valType:"number",min:.3,dflt:1,editType:"calc+arraydraw"},arrowwidth:{valType:"number",min:.1,editType:"calc+arraydraw"},standoff:{valType:"number",min:0,dflt:0,editType:"calc+arraydraw"},startstandoff:{valType:"number",min:0,dflt:0,editType:"calc+arraydraw"},ax:{valType:"any",editType:"calc+arraydraw"},ay:{valType:"any",editType:"calc+arraydraw"},axref:{valType:"enumerated",dflt:"pixel",values:["pixel",o.idRegex.x.toString()],editType:"calc"},ayref:{valType:"enumerated",dflt:"pixel",values:["pixel",o.idRegex.y.toString()],editType:"calc"},xref:{valType:"enumerated",values:["paper",o.idRegex.x.toString()],editType:"calc"},x:{valType:"any",editType:"calc+arraydraw"},xanchor:{valType:"enumerated",values:["auto","left","center","right"],dflt:"auto",editType:"calc+arraydraw"},xshift:{valType:"number",dflt:0,editType:"calc+arraydraw"},yref:{valType:"enumerated",values:["paper",o.idRegex.y.toString()],editType:"calc"},y:{valType:"any",editType:"calc+arraydraw"},yanchor:{valType:"enumerated",values:["auto","top","middle","bottom"],dflt:"auto",editType:"calc+arraydraw"},yshift:{valType:"number",dflt:0,editType:"calc+arraydraw"},clicktoshow:{valType:"enumerated",values:[!1,"onoff","onout"],dflt:!1,editType:"arraydraw"},xclick:{valType:"any",editType:"arraydraw"},yclick:{valType:"any",editType:"arraydraw"},hovertext:{valType:"string",editType:"arraydraw"},hoverlabel:{bgcolor:{valType:"color",editType:"arraydraw"},bordercolor:{valType:"color",editType:"arraydraw"},font:i({editType:"arraydraw"}),editType:"arraydraw"},captureevents:{valType:"boolean",editType:"arraydraw"},editType:"calc",_deprecated:{ref:{valType:"string",editType:"calc"}}})},function(e,t,n){"use strict";e.exports={name:"sliders",containerClassName:"slider-container",groupClassName:"slider-group",inputAreaClass:"slider-input-area",railRectClass:"slider-rail-rect",railTouchRectClass:"slider-rail-touch-rect",gripRectClass:"slider-grip-rect",tickRectClass:"slider-tick-rect",inputProxyClass:"slider-input-proxy",labelsClass:"slider-labels",labelGroupClass:"slider-label-group",labelClass:"slider-label",currentValueClass:"slider-current-value",railHeight:5,menuIndexAttrName:"slider-active-index",autoMarginIdRoot:"slider-",minWidth:30,minHeight:30,textPadX:40,arrowOffsetX:4,railRadius:2,railWidth:5,railBorder:4,railBorderWidth:1,railBorderColor:"#bec8d9",railBgColor:"#f8fafc",railInset:8,stepInset:10,gripRadius:10,gripWidth:20,gripHeight:20,gripBorder:20,gripBorderWidth:1,gripBorderColor:"#bec8d9",gripBgColor:"#f6f8fa",gripBgActiveColor:"#dbdde0",labelPadding:8,labelOffset:0,tickWidth:1,tickColor:"#333",tickOffset:25,tickLength:7,minorTickOffset:25,minorTickColor:"#333",minorTickLength:4,currentValuePadding:8,currentValueInset:0}},function(e,t,n){"use strict";e.exports=function(e,t,n){return 0===e.length?e:t?(n||e.sort(t),function(e,t){for(var n=1,r=e.length,i=e[0],o=e[0],a=1;a<r;++a)if(o=i,t(i=e[a],o)){if(a===n){n++;continue}e[n++]=i}return e.length=n,e}(e,t)):(n||e.sort(),function(e){for(var t=1,n=e.length,r=e[0],i=e[0],o=1;o<n;++o,i=r)if(i=r,(r=e[o])!==i){if(o===t){t++;continue}e[t++]=r}return e.length=t,e}(e))}},function(e,t){function n(e,t,n){this.shortMessage=t||"",this.longMessage=n||"",this.rawError=e||"",this.message="gl-shader: "+(t||e||"")+(n?"\n"+n:""),this.stack=(new Error).stack}n.prototype=new Error,n.prototype.name="GLError",n.prototype.constructor=n,e.exports=n},function(e,t,n){"use strict";var r=n(257),i=n(1042);e.exports=function(e,t){var n=e.length;if(1===n){var o=r(e[0],t);return o[0]?o:[o[1]]}var a=new Array(2*n),s=[.1,.1],l=[.1,.1],u=0;r(e[0],t,s),s[0]&&(a[u++]=s[0]);for(var c=1;c<n;++c){r(e[c],t,l);var d=s[1];i(d,l[0],s),s[0]&&(a[u++]=s[0]);var h=l[1],f=s[1],p=h+f,g=f-(p-h);s[1]=p,g&&(a[u++]=g)}s[1]&&(a[u++]=s[1]);0===u&&(a[u++]=0);return a.length=u,a}},function(e,t,n){(function(e){!function(e,t){"use strict";function r(e,t){if(!e)throw new Error(t||"Assertion failed")}function i(e,t){e.super_=t;var n=function(){};n.prototype=t.prototype,e.prototype=new n,e.prototype.constructor=e}function o(e,t,n){if(o.isBN(e))return e;this.negative=0,this.words=null,this.length=0,this.red=null,null!==e&&("le"!==t&&"be"!==t||(n=t,t=10),this._init(e||0,t||10,n||"be"))}var a;"object"==typeof e?e.exports=o:t.BN=o,o.BN=o,o.wordSize=26;try{a=n(1845).Buffer}catch(e){}function s(e,t,n){for(var r=0,i=Math.min(e.length,n),o=t;o<i;o++){var a=e.charCodeAt(o)-48;r<<=4,r|=a>=49&&a<=54?a-49+10:a>=17&&a<=22?a-17+10:15&a}return r}function l(e,t,n,r){for(var i=0,o=Math.min(e.length,n),a=t;a<o;a++){var s=e.charCodeAt(a)-48;i*=r,i+=s>=49?s-49+10:s>=17?s-17+10:s}return i}o.isBN=function(e){return e instanceof o||null!==e&&"object"==typeof e&&e.constructor.wordSize===o.wordSize&&Array.isArray(e.words)},o.max=function(e,t){return e.cmp(t)>0?e:t},o.min=function(e,t){return e.cmp(t)<0?e:t},o.prototype._init=function(e,t,n){if("number"==typeof e)return this._initNumber(e,t,n);if("object"==typeof e)return this._initArray(e,t,n);"hex"===t&&(t=16),r(t===(0|t)&&t>=2&&t<=36);var i=0;"-"===(e=e.toString().replace(/\s+/g,""))[0]&&i++,16===t?this._parseHex(e,i):this._parseBase(e,t,i),"-"===e[0]&&(this.negative=1),this.strip(),"le"===n&&this._initArray(this.toArray(),t,n)},o.prototype._initNumber=function(e,t,n){e<0&&(this.negative=1,e=-e),e<67108864?(this.words=[67108863&e],this.length=1):e<4503599627370496?(this.words=[67108863&e,e/67108864&67108863],this.length=2):(r(e<9007199254740992),this.words=[67108863&e,e/67108864&67108863,1],this.length=3),"le"===n&&this._initArray(this.toArray(),t,n)},o.prototype._initArray=function(e,t,n){if(r("number"==typeof e.length),e.length<=0)return this.words=[0],this.length=1,this;this.length=Math.ceil(e.length/3),this.words=new Array(this.length);for(var i=0;i<this.length;i++)this.words[i]=0;var o,a,s=0;if("be"===n)for(i=e.length-1,o=0;i>=0;i-=3)a=e[i]|e[i-1]<<8|e[i-2]<<16,this.words[o]|=a<<s&67108863,this.words[o+1]=a>>>26-s&67108863,(s+=24)>=26&&(s-=26,o++);else if("le"===n)for(i=0,o=0;i<e.length;i+=3)a=e[i]|e[i+1]<<8|e[i+2]<<16,this.words[o]|=a<<s&67108863,this.words[o+1]=a>>>26-s&67108863,(s+=24)>=26&&(s-=26,o++);return this.strip()},o.prototype._parseHex=function(e,t){this.length=Math.ceil((e.length-t)/6),this.words=new Array(this.length);for(var n=0;n<this.length;n++)this.words[n]=0;var r,i,o=0;for(n=e.length-6,r=0;n>=t;n-=6)i=s(e,n,n+6),this.words[r]|=i<<o&67108863,this.words[r+1]|=i>>>26-o&4194303,(o+=24)>=26&&(o-=26,r++);n+6!==t&&(i=s(e,t,n+6),this.words[r]|=i<<o&67108863,this.words[r+1]|=i>>>26-o&4194303),this.strip()},o.prototype._parseBase=function(e,t,n){this.words=[0],this.length=1;for(var r=0,i=1;i<=67108863;i*=t)r++;r--,i=i/t|0;for(var o=e.length-n,a=o%r,s=Math.min(o,o-a)+n,u=0,c=n;c<s;c+=r)u=l(e,c,c+r,t),this.imuln(i),this.words[0]+u<67108864?this.words[0]+=u:this._iaddn(u);if(0!==a){var d=1;for(u=l(e,c,e.length,t),c=0;c<a;c++)d*=t;this.imuln(d),this.words[0]+u<67108864?this.words[0]+=u:this._iaddn(u)}},o.prototype.copy=function(e){e.words=new Array(this.length);for(var t=0;t<this.length;t++)e.words[t]=this.words[t];e.length=this.length,e.negative=this.negative,e.red=this.red},o.prototype.clone=function(){var e=new o(null);return this.copy(e),e},o.prototype._expand=function(e){for(;this.length<e;)this.words[this.length++]=0;return this},o.prototype.strip=function(){for(;this.length>1&&0===this.words[this.length-1];)this.length--;return this._normSign()},o.prototype._normSign=function(){return 1===this.length&&0===this.words[0]&&(this.negative=0),this},o.prototype.inspect=function(){return(this.red?"<BN-R: ":"<BN: ")+this.toString(16)+">"};var u=["","0","00","000","0000","00000","000000","0000000","00000000","000000000","0000000000","00000000000","000000000000","0000000000000","00000000000000","000000000000000","0000000000000000","00000000000000000","000000000000000000","0000000000000000000","00000000000000000000","000000000000000000000","0000000000000000000000","00000000000000000000000","000000000000000000000000","0000000000000000000000000"],c=[0,0,25,16,12,11,10,9,8,8,7,7,7,7,6,6,6,6,6,6,6,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5],d=[0,0,33554432,43046721,16777216,48828125,60466176,40353607,16777216,43046721,1e7,19487171,35831808,62748517,7529536,11390625,16777216,24137569,34012224,47045881,64e6,4084101,5153632,6436343,7962624,9765625,11881376,14348907,17210368,20511149,243e5,28629151,33554432,39135393,45435424,52521875,60466176];function h(e,t,n){n.negative=t.negative^e.negative;var r=e.length+t.length|0;n.length=r,r=r-1|0;var i=0|e.words[0],o=0|t.words[0],a=i*o,s=67108863&a,l=a/67108864|0;n.words[0]=s;for(var u=1;u<r;u++){for(var c=l>>>26,d=67108863&l,h=Math.min(u,t.length-1),f=Math.max(0,u-e.length+1);f<=h;f++){var p=u-f|0;c+=(a=(i=0|e.words[p])*(o=0|t.words[f])+d)/67108864|0,d=67108863&a}n.words[u]=0|d,l=0|c}return 0!==l?n.words[u]=0|l:n.length--,n.strip()}o.prototype.toString=function(e,t){var n;if(t=0|t||1,16===(e=e||10)||"hex"===e){n="";for(var i=0,o=0,a=0;a<this.length;a++){var s=this.words[a],l=(16777215&(s<<i|o)).toString(16);n=0!==(o=s>>>24-i&16777215)||a!==this.length-1?u[6-l.length]+l+n:l+n,(i+=2)>=26&&(i-=26,a--)}for(0!==o&&(n=o.toString(16)+n);n.length%t!=0;)n="0"+n;return 0!==this.negative&&(n="-"+n),n}if(e===(0|e)&&e>=2&&e<=36){var h=c[e],f=d[e];n="";var p=this.clone();for(p.negative=0;!p.isZero();){var g=p.modn(f).toString(e);n=(p=p.idivn(f)).isZero()?g+n:u[h-g.length]+g+n}for(this.isZero()&&(n="0"+n);n.length%t!=0;)n="0"+n;return 0!==this.negative&&(n="-"+n),n}r(!1,"Base should be between 2 and 36")},o.prototype.toNumber=function(){var e=this.words[0];return 2===this.length?e+=67108864*this.words[1]:3===this.length&&1===this.words[2]?e+=4503599627370496+67108864*this.words[1]:this.length>2&&r(!1,"Number can only safely store up to 53 bits"),0!==this.negative?-e:e},o.prototype.toJSON=function(){return this.toString(16)},o.prototype.toBuffer=function(e,t){return r(void 0!==a),this.toArrayLike(a,e,t)},o.prototype.toArray=function(e,t){return this.toArrayLike(Array,e,t)},o.prototype.toArrayLike=function(e,t,n){var i=this.byteLength(),o=n||Math.max(1,i);r(i<=o,"byte array longer than desired length"),r(o>0,"Requested array length <= 0"),this.strip();var a,s,l="le"===t,u=new e(o),c=this.clone();if(l){for(s=0;!c.isZero();s++)a=c.andln(255),c.iushrn(8),u[s]=a;for(;s<o;s++)u[s]=0}else{for(s=0;s<o-i;s++)u[s]=0;for(s=0;!c.isZero();s++)a=c.andln(255),c.iushrn(8),u[o-s-1]=a}return u},Math.clz32?o.prototype._countBits=function(e){return 32-Math.clz32(e)}:o.prototype._countBits=function(e){var t=e,n=0;return t>=4096&&(n+=13,t>>>=13),t>=64&&(n+=7,t>>>=7),t>=8&&(n+=4,t>>>=4),t>=2&&(n+=2,t>>>=2),n+t},o.prototype._zeroBits=function(e){if(0===e)return 26;var t=e,n=0;return 0==(8191&t)&&(n+=13,t>>>=13),0==(127&t)&&(n+=7,t>>>=7),0==(15&t)&&(n+=4,t>>>=4),0==(3&t)&&(n+=2,t>>>=2),0==(1&t)&&n++,n},o.prototype.bitLength=function(){var e=this.words[this.length-1],t=this._countBits(e);return 26*(this.length-1)+t},o.prototype.zeroBits=function(){if(this.isZero())return 0;for(var e=0,t=0;t<this.length;t++){var n=this._zeroBits(this.words[t]);if(e+=n,26!==n)break}return e},o.prototype.byteLength=function(){return Math.ceil(this.bitLength()/8)},o.prototype.toTwos=function(e){return 0!==this.negative?this.abs().inotn(e).iaddn(1):this.clone()},o.prototype.fromTwos=function(e){return this.testn(e-1)?this.notn(e).iaddn(1).ineg():this.clone()},o.prototype.isNeg=function(){return 0!==this.negative},o.prototype.neg=function(){return this.clone().ineg()},o.prototype.ineg=function(){return this.isZero()||(this.negative^=1),this},o.prototype.iuor=function(e){for(;this.length<e.length;)this.words[this.length++]=0;for(var t=0;t<e.length;t++)this.words[t]=this.words[t]|e.words[t];return this.strip()},o.prototype.ior=function(e){return r(0==(this.negative|e.negative)),this.iuor(e)},o.prototype.or=function(e){return this.length>e.length?this.clone().ior(e):e.clone().ior(this)},o.prototype.uor=function(e){return this.length>e.length?this.clone().iuor(e):e.clone().iuor(this)},o.prototype.iuand=function(e){var t;t=this.length>e.length?e:this;for(var n=0;n<t.length;n++)this.words[n]=this.words[n]&e.words[n];return this.length=t.length,this.strip()},o.prototype.iand=function(e){return r(0==(this.negative|e.negative)),this.iuand(e)},o.prototype.and=function(e){return this.length>e.length?this.clone().iand(e):e.clone().iand(this)},o.prototype.uand=function(e){return this.length>e.length?this.clone().iuand(e):e.clone().iuand(this)},o.prototype.iuxor=function(e){var t,n;this.length>e.length?(t=this,n=e):(t=e,n=this);for(var r=0;r<n.length;r++)this.words[r]=t.words[r]^n.words[r];if(this!==t)for(;r<t.length;r++)this.words[r]=t.words[r];return this.length=t.length,this.strip()},o.prototype.ixor=function(e){return r(0==(this.negative|e.negative)),this.iuxor(e)},o.prototype.xor=function(e){return this.length>e.length?this.clone().ixor(e):e.clone().ixor(this)},o.prototype.uxor=function(e){return this.length>e.length?this.clone().iuxor(e):e.clone().iuxor(this)},o.prototype.inotn=function(e){r("number"==typeof e&&e>=0);var t=0|Math.ceil(e/26),n=e%26;this._expand(t),n>0&&t--;for(var i=0;i<t;i++)this.words[i]=67108863&~this.words[i];return n>0&&(this.words[i]=~this.words[i]&67108863>>26-n),this.strip()},o.prototype.notn=function(e){return this.clone().inotn(e)},o.prototype.setn=function(e,t){r("number"==typeof e&&e>=0);var n=e/26|0,i=e%26;return this._expand(n+1),this.words[n]=t?this.words[n]|1<<i:this.words[n]&~(1<<i),this.strip()},o.prototype.iadd=function(e){var t,n,r;if(0!==this.negative&&0===e.negative)return this.negative=0,t=this.isub(e),this.negative^=1,this._normSign();if(0===this.negative&&0!==e.negative)return e.negative=0,t=this.isub(e),e.negative=1,t._normSign();this.length>e.length?(n=this,r=e):(n=e,r=this);for(var i=0,o=0;o<r.length;o++)t=(0|n.words[o])+(0|r.words[o])+i,this.words[o]=67108863&t,i=t>>>26;for(;0!==i&&o<n.length;o++)t=(0|n.words[o])+i,this.words[o]=67108863&t,i=t>>>26;if(this.length=n.length,0!==i)this.words[this.length]=i,this.length++;else if(n!==this)for(;o<n.length;o++)this.words[o]=n.words[o];return this},o.prototype.add=function(e){var t;return 0!==e.negative&&0===this.negative?(e.negative=0,t=this.sub(e),e.negative^=1,t):0===e.negative&&0!==this.negative?(this.negative=0,t=e.sub(this),this.negative=1,t):this.length>e.length?this.clone().iadd(e):e.clone().iadd(this)},o.prototype.isub=function(e){if(0!==e.negative){e.negative=0;var t=this.iadd(e);return e.negative=1,t._normSign()}if(0!==this.negative)return this.negative=0,this.iadd(e),this.negative=1,this._normSign();var n,r,i=this.cmp(e);if(0===i)return this.negative=0,this.length=1,this.words[0]=0,this;i>0?(n=this,r=e):(n=e,r=this);for(var o=0,a=0;a<r.length;a++)o=(t=(0|n.words[a])-(0|r.words[a])+o)>>26,this.words[a]=67108863&t;for(;0!==o&&a<n.length;a++)o=(t=(0|n.words[a])+o)>>26,this.words[a]=67108863&t;if(0===o&&a<n.length&&n!==this)for(;a<n.length;a++)this.words[a]=n.words[a];return this.length=Math.max(this.length,a),n!==this&&(this.negative=1),this.strip()},o.prototype.sub=function(e){return this.clone().isub(e)};var f=function(e,t,n){var r,i,o,a=e.words,s=t.words,l=n.words,u=0,c=0|a[0],d=8191&c,h=c>>>13,f=0|a[1],p=8191&f,g=f>>>13,v=0|a[2],m=8191&v,y=v>>>13,b=0|a[3],C=8191&b,A=b>>>13,_=0|a[4],I=8191&_,x=_>>>13,w=0|a[5],M=8191&w,S=w>>>13,T=0|a[6],E=8191&T,O=T>>>13,k=0|a[7],L=8191&k,D=k>>>13,P=0|a[8],N=8191&P,j=P>>>13,R=0|a[9],z=8191&R,F=R>>>13,B=0|s[0],H=8191&B,U=B>>>13,V=0|s[1],Y=8191&V,W=V>>>13,G=0|s[2],X=8191&G,q=G>>>13,Z=0|s[3],Q=8191&Z,K=Z>>>13,J=0|s[4],$=8191&J,ee=J>>>13,te=0|s[5],ne=8191&te,re=te>>>13,ie=0|s[6],oe=8191&ie,ae=ie>>>13,se=0|s[7],le=8191&se,ue=se>>>13,ce=0|s[8],de=8191&ce,he=ce>>>13,fe=0|s[9],pe=8191&fe,ge=fe>>>13;n.negative=e.negative^t.negative,n.length=19;var ve=(u+(r=Math.imul(d,H))|0)+((8191&(i=(i=Math.imul(d,U))+Math.imul(h,H)|0))<<13)|0;u=((o=Math.imul(h,U))+(i>>>13)|0)+(ve>>>26)|0,ve&=67108863,r=Math.imul(p,H),i=(i=Math.imul(p,U))+Math.imul(g,H)|0,o=Math.imul(g,U);var me=(u+(r=r+Math.imul(d,Y)|0)|0)+((8191&(i=(i=i+Math.imul(d,W)|0)+Math.imul(h,Y)|0))<<13)|0;u=((o=o+Math.imul(h,W)|0)+(i>>>13)|0)+(me>>>26)|0,me&=67108863,r=Math.imul(m,H),i=(i=Math.imul(m,U))+Math.imul(y,H)|0,o=Math.imul(y,U),r=r+Math.imul(p,Y)|0,i=(i=i+Math.imul(p,W)|0)+Math.imul(g,Y)|0,o=o+Math.imul(g,W)|0;var ye=(u+(r=r+Math.imul(d,X)|0)|0)+((8191&(i=(i=i+Math.imul(d,q)|0)+Math.imul(h,X)|0))<<13)|0;u=((o=o+Math.imul(h,q)|0)+(i>>>13)|0)+(ye>>>26)|0,ye&=67108863,r=Math.imul(C,H),i=(i=Math.imul(C,U))+Math.imul(A,H)|0,o=Math.imul(A,U),r=r+Math.imul(m,Y)|0,i=(i=i+Math.imul(m,W)|0)+Math.imul(y,Y)|0,o=o+Math.imul(y,W)|0,r=r+Math.imul(p,X)|0,i=(i=i+Math.imul(p,q)|0)+Math.imul(g,X)|0,o=o+Math.imul(g,q)|0;var be=(u+(r=r+Math.imul(d,Q)|0)|0)+((8191&(i=(i=i+Math.imul(d,K)|0)+Math.imul(h,Q)|0))<<13)|0;u=((o=o+Math.imul(h,K)|0)+(i>>>13)|0)+(be>>>26)|0,be&=67108863,r=Math.imul(I,H),i=(i=Math.imul(I,U))+Math.imul(x,H)|0,o=Math.imul(x,U),r=r+Math.imul(C,Y)|0,i=(i=i+Math.imul(C,W)|0)+Math.imul(A,Y)|0,o=o+Math.imul(A,W)|0,r=r+Math.imul(m,X)|0,i=(i=i+Math.imul(m,q)|0)+Math.imul(y,X)|0,o=o+Math.imul(y,q)|0,r=r+Math.imul(p,Q)|0,i=(i=i+Math.imul(p,K)|0)+Math.imul(g,Q)|0,o=o+Math.imul(g,K)|0;var Ce=(u+(r=r+Math.imul(d,$)|0)|0)+((8191&(i=(i=i+Math.imul(d,ee)|0)+Math.imul(h,$)|0))<<13)|0;u=((o=o+Math.imul(h,ee)|0)+(i>>>13)|0)+(Ce>>>26)|0,Ce&=67108863,r=Math.imul(M,H),i=(i=Math.imul(M,U))+Math.imul(S,H)|0,o=Math.imul(S,U),r=r+Math.imul(I,Y)|0,i=(i=i+Math.imul(I,W)|0)+Math.imul(x,Y)|0,o=o+Math.imul(x,W)|0,r=r+Math.imul(C,X)|0,i=(i=i+Math.imul(C,q)|0)+Math.imul(A,X)|0,o=o+Math.imul(A,q)|0,r=r+Math.imul(m,Q)|0,i=(i=i+Math.imul(m,K)|0)+Math.imul(y,Q)|0,o=o+Math.imul(y,K)|0,r=r+Math.imul(p,$)|0,i=(i=i+Math.imul(p,ee)|0)+Math.imul(g,$)|0,o=o+Math.imul(g,ee)|0;var Ae=(u+(r=r+Math.imul(d,ne)|0)|0)+((8191&(i=(i=i+Math.imul(d,re)|0)+Math.imul(h,ne)|0))<<13)|0;u=((o=o+Math.imul(h,re)|0)+(i>>>13)|0)+(Ae>>>26)|0,Ae&=67108863,r=Math.imul(E,H),i=(i=Math.imul(E,U))+Math.imul(O,H)|0,o=Math.imul(O,U),r=r+Math.imul(M,Y)|0,i=(i=i+Math.imul(M,W)|0)+Math.imul(S,Y)|0,o=o+Math.imul(S,W)|0,r=r+Math.imul(I,X)|0,i=(i=i+Math.imul(I,q)|0)+Math.imul(x,X)|0,o=o+Math.imul(x,q)|0,r=r+Math.imul(C,Q)|0,i=(i=i+Math.imul(C,K)|0)+Math.imul(A,Q)|0,o=o+Math.imul(A,K)|0,r=r+Math.imul(m,$)|0,i=(i=i+Math.imul(m,ee)|0)+Math.imul(y,$)|0,o=o+Math.imul(y,ee)|0,r=r+Math.imul(p,ne)|0,i=(i=i+Math.imul(p,re)|0)+Math.imul(g,ne)|0,o=o+Math.imul(g,re)|0;var _e=(u+(r=r+Math.imul(d,oe)|0)|0)+((8191&(i=(i=i+Math.imul(d,ae)|0)+Math.imul(h,oe)|0))<<13)|0;u=((o=o+Math.imul(h,ae)|0)+(i>>>13)|0)+(_e>>>26)|0,_e&=67108863,r=Math.imul(L,H),i=(i=Math.imul(L,U))+Math.imul(D,H)|0,o=Math.imul(D,U),r=r+Math.imul(E,Y)|0,i=(i=i+Math.imul(E,W)|0)+Math.imul(O,Y)|0,o=o+Math.imul(O,W)|0,r=r+Math.imul(M,X)|0,i=(i=i+Math.imul(M,q)|0)+Math.imul(S,X)|0,o=o+Math.imul(S,q)|0,r=r+Math.imul(I,Q)|0,i=(i=i+Math.imul(I,K)|0)+Math.imul(x,Q)|0,o=o+Math.imul(x,K)|0,r=r+Math.imul(C,$)|0,i=(i=i+Math.imul(C,ee)|0)+Math.imul(A,$)|0,o=o+Math.imul(A,ee)|0,r=r+Math.imul(m,ne)|0,i=(i=i+Math.imul(m,re)|0)+Math.imul(y,ne)|0,o=o+Math.imul(y,re)|0,r=r+Math.imul(p,oe)|0,i=(i=i+Math.imul(p,ae)|0)+Math.imul(g,oe)|0,o=o+Math.imul(g,ae)|0;var Ie=(u+(r=r+Math.imul(d,le)|0)|0)+((8191&(i=(i=i+Math.imul(d,ue)|0)+Math.imul(h,le)|0))<<13)|0;u=((o=o+Math.imul(h,ue)|0)+(i>>>13)|0)+(Ie>>>26)|0,Ie&=67108863,r=Math.imul(N,H),i=(i=Math.imul(N,U))+Math.imul(j,H)|0,o=Math.imul(j,U),r=r+Math.imul(L,Y)|0,i=(i=i+Math.imul(L,W)|0)+Math.imul(D,Y)|0,o=o+Math.imul(D,W)|0,r=r+Math.imul(E,X)|0,i=(i=i+Math.imul(E,q)|0)+Math.imul(O,X)|0,o=o+Math.imul(O,q)|0,r=r+Math.imul(M,Q)|0,i=(i=i+Math.imul(M,K)|0)+Math.imul(S,Q)|0,o=o+Math.imul(S,K)|0,r=r+Math.imul(I,$)|0,i=(i=i+Math.imul(I,ee)|0)+Math.imul(x,$)|0,o=o+Math.imul(x,ee)|0,r=r+Math.imul(C,ne)|0,i=(i=i+Math.imul(C,re)|0)+Math.imul(A,ne)|0,o=o+Math.imul(A,re)|0,r=r+Math.imul(m,oe)|0,i=(i=i+Math.imul(m,ae)|0)+Math.imul(y,oe)|0,o=o+Math.imul(y,ae)|0,r=r+Math.imul(p,le)|0,i=(i=i+Math.imul(p,ue)|0)+Math.imul(g,le)|0,o=o+Math.imul(g,ue)|0;var xe=(u+(r=r+Math.imul(d,de)|0)|0)+((8191&(i=(i=i+Math.imul(d,he)|0)+Math.imul(h,de)|0))<<13)|0;u=((o=o+Math.imul(h,he)|0)+(i>>>13)|0)+(xe>>>26)|0,xe&=67108863,r=Math.imul(z,H),i=(i=Math.imul(z,U))+Math.imul(F,H)|0,o=Math.imul(F,U),r=r+Math.imul(N,Y)|0,i=(i=i+Math.imul(N,W)|0)+Math.imul(j,Y)|0,o=o+Math.imul(j,W)|0,r=r+Math.imul(L,X)|0,i=(i=i+Math.imul(L,q)|0)+Math.imul(D,X)|0,o=o+Math.imul(D,q)|0,r=r+Math.imul(E,Q)|0,i=(i=i+Math.imul(E,K)|0)+Math.imul(O,Q)|0,o=o+Math.imul(O,K)|0,r=r+Math.imul(M,$)|0,i=(i=i+Math.imul(M,ee)|0)+Math.imul(S,$)|0,o=o+Math.imul(S,ee)|0,r=r+Math.imul(I,ne)|0,i=(i=i+Math.imul(I,re)|0)+Math.imul(x,ne)|0,o=o+Math.imul(x,re)|0,r=r+Math.imul(C,oe)|0,i=(i=i+Math.imul(C,ae)|0)+Math.imul(A,oe)|0,o=o+Math.imul(A,ae)|0,r=r+Math.imul(m,le)|0,i=(i=i+Math.imul(m,ue)|0)+Math.imul(y,le)|0,o=o+Math.imul(y,ue)|0,r=r+Math.imul(p,de)|0,i=(i=i+Math.imul(p,he)|0)+Math.imul(g,de)|0,o=o+Math.imul(g,he)|0;var we=(u+(r=r+Math.imul(d,pe)|0)|0)+((8191&(i=(i=i+Math.imul(d,ge)|0)+Math.imul(h,pe)|0))<<13)|0;u=((o=o+Math.imul(h,ge)|0)+(i>>>13)|0)+(we>>>26)|0,we&=67108863,r=Math.imul(z,Y),i=(i=Math.imul(z,W))+Math.imul(F,Y)|0,o=Math.imul(F,W),r=r+Math.imul(N,X)|0,i=(i=i+Math.imul(N,q)|0)+Math.imul(j,X)|0,o=o+Math.imul(j,q)|0,r=r+Math.imul(L,Q)|0,i=(i=i+Math.imul(L,K)|0)+Math.imul(D,Q)|0,o=o+Math.imul(D,K)|0,r=r+Math.imul(E,$)|0,i=(i=i+Math.imul(E,ee)|0)+Math.imul(O,$)|0,o=o+Math.imul(O,ee)|0,r=r+Math.imul(M,ne)|0,i=(i=i+Math.imul(M,re)|0)+Math.imul(S,ne)|0,o=o+Math.imul(S,re)|0,r=r+Math.imul(I,oe)|0,i=(i=i+Math.imul(I,ae)|0)+Math.imul(x,oe)|0,o=o+Math.imul(x,ae)|0,r=r+Math.imul(C,le)|0,i=(i=i+Math.imul(C,ue)|0)+Math.imul(A,le)|0,o=o+Math.imul(A,ue)|0,r=r+Math.imul(m,de)|0,i=(i=i+Math.imul(m,he)|0)+Math.imul(y,de)|0,o=o+Math.imul(y,he)|0;var Me=(u+(r=r+Math.imul(p,pe)|0)|0)+((8191&(i=(i=i+Math.imul(p,ge)|0)+Math.imul(g,pe)|0))<<13)|0;u=((o=o+Math.imul(g,ge)|0)+(i>>>13)|0)+(Me>>>26)|0,Me&=67108863,r=Math.imul(z,X),i=(i=Math.imul(z,q))+Math.imul(F,X)|0,o=Math.imul(F,q),r=r+Math.imul(N,Q)|0,i=(i=i+Math.imul(N,K)|0)+Math.imul(j,Q)|0,o=o+Math.imul(j,K)|0,r=r+Math.imul(L,$)|0,i=(i=i+Math.imul(L,ee)|0)+Math.imul(D,$)|0,o=o+Math.imul(D,ee)|0,r=r+Math.imul(E,ne)|0,i=(i=i+Math.imul(E,re)|0)+Math.imul(O,ne)|0,o=o+Math.imul(O,re)|0,r=r+Math.imul(M,oe)|0,i=(i=i+Math.imul(M,ae)|0)+Math.imul(S,oe)|0,o=o+Math.imul(S,ae)|0,r=r+Math.imul(I,le)|0,i=(i=i+Math.imul(I,ue)|0)+Math.imul(x,le)|0,o=o+Math.imul(x,ue)|0,r=r+Math.imul(C,de)|0,i=(i=i+Math.imul(C,he)|0)+Math.imul(A,de)|0,o=o+Math.imul(A,he)|0;var Se=(u+(r=r+Math.imul(m,pe)|0)|0)+((8191&(i=(i=i+Math.imul(m,ge)|0)+Math.imul(y,pe)|0))<<13)|0;u=((o=o+Math.imul(y,ge)|0)+(i>>>13)|0)+(Se>>>26)|0,Se&=67108863,r=Math.imul(z,Q),i=(i=Math.imul(z,K))+Math.imul(F,Q)|0,o=Math.imul(F,K),r=r+Math.imul(N,$)|0,i=(i=i+Math.imul(N,ee)|0)+Math.imul(j,$)|0,o=o+Math.imul(j,ee)|0,r=r+Math.imul(L,ne)|0,i=(i=i+Math.imul(L,re)|0)+Math.imul(D,ne)|0,o=o+Math.imul(D,re)|0,r=r+Math.imul(E,oe)|0,i=(i=i+Math.imul(E,ae)|0)+Math.imul(O,oe)|0,o=o+Math.imul(O,ae)|0,r=r+Math.imul(M,le)|0,i=(i=i+Math.imul(M,ue)|0)+Math.imul(S,le)|0,o=o+Math.imul(S,ue)|0,r=r+Math.imul(I,de)|0,i=(i=i+Math.imul(I,he)|0)+Math.imul(x,de)|0,o=o+Math.imul(x,he)|0;var Te=(u+(r=r+Math.imul(C,pe)|0)|0)+((8191&(i=(i=i+Math.imul(C,ge)|0)+Math.imul(A,pe)|0))<<13)|0;u=((o=o+Math.imul(A,ge)|0)+(i>>>13)|0)+(Te>>>26)|0,Te&=67108863,r=Math.imul(z,$),i=(i=Math.imul(z,ee))+Math.imul(F,$)|0,o=Math.imul(F,ee),r=r+Math.imul(N,ne)|0,i=(i=i+Math.imul(N,re)|0)+Math.imul(j,ne)|0,o=o+Math.imul(j,re)|0,r=r+Math.imul(L,oe)|0,i=(i=i+Math.imul(L,ae)|0)+Math.imul(D,oe)|0,o=o+Math.imul(D,ae)|0,r=r+Math.imul(E,le)|0,i=(i=i+Math.imul(E,ue)|0)+Math.imul(O,le)|0,o=o+Math.imul(O,ue)|0,r=r+Math.imul(M,de)|0,i=(i=i+Math.imul(M,he)|0)+Math.imul(S,de)|0,o=o+Math.imul(S,he)|0;var Ee=(u+(r=r+Math.imul(I,pe)|0)|0)+((8191&(i=(i=i+Math.imul(I,ge)|0)+Math.imul(x,pe)|0))<<13)|0;u=((o=o+Math.imul(x,ge)|0)+(i>>>13)|0)+(Ee>>>26)|0,Ee&=67108863,r=Math.imul(z,ne),i=(i=Math.imul(z,re))+Math.imul(F,ne)|0,o=Math.imul(F,re),r=r+Math.imul(N,oe)|0,i=(i=i+Math.imul(N,ae)|0)+Math.imul(j,oe)|0,o=o+Math.imul(j,ae)|0,r=r+Math.imul(L,le)|0,i=(i=i+Math.imul(L,ue)|0)+Math.imul(D,le)|0,o=o+Math.imul(D,ue)|0,r=r+Math.imul(E,de)|0,i=(i=i+Math.imul(E,he)|0)+Math.imul(O,de)|0,o=o+Math.imul(O,he)|0;var Oe=(u+(r=r+Math.imul(M,pe)|0)|0)+((8191&(i=(i=i+Math.imul(M,ge)|0)+Math.imul(S,pe)|0))<<13)|0;u=((o=o+Math.imul(S,ge)|0)+(i>>>13)|0)+(Oe>>>26)|0,Oe&=67108863,r=Math.imul(z,oe),i=(i=Math.imul(z,ae))+Math.imul(F,oe)|0,o=Math.imul(F,ae),r=r+Math.imul(N,le)|0,i=(i=i+Math.imul(N,ue)|0)+Math.imul(j,le)|0,o=o+Math.imul(j,ue)|0,r=r+Math.imul(L,de)|0,i=(i=i+Math.imul(L,he)|0)+Math.imul(D,de)|0,o=o+Math.imul(D,he)|0;var ke=(u+(r=r+Math.imul(E,pe)|0)|0)+((8191&(i=(i=i+Math.imul(E,ge)|0)+Math.imul(O,pe)|0))<<13)|0;u=((o=o+Math.imul(O,ge)|0)+(i>>>13)|0)+(ke>>>26)|0,ke&=67108863,r=Math.imul(z,le),i=(i=Math.imul(z,ue))+Math.imul(F,le)|0,o=Math.imul(F,ue),r=r+Math.imul(N,de)|0,i=(i=i+Math.imul(N,he)|0)+Math.imul(j,de)|0,o=o+Math.imul(j,he)|0;var Le=(u+(r=r+Math.imul(L,pe)|0)|0)+((8191&(i=(i=i+Math.imul(L,ge)|0)+Math.imul(D,pe)|0))<<13)|0;u=((o=o+Math.imul(D,ge)|0)+(i>>>13)|0)+(Le>>>26)|0,Le&=67108863,r=Math.imul(z,de),i=(i=Math.imul(z,he))+Math.imul(F,de)|0,o=Math.imul(F,he);var De=(u+(r=r+Math.imul(N,pe)|0)|0)+((8191&(i=(i=i+Math.imul(N,ge)|0)+Math.imul(j,pe)|0))<<13)|0;u=((o=o+Math.imul(j,ge)|0)+(i>>>13)|0)+(De>>>26)|0,De&=67108863;var Pe=(u+(r=Math.imul(z,pe))|0)+((8191&(i=(i=Math.imul(z,ge))+Math.imul(F,pe)|0))<<13)|0;return u=((o=Math.imul(F,ge))+(i>>>13)|0)+(Pe>>>26)|0,Pe&=67108863,l[0]=ve,l[1]=me,l[2]=ye,l[3]=be,l[4]=Ce,l[5]=Ae,l[6]=_e,l[7]=Ie,l[8]=xe,l[9]=we,l[10]=Me,l[11]=Se,l[12]=Te,l[13]=Ee,l[14]=Oe,l[15]=ke,l[16]=Le,l[17]=De,l[18]=Pe,0!==u&&(l[19]=u,n.length++),n};function p(e,t,n){return(new g).mulp(e,t,n)}function g(e,t){this.x=e,this.y=t}Math.imul||(f=h),o.prototype.mulTo=function(e,t){var n=this.length+e.length;return 10===this.length&&10===e.length?f(this,e,t):n<63?h(this,e,t):n<1024?function(e,t,n){n.negative=t.negative^e.negative,n.length=e.length+t.length;for(var r=0,i=0,o=0;o<n.length-1;o++){var a=i;i=0;for(var s=67108863&r,l=Math.min(o,t.length-1),u=Math.max(0,o-e.length+1);u<=l;u++){var c=o-u,d=(0|e.words[c])*(0|t.words[u]),h=67108863&d;s=67108863&(h=h+s|0),i+=(a=(a=a+(d/67108864|0)|0)+(h>>>26)|0)>>>26,a&=67108863}n.words[o]=s,r=a,a=i}return 0!==r?n.words[o]=r:n.length--,n.strip()}(this,e,t):p(this,e,t)},g.prototype.makeRBT=function(e){for(var t=new Array(e),n=o.prototype._countBits(e)-1,r=0;r<e;r++)t[r]=this.revBin(r,n,e);return t},g.prototype.revBin=function(e,t,n){if(0===e||e===n-1)return e;for(var r=0,i=0;i<t;i++)r|=(1&e)<<t-i-1,e>>=1;return r},g.prototype.permute=function(e,t,n,r,i,o){for(var a=0;a<o;a++)r[a]=t[e[a]],i[a]=n[e[a]]},g.prototype.transform=function(e,t,n,r,i,o){this.permute(o,e,t,n,r,i);for(var a=1;a<i;a<<=1)for(var s=a<<1,l=Math.cos(2*Math.PI/s),u=Math.sin(2*Math.PI/s),c=0;c<i;c+=s)for(var d=l,h=u,f=0;f<a;f++){var p=n[c+f],g=r[c+f],v=n[c+f+a],m=r[c+f+a],y=d*v-h*m;m=d*m+h*v,v=y,n[c+f]=p+v,r[c+f]=g+m,n[c+f+a]=p-v,r[c+f+a]=g-m,f!==s&&(y=l*d-u*h,h=l*h+u*d,d=y)}},g.prototype.guessLen13b=function(e,t){var n=1|Math.max(t,e),r=1&n,i=0;for(n=n/2|0;n;n>>>=1)i++;return 1<<i+1+r},g.prototype.conjugate=function(e,t,n){if(!(n<=1))for(var r=0;r<n/2;r++){var i=e[r];e[r]=e[n-r-1],e[n-r-1]=i,i=t[r],t[r]=-t[n-r-1],t[n-r-1]=-i}},g.prototype.normalize13b=function(e,t){for(var n=0,r=0;r<t/2;r++){var i=8192*Math.round(e[2*r+1]/t)+Math.round(e[2*r]/t)+n;e[r]=67108863&i,n=i<67108864?0:i/67108864|0}return e},g.prototype.convert13b=function(e,t,n,i){for(var o=0,a=0;a<t;a++)o+=0|e[a],n[2*a]=8191&o,o>>>=13,n[2*a+1]=8191&o,o>>>=13;for(a=2*t;a<i;++a)n[a]=0;r(0===o),r(0==(-8192&o))},g.prototype.stub=function(e){for(var t=new Array(e),n=0;n<e;n++)t[n]=0;return t},g.prototype.mulp=function(e,t,n){var r=2*this.guessLen13b(e.length,t.length),i=this.makeRBT(r),o=this.stub(r),a=new Array(r),s=new Array(r),l=new Array(r),u=new Array(r),c=new Array(r),d=new Array(r),h=n.words;h.length=r,this.convert13b(e.words,e.length,a,r),this.convert13b(t.words,t.length,u,r),this.transform(a,o,s,l,r,i),this.transform(u,o,c,d,r,i);for(var f=0;f<r;f++){var p=s[f]*c[f]-l[f]*d[f];l[f]=s[f]*d[f]+l[f]*c[f],s[f]=p}return this.conjugate(s,l,r),this.transform(s,l,h,o,r,i),this.conjugate(h,o,r),this.normalize13b(h,r),n.negative=e.negative^t.negative,n.length=e.length+t.length,n.strip()},o.prototype.mul=function(e){var t=new o(null);return t.words=new Array(this.length+e.length),this.mulTo(e,t)},o.prototype.mulf=function(e){var t=new o(null);return t.words=new Array(this.length+e.length),p(this,e,t)},o.prototype.imul=function(e){return this.clone().mulTo(e,this)},o.prototype.imuln=function(e){r("number"==typeof e),r(e<67108864);for(var t=0,n=0;n<this.length;n++){var i=(0|this.words[n])*e,o=(67108863&i)+(67108863&t);t>>=26,t+=i/67108864|0,t+=o>>>26,this.words[n]=67108863&o}return 0!==t&&(this.words[n]=t,this.length++),this},o.prototype.muln=function(e){return this.clone().imuln(e)},o.prototype.sqr=function(){return this.mul(this)},o.prototype.isqr=function(){return this.imul(this.clone())},o.prototype.pow=function(e){var t=function(e){for(var t=new Array(e.bitLength()),n=0;n<t.length;n++){var r=n/26|0,i=n%26;t[n]=(e.words[r]&1<<i)>>>i}return t}(e);if(0===t.length)return new o(1);for(var n=this,r=0;r<t.length&&0===t[r];r++,n=n.sqr());if(++r<t.length)for(var i=n.sqr();r<t.length;r++,i=i.sqr())0!==t[r]&&(n=n.mul(i));return n},o.prototype.iushln=function(e){r("number"==typeof e&&e>=0);var t,n=e%26,i=(e-n)/26,o=67108863>>>26-n<<26-n;if(0!==n){var a=0;for(t=0;t<this.length;t++){var s=this.words[t]&o,l=(0|this.words[t])-s<<n;this.words[t]=l|a,a=s>>>26-n}a&&(this.words[t]=a,this.length++)}if(0!==i){for(t=this.length-1;t>=0;t--)this.words[t+i]=this.words[t];for(t=0;t<i;t++)this.words[t]=0;this.length+=i}return this.strip()},o.prototype.ishln=function(e){return r(0===this.negative),this.iushln(e)},o.prototype.iushrn=function(e,t,n){var i;r("number"==typeof e&&e>=0),i=t?(t-t%26)/26:0;var o=e%26,a=Math.min((e-o)/26,this.length),s=67108863^67108863>>>o<<o,l=n;if(i-=a,i=Math.max(0,i),l){for(var u=0;u<a;u++)l.words[u]=this.words[u];l.length=a}if(0===a);else if(this.length>a)for(this.length-=a,u=0;u<this.length;u++)this.words[u]=this.words[u+a];else this.words[0]=0,this.length=1;var c=0;for(u=this.length-1;u>=0&&(0!==c||u>=i);u--){var d=0|this.words[u];this.words[u]=c<<26-o|d>>>o,c=d&s}return l&&0!==c&&(l.words[l.length++]=c),0===this.length&&(this.words[0]=0,this.length=1),this.strip()},o.prototype.ishrn=function(e,t,n){return r(0===this.negative),this.iushrn(e,t,n)},o.prototype.shln=function(e){return this.clone().ishln(e)},o.prototype.ushln=function(e){return this.clone().iushln(e)},o.prototype.shrn=function(e){return this.clone().ishrn(e)},o.prototype.ushrn=function(e){return this.clone().iushrn(e)},o.prototype.testn=function(e){r("number"==typeof e&&e>=0);var t=e%26,n=(e-t)/26,i=1<<t;return!(this.length<=n)&&!!(this.words[n]&i)},o.prototype.imaskn=function(e){r("number"==typeof e&&e>=0);var t=e%26,n=(e-t)/26;if(r(0===this.negative,"imaskn works only with positive numbers"),this.length<=n)return this;if(0!==t&&n++,this.length=Math.min(n,this.length),0!==t){var i=67108863^67108863>>>t<<t;this.words[this.length-1]&=i}return this.strip()},o.prototype.maskn=function(e){return this.clone().imaskn(e)},o.prototype.iaddn=function(e){return r("number"==typeof e),r(e<67108864),e<0?this.isubn(-e):0!==this.negative?1===this.length&&(0|this.words[0])<e?(this.words[0]=e-(0|this.words[0]),this.negative=0,this):(this.negative=0,this.isubn(e),this.negative=1,this):this._iaddn(e)},o.prototype._iaddn=function(e){this.words[0]+=e;for(var t=0;t<this.length&&this.words[t]>=67108864;t++)this.words[t]-=67108864,t===this.length-1?this.words[t+1]=1:this.words[t+1]++;return this.length=Math.max(this.length,t+1),this},o.prototype.isubn=function(e){if(r("number"==typeof e),r(e<67108864),e<0)return this.iaddn(-e);if(0!==this.negative)return this.negative=0,this.iaddn(e),this.negative=1,this;if(this.words[0]-=e,1===this.length&&this.words[0]<0)this.words[0]=-this.words[0],this.negative=1;else for(var t=0;t<this.length&&this.words[t]<0;t++)this.words[t]+=67108864,this.words[t+1]-=1;return this.strip()},o.prototype.addn=function(e){return this.clone().iaddn(e)},o.prototype.subn=function(e){return this.clone().isubn(e)},o.prototype.iabs=function(){return this.negative=0,this},o.prototype.abs=function(){return this.clone().iabs()},o.prototype._ishlnsubmul=function(e,t,n){var i,o,a=e.length+n;this._expand(a);var s=0;for(i=0;i<e.length;i++){o=(0|this.words[i+n])+s;var l=(0|e.words[i])*t;s=((o-=67108863&l)>>26)-(l/67108864|0),this.words[i+n]=67108863&o}for(;i<this.length-n;i++)s=(o=(0|this.words[i+n])+s)>>26,this.words[i+n]=67108863&o;if(0===s)return this.strip();for(r(-1===s),s=0,i=0;i<this.length;i++)s=(o=-(0|this.words[i])+s)>>26,this.words[i]=67108863&o;return this.negative=1,this.strip()},o.prototype._wordDiv=function(e,t){var n=(this.length,e.length),r=this.clone(),i=e,a=0|i.words[i.length-1];0!==(n=26-this._countBits(a))&&(i=i.ushln(n),r.iushln(n),a=0|i.words[i.length-1]);var s,l=r.length-i.length;if("mod"!==t){(s=new o(null)).length=l+1,s.words=new Array(s.length);for(var u=0;u<s.length;u++)s.words[u]=0}var c=r.clone()._ishlnsubmul(i,1,l);0===c.negative&&(r=c,s&&(s.words[l]=1));for(var d=l-1;d>=0;d--){var h=67108864*(0|r.words[i.length+d])+(0|r.words[i.length+d-1]);for(h=Math.min(h/a|0,67108863),r._ishlnsubmul(i,h,d);0!==r.negative;)h--,r.negative=0,r._ishlnsubmul(i,1,d),r.isZero()||(r.negative^=1);s&&(s.words[d]=h)}return s&&s.strip(),r.strip(),"div"!==t&&0!==n&&r.iushrn(n),{div:s||null,mod:r}},o.prototype.divmod=function(e,t,n){return r(!e.isZero()),this.isZero()?{div:new o(0),mod:new o(0)}:0!==this.negative&&0===e.negative?(s=this.neg().divmod(e,t),"mod"!==t&&(i=s.div.neg()),"div"!==t&&(a=s.mod.neg(),n&&0!==a.negative&&a.iadd(e)),{div:i,mod:a}):0===this.negative&&0!==e.negative?(s=this.divmod(e.neg(),t),"mod"!==t&&(i=s.div.neg()),{div:i,mod:s.mod}):0!=(this.negative&e.negative)?(s=this.neg().divmod(e.neg(),t),"div"!==t&&(a=s.mod.neg(),n&&0!==a.negative&&a.isub(e)),{div:s.div,mod:a}):e.length>this.length||this.cmp(e)<0?{div:new o(0),mod:this}:1===e.length?"div"===t?{div:this.divn(e.words[0]),mod:null}:"mod"===t?{div:null,mod:new o(this.modn(e.words[0]))}:{div:this.divn(e.words[0]),mod:new o(this.modn(e.words[0]))}:this._wordDiv(e,t);var i,a,s},o.prototype.div=function(e){return this.divmod(e,"div",!1).div},o.prototype.mod=function(e){return this.divmod(e,"mod",!1).mod},o.prototype.umod=function(e){return this.divmod(e,"mod",!0).mod},o.prototype.divRound=function(e){var t=this.divmod(e);if(t.mod.isZero())return t.div;var n=0!==t.div.negative?t.mod.isub(e):t.mod,r=e.ushrn(1),i=e.andln(1),o=n.cmp(r);return o<0||1===i&&0===o?t.div:0!==t.div.negative?t.div.isubn(1):t.div.iaddn(1)},o.prototype.modn=function(e){r(e<=67108863);for(var t=(1<<26)%e,n=0,i=this.length-1;i>=0;i--)n=(t*n+(0|this.words[i]))%e;return n},o.prototype.idivn=function(e){r(e<=67108863);for(var t=0,n=this.length-1;n>=0;n--){var i=(0|this.words[n])+67108864*t;this.words[n]=i/e|0,t=i%e}return this.strip()},o.prototype.divn=function(e){return this.clone().idivn(e)},o.prototype.egcd=function(e){r(0===e.negative),r(!e.isZero());var t=this,n=e.clone();t=0!==t.negative?t.umod(e):t.clone();for(var i=new o(1),a=new o(0),s=new o(0),l=new o(1),u=0;t.isEven()&&n.isEven();)t.iushrn(1),n.iushrn(1),++u;for(var c=n.clone(),d=t.clone();!t.isZero();){for(var h=0,f=1;0==(t.words[0]&f)&&h<26;++h,f<<=1);if(h>0)for(t.iushrn(h);h-- >0;)(i.isOdd()||a.isOdd())&&(i.iadd(c),a.isub(d)),i.iushrn(1),a.iushrn(1);for(var p=0,g=1;0==(n.words[0]&g)&&p<26;++p,g<<=1);if(p>0)for(n.iushrn(p);p-- >0;)(s.isOdd()||l.isOdd())&&(s.iadd(c),l.isub(d)),s.iushrn(1),l.iushrn(1);t.cmp(n)>=0?(t.isub(n),i.isub(s),a.isub(l)):(n.isub(t),s.isub(i),l.isub(a))}return{a:s,b:l,gcd:n.iushln(u)}},o.prototype._invmp=function(e){r(0===e.negative),r(!e.isZero());var t=this,n=e.clone();t=0!==t.negative?t.umod(e):t.clone();for(var i,a=new o(1),s=new o(0),l=n.clone();t.cmpn(1)>0&&n.cmpn(1)>0;){for(var u=0,c=1;0==(t.words[0]&c)&&u<26;++u,c<<=1);if(u>0)for(t.iushrn(u);u-- >0;)a.isOdd()&&a.iadd(l),a.iushrn(1);for(var d=0,h=1;0==(n.words[0]&h)&&d<26;++d,h<<=1);if(d>0)for(n.iushrn(d);d-- >0;)s.isOdd()&&s.iadd(l),s.iushrn(1);t.cmp(n)>=0?(t.isub(n),a.isub(s)):(n.isub(t),s.isub(a))}return(i=0===t.cmpn(1)?a:s).cmpn(0)<0&&i.iadd(e),i},o.prototype.gcd=function(e){if(this.isZero())return e.abs();if(e.isZero())return this.abs();var t=this.clone(),n=e.clone();t.negative=0,n.negative=0;for(var r=0;t.isEven()&&n.isEven();r++)t.iushrn(1),n.iushrn(1);for(;;){for(;t.isEven();)t.iushrn(1);for(;n.isEven();)n.iushrn(1);var i=t.cmp(n);if(i<0){var o=t;t=n,n=o}else if(0===i||0===n.cmpn(1))break;t.isub(n)}return n.iushln(r)},o.prototype.invm=function(e){return this.egcd(e).a.umod(e)},o.prototype.isEven=function(){return 0==(1&this.words[0])},o.prototype.isOdd=function(){return 1==(1&this.words[0])},o.prototype.andln=function(e){return this.words[0]&e},o.prototype.bincn=function(e){r("number"==typeof e);var t=e%26,n=(e-t)/26,i=1<<t;if(this.length<=n)return this._expand(n+1),this.words[n]|=i,this;for(var o=i,a=n;0!==o&&a<this.length;a++){var s=0|this.words[a];o=(s+=o)>>>26,s&=67108863,this.words[a]=s}return 0!==o&&(this.words[a]=o,this.length++),this},o.prototype.isZero=function(){return 1===this.length&&0===this.words[0]},o.prototype.cmpn=function(e){var t,n=e<0;if(0!==this.negative&&!n)return-1;if(0===this.negative&&n)return 1;if(this.strip(),this.length>1)t=1;else{n&&(e=-e),r(e<=67108863,"Number is too big");var i=0|this.words[0];t=i===e?0:i<e?-1:1}return 0!==this.negative?0|-t:t},o.prototype.cmp=function(e){if(0!==this.negative&&0===e.negative)return-1;if(0===this.negative&&0!==e.negative)return 1;var t=this.ucmp(e);return 0!==this.negative?0|-t:t},o.prototype.ucmp=function(e){if(this.length>e.length)return 1;if(this.length<e.length)return-1;for(var t=0,n=this.length-1;n>=0;n--){var r=0|this.words[n],i=0|e.words[n];if(r!==i){r<i?t=-1:r>i&&(t=1);break}}return t},o.prototype.gtn=function(e){return 1===this.cmpn(e)},o.prototype.gt=function(e){return 1===this.cmp(e)},o.prototype.gten=function(e){return this.cmpn(e)>=0},o.prototype.gte=function(e){return this.cmp(e)>=0},o.prototype.ltn=function(e){return-1===this.cmpn(e)},o.prototype.lt=function(e){return-1===this.cmp(e)},o.prototype.lten=function(e){return this.cmpn(e)<=0},o.prototype.lte=function(e){return this.cmp(e)<=0},o.prototype.eqn=function(e){return 0===this.cmpn(e)},o.prototype.eq=function(e){return 0===this.cmp(e)},o.red=function(e){return new _(e)},o.prototype.toRed=function(e){return r(!this.red,"Already a number in reduction context"),r(0===this.negative,"red works only with positives"),e.convertTo(this)._forceRed(e)},o.prototype.fromRed=function(){return r(this.red,"fromRed works only with numbers in reduction context"),this.red.convertFrom(this)},o.prototype._forceRed=function(e){return this.red=e,this},o.prototype.forceRed=function(e){return r(!this.red,"Already a number in reduction context"),this._forceRed(e)},o.prototype.redAdd=function(e){return r(this.red,"redAdd works only with red numbers"),this.red.add(this,e)},o.prototype.redIAdd=function(e){return r(this.red,"redIAdd works only with red numbers"),this.red.iadd(this,e)},o.prototype.redSub=function(e){return r(this.red,"redSub works only with red numbers"),this.red.sub(this,e)},o.prototype.redISub=function(e){return r(this.red,"redISub works only with red numbers"),this.red.isub(this,e)},o.prototype.redShl=function(e){return r(this.red,"redShl works only with red numbers"),this.red.shl(this,e)},o.prototype.redMul=function(e){return r(this.red,"redMul works only with red numbers"),this.red._verify2(this,e),this.red.mul(this,e)},o.prototype.redIMul=function(e){return r(this.red,"redMul works only with red numbers"),this.red._verify2(this,e),this.red.imul(this,e)},o.prototype.redSqr=function(){return r(this.red,"redSqr works only with red numbers"),this.red._verify1(this),this.red.sqr(this)},o.prototype.redISqr=function(){return r(this.red,"redISqr works only with red numbers"),this.red._verify1(this),this.red.isqr(this)},o.prototype.redSqrt=function(){return r(this.red,"redSqrt works only with red numbers"),this.red._verify1(this),this.red.sqrt(this)},o.prototype.redInvm=function(){return r(this.red,"redInvm works only with red numbers"),this.red._verify1(this),this.red.invm(this)},o.prototype.redNeg=function(){return r(this.red,"redNeg works only with red numbers"),this.red._verify1(this),this.red.neg(this)},o.prototype.redPow=function(e){return r(this.red&&!e.red,"redPow(normalNum)"),this.red._verify1(this),this.red.pow(this,e)};var v={k256:null,p224:null,p192:null,p25519:null};function m(e,t){this.name=e,this.p=new o(t,16),this.n=this.p.bitLength(),this.k=new o(1).iushln(this.n).isub(this.p),this.tmp=this._tmp()}function y(){m.call(this,"k256","ffffffff ffffffff ffffffff ffffffff ffffffff ffffffff fffffffe fffffc2f")}function b(){m.call(this,"p224","ffffffff ffffffff ffffffff ffffffff 00000000 00000000 00000001")}function C(){m.call(this,"p192","ffffffff ffffffff ffffffff fffffffe ffffffff ffffffff")}function A(){m.call(this,"25519","7fffffffffffffff ffffffffffffffff ffffffffffffffff ffffffffffffffed")}function _(e){if("string"==typeof e){var t=o._prime(e);this.m=t.p,this.prime=t}else r(e.gtn(1),"modulus must be greater than 1"),this.m=e,this.prime=null}function I(e){_.call(this,e),this.shift=this.m.bitLength(),this.shift%26!=0&&(this.shift+=26-this.shift%26),this.r=new o(1).iushln(this.shift),this.r2=this.imod(this.r.sqr()),this.rinv=this.r._invmp(this.m),this.minv=this.rinv.mul(this.r).isubn(1).div(this.m),this.minv=this.minv.umod(this.r),this.minv=this.r.sub(this.minv)}m.prototype._tmp=function(){var e=new o(null);return e.words=new Array(Math.ceil(this.n/13)),e},m.prototype.ireduce=function(e){var t,n=e;do{this.split(n,this.tmp),t=(n=(n=this.imulK(n)).iadd(this.tmp)).bitLength()}while(t>this.n);var r=t<this.n?-1:n.ucmp(this.p);return 0===r?(n.words[0]=0,n.length=1):r>0?n.isub(this.p):void 0!==n.strip?n.strip():n._strip(),n},m.prototype.split=function(e,t){e.iushrn(this.n,0,t)},m.prototype.imulK=function(e){return e.imul(this.k)},i(y,m),y.prototype.split=function(e,t){for(var n=Math.min(e.length,9),r=0;r<n;r++)t.words[r]=e.words[r];if(t.length=n,e.length<=9)return e.words[0]=0,void(e.length=1);var i=e.words[9];for(t.words[t.length++]=4194303&i,r=10;r<e.length;r++){var o=0|e.words[r];e.words[r-10]=(4194303&o)<<4|i>>>22,i=o}i>>>=22,e.words[r-10]=i,0===i&&e.length>10?e.length-=10:e.length-=9},y.prototype.imulK=function(e){e.words[e.length]=0,e.words[e.length+1]=0,e.length+=2;for(var t=0,n=0;n<e.length;n++){var r=0|e.words[n];t+=977*r,e.words[n]=67108863&t,t=64*r+(t/67108864|0)}return 0===e.words[e.length-1]&&(e.length--,0===e.words[e.length-1]&&e.length--),e},i(b,m),i(C,m),i(A,m),A.prototype.imulK=function(e){for(var t=0,n=0;n<e.length;n++){var r=19*(0|e.words[n])+t,i=67108863&r;r>>>=26,e.words[n]=i,t=r}return 0!==t&&(e.words[e.length++]=t),e},o._prime=function(e){if(v[e])return v[e];var t;if("k256"===e)t=new y;else if("p224"===e)t=new b;else if("p192"===e)t=new C;else{if("p25519"!==e)throw new Error("Unknown prime "+e);t=new A}return v[e]=t,t},_.prototype._verify1=function(e){r(0===e.negative,"red works only with positives"),r(e.red,"red works only with red numbers")},_.prototype._verify2=function(e,t){r(0==(e.negative|t.negative),"red works only with positives"),r(e.red&&e.red===t.red,"red works only with red numbers")},_.prototype.imod=function(e){return this.prime?this.prime.ireduce(e)._forceRed(this):e.umod(this.m)._forceRed(this)},_.prototype.neg=function(e){return e.isZero()?e.clone():this.m.sub(e)._forceRed(this)},_.prototype.add=function(e,t){this._verify2(e,t);var n=e.add(t);return n.cmp(this.m)>=0&&n.isub(this.m),n._forceRed(this)},_.prototype.iadd=function(e,t){this._verify2(e,t);var n=e.iadd(t);return n.cmp(this.m)>=0&&n.isub(this.m),n},_.prototype.sub=function(e,t){this._verify2(e,t);var n=e.sub(t);return n.cmpn(0)<0&&n.iadd(this.m),n._forceRed(this)},_.prototype.isub=function(e,t){this._verify2(e,t);var n=e.isub(t);return n.cmpn(0)<0&&n.iadd(this.m),n},_.prototype.shl=function(e,t){return this._verify1(e),this.imod(e.ushln(t))},_.prototype.imul=function(e,t){return this._verify2(e,t),this.imod(e.imul(t))},_.prototype.mul=function(e,t){return this._verify2(e,t),this.imod(e.mul(t))},_.prototype.isqr=function(e){return this.imul(e,e.clone())},_.prototype.sqr=function(e){return this.mul(e,e)},_.prototype.sqrt=function(e){if(e.isZero())return e.clone();var t=this.m.andln(3);if(r(t%2==1),3===t){var n=this.m.add(new o(1)).iushrn(2);return this.pow(e,n)}for(var i=this.m.subn(1),a=0;!i.isZero()&&0===i.andln(1);)a++,i.iushrn(1);r(!i.isZero());var s=new o(1).toRed(this),l=s.redNeg(),u=this.m.subn(1).iushrn(1),c=this.m.bitLength();for(c=new o(2*c*c).toRed(this);0!==this.pow(c,u).cmp(l);)c.redIAdd(l);for(var d=this.pow(c,i),h=this.pow(e,i.addn(1).iushrn(1)),f=this.pow(e,i),p=a;0!==f.cmp(s);){for(var g=f,v=0;0!==g.cmp(s);v++)g=g.redSqr();r(v<p);var m=this.pow(d,new o(1).iushln(p-v-1));h=h.redMul(m),d=m.redSqr(),f=f.redMul(d),p=v}return h},_.prototype.invm=function(e){var t=e._invmp(this.m);return 0!==t.negative?(t.negative=0,this.imod(t).redNeg()):this.imod(t)},_.prototype.pow=function(e,t){if(t.isZero())return new o(1).toRed(this);if(0===t.cmpn(1))return e.clone();var n=new Array(16);n[0]=new o(1).toRed(this),n[1]=e;for(var r=2;r<n.length;r++)n[r]=this.mul(n[r-1],e);var i=n[0],a=0,s=0,l=t.bitLength()%26;for(0===l&&(l=26),r=t.length-1;r>=0;r--){for(var u=t.words[r],c=l-1;c>=0;c--){var d=u>>c&1;i!==n[0]&&(i=this.sqr(i)),0!==d||0!==a?(a<<=1,a|=d,(4===++s||0===r&&0===c)&&(i=this.mul(i,n[a]),s=0,a=0)):s=0}l=26}return i},_.prototype.convertTo=function(e){var t=e.umod(this.m);return t===e?t.clone():t},_.prototype.convertFrom=function(e){var t=e.clone();return t.red=null,t},o.mont=function(e){return new I(e)},i(I,_),I.prototype.convertTo=function(e){return this.imod(e.ushln(this.shift))},I.prototype.convertFrom=function(e){var t=this.imod(e.mul(this.rinv));return t.red=null,t},I.prototype.imul=function(e,t){if(e.isZero()||t.isZero())return e.words[0]=0,e.length=1,e;var n=e.imul(t),r=n.maskn(this.shift).mul(this.minv).imaskn(this.shift).mul(this.m),i=n.isub(r).iushrn(this.shift),o=i;return i.cmp(this.m)>=0?o=i.isub(this.m):i.cmpn(0)<0&&(o=i.iadd(this.m)),o._forceRed(this)},I.prototype.mul=function(e,t){if(e.isZero()||t.isZero())return new o(0)._forceRed(this);var n=e.mul(t),r=n.maskn(this.shift).mul(this.minv).imaskn(this.shift).mul(this.m),i=n.isub(r).iushrn(this.shift),a=i;return i.cmp(this.m)>=0?a=i.isub(this.m):i.cmpn(0)<0&&(a=i.iadd(this.m)),a._forceRed(this)},I.prototype.invm=function(e){return this.imod(e._invmp(this.m).mul(this.r2))._forceRed(this)}}(e,this)}).call(this,n(188)(e))},function(e,t,n){"use strict";var r=n(193),i=n(177).hovertemplateAttrs,o=n(177).texttemplateAttrs,a=n(161),s=n(94),l=n(1075),u=n(63).extendFlat,c=s({editType:"calc",arrayOk:!0,colorEditType:"style"}),d=u({},r.marker.line.width,{dflt:0}),h=u({width:d,editType:"calc"},a("marker.line")),f=u({line:h,editType:"calc"},a("marker"),{opacity:{valType:"number",arrayOk:!0,dflt:1,min:0,max:1,editType:"style"}});e.exports={x:r.x,x0:r.x0,dx:r.dx,y:r.y,y0:r.y0,dy:r.dy,xperiod:r.xperiod,yperiod:r.yperiod,xperiod0:r.xperiod0,yperiod0:r.yperiod0,xperiodalignment:r.xperiodalignment,yperiodalignment:r.yperiodalignment,text:r.text,texttemplate:o({editType:"plot"},{keys:l.eventDataKeys}),hovertext:r.hovertext,hovertemplate:i({},{keys:l.eventDataKeys}),textposition:{valType:"enumerated",values:["inside","outside","auto","none"],dflt:"none",arrayOk:!0,editType:"calc"},insidetextanchor:{valType:"enumerated",values:["end","middle","start"],dflt:"end",editType:"plot"},textangle:{valType:"angle",dflt:"auto",editType:"plot"},textfont:u({},c,{}),insidetextfont:u({},c,{}),outsidetextfont:u({},c,{}),constraintext:{valType:"enumerated",values:["inside","outside","both","none"],dflt:"both",editType:"calc"},cliponaxis:u({},r.cliponaxis,{}),orientation:{valType:"enumerated",values:["v","h"],editType:"calc+clearAxisTypes"},base:{valType:"any",dflt:null,arrayOk:!0,editType:"calc"},offset:{valType:"number",dflt:null,arrayOk:!0,editType:"calc"},width:{valType:"number",dflt:null,min:0,arrayOk:!0,editType:"calc"},marker:f,offsetgroup:{valType:"string",dflt:"",editType:"calc"},alignmentgroup:{valType:"string",dflt:"",editType:"calc"},selected:{marker:{opacity:r.selected.marker.opacity,color:r.selected.marker.color,editType:"style"},textfont:r.selected.textfont,editType:"style"},unselected:{marker:{opacity:r.unselected.marker.opacity,color:r.unselected.marker.color,editType:"style"},textfont:r.unselected.textfont,editType:"style"},r:r.r,t:r.t,_deprecated:{bardir:{valType:"enumerated",editType:"calc",values:["v","h"]}}}},function(e,t,n){"use strict";e.exports={BOTTOMSTART:[1,9,13,104,713],TOPSTART:[4,6,7,104,713],LEFTSTART:[8,12,14,208,1114],RIGHTSTART:[2,3,11,208,1114],NEWDELTA:[null,[-1,0],[0,-1],[-1,0],[1,0],null,[0,-1],[-1,0],[0,1],[0,1],null,[0,1],[1,0],[1,0],[0,-1]],CHOOSESADDLE:{104:[4,1],208:[2,8],713:[7,13],1114:[11,14]},SADDLEREMAINDER:{1:4,2:8,4:1,7:13,8:2,11:14,13:7,14:11},LABELDISTANCE:2,LABELINCREASE:10,LABELMIN:3,LABELMAX:10,LABELOPTIMIZER:{EDGECOST:1,ANGLECOST:1,NEIGHBORCOST:5,SAMELEVELFACTOR:10,SAMELEVELDISTANCE:5,MAXCOST:100,INITIALSEARCHPOINTS:10,ITERATIONS:5}}},function(e,t,n){!function(e){"use strict";function t(t,n,i,o){if(i&&i.call){var a=i;i=null}else a=r(t,i,"rangeFinder");"number"==typeof n&&(n=e.Pos(n,0));var s=r(t,i,"minFoldSize");function l(e){var r=a(t,n);if(!r||r.to.line-r.from.line<s)return null;for(var i=t.findMarksAt(r.from),l=0;l<i.length;++l)if(i[l].__isFold&&"fold"!==o){if(!e)return null;r.cleared=!0,i[l].clear()}return r}var u=l(!0);if(r(t,i,"scanUp"))for(;!u&&n.line>t.firstLine();)n=e.Pos(n.line-1,0),u=l(!1);if(u&&!u.cleared&&"unfold"!==o){var c=function(e,t,n){var i=r(e,t,"widget");if("function"==typeof i&&(i=i(n.from,n.to)),"string"==typeof i){var o=document.createTextNode(i);(i=document.createElement("span")).appendChild(o),i.className="CodeMirror-foldmarker"}else i&&(i=i.cloneNode(!0));return i}(t,i,u);e.on(c,"mousedown",(function(t){d.clear(),e.e_preventDefault(t)}));var d=t.markText(u.from,u.to,{replacedWith:c,clearOnEnter:r(t,i,"clearOnEnter"),__isFold:!0});d.on("clear",(function(n,r){e.signal(t,"unfold",t,n,r)})),e.signal(t,"fold",t,u.from,u.to)}}e.newFoldFunction=function(e,n){return function(r,i){t(r,i,{rangeFinder:e,widget:n})}},e.defineExtension("foldCode",(function(e,n,r){t(this,e,n,r)})),e.defineExtension("isFolded",(function(e){for(var t=this.findMarksAt(e),n=0;n<t.length;++n)if(t[n].__isFold)return!0})),e.commands.toggleFold=function(e){e.foldCode(e.getCursor())},e.commands.fold=function(e){e.foldCode(e.getCursor(),null,"fold")},e.commands.unfold=function(e){e.foldCode(e.getCursor(),null,"unfold")},e.commands.foldAll=function(t){t.operation((function(){for(var n=t.firstLine(),r=t.lastLine();n<=r;n++)t.foldCode(e.Pos(n,0),null,"fold")}))},e.commands.unfoldAll=function(t){t.operation((function(){for(var n=t.firstLine(),r=t.lastLine();n<=r;n++)t.foldCode(e.Pos(n,0),null,"unfold")}))},e.registerHelper("fold","combine",(function(){var e=Array.prototype.slice.call(arguments,0);return function(t,n){for(var r=0;r<e.length;++r){var i=e[r](t,n);if(i)return i}}})),e.registerHelper("fold","auto",(function(e,t){for(var n=e.getHelpers(t,"fold"),r=0;r<n.length;r++){var i=n[r](e,t);if(i)return i}}));var n={rangeFinder:e.fold.auto,widget:"↔",minFoldSize:0,scanUp:!1,clearOnEnter:!0};function r(e,t,r){if(t&&void 0!==t[r])return t[r];var i=e.options.foldOptions;return i&&void 0!==i[r]?i[r]:n[r]}e.defineOption("foldOptions",null),e.defineExtension("foldOption",(function(e,t){return r(this,e,t)}))}(n(122))},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.Toolkit=void 0;const r=n(2042);class i{static UID(){return i.TESTING?(i.TESTING_UID++,""+i.TESTING_UID):"xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx".replace(/[xy]/g,e=>{const t=16*Math.random()|0;return("x"===e?t:3&t|8).toString(16)})}static closest(e,t){return document.body.closest?e.closest(t):r(e,t)}}t.Toolkit=i,i.TESTING=!1,i.TESTING_UID=0},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.Point=void 0;const r=n(1098);class i{constructor(e,t){this.x=e,this.y=t}translate(e,t){this.x+=e,this.y+=t}clone(){return new i(this.x,this.y)}toSVG(){return this.x+" "+this.y}asMatrix(){return new r.Matrix([[this.x],[this.y],[1]])}transform(e){let t=e.mmul(this.asMatrix());this.x=t.get(0,0),this.y=t.get(1,0)}static middlePoint(e,t){return new i((t.x+e.x)/2,(t.y+e.y)/2)}static multiply(...e){let t=e[0];for(let n=1;n<e.length;n++)t=t.mmul(e[n]);return t}static scaleMatrix(e,t){return new r.Matrix([[e,0,0],[0,t,0],[0,0,1]])}static translateMatrix(e,t){return new r.Matrix([[1,0,e],[0,1,t],[0,0,1]])}static rotateMatrix(e){return new r.Matrix([[Math.cos(e),-1*Math.sin(e),0],[Math.sin(e),Math.cos(e),0],[0,0,1]])}static createScaleMatrix(e,t,n){return this.multiply(i.translateMatrix(n.x,n.y),i.scaleMatrix(e,t),i.translateMatrix(-n.x,-n.y))}static createRotateMatrix(e,t){return this.multiply(i.translateMatrix(t.x,t.y),i.rotateMatrix(e),i.translateMatrix(-t.x,-t.y))}}t.Point=i},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.AbstractDisplacementState=void 0;const r=n(404),i=n(182);class o extends r.State{constructor(e){super(e),this.registerAction(new i.Action({type:i.InputType.MOUSE_DOWN,fire:e=>{this.initialX=e.event.clientX,this.initialY=e.event.clientY;const t=this.engine.getRelativePoint(e.event.clientX,e.event.clientY);this.initialXRelative=t.x,this.initialYRelative=t.y}})),this.registerAction(new i.Action({type:i.InputType.MOUSE_MOVE,fire:e=>{const{event:t}=e;0!==t.buttons?this.fireMouseMoved({displacementX:t.clientX-this.initialX,displacementY:t.clientY-this.initialY,virtualDisplacementX:(t.clientX-this.initialX)/(this.engine.getModel().getZoomLevel()/100),virtualDisplacementY:(t.clientY-this.initialY)/(this.engine.getModel().getZoomLevel()/100),event:t}):this.eject()}})),this.registerAction(new i.Action({type:i.InputType.MOUSE_UP,fire:e=>{this.eject()}}))}}t.AbstractDisplacementState=o},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.State=void 0;const r=n(182),i=n(67);t.State=class{constructor(e){this.actions=[],this.keys=[],this.childStates=[],this.options=e}setEngine(e){this.engine=e}getOptions(){return this.options}eject(){this.engine.getStateMachine().popState()}transitionWithEvent(e,t){this.engine.getStateMachine().pushState(e),this.engine.getActionEventBus().fireAction(t)}registerAction(e){this.actions.push(e)}tryActivateParentState(e){return this.keys.length>0&&!this.isKeysFullfilled(e)&&(this.eject(),!0)}tryActivateChildState(e){const t=this.findStateToActivate(e);return!!t&&(this.engine.getStateMachine().pushState(t),!0)}findStateToActivate(e){for(let t of this.childStates)if(t.isKeysFullfilled(e))return t;return null}isKeysFullfilled(e){return i.intersection(this.keys,e).length===this.keys.length}activated(e){const t=this.engine.getActionEventBus().getKeys();if(!this.tryActivateParentState(t)&&!this.tryActivateChildState(t)){this.handler1=this.engine.getActionEventBus().registerAction(new r.Action({type:r.InputType.KEY_DOWN,fire:()=>{this.tryActivateChildState(this.engine.getActionEventBus().getKeys())}})),this.handler2=this.engine.getActionEventBus().registerAction(new r.Action({type:r.InputType.KEY_UP,fire:()=>{this.tryActivateParentState(this.engine.getActionEventBus().getKeys())}}));for(let e of this.actions)this.engine.getActionEventBus().registerAction(e)}}deactivated(e){this.handler1&&this.handler1(),this.handler2&&this.handler2();for(let e of this.actions)this.engine.getActionEventBus().deregisterAction(e)}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.NodeModel=void 0;const r=n(67),i=n(151),o=n(69);class a extends o.BasePositionModel{constructor(e){super(e),this.ports={},this.width=0,this.height=0}getBoundingBox(){return new i.Rectangle(this.getPosition(),this.width,this.height)}setPosition(e,t){let n=this.position;super.setPosition(e,t),r.forEach(this.ports,r=>{r.setPosition(r.getX()+e-n.x,r.getY()+t-n.y)})}deserialize(e){super.deserialize(e),r.forEach(e.data.ports,t=>{let n=e.engine.getFactoryForPort(t.type).generateModel({});n.deserialize(Object.assign(Object.assign({},e),{data:t})),e.registerModel(n),this.addPort(n)})}serialize(){return Object.assign(Object.assign({},super.serialize()),{ports:r.map(this.ports,e=>e.serialize())})}doClone(e={},t){t.ports={},r.forEach(this.ports,n=>{t.addPort(n.clone(e))})}remove(){super.remove(),r.forEach(this.ports,e=>{r.forEach(e.getLinks(),e=>{e.remove()})})}getPortFromID(e){for(var t in this.ports)if(this.ports[t].getID()===e)return this.ports[t];return null}getLink(e){for(let t in this.ports){const n=this.ports[t].getLinks();if(n[e])return n[e]}}getPort(e){return this.ports[e]}getPorts(){return this.ports}removePort(e){for(let t of r.values(e.getLinks()))t.clearPort(e);this.ports[e.getName()]&&(this.ports[e.getName()].setParent(null),delete this.ports[e.getName()])}addPort(e){return e.setParent(this),this.ports[e.getName()]=e,e}updateDimensions({width:e,height:t}){this.width=e,this.height=t}}t.NodeModel=a},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.PortModel=t.PortModelAlignment=void 0;const r=n(67),i=n(151),o=n(69);!function(e){e.TOP="top",e.LEFT="left",e.BOTTOM="bottom",e.RIGHT="right"}(t.PortModelAlignment||(t.PortModelAlignment={}));class a extends o.BasePositionModel{constructor(e){super(e),this.links={},this.reportedPosition=!1}deserialize(e){super.deserialize(e),this.reportedPosition=!1,this.options.name=e.data.name,this.options.alignment=e.data.alignment}serialize(){return Object.assign(Object.assign({},super.serialize()),{name:this.options.name,alignment:this.options.alignment,parentNode:this.parent.getID(),links:r.map(this.links,e=>e.getID())})}setPosition(e,t){let n=this.position;super.setPosition(e,t),r.forEach(this.getLinks(),r=>{let i=r.getPointForPort(this);i.setPosition(i.getX()+e-n.x,i.getY()+t-n.y)})}doClone(e={},t){t.links={},t.parentNode=this.getParent().clone(e)}getNode(){return this.getParent()}getName(){return this.options.name}getMaximumLinks(){return this.options.maximumLinks}setMaximumLinks(e){this.options.maximumLinks=e}removeLink(e){delete this.links[e.getID()]}addLink(e){this.links[e.getID()]=e}getLinks(){return this.links}createLinkModel(){if(r.isFinite(this.options.maximumLinks)){var e=r.size(this.links);if(1===this.options.maximumLinks&&e>=1)return r.values(this.links)[0];if(e>=this.options.maximumLinks)return null}return null}reportPosition(){r.forEach(this.getLinks(),e=>{e.getPointForPort(this).setPosition(this.getCenter())}),this.fireEvent({entity:this},"reportInitialPosition")}getCenter(){return new i.Point(this.getX()+this.width/2,this.getY()+this.height/2)}updateCoords(e){this.width=e.getWidth(),this.height=e.getHeight(),this.setPosition(e.getTopLeft()),this.reportedPosition=!0,this.reportPosition()}canLinkToPort(e){return!0}isLocked(){return super.isLocked()||this.getParent().isLocked()}}t.PortModel=a},function(e,t,n){e.exports=n(2064)},function(e,t,n){var r=n(407),i=n(219),o=n(307);n(126);function a(e){e=e||{},this.heuristic=e.heuristic||o.manhattan,this.trackJumpRecursion=e.trackJumpRecursion||!1}a.prototype.findPath=function(e,t,n,o,a){var s,l=this.openList=new r((function(e,t){return e.f-t.f})),u=this.startNode=a.getNodeAt(e,t),c=this.endNode=a.getNodeAt(n,o);for(this.grid=a,u.g=0,u.f=0,l.push(u),u.opened=!0;!l.empty();){if((s=l.pop()).closed=!0,s===c)return i.expandPath(i.backtrace(c));this._identifySuccessors(s)}return[]},a.prototype._identifySuccessors=function(e){var t,n,r,i,a,s,l,u,c,d,h=this.grid,f=this.heuristic,p=this.openList,g=this.endNode.x,v=this.endNode.y,m=e.x,y=e.y,b=Math.abs;Math.max;for(i=0,a=(t=this._findNeighbors(e)).length;i<a;++i)if(n=t[i],r=this._jump(n[0],n[1],m,y)){if(s=r[0],l=r[1],(d=h.getNodeAt(s,l)).closed)continue;u=o.octile(b(s-m),b(l-y)),c=e.g+u,(!d.opened||c<d.g)&&(d.g=c,d.h=d.h||f(b(s-g),b(l-v)),d.f=d.g+d.h,d.parent=e,d.opened?p.updateItem(d):(p.push(d),d.opened=!0))}},e.exports=a},function(e,t,n){var r=n(410),i=n(235),o=Object.prototype.hasOwnProperty;e.exports=function(e,t,n){var a=e[t];o.call(e,t)&&i(a,n)&&(void 0!==n||t in e)||r(e,t,n)}},function(e,t,n){var r=n(1139);e.exports=function(e,t,n){"__proto__"==t&&r?r(e,t,{configurable:!0,enumerable:!0,value:n,writable:!0}):e[t]=n}},function(e,t,n){var r=n(610),i=n(2102)(r);e.exports=i},function(e,t,n){var r=n(81),i=n(613),o=n(2109),a=n(1152);e.exports=function(e,t){return r(e)?e:i(e,t)?[e]:o(a(e))}},function(e,t){e.exports=function(e,t){for(var n=-1,r=null==e?0:e.length,i=Array(r);++n<r;)i[n]=t(e[n],n,e);return i}},function(e,t,n){var r=n(220),i=n(1161),o=n(1162);e.exports=function(e,t){return o(i(e,t,r),e+"")}},function(e,t,n){var r=n(235),i=n(174),o=n(361),a=n(130);e.exports=function(e,t,n){if(!a(n))return!1;var s=typeof t;return!!("number"==s?i(n)&&o(t,n.length):"string"==s&&t in n)&&r(n[t],e)}},function(e,t,n){"use strict";var r=n(70);e.exports={longestPath:function(e){var t={};r.forEach(e.sources(),(function n(i){var o=e.node(i);if(r.has(t,i))return o.rank;t[i]=!0;var a=r.min(r.map(e.outEdges(i),(function(t){return n(t.w)-e.edge(t).minlen})));return a!==Number.POSITIVE_INFINITY&&null!=a||(a=0),o.rank=a}))},slack:function(e,t){return e.node(t.w).rank-e.node(t.v).rank-e.edge(t).minlen}}},function(e,t,n){"use strict";n.r(t);var r=n(904);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";n.r(t);var r=n(905);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";n.r(t);var r=n(906);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";n.r(t);var r=n(913);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";n.r(t);var r=n(1197);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";n.r(t);var r=n(1198);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.selectionMaxWidth=t.actionsColumnWidth=t.rowActions=t.baseIconSize=t.elementSize=void 0;var r=function(e){return"default"===e.options.padding?"medium":"small"};t.elementSize=r;var i=function(e){return"medium"===r(e)?48:32};t.baseIconSize=i;var o=function(e){return e.actions.filter((function(e){return"row"===e.position||"function"==typeof e}))};t.rowActions=o;t.actionsColumnWidth=function(e){return o(e).length*i(e)};t.selectionMaxWidth=function(e,t){return i(e)+9*t}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){(0,o.default)(1,arguments);var n=t||{},a=n.locale,s=a&&a.options&&a.options.weekStartsOn,l=null==s?0:(0,r.default)(s),u=null==n.weekStartsOn?l:(0,r.default)(n.weekStartsOn);if(!(u>=0&&u<=6))throw new RangeError("weekStartsOn must be between 0 and 6 inclusively");var c=(0,i.default)(e),d=c.getUTCDay(),h=(d<u?7:0)+d-u;return c.setUTCDate(c.getUTCDate()-h),c.setUTCHours(0,0,0,0),c};var r=a(n(77)),i=a(n(47)),o=a(n(40));function a(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){(0,i.default)(1,arguments);var t=1,n=(0,r.default)(e),o=n.getUTCDay(),a=(o<t?7:0)+o-t;return n.setUTCDate(n.getUTCDate()-a),n.setUTCHours(0,0,0,0),n};var r=o(n(47)),i=o(n(40));function o(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";n.r(t);var r=n(920);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";n.r(t);var r=n(1199);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";n.r(t);var r=n(1194);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";n.r(t);var r=n(914);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";n.r(t);var r=n(926);n.d(t,"default",(function(){return r.a}))},function(e,t,n){"use strict";n.r(t);var r={};n.r(r),n.d(r,"BACKSPACE",(function(){return k})),n.d(r,"TAB",(function(){return L})),n.d(r,"ENTER",(function(){return D})),n.d(r,"SHIFT",(function(){return P})),n.d(r,"ESCAPE",(function(){return N})),n.d(r,"SPACE",(function(){return j})),n.d(r,"ARROW_LEFT",(function(){return R})),n.d(r,"ARROW_UP",(function(){return z})),n.d(r,"ARROW_RIGHT",(function(){return F})),n.d(r,"ARROW_DOWN",(function(){return B})),n.d(r,"DELETE",(function(){return H})),n.d(r,"isKeyboardClick",(function(){return U}));var i={};n.r(i),n.d(i,"setRef",(function(){return E})),n.d(i,"getRef",(function(){return T})),n.d(i,"arraysEqual",(function(){return V})),n.d(i,"shallowCompareKeys",(function(){return Y})),n.d(i,"deepCompareKeys",(function(){return W})),n.d(i,"getDeepUnequalKeyValues",(function(){return G})),n.d(i,"elementIsOrContains",(function(){return ee})),n.d(i,"throttleEvent",(function(){return te})),n.d(i,"throttleReactEventCallback",(function(){return ne})),n.d(i,"throttle",(function(){return re})),n.d(i,"isFunction",(function(){return K})),n.d(i,"safeInvoke",(function(){return J})),n.d(i,"safeInvokeOrValue",(function(){return $})),n.d(i,"isNodeEnv",(function(){return h.e})),n.d(i,"arrayLengthCompare",(function(){return h.b})),n.d(i,"approxEqual",(function(){return h.a})),n.d(i,"clamp",(function(){return h.c})),n.d(i,"countDecimalPlaces",(function(){return h.d})),n.d(i,"isReactNodeEmpty",(function(){return oe})),n.d(i,"isReactChildrenElementOrElements",(function(){return ae})),n.d(i,"ensureElement",(function(){return se})),n.d(i,"isReactElement",(function(){return le})),n.d(i,"getDisplayName",(function(){return ue})),n.d(i,"isElementOfType",(function(){return ce})),n.d(i,"createReactRef",(function(){return de})),n.d(i,"safeInvokeMember",(function(){return he}));var o={};n.r(o),n.d(o,"show",(function(){return mt})),n.d(o,"hide",(function(){return yt})),n.d(o,"isOpen",(function(){return bt}));var a=n(30),s=function(){function e(e,t){var n=this;this.container=e,this.className=t,this.isRunning=!1,this.handleKeyDown=function(e){9===e.which&&(n.reset(),n.container.addEventListener("mousedown",n.handleMouseDown))},this.handleMouseDown=function(){n.reset(),n.container.classList.add(n.className),n.container.addEventListener("keydown",n.handleKeyDown)}}return e.prototype.isActive=function(){return this.isRunning},e.prototype.start=function(){this.container.addEventListener("mousedown",this.handleMouseDown),this.isRunning=!0},e.prototype.stop=function(){this.reset(),this.isRunning=!1},e.prototype.reset=function(){this.container.classList.remove(this.className),this.container.removeEventListener("keydown",this.handleKeyDown),this.container.removeEventListener("mousedown",this.handleMouseDown)},e}(),l="undefined"!=typeof document?new s(document.documentElement,a.FOCUS_DISABLED):{isActive:function(){return!0},start:function(){return!0},stop:function(){return!0}},u={alwaysShowFocus:function(){return l.stop()},isActive:function(){return l.isActive()},onlyShowFocusOnTabs:function(){return l.start()}},c=n(2),d=n(0),h=n(875),f=function(e){function t(t,n){var r=e.call(this,t,n)||this;return r.timeoutIds=[],r.clearTimeouts=function(){if(r.timeoutIds.length>0){for(var e=0,t=r.timeoutIds;e<t.length;e++){var n=t[e];window.clearTimeout(n)}r.timeoutIds=[]}},Object(h.e)("production")||r.validateProps(r.props),r}return Object(c.__extends)(t,e),t.prototype.componentWillReceiveProps=function(e){Object(h.e)("production")||this.validateProps(e)},t.prototype.componentWillUnmount=function(){this.clearTimeouts()},t.prototype.setTimeout=function(e,t){var n=window.setTimeout(e,t);return this.timeoutIds.push(n),function(){return window.clearTimeout(n)}},t.prototype.validateProps=function(e){},t}(d.Component),p=function(e){function t(t,n){var r=e.call(this,t,n)||this;return r.timeoutIds=[],r.requestIds=[],r.clearTimeouts=function(){if(r.timeoutIds.length>0){for(var e=0,t=r.timeoutIds;e<t.length;e++){var n=t[e];window.clearTimeout(n)}r.timeoutIds=[]}},r.cancelAnimationFrames=function(){if(r.requestIds.length>0){for(var e=0,t=r.requestIds;e<t.length;e++){var n=t[e];window.cancelAnimationFrame(n)}r.requestIds=[]}},Object(h.e)("production")||r.validateProps(r.props),r}return Object(c.__extends)(t,e),t.prototype.componentDidUpdate=function(e,t,n){Object(h.e)("production")||this.validateProps(this.props)},t.prototype.componentWillUnmount=function(){this.clearTimeouts(),this.cancelAnimationFrames()},t.prototype.requestAnimationFrame=function(e){var t=window.requestAnimationFrame(e);return this.requestIds.push(t),function(){return window.cancelAnimationFrame(t)}},t.prototype.setTimeout=function(e,t){var n=window.setTimeout(e,t);return this.timeoutIds.push(n),function(){return window.clearTimeout(n)}},t.prototype.validateProps=function(e){},t}(d.Component),g=function(e){function t(t,n){var r=e.call(this,t,n)||this;return r.timeoutIds=[],r.clearTimeouts=function(){if(r.timeoutIds.length>0){for(var e=0,t=r.timeoutIds;e<t.length;e++){var n=t[e];window.clearTimeout(n)}r.timeoutIds=[]}},Object(h.e)("production")||r.validateProps(r.props),r}return Object(c.__extends)(t,e),t.prototype.componentWillReceiveProps=function(e){Object(h.e)("production")||this.validateProps(e)},t.prototype.componentWillUnmount=function(){this.clearTimeouts()},t.prototype.setTimeout=function(e,t){var n=window.setTimeout(e,t);return this.timeoutIds.push(n),function(){return window.clearTimeout(n)}},t.prototype.validateProps=function(e){},t}(d.PureComponent),v=function(e){function t(t,n){var r=e.call(this,t,n)||this;return r.timeoutIds=[],r.requestIds=[],r.clearTimeouts=function(){if(r.timeoutIds.length>0){for(var e=0,t=r.timeoutIds;e<t.length;e++){var n=t[e];window.clearTimeout(n)}r.timeoutIds=[]}},r.cancelAnimationFrames=function(){if(r.requestIds.length>0){for(var e=0,t=r.requestIds;e<t.length;e++){var n=t[e];window.cancelAnimationFrame(n)}r.requestIds=[]}},Object(h.e)("production")||r.validateProps(r.props),r}return Object(c.__extends)(t,e),t.prototype.componentDidUpdate=function(e,t,n){Object(h.e)("production")||this.validateProps(this.props)},t.prototype.componentWillUnmount=function(){this.clearTimeouts(),this.cancelAnimationFrames()},t.prototype.requestAnimationFrame=function(e){var t=window.requestAnimationFrame(e);return this.requestIds.push(t),function(){return window.cancelAnimationFrame(t)}},t.prototype.setTimeout=function(e,t){var n=window.setTimeout(e,t);return this.timeoutIds.push(n),function(){return window.clearTimeout(n)}},t.prototype.validateProps=function(e){},t}(d.PureComponent),m=n(328),y={START:"start",END:"end"},b={BLACK:"#10161A",BLUE1:"#0E5A8A",BLUE2:"#106BA3",BLUE3:"#137CBD",BLUE4:"#2B95D6",BLUE5:"#48AFF0",COBALT1:"#1F4B99",COBALT2:"#2458B3",COBALT3:"#2965CC",COBALT4:"#4580E6",COBALT5:"#669EFF",DARK_GRAY1:"#182026",DARK_GRAY2:"#202B33",DARK_GRAY3:"#293742",DARK_GRAY4:"#30404D",DARK_GRAY5:"#394B59",FOREST1:"#1D7324",FOREST2:"#238C2C",FOREST3:"#29A634",FOREST4:"#43BF4D",FOREST5:"#62D96B",GOLD1:"#A67908",GOLD2:"#BF8C0A",GOLD3:"#D99E0B",GOLD4:"#F2B824",GOLD5:"#FFC940",GRAY1:"#5C7080",GRAY2:"#738694",GRAY3:"#8A9BA8",GRAY4:"#A7B6C2",GRAY5:"#BFCCD6",GREEN1:"#0A6640",GREEN2:"#0D8050",GREEN3:"#0F9960",GREEN4:"#15B371",GREEN5:"#3DCC91",INDIGO1:"#5642A6",INDIGO2:"#634DBF",INDIGO3:"#7157D9",INDIGO4:"#9179F2",INDIGO5:"#AD99FF",LIGHT_GRAY1:"#CED9E0",LIGHT_GRAY2:"#D8E1E8",LIGHT_GRAY3:"#E1E8ED",LIGHT_GRAY4:"#EBF1F5",LIGHT_GRAY5:"#F5F8FA",LIME1:"#728C23",LIME2:"#87A629",LIME3:"#9BBF30",LIME4:"#B6D94C",LIME5:"#D1F26D",ORANGE1:"#A66321",ORANGE2:"#BF7326",ORANGE3:"#D9822B",ORANGE4:"#F29D49",ORANGE5:"#FFB366",RED1:"#A82A2A",RED2:"#C23030",RED3:"#DB3737",RED4:"#F55656",RED5:"#FF7373",ROSE1:"#A82255",ROSE2:"#C22762",ROSE3:"#DB2C6F",ROSE4:"#F5498B",ROSE5:"#FF66A1",SEPIA1:"#63411E",SEPIA2:"#7D5125",SEPIA3:"#96622D",SEPIA4:"#B07B46",SEPIA5:"#C99765",TURQUOISE1:"#008075",TURQUOISE2:"#00998C",TURQUOISE3:"#00B3A4",TURQUOISE4:"#14CCBD",TURQUOISE5:"#2EE6D6",VERMILION1:"#9E2B0E",VERMILION2:"#B83211",VERMILION3:"#D13913",VERMILION4:"#EB532D",VERMILION5:"#FF6E4A",VIOLET1:"#5C255C",VIOLET2:"#752F75",VIOLET3:"#8F398F",VIOLET4:"#A854A8",VIOLET5:"#C274C2",WHITE:"#FFFFFF"},C=n(205),A=n(206),_=n(60),I=["active","alignText","asyncControl","containerRef","current","elementRef","fill","icon","inputRef","intent","inline","large","loading","leftElement","leftIcon","minimal","onRemove","outlined","panel","panelClassName","popoverProps","rightElement","rightIcon","round","small","text"];function x(e,t,n){return void 0===t&&(t=I),void 0===n&&(n=!1),n&&(t=t.concat(I)),t.reduce((function(e,t){return-1!==t.indexOf("-")||e.hasOwnProperty(t)&&delete e[t],e}),Object(c.__assign)({},e))}function w(e){return null!=e&&void 0!==e.current}function M(e){return"function"==typeof e}function S(e,t){return function(n){e(n),t(n)}}function T(e){var t;return null===e?null:null!==(t=e.current)&&void 0!==t?t:e}function E(e,t){void 0!==e&&(w(e)?e.current=t:M(e)&&e(t))}function O(e,t,n){return w(n)?(e[t]=n,n):function(r){e[t]=r,M(n)&&n(r)}}var k=8,L=9,D=13,P=16,N=27,j=32,R=37,z=38,F=39,B=40,H=46;function U(e){return e===D||e===j}function V(e,t,n){return void 0===n&&(n=function(e,t){return e===t}),null==e&&null==t||null!=e&&null!=t&&e.length===t.length&&e.every((function(e,r){return n(e,t[r])}))}function Y(e,t,n){if(null==e&&null==t)return!0;if(null==e||null==t)return!1;if(Array.isArray(e)||Array.isArray(t))return!1;if(null!=n)return X(e,t,n);var r=Object.keys(e),i=Object.keys(t);return X(e,t,{include:r})&&X(e,t,{include:i})}function W(e,t,n){if(e===t)return!0;if(null==e&&null==t)return!0;if(null==e||null==t)return!1;if(Array.isArray(e)||Array.isArray(t))return V(e,t,W);if(Z(e)||Z(t))return e===t;if(null!=n)return q(e,t,n);if(e.constructor!==t.constructor)return!1;var r=Object.keys(e),i=Object.keys(t);return null!=r&&null!=i&&(0===r.length&&0===i.length||V(r,i)&&q(e,t,r))}function G(e,t,n){return void 0===e&&(e={}),void 0===t&&(t={}),function(e,t,n,r){return n.filter((function(n){return!r(e,t,n)})).map((function(n){return{key:n,valueA:e[n],valueB:t[n]}}))}(e,t,null==n?function(e,t){var n=Object.keys(e),r=Object.keys(t),i=Q(n.concat(r));return Object.keys(i)}(e,t):n,(function(e,t,n){return W(e,t,[n])}))}function X(e,t,n){return function(e,t,n){if(function(e){return null!=e&&null!=e.include}(n))return n.include;if(function(e){return null!=e&&null!=e.exclude}(n)){var r=Object.keys(e),i=Object.keys(t),o=Q(r.concat(i));return n.exclude.forEach((function(e){return delete o[e]})),Object.keys(o)}return[]}(e,t,n).every((function(n){return e.hasOwnProperty(n)===t.hasOwnProperty(n)&&e[n]===t[n]}))}function q(e,t,n){return n.every((function(n){return e.hasOwnProperty(n)===t.hasOwnProperty(n)&&W(e[n],t[n])}))}function Z(e){return"number"==typeof e||"string"==typeof e||"boolean"==typeof e}function Q(e){return e.reduce((function(e,t){return e[t]=!0,e}),{})}function K(e){return"function"==typeof e}function J(e){for(var t=[],n=1;n<arguments.length;n++)t[n-1]=arguments[n];if(K(e))return e.apply(void 0,t)}function $(e){for(var t=[],n=1;n<arguments.length;n++)t[n-1]=arguments[n];return K(e)?e.apply(void 0,t):e}function ee(e,t){return e===t||e.contains(t)}function te(e,t,n){var r=ie((function(t){e.dispatchEvent(new CustomEvent(n,t))}));return e.addEventListener(t,r),r}function ne(e,t){return void 0===t&&(t={}),ie(e,(function(e){t.preventDefault&&e.preventDefault()}),(function(e){return e.persist()}))}function re(e){return ie(e)}function ie(e,t,n){var r=!1;return function(){for(var i=[],o=0;o<arguments.length;o++)i[o]=arguments[o];K(t)&&t.apply(void 0,i),r||(r=!0,K(n)&&n.apply(void 0,i),requestAnimationFrame((function(){e.apply(void 0,i),r=!1})))}}function oe(e,t){return void 0===t&&(t=!1),null==e||""===e||!1===e||!t&&Array.isArray(e)&&(0===e.length||e.every((function(e){return oe(e,!0)})))}function ae(e){return!oe(e,!0)&&!0!==e}function se(e,t){return void 0===t&&(t="span"),null==e||"boolean"==typeof e?void 0:"string"==typeof e?e.trim().length>0?d.createElement(t,{},e):void 0:"number"==typeof e||"symbol"==typeof e.type||Array.isArray(e)?d.createElement(t,{},e):le(e)?e:void 0}function le(e){return"object"==typeof e&&void 0!==e.type&&void 0!==e.props}function ue(e){return e.displayName||e.name||"Unknown"}function ce(e,t){return null!=e&&null!=e.type&&null!=e.type.displayName&&e.type.displayName===t.displayName}function de(){return void 0!==d.createRef?d.createRef():{current:null}}function he(e,t){for(var n=[],r=2;r<arguments.length;r++)n[r-2]=arguments[r];if(null!=e){var i=e[t];if(K(i))return i.apply(void 0,n)}}n(876);var fe=n(11),pe=n.n(fe),ge=n(22),ve=n(13),me=n(50),ye=n.n(me),be=n(9),Ce=n.n(be),Ae=n(224),_e=n.n(Ae),Ie=n(8),xe=n.n(Ie),we=n(625),Me=n.n(we),Se=n(641),Te=n.n(Se),Ee=Te()(),Oe=Te()(),ke=function(e){function t(){for(var t,n=arguments.length,r=new Array(n),i=0;i<n;i++)r[i]=arguments[i];return t=e.call.apply(e,[this].concat(r))||this,xe()(Ce()(t),"referenceNode",void 0),xe()(Ce()(t),"setReferenceNode",(function(e){e&&t.referenceNode!==e&&(t.referenceNode=e,t.forceUpdate())})),t}_e()(t,e);var n=t.prototype;return n.componentWillUnmount=function(){this.referenceNode=null},n.render=function(){return d.createElement(Ee.Provider,{value:this.referenceNode},d.createElement(Oe.Provider,{value:this.setReferenceNode},this.props.children))},t}(d.Component),Le=function(e){return Array.isArray(e)?e[0]:e},De=function(e){if("function"==typeof e){for(var t=arguments.length,n=new Array(t>1?t-1:0),r=1;r<t;r++)n[r-1]=arguments[r];return e.apply(void 0,n)}},Pe=function(e,t){if("function"==typeof e)return De(e,t);null!=e&&(e.current=t)},Ne=function(e){function t(){for(var t,n=arguments.length,r=new Array(n),i=0;i<n;i++)r[i]=arguments[i];return t=e.call.apply(e,[this].concat(r))||this,xe()(Ce()(t),"refHandler",(function(e){Pe(t.props.innerRef,e),De(t.props.setReferenceNode,e)})),t}_e()(t,e);var n=t.prototype;return n.componentWillUnmount=function(){Pe(this.props.innerRef,null)},n.render=function(){return Me()(Boolean(this.props.setReferenceNode),"`Reference` should not be used outside of a `Manager` component."),Le(this.props.children)({ref:this.refHandler})},t}(d.Component);function je(e){return d.createElement(Oe.Consumer,null,(function(t){return d.createElement(Ne,ye()({setReferenceNode:t},e))}))}var Re=n(626),ze=n.n(Re),Fe=n(1208),Be=n.n(Fe),He=n(642),Ue={position:"absolute",top:0,left:0,opacity:0,pointerEvents:"none"},Ve={},Ye=function(e){function t(){for(var t,n=arguments.length,r=new Array(n),i=0;i<n;i++)r[i]=arguments[i];return t=e.call.apply(e,[this].concat(r))||this,xe()(Ce()(t),"state",{data:void 0,placement:void 0}),xe()(Ce()(t),"popperInstance",void 0),xe()(Ce()(t),"popperNode",null),xe()(Ce()(t),"arrowNode",null),xe()(Ce()(t),"setPopperNode",(function(e){e&&t.popperNode!==e&&(Pe(t.props.innerRef,e),t.popperNode=e,t.updatePopperInstance())})),xe()(Ce()(t),"setArrowNode",(function(e){t.arrowNode=e})),xe()(Ce()(t),"updateStateModifier",{enabled:!0,order:900,fn:function(e){var n=e.placement;return t.setState({data:e,placement:n}),e}}),xe()(Ce()(t),"getOptions",(function(){return{placement:t.props.placement,eventsEnabled:t.props.eventsEnabled,positionFixed:t.props.positionFixed,modifiers:ye()({},t.props.modifiers,{arrow:ye()({},t.props.modifiers&&t.props.modifiers.arrow,{enabled:!!t.arrowNode,element:t.arrowNode}),applyStyle:{enabled:!1},updateStateModifier:t.updateStateModifier})}})),xe()(Ce()(t),"getPopperStyle",(function(){return t.popperNode&&t.state.data?ye()({position:t.state.data.offsets.popper.position},t.state.data.styles):Ue})),xe()(Ce()(t),"getPopperPlacement",(function(){return t.state.data?t.state.placement:void 0})),xe()(Ce()(t),"getArrowStyle",(function(){return t.arrowNode&&t.state.data?t.state.data.arrowStyles:Ve})),xe()(Ce()(t),"getOutOfBoundariesState",(function(){return t.state.data?t.state.data.hide:void 0})),xe()(Ce()(t),"destroyPopperInstance",(function(){t.popperInstance&&(t.popperInstance.destroy(),t.popperInstance=null)})),xe()(Ce()(t),"updatePopperInstance",(function(){t.destroyPopperInstance();var e=Ce()(t).popperNode,n=t.props.referenceElement;n&&e&&(t.popperInstance=new He.a(n,e,t.getOptions()))})),xe()(Ce()(t),"scheduleUpdate",(function(){t.popperInstance&&t.popperInstance.scheduleUpdate()})),t}_e()(t,e);var n=t.prototype;return n.componentDidUpdate=function(e,t){this.props.placement===e.placement&&this.props.referenceElement===e.referenceElement&&this.props.positionFixed===e.positionFixed&&Be()(this.props.modifiers,e.modifiers,{strict:!0})?this.props.eventsEnabled!==e.eventsEnabled&&this.popperInstance&&(this.props.eventsEnabled?this.popperInstance.enableEventListeners():this.popperInstance.disableEventListeners()):this.updatePopperInstance(),t.placement!==this.state.placement&&this.scheduleUpdate()},n.componentWillUnmount=function(){Pe(this.props.innerRef,null),this.destroyPopperInstance()},n.render=function(){return Le(this.props.children)({ref:this.setPopperNode,style:this.getPopperStyle(),placement:this.getPopperPlacement(),outOfBoundaries:this.getOutOfBoundariesState(),scheduleUpdate:this.scheduleUpdate,arrowProps:{ref:this.setArrowNode,style:this.getArrowStyle()}})},t}(d.Component);xe()(Ye,"defaultProps",{placement:"bottom",eventsEnabled:!0,referenceElement:void 0,positionFixed:!1});He.a.placements;function We(e){var t=e.referenceElement,n=ze()(e,["referenceElement"]);return d.createElement(Ee.Consumer,null,(function(e){return d.createElement(Ye,ye()({referenceElement:void 0!==t?t:e},n))}))}var Ge=n(29),Xe=n(228),qe=!K(ge.createPortal),Ze={blueprintPortalClassName:function(e,t){if(null!=e[t]&&"string"!=typeof e[t])return new Error(Ge.J)}},Qe=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.context={},t.state={hasMounted:!1},t.portalElement=null,t}return Object(c.__extends)(t,e),t.prototype.render=function(){return qe||"undefined"==typeof document||!this.state.hasMounted||null===this.portalElement?null:ge.createPortal(this.props.children,this.portalElement)},t.prototype.componentDidMount=function(){this.props.container&&(this.portalElement=this.createContainerElement(),this.props.container.appendChild(this.portalElement),this.setState({hasMounted:!0},this.props.onChildrenMount),qe&&this.unstableRenderNoPortal())},t.prototype.componentDidUpdate=function(e){null!=this.portalElement&&e.className!==this.props.className&&(void 0!==e.className&&this.portalElement.classList.remove(e.className),Ke(this.portalElement.classList,this.props.className)),qe&&this.unstableRenderNoPortal()},t.prototype.componentWillUnmount=function(){null!=this.portalElement&&(qe&&ge.unmountComponentAtNode(this.portalElement),this.portalElement.remove())},t.prototype.createContainerElement=function(){var e=document.createElement("div");return e.classList.add(a.PORTAL),Ke(e.classList,this.props.className),null!=this.context&&Ke(e.classList,this.context.blueprintPortalClassName),e},t.prototype.unstableRenderNoPortal=function(){null!==this.portalElement&&ge.unstable_renderSubtreeIntoContainer(this,d.createElement("div",null,this.props.children),this.portalElement)},t.displayName="Blueprint3.Portal",t.contextTypes=Ze,t.defaultProps={container:"undefined"!=typeof document?document.body:void 0},t}(d.Component);function Ke(e,t){null!=t&&""!==t&&e.add.apply(e,t.split(" "))}var Je=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.state={hasEverOpened:t.props.isOpen},t.containerElement=null,t.refHandlers={container:function(e){return t.containerElement=Object(ge.findDOMNode)(e)}},t.maybeRenderChild=function(e){if(K(e)&&(e=e()),null==e)return null;var n="object"==typeof e?d.cloneElement(e,{className:pe()(e.props.className,a.OVERLAY_CONTENT),tabIndex:t.props.enforceFocus||t.props.autoFocus?0:void 0}):d.createElement("span",{className:a.OVERLAY_CONTENT},e),r=t.props,i=r.onOpening,o=r.onOpened,s=r.onClosing,l=r.onClosed,u=r.transitionDuration,c=r.transitionName,h=Xe.CSSTransition;return d.createElement(h,{classNames:c,onEntering:i,onEntered:o,onExiting:s,onExited:l,timeout:u,addEndListener:t.handleTransitionAddEnd},n)},t.handleBackdropMouseDown=function(e){var n,r=t.props,i=r.backdropProps,o=r.canOutsideClickClose,a=r.enforceFocus,s=r.onClose;o&&(null==s||s(e)),a&&t.bringFocusInsideOverlay(),null===(n=null==i?void 0:i.onMouseDown)||void 0===n||n.call(i,e)},t.handleDocumentClick=function(e){var r=t.props,i=r.canOutsideClickClose,o=r.isOpen,a=r.onClose,s=e.composed?e.composedPath()[0]:e.target,l=n.openStack.indexOf(t),u=n.openStack.slice(l).some((function(e){var t=e.containerElement;return t&&t.contains(s)&&!t.isSameNode(s)}));o&&i&&!u&&(null==a||a(e))},t.handleDocumentFocus=function(e){var n=e.composed?e.composedPath()[0]:e.target;t.props.enforceFocus&&null!=t.containerElement&&n instanceof Node&&!t.containerElement.contains(n)&&(e.preventDefault(),e.stopImmediatePropagation(),t.bringFocusInsideOverlay())},t.handleKeyDown=function(e){var n=t.props,i=n.canEscapeKeyClose,o=n.onClose;e.which===r.ESCAPE&&i&&(null==o||o(e),e.preventDefault())},t.handleTransitionAddEnd=function(){},t}var n;return Object(c.__extends)(t,e),n=t,t.getDerivedStateFromProps=function(e){var t=e.isOpen;return t?{hasEverOpened:t}:null},t.prototype.render=function(){var e,t;if(this.props.lazy&&!this.state.hasEverOpened)return null;var n=this.props,r=n.children,i=n.className,o=n.usePortal,s=n.isOpen,l=s&&null!==(t=d.Children.map(r,this.maybeRenderChild))&&void 0!==t?t:[],u=this.maybeRenderBackdrop();null!==u&&l.unshift(u);var c=pe()(a.OVERLAY,((e={})[a.OVERLAY_OPEN]=s,e[a.OVERLAY_INLINE]=!o,e),i),h=d.createElement(Xe.TransitionGroup,{appear:!0,className:c,component:"div",onKeyDown:this.handleKeyDown,ref:this.refHandlers.container},l);return o?d.createElement(Qe,{className:this.props.portalClassName,container:this.props.portalContainer},h):h},t.prototype.componentDidMount=function(){this.props.isOpen&&this.overlayWillOpen()},t.prototype.componentDidUpdate=function(e){e.isOpen&&!this.props.isOpen?this.overlayWillClose():!e.isOpen&&this.props.isOpen&&this.overlayWillOpen()},t.prototype.componentWillUnmount=function(){this.overlayWillClose()},t.prototype.bringFocusInsideOverlay=function(){var e=this;return this.requestAnimationFrame((function(){if(null!=e.containerElement&&null!=document.activeElement&&e.props.isOpen&&!e.containerElement.contains(document.activeElement)){var t=e.containerElement.querySelector("[autofocus]"),n=e.containerElement.querySelector("[tabindex]");null!=t?t.focus():null!=n&&n.focus()}}))},t.prototype.maybeRenderBackdrop=function(){var e=this.props,t=e.backdropClassName,n=e.backdropProps,r=e.hasBackdrop,i=e.isOpen,o=e.transitionDuration,s=e.transitionName;return r&&i?d.createElement(Xe.CSSTransition,{classNames:s,key:"__backdrop",timeout:o,addEndListener:this.handleTransitionAddEnd},d.createElement("div",Object(c.__assign)({},n,{className:pe()(a.OVERLAY_BACKDROP,t,null==n?void 0:n.className),onMouseDown:this.handleBackdropMouseDown,tabIndex:this.props.canOutsideClickClose?0:void 0}))):null},t.prototype.overlayWillClose=function(){document.removeEventListener("focus",this.handleDocumentFocus,!0),document.removeEventListener("mousedown",this.handleDocumentClick);var e=n.openStack,t=e.indexOf(this);if(-1!==t){if(e.splice(t,1),e.length>0){var r=n.getLastOpened();r.props.enforceFocus&&document.addEventListener("focus",r.handleDocumentFocus,!0)}0===e.filter((function(e){return e.props.usePortal&&e.props.hasBackdrop})).length&&document.body.classList.remove(a.OVERLAY_OPEN)}},t.prototype.overlayWillOpen=function(){var e=n.openStack;e.length>0&&document.removeEventListener("focus",n.getLastOpened().handleDocumentFocus,!0),e.push(this),this.props.autoFocus&&this.bringFocusInsideOverlay(),this.props.enforceFocus&&document.addEventListener("focus",this.handleDocumentFocus,!0),this.props.canOutsideClickClose&&!this.props.hasBackdrop&&document.addEventListener("mousedown",this.handleDocumentClick),this.props.hasBackdrop&&this.props.usePortal&&document.body.classList.add(a.OVERLAY_OPEN)},t.displayName="Blueprint3.Overlay",t.defaultProps={autoFocus:!0,backdropProps:{},canEscapeKeyClose:!0,canOutsideClickClose:!0,enforceFocus:!0,hasBackdrop:!0,isOpen:!1,lazy:!0,transitionDuration:300,transitionName:a.OVERLAY,usePortal:!0},t.openStack=[],t.getLastOpened=function(){return n.openStack[n.openStack.length-1]},t=n=Object(c.__decorate)([ve.polyfill],t)}(v),$e=n(327),et=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.element=null,t.observer=new $e.default((function(e){var n,r;return null===(r=(n=t.props).onResize)||void 0===r?void 0:r.call(n,e)})),t}return Object(c.__extends)(t,e),t.prototype.render=function(){return d.Children.only(this.props.children)},t.prototype.componentDidMount=function(){this.observeElement()},t.prototype.componentDidUpdate=function(e){this.observeElement(this.props.observeParents!==e.observeParents)},t.prototype.componentWillUnmount=function(){this.observer.disconnect()},t.prototype.observeElement=function(e){void 0===e&&(e=!1);var t=this.getElement();if(t instanceof Element){if((t!==this.element||e)&&(this.observer.disconnect(),this.element=t,this.observer.observe(t),this.props.observeParents))for(var n=t.parentElement;null!=n;)this.observer.observe(n),n=n.parentElement}else this.observer.disconnect()},t.prototype.getElement=function(){try{return Object(ge.findDOMNode)(this)}catch(e){return null}},t.displayName="Blueprint3.ResizeSensor",t=Object(c.__decorate)([ve.polyfill],t)}(v),tt=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.popover=null,t}return Object(c.__extends)(t,e),t.prototype.render=function(){var e,t=this,n=this.props,r=n.children,i=n.intent,o=n.popoverClassName,s=Object(c.__rest)(n,["children","intent","popoverClassName"]),l=pe()(a.TOOLTIP,((e={})[a.MINIMAL]=this.props.minimal,e),a.intentClass(i),o);return d.createElement(pt,Object(c.__assign)({interactionKind:ft.HOVER_TARGET_ONLY,modifiers:{arrow:{enabled:!this.props.minimal}}},s,{autoFocus:!1,canEscapeKeyClose:!1,enforceFocus:!1,lazy:!0,popoverClassName:l,portalContainer:this.props.portalContainer,ref:function(e){return t.popover=e}}),r)},t.prototype.reposition=function(){null!=this.popover&&this.popover.reposition()},t.displayName="Blueprint3.Tooltip",t.defaultProps={hoverCloseDelay:0,hoverOpenDelay:100,minimal:!1,transitionDuration:100},t=Object(c.__decorate)([ve.polyfill],t)}(v);function nt(e){return e.split("-")[0]}function rt(e){return-1!==["left","right"].indexOf(e)}function it(e){switch(e){case"top":return"bottom";case"left":return"right";case"bottom":return"top";default:return"left"}}function ot(e){switch(e.split("-")[1]){case"start":return"left";case"end":return"right";default:return"center"}}function at(e){var t=nt(e.placement);if(null==e.arrowElement)return rt(t)?it(t)+" "+ot(t):ot(t)+" "+it(t);var n=e.arrowElement.clientHeight/2,r=e.offsets.arrow;return rt(t)?it(t)+" "+(r.top+n)+"px":r.left+n+"px "+it(t)}var st=function(e){if(null==e.arrowElement)return e;var t=e.arrowElement.clientWidth,n=nt(e.placement),r=rt(n),i=r?"width":"height",o=r?"left":"top",a=Math.round(t/2/Math.sqrt(2));return"top"===n||"left"===n?(e.offsets.popper[o]-=a+4,e.offsets.arrow[o]=e.offsets.popper[i]-t+a):(e.offsets.popper[o]+=a+4,e.offsets.arrow[o]=-a),e};function lt(e){if(null==e)return 0;switch(nt(e)){case"top":return-90;case"left":return 180;case"bottom":return 90;default:return 0}}var ut=function(e){var t=e.arrowProps,n=t.ref,r=t.style,i=e.placement;return d.createElement("div",{className:a.POPOVER_ARROW,ref:n,style:null==r.left||isNaN(+r.left)?{}:r},d.createElement("svg",{viewBox:"0 0 30 30",style:{transform:"rotate("+lt(i)+"deg)"}},d.createElement("path",{className:a.POPOVER_ARROW+"-border",d:"M8.11 6.302c1.015-.936 1.887-2.922 1.887-4.297v26c0-1.378-.868-3.357-1.888-4.297L.925 17.09c-1.237-1.14-1.233-3.034 0-4.17L8.11 6.302z"}),d.createElement("path",{className:a.POPOVER_ARROW+"-fill",d:"M8.787 7.036c1.22-1.125 2.21-3.376 2.21-5.03V0v30-2.005c0-1.654-.983-3.9-2.21-5.03l-7.183-6.616c-.81-.746-.802-1.96 0-2.7l7.183-6.614z"})))};function ct(e){switch(e){case _.a.TOP_LEFT:return"top-start";case _.a.TOP:return"top";case _.a.TOP_RIGHT:return"top-end";case _.a.RIGHT_TOP:return"right-start";case _.a.RIGHT:return"right";case _.a.RIGHT_BOTTOM:return"right-end";case _.a.BOTTOM_RIGHT:return"bottom-end";case _.a.BOTTOM:return"bottom";case _.a.BOTTOM_LEFT:return"bottom-start";case _.a.LEFT_BOTTOM:return"left-end";case _.a.LEFT:return"left";case _.a.LEFT_TOP:return"left-start";case"auto":case"auto-start":case"auto-end":return e;default:return function(e){throw new Error("Unexpected position: "+e)}(e)}}ut.displayName="Blueprint3.PopoverArrow";var dt,ht,ft={CLICK:"click",CLICK_TARGET_ONLY:"click-target",HOVER:"hover",HOVER_TARGET_ONLY:"hover-target"},pt=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.popoverRef=de(),t.popoverElement=null,t.targetElement=null,t.state={hasDarkParent:!1,isOpen:t.getIsOpen(t.props),transformOrigin:""},t.isMouseInTargetOrPopover=!1,t.lostFocusOnSamePage=!0,t.handlePopoverRef=O(t,"popoverElement",t.props.popoverRef),t.handleTargetRef=function(e){return t.targetElement=e},t.reposition=function(){var e;return null===(e=t.popperScheduleUpdate)||void 0===e?void 0:e.call(t)},t.renderPopover=function(e){var n,r=t.props,i=r.usePortal,o=r.interactionKind,s=t.state.transformOrigin;t.popperScheduleUpdate=e.scheduleUpdate;var l={onClick:t.handlePopoverClick};(o===ft.HOVER||!i&&o===ft.HOVER_TARGET_ONLY)&&(l.onMouseEnter=t.handleMouseEnter,l.onMouseLeave=t.handleMouseLeave);var u=pe()(a.POPOVER,((n={})[a.DARK]=t.props.inheritDarkTheme&&t.state.hasDarkParent,n[a.MINIMAL]=t.props.minimal,n[a.POPOVER_CAPTURING_DISMISS]=t.props.captureDismiss,n),t.props.popoverClassName);return d.createElement("div",{className:a.TRANSITION_CONTAINER,ref:e.ref,style:e.style},d.createElement(et,{onResize:t.reposition},d.createElement("div",Object(c.__assign)({className:u,style:{transformOrigin:s},ref:t.popoverRef},l),t.isArrowEnabled()&&d.createElement(ut,{arrowProps:e.arrowProps,placement:e.placement}),d.createElement("div",{className:a.POPOVER_CONTENT},t.understandChildren().content))))},t.renderTarget=function(e){var n,r,i=t.props,o=i.fill,s=i.openOnTargetFocus,l=i.targetClassName,u=i.targetProps,h=void 0===u?{}:u,f=t.state.isOpen,p=t.isControlled(),g=t.isHoverInteractionKind(),v=t.props.targetTagName;o&&(v="div");var m=g?{onBlur:t.handleTargetBlur,onFocus:t.handleTargetFocus,onMouseEnter:t.handleMouseEnter,onMouseLeave:t.handleMouseLeave}:{onClick:t.handleTargetClick};m.className=pe()(a.POPOVER_TARGET,((n={})[a.POPOVER_OPEN]=f,n),h.className,l),m.ref=e.ref;var y=se(t.understandChildren().target);if(void 0===y)return null;var b=y.props.tabIndex,C=null==b&&s&&g?0:b,A=d.cloneElement(y,{className:pe()(y.props.className,(r={},r[a.ACTIVE]=f&&!p&&!g,r)),disabled:!(!f||!ce(y,tt))||y.props.disabled,tabIndex:C}),_=d.createElement(v,Object(c.__assign)(Object(c.__assign)({},h),m),A);return d.createElement(et,{onResize:t.reposition},_)},t.isControlled=function(){return void 0!==t.props.isOpen},t.handleTargetFocus=function(e){var n,r;if(t.props.openOnTargetFocus&&t.isHoverInteractionKind()){if(null==e.relatedTarget&&!t.lostFocusOnSamePage)return;t.handleMouseEnter(e)}null===(r=null===(n=t.props.targetProps)||void 0===n?void 0:n.onFocus)||void 0===r||r.call(n,e)},t.handleTargetBlur=function(e){var n,r;t.props.openOnTargetFocus&&t.isHoverInteractionKind()&&(null==e.relatedTarget||t.isElementInPopover(e.relatedTarget)||t.handleMouseLeave(e)),t.lostFocusOnSamePage=null!=e.relatedTarget,null===(r=null===(n=t.props.targetProps)||void 0===n?void 0:n.onBlur)||void 0===r||r.call(n,e)},t.handleMouseEnter=function(e){var n,r;t.isMouseInTargetOrPopover=!0,t.props.usePortal||!t.isElementInPopover(e.target)||t.props.interactionKind!==ft.HOVER_TARGET_ONLY||t.props.openOnTargetFocus?t.props.disabled||t.setOpenState(!0,e,t.props.hoverOpenDelay):t.handleMouseLeave(e),null===(r=null===(n=t.props.targetProps)||void 0===n?void 0:n.onMouseEnter)||void 0===r||r.call(n,e)},t.handleMouseLeave=function(e){var n,r;t.isMouseInTargetOrPopover=!1,t.setTimeout((function(){t.isMouseInTargetOrPopover||t.setOpenState(!1,e,t.props.hoverCloseDelay)})),null===(r=null===(n=t.props.targetProps)||void 0===n?void 0:n.onMouseLeave)||void 0===r||r.call(n,e)},t.handlePopoverClick=function(e){var n=e.target,r=n.closest("."+a.POPOVER),i=r===t.popoverRef.current,o=null==r?void 0:r.classList.contains(a.POPOVER_CAPTURING_DISMISS),s=n.closest("."+a.POPOVER_DISMISS+", ."+a.POPOVER_DISMISS_OVERRIDE),l=null!=s&&s.classList.contains(a.POPOVER_DISMISS),u=null!=n.closest(":disabled, ."+a.DISABLED);!l||u||o&&!i||t.setOpenState(!1,e)},t.handleOverlayClose=function(e){if(null!==t.targetElement&&void 0!==e){var n=e.target;(!ee(t.targetElement,n)||e.nativeEvent instanceof KeyboardEvent)&&t.setOpenState(!1,e)}},t.handleTargetClick=function(e){var n,r;t.props.disabled||t.isElementInPopover(e.target)||(null==t.props.isOpen?t.setState((function(e){return{isOpen:!e.isOpen}})):t.setOpenState(!t.props.isOpen,e)),null===(r=null===(n=t.props.targetProps)||void 0===n?void 0:n.onClick)||void 0===r||r.call(n,e)},t.updatePopoverState=function(e){return t.setState({transformOrigin:at(e)}),e},t}return Object(c.__extends)(t,e),t.prototype.render=function(){var e,t=this.props,n=t.className,r=t.disabled,i=t.fill,o=t.placement,s=this.state.isOpen,l=this.props.wrapperTagName;i&&(l="div");var u=null==se(this.understandChildren().content);!u||r||!1===s||h.e("production")||console.warn(Ge.F);var c=pe()(a.POPOVER_WRAPPER,n,((e={})[a.FILL]=i,e)),f=d.createElement(l,{className:c},d.createElement(je,{innerRef:this.handleTargetRef},this.renderTarget),d.createElement(Je,{autoFocus:this.props.autoFocus,backdropClassName:a.POPOVER_BACKDROP,backdropProps:this.props.backdropProps,canEscapeKeyClose:this.props.canEscapeKeyClose,canOutsideClickClose:this.props.interactionKind===ft.CLICK,className:this.props.portalClassName,enforceFocus:this.props.enforceFocus,hasBackdrop:this.props.hasBackdrop,isOpen:s&&!u,onClose:this.handleOverlayClose,onClosed:this.props.onClosed,onClosing:this.props.onClosing,onOpened:this.props.onOpened,onOpening:this.props.onOpening,transitionDuration:this.props.transitionDuration,transitionName:a.POPOVER,usePortal:this.props.usePortal,portalContainer:this.props.portalContainer},d.createElement(We,{innerRef:this.handlePopoverRef,placement:null!=o?o:ct(this.props.position),modifiers:this.getPopperModifiers()},this.renderPopover)));return d.createElement(ke,null,f)},t.prototype.componentDidMount=function(){this.updateDarkParent()},t.prototype.componentDidUpdate=function(t,n){e.prototype.componentDidUpdate.call(this,t,n),this.updateDarkParent();var r=this.getIsOpen(this.props);null!=this.props.isOpen&&r!==this.state.isOpen?(this.setOpenState(r),this.setState({isOpen:r})):this.props.disabled&&this.state.isOpen&&null==this.props.isOpen&&this.setOpenState(!1)},t.prototype.validateProps=function(e){if(null==e.isOpen&&null!=e.onInteraction&&console.warn(Ge.I),e.hasBackdrop&&!e.usePortal&&console.warn(Ge.G),e.hasBackdrop&&e.interactionKind!==ft.CLICK)throw new Error(Ge.B);var t=d.Children.count(e.children),n=void 0!==e.content,r=void 0!==e.target;if(0===t&&!r)throw new Error(Ge.C);t>2&&console.warn(Ge.H),t>0&&r&&console.warn(Ge.E),2===t&&n&&console.warn(Ge.D)},t.prototype.updateDarkParent=function(){if(this.props.usePortal&&this.state.isOpen){var e=null!=this.targetElement&&null!=this.targetElement.closest("."+a.DARK);this.setState({hasDarkParent:e})}},t.prototype.understandChildren=function(){var e=this.props,t=e.children,n=e.content,r=e.target,i=d.Children.toArray(t),o=i[0],a=i[1];return{content:null==a?n:a,target:null==o?r:o}},t.prototype.getIsOpen=function(e){return!e.disabled&&(null!=e.isOpen?e.isOpen:e.defaultIsOpen)},t.prototype.getPopperModifiers=function(){var e=this.props,t=e.boundary,n=e.modifiers,r=n,i=r.flip,o=void 0===i?{}:i,a=r.preventOverflow,s=void 0===a?{}:a;return Object(c.__assign)(Object(c.__assign)({},n),{arrowOffset:{enabled:this.isArrowEnabled(),fn:st,order:510},flip:Object(c.__assign)({boundariesElement:t},o),preventOverflow:Object(c.__assign)({boundariesElement:t},s),updatePopoverState:{enabled:!0,fn:this.updatePopoverState,order:900}})},t.prototype.setOpenState=function(e,t,n){var r,i,o,a,s,l=this;null===(r=this.cancelOpenTimeout)||void 0===r||r.call(this),void 0!==n&&n>0?this.cancelOpenTimeout=this.setTimeout((function(){return l.setOpenState(e,t)}),n):(null==this.props.isOpen?this.setState({isOpen:e}):null===(o=(i=this.props).onInteraction)||void 0===o||o.call(i,e,t),e||null===(s=(a=this.props).onClose)||void 0===s||s.call(a,t))},t.prototype.isArrowEnabled=function(){var e=this.props,t=e.minimal,n=e.modifiers;return!t&&(null==(null==n?void 0:n.arrow)||n.arrow.enabled)},t.prototype.isElementInPopover=function(e){return null!=this.popoverElement&&this.popoverElement.contains(e)},t.prototype.isHoverInteractionKind=function(){return this.props.interactionKind===ft.HOVER||this.props.interactionKind===ft.HOVER_TARGET_ONLY},t.displayName="Blueprint3.Popover",t.defaultProps={boundary:"scrollParent",captureDismiss:!1,defaultIsOpen:!1,disabled:!1,fill:!1,hasBackdrop:!1,hoverCloseDelay:300,hoverOpenDelay:150,inheritDarkTheme:!0,interactionKind:ft.CLICK,minimal:!1,modifiers:{},openOnTargetFocus:!0,position:"auto",targetTagName:"span",transitionDuration:300,usePortal:!0,wrapperTagName:"span"},t=Object(c.__decorate)([ve.polyfill],t)}(v),gt={preventOverflow:{boundariesElement:"viewport"}},vt=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.state={isDarkTheme:!1,isOpen:!1},t.cancelContextMenu=function(e){return e.preventDefault()},t.handleBackdropContextMenu=function(e){e.persist(),e.preventDefault(),t.setTimeout((function(){var t=document.elementFromPoint(e.clientX,e.clientY),n=(e.view,Object(c.__rest)(e,["view"]));null==t||t.dispatchEvent(new MouseEvent("contextmenu",n))}),100)},t.handlePopoverInteraction=function(e){e||t.requestAnimationFrame((function(){return t.hide()}))},t}return Object(c.__extends)(t,e),t.prototype.render=function(){var e,t=d.createElement("div",{onContextMenu:this.cancelContextMenu},this.state.menu),n=pe()(((e={})[a.DARK]=this.state.isDarkTheme,e)),r=void 0===this.state.offset?"":this.state.offset.left+"x"+this.state.offset.top;return d.createElement("div",{className:a.CONTEXT_MENU_POPOVER_TARGET,style:this.state.offset},d.createElement(pt,Object(c.__assign)({},this.props,{backdropProps:{onContextMenu:this.handleBackdropContextMenu},content:t,enforceFocus:!1,key:r,hasBackdrop:!0,isOpen:this.state.isOpen,minimal:!0,modifiers:gt,onInteraction:this.handlePopoverInteraction,position:_.a.RIGHT_TOP,popoverClassName:n,target:d.createElement("div",null),transitionDuration:100})))},t.prototype.show=function(e,t,n,r){void 0===r&&(r=!1),this.setState({isOpen:!0,menu:e,offset:t,onClose:n,isDarkTheme:r})},t.prototype.hide=function(){var e,t;null===(t=(e=this.state).onClose)||void 0===t||t.call(e),this.setState({isOpen:!1,onClose:void 0})},t=Object(c.__decorate)([ve.polyfill],t)}(v);function mt(e,t,n,r){void 0===dt&&((dt=document.createElement("div")).classList.add(a.CONTEXT_MENU),document.body.appendChild(dt),ht=ge.render(d.createElement(vt,{onClosed:Ct}),dt)),ht.show(e,t,n,r)}function yt(){null==ht||ht.hide()}function bt(){return null!=ht&&ht.state.isOpen}function Ct(){null!=dt&&(ge.unmountComponentAtNode(dt),dt.remove(),dt=void 0,ht=void 0)}var At=n(505),_t=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}var n;return Object(c.__extends)(t,e),n=t,t.prototype.render=function(){var e=this.props.icon;if(null==e||"boolean"==typeof e)return null;if("string"!=typeof e)return e;var t=this.props,r=t.className,i=t.color,o=t.htmlTitle,s=t.iconSize,l=void 0===s?n.SIZE_STANDARD:s,u=t.intent,h=t.title,f=void 0===h?e:h,p=t.tagName,g=void 0===p?"span":p,v=Object(c.__rest)(t,["className","color","htmlTitle","iconSize","intent","title","tagName"]),m=l>=n.SIZE_LARGE?n.SIZE_LARGE:n.SIZE_STANDARD,y=this.renderSvgPaths(m,e),b=pe()(a.ICON,a.iconClass(e),a.intentClass(u),r),C="0 0 "+m+" "+m;return d.createElement(g,Object(c.__assign)(Object(c.__assign)({},v),{className:b,title:o}),d.createElement("svg",{fill:i,"data-icon":e,width:l,height:l,viewBox:C},f&&d.createElement("desc",null,f),y))},t.prototype.renderSvgPaths=function(e,t){var r=(e===n.SIZE_STANDARD?At.a:At.b)[t];return null==r?null:r.map((function(e,t){return d.createElement("path",{key:t,d:e,fillRule:"evenodd"})}))},t.displayName="Blueprint3.Icon",t.SIZE_STANDARD=16,t.SIZE_LARGE=20,t=n=Object(c.__decorate)([ve.polyfill],t)}(v),It=45,xt="M 50,50 m 0,-45 a 45,45 0 1 1 0,90 a 45,45 0 1 1 0,-90",wt=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}var n;return Object(c.__extends)(t,e),n=t,t.prototype.componentDidUpdate=function(e){e.value!==this.props.value&&this.forceUpdate()},t.prototype.render=function(){var e,t=this.props,r=t.className,i=t.intent,o=t.value,s=t.tagName,l=void 0===s?"div":s,u=this.getSize(),c=pe()(a.SPINNER,a.intentClass(i),((e={})[a.SPINNER_NO_SPIN]=null!=o,e),r),f=Math.min(16,4*n.SIZE_LARGE/u),p=280-280*(null==o?.25:Object(h.c)(o,0,1));return d.createElement(l,{className:c},d.createElement(l,{className:a.SPINNER_ANIMATION},d.createElement("svg",{width:u,height:u,strokeWidth:f.toFixed(2),viewBox:this.getViewBox(f)},d.createElement("path",{className:a.SPINNER_TRACK,d:xt}),d.createElement("path",{className:a.SPINNER_HEAD,d:xt,pathLength:280,strokeDasharray:"280 280",strokeDashoffset:p}))))},t.prototype.validateProps=function(e){var t=e.className,n=void 0===t?"":t;null!=e.size&&(n.indexOf(a.SMALL)>=0||n.indexOf(a.LARGE)>=0)&&console.warn(Ge.O)},t.prototype.getSize=function(){var e=this.props,t=e.className,r=void 0===t?"":t,i=e.size;return null==i?r.indexOf(a.SMALL)>=0?n.SIZE_SMALL:r.indexOf(a.LARGE)>=0?n.SIZE_LARGE:n.SIZE_STANDARD:Math.max(10,i)},t.prototype.getViewBox=function(e){var t=It+e/2,n=(50-t).toFixed(2),r=(2*t).toFixed(2);return n+" "+n+" "+r+" "+r},t.displayName="Blueprint3.Spinner",t.SIZE_SMALL=20,t.SIZE_STANDARD=50,t.SIZE_LARGE=100,t=n=Object(c.__decorate)([ve.polyfill],t)}(v),Mt=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.state={isActive:!1},t.handleKeyDown=function(e){var n,i;r.isKeyboardClick(e.which)&&(e.preventDefault(),e.which!==t.currentKeyDown&&t.setState({isActive:!0})),t.currentKeyDown=e.which,null===(i=(n=t.props).onKeyDown)||void 0===i||i.call(n,e)},t.handleKeyUp=function(e){var n,i,o;r.isKeyboardClick(e.which)&&(t.setState({isActive:!1}),null===(n=T(t.buttonRef))||void 0===n||n.click()),t.currentKeyDown=void 0,null===(o=(i=t.props).onKeyUp)||void 0===o||o.call(i,e)},t.handleBlur=function(e){var n,r;t.state.isActive&&t.setState({isActive:!1}),null===(r=(n=t.props).onBlur)||void 0===r||r.call(n,e)},t}return Object(c.__extends)(t,e),t.prototype.getCommonButtonProps=function(){var e,t=this.props,n=t.active,r=t.alignText,i=t.fill,o=t.large,s=t.loading,l=t.outlined,u=t.minimal,c=t.small,d=t.tabIndex,h=this.props.disabled||s;return{className:pe()(a.BUTTON,((e={})[a.ACTIVE]=!h&&(n||this.state.isActive),e[a.DISABLED]=h,e[a.FILL]=i,e[a.LARGE]=o,e[a.LOADING]=s,e[a.MINIMAL]=u,e[a.OUTLINED]=l,e[a.SMALL]=c,e),a.alignmentClass(r),a.intentClass(this.props.intent),this.props.className),disabled:h,onBlur:this.handleBlur,onClick:h?void 0:this.props.onClick,onKeyDown:this.handleKeyDown,onKeyUp:this.handleKeyUp,tabIndex:h?-1:d}},t.prototype.renderChildren=function(){var e=this.props,t=e.children,n=e.icon,r=e.loading,o=e.rightIcon,s=e.text;return[r&&d.createElement(wt,{key:"loading",className:a.BUTTON_SPINNER,size:_t.SIZE_LARGE}),d.createElement(_t,{key:"leftIcon",icon:n}),(!i.isReactNodeEmpty(s)||!i.isReactNodeEmpty(t))&&d.createElement("span",{key:"text",className:a.BUTTON_TEXT},s,t),d.createElement(_t,{key:"rightIcon",icon:o})]},t}(v),St=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.buttonRef=null,t.handleRef=O(t,"buttonRef",t.props.elementRef),t}return Object(c.__extends)(t,e),t.prototype.render=function(){return d.createElement("button",Object(c.__assign)({type:"button",ref:this.handleRef},x(this.props),this.getCommonButtonProps()),this.renderChildren())},t.displayName="Blueprint3.Button",t}(Mt),Tt=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.buttonRef=null,t.handleRef=O(t,"buttonRef",t.props.elementRef),t}return Object(c.__extends)(t,e),t.prototype.render=function(){var e=this.props,t=e.href,n=e.tabIndex,r=void 0===n?0:n,i=this.getCommonButtonProps();return d.createElement("a",Object(c.__assign)({role:"button",ref:this.handleRef},x(this.props),i,{href:i.disabled?void 0:t,tabIndex:i.disabled?-1:r}),this.renderChildren())},t.displayName="Blueprint3.AnchorButton",t}(Mt);function Et(e,t){return function(n){var r=n.className,i=n.elementRef,o=Object(c.__rest)(n,["className","elementRef"]);return d.createElement(e,Object(c.__assign)(Object(c.__assign)({},o),{className:pe()(t,r),ref:i}))}}var Ot,kt=Et("h1",a.HEADING),Lt=Et("h2",a.HEADING),Dt=Et("h3",a.HEADING),Pt=Et("h4",a.HEADING),Nt=Et("h5",a.HEADING),jt=Et("h6",a.HEADING),Rt=Et("blockquote",a.BLOCKQUOTE),zt=Et("code",a.CODE),Ft=Et("pre",a.CODE_BLOCK),Bt=Et("label",a.LABEL),Ht=Et("ol",a.LIST),Ut=Et("ul",a.LIST),Vt=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(c.__extends)(t,e),t.prototype.render=function(){return d.createElement(Je,Object(c.__assign)({},this.props,{className:a.OVERLAY_SCROLL_CONTAINER,hasBackdrop:!0}),d.createElement("div",{className:a.DIALOG_CONTAINER},d.createElement("div",{className:pe()(a.DIALOG,this.props.className),style:this.props.style},this.maybeRenderHeader(),this.props.children)))},t.prototype.validateProps=function(e){null==e.title&&(null!=e.icon&&console.warn(Ge.i),null!=e.isCloseButtonShown&&console.warn(Ge.h))},t.prototype.maybeRenderCloseButton=function(){return!1!==this.props.isCloseButtonShown?d.createElement(St,{"aria-label":"Close",className:a.DIALOG_CLOSE_BUTTON,icon:d.createElement(_t,{icon:"small-cross",iconSize:_t.SIZE_LARGE}),minimal:!0,onClick:this.props.onClose}):void 0},t.prototype.maybeRenderHeader=function(){var e=this.props,t=e.icon,n=e.title;if(null!=n)return d.createElement("div",{className:a.DIALOG_HEADER},d.createElement(_t,{icon:t,iconSize:_t.SIZE_LARGE}),d.createElement(Pt,null,n),this.maybeRenderCloseButton())},t.defaultProps={canOutsideClickClose:!0,isOpen:!1},t.displayName="Blueprint3.Dialog",t=Object(c.__decorate)([ve.polyfill],t)}(v),Yt=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.handleCancel=function(e){return t.internalHandleCallbacks(!1,e)},t.handleConfirm=function(e){return t.internalHandleCallbacks(!0,e)},t}return Object(c.__extends)(t,e),t.prototype.render=function(){var e=this.props,t=e.canEscapeKeyCancel,n=e.canOutsideClickCancel,r=e.children,i=e.className,o=e.icon,s=e.intent,l=e.loading,u=e.cancelButtonText,h=e.confirmButtonText,f=(e.onClose,Object(c.__rest)(e,["canEscapeKeyCancel","canOutsideClickCancel","children","className","icon","intent","loading","cancelButtonText","confirmButtonText","onClose"]));return d.createElement(Vt,Object(c.__assign)({},f,{className:pe()(a.ALERT,i),canEscapeKeyClose:t,canOutsideClickClose:n,onClose:this.handleCancel,portalContainer:this.props.portalContainer}),d.createElement("div",{className:a.ALERT_BODY},d.createElement(_t,{icon:o,iconSize:40,intent:s}),d.createElement("div",{className:a.ALERT_CONTENTS},r)),d.createElement("div",{className:a.ALERT_FOOTER},d.createElement(St,{loading:l,intent:s,text:h,onClick:this.handleConfirm}),u&&d.createElement(St,{text:u,disabled:l,onClick:this.handleCancel})))},t.prototype.validateProps=function(e){null==e.onClose&&null==e.cancelButtonText!=(null==e.onCancel)&&console.warn(Ge.c);var t=null!=e.onCancel||null!=e.onClose;e.canEscapeKeyCancel&&!t&&console.warn(Ge.a),e.canOutsideClickCancel&&!t&&console.warn(Ge.b)},t.prototype.internalHandleCallbacks=function(e,t){var n,r=this.props,i=r.onCancel,o=r.onClose,a=r.onConfirm;null===(n=e?a:i)||void 0===n||n(t),null==o||o(e,t)},t.defaultProps={canEscapeKeyCancel:!1,canOutsideClickCancel:!1,confirmButtonText:"OK",isOpen:!1,loading:!1},t.displayName="Blueprint3.Alert",t=Object(c.__decorate)([ve.polyfill],t)}(v),Wt=function(e){var t,n=pe()(a.BREADCRUMB,((t={})[a.BREADCRUMB_CURRENT]=e.current,t[a.DISABLED]=e.disabled,t),e.className),r=null!=e.icon?d.createElement(_t,{icon:e.icon}):void 0;return null==e.href&&null==e.onClick?d.createElement("span",{className:n},r,e.text,e.children):d.createElement("a",{className:n,href:e.href,onClick:e.disabled?void 0:e.onClick,tabIndex:e.disabled?void 0:0,target:e.target},r,e.text,e.children)},Gt=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(c.__extends)(t,e),t.prototype.render=function(){var e=this.props,t=e.className,n=e.title;return null==n?d.createElement("li",{className:pe()(a.MENU_DIVIDER,t)}):d.createElement("li",{className:pe()(a.MENU_HEADER,t)},d.createElement(jt,null,n))},t.displayName="Blueprint3.MenuDivider",t}(d.Component),Xt=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.state={isContentOverflowing:!1,textContent:""},t.textRef=null,t}return Object(c.__extends)(t,e),t.prototype.componentDidMount=function(){this.update()},t.prototype.componentDidUpdate=function(){this.update()},t.prototype.render=function(){var e,t=this,n=pe()(((e={})[a.TEXT_OVERFLOW_ELLIPSIS]=this.props.ellipsize,e),this.props.className),r=this.props,i=r.children,o=r.tagName,s=r.title;return d.createElement(o,{className:n,ref:function(e){return t.textRef=e},title:null!=s?s:this.state.isContentOverflowing?this.state.textContent:void 0},i)},t.prototype.update=function(){var e;if(null!=(null===(e=this.textRef)||void 0===e?void 0:e.textContent)){var t={isContentOverflowing:this.props.ellipsize&&this.textRef.scrollWidth>this.textRef.clientWidth,textContent:this.textRef.textContent};this.setState(t)}},t.displayName="Blueprint3.Text",t.defaultProps={ellipsize:!1,tagName:"div"},t=Object(c.__decorate)([ve.polyfill],t)}(v),qt=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(c.__extends)(t,e),t.prototype.render=function(){var e,t,n=this.props,r=n.active,i=n.className,o=n.children,s=n.disabled,l=n.icon,u=n.intent,h=(n.labelClassName,n.labelElement),f=n.multiline,p=(n.popoverProps,n.shouldDismissPopover),g=n.text,v=n.textClassName,m=n.tagName,y=void 0===m?"a":m,b=n.htmlTitle,C=Object(c.__rest)(n,["active","className","children","disabled","icon","intent","labelClassName","labelElement","multiline","popoverProps","shouldDismissPopover","text","textClassName","tagName","htmlTitle"]),A=null!=o,_=a.intentClass(u),I=pe()(a.MENU_ITEM,_,((e={})[a.ACTIVE]=r,e[a.INTENT_PRIMARY]=r&&null==_,e[a.DISABLED]=s,e[a.POPOVER_DISMISS]=p&&!s&&!A,e),i),x=d.createElement(y,Object(c.__assign)(Object(c.__assign)(Object(c.__assign)({},C),s?Qt:{}),{className:I}),d.createElement(_t,{icon:l}),d.createElement(Xt,{className:pe()(a.FILL,v),ellipsize:!f,title:b},g),this.maybeRenderLabel(h),A?d.createElement(_t,{icon:"caret-right"}):void 0),w=pe()(((t={})[a.MENU_SUBMENU]=A,t));return d.createElement("li",{className:w},this.maybeRenderPopover(x,o))},t.prototype.maybeRenderLabel=function(e){var t=this.props,n=t.label,r=t.labelClassName;return null==n&&null==e?null:d.createElement("span",{className:pe()(a.MENU_ITEM_LABEL,r)},n,e)},t.prototype.maybeRenderPopover=function(e,t){if(null==t)return e;var n=this.props,r=n.disabled,i=n.popoverProps;return d.createElement(pt,Object(c.__assign)({autoFocus:!1,captureDismiss:!1,disabled:r,enforceFocus:!1,hoverCloseDelay:0,interactionKind:ft.HOVER,modifiers:Zt,position:_.a.RIGHT_TOP,usePortal:!1},i,{content:d.createElement(Kt,null,t),minimal:!0,popoverClassName:pe()(a.MENU_SUBMENU,null==i?void 0:i.popoverClassName),target:e}))},t.defaultProps={disabled:!1,multiline:!1,popoverProps:{},shouldDismissPopover:!0,text:""},t.displayName="Blueprint3.MenuItem",t=Object(c.__decorate)([ve.polyfill],t)}(v),Zt={flip:{boundariesElement:"viewport",padding:20},offset:{offset:-5},preventOverflow:{boundariesElement:"viewport",padding:20}},Qt={href:void 0,onClick:void 0,onMouseDown:void 0,onMouseEnter:void 0,onMouseLeave:void 0,tabIndex:-1},Kt=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(c.__extends)(t,e),t.prototype.render=function(){var e,t=this.props,n=t.className,r=t.children,i=t.large,o=t.ulRef,s=Object(c.__rest)(t,["className","children","large","ulRef"]),l=pe()(a.MENU,((e={})[a.LARGE]=i,e),n);return d.createElement("ul",Object(c.__assign)({},s,{className:l,ref:o}),r)},t.displayName="Blueprint3.Menu",t.Divider=Gt,t.Item=qt,t=Object(c.__decorate)([ve.polyfill],t)}(v);!function(e){e[e.NONE=0]="NONE",e[e.GROW=1]="GROW",e[e.SHRINK=2]="SHRINK"}(Ot||(Ot={}));var Jt,$t=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.state={direction:Ot.NONE,lastOverflowCount:0,overflow:[],visible:t.props.items},t.previousWidths=new Map,t.spacer=null,t.resize=function(e){var n=e.some((function(e){var n=t.previousWidths.get(e.target)||0;return e.contentRect.width>n}));t.repartition(n),e.forEach((function(e){return t.previousWidths.set(e.target,e.contentRect.width)}))},t}return Object(c.__extends)(t,e),t.ofType=function(){return t},t.prototype.componentDidMount=function(){this.repartition(!1)},t.prototype.shouldComponentUpdate=function(e,t){return!(this.state!==t&&Y(this.state,t))},t.prototype.componentDidUpdate=function(e,t){var n,r;e.observeParents!==this.props.observeParents&&console.warn(Ge.y),e.collapseFrom===this.props.collapseFrom&&e.items===this.props.items&&e.minVisibleItems===this.props.minVisibleItems&&e.overflowRenderer===this.props.overflowRenderer&&e.visibleItemRenderer===this.props.visibleItemRenderer||this.setState({direction:Ot.GROW,lastOverflowCount:0,overflow:[],visible:this.props.items}),Y(t,this.state)||this.repartition(!1);var i=this.state,o=i.direction,a=i.overflow,s=i.lastOverflowCount;o===Ot.NONE&&o!==t.direction&&a.length!==s&&(null===(r=(n=this.props).onOverflow)||void 0===r||r.call(n,a))},t.prototype.render=function(){var e=this,t=this.props,n=t.className,r=t.collapseFrom,i=t.observeParents,o=t.style,s=t.tagName,l=void 0===s?"div":s,u=t.visibleItemRenderer,c=this.maybeRenderOverflow(),h=d.createElement(l,{className:pe()(a.OVERFLOW_LIST,n),style:o},r===y.START?c:null,this.state.visible.map(u),r===y.END?c:null,d.createElement("div",{className:a.OVERFLOW_LIST_SPACER,ref:function(t){return e.spacer=t}}));return d.createElement(et,{onResize:this.resize,observeParents:i},h)},t.prototype.maybeRenderOverflow=function(){var e=this.state.overflow;return 0===e.length?null:this.props.overflowRenderer(e)},t.prototype.repartition=function(e){var t=this;null!=this.spacer&&(e?this.setState((function(e){return{direction:Ot.GROW,lastOverflowCount:e.direction===Ot.NONE?e.overflow.length:e.lastOverflowCount,overflow:[],visible:t.props.items}})):this.spacer.getBoundingClientRect().width<.9?this.setState((function(e){if(e.visible.length<=t.props.minVisibleItems)return null;var n=t.props.collapseFrom===y.START,r=e.visible.slice(),i=n?r.shift():r.pop();if(void 0===i)return null;var o=n?Object(c.__spreadArrays)(e.overflow,[i]):Object(c.__spreadArrays)([i],e.overflow);return{direction:e.direction===Ot.NONE?Ot.SHRINK:e.direction,overflow:o,visible:r}})):this.setState({direction:Ot.NONE}))},t.displayName="Blueprint3.OverflowList",t.defaultProps={collapseFrom:y.START,minVisibleItems:0},t}(d.Component),en=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.renderOverflow=function(e){var n=t.props.collapseFrom,r=n===y.END?_.a.BOTTOM_RIGHT:_.a.BOTTOM_LEFT,i=e;return n===y.START&&(i=e.slice().reverse()),d.createElement("li",null,d.createElement(pt,Object(c.__assign)({position:r},t.props.popoverProps),d.createElement("span",{className:a.BREADCRUMBS_COLLAPSED}),d.createElement(Kt,null,i.map(t.renderOverflowBreadcrumb))))},t.renderOverflowBreadcrumb=function(e,t){var n=null!=e.href||null!=e.onClick,r=x(e);return d.createElement(qt,Object(c.__assign)({disabled:!n},r,{text:e.text,key:t}))},t.renderBreadcrumbWrapper=function(e,n){var r=t.props.items[t.props.items.length-1]===e;return d.createElement("li",{key:n},t.renderBreadcrumb(e,r))},t}return Object(c.__extends)(t,e),t.prototype.render=function(){var e=this.props,t=e.className,n=e.collapseFrom,r=e.items,i=e.minVisibleItems,o=e.overflowListProps,s=void 0===o?{}:o;return d.createElement($t,Object(c.__assign)({collapseFrom:n,minVisibleItems:i,tagName:"ul"},s,{className:pe()(a.BREADCRUMBS,s.className,t),items:r,overflowRenderer:this.renderOverflow,visibleItemRenderer:this.renderBreadcrumbWrapper}))},t.prototype.renderBreadcrumb=function(e,t){return t&&null!=this.props.currentBreadcrumbRenderer?this.props.currentBreadcrumbRenderer(e):null!=this.props.breadcrumbRenderer?this.props.breadcrumbRenderer(e):d.createElement(Wt,Object(c.__assign)({current:t},e))},t.defaultProps={collapseFrom:y.START},t=Object(c.__decorate)([ve.polyfill],t)}(v),tn=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(c.__extends)(t,e),t.prototype.render=function(){var e,t=this.props,n=t.alignText,r=t.className,i=t.fill,o=t.minimal,s=t.large,l=t.vertical,u=Object(c.__rest)(t,["alignText","className","fill","minimal","large","vertical"]),h=pe()(a.BUTTON_GROUP,((e={})[a.FILL]=i,e[a.LARGE]=s,e[a.MINIMAL]=o,e[a.VERTICAL]=l,e),a.alignmentClass(n),r);return d.createElement("div",Object(c.__assign)({},u,{className:h}),this.props.children)},t.displayName="Blueprint3.ButtonGroup",t=Object(c.__decorate)([ve.polyfill],t)}(v),nn=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(c.__extends)(t,e),t.prototype.render=function(){var e,t=this.props,n=t.className,r=t.children,i=t.icon,o=t.intent,s=t.title,l=Object(c.__rest)(t,["className","children","icon","intent","title"]),u=this.getIconName(i,o),h=pe()(a.CALLOUT,a.intentClass(o),((e={})[a.CALLOUT_ICON]=null!=u,e),n);return d.createElement("div",Object(c.__assign)({className:h},l),u&&d.createElement(_t,{icon:u,iconSize:_t.SIZE_LARGE}),s&&d.createElement(Pt,null,s),r)},t.prototype.getIconName=function(e,t){if(null!==e){if(void 0!==e)return e;switch(t){case A.a.DANGER:return"error";case A.a.PRIMARY:return"info-sign";case A.a.WARNING:return"warning-sign";case A.a.SUCCESS:return"tick";default:return}}},t.displayName="Blueprint3.Callout",t=Object(c.__decorate)([ve.polyfill],t)}(v),rn=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(c.__extends)(t,e),t.prototype.render=function(){var e,t=this.props,n=t.className,r=t.elevation,i=t.interactive,o=Object(c.__rest)(t,["className","elevation","interactive"]),s=pe()(a.CARD,((e={})[a.INTERACTIVE]=i,e),a.elevationClass(r),n);return d.createElement("div",Object(c.__assign)({className:s},o))},t.displayName="Blueprint3.Card",t.defaultProps={elevation:C.a.ZERO,interactive:!1},t=Object(c.__decorate)([ve.polyfill],t)}(v);!function(e){e[e.OPEN_START=0]="OPEN_START",e[e.OPENING=1]="OPENING",e[e.OPEN=2]="OPEN",e[e.CLOSING_START=3]="CLOSING_START",e[e.CLOSING=4]="CLOSING",e[e.CLOSED=5]="CLOSED"}(Jt||(Jt={}));var on=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.state={animationState:t.props.isOpen?Jt.OPEN:Jt.CLOSED,height:void 0,heightWhenOpen:void 0},t.contents=null,t.contentsRefHandler=function(e){if(t.contents=e,null!=t.contents){var n=t.contents.clientHeight;t.setState({animationState:t.props.isOpen?Jt.OPEN:Jt.CLOSED,height:0===n?void 0:n+"px",heightWhenOpen:0===n?void 0:n})}},t}return Object(c.__extends)(t,e),t.getDerivedStateFromProps=function(e,t){var n=e.isOpen,r=t.animationState;if(n)switch(r){case Jt.OPEN:case Jt.OPENING:break;default:return{animationState:Jt.OPEN_START}}else switch(r){case Jt.CLOSED:case Jt.CLOSING:break;default:return{animationState:Jt.CLOSING_START,height:t.heightWhenOpen+"px"}}return null},t.prototype.render=function(){var e=this.state.animationState!==Jt.CLOSED,t=e||this.props.keepChildrenMounted,n=e&&this.state.animationState!==Jt.CLOSING,r="auto"===this.state.height,i={height:e?this.state.height:void 0,overflowY:r?"visible":void 0,transition:r?"none":void 0},o={transform:n?"translateY(0)":"translateY(-"+this.state.heightWhenOpen+"px)",transition:r?"none":void 0};return d.createElement(this.props.component,{className:pe()(a.COLLAPSE,this.props.className),style:i},d.createElement("div",{className:a.COLLAPSE_BODY,ref:this.contentsRefHandler,style:o,"aria-hidden":!e&&this.props.keepChildrenMounted},t?this.props.children:null))},t.prototype.componentDidMount=function(){this.forceUpdate(),this.props.isOpen?this.setState({animationState:Jt.OPEN,height:"auto"}):this.setState({animationState:Jt.CLOSED,height:"0px"})},t.prototype.componentDidUpdate=function(){var e=this;if(null!=this.contents){var t=this.props.transitionDuration,n=this.state.animationState;if(n===Jt.OPEN_START){var r=this.contents.clientHeight;this.setState({animationState:Jt.OPENING,height:r+"px",heightWhenOpen:r}),this.setTimeout((function(){return e.onDelayedStateChange()}),t)}else if(n===Jt.CLOSING_START){var i=this.contents.clientHeight;this.setTimeout((function(){return e.setState({animationState:Jt.CLOSING,height:"0px",heightWhenOpen:i})})),this.setTimeout((function(){return e.onDelayedStateChange()}),t)}}},t.prototype.onDelayedStateChange=function(){switch(this.state.animationState){case Jt.OPENING:this.setState({animationState:Jt.OPEN,height:"auto"});break;case Jt.CLOSING:this.setState({animationState:Jt.CLOSED})}},t.displayName="Blueprint3.Collapse",t.defaultProps={component:"div",isOpen:!1,keepChildrenMounted:!1,transitionDuration:200},t=Object(c.__decorate)([ve.polyfill],t)}(v),an=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(c.__extends)(t,e),t.prototype.render=function(){var e,t=this,n=this.props.collapseFrom,r=d.Children.count(this.props.children),i=this.partitionChildren(),o=i[0],s=i[1],l=o.map((function(e,i){var o=n===y.START?r-1-i:i;return d.createElement("li",{className:t.props.visibleItemClassName,key:o},t.props.visibleItemRenderer(e.props,o))}));if(n===y.START&&l.reverse(),s.length>0){var u=n===y.END?_.a.BOTTOM_RIGHT:_.a.BOTTOM_LEFT;e=d.createElement("li",{className:this.props.visibleItemClassName},d.createElement(pt,Object(c.__assign)({content:d.createElement(Kt,null,s),position:u},this.props.dropdownProps),this.props.dropdownTarget))}return d.createElement("ul",{className:pe()(a.COLLAPSIBLE_LIST,this.props.className)},n===y.START?e:null,l,n===y.END?e:null)},t.prototype.partitionChildren=function(){var e=d.Children.map(this.props.children,(function(e,t){if(!ce(e,qt))throw new Error(Ge.e);return d.cloneElement(e,{key:"visible-"+t})}));if(null==e)return[[],[]];this.props.collapseFrom===y.START&&e.reverse();var t=this.props.visibleItemCount;return[e.slice(0,t),e.slice(t)]},t.displayName="Blueprint3.CollapsibleList",t.defaultProps={collapseFrom:y.START,visibleItemCount:3},t}(d.Component);function sn(e){var t;return K(e.prototype.renderContextMenu)||console.warn(Ge.g),(t=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(c.__extends)(t,e),t.prototype.render=function(){var t=this,n=e.prototype.render.call(this);if(null==n)return n;if(!d.isValidElement(n))return console.warn(Ge.f),n;var r=n.props.onContextMenu;return d.cloneElement(n,{onContextMenu:function(e){if(!e.defaultPrevented){if(K(t.renderContextMenu)){var n=t.renderContextMenu(e);if(null!=n){var i=function(e){return e instanceof Element&&null!=e.closest("."+a.DARK)}(ge.findDOMNode(t));e.preventDefault(),mt(n,{left:e.clientX,top:e.clientY},t.onContextMenuClose,i)}}null==r||r(e)}}})},t}(e)).displayName="ContextMenuTarget("+ue(e)+")",t}var ln=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(c.__extends)(t,e),t.prototype.render=function(){var e=this.props.className;return d.createElement("div",{className:a.DIALOG_STEP_CONTAINER},d.createElement("div",{className:pe()(a.DIALOG_STEP,e),role:"dialogsteplist"}))},t.displayName="Blueprint3.DialogStep",t=Object(c.__decorate)([ve.polyfill],t)}(v),un={lastViewedIndex:0,selectedIndex:0},cn=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.state=un,t.renderDialogStep=function(e,n){var r,i,o=n+1,s=t.state.lastViewedIndex>=n,l=t.state.selectedIndex===n;return d.createElement("div",{className:pe()(a.DIALOG_STEP_CONTAINER,(r={},r[a.ACTIVE]=s,r)),key:n},d.createElement("div",{className:a.DIALOG_STEP,onClick:t.handleClickDialogStep(n)},d.createElement("div",{className:a.DIALOG_STEP_ICON},o),d.createElement("div",{className:pe()(a.DIALOG_STEP_TITLE,(i={},i[a.ACTIVE]=l,i))},e.props.title)))},t.handleClickDialogStep=function(e){if(!(e>t.state.lastViewedIndex))return t.getDialogStepChangeHandler(e)},t}return Object(c.__extends)(t,e),t.prototype.render=function(){return d.createElement(Vt,Object(c.__assign)({},this.props,{style:this.getDialogStyle()}),d.createElement("div",{className:a.MULTISTEP_DIALOG_PANELS},this.renderLeftPanel(),this.maybeRenderRightPanel()))},t.prototype.componentDidUpdate=function(e){void 0!==e.resetOnClose&&!e.resetOnClose||e.isOpen||!this.props.isOpen||this.setState(un)},t.prototype.getDialogStyle=function(){return Object(c.__assign)({minWidth:800,paddingBottom:0},this.props.style)},t.prototype.renderLeftPanel=function(){return d.createElement("div",{className:a.MULTISTEP_DIALOG_LEFT_PANEL},this.getDialogStepChildren().filter(dn).map(this.renderDialogStep))},t.prototype.maybeRenderRightPanel=function(){var e=this.getDialogStepChildren();if(e.length<=this.state.selectedIndex)return null;var t=e[this.state.selectedIndex].props,n=t.className,r=t.panel,i=t.panelClassName;return d.createElement("div",{className:pe()(a.MULTISTEP_DIALOG_RIGHT_PANEL,n,i)},r,this.renderFooter())},t.prototype.renderFooter=function(){return d.createElement("div",{className:a.MULTISTEP_DIALOG_FOOTER},d.createElement("div",{className:a.DIALOG_FOOTER_ACTIONS},this.renderButtons()))},t.prototype.renderButtons=function(){var e=[];return this.state.selectedIndex>0&&e.push(d.createElement(St,{key:"back",onClick:this.getBackClickHandler(),text:"Back"})),this.state.selectedIndex===this.getDialogStepChildren().length-1?e.push(this.renderFinalButton()):e.push(d.createElement(St,Object(c.__assign)({intent:"primary",key:"next",onClick:this.getNextClickHandler(),text:"Next"},this.props.nextButtonProps))),e},t.prototype.getBackClickHandler=function(){return this.getDialogStepChangeHandler(this.state.selectedIndex-1)},t.prototype.getNextClickHandler=function(){return this.getDialogStepChangeHandler(this.state.selectedIndex+1)},t.prototype.getDialogStepChangeHandler=function(e){var t=this;return function(n){if(void 0!==t.props.onChange){var r=t.getDialogStepChildren(),i=r[t.state.selectedIndex].props.id,o=r[e].props.id;t.props.onChange(o,i,n)}t.setState({lastViewedIndex:Math.max(t.state.lastViewedIndex,e),selectedIndex:e})}},t.prototype.renderFinalButton=function(){return d.createElement(St,Object(c.__assign)({intent:"primary",key:"final",text:"Submit"},this.props.finalButtonProps))},t.prototype.getDialogStepChildren=function(e){return void 0===e&&(e=this.props),d.Children.toArray(e.children).filter(dn)},t.displayName="Blueprint3.MultistepDialog",t.defaultProps={canOutsideClickClose:!0,isOpen:!1},t=Object(c.__decorate)([ve.polyfill],t)}(v);function dn(e){return i.isElementOfType(e,ln)}var hn=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(c.__extends)(t,e),t.prototype.render=function(){var e=this.props,t=e.className,n=e.tagName,r=void 0===n?"div":n,i=Object(c.__rest)(e,["className","tagName"]),o=pe()(a.DIVIDER,t);return d.createElement(r,Object(c.__assign)(Object(c.__assign)({},i),{className:o}))},t.displayName="Blueprint3.Divider",t=Object(c.__decorate)([ve.polyfill],t)}(v),fn=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.handleOpening=function(e){var n,r;t.lastActiveElementBeforeOpened=document.activeElement,null===(r=(n=t.props).onOpening)||void 0===r||r.call(n,e)},t.handleClosed=function(e){var n,r;t.lastActiveElementBeforeOpened instanceof HTMLElement&&t.lastActiveElementBeforeOpened.focus(),null===(r=(n=t.props).onClosed)||void 0===r||r.call(n,e)},t}return Object(c.__extends)(t,e),t.prototype.render=function(){var e,t,n,r=this.props,i=r.size,o=r.style,s=r.position,l=r.vertical,u=s?Object(_.b)(s):void 0,h=pe()(a.DRAWER,((e={})[a.VERTICAL]=!u&&l,e[null!==(n=a.positionClass(u))&&void 0!==n?n:""]=!0,e),this.props.className),f=null==i?o:Object(c.__assign)(Object(c.__assign)({},o),((t={})[(u?Object(_.c)(u):l)?"height":"width"]=i,t));return d.createElement(Je,Object(c.__assign)({},this.props,{className:a.OVERLAY_CONTAINER,onOpening:this.handleOpening,onClosed:this.handleClosed}),d.createElement("div",{className:h,style:f},this.maybeRenderHeader(),this.props.children))},t.prototype.validateProps=function(e){null==e.title&&(null!=e.icon&&console.warn(Ge.i),null!=e.isCloseButtonShown&&console.warn(Ge.h)),null!=e.position&&(e.vertical&&console.warn(Ge.k),e.position!==Object(_.b)(e.position)&&console.warn(Ge.j))},t.prototype.maybeRenderCloseButton=function(){return!1!==this.props.isCloseButtonShown?d.createElement(St,{"aria-label":"Close",className:a.DIALOG_CLOSE_BUTTON,icon:d.createElement(_t,{icon:"small-cross",iconSize:_t.SIZE_LARGE}),minimal:!0,onClick:this.props.onClose}):null},t.prototype.maybeRenderHeader=function(){var e=this.props,t=e.icon,n=e.title;return null==n?null:d.createElement("div",{className:a.DRAWER_HEADER},d.createElement(_t,{icon:t,iconSize:_t.SIZE_LARGE}),d.createElement(Pt,null,n),this.maybeRenderCloseButton())},t.displayName="Blueprint3.Drawer",t.defaultProps={canOutsideClickClose:!0,isOpen:!1,style:{},vertical:!1},t.SIZE_SMALL="360px",t.SIZE_STANDARD="50%",t.SIZE_LARGE="90%",t=Object(c.__decorate)([ve.polyfill],t)}(v),pn="undefined"!=typeof navigator?navigator.userAgent:"",gn={isEdge:/Edge/.test(pn),isInternetExplorer:/Trident|rv:11/.test(pn),isWebkit:/AppleWebKit/.test(pn)},vn=function(){return gn.isInternetExplorer},mn=function(e){function t(t,n){var i=e.call(this,t,n)||this;i.inputElement=null,i.valueElement=null,i.refHandlers={content:function(e){i.valueElement=e},input:function(e){if(null!=e&&(i.inputElement=e,i.props.alwaysRenderInput||i.inputElement.focus(),null!=i.state&&i.state.isEditing)){var t=function(e){switch(e.type){case"textarea":return!0;case"text":case"search":case"tel":case"url":case"password":return!0;default:return!1}}(e);if(t){var n=e.value.length;e.setSelectionRange(i.props.selectAllOnFocus?0:n,n)}t&&i.props.selectAllOnFocus||(e.scrollLeft=e.scrollWidth)}}},i.cancelEditing=function(){var e,t,n,r,o=i.state,a=o.lastValue,s=o.value;i.setState({isEditing:!1,value:a}),s!==a&&(null===(t=(e=i.props).onChange)||void 0===t||t.call(e,a)),null===(r=(n=i.props).onCancel)||void 0===r||r.call(n,a)},i.toggleEditing=function(){var e,t;if(i.state.isEditing){var n=i.state.value;i.setState({isEditing:!1,lastValue:n}),null===(t=(e=i.props).onConfirm)||void 0===t||t.call(e,n)}else i.props.disabled||i.setState({isEditing:!0})},i.handleFocus=function(){var e=i.props,t=e.alwaysRenderInput,n=e.disabled,r=e.selectAllOnFocus;if(n||i.setState({isEditing:!0}),t&&r&&null!=i.inputElement){var o=i.inputElement.value.length;i.inputElement.setSelectionRange(0,o)}},i.handleTextChange=function(e){var t,n,r=e.target.value;null==i.props.value&&i.setState({value:r}),null===(n=(t=i.props).onChange)||void 0===n||n.call(t,r)},i.handleKeyEvent=function(e){var t=e.altKey,n=e.ctrlKey,o=e.metaKey,a=e.shiftKey,s=e.which;if(s!==r.ESCAPE){var l=t||n||o||a;s===r.ENTER&&((t||a)&&e.preventDefault(),i.props.confirmOnEnterKey&&i.props.multiline?null!=e.target&&l?(!function(e,t){var n=e.selectionEnd,r=e.selectionStart,i=e.value;if(r>=0){var o=i.substring(0,r),a=i.substring(n,i.length),s=t.length;e.value=""+o+t+a,e.selectionStart=r+s,e.selectionEnd=r+s}}(e.target,"\n"),i.handleTextChange(e)):i.toggleEditing():i.props.multiline&&!l||i.toggleEditing())}else i.cancelEditing()};var o=null==t.value?t.defaultValue:t.value;return i.state={inputHeight:0,inputWidth:0,isEditing:!0===t.isEditing&&!1===t.disabled,lastValue:o,value:o},i}return Object(c.__extends)(t,e),t.prototype.render=function(){var e,t,n,r=this.props,i=r.alwaysRenderInput,o=r.disabled,s=r.multiline,l=null!==(t=this.props.value)&&void 0!==t?t:this.state.value,u=null!=l&&""!==l,c=pe()(a.EDITABLE_TEXT,a.intentClass(this.props.intent),((e={})[a.DISABLED]=o,e[a.EDITABLE_TEXT_EDITING]=this.state.isEditing,e[a.EDITABLE_TEXT_PLACEHOLDER]=!u,e[a.MULTILINE]=s,e),this.props.className);n=s?{height:this.state.isEditing?void 0:this.state.inputHeight}:{height:this.state.inputHeight,lineHeight:null!=this.state.inputHeight?this.state.inputHeight+"px":void 0,minWidth:this.props.minWidth};var h=i||this.state.isEditing||o?void 0:0,f=i&&!this.state.isEditing;return d.createElement("div",{className:c,onFocus:this.handleFocus,tabIndex:h},i||this.state.isEditing?this.renderInput(l):void 0,f?void 0:d.createElement("span",{className:a.EDITABLE_TEXT_CONTENT,ref:this.refHandlers.content,style:n},u?l:this.props.placeholder))},t.prototype.componentDidMount=function(){this.updateInputDimensions()},t.prototype.componentDidUpdate=function(e,t){var n,r,i={};this.props.value===e.value||null==e.value&&null==this.props.value||(i.value=this.props.value),null!=this.props.isEditing&&this.props.isEditing!==e.isEditing&&(i.isEditing=this.props.isEditing),(this.props.disabled||null==this.props.disabled&&e.disabled)&&(i.isEditing=!1),this.setState(i),this.state.isEditing&&!t.isEditing&&(null===(r=(n=this.props).onEdit)||void 0===r||r.call(n,this.state.value)),this.updateInputDimensions()},t.prototype.renderInput=function(e){var t=this.props,n=t.disabled,r=t.maxLength,i=t.multiline,o=t.type,s=t.placeholder,l={className:a.EDITABLE_TEXT_INPUT,disabled:n,maxLength:r,onBlur:this.toggleEditing,onChange:this.handleTextChange,onKeyDown:this.handleKeyEvent,placeholder:s,value:e},u=this.state,h=u.inputHeight,f=u.inputWidth;return 0!==h&&0!==f&&(l.style={height:h,lineHeight:i||null==h?void 0:h+"px",width:i?"100%":f}),i?d.createElement("textarea",Object(c.__assign)({ref:this.refHandlers.input},l)):d.createElement("input",Object(c.__assign)({ref:this.refHandlers.input,type:o},l))},t.prototype.updateInputDimensions=function(){if(null!=this.valueElement){var e=this.props,t=e.maxLines,n=e.minLines,r=e.minWidth,i=e.multiline,o=this.valueElement,a=o.parentElement,s=o.textContent,l=this.valueElement,u=l.scrollHeight,c=l.scrollWidth,d=yn(this.valueElement);i&&this.state.isEditing&&/\n$/.test(null!=s?s:"")&&(u+=d),d>0&&(u=Object(h.c)(u,n*d,t*d)),u=Math.max(u,(f=this.valueElement,(""===(p=getComputedStyle(f).fontSize)?0:parseInt(p.slice(0,-2),10))+1),yn(a)),c+=vn()?30:5,this.setState({inputHeight:u,inputWidth:Math.max(c,r)}),i&&this.state.isEditing&&this.setTimeout((function(){return a.style.height=u+"px"}))}var f,p},t.displayName="Blueprint3.EditableText",t.defaultProps={alwaysRenderInput:!1,confirmOnEnterKey:!1,defaultValue:"",disabled:!1,maxLines:1/0,minLines:1,minWidth:80,multiline:!1,placeholder:"Click to Edit",type:"text"},t=Object(c.__decorate)([ve.polyfill],t)}(v);function yn(e){var t=parseInt(getComputedStyle(e).lineHeight.slice(0,-2),10);if(isNaN(t)){var n=document.createElement("span");n.innerHTML="<br>",e.appendChild(n);var r=e.offsetHeight;n.innerHTML="<br><br>";var i=e.offsetHeight;e.removeChild(n),t=i-r}return t}var bn,Cn=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(c.__extends)(t,e),t.prototype.render=function(){var e,t=this.props,n=t.children,r=t.className,i=t.fill,o=t.vertical,s=Object(c.__rest)(t,["children","className","fill","vertical"]),l=pe()(a.CONTROL_GROUP,((e={})[a.FILL]=i,e[a.VERTICAL]=o,e),r);return d.createElement("div",Object(c.__assign)({},s,{className:l}),n)},t.displayName="Blueprint3.ControlGroup",t=Object(c.__decorate)([ve.polyfill],t)}(v),An=function(e){var t,n=e.alignIndicator,r=e.children,i=e.className,o=e.indicatorChildren,s=e.inline,l=e.inputRef,u=e.label,h=e.labelElement,f=e.large,p=e.style,g=e.type,v=e.typeClassName,m=e.tagName,y=void 0===m?"label":m,b=Object(c.__rest)(e,["alignIndicator","children","className","indicatorChildren","inline","inputRef","label","labelElement","large","style","type","typeClassName","tagName"]),C=pe()(a.CONTROL,v,((t={})[a.DISABLED]=b.disabled,t[a.INLINE]=s,t[a.LARGE]=f,t),a.alignmentClass(n),i);return d.createElement(y,{className:C,style:p},d.createElement("input",Object(c.__assign)({},b,{ref:l,type:g})),d.createElement("span",{className:a.CONTROL_INDICATOR},o),u,h,r)},_n=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(c.__extends)(t,e),t.prototype.render=function(){var e=this.props,t=e.innerLabelChecked,n=e.innerLabel,r=Object(c.__rest)(e,["innerLabelChecked","innerLabel"]),i=n||t?[d.createElement("div",{key:"checked",className:a.CONTROL_INDICATOR_CHILD},d.createElement("div",{className:a.SWITCH_INNER_TEXT},t||n)),d.createElement("div",{key:"unchecked",className:a.CONTROL_INDICATOR_CHILD},d.createElement("div",{className:a.SWITCH_INNER_TEXT},n))]:null;return d.createElement(An,Object(c.__assign)({},r,{type:"checkbox",typeClassName:a.SWITCH,indicatorChildren:i}))},t.displayName="Blueprint3.Switch",t=Object(c.__decorate)([ve.polyfill],t)}(v),In=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(c.__extends)(t,e),t.prototype.render=function(){return d.createElement(An,Object(c.__assign)({},this.props,{type:"radio",typeClassName:a.RADIO}))},t.displayName="Blueprint3.Radio",t=Object(c.__decorate)([ve.polyfill],t)}(v),xn=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.state={indeterminate:t.props.indeterminate||t.props.defaultIndeterminate||!1},t.input=null,t.handleInputRef=O(t,"input",t.props.inputRef),t.handleChange=function(e){var n,r,i=e.target.indeterminate;null==t.props.indeterminate&&t.setState({indeterminate:i}),null===(r=(n=t.props).onChange)||void 0===r||r.call(n,e)},t}return Object(c.__extends)(t,e),t.getDerivedStateFromProps=function(e){var t=e.indeterminate;return null!=t?{indeterminate:t}:null},t.prototype.render=function(){var e=this.props,t=(e.defaultIndeterminate,e.indeterminate,Object(c.__rest)(e,["defaultIndeterminate","indeterminate"]));return d.createElement(An,Object(c.__assign)({},t,{inputRef:this.handleInputRef,onChange:this.handleChange,type:"checkbox",typeClassName:a.CHECKBOX}))},t.prototype.componentDidMount=function(){this.updateIndeterminate()},t.prototype.componentDidUpdate=function(){this.updateIndeterminate()},t.prototype.updateIndeterminate=function(){null!=this.input&&(this.input.indeterminate=this.state.indeterminate)},t.displayName="Blueprint3.Checkbox",t=Object(c.__decorate)([ve.polyfill],t)}(v),wn=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.handleInputChange=function(e){var n,r,i,o;null===(r=(n=t.props).onInputChange)||void 0===r||r.call(n,e),null===(o=null===(i=t.props.inputProps)||void 0===i?void 0:i.onChange)||void 0===o||o.call(i,e)},t}return Object(c.__extends)(t,e),t.prototype.render=function(){var e,t,n,r=this.props,i=r.buttonText,o=r.className,s=r.disabled,l=r.fill,u=r.hasSelection,h=r.inputProps,f=r.large,p=(r.onInputChange,r.text),g=Object(c.__rest)(r,["buttonText","className","disabled","fill","hasSelection","inputProps","large","onInputChange","text"]),v=pe()(a.FILE_INPUT,((e={})[a.FILE_INPUT_HAS_SELECTION]=u,e[a.DISABLED]=s,e[a.FILL]=l,e[a.LARGE]=f,e),o),m=((t={})[a.getClassNamespace()+"-button-text"]=i,t.className=pe()(a.FILE_UPLOAD_INPUT,((n={})[a.FILE_UPLOAD_INPUT_CUSTOM_TEXT]=!!i,n)),t);return d.createElement("label",Object(c.__assign)({},g,{className:v}),d.createElement("input",Object(c.__assign)({},h,{onChange:this.handleInputChange,type:"file",disabled:s})),d.createElement("span",Object(c.__assign)({},m),p))},t.displayName="Blueprint3.FileInput",t.defaultProps={hasSelection:!1,inputProps:{},text:"Choose file..."},t=Object(c.__decorate)([ve.polyfill],t)}(v),Mn=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(c.__extends)(t,e),t.prototype.render=function(){var e=this.props,t=e.children,n=e.contentClassName,r=e.helperText,i=e.label,o=e.labelFor,s=e.labelInfo,l=e.style;return d.createElement("div",{className:this.getClassName(),style:l},i&&d.createElement("label",{className:a.LABEL,htmlFor:o},i," ",d.createElement("span",{className:a.TEXT_MUTED},s)),d.createElement("div",{className:pe()(a.FORM_CONTENT,n)},t,r&&d.createElement("div",{className:a.FORM_HELPER_TEXT},r)))},t.prototype.getClassName=function(){var e,t=this.props,n=t.className,r=t.disabled,i=t.inline,o=t.intent;return pe()(a.FORM_GROUP,a.intentClass(o),((e={})[a.DISABLED]=r,e[a.INLINE]=i,e),n)},t.displayName="Blueprint3.FormGroup",t=Object(c.__decorate)([ve.polyfill],t)}(v),Sn=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.state={hasPendingUpdate:!1,isComposing:!1,nextValue:t.props.value,value:t.props.value},t.handleCompositionStart=function(e){var n,r;t.setState({isComposing:!0,nextValue:t.state.value}),null===(r=(n=t.props).onCompositionStart)||void 0===r||r.call(n,e)},t.handleCompositionEnd=function(e){var n,r;t.setState({isComposing:!1}),null===(r=(n=t.props).onCompositionEnd)||void 0===r||r.call(n,e)},t.handleChange=function(e){var n,r,i=e.target.value;t.setState({nextValue:i}),null===(r=(n=t.props).onChange)||void 0===r||r.call(n,e)},t}return Object(c.__extends)(t,e),t.getDerivedStateFromProps=function(e,t){return t.isComposing||void 0===e.value?null:t.nextValue!==t.value?e.value===t.nextValue?t.hasPendingUpdate?{value:e.value,hasPendingUpdate:!1}:{value:t.nextValue}:e.value===t.value?{hasPendingUpdate:!0}:{value:e.value,nextValue:e.value,hasPendingUpdate:!1}:{value:e.value,nextValue:e.value,hasPendingUpdate:!1}},t.prototype.render=function(){var e=this.state,t=e.isComposing,n=e.hasPendingUpdate,r=e.value,i=e.nextValue,o=this.props,a=o.inputRef,s=Object(c.__rest)(o,["inputRef"]);return d.createElement("input",Object(c.__assign)({},s,{ref:a,value:t||n?i:r,onCompositionStart:this.handleCompositionStart,onCompositionEnd:this.handleCompositionEnd,onChange:this.handleChange}))},t.displayName="Blueprint3.AsyncControllableInput",t=Object(c.__decorate)([ve.polyfill],t)}(d.PureComponent),Tn=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.state={},t.leftElement=null,t.rightElement=null,t.refHandlers={leftElement:function(e){return t.leftElement=e},rightElement:function(e){return t.rightElement=e}},t}return Object(c.__extends)(t,e),t.prototype.render=function(){var e,t=this.props,n=t.asyncControl,r=void 0!==n&&n,i=t.className,o=t.disabled,s=t.fill,l=t.inputRef,u=t.intent,h=t.large,f=t.small,p=t.round,g=pe()(a.INPUT_GROUP,a.intentClass(u),((e={})[a.DISABLED]=o,e[a.FILL]=s,e[a.LARGE]=h,e[a.SMALL]=f,e[a.ROUND]=p,e),i),v=Object(c.__assign)(Object(c.__assign)({},this.props.style),{paddingLeft:this.state.leftElementWidth,paddingRight:this.state.rightElementWidth}),m=Object(c.__assign)(Object(c.__assign)({type:"text"},x(this.props)),{className:a.INPUT,style:v});return d.createElement("div",{className:g},this.maybeRenderLeftElement(),r?d.createElement(Sn,Object(c.__assign)({},m,{inputRef:l})):d.createElement("input",Object(c.__assign)({},m,{ref:l})),this.maybeRenderRightElement())},t.prototype.componentDidMount=function(){this.updateInputWidth()},t.prototype.componentDidUpdate=function(e){var t=this.props,n=t.leftElement,r=t.rightElement;e.leftElement===n&&e.rightElement===r||this.updateInputWidth()},t.prototype.validateProps=function(e){null!=e.leftElement&&null!=e.leftIcon&&console.warn(Ge.o)},t.prototype.maybeRenderLeftElement=function(){var e=this.props,t=e.leftElement,n=e.leftIcon;return null!=t?d.createElement("span",{className:a.INPUT_LEFT_CONTAINER,ref:this.refHandlers.leftElement},t):null!=n?d.createElement(_t,{icon:n}):void 0},t.prototype.maybeRenderRightElement=function(){var e=this.props.rightElement;if(null!=e)return d.createElement("span",{className:a.INPUT_ACTION,ref:this.refHandlers.rightElement},e)},t.prototype.updateInputWidth=function(){var e=this.state,t=e.leftElementWidth,n=e.rightElementWidth;if(null!=this.leftElement){var r=this.leftElement.clientWidth;(void 0===t||Math.abs(r-t)>2)&&this.setState({leftElementWidth:r})}else this.setState({leftElementWidth:void 0});if(null!=this.rightElement){r=this.rightElement.clientWidth;(void 0===n||Math.abs(r-n)>2)&&this.setState({rightElementWidth:r})}else this.setState({rightElementWidth:void 0})},t.displayName="Blueprint3.InputGroup",t=Object(c.__decorate)([ve.polyfill],t)}(v);function En(e){var t=1.9.toLocaleString(e),n=1..toLocaleString(e),r=9..toLocaleString(e),i=new RegExp(n+"(.+)"+r).exec(t);return i&&i[1]||"."}function On(e,t){return void 0===t&&(t="en-US"),Nn(e.toLocaleString(t),t)}function kn(e){return void 0===e&&(e=""),e.toString()}function Ln(e,t){var n=""+e;if(parseFloat(n).toString()===e.toString())return e.toString();if(void 0!==t){var r=En(t);return Nn(n,t).split("").map((function(e){return function(e,t){var n=[0,1,2,3,4,5,6,7,8,9].map((function(e){return e.toLocaleString(t)})).indexOf(e);return-1!==n?n:e}(e,t)})).join("").replace(r,".")}return e.toString()}function Dn(e,t){return null==e.key||(!!(e.ctrlKey||e.altKey||e.metaKey)||(!(1===e.key.length)||Pn(e.key,t)))}function Pn(e,t){if(void 0!==t){var n=En(t).replace(".","\\."),r=[0,1,2,3,4,5,6,7,8,9].map((function(e){return e.toLocaleString(t)})).join("");return new RegExp("^[Ee"+r+"\\+\\-"+n+"]$").test(e)}return/^[Ee0-9\+\-\.]$/.test(e)}function Nn(e,t){return function(e){return e.replace(/[\uFF10-\uFF19]/g,(function(e){return String.fromCharCode(e.charCodeAt(0)-65248)}))}(e).split("").filter((function(e){return Pn(e,t)})).join("")}!function(e){e[e.DOWN=-1]="DOWN",e[e.UP=1]="UP"}(bn||(bn={}));var jn=["allowNumericCharactersOnly","buttonPosition","clampValueOnBlur","className","defaultValue","majorStepSize","minorStepSize","onButtonClick","onValueChange","selectAllOnFocus","selectAllOnIncrement","stepSize"],Rn=function(e){function t(){var t,i=e.apply(this,arguments)||this;return i.state={currentImeInputInvalid:!1,shouldSelectAfterUpdate:!1,stepMaxPrecision:n.getStepMaxPrecision(i.props),value:kn(null!==(t=i.props.value)&&void 0!==t?t:i.props.defaultValue)},i.didPasteEventJustOccur=!1,i.delta=0,i.inputElement=null,i.inputRef=O(i,"inputElement",i.props.inputRef),i.incrementButtonHandlers=i.getButtonEventHandlers(bn.UP),i.decrementButtonHandlers=i.getButtonEventHandlers(bn.DOWN),i.handleButtonClick=function(e,t){var n,r,o=i.updateDelta(t,e),a=i.incrementValue(o);null===(r=(n=i.props).onButtonClick)||void 0===r||r.call(n,Number(Ln(a,i.props.locale)),a)},i.stopContinuousChange=function(){i.delta=0,i.clearTimeouts(),clearInterval(i.intervalId),document.removeEventListener("mouseup",i.stopContinuousChange)},i.handleContinuousChange=function(){var e,t,n,r;if(void 0!==i.props.min||void 0!==i.props.max){var o=null!==(e=i.props.min)&&void 0!==e?e:-1/0,a=null!==(t=i.props.max)&&void 0!==t?t:1/0,s=Number(Ln(i.state.value,i.props.locale));if(s<=o||s>=a)return void i.stopContinuousChange()}var l=i.incrementValue(i.delta);null===(r=(n=i.props).onButtonClick)||void 0===r||r.call(n,Number(Ln(l,i.props.locale)),l)},i.handleInputFocus=function(e){var t,n;i.setState({shouldSelectAfterUpdate:i.props.selectAllOnFocus}),null===(n=(t=i.props).onFocus)||void 0===n||n.call(t,e)},i.handleInputBlur=function(e){var t,n;if(i.setState({shouldSelectAfterUpdate:!1}),i.props.clampValueOnBlur){var r=e.target.value;i.handleNextValue(i.roundAndClampValue(r))}null===(n=(t=i.props).onBlur)||void 0===n||n.call(t,e)},i.handleInputKeyDown=function(e){var t,n;if(!i.props.disabled&&!i.props.readOnly){var o,a=e.keyCode;if(a===r.ARROW_UP?o=bn.UP:a===r.ARROW_DOWN&&(o=bn.DOWN),void 0!==o){e.preventDefault();var s=i.updateDelta(o,e);i.incrementValue(s)}null===(n=(t=i.props).onKeyDown)||void 0===n||n.call(t,e)}},i.handleCompositionEnd=function(e){i.props.allowNumericCharactersOnly&&(i.handleNextValue(Nn(e.data,i.props.locale)),i.setState({currentImeInputInvalid:!1}))},i.handleCompositionUpdate=function(e){if(i.props.allowNumericCharactersOnly){var t=e.data;0===Nn(t,i.props.locale).length&&t.length>0?i.setState({currentImeInputInvalid:!0}):i.setState({currentImeInputInvalid:!1})}},i.handleInputKeyPress=function(e){var t,n;i.props.allowNumericCharactersOnly&&!Dn(e,i.props.locale)&&e.preventDefault(),null===(n=(t=i.props).onKeyPress)||void 0===n||n.call(t,e)},i.handleInputPaste=function(e){var t,n;i.didPasteEventJustOccur=!0,null===(n=(t=i.props).onPaste)||void 0===n||n.call(t,e)},i.handleInputChange=function(e){var t=e.target.value,n=t;i.props.allowNumericCharactersOnly&&i.didPasteEventJustOccur&&(i.didPasteEventJustOccur=!1,n=Nn(t,i.props.locale)),i.handleNextValue(n),i.setState({shouldSelectAfterUpdate:!1})},i}var n;return Object(c.__extends)(t,e),n=t,t.getDerivedStateFromProps=function(e,t){var r,i,o={prevMaxProp:e.max,prevMinProp:e.min},a=e.min!==t.prevMinProp,s=e.max!==t.prevMaxProp,l=a||s,u=null!==(i=null===(r=e.value)||void 0===r?void 0:r.toString())&&void 0!==i?i:t.value,d=n.getStepMaxPrecision(e),h=u!==n.VALUE_EMPTY?n.roundAndClampValue(u,d,e.min,e.max,0,e.locale):n.VALUE_EMPTY;return l&&h!==t.value?Object(c.__assign)(Object(c.__assign)({},o),{stepMaxPrecision:d,value:h}):Object(c.__assign)(Object(c.__assign)({},o),{stepMaxPrecision:d,value:u})},t.getStepMaxPrecision=function(e){return null!=e.minorStepSize?i.countDecimalPlaces(e.minorStepSize):i.countDecimalPlaces(e.stepSize)},t.roundAndClampValue=function(e,t,r,i,o,a){if(void 0===o&&(o=0),!function(e,t){var n=Ln(e,t);return null!=e&&n-parseFloat(n)+1>=0}(e,a))return n.VALUE_EMPTY;var s=Ln(e,a);return On(function(e,t,n){var r=null!=t?t:-1/0,i=null!=n?n:1/0;return Object(h.c)(e,r,i)}(function(e,t){var n=Math.pow(10,t);return Math.round(e*n)/n}(Number(s)+o,t),r,i),a)},t.prototype.render=function(){var e,t=this.props,n=t.buttonPosition,r=t.className,i=t.fill,o=t.large,s=pe()(a.NUMERIC_INPUT,((e={})[a.LARGE]=o,e),r),l=this.renderButtons();return d.createElement(Cn,{className:s,fill:i},n===_.a.LEFT&&l,this.renderInput(),n===_.a.RIGHT&&l)},t.prototype.componentDidUpdate=function(t,r){var i,o,a;e.prototype.componentDidUpdate.call(this,t,r),this.state.shouldSelectAfterUpdate&&(null===(i=this.inputElement)||void 0===i||i.setSelectionRange(0,this.state.value.length));var s=this.props.min!==t.min,l=this.props.max!==t.max,u=s||l,c=this.props.locale!==t.locale,d=this.state.value!==r.value;if(u&&d||c&&r.value!==n.VALUE_EMPTY){var h=Ln(c?r.value:this.state.value,t.locale),f=On(+h,this.props.locale);null===(a=(o=this.props).onValueChange)||void 0===a||a.call(o,+h,f,this.inputElement)}},t.prototype.validateProps=function(e){var t=e.majorStepSize,r=e.max,i=e.min,o=e.minorStepSize,a=e.stepSize,s=e.value;if(null!=i&&null!=r&&i>r)throw new Error(Ge.w);if(a<=0)throw new Error(Ge.x);if(o&&o<=0)throw new Error(Ge.v);if(t&&t<=0)throw new Error(Ge.t);if(o&&o>a)throw new Error(Ge.u);if(t&&t<a)throw new Error(Ge.s);if(null!=s){var l=n.getStepMaxPrecision(e),u=n.roundAndClampValue(s.toString(),l,i,r,0,this.props.locale),c=u!==s.toString(),d=On(Number(Ln(s,this.props.locale)),this.props.locale);c&&u!==d&&console.warn(Ge.r)}},t.prototype.renderButtons=function(){var e=this.props,t=e.intent,n=e.max,r=e.min,i=e.locale,o=Ln(this.state.value,i),s=this.props.disabled||this.props.readOnly,l=void 0!==n&&""!==o&&+o>=n,u=void 0!==r&&""!==o&&+o<=r;return d.createElement(tn,{className:a.FIXED,key:"button-group",vertical:!0},d.createElement(St,Object(c.__assign)({disabled:s||l,icon:"chevron-up",intent:t},this.incrementButtonHandlers)),d.createElement(St,Object(c.__assign)({disabled:s||u,icon:"chevron-down",intent:t},this.decrementButtonHandlers)))},t.prototype.renderInput=function(){var e=x(this.props,jn,!0);return d.createElement(Tn,Object(c.__assign)({asyncControl:this.props.asyncControl,autoComplete:"off"},e,{intent:this.state.currentImeInputInvalid?A.a.DANGER:this.props.intent,inputRef:this.inputRef,large:this.props.large,leftIcon:this.props.leftIcon,onFocus:this.handleInputFocus,onBlur:this.handleInputBlur,onChange:this.handleInputChange,onCompositionEnd:this.handleCompositionEnd,onCompositionUpdate:this.handleCompositionUpdate,onKeyDown:this.handleInputKeyDown,onKeyPress:this.handleInputKeyPress,onPaste:this.handleInputPaste,rightElement:this.props.rightElement,value:this.state.value}))},t.prototype.getButtonEventHandlers=function(e){var t=this;return{onKeyDown:function(n){!t.props.disabled&&r.isKeyboardClick(n.keyCode)&&t.handleButtonClick(n,e)},onMouseDown:function(n){t.props.disabled||(t.handleButtonClick(n,e),t.startContinuousChange())}}},t.prototype.startContinuousChange=function(){var e=this;document.addEventListener("mouseup",this.stopContinuousChange),this.setTimeout((function(){e.intervalId=window.setInterval(e.handleContinuousChange,n.CONTINUOUS_CHANGE_INTERVAL)}),n.CONTINUOUS_CHANGE_DELAY)},t.prototype.handleNextValue=function(e){var t,n;null==this.props.value&&this.setState({value:e}),null===(n=(t=this.props).onValueChange)||void 0===n||n.call(t,Number(Ln(e,this.props.locale)),e,this.inputElement)},t.prototype.incrementValue=function(e){var t=this.state.value===n.VALUE_EMPTY?n.VALUE_ZERO:this.state.value,r=this.roundAndClampValue(t,e);return r!==this.state.value&&(this.handleNextValue(r),this.setState({shouldSelectAfterUpdate:this.props.selectAllOnIncrement})),r},t.prototype.getIncrementDelta=function(e,t,n){var r=this.props,i=r.majorStepSize,o=r.minorStepSize,a=r.stepSize;return t&&null!=i?e*i:n&&null!=o?e*o:e*a},t.prototype.roundAndClampValue=function(e,t){return void 0===t&&(t=0),n.roundAndClampValue(e,this.state.stepMaxPrecision,this.props.min,this.props.max,t,this.props.locale)},t.prototype.updateDelta=function(e,t){return this.delta=this.getIncrementDelta(e,t.shiftKey,t.altKey),this.delta},t.displayName="Blueprint3.NumericInput",t.VALUE_EMPTY="",t.VALUE_ZERO="0",t.defaultProps={allowNumericCharactersOnly:!0,buttonPosition:_.a.RIGHT,clampValueOnBlur:!1,defaultValue:n.VALUE_EMPTY,large:!1,majorStepSize:10,minorStepSize:.1,selectAllOnFocus:!1,selectAllOnIncrement:!1,stepSize:1},t.CONTINUOUS_CHANGE_DELAY=300,t.CONTINUOUS_CHANGE_INTERVAL=100,t=n=Object(c.__decorate)([ve.polyfill],t)}(v),zn=0;function Fn(){return Bn.displayName+"-"+zn++}for(var Bn=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.autoGroupName=Fn(),t}return Object(c.__extends)(t,e),t.prototype.render=function(){var e=this.props.label;return d.createElement("div",{className:this.props.className},null==e?null:d.createElement("label",{className:a.LABEL},e),Array.isArray(this.props.options)?this.renderOptions():this.renderChildren())},t.prototype.validateProps=function(){null!=this.props.children&&null!=this.props.options&&console.warn(Ge.K)},t.prototype.renderChildren=function(){var e=this;return d.Children.map(this.props.children,(function(t){return ce(t,In)?d.cloneElement(t,e.getRadioProps(t.props)):t}))},t.prototype.renderOptions=function(){var e,t=this;return null===(e=this.props.options)||void 0===e?void 0:e.map((function(e){return d.createElement(In,Object(c.__assign)({},t.getRadioProps(e),{key:e.value,labelElement:e.label||e.value}))}))},t.prototype.getRadioProps=function(e){var t=this.props.name,n=e.className,r=e.disabled,i=e.value;return{checked:i===this.props.selectedValue,className:n,disabled:r||this.props.disabled,inline:this.props.inline,name:null==t?this.autoGroupName:t,onChange:this.props.onChange,value:i}},t.displayName="Blueprint3.RadioGroup",t=Object(c.__decorate)([ve.polyfill],t)}(v),Hn=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.state={},t.textareaElement=null,t.handleRef=O(t,"textareaElement",t.props.inputRef),t.handleChange=function(e){t.props.growVertically&&t.setState({height:e.target.scrollHeight}),null!=t.props.onChange&&t.props.onChange(e)},t}return Object(c.__extends)(t,e),t.prototype.componentDidMount=function(){this.props.growVertically&&null!==this.textareaElement&&this.setState({height:T(this.textareaElement).scrollHeight})},t.prototype.componentDidUpdate=function(e){var t=this.props.inputRef;e.inputRef!==t&&(w(t)?(t.current=this.textareaElement.current,this.textareaElement=t):M(t)&&t(this.textareaElement))},t.prototype.render=function(){var e,t=this.props,n=t.className,r=t.fill,i=(t.inputRef,t.intent),o=t.large,s=t.small,l=t.growVertically,u=Object(c.__rest)(t,["className","fill","inputRef","intent","large","small","growVertically"]),h=pe()(a.INPUT,a.intentClass(i),((e={})[a.FILL]=r,e[a.LARGE]=o,e[a.SMALL]=s,e),n),f=u.style,p=void 0===f?{}:f;return l&&null!=this.state.height&&(p=Object(c.__assign)(Object(c.__assign)({},p),{height:this.state.height+"px"})),d.createElement("textarea",Object(c.__assign)({},u,{className:h,onChange:this.handleChange,ref:this.handleRef,style:p}))},t.displayName="Blueprint3.TextArea",t=Object(c.__decorate)([ve.polyfill],t)}(v),Un=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(c.__extends)(t,e),t.prototype.render=function(){var e,t=this.props,n=t.className,r=t.disabled,i=t.elementRef,o=t.fill,s=t.iconProps,l=t.large,u=t.minimal,h=t.options,f=void 0===h?[]:h,p=Object(c.__rest)(t,["className","disabled","elementRef","fill","iconProps","large","minimal","options"]),g=pe()(a.HTML_SELECT,((e={})[a.DISABLED]=r,e[a.FILL]=o,e[a.LARGE]=l,e[a.MINIMAL]=u,e),n),v=f.map((function(e){var t="object"==typeof e?e:{value:e};return d.createElement("option",Object(c.__assign)({},t,{key:t.value,children:t.label||t.value}))}));return d.createElement("div",{className:g},d.createElement("select",Object(c.__assign)({disabled:r,ref:i},p,{multiple:!1}),v,p.children),d.createElement(_t,Object(c.__assign)({icon:"double-caret-vertical"},s)))},t=Object(c.__decorate)([ve.polyfill],t)}(v),Vn=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(c.__extends)(t,e),t.prototype.render=function(){var e,t=this.props,n=t.bordered,r=t.className,i=t.condensed,o=t.elementRef,s=t.interactive,l=t.small,u=t.striped,h=Object(c.__rest)(t,["bordered","className","condensed","elementRef","interactive","small","striped"]),f=pe()(a.HTML_TABLE,((e={})[a.HTML_TABLE_BORDERED]=n,e[a.HTML_TABLE_CONDENSED]=i,e[a.HTML_TABLE_STRIPED]=u,e[a.INTERACTIVE]=s,e[a.SMALL]=l,e),r);return d.createElement("table",Object(c.__assign)({},h,{ref:o,className:f}))},t=Object(c.__decorate)([ve.polyfill],t)}(v),Yn={8:"backspace",9:"tab",13:"enter",20:"capslock",27:"esc",32:"space",33:"pageup",34:"pagedown",35:"end",36:"home",37:"left",38:"up",39:"right",40:"down",45:"ins",46:"del",48:"0",49:"1",50:"2",51:"3",52:"4",53:"5",54:"6",55:"7",56:"8",57:"9",65:"a",66:"b",67:"c",68:"d",69:"e",70:"f",71:"g",72:"h",73:"i",74:"j",75:"k",76:"l",77:"m",78:"n",79:"o",80:"p",81:"q",82:"r",83:"s",84:"t",85:"u",86:"v",87:"w",88:"x",89:"y",90:"z",106:"*",107:"+",109:"-",110:".",111:"/",186:";",187:"=",188:",",189:"-",190:".",191:"/",192:"`",219:"[",220:"\\",221:"]",222:"'"},Wn={16:"shift",17:"ctrl",18:"alt",91:"meta",93:"meta",224:"meta"},Gn={alt:1,ctrl:2,meta:4,shift:8},Xn={cmd:"meta",command:"meta",escape:"esc",minus:"-",mod:er()?"meta":"ctrl",option:"alt",plus:"+",return:"enter",win:"meta"},qn={"~":"`","!":"1","@":"2","#":"3",$:"4","%":"5","^":"6","&":"7","*":"8","(":"9",")":"0",_:"-","+":"=","{":"[","}":"]","|":"\\",":":";",'"':"'","<":",",">":".","?":"/"},Zn=1;Zn<=12;++Zn)Yn[111+Zn]="f"+Zn;for(Zn=0;Zn<=9;++Zn)Yn[96+Zn]="num"+Zn.toString();function Qn(e,t){return e.modifiers===t.modifiers&&e.key===t.key}var Kn=function(e){for(var t,n=0,r=0,i=e.replace(/\s/g,"").toLowerCase().split("+");r<i.length;r++){var o=i[r];if(""===o)throw new Error('Failed to parse key combo "'+e+'".\n                Valid key combos look like "cmd + plus", "shift+p", or "!"');null!=Xn[o]&&(o=Xn[o]),null!=Gn[o]?n+=Gn[o]:null!=qn[o]?(n+=Gn.shift,t=qn[o]):t=o.toLowerCase()}return{modifiers:n,key:t}},Jn=function(e){var t=[];e.ctrlKey&&t.push("ctrl"),e.altKey&&t.push("alt"),e.shiftKey&&t.push("shift"),e.metaKey&&t.push("meta");var n=e.which;return null!=Wn[n]||(null!=Yn[n]?t.push(Yn[n]):t.push(String.fromCharCode(n).toLowerCase())),t.join(" + ")},$n=function(e){var t,n=e.which;null!=Wn[n]||(t=null!=Yn[n]?Yn[n]:String.fromCharCode(n).toLowerCase());var r=0;return e.altKey&&(r+=Gn.alt),e.ctrlKey&&(r+=Gn.ctrl),e.metaKey&&(r+=Gn.meta),e.shiftKey&&(r+=Gn.shift),{modifiers:r,key:t}};function er(e){var t=null!=e?e:"undefined"!=typeof navigator?navigator.platform:void 0;return null!=t&&/Mac|iPod|iPhone|iPad/.test(t)}var tr={alt:"key-option",cmd:"key-command",ctrl:"key-control",delete:"key-delete",down:"arrow-down",enter:"key-enter",left:"arrow-left",meta:"key-command",right:"arrow-right",shift:"key-shift",up:"arrow-up"},nr=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.renderKey=function(e,t){var n=tr[e],r="key-"+t;return null==n?d.createElement("kbd",{className:a.KEY,key:r},e):d.createElement("kbd",{className:pe()(a.KEY,a.MODIFIER_KEY),key:r},d.createElement(_t,{icon:n})," ",e)},t.renderMinimalKey=function(e,t){var n=tr[e];return null==n?e:d.createElement(_t,{icon:n,key:"key-"+t})},t}return Object(c.__extends)(t,e),t.prototype.render=function(){var e=this.props,t=e.className,n=e.combo,r=e.minimal,i=function(e,t){return e.replace(/\s/g,"").split("+").map((function(e){var n=null!=Xn[e]?Xn[e]:e;return"meta"===n?er(t)?"cmd":"ctrl":n}))}(n).map((function(e){return 1===e.length?e.toUpperCase():e})).map(r?this.renderMinimalKey:this.renderKey);return d.createElement("span",{className:pe()(a.KEY_COMBO,t)},i)},t.displayName="Blueprint3.KeyCombo",t=Object(c.__decorate)([ve.polyfill],t)}(v),rr=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(c.__extends)(t,e),t.prototype.render=function(){var e=this.props,t=e.label,n=e.className,r=Object(c.__rest)(e,["label","className"]),i=pe()(a.HOTKEY,n);return d.createElement("div",{className:i},d.createElement("div",{className:a.HOTKEY_LABEL},t),d.createElement(nr,Object(c.__assign)({},r)))},t.prototype.validateProps=function(e){if(!0!==e.global&&null==e.group)throw new Error("non-global <Hotkey>s must define a group")},t.displayName="Blueprint3.Hotkey",t.defaultProps={allowInInput:!1,disabled:!1,global:!1,preventDefault:!1,stopPropagation:!1},t=Object(c.__decorate)([ve.polyfill],t)}(v),ir=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(c.__extends)(t,e),t.prototype.render=function(){if(!ae(this.props.children))return null;var e,t=d.Children.map(this.props.children,(function(e){return e.props}));t.sort((function(e,t){return e.global===t.global&&e.group&&t.group?e.group.localeCompare(t.group):e.global?-1:1}));for(var n=[],r=0,i=t;r<i.length;r++){var o=i[r],s=o.group;s!==e&&(n.push(d.createElement(Pt,{key:"group-"+n.length},s)),e=s),n.push(d.createElement(rr,Object(c.__assign)({key:n.length},o)))}var l=pe()(a.HOTKEY_COLUMN,this.props.className);return d.createElement("div",{className:l},n)},t.prototype.validateProps=function(e){ae(e.children)&&d.Children.forEach(e.children,(function(e){if(!ce(e,rr))throw new Error(Ge.l)}))},t.displayName="Blueprint3.Hotkeys",t.defaultProps={tabIndex:0},t=Object(c.__decorate)([ve.polyfill],t)}(v),or=new(function(){function e(){var e=this;this.componentProps={globalHotkeysGroup:"Global hotkeys"},this.container=null,this.hotkeysQueue=[],this.isDialogShowing=!1,this.show=function(){e.isDialogShowing=!0,e.render()},this.hide=function(){e.isDialogShowing=!1,e.render()}}return e.prototype.render=function(){null==this.container&&(this.container=this.getContainer()),ge.render(this.renderComponent(),this.container)},e.prototype.unmount=function(){null!=this.container&&(ge.unmountComponentAtNode(this.container),this.container.remove(),this.container=null)},e.prototype.enqueueHotkeysForDisplay=function(e){this.hotkeysQueue.push(e),window.clearTimeout(this.showTimeoutToken),this.showTimeoutToken=window.setTimeout(this.show,10)},e.prototype.hideAfterDelay=function(){window.clearTimeout(this.hideTimeoutToken),this.hideTimeoutToken=window.setTimeout(this.hide,10)},e.prototype.isShowing=function(){return this.isDialogShowing},e.prototype.getContainer=function(){return null==this.container&&(this.container=document.createElement("div"),this.container.classList.add(a.PORTAL),document.body.appendChild(this.container)),this.container},e.prototype.renderComponent=function(){return d.createElement(Vt,Object(c.__assign)({},this.componentProps,{className:pe()(a.HOTKEY_DIALOG,this.componentProps.className),isOpen:this.isDialogShowing,onClose:this.hide}),d.createElement("div",{className:a.DIALOG_BODY},this.renderHotkeys()))},e.prototype.renderHotkeys=function(){var e=this,t=this.emptyHotkeyQueue().map((function(t,n){var r=!0===t.global&&null==t.group?e.componentProps.globalHotkeysGroup:t.group;return d.createElement(rr,Object(c.__assign)({key:n},t,{group:r}))}));return d.createElement(ir,null,t)},e.prototype.emptyHotkeyQueue=function(){var e=this.hotkeysQueue.reduce((function(e,t){return e.concat(t)}),[]);return this.hotkeysQueue.length=0,e},e}());function ar(){return or.isShowing()}function sr(e){for(var t in e)e.hasOwnProperty(t)&&(or.componentProps[t]=e[t])}function lr(){or.hide()}var ur;!function(e){e.LOCAL="local",e.GLOBAL="global"}(ur||(ur={}));var cr=function(){function e(e){var t=this;this.scope=e,this.actions=[],this.handleKeyDown=function(e){var n,r=$n(e);t.isTextInput(e)||!Qn(Kn("?"),r)?ar()||t.invokeNamedCallbackIfComboRecognized(r,"onKeyDown",e):ar()?or.hideAfterDelay():(n=t.actions.map((function(e){return e.props})),or.enqueueHotkeysForDisplay(n))},this.handleKeyUp=function(e){ar()||t.invokeNamedCallbackIfComboRecognized($n(e),"onKeyUp",e)}}return e.prototype.count=function(){return this.actions.length},e.prototype.clear=function(){this.actions=[]},e.prototype.setHotkeys=function(e){var t=this,n=[];d.Children.forEach(e.children,(function(e){ce(e,rr)&&t.isScope(e.props)&&n.push({combo:Kn(e.props.combo),props:e.props})})),this.actions=n},e.prototype.invokeNamedCallbackIfComboRecognized=function(e,t,n){for(var r,i,o=this.isTextInput(n),a=0,s=this.actions;a<s.length;a++){var l=s[a];!(o&&!l.props.allowInInput||l.props.disabled)&&Qn(l.combo,e)&&(l.props.preventDefault&&n.preventDefault(),l.props.stopPropagation&&(n.isPropagationStopped=!0,n.stopPropagation()),null===(i=(r=l.props)[t])||void 0===i||i.call(r,n))}},e.prototype.isScope=function(e){return(e.global?ur.GLOBAL:ur.LOCAL)===this.scope},e.prototype.isTextInput=function(e){var t=e.target;if(null==t||null==t.closest)return!1;var n=t.closest("input, textarea, [contenteditable=true]");if(null==n)return!1;if("input"===n.tagName.toLowerCase()){var r=n.type;if("checkbox"===r||"radio"===r)return!1}return!n.readOnly},e}();function dr(e){var t;return K(e.prototype.renderHotkeys)||console.warn(Ge.n),(t=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.globalHotkeysEvents=new cr(ur.GLOBAL),t.localHotkeysEvents=new cr(ur.LOCAL),t}return Object(c.__extends)(t,e),t.prototype.componentDidMount=function(){null!=e.prototype.componentDidMount&&e.prototype.componentDidMount.call(this),document.addEventListener("keydown",this.globalHotkeysEvents.handleKeyDown),document.addEventListener("keyup",this.globalHotkeysEvents.handleKeyUp)},t.prototype.componentWillUnmount=function(){var t;null===(t=e.prototype.componentWillUnmount)||void 0===t||t.call(this),document.removeEventListener("keydown",this.globalHotkeysEvents.handleKeyDown),document.removeEventListener("keyup",this.globalHotkeysEvents.handleKeyUp),this.globalHotkeysEvents.clear(),this.localHotkeysEvents.clear()},t.prototype.render=function(){var t=this,n=e.prototype.render.call(this);if(null==n)return n;if(!d.isValidElement(n))return console.warn(Ge.m),n;if(K(this.renderHotkeys)){var r=this.renderHotkeys();if(this.localHotkeysEvents&&this.localHotkeysEvents.setHotkeys(r.props),this.globalHotkeysEvents&&this.globalHotkeysEvents.setHotkeys(r.props),this.localHotkeysEvents.count()>0){var i=void 0===r.props.tabIndex?0:r.props.tabIndex,o=n.props,a=o.onKeyDown,s=o.onKeyUp;return d.cloneElement(n,{onKeyDown:function(e){t.localHotkeysEvents.handleKeyDown(e.nativeEvent),null==a||a(e)},onKeyUp:function(e){t.localHotkeysEvents.handleKeyUp(e.nativeEvent),null==s||s(e)},tabIndex:i})}}return n},t}(e)).displayName="HotkeysTarget("+ue(e)+")",t}var hr=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(c.__extends)(t,e),t.prototype.render=function(){var e=this.props,t=e.className,n=Object(c.__rest)(e,["className"]);return d.createElement("div",Object(c.__assign)({className:pe()(a.NAVBAR_DIVIDER,t)},n))},t.displayName="Blueprint3.NavbarDivider",t=Object(c.__decorate)([ve.polyfill],t)}(v),fr=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(c.__extends)(t,e),t.prototype.render=function(){var e=this.props,t=e.align,n=e.children,r=e.className,i=Object(c.__rest)(e,["align","children","className"]),o=pe()(a.NAVBAR_GROUP,a.alignmentClass(t),r);return d.createElement("div",Object(c.__assign)({className:o},i),n)},t.displayName="Blueprint3.NavbarGroup",t.defaultProps={align:m.a.LEFT},t=Object(c.__decorate)([ve.polyfill],t)}(v),pr=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(c.__extends)(t,e),t.prototype.render=function(){var e=this.props,t=e.children,n=e.className,r=Object(c.__rest)(e,["children","className"]);return d.createElement("div",Object(c.__assign)({className:pe()(a.NAVBAR_HEADING,n)},r),t)},t.displayName="Blueprint3.NavbarHeading",t=Object(c.__decorate)([ve.polyfill],t)}(v),gr=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(c.__extends)(t,e),t.prototype.render=function(){var e,t=this.props,n=t.children,r=t.className,i=t.fixedToTop,o=Object(c.__rest)(t,["children","className","fixedToTop"]),s=pe()(a.NAVBAR,((e={})[a.FIXED_TOP]=i,e),r);return d.createElement("div",Object(c.__assign)({className:s},o),n)},t.displayName="Blueprint3.Navbar",t.Divider=hr,t.Group=fr,t.Heading=pr,t=Object(c.__decorate)([ve.polyfill],t)}(v),vr=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(c.__extends)(t,e),t.prototype.render=function(){var e=this.props,t=e.action,n=e.children,r=e.className,i=e.description,o=e.title;return d.createElement("div",{className:pe()(a.NON_IDEAL_STATE,r)},this.maybeRenderVisual(),o&&d.createElement(Pt,null,o),i&&se(i,"div"),t,n)},t.prototype.maybeRenderVisual=function(){var e=this.props.icon;return null==e?null:d.createElement("div",{className:a.NON_IDEAL_STATE_VISUAL},d.createElement(_t,{icon:e,iconSize:3*_t.SIZE_LARGE}))},t.displayName="Blueprint3.NonIdealState",t=Object(c.__decorate)([ve.polyfill],t)}(v),mr=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.handleClose=function(){return t.props.onClose(t.props.panel)},t}return Object(c.__extends)(t,e),t.prototype.render=function(){var e=this.props,t=e.panel,n=e.onOpen;return d.createElement("div",{className:a.PANEL_STACK_VIEW},this.maybeRenderHeader(),d.createElement(t.component,Object(c.__assign)({openPanel:n,closePanel:this.handleClose},t.props)))},t.prototype.maybeRenderHeader=function(){return this.props.showHeader?d.createElement("div",{className:a.PANEL_STACK_HEADER},d.createElement("span",null,this.maybeRenderBack()),d.createElement(Xt,{className:a.HEADING,ellipsize:!0,title:this.props.panel.htmlTitle},this.props.panel.title),d.createElement("span",null)):null},t.prototype.maybeRenderBack=function(){return void 0===this.props.previousPanel?null:d.createElement(St,{className:a.PANEL_STACK_HEADER_BACK,icon:"chevron-left",minimal:!0,onClick:this.handleClose,small:!0,text:this.props.previousPanel.title,title:this.props.previousPanel.htmlTitle})},t=Object(c.__decorate)([ve.polyfill],t)}(v),yr=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.state={direction:"push",stack:null!=t.props.stack?t.props.stack.slice().reverse():void 0!==t.props.initialPanel?[t.props.initialPanel]:[]},t.renderPanel=function(e,n){var r=t.props,i=r.renderActivePanelOnly,o=r.showPanelHeader,s=void 0===o||o,l=t.state.stack,u=l.length-n,c=i?l.length:u;return d.createElement(Xe.CSSTransition,{classNames:a.PANEL_STACK,key:c,timeout:400},d.createElement(mr,{onClose:t.handlePanelClose,onOpen:t.handlePanelOpen,panel:e,previousPanel:l[n+1],showHeader:s}))},t.handlePanelClose=function(e){var n,r,i=t.state.stack;i[0]!==e||i.length<=1||(null===(r=(n=t.props).onClose)||void 0===r||r.call(n,e),null==t.props.stack&&t.setState((function(e){return{direction:"pop",stack:e.stack.slice(1)}})))},t.handlePanelOpen=function(e){var n,r;null===(r=(n=t.props).onOpen)||void 0===r||r.call(n,e),null==t.props.stack&&t.setState((function(t){return{direction:"push",stack:Object(c.__spreadArrays)([e],t.stack)}}))},t}return Object(c.__extends)(t,e),t.prototype.componentDidUpdate=function(t,n){e.prototype.componentDidUpdate.call(this,t,n),this.props.stack!==t.stack&&null!=t.stack&&this.setState({stack:this.props.stack.slice().reverse()}),(null!=this.props.stack?this.props.stack.length:0)!==(null!=t.stack?t.stack.length:0)&&null!=t.stack&&this.setState({direction:t.stack.length-this.props.stack.length<0?"push":"pop"})},t.prototype.render=function(){var e=pe()(a.PANEL_STACK,a.PANEL_STACK+"-"+this.state.direction,this.props.className);return d.createElement(Xe.TransitionGroup,{className:e,component:"div"},this.renderPanels())},t.prototype.validateProps=function(e){if(null==e.initialPanel&&null==e.stack||null!=e.initialPanel&&null!=e.stack)throw new Error(Ge.z);if(null!=e.stack&&0===e.stack.length)throw new Error(Ge.A)},t.prototype.renderPanels=function(){var e=this.props.renderActivePanelOnly,t=void 0===e||e,n=this.state.stack;return 0===n.length?null:(t?[n[0]]:n).map(this.renderPanel).reverse()},t=Object(c.__decorate)([ve.polyfill],t)}(v),br=Object(c.__assign)(Object(c.__assign)({},_.a),{AUTO:"auto",AUTO_END:"auto-end",AUTO_START:"auto-start"}),Cr=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(c.__extends)(t,e),t.prototype.render=function(){var e,t=this.props,n=t.animate,r=void 0===n||n,i=t.className,o=t.intent,s=t.stripes,l=void 0===s||s,u=t.value,c=pe()(a.PROGRESS_BAR,a.intentClass(o),((e={})[a.PROGRESS_NO_ANIMATION]=!r,e[a.PROGRESS_NO_STRIPES]=!l,e),i),f=null==u?void 0:100*Object(h.c)(u,0,1)+"%";return d.createElement("div",{className:c},d.createElement("div",{className:a.PROGRESS_METER,style:{width:f}}))},t.displayName="Blueprint3.ProgressBar",t=Object(c.__decorate)([ve.polyfill],t)}(v),Ar={FULL:"full",START:"start",END:"end"},_r={LOCK:"lock",PUSH:"push",NONE:"none"};function Ir(e){return(100*e).toFixed(2)+"%"}function xr(e,t,n,r){for(var i=t<n?1:-1,o=t;o!==n+i;o+=i)e[o]=r}var wr=["max","min","stepSize","tickSize","value"],Mr=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.state={isMoving:!1},t.handleElement=null,t.refHandlers={handle:function(e){return t.handleElement=e}},t.beginHandleMovement=function(e){document.addEventListener("mousemove",t.handleHandleMovement),document.addEventListener("mouseup",t.endHandleMovement),t.setState({isMoving:!0}),t.changeValue(t.clientToValue(t.mouseEventClientOffset(e)))},t.beginHandleTouchMovement=function(e){document.addEventListener("touchmove",t.handleHandleTouchMovement),document.addEventListener("touchend",t.endHandleTouchMovement),document.addEventListener("touchcancel",t.endHandleTouchMovement),t.setState({isMoving:!0}),t.changeValue(t.clientToValue(t.touchEventClientOffset(e)))},t.getStyleProperties=function(){if(null==t.handleElement)return{};var e=t.props,n=e.min,r=void 0===n?0:n,i=e.tickSizeRatio,o=e.value,a=e.vertical,s=t.getHandleMidpointAndOffset(t.handleElement,!0).handleMidpoint,l="calc("+Ir((o-r)*i)+" - "+s+"px)";return a?{bottom:l}:{left:l}},t.endHandleMovement=function(e){t.handleMoveEndedAt(t.mouseEventClientOffset(e))},t.endHandleTouchMovement=function(e){t.handleMoveEndedAt(t.touchEventClientOffset(e))},t.handleMoveEndedAt=function(e){var n,r;t.removeDocumentEventListeners(),t.setState({isMoving:!1});var i=t.changeValue(t.clientToValue(e));null===(r=(n=t.props).onRelease)||void 0===r||r.call(n,i)},t.handleHandleMovement=function(e){t.handleMovedTo(t.mouseEventClientOffset(e))},t.handleHandleTouchMovement=function(e){t.handleMovedTo(t.touchEventClientOffset(e))},t.handleMovedTo=function(e){t.state.isMoving&&!t.props.disabled&&t.changeValue(t.clientToValue(e))},t.handleKeyDown=function(e){var n=t.props,i=n.stepSize,o=n.value,a=e.which;a===r.ARROW_DOWN||a===r.ARROW_LEFT?(t.changeValue(o-i),e.preventDefault()):a!==r.ARROW_UP&&a!==r.ARROW_RIGHT||(t.changeValue(o+i),e.preventDefault())},t.handleKeyUp=function(e){var n,i;[r.ARROW_UP,r.ARROW_DOWN,r.ARROW_LEFT,r.ARROW_RIGHT].indexOf(e.which)>=0&&(null===(i=(n=t.props).onRelease)||void 0===i||i.call(n,t.props.value))},t}return Object(c.__extends)(t,e),t.prototype.componentDidMount=function(){this.forceUpdate()},t.prototype.render=function(){var e,t=this.props,n=t.className,r=t.disabled,i=t.label,o=this.state.isMoving;return d.createElement("span",{className:pe()(a.SLIDER_HANDLE,(e={},e[a.ACTIVE]=o,e),n),onKeyDown:r?void 0:this.handleKeyDown,onKeyUp:r?void 0:this.handleKeyUp,onMouseDown:r?void 0:this.beginHandleMovement,onTouchStart:r?void 0:this.beginHandleTouchMovement,ref:this.refHandlers.handle,style:this.getStyleProperties(),tabIndex:0},null==i?null:d.createElement("span",{className:a.SLIDER_LABEL},i))},t.prototype.componentWillUnmount=function(){this.removeDocumentEventListeners()},t.prototype.clientToValue=function(e){var t=this.props,n=t.stepSize,r=t.tickSize,i=t.value,o=t.vertical;if(null==this.handleElement)return i;var a=(o?window.innerHeight-e:e)-this.getHandleElementCenterPixel(this.handleElement);return isNaN(a)?i:i+Math.round(a/(r*n))*n},t.prototype.mouseEventClientOffset=function(e){return this.props.vertical?e.clientY:e.clientX},t.prototype.touchEventClientOffset=function(e){var t=e.changedTouches[0];return this.props.vertical?t.clientY:t.clientX},t.prototype.validateProps=function(e){for(var t=0,n=wr;t<n.length;t++){var r=n[t];if("number"!=typeof e[r])throw new Error("[Blueprint] <Handle> requires number value for "+r+" prop")}},t.prototype.changeValue=function(e,t){return void 0===t&&(t=this.props.onChange),e=this.clamp(e),isNaN(e)||this.props.value===e||null==t||t(e),e},t.prototype.clamp=function(e){return Object(h.c)(e,this.props.min,this.props.max)},t.prototype.getHandleElementCenterPixel=function(e){var t=this.getHandleMidpointAndOffset(e),n=t.handleMidpoint;return t.handleOffset+n},t.prototype.getHandleMidpointAndOffset=function(e,t){if(void 0===t&&(t=!1),null==e)return{handleMidpoint:0,handleOffset:0};var n=this.props.vertical,r=e.getBoundingClientRect(),i=n?t?"width":"height":t?"height":"width",o=n?window.innerHeight-(r.top+r[i]):r.left;return{handleMidpoint:r[i]/2,handleOffset:o}},t.prototype.removeDocumentEventListeners=function(){document.removeEventListener("mousemove",this.handleHandleMovement),document.removeEventListener("mouseup",this.endHandleMovement),document.removeEventListener("touchmove",this.handleHandleTouchMovement),document.removeEventListener("touchend",this.endHandleTouchMovement),document.removeEventListener("touchcancel",this.endHandleTouchMovement)},t.displayName="Blueprint3.SliderHandle",t=Object(c.__decorate)([ve.polyfill],t)}(v),Sr=function(){return null};Sr.displayName="Blueprint3.MultiSliderHandle";var Tr,Er=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.state={labelPrecision:Or(t.props),tickSize:0,tickSizeRatio:0},t.handleElements=[],t.trackElement=null,t.addHandleRef=function(e){null!=e&&t.handleElements.push(e)},t.maybeHandleTrackClick=function(e){if(t.canHandleTrackEvent(e)){var n=t.nearestHandleForValue(t.handleElements,(function(t){return t.mouseEventClientOffset(e)}));n&&n.beginHandleMovement(e)}},t.maybeHandleTrackTouch=function(e){if(t.canHandleTrackEvent(e)){var n=t.nearestHandleForValue(t.handleElements,(function(t){return t.touchEventClientOffset(e)}));n&&n.beginHandleTouchMovement(e)}},t.canHandleTrackEvent=function(e){var n=e.target;return!t.props.disabled&&null==n.closest("."+a.SLIDER_HANDLE)},t.getHandlerForIndex=function(e,n){return function(r){null==n||n(t.getNewHandleValues(r,e))}},t.handleChange=function(e){var n,r,i=kr(t.props),o=i.map((function(e){return e.value}));V(e,o)||(null===(r=(n=t.props).onChange)||void 0===r||r.call(n,e),i.forEach((function(t,n){var r;o[n]!==e[n]&&(null===(r=t.onChange)||void 0===r||r.call(t,e[n]))})))},t.handleRelease=function(e){var n,r,i=kr(t.props);null===(r=(n=t.props).onRelease)||void 0===r||r.call(n,e),i.forEach((function(t,n){var r;null===(r=t.onRelease)||void 0===r||r.call(t,e[n])}))},t}var n;return Object(c.__extends)(t,e),n=t,t.getDerivedStateFromProps=function(e){return{labelPrecision:n.getLabelPrecision(e)}},t.getLabelPrecision=function(e){var t=e.labelPrecision,n=e.stepSize;return null==t?h.d(n):t},t.prototype.getSnapshotBeforeUpdate=function(e){var t=kr(e);return kr(this.props).length!==t.length&&(this.handleElements=[]),null},t.prototype.render=function(){var e,t=this,n=pe()(a.SLIDER,((e={})[a.DISABLED]=this.props.disabled,e[a.SLIDER+"-unlabeled"]=!1===this.props.labelRenderer,e[a.VERTICAL]=this.props.vertical,e),this.props.className);return d.createElement("div",{className:n,onMouseDown:this.maybeHandleTrackClick,onTouchStart:this.maybeHandleTrackTouch},d.createElement("div",{className:a.SLIDER_TRACK,ref:function(e){return t.trackElement=e}},this.renderTracks()),d.createElement("div",{className:a.SLIDER_AXIS},this.renderLabels()),this.renderHandles())},t.prototype.componentDidMount=function(){this.updateTickSize()},t.prototype.componentDidUpdate=function(t,n){e.prototype.componentDidUpdate.call(this,t,n),this.updateTickSize()},t.prototype.validateProps=function(e){if(e.stepSize<=0)throw new Error(Ge.N);if(void 0!==e.labelStepSize&&void 0!==e.labelValues)throw new Error(Ge.q);if(void 0!==e.labelStepSize&&e.labelStepSize<=0)throw new Error(Ge.M);var t=!1;if(d.Children.forEach(e.children,(function(e){e&&!ce(e,n.Handle)&&(t=!0)})),t)throw new Error(Ge.p)},t.prototype.formatLabel=function(e,t){void 0===t&&(t=!1);var n=this.props.labelRenderer;return!1===n?void 0:K(n)?n(e,{isHandleTooltip:t}):e.toFixed(this.state.labelPrecision)},t.prototype.renderLabels=function(){var e=this;if(!1===this.props.labelRenderer)return null;var t=this.getLabelValues(),n=this.props,r=n.max,i=n.min;return t.map((function(t,n){var o=Ir((t-i)/(r-i)),s=e.props.vertical?{bottom:o}:{left:o};return d.createElement("div",{className:a.SLIDER_LABEL,key:n,style:s},e.formatLabel(t))}))},t.prototype.renderTracks=function(){var e=Lr(this.props);e.push({value:this.props.max});for(var t={value:this.props.min},n=[],r=0;r<e.length;r++){var i=e[r];n.push(this.renderTrackFill(r,t,i)),t=i}return n},t.prototype.renderTrackFill=function(e,t,n){var r=[this.getOffsetRatio(t.value),this.getOffsetRatio(n.value)].sort((function(e,t){return e-t})),i=r[0],o=r[1],s=Ir(i),l=Ir(1-o),u=this.props.vertical?{bottom:s,top:l,left:0}:{left:s,right:l,top:0},h=Object(c.__assign)(Object(c.__assign)({},u),t.trackStyleAfter||n.trackStyleBefore||{}),f=pe()(a.SLIDER_PROGRESS,a.intentClass(this.getTrackIntent(t,n)));return d.createElement("div",{key:"track-"+e,className:f,style:h})},t.prototype.renderHandles=function(){var e=this,t=this.props,n=t.disabled,r=t.max,i=t.min,o=t.stepSize,s=t.vertical,l=kr(this.props);return 0===l.length?null:l.map((function(t,u){var c,h=t.value,f=t.type,p=t.className;return d.createElement(Mr,{className:pe()((c={},c[a.START]=f===Ar.START,c[a.END]=f===Ar.END,c),p),disabled:n,key:u+"-"+l.length,label:e.formatLabel(h,!0),max:r,min:i,onChange:e.getHandlerForIndex(u,e.handleChange),onRelease:e.getHandlerForIndex(u,e.handleRelease),ref:e.addHandleRef,stepSize:o,tickSize:e.state.tickSize,tickSizeRatio:e.state.tickSizeRatio,value:h,vertical:s})}))},t.prototype.nearestHandleForValue=function(e,t){return function(e,t){if(0!==e.length){for(var n=e[0],r=t(n),i=1;i<e.length;i++){var o=e[i],a=t(o);a<r&&(n=o,r=a)}return n}}(e,(function(e){var n=t(e),r=e.clientToValue(n),i=e.props.value;return Math.abs(r-i)}))},t.prototype.getNewHandleValues=function(e,t){var n=kr(this.props).map((function(e){return e.value})),r=n.slice();r[t]=e,r.sort((function(e,t){return e-t}));var i=r.indexOf(e),o=this.findFirstLockedHandleIndex(t,i);return-1!==o?(xr(n,t,o,n[o]),n):(xr(r,t,i,e),r)},t.prototype.findFirstLockedHandleIndex=function(e,t){for(var n=e<t?1:-1,r=kr(this.props),i=e+n;i!==t+n;i+=n)if(r[i].interactionKind!==_r.PUSH)return i;return-1},t.prototype.getLabelValues=function(){var e=this.props,t=e.labelStepSize,n=e.labelValues,r=e.min,i=e.max,o=[];if(void 0!==n)o=n;else for(var a=r;a<i||h.a(a,i);a+=null!=t?t:1)o.push(a);return o},t.prototype.getOffsetRatio=function(e){return h.c((e-this.props.min)*this.state.tickSizeRatio,0,1)},t.prototype.getTrackIntent=function(e,t){return this.props.showTrackFill?void 0!==e.intentAfter?e.intentAfter:void 0!==t&&void 0!==t.intentBefore?t.intentBefore:this.props.defaultTrackIntent:A.a.NONE},t.prototype.updateTickSize=function(){if(null!=this.trackElement){var e=this.props.vertical?this.trackElement.clientHeight:this.trackElement.clientWidth,t=1/(this.props.max-this.props.min),n=e*t;this.setState({tickSize:n,tickSizeRatio:t})}},t.defaultSliderProps={disabled:!1,max:10,min:0,showTrackFill:!0,stepSize:1,vertical:!1},t.defaultProps=Object(c.__assign)(Object(c.__assign)({},n.defaultSliderProps),{defaultTrackIntent:A.a.NONE}),t.displayName="Blueprint3.MultiSlider",t.Handle=Sr,t=n=Object(c.__decorate)([ve.polyfill],t)}(v);function Or(e){var t=e.labelPrecision,n=e.stepSize,r=void 0===n?Er.defaultSliderProps.stepSize:n;return null==t?h.d(r):t}function kr(e){return Lr(e,(function(e){return e.interactionKind!==_r.NONE}))}function Lr(e,t){var n=e.children;void 0===t&&(t=function(){return!0});var r=d.Children.map(n,(function(e){return ce(e,Er.Handle)&&t(e.props)?e.props:null})),i=null!=r?r:[];return(i=i.filter((function(e){return null!==e}))).sort((function(e,t){return e.value-t.value})),i}!function(e){e[e.START=0]="START",e[e.END=1]="END"}(Tr||(Tr={}));var Dr=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(c.__extends)(t,e),t.prototype.render=function(){var e=this.props,t=e.value,n=Object(c.__rest)(e,["value"]);return d.createElement(Er,Object(c.__assign)({},n),d.createElement(Er.Handle,{value:t[Tr.START],type:"start",intentAfter:n.intent}),d.createElement(Er.Handle,{value:t[Tr.END],type:"end"}))},t.prototype.validateProps=function(e){var t=e.value;if(null==t||null==t[Tr.START]||null==t[Tr.END])throw new Error(Ge.L)},t.defaultProps=Object(c.__assign)(Object(c.__assign)({},Er.defaultSliderProps),{intent:A.a.PRIMARY,value:[0,10]}),t.displayName="Blueprint3.RangeSlider",t=Object(c.__decorate)([ve.polyfill],t)}(v),Pr=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(c.__extends)(t,e),t.prototype.render=function(){var e=this.props,t=e.initialValue,n=e.intent,r=e.value,i=e.onChange,o=e.onRelease,a=Object(c.__rest)(e,["initialValue","intent","value","onChange","onRelease"]);return d.createElement(Er,Object(c.__assign)({},a),d.createElement(Er.Handle,{value:r,intentAfter:r<t?n:void 0,intentBefore:r>=t?n:void 0,onChange:i,onRelease:o}),d.createElement(Er.Handle,{value:t,interactionKind:"none"}))},t.defaultProps=Object(c.__assign)(Object(c.__assign)({},Er.defaultSliderProps),{initialValue:0,intent:A.a.PRIMARY,value:0}),t.displayName="Blueprint3.Slider",t=Object(c.__decorate)([ve.polyfill],t)}(v),Nr=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(c.__extends)(t,e),t.prototype.render=function(){var e=this.props,t=e.className,n=e.panel;return d.createElement("div",{className:pe()(a.TAB_PANEL,t),role:"tablist"},n)},t.defaultProps={disabled:!1},t.displayName="Blueprint3.Tab",t=Object(c.__decorate)([ve.polyfill],t)}(v),jr=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.handleClick=function(e){return t.props.onClick(t.props.id,e)},t}return Object(c.__extends)(t,e),t.prototype.render=function(){var e=this.props,t=e.className,n=e.children,r=e.disabled,i=e.id,o=e.parentId,s=e.selected,l=e.title,u=Object(c.__rest)(e,["className","children","disabled","id","parentId","selected","title"]);return d.createElement("div",Object(c.__assign)({},x(u),{"aria-controls":Rr(o,i),"aria-disabled":r,"aria-expanded":s,"aria-selected":s,className:pe()(a.TAB,t),"data-tab-id":i,id:zr(o,i),onClick:r?void 0:this.handleClick,role:"tab",tabIndex:r?void 0:0}),l,n)},t.displayName="Blueprint3.TabTitle",t=Object(c.__decorate)([ve.polyfill],t)}(v);function Rr(e,t){return a.TAB_PANEL+"_"+e+"_"+t}function zr(e,t){return a.TAB+"-title_"+e+"_"+t}var Fr=function(){return d.createElement("div",{className:a.FLEX_EXPANDER})},Br="."+a.TAB,Hr=function(e){function t(t){var n=e.call(this,t)||this;n.tablistElement=null,n.refHandlers={tablist:function(e){return n.tablistElement=e}},n.handleKeyDown=function(e){var t,r=null===(t=document.activeElement)||void 0===t?void 0:t.closest(Br);if(null!=r){var i=n.getTabElements().filter((function(e){return"false"===e.getAttribute("aria-disabled")})),o=i.indexOf(r),a=n.getKeyCodeDirection(e);if(o>=0&&void 0!==a){e.preventDefault();var s=i.length;i[(o+a+s)%s].focus()}}},n.handleKeyPress=function(e){var t=e.target.closest(Br);null!=t&&r.isKeyboardClick(e.which)&&(e.preventDefault(),t.click())},n.handleTabClick=function(e,t){var r,i;null===(i=(r=n.props).onChange)||void 0===i||i.call(r,e,n.state.selectedTabId,t),void 0===n.props.selectedTabId&&n.setState({selectedTabId:e})},n.renderTabPanel=function(e){var t=e.props,r=t.className,i=t.panel,o=t.id,s=t.panelClassName;if(void 0!==i)return d.createElement("div",{"aria-labelledby":zr(n.props.id,o),"aria-hidden":o!==n.state.selectedTabId,className:pe()(a.TAB_PANEL,r,s),id:Rr(n.props.id,o),key:o,role:"tabpanel"},i)},n.renderTabTitle=function(e){if(Vr(e)){var t=e.props.id;return d.createElement(jr,Object(c.__assign)({},e.props,{parentId:n.props.id,onClick:n.handleTabClick,selected:t===n.state.selectedTabId}))}return e};var i=n.getInitialSelectedTabId();return n.state={selectedTabId:i},n}return Object(c.__extends)(t,e),t.getDerivedStateFromProps=function(e){var t=e.selectedTabId;return void 0!==t?{selectedTabId:t}:null},t.prototype.render=function(){var e,t,n=this.state,r=n.indicatorWrapperStyle,i=n.selectedTabId,o=d.Children.map(this.props.children,this.renderTabTitle),s=this.getTabChildren().filter(this.props.renderActiveTabPanelOnly?function(e){return e.props.id===i}:function(){return!0}).map(this.renderTabPanel),l=this.props.animate?d.createElement("div",{className:a.TAB_INDICATOR_WRAPPER,style:r},d.createElement("div",{className:a.TAB_INDICATOR})):null,u=pe()(a.TABS,((e={})[a.VERTICAL]=this.props.vertical,e),this.props.className),c=pe()(a.TAB_LIST,((t={})[a.LARGE]=this.props.large,t));return d.createElement("div",{className:u},d.createElement("div",{className:c,onKeyDown:this.handleKeyDown,onKeyPress:this.handleKeyPress,ref:this.refHandlers.tablist,role:"tablist"},l,o),s)},t.prototype.componentDidMount=function(){this.moveSelectionIndicator(!1)},t.prototype.componentDidUpdate=function(e,t){if(this.state.selectedTabId!==t.selectedTabId)this.moveSelectionIndicator();else if(null!=t.selectedTabId){!V(this.getTabChildrenProps(e),this.getTabChildrenProps(),Y)&&this.moveSelectionIndicator()}},t.prototype.getInitialSelectedTabId=function(){var e=this.props,t=e.defaultSelectedTabId,n=e.selectedTabId;if(void 0!==n)return n;if(void 0!==t)return t;var r=this.getTabChildren();return 0===r.length?void 0:r[0].props.id},t.prototype.getKeyCodeDirection=function(e){return Ur(e,r.ARROW_LEFT,r.ARROW_UP)?-1:Ur(e,r.ARROW_RIGHT,r.ARROW_DOWN)?1:void 0},t.prototype.getTabChildrenProps=function(e){return void 0===e&&(e=this.props),this.getTabChildren(e).map((function(e){return e.props}))},t.prototype.getTabChildren=function(e){return void 0===e&&(e=this.props),d.Children.toArray(e.children).filter(Vr)},t.prototype.getTabElements=function(e){return void 0===e&&(e=""),null==this.tablistElement?[]:Array.from(this.tablistElement.querySelectorAll(Br+e))},t.prototype.moveSelectionIndicator=function(e){if(void 0===e&&(e=!0),null!=this.tablistElement&&this.props.animate){var t=Br+'[data-tab-id="'+this.state.selectedTabId+'"]',n=this.tablistElement.querySelector(t),r={display:"none"};if(null!=n){var i=n.clientHeight,o=n.clientWidth,a=n.offsetLeft,s=n.offsetTop;r={height:i,transform:"translateX("+Math.floor(a)+"px) translateY("+Math.floor(s)+"px)",width:o},e||(r.transition="none")}this.setState({indicatorWrapperStyle:r})}},t.Expander=Fr,t.Tab=Nr,t.defaultProps={animate:!0,large:!1,renderActiveTabPanelOnly:!1,vertical:!1},t.displayName="Blueprint3.Tabs",t=Object(c.__decorate)([ve.polyfill],t)}(v);function Ur(e){for(var t=[],n=1;n<arguments.length;n++)t[n-1]=arguments[n];return t.indexOf(e.which)>=0}function Vr(e){return ce(e,Nr)}var Yr=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.onRemoveClick=function(e){var n,r;null===(r=(n=t.props).onRemove)||void 0===r||r.call(n,e,t.props)},t}return Object(c.__extends)(t,e),t.prototype.render=function(){var e,t=this.props,n=t.active,r=t.children,o=t.className,s=t.fill,l=t.icon,u=t.intent,h=t.interactive,f=t.large,p=t.minimal,g=t.multiline,v=t.onRemove,m=t.rightIcon,y=t.round,b=t.tabIndex,C=void 0===b?0:b,A=t.htmlTitle,_=t.elementRef,I=Object(c.__rest)(t,["active","children","className","fill","icon","intent","interactive","large","minimal","multiline","onRemove","rightIcon","round","tabIndex","htmlTitle","elementRef"]),x=i.isFunction(v),w=pe()(a.TAG,a.intentClass(u),((e={})[a.ACTIVE]=n,e[a.FILL]=s,e[a.INTERACTIVE]=h,e[a.LARGE]=f,e[a.MINIMAL]=p,e[a.ROUND]=y,e),o),M=f||w.indexOf(a.LARGE)>=0,S=x?d.createElement("button",{type:"button",className:a.TAG_REMOVE,onClick:this.onRemoveClick,tabIndex:h?C:void 0},d.createElement(_t,{icon:"small-cross",iconSize:M?_t.SIZE_LARGE:_t.SIZE_STANDARD})):null;return d.createElement("span",Object(c.__assign)({},I,{className:w,tabIndex:h?C:void 0,ref:_}),d.createElement(_t,{icon:l}),!oe(r)&&d.createElement(Xt,{className:a.FILL,ellipsize:!g,tagName:"span",title:A},r),d.createElement(_t,{icon:m}),S)},t.displayName="Blueprint3.Tag",t=Object(c.__decorate)([ve.polyfill],t)}(v),Wr=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.state={activeIndex:-1,inputValue:t.props.inputValue||"",isInputFocused:!1},t.inputElement=null,t.handleRef=O(t,"inputElement",t.props.inputRef),t.addTags=function(e,n){void 0===n&&(n="default");var r=t.props,o=r.inputValue,a=r.onAdd,s=r.onChange,l=r.values,u=t.getValues(e),d=!1!==(null==a?void 0:a(u,n))&&void 0===o;i.isFunction(s)&&(d=!1!==s(Object(c.__spreadArrays)(l,u))&&d),d&&t.setState({inputValue:""})},t.maybeRenderTag=function(e,n){if(!e)return null;var r=t.props,o=r.large,a=r.tagProps,s=i.isFunction(a)?a(e,n):a;return d.createElement(Yr,Object(c.__assign)({active:n===t.state.activeIndex,"data-tag-index":n,key:e+"__"+n,large:o,onRemove:t.props.disabled?void 0:t.handleRemoveTag},s),e)},t.handleContainerClick=function(){var e;null===(e=T(t.inputElement))||void 0===e||e.focus()},t.handleContainerBlur=function(e){var n=e.currentTarget;t.requestAnimationFrame((function(){n.contains(document.activeElement)||(t.props.addOnBlur&&void 0!==t.state.inputValue&&t.state.inputValue.length>0&&t.addTags(t.state.inputValue,"blur"),t.setState({activeIndex:-1,isInputFocused:!1}))}))},t.handleInputFocus=function(e){var n,r;t.setState({isInputFocused:!0}),null===(r=null===(n=t.props.inputProps)||void 0===n?void 0:n.onFocus)||void 0===r||r.call(n,e)},t.handleInputChange=function(e){var n,r,i,o;t.setState({activeIndex:-1,inputValue:e.currentTarget.value}),null===(r=(n=t.props).onInputChange)||void 0===r||r.call(n,e),null===(o=null===(i=t.props.inputProps)||void 0===i?void 0:i.onChange)||void 0===o||o.call(i,e)},t.handleInputKeyDown=function(e){var n=e.currentTarget,i=n.selectionEnd,o=n.value,a=t.state.activeIndex,s=a;if(e.which===r.ENTER&&o.length>0)t.addTags(o,"default");else if(0===i&&t.props.values.length>0)if(e.which===r.ARROW_LEFT||e.which===r.ARROW_RIGHT){var l=t.getNextActiveIndex(e.which===r.ARROW_RIGHT?1:-1);l!==a&&(e.stopPropagation(),s=l,t.setState({activeIndex:l}))}else e.which===r.BACKSPACE?t.handleBackspaceToRemove(e):e.which===r.DELETE&&t.handleDeleteToRemove(e);t.invokeKeyPressCallback("onKeyDown",e,s)},t.handleInputKeyUp=function(e){t.invokeKeyPressCallback("onKeyUp",e,t.state.activeIndex)},t.handleInputPaste=function(e){var n=t.props.separator,r=e.clipboardData.getData("text");t.props.addOnPaste&&0!==r.length&&!1!==n&&1!==r.split(n).length&&(e.preventDefault(),t.addTags(r,"paste"))},t.handleRemoveTag=function(e){var n=+e.currentTarget.parentElement.getAttribute("data-tag-index");t.removeIndexFromValues(n)},t}return Object(c.__extends)(t,e),t.getDerivedStateFromProps=function(e,t){return e.inputValue!==t.prevInputValueProp?{inputValue:e.inputValue,prevInputValueProp:e.inputValue}:null},t.prototype.render=function(){var e,t=this.props,n=t.className,r=t.disabled,i=t.fill,o=t.inputProps,s=t.intent,l=t.large,u=t.leftIcon,h=t.placeholder,f=t.values,p=pe()(a.INPUT,a.TAG_INPUT,((e={})[a.ACTIVE]=this.state.isInputFocused,e[a.DISABLED]=r,e[a.FILL]=i,e[a.LARGE]=l,e),a.intentClass(s),n),g=p.indexOf(a.LARGE)>-1,v=f.some((function(e){return!!e})),m=null==h||v?null==o?void 0:o.placeholder:h;return d.createElement("div",{className:p,onBlur:this.handleContainerBlur,onClick:this.handleContainerClick},d.createElement(_t,{className:a.TAG_INPUT_ICON,icon:u,iconSize:g?_t.SIZE_LARGE:_t.SIZE_STANDARD}),d.createElement("div",{className:a.TAG_INPUT_VALUES},f.map(this.maybeRenderTag),this.props.children,d.createElement("input",Object(c.__assign)({value:this.state.inputValue},o,{onFocus:this.handleInputFocus,onChange:this.handleInputChange,onKeyDown:this.handleInputKeyDown,onKeyUp:this.handleInputKeyUp,onPaste:this.handleInputPaste,placeholder:m,ref:this.handleRef,className:pe()(a.INPUT_GHOST,null==o?void 0:o.className),disabled:r}))),this.props.rightElement)},t.prototype.getNextActiveIndex=function(e){var t=this.state.activeIndex;return-1===t?e<0?this.findNextIndex(this.props.values.length,-1):-1:this.findNextIndex(t,e)},t.prototype.findNextIndex=function(e,t){for(var n=this.props.values,r=e+t;r>0&&r<n.length&&!n[r];)r+=t;return i.clamp(r,0,n.length)},t.prototype.getValues=function(e){var t=this.props.separator;return(!1===t?[e]:e.split(t)).map((function(e){return e.trim()})).filter((function(e){return e.length>0}))},t.prototype.handleBackspaceToRemove=function(e){var t=this.state.activeIndex;this.setState({activeIndex:this.getNextActiveIndex(-1)}),this.isValidIndex(t)&&(e.stopPropagation(),this.removeIndexFromValues(t))},t.prototype.handleDeleteToRemove=function(e){var t=this.state.activeIndex;this.isValidIndex(t)&&(e.stopPropagation(),this.removeIndexFromValues(t))},t.prototype.removeIndexFromValues=function(e){var t=this.props,n=t.onChange,r=t.onRemove,o=t.values;null==r||r(o[e],e),i.isFunction(n)&&n(o.filter((function(t,n){return n!==e})))},t.prototype.invokeKeyPressCallback=function(e,t,n){var r,i,o,a;null===(i=(r=this.props)[e])||void 0===i||i.call(r,t,-1===n?void 0:n),null===(a=(o=this.props.inputProps)[e])||void 0===a||a.call(o,t)},t.prototype.isValidIndex=function(e){return-1!==e&&e<this.props.values.length},t.displayName="Blueprint3.TagInput",t.defaultProps={addOnBlur:!1,addOnPaste:!0,inputProps:{},separator:/[,\n\r]/,tagProps:{}},t=Object(c.__decorate)([ve.polyfill],t)}(v),Gr=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.handleActionClick=function(e){var n,r;null===(r=null===(n=t.props.action)||void 0===n?void 0:n.onClick)||void 0===r||r.call(n,e),t.triggerDismiss(!1)},t.handleCloseClick=function(){return t.triggerDismiss(!1)},t.startTimeout=function(){t.clearTimeouts(),t.props.timeout>0&&t.setTimeout((function(){return t.triggerDismiss(!0)}),t.props.timeout)},t}return Object(c.__extends)(t,e),t.prototype.render=function(){var e=this.props,t=e.className,n=e.icon,r=e.intent,i=e.message;return d.createElement("div",{className:pe()(a.TOAST,a.intentClass(r),t),onBlur:this.startTimeout,onFocus:this.clearTimeouts,onMouseEnter:this.clearTimeouts,onMouseLeave:this.startTimeout,tabIndex:0},d.createElement(_t,{icon:n}),d.createElement("span",{className:a.TOAST_MESSAGE},i),d.createElement(tn,{minimal:!0},this.maybeRenderActionButton(),d.createElement(St,{icon:"cross",onClick:this.handleCloseClick})))},t.prototype.componentDidMount=function(){this.startTimeout()},t.prototype.componentDidUpdate=function(e){e.timeout!==this.props.timeout&&(this.props.timeout>0?this.startTimeout():this.clearTimeouts())},t.prototype.componentWillUnmount=function(){this.clearTimeouts()},t.prototype.maybeRenderActionButton=function(){var e=this.props.action;return null==e?void 0:d.createElement(Tt,Object(c.__assign)({},e,{intent:void 0,onClick:this.handleActionClick}))},t.prototype.triggerDismiss=function(e){var t,n;this.clearTimeouts(),null===(n=(t=this.props).onDismiss)||void 0===n||n.call(t,e)},t.defaultProps={className:"",message:"",timeout:5e3},t.displayName="Blueprint3.Toast",t=Object(c.__decorate)([ve.polyfill],t)}(v),Xr=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.state={toasts:[]},t.toastId=0,t.renderToast=function(e){return d.createElement(Gr,Object(c.__assign)({},e,{onDismiss:t.getDismissHandler(e)}))},t.getDismissHandler=function(e){return function(n){t.dismiss(e.key,n)}},t.handleClose=function(e){e.which===N&&t.clear()},t}var n;return Object(c.__extends)(t,e),n=t,t.create=function(e,t){void 0===t&&(t=document.body),null==e||null==e.usePortal||Object(h.e)("production")||console.warn(Ge.R);var r=document.createElement("div");t.appendChild(r);var i=ge.render(d.createElement(n,Object(c.__assign)({},e,{usePortal:!1})),r);if(null==i)throw new Error(Ge.P);return i},t.prototype.show=function(e,t){this.props.maxToasts&&this.dismissIfAtLimit();var n=this.createToastOptions(e,t);return void 0===t||this.isNewToastKey(t)?this.setState((function(e){return{toasts:Object(c.__spreadArrays)([n],e.toasts)}})):this.setState((function(e){return{toasts:e.toasts.map((function(e){return e.key===t?n:e}))}})),n.key},t.prototype.dismiss=function(e,t){void 0===t&&(t=!1),this.setState((function(n){return{toasts:n.toasts.filter((function(n){var r,i=n.key===e;return i&&(null===(r=n.onDismiss)||void 0===r||r.call(n,t)),!i}))}}))},t.prototype.clear=function(){this.state.toasts.forEach((function(e){var t;return null===(t=e.onDismiss)||void 0===t?void 0:t.call(e,!1)})),this.setState({toasts:[]})},t.prototype.getToasts=function(){return this.state.toasts},t.prototype.render=function(){var e=pe()(a.TOAST_CONTAINER,this.getPositionClasses(),this.props.className);return d.createElement(Je,{autoFocus:this.props.autoFocus,canEscapeKeyClose:this.props.canEscapeKeyClear,canOutsideClickClose:!1,className:e,enforceFocus:!1,hasBackdrop:!1,isOpen:this.state.toasts.length>0||null!=this.props.children,onClose:this.handleClose,transitionDuration:350,transitionName:a.TOAST,usePortal:this.props.usePortal},this.state.toasts.map(this.renderToast,this),this.props.children)},t.prototype.validateProps=function(e){var t=e.maxToasts;if(void 0!==t&&t<1)throw new Error(Ge.Q)},t.prototype.isNewToastKey=function(e){return this.state.toasts.every((function(t){return t.key!==e}))},t.prototype.dismissIfAtLimit=function(){this.state.toasts.length===this.props.maxToasts&&this.dismiss(this.state.toasts[this.state.toasts.length-1].key)},t.prototype.createToastOptions=function(e,t){return void 0===t&&(t="toast-"+this.toastId++),Object(c.__assign)(Object(c.__assign)({},e),{key:t})},t.prototype.getPositionClasses=function(){return this.props.position.split("-").map((function(e){return a.TOAST_CONTAINER+"-"+e.toLowerCase()}))},t.displayName="Blueprint3.Toaster",t.defaultProps={autoFocus:!1,canEscapeKeyClear:!0,position:_.a.TOP,usePortal:!0},t=n=Object(c.__decorate)([ve.polyfill],t)}(v),qr=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.handleCaretClick=function(e){var n;e.stopPropagation();var r=t.props,i=r.isExpanded,o=r.onCollapse,a=r.onExpand;null===(n=i?o:a)||void 0===n||n(t,e)},t.handleClick=function(e){var n,r;null===(r=(n=t.props).onClick)||void 0===r||r.call(n,t,e)},t.handleContentRef=function(e){var n,r;null===(r=(n=t.props).contentRef)||void 0===r||r.call(n,t,e)},t.handleContextMenu=function(e){var n,r;null===(r=(n=t.props).onContextMenu)||void 0===r||r.call(n,t,e)},t.handleDoubleClick=function(e){var n,r;null===(r=(n=t.props).onDoubleClick)||void 0===r||r.call(n,t,e)},t.handleMouseEnter=function(e){var n,r;null===(r=(n=t.props).onMouseEnter)||void 0===r||r.call(n,t,e)},t.handleMouseLeave=function(e){var n,r;null===(r=(n=t.props).onMouseLeave)||void 0===r||r.call(n,t,e)},t}return Object(c.__extends)(t,e),t.ofType=function(){return t},t.prototype.render=function(){var e,t=this.props,n=t.children,r=t.className,i=t.disabled,o=t.icon,s=t.isExpanded,l=t.isSelected,u=t.label,h=pe()(a.TREE_NODE,((e={})[a.DISABLED]=i,e[a.TREE_NODE_SELECTED]=l,e[a.TREE_NODE_EXPANDED]=s,e),r),f=pe()(a.TREE_NODE_CONTENT,a.TREE_NODE_CONTENT+"-"+this.props.depth),p=!0===i?{}:{onClick:this.handleClick,onContextMenu:this.handleContextMenu,onDoubleClick:this.handleDoubleClick,onMouseEnter:this.handleMouseEnter,onMouseLeave:this.handleMouseLeave};return d.createElement("li",{className:h},d.createElement("div",Object(c.__assign)({className:f,ref:this.handleContentRef},p),this.maybeRenderCaret(),d.createElement(_t,{className:a.TREE_NODE_ICON,icon:o}),d.createElement("span",{className:a.TREE_NODE_LABEL},u),this.maybeRenderSecondaryLabel()),d.createElement(on,{isOpen:s},n))},t.prototype.maybeRenderCaret=function(){var e=this.props,t=e.children,n=e.isExpanded,r=e.disabled,i=e.hasCaret;if(void 0===i?d.Children.count(t)>0:i){var o=pe()(a.TREE_NODE_CARET,n?a.TREE_NODE_CARET_OPEN:a.TREE_NODE_CARET_CLOSED),s=!0===r?void 0:this.handleCaretClick;return d.createElement(_t,{className:o,onClick:s,icon:"chevron-right"})}return d.createElement("span",{className:a.TREE_NODE_CARET_NONE})},t.prototype.maybeRenderSecondaryLabel=function(){return null!=this.props.secondaryLabel?d.createElement("span",{className:a.TREE_NODE_SECONDARY_LABEL},this.props.secondaryLabel):void 0},t.displayName="Blueprint3.TreeNode",t}(d.Component),Zr=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.nodeRefs={},t.handleNodeCollapse=function(e,n){t.handlerHelper(t.props.onNodeCollapse,e,n)},t.handleNodeClick=function(e,n){t.handlerHelper(t.props.onNodeClick,e,n)},t.handleContentRef=function(e,n){null!=n?t.nodeRefs[e.props.id]=n:delete t.nodeRefs[e.props.id]},t.handleNodeContextMenu=function(e,n){t.handlerHelper(t.props.onNodeContextMenu,e,n)},t.handleNodeDoubleClick=function(e,n){t.handlerHelper(t.props.onNodeDoubleClick,e,n)},t.handleNodeExpand=function(e,n){t.handlerHelper(t.props.onNodeExpand,e,n)},t.handleNodeMouseEnter=function(e,n){t.handlerHelper(t.props.onNodeMouseEnter,e,n)},t.handleNodeMouseLeave=function(e,n){t.handlerHelper(t.props.onNodeMouseLeave,e,n)},t}return Object(c.__extends)(t,e),t.ofType=function(){return t},t.nodeFromPath=function(e,n){return 1===e.length?n[e[0]]:t.nodeFromPath(e.slice(1),n[e[0]].childNodes)},t.prototype.render=function(){return d.createElement("div",{className:pe()(a.TREE,this.props.className)},this.renderNodes(this.props.contents,[],a.TREE_ROOT))},t.prototype.getNodeContentElement=function(e){return this.nodeRefs[e]},t.prototype.renderNodes=function(e,t,n){var r=this;if(null==e)return null;var i=e.map((function(e,n){var i=t.concat(n),o=qr.ofType();return d.createElement(o,Object(c.__assign)({},e,{key:e.id,contentRef:r.handleContentRef,depth:i.length-1,onClick:r.handleNodeClick,onContextMenu:r.handleNodeContextMenu,onCollapse:r.handleNodeCollapse,onDoubleClick:r.handleNodeDoubleClick,onExpand:r.handleNodeExpand,onMouseEnter:r.handleNodeMouseEnter,onMouseLeave:r.handleNodeMouseLeave,path:i}),r.renderNodes(e.childNodes,i))}));return d.createElement("ul",{className:pe()(a.TREE_NODE_LIST,n)},i)},t.prototype.handlerHelper=function(e,n,r){K(e)&&e(t.nodeFromPath(n.props.path,this.props.contents),n.props.path,r)},t.displayName="Blueprint3.Tree",t}(d.Component),Qr=o;n.d(t,"FocusStyleManager",(function(){return u})),n.d(t,"Classes",(function(){return a})),n.d(t,"Keys",(function(){return r})),n.d(t,"Utils",(function(){return i})),n.d(t,"AbstractComponent",(function(){return f})),n.d(t,"AbstractComponent2",(function(){return p})),n.d(t,"AbstractPureComponent",(function(){return g})),n.d(t,"AbstractPureComponent2",(function(){return v})),n.d(t,"Alignment",(function(){return m.a})),n.d(t,"Boundary",(function(){return y})),n.d(t,"Colors",(function(){return b})),n.d(t,"Elevation",(function(){return C.a})),n.d(t,"Intent",(function(){return A.a})),n.d(t,"Position",(function(){return _.a})),n.d(t,"isPositionHorizontal",(function(){return _.c})),n.d(t,"isPositionVertical",(function(){return _.d})),n.d(t,"getPositionIgnoreAngles",(function(){return _.b})),n.d(t,"DISPLAYNAME_PREFIX",(function(){return"Blueprint3"})),n.d(t,"removeNonHTMLProps",(function(){return x})),n.d(t,"isRefObject",(function(){return w})),n.d(t,"isRefCallback",(function(){return M})),n.d(t,"combineRefs",(function(){return S})),n.d(t,"getRef",(function(){return T})),n.d(t,"setRef",(function(){return E})),n.d(t,"refHandler",(function(){return O})),n.d(t,"ContextMenu",(function(){return Qr})),n.d(t,"Alert",(function(){return Yt})),n.d(t,"Breadcrumb",(function(){return Wt})),n.d(t,"Breadcrumbs",(function(){return en})),n.d(t,"Button",(function(){return St})),n.d(t,"AnchorButton",(function(){return Tt})),n.d(t,"ButtonGroup",(function(){return tn})),n.d(t,"Callout",(function(){return nn})),n.d(t,"Card",(function(){return rn})),n.d(t,"AnimationStates",(function(){return Jt})),n.d(t,"Collapse",(function(){return on})),n.d(t,"CollapsibleList",(function(){return an})),n.d(t,"ContextMenuTarget",(function(){return sn})),n.d(t,"Dialog",(function(){return Vt})),n.d(t,"MultistepDialog",(function(){return cn})),n.d(t,"DialogStep",(function(){return ln})),n.d(t,"Divider",(function(){return hn})),n.d(t,"Drawer",(function(){return fn})),n.d(t,"EditableText",(function(){return mn})),n.d(t,"ControlGroup",(function(){return Cn})),n.d(t,"Switch",(function(){return _n})),n.d(t,"Radio",(function(){return In})),n.d(t,"Checkbox",(function(){return xn})),n.d(t,"FileInput",(function(){return wn})),n.d(t,"FormGroup",(function(){return Mn})),n.d(t,"InputGroup",(function(){return Tn})),n.d(t,"NumericInput",(function(){return Rn})),n.d(t,"RadioGroup",(function(){return Bn})),n.d(t,"TextArea",(function(){return Hn})),n.d(t,"H1",(function(){return kt})),n.d(t,"H2",(function(){return Lt})),n.d(t,"H3",(function(){return Dt})),n.d(t,"H4",(function(){return Pt})),n.d(t,"H5",(function(){return Nt})),n.d(t,"H6",(function(){return jt})),n.d(t,"Blockquote",(function(){return Rt})),n.d(t,"Code",(function(){return zt})),n.d(t,"Pre",(function(){return Ft})),n.d(t,"Label",(function(){return Bt})),n.d(t,"OL",(function(){return Ht})),n.d(t,"UL",(function(){return Ut})),n.d(t,"HTMLSelect",(function(){return Un})),n.d(t,"HTMLTable",(function(){return Vn})),n.d(t,"Hotkey",(function(){return rr})),n.d(t,"KeyCombo",(function(){return nr})),n.d(t,"HotkeysTarget",(function(){return dr})),n.d(t,"comboMatches",(function(){return Qn})),n.d(t,"getKeyCombo",(function(){return $n})),n.d(t,"getKeyComboString",(function(){return Jn})),n.d(t,"parseKeyCombo",(function(){return Kn})),n.d(t,"hideHotkeysDialog",(function(){return lr})),n.d(t,"setHotkeysDialogProps",(function(){return sr})),n.d(t,"Icon",(function(){return _t})),n.d(t,"Menu",(function(){return Kt})),n.d(t,"MenuDivider",(function(){return Gt})),n.d(t,"MenuItem",(function(){return qt})),n.d(t,"Navbar",(function(){return gr})),n.d(t,"NavbarDivider",(function(){return hr})),n.d(t,"NavbarGroup",(function(){return fr})),n.d(t,"NavbarHeading",(function(){return pr})),n.d(t,"NonIdealState",(function(){return vr})),n.d(t,"OverflowDirection",(function(){return Ot})),n.d(t,"OverflowList",(function(){return $t})),n.d(t,"Overlay",(function(){return Je})),n.d(t,"Text",(function(){return Xt})),n.d(t,"PanelStack",(function(){return yr})),n.d(t,"PopoverInteractionKind",(function(){return ft})),n.d(t,"Popover",(function(){return pt})),n.d(t,"PopoverPosition",(function(){return br})),n.d(t,"Portal",(function(){return Qe})),n.d(t,"ProgressBar",(function(){return Cr})),n.d(t,"ResizeSensor",(function(){return et})),n.d(t,"HandleType",(function(){return Ar})),n.d(t,"HandleInteractionKind",(function(){return _r})),n.d(t,"MultiSlider",(function(){return Er})),n.d(t,"RangeSlider",(function(){return Dr})),n.d(t,"Slider",(function(){return Pr})),n.d(t,"Spinner",(function(){return wt})),n.d(t,"Tab",(function(){return Nr})),n.d(t,"Expander",(function(){return Fr})),n.d(t,"Tabs",(function(){return Hr})),n.d(t,"Tag",(function(){return Yr})),n.d(t,"TagInput",(function(){return Wr})),n.d(t,"Toast",(function(){return Gr})),n.d(t,"Toaster",(function(){return Xr})),n.d(t,"Tooltip",(function(){return tt})),n.d(t,"Tree",(function(){return Zr})),n.d(t,"TreeNode",(function(){return qr})),n.d(t,"Hotkeys",(function(){return ir}))},function(e,t,n){"use strict";n.r(t);var r=n(5),i=n(21),o=n(1),a=n(0),s=(n(3),n(4)),l=n(7),u=n(2362),c=n(2377),d=n(336),h=n(452),f=n(1196),p=n(57),g=Object(p.a)(a.createElement("path",{d:"M19 6.41L17.59 5 12 10.59 6.41 5 5 6.41 10.59 12 5 17.59 6.41 19 12 13.41 17.59 19 19 17.59 13.41 12z"}),"Close"),v=Object(p.a)(a.createElement("path",{d:"M7 10l5 5 5-5z"}),"ArrowDropDown"),m=n(49),y=(n(140),n(186)),b=n(100),C=n(53),A=n(87);function _(e){return void 0!==e.normalize?e.normalize("NFD").replace(/[\u0300-\u036f]/g,""):e}function I(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},t=e.ignoreAccents,n=void 0===t||t,r=e.ignoreCase,i=void 0===r||r,o=e.limit,a=e.matchFrom,s=void 0===a?"any":a,l=e.stringify,u=e.trim,c=void 0!==u&&u;return function(e,t){var r=t.inputValue,a=t.getOptionLabel,u=c?r.trim():r;i&&(u=u.toLowerCase()),n&&(u=_(u));var d=e.filter((function(e){var t=(l||a)(e);return i&&(t=t.toLowerCase()),n&&(t=_(t)),"start"===s?0===t.indexOf(u):t.indexOf(u)>-1}));return"number"==typeof o?d.slice(0,o):d}}function x(e,t){for(var n=0;n<e.length;n+=1)if(t(e[n]))return n;return-1}var w=I();function M(e){var t=e.autoComplete,n=void 0!==t&&t,r=e.autoHighlight,i=void 0!==r&&r,s=e.autoSelect,l=void 0!==s&&s,u=e.blurOnSelect,c=void 0!==u&&u,d=e.clearOnBlur,h=void 0===d?!e.freeSolo:d,f=e.clearOnEscape,p=void 0!==f&&f,g=e.componentName,v=void 0===g?"useAutocomplete":g,_=e.debug,I=void 0!==_&&_,M=e.defaultValue,S=void 0===M?e.multiple?[]:null:M,T=e.disableClearable,E=void 0!==T&&T,O=e.disableCloseOnSelect,k=void 0!==O&&O,L=e.disabledItemsFocusable,D=void 0!==L&&L,P=e.disableListWrap,N=void 0!==P&&P,j=e.filterOptions,R=void 0===j?w:j,z=e.filterSelectedOptions,F=void 0!==z&&z,B=e.freeSolo,H=void 0!==B&&B,U=e.getOptionDisabled,V=e.getOptionLabel,Y=void 0===V?function(e){return e}:V,W=e.getOptionSelected,G=void 0===W?function(e,t){return e===t}:W,X=e.groupBy,q=e.handleHomeEndKeys,Z=void 0===q?!e.freeSolo:q,Q=e.id,K=e.includeInputInList,J=void 0!==K&&K,$=e.inputValue,ee=e.multiple,te=void 0!==ee&&ee,ne=e.onChange,re=e.onClose,ie=e.onHighlightChange,oe=e.onInputChange,ae=e.onOpen,se=e.open,le=e.openOnFocus,ue=void 0!==le&&le,ce=e.options,de=e.selectOnFocus,he=void 0===de?!e.freeSolo:de,fe=e.value,pe=Object(y.a)(Q),ge=Y;var ve=a.useRef(!1),me=a.useRef(!0),ye=a.useRef(null),be=a.useRef(null),Ce=a.useState(null),Ae=Ce[0],_e=Ce[1],Ie=a.useState(-1),xe=Ie[0],we=Ie[1],Me=i?0:-1,Se=a.useRef(Me),Te=Object(b.a)({controlled:fe,default:S,name:v}),Ee=Object(m.a)(Te,2),Oe=Ee[0],ke=Ee[1],Le=Object(b.a)({controlled:$,default:"",name:v,state:"inputValue"}),De=Object(m.a)(Le,2),Pe=De[0],Ne=De[1],je=a.useState(!1),Re=je[0],ze=je[1],Fe=Object(C.a)((function(e,t){var n;if(te)n="";else if(null==t)n="";else{var r=ge(t);n="string"==typeof r?r:""}Pe!==n&&(Ne(n),oe&&oe(e,n,"reset"))}));a.useEffect((function(){Fe(null,Oe)}),[Oe,Fe]);var Be=Object(b.a)({controlled:se,default:!1,name:v,state:"open"}),He=Object(m.a)(Be,2),Ue=He[0],Ve=He[1],Ye=!te&&null!=Oe&&Pe===ge(Oe),We=Ue,Ge=We?R(ce.filter((function(e){return!F||!(te?Oe:[Oe]).some((function(t){return null!==t&&G(e,t)}))})),{inputValue:Ye?"":Pe,getOptionLabel:ge}):[],Xe=Object(C.a)((function(e){-1===e?ye.current.focus():Ae.querySelector('[data-tag-index="'.concat(e,'"]')).focus()}));a.useEffect((function(){te&&xe>Oe.length-1&&(we(-1),Xe(-1))}),[Oe,te,xe,Xe]);var qe=Object(C.a)((function(e){var t=e.event,n=e.index,r=e.reason,i=void 0===r?"auto":r;if(Se.current=n,-1===n?ye.current.removeAttribute("aria-activedescendant"):ye.current.setAttribute("aria-activedescendant","".concat(pe,"-option-").concat(n)),ie&&ie(t,-1===n?null:Ge[n],i),be.current){var o=be.current.querySelector("[data-focus]");o&&o.removeAttribute("data-focus");var a=be.current.parentElement.querySelector('[role="listbox"]');if(a)if(-1!==n){var s=be.current.querySelector('[data-option-index="'.concat(n,'"]'));if(s&&(s.setAttribute("data-focus","true"),a.scrollHeight>a.clientHeight&&"mouse"!==i)){var l=s,u=a.clientHeight+a.scrollTop,c=l.offsetTop+l.offsetHeight;c>u?a.scrollTop=c-a.clientHeight:l.offsetTop-l.offsetHeight*(X?1.3:0)<a.scrollTop&&(a.scrollTop=l.offsetTop-l.offsetHeight*(X?1.3:0))}}else a.scrollTop=0}})),Ze=Object(C.a)((function(e){var t=e.event,r=e.diff,i=e.direction,o=void 0===i?"next":i,a=e.reason,s=void 0===a?"auto":a;if(We){var l=function(e,t){if(!be.current||-1===e)return-1;for(var n=e;;){if("next"===t&&n===Ge.length||"previous"===t&&-1===n)return-1;var r=be.current.querySelector('[data-option-index="'.concat(n,'"]')),i=!D&&(r&&(r.disabled||"true"===r.getAttribute("aria-disabled")));if(!(r&&!r.hasAttribute("tabindex")||i))return n;n+="next"===t?1:-1}}(function(){var e=Ge.length-1;if("reset"===r)return Me;if("start"===r)return 0;if("end"===r)return e;var t=Se.current+r;return t<0?-1===t&&J?-1:N&&-1!==Se.current||Math.abs(r)>1?0:e:t>e?t===e+1&&J?-1:N||Math.abs(r)>1?e:0:t}(),o);if(qe({index:l,reason:s,event:t}),n&&"reset"!==r)if(-1===l)ye.current.value=Pe;else{var u=ge(Ge[l]);ye.current.value=u,0===u.toLowerCase().indexOf(Pe.toLowerCase())&&Pe.length>0&&ye.current.setSelectionRange(Pe.length,u.length)}}})),Qe=a.useCallback((function(){if(We){var e=te?Oe[0]:Oe;if(0!==Ge.length&&null!=e){if(be.current)if(F||null==e)Se.current>=Ge.length-1?qe({index:Ge.length-1}):qe({index:Se.current});else{var t=Ge[Se.current];if(te&&t&&-1!==x(Oe,(function(e){return G(t,e)})))return;var n=x(Ge,(function(t){return G(t,e)}));-1===n?Ze({diff:"reset"}):qe({index:n})}}else Ze({diff:"reset"})}}),[0===Ge.length,!te&&Oe,F,Ze,qe,We,Pe,te]),Ke=Object(C.a)((function(e){Object(A.a)(be,e),e&&Qe()}));a.useEffect((function(){Qe()}),[Qe]);var Je=function(e){Ue||(Ve(!0),ae&&ae(e))},$e=function(e,t){Ue&&(Ve(!1),re&&re(e,t))},et=function(e,t,n,r){Oe!==t&&(ne&&ne(e,t,n,r),ke(t))},tt=a.useRef(!1),nt=function(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:"select-option",r=arguments.length>3&&void 0!==arguments[3]?arguments[3]:"options",i=n,o=t;if(te){var a=x(o=Array.isArray(Oe)?Oe.slice():[],(function(e){return G(t,e)}));-1===a?o.push(t):"freeSolo"!==r&&(o.splice(a,1),i="remove-option")}Fe(e,o),et(e,o,i,{option:t}),k||$e(e,i),(!0===c||"touch"===c&&tt.current||"mouse"===c&&!tt.current)&&ye.current.blur()};var rt=function(e,t){if(te){$e(e,"toggleInput");var n=xe;-1===xe?""===Pe&&"previous"===t&&(n=Oe.length-1):((n+="next"===t?1:-1)<0&&(n=0),n===Oe.length&&(n=-1)),n=function(e,t){if(-1===e)return-1;for(var n=e;;){if("next"===t&&n===Oe.length||"previous"===t&&-1===n)return-1;var r=Ae.querySelector('[data-tag-index="'.concat(n,'"]'));if(!r||r.hasAttribute("tabindex")&&!r.disabled&&"true"!==r.getAttribute("aria-disabled"))return n;n+="next"===t?1:-1}}(n,t),we(n),Xe(n)}},it=function(e){ve.current=!0,Ne(""),oe&&oe(e,"","clear"),et(e,te?[]:null,"clear")},ot=function(e){return function(t){switch(-1!==xe&&-1===["ArrowLeft","ArrowRight"].indexOf(t.key)&&(we(-1),Xe(-1)),t.key){case"Home":We&&Z&&(t.preventDefault(),Ze({diff:"start",direction:"next",reason:"keyboard",event:t}));break;case"End":We&&Z&&(t.preventDefault(),Ze({diff:"end",direction:"previous",reason:"keyboard",event:t}));break;case"PageUp":t.preventDefault(),Ze({diff:-5,direction:"previous",reason:"keyboard",event:t}),Je(t);break;case"PageDown":t.preventDefault(),Ze({diff:5,direction:"next",reason:"keyboard",event:t}),Je(t);break;case"ArrowDown":t.preventDefault(),Ze({diff:1,direction:"next",reason:"keyboard",event:t}),Je(t);break;case"ArrowUp":t.preventDefault(),Ze({diff:-1,direction:"previous",reason:"keyboard",event:t}),Je(t);break;case"ArrowLeft":rt(t,"previous");break;case"ArrowRight":rt(t,"next");break;case"Enter":if(229===t.which)break;if(-1!==Se.current&&We){var r=Ge[Se.current],i=!!U&&U(r);if(t.preventDefault(),i)return;nt(t,r,"select-option"),n&&ye.current.setSelectionRange(ye.current.value.length,ye.current.value.length)}else H&&""!==Pe&&!1===Ye&&(te&&t.preventDefault(),nt(t,Pe,"create-option","freeSolo"));break;case"Escape":We?(t.preventDefault(),t.stopPropagation(),$e(t,"escape")):p&&(""!==Pe||te&&Oe.length>0)&&(t.preventDefault(),t.stopPropagation(),it(t));break;case"Backspace":if(te&&""===Pe&&Oe.length>0){var o=-1===xe?Oe.length-1:xe,a=Oe.slice();a.splice(o,1),et(t,a,"remove-option",{option:Oe[o]})}}e.onKeyDown&&e.onKeyDown(t)}},at=function(e){ze(!0),ue&&!ve.current&&Je(e)},st=function(e){null===be.current||document.activeElement!==be.current.parentElement?(ze(!1),me.current=!0,ve.current=!1,I&&""!==Pe||(l&&-1!==Se.current&&We?nt(e,Ge[Se.current],"blur"):l&&H&&""!==Pe?nt(e,Pe,"blur","freeSolo"):h&&Fe(e,Oe),$e(e,"blur"))):ye.current.focus()},lt=function(e){var t=e.target.value;Pe!==t&&(Ne(t),oe&&oe(e,t,"input")),""===t?E||te||et(e,null,"clear"):Je(e)},ut=function(e){qe({event:e,index:Number(e.currentTarget.getAttribute("data-option-index")),reason:"mouse"})},ct=function(){tt.current=!0},dt=function(e){var t=Number(e.currentTarget.getAttribute("data-option-index"));nt(e,Ge[t],"select-option"),tt.current=!1},ht=function(e){return function(t){var n=Oe.slice();n.splice(e,1),et(t,n,"remove-option",{option:Oe[e]})}},ft=function(e){Ue?$e(e,"toggleInput"):Je(e)},pt=function(e){e.target.getAttribute("id")!==pe&&e.preventDefault()},gt=function(){ye.current.focus(),he&&me.current&&ye.current.selectionEnd-ye.current.selectionStart==0&&ye.current.select(),me.current=!1},vt=function(e){""!==Pe&&Ue||ft(e)},mt=H&&Pe.length>0;mt=mt||(te?Oe.length>0:null!==Oe);var yt=Ge;if(X){new Map;yt=Ge.reduce((function(e,t,n){var r=X(t);return e.length>0&&e[e.length-1].group===r?e[e.length-1].options.push(t):e.push({key:n,index:n,group:r,options:[t]}),e}),[])}return{getRootProps:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};return Object(o.a)({"aria-owns":We?"".concat(pe,"-popup"):null,role:"combobox","aria-expanded":We},e,{onKeyDown:ot(e),onMouseDown:pt,onClick:gt})},getInputLabelProps:function(){return{id:"".concat(pe,"-label"),htmlFor:pe}},getInputProps:function(){return{id:pe,value:Pe,onBlur:st,onFocus:at,onChange:lt,onMouseDown:vt,"aria-activedescendant":We?"":null,"aria-autocomplete":n?"both":"list","aria-controls":We?"".concat(pe,"-popup"):null,autoComplete:"off",ref:ye,autoCapitalize:"none",spellCheck:"false"}},getClearProps:function(){return{tabIndex:-1,onClick:it}},getPopupIndicatorProps:function(){return{tabIndex:-1,onClick:ft}},getTagProps:function(e){var t=e.index;return{key:t,"data-tag-index":t,tabIndex:-1,onDelete:ht(t)}},getListboxProps:function(){return{role:"listbox",id:"".concat(pe,"-popup"),"aria-labelledby":"".concat(pe,"-label"),ref:Ke,onMouseDown:function(e){e.preventDefault()}}},getOptionProps:function(e){var t=e.index,n=e.option,r=(te?Oe:[Oe]).some((function(e){return null!=e&&G(n,e)})),i=!!U&&U(n);return{key:t,tabIndex:-1,role:"option",id:"".concat(pe,"-option-").concat(t),onMouseOver:ut,onClick:dt,onTouchStart:ct,"data-option-index":t,"aria-disabled":i,"aria-selected":r}},id:pe,inputValue:Pe,value:Oe,dirty:mt,popupOpen:We,focused:Re||-1!==xe,anchorEl:Ae,setAnchorEl:_e,focusedTag:xe,groupedOptions:yt}}function S(e){e.anchorEl,e.open;var t=Object(r.a)(e,["anchorEl","open"]);return a.createElement("div",t)}var T=a.createElement(g,{fontSize:"small"}),E=a.createElement(v,null),O=a.forwardRef((function(e,t){e.autoComplete,e.autoHighlight,e.autoSelect,e.blurOnSelect;var n,i=e.ChipProps,l=e.classes,p=e.className,g=e.clearOnBlur,v=(void 0===g&&e.freeSolo,e.clearOnEscape,e.clearText),m=void 0===v?"Clear":v,y=e.closeIcon,b=void 0===y?T:y,C=e.closeText,A=void 0===C?"Close":C,_=(e.debug,e.defaultValue),I=(void 0===_&&e.multiple,e.disableClearable),x=void 0!==I&&I,w=(e.disableCloseOnSelect,e.disabled),O=void 0!==w&&w,k=(e.disabledItemsFocusable,e.disableListWrap,e.disablePortal),L=void 0!==k&&k,D=(e.filterOptions,e.filterSelectedOptions,e.forcePopupIcon),P=void 0===D?"auto":D,N=e.freeSolo,j=void 0!==N&&N,R=e.fullWidth,z=void 0!==R&&R,F=e.getLimitTagsText,B=void 0===F?function(e){return"+".concat(e)}:F,H=(e.getOptionDisabled,e.getOptionLabel),U=void 0===H?function(e){return e}:H,V=(e.getOptionSelected,e.groupBy),Y=e.handleHomeEndKeys,W=(void 0===Y&&e.freeSolo,e.id,e.includeInputInList,e.inputValue,e.limitTags),G=void 0===W?-1:W,X=e.ListboxComponent,q=void 0===X?"ul":X,Z=e.ListboxProps,Q=e.loading,K=void 0!==Q&&Q,J=e.loadingText,$=void 0===J?"Loading…":J,ee=e.multiple,te=void 0!==ee&&ee,ne=e.noOptionsText,re=void 0===ne?"No options":ne,ie=(e.onChange,e.onClose,e.onHighlightChange,e.onInputChange,e.onOpen,e.open,e.openOnFocus,e.openText),oe=void 0===ie?"Open":ie,ae=(e.options,e.PaperComponent),se=void 0===ae?d.a:ae,le=e.PopperComponent,ue=void 0===le?u.a:le,ce=e.popupIcon,de=void 0===ce?E:ce,he=e.renderGroup,fe=e.renderInput,pe=e.renderOption,ge=e.renderTags,ve=e.selectOnFocus,me=(void 0===ve&&e.freeSolo,e.size),ye=void 0===me?"medium":me,be=(e.value,Object(r.a)(e,["autoComplete","autoHighlight","autoSelect","blurOnSelect","ChipProps","classes","className","clearOnBlur","clearOnEscape","clearText","closeIcon","closeText","debug","defaultValue","disableClearable","disableCloseOnSelect","disabled","disabledItemsFocusable","disableListWrap","disablePortal","filterOptions","filterSelectedOptions","forcePopupIcon","freeSolo","fullWidth","getLimitTagsText","getOptionDisabled","getOptionLabel","getOptionSelected","groupBy","handleHomeEndKeys","id","includeInputInList","inputValue","limitTags","ListboxComponent","ListboxProps","loading","loadingText","multiple","noOptionsText","onChange","onClose","onHighlightChange","onInputChange","onOpen","open","openOnFocus","openText","options","PaperComponent","PopperComponent","popupIcon","renderGroup","renderInput","renderOption","renderTags","selectOnFocus","size","value"])),Ce=L?S:ue,Ae=M(Object(o.a)({},e,{componentName:"Autocomplete"})),_e=Ae.getRootProps,Ie=Ae.getInputProps,xe=Ae.getInputLabelProps,we=Ae.getPopupIndicatorProps,Me=Ae.getClearProps,Se=Ae.getTagProps,Te=Ae.getListboxProps,Ee=Ae.getOptionProps,Oe=Ae.value,ke=Ae.dirty,Le=Ae.id,De=Ae.popupOpen,Pe=Ae.focused,Ne=Ae.focusedTag,je=Ae.anchorEl,Re=Ae.setAnchorEl,ze=Ae.inputValue,Fe=Ae.groupedOptions;if(te&&Oe.length>0){var Be=function(e){return Object(o.a)({className:Object(s.a)(l.tag,"small"===ye&&l.tagSizeSmall),disabled:O},Se(e))};n=ge?ge(Oe,Be):Oe.map((function(e,t){return a.createElement(f.a,Object(o.a)({label:U(e),size:ye},Be({index:t}),i))}))}if(G>-1&&Array.isArray(n)){var He=n.length-G;!Pe&&He>0&&(n=n.splice(0,G)).push(a.createElement("span",{className:l.tag,key:n.length},B(He)))}var Ue=he||function(e){return a.createElement("li",{key:e.key},a.createElement(c.a,{className:l.groupLabel,component:"div"},e.group),a.createElement("ul",{className:l.groupUl},e.children))},Ve=pe||U,Ye=function(e,t){var n=Ee({option:e,index:t});return a.createElement("li",Object(o.a)({},n,{className:l.option}),Ve(e,{selected:n["aria-selected"],inputValue:ze}))},We=!x&&!O,Ge=(!j||!0===P)&&!1!==P;return a.createElement(a.Fragment,null,a.createElement("div",Object(o.a)({ref:t,className:Object(s.a)(l.root,p,Pe&&l.focused,z&&l.fullWidth,We&&l.hasClearIcon,Ge&&l.hasPopupIcon)},_e(be)),fe({id:Le,disabled:O,fullWidth:!0,size:"small"===ye?"small":void 0,InputLabelProps:xe(),InputProps:{ref:Re,className:l.inputRoot,startAdornment:n,endAdornment:a.createElement("div",{className:l.endAdornment},We?a.createElement(h.a,Object(o.a)({},Me(),{"aria-label":m,title:m,className:Object(s.a)(l.clearIndicator,ke&&l.clearIndicatorDirty)}),b):null,Ge?a.createElement(h.a,Object(o.a)({},we(),{disabled:O,"aria-label":De?A:oe,title:De?A:oe,className:Object(s.a)(l.popupIndicator,De&&l.popupIndicatorOpen)}),de):null)},inputProps:Object(o.a)({className:Object(s.a)(l.input,-1===Ne&&l.inputFocused),disabled:O},Ie())})),De&&je?a.createElement(Ce,{className:Object(s.a)(l.popper,L&&l.popperDisablePortal),style:{width:je?je.clientWidth:null},role:"presentation",anchorEl:je,open:!0},a.createElement(se,{className:l.paper},K&&0===Fe.length?a.createElement("div",{className:l.loading},$):null,0!==Fe.length||j||K?null:a.createElement("div",{className:l.noOptions},re),Fe.length>0?a.createElement(q,Object(o.a)({className:l.listbox},Te(),Z),Fe.map((function(e,t){return V?Ue({key:e.key,group:e.group,children:e.options.map((function(t,n){return Ye(t,e.index+n)}))}):Ye(e,t)}))):null)):null)})),k=Object(l.a)((function(e){var t;return{root:{"&$focused $clearIndicatorDirty":{visibility:"visible"},"@media (pointer: fine)":{"&:hover $clearIndicatorDirty":{visibility:"visible"}}},fullWidth:{width:"100%"},focused:{},tag:{margin:3,maxWidth:"calc(100% - 6px)"},tagSizeSmall:{margin:2,maxWidth:"calc(100% - 4px)"},hasPopupIcon:{},hasClearIcon:{},inputRoot:{flexWrap:"wrap","$hasPopupIcon &, $hasClearIcon &":{paddingRight:30},"$hasPopupIcon$hasClearIcon &":{paddingRight:56},"& $input":{width:0,minWidth:30},'&[class*="MuiInput-root"]':{paddingBottom:1,"& $input":{padding:4},"& $input:first-child":{padding:"6px 0"}},'&[class*="MuiInput-root"][class*="MuiInput-marginDense"]':{"& $input":{padding:"4px 4px 5px"},"& $input:first-child":{padding:"3px 0 6px"}},'&[class*="MuiOutlinedInput-root"]':{padding:9,"$hasPopupIcon &, $hasClearIcon &":{paddingRight:39},"$hasPopupIcon$hasClearIcon &":{paddingRight:65},"& $input":{padding:"9.5px 4px"},"& $input:first-child":{paddingLeft:6},"& $endAdornment":{right:9}},'&[class*="MuiOutlinedInput-root"][class*="MuiOutlinedInput-marginDense"]':{padding:6,"& $input":{padding:"4.5px 4px"}},'&[class*="MuiFilledInput-root"]':{paddingTop:19,paddingLeft:8,"$hasPopupIcon &, $hasClearIcon &":{paddingRight:39},"$hasPopupIcon$hasClearIcon &":{paddingRight:65},"& $input":{padding:"9px 4px"},"& $endAdornment":{right:9}},'&[class*="MuiFilledInput-root"][class*="MuiFilledInput-marginDense"]':{paddingBottom:1,"& $input":{padding:"4.5px 4px"}}},input:{flexGrow:1,textOverflow:"ellipsis",opacity:0},inputFocused:{opacity:1},endAdornment:{position:"absolute",right:0,top:"calc(50% - 14px)"},clearIndicator:{marginRight:-2,padding:4,visibility:"hidden"},clearIndicatorDirty:{},popupIndicator:{padding:2,marginRight:-2},popupIndicatorOpen:{transform:"rotate(180deg)"},popper:{zIndex:e.zIndex.modal},popperDisablePortal:{position:"absolute"},paper:Object(o.a)({},e.typography.body1,{overflow:"hidden",margin:"4px 0"}),listbox:{listStyle:"none",margin:0,padding:"8px 0",maxHeight:"40vh",overflow:"auto"},loading:{color:e.palette.text.secondary,padding:"14px 16px"},noOptions:{color:e.palette.text.secondary,padding:"14px 16px"},option:(t={minHeight:48,display:"flex",justifyContent:"flex-start",alignItems:"center",cursor:"pointer",paddingTop:6,boxSizing:"border-box",outline:"0",WebkitTapHighlightColor:"transparent",paddingBottom:6,paddingLeft:16,paddingRight:16},Object(i.a)(t,e.breakpoints.up("sm"),{minHeight:"auto"}),Object(i.a)(t,'&[aria-selected="true"]',{backgroundColor:e.palette.action.selected}),Object(i.a)(t,'&[data-focus="true"]',{backgroundColor:e.palette.action.hover}),Object(i.a)(t,"&:active",{backgroundColor:e.palette.action.selected}),Object(i.a)(t,'&[aria-disabled="true"]',{opacity:e.palette.action.disabledOpacity,pointerEvents:"none"}),t),groupLabel:{backgroundColor:e.palette.background.paper,top:-8},groupUl:{padding:0,"& $option":{paddingLeft:24}}}}),{name:"MuiAutocomplete"})(O);n.d(t,"default",(function(){return k})),n.d(t,"createFilterOptions",(function(){return I}))},function(e,t,n){"use strict";n.d(t,"a",(function(){return r}));class r{constructor(e={}){this.trusted=!!e.trusted,this._data=e.data||{},this._metadata=e.metadata||{},this._callback=e.callback||i.noOp}get data(){return this._data}get metadata(){return this._metadata}setData(e){this._data=e.data||this._data,this._metadata=e.metadata||this._metadata,this._callback(e)}}var i;!function(e){e.noOp=function(){}}(i||(i={}))},function(e,t,n){"use strict";n.d(t,"a",(function(){return a}));var r=n(263),i=n(16),o=n(18);class a{constructor(e){this._currentChanged=new i.Signal(this),this._isDisposed=!1,this._widgetAdded=new i.Signal(this),this._widgetUpdated=new i.Signal(this);const t=this._focusTracker=new o.FocusTracker,n=this._pool=new r.RestorablePool(e);this.namespace=e.namespace,t.currentChanged.connect((e,t)=>{t.newValue!==this.currentWidget&&(n.current=t.newValue)},this),n.added.connect((e,t)=>{this._widgetAdded.emit(t)},this),n.currentChanged.connect((e,r)=>{null===r&&t.currentWidget?n.current=t.currentWidget:(this.onCurrentChanged(r),this._currentChanged.emit(r))},this),n.updated.connect((e,t)=>{this._widgetUpdated.emit(t)},this)}get currentChanged(){return this._currentChanged}get currentWidget(){return this._pool.current||null}get restored(){return this._pool.restored}get size(){return this._pool.size}get widgetAdded(){return this._widgetAdded}get widgetUpdated(){return this._widgetUpdated}async add(e){this._focusTracker.add(e),await this._pool.add(e),this._focusTracker.activeWidget||(this._pool.current=e)}get isDisposed(){return this._isDisposed}dispose(){this.isDisposed||(this._isDisposed=!0,this._pool.dispose(),this._focusTracker.dispose(),i.Signal.clearData(this))}find(e){return this._pool.find(e)}forEach(e){return this._pool.forEach(e)}filter(e){return this._pool.filter(e)}inject(e){return this._pool.inject(e)}has(e){return this._pool.has(e)}async restore(e){return this._pool.restore(e)}async save(e){return this._pool.save(e)}onCurrentChanged(e){}}},function(e,t,n){"use strict";var r=n(1),i=n(2350),o=n(167);t.a=function(e){var t=Object(i.a)(e);return function(e,n){return t(e,Object(r.a)({defaultTheme:o.a},n))}}},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(116),n(3),n(4)),s=n(7),l=n(633),u=n(2361),c=n(22),d=n(87),h=n(52),f={vertical:"top",horizontal:"right"},p={vertical:"top",horizontal:"left"},g=o.forwardRef((function(e,t){var n=e.autoFocus,s=void 0===n||n,g=e.children,v=e.classes,m=e.disableAutoFocusItem,y=void 0!==m&&m,b=e.MenuListProps,C=void 0===b?{}:b,A=e.onClose,_=e.onEntering,I=e.open,x=e.PaperProps,w=void 0===x?{}:x,M=e.PopoverClasses,S=e.transitionDuration,T=void 0===S?"auto":S,E=e.variant,O=void 0===E?"selectedMenu":E,k=Object(i.a)(e,["autoFocus","children","classes","disableAutoFocusItem","MenuListProps","onClose","onEntering","open","PaperProps","PopoverClasses","transitionDuration","variant"]),L=Object(h.a)(),D=s&&!y&&I,P=o.useRef(null),N=o.useRef(null),j=-1;o.Children.map(g,(function(e,t){o.isValidElement(e)&&(e.props.disabled||("menu"!==O&&e.props.selected||-1===j)&&(j=t))}));var R=o.Children.map(g,(function(e,t){return t===j?o.cloneElement(e,{ref:function(t){N.current=c.findDOMNode(t),Object(d.a)(e.ref,t)}}):e}));return o.createElement(l.a,Object(r.a)({getContentAnchorEl:function(){return N.current},classes:M,onClose:A,onEntering:function(e,t){P.current&&P.current.adjustStyleForScrollbar(e,L),_&&_(e,t)},anchorOrigin:"rtl"===L.direction?f:p,transformOrigin:"rtl"===L.direction?f:p,PaperProps:Object(r.a)({},w,{classes:Object(r.a)({},w.classes,{root:v.paper})}),open:I,ref:t,transitionDuration:T},k),o.createElement(u.a,Object(r.a)({onKeyDown:function(e){"Tab"===e.key&&(e.preventDefault(),A&&A(e,"tabKeyDown"))},actions:P,autoFocus:s&&(-1===j||y),autoFocusItem:D,variant:O},C,{className:Object(a.a)(v.list,C.className)}),R))}));t.a=Object(s.a)({paper:{maxHeight:"calc(100% - 96px)",WebkitOverflowScrolling:"touch"},list:{outline:0}},{name:"MuiMenu"})(g)},function(e,t,n){"use strict";t.byteLength=function(e){var t=u(e),n=t[0],r=t[1];return 3*(n+r)/4-r},t.toByteArray=function(e){var t,n,r=u(e),a=r[0],s=r[1],l=new o(function(e,t,n){return 3*(t+n)/4-n}(0,a,s)),c=0,d=s>0?a-4:a;for(n=0;n<d;n+=4)t=i[e.charCodeAt(n)]<<18|i[e.charCodeAt(n+1)]<<12|i[e.charCodeAt(n+2)]<<6|i[e.charCodeAt(n+3)],l[c++]=t>>16&255,l[c++]=t>>8&255,l[c++]=255&t;2===s&&(t=i[e.charCodeAt(n)]<<2|i[e.charCodeAt(n+1)]>>4,l[c++]=255&t);1===s&&(t=i[e.charCodeAt(n)]<<10|i[e.charCodeAt(n+1)]<<4|i[e.charCodeAt(n+2)]>>2,l[c++]=t>>8&255,l[c++]=255&t);return l},t.fromByteArray=function(e){for(var t,n=e.length,i=n%3,o=[],a=0,s=n-i;a<s;a+=16383)o.push(c(e,a,a+16383>s?s:a+16383));1===i?(t=e[n-1],o.push(r[t>>2]+r[t<<4&63]+"==")):2===i&&(t=(e[n-2]<<8)+e[n-1],o.push(r[t>>10]+r[t>>4&63]+r[t<<2&63]+"="));return o.join("")};for(var r=[],i=[],o="undefined"!=typeof Uint8Array?Uint8Array:Array,a="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/",s=0,l=a.length;s<l;++s)r[s]=a[s],i[a.charCodeAt(s)]=s;function u(e){var t=e.length;if(t%4>0)throw new Error("Invalid string. Length must be a multiple of 4");var n=e.indexOf("=");return-1===n&&(n=t),[n,n===t?0:4-n%4]}function c(e,t,n){for(var i,o,a=[],s=t;s<n;s+=3)i=(e[s]<<16&16711680)+(e[s+1]<<8&65280)+(255&e[s+2]),a.push(r[(o=i)>>18&63]+r[o>>12&63]+r[o>>6&63]+r[63&o]);return a.join("")}i["-".charCodeAt(0)]=62,i["_".charCodeAt(0)]=63},function(e,t,n){(function(r){var i,o,a;a="object"==typeof self&&self.self==self&&self||"object"==typeof r&&r.global==r&&r,i=[n(28),n(71),t],void 0===(o=function(e,t,n){a.Backbone=function(e,t,n,r){var i=e.Backbone,o=Array.prototype.slice;t.VERSION="1.2.3",t.$=r,t.noConflict=function(){return e.Backbone=i,this},t.emulateHTTP=!1,t.emulateJSON=!1;var a=function(e,t,r){n.each(t,(function(t,i){n[i]&&(e.prototype[i]=function(e,t,r){switch(e){case 1:return function(){return n[t](this[r])};case 2:return function(e){return n[t](this[r],e)};case 3:return function(e,i){return n[t](this[r],s(e,this),i)};case 4:return function(e,i,o){return n[t](this[r],s(e,this),i,o)};default:return function(){var e=o.call(arguments);return e.unshift(this[r]),n[t].apply(n,e)}}}(t,i,r))}))},s=function(e,t){return n.isFunction(e)?e:n.isObject(e)&&!t._isModel(e)?l(e):n.isString(e)?function(t){return t.get(e)}:e},l=function(e){var t=n.matches(e);return function(e){return t(e.attributes)}},u=t.Events={},c=/\s+/,d=function(e,t,r,i,o){var a,s=0;if(r&&"object"==typeof r){void 0!==i&&"context"in o&&void 0===o.context&&(o.context=i);for(a=n.keys(r);s<a.length;s++)t=d(e,t,a[s],r[a[s]],o)}else if(r&&c.test(r))for(a=r.split(c);s<a.length;s++)t=e(t,a[s],i,o);else t=e(t,r,i,o);return t};u.on=function(e,t,n){return h(this,e,t,n)};var h=function(e,t,n,r,i){return e._events=d(f,e._events||{},t,n,{context:r,ctx:e,listening:i}),i&&((e._listeners||(e._listeners={}))[i.id]=i),e};u.listenTo=function(e,t,r){if(!e)return this;var i=e._listenId||(e._listenId=n.uniqueId("l")),o=this._listeningTo||(this._listeningTo={}),a=o[i];if(!a){var s=this._listenId||(this._listenId=n.uniqueId("l"));a=o[i]={obj:e,objId:i,id:s,listeningTo:o,count:0}}return h(e,t,r,this,a),this};var f=function(e,t,n,r){if(n){var i=e[t]||(e[t]=[]),o=r.context,a=r.ctx,s=r.listening;s&&s.count++,i.push({callback:n,context:o,ctx:o||a,listening:s})}return e};u.off=function(e,t,n){return this._events?(this._events=d(p,this._events,e,t,{context:n,listeners:this._listeners}),this):this},u.stopListening=function(e,t,r){var i=this._listeningTo;if(!i)return this;for(var o=e?[e._listenId]:n.keys(i),a=0;a<o.length;a++){var s=i[o[a]];if(!s)break;s.obj.off(t,r,this)}return n.isEmpty(i)&&(this._listeningTo=void 0),this};var p=function(e,t,r,i){if(e){var o,a=0,s=i.context,l=i.listeners;if(t||r||s){for(var u=t?[t]:n.keys(e);a<u.length;a++){var c=e[t=u[a]];if(!c)break;for(var d=[],h=0;h<c.length;h++){var f=c[h];r&&r!==f.callback&&r!==f.callback._callback||s&&s!==f.context?d.push(f):(o=f.listening)&&0==--o.count&&(delete l[o.id],delete o.listeningTo[o.objId])}d.length?e[t]=d:delete e[t]}return n.size(e)?e:void 0}for(var p=n.keys(l);a<p.length;a++)delete l[(o=l[p[a]]).id],delete o.listeningTo[o.objId]}};u.once=function(e,t,r){var i=d(g,{},e,t,n.bind(this.off,this));return this.on(i,void 0,r)},u.listenToOnce=function(e,t,r){var i=d(g,{},t,r,n.bind(this.stopListening,this,e));return this.listenTo(e,i)};var g=function(e,t,r,i){if(r){var o=e[t]=n.once((function(){i(t,o),r.apply(this,arguments)}));o._callback=r}return e};u.trigger=function(e){if(!this._events)return this;for(var t=Math.max(0,arguments.length-1),n=Array(t),r=0;r<t;r++)n[r]=arguments[r+1];return d(v,this._events,e,void 0,n),this};var v=function(e,t,n,r){if(e){var i=e[t],o=e.all;i&&o&&(o=o.slice()),i&&m(i,r),o&&m(o,[t].concat(r))}return e},m=function(e,t){var n,r=-1,i=e.length,o=t[0],a=t[1],s=t[2];switch(t.length){case 0:for(;++r<i;)(n=e[r]).callback.call(n.ctx);return;case 1:for(;++r<i;)(n=e[r]).callback.call(n.ctx,o);return;case 2:for(;++r<i;)(n=e[r]).callback.call(n.ctx,o,a);return;case 3:for(;++r<i;)(n=e[r]).callback.call(n.ctx,o,a,s);return;default:for(;++r<i;)(n=e[r]).callback.apply(n.ctx,t);return}};u.bind=u.on,u.unbind=u.off,n.extend(t,u);var y=t.Model=function(e,t){var r=e||{};t||(t={}),this.cid=n.uniqueId(this.cidPrefix),this.attributes={},t.collection&&(this.collection=t.collection),t.parse&&(r=this.parse(r,t)||{}),r=n.defaults({},r,n.result(this,"defaults")),this.set(r,t),this.changed={},this.initialize.apply(this,arguments)};n.extend(y.prototype,u,{changed:null,validationError:null,idAttribute:"id",cidPrefix:"c",initialize:function(){},toJSON:function(e){return n.clone(this.attributes)},sync:function(){return t.sync.apply(this,arguments)},get:function(e){return this.attributes[e]},escape:function(e){return n.escape(this.get(e))},has:function(e){return null!=this.get(e)},matches:function(e){return!!n.iteratee(e,this)(this.attributes)},set:function(e,t,r){if(null==e)return this;var i;if("object"==typeof e?(i=e,r=t):(i={})[e]=t,r||(r={}),!this._validate(i,r))return!1;var o=r.unset,a=r.silent,s=[],l=this._changing;this._changing=!0,l||(this._previousAttributes=n.clone(this.attributes),this.changed={});var u=this.attributes,c=this.changed,d=this._previousAttributes;for(var h in i)t=i[h],n.isEqual(u[h],t)||s.push(h),n.isEqual(d[h],t)?delete c[h]:c[h]=t,o?delete u[h]:u[h]=t;if(this.id=this.get(this.idAttribute),!a){s.length&&(this._pending=r);for(var f=0;f<s.length;f++)this.trigger("change:"+s[f],this,u[s[f]],r)}if(l)return this;if(!a)for(;this._pending;)r=this._pending,this._pending=!1,this.trigger("change",this,r);return this._pending=!1,this._changing=!1,this},unset:function(e,t){return this.set(e,void 0,n.extend({},t,{unset:!0}))},clear:function(e){var t={};for(var r in this.attributes)t[r]=void 0;return this.set(t,n.extend({},e,{unset:!0}))},hasChanged:function(e){return null==e?!n.isEmpty(this.changed):n.has(this.changed,e)},changedAttributes:function(e){if(!e)return!!this.hasChanged()&&n.clone(this.changed);var t=this._changing?this._previousAttributes:this.attributes,r={};for(var i in e){var o=e[i];n.isEqual(t[i],o)||(r[i]=o)}return!!n.size(r)&&r},previous:function(e){return null!=e&&this._previousAttributes?this._previousAttributes[e]:null},previousAttributes:function(){return n.clone(this._previousAttributes)},fetch:function(e){e=n.extend({parse:!0},e);var t=this,r=e.success;return e.success=function(n){var i=e.parse?t.parse(n,e):n;if(!t.set(i,e))return!1;r&&r.call(e.context,t,n,e),t.trigger("sync",t,n,e)},R(this,e),this.sync("read",this,e)},save:function(e,t,r){var i;null==e||"object"==typeof e?(i=e,r=t):(i={})[e]=t;var o=(r=n.extend({validate:!0,parse:!0},r)).wait;if(i&&!o){if(!this.set(i,r))return!1}else if(!this._validate(i,r))return!1;var a=this,s=r.success,l=this.attributes;r.success=function(e){a.attributes=l;var t=r.parse?a.parse(e,r):e;if(o&&(t=n.extend({},i,t)),t&&!a.set(t,r))return!1;s&&s.call(r.context,a,e,r),a.trigger("sync",a,e,r)},R(this,r),i&&o&&(this.attributes=n.extend({},l,i));var u=this.isNew()?"create":r.patch?"patch":"update";"patch"!==u||r.attrs||(r.attrs=i);var c=this.sync(u,this,r);return this.attributes=l,c},destroy:function(e){e=e?n.clone(e):{};var t=this,r=e.success,i=e.wait,o=function(){t.stopListening(),t.trigger("destroy",t,t.collection,e)};e.success=function(n){i&&o(),r&&r.call(e.context,t,n,e),t.isNew()||t.trigger("sync",t,n,e)};var a=!1;return this.isNew()?n.defer(e.success):(R(this,e),a=this.sync("delete",this,e)),i||o(),a},url:function(){var e=n.result(this,"urlRoot")||n.result(this.collection,"url")||j();if(this.isNew())return e;var t=this.get(this.idAttribute);return e.replace(/[^\/]$/,"$&/")+encodeURIComponent(t)},parse:function(e,t){return e},clone:function(){return new this.constructor(this.attributes)},isNew:function(){return!this.has(this.idAttribute)},isValid:function(e){return this._validate({},n.defaults({validate:!0},e))},_validate:function(e,t){if(!t.validate||!this.validate)return!0;e=n.extend({},this.attributes,e);var r=this.validationError=this.validate(e,t)||null;return!r||(this.trigger("invalid",this,r,n.extend(t,{validationError:r})),!1)}}),a(y,{keys:1,values:1,pairs:1,invert:1,pick:0,omit:0,chain:1,isEmpty:1},"attributes");var b=t.Collection=function(e,t){t||(t={}),t.model&&(this.model=t.model),void 0!==t.comparator&&(this.comparator=t.comparator),this._reset(),this.initialize.apply(this,arguments),e&&this.reset(e,n.extend({silent:!0},t))},C={add:!0,remove:!0,merge:!0},A={add:!0,remove:!1},_=function(e,t,n){n=Math.min(Math.max(n,0),e.length);for(var r=Array(e.length-n),i=t.length,o=0;o<r.length;o++)r[o]=e[o+n];for(o=0;o<i;o++)e[o+n]=t[o];for(o=0;o<r.length;o++)e[o+i+n]=r[o]};n.extend(b.prototype,u,{model:y,initialize:function(){},toJSON:function(e){return this.map((function(t){return t.toJSON(e)}))},sync:function(){return t.sync.apply(this,arguments)},add:function(e,t){return this.set(e,n.extend({merge:!1},t,A))},remove:function(e,t){t=n.extend({},t);var r=!n.isArray(e);e=r?[e]:n.clone(e);var i=this._removeModels(e,t);return!t.silent&&i&&this.trigger("update",this,t),r?i[0]:i},set:function(e,t){if(null!=e){(t=n.defaults({},t,C)).parse&&!this._isModel(e)&&(e=this.parse(e,t));var r=!n.isArray(e);e=r?[e]:e.slice();var i=t.at;null!=i&&(i=+i),i<0&&(i+=this.length+1);for(var o,a=[],s=[],l=[],u={},c=t.add,d=t.merge,h=t.remove,f=!1,p=this.comparator&&null==i&&!1!==t.sort,g=n.isString(this.comparator)?this.comparator:null,v=0;v<e.length;v++){o=e[v];var m=this.get(o);if(m){if(d&&o!==m){var y=this._isModel(o)?o.attributes:o;t.parse&&(y=m.parse(y,t)),m.set(y,t),p&&!f&&(f=m.hasChanged(g))}u[m.cid]||(u[m.cid]=!0,a.push(m)),e[v]=m}else c&&(o=e[v]=this._prepareModel(o,t))&&(s.push(o),this._addReference(o,t),u[o.cid]=!0,a.push(o))}if(h){for(v=0;v<this.length;v++)u[(o=this.models[v]).cid]||l.push(o);l.length&&this._removeModels(l,t)}var b=!1,A=!p&&c&&h;if(a.length&&A?(b=this.length!=a.length||n.some(this.models,(function(e,t){return e!==a[t]})),this.models.length=0,_(this.models,a,0),this.length=this.models.length):s.length&&(p&&(f=!0),_(this.models,s,null==i?this.length:i),this.length=this.models.length),f&&this.sort({silent:!0}),!t.silent){for(v=0;v<s.length;v++)null!=i&&(t.index=i+v),(o=s[v]).trigger("add",o,this,t);(f||b)&&this.trigger("sort",this,t),(s.length||l.length)&&this.trigger("update",this,t)}return r?e[0]:e}},reset:function(e,t){t=t?n.clone(t):{};for(var r=0;r<this.models.length;r++)this._removeReference(this.models[r],t);return t.previousModels=this.models,this._reset(),e=this.add(e,n.extend({silent:!0},t)),t.silent||this.trigger("reset",this,t),e},push:function(e,t){return this.add(e,n.extend({at:this.length},t))},pop:function(e){var t=this.at(this.length-1);return this.remove(t,e)},unshift:function(e,t){return this.add(e,n.extend({at:0},t))},shift:function(e){var t=this.at(0);return this.remove(t,e)},slice:function(){return o.apply(this.models,arguments)},get:function(e){if(null!=e){var t=this.modelId(this._isModel(e)?e.attributes:e);return this._byId[e]||this._byId[t]||this._byId[e.cid]}},at:function(e){return e<0&&(e+=this.length),this.models[e]},where:function(e,t){return this[t?"find":"filter"](e)},findWhere:function(e){return this.where(e,!0)},sort:function(e){var t=this.comparator;if(!t)throw new Error("Cannot sort a set without a comparator");e||(e={});var r=t.length;return n.isFunction(t)&&(t=n.bind(t,this)),1===r||n.isString(t)?this.models=this.sortBy(t):this.models.sort(t),e.silent||this.trigger("sort",this,e),this},pluck:function(e){return n.invoke(this.models,"get",e)},fetch:function(e){var t=(e=n.extend({parse:!0},e)).success,r=this;return e.success=function(n){var i=e.reset?"reset":"set";r[i](n,e),t&&t.call(e.context,r,n,e),r.trigger("sync",r,n,e)},R(this,e),this.sync("read",this,e)},create:function(e,t){var r=(t=t?n.clone(t):{}).wait;if(!(e=this._prepareModel(e,t)))return!1;r||this.add(e,t);var i=this,o=t.success;return t.success=function(e,t,n){r&&i.add(e,n),o&&o.call(n.context,e,t,n)},e.save(null,t),e},parse:function(e,t){return e},clone:function(){return new this.constructor(this.models,{model:this.model,comparator:this.comparator})},modelId:function(e){return e[this.model.prototype.idAttribute||"id"]},_reset:function(){this.length=0,this.models=[],this._byId={}},_prepareModel:function(e,t){if(this._isModel(e))return e.collection||(e.collection=this),e;(t=t?n.clone(t):{}).collection=this;var r=new this.model(e,t);return r.validationError?(this.trigger("invalid",this,r.validationError,t),!1):r},_removeModels:function(e,t){for(var n=[],r=0;r<e.length;r++){var i=this.get(e[r]);if(i){var o=this.indexOf(i);this.models.splice(o,1),this.length--,t.silent||(t.index=o,i.trigger("remove",i,this,t)),n.push(i),this._removeReference(i,t)}}return!!n.length&&n},_isModel:function(e){return e instanceof y},_addReference:function(e,t){this._byId[e.cid]=e;var n=this.modelId(e.attributes);null!=n&&(this._byId[n]=e),e.on("all",this._onModelEvent,this)},_removeReference:function(e,t){delete this._byId[e.cid];var n=this.modelId(e.attributes);null!=n&&delete this._byId[n],this===e.collection&&delete e.collection,e.off("all",this._onModelEvent,this)},_onModelEvent:function(e,t,n,r){if("add"!==e&&"remove"!==e||n===this){if("destroy"===e&&this.remove(t,r),"change"===e){var i=this.modelId(t.previousAttributes()),o=this.modelId(t.attributes);i!==o&&(null!=i&&delete this._byId[i],null!=o&&(this._byId[o]=t))}this.trigger.apply(this,arguments)}}}),a(b,{forEach:3,each:3,map:3,collect:3,reduce:4,foldl:4,inject:4,reduceRight:4,foldr:4,find:3,detect:3,filter:3,select:3,reject:3,every:3,all:3,some:3,any:3,include:3,includes:3,contains:3,invoke:0,max:3,min:3,toArray:1,size:1,first:3,head:3,take:3,initial:3,rest:3,tail:3,drop:3,last:3,without:0,difference:0,indexOf:3,shuffle:1,lastIndexOf:3,isEmpty:1,chain:1,sample:3,partition:3,groupBy:3,countBy:3,sortBy:3,indexBy:3},"models");var I=t.View=function(e){this.cid=n.uniqueId("view"),n.extend(this,n.pick(e,w)),this._ensureElement(),this.initialize.apply(this,arguments)},x=/^(\S+)\s*(.*)$/,w=["model","collection","el","id","attributes","className","tagName","events"];n.extend(I.prototype,u,{tagName:"div",$:function(e){return this.$el.find(e)},initialize:function(){},render:function(){return this},remove:function(){return this._removeElement(),this.stopListening(),this},_removeElement:function(){this.$el.remove()},setElement:function(e){return this.undelegateEvents(),this._setElement(e),this.delegateEvents(),this},_setElement:function(e){this.$el=e instanceof t.$?e:t.$(e),this.el=this.$el[0]},delegateEvents:function(e){if(e||(e=n.result(this,"events")),!e)return this;for(var t in this.undelegateEvents(),e){var r=e[t];if(n.isFunction(r)||(r=this[r]),r){var i=t.match(x);this.delegate(i[1],i[2],n.bind(r,this))}}return this},delegate:function(e,t,n){return this.$el.on(e+".delegateEvents"+this.cid,t,n),this},undelegateEvents:function(){return this.$el&&this.$el.off(".delegateEvents"+this.cid),this},undelegate:function(e,t,n){return this.$el.off(e+".delegateEvents"+this.cid,t,n),this},_createElement:function(e){return document.createElement(e)},_ensureElement:function(){if(this.el)this.setElement(n.result(this,"el"));else{var e=n.extend({},n.result(this,"attributes"));this.id&&(e.id=n.result(this,"id")),this.className&&(e.class=n.result(this,"className")),this.setElement(this._createElement(n.result(this,"tagName"))),this._setAttributes(e)}},_setAttributes:function(e){this.$el.attr(e)}}),t.sync=function(e,r,i){var o=M[e];n.defaults(i||(i={}),{emulateHTTP:t.emulateHTTP,emulateJSON:t.emulateJSON});var a={type:o,dataType:"json"};if(i.url||(a.url=n.result(r,"url")||j()),null!=i.data||!r||"create"!==e&&"update"!==e&&"patch"!==e||(a.contentType="application/json",a.data=JSON.stringify(i.attrs||r.toJSON(i))),i.emulateJSON&&(a.contentType="application/x-www-form-urlencoded",a.data=a.data?{model:a.data}:{}),i.emulateHTTP&&("PUT"===o||"DELETE"===o||"PATCH"===o)){a.type="POST",i.emulateJSON&&(a.data._method=o);var s=i.beforeSend;i.beforeSend=function(e){if(e.setRequestHeader("X-HTTP-Method-Override",o),s)return s.apply(this,arguments)}}"GET"===a.type||i.emulateJSON||(a.processData=!1);var l=i.error;i.error=function(e,t,n){i.textStatus=t,i.errorThrown=n,l&&l.call(i.context,e,t,n)};var u=i.xhr=t.ajax(n.extend(a,i));return r.trigger("request",r,u,i),u};var M={create:"POST",update:"PUT",patch:"PATCH",delete:"DELETE",read:"GET"};t.ajax=function(){return t.$.ajax.apply(t.$,arguments)};var S=t.Router=function(e){e||(e={}),e.routes&&(this.routes=e.routes),this._bindRoutes(),this.initialize.apply(this,arguments)},T=/\((.*?)\)/g,E=/(\(\?)?:\w+/g,O=/\*\w+/g,k=/[\-{}\[\]+?.,\\\^$|#\s]/g;n.extend(S.prototype,u,{initialize:function(){},route:function(e,r,i){n.isRegExp(e)||(e=this._routeToRegExp(e)),n.isFunction(r)&&(i=r,r=""),i||(i=this[r]);var o=this;return t.history.route(e,(function(n){var a=o._extractParameters(e,n);!1!==o.execute(i,a,r)&&(o.trigger.apply(o,["route:"+r].concat(a)),o.trigger("route",r,a),t.history.trigger("route",o,r,a))})),this},execute:function(e,t,n){e&&e.apply(this,t)},navigate:function(e,n){return t.history.navigate(e,n),this},_bindRoutes:function(){if(this.routes){this.routes=n.result(this,"routes");for(var e,t=n.keys(this.routes);null!=(e=t.pop());)this.route(e,this.routes[e])}},_routeToRegExp:function(e){return e=e.replace(k,"\\$&").replace(T,"(?:$1)?").replace(E,(function(e,t){return t?e:"([^/?]+)"})).replace(O,"([^?]*?)"),new RegExp("^"+e+"(?:\\?([\\s\\S]*))?$")},_extractParameters:function(e,t){var r=e.exec(t).slice(1);return n.map(r,(function(e,t){return t===r.length-1?e||null:e?decodeURIComponent(e):null}))}});var L=t.History=function(){this.handlers=[],this.checkUrl=n.bind(this.checkUrl,this),"undefined"!=typeof window&&(this.location=window.location,this.history=window.history)},D=/^[#\/]|\s+$/g,P=/^\/+|\/+$/g,N=/#.*$/;L.started=!1,n.extend(L.prototype,u,{interval:50,atRoot:function(){return this.location.pathname.replace(/[^\/]$/,"$&/")===this.root&&!this.getSearch()},matchRoot:function(){return this.decodeFragment(this.location.pathname).slice(0,this.root.length-1)+"/"===this.root},decodeFragment:function(e){return decodeURI(e.replace(/%25/g,"%2525"))},getSearch:function(){var e=this.location.href.replace(/#.*/,"").match(/\?.+/);return e?e[0]:""},getHash:function(e){var t=(e||this).location.href.match(/#(.*)$/);return t?t[1]:""},getPath:function(){var e=this.decodeFragment(this.location.pathname+this.getSearch()).slice(this.root.length-1);return"/"===e.charAt(0)?e.slice(1):e},getFragment:function(e){return null==e&&(e=this._usePushState||!this._wantsHashChange?this.getPath():this.getHash()),e.replace(D,"")},start:function(e){if(L.started)throw new Error("Backbone.history has already been started");if(L.started=!0,this.options=n.extend({root:"/"},this.options,e),this.root=this.options.root,this._wantsHashChange=!1!==this.options.hashChange,this._hasHashChange="onhashchange"in window&&(void 0===document.documentMode||document.documentMode>7),this._useHashChange=this._wantsHashChange&&this._hasHashChange,this._wantsPushState=!!this.options.pushState,this._hasPushState=!(!this.history||!this.history.pushState),this._usePushState=this._wantsPushState&&this._hasPushState,this.fragment=this.getFragment(),this.root=("/"+this.root+"/").replace(P,"/"),this._wantsHashChange&&this._wantsPushState){if(!this._hasPushState&&!this.atRoot()){var t=this.root.slice(0,-1)||"/";return this.location.replace(t+"#"+this.getPath()),!0}this._hasPushState&&this.atRoot()&&this.navigate(this.getHash(),{replace:!0})}if(!this._hasHashChange&&this._wantsHashChange&&!this._usePushState){this.iframe=document.createElement("iframe"),this.iframe.src="javascript:0",this.iframe.style.display="none",this.iframe.tabIndex=-1;var r=document.body,i=r.insertBefore(this.iframe,r.firstChild).contentWindow;i.document.open(),i.document.close(),i.location.hash="#"+this.fragment}var o=window.addEventListener||function(e,t){return attachEvent("on"+e,t)};if(this._usePushState?o("popstate",this.checkUrl,!1):this._useHashChange&&!this.iframe?o("hashchange",this.checkUrl,!1):this._wantsHashChange&&(this._checkUrlInterval=setInterval(this.checkUrl,this.interval)),!this.options.silent)return this.loadUrl()},stop:function(){var e=window.removeEventListener||function(e,t){return detachEvent("on"+e,t)};this._usePushState?e("popstate",this.checkUrl,!1):this._useHashChange&&!this.iframe&&e("hashchange",this.checkUrl,!1),this.iframe&&(document.body.removeChild(this.iframe),this.iframe=null),this._checkUrlInterval&&clearInterval(this._checkUrlInterval),L.started=!1},route:function(e,t){this.handlers.unshift({route:e,callback:t})},checkUrl:function(e){var t=this.getFragment();if(t===this.fragment&&this.iframe&&(t=this.getHash(this.iframe.contentWindow)),t===this.fragment)return!1;this.iframe&&this.navigate(t),this.loadUrl()},loadUrl:function(e){return!!this.matchRoot()&&(e=this.fragment=this.getFragment(e),n.some(this.handlers,(function(t){if(t.route.test(e))return t.callback(e),!0})))},navigate:function(e,t){if(!L.started)return!1;t&&!0!==t||(t={trigger:!!t}),e=this.getFragment(e||"");var n=this.root;""!==e&&"?"!==e.charAt(0)||(n=n.slice(0,-1)||"/");var r=n+e;if(e=this.decodeFragment(e.replace(N,"")),this.fragment!==e){if(this.fragment=e,this._usePushState)this.history[t.replace?"replaceState":"pushState"]({},document.title,r);else{if(!this._wantsHashChange)return this.location.assign(r);if(this._updateHash(this.location,e,t.replace),this.iframe&&e!==this.getHash(this.iframe.contentWindow)){var i=this.iframe.contentWindow;t.replace||(i.document.open(),i.document.close()),this._updateHash(i.location,e,t.replace)}}return t.trigger?this.loadUrl(e):void 0}},_updateHash:function(e,t,n){if(n){var r=e.href.replace(/(javascript:|#).*$/,"");e.replace(r+"#"+t)}else e.hash="#"+t}}),t.history=new L,y.extend=b.extend=S.extend=I.extend=L.extend=function(e,t){var r,i=this;r=e&&n.has(e,"constructor")?e.constructor:function(){return i.apply(this,arguments)},n.extend(r,i,t);var o=function(){this.constructor=r};return o.prototype=i.prototype,r.prototype=new o,e&&n.extend(r.prototype,e),r.__super__=i.prototype,r};var j=function(){throw new Error('A "url" property or function must be specified')},R=function(e,t){var n=t.error;t.error=function(r){n&&n.call(t.context,e,r,t),e.trigger("error",e,r,t)}};return t}(a,n,e,t)}.apply(t,i))||(e.exports=o)}).call(this,n(61))},function(e,t,n){"use strict";n.d(t,"a",(function(){return o}));var r,i=n(16);class o{constructor(e={}){if(this._map=new Map,this._changed=new i.Signal(this),this._isDisposed=!1,this._itemCmp=e.itemCmp||r.itemCmp,e.values)for(const t in e.values)this._map.set(t,e.values[t])}get type(){return"Map"}get changed(){return this._changed}get isDisposed(){return this._isDisposed}get size(){return this._map.size}set(e,t){const n=this._map.get(e);if(void 0===t)throw Error("Cannot set an undefined value, use remove");const r=this._itemCmp;return void 0!==n&&r(n,t)||(this._map.set(e,t),this._changed.emit({type:n?"change":"add",key:e,oldValue:n,newValue:t})),n}get(e){return this._map.get(e)}has(e){return this._map.has(e)}keys(){const e=[];return this._map.forEach((t,n)=>{e.push(n)}),e}values(){const e=[];return this._map.forEach((t,n)=>{e.push(t)}),e}delete(e){const t=this._map.get(e);return this._map.delete(e)&&this._changed.emit({type:"remove",key:e,oldValue:t,newValue:void 0}),t}clear(){const e=this.keys();for(let t=0;t<e.length;t++)this.delete(e[t])}dispose(){this.isDisposed||(this._isDisposed=!0,i.Signal.clearData(this),this._map.clear())}}!function(e){e.itemCmp=function(e,t){return e===t}}(r||(r={}))},function(e,t,n){e.exports=function(){"use strict";function e(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}function t(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,r=new Array(t);n<t;n++)r[n]=e[n];return r}function n(e,n){var r;if("undefined"==typeof Symbol||null==e[Symbol.iterator]){if(Array.isArray(e)||(r=function(e,n){if(e){if("string"==typeof e)return t(e,n);var r=Object.prototype.toString.call(e).slice(8,-1);return"Object"===r&&e.constructor&&(r=e.constructor.name),"Map"===r||"Set"===r?Array.from(e):"Arguments"===r||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(r)?t(e,n):void 0}}(e))||n&&e&&"number"==typeof e.length){r&&(e=r);var i=0;return function(){return i>=e.length?{done:!0}:{done:!1,value:e[i++]}}}throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}return(r=e[Symbol.iterator]()).next.bind(r)}var r=function(e){var t={exports:{}};return e(t,t.exports),t.exports}((function(e){function t(){return{baseUrl:null,breaks:!1,gfm:!0,headerIds:!0,headerPrefix:"",highlight:null,langPrefix:"language-",mangle:!0,pedantic:!1,renderer:null,sanitize:!1,sanitizer:null,silent:!1,smartLists:!1,smartypants:!1,tokenizer:null,walkTokens:null,xhtml:!1}}e.exports={defaults:{baseUrl:null,breaks:!1,gfm:!0,headerIds:!0,headerPrefix:"",highlight:null,langPrefix:"language-",mangle:!0,pedantic:!1,renderer:null,sanitize:!1,sanitizer:null,silent:!1,smartLists:!1,smartypants:!1,tokenizer:null,walkTokens:null,xhtml:!1},getDefaults:t,changeDefaults:function(t){e.exports.defaults=t}}})),i=/[&<>"']/,o=/[&<>"']/g,a=/[<>"']|&(?!#?\w+;)/,s=/[<>"']|&(?!#?\w+;)/g,l={"&":"&amp;","<":"&lt;",">":"&gt;",'"':"&quot;","'":"&#39;"},u=function(e){return l[e]},c=/&(#(?:\d+)|(?:#x[0-9A-Fa-f]+)|(?:\w+));?/gi;function d(e){return e.replace(c,(function(e,t){return"colon"===(t=t.toLowerCase())?":":"#"===t.charAt(0)?"x"===t.charAt(1)?String.fromCharCode(parseInt(t.substring(2),16)):String.fromCharCode(+t.substring(1)):""}))}var h=/(^|[^\[])\^/g,f=/[^\w:]/g,p=/^$|^[a-z][a-z0-9+.-]*:|^[?#]/i,g={},v=/^[^:]+:\/*[^/]*$/,m=/^([^:]+:)[\s\S]*$/,y=/^([^:]+:\/*[^/]*)[\s\S]*$/;function b(e,t){g[" "+e]||(v.test(e)?g[" "+e]=e+"/":g[" "+e]=C(e,"/",!0));var n=-1===(e=g[" "+e]).indexOf(":");return"//"===t.substring(0,2)?n?t:e.replace(m,"$1")+t:"/"===t.charAt(0)?n?t:e.replace(y,"$1")+t:e+t}function C(e,t,n){var r=e.length;if(0===r)return"";for(var i=0;i<r;){var o=e.charAt(r-i-1);if(o!==t||n){if(o===t||!n)break;i++}else i++}return e.substr(0,r-i)}var A=function(e,t){if(t){if(i.test(e))return e.replace(o,u)}else if(a.test(e))return e.replace(s,u);return e},_=d,I=function(e,t){e=e.source||e,t=t||"";var n={replace:function(t,r){return r=(r=r.source||r).replace(h,"$1"),e=e.replace(t,r),n},getRegex:function(){return new RegExp(e,t)}};return n},x=function(e,t,n){if(e){var r;try{r=decodeURIComponent(d(n)).replace(f,"").toLowerCase()}catch(e){return null}if(0===r.indexOf("javascript:")||0===r.indexOf("vbscript:")||0===r.indexOf("data:"))return null}t&&!p.test(n)&&(n=b(t,n));try{n=encodeURI(n).replace(/%25/g,"%")}catch(e){return null}return n},w={exec:function(){}},M=function(e){for(var t,n,r=1;r<arguments.length;r++)for(n in t=arguments[r])Object.prototype.hasOwnProperty.call(t,n)&&(e[n]=t[n]);return e},S=function(e,t){var n=e.replace(/\|/g,(function(e,t,n){for(var r=!1,i=t;--i>=0&&"\\"===n[i];)r=!r;return r?"|":" |"})).split(/ \|/),r=0;if(n.length>t)n.splice(t);else for(;n.length<t;)n.push("");for(;r<n.length;r++)n[r]=n[r].trim().replace(/\\\|/g,"|");return n},T=C,E=function(e,t){if(-1===e.indexOf(t[1]))return-1;for(var n=e.length,r=0,i=0;i<n;i++)if("\\"===e[i])i++;else if(e[i]===t[0])r++;else if(e[i]===t[1]&&--r<0)return i;return-1},O=function(e){e&&e.sanitize&&!e.silent&&console.warn("marked(): sanitize and sanitizer parameters are deprecated since version 0.7.0, should not be used and will be removed in the future. Read more here: https://marked.js.org/#/USING_ADVANCED.md#options")},k=function(e,t){if(t<1)return"";for(var n="";t>1;)1&t&&(n+=e),t>>=1,e+=e;return n+e},L=r.defaults,D=T,P=S,N=A,j=E;function R(e,t,n){var r=t.href,i=t.title?N(t.title):null,o=e[1].replace(/\\([\[\]])/g,"$1");return"!"!==e[0].charAt(0)?{type:"link",raw:n,href:r,title:i,text:o}:{type:"image",raw:n,href:r,title:i,text:N(o)}}var z=function(){function e(e){this.options=e||L}var t=e.prototype;return t.space=function(e){var t=this.rules.block.newline.exec(e);if(t)return t[0].length>1?{type:"space",raw:t[0]}:{raw:"\n"}},t.code=function(e,t){var n=this.rules.block.code.exec(e);if(n){var r=t[t.length-1];if(r&&"paragraph"===r.type)return{raw:n[0],text:n[0].trimRight()};var i=n[0].replace(/^ {1,4}/gm,"");return{type:"code",raw:n[0],codeBlockStyle:"indented",text:this.options.pedantic?i:D(i,"\n")}}},t.fences=function(e){var t=this.rules.block.fences.exec(e);if(t){var n=t[0],r=function(e,t){var n=e.match(/^(\s+)(?:```)/);if(null===n)return t;var r=n[1];return t.split("\n").map((function(e){var t=e.match(/^\s+/);return null===t?e:t[0].length>=r.length?e.slice(r.length):e})).join("\n")}(n,t[3]||"");return{type:"code",raw:n,lang:t[2]?t[2].trim():t[2],text:r}}},t.heading=function(e){var t=this.rules.block.heading.exec(e);if(t){var n=t[2].trim();if(/#$/.test(n)){var r=D(n,"#");this.options.pedantic?n=r.trim():r&&!/ $/.test(r)||(n=r.trim())}return{type:"heading",raw:t[0],depth:t[1].length,text:n}}},t.nptable=function(e){var t=this.rules.block.nptable.exec(e);if(t){var n={type:"table",header:P(t[1].replace(/^ *| *\| *$/g,"")),align:t[2].replace(/^ *|\| *$/g,"").split(/ *\| */),cells:t[3]?t[3].replace(/\n$/,"").split("\n"):[],raw:t[0]};if(n.header.length===n.align.length){var r,i=n.align.length;for(r=0;r<i;r++)/^ *-+: *$/.test(n.align[r])?n.align[r]="right":/^ *:-+: *$/.test(n.align[r])?n.align[r]="center":/^ *:-+ *$/.test(n.align[r])?n.align[r]="left":n.align[r]=null;for(i=n.cells.length,r=0;r<i;r++)n.cells[r]=P(n.cells[r],n.header.length);return n}}},t.hr=function(e){var t=this.rules.block.hr.exec(e);if(t)return{type:"hr",raw:t[0]}},t.blockquote=function(e){var t=this.rules.block.blockquote.exec(e);if(t){var n=t[0].replace(/^ *> ?/gm,"");return{type:"blockquote",raw:t[0],text:n}}},t.list=function(e){var t=this.rules.block.list.exec(e);if(t){var n,r,i,o,a,s,l,u,c=t[0],d=t[2],h=d.length>1,f={type:"list",raw:c,ordered:h,start:h?+d.slice(0,-1):"",loose:!1,items:[]},p=t[0].match(this.rules.block.item),g=!1,v=p.length;i=this.rules.block.listItemStart.exec(p[0]);for(var m=0;m<v;m++){if(c=n=p[m],m!==v-1){if(o=this.rules.block.listItemStart.exec(p[m+1]),this.options.pedantic?o[1].length>i[1].length:o[1].length>i[0].length||o[1].length>3){p.splice(m,2,p[m]+"\n"+p[m+1]),m--,v--;continue}(!this.options.pedantic||this.options.smartLists?o[2][o[2].length-1]!==d[d.length-1]:h===(1===o[2].length))&&(a=p.slice(m+1).join("\n"),f.raw=f.raw.substring(0,f.raw.length-a.length),m=v-1),i=o}r=n.length,~(n=n.replace(/^ *([*+-]|\d+[.)]) ?/,"")).indexOf("\n ")&&(r-=n.length,n=this.options.pedantic?n.replace(/^ {1,4}/gm,""):n.replace(new RegExp("^ {1,"+r+"}","gm"),"")),s=g||/\n\n(?!\s*$)/.test(n),m!==v-1&&(g="\n"===n.charAt(n.length-1),s||(s=g)),s&&(f.loose=!0),this.options.gfm&&(u=void 0,(l=/^\[[ xX]\] /.test(n))&&(u=" "!==n[1],n=n.replace(/^\[[ xX]\] +/,""))),f.items.push({type:"list_item",raw:c,task:l,checked:u,loose:s,text:n})}return f}},t.html=function(e){var t=this.rules.block.html.exec(e);if(t)return{type:this.options.sanitize?"paragraph":"html",raw:t[0],pre:!this.options.sanitizer&&("pre"===t[1]||"script"===t[1]||"style"===t[1]),text:this.options.sanitize?this.options.sanitizer?this.options.sanitizer(t[0]):N(t[0]):t[0]}},t.def=function(e){var t=this.rules.block.def.exec(e);if(t)return t[3]&&(t[3]=t[3].substring(1,t[3].length-1)),{tag:t[1].toLowerCase().replace(/\s+/g," "),raw:t[0],href:t[2],title:t[3]}},t.table=function(e){var t=this.rules.block.table.exec(e);if(t){var n={type:"table",header:P(t[1].replace(/^ *| *\| *$/g,"")),align:t[2].replace(/^ *|\| *$/g,"").split(/ *\| */),cells:t[3]?t[3].replace(/\n$/,"").split("\n"):[]};if(n.header.length===n.align.length){n.raw=t[0];var r,i=n.align.length;for(r=0;r<i;r++)/^ *-+: *$/.test(n.align[r])?n.align[r]="right":/^ *:-+: *$/.test(n.align[r])?n.align[r]="center":/^ *:-+ *$/.test(n.align[r])?n.align[r]="left":n.align[r]=null;for(i=n.cells.length,r=0;r<i;r++)n.cells[r]=P(n.cells[r].replace(/^ *\| *| *\| *$/g,""),n.header.length);return n}}},t.lheading=function(e){var t=this.rules.block.lheading.exec(e);if(t)return{type:"heading",raw:t[0],depth:"="===t[2].charAt(0)?1:2,text:t[1]}},t.paragraph=function(e){var t=this.rules.block.paragraph.exec(e);if(t)return{type:"paragraph",raw:t[0],text:"\n"===t[1].charAt(t[1].length-1)?t[1].slice(0,-1):t[1]}},t.text=function(e,t){var n=this.rules.block.text.exec(e);if(n){var r=t[t.length-1];return r&&"text"===r.type?{raw:n[0],text:n[0]}:{type:"text",raw:n[0],text:n[0]}}},t.escape=function(e){var t=this.rules.inline.escape.exec(e);if(t)return{type:"escape",raw:t[0],text:N(t[1])}},t.tag=function(e,t,n){var r=this.rules.inline.tag.exec(e);if(r)return!t&&/^<a /i.test(r[0])?t=!0:t&&/^<\/a>/i.test(r[0])&&(t=!1),!n&&/^<(pre|code|kbd|script)(\s|>)/i.test(r[0])?n=!0:n&&/^<\/(pre|code|kbd|script)(\s|>)/i.test(r[0])&&(n=!1),{type:this.options.sanitize?"text":"html",raw:r[0],inLink:t,inRawBlock:n,text:this.options.sanitize?this.options.sanitizer?this.options.sanitizer(r[0]):N(r[0]):r[0]}},t.link=function(e){var t=this.rules.inline.link.exec(e);if(t){var n=t[2].trim();if(!this.options.pedantic&&/^</.test(n)){if(!/>$/.test(n))return;var r=D(n.slice(0,-1),"\\");if((n.length-r.length)%2==0)return}else{var i=j(t[2],"()");if(i>-1){var o=(0===t[0].indexOf("!")?5:4)+t[1].length+i;t[2]=t[2].substring(0,i),t[0]=t[0].substring(0,o).trim(),t[3]=""}}var a=t[2],s="";if(this.options.pedantic){var l=/^([^'"]*[^\s])\s+(['"])(.*)\2/.exec(a);l&&(a=l[1],s=l[3])}else s=t[3]?t[3].slice(1,-1):"";return a=a.trim(),/^</.test(a)&&(a=this.options.pedantic&&!/>$/.test(n)?a.slice(1):a.slice(1,-1)),R(t,{href:a?a.replace(this.rules.inline._escapes,"$1"):a,title:s?s.replace(this.rules.inline._escapes,"$1"):s},t[0])}},t.reflink=function(e,t){var n;if((n=this.rules.inline.reflink.exec(e))||(n=this.rules.inline.nolink.exec(e))){var r=(n[2]||n[1]).replace(/\s+/g," ");if(!(r=t[r.toLowerCase()])||!r.href){var i=n[0].charAt(0);return{type:"text",raw:i,text:i}}return R(n,r,n[0])}},t.strong=function(e,t,n){void 0===n&&(n="");var r=this.rules.inline.strong.start.exec(e);if(r&&(!r[1]||r[1]&&(""===n||this.rules.inline.punctuation.exec(n)))){t=t.slice(-1*e.length);var i,o="**"===r[0]?this.rules.inline.strong.endAst:this.rules.inline.strong.endUnd;for(o.lastIndex=0;null!=(r=o.exec(t));)if(i=this.rules.inline.strong.middle.exec(t.slice(0,r.index+3)))return{type:"strong",raw:e.slice(0,i[0].length),text:e.slice(2,i[0].length-2)}}},t.em=function(e,t,n){void 0===n&&(n="");var r=this.rules.inline.em.start.exec(e);if(r&&(!r[1]||r[1]&&(""===n||this.rules.inline.punctuation.exec(n)))){t=t.slice(-1*e.length);var i,o="*"===r[0]?this.rules.inline.em.endAst:this.rules.inline.em.endUnd;for(o.lastIndex=0;null!=(r=o.exec(t));)if(i=this.rules.inline.em.middle.exec(t.slice(0,r.index+2)))return{type:"em",raw:e.slice(0,i[0].length),text:e.slice(1,i[0].length-1)}}},t.codespan=function(e){var t=this.rules.inline.code.exec(e);if(t){var n=t[2].replace(/\n/g," "),r=/[^ ]/.test(n),i=/^ /.test(n)&&/ $/.test(n);return r&&i&&(n=n.substring(1,n.length-1)),n=N(n,!0),{type:"codespan",raw:t[0],text:n}}},t.br=function(e){var t=this.rules.inline.br.exec(e);if(t)return{type:"br",raw:t[0]}},t.del=function(e){var t=this.rules.inline.del.exec(e);if(t)return{type:"del",raw:t[0],text:t[2]}},t.autolink=function(e,t){var n,r,i=this.rules.inline.autolink.exec(e);if(i)return r="@"===i[2]?"mailto:"+(n=N(this.options.mangle?t(i[1]):i[1])):n=N(i[1]),{type:"link",raw:i[0],text:n,href:r,tokens:[{type:"text",raw:n,text:n}]}},t.url=function(e,t){var n;if(n=this.rules.inline.url.exec(e)){var r,i;if("@"===n[2])i="mailto:"+(r=N(this.options.mangle?t(n[0]):n[0]));else{var o;do{o=n[0],n[0]=this.rules.inline._backpedal.exec(n[0])[0]}while(o!==n[0]);r=N(n[0]),i="www."===n[1]?"http://"+r:r}return{type:"link",raw:n[0],text:r,href:i,tokens:[{type:"text",raw:r,text:r}]}}},t.inlineText=function(e,t,n){var r,i=this.rules.inline.text.exec(e);if(i)return r=t?this.options.sanitize?this.options.sanitizer?this.options.sanitizer(i[0]):N(i[0]):i[0]:N(this.options.smartypants?n(i[0]):i[0]),{type:"text",raw:i[0],text:r}},e}(),F=w,B=I,H=M,U={newline:/^(?: *(?:\n|$))+/,code:/^( {4}[^\n]+(?:\n(?: *(?:\n|$))*)?)+/,fences:/^ {0,3}(`{3,}(?=[^`\n]*\n)|~{3,})([^\n]*)\n(?:|([\s\S]*?)\n)(?: {0,3}\1[~`]* *(?:\n+|$)|$)/,hr:/^ {0,3}((?:- *){3,}|(?:_ *){3,}|(?:\* *){3,})(?:\n+|$)/,heading:/^ {0,3}(#{1,6})(?=\s|$)(.*)(?:\n+|$)/,blockquote:/^( {0,3}> ?(paragraph|[^\n]*)(?:\n|$))+/,list:/^( {0,3})(bull) [\s\S]+?(?:hr|def|\n{2,}(?! )(?! {0,3}bull )\n*|\s*$)/,html:"^ {0,3}(?:<(script|pre|style)[\\s>][\\s\\S]*?(?:</\\1>[^\\n]*\\n+|$)|comment[^\\n]*(\\n+|$)|<\\?[\\s\\S]*?(?:\\?>\\n*|$)|<![A-Z][\\s\\S]*?(?:>\\n*|$)|<!\\[CDATA\\[[\\s\\S]*?(?:\\]\\]>\\n*|$)|</?(tag)(?: +|\\n|/?>)[\\s\\S]*?(?:\\n{2,}|$)|<(?!script|pre|style)([a-z][\\w-]*)(?:attribute)*? */?>(?=[ \\t]*(?:\\n|$))[\\s\\S]*?(?:\\n{2,}|$)|</(?!script|pre|style)[a-z][\\w-]*\\s*>(?=[ \\t]*(?:\\n|$))[\\s\\S]*?(?:\\n{2,}|$))",def:/^ {0,3}\[(label)\]: *\n? *<?([^\s>]+)>?(?:(?: +\n? *| *\n *)(title))? *(?:\n+|$)/,nptable:F,table:F,lheading:/^([^\n]+)\n {0,3}(=+|-+) *(?:\n+|$)/,_paragraph:/^([^\n]+(?:\n(?!hr|heading|lheading|blockquote|fences|list|html| +\n)[^\n]+)*)/,text:/^[^\n]+/,_label:/(?!\s*\])(?:\\[\[\]]|[^\[\]])+/,_title:/(?:"(?:\\"?|[^"\\])*"|'[^'\n]*(?:\n[^'\n]+)*\n?'|\([^()]*\))/};U.def=B(U.def).replace("label",U._label).replace("title",U._title).getRegex(),U.bullet=/(?:[*+-]|\d{1,9}[.)])/,U.item=/^( *)(bull) ?[^\n]*(?:\n(?! *bull ?)[^\n]*)*/,U.item=B(U.item,"gm").replace(/bull/g,U.bullet).getRegex(),U.listItemStart=B(/^( *)(bull)/).replace("bull",U.bullet).getRegex(),U.list=B(U.list).replace(/bull/g,U.bullet).replace("hr","\\n+(?=\\1?(?:(?:- *){3,}|(?:_ *){3,}|(?:\\* *){3,})(?:\\n+|$))").replace("def","\\n+(?="+U.def.source+")").getRegex(),U._tag="address|article|aside|base|basefont|blockquote|body|caption|center|col|colgroup|dd|details|dialog|dir|div|dl|dt|fieldset|figcaption|figure|footer|form|frame|frameset|h[1-6]|head|header|hr|html|iframe|legend|li|link|main|menu|menuitem|meta|nav|noframes|ol|optgroup|option|p|param|section|source|summary|table|tbody|td|tfoot|th|thead|title|tr|track|ul",U._comment=/<!--(?!-?>)[\s\S]*?(?:-->|$)/,U.html=B(U.html,"i").replace("comment",U._comment).replace("tag",U._tag).replace("attribute",/ +[a-zA-Z:_][\w.:-]*(?: *= *"[^"\n]*"| *= *'[^'\n]*'| *= *[^\s"'=<>`]+)?/).getRegex(),U.paragraph=B(U._paragraph).replace("hr",U.hr).replace("heading"," {0,3}#{1,6} ").replace("|lheading","").replace("blockquote"," {0,3}>").replace("fences"," {0,3}(?:`{3,}(?=[^`\\n]*\\n)|~{3,})[^\\n]*\\n").replace("list"," {0,3}(?:[*+-]|1[.)]) ").replace("html","</?(?:tag)(?: +|\\n|/?>)|<(?:script|pre|style|!--)").replace("tag",U._tag).getRegex(),U.blockquote=B(U.blockquote).replace("paragraph",U.paragraph).getRegex(),U.normal=H({},U),U.gfm=H({},U.normal,{nptable:"^ *([^|\\n ].*\\|.*)\\n {0,3}([-:]+ *\\|[-| :]*)(?:\\n((?:(?!\\n|hr|heading|blockquote|code|fences|list|html).*(?:\\n|$))*)\\n*|$)",table:"^ *\\|(.+)\\n {0,3}\\|?( *[-:]+[-| :]*)(?:\\n *((?:(?!\\n|hr|heading|blockquote|code|fences|list|html).*(?:\\n|$))*)\\n*|$)"}),U.gfm.nptable=B(U.gfm.nptable).replace("hr",U.hr).replace("heading"," {0,3}#{1,6} ").replace("blockquote"," {0,3}>").replace("code"," {4}[^\\n]").replace("fences"," {0,3}(?:`{3,}(?=[^`\\n]*\\n)|~{3,})[^\\n]*\\n").replace("list"," {0,3}(?:[*+-]|1[.)]) ").replace("html","</?(?:tag)(?: +|\\n|/?>)|<(?:script|pre|style|!--)").replace("tag",U._tag).getRegex(),U.gfm.table=B(U.gfm.table).replace("hr",U.hr).replace("heading"," {0,3}#{1,6} ").replace("blockquote"," {0,3}>").replace("code"," {4}[^\\n]").replace("fences"," {0,3}(?:`{3,}(?=[^`\\n]*\\n)|~{3,})[^\\n]*\\n").replace("list"," {0,3}(?:[*+-]|1[.)]) ").replace("html","</?(?:tag)(?: +|\\n|/?>)|<(?:script|pre|style|!--)").replace("tag",U._tag).getRegex(),U.pedantic=H({},U.normal,{html:B("^ *(?:comment *(?:\\n|\\s*$)|<(tag)[\\s\\S]+?</\\1> *(?:\\n{2,}|\\s*$)|<tag(?:\"[^\"]*\"|'[^']*'|\\s[^'\"/>\\s]*)*?/?> *(?:\\n{2,}|\\s*$))").replace("comment",U._comment).replace(/tag/g,"(?!(?:a|em|strong|small|s|cite|q|dfn|abbr|data|time|code|var|samp|kbd|sub|sup|i|b|u|mark|ruby|rt|rp|bdi|bdo|span|br|wbr|ins|del|img)\\b)\\w+(?!:|[^\\w\\s@]*@)\\b").getRegex(),def:/^ *\[([^\]]+)\]: *<?([^\s>]+)>?(?: +(["(][^\n]+[")]))? *(?:\n+|$)/,heading:/^(#{1,6})(.*)(?:\n+|$)/,fences:F,paragraph:B(U.normal._paragraph).replace("hr",U.hr).replace("heading"," *#{1,6} *[^\n]").replace("lheading",U.lheading).replace("blockquote"," {0,3}>").replace("|fences","").replace("|list","").replace("|html","").getRegex()});var V={escape:/^\\([!"#$%&'()*+,\-./:;<=>?@\[\]\\^_`{|}~])/,autolink:/^<(scheme:[^\s\x00-\x1f<>]*|email)>/,url:F,tag:"^comment|^</[a-zA-Z][\\w:-]*\\s*>|^<[a-zA-Z][\\w-]*(?:attribute)*?\\s*/?>|^<\\?[\\s\\S]*?\\?>|^<![a-zA-Z]+\\s[\\s\\S]*?>|^<!\\[CDATA\\[[\\s\\S]*?\\]\\]>",link:/^!?\[(label)\]\(\s*(href)(?:\s+(title))?\s*\)/,reflink:/^!?\[(label)\]\[(?!\s*\])((?:\\[\[\]]?|[^\[\]\\])+)\]/,nolink:/^!?\[(?!\s*\])((?:\[[^\[\]]*\]|\\[\[\]]|[^\[\]])*)\](?:\[\])?/,reflinkSearch:"reflink|nolink(?!\\()",strong:{start:/^(?:(\*\*(?=[*punctuation]))|\*\*)(?![\s])|__/,middle:/^\*\*(?:(?:(?!overlapSkip)(?:[^*]|\\\*)|overlapSkip)|\*(?:(?!overlapSkip)(?:[^*]|\\\*)|overlapSkip)*?\*)+?\*\*$|^__(?![\s])((?:(?:(?!overlapSkip)(?:[^_]|\\_)|overlapSkip)|_(?:(?!overlapSkip)(?:[^_]|\\_)|overlapSkip)*?_)+?)__$/,endAst:/[^punctuation\s]\*\*(?!\*)|[punctuation]\*\*(?!\*)(?:(?=[punctuation_\s]|$))/,endUnd:/[^\s]__(?!_)(?:(?=[punctuation*\s])|$)/},em:{start:/^(?:(\*(?=[punctuation]))|\*)(?![*\s])|_/,middle:/^\*(?:(?:(?!overlapSkip)(?:[^*]|\\\*)|overlapSkip)|\*(?:(?!overlapSkip)(?:[^*]|\\\*)|overlapSkip)*?\*)+?\*$|^_(?![_\s])(?:(?:(?!overlapSkip)(?:[^_]|\\_)|overlapSkip)|_(?:(?!overlapSkip)(?:[^_]|\\_)|overlapSkip)*?_)+?_$/,endAst:/[^punctuation\s]\*(?!\*)|[punctuation]\*(?!\*)(?:(?=[punctuation_\s]|$))/,endUnd:/[^\s]_(?!_)(?:(?=[punctuation*\s])|$)/},code:/^(`+)([^`]|[^`][\s\S]*?[^`])\1(?!`)/,br:/^( {2,}|\\)\n(?!\s*$)/,del:F,text:/^(`+|[^`])(?:(?= {2,}\n)|[\s\S]*?(?:(?=[\\<!\[`*]|\b_|$)|[^ ](?= {2,}\n)))/,punctuation:/^([\s*punctuation])/,_punctuation:"!\"#$%&'()+\\-.,/:;<=>?@\\[\\]`^{|}~"};V.punctuation=B(V.punctuation).replace(/punctuation/g,V._punctuation).getRegex(),V._blockSkip="\\[[^\\]]*?\\]\\([^\\)]*?\\)|`[^`]*?`|<[^>]*?>",V._overlapSkip="__[^_]*?__|\\*\\*\\[^\\*\\]*?\\*\\*",V._comment=B(U._comment).replace("(?:--\x3e|$)","--\x3e").getRegex(),V.em.start=B(V.em.start).replace(/punctuation/g,V._punctuation).getRegex(),V.em.middle=B(V.em.middle).replace(/punctuation/g,V._punctuation).replace(/overlapSkip/g,V._overlapSkip).getRegex(),V.em.endAst=B(V.em.endAst,"g").replace(/punctuation/g,V._punctuation).getRegex(),V.em.endUnd=B(V.em.endUnd,"g").replace(/punctuation/g,V._punctuation).getRegex(),V.strong.start=B(V.strong.start).replace(/punctuation/g,V._punctuation).getRegex(),V.strong.middle=B(V.strong.middle).replace(/punctuation/g,V._punctuation).replace(/overlapSkip/g,V._overlapSkip).getRegex(),V.strong.endAst=B(V.strong.endAst,"g").replace(/punctuation/g,V._punctuation).getRegex(),V.strong.endUnd=B(V.strong.endUnd,"g").replace(/punctuation/g,V._punctuation).getRegex(),V.blockSkip=B(V._blockSkip,"g").getRegex(),V.overlapSkip=B(V._overlapSkip,"g").getRegex(),V._escapes=/\\([!"#$%&'()*+,\-./:;<=>?@\[\]\\^_`{|}~])/g,V._scheme=/[a-zA-Z][a-zA-Z0-9+.-]{1,31}/,V._email=/[a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+(@)[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?)+(?![-_])/,V.autolink=B(V.autolink).replace("scheme",V._scheme).replace("email",V._email).getRegex(),V._attribute=/\s+[a-zA-Z:_][\w.:-]*(?:\s*=\s*"[^"]*"|\s*=\s*'[^']*'|\s*=\s*[^\s"'=<>`]+)?/,V.tag=B(V.tag).replace("comment",V._comment).replace("attribute",V._attribute).getRegex(),V._label=/(?:\[(?:\\.|[^\[\]\\])*\]|\\.|`[^`]*`|[^\[\]\\`])*?/,V._href=/<(?:\\.|[^\n<>\\])+>|[^\s\x00-\x1f]*/,V._title=/"(?:\\"?|[^"\\])*"|'(?:\\'?|[^'\\])*'|\((?:\\\)?|[^)\\])*\)/,V.link=B(V.link).replace("label",V._label).replace("href",V._href).replace("title",V._title).getRegex(),V.reflink=B(V.reflink).replace("label",V._label).getRegex(),V.reflinkSearch=B(V.reflinkSearch,"g").replace("reflink",V.reflink).replace("nolink",V.nolink).getRegex(),V.normal=H({},V),V.pedantic=H({},V.normal,{strong:{start:/^__|\*\*/,middle:/^__(?=\S)([\s\S]*?\S)__(?!_)|^\*\*(?=\S)([\s\S]*?\S)\*\*(?!\*)/,endAst:/\*\*(?!\*)/g,endUnd:/__(?!_)/g},em:{start:/^_|\*/,middle:/^()\*(?=\S)([\s\S]*?\S)\*(?!\*)|^_(?=\S)([\s\S]*?\S)_(?!_)/,endAst:/\*(?!\*)/g,endUnd:/_(?!_)/g},link:B(/^!?\[(label)\]\((.*?)\)/).replace("label",V._label).getRegex(),reflink:B(/^!?\[(label)\]\s*\[([^\]]*)\]/).replace("label",V._label).getRegex()}),V.gfm=H({},V.normal,{escape:B(V.escape).replace("])","~|])").getRegex(),_extended_email:/[A-Za-z0-9._+-]+(@)[a-zA-Z0-9-_]+(?:\.[a-zA-Z0-9-_]*[a-zA-Z0-9])+(?![-_])/,url:/^((?:ftp|https?):\/\/|www\.)(?:[a-zA-Z0-9\-]+\.?)+[^\s<]*|^email/,_backpedal:/(?:[^?!.,:;*_~()&]+|\([^)]*\)|&(?![a-zA-Z0-9]+;$)|[?!.,:;*_~)]+(?!$))+/,del:/^(~~?)(?=[^\s~])([\s\S]*?[^\s~])\1(?=[^~]|$)/,text:/^([`~]+|[^`~])(?:(?= {2,}\n)|[\s\S]*?(?:(?=[\\<!\[`*~]|\b_|https?:\/\/|ftp:\/\/|www\.|$)|[^ ](?= {2,}\n)|[^a-zA-Z0-9.!#$%&'*+\/=?_`{\|}~-](?=[a-zA-Z0-9.!#$%&'*+\/=?_`{\|}~-]+@))|(?=[a-zA-Z0-9.!#$%&'*+\/=?_`{\|}~-]+@))/}),V.gfm.url=B(V.gfm.url,"i").replace("email",V.gfm._extended_email).getRegex(),V.breaks=H({},V.gfm,{br:B(V.br).replace("{2,}","*").getRegex(),text:B(V.gfm.text).replace("\\b_","\\b_| {2,}\\n").replace(/\{2,\}/g,"*").getRegex()});var Y={block:U,inline:V},W=r.defaults,G=Y.block,X=Y.inline,q=k;function Z(e){return e.replace(/---/g,"—").replace(/--/g,"–").replace(/(^|[-\u2014/(\[{"\s])'/g,"$1‘").replace(/'/g,"’").replace(/(^|[-\u2014/(\[{\u2018\s])"/g,"$1“").replace(/"/g,"”").replace(/\.{3}/g,"…")}function Q(e){var t,n,r="",i=e.length;for(t=0;t<i;t++)n=e.charCodeAt(t),Math.random()>.5&&(n="x"+n.toString(16)),r+="&#"+n+";";return r}var K=function(){function t(e){this.tokens=[],this.tokens.links=Object.create(null),this.options=e||W,this.options.tokenizer=this.options.tokenizer||new z,this.tokenizer=this.options.tokenizer,this.tokenizer.options=this.options;var t={block:G.normal,inline:X.normal};this.options.pedantic?(t.block=G.pedantic,t.inline=X.pedantic):this.options.gfm&&(t.block=G.gfm,this.options.breaks?t.inline=X.breaks:t.inline=X.gfm),this.tokenizer.rules=t}t.lex=function(e,n){return new t(n).lex(e)},t.lexInline=function(e,n){return new t(n).inlineTokens(e)};var n,r,i,o=t.prototype;return o.lex=function(e){return e=e.replace(/\r\n|\r/g,"\n").replace(/\t/g,"    "),this.blockTokens(e,this.tokens,!0),this.inline(this.tokens),this.tokens},o.blockTokens=function(e,t,n){var r,i,o,a;for(void 0===t&&(t=[]),void 0===n&&(n=!0),this.options.pedantic&&(e=e.replace(/^ +$/gm,""));e;)if(r=this.tokenizer.space(e))e=e.substring(r.raw.length),r.type&&t.push(r);else if(r=this.tokenizer.code(e,t))e=e.substring(r.raw.length),r.type?t.push(r):((a=t[t.length-1]).raw+="\n"+r.raw,a.text+="\n"+r.text);else if(r=this.tokenizer.fences(e))e=e.substring(r.raw.length),t.push(r);else if(r=this.tokenizer.heading(e))e=e.substring(r.raw.length),t.push(r);else if(r=this.tokenizer.nptable(e))e=e.substring(r.raw.length),t.push(r);else if(r=this.tokenizer.hr(e))e=e.substring(r.raw.length),t.push(r);else if(r=this.tokenizer.blockquote(e))e=e.substring(r.raw.length),r.tokens=this.blockTokens(r.text,[],n),t.push(r);else if(r=this.tokenizer.list(e)){for(e=e.substring(r.raw.length),o=r.items.length,i=0;i<o;i++)r.items[i].tokens=this.blockTokens(r.items[i].text,[],!1);t.push(r)}else if(r=this.tokenizer.html(e))e=e.substring(r.raw.length),t.push(r);else if(n&&(r=this.tokenizer.def(e)))e=e.substring(r.raw.length),this.tokens.links[r.tag]||(this.tokens.links[r.tag]={href:r.href,title:r.title});else if(r=this.tokenizer.table(e))e=e.substring(r.raw.length),t.push(r);else if(r=this.tokenizer.lheading(e))e=e.substring(r.raw.length),t.push(r);else if(n&&(r=this.tokenizer.paragraph(e)))e=e.substring(r.raw.length),t.push(r);else if(r=this.tokenizer.text(e,t))e=e.substring(r.raw.length),r.type?t.push(r):((a=t[t.length-1]).raw+="\n"+r.raw,a.text+="\n"+r.text);else if(e){var s="Infinite loop on byte: "+e.charCodeAt(0);if(this.options.silent){console.error(s);break}throw new Error(s)}return t},o.inline=function(e){var t,n,r,i,o,a,s=e.length;for(t=0;t<s;t++)switch((a=e[t]).type){case"paragraph":case"text":case"heading":a.tokens=[],this.inlineTokens(a.text,a.tokens);break;case"table":for(a.tokens={header:[],cells:[]},i=a.header.length,n=0;n<i;n++)a.tokens.header[n]=[],this.inlineTokens(a.header[n],a.tokens.header[n]);for(i=a.cells.length,n=0;n<i;n++)for(o=a.cells[n],a.tokens.cells[n]=[],r=0;r<o.length;r++)a.tokens.cells[n][r]=[],this.inlineTokens(o[r],a.tokens.cells[n][r]);break;case"blockquote":this.inline(a.tokens);break;case"list":for(i=a.items.length,n=0;n<i;n++)this.inline(a.items[n].tokens)}return e},o.inlineTokens=function(e,t,n,r){var i;void 0===t&&(t=[]),void 0===n&&(n=!1),void 0===r&&(r=!1);var o,a,s,l=e;if(this.tokens.links){var u=Object.keys(this.tokens.links);if(u.length>0)for(;null!=(o=this.tokenizer.rules.inline.reflinkSearch.exec(l));)u.includes(o[0].slice(o[0].lastIndexOf("[")+1,-1))&&(l=l.slice(0,o.index)+"["+q("a",o[0].length-2)+"]"+l.slice(this.tokenizer.rules.inline.reflinkSearch.lastIndex))}for(;null!=(o=this.tokenizer.rules.inline.blockSkip.exec(l));)l=l.slice(0,o.index)+"["+q("a",o[0].length-2)+"]"+l.slice(this.tokenizer.rules.inline.blockSkip.lastIndex);for(;e;)if(a||(s=""),a=!1,i=this.tokenizer.escape(e))e=e.substring(i.raw.length),t.push(i);else if(i=this.tokenizer.tag(e,n,r))e=e.substring(i.raw.length),n=i.inLink,r=i.inRawBlock,t.push(i);else if(i=this.tokenizer.link(e))e=e.substring(i.raw.length),"link"===i.type&&(i.tokens=this.inlineTokens(i.text,[],!0,r)),t.push(i);else if(i=this.tokenizer.reflink(e,this.tokens.links))e=e.substring(i.raw.length),"link"===i.type&&(i.tokens=this.inlineTokens(i.text,[],!0,r)),t.push(i);else if(i=this.tokenizer.strong(e,l,s))e=e.substring(i.raw.length),i.tokens=this.inlineTokens(i.text,[],n,r),t.push(i);else if(i=this.tokenizer.em(e,l,s))e=e.substring(i.raw.length),i.tokens=this.inlineTokens(i.text,[],n,r),t.push(i);else if(i=this.tokenizer.codespan(e))e=e.substring(i.raw.length),t.push(i);else if(i=this.tokenizer.br(e))e=e.substring(i.raw.length),t.push(i);else if(i=this.tokenizer.del(e))e=e.substring(i.raw.length),i.tokens=this.inlineTokens(i.text,[],n,r),t.push(i);else if(i=this.tokenizer.autolink(e,Q))e=e.substring(i.raw.length),t.push(i);else if(n||!(i=this.tokenizer.url(e,Q))){if(i=this.tokenizer.inlineText(e,r,Z))e=e.substring(i.raw.length),s=i.raw.slice(-1),a=!0,t.push(i);else if(e){var c="Infinite loop on byte: "+e.charCodeAt(0);if(this.options.silent){console.error(c);break}throw new Error(c)}}else e=e.substring(i.raw.length),t.push(i);return t},n=t,i=[{key:"rules",get:function(){return{block:G,inline:X}}}],(r=null)&&e(n.prototype,r),i&&e(n,i),t}(),J=r.defaults,$=x,ee=A,te=function(){function e(e){this.options=e||J}var t=e.prototype;return t.code=function(e,t,n){var r=(t||"").match(/\S*/)[0];if(this.options.highlight){var i=this.options.highlight(e,r);null!=i&&i!==e&&(n=!0,e=i)}return e=e.replace(/\n$/,"")+"\n",r?'<pre><code class="'+this.options.langPrefix+ee(r,!0)+'">'+(n?e:ee(e,!0))+"</code></pre>\n":"<pre><code>"+(n?e:ee(e,!0))+"</code></pre>\n"},t.blockquote=function(e){return"<blockquote>\n"+e+"</blockquote>\n"},t.html=function(e){return e},t.heading=function(e,t,n,r){return this.options.headerIds?"<h"+t+' id="'+this.options.headerPrefix+r.slug(n)+'">'+e+"</h"+t+">\n":"<h"+t+">"+e+"</h"+t+">\n"},t.hr=function(){return this.options.xhtml?"<hr/>\n":"<hr>\n"},t.list=function(e,t,n){var r=t?"ol":"ul";return"<"+r+(t&&1!==n?' start="'+n+'"':"")+">\n"+e+"</"+r+">\n"},t.listitem=function(e){return"<li>"+e+"</li>\n"},t.checkbox=function(e){return"<input "+(e?'checked="" ':"")+'disabled="" type="checkbox"'+(this.options.xhtml?" /":"")+"> "},t.paragraph=function(e){return"<p>"+e+"</p>\n"},t.table=function(e,t){return t&&(t="<tbody>"+t+"</tbody>"),"<table>\n<thead>\n"+e+"</thead>\n"+t+"</table>\n"},t.tablerow=function(e){return"<tr>\n"+e+"</tr>\n"},t.tablecell=function(e,t){var n=t.header?"th":"td";return(t.align?"<"+n+' align="'+t.align+'">':"<"+n+">")+e+"</"+n+">\n"},t.strong=function(e){return"<strong>"+e+"</strong>"},t.em=function(e){return"<em>"+e+"</em>"},t.codespan=function(e){return"<code>"+e+"</code>"},t.br=function(){return this.options.xhtml?"<br/>":"<br>"},t.del=function(e){return"<del>"+e+"</del>"},t.link=function(e,t,n){if(null===(e=$(this.options.sanitize,this.options.baseUrl,e)))return n;var r='<a href="'+ee(e)+'"';return t&&(r+=' title="'+t+'"'),r+=">"+n+"</a>"},t.image=function(e,t,n){if(null===(e=$(this.options.sanitize,this.options.baseUrl,e)))return n;var r='<img src="'+e+'" alt="'+n+'"';return t&&(r+=' title="'+t+'"'),r+=this.options.xhtml?"/>":">"},t.text=function(e){return e},e}(),ne=function(){function e(){}var t=e.prototype;return t.strong=function(e){return e},t.em=function(e){return e},t.codespan=function(e){return e},t.del=function(e){return e},t.html=function(e){return e},t.text=function(e){return e},t.link=function(e,t,n){return""+n},t.image=function(e,t,n){return""+n},t.br=function(){return""},e}(),re=function(){function e(){this.seen={}}var t=e.prototype;return t.serialize=function(e){return e.toLowerCase().trim().replace(/<[!\/a-z].*?>/gi,"").replace(/[\u2000-\u206F\u2E00-\u2E7F\\'!"#$%&()*+,./:;<=>?@[\]^`{|}~]/g,"").replace(/\s/g,"-")},t.getNextSafeSlug=function(e,t){var n=e,r=0;if(this.seen.hasOwnProperty(n)){r=this.seen[e];do{n=e+"-"+ ++r}while(this.seen.hasOwnProperty(n))}return t||(this.seen[e]=r,this.seen[n]=0),n},t.slug=function(e,t){void 0===t&&(t={});var n=this.serialize(e);return this.getNextSafeSlug(n,t.dryrun)},e}(),ie=r.defaults,oe=_,ae=function(){function e(e){this.options=e||ie,this.options.renderer=this.options.renderer||new te,this.renderer=this.options.renderer,this.renderer.options=this.options,this.textRenderer=new ne,this.slugger=new re}e.parse=function(t,n){return new e(n).parse(t)},e.parseInline=function(t,n){return new e(n).parseInline(t)};var t=e.prototype;return t.parse=function(e,t){void 0===t&&(t=!0);var n,r,i,o,a,s,l,u,c,d,h,f,p,g,v,m,y,b,C="",A=e.length;for(n=0;n<A;n++)switch((d=e[n]).type){case"space":continue;case"hr":C+=this.renderer.hr();continue;case"heading":C+=this.renderer.heading(this.parseInline(d.tokens),d.depth,oe(this.parseInline(d.tokens,this.textRenderer)),this.slugger);continue;case"code":C+=this.renderer.code(d.text,d.lang,d.escaped);continue;case"table":for(u="",l="",o=d.header.length,r=0;r<o;r++)l+=this.renderer.tablecell(this.parseInline(d.tokens.header[r]),{header:!0,align:d.align[r]});for(u+=this.renderer.tablerow(l),c="",o=d.cells.length,r=0;r<o;r++){for(l="",a=(s=d.tokens.cells[r]).length,i=0;i<a;i++)l+=this.renderer.tablecell(this.parseInline(s[i]),{header:!1,align:d.align[i]});c+=this.renderer.tablerow(l)}C+=this.renderer.table(u,c);continue;case"blockquote":c=this.parse(d.tokens),C+=this.renderer.blockquote(c);continue;case"list":for(h=d.ordered,f=d.start,p=d.loose,o=d.items.length,c="",r=0;r<o;r++)m=(v=d.items[r]).checked,y=v.task,g="",v.task&&(b=this.renderer.checkbox(m),p?v.tokens.length>0&&"text"===v.tokens[0].type?(v.tokens[0].text=b+" "+v.tokens[0].text,v.tokens[0].tokens&&v.tokens[0].tokens.length>0&&"text"===v.tokens[0].tokens[0].type&&(v.tokens[0].tokens[0].text=b+" "+v.tokens[0].tokens[0].text)):v.tokens.unshift({type:"text",text:b}):g+=b),g+=this.parse(v.tokens,p),c+=this.renderer.listitem(g,y,m);C+=this.renderer.list(c,h,f);continue;case"html":C+=this.renderer.html(d.text);continue;case"paragraph":C+=this.renderer.paragraph(this.parseInline(d.tokens));continue;case"text":for(c=d.tokens?this.parseInline(d.tokens):d.text;n+1<A&&"text"===e[n+1].type;)c+="\n"+((d=e[++n]).tokens?this.parseInline(d.tokens):d.text);C+=t?this.renderer.paragraph(c):c;continue;default:var _='Token with "'+d.type+'" type was not found.';if(this.options.silent)return void console.error(_);throw new Error(_)}return C},t.parseInline=function(e,t){t=t||this.renderer;var n,r,i="",o=e.length;for(n=0;n<o;n++)switch((r=e[n]).type){case"escape":i+=t.text(r.text);break;case"html":i+=t.html(r.text);break;case"link":i+=t.link(r.href,r.title,this.parseInline(r.tokens,t));break;case"image":i+=t.image(r.href,r.title,r.text);break;case"strong":i+=t.strong(this.parseInline(r.tokens,t));break;case"em":i+=t.em(this.parseInline(r.tokens,t));break;case"codespan":i+=t.codespan(r.text);break;case"br":i+=t.br();break;case"del":i+=t.del(this.parseInline(r.tokens,t));break;case"text":i+=t.text(r.text);break;default:var a='Token with "'+r.type+'" type was not found.';if(this.options.silent)return void console.error(a);throw new Error(a)}return i},e}(),se=M,le=O,ue=A,ce=r.getDefaults,de=r.changeDefaults,he=r.defaults;function fe(e,t,n){if(null==e)throw new Error("marked(): input parameter is undefined or null");if("string"!=typeof e)throw new Error("marked(): input parameter is of type "+Object.prototype.toString.call(e)+", string expected");if("function"==typeof t&&(n=t,t=null),t=se({},fe.defaults,t||{}),le(t),n){var r,i=t.highlight;try{r=K.lex(e,t)}catch(e){return n(e)}var o=function(e){var o;if(!e)try{o=ae.parse(r,t)}catch(t){e=t}return t.highlight=i,e?n(e):n(null,o)};if(!i||i.length<3)return o();if(delete t.highlight,!r.length)return o();var a=0;return fe.walkTokens(r,(function(e){"code"===e.type&&(a++,setTimeout((function(){i(e.text,e.lang,(function(t,n){if(t)return o(t);null!=n&&n!==e.text&&(e.text=n,e.escaped=!0),0==--a&&o()}))}),0))})),void(0===a&&o())}try{var s=K.lex(e,t);return t.walkTokens&&fe.walkTokens(s,t.walkTokens),ae.parse(s,t)}catch(e){if(e.message+="\nPlease report this to https://github.com/markedjs/marked.",t.silent)return"<p>An error occurred:</p><pre>"+ue(e.message+"",!0)+"</pre>";throw e}}return fe.options=fe.setOptions=function(e){return se(fe.defaults,e),de(fe.defaults),fe},fe.getDefaults=ce,fe.defaults=he,fe.use=function(e){var t=se({},e);if(e.renderer&&function(){var n=fe.defaults.renderer||new te,r=function(t){var r=n[t];n[t]=function(){for(var i=arguments.length,o=new Array(i),a=0;a<i;a++)o[a]=arguments[a];var s=e.renderer[t].apply(n,o);return!1===s&&(s=r.apply(n,o)),s}};for(var i in e.renderer)r(i);t.renderer=n}(),e.tokenizer&&function(){var n=fe.defaults.tokenizer||new z,r=function(t){var r=n[t];n[t]=function(){for(var i=arguments.length,o=new Array(i),a=0;a<i;a++)o[a]=arguments[a];var s=e.tokenizer[t].apply(n,o);return!1===s&&(s=r.apply(n,o)),s}};for(var i in e.tokenizer)r(i);t.tokenizer=n}(),e.walkTokens){var n=fe.defaults.walkTokens;t.walkTokens=function(t){e.walkTokens(t),n&&n(t)}}fe.setOptions(t)},fe.walkTokens=function(e,t){for(var r,i=n(e);!(r=i()).done;){var o=r.value;switch(t(o),o.type){case"table":for(var a,s=n(o.tokens.header);!(a=s()).done;){var l=a.value;fe.walkTokens(l,t)}for(var u,c=n(o.tokens.cells);!(u=c()).done;)for(var d,h=n(u.value);!(d=h()).done;){var f=d.value;fe.walkTokens(f,t)}break;case"list":fe.walkTokens(o.items,t);break;default:o.tokens&&fe.walkTokens(o.tokens,t)}}},fe.parseInline=function(e,t){if(null==e)throw new Error("marked.parseInline(): input parameter is undefined or null");if("string"!=typeof e)throw new Error("marked.parseInline(): input parameter is of type "+Object.prototype.toString.call(e)+", string expected");t=se({},fe.defaults,t||{}),le(t);try{var n=K.lexInline(e,t);return t.walkTokens&&fe.walkTokens(n,t.walkTokens),ae.parseInline(n,t)}catch(e){if(e.message+="\nPlease report this to https://github.com/markedjs/marked.",t.silent)return"<p>An error occurred:</p><pre>"+ue(e.message+"",!0)+"</pre>";throw e}},fe.Parser=ae,fe.parser=ae.parse,fe.Renderer=te,fe.TextRenderer=ne,fe.Lexer=K,fe.lexer=K.lex,fe.Tokenizer=z,fe.Slugger=re,fe.parse=fe,fe}()},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDE4IDE4Ij4KICAgIDxnIGNsYXNzPSJqcC1pY29uMyIgZmlsbD0iIzYxNjE2MSI+CiAgICAgICAgPHBhdGggZD0iTTkgMTMuNWMtMi40OSAwLTQuNS0yLjAxLTQuNS00LjVTNi41MSA0LjUgOSA0LjVjMS4yNCAwIDIuMzYuNTIgMy4xNyAxLjMzTDEwIDhoNVYzbC0xLjc2IDEuNzZDMTIuMTUgMy42OCAxMC42NiAzIDkgMyA1LjY5IDMgMy4wMSA1LjY5IDMuMDEgOVM1LjY5IDE1IDkgMTVjMi45NyAwIDUuNDMtMi4xNiA1LjktNWgtMS41MmMtLjQ2IDItMi4yNCAzLjUtNC4zOCAzLjV6Ii8+CiAgICA8L2c+Cjwvc3ZnPgo="},function(e,t,n){"use strict";n.d(t,"a",(function(){return a}));var r=n(1),i=n(140);function o(e){return e&&"object"===Object(i.a)(e)&&e.constructor===Object}function a(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{clone:!0},i=n.clone?Object(r.a)({},e):e;return o(e)&&o(t)&&Object.keys(t).forEach((function(r){"__proto__"!==r&&(o(t[r])&&r in e?i[r]=a(e[r],t[r],n):i[r]=t[r])})),i}},function(e,t,n){"use strict";function r(e){if(Array.isArray(e))return e}n.d(t,"a",(function(){return r}))},function(e,t,n){"use strict";function r(){throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}n.d(t,"a",(function(){return r}))},function(e,t,n){"use strict";function r(e){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e))return Array.from(e)}n.d(t,"a",(function(){return r}))},function(e,t,n){"use strict";t.a={mobileStepper:1e3,speedDial:1050,appBar:1100,drawer:1200,modal:1300,snackbar:1400,tooltip:1500}},function(e,t,n){"use strict";function r(e,t){return(r=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}n.d(t,"a",(function(){return r}))},function(e,t,n){"use strict";var r=n(0),i=n.n(r).a.createContext(null);t.a=i},function(e,t,n){"use strict";var r="function"==typeof Symbol&&Symbol.for;t.a=r?Symbol.for("mui.nested"):"__THEME_NESTED__"},function(e,t,n){"use strict";n.d(t,"a",(function(){return s}));var r=n(443),i=n(445),o=n(273),a=n(444);function s(e){return Object(r.a)(e)||Object(i.a)(e)||Object(o.a)(e)||Object(a.a)()}},function(e,t,n){"use strict";var r=n(0),i=r.createContext({});t.a=i},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(4)),s=n(7),l=n(26),u=n(231),c=n(15),d=o.forwardRef((function(e,t){var n=e.edge,s=void 0!==n&&n,l=e.children,d=e.classes,h=e.className,f=e.color,p=void 0===f?"default":f,g=e.disabled,v=void 0!==g&&g,m=e.disableFocusRipple,y=void 0!==m&&m,b=e.size,C=void 0===b?"medium":b,A=Object(i.a)(e,["edge","children","classes","className","color","disabled","disableFocusRipple","size"]);return o.createElement(u.a,Object(r.a)({className:Object(a.a)(d.root,h,"default"!==p&&d["color".concat(Object(c.a)(p))],v&&d.disabled,"small"===C&&d["size".concat(Object(c.a)(C))],{start:d.edgeStart,end:d.edgeEnd}[s]),centerRipple:!0,focusRipple:!y,disabled:v,ref:t},A),o.createElement("span",{className:d.label},l))}));t.a=Object(s.a)((function(e){return{root:{textAlign:"center",flex:"0 0 auto",fontSize:e.typography.pxToRem(24),padding:12,borderRadius:"50%",overflow:"visible",color:e.palette.action.active,transition:e.transitions.create("background-color",{duration:e.transitions.duration.shortest}),"&:hover":{backgroundColor:Object(l.d)(e.palette.action.active,e.palette.action.hoverOpacity),"@media (hover: none)":{backgroundColor:"transparent"}},"&$disabled":{backgroundColor:"transparent",color:e.palette.action.disabled}},edgeStart:{marginLeft:-12,"$sizeSmall&":{marginLeft:-3}},edgeEnd:{marginRight:-12,"$sizeSmall&":{marginRight:-3}},colorInherit:{color:"inherit"},colorPrimary:{color:e.palette.primary.main,"&:hover":{backgroundColor:Object(l.d)(e.palette.primary.main,e.palette.action.hoverOpacity),"@media (hover: none)":{backgroundColor:"transparent"}}},colorSecondary:{color:e.palette.secondary.main,"&:hover":{backgroundColor:Object(l.d)(e.palette.secondary.main,e.palette.action.hoverOpacity),"@media (hover: none)":{backgroundColor:"transparent"}}},disabled:{},sizeSmall:{padding:3,fontSize:e.typography.pxToRem(18)},label:{width:"100%",display:"flex",alignItems:"inherit",justifyContent:"inherit"}}}),{name:"MuiIconButton"})(d)},function(e,t,n){"use strict";function r(){var e=document.createElement("div");e.style.width="99px",e.style.height="99px",e.style.position="absolute",e.style.top="-9999px",e.style.overflow="scroll",document.body.appendChild(e);var t=e.offsetWidth-e.clientWidth;return document.body.removeChild(e),t}n.d(t,"a",(function(){return r}))},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(4)),s=n(1311),l=n(7),u=o.forwardRef((function(e,t){var n=e.disableUnderline,l=e.classes,u=e.fullWidth,c=void 0!==u&&u,d=e.inputComponent,h=void 0===d?"input":d,f=e.multiline,p=void 0!==f&&f,g=e.type,v=void 0===g?"text":g,m=Object(i.a)(e,["disableUnderline","classes","fullWidth","inputComponent","multiline","type"]);return o.createElement(s.a,Object(r.a)({classes:Object(r.a)({},l,{root:Object(a.a)(l.root,!n&&l.underline),underline:null}),fullWidth:c,inputComponent:h,multiline:p,ref:t,type:v},m))}));u.muiName="Input",t.a=Object(l.a)((function(e){var t="light"===e.palette.type?"rgba(0, 0, 0, 0.42)":"rgba(255, 255, 255, 0.7)";return{root:{position:"relative"},formControl:{"label + &":{marginTop:16}},focused:{},disabled:{},colorSecondary:{"&$underline:after":{borderBottomColor:e.palette.secondary.main}},underline:{"&:after":{borderBottom:"2px solid ".concat(e.palette.primary.main),left:0,bottom:0,content:'""',position:"absolute",right:0,transform:"scaleX(0)",transition:e.transitions.create("transform",{duration:e.transitions.duration.shorter,easing:e.transitions.easing.easeOut}),pointerEvents:"none"},"&$focused:after":{transform:"scaleX(1)"},"&$error:after":{borderBottomColor:e.palette.error.main,transform:"scaleX(1)"},"&:before":{borderBottom:"1px solid ".concat(t),left:0,bottom:0,content:'"\\00a0"',position:"absolute",right:0,transition:e.transitions.create("border-bottom-color",{duration:e.transitions.duration.shorter}),pointerEvents:"none"},"&:hover:not($disabled):before":{borderBottom:"2px solid ".concat(e.palette.text.primary),"@media (hover: none)":{borderBottom:"1px solid ".concat(t)}},"&$disabled:before":{borderBottomStyle:"dotted"}},error:{},marginDense:{},multiline:{},fullWidth:{},input:{},inputMarginDense:{},inputMultiline:{},inputTypeSearch:{}}}),{name:"MuiInput"})(u)},function(e,t,n){"use strict";var r=n(0),i=n(57);t.a=Object(i.a)(r.createElement("path",{d:"M7 10l5 5 5-5z"}),"ArrowDropDown")},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(4)),s=n(15),l=o.forwardRef((function(e,t){var n=e.classes,l=e.className,u=e.disabled,c=e.IconComponent,d=e.inputRef,h=e.variant,f=void 0===h?"standard":h,p=Object(i.a)(e,["classes","className","disabled","IconComponent","inputRef","variant"]);return o.createElement(o.Fragment,null,o.createElement("select",Object(r.a)({className:Object(a.a)(n.root,n.select,n[f],l,u&&n.disabled),disabled:u,ref:d||t},p)),e.multiple?null:o.createElement(c,{className:Object(a.a)(n.icon,n["icon".concat(Object(s.a)(f))],u&&n.disabled)}))}));t.a=l},function(e,t,n){"use strict";var r=n(0),i=r.createContext();t.a=i},function(e,t,n){"use strict";(function(e){function r(t,n){var r,i,o,a=void 0!==(r=void 0!==n?n:"undefined"!=typeof window?window:"undefined"!=typeof self?self:e).document&&r.document.attachEvent;if(!a){var s=(o=r.requestAnimationFrame||r.mozRequestAnimationFrame||r.webkitRequestAnimationFrame||function(e){return r.setTimeout(e,20)},function(e){return o(e)}),l=(i=r.cancelAnimationFrame||r.mozCancelAnimationFrame||r.webkitCancelAnimationFrame||r.clearTimeout,function(e){return i(e)}),u=function(e){var t=e.__resizeTriggers__,n=t.firstElementChild,r=t.lastElementChild,i=n.firstElementChild;r.scrollLeft=r.scrollWidth,r.scrollTop=r.scrollHeight,i.style.width=n.offsetWidth+1+"px",i.style.height=n.offsetHeight+1+"px",n.scrollLeft=n.scrollWidth,n.scrollTop=n.scrollHeight},c=function(e){if(!(e.target.className&&"function"==typeof e.target.className.indexOf&&e.target.className.indexOf("contract-trigger")<0&&e.target.className.indexOf("expand-trigger")<0)){var t=this;u(this),this.__resizeRAF__&&l(this.__resizeRAF__),this.__resizeRAF__=s((function(){(function(e){return e.offsetWidth!=e.__resizeLast__.width||e.offsetHeight!=e.__resizeLast__.height})(t)&&(t.__resizeLast__.width=t.offsetWidth,t.__resizeLast__.height=t.offsetHeight,t.__resizeListeners__.forEach((function(n){n.call(t,e)})))}))}},d=!1,h="",f="animationstart",p="Webkit Moz O ms".split(" "),g="webkitAnimationStart animationstart oAnimationStart MSAnimationStart".split(" "),v=r.document.createElement("fakeelement");if(void 0!==v.style.animationName&&(d=!0),!1===d)for(var m=0;m<p.length;m++)if(void 0!==v.style[p[m]+"AnimationName"]){h="-"+p[m].toLowerCase()+"-",f=g[m],d=!0;break}var y="resizeanim",b="@"+h+"keyframes "+y+" { from { opacity: 0; } to { opacity: 0; } } ",C=h+"animation: 1ms "+y+"; "}return{addResizeListener:function(e,n){if(a)e.attachEvent("onresize",n);else{if(!e.__resizeTriggers__){var i=e.ownerDocument,o=r.getComputedStyle(e);o&&"static"==o.position&&(e.style.position="relative"),function(e){if(!e.getElementById("detectElementResize")){var n=(b||"")+".resize-triggers { "+(C||"")+'visibility: hidden; opacity: 0; } .resize-triggers, .resize-triggers > div, .contract-trigger:before { content: " "; display: block; position: absolute; top: 0; left: 0; height: 100%; width: 100%; overflow: hidden; z-index: -1; } .resize-triggers > div { background: #eee; overflow: auto; } .contract-trigger:before { width: 200%; height: 200%; }',r=e.head||e.getElementsByTagName("head")[0],i=e.createElement("style");i.id="detectElementResize",i.type="text/css",null!=t&&i.setAttribute("nonce",t),i.styleSheet?i.styleSheet.cssText=n:i.appendChild(e.createTextNode(n)),r.appendChild(i)}}(i),e.__resizeLast__={},e.__resizeListeners__=[],(e.__resizeTriggers__=i.createElement("div")).className="resize-triggers";var s='<div class="expand-trigger"><div></div></div><div class="contract-trigger"></div>';if(window.trustedTypes){var l=trustedTypes.createPolicy("react-virtualized-auto-sizer",{createHTML:function(){return s}});e.__resizeTriggers__.innerHTML=l.createHTML("")}else e.__resizeTriggers__.innerHTML=s;e.appendChild(e.__resizeTriggers__),u(e),e.addEventListener("scroll",c,!0),f&&(e.__resizeTriggers__.__animationListener__=function(t){t.animationName==y&&u(e)},e.__resizeTriggers__.addEventListener(f,e.__resizeTriggers__.__animationListener__))}e.__resizeListeners__.push(n)}},removeResizeListener:function(e,t){if(a)e.detachEvent("onresize",t);else if(e.__resizeListeners__.splice(e.__resizeListeners__.indexOf(t),1),!e.__resizeListeners__.length){e.removeEventListener("scroll",c,!0),e.__resizeTriggers__.__animationListener__&&(e.__resizeTriggers__.removeEventListener(f,e.__resizeTriggers__.__animationListener__),e.__resizeTriggers__.__animationListener__=null);try{e.__resizeTriggers__=!e.removeChild(e.__resizeTriggers__)}catch(e){}}}}}n.d(t,"a",(function(){return r}))}).call(this,n(61))},function(e,t,n){"use strict";t.a=function(e){var t={};return function(n){return void 0===t[n]&&(t[n]=e(n)),t[n]}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){(0,i.default)(1,arguments);var t=(0,r.default)(e);return t.setHours(0,0,0,0),t};var r=o(n(47)),i=o(n(40));function o(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){(0,o.default)(2,arguments);var n=(0,i.default)(e),a=(0,r.default)(t);if(isNaN(a))return new Date(NaN);if(!a)return n;var s=n.getDate(),l=new Date(n.getTime());l.setMonth(n.getMonth()+a+1,0);var u=l.getDate();return s>=u?l:(n.setFullYear(l.getFullYear(),l.getMonth(),s),n)};var r=a(n(77)),i=a(n(47)),o=a(n(40));function a(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";var r=n(5),i=n(1),o=n(0),a=n(22),s=(n(3),n(277)),l=n(653),u=n(59),c=n(2354),d=n(107),h=n(37),f=n(53),p=n(446),g=n(331),v=n(2376),m={root:{zIndex:-1,position:"fixed",right:0,bottom:0,top:0,left:0,backgroundColor:"rgba(0, 0, 0, 0.5)",WebkitTapHighlightColor:"transparent"},invisible:{backgroundColor:"transparent"}},y=o.forwardRef((function(e,t){var n=e.invisible,a=void 0!==n&&n,s=e.open,l=Object(r.a)(e,["invisible","open"]);return s?o.createElement("div",Object(i.a)({"aria-hidden":!0,ref:t},l,{style:Object(i.a)({},m.root,a?m.invisible:{},l.style)})):null}));var b=new g.b,C=o.forwardRef((function(e,t){var n=Object(s.a)(),m=Object(l.a)({name:"MuiModal",props:Object(i.a)({},e),theme:n}),C=m.BackdropComponent,A=void 0===C?y:C,_=m.BackdropProps,I=m.children,x=m.closeAfterTransition,w=void 0!==x&&x,M=m.container,S=m.disableAutoFocus,T=void 0!==S&&S,E=m.disableBackdropClick,O=void 0!==E&&E,k=m.disableEnforceFocus,L=void 0!==k&&k,D=m.disableEscapeKeyDown,P=void 0!==D&&D,N=m.disablePortal,j=void 0!==N&&N,R=m.disableRestoreFocus,z=void 0!==R&&R,F=m.disableScrollLock,B=void 0!==F&&F,H=m.hideBackdrop,U=void 0!==H&&H,V=m.keepMounted,Y=void 0!==V&&V,W=m.manager,G=void 0===W?b:W,X=m.onBackdropClick,q=m.onClose,Z=m.onEscapeKeyDown,Q=m.onRendered,K=m.open,J=Object(r.a)(m,["BackdropComponent","BackdropProps","children","closeAfterTransition","container","disableAutoFocus","disableBackdropClick","disableEnforceFocus","disableEscapeKeyDown","disablePortal","disableRestoreFocus","disableScrollLock","hideBackdrop","keepMounted","manager","onBackdropClick","onClose","onEscapeKeyDown","onRendered","open"]),$=o.useState(!0),ee=$[0],te=$[1],ne=o.useRef({}),re=o.useRef(null),ie=o.useRef(null),oe=Object(h.a)(ie,t),ae=function(e){return!!e.children&&e.children.props.hasOwnProperty("in")}(m),se=function(){return Object(u.a)(re.current)},le=function(){return ne.current.modalRef=ie.current,ne.current.mountNode=re.current,ne.current},ue=function(){G.mount(le(),{disableScrollLock:B}),ie.current.scrollTop=0},ce=Object(f.a)((function(){var e=function(e){return e="function"==typeof e?e():e,a.findDOMNode(e)}(M)||se().body;G.add(le(),e),ie.current&&ue()})),de=o.useCallback((function(){return G.isTopModal(le())}),[G]),he=Object(f.a)((function(e){re.current=e,e&&(Q&&Q(),K&&de()?ue():Object(g.a)(ie.current,!0))})),fe=o.useCallback((function(){G.remove(le())}),[G]);if(o.useEffect((function(){return function(){fe()}}),[fe]),o.useEffect((function(){K?ce():ae&&w||fe()}),[K,fe,ae,w,ce]),!Y&&!K&&(!ae||ee))return null;var pe=function(e){return{root:{position:"fixed",zIndex:e.zIndex.modal,right:0,bottom:0,top:0,left:0},hidden:{visibility:"hidden"}}}(n||{zIndex:p.a}),ge={};return void 0===I.props.tabIndex&&(ge.tabIndex=I.props.tabIndex||"-1"),ae&&(ge.onEnter=Object(d.a)((function(){te(!1)}),I.props.onEnter),ge.onExited=Object(d.a)((function(){te(!0),w&&fe()}),I.props.onExited)),o.createElement(c.a,{ref:he,container:M,disablePortal:j},o.createElement("div",Object(i.a)({ref:oe,onKeyDown:function(e){"Escape"===e.key&&de()&&(Z&&Z(e),P||(e.stopPropagation(),q&&q(e,"escapeKeyDown")))},role:"presentation"},J,{style:Object(i.a)({},pe.root,!K&&ee?pe.hidden:{},J.style)}),U?null:o.createElement(A,Object(i.a)({open:K,onClick:function(e){e.target===e.currentTarget&&(X&&X(e),!O&&q&&q(e,"backdropClick"))}},_)),o.createElement(v.a,{disableEnforceFocus:L,disableAutoFocus:T,disableRestoreFocus:z,getDoc:se,isEnabled:de,open:K},o.cloneElement(I,ge))))}));t.a=C},function(e,t,n){"use strict";n.d(t,"a",(function(){return a}));var r=n(17),i=n(14),o=n(439);class a extends o.a{constructor(e={}){super({itemCmp:r.JSONExt.deepEqual,values:e.values})}toJSON(){const e=Object.create(null),t=this.keys();for(const n of t){const t=this.get(n);void 0!==t&&(e[n]=r.JSONExt.deepCopy(t))}return e}}!function(e){class t extends i.b{constructor(e,t){super(e),this.args=t}}e.ChangeMessage=t}(a||(a={}))},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){(0,i.default)(2,arguments);var n=(0,r.default)(e),o=(0,r.default)(t);return n.getTime()>o.getTime()};var r=o(n(47)),i=o(n(40));function o(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){(0,i.default)(1,arguments);var t=(0,r.default)(e),n=new Date(0);return n.setFullYear(t.getFullYear(),0,1),n.setHours(0,0,0,0),n};var r=o(n(47)),i=o(n(40));function o(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},,,,,,,,,,,,,,,,,,,,,function(e,t,n){"use strict";(function(t){function n(e){if("string"!=typeof e)throw new TypeError("Path must be a string. Received "+JSON.stringify(e))}function r(e,t){for(var n,r="",i=0,o=-1,a=0,s=0;s<=e.length;++s){if(s<e.length)n=e.charCodeAt(s);else{if(47===n)break;n=47}if(47===n){if(o===s-1||1===a);else if(o!==s-1&&2===a){if(r.length<2||2!==i||46!==r.charCodeAt(r.length-1)||46!==r.charCodeAt(r.length-2))if(r.length>2){var l=r.lastIndexOf("/");if(l!==r.length-1){-1===l?(r="",i=0):i=(r=r.slice(0,l)).length-1-r.lastIndexOf("/"),o=s,a=0;continue}}else if(2===r.length||1===r.length){r="",i=0,o=s,a=0;continue}t&&(r.length>0?r+="/..":r="..",i=2)}else r.length>0?r+="/"+e.slice(o+1,s):r=e.slice(o+1,s),i=s-o-1;o=s,a=0}else 46===n&&-1!==a?++a:a=-1}return r}var i={resolve:function(){for(var e,i="",o=!1,a=arguments.length-1;a>=-1&&!o;a--){var s;a>=0?s=arguments[a]:(void 0===e&&(e=t.cwd()),s=e),n(s),0!==s.length&&(i=s+"/"+i,o=47===s.charCodeAt(0))}return i=r(i,!o),o?i.length>0?"/"+i:"/":i.length>0?i:"."},normalize:function(e){if(n(e),0===e.length)return".";var t=47===e.charCodeAt(0),i=47===e.charCodeAt(e.length-1);return 0!==(e=r(e,!t)).length||t||(e="."),e.length>0&&i&&(e+="/"),t?"/"+e:e},isAbsolute:function(e){return n(e),e.length>0&&47===e.charCodeAt(0)},join:function(){if(0===arguments.length)return".";for(var e,t=0;t<arguments.length;++t){var r=arguments[t];n(r),r.length>0&&(void 0===e?e=r:e+="/"+r)}return void 0===e?".":i.normalize(e)},relative:function(e,t){if(n(e),n(t),e===t)return"";if((e=i.resolve(e))===(t=i.resolve(t)))return"";for(var r=1;r<e.length&&47===e.charCodeAt(r);++r);for(var o=e.length,a=o-r,s=1;s<t.length&&47===t.charCodeAt(s);++s);for(var l=t.length-s,u=a<l?a:l,c=-1,d=0;d<=u;++d){if(d===u){if(l>u){if(47===t.charCodeAt(s+d))return t.slice(s+d+1);if(0===d)return t.slice(s+d)}else a>u&&(47===e.charCodeAt(r+d)?c=d:0===d&&(c=0));break}var h=e.charCodeAt(r+d);if(h!==t.charCodeAt(s+d))break;47===h&&(c=d)}var f="";for(d=r+c+1;d<=o;++d)d!==o&&47!==e.charCodeAt(d)||(0===f.length?f+="..":f+="/..");return f.length>0?f+t.slice(s+c):(s+=c,47===t.charCodeAt(s)&&++s,t.slice(s))},_makeLong:function(e){return e},dirname:function(e){if(n(e),0===e.length)return".";for(var t=e.charCodeAt(0),r=47===t,i=-1,o=!0,a=e.length-1;a>=1;--a)if(47===(t=e.charCodeAt(a))){if(!o){i=a;break}}else o=!1;return-1===i?r?"/":".":r&&1===i?"//":e.slice(0,i)},basename:function(e,t){if(void 0!==t&&"string"!=typeof t)throw new TypeError('"ext" argument must be a string');n(e);var r,i=0,o=-1,a=!0;if(void 0!==t&&t.length>0&&t.length<=e.length){if(t.length===e.length&&t===e)return"";var s=t.length-1,l=-1;for(r=e.length-1;r>=0;--r){var u=e.charCodeAt(r);if(47===u){if(!a){i=r+1;break}}else-1===l&&(a=!1,l=r+1),s>=0&&(u===t.charCodeAt(s)?-1==--s&&(o=r):(s=-1,o=l))}return i===o?o=l:-1===o&&(o=e.length),e.slice(i,o)}for(r=e.length-1;r>=0;--r)if(47===e.charCodeAt(r)){if(!a){i=r+1;break}}else-1===o&&(a=!1,o=r+1);return-1===o?"":e.slice(i,o)},extname:function(e){n(e);for(var t=-1,r=0,i=-1,o=!0,a=0,s=e.length-1;s>=0;--s){var l=e.charCodeAt(s);if(47!==l)-1===i&&(o=!1,i=s+1),46===l?-1===t?t=s:1!==a&&(a=1):-1!==t&&(a=-1);else if(!o){r=s+1;break}}return-1===t||-1===i||0===a||1===a&&t===i-1&&t===r+1?"":e.slice(t,i)},format:function(e){if(null===e||"object"!=typeof e)throw new TypeError('The "pathObject" argument must be of type Object. Received type '+typeof e);return function(e,t){var n=t.dir||t.root,r=t.base||(t.name||"")+(t.ext||"");return n?n===t.root?n+r:n+e+r:r}("/",e)},parse:function(e){n(e);var t={root:"",dir:"",base:"",ext:"",name:""};if(0===e.length)return t;var r,i=e.charCodeAt(0),o=47===i;o?(t.root="/",r=1):r=0;for(var a=-1,s=0,l=-1,u=!0,c=e.length-1,d=0;c>=r;--c)if(47!==(i=e.charCodeAt(c)))-1===l&&(u=!1,l=c+1),46===i?-1===a?a=c:1!==d&&(d=1):-1!==a&&(d=-1);else if(!u){s=c+1;break}return-1===a||-1===l||0===d||1===d&&a===l-1&&a===s+1?-1!==l&&(t.base=t.name=0===s&&o?e.slice(1,l):e.slice(s,l)):(0===s&&o?(t.name=e.slice(1,a),t.base=e.slice(1,l)):(t.name=e.slice(s,a),t.base=e.slice(s,l)),t.ext=e.slice(a,l)),s>0?t.dir=e.slice(0,s-1):o&&(t.dir="/"),t},sep:"/",delimiter:":",win32:null,posix:null};i.posix=i,e.exports=i}).call(this,n(145))},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.getKernelModel=t.shutdownKernel=t.interruptKernel=t.restartKernel=t.startNew=t.listRunning=t.KERNEL_SERVICE_URL=void 0;const r=n(129),i=n(48),o=n(488);t.KERNEL_SERVICE_URL="api/kernels",t.listRunning=async function(e=r.ServerConnection.makeSettings()){const n=i.URLExt.join(e.baseUrl,t.KERNEL_SERVICE_URL),a=await r.ServerConnection.makeRequest(n,{},e);if(200!==a.status){throw await r.ServerConnection.ResponseError.create(a)}const s=await a.json();return o.validateModels(s),s},t.startNew=async function(e={},n=r.ServerConnection.makeSettings()){const a=i.URLExt.join(n.baseUrl,t.KERNEL_SERVICE_URL),s={method:"POST",body:JSON.stringify(e)},l=await r.ServerConnection.makeRequest(a,s,n);if(201!==l.status){throw await r.ServerConnection.ResponseError.create(l)}const u=await l.json();return o.validateModel(u),u},t.restartKernel=async function(e,n=r.ServerConnection.makeSettings()){const a=i.URLExt.join(n.baseUrl,t.KERNEL_SERVICE_URL,encodeURIComponent(e),"restart"),s=await r.ServerConnection.makeRequest(a,{method:"POST"},n);if(200!==s.status){throw await r.ServerConnection.ResponseError.create(s)}const l=await s.json();o.validateModel(l)},t.interruptKernel=async function(e,n=r.ServerConnection.makeSettings()){const o=i.URLExt.join(n.baseUrl,t.KERNEL_SERVICE_URL,encodeURIComponent(e),"interrupt"),a=await r.ServerConnection.makeRequest(o,{method:"POST"},n);if(204!==a.status){throw await r.ServerConnection.ResponseError.create(a)}},t.shutdownKernel=async function(e,n=r.ServerConnection.makeSettings()){const o=i.URLExt.join(n.baseUrl,t.KERNEL_SERVICE_URL,encodeURIComponent(e)),a=await r.ServerConnection.makeRequest(o,{method:"DELETE"},n);if(404===a.status){const t=`The kernel "${e}" does not exist on the server`;console.warn(t)}else if(204!==a.status){throw await r.ServerConnection.ResponseError.create(a)}},t.getKernelModel=async function(e,n=r.ServerConnection.makeSettings()){const a=i.URLExt.join(n.baseUrl,t.KERNEL_SERVICE_URL,encodeURIComponent(e)),s=await r.ServerConnection.makeRequest(a,{},n);if(404===s.status)return;if(200!==s.status){throw await r.ServerConnection.ResponseError.create(s)}const l=await s.json();return o.validateModel(l),l}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.validateModels=t.validateModel=t.validateMessage=void 0;const r=n(352),i=["username","version","session","msg_id","msg_type"],o={stream:{name:"string",text:"string"},display_data:{data:"object",metadata:"object"},execute_input:{code:"string",execution_count:"number"},execute_result:{execution_count:"number",data:"object",metadata:"object"},error:{ename:"string",evalue:"string",traceback:"object"},status:{execution_state:["string",["starting","idle","busy","restarting","dead"]]},clear_output:{wait:"boolean"},comm_open:{comm_id:"string",target_name:"string",data:"object"},comm_msg:{comm_id:"string",data:"object"},comm_close:{comm_id:"string"},shutdown_reply:{restart:"boolean"}};function a(e){r.validateProperty(e,"name","string"),r.validateProperty(e,"id","string")}t.validateMessage=function(e){r.validateProperty(e,"metadata","object"),r.validateProperty(e,"content","object"),r.validateProperty(e,"channel","string"),function(e){for(let t=0;t<i.length;t++)r.validateProperty(e,i[t],"string")}(e.header),"iopub"===e.channel&&function(e){if("iopub"===e.channel){const t=o[e.header.msg_type];if(void 0===t)return;const n=Object.keys(t),i=e.content;for(let e=0;e<n.length;e++){let o=t[n[e]];Array.isArray(o)||(o=[o]),r.validateProperty(i,n[e],...o)}}}(e)},t.validateModel=a,t.validateModels=function(e){if(!Array.isArray(e))throw new Error("Invalid kernel list");e.forEach(e=>a(e))}},function(e,t,n){"use strict";var r=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),i=this&&this.__setModuleDefault||(Object.create?function(e,t){Object.defineProperty(e,"default",{enumerable:!0,value:t})}:function(e,t){e.default=t}),o=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)"default"!==n&&Object.prototype.hasOwnProperty.call(e,n)&&r(t,e,n);return i(t,e),t},a=this&&this.__exportStar||function(e,t){for(var n in e)"default"===n||Object.prototype.hasOwnProperty.call(t,n)||r(t,e,n)};Object.defineProperty(t,"__esModule",{value:!0}),t.KernelSpecAPI=t.KernelSpec=void 0;const s=o(n(1364));t.KernelSpec=s;const l=o(n(843));t.KernelSpecAPI=l,a(n(1366),t)},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.updateSession=t.startSession=t.getSessionModel=t.shutdownSession=t.getSessionUrl=t.listRunning=t.SESSION_SERVICE_URL=void 0;const r=n(129),i=n(48),o=n(1370);function a(e,n){return i.URLExt.join(e,t.SESSION_SERVICE_URL,n)}t.SESSION_SERVICE_URL="api/sessions",t.listRunning=async function(e=r.ServerConnection.makeSettings()){const n=i.URLExt.join(e.baseUrl,t.SESSION_SERVICE_URL),a=await r.ServerConnection.makeRequest(n,{},e);if(200!==a.status){throw await r.ServerConnection.ResponseError.create(a)}const s=await a.json();if(!Array.isArray(s))throw new Error("Invalid Session list");return s.forEach(e=>{o.updateLegacySessionModel(e),o.validateModel(e)}),s},t.getSessionUrl=a,t.shutdownSession=async function(e,t=r.ServerConnection.makeSettings()){var n;const i=a(t.baseUrl,e),o=await r.ServerConnection.makeRequest(i,{method:"DELETE"},t);if(404===o.status){const t=null!==(n=(await o.json()).message)&&void 0!==n?n:`The session "${e}"" does not exist on the server`;console.warn(t)}else{if(410===o.status)throw new r.ServerConnection.ResponseError(o,"The kernel was deleted but the session was not");if(204!==o.status){throw await r.ServerConnection.ResponseError.create(o)}}},t.getSessionModel=async function(e,t=r.ServerConnection.makeSettings()){const n=a(t.baseUrl,e),i=await r.ServerConnection.makeRequest(n,{},t);if(200!==i.status){throw await r.ServerConnection.ResponseError.create(i)}const s=await i.json();return o.updateLegacySessionModel(s),o.validateModel(s),s},t.startSession=async function(e,n=r.ServerConnection.makeSettings()){const a=i.URLExt.join(n.baseUrl,t.SESSION_SERVICE_URL),s={method:"POST",body:JSON.stringify(e)},l=await r.ServerConnection.makeRequest(a,s,n);if(201!==l.status){throw await r.ServerConnection.ResponseError.create(l)}const u=await l.json();return o.updateLegacySessionModel(u),o.validateModel(u),u},t.updateSession=async function(e,t=r.ServerConnection.makeSettings()){const n=a(t.baseUrl,e.id),i={method:"PATCH",body:JSON.stringify(e)},s=await r.ServerConnection.makeRequest(n,i,t);if(200!==s.status){throw await r.ServerConnection.ResponseError.create(s)}const l=await s.json();return o.updateLegacySessionModel(l),o.validateModel(l),l}},function(e,t,n){var r=n(1382),i=n(158);e.exports=function e(t,n,o,a,s){return t===n||(null==t||null==n||!i(t)&&!i(n)?t!=t&&n!=n:r(t,n,o,a,e,s))}},function(e,t,n){var r=n(207)(n(146),"Map");e.exports=r},function(e,t,n){var r=n(1399),i=n(1406),o=n(1408),a=n(1409),s=n(1410);function l(e){var t=-1,n=null==e?0:e.length;for(this.clear();++t<n;){var r=e[t];this.set(r[0],r[1])}}l.prototype.clear=r,l.prototype.delete=i,l.prototype.get=o,l.prototype.has=a,l.prototype.set=s,e.exports=l},function(e,t){e.exports=function(e){var t=-1,n=Array(e.size);return e.forEach((function(e){n[++t]=e})),n}},function(e,t){e.exports=function(e,t){for(var n=-1,r=t.length,i=e.length;++n<r;)e[i+n]=t[n];return e}},function(e,t,n){var r=n(858),i=n(859),o=Object.prototype.propertyIsEnumerable,a=Object.getOwnPropertySymbols,s=a?function(e){return null==e?[]:(e=Object(e),r(a(e),(function(t){return o.call(e,t)})))}:i;e.exports=s},function(e,t){e.exports=function(e){return"number"==typeof e&&e>-1&&e%1==0&&e<=9007199254740991}},function(e,t,n){(function(e){var r=n(850),i=t&&!t.nodeType&&t,o=i&&"object"==typeof e&&e&&!e.nodeType&&e,a=o&&o.exports===i&&r.process,s=function(){try{var e=o&&o.require&&o.require("util").types;return e||a&&a.binding&&a.binding("util")}catch(e){}}();e.exports=s}).call(this,n(188)(e))},function(e,t,n){var r=n(363),i=n(1421),o=Object.prototype.hasOwnProperty;e.exports=function(e){if(!r(e))return i(e);var t=[];for(var n in Object(e))o.call(e,n)&&"constructor"!=n&&t.push(n);return t}},function(e,t,n){!function(e){"use strict";e.modeInfo=[{name:"APL",mime:"text/apl",mode:"apl",ext:["dyalog","apl"]},{name:"PGP",mimes:["application/pgp","application/pgp-encrypted","application/pgp-keys","application/pgp-signature"],mode:"asciiarmor",ext:["asc","pgp","sig"]},{name:"ASN.1",mime:"text/x-ttcn-asn",mode:"asn.1",ext:["asn","asn1"]},{name:"Asterisk",mime:"text/x-asterisk",mode:"asterisk",file:/^extensions\.conf$/i},{name:"Brainfuck",mime:"text/x-brainfuck",mode:"brainfuck",ext:["b","bf"]},{name:"C",mime:"text/x-csrc",mode:"clike",ext:["c","h","ino"]},{name:"C++",mime:"text/x-c++src",mode:"clike",ext:["cpp","c++","cc","cxx","hpp","h++","hh","hxx"],alias:["cpp"]},{name:"Cobol",mime:"text/x-cobol",mode:"cobol",ext:["cob","cpy"]},{name:"C#",mime:"text/x-csharp",mode:"clike",ext:["cs"],alias:["csharp","cs"]},{name:"Clojure",mime:"text/x-clojure",mode:"clojure",ext:["clj","cljc","cljx"]},{name:"ClojureScript",mime:"text/x-clojurescript",mode:"clojure",ext:["cljs"]},{name:"Closure Stylesheets (GSS)",mime:"text/x-gss",mode:"css",ext:["gss"]},{name:"CMake",mime:"text/x-cmake",mode:"cmake",ext:["cmake","cmake.in"],file:/^CMakeLists\.txt$/},{name:"CoffeeScript",mimes:["application/vnd.coffeescript","text/coffeescript","text/x-coffeescript"],mode:"coffeescript",ext:["coffee"],alias:["coffee","coffee-script"]},{name:"Common Lisp",mime:"text/x-common-lisp",mode:"commonlisp",ext:["cl","lisp","el"],alias:["lisp"]},{name:"Cypher",mime:"application/x-cypher-query",mode:"cypher",ext:["cyp","cypher"]},{name:"Cython",mime:"text/x-cython",mode:"python",ext:["pyx","pxd","pxi"]},{name:"Crystal",mime:"text/x-crystal",mode:"crystal",ext:["cr"]},{name:"CSS",mime:"text/css",mode:"css",ext:["css"]},{name:"CQL",mime:"text/x-cassandra",mode:"sql",ext:["cql"]},{name:"D",mime:"text/x-d",mode:"d",ext:["d"]},{name:"Dart",mimes:["application/dart","text/x-dart"],mode:"dart",ext:["dart"]},{name:"diff",mime:"text/x-diff",mode:"diff",ext:["diff","patch"]},{name:"Django",mime:"text/x-django",mode:"django"},{name:"Dockerfile",mime:"text/x-dockerfile",mode:"dockerfile",file:/^Dockerfile$/},{name:"DTD",mime:"application/xml-dtd",mode:"dtd",ext:["dtd"]},{name:"Dylan",mime:"text/x-dylan",mode:"dylan",ext:["dylan","dyl","intr"]},{name:"EBNF",mime:"text/x-ebnf",mode:"ebnf"},{name:"ECL",mime:"text/x-ecl",mode:"ecl",ext:["ecl"]},{name:"edn",mime:"application/edn",mode:"clojure",ext:["edn"]},{name:"Eiffel",mime:"text/x-eiffel",mode:"eiffel",ext:["e"]},{name:"Elm",mime:"text/x-elm",mode:"elm",ext:["elm"]},{name:"Embedded Javascript",mime:"application/x-ejs",mode:"htmlembedded",ext:["ejs"]},{name:"Embedded Ruby",mime:"application/x-erb",mode:"htmlembedded",ext:["erb"]},{name:"Erlang",mime:"text/x-erlang",mode:"erlang",ext:["erl"]},{name:"Esper",mime:"text/x-esper",mode:"sql"},{name:"Factor",mime:"text/x-factor",mode:"factor",ext:["factor"]},{name:"FCL",mime:"text/x-fcl",mode:"fcl"},{name:"Forth",mime:"text/x-forth",mode:"forth",ext:["forth","fth","4th"]},{name:"Fortran",mime:"text/x-fortran",mode:"fortran",ext:["f","for","f77","f90","f95"]},{name:"F#",mime:"text/x-fsharp",mode:"mllike",ext:["fs"],alias:["fsharp"]},{name:"Gas",mime:"text/x-gas",mode:"gas",ext:["s"]},{name:"Gherkin",mime:"text/x-feature",mode:"gherkin",ext:["feature"]},{name:"GitHub Flavored Markdown",mime:"text/x-gfm",mode:"gfm",file:/^(readme|contributing|history)\.md$/i},{name:"Go",mime:"text/x-go",mode:"go",ext:["go"]},{name:"Groovy",mime:"text/x-groovy",mode:"groovy",ext:["groovy","gradle"],file:/^Jenkinsfile$/},{name:"HAML",mime:"text/x-haml",mode:"haml",ext:["haml"]},{name:"Haskell",mime:"text/x-haskell",mode:"haskell",ext:["hs"]},{name:"Haskell (Literate)",mime:"text/x-literate-haskell",mode:"haskell-literate",ext:["lhs"]},{name:"Haxe",mime:"text/x-haxe",mode:"haxe",ext:["hx"]},{name:"HXML",mime:"text/x-hxml",mode:"haxe",ext:["hxml"]},{name:"ASP.NET",mime:"application/x-aspx",mode:"htmlembedded",ext:["aspx"],alias:["asp","aspx"]},{name:"HTML",mime:"text/html",mode:"htmlmixed",ext:["html","htm","handlebars","hbs"],alias:["xhtml"]},{name:"HTTP",mime:"message/http",mode:"http"},{name:"IDL",mime:"text/x-idl",mode:"idl",ext:["pro"]},{name:"Pug",mime:"text/x-pug",mode:"pug",ext:["jade","pug"],alias:["jade"]},{name:"Java",mime:"text/x-java",mode:"clike",ext:["java"]},{name:"Java Server Pages",mime:"application/x-jsp",mode:"htmlembedded",ext:["jsp"],alias:["jsp"]},{name:"JavaScript",mimes:["text/javascript","text/ecmascript","application/javascript","application/x-javascript","application/ecmascript"],mode:"javascript",ext:["js"],alias:["ecmascript","js","node"]},{name:"JSON",mimes:["application/json","application/x-json"],mode:"javascript",ext:["json","map"],alias:["json5"]},{name:"JSON-LD",mime:"application/ld+json",mode:"javascript",ext:["jsonld"],alias:["jsonld"]},{name:"JSX",mime:"text/jsx",mode:"jsx",ext:["jsx"]},{name:"Jinja2",mime:"text/jinja2",mode:"jinja2",ext:["j2","jinja","jinja2"]},{name:"Julia",mime:"text/x-julia",mode:"julia",ext:["jl"]},{name:"Kotlin",mime:"text/x-kotlin",mode:"clike",ext:["kt"]},{name:"LESS",mime:"text/x-less",mode:"css",ext:["less"]},{name:"LiveScript",mime:"text/x-livescript",mode:"livescript",ext:["ls"],alias:["ls"]},{name:"Lua",mime:"text/x-lua",mode:"lua",ext:["lua"]},{name:"Markdown",mime:"text/x-markdown",mode:"markdown",ext:["markdown","md","mkd"]},{name:"mIRC",mime:"text/mirc",mode:"mirc"},{name:"MariaDB SQL",mime:"text/x-mariadb",mode:"sql"},{name:"Mathematica",mime:"text/x-mathematica",mode:"mathematica",ext:["m","nb","wl","wls"]},{name:"Modelica",mime:"text/x-modelica",mode:"modelica",ext:["mo"]},{name:"MUMPS",mime:"text/x-mumps",mode:"mumps",ext:["mps"]},{name:"MS SQL",mime:"text/x-mssql",mode:"sql"},{name:"mbox",mime:"application/mbox",mode:"mbox",ext:["mbox"]},{name:"MySQL",mime:"text/x-mysql",mode:"sql"},{name:"Nginx",mime:"text/x-nginx-conf",mode:"nginx",file:/nginx.*\.conf$/i},{name:"NSIS",mime:"text/x-nsis",mode:"nsis",ext:["nsh","nsi"]},{name:"NTriples",mimes:["application/n-triples","application/n-quads","text/n-triples"],mode:"ntriples",ext:["nt","nq"]},{name:"Objective-C",mime:"text/x-objectivec",mode:"clike",ext:["m"],alias:["objective-c","objc"]},{name:"Objective-C++",mime:"text/x-objectivec++",mode:"clike",ext:["mm"],alias:["objective-c++","objc++"]},{name:"OCaml",mime:"text/x-ocaml",mode:"mllike",ext:["ml","mli","mll","mly"]},{name:"Octave",mime:"text/x-octave",mode:"octave",ext:["m"]},{name:"Oz",mime:"text/x-oz",mode:"oz",ext:["oz"]},{name:"Pascal",mime:"text/x-pascal",mode:"pascal",ext:["p","pas"]},{name:"PEG.js",mime:"null",mode:"pegjs",ext:["jsonld"]},{name:"Perl",mime:"text/x-perl",mode:"perl",ext:["pl","pm"]},{name:"PHP",mimes:["text/x-php","application/x-httpd-php","application/x-httpd-php-open"],mode:"php",ext:["php","php3","php4","php5","php7","phtml"]},{name:"Pig",mime:"text/x-pig",mode:"pig",ext:["pig"]},{name:"Plain Text",mime:"text/plain",mode:"null",ext:["txt","text","conf","def","list","log"]},{name:"PLSQL",mime:"text/x-plsql",mode:"sql",ext:["pls"]},{name:"PostgreSQL",mime:"text/x-pgsql",mode:"sql"},{name:"PowerShell",mime:"application/x-powershell",mode:"powershell",ext:["ps1","psd1","psm1"]},{name:"Properties files",mime:"text/x-properties",mode:"properties",ext:["properties","ini","in"],alias:["ini","properties"]},{name:"ProtoBuf",mime:"text/x-protobuf",mode:"protobuf",ext:["proto"]},{name:"Python",mime:"text/x-python",mode:"python",ext:["BUILD","bzl","py","pyw"],file:/^(BUCK|BUILD)$/},{name:"Puppet",mime:"text/x-puppet",mode:"puppet",ext:["pp"]},{name:"Q",mime:"text/x-q",mode:"q",ext:["q"]},{name:"R",mime:"text/x-rsrc",mode:"r",ext:["r","R"],alias:["rscript"]},{name:"reStructuredText",mime:"text/x-rst",mode:"rst",ext:["rst"],alias:["rst"]},{name:"RPM Changes",mime:"text/x-rpm-changes",mode:"rpm"},{name:"RPM Spec",mime:"text/x-rpm-spec",mode:"rpm",ext:["spec"]},{name:"Ruby",mime:"text/x-ruby",mode:"ruby",ext:["rb"],alias:["jruby","macruby","rake","rb","rbx"]},{name:"Rust",mime:"text/x-rustsrc",mode:"rust",ext:["rs"]},{name:"SAS",mime:"text/x-sas",mode:"sas",ext:["sas"]},{name:"Sass",mime:"text/x-sass",mode:"sass",ext:["sass"]},{name:"Scala",mime:"text/x-scala",mode:"clike",ext:["scala"]},{name:"Scheme",mime:"text/x-scheme",mode:"scheme",ext:["scm","ss"]},{name:"SCSS",mime:"text/x-scss",mode:"css",ext:["scss"]},{name:"Shell",mimes:["text/x-sh","application/x-sh"],mode:"shell",ext:["sh","ksh","bash"],alias:["bash","sh","zsh"],file:/^PKGBUILD$/},{name:"Sieve",mime:"application/sieve",mode:"sieve",ext:["siv","sieve"]},{name:"Slim",mimes:["text/x-slim","application/x-slim"],mode:"slim",ext:["slim"]},{name:"Smalltalk",mime:"text/x-stsrc",mode:"smalltalk",ext:["st"]},{name:"Smarty",mime:"text/x-smarty",mode:"smarty",ext:["tpl"]},{name:"Solr",mime:"text/x-solr",mode:"solr"},{name:"SML",mime:"text/x-sml",mode:"mllike",ext:["sml","sig","fun","smackspec"]},{name:"Soy",mime:"text/x-soy",mode:"soy",ext:["soy"],alias:["closure template"]},{name:"SPARQL",mime:"application/sparql-query",mode:"sparql",ext:["rq","sparql"],alias:["sparul"]},{name:"Spreadsheet",mime:"text/x-spreadsheet",mode:"spreadsheet",alias:["excel","formula"]},{name:"SQL",mime:"text/x-sql",mode:"sql",ext:["sql"]},{name:"SQLite",mime:"text/x-sqlite",mode:"sql"},{name:"Squirrel",mime:"text/x-squirrel",mode:"clike",ext:["nut"]},{name:"Stylus",mime:"text/x-styl",mode:"stylus",ext:["styl"]},{name:"Swift",mime:"text/x-swift",mode:"swift",ext:["swift"]},{name:"sTeX",mime:"text/x-stex",mode:"stex"},{name:"LaTeX",mime:"text/x-latex",mode:"stex",ext:["text","ltx","tex"],alias:["tex"]},{name:"SystemVerilog",mime:"text/x-systemverilog",mode:"verilog",ext:["v","sv","svh"]},{name:"Tcl",mime:"text/x-tcl",mode:"tcl",ext:["tcl"]},{name:"Textile",mime:"text/x-textile",mode:"textile",ext:["textile"]},{name:"TiddlyWiki",mime:"text/x-tiddlywiki",mode:"tiddlywiki"},{name:"Tiki wiki",mime:"text/tiki",mode:"tiki"},{name:"TOML",mime:"text/x-toml",mode:"toml",ext:["toml"]},{name:"Tornado",mime:"text/x-tornado",mode:"tornado"},{name:"troff",mime:"text/troff",mode:"troff",ext:["1","2","3","4","5","6","7","8","9"]},{name:"TTCN",mime:"text/x-ttcn",mode:"ttcn",ext:["ttcn","ttcn3","ttcnpp"]},{name:"TTCN_CFG",mime:"text/x-ttcn-cfg",mode:"ttcn-cfg",ext:["cfg"]},{name:"Turtle",mime:"text/turtle",mode:"turtle",ext:["ttl"]},{name:"TypeScript",mime:"application/typescript",mode:"javascript",ext:["ts"],alias:["ts"]},{name:"TypeScript-JSX",mime:"text/typescript-jsx",mode:"jsx",ext:["tsx"],alias:["tsx"]},{name:"Twig",mime:"text/x-twig",mode:"twig"},{name:"Web IDL",mime:"text/x-webidl",mode:"webidl",ext:["webidl"]},{name:"VB.NET",mime:"text/x-vb",mode:"vb",ext:["vb"]},{name:"VBScript",mime:"text/vbscript",mode:"vbscript",ext:["vbs"]},{name:"Velocity",mime:"text/velocity",mode:"velocity",ext:["vtl"]},{name:"Verilog",mime:"text/x-verilog",mode:"verilog",ext:["v"]},{name:"VHDL",mime:"text/x-vhdl",mode:"vhdl",ext:["vhd","vhdl"]},{name:"Vue.js Component",mimes:["script/x-vue","text/x-vue"],mode:"vue",ext:["vue"]},{name:"XML",mimes:["application/xml","text/xml"],mode:"xml",ext:["xml","xsl","xsd","svg"],alias:["rss","wsdl","xsd"]},{name:"XQuery",mime:"application/xquery",mode:"xquery",ext:["xy","xquery"]},{name:"Yacas",mime:"text/x-yacas",mode:"yacas",ext:["ys"]},{name:"YAML",mimes:["text/x-yaml","text/yaml"],mode:"yaml",ext:["yaml","yml"],alias:["yml"]},{name:"Z80",mime:"text/x-z80",mode:"z80",ext:["z80"]},{name:"mscgen",mime:"text/x-mscgen",mode:"mscgen",ext:["mscgen","mscin","msc"]},{name:"xu",mime:"text/x-xu",mode:"mscgen",ext:["xu"]},{name:"msgenny",mime:"text/x-msgenny",mode:"mscgen",ext:["msgenny"]}];for(var t=0;t<e.modeInfo.length;t++){var n=e.modeInfo[t];n.mimes&&(n.mime=n.mimes[0])}e.findModeByMIME=function(t){t=t.toLowerCase();for(var n=0;n<e.modeInfo.length;n++){var r=e.modeInfo[n];if(r.mime==t)return r;if(r.mimes)for(var i=0;i<r.mimes.length;i++)if(r.mimes[i]==t)return r}return/\+xml$/.test(t)?e.findModeByMIME("application/xml"):/\+json$/.test(t)?e.findModeByMIME("application/json"):void 0},e.findModeByExtension=function(t){t=t.toLowerCase();for(var n=0;n<e.modeInfo.length;n++){var r=e.modeInfo[n];if(r.ext)for(var i=0;i<r.ext.length;i++)if(r.ext[i]==t)return r}},e.findModeByFileName=function(t){for(var n=0;n<e.modeInfo.length;n++){var r=e.modeInfo[n];if(r.file&&r.file.test(t))return r}var i=t.lastIndexOf("."),o=i>-1&&t.substring(i+1,t.length);if(o)return e.findModeByExtension(o)},e.findModeByName=function(t){t=t.toLowerCase();for(var n=0;n<e.modeInfo.length;n++){var r=e.modeInfo[n];if(r.name.toLowerCase()==t)return r;if(r.alias)for(var i=0;i<r.alias.length;i++)if(r.alias[i].toLowerCase()==t)return r}}}(n(24))},function(e,t,n){!function(e){"use strict";var t,n,r=e.Pos;function i(e,t){for(var n=function(e){var t=e.flags;return null!=t?t:(e.ignoreCase?"i":"")+(e.global?"g":"")+(e.multiline?"m":"")}(e),r=n,i=0;i<t.length;i++)-1==r.indexOf(t.charAt(i))&&(r+=t.charAt(i));return n==r?e:new RegExp(e.source,r)}function o(e){return/\\s|\\n|\n|\\W|\\D|\[\^/.test(e.source)}function a(e,t,n){t=i(t,"g");for(var o=n.line,a=n.ch,s=e.lastLine();o<=s;o++,a=0){t.lastIndex=a;var l=e.getLine(o),u=t.exec(l);if(u)return{from:r(o,u.index),to:r(o,u.index+u[0].length),match:u}}}function s(e,t,n){if(!o(t))return a(e,t,n);t=i(t,"gm");for(var s,l=1,u=n.line,c=e.lastLine();u<=c;){for(var d=0;d<l&&!(u>c);d++){var h=e.getLine(u++);s=null==s?h:s+"\n"+h}l*=2,t.lastIndex=n.ch;var f=t.exec(s);if(f){var p=s.slice(0,f.index).split("\n"),g=f[0].split("\n"),v=n.line+p.length-1,m=p[p.length-1].length;return{from:r(v,m),to:r(v+g.length-1,1==g.length?m+g[0].length:g[g.length-1].length),match:f}}}}function l(e,t,n){for(var r,i=0;i<=e.length;){t.lastIndex=i;var o=t.exec(e);if(!o)break;var a=o.index+o[0].length;if(a>e.length-n)break;(!r||a>r.index+r[0].length)&&(r=o),i=o.index+1}return r}function u(e,t,n){t=i(t,"g");for(var o=n.line,a=n.ch,s=e.firstLine();o>=s;o--,a=-1){var u=e.getLine(o),c=l(u,t,a<0?0:u.length-a);if(c)return{from:r(o,c.index),to:r(o,c.index+c[0].length),match:c}}}function c(e,t,n){if(!o(t))return u(e,t,n);t=i(t,"gm");for(var a,s=1,c=e.getLine(n.line).length-n.ch,d=n.line,h=e.firstLine();d>=h;){for(var f=0;f<s&&d>=h;f++){var p=e.getLine(d--);a=null==a?p:p+"\n"+a}s*=2;var g=l(a,t,c);if(g){var v=a.slice(0,g.index).split("\n"),m=g[0].split("\n"),y=d+v.length,b=v[v.length-1].length;return{from:r(y,b),to:r(y+m.length-1,1==m.length?b+m[0].length:m[m.length-1].length),match:g}}}}function d(e,t,n,r){if(e.length==t.length)return n;for(var i=0,o=n+Math.max(0,e.length-t.length);;){if(i==o)return i;var a=i+o>>1,s=r(e.slice(0,a)).length;if(s==n)return a;s>n?o=a:i=a+1}}function h(e,i,o,a){if(!i.length)return null;var s=a?t:n,l=s(i).split(/\r|\n\r?/);e:for(var u=o.line,c=o.ch,h=e.lastLine()+1-l.length;u<=h;u++,c=0){var f=e.getLine(u).slice(c),p=s(f);if(1==l.length){var g=p.indexOf(l[0]);if(-1==g)continue e;return o=d(f,p,g,s)+c,{from:r(u,d(f,p,g,s)+c),to:r(u,d(f,p,g+l[0].length,s)+c)}}var v=p.length-l[0].length;if(p.slice(v)==l[0]){for(var m=1;m<l.length-1;m++)if(s(e.getLine(u+m))!=l[m])continue e;var y=e.getLine(u+l.length-1),b=s(y),C=l[l.length-1];if(b.slice(0,C.length)==C)return{from:r(u,d(f,p,v,s)+c),to:r(u+l.length-1,d(y,b,C.length,s))}}}}function f(e,i,o,a){if(!i.length)return null;var s=a?t:n,l=s(i).split(/\r|\n\r?/);e:for(var u=o.line,c=o.ch,h=e.firstLine()-1+l.length;u>=h;u--,c=-1){var f=e.getLine(u);c>-1&&(f=f.slice(0,c));var p=s(f);if(1==l.length){var g=p.lastIndexOf(l[0]);if(-1==g)continue e;return{from:r(u,d(f,p,g,s)),to:r(u,d(f,p,g+l[0].length,s))}}var v=l[l.length-1];if(p.slice(0,v.length)==v){var m=1;for(o=u-l.length+1;m<l.length-1;m++)if(s(e.getLine(o+m))!=l[m])continue e;var y=e.getLine(u+1-l.length),b=s(y);if(b.slice(b.length-l[0].length)==l[0])return{from:r(u+1-l.length,d(y,b,y.length-l[0].length,s)),to:r(u,d(f,p,v.length,s))}}}}function p(e,t,n,o){var l;this.atOccurrence=!1,this.doc=e,n=n?e.clipPos(n):r(0,0),this.pos={from:n,to:n},"object"==typeof o?l=o.caseFold:(l=o,o=null),"string"==typeof t?(null==l&&(l=!1),this.matches=function(n,r){return(n?f:h)(e,t,r,l)}):(t=i(t,"gm"),o&&!1===o.multiline?this.matches=function(n,r){return(n?u:a)(e,t,r)}:this.matches=function(n,r){return(n?c:s)(e,t,r)})}String.prototype.normalize?(t=function(e){return e.normalize("NFD").toLowerCase()},n=function(e){return e.normalize("NFD")}):(t=function(e){return e.toLowerCase()},n=function(e){return e}),p.prototype={findNext:function(){return this.find(!1)},findPrevious:function(){return this.find(!0)},find:function(t){for(var n=this.matches(t,this.doc.clipPos(t?this.pos.from:this.pos.to));n&&0==e.cmpPos(n.from,n.to);)t?n.from.ch?n.from=r(n.from.line,n.from.ch-1):n=n.from.line==this.doc.firstLine()?null:this.matches(t,this.doc.clipPos(r(n.from.line-1))):n.to.ch<this.doc.getLine(n.to.line).length?n.to=r(n.to.line,n.to.ch+1):n=n.to.line==this.doc.lastLine()?null:this.matches(t,r(n.to.line+1,0));if(n)return this.pos=n,this.atOccurrence=!0,this.pos.match||!0;var i=r(t?this.doc.firstLine():this.doc.lastLine()+1,0);return this.pos={from:i,to:i},this.atOccurrence=!1},from:function(){if(this.atOccurrence)return this.pos.from},to:function(){if(this.atOccurrence)return this.pos.to},replace:function(t,n){if(this.atOccurrence){var i=e.splitLines(t);this.doc.replaceRange(i,this.pos.from,this.pos.to,n),this.pos.to=r(this.pos.from.line+i.length-1,i[i.length-1].length+(1==i.length?this.pos.from.ch:0))}}},e.defineExtension("getSearchCursor",(function(e,t,n){return new p(this.doc,e,t,n)})),e.defineDocExtension("getSearchCursor",(function(e,t,n){return new p(this,e,t,n)})),e.defineExtension("selectMatches",(function(t,n){for(var r=[],i=this.getSearchCursor(t,this.getCursor("from"),n);i.findNext()&&!(e.cmpPos(i.to(),this.getCursor("to"))>0);)r.push({anchor:i.from(),head:i.to()});r.length&&this.setSelections(r,0)}))}(n(24))},function(e,t,n){"use strict";function r(e){return null!=e&&void 0!==e.current}function i(e){return"function"==typeof e}Object.defineProperty(t,"__esModule",{value:!0}),t.refHandler=t.setRef=t.getRef=t.combineRefs=t.isRefCallback=t.isRefObject=void 0,t.isRefObject=r,t.isRefCallback=i,t.combineRefs=function(e,t){return function(n){e(n),t(n)}},t.getRef=function(e){var t;return null===e?null:null!==(t=e.current)&&void 0!==t?t:e},t.setRef=function(e,t){void 0!==e&&(r(e)?e.current=t:i(e)&&e(t))},t.refHandler=function(e,t,n){return r(n)?(e[t]=n,n):function(r){e[t]=r,i(n)&&n(r)}}},function(e,t,n){"use strict";function r(e){return"function"==typeof e}Object.defineProperty(t,"__esModule",{value:!0}),t.safeInvokeOrValue=t.safeInvoke=t.isFunction=void 0,t.isFunction=r,t.safeInvoke=function(e){for(var t=[],n=1;n<arguments.length;n++)t[n-1]=arguments[n];if(r(e))return e.apply(void 0,t)},t.safeInvokeOrValue=function(e){for(var t=[],n=1;n<arguments.length;n++)t[n-1]=arguments[n];return r(e)?e.apply(void 0,t):e}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.TOASTER_MAX_TOASTS_INVALID=t.DRAWER_ANGLE_POSITIONS_ARE_CASTED=t.DRAWER_VERTICAL_IS_IGNORED=t.DIALOG_WARN_NO_HEADER_CLOSE_BUTTON=t.DIALOG_WARN_NO_HEADER_ICON=t.TOASTER_WARN_INLINE=t.TOASTER_CREATE_NULL=t.SPINNER_WARN_CLASSES_SIZE=t.MULTISLIDER_WARN_LABEL_STEP_SIZE_LABEL_VALUES_MUTEX=t.MULTISLIDER_INVALID_CHILD=t.RANGESLIDER_NULL_VALUE=t.SLIDER_ZERO_LABEL_STEP=t.SLIDER_ZERO_STEP=t.RADIOGROUP_WARN_CHILDREN_OPTIONS_MUTEX=t.PORTAL_CONTEXT_CLASS_NAME_STRING=t.POPOVER_WARN_UNCONTROLLED_ONINTERACTION=t.POPOVER_WARN_HAS_BACKDROP_INLINE=t.POPOVER_WARN_EMPTY_CONTENT=t.POPOVER_WARN_DOUBLE_TARGET=t.POPOVER_WARN_DOUBLE_CONTENT=t.POPOVER_WARN_TOO_MANY_CHILDREN=t.POPOVER_HAS_BACKDROP_INTERACTION=t.POPOVER_REQUIRES_TARGET=t.OVERFLOW_LIST_OBSERVE_PARENTS_CHANGED=t.PANEL_STACK_REQUIRES_PANEL=t.PANEL_STACK_INITIAL_PANEL_STACK_MUTEX=t.NUMERIC_INPUT_CONTROLLED_VALUE_INVALID=t.NUMERIC_INPUT_STEP_SIZE_NON_POSITIVE=t.NUMERIC_INPUT_MAJOR_STEP_SIZE_NON_POSITIVE=t.NUMERIC_INPUT_MINOR_STEP_SIZE_NON_POSITIVE=t.NUMERIC_INPUT_MAJOR_STEP_SIZE_BOUND=t.NUMERIC_INPUT_MINOR_STEP_SIZE_BOUND=t.NUMERIC_INPUT_MIN_MAX=t.INPUT_WARN_LEFT_ELEMENT_LEFT_ICON_MUTEX=t.HOTKEYS_WARN_DECORATOR_NEEDS_REACT_ELEMENT=t.HOTKEYS_WARN_DECORATOR_NO_METHOD=t.HOTKEYS_HOTKEY_CHILDREN=t.CONTEXTMENU_WARN_DECORATOR_NEEDS_REACT_ELEMENT=t.CONTEXTMENU_WARN_DECORATOR_NO_METHOD=t.COLLAPSIBLE_LIST_INVALID_CHILD=t.ALERT_WARN_CANCEL_OUTSIDE_CLICK=t.ALERT_WARN_CANCEL_ESCAPE_KEY=t.ALERT_WARN_CANCEL_PROPS=t.CLAMP_MIN_MAX=void 0;var r="[Blueprint]";t.CLAMP_MIN_MAX=r+" clamp: max cannot be less than min",t.ALERT_WARN_CANCEL_PROPS=r+" <Alert> cancelButtonText and onCancel should be set together.",t.ALERT_WARN_CANCEL_ESCAPE_KEY=r+" <Alert> canEscapeKeyCancel enabled without onCancel or onClose handler.",t.ALERT_WARN_CANCEL_OUTSIDE_CLICK=r+" <Alert> canOutsideClickCancel enbaled without onCancel or onClose handler.",t.COLLAPSIBLE_LIST_INVALID_CHILD=r+" <CollapsibleList> children must be <MenuItem>s",t.CONTEXTMENU_WARN_DECORATOR_NO_METHOD=r+" @ContextMenuTarget-decorated class should implement renderContextMenu.",t.CONTEXTMENU_WARN_DECORATOR_NEEDS_REACT_ELEMENT=r+' "@ContextMenuTarget-decorated components must return a single JSX.Element or an empty render.',t.HOTKEYS_HOTKEY_CHILDREN=r+" <Hotkeys> only accepts <Hotkey> children.",t.HOTKEYS_WARN_DECORATOR_NO_METHOD=r+" @HotkeysTarget-decorated class should implement renderHotkeys.",t.HOTKEYS_WARN_DECORATOR_NEEDS_REACT_ELEMENT=r+' "@HotkeysTarget-decorated components must return a single JSX.Element or an empty render.',t.INPUT_WARN_LEFT_ELEMENT_LEFT_ICON_MUTEX=r+" <InputGroup> leftElement and leftIcon prop are mutually exclusive, with leftElement taking priority.",t.NUMERIC_INPUT_MIN_MAX=r+" <NumericInput> requires min to be no greater than max if both are defined.",t.NUMERIC_INPUT_MINOR_STEP_SIZE_BOUND=r+" <NumericInput> requires minorStepSize to be no greater than stepSize.",t.NUMERIC_INPUT_MAJOR_STEP_SIZE_BOUND=r+" <NumericInput> requires stepSize to be no greater than majorStepSize.",t.NUMERIC_INPUT_MINOR_STEP_SIZE_NON_POSITIVE=r+" <NumericInput> requires minorStepSize to be strictly greater than zero.",t.NUMERIC_INPUT_MAJOR_STEP_SIZE_NON_POSITIVE=r+" <NumericInput> requires majorStepSize to be strictly greater than zero.",t.NUMERIC_INPUT_STEP_SIZE_NON_POSITIVE=r+" <NumericInput> requires stepSize to be strictly greater than zero.",t.NUMERIC_INPUT_CONTROLLED_VALUE_INVALID=r+" <NumericInput> controlled value prop does not adhere to stepSize, min, and/or max constraints.",t.PANEL_STACK_INITIAL_PANEL_STACK_MUTEX=r+" <PanelStack> requires exactly one of initialPanel and stack prop",t.PANEL_STACK_REQUIRES_PANEL=r+" <PanelStack> requires at least one panel in the stack",t.OVERFLOW_LIST_OBSERVE_PARENTS_CHANGED=r+" <OverflowList> does not support changing observeParents after mounting.",t.POPOVER_REQUIRES_TARGET=r+" <Popover> requires target prop or at least one child element.",t.POPOVER_HAS_BACKDROP_INTERACTION=r+" <Popover hasBackdrop={true}> requires interactionKind={PopoverInteractionKind.CLICK}.",t.POPOVER_WARN_TOO_MANY_CHILDREN=r+" <Popover> supports one or two children; additional children are ignored. First child is the target, second child is the content. You may instead supply these two as props.",t.POPOVER_WARN_DOUBLE_CONTENT=r+" <Popover> with two children ignores content prop; use either prop or children.",t.POPOVER_WARN_DOUBLE_TARGET=r+" <Popover> with children ignores target prop; use either prop or children.",t.POPOVER_WARN_EMPTY_CONTENT=r+" Disabling <Popover> with empty/whitespace content...",t.POPOVER_WARN_HAS_BACKDROP_INLINE=r+" <Popover usePortal={false}> ignores hasBackdrop",t.POPOVER_WARN_UNCONTROLLED_ONINTERACTION=r+" <Popover> onInteraction is ignored when uncontrolled.",t.PORTAL_CONTEXT_CLASS_NAME_STRING=r+" <Portal> context blueprintPortalClassName must be string",t.RADIOGROUP_WARN_CHILDREN_OPTIONS_MUTEX=r+" <RadioGroup> children and options prop are mutually exclusive, with options taking priority.",t.SLIDER_ZERO_STEP=r+" <Slider> stepSize must be greater than zero.",t.SLIDER_ZERO_LABEL_STEP=r+" <Slider> labelStepSize must be greater than zero.",t.RANGESLIDER_NULL_VALUE=r+" <RangeSlider> value prop must be an array of two non-null numbers.",t.MULTISLIDER_INVALID_CHILD=r+" <MultiSlider> children must be <SliderHandle>s or <SliderTrackStop>s",t.MULTISLIDER_WARN_LABEL_STEP_SIZE_LABEL_VALUES_MUTEX=r+" <MultiSlider> labelStepSize and labelValues prop are mutually exclusive, with labelStepSize taking priority.",t.SPINNER_WARN_CLASSES_SIZE=r+" <Spinner> Classes.SMALL/LARGE are ignored if size prop is set.",t.TOASTER_CREATE_NULL=r+" Toaster.create() is not supported inside React lifecycle methods in React 16. See usage example on the docs site.",t.TOASTER_WARN_INLINE=r+" Toaster.create() ignores inline prop as it always creates a new element.",t.DIALOG_WARN_NO_HEADER_ICON=r+" <Dialog> iconName is ignored if title is omitted.",t.DIALOG_WARN_NO_HEADER_CLOSE_BUTTON=r+" <Dialog> isCloseButtonShown prop is ignored if title is omitted.",t.DRAWER_VERTICAL_IS_IGNORED=r+" <Drawer> vertical is ignored if position is defined",t.DRAWER_ANGLE_POSITIONS_ARE_CASTED=r+" <Drawer> all angle positions are casted into pure position (TOP, BOTTOM, LEFT or RIGHT)",t.TOASTER_MAX_TOASTS_INVALID=r+" <Toaster> maxToasts is set to an invalid number, must be greater than 0"},function(e,t,n){"use strict";n.d(t,"a",(function(){return r})),n.d(t,"b",(function(){return i}));var r={add:["M10.99 6.99h-2v-2c0-.55-.45-1-1-1s-1 .45-1 1v2h-2c-.55 0-1 .45-1 1s.45 1 1 1h2v2c0 .55.45 1 1 1s1-.45 1-1v-2h2c.55 0 1-.45 1-1s-.45-1-1-1zm-3-7c-4.42 0-8 3.58-8 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm0 14c-3.31 0-6-2.69-6-6s2.69-6 6-6 6 2.69 6 6-2.68 6-6 6z"],"add-column-left":["M15 0H1C.45 0 0 .45 0 1v14c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-5 14H2V2h8v12zm4 0h-3V2h3v12zM4 9h1v1c0 .55.45 1 1 1s1-.45 1-1V9h1c.55 0 1-.45 1-1s-.45-1-1-1H7V6c0-.55-.45-1-1-1s-1 .45-1 1v1H4c-.55 0-1 .45-1 1s.45 1 1 1z"],"add-column-right":["M8 9h1v1c0 .55.45 1 1 1s1-.45 1-1V9h1c.55 0 1-.45 1-1s-.45-1-1-1h-1V6c0-.55-.45-1-1-1s-1 .45-1 1v1H8c-.55 0-1 .45-1 1s.45 1 1 1zm7-9H1C.45 0 0 .45 0 1v14c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zM5 14H2V2h3v12zm9 0H6V2h8v12z"],"add-row-bottom":["M6 11h1v1c0 .55.45 1 1 1s1-.45 1-1v-1h1c.55 0 1-.45 1-1s-.45-1-1-1H9V8c0-.55-.45-1-1-1s-1 .45-1 1v1H6c-.55 0-1 .45-1 1s.45 1 1 1zm9-11H1C.45 0 0 .45 0 1v14c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 14H2V6h12v8zm0-9H2V2h12v3z"],"add-row-top":["M15 0H1C.45 0 0 .45 0 1v14c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 14H2v-3h12v3zm0-4H2V2h12v8zM6 7h1v1c0 .55.45 1 1 1s1-.45 1-1V7h1c.55 0 1-.45 1-1s-.45-1-1-1H9V4c0-.55-.45-1-1-1s-1 .45-1 1v1H6c-.55 0-1 .45-1 1s.45 1 1 1z"],"add-to-artifact":["M14 4.01h-1v-1c0-.55-.45-1-1-1s-1 .45-1 1v1h-1c-.55 0-1 .45-1 1s.45 1 1 1h1v1c0 .55.45 1 1 1s1-.45 1-1v-1h1c.55 0 1-.45 1-1 0-.56-.45-1-1-1zm-13 2h6c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1zm8 6H1c-.55 0-1 .45-1 1s.45 1 1 1h8c.55 0 1-.45 1-1 0-.56-.45-1-1-1zm0-4H1c-.55 0-1 .45-1 1s.45 1 1 1h8c.55 0 1-.45 1-1 0-.56-.45-1-1-1z"],"add-to-folder":["M.01 7V5H16v7c0 .55-.45 1-1 1H9.005v-2.99C8.974 8.332 7.644 7 5.996 7H.01zM15 2H7.416L5.706.29a.996.996 0 00-.71-.29H1C.45 0 0 .45 0 1v3h15.99V3c.01-.55-.44-1-.99-1zM5.997 9H2c-.55 0-1 .45-1 1s.45 1 1 1h1.589L.3 14.29a1.003 1.003 0 001.42 1.42l3.287-3.29v1.59c0 .55.45 1 1 1 .549 0 .999-.45.999-1v-4A1.02 1.02 0 005.996 9z"],airplane:["M16 1.5A1.498 1.498 0 0013.44.44L9.91 3.97 2 1 1 3l5.93 3.95L3.88 10H1l-1 1 3 2 2 3 1-1v-2.88l3.05-3.05L13 15l2-1-2.97-7.91 3.53-3.53c.27-.27.44-.65.44-1.06z"],"align-center":["M4 4c-.55 0-1 .45-1 1s.45 1 1 1h8c.55 0 1-.45 1-1s-.45-1-1-1H4zM1 3h14c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1zm13 10H2c-.55 0-1 .45-1 1s.45 1 1 1h12c.55 0 1-.45 1-1s-.45-1-1-1zm1-6H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zm-5 5c.55 0 1-.45 1-1s-.45-1-1-1H6c-.55 0-1 .45-1 1s.45 1 1 1h4z"],"align-justify":["M15 12.98H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zm-14-10h14c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1 0 .56.45 1 1 1zm14 4H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zm0-3H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zm0 6H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1z"],"align-left":["M13 13H1c-.55 0-1 .45-1 1s.45 1 1 1h12c.55 0 1-.45 1-1s-.45-1-1-1zM1 3h14c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1zm0 3h8c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1zm14 1H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zM1 12h4c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1z"],"align-right":["M15 12.98H3c-.55 0-1 .45-1 1s.45 1 1 1h12c.55 0 1-.45 1-1s-.45-1-1-1zm-14-10h14c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1 0 .56.45 1 1 1zm14 1H7c-.55 0-1 .45-1 1s.45 1 1 1h8c.55 0 1-.45 1-1s-.45-1-1-1zm0 6h-4c-.55 0-1 .45-1 1s.45 1 1 1h4c.55 0 1-.45 1-1s-.45-1-1-1zm0-3H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1z"],"alignment-bottom":["M10 12h3c.55 0 1-.45 1-1V7c0-.55-.45-1-1-1h-3c-.55 0-1 .45-1 1v4c0 .55.45 1 1 1zm5 2H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zM3 12h3c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1H3c-.55 0-1 .45-1 1v9c0 .55.45 1 1 1z"],"alignment-horizontal-center":["M15 7h-1V6c0-.55-.45-1-1-1h-3c-.55 0-1 .45-1 1v1H7V3c0-.55-.45-1-1-1H3c-.55 0-1 .45-1 1v4H1c-.55 0-1 .45-1 1s.45 1 1 1h1v4c0 .55.45 1 1 1h3c.55 0 1-.45 1-1V9h2v1c0 .55.45 1 1 1h3c.55 0 1-.45 1-1V9h1c.55 0 1-.45 1-1s-.45-1-1-1z"],"alignment-left":["M9 9H5c-.55 0-1 .45-1 1v3c0 .55.45 1 1 1h4c.55 0 1-.45 1-1v-3c0-.55-.45-1-1-1zM1 0C.45 0 0 .45 0 1v14c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1zm13 2H5c-.55 0-1 .45-1 1v3c0 .55.45 1 1 1h9c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1z"],"alignment-right":["M11 9H7c-.55 0-1 .45-1 1v3c0 .55.45 1 1 1h4c.55 0 1-.45 1-1v-3c0-.55-.45-1-1-1zm4-9c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1zm-4 2H2c-.55 0-1 .45-1 1v3c0 .55.45 1 1 1h9c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1z"],"alignment-top":["M15 0H1C.45 0 0 .45 0 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zM6 4H3c-.55 0-1 .45-1 1v9c0 .55.45 1 1 1h3c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1zm7 0h-3c-.55 0-1 .45-1 1v4c0 .55.45 1 1 1h3c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1z"],"alignment-vertical-center":["M13 2H9V1c0-.55-.45-1-1-1S7 .45 7 1v1H3c-.55 0-1 .45-1 1v3c0 .55.45 1 1 1h4v2H6c-.55 0-1 .45-1 1v3c0 .55.45 1 1 1h1v1c0 .55.45 1 1 1s1-.45 1-1v-1h1c.55 0 1-.45 1-1v-3c0-.55-.45-1-1-1H9V7h4c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1z"],annotation:["M15.52 2.77c.3-.29.48-.7.48-1.15C16 .73 15.27 0 14.38 0c-.45 0-.85.18-1.15.48l-1.34 1.34 2.3 2.3 1.33-1.35zM7.4 10.9l6.21-6.21-2.3-2.3L5.1 8.6l2.3 2.3zM14 14H2V2h6.34l2-2H1C.45 0 0 .45 0 1v14c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V5.66l-2 2V14zM3 13l3.58-1.29-2.29-2.27L3 13z"],"app-header":["M15 0a1 1 0 011 1v14a1 1 0 01-1 1H1a1 1 0 01-1-1V1a1 1 0 011-1h14zM6 4a1 1 0 00-1.993-.117L4 4v8a1 1 0 001.993.117L6 12V9h4v3a1 1 0 001.993.117L12 12V4a1 1 0 00-1.993-.117L10 4v3H6V4z"],application:["M3.5 7h7c.28 0 .5-.22.5-.5s-.22-.5-.5-.5h-7c-.28 0-.5.22-.5.5s.22.5.5.5zM15 1H1c-.55 0-1 .45-1 1v12c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1zm-1 12H2V5h12v8zM3.5 9h4c.28 0 .5-.22.5-.5S7.78 8 7.5 8h-4c-.28 0-.5.22-.5.5s.22.5.5.5zm0 2h5c.28 0 .5-.22.5-.5s-.22-.5-.5-.5h-5c-.28 0-.5.22-.5.5s.22.5.5.5z"],applications:["M3.5 11h2c.28 0 .5-.22.5-.5s-.22-.5-.5-.5h-2c-.28 0-.5.22-.5.5s.22.5.5.5zm0-2h5c.28 0 .5-.22.5-.5S8.78 8 8.5 8h-5c-.28 0-.5.22-.5.5s.22.5.5.5zM11 4H1c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1h10c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1zm-1 10H2V7h8v7zm5-14H5c-.55 0-1 .45-1 1v2h2V2h8v7h-1v2h2c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zM3.5 13h3c.28 0 .5-.22.5-.5s-.22-.5-.5-.5h-3c-.28 0-.5.22-.5.5s.22.5.5.5z"],archive:["M13.382 0a1 1 0 01.894.553L16 4v11a1 1 0 01-1 1H1a1 1 0 01-1-1V4L1.724.553A1 1 0 012.618 0h10.764zM8 6c-.55 0-1 .45-1 1v2.59l-.29-.29-.081-.076A.97.97 0 006 9a1.003 1.003 0 00-.71 1.71l2 2 .096.084c.168.13.38.206.614.206.28 0 .53-.11.71-.29l2-2 .084-.096A1.003 1.003 0 009.29 9.29l-.29.3V7l-.007-.116A1.004 1.004 0 008 6zm5-4H3L2 4h12l-1-2z"],array:["M15 0a1 1 0 01.993.883L16 1v14a1 1 0 01-.883.993L15 16h-3a1 1 0 01-.117-1.993L12 14h2V2h-2a1 1 0 01-.993-.883L11 1a1 1 0 01.883-.993L12 0h3zM4 0a1 1 0 01.117 1.993L4 2H2v12h2a1 1 0 01.993.883L5 15a1 1 0 01-.883.993L4 16H1a1 1 0 01-.993-.883L0 15V1A1 1 0 01.883.007L1 0h3zm4 7a1 1 0 110 2 1 1 0 010-2zM5 7a1 1 0 110 2 1 1 0 010-2zm6 0a1 1 0 110 2 1 1 0 010-2z"],"array-boolean":["M15 0a1 1 0 01.993.883L16 1v14a1 1 0 01-.883.993L15 16h-3a1 1 0 01-.117-1.993L12 14h2V2h-2a1 1 0 01-.993-.883L11 1a1 1 0 01.883-.993L12 0h3zM4 0a1 1 0 01.117 1.993L4 2H2v12h2a1 1 0 01.993.883L5 15a1 1 0 01-.883.993L4 16H1a1 1 0 01-.993-.883L0 15V1A1 1 0 01.883.007L1 0h3zm7 6a1 1 0 01.993.883L12 7v2a1 1 0 01-.883.993L11 10H5a1 1 0 01-.993-.883L4 9V7a1 1 0 01.883-.993L5 6h6zm0 1H8v2h3V7z"],"array-date":["M15 0a1 1 0 01.993.883L16 1v14a1 1 0 01-.883.993L15 16h-3a1 1 0 01-.117-1.993L12 14h2V2h-2a1 1 0 01-.993-.883L11 1a1 1 0 01.883-.993L12 0h3zM4 0a1 1 0 01.117 1.993L4 2H2v12h2a1 1 0 01.993.883L5 15a1 1 0 01-.883.993L4 16H1a1 1 0 01-.993-.883L0 15V1A1 1 0 01.883.007L1 0h3zm6.5 4a.5.5 0 01.5.5V5a1 1 0 01.993.883L12 6v5a1 1 0 01-.883.993L11 12H5a1 1 0 01-.993-.883L4 11V6a1 1 0 01.883-.993L5 5v-.5a.5.5 0 011 0V5h4v-.5a.5.5 0 01.5-.5zm.5 3H5v4h6V7z"],"array-numeric":["M15 0a1 1 0 01.993.883L16 1v14a1 1 0 01-.883.993L15 16h-3a1 1 0 01-.117-1.993L12 14h2V2h-2a1 1 0 01-.993-.883L11 1a1 1 0 01.883-.993L12 0h3zM4 0a1 1 0 01.117 1.993L4 2H2v12h2a1 1 0 01.993.883L5 15a1 1 0 01-.883.993L4 16H1a1 1 0 01-.993-.883L0 15V1A1 1 0 01.883.007L1 0h3zm6.904 5c.256 0 .483.037.68.112a1.343 1.343 0 01.812.788c.072.184.108.385.108.604 0 .23-.05.441-.152.636a1.314 1.314 0 01-.456.492v.016l.08.04c.055.031.111.072.168.124.085.077.163.172.232.284a1.673 1.673 0 01.24.872c0 .25-.043.477-.128.68a1.518 1.518 0 01-.896.852 1.937 1.937 0 01-.68.116c-.427 0-.792-.101-1.096-.304a1.362 1.362 0 01-.584-.864c-.01-.053.01-.088.064-.104l.696-.16.033-.002c.03.002.051.022.063.058.059.16.155.296.288.408.133.112.312.168.536.168.256 0 .453-.076.592-.228a.827.827 0 00.208-.58c0-.277-.08-.495-.24-.652-.16-.157-.376-.236-.648-.236h-.232l-.035-.005c-.03-.01-.045-.035-.045-.075v-.632l.005-.035c.01-.03.035-.045.075-.045h.216l.138-.009a.734.734 0 00.438-.207c.144-.144.216-.336.216-.576a.745.745 0 00-.192-.532c-.128-.136-.307-.204-.536-.204-.203 0-.363.05-.48.152a.815.815 0 00-.248.408c-.016.048-.048.067-.096.056l-.68-.16-.034-.012c-.028-.016-.038-.044-.03-.084a1.347 1.347 0 01.516-.828c.136-.104.296-.185.48-.244A1.98 1.98 0 0110.904 5zm-6.152.088l.035.005c.03.01.045.035.045.075v5.28l-.005.035c-.01.03-.035.045-.075.045h-.736l-.035-.005c-.03-.01-.045-.035-.045-.075V6.16H3.92l-.832.584-.032.016C3.02 6.773 3 6.751 3 6.696V5.88l.006-.04a.157.157 0 01.05-.072l.872-.632.04-.027a.236.236 0 01.104-.021h.68zM7.344 5c.256 0 .483.04.68.12.197.08.364.188.5.324s.24.296.312.48c.072.184.108.383.108.596 0 .245-.045.47-.136.676-.09.205-.216.404-.376.596l-1.584 1.92v.016h2.016l.035.005c.03.01.045.035.045.075v.64l-.005.035c-.01.03-.035.045-.075.045H5.808l-.035-.005c-.03-.01-.045-.035-.045-.075v-.6l.004-.04a.132.132 0 01.036-.064l1.92-2.392.1-.133a1.95 1.95 0 00.156-.267.985.985 0 00.096-.432.736.736 0 00-.188-.512c-.125-.139-.303-.208-.532-.208-.219 0-.39.061-.512.184a.826.826 0 00-.224.496c-.01.053-.04.075-.088.064L5.792 6.4l-.034-.012c-.028-.016-.038-.044-.03-.084a1.425 1.425 0 01.94-1.192A1.88 1.88 0 017.344 5z"],"array-string":["M15 0a1 1 0 01.993.883L16 1v14a1 1 0 01-.883.993L15 16h-3a1 1 0 01-.117-1.993L12 14h2V2h-2a1 1 0 01-.993-.883L11 1a1 1 0 01.883-.993L12 0h3zM4 0a1 1 0 01.117 1.993L4 2H2v12h2a1 1 0 01.993.883L5 15a1 1 0 01-.883.993L4 16H1a1 1 0 01-.993-.883L0 15V1A1 1 0 01.883.007L1 0h3zm1.61 5c.514 0 .962.212 1.343.637.382.425.573.997.573 1.716 0 .838-.258 1.588-.773 2.252-.514.663-1.327 1.2-2.437 1.609v-.465l.233-.095a3.09 3.09 0 001.274-1.017c.366-.505.55-1.03.55-1.577a.478.478 0 00-.057-.26c-.018-.037-.043-.056-.074-.056s-.08.025-.149.075c-.198.142-.446.214-.744.214-.36 0-.675-.145-.944-.433A1.453 1.453 0 014 6.572c0-.422.155-.79.465-1.102.31-.313.692-.47 1.144-.47zm4.474 0c.514 0 .963.212 1.344.637.381.425.572.997.572 1.716 0 .838-.257 1.588-.772 2.252-.515.663-1.327 1.2-2.437 1.609v-.465l.233-.095a3.09 3.09 0 001.274-1.017c.366-.505.549-1.03.549-1.577a.478.478 0 00-.056-.26c-.019-.037-.044-.056-.075-.056-.03 0-.08.025-.149.075-.198.142-.446.214-.744.214-.36 0-.674-.145-.944-.433a1.453 1.453 0 01-.405-1.028c0-.422.155-.79.466-1.102.31-.313.691-.47 1.144-.47z"],"array-timestamp":["M15 0a1 1 0 01.993.883L16 1v14a1 1 0 01-.883.993L15 16h-3a1 1 0 01-.117-1.993L12 14h2V2h-2a1 1 0 01-.993-.883L11 1a1 1 0 01.883-.993L12 0h3zM4 0a1 1 0 01.117 1.993L4 2H2v12h2a1 1 0 01.993.883L5 15a1 1 0 01-.883.993L4 16H1a1 1 0 01-.993-.883L0 15V1A1 1 0 01.883.007L1 0h3zm4 3a5 5 0 110 10A5 5 0 018 3zm0 1a4 4 0 100 8 4 4 0 000-8zm2.354 1.646a.5.5 0 01.057.638l-.057.07-2 2a.5.5 0 01-.638.057l-.07-.057-1-1a.5.5 0 01.638-.765l.07.057.646.647 1.646-1.647a.5.5 0 01.708 0z"],"arrow-bottom-left":["M14 3a1.003 1.003 0 00-1.71-.71L4 10.59V6c0-.55-.45-1-1-1s-1 .45-1 1v7c0 .55.45 1 1 1h7c.55 0 1-.45 1-1s-.45-1-1-1H5.41l8.29-8.29c.19-.18.3-.43.3-.71z"],"arrow-bottom-right":["M13 5c-.55 0-1 .45-1 1v4.59l-8.29-8.3a1.003 1.003 0 00-1.42 1.42l8.3 8.29H6c-.55 0-1 .45-1 1s.45 1 1 1h7c.55 0 1-.45 1-1V6c0-.55-.45-1-1-1z"],"arrow-down":["M13 8c-.3 0-.5.1-.7.3L9 11.6V2c0-.5-.4-1-1-1s-1 .5-1 1v9.6L3.7 8.3C3.5 8.1 3.3 8 3 8c-.5 0-1 .5-1 1 0 .3.1.5.3.7l5 5c.2.2.4.3.7.3s.5-.1.7-.3l5-5c.2-.2.3-.4.3-.7 0-.6-.4-1-1-1z"],"arrow-left":["M13.99 6.99H4.41L7.7 3.7a1.003 1.003 0 00-1.42-1.42l-5 5a1.014 1.014 0 000 1.42l5 5a1.003 1.003 0 001.42-1.42L4.41 8.99H14c.55 0 1-.45 1-1s-.46-1-1.01-1z"],"arrow-right":["M14.7 7.29l-5-5a.965.965 0 00-.71-.3 1.003 1.003 0 00-.71 1.71l3.29 3.29H1.99c-.55 0-1 .45-1 1s.45 1 1 1h9.59l-3.29 3.29a1.003 1.003 0 001.42 1.42l5-5c.18-.18.29-.43.29-.71s-.12-.52-.3-.7z"],"arrow-top-left":["M13.71 12.29L5.41 4H10c.55 0 1-.45 1-1s-.45-1-1-1H3c-.55 0-1 .45-1 1v7c0 .55.45 1 1 1s1-.45 1-1V5.41l8.29 8.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71z"],"arrow-top-right":["M13 2H6c-.55 0-1 .45-1 1s.45 1 1 1h4.59L2.3 12.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71L12 5.41V10c0 .55.45 1 1 1s1-.45 1-1V3c0-.55-.45-1-1-1z"],"arrow-up":["M13.7 6.3l-5-5C8.5 1.1 8.3 1 8 1s-.5.1-.7.3l-5 5c-.2.2-.3.4-.3.7 0 .6.5 1 1 1 .3 0 .5-.1.7-.3L7 4.4V14c0 .6.4 1 1 1s1-.4 1-1V4.4l3.3 3.3c.2.2.4.3.7.3.6 0 1-.4 1-1 0-.3-.1-.5-.3-.7z"],"arrows-horizontal":["M15.7 7.3l-4-4c-.2-.2-.4-.3-.7-.3-.6 0-1 .5-1 1 0 .3.1.5.3.7L12.6 7H3.4l2.3-2.3c.2-.2.3-.4.3-.7 0-.5-.4-1-1-1-.3 0-.5.1-.7.3l-4 4c-.2.2-.3.4-.3.7s.1.5.3.7l4 4c.2.2.4.3.7.3.6 0 1-.4 1-1 0-.3-.1-.5-.3-.7L3.4 9h9.2l-2.3 2.3c-.2.2-.3.4-.3.7 0 .6.4 1 1 1 .3 0 .5-.1.7-.3l4-4c.2-.2.3-.4.3-.7s-.1-.5-.3-.7z"],"arrows-vertical":["M12 10c-.3 0-.5.1-.7.3L9 12.6V3.4l2.3 2.3c.2.2.4.3.7.3.6 0 1-.4 1-1 0-.3-.1-.5-.3-.7l-4-4C8.5.1 8.3 0 8 0s-.5.1-.7.3l-4 4c-.2.2-.3.4-.3.7 0 .6.5 1 1 1 .3 0 .5-.1.7-.3L7 3.4v9.2l-2.3-2.3c-.2-.2-.4-.3-.7-.3-.5 0-1 .4-1 1 0 .3.1.5.3.7l4 4c.2.2.4.3.7.3s.5-.1.7-.3l4-4c.2-.2.3-.4.3-.7 0-.6-.4-1-1-1z"],asterisk:["M14.54 11.18l.01-.02L9.8 8l4.75-3.17-.01-.02c.27-.17.46-.46.46-.81 0-.55-.45-1-1-1-.21 0-.39.08-.54.18l-.01-.02L9 6.13V1c0-.55-.45-1-1-1S7 .45 7 1v5.13L2.55 3.17l-.01.01A.969.969 0 002 3c-.55 0-1 .45-1 1 0 .35.19.64.46.82l-.01.01L6.2 8l-4.75 3.17.01.02c-.27.17-.46.46-.46.81 0 .55.45 1 1 1 .21 0 .39-.08.54-.18l.01.02L7 9.87V15c0 .55.45 1 1 1s1-.45 1-1V9.87l4.45 2.96.01-.02c.15.11.33.19.54.19.55 0 1-.45 1-1 0-.35-.19-.64-.46-.82z"],"automatic-updates":["M8 14c-3.31 0-6-2.69-6-6 0-1.77.78-3.36 2-4.46V5c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1H1C.45 0 0 .45 0 1s.45 1 1 1h1.74A7.95 7.95 0 000 8c0 4.42 3.58 8 8 8 .55 0 1-.45 1-1s-.45-1-1-1zM8 2a5.9 5.9 0 012.95.81l1.47-1.47A7.893 7.893 0 008 0c-.55 0-1 .45-1 1s.45 1 1 1zm2.71 6.71l5-5a1.003 1.003 0 00-1.42-1.42L10 6.59l-1.29-1.3a1.003 1.003 0 00-1.42 1.42l2 2c.18.18.43.29.71.29s.53-.11.71-.29zM16 8c0-.55-.06-1.08-.16-1.6l-1.87 1.87A5.966 5.966 0 0112 12.45V11c0-.55-.45-1-1-1s-1 .45-1 1v4c0 .55.45 1 1 1h4c.55 0 1-.45 1-1s-.45-1-1-1h-1.74A7.95 7.95 0 0016 8z"],backlink:["M14 10a1 1 0 110 2h-.585l2.292 2.293a1 1 0 01-1.32 1.497l-.094-.083L12 13.415V14a1 1 0 11-2 0l.003-3.075.012-.1.012-.059.033-.108.034-.081.052-.098.067-.096.08-.09a1.01 1.01 0 01.112-.097l.11-.071.143-.065.076-.024.091-.02.116-.014L14 10zM6.036 6.136l-3.45 3.45-.117.127a2 2 0 002.818 2.818l.127-.117 3.45-3.449a4 4 0 01-.885 3.704l-.15.16-1 1A4 4 0 011.02 8.33l.15-.16 1-1a3.998 3.998 0 013.865-1.035zm4.671-1.843a1 1 0 01.083 1.32l-.083.094-5 5a1 1 0 01-1.497-1.32l.083-.094 5-5a1 1 0 011.414 0zm3.121-3.121a4 4 0 01.151 5.497l-.15.16-1 1a3.998 3.998 0 01-3.864 1.036l3.45-3.45.116-.128a2 2 0 00-2.818-2.818l-.127.117-3.45 3.45A4 4 0 017.02 2.33l.15-.16 1-1a4 4 0 015.657 0z"],badge:["M13.36 4.59c-.15-1.13.5-2.01 1.1-2.87L13.43.53c-1.72.88-4.12.65-5.63-.53-1.51 1.18-3.91 1.41-5.63.52l-1.03 1.2c.61.86 1.25 1.74 1.1 2.87-.3 2.29-2.45 4.17-1.32 6.68.45 1.14 1.44 1.9 2.72 2.2 1.56.36 3.52.72 4.16 2.53.64-1.81 2.6-2.16 4.16-2.54 1.28-.3 2.27-1.06 2.72-2.2 1.12-2.5-1.03-4.38-1.32-6.67z"],"ban-circle":["M8 0C3.58 0 0 3.58 0 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm3 9H5c-.55 0-1-.45-1-1s.45-1 1-1h6c.55 0 1 .45 1 1s-.45 1-1 1z"],"bank-account":["M15.36 6.46l-.62-.14c-.31-1.12-.98-2.15-1.87-2.99l.4-1.77a.438.438 0 00-.49-.56c-.85.09-1.6.42-2.14.98-.84-.32-1.87-.51-2.85-.51-2.49 0-4.63 1.17-5.92 2.89-.18-.04-.36-.09-.53-.09-.76 0-1.34.61-1.34 1.4 0 .56.31 1.03.76 1.26-.05.33-.09.7-.09 1.07 0 1.68.71 3.17 1.83 4.34l-.27 1.59c-.09.56.35 1.07.89 1.07h.58c.45 0 .8-.33.89-.79l.04-.37c.94.42 2 .7 3.16.7 1.11 0 2.23-.23 3.16-.7l.05.37c.09.47.45.79.89.79h.58c.53 0 .98-.51.89-1.07l-.27-1.54c.62-.61 1.07-1.35 1.38-2.15l.8-.19c.4-.09.71-.47.71-.93V7.4c.09-.47-.22-.84-.62-.94zM12 8c-.6 0-1-.7-1-1.5S11.4 5 12 5s1 .7 1 1.5S12.6 8 12 8zM6.21 4.92c-.41.2-.91.04-1.12-.36-.21-.4-.04-.88.37-1.07 1.35-.65 2.73-.65 4.08 0 .41.2.58.68.37 1.07-.21.4-.71.56-1.12.36-.87-.43-1.71-.43-2.58 0z"],barcode:["M0 14h2V2H0v12zm6 0h1V2H6v12zm2 0h1V2H8v12zm-5 0h2V2H3v12zM15 2v12h1V2h-1zm-5 12h1V2h-1v12zm2 0h2V2h-2v12z"],blank:[],"blocked-person":["M9.39 12.69c-1.2-.53-1.04-.85-1.08-1.29-.01-.07-.01-.13-.02-.2.41-.37.75-.87.97-1.44 0 0 .01-.03.01-.04.05-.13.09-.26.13-.39.27-.06.43-.36.5-.63.01-.03.03-.08.05-.12C8.18 7.8 6.94 6.04 6.94 4c0-.32.04-.62.09-.92-.17-.03-.35-.08-.51-.08-.65 0-1.37.2-1.88.59-.5.38-.87.92-1.05 1.51-.04.14-.07.27-.09.41-.09.48-.14 1.23-.14 1.74v.06c-.19.08-.36.27-.4.68-.03.31.1.59.16.7.06.28.23.59.51.64.04.14.08.27.13.39 0 .01.01.02.01.02v.01c.22.59.57 1.1.99 1.46 0 .06-.01.12-.01.17-.04.44.08.76-1.12 1.29-1.2.53-3.01 1.1-3.38 1.95C-.12 15.5.03 16 .03 16h12.96s.15-.5-.22-1.36c-.37-.85-2.18-1.42-3.38-1.95zM11.97 0C9.75 0 7.94 1.79 7.94 4s1.8 4 4.03 4S16 6.21 16 4s-1.8-4-4.03-4zM9.96 4c0-1.1.9-2 2.01-2 .37 0 .72.11 1.02.28l-2.75 2.73c-.17-.3-.28-.64-.28-1.01zm2.01 2c-.37 0-.72-.11-1.02-.28l2.75-2.73c.18.3.28.64.28 1.01.01 1.1-.9 2-2.01 2z"],bold:["M11.7 7c.2-.4.3-1 .3-1.5v-.4V5c0-.1 0-.2-.1-.3v-.1C11.4 3.1 10.1 2 8.5 2H4c-.5 0-1 .4-1 1v10c0 .5.4 1 1 1h5c2.2 0 4-1.8 4-4 0-1.2-.5-2.3-1.3-3zM6 5h2c.6 0 1 .4 1 1s-.4 1-1 1H6V5zm3 6H6V9h3c.6 0 1 .4 1 1s-.4 1-1 1z"],book:["M2 1v14c0 .55.45 1 1 1h1V0H3c-.55 0-1 .45-1 1zm11-1h-1v7l-2-2-2 2V0H5v16h8c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1z"],bookmark:["M11.2.01h-.15C11.03.01 11.02 0 11 0H5c-.02 0-.03.01-.05.01H4.8c-.44 0-.8.37-.8.82v14.75c0 .45.25.56.57.24l2.87-2.94c.31-.32.82-.32 1.13 0l2.87 2.94c.31.32.57.21.57-.24V.83C12 .38 11.64.01 11.2.01z"],box:["M6 10h4c.55 0 1-.45 1-1s-.45-1-1-1H6c-.55 0-1 .45-1 1s.45 1 1 1zm9.93-4.37v-.02L13.94.63C13.78.26 13.42 0 13 0H3c-.42 0-.78.26-.93.63L.08 5.61l-.01.02C.03 5.74 0 5.87 0 6v9c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V6c0-.13-.03-.26-.07-.37zM9 2h3.32l1.2 3H9V2zM3.68 2H7v3H2.48l1.2-3zM14 14H2V7h12v7z"],briefcase:["M15 3.98h-3v-2c0-.55-.45-1-1-1H5c-.55 0-1 .45-1 1v2H1c-.55 0-1 .45-1 1v4h3v-1h2v1h6v-1h2v1h3v-4c0-.55-.45-1-1-1zm-5 0H6v-1h4v1zm3 7h-2v-1H5v1H3v-1H0v4c0 .55.45 1 1 1h14c.55 0 1-.45 1-1v-4h-3v1z"],"bring-data":["M14 14a1 1 0 010 2H2a1 1 0 010-2h12zM7.995 3.005c.55 0 1 .45 1 .999v5.584l1.29-1.288a1.002 1.002 0 011.42 1.419l-3 2.996a1.015 1.015 0 01-1.42 0l-3-2.997A1.002 1.002 0 015.705 8.3l1.29 1.29V4.013c0-.55.45-1.009 1-1.009zM14 0a1 1 0 110 2 1 1 0 010-2zm-3 0a1 1 0 110 2 1 1 0 010-2zM8 0a1 1 0 110 2 1 1 0 010-2zM5 0a1 1 0 110 2 1 1 0 010-2zM2 0a1 1 0 110 2 1 1 0 010-2z"],build:["M15.39 12.41L7.7 6l1.07-1.1c.34-.34-.12-.63.12-1.26.88-2.17 3.41-2.35 3.41-2.35s.36-.37.71-.72C9.74-.81 7.53.53 6.54 1.4L3.12 4.9l-.71.72c-.39.4-.39 1.05 0 1.45l-.7.72c-.39-.4-1.02-.4-1.41 0s-.39 1.05 0 1.45l1.41 1.45c.39.4 1.02.4 1.41 0s.39-1.05 0-1.45l.71-.72c.39.4 1.02.4 1.41 0l.8-.82 6.39 7.67c.82.82 2.14.82 2.96 0 .81-.82.81-2.15 0-2.96z"],calculator:["M13 0H3c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1h10c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zM6 14H4v-2h2v2zm0-3H4V9h2v2zm0-3H4V6h2v2zm3 6H7v-2h2v2zm0-3H7V9h2v2zm0-3H7V6h2v2zm3 6h-2V9h2v5zm0-6h-2V6h2v2zm0-3H4V2h8v3z"],calendar:["M11 3c.6 0 1-.5 1-1V1c0-.6-.4-1-1-1s-1 .4-1 1v1c0 .5.4 1 1 1zm3-2h-1v1c0 1.1-.9 2-2 2s-2-.9-2-2V1H6v1c0 1.1-.9 2-2 2s-2-.9-2-2V1H1c-.6 0-1 .5-1 1v12c0 .6.4 1 1 1h13c.6 0 1-.4 1-1V2c0-.6-.5-1-1-1zM5 13H2v-3h3v3zm0-4H2V6h3v3zm4 4H6v-3h3v3zm0-4H6V6h3v3zm4 4h-3v-3h3v3zm0-4h-3V6h3v3zM4 3c.6 0 1-.5 1-1V1c0-.6-.4-1-1-1S3 .4 3 1v1c0 .5.4 1 1 1z"],camera:["M15 3h-2.59L10.7 1.29A.956.956 0 0010 1H6c-.28 0-.53.11-.71.29L3.59 3H1c-.55 0-1 .45-1 1v7c0 .55.45 1 1 1h2.56c1.1 1.22 2.67 2 4.44 2s3.34-.78 4.44-2H15c.55 0 1-.45 1-1V4c0-.55-.45-1-1-1zM3 6H1V5h2v1zm5 6c-2.21 0-4-1.79-4-4s1.79-4 4-4 4 1.79 4 4-1.79 4-4 4zm0-6c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2z"],"caret-down":["M12 6.5c0-.28-.22-.5-.5-.5h-7a.495.495 0 00-.37.83l3.5 4c.09.1.22.17.37.17s.28-.07.37-.17l3.5-4c.08-.09.13-.2.13-.33z"],"caret-left":["M9.5 4c-.13 0-.24.05-.33.13l-4 3.5c-.1.09-.17.22-.17.37s.07.28.17.37l4 3.5a.495.495 0 00.83-.37v-7c0-.28-.22-.5-.5-.5z"],"caret-right":["M11 8c0-.15-.07-.28-.17-.37l-4-3.5A.495.495 0 006 4.5v7a.495.495 0 00.83.37l4-3.5c.1-.09.17-.22.17-.37z"],"caret-up":["M11.87 9.17s.01 0 0 0l-3.5-4C8.28 5.07 8.15 5 8 5s-.28.07-.37.17l-3.5 4a.495.495 0 00.37.83h7a.495.495 0 00.37-.83z"],"cell-tower":["M8.97 6.76c-.01-.05-.04-.08-.06-.13-.02-.05-.03-.1-.05-.15.08-.14.14-.3.14-.48 0-.55-.45-1-1-1s-1 .45-1 1c0 .18.06.34.14.48-.03.05-.03.1-.05.15-.02.05-.05.08-.06.13l-2 8c-.13.54.19 1.08.73 1.21a.995.995 0 001.21-.73L7.53 13h.94l.56 2.24a1 1 0 001.94-.48l-2-8zM3.72 1.7C4.1 1.3 4.09.67 3.7.28S2.67-.09 2.28.3c-3.05 3.12-3.05 8.28 0 11.4a.996.996 0 101.43-1.39c-2.28-2.35-2.28-6.27.01-8.61zM11.6 3.2c-.44-.33-1.07-.24-1.4.2-.33.44-.24 1.07.2 1.4.43.32.53 1.96-.04 2.43-.42.35-.48.98-.13 1.41.35.42.98.48 1.41.13 1.59-1.33 1.39-4.5-.04-5.57z","M13.72.3c-.39-.4-1.02-.4-1.41-.02s-.41 1.02-.03 1.42c2.29 2.34 2.29 6.26 0 8.6-.39.39-.38 1.03.02 1.41s1.03.38 1.41-.02c3.05-3.11 3.05-8.27.01-11.39zM5.4 7.23c-.57-.47-.47-2.11-.04-2.43.44-.33.53-.96.2-1.4s-.96-.53-1.4-.2c-1.44 1.07-1.63 4.24-.04 5.57.42.35 1.05.3 1.41-.13.35-.42.29-1.06-.13-1.41z"],changes:["M8.29 7.71l3 3c.18.18.43.29.71.29s.53-.11.71-.29l3-3a1.003 1.003 0 00-1.42-1.42L13 7.59V1c0-.55-.45-1-1-1s-1 .45-1 1v6.59l-1.29-1.3a1.003 1.003 0 00-1.42 1.42zM14.5 13h-13c-.83 0-1.5.67-1.5 1.5S.67 16 1.5 16h13c.83 0 1.5-.67 1.5-1.5s-.67-1.5-1.5-1.5zM1 5c.28 0 .53-.11.71-.29L3 3.41V10c0 .55.45 1 1 1s1-.45 1-1V3.41L6.29 4.7c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71l-3-3C4.53.11 4.28 0 4 0s-.53.11-.71.29l-3 3A1.003 1.003 0 001 5z"],chart:["M0 15c0 .55.45 1 1 1h2c.55 0 1-.45 1-1V9.4L0 11v4zm6-5.5V15c0 .55.45 1 1 1h2c.55 0 1-.45 1-1v-5l-1 1-3-1.5zM13 7l-1 1v7c0 .55.45 1 1 1h2c.55 0 1-.45 1-1V7.88c-.26.07-.58.12-1 .12-1.96 0-2-1-2-1zm2-6h-3c-.55 0-1 .45-1 1s.45 1 1 1h.59L8.8 6.78 5.45 5.11v.01C5.31 5.05 5.16 5 5 5s-.31.05-.44.11V5.1l-4 2v.01C.23 7.28 0 7.61 0 8c0 .55.45 1 1 1 .16 0 .31-.05.44-.11v.01L5 7.12 8.55 8.9v-.01c.14.06.29.11.45.11.28 0 .53-.11.71-.29L14 4.41V5c0 .55.45 1 1 1s1-.45 1-1V2c0-.55-.45-1-1-1z"],chat:["M6 10c-1.1 0-2-.9-2-2V3H1c-.55 0-1 .45-1 1v8c0 .55.45 1 1 1v2a1.003 1.003 0 001.71.71L5.41 13H10c.55 0 1-.45 1-1v-1.17l-.83-.83H6zm9-10H6c-.55 0-1 .45-1 1v7c0 .55.45 1 1 1h4.59l2.71 2.71c.17.18.42.29.7.29.55 0 1-.45 1-1V9c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1z"],"chevron-backward":["M7.41 8l3.29-3.29c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71L6 6.59V4c0-.55-.45-1-1-1s-1 .45-1 1v8c0 .55.45 1 1 1s1-.45 1-1V9.41l3.29 3.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71L7.41 8z"],"chevron-down":["M12 5c-.28 0-.53.11-.71.29L8 8.59l-3.29-3.3a1.003 1.003 0 00-1.42 1.42l4 4c.18.18.43.29.71.29s.53-.11.71-.29l4-4A1.003 1.003 0 0012 5z"],"chevron-forward":["M10 3c-.55 0-1 .45-1 1v2.59l-3.29-3.3a1.003 1.003 0 00-1.42 1.42L7.59 8 4.3 11.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71L9 9.41V12c0 .55.45 1 1 1s1-.45 1-1V4c0-.55-.45-1-1-1z"],"chevron-left":["M7.41 8l3.29-3.29c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71l-4 4C5.11 7.47 5 7.72 5 8c0 .28.11.53.29.71l4 4a1.003 1.003 0 001.42-1.42L7.41 8z"],"chevron-right":["M10.71 7.29l-4-4a1.003 1.003 0 00-1.42 1.42L8.59 8 5.3 11.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71l4-4c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71z"],"chevron-up":["M12.71 9.29l-4-4C8.53 5.11 8.28 5 8 5s-.53.11-.71.29l-4 4a1.003 1.003 0 001.42 1.42L8 7.41l3.29 3.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71z"],circle:["M8 0C3.6 0 0 3.6 0 8s3.6 8 8 8 8-3.6 8-8-3.6-8-8-8zm0 14c-3.3 0-6-2.7-6-6s2.7-6 6-6 6 2.7 6 6-2.7 6-6 6z"],"circle-arrow-down":["M11 7c-.28 0-.53.11-.71.29L9 8.59V5c0-.55-.45-1-1-1s-1 .45-1 1v3.59l-1.29-1.3a1.003 1.003 0 00-1.42 1.42l3 3c.18.18.43.29.71.29s.53-.11.71-.29l3-3A1.003 1.003 0 0011 7zM8 0C3.58 0 0 3.58 0 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm0 14c-3.31 0-6-2.69-6-6s2.69-6 6-6 6 2.69 6 6-2.69 6-6 6z"],"circle-arrow-left":["M11 7H7.41L8.7 5.71c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71l-3 3C4.11 7.47 4 7.72 4 8c0 .28.11.53.29.71l3 3a1.003 1.003 0 001.42-1.42L7.41 9H11c.55 0 1-.45 1-1s-.45-1-1-1zM8 0C3.58 0 0 3.58 0 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm0 14c-3.31 0-6-2.69-6-6s2.69-6 6-6 6 2.69 6 6-2.69 6-6 6z"],"circle-arrow-right":["M8.71 4.29a1.003 1.003 0 00-1.42 1.42L8.59 7H5c-.55 0-1 .45-1 1s.45 1 1 1h3.59L7.3 10.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71l3-3c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71l-3-3zM8 0C3.58 0 0 3.58 0 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm0 14c-3.31 0-6-2.69-6-6s2.69-6 6-6 6 2.69 6 6-2.69 6-6 6z"],"circle-arrow-up":["M8.71 4.29C8.53 4.11 8.28 4 8 4s-.53.11-.71.29l-3 3a1.003 1.003 0 001.42 1.42L7 7.41V11c0 .55.45 1 1 1s1-.45 1-1V7.41l1.29 1.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71l-3-3zM8 0C3.58 0 0 3.58 0 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm0 14c-3.31 0-6-2.69-6-6s2.69-6 6-6 6 2.69 6 6-2.69 6-6 6z"],citation:["M15.02 5c0-1.66-1.34-3-3-3s-3 1.34-3 3a2.996 2.996 0 003.6 2.94C12.1 9.76 11.14 11 10.02 11c-.55 0-1 .45-1 1s.45 1 1 1c2.76 0 5-3.13 5-7 0-.2-.02-.39-.04-.58.01-.14.04-.28.04-.42zm-11-3c-1.66 0-3 1.34-3 3a2.996 2.996 0 003.6 2.94C4.1 9.76 3.14 11 2.02 11c-.55 0-1 .45-1 1s.45 1 1 1c2.76 0 5-3.13 5-7 0-.2-.02-.39-.04-.58.01-.14.04-.28.04-.42 0-1.66-1.35-3-3-3z"],clean:["M12 8l-1.2 2.796-2.8 1.2 2.8 1.197L12 16l1.2-2.807L16 12l-2.8-1.204zM5 0L3.5 3.5 0 4.995 3.5 6.5 5 10l1.5-3.5L10 5 6.5 3.5z"],clipboard:["M11 2c0-.55-.45-1-1-1h.22C9.88.4 9.24 0 8.5 0S7.12.4 6.78 1H7c-.55 0-1 .45-1 1v1h5V2zm2 0h-1v2H5V2H4c-.55 0-1 .45-1 1v12c0 .55.45 1 1 1h9c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1z"],cloud:["M12 6c-.03 0-.07 0-.1.01A5 5 0 002 7c0 .11.01.22.02.33A3.51 3.51 0 000 10.5C0 12.43 1.57 14 3.5 14H12c2.21 0 4-1.79 4-4s-1.79-4-4-4z"],"cloud-download":["M11 11c-.28 0-.53.11-.71.29L9 12.59V8c0-.55-.45-1-1-1s-1 .45-1 1v4.59L5.71 11.3A.965.965 0 005 11a1.003 1.003 0 00-.71 1.71l3 3c.18.18.43.29.71.29s.53-.11.71-.29l3-3A1.003 1.003 0 0011 11zm1-7c-.03 0-.07 0-.1.01A5 5 0 002 5c0 .11.01.22.02.33A3.51 3.51 0 000 8.5c0 1.41.84 2.61 2.03 3.17C2.2 10.17 3.46 9 5 9c.06 0 .13.02.19.02C5.07 8.7 5 8.36 5 8c0-1.66 1.34-3 3-3s3 1.34 3 3c0 .36-.07.7-.19 1.02.06 0 .13-.02.19-.02 1.48 0 2.7 1.07 2.95 2.47A3.964 3.964 0 0016 8c0-2.21-1.79-4-4-4z"],"cloud-upload":["M8.71 7.29C8.53 7.11 8.28 7 8 7s-.53.11-.71.29l-3 3a1.003 1.003 0 001.42 1.42L7 10.41V15c0 .55.45 1 1 1s1-.45 1-1v-4.59l1.29 1.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71l-3-3zM12 4c-.03 0-.07 0-.1.01A5 5 0 002 5c0 .11.01.22.02.33a3.495 3.495 0 00.07 6.37c-.05-.23-.09-.46-.09-.7 0-.83.34-1.58.88-2.12l3-3a2.993 2.993 0 014.24 0l3 3c.54.54.88 1.29.88 2.12 0 .16-.02.32-.05.47C15.17 10.78 16 9.5 16 8c0-2.21-1.79-4-4-4z"],code:["M15.71 7.29l-3-3a1.003 1.003 0 00-1.42 1.42L13.59 8l-2.29 2.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71l3-3c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71zM5 5a1.003 1.003 0 00-1.71-.71l-3 3C.11 7.47 0 7.72 0 8c0 .28.11.53.29.71l3 3a1.003 1.003 0 001.42-1.42L2.41 8 4.7 5.71c.19-.18.3-.43.3-.71zm4-3c-.48 0-.87.35-.96.81l-2 10c-.01.06-.04.12-.04.19 0 .55.45 1 1 1 .48 0 .87-.35.96-.81l2-10c.01-.06.04-.12.04-.19 0-.55-.45-1-1-1z"],"code-block":["M15 3h-2V2c0-.55-.45-1-1-1h-2c-.55 0-1 .45-1 1v1H7V2c0-.55-.45-1-1-1H4c-.55 0-1 .45-1 1v1H1c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V4c0-.55-.45-1-1-1zm-8.29 8.29a1.003 1.003 0 01-1.42 1.42l-3-3C2.11 9.53 2 9.28 2 9s.11-.53.29-.71l3-3a1.003 1.003 0 011.42 1.42L4.41 9l2.3 2.29zm7-1.58l-3 3a1.003 1.003 0 01-1.42-1.42L11.59 9l-2.3-2.29a1.003 1.003 0 011.42-1.42l3 3c.18.18.29.43.29.71s-.11.53-.29.71z"],cog:["M15.19 6.39h-1.85c-.11-.37-.27-.71-.45-1.04l1.36-1.36c.31-.31.31-.82 0-1.13l-1.13-1.13a.803.803 0 00-1.13 0l-1.36 1.36c-.33-.17-.67-.33-1.04-.44V.79c0-.44-.36-.8-.8-.8h-1.6c-.44 0-.8.36-.8.8v1.86c-.39.12-.75.28-1.1.47l-1.3-1.3c-.3-.3-.79-.3-1.09 0L1.82 2.91c-.3.3-.3.79 0 1.09l1.3 1.3c-.2.34-.36.7-.48 1.09H.79c-.44 0-.8.36-.8.8v1.6c0 .44.36.8.8.8h1.85c.11.37.27.71.45 1.04l-1.36 1.36c-.31.31-.31.82 0 1.13l1.13 1.13c.31.31.82.31 1.13 0l1.36-1.36c.33.18.67.33 1.04.44v1.86c0 .44.36.8.8.8h1.6c.44 0 .8-.36.8-.8v-1.86c.39-.12.75-.28 1.1-.47l1.3 1.3c.3.3.79.3 1.09 0l1.09-1.09c.3-.3.3-.79 0-1.09l-1.3-1.3c.19-.35.36-.71.48-1.1h1.85c.44 0 .8-.36.8-.8v-1.6a.816.816 0 00-.81-.79zm-7.2 4.6c-1.66 0-3-1.34-3-3s1.34-3 3-3 3 1.34 3 3-1.34 3-3 3z"],"collapse-all":["M7.29 6.71c.18.18.43.29.71.29s.53-.11.71-.29l4-4a1.003 1.003 0 00-1.42-1.42L8 4.59l-3.29-3.3a1.003 1.003 0 00-1.42 1.42l4 4zm1.42 2.58C8.53 9.11 8.28 9 8 9s-.53.11-.71.29l-4 4a1.003 1.003 0 001.42 1.42L8 11.41l3.29 3.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71l-4-4z"],"column-layout":["M15 1H1c-.55 0-1 .45-1 1v12c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1zM4 13H2V3h2v10zm3 0H5V3h2v10zm7 0H8V3h6v10z"],comment:["M14 1H1c-.55 0-1 .45-1 1v9c0 .55.45 1 1 1h2v3a1.003 1.003 0 001.71.71L8.41 12H14c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1zM3.5 8C2.67 8 2 7.33 2 6.5S2.67 5 3.5 5 5 5.67 5 6.5 4.33 8 3.5 8zm4 0C6.67 8 6 7.33 6 6.5S6.67 5 7.5 5 9 5.67 9 6.5 8.33 8 7.5 8zm4 0c-.83 0-1.5-.67-1.5-1.5S10.67 5 11.5 5s1.5.67 1.5 1.5S12.33 8 11.5 8z"],comparison:["M7.99-.01c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1s1-.45 1-1v-14c0-.55-.45-1-1-1zm-3 3h-4c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h4c.55 0 1-.45 1-1v-2c0-.55-.45-1-1-1zm10 0h-4c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h4c.55 0 1-.45 1-1v-2c0-.55-.45-1-1-1zm0 3h-4v-2h4v2zm0 3h-4c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h4c.55 0 1-.45 1-1v-2c0-.55-.45-1-1-1zm0 3h-4v-2h4v2zm-10-3h-4c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h4c.55 0 1-.45 1-1v-2c0-.55-.45-1-1-1z"],compass:["M12 8c0 .14-.03.27-.08.39l-3 6.99c-.15.37-.51.62-.92.62s-.77-.25-.92-.61l-3-6.99a1.006 1.006 0 010-.79l3-6.99C7.23.25 7.59 0 8 0s.77.25.92.61l3 6.99c.05.13.08.26.08.4zM8 3.54L6.09 8h3.82L8 3.54z"],compressed:["M15.93 5.63v-.02L13.94.63C13.78.26 13.42 0 13 0H3c-.42 0-.78.26-.93.63L.08 5.61l-.01.02C.03 5.74 0 5.87 0 6v9c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V6c0-.13-.03-.26-.07-.37zM9 2h3.32l1.2 3H9V2zM3.68 2H7v3H2.48l1.2-3zM14 14H2V7h5v2.59l-1.29-1.3a1.003 1.003 0 00-1.42 1.42l3 3c.18.18.43.29.71.29s.53-.11.71-.29l3-3a1.003 1.003 0 00-1.42-1.42L9 9.59V7h5v7z"],confirm:["M8.7 4.29a.965.965 0 00-.71-.3 1.003 1.003 0 00-.71 1.71l2 2c.18.18.43.29.71.29s.53-.11.71-.29l5-5a1.003 1.003 0 00-1.42-1.42l-4.29 4.3L8.7 4.29zm5.22 3.01c.03.23.07.45.07.69 0 3.31-2.69 6-6 6s-6-2.69-6-6 2.69-6 6-6c.81 0 1.59.17 2.3.46l1.5-1.5A7.998 7.998 0 00-.01 7.99c0 4.42 3.58 8 8 8s8-3.58 8-8c0-.83-.13-1.64-.36-2.39l-1.71 1.7z"],console:["M15 15H1c-.55 0-1-.45-1-1V2c0-.55.45-1 1-1h14c.55 0 1 .45 1 1v12c0 .55-.45 1-1 1zM14 5H2v8h12V5zM4 6c.28 0 .53.11.71.29l2 2c.18.18.29.43.29.71s-.11.53-.29.71l-2 2a1.003 1.003 0 01-1.42-1.42L4.59 9l-1.3-1.29A1.003 1.003 0 014 6zm5 4h3c.55 0 1 .45 1 1s-.45 1-1 1H9c-.55 0-1-.45-1-1s.45-1 1-1z"],contrast:["M15.2 6.4h-1.44c-.13-.47-.32-.92-.56-1.34L14.26 4c.31-.31.31-.82 0-1.13l-1.13-1.13a.803.803 0 00-1.13 0L10.94 2.8c-.42-.24-.86-.42-1.34-.56V.8c0-.44-.36-.8-.8-.8H7.2c-.44 0-.8.36-.8.8v1.44c-.5.14-.96.34-1.4.59l-1-1c-.3-.3-.79-.3-1.09 0L1.83 2.91c-.3.3-.3.79 0 1.09l1 1c-.25.44-.45.9-.59 1.4H.8c-.44 0-.8.36-.8.8v1.6c0 .44.36.8.8.8h1.44c.13.47.32.92.56 1.34L1.74 12c-.31.31-.31.82 0 1.13l1.13 1.13c.31.31.82.31 1.13 0l1.06-1.06c.42.24.86.42 1.34.56v1.44c0 .44.36.8.8.8h1.6c.44 0 .8-.36.8-.8v-1.44c.5-.14.96-.33 1.4-.59l1 1c.3.3.79.3 1.09 0l1.09-1.09c.3-.3.3-.79 0-1.09l-1-1c.25-.43.45-.9.59-1.4h1.44c.44 0 .8-.36.8-.8V7.2a.818.818 0 00-.81-.8zM8 12c-2.21 0-4-1.79-4-4s1.79-4 4-4v8z"],control:["M13 8H8v5h5V8zm0-5H8v4h5V3zm2-3H1C.45 0 0 .45 0 1v14c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 14H2V2h12v12zM7 3H3v10h4V3z"],"credit-card":["M14.99 2.95h-14c-.55 0-1 .45-1 1v1h16v-1c0-.55-.45-1-1-1zm-15 10c0 .55.45 1 1 1h14c.55 0 1-.45 1-1v-6h-16v6zm5.5-2h5c.28 0 .5.22.5.5s-.22.5-.5.5h-5c-.28 0-.5-.22-.5-.5s.23-.5.5-.5zm-3 0h1c.28 0 .5.22.5.5s-.22.5-.5.5h-1c-.28 0-.5-.22-.5-.5s.23-.5.5-.5z"],cross:["M9.41 8l3.29-3.29c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71L8 6.59l-3.29-3.3a1.003 1.003 0 00-1.42 1.42L6.59 8 3.3 11.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71L8 9.41l3.29 3.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71L9.41 8z"],crown:["M2 6l3 2 3-4 3 4 3-2-1 6H3L2 6zm6-5a1 1 0 110 2 1 1 0 010-2zM1 3a1 1 0 110 2 1 1 0 010-2zm14 0a1 1 0 110 2 1 1 0 010-2zM3 13h10v2H3v-2z"],cube:["M14.194 3.54L8 7.41 1.806 3.54 7.504.283a1 1 0 01.992 0l5.698 3.255zm.75.71a1 1 0 01.056.33v6.84a1 1 0 01-.504.868L8.5 15.714V8.277l6.444-4.027zm-13.888 0L7.5 8.277v7.437l-5.996-3.426A1 1 0 011 11.42V4.58a1 1 0 01.056-.33z"],"cube-add":["M14 2h1a1 1 0 010 2h-1v1a1 1 0 01-2 0V4h-1a1 1 0 010-2h1V1a1 1 0 012 0v1zM9.136.65a3.001 3.001 0 00.992 5.222c.018.058.038.115.059.172L8 7.41 1.806 3.54 7.504.283a1 1 0 01.992 0l.64.365zM15 7.235v4.184a1 1 0 01-.504.868L8.5 15.714V8.277l2.187-1.367A2.994 2.994 0 0013 8c.768 0 1.47-.289 2-.764zM1.056 4.25L7.5 8.277v7.437l-5.996-3.426A1 1 0 011 11.42V4.58a1 1 0 01.056-.33z"],"cube-remove":["M10.365 5.933L8 7.41 1.806 3.54 7.504.283a1 1 0 01.992 0l.64.365a3.001 3.001 0 001.228 5.283zM15 6v5.42a1 1 0 01-.504.868L8.5 15.714V8.277L12.143 6H15zM1.056 4.25L7.5 8.277v7.437l-5.996-3.426A1 1 0 011 11.42V4.58a1 1 0 01.056-.33zM11 2h4a1 1 0 010 2h-4a1 1 0 010-2z"],"curved-range-chart":["M15 12H3.12l1.81-1.39c1.73 1.01 5.53-.03 9.08-2.61l-1.22-1.5C10.3 8.3 7.86 9.37 6.65 9.29L14.3 3.4l-.6-.8-7.83 6.03c-.01-1.07 1.8-3.19 4.47-5.13L9.12 2C5.38 4.7 3.34 8.1 4.25 9.87L2 11.6V3c0-.55-.45-1-1-1s-1 .45-1 1v10c0 .55.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1z"],cut:["M13 2s.71-1.29 0-2L8.66 5.07l1.05 1.32L13 2zm.07 8c-.42 0-.82.09-1.18.26L3.31 0c-.69.71 0 2 0 2l3.68 5.02-2.77 3.24A2.996 2.996 0 000 13c0 1.66 1.34 3 3 3s3-1.34 3-3c0-.46-.11-.89-.29-1.27L8.1 8.54l2.33 3.19c-.18.39-.29.82-.29 1.27 0 1.66 1.31 3 2.93 3S16 14.66 16 13s-1.31-3-2.93-3zM3 14c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zm10.07 0c-.54 0-.98-.45-.98-1s.44-1 .98-1 .98.45.98 1-.44 1-.98 1z"],cycle:["M13 9a3 3 0 110 6 3 3 0 010-6zM3 9a3 3 0 110 6 3 3 0 010-6zm6.169-5.27l.087.09 1.51 1.746 1.589.549a1 1 0 01.65 1.16l-.032.112a1 1 0 01-1.159.65l-.112-.032-1.843-.636a1 1 0 01-.337-.198l-.092-.093-.959-1.109L7.041 7.5l1.691 1.819a1 1 0 01.26.556L9 10v3a1 1 0 01-1.993.117L7 13l-.001-2.608-2.056-2.211a1 1 0 01-.081-1.264l.082-.1 2.825-3.026a1 1 0 011.4-.061zM13 10.5a1.5 1.5 0 100 3 1.5 1.5 0 000-3zm-10 0a1.5 1.5 0 100 3 1.5 1.5 0 000-3zM11 1a1.5 1.5 0 110 3 1.5 1.5 0 010-3z"],dashboard:["M5 4c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zM4 7c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm4-2c.55 0 1-.45 1-1s-.45-1-1-1-1 .45-1 1 .45 1 1 1zm-2 6c0 1.1.9 2 2 2s2-.9 2-2c0-.53-2-5-2-5s-2 4.47-2 5zM8 0C3.58 0 0 3.58 0 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm0 14c-3.31 0-6-2.69-6-6s2.69-6 6-6 6 2.69 6 6-2.69 6-6 6zm4-9c0-.55-.45-1-1-1s-1 .45-1 1 .45 1 1 1 1-.45 1-1zm0 2c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1z"],"data-connection":["M1 9.52c.889.641 2.308 1.133 4.003 1.354L5 11a5.994 5.994 0 002.664 4.988c-.217.008-.44.012-.664.012-3.215 0-5.846-.85-5.993-1.906L1 14V9.52zM11 6c2.762 0 5 2.238 5 5s-2.238 5-5 5-5-2.238-5-5 2.238-5 5-5zm1 1l-4 5h2.5l-.5 3 4-5h-2.5l.5-3zm1-3.48v1.822a6.002 6.002 0 00-7.9 4.556l-.248-.03c-2.168-.28-3.733-.966-3.845-1.774L1 8V3.52C2.22 4.4 4.44 5 7 5s4.78-.6 6-1.48zM7 0c3.31 0 6 .9 6 2s-2.69 2-6 2c-3.32 0-6-.9-6-2s2.68-2 6-2z"],"data-lineage":["M1.067 0C.477 0 0 .478 0 1.067V3.2c0 .59.478 1.067 1.067 1.067h2.24a5.342 5.342 0 002.9 3.734 5.337 5.337 0 00-2.9 3.733h-2.24C.477 11.733 0 12.21 0 12.8v2.133C0 15.523.478 16 1.067 16H6.4c.59 0 1.067-.478 1.067-1.067V12.8c0-.59-.478-1.067-1.067-1.067H4.401a4.27 4.27 0 013.92-3.194l.212-.006V9.6c0 .59.478 1.067 1.067 1.067h5.333c.59 0 1.067-.478 1.067-1.067V6.4c0-.59-.478-1.067-1.067-1.067H9.6c-.59 0-1.067.478-1.067 1.067v1.067a4.268 4.268 0 01-4.132-3.2H6.4c.59 0 1.067-.478 1.067-1.067V1.067C7.467.477 6.989 0 6.4 0H1.067z"],database:["M8 4c3.31 0 6-.9 6-2s-2.69-2-6-2C4.68 0 2 .9 2 2s2.68 2 6 2zm-6-.48V8c0 1.1 2.69 2 6 2s6-.9 6-2V3.52C12.78 4.4 10.56 5 8 5s-4.78-.6-6-1.48zm0 6V14c0 1.1 2.69 2 6 2s6-.9 6-2V9.52C12.78 10.4 10.56 11 8 11s-4.78-.6-6-1.48z"],delete:["M11.99 4.99a1.003 1.003 0 00-1.71-.71l-2.29 2.3L5.7 4.29a.965.965 0 00-.71-.3 1.003 1.003 0 00-.71 1.71l2.29 2.29-2.29 2.29A1.003 1.003 0 005.7 11.7l2.29-2.29 2.29 2.29a1.003 1.003 0 001.42-1.42L9.41 7.99 11.7 5.7c.18-.18.29-.43.29-.71zm-4-5c-4.42 0-8 3.58-8 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm0 14c-3.31 0-6-2.69-6-6s2.69-6 6-6 6 2.69 6 6-2.68 6-6 6z"],delta:["M8 0L0 16h16L8 0zM7 5l5 10H2L7 5z"],"derive-column":["M6.08 6.67h-.84c.24-.92.56-1.6.96-2.03.24-.27.48-.4.71-.4.05 0 .08.01.11.04s.04.06.04.1c0 .04-.03.11-.1.21-.06.1-.1.2-.1.29 0 .13.05.24.15.33.1.09.23.14.39.14.17 0 .31-.06.42-.17A.58.58 0 008 4.73c0-.22-.09-.39-.26-.53-.17-.13-.44-.2-.81-.2-.59 0-1.12.16-1.59.48-.48.32-.93.85-1.36 1.59-.15.26-.29.42-.42.49s-.35.11-.64.1l-.19.65h.81l-1.19 4.37c-.2.72-.33 1.16-.4 1.33-.1.24-.26.45-.46.62-.08.07-.18.1-.3.1-.03 0-.06-.01-.08-.03l-.03-.04c0-.02.03-.06.09-.11.06-.06.09-.14.09-.26 0-.13-.05-.23-.14-.32a.6.6 0 00-.4-.13c-.21 0-.38.05-.51.16s-.21.25-.21.4c0 .16.08.3.23.42.16.12.4.18.74.18.53 0 .99-.13 1.4-.39.41-.26.76-.65 1.07-1.19.3-.54.62-1.4.94-2.59l.68-2.53h.82l.2-.63zM15 0H8c-.55 0-1 .45-1 1v2h2V2h5v12H9v-1H7v2c0 .55.45 1 1 1h7c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zM8.3 9.94c.18.52.33.89.46 1.13.13.24.28.4.44.51.17.1.37.16.62.16.24 0 .49-.08.74-.25.33-.21.66-.58 1.01-1.09l-.21-.11c-.23.31-.41.5-.52.57a.44.44 0 01-.26.07c-.12 0-.24-.07-.36-.21-.2-.24-.46-.91-.8-2 .3-.49.55-.81.75-.96.15-.11.3-.16.47-.16.06 0 .17.02.34.06.16.04.31.06.43.06.17 0 .31-.06.43-.17.1-.11.16-.25.16-.43 0-.19-.06-.33-.17-.44-.12-.11-.28-.16-.49-.16-.19 0-.37.04-.54.13-.17.09-.39.27-.65.56-.2.21-.48.58-.87 1.11-.15-.66-.41-1.26-.78-1.81l-2.05.33-.04.21c.15-.03.28-.04.39-.04.2 0 .37.08.5.25.21.26.5 1.03.88 2.33-.29.37-.49.61-.6.72-.18.18-.33.3-.44.36-.09.04-.19.07-.3.07-.09 0-.23-.04-.42-.13a.866.866 0 00-.36-.09c-.2 0-.36.06-.49.18a.59.59 0 00-.19.46c0 .17.06.32.18.43.12.11.28.16.48.16.2 0 .38-.04.55-.11.17-.08.39-.24.65-.49.24-.27.6-.66 1.06-1.21z"],desktop:["M15 0H1C.45 0 0 .45 0 1v10c0 .55.45 1 1 1h4.75l-.5 2H4c-.55 0-1 .45-1 1s.45 1 1 1h8c.55 0 1-.45 1-1s-.45-1-1-1h-1.25l-.5-2H15c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 10H2V2h12v8z"],diagnosis:["M3.2 1a1 1 0 01.117 1.993L3.2 3H3v3a2 2 0 001.85 1.995L5 8a2 2 0 001.995-1.85L7 6V3h-.2a1 1 0 01-.993-.883L5.8 2a1 1 0 01.883-.993L6.8 1H8a1 1 0 01.993.883L9 2v4a4.002 4.002 0 01-3.007 3.876v.007L6 10a3 3 0 005.995.176L12 10V7.792a2.5 2.5 0 112 0V10a5 5 0 01-10 0c0-.042.003-.084.008-.125A4 4 0 011.005 6.2L1 6V2a1 1 0 01.883-.993L2 1h1.2z"],"diagram-tree":["M15 8v3h-2V9H9v2H7V9H3v2H1V8a1 1 0 011-1h5V5h2v2h5a1 1 0 011 1zM1 12h2a1 1 0 011 1v2a1 1 0 01-1 1H1a1 1 0 01-1-1v-2a1 1 0 011-1zm12 0h2a1 1 0 011 1v2a1 1 0 01-1 1h-2a1 1 0 01-1-1v-2a1 1 0 011-1zm-6 0h2a1 1 0 011 1v2a1 1 0 01-1 1H7a1 1 0 01-1-1v-2a1 1 0 011-1zM7 0h2a1 1 0 011 1v2a1 1 0 01-1 1H7a1 1 0 01-1-1V1a1 1 0 011-1z"],"direction-left":["M16 1.99l-16 6 16 6-4-6z"],"direction-right":["M16 7.99l-16-6 4 6-4 6z"],disable:["M7.99-.01c-4.42 0-8 3.58-8 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm-6 8c0-3.31 2.69-6 6-6 1.3 0 2.49.42 3.47 1.12l-8.35 8.35c-.7-.98-1.12-2.17-1.12-3.47zm6 6c-1.3 0-2.49-.42-3.47-1.12l8.35-8.35c.7.98 1.12 2.17 1.12 3.47 0 3.32-2.68 6-6 6z"],document:["M9 0H3c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1h10c.55 0 1-.45 1-1V5L9 0zm3 14H4V2h4v4h4v8z"],"document-open":["M6 12c0 .55.45 1 1 1s1-.45 1-1V8c0-.55-.45-1-1-1H3c-.55 0-1 .45-1 1s.45 1 1 1h1.59L1.3 12.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71L6 10.41V12zm4-12H4c-.55 0-1 .45-1 1v4h2V2h4v4h4v8H5.24l-1.8 1.8c.16.12.35.2.56.2h10c.55 0 1-.45 1-1V5l-5-5z"],"document-share":["M10 14H2V2h4v4h1c0-.83.36-1.55.91-2.09l-.03-.03.9-.9C8.3 2.45 8 1.77 8 1L7 0H1C.45 0 0 .45 0 1v14c0 .55.45 1 1 1h10c.55 0 1-.45 1-1V8.22c-.53.48-1.23.78-2 .78v5zm5-14h-4c-.55 0-1 .45-1 1s.45 1 1 1h1.59l-3.3 3.29a1.003 1.003 0 001.42 1.42L14 3.41V5c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1z"],dollar:["M12.83 9.51c-.1-.3-.25-.58-.45-.84s-.45-.49-.75-.7c-.3-.2-.65-.36-1.05-.48-.16-.04-.43-.11-.8-.2-.35-.09-.73-.18-1.12-.28-.39-.1-.74-.19-1.06-.27-.31-.08-.49-.12-.54-.13-.43-.12-.78-.29-1.05-.52-.27-.23-.4-.55-.4-.95 0-.29.07-.53.21-.72.14-.19.32-.34.54-.46.22-.11.46-.19.72-.24.26-.05.52-.08.77-.08.74 0 1.35.15 1.83.46.48.3.75.83.81 1.56h2.14c0-.6-.13-1.13-.38-1.58-.25-.45-.59-.84-1.02-1.15-.43-.31-.93-.54-1.49-.7-.24-.06-.49-.1-.75-.14V1c0-.55-.45-1-1-1s-1 .45-1 1v1.08c-.23.03-.46.07-.68.13-.54.13-1.02.34-1.44.61-.42.28-.76.63-1.02 1.05-.26.43-.39.93-.39 1.5 0 .3.04.59.13.88.09.29.23.56.44.82.21.26.48.49.83.7.35.21.79.38 1.31.51.85.21 1.56.38 2.14.52.58.13 1.08.28 1.52.42.25.09.48.23.69.44.21.21.32.53.32.97 0 .21-.05.42-.14.63-.09.21-.24.39-.45.55-.21.16-.47.29-.81.39-.33.1-.73.15-1.2.15-.43 0-.84-.05-1.21-.14-.37-.09-.7-.24-.99-.43-.29-.2-.51-.45-.67-.76-.16-.31-.24-.68-.24-1.12H3c.01.71.15 1.32.43 1.84.27.52.64.94 1.1 1.27.46.33.99.58 1.61.74.27.07.56.12.85.16V15c0 .55.45 1 1 1s1-.45 1-1v-1.05c.3-.03.61-.08.9-.15.58-.13 1.1-.34 1.56-.63.46-.29.83-.66 1.11-1.11.28-.45.42-1 .42-1.64 0-.31-.05-.61-.15-.91z"],dot:["M8 5a3 3 0 100 6 3 3 0 100-6z"],"double-caret-horizontal":["M13.71 7.29l-3-3A1.003 1.003 0 009 5v6a1.003 1.003 0 001.71.71l3-3c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71zM6 4c-.28 0-.53.11-.71.29l-3 3C2.11 7.47 2 7.72 2 8c0 .28.11.53.29.71l3 3A1.003 1.003 0 007 11V5c0-.55-.45-1-1-1z"],"double-caret-vertical":["M5 7h6a1.003 1.003 0 00.71-1.71l-3-3C8.53 2.11 8.28 2 8 2s-.53.11-.71.29l-3 3A1.003 1.003 0 005 7zm6 2H5a1.003 1.003 0 00-.71 1.71l3 3c.18.18.43.29.71.29s.53-.11.71-.29l3-3A1.003 1.003 0 0011 9z"],"double-chevron-down":["M7.29 8.71c.18.18.43.29.71.29s.53-.11.71-.29l4-4a1.003 1.003 0 00-1.42-1.42L8 6.59l-3.29-3.3a1.003 1.003 0 00-1.42 1.42l4 4zM12 8c-.28 0-.53.11-.71.29L8 11.59l-3.29-3.3a1.003 1.003 0 00-1.42 1.42l4 4c.18.18.43.29.71.29s.53-.11.71-.29l4-4A1.003 1.003 0 0012 8z"],"double-chevron-left":["M4.41 8L7.7 4.71c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71l-4 4C2.11 7.47 2 7.72 2 8c0 .28.11.53.29.71l4 4a1.003 1.003 0 001.42-1.42L4.41 8zm5 0l3.29-3.29c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71l-4 4C7.11 7.47 7 7.72 7 8c0 .28.11.53.29.71l4 4a1.003 1.003 0 001.42-1.42L9.41 8z"],"double-chevron-right":["M9 8c0-.28-.11-.53-.29-.71l-4-4a1.003 1.003 0 00-1.42 1.42L6.59 8 3.3 11.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71l4-4C8.89 8.53 9 8.28 9 8zm4.71-.71l-4-4a1.003 1.003 0 00-1.42 1.42L11.59 8 8.3 11.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71l4-4c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71z"],"double-chevron-up":["M4 8c.28 0 .53-.11.71-.29L8 4.41l3.29 3.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71l-4-4C8.53 2.11 8.28 2 8 2s-.53.11-.71.29l-4 4A1.003 1.003 0 004 8zm4.71-.71C8.53 7.11 8.28 7 8 7s-.53.11-.71.29l-4 4a1.003 1.003 0 001.42 1.42L8 9.41l3.29 3.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71l-4-4z"],"doughnut-chart":["M11.86 7h4.05C15.45 3.39 12.61.52 9 .07v4.07A4 4 0 0111.86 7zM12 8c0 2.21-1.79 4-4 4s-4-1.79-4-4 1.79-4 4-4V0C3.58 0 0 3.58 0 8s3.58 8 8 8 8-3.58 8-8h-4z"],download:["M7.99-.01c-4.42 0-8 3.58-8 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zM11.7 9.7l-3 3c-.18.18-.43.29-.71.29s-.53-.11-.71-.29l-3-3A1.003 1.003 0 015.7 8.28l1.29 1.29V3.99c0-.55.45-1 1-1s1 .45 1 1v5.59l1.29-1.29a1.003 1.003 0 011.71.71c0 .27-.11.52-.29.7z"],"drag-handle-horizontal":["M2 9c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm0-4c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm4 4c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm8-2c.55 0 1-.45 1-1s-.45-1-1-1-1 .45-1 1 .45 1 1 1zm0 2c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm-4-4c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm0 4c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zM6 5c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1z"],"drag-handle-vertical":["M6 9c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm4-6c.55 0 1-.45 1-1s-.45-1-1-1-1 .45-1 1 .45 1 1 1zM6 13c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm0-8c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm0-4c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm4 8c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm0-4c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm0 8c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1z"],draw:["M14.9 11c-.3 0-.5.1-.7.3l-3 3c-.2.2-.3.4-.3.7 0 .6.5 1 1 1 .3 0 .5-.1.7-.3l3-3c.2-.2.3-.4.3-.7 0-.5-.4-1-1-1zm-1-1v-.2l-1-5c-.1-.3-.3-.6-.6-.7l-11-4-.3.3 5.8 5.8c.2-.1.4-.2.6-.2.8 0 1.5.7 1.5 1.5S8.3 9 7.4 9s-1.5-.7-1.5-1.5c0-.2.1-.4.2-.6L.3 1.1l-.3.3 4 11c.1.3.4.6.7.6l5 1h.2c.3 0 .5-.1.7-.3l3-3c.2-.2.3-.4.3-.7z"],"drawer-left":["M7 0a1 1 0 011 1v14a1 1 0 01-1 1H1a1 1 0 01-1-1V1a1 1 0 011-1h6zM6 2H2v12h4V2zm2 5h4.59L11.3 5.71A.965.965 0 0111 5a1.003 1.003 0 011.71-.71l3 3c.18.18.29.43.29.71 0 .28-.11.53-.29.71l-3 3a1.003 1.003 0 01-1.42-1.42L12.59 9H8V7z"],"drawer-left-filled":["M1 0h6a1 1 0 011 1v14a1 1 0 01-1 1H1a1 1 0 01-1-1V1a1 1 0 011-1zm7 7h4.59L11.3 5.71A.965.965 0 0111 5a1.003 1.003 0 011.71-.71l3 3c.18.18.29.43.29.71 0 .28-.11.53-.29.71l-3 3a1.003 1.003 0 01-1.42-1.42L12.59 9H8V7z"],"drawer-right":["M15 0a1 1 0 011 1v14a1 1 0 01-1 1H9a1 1 0 01-1-1V1a1 1 0 011-1h6zm-1 2h-4v12h4V2zM8 7H3.41L4.7 5.71c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71l-3 3C.11 7.47 0 7.72 0 8c0 .28.11.53.29.71l3 3a1.003 1.003 0 001.42-1.42L3.41 9H8V7z"],"drawer-right-filled":["M9 0h6a1 1 0 011 1v14a1 1 0 01-1 1H9a1 1 0 01-1-1V1a1 1 0 011-1zM8 7H3.41L4.7 5.71c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71l-3 3C.11 7.47 0 7.72 0 8c0 .28.11.53.29.71l3 3a1.003 1.003 0 001.42-1.42L3.41 9H8V7z"],"drive-time":["M15.12 4.76h-1.05l-.76-2.12c-.19-.53-.76-1.08-1.27-1.24 0 0-1.32-.4-4.04-.4-2.72 0-4.04.4-4.04.4-.5.16-1.07.71-1.26 1.24l-.77 2.12H.88c-.48 0-.88.42-.88.94s.4.94.88.94h.38L1 7c-.03.69 0 1.44 0 2v5c0 .66.38 1 1 1s1-.34 1-1v-1h10v1c0 .66.38 1 1 1s1-.34 1-1V9c0-.56-.01-1.37 0-2l-.26-.37h.38c.48 0 .88-.42.88-.93 0-.52-.4-.94-.88-.94zM5 10H3V8h2v2zm8 0h-2V8h2v2zm0-4H3c-.18 0-.06-.82 0-1l.73-1.63C3.79 3.19 3.82 3 4 3h8c.18 0 .21.19.27.37L13 5c.06.18.18 1 0 1z"],duplicate:["M15 0H5c-.55 0-1 .45-1 1v2h2V2h8v7h-1v2h2c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-4 4H1c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1h10c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1zm-1 10H2V6h8v8z"],edit:["M3.25 10.26l2.47 2.47 6.69-6.69-2.46-2.48-6.7 6.7zM.99 14.99l3.86-1.39-2.46-2.44-1.4 3.83zm12.25-14c-.48 0-.92.2-1.24.51l-1.44 1.44 2.47 2.47 1.44-1.44c.32-.32.51-.75.51-1.24.01-.95-.77-1.74-1.74-1.74z"],eject:["M4 9h8a1.003 1.003 0 00.71-1.71l-4-4C8.53 3.11 8.28 3 8 3s-.53.11-.71.29l-4 4A1.003 1.003 0 004 9zm8 1H4c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h8c.55 0 1-.45 1-1v-1c0-.55-.45-1-1-1z"],emoji:["M8 0a8 8 0 110 16A8 8 0 018 0zm0 1a7 7 0 100 14A7 7 0 008 1zM4 8c.228 2.262 2 4 4 4 1.938 0 3.77-1.738 3.984-3.8L12 8h1c-.128 2.888-2.317 5-5 5a5 5 0 01-4.995-4.783L3 8h1zm2-3a1 1 0 110 2 1 1 0 010-2zm4 0a1 1 0 110 2 1 1 0 010-2z"],endorsed:["M15.86 7.5l-.81-1.42V4.5c0-.36-.19-.68-.49-.87l-1.37-.8-.81-1.41c-.19-.31-.51-.49-.86-.49H9.89L8.5.14a.948.948 0 00-1 0l-1.39.8H4.52a1 1 0 00-.86.49l-.8 1.37-1.44.83c-.3.19-.49.51-.49.87v1.65l-.8 1.37c-.08.15-.13.32-.13.49s.05.34.14.49l.8 1.37v1.65c0 .36.19.68.49.87l1.42.81.8 1.37c.19.31.51.49.86.49H6.1l1.39.8c.15.09.32.14.48.14s.34-.05.49-.14l1.39-.8h1.63a1 1 0 00.86-.49l.81-1.41 1.37-.8c.3-.19.49-.51.49-.87V9.93l.81-1.42a.89.89 0 00.04-1.01zm-4.12-.82l-4.01 4.01c-.18.18-.43.29-.71.29s-.53-.11-.71-.29l-2-2c-.18-.19-.3-.44-.3-.71a1.003 1.003 0 011.71-.71l1.3 1.3 3.3-3.3a1.003 1.003 0 011.71.71.95.95 0 01-.29.7z"],envelope:["M0 3.06v9.88L4.94 8 0 3.06zM14.94 2H1.06L8 8.94 14.94 2zm-6.41 8.53c-.14.14-.32.22-.53.22s-.39-.08-.53-.22L6 9.06 1.06 14h13.88L10 9.06l-1.47 1.47zM11.06 8L16 12.94V3.06L11.06 8z"],equals:["M3 5h10a1 1 0 010 2H3a1 1 0 110-2zm0 4h10a1 1 0 010 2H3a1 1 0 010-2z"],eraser:["M8.06 13.91l7.63-7.44c.41-.4.41-1.05 0-1.45L10.86.3c-.41-.4-1.08-.4-1.49 0L.31 9.13c-.41.4-.41 1.05 0 1.45l5.58 5.44h8.12v-.01c.55 0 1-.45 1-1s-.45-1-1-1H7.96l.1-.1zm-2.17.06L1.67 9.85l4.22-4.11 4.22 4.11-4.22 4.12z"],error:["M7.99-.01c-4.42 0-8 3.58-8 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm1 13h-2v-2h2v2zm0-3h-2v-7h2v7z"],euro:["M6.52 3.18c.51-.27 1.12-.4 1.83-.4.48 0 .91.06 1.27.18.37.12.68.29.96.51.18.14.3.33.44.51l1.53-1.53c-.12-.11-.23-.22-.36-.32a5.61 5.61 0 00-1.74-.83c-.66-.2-1.36-.3-2.1-.3-.99 0-1.88.18-2.66.53-.79.35-1.45.82-2 1.41-.55.58-.96 1.27-1.26 2.06H2c-.55 0-1 .45-1 1s.45 1 1 1h.04c-.01.17-.04.33-.04.5 0 .17.03.33.04.5H2c-.55 0-1 .45-1 1s.45 1 1 1h.43c0 .01 0 .02.01.02a6.2 6.2 0 001.25 2.07 5.77 5.77 0 002 1.4c.78.34 1.67.51 2.66.51.81 0 1.54-.12 2.21-.36.67-.24 1.25-.59 1.75-1.03l.03-.03-1.55-1.33c-.01.01-.02.03-.03.04-.29.3-.63.53-1.02.69-.4.17-.85.25-1.37.25-.71 0-1.32-.13-1.83-.4s-.93-.62-1.25-1.07c-.19-.24-.34-.49-.46-.76H9c.55 0 1-.45 1-1s-.45-1-1-1H4.35c-.01-.17-.03-.33-.03-.5 0-.17.02-.34.03-.5H10c.55 0 1-.45 1-1s-.45-1-1-1H4.83c.13-.27.27-.52.44-.76.32-.44.74-.8 1.25-1.06zM14 8.98v0z"],exchange:["M1.99 5.99c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.89-2-2-2zm4.15 1.86a.495.495 0 10.7-.7L5.7 5.99h5.79c.28 0 .5-.22.5-.5s-.22-.5-.5-.5H5.7l1.15-1.15a.495.495 0 10-.7-.7l-2 2c-.1.09-.16.21-.16.35s.06.26.15.35l2 2.01zm7.85-1.86c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.89-2-2-2zM9.85 8.14a.533.533 0 00-.36-.15.495.495 0 00-.35.85l1.15 1.15h-5.8c-.28 0-.5.22-.5.5s.22.5.5.5h5.79l-1.15 1.15a.495.495 0 10.7.7l2-2c.09-.09.15-.22.15-.35s-.06-.26-.15-.35l-1.98-2z"],"exclude-row":["M0 10a1.003 1.003 0 001.71.71L3 9.41l1.29 1.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71L4.41 8 5.7 6.71c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71L3 6.59l-1.29-1.3A1.003 1.003 0 00.29 6.71L1.59 8 .29 9.29C.11 9.47 0 9.72 0 10zm1-7h14c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1zm14 10H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zm-1-7H9c-1.1 0-2 .9-2 2s.9 2 2 2h5c1.1 0 2-.9 2-2s-.9-2-2-2z"],"expand-all":["M4 7c.28 0 .53-.11.71-.29L8 3.41l3.29 3.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71l-4-4C8.53 1.11 8.28 1 8 1s-.53.11-.71.29l-4 4A1.003 1.003 0 004 7zm8 2c-.28 0-.53.11-.71.29L8 12.59l-3.29-3.3a1.003 1.003 0 00-1.42 1.42l4 4c.18.18.43.29.71.29s.53-.11.71-.29l4-4A1.003 1.003 0 0012 9z"],export:["M4 6c.28 0 .53-.11.71-.29L7 3.41V11c0 .55.45 1 1 1s1-.45 1-1V3.41l2.29 2.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71l-4-4C8.53.11 8.28 0 8 0s-.53.11-.71.29l-4 4A1.003 1.003 0 004 6zm11 5c-.55 0-1 .45-1 1v2H2v-2c0-.55-.45-1-1-1s-1 .45-1 1v3c0 .55.45 1 1 1h14c.55 0 1-.45 1-1v-3c0-.55-.45-1-1-1z"],"eye-off":["M16 7.97v-.02-.01-.02-.02a.672.672 0 00-.17-.36c-.49-.63-1.07-1.2-1.65-1.72l-3.16 2.26a2.978 2.978 0 01-2.98 2.9c-.31 0-.6-.06-.88-.15L5.09 12.3c.44.19.9.36 1.37.47.97.23 1.94.24 2.92.05.88-.17 1.74-.54 2.53-.98 1.25-.7 2.39-1.67 3.38-2.75.18-.2.37-.41.53-.62.09-.1.15-.22.17-.36v-.02-.02-.01-.02-.03c.01-.02.01-.03.01-.04zm-.43-4.17c.25-.18.43-.46.43-.8 0-.55-.45-1-1-1-.22 0-.41.08-.57.2l-.01-.01-2.67 1.91c-.69-.38-1.41-.69-2.17-.87a6.8 6.8 0 00-2.91-.05c-.88.18-1.74.54-2.53.99-1.25.7-2.39 1.67-3.38 2.75-.18.2-.37.41-.53.62-.23.29-.23.63-.01.92.51.66 1.11 1.25 1.73 1.79.18.16.38.29.56.44l-2.09 1.5.01.01c-.25.18-.43.46-.43.8 0 .55.45 1 1 1 .22 0 .41-.08.57-.2l.01.01 14-10-.01-.01zm-10.41 5a3.03 3.03 0 01-.11-.8 2.99 2.99 0 012.99-2.98c.62 0 1.19.21 1.66.53L5.16 8.8z"],"eye-on":["M10.29 6.7c.18.18.43.29.71.29s.53-.11.71-.29l4-4c.17-.18.29-.43.29-.7a1.003 1.003 0 00-1.71-.71L11 4.58 9.71 3.29A.997.997 0 009 3c-.55 0-1 .44-1 1a1 1 0 00.3.7l1.99 2zM16 7.96v-.02-.01-.02-.02a.64.64 0 00-.17-.36c-.3-.4-.65-.76-1-1.12l-1.7 1.7c-.55.55-1.3.88-2.13.88-.06 0-.11-.01-.17-.02C10.42 10.15 9.32 11 8.01 11A3.005 3.005 0 016.4 5.46c-.24-.43-.39-.93-.39-1.46 0-.26.04-.5.1-.74-.7.2-1.37.5-2.01.86-1.26.7-2.4 1.68-3.4 2.77-.18.21-.36.41-.53.63-.22.29-.22.64 0 .93.51.67 1.12 1.27 1.73 1.81 1.33 1.17 2.85 2.15 4.53 2.55.97.23 1.95.24 2.92.05.89-.18 1.74-.54 2.54-.99 1.25-.71 2.4-1.69 3.39-2.78.18-.2.37-.41.54-.63.09-.1.15-.23.17-.37v-.02-.02-.01-.02-.03c.01-.01.01-.02.01-.03zM8.01 9c.48 0 .87-.35.96-.81a.55.55 0 01-.07-.09l-.02.01L7.8 7.03c-.45.1-.79.48-.79.96 0 .56.45 1.01 1 1.01z"],"eye-open":["M8.002 7.003a1.003 1.003 0 000 2.005 1.003 1.003 0 000-2.005zm7.988.972v-.02-.01-.02-.02a.675.675 0 00-.17-.36c-.509-.673-1.118-1.264-1.737-1.806-1.328-1.173-2.846-2.155-4.523-2.546a6.702 6.702 0 00-2.925-.06c-.889.18-1.738.541-2.546.992C2.84 4.837 1.692 5.81.694 6.902c-.18.211-.36.411-.53.632a.742.742 0 000 .932c.51.672 1.119 1.264 1.738 1.805 1.328 1.173 2.846 2.156 4.523 2.547.968.23 1.947.24 2.925.04.889-.18 1.738-.542 2.546-.993 1.248-.712 2.397-1.684 3.395-2.777.18-.2.37-.411.54-.632.09-.1.149-.23.169-.36v-.02-.02-.01-.02-.03c0-.01-.01-.01-.01-.02zm-7.988 3.038a2.998 2.998 0 01-2.995-3.008 2.998 2.998 0 012.995-3.008 2.998 2.998 0 012.996 3.008 2.998 2.998 0 01-2.996 3.008z"],"fast-backward":["M14 3c-.24 0-.44.09-.62.23l-.01-.01L9 6.72V4c0-.55-.45-1-1-1-.24 0-.44.09-.62.23v-.01l-5 4 .01.01C2.16 7.41 2 7.68 2 8s.16.59.38.77v.01l5 4 .01-.01c.17.14.37.23.61.23.55 0 1-.45 1-1V9.28l4.38 3.5.01-.01c.17.14.37.23.61.23.55 0 1-.45 1-1V4c0-.55-.45-1-1-1z"],"fast-forward":["M15 8c0-.32-.16-.59-.38-.77l.01-.01-5-4-.01.01A.987.987 0 009 3c-.55 0-1 .45-1 1v2.72l-4.38-3.5v.01A.987.987 0 003 3c-.55 0-1 .45-1 1v8c0 .55.45 1 1 1 .24 0 .44-.09.62-.23l.01.01L8 9.28V12c0 .55.45 1 1 1 .24 0 .44-.09.62-.23l.01.01 5-4-.01-.01c.22-.18.38-.45.38-.77z"],feed:["M1.99 11.99c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.89-2-2-2zm1-4c-.55 0-1 .45-1 1s.45 1 1 1c1.66 0 3 1.34 3 3 0 .55.45 1 1 1s1-.45 1-1c0-2.76-2.24-5-5-5zm0-4c-.55 0-1 .45-1 1s.45 1 1 1c3.87 0 7 3.13 7 7 0 .55.45 1 1 1s1-.45 1-1a9 9 0 00-9-9zm0-4c-.55 0-1 .45-1 1s.45 1 1 1c6.08 0 11 4.92 11 11 0 .55.45 1 1 1s1-.45 1-1c0-7.18-5.82-13-13-13z"],"feed-subscribed":["M3 2c1.06 0 2.08.16 3.06.45.13-.71.52-1.32 1.05-1.76C5.82.25 4.44 0 3 0c-.55 0-1 .45-1 1s.45 1 1 1zM2 12c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm8.32-6.33a.99.99 0 001.4 0l3.98-3.98c.19-.18.3-.42.3-.7 0-.55-.45-.99-1-.99-.28 0-.52.11-.7.29l-3.28 3.28-1.29-1.29a.99.99 0 00-.7-.29 1 1 0 00-1 .99c0 .27.11.52.29.7l2 1.99zm3.73.53l-.93.93-.02-.02c-.17.17-.35.33-.56.45C13.47 9.16 14 11.02 14 13c0 .55.45 1 1 1s1-.45 1-1c0-2.5-.73-4.82-1.95-6.8zM3 8c-.55 0-1 .45-1 1s.45 1 1 1c1.66 0 3 1.34 3 3 0 .55.45 1 1 1s1-.45 1-1c0-2.76-2.24-5-5-5zm5.91-.91l-.03.03-2-2 .03-.03c-.11-.11-.23-.2-.33-.33A8.9 8.9 0 003 4c-.55 0-1 .45-1 1s.45 1 1 1c3.87 0 7 3.13 7 7 0 .55.45 1 1 1s1-.45 1-1c0-1.87-.57-3.61-1.55-5.06-.61-.11-1.13-.42-1.54-.85z"],film:["M15 1h-5v2H6V1H1c-.55 0-1 .45-1 1v11c0 .55.45 1 1 1h5v-2h4v2h5c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1zM4 13H2v-2h2v2zm0-3H2V8h2v2zm0-3H2V5h2v2zm0-3H2V2h2v2zm6 6H6V5h4v5zm4 3h-2v-2h2v2zm0-3h-2V8h2v2zm0-3h-2V5h2v2zm0-3h-2V2h2v2z"],filter:["M13.99.99h-12a1.003 1.003 0 00-.71 1.71l4.71 4.71V14a1.003 1.003 0 001.71.71l2-2c.18-.18.29-.43.29-.71V7.41L14.7 2.7a1.003 1.003 0 00-.71-1.71z"],"filter-keep":["M15 10c-.28 0-.53.11-.71.29L12 12.59l-1.29-1.29A.965.965 0 0010 11a1.003 1.003 0 00-.71 1.71l2 2c.18.18.43.29.71.29s.53-.11.71-.29l3-3A1.003 1.003 0 0015 10zm-3-8c0-.55-.45-1-1-1H1a1.003 1.003 0 00-.71 1.71L4 6.41V12a1.003 1.003 0 001.71.71l2-2c.18-.18.29-.43.29-.71V6.41l3.71-3.71c.18-.17.29-.42.29-.7z"],"filter-list":["M9 8c0 .55.45 1 1 1h5c.55 0 1-.45 1-1s-.45-1-1-1h-5c-.55 0-1 .45-1 1zm3-6c0-.55-.45-1-1-1H1a1.003 1.003 0 00-.71 1.71L4 6.41V12a1.003 1.003 0 001.71.71l2-2c.18-.18.29-.43.29-.71V6.41l3.71-3.71c.18-.17.29-.42.29-.7zm3 8h-5c-.55 0-1 .45-1 1s.45 1 1 1h5c.55 0 1-.45 1-1s-.45-1-1-1zm0 3h-5c-.55 0-1 .45-1 1s.45 1 1 1h5c.55 0 1-.45 1-1s-.45-1-1-1z"],"filter-open":["M15.707 10.293a1 1 0 010 1.414l-3 3c-.63.63-1.707.184-1.707-.707V8c0-.89 1.077-1.337 1.707-.707l3 3zM12 2c0 .28-.11.53-.29.7L8 6.41V10c0 .28-.11.53-.29.71l-2 2A1.003 1.003 0 014 12V6.41L.29 2.71A1.003 1.003 0 011 1h10c.55 0 1 .45 1 1z"],"filter-remove":["M12 2c0-.55-.45-1-1-1H1a1.003 1.003 0 00-.71 1.71L4 6.41V12a1.003 1.003 0 001.71.71l2-2c.18-.18.29-.43.29-.71V6.41l3.71-3.71c.18-.17.29-.42.29-.7zm2.41 10l1.29-1.29c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71L13 10.59 11.71 9.3A.965.965 0 0011 9a1.003 1.003 0 00-.71 1.71l1.3 1.29-1.29 1.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71l1.29-1.3 1.29 1.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71L14.41 12z"],flag:["M2.99 2.99c-.55 0-1 .45-1 1v11c0 .55.45 1 1 1s1-.45 1-1v-11c0-.55-.45-1-1-1zm0-3c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm2 3.03v7.23c2.07-2.11 5.92 1.75 9 0V3.02c-3 2.07-6.94-2.03-9 0z"],flame:["M9.217 0c0 1.368.368 2.462 1.104 3.282C12.774 5.197 14 7.385 14 9.846c0 2.735-1.472 4.786-4.415 6.154 2.165-2.4 1.84-3.385-.368-6.4-2.342 1.2-1.967 2-1.592 3.6-.786 0-1.5 0-1.875-.4 0 .547.898 2 1.464 3.2-2.943-.82-6.092-5.744-4.988-6.154.736-.273 1.594-.137 2.575.41C3.575 5.333 5.047 1.915 9.217 0z"],flash:["M4 8c0-.55-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1h2c.55 0 1-.45 1-1zm4-4c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1S7 .45 7 1v2c0 .55.45 1 1 1zM3.79 5.21a1.003 1.003 0 001.42-1.42l-1.5-1.5a1.003 1.003 0 00-1.42 1.42l1.5 1.5zm.71 5.29c-.28 0-.53.11-.71.29l-1.5 1.5a1.003 1.003 0 001.42 1.42l1.5-1.5a1.003 1.003 0 00-.71-1.71zm7-5c.28 0 .53-.11.71-.29l1.5-1.5a1.003 1.003 0 00-1.42-1.42l-1.5 1.5a1.003 1.003 0 00.71 1.71zm.71 5.29a1.003 1.003 0 00-1.42 1.42l1.5 1.5a1.003 1.003 0 001.42-1.42l-1.5-1.5zM15 7h-2c-.55 0-1 .45-1 1s.45 1 1 1h2c.55 0 1-.45 1-1s-.45-1-1-1zM8 5C6.34 5 5 6.34 5 8s1.34 3 3 3 3-1.34 3-3-1.34-3-3-3zm0 4c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zm0 3c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1s1-.45 1-1v-2c0-.55-.45-1-1-1z"],"floppy-disk":["M15.71 2.29l-2-2A.997.997 0 0013 0h-1v6H4V0H1C.45 0 0 .45 0 1v14c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V3c0-.28-.11-.53-.29-.71zM14 15H2V9c0-.55.45-1 1-1h10c.55 0 1 .45 1 1v6zM11 1H9v4h2V1z"],"flow-branch":["M10.643 6.595c.22.418.344.894.344 1.399 0 .439-.094.855-.263 1.231l3.265 3.462-.002-1.75a.973.973 0 01.314-.68.99.99 0 011.388.048c.186.2.316.46.3.715l-.009 4.03a.959.959 0 01-.3.68.972.972 0 01-.698.266l-4.053.002a.97.97 0 01-.679-.314 1.031 1.031 0 01.05-1.42.972.972 0 01.698-.266l1.7-.001-3.305-3.35a2.998 2.998 0 01-4.228-1.653H.999a1 1 0 010-2h4.166a2.998 2.998 0 014.06-1.735l3.449-3.268-1.745.002a.979.979 0 01-.631-1.692c.199-.186.459-.316.713-.3l4.025.009c.247.008.493.1.679.3.186.2.274.451.265.7l.002 4.046a.972.972 0 01-.313.68 1.03 1.03 0 01-1.42-.05.973.973 0 01-.266-.7V3.295l-3.34 3.301z"],"flow-end":["M9.702 7.31c.176.176.293.41.293.684a.976.976 0 01-.283.695c-1.888 1.91-2.892 2.918-3.011 3.027-.179.164-.42.284-.693.284a.995.995 0 01-.997-.985c0-.274.112-.541.292-.72.12-.12.624-.551 1.514-1.293H.98c-.536 0-.975-.47-.975-1.008 0-.537.439-.996.975-.996h5.837c-.895-.752-1.4-1.187-1.514-1.304a1.03 1.03 0 01-.292-.705C5.01 4.45 5.464 4 6 4c.272 0 .52.108.695.294A535.7 535.7 0 009.702 7.31zM13 11.002c-1.657 0-3-1.347-3-3.008a3.004 3.004 0 013-3.007c1.657 0 3 1.346 3 3.007a3.004 3.004 0 01-3 3.008z"],"flow-linear":["M4.16 9.002H.977C.44 9.002 0 8.532 0 7.994c0-.537.44-.99.978-.99h3.18A3.01 3.01 0 016.995 5a3.01 3.01 0 012.839 2.004h2.98c-.898-.756-1.404-1.193-1.518-1.31a1.03 1.03 0 01-.293-.705c0-.538.454-.989.992-.989.274 0 .521.108.697.294.118.124 1.122 1.13 3.014 3.016a.96.96 0 01.293.684.975.975 0 01-.284.695l-3.018 3.027a.974.974 0 01-.694.284c-.553 0-1-.447-1-.985 0-.274.117-.545.293-.72l1.518-1.293H9.833A3.01 3.01 0 016.996 11 3.01 3.01 0 014.16 9.002z"],"flow-review":["M5.175 7.004a3.003 3.003 0 012.83-2.001c1.305 0 2.416.835 2.83 2.001h1.985c-.896-.756-1.401-1.193-1.515-1.31a1.03 1.03 0 01-.292-.705c0-.538.453-.989.99-.989a.95.95 0 01.696.294c.117.124 1.12 1.13 3.008 3.016.176.176.293.41.293.684a.976.976 0 01-.283.695l-3.013 3.027a.995.995 0 01-1.691-.702c0-.273.116-.544.292-.72l1.515-1.292h-1.98a3.003 3.003 0 01-2.835 2.016A3.003 3.003 0 015.17 9.002H3.18l1.515 1.292c.176.176.292.447.292.72a.995.995 0 01-1.69.702L.282 8.69A.976.976 0 010 7.994c0-.273.117-.508.293-.684A535.858 535.858 0 003.3 4.294.95.95 0 013.997 4c.537 0 .99.45.99.989 0 .273-.12.528-.292.705-.114.117-.62.554-1.515 1.31h1.995z"],"flow-review-branch":["M10.392 10.647A3.002 3.002 0 016.16 8.995H3.37l1.338 1.318c.172.178.287.41.282.683-.01.536-.524.995-.99.995-.465 0-.63-.187-.747-.294L.281 8.682A.956.956 0 010 7.994a.971.971 0 01.294-.687l3.01-3.028a.973.973 0 01.697-.27c.536.01.998.485.989 1.021a.971.971 0 01-.295.687L3.37 6.997h2.79a3.002 3.002 0 014.106-1.716l2.416-2.277-1.732.004a.99.99 0 01-.679-.329.978.978 0 01.05-1.378c.199-.186.459-.315.714-.3l4.012.005c.248.009.493.1.68.3.185.2.273.45.264.699L15.99 6.05a.973.973 0 01-.314.679 1.03 1.03 0 01-1.421-.048.971.971 0 01-.265-.699V4.29L11.65 6.602c.219.416.343.89.343 1.394 0 .451-.1.88-.279 1.263L14 11.68l-.004-1.73a.982.982 0 01.323-.68.978.978 0 011.378.049c.187.2.316.46.3.714l-.004 4.011a.983.983 0 01-.3.691.972.972 0 01-.7.265l-4.046-.001a.987.987 0 01-.679-.326 1.017 1.017 0 01.048-1.41.972.972 0 01.699-.265h1.693l-2.315-2.35z"],flows:["M13.5 6a2.5 2.5 0 00-2.45 2h-1.3L5.74 4l-.75.75L8.25 8h-3.3a2.5 2.5 0 100 1h3.3l-3.26 3.25.75.75 4.01-4h1.3a2.5 2.5 0 102.45-3z"],"folder-close":["M-.01 14c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V7h-16v7zm15-10H7.41L5.7 2.3a.965.965 0 00-.71-.3h-4c-.55 0-1 .45-1 1v3h16V5c0-.55-.45-1-1-1z"],"folder-new":["M10.165 7a3.003 3.003 0 002.827 2 3.003 3.003 0 002.827-2H16v7c0 .55-.45 1-1 1H1.01c-.55 0-1-.45-1-1V7h10.155zM8.76 6H0V3c0-.55.45-1 1-1h1.998c.28 0 .53.11.71.29L5.417 4h2.578c0 .768.29 1.469.765 2zm6.23-3c.55 0 1 .45 1 1s-.45 1-1 1h-.999v1c0 .55-.45 1-1 1-.549 0-.998-.45-.998-1V5h-1c-.55 0-1-.45-1-1s.45-1 1-1h1V2c0-.55.45-1 .999-1 .55 0 1 .45 1 1v1h.999z"],"folder-open":["M2.06 6.69c.14-.4.5-.69.94-.69h11V5c0-.55-.45-1-1-1H6.41l-1.7-1.71A.997.997 0 004 2H1c-.55 0-1 .45-1 1v9.84l2.05-6.15h.01zM16 8c0-.55-.45-1-1-1H4a.99.99 0 00-.94.69l-2 6c-.04.09-.06.2-.06.31 0 .55.45 1 1 1h11c.44 0 .81-.29.94-.69l2-6c.04-.09.06-.2.06-.31z"],"folder-shared":["M8.76 5.98c-.47-.53-.77-1.22-.77-1.99h-.58L5.7 2.29a.965.965 0 00-.71-.3h-4c-.55 0-1 .45-1 1v3h8.76l.01-.01zm6.23-2.99h-4c-.55 0-1 .45-1 1s.45 1 1 1h1.59l-3.3 3.3a.99.99 0 00-.29.7 1.003 1.003 0 001.71.71l3.29-3.29V8c0 .55.45 1 1 1s1-.45 1-1V4c0-.56-.45-1.01-1-1.01zm-1.98 7.23l-.9.9-.01-.01c-.54.55-1.28.89-2.11.89-1.66 0-3-1.34-3-3 0-.77.3-1.47.78-2H-.01v7c0 .55.45 1 1 1h14c.55 0 1-.45 1-1v-3.18c-.31.11-.65.18-1 .18-.76-.01-1.45-.31-1.98-.78z"],"folder-shared-open":["M13.02 10.22l-.9.9-.01-.01c-.54.55-1.28.89-2.11.89-1.66 0-3-1.34-3-3 0-.77.3-1.47.78-2H4a.99.99 0 00-.94.69l-2 6c-.04.09-.06.2-.06.31 0 .55.45 1 1 1h11c.44 0 .81-.29.94-.69l1.11-3.32c-.01 0-.03.01-.05.01-.77 0-1.45-.3-1.98-.78zM2.06 6.69c.14-.4.5-.69.94-.69h5.76l.01-.01C8.3 5.46 8 4.77 8 4H6.41l-1.7-1.71A.997.997 0 004 2H1c-.55 0-1 .45-1 1v9.84l2.05-6.15h.01zM15 3h-4c-.55 0-1 .45-1 1s.45 1 1 1h1.59l-3.3 3.29a1.003 1.003 0 001.42 1.42L14 6.41V8c0 .55.45 1 1 1s1-.45 1-1V4c0-.55-.45-1-1-1z"],follower:["M9.37 12.69c-1.2-.53-1.04-.85-1.08-1.29-.01-.06-.01-.12-.01-.19.41-.37.75-.87.97-1.44 0 0 .01-.03.01-.04.05-.13.09-.26.12-.39.28-.06.44-.36.5-.63.06-.11.19-.39.16-.7-.04-.4-.2-.59-.38-.67v-.07c0-.52-.05-1.26-.14-1.74a2.72 2.72 0 00-.09-.43 3.02 3.02 0 00-1.04-1.51C7.87 3.2 7.15 3 6.5 3c-.64 0-1.36.2-1.87.59-.5.38-.87.92-1.05 1.51-.04.13-.07.27-.09.4-.09.49-.14 1.24-.14 1.75v.06c-.19.07-.36.26-.4.68-.03.31.1.59.16.7.06.28.23.59.51.64.04.14.08.27.13.39 0 .01.01.02.01.02v.01c.22.59.57 1.1.99 1.46 0 .06-.01.12-.01.17-.04.44.08.76-1.12 1.29-1.2.53-3.01 1.1-3.38 1.95C-.13 15.5.02 16 .02 16h12.96s.15-.5-.22-1.36c-.38-.85-2.19-1.42-3.39-1.95zm6.33-10.4l-2-2a1.003 1.003 0 00-1.42 1.42l.3.29H9.99c-.55 0-1 .45-1 1s.45 1 1 1h2.58l-.29.29a1.003 1.003 0 001.42 1.42l2-2c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71z"],following:["M9.37 12.69c-1.2-.53-1.04-.85-1.08-1.29-.01-.06-.01-.12-.01-.19.41-.37.75-.87.97-1.44 0 0 .01-.03.01-.04.05-.13.09-.26.12-.39.28-.06.44-.36.5-.63.06-.11.19-.39.16-.7-.04-.4-.2-.59-.38-.67v-.07c0-.52-.05-1.26-.14-1.74a2.72 2.72 0 00-.09-.43 3.02 3.02 0 00-1.04-1.51C7.87 3.2 7.15 3 6.5 3c-.64 0-1.36.2-1.87.59-.5.38-.87.92-1.05 1.51-.04.13-.07.27-.09.4-.09.49-.14 1.24-.14 1.75v.06c-.19.07-.36.26-.4.68-.03.31.1.59.16.7.06.28.23.59.51.64.04.14.08.27.13.39 0 .01.01.02.01.02v.01c.22.59.57 1.1.99 1.46 0 .06-.01.12-.01.17-.04.44.08.76-1.12 1.29-1.2.53-3.01 1.1-3.38 1.95C-.13 15.5.02 16 .02 16h12.96s.15-.5-.22-1.36c-.38-.85-2.19-1.42-3.39-1.95zM14.99 2h-2.58l.29-.29A1.003 1.003 0 0011.28.29l-2 2c-.17.18-.29.43-.29.71 0 .28.11.53.29.71l2 2a1.003 1.003 0 001.42-1.42L12.41 4h2.58c.55 0 1-.45 1-1s-.45-1-1-1z"],font:["M13.93 14.67L8.94.67h-.01C8.79.28 8.43 0 8 0s-.79.28-.93.67h-.01l-5 14h.01c-.04.1-.07.21-.07.33 0 .55.45 1 1 1 .43 0 .79-.28.93-.67h.01L5.49 11h5.02l1.55 4.34h.01c.14.38.5.66.93.66.55 0 1-.45 1-1 0-.12-.03-.23-.07-.33zM6.2 9L8 3.97 9.8 9H6.2z"],fork:["M13.7 9.29a1.003 1.003 0 00-1.42 1.42l.29.29H11.4l-5-5h6.17l-.29.29a1.003 1.003 0 001.42 1.42l2-2c.18-.18.29-.43.29-.71s-.11-.53-.29-.71l-2-2a1.003 1.003 0 00-1.42 1.42l.29.29H.99c-.55 0-1 .45-1 1s.45 1 1 1h2.59l6.71 6.71c.18.18.43.29.71.29h1.59l-.29.29a1.003 1.003 0 001.42 1.42l2-2c.18-.18.29-.43.29-.71s-.11-.53-.29-.71l-2.02-2z"],form:["M2 11v2h2v-2H2zM1 9h4c.55 0 1 .45 1 1v4c0 .55-.45 1-1 1H1c-.55 0-1-.45-1-1v-4c0-.55.45-1 1-1zm9-6h5c.55 0 1 .45 1 1s-.45 1-1 1h-5c-.55 0-1-.45-1-1s.45-1 1-1zM6 1a1.003 1.003 0 01.71 1.71l-3 4C3.53 6.89 3.28 7 3 7s-.53-.11-.71-.29l-2-2a1.003 1.003 0 011.42-1.42L3 4.59l2.29-3.3C5.47 1.11 5.72 1 6 1zm4 10h5c.55 0 1 .45 1 1s-.45 1-1 1h-5c-.55 0-1-.45-1-1s.45-1 1-1z"],"full-circle":["M8 0a8 8 0 100 16A8 8 0 108 0z"],"full-stacked-chart":["M13 12h1c.55 0 1-.45 1-1V8h-3v3c0 .55.45 1 1 1zM10 2c0-.55-.45-1-1-1H8c-.55 0-1 .45-1 1v3h3V2zm0 4H7v3h3V6zm5-4c0-.55-.45-1-1-1h-1c-.55 0-1 .45-1 1v2h3V2zm0 3h-3v2h3V5zM5 5H2v3h3V5zm-2 7h1c.55 0 1-.45 1-1V9H2v2c0 .55.45 1 1 1zm12 1H2c-.55 0-1 .45-1 1s.45 1 1 1h13c.55 0 1-.45 1-1s-.45-1-1-1zM5 2c0-.55-.45-1-1-1H3c-.55 0-1 .45-1 1v2h3V2zm3 10h1c.55 0 1-.45 1-1v-1H7v1c0 .55.45 1 1 1z"],fullscreen:["M3.41 2H5c.55 0 1-.45 1-1s-.45-1-1-1H1C.45 0 0 .45 0 1v4c0 .55.45 1 1 1s1-.45 1-1V3.41L5.29 6.7c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71L3.41 2zM6 9c-.28 0-.53.11-.71.29L2 12.59V11c0-.55-.45-1-1-1s-1 .45-1 1v4c0 .55.45 1 1 1h4c.55 0 1-.45 1-1s-.45-1-1-1H3.41l3.29-3.29c.19-.18.3-.43.3-.71 0-.55-.45-1-1-1zm9 1c-.55 0-1 .45-1 1v1.59L10.71 9.3A.965.965 0 0010 9a1.003 1.003 0 00-.71 1.71l3.3 3.29H11c-.55 0-1 .45-1 1s.45 1 1 1h4c.55 0 1-.45 1-1v-4c0-.55-.45-1-1-1zm0-10h-4c-.55 0-1 .45-1 1s.45 1 1 1h1.59l-3.3 3.29a1.003 1.003 0 001.42 1.42L14 3.41V5c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1z"],function:["M8.12 4.74H6.98c.33-1.29.75-2.24 1.28-2.84.33-.37.64-.56.95-.56.06 0 .11.02.15.05.04.04.06.09.06.15 0 .05-.04.15-.13.29-.09.14-.13.28-.13.4 0 .18.07.33.2.46.14.13.31.19.52.19.22 0 .41-.08.56-.23.15-.16.23-.37.23-.63 0-.3-.11-.55-.34-.74C10.1 1.09 9.74 1 9.24 1c-.78 0-1.49.22-2.12.67-.64.45-1.24 1.2-1.81 2.23-.2.36-.38.59-.56.69-.18.1-.46.15-.85.15l-.26.9h1.08l-1.59 6.12c-.27 1.01-.44 1.63-.54 1.86-.14.34-.34.63-.62.87-.11.1-.24.15-.4.15a.15.15 0 01-.11-.04l-.04-.05c0-.03.04-.08.12-.16.08-.08.12-.2.12-.36 0-.18-.06-.33-.19-.44-.12-.12-.3-.18-.54-.18-.28 0-.51.08-.68.23-.16.14-.25.32-.25.53 0 .22.1.42.31.59.21.17.53.25.97.25.7 0 1.32-.18 1.87-.54.54-.36 1.02-.92 1.42-1.67.41-.75.82-1.96 1.25-3.63l.91-3.54h1.1l.29-.89zm5.43 1.52c.2-.15.41-.23.62-.23.08 0 .23.03.45.09s.41.09.57.09c.23 0 .42-.08.57-.23.16-.16.24-.36.24-.61 0-.26-.08-.47-.23-.62-.15-.15-.37-.23-.66-.23-.25 0-.5.06-.72.18-.23.12-.51.38-.86.78-.26.3-.64.81-1.15 1.55-.2-.91-.55-1.75-1.05-2.51l-2.72.46-.06.29c.2-.04.37-.06.51-.06.27 0 .49.11.67.34.28.36.67 1.45 1.17 3.26-.39.52-.66.85-.8 1.01-.24.26-.44.42-.59.5-.12.06-.25.09-.41.09-.11 0-.3-.06-.56-.18-.18-.08-.34-.12-.48-.12-.27 0-.48.08-.66.25-.17.17-.26.38-.26.64 0 .25.08.44.24.6.16.15.37.23.64.23.26 0 .5-.05.73-.16.23-.11.52-.34.86-.69.35-.35.82-.9 1.43-1.67.23.73.44 1.25.61 1.58s.37.57.59.71c.22.15.5.22.83.22.32 0 .65-.11.98-.34.44-.3.88-.81 1.34-1.53l-.26-.15c-.31.43-.54.7-.69.8-.1.07-.22.1-.35.1-.16 0-.32-.1-.48-.3-.27-.34-.62-1.27-1.06-2.8.4-.68.73-1.13 1-1.34z"],"gantt-chart":["M10 10c0 .55.45 1 1 1h4c.55 0 1-.45 1-1s-.45-1-1-1h-4c-.55 0-1 .45-1 1zM6 7c0 .55.45 1 1 1h4c.55 0 1-.45 1-1s-.45-1-1-1H7c-.55 0-1 .45-1 1zm9 5H2V3c0-.55-.45-1-1-1s-1 .45-1 1v10c0 .55.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zM4 5h3c.55 0 1-.45 1-1s-.45-1-1-1H4c-.55 0-1 .45-1 1s.45 1 1 1z"],geofence:["M6 9c.55 0 1 .45 1 1v4c0 .55-.45 1-1 1s-1-.45-1-1v-1.59l-3.29 3.3A1.003 1.003 0 010 15c0-.28.11-.53.3-.71L3.59 11H2c-.55 0-1-.45-1-1s.45-1 1-1zM9.088.004l.097.013.097.024.057.018.1.042.054.029.095.061.052.04 6 5 .05.046.076.08.053.07.06.095.051.11c.056.141.079.294.067.446l-.014.105-.037.143-.035.087-.043.083-4 7-.034.056-.059.08-.038.044-.096.092-.114.082-.116.062-.086.034-.109.03-.1.017-.069.006H8.83c.088-.25.144-.515.163-.79L9 13v-3a3 3 0 00-2.824-2.995L6 7H3c-.351 0-.689.06-1.002.171L2 5l.002-.07.013-.1.015-.073.025-.085.043-.104.056-.101.045-.066.079-.093.084-.078.083-.062 6-4 .07-.043.12-.056.111-.036.108-.022.083-.01h.031c.046-.002.083 0 .12.003z"],geolocation:["M-.01 6.66l7.34 2 2 7.33 6.66-16z"],geosearch:["M8.82 12.4h.66c.23 0 .36-.17.36-.4v-1.48l.19-.18c-.27.03-.55.06-.83.06-.28 0-.56-.03-.84-.07.02.04.05.08.07.13V12c0 .23.15.4.39.4zM6.4 15.1A5.51 5.51 0 01.9 9.6c0-.49.06-.98.18-1.43.03 0 .05-.01.08-.01h.08v.44c0 .19.17.34.36.34.03 0 .07-.01.1-.01l.71.7c.07.07.19.07.26 0s.07-.19 0-.26l-.7-.72c0-.02.03-.03.03-.05v-.11c0-.15.08-.2.23-.33h.42c.08 0 .15-.01.22-.04h.02c.02-.02.03-.02.04-.04.01-.01.01-.01.02-.01l.02-.01.9-.9c-.13-.26-.24-.52-.34-.8h-.5v-.43c0-.01.05.05.04-.08h.31c-.03-.13-.06-.26-.08-.39h-.57c.16-.12.34-.24.51-.36-.02-.23-.04-.46-.04-.7 0-.12.01-.23.02-.34A6.385 6.385 0 000 9.6C0 13.13 2.87 16 6.4 16c3.1 0 5.67-2.22 6.26-5.15l-.78-.88c-.21 2.85-2.58 5.13-5.48 5.13zm-1.7-2.93v-.28h.12c.23 0 .39-.19.39-.42v-.54s.01-.01 0-.01L3.77 9.45h-.62c-.23 0-.38.19-.38.42v1.6c0 .23.14.42.38.42h.26v1.61c0 .23.22.41.45.41s.45-.18.45-.41v-.97H4.3c.24 0 .4-.13.4-.36zm11.07-2.34l-2.94-2.94c.11-.17.21-.34.3-.52.01-.03.03-.06.04-.09.08-.18.16-.36.22-.55v-.01c.06-.19.1-.38.14-.58.01-.05.01-.09.02-.14.03-.2.05-.4.05-.61a4.4 4.4 0 00-4.4-4.4C6.77 0 4.8 1.97 4.8 4.4s1.97 4.4 4.4 4.4c.21 0 .41-.02.61-.05.04 0 .09-.01.14-.02.2-.03.39-.08.58-.14h.01c.19-.06.37-.14.55-.22.03-.01.06-.03.09-.04.18-.09.35-.19.52-.3l2.94 2.94a.8.8 0 00.57.23c.44 0 .8-.36.8-.8a.895.895 0 00-.24-.57zM9.2 7.6C7.43 7.6 6 6.17 6 4.4c0-1.77 1.43-3.2 3.2-3.2s3.2 1.43 3.2 3.2c0 1.77-1.43 3.2-3.2 3.2zm1.54 4.26v-.52c0-.09-.1-.17-.19-.17s-.19.07-.19.17v.52c0 .09.1.17.19.17s.19-.07.19-.17z"],"git-branch":["M12 1c-1.66 0-3 1.34-3 3 0 1.25.76 2.32 1.85 2.77A2.02 2.02 0 019 8H7c-.73 0-1.41.2-2 .55V5.82C6.16 5.4 7 4.3 7 3c0-1.66-1.34-3-3-3S1 1.34 1 3c0 1.3.84 2.4 2 2.82v4.37c-1.16.4-2 1.51-2 2.81 0 1.66 1.34 3 3 3s3-1.34 3-3c0-1.04-.53-1.95-1.32-2.49.35-.31.81-.51 1.32-.51h2c1.92 0 3.52-1.35 3.91-3.15A2.996 2.996 0 0012 1zM4 2c.55 0 1 .45 1 1s-.45 1-1 1-1-.45-1-1 .45-1 1-1zm0 12c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zm8-9c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1z"],"git-commit":["M15 7h-3.14c-.45-1.72-2-3-3.86-3S4.59 5.28 4.14 7H1c-.55 0-1 .45-1 1s.45 1 1 1h3.14c.45 1.72 2 3 3.86 3s3.41-1.28 3.86-3H15c.55 0 1-.45 1-1s-.45-1-1-1zm-7 3c-1.1 0-2-.9-2-2s.9-2 2-2 2 .9 2 2-.9 2-2 2z"],"git-merge":["M12 6c-1.3 0-2.4.84-2.82 2H9c-1.62 0-3-.96-3.63-2.34C6.33 5.16 7 4.16 7 3c0-1.66-1.34-3-3-3S1 1.34 1 3c0 1.3.84 2.4 2 2.81v4.37C1.84 10.6 1 11.7 1 13c0 1.66 1.34 3 3 3s3-1.34 3-3c0-1.3-.84-2.4-2-2.82V8.43A5.89 5.89 0 009 10h.18A2.996 2.996 0 0015 9c0-1.66-1.34-3-3-3zm-8 8c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zM4 4c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zm8 6c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1z"],"git-new-branch":["M14 2h-1V1c0-.55-.45-1-1-1s-1 .45-1 1v1h-1c-.55 0-1 .45-1 1s.45 1 1 1h1v1c0 .55.45 1 1 1s1-.45 1-1V4h1c.55 0 1-.45 1-1s-.45-1-1-1zm-3.18 4.8C10.51 7.51 9.82 8 9 8H7c-.73 0-1.41.2-2 .55V5.82C6.16 5.4 7 4.3 7 3c0-1.66-1.34-3-3-3S1 1.34 1 3c0 1.3.84 2.4 2 2.82v4.37c-1.16.4-2 1.51-2 2.81 0 1.66 1.34 3 3 3s3-1.34 3-3c0-1.04-.53-1.95-1.32-2.49.35-.31.81-.51 1.32-.51h2c1.9 0 3.49-1.33 3.89-3.11-.29.07-.58.11-.89.11-.41 0-.8-.08-1.18-.2zM4 2c.55 0 1 .45 1 1s-.45 1-1 1-1-.45-1-1 .45-1 1-1zm0 12c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1z"],"git-pull":["M3 1C1.34 1 0 2.34 0 4c0 1.3.84 2.4 2 2.82v3.37C.84 10.6 0 11.7 0 13c0 1.66 1.34 3 3 3s3-1.34 3-3c0-1.3-.84-2.4-2-2.82V6.82C5.16 6.4 6 5.3 6 4c0-1.66-1.34-3-3-3zm0 13c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zm0-9c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zm11 5.18V6c0-1.66-1.34-3-3-3H9.41l1.29-1.29c.19-.18.3-.43.3-.71A1.003 1.003 0 009.29.29l-3 3C6.11 3.47 6 3.72 6 4c0 .28.11.53.29.71l3 3a1.003 1.003 0 001.42-1.42L9.41 5H11c.55 0 1 .45 1 1v4.18A2.996 2.996 0 0013 16c1.66 0 3-1.34 3-3 0-1.3-.84-2.4-2-2.82zM13 14c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1z"],"git-push":["M4 6h1V5H4v1zm9 3c0-.28-.11-.53-.29-.71l-3-3C9.53 5.11 9.28 5 9 5s-.53.11-.71.29l-3 3a1.003 1.003 0 001.42 1.42L8 8.41V15c0 .55.45 1 1 1s1-.45 1-1V8.41l1.29 1.29c.18.19.43.3.71.3.55 0 1-.45 1-1zM5 3H4v1h1V3zm10-3H1C.45 0 0 .45 0 1v13c0 .55.45 1 1 1h5v-2H2v-1h4v-1H3V2h11v9h-2v1h2v1h-2v2h3c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1z"],"git-repo":["M5 9H4v1h1V9zm10-9H1C.45 0 0 .45 0 1v13c0 .55.45 1 1 1h3v1l2-1 2 1v-1h7c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zM4 13H2v-1h2v1zm10 0H8v-1h6v1zm0-2H3V2h11v9zM5 3H4v1h1V3zm0 4H4v1h1V7zm0-2H4v1h1V5z"],glass:["M2 0v4c0 2.97 2.16 5.43 5 5.91V14H5c-.55 0-1 .45-1 1s.45 1 1 1h6c.55 0 1-.45 1-1s-.45-1-1-1H9V9.91c2.84-.48 5-2.94 5-5.91V0H2z"],globe:["M4.45 7.83c-.26 0-.41.21-.41.46v1.75c0 .26.16.46.41.46h.29v1.77c0 .25.24.45.49.45s.49-.2.49-.45V11.2h-.01c.26 0 .44-.14.44-.4v-.3h.14c.26 0 .43-.2.43-.46v-.59s.01-.01 0-.01l-1.58-1.6h-.69zM8.51 3.9h.22c.06 0 .12-.01.12-.07 0-.06-.05-.07-.12-.07h-.22c-.06 0-.12.01-.12.07.01.06.06.07.12.07zm-2.33-.05c.07-.07.07-.19 0-.26l-.5-.5a.187.187 0 00-.26 0c-.07.07-.07.19 0 .26l.5.5c.07.07.19.07.26 0zm3.06.89c.07 0 .14-.06.14-.12v-.31c0-.07-.07-.12-.14-.12s-.14.06-.14.12v.31c0 .07.07.12.14.12zM8 0C3.58 0 0 3.58 0 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm0 14c-3.31 0-6-2.69-6-6 0-.55.1-1.07.23-1.57h.11v.47c0 .2.18.37.39.37.03 0 .08-.01.11-.02l.78.77c.08.08.2.08.28 0 .08-.08.08-.2 0-.28l-.75-.78c0-.02.04-.04.04-.06v-.12c0-.16.09-.22.25-.36h.46c.09 0 .17-.01.24-.05h.02c.02-.01.03-.02.05-.03.01-.01.01-.01.02-.01l.02-.02 1.59-1.58c.18-.18.18-.46 0-.64s-.47-.15-.65.03l-.3.34h-.57v-.48c0-.01.05.05.05-.09h.64c.12 0 .22-.09.22-.21s-.1-.21-.22-.21H4.1c.18-.15.34-.31.54-.44l.01-.01c.21-.14.45-.25.68-.37.15-.07.29-.15.44-.21.17-.07.35-.11.53-.17.18-.05.35-.12.53-.16a6.05 6.05 0 013.47.35c.05.02.1.05.16.08.25.11.48.24.71.39.25.16.49.34.71.55H10.6s0-.03-.01-.03c-.04 0-.09 0-.13.03l-.51.51a.17.17 0 000 .23c.06.06.17.06.23 0l.42-.44.01-.02h.25c0 .14-.07.09-.07.12v.07c0 .22-.15.37-.36.37h-.38c-.19 0-.38.21-.38.4v.17h-.1c-.12 0-.2.06-.2.18v.25h-.23c-.17 0-.3.11-.3.28 0 .17.13.26.3.26.07 0 .14.03.19-.11l.04.01.49-.46h.17l.39.37c.03.03.08.02.12-.01.03-.03.03-.12 0-.15l-.32-.35h.23l.09.12c.18.18.48.17.66-.01l.09-.1h.4c.02 0 .08.05.08.05v.24l-.05-.01h-.36c-.11 0-.21.1-.21.21 0 .11.09.21.21.21h.41v.15c-.14.21-.24.42-.45.42h-.94v-.01l-.44-.44a.47.47 0 00-.66 0l-.42.43v.01H8.6c-.26 0-.49.21-.49.46v.92c0 .26.23.45.49.45h.9c.34.14.57.35.72.69v1.68c0 .26.17.44.42.44h.72c.26 0 .4-.18.4-.44V9l.89-.86.03-.02.02-.01h.03c.07-.08.15-.19.15-.31v-.91c0-.18-.16-.32-.31-.46H13c.01.28.21.42.46.42h.42c.08.37.12.76.12 1.15 0 3.31-2.69 6-6 6zm4.54-4.27c-.1 0-.21.08-.21.18v.57c0 .1.11.18.21.18.1 0 .21-.08.21-.18v-.57c0-.1-.11-.18-.21-.18zM8.37 3.19c0-.25-.2-.42-.46-.42h-.54c-.25 0-.42.18-.42.43 0 .03-.1.04.05.08v.47c0 .15.06.27.21.27s.21-.12.21-.27v-.14h.5c.24 0 .45-.16.45-.42z"],"globe-network":["M8 0C3.58 0 0 3.58 0 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm5.17 5h-2.44c-.21-1.11-.51-2.03-.91-2.69 1.43.46 2.61 1.43 3.35 2.69zM10 8c0 .73-.05 1.39-.12 2H6.12C6.05 9.39 6 8.73 6 8s.05-1.39.12-2h3.76c.07.61.12 1.27.12 2zM8 2c.67 0 1.36 1.1 1.73 3H6.27C6.64 3.1 7.33 2 8 2zm-1.82.31c-.4.66-.71 1.58-.91 2.69H2.83a6.025 6.025 0 013.35-2.69zM2 8c0-.7.13-1.37.35-2h2.76C5.04 6.62 5 7.28 5 8s.04 1.38.11 2H2.35C2.13 9.37 2 8.7 2 8zm.83 3h2.44c.21 1.11.51 2.03.91 2.69A6.025 6.025 0 012.83 11zM8 14c-.67 0-1.36-1.1-1.73-3h3.46c-.37 1.9-1.06 3-1.73 3zm1.82-.31c.4-.66.7-1.58.91-2.69h2.44a6.025 6.025 0 01-3.35 2.69zM13.65 10h-2.76c.07-.62.11-1.28.11-2s-.04-1.38-.11-2h2.76c.22.63.35 1.3.35 2s-.13 1.37-.35 2z"],graph:["M14 3c-1.06 0-1.92.83-1.99 1.88l-1.93.97A2.95 2.95 0 008 5c-.56 0-1.08.16-1.52.43L3.97 3.34C3.98 3.23 4 3.12 4 3c0-1.1-.9-2-2-2s-2 .9-2 2 .9 2 2 2c.24 0 .47-.05.68-.13l2.51 2.09C5.08 7.29 5 7.63 5 8c0 .96.46 1.81 1.16 2.35l-.56 1.69c-.91.19-1.6.99-1.6 1.96 0 1.1.9 2 2 2s2-.9 2-2c0-.51-.2-.97-.51-1.32l.56-1.69A2.99 2.99 0 0011 8c0-.12-.02-.24-.04-.36l1.94-.97c.32.21.69.33 1.1.33 1.1 0 2-.9 2-2s-.9-2-2-2z"],"graph-remove":["M12.89 8.11l-.01.01-.38-.38-.38.38-.02-.02c-.54.55-1.27.9-2.1.9-1.66 0-3-1.34-3-3 0-.83.35-1.56.9-2.1l-.02-.02.38-.38-.38-.38.01-.01C7.35 2.57 7 1.83 7 1c0-.34.07-.65.17-.96A8.004 8.004 0 000 8c0 4.42 3.58 8 8 8 4.14 0 7.54-3.14 7.96-7.17-.31.1-.62.17-.96.17-.83 0-1.57-.35-2.11-.89zm1.02-4.61l1.79-1.79c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71l-1.79 1.8L10.71.3A.965.965 0 0010 0a1.003 1.003 0 00-.71 1.71l1.79 1.79-1.79 1.79a1.003 1.003 0 001.42 1.42l1.79-1.79 1.79 1.79a1.003 1.003 0 001.42-1.42l-1.8-1.79z"],"greater-than":["M2.713 5.958a1 1 0 01.574-1.916l10 3c.95.285.95 1.63 0 1.916l-10 3a1 1 0 01-.574-1.916L9.52 8 2.713 5.958z"],"greater-than-or-equal-to":["M2.713 3.958a1 1 0 01.574-1.916l10 3c.95.285.95 1.63 0 1.916l-10 3a1 1 0 01-.574-1.916L9.52 6 2.713 3.958zM3 12h10a1 1 0 010 2H3a1 1 0 010-2z"],grid:["M15 9c.55 0 1-.45 1-1s-.45-1-1-1h-1V4h1c.55 0 1-.45 1-1s-.45-1-1-1h-1V1c0-.55-.45-1-1-1s-1 .45-1 1v1H9V1c0-.55-.45-1-1-1S7 .45 7 1v1H4V1c0-.55-.45-1-1-1S2 .45 2 1v1H1c-.55 0-1 .45-1 1s.45 1 1 1h1v3H1c-.55 0-1 .45-1 1s.45 1 1 1h1v3H1c-.55 0-1 .45-1 1s.45 1 1 1h1v1c0 .55.45 1 1 1s1-.45 1-1v-1h3v1c0 .55.45 1 1 1s1-.45 1-1v-1h3v1c0 .55.45 1 1 1s1-.45 1-1v-1h1c.55 0 1-.45 1-1s-.45-1-1-1h-1V9h1zm-8 3H4V9h3v3zm0-5H4V4h3v3zm5 5H9V9h3v3zm0-5H9V4h3v3z"],"grid-view":["M0 1v6h7V0H1C.45 0 0 .45 0 1zm0 14c0 .55.45 1 1 1h6V9H0v6zM15 0H9v7h7V1c0-.55-.45-1-1-1zM9 16h6c.55 0 1-.45 1-1V9H9v7z"],"group-objects":["M5 6c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm6-3H5C2.24 3 0 5.24 0 8s2.24 5 5 5h6c2.76 0 5-2.24 5-5s-2.24-5-5-5zm0 9H5c-2.21 0-4-1.79-4-4s1.79-4 4-4h6c2.21 0 4 1.79 4 4s-1.79 4-4 4zm0-6c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2z"],"grouped-bar-chart":["M10 12c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1s-1 .45-1 1v8c0 .55.45 1 1 1zm3 0c.55 0 1-.45 1-1V6c0-.55-.45-1-1-1s-1 .45-1 1v5c0 .55.45 1 1 1zm2 1H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zm-9-1c.55 0 1-.45 1-1V8c0-.55-.45-1-1-1s-1 .45-1 1v3c0 .55.45 1 1 1zm-3 0c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1s-1 .45-1 1v9c0 .55.45 1 1 1z"],hand:["M15 5c0-.55-.45-1-1-1-.41 0-.76.24-.91.59v.01s0 .01-.01.01L11.57 8h-.36l.78-4.84C12 3.11 12 3.05 12 3a1 1 0 00-1.99-.16v.01L9.18 8H9V1c0-.55-.45-1-1-1S7 .45 7 1v7h-.09l-.93-5.18A1 1 0 005 2c-.55 0-1 .45-1 1 0 .05 0 .11.01.16L5.26 11h-.04L2.83 7.44C2.65 7.18 2.35 7 2 7c-.55 0-1 .45-1 1 0 .17.04.33.12.47l3 5.69h.01v.01A5.002 5.002 0 0013 11v-.59l1.93-5.05c.05-.11.07-.23.07-.36z"],"hand-down":["M14.72 7.87c-1.54-.67-2.99-2.68-3.7-3.95C10.11 1.95 9.93 0 6.14 0 4.05 0 2.71.61 1.92 2.12 1.27 3.36 1 5.21 1 7.83v.79c0 .65.6 1.18 1.35 1.18.34 0 .64-.11.88-.29.17.48.68.84 1.29.84.41 0 .78-.16 1.03-.42.23.37.67.63 1.19.63.57 0 1.05-.31 1.25-.74l.01.63v4.05c0 .83.67 1.5 1.5 1.5s1.5-.67 1.5-1.5V7.9c.58.41 1.55 1.21 2.47 1.29 1.57.14 1.82-1.07 1.25-1.32z"],"hand-left":["M12.08 4.97c-1.26-.71-3.27-2.15-3.95-3.7C7.88.7 6.67.96 6.81 2.52c.09.93.89 1.9 1.3 2.48H1.5C.67 5 0 5.67 0 6.5S.67 8 1.5 8h4.05l.63.01c-.44.2-.75.69-.75 1.25 0 .52.26.96.63 1.19-.26.25-.42.61-.42 1.03 0 .61.35 1.12.84 1.29-.18.24-.29.54-.29.88 0 .75.54 1.35 1.19 1.35h.79c2.62 0 4.47-.28 5.71-.92 1.51-.79 2.12-2.14 2.12-4.22 0-3.79-1.95-3.97-3.92-4.89z"],"hand-right":["M14.5 5H7.89c.41-.58 1.21-1.55 1.3-2.47C9.34.97 8.12.71 7.87 1.28c-.67 1.54-2.68 2.99-3.95 3.7C1.95 5.89 0 6.07 0 9.86c0 2.09.61 3.43 2.12 4.22 1.24.65 3.09.92 5.71.92h.79c.65 0 1.18-.6 1.18-1.35 0-.34-.11-.64-.29-.88.48-.17.84-.68.84-1.29 0-.41-.16-.78-.42-1.03.37-.23.63-.67.63-1.19 0-.57-.31-1.05-.74-1.25l.63-.01h4.05c.83 0 1.5-.67 1.5-1.5S15.33 5 14.5 5z"],"hand-up":["M13.65 6.19c-.34 0-.64.11-.88.29-.17-.48-.68-.84-1.29-.84-.41 0-.78.16-1.03.42-.23-.37-.67-.63-1.19-.63-.57 0-1.05.31-1.25.74L8 5.55V1.5C8 .67 7.33 0 6.5 0S5 .67 5 1.5v6.61c-.58-.41-1.55-1.21-2.48-1.3C.96 6.67.7 7.88 1.28 8.13c1.54.67 2.99 2.68 3.7 3.95C5.89 14.05 6.07 16 9.86 16c2.09 0 3.43-.61 4.22-2.12.64-1.24.92-3.09.92-5.71v-.79c0-.65-.6-1.19-1.35-1.19z"],hat:["M15 10c.495 0 .933.379.993.882L16 11v.505c0 1.461-3.524 2.45-7.707 2.493L8 14c-4.31 0-8-1-8-2.495V11c0-.561.466-1 1-1 .895 0 3 1 7 1l.381-.003C12.135 10.937 14.134 10 15 10zm-4-8c1.13 0 2.02 2.153 2.671 6.46-1.063.266-2.644.652-4.887.727l-.403.01L8 9.2c-2.664 0-4.488-.444-5.673-.74C2.98 4.153 3.87 2 5 2c2 0 1.329 2 3 2s1-2 3-2z"],header:["M13 1c-.55 0-1 .45-1 1v5H4V2c0-.55-.45-1-1-1s-1 .45-1 1v12c0 .55.45 1 1 1s1-.45 1-1V9h8v5c0 .55.45 1 1 1s1-.45 1-1V2c0-.55-.45-1-1-1z"],"header-one":["M14.06 8c-.04.23-.12.44-.25.61-.13.17-.29.3-.48.41-.18.11-.39.18-.62.23-.23.04-.46.07-.71.07v1.03h1.74V16H15V8h-.94zM7 0c-.56 0-1 .45-1 1v4H2V1c0-.55-.45-1-1-1-.56 0-1 .45-1 1v10c0 .55.45 1 1 1 .56 0 1-.45 1-1V7h4v4c0 .55.45 1 1 1 .56 0 1-.45 1-1V1c0-.54-.45-1-1-1z"],"header-two":["M13.17 13.93c-.17.15-.33.29-.46.44-.13.16-.22.32-.27.49h3.55V16H11c.01-.65.16-1.22.44-1.71s.67-.91 1.17-1.27c.24-.18.49-.36.75-.54.25-.18.49-.36.71-.57.21-.2.39-.42.53-.65.14-.24.21-.51.22-.82 0-.14-.02-.29-.05-.45-.03-.16-.09-.31-.18-.45a1.13 1.13 0 00-.37-.35c-.16-.09-.37-.14-.63-.14-.24 0-.43.05-.59.15-.16.1-.29.24-.38.42-.1.17-.17.38-.21.62-.05.24-.07.5-.08.77h-1.19c0-.43.05-.83.16-1.2s.27-.69.49-.96c.21-.25.48-.46.79-.62.31-.15.67-.23 1.07-.23.45 0 .82.08 1.11.23.3.16.55.36.73.6.19.24.32.5.39.79.08.28.12.54.12.79 0 .31-.04.6-.13.85s-.22.49-.37.7c-.15.21-.32.41-.52.59s-.4.35-.61.51l-.63.45c-.21.14-.39.28-.57.42zM0 1c0-.55.44-1 1-1 .55 0 1 .46 1 1v10c0 .55-.44 1-1 1-.55 0-1-.46-1-1V1zm6 0c0-.55.44-1 1-1 .55 0 1 .46 1 1v10c0 .55-.44 1-1 1-.55 0-1-.46-1-1V1zM2 5h4v2H2V5z"],headset:["M14.85 6.34C14.18 2.72 11.37 0 8 0S1.82 2.72 1.15 6.34C.47 6.9 0 8.1 0 9.5 0 11.43.9 13 2 13c0 1.1.9 2 2 2h2c0 .55.45 1 1 1h2c.55 0 1-.45 1-1v-1c0-.55-.45-1-1-1H7c-.55 0-1 .45-1 1H4c-.55 0-1-.45-1-1 .55 0 1-.45 1-1V7c0-.45-.3-.81-.71-.94C3.97 3.7 5.81 2 8 2s4.03 1.7 4.71 4.06c-.41.13-.71.49-.71.94v5c0 .55.45 1 1 1h1c1.1 0 2-1.57 2-3.5 0-1.4-.47-2.6-1.15-3.16z"],heart:["M16 5.095c0-2.255-1.88-4.083-4.2-4.083-1.682 0-3.13.964-3.8 2.352a4.206 4.206 0 00-3.8-2.352C1.88 1.012 0 2.84 0 5.095c0 .066.007.13.01.194H.004c.001.047.01.096.014.143l.013.142c.07.8.321 1.663.824 2.573C2.073 10.354 4.232 12.018 8 15c3.767-2.982 5.926-4.647 7.144-6.854.501-.905.752-1.766.823-2.562.007-.055.012-.11.016-.164.003-.043.012-.088.013-.13h-.006c.003-.066.01-.13.01-.195z"],"heart-broken":["M7.71 8.87L6.17 6.55l.02-.01A.906.906 0 016 6c0-.07.03-.13.04-.19h-.02l.78-3.92C6.09 1.34 5.19 1 4.2 1 1.88 1 0 2.83 0 5.09c0 .07.01.13.01.19H0c0 .05.01.1.01.14 0 .05.01.1.01.14.07.8.32 1.66.82 2.57 1.07 1.94 2.88 3.47 5.86 5.84l-.68-2.74h.02C6.03 11.16 6 11.08 6 11c0-.28.11-.53.29-.71l1.42-1.42zM16 5.09C16 2.83 14.12 1 11.8 1c-1.2 0-2.27.5-3.04 1.28l-.7 3.51 1.77 2.66-.01.01c.1.15.18.33.18.54 0 .28-.11.53-.29.71l-1.6 1.6.75 3.01c3.23-2.56 5.16-4.15 6.28-6.18.5-.91.75-1.77.82-2.56.01-.05.01-.11.02-.16 0-.04.01-.09.01-.13h-.01c.01-.07.02-.14.02-.2z"],"heat-grid":["M0 10h5V7H0v3zm1-2h3v1H1V8zm14-5h-4v3h5V4c0-.55-.45-1-1-1zm0 2h-3V4h3v1zM0 4v2h5V3H1c-.55 0-1 .45-1 1zm0 9c0 .55.45 1 1 1h4v-3H0v2zm6-7h4V3H6v3zm0 8h4v-3H6v3zm1-2h2v1H7v-1zm4 2h4c.55 0 1-.45 1-1v-2h-5v3zm0-4h5V7h-5v3zm-5 0h4V7H6v3z"],heatmap:["M2 11c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm11-7c1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2 .9 2 2 2zm3 4.5A2.5 2.5 0 0013.5 6c-.98 0-1.82.57-2.23 1.39-.6-.78-1.51-1.3-2.56-1.36.18-.49.29-.99.29-1.53C9 2.01 6.99 0 4.5 0S0 2.01 0 4.5 2.01 9 4.5 9c.19 0 .37-.03.56-.06-.03.19-.06.37-.06.56C5 11.43 6.57 13 8.5 13c1.63 0 2.98-1.11 3.37-2.62.44.38 1 .62 1.63.62A2.5 2.5 0 0016 8.5zM14.5 13c-.83 0-1.5.67-1.5 1.5s.67 1.5 1.5 1.5 1.5-.67 1.5-1.5-.67-1.5-1.5-1.5z"],help:["M8 0C3.58 0 0 3.58 0 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm1 13H7v-2h2v2zm1.93-6.52c-.14.32-.35.64-.62.97L9.25 8.83c-.12.15-.24.29-.28.42-.04.13-.09.3-.09.52V10H7.12V8.88s.05-.51.21-.71L8.4 6.73c.22-.26.35-.49.44-.68.09-.19.12-.38.12-.58 0-.3-.1-.55-.28-.75-.18-.19-.44-.28-.76-.28-.33 0-.59.1-.78.29-.19.19-.33.46-.4.81-.03.11-.1.15-.2.14l-1.7-.25c-.12-.01-.16-.08-.14-.19.12-.82.46-1.47 1.03-1.94.57-.48 1.32-.72 2.25-.72.47 0 .9.07 1.29.22s.72.34 1 .59c.28.25.49.55.65.89.15.35.22.72.22 1.12s-.07.75-.21 1.08z"],"helper-management":["M13 5h-2v2h2V5zm0 6h-2v2h2v-2zm0-3h-2v2h2V8zm2-8H1C.4 0 0 .4 0 1v14c0 .6.4 1 1 1h14c.6 0 1-.4 1-1V1c0-.6-.4-1-1-1zm-1 14H2V2h12v12zm-7-3H5v2h2v-2zm3 0H8v2h2v-2z"],"high-priority":["M9 14v2H7v-2h2zm1-14L9 12H7L6 0h4z"],highlight:["M9.12 11.07l2-2.02.71.71 4-4.04L10.17 0l-4 4.04.71.71-2 2.02 4.24 4.3zM2 12.97h4c.28 0 .53-.11.71-.3l1-1.01-3.42-3.45-3 3.03c-.18.18-.29.44-.29.72 0 .55.45 1.01 1 1.01zm13 1.01H1c-.55 0-1 .45-1 1.01S.45 16 1 16h14c.55 0 1-.45 1-1.01s-.45-1.01-1-1.01z"],history:["M8 3c-.55 0-1 .45-1 1v4c0 .28.11.53.29.71l2 2a1.003 1.003 0 001.42-1.42L9 7.59V4c0-.55-.45-1-1-1zm0-3a7.95 7.95 0 00-6 2.74V1c0-.55-.45-1-1-1S0 .45 0 1v4c0 .55.45 1 1 1h4c.55 0 1-.45 1-1s-.45-1-1-1H3.54C4.64 2.78 6.23 2 8 2c3.31 0 6 2.69 6 6 0 2.61-1.67 4.81-4 5.63v-.01c-.63.23-1.29.38-2 .38-3.31 0-6-2.69-6-6 0-.55-.45-1-1-1s-1 .45-1 1c0 4.42 3.58 8 8 8 .34 0 .67-.03 1-.07.02 0 .04-.01.06-.01C12.98 15.4 16 12.06 16 8c0-4.42-3.58-8-8-8z"],home:["M2 10v5c0 .55.45 1 1 1h3v-5h4v5h3c.55 0 1-.45 1-1v-5L8 4l-6 6zm13.71-2.71L14 5.59V2c0-.55-.45-1-1-1s-1 .45-1 1v1.59L8.71.29C8.53.11 8.28 0 8 0s-.53.11-.71.29l-7 7a1.003 1.003 0 001.42 1.42L8 2.41l6.29 6.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71z"],"horizontal-bar-chart":["M4 5h7c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1H4c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1zM1 1c-.55 0-1 .45-1 1v13c0 .55.45 1 1 1s1-.45 1-1V2c0-.55-.45-1-1-1zm14 6H4c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h11c.55 0 1-.45 1-1V8c0-.55-.45-1-1-1zm-6 5H4c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h5c.55 0 1-.45 1-1v-1c0-.55-.45-1-1-1z"],"horizontal-bar-chart-asc":["M1 3h5c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1zm0 4h7c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1zm14 6H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zM1 11h10c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1z"],"horizontal-bar-chart-desc":["M15 1H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zM8 9H1c-.55 0-1 .45-1 1s.45 1 1 1h7c.55 0 1-.45 1-1s-.45-1-1-1zm-2 4H1c-.55 0-1 .45-1 1s.45 1 1 1h5c.55 0 1-.45 1-1s-.45-1-1-1zm5-8H1c-.55 0-1 .45-1 1s.45 1 1 1h10c.55 0 1-.45 1-1s-.45-1-1-1z"],"horizontal-distribution":["M2 0c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1zm13 0c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1zm-5 2H7c-.55 0-1 .45-1 1v9c0 .55.45 1 1 1h3c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1z"],"id-number":["M2 5v7h12V5H2zm0-2h12c1.1 0 2 .9 2 2v7c0 1.1-.9 2-2 2H2c-1.1 0-2-.9-2-2V5c0-1.1.9-2 2-2z","M7.9 10.48c-.14-.33-.84-.55-1.3-.75-.46-.2-.4-.33-.42-.5v-.07c.16-.14.29-.33.37-.56 0 0 0-.01.01-.02.02-.05.03-.1.05-.15.1-.01.16-.13.19-.23.03-.04.07-.15.06-.27-.02-.16-.08-.24-.15-.26v-.03c0-.2-.02-.48-.05-.67-.01-.05-.02-.1-.03-.16-.07-.23-.21-.44-.4-.58-.2-.15-.48-.23-.73-.23s-.53.08-.72.23c-.19.14-.33.35-.4.58-.02.05-.03.1-.03.16-.05.18-.06.47-.06.67v.03c-.07.03-.14.1-.15.26-.02.12.03.22.06.27.02.1.09.22.2.24.01.05.03.1.05.15v.01c.08.23.22.42.38.56v.07c-.02.17.03.29-.43.5-.46.2-1.16.42-1.3.75s-.09.52-.09.52H8c-.01 0 .05-.19-.1-.52zM10 6h2c.55 0 1 .45 1 1s-.45 1-1 1h-2c-.55 0-1-.45-1-1s.45-1 1-1zM10 9h2c.55 0 1 .45 1 1s-.45 1-1 1h-2c-.55 0-1-.45-1-1s.45-1 1-1z"],"image-rotate-left":["M13 2h-1.59l.29-.29c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71l-2 2C8.11 2.47 8 2.72 8 3c0 .28.11.53.29.71l2 2a1.003 1.003 0 001.42-1.42l-.3-.29H13c.55 0 1 .45 1 1v3c0 .55.45 1 1 1s1-.45 1-1V5c0-1.66-1.34-3-3-3zm-5.5 9c.28 0 .5-.22.5-.5s-.22-.5-.5-.5-.5.22-.5.5.22.5.5.5zM10 7H1c-.55 0-1 .45-1 1v7c0 .55.45 1 1 1h9c.55 0 1-.45 1-1V8c0-.55-.45-1-1-1zm-1 6.33L7 12l-1 1-2-3-2 2.67V9h7v4.33z"],"image-rotate-right":["M5.71 5.71l2-2C7.89 3.53 8 3.28 8 3c0-.28-.11-.53-.29-.71l-2-2a1.003 1.003 0 00-1.42 1.42l.3.29H3C1.34 2 0 3.34 0 5v3c0 .55.45 1 1 1s1-.45 1-1V5c0-.55.45-1 1-1h1.59l-.3.29a1.003 1.003 0 001.42 1.42zM12.5 11c.28 0 .5-.22.5-.5s-.22-.5-.5-.5-.5.22-.5.5.22.5.5.5zM15 7H6c-.55 0-1 .45-1 1v7c0 .55.45 1 1 1h9c.55 0 1-.45 1-1V8c0-.55-.45-1-1-1zm-1 6.33L12 12l-1 1-2-3-2 2.67V9h7v4.33z"],import:["M7.29 11.71c.18.18.43.29.71.29s.53-.11.71-.29l4-4a1.003 1.003 0 00-1.42-1.42L9 8.59V1c0-.55-.45-1-1-1S7 .45 7 1v7.59l-2.29-2.3a1.003 1.003 0 00-1.42 1.42l4 4zM15 11c-.55 0-1 .45-1 1v2H2v-2c0-.55-.45-1-1-1s-1 .45-1 1v3c0 .55.45 1 1 1h14c.55 0 1-.45 1-1v-3c0-.55-.45-1-1-1z"],inbox:["M13.91 2.6c-.16-.36-.51-.61-.92-.61h-10c-.41 0-.77.25-.92.61L-.01 7.45v5.54c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V7.45L13.91 2.6zm-1.92 5.39c-.55 0-1 .45-1 1v1h-6v-1c0-.55-.45-1-1-1H1.94l1.71-4h8.68l1.71 4h-2.05z"],"inbox-filtered":["M6.432 2c.094.14.202.273.324.394L8.42 4H3.66L1.95 8H4c.55 0 1 .45 1 1v1h6.557c.693 0 1.363-.262 1.837-.736l.103-.102.85-1.14a2.564 2.564 0 00.623-1.682V5.058L16 7.46V13c0 .55-.45 1-1 1H1c-.55 0-1-.45-1-1V7.46l2.08-4.85C2.23 2.25 2.59 2 3 2h3.432zm9.048-2c.31 0 .52.26.52.57 0 .16-.06.3-.17.41l-2.86 2.73v2.63c0 .16-.06.3-.17.41l-.82 1.1c-.1.1-.25.17-.41.17-.31 0-.57-.26-.57-.57V3.71L8.17.98A.566.566 0 018 .57c0-.31.26-.57.57-.57h6.91z"],"inbox-geo":["M6.341 2A5.99 5.99 0 006 4H3.66L1.95 8H4c.55 0 1 .45 1 1v1h7a5.978 5.978 0 004-1.528V13c0 .55-.45 1-1 1H1c-.55 0-1-.45-1-1V7.46l2.08-4.85C2.23 2.25 2.59 2 3 2h3.341zm3.679 2.145c0-.125.075-.23.205-.225h.345l.79.8c.005 0 0 .005 0 .005v.295c0 .13-.085.23-.215.23h-.07v.15c0 .13-.09.2-.215.2v.535c0 .125-.12.225-.245.225s-.245-.1-.245-.225V5.25h-.145c-.125 0-.205-.1-.205-.23v-.875zm2.235-2.195c-.03 0-.055-.005-.06-.035 0-.03.03-.035.06-.035h.11c.035 0 .06.005.06.035 0 .03-.03.035-.06.035h-.11zm-1.165-.025a.094.094 0 01-.13 0l-.25-.25a.094.094 0 010-.13.094.094 0 01.13 0l.25.25a.094.094 0 010 .13zm1.53.445c-.035 0-.07-.025-.07-.06v-.155c0-.03.035-.06.07-.06s.07.025.07.06v.155c0 .03-.035.06-.07.06zM12 0c2.21 0 4 1.79 4 4s-1.79 4-4 4-4-1.79-4-4 1.79-4 4-4zm0 7c1.655 0 3-1.345 3-3 0-.195-.02-.39-.06-.575h-.21c-.125 0-.225-.07-.23-.21h-.215c.075.07.155.14.155.23V3.9c0 .06-.04.115-.075.155h-.015l-.01.005-.015.01-.445.43v.815c0 .13-.07.22-.2.22h-.36c-.125 0-.21-.09-.21-.22v-.84a.627.627 0 00-.36-.345h-.45c-.13 0-.245-.095-.245-.225v-.46c0-.125.115-.23.245-.23l.13-.005.21-.215c.09-.09.24-.09.33 0l.22.225h.47c.105 0 .155-.105.225-.21v-.075h-.205a.106.106 0 01-.105-.105.11.11 0 01.105-.105h.18l.025.005v-.12s-.03-.025-.04-.025h-.2l-.045.05a.235.235 0 01-.33.005l-.045-.06h-.115l.16.175c.015.015.015.06 0 .075-.02.015-.045.02-.06.005l-.195-.185h-.085l-.245.23-.02-.005c-.025.07-.06.055-.095.055-.085 0-.15-.045-.15-.13s.065-.14.15-.14h.115v-.125c0-.06.04-.09.1-.09h.05V2.36c0-.095.095-.2.19-.2h.19c.105 0 .18-.075.18-.185V1.94c0-.015.035.01.035-.06h-.125l-.005.01-.21.22a.085.085 0 01-.115 0 .085.085 0 010-.115l.255-.255c.02-.015.045-.015.065-.015.005 0 .005.015.005.015h.64a2.327 2.327 0 00-.355-.275 2.452 2.452 0 00-.355-.195c-.03-.015-.055-.03-.08-.04a3.025 3.025 0 00-1.735-.175c-.09.02-.175.055-.265.08-.09.03-.18.05-.265.085-.075.03-.145.07-.22.105-.115.06-.235.115-.34.185l-.005.005c-.1.065-.18.145-.27.22h.455c.06 0 .11.045.11.105s-.05.105-.11.105h-.32c0 .07-.025.04-.025.045v.24h.285l.15-.17c.09-.09.235-.105.325-.015.09.09.09.23 0 .32l-.795.79-.01.01c-.005 0-.005 0-.01.005l-.025.015h-.01a.235.235 0 01-.12.025h-.23c-.08.07-.125.1-.125.18v.06c0 .01-.02.02-.02.03l.375.39c.04.04.04.1 0 .14-.04.04-.1.04-.14 0l-.39-.385a.213.213 0 01-.055.01c-.105 0-.195-.085-.195-.185v-.235h-.055A3.1 3.1 0 009 4c0 1.655 1.345 3 3 3zm2.27-2.135c.05 0 .105.04.105.09v.285c0 .05-.055.09-.105.09-.05 0-.105-.04-.105-.09v-.285c0-.05.055-.09.105-.09zm-2.085-3.27c0 .13-.105.21-.225.21h-.25v.07c0 .075-.03.135-.105.135s-.105-.06-.105-.135V1.64c-.075-.02-.025-.025-.025-.04 0-.125.085-.215.21-.215h.27c.13 0 .23.085.23.21z"],"inbox-search":["M5.639 2a5.391 5.391 0 00-.144 2H3.66L1.95 8H4c.55 0 1 .45 1 1v1h6V9c0-.088.012-.174.033-.255.12-.007.238-.019.39-.038.154-.008.252-.03.442-.077a5.34 5.34 0 00.24-.05h.05l.122-.04 1.266 1.271c.425.47 1.116.769 1.847.769.21 0 .414-.025.61-.071V13c0 .55-.45 1-1 1H1c-.55 0-1-.45-1-1V7.46l2.08-4.85C2.23 2.25 2.59 2 3 2h2.639zM15.82 7.53c.1.12.17.27.18.44 0 .34-.27.61-.61.61a.57.57 0 01-.43-.18l-2.24-2.25c-.13.08-.26.16-.4.23-.02.01-.05.02-.07.03-.14.06-.27.12-.42.17h-.01c-.14.05-.29.08-.44.11-.04.01-.08.02-.11.02-.15.02-.3.04-.46.04-1.85 0-3.35-1.51-3.35-3.37S8.96.01 10.81 0c1.85 0 3.35 1.51 3.35 3.37 0 .16-.02.31-.04.47-.01.04-.01.07-.02.11-.02.15-.05.29-.1.44v.01c-.05.15-.11.28-.17.42-.01.02-.02.05-.03.07-.07.14-.14.27-.23.4l2.25 2.24zm-5.01-1.94c1.22 0 2.21-.99 2.21-2.22 0-1.23-.99-2.22-2.21-2.22S8.6 2.14 8.6 3.37c0 1.22.99 2.22 2.21 2.22z"],"inbox-update":["M8.1 2a5.023 5.023 0 000 2H3.66L1.95 8H4c.55 0 1 .45 1 1v1h6V9c0-.55.45-1 1-1h2.05c.708 0 1.352-.241 1.905-.645L16 7.46V13c0 .55-.45 1-1 1H1c-.55 0-1-.45-1-1V7.46l2.08-4.85C2.23 2.25 2.59 2 3 2h5.1zM13 6a3 3 0 110-6 3 3 0 010 6z"],"info-sign":["M8 0C3.58 0 0 3.58 0 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zM7 3h2v2H7V3zm3 10H6v-1h1V7H6V6h3v6h1v1z"],inheritance:["M5 8c0 1.66 1.34 3 3 3h4.59L11.3 9.71A.965.965 0 0111 9a1.003 1.003 0 011.71-.71l3 3c.18.18.29.43.29.71 0 .28-.11.53-.29.71l-3 3a1.003 1.003 0 01-1.42-1.42l1.3-1.29H8c-2.76 0-5-2.24-5-5H1a1 1 0 01-1-1V1a1 1 0 011-1h6a1 1 0 011 1v6a1 1 0 01-1 1H5zM2 2v4h4V2H2z"],"inherited-group":["M1 7c.51 0 .935.388.993.884L2 8v3c0 .51.388.935.884.993L3 12h1.59l-.3-.29a1.003 1.003 0 011.324-1.504l.096.084 2 2c.18.18.29.43.29.71 0 .233-.076.446-.206.614l-.084.096-2 2A1.003 1.003 0 014 15c0-.24.08-.458.224-.629l.076-.081.29-.29H3a2.996 2.996 0 01-2.995-2.823L0 11V8c0-.55.45-1 1-1zm5.388-7c.629 0 1.338.21 1.838.6.48.38.85.91 1.019 1.52.04.13.07.27.09.4.09.48.14 1.22.14 1.73v.07c.18.08.34.27.37.67.03.32-.09.59-.16.71-.06.28-.21.58-.48.63-.03.13-.07.26-.12.39 0 .01-.01.04-.01.04-.22.58-.55 1.08-.949 1.45v.18c.04.45-.12.77 1.059 1.3 1.179.53 2.947 1.09 3.307 1.95.37.86.22 1.36.22 1.36H9c0-.539-.21-1.045-.583-1.417l-2-2A1.997 1.997 0 005 9c-.149 0-.296-.015-.442-.045.099-.19.082-.37.101-.575 0-.05.01-.11.01-.17-.41-.35-.75-.86-.969-1.45v-.01s-.01-.01-.01-.02c-.04-.12-.09-.26-.12-.39-.28-.05-.44-.36-.5-.64-.06-.12-.19-.39-.16-.71.04-.41.21-.6.39-.68v-.06c0-.51.05-1.26.14-1.74.02-.13.05-.27.09-.4.17-.6.54-1.13 1.02-1.51C5.048.21 5.757 0 6.387 0zm4.625 2.04c.49 0 1.05.16 1.439.46.38.29.67.7.8 1.17.03.1.05.21.07.31.07.37.11.94.11 1.33v.05c.139.06.269.21.289.51.02.25-.07.45-.13.54-.05.21-.16.44-.38.48a1.711 1.711 0 01-.1.33c-.17.44-.43.83-.749 1.11v.14c.03.35-.09.59.83 1 .929.41 2.317.84 2.597 1.5.29.66.17 1.04.17 1.04H13.66v.01c-.05-.24-.14-.5-.25-.76-.36-.86-1.119-1.33-2.687-2-.14-.06-.59-.25-.6-.25-.21-.09-.36-.15-.5-.22.02-.1.02-.2.03-.31 0-.04.01-.08.01-.13-.07-.06-.13-.12-.19-.19.22-.32.4-.67.54-1.05.02-.06.02-.06.03-.1.29-.23.48-.57.59-.96.16-.33.25-.73.21-1.16-.03-.4-.16-.76-.37-1.03-.02-.53-.07-1.13-.15-1.54-.01-.06-.02-.12-.03-.19.23-.06.48-.09.72-.09z"],"inner-join":["M6.6 3.3C5.3 4.4 4.5 6.1 4.5 8s.8 3.6 2.1 4.7c-.5.2-1 .3-1.6.3-2.8 0-5-2.2-5-5s2.2-5 5-5c.6 0 1.1.1 1.6.3zm-1.96 8.68C3.92 10.83 3.5 9.46 3.5 8s.42-2.83 1.14-3.98C2.6 4.2 1 5.91 1 8s1.6 3.8 3.64 3.98zM8 4c-1.2.9-2 2.4-2 4s.8 3.1 2 4c1.2-.9 2-2.3 2-4s-.8-3.1-2-4zm3-1c2.8 0 5 2.2 5 5s-2.2 5-5 5c-.6 0-1.1-.1-1.6-.3 1.3-1.1 2.1-2.9 2.1-4.7s-.8-3.5-2.1-4.7c.5-.2 1-.3 1.6-.3zm.35 1.02c.73 1.15 1.14 2.52 1.14 3.98s-.42 2.83-1.14 3.98c2.04-.18 3.64-1.9 3.64-3.98s-1.6-3.8-3.64-3.98z"],insert:["M5 9h2v2c0 .6.4 1 1 1s1-.4 1-1V9h2c.6 0 1-.4 1-1s-.4-1-1-1H9V5c0-.6-.4-1-1-1s-1 .4-1 1v2H5c-.6 0-1 .4-1 1s.4 1 1 1zm10-9H1C.4 0 0 .4 0 1v14c0 .6.4 1 1 1h14c.6 0 1-.4 1-1V1c0-.6-.4-1-1-1zm-1 14H2V2h12v12z"],intersection:["M10 3c-.92 0-1.76.26-2.5.69C6.76 3.26 5.92 3 5 3 2.24 3 0 5.24 0 8s2.24 5 5 5c.92 0 1.76-.26 2.5-.69.74.43 1.58.69 2.5.69 2.76 0 5-2.24 5-5s-2.24-5-5-5zm-4.1 7.85c-.29.09-.59.15-.9.15-1.66 0-3-1.34-3-3s1.34-3 3-3c.31 0 .61.06.9.15C5.33 5.96 5 6.94 5 8s.33 2.04.9 2.85zM10 11c-.31 0-.61-.06-.9-.15.57-.81.9-1.79.9-2.85s-.33-2.04-.9-2.85c.29-.09.59-.15.9-.15 1.66 0 3 1.34 3 3s-1.34 3-3 3z"],"ip-address":["M5 2.66C5 4.14 8 8 8 8s3-3.86 3-5.34C10.99 1.2 9.66 0 8 0S5 1.2 5 2.66zM7 3c0-.55.45-1 1-1s1 .45 1 1-.45 1-1 1-1-.45-1-1zM10.5 10H8v5h1v-4h1v1H9v1h2v-3h-.5zM2 9h12c.55 0 1 .45 1 1v5c0 .55-.45 1-1 1H2c-.55 0-1-.45-1-1v-5c0-.55.45-1 1-1zm4 1v5h1v-5H6z"],issue:["M8 16A8 8 0 118 0a8 8 0 010 16zm0-2A6 6 0 108 2a6 6 0 000 12zm1-2H7v-2h2v2zm0-3H7V4h2v5z"],"issue-closed":["M9.296.104a2.99 2.99 0 00-1.003.664 2.987 2.987 0 00-.75 1.25 6 6 0 106.28 4.527c.043-.039.085-.079.127-.12l1.456-1.456A8 8 0 119.296.105zm2.532 5.2a.997.997 0 01-.707-.294L9.707 3.596a1 1 0 011.414-1.414l.707.707 1.768-1.768a1 1 0 111.414 1.415L12.536 5.01a.997.997 0 01-.708.293zM9 12H7v-2h2v2zm0-3H7V4h2v5z"],"issue-new":["M10.568.421c-.01.04-.018.08-.026.121-.837.156-1.53.73-1.85 1.497a6 6 0 105.27 5.273 2.51 2.51 0 001.496-1.854c.04-.008.081-.016.121-.026A8 8 0 1110.568.421zM9 12H7v-2h2v2zm0-3H7V4h2v5zm1-6c0-.55.45-1 1-1h1V1c0-.55.45-1 1-1s1 .45 1 1v1h1c.55 0 1 .45 1 1s-.45 1-1 1h-1v1.005c0 .55-.45 1-1 1s-1-.45-1-1V4h-1c-.55 0-1-.45-1-1z"],italic:["M9.8 4H11c.5 0 1-.4 1-1s-.4-1-1-1H7c-.5 0-1 .4-1 1s.4 1 1 1h.8l-1.6 8H5c-.5 0-1 .4-1 1s.4 1 1 1h4c.5 0 1-.4 1-1s-.4-1-1-1h-.8l1.6-8z"],"join-table":["M15 5h-3V2c0-.55-.45-1-1-1H1c-.55 0-1 .45-1 1v8c0 .55.45 1 1 1h3v3c0 .55.45 1 1 1h10c.55 0 1-.45 1-1V6c0-.55-.45-1-1-1zm-5-1v2H6V4h4zm0 6H6V7h4v3zM2 4h3v2H2V4zm0 5V7h3v2H2zm4 4v-2h4v2H6zm8 0h-3v-2h3v2zm0-3h-3V8h3v2z"],key:["M11 0C8.24 0 6 2.24 6 5c0 1.02.31 1.96.83 2.75L.29 14.29a1.003 1.003 0 001.42 1.42L3 14.41l1.29 1.29c.18.19.43.3.71.3s.53-.11.71-.29l2-2c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71L6.41 11l1.83-1.83c.8.52 1.74.83 2.76.83 2.76 0 5-2.24 5-5s-2.24-5-5-5zm0 8c-.23 0-.45-.03-.66-.08-.01 0-.02-.01-.03-.01-.21-.05-.41-.12-.6-.21a3.014 3.014 0 01-1.62-2c0-.01-.01-.02-.01-.03C8.03 5.45 8 5.23 8 5c0-1.66 1.34-3 3-3s3 1.34 3 3-1.34 3-3 3z"],"key-backspace":["M15 2H6c-.28 0-.53.11-.71.29l-5 5C.11 7.47 0 7.72 0 8c0 .28.11.53.29.71l5 5c.18.18.43.29.71.29h9c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1zm-2.29 7.29a1.003 1.003 0 01-1.42 1.42L10 9.41 8.71 10.7c-.18.19-.43.3-.71.3a1.003 1.003 0 01-.71-1.71L8.59 8l-1.3-1.29a1.003 1.003 0 011.42-1.42L10 6.59l1.29-1.29c.18-.19.43-.3.71-.3a1.003 1.003 0 01.71 1.71L11.41 8l1.3 1.29z"],"key-command":["M12 9h-1V7h1c1.66 0 3-1.34 3-3s-1.34-3-3-3-3 1.34-3 3v1H7V4c0-1.66-1.34-3-3-3S1 2.34 1 4s1.34 3 3 3h1v2H4c-1.66 0-3 1.34-3 3s1.34 3 3 3 3-1.34 3-3v-1h2v1c0 1.66 1.34 3 3 3s3-1.34 3-3-1.34-3-3-3zm0-6c.55 0 1 .45 1 1s-.45 1-1 1-1-.45-1-1 .45-1 1-1zM4 13c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zm0-8c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zm5 4H7V7h2v2zm3 4c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1z"],"key-control":["M12.71 5.29l-4-4C8.53 1.11 8.28 1 8 1s-.53.11-.71.29l-4 4a1.003 1.003 0 001.42 1.42L8 3.41l3.29 3.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71z"],"key-delete":["M15.71 7.29l-5-5A.997.997 0 0010 2H1c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1h9c.28 0 .53-.11.71-.29l5-5c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71zm-7 2a1.003 1.003 0 01-1.42 1.42L6 9.41 4.71 10.7c-.18.19-.43.3-.71.3a1.003 1.003 0 01-.71-1.71L4.59 8l-1.3-1.29a1.003 1.003 0 011.42-1.42L6 6.59 7.29 5.3c.18-.19.43-.3.71-.3a1.003 1.003 0 01.71 1.71L7.41 8l1.3 1.29z"],"key-enter":["M14 2c-.55 0-1 .45-1 1v3c0 1.66-1.34 3-3 3H4.41L5.7 7.71c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71l-3 3c-.18.18-.29.43-.29.71 0 .28.11.53.29.71l3 3a1.003 1.003 0 001.42-1.42L4.41 11H10c2.76 0 5-2.24 5-5V3c0-.55-.45-1-1-1z"],"key-escape":["M2 7c.55 0 1-.45 1-1V4.41L7.29 8.7c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71L4.41 3H6c.55 0 1-.45 1-1s-.45-1-1-1H2c-.55 0-1 .45-1 1v4c0 .55.45 1 1 1zm7-5.9v2A5 5 0 113.1 9h-2c.49 3.39 3.38 6 6.9 6 3.87 0 7-3.13 7-7 0-3.52-2.61-6.41-6-6.9z"],"key-option":["M11 4h4c.55 0 1-.45 1-1s-.45-1-1-1h-4c-.55 0-1 .45-1 1s.45 1 1 1zm4 8h-3.43L5.86 2.49h-.02A.975.975 0 005 2H1c-.55 0-1 .45-1 1s.45 1 1 1h3.43l5.71 9.51.01-.01c.18.3.49.5.85.5h4c.55 0 1-.45 1-1s-.45-1-1-1z"],"key-shift":["M13.71 7.29l-5-5C8.53 2.11 8.28 2 8 2s-.53.11-.71.29l-5 5A1.003 1.003 0 003 9h2v4c0 .55.45 1 1 1h4c.55 0 1-.45 1-1V9h2a1.003 1.003 0 00.71-1.71z"],"key-tab":["M15 10H4.41L5.7 8.71c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71L2 9.59V8c0-.55-.45-1-1-1s-1 .45-1 1v6c0 .55.45 1 1 1s1-.45 1-1v-1.59l2.29 2.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71L4.41 12H15c.55 0 1-.45 1-1s-.45-1-1-1zm0-9c-.55 0-1 .45-1 1v1.59L11.71 1.3A.965.965 0 0011 1a1.003 1.003 0 00-.71 1.71L11.59 4H1c-.55 0-1 .45-1 1s.45 1 1 1h10.59L10.3 7.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71L14 6.41V8c0 .55.45 1 1 1s1-.45 1-1V2c0-.55-.45-1-1-1z"],"known-vehicle":["M15 3a.997.997 0 00-.707.293L12 5.586l-1.293-1.293a1 1 0 10-1.414 1.414l2 2a.997.997 0 001.414 0l3-3A1 1 0 0015 3zm-.879 6.121l-.007-.007c-.313.309-.69.552-1.114.702V10h-.998H12h-1v-.184c-.424-.15-.8-.395-1.112-.704l-.01.01-2-2 .012-.012A2.978 2.978 0 017.184 6H3c-.176 0-.06-.824 0-1l.73-1.63C3.79 3.192 3.823 3 4 3H7.78C8.328 2.39 9.115 2 10 2c.768 0 1.461.293 1.987.77l.844-.844c-.238-.244-.524-.442-.794-.524C12.037 1.402 10.72 1 8 1c-2.72 0-4.037.402-4.037.402-.508.155-1.078.711-1.268 1.237l-.763 2.117H.88c-.484 0-.88.423-.88.939s.396.939.88.939h.375L1 7c-.034.685 0 1.436 0 2v5c0 .657.384 1 1 1s1-.343 1-1v-1h10v1c0 .657.384 1 1 1s1-.343 1-1V9l-.003-.754-.876.875zM5.001 10H3V8h2v2z"],"lab-test":["M11 1a1 1 0 010 2v3l3 7v1.25a.75.75 0 01-.75.75H2.75a.75.75 0 01-.75-.75V13l3-7V3a1 1 0 110-2h6zM9 3H7v3l-1.714 4h5.428L9 6V3z"],label:["M11 2H1c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V7l-5-5zm3 10H2V4h8v2H3v1h7v1h4v4zm-3-5V4l3 3h-3zm-8 3h10V9H3v1z"],layer:["M16 8c0-.37-.21-.68-.51-.85l.01-.01-7-4-.01.01C8.34 3.06 8.18 3 8 3s-.34.06-.49.15l-.01-.02-7 4 .01.01C.21 7.32 0 7.63 0 8s.21.68.51.85l-.01.01 7 4 .01-.01c.15.09.31.15.49.15s.34-.06.49-.15l.01.01 7-4-.01-.01c.3-.17.51-.48.51-.85z"],"layer-outline":["M7.504 3.132l-7 4a1 1 0 000 1.736l7 4a1 1 0 00.992 0l7-4a1 1 0 000-1.736l-7-4a1 1 0 00-.992 0zM8 5.152L12.983 8 8 10.847 3.016 8 8 5.152z"],layers:["M.55 4.89l7 3.5c.14.07.29.11.45.11s.31-.04.45-.11l7-3.5a.998.998 0 00-.06-1.81L8.4.08a1.006 1.006 0 00-.79 0l-6.99 3a.992.992 0 00-.07 1.81zM15 10c-.16 0-.31.04-.45.11L8 13.38 1.45 10.1c-.14-.06-.29-.1-.45-.1-.55 0-1 .45-1 1 0 .39.23.73.55.89l7 3.5c.14.07.29.11.45.11s.31-.04.45-.11l7-3.5c.32-.16.55-.5.55-.89 0-.55-.45-1-1-1zm0-3.5c-.16 0-.31.04-.45.11L8 9.88 1.45 6.61A.997.997 0 001 6.5c-.55 0-1 .45-1 1 0 .39.23.73.55.89l7 3.5c.14.07.29.11.45.11s.31-.04.45-.11l7-3.5c.32-.16.55-.5.55-.89 0-.55-.45-1-1-1z"],layout:["M14 4c-1.1 0-2 .9-2 2 0 .47.17.9.44 1.24l-.68.91A1.996 1.996 0 009.07 9.5H7.93C7.71 8.64 6.93 8 6 8c-.47 0-.9.17-1.24.44l-.91-.68c.1-.23.15-.49.15-.76 0-.37-.11-.71-.28-1.01l2.27-2.27c.3.17.64.28 1.01.28 1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2c0 .37.11.71.28 1.01L3.01 5.28C2.71 5.11 2.37 5 2 5 .9 5 0 5.9 0 7s.9 2 2 2c.47 0 .9-.17 1.24-.44l.91.68c-.1.23-.15.49-.15.76 0 .37.11.71.28 1.01l-1.27 1.27C2.71 12.11 2.37 12 2 12c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2c0-.37-.11-.71-.28-1.01l1.27-1.27c.3.17.64.28 1.01.28.93 0 1.71-.64 1.93-1.5h1.14c.22.86 1 1.5 1.93 1.5 1.1 0 2-.9 2-2 0-.47-.17-.9-.44-1.24l.68-.91c.23.1.49.15.76.15 1.1 0 2-.9 2-2s-.9-2-2-2z"],"layout-auto":["M14 9.5c-.56 0-1.06.23-1.42.59L8.99 8l3.59-2.09A2.002 2.002 0 0016 4.5c0-1.1-.9-2-2-2s-2 .9-2 2c0 .19.03.37.08.54L8.5 7.13v-3.2c.86-.22 1.5-1 1.5-1.93 0-1.1-.9-2-2-2S6 .9 6 2c0 .93.64 1.71 1.5 1.93v3.2L3.92 5.04c.05-.17.08-.35.08-.54 0-1.1-.9-2-2-2s-2 .9-2 2 .9 2 2 2c.56 0 1.06-.23 1.42-.59L7.01 8l-3.59 2.09A2.002 2.002 0 000 11.5c0 1.1.9 2 2 2s2-.9 2-2c0-.19-.03-.37-.08-.54L7.5 8.87v3.2c-.86.22-1.5 1-1.5 1.93 0 1.1.9 2 2 2s2-.9 2-2c0-.93-.64-1.71-1.5-1.93v-3.2l3.58 2.09c-.05.17-.08.35-.08.54 0 1.1.9 2 2 2s2-.9 2-2-.9-2-2-2z"],"layout-balloon":["M14 11c-.2 0-.38.04-.56.09L12.42 9.4c.36-.36.58-.85.58-1.4 0-.55-.22-1.04-.58-1.4l1.01-1.69c.19.05.37.09.57.09 1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2c0 .55.22 1.04.58 1.4l-1.01 1.69C11.38 6.04 11.2 6 11 6c-.93 0-1.71.64-1.93 1.5H6.93C6.71 6.64 5.93 6 5 6c-.2 0-.38.04-.56.09L3.42 4.4C3.78 4.04 4 3.55 4 3c0-1.1-.9-2-2-2s-2 .9-2 2 .9 2 2 2c.2 0 .38-.04.56-.09L3.58 6.6C3.22 6.96 3 7.45 3 8c0 .55.22 1.04.58 1.4l-1.01 1.69C2.38 11.04 2.2 11 2 11c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2c0-.55-.22-1.04-.58-1.4l1.01-1.69c.19.05.37.09.57.09.93 0 1.71-.64 1.93-1.5h2.14c.22.86 1 1.5 1.93 1.5.2 0 .38-.04.56-.09l1.01 1.69c-.35.36-.57.85-.57 1.4 0 1.1.9 2 2 2s2-.9 2-2-.9-2-2-2z"],"layout-circle":["M14.16 6.02c-.12-.36-.26-.7-.43-1.03.17-.29.27-.63.27-.99 0-1.1-.9-2-2-2-.36 0-.7.1-.99.27-.33-.17-.67-.31-1.03-.43A1.987 1.987 0 008 0C6.95 0 6.1.81 6.02 1.84c-.36.12-.7.26-1.03.43C4.7 2.1 4.36 2 4 2c-1.1 0-2 .9-2 2 0 .36.1.7.27.99-.17.33-.31.67-.43 1.03C.81 6.1 0 6.95 0 8c0 1.05.81 1.9 1.84 1.98.12.36.26.7.43 1.03-.17.29-.27.63-.27.99 0 1.1.9 2 2 2 .36 0 .7-.1.99-.27.33.17.67.32 1.03.43C6.1 15.19 6.95 16 8 16c1.05 0 1.9-.81 1.98-1.84.36-.12.7-.26 1.03-.43.29.17.63.27.99.27 1.1 0 2-.9 2-2 0-.36-.1-.7-.27-.99.17-.33.31-.67.43-1.03C15.19 9.9 16 9.05 16 8c0-1.05-.81-1.9-1.84-1.98zm-.99 3.79c-.05.16-.11.31-.17.46-.3-.17-.64-.27-1-.27-1.1 0-2 .9-2 2 0 .36.1.7.27 1-.15.07-.3.12-.46.17C9.5 12.48 8.81 12 8 12s-1.5.48-1.81 1.17c-.16-.06-.32-.11-.46-.17.17-.3.27-.64.27-1 0-1.1-.9-2-2-2-.36 0-.7.1-1 .27-.07-.15-.12-.3-.17-.46C3.52 9.5 4 8.81 4 8s-.48-1.5-1.17-1.81c.06-.16.11-.32.17-.46.3.17.64.27 1 .27 1.1 0 2-.9 2-2 0-.36-.1-.7-.27-1 .15-.07.3-.12.46-.17C6.5 3.52 7.19 4 8 4s1.5-.48 1.81-1.17c.16.06.32.11.46.17-.17.3-.27.64-.27 1 0 1.1.9 2 2 2 .36 0 .7-.1 1-.27.07.15.12.3.17.46C12.48 6.5 12 7.19 12 8s.48 1.5 1.17 1.81z"],"layout-grid":["M2 12c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm0-6C.9 6 0 6.9 0 8s.9 2 2 2 2-.9 2-2-.9-2-2-2zm0-6C.9 0 0 .9 0 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm12 4c1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2 .9 2 2 2zm0 2c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zM8 0C6.9 0 6 .9 6 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm6 12c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zM8 6c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm0 6c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2z"],"layout-group-by":["M2 6C.9 6 0 6.9 0 8s.9 2 2 2 2-.9 2-2-.9-2-2-2zm0 5c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm12-7c1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2 .9 2 2 2zM2 1C.9 1 0 1.9 0 3s.9 2 2 2 2-.9 2-2-.9-2-2-2zm7 8c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm0-6c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm5 3c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm0 6c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2z"],"layout-hierarchy":["M14.5 12.07V9.93c.86-.22 1.5-1 1.5-1.93 0-1.1-.9-2-2-2-.93 0-1.71.64-1.93 1.5H9.93c-.18-.7-.73-1.25-1.43-1.43V3.93c.86-.22 1.5-1 1.5-1.93 0-1.1-.9-2-2-2S6 .9 6 2c0 .93.64 1.71 1.5 1.93v2.14c-.7.18-1.25.73-1.43 1.43H3.93C3.71 6.64 2.93 6 2 6 .9 6 0 6.9 0 8c0 .93.64 1.71 1.5 1.93v2.14c-.86.22-1.5 1-1.5 1.93 0 1.1.9 2 2 2s2-.9 2-2c0-.93-.64-1.71-1.5-1.93V9.93c.7-.18 1.25-.73 1.43-1.43h2.14c.18.7.73 1.25 1.43 1.43v2.14c-.86.22-1.5 1-1.5 1.93 0 1.1.9 2 2 2s2-.9 2-2c0-.93-.64-1.71-1.5-1.93V9.93c.7-.18 1.25-.73 1.43-1.43h2.14c.18.7.73 1.25 1.43 1.43v2.14c-.86.22-1.5 1-1.5 1.93 0 1.1.9 2 2 2s2-.9 2-2c0-.93-.64-1.71-1.5-1.93z"],"layout-linear":["M14 6c-.93 0-1.71.64-1.93 1.5H9.93C9.71 6.64 8.93 6 8 6s-1.71.64-1.93 1.5H3.93C3.71 6.64 2.93 6 2 6 .9 6 0 6.9 0 8s.9 2 2 2c.93 0 1.71-.64 1.93-1.5h2.13C6.29 9.36 7.07 10 8 10s1.71-.64 1.93-1.5h2.13c.22.86 1 1.5 1.93 1.5 1.1 0 2-.9 2-2C16 6.9 15.1 6 14 6z"],"layout-skew-grid":["M2 6C.9 6 0 6.9 0 8s.9 2 2 2 2-.9 2-2-.9-2-2-2zm12-2c1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2 .9 2 2 2zM2 12c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zM2 0C.9 0 0 .9 0 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm6 9c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm6-3c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zM8 3c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm6 9c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2z"],"layout-sorted-clusters":["M2 6C.9 6 0 6.9 0 8s.9 2 2 2 2-.9 2-2-.9-2-2-2zm0 6c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zM2 0C.9 0 0 .9 0 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm12 6c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zM8 9c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm0-6c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2z"],learning:["M8.441 1.104a.985.985 0 00-.882 0L.365 5c-.487.253-.487.747 0 1L7.56 9.896a.985.985 0 00.882 0L15.635 6c.487-.253.487-.747 0-1L8.44 1.104z","M14 5.5l.016 4.514c.002.548.447.99.994.99a.99.99 0 00.99-.99V5.5h-2zM3.371 9.047l4.387 2.432a.5.5 0 00.485 0l4.39-2.432a.25.25 0 01.371.218v2.955a.25.25 0 01-.134.222l-4.635 2.436a.5.5 0 01-.466 0l-4.635-2.436A.25.25 0 013 12.22V9.265a.25.25 0 01.371-.218z"],"left-join":["M6.6 3.3C6.1 3.1 5.6 3 5 3 2.2 3 0 5.2 0 8s2.2 5 5 5c.6 0 1.1-.1 1.6-.3C5.3 11.6 4.5 9.9 4.5 8s.8-3.6 2.1-4.7zM8 4c-1.2.9-2 2.4-2 4s.8 3.1 2 4c1.2-.9 2-2.3 2-4s-.8-3.1-2-4zm3-1c2.8 0 5 2.2 5 5s-2.2 5-5 5c-.6 0-1.1-.1-1.6-.3 1.3-1.1 2.1-2.9 2.1-4.7s-.8-3.5-2.1-4.7c.5-.2 1-.3 1.6-.3zm.35 1.02c.73 1.15 1.14 2.52 1.14 3.98s-.42 2.83-1.14 3.98c2.04-.18 3.64-1.9 3.64-3.98s-1.6-3.8-3.64-3.98z"],"less-than":["M13.287 5.958a1 1 0 00-.574-1.916l-10 3c-.95.285-.95 1.631 0 1.916l10 3a1 1 0 00.574-1.916L6.48 8l6.807-2.042z"],"less-than-or-equal-to":["M13.287 3.958a1 1 0 00-.575-1.916l-10 3c-.95.285-.95 1.63 0 1.916l10 3a1 1 0 00.575-1.916L6.48 6l6.807-2.042zM13 12H3a1 1 0 000 2h10a1 1 0 000-2z"],lifesaver:["M9.405 11.746C8.968 11.91 8.495 12 8 12c-.494 0-.968-.09-1.405-.254l-.702 1.873C6.548 13.865 7.258 14 8 14c.742 0 1.452-.135 2.107-.38l-.702-1.874zm2.341-2.341l1.873.702C13.865 9.452 14 8.742 14 8c0-.742-.135-1.452-.38-2.107l-1.874.702c.164.437.254.91.254 1.405 0 .494-.09.968-.254 1.405zM9.405 4.254l.702-1.873A5.987 5.987 0 008 2c-.742 0-1.452.135-2.107.38l.702 1.874C7.032 4.09 7.505 4 8 4c.494 0 .968.09 1.405.254zM4.254 6.595L2.38 5.893A5.987 5.987 0 002 8c0 .742.135 1.452.38 2.107l1.874-.702A3.991 3.991 0 014 8c0-.494.09-.968.254-1.405zM8 16A8 8 0 118 0a8 8 0 010 16zm0-6a2 2 0 100-4 2 2 0 000 4z"],lightbulb:["M9.01 14h-2c-.55 0-1 .45-1 1s.45 1 1 1h2c.55 0 1-.45 1-1s-.44-1-1-1zm1-3h-4c-.55 0-1 .45-1 1s.45 1 1 1h4c.55 0 1-.45 1-1s-.44-1-1-1zm-2-11C5.26 0 3.03 1.95 3.03 4.35c0 2.37 1.63 2.64 1.94 5.22 0 .24.22.44.5.44h5.09c.28 0 .5-.19.5-.44C11.37 6.99 13 6.72 13 4.35 13 1.95 10.77 0 8.01 0z"],link:["M4.99 11.99c.28 0 .53-.11.71-.29l6-6a1.003 1.003 0 00-1.42-1.42l-6 6a1.003 1.003 0 00.71 1.71zm3.85-2.02L6.4 12.41l-1 1-.01-.01c-.36.36-.85.6-1.4.6-1.1 0-2-.9-2-2 0-.55.24-1.04.6-1.4l-.01-.01 1-1 2.44-2.44c-.33-.1-.67-.16-1.03-.16-1.1 0-2.09.46-2.81 1.19l-.02-.02-1 1 .02.02c-.73.72-1.19 1.71-1.19 2.81 0 2.21 1.79 4 4 4 1.1 0 2.09-.46 2.81-1.19l.02.02 1-1-.02-.02c.73-.72 1.19-1.71 1.19-2.81 0-.35-.06-.69-.15-1.02zm7.15-5.98c0-2.21-1.79-4-4-4-1.1 0-2.09.46-2.81 1.19l-.02-.02-1 1 .02.02c-.72.72-1.19 1.71-1.19 2.81 0 .36.06.69.15 1.02l2.44-2.44 1-1 .01.01c.36-.36.85-.6 1.4-.6 1.1 0 2 .9 2 2 0 .55-.24 1.04-.6 1.4l.01.01-1 1-2.43 2.45c.33.09.67.15 1.02.15 1.1 0 2.09-.46 2.81-1.19l.02.02 1-1-.02-.02a3.92 3.92 0 001.19-2.81z"],list:["M1 3h14c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1zm14 10H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zm0-4H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zm0-4H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1z"],"list-columns":["M6 1c.55 0 1 .45 1 1s-.45 1-1 1H1c-.55 0-1-.45-1-1s.45-1 1-1h5zm0 4c.55 0 1 .45 1 1s-.45 1-1 1H1c-.55 0-1-.45-1-1s.45-1 1-1h5zm0 4c.55 0 1 .45 1 1s-.45 1-1 1H1c-.55 0-1-.45-1-1s.45-1 1-1h5zm0 4c.55 0 1 .45 1 1s-.45 1-1 1H1c-.55 0-1-.45-1-1s.45-1 1-1h5zm9-12c.55 0 1 .45 1 1s-.45 1-1 1h-5c-.55 0-1-.45-1-1s.45-1 1-1h5zm0 4c.55 0 1 .45 1 1s-.45 1-1 1h-5c-.55 0-1-.45-1-1s.45-1 1-1h5zm0 4c.55 0 1 .45 1 1s-.45 1-1 1h-5c-.55 0-1-.45-1-1s.45-1 1-1h5zm0 4c.55 0 1 .45 1 1s-.45 1-1 1h-5c-.55 0-1-.45-1-1s.45-1 1-1h5z"],"list-detail-view":["M6 9H1c-.55 0-1 .45-1 1s.45 1 1 1h5c.55 0 1-.45 1-1s-.45-1-1-1zm0 4H1c-.55 0-1 .45-1 1s.45 1 1 1h5c.55 0 1-.45 1-1s-.45-1-1-1zm9-12h-5c-.55 0-1 .45-1 1v12c0 .55.45 1 1 1h5c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1zM6 5H1c-.55 0-1 .45-1 1s.45 1 1 1h5c.55 0 1-.45 1-1s-.45-1-1-1zm0-4H1c-.55 0-1 .45-1 1s.45 1 1 1h5c.55 0 1-.45 1-1s-.45-1-1-1z"],locate:["M15 7h-.09A6.98 6.98 0 009 1.1V1c0-.55-.45-1-1-1S7 .45 7 1v.09A6.98 6.98 0 001.1 7H1c-.55 0-1 .45-1 1s.45 1 1 1h.1A6.969 6.969 0 007 14.91V15c0 .55.45 1 1 1s1-.45 1-1v-.09A6.98 6.98 0 0014.9 9h.1c.55 0 1-.45 1-1s-.45-1-1-1zm-6.02 5.9c-.05-.5-.46-.9-.98-.9s-.93.4-.98.9A5.017 5.017 0 013.1 8.98c.5-.05.9-.46.9-.98s-.4-.93-.9-.98A5.017 5.017 0 017.02 3.1c.05.5.46.9.98.9s.93-.4.98-.9c1.97.39 3.52 1.95 3.92 3.92-.5.05-.9.46-.9.98s.4.93.9.98a5.017 5.017 0 01-3.92 3.92zM8 6c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2z"],lock:["M13.96 7H12V3.95C12 1.77 10.21 0 8 0S4 1.77 4 3.95V7H1.96c-.55 0-.96.35-.96.9v6.91c0 .54.41 1.19.96 1.19h12c.55 0 1.04-.65 1.04-1.19V7.9c0-.55-.49-.9-1.04-.9zM6 7V3.95c0-1.09.9-1.97 2-1.97s2 .88 2 1.97V7H6z"],"log-in":["M11 8c0-.28-.11-.53-.29-.71l-3-3a1.003 1.003 0 00-1.42 1.42L7.59 7H1c-.55 0-1 .45-1 1s.45 1 1 1h6.59L6.3 10.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71l3-3c.18-.18.29-.43.29-.71zm4-8H9c-.55 0-1 .45-1 1s.45 1 1 1h5v12H9c-.55 0-1 .45-1 1s.45 1 1 1h6c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1z"],"log-out":["M7 14H2V2h5c.55 0 1-.45 1-1s-.45-1-1-1H1C.45 0 0 .45 0 1v14c0 .55.45 1 1 1h6c.55 0 1-.45 1-1s-.45-1-1-1zm8.71-6.71l-3-3a1.003 1.003 0 00-1.42 1.42L12.59 7H6c-.55 0-1 .45-1 1s.45 1 1 1h6.59l-1.29 1.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71l3-3c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71z"],manual:["M15.99 1.13c-.02-.41-.33-.77-.78-.87C12.26-.36 9.84.13 8 1.7 6.16.13 3.74-.36.78.26.33.35.03.72.01 1.13H0v12c0 .08 0 .17.02.26.12.51.65.82 1.19.71 2.63-.55 4.59-.04 6.01 1.57.02.03.06.04.08.06.02.02.03.04.05.06.04.03.09.04.13.07.05.03.09.05.14.07.11.04.23.07.35.07h.04c.12 0 .24-.03.35-.07.05-.02.09-.05.14-.07.04-.02.09-.04.13-.07.02-.02.03-.04.05-.06.03-.02.06-.03.08-.06 1.42-1.6 3.39-2.12 6.01-1.57.54.11 1.07-.21 1.19-.71.04-.09.04-.18.04-.26l-.01-12zM7 12.99c-1.4-.83-3.07-1.14-5-.93V1.96c2.11-.28 3.75.2 5 1.46v9.57zm7-.92c-1.93-.21-3.6.1-5 .93V3.42c1.25-1.26 2.89-1.74 5-1.46v10.11z"],"manually-entered-data":["M1 8h3.76l2-2H1c-.55 0-1 .45-1 1s.45 1 1 1zm14.49-4.01c.31-.32.51-.76.51-1.24C16 1.78 15.22 1 14.25 1c-.48 0-.92.2-1.24.51l-1.44 1.44 2.47 2.47 1.45-1.43zM1 4h7.76l2-2H1c-.55 0-1 .45-1 1s.45 1 1 1zm0 6c-.55 0-1 .45-1 1 0 .48.35.86.8.96L2.76 10H1zm9.95-6.43l-6.69 6.69 2.47 2.47 6.69-6.69-2.47-2.47zm4.25 2.47L13.24 8H15c.55 0 1-.45 1-1 0-.48-.35-.86-.8-.96zM2 15l3.86-1.39-2.46-2.44L2 15zm13-5h-3.76l-2 2H15c.55 0 1-.45 1-1s-.45-1-1-1z"],map:["M15.55 3.17l-4.49-3A.975.975 0 009.99.15L5.53 2.82 1.56.17A1.003 1.003 0 000 1v11c0 .35.18.65.45.83l4.49 3a.975.975 0 001.07.02l4.46-2.67 3.97 2.65A1.003 1.003 0 0016 15V4c0-.35-.18-.65-.45-.83zM5 13.46l-3-2v-8.6l2.94 1.96c.02.02.04.03.06.04v8.6zm5-2.32s-.01 0-.01.01L6 13.53V4.86s.01 0 .01-.01L10 2.47v8.67zm4 1.99l-2.94-1.96c-.02-.01-.04-.02-.05-.03v-8.6l3 2v8.59z"],"map-create":["M14 6.82v6.32l-2.94-1.96c-.02-.01-.04-.02-.05-.03V6.22c-.08-.07-.15-.16-.22-.24-.28-.02-.54-.08-.79-.16v5.32s-.01 0-.01.01L6 13.53V4.86s.01 0 .01-.01l2.05-1.23C8.02 3.42 8 3.21 8 3c0-.98.47-1.84 1.2-2.39l-3.67 2.2L1.56.17A1.003 1.003 0 000 1v11c0 .35.18.65.45.83l4.49 3a.975.975 0 001.07.02l4.46-2.67 3.97 2.65A1.003 1.003 0 0016 15V5.82c-.25.09-.52.14-.8.16-.33.36-.73.67-1.2.84zm-9 6.64l-3-2v-8.6l2.94 1.96c.02.02.04.03.06.04v8.6zM11 4h1v1c0 .55.45 1 1 1s1-.45 1-1V4h1c.55 0 1-.45 1-1s-.45-1-1-1h-1V1c0-.55-.45-1-1-1s-1 .45-1 1v1h-1c-.55 0-1 .45-1 1s.45 1 1 1z"],"map-marker":["M8.46 0C5.42 0 2.95 2.39 2.95 5.33 2.95 8.28 8.46 16 8.46 16s5.51-7.72 5.51-10.67C13.96 2.39 11.5 0 8.46 0zm0 8a2.5 2.5 0 010-5 2.5 2.5 0 010 5z"],maximize:["M5.99 8.99c-.28 0-.53.11-.71.29l-3.29 3.29v-1.59c0-.55-.45-1-1-1s-1 .45-1 1v4c0 .55.45 1 1 1h4c.55 0 1-.45 1-1s-.45-1-1-1H3.41L6.7 10.7a1.003 1.003 0 00-.71-1.71zm9-9h-4c-.55 0-1 .45-1 1s.45 1 1 1h1.59l-3.3 3.3a.99.99 0 00-.29.7 1.003 1.003 0 001.71.71l3.29-3.29V5c0 .55.45 1 1 1s1-.45 1-1V1c0-.56-.45-1.01-1-1.01z"],media:["M11.99 6.99c.55 0 1-.45 1-1s-.45-1-1-1-1 .45-1 1 .45 1 1 1zm3-5h-14c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1h14c.55 0 1-.45 1-1v-10c0-.55-.45-1-1-1zm-1 9l-5-3-1 2-3-4-3 5v-7h12v7z"],menu:["M1 4h14c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1zm14 8H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zm0-5H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1z"],"menu-closed":["M14.99 6.99h-9c-.55 0-1 .45-1 1s.45 1 1 1h9c.55 0 1-.45 1-1s-.45-1-1-1zm-12-2c-.28 0-.53.11-.71.29l-2 2a1.014 1.014 0 000 1.42l2 2a1.003 1.003 0 001.71-.71v-4c0-.55-.45-1-1-1zm3-1h9c.55 0 1-.45 1-1s-.45-1-1-1h-9c-.55 0-1 .45-1 1s.45 1 1 1zm9 8h-9c-.55 0-1 .45-1 1s.45 1 1 1h9c.55 0 1-.45 1-1s-.45-1-1-1z"],"menu-open":["M9.99 11.99h-9c-.55 0-1 .45-1 1s.45 1 1 1h9c.55 0 1-.45 1-1s-.45-1-1-1zm0-5h-9c-.55 0-1 .45-1 1s.45 1 1 1h9c.55 0 1-.45 1-1s-.45-1-1-1zm0-5h-9c-.55 0-1 .45-1 1s.45 1 1 1h9c.55 0 1-.45 1-1s-.45-1-1-1zm5.71 5.3l-2-2a1.003 1.003 0 00-1.71.71v4a1.003 1.003 0 001.71.71l2-2c.18-.18.29-.43.29-.71s-.11-.53-.29-.71z"],"merge-columns":["M5.71 5.29a1.003 1.003 0 00-1.42 1.42l.3.29H2V2h3v1.51c.52.06.99.29 1.34.65l.66.66V1c0-.55-.45-1-1-1H1C.45 0 0 .45 0 1v14c0 .55.45 1 1 1h5c.55 0 1-.45 1-1v-3.82l-.66.66c-.35.35-.82.59-1.34.65V14H2V9h2.59l-.3.29a1.003 1.003 0 001.42 1.42l2-2C7.89 8.53 8 8.28 8 8c0-.28-.11-.53-.29-.71l-2-2zM15 0h-5c-.55 0-1 .45-1 1v3.82l.66-.66c.35-.35.82-.59 1.34-.65V2h3v5h-2.59l.29-.29c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71l-2 2C8.11 7.47 8 7.72 8 8c0 .28.11.53.29.71l2 2a1.003 1.003 0 001.42-1.42l-.3-.29H14v5h-3v-1.51c-.52-.06-.99-.29-1.34-.65L9 11.18V15c0 .55.45 1 1 1h5c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1z"],"merge-links":["M8 7c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm0 4c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm0-8c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm6 3c-.93 0-1.71.64-1.93 1.5H11V3c0-1.66-1.34-3-3-3S5 1.34 5 3v4.5H3.93C3.71 6.64 2.93 6 2 6 .9 6 0 6.9 0 8s.9 2 2 2c.93 0 1.71-.64 1.93-1.5H5V13c0 1.66 1.34 3 3 3s3-1.34 3-3V8.5h1.07c.22.86 1 1.5 1.93 1.5 1.1 0 2-.9 2-2s-.9-2-2-2zm-4 7c0 1.1-.9 2-2 2s-2-.9-2-2V3c0-1.1.9-2 2-2s2 .9 2 2v10z"],minimize:["M15.99.99a1.003 1.003 0 00-1.71-.71l-3.29 3.29V1.99c0-.55-.45-1-1-1s-1 .45-1 1v4c0 .55.45 1 1 1h4c.55 0 1-.45 1-1s-.45-1-1-1H12.4l3.3-3.29c.18-.18.29-.43.29-.71zm-10 8h-4c-.55 0-1 .45-1 1s.45 1 1 1h1.59L.29 14.28a1.003 1.003 0 001.42 1.42L5 12.41V14c0 .55.45 1 1 1s1-.45 1-1v-4a1.02 1.02 0 00-1.01-1.01z"],minus:["M13 7H3c-.55 0-1 .45-1 1s.45 1 1 1h10c.55 0 1-.45 1-1s-.45-1-1-1z"],"mobile-phone":["M12 0H4c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1h8c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zM8 15c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zm3-3H5V3h6v9z"],"mobile-video":["M15 4c-.28 0-.53.11-.71.29L12 6.59V4c0-.55-.45-1-1-1H1c-.55 0-1 .45-1 1v8c0 .55.45 1 1 1h10c.55 0 1-.45 1-1V9.41l2.29 2.29c.18.19.43.3.71.3.55 0 1-.45 1-1V5c0-.55-.45-1-1-1z"],modal:["M15 1a1 1 0 011 1v12a1 1 0 01-1 1H1a1 1 0 01-1-1V2a1 1 0 011-1h14zm-1 4H2v8h12V5zm-3-3H9v2h2V2zm3 0h-2v2h2V2z"],"modal-filled":["M15 1H1c-.55 0-1 .45-1 1v12c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1zm1 4H0V3h16v2zm-3-2h-2V1h2v2z"],moon:["M15 11.38A7.835 7.835 0 017.85 16C3.51 16 0 12.49 0 8.15 0 4.97 1.89 2.23 4.62 1c-.45.99-.7 2.08-.7 3.23a7.85 7.85 0 007.85 7.85c1.15 0 2.24-.25 3.23-.7z"],more:["M2 6.03a2 2 0 100 4 2 2 0 100-4zM14 6.03a2 2 0 100 4 2 2 0 100-4zM8 6.03a2 2 0 100 4 2 2 0 100-4z"],mountain:["M16 13H3l6-9h1l2 2h1l3 7zm-2.5-3.5l-1-2.5h-1l-2-2-3 4.5L9 8l1 1 1-1 2.5 1.5zM5.94 7l-4.122 6H0l5-6h.94z"],move:["M15.71 7.29l-2-2a1.003 1.003 0 00-1.42 1.42l.3.29H9V3.41l.29.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71l-2-2C8.53.11 8.28 0 8 0s-.53.11-.71.29l-2 2a1.003 1.003 0 001.42 1.42l.29-.3V7H3.41l.29-.29c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71l-2 2C.11 7.47 0 7.72 0 8c0 .28.11.53.29.71l2 2a1.003 1.003 0 001.42-1.42L3.41 9H7v3.59l-.29-.29A.965.965 0 006 12a1.003 1.003 0 00-.71 1.71l2 2c.18.18.43.29.71.29s.53-.11.71-.29l2-2a1.003 1.003 0 00-1.42-1.42l-.29.3V9h3.59l-.29.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71l2-2c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71z"],mugshot:["M15 0H1C.45 0 0 .45 0 1v14c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 14h-.15c-.03-.09-.04-.16-.08-.25-.34-.79-2.01-1.31-3.12-1.8-1.11-.49-.96-.79-1-1.2-.01-.06-.01-.12-.01-.18.38-.34.69-.8.89-1.33 0 0 .01-.03.01-.04.04-.12.08-.24.11-.36.25-.05.4-.33.46-.59.06-.1.18-.36.15-.65-.04-.37-.19-.55-.35-.62v-.06c0-.48-.04-1.16-.13-1.61-.02-.12-.05-.25-.08-.37-.16-.55-.51-1.05-.96-1.39C9.26 3.19 8.6 3 8 3c-.59 0-1.26.19-1.73.55-.45.35-.8.84-.96 1.39-.04.13-.06.25-.08.38-.09.45-.13 1.13-.13 1.61v.06c-.18.06-.33.24-.37.62-.03.29.09.54.15.65.06.26.21.54.47.59.03.12.07.25.11.36 0 .01.01.02.01.02v.01c.21.54.53 1.01.92 1.35 0 .05-.01.11-.01.16-.04.41.08.7-1.03 1.2-1.11.49-2.77 1.01-3.12 1.8-.04.09-.05.16-.08.25H2V2h12v12z"],"multi-select":["M12 3.98H4c-.55 0-1 .45-1 1v1h8v5h1c.55 0 1-.45 1-1v-5c0-.55-.45-1-1-1zm3-3H7c-.55 0-1 .45-1 1v1h8v5h1c.55 0 1-.45 1-1v-5c0-.55-.45-1-1-1zm-6 6H1c-.55 0-1 .45-1 1v5c0 .55.45 1 1 1h8c.55 0 1-.45 1-1v-5c0-.55-.45-1-1-1zm-1 5H2v-3h6v3z"],music:["M15 0c-.07 0-.13.03-.19.04V.02l-10 2v.02C4.35 2.13 4 2.52 4 3v9.12c-.31-.07-.65-.12-1-.12-1.66 0-3 .9-3 2s1.34 2 3 2 3-.9 3-2V6.32l8-1.6v5.4c-.31-.07-.65-.12-1-.12-1.66 0-3 .9-3 2s1.34 2 3 2 3-.9 3-2V1c0-.55-.45-1-1-1z"],"new-drawing":["M14.9 11c.6 0 1 .5 1 1 0 .257-.073.44-.22.614l-.08.086-3 3c-.2.2-.4.3-.7.3-.5 0-1-.4-1-1 0-.257.073-.44.22-.614l.08-.086 3-3c.2-.2.4-.3.7-.3zM1.3.1l6.734 2.45a3.005 3.005 0 002.095 3.322 3.005 3.005 0 003.401 2.081L13.9 9.8v.2c0 .257-.073.44-.22.614l-.08.086-3 3c-.171.171-.343.27-.577.294L9.9 14h-.2l-5-1-.1-.01c-.231-.05-.45-.26-.56-.49L4 12.4l-4-11 .3-.3 5.8 5.8c-.1.2-.2.4-.2.6 0 .8.6 1.5 1.5 1.5s1.5-.7 1.5-1.5S8.2 6 7.4 6c-.16 0-.32.064-.48.14l-.12.06L1 .4l.3-.3zM13 0c.55 0 1 .45 1 1v1h1c.55 0 1 .45 1 1s-.45 1-1 1h-1v1c0 .503-.376.922-.861.99l-.013.002A.999.999 0 0113 6l.097-.006-.027.004a1 1 0 01-.037.001L13 6c-.55 0-1-.45-1-1V4h-1a.993.993 0 01-.855-.482A1 1 0 0110 3c0-.55.45-1 1-1h1V1c0-.55.45-1 1-1z"],"new-grid-item":["M6 0H1C.45 0 0 .45 0 1v5c0 .55.45 1 1 1h5c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm5 14c0-.55-.45-1-1-1s-1 .45-1 1v1c0 .55.45 1 1 1h1c.55 0 1-.45 1-1s-.45-1-1-1zM6 9H1c-.55 0-1 .45-1 1v5c0 .55.45 1 1 1h5c.55 0 1-.45 1-1v-5c0-.55-.45-1-1-1zm9 4c-.55 0-1 .45-1 1-.55 0-1 .45-1 1s.45 1 1 1h1c.55 0 1-.45 1-1v-1c0-.55-.45-1-1-1zm-4-4h-1c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1s1-.45 1-1c.55 0 1-.45 1-1s-.45-1-1-1zm4-9h-5c-.55 0-1 .45-1 1v5c0 .55.45 1 1 1h5c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm0 9h-1c-.55 0-1 .45-1 1s.45 1 1 1c0 .55.45 1 1 1s1-.45 1-1v-1c0-.55-.45-1-1-1z"],"new-layer":["M13.982 6.272l1.518.868-.01.01c.3.17.51.48.51.85s-.21.68-.51.85l.01.01-7 4-.01-.01A.94.94 0 018 13a.94.94 0 01-.49-.15l-.01.01-7-4 .01-.01A.977.977 0 010 8c0-.37.21-.68.51-.86L.5 7.13l7-4 .01.02A.94.94 0 018 3c.086 0 .168.014.246.038a2 2 0 105.736 3.234zM14 3c.55 0 1 .45 1 1s-.45 1-1 1h-1v1c0 .55-.45 1-1 1s-1-.45-1-1V5h-1c-.55 0-1-.45-1-1s.45-1 1-1h1V2c0-.55.45-1 1-1s1 .45 1 1v1h1z"],"new-layers":["M13 3h2a1 1 0 010 2h-2v2a1 1 0 01-2 0V5H9a1 1 0 110-2h2V1a1 1 0 012 0v2zm-3-1.983V2H9a2 2 0 100 4h1v1c0 .279.057.544.16.785l-1.71.855c-.14.07-.29.11-.45.11-.16 0-.31-.04-.45-.11l-7-3.5a.992.992 0 01.07-1.81l6.99-3a1.006 1.006 0 01.79 0l1.6.687zm.91 7.66a2 2 0 003.085-1.54l.555-.277c.14-.07.29-.11.45-.11.55 0 1 .45 1 1 0 .39-.23.73-.55.89l-7 3.5c-.14.07-.29.11-.45.11-.16 0-.31-.04-.45-.11l-7-3.5C.23 8.48 0 8.14 0 7.75c0-.55.45-1 1-1 .16 0 .31.04.45.11L8 10.13l2.91-1.453zM15 10.25c.55 0 1 .45 1 1 0 .39-.23.73-.55.89l-7 3.5c-.14.07-.29.11-.45.11-.16 0-.31-.04-.45-.11l-7-3.5c-.32-.16-.55-.5-.55-.89 0-.55.45-1 1-1 .16 0 .31.04.45.1L8 13.63l6.55-3.27c.14-.07.29-.11.45-.11z"],"new-link":["M15 3h-1V2c0-.55-.45-1-1-1s-1 .45-1 1v1h-1c-.55 0-1 .45-1 1s.45 1 1 1h1v1c0 .55.45 1 1 1s1-.45 1-1V5h1c.55 0 1-.45 1-1s-.45-1-1-1zm-3.5 6a2.5 2.5 0 00-2.45 2h-4.1a2.5 2.5 0 100 1h4.1a2.5 2.5 0 102.45-3z"],"new-object":["M8 4c0 .6.4 1 1 1h2v2c0 .6.4 1 1 1s1-.4 1-1V5h2c.6 0 1-.4 1-1s-.4-1-1-1h-2V1c0-.6-.4-1-1-1s-1 .4-1 1v2H9c-.6 0-1 .5-1 1zm6.5 2.5V7c0 1.4-1.1 2.5-2.5 2.5S9.5 8.4 9.5 7v-.5H9C7.6 6.5 6.5 5.4 6.5 4S7.6 1.5 9 1.5h.5V1c0-.3.1-.6.1-.8C9.1.1 8.6 0 8 0 3.6 0 0 3.6 0 8s3.6 8 8 8 8-3.6 8-8c0-.6-.1-1.3-.2-1.9-.4.3-.8.4-1.3.4z"],"new-person":["M9.12 12.69c-1.17-.53-1.01-.85-1.05-1.29-.01-.06-.01-.12-.01-.19.4-.37.73-.87.94-1.44 0 0 .01-.03.01-.04.05-.14.09-.27.12-.4.27-.06.43-.36.49-.63.06-.11.19-.39.16-.7-.04-.41-.2-.6-.38-.68v-.07c0-.51-.05-1.25-.14-1.74-.02-.13-.05-.27-.09-.4-.17-.6-.53-1.14-1.01-1.52C7.66 3.2 6.96 3 6.33 3c-.62 0-1.33.2-1.82.59-.49.38-.85.92-1.02 1.52-.04.13-.07.26-.09.4-.09.49-.13 1.23-.13 1.74v.06c-.19.08-.35.27-.39.68-.03.31.1.59.16.7.06.28.22.59.5.64.03.14.07.27.11.4 0 .01.01.02.01.02v.01c.22.59.55 1.1.96 1.46 0 .06-.01.12-.01.17-.04.44.08.76-1.09 1.29-1.17.53-2.93 1.1-3.29 1.95-.35.87-.2 1.37-.2 1.37h12.6s.15-.5-.22-1.36c-.36-.85-2.12-1.42-3.29-1.95zM14.89 2h-1V1c0-.55-.45-1-1-1s-1 .45-1 1v1h-1c-.55 0-1 .45-1 1s.45 1 1 1h1v1c0 .55.45 1 1 1s1-.45 1-1V4h1c.55 0 1-.45 1-1s-.45-1-1-1z"],"new-prescription":["M9.82 11.66l2.48-2.87c.12-.2.13-.37.04-.53-.11-.19-.3-.26-.52-.26h-1.29c-.27 0-.49.13-.63.34L8.44 9.9 6.95 8a.482.482 0 00-.08-.1L5.82 6.55c.57-.24 1.04-.57 1.42-1.01.49-.57.74-1.27.74-2.08 0-.51-.1-.99-.32-1.42-.21-.43-.51-.8-.89-1.11A4.1 4.1 0 005.42.24C4.91.08 4.34 0 3.72 0H.61C.26 0 0 .23 0 .56v9.89c0 .33.26.55.61.55h.8c.36 0 .61-.23.61-.56V6.99H3.3l3.73 4.74-2.71 3.48c-.12.2-.13.37-.04.53.11.19.3.26.52.26h1.27c.27 0 .51-.12.64-.34l1.69-2.15 1.66 2.14c.12.21.34.35.62.35h1.43c.2 0 .39-.08.5-.25.12-.18.09-.38-.02-.55l-2.77-3.54zM4.18 5H1.99V2.02h2.19c.62 0 1.08.13 1.38.37.29.22.44.62.44 1.08 0 .45-.15.94-.44 1.17-.31.23-.76.36-1.38.36zM15 2h-1V1c0-.55-.45-1-1-1s-1 .45-1 1v1h-1c-.55 0-1 .45-1 1s.45 1 1 1h1v1c0 .55.45 1 1 1s1-.45 1-1V4h1c.55 0 1-.45 1-1s-.45-1-1-1zM9.99 3.01c0 .02.01.04.01.06V2.95c0 .02-.01.04-.01.06z"],"new-text-box":["M5 6.5c0 .28.22.5.5.5H7v3.5c0 .28.22.5.5.5s.5-.22.5-.5V7h1.5c.28 0 .5-.22.5-.5S9.78 6 9.5 6h-4c-.28 0-.5.22-.5.5zM15 2h-1V1c0-.55-.45-1-1-1s-1 .45-1 1v1h-1c-.55 0-1 .45-1 1s.45 1 1 1h1v1c0 .55.45 1 1 1s1-.45 1-1V4h1c.55 0 1-.45 1-1s-.45-1-1-1zm-2 5c-.55 0-1 .45-1 1v5H3V4h5c.55 0 1-.45 1-1s-.45-1-1-1H2c-.55 0-1 .45-1 1v11c0 .55.45 1 1 1h11c.55 0 1-.45 1-1V8c0-.55-.45-1-1-1z"],ninja:["M16 5s-2.52 2.11-4.96 1.99C11.03 4.89 10.39.23 5 0c0 0 2.11 2.54 1.96 4.99C4.86 5.01.23 5.65 0 11c0 0 2.56-2.12 5.02-1.95.02 2.11.67 6.72 5.98 6.95 0 0-2.09-2.54-1.94-4.99 2.11-.02 6.71-.68 6.94-6.01zM8 9.5c-.83 0-1.5-.67-1.5-1.5S7.17 6.5 8 6.5s1.5.67 1.5 1.5S8.83 9.5 8 9.5z"],"not-equal-to":["M7.58 5l.44-2.196a1 1 0 011.96.392L9.62 5H13a1 1 0 010 2H9.22l-.4 2H13a1 1 0 010 2H8.42l-.44 2.196a1 1 0 01-1.96-.392L6.38 11H3a1 1 0 010-2h3.78l.4-2H3a1 1 0 110-2h4.58z"],notifications:["M8 16c1.1 0 2-.9 2-2H6c0 1.1.9 2 2 2zm6-5c-.55 0-1-.45-1-1V6c0-2.43-1.73-4.45-4.02-4.9 0-.04.02-.06.02-.1 0-.55-.45-1-1-1S7 .45 7 1c0 .04.02.06.02.1A4.992 4.992 0 003 6v4c0 .55-.45 1-1 1s-1 .45-1 1 .45 1 1 1h12c.55 0 1-.45 1-1s-.45-1-1-1z"],"notifications-snooze":["M9 14c0 1.1-.9 2-2 2s-2-.9-2-2zM7 0c.404 0 .755.243.912.59L7.9.6c-.7.6-.9 1.36-.9 1.9 0 .8.267 1.433.8 1.9-.533.6-.795 1.222-.784 1.867l.004.358A2.8 2.8 0 009.82 9.4L12 9.399V10c0 .51.388.935.884.993L13 11c.55 0 1 .45 1 1s-.45 1-1 1H1c-.55 0-1-.45-1-1s.45-1 1-1 1-.45 1-1V6c0-2.43 1.73-4.45 4.02-4.9L6 1c0-.55.45-1 1-1z","M13 6.702a.632.632 0 00-.632-.632h-1.743l2.208-2.734A.75.75 0 0013 2.864v-.3A.565.565 0 0012.435 2H9.561a.561.561 0 100 1.123h1.814L9.221 5.795A1 1 0 009 6.423v.279c0 .349.283.631.632.631h2.736A.632.632 0 0013 6.702z"],"notifications-updated":["M8 16c1.1 0 2-.9 2-2H6c0 1.1.9 2 2 2zm3.399-13.667l-.413.412A2.99 2.99 0 009 1.99a3 3 0 00-3 2.99c0 .8.32 1.558.876 2.114l2.002 1.992A2.99 2.99 0 0013 9.184V10c0 .55.45 1 1 1s1 .45 1 1-.45 1-1 1H2c-.55 0-1-.45-1-1s.45-1 1-1 1-.45 1-1V6c0-2.43 1.73-4.45 4.02-4.9 0-.04-.02-.06-.02-.1 0-.55.45-1 1-1s1 .45 1 1c0 .04-.02.06-.02.1a4.97 4.97 0 012.419 1.233zM10.29 7.67l-2-1.99a.99.99 0 01-.29-.7 1 1 0 011-.99c.27 0 .52.11.7.29l1.29 1.29 3.28-3.28c.18-.18.42-.29.7-.29.55 0 1 .44 1 .99 0 .28-.11.52-.3.7l-3.98 3.98a.99.99 0 01-1.4 0z"],"numbered-list":["M2.76 7h1.26V0h-.94c-.04.21-.12.39-.25.54-.13.15-.29.27-.48.36-.18.09-.39.16-.62.2-.23.04-.46.06-.71.06v.9h1.74V7zm-.59 7.17c.18-.12.37-.25.58-.37a10.763 10.763 0 001.24-.83c.2-.16.37-.33.52-.51.15-.19.28-.39.37-.61.09-.22.14-.47.14-.74 0-.22-.04-.45-.12-.7-.08-.26-.21-.49-.4-.69-.18-.21-.43-.39-.72-.52-.3-.14-.68-.21-1.12-.21-.41 0-.77.07-1.08.2-.32.14-.58.32-.8.56-.22.23-.38.51-.49.84-.11.32-.16.67-.16 1.05h1.19c.01-.24.03-.47.08-.67.05-.21.11-.39.21-.54.09-.15.22-.27.38-.36.16-.09.35-.13.59-.13.26 0 .47.04.63.12.16.08.29.18.38.3.09.12.15.25.18.39s.05.27.05.4c-.01.27-.08.5-.22.71-.14.21-.32.4-.53.57-.22.18-.45.34-.71.49-.26.15-.51.31-.74.47-.5.31-.89.68-1.17 1.11-.3.41-.44.91-.45 1.48h5v-1H1.43c.05-.15.14-.29.27-.43.14-.13.29-.26.47-.38zM15.01 1.99h-7c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h7c.55 0 1-.45 1-1v-1c0-.55-.44-1-1-1zm0 9h-7c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h7c.55 0 1-.45 1-1v-1c0-.55-.44-1-1-1z"],numerical:["M2.79 4.61c-.13.17-.29.3-.48.41-.18.11-.39.18-.62.23-.23.04-.46.07-.71.07v1.03h1.74V12h1.26V4h-.94c-.04.23-.12.44-.25.61zm4.37 5.31c.18-.14.37-.28.58-.42l.63-.45c.21-.16.41-.33.61-.51s.37-.38.52-.59c.15-.21.28-.45.37-.7.09-.25.13-.54.13-.85 0-.25-.04-.52-.12-.8-.07-.29-.2-.55-.39-.79a2.18 2.18 0 00-.73-.6c-.29-.15-.66-.23-1.11-.23-.41 0-.77.08-1.08.23-.31.16-.58.37-.79.64-.22.27-.38.59-.49.96-.11.37-.16.77-.16 1.2h1.19c.01-.27.03-.53.08-.77.04-.24.11-.45.21-.62.09-.18.22-.32.38-.42.16-.1.35-.15.59-.15.26 0 .47.05.63.14.15.09.28.21.37.35.09.14.15.29.18.45.03.16.05.31.05.45-.01.31-.08.58-.22.82-.14.23-.32.45-.53.65-.22.21-.46.39-.71.57-.26.18-.51.36-.75.54-.5.36-.89.78-1.17 1.27-.28.49-.43 1.06-.44 1.71h5v-1.15H6.43c.05-.17.14-.33.27-.49.13-.15.29-.29.46-.44zm8.5-1.56c-.23-.35-.54-.57-.95-.65v-.02c.34-.13.6-.34.76-.63.16-.29.24-.63.24-1.02 0-.34-.06-.64-.19-.9s-.3-.47-.51-.64c-.21-.17-.45-.3-.72-.38-.27-.09-.54-.13-.82-.13-.36 0-.68.07-.96.2-.28.13-.53.32-.72.55-.2.23-.36.51-.47.83-.11.32-.18.66-.19 1.04h1.15c-.01-.2.01-.39.06-.58.05-.19.12-.36.22-.51.1-.15.22-.27.37-.36.15-.09.32-.13.53-.13.32 0 .59.1.79.3.21.2.31.46.31.79 0 .23-.05.43-.14.59-.09.16-.21.29-.35.38-.15.09-.32.16-.51.19-.19.04-.38.05-.57.04v.93c.23-.01.45 0 .67.02.22.02.42.08.59.17.18.09.32.23.43.4.11.18.16.41.16.71 0 .44-.13.78-.39 1.02s-.58.36-.97.36c-.45 0-.79-.16-1.02-.47-.23-.31-.33-.7-.32-1.17H11c.01.4.06.77.17 1.1.11.33.26.61.47.85.21.23.46.42.77.54.31.13.67.19 1.08.19.34 0 .66-.05.96-.16.3-.11.57-.27.8-.47.23-.2.41-.45.55-.74.13-.27.2-.6.2-.97 0-.5-.11-.92-.34-1.27z"],office:["M15 5h-3V1c0-.55-.45-1-1-1H1C.45 0 0 .45 0 1v14c0 .55.45 1 1 1h3v-4h4v4h7c.55 0 1-.45 1-1V6c0-.55-.45-1-1-1zM5 10H2V7h3v3zm0-5H2V2h3v3zm5 5H7V7h3v3zm0-5H7V2h3v3zm4 9h-2v-2h2v2zm0-4h-2V7h2v3z"],offline:["M8 0C3.58 0 0 3.58 0 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zM6 14l1-5H4l6-7-1 5h3l-6 7z"],"oil-field":["M15 14h-1.35l-3.34-7.51 2.46-.95 1.45 3.21c.09.2.36.3.6.23.1-.03.18-.08.24-.15.05-.08 1.23-1.56.87-4.2-.11-.79-.52-4.62-3.26-4.62-.93 0-1.68.62-1.67 1.37 0 .14.03.28.09.42l.87 1.92L.64 8.07v.01A.98.98 0 000 9c0 .55.45 1 1 1 .13 0 .25-.03.36-.07v.01l1.04-.4L3.67 14H2c-.55 0-1 .45-1 1s.45 1 1 1h13c.55 0 1-.45 1-1s-.45-1-1-1zM4.27 8.81L7.14 7.7 5.2 12.08l-.93-3.27zM6.54 14L9 8.46 11.46 14H6.54z"],"one-column":["M11.99-.01h-3c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1h3c.55 0 1-.45 1-1v-14c0-.55-.45-1-1-1zm-6 5c-.28 0-.53.11-.71.29l-2 2a1.014 1.014 0 000 1.42l2 2a1.003 1.003 0 001.71-.71v-4c0-.55-.45-1-1-1z"],outdated:["M8 0c4.42 0 8 3.58 8 8 0 4.06-3.02 7.4-6.94 7.92-.02 0-.04.01-.06.01-.33.04-.66.07-1 .07-4.42 0-8-3.58-8-8 0-.55.45-1 1-1s1 .45 1 1c0 3.31 2.69 6 6 6 .71 0 1.37-.15 2-.38v.01c2.33-.82 4-3.02 4-5.63 0-3.31-2.69-6-6-6-1.78 0-3.36.78-4.46 2H5c.55 0 1 .45 1 1s-.45 1-1 1H1c-.55 0-1-.45-1-1V1c0-.55.45-1 1-1s1 .45 1 1v1.74A7.95 7.95 0 018 0zm1 12H7v-2h2v2zm0-3H7V4h2v5z"],"page-layout":["M15 .95H1c-.55 0-1 .45-1 1v12c0 .55.45 1 1 1h14c.55 0 1-.45 1-1v-12c0-.55-.45-1-1-1zm-9 12H2v-6h4v6zm8 0H7v-6h7v6zm0-7H2v-3h12v3z"],"panel-stats":["M10 4h3v1h-3zM10 6h3v1h-3zM10 8h3v1h-3zM10 10h3v1h-3z","M15 1H1c-.6 0-1 .4-1 1v11c0 .6.4 1 1 1h14c.6 0 1-.4 1-1V2c0-.6-.4-1-1-1zM8 12H2V3h6v9zm6 0H9V3h5v9z"],"panel-table":["M15 1H1c-.6 0-1 .4-1 1v11c0 .6.4 1 1 1h14c.6 0 1-.4 1-1V2c0-.6-.4-1-1-1zM8 9H6V7h2v2zm0-3H6V4h2v2zm-6 6V3h3v9H2zm4 0v-2h2v2H6zm8 0H9v-2h5v2zm0-3H9V7h5v2zm0-3H9V4h5v2z"],paperclip:["M14.68 2.31A4.54 4.54 0 0011.46.99c-1.15 0-2.31.44-3.19 1.32L.95 9.63c-.63.63-.95 1.46-.95 2.28a3.21 3.21 0 003.23 3.22c.83 0 1.66-.31 2.3-.95l7.31-7.32c.76-.77.76-1.98.01-2.73s-1.99-.76-2.75 0l-6.07 6.08c-.24.25-.24.65.01.9s.65.25.91.01l6.07-6.08c.25-.25.67-.25.91-.01.25.25.25.67 0 .92l-7.31 7.32c-.75.75-2.04.74-2.76.01-.75-.75-.73-2.02.01-2.76L9.2 3.21c1.24-1.24 3.35-1.26 4.58-.03 1.24 1.24 1.24 3.36 0 4.6l-7.12 7.13c-.24.25-.24.64.01.88.24.24.63.24.88.01v.01l7.13-7.13A4.41 4.41 0 0016 5.51c0-1.16-.44-2.32-1.32-3.2z"],paragraph:["M13 1H6C3.8 1 2 2.8 2 5s1.8 4 4 4v5c0 .6.4 1 1 1s1-.5 1-1V3h2v11c0 .6.4 1 1 1s1-.5 1-1V3h1c.5 0 1-.4 1-1s-.4-1-1-1z"],path:["M14.5 0h-13C.67 0 0 .67 0 1.5S.67 3 1.5 3H7v3H3.5C2.67 6 2 6.67 2 7.5S2.67 9 3.5 9H7v3H5.5c-.83 0-1.5.67-1.5 1.5S4.67 15 5.5 15h5c.83 0 1.5-.67 1.5-1.5s-.67-1.5-1.5-1.5H9V9h3.5c.83 0 1.5-.67 1.5-1.5S13.33 6 12.5 6H9V3h5.5c.83 0 1.5-.67 1.5-1.5S15.33 0 14.5 0z"],"path-search":["M15 14.62l-4-2.4V9.77c-.32.09-.66.15-1 .18v2.27l-4 2.4V8.71c-.38-.31-.72-.66-1-1.06v6.97l-4-2.4V8c.55 0 1-.45 1-1s-.45-1-1-1V1.38l3.15 1.89c.08-.34.18-.66.32-.97L.76.07v.01A.496.496 0 00.5 0C.22 0 0 .22 0 .5v12c0 .18.1.33.25.42v.01l5 3v-.01c.07.05.16.08.25.08s.18-.03.25-.08v.01l4.74-2.85 4.74 2.85v-.01c.09.05.18.08.27.08.28 0 .5-.22.5-.5v-3.78c-.3.17-.63.28-1 .28v2.62zM2 5c0 .55.45 1 1 1s1-.45 1-1-.45-1-1-1-1 .45-1 1zm6-1c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm7.75-.92l-1.19-.72c.18.43.29.9.36 1.38l.08.04v3.39l1 1V3.5c0-.18-.1-.33-.25-.42zM10 2c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm3.3 4.89c.44-.7.7-1.51.7-2.39C14 2.01 11.99 0 9.5 0S5 2.01 5 4.5 7.01 9 9.5 9c.88 0 1.69-.26 2.39-.7l2.41 2.41c.17.18.42.29.7.29a1.003 1.003 0 00.71-1.71l-2.41-2.4zM9.5 8C7.57 8 6 6.43 6 4.5S7.57 1 9.5 1 13 2.57 13 4.5 11.43 8 9.5 8z"],pause:["M6 3H4c-.55 0-1 .45-1 1v8c0 .55.45 1 1 1h2c.55 0 1-.45 1-1V4c0-.55-.45-1-1-1zm6 0h-2c-.55 0-1 .45-1 1v8c0 .55.45 1 1 1h2c.55 0 1-.45 1-1V4c0-.55-.45-1-1-1z"],people:["M13.69 13.98c-.05-.24-.14-.5-.25-.76-.36-.86-1.12-1.33-2.69-2-.14-.06-.59-.25-.6-.25-.21-.09-.36-.15-.5-.22.02-.1.02-.2.03-.31 0-.04.01-.08.01-.13-.07-.06-.13-.12-.19-.19.22-.32.4-.67.54-1.05.02-.06.02-.06.03-.1.29-.23.48-.57.59-.96.16-.33.25-.73.21-1.16-.03-.4-.16-.76-.37-1.03-.02-.53-.07-1.13-.15-1.54-.01-.06-.02-.12-.03-.19.23-.06.48-.09.72-.09.49 0 1.05.16 1.44.46.38.29.67.7.8 1.17.03.1.05.21.07.31.07.37.11.94.11 1.33v.05c.14.06.27.21.29.51.02.25-.07.45-.13.54-.05.21-.16.44-.38.48-.02.1-.05.2-.09.3 0 .01-.01.03-.01.03-.17.44-.43.83-.75 1.11v.14c.03.35-.09.59.83 1 .93.41 2.32.84 2.6 1.5.29.66.17 1.04.17 1.04h-2.3zm-1.17-.38c.37.86.22 1.36.22 1.36H.06s-.14-.5.22-1.36 2.13-1.43 3.31-1.96c1.17-.54 1.05-.86 1.09-1.3 0-.05.01-.11.01-.17-.41-.35-.75-.86-.97-1.45v-.01s-.01-.01-.01-.02c-.04-.12-.09-.26-.12-.39-.28-.05-.44-.36-.5-.64-.06-.12-.19-.39-.16-.71.04-.41.21-.6.39-.68v-.06c0-.51.05-1.26.14-1.74.02-.13.05-.27.09-.4.17-.6.54-1.13 1.02-1.51.5-.39 1.21-.6 1.84-.6s1.34.21 1.84.6c.48.38.85.91 1.02 1.52.04.13.07.27.09.4.09.48.14 1.22.14 1.73v.07c.18.08.34.27.37.67.03.32-.09.59-.16.71-.06.28-.21.58-.48.63-.03.13-.07.26-.12.39 0 .01-.01.04-.01.04-.22.58-.55 1.08-.95 1.45v.18c.04.45-.12.77 1.06 1.3 1.18.53 2.95 1.09 3.31 1.95z"],percentage:["M6 6V4c0-1.1-.9-2-2-2H3c-1.1 0-2 .9-2 2v2c0 1.1.9 2 2 2h1c1.1 0 2-.9 2-2zM3.5 6c-.28 0-.5-.22-.5-.5v-1c0-.28.22-.5.5-.5s.5.22.5.5v1c0 .28-.22.5-.5.5zM13 8h-1c-1.1 0-2 .9-2 2v2c0 1.1.9 2 2 2h1c1.1 0 2-.9 2-2v-2c0-1.1-.9-2-2-2zm0 3.5c0 .28-.22.5-.5.5s-.5-.22-.5-.5v-1c0-.28.22-.5.5-.5s.5.22.5.5v1zM12 3a1.003 1.003 0 00-1.87-.5l-5.99 9.98c-.09.15-.14.33-.14.52a1.003 1.003 0 001.87.5l5.99-9.98c.09-.15.14-.33.14-.52z"],person:["M15.68 14.32c-.46-1.05-2.68-1.75-4.16-2.4-1.48-.65-1.28-1.05-1.33-1.59-.01-.07-.01-.15-.01-.23.51-.45.92-1.07 1.19-1.78 0 0 .01-.04.02-.05.06-.15.11-.32.15-.48.34-.07.54-.44.61-.78.08-.14.23-.48.2-.87-.05-.5-.25-.73-.47-.82v-.09c0-.63-.06-1.55-.17-2.15A3.671 3.671 0 0010.32.72C9.68.25 8.79-.01 8-.01c-.79 0-1.68.25-2.31.73-.61.47-1.06 1.13-1.28 1.86-.05.17-.09.33-.11.5-.12.6-.17 1.51-.17 2.15v.08c-.24.09-.45.32-.5.83-.03.38.13.72.2.86.08.35.28.72.63.78.04.17.09.33.15.49 0 .01.01.02.01.03l.01.01c.27.72.7 1.35 1.22 1.8 0 .07-.01.14-.01.21-.05.54.1.94-1.37 1.59-1.48.65-3.7 1.35-4.16 2.4-.46 1.05-.27 1.67-.27 1.67h15.92c-.01.01.18-.61-.28-1.66z"],phone:["M15.9 12.41c-.06-.06-3.37-2-3.48-2.05a.794.794 0 00-.32-.08c-.15 0-.34.11-.57.32-.23.22-.94 1.19-1.15 1.4-.21.22-.38.32-.52.32-.07 0-.15-.02-.25-.06-.1-.04-1.16-.58-3.36-2.52-2.2-1.93-2.49-3.2-2.5-3.55 0-.14.11-.31.32-.52.22-.21.45-.41.7-.6.25-.19.49-.4.7-.62.22-.23.32-.42.32-.57 0-.11-.03-.21-.08-.32C5.66 3.46 3.66.15 3.59.08 3.44-.07 2.85 0 2.55.16.16 1.46-.03 3.2 0 3.89c.04.71.49 4.46 4.16 7.95C8.72 16.17 11.89 16 12.1 16c.69 0 2.82-.38 3.72-2.55.13-.32.25-.87.08-1.04z"],"pie-chart":["M7 1.08c-3.37.5-5.97 3.4-5.97 6.92 0 3.87 3.13 7 6.98 7 3.52 0 6.42-2.61 6.91-6H7V1.08z","M8 0v8h8c0-4.42-3.58-8-8-8z"],pin:["M9.41.92c-.51.51-.41 1.5.15 2.56L4.34 7.54C2.8 6.48 1.45 6.05.92 6.58l3.54 3.54-3.54 4.95 4.95-3.54 3.54 3.54c.53-.53.1-1.88-.96-3.42l4.06-5.22c1.06.56 2.04.66 2.55.15L9.41.92z"],pivot:["M4.57 7.02L.3 11.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71l4.27-4.27c-.58-.35-1.07-.84-1.41-1.42zM15 8c-.55 0-1 .45-1 1v.59l-2.57-2.57c-.34.58-.83 1.07-1.41 1.41L12.59 11H12c-.55 0-1 .45-1 1s.45 1 1 1h3c.55 0 1-.45 1-1V9c0-.55-.45-1-1-1zm-4-3c0-1.66-1.34-3-3-3S5 3.34 5 5s1.34 3 3 3 3-1.34 3-3zM8 6c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1z"],"pivot-table":["M2 4H1c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1h1c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1zm0-4H1C.45 0 0 .45 0 1v1c0 .55.45 1 1 1h1c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm11.71 4.29C13.53 4.11 13.28 4 13 4s-.53.11-.71.29l-2 2a1.003 1.003 0 001.42 1.42l.29-.3V9c0 1.66-1.34 3-3 3H7.41l.29-.29c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71l-2 2c-.18.18-.29.43-.29.71 0 .28.11.53.29.71l2 2a1.003 1.003 0 001.42-1.42l-.3-.29H9c2.76 0 5-2.24 5-5V7.41l.29.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71l-2-2zM15 0H5c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h10c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1z"],play:["M12 8c0-.35-.19-.64-.46-.82l.01-.02-6-4-.01.02A.969.969 0 005 3c-.55 0-1 .45-1 1v8c0 .55.45 1 1 1 .21 0 .39-.08.54-.18l.01.02 6-4-.01-.02c.27-.18.46-.47.46-.82z"],plus:["M13 7H9V3c0-.55-.45-1-1-1s-1 .45-1 1v4H3c-.55 0-1 .45-1 1s.45 1 1 1h4v4c0 .55.45 1 1 1s1-.45 1-1V9h4c.55 0 1-.45 1-1s-.45-1-1-1z"],"polygon-filter":["M14 5c-.24 0-.47.05-.68.13L9.97 2.34c.01-.11.03-.22.03-.34 0-1.1-.9-2-2-2S6 .9 6 2c0 .04.01.08.01.12L2.88 4.21C2.61 4.08 2.32 4 2 4 .9 4 0 4.9 0 6c0 .74.4 1.38 1 1.72v4.55c-.6.35-1 .99-1 1.73 0 1.1.9 2 2 2 .74 0 1.38-.4 1.72-1h4.55c.35.6.98 1 1.72 1 1.1 0 2-.9 2-2 0-.37-.11-.7-.28-1L14 9c1.11-.01 2-.9 2-2s-.9-2-2-2zm-4.01 7c-.73 0-1.37.41-1.71 1H3.73c-.18-.3-.43-.55-.73-.72V7.72c.6-.34 1-.98 1-1.72 0-.04-.01-.08-.01-.12l3.13-2.09c.27.13.56.21.88.21.24 0 .47-.05.68-.13l3.35 2.79c-.01.11-.03.22-.03.34 0 .37.11.7.28 1l-2.29 4z"],power:["M8 8c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1S7 .45 7 1v6c0 .55.45 1 1 1zm3-5.32v2.34c1.21.91 2 2.35 2 3.98 0 2.76-2.24 5-5 5s-5-2.24-5-5c0-1.63.79-3.06 2-3.98V2.68C2.64 3.81 1 6.21 1 9c0 3.87 3.13 7 7 7s7-3.13 7-7c0-2.79-1.64-5.19-4-6.32z"],"predictive-analysis":["M16 6.41c0-1.01-.49-1.94-1.29-2.49-.43-1.92-2.07-3.28-4-3.28-.46 0-.92.08-1.35.24C8.83.31 8.11 0 7.34 0c-.9 0-1.74.44-2.28 1.16-.12-.01-.24-.02-.36-.02-1.31 0-2.42.89-2.77 2.17C.78 3.72 0 4.84 0 6.13c0 .38.07.76.21 1.12C.07 7.6 0 7.98 0 8.36c0 1.11.58 2.11 1.51 2.63.54.56 1.27.87 2.03.87.49 0 .95-.12 1.37-.36a2.85 2.85 0 002.18 1.04c.52 0 1.03-.14 1.47-.42.49.39 1.07.65 1.69.73 1.04 1.15 1.84 2.63 1.84 2.64 0 0 .28.49.26.49.77 0 1.41-.16 1.32-1.04 0 .02-.73-2.31-.73-2.31.41-.21.75-.55.97-.98.9-.52 1.47-1.53 1.47-2.61 0-.24-.03-.48-.08-.71.45-.52.7-1.21.7-1.92zm-1.23 1.02l-.15-.16-.61-.67c-.27-.29-.54-.94-.58-1.39l-.1-1.01c-.05-.59-.94-.58-.91.11 0 .02.1 1.01.1 1.01.03.29.12.62.24.93-.06-.01-.12-.02-.18-.02 0 0-2.06-.1-2.05-.11-.58-.02-.71.97-.04 1l2.05.11c.42.02 1.04.3 1.29.58l.49.54.02.05c.08.21.12.44.12.66 0 .74-.41 1.41-1.07 1.75l-.16.08-.07.18c-.15.38-.48.66-.88.74l-.54.11.7 2.2c-.38-.61-.95-1.43-1.62-2.14l-.12-.13-.17-.01c-.41-.03-.8-.17-1.14-.38l1.36-1.18c.35-.31.83-.44.99-.39 0 0 .63.17.62.18.63.16.83-.74.23-.97l-.62-.18c-.55-.16-1.33.18-1.79.58l-1.53 1.33-.31.26c-.35.29-.75.44-1.2.44-.64 0-1.23-.33-1.58-.86V9.15c0-.4.17-.79.27-.85 0 0 .52-.34.51-.35.71-.53.18-1.23-.49-.89 0-.01-.52.35-.52.35-.26.15-.45.44-.58.77-.11-.11-.22-.2-.34-.28 0 0-1.53-1.01-1.53-1.02-.65-.45-1.2.51-.49.89 0-.01 1.51 1.02 1.51 1.02.37.24.62.78.62 1.09v.67c-.34.19-.63.29-.99.29-.54 0-1.05-.23-1.41-.63l-.05-.06-.07-.04c-.65-.34-1.05-1-1.05-1.73 0-.3.07-.6.2-.87l.12-.25L1.15 7c-.13-.27-.2-.56-.2-.87 0-.9.61-1.68 1.48-1.89l.31-.08.05-.34a1.926 1.926 0 012.38-1.58l.32.08.18-.31c.35-.6.99-.97 1.67-.97.44 0 .86.15 1.2.42l-.36.36v-.01l-.25.26c-.33.27-.74.42-.89.4 0 0-.67-.1-.67-.11-.67-.13-.87.86-.14 1.02.01 0 .67.11.67.11.02 0 .05 0 .07.01-.11.37-.15.77-.1 1.12 0 0 .17.99.15.99.11.52 1.06.36.93-.18 0-.01-.15-.99-.15-.99-.05-.37.12-.94.36-1.19l.39-.4c.05-.05.1-.09.15-.14l.74-.76c.4-.18.83-.27 1.27-.27 1.55 0 2.86 1.12 3.11 2.67l.04.25.21.12c.61.35.98 1 .98 1.7 0 .36-.1.7-.28 1.01z"],prescription:["M10.91 8.34c.14-.21.36-.34.63-.34h1.29c.22 0 .41.07.52.26.09.16.08.33-.04.53l-2.49 2.87 2.77 3.54c.12.17.14.37.02.55-.11.17-.3.25-.5.25h-1.44a.69.69 0 01-.61-.35L9.4 13.51l-1.69 2.15c-.13.21-.36.34-.63.34H5.8c-.22 0-.41-.07-.52-.26-.09-.16-.08-.33.04-.53l2.71-3.48L4.3 6.99H3.03v3.47c0 .33-.26.56-.62.56h-.8c-.35-.01-.61-.23-.61-.56V.56c0-.33.26-.56.62-.56h3.11c.62 0 1.19.08 1.7.24.51.16.96.39 1.34.69a3.194 3.194 0 011.21 2.53c0 .81-.25 1.5-.74 2.08-.37.44-.84.77-1.42 1.01L7.88 7.9c.04.04.07.08.08.1l1.49 1.9 1.46-1.56zM5.18 5c.62 0 1.08-.13 1.39-.37.29-.23.44-.71.44-1.16s-.15-.87-.44-1.1C6.26 2.12 5.8 2 5.18 2H2.99v3h2.19z"],presentation:["M15 1H9c0-.55-.45-1-1-1S7 .45 7 1H1c-.55 0-1 .45-1 1s.45 1 1 1v8c0 .55.45 1 1 1h3.59L3.3 14.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71L7 13.41V15c0 .55.45 1 1 1s1-.45 1-1v-1.59l2.29 2.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71L10.41 12H14c.55 0 1-.45 1-1V3c.55 0 1-.45 1-1s-.45-1-1-1zm-2 9H3V3h10v7z"],print:["M12 2.02c0-.55-.45-1-1-1H5c-.55 0-1 .45-1 1v1h8v-1zm3 2H1c-.55 0-1 .45-1 1v6c0 .55.45 1 1 1h1v-3h12v3h1c.55 0 1-.45 1-1v-6c0-.56-.45-1-1-1zm-1 3h-2v-1h2v1zm-3 6H5v-3H3v4c0 .55.45 1 1 1h8c.55 0 1-.45 1-1v-4h-2v3z"],projects:["M14 3c0-.55-.45-1-1-1H3c-.55 0-1 .45-1 1v1h12V3zm-2-3H4c-.55 0-1 .45-1 1h10c0-.55-.45-1-1-1zm3 5H1c-.55 0-1 .45-1 1v9c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V6c0-.55-.45-1-1-1zm-3 6c0 .55-.45 1-1 1H5c-.55 0-1-.45-1-1V9h1v2h6V9h1v2z"],properties:["M2 6C.9 6 0 6.9 0 8s.9 2 2 2 2-.9 2-2-.9-2-2-2zm4-3h9c.55 0 1-.45 1-1s-.45-1-1-1H6c-.55 0-1 .45-1 1s.45 1 1 1zm-4 9c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm13-5H6c-.55 0-1 .45-1 1s.45 1 1 1h9c.55 0 1-.45 1-1s-.45-1-1-1zm0 6H6c-.55 0-1 .45-1 1s.45 1 1 1h9c.55 0 1-.45 1-1s-.45-1-1-1zM2 0C.9 0 0 .9 0 2s.9 2 2 2 2-.9 2-2-.9-2-2-2z"],property:["M3 12c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm-.5-6.5a2.5 2.5 0 000 5 2.5 2.5 0 000-5zM7 3h8c.55 0 1-.45 1-1s-.45-1-1-1H7c-.55 0-1 .45-1 1s.45 1 1 1zm8 10H7c-.55 0-1 .45-1 1s.45 1 1 1h8c.55 0 1-.45 1-1s-.45-1-1-1zM3 0C1.9 0 1 .9 1 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm12 6H7c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h8c.55 0 1-.45 1-1V7c0-.55-.45-1-1-1z"],"publish-function":["M12.16 3.76c.15-.11.3-.16.47-.16.06 0 .17.02.34.06.16.04.31.06.43.06a.58.58 0 00.6-.6c0-.19-.06-.33-.17-.44-.11-.11-.28-.16-.49-.16-.19 0-.37.04-.54.13-.17.09-.39.27-.65.55-.2.21-.48.58-.87 1.11a5.22 5.22 0 00-.78-1.79l-2.05.32-.04.21c.15-.03.28-.04.39-.04.2 0 .37.08.5.25.21.26.5 1.03.88 2.33-.29.36-.49.6-.6.71-.18.19-.33.31-.45.36-.09.04-.19.07-.3.07-.09 0-.23-.04-.42-.13a.904.904 0 00-.36-.09c-.2 0-.36.06-.49.18a.59.59 0 00-.19.46c0 .18.06.32.18.43.12.11.28.16.48.16.2 0 .38-.04.55-.12.17-.08.39-.24.65-.49s.62-.65 1.07-1.19c.18.52.33.89.46 1.13.13.24.28.4.44.51.17.1.37.16.62.16.24 0 .49-.08.74-.25.33-.21.66-.58 1.01-1.09l-.21-.11c-.23.31-.41.5-.52.57a.44.44 0 01-.26.07c-.12 0-.24-.07-.36-.21-.2-.24-.46-.91-.8-2 .29-.49.54-.81.74-.96zM6.37 5.83l.68-2.53h.83l.2-.64h-.84c.24-.91.56-1.59.96-2.01.24-.27.48-.4.71-.4.05 0 .08.01.11.04s.04.06.04.1c0 .04-.03.11-.1.21-.06.1-.1.2-.1.29 0 .13.05.24.15.33.1.09.23.14.39.14.17 0 .31-.06.42-.17.12-.12.18-.27.18-.46 0-.21-.08-.39-.25-.52C9.57.07 9.3 0 8.93 0c-.59 0-1.12.16-1.59.48-.48.32-.93.85-1.36 1.59-.15.26-.29.42-.42.49s-.35.11-.64.1l-.19.65h.81L4.35 7.68c-.2.72-.33 1.16-.4 1.33-.1.24-.26.45-.46.62a.48.48 0 01-.31.1c-.03 0-.06-.01-.08-.03l-.03-.03c0-.02.03-.06.09-.11.06-.06.09-.15.09-.26 0-.13-.05-.23-.14-.32-.1-.09-.23-.13-.41-.13-.21 0-.38.05-.51.16A.52.52 0 002 9.4c0 .16.08.3.23.42.16.12.4.18.74.18.53 0 .99-.13 1.4-.39.41-.26.76-.65 1.07-1.19.3-.53.61-1.39.93-2.59zm2.34 3.46A.997.997 0 008 9c-.28 0-.53.11-.71.29l-2 2a1.003 1.003 0 001.42 1.42l.29-.3V15c0 .55.45 1 1 1s1-.45 1-1v-2.59l.29.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71l-2-2z"],pulse:["M15 8h-1.46l-1.7-2.55-.02.01A.984.984 0 0011 5c-.43 0-.79.27-.93.65h-.01l-1.69 4.51-1.38-8.32h-.02A.989.989 0 006 1c-.41 0-.77.25-.92.61L2.34 8H1c-.55 0-1 .45-1 1s.45 1 1 1h2c.41 0 .77-.25.92-.61l1.65-3.86 1.44 8.63h.02c.08.47.47.84.97.84.43 0 .79-.27.93-.65h.01l2.31-6.17.92 1.38.02-.01c.17.26.46.45.81.45h2c.55 0 1-.45 1-1s-.45-1-1-1z"],random:["M11.48 4h1.11l-.29.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71l2-2c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71l-2-2a1.003 1.003 0 00-1.42 1.42l.3.29H11c-.32 0-.59.16-.77.38l-.01-.01L8.28 4.8l1.28 1.6L11.48 4zm2.23 6.29a1.003 1.003 0 00-1.42 1.42l.3.29h-1.11l-7.7-9.62h-.01A.996.996 0 003 2H1c-.55 0-1 .45-1 1s.45 1 1 1h1.52l7.7 9.62.01-.01c.18.23.45.39.77.39h1.59l-.29.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71l2-2c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71l-2-2zM2.52 12H1c-.55 0-1 .45-1 1s.45 1 1 1h2c.32 0 .59-.16.77-.38l.01.01 1.94-2.42L4.44 9.6 2.52 12z"],record:["M8 3a5 5 0 100 10A5 5 0 108 3z"],redo:["M12 11c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm3.71-6.71l-3-3a1.003 1.003 0 00-1.42 1.42L12.59 4H5C2.24 4 0 6.24 0 9s2.24 5 5 5h4v-2H5c-1.66 0-3-1.34-3-3s1.34-3 3-3h7.59L11.3 7.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71l3-3c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71z"],refresh:["M14.99 6.99c-.55 0-1 .45-1 1 0 3.31-2.69 6-6 6-1.77 0-3.36-.78-4.46-2h1.46c.55 0 1-.45 1-1s-.45-1-1-1h-4c-.55 0-1 .45-1 1v4c0 .55.45 1 1 1s1-.45 1-1v-1.74a7.95 7.95 0 006 2.74c4.42 0 8-3.58 8-8 0-.55-.45-1-1-1zm0-7c-.55 0-1 .45-1 1v1.74a7.95 7.95 0 00-6-2.74c-4.42 0-8 3.58-8 8 0 .55.45 1 1 1s1-.45 1-1c0-3.31 2.69-6 6-6 1.77 0 3.36.78 4.46 2h-1.46c-.55 0-1 .45-1 1s.45 1 1 1h4c.55 0 1-.45 1-1v-4c0-.55-.45-1-1-1z"],"regression-chart":["M13 6.5c0 .83.67 1.5 1.5 1.5S16 7.33 16 6.5 15.33 5 14.5 5 13 5.67 13 6.5zM8.5 5c.83 0 1.5-.67 1.5-1.5S9.33 2 8.5 2 7 2.67 7 3.5 7.67 5 8.5 5zM9 9.5c0 .83.67 1.5 1.5 1.5s1.5-.67 1.5-1.5S11.33 8 10.5 8 9 8.67 9 9.5zM4.5 8C5.33 8 6 7.33 6 6.5S5.33 5 4.5 5 3 5.67 3 6.5 3.67 8 4.5 8zM15 12H3.26l12.03-8.59-.58-.81L2 11.67V3c0-.55-.45-1-1-1s-1 .45-1 1v10c0 .55.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1z"],remove:["M10.99 6.99h-6c-.55 0-1 .45-1 1s.45 1 1 1h6c.55 0 1-.45 1-1s-.45-1-1-1zm-3-7c-4.42 0-8 3.58-8 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm0 14c-3.31 0-6-2.69-6-6s2.69-6 6-6 6 2.69 6 6-2.68 6-6 6z"],"remove-column":["M14 0H4c-.55 0-1 .45-1 1v3h2V2h3v12H5v-2H3v3c0 .55.45 1 1 1h10c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 14h-3V2h3v12zm-8.71-3.29a1.003 1.003 0 001.42-1.42L4.41 8 5.7 6.71c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71L3 6.59l-1.29-1.3A1.003 1.003 0 00.29 6.71L1.59 8 .29 9.29a1.003 1.003 0 001.42 1.42L3 9.41l1.29 1.3z"],"remove-column-left":["M4 9h4c.55 0 1-.45 1-1s-.45-1-1-1H4c-.55 0-1 .45-1 1s.45 1 1 1zm11-9H1C.45 0 0 .45 0 1v14c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-5 14H2V2h8v12zm4 0h-3V2h3v12z"],"remove-column-right":["M15 0H1C.45 0 0 .45 0 1v14c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zM5 14H2V2h3v12zm9 0H6V2h8v12zM8 9h4c.55 0 1-.45 1-1s-.45-1-1-1H8c-.55 0-1 .45-1 1s.45 1 1 1z"],"remove-row-bottom":["M15 0H1C.45 0 0 .45 0 1v14c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 14H2V6h12v8zm0-9H2V2h12v3zm-8 6h4c.55 0 1-.45 1-1s-.45-1-1-1H6c-.55 0-1 .45-1 1s.45 1 1 1z"],"remove-row-top":["M15 0H1C.45 0 0 .45 0 1v14c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 14H2v-3h12v3zm0-4H2V2h12v8zM6 7h4c.55 0 1-.45 1-1s-.45-1-1-1H6c-.55 0-1 .45-1 1s.45 1 1 1z"],repeat:["M10 5c0 .55.45 1 1 1h4c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1s-1 .45-1 1v1.74A7.95 7.95 0 008 0C3.58 0 0 3.58 0 8c0 4.06 3.02 7.4 6.94 7.92.02 0 .04.01.06.01.33.04.66.07 1 .07 4.42 0 8-3.58 8-8 0-.55-.45-1-1-1s-1 .45-1 1c0 3.31-2.69 6-6 6-.71 0-1.37-.15-2-.38v.01C3.67 12.81 2 10.61 2 8c0-3.31 2.69-6 6-6 1.77 0 3.36.78 4.46 2H11c-.55 0-1 .45-1 1z"],reset:["M6 5c0 .55-.45 1-1 1H1c-.55 0-1-.45-1-1V1c0-.55.45-1 1-1s1 .45 1 1v1.74A7.95 7.95 0 018 0c4.42 0 8 3.58 8 8 0 4.06-3.02 7.4-6.94 7.92-.02 0-.04.01-.06.01-.33.04-.66.07-1 .07-4.42 0-8-3.58-8-8 0-.55.45-1 1-1s1 .45 1 1c0 3.31 2.69 6 6 6 .71 0 1.37-.15 2-.38v.01c2.33-.82 4-3.02 4-5.63 0-3.31-2.69-6-6-6-1.77 0-3.36.78-4.46 2H5c.55 0 1 .45 1 1z"],resolve:["M6.6 3.3C6.1 3.1 5.6 3 5 3 2.2 3 0 5.2 0 8s2.2 5 5 5c.6 0 1.1-.1 1.6-.3C5.3 11.6 4.5 9.9 4.5 8s.8-3.6 2.1-4.7zM8 4c-1.2.9-2 2.4-2 4s.8 3.1 2 4c1.2-.9 2-2.3 2-4s-.8-3.1-2-4zm3-1c-.6 0-1.1.1-1.6.3 1.3 1.2 2.1 2.9 2.1 4.7s-.8 3.6-2.1 4.7c.5.2 1 .3 1.6.3 2.8 0 5-2.2 5-5s-2.2-5-5-5z"],rig:["M5.71 3c0 1.1.96 2 2.14 2C9.04 5 10 3.96 10 3c0-1.96-1.47-3-2.14-3H5c0 1.96 2.68 1.4.71 3zm2.5 3l.01.01s0-.01-.01-.01zm6.5 8.29L10 9.59V7c0-.55-.45-1-1-1H7c-.55 0-1 .45-1 1v2.58l-4.71 4.7c-.18.19-.29.44-.29.72a1.003 1.003 0 001.71.71L6 12.42V15c0 .55.45 1 1 1h2c.55 0 1-.45 1-1v-2.58l3.29 3.29a1.003 1.003 0 001.42-1.42z"],"right-join":["M6.6 3.3C5.3 4.4 4.5 6.1 4.5 8s.8 3.6 2.1 4.7c-.5.2-1 .3-1.6.3-2.8 0-5-2.2-5-5s2.2-5 5-5c.6 0 1.1.1 1.6.3zm-1.96 8.68C3.92 10.83 3.5 9.46 3.5 8s.42-2.83 1.14-3.98C2.6 4.2 1 5.91 1 8s1.6 3.8 3.64 3.98zM8 4c-1.2.9-2 2.4-2 4s.8 3.1 2 4c1.2-.9 2-2.3 2-4s-.8-3.1-2-4zm3-1c2.8 0 5 2.2 5 5s-2.2 5-5 5c-.6 0-1.1-.1-1.6-.3 1.3-1.1 2.1-2.9 2.1-4.7s-.8-3.5-2.1-4.7c.5-.2 1-.3 1.6-.3z"],ring:["M8 0C3.58 0 0 3.58 0 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm0 12c-2.21 0-4-1.79-4-4s1.79-4 4-4 4 1.79 4 4-1.79 4-4 4z"],"rotate-document":["M12 2h-1.59l.29-.29c.19-.18.3-.43.3-.71A1.003 1.003 0 009.29.29l-2 2C7.11 2.47 7 2.72 7 3c0 .28.11.53.29.71l2 2a1.003 1.003 0 001.42-1.42l-.3-.29H12c.55 0 1 .45 1 1v3c0 .55.45 1 1 1s1-.45 1-1V5c0-1.66-1.34-3-3-3zM5.71 5.29A.997.997 0 005 5H1c-.55 0-1 .45-1 1v9c0 .55.45 1 1 1h7c.55 0 1-.45 1-1V9c0-.28-.11-.53-.29-.71l-3-3zM7 14H2V7h2v2c0 .55.45 1 1 1h2v4z"],"rotate-page":["M8 6H2c-.55 0-1 .45-1 1v8c0 .55.45 1 1 1h6c.55 0 1-.45 1-1V7c0-.55-.45-1-1-1zm-1 8H3V8h4v6zm5-12h-1.59l.29-.29c.19-.18.3-.43.3-.71A1.003 1.003 0 009.29.29l-2 2C7.11 2.47 7 2.72 7 3c0 .28.11.53.29.71l2 2a1.003 1.003 0 001.42-1.42l-.3-.29H12c.55 0 1 .45 1 1v3c0 .55.45 1 1 1s1-.45 1-1V5c0-1.66-1.34-3-3-3z"],route:["M11.669 5.066l.099.189c.113.213.236.434.367.661.226.39.468.78.709 1.151l-.198-.004-.48-.004c-1.745.003-2.369.233-2.369.688 0 .053.226.19 1.038.436l.84.24C13.9 9.064 15 9.83 15 11.63c0 2.123-1.607 3.122-4.027 3.366-.651.065-1.266.075-2.043.05l-.958-.035H5.196l.268-.406c.336-.517.672-1.052.998-1.593h1.636l.572.023c.857.036 1.475.034 2.103-.03 1.526-.153 2.227-.59 2.227-1.375 0-.531-.402-.84-1.66-1.22l-.691-.198c-1.04-.293-1.764-.562-2.222-.946C8.8 8.366 9 7.612 9 6.997a5.03 5.03 0 00-.184-1.334c.645-.395 1.598-.562 2.853-.597zM4 3a4.007 4.007 0 014 3.997C8 9.21 4 15 4 15l-.416-.62C2.56 12.827 0 8.767 0 6.997A4.002 4.002 0 014 3zm0 2a2 2 0 10.001 4.001A2 2 0 004 5zm10-4c1.103 0 1.996.896 2 1.999C16 4.105 14 7 14 7l-.293-.44C13.15 5.707 12 3.838 12 2.999 12 1.896 12.897 1 14 1z"],satellite:["M3 9c0-.6.4-1 1-1s1 .4 1 1c0 1.1.9 2 2 2 .6 0 1 .4 1 1s-.4 1-1 1c-2.2 0-4-1.8-4-4zM0 9c0-.6.4-1 1-1s1 .4 1 1c0 2.8 2.2 5 5 5 .6 0 1 .4 1 1s-.4 1-1 1c-3.9 0-7-3.1-7-7zm7 1c-.6 0-1-.4-1-1s.4-1 1-1 1 .4 1 1-.4 1-1 1zm1.3-2.8c-.4-.4-.4-1 0-1.4l4.5-4.5c.4-.4 1-.4 1.4 0l.5.5c.4.4.4 1 0 1.4l-4.5 4.5c-.4.4-1 .4-1.4 0l-.5-.5zM5.2.3c.4-.4 1-.4 1.4 0l2.1 2.1c.4.4.4 1 0 1.4l-.9.9c-.4.4-1 .4-1.4 0L4.3 2.6c-.4-.4-.4-1 0-1.4l.9-.9zm7 7c.4-.4 1-.4 1.4 0l2.1 2.1c.4.4.4 1 0 1.4l-.9.9c-.4.4-1 .4-1.4 0l-2.1-2.1c-.4-.4-.4-1 0-1.4l.9-.9z"],saved:["M6.71 9.29a1.003 1.003 0 00-1.42 1.42l2 2a.997.997 0 001.6-.27h.01l2-4h-.01c.06-.13.11-.28.11-.44 0-.55-.45-1-1-1-.39 0-.72.23-.89.56H9.1l-1.38 2.76-1.01-1.03zM9 0H3c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1h10c.55 0 1-.45 1-1V5L9 0zm3 14H4V2h4v4h4v8z"],"scatter-plot":["M15 12H2V3c0-.55-.45-1-1-1s-1 .45-1 1v10c0 .55.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zm-.5-7c.83 0 1.5-.67 1.5-1.5S15.33 2 14.5 2 13 2.67 13 3.5 13.67 5 14.5 5zm-3 4c.83 0 1.5-.67 1.5-1.5S12.33 6 11.5 6 10 6.67 10 7.5 10.67 9 11.5 9zm-4-2C8.33 7 9 6.33 9 5.5S8.33 4 7.5 4 6 4.67 6 5.5 6.67 7 7.5 7zm-3 4c.83 0 1.5-.67 1.5-1.5S5.33 8 4.5 8 3 8.67 3 9.5 3.67 11 4.5 11z"],search:["M15.55 13.43l-2.67-2.68a6.94 6.94 0 001.11-3.76c0-3.87-3.13-7-7-7s-7 3.13-7 7 3.13 7 7 7c1.39 0 2.68-.42 3.76-1.11l2.68 2.67a1.498 1.498 0 102.12-2.12zm-8.56-1.44c-2.76 0-5-2.24-5-5s2.24-5 5-5 5 2.24 5 5-2.24 5-5 5z"],"search-around":["M13.5 11c-.51 0-.98.15-1.38.42l-2.4-2.41c.17-.3.28-.64.28-1.01s-.11-.71-.28-1.01l2.41-2.41c.39.27.86.42 1.37.42a2.5 2.5 0 000-5A2.5 2.5 0 0011 2.5c0 .51.15.98.42 1.38l-2.41 2.4C8.71 6.11 8.37 6 8 6s-.71.11-1.01.28l-2.41-2.4c.27-.4.42-.87.42-1.38a2.5 2.5 0 00-5 0A2.5 2.5 0 002.5 5c.51 0 .98-.15 1.38-.42l2.41 2.41C6.11 7.29 6 7.63 6 8s.11.71.28 1.01l-2.41 2.41c-.39-.27-.86-.42-1.37-.42a2.5 2.5 0 000 5A2.5 2.5 0 005 13.5c0-.51-.15-.98-.42-1.38l2.41-2.41c.3.18.64.29 1.01.29s.71-.11 1.01-.28l2.41 2.41c-.27.39-.42.86-.42 1.37a2.5 2.5 0 005 0 2.5 2.5 0 00-2.5-2.5zm0-10c.83 0 1.5.67 1.5 1.5S14.33 4 13.5 4 12 3.33 12 2.5 12.67 1 13.5 1zm-11 3C1.67 4 1 3.33 1 2.5S1.67 1 2.5 1 4 1.67 4 2.5 3.33 4 2.5 4zm0 11c-.83 0-1.5-.67-1.5-1.5S1.67 12 2.5 12s1.5.67 1.5 1.5S3.33 15 2.5 15zm11 0c-.83 0-1.5-.67-1.5-1.5s.67-1.5 1.5-1.5 1.5.67 1.5 1.5-.67 1.5-1.5 1.5z"],"search-template":["M15.55 13.43l-2.67-2.67c.7-1.09 1.11-2.38 1.11-3.77 0-3.87-3.13-7-7-7s-7 3.13-7 7 3.13 7 7 7c1.39 0 2.68-.41 3.77-1.11l2.67 2.67a1.498 1.498 0 102.12-2.12zm-8.56-1.44c-2.76 0-5-2.24-5-5s2.24-5 5-5 5 2.24 5 5-2.24 5-5 5zm2.5-6h-5c-.28 0-.5.22-.5.5s.22.5.5.5h5c.28 0 .5-.22.5-.5s-.22-.5-.5-.5zm0-2h-5c-.28 0-.5.22-.5.5s.22.5.5.5h5c.28 0 .5-.22.5-.5s-.22-.5-.5-.5zm0 4h-5c-.28 0-.5.22-.5.5s.22.5.5.5h5c.28 0 .5-.22.5-.5s-.22-.5-.5-.5z"],"search-text":["M9 4H5c-.55 0-1 .45-1 1s.45 1 1 1h1v3c0 .55.45 1 1 1s1-.45 1-1V6h1c.55 0 1-.45 1-1s-.45-1-1-1zm6.56 9.44l-2.67-2.67C13.59 9.68 14 8.39 14 7c0-3.87-3.13-7-7-7S0 3.13 0 7s3.13 7 7 7c1.39 0 2.68-.41 3.77-1.11l2.67 2.67a1.498 1.498 0 102.12-2.12zM7 12c-2.76 0-5-2.24-5-5s2.24-5 5-5 5 2.24 5 5-2.24 5-5 5z"],"segmented-control":["M15 4H1c-.55 0-1 .45-1 1v6c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1zm-1 6H8V6h6v4z"],select:["M16 15c0-.28-.12-.52-.31-.69l.02-.02-3.12-3.12 3.41-.84-8.05-2.86c.03-.09.05-.17.05-.27V2c0-.55-.45-1-1-1H3c0-.55-.45-1-1-1S1 .45 1 1c-.55 0-1 .45-1 1s.45 1 1 1v4c0 .55.45 1 1 1h5.2c.1 0 .18-.02.27-.05L10.33 16l.85-3.41 3.12 3.12.02-.02c.16.19.4.31.68.31.04 0 .07-.02.1-.02s.06.02.1.02c.44 0 .8-.36.8-.8 0-.04-.02-.07-.02-.1s.02-.06.02-.1zM6 6H3V3h3v3z"],selection:["M8 0C3.58 0 0 3.58 0 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm0 14c-3.31 0-6-2.69-6-6s2.69-6 6-6 6 2.69 6 6-2.69 6-6 6zm0-9C6.34 5 5 6.34 5 8s1.34 3 3 3 3-1.34 3-3-1.34-3-3-3z"],"send-message":["M15.398 9.01l-13.87 6.865c-.536.267-1.176.081-1.422-.427A.953.953 0 010 15v-4.838l8.67-2.168L0 5.923V1.001C0 .47.407 0 1.004 0c.169 0 .416.04.567.116L15.403 7.07a1.084 1.084 0 01-.005 1.939z"],"send-to":["M15 7.5c-.8 0-1.5-.4-2-1l-1.2 1.2c-.4.5-1.1.7-1.8.7-1.4.1-2.5-1-2.5-2.4 0-.7.3-1.3.7-1.8L9.5 3c-.6-.5-1-1.2-1-2 0-.3.1-.7.2-1H8C3.6 0 0 3.6 0 8s3.6 8 8 8 8-3.6 8-8v-.7c-.3.1-.6.2-1 .2zM15 0h-4c-.6 0-1 .5-1 1s.4 1 1 1h1.6L9.3 5.3c-.2.2-.3.4-.3.7 0 .5.4 1 1 1 .3 0 .5-.1.7-.3L14 3.4V5c0 .6.4 1 1 1 .5 0 1-.4 1-1V1c0-.5-.4-1-1-1z"],"send-to-graph":["M6 9H2c-.55 0-1 .45-1 1s.45 1 1 1h1.59L.3 14.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71L5 12.41V14c0 .55.45 1 1 1s1-.45 1-1v-4c0-.55-.45-1-1-1zm8 .5c-.56 0-1.06.23-1.42.59l-2.13-1.24L8.99 8l3.59-2.09A2.002 2.002 0 0016 4.5c0-1.1-.9-2-2-2s-2 .9-2 2c0 .19.03.37.08.54L8.5 7.13v-3.2c.86-.22 1.5-1 1.5-1.93 0-1.1-.9-2-2-2S6 .9 6 2c0 .93.64 1.71 1.5 1.93v3.2l-.88-.52-2.7-1.57c.05-.17.08-.35.08-.54 0-1.1-.9-2-2-2s-2 .9-2 2 .9 2 2 2c.56 0 1.06-.23 1.42-.59l2.13 1.24 3.84 2.24 2.7 1.57c-.06.17-.09.35-.09.54 0 1.1.9 2 2 2s2-.9 2-2-.9-2-2-2z"],"send-to-map":["M6 9H2c-.55 0-1 .45-1 1s.45 1 1 1h1.59L.3 14.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71L5 12.41V14c0 .55.45 1 1 1s1-.45 1-1v-4c0-.55-.45-1-1-1zm9.55-5.83l-4.49-3A.975.975 0 009.99.15L5.53 2.82 1.56.17A1.003 1.003 0 000 1v6h2V2.87l2.94 1.96.06.03V7h1V4.86s.01 0 .01-.01L10 2.47v8.67s-.01 0-.01.01l-.99.58v2.33l1.47-.88 3.97 2.65A1.003 1.003 0 0016 15V4c0-.35-.18-.65-.45-.83zM14 13.13l-2.94-1.96c-.02-.01-.04-.02-.05-.03v-8.6l3 2v8.59z"],"series-add":["M10.68 7.9c.44.54 1.07.92 1.79 1.05l-2.76 2.76c-.18.18-.43.29-.71.29s-.53-.11-.71-.3L5 8.41l-3 3V13h13c.55 0 1 .45 1 1s-.45 1-1 1H1c-.55 0-1-.45-1-1V4c0-.55.45-1 1-1s1 .45 1 1v4.59l2.29-2.3C4.47 6.11 4.72 6 5 6s.53.11.71.29L9 9.59l1.68-1.69zM15 3c.55 0 1 .45 1 1s-.45 1-1 1h-1v1c0 .55-.45 1-1 1s-1-.45-1-1V5h-1c-.55 0-1-.45-1-1s.45-1 1-1h1V2c0-.55.45-1 1-1s1 .45 1 1v1h1z"],"series-configuration":["M9.94 9.64c.65.23 1.34.36 2.06.36.14 0 .29-.01.43-.01L9.7 12.71c-.18.18-.43.29-.71.29-.28 0-.53-.11-.71-.3L5 9.41l-3 3V14h12.99c.55 0 1 .45 1 1s-.45 1-1 1H1c-.55 0-1-.45-1-1V5c0-.55.45-1 1-1s1 .45 1 1v4.59l2.29-2.3C4.47 7.11 4.72 7 5 7c.28 0 .53.11.71.29L9 10.59l.94-.95zm4.73-6.44h.92c.22 0 .4.18.4.4v.8c0 .22-.18.4-.4.4h-.93c-.06.2-.14.38-.24.55l.66.65c.15.15.15.4 0 .55l-.54.55c-.15.15-.4.15-.55 0l-.65-.65c-.17.1-.36.18-.55.24v.91c0 .22-.18.4-.4.4h-.8c-.22 0-.4-.18-.4-.4v-.93c-.18-.06-.36-.13-.52-.22l-.68.68c-.15.16-.41.16-.57 0l-.56-.56a.417.417 0 010-.57l.68-.68c-.08-.16-.16-.33-.22-.52h-.93c-.22 0-.4-.18-.4-.4v-.8c0-.22.18-.4.4-.4h.93c.06-.2.14-.38.24-.55l-.65-.64a.392.392 0 010-.55l.54-.55a.38.38 0 01.54 0l.65.65c.18-.1.36-.18.55-.24V.4c0-.22.18-.4.4-.4h.8c.22 0 .4.18.4.4v.93c.18.06.35.14.52.22l.68-.68c.15-.16.41-.16.57 0l.57.57c.15.16.15.41 0 .57l-.68.68c.09.16.16.33.22.51zm-4.18.8c0 .83.67 1.5 1.5 1.5s1.5-.67 1.5-1.5-.67-1.5-1.5-1.5c-.82 0-1.5.67-1.5 1.5z"],"series-derived":["M10.66 7.92c.44.54 1.07.91 1.8 1.03L9.71 11.7c-.18.19-.43.3-.71.3s-.53-.11-.71-.3L5 8.41l-3 3V13h13c.55 0 1 .45 1 1s-.45 1-1 1H1c-.55 0-1-.45-1-1V4c0-.55.45-1 1-1s1 .45 1 1v4.59l2.29-2.3C4.47 6.11 4.72 6 5 6s.53.11.71.29L9 9.59l1.66-1.67zM12.3 5.3l.3-.3H8c-.6 0-1-.4-1-1s.4-1 1-1h4.6l-.3-.3c-.2-.2-.3-.4-.3-.7 0-.6.5-1 1-1 .3 0 .5.1.7.3l2 2c.2.2.3.4.3.7s-.1.5-.3.7l-2 2c-.2.2-.4.3-.7.3-.6 0-1-.4-1-1 0-.3.1-.5.3-.7z"],"series-filtered":["M9.29 9.3c.3.62.8 1.12 1.42 1.41l-1 1c-.18.18-.43.29-.71.29s-.53-.11-.71-.3L5 8.41l-3 3V13h13c.55 0 1 .45 1 1s-.45 1-1 1H1c-.55 0-1-.45-1-1V4c0-.55.45-1 1-1s1 .45 1 1v4.59l2.29-2.3C4.47 6.11 4.72 6 5 6s.53.11.71.29L9 9.59l.29-.29zM15.48 1c.31 0 .52.26.52.57 0 .16-.06.3-.17.41l-2.86 2.73v2.63c0 .16-.06.3-.17.41l-.82 1.1c-.1.1-.25.17-.41.17-.31 0-.57-.26-.57-.57V4.71L8.17 1.98A.566.566 0 018 1.57c0-.31.26-.57.57-.57h6.91z"],"series-search":["M9.6 8.94a4.937 4.937 0 001.82.01c.1-.01.22-.04.39-.08l.23-.07c.04-.01.08-.02.11-.04l.22.22-2.7 2.72c-.18.19-.43.3-.71.3s-.53-.11-.71-.3L4.98 8.41l-2.99 3V13h12.94c.55 0 1 .45 1 1s-.45 1-1 1H1c-.55 0-1-.45-1-1V3.99c0-.55.45-1 1-1s1 .45 1 1v4.59l2.28-2.3c.17-.18.42-.29.7-.29s.53.11.7.29l3.28 3.3.64-.64zm6.22-.41c.1.12.17.27.18.44 0 .34-.27.61-.61.61a.57.57 0 01-.43-.18l-2.24-2.25c-.13.08-.26.16-.4.23-.02.01-.05.02-.07.03-.14.06-.27.12-.42.17h-.01c-.14.05-.29.08-.44.11-.04.01-.08.02-.11.02-.15.02-.3.04-.46.04-1.85 0-3.35-1.51-3.35-3.37S8.96 1.01 10.81 1c1.85 0 3.35 1.51 3.35 3.37 0 .16-.02.31-.04.47-.01.04-.01.07-.02.11-.02.15-.05.29-.1.44v.01c-.05.15-.11.28-.17.42-.01.02-.02.05-.03.07-.07.14-.14.27-.23.4l2.25 2.24zm-5.01-1.94c1.22 0 2.21-.99 2.21-2.22 0-1.23-.99-2.22-2.21-2.22S8.6 3.14 8.6 4.37c0 1.22.99 2.22 2.21 2.22z"],settings:["M3 1c0-.55-.45-1-1-1S1 .45 1 1v3h2V1zm0 4H1c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h2c.55 0 1-.45 1-1V6c0-.55-.45-1-1-1zm12-4c0-.55-.45-1-1-1s-1 .45-1 1v2h2V1zM9 1c0-.55-.45-1-1-1S7 .45 7 1v6h2V1zM1 15c0 .55.45 1 1 1s1-.45 1-1v-5H1v5zM15 4h-2c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h2c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1zm-2 11c0 .55.45 1 1 1s1-.45 1-1V9h-2v6zM9 8H7c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h2c.55 0 1-.45 1-1V9c0-.55-.45-1-1-1zm-2 7c0 .55.45 1 1 1s1-.45 1-1v-2H7v2z"],share:["M10.99 13.99h-9v-9h4.76l2-2H.99c-.55 0-1 .45-1 1v11c0 .55.45 1 1 1h11c.55 0 1-.45 1-1V7.24l-2 2v4.75zm4-14h-5c-.55 0-1 .45-1 1s.45 1 1 1h2.59L7.29 7.28a1 1 0 00-.3.71 1.003 1.003 0 001.71.71l5.29-5.29V6c0 .55.45 1 1 1s1-.45 1-1V1c0-.56-.45-1.01-1-1.01z"],shield:["M8 16c4.667-3.048 7-7.238 7-12.571-1.556 0-3.889-1.143-7-3.429-3.111 2.286-5.444 3.429-7 3.429C1 8.762 3.333 12.952 8 16zM8 2.121c2.005 1.388 3.715 2.304 5.186 2.735-.342 3.702-2.05 6.683-5.186 9.038V2.121z"],shop:["M3 2h10c.55 0 1-.45 1-1s-.45-1-1-1H3c-.55 0-1 .45-1 1s.45 1 1 1zm9 11H4v-3H2v5c0 .55.45 1 1 1h10c.55 0 1-.45 1-1v-5h-2v3zm4-6l-1.01-3.17C14.9 3.36 14.49 3 14 3H2c-.49 0-.9.36-.98.83L.01 7H0c0 1.1.9 2 2 2s2-.9 2-2c0 1.1.9 2 2 2s2-.9 2-2c0 1.1.9 2 2 2s2-.9 2-2c0 1.1.9 2 2 2s2-.9 2-2z"],"shopping-cart":["M14 10H7.72l-.33-1H13c.39 0 .72-.23.89-.56h.01l2-4h-.01c.06-.13.11-.28.11-.44 0-.55-.45-1-1-1H5.39l-.44-1.32h-.01C4.8 1.29 4.44 1 4 1H1c-.55 0-1 .45-1 1s.45 1 1 1h2.28l2.33 7H4c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2h6c0 1.1.9 2 2 2s2-.9 2-2-.9-2-2-2zM6.05 5h7.33l-1 2H6.72l-.67-2z"],"signal-search":["M5.474 7.971A5.31 5.31 0 006.66 8.9l.007.019.018.056c.015.038.038.06.045.098l1.5 5.999a.75.75 0 01-1.455.36l-.42-1.68h-.704l-.42 1.68a.746.746 0 01-.907.547.746.746 0 01-.547-.907l1.5-6c.007-.037.03-.06.044-.097.015-.037.015-.075.038-.112a.722.722 0 01-.105-.36c0-.207.084-.394.22-.53zM2.795 5.277a.763.763 0 00-.015-1.065.756.756 0 00-1.065.015c-2.286 2.34-2.286 6.21 0 8.549a.747.747 0 101.072-1.042c-1.709-1.763-1.709-4.702.008-6.457zM7.808 9.388a5.318 5.318 0 001.58.211 2.236 2.236 0 01-.656.98.756.756 0 01-1.057-.098.756.756 0 01.097-1.057l.036-.036zM11.544 9.105l.378.378a6.02 6.02 0 01-1.638 3.285c-.285.3-.757.3-1.057.015a.74.74 0 01-.015-1.057 4.52 4.52 0 001.185-2.24c.4-.083.785-.212 1.147-.381z","M4.054 9.424c-.427-.352-.352-1.582-.03-1.822a.752.752 0 00.15-1.05.752.752 0 00-1.05-.15c-1.079.802-1.221 3.18-.03 4.177a.75.75 0 10.96-1.155zM9.318 0a4.318 4.318 0 014.317 4.318c0 .206-.02.402-.049.598-.01.05-.01.088-.02.138-.039.196-.078.382-.137.569v.01c-.059.186-.137.363-.216.54l-.039.087a5.285 5.285 0 01-.294.51l2.884 2.886a.878.878 0 01.236.559.787.787 0 01-.785.785.785.785 0 01-.56-.226L11.772 7.89a5.285 5.285 0 01-.51.295l-.089.039c-.176.079-.353.157-.54.216h-.01a3.701 3.701 0 01-.568.137c-.05.01-.099.02-.138.02-.196.03-.392.049-.598.049A4.318 4.318 0 015 4.327 4.332 4.332 0 019.318 0zm-.02 1.1A3.195 3.195 0 006.1 4.298a3.195 3.195 0 003.198 3.198 3.195 3.195 0 003.198-3.198A3.195 3.195 0 009.298 1.1z"],"sim-card":["M13.71 4.29l-4-4A.997.997 0 009 0H3c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1h10c.55 0 1-.45 1-1V5c0-.28-.11-.53-.29-.71zM7 6h2v2H7V6zM4 6h2v2H4V6zm2 8H4v-2h2v2zm3 0H7v-2h2v2zm3 0h-2v-2h2v2zm0-3H4V9h8v2zm0-3h-2V6h2v2z"],slash:["M10 2a.99.99 0 00-.96.73l-2.99 9.96A1.003 1.003 0 007 14c.46 0 .85-.31.96-.73l2.99-9.96A1.003 1.003 0 0010 2z"],"small-cross":["M9.41 8l2.29-2.29c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71L8 6.59l-2.29-2.3a1.003 1.003 0 00-1.42 1.42L6.59 8 4.3 10.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71L8 9.41l2.29 2.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71L9.41 8z"],"small-minus":["M11 7H5c-.55 0-1 .45-1 1s.45 1 1 1h6c.55 0 1-.45 1-1s-.45-1-1-1z"],"small-plus":["M11 7H9V5c0-.55-.45-1-1-1s-1 .45-1 1v2H5c-.55 0-1 .45-1 1s.45 1 1 1h2v2c0 .55.45 1 1 1s1-.45 1-1V9h2c.55 0 1-.45 1-1s-.45-1-1-1z"],"small-tick":["M12 5c-.28 0-.53.11-.71.29L7 9.59l-2.29-2.3a1.003 1.003 0 00-1.42 1.42l3 3c.18.18.43.29.71.29s.53-.11.71-.29l5-5A1.003 1.003 0 0012 5z"],snowflake:["M13.364 9l.879.879a1 1 0 11-1.415 1.414l-2.12-2.121A1.003 1.003 0 0110.568 9H9v1.604c.042.03.083.065.121.103l2.122 2.121a1 1 0 01-1.415 1.415L9 13.414V15a1 1 0 01-2 0v-1.636l-.879.879a1 1 0 11-1.414-1.415l2.121-2.12c.054-.054.111-.1.172-.139V9H5.38c-.038.06-.084.118-.137.172l-2.122 2.12A1 1 0 111.707 9.88L2.586 9H1a1 1 0 110-2h1.536l-.829-.828a1 1 0 011.414-1.415L5.243 6.88c.038.038.072.079.103.121H7V5.38a1.003 1.003 0 01-.172-.137L4.708 3.12A1 1 0 016.12 1.707L7 2.586V1a1 1 0 112 0v1.536l.828-.829a1 1 0 011.415 1.414L9.12 5.243A1.007 1.007 0 019 5.346V7h1.604c.03-.042.065-.083.103-.121l2.121-2.122a1 1 0 011.415 1.415L13.414 7H15a1 1 0 010 2h-1.636z"],"social-media":["M9.5 4c.4 0 .8-.1 1.1-.3C12 4.5 12.9 6 13 7.6c0 .5.5.9 1 .9.6 0 1-.4 1-1v-.2c-.2-2.4-1.5-4.4-3.5-5.5-.1-1-.9-1.8-2-1.8s-2 .9-2 2 .9 2 2 2zM4 8.5c0-.7-.4-1.3-.9-1.7.3-1.4 1.2-2.6 2.5-3.3.3-.1.6-.4.6-.9s-.4-1-1-1c-.2 0-.3 0-.5.1-1.9 1-3.2 2.8-3.6 5C.4 7.1 0 7.8 0 8.5c0 1.1.9 2 2 2s2-.9 2-2zm8.8 1.2c-1.1 0-2 .9-2 2v.3c-.8.6-1.8.9-2.8.9-1.2 0-2.3-.4-3.2-1.1-.2-.2-.4-.3-.7-.3-.6 0-1 .4-1 1 0 .3.1.6.3.8C4.6 14.4 6.2 15 8 15c1.5 0 3-.5 4.1-1.3.2.1.5.1.7.1 1.1 0 2-.9 2-2s-.9-2.1-2-2.1z"],sort:["M5 12c-.28 0-.53.11-.71.29l-.29.3V9c0-.55-.45-1-1-1s-1 .45-1 1v3.59l-.29-.29A.965.965 0 001 12a1.003 1.003 0 00-.71 1.71l2 2c.18.18.43.29.71.29s.53-.11.71-.29l2-2A1.003 1.003 0 005 12zm3-9h7c.55 0 1-.45 1-1s-.45-1-1-1H8c-.55 0-1 .45-1 1s.45 1 1 1zm7 2H8c-.55 0-1 .45-1 1s.45 1 1 1h7c.55 0 1-.45 1-1s-.45-1-1-1zm0 8H8c-.55 0-1 .45-1 1s.45 1 1 1h7c.55 0 1-.45 1-1s-.45-1-1-1zm0-4H8c-.55 0-1 .45-1 1s.45 1 1 1h7c.55 0 1-.45 1-1s-.45-1-1-1z"],"sort-alphabetical":["M6 12c-.28 0-.53.11-.71.29l-.29.3V9c0-.55-.45-1-1-1s-1 .45-1 1v3.59l-.29-.29A.965.965 0 002 12a1.003 1.003 0 00-.71 1.71l2 2c.19.18.44.29.71.29.28 0 .53-.11.71-.29l2-2c.18-.18.29-.43.29-.71a.99.99 0 00-1-1zm7.93-.95v-1.04H9.25v1.11h2.94L9 14.96V16h5.02v-1.11h-3.27l3.18-3.84zm-1.42-4.84l.62 1.78H15L11.94.01H10.1L7 7.99h1.81l.64-1.78h3.06zm-1.52-4.24h.02l1.03 2.93H9.92l1.07-2.93z"],"sort-alphabetical-desc":["M5.99 11.99c-.28 0-.53.11-.71.29l-.29.29V8.99c0-.55-.45-1-1-1s-1 .45-1 1v3.59l-.29-.29a1.003 1.003 0 00-1.42 1.42l2 2c.18.18.43.29.71.29.28 0 .53-.11.71-.29l2-2c.18-.18.29-.43.29-.71 0-.56-.45-1.01-1-1.01zM12.7 10h-1.38L9 15.99h1.36l.48-1.33h2.3l.46 1.33H15L12.7 10zm-1.51 3.67l.8-2.2h.02l.77 2.2h-1.59zm3.8-7.17h-4.57l4.45-5.12V0H8.34v1.48h4.1L7.99 6.59v1.39h7V6.5z"],"sort-asc":["M8 7h3c.55 0 1-.45 1-1s-.45-1-1-1H8c-.55 0-1 .45-1 1s.45 1 1 1zm0-4h1c.55 0 1-.45 1-1s-.45-1-1-1H8c-.55 0-1 .45-1 1s.45 1 1 1zm0 8h5c.55 0 1-.45 1-1s-.45-1-1-1H8c-.55 0-1 .45-1 1s.45 1 1 1zm-3 1c-.28 0-.53.11-.71.29l-.29.3V9c0-.55-.45-1-1-1s-1 .45-1 1v3.59l-.29-.29A.965.965 0 001 12a1.003 1.003 0 00-.71 1.71l2 2c.18.18.43.29.71.29s.53-.11.71-.29l2-2A1.003 1.003 0 005 12zm10 1H8c-.55 0-1 .45-1 1s.45 1 1 1h7c.55 0 1-.45 1-1s-.45-1-1-1z"],"sort-desc":["M5 12c-.28 0-.53.11-.71.29l-.29.3V9c0-.55-.45-1-1-1s-1 .45-1 1v3.59l-.29-.29A.965.965 0 001 12a1.003 1.003 0 00-.71 1.71l2 2c.18.18.43.29.71.29s.53-.11.71-.29l2-2A1.003 1.003 0 005 12zm4 1H8c-.55 0-1 .45-1 1s.45 1 1 1h1c.55 0 1-.45 1-1s-.45-1-1-1zm4-8H8c-.55 0-1 .45-1 1s.45 1 1 1h5c.55 0 1-.45 1-1s-.45-1-1-1zm-2 4H8c-.55 0-1 .45-1 1s.45 1 1 1h3c.55 0 1-.45 1-1s-.45-1-1-1zm4-8H8c-.55 0-1 .45-1 1s.45 1 1 1h7c.55 0 1-.45 1-1s-.45-1-1-1z"],"sort-numerical":["M6 11.99c-.28 0-.53.11-.71.29l-.29.3V8.99c0-.55-.45-1-1-1s-1 .45-1 1v3.59l-.29-.29c-.18-.18-.43-.3-.71-.3a1.003 1.003 0 00-.71 1.71l2 2c.18.18.43.29.71.29.28 0 .53-.11.71-.29l2-2A1.003 1.003 0 006 11.99zm7.91-.08c-.06-.36-.17-.68-.33-.96-.16-.28-.37-.51-.64-.69-.27-.17-.61-.26-1.03-.26-.28 0-.54.06-.78.17-.23.11-.43.26-.6.45-.17.19-.3.41-.39.67a2.492 2.492 0 00-.04 1.52 1.623 1.623 0 00.89 1.03c.22.11.45.16.68.16.26 0 .5-.05.7-.15s.38-.26.53-.5l.02.02c-.01.16-.03.34-.07.54-.03.2-.09.4-.17.57-.08.18-.18.33-.31.45s-.29.19-.5.19a.63.63 0 01-.48-.21c-.13-.14-.21-.31-.25-.5H10.1c.03.25.1.48.19.68.1.2.22.37.38.5.16.14.33.24.54.31s.42.1.65.1c.39 0 .72-.09.99-.27.27-.18.49-.41.66-.7.17-.29.29-.61.37-.97.08-.36.12-.72.12-1.07 0-.36-.03-.72-.09-1.08zm-1.14.54c-.04.13-.09.24-.16.34a.78.78 0 01-.27.24c-.11.06-.24.09-.39.09a.75.75 0 01-.37-.09.777.777 0 01-.26-.25c-.07-.1-.12-.22-.15-.35-.03-.13-.05-.26-.05-.4 0-.13.02-.26.05-.39.04-.13.09-.24.16-.34.07-.1.16-.18.26-.24s.22-.09.35-.09c.14 0 .26.03.37.09.11.06.2.14.28.24a1.32 1.32 0 01.23.74c0 .15-.02.28-.05.41zm-1.56-4.47H13V0h-1.42c-.05.3-.16.56-.31.76-.16.21-.35.37-.58.5-.23.13-.49.21-.78.26-.3.05-.6.07-.91.06V2.8h2.21v5.18z"],"sort-numerical-desc":["M6 11.99c-.28 0-.53.11-.71.29l-.29.3V8.99c0-.55-.45-1-1-1s-1 .45-1 1v3.59l-.29-.29a.982.982 0 00-.71-.3 1.003 1.003 0 00-.71 1.71l2 2c.18.18.43.29.71.29s.53-.11.71-.29l2-2A1.003 1.003 0 006 11.99zm7.86-9.45c-.09-.48-.26-.9-.5-1.28S12.8.58 12.4.35C12 .12 11.49 0 10.86 0c-.43 0-.82.07-1.17.22s-.65.35-.9.6-.44.55-.58.89c-.14.34-.2.71-.2 1.11 0 .31.05.61.15.91.1.3.26.57.48.8.23.24.52.43.85.58.33.14.68.21 1.03.21.4 0 .75-.07 1.05-.2.3-.13.57-.35.79-.66l.02.02c-.02.21-.05.45-.1.73-.05.27-.13.53-.25.76-.12.24-.27.44-.47.6-.19.16-.44.25-.75.25a.98.98 0 01-.72-.29c-.19-.18-.31-.4-.37-.66H8.15c.05.34.14.64.29.9.15.26.34.49.57.67.23.18.5.32.8.41.31.1.63.15.98.15.58 0 1.08-.12 1.48-.36.4-.24.73-.55.99-.93.26-.39.44-.82.56-1.29.12-.48.18-.96.18-1.44s-.05-.96-.14-1.44zm-1.71.72c-.05.17-.14.32-.24.46-.11.13-.24.24-.41.31-.16.08-.36.12-.58.12-.21 0-.39-.04-.55-.13-.16-.08-.29-.19-.39-.33-.12-.14-.19-.29-.24-.46-.05-.17-.08-.35-.08-.54 0-.18.03-.35.08-.52.06-.16.14-.31.25-.44.11-.13.24-.24.4-.32.16-.08.33-.12.52-.12.21 0 .4.04.56.12.16.08.3.19.41.32.11.14.2.29.26.46.06.17.09.35.09.52 0 .2-.03.38-.08.55zm-.46 7.31c-.12.15-.26.28-.44.37-.17.09-.37.16-.58.2-.22.04-.44.05-.67.05v.92h1.65v3.88h1.33V10h-1.06c-.03.23-.11.42-.23.57z"],"split-columns":["M12 10a1.003 1.003 0 001.71.71l2-2c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71l-2-2a1.003 1.003 0 00-1.42 1.42l.3.29H9V2h3v1.71c.31-.13.64-.21 1-.21s.69.08 1 .21V1c0-.55-.45-1-1-1H3c-.55 0-1 .45-1 1v2.71c.31-.13.64-.21 1-.21s.69.08 1 .21V2h3v5H3.41l.29-.29c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71l-2 2C.11 7.47 0 7.72 0 8c0 .28.11.53.29.71l2 2a1.003 1.003 0 001.42-1.42L3.41 9H7v5H4v-1.71c-.31.13-.64.21-1 .21s-.69-.08-1-.21V15c0 .55.45 1 1 1h10c.55 0 1-.45 1-1v-2.71c-.31.13-.64.21-1 .21s-.69-.08-1-.21V14H9V9h3.59l-.29.29c-.19.18-.3.43-.3.71z"],square:["M15 0H1C.45 0 0 .45 0 1v14c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 14H2V2h12v12z"],"stacked-chart":["M10 2c0-.55-.45-1-1-1H8c-.55 0-1 .45-1 1v3h3V2zm3 10h1c.55 0 1-.45 1-1V8h-3v3c0 .55.45 1 1 1zm2-7c0-.55-.45-1-1-1h-1c-.55 0-1 .45-1 1v2h3V5zm-5 1H7v3h3V6zM5 7c0-.55-.45-1-1-1H3c-.55 0-1 .45-1 1v1h3V7zm3 5h1c.55 0 1-.45 1-1v-1H7v1c0 .55.45 1 1 1zm7 1H2c-.55 0-1 .45-1 1s.45 1 1 1h13c.55 0 1-.45 1-1s-.45-1-1-1zM3 12h1c.55 0 1-.45 1-1V9H2v2c0 .55.45 1 1 1z"],star:["M8 0l2.5 5.3 5.5.8-4 4.1.9 5.8L8 13.3 3.1 16l.9-5.8-4-4.1 5.5-.8z"],"star-empty":["M16 6.11l-5.53-.84L8 0 5.53 5.27 0 6.11l4 4.1L3.06 16 8 13.27 12.94 16 12 10.21l4-4.1zM4.91 13.2l.59-3.62L3 7.02l3.45-.53L8 3.2l1.55 3.29 3.45.53-2.5 2.56.59 3.62L8 11.49 4.91 13.2z"],"step-backward":["M12 3c-.24 0-.44.09-.62.23l-.01-.01L7 6.72V4c0-.55-.45-1-1-1H5c-.55 0-1 .45-1 1v8c0 .55.45 1 1 1h1c.55 0 1-.45 1-1V9.28l4.38 3.5.01-.01c.17.14.37.23.61.23.55 0 1-.45 1-1V4c0-.55-.45-1-1-1z"],"step-chart":["M15 12H2v-2h3c.55 0 1-.45 1-1V7h2v1c0 .55.45 1 1 1h4c.55 0 1-.45 1-1V5h1c.55 0 1-.45 1-1s-.45-1-1-1h-2c-.55 0-1 .45-1 1v3h-2V6c0-.55-.45-1-1-1H5c-.55 0-1 .45-1 1v2H2V3c0-.55-.45-1-1-1s-1 .45-1 1v10c0 .55.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1z"],"step-forward":["M12 3h-1c-.55 0-1 .45-1 1v2.72l-4.38-3.5v.01A.987.987 0 005 3c-.55 0-1 .45-1 1v8c0 .55.45 1 1 1 .24 0 .44-.09.62-.23l.01.01L10 9.28V12c0 .55.45 1 1 1h1c.55 0 1-.45 1-1V4c0-.55-.45-1-1-1z"],stop:["M12 3H4c-.55 0-1 .45-1 1v8c0 .55.45 1 1 1h8c.55 0 1-.45 1-1V4c0-.55-.45-1-1-1z"],stopwatch:["M9 2v1.083A6.002 6.002 0 018 15 6 6 0 017 3.083V2H6a1 1 0 110-2h4a1 1 0 010 2H9zM8 5a4 4 0 104 4H8V5z"],strikethrough:["M14 7H8.65c-.38-.09-.73-.18-1.04-.26-.31-.08-.49-.13-.54-.14-.43-.11-.79-.29-1.05-.52-.27-.23-.4-.55-.4-.95 0-.29.07-.53.21-.72s.32-.34.54-.46c.22-.11.46-.19.72-.24.26-.05.52-.07.77-.07.74 0 1.36.15 1.84.46.32.2.55.5.68.9h2.22c-.06-.33-.17-.64-.32-.92-.25-.45-.59-.84-1.02-1.15-.43-.31-.93-.54-1.49-.7S8.59 2 7.95 2c-.55 0-1.1.07-1.63.2-.54.13-1.02.34-1.45.62-.42.28-.76.63-1.02 1.05-.26.42-.39.92-.39 1.5 0 .3.04.59.13.88.08.26.21.51.39.75H2c-.55 0-1 .45-1 1s.45 1 1 1h7.13c.25.07.49.14.71.22.25.09.48.23.7.44.21.21.32.53.32.97 0 .21-.05.43-.14.63-.09.21-.24.39-.45.55-.21.16-.48.29-.81.39-.33.1-.73.15-1.2.15-.44 0-.84-.05-1.21-.14-.37-.09-.7-.24-.99-.43-.29-.2-.51-.45-.67-.76-.01 0-.01-.01-.02-.02H3.14a3.68 3.68 0 001.39 2.03c.46.34 1 .58 1.62.74.61.15 1.27.23 1.97.23.61 0 1.2-.07 1.79-.2.58-.13 1.11-.34 1.56-.63.46-.29.83-.66 1.11-1.11.28-.45.42-1 .42-1.64 0-.3-.05-.6-.15-.9-.05-.19-.13-.36-.22-.52H14c.55 0 1-.45 1-1s-.45-1-1-1z"],style:["M14 14H2V2h8.76l2-2H1C.45 0 0 .45 0 1v14c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V6.24l-2 2V14zm1.4-14L9.7 5.7l2.1 2.1L16 3.6V0h-.6zM4 11.92c2.33.15 4.42.15 6.15-1.5.82-.83.82-2.25 0-3.08-.45-.38-.98-.6-1.5-.6-.53 0-1.05.22-1.43.6-.82.91-1.27 3.38-3.22 4.58z"],"swap-horizontal":["M0 7.02L.05 7H0v.02zm2-2.03h9.57l-1.29 1.29A1.003 1.003 0 0011.7 7.7l2.99-3c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71l-2.99-3a1.07 1.07 0 00-.71-.28 1.003 1.003 0 00-.71 1.71L11.57 3H2c-.55 0-1 .45-1 1a1 1 0 001 .99zM15.96 9H16v-.02l-.04.02zM14 11.01H4.43l1.29-1.29A1.003 1.003 0 004.3 8.3l-2.99 3a.99.99 0 00-.29.7c0 .28.11.53.29.71l2.99 3a1.003 1.003 0 001.42-1.42L4.43 13H14c.55 0 1-.45 1-1s-.45-.99-1-.99z"],"swap-vertical":["M9 0h-.02L9 .04V0zM7 16h.02L7 15.95V16zM4.7 1.31c-.18-.18-.43-.29-.7-.29s-.53.11-.71.29l-3 2.99a1.003 1.003 0 001.42 1.42L3 4.43V14c0 .55.45 1 1 1s1-.45 1-1V4.43l1.29 1.29c.18.18.43.29.7.29A1.003 1.003 0 007.7 4.3l-3-2.99zM15 9.99c-.28 0-.53.11-.71.29L13 11.57V2c0-.55-.45-1-1-1s-1 .45-1 1v9.57l-1.29-1.29a.99.99 0 00-.7-.29 1.003 1.003 0 00-.71 1.71l3 2.99c.18.18.43.29.71.29.28 0 .53-.11.71-.29l3-2.99c.18-.18.29-.43.29-.71-.01-.55-.46-1-1.01-1z"],switch:["M9.293 2.293l1.414 1.414-4.999 5a3 3 0 11-1.415-1.415l5-5zM13 7a3 3 0 110 6 3 3 0 010-6zM3 9a1 1 0 100 2 1 1 0 000-2zm10 0a1 1 0 100 2 1 1 0 000-2z"],"symbol-circle":["M8 3.01a5 5 0 100 10 5 5 0 100-10z"],"symbol-cross":["M12 6.01h-2v-2c0-.55-.45-1-1-1H7c-.55 0-1 .45-1 1v2H4c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h2v2c0 .55.45 1 1 1h2c.55 0 1-.45 1-1v-2h2c.55 0 1-.45 1-1v-2c0-.56-.45-1-1-1z"],"symbol-diamond":["M12 8.01c0-.19-.07-.36-.16-.51l.01-.01-3-5-.01.01c-.17-.29-.48-.49-.84-.49s-.67.2-.84.49l-.02-.01-3 5 .02.01c-.09.15-.16.32-.16.51s.07.36.16.51h-.02l3 5 .01-.01c.18.29.49.5.85.5s.67-.2.84-.49l.01.01 3-5-.01-.01c.09-.16.16-.32.16-.51z"],"symbol-square":["M12 3.01H4c-.55 0-1 .45-1 1v8c0 .55.45 1 1 1h8c.55 0 1-.45 1-1v-8c0-.56-.45-1-1-1z"],"symbol-triangle-down":["M13 4.01c0-.55-.45-1-1-1H4c-.55 0-1 .45-1 1 0 .16.05.31.11.44H3.1l4 8h.01c.16.33.49.56.89.56s.72-.23.89-.56h.01l4-8h-.01c.06-.14.11-.28.11-.44z"],"symbol-triangle-up":["M12.89 11.56l-3.99-8h-.01c-.17-.32-.5-.55-.89-.55s-.72.23-.89.55H7.1l-4 8h.01c-.06.14-.11.29-.11.45 0 .55.45 1 1 1h8c.55 0 1-.45 1-1 0-.16-.05-.31-.11-.45z"],tag:["M1 3a2 2 0 012-2h4.584a2 2 0 011.414.586l5.413 5.412a2 2 0 010 2.829L9.827 14.41a2 2 0 01-2.829 0L1.586 8.998A2 2 0 011 7.584V3zm3.487-.007a1.494 1.494 0 100 2.988 1.494 1.494 0 000-2.988z"],"take-action":["M9 11a1.003 1.003 0 001.71.71l4-4a1.003 1.003 0 00-1.42-1.42l-4 4c-.18.18-.29.43-.29.71zM4 6c.28 0 .53-.11.71-.29l4-4A1.003 1.003 0 007.29.29l-4 4A1.003 1.003 0 004 6zm4 4l5-5-.79-.79.5-.5a1.003 1.003 0 00-1.42-1.42l-.5.5L10 2 5 7l.79.79-5.5 5.5a1.003 1.003 0 001.42 1.42l5.5-5.5L8 10zm7 4H7c-.55 0-1 .45-1 1s.45 1 1 1h8c.55 0 1-.45 1-1s-.45-1-1-1z"],taxi:["M15.12 6.63h-.38L15 7c-.01.3-.01.64 0 .98V8c0 .07-.03.13-.04.19h.02L14 13.1v.9c0 .55-.45 1-1 1s-1-.45-1-1v-1H4v1c0 .55-.45 1-1 1s-1-.45-1-1v-.9l-.98-4.9h.02C1.03 8.13 1 8.07 1 8H.99c0-.33 0-.67.01-1l.26-.37H.88C.4 6.63 0 6.21 0 5.69s.4-.94.88-.94h1.05l.77-2.11c.19-.53.76-1.08 1.26-1.24 0 0 .68-.2 2.05-.32C6.01 1.05 6 1.03 6 1c0-.55.45-1 1-1h2c.55 0 1 .45 1 1 0 .03-.01.05-.02.08 1.37.12 2.05.32 2.05.32.51.15 1.08.71 1.27 1.24l.76 2.12h1.05c.49 0 .89.42.89.93 0 .52-.4.94-.88.94zM11 10h2V8h-2v2zm-8 0h2V8H3v2zm10-5l-.73-1.63C12.21 3.19 12.18 3 12 3H4c-.18 0-.21.19-.27.37L3 5c-.06.18-.18 1 0 1h10c.18 0 .06-.82 0-1z"],"text-highlight":["M9 10H2V6h7V4H1c-.55 0-1 .45-1 1v6c0 .55.45 1 1 1h8v-2zm4 3h-1V3h1c.55 0 1-.45 1-1s-.45-1-1-1h-1c-.37 0-.7.11-1 .28-.3-.17-.63-.28-1-.28H9c-.55 0-1 .45-1 1s.45 1 1 1h1v10H9c-.55 0-1 .45-1 1s.45 1 1 1h1c.37 0 .7-.11 1-.28.3.17.63.28 1 .28h1c.55 0 1-.45 1-1s-.45-1-1-1zm2-9h-2v2h1v4h-1v2h2c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1z"],th:["M15 1H1c-.6 0-1 .5-1 1v12c0 .6.4 1 1 1h14c.6 0 1-.4 1-1V2c0-.5-.4-1-1-1zM6 13H2v-2h4v2zm0-3H2V8h4v2zm0-3H2V5h4v2zm8 6H7v-2h7v2zm0-3H7V8h7v2zm0-3H7V5h7v2z"],"th-derived":["M5.6 10l-.3.3c-.2.2-.3.4-.3.7 0 .6.4 1 1 1 .3 0 .5-.1.7-.3l2-2c.2-.2.3-.4.3-.7s-.1-.5-.3-.7l-2-2C6.5 6.1 6.3 6 6 6c-.5 0-1 .4-1 1 0 .3.1.5.3.7l.3.3H1c-.6 0-1 .4-1 1s.4 1 1 1h4.6zM15 1H2c-.5 0-1 .5-1 1v5h2V5h11v2H8.8l.6.6c.1.1.2.3.3.4H14v2H9.7c-.1.1-.2.3-.3.4l-.6.6H14v2H3v-2H1v3c0 .5.5 1 1 1h13c.6 0 1-.5 1-1V2c0-.5-.4-1-1-1z"],"th-disconnect":["M12 1h3c.6 0 1 .5 1 1v12c0 .6-.4 1-1 1h-4.97l.286-2H14v-2h-3.398l.143-1H14V8h-2.97l.143-1H14V5h-2.541l.51-3.576C11.99 1.282 12 1.14 12 1zM5.97 1l-.572 4H2v2h3.112L4.97 8H2v2h2.684l-.143 1H2v2h2.255l-.225 1.576c-.02.142-.03.284-.03.424H1c-.6 0-1-.4-1-1V2c0-.5.4-1 1-1h4.97zM8.01.859a1 1 0 111.98.282l-2 14a1 1 0 11-1.98-.282l2-14z"],"th-filtered":["M10 10h3l1.78-2.226a1 1 0 00.22-.625V4.3l1-.9V14c0 .6-.4 1-1 1H1c-.6 0-1-.4-1-1V2c0-.5.4-1 1-1h4.333L9 4.3V5H7v2h2v1H7v2h3zm-4 3v-2H2v2h4zm0-3V8H2v2h4zm0-3V5H2v2h4zm8 6v-2H7v2h7z","M15.48 0c.31 0 .52.26.52.57 0 .16-.06.3-.17.41l-2.86 2.73v2.63c0 .16-.06.3-.17.41l-.82 1.1c-.1.1-.25.17-.41.17-.31 0-.57-.26-.57-.57V3.71L8.17.98A.566.566 0 018 .57c0-.31.26-.57.57-.57h6.91z"],"th-list":["M15 1H1c-.6 0-1 .5-1 1v12c0 .6.4 1 1 1h14c.6 0 1-.4 1-1V2c0-.5-.4-1-1-1zm-1 12H2v-2h12v2zm0-3H2V8h12v2zm0-3H2V5h12v2z"],"thumbs-down":["M2 2H0v7h2c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1zm13.99 4.38c.08-.58-.44-1.02-1.15-1.05-.25-.01-.52-.03-.81-.05.02 0 .05-.01.07-.01.7-.1 1.34-.49 1.41-1.07.06-.58-.46-.97-1.17-1.04-.25-.02-.52-.04-.79-.06.47-.15.84-.42.87-.93.04-.58-.79-1.03-1.5-1.09-.27-.02-.51-.04-.73-.05h-.09c-.23-.02-.43-.02-.62-.03C8.35.95 5.66 1.47 4 2.51v6c2.14 1.29 4.76 3.59 4.21 5.51-.18.59.31 1.05.98.98.81-.09 1.37-.91 1.4-1.78.04-1-.15-2.01-.5-2.91-.04-.25.01-.5.37-.53.49-.03 1.11-.06 1.59-.08.26 0 .51-.01.75-.02h.01c.41-.02.8-.05 1.13-.09.7-.09 1.35-.47 1.43-1.05.08-.58-.44-.97-1.15-1.05-.05-.01-.11-.01-.16-.02.17-.01.33-.03.49-.05.72-.08 1.37-.46 1.44-1.04z"],"thumbs-up":["M15.99 9.62c-.08-.58-.73-.96-1.43-1.05-.15-.02-.32-.04-.49-.05.06-.01.11-.01.16-.02.71-.08 1.23-.47 1.15-1.05-.08-.58-.73-.96-1.43-1.05-.34-.04-.72-.07-1.13-.09h-.01c-.24-.01-.49-.02-.75-.02-.48-.02-1.11-.04-1.59-.08-.36-.03-.41-.28-.37-.53.35-.9.54-1.91.5-2.91-.04-.85-.6-1.68-1.41-1.77-.67-.07-1.16.39-.99.98C8.76 3.91 6.13 6.2 4 7.49v6c1.66 1.03 4.35 1.56 7.48 1.5.19 0 .39-.01.62-.02h.09c.22-.01.46-.03.73-.05.71-.06 1.54-.51 1.5-1.09-.03-.51-.4-.79-.87-.93.27-.02.54-.04.79-.06.71-.06 1.24-.45 1.17-1.04-.06-.58-.7-.97-1.41-1.07-.02 0-.05-.01-.07-.01.29-.02.57-.03.81-.05.71-.03 1.23-.47 1.15-1.05zM2 7H0v7h2c.55 0 1-.45 1-1V8c0-.56-.45-1-1-1z"],tick:["M14 3c-.28 0-.53.11-.71.29L6 10.59l-3.29-3.3a1.003 1.003 0 00-1.42 1.42l4 4c.18.18.43.29.71.29s.53-.11.71-.29l8-8A1.003 1.003 0 0014 3z"],"tick-circle":["M8 16c-4.42 0-8-3.58-8-8s3.58-8 8-8 8 3.58 8 8-3.58 8-8 8zm4-11c-.28 0-.53.11-.71.29L7 9.59l-2.29-2.3a1.003 1.003 0 00-1.42 1.42l3 3c.18.18.43.29.71.29s.53-.11.71-.29l5-5A1.003 1.003 0 0012 5z"],time:["M8 0C3.58 0 0 3.58 0 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm0 14c-3.31 0-6-2.69-6-6s2.69-6 6-6 6 2.69 6 6-2.69 6-6 6zm1-6.41V4c0-.55-.45-1-1-1s-1 .45-1 1v4c0 .28.11.53.29.71l2 2a1.003 1.003 0 001.42-1.42L9 7.59z"],"timeline-area-chart":["M15 2.59L9.91 7.68 6.6 5.2l-.01.01C6.42 5.09 6.23 5 6 5c-.24 0-.44.09-.62.23v-.01L3 7.12V11h12V2.59zM15 12H2V3c0-.55-.45-1-1-1s-1 .45-1 1v10c0 .55.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1z"],"timeline-bar-chart":["M8 12h1c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1H8c-.55 0-1 .45-1 1v9c0 .55.45 1 1 1zm5 0h1c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1h-1c-.55 0-1 .45-1 1v6c0 .55.45 1 1 1zm2 1H2c-.55 0-1 .45-1 1s.45 1 1 1h13c.55 0 1-.45 1-1s-.45-1-1-1zM3 12h1c.55 0 1-.45 1-1V7c0-.55-.45-1-1-1H3c-.55 0-1 .45-1 1v4c0 .55.45 1 1 1z"],"timeline-events":["M8 11H7v1h1v-1zm-4 0H3v1h1v-1zm7-8c.6 0 1-.5 1-1V1c0-.5-.4-1-1-1s-1 .5-1 1v1c0 .5.5 1 1 1zM4 3c.5 0 1-.5 1-1V1c0-.5-.5-1-1-1S3 .5 3 1v1c0 .5.5 1 1 1zm10-2h-1v1c0 1.1-.9 2-2 2s-2-.9-2-2V1H6v1c0 1.1-.9 2-2 2s-2-.9-2-2V1H1c-.5 0-1 .5-1 1v12c0 .5.5 1 1 1h13c.6 0 1-.5 1-1V2c0-.5-.4-1-1-1zM5 13H2v-3h3v3zm0-4H2V6h3v3zm4 4H6v-3h3v3zm0-4H6V6h3v3zm4 4h-3v-3h3v3zm0-4h-3V6h3v3zm-1-2h-1v1h1V7z"],"timeline-line-chart":["M15 12H2V9.41l3-3L8.29 9.7c.18.19.43.3.71.3s.53-.11.71-.29l6-6a1.003 1.003 0 00-1.42-1.42L9 7.59l-3.29-3.3C5.53 4.11 5.28 4 5 4s-.53.11-.71.29L2 6.59V3c0-.55-.45-1-1-1s-1 .45-1 1v10c0 .55.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1z"],tint:["M7.88 1s-4.9 6.28-4.9 8.9c.01 2.82 2.34 5.1 4.99 5.1 2.65-.01 5.03-2.3 5.03-5.13C12.99 7.17 7.88 1 7.88 1z"],torch:["M5 15c0 .55.45 1 1 1h4c.55 0 1-.45 1-1v-1H5v1zm7-15H4c-.55 0-1 .45-1 1v1h10V1c0-.55-.45-1-1-1zM5 7v6h6V7l2-4H3l2 4zm2 0c0-.55.45-1 1-1s1 .45 1 1v2c0 .55-.45 1-1 1s-1-.45-1-1V7z"],tractor:["M3.5 9a3.5 3.5 0 110 7 3.5 3.5 0 010-7zm9.5 1a3 3 0 110 6 3 3 0 010-6zm-9.5 1a1.5 1.5 0 100 3 1.5 1.5 0 000-3zm9.5 1a1 1 0 100 2 1 1 0 000-2zM5 0c1.46 0 2.527.668 3 2l.815 3.255a78.9 78.9 0 012.186.195L11 2h2l.001 3.688c.698.095 1.37.198 2.013.312.623.11.986.479.986 1v3.354a4.001 4.001 0 00-6.873 1.645H7.999l-.026-.002A4.5 4.5 0 00.659 9.01l-.654.001v-.829C.003 7.386.002 6.423 0 6.022 0 5.5.376 4.99 1 4.99V1a1 1 0 011-1h3zm1 2H3v2.99c1.29.024 2.554.069 3.781.135L6 2z"],train:["M13 14h-1l1 2H3l1-2H3c-1.1 0-2-.9-2-2V2C1 .9 4.13 0 8 0s7 .9 7 2v10c0 1.1-.9 2-2 2zm-2-2h2v-2h-2v2zM9 7h4V3H9v4zm-6 5h2v-2H3v2zm0-5h4V3H3v4z"],translate:["M15.89 14.56l-3.99-8h-.01c-.17-.33-.5-.56-.89-.56s-.72.23-.89.56h-.01L9 8.76 7.17 7.38l.23-.18C8.37 6.47 9 5.31 9 4V3h1c.55 0 1-.45 1-1s-.45-1-1-1H7c0-.55-.45-1-1-1H5c-.55 0-1 .45-1 1H1c-.55 0-1 .45-1 1s.45 1 1 1h6v1c0 .66-.32 1.25-.82 1.61l-.68.51-.68-.5C4.32 5.25 4 4.66 4 4H2c0 1.31.63 2.47 1.6 3.2l.23.17L1.4 9.2l.01.01C1.17 9.4 1 9.67 1 10c0 .55.45 1 1 1 .23 0 .42-.09.59-.21l.01.01 2.9-2.17 2.6 1.95-1.99 3.98h.01c-.07.13-.12.28-.12.44 0 .55.45 1 1 1 .39 0 .72-.23.89-.56h.01L8.62 14h4.76l.72 1.45h.01c.17.32.5.55.89.55.55 0 1-.45 1-1 0-.16-.05-.31-.11-.44zM9.62 12L11 9.24 12.38 12H9.62z"],trash:["M14.49 3.99h-13c-.28 0-.5.22-.5.5s.22.5.5.5h.5v10c0 .55.45 1 1 1h10c.55 0 1-.45 1-1v-10h.5c.28 0 .5-.22.5-.5s-.22-.5-.5-.5zm-8.5 9c0 .55-.45 1-1 1s-1-.45-1-1v-6c0-.55.45-1 1-1s1 .45 1 1v6zm3 0c0 .55-.45 1-1 1s-1-.45-1-1v-6c0-.55.45-1 1-1s1 .45 1 1v6zm3 0c0 .55-.45 1-1 1s-1-.45-1-1v-6c0-.55.45-1 1-1s1 .45 1 1v6zm2-12h-4c0-.55-.45-1-1-1h-2c-.55 0-1 .45-1 1h-4c-.55 0-1 .45-1 1v1h14v-1c0-.55-.45-1-1-1z"],tree:["M9 11.857V16H7v-4.143L1 13l3.885-4.44L3 9l3.07-4.297L5 5l3-5 3 5-1.07-.297L13 9l-1.885-.44L15 13l-6-1.143z"],"trending-down":["M15 7c-.55 0-1 .45-1 1v.59l-4.29-4.3A.997.997 0 009 4c-.16 0-.31.05-.44.11V4.1L5 5.88 1.45 4.11v.01C1.31 4.05 1.16 4 1 4c-.55 0-1 .45-1 1 0 .39.23.72.56.89v.01l4 2v-.01c.13.06.28.11.44.11s.31-.05.44-.11v.01L8.8 6.22 12.59 10H12c-.55 0-1 .45-1 1s.45 1 1 1h3c.55 0 1-.45 1-1V8c0-.55-.45-1-1-1z"],"trending-up":["M15 4h-3c-.55 0-1 .45-1 1s.45 1 1 1h.59L8.8 9.78 5.45 8.11v.01C5.31 8.05 5.16 8 5 8s-.31.05-.44.11V8.1l-4 2v.01c-.33.17-.56.5-.56.89 0 .55.45 1 1 1 .16 0 .31-.05.44-.11v.01L5 10.12l3.55 1.78v-.01c.14.06.29.11.45.11.28 0 .53-.11.71-.29L14 7.41V8c0 .55.45 1 1 1s1-.45 1-1V5c0-.55-.45-1-1-1z"],truck:["M12.5 0a.5.5 0 01.5.5V9a1 1 0 011 1v2h.5a.5.5 0 01.5.5v1a.5.5 0 01-.5.5H13v1a1 1 0 01-2 0v-1H5v1a1 1 0 01-2 0v-1H1.5a.5.5 0 01-.5-.5v-1a.5.5 0 01.5-.5H2v-2a1 1 0 011-1V.5a.5.5 0 011 0V3a2 2 0 012-2h4a2 2 0 012 2V.5a.5.5 0 01.5-.5zM9 8H7a1 1 0 00-1 1v2a1 1 0 001 1h2a1 1 0 001-1V9a1 1 0 00-1-1zm3.5 3h-1a.5.5 0 100 1h1a.5.5 0 100-1zm-8 0h-1a.5.5 0 100 1h1a.5.5 0 100-1zM9 9a.5.5 0 01.5.5v1l-.008.09A.5.5 0 019 11H7l-.09-.008a.5.5 0 01-.41-.492v-1l.008-.09A.5.5 0 017 9zm2-5H5v2h6V4z"],"two-columns":["M3.99-.01h-3c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1h3c.55 0 1-.45 1-1v-14c0-.55-.45-1-1-1zm11.71 7.3l-2-2a1.003 1.003 0 00-1.71.71v4a1.003 1.003 0 001.71.71l2-2c.18-.18.29-.43.29-.71s-.11-.53-.29-.71zM9.99-.01h-3c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1h3c.55 0 1-.45 1-1v-14c0-.55-.45-1-1-1z"],unarchive:["M13.382 0a1 1 0 01.894.553L16 4v11a1 1 0 01-1 1H1a1 1 0 01-1-1V4L1.724.553A1 1 0 012.618 0h10.764zM8 6c-.28 0-.53.11-.71.29l-2 2-.084.096A1.003 1.003 0 006.71 9.71l.29-.3V12l.007.116c.058.496.482.884.993.884.55 0 1-.45 1-1V9.41l.29.29.081.076A.97.97 0 0010 10a1.003 1.003 0 00.71-1.71l-2-2-.096-.084A1.002 1.002 0 008 6zm5-4H3L2 4h12l-1-2z"],underline:["M8 14c2.8 0 5-2.2 5-5V3c0-.6-.4-1-1-1s-1 .4-1 1v6c0 1.7-1.3 3-3 3s-3-1.3-3-3V3c0-.6-.4-1-1-1s-1 .4-1 1v6c0 2.8 2.2 5 5 5zM13.5 15h-11c-.3 0-.5.2-.5.5s.2.5.5.5h11c.3 0 .5-.2.5-.5s-.2-.5-.5-.5z"],undo:["M4 11c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm7-7H3.41L4.7 2.71c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71l-3 3C.11 4.47 0 4.72 0 5c0 .28.11.53.29.71l3 3a1.003 1.003 0 001.42-1.42L3.41 6H11c1.66 0 3 1.34 3 3s-1.34 3-3 3H7v2h4c2.76 0 5-2.24 5-5s-2.24-5-5-5z"],"ungroup-objects":["M3.5 5C1.57 5 0 6.57 0 8.5S1.57 12 3.5 12 7 10.43 7 8.5 5.43 5 3.5 5zm9 0C10.57 5 9 6.57 9 8.5s1.57 3.5 3.5 3.5S16 10.43 16 8.5 14.43 5 12.5 5z"],"unknown-vehicle":["M10.507 9.75v-3.5c0-.089.023-.171.051-.25h-7.55c-.176 0-.061-.824 0-1l.729-1.63c.06-.177.095-.37.27-.37h4.5V1.01c-.166-.003-.32-.01-.5-.01-2.72 0-4.036.402-4.036.402-.508.155-1.079.711-1.268 1.237L1.94 4.756H.887c-.483 0-.88.423-.88.939s.397.939.88.939h.376L1.008 7c-.034.685 0 1.436 0 2v5c0 .657.384 1 1 1s1-.343 1-1v-1h10v1c0 .657.383 1 1 1s1-.343 1-1v-3.5h-3.75a.75.75 0 01-.75-.75zm-5.5.25h-2V8h2v2zm11-4.305zM15.34.826a2.807 2.807 0 00-.932-.598c-.386-.16-.868-.241-1.445-.241-.447 0-.851.076-1.213.228-.362.153-.67.364-.926.636s-.456.592-.598.963a3.535 3.535 0 00-.218 1.144V3h1.789c.003-.208.023-.405.069-.588.049-.193.124-.362.225-.506.102-.144.232-.259.39-.345.159-.087.348-.13.567-.13.325 0 .58.09.762.272.183.18.275.46.275.839.008.222-.031.407-.116.555a1.654 1.654 0 01-.335.408 7.4 7.4 0 01-.452.37c-.162.123-.316.27-.463.438a2.556 2.556 0 00-.384.611c-.11.239-.177.535-.2.889V6h1.645v-.1c.032-.248.111-.453.237-.618.126-.164.27-.31.433-.438.163-.128.335-.255.518-.383a2.413 2.413 0 00.878-1.117c.102-.255.152-.58.152-.975A2.241 2.241 0 0015.34.826zM12.007 7v2h2V7h-2z"],unlock:["M11.99-.01c-2.21 0-4 1.79-4 4v3h-7c-.55 0-1 .45-1 1v7c0 .55.45 1 1 1h12c.55 0 1-.45 1-1v-7c0-.55-.45-1-1-1h-3v-3c0-1.1.9-2 2-2s2 .9 2 2v1c0 .55.45 1 1 1s1-.45 1-1v-1c0-2.21-1.79-4-4-4z"],unpin:["M9.39 1c-.5.5-.4 1.48.15 2.53L4.38 7.54C2.85 6.5 1.52 6.07 1 6.59l3.5 3.5c-.02.02-1.4 2.8-1.4 2.8l2.8-1.4 3.5 3.5c.53-.53.1-1.86-.95-3.38l4.02-5.16c1.04.55 2.01.65 2.51.14L9.39 1z"],unresolve:["M11 3c-.55 0-1.07.09-1.57.26a6.46 6.46 0 010 9.48c.5.17 1.02.26 1.57.26 2.76 0 5-2.24 5-5s-2.24-5-5-5zM9.78 9.38l.09-.27c.08-.36.13-.73.13-1.11s-.05-.75-.13-1.11l-.09-.27a5.32 5.32 0 00-.29-.79l-.12-.21c-.14-.27-.31-.52-.51-.76a.7.7 0 00-.08-.1c-.24-.27-.49-.52-.78-.74-.43-.32-.92-.58-1.45-.75l.01-.01c-.1-.03-.2-.05-.3-.08-.12-.03-.23-.07-.36-.09A5.28 5.28 0 005 3C2.24 3 0 5.24 0 8s2.24 5 5 5c.31 0 .61-.04.9-.09.12-.02.24-.06.36-.09.1-.03.21-.04.3-.08l-.01-.01c.88-.29 1.64-.8 2.22-1.49.03-.03.06-.07.09-.1.19-.24.36-.49.51-.76.04-.07.08-.14.11-.21.13-.25.23-.52.3-.79z"],updated:["M8 0a7.95 7.95 0 00-6 2.74V1c0-.55-.45-1-1-1S0 .45 0 1v4c0 .55.45 1 1 1h4c.55 0 1-.45 1-1s-.45-1-1-1H3.54C4.64 2.78 6.22 2 8 2c3.31 0 6 2.69 6 6 0 2.61-1.67 4.81-4 5.63-.63.22-1.29.37-2 .37-3.31 0-6-2.69-6-6 0-.55-.45-1-1-1s-1 .45-1 1c0 4.42 3.58 8 8 8 .34 0 .67-.03 1-.07.02 0 .04-.01.06-.01C12.98 15.4 16 12.06 16 8c0-4.42-3.58-8-8-8zm3 5c-.28 0-.53.11-.71.29L7 8.58 5.71 7.29a1.003 1.003 0 00-1.42 1.42l2 2c.18.18.43.29.71.29.28 0 .53-.11.71-.29l4-4A1.003 1.003 0 0011 5z"],upload:["M8 0C3.58 0 0 3.58 0 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm3 8c-.28 0-.53-.11-.71-.29L9 6.41V12c0 .55-.45 1-1 1s-1-.45-1-1V6.41l-1.29 1.3a1.003 1.003 0 01-1.42-1.42l3-3C7.47 3.11 7.72 3 8 3s.53.11.71.29l3 3A1.003 1.003 0 0111 8z"],user:["M7.99-.01A7.998 7.998 0 00.03 8.77c.01.09.03.18.04.28.02.15.04.31.07.47.02.11.05.22.08.34.03.13.06.26.1.38.04.12.08.25.12.37.04.11.08.21.12.32a6.583 6.583 0 00.3.65c.07.14.14.27.22.4.04.07.08.13.12.2l.27.42.1.13a7.973 7.973 0 003.83 2.82c.03.01.05.02.07.03.37.12.75.22 1.14.29l.2.03c.39.06.79.1 1.2.1s.81-.04 1.2-.1l.2-.03c.39-.07.77-.16 1.14-.29.03-.01.05-.02.07-.03a8.037 8.037 0 003.83-2.82c.03-.04.06-.08.09-.13.1-.14.19-.28.28-.42.04-.07.08-.13.12-.2.08-.13.15-.27.22-.41.04-.08.08-.17.12-.26.06-.13.11-.26.17-.39.04-.1.08-.21.12-.32.04-.12.08-.24.12-.37.04-.13.07-.25.1-.38.03-.11.06-.22.08-.34.03-.16.05-.31.07-.47.01-.09.03-.18.04-.28.02-.26.04-.51.04-.78-.03-4.41-3.61-7.99-8.03-7.99zm0 14.4c-1.98 0-3.75-.9-4.92-2.31.67-.36 1.49-.66 2.14-.95 1.16-.52 1.04-.84 1.08-1.27.01-.06.01-.11.01-.17-.41-.36-.74-.86-.96-1.44v-.01c0-.01-.01-.02-.01-.02-.05-.13-.09-.26-.12-.39-.28-.05-.44-.35-.5-.63-.06-.11-.18-.38-.15-.69.04-.41.2-.59.38-.67v-.06c0-.51.05-1.24.14-1.72.02-.13.05-.26.09-.39.17-.59.53-1.12 1.01-1.49.49-.38 1.19-.59 1.82-.59.62 0 1.32.2 1.82.59.48.37.84.9 1.01 1.49.04.13.07.26.09.4.09.48.14 1.21.14 1.72v.07c.18.08.33.26.37.66.03.31-.1.58-.16.69-.06.27-.21.57-.48.62-.03.13-.07.26-.12.38 0 .01-.01.04-.01.04-.21.57-.54 1.06-.94 1.42 0 .06.01.13.01.19.04.43-.12.75 1.05 1.27.65.29 1.47.6 2.14.95a6.415 6.415 0 01-4.93 2.31z"],variable:["M3.94 3.15c.47-.66 1.05-1.24 1.76-1.73l.13-.4c-1.11.45-2.05 1.01-2.84 1.7-1.02.88-1.8 1.9-2.32 3.05C.22 6.76 0 7.75 0 8.75c0 1.75.66 3.5 1.99 5.25l.13-.42c-.39-.94-.59-1.82-.59-2.63 0-1.28.22-2.64.67-4.1.45-1.45 1.03-2.69 1.74-3.7zm7.51 6.41l-.27-.15c-.3.41-.52.66-.66.77-.09.06-.21.1-.33.1-.15 0-.3-.1-.45-.28-.25-.33-.59-1.22-1.01-2.69.38-.65.69-1.08.95-1.28.19-.15.39-.22.59-.22.08 0 .22.03.43.08.2.06.39.08.54.08.22 0 .4-.07.54-.22.15-.15.22-.34.22-.57 0-.25-.07-.45-.22-.59-.15-.15-.35-.22-.63-.22-.24 0-.47.06-.69.17-.21.11-.49.36-.82.74-.25.28-.61.78-1.1 1.48a6.72 6.72 0 00-.97-2.38l-2.59.44-.05.27c.19-.04.36-.06.49-.06.26 0 .47.11.64.33.26.34.63 1.38 1.11 3.12-.37.49-.63.81-.77.96-.23.24-.41.4-.56.47-.11.06-.24.09-.39.09-.11 0-.29-.06-.53-.18-.17-.07-.32-.11-.45-.11-.25 0-.46.08-.62.24-.16.16-.24.37-.24.61 0 .23.08.42.23.57.15.15.35.22.61.22.25 0 .48-.05.7-.15.22-.1.49-.32.82-.65.33-.33.78-.86 1.36-1.59.22.69.42 1.19.58 1.51.16.31.35.54.56.68.21.14.47.21.79.21.31 0 .62-.11.93-.33.4-.29.82-.77 1.26-1.47zm2.56-8.54l-.12.42c.39.95.59 1.82.59 2.64 0 1.09-.17 2.26-.5 3.51-.26.96-.6 1.87-1.02 2.71-.42.85-.82 1.51-1.21 1.98-.39.48-.87.92-1.44 1.32l-.14.4c1.11-.45 2.05-1.02 2.84-1.7 1.03-.89 1.81-1.91 2.33-3.05.44-.99.66-1.99.66-3 0-1.73-.66-3.48-1.99-5.23z"],"vertical-bar-chart-asc":["M6 7c-.55 0-1 .45-1 1v7c0 .55.45 1 1 1s1-.45 1-1V8c0-.55-.45-1-1-1zM2 9c-.55 0-1 .45-1 1v5c0 .55.45 1 1 1s1-.45 1-1v-5c0-.55-.45-1-1-1zm8-5c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1s1-.45 1-1V5c0-.55-.45-1-1-1zm4-4c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1z"],"vertical-bar-chart-desc":["M6 4c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1s1-.45 1-1V5c0-.55-.45-1-1-1zM2 0c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1zm8 7c-.55 0-1 .45-1 1v7c0 .55.45 1 1 1s1-.45 1-1V8c0-.55-.45-1-1-1zm4 2c-.55 0-1 .45-1 1v5c0 .55.45 1 1 1s1-.45 1-1v-5c0-.55-.45-1-1-1z"],"vertical-distribution":["M1 2h14c.55 0 1-.45 1-1s-.45-1-1-1H1C.45 0 0 .45 0 1s.45 1 1 1zm14 11H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zM3 5c-.55 0-1 .45-1 1v3c0 .55.45 1 1 1h9c.55 0 1-.45 1-1V6c0-.55-.45-1-1-1H3z"],video:["M15 2H1c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1zM5 11V5l6 3-6 3z"],virus:["M11.918 11.107l.737.737.052-.051A1 1 0 0114.2 13.12l-.078.087-1.414 1.414a1 1 0 01-1.492-1.327l.029-.033-.863-.863c-.426.231-.89.402-1.38.502L9 14l.117.007A1 1 0 019 16H7l-.117-.007A1 1 0 017 14v-1.1a4.967 4.967 0 01-1.447-.539l-.846.846.078.087a1 1 0 01-1.492 1.327l-1.414-1.414-.078-.087a1 1 0 011.492-1.327l.744-.744A4.986 4.986 0 013.23 9.5H2a1 1 0 01-1.993.117L0 9.5v-2a1 1 0 011.993-.117L2 7.5h1.025a4.973 4.973 0 01.905-2.405l-.512-.513-.125.125A1 1 0 011.8 3.38l.078-.087 1.414-1.414a1 1 0 011.529 1.277l.573.575a4.969 4.969 0 011.604-.63V2l-.116-.007a1 1 0 010-1.986L7 0h2a1 1 0 01.117 1.993L9 2l.001 1.1c.639.13 1.233.381 1.757.73l.535-.537-.078-.087a1 1 0 011.492-1.327l1.414 1.414.078.087a1 1 0 01-1.492 1.327l-.535.536a4.97 4.97 0 01.803 2.257H14l.007-.117A1 1 0 0116 7.5v2l-.007.117A1 1 0 0114 9.5h-1.229a4.987 4.987 0 01-.853 1.607zM10 9a1 1 0 100 2 1 1 0 000-2zM6.5 5a1.5 1.5 0 100 3 1.5 1.5 0 000-3z"],"volume-down":["M9 2c-.28 0-.53.11-.71.29L5.59 5H3c-.55 0-1 .45-1 1v4c0 .55.45 1 1 1h2.59l2.71 2.71c.17.18.42.29.7.29.55 0 1-.45 1-1V3c0-.55-.45-1-1-1zm3.57 1.44l-1.59 1.22C11.62 5.61 12 6.76 12 8s-.38 2.39-1.02 3.34l1.59 1.22C13.47 11.27 14 9.7 14 8c0-1.7-.53-3.27-1.43-4.56z"],"volume-off":["M11 2c-.28 0-.53.11-.71.29L7.59 5H5c-.55 0-1 .45-1 1v4c0 .55.45 1 1 1h2.59l2.71 2.71c.17.18.42.29.7.29.55 0 1-.45 1-1V3c0-.55-.45-1-1-1z"],"volume-up":["M7 1.86c-.28 0-.53.11-.71.29l-2.7 2.71H1c-.55 0-1 .45-1 1v4c0 .55.45 1 1 1h2.59l2.71 2.71a1.003 1.003 0 001.71-.71v-10c-.01-.55-.46-1-1.01-1zm6.74-.99l-1.58 1.22A9.985 9.985 0 0114 7.86c0 2.16-.69 4.15-1.85 5.78l1.58 1.22c1.42-1.97 2.26-4.38 2.26-7 .01-2.61-.84-5.02-2.25-6.99zM8.98 4.52C9.62 5.48 10 6.63 10 7.86s-.38 2.39-1.02 3.34l1.59 1.22c.9-1.29 1.43-2.86 1.43-4.56 0-1.7-.53-3.27-1.43-4.56L8.98 4.52z"],walk:["M13 8h-2c-.16 0-.31-.05-.44-.11v.01l-1.02-.51-.37 1.86 1.38.92-.01.02c.27.17.46.46.46.81v4c0 .55-.45 1-1 1s-1-.45-1-1v-3.46l-1.27-.85-1.8 4.67h-.01A.98.98 0 015 16c-.55 0-1-.45-1-1 0-.13.03-.25.07-.36h-.01L7.39 6H5.62l-.73 1.45h-.01C4.72 7.77 4.39 8 4 8c-.55 0-1-.45-1-1 0-.16.05-.31.11-.44H3.1l1-2h.01c.17-.33.5-.56.89-.56h3.16l.29-.75C8.17 2.9 8 2.47 8 2c0-1.1.9-2 2-2s2 .9 2 2c0 1-.73 1.82-1.69 1.97l-.5 1.32 1.43.71H13c.55 0 1 .45 1 1s-.45 1-1 1z"],"warning-sign":["M15.84 13.5l.01-.01-7-12-.01.01c-.17-.3-.48-.5-.85-.5s-.67.2-.85.5l-.01-.01-7 12 .01.01c-.09.15-.15.31-.15.5 0 .55.45 1 1 1h14c.55 0 1-.45 1-1 0-.19-.06-.35-.15-.5zm-6.85-.51h-2v-2h2v2zm0-3h-2v-5h2v5z"],"waterfall-chart":["M8 7c.55 0 1-.45 1-1s-.45-1-1-1-1 .45-1 1 .45 1 1 1zm-4 4h1c.55 0 1-.45 1-1V8c0-.55-.45-1-1-1H4c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1zm7-6c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1s-1 .45-1 1v1c0 .55.45 1 1 1zm4-3h-1c-.55 0-1 .45-1 1v7c0 .55.45 1 1 1h1c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1zm0 10H2V3c0-.55-.45-1-1-1s-1 .45-1 1v10c0 .55.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1z"],widget:["M13 11h2V5h-2v6zM3 5H1v6h2V5zm11-1c1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2 .9 2 2 2zM2 12c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm12 0c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zM5 3h6V1H5v2zM2 0C.9 0 0 .9 0 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm3 15h6v-2H5v2z"],"widget-button":["M1 3h14c.55 0 1 .45 1 1v8c0 .55-.45 1-1 1H1c-.55 0-1-.45-1-1V4c0-.55.45-1 1-1zm1 2v6h12V5H2zm3 4c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zm3 0c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zm3 0c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1z"],"widget-footer":["M14 0H2c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1h12c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 14H3v-3h10v3zm0-4H3V2h10v8z"],"widget-header":["M14 0H2c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1h12c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 14H3V6h10v8zm0-9H3V2h10v3z"],wrench:["M15.83 3.7l-3.06 3.05-2.84-.7-.7-2.83L12.29.17a5.004 5.004 0 00-4.83 1.29 4.967 4.967 0 00-1.12 5.36L.58 12.58c-.36.36-.58.86-.58 1.41 0 1.1.9 2 2 2 .55 0 1.05-.22 1.41-.59l5.77-5.77c1.79.69 3.91.33 5.35-1.12 1.32-1.3 1.74-3.15 1.3-4.81z"],"zoom-in":["M7.99 5.99v-2c0-.55-.45-1-1-1s-1 .45-1 1v2h-2c-.55 0-1 .45-1 1s.45 1 1 1h2v2c0 .55.45 1 1 1s1-.45 1-1v-2h2c.55 0 1-.45 1-1s-.45-1-1-1h-2zm7.56 7.44l-2.67-2.68a6.94 6.94 0 001.11-3.76c0-3.87-3.13-7-7-7s-7 3.13-7 7 3.13 7 7 7c1.39 0 2.68-.42 3.76-1.11l2.68 2.67a1.498 1.498 0 102.12-2.12zm-8.56-1.44c-2.76 0-5-2.24-5-5s2.24-5 5-5 5 2.24 5 5-2.24 5-5 5z"],"zoom-out":["M3.99 5.99c-.55 0-1 .45-1 1s.45 1 1 1h6c.55 0 1-.45 1-1s-.45-1-1-1h-6zm11.56 7.44l-2.67-2.68a6.94 6.94 0 001.11-3.76c0-3.87-3.13-7-7-7s-7 3.13-7 7 3.13 7 7 7c1.39 0 2.68-.42 3.76-1.11l2.68 2.67a1.498 1.498 0 102.12-2.12zm-8.56-1.44c-2.76 0-5-2.24-5-5s2.24-5 5-5 5 2.24 5 5-2.24 5-5 5z"],"zoom-to-fit":["M11 10a1.003 1.003 0 001.71.71l2-2c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71l-2-2a1.003 1.003 0 00-1.42 1.42L12.59 8 11.3 9.29c-.19.18-.3.43-.3.71zM1 5c.55 0 1-.45 1-1V2h2c.55 0 1-.45 1-1s-.45-1-1-1H1C.45 0 0 .45 0 1v3c0 .55.45 1 1 1zm4 1a1.003 1.003 0 00-1.71-.71l-2 2C1.11 7.47 1 7.72 1 8c0 .28.11.53.29.71l2 2a1.003 1.003 0 001.42-1.42L3.41 8 4.7 6.71c.19-.18.3-.43.3-.71zm1-1c.28 0 .53-.11.71-.29L8 3.41 9.29 4.7c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71l-2-2C8.53 1.11 8.28 1 8 1s-.53.11-.71.29l-2 2A1.003 1.003 0 006 5zm9 6c-.55 0-1 .45-1 1v2h-2c-.55 0-1 .45-1 1s.45 1 1 1h3c.55 0 1-.45 1-1v-3c0-.55-.45-1-1-1zm0-11h-3c-.55 0-1 .45-1 1s.45 1 1 1h2v2c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1zM4 14H2v-2c0-.55-.45-1-1-1s-1 .45-1 1v3c0 .55.45 1 1 1h3c.55 0 1-.45 1-1s-.45-1-1-1zm6-3c-.28 0-.53.11-.71.29L8 12.59 6.71 11.3A.965.965 0 006 11a1.003 1.003 0 00-.71 1.71l2 2c.18.18.43.29.71.29s.53-.11.71-.29l2-2A1.003 1.003 0 0010 11z"]},i={add:["M10 0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zm0 18c-4.42 0-8-3.58-8-8s3.58-8 8-8 8 3.58 8 8-3.58 8-8 8zm5-9h-4V5c0-.55-.45-1-1-1s-1 .45-1 1v4H5c-.55 0-1 .45-1 1s.45 1 1 1h4v4c0 .55.45 1 1 1s1-.45 1-1v-4h4c.55 0 1-.45 1-1s-.45-1-1-1z"],"add-column-left":["M4 11h2v2c0 .55.45 1 1 1s1-.45 1-1v-2h2c.55 0 1-.45 1-1s-.45-1-1-1H8V7c0-.55-.45-1-1-1s-1 .45-1 1v2H4c-.55 0-1 .45-1 1s.45 1 1 1zM19 0H1C.45 0 0 .45 0 1v18c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-7 18H2V2h10v16zm6 0h-5V2h5v16z"],"add-column-right":["M10 11h2v2c0 .55.45 1 1 1s1-.45 1-1v-2h2c.55 0 1-.45 1-1s-.45-1-1-1h-2V7c0-.55-.45-1-1-1s-1 .45-1 1v2h-2c-.55 0-1 .45-1 1s.45 1 1 1zm9-11H1C.45 0 0 .45 0 1v18c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zM7 18H2V2h5v16zm11 0H8V2h10v16z"],"add-row-bottom":["M19 0H1C.45 0 0 .45 0 1v18c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 18H2V8h16v10zm0-11H2V2h16v5zM7 14h2v2c0 .55.45 1 1 1s1-.45 1-1v-2h2c.55 0 1-.45 1-1s-.45-1-1-1h-2v-2c0-.55-.45-1-1-1s-1 .45-1 1v2H7c-.55 0-1 .45-1 1s.45 1 1 1z"],"add-row-top":["M7 8h2v2c0 .55.45 1 1 1s1-.45 1-1V8h2c.55 0 1-.45 1-1s-.45-1-1-1h-2V4c0-.55-.45-1-1-1s-1 .45-1 1v2H7c-.55 0-1 .45-1 1s.45 1 1 1zm12-8H1C.45 0 0 .45 0 1v18c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 18H2v-5h16v5zm0-6H2V2h16v10z"],"add-to-artifact":["M13 12H1c-.55 0-1 .45-1 1s.45 1 1 1h12c.55 0 1-.45 1-1s-.45-1-1-1zm0 4H1c-.55 0-1 .45-1 1s.45 1 1 1h12c.55 0 1-.45 1-1s-.45-1-1-1zM1 6h9c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1zm12 2H1c-.55 0-1 .45-1 1s.45 1 1 1h12c.55 0 1-.45 1-1s-.45-1-1-1zm6-4h-2V2c0-.55-.45-1-1-1s-1 .45-1 1v2h-2c-.55 0-1 .45-1 1s.45 1 1 1h2v2c0 .55.45 1 1 1s1-.45 1-1V6h2c.55 0 1-.45 1-1s-.45-1-1-1z"],"add-to-folder":["M.01 10V6H20v10c0 .55-.45 1-1 1H9.995v-3.99C9.965 11.332 8.635 10 6.987 10H.01zM19 3c.55 0 1 .45.99 1v1H0V2c0-.55.45-1 1-1h5.997c.28 0 .53.11.71.29L9.414 3H19zM6.987 12c.55 0 .999.45 1.009 1.01v5c0 .55-.45 1-1 1s-.999-.45-.999-1v-2.59l-4.288 4.29a1.003 1.003 0 01-1.42-1.42L4.579 14H1.989c-.55 0-1-.45-1-1s.45-1 1-1h4.998z"],airplane:["M20 2c0-1.1-.9-2-2-2-.55 0-1.05.22-1.41.59l-4.84 4.84L2 1 1 3l7.53 5.64L4.17 13H1l-1 1 4 2 2 4 1-1v-3.17l4.36-4.36L17 19l2-1-4.43-9.74 4.84-4.84c.37-.37.59-.87.59-1.42z"],"align-center":["M5 5c-.55 0-1 .45-1 1s.45 1 1 1h10c.55 0 1-.45 1-1s-.45-1-1-1H5zM1 3h18c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1zm12 12c.55 0 1-.45 1-1s-.45-1-1-1H7c-.55 0-1 .45-1 1s.45 1 1 1h6zm4 2H3c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zm2-8H1c-.55 0-1 .45-1 1s.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1z"],"align-justify":["M1 3h18c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1zm18 14H1c-.55 0-1 .45-1 1s.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1zm0-12H1c-.55 0-1 .45-1 1s.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1zm0 4H1c-.55 0-1 .45-1 1s.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1zm0 4H1c-.55 0-1 .45-1 1s.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1z"],"align-left":["M1 7h10c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1zm0-4h18c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1zm14 14H1c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zm4-8H1c-.55 0-1 .45-1 1s.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1zM1 15h6c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1z"],"align-right":["M19 17H5c-.55 0-1 .45-1 1s.45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1zM1 3h18c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1zm18 10h-6c-.55 0-1 .45-1 1s.45 1 1 1h6c.55 0 1-.45 1-1s-.45-1-1-1zm0-4H1c-.55 0-1 .45-1 1s.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1zm0-4H9c-.55 0-1 .45-1 1s.45 1 1 1h10c.55 0 1-.45 1-1s-.45-1-1-1z"],"alignment-bottom":["M12 16h4c.55 0 1-.45 1-1V8c0-.55-.45-1-1-1h-4c-.55 0-1 .45-1 1v7c0 .55.45 1 1 1zm7 2H1c-.55 0-1 .45-1 1s.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1zM4 16h4c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1H4c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1z"],"alignment-horizontal-center":["M19 9h-2V7c0-.55-.45-1-1-1h-4c-.55 0-1 .45-1 1v2H9V3c0-.55-.45-1-1-1H4c-.55 0-1 .45-1 1v6H1c-.55 0-1 .45-1 1s.45 1 1 1h2v6c0 .55.45 1 1 1h4c.55 0 1-.45 1-1v-6h2v2c0 .55.45 1 1 1h4c.55 0 1-.45 1-1v-2h2c.55 0 1-.45 1-1s-.45-1-1-1z"],"alignment-left":["M1 0C.45 0 0 .45 0 1v18c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1zm11 11H5c-.55 0-1 .45-1 1v4c0 .55.45 1 1 1h7c.55 0 1-.45 1-1v-4c0-.55-.45-1-1-1zm7-8H5c-.55 0-1 .45-1 1v4c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V4c0-.55-.45-1-1-1z"],"alignment-right":["M19 0c-.55 0-1 .45-1 1v18c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1zm-4 11H8c-.55 0-1 .45-1 1v4c0 .55.45 1 1 1h7c.55 0 1-.45 1-1v-4c0-.55-.45-1-1-1zm0-8H1c-.55 0-1 .45-1 1v4c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V4c0-.55-.45-1-1-1z"],"alignment-top":["M8 4H4c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1h4c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1zm11-4H1C.45 0 0 .45 0 1s.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1zm-3 4h-4c-.55 0-1 .45-1 1v7c0 .55.45 1 1 1h4c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1z"],"alignment-vertical-center":["M17 3h-6V1c0-.55-.45-1-1-1S9 .45 9 1v2H3c-.55 0-1 .45-1 1v4c0 .55.45 1 1 1h6v2H7c-.55 0-1 .45-1 1v4c0 .55.45 1 1 1h2v2c0 .55.45 1 1 1s1-.45 1-1v-2h2c.55 0 1-.45 1-1v-4c0-.55-.45-1-1-1h-2V9h6c.55 0 1-.45 1-1V4c0-.55-.45-1-1-1z"],annotation:["M9.41 13.41l7.65-7.65-2.83-2.83-7.65 7.65 2.83 2.83zm10-10c.37-.36.59-.86.59-1.41 0-1.1-.9-2-2-2-.55 0-1.05.22-1.41.59l-1.65 1.65 2.83 2.83 1.64-1.66zM18 18H2V2h8.93l2-2H1C.45 0 0 .45 0 1v18c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V7.07l-2 2V18zM4 16l4.41-1.59-2.81-2.79L4 16z"],"app-header":["M19 0a1 1 0 011 1v18a1 1 0 01-1 1H1a1 1 0 01-1-1V1a1 1 0 011-1h18zM8 6a1 1 0 00-1.993-.117L6 6v8a1 1 0 001.993.117L8 14v-3h4v3a1 1 0 001.993.117L14 14V6a1 1 0 00-1.993-.117L12 6v3H8V6z"],application:["M3.5 9h9c.28 0 .5-.22.5-.5s-.22-.5-.5-.5h-9c-.28 0-.5.22-.5.5s.22.5.5.5zm0 2h5c.28 0 .5-.22.5-.5s-.22-.5-.5-.5h-5c-.28 0-.5.22-.5.5s.22.5.5.5zM19 1H1c-.55 0-1 .45-1 1v16c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1zm-1 16H2V6h16v11zM3.5 13h7c.28 0 .5-.22.5-.5s-.22-.5-.5-.5h-7c-.28 0-.5.22-.5.5s.22.5.5.5z"],applications:["M15 5H1c-.55 0-1 .45-1 1v13c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V6c0-.55-.45-1-1-1zm-1 13H2V8h12v10zM3.5 10h7c.28 0 .5-.22.5-.5s-.22-.5-.5-.5h-7c-.28 0-.5.22-.5.5s.22.5.5.5zm0 2h3c.28 0 .5-.22.5-.5s-.22-.5-.5-.5h-3c-.28 0-.5.22-.5.5s.22.5.5.5zm0 2h5c.28 0 .5-.22.5-.5s-.22-.5-.5-.5h-5c-.28 0-.5.22-.5.5s.22.5.5.5zM19 0H5c-.55 0-1 .45-1 1v3h2V3h12v10h-1v2h2c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1z"],archive:["M16.434 0a1 1 0 01.857.486L20 5v14a1 1 0 01-1 1H1a1 1 0 01-1-1V5L2.709.486A1 1 0 013.566 0h12.868zM10 8c-.55 0-1 .45-1 1v4.58l-1.29-1.29-.081-.073A.996.996 0 007 11.99a1.003 1.003 0 00-.71 1.71l3 3 .096.084c.168.13.38.206.614.206.28 0 .53-.11.71-.29l3-3 .084-.096a1.003 1.003 0 00-1.504-1.324L11 13.58V9l-.007-.116A1.004 1.004 0 0010 8zm6-6H4L2 5.002h16L16 2z"],array:["M19 0a1 1 0 01.993.883L20 1v18a1 1 0 01-.883.993L19 20h-4a1 1 0 01-.117-1.993L15 18h3V2h-3a1 1 0 01-.993-.883L14 1a1 1 0 01.883-.993L15 0h4zM5 0a1 1 0 01.117 1.993L5 2H2v16h3a1 1 0 01.993.883L6 19a1 1 0 01-.883.993L5 20H1a1 1 0 01-.993-.883L0 19V1A1 1 0 01.883.007L1 0h4zm5 9a1 1 0 110 2 1 1 0 010-2zM6 9a1 1 0 110 2 1 1 0 010-2zm8 0a1 1 0 110 2 1 1 0 010-2z"],"array-boolean":["M19 0a1 1 0 01.993.883L20 1v18a1 1 0 01-.883.993L19 20h-4a1 1 0 01-.117-1.993L15 18h3V2h-3a1 1 0 01-.993-.883L14 1a1 1 0 01.883-.993L15 0h4zM5 0a1 1 0 01.117 1.993L5 2H2v16h3a1 1 0 01.993.883L6 19a1 1 0 01-.883.993L5 20H1a1 1 0 01-.993-.883L0 19V1A1 1 0 01.883.007L1 0h4zm10 7a1 1 0 01.993.883L16 8v4a1 1 0 01-.883.993L15 13H5a1 1 0 01-.993-.883L4 12V8a1 1 0 01.883-.993L5 7h10zm0 1h-5v4h5V8z"],"array-date":["M19 0a1 1 0 01.993.883L20 1v18a1 1 0 01-.883.993L19 20h-4a1 1 0 01-.117-1.993L15 18h3V2h-3a1 1 0 01-.993-.883L14 1a1 1 0 01.883-.993L15 0h4zM5 0a1 1 0 01.117 1.993L5 2H2v16h3a1 1 0 01.993.883L6 19a1 1 0 01-.883.993L5 20H1a1 1 0 01-.993-.883L0 19V1A1 1 0 01.883.007L1 0h4zm2.5 5a.5.5 0 01.5.5V6h4v-.5a.5.5 0 01.5-.5h1a.5.5 0 01.5.5V6h1a1 1 0 01.993.883L16 7v7a1 1 0 01-.883.993L15 15H5a1 1 0 01-.993-.883L4 14V7a1 1 0 01.883-.993L5 6h1v-.5a.5.5 0 01.5-.5h1zM15 9H5v5h10V9z"],"array-numeric":["M19 0a1 1 0 01.993.883L20 1v18a1 1 0 01-.883.993L19 20h-4a1 1 0 01-.117-1.993L15 18h3V2h-3a1 1 0 01-.993-.883L14 1a1 1 0 01.883-.993L15 0h4zM5 0a1 1 0 01.117 1.993L5 2H2v16h3a1 1 0 01.993.883L6 19a1 1 0 01-.883.993L5 20H1a1 1 0 01-.993-.883L0 19V1A1 1 0 01.883.007L1 0h4zm8.995 6.09c.32 0 .603.047.85.14a1.679 1.679 0 011.015.985c.09.23.135.482.135.755 0 .287-.063.552-.19.795a1.642 1.642 0 01-.57.615v.02l.101.05c.068.039.138.09.209.155.107.097.203.215.29.355a2.09 2.09 0 01.3 1.09c0 .313-.053.597-.16.85a1.898 1.898 0 01-1.12 1.065 2.42 2.42 0 01-.85.145c-.533 0-.99-.127-1.37-.38a1.702 1.702 0 01-.73-1.08c-.013-.067.013-.11.08-.13l.87-.2.041-.003c.038.004.064.028.079.073.073.2.193.37.36.51.167.14.39.21.67.21.32 0 .567-.095.74-.285.173-.19.26-.432.26-.725 0-.347-.1-.618-.3-.815-.2-.197-.47-.295-.81-.295h-.29l-.044-.006c-.037-.013-.056-.044-.056-.094V9.1l.006-.044c.013-.037.044-.056.094-.056h.27l.145-.008a.925.925 0 00.575-.262c.18-.18.27-.42.27-.72 0-.273-.08-.495-.24-.665-.16-.17-.383-.255-.67-.255-.253 0-.453.063-.6.19-.147.127-.25.297-.31.51-.02.06-.06.083-.12.07l-.85-.2-.042-.015c-.035-.02-.048-.055-.038-.105a1.684 1.684 0 01.645-1.035c.17-.13.37-.232.6-.305.23-.073.482-.11.755-.11zm-7.99.11l.044.006c.037.013.056.044.056.094v6.6l-.006.044c-.013.037-.044.056-.094.056h-.92l-.044-.006c-.037-.013-.056-.044-.056-.094V7.54h-.02l-1.04.73-.04.02c-.047.016-.07-.01-.07-.08V7.19l.008-.051a.196.196 0 01.062-.089l1.09-.79.051-.033a.295.295 0 01.129-.027h.85zm3.34-.11c.32 0 .603.05.85.15.247.1.455.235.625.405.17.17.3.37.39.6.09.23.135.478.135.745 0 .307-.057.588-.17.845a3.16 3.16 0 01-.47.745l-1.98 2.4V12h2.52l.044.006c.037.013.056.044.056.094v.8l-.006.044c-.013.037-.044.056-.094.056h-3.82l-.044-.006c-.037-.013-.056-.044-.056-.094v-.75l.006-.05a.165.165 0 01.044-.08l2.4-2.99.124-.167c.077-.11.143-.222.196-.333.08-.167.12-.347.12-.54a.92.92 0 00-.235-.64c-.157-.173-.378-.26-.665-.26-.273 0-.487.077-.64.23-.153.153-.247.36-.28.62-.013.067-.05.093-.11.08l-.88-.18-.043-.015c-.035-.02-.047-.055-.037-.105a1.78 1.78 0 01.56-1.115c.167-.157.372-.282.615-.375a2.35 2.35 0 01.845-.14z"],"array-string":["M19 0a1 1 0 01.993.883L20 1v18a1 1 0 01-.883.993L19 20h-4a1 1 0 01-.117-1.993L15 18h3V2h-3a1 1 0 01-.993-.883L14 1a1 1 0 01.883-.993L15 0h4zM5 0a1 1 0 01.117 1.993L5 2H2v16h3a1 1 0 01.993.883L6 19a1 1 0 01-.883.993L5 20H1a1 1 0 01-.993-.883L0 19V1A1 1 0 01.883.007L1 0h4zm2.012 6c.643 0 1.203.266 1.68.797.477.53.715 1.246.715 2.145a4.472 4.472 0 01-.965 2.814c-.644.83-1.66 1.5-3.047 2.011v-.581l.26-.104a3.87 3.87 0 001.624-1.285c.457-.632.686-1.29.686-1.971 0-.148-.023-.256-.07-.326-.023-.047-.054-.07-.093-.07-.038 0-.1.031-.186.093-.248.179-.558.268-.93.268-.45 0-.843-.18-1.18-.541A1.817 1.817 0 015 7.965c0-.527.194-.986.581-1.378A1.934 1.934 0 017.011 6zm5.593 0c.643 0 1.203.266 1.68.797.477.53.715 1.246.715 2.145a4.472 4.472 0 01-.965 2.814c-.644.83-1.659 1.5-3.047 2.011v-.581l.26-.104a3.87 3.87 0 001.624-1.285c.457-.632.686-1.29.686-1.971 0-.148-.023-.256-.07-.326-.023-.047-.054-.07-.093-.07-.038 0-.1.031-.186.093-.248.179-.558.268-.93.268-.45 0-.843-.18-1.18-.541a1.817 1.817 0 01-.506-1.285c0-.527.194-.986.581-1.378A1.934 1.934 0 0112.604 6z"],"array-timestamp":["M19 0a1 1 0 01.993.883L20 1v18a1 1 0 01-.883.993L19 20h-4a1 1 0 01-.117-1.993L15 18h3V2h-3a1 1 0 01-.993-.883L14 1a1 1 0 01.883-.993L15 0h4zM5 0a1 1 0 01.117 1.993L5 2H2v16h3a1 1 0 01.993.883L6 19a1 1 0 01-.883.993L5 20H1a1 1 0 01-.993-.883L0 19V1A1 1 0 01.883.007L1 0h4zm5 4a6 6 0 110 12 6 6 0 010-12zm0 1a5 5 0 100 10 5 5 0 000-10zm2.854 2.146a.5.5 0 01.057.638l-.057.07-2.5 2.5a.5.5 0 01-.638.057l-.07-.057-1.5-1.5a.5.5 0 01.638-.765l.07.057L10 9.293l2.146-2.147a.5.5 0 01.708 0z"],"arrow-bottom-left":["M18 3a1.003 1.003 0 00-1.71-.71L4 14.59V7c0-.55-.45-1-1-1s-1 .45-1 1v10c0 .55.45 1 1 1h10c.55 0 1-.45 1-1s-.45-1-1-1H5.41l12.3-12.29c.18-.18.29-.43.29-.71z"],"arrow-bottom-right":["M17 6c-.55 0-1 .45-1 1v7.59L3.71 2.29a1.003 1.003 0 00-1.42 1.42L14.59 16H7c-.55 0-1 .45-1 1s.45 1 1 1h10c.55 0 1-.45 1-1V7c0-.55-.45-1-1-1z"],"arrow-down":["M16 11c-.3 0-.5.1-.7.3L11 15.6V2c0-.5-.4-1-1-1s-1 .5-1 1v13.6l-4.3-4.3c-.2-.2-.4-.3-.7-.3-.5 0-1 .4-1 1 0 .3.1.5.3.7l6 6c.2.2.4.3.7.3s.5-.1.7-.3l6-6c.2-.2.3-.4.3-.7 0-.6-.5-1-1-1z"],"arrow-left":["M18 9H4.41L8.7 4.71c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71l-6 6c-.18.18-.29.43-.29.71 0 .28.11.53.29.71l6 6a1.003 1.003 0 001.42-1.42L4.41 11H18c.55 0 1-.45 1-1s-.45-1-1-1z"],"arrow-right":["M18.71 9.29l-6-6a1.003 1.003 0 00-1.42 1.42L15.59 9H2c-.55 0-1 .45-1 1s.45 1 1 1h13.59l-4.29 4.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71l6-6c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71z"],"arrow-top-left":["M17.71 16.29L5.41 4H13c.55 0 1-.45 1-1s-.45-1-1-1H3c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1s1-.45 1-1V5.41L16.29 17.7c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71z"],"arrow-top-right":["M17 2H7c-.55 0-1 .45-1 1s.45 1 1 1h7.59L2.29 16.29a1.003 1.003 0 001.42 1.42L16 5.41V13c0 .55.45 1 1 1s1-.45 1-1V3c0-.55-.45-1-1-1z"],"arrow-up":["M16.7 7.3l-6-6c-.2-.2-.4-.3-.7-.3s-.5.1-.7.3l-6 6c-.2.2-.3.4-.3.7 0 .6.5 1 1 1 .3 0 .5-.1.7-.3L9 4.4V18c0 .5.4 1 1 1s1-.5 1-1V4.4l4.3 4.3c.2.2.4.3.7.3.5 0 1-.4 1-1 0-.3-.1-.5-.3-.7z"],"arrows-horizontal":["M19.7 9.3l-5-5c-.2-.2-.4-.3-.7-.3-.6 0-1 .4-1 1 0 .3.1.5.3.7L16.6 9H3.4l3.3-3.3c.2-.2.3-.4.3-.7 0-.6-.4-1-1-1-.3 0-.5.1-.7.3l-5 5c-.2.2-.3.4-.3.7s.1.5.3.7l5 5c.2.2.4.3.7.3.6 0 1-.4 1-1 0-.3-.1-.5-.3-.7L3.4 11h13.2l-3.3 3.3c-.2.2-.3.4-.3.7 0 .6.4 1 1 1 .3 0 .5-.1.7-.3l5-5c.2-.2.3-.4.3-.7s-.1-.5-.3-.7z"],"arrows-vertical":["M15 13c-.3 0-.5.1-.7.3L11 16.6V3.4l3.3 3.3c.2.2.4.3.7.3.6 0 1-.4 1-1 0-.3-.1-.5-.3-.7l-5-5c-.2-.2-.4-.3-.7-.3s-.5.1-.7.3l-5 5c-.2.2-.3.4-.3.7 0 .6.4 1 1 1 .3 0 .5-.1.7-.3L9 3.4v13.2l-3.3-3.3c-.2-.2-.4-.3-.7-.3-.6 0-1 .4-1 1 0 .3.1.5.3.7l5 5c.2.2.4.3.7.3s.5-.1.7-.3l5-5c.2-.2.3-.4.3-.7 0-.5-.4-1-1-1z"],asterisk:["M18.52 14.17l.01-.02L11.89 10l6.64-4.15-.01-.02A.97.97 0 0019 5c0-.55-.45-1-1-1-.2 0-.37.07-.52.17l-.01-.02L11 8.2V1c0-.55-.45-1-1-1S9 .45 9 1v7.2L2.53 4.15l-.01.02A.922.922 0 002 4c-.55 0-1 .45-1 1 0 .36.2.66.48.83l-.01.02L8.11 10l-6.64 4.15.01.02A.97.97 0 001 15c0 .55.45 1 1 1 .2 0 .37-.07.52-.17l.01.02L9 11.8V19c0 .55.45 1 1 1s1-.45 1-1v-7.2l6.47 4.04.01-.02c.15.11.32.18.52.18.55 0 1-.45 1-1 0-.36-.2-.66-.48-.83z"],"automatic-updates":["M10 18c-4.42 0-8-3.58-8-8 0-2.52 1.18-4.76 3-6.22V5c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1H2c-.55 0-1 .45-1 1s.45 1 1 1h2.06C1.61 3.82 0 6.71 0 10c0 5.52 4.48 10 10 10 .55 0 1-.45 1-1s-.45-1-1-1zm0-16c1.64 0 3.15.49 4.42 1.34l1.43-1.43A9.869 9.869 0 0010 0c-.55 0-1 .45-1 1s.45 1 1 1zm10 8c0-1.13-.2-2.21-.54-3.22L17.84 8.4A7.962 7.962 0 0115 16.22V15c0-.55-.45-1-1-1s-1 .45-1 1v4c0 .55.45 1 1 1h4c.55 0 1-.45 1-1s-.45-1-1-1h-2.06c2.45-1.82 4.06-4.71 4.06-8zm0-7a1.003 1.003 0 00-1.71-.71L12 8.59l-2.29-2.3a1.003 1.003 0 00-1.42 1.42l3 3c.18.18.43.29.71.29s.53-.11.71-.29l7-7c.18-.18.29-.43.29-.71z"],backlink:["M18.387 19.79l-.094-.083L14 15.415V18a1 1 0 01-2 0l.003-5.075.017-.126.03-.111.044-.111.052-.098.067-.096.08-.09a1.01 1.01 0 01.112-.097l.11-.071.114-.054.105-.035.15-.03L13 12h5a1 1 0 110 2h-2.585l4.292 4.293a1 1 0 01-1.32 1.497zM7.036 9.136l-4.45 4.45-.117.127a2 2 0 002.818 2.818l.127-.117 4.45-4.449a4 4 0 01-.885 3.704l-.15.16-2 2A4 4 0 011.02 12.33l.15-.16 2-2a3.998 3.998 0 013.865-1.035zm6.671-3.843a1 1 0 01.083 1.32l-.083.094-7 7a1 1 0 01-1.497-1.32l.083-.094 7-7a1 1 0 011.414 0zm4.121-4.121a4 4 0 01.151 5.497l-.15.16-2 2a3.998 3.998 0 01-3.864 1.036l4.45-4.45.116-.128a2 2 0 00-2.818-2.818l-.127.117-4.45 4.45a4 4 0 01.885-3.705l.15-.16 2-2a4 4 0 015.657 0z"],badge:["M16.94 5.73c-.19-1.41.62-2.52 1.38-3.59L17.03.65C14.89 1.76 11.88 1.48 10 0 8.12 1.48 5.11 1.76 2.97.65L1.68 2.14c.76 1.07 1.57 2.18 1.38 3.59C2.68 8.59 0 10.94 1.4 14.08c.56 1.43 1.81 2.37 3.4 2.75 1.95.46 4.4.91 5.2 3.17.8-2.26 3.25-2.71 5.2-3.17 1.6-.38 2.84-1.32 3.4-2.75 1.4-3.14-1.28-5.49-1.66-8.35z"],"ban-circle":["M10 0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zm5 11H5c-.55 0-1-.45-1-1s.45-1 1-1h10c.55 0 1 .45 1 1s-.45 1-1 1z"],"bank-account":["M19.2 8.02l-.78-.18C18.03 6.4 17.2 5.08 16.08 4l.5-2.28c.11-.42-.22-.78-.61-.72-1.06.12-2 .54-2.67 1.26-1.06-.42-2.34-.66-3.56-.66-3.12 0-5.79 1.5-7.4 3.72-.23-.05-.45-.11-.67-.11C.72 5.21 0 5.98 0 7c0 .72.39 1.32.95 1.62-.06.42-.12.9-.12 1.38 0 2.16.89 4.08 2.28 5.58l-.33 2.04c-.11.72.45 1.38 1.12 1.38h.72c.56 0 1-.42 1.11-1.02l.06-.48c1.17.54 2.5.9 3.95.9 1.39 0 2.78-.3 3.95-.9l.06.48c.11.6.56 1.02 1.11 1.02h.72c.67 0 1.22-.66 1.11-1.38l-.33-1.98c.78-.78 1.34-1.74 1.73-2.76l1-.24c.5-.12.89-.6.89-1.2V9.22c.11-.6-.28-1.08-.78-1.2zM15 10c-.6 0-1-.7-1-1.5S14.4 7 15 7s1 .7 1 1.5-.4 1.5-1 1.5zM7.55 5.83a.99.99 0 01-1.38-.28.99.99 0 01.28-1.38c2.34-1.56 4.77-1.56 7.11 0 .46.31.58.93.28 1.39-.31.46-.93.58-1.39.28-1.67-1.12-3.23-1.12-4.9-.01z"],barcode:["M6 16.98h2v-14H6v14zm3 0h1v-14H9v14zm-6 0h2v-14H3v14zm-3 0h2v-14H0v14zm16 0h2v-14h-2v14zm-4 0h1v-14h-1v14zm7-14v14h1v-14h-1zm-5 14h1v-14h-1v14z"],blank:[],"blocked-person":["M11.55 15.92c-1.48-.65-1.28-1.05-1.33-1.59-.01-.07-.01-.15-.01-.23.51-.45.92-1.07 1.19-1.78 0 0 .01-.04.02-.05.06-.15.11-.32.15-.48.34-.07.54-.44.61-.78.06-.11.14-.35.17-.62C10.33 9.42 8.92 7.38 8.92 5c0-.3.05-.58.09-.87-.33-.08-.67-.13-.99-.13-.79 0-1.68.25-2.31.73-.61.47-1.07 1.13-1.29 1.86-.05.16-.09.33-.11.5-.12.6-.17 1.51-.17 2.14v.08c-.24.09-.45.32-.49.83-.04.39.12.73.2.87.08.35.28.72.63.78.04.17.09.33.15.48 0 .01.01.02.01.03l.01.01c.27.72.7 1.35 1.22 1.8 0 .07-.01.14-.01.21-.05.54.1.94-1.38 1.59C3 16.56.77 17.26.32 18.31-.15 19.38.04 20 .04 20h15.95s.18-.62-.27-1.67c-.46-1.06-2.69-1.75-4.17-2.41zM14.97 0c-2.78 0-5.03 2.24-5.03 5s2.25 5 5.03 5S20 7.76 20 5s-2.25-5-5.03-5zm-3.03 5c0-1.66 1.35-3 3.02-3 .47 0 .9.11 1.29.3l-4.01 3.99c-.18-.4-.3-.83-.3-1.29zm3.03 3c-.47 0-.9-.11-1.29-.3l4.01-3.99c.19.39.3.82.3 1.29 0 1.66-1.36 3-3.02 3z"],bold:["M14.3 9c.4-.8.7-1.6.7-2.5C15 4 13 2 10.5 2H5c-.6 0-1 .4-1 1v13c0 .6.4 1 1 1h6.5c2.5 0 4.5-2 4.5-4.5 0-1.4-.7-2.7-1.7-3.5zM7 5h3.5c.8 0 1.5.7 1.5 1.5S11.3 8 10.5 8H7V5zm4.5 9H7v-3h4.5c.8 0 1.5.7 1.5 1.5s-.7 1.5-1.5 1.5z"],book:["M3 1v18c0 .55.45 1 1 1h2V0H4c-.55 0-1 .45-1 1zm14-1h-2v8l-2-2-2 2V0H7v20h10c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1z"],bookmark:["M6 0c-.55 0-1 .45-1 1v18c0 .55.32.68.71.29L9.3 15.7a.996.996 0 011.41 0l3.59 3.59c.38.39.7.26.7-.29v-8-4.5V1c0-.55-.45-1-1-1H6z"],box:["M19.89 6.56l-2.99-6h-.01C16.72.23 16.39 0 16 0H4c-.39 0-.72.23-.89.56H3.1l-3 6h.01C.05 6.69 0 6.84 0 7v12c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V7c0-.16-.05-.31-.11-.44zM11 2h4.38l2 4H11V2zM4.62 2H9v4H2.62l2-4zM18 18H2V8h16v10zM8 12h4c.55 0 1-.45 1-1s-.45-1-1-1H8c-.55 0-1 .45-1 1s.45 1 1 1z"],briefcase:["M19 5h-4V2c0-.55-.45-1-1-1H6c-.55 0-1 .45-1 1v3H1c-.55 0-1 .45-1 1v5h4v-1h2v1h8v-1h2v1h4V6c0-.55-.45-1-1-1zm-6 0H7V3h6v2zm3 8h-2v-1H6v1H4v-1H0v6c0 .55.45 1 1 1h18c.55 0 1-.45 1-1v-6h-4v1z"],"bring-data":["M18 18a1 1 0 010 2H2a1 1 0 010-2h16zM9.995 3.005c.55 0 1 .45 1 .999v9.584l1.29-1.288a1.002 1.002 0 011.42 1.419l-3 2.996a1.015 1.015 0 01-1.42 0l-3-2.997a1.002 1.002 0 011.42-1.419l1.29 1.29V4.013c0-.55.45-1.009 1-1.009zM16 0a1 1 0 110 2 1 1 0 010-2zm-3 0a1 1 0 110 2 1 1 0 010-2zm-3 0a1 1 0 110 2 1 1 0 010-2zM7 0a1 1 0 110 2 1 1 0 010-2zM4 0a1 1 0 110 2 1 1 0 010-2z"],build:["M19.43 16.67L9.31 7.81l1.47-1.56c.41-.44-.15-.8.15-1.6 1.08-2.76 4.19-2.99 4.19-2.99s.45-.47.87-.92C11.98-1 9.26.7 8.04 1.8L3.83 6.25l-.86.92c-.48.51-.48 1.33 0 1.84l-.87.92c-.48-.51-1.26-.51-1.74 0s-.48 1.33 0 1.84l1.74 1.84c.48.51 1.26.51 1.74 0s.48-1.33 0-1.84l.87-.92c.48.51 1.26.51 1.74 0l1.41-1.49 8.81 10.07c.76.76 2 .76 2.76 0 .76-.76.76-2 0-2.76z"],calculator:["M16 0H4c-.55 0-1 .45-1 1v18c0 .55.45 1 1 1h12c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zM7 18H5v-2h2v2zm0-4H5v-2h2v2zm0-4H5V8h2v2zm4 8H9v-2h2v2zm0-4H9v-2h2v2zm0-4H9V8h2v2zm4 8h-2v-6h2v6zm0-8h-2V8h2v2zm0-4H5V2h10v4z"],calendar:["M15 5c.6 0 1-.4 1-1V2c0-.5-.4-1-1-1s-1 .5-1 1v2c0 .6.4 1 1 1zM5 5c.6 0 1-.4 1-1V2c0-.5-.4-1-1-1s-1 .5-1 1v2c0 .6.4 1 1 1zm13-2h-1v1c0 1.1-.9 2-2 2s-2-.9-2-2V3H7v1c0 1.1-.9 2-2 2s-2-.9-2-2V3H2c-.5 0-1 .5-1 1v14c0 .5.5 1 1 1h16c.5 0 1-.5 1-1V4c0-.5-.5-1-1-1zM7 17H3v-4h4v4zm0-5H3V8h4v4zm5 5H8v-4h4v4zm0-5H8V8h4v4zm5 5h-4v-4h4v4zm0-5h-4V8h4v4z"],camera:["M10 8c-1.66 0-3 1.34-3 3s1.34 3 3 3 3-1.34 3-3-1.34-3-3-3zm9-4h-3.59L13.7 2.29A.956.956 0 0013 2H7c-.28 0-.53.11-.71.29L4.59 4H1c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1h4.11c1.26 1.24 2.99 2 4.89 2s3.63-.76 4.89-2H19c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1zM4 8H2V6h2v2zm6 8c-2.76 0-5-2.24-5-5s2.24-5 5-5 5 2.24 5 5-2.24 5-5 5z"],"caret-down":["M16 7c0-.55-.45-1-1-1H5c-.55 0-1 .45-1 1 0 .24.1.46.24.63l-.01.01 5 6 .01-.01c.19.22.45.37.76.37s.57-.15.76-.37l.01.01 5-6-.01-.01c.14-.17.24-.39.24-.63z"],"caret-left":["M13 4c-.24 0-.46.1-.63.24l-.01-.01-6 5 .01.01c-.22.19-.37.45-.37.76s.15.57.37.76l-.01.01 6 5 .01-.01c.17.14.39.24.63.24.55 0 1-.45 1-1V5c0-.55-.45-1-1-1z"],"caret-right":["M14 10c0-.31-.15-.57-.37-.76l.01-.01-6-5-.01.01C7.46 4.1 7.24 4 7 4c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1 .24 0 .46-.1.63-.24l.01.01 6-5-.01-.01c.22-.19.37-.45.37-.76z"],"caret-up":["M15.76 12.37l.01-.01-5-6-.01.01C10.57 6.15 10.31 6 10 6s-.57.15-.76.37l-.01-.01-5 6 .01.01c-.14.17-.24.39-.24.63 0 .55.45 1 1 1h10c.55 0 1-.45 1-1 0-.24-.1-.46-.24-.63z"],"cell-tower":["M11.5 8.32c.31-.35.51-.81.51-1.32 0-1.1-.9-2-2-2s-2 .9-2 2c0 .51.2.97.51 1.32L5.06 18.69c-.17.52.11 1.09.63 1.26s1.09-.11 1.26-.63L8.39 15h3.23l1.44 4.32c.17.52.74.81 1.26.63s.81-.74.63-1.26L11.5 8.32zM10.95 13H9.06l.95-2.84.94 2.84zM5.31 10.73a.996.996 0 101.37-1.45c-1.4-1.33-1.28-3.35-.01-4.54.4-.38.43-1.01.05-1.41-.36-.41-1-.43-1.4-.06-2.09 1.95-2.28 5.3-.01 7.46z","M4.6 12.2C3 11.1 2 9 2 7c0-2.1.9-3.9 2.6-5.2.5-.3.5-1 .2-1.4-.3-.5-1-.5-1.4-.2C1.2 1.9-.1 4.2 0 7c.1 2.7 1.4 5.3 3.4 6.8.2.1.4.2.6.2.3 0 .6-.1.8-.4.4-.5.3-1.1-.2-1.4zM13.27 10.69c.38.4 1.01.42 1.41.04 2.27-2.16 2.08-5.51-.01-7.46a.996.996 0 10-1.36 1.46c1.28 1.19 1.39 3.21-.01 4.54-.39.39-.41 1.02-.03 1.42z","M16.6.2c-.4-.3-1.1-.3-1.4.2-.3.4-.3 1.1.2 1.4C17.1 3.1 18 4.9 18 7c0 2-1 4.1-2.6 5.2-.5.3-.6.9-.2 1.4.2.3.5.4.8.4.2 0 .4-.1.6-.2C18.7 12.3 20 9.7 20 7c.09-2.8-1.2-5.1-3.4-6.8z"],changes:["M18 16H2c-1.1 0-2 .9-2 2s.9 2 2 2h16c1.1 0 2-.9 2-2s-.9-2-2-2zM3 5c.28 0 .53-.11.71-.29L5 3.41V13c0 .55.45 1 1 1s1-.45 1-1V3.41L8.29 4.7c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71l-3-3C6.53.11 6.28 0 6 0s-.53.11-.71.29l-3 3A1.003 1.003 0 003 5zm7.29 5.71l3 3c.18.18.43.29.71.29s.53-.11.71-.29l3-3a1.003 1.003 0 00-1.42-1.42L15 10.59V1c0-.55-.45-1-1-1s-1 .45-1 1v9.59L11.71 9.3A.965.965 0 0011 9a1.003 1.003 0 00-.71 1.71z"],chart:["M7 11v8c0 .55.45 1 1 1h4c.55 0 1-.45 1-1v-8l-2 2-4-2zm-7 8c0 .55.45 1 1 1h4c.55 0 1-.45 1-1v-8l-6 3v5zM17 7l-3 3v9c0 .55.45 1 1 1h4c.55 0 1-.45 1-1V8.74c-.26.15-.58.26-1 .26-1.92 0-2-2-2-2zm2-6h-4c-.55 0-1 .45-1 1s.45 1 1 1h1.59L10.8 8.78 7.45 7.11v.01C7.31 7.05 7.16 7 7 7s-.31.05-.44.11V7.1l-6 3v.01c-.33.17-.56.5-.56.89 0 .55.45 1 1 1 .16 0 .31-.05.44-.11v.01L7 9.12l3.55 1.78v-.01c.14.06.29.11.45.11.28 0 .53-.11.71-.29L18 4.41V6c0 .55.45 1 1 1s1-.45 1-1V2c0-.55-.45-1-1-1z"],chat:["M19 0H7c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1h5.59l3.71 3.71c.17.18.42.29.7.29.55 0 1-.45 1-1v-3h1c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zM7 13c-1.1 0-2-.9-2-2V4H1c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1h1v3a1.003 1.003 0 001.71.71L7.41 16H13c.55 0 1-.45 1-1v-.17L12.17 13H7z"],"chevron-backward":["M8.41 10l5.29-5.29c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71L7 8.59V4c0-.55-.45-1-1-1s-1 .45-1 1v12c0 .55.45 1 1 1s1-.45 1-1v-4.59l5.29 5.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71L8.41 10z"],"chevron-down":["M16 6c-.28 0-.53.11-.71.29L10 11.59l-5.29-5.3a1.003 1.003 0 00-1.42 1.42l6 6c.18.18.43.29.71.29s.53-.11.71-.29l6-6A1.003 1.003 0 0016 6z"],"chevron-forward":["M13 3c-.55 0-1 .45-1 1v4.59l-5.29-5.3a1.003 1.003 0 00-1.42 1.42l5.3 5.29-5.29 5.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71l5.29-5.3V16c0 .55.45 1 1 1s1-.45 1-1V4c0-.55-.45-1-1-1z"],"chevron-left":["M8.41 10l5.29-5.29c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71l-6 6c-.18.18-.29.43-.29.71 0 .28.11.53.29.71l6 6a1.003 1.003 0 001.42-1.42L8.41 10z"],"chevron-right":["M13.71 9.29l-6-6a1.003 1.003 0 00-1.42 1.42l5.3 5.29-5.29 5.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71l6-6c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71z"],"chevron-up":["M16.71 12.29l-6-6C10.53 6.11 10.28 6 10 6s-.53.11-.71.29l-6 6a1.003 1.003 0 001.42 1.42L10 8.41l5.29 5.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71z"],circle:["M10 0C4.5 0 0 4.5 0 10s4.5 10 10 10 10-4.5 10-10S15.5 0 10 0zm0 18c-4.4 0-8-3.6-8-8s3.6-8 8-8 8 3.6 8 8-3.6 8-8 8z"],"circle-arrow-down":["M14 10c-.28 0-.53.11-.71.29L11 12.59V5c0-.55-.45-1-1-1s-1 .45-1 1v7.59L6.71 10.3A.965.965 0 006 10a1.003 1.003 0 00-.71 1.71l4 4c.18.18.43.29.71.29s.53-.11.71-.29l4-4A1.003 1.003 0 0014 10zM10 0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zm0 18c-4.42 0-8-3.58-8-8s3.58-8 8-8 8 3.58 8 8-3.58 8-8 8z"],"circle-arrow-left":["M15 9H7.41L9.7 6.71c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71l-4 4c-.18.18-.29.43-.29.71 0 .28.11.53.29.71l4 4a1.003 1.003 0 001.42-1.42L7.41 11H15c.55 0 1-.45 1-1s-.45-1-1-1zm-5-9C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zm0 18c-4.42 0-8-3.58-8-8s3.58-8 8-8 8 3.58 8 8-3.58 8-8 8z"],"circle-arrow-right":["M15.71 9.29l-4-4a1.003 1.003 0 00-1.42 1.42L12.59 9H5c-.55 0-1 .45-1 1s.45 1 1 1h7.59l-2.29 2.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71l4-4c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71zM10 0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zm0 18c-4.42 0-8-3.58-8-8s3.58-8 8-8 8 3.58 8 8-3.58 8-8 8z"],"circle-arrow-up":["M10 0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zm0 18c-4.42 0-8-3.58-8-8s3.58-8 8-8 8 3.58 8 8-3.58 8-8 8zm.71-13.71C10.53 4.11 10.28 4 10 4s-.53.11-.71.29l-4 4a1.003 1.003 0 001.42 1.42L9 7.41V15c0 .55.45 1 1 1s1-.45 1-1V7.41l2.29 2.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71l-4-4z"],citation:["M4 1C1.79 1 0 2.79 0 5s1.79 4 4 4c.1 0 .2-.01.3-.02C3.82 11.32 2.53 13 1 13c-.55 0-1 .45-1 1s.45 1 1 1c3.87 0 7-4.48 7-10 0-2.21-1.79-4-4-4zM16 1c-2.21 0-4 1.79-4 4s1.79 4 4 4c.1 0 .2-.01.3-.02C15.82 11.32 14.53 13 13 13c-.55 0-1 .45-1 1s.45 1 1 1c3.87 0 7-4.48 7-10 0-2.21-1.79-4-4-4z"],clean:["M7 0L5 5 0 6.998 5 9l2 5 2-5 5-1.995L9 5zM15 10l-1.5 3.496-3.5 1.499 3.5 1.498L15 20l1.5-3.507L20 15l-3.5-1.504z"],clipboard:["M13 2c0-.55-.45-1-1-1h-.78a1.98 1.98 0 00-3.44 0H7c-.55 0-1 .45-1 1v2h7V2z","M16 2h-2v3H5V2H3c-.55 0-1 .45-1 1v16c0 .55.45 1 1 1h13c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1z"],cloud:["M15 7c-.12 0-.24.03-.36.04C13.83 4.69 11.62 3 9 3 5.69 3 3 5.69 3 9c0 .05.01.09.01.14A3.98 3.98 0 000 13c0 2.21 1.79 4 4 4h11c2.76 0 5-2.24 5-5s-2.24-5-5-5z"],"cloud-download":["M15 4c-.12 0-.24.03-.36.04C13.83 1.69 11.62 0 9 0 5.69 0 3 2.69 3 6c0 .05.01.09.01.14A3.98 3.98 0 000 10c0 2.21 1.79 4 4 4h.78c.55-.61 1.34-1 2.22-1v-2c0-1.66 1.34-3 3-3s3 1.34 3 3v2c.88 0 1.66.38 2.2.98C17.87 13.87 20 11.69 20 9c0-2.76-2.24-5-5-5zm-2 11c-.28 0-.53.11-.71.29L11 16.59V11c0-.55-.45-1-1-1s-1 .45-1 1v5.59L7.71 15.3A.965.965 0 007 15a1.003 1.003 0 00-.71 1.71l3 3c.18.18.43.29.71.29s.53-.11.71-.29l3-3A1.003 1.003 0 0013 15z"],"cloud-upload":["M10.71 10.29c-.18-.18-.43-.29-.71-.29s-.53.11-.71.29l-3 3a1.003 1.003 0 001.42 1.42L9 13.41V19c0 .55.45 1 1 1s1-.45 1-1v-5.59l1.29 1.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71l-3-3zM15 4c-.12 0-.24.03-.36.04C13.83 1.69 11.62 0 9 0 5.69 0 3 2.69 3 6c0 .05.01.09.01.14A3.98 3.98 0 000 10c0 2.21 1.79 4 4 4 0-.83.34-1.58.88-2.12l3-3a2.993 2.993 0 014.24 0l3 3-.01.01c.52.52.85 1.23.87 2.02C18.28 13.44 20 11.42 20 9c0-2.76-2.24-5-5-5z"],code:["M6 6a1.003 1.003 0 00-1.71-.71l-4 4C.11 9.47 0 9.72 0 10c0 .28.11.53.29.71l4 4a1.003 1.003 0 001.42-1.42L2.41 10 5.7 6.71c.19-.18.3-.43.3-.71zm6-4c-.46 0-.83.31-.95.73l-4 14c-.02.09-.05.17-.05.27 0 .55.45 1 1 1 .46 0 .83-.31.95-.73l4-14c.02-.09.05-.17.05-.27 0-.55-.45-1-1-1zm7.71 7.29l-4-4a1.003 1.003 0 00-1.42 1.42l3.3 3.29-3.29 3.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71l4-4c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71z"],"code-block":["M19 5h-2V3c0-.55-.45-1-1-1h-4c-.55 0-1 .45-1 1v2H9V3c0-.55-.45-1-1-1H4c-.55 0-1 .45-1 1v2H1c-.55 0-1 .45-1 1v12c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V6c0-.55-.45-1-1-1zM8.71 15.29a1.003 1.003 0 01-1.42 1.42l-4-4C3.11 12.53 3 12.28 3 12s.11-.53.29-.71l4-4a1.003 1.003 0 011.42 1.42L5.41 12l3.3 3.29zm8-2.58l-4 4a1.003 1.003 0 01-1.42-1.42l3.3-3.29-3.29-3.29A.965.965 0 0111 8a1.003 1.003 0 011.71-.71l4 4c.18.18.29.43.29.71s-.11.53-.29.71z"],cog:["M19 8h-2.31c-.14-.46-.33-.89-.56-1.3l1.7-1.7a.996.996 0 000-1.41l-1.41-1.41a.996.996 0 00-1.41 0l-1.7 1.7c-.41-.22-.84-.41-1.3-.55V1c0-.55-.45-1-1-1H9c-.55 0-1 .45-1 1v2.33c-.48.14-.94.34-1.37.58L5 2.28a.972.972 0 00-1.36 0L2.28 3.64c-.37.38-.37.99 0 1.36L3.9 6.62c-.24.44-.44.89-.59 1.38H1c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h2.31c.14.46.33.89.56 1.3L2.17 15a.996.996 0 000 1.41l1.41 1.41c.39.39 1.02.39 1.41 0l1.7-1.7c.41.22.84.41 1.3.55V19c0 .55.45 1 1 1h2c.55 0 1-.45 1-1v-2.33c.48-.14.94-.35 1.37-.59L15 17.72c.37.37.98.37 1.36 0l1.36-1.36c.37-.37.37-.98 0-1.36l-1.62-1.62c.24-.43.45-.89.6-1.38H19c.55 0 1-.45 1-1V9c0-.55-.45-1-1-1zm-9 6c-2.21 0-4-1.79-4-4s1.79-4 4-4 4 1.79 4 4-1.79 4-4 4z"],"collapse-all":["M9.29 8.71c.18.18.43.29.71.29s.53-.11.71-.29l6-6a1.003 1.003 0 00-1.42-1.42L10 6.59l-5.29-5.3a1.003 1.003 0 00-1.42 1.42l6 6zm1.42 2.58c-.18-.18-.43-.29-.71-.29s-.53.11-.71.29l-6 6a1.003 1.003 0 001.42 1.42l5.29-5.3 5.29 5.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71l-6-6z"],"column-layout":["M19 1H1c-.55 0-1 .45-1 1v16c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1zM5 17H2V3h3v14zm4 0H6V3h3v14zm9 0h-8V3h8v14z"],comment:["M19 1H1c-.55 0-1 .45-1 1v12c0 .55.45 1 1 1h3v4a1.003 1.003 0 001.71.71l4.7-4.71H19c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1zM4 10c-1.1 0-2-.9-2-2s.9-2 2-2 2 .9 2 2-.9 2-2 2zm6 0c-1.1 0-2-.9-2-2s.9-2 2-2 2 .9 2 2-.9 2-2 2zm6 0c-1.1 0-2-.9-2-2s.9-2 2-2 2 .9 2 2-.9 2-2 2z"],comparison:["M6 8H1c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h5c.55 0 1-.45 1-1V9c0-.55-.45-1-1-1zm13-6h-5c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h5c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1zm0 3h-5V3h5v2zM6 14H1c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h5c.55 0 1-.45 1-1v-2c0-.55-.45-1-1-1zM6 2H1c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h5c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1zm4-2c-.55 0-1 .45-1 1v18c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1zm9 14h-5c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h5c.55 0 1-.45 1-1v-2c0-.55-.45-1-1-1zm0 3h-5v-2h5v2zm0-9h-5c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h5c.55 0 1-.45 1-1V9c0-.55-.45-1-1-1zm0 3h-5V9h5v2z"],compass:["M15 10c0 .14-.03.28-.09.4l-3.99 8.98-.01.02a.991.991 0 01-1.82 0l-.01-.02-3.99-8.98c-.06-.12-.09-.26-.09-.4s.03-.28.09-.4L9.08.62 9.09.6a.991.991 0 011.82 0l.01.02 3.99 8.98c.06.12.09.26.09.4zm-5-6.54L7.09 10h5.81L10 3.46z"],compressed:["M19.89 6.56l-2.99-6h-.01C16.72.23 16.39 0 16 0H4c-.39 0-.72.23-.89.56H3.1l-3 6h.01C.05 6.69 0 6.84 0 7v12c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V7c0-.16-.05-.31-.11-.44zM11 2h4.38l2 4H11V2zM4.62 2H9v4H2.62l2-4zM18 18H2V8h7v4.59L6.71 10.3A.965.965 0 006 10a1.003 1.003 0 00-.71 1.71l4 4c.18.18.43.29.71.29s.53-.11.71-.29l4-4a1.003 1.003 0 00-1.42-1.42L11 12.59V8h7v10z"],confirm:["M9.71 5.29a1.003 1.003 0 00-1.42 1.42l3 3c.18.18.43.29.71.29s.53-.11.71-.29l7-7a1.003 1.003 0 00-1.42-1.42L12 7.59l-2.29-2.3zm7.93 2.32c.23.75.36 1.56.36 2.39 0 4.42-3.58 8-8 8s-8-3.58-8-8a7.998 7.998 0 0111.8-7.04l1.46-1.46C13.73.56 11.93 0 10 0 4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10c0-1.4-.29-2.73-.81-3.95l-1.55 1.56z"],console:["M19 19H1c-.55 0-1-.45-1-1V2c0-.55.45-1 1-1h18c.55 0 1 .45 1 1v16c0 .55-.45 1-1 1zM18 6H2v11h16V6zM4 8c.28 0 .53.11.71.29l2 2c.18.18.29.43.29.71s-.11.53-.29.71l-2 2a1.003 1.003 0 01-1.42-1.42L4.59 11l-1.3-1.29A1.003 1.003 0 014 8zm5 4h3c.55 0 1 .45 1 1s-.45 1-1 1H9c-.55 0-1-.45-1-1s.45-1 1-1z"],contrast:["M19 8h-1.26c-.19-.73-.48-1.42-.85-2.06l.94-.94a.996.996 0 000-1.41l-1.41-1.41a.996.996 0 00-1.41 0l-.94.94c-.65-.38-1.34-.67-2.07-.86V1c0-.55-.45-1-1-1H9c-.55 0-1 .45-1 1v1.26c-.76.2-1.47.5-2.13.89L5 2.28a.972.972 0 00-1.36 0L2.28 3.64c-.37.38-.37.98 0 1.36l.87.87c-.39.66-.69 1.37-.89 2.13H1c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h1.26c.19.73.48 1.42.85 2.06l-.94.94a.996.996 0 000 1.41l1.41 1.41c.39.39 1.02.39 1.41 0l.94-.94c.64.38 1.33.66 2.06.85V19c0 .55.45 1 1 1h2c.55 0 1-.45 1-1v-1.26c.76-.2 1.47-.5 2.13-.89l.88.87c.37.37.98.37 1.36 0l1.36-1.36c.37-.38.37-.98 0-1.36l-.87-.87c.4-.65.7-1.37.89-2.13H19c.55 0 1-.45 1-1V9c0-.55-.45-1-1-1zm-9 7c-2.76 0-5-2.24-5-5s2.24-5 5-5v10z"],control:["M17 10h-7v7h7v-7zm0-7h-7v6h7V3zM9 3H3v14h6V3zm10-3H1C.45 0 0 .45 0 1v18c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 18H2V2h16v16z"],"credit-card":["M19 3H1c-.55 0-1 .45-1 1v2h20V4c0-.55-.45-1-1-1zM0 16c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V8H0v8zm6.5-2h7c.28 0 .5.22.5.5s-.22.5-.5.5h-7c-.28 0-.5-.22-.5-.5s.22-.5.5-.5zm-4 0h2c.28 0 .5.22.5.5s-.22.5-.5.5h-2c-.28 0-.5-.22-.5-.5s.22-.5.5-.5z"],cross:["M11.41 10l4.29-4.29c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71L10 8.59l-4.29-4.3a1.003 1.003 0 00-1.42 1.42L8.59 10 4.3 14.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71l4.29-4.3 4.29 4.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71L11.41 10z"],crown:["M2 8l4 2 4-5 4 5 4-2-1 7H3L2 8zm8-6a1 1 0 110 2 1 1 0 010-2zM1 5a1 1 0 110 2 1 1 0 010-2zm18 0a1 1 0 110 2 1 1 0 010-2zM3 16h14v2H3v-2z"],cube:["M1.953 4.481l7.41-4.02c.394-.215.88-.215 1.275 0l7.409 4.02L10 9.22 1.953 4.48zm-.817.68L9.5 10.085v9.281a1.316 1.316 0 01-.138-.064l-7.714-4.186A1.211 1.211 0 011 14.057v-8.35c0-.193.048-.38.136-.547zm17.728 0c.088.166.136.353.136.546v8.35c0 .438-.247.842-.648 1.06l-7.714 4.186c-.045.024-.091.046-.138.064v-9.281l8.364-4.926z"],"cube-add":["M17 3h2a1 1 0 010 2h-2v2a1 1 0 01-2 0V5h-2a1 1 0 010-2h2V1a1 1 0 012 0v2zm-3.969 4.435L10 9.22 1.953 4.48l7.41-4.02c.394-.215.88-.215 1.275 0l1.33.721A3.001 3.001 0 0013 7c0 .148.01.293.031.435zm.319.972A3 3 0 0019 7v7.057c0 .438-.247.842-.648 1.06l-7.714 4.186c-.045.024-.091.046-.138.064v-9.281l2.85-1.679zM1.136 5.16L9.5 10.086v9.281a1.316 1.316 0 01-.138-.064l-7.714-4.186A1.211 1.211 0 011 14.057v-8.35c0-.193.048-.38.136-.547z"],"cube-remove":["M11.968 1.182A3.001 3.001 0 0013 7h.77L10 9.22 1.953 4.48l7.41-4.02c.394-.215.88-.215 1.275 0l1.33.721zM19 7v7.057c0 .438-.247.842-.648 1.06l-7.714 4.186c-.045.024-.091.046-.138.064v-9.281L15.74 7H19zM1.136 5.16L9.5 10.086v9.281a1.316 1.316 0 01-.138-.064l-7.714-4.186A1.211 1.211 0 011 14.057v-8.35c0-.193.048-.38.136-.547zM13 3h6a1 1 0 010 2h-6a1 1 0 010-2z"],"curved-range-chart":["M19 16H3.02l2.14-1.74c2.25 1.7 7.33.46 11.83-2.99l-1.29-1.5c-3.56 2.74-7.31 4.03-8.93 3.19l10.55-8.57-.63-.78-10.59 8.6c-.64-1.64 1.46-4.91 5.09-7.7L9.9 3.01c-4.6 3.54-6.91 8.12-5.41 10.51L2 15.54V3c0-.55-.45-1-1-1s-1 .45-1 1v14a.998.998 0 001 1h18c.55 0 1-.45 1-1s-.45-1-1-1z"],cut:["M16 2s.72-1.28 0-2l-5.29 6.25 1.28 1.54L16 2zm.08 10c-.55 0-1.07.12-1.54.32L4.31 0c-.7.72 0 2 0 2l4.45 6.56-3.19 3.77C5.09 12.12 4.56 12 4 12c-2.21 0-4 1.79-4 4s1.79 4 4 4 4-1.79 4-4c0-.65-.17-1.26-.45-1.8l2.54-3.67 2.49 3.67c-.27.54-.44 1.15-.44 1.8 0 2.21 1.76 4 3.92 4 2.17 0 3.92-1.79 3.92-4 .02-2.21-1.74-4-3.9-4zM4 18c-1.1 0-2-.9-2-2s.9-2 2-2 2 .9 2 2-.9 2-2 2zm12.08 0c-1.08 0-1.96-.9-1.96-2s.88-2 1.96-2 1.96.9 1.96 2-.88 2-1.96 2z"],cycle:["M16 10a4 4 0 110 8 4 4 0 010-8zM4 10a4 4 0 110 8 4 4 0 010-8zm7.299-5.543l.087.089 1.93 2.232 2.048.708a1 1 0 01.65 1.16l-.031.112a1 1 0 01-1.16.65l-.112-.031-2.302-.796a1 1 0 01-.337-.197l-.092-.094-1.387-1.603-1.891 1.982 2.046 2.274a1 1 0 01.25.547l.007.122v4.24a1 1 0 01-1.993.117l-.007-.117-.001-3.857-2.408-2.676a1 1 0 01-.063-1.26l.082-.099 3.29-3.45a1 1 0 011.394-.053zM16 12a2 2 0 100 4 2 2 0 000-4zM4 12a2 2 0 100 4 2 2 0 000-4zm9.5-10a1.5 1.5 0 110 3 1.5 1.5 0 010-3z"],dashboard:["M6 5c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zM4 9c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm6-4c.55 0 1-.45 1-1s-.45-1-1-1-1 .45-1 1 .45 1 1 1zm0-5C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zm0 18c-4.42 0-8-3.58-8-8s3.58-8 8-8 8 3.58 8 8-3.58 8-8 8zm6-9c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm-8 5c0 1.1.9 2 2 2s2-.9 2-2c0-.33-2-8-2-8s-2 7.67-2 8zm6-9c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1z"],"data-connection":["M2 11.9c.935.674 2.339 1.217 4.023 1.536A6.996 6.996 0 009.393 20c-3.988-.019-7.231-1.083-7.387-2.4L2 17.5v-5.6zM13 8c3.315 0 6 2.685 6 6s-2.685 6-6 6-6-2.685-6-6 2.685-6 6-6zm1 1l-4 6h2.5l-.5 4 4-6h-2.5l.5-4zm3-4.6v3.855a7.003 7.003 0 00-10.779 3.992c-2.408-.391-4.097-1.202-4.214-2.142L2 10V4.4c1.525 1.1 4.3 1.85 7.5 1.85S15.475 5.5 17 4.4zM9.5 0C13.637 0 17 1.125 17 2.5S13.637 5 9.5 5C5.35 5 2 3.875 2 2.5S5.35 0 9.5 0z"],"data-lineage":["M1.053 0C.47 0 0 .471 0 1.053V4.21c0 .58.471 1.052 1.053 1.052h3.275a6.332 6.332 0 003.728 4.738 6.33 6.33 0 00-3.728 4.737l-3.275-.001C.47 14.737 0 15.208 0 15.789v3.158C0 19.53.471 20 1.053 20h7.435c.581 0 1.053-.471 1.053-1.053V15.79c0-.58-.472-1.052-1.053-1.052H5.406a5.293 5.293 0 015.195-4.21v2.105c0 .58.471 1.052 1.052 1.052h7.294c.582 0 1.053-.471 1.053-1.052V7.368c0-.58-.471-1.052-1.053-1.052h-7.294c-.581 0-1.052.471-1.052 1.052v2.106a5.293 5.293 0 01-5.194-4.21h3.081c.581 0 1.053-.472 1.053-1.053V1.053C9.54.47 9.069 0 8.488 0H1.053z"],database:["M2.01 5.1v5.4c0 1.38 3.58 2.5 8 2.5s8-1.12 8-2.5V5.1c-1.49 1.13-4.51 1.9-8 1.9-3.48 0-6.5-.77-8-1.9zm8 .9c4.42 0 8-1.12 8-2.5s-3.58-2.5-8-2.5-8 1.12-8 2.5S5.6 6 10.01 6zm-8 6.1v5.4c0 1.38 3.58 2.5 8 2.5s8-1.12 8-2.5v-5.4c-1.49 1.13-4.51 1.9-8 1.9-3.48 0-6.5-.77-8-1.9z"],delete:["M15 6a1.003 1.003 0 00-1.71-.71L10 8.59l-3.29-3.3a1.003 1.003 0 00-1.42 1.42L8.59 10 5.3 13.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71l3.29-3.3 3.29 3.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71L11.41 10l3.29-3.29c.19-.18.3-.43.3-.71zm-5-6C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zm0 18c-4.42 0-8-3.58-8-8s3.58-8 8-8 8 3.58 8 8-3.58 8-8 8z"],delta:["M10 0L0 20h20L10 0zM9 6l6 12H3L9 6z"],"derive-column":["M7.1 8.2h-.99c.28-1.11.66-1.92 1.12-2.43.28-.32.56-.48.83-.48.05 0 .1.02.13.05.03.03.05.07.05.12 0 .04-.04.13-.11.25a.64.64 0 00-.12.35c0 .15.06.28.18.39.12.11.27.16.45.16.2 0 .36-.07.49-.2s.2-.31.2-.54c0-.26-.1-.47-.3-.63-.19-.16-.51-.24-.95-.24-.68 0-1.3.19-1.85.58-.56.38-1.09 1.02-1.59 1.91-.17.3-.34.5-.49.59-.15.08-.4.13-.74.12l-.23.77h.95l-1.39 5.24c-.23.86-.39 1.39-.47 1.59-.12.29-.3.54-.54.75-.1.08-.21.12-.35.12-.04 0-.07-.01-.1-.03l-.03-.04c0-.02.03-.07.1-.13.07-.07.1-.17.1-.31 0-.15-.05-.28-.16-.38-.11-.1-.27-.15-.47-.15-.25 0-.44.07-.59.2-.15.12-.23.28-.23.46 0 .19.09.36.27.5.19.14.47.21.86.21.61 0 1.16-.15 1.63-.46.48-.31.89-.78 1.25-1.43.35-.64.72-1.68 1.09-3.11l.8-3.03h.96l.24-.77zM19 0h-9c-.55 0-1 .45-1 1v3h2V2h7v16h-7v-2H9v3c0 .55.45 1 1 1h9c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-8.79 13.49c.15.28.32.49.52.61.19.12.44.19.73.19.28 0 .57-.1.86-.3.38-.25.77-.69 1.17-1.31l-.25-.14c-.27.37-.48.6-.61.69-.09.06-.19.09-.31.09-.14 0-.28-.09-.42-.26-.23-.29-.54-1.09-.93-2.4.35-.59.64-.97.87-1.15.17-.13.35-.2.55-.2.07 0 .2.03.39.08s.36.08.5.08c.2 0 .37-.07.5-.2.15-.14.22-.31.22-.52 0-.22-.07-.4-.2-.53s-.33-.2-.58-.2c-.22 0-.43.05-.63.15-.2.1-.45.32-.75.67-.23.25-.56.7-1.01 1.33a6.52 6.52 0 00-.91-2.15l-2.39.39-.05.25c.18-.03.33-.05.45-.05.24 0 .43.1.59.3.25.31.59 1.24 1.02 2.8-.34.44-.58.73-.7.87-.21.22-.38.36-.52.43-.1.05-.22.08-.35.08-.1 0-.26-.05-.49-.16a1.01 1.01 0 00-.42-.11c-.23 0-.42.07-.57.22-.15.14-.23.33-.23.55 0 .21.07.38.21.51.14.13.33.2.56.2.23 0 .44-.05.64-.14.2-.09.45-.29.75-.59s.72-.78 1.25-1.43c.21.61.39 1.06.54 1.35z"],desktop:["M19 0H1C.45 0 0 .45 0 1v13c0 .55.45 1 1 1h5.67l-.5 3H5c-.55 0-1 .45-1 1s.45 1 1 1h10c.55 0 1-.45 1-1s-.45-1-1-1h-1.17l-.5-3H19c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 13H2V2h16v11z"],diagnosis:["M4 2a1 1 0 01.117 1.993L4 4v5a2 2 0 001.85 1.995L6 11a2 2 0 001.995-1.85L8 9V4a1 1 0 01-.117-1.993L8 2h1a1 1 0 01.993.883L10 3v6a4.002 4.002 0 01-3 3.874V13a3 3 0 003 3 4 4 0 003.995-3.8L14 12V8.792a2.5 2.5 0 112 0V12a6 6 0 01-6 6 5 5 0 01-4.995-4.783L5 13v-.126A4.002 4.002 0 012.005 9.2L2 9V3a1 1 0 01.883-.993L3 2h1z"],"diagram-tree":["M19 10v5h-2v-4h-6v4H9v-4H3v4H1v-5a1 1 0 011-1h7V5h2v4h7a1 1 0 011 1zM1 16h2a1 1 0 011 1v2a1 1 0 01-1 1H1a1 1 0 01-1-1v-2a1 1 0 011-1zm16 0h2a1 1 0 011 1v2a1 1 0 01-1 1h-2a1 1 0 01-1-1v-2a1 1 0 011-1zm-8 0h2a1 1 0 011 1v2a1 1 0 01-1 1H9a1 1 0 01-1-1v-2a1 1 0 011-1zM9 0h2a1 1 0 011 1v2a1 1 0 01-1 1H9a1 1 0 01-1-1V1a1 1 0 011-1z"],"direction-left":["M20 3.02l-20 7 20 7-5-7z"],"direction-right":["M20 10.02l-20-7 5 7-5 7z"],disable:["M10 0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zM2 10c0-4.42 3.58-8 8-8 1.85 0 3.55.63 4.9 1.69L3.69 14.9A7.902 7.902 0 012 10zm8 8c-1.85 0-3.55-.63-4.9-1.69L16.31 5.1A7.902 7.902 0 0118 10c0 4.42-3.58 8-8 8z"],document:["M11.98 0h-8c-.55 0-1 .45-1 1v18c0 .55.45 1 1 1h13c.55 0 1-.45 1-1V6l-6-6zm4 18h-11V2h6v5h5v11z"],"document-open":["M8 15c0 .55.45 1 1 1s1-.45 1-1v-5c0-.55-.45-1-1-1H4c-.55 0-1 .45-1 1s.45 1 1 1h2.59L1.3 16.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71L8 12.41V15zm5-15H5c-.55 0-1 .45-1 1v6h2V2h6v5h5v11H6v-.76L4.04 19.2c.1.45.48.8.96.8h13c.55 0 1-.45 1-1V6l-6-6z"],"document-share":["M14.09 10.09c-.31.31-.67.57-1.09.72V18H2V2h6v5h1.18c.15-.42.39-.8.7-1.11v-.01l2.45-2.45c-.42-.29-.78-.65-1.01-1.11L9 0H1C.45 0 0 .45 0 1v18c0 .55.45 1 1 1h13c.55 0 1-.45 1-1V9.24l-.88.88-.03-.03zM19 0h-5c-.55 0-1 .45-1 1s.45 1 1 1h2.59L11.3 7.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71L18 3.41V6c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1z"],dollar:["M15.57 11.19c-.27-.51-.63-.93-1.07-1.26-.44-.33-.95-.6-1.51-.79-.56-.2-1.14-.36-1.72-.5-.6-.14-1.19-.26-1.75-.38-.57-.13-1.07-.27-1.51-.44-.44-.17-.8-.38-1.07-.63s-.41-.59-.41-1c0-.33.09-.6.28-.81.19-.21.42-.36.69-.47.27-.11.57-.18.88-.22.31-.04.58-.06.8-.06.71 0 1.35.14 1.9.41.55.27.91.81 1.06 1.62h3.36c-.09-.84-.32-1.56-.69-2.16-.37-.6-.83-1.08-1.38-1.45-.56-.37-1.18-.64-1.86-.81-.19-.05-.38-.07-.57-.1V1c0-.55-.45-1-1-1H9c-.55 0-1 .45-1 1v1.1c-.22.03-.43.05-.66.1-.73.13-1.39.37-1.98.71-.6.34-1.09.8-1.47 1.35-.39.56-.58 1.25-.58 2.08 0 .76.13 1.41.4 1.93.26.52.62.95 1.06 1.28.44.33.94.6 1.5.79.55.2 1.13.36 1.74.5.58.14 1.16.26 1.72.38s1.07.26 1.51.43c.44.17.8.39 1.09.66.28.27.43.63.45 1.06.02.43-.08.78-.3 1.04-.22.26-.49.47-.83.6-.34.14-.7.23-1.09.28-.39.05-.73.07-1.03.07-.87 0-1.61-.2-2.23-.59-.62-.39-.98-1.08-1.07-2.06H3c.02.9.19 1.68.52 2.34.33.66.78 1.21 1.35 1.65.57.44 1.25.77 2.03.98.35.1.71.16 1.08.21V19c0 .55.45 1 1 1h1c.55 0 1-.45 1-1v-1.13c.25-.04.5-.07.76-.13.77-.18 1.47-.46 2.1-.85.63-.39 1.14-.9 1.54-1.53.4-.63.59-1.39.59-2.29.01-.75-.13-1.37-.4-1.88z"],dot:["M10 6a4 4 0 100 8 4 4 0 100-8z"],"double-caret-horizontal":["M8 4c-.24 0-.46.1-.63.24l-.01-.01-6 5 .01.01c-.22.19-.37.45-.37.76s.15.57.37.76l-.01.01 6 5 .01-.01c.17.14.39.24.63.24.55 0 1-.45 1-1V5c0-.55-.45-1-1-1zm11 6c0-.31-.15-.57-.37-.76l.01-.01-6-5-.01.01C12.46 4.1 12.24 4 12 4c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1 .24 0 .46-.1.63-.24l.01.01 6-5-.01-.01c.22-.19.37-.45.37-.76z"],"double-caret-vertical":["M5 9h10c.55 0 1-.45 1-1 0-.24-.1-.46-.24-.63l.01-.01-5-6-.01.01C10.57 1.15 10.31 1 10 1s-.57.15-.76.37l-.01-.01-5 6 .01.01C4.1 7.54 4 7.76 4 8c0 .55.45 1 1 1zm10 2H5c-.55 0-1 .45-1 1 0 .24.1.46.24.63l-.01.01 5 6 .01-.01c.19.22.45.37.76.37s.57-.15.76-.37l.01.01 5-6-.01-.01c.14-.17.24-.39.24-.63 0-.55-.45-1-1-1z"],"double-chevron-down":["M9.29 10.71c.18.18.43.29.71.29s.53-.11.71-.29l6-6a1.003 1.003 0 00-1.42-1.42L10 8.59l-5.29-5.3a1.003 1.003 0 00-1.42 1.42l6 6zM16 9c-.28 0-.53.11-.71.29L10 14.59l-5.29-5.3a1.003 1.003 0 00-1.42 1.42l6 6c.18.18.43.29.71.29s.53-.11.71-.29l6-6A1.003 1.003 0 0016 9z"],"double-chevron-left":["M5.41 10l5.29-5.29c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71l-6 6c-.18.18-.29.43-.29.71 0 .28.11.53.29.71l6 6a1.003 1.003 0 001.42-1.42L5.41 10zm6 0l5.29-5.29c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71l-6 6c-.18.18-.29.43-.29.71 0 .28.11.53.29.71l6 6a1.003 1.003 0 001.42-1.42L11.41 10z"],"double-chevron-right":["M11 10c0-.28-.11-.53-.29-.71l-6-6a1.003 1.003 0 00-1.42 1.42L8.59 10 3.3 15.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71l6-6c.18-.18.29-.43.29-.71zm5.71-.71l-6-6a1.003 1.003 0 00-1.42 1.42l5.3 5.29-5.29 5.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71l6-6c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71z"],"double-chevron-up":["M4 11c.28 0 .53-.11.71-.29L10 5.41l5.29 5.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71l-6-6A.997.997 0 0010 3c-.28 0-.53.11-.71.29l-6 6A1.003 1.003 0 004 11zm6.71-1.71A.997.997 0 0010 9c-.28 0-.53.11-.71.29l-6 6a1.003 1.003 0 001.42 1.42l5.29-5.3 5.29 5.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71l-6-6z"],"doughnut-chart":["M16 10c0 3.31-2.69 6-6 6s-6-2.69-6-6 2.69-6 6-6V0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10h-4zm-.09-1h4.04C19.48 4.28 15.72.52 11 .05V4.1A5.98 5.98 0 0115.91 9z"],download:["M10 0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zm4.71 11.71l-4 4c-.18.18-.43.29-.71.29s-.53-.11-.71-.29l-4-4a1.003 1.003 0 011.42-1.42L9 12.59V5c0-.55.45-1 1-1s1 .45 1 1v7.59l2.29-2.29c.18-.19.43-.3.71-.3a1.003 1.003 0 01.71 1.71z"],"drag-handle-horizontal":["M7.5 11c-.83 0-1.5.67-1.5 1.5S6.67 14 7.5 14 9 13.33 9 12.5 8.33 11 7.5 11zm-5-5C1.67 6 1 6.67 1 7.5S1.67 9 2.5 9 4 8.33 4 7.5 3.33 6 2.5 6zm0 5c-.83 0-1.5.67-1.5 1.5S1.67 14 2.5 14 4 13.33 4 12.5 3.33 11 2.5 11zm15-2c.83 0 1.5-.67 1.5-1.5S18.33 6 17.5 6 16 6.67 16 7.5 16.67 9 17.5 9zm-5 2c-.83 0-1.5.67-1.5 1.5s.67 1.5 1.5 1.5 1.5-.67 1.5-1.5-.67-1.5-1.5-1.5zm5 0c-.83 0-1.5.67-1.5 1.5s.67 1.5 1.5 1.5 1.5-.67 1.5-1.5-.67-1.5-1.5-1.5zm-10-5C6.67 6 6 6.67 6 7.5S6.67 9 7.5 9 9 8.33 9 7.5 8.33 6 7.5 6zm5 0c-.83 0-1.5.67-1.5 1.5S11.67 9 12.5 9 14 8.33 14 7.5 13.33 6 12.5 6z"],"drag-handle-vertical":["M7.5 6C6.67 6 6 6.67 6 7.5S6.67 9 7.5 9 9 8.33 9 7.5 8.33 6 7.5 6zm0 5c-.83 0-1.5.67-1.5 1.5S6.67 14 7.5 14 9 13.33 9 12.5 8.33 11 7.5 11zm0 5c-.83 0-1.5.67-1.5 1.5S6.67 19 7.5 19 9 18.33 9 17.5 8.33 16 7.5 16zm5-12c.83 0 1.5-.67 1.5-1.5S13.33 1 12.5 1 11 1.67 11 2.5 11.67 4 12.5 4zm-5-3C6.67 1 6 1.67 6 2.5S6.67 4 7.5 4 9 3.33 9 2.5 8.33 1 7.5 1zm5 10c-.83 0-1.5.67-1.5 1.5s.67 1.5 1.5 1.5 1.5-.67 1.5-1.5-.67-1.5-1.5-1.5zm0 5c-.83 0-1.5.67-1.5 1.5s.67 1.5 1.5 1.5 1.5-.67 1.5-1.5-.67-1.5-1.5-1.5zm0-10c-.83 0-1.5.67-1.5 1.5S11.67 9 12.5 9 14 8.33 14 7.5 13.33 6 12.5 6z"],draw:["M17.7 12.7c0-.1 0-.2-.1-.3l-2-7c-.1-.3-.3-.6-.6-.7L1.8 0l-.6.5L7.7 7c.3-.2.6-.3 1-.3 1.1 0 2 .9 2 2s-.9 2-2 2-2-.9-2-2c0-.4.1-.7.3-1L.5 1.2l-.5.6L4.7 15c.1.3.4.5.7.6l7 2c.1 0 .2.1.3.1.3 0 .5-.1.7-.3l4-4c.2-.2.3-.5.3-.7zm1 1c-.3 0-.5.1-.7.3l-4 4c-.2.2-.3.4-.3.7 0 .5.4 1 1 1 .3 0 .5-.1.7-.3l4-4c.2-.2.3-.4.3-.7 0-.6-.5-1-1-1z"],"drawer-left":["M9 0a1 1 0 011 1v18a1 1 0 01-1 1H1a1 1 0 01-1-1V1a1 1 0 011-1h8zM8 2H2v16h6V2zm2 7h6.59L14.3 6.71A.965.965 0 0114 6a1.003 1.003 0 011.71-.71l4 4c.18.18.29.43.29.71 0 .28-.11.53-.29.71l-4 4a1.003 1.003 0 01-1.42-1.42l2.3-2.29H10V9z"],"drawer-left-filled":["M1 0h8a1 1 0 011 1v18a1 1 0 01-1 1H1a1 1 0 01-1-1V1a1 1 0 011-1zm9 9h6.59L14.3 6.71A.965.965 0 0114 6a1.003 1.003 0 011.71-.71l4 4c.18.18.29.43.29.71 0 .28-.11.53-.29.71l-4 4a1.003 1.003 0 01-1.42-1.42l2.3-2.29H10V9z"],"drawer-right":["M19 0a1 1 0 011 1v18a1 1 0 01-1 1h-8a1 1 0 01-1-1V1a1 1 0 011-1h8zm-1 2h-6v16h6V2zm-8 7H3.41L5.7 6.71c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71l-4 4C.11 9.47 0 9.72 0 10c0 .28.11.53.29.71l4 4a1.003 1.003 0 001.42-1.42L3.41 11H10V9z"],"drawer-right-filled":["M11 0h8a1 1 0 011 1v18a1 1 0 01-1 1h-8a1 1 0 01-1-1V1a1 1 0 011-1zm-1 9H3.41L5.7 6.71c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71l-4 4C.11 9.47 0 9.72 0 10c0 .28.11.53.29.71l4 4a1.003 1.003 0 001.42-1.42L3.41 11H10V9z"],"drive-time":["M20.01 7.7c0-.63-.5-1.14-1.1-1.14h-1.32l-.95-2.57c-.24-.64-.95-1.31-1.59-1.5 0 0-1.65-.49-5.05-.49s-5.04.49-5.04.49c-.63.19-1.35.86-1.59 1.5l-.95 2.57H1.1C.5 6.56 0 7.07 0 7.7c0 .63.5 1.14 1.1 1.14h.47l-.34.91c-.24.64-.43 1.72-.43 2.4v5.39c0 .8.63 1.45 1.4 1.45.77 0 1.4-.65 1.4-1.45v-.83h12.8v.83c0 .8.63 1.45 1.4 1.45s1.4-.65 1.4-1.45v-5.39c0-.68-.19-1.77-.43-2.4l-.34-.91h.47c.61 0 1.11-.51 1.11-1.14zm-16.47.34l1.12-3.16c.08-.22.32-.39.54-.39h9.6c.22 0 .46.17.54.39l1.12 3.16c.08.21-.04.39-.26.39H3.8c-.22-.01-.34-.18-.26-.39zm.96 4.94c-.83 0-1.5-.67-1.5-1.5s.67-1.5 1.5-1.5 1.5.68 1.5 1.5c0 .83-.67 1.5-1.5 1.5zm11 0c-.83 0-1.5-.67-1.5-1.5s.67-1.5 1.5-1.5 1.5.67 1.5 1.5-.67 1.5-1.5 1.5z"],duplicate:["M15 4H1c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1zm-1 14H2V6h12v12zm5-18H5c-.55 0-1 .45-1 1v2h2V2h12v12h-1v2h2c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1z"],edit:["M4.59 12.59l2.83 2.83 7.65-7.65-2.83-2.83-7.65 7.65zM2 18l4.41-1.59-2.81-2.79L2 18zM16 2c-.55 0-1.05.22-1.41.59l-1.65 1.65 2.83 2.83 1.65-1.65A2.006 2.006 0 0016 2z"],eject:["M4 12h12c.55 0 1-.45 1-1 0-.25-.1-.47-.25-.64l.01-.01-6-7-.01.01C10.57 3.14 10.3 3 10 3s-.57.14-.75.36l-.01-.01-6 7 .01.01c-.15.17-.25.39-.25.64 0 .55.45 1 1 1zm12 1H4c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h12c.55 0 1-.45 1-1v-2c0-.55-.45-1-1-1z"],emoji:["M10 0c5.523 0 10 4.477 10 10s-4.477 10-10 10S0 15.523 0 10 4.477 0 10 0zm0 2a8 8 0 100 16 8 8 0 000-16zm-4 8l.015.215C6.219 12.42 7.925 14 10 14a4 4 0 003.995-3.8L14 10h2l-.013.238C15.754 13.552 13.163 16 10 16a6 6 0 01-5.996-5.775L4 10h2zm1.5-4a1.5 1.5 0 110 3 1.5 1.5 0 010-3zm5 0a1.5 1.5 0 110 3 1.5 1.5 0 010-3z"],endorsed:["M19.83 9.38L18.81 7.6V5.62c0-.45-.23-.85-.61-1.08l-1.71-1-1.02-1.76a1.25 1.25 0 00-1.08-.61h-2.03l-1.74-1c-.38-.23-.87-.23-1.25 0l-1.74 1H5.65c-.44 0-.85.23-1.08.61L3.58 3.5l-1.8 1.04c-.38.24-.62.64-.62 1.08v2.06L.17 9.4c-.11.19-.17.4-.17.61s.06.42.17.61l.99 1.72v2.06c0 .45.23.85.61 1.08l1.78 1.02.99 1.72c.23.38.63.61 1.08.61h1.99l1.74 1c.19.11.41.17.62.17.21 0 .42-.06.61-.17l1.74-1h2.03c.44 0 .85-.23 1.08-.61l1.02-1.76 1.71-1c.38-.23.61-.64.61-1.08v-1.97l1.02-1.78c.27-.38.27-.85.04-1.25zm-5.08-.71l-5.01 5.01c-.18.18-.43.29-.71.29-.28 0-.53-.11-.71-.29l-3.01-3.01a1.003 1.003 0 011.42-1.42l2.3 2.3 4.31-4.3a1.003 1.003 0 011.71.71c0 .28-.12.53-.3.71z"],envelope:["M0 4.01v11.91l6.27-6.27L0 4.01zm18.91-1.03H1.09L10 10.97l8.91-7.99zm-5.18 6.66L20 15.92V4.01l-6.27 5.63zm-3.23 2.9c-.13.12-.31.19-.5.19s-.37-.07-.5-.19l-2.11-1.89-6.33 6.33h17.88l-6.33-6.33-2.11 1.89z"],equals:["M4 7h12a1 1 0 010 2H4a1 1 0 110-2zm0 4h12a1 1 0 010 2H4a1 1 0 010-2z"],eraser:["M18.71 8.43c.39-.4.39-1.05 0-1.45l-5.53-5.72a.967.967 0 00-1.4 0L1.29 12.1c-.39.4-.39 1.05 0 1.45l4.25 4.39 2.13 2.05h9.27c.02 0 .03.01.05.01.55 0 1-.45 1-1s-.45-1-1-1H9.46l.05-.05h.01l.81-.84 8.38-8.68zM7.52 17.94l-4.95-5.12 4.46-4.61 4.95 5.12-4.46 4.61z"],error:["M10 0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zm1 16H9v-2h2v2zm0-3H9V4h2v9z"],euro:["M8.89 4.47c.56-.31 1.23-.47 2.03-.47.44 0 .85.07 1.25.22.4.14.76.35 1.07.6.17.14.33.3.47.47l2.32-2.32c-.16-.15-.3-.32-.47-.46-.62-.49-1.33-.87-2.12-1.13-.8-.25-1.64-.38-2.52-.38-1.24 0-2.35.22-3.33.66-.99.44-1.82 1.05-2.49 1.82-.68.78-1.2 1.68-1.56 2.72-.09.26-.13.54-.2.8H2c-.55 0-1 .45-1 1s.45 1 1 1h1.04c-.01.2-.04.38-.04.58 0 .15.03.28.03.42H2c-.55 0-1 .45-1 1s.45 1 1 1h1.31c.07.3.13.6.23.89.36 1.02.88 1.92 1.56 2.67.68.76 1.51 1.35 2.49 1.79.98.43 2.09.65 3.33.65.99 0 1.9-.15 2.73-.46.83-.3 1.55-.74 2.17-1.32.03-.03.05-.06.08-.09l-2.41-2.15c-.01.01-.02.02-.02.03-.61.67-1.46 1-2.54 1-.8 0-1.47-.16-2.03-.47-.56-.31-1.01-.72-1.35-1.24-.28-.38-.47-.83-.63-1.3H12c.55 0 1-.45 1-1s-.45-1-1-1H6.56c0-.14-.02-.28-.02-.42 0-.2.02-.39.03-.58H13c.55 0 1-.45 1-1s-.45-1-1-1H6.94c.15-.46.34-.9.59-1.28.35-.52.8-.94 1.36-1.25zM18 11.38v0z"],exchange:["M2.5 8a2.5 2.5 0 000 5 2.5 2.5 0 000-5zm10.35 3.15a.495.495 0 10-.7.7L13.3 13H5.5c-.28 0-.5.22-.5.5s.22.5.5.5h7.79l-1.15 1.15c-.08.09-.14.21-.14.35a.495.495 0 00.85.35l2-2c.09-.09.15-.21.15-.35s-.06-.26-.15-.35l-2-2zM17.5 8a2.5 2.5 0 000 5 2.5 2.5 0 000-5zM7.15 9.85a.495.495 0 10.7-.7L6.71 8h7.79c.28 0 .5-.22.5-.5s-.22-.5-.5-.5H6.71l1.15-1.15c.08-.09.14-.21.14-.35a.495.495 0 00-.85-.35l-2 2c-.09.09-.15.21-.15.35s.06.26.15.35l2 2z"],"exclude-row":["M1 3h18c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1zM0 13a1.003 1.003 0 001.71.71L4 11.41l2.29 2.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71L5.41 10 7.7 7.71c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71L4 8.59l-2.29-2.3A1.003 1.003 0 00.29 7.71L2.59 10 .3 12.29c-.19.18-.3.43-.3.71zm18-5h-7c-1.1 0-2 .9-2 2s.9 2 2 2h7c1.1 0 2-.9 2-2s-.9-2-2-2zm1 9H1c-.55 0-1 .45-1 1s.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1z"],"expand-all":["M4 9c.28 0 .53-.11.71-.29L10 3.41l5.29 5.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71l-6-6C10.53 1.11 10.28 1 10 1s-.53.11-.71.29l-6 6A1.003 1.003 0 004 9zm12 2c-.28 0-.53.11-.71.29L10 16.59 4.71 11.3A.965.965 0 004 11a1.003 1.003 0 00-.71 1.71l6 6c.18.18.43.29.71.29s.53-.11.71-.29l6-6A1.003 1.003 0 0016 11z"],export:["M5 7c.28 0 .53-.11.71-.29L9 3.41V15c0 .55.45 1 1 1s1-.45 1-1V3.41l3.29 3.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71l-5-5C10.53.11 10.28 0 10 0s-.53.11-.71.29l-5 5A1.003 1.003 0 005 7zm14 7c-.55 0-1 .45-1 1v3H2v-3c0-.55-.45-1-1-1s-1 .45-1 1v4c0 .55.45 1 1 1h18c.55 0 1-.45 1-1v-4c0-.55-.45-1-1-1z"],"eye-off":["M20 9.96v-.03-.01-.02-.02a.794.794 0 00-.21-.43c-.55-.69-1.19-1.3-1.85-1.87l-3.93 2.62a3.966 3.966 0 01-3.96 3.77c-.47 0-.91-.1-1.33-.24l-2.24 1.49c.52.21 1.05.39 1.6.51 1.21.27 2.43.28 3.64.05 1.11-.21 2.17-.64 3.17-1.18 1.56-.84 2.99-2 4.23-3.3.23-.24.46-.49.67-.75a.87.87 0 00.21-.43v-.02-.02-.01-.03V10v-.04zm-.46-5.14c.27-.18.46-.47.46-.82 0-.55-.45-1-1-1-.21 0-.39.08-.54.18l-.01-.02L15 5.46c-.95-.53-1.95-.96-3.01-1.2a9.158 9.158 0 00-3.65-.04c-1.11.21-2.17.64-3.17 1.18-1.56.84-2.99 2-4.23 3.3-.23.24-.46.48-.67.75-.27.34-.27.76 0 1.1.64.79 1.39 1.5 2.16 2.15.26.21.52.41.79.61L.44 15.16l.01.02A1 1 0 000 16c0 .55.45 1 1 1 .21 0 .39-.08.54-.18l.01.02 18-12-.01-.02zm-8.67 3.4c-.25-.12-.53-.2-.83-.2-1.1 0-1.99.89-1.99 1.99 0 .03.02.06.02.09l-1.78 1.19c-.14-.4-.22-.83-.22-1.28 0-2.19 1.78-3.97 3.98-3.97 1.01 0 1.91.38 2.61 1l-1.79 1.18z"],"eye-on":["M13.3 8.71c.18.18.43.29.71.29s.53-.11.71-.29l4.99-5a1.003 1.003 0 00-1.42-1.42L14 6.58l-2.29-2.29a.956.956 0 00-.7-.29 1.003 1.003 0 00-.71 1.71l3 3zM20 9.96v-.03-.01-.02-.02a.823.823 0 00-.21-.44c-.44-.55-.94-1.05-1.46-1.52l-2.2 2.2c-.55.54-1.3.88-2.12.88-.05 0-.09-.01-.14-.01a3.978 3.978 0 01-3.86 3.02 4.007 4.007 0 01-1.66-7.65A2.97 2.97 0 018.02 5c0-.28.05-.54.12-.8-1.05.22-2.07.64-3.02 1.15-1.57.85-3 2.02-4.24 3.33-.23.25-.46.5-.67.76-.28.35-.28.77 0 1.12.64.8 1.4 1.52 2.17 2.17 1.66 1.41 3.56 2.58 5.66 3.06 1.21.27 2.43.29 3.65.05 1.11-.21 2.18-.65 3.18-1.19 1.57-.85 3-2.02 4.24-3.33.23-.24.46-.49.67-.76.11-.12.18-.27.21-.44v-.02-.02-.01-.03V10c.01-.01.01-.03.01-.04zm-9.99 2.05c1.03 0 1.87-.79 1.98-1.8l-.09-.09-.01.01-2.1-2.11c-1 .11-1.77.95-1.77 1.98-.01 1.11.89 2.01 1.99 2.01z"],"eye-open":["M10.01 7.984A2.008 2.008 0 008.012 9.99c0 1.103.9 2.006 1.998 2.006a2.008 2.008 0 001.998-2.006c0-1.103-.9-2.006-1.998-2.006zM20 9.96v-.03-.01-.02-.02a.827.827 0 00-.21-.442c-.64-.802-1.398-1.514-2.168-2.166-1.658-1.404-3.566-2.587-5.664-3.058a8.982 8.982 0 00-3.656-.05c-1.11.2-2.178.641-3.177 1.183-1.569.852-2.997 2.016-4.246 3.33-.23.25-.46.49-.67.761-.279.351-.279.773 0 1.124.64.802 1.4 1.514 2.169 2.166 1.658 1.404 3.566 2.577 5.664 3.058 1.209.271 2.438.281 3.656.05 1.11-.21 2.178-.651 3.177-1.193 1.569-.852 2.997-2.016 4.246-3.33.23-.24.46-.49.67-.751.11-.12.179-.271.209-.442v-.02-.02-.01-.03V10v-.04zM10.01 14A4.003 4.003 0 016.014 9.99a4.003 4.003 0 013.996-4.011 4.003 4.003 0 013.996 4.011 4.003 4.003 0 01-3.996 4.011z"],"fast-backward":["M18 3c-.23 0-.42.09-.59.21l-.01-.01L11 8V4c0-.55-.45-1-1-1-.23 0-.42.09-.59.21L9.4 3.2l-8 6 .01.01C1.17 9.4 1 9.67 1 10s.17.6.41.79l-.01.01 8 6 .01-.01c.17.12.36.21.59.21.55 0 1-.45 1-1v-4l6.4 4.8.01-.01c.17.12.36.21.59.21.55 0 1-.45 1-1V4c0-.55-.45-1-1-1z"],"fast-forward":["M19 10c0-.33-.17-.6-.41-.79l.01-.01-8-6-.01.01C10.42 3.09 10.23 3 10 3c-.55 0-1 .45-1 1v4L2.6 3.2l-.01.01C2.42 3.09 2.23 3 2 3c-.55 0-1 .45-1 1v12c0 .55.45 1 1 1 .23 0 .42-.09.59-.21l.01.01L9 12v4c0 .55.45 1 1 1 .23 0 .42-.09.59-.21l.01.01 8-6-.01-.01c.24-.19.41-.46.41-.79z"],feed:["M2.5 15a2.5 2.5 0 000 5 2.5 2.5 0 000-5zm.5-5c-.55 0-1 .45-1 1s.45 1 1 1c2.76 0 5 2.24 5 5 0 .55.45 1 1 1s1-.45 1-1c0-3.87-3.13-7-7-7zM3 0c-.55 0-1 .45-1 1s.45 1 1 1c8.28 0 15 6.72 15 15 0 .55.45 1 1 1s1-.45 1-1C20 7.61 12.39 0 3 0zm0 5c-.55 0-1 .45-1 1s.45 1 1 1c5.52 0 10 4.48 10 10 0 .55.45 1 1 1s1-.45 1-1C15 10.37 9.63 5 3 5z"],"feed-subscribed":["M2.5 15a2.5 2.5 0 000 5 2.5 2.5 0 000-5zM3 2c1.76 0 3.44.31 5.01.87.03-.71.31-1.35.75-1.85C6.96.37 5.03 0 3 0c-.55 0-1 .45-1 1s.45 1 1 1zm10.32 4.67a.99.99 0 001.4 0l4.98-4.98c.19-.17.3-.42.3-.7 0-.55-.45-1-1-1a.99.99 0 00-.7.29l-4.27 4.27-2.28-2.28a.99.99 0 00-.7-.29c-.55 0-.99.45-.99 1 0 .28.11.52.29.7l2.97 2.99zM3 10c-.55 0-1 .45-1 1s.45 1 1 1c2.76 0 5 2.24 5 5 0 .55.45 1 1 1s1-.45 1-1c0-3.87-3.13-7-7-7zm13.94-2.69l-.82.82-.02-.02c-.2.2-.42.37-.67.51A14.8 14.8 0 0118 17c0 .55.45 1 1 1s1-.45 1-1c0-3.61-1.14-6.94-3.06-9.69zM3 5c-.55 0-1 .45-1 1s.45 1 1 1c5.52 0 10 4.48 10 10 0 .55.45 1 1 1s1-.45 1-1C15 10.37 9.63 5 3 5z"],film:["M19 2h-5v3H6V2H1c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1h5v-3h8v3h5c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1zM4 17H2v-2h2v2zm0-3H2v-2h2v2zm0-3H2V9h2v2zm0-3H2V6h2v2zm0-3H2V3h2v2zm10 8H6V7h8v6zm4 4h-2v-2h2v2zm0-3h-2v-2h2v2zm0-3h-2V9h2v2zm0-3h-2V6h2v2zm0-3h-2V3h2v2z"],filter:["M18 1H2a1.003 1.003 0 00-.71 1.71L7 8.41V18a1.003 1.003 0 001.71.71l4-4c.18-.18.29-.43.29-.71V8.41l5.71-5.71c.18-.17.29-.42.29-.7 0-.55-.45-1-1-1z"],"filter-keep":["M15 2c0-.55-.45-1-1-1H1a1.003 1.003 0 00-.71 1.71L5 7.41V16a1.003 1.003 0 001.71.71l3-3c.18-.18.29-.43.29-.71V7.41l4.71-4.71c.18-.17.29-.42.29-.7zm4 11c-.28 0-.53.11-.71.29L15 16.59l-1.29-1.29A.965.965 0 0013 15a1.003 1.003 0 00-.71 1.71l2 2c.18.18.43.29.71.29s.53-.11.71-.29l4-4A1.003 1.003 0 0019 13z"],"filter-list":["M15 2c0-.55-.45-1-1-1H1a1.003 1.003 0 00-.71 1.71L5 7.41V16a1.003 1.003 0 001.71.71l3-3c.18-.18.29-.43.29-.71V7.41l4.71-4.71c.18-.17.29-.42.29-.7zm-4 8c0 .55.45 1 1 1h7c.55 0 1-.45 1-1s-.45-1-1-1h-7c-.55 0-1 .45-1 1zm8 7h-7c-.55 0-1 .45-1 1s.45 1 1 1h7c.55 0 1-.45 1-1s-.45-1-1-1zm0-4h-7c-.55 0-1 .45-1 1s.45 1 1 1h7c.55 0 1-.45 1-1s-.45-1-1-1z"],"filter-open":["M15 2c0 .28-.11.53-.29.7L10 7.41V13c0 .28-.11.53-.29.71l-3 3A1.003 1.003 0 015 16V7.41L.29 2.71A1.003 1.003 0 011 1h13c.55 0 1 .45 1 1zm4.707 11.293a1 1 0 010 1.414l-4 4c-.63.63-1.707.184-1.707-.707v-8c0-.89 1.077-1.337 1.707-.707l4 4z"],"filter-remove":["M15 2c0-.55-.45-1-1-1H1a1.003 1.003 0 00-.71 1.71L5 7.41V16a1.003 1.003 0 001.71.71l3-3c.18-.18.29-.43.29-.71V7.41l4.71-4.71c.18-.17.29-.42.29-.7zm2.91 13.5l1.79-1.79c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71l-1.79 1.79-1.79-1.79a1.003 1.003 0 00-1.42 1.42l1.79 1.79-1.79 1.79a1.003 1.003 0 001.42 1.42l1.79-1.79 1.79 1.79a1.003 1.003 0 001.42-1.42l-1.8-1.79z"],flag:["M3 3c-.55 0-1 .45-1 1v15c0 .55.45 1 1 1s1-.45 1-1V4c0-.55-.45-1-1-1zm0-3c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm2 4.08v8.28c3.01-2.41 8.67 2.64 13 0V4.08C13.61 7.14 8.01 1 5 4.08z"],flame:["M11.622 0c0 1.71.49 3.077 1.472 4.103C16.364 6.496 18 9.23 18 12.308c0 3.418-1.962 5.983-5.887 7.692 2.887-3 2.453-4.23-.49-8C8.5 13.5 9 14.5 9.5 16.5c-1.048 0-2 0-2.5-.5 0 .684 1.197 2.5 1.952 4-3.924-1.026-8.123-7.18-6.651-7.692.981-.342 2.126-.171 3.434.513C4.1 6.667 6.062 2.393 11.622 0z"],flash:["M4.96 6.37a1.003 1.003 0 001.42-1.42l-2-2a1.07 1.07 0 00-.71-.28 1.003 1.003 0 00-.71 1.71l2 1.99zm9.37.3c.28 0 .53-.11.71-.29l2-2a1.003 1.003 0 00-1.42-1.42l-2 2a1.003 1.003 0 00.71 1.71zM10 5c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1S9 .45 9 1v3c0 .55.45 1 1 1zm-5 5c0-.55-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1h3c.55 0 1-.45 1-1zm14-1h-3c-.55 0-1 .45-1 1s.45 1 1 1h3c.55 0 1-.45 1-1s-.45-1-1-1zm-9-3c-2.21 0-4 1.79-4 4s1.79 4 4 4 4-1.79 4-4-1.79-4-4-4zm0 6c-1.1 0-2-.9-2-2s.9-2 2-2 2 .9 2 2-.9 2-2 2zm5.04 1.63a1.003 1.003 0 00-1.42 1.42l2 2a1.003 1.003 0 001.42-1.42l-2-2zM10 15c-.55 0-1 .45-1 1v3c0 .55.45 1 1 1s1-.45 1-1v-3c0-.55-.45-1-1-1zm-4.33-1.67c-.28 0-.53.11-.71.29l-2 2a1.003 1.003 0 001.42 1.42l2-2a1.003 1.003 0 00-.71-1.71z"],"floppy-disk":["M14 1h-3v5h3V1zm5.71 2.29l-3-3A.997.997 0 0016 0h-1v7H5V0H1C.45 0 0 .45 0 1v18c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V4c0-.28-.11-.53-.29-.71zM17 19H3v-8c0-.55.45-1 1-1h12c.55 0 1 .45 1 1v8z"],"flow-branch":["M14.425 7.953a3.98 3.98 0 01.562 2.045 3.98 3.98 0 01-.583 2.08L18 15.671V12.98c0-.248.097-.496.29-.689.379-.379 1.047-.38 1.426 0a.94.94 0 01.283.696l-.001 5.049a.957.957 0 01-.276.69.955.955 0 01-.69.273h-5.059a.971.971 0 01-.689-.289 1.026 1.026 0 010-1.417.972.972 0 01.69-.29h2.702l-3.634-3.573a3.998 3.998 0 01-5.924-2.431H1a1 1 0 010-2h6.12a3.998 3.998 0 015.96-2.409L16.665 3l-2.694-.001a.972.972 0 01-.689-.29 1.035 1.035 0 010-1.425.94.94 0 01.696-.283l5.05.001c.248 0 .497.083.69.276a.954.954 0 01.272.69l.001 5.052a.971.971 0 01-.29.689 1.028 1.028 0 01-1.419 0 .972.972 0 01-.29-.69V4.323l-3.567 3.63z"],"flow-end":["M12 9.919a3.998 3.998 0 014-3.92c2.21 0 4 1.79 4 3.997a3.998 3.998 0 01-4 3.996 3.998 3.998 0 01-4-3.916.967.967 0 01-.28.612L7.685 14.71a.958.958 0 01-.686.285c-.536 0-.994-.461-.994-.997 0-.273.107-.528.283-.704l2.379-2.302H.98c-.537 0-.976-.46-.976-.996s.44-.992.976-.992h7.676L6.287 6.687a.957.957 0 01-.283-.686c0-.536.458-.996.994-.996.274 0 .51.1.686.285l4.027 4.024c.159.158.27.365.29.605z"],"flow-linear":["M5.125 10.997H.976C.439 10.997 0 10.537 0 10c0-.536.44-.993.976-.993h4.148a4.002 4.002 0 017.752 0h3.776L14.293 6.69a.962.962 0 01-.285-.687c0-.537.46-1.001.996-1.001a.96.96 0 01.698.3l4.005 4.015c.176.176.293.41.293.683a.972.972 0 01-.283.693L15.702 14.7a.997.997 0 01-.698.297c-.537 0-.996-.453-.996-.99 0-.273.107-.517.283-.692l2.371-2.318h-3.787a4.002 4.002 0 01-7.75 0z"],"flow-review":["M6.13 9.004A4.005 4.005 0 0110.012 6c1.87 0 3.44 1.278 3.881 3.005h2.768l-2.354-2.317a.97.97 0 01-.283-.691c0-.536.462-.995 1-.995.273 0 .517.107.693.283l4 4.041a.97.97 0 01.284.692.956.956 0 01-.293.682l-3.991 3.997a.944.944 0 01-.694.292c-.537 0-1-.46-1-.997a.97.97 0 01.284-.692l2.345-2.29h-2.765a4.005 4.005 0 01-3.875 2.981 4.005 4.005 0 01-3.874-2.981H3.349l2.376 2.308a.97.97 0 01.283.691 1 1 0 01-.994.983.989.989 0 01-.713-.291L.293 10.699A.956.956 0 010 10.017a.97.97 0 01.283-.692l4.03-4.037a.996.996 0 01.701-.283c.537 0 .994.464.994 1a.97.97 0 01-.283.691L3.34 9.004h2.79z"],"flow-review-branch":["M13.04 13.424c-.6.36-1.302.568-2.052.568a4 4 0 01-3.868-2.999H3.342l2.372 2.31c.176.176.283.42.283.694 0 .537-.452.998-.988.998a.935.935 0 01-.691-.289L.292 10.683A.96.96 0 010 9.999c0-.274.107-.518.283-.694l4.035-4.04a.973.973 0 01.691-.288c.536 0 .988.47.988 1.007a.975.975 0 01-.283.694L3.332 8.984h3.786a4 4 0 013.87-3.006c.771 0 1.492.22 2.102.599l3.565-3.57-2.538-.003a.974.974 0 01-.69-.29c-.38-.38-.38-1.052-.002-1.431A.94.94 0 0114.122 1l4.896.005a.96.96 0 01.69.277c.193.193.27.442.27.69l.005 4.9a.971.971 0 01-.289.69 1.023 1.023 0 01-1.416 0 .975.975 0 01-.29-.691l-.003-2.54-3.554 3.62c.351.596.553 1.291.553 2.034 0 .763-.213 1.477-.583 2.084l3.595 3.595.003-2.54c0-.249.097-.497.29-.69.38-.38 1.05-.381 1.429-.002a.94.94 0 01.282.697l-.005 4.9a.927.927 0 01-.277.675.974.974 0 01-.69.291L13.974 19a.97.97 0 01-.69-.29 1.03 1.03 0 01.002-1.42.974.974 0 01.69-.29l2.696-.003-3.632-3.573z"],flows:["M17.5 7.93a2.5 2.5 0 00-2.45 2h-2.3l-4.01-4-.75.75 3.26 3.25h-6.3a2.5 2.5 0 100 1h6.3l-3.26 3.25.75.75 4.01-4h2.3a2.5 2.5 0 102.45-3z"],"folder-close":["M0 17c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V7H0v10zM19 4H9.41l-1.7-1.71A.997.997 0 007 2H1c-.55 0-1 .45-1 1v3h20V5c0-.55-.45-1-1-1z"],"folder-new":["M12.994 7c0 1.655 1.344 3 2.998 3a3.002 3.002 0 002.999-3H20v10c0 .55-.45 1-1 1H1.01c-.55 0-1-.45-1-1V7h12.984zM10.76 6H0V3c0-.55.45-1 1-1h3.998c.28 0 .53.11.71.29L7.415 4h2.579c0 .768.29 1.469.765 2zm8.23-3c.55 0 1 .45 1 1s-.45 1-1 1h-1.998v2c0 .55-.45 1-1 1s-1-.45-1-1V5h-1.998c-.55 0-1-.45-1-1s.45-1 1-1h1.999V1c0-.55.45-1 .999-1 .55 0 1 .45 1 1v2h1.999z"],"folder-open":["M20 9c0-.55-.45-1-1-1H5c-.43 0-.79.27-.93.65h-.01l-3 8h.01c-.04.11-.07.23-.07.35 0 .55.45 1 1 1h14c.43 0 .79-.27.93-.65h.01l3-8h-.01c.04-.11.07-.23.07-.35zM3.07 7.63C3.22 7.26 3.58 7 4 7h14V5c0-.55-.45-1-1-1H8.41l-1.7-1.71A.997.997 0 006 2H1c-.55 0-1 .45-1 1v12.31l3.07-7.68z"],"folder-shared":["M11 4H9.41l-1.7-1.71A.997.997 0 007 2H1c-.55 0-1 .45-1 1v3h11.78C11.3 5.47 11 4.77 11 4zm8-1h-5c-.55 0-1 .45-1 1s.45 1 1 1h2.59L12.3 9.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71L18 6.41V9c0 .55.45 1 1 1s1-.45 1-1V4c0-.55-.45-1-1-1zm-2.46 7.7l-1.42 1.42a2.996 2.996 0 11-4.24-4.24l.88-.88H0v10c0 .55.45 1 1 1h18c.55 0 1-.45 1-1v-5.18c-.31.11-.65.18-1 .18-1.02 0-1.92-.52-2.46-1.3z"],"folder-shared-open":["M3.07 7.63C3.22 7.26 3.58 7 4 7h7.76l.54-.54A2.97 2.97 0 0111 4H8.41l-1.7-1.71A.997.997 0 006 2H1c-.55 0-1 .45-1 1v12.31l3.07-7.68zm13.47 3.07l-1.42 1.42A2.996 2.996 0 0110 10c0-.77.3-1.47.78-2H5c-.43 0-.79.27-.93.65h-.01l-3 8h.01c-.04.11-.07.23-.07.35 0 .55.45 1 1 1h14c.43 0 .79-.27.93-.65h.01l2.01-5.36c-1-.01-1.88-.52-2.41-1.29zM19 3h-5c-.55 0-1 .45-1 1s.45 1 1 1h2.59L12.3 9.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71L18 6.41V9c0 .55.45 1 1 1s1-.45 1-1V4c0-.55-.45-1-1-1z"],follower:["M11.54 15.92c-1.48-.65-1.28-1.05-1.33-1.59-.01-.07-.01-.15-.01-.23.51-.45.92-1.07 1.19-1.78 0 0 .01-.04.02-.05.06-.15.11-.32.15-.48.34-.07.54-.44.61-.78.08-.14.23-.48.2-.87-.05-.5-.25-.73-.47-.82v-.09c0-.63-.06-1.55-.17-2.15-.02-.17-.06-.33-.11-.5a3.69 3.69 0 00-1.29-1.86C9.69 4.25 8.8 4 8.01 4c-.8 0-1.69.25-2.32.73-.61.47-1.06 1.13-1.28 1.86-.05.17-.09.33-.11.5-.12.6-.18 1.51-.18 2.14v.08c-.23.09-.44.32-.49.83-.04.39.12.73.2.87.08.35.28.72.63.78.04.17.09.33.15.48 0 .01.01.02.01.03l.01.01c.27.72.7 1.35 1.22 1.8 0 .07-.01.14-.01.21-.05.54.1.94-1.38 1.59-1.48.65-3.71 1.35-4.16 2.4C-.16 19.38.02 20 .02 20h15.95s.18-.62-.27-1.67c-.46-1.06-2.68-1.75-4.16-2.41zm8.15-12.63l-3-3a.956.956 0 00-.7-.29 1.003 1.003 0 00-.71 1.71L16.58 3H13c-.55 0-1 .45-1 1s.45 1 1 1h3.58l-1.29 1.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71l3-3c.18-.18.29-.43.29-.71 0-.28-.11-.53-.3-.71z"],following:["M11.55 15.92c-1.48-.65-1.28-1.05-1.33-1.59-.01-.07-.01-.15-.01-.23.51-.45.92-1.07 1.19-1.78 0 0 .01-.04.02-.05.06-.15.11-.32.15-.48.34-.07.54-.44.61-.78.08-.14.23-.48.2-.87-.05-.5-.25-.73-.47-.82v-.09c0-.63-.06-1.55-.17-2.15-.02-.17-.06-.33-.11-.5a3.69 3.69 0 00-1.29-1.86C9.7 4.25 8.81 4 8.02 4c-.79 0-1.68.25-2.31.73-.61.47-1.07 1.13-1.29 1.86-.05.16-.09.33-.11.5-.12.6-.18 1.51-.18 2.14v.08c-.23.09-.44.32-.48.83-.04.39.12.73.2.87.08.35.28.72.63.78.04.17.09.33.15.48 0 .01.01.02.01.03l.01.01c.27.72.7 1.35 1.22 1.8 0 .07-.01.14-.01.21-.05.54.1.94-1.38 1.59C3 16.56.77 17.26.32 18.31-.15 19.38.04 20 .04 20h15.95s.18-.62-.27-1.67c-.46-1.06-2.69-1.75-4.17-2.41zM19 3h-3.58l1.29-1.29A1.003 1.003 0 0015.29.29l-3 3c-.17.18-.28.43-.28.71 0 .28.11.53.29.71l3 3c.18.18.43.29.7.29a1.003 1.003 0 00.71-1.71L15.42 5H19c.55 0 1-.45 1-1s-.45-1-1-1z"],font:["M17.93 18.64l-7-18C10.78.27 10.42 0 10 0s-.78.27-.93.64l-7 18c-.04.11-.07.23-.07.36 0 .55.45 1 1 1 .42 0 .78-.27.93-.64L6.41 13h7.19l2.47 6.36c.15.37.51.64.93.64.55 0 1-.45 1-1 0-.13-.03-.25-.07-.36zM7.18 11L10 3.76 12.82 11H7.18z"],fork:["M16.71 11.29a1.003 1.003 0 00-1.42 1.42l1.3 1.29h-2.17l-8-8h10.17L15.3 7.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71l3-3c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71l-3-3a1.003 1.003 0 00-1.42 1.42L16.59 4H1c-.55 0-1 .45-1 1s.45 1 1 1h2.59l9.71 9.71c.17.18.42.29.7.29h2.59l-1.29 1.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71l3-3c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71l-3-3z"],form:["M2 13v4h4v-4H2zm-1-2h6c.55 0 1 .45 1 1v6c0 .55-.45 1-1 1H1c-.55 0-1-.45-1-1v-6c0-.55.45-1 1-1zm11-7h7c.55 0 1 .45 1 1s-.45 1-1 1h-7c-.55 0-1-.45-1-1s.45-1 1-1zM8 1a1.003 1.003 0 01.71 1.71l-5 6C3.53 8.89 3.28 9 3 9s-.53-.11-.71-.29l-2-2a1.003 1.003 0 011.42-1.42L3 6.59l4.29-5.3C7.47 1.11 7.72 1 8 1zm4 13h7c.55 0 1 .45 1 1s-.45 1-1 1h-7c-.55 0-1-.45-1-1s.45-1 1-1z"],"full-circle":["M9.96 0a10 10 0 100 20 10 10 0 100-20z"],"full-stacked-chart":["M15 16h2c.55 0 1-.45 1-1v-5h-4v5c0 .55.45 1 1 1zM12 2c0-.55-.45-1-1-1H9c-.55 0-1 .45-1 1v4h4V2zm6 4h-4v3h4V6zm0-4c0-.55-.45-1-1-1h-2c-.55 0-1 .45-1 1v3h4V2zm-6 5H8v5h4V7zm-9 9h2c.55 0 1-.45 1-1v-3H2v3c0 .55.45 1 1 1zm6 0h2c.55 0 1-.45 1-1v-2H8v2c0 .55.45 1 1 1zm10 1H1c-.55 0-1 .45-1 1s.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1zM6 2c0-.55-.45-1-1-1H3c-.55 0-1 .45-1 1v3h4V2zm0 4H2v5h4V6z"],fullscreen:["M3.41 2H6c.55 0 1-.45 1-1s-.45-1-1-1H1C.45 0 0 .45 0 1v5c0 .55.45 1 1 1s1-.45 1-1V3.41L7.29 8.7c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71L3.41 2zM8 11c-.28 0-.53.11-.71.29L2 16.59V14c0-.55-.45-1-1-1s-1 .45-1 1v5c0 .55.45 1 1 1h5c.55 0 1-.45 1-1s-.45-1-1-1H3.41l5.29-5.29c.19-.18.3-.43.3-.71 0-.55-.45-1-1-1zM19 0h-5c-.55 0-1 .45-1 1s.45 1 1 1h2.59L11.3 7.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71L18 3.41V6c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1zm0 13c-.55 0-1 .45-1 1v2.59l-5.29-5.29A.965.965 0 0012 11a1.003 1.003 0 00-.71 1.71l5.3 5.29H14c-.55 0-1 .45-1 1s.45 1 1 1h5c.55 0 1-.45 1-1v-5c0-.55-.45-1-1-1z"],function:["M10.14 5.82H8.73c.4-1.66.94-2.87 1.6-3.64.4-.48.8-.72 1.18-.72.08 0 .14.02.19.07.05.05.07.1.07.18 0 .07-.05.19-.16.37s-.16.36-.16.52c0 .23.08.43.25.59a.9.9 0 00.64.25c.28 0 .51-.1.7-.3.19-.2.28-.47.28-.81 0-.39-.14-.7-.42-.94-.28-.24-.74-.36-1.36-.36-.97 0-1.86.29-2.65.87-.79.56-1.54 1.52-2.26 2.85-.24.46-.48.75-.7.88-.22.13-.57.19-1.06.19l-.32 1.15H5.9l-1.99 7.85c-.33 1.29-.56 2.09-.67 2.39-.17.44-.43.81-.77 1.12a.74.74 0 01-.5.19c-.05 0-.1-.02-.14-.05l-.04-.07c0-.03.05-.1.15-.2.1-.1.15-.26.15-.47 0-.23-.08-.42-.23-.57-.16-.15-.38-.23-.67-.23-.35 0-.63.1-.85.29-.21.2-.32.43-.32.7 0 .29.13.54.39.75.25.22.65.33 1.2.33.88 0 1.66-.23 2.33-.69.68-.46 1.27-1.17 1.78-2.14.51-.96 1.03-2.52 1.56-4.66l1.14-4.54H9.8l.34-1.15zm6.8 1.95c.25-.2.51-.29.78-.29.1 0 .29.04.56.11.27.08.51.11.72.11.29 0 .52-.1.72-.3.18-.19.28-.45.28-.77 0-.33-.1-.6-.29-.8-.19-.2-.47-.29-.82-.29-.32 0-.62.08-.9.23-.28.15-.64.49-1.08 1-.33.38-.81 1.05-1.44 2a9.712 9.712 0 00-1.31-3.22l-3.4.59-.07.37c.25-.05.47-.08.64-.08.34 0 .62.15.84.44.35.46.84 1.85 1.46 4.19-.49.66-.82 1.09-1 1.3-.3.33-.55.54-.74.64-.15.08-.32.12-.51.12-.14 0-.38-.08-.7-.24-.22-.1-.42-.16-.59-.16-.33 0-.6.11-.82.32-.21.22-.32.49-.32.83 0 .31.1.57.3.77.2.2.47.29.8.29.32 0 .63-.07.92-.21.29-.14.64-.43 1.08-.88.43-.45 1.03-1.16 1.79-2.14.29.93.55 1.61.76 2.03.21.42.46.73.74.91.28.19.62.28 1.04.28.4 0 .81-.15 1.23-.44.55-.38 1.1-1.04 1.68-1.97l-.35-.21c-.39.55-.68.89-.87 1.03-.12.09-.27.13-.44.13-.2 0-.4-.13-.59-.38-.33-.43-.77-1.63-1.33-3.6.47-.86.89-1.44 1.23-1.71z"],"gantt-chart":["M4 7h5c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1H4c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1zm3 2v1c0 .55.45 1 1 1h6c.55 0 1-.45 1-1V9c0-.55-.45-1-1-1H8c-.55 0-1 .45-1 1zm12 3h-6c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h6c.55 0 1-.45 1-1v-1c0-.55-.45-1-1-1zm0 4H2V3c0-.55-.45-1-1-1s-1 .45-1 1v14c0 .55.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1z"],geofence:["M8 11l.075.003.126.017.111.03.111.044.098.052.096.067.09.08c.036.035.068.073.097.112l.071.11.054.114.035.105.03.148L9 12V18a1 1 0 01-1.993.117L7 18v-3.586l-5.293 5.293a1 1 0 01-1.497-1.32l.083-.094L5.584 13h-3.58a1 1 0 01-.117-1.993L2.004 11H8zm3.018-11a1.003 1.003 0 01.39.087l.12.063.031.02.1.078 8.027 7.026.062.064.068.086.044.068.064.128.04.117.024.113.011.108v.1l-.007.073-.019.103-.037.121-.039.09-.05.087-4.996 7.994c-.06.097-.137.183-.226.254l-.093.067-.095.053-.087.037-.125.037a1.018 1.018 0 01-.218.026H11v-5a3 3 0 00-2.824-2.995L8 9H3V6a1 1 0 01.321-.734l.098-.08 7-5a1.01 1.01 0 01.45-.178L11.018 0z"],geolocation:["M0 8.33l9.17 2.5 2.5 9.17L20 0z"],geosearch:["M8 18.88c-3.79 0-6.88-3.09-6.88-6.88 0-.61.08-1.22.23-1.79.03.01.06-.01.1-.01h.09v.55c0 .23.21.42.44.42.04 0 .09-.01.12-.02l.9.88c.09.09.23.09.32 0s.09-.23 0-.32l-.86-.9c0-.02.05-.04.05-.07v-.13c0-.18.1-.25.29-.41h.53c.1 0 .19-.01.27-.05.01-.01.02 0 .03-.01.02-.01.03-.02.05-.04.01-.01.02-.01.02-.02l.02-.02 1.13-1.13c-.16-.32-.3-.65-.42-.99h-.64v-.53c0-.01.06.06.06-.1h.38c-.04-.16-.08-.32-.1-.48h-.71c.2-.16.42-.31.64-.45C4.02 6.09 4 5.8 4 5.5c0-.14.01-.28.02-.43C1.62 6.46 0 9.04 0 12c0 4.41 3.59 8 8 8 3.87 0 7.09-2.77 7.82-6.44l-.97-1.1c-.26 3.57-3.23 6.42-6.85 6.42zm-2.12-3.67v-.35h.15c.29 0 .49-.23.49-.53v-.68c0-.01.01-.01 0-.02L4.71 11.8h-.77c-.29 0-.47.24-.47.53v2c0 .29.18.53.47.53h.33v2.02c0 .28.28.51.56.51s.56-.23.56-.51v-1.22h-.01c.29 0 .5-.16.5-.45zm13.83-2.92l-3.68-3.68c.14-.21.27-.42.38-.65.02-.04.04-.07.05-.11.11-.22.2-.45.28-.69v-.01c.07-.24.13-.48.17-.73l.03-.17c.04-.24.06-.49.06-.75C17 2.46 14.54 0 11.5 0S6 2.46 6 5.5 8.46 11 11.5 11c.26 0 .51-.02.76-.06l.17-.03c.25-.04.49-.1.73-.17h.01c.24-.08.47-.17.69-.28.04-.02.07-.04.11-.05.23-.11.44-.24.65-.38l3.68 3.68c.17.18.42.29.7.29a1.003 1.003 0 00.71-1.71zM11.5 9.5c-2.21 0-4-1.79-4-4s1.79-4 4-4 4 1.79 4 4-1.79 4-4 4zm1.93 5.33v-.65c0-.11-.13-.21-.24-.21-.11 0-.24.09-.24.21v.65c0 .11.13.21.24.21.11 0 .24-.1.24-.21zm-2.41.67h.83c.29 0 .46-.21.46-.5v-1.86l.23-.22c-.34.05-.69.08-1.04.08-.36 0-.7-.03-1.05-.08.03.05.06.1.08.16V15c.01.29.2.5.49.5z"],"git-branch":["M15 2c-1.66 0-3 1.34-3 3 0 1.3.84 2.4 2 2.82V9c0 1.1-.9 2-2 2H8c-.73 0-1.41.21-2 .55V5.82C7.16 5.4 8 4.3 8 3c0-1.66-1.34-3-3-3S2 1.34 2 3c0 1.3.84 2.4 2 2.82v8.37C2.84 14.6 2 15.7 2 17c0 1.66 1.34 3 3 3s3-1.34 3-3c0-1.25-.77-2.3-1.85-2.75C6.45 13.52 7.16 13 8 13h4c2.21 0 4-1.79 4-4V7.82C17.16 7.4 18 6.3 18 5c0-1.66-1.34-3-3-3zM5 2c.55 0 1 .45 1 1s-.45 1-1 1-1-.45-1-1 .45-1 1-1zm0 16c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zM15 6c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1z"],"git-commit":["M19 9h-4.1a5 5 0 00-9.8 0H1c-.55 0-1 .45-1 1s.45 1 1 1h4.1a5 5 0 009.8 0H19c.55 0 1-.45 1-1s-.45-1-1-1zm-9 4c-1.66 0-3-1.34-3-3s1.34-3 3-3 3 1.34 3 3-1.34 3-3 3z"],"git-merge":["M15 8c-1.3 0-2.4.84-2.82 2H11c-2.49 0-4.54-1.83-4.92-4.21A2.995 2.995 0 005 0C3.34 0 2 1.34 2 3c0 1.3.84 2.4 2 2.81v8.37C2.84 14.6 2 15.7 2 17c0 1.66 1.34 3 3 3s3-1.34 3-3c0-1.3-.84-2.4-2-2.82V9.86C7.27 11.17 9.03 12 11 12h1.18A2.996 2.996 0 0018 11c0-1.66-1.34-3-3-3zM5 18c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zM5 4c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zm10 8c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1z"],"git-new-branch":["M17 3h-1V2c0-.55-.45-1-1-1s-1 .45-1 1v1h-1c-.55 0-1 .45-1 1s.45 1 1 1h1v1c0 .55.45 1 1 1s1-.45 1-1V5h1c.55 0 1-.45 1-1s-.45-1-1-1zm-3 4.86V9c0 1.1-.9 2-2 2H8c-.73 0-1.41.21-2 .55V5.82C7.16 5.4 8 4.3 8 3c0-1.66-1.34-3-3-3S2 1.34 2 3c0 1.3.84 2.4 2 2.82v8.37C2.84 14.6 2 15.7 2 17c0 1.66 1.34 3 3 3s3-1.34 3-3c0-1.25-.77-2.3-1.85-2.75C6.45 13.52 7.16 13 8 13h4c2.21 0 4-1.79 4-4V7.86c-.32.08-.65.14-1 .14s-.68-.06-1-.14zM5 2c.55 0 1 .45 1 1s-.45 1-1 1-1-.45-1-1 .45-1 1-1zm0 16c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1z"],"git-pull":["M17 14.18V7c0-2.21-1.79-4-4-4h-2.59l1.29-1.29c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71l-3 3C7.11 3.47 7 3.72 7 4c0 .28.11.53.29.71l3 3a1.003 1.003 0 001.42-1.42L10.41 5H13c1.1 0 2 .9 2 2v7.18A2.996 2.996 0 0016 20c1.66 0 3-1.34 3-3 0-1.3-.84-2.4-2-2.82zM16 18c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zM4 1C2.34 1 1 2.34 1 4c0 1.3.84 2.4 2 2.82v7.37C1.84 14.6 1 15.7 1 17c0 1.66 1.34 3 3 3s3-1.34 3-3c0-1.3-.84-2.4-2-2.82V6.82C6.16 6.4 7 5.3 7 4c0-1.66-1.34-3-3-3zm0 17c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zM4 5c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1z"],"git-push":["M15 11c0-.28-.11-.53-.29-.71l-3-3C11.53 7.11 11.28 7 11 7s-.53.11-.71.29l-3 3a1.003 1.003 0 001.42 1.42l1.29-1.3V19c0 .55.45 1 1 1s1-.45 1-1v-8.59l1.29 1.29c.18.19.43.3.71.3.55 0 1-.45 1-1zm4-11H1C.45 0 0 .45 0 1v16c0 .55.45 1 1 1h7v-2H2v-2h6v-1H4V2h14v11h-4v1h4v2h-4v2h5c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zM5 8h2V6H5v2zm2-5H5v2h2V3z"],"git-repo":["M7 3H5v2h2V3zm0 6H5v2h2V9zm0-3H5v2h2V6zm12-6H1C.45 0 0 .45 0 1v16c0 .55.45 1 1 1h4v2l2-1 2 1v-2h10c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 16H9v-1H5v1H2v-2h16v2zm0-3H4V2h14v11z"],glass:["M17 6V0H3v6c0 3.53 2.61 6.43 6 6.92V18H6c-.55 0-1 .45-1 1s.45 1 1 1h8c.55 0 1-.45 1-1s-.45-1-1-1h-3v-5.08c3.39-.49 6-3.39 6-6.92z"],globe:["M7.53 4.37c.1-.1.1-.26 0-.35l-.68-.68c-.1-.1-.25-.1-.35 0-.1.1-.1.26 0 .35l.68.68c.1.1.25.1.35 0zm3.17.06h.3c.09 0 .16-.01.16-.1 0-.09-.07-.1-.16-.1h-.3c-.09 0-.16.01-.16.1s.07.1.16.1zm.98 1.15c.09 0 .19-.08.19-.17v-.42c0-.09-.1-.17-.19-.17s-.19.08-.19.17v.42c0 .09.1.17.19.17zm-6.5 4.19c-.35 0-.56.28-.56.63v2.37c0 .35.21.62.56.62h.39v2.4c0 .34.33.61.67.61s.67-.27.67-.61v-1.44h-.02c.35 0 .6-.19.6-.54v-.41h.18c.35 0 .58-.28.58-.62v-.81c0-.01.01-.01 0-.02L6.1 9.77h-.92zM10 0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zm0 18c-4.42 0-8-3.58-8-8 0-.74.11-1.46.3-2.14h.03v.65c0 .28.25.5.53.5.05 0 .1-.01.15-.02l1.05 1.05c.1.11.28.11.38 0 .1-.1.11-.27 0-.38L3.42 8.59c0-.03.05-.05.05-.08v-.16c0-.22.12-.3.34-.49h.63c.12 0 .23-.01.32-.07.01-.01.02 0 .03-.01.02-.02.04-.03.06-.04.01-.01.02-.01.03-.02l.02-.02 2.15-2.15c.24-.24.24-.63 0-.86-.23-.24-.62-.19-.86.04l-.41.46H5v-.64c0-.01.07.07.07-.12h.87c.17 0 .3-.12.3-.29 0-.17-.13-.29-.3-.29H4.88C6.27 2.7 8.05 2 10 2s3.73.7 5.12 1.86h-1.58l-.01-.04c-.06 0-.12 0-.17.04l-.71.7c-.09.09-.09.23 0 .31.09.09.23.09.32 0l.56-.6.01-.03h.34c0 .19-.1.13-.1.16v.1c0 .29-.2.5-.49.5h-.51c-.25 0-.52.28-.52.54v.23h-.12c-.16 0-.27.08-.27.24v.33h-.32c-.23 0-.41.15-.41.38 0 .22.18.35.41.35.1 0 .19.04.26-.16l.06.01.66-.59h.23l.53.5c.04.04.11.03.16-.01.04-.04.04-.16 0-.2L13 6.15h.32l.12.16c.25.25.65.23.89-.02l.12-.14H15c.02 0 .11.07.11.07v.33s-.06-.01-.07-.01h-.49c-.16 0-.28.13-.28.29 0 .16.13.29.28.29h.49c.01 0 .07-.01.07-.01v.2c-.19.28-.33.57-.62.57h-1.28s0-.01-.01-.01l-.58-.58a.622.622 0 00-.89 0l-.58.58s0 .01-.01.01h-.34c-.35 0-.67.28-.67.63v1.25c0 .35.32.61.67.61h1.22c.46.19.78.48.97.94v2.28c0 .35.23.6.58.6h.98c.35 0 .54-.25.54-.6v-2.2l1.21-1.17.04-.02.02-.01h.04c.1-.11.2-.26.2-.42V8.49c0-.25-.22-.44-.42-.63h.58c.02.38.29.57.63.57h.43c.13.51.18 1.03.18 1.57 0 4.42-3.58 8-8 8zm6.16-5.65c-.14 0-.29.11-.29.25v.77c0 .14.15.25.29.25.14 0 .29-.11.29-.25v-.77c0-.14-.15-.25-.29-.25zM10.5 3.48c0-.34-.28-.57-.62-.57h-.74c-.34 0-.57.25-.57.59 0 .05-.13.06.06.1v.64c0 .2.09.36.29.36.2 0 .29-.16.29-.36v-.19h.68c.33 0 .61-.23.61-.57z"],"globe-network":["M10 0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zm7.39 7h-3.63c-.31-1.99-.92-3.66-1.72-4.73 2.45.65 4.41 2.42 5.35 4.73zM13 10c0 .69-.04 1.36-.11 2H7.11a18.419 18.419 0 010-4h5.77c.08.64.12 1.31.12 2zm-3-8c1.07 0 2.25 2.05 2.75 5h-5.5c.5-2.95 1.68-5 2.75-5zm-2.04.27C7.16 3.34 6.55 5.01 6.24 7H2.61c.94-2.31 2.9-4.08 5.35-4.73zM2 10c0-.69.11-1.36.28-2h3.83a18.419 18.419 0 000 4H2.28c-.17-.64-.28-1.31-.28-2zm.61 3h3.63c.31 1.99.92 3.66 1.72 4.73A7.996 7.996 0 012.61 13zM10 18c-1.07 0-2.25-2.05-2.75-5h5.5c-.5 2.95-1.68 5-2.75 5zm2.04-.27c.79-1.07 1.4-2.74 1.72-4.73h3.63a7.996 7.996 0 01-5.35 4.73zM13.89 12a18.419 18.419 0 000-4h3.83c.17.64.28 1.31.28 2s-.11 1.36-.28 2h-3.83z"],graph:["M17.5 4A2.5 2.5 0 0015 6.5c0 .06.01.12.02.18l-1.9.84C12.38 6.6 11.27 6 10 6c-.83 0-1.59.25-2.23.68L4.91 4.14c.05-.21.09-.42.09-.64a2.5 2.5 0 00-5 0A2.5 2.5 0 002.5 6c.42 0 .81-.11 1.16-.3l2.79 2.48C6.17 8.73 6 9.34 6 10c0 1.41.73 2.64 1.83 3.35l-.56 1.67A2.498 2.498 0 005 17.5a2.5 2.5 0 005 0c0-.74-.32-1.39-.83-1.85l.56-1.68c.09.01.18.03.27.03 2.21 0 4-1.79 4-4 0-.22-.03-.44-.07-.65l2.02-.9c.43.34.96.55 1.55.55a2.5 2.5 0 000-5z"],"graph-remove":["M17.41 4l2.29-2.29c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71L16 2.59 13.71.3A.965.965 0 0013 0a1.003 1.003 0 00-.71 1.71L14.59 4 12.3 6.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71L16 5.41l2.29 2.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71L17.41 4zM19 10c-.83 0-1.55-.36-2.09-.91l-.03.03-.88-.88-.88.88a2.996 2.996 0 11-4.24-4.24l.88-.88-.88-.88.03-.03C10.36 2.55 10 1.83 10 1c0-.35.07-.68.18-.99-.06 0-.12-.01-.18-.01C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10c0-.06-.01-.12-.01-.18-.31.11-.64.18-.99.18z"],"greater-than":["M12.838 10l-9.154 3.051a1 1 0 00.632 1.898l12-4c.912-.304.912-1.594 0-1.898l-12-4a1 1 0 00-.632 1.898L12.838 10z"],"greater-than-or-equal-to":["M3.684 11.051a1 1 0 00.632 1.898l12-4c.912-.304.912-1.594 0-1.898l-12-4a1 1 0 00-.632 1.898L12.838 8l-9.154 3.051zM4 15h12a1 1 0 110 2H4a1 1 0 010-2z"],grid:["M19 11c.55 0 1-.45 1-1s-.45-1-1-1h-2V5h2c.55 0 1-.45 1-1s-.45-1-1-1h-2V1c0-.55-.45-1-1-1s-1 .45-1 1v2h-4V1c0-.55-.45-1-1-1S9 .45 9 1v2H5V1c0-.55-.45-1-1-1S3 .45 3 1v2H1c-.55 0-1 .45-1 1s.45 1 1 1h2v4H1c-.55 0-1 .45-1 1s.45 1 1 1h2v4H1c-.55 0-1 .45-1 1s.45 1 1 1h2v2c0 .55.45 1 1 1s1-.45 1-1v-2h4v2c0 .55.45 1 1 1s1-.45 1-1v-2h4v2c0 .55.45 1 1 1s1-.45 1-1v-2h2c.55 0 1-.45 1-1s-.45-1-1-1h-2v-4h2zM9 15H5v-4h4v4zm0-6H5V5h4v4zm6 6h-4v-4h4v4zm0-6h-4V5h4v4z"],"grid-view":["M0 19c0 .55.45 1 1 1h8v-9H0v8zM0 1v8h9V0H1C.45 0 0 .45 0 1zm19-1h-8v9h9V1c0-.55-.45-1-1-1zm-8 20h8c.55 0 1-.45 1-1v-8h-9v9z"],"group-objects":["M6 7c-1.66 0-3 1.34-3 3s1.34 3 3 3 3-1.34 3-3-1.34-3-3-3zm8-3H6c-3.31 0-6 2.69-6 6s2.69 6 6 6h8c3.31 0 6-2.69 6-6s-2.69-6-6-6zm0 11H6c-2.76 0-5-2.24-5-5s2.24-5 5-5h8c2.76 0 5 2.24 5 5s-2.24 5-5 5zm0-8c-1.66 0-3 1.34-3 3s1.34 3 3 3 3-1.34 3-3-1.34-3-3-3z"],"grouped-bar-chart":["M12 16h1c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1h-1c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1zm7 1H1c-.55 0-1 .45-1 1s.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1zm-3-1h1c.55 0 1-.45 1-1V8c0-.55-.45-1-1-1h-1c-.55 0-1 .45-1 1v7c0 .55.45 1 1 1zm-9 0h1c.55 0 1-.45 1-1v-5c0-.55-.45-1-1-1H7c-.55 0-1 .45-1 1v5c0 .55.45 1 1 1zm-4 0h1c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1H3c-.55 0-1 .45-1 1v13c0 .55.45 1 1 1z"],hand:["M17 5c-.42 0-.79.27-.93.64L14.38 10h-.77l1.34-6.67c.03-.1.05-.21.05-.33a.998.998 0 00-1.98-.19h-.01L11.57 10H11V1c0-.55-.45-1-1-1S9 .45 9 1v9h-.2L6.97 2.76a.997.997 0 00-1.73-.41l-.03.03c-.01.02-.02.03-.03.04-.01.02-.01.03-.02.04v.01c-.01.01-.02.02-.02.03v.01c-.02.01-.02.02-.03.03 0 0 0 .01-.01.01 0 .01 0 .02-.01.03 0 0 0 .01-.01.01 0 .01-.01.02-.01.03 0 0 0 .01-.01.01 0 .01-.01.02-.01.03 0 .01 0 .01-.01.02 0 .01-.01.02-.01.03 0 .01 0 .01-.01.02 0 .01-.01.02-.01.03v.02c0 .01 0 .02-.01.03V3c0 .05 0 .09.01.14l1.45 10.25L6 12.7v.01L3.84 9.45h-.01A.98.98 0 003 9c-.55 0-1 .45-1 1 0 .2.06.39.17.55L6 18.44C7.06 19.4 8.46 20 10 20c3.31 0 6-2.69 6-6v-1.84l.01-.03v-.06l1.94-5.75A1.003 1.003 0 0017 5z"],"hand-down":["M17.68 9.84C15.91 9 14.27 6.49 13.45 4.9 12.41 2.43 12.21 0 7.87 0 5.49 0 3.95.76 3.05 2.65 2.31 4.2 2 5.48 2 9.79v.99c0 .82.69 1.48 1.54 1.48.38 0 .73-.14 1-.36.19.6.78 1.05 1.47 1.05.47 0 .89-.2 1.17-.52.26.47.77.79 1.36.79.65 0 1.2-.39 1.43-.93l.03.77v5.44c0 .48.23.91.59 1.18.21.19.5.32.85.32h.06c.83 0 1.5-.67 1.5-1.5v-8.24l.01-.67c.85.98 1.92 1.76 3.24 1.89 1.79.19 2.09-1.33 1.43-1.64z"],"hand-left":["M15.1 6.54c-1.58-.81-4.09-2.46-4.94-4.23-.31-.65-1.82-.35-1.64 1.43.13 1.33.91 2.4 1.89 3.24L9.74 7H1.5C.67 7 0 7.67 0 8.5v.06c0 .36.13.64.32.85.27.36.7.59 1.18.59h5.44l.78.01c-.54.23-.93.78-.93 1.43 0 .59.32 1.1.79 1.36-.32.28-.52.7-.52 1.17 0 .69.44 1.28 1.05 1.47-.22.27-.36.62-.36 1 0 .85.66 1.54 1.48 1.54h.99c4.31 0 5.59-.31 7.14-1.05 1.89-.9 2.65-2.44 2.65-4.82-.01-4.32-2.44-4.52-4.91-5.57z"],"hand-right":["M20 8.5c0-.83-.67-1.5-1.5-1.5h-8.24l-.67-.01c.98-.85 1.76-1.92 1.89-3.24.18-1.79-1.33-2.08-1.65-1.43-.84 1.76-3.35 3.41-4.93 4.23C2.43 7.59 0 7.79 0 12.13c0 2.38.76 3.92 2.65 4.82C4.2 17.69 5.48 18 9.79 18h.99c.82 0 1.48-.69 1.48-1.54 0-.38-.14-.73-.36-1 .6-.19 1.05-.78 1.05-1.47 0-.47-.2-.89-.52-1.17.47-.26.79-.77.79-1.36 0-.65-.39-1.2-.93-1.43l.77-.03h5.44c.48 0 .91-.23 1.18-.59.19-.21.32-.49.32-.85v-.03-.03z"],"hand-up":["M16.46 7.74c-.38 0-.73.14-1 .36-.19-.6-.78-1.05-1.47-1.05-.47 0-.89.2-1.17.52-.26-.47-.77-.79-1.36-.79-.65 0-1.2.39-1.43.93L10 6.94V1.5c0-.48-.23-.91-.59-1.18C9.2.13 8.92 0 8.56 0H8.5C7.67 0 7 .67 7 1.5v8.24l-.01.67c-.84-.98-1.92-1.76-3.24-1.89-1.79-.18-2.08 1.33-1.43 1.65 1.77.84 3.41 3.35 4.23 4.94 1.05 2.47 1.25 4.9 5.58 4.9 2.38 0 3.92-.76 4.82-2.65.74-1.56 1.05-2.84 1.05-7.15v-.99c0-.81-.69-1.48-1.54-1.48z"],hat:["M18.5 13c1.118 0 1.466.534 1.498 1.366L20 14.5v.5c0 1.945-5.69 3-10 3S0 16.945 0 15v-.5c0-.908.323-1.5 1.5-1.5.895 0 3.5 1.2 8.5 1.2l.411-.003C15.143 14.134 17.631 13 18.5 13zm-5-10c1.725 0 2.954 2.729 3.688 8.186-1.455.263-3.805.72-6.392.801l-.434.01L10 12c-2.896 0-5.585-.524-7.189-.814C3.546 5.73 4.775 3 6.5 3 8.6 3 8.329 5.5 10 5.5S11.5 3 13.5 3z"],header:["M16 1c-.55 0-1 .45-1 1v7H5V2c0-.55-.45-1-1-1s-1 .45-1 1v16c0 .55.45 1 1 1s1-.45 1-1v-7h10v7c0 .55.45 1 1 1s1-.45 1-1V2c0-.55-.45-1-1-1z"],"header-one":["M10 0c.55 0 1 .45 1 1v14c0 .55-.45 1-1 1s-1-.45-1-1V9H2v6c0 .55-.45 1-1 1s-1-.45-1-1V1c0-.55.45-1 1-1s1 .45 1 1v6h7V1c0-.55.45-1 1-1zm7.4 10.77c.17-.2.29-.46.34-.77H19v10h-1.5v-7.11H15v-1.24c.32 0 .63-.03.93-.08.31-.06.58-.16.83-.29.26-.12.47-.3.64-.51z"],"header-two":["M16.6 17.41c-.22.17-.4.36-.56.55-.16.19-.27.4-.33.61h4.28V20H14c.01-.81.18-1.52.53-2.13.35-.6.81-1.13 1.41-1.58.28-.23.58-.46.89-.68.31-.22.59-.46.85-.71.26-.26.48-.53.63-.83.16-.3.25-.64.26-1.02 0-.18-.02-.37-.06-.57-.04-.2-.11-.39-.22-.56s-.26-.31-.45-.43-.44-.18-.75-.18c-.28 0-.52.06-.71.19s-.34.3-.45.52c-.11.22-.2.48-.25.78-.05.3-.08.62-.09.97h-1.43c0-.54.07-1.04.2-1.5.13-.47.32-.87.58-1.2.26-.34.58-.6.95-.78.37-.19.81-.29 1.3-.29.54 0 .99.09 1.35.29.36.19.65.44.87.74.22.29.38.62.47.97.09.35.14.68.14 1 0 .4-.05.75-.16 1.07-.11.32-.26.61-.44.88-.19.27-.4.52-.63.74-.24.22-.48.43-.73.63s-.5.38-.75.56c-.26.17-.5.35-.71.53zM10 0c.55 0 1 .45 1 1v14c0 .55-.45 1-1 1s-1-.45-1-1V9H2v6c0 .55-.45 1-1 1s-1-.45-1-1V1c0-.55.45-1 1-1s1 .45 1 1v6h7V1c0-.55.45-1 1-1z"],headset:["M18.97 9H19A9 9 0 001 9h.03C.41 9.73 0 10.8 0 12c0 1.74.84 3.2 2 3.76V16c0 1.66 1.34 3 3 3h3c0 .55.45 1 1 1h2c.55 0 1-.45 1-1v-2c0-.55-.45-1-1-1H9c-.55 0-1 .45-1 1H5c-.55 0-1-.45-1-1 .55 0 1-.45 1-1V9c0-.55-.45-1-1-1h-.92C3.57 4.61 6.47 2 10 2s6.43 2.61 6.92 6H16c-.55 0-1 .45-1 1v6c0 .55.45 1 1 1h1c1.66 0 3-1.79 3-4 0-1.2-.41-2.27-1.03-3z"],heart:["M20 6.25C20 3.35 17.65 1 14.75 1c-1.02 0-1.95.31-2.75.82v-.04c-.09.06-.17.12-.26.19-.04.03-.09.06-.14.1-.68.51-1.24 1.18-1.6 1.96-.4-.86-1.04-1.57-1.8-2.1-.04-.02-.07-.05-.1-.08a7 7 0 00-.6-.33c-.13-.04-.23-.1-.35-.15-.05-.02-.1-.05-.15-.07v.02C6.45 1.13 5.87 1 5.25 1A5.25 5.25 0 000 6.25c0 .09.01.17.01.25H0c0 .06.01.12.02.18s.01.12.02.18C.13 7.89.44 9 1.07 10.17 2.23 12.33 4.1 14.11 7 16.53v.01c.9.75 1.89 1.55 3 2.46.71-.58 1.38-1.12 2-1.63 3.48-2.86 5.64-4.78 6.93-7.18.63-1.17.94-2.27 1.03-3.3.01-.07.01-.14.02-.21 0-.06.01-.11.02-.17h-.01c0-.09.01-.17.01-.26z"],"heart-broken":["M8.11 7.45C8.05 7.31 8 7.16 8 7c0-.07.03-.13.04-.19h-.02l.86-4.32A5.159 5.159 0 005.25 1 5.25 5.25 0 000 6.25c0 .09.01.17.01.25H0c0 .06.01.12.02.18s.01.12.02.18C.13 7.89.44 9 1.07 10.17c1.38 2.58 3.76 4.6 7.71 7.83l-.76-3.8h.02c-.01-.07-.04-.13-.04-.2 0-.21.08-.39.18-.54l-.02-.01 1.68-2.52-1.73-3.48zM20 6.25C20 3.35 17.65 1 14.75 1c-1.54 0-2.92.67-3.88 1.73l-.83 4.13 1.85 3.69h-.01c.07.14.12.29.12.45 0 .21-.08.39-.18.54l.02.01-1.77 2.66.81 4.07c4.16-3.39 6.63-5.45 8.05-8.1.63-1.17.94-2.27 1.03-3.3.01-.07.01-.14.02-.21 0-.06.01-.11.02-.17h-.01c0-.08.01-.16.01-.25z"],"heat-grid":["M14 12h6V8h-6v4zM0 12h6V8H0v4zm1-3h4v2H1V9zm-1 7c0 .55.45 1 1 1h5v-4H0v3zM19 3h-5v4h6V4c0-.55-.45-1-1-1zm0 3h-4V4h4v2zM0 4v3h6V3H1c-.55 0-1 .45-1 1zm7 3h6V3H7v4zm7 10h5c.55 0 1-.45 1-1v-3h-6v4zm-7 0h6v-4H7v4zm1-3h4v2H8v-2zm-1-2h6V8H7v4z"],heatmap:["M6 0a6 6 0 100 12A6 6 0 106 0z","M10.5 8a4.5 4.5 0 100 9 4.5 4.5 0 100-9z","M16.5 7a3.5 3.5 0 100 7 3.5 3.5 0 100-7zM18 16a2 2 0 100 4 2 2 0 100-4zM2.5 14a2.5 2.5 0 100 5 2.5 2.5 0 100-5zM16.5 0a2.5 2.5 0 100 5 2.5 2.5 0 100-5z"],help:["M10 0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zM7.41 4.62c.65-.54 1.51-.82 2.56-.82.54 0 1.03.08 1.48.25.44.17.83.39 1.14.68.32.29.56.63.74 1.02.17.39.26.82.26 1.27s-.08.87-.24 1.23c-.16.37-.4.73-.71 1.11l-1.21 1.58c-.14.17-.28.33-.32.48-.05.15-.11.35-.11.6v.97H9v-2s.06-.58.24-.81l1.21-1.64c.25-.3.41-.56.51-.77s.14-.44.14-.67c0-.35-.11-.63-.32-.85s-.5-.33-.88-.33c-.37 0-.67.11-.89.33-.22.23-.37.54-.46.94-.03.12-.11.17-.23.16l-1.95-.29c-.12-.01-.16-.08-.14-.22.13-.93.52-1.67 1.18-2.22zM9 14h2.02L11 16H9v-2z"],"helper-management":["M17 10h-3v3h3v-3zm0 4h-3v3h3v-3zm0-8h-3v3h3V6zm2-6H1C.4 0 0 .4 0 1v18c0 .5.4 1 1 1h18c.5 0 1-.5 1-1V1c0-.6-.5-1-1-1zm-1 18H2V2h16v16zm-9-4H6v3h3v-3zm4 0h-3v3h3v-3z"],"high-priority":["M12 16v4H8v-4h4zm1-16l-1 14H8L7 0h6z"],highlight:["M11.22 14.09l3.03-3.03.71.71L20 6.73l-5.71-5.71-5.04 5.04.71.71-3.02 3.04 4.28 4.28zm6.8 3.91h-16c-.55 0-1 .45-1 1s.45 1 1 1h16c.55 0 1-.45 1-1s-.45-1-1-1zm-15-1h4.04c.28 0 .53-.11.71-.3l2.02-2.02-3.44-3.45-4.04 4.04c-.18.18-.3.44-.3.71.01.57.46 1.02 1.01 1.02z"],history:["M10 0C6.71 0 3.82 1.6 2 4.05V2c0-.55-.45-1-1-1s-1 .45-1 1v4c0 .55.45 1 1 1h4c.55 0 1-.45 1-1s-.45-1-1-1H3.76C5.23 3.17 7.47 2 10 2c4.42 0 8 3.58 8 8s-3.58 8-8 8-8-3.58-8-8c0-.55-.45-1-1-1s-1 .45-1 1c0 5.52 4.48 10 10 10s10-4.48 10-10S15.52 0 10 0zm0 3c-.55 0-1 .45-1 1v6c0 .28.11.53.29.71l3 3a1.003 1.003 0 001.42-1.42L11 9.59V4c0-.55-.45-1-1-1z"],home:["M2 12v7c0 .55.45 1 1 1h5v-7h4v7h5c.55 0 1-.45 1-1v-7l-8-8-8 8zm17.71-2.71L17 6.59V3c0-.55-.45-1-1-1s-1 .45-1 1v1.59L10.71.3C10.53.11 10.28 0 10 0s-.53.11-.71.29l-9 9a1.003 1.003 0 001.42 1.42L10 2.41l8.29 8.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71z"],"horizontal-bar-chart":["M1 1c-.55 0-1 .45-1 1v16c0 .55.45 1 1 1s1-.45 1-1V2c0-.55-.45-1-1-1zm3 5h11c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1H4c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1zm8 8H4c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h8c.55 0 1-.45 1-1v-2c0-.55-.45-1-1-1zm7-6H4c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h15c.55 0 1-.45 1-1V9c0-.55-.45-1-1-1z"],"horizontal-bar-chart-asc":["M1 9h11c.55 0 1-.45 1-1V7c0-.55-.45-1-1-1H1c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1zm0-5h9c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1H1c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1zm18 12H1c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h18c.55 0 1-.45 1-1v-1c0-.55-.45-1-1-1zM1 14h14c.55 0 1-.45 1-1v-1c0-.55-.45-1-1-1H1c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1z"],"horizontal-bar-chart-desc":["M10 16H1c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h9c.55 0 1-.45 1-1v-1c0-.55-.45-1-1-1zm2-5H1c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h11c.55 0 1-.45 1-1v-1c0-.55-.45-1-1-1zm3-5H1c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V7c0-.55-.45-1-1-1zm4-5H1c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1z"],"horizontal-distribution":["M12 2H8c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1h4c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1zM1 0C.45 0 0 .45 0 1v18c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1zm18 0c-.55 0-1 .45-1 1v18c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1z"],"id-number":["M2 5v10h16V5H2zm0-2h16c1.1 0 2 .9 2 2v10c0 1.1-.9 2-2 2H2c-1.1 0-2-.9-2-2V5c0-1.1.9-2 2-2z","M8.88 12.38c-.17-.39-1.01-.66-1.56-.9-.56-.24-.48-.39-.5-.6v-.09c.19-.17.35-.4.45-.67 0 0 0-.02.01-.02l.06-.18c.13-.03.2-.17.23-.29.03-.05.09-.18.08-.33-.04-.18-.11-.27-.2-.3v-.03c0-.24-.02-.58-.06-.81-.01-.06-.02-.12-.04-.19-.08-.27-.25-.52-.48-.7C6.63 7.09 6.3 7 6 7s-.63.09-.87.27c-.23.17-.4.42-.48.7-.02.06-.03.13-.04.19-.04.22-.06.57-.06.81V9c-.09.03-.17.12-.19.31-.01.14.05.27.08.32.03.14.1.27.23.3.02.06.03.12.06.18v.01c.11.27.27.51.47.68v.08c-.02.2.04.35-.51.6-.56.24-1.39.51-1.56.9-.19.39-.12.62-.12.62h5.98c-.01 0 .06-.23-.11-.62zM12 7h4c.55 0 1 .45 1 1s-.45 1-1 1h-4c-.55 0-1-.45-1-1s.45-1 1-1zM12 11h4c.55 0 1 .45 1 1s-.45 1-1 1h-4c-.55 0-1-.45-1-1s.45-1 1-1z"],"image-rotate-left":["M10.5 13c.83 0 1.5-.67 1.5-1.5s-.67-1.5-1.5-1.5-1.5.67-1.5 1.5.67 1.5 1.5 1.5zM14 7H1c-.55 0-1 .45-1 1v11c0 .55.45 1 1 1h13c.55 0 1-.45 1-1V8c0-.55-.45-1-1-1zm-1 10l-5-3-1 2-2-4-3 4.5V9h11v8zm3-15h-1.59l.29-.29c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71l-2 2c-.18.18-.29.43-.29.71 0 .28.11.53.29.71l2 2a1.003 1.003 0 001.42-1.42l-.3-.29H16c1.1 0 2 .9 2 2v3c0 .55.45 1 1 1s1-.45 1-1V6c0-2.21-1.79-4-4-4z"],"image-rotate-right":["M5.29 4.29a1.003 1.003 0 001.42 1.42l2-2C8.89 3.53 9 3.28 9 3c0-.28-.11-.53-.29-.71l-2-2a1.003 1.003 0 00-1.42 1.42l.3.29H4C1.79 2 0 3.79 0 6v3c0 .55.45 1 1 1s1-.45 1-1V6c0-1.1.9-2 2-2h1.59l-.3.29zM15.5 13c.83 0 1.5-.67 1.5-1.5s-.67-1.5-1.5-1.5-1.5.67-1.5 1.5.67 1.5 1.5 1.5zM19 7H6c-.55 0-1 .45-1 1v11c0 .55.45 1 1 1h13c.55 0 1-.45 1-1V8c0-.55-.45-1-1-1zm-1 10l-5-3-1 2-2-4-3 4.5V9h11v8z"],import:["M9.29 15.71c.18.18.43.29.71.29s.53-.11.71-.29l5-5a1.003 1.003 0 00-1.42-1.42L11 12.59V1c0-.55-.45-1-1-1S9 .45 9 1v11.59l-3.29-3.3a1.003 1.003 0 00-1.42 1.42l5 5zM19 14c-.55 0-1 .45-1 1v3H2v-3c0-.55-.45-1-1-1s-1 .45-1 1v4c0 .55.45 1 1 1h18c.55 0 1-.45 1-1v-4c0-.55-.45-1-1-1z"],inbox:["M16.92 3.56l-.01-.02c-.16-.35-.5-.6-.91-.6H4c-.41 0-.76.25-.91.6l-.01.02L0 10.49v6.46c0 .55.45 1 1 1h18c.55 0 1-.45 1-1v-6.46l-3.08-6.93zM15 10.95c-.55 0-1 .45-1 1v1H6v-1c0-.55-.45-1-1-1H1.98l2.67-6h10.7l2.67 6H15z"],"inbox-filtered":["M10.262 3l1.958 1.958v.05H4.65l-2.67 5.997H5c.55 0 1 .45 1 .999v1h8v-1c0-.55.45-1 1-1h3.02l-.635-1.426.625-.63c.354-.353.598-.8.707-1.289L20 10.545v6.456c0 .55-.45.999-1 .999H1c-.55 0-1-.45-1-1v-6.455L3.08 3.62l.01-.02c.15-.35.5-.6.91-.6h6.262zm9.088-3a.642.642 0 01.46 1.1l-3.03 3.03v2.95c0 .18-.07.34-.19.46l-1.28 1.29c-.11.1-.27.17-.45.17-.35 0-.64-.29-.64-.64V4.13L11.19 1.1a.642.642 0 01.45-1.1h7.71z"],"inbox-geo":["M7.427 3a7.467 7.467 0 00-.411 2.009H4.65l-2.67 5.996H5c.55 0 1 .45 1 .999v1h8V13c.165.01.332 0 .5 0a7.48 7.48 0 005.5-2.4V17c0 .55-.45 1-1 1H1c-.55 0-1-.45-1-1v-6.455L3.08 3.62l.01-.02c.15-.35.5-.6.91-.6h3.427zm5.715-.596a.133.133 0 01-.193 0l-.374-.374a.133.133 0 010-.193.133.133 0 01.193 0l.373.374a.133.133 0 010 .193zm1.743.033c-.05 0-.088-.006-.088-.055 0-.05.038-.056.088-.056h.165c.05 0 .088.006.088.055 0 .05-.038.056-.088.056h-.165zm.539.632c-.05 0-.104-.044-.104-.094v-.23c0-.05.054-.094.104-.094.05 0 .104.044.104.094v.23c0 .05-.055.094-.104.094zm-3.575 2.304h.506l1.182 1.2c.006.005 0 .005 0 .01v.446c0 .187-.126.341-.319.341h-.098v.226c0 .192-.138.296-.33.296h.01v.792c0 .188-.181.336-.368.336s-.369-.149-.369-.335v-1.32h-.214c-.193 0-.308-.149-.308-.341V5.72c0-.192.115-.346.308-.346zM14.5 0C17.536 0 20 2.464 20 5.5S17.536 11 14.5 11A5.502 5.502 0 019 5.5C9 2.464 11.464 0 14.5 0zm0 9.9c2.431 0 4.4-1.969 4.4-4.4 0-.297-.027-.583-.099-.864h-.236c-.188 0-.336-.104-.347-.313h-.319c.11.104.231.209.231.346v.705c0 .088-.055.17-.11.23h-.022l-.011.006-.022.011-.666.643v1.21c0 .193-.104.33-.296.33h-.54c-.192 0-.319-.137-.319-.33V6.221a.915.915 0 00-.533-.518h-.671c-.192 0-.368-.143-.368-.335V4.68c0-.192.176-.346.368-.346l.193-.005.319-.32a.342.342 0 01.489 0l.319.32c.005 0 .005.005.005.005h.704c.16 0 .237-.16.341-.313v-.11l-.038.005h-.27a.159.159 0 01-.153-.16c0-.087.066-.159.154-.159h.269l.039.006V3.42s-.05-.038-.061-.038h-.302l-.067.076a.342.342 0 01-.489.011l-.066-.088h-.176l.248.259c.021.022.021.088 0 .11-.028.022-.067.028-.088.006l-.292-.276h-.127l-.363.325-.033-.006c-.038.11-.087.089-.143.089-.126 0-.225-.072-.225-.193 0-.127.099-.209.225-.209h.176v-.182c0-.088.061-.131.149-.131h.066v-.127c0-.143.149-.297.286-.297h.28c.16 0 .27-.115.27-.275V2.42c0-.016.055.017.055-.088h-.187l-.005.017-.308.33a.123.123 0 01-.177 0c-.049-.044-.049-.121 0-.171l.391-.385c.027-.022.06-.022.094-.022l.005.022h.869A4.376 4.376 0 0014.5 1.1a4.402 4.402 0 00-2.816 1.018h.583c.094 0 .165.066.165.159s-.072.16-.165.16h-.478c0 .104-.039.06-.039.066v.351h.429l.226-.252c.132-.127.346-.155.473-.022a.332.332 0 010 .473l-1.183 1.182-.011.011c-.005.005-.011.005-.016.011a.115.115 0 00-.034.022c-.005.006-.01 0-.016.006a.309.309 0 01-.176.038h-.347c-.12.104-.187.148-.187.27v.088c0 .016-.027.027-.027.043l.561.589c.06.06.055.154 0 .209a.143.143 0 01-.209 0l-.578-.578a.425.425 0 01-.082.011c-.154 0-.292-.12-.292-.274v-.358h-.016c-.104.374-.165.77-.165 1.177 0 2.431 1.969 4.4 4.4 4.4zm3.388-3.107c.077 0 .16.06.16.137v.424c0 .077-.083.137-.16.137s-.16-.06-.16-.137V6.93c0-.077.083-.137.16-.137zm-3.113-4.879c0 .187-.154.314-.335.314h-.374v.104c0 .11-.05.198-.16.198s-.16-.088-.16-.198V1.98c-.104-.022-.033-.028-.033-.055 0-.187.127-.325.314-.325h.407c.187 0 .341.127.341.314z"],"inbox-search":["M7.136 3a6.327 6.327 0 00-.098 2.009H4.65l-2.67 5.996H5c.55 0 1 .45 1 .999v1h8v-1c0-.55.45-1 1-1h1.076l1.14 1.14a2.767 2.767 0 001.974.806c.282 0 .554-.042.81-.12V17c0 .55-.45 1-1 1H1c-.55 0-1-.45-1-1v-6.455L3.08 3.62l.01-.02c.15-.35.5-.6.91-.6h3.136zm3.244 1.33c0 1.62 1.31 2.93 2.93 2.93s2.93-1.31 2.93-2.93-1.31-2.93-2.93-2.93-2.93 1.31-2.93 2.93zm6.47 2.43l2.89 2.85c.13.15.22.35.23.56 0 .43-.35.78-.78.78-.23 0-.42-.08-.56-.22l-2.87-2.87c-.17.1-.33.2-.51.29-.03.01-.06.03-.09.04-.18.07-.35.15-.55.21-.19.06-.37.11-.57.14-.05.01-.1.02-.14.02-.2.03-.39.05-.6.05A4.3 4.3 0 019 4.31C9 1.93 10.93.01 13.3 0c2.37 0 4.3 1.93 4.3 4.3 0 .21-.02.4-.05.6-.01.05-.01.09-.02.14-.04.2-.08.38-.14.58-.05.19-.13.36-.21.54-.01.03-.03.06-.04.09-.08.18-.18.34-.29.51z"],"inbox-update":["M10.083 3a6.04 6.04 0 00.001 2.009H4.65l-2.67 5.996H5c.55 0 1 .45 1 .999v1h8v-1c0-.55.45-1 1-1h3.02l-.53-1.19a5.97 5.97 0 001.824-.811L20 10.545v6.456c0 .55-.45.999-1 .999H1c-.55 0-1-.45-1-1v-6.455L3.08 3.62l.01-.02c.15-.35.5-.6.91-.6h6.083zM16 8a4 4 0 110-8 4 4 0 010 8z"],"info-sign":["M10 0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zM9 4h2v2H9V4zm4 12H7v-1h2V8H8V7h3v8h2v1z"],inheritance:["M6 10c0 2.21 1.79 4 4 4h6.59l-2.29-2.29A.965.965 0 0114 11a1.003 1.003 0 011.71-.71l4 4c.18.18.29.43.29.71 0 .28-.11.53-.29.71l-4 4a1.003 1.003 0 01-1.42-1.42l2.3-2.29H10c-3.31 0-6-2.69-6-6H1a1 1 0 01-1-1V1a1 1 0 011-1h8a1 1 0 011 1v8a1 1 0 01-1 1H6zM2 2v6h6V2H2z"],"inherited-group":["M1 9c.55 0 1 .45 1 1v4c0 1.1.9 2 2 2h2.59l-.3-.29a1.003 1.003 0 011.42-1.42l2 2c.18.18.29.43.29.71 0 .28-.11.53-.29.71l-2 2A1.003 1.003 0 016 19c0-.28.11-.53.3-.71l.29-.29H4c-2.21 0-4-1.79-4-4v-4c0-.55.45-1 1-1zm6.996-9c.79 0 1.68.25 2.309.73a3.63 3.63 0 011.4 2.36c.11.6.17 1.52.17 2.15v.09c.22.09.42.32.47.82.03.39-.12.73-.2.87-.07.34-.27.71-.61.78-.04.16-.09.33-.15.48-.01.01-.02.05-.02.05-.27.71-.68 1.33-1.19 1.78 0 .08 0 .16.01.23.05.55-.15.95 1.33 1.6 1.469.66 3.698 1.35 4.178 2.39.45 1.05.27 1.67.27 1.67h-5.227a1.982 1.982 0 00-.319-.417l-2-2A2.003 2.003 0 005 15H4c-.548 0-1-.452-1-1v-1.462c.511-.213 1.023-.413 1.468-.608 1.479-.65 1.329-1.05 1.379-1.59l.01-.21c-.52-.45-.95-1.08-1.22-1.8l-.01-.01-.01-.03c-.07-.15-.12-.32-.16-.49-.34-.06-.54-.43-.62-.78-.08-.14-.24-.48-.2-.87.05-.51.26-.74.49-.83v-.08c0-.64.05-1.55.17-2.15a3.648 3.648 0 011.4-2.36C6.317.25 7.207 0 7.996 0zm5.997 3c.59 0 1.26.19 1.73.55.46.35.8.85.97 1.4.04.13.07.25.08.38.08.45.13 1.14.13 1.61v.07c.16.07.31.24.35.62.02.29-.09.55-.15.65-.05.26-.2.53-.46.59-.03.12-.07.25-.11.36-.01.01-.01.04-.01.04-.2.53-.51 1-.89 1.34 0 .06 0 .12.01.17.04.41-.11.71 1 1.19 1.099.5 2.768 1.01 3.128 1.79.34.79.2 1.25.2 1.25h-3.039V15c-.06-.33-.17-.69-.33-1.06-.45-.97-1.37-1.52-3.238-2.3-.17-.07-.76-.31-.77-.32-.1-.04-.2-.08-.28-.12.05-.14.04-.29.06-.45l.01-.16c-.25-.21-.47-.48-.65-.79.22-.34.41-.71.56-1.12l.028-.078-.002.013-.006.035.06-.15c.36-.26.6-.67.72-1.13.18-.37.29-.82.25-1.3-.05-.5-.21-.92-.47-1.22-.02-.53-.06-1.11-.12-1.59.38-.17.83-.26 1.24-.26z"],"inner-join":["M8.7 4.7C7.4 6 6.5 7.9 6.5 10s.8 4 2.2 5.3c-.8.5-1.7.7-2.7.7-3.3 0-6-2.7-6-6s2.7-6 6-6c1 0 1.9.2 2.7.7zm-3.34 9.25c-.55-1.2-.86-2.54-.86-3.95s.31-2.75.86-3.95a4.001 4.001 0 000 7.9zM14 4c3.3 0 6 2.7 6 6s-2.7 6-6 6c-1 0-1.9-.2-2.7-.7 1.3-1.3 2.2-3.2 2.2-5.3s-.8-3.9-2.2-5.3C12.1 4.2 13 4 14 4zm.6 2.05c.55 1.2.86 2.54.86 3.95s-.31 2.75-.86 3.95c1.9-.31 3.36-1.96 3.36-3.95S16.5 6.36 14.6 6.05zM10 5.5C8.8 6.7 8 8.2 8 10s.8 3.3 2 4.4c1.2-1.1 2-2.7 2-4.5s-.8-3.3-2-4.4z"],insert:["M19 0H1C.4 0 0 .4 0 1v18c0 .5.4 1 1 1h18c.5 0 1-.5 1-1V1c0-.6-.5-1-1-1zm-1 18H2V2h16v16zM5 11h4v4c0 .6.4 1 1 1s1-.4 1-1v-4h4c.6 0 1-.4 1-1s-.4-1-1-1h-4V5c0-.6-.4-1-1-1s-1 .4-1 1v4H5c-.6 0-1 .4-1 1s.4 1 1 1z"],intersection:["M13 4c-1.31 0-2.51.43-3.5 1.14A5.977 5.977 0 006 4c-3.31 0-6 2.69-6 6s2.69 6 6 6c1.31 0 2.51-.43 3.5-1.14.99.71 2.19 1.14 3.5 1.14 3.31 0 6-2.69 6-6s-2.69-6-6-6zm-4.93 9.41c-.61.37-1.31.59-2.07.59-2.21 0-4-1.79-4-4s1.79-4 4-4c.76 0 1.46.22 2.07.59C7.4 7.56 7 8.73 7 10s.4 2.44 1.07 3.41zM13 14c-.76 0-1.46-.22-2.07-.59C11.6 12.44 12 11.27 12 10s-.4-2.44-1.07-3.41C11.54 6.22 12.24 6 13 6c2.21 0 4 1.79 4 4s-1.79 4-4 4z"],"ip-address":["M6 3.66C6 5.69 10 11 10 11s4-5.31 4-7.34C13.99 1.64 12.21 0 10 0S6 1.64 6 3.66zM8 4c0-1.1.9-2 2-2s2 .9 2 2-.9 2-2 2-2-.9-2-2zM14 13.5V13h-4v1h3v2h-2v1h3v-3.5zM3 12h14c.55 0 1 .45 1 1v6c0 .55-.45 1-1 1H3c-.55 0-1-.45-1-1v-6c0-.55.45-1 1-1zm4 1v6h1v-6H7zm3 1v5h1v-5h-1z"],issue:["M10 20C4.477 20 0 15.523 0 10S4.477 0 10 0s10 4.477 10 10-4.477 10-10 10zm0-2a8 8 0 100-16 8 8 0 000 16zm1-2H9v-2h2v2zm0-3H9V4h2v9z"],"issue-closed":["M15.364 5.9a.997.997 0 01-.707-.293l-2.121-2.122a1 1 0 111.414-1.414l1.414 1.414L18.192.657a1 1 0 011.414 1.414l-3.535 3.536a.997.997 0 01-.707.292zM11.78.157a3.002 3.002 0 00-1.437 1.85 8 8 0 107.1 5.055l.042-.042 1.472-1.472A9.959 9.959 0 0120 10c0 5.523-4.477 10-10 10S0 15.523 0 10 4.477 0 10 0c.608 0 1.202.054 1.78.158zM11 16H9v-2h2v2zm0-3H9V4h2v9z"],"issue-new":["M13.167.512a2.98 2.98 0 00-.131.524c-.74.115-1.39.5-1.848 1.052a8 8 0 106.724 6.724 2.997 2.997 0 001.052-1.848 2.98 2.98 0 00.524-.13A9.99 9.99 0 0120 10c0 5.523-4.477 10-10 10S0 15.523 0 10 4.477 0 10 0a9.99 9.99 0 013.167.512zM11 16H9v-2h2v2zm0-3H9V4h2v9zm6-10h1.5a1 1 0 010 2H17v1.5a1 1 0 01-2 0V5h-1.5a1 1 0 010-2H15V1.5a1 1 0 012 0V3z"],italic:["M11.7 4H14c.6 0 1-.4 1-1s-.4-1-1-1H7c-.6 0-1 .4-1 1s.4 1 1 1h2.2L7.3 15H5c-.6 0-1 .4-1 1s.4 1 1 1h7c.6 0 1-.4 1-1s-.4-1-1-1H9.8l1.9-11z"],"join-table":["M19 6h-4V2c0-.55-.45-1-1-1H1c-.55 0-1 .45-1 1v11c0 .55.45 1 1 1h4v4c0 .55.45 1 1 1h13c.55 0 1-.45 1-1V7c0-.55-.45-1-1-1zM6 12H2V9h4v3zm0-4H2V5h4v3zm7 9H7v-3h6v3zm0-4H7V9h6v4zm0-5H7V5h6v3zm5 9h-4v-3h4v3zm0-4h-4v-3h4v3z"],key:["M14 0c-3.31 0-6 2.69-6 6 0 1.11.32 2.14.85 3.03L.44 17.44a1.498 1.498 0 102.12 2.12l.79-.79.94.94c.18.18.43.29.71.29s.53-.11.71-.29l3-3c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71l-.94-.94 3.2-3.2A5.9 5.9 0 0014 12c3.31 0 6-2.69 6-6s-2.69-6-6-6zm0 9c-1.66 0-3-1.34-3-3s1.34-3 3-3 3 1.34 3 3-1.34 3-3 3z"],"key-backspace":["M19 3H7c-.28 0-.53.11-.71.29l-6 6C.11 9.47 0 9.72 0 10c0 .28.11.53.29.71l6 6c.18.18.43.29.71.29h12c.55 0 1-.45 1-1V4c0-.55-.45-1-1-1zm-2.29 9.29a1.003 1.003 0 01-1.42 1.42L13 11.41l-2.29 2.29c-.18.19-.43.3-.71.3a1.003 1.003 0 01-.71-1.71l2.3-2.29-2.3-2.29a1.003 1.003 0 011.42-1.42L13 8.59l2.29-2.29c.18-.19.43-.3.71-.3a1.003 1.003 0 01.71 1.71L14.41 10l2.3 2.29z"],"key-command":["M15.5 12H14V8h1.5C17.43 8 19 6.43 19 4.5S17.43 1 15.5 1 12 2.57 12 4.5V6H8V4.5C8 2.57 6.43 1 4.5 1S1 2.57 1 4.5 2.57 8 4.5 8H6v4H4.5C2.57 12 1 13.57 1 15.5S2.57 19 4.5 19 8 17.43 8 15.5V14h4v1.5c0 1.93 1.57 3.5 3.5 3.5s3.5-1.57 3.5-3.5-1.57-3.5-3.5-3.5zm0-9c.83 0 1.5.67 1.5 1.5S16.33 6 15.5 6 14 5.33 14 4.5 14.67 3 15.5 3zm-11 14c-.83 0-1.5-.67-1.5-1.5S3.67 14 4.5 14s1.5.67 1.5 1.5S5.33 17 4.5 17zm0-11C3.67 6 3 5.33 3 4.5S3.67 3 4.5 3 6 3.67 6 4.5 5.33 6 4.5 6zm7.5 6H8V8h4v4zm3.5 5c-.83 0-1.5-.67-1.5-1.5s.67-1.5 1.5-1.5 1.5.67 1.5 1.5-.67 1.5-1.5 1.5z"],"key-control":["M16.71 7.29l-6-6C10.53 1.11 10.28 1 10 1s-.53.11-.71.29l-6 6a1.003 1.003 0 001.42 1.42L10 3.41l5.29 5.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71z"],"key-delete":["M19.71 9.29l-6-6A.997.997 0 0013 3H1c-.55 0-1 .45-1 1v12c0 .55.45 1 1 1h12c.28 0 .53-.11.71-.29l6-6c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71zm-9 3a1.003 1.003 0 01-1.42 1.42L7 11.41 4.71 13.7c-.18.19-.43.3-.71.3a1.003 1.003 0 01-.71-1.71L5.59 10l-2.3-2.29a1.003 1.003 0 011.42-1.42L7 8.59 9.29 6.3c.18-.19.43-.3.71-.3a1.003 1.003 0 01.71 1.71L8.41 10l2.3 2.29z"],"key-enter":["M18 2c-.55 0-1 .45-1 1v5c0 2.21-1.79 4-4 4H4.41L6.7 9.71c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71l-4 4c-.18.18-.29.43-.29.71 0 .28.11.53.29.71l4 4a1.003 1.003 0 001.42-1.42L4.41 14H13c3.31 0 6-2.69 6-6V3c0-.55-.45-1-1-1z"],"key-escape":["M2 8c.55 0 1-.45 1-1V4.41l6.29 6.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71L4.41 3H7c.55 0 1-.45 1-1s-.45-1-1-1H2c-.55 0-1 .45-1 1v5c0 .55.45 1 1 1zm9-6.94V3.1c3.39.49 6 3.38 6 6.9 0 3.87-3.13 7-7 7-3.52 0-6.41-2.61-6.9-6H1.06c.5 4.5 4.31 8 8.94 8a9 9 0 009-9c0-4.63-3.5-8.44-8-8.94z"],"key-option":["M13 4h6c.55 0 1-.45 1-1s-.45-1-1-1h-6c-.55 0-1 .45-1 1s.45 1 1 1zm6 12h-4.42L6.87 2.5l-.02.01A.977.977 0 006 2H1c-.55 0-1 .45-1 1s.45 1 1 1h4.42l7.71 13.5.01-.01c.18.3.49.51.86.51h5c.55 0 1-.45 1-1s-.45-1-1-1z"],"key-shift":["M17.74 10.35l-6.99-8.01-.01.01C10.56 2.14 10.3 2 10 2s-.56.14-.74.35l-.01-.01-7 8 .01.01A.95.95 0 002 11c0 .55.45 1 1 1h3v5c0 .55.45 1 1 1h6c.55 0 1-.45 1-1v-5h3c.55 0 1-.45 1-1 0-.25-.1-.48-.26-.65z"],"key-tab":["M19 13H4.41l2.29-2.29c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71L2 12.59V10c0-.55-.45-1-1-1s-1 .45-1 1v8c0 .55.45 1 1 1s1-.45 1-1v-2.59l3.29 3.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71L4.41 15H19c.55 0 1-.45 1-1s-.45-1-1-1zm0-12c-.55 0-1 .45-1 1v2.59L14.71 1.3A.965.965 0 0014 1a1.003 1.003 0 00-.71 1.71L15.59 5H1c-.55 0-1 .45-1 1s.45 1 1 1h14.59L13.3 9.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71L18 7.41V10c0 .55.45 1 1 1s1-.45 1-1V2c0-.55-.45-1-1-1z"],"known-vehicle":["M19 4a.997.997 0 00-.707.293L14 8.586l-2.293-2.293a1 1 0 00-1.414 1.414l3 3a.997.997 0 001.414 0l5-5A1 1 0 0019 4zm-2.048 7.291c.011.072.048.134.048.209a1.5 1.5 0 01-1.5 1.5c-.225 0-.433-.057-.624-.145-.279.085-.57.145-.876.145a2.99 2.99 0 01-2.121-.879l-3-3 .007-.007A3.027 3.027 0 018.184 8H4V7l1-3h10l.19.568 1.307-1.308c-.336-.356-.758-.658-1.165-.772 0 0-1.74-.488-5.332-.488s-5.332.488-5.332.488c-.67.188-1.424.864-1.674 1.502L2.99 4H3L2 7H1a1 1 0 000 2h.333l-.28.84L1 10v7.5a1.5 1.5 0 103 0V17h12v.5a1.5 1.5 0 003 0V10l-.19-.568-1.858 1.86zM4.5 13a1.5 1.5 0 110-3 1.5 1.5 0 010 3z"],"lab-test":["M13 2a1 1 0 010 2v4l4 8v1a1 1 0 01-1 1H4a1 1 0 01-1-1v-1l4-8V4a1 1 0 110-2h6zm-2 2H9v4l-2 4h6l-2-4V4z"],label:["M3 12h14v-1H3v1zm11-9H1c-.55 0-1 .45-1 1v12c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V9l-6-6zm4 12H2V5h11v3H3v1h10v1h5v5zm-4-6V5l4 4h-4z"],layer:["M19.5 9.1l-9-5c-.2-.1-.3-.1-.5-.1s-.3 0-.5.1l-9 5c-.3.2-.5.5-.5.9s.2.7.5.9l9 5c.2.1.3.1.5.1s.3 0 .5-.1l9-5c.3-.2.5-.5.5-.9s-.2-.7-.5-.9z"],"layer-outline":["M9.514 4.126l-9 5a1 1 0 000 1.748l9 5a1 1 0 00.972 0l9-5a1 1 0 000-1.748l-9-5a1 1 0 00-.972 0zM10 6.144l6.94 3.855L10 13.855 3.059 9.999 10 6.144z"],layers:["M.5 6.9l9 5c.2.1.3.1.5.1s.3 0 .5-.1l9-5c.3-.2.5-.5.5-.9s-.2-.7-.5-.9l-9-5c-.2-.1-.3-.1-.5-.1s-.3 0-.5.1l-9 5c-.3.2-.5.5-.5.9s.2.7.5.9z","M19 9c-.2 0-.3 0-.5.1L10 13.9 1.5 9.1C1.3 9 1.2 9 1 9c-.6 0-1 .4-1 1 0 .4.2.7.5.9l9 5c.2.1.3.1.5.1s.3 0 .5-.1l9-5c.3-.2.5-.5.5-.9 0-.6-.4-1-1-1z","M19 13c-.2 0-.3 0-.5.1L10 17.9l-8.5-4.7c-.2-.2-.3-.2-.5-.2-.6 0-1 .4-1 1 0 .4.2.7.5.9l9 5c.2.1.3.1.5.1s.3 0 .5-.1l9-5c.3-.2.5-.5.5-.9 0-.6-.4-1-1-1z"],layout:["M18 6c-1.1 0-2 .9-2 2 0 .37.11.71.28 1.01l-2.27 2.27c-.3-.17-.64-.28-1.01-.28-.93 0-1.71.64-1.93 1.5H8.93c-.22-.86-1-1.5-1.93-1.5-.37 0-.71.11-1.01.28L3.72 9.01C3.89 8.71 4 8.37 4 8c0-.34-.09-.66-.24-.94l3.66-3.38c.31.2.68.32 1.08.32 1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2c0 .34.09.66.24.94L3.08 6.32C2.77 6.12 2.4 6 2 6 .9 6 0 6.9 0 8s.9 2 2 2c.37 0 .71-.11 1.01-.28l2.27 2.27c-.17.3-.28.64-.28 1.01s.11.71.28 1.01l-2.27 2.27C2.71 16.11 2.37 16 2 16c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2c0-.37-.11-.71-.28-1.01l2.27-2.27c.3.17.64.28 1.01.28.93 0 1.71-.64 1.93-1.5h2.14c.22.86 1 1.5 1.93 1.5 1.1 0 2-.9 2-2 0-.37-.11-.71-.28-1.01l2.27-2.27c.3.17.64.28 1.01.28 1.1 0 2-.9 2-2s-.9-2-2-2z"],"layout-auto":["M18 13c-.53 0-1.01.21-1.37.55L11.9 10.6c.06-.19.1-.39.1-.6s-.04-.41-.1-.6l4.72-2.95c.37.34.85.55 1.38.55 1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2c0 .21.04.41.1.6l-4.73 2.96c-.24-.23-.54-.4-.87-.48V3.93c.86-.22 1.5-1 1.5-1.93 0-1.1-.9-2-2-2S8 .9 8 2c0 .93.64 1.71 1.5 1.93v4.14c-.33.09-.63.26-.87.48L3.9 5.6c.06-.19.1-.39.1-.6 0-1.1-.9-2-2-2s-2 .9-2 2 .9 2 2 2c.53 0 1.01-.21 1.37-.55L8.1 9.4c-.06.19-.1.39-.1.6s.04.41.1.6l-4.72 2.95C3.01 13.21 2.53 13 2 13c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2c0-.21-.04-.41-.1-.6l4.73-2.96c.24.23.54.4.87.48v4.14C8.64 16.29 8 17.07 8 18c0 1.1.9 2 2 2s2-.9 2-2c0-.93-.64-1.71-1.5-1.93v-4.14c.33-.09.63-.26.87-.48l4.73 2.96c-.06.18-.1.38-.1.59 0 1.1.9 2 2 2s2-.9 2-2-.9-2-2-2z"],"layout-balloon":["M18 16c-.14 0-.28.02-.42.05l-1.73-3.45c.69-.45 1.14-1.22 1.14-2.1s-.46-1.65-1.14-2.1l1.73-3.45c.14.03.28.05.42.05 1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2c0 .6.27 1.13.69 1.5l-1.77 3.54c-.14-.02-.28-.04-.42-.04a2.5 2.5 0 00-2.45 2h-4.1A2.5 2.5 0 005.5 8c-.14 0-.28.02-.42.04L3.31 4.5C3.73 4.13 4 3.6 4 3c0-1.1-.9-2-2-2s-2 .9-2 2 .9 2 2 2c.14 0 .28-.02.42-.05L4.14 8.4C3.46 8.85 3 9.62 3 10.5s.46 1.65 1.14 2.1l-1.73 3.45A1.84 1.84 0 002 16c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2c0-.6-.27-1.13-.69-1.5l1.77-3.54c.14.02.28.04.42.04a2.5 2.5 0 002.45-2h4.1a2.5 2.5 0 002.45 2c.14 0 .28-.02.42-.04l1.77 3.54c-.42.37-.69.9-.69 1.5 0 1.1.9 2 2 2s2-.9 2-2-.9-2-2-2z"],"layout-circle":["M18.3 8c-.2-.9-.6-1.7-1.1-2.5.2-.3.3-.7.3-1 0-1.1-.9-2-2-2-.4 0-.7.1-1 .3-.8-.5-1.6-.8-2.5-1.1-.1-1-1-1.7-2-1.7S8.2.8 8 1.7c-.9.3-1.7.6-2.5 1.1-.3-.2-.7-.3-1-.3-1.1 0-2 .9-2 2 0 .4.1.7.3 1-.5.8-.8 1.6-1.1 2.5C.8 8.2 0 9 0 10s.8 1.8 1.7 2c.2.9.6 1.7 1.1 2.5-.2.3-.3.7-.3 1 0 1.1.9 2 2 2 .4 0 .7-.1 1-.3.8.5 1.6.8 2.5 1.1.1 1 1 1.7 2 1.7s1.8-.8 2-1.7c.9-.2 1.7-.6 2.5-1.1.3.2.7.3 1 .3 1.1 0 2-.9 2-2 0-.4-.1-.7-.3-1 .5-.8.8-1.6 1.1-2.5 1-.1 1.7-1 1.7-2s-.8-1.8-1.7-2zm-1.8 5.8c-.3-.2-.6-.3-1-.3-1.1 0-2 .9-2 2 0 .4.1.7.3 1-.6.3-1.2.6-1.9.8-.3-.7-1-1.3-1.9-1.3-.8 0-1.6.5-1.9 1.3-.7-.2-1.3-.4-1.9-.8.2-.3.3-.6.3-1 0-1.1-.9-2-2-2-.4 0-.7.1-1 .3-.3-.6-.6-1.2-.8-1.9.8-.3 1.3-1.1 1.3-1.9s-.5-1.6-1.2-1.8c.2-.7.4-1.3.8-1.9.3.2.6.3 1 .3 1.1 0 2-.9 2-2 0-.4-.1-.7-.3-1 .6-.3 1.2-.6 1.9-.8.2.7 1 1.2 1.8 1.2s1.6-.5 1.9-1.3c.7.2 1.3.4 1.9.8-.2.3-.3.6-.3 1 0 1.1.9 2 2 2 .4 0 .7-.1 1-.3.3.6.6 1.2.8 1.9-.8.3-1.3 1.1-1.3 1.9s.5 1.6 1.2 1.8c-.1.7-.4 1.4-.7 2z"],"layout-grid":["M2 0a2 2 0 100 4 2 2 0 100-4zM10 0a2 2 0 100 4 2 2 0 100-4zM18 0a2 2 0 100 4 2 2 0 100-4zM18 8a2 2 0 100 4 2 2 0 100-4zM18 16a2 2 0 100 4 2 2 0 100-4zM10 16a2 2 0 100 4 2 2 0 100-4zM2 16a2 2 0 100 4 2 2 0 100-4zM2 8a2 2 0 100 4 2 2 0 100-4zM10 8a2 2 0 100 4 2 2 0 100-4z"],"layout-group-by":["M2 2a2 2 0 100 4 2 2 0 100-4zM18 0a2 2 0 100 4 2 2 0 100-4zM18 8a2 2 0 100 4 2 2 0 100-4zM18 16a2 2 0 100 4 2 2 0 100-4zM2 14a2 2 0 100 4 2 2 0 100-4zM2 8a2 2 0 100 4 2 2 0 100-4zM13 12a2 2 0 100 4 2 2 0 100-4zM13 4a2 2 0 100 4 2 2 0 100-4z"],"layout-hierarchy":["M18.5 16.07v-4.14c.86-.22 1.5-1 1.5-1.93 0-1.1-.9-2-2-2-.93 0-1.71.64-1.93 1.5h-4.14c-.18-.7-.73-1.25-1.43-1.43V3.93c.86-.22 1.5-1 1.5-1.93 0-1.1-.9-2-2-2S8 .9 8 2c0 .93.64 1.71 1.5 1.93v4.14c-.7.18-1.25.73-1.43 1.43H3.93C3.71 8.64 2.93 8 2 8c-1.1 0-2 .9-2 2 0 .93.64 1.71 1.5 1.93v4.14c-.86.22-1.5 1-1.5 1.93 0 1.1.9 2 2 2s2-.9 2-2c0-.93-.64-1.71-1.5-1.93v-4.14c.7-.18 1.25-.73 1.43-1.43h4.14c.18.7.73 1.25 1.43 1.43v4.14c-.86.22-1.5 1-1.5 1.93 0 1.1.9 2 2 2s2-.9 2-2c0-.93-.64-1.71-1.5-1.93v-4.14c.7-.18 1.25-.73 1.43-1.43h4.14c.18.7.73 1.25 1.43 1.43v4.14c-.86.22-1.5 1-1.5 1.93 0 1.1.9 2 2 2s2-.9 2-2c0-.93-.64-1.71-1.5-1.93z"],"layout-linear":["M16.5 7a2.5 2.5 0 00-2.45 2h-2.1a2.5 2.5 0 00-4.9 0h-2.1a2.5 2.5 0 100 1h2.1a2.5 2.5 0 004.9 0h2.1a2.5 2.5 0 102.45-3z"],"layout-skew-grid":["M2 0a2 2 0 100 4 2 2 0 100-4zM18 0a2 2 0 100 4 2 2 0 100-4zM18 8a2 2 0 100 4 2 2 0 100-4zM18 16a2 2 0 100 4 2 2 0 100-4zM2 16a2 2 0 100 4 2 2 0 100-4zM2 8a2 2 0 100 4 2 2 0 100-4zM10 12a2 2 0 100 4 2 2 0 100-4zM10 4a2 2 0 100 4 2 2 0 100-4z"],"layout-sorted-clusters":["M2 8c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm0 8c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zM2 0C.9 0 0 .9 0 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm16 8c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm-8 4c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm0-8c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2z"],learning:["M10.551 1.127a1.256 1.256 0 00-1.102 0L.456 5.89c-.608.309-.608.913 0 1.222l8.993 4.762c.334.17.767.17 1.102 0l8.992-4.762c.61-.309.61-.913 0-1.222l-8.992-4.762z","M18 6.5l.016 4.514c.002.548.447.99.994.99a.99.99 0 00.99-.99V6.5h-2zM3.366 10.033l6.401 3.358a.5.5 0 00.465 0l6.406-3.358a.25.25 0 01.366.221v5.109a.25.25 0 01-.139.224l-6.64 3.302a.5.5 0 01-.446 0l-6.64-3.302A.25.25 0 013 15.363v-5.108a.25.25 0 01.366-.222z"],"left-join":["M8.7 4.7C7.4 6 6.5 7.9 6.5 10s.8 4 2.2 5.3c-.8.5-1.7.7-2.7.7-3.3 0-6-2.7-6-6s2.7-6 6-6c1 0 1.9.2 2.7.7zM14 4c3.3 0 6 2.7 6 6s-2.7 6-6 6c-1 0-1.9-.2-2.7-.7 1.3-1.3 2.2-3.2 2.2-5.3s-.8-3.9-2.2-5.3C12.1 4.2 13 4 14 4zm.6 2.05c.55 1.2.86 2.54.86 3.95s-.31 2.75-.86 3.95c1.9-.31 3.36-1.96 3.36-3.95S16.5 6.36 14.6 6.05zM10 5.5C8.8 6.7 8 8.2 8 10s.8 3.3 2 4.4c1.2-1.1 2-2.7 2-4.5s-.8-3.3-2-4.4z"],"less-than":["M7.162 10l9.154 3.052a1 1 0 01-.632 1.897l-12-4c-.912-.304-.912-1.594 0-1.897l12-4a1 1 0 01.632 1.897L7.162 10z"],"less-than-or-equal-to":["M16.316 11.051L7.162 8l9.154-3.051a1 1 0 10-.632-1.898l-12 4c-.912.304-.912 1.594 0 1.898l12 4a1 1 0 10.632-1.898zM16 15H4a1 1 0 100 2h12a1 1 0 100-2z"],lifesaver:["M8.143 14.644L7.028 17.43c.919.368 1.922.57 2.972.57s2.053-.202 2.972-.57l-1.115-2.786A4.986 4.986 0 0110 15a4.986 4.986 0 01-1.857-.356zm-2.787-2.787A4.986 4.986 0 015 10c0-.656.126-1.283.356-1.857L2.57 7.028A7.978 7.978 0 002 10c0 1.05.202 2.053.57 2.972l2.786-1.115zm2.787-6.5A4.986 4.986 0 0110 5c.656 0 1.283.126 1.857.356l1.115-2.786A7.978 7.978 0 0010 2c-1.05 0-2.053.202-2.972.57l1.115 2.786zm6.5 2.786c.23.574.357 1.2.357 1.857 0 .656-.126 1.283-.356 1.857l2.786 1.115c.368-.919.57-1.922.57-2.972s-.202-2.053-.57-2.972l-2.786 1.115zM10 13a3 3 0 100-6 3 3 0 000 6zm0 7C4.477 20 0 15.523 0 10S4.477 0 10 0s10 4.477 10 10-4.477 10-10 10z"],lightbulb:["M6.33 13.39c0 .34.27.61.6.61h6.13c.33 0 .6-.27.6-.61C14.03 9.78 16 9.4 16 6.09 16 2.72 13.31 0 10 0S4 2.72 4 6.09c0 3.31 1.97 3.69 2.33 7.3zM13 15H7c-.55 0-1 .45-1 1s.45 1 1 1h6c.55 0 1-.45 1-1s-.45-1-1-1zm-1 3H8c-.55 0-1 .45-1 1s.45 1 1 1h4c.55 0 1-.45 1-1s-.45-1-1-1z"],link:["M10.85 11.98l-4.44 4.44-1 1c-.36.36-.86.58-1.41.58-1.1 0-2-.9-2-2 0-.55.22-1.05.59-1.41l5.44-5.44C7.69 9.06 7.36 9 7 9c-1.11 0-2.09.46-2.82 1.18l-.01-.01-3 3 .01.01C.46 13.91 0 14.89 0 16c0 2.21 1.79 4 4 4 1.11 0 2.09-.46 2.82-1.18l.01.01 3-3-.01-.01C10.54 15.09 11 14.11 11 13c0-.36-.06-.69-.15-1.02zM20 4c0-2.21-1.79-4-4-4-1.11 0-2.09.46-2.82 1.18l-.01-.01-3 3 .01.01C9.46 4.91 9 5.89 9 7c0 .36.06.69.15 1.02l4.44-4.44 1-1c.36-.36.86-.58 1.41-.58 1.1 0 2 .9 2 2 0 .55-.22 1.05-.59 1.41l-5.44 5.44c.34.09.67.15 1.03.15 1.11 0 2.09-.46 2.82-1.18l.01.01 3-3-.01-.01C19.54 6.09 20 5.11 20 4zM5 14a1.003 1.003 0 001.71.71l8-8a1.003 1.003 0 00-1.42-1.42l-2 2-2 2-2 2-2 2c-.18.18-.29.43-.29.71z"],list:["M1.03 1C.46 1 0 1.46 0 2.03v.95C0 3.54.46 4 1.03 4h17.95C19.54 4 20 3.54 20 2.97v-.94C20 1.46 19.54 1 18.97 1H1.03zM0 17.97C0 18.54.46 19 1.03 19h17.95c.56 0 1.03-.46 1.03-1.03v-.95c0-.56-.46-1.03-1.03-1.03H1.03C.46 16 0 16.46 0 17.03v.94zM0 12.97C0 13.54.46 14 1.03 14h17.95c.56 0 1.03-.46 1.03-1.03v-.95c0-.56-.46-1.03-1.03-1.03H1.03C.46 11 0 11.46 0 12.03v.94zM0 7.97C0 8.54.46 9 1.03 9h17.95C19.54 9 20 8.54 20 7.97v-.94C20 6.46 19.54 6 18.97 6H1.03C.46 6 0 6.46 0 7.03v.94z"],"list-columns":["M0 2.973v-.936C0 1.468.46 1.01 1.029 1H7.97C8.541 1 9 1.468 9 2.027v.946C9 3.542 8.53 4 7.971 4H1.03C.459 4 0 3.542 0 2.973zm0 5v-.936C0 6.468.46 6.01 1.029 6H7.97C8.541 6 9 6.468 9 7.027v.946C9 8.542 8.53 9 7.971 9H1.03C.459 9 0 8.542 0 7.973zm0 5v-.936C0 11.468.46 11.01 1.029 11H7.97C8.541 11 9 11.468 9 12.027v.946C9 13.542 8.53 14 7.971 14H1.03C.459 14 0 13.542 0 12.973zm0 5v-.936C0 16.468.46 16.01 1.029 16H7.97C8.541 16 9 16.468 9 17.027v.946C9 18.542 8.53 19 7.971 19H1.03C.459 19 0 18.542 0 17.973zm11-15v-.936c0-.569.46-1.027 1.029-1.037h6.942C19.541 1 20 1.468 20 2.027v.946C20 3.542 19.53 4 18.971 4H12.03C11.459 4 11 3.542 11 2.973zm0 5v-.936c0-.569.46-1.027 1.029-1.037h6.942C19.541 6 20 6.468 20 7.027v.946C20 8.542 19.53 9 18.971 9H12.03C11.459 9 11 8.542 11 7.973zm0 5v-.936c0-.569.46-1.027 1.029-1.037h6.942c.57 0 1.029.468 1.029 1.027v.946c0 .569-.47 1.027-1.029 1.027H12.03c-.57 0-1.029-.458-1.029-1.027zm0 5v-.936c0-.569.46-1.027 1.029-1.037h6.942c.57 0 1.029.468 1.029 1.027v.946c0 .569-.47 1.027-1.029 1.027H12.03c-.57 0-1.029-.458-1.029-1.027z"],"list-detail-view":["M8 6H1c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h7c.55 0 1-.45 1-1V7c0-.55-.45-1-1-1zm0 5H1c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h7c.55 0 1-.45 1-1v-1c0-.55-.45-1-1-1zm0 5H1c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h7c.55 0 1-.45 1-1v-1c0-.55-.45-1-1-1zM8 1H1c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h7c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1zm11 0h-7c-.55 0-1 .45-1 1v16c0 .55.45 1 1 1h7c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1z"],locate:["M10 8c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm9 1h-1.07c-.45-3.61-3.32-6.45-6.93-6.91V1c0-.55-.45-1-1-1S9 .45 9 1v1.09C5.39 2.55 2.52 5.39 2.07 9H1c-.55 0-1 .45-1 1s.45 1 1 1h1.07c.45 3.61 3.32 6.45 6.93 6.91V19c0 .55.45 1 1 1s1-.45 1-1v-1.09c3.61-.46 6.48-3.29 6.93-6.91H19c.55 0 1-.45 1-1s-.45-1-1-1zm-4 2h.9a5.98 5.98 0 01-4.9 4.91V15c0-.55-.45-1-1-1s-1 .45-1 1v.91A5.98 5.98 0 014.1 11H5c.55 0 1-.45 1-1s-.45-1-1-1h-.9A5.98 5.98 0 019 4.09V5c0 .55.45 1 1 1s1-.45 1-1v-.91A5.98 5.98 0 0115.9 9H15c-.55 0-1 .45-1 1s.45 1 1 1z"],lock:["M15.93 9H14V4.99c0-2.21-1.79-4-4-4s-4 1.79-4 4V9H3.93c-.55 0-.93.44-.93.99v8c0 .55.38 1.01.93 1.01h12c.55 0 1.07-.46 1.07-1.01v-8c0-.55-.52-.99-1.07-.99zM8 9V4.99c0-1.1.9-2 2-2s2 .9 2 2V9H8z"],"log-in":["M19 0h-8c-.55 0-1 .45-1 1s.45 1 1 1h7v16h-7c-.55 0-1 .45-1 1s.45 1 1 1h8c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-4 10c0-.28-.11-.53-.29-.71l-5-5a1.003 1.003 0 00-1.42 1.42L11.59 9H1c-.55 0-1 .45-1 1s.45 1 1 1h10.59L8.3 14.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71l5-5c.18-.18.29-.43.29-.71z"],"log-out":["M19.71 9.29l-5-5a1.003 1.003 0 00-1.42 1.42L16.59 9H6c-.55 0-1 .45-1 1s.45 1 1 1h10.59l-3.29 3.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71l5-5c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71zM9 18H2V2h7c.55 0 1-.45 1-1s-.45-1-1-1H1C.45 0 0 .45 0 1v18c0 .55.45 1 1 1h8c.55 0 1-.45 1-1s-.45-1-1-1z"],manual:["M20 1.1a.976.976 0 00-.83-.88C15.15-.43 12.07.34 10 2.5 7.93.34 4.85-.43.84.22.37.3.03.67 0 1.1v15.01c0 .07 0 .14.01.21.09.52.61.88 1.15.79 3.85-.62 6.4.16 8 2.46.02.02.03.04.05.07.02.02.04.04.06.07l.01.01a1.07 1.07 0 00.28.19c.01 0 .01.01.02.01.03.01.07.03.1.04.01 0 .02.01.04.01.03.01.07.02.1.02.01 0 .02 0 .04.01H10c.04 0 .09 0 .13-.01.01 0 .03 0 .04-.01.03-.01.06-.01.1-.02.01 0 .03-.01.04-.01.03-.01.07-.02.1-.04.01 0 .02-.01.03-.01.07-.03.13-.07.19-.11.01 0 .01-.01.02-.01.02-.02.04-.03.06-.05.01-.01.02-.02.03-.02l.05-.05c.01-.01.02-.02.02-.03.01-.02.02-.03.04-.05 1.61-2.3 4.15-3.09 8-2.46.54.09 1.06-.26 1.15-.79-.01-.05 0-.09 0-.13V1.1zM9 16.63c-1.78-1.31-4.12-1.83-7-1.55V2c3.26-.37 5.51.39 7 2.35v12.28zm9-1.56c-2.88-.28-5.22.24-7 1.55V4.34c1.49-1.96 3.74-2.71 7-2.35v13.08z"],"manually-entered-data":["M1 12h4.34l2-2H1c-.55 0-1 .45-1 1s.45 1 1 1zm16.77-3.94l1.65-1.65c.36-.36.58-.86.58-1.41 0-1.1-.9-2-2-2-.55 0-1.05.22-1.41.59l-1.65 1.65 2.83 2.82zM1 4h12.34l2-2H1c-.55 0-1 .45-1 1s.45 1 1 1zM0 15c0 .55.45 1 1 1h.34l2-2H1c-.55 0-1 .45-1 1zm1-7h8.34l2-2H1c-.55 0-1 .45-1 1s.45 1 1 1zm18 2h-.34l-2 2H19c.55 0 1-.45 1-1s-.45-1-1-1zm0 4h-4.34l-2 2H19c.55 0 1-.45 1-1s-.45-1-1-1zM4 19l4.41-1.59-2.81-2.79L4 19zM14.23 5.94l-7.65 7.65 2.83 2.83 7.65-7.65-2.83-2.83z"],map:["M19.54 4.18l.01-.02-6-4-.01.02C13.39.08 13.21 0 13 0s-.39.08-.54.18l-.01-.02L7 3.8 1.55.17l-.01.01A.969.969 0 001 0C.45 0 0 .45 0 1v14c0 .35.19.64.46.82l-.01.02 6 4 .01-.02c.15.1.33.18.54.18s.39-.08.54-.18l.01.02L13 16.2l5.45 3.63.01-.02c.15.11.33.19.54.19.55 0 1-.45 1-1V5c0-.35-.19-.64-.46-.82zM6 17.13l-4-2.67V2.87l4 2.67v11.59zm6-2.67l-4 2.67V5.54l4-2.67v11.59zm6 2.67l-4-2.67V2.87l4 2.67v11.59z"],"map-create":["M18 9.22v7.91l-4-2.67V9.22c-.61-.55-1-1.33-1-2.22-.35 0-.69-.07-1-.18v7.65l-4 2.67V5.54l2.02-1.35c0-.06-.02-.13-.02-.19 0-1.66 1.34-3 3-3 0-.34.07-.66.17-.97C13.12.02 13.06 0 13 0c-.21 0-.39.08-.54.18l-.01-.02L7 3.8 1.55.17l-.01.01A.969.969 0 001 0C.45 0 0 .45 0 1v14c0 .35.19.64.46.82l-.01.02 6 4 .01-.02c.15.1.33.18.54.18s.39-.08.54-.18l.01.02L13 16.2l5.45 3.63.01-.02c.15.11.33.19.54.19.55 0 1-.45 1-1V6.82c-.31.11-.65.18-1 .18 0 .89-.39 1.67-1 2.22zM6 17.13l-4-2.67V2.87l4 2.67v11.59zM12 4c0 .55.45 1 1 1h2v2c0 .55.45 1 1 1s1-.45 1-1V5h2c.55 0 1-.45 1-1s-.45-1-1-1h-2V1c0-.55-.45-1-1-1s-1 .45-1 1v2h-2c-.55 0-1 .45-1 1z"],"map-marker":["M9.98 0c-3.87 0-7 2.98-7 6.67 0 3.68 7 13.33 7 13.33s7-9.65 7-13.33c0-3.68-3.14-6.67-7-6.67zm0 10c-1.66 0-3-1.34-3-3s1.34-3 3-3 3 1.34 3 3-1.34 3-3 3z"],maximize:["M19 0h-5c-.55 0-1 .45-1 1s.45 1 1 1h2.59L11.3 7.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71L18 3.41V6c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1zM8 11c-.28 0-.53.11-.71.29L2 16.59V14c0-.55-.45-1-1-1s-1 .45-1 1v5c0 .55.45 1 1 1h5c.55 0 1-.45 1-1s-.45-1-1-1H3.41l5.29-5.29c.19-.18.3-.43.3-.71 0-.55-.45-1-1-1z"],media:["M15 9c1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2 .9 2 2 2zm4-7H1c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1zm-1 13l-6-5-2 2-4-5-4 8V4h16v11z"],menu:["M1 6h18c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1s.45 1 1 1zm18 3H1c-.55 0-1 .45-1 1s.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1zm0 5H1c-.55 0-1 .45-1 1s.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1z"],"menu-closed":["M8 6h11c.55 0 1-.45 1-1s-.45-1-1-1H8c-.55 0-1 .45-1 1s.45 1 1 1zM4 6c-.28 0-.53.11-.71.29l-3 3C.11 9.47 0 9.72 0 10c0 .28.11.53.29.71l3 3A1.003 1.003 0 005 13V7c0-.55-.45-1-1-1zm15 8H8c-.55 0-1 .45-1 1s.45 1 1 1h11c.55 0 1-.45 1-1s-.45-1-1-1zm0-5H8c-.55 0-1 .45-1 1s.45 1 1 1h11c.55 0 1-.45 1-1s-.45-1-1-1z"],"menu-open":["M12 9H1c-.55 0-1 .45-1 1s.45 1 1 1h11c.55 0 1-.45 1-1s-.45-1-1-1zm0 5H1c-.55 0-1 .45-1 1s.45 1 1 1h11c.55 0 1-.45 1-1s-.45-1-1-1zm0-10H1c-.55 0-1 .45-1 1s.45 1 1 1h11c.55 0 1-.45 1-1s-.45-1-1-1zm7.71 5.29l-3-3A1.003 1.003 0 0015 7v6a1.003 1.003 0 001.71.71l3-3c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71z"],"merge-columns":["M6.71 6.29a1.003 1.003 0 00-1.42 1.42L6.59 9H2V2h5v2.18c.42.15.8.39 1.11.7l.01-.01.88.89V1c0-.55-.45-1-1-1H1C.45 0 0 .45 0 1v18c0 .55.45 1 1 1h7c.55 0 1-.45 1-1v-4.76l-.88.88-.01-.01c-.31.31-.69.56-1.11.71V18H2v-7h4.59L5.3 12.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71l3-3c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71l-3-3zM19 0h-7c-.55 0-1 .45-1 1v4.76l.88-.88.01.01c.31-.31.69-.55 1.11-.7V2h5v7h-4.59l1.29-1.29c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71l-3 3c-.18.18-.29.43-.29.71 0 .28.11.53.29.71l3 3a1.003 1.003 0 001.42-1.42L13.41 11H18v7h-5v-2.18c-.42-.15-.8-.39-1.11-.7l-.01.01-.88-.89V19c0 .55.45 1 1 1h7c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1z"],"merge-links":["M10 13c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm8-5c-.93 0-1.71.64-1.93 1.5H14V4c0-2.21-1.79-4-4-4S6 1.79 6 4v5.5H3.93C3.71 8.64 2.93 8 2 8c-1.1 0-2 .9-2 2s.9 2 2 2c.93 0 1.71-.64 1.93-1.5H6V16c0 2.21 1.79 4 4 4s4-1.79 4-4v-5.5h2.07c.22.86 1 1.5 1.93 1.5 1.1 0 2-.9 2-2s-.9-2-2-2zm-5 8c0 1.66-1.34 3-3 3s-3-1.34-3-3V4c0-1.66 1.34-3 3-3s3 1.34 3 3v12zM10 3c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2z"],minimize:["M8 11H3c-.55 0-1 .45-1 1s.45 1 1 1h2.59L.3 18.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71L7 14.41V17c0 .55.45 1 1 1s1-.45 1-1v-5c0-.55-.45-1-1-1zM20 1a1.003 1.003 0 00-1.71-.71L13 5.59V3c0-.55-.45-1-1-1s-1 .45-1 1v5c0 .55.45 1 1 1h5c.55 0 1-.45 1-1s-.45-1-1-1h-2.59l5.29-5.29c.19-.18.3-.43.3-.71z"],minus:["M16 9H4c-.55 0-1 .45-1 1s.45 1 1 1h12c.55 0 1-.45 1-1s-.45-1-1-1z"],"mobile-phone":["M15 0H5c-.55 0-1 .45-1 1v18c0 .55.45 1 1 1h10c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-5 19c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zm4-3H6V3h8v13z"],"mobile-video":["M19 5c-.28 0-.53.11-.71.29L15 8.59V5c0-.55-.45-1-1-1H1c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1h13c.55 0 1-.45 1-1v-3.59l3.29 3.29c.18.19.43.3.71.3.55 0 1-.45 1-1V6c0-.55-.45-1-1-1z"],modal:["M19 1a1 1 0 011 1v16a1 1 0 01-1 1H1a1 1 0 01-1-1V2a1 1 0 011-1h18zm-1 4H2v12h16V5zm-3-3h-2v2h2V2zm3 0h-2v2h2V2z"],"modal-filled":["M20 5v13a1 1 0 01-1 1H1a1 1 0 01-1-1V5h20zm-3-4h2a1 1 0 011 1v1h-3V1zm-2 2H0V2a1 1 0 011-1h14v2z"],moon:["M19 14.15A9.94 9.94 0 019.94 20C4.45 20 0 15.55 0 10.06 0 6.03 2.4 2.56 5.85 1a9.811 9.811 0 00-.88 4.09c0 5.49 4.45 9.94 9.94 9.94 1.46 0 2.84-.31 4.09-.88z"],more:["M3.5 8a2.5 2.5 0 100 5 2.5 2.5 0 100-5zM17.5 8a2.5 2.5 0 100 5 2.5 2.5 0 100-5zM10.5 8a2.5 2.5 0 100 5 2.5 2.5 0 100-5z"],mountain:["M20 16H4l7-11h1l2 2h1l5 9zm-4-5l-1.5-3h-1l-1-1-1-1L8 11.5l3-1.5 1 1 1-1 3 1zM8.055 8L2.79 16H0l7-8h1.055z"],move:["M19.71 9.29l-3-3a1.003 1.003 0 00-1.42 1.42L16.59 9H11V3.41l1.29 1.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71l-3-3C10.53.11 10.28 0 10 0s-.53.11-.71.29l-3 3a1.003 1.003 0 001.42 1.42L9 3.41V9H3.41L4.7 7.71c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71l-3 3C.11 9.47 0 9.72 0 10c0 .28.11.53.29.71l3 3a1.003 1.003 0 001.42-1.42L3.41 11H9v5.59L7.71 15.3A.965.965 0 007 15a1.003 1.003 0 00-.71 1.71l3 3c.18.18.43.29.71.29s.53-.11.71-.29l3-3a1.003 1.003 0 00-1.42-1.42L11 16.59V11h5.59l-1.29 1.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71l3-3c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71z"],mugshot:["M19 0H1C.45 0 0 .45 0 1v18c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 18h-.07c-.05-.2-.12-.42-.22-.67-.46-1.05-2.68-1.75-4.16-2.4-1.48-.65-1.28-1.05-1.33-1.59-.01-.07-.01-.15-.01-.23.51-.45.92-1.07 1.19-1.78 0 0 .01-.04.02-.05.06-.15.11-.32.15-.48.34-.07.54-.44.61-.78.08-.14.23-.48.2-.87-.05-.5-.25-.73-.47-.82v-.09c0-.63-.06-1.55-.17-2.15-.02-.17-.06-.33-.11-.5a3.67 3.67 0 00-1.29-1.86C11.7 3.25 10.81 3 10.02 3s-1.68.25-2.31.73c-.61.47-1.07 1.13-1.29 1.86-.05.16-.09.33-.11.5-.12.6-.17 1.51-.17 2.14v.08c-.24.09-.44.32-.49.83-.04.39.12.73.2.87.08.35.28.72.63.78.04.17.09.33.15.48 0 .01.01.02.01.03l.01.01c.27.72.7 1.35 1.22 1.8 0 .07-.01.14-.01.21-.05.54.1.94-1.38 1.59-1.48.65-3.7 1.35-4.16 2.4-.12.27-.18.49-.23.69H2V2h16v16z"],"multi-select":["M19 3H7c-.55 0-1 .45-1 1v1h12v6h1c.55 0 1-.45 1-1V4c0-.55-.45-1-1-1zm-6 6H1c-.55 0-1 .45-1 1v6c0 .55.45 1 1 1h12c.55 0 1-.45 1-1v-6c0-.55-.45-1-1-1zm-1 6H2v-4h10v4zm4-9H4c-.55 0-1 .45-1 1v1h12v6h1c.55 0 1-.45 1-1V7c0-.55-.45-1-1-1z"],music:["M19 0c-.08 0-.16.03-.24.05V.03l-12 3v.02C6.33 3.16 6 3.53 6 4v11.35c-.59-.22-1.27-.35-2-.35-2.21 0-4 1.12-4 2.5S1.79 20 4 20c1.94 0 3.55-.86 3.92-2H8V7.78l10-2.5v7.07c-.59-.22-1.27-.35-2-.35-2.21 0-4 1.12-4 2.5s1.79 2.5 4 2.5c1.94 0 3.55-.86 3.92-2H20V1c0-.55-.45-1-1-1z"],"new-drawing":["M18.7 13.7c.5 0 1 .4 1 1 0 .257-.073.44-.22.614l-.08.086-4 4c-.2.2-.4.3-.7.3-.6 0-1-.5-1-1 0-.257.073-.44.22-.614L14 18l4-4c.2-.2.4-.3.7-.3zM1.8 0l8.378 2.982A3.003 3.003 0 0013 7a3.003 3.003 0 003.877 2.87l.723 2.53.049.06a.41.41 0 01.051.24c0 .167-.07.403-.208.593l-.092.107-4 4c-.2.2-.4.3-.7.3-.075 0-.15-.056-.225-.084L12.4 17.6l-7-2-.112-.042c-.223-.094-.431-.244-.542-.45L4.7 15 0 1.8l.5-.6L7 7.7c-.2.3-.3.6-.3 1 0 1.1.9 2 2 2s2-.9 2-2-.9-2-2-2a1.68 1.68 0 00-.871.22L7.7 7 1.2.5l.6-.5zM16 0c.55 0 1 .45 1 1v2h2c.55 0 1 .45 1 1s-.45 1-1 1h-2v2c0 .432-.278.803-.664.941l-.01.004A.989.989 0 0116 8c-.55 0-1-.45-1-1V5h-2c-.55 0-1-.45-1-1l.007-.116C12.065 3.388 12.489 3 13 3h2V1c0-.55.45-1 1-1z"],"new-grid-item":["M8 0H1C.45 0 0 .45 0 1v7c0 .55.45 1 1 1h7c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm0 11H1c-.55 0-1 .45-1 1v7c0 .55.45 1 1 1h7c.55 0 1-.45 1-1v-7c0-.55-.45-1-1-1zm6 7h-1v-1c0-.55-.45-1-1-1s-1 .45-1 1v2c0 .55.45 1 1 1h2c.55 0 1-.45 1-1s-.45-1-1-1zm5-7h-2c-.55 0-1 .45-1 1s.45 1 1 1h1v1c0 .55.45 1 1 1s1-.45 1-1v-2c0-.55-.45-1-1-1zm0-11h-7c-.55 0-1 .45-1 1v7c0 .55.45 1 1 1h7c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-5 11h-2c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1s1-.45 1-1v-1h1c.55 0 1-.45 1-1s-.45-1-1-1zm5 5c-.55 0-1 .45-1 1v1h-1c-.55 0-1 .45-1 1s.45 1 1 1h2c.55 0 1-.45 1-1v-2c0-.55-.45-1-1-1z"],"new-layer":["M11.513 2.663A2 2 0 0013 6h1v1a2 2 0 104 0v-.733l1.5.833c.3.2.5.5.5.9s-.2.7-.5.9l-9 5c-.2.1-.3.1-.5.1s-.3 0-.5-.1l-9-5C.2 8.7 0 8.4 0 8s.2-.7.5-.9l9-5c.2-.1.3-.1.5-.1s.3 0 .5.1l1.013.563zM17 3h2a1 1 0 010 2h-2v2a1 1 0 01-2 0V5h-2a1 1 0 010-2h2V1a1 1 0 012 0v2z"],"new-layers":["M17 3h2a1 1 0 010 2h-2v2a1 1 0 01-2 0V5h-2a1 1 0 010-2h2V1a1 1 0 012 0v2zm-1.252 5.984L10.5 11.9c-.2.1-.3.1-.5.1s-.3 0-.5-.1l-9-5C.2 6.7 0 6.4 0 6s.2-.7.5-.9l9-5c.2-.1.3-.1.5-.1s.3 0 .5.1L13.92 2H13a2 2 0 100 4h1v1a2 2 0 001.748 1.984zm2.07-1.15C17.935 7.58 18 7.298 18 7V6h1c.353 0 .684-.091.972-.251.018.078.028.162.028.251 0 .4-.2.7-.5.9l-1.682.934zM19 9c.6 0 1 .4 1 1 0 .4-.2.7-.5.9l-9 5c-.2.1-.3.1-.5.1s-.3 0-.5-.1l-9-5c-.3-.2-.5-.5-.5-.9 0-.6.4-1 1-1 .2 0 .3 0 .5.1l8.5 4.8 8.5-4.8c.2-.1.3-.1.5-.1zm0 4c.6 0 1 .4 1 1 0 .4-.2.7-.5.9l-9 5c-.2.1-.3.1-.5.1s-.3 0-.5-.1l-9-5c-.3-.2-.5-.5-.5-.9 0-.6.4-1 1-1 .2 0 .3 0 .5.2l8.5 4.7 8.5-4.8c.2-.1.3-.1.5-.1z"],"new-link":["M14.5 12a2.5 2.5 0 00-2.45 2h-7.1a2.5 2.5 0 100 1h7.1a2.5 2.5 0 102.45-3zM19 5h-2V3c0-.55-.45-1-1-1s-1 .45-1 1v2h-2c-.55 0-1 .45-1 1s.45 1 1 1h2v2c0 .55.45 1 1 1s1-.45 1-1V7h2c.55 0 1-.45 1-1s-.45-1-1-1z"],"new-object":["M12 4c0 .6.4 1 1 1h2v2c0 .6.4 1 1 1 .5 0 1-.4 1-1V5h2c.5 0 1-.4 1-1s-.5-1-1-1h-2V1c0-.6-.5-1-1-1-.6 0-1 .4-1 1v2h-2c-.6 0-1 .5-1 1zm7 3c0 1.7-1.3 3-3 3s-3-1.3-3-3c-1.7 0-3-1.3-3-3s1.3-3 3-3c0-.2 0-.4.1-.5-1-.3-2-.5-3.1-.5C4.5 0 0 4.5 0 10s4.5 10 10 10 10-4.5 10-10c0-1.1-.2-2.1-.5-3H19z"],"new-person":["M11.41 15.92c-1.46-.65-1.26-1.05-1.31-1.59-.01-.07-.01-.15-.01-.23.5-.45.91-1.07 1.18-1.78 0 0 .01-.04.02-.05.06-.15.11-.32.15-.48.33-.07.53-.44.6-.78.08-.14.23-.48.2-.87-.05-.5-.24-.73-.47-.82v-.09c0-.63-.06-1.55-.17-2.15-.02-.17-.06-.33-.11-.5-.22-.73-.67-1.4-1.27-1.86C9.58 4.25 8.7 4 7.92 4c-.78 0-1.66.25-2.28.73-.61.47-1.06 1.13-1.27 1.86-.05.16-.08.33-.11.5-.12.6-.18 1.51-.18 2.14v.08c-.23.09-.43.32-.48.83-.04.39.12.73.2.87.08.35.28.72.62.78.04.17.09.33.15.48 0 .01.01.02.01.03l.01.01c.27.72.69 1.35 1.21 1.8 0 .07-.01.14-.01.21-.05.54.1.94-1.36 1.59-1.46.65-3.66 1.35-4.11 2.4C-.14 19.38.04 20 .04 20h15.75s.18-.62-.27-1.67c-.45-1.06-2.65-1.75-4.11-2.41zM18.87 3h-2V1c0-.55-.45-1-1-1s-1 .45-1 1v2h-2c-.55 0-1 .45-1 1s.45 1 1 1h2v2c0 .55.45 1 1 1s1-.45 1-1V5h2c.55 0 1-.45 1-1s-.45-1-1-1z"],"new-prescription":["M11.95 10.23c.16-.18.22-.22.46-.22h1.48c.25 0 .47.08.59.33.1.2.09.41-.05.66l-2.71 3.58L14.88 19c.13.21.16.46.03.69-.12.21-.34.31-.57.31H12.7c-.31 0-.56-.17-.7-.44l-1.9-2.67-1.93 2.68c-.15.27-.42.43-.73.43H5.98c-.25 0-.47-.08-.59-.33-.1-.2-.09-.41.05-.66l3.09-4.35L4.26 9H3v4.32c0 .41-.3.69-.7.69H.7c-.41 0-.7-.28-.7-.69V.69C0 .28.3 0 .7 0h4.42c.71 0 1.36.1 1.94.3.59.2 1.11.49 1.54.87.44.38.78.84 1.02 1.39.25.54.37 1.13.37 1.77 0 1.01-.28 1.88-.84 2.6-.43.54-1.35 1.29-2 1.59l3.09 3.94 1.71-2.23zM4.71 6.04c.71 0 1.45-.16 1.81-.46.33-.28.5-.69.5-1.25s-.17-.97-.5-1.25c-.35-.3-1.1-.46-1.81-.46h-1.7v3.42h1.7zM19 3c.55 0 1 .45 1 1s-.45 1-1 1h-2v2c0 .55-.45 1-1 1s-1-.45-1-1V5h-2c-.55 0-1-.45-1-1s.45-1 1-1h2V1c0-.55.45-1 1-1s1 .45 1 1v2h2z"],"new-text-box":["M19 3h-2V1c0-.55-.45-1-1-1s-1 .45-1 1v2h-2c-.55 0-1 .45-1 1s.45 1 1 1h2v2c0 .55.45 1 1 1s1-.45 1-1V5h2c.55 0 1-.45 1-1s-.45-1-1-1zM5 7.5v1c0 .28.22.5.5.5s.5-.22.5-.5V8h2v7h-.5c-.28 0-.5.22-.5.5s.22.5.5.5h2c.28 0 .5-.22.5-.5s-.22-.5-.5-.5H9V8h2v.5c0 .28.22.5.5.5s.5-.22.5-.5v-1c0-.28-.22-.5-.5-.5h-6c-.28 0-.5.22-.5.5zM16 9c-.55 0-1 .45-1 1v8H2V5h8c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1v15c0 .55.45 1 1 1h15c.55 0 1-.45 1-1v-9c0-.55-.45-1-1-1z"],ninja:["M20 6s-2.98 2.43-6.12 2.19C13.52 5.31 12.05 0 6 0c0 0 2.41 2.99 2.16 6.12C5.27 6.49 0 7.97 0 14c0 0 2.98-2.43 6.11-2.19C6.47 14.69 7.94 20 14 20c0 0-2.42-2.99-2.16-6.13C14.73 13.51 20 12.02 20 6zm-10 6c-1.1 0-2-.9-2-2s.9-2 2-2 2 .9 2 2-.9 2-2 2z"],"not-equal-to":["M9.487 7l.532-3.196a1 1 0 011.962.392L11.513 7H16a1 1 0 010 2h-4.82l-.333 2H16a1 1 0 010 2h-5.487l-.532 3.196a1 1 0 01-1.962-.392L8.487 13H4a1 1 0 010-2h4.82l.333-2H4a1 1 0 110-2h5.487z"],notifications:["M10 20c1.1 0 2-.9 2-2H8c0 1.1.9 2 2 2zm7-5c-.55 0-1-.45-1-1V8c0-2.61-1.67-4.81-4-5.63V2c0-1.1-.9-2-2-2S8 .9 8 2v.37C5.67 3.19 4 5.39 4 8v6c0 .55-.45 1-1 1s-1 .45-1 1 .45 1 1 1h14c.55 0 1-.45 1-1s-.45-1-1-1z"],"notifications-snooze":["M10 18c0 1.1-.9 2-2 2s-2-.9-2-2zM8 0c.476 0 .914.168 1.258.448C8.508.878 8.09 1.562 8 2.5c-.133 1.4.4 2.367 1.6 2.9C8.533 6.6 8 7.467 8 8v1.2a2.8 2.8 0 002.8 2.8H14v2c0 .51.388.935.884.993L15 15c.55 0 1 .45 1 1s-.45 1-1 1H1c-.55 0-1-.45-1-1s.45-1 1-1 1-.45 1-1V8c0-2.61 1.67-4.81 4-5.63V2c0-1.1.9-2 2-2z","M16 9.25v-.395a.75.75 0 00-.75-.75h-2.813L15.834 3.9A.75.75 0 0016 3.43v-.68a.75.75 0 00-.75-.75h-4.5a.75.75 0 00-.75.75v.184c0 .414.336.75.75.75h2.813L10.22 7.831a1 1 0 00-.221.627v.792c0 .414.336.75.75.75h4.5a.75.75 0 00.75-.75z"],"notifications-updated":["M10 20c1.1 0 2-.9 2-2H8c0 1.1.9 2 2 2zm2-17.834A2.994 2.994 0 008 4.99c0 .808.319 1.557.876 2.114l2.97 2.99a2.99 2.99 0 004.154.072V14c0 .55.45 1 1 1s1 .45 1 1-.45 1-1 1H3c-.55 0-1-.45-1-1s.45-1 1-1 1-.45 1-1V8c0-2.61 1.67-4.81 4-5.63V2c0-1.1.9-2 2-2s2 .9 2 2v.166zm1.26 6.514l-2.97-2.99a.973.973 0 01-.29-.7c0-.55.44-1 .99-1 .27 0 .52.11.7.29l2.28 2.28 4.27-4.27a.99.99 0 01.7-.29c.55 0 1 .45 1 1 0 .28-.11.53-.3.7l-4.98 4.98a.99.99 0 01-1.4 0z"],"numbered-list":["M1.74 9.01h1.27V1h-.95c-.04.24-.12.45-.26.62-.13.17-.29.3-.47.41-.19.11-.4.18-.63.23-.23.04-.46.07-.71.07v1.03h1.75v5.65zm.43 7.93c.18-.14.37-.28.58-.43.21-.14.42-.29.63-.45.21-.16.41-.33.61-.5.2-.18.37-.38.52-.59.15-.21.28-.45.37-.7.09-.25.14-.54.14-.85 0-.25-.04-.52-.12-.8-.08-.28-.21-.54-.39-.78-.19-.24-.43-.44-.73-.59-.3-.17-.68-.25-1.12-.25-.41 0-.77.08-1.08.23-.32.16-.58.37-.8.64-.22.27-.38.59-.49.96-.11.37-.16.77-.16 1.21h1.19c.01-.28.03-.53.08-.77s.12-.45.21-.62c.09-.18.22-.31.38-.42.16-.1.35-.15.59-.15.26 0 .47.05.63.14.16.09.29.21.38.35.09.14.15.29.18.45.03.16.05.31.05.45-.01.31-.08.58-.22.81-.14.24-.32.45-.53.66-.22.2-.45.39-.71.57-.26.18-.51.36-.74.54-.5.36-.89.78-1.17 1.27-.3.47-.45 1.04-.46 1.69H5v-1.14H1.43c.05-.17.14-.33.27-.49.13-.15.29-.3.47-.44zM18 4.02H8c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h10c.55 0 1-.45 1-1v-1c0-.56-.45-1-1-1zm0 9H8c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h10c.55 0 1-.45 1-1v-1c0-.56-.45-1-1-1z"],numerical:["M2.39 5.75c-.17.21-.38.39-.63.52s-.52.23-.83.29c-.3.05-.61.08-.93.08v1.24h2.49V15h1.49V4.98H2.73c-.05.31-.17.57-.34.77zm17.2 4.71c-.27-.44-.65-.71-1.14-.82v-.02c.42-.16.72-.43.92-.79.2-.36.29-.79.29-1.27 0-.42-.08-.8-.23-1.12-.15-.33-.36-.59-.62-.8-.26-.21-.55-.37-.87-.48-.32-.11-.65-.16-.98-.16-.43 0-.82.08-1.16.25-.34.16-.63.39-.87.69-.24.29-.43.64-.57 1.04-.14.4-.22.83-.23 1.3h1.39c-.01-.25.02-.49.07-.72.06-.23.14-.44.26-.63s.27-.34.45-.45c.18-.11.39-.17.63-.17.39 0 .71.12.96.37s.37.58.37.99c0 .29-.05.54-.16.74-.11.2-.25.36-.43.47-.18.11-.38.19-.61.24-.23.05-.46.06-.68.05v1.17c.28-.01.55 0 .81.03s.5.1.71.21c.21.11.38.28.51.5.13.22.2.52.2.89 0 .55-.16.97-.47 1.27-.31.3-.7.45-1.17.45-.55 0-.95-.19-1.23-.58-.27-.39-.4-.88-.38-1.46h-1.39c.01.5.08.96.21 1.38.13.41.32.77.57 1.06.25.29.56.52.93.68.37.16.8.24 1.3.24.41 0 .79-.07 1.16-.21.37-.14.69-.33.96-.58.28-.25.5-.56.66-.92a3 3 0 00.24-1.23c0-.64-.14-1.17-.41-1.61zM8.58 12.41c.21-.18.45-.36.7-.53.25-.18.5-.36.75-.56.25-.2.49-.41.73-.63.23-.22.44-.47.63-.74.18-.27.33-.56.44-.88.11-.32.16-.67.16-1.07 0-.32-.05-.65-.14-1-.09-.35-.25-.68-.47-.97-.22-.3-.51-.55-.87-.74-.36-.2-.81-.29-1.35-.29-.49 0-.93.1-1.3.29-.37.18-.69.44-.95.78-.26.33-.45.73-.58 1.2-.13.46-.2.96-.2 1.5h1.43c.01-.35.04-.67.09-.97.05-.3.14-.56.25-.78.11-.22.26-.39.45-.52s.43-.19.71-.19c.31 0 .56.06.75.18.19.12.34.26.45.43.11.17.18.36.22.56.04.2.06.39.06.57-.01.38-.1.72-.26 1.02-.15.3-.37.57-.63.83-.26.25-.54.49-.85.71-.31.22-.61.45-.89.68-.6.45-1.06.98-1.41 1.58-.35.61-.52 1.32-.53 2.13h6.01v-1.43H7.69c.06-.21.17-.42.33-.61s.34-.38.56-.55z"],office:["M19 6h-5V1c0-.55-.45-1-1-1H1C.45 0 0 .45 0 1v18c0 .55.45 1 1 1h4v-6h4v6h10c.55 0 1-.45 1-1V7c0-.55-.45-1-1-1zM6 12H2V8h4v4zm0-6H2V2h4v4zm6 6H8V8h4v4zm0-6H8V2h4v4zm6 11h-4v-3h4v3zm0-5h-4V8h4v4z"],offline:["M10 0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zM7 18l2-7H5l8-9-2 7h4l-8 9z"],"oil-field":["M19 17.99h-1.36l-4.35-9.57 2.91-.86 1.66 4.1c.11.27.43.4.72.31.12-.04.22-.11.28-.2.06-.11 1.47-2.08 1.05-5.6C19.79 5.12 19.3 0 16.01 0 14.89.01 13.99.83 14 1.84c0 .19.04.38.1.56l1.34 3.31L.72 10.03v.02c-.41.12-.72.49-.72.94 0 .55.45 1 1 1 .1 0 .19-.03.28-.06v.02l2-.59 1.47 6.63H3c-.55 0-1 .45-1 1s.45 1 1 1h16c.55 0 1-.45 1-1s-.45-1-1-1zM5.2 10.8l3.95-1.16-2.83 6.22L5.2 10.8zm2.35 7.19l3.95-8.68 3.95 8.68h-7.9z"],"one-column":["M14.94 0h-4c-.55 0-1 .45-1 1v18c0 .55.45 1 1 1h4c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-8 6c-.28 0-.53.11-.71.29l-3 3c-.18.18-.29.43-.29.71s.11.53.29.71l3 3A1.003 1.003 0 007.94 13V7c0-.55-.45-1-1-1z"],outdated:["M10 0c5.52 0 10 4.48 10 10s-4.48 10-10 10S0 15.52 0 10c0-.55.45-1 1-1s1 .45 1 1c0 4.42 3.58 8 8 8s8-3.58 8-8-3.58-8-8-8C7.47 2 5.22 3.17 3.76 5H5c.55 0 1 .45 1 1s-.45 1-1 1H1c-.55 0-1-.45-1-1V2c0-.55.45-1 1-1s1 .45 1 1v2.05C3.82 1.6 6.71 0 10 0zm1 16H9v-2h2v2zm0-3H9V4h2v9z"],"page-layout":["M19 1H1c-.55 0-1 .45-1 1v16c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1zM7 17H2V8h5v9zm11 0H8V8h10v9zm0-10H2V3h16v4z"],"panel-stats":["M1 1h18a1 1 0 011 1v15a1 1 0 01-1 1H1a1 1 0 01-1-1V2a1 1 0 011-1zm1 2v13h16V3H2zm9 0h1v13h-1V3zm2 7h3.952v1H13v-1zm0 2h3.952v1H13v-1zm0 2h3.952v1H13v-1zm0-6h3.952v1H13V8zm0-2h3.952v1H13V6zm0-2h3.952v1H13V4z"],"panel-table":["M19 1H1c-.6 0-1 .4-1 1v15c0 .6.4 1 1 1h18c.6 0 1-.4 1-1V2c0-.6-.4-1-1-1zm-9 11H7V9h3v3zm0-4H7V5h3v3zm-8 8V3h4v13H2zm5 0v-3h3v3H7zm11 0h-7v-3h7v3zm0-4h-7V9h7v3zm0-4h-7V5h7v3z"],paperclip:["M18.35 2.67A5.664 5.664 0 0014.33 1c-1.44 0-2.89.56-3.99 1.67l-9.16 9.27C.4 12.73 0 13.78 0 14.83s.39 2.1 1.18 2.9c.78.79 1.82 1.18 2.85 1.18 1.04 0 2.07-.39 2.87-1.2l9.14-9.27c.96-.96.96-2.5.02-3.45-.94-.95-2.49-.96-3.44 0l-7.59 7.69c-.31.32-.3.83.01 1.14.31.31.81.31 1.13.02l7.59-7.69c.31-.31.84-.31 1.13-.02.31.31.31.85 0 1.16l-9.14 9.27c-.93.95-2.54.93-3.45.02-.94-.95-.92-2.55.02-3.49l9.16-9.25c1.55-1.56 4.18-1.59 5.72-.03 1.56 1.57 1.55 4.26 0 5.82l-8.89 9.02c-.3.31-.3.81.01 1.11.3.3.79.31 1.1.01v.01l8.91-9.02A5.645 5.645 0 0020 6.73c0-1.48-.55-2.94-1.65-4.06z"],paragraph:["M16.5 1H7C4.2 1 2 3.2 2 6s2.2 5 5 5v6.5c0 .8.7 1.5 1.5 1.5s1.5-.7 1.5-1.5V4h2v13.5c0 .8.7 1.5 1.5 1.5s1.5-.7 1.5-1.5V4h1.5c.8 0 1.5-.7 1.5-1.5S17.3 1 16.5 1z"],path:["M18 0H2C.9 0 0 .9 0 2s.9 2 2 2h7v4H4c-1.1 0-2 .9-2 2s.9 2 2 2h5v4H6c-1.1 0-2 .9-2 2s.9 2 2 2h8c1.1 0 2-.9 2-2s-.9-2-2-2h-3v-4h5c1.1 0 2-.9 2-2s-.9-2-2-2h-5V4h7c1.1 0 2-.9 2-2s-.9-2-2-2z"],"path-search":["M4 7c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm15 11.69l-5-2.5v-3.63c-.32.11-.66.22-1 .29v3.32l-6 2.57v-7.25c-.36-.27-.69-.57-1-.9v8.1l-5-2.5V10c.55 0 1-.45 1-1s-.45-1-1-1V1.31l3.43 1.71c.11-.31.24-.62.39-.92L.72.05A.545.545 0 00.5 0C.22 0 0 .22 0 .5v16c0 .2.12.36.28.44l6 3c.07.04.14.06.22.06.07 0 .14-.01.2-.04l6.79-2.91 5.79 2.9c.07.03.14.05.22.05.28 0 .5-.22.5-.5v-4.21c-.31.13-.64.21-1 .21v3.19zM10 5c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm3-1c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm6.72-.94l-1.43-.72c.2.43.36.89.48 1.36l.23.11V5.5c-.55 0-1 .45-1 1s.45 1 1 1v1.96l1 1V3.5c0-.2-.12-.36-.28-.44zm-3.69 5.56c.14-.21.27-.42.38-.65.02-.04.04-.07.05-.11.11-.22.2-.45.28-.69v-.01c.07-.24.13-.48.17-.73l.03-.17c.04-.25.06-.5.06-.76C17 2.46 14.54 0 11.5 0S6 2.46 6 5.5 8.46 11 11.5 11c.26 0 .51-.02.76-.06l.17-.03c.25-.04.49-.1.73-.17h.01c.24-.08.47-.17.69-.28.04-.02.07-.03.11-.05.23-.11.44-.24.65-.38l.18.18 3.5 3.5c.17.18.42.29.7.29a1.003 1.003 0 00.71-1.71l-3.68-3.67zm-4.53.88c-2.21 0-4-1.79-4-4s1.79-4 4-4 4 1.79 4 4-1.79 4-4 4z"],pause:["M7 3H4c-.55 0-1 .45-1 1v12c0 .55.45 1 1 1h3c.55 0 1-.45 1-1V4c0-.55-.45-1-1-1zm9 0h-3c-.55 0-1 .45-1 1v12c0 .55.45 1 1 1h3c.55 0 1-.45 1-1V4c0-.55-.45-1-1-1z"],people:["M16.94 17a4.92 4.92 0 00-.33-1.06c-.45-.97-1.37-1.52-3.24-2.3-.17-.07-.76-.31-.77-.32-.1-.04-.2-.08-.28-.12.05-.14.04-.29.06-.45 0-.05.01-.11.01-.16-.25-.21-.47-.48-.65-.79.22-.34.41-.71.56-1.12l.04-.11c-.01.02-.01.02-.02.08l.06-.15c.36-.26.6-.67.72-1.13.18-.37.29-.82.25-1.3-.05-.5-.21-.92-.47-1.22-.02-.53-.06-1.11-.12-1.59.38-.17.83-.26 1.24-.26.59 0 1.26.19 1.73.55.46.35.8.85.97 1.4.04.13.07.25.08.38.08.45.13 1.14.13 1.61v.07c.16.07.31.24.35.62.02.29-.09.55-.15.65-.05.26-.2.53-.46.59-.03.12-.07.25-.11.36-.01.01-.01.04-.01.04-.2.53-.51 1-.89 1.34 0 .06 0 .12.01.17.04.41-.11.71 1 1.19 1.1.5 2.77 1.01 3.13 1.79.34.79.2 1.25.2 1.25h-3.04zm-5.42-3.06c1.47.66 3.7 1.35 4.18 2.39.45 1.05.27 1.67.27 1.67H.04s-.19-.62.27-1.67c.46-1.05 2.68-1.75 4.16-2.4 1.48-.65 1.33-1.05 1.38-1.59 0-.07.01-.14.01-.21-.52-.45-.95-1.08-1.22-1.8l-.01-.01c0-.01-.01-.02-.01-.03-.07-.15-.12-.32-.16-.49-.34-.06-.54-.43-.62-.78-.08-.14-.24-.48-.2-.87.05-.51.26-.74.49-.83v-.08c0-.64.05-1.55.17-2.15a3.648 3.648 0 011.4-2.36C6.32 2.25 7.21 2 8 2s1.68.25 2.31.73a3.63 3.63 0 011.4 2.36c.11.6.17 1.52.17 2.15v.09c.22.09.42.32.47.82.03.39-.12.73-.2.87-.07.34-.27.71-.61.78-.04.16-.09.33-.15.48-.01.01-.02.05-.02.05-.27.71-.68 1.33-1.19 1.78 0 .08 0 .16.01.23.05.55-.15.95 1.33 1.6z"],percentage:["M15 10c-1.66 0-3 1.34-3 3v2c0 1.66 1.34 3 3 3s3-1.34 3-3v-2c0-1.66-1.34-3-3-3zm1 5c0 .55-.45 1-1 1s-1-.45-1-1v-2c0-.55.45-1 1-1s1 .45 1 1v2zM8 7V5c0-1.66-1.34-3-3-3S2 3.34 2 5v2c0 1.66 1.34 3 3 3s3-1.34 3-3zM4 7V5c0-.55.45-1 1-1s1 .45 1 1v2c0 .55-.45 1-1 1s-1-.45-1-1zm11-4a1.003 1.003 0 00-1.88-.48L5.14 16.49a1.003 1.003 0 101.74.99l7.99-13.97c.08-.15.13-.32.13-.51z"],person:["M19.61 17.91c-.57-1.32-3.35-2.19-5.19-3.01-1.85-.82-1.59-1.31-1.66-1.99-.01-.09-.01-.19-.02-.29.63-.56 1.15-1.33 1.49-2.22 0 0 .02-.05.02-.06.07-.19.13-.39.19-.6.42-.09.67-.55.76-.98.1-.17.29-.6.25-1.08-.06-.62-.31-.91-.59-1.03v-.11c0-.79-.07-1.93-.22-2.68A4.55 4.55 0 0012.9.92C12.11.32 11 0 10.01 0s-2.1.32-2.89.92a4.55 4.55 0 00-1.74 2.94c-.14.75-.22 1.89-.22 2.68v.1c-.29.11-.55.4-.61 1.04-.04.48.15.91.25 1.08.1.44.35.91.79.98.05.21.12.41.19.6 0 .01.01.03.01.04l.01.02c.34.91.87 1.69 1.52 2.25 0 .09-.01.18-.02.26-.07.68.13 1.17-1.72 1.99S.96 16.59.39 17.91C-.18 19.23.05 20 .05 20h19.9s.23-.77-.34-2.09z"],phone:["M19.91 15.51c-.08-.08-4.21-2.5-4.35-2.57a.876.876 0 00-.4-.1c-.19 0-.42.13-.71.4-.28.27-1.17 1.49-1.43 1.76s-.48.4-.65.4c-.08 0-.19-.02-.32-.07s-1.45-.73-4.2-3.15-3.11-4-3.13-4.44c0-.17.13-.39.4-.65.28-.25.57-.51.89-.74.32-.24.61-.5.88-.78s.4-.52.4-.71c0-.13-.03-.27-.1-.4C7.12 4.32 4.62.19 4.53.1c-.19-.18-.92-.1-1.29.1C.25 1.82 0 4 .05 4.86c.05.89.61 5.58 5.2 9.93 5.7 5.41 9.66 5.2 9.92 5.2.87 0 3.52-.48 4.65-3.19.16-.38.31-1.07.09-1.29z"],"pie-chart":["M9 .98c-4.5.5-8 4.31-8 8.94 0 4.97 4.03 9.04 9 9.04 4.63 0 8.44-3.96 8.94-7.96H9V.98z","M10-.08V10h10C20 4 15.52-.08 10-.08z"],pin:["M11.77 1.16c-.81.81-.74 2.28.02 3.76L6.1 8.71c-2.17-1.46-4.12-2-4.94-1.18l4.95 4.95-4.95 6.36 6.36-4.95 4.95 4.95c.82-.82.27-2.77-1.19-4.94l3.8-5.69c1.47.76 2.94.84 3.76.02l-7.07-7.07z"],pivot:["M5.83 9.75L.29 15.29a1.003 1.003 0 001.42 1.42l5.54-5.54c-.57-.37-1.05-.85-1.42-1.42zM19 11c-.55 0-1 .45-1 1v1.59l-3.83-3.83c-.37.56-.85 1.04-1.41 1.41L16.59 15H15c-.55 0-1 .45-1 1s.45 1 1 1h4c.55 0 1-.45 1-1v-4c0-.55-.45-1-1-1zm-5-4c0-2.21-1.79-4-4-4S6 4.79 6 7s1.79 4 4 4 4-1.79 4-4zm-4 2c-1.1 0-2-.9-2-2s.9-2 2-2 2 .9 2 2-.9 2-2 2z"],"pivot-table":["M3 5H1c-.55 0-1 .45-1 1v13c0 .55.45 1 1 1h2c.55 0 1-.45 1-1V6c0-.55-.45-1-1-1zm0-5H1C.45 0 0 .45 0 1v2c0 .55.45 1 1 1h2c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm13.71 5.29C16.53 5.11 16.28 5 16 5s-.53.11-.71.29l-3 3a1.003 1.003 0 001.42 1.42L15 8.41V11c0 2.21-1.79 4-4 4H8.41l1.29-1.29c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71l-3 3c-.18.18-.29.43-.29.71 0 .28.11.53.29.71l3 3a1.003 1.003 0 001.42-1.42L8.41 17H11c3.31 0 6-2.69 6-6V8.41l1.29 1.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71l-3-3zM19 0H6c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h13c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1z"],play:["M16 10c0-.36-.2-.67-.49-.84l.01-.01-10-6-.01.01A.991.991 0 005 3c-.55 0-1 .45-1 1v12c0 .55.45 1 1 1 .19 0 .36-.07.51-.16l.01.01 10-6-.01-.01c.29-.17.49-.48.49-.84z"],plus:["M16 9h-5V4c0-.55-.45-1-1-1s-1 .45-1 1v5H4c-.55 0-1 .45-1 1s.45 1 1 1h5v5c0 .55.45 1 1 1s1-.45 1-1v-5h5c.55 0 1-.45 1-1s-.45-1-1-1z"],"polygon-filter":["M18 7c-.27 0-.52.05-.75.15l-6.28-4.88c.01-.09.03-.18.03-.27 0-1.1-.9-2-2-2S7 .9 7 2c0 .06.01.12.02.19l-4.19 3C2.57 5.07 2.29 5 2 5 .9 5 0 5.9 0 7c0 .74.4 1.38 1 1.72v7.55c-.6.35-1 .99-1 1.73 0 1.1.9 2 2 2 .74 0 1.38-.4 1.72-1h7.55c.35.6.98 1 1.72 1 1.1 0 2-.9 2-2 0-.37-.11-.72-.29-1.02L18.03 11A2 2 0 0018 7zm-5.03 9c-.72.01-1.35.41-1.69 1H3.72c-.17-.3-.42-.55-.72-.72V8.72c.6-.34 1-.98 1-1.72 0-.06-.01-.12-.02-.19l4.19-3c.26.12.54.19.83.19.27 0 .52-.05.75-.15l6.28 4.88c-.01.09-.03.18-.03.27 0 .37.11.72.29 1.02L12.97 16z"],power:["M10 10c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1S9 .45 9 1v8c0 .55.45 1 1 1zm3-7.45v2.16c2.36 1.12 4 3.5 4 6.29 0 3.87-3.13 7-7 7s-7-3.13-7-7c0-2.79 1.64-5.17 4-6.29V2.55C3.51 3.79 1 7.09 1 11a9 9 0 0018 0c0-3.91-2.51-7.21-6-8.45z"],"predictive-analysis":["M20 8.01c0-1.26-.61-2.43-1.61-3.12C17.86 2.5 15.8.79 13.4.79c-.58 0-1.14.1-1.69.29A3.533 3.533 0 009.17 0C8.05 0 7 .55 6.32 1.45c-.15-.02-.3-.03-.45-.03-1.63 0-3.03 1.12-3.46 2.71C.97 4.65 0 6.05 0 7.66c0 .48.09.95.26 1.4-.17.44-.26.91-.26 1.39 0 1.38.72 2.64 1.89 3.29.67.7 1.59 1.09 2.54 1.09.61 0 1.19-.15 1.71-.45.68.82 1.68 1.3 2.73 1.3.66 0 1.28-.18 1.83-.52.61.49 1.34.81 2.11.91 1.3 1.43 2.3 3.28 2.31 3.3 0 0 .35.61.33.61.96-.01 1.77-.2 1.64-1.3.01.02-.92-2.89-.92-2.89.52-.26.94-.69 1.21-1.23 1.12-.66 1.84-1.91 1.84-3.26 0-.3-.03-.6-.1-.89.57-.64.88-1.51.88-2.4zm-1.54 1.28l-.18-.2-.77-.84c-.33-.37-.67-1.17-.73-1.73 0 0-.13-1.25-.13-1.26-.06-.74-1.17-.73-1.13.14 0 .02.13 1.26.13 1.26.04.36.15.77.3 1.17-.08-.01-.15-.02-.22-.02 0 0-2.57-.12-2.57-.13-.73-.03-.89 1.22-.05 1.25l2.57.13c.53.03 1.29.37 1.61.72l.61.67.02.06c.1.27.14.55.14.83 0 .93-.51 1.77-1.34 2.18l-.2.1-.09.23c-.19.48-.6.82-1.1.93l-.67.14.87 2.75c-.48-.76-1.19-1.79-2.02-2.67l-.15-.16-.21-.02c-.51-.04-.99-.21-1.42-.48l1.7-1.48c.44-.39 1.04-.55 1.24-.49 0 0 .78.22.78.23.78.2 1.03-.92.29-1.21l-.78-.23c-.69-.2-1.67.22-2.24.72l-1.91 1.66-.39.32c-.44.36-.93.55-1.5.55-.8 0-1.54-.41-1.97-1.07v-1.88c0-.5.21-.98.34-1.07 0 0 .65-.43.64-.43.87-.69.21-1.57-.64-1.14 0-.01-.65.43-.65.43-.31.2-.54.56-.7.97-.13-.13-.28-.25-.43-.35 0 0-1.91-1.26-1.91-1.28-.81-.56-1.5.63-.61 1.11 0-.02 1.89 1.28 1.89 1.28.46.31.77.97.77 1.36v.84c-.43.24-.78.36-1.24.36-.67 0-1.31-.29-1.77-.79l-.07-.08-.09-.05a2.425 2.425 0 01-1.31-2.16c0-.38.09-.74.25-1.08l.15-.31-.14-.33c-.17-.34-.25-.7-.25-1.08 0-1.13.76-2.1 1.85-2.37l.39-.09.07-.43a2.41 2.41 0 012.39-2.05c.19 0 .39.02.58.07l.4.1.22-.38A2.41 2.41 0 019.17 1.3c.55 0 1.08.19 1.5.53l-.44.45-.01-.01-.31.31c-.41.35-.92.53-1.11.5 0 0-.84-.13-.84-.14-.83-.15-1.09 1.08-.18 1.29.01 0 .84.14.84.14.03 0 .06 0 .09.01-.14.46-.18.96-.12 1.4 0 0 .21 1.24.19 1.23.13.65 1.32.44 1.16-.22 0-.01-.19-1.23-.19-1.23-.07-.48.15-1.19.45-1.5l.48-.5c.07-.06.13-.12.19-.18l.93-.95c.5-.23 1.04-.34 1.59-.34 1.93 0 3.57 1.4 3.89 3.34l.05.31.26.15a2.445 2.445 0 01.87 3.4z"],prescription:["M13.95 10.23c.16-.18.22-.22.46-.22h1.48c.25 0 .47.08.59.33.1.2.09.41-.05.66l-2.71 3.58L16.88 19c.13.21.16.46.03.69-.12.21-.34.31-.57.31H14.7c-.31 0-.56-.17-.7-.44l-1.9-2.67-1.93 2.68c-.15.27-.42.43-.73.43H7.98c-.25 0-.47-.08-.59-.33-.1-.2-.09-.41.05-.66l3.09-4.35L6.26 9H5v4.32c0 .41-.3.69-.7.69H2.7c-.41 0-.7-.28-.7-.69V.69c0-.41.3-.69.7-.69h4.42c.71 0 1.36.1 1.94.3.59.2 1.11.49 1.54.87.44.38.78.84 1.02 1.39.24.54.36 1.14.36 1.78 0 1.01-.28 1.88-.84 2.6-.43.54-1.35 1.29-2 1.59l3.09 3.94 1.72-2.24zM6.71 6.04c.71 0 1.45-.16 1.81-.46.33-.28.5-.69.5-1.25s-.17-.97-.5-1.25c-.35-.3-1.1-.46-1.81-.46h-1.7v3.42h1.7z"],presentation:["M19 1h-8c0-.55-.45-1-1-1S9 .45 9 1H1c-.55 0-1 .45-1 1s.45 1 1 1h1v11c0 .55.45 1 1 1h4.59L4.3 18.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71L9 16.41V19c0 .55.45 1 1 1s1-.45 1-1v-2.59l3.29 3.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71L12.41 15H17c.55 0 1-.45 1-1V3h1c.55 0 1-.45 1-1s-.45-1-1-1zm-3 12H4V3h12v10z"],print:["M14 16H6v-4H4v5c0 .55.45 1 1 1h10c.55 0 1-.45 1-1v-5h-2v4zm2-13c0-.55-.45-1-1-1H5c-.55 0-1 .45-1 1v1h12V3zm3 2H1c-.55 0-1 .45-1 1v7c0 .55.45 1 1 1h2v-3h14v3h2c.55 0 1-.45 1-1V6c0-.55-.45-1-1-1zm-1 4h-2V7h2v2z"],projects:["M18 4c0-.55-.45-1-1-1H3c-.55 0-1 .45-1 1v2h16V4zm-2-3c0-.55-.45-1-1-1H5c-.55 0-1 .45-1 1v1h12V1zm3 6H1c-.55 0-1 .45-1 1v11c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V8c0-.55-.45-1-1-1zm-5 7c0 .55-.45 1-1 1H7c-.55 0-1-.45-1-1v-2h1v2h6v-2h1v2z"],properties:["M2 15c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm0-7c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm5-4h12c.55 0 1-.45 1-1s-.45-1-1-1H7c-.55 0-1 .45-1 1s.45 1 1 1zM2 1C.9 1 0 1.9 0 3s.9 2 2 2 2-.9 2-2-.9-2-2-2zm17 8H7c-.55 0-1 .45-1 1s.45 1 1 1h12c.55 0 1-.45 1-1s-.45-1-1-1zm0 7H7c-.55 0-1 .45-1 1s.45 1 1 1h12c.55 0 1-.45 1-1s-.45-1-1-1z"],property:["M3 5c1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2 .9 2 2 2zm5-1h11c.55 0 1-.45 1-1s-.45-1-1-1H8c-.55 0-1 .45-1 1s.45 1 1 1zM3 15c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm16 1H8c-.55 0-1 .45-1 1s.45 1 1 1h11c.55 0 1-.45 1-1s-.45-1-1-1zm-1-8H9c-1.1 0-2 .9-2 2s.9 2 2 2h9c1.1 0 2-.9 2-2s-.9-2-2-2zM3 7c-1.66 0-3 1.34-3 3s1.34 3 3 3 3-1.34 3-3-1.34-3-3-3z"],"publish-function":["M7.01 10.11c.35-.64.72-1.68 1.09-3.11l.8-3.03h.96l.24-.77h-.99c.28-1.11.66-1.92 1.12-2.43.28-.32.56-.48.83-.48.05 0 .1.02.13.05.03.03.05.07.05.12 0 .04-.04.13-.11.25-.08.12-.11.24-.11.35 0 .15.06.28.18.39.12.11.27.16.45.16.2 0 .36-.07.49-.2s.2-.31.2-.54c0-.26-.1-.47-.3-.63-.2-.16-.52-.24-.96-.24-.68 0-1.3.19-1.86.58-.55.38-1.08 1.02-1.58 1.91-.17.3-.34.5-.49.59-.15.08-.4.13-.74.12l-.23.77h.95L5.74 9.21c-.23.86-.39 1.39-.47 1.59-.12.29-.3.54-.54.75-.1.08-.21.12-.35.12-.04 0-.07-.01-.1-.03l-.03-.04c0-.02.03-.07.1-.13.07-.07.1-.17.1-.31 0-.15-.05-.28-.16-.38-.11-.1-.27-.15-.47-.15-.25 0-.44.07-.59.2-.15.12-.23.28-.23.46 0 .19.09.36.27.5.19.14.47.21.86.21.61 0 1.16-.15 1.63-.46.48-.31.89-.79 1.25-1.43zm3.7 1.18c-.18-.18-.43-.29-.71-.29s-.53.11-.71.29l-3 3a1.003 1.003 0 001.42 1.42L9 14.41V19c0 .55.45 1 1 1s1-.45 1-1v-4.59l1.29 1.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71l-3-3zm4.15-6.78c.17-.13.36-.2.55-.2.07 0 .2.03.39.08s.36.08.5.08c.2 0 .37-.07.5-.2.13-.14.2-.31.2-.52 0-.22-.07-.4-.2-.53s-.33-.2-.58-.2c-.22 0-.43.05-.63.15-.2.1-.45.32-.75.67-.23.25-.56.7-1.01 1.33a6.52 6.52 0 00-.91-2.15l-2.38.39-.05.25c.18-.03.33-.05.45-.05.24 0 .43.1.59.3.25.31.59 1.24 1.02 2.79-.34.44-.58.73-.7.87-.21.22-.38.36-.52.43-.1.05-.22.08-.35.08-.1 0-.26-.05-.49-.16a1.01 1.01 0 00-.42-.11c-.23 0-.42.07-.57.22-.17.14-.24.32-.24.55 0 .21.07.38.21.51.14.13.33.2.56.2.23 0 .44-.05.64-.14.2-.09.45-.29.75-.59s.72-.78 1.25-1.43c.2.62.38 1.07.53 1.35.15.28.32.49.52.61.19.12.44.19.73.19.28 0 .57-.1.86-.3.38-.25.77-.69 1.17-1.31l-.25-.14c-.27.37-.48.6-.61.69-.09.06-.19.09-.31.09-.14 0-.28-.09-.42-.26-.23-.29-.54-1.09-.93-2.4.37-.58.66-.96.9-1.14z"],pulse:["M19 10h-2.38L14.9 6.55h-.01c-.17-.32-.5-.55-.89-.55-.43 0-.79.28-.93.66h-.01l-2.75 7.57L7.98 1.82h-.02A.978.978 0 007 1c-.44 0-.8.29-.94.69h-.01L3.28 10H1c-.55 0-1 .45-1 1s.45 1 1 1h3c.44 0 .8-.29.94-.69h.01l1.78-5.34 2.29 12.21h.02c.08.46.47.82.96.82.43 0 .79-.28.93-.66h.01l3.21-8.82.96 1.92h.01c.16.33.49.56.88.56h3c.55 0 1-.45 1-1s-.45-1-1-1z"],random:["M14.47 5h2.12L15.3 6.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71l3-3c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71l-3-3a1.003 1.003 0 00-1.42 1.42L16.59 3H14c-.31 0-.57.15-.76.37l-.01-.01-2.93 3.52 1.3 1.56L14.47 5zm2.24 7.29a1.003 1.003 0 00-1.42 1.42l1.3 1.29h-2.12L4.77 3.36l-.01.01A.998.998 0 004 3H1c-.55 0-1 .45-1 1s.45 1 1 1h2.53l9.7 11.64.01-.01c.19.22.45.37.76.37h2.59l-1.29 1.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71l3-3c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71l-3-3zM3.53 15H1c-.55 0-1 .45-1 1s.45 1 1 1h3c.31 0 .57-.15.76-.37l.01.01 2.93-3.52-1.3-1.56L3.53 15z"],record:["M10 3a7 7 0 100 14 7 7 0 100-14z"],redo:["M19.71 5.29l-4-4a1.003 1.003 0 00-1.42 1.42L16.59 5H6c-3.31 0-6 2.69-6 6s2.69 6 6 6h5v-2H6c-2.21 0-4-1.79-4-4s1.79-4 4-4h10.59L14.3 9.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71l4-4c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71zM15 14c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2z"],refresh:["M19 1c-.55 0-1 .45-1 1v2.06C16.18 1.61 13.29 0 10 0 4.48 0 0 4.48 0 10c0 .55.45 1 1 1s1-.45 1-1c0-4.42 3.58-8 8-8 2.52 0 4.76 1.18 6.22 3H15c-.55 0-1 .45-1 1s.45 1 1 1h4c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1zm0 8c-.55 0-1 .45-1 1 0 4.42-3.58 8-8 8-2.52 0-4.76-1.18-6.22-3H5c.55 0 1-.45 1-1s-.45-1-1-1H1c-.55 0-1 .45-1 1v4c0 .55.45 1 1 1s1-.45 1-1v-2.06C3.82 18.39 6.71 20 10 20c5.52 0 10-4.48 10-10 0-.55-.45-1-1-1z"],"regression-chart":["M19 16H3.1L19.31 3.39l-.61-.79L2 15.59V3c0-.55-.45-1-1-1s-1 .45-1 1v14c0 .55.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1zm-9-9c1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2 .9 2 2 2zm-5 4c1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2 .9 2 2 2zm10-2c0 1.1.9 2 2 2s2-.9 2-2-.9-2-2-2-2 .9-2 2zm-5 4c0 1.1.9 2 2 2s2-.9 2-2-.9-2-2-2-2 .9-2 2z"],remove:["M10 0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zm0 18c-4.42 0-8-3.58-8-8s3.58-8 8-8 8 3.58 8 8-3.58 8-8 8zm5-9H5c-.55 0-1 .45-1 1s.45 1 1 1h10c.55 0 1-.45 1-1s-.45-1-1-1z"],"remove-column":["M19 0H5c-.55 0-1 .45-1 1v4h2V2h5v16H6v-3H4v4c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 18h-5V2h5v16zM6.29 13.71a1.003 1.003 0 001.42-1.42L5.41 10 7.7 7.71c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71L4 8.59l-2.29-2.3A1.003 1.003 0 00.29 7.71L2.59 10 .3 12.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71L4 11.41l2.29 2.3z"],"remove-column-left":["M4 11h6c.55 0 1-.45 1-1s-.45-1-1-1H4c-.55 0-1 .45-1 1s.45 1 1 1zM19 0H1C.45 0 0 .45 0 1v18c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-7 18H2V2h10v16zm6 0h-5V2h5v16z"],"remove-column-right":["M19 0H1C.45 0 0 .45 0 1v18c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zM7 18H2V2h5v16zm11 0H8V2h10v16zm-8-7h6c.55 0 1-.45 1-1s-.45-1-1-1h-6c-.55 0-1 .45-1 1s.45 1 1 1z"],"remove-row-bottom":["M7 14h6c.55 0 1-.45 1-1s-.45-1-1-1H7c-.55 0-1 .45-1 1s.45 1 1 1zM19 0H1C.45 0 0 .45 0 1v18c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 18H2V8h16v10zm0-11H2V2h16v5z"],"remove-row-top":["M7 8h6c.55 0 1-.45 1-1s-.45-1-1-1H7c-.55 0-1 .45-1 1s.45 1 1 1zm12-8H1C.45 0 0 .45 0 1v18c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 18H2v-5h16v5zm0-6H2V2h16v10z"],repeat:["M14 6c0 .55.45 1 1 1h4c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1s-1 .45-1 1v2.05C16.18 1.6 13.29 0 10 0 4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10c0-.55-.45-1-1-1s-1 .45-1 1c0 4.42-3.58 8-8 8s-8-3.58-8-8 3.58-8 8-8c2.53 0 4.77 1.17 6.24 3H15c-.55 0-1 .45-1 1z"],reset:["M6 6c0 .55-.45 1-1 1H1c-.55 0-1-.45-1-1V2c0-.55.45-1 1-1s1 .45 1 1v2.05C3.82 1.6 6.71 0 10 0c5.52 0 10 4.48 10 10s-4.48 10-10 10S0 15.52 0 10c0-.55.45-1 1-1s1 .45 1 1c0 4.42 3.58 8 8 8s8-3.58 8-8-3.58-8-8-8C7.47 2 5.23 3.17 3.76 5H5c.55 0 1 .45 1 1z"],resolve:["M8.7 4.7C7.9 4.2 7 4 6 4c-3.3 0-6 2.7-6 6s2.7 6 6 6c1 0 1.9-.2 2.7-.7C7.3 14 6.5 12.1 6.5 10s.9-4 2.2-5.3zM14 4c-1 0-1.9.2-2.7.7 1.4 1.4 2.2 3.2 2.2 5.3s-.9 4-2.2 5.3c.8.5 1.7.7 2.7.7 3.3 0 6-2.7 6-6s-2.7-6-6-6zm-4 1.5C8.8 6.7 8 8.2 8 10s.8 3.3 2 4.4c1.2-1.1 2-2.7 2-4.5s-.8-3.3-2-4.4z"],rig:["M7 4.2C7 5.75 8.34 7 10 7s3-1.46 3-2.8C13 1.45 10.94 0 10 0H6c0 2.74 3.76 1.96 1 4.2zm11.71 14.09L13 12.59V9.01c0-.55-.45-1-1-1H8c-.55 0-1 .45-1 1v3.58l-5.71 5.7a1.003 1.003 0 001.42 1.42L7 15.42V19c0 .55.45 1 1 1h4c.55 0 1-.45 1-1v-3.58l4.29 4.29a1.003 1.003 0 001.42-1.42zM10.21 8c.01 0 .01.01 0 0 .01.01.01 0 0 0z"],"right-join":["M8.7 4.7C7.4 6 6.5 7.9 6.5 10s.8 4 2.2 5.3c-.8.5-1.7.7-2.7.7-3.3 0-6-2.7-6-6s2.7-6 6-6c1 0 1.9.2 2.7.7zm-3.34 9.25c-.55-1.2-.86-2.54-.86-3.95s.31-2.75.86-3.95a4.001 4.001 0 000 7.9zM14 4c3.3 0 6 2.7 6 6s-2.7 6-6 6c-1 0-1.9-.2-2.7-.7 1.3-1.3 2.2-3.2 2.2-5.3s-.8-3.9-2.2-5.3C12.1 4.2 13 4 14 4zm-4 1.5C8.8 6.7 8 8.2 8 10s.8 3.3 2 4.4c1.2-1.1 2-2.7 2-4.5s-.8-3.3-2-4.4z"],ring:["M10 0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zm0 15c-2.76 0-5-2.24-5-5s2.24-5 5-5 5 2.24 5 5-2.24 5-5 5z"],"rotate-document":["M8.71 6.29A.997.997 0 008 6H3c-.55 0-1 .45-1 1v12c0 .55.45 1 1 1h9c.55 0 1-.45 1-1v-8c0-.28-.11-.53-.29-.71l-4-4zM11 18H4V8h3v3c0 .55.45 1 1 1h3v6zm3-16h-1.59l.29-.29c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71l-2 2C9.11 2.47 9 2.72 9 3c0 .28.11.53.29.71l2 2a1.003 1.003 0 001.42-1.42l-.3-.29H14c1.1 0 2 .9 2 2v3c0 .55.45 1 1 1s1-.45 1-1V6c0-2.21-1.79-4-4-4z"],"rotate-page":["M14 2h-1.59l.29-.29c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71l-2 2C9.11 2.47 9 2.72 9 3c0 .28.11.53.29.71l2 2a1.003 1.003 0 001.42-1.42l-.3-.29H14c1.1 0 2 .9 2 2v3c0 .55.45 1 1 1s1-.45 1-1V6c0-2.21-1.79-4-4-4zm-2 5H3c-.55 0-1 .45-1 1v11c0 .55.45 1 1 1h9c.55 0 1-.45 1-1V8c0-.55-.45-1-1-1zm-1 11H4V9h7v9z"],route:["M14.028 6.016c.146.275.31.57.485.872.304.524.628 1.047.952 1.545l.118.178-.208-.006-.577-.005c-2.093.004-2.841.303-2.841.895 0 .069.271.248 1.245.567l1.008.313c2.671.831 3.99 1.827 3.99 4.167 0 2.76-1.928 4.059-4.832 4.376-.782.085-1.52.098-2.452.066l-1.15-.046H6.221l.535-.811a67.46 67.46 0 001.122-1.787h2.04l.686.03c1.028.046 1.77.043 2.523-.039 1.832-.2 2.673-.767 2.673-1.789 0-.69-.483-1.09-1.992-1.585l-.83-.257c-1.192-.364-2.037-.7-2.59-1.165.399-1 .612-1.844.612-2.538a6.018 6.018 0 00-.382-2.098c.745-.573 1.884-.822 3.41-.883zM5 4.2c2.648 0 4.791 2.151 4.8 4.797C9.8 11.652 5 18.6 5 18.6l-.5-.744C3.273 15.993.2 11.121.2 8.997A4.802 4.802 0 015 4.2zm0 2.4a2.4 2.4 0 10.002 4.802A2.4 2.4 0 005 6.6zM17 .333a2.671 2.671 0 012.667 2.665C19.667 4.473 17 8.333 17 8.333l-.391-.587c-.741-1.137-2.276-3.629-2.276-4.748A2.668 2.668 0 0117 .333z"],satellite:["M9 18c.6 0 1 .4 1 1s-.4 1-1 1c-5 0-9-4-9-9 0-.6.4-1 1-1s1 .4 1 1c0 3.9 3.1 7 7 7zm0-4c.6 0 1 .4 1 1s-.4 1-1 1c-2.8 0-5-2.2-5-5 0-.6.4-1 1-1s1 .4 1 1c0 1.7 1.3 3 3 3zm5.7-3.7c.4-.4 1-.4 1.4 0l3.6 3.6c.4.4.4 1 0 1.4l-1.4 1.4c-.4.4-1 .4-1.4 0l-3.6-3.6c-.4-.4-.4-1 0-1.4l1.4-1.4zM4.7.3c.4-.4 1-.4 1.4 0l3.6 3.6c.4.4.4 1 0 1.4L8.3 6.7c-.4.4-1 .4-1.4 0L3.3 3.1c-.4-.4-.4-1 0-1.4L4.7.3zm11.1 1c.4-.4 1-.4 1.4 0l1.6 1.6c.4.4.4 1 0 1.4l-6.5 6.5c-.4.4-1 .4-1.4 0L9.3 9.2c-.4-.4-.4-1 0-1.4l6.5-6.5zM9 12c-.6 0-1-.4-1-1s.4-1 1-1 1 .4 1 1-.4 1-1 1z"],saved:["M12 0H4c-.55 0-1 .45-1 1v18c0 .55.45 1 1 1h13c.55 0 1-.45 1-1V6l-6-6zm4 18H5V2h6v5h5v11zm-8.29-6.71a1.003 1.003 0 00-1.42 1.42l3 3c.18.18.43.29.71.29.32 0 .59-.16.77-.38l.01.01 4-5-.01-.01c.14-.18.23-.38.23-.62 0-.55-.45-1-1-1-.32 0-.59.16-.77.38l-.01-.01-3.3 4.13-2.21-2.21z"],"scatter-plot":["M9 9c1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2 .9 2 2 2zm5 2c1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2 .9 2 2 2zm4-5c1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2 .9 2 2 2zm1 10H2V3c0-.55-.45-1-1-1s-1 .45-1 1v14c0 .55.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1zM5 15c1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2 .9 2 2 2z"],search:["M19.56 17.44l-4.94-4.94A8.004 8.004 0 0016 8c0-4.42-3.58-8-8-8S0 3.58 0 8s3.58 8 8 8c1.67 0 3.21-.51 4.5-1.38l4.94 4.94a1.498 1.498 0 102.12-2.12zM8 14c-3.31 0-6-2.69-6-6s2.69-6 6-6 6 2.69 6 6-2.69 6-6 6z"],"search-around":["M9.9 6.9a3 3 0 100 6 3 3 0 100-6zM3 14c-1.7 0-3 1.3-3 3s1.3 3 3 3 3-1.3 3-3-1.3-3-3-3zm0 5c-1.1 0-2-.9-2-2s.9-2 2-2 2 .9 2 2-.9 2-2 2zM3 0C1.3 0 0 1.3 0 3s1.3 3 3 3 3-1.3 3-3-1.3-3-3-3zm0 5c-1.1 0-2-.9-2-2s.9-2 2-2 2 .9 2 2-.9 2-2 2zM17 14c-1.7 0-3 1.3-3 3s1.3 3 3 3 3-1.3 3-3-1.3-3-3-3zm0 5c-1.1 0-2-.9-2-2s.9-2 2-2 2 .9 2 2-.9 2-2 2zM17 0c-1.7 0-3 1.3-3 3s1.3 3 3 3 3-1.3 3-3-1.3-3-3-3zm0 5c-1.1 0-2-.9-2-2s.9-2 2-2 2 .9 2 2-.9 2-2 2zM10 10L5 5","M5.379 4.671l5.02 5.02-.707.708-5.02-5.02zM10 10l5-5","M14.621 4.671l.707.708-5.02 5.02-.707-.707z","M10 10l5 5M10.379 9.671l5.02 5.02-.707.708-5.02-5.02z","M10 10l-5 5M9.621 9.671l.707.708-5.02 5.02-.707-.707z"],"search-template":["M13 8H5c-.55 0-1 .45-1 1s.45 1 1 1h8c.55 0 1-.45 1-1s-.45-1-1-1zm0 3H5c-.55 0-1 .45-1 1s.45 1 1 1h8c.55 0 1-.45 1-1s-.45-1-1-1zm0-6H5c-.55 0-1 .45-1 1s.45 1 1 1h8c.55 0 1-.45 1-1s-.45-1-1-1zm6.56 12.44l-3.23-3.23A8.939 8.939 0 0018 9a9 9 0 10-9 9c1.94 0 3.74-.62 5.21-1.67l3.23 3.23a1.498 1.498 0 102.12-2.12zM9 16c-3.87 0-7-3.13-7-7s3.13-7 7-7 7 3.13 7 7-3.13 7-7 7z"],"search-text":["M19.56 17.44l-3.23-3.23A8.939 8.939 0 0018 9a9 9 0 10-9 9c1.94 0 3.74-.62 5.21-1.67l3.23 3.23a1.498 1.498 0 102.12-2.12zM9 16c-3.87 0-7-3.13-7-7s3.13-7 7-7 7 3.13 7 7-3.13 7-7 7zm3.5-11h-7c-.28 0-.5.22-.5.5v2c0 .28.22.5.5.5s.5-.22.5-.5V7h2v6h-.5c-.28 0-.5.22-.5.5s.22.5.5.5h3c.28 0 .5-.22.5-.5s-.22-.5-.5-.5H10V7h2v.5c0 .28.22.5.5.5s.5-.22.5-.5v-2c0-.28-.22-.5-.5-.5z"],"segmented-control":["M19 5H1c-.55 0-1 .45-1 1v8c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V6c0-.55-.45-1-1-1zm-1 8h-8V7h8v6z"],select:["M19.71 18.29l-4.25-4.25L20 12.91 9.93 9.33c.04-.1.07-.21.07-.33V3c0-.55-.45-1-1-1H4V1c0-.55-.45-1-1-1S2 .45 2 1v1H1c-.55 0-1 .45-1 1s.45 1 1 1h1v5c0 .55.45 1 1 1h6c.12 0 .23-.03.34-.07L12.91 20l1.14-4.54 4.25 4.25c.17.18.42.29.7.29a1.003 1.003 0 00.71-1.71zM8 8H4V4h4v4z"],selection:["M10 0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zm0 18c-4.42 0-8-3.58-8-8s3.58-8 8-8 8 3.58 8 8-3.58 8-8 8z","M10 6a4 4 0 100 8 4 4 0 100-8z"],"send-message":["M1.754.135L19.393 9.06c.57.288.775.943.458 1.462-.107.176-.266.32-.458.418l-17.64 8.924c-.57.289-1.288.102-1.604-.417A1.001 1.001 0 010 18.925v-6.851L11.021 10 0 7.938V1.075C0 .481.529 0 1.18 0c.201 0 .399.047.574.135z"],"send-to":["M19 0h-5c-.6 0-1 .4-1 1s.4 1 1 1h2.6l-4.3 4.3c-.2.2-.3.4-.3.7 0 .6.4 1 1 1 .3 0 .5-.1.7-.3L18 3.4V6c0 .5.5 1 1 1s1-.5 1-1V1c0-.6-.5-1-1-1zm0 9c-1 0-1.9-.5-2.5-1.3l-1.4 1.4c-.5.6-1.3.9-2.1.9-1.7 0-3-1.3-3-3 0-.8.3-1.6.9-2.1l1.4-1.4C11.5 2.9 11 2 11 1c0-.3.1-.6.2-.9-.4-.1-.8-.1-1.2-.1C4.5 0 0 4.5 0 10s4.5 10 10 10 10-4.5 10-10c0-.4 0-.8-.1-1.2-.3.1-.6.2-.9.2z"],"send-to-graph":["M8 11H3c-.55 0-1 .45-1 1s.45 1 1 1h2.59L.3 18.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71L7 14.41V17c0 .55.45 1 1 1s1-.45 1-1v-5c0-.55-.45-1-1-1zm10 2c-.53 0-1.01.21-1.37.55L11.9 10.6c.06-.19.1-.39.1-.6 0-.21-.04-.41-.1-.6l4.72-2.95c.37.34.85.55 1.38.55 1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2c0 .21.04.41.1.6l-4.73 2.96c-.24-.23-.54-.4-.87-.48V3.93c.86-.22 1.5-1 1.5-1.93 0-1.1-.9-2-2-2S8 .9 8 2c0 .93.64 1.71 1.5 1.93v4.14c-.33.09-.63.26-.87.48L7.6 7.91 5.42 6.55 3.9 5.6c.06-.19.1-.39.1-.6 0-1.1-.9-2-2-2s-2 .9-2 2 .9 2 2 2c.53 0 1.01-.21 1.37-.55L9 9.96V10h.06L12 11.84l.4.25 1.51.94 2.19 1.37c-.06.19-.1.39-.1.6 0 1.1.9 2 2 2s2-.9 2-2-.9-2-2-2zm-7-2.96l-.06-.04H11v.04z"],"send-to-map":["M8 11H3c-.55 0-1 .45-1 1s.45 1 1 1h2.59L.3 18.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71L7 14.41V17c0 .55.45 1 1 1s1-.45 1-1v-5c0-.55-.45-1-1-1zm11.54-6.82l.01-.02-6-4-.01.02C13.39.08 13.21 0 13 0s-.39.08-.54.18l-.01-.02L7 3.8 1.55.17l-.01.01A.969.969 0 001 0C.45 0 0 .45 0 1v9c0-.55.45-1 1-1h1V2.87l4 2.67V9h2V5.54l4-2.67v11.6l-1 .67v2.4l2-1.33 5.45 3.63.01-.02c.15.1.33.18.54.18.55 0 1-.45 1-1V5c0-.35-.19-.64-.46-.82zM18 17.13l-4-2.67V2.87l4 2.67v11.59z"],"series-add":["M13.29 9.29c.3.62.8 1.12 1.42 1.42l-3 3c-.18.18-.43.29-.71.29s-.53-.11-.71-.3L7 10.41l-5 5V17h17c.55 0 1 .45 1 1s-.45 1-1 1H1a.998.998 0 01-1-1V4c0-.55.45-1 1-1s1 .45 1 1v8.59l4.29-4.3C6.47 8.11 6.72 8 7 8s.53.11.71.29l3.29 3.3 2.29-2.3zM12 5c0-.5.4-1 1-1h2V2c0-.6.4-1 1-1 .5 0 1 .4 1 1v2h2c.5 0 1 .4 1 1s-.5 1-1 1h-2v2c0 .6-.5 1-1 1-.6 0-1-.4-1-1V6h-2c-.6 0-1-.4-1-1z"],"series-configuration":["M11.91 10.67c.52.45 1.13.8 1.8 1.03l-2.01 2.01c-.18.18-.43.29-.71.29-.28 0-.53-.11-.71-.3L7 10.41l-5 5V17h16.99c.55 0 1 .45 1 1s-.45 1-1 1H1a.998.998 0 01-1-1V4c0-.55.45-1 1-1s1 .45 1 1v8.59l4.29-4.3C6.47 8.11 6.72 8 7 8c.28 0 .53.11.71.29l3.29 3.3.91-.92zM18.5 4.6h1.04c.25 0 .45.2.46.44v.9c0 .25-.2.45-.45.45h-1.04c-.07.22-.16.42-.27.62l.73.73c.17.17.17.44 0 .61l-.61.61c-.17.17-.44.17-.61 0l-.73-.73c-.2.11-.4.2-.62.26v1.05c0 .25-.2.45-.45.45h-.9c-.25 0-.45-.2-.45-.45V8.51c-.21-.06-.4-.15-.58-.25l-.76.77c-.17.17-.46.17-.64 0l-.64-.64a.465.465 0 010-.64l.76-.77c-.1-.19-.19-.38-.25-.59h-1.04c-.25 0-.45-.2-.45-.45v-.9c0-.25.2-.45.45-.45h1.04c.07-.22.16-.42.27-.61l-.73-.73a.429.429 0 010-.61l.61-.61c.17-.17.44-.17.61 0l.73.73c.2-.11.4-.2.62-.26V1.45a.44.44 0 01.44-.45h.9c.25 0 .45.2.45.45V2.5c.21.06.4.15.58.25l.76-.77c.17-.17.46-.17.64 0l.64.64c.17.17.17.46 0 .64l-.76.77c.1.17.19.36.25.57zm-4.69.9c0 .93.75 1.69 1.69 1.69.93 0 1.69-.75 1.69-1.69s-.75-1.69-1.69-1.69-1.69.76-1.69 1.69z"],"series-derived":["M18.82 6.58c-.03.05-.07.09-.11.13 0 0 0-.01-.01-.01l-2 2c-.2.2-.4.3-.7.3-.6 0-1-.4-1-1 0-.3.1-.5.3-.7L16.6 6H11c-.6 0-1-.4-1-1s.4-1 1-1h5.6l-1.3-1.3c-.2-.2-.3-.4-.3-.7 0-.6.4-1 1-1 .3 0 .5.1.7.3l3 3c.2.2.3.4.3.7s-.1.5-.3.7l-.88.88zm-5.53 2.71c.3.62.8 1.12 1.42 1.42l-3 3c-.18.18-.43.29-.71.29s-.53-.11-.71-.3L7 10.41l-5 5V17h17c.55 0 1 .45 1 1s-.45 1-1 1H1a.998.998 0 01-1-1V4c0-.55.45-1 1-1s1 .45 1 1v8.59l4.29-4.3C6.47 8.11 6.72 8 7 8s.53.11.71.29l3.29 3.3 2.29-2.3z"],"series-filtered":["M12.14 10.45c.21.67.65 1.23 1.22 1.61l-1.65 1.65c-.18.18-.43.29-.71.29s-.53-.11-.71-.3L7 10.41l-5 5V17h17c.55 0 1 .45 1 1s-.45 1-1 1H1a.998.998 0 01-1-1V4c0-.55.45-1 1-1s1 .45 1 1v8.59l4.29-4.3C6.47 8.11 6.72 8 7 8s.53.11.71.29l3.29 3.3 1.14-1.14zM19.35 1a.642.642 0 01.46 1.1l-3.03 3.03v2.95c0 .18-.07.34-.19.46l-1.28 1.29c-.11.1-.27.17-.45.17-.35 0-.64-.29-.64-.64V5.13L11.19 2.1a.642.642 0 01.45-1.1h7.71z"],"series-search":["M11.28 11.31l-.28.28-3.29-3.3C7.53 8.11 7.28 8 7 8s-.53.11-.71.29L2 12.59V4c0-.55-.45-1-1-1s-1 .45-1 1v14a.998.998 0 001 1h18c.55 0 1-.45 1-1s-.45-1-1-1H2v-1.59l5-5 3.29 3.29c.18.19.43.3.71.3s.53-.11.71-.29l2.09-2.09c-.17.02-.34.02-.51.02-.7 0-1.38-.12-2.01-.33zm-.93-6c0-1.62 1.31-2.93 2.93-2.93s2.93 1.31 2.93 2.93-1.31 2.93-2.93 2.93-2.93-1.31-2.93-2.93zm6.47 2.43c.11-.17.21-.33.29-.51.01-.03.03-.06.04-.09.08-.18.16-.35.21-.54.06-.2.1-.38.14-.58.01-.05.01-.09.02-.14.03-.2.05-.39.05-.6 0-2.37-1.93-4.3-4.3-4.3-2.37.01-4.3 1.93-4.3 4.31s1.93 4.3 4.3 4.3c.21 0 .4-.02.6-.05.04 0 .09-.01.14-.02.2-.03.38-.08.57-.14.2-.06.37-.14.55-.21.03-.01.06-.03.09-.04.18-.09.34-.19.51-.29l2.87 2.87c.14.14.33.22.56.22.43 0 .78-.35.78-.78a.938.938 0 00-.23-.56l-2.89-2.85z"],settings:["M4 1c0-.55-.45-1-1-1S2 .45 2 1v5h2V1zM2 19c0 .55.45 1 1 1s1-.45 1-1v-6H2v6zm9-18c0-.55-.45-1-1-1S9 .45 9 1v8h2V1zm7 0c0-.55-.45-1-1-1s-1 .45-1 1v3h2V1zM9 19c0 .55.45 1 1 1s1-.45 1-1v-3H9v3zm9-14h-2c-.55 0-1 .45-1 1v3c0 .55.45 1 1 1h2c.55 0 1-.45 1-1V6c0-.55-.45-1-1-1zm-2 14c0 .55.45 1 1 1s1-.45 1-1v-8h-2v8zM4 7H2c-.55 0-1 .45-1 1v3c0 .55.45 1 1 1h2c.55 0 1-.45 1-1V8c0-.55-.45-1-1-1zm7 3H9c-.55 0-1 .45-1 1v3c0 .55.45 1 1 1h2c.55 0 1-.45 1-1v-3c0-.55-.45-1-1-1z"],share:["M15 18H2V5h8.76l2-2H1c-.55 0-1 .45-1 1v15c0 .55.45 1 1 1h15c.55 0 1-.45 1-1V7.24l-2 2V18zm4-18h-7c-.55 0-1 .45-1 1s.45 1 1 1h4.59l-7.3 7.29a1.003 1.003 0 001.42 1.42L18 3.41V8c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1z"],shield:["M10 20c6-3.81 9-9.048 9-15.714-2 0-5-1.429-9-4.286-4 2.857-7 4.286-9 4.286C1 10.952 4 16.19 10 20zm0-17.348c2.577 1.734 4.776 2.88 6.667 3.419-.44 4.627-2.636 8.353-6.667 11.297V2.652z"],shop:["M17.94 3.63c-.01-.02-.01-.03-.02-.04l-.03-.09h-.01c-.18-.3-.49-.5-.86-.5h-14c-.42 0-.77.25-.92.61L0 8.5h.02a2.5 2.5 0 005 0 2.5 2.5 0 005 0 2.5 2.5 0 005 0 2.5 2.5 0 005 0l-2.08-4.87zM3.02 2h14c.55 0 1-.45 1-1s-.45-1-1-1h-14c-.55 0-1 .45-1 1s.44 1 1 1zm13 14h-12v-4h-2v7c0 .55.45 1 1 1h14c.55 0 1-.45 1-1v-7h-2v4z"],"shopping-cart":["M18 14H8.72l-.67-2H17c.44 0 .8-.29.94-.69h.01l2-6h-.01c.03-.1.06-.2.06-.31 0-.55-.45-1-1-1H5.39l-.44-1.32h-.01C4.8 2.29 4.44 2 4 2H1c-.55 0-1 .45-1 1s.45 1 1 1h2.28l3.33 10H5c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2h9c0 1.1.9 2 2 2s2-.9 2-2-.9-2-2-2zM6.05 6h11.56l-1.33 4H7.39L6.05 6z"],"signal-search":["M7.15 10.33c.888.8 1.999 1.36 3.228 1.574l2.326 6.98a.846.846 0 01-.535 1.07.844.844 0 01-1.072-.535l-1.225-3.671H7.125L5.9 19.419a.85.85 0 01-1.072.536.85.85 0 01-.536-1.071l2.857-8.555zm1.353 1.305l-.808 2.413h1.607l-.8-2.413zM5 5.5c0 .76.13 1.49.37 2.17-.496 1.056-.313 2.356.704 3.29.385.353.404.94.038 1.311a.982.982 0 01-1.356.038c-2.183-2.01-2-5.125.01-6.94a.95.95 0 01.24-.156A6.421 6.421 0 005 5.5z","M3.874 13.185c-1.346-.918-2.187-2.67-2.187-4.34 0-1.752.757-3.254 2.187-4.339.42-.25.42-.834.168-1.168-.252-.418-.84-.418-1.177-.167C1.014 4.59-.08 6.509.005 8.846c.084 2.253 1.177 4.423 2.86 5.675.168.083.336.166.504.166.253 0 .505-.083.673-.333.337-.418.253-.918-.168-1.169zM12.246 12.309a.98.98 0 01-1.354-.037.917.917 0 01-.206-.324 6.54 6.54 0 001.959-.049 5.125 5.125 0 01-.399.41zM14.631 11.476l1.228 1.229a6.6 6.6 0 01-1.723 1.816c-.169.083-.337.166-.505.166-.253 0-.505-.083-.673-.333-.337-.418-.253-.918.168-1.169.62-.422 1.133-1.022 1.505-1.709z","M11.5 0C14.54 0 17 2.46 17 5.5c0 .26-.02.51-.06.75l-.03.17c-.04.25-.1.49-.17.73v.01c-.08.24-.17.47-.28.69-.01.04-.03.07-.05.11-.11.23-.24.44-.38.65l3.68 3.68A1.003 1.003 0 0119 14c-.28 0-.53-.11-.7-.29l-3.68-3.68c-.21.14-.42.27-.65.38-.04.01-.07.03-.11.05-.22.11-.45.2-.69.28h-.01c-.24.07-.48.13-.73.17l-.17.03c-.25.04-.5.06-.76.06C8.46 11 6 8.54 6 5.5S8.46 0 11.5 0zm0 1.5c-2.21 0-4 1.79-4 4s1.79 4 4 4 4-1.79 4-4-1.79-4-4-4z"],"sim-card":["M16.71 5.29l-5-5A.997.997 0 0011 0H4c-.55 0-1 .45-1 1v18c0 .55.45 1 1 1h12c.55 0 1-.45 1-1V6c0-.28-.11-.53-.29-.71zM9 7h2v3H9V7zM6 7h2v3H6V7zm2 11H6v-3h2v3zm3 0H9v-3h2v3zm3 0h-2v-3h2v3zm0-4H6v-3h8v3zm0-4h-2V7h2v3z"],slash:["M12 2c-.46 0-.85.32-.97.74L7.04 16.7c-.02.1-.04.2-.04.3 0 .55.45 1 1 1 .46 0 .85-.32.97-.74L12.96 3.3c.02-.1.04-.2.04-.3 0-.55-.45-1-1-1z"],"small-cross":["M11.41 10l3.29-3.29c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71L10 8.59l-3.29-3.3a1.003 1.003 0 00-1.42 1.42L8.59 10 5.3 13.29c-.19.18-.3.43-.3.71a1.003 1.003 0 001.71.71l3.29-3.3 3.29 3.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71L11.41 10z"],"small-minus":["M14 9H6c-.55 0-1 .45-1 1s.45 1 1 1h8c.55 0 1-.45 1-1s-.45-1-1-1z"],"small-plus":["M14 9h-3V6c0-.55-.45-1-1-1s-1 .45-1 1v3H6c-.55 0-1 .45-1 1s.45 1 1 1h3v3c0 .55.45 1 1 1s1-.45 1-1v-3h3c.55 0 1-.45 1-1s-.45-1-1-1z"],"small-tick":["M15 5c-.28 0-.53.11-.71.29L8 11.59l-2.29-2.3a1.003 1.003 0 00-1.42 1.42l3 3c.18.18.43.29.71.29s.53-.11.71-.29l7-7A1.003 1.003 0 0015 5z"],snowflake:["M11 11.776v2.81l2.31 2.242a.987.987 0 010 1.415c-.399.39-1.044.39-1.442 0L11 17.414V19a.99.99 0 01-.996 1A.996.996 0 019 19v-1.636l-.912.879c-.398.39-1.043.39-1.441 0a.987.987 0 010-1.415L9 14.536v-2.79l-2.548 1.435-.837 3.063c-.146.534-.705.85-1.248.707a.998.998 0 01-.721-1.224l.309-1.132-1.4.793a1.03 1.03 0 01-1.393-.366.99.99 0 01.373-1.366l1.445-.818-1.224-.322a.998.998 0 01-.72-1.225c.145-.533.704-.85 1.248-.707l3.193.84 2.462-1.395-2.532-1.434-3.123.82a1.022 1.022 0 01-1.249-.706.998.998 0 01.721-1.225L2.91 7.18l-1.4-.793a.99.99 0 01-.373-1.366 1.03 1.03 0 011.392-.366l1.445.818-.328-1.2a.998.998 0 01.72-1.225 1.022 1.022 0 011.25.707l.855 3.132L9 8.311V5.414L6.647 3.121a.987.987 0 010-1.414 1.033 1.033 0 011.441 0L9 2.586V1c0-.552.44-1 1.004-1A.99.99 0 0111 1l-.007 1.536.875-.829a1.033 1.033 0 011.441 0 .987.987 0 010 1.414L11 5.364v2.918l2.53-1.42.855-3.131c.146-.534.705-.85 1.249-.707a.998.998 0 01.72 1.224l-.327 1.2 1.4-.792a1.03 1.03 0 011.392.366.99.99 0 01-.373 1.366l-1.355.768 1.153.303a.998.998 0 01.721 1.225c-.146.533-.705.85-1.249.707l-3.123-.821-2.576 1.459 2.506 1.42 3.193-.84a1.022 1.022 0 011.249.707.998.998 0 01-.72 1.225l-1.224.322 1.4.793a.99.99 0 01.373 1.366 1.03 1.03 0 01-1.393.366l-1.356-.768.31 1.132a.998.998 0 01-.721 1.224 1.022 1.022 0 01-1.249-.707l-.837-3.063L11 11.776z"],"social-media":["M11.5 5c.8 0 1.6-.4 2-1 2 1.2 3.3 3.3 3.5 5.7 0 .5.5.9 1 .9.6 0 1-.5 1-1v-.1c-.2-3.3-2.2-6.2-5.1-7.6C13.7.8 12.7 0 11.5 0 10.1 0 9 1.1 9 2.5S10.1 5 11.5 5zm5 7c-1.4 0-2.5 1.1-2.5 2.5 0 .4.1.7.2 1.1-1.1.9-2.6 1.4-4.2 1.4-1.9 0-3.6-.8-4.9-2-.2-.2-.5-.4-.8-.4-.5 0-1 .5-1 1 0 .3.1.5.3.7C5.3 18 7.5 19 10 19c2.2 0 4.2-.8 5.8-2.1.2.1.5.1.7.1 1.4 0 2.5-1.1 2.5-2.5S17.9 12 16.5 12zM5 10.5c0-1.1-.7-2.1-1.7-2.4.5-1.9 1.9-3.5 3.6-4.4.3-.2.6-.5.6-.9 0-.5-.4-1-1-1-.2 0-.4.1-.6.2-2.4 1.2-4.2 3.6-4.7 6.4C.5 8.9 0 9.6 0 10.5 0 11.9 1.1 13 2.5 13S5 11.9 5 10.5z"],sort:["M19 16h-9c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h9c.55 0 1-.45 1-1v-1c0-.55-.45-1-1-1zm0-5h-9c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h9c.55 0 1-.45 1-1v-1c0-.55-.45-1-1-1zM7 15c-.28 0-.53.11-.71.29L5 16.59V11c0-.55-.45-1-1-1s-1 .45-1 1v5.59L1.71 15.3A.965.965 0 001 15a1.003 1.003 0 00-.71 1.71l3 3c.18.18.43.29.71.29s.53-.11.71-.29l3-3A1.003 1.003 0 007 15zM19 1h-9c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h9c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1zm0 5h-9c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h9c.55 0 1-.45 1-1V7c0-.55-.45-1-1-1z"],"sort-alphabetical":["M8 15c-.28 0-.53.11-.71.29L6 16.59v-5.58c0-.55-.45-1-1-1s-1 .45-1 1v5.58L2.71 15.3c-.18-.18-.43-.3-.71-.3a1.003 1.003 0 00-.71 1.71l3 3c.18.18.43.29.71.29s.53-.11.71-.29l3-3A1.003 1.003 0 008 15zm8.89-.79v-1.22H11.3v1.3h3.51L11 18.78V20h5.99v-1.3h-3.91l3.81-4.49zM14.97 0h-1.95L9.01 11.01h1.89l.98-2.92h4.17l.98 2.92h1.96L14.97 0zm-2.59 6.63l1.58-4.74H14l1.57 4.74h-3.19z"],"sort-alphabetical-desc":["M8.01 15c-.28 0-.53.11-.71.29L6 16.59v-5.58c0-.55-.45-1-1-1s-1 .45-1 1v5.58L2.71 15.3c-.18-.18-.43-.3-.71-.3a1.003 1.003 0 00-.71 1.71l3 3a1.014 1.014 0 001.42 0l3-3c.18-.18.29-.43.29-.71.01-.55-.44-1-.99-1zm4.44-5.65l6.4-7.88V0H10.5v1.67h5.91L10 9.44v1.57h9V9.35h-6.55zm1.27 3.64L11 20h1.59l.56-1.56h2.68l.55 1.56h1.64l-2.68-7.01h-1.62zm-.16 4.3l.93-2.57h.02l.9 2.57h-1.85z"],"sort-asc":["M10 8h5c.55 0 1-.45 1-1V6c0-.55-.45-1-1-1h-5c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1zm0 5h7c.55 0 1-.45 1-1v-1c0-.55-.45-1-1-1h-7c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1zm0-10h3c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1h-3c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1zm9 12h-9c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h9c.55 0 1-.45 1-1v-1c0-.55-.45-1-1-1zM7 14c-.28 0-.53.11-.71.29L5 15.59V10c0-.55-.45-1-1-1s-1 .45-1 1v5.59L1.71 14.3A.965.965 0 001 14a1.003 1.003 0 00-.71 1.71l3 3c.18.18.43.29.71.29s.53-.11.71-.29l3-3A1.003 1.003 0 007 14z"],"sort-desc":["M13 15h-3c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h3c.55 0 1-.45 1-1v-1c0-.55-.45-1-1-1zm-6-1c-.28 0-.53.11-.71.29L5 15.59V10c0-.55-.45-1-1-1s-1 .45-1 1v5.59L1.71 14.3A.965.965 0 001 14a1.003 1.003 0 00-.71 1.71l3 3c.18.18.43.29.71.29s.53-.11.71-.29l3-3A1.003 1.003 0 007 14zM19 0h-9c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h9c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-4 10h-5c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h5c.55 0 1-.45 1-1v-1c0-.55-.45-1-1-1zm2-5h-7c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1h7c.55 0 1-.45 1-1V6c0-.55-.45-1-1-1z"],"sort-numerical":["M9 14.99c-.28 0-.53.11-.71.29L7 16.58v-5.59c0-.55-.45-1-1-1s-1 .45-1 1v5.59l-1.29-1.29a.965.965 0 00-.71-.3 1.003 1.003 0 00-.71 1.71l3 3c.18.18.43.29.71.29.28 0 .53-.11.71-.29l3-3c.18-.18.29-.43.29-.71a.99.99 0 00-1-1zm8.88.23c-.08-.42-.22-.79-.42-1.12-.2-.33-.47-.6-.8-.8-.33-.2-.76-.3-1.28-.3a2.333 2.333 0 00-1.72.71c-.21.22-.37.48-.49.78-.11.3-.17.62-.17.97 0 .27.04.54.13.8.08.26.22.5.4.7.19.21.43.38.71.5a2.142 2.142 0 001.72.02c.25-.12.47-.31.66-.58l.02.02c-.01.19-.04.4-.08.63-.04.24-.11.46-.21.67-.1.21-.23.38-.39.53a.92.92 0 01-.62.22c-.24 0-.44-.08-.6-.25-.16-.17-.27-.36-.31-.59h-1.31c.04.29.12.56.24.79.12.23.28.43.48.59.19.16.42.28.67.36.25.08.52.12.82.12.49 0 .9-.1 1.23-.31.34-.21.61-.48.82-.82.21-.34.37-.71.47-1.13.1-.42.15-.83.15-1.25 0-.43-.04-.85-.12-1.26zm-1.42.63c-.05.15-.11.28-.2.4-.09.12-.2.21-.34.27s-.3.1-.49.1c-.17 0-.33-.04-.46-.11s-.24-.17-.33-.29c-.08-.12-.15-.25-.19-.4-.04-.15-.06-.31-.06-.47 0-.15.02-.3.07-.45.05-.15.11-.28.2-.39.09-.12.2-.21.33-.28.13-.07.27-.11.44-.11.17 0 .33.04.47.11.14.07.25.17.34.28a1.387 1.387 0 01.28.86c.01.17-.02.33-.06.48zM15.32 11H17V0h-1.25c-.05.34-.17.62-.34.85-.17.23-.39.42-.63.57-.25.15-.52.25-.83.31-.3.06-.62.09-.94.09v1.41h2.31V11z"],"sort-numerical-desc":["M9 15c-.28 0-.53.11-.71.29L7 16.59v-5.58c0-.55-.45-1-1-1s-1 .45-1 1v5.58L3.71 15.3c-.18-.18-.43-.3-.71-.3a1.003 1.003 0 00-.71 1.71l3 3c.18.18.43.29.71.29.28 0 .53-.11.71-.29l3-3A1.003 1.003 0 009 15zm6.7-1.33a1.5 1.5 0 01-.44.43c-.17.11-.37.19-.58.23-.22.04-.44.06-.67.05v1.07h1.66V20H17v-6.99h-1.06c-.04.26-.12.48-.24.66zm3.15-10.3c-.11-.68-.29-1.26-.55-1.76-.26-.5-.62-.89-1.08-1.18C16.75.14 16.17 0 15.46 0c-.54 0-1.03.09-1.46.27-.43.18-.79.44-1.09.76-.3.33-.52.71-.67 1.15-.16.44-.24.92-.24 1.43 0 .54.08 1.04.23 1.47.15.44.37.81.65 1.12.28.31.61.55 1 .72.39.17.82.26 1.3.26.46 0 .88-.11 1.26-.33.38-.22.68-.53.9-.94l.03.03c-.03.35-.07.74-.12 1.16-.05.42-.15.81-.29 1.18-.14.37-.35.68-.61.92-.26.25-.62.37-1.06.37-.43 0-.77-.13-1.03-.4-.25-.27-.4-.62-.44-1.05h-1.64c.02.43.11.83.29 1.18.17.35.39.66.67.91a3.027 3.027 0 002.07.8c.71 0 1.3-.17 1.79-.5.48-.33.87-.76 1.17-1.29.3-.53.51-1.12.64-1.76.13-.64.19-1.28.19-1.92.01-.77-.05-1.49-.15-2.17zM17.1 4.44c-.08.27-.19.5-.34.71-.15.21-.34.37-.57.49-.23.12-.5.18-.8.18-.3 0-.56-.06-.78-.19-.22-.13-.4-.29-.55-.49-.14-.2-.25-.44-.32-.7-.07-.27-.11-.55-.11-.84 0-.28.04-.55.11-.82.07-.26.18-.49.32-.7.14-.2.33-.36.55-.48.22-.12.48-.17.78-.17.31 0 .57.06.8.18.23.12.42.28.57.48.15.2.26.43.34.69.08.26.11.53.11.82 0 .29-.04.57-.11.84z"],"split-columns":["M15 13a1.003 1.003 0 001.71.71l3-3c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71l-3-3a1.003 1.003 0 00-1.42 1.42L16.59 9H11V2h5v2c.77 0 1.47.3 2 .78V1c0-.55-.45-1-1-1H3c-.55 0-1 .45-1 1v3.78C2.53 4.3 3.23 4 4 4V2h5v7H3.41L4.7 7.71c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71l-3 3C.11 9.47 0 9.72 0 10c0 .28.11.53.29.71l3 3a1.003 1.003 0 001.42-1.42L3.41 11H9v7H4v-2c-.77 0-1.47-.3-2-.78V19c0 .55.45 1 1 1h14c.55 0 1-.45 1-1v-3.78c-.53.48-1.23.78-2 .78v2h-5v-7h5.59l-1.29 1.29c-.19.18-.3.43-.3.71z"],square:["M19 0H1C.45 0 0 .45 0 1v18c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 18H2V2h16v16z"],"stacked-chart":["M12 2c0-.55-.45-1-1-1H9c-.55 0-1 .45-1 1v4h4V2zm3 14h2c.55 0 1-.45 1-1v-5h-4v5c0 .55.45 1 1 1zm3-10c0-.55-.45-1-1-1h-2c-.55 0-1 .45-1 1v3h4V6zm-6 1H8v5h4V7zm-9 9h2c.55 0 1-.45 1-1v-3H2v3c0 .55.45 1 1 1zm16 1H1c-.55 0-1 .45-1 1s.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1zM6 9c0-.55-.45-1-1-1H3c-.55 0-1 .45-1 1v2h4V9zm3 7h2c.55 0 1-.45 1-1v-2H8v2c0 .55.45 1 1 1z"],star:["M10 0l3.1 6.6 6.9 1-5 5.1 1.2 7.3-6.2-3.4L3.8 20 5 12.7 0 7.6l6.9-1z"],"star-empty":["M20 7.6l-6.9-1.1L10 0 6.9 6.6 0 7.6l5 5.1L3.8 20l6.2-3.4 6.2 3.4-1.2-7.2 5-5.2zM10 15l-4.5 2.4.9-5.2-3.6-3.6 5-.8L10 3.1l2.2 4.7 5 .8-3.6 3.7.9 5.2L10 15z"],"step-backward":["M15 3c-.23 0-.42.09-.59.21l-.01-.01L8 8V4c0-.55-.45-1-1-1H5c-.55 0-1 .45-1 1v12c0 .55.45 1 1 1h2c.55 0 1-.45 1-1v-4l6.4 4.8.01-.01c.17.12.36.21.59.21.55 0 1-.45 1-1V4c0-.55-.45-1-1-1z"],"step-chart":["M19 16H2v-3h4c.55 0 1-.45 1-1V8h3v2c0 .55.45 1 1 1h5c.55 0 1-.45 1-1V6h2c.55 0 1-.45 1-1s-.45-1-1-1h-3c-.55 0-1 .45-1 1v4h-3V7c0-.55-.45-1-1-1H6c-.55 0-1 .45-1 1v4H2V3c0-.55-.45-1-1-1s-1 .45-1 1v14c0 .55.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1z"],"step-forward":["M15 3h-2c-.55 0-1 .45-1 1v4L5.6 3.2l-.01.01C5.42 3.09 5.23 3 5 3c-.55 0-1 .45-1 1v12c0 .55.45 1 1 1 .23 0 .42-.09.59-.21l.01.01L12 12v4c0 .55.45 1 1 1h2c.55 0 1-.45 1-1V4c0-.55-.45-1-1-1z"],stop:["M16 3H4c-.55 0-1 .45-1 1v12c0 .55.45 1 1 1h12c.55 0 1-.45 1-1V4c0-.55-.45-1-1-1z"],stopwatch:["M10 6a6 6 0 106 6h-6V6zm-.998-1.938A1.015 1.015 0 019 4V2H7a1 1 0 110-2h6a1 1 0 010 2h-2v2c0 .02 0 .041-.002.062A8.001 8.001 0 0110 20a8 8 0 01-.998-15.938z"],strikethrough:["M18 9h-4.46a4.7 4.7 0 00-.4-.14c-.19-.05-.51-.14-.96-.25-.45-.11-.9-.23-1.37-.35-.47-.12-.89-.23-1.27-.33s-.6-.16-.65-.17c-.53-.15-.95-.37-1.27-.66-.32-.28-.49-.68-.49-1.19 0-.36.09-.66.26-.9s.39-.43.65-.57c.26-.14.55-.24.87-.3s.63-.09.93-.09c.89 0 1.63.19 2.21.57.45.3.75.76.89 1.38h2.63c-.06-.52-.2-.98-.42-1.4-.3-.57-.71-1.05-1.23-1.43a5.33 5.33 0 00-1.79-.87c-.7-.2-1.42-.3-2.19-.3-.66 0-1.31.08-1.96.25s-1.22.43-1.73.77-.92.79-1.23 1.32c-.31.52-.46 1.15-.46 1.87 0 .37.05.74.15 1.1.1.36.28.7.53 1.02.18.24.41.47.69.67H2c-.55 0-1 .45-1 1s.45 1 1 1h10.14c.02.01.05.02.07.02.3.11.58.29.84.55.25.26.38.67.38 1.21 0 .27-.06.53-.17.79-.11.26-.29.49-.54.69-.25.2-.57.36-.97.49s-.88.19-1.44.19c-.52 0-1.01-.06-1.45-.17-.45-.11-.84-.29-1.19-.54s-.61-.56-.8-.95c-.05-.08-.09-.18-.12-.28H4.11c.09.43.22.82.4 1.18.33.65.77 1.18 1.32 1.59.55.41 1.2.72 1.94.92.74.2 1.53.3 2.37.3.73 0 1.44-.08 2.14-.25.7-.17 1.33-.43 1.88-.79.55-.36.99-.83 1.33-1.39.34-.56.51-1.25.51-2.05 0-.37-.06-.75-.18-1.12a3.12 3.12 0 00-.15-.39H18c.55 0 1-.45 1-1s-.45-1-1-1z"],style:["M18 18H2V2h12.3l2-2H1C.4 0 0 .4 0 1v18c0 .6.4 1 1 1h18c.6 0 1-.4 1-1V7.7l-2 2V18zm1.2-18l-7.6 7.6 2.8 2.8L20 4.8V0h-.8zM4 15.9c3.1.2 5.9.2 8.2-2 1.1-1.1 1.1-3 0-4.1-.6-.5-1.3-.8-2-.8s-1.4.3-1.9.8C7.2 11 6.6 14.3 4 15.9z"],"swap-horizontal":["M16.02 10c-.01 0-.01 0 0 0H16h.02zM2 6h13.58l-2.29 2.29a1 1 0 00-.3.71 1.003 1.003 0 001.71.71l4-4c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71l-4-4a1.003 1.003 0 00-1.42 1.42L15.58 4H2c-.55 0-1 .45-1 1s.45 1 1 1zm2 4h-.02H4zm14 4H4.42l2.29-2.29a1 1 0 00.3-.71 1.003 1.003 0 00-1.71-.71l-4 4c-.18.18-.29.43-.29.71 0 .28.11.53.29.71l4 4a1.003 1.003 0 001.42-1.42L4.42 16H18c.55 0 1-.45 1-1s-.45-1-1-1z"],"swap-vertical":["M9.71 5.3l-4-4A.997.997 0 005 1.01c-.28 0-.53.11-.71.29l-4 4a1.003 1.003 0 001.42 1.42L4 4.42V18c0 .55.45 1 1 1s1-.45 1-1V4.42l2.29 2.29a1 1 0 00.71.3 1.003 1.003 0 00.71-1.71zM10 3.98c0 .01 0 .01 0 0V4v-.02zm0 12.04c0-.01 0-.01 0 0V16v.02zm9-3.03c-.28 0-.53.11-.71.29L16 15.58V2c0-.55-.45-1-1-1s-1 .45-1 1v13.58l-2.29-2.29a1.003 1.003 0 00-1.42 1.42l4 4c.18.18.43.29.71.29.28 0 .53-.11.71-.29l4-4c.18-.18.29-.43.29-.71 0-.56-.45-1.01-1-1.01z"],switch:["M12.293 2.293l1.414 1.414-7.127 7.129a3.5 3.5 0 11-1.415-1.415l7.128-7.128zM16.5 9a3.5 3.5 0 110 7 3.5 3.5 0 010-7zm-13 2a1.5 1.5 0 100 3 1.5 1.5 0 000-3zm13 0a1.5 1.5 0 100 3 1.5 1.5 0 000-3z"],"symbol-circle":["M10 4.01a6 6 0 100 12 6 6 0 100-12z"],"symbol-cross":["M15 8.01h-3v-3c0-.55-.45-1-1-1H9c-.55 0-1 .45-1 1v3H5c-.55 0-1 .45-1 1v2c0 .55.45 1 1 1h3v3c0 .55.45 1 1 1h2c.55 0 1-.45 1-1v-3h3c.55 0 1-.45 1-1v-2c0-.56-.45-1-1-1z"],"symbol-diamond":["M15 10.01c0-.21-.08-.39-.18-.54l.02-.01-4-6-.02.01c-.18-.28-.47-.46-.82-.46s-.64.18-.82.45l-.01-.01-4 6 .02.01c-.11.16-.19.34-.19.55s.08.39.18.54l-.02.01 4 6 .02-.01c.18.27.47.46.82.46s.64-.19.82-.46l.02.01 4-6-.02-.01c.1-.16.18-.34.18-.54z"],"symbol-square":["M15 4.01H5c-.55 0-1 .45-1 1v10c0 .55.45 1 1 1h10c.55 0 1-.45 1-1v-10c0-.56-.45-1-1-1z"],"symbol-triangle-down":["M16 5c0-.55-.45-1-1-1H5c-.55 0-1 .45-1 1 0 .16.05.31.11.44H4.1l5 10h.01c.17.33.5.56.89.56s.72-.23.89-.56h.01l5-10h-.01c.06-.13.11-.28.11-.44z"],"symbol-triangle-up":["M15.89 14.56l-4.99-10h-.01c-.17-.33-.5-.56-.89-.56s-.72.23-.89.56H9.1l-5 10h.01c-.06.13-.11.28-.11.44 0 .55.45 1 1 1h10c.55 0 1-.45 1-1 0-.16-.05-.31-.11-.44z"],tag:["M2 4a2 2 0 012-2h4.588a2 2 0 011.414.586l7.41 7.41a2 2 0 010 2.828l-4.588 4.588a2 2 0 01-2.829 0l-7.41-7.41A2 2 0 012 8.588V4zm3.489-.006a1.495 1.495 0 100 2.99 1.495 1.495 0 000-2.99z"],"take-action":["M5 7c.28 0 .53-.11.71-.29l5-5A1.003 1.003 0 009.29.29l-5 5A1.003 1.003 0 005 7zm6 6a1.003 1.003 0 001.71.71l5-5a1.003 1.003 0 00-1.42-1.42l-5 5c-.18.18-.29.43-.29.71zm8 5h-1c0-.55-.45-1-1-1h-7c-.55 0-1 .45-1 1H8c-.55 0-1 .45-1 1s.45 1 1 1h11c.55 0 1-.45 1-1s-.45-1-1-1zm-9-6l6-6-1.29-1.29a1.003 1.003 0 00-1.42-1.42L12 2 6 8l1.29 1.29-7 7a1.003 1.003 0 001.42 1.42l7-7L10 12z"],taxi:["M19 9h-.33l.33 1v.5c0 .15-.03.3-.07.44h.01L17 17.23v.27c0 .83-.67 1.5-1.5 1.5s-1.5-.67-1.5-1.5V17H6v.5c0 .83-.67 1.5-1.5 1.5S3 18.33 3 17.5v-.27l-1.93-6.28h.01c-.05-.15-.08-.3-.08-.45V10s.02-.06.05-.16c.06-.17.16-.47.28-.84H1c-.55 0-1-.45-1-1s.45-1 1-1h1l1-3h-.01v-.01c.25-.64 1-1.31 1.67-1.5 0 0 .78-.21 2.33-.36V1c0-.55.45-1 1-1h4c.55 0 1 .45 1 1v1.13c1.55.14 2.33.36 2.33.36.67.19 1.42.86 1.67 1.5V4H17l1 3h1c.55 0 1 .45 1 1s-.45 1-1 1zM3 11.5c0 .83.67 1.5 1.5 1.5S6 12.33 6 11.5 5.33 10 4.5 10 3 10.67 3 11.5zM16 7l-1-3H5L4 7v1h12V7zm-.5 3c-.83 0-1.5.67-1.5 1.5s.67 1.5 1.5 1.5 1.5-.67 1.5-1.5-.67-1.5-1.5-1.5z"],"text-highlight":["M16 17c-.55 0-1-.45-1-1V4c0-.55.45-1 1-1s1-.45 1-1-.45-1-1-1c-.77 0-1.47.3-2 .78-.53-.48-1.23-.78-2-.78-.55 0-1 .45-1 1s.45 1 1 1 1 .45 1 1v12c0 .55-.45 1-1 1s-1 .45-1 1 .45 1 1 1c.77 0 1.47-.3 2-.78.53.48 1.23.78 2 .78.55 0 1-.45 1-1s-.45-1-1-1zm-4-4H2V7h10V5H1c-.55 0-1 .45-1 1v8c0 .55.45 1 1 1h11v-2zm7-8h-3v2h2v6h-2v2h3c.55 0 1-.45 1-1V6c0-.55-.45-1-1-1z"],th:["M19 1H1c-.6 0-1 .5-1 1v16c0 .5.4 1 1 1h18c.5 0 1-.5 1-1V2c0-.5-.5-1-1-1zM7 17H2v-3h5v3zm0-4H2v-3h5v3zm0-4H2V6h5v3zm11 8H8v-3h10v3zm0-4H8v-3h10v3zm0-4H8V6h10v3z"],"th-derived":["M5.3 13.3c-.2.2-.3.4-.3.7 0 .6.4 1 1 1 .3 0 .5-.1.7-.3l3-3c.2-.2.3-.4.3-.7s-.1-.5-.3-.7l-3-3C6.5 7.1 6.3 7 6 7c-.6 0-1 .4-1 1 0 .3.1.5.3.7L6.6 10H1c-.6 0-1 .4-1 1s.4 1 1 1h5.6l-1.3 1.3zM19 1H3c-.5 0-1 .5-1 1v6h1c0-1.7 1.3-3 3-3 .8 0 1.6.3 2.1.9l.1.1H9v.8l1 1V6h8v3h-6.8c.3.3.5.6.6 1H18v3h-6.8l-.1.1-.9.9H18v3h-8v-2.8l-1 1V17H4v-.8c-.6-.5-1-1.3-1-2.2H2v4c0 .5.5 1 1 1h16c.6 0 1-.5 1-1V2c0-.5-.5-1-1-1z"],"th-disconnect":["M14.25 1H19c.5 0 1 .5 1 1v16c0 .5-.5 1-1 1h-7.221l.278-2H18v-3h-5.527l.14-1H18v-3h-4.971l.139-1H18V6h-4.416l.637-4.587c.02-.139.03-.277.03-.413zM8.221 1l-.694 5H2v3h5.11l-.139 1H2v3h4.555l-.14 1H2v3h3.999l-.22 1.587c-.02.139-.03.277-.03.413H1c-.6 0-1-.5-1-1V2c0-.5.4-1 1-1h7.221zM10.26.862a1 1 0 011.98.276l-2.5 18a1 1 0 01-1.98-.276l2.5-18z"],"th-filtered":["M17.333 10l1.435-1.722a1 1 0 00.232-.64V4.85l1-.9V18c0 .5-.5 1-1 1H1c-.6 0-1-.5-1-1V2c0-.5.4-1 1-1h6.722L12 4.85V6H8v3h4v1H8v3h10v-3h-.667zM7 17v-3H2v3h5zm0-4v-3H2v3h5zm0-4V6H2v3h5zm11 8v-3H8v3h10z","M19.35 0a.642.642 0 01.46 1.1l-3.03 3.03v2.95c0 .18-.07.34-.19.46l-1.28 1.29c-.11.1-.27.17-.45.17-.35 0-.64-.29-.64-.64V4.13L11.19 1.1a.642.642 0 01.45-1.1h7.71z"],"th-list":["M19 1H1c-.6 0-1 .5-1 1v16c0 .5.4 1 1 1h18c.5 0 1-.5 1-1V2c0-.5-.5-1-1-1zm-1 16H2v-3h16v3zm0-4H2v-3h16v3zm0-4H2V6h16v3z"],"thumbs-down":["M18.55 6.56c-.31-.01-.65-.03-1.02-.06.03 0 .06-.01.09-.01.88-.12 1.68-.63 1.76-1.37.08-.75-.58-1.25-1.46-1.33-.32-.03-.65-.05-.99-.08.59-.19 1.05-.54 1.09-1.2.05-.75-.99-1.32-1.87-1.41-.34-.03-.64-.05-.91-.07h-.11c-.28-.02-.54-.02-.77-.02-3.92-.08-7.29.6-9.36 1.93v7.72c2.67 1.66 5.95 4.61 5.26 7.08-.21.76.39 1.35 1.23 1.26 1.01-.11 1.71-1.18 1.75-2.28.05-1.29-.19-2.59-.62-3.74-.05-.32.01-.65.47-.68.61-.04 1.39-.08 1.99-.1.32 0 .64-.01.94-.03h.01c.52-.03 1-.07 1.42-.12.88-.11 1.69-.6 1.79-1.35.1-.75-.55-1.25-1.44-1.35-.07-.01-.13-.02-.2-.02.21-.02.42-.04.61-.06.88-.11 1.69-.6 1.79-1.35.09-.75-.56-1.31-1.45-1.36zM3 3H0v8h3c.55 0 1-.45 1-1V4c0-.55-.45-1-1-1z"],"thumbs-up":["M3 9H0v8h3c.55 0 1-.45 1-1v-6c0-.55-.45-1-1-1zm16.99 3.09c-.1-.75-.91-1.24-1.79-1.35-.19-.02-.4-.05-.61-.06.07-.01.14-.01.2-.02.88-.1 1.53-.61 1.44-1.35-.1-.74-.91-1.24-1.79-1.35-.42-.05-.9-.09-1.42-.12h-.01l-.94-.03c-.6-.02-1.39-.05-1.99-.1-.45-.03-.51-.36-.47-.68.43-1.15.67-2.45.62-3.74-.04-1.11-.74-2.17-1.75-2.28-.84-.09-1.45.5-1.23 1.26.7 2.47-2.58 5.43-5.25 7.08v7.72c2.08 1.33 5.44 2.01 9.35 1.93.24 0 .49-.01.77-.02h.11c.27-.02.57-.04.91-.07.88-.08 1.92-.66 1.87-1.41-.04-.65-.5-1.01-1.09-1.2.34-.03.67-.05.99-.08.89-.08 1.55-.58 1.46-1.33-.08-.75-.88-1.25-1.76-1.37-.03 0-.06-.01-.09-.01.37-.02.71-.04 1.02-.06.91-.05 1.55-.61 1.45-1.36z"],tick:["M17 4c-.28 0-.53.11-.71.29L7 13.59 3.71 10.3A.965.965 0 003 10a1.003 1.003 0 00-.71 1.71l4 4c.18.18.43.29.71.29s.53-.11.71-.29l10-10A1.003 1.003 0 0017 4z"],"tick-circle":["M10 20C4.48 20 0 15.52 0 10S4.48 0 10 0s10 4.48 10 10-4.48 10-10 10zm5-14c-.28 0-.53.11-.71.29L8 12.59l-2.29-2.3a1.003 1.003 0 00-1.42 1.42l3 3c.18.18.43.29.71.29.28 0 .53-.11.71-.29l7-7A1.003 1.003 0 0015 6z"],time:["M11 9.59V4c0-.55-.45-1-1-1s-1 .45-1 1v6c0 .28.11.53.29.71l3 3a1.003 1.003 0 001.42-1.42L11 9.59zM10 0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zm0 18c-4.42 0-8-3.58-8-8s3.58-8 8-8 8 3.58 8 8-3.58 8-8 8z"],"timeline-area-chart":["M19 16H2V3c0-.55-.45-1-1-1s-1 .45-1 1v14c0 .55.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1zm0-13.41l-7.07 7.07-4.3-3.44-.01.01A.987.987 0 007 6c-.24 0-.46.1-.63.24l-.01-.01L3 9.03V15h16V2.59z"],"timeline-bar-chart":["M19 17H1c-.55 0-1 .45-1 1s.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1zM9 16h2c.55 0 1-.45 1-1V2c0-.55-.45-1-1-1H9c-.55 0-1 .45-1 1v13c0 .55.45 1 1 1zm6 0h2c.55 0 1-.45 1-1V6c0-.55-.45-1-1-1h-2c-.55 0-1 .45-1 1v9c0 .55.45 1 1 1zM3 16h2c.55 0 1-.45 1-1V9c0-.55-.45-1-1-1H3c-.55 0-1 .45-1 1v6c0 .55.45 1 1 1z"],"timeline-events":["M5 5c.6 0 1-.4 1-1V2c0-.5-.4-1-1-1s-1 .5-1 1v2c0 .6.4 1 1 1zm10 0c.6 0 1-.4 1-1V2c0-.5-.4-1-1-1s-1 .5-1 1v2c0 .6.4 1 1 1zm-9 9H4v2h2v-2zM17 3v1c0 1.1-.9 2-2 2s-2-.9-2-2V3H7v1c0 1.1-.9 2-2 2s-2-.9-2-2V3H2c-.5 0-1 .5-1 1v14c0 .5.5 1 1 1h16c.5 0 1-.5 1-1V4c0-.5-.5-1-1-1h-1zM7 17H3v-4h4v4zm0-5H3V8h4v4zm5 5H8v-4h4v4zm0-5H8V8h4v4zm5 5h-4v-4h4v4zm0-5h-4V8h4v4zm-6 2H9v2h2v-2zm5-5h-2v2h2V9z"],"timeline-line-chart":["M19 16H2v-1.59l5-5 3.29 3.29c.18.19.43.3.71.3s.53-.11.71-.29l7-7a1.003 1.003 0 00-1.42-1.42L11 10.59l-3.29-3.3C7.53 7.11 7.28 7 7 7s-.53.11-.71.29L2 11.59V3c0-.55-.45-1-1-1s-1 .45-1 1v14a.998.998 0 001 1h18c.55 0 1-.45 1-1s-.45-1-1-1z"],tint:["M9.86 2S3.98 9.18 3.98 12.17C3.99 15.4 6.78 18 9.96 18c3.18-.01 6.04-2.63 6.03-5.86C15.99 9.05 9.86 2 9.86 2z"],torch:["M6.97 19c0 .55.45 1 1 1h4c.55 0 1-.45 1-1v-2h-6v2zm-3-15l3 4v8h6V8l3-4h-12zm5 5c0-.55.45-1 1-1s1 .45 1 1v2c0 .55-.45 1-1 1s-1-.45-1-1V9zm6-9h-10c-.55 0-1 .45-1 1v2h12V1c0-.55-.45-1-1-1z"],tractor:["M4.5 11a4.5 4.5 0 110 9 4.5 4.5 0 010-9zm11.499 1a4 4 0 110 8 4 4 0 010-8zm-11.5 1.571a1.928 1.928 0 100 3.857 1.928 1.928 0 000-3.857zM16 14.667a1.333 1.333 0 100 2.666 1.333 1.333 0 000-2.666zM5.999 0C7.46 0 8.527.668 9 2l.851 4.256c1.433.096 2.82.217 4.147.362V2h2L16 6.862c.962.13 1.886.275 2.767.435.779.141 1.232.614 1.232 1.284L20 13a4.995 4.995 0 00-4-1.997A5.001 5.001 0 0011.099 15h-1.12a5.499 5.499 0 00-5.478-4.994 5.482 5.482 0 00-3.377 1.157H.004v-1.18L0 7.327c-.002-.597.37-1.18.999-1.302V1a1 1 0 011-1h4zm1 2H3v4h.75c1.386.027 2.749.073 4.079.139L6.999 2z"],train:["M16 18h-2l2 2H4l.12-.12L6 18H4c-1.1 0-2-.9-2-2V2c0-1.1 3.58-2 8-2s8 .9 8 2v14c0 1.1-.9 2-2 2zM5.5 15c.83 0 1.5-.67 1.5-1.5S6.33 12 5.5 12 4 12.67 4 13.5 4.67 15 5.5 15zM9 3H4v6h5V3zm7 0h-5v6h5V3zm-1.5 9c-.83 0-1.5.67-1.5 1.5s.67 1.5 1.5 1.5 1.5-.67 1.5-1.5-.67-1.5-1.5-1.5z"],translate:["M19.89 18.56l-4.99-10h-.01c-.17-.33-.5-.56-.89-.56s-.72.23-.89.56h-.01l-1.73 3.46-2.8-2.3 1.99-1.64C11.44 7.34 12 6.23 12 5V4h1c.55 0 1-.45 1-1s-.45-1-1-1H8V1c0-.55-.45-1-1-1S6 .45 6 1v1H1c-.55 0-1 .45-1 1s.45 1 1 1h9v1c0 .62-.28 1.18-.73 1.54L7 8.42 4.73 6.54C4.28 6.18 4 5.62 4 5H2c0 1.23.56 2.34 1.44 3.07l1.99 1.64-3.06 2.52.01.01c-.23.18-.38.45-.38.76 0 .55.45 1 1 1 .24 0 .45-.1.63-.24l.01.01L7 11l3.36 2.77.01-.01c.02.02.05.03.08.05.01 0 .01.01.02.02l-2.36 4.73h.01c-.07.13-.12.28-.12.44 0 .55.45 1 1 1 .39 0 .72-.23.89-.56h.01L11.12 17h5.76l1.22 2.45h.01c.17.32.5.55.89.55.55 0 1-.45 1-1 0-.16-.05-.31-.11-.44zM12.12 15L14 11.24 15.88 15h-3.76z"],trash:["M17 1h-5c0-.55-.45-1-1-1H9c-.55 0-1 .45-1 1H3c-.55 0-1 .45-1 1v1h16V2c0-.55-.45-1-1-1zm.5 3h-15c-.28 0-.5.22-.5.5s.22.5.5.5H3v14c0 .55.45 1 1 1h12c.55 0 1-.45 1-1V5h.5c.28 0 .5-.22.5-.5s-.22-.5-.5-.5zM7 16c0 .55-.45 1-1 1s-1-.45-1-1V8c0-.55.45-1 1-1s1 .45 1 1v8zm4 0c0 .55-.45 1-1 1s-1-.45-1-1V8c0-.55.45-1 1-1s1 .45 1 1v8zm4 0c0 .55-.45 1-1 1s-1-.45-1-1V8c0-.55.45-1 1-1s1 .45 1 1v8z"],tree:["M11 15.542V20H9v-4.458L2 17l4.5-5.625L4 12l3.655-5.483L6 7l4-7 4 7-1.655-.483L16 12l-2.5-.625L18 17l-7-1.458z"],"trending-down":["M19 10c-.55 0-1 .45-1 1v1.37l-6.25-7.03-.01.01A.971.971 0 0011 5c-.23 0-.42.09-.59.21l-.01-.01-3.43 2.58-5.42-3.61-.01.01A.969.969 0 001 4c-.55 0-1 .45-1 1 0 .35.19.64.46.82l-.01.01 6 4 .01-.02c.15.11.33.19.54.19.23 0 .42-.09.59-.21l.01.01 3.26-2.45L16.77 14H15c-.55 0-1 .45-1 1s.45 1 1 1h4c.55 0 1-.45 1-1v-4c0-.55-.45-1-1-1z"],"trending-up":["M19 4h-4c-.55 0-1 .45-1 1s.45 1 1 1h1.77l-5.91 6.65L7.6 10.2l-.01.01C7.42 10.09 7.23 10 7 10c-.21 0-.39.08-.54.18l-.01-.02-6 4 .01.02c-.27.18-.46.47-.46.82 0 .55.45 1 1 1 .21 0 .39-.08.54-.18l.01.02 5.41-3.61 3.43 2.58.01-.01c.18.11.37.2.6.2.3 0 .56-.14.74-.34l.01.01L18 7.63V9c0 .55.45 1 1 1s1-.45 1-1V5c0-.55-.45-1-1-1z"],truck:["M16 0a1 1 0 011 1v11a1 1 0 011 1v3h.5a.5.5 0 01.5.5v1a.5.5 0 01-.5.5H17v1a1 1 0 01-1 1h-1a1 1 0 01-1-1v-1H6v1a1 1 0 01-1 1H4a1 1 0 01-1-1v-1H1.5a.5.5 0 01-.5-.5v-1a.5.5 0 01.5-.5H2v-3a1 1 0 011-1V1a1 1 0 112 0v3a2 2 0 012-2h6a2 2 0 012 2V1a1 1 0 011-1zm-4 10H8a1 1 0 00-1 1v4a1 1 0 001 1h4a1 1 0 001-1v-4a1 1 0 00-1-1zm-7 4H4a1 1 0 000 2h1a1 1 0 000-2zm11 0h-1a1 1 0 000 2h1a1 1 0 000-2zm-4.5 0a.5.5 0 110 1h-3l-.09-.008A.5.5 0 018.5 14zm0-1.5a.5.5 0 110 1h-3l-.09-.008a.5.5 0 01.09-.992zm0-1.5a.5.5 0 110 1h-3l-.09-.008A.5.5 0 018.5 11zM14 5H6v3h8V5z"],"two-columns":["M5 0H1C.45 0 0 .45 0 1v18c0 .55.45 1 1 1h4c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm14.71 9.29l-3-3A1.003 1.003 0 0015 7v6a1.003 1.003 0 001.71.71l3-3c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71zM12 0H8c-.55 0-1 .45-1 1v18c0 .55.45 1 1 1h4c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1z"],unarchive:["M16.434 0a1 1 0 01.857.486L20 5v14a1 1 0 01-1 1H1a1 1 0 01-1-1V5L2.709.486A1 1 0 013.566 0h12.868zM10 8c-.28 0-.53.11-.71.29l-3 3-.084.096A1.003 1.003 0 007.71 12.71L9 11.41v4.58l.007.116c.058.496.482.884.993.884.55 0 1-.45 1-1v-4.58l1.29 1.29.081.073c.171.139.389.227.629.227a1.003 1.003 0 00.71-1.71l-3-3-.096-.084A1.002 1.002 0 0010 8zm6-6H4L2 5.002h16L16 2z"],underline:["M10 17c3.3 0 6-2.7 6-6V3.5c0-.8-.7-1.5-1.5-1.5S13 2.7 13 3.5V11c0 1.7-1.3 3-3 3s-3-1.3-3-3V3.5C7 2.7 6.3 2 5.5 2S4 2.7 4 3.5V11c0 3.3 2.7 6 6 6zM16.5 19h-13c-.3 0-.5.2-.5.5s.2.5.5.5h13c.3 0 .5-.2.5-.5s-.2-.5-.5-.5z"],undo:["M5 14c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm9-9H3.41L5.7 2.71c.19-.18.3-.43.3-.71a1.003 1.003 0 00-1.71-.71l-4 4C.11 5.47 0 5.72 0 6c0 .28.11.53.29.71l4 4a1.003 1.003 0 001.42-1.42L3.41 7H14c2.21 0 4 1.79 4 4s-1.79 4-4 4H9v2h5c3.31 0 6-2.69 6-6s-2.69-6-6-6z"],"ungroup-objects":["M4.5 6C2.01 6 0 8.01 0 10.5S2.01 15 4.5 15 9 12.99 9 10.5 6.99 6 4.5 6zm11 0C13.01 6 11 8.01 11 10.5s2.01 4.5 4.5 4.5 4.5-2.01 4.5-4.5S17.99 6 15.5 6z"],"unknown-vehicle":["M13 11.988v-4H4v-1l1-3h6V2.003a35.867 35.867 0 00-1-.015c-3.593 0-5.332.488-5.332.488-.67.188-1.424.864-1.674 1.503l-.004.009H3l-1 3H1a1 1 0 100 2h.333l-.28.84-.053.16v7.5a1.5 1.5 0 103 0v-.5h12v.5a1.5 1.5 0 103 0v-4.5h-5a1 1 0 01-1-1zm-8.5 1a1.5 1.5 0 110-3 1.5 1.5 0 010 3zM19.83 2.782a2.392 2.392 0 00-.592-.853c-.276-.264-.64-.485-1.09-.663C17.695 1.09 17.132 1 16.457 1c-.523 0-.996.084-1.418.253a3.157 3.157 0 00-1.084.703c-.299.3-.532.656-.698 1.065-.166.41-.254.861-.264 1.353h2.096c0-.246.028-.476.085-.69.057-.214.145-.4.264-.56.119-.16.27-.287.456-.383.185-.095.406-.143.663-.143.38 0 .677.1.89.3.215.2.321.51.321.93.01.245-.035.45-.135.614-.1.164-.23.314-.392.45a8.598 8.598 0 01-.527.41 3.53 3.53 0 00-.542.485c-.171.187-.32.412-.45.676-.127.265-.206.592-.234.984v.614h1.924v-.519c.038-.273.13-.5.278-.683.147-.182.316-.343.506-.484a13.5 13.5 0 01.606-.424c.214-.14.408-.312.584-.512s.323-.442.442-.724.178-.642.178-1.079c0-.264-.059-.548-.178-.854zm-4.54 6.099v2.103h2.237V8.881H15.29z"],unlock:["M14 1c-2.21 0-4 1.79-4 4v4H2c-.55 0-1 .45-1 1v8c0 .55.45 1 1 1h12c.55 0 1-.45 1-1v-8c0-.55-.45-1-1-1h-2V5c0-1.1.9-2 2-2s2 .9 2 2v2c0 .55.45 1 1 1s1-.45 1-1V5c0-2.21-1.79-4-4-4z"],unpin:["M11.77 1.16c-.81.81-.74 2.28.02 3.76L6.1 8.71c-2.17-1.46-4.12-2-4.94-1.18l4.95 4.95-2.12 3.54 3.54-2.12 4.95 4.95c.82-.82.27-2.77-1.19-4.94l3.8-5.69c1.47.76 2.94.84 3.76.02l-7.08-7.08z"],unresolve:["M11.47 12.46c.16-.36.29-.74.38-1.14 0-.02.01-.04.01-.06.09-.4.14-.82.14-1.26 0-.44-.05-.86-.14-1.27 0-.02-.01-.04-.01-.06-.09-.4-.22-.78-.38-1.14-.01-.02-.02-.03-.02-.05a5.94 5.94 0 00-.61-1.03c0-.01-.01-.01-.01-.02a6.308 6.308 0 00-2.1-1.77c-.19-.1-.39-.18-.59-.26-.03-.01-.06-.02-.1-.03-.17-.07-.34-.12-.52-.17-.05-.01-.1-.03-.15-.04a4.34 4.34 0 00-.52-.09c-.05-.01-.11-.02-.17-.03C6.46 4.02 6.23 4 6 4c-3.31 0-6 2.69-6 6s2.69 6 6 6c.23 0 .46-.02.68-.04l.17-.03c.17-.02.34-.06.51-.09.05-.01.1-.03.15-.04.18-.05.36-.1.53-.17l.09-.03a5.973 5.973 0 002.68-2.04c0-.01.01-.01.01-.02.24-.32.44-.66.61-1.03.02-.01.03-.03.04-.05zM14 4c-.99 0-1.91.24-2.73.66a7.51 7.51 0 010 10.68c.82.42 1.74.66 2.73.66 3.31 0 6-2.69 6-6s-2.69-6-6-6z"],updated:["M10 0C6.71 0 3.82 1.6 2 4.05V2c0-.55-.45-1-1-1s-1 .45-1 1v4c0 .55.45 1 1 1h4c.55 0 1-.45 1-1s-.45-1-1-1H3.76C5.22 3.17 7.47 2 10 2c4.42 0 8 3.58 8 8s-3.58 8-8 8-8-3.58-8-8c0-.55-.45-1-1-1s-1 .45-1 1c0 5.52 4.48 10 10 10s10-4.48 10-10S15.52 0 10 0zm4 7c-.28 0-.53.11-.71.29L9 11.58 6.71 9.29a1.003 1.003 0 00-1.42 1.42l3 3c.18.18.43.29.71.29.28 0 .53-.11.71-.29l5-5A1.003 1.003 0 0014 7z"],upload:["M10 0C4.48 0 0 4.48 0 10s4.48 10 10 10 10-4.48 10-10S15.52 0 10 0zm4 10c-.28 0-.53-.11-.71-.29L11 7.41V15c0 .55-.45 1-1 1s-1-.45-1-1V7.41l-2.29 2.3a1.003 1.003 0 01-1.42-1.42l4-4c.18-.18.43-.29.71-.29s.53.11.71.29l4 4A1.003 1.003 0 0114 10z"],user:["M10 0C4.48 0 0 4.48 0 10c0 .33.02.65.05.97.01.12.03.23.05.35.03.2.05.4.09.59.03.14.06.28.1.42l.12.48c.05.16.1.31.15.46.05.13.09.27.15.4.06.16.13.32.21.48.05.11.1.22.16.33.09.17.17.34.27.5.05.09.1.17.15.25.11.18.22.35.34.52.04.06.08.11.12.17 1.19 1.62 2.85 2.86 4.78 3.53l.09.03c.46.15.93.27 1.42.36.08.01.17.03.25.04.49.07.99.12 1.5.12s1.01-.05 1.5-.12c.08-.01.17-.02.25-.04.49-.09.96-.21 1.42-.36l.09-.03c1.93-.67 3.59-1.91 4.78-3.53.04-.05.08-.1.12-.16.12-.17.23-.35.34-.53.05-.08.1-.16.15-.25.1-.17.19-.34.27-.51.05-.11.1-.21.15-.32.07-.16.14-.32.21-.49.05-.13.1-.26.14-.39.05-.15.11-.31.15-.46.05-.16.08-.32.12-.48.03-.14.07-.28.1-.42.04-.19.06-.39.09-.59.02-.12.04-.23.05-.35.05-.32.07-.64.07-.97 0-5.52-4.48-10-10-10zm0 18a7.94 7.94 0 01-6.15-2.89c.84-.44 1.86-.82 2.67-1.19 1.45-.65 1.3-1.05 1.35-1.59.01-.07.01-.14.01-.21-.51-.45-.93-1.08-1.2-1.8l-.01-.01c0-.01-.01-.02-.01-.03a4.42 4.42 0 01-.15-.48c-.33-.07-.53-.44-.61-.79-.08-.14-.23-.48-.2-.87.05-.51.26-.74.49-.83v-.08c0-.63.06-1.55.17-2.15.02-.17.06-.33.11-.5.21-.73.66-1.4 1.26-1.86.62-.47 1.5-.72 2.28-.72.78 0 1.65.25 2.27.73.6.46 1.05 1.12 1.26 1.86.05.16.08.33.11.5.11.6.17 1.51.17 2.15v.09c.22.1.42.33.46.82.04.39-.12.73-.2.87-.07.34-.27.71-.6.78-.04.16-.09.33-.15.48 0 .01-.02.05-.02.05-.26.71-.67 1.33-1.17 1.78 0 .08.01.16.01.23.05.54-.15.94 1.31 1.59.81.36 1.84.74 2.68 1.19A7.958 7.958 0 0110 18z"],variable:["M4.93 3.79a9.1 9.1 0 012.2-2.27L7.29 1c-1.38.59-2.57 1.33-3.55 2.22C2.46 4.39 1.49 5.72.83 7.23.28 8.51 0 9.81 0 11.12c0 2.28.83 4.57 2.49 6.86l.16-.55c-.49-1.23-.73-2.38-.73-3.44 0-1.67.28-3.46.84-5.36.55-1.9 1.28-3.51 2.17-4.84zm9.38 8.39l-.33-.2c-.37.54-.65.87-.82 1a.74.74 0 01-.42.12c-.19 0-.38-.12-.57-.37-.31-.42-.73-1.59-1.26-3.5.47-.85.86-1.41 1.19-1.67.23-.19.48-.29.74-.29.1 0 .28.04.53.11.26.07.48.11.68.11.27 0 .5-.1.68-.29.18-.19.27-.44.27-.75 0-.33-.09-.58-.27-.77-.18-.19-.44-.29-.78-.29-.3 0-.59.07-.86.22s-.61.47-1.02.97c-.31.37-.77 1.02-1.37 1.94a9.683 9.683 0 00-1.24-3.14l-3.24.59-.06.36c.24-.05.44-.07.61-.07.32 0 .59.14.8.43.33.45.8 1.8 1.39 4.07-.47.64-.78 1.06-.96 1.26-.28.32-.52.53-.7.62-.14.08-.3.11-.48.11-.14 0-.36-.08-.67-.23-.21-.1-.4-.15-.57-.15-.31 0-.57.11-.78.32s-.31.48-.31.8c0 .31.09.55.28.75.19.19.44.29.76.29.31 0 .6-.07.87-.2s.61-.42 1.02-.86c.41-.44.98-1.13 1.7-2.08.28.9.52 1.56.72 1.97.2.41.44.71.7.89.26.18.59.27.99.27.38 0 .77-.14 1.17-.43.54-.36 1.07-1 1.61-1.91zM17.51 1l-.15.54c.49 1.24.73 2.39.73 3.45 0 1.43-.21 2.96-.63 4.6-.33 1.26-.75 2.45-1.27 3.55-.52 1.11-1.02 1.97-1.51 2.6-.49.62-1.09 1.2-1.8 1.72l-.17.53c1.38-.59 2.57-1.34 3.55-2.23 1.29-1.17 2.26-2.5 2.91-4 .55-1.28.83-2.59.83-3.91 0-2.27-.83-4.56-2.49-6.85z"],"vertical-bar-chart-asc":["M8 7H7c-.55 0-1 .45-1 1v11c0 .55.45 1 1 1h1c.55 0 1-.45 1-1V8c0-.55-.45-1-1-1zM3 9H2c-.55 0-1 .45-1 1v9c0 .55.45 1 1 1h1c.55 0 1-.45 1-1v-9c0-.55-.45-1-1-1zm10-5h-1c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1h1c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1zm5-4h-1c-.55 0-1 .45-1 1v18c0 .55.45 1 1 1h1c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1z"],"vertical-bar-chart-desc":["M3 0H2c-.55 0-1 .45-1 1v18c0 .55.45 1 1 1h1c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm5 4H7c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1h1c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1zm5 3h-1c-.55 0-1 .45-1 1v11c0 .55.45 1 1 1h1c.55 0 1-.45 1-1V8c0-.55-.45-1-1-1zm5 2h-1c-.55 0-1 .45-1 1v9c0 .55.45 1 1 1h1c.55 0 1-.45 1-1v-9c0-.55-.45-1-1-1z"],"vertical-distribution":["M1 2h18c.55 0 1-.45 1-1s-.45-1-1-1H1C.45 0 0 .45 0 1s.45 1 1 1zm2 5c-.55 0-1 .45-1 1v4c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V8c0-.55-.45-1-1-1H3zm16 11H1c-.55 0-1 .45-1 1s.45 1 1 1h18c.55 0 1-.45 1-1s-.45-1-1-1z"],video:["M19 2H1c-.55 0-1 .45-1 1v14c0 .55.45 1 1 1h18c.55 0 1-.45 1-1V3c0-.55-.45-1-1-1zM7 14V6l6 4-6 4z"],virus:["M15.249 13.835l1.251 1.251.354-.354.087-.077a1 1 0 011.327 1.491l-2.122 2.122-.087.077a1 1 0 01-1.327-1.491l.354-.354-1.251-1.251A6.466 6.466 0 0111 16.424L10.999 18h.501a1 1 0 01.117 1.993L11.5 20h-3a1 1 0 01-.117-1.993L8.5 18h.499v-1.577a6.46 6.46 0 01-2.538-.97L5.414 16.5l.354.354a1 1 0 01-1.327 1.491l-.087-.077-2.122-2.122a1 1 0 011.327-1.491l.087.077.354.354.97-.97a6.472 6.472 0 01-1.384-3.057l-.025.002L2 11.06v.44a1 1 0 01-1.993.117L0 11.5v-3a1 1 0 011.993-.117L2 8.5v.56h1.567A6.471 6.471 0 014.97 5.883l-.971-.969-.353.354-.087.077a1 1 0 01-1.327-1.491l2.122-2.122.087-.077a1 1 0 011.327 1.491l-.354.353 1.047 1.048A6.46 6.46 0 019 3.577L9 2h-.5A1 1 0 018.383.007L8.5 0h3a1 1 0 01.117 1.993L11.5 2H11v1.577a6.466 6.466 0 012.838 1.176l.04-.046L15.086 3.5l-.353-.353a1 1 0 011.327-1.491l.087.077 2.122 2.122a1 1 0 01-1.327 1.491l-.087-.077-.354-.354-1.207 1.207-.046.041a6.467 6.467 0 011.16 2.733H18V8.5a1 1 0 011.993-.117L20 8.5v3a1 1 0 01-1.993.117L18 11.5v-.605h-1.561a6.466 6.466 0 01-1.19 2.94zM12.5 11a1.5 1.5 0 100 3 1.5 1.5 0 000-3zM8 6a2 2 0 100 4 2 2 0 000-4z"],"volume-down":["M15.92 3.93l-1.6 1.18A7.948 7.948 0 0116 10c0 1.84-.63 3.54-1.68 4.89l1.6 1.18A9.878 9.878 0 0018 10c0-2.29-.78-4.39-2.08-6.07zM11 3c-.28 0-.53.11-.71.29L7.59 6H3c-.55 0-1 .45-1 1v6c0 .55.45 1 1 1h4.59l2.71 2.71c.17.18.42.29.7.29.55 0 1-.45 1-1V4c0-.55-.45-1-1-1z"],"volume-off":["M14 3c-.28 0-.53.11-.71.29L10.59 6H6c-.55 0-1 .45-1 1v6c0 .55.45 1 1 1h4.59l2.71 2.71c.17.18.42.29.7.29.55 0 1-.45 1-1V4c0-.55-.45-1-1-1z"],"volume-up":["M9 3.43c-.28 0-.53.11-.71.29l-2.7 2.71H1c-.55 0-1 .45-1 1v6c0 .55.45 1 1 1h4.59l2.71 2.71a1.003 1.003 0 001.71-.71v-12c-.01-.55-.46-1-1.01-1zm8.31-1.56l-1.62 1.2C17.14 5.16 18 7.69 18 10.43s-.86 5.27-2.31 7.37l1.62 1.2C19 16.57 20 13.62 20 10.43c0-3.18-1-6.13-2.69-8.56zm-3.39 2.49l-1.6 1.18A7.948 7.948 0 0114 10.43c0 1.84-.63 3.54-1.68 4.89l1.6 1.18A9.94 9.94 0 0016 10.43c0-2.28-.78-4.38-2.08-6.07z"],walk:["M16 10h-2c-.23 0-.42-.09-.59-.21l-.01.01-1.69-1.27-.63 3.14 2.62 2.62c.19.18.3.43.3.71v4c0 .55-.45 1-1 1s-1-.45-1-1v-3.59L9.39 12.8l-2.45 6.55h-.01c-.14.38-.5.65-.93.65-.55 0-1-.45-1-1 0-.12.03-.24.07-.35h-.01L9.43 7h-2.9l-1.7 2.55-.01-.01c-.18.27-.47.46-.82.46-.55 0-1-.45-1-1 0-.21.08-.39.18-.54l-.01-.01 2-3 .02.01C5.36 5.19 5.65 5 6 5h4.18l.36-.96c-.33-.43-.54-.96-.54-1.54a2.5 2.5 0 015 0A2.5 2.5 0 0112.5 5c-.06 0-.12-.01-.18-.02l-.44 1.18L14.33 8H16c.55 0 1 .45 1 1s-.45 1-1 1z"],"warning-sign":["M19.86 17.52l.01-.01-9-16-.01.01C10.69 1.21 10.37 1 10 1s-.69.21-.86.52l-.01-.01-9 16 .01.01c-.08.14-.14.3-.14.48 0 .55.45 1 1 1h18c.55 0 1-.45 1-1 0-.18-.06-.34-.14-.48zM11 17H9v-2h2v2zm0-3H9V6h2v8z"],"waterfall-chart":["M13 7h2c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1h-2c-.55 0-1 .45-1 1v1c0 .55.45 1 1 1zm-9 8h1c.55 0 1-.45 1-1v-4c0-.55-.45-1-1-1H4c-.55 0-1 .45-1 1v4c0 .55.45 1 1 1zm4-6h2c.55 0 1-.45 1-1s-.45-1-1-1H8c-.55 0-1 .45-1 1s.45 1 1 1zm11-5h-1c-.55 0-1 .45-1 1v9c0 .55.45 1 1 1h1c.55 0 1-.45 1-1V5c0-.55-.45-1-1-1zm0 12H2V3c0-.55-.45-1-1-1s-1 .45-1 1v14a.998.998 0 001 1h18c.55 0 1-.45 1-1s-.45-1-1-1z"],widget:["M18 4c1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2 .9 2 2 2zM2 16c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm15-1h2V5h-2v10zM3 5H1v10h2V5zM2 0C.9 0 0 .9 0 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm3 3h10V1H5v2zm13 13c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zM5 19h10v-2H5v2z"],"widget-button":["M1 4h18c.55 0 1 .45 1 1v10c0 .55-.45 1-1 1H1c-.55 0-1-.45-1-1V5c0-.55.45-1 1-1zm1 2v8h16V6H2zm4 5c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zm4 0c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zm4 0c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1z"],"widget-footer":["M17 0H3c-.55 0-1 .45-1 1v18c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 18H4v-4h12v4zm0-5H4V2h12v11z"],"widget-header":["M17 0H3c-.55 0-1 .45-1 1v18c0 .55.45 1 1 1h14c.55 0 1-.45 1-1V1c0-.55-.45-1-1-1zm-1 18H4V7h12v11zm0-12H4V2h12v4z"],wrench:["M19.8 4.44L16.13 8.1l-3.55-.71-.71-3.53L15.54.21c-2.01-.53-4.23-.03-5.8 1.53-1.86 1.85-2.23 4.6-1.14 6.83L.59 16.59C.22 16.95 0 17.45 0 18a2 2 0 002 2c.55 0 1.05-.22 1.41-.59l8.03-8.04c2.23 1.05 4.97.67 6.82-1.16 1.57-1.56 2.07-3.77 1.54-5.77z"],"zoom-in":["M19.56 17.44l-4.94-4.94A8.004 8.004 0 0016 8c0-4.42-3.58-8-8-8S0 3.58 0 8s3.58 8 8 8c1.67 0 3.21-.51 4.5-1.38l4.94 4.94a1.498 1.498 0 102.12-2.12zM8 14c-3.31 0-6-2.69-6-6s2.69-6 6-6 6 2.69 6 6-2.69 6-6 6zm3-7H9V5c0-.55-.45-1-1-1s-1 .45-1 1v2H5c-.55 0-1 .45-1 1s.45 1 1 1h2v2c0 .55.45 1 1 1s1-.45 1-1V9h2c.55 0 1-.45 1-1s-.45-1-1-1z"],"zoom-out":["M11 7H5c-.55 0-1 .45-1 1s.45 1 1 1h6c.55 0 1-.45 1-1s-.45-1-1-1zm8.56 10.44l-4.94-4.94A8.004 8.004 0 0016 8c0-4.42-3.58-8-8-8S0 3.58 0 8s3.58 8 8 8c1.67 0 3.21-.51 4.5-1.38l4.94 4.94a1.498 1.498 0 102.12-2.12zM8 14c-3.31 0-6-2.69-6-6s2.69-6 6-6 6 2.69 6 6-2.69 6-6 6z"],"zoom-to-fit":["M1 7c.55 0 1-.45 1-1V2h4c.55 0 1-.45 1-1s-.45-1-1-1H1C.45 0 0 .45 0 1v5c0 .55.45 1 1 1zm5 1a1.003 1.003 0 00-1.71-.71l-2 2c-.18.18-.29.43-.29.71 0 .28.11.53.29.71l2 2a1.003 1.003 0 001.42-1.42L4.41 10 5.7 8.71c.19-.18.3-.43.3-.71zm2-2c.28 0 .53-.11.71-.29L10 4.41l1.29 1.29c.18.19.43.3.71.3a1.003 1.003 0 00.71-1.71l-2-2C10.53 2.11 10.28 2 10 2s-.53.11-.71.29l-2 2A1.003 1.003 0 008 6zM6 18H2v-4c0-.55-.45-1-1-1s-1 .45-1 1v5c0 .55.45 1 1 1h5c.55 0 1-.45 1-1s-.45-1-1-1zm8-6a1.003 1.003 0 001.71.71l2-2c.18-.18.29-.43.29-.71 0-.28-.11-.53-.29-.71l-2-2a1.003 1.003 0 00-1.42 1.42l1.3 1.29-1.29 1.29c-.19.18-.3.43-.3.71zm5-12h-5c-.55 0-1 .45-1 1s.45 1 1 1h4v4c0 .55.45 1 1 1s1-.45 1-1V1c0-.55-.45-1-1-1zm-7 14c-.28 0-.53.11-.71.29L10 15.59 8.71 14.3A.965.965 0 008 14a1.003 1.003 0 00-.71 1.71l2 2c.18.18.43.29.71.29s.53-.11.71-.29l2-2A1.003 1.003 0 0012 14zm7-1c-.55 0-1 .45-1 1v4h-4c-.55 0-1 .45-1 1s.45 1 1 1h5c.55 0 1-.45 1-1v-5c0-.55-.45-1-1-1z"]}},function(e,t,n){"use strict";n.r(t),n.d(t,"__extends",(function(){return i})),n.d(t,"__assign",(function(){return o})),n.d(t,"__rest",(function(){return a})),n.d(t,"__decorate",(function(){return s})),n.d(t,"__param",(function(){return l})),n.d(t,"__metadata",(function(){return u})),n.d(t,"__awaiter",(function(){return c})),n.d(t,"__generator",(function(){return d})),n.d(t,"__createBinding",(function(){return h})),n.d(t,"__exportStar",(function(){return f})),n.d(t,"__values",(function(){return p})),n.d(t,"__read",(function(){return g})),n.d(t,"__spread",(function(){return v})),n.d(t,"__spreadArrays",(function(){return m})),n.d(t,"__await",(function(){return y})),n.d(t,"__asyncGenerator",(function(){return b})),n.d(t,"__asyncDelegator",(function(){return C})),n.d(t,"__asyncValues",(function(){return A})),n.d(t,"__makeTemplateObject",(function(){return _})),n.d(t,"__importStar",(function(){return I})),n.d(t,"__importDefault",(function(){return x})),n.d(t,"__classPrivateFieldGet",(function(){return w})),n.d(t,"__classPrivateFieldSet",(function(){return M}));
/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
var r=function(e,t){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(e,t)};function i(e,t){function n(){this.constructor=e}r(e,t),e.prototype=null===t?Object.create(t):(n.prototype=t.prototype,new n)}var o=function(){return(o=Object.assign||function(e){for(var t,n=1,r=arguments.length;n<r;n++)for(var i in t=arguments[n])Object.prototype.hasOwnProperty.call(t,i)&&(e[i]=t[i]);return e}).apply(this,arguments)};function a(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(null!=e&&"function"==typeof Object.getOwnPropertySymbols){var i=0;for(r=Object.getOwnPropertySymbols(e);i<r.length;i++)t.indexOf(r[i])<0&&Object.prototype.propertyIsEnumerable.call(e,r[i])&&(n[r[i]]=e[r[i]])}return n}function s(e,t,n,r){var i,o=arguments.length,a=o<3?t:null===r?r=Object.getOwnPropertyDescriptor(t,n):r;if("object"==typeof Reflect&&"function"==typeof Reflect.decorate)a=Reflect.decorate(e,t,n,r);else for(var s=e.length-1;s>=0;s--)(i=e[s])&&(a=(o<3?i(a):o>3?i(t,n,a):i(t,n))||a);return o>3&&a&&Object.defineProperty(t,n,a),a}function l(e,t){return function(n,r){t(n,r,e)}}function u(e,t){if("object"==typeof Reflect&&"function"==typeof Reflect.metadata)return Reflect.metadata(e,t)}function c(e,t,n,r){return new(n||(n=Promise))((function(i,o){function a(e){try{l(r.next(e))}catch(e){o(e)}}function s(e){try{l(r.throw(e))}catch(e){o(e)}}function l(e){var t;e.done?i(e.value):(t=e.value,t instanceof n?t:new n((function(e){e(t)}))).then(a,s)}l((r=r.apply(e,t||[])).next())}))}function d(e,t){var n,r,i,o,a={label:0,sent:function(){if(1&i[0])throw i[1];return i[1]},trys:[],ops:[]};return o={next:s(0),throw:s(1),return:s(2)},"function"==typeof Symbol&&(o[Symbol.iterator]=function(){return this}),o;function s(o){return function(s){return function(o){if(n)throw new TypeError("Generator is already executing.");for(;a;)try{if(n=1,r&&(i=2&o[0]?r.return:o[0]?r.throw||((i=r.return)&&i.call(r),0):r.next)&&!(i=i.call(r,o[1])).done)return i;switch(r=0,i&&(o=[2&o[0],i.value]),o[0]){case 0:case 1:i=o;break;case 4:return a.label++,{value:o[1],done:!1};case 5:a.label++,r=o[1],o=[0];continue;case 7:o=a.ops.pop(),a.trys.pop();continue;default:if(!(i=a.trys,(i=i.length>0&&i[i.length-1])||6!==o[0]&&2!==o[0])){a=0;continue}if(3===o[0]&&(!i||o[1]>i[0]&&o[1]<i[3])){a.label=o[1];break}if(6===o[0]&&a.label<i[1]){a.label=i[1],i=o;break}if(i&&a.label<i[2]){a.label=i[2],a.ops.push(o);break}i[2]&&a.ops.pop(),a.trys.pop();continue}o=t.call(e,a)}catch(e){o=[6,e],r=0}finally{n=i=0}if(5&o[0])throw o[1];return{value:o[0]?o[1]:void 0,done:!0}}([o,s])}}}function h(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}function f(e,t){for(var n in e)"default"===n||t.hasOwnProperty(n)||(t[n]=e[n])}function p(e){var t="function"==typeof Symbol&&Symbol.iterator,n=t&&e[t],r=0;if(n)return n.call(e);if(e&&"number"==typeof e.length)return{next:function(){return e&&r>=e.length&&(e=void 0),{value:e&&e[r++],done:!e}}};throw new TypeError(t?"Object is not iterable.":"Symbol.iterator is not defined.")}function g(e,t){var n="function"==typeof Symbol&&e[Symbol.iterator];if(!n)return e;var r,i,o=n.call(e),a=[];try{for(;(void 0===t||t-- >0)&&!(r=o.next()).done;)a.push(r.value)}catch(e){i={error:e}}finally{try{r&&!r.done&&(n=o.return)&&n.call(o)}finally{if(i)throw i.error}}return a}function v(){for(var e=[],t=0;t<arguments.length;t++)e=e.concat(g(arguments[t]));return e}function m(){for(var e=0,t=0,n=arguments.length;t<n;t++)e+=arguments[t].length;var r=Array(e),i=0;for(t=0;t<n;t++)for(var o=arguments[t],a=0,s=o.length;a<s;a++,i++)r[i]=o[a];return r}function y(e){return this instanceof y?(this.v=e,this):new y(e)}function b(e,t,n){if(!Symbol.asyncIterator)throw new TypeError("Symbol.asyncIterator is not defined.");var r,i=n.apply(e,t||[]),o=[];return r={},a("next"),a("throw"),a("return"),r[Symbol.asyncIterator]=function(){return this},r;function a(e){i[e]&&(r[e]=function(t){return new Promise((function(n,r){o.push([e,t,n,r])>1||s(e,t)}))})}function s(e,t){try{(n=i[e](t)).value instanceof y?Promise.resolve(n.value.v).then(l,u):c(o[0][2],n)}catch(e){c(o[0][3],e)}var n}function l(e){s("next",e)}function u(e){s("throw",e)}function c(e,t){e(t),o.shift(),o.length&&s(o[0][0],o[0][1])}}function C(e){var t,n;return t={},r("next"),r("throw",(function(e){throw e})),r("return"),t[Symbol.iterator]=function(){return this},t;function r(r,i){t[r]=e[r]?function(t){return(n=!n)?{value:y(e[r](t)),done:"return"===r}:i?i(t):t}:i}}function A(e){if(!Symbol.asyncIterator)throw new TypeError("Symbol.asyncIterator is not defined.");var t,n=e[Symbol.asyncIterator];return n?n.call(e):(e=p(e),t={},r("next"),r("throw"),r("return"),t[Symbol.asyncIterator]=function(){return this},t);function r(n){t[n]=e[n]&&function(t){return new Promise((function(r,i){(function(e,t,n,r){Promise.resolve(r).then((function(t){e({value:t,done:n})}),t)})(r,i,(t=e[n](t)).done,t.value)}))}}}function _(e,t){return Object.defineProperty?Object.defineProperty(e,"raw",{value:t}):e.raw=t,e}function I(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)Object.hasOwnProperty.call(e,n)&&(t[n]=e[n]);return t.default=e,t}function x(e){return e&&e.__esModule?e:{default:e}}function w(e,t){if(!t.has(e))throw new TypeError("attempted to get private field on non-instance");return t.get(e)}function M(e,t,n){if(!t.has(e))throw new TypeError("attempted to set private field on non-instance");return t.set(e,n),n}},function(e,t,n){"use strict";var r=n(1489);e.exports=Function.prototype.bind||r},function(e,t,n){"use strict";var r=n(507),i=n(884),o=i("%Function.prototype.apply%"),a=i("%Function.prototype.call%"),s=i("%Reflect.apply%",!0)||r.call(a,o),l=i("%Object.getOwnPropertyDescriptor%",!0),u=i("%Object.defineProperty%",!0),c=i("%Math.max%");if(u)try{u({},"a",{value:1})}catch(e){u=null}e.exports=function(e){var t=s(r,a,arguments);if(l&&u){var n=l(t,"length");n.configurable&&u(t,"length",{value:1+c(0,e.length-(arguments.length-1))})}return t};var d=function(){return s(r,o,arguments)};u?u(e.exports,"apply",{value:d}):e.exports.apply=d},function(e,t,n){"use strict";var r=n(1524),i=n(510),o=n(241),a=n(892),s=n(1526);function l(e,t,n){var r=this._refs[n];if("string"==typeof r){if(!this._refs[r])return l.call(this,e,t,r);r=this._refs[r]}if((r=r||this._schemas[n])instanceof a)return p(r.schema,this._opts.inlineRefs)?r.schema:r.validate||this._compile(r);var i,o,s,c=u.call(this,t,n);return c&&(i=c.schema,t=c.root,s=c.baseId),i instanceof a?o=i.validate||e.call(this,i.schema,t,void 0,s):void 0!==i&&(o=p(i,this._opts.inlineRefs)?i:e.call(this,i,t,void 0,s)),o}function u(e,t){var n=r.parse(t),i=v(n),o=g(this._getId(e.schema));if(0===Object.keys(e.schema).length||i!==o){var s=y(i),l=this._refs[s];if("string"==typeof l)return c.call(this,e,l,n);if(l instanceof a)l.validate||this._compile(l),e=l;else{if(!((l=this._schemas[s])instanceof a))return;if(l.validate||this._compile(l),s==y(t))return{schema:l,root:e,baseId:o};e=l}if(!e.schema)return;o=g(this._getId(e.schema))}return h.call(this,n,o,e.schema,e)}function c(e,t,n){var r=u.call(this,e,t);if(r){var i=r.schema,o=r.baseId;e=r.root;var a=this._getId(i);return a&&(o=b(o,a)),h.call(this,n,o,i,e)}}e.exports=l,l.normalizeId=y,l.fullPath=g,l.url=b,l.ids=function(e){var t=y(this._getId(e)),n={"":t},a={"":g(t,!1)},l={},u=this;return s(e,{allKeys:!0},(function(e,t,s,c,d,h,f){if(""!==t){var p=u._getId(e),g=n[c],v=a[c]+"/"+d;if(void 0!==f&&(v+="/"+("number"==typeof f?f:o.escapeFragment(f))),"string"==typeof p){p=g=y(g?r.resolve(g,p):p);var m=u._refs[p];if("string"==typeof m&&(m=u._refs[m]),m&&m.schema){if(!i(e,m.schema))throw new Error('id "'+p+'" resolves to more than one schema')}else if(p!=y(v))if("#"==p[0]){if(l[p]&&!i(e,l[p]))throw new Error('id "'+p+'" resolves to more than one schema');l[p]=e}else u._refs[p]=v}n[t]=g,a[t]=v}})),l},l.inlineRef=p,l.schema=u;var d=o.toHash(["properties","patternProperties","enum","dependencies","definitions"]);function h(e,t,n,r){if(e.fragment=e.fragment||"","/"==e.fragment.slice(0,1)){for(var i=e.fragment.split("/"),a=1;a<i.length;a++){var s=i[a];if(s){if(void 0===(n=n[s=o.unescapeFragment(s)]))break;var l;if(!d[s]&&((l=this._getId(n))&&(t=b(t,l)),n.$ref)){var c=b(t,n.$ref),h=u.call(this,r,c);h&&(n=h.schema,r=h.root,t=h.baseId)}}}return void 0!==n&&n!==r.schema?{schema:n,root:r,baseId:t}:void 0}}var f=o.toHash(["type","format","pattern","maxLength","minLength","maxProperties","minProperties","maxItems","minItems","maximum","minimum","uniqueItems","multipleOf","required","enum"]);function p(e,t){return!1!==t&&(void 0===t||!0===t?function e(t){var n;if(Array.isArray(t)){for(var r=0;r<t.length;r++)if("object"==typeof(n=t[r])&&!e(n))return!1}else for(var i in t){if("$ref"==i)return!1;if("object"==typeof(n=t[i])&&!e(n))return!1}return!0}(e):t?function e(t){var n,r=0;if(Array.isArray(t)){for(var i=0;i<t.length;i++)if("object"==typeof(n=t[i])&&(r+=e(n)),r==1/0)return 1/0}else for(var o in t){if("$ref"==o)return 1/0;if(f[o])r++;else if("object"==typeof(n=t[o])&&(r+=e(n)+1),r==1/0)return 1/0}return r}(e)<=t:void 0)}function g(e,t){return!1!==t&&(e=y(e)),v(r.parse(e))}function v(e){return r.serialize(e).split("#")[0]+"#"}var m=/#\/?$/;function y(e){return e?e.replace(m,""):""}function b(e,t){return t=y(t),r.resolve(e,t)}},function(e,t,n){"use strict";e.exports=function e(t,n){if(t===n)return!0;if(t&&n&&"object"==typeof t&&"object"==typeof n){if(t.constructor!==n.constructor)return!1;var r,i,o;if(Array.isArray(t)){if((r=t.length)!=n.length)return!1;for(i=r;0!=i--;)if(!e(t[i],n[i]))return!1;return!0}if(t.constructor===RegExp)return t.source===n.source&&t.flags===n.flags;if(t.valueOf!==Object.prototype.valueOf)return t.valueOf()===n.valueOf();if(t.toString!==Object.prototype.toString)return t.toString()===n.toString();if((r=(o=Object.keys(t)).length)!==Object.keys(n).length)return!1;for(i=r;0!=i--;)if(!Object.prototype.hasOwnProperty.call(n,o[i]))return!1;for(i=r;0!=i--;){var a=o[i];if(!e(t[a],n[a]))return!1}return!0}return t!=t&&n!=n}},function(e,t,n){"use strict";var r=n(509);function i(e,t,n){this.message=n||i.message(e,t),this.missingRef=r.url(e,t),this.missingSchema=r.normalizeId(r.fullPath(this.missingRef))}function o(e){return e.prototype=Object.create(Error.prototype),e.prototype.constructor=e,e}e.exports={Validation:o((function(e){this.message="validation failed",this.errors=e,this.ajv=this.validation=!0})),MissingRef:o(i)},i.message=function(e,t){return"can't resolve reference "+t+" from id "+e}},function(e,t,n){"use strict";function r(e){return function(){return null}}n.d(t,"a",(function(){return r}))},function(e,t,n){"use strict";n.d(t,"b",(function(){return v}));var r=n(627),i=n(85),o=n(319),a=n(628),s=n(320),l=n(321),u=n(322),c=n(323),d=n(629),h=n(324),f=n(326),p=n(325),g=n(435),v=Object(r.a)(Object(i.a)(o.h,a.a,s.d,l.a,u.b,c.c,d.a,h.b,f.b,p.a)),m=Object(g.a)("div")(v,{name:"MuiBox"});t.a=m},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.Rect=t.DropInfo=t.DragDrop=t.Orientation=t.DockLocation=t.BorderSet=t.BorderNode=t.TabSetNode=t.TabNode=t.SplitterNode=t.RowNode=t.Node=t.Model=t.Action=t.Actions=t.I18nLabel=t.Layout=void 0;var r=n(190);t.DockLocation=r.default;var i=n(515);t.DragDrop=i.default;var o=n(370);t.DropInfo=o.default;var a=n(210);Object.defineProperty(t,"I18nLabel",{enumerable:!0,get:function(){return a.I18nLabel}});var s=n(933);t.Action=s.default;var l=n(159);t.Actions=l.default;var u=n(243);t.BorderNode=u.default;var c=n(935);t.BorderSet=c.default;var d=n(1566);t.Model=d.default;var h=n(245);t.Node=h.default;var f=n(516);t.RowNode=f.default;var p=n(371);t.SplitterNode=p.default;var g=n(290);t.TabNode=g.default;var v=n(191);t.TabSetNode=v.default;var m=n(111);t.Orientation=m.default;var y=n(131);t.Rect=y.default;var b=n(1567);t.Layout=b.default,t.default={Layout:b.default,I18nLabel:a.I18nLabel,Actions:l.default,Action:s.default,Model:d.default,Node:h.default,RowNode:f.default,SplitterNode:p.default,TabNode:g.default,TabSetNode:v.default,BorderNode:u.default,BorderSet:c.default,DockLocation:r.default,Orientation:m.default,DragDrop:i.default,DropInfo:o.default,Rect:y.default}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(131),i=!("undefined"==typeof window||!window.document||!window.document.createElement),o=function(){function e(){this._manualGlassManagement=!1,this._startX=0,this._startY=0,this._glassShowing=!1,this._dragging=!1,this._active=!1,i&&(this._glass=document.createElement("div"),this._glass.style.zIndex="998",this._glass.style.position="absolute",this._glass.style.backgroundColor="transparent",this._glass.style.outline="none"),this._onMouseMove=this._onMouseMove.bind(this),this._onMouseUp=this._onMouseUp.bind(this),this._onKeyPress=this._onKeyPress.bind(this),this._lastClick=0,this._clickX=0,this._clickY=0}return e.prototype.addGlass=function(e,t){this._glassShowing?this._manualGlassManagement=!0:(t||(t=window.document),this._document=t,new r.default(0,0,t.documentElement.scrollWidth,t.documentElement.scrollHeight).positionElement(this._glass),t.body.appendChild(this._glass),this._glass.tabIndex=-1,this._glass.focus(),this._glass.addEventListener("keydown",this._onKeyPress),this._glassShowing=!0,this._fDragCancel=e,this._manualGlassManagement=!1)},e.prototype.hideGlass=function(){this._glassShowing&&(this._document.body.removeChild(this._glass),this._glassShowing=!1,this._document=void 0)},e.prototype.startDrag=function(e,t,n,r,i,o,a,s){if(!(e&&this._lastEvent&&this._lastEvent.type.startsWith("touch")&&e.type.startsWith("mouse")&&e.timeStamp-this._lastEvent.timeStamp<500)){this._lastEvent=e,this._document=s||window.document;var l=this._getLocationEvent(e);this.addGlass(i,s),this._dragging&&console.warn("this._dragging true on startDrag should never happen"),e?(this._startX=l.clientX,this._startY=l.clientY,window.matchMedia&&!window.matchMedia("(pointer: fine)").matches||(this._glass.style.cursor=getComputedStyle(e.target).cursor),this._stopPropagation(e),this._preventDefault(e)):(this._startX=0,this._startY=0,this._glass.style.cursor="default"),this._dragging=!1,this._fDragStart=t,this._fDragMove=n,this._fDragEnd=r,this._fDragCancel=i,this._fClick=o,this._fDblClick=a,this._active=!0,this._document.addEventListener("mouseup",this._onMouseUp,{passive:!1}),this._document.addEventListener("mousemove",this._onMouseMove,{passive:!1}),this._document.addEventListener("touchend",this._onMouseUp,{passive:!1}),this._document.addEventListener("touchmove",this._onMouseMove,{passive:!1})}},e.prototype.isDragging=function(){return this._dragging},e.prototype.isActive=function(){return this._active},e.prototype.toString=function(){return"(DragDrop: startX="+this._startX+", startY="+this._startY+", dragging="+this._dragging+")"},e.prototype._onKeyPress=function(e){void 0!==this._fDragCancel&&27===e.keyCode&&(this._document.removeEventListener("mousemove",this._onMouseMove),this._document.removeEventListener("mouseup",this._onMouseUp),this._document.removeEventListener("touchend",this._onMouseUp),this._document.removeEventListener("touchmove",this._onMouseMove),this.hideGlass(),this._fDragCancel(this._dragging),this._dragging=!1,this._active=!1)},e.prototype._getLocationEvent=function(e){var t=e;return e&&e.touches&&(t=e.touches[0]),t},e.prototype._getLocationEventEnd=function(e){var t=e;return e.changedTouches&&(t=e.changedTouches[0]),t},e.prototype._stopPropagation=function(e){e.stopPropagation&&e.stopPropagation()},e.prototype._preventDefault=function(e){return e.preventDefault&&e.cancelable&&e.preventDefault(),e},e.prototype._onMouseMove=function(e){this._lastEvent=e;var t=this._getLocationEvent(e);return this._stopPropagation(e),this._preventDefault(e),!this._dragging&&(Math.abs(this._startX-t.clientX)>5||Math.abs(this._startY-t.clientY)>5)&&(this._dragging=!0,this._fDragStart&&(this._glass.style.cursor="move",this._dragging=this._fDragStart({clientX:this._startX,clientY:this._startY}))),this._dragging&&this._fDragMove&&this._fDragMove(t),!1},e.prototype._onMouseUp=function(e){this._lastEvent=e;var t=this._getLocationEventEnd(e);if(this._stopPropagation(e),this._preventDefault(e),this._active=!1,this._document.removeEventListener("mousemove",this._onMouseMove),this._document.removeEventListener("mouseup",this._onMouseUp),this._document.removeEventListener("touchend",this._onMouseUp),this._document.removeEventListener("touchmove",this._onMouseMove),this._manualGlassManagement||this.hideGlass(),this._dragging)this._dragging=!1,this._fDragEnd&&this._fDragEnd(e);else if(this._fDragCancel&&this._fDragCancel(this._dragging),Math.abs(this._startX-t.clientX)<=5&&Math.abs(this._startY-t.clientY)<=5){var n=(new Date).getTime();Math.abs(this._clickX-t.clientX)<=5&&Math.abs(this._clickY-t.clientY)<=5&&n-this._lastClick<500&&this._fDblClick&&this._fDblClick(e),this._fClick&&this._fClick(e),this._lastClick=n,this._clickX=t.clientX,this._clickY=t.clientY}return!1},e.instance=new e,e}();t.default=o},function(e,t,n){"use strict";var r,i=this&&this.__extends||(r=function(e,t){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(e[n]=t[n])})(e,t)},function(e,t){function n(){this.constructor=e}r(e,t),e.prototype=null===t?Object.create(t):(n.prototype=t.prototype,new n)});Object.defineProperty(t,"__esModule",{value:!0});var o=n(244),a=n(190),s=n(370),l=n(111),u=n(131),c=n(243),d=n(245),h=n(371),f=n(191),p=function(e){function t(n,r){var i=e.call(this,n)||this;return i._dirty=!0,i._drawChildren=[],i.minHeight=0,i.minWidth=0,t._attributeDefinitions.fromJson(r,i._attributes),n._addNode(i),i}return i(t,e),t._fromJson=function(e,n){var r=new t(n,e);if(null!=e.children)for(var i=0,o=e.children;i<o.length;i++){var a=o[i];if(a.type===f.default.TYPE){var s=f.default._fromJson(a,n);r._addChild(s)}else{s=t._fromJson(a,n);r._addChild(s)}}return r},t._createAttributeDefinitions=function(){var e=new o.default;return e.add("type",t.TYPE,!0),e.add("id",void 0),e.add("weight",100),e.add("width",void 0),e.add("height",void 0),e},t.prototype.getWeight=function(){return this._attributes.weight},t.prototype.getWidth=function(){return this._getAttributeAsNumberOrUndefined("width")},t.prototype.getHeight=function(){return this._getAttributeAsNumberOrUndefined("height")},t.prototype._setWeight=function(e){this._attributes.weight=e},t.prototype._layout=function(t,n){e.prototype._layout.call(this,t,n);for(var r=this._rect._getSize(this.getOrientation()),i=0,o=0,a=0,s=0,c=this._getDrawChildren(),d=0,f=c;d<f.length;d++){var p=(D=f[d])._getPrefSize(this.getOrientation());D._isFixed()?void 0!==p&&(o+=p):void 0===p?i+=D.getWeight():(a+=p,s+=D.getWeight())}var g=!1,v=r-o-a;v<0&&(v=r-o,g=!0,i+=s);for(var m=0,y=0,b=0,C=c;b<C.length;b++){p=(D=C[b])._getPrefSize(this.getOrientation());if(D._isFixed())void 0!==p&&D._setTempSize(p);else if(null==p||g){if(0===i)D._setTempSize(0);else{var A=D.getMinSize(this.getOrientation()),_=Math.floor(v*(D.getWeight()/i));D._setTempSize(Math.max(A,_))}y+=D._getTempSize()}else D._setTempSize(p);m+=D._getTempSize()}if(y>0){for(;m<r;)for(var I=0,x=c;I<x.length;I++){if(!((D=x[I])instanceof h.default)){p=D._getPrefSize(this.getOrientation());!D._isFixed()&&(void 0===p||g)&&m<r&&(D._setTempSize(D._getTempSize()+1),m++)}}for(;m>r;){for(var w=!1,M=0,S=c;M<S.length;M++){if(!((D=S[M])instanceof h.default)){A=D.getMinSize(this.getOrientation());(_=D._getTempSize())>A&&m>r&&(D._setTempSize(D._getTempSize()-1),m--,w=!0)}}if(!w)break}for(;m>r;){w=!1;for(var T=0,E=c;T<E.length;T++){if(!((D=E[T])instanceof h.default))(_=D._getTempSize())>0&&m>r&&(D._setTempSize(D._getTempSize()-1),m--,w=!0)}if(!w)break}}for(var O=0,k=0,L=c;k<L.length;k++){var D=L[k];this.getOrientation()===l.default.HORZ?D._layout(new u.default(this._rect.x+O,this._rect.y,D._getTempSize(),this._rect.height),n):D._layout(new u.default(this._rect.x,this._rect.y+O,this._rect.width,D._getTempSize()),n),O+=D._getTempSize()}return!0},t.prototype._getSplitterBounds=function(e,t){void 0===t&&(t=!1);var n=[0,0],r=this._getDrawChildren(),i=r.indexOf(e),o=r[i-1],a=r[i+1];if(this.getOrientation()===l.default.HORZ){var s=t?o.getMinWidth():0,u=t?a.getMinWidth():0;n[0]=o.getRect().x+s,n[1]=a.getRect().getRight()-e.getWidth()-u}else{s=t?o.getMinHeight():0,u=t?a.getMinHeight():0;n[0]=o.getRect().y+s,n[1]=a.getRect().getBottom()-e.getHeight()-u}return n},t.prototype._calculateSplit=function(e,t){var n,r=this._getDrawChildren(),i=r.indexOf(e),o=this._getSplitterBounds(e),a=r[i-1].getWeight()+r[i+1].getWeight(),s=Math.max(0,t-o[0]),l=Math.max(0,o[1]-t);if(s+l>0){var u=s*a/(s+l),c=l*a/(s+l);n={node1Id:r[i-1].getId(),weight1:u,pixelWidth1:s,node2Id:r[i+1].getId(),weight2:c,pixelWidth2:l}}return n},t.prototype._getDrawChildren=function(){if(this._dirty){this._drawChildren=[];for(var e=0;e<this._children.length;e++){var t=this._children[e];if(0!==e){var n=new h.default(this._model);n._setParent(this),this._drawChildren.push(n)}this._drawChildren.push(t)}this._dirty=!1}return this._drawChildren},t.prototype.getMinSize=function(e){return e===l.default.HORZ?this.getMinWidth():this.getMinHeight()},t.prototype.getMinWidth=function(){return this.minWidth},t.prototype.getMinHeight=function(){return this.minHeight},t.prototype.calcMinSize=function(){var e=this;this.minHeight=0,this.minWidth=0;var n=!0;this._children.forEach((function(r){var i=r;i instanceof t&&i.calcMinSize(),e.getOrientation()===l.default.VERT?(e.minHeight+=i.getMinHeight(),n||(e.minHeight+=e._model.getSplitterSize()),e.minWidth=Math.max(e.minWidth,i.getMinWidth())):(e.minWidth+=i.getMinWidth(),n||(e.minWidth+=e._model.getSplitterSize()),e.minHeight=Math.max(e.minHeight,i.getMinHeight())),n=!1}))},t.prototype._tidy=function(){for(var e=0;e<this._children.length;){if((c=this._children[e])instanceof t){c._tidy();var n=c.getChildren();if(0===n.length)this._removeChild(c);else if(1===n.length){var r=n[0];if(this._removeChild(c),r instanceof t){for(var i=0,o=r.getChildren(),a=0,s=o;a<s.length;a++){i+=(u=s[a]).getWeight()}for(var l=0;l<o.length;l++){var u;(u=o[l])._setWeight(c.getWeight()*u.getWeight()/i),this._addChild(u,e+l)}}else r._setWeight(c.getWeight()),this._addChild(r,e)}else e++}else c instanceof f.default&&0===c.getChildren().length&&c.isEnableDeleteWhenEmpty()?(this._removeChild(c),c===this._model.getMaximizedTabset()&&this._model._setMaximizedTabset(void 0)):e++}if(this===this._model.getRoot()&&0===this._children.length){var c=new f.default(this._model,{type:"tabset"});this._model._setActiveTabset(c),this._addChild(c)}},t.prototype.canDrop=function(e,t,n){var r,i=n-this._rect.y,o=t-this._rect.x,l=this._rect.width,u=this._rect.height;if(this._model.isEnableEdgeDock()&&void 0===this._parent){if(t<this._rect.x+10&&i>u/2-50&&i<u/2+50)(d=(c=a.default.LEFT).getDockRect(this._rect)).width=d.width/2,r=new s.default(this,d,c,-1,"flexlayout__outline_rect_edge");else if(t>this._rect.getRight()-10&&i>u/2-50&&i<u/2+50){(d=(c=a.default.RIGHT).getDockRect(this._rect)).width=d.width/2,d.x+=d.width,r=new s.default(this,d,c,-1,"flexlayout__outline_rect_edge")}else if(n<this._rect.y+10&&o>l/2-50&&o<l/2+50){(d=(c=a.default.TOP).getDockRect(this._rect)).height=d.height/2,r=new s.default(this,d,c,-1,"flexlayout__outline_rect_edge")}else if(n>this._rect.getBottom()-10&&o>l/2-50&&o<l/2+50){var c,d;(d=(c=a.default.BOTTOM).getDockRect(this._rect)).height=d.height/2,d.y+=d.height,r=new s.default(this,d,c,-1,"flexlayout__outline_rect_edge")}if(void 0!==r&&!e._canDockInto(e,r))return}return r},t.prototype.drop=function(e,n,r){var i,o=n,s=e.getParent();s&&s._removeChild(e),void 0!==s&&s.getType()===f.default.TYPE&&s._setSelected(0),void 0!==s&&s.getType()===c.default.TYPE&&s._setSelected(-1),e instanceof f.default?i=e:(i=new f.default(this._model,{}))._addChild(e);var l=this._children.reduce((function(e,t){return e+t.getWeight()}),0);if(0===l&&(l=100),i._setWeight(l/3),o===a.default.LEFT)this._addChild(i,0);else if(o===a.default.RIGHT)this._addChild(i);else if(o===a.default.TOP){var u=new t(this._model,{}),d=new t(this._model,{});d._setWeight(75),i._setWeight(25),this._children.forEach((function(e){d._addChild(e)})),this._removeAll(),u._addChild(i),u._addChild(d),this._addChild(u)}else if(o===a.default.BOTTOM){u=new t(this._model,{});var h=new t(this._model,{});h._setWeight(75),i._setWeight(25),this._children.forEach((function(e){h._addChild(e)})),this._removeAll(),u._addChild(h),u._addChild(i),this._addChild(u)}this._model._setActiveTabset(i),this._model._tidy()},t.prototype._toJson=function(){var e={};return t._attributeDefinitions.toJson(e,this._attributes),e.children=[],this._children.forEach((function(t){e.children.push(t._toJson())})),e},t.prototype.isEnableDrop=function(){return!0},t.prototype._getPrefSize=function(e){var t=this.getWidth();return e===l.default.VERT&&(t=this.getHeight()),t},t.prototype._getAttributeDefinitions=function(){return t._attributeDefinitions},t.prototype._updateAttrs=function(e){t._attributeDefinitions.update(e,this._attributes)},t.TYPE="row",t._attributeDefinitions=t._createAttributeDefinitions(),t}(d.default);t.default=p},function(e,t,n){"use strict";function r(e){return e&&e.$$typeof&&"Symbol(react.forward_ref)"===e.$$typeof.toString()}Object.defineProperty(t,"__esModule",{value:!0}),t.isRefForwardingComponent=r,t.isRefable=function(e){return(t=e)&&t.prototype&&"function"==typeof t.prototype.render||r(e);var t}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(0),i=n(372),o=n(19);t.useDragDropManager=function(){var e=r.useContext(i.context).dragDropManager;return o(null!=e,"Expected drag drop context"),e}},function(e,t,n){var r,i;r=[n,t,n(1191)],void 0===(i=function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.updateNodeId=t.getNodeFullPath=t.debounce=void 0,t.debounce=function(e,t){let n;return()=>{clearTimeout(n),n=setTimeout(()=>{e()},t)}},t.getNodeFullPath=function(e,t){const r=e=>e.treeIndex;let i;for(let o=0;o<t.length;o++){const a=t.slice(0,o+1),s=n.getNodeAtPath({treeData:e,path:a,getNodeKey:r}).node.title;i=0==o?s:i.concat("."+s)}return i},t.updateNodeId=function(e,t,r){const i={};return n.walk({treeData:e,getNodeKey:e=>e.treeIndex,callback:e=>{const n=e.node.id;if(n.startsWith(t)){const o=n.replace(t,r);e.node.id=o,i[n]=o}}}),i}}.apply(t,r))||(e.exports=i)},function(e,t,n){"use strict";(function(e){function r(){return(r=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e}).apply(this,arguments)}function i(e){return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}var o,a=(o=function(e,t){return(o=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(e,t)},function(e,t){function n(){this.constructor=e}o(e,t),e.prototype=null===t?Object.create(t):(n.prototype=t.prototype,new n)});Object.defineProperty(t,"__esModule",{value:!0}),t.UnControlled=t.Controlled=void 0;var s,l=n(0),u="undefined"==typeof navigator||!0===e.PREVENT_CODEMIRROR_RENDER;u||(s=n(122));var c=function(){function e(){}return e.equals=function(e,t){var n=this,r=Object.keys,o=i(e),a=i(t);return e&&t&&"object"===o&&o===a?r(e).length===r(t).length&&r(e).every((function(r){return n.equals(e[r],t[r])})):e===t},e}(),d=function(){function e(e,t){this.editor=e,this.props=t}return e.prototype.delegateCursor=function(e,t,n){var r=this.editor.getDoc();n&&this.editor.focus(),t?r.setCursor(e):r.setCursor(e,null,{scroll:!1})},e.prototype.delegateScroll=function(e){this.editor.scrollTo(e.x,e.y)},e.prototype.delegateSelection=function(e,t){this.editor.getDoc().setSelections(e),t&&this.editor.focus()},e.prototype.apply=function(e){e&&e.selection&&e.selection.ranges&&this.delegateSelection(e.selection.ranges,e.selection.focus||!1),e&&e.cursor&&this.delegateCursor(e.cursor,e.autoScroll||!1,this.editor.getOption("autofocus")||!1),e&&e.scroll&&this.delegateScroll(e.scroll)},e.prototype.applyNext=function(e,t,n){e&&e.selection&&e.selection.ranges&&t&&t.selection&&t.selection.ranges&&!c.equals(e.selection.ranges,t.selection.ranges)&&this.delegateSelection(t.selection.ranges,t.selection.focus||!1),e&&e.cursor&&t&&t.cursor&&!c.equals(e.cursor,t.cursor)&&this.delegateCursor(n.cursor||t.cursor,t.autoScroll||!1,t.autoCursor||!1),e&&e.scroll&&t&&t.scroll&&!c.equals(e.scroll,t.scroll)&&this.delegateScroll(t.scroll)},e.prototype.applyUserDefined=function(e,t){t&&t.cursor&&this.delegateCursor(t.cursor,e.autoScroll||!1,this.editor.getOption("autofocus")||!1)},e.prototype.wire=function(e){var t=this;Object.keys(e||{}).filter((function(e){return/^on/.test(e)})).forEach((function(e){switch(e){case"onBlur":t.editor.on("blur",(function(e,n){t.props.onBlur(t.editor,n)}));break;case"onContextMenu":t.editor.on("contextmenu",(function(e,n){t.props.onContextMenu(t.editor,n)}));break;case"onCopy":t.editor.on("copy",(function(e,n){t.props.onCopy(t.editor,n)}));break;case"onCursor":t.editor.on("cursorActivity",(function(e){t.props.onCursor(t.editor,t.editor.getDoc().getCursor())}));break;case"onCursorActivity":t.editor.on("cursorActivity",(function(e){t.props.onCursorActivity(t.editor)}));break;case"onCut":t.editor.on("cut",(function(e,n){t.props.onCut(t.editor,n)}));break;case"onDblClick":t.editor.on("dblclick",(function(e,n){t.props.onDblClick(t.editor,n)}));break;case"onDragEnter":t.editor.on("dragenter",(function(e,n){t.props.onDragEnter(t.editor,n)}));break;case"onDragLeave":t.editor.on("dragleave",(function(e,n){t.props.onDragLeave(t.editor,n)}));break;case"onDragOver":t.editor.on("dragover",(function(e,n){t.props.onDragOver(t.editor,n)}));break;case"onDragStart":t.editor.on("dragstart",(function(e,n){t.props.onDragStart(t.editor,n)}));break;case"onDrop":t.editor.on("drop",(function(e,n){t.props.onDrop(t.editor,n)}));break;case"onFocus":t.editor.on("focus",(function(e,n){t.props.onFocus(t.editor,n)}));break;case"onGutterClick":t.editor.on("gutterClick",(function(e,n,r,i){t.props.onGutterClick(t.editor,n,r,i)}));break;case"onInputRead":t.editor.on("inputRead",(function(e,n){t.props.onInputRead(t.editor,n)}));break;case"onKeyDown":t.editor.on("keydown",(function(e,n){t.props.onKeyDown(t.editor,n)}));break;case"onKeyHandled":t.editor.on("keyHandled",(function(e,n,r){t.props.onKeyHandled(t.editor,n,r)}));break;case"onKeyPress":t.editor.on("keypress",(function(e,n){t.props.onKeyPress(t.editor,n)}));break;case"onKeyUp":t.editor.on("keyup",(function(e,n){t.props.onKeyUp(t.editor,n)}));break;case"onMouseDown":t.editor.on("mousedown",(function(e,n){t.props.onMouseDown(t.editor,n)}));break;case"onPaste":t.editor.on("paste",(function(e,n){t.props.onPaste(t.editor,n)}));break;case"onRenderLine":t.editor.on("renderLine",(function(e,n,r){t.props.onRenderLine(t.editor,n,r)}));break;case"onScroll":t.editor.on("scroll",(function(e){t.props.onScroll(t.editor,t.editor.getScrollInfo())}));break;case"onSelection":t.editor.on("beforeSelectionChange",(function(e,n){t.props.onSelection(t.editor,n)}));break;case"onTouchStart":t.editor.on("touchstart",(function(e,n){t.props.onTouchStart(t.editor,n)}));break;case"onUpdate":t.editor.on("update",(function(e){t.props.onUpdate(t.editor)}));break;case"onViewportChange":t.editor.on("viewportChange",(function(e,n,r){t.props.onViewportChange(t.editor,n,r)}))}}))},e}(),h=function(e){function t(t){var n=e.call(this,t)||this;return u||(n.applied=!1,n.appliedNext=!1,n.appliedUserDefined=!1,n.deferred=null,n.emulating=!1,n.hydrated=!1,n.initCb=function(){n.props.editorDidConfigure&&n.props.editorDidConfigure(n.editor)},n.mounted=!1),n}return a(t,e),t.prototype.hydrate=function(e){var t=this,n=e&&e.options?e.options:{},i=r({},s.defaults,this.editor.options,n);Object.keys(i).some((function(e){return t.editor.getOption(e)!==i[e]}))&&Object.keys(i).forEach((function(e){n.hasOwnProperty(e)&&t.editor.getOption(e)!==i[e]&&(t.editor.setOption(e,i[e]),t.mirror.setOption(e,i[e]))})),this.hydrated||(this.deferred?this.resolveChange(e.value):this.initChange(e.value||"")),this.hydrated=!0},t.prototype.initChange=function(e){this.emulating=!0;var t=this.editor.getDoc(),n=t.lastLine(),r=t.getLine(t.lastLine()).length;t.replaceRange(e||"",{line:0,ch:0},{line:n,ch:r}),this.mirror.setValue(e),t.clearHistory(),this.mirror.clearHistory(),this.emulating=!1},t.prototype.resolveChange=function(e){this.emulating=!0;var t=this.editor.getDoc();if("undo"===this.deferred.origin?t.undo():"redo"===this.deferred.origin?t.redo():t.replaceRange(this.deferred.text,this.deferred.from,this.deferred.to,this.deferred.origin),e&&e!==t.getValue()){var n=t.getCursor();t.setValue(e),t.setCursor(n)}this.emulating=!1,this.deferred=null},t.prototype.mirrorChange=function(e){var t=this.editor.getDoc();return"undo"===e.origin?(t.setHistory(this.mirror.getHistory()),this.mirror.undo()):"redo"===e.origin?(t.setHistory(this.mirror.getHistory()),this.mirror.redo()):this.mirror.replaceRange(e.text,e.from,e.to,e.origin),this.mirror.getValue()},t.prototype.componentDidMount=function(){var e=this;u||(this.props.defineMode&&this.props.defineMode.name&&this.props.defineMode.fn&&s.defineMode(this.props.defineMode.name,this.props.defineMode.fn),this.editor=s(this.ref,this.props.options),this.shared=new d(this.editor,this.props),this.mirror=s((function(){}),this.props.options),this.editor.on("electricInput",(function(){e.mirror.setHistory(e.editor.getDoc().getHistory())})),this.editor.on("cursorActivity",(function(){e.mirror.setCursor(e.editor.getDoc().getCursor())})),this.editor.on("beforeChange",(function(t,n){if(!e.emulating){n.cancel(),e.deferred=n;var r=e.mirrorChange(e.deferred);e.props.onBeforeChange&&e.props.onBeforeChange(e.editor,e.deferred,r)}})),this.editor.on("change",(function(t,n){e.mounted&&e.props.onChange&&e.props.onChange(e.editor,n,e.editor.getValue())})),this.hydrate(this.props),this.shared.apply(this.props),this.applied=!0,this.mounted=!0,this.shared.wire(this.props),this.editor.getOption("autofocus")&&this.editor.focus(),this.props.editorDidMount&&this.props.editorDidMount(this.editor,this.editor.getValue(),this.initCb))},t.prototype.componentDidUpdate=function(e){if(!u){var t={cursor:null};this.props.value!==e.value&&(this.hydrated=!1),this.props.autoCursor||void 0===this.props.autoCursor||(t.cursor=this.editor.getDoc().getCursor()),this.hydrate(this.props),this.appliedNext||(this.shared.applyNext(e,this.props,t),this.appliedNext=!0),this.shared.applyUserDefined(e,t),this.appliedUserDefined=!0}},t.prototype.componentWillUnmount=function(){u||this.props.editorWillUnmount&&this.props.editorWillUnmount(s)},t.prototype.shouldComponentUpdate=function(e,t){return!u},t.prototype.render=function(){var e=this;if(u)return null;var t=this.props.className?"react-codemirror2 "+this.props.className:"react-codemirror2";return l.createElement("div",{className:t,ref:function(t){return e.ref=t}})},t}(l.Component);t.Controlled=h;var f=function(e){function t(t){var n=e.call(this,t)||this;return u||(n.applied=!1,n.appliedUserDefined=!1,n.continueChange=!1,n.detached=!1,n.hydrated=!1,n.initCb=function(){n.props.editorDidConfigure&&n.props.editorDidConfigure(n.editor)},n.mounted=!1,n.onBeforeChangeCb=function(){n.continueChange=!0}),n}return a(t,e),t.prototype.hydrate=function(e){var t=this,n=e&&e.options?e.options:{},i=r({},s.defaults,this.editor.options,n);if(Object.keys(i).some((function(e){return t.editor.getOption(e)!==i[e]}))&&Object.keys(i).forEach((function(e){n.hasOwnProperty(e)&&t.editor.getOption(e)!==i[e]&&t.editor.setOption(e,i[e])})),!this.hydrated){var o=this.editor.getDoc(),a=o.lastLine(),l=o.getLine(o.lastLine()).length;o.replaceRange(e.value||"",{line:0,ch:0},{line:a,ch:l})}this.hydrated=!0},t.prototype.componentDidMount=function(){var e=this;u||(this.detached=!0===this.props.detach,this.props.defineMode&&this.props.defineMode.name&&this.props.defineMode.fn&&s.defineMode(this.props.defineMode.name,this.props.defineMode.fn),this.editor=s(this.ref,this.props.options),this.shared=new d(this.editor,this.props),this.editor.on("beforeChange",(function(t,n){e.props.onBeforeChange&&e.props.onBeforeChange(e.editor,n,e.editor.getValue(),e.onBeforeChangeCb)})),this.editor.on("change",(function(t,n){e.mounted&&e.props.onChange&&(e.props.onBeforeChange?e.continueChange&&e.props.onChange(e.editor,n,e.editor.getValue()):e.props.onChange(e.editor,n,e.editor.getValue()))})),this.hydrate(this.props),this.shared.apply(this.props),this.applied=!0,this.mounted=!0,this.shared.wire(this.props),this.editor.getDoc().clearHistory(),this.props.editorDidMount&&this.props.editorDidMount(this.editor,this.editor.getValue(),this.initCb))},t.prototype.componentDidUpdate=function(e){if(this.detached&&!1===this.props.detach&&(this.detached=!1,e.editorDidAttach&&e.editorDidAttach(this.editor)),this.detached||!0!==this.props.detach||(this.detached=!0,e.editorDidDetach&&e.editorDidDetach(this.editor)),!u&&!this.detached){var t={cursor:null};this.props.value!==e.value&&(this.hydrated=!1,this.applied=!1,this.appliedUserDefined=!1),e.autoCursor||void 0===e.autoCursor||(t.cursor=this.editor.getDoc().getCursor()),this.hydrate(this.props),this.applied||(this.shared.apply(e),this.applied=!0),this.appliedUserDefined||(this.shared.applyUserDefined(e,t),this.appliedUserDefined=!0)}},t.prototype.componentWillUnmount=function(){u||this.props.editorWillUnmount&&this.props.editorWillUnmount(s)},t.prototype.shouldComponentUpdate=function(e,t){var n=!0;return u&&(n=!1),this.detached&&e.detach&&(n=!1),n},t.prototype.render=function(){var e=this;if(u)return null;var t=this.props.className?"react-codemirror2 "+this.props.className:"react-codemirror2";return l.createElement("div",{className:t,ref:function(t){return e.ref=t}})},t}(l.Component);t.UnControlled=f}).call(this,n(61))},function(e,t,n){"use strict";function r(e){return(r="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}Object.defineProperty(t,"__esModule",{value:!0}),t.matchesSelector=d,t.matchesSelectorAndParentsTo=function(e,t,n){var r=e;do{if(d(r,t))return!0;if(r===n)return!1;r=r.parentNode}while(r);return!1},t.addEvent=function(e,t,n,r){if(!e)return;var i=l({capture:!0},r);e.addEventListener?e.addEventListener(t,n,i):e.attachEvent?e.attachEvent("on"+t,n):e["on"+t]=n},t.removeEvent=function(e,t,n,r){if(!e)return;var i=l({capture:!0},r);e.removeEventListener?e.removeEventListener(t,n,i):e.detachEvent?e.detachEvent("on"+t,n):e["on"+t]=null},t.outerHeight=function(e){var t=e.clientHeight,n=e.ownerDocument.defaultView.getComputedStyle(e);return t+=(0,i.int)(n.borderTopWidth),t+=(0,i.int)(n.borderBottomWidth)},t.outerWidth=function(e){var t=e.clientWidth,n=e.ownerDocument.defaultView.getComputedStyle(e);return t+=(0,i.int)(n.borderLeftWidth),t+=(0,i.int)(n.borderRightWidth)},t.innerHeight=function(e){var t=e.clientHeight,n=e.ownerDocument.defaultView.getComputedStyle(e);return t-=(0,i.int)(n.paddingTop),t-=(0,i.int)(n.paddingBottom)},t.innerWidth=function(e){var t=e.clientWidth,n=e.ownerDocument.defaultView.getComputedStyle(e);return t-=(0,i.int)(n.paddingLeft),t-=(0,i.int)(n.paddingRight)},t.offsetXYFromParent=function(e,t,n){var r=t===t.ownerDocument.body?{left:0,top:0}:t.getBoundingClientRect(),i=(e.clientX+t.scrollLeft-r.left)/n,o=(e.clientY+t.scrollTop-r.top)/n;return{x:i,y:o}},t.createCSSTransform=function(e,t){var n=h(e,t,"px");return u({},(0,o.browserPrefixToKey)("transform",o.default),n)},t.createSVGTransform=function(e,t){return h(e,t,"")},t.getTranslation=h,t.getTouch=function(e,t){return e.targetTouches&&(0,i.findInArray)(e.targetTouches,(function(e){return t===e.identifier}))||e.changedTouches&&(0,i.findInArray)(e.changedTouches,(function(e){return t===e.identifier}))},t.getTouchIdentifier=function(e){if(e.targetTouches&&e.targetTouches[0])return e.targetTouches[0].identifier;if(e.changedTouches&&e.changedTouches[0])return e.changedTouches[0].identifier},t.addUserSelectStyles=function(e){if(!e)return;var t=e.getElementById("react-draggable-style-el");t||((t=e.createElement("style")).type="text/css",t.id="react-draggable-style-el",t.innerHTML=".react-draggable-transparent-selection *::-moz-selection {all: inherit;}\n",t.innerHTML+=".react-draggable-transparent-selection *::selection {all: inherit;}\n",e.getElementsByTagName("head")[0].appendChild(t));e.body&&f(e.body,"react-draggable-transparent-selection")},t.removeUserSelectStyles=function(e){if(!e)return;try{if(e.body&&p(e.body,"react-draggable-transparent-selection"),e.selection)e.selection.empty();else{var t=(e.defaultView||window).getSelection();t&&"Caret"!==t.type&&t.removeAllRanges()}}catch(e){}},t.addClassName=f,t.removeClassName=p;var i=n(380),o=function(e){if(e&&e.__esModule)return e;if(null===e||"object"!==r(e)&&"function"!=typeof e)return{default:e};var t=a();if(t&&t.has(e))return t.get(e);var n={},i=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var o in e)if(Object.prototype.hasOwnProperty.call(e,o)){var s=i?Object.getOwnPropertyDescriptor(e,o):null;s&&(s.get||s.set)?Object.defineProperty(n,o,s):n[o]=e[o]}n.default=e,t&&t.set(e,n);return n}(n(1640));function a(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return a=function(){return e},e}function s(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),n.push.apply(n,r)}return n}function l(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?s(Object(n),!0).forEach((function(t){u(e,t,n[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):s(Object(n)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))}))}return e}function u(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}var c="";function d(e,t){return c||(c=(0,i.findInArray)(["matches","webkitMatchesSelector","mozMatchesSelector","msMatchesSelector","oMatchesSelector"],(function(t){return(0,i.isFunction)(e[t])}))),!!(0,i.isFunction)(e[c])&&e[c](t)}function h(e,t,n){var r=e.x,i=e.y,o="translate(".concat(r).concat(n,",").concat(i).concat(n,")");if(t){var a="".concat("string"==typeof t.x?t.x:t.x+n),s="".concat("string"==typeof t.y?t.y:t.y+n);o="translate(".concat(a,", ").concat(s,")")+o}return o}function f(e,t){e.classList?e.classList.add(t):e.className.match(new RegExp("(?:^|\\s)".concat(t,"(?!\\S)")))||(e.className+=" ".concat(t))}function p(e,t){e.classList?e.classList.remove(t):e.className=e.className.replace(new RegExp("(?:^|\\s)".concat(t,"(?!\\S)"),"g"),"")}},function(e,t,n){"use strict";var r=n(42),i=n(249).isArrayOrTypedArray;function o(e,t){return function(){var n,r,a,s,l,u=e;for(s=0;s<t.length-1;s++){if(-1===(n=t[s])){for(r=!0,a=[],l=0;l<u.length;l++)a[l]=o(u[l],t.slice(s+1))(),a[l]!==a[0]&&(r=!1);return r?a[0]:a}if("number"==typeof n&&!i(u))return;if("object"!=typeof(u=u[n])||null===u)return}if("object"==typeof u&&null!==u&&null!==(a=u[t[s]]))return a}}e.exports=function(e,t){if(r(t))t=String(t);else if("string"!=typeof t||"[-1]"===t.substr(t.length-4))throw"bad property string";for(var n,i,a,s=0,u=t.split(".");s<u.length;){if(n=String(u[s]).match(/^([^\[\]]*)((\[\-?[0-9]*\])+)$/)){if(n[1])u[s]=n[1];else{if(0!==s)throw"bad property string";u.splice(0,1)}for(i=n[2].substr(1,n[2].length-2).split("]["),a=0;a<i.length;a++)s++,u.splice(s,0,Number(i[a]))}s++}return"object"!=typeof e?function(e,t,n){return{set:function(){throw"bad container"},get:function(){},astr:t,parts:n,obj:e}}(e,t,u):{set:l(e,u,t),get:o(e,u),astr:t,parts:u,obj:e}};var a=/(^|\.)args\[/;function s(e,t){return void 0===e||null===e&&!t.match(a)}function l(e,t,n){return function(r){var o,a,l=e,h="",f=[[e,h]],p=s(r,n);for(a=0;a<t.length-1;a++){if("number"==typeof(o=t[a])&&!i(l))throw"array index but container is not an array";if(-1===o){if(p=!c(l,t.slice(a+1),r,n))break;return}if(!d(l,o,t[a+1],p))break;if("object"!=typeof(l=l[o])||null===l)throw"container is not an object";h=u(h,o),f.push([l,h])}if(p){if(a===t.length-1&&(delete l[t[a]],Array.isArray(l)&&+t[a]==l.length-1))for(;l.length&&void 0===l[l.length-1];)l.pop()}else l[t[a]]=r}}function u(e,t){var n=t;return r(t)?n="["+t+"]":e&&(n="."+t),e+n}function c(e,t,n,r){var o,a=i(n),u=!0,c=n,h=r.replace("-1",0),f=!a&&s(n,h),p=t[0];for(o=0;o<e.length;o++)h=r.replace("-1",o),a&&(f=s(c=n[o%n.length],h)),f&&(u=!1),d(e,o,p,f)&&l(e[o],t,r.replace("-1",o))(c);return u}function d(e,t,n,r){if(void 0===e[t]){if(r)return!1;e[t]="number"==typeof n?[]:{}}return!0}},function(e,t,n){"use strict";var r=n(42);e.exports=function(e,t){if(e>0)return Math.log(e)/Math.LN10;var n=Math.log(Math.min(t[0],t[1]))/Math.LN10;return r(n)||(n=Math.log(Math.max(t[0],t[1]))/Math.LN10-6),n}},function(e,t,n){"use strict";var r=n(94),i=n(250).hoverlabel,o=n(63).extendFlat;e.exports={hoverlabel:{bgcolor:o({},i.bgcolor,{arrayOk:!0}),bordercolor:o({},i.bordercolor,{arrayOk:!0}),font:r({arrayOk:!0,editType:"none"}),align:o({},i.align,{arrayOk:!0}),namelength:o({},i.namelength,{arrayOk:!0}),editType:"none"}}},function(e,t,n){"use strict";e.exports={SHOW_PLACEHOLDER:100,HIDE_PLACEHOLDER:1e3,DESELECTDIM:.2}},function(e,t,n){"use strict";e.exports=function(){}},function(e,t,n){"use strict";var r=n(39),i=n(252),o=n(528),a=n(968);function s(e){var t=e&&e.parentNode;t&&t.removeChild(e)}function l(e,t,n){var r="plotly.js-style-"+e,o=document.getElementById(r);o||((o=document.createElement("style")).setAttribute("id",r),o.appendChild(document.createTextNode("")),document.head.appendChild(o));var a=o.sheet;a.insertRule?a.insertRule(t+"{"+n+"}",0):a.addRule?a.addRule(t,n,0):i.warn("addStyleRule failed")}function u(e){var t=window.getComputedStyle(e,null),n=t.getPropertyValue("-webkit-transform")||t.getPropertyValue("-moz-transform")||t.getPropertyValue("-ms-transform")||t.getPropertyValue("-o-transform")||t.getPropertyValue("transform");return"none"===n?null:n.replace("matrix","").replace("3d","").slice(1,-1).split(",").map((function(e){return+e}))}function c(e){for(var t=[];d(e);)t.push(e),e=e.parentNode;return t}function d(e){return e&&(e instanceof Element||e instanceof HTMLElement)}e.exports={getGraphDiv:function(e){var t;if("string"==typeof e){if(null===(t=document.getElementById(e)))throw new Error("No DOM element with id '"+e+"' exists on the page.");return t}if(null==e)throw new Error("DOM element provided is null or undefined");return e},isPlotDiv:function(e){var t=r.select(e);return t.node()instanceof HTMLElement&&t.size()&&t.classed("js-plotly-plot")},removeElement:s,addStyleRule:function(e,t){l("global",e,t)},addRelatedStyleRule:l,deleteRelatedStyleRule:function(e){var t="plotly.js-style-"+e,n=document.getElementById(t);n&&s(n)},getFullTransformMatrix:function(e){var t=c(e),n=[1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1];return t.forEach((function(e){var t=u(e);if(t){var r=o.convertCssMatrix(t);n=a.multiply(n,n,r)}})),n},getElementTransformMatrix:u,getElementAndAncestors:c,equalDomRects:function(e,t){return e&&t&&e.x===t.x&&e.y===t.y&&e.top===t.top&&e.left===t.left&&e.right===t.right&&e.bottom===t.bottom}}},function(e,t,n){"use strict";var r=n(968);t.init2dArray=function(e,t){for(var n=new Array(e),r=0;r<e;r++)n[r]=new Array(t);return n},t.transposeRagged=function(e){var t,n,r=0,i=e.length;for(t=0;t<i;t++)r=Math.max(r,e[t].length);var o=new Array(r);for(t=0;t<r;t++)for(o[t]=new Array(i),n=0;n<i;n++)o[t][n]=e[n][t];return o},t.dot=function(e,n){if(!e.length||!n.length||e.length!==n.length)return null;var r,i,o=e.length;if(e[0].length)for(r=new Array(o),i=0;i<o;i++)r[i]=t.dot(e[i],n);else if(n[0].length){var a=t.transposeRagged(n);for(r=new Array(a.length),i=0;i<a.length;i++)r[i]=t.dot(e,a[i])}else for(r=0,i=0;i<o;i++)r+=e[i]*n[i];return r},t.translationMatrix=function(e,t){return[[1,0,e],[0,1,t],[0,0,1]]},t.rotationMatrix=function(e){var t=e*Math.PI/180;return[[Math.cos(t),-Math.sin(t),0],[Math.sin(t),Math.cos(t),0],[0,0,1]]},t.rotationXYMatrix=function(e,n,r){return t.dot(t.dot(t.translationMatrix(n,r),t.rotationMatrix(e)),t.translationMatrix(-n,-r))},t.apply3DTransform=function(e){return function(){var n=arguments,r=1===arguments.length?n[0]:[n[0],n[1],n[2]||0];return t.dot(e,[r[0],r[1],r[2],1]).slice(0,3)}},t.apply2DTransform=function(e){return function(){var n=arguments;3===n.length&&(n=n[0]);var r=1===arguments.length?n[0]:[n[0],n[1]];return t.dot(e,[r[0],r[1],1]).slice(0,2)}},t.apply2DTransform2=function(e){var n=t.apply2DTransform(e);return function(e){return n(e.slice(0,2)).concat(n(e.slice(2,4)))}},t.convertCssMatrix=function(e){if(e){var t=e.length;if(16===t)return e;if(6===t)return[e[0],e[1],0,0,e[2],e[3],0,0,0,0,1,0,e[4],e[5],0,1]}return[1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1]},t.inverseTransformMatrix=function(e){var t=[];return r.invert(t,e),[[t[0],t[1],t[2],t[3]],[t[4],t[5],t[6],t[7]],[t[8],t[9],t[10],t[11]],[t[12],t[13],t[14],t[15]]]}},function(e,t){e.exports=function(){var e=new Float32Array(16);return e[0]=1,e[1]=0,e[2]=0,e[3]=0,e[4]=0,e[5]=1,e[6]=0,e[7]=0,e[8]=0,e[9]=0,e[10]=1,e[11]=0,e[12]=0,e[13]=0,e[14]=0,e[15]=1,e}},function(e,t){e.exports=function(e){return e[0]=1,e[1]=0,e[2]=0,e[3]=0,e[4]=0,e[5]=1,e[6]=0,e[7]=0,e[8]=0,e[9]=0,e[10]=1,e[11]=0,e[12]=0,e[13]=0,e[14]=0,e[15]=1,e}},function(e,t){e.exports=function(e,t){if(e===t){var n=t[1],r=t[2],i=t[3],o=t[6],a=t[7],s=t[11];e[1]=t[4],e[2]=t[8],e[3]=t[12],e[4]=n,e[6]=t[9],e[7]=t[13],e[8]=r,e[9]=o,e[11]=t[14],e[12]=i,e[13]=a,e[14]=s}else e[0]=t[0],e[1]=t[4],e[2]=t[8],e[3]=t[12],e[4]=t[1],e[5]=t[5],e[6]=t[9],e[7]=t[13],e[8]=t[2],e[9]=t[6],e[10]=t[10],e[11]=t[14],e[12]=t[3],e[13]=t[7],e[14]=t[11],e[15]=t[15];return e}},function(e,t){e.exports=function(e){var t=e[0],n=e[1],r=e[2],i=e[3],o=e[4],a=e[5],s=e[6],l=e[7],u=e[8],c=e[9],d=e[10],h=e[11],f=e[12],p=e[13],g=e[14],v=e[15];return(t*a-n*o)*(d*v-h*g)-(t*s-r*o)*(c*v-h*p)+(t*l-i*o)*(c*g-d*p)+(n*s-r*a)*(u*v-h*f)-(n*l-i*a)*(u*g-d*f)+(r*l-i*s)*(u*p-c*f)}},function(e,t){e.exports=function(e,t,n){var r,i,o,a,s,l,u,c,d,h,f,p,g=n[0],v=n[1],m=n[2];t===e?(e[12]=t[0]*g+t[4]*v+t[8]*m+t[12],e[13]=t[1]*g+t[5]*v+t[9]*m+t[13],e[14]=t[2]*g+t[6]*v+t[10]*m+t[14],e[15]=t[3]*g+t[7]*v+t[11]*m+t[15]):(r=t[0],i=t[1],o=t[2],a=t[3],s=t[4],l=t[5],u=t[6],c=t[7],d=t[8],h=t[9],f=t[10],p=t[11],e[0]=r,e[1]=i,e[2]=o,e[3]=a,e[4]=s,e[5]=l,e[6]=u,e[7]=c,e[8]=d,e[9]=h,e[10]=f,e[11]=p,e[12]=r*g+s*v+d*m+t[12],e[13]=i*g+l*v+h*m+t[13],e[14]=o*g+u*v+f*m+t[14],e[15]=a*g+c*v+p*m+t[15]);return e}},function(e,t){e.exports=function(e,t,n){var r=n[0],i=n[1],o=n[2];return e[0]=t[0]*r,e[1]=t[1]*r,e[2]=t[2]*r,e[3]=t[3]*r,e[4]=t[4]*i,e[5]=t[5]*i,e[6]=t[6]*i,e[7]=t[7]*i,e[8]=t[8]*o,e[9]=t[9]*o,e[10]=t[10]*o,e[11]=t[11]*o,e[12]=t[12],e[13]=t[13],e[14]=t[14],e[15]=t[15],e}},function(e,t){e.exports=function(e,t){var n=t[0],r=t[1],i=t[2],o=t[3],a=n+n,s=r+r,l=i+i,u=n*a,c=r*a,d=r*s,h=i*a,f=i*s,p=i*l,g=o*a,v=o*s,m=o*l;return e[0]=1-d-p,e[1]=c+m,e[2]=h-v,e[3]=0,e[4]=c-m,e[5]=1-u-p,e[6]=f+g,e[7]=0,e[8]=h+v,e[9]=f-g,e[10]=1-u-d,e[11]=0,e[12]=0,e[13]=0,e[14]=0,e[15]=1,e}},function(e,t,n){var r=n(530);e.exports=function(e,t,n,i){var o,a,s,l,u,c,d,h,f,p,g=t[0],v=t[1],m=t[2],y=i[0],b=i[1],C=i[2],A=n[0],_=n[1],I=n[2];if(Math.abs(g-A)<1e-6&&Math.abs(v-_)<1e-6&&Math.abs(m-I)<1e-6)return r(e);d=g-A,h=v-_,f=m-I,p=1/Math.sqrt(d*d+h*h+f*f),o=b*(f*=p)-C*(h*=p),a=C*(d*=p)-y*f,s=y*h-b*d,(p=Math.sqrt(o*o+a*a+s*s))?(o*=p=1/p,a*=p,s*=p):(o=0,a=0,s=0);l=h*s-f*a,u=f*o-d*s,c=d*a-h*o,(p=Math.sqrt(l*l+u*u+c*c))?(l*=p=1/p,u*=p,c*=p):(l=0,u=0,c=0);return e[0]=o,e[1]=l,e[2]=d,e[3]=0,e[4]=a,e[5]=u,e[6]=h,e[7]=0,e[8]=s,e[9]=c,e[10]=f,e[11]=0,e[12]=-(o*g+a*v+s*m),e[13]=-(l*g+u*v+c*m),e[14]=-(d*g+h*v+f*m),e[15]=1,e}},function(e,t,n){"use strict";e.exports=function(e){var t=e.editType;return{t:{valType:"number",dflt:0,editType:t},r:{valType:"number",dflt:0,editType:t},b:{valType:"number",dflt:0,editType:t},l:{valType:"number",dflt:0,editType:t},editType:t}}},function(e,t,n){"use strict";var r=n(42),i=n(252),o=n(977),a=n(72).BADNUM;function s(e,t){return e<t}function l(e,t){return e<=t}function u(e,t){return e>t}function c(e,t){return e>=t}t.findBin=function(e,t,n){if(r(t.start))return n?Math.ceil((e-t.start)/t.size-1e-9)-1:Math.floor((e-t.start)/t.size+1e-9);var o,a,d=0,h=t.length,f=0,p=h>1?(t[h-1]-t[0])/(h-1):1;for(a=p>=0?n?s:l:n?c:u,e+=1e-9*p*(n?-1:1)*(p>=0?1:-1);d<h&&f++<100;)a(t[o=Math.floor((d+h)/2)],e)?d=o+1:h=o;return f>90&&i.log("Long binary search..."),d-1},t.sorterAsc=function(e,t){return e-t},t.sorterDes=function(e,t){return t-e},t.distinctVals=function(e,n){var r,i=(n||{}).unitMinDiff,o=e.slice();for(o.sort(t.sorterAsc),r=o.length-1;r>-1&&o[r]===a;r--);var s=1;i||(s=o[r]-o[0]||1);for(var l,u=s/(r||1)/1e4,c=[],d=0;d<=r;d++){var h=o[d],f=h-l;void 0===l?(c.push(h),l=h):f>u&&(s=Math.min(s,f),c.push(h),l=h)}return{vals:c,minDiff:s}},t.roundUp=function(e,t,n){for(var r,i=0,o=t.length-1,a=0,s=n?0:1,l=n?1:0,u=n?Math.ceil:Math.floor;i<o&&a++<100;)t[r=u((i+o)/2)]<=e?i=r+s:o=r-l;return t[i]},t.sort=function(e,t){for(var n=0,r=0,i=1;i<e.length;i++){var o=t(e[i],e[i-1]);if(o<0?n=1:o>0&&(r=1),n&&r)return e.sort(t)}return r?e:e.reverse()},t.findIndexOfMin=function(e,t){t=t||o;for(var n,r=1/0,i=0;i<e.length;i++){var a=t(e[i]);a<r&&(r=a,n=i)}return n}},function(e,t,n){"use strict";var r={};function i(e){e&&null!==e.timer&&(clearTimeout(e.timer),e.timer=null)}t.throttle=function(e,t,n){var o=r[e],a=Date.now();if(!o){for(var s in r)r[s].ts<a-6e4&&delete r[s];o=r[e]={ts:0,timer:null}}function l(){n(),o.ts=Date.now(),o.onDone&&(o.onDone(),o.onDone=null)}i(o),a>o.ts+t?l():o.timer=setTimeout((function(){l(),o.timer=null}),t)},t.done=function(e){var t=r[e];return t&&t.timer?new Promise((function(e){var n=t.onDone;t.onDone=function(){n&&n(),e(),t.onDone=null}})):Promise.resolve()},t.clear=function(e){if(e)i(r[e]),delete r[e];else for(var n in r)t.clear(n)}},function(e,t,n){"use strict";var r,i="object"==typeof Reflect?Reflect:null,o=i&&"function"==typeof i.apply?i.apply:function(e,t,n){return Function.prototype.apply.call(e,t,n)};r=i&&"function"==typeof i.ownKeys?i.ownKeys:Object.getOwnPropertySymbols?function(e){return Object.getOwnPropertyNames(e).concat(Object.getOwnPropertySymbols(e))}:function(e){return Object.getOwnPropertyNames(e)};var a=Number.isNaN||function(e){return e!=e};function s(){s.init.call(this)}e.exports=s,e.exports.once=function(e,t){return new Promise((function(n,r){function i(){void 0!==o&&e.removeListener("error",o),n([].slice.call(arguments))}var o;"error"!==t&&(o=function(n){e.removeListener(t,i),r(n)},e.once("error",o)),e.once(t,i)}))},s.EventEmitter=s,s.prototype._events=void 0,s.prototype._eventsCount=0,s.prototype._maxListeners=void 0;var l=10;function u(e){if("function"!=typeof e)throw new TypeError('The "listener" argument must be of type Function. Received type '+typeof e)}function c(e){return void 0===e._maxListeners?s.defaultMaxListeners:e._maxListeners}function d(e,t,n,r){var i,o,a,s;if(u(n),void 0===(o=e._events)?(o=e._events=Object.create(null),e._eventsCount=0):(void 0!==o.newListener&&(e.emit("newListener",t,n.listener?n.listener:n),o=e._events),a=o[t]),void 0===a)a=o[t]=n,++e._eventsCount;else if("function"==typeof a?a=o[t]=r?[n,a]:[a,n]:r?a.unshift(n):a.push(n),(i=c(e))>0&&a.length>i&&!a.warned){a.warned=!0;var l=new Error("Possible EventEmitter memory leak detected. "+a.length+" "+String(t)+" listeners added. Use emitter.setMaxListeners() to increase limit");l.name="MaxListenersExceededWarning",l.emitter=e,l.type=t,l.count=a.length,s=l,console&&console.warn&&console.warn(s)}return e}function h(){if(!this.fired)return this.target.removeListener(this.type,this.wrapFn),this.fired=!0,0===arguments.length?this.listener.call(this.target):this.listener.apply(this.target,arguments)}function f(e,t,n){var r={fired:!1,wrapFn:void 0,target:e,type:t,listener:n},i=h.bind(r);return i.listener=n,r.wrapFn=i,i}function p(e,t,n){var r=e._events;if(void 0===r)return[];var i=r[t];return void 0===i?[]:"function"==typeof i?n?[i.listener||i]:[i]:n?function(e){for(var t=new Array(e.length),n=0;n<t.length;++n)t[n]=e[n].listener||e[n];return t}(i):v(i,i.length)}function g(e){var t=this._events;if(void 0!==t){var n=t[e];if("function"==typeof n)return 1;if(void 0!==n)return n.length}return 0}function v(e,t){for(var n=new Array(t),r=0;r<t;++r)n[r]=e[r];return n}Object.defineProperty(s,"defaultMaxListeners",{enumerable:!0,get:function(){return l},set:function(e){if("number"!=typeof e||e<0||a(e))throw new RangeError('The value of "defaultMaxListeners" is out of range. It must be a non-negative number. Received '+e+".");l=e}}),s.init=function(){void 0!==this._events&&this._events!==Object.getPrototypeOf(this)._events||(this._events=Object.create(null),this._eventsCount=0),this._maxListeners=this._maxListeners||void 0},s.prototype.setMaxListeners=function(e){if("number"!=typeof e||e<0||a(e))throw new RangeError('The value of "n" is out of range. It must be a non-negative number. Received '+e+".");return this._maxListeners=e,this},s.prototype.getMaxListeners=function(){return c(this)},s.prototype.emit=function(e){for(var t=[],n=1;n<arguments.length;n++)t.push(arguments[n]);var r="error"===e,i=this._events;if(void 0!==i)r=r&&void 0===i.error;else if(!r)return!1;if(r){var a;if(t.length>0&&(a=t[0]),a instanceof Error)throw a;var s=new Error("Unhandled error."+(a?" ("+a.message+")":""));throw s.context=a,s}var l=i[e];if(void 0===l)return!1;if("function"==typeof l)o(l,this,t);else{var u=l.length,c=v(l,u);for(n=0;n<u;++n)o(c[n],this,t)}return!0},s.prototype.addListener=function(e,t){return d(this,e,t,!1)},s.prototype.on=s.prototype.addListener,s.prototype.prependListener=function(e,t){return d(this,e,t,!0)},s.prototype.once=function(e,t){return u(t),this.on(e,f(this,e,t)),this},s.prototype.prependOnceListener=function(e,t){return u(t),this.prependListener(e,f(this,e,t)),this},s.prototype.removeListener=function(e,t){var n,r,i,o,a;if(u(t),void 0===(r=this._events))return this;if(void 0===(n=r[e]))return this;if(n===t||n.listener===t)0==--this._eventsCount?this._events=Object.create(null):(delete r[e],r.removeListener&&this.emit("removeListener",e,n.listener||t));else if("function"!=typeof n){for(i=-1,o=n.length-1;o>=0;o--)if(n[o]===t||n[o].listener===t){a=n[o].listener,i=o;break}if(i<0)return this;0===i?n.shift():function(e,t){for(;t+1<e.length;t++)e[t]=e[t+1];e.pop()}(n,i),1===n.length&&(r[e]=n[0]),void 0!==r.removeListener&&this.emit("removeListener",e,a||t)}return this},s.prototype.off=s.prototype.removeListener,s.prototype.removeAllListeners=function(e){var t,n,r;if(void 0===(n=this._events))return this;if(void 0===n.removeListener)return 0===arguments.length?(this._events=Object.create(null),this._eventsCount=0):void 0!==n[e]&&(0==--this._eventsCount?this._events=Object.create(null):delete n[e]),this;if(0===arguments.length){var i,o=Object.keys(n);for(r=0;r<o.length;++r)"removeListener"!==(i=o[r])&&this.removeAllListeners(i);return this.removeAllListeners("removeListener"),this._events=Object.create(null),this._eventsCount=0,this}if("function"==typeof(t=n[e]))this.removeListener(e,t);else if(void 0!==t)for(r=t.length-1;r>=0;r--)this.removeListener(e,t[r]);return this},s.prototype.listeners=function(e){return p(this,e,!0)},s.prototype.rawListeners=function(e){return p(this,e,!1)},s.listenerCount=function(e,t){return"function"==typeof e.listenerCount?e.listenerCount(t):g.call(e,t)},s.prototype.listenerCount=g,s.prototype.eventNames=function(){return this._eventsCount>0?r(this._events):[]}},function(e,t,n){"use strict";var r=n(134),i=n(94),o=n(63).extendFlat,a=n(124).overrideAll;e.exports=a({thicknessmode:{valType:"enumerated",values:["fraction","pixels"],dflt:"pixels"},thickness:{valType:"number",min:0,dflt:30},lenmode:{valType:"enumerated",values:["fraction","pixels"],dflt:"fraction"},len:{valType:"number",min:0,dflt:1},x:{valType:"number",dflt:1.02,min:-2,max:3},xanchor:{valType:"enumerated",values:["left","center","right"],dflt:"left"},xpad:{valType:"number",min:0,dflt:10},y:{valType:"number",dflt:.5,min:-2,max:3},yanchor:{valType:"enumerated",values:["top","middle","bottom"],dflt:"middle"},ypad:{valType:"number",min:0,dflt:10},outlinecolor:r.linecolor,outlinewidth:r.linewidth,bordercolor:r.linecolor,borderwidth:{valType:"number",min:0,dflt:0},bgcolor:{valType:"color",dflt:"rgba(0,0,0,0)"},tickmode:r.tickmode,nticks:r.nticks,tick0:r.tick0,dtick:r.dtick,tickvals:r.tickvals,ticktext:r.ticktext,ticks:o({},r.ticks,{dflt:""}),ticklabelposition:{valType:"enumerated",values:["outside","inside","outside top","inside top","outside bottom","inside bottom"],dflt:"outside"},ticklen:r.ticklen,tickwidth:r.tickwidth,tickcolor:r.tickcolor,showticklabels:r.showticklabels,tickfont:i({}),tickangle:r.tickangle,tickformat:r.tickformat,tickformatstops:r.tickformatstops,tickprefix:r.tickprefix,showtickprefix:r.showtickprefix,ticksuffix:r.ticksuffix,showticksuffix:r.showticksuffix,separatethousands:r.separatethousands,exponentformat:r.exponentformat,minexponent:r.minexponent,showexponent:r.showexponent,title:{text:{valType:"string"},font:i({}),side:{valType:"enumerated",values:["right","top","bottom"],dflt:"top"}},_deprecated:{title:{valType:"string"},titlefont:i({}),titleside:{valType:"enumerated",values:["right","top","bottom"],dflt:"top"}}},"colorbars","from-root")},function(e,t,n){"use strict";var r=n(987),i=n(10).isArrayOrTypedArray;e.exports=function(e,t,n,o){function a(n){var r=e[n];return void 0!==r?r:(t._template||{})[n]}var s=a("tick0"),l=a("dtick"),u=a("tickvals"),c=n("tickmode",i(u)?"array":l?"linear":"auto");if("auto"===c)n("nticks");else if("linear"===c){var d=t.dtick=r.dtick(l,o);t.tick0=r.tick0(s,o,t.calendar,d)}else if("multicategory"!==o){void 0===n("tickvals")?t.tickmode="auto":n("ticktext")}}},function(e,t,n){"use strict";var r=n(10),i=n(41).contrast,o=n(134),a=n(178);function s(e){var t=["showexponent","showtickprefix","showticksuffix"].filter((function(t){return void 0!==e[t]}));if(t.every((function(n){return e[n]===e[t[0]]}))||1===t.length)return e[t[0]]}function l(e,t){function n(n,i){return r.coerce(e,t,o.tickformatstops,n,i)}n("enabled")&&(n("dtickrange"),n("value"))}e.exports=function(e,t,n,u,c,d){d&&1!==d.pass||function(e,t,n,r,i){var o=s(e);n("tickprefix")&&n("showtickprefix",o);n("ticksuffix",i.tickSuffixDflt)&&n("showticksuffix",o)}(e,0,n,0,c),d&&2!==d.pass||function(e,t,n,u,c){var d=s(e);n("tickprefix")&&n("showtickprefix",d);n("ticksuffix",c.tickSuffixDflt)&&n("showticksuffix",d);if(n("showticklabels")){var h=c.font||{},f=t.color,p=-1!==(t.ticklabelposition||"").indexOf("inside")?i(c.bgColor):f&&f!==o.color.dflt?f:h.color;if(r.coerceFont(n,"tickfont",{family:h.family,size:h.size,color:p}),n("tickangle"),"category"!==u){var g=n("tickformat");a(e,t,{name:"tickformatstops",inclusionAttr:"enabled",handleItemDefaults:l}),t.tickformatstops.length||delete t.tickformatstops,g||"date"===u||(n("showexponent",d),n("exponentformat"),n("minexponent"),n("separatethousands"))}}}(e,t,n,u,c)}},function(e,t,n){"use strict";e.exports={PTS_LINESONLY:20,minTolerance:.2,toleranceGrowth:10,maxScreensAway:20,eventDataKeys:[]}},function(e,t,n){"use strict";var r=n(42),i=n(10),o=n(72).BADNUM,a=i.isArrayOrTypedArray,s=i.isDateTime,l=i.cleanNumber,u=Math.round;function c(e,t){return t?r(e):"number"==typeof e}function d(e){return Math.max(1,(e-1)/1e3)}e.exports=function(e,t,n){var i=e,h=n.noMultiCategory;if(a(i)&&!i.length)return"-";if(!h&&function(e){return a(e[0])&&a(e[1])}(i))return"multicategory";if(h&&Array.isArray(i[0])){for(var f=[],p=0;p<i.length;p++)if(a(i[p]))for(var g=0;g<i[p].length;g++)f.push(i[p][g]);i=f}if(function(e,t){for(var n=e.length,i=d(n),o=0,a=0,l={},c=0;c<n;c+=i){var h=u(c),f=e[h],p=String(f);l[p]||(l[p]=1,s(f,t)&&o++,r(f)&&a++)}return o>2*a}(i,t))return"date";var v="strict"!==n.autotypenumbers;return function(e,t){for(var n=e.length,r=d(n),i=0,a=0,s={},c=0;c<n;c+=r){var h=u(c),f=e[h],p=String(f);if(!s[p]){s[p]=1;var g=typeof f;"boolean"===g?a++:(t?l(f)!==o:"number"===g)?i++:"string"===g&&a++}}return a>2*i}(i,v)?"category":function(e,t){for(var n=e.length,r=0;r<n;r++)if(c(e[r],t))return!0;return!1}(i,v)?"linear":"-"}},function(e,t,n){"use strict";t.isGrouped=function(e){return-1!==(e.traceorder||"").indexOf("grouped")},t.isVertical=function(e){return"h"!==e.orientation},t.isReversed=function(e){return-1!==(e.traceorder||"").indexOf("reversed")}},function(e,t,n){"use strict";var r=n(10),i=n(41),o=n(162).isUnifiedHover;e.exports=function(e,t,n,a){function s(e){a.font[e]||(a.font[e]=t.legend?t.legend.font[e]:t.font[e])}a=a||{},t&&o(t.hovermode)&&(a.font||(a.font={}),s("size"),s("family"),s("color"),t.legend?(a.bgcolor||(a.bgcolor=i.combine(t.legend.bgcolor,t.paper_bgcolor)),a.bordercolor||(a.bordercolor=t.legend.bordercolor)):a.bgcolor||(a.bgcolor=t.paper_bgcolor)),n("hoverlabel.bgcolor",a.bgcolor),n("hoverlabel.bordercolor",a.bordercolor),n("hoverlabel.namelength",a.namelength),r.coerceFont(n,"hoverlabel.font",a.font),n("hoverlabel.align",a.align)}},function(e,t,n){"use strict";e.exports=function(e){var t=e._fullLayout;t._glcanvas&&t._glcanvas.size()&&t._glcanvas.each((function(e){e.regl&&e.regl.clear({color:!0,depth:!0})}))}},function(e,t,n){"use strict";var r=n(39),i=n(27),o=n(82),a=n(10),s=n(548),l=n(41),u=n(56),c=n(385),d=n(1700),h=n(55),f=n(97),p=n(390),g=p.enforce,v=p.clean,m=n(386).doAutoRange;function y(e,t,n){for(var r=0;r<n.length;r++){var i=n[r][0],o=n[r][1];if(!(i[0]>=e[1]||i[1]<=e[0])&&(o[0]<t[1]&&o[1]>t[0]))return!0}return!1}function b(e){var n,i,s,c,p,g,v=e._fullLayout,m=v._size,b=m.p,A=h.list(e,"",!0);if(v._paperdiv.style({width:e._context.responsive&&v.autosize&&!e._context._hasZeroWidth&&!e.layout.width?"100%":v.width+"px",height:e._context.responsive&&v.autosize&&!e._context._hasZeroHeight&&!e.layout.height?"100%":v.height+"px"}).selectAll(".main-svg").call(u.setSize,v.width,v.height),e._context.setBackground(e,v.paper_bgcolor),t.drawMainTitle(e),d.manage(e),!v._has("cartesian"))return o.previousPromises(e);function I(e,t,n){var r=e._lw/2;return"x"===e._id.charAt(0)?t?"top"===n?t._offset-b-r:t._offset+t._length+b+r:m.t+m.h*(1-(e.position||0))+r%1:t?"right"===n?t._offset+t._length+b+r:t._offset-b-r:m.l+m.w*(e.position||0)+r%1}for(n=0;n<A.length;n++){var x=(c=A[n])._anchorAxis;c._linepositions={},c._lw=u.crispRound(e,c.linewidth,1),c._mainLinePosition=I(c,x,c.side),c._mainMirrorPosition=c.mirror&&x?I(c,x,f.OPPOSITE_SIDE[c.side]):null}var w=[],M=[],S=[],T=1===l.opacity(v.paper_bgcolor)&&1===l.opacity(v.plot_bgcolor)&&v.paper_bgcolor===v.plot_bgcolor;for(i in v._plots)if((s=v._plots[i]).mainplot)s.bg&&s.bg.remove(),s.bg=void 0;else{var E=s.xaxis.domain,O=s.yaxis.domain,k=s.plotgroup;if(y(E,O,S)){var L=k.node(),D=s.bg=a.ensureSingle(k,"rect","bg");L.insertBefore(D.node(),L.childNodes[0]),M.push(i)}else k.select("rect.bg").remove(),S.push([E,O]),T||(w.push(i),M.push(i))}var P,N,j,R,z,F,B,H,U,V,Y,W,G,X=v._bgLayer.selectAll(".bg").data(w);for(X.enter().append("rect").classed("bg",!0),X.exit().remove(),X.each((function(e){v._plots[e].bg=r.select(this)})),n=0;n<M.length;n++)s=v._plots[M[n]],p=s.xaxis,g=s.yaxis,s.bg&&void 0!==p._offset&&void 0!==g._offset&&s.bg.call(u.setRect,p._offset-b,g._offset-b,p._length+2*b,g._length+2*b).call(l.fill,v.plot_bgcolor).style("stroke-width",0);if(!v._hasOnlyLargeSploms)for(i in v._plots){s=v._plots[i],p=s.xaxis,g=s.yaxis;var q,Z,Q=s.clipId="clip"+v._uid+i+"plot",K=a.ensureSingleById(v._clips,"clipPath",Q,(function(e){e.classed("plotclip",!0).append("rect")}));s.clipRect=K.select("rect").attr({width:p._length,height:g._length}),u.setTranslate(s.plot,p._offset,g._offset),s._hasClipOnAxisFalse?(q=null,Z=Q):(q=Q,Z=null),u.setClipUrl(s.plot,q,e),s.layerClipId=Z}function J(e){return"M"+P+","+e+"H"+N}function $(e){return"M"+p._offset+","+e+"h"+p._length}function ee(e){return"M"+e+","+H+"V"+B}function te(e){return"M"+e+","+g._offset+"v"+g._length}function ne(e,t,n){if(!e.showline||i!==e._mainSubplot)return"";if(!e._anchorAxis)return n(e._mainLinePosition);var r=t(e._mainLinePosition);return e.mirror&&(r+=t(e._mainMirrorPosition)),r}for(i in v._plots){s=v._plots[i],p=s.xaxis,g=s.yaxis;var re="M0,0";C(p,i)&&(z=_(p,"left",g,A),P=p._offset-(z?b+z:0),F=_(p,"right",g,A),N=p._offset+p._length+(F?b+F:0),j=I(p,g,"bottom"),R=I(p,g,"top"),!(G=!p._anchorAxis||i!==p._mainSubplot)||"allticks"!==p.mirror&&"all"!==p.mirror||(p._linepositions[i]=[j,R]),re=ne(p,J,$),G&&p.showline&&("all"===p.mirror||"allticks"===p.mirror)&&(re+=J(j)+J(R)),s.xlines.style("stroke-width",p._lw+"px").call(l.stroke,p.showline?p.linecolor:"rgba(0,0,0,0)")),s.xlines.attr("d",re);var ie="M0,0";C(g,i)&&(Y=_(g,"bottom",p,A),B=g._offset+g._length+(Y?b:0),W=_(g,"top",p,A),H=g._offset-(W?b:0),U=I(g,p,"left"),V=I(g,p,"right"),!(G=!g._anchorAxis||i!==g._mainSubplot)||"allticks"!==g.mirror&&"all"!==g.mirror||(g._linepositions[i]=[U,V]),ie=ne(g,ee,te),G&&g.showline&&("all"===g.mirror||"allticks"===g.mirror)&&(ie+=ee(U)+ee(V)),s.ylines.style("stroke-width",g._lw+"px").call(l.stroke,g.showline?g.linecolor:"rgba(0,0,0,0)")),s.ylines.attr("d",ie)}return h.makeClipPaths(e),o.previousPromises(e)}function C(e,t){return(e.ticks||e.showline)&&(t===e._mainSubplot||"all"===e.mirror||"allticks"===e.mirror)}function A(e,t,n){if(!n.showline||!n._lw)return!1;if("all"===n.mirror||"allticks"===n.mirror)return!0;var r=n._anchorAxis;if(!r)return!1;var i=f.FROM_BL[t];return n.side===t?r.domain[i]===e.domain[i]:n.mirror&&r.domain[1-i]===e.domain[1-i]}function _(e,t,n,r){if(A(e,t,n))return n._lw;for(var i=0;i<r.length;i++){var o=r[i];if(o._mainAxis===n._mainAxis&&A(e,t,o))return o._lw}return 0}function I(e,t){var n=e.title,r=e._size,i=0;switch("start"===t?i=n.pad.l:"end"===t&&(i=-n.pad.r),n.xref){case"paper":return r.l+r.w*n.x+i;case"container":default:return e.width*n.x+i}}function x(e,t){var n=e.title,r=e._size,i=0;if("0em"!==t&&t?t===f.CAP_SHIFT+"em"&&(i=n.pad.t):i=-n.pad.b,"auto"===n.y)return r.t/2;switch(n.yref){case"paper":return r.t+r.h-r.h*n.y+i;case"container":default:return e.height-e.height*n.y+i}}t.layoutStyles=function(e){return a.syncOrAsync([o.doAutoMargin,b],e)},t.drawMainTitle=function(e){var t=e._fullLayout,n=function(e){var t=e.title,n="middle";a.isRightAnchor(t)?n="end":a.isLeftAnchor(t)&&(n="start");return n}(t),r=function(e){var t=e.title,n="0em";a.isTopAnchor(t)?n=f.CAP_SHIFT+"em":a.isMiddleAnchor(t)&&(n=f.MID_SHIFT+"em");return n}(t);c.draw(e,"gtitle",{propContainer:t,propName:"title.text",placeholder:t._dfltTitle.plot,attributes:{x:I(t,n),y:x(t,r),"text-anchor":n,dy:r}})},t.doTraceStyle=function(e){var n,r=e.calcdata,a=[];for(n=0;n<r.length;n++){var l=r[n],u=l[0]||{},c=u.trace||{},d=c._module||{},h=d.arraysToCalcdata;h&&h(l,c);var f=d.editStyle;f&&a.push({fn:f,cd0:u})}if(a.length){for(n=0;n<a.length;n++){var p=a[n];p.fn(e,p.cd0)}s(e),t.redrawReglTraces(e)}return o.style(e),i.getComponentMethod("legend","draw")(e),o.previousPromises(e)},t.doColorBars=function(e){return i.getComponentMethod("colorbar","draw")(e),o.previousPromises(e)},t.layoutReplot=function(e){var t=e.layout;return e.layout=void 0,i.call("plot",e,"",t)},t.doLegend=function(e){return i.getComponentMethod("legend","draw")(e),o.previousPromises(e)},t.doTicksRelayout=function(e){return h.draw(e,"redraw"),e._fullLayout._hasOnlyLargeSploms&&(i.subplotsRegistry.splom.updateGrid(e),s(e),t.redrawReglTraces(e)),t.drawMainTitle(e),o.previousPromises(e)},t.doModeBar=function(e){var t=e._fullLayout;d.manage(e);for(var n=0;n<t._basePlotModules.length;n++){var r=t._basePlotModules[n].updateFx;r&&r(e)}return o.previousPromises(e)},t.doCamera=function(e){for(var t=e._fullLayout,n=t._subplots.gl3d,r=0;r<n.length;r++){var i=t[n[r]];i._scene.setViewport(i)}},t.drawData=function(e){var n=e._fullLayout;s(e);for(var r=n._basePlotModules,a=0;a<r.length;a++)r[a].plot(e);return t.redrawReglTraces(e),o.style(e),i.getComponentMethod("shapes","draw")(e),i.getComponentMethod("annotations","draw")(e),i.getComponentMethod("images","draw")(e),n._replotting=!1,o.previousPromises(e)},t.redrawReglTraces=function(e){var t=e._fullLayout;if(t._has("regl")){var n,r,i=e._fullData,o=[],s=[];for(t._hasOnlyLargeSploms&&t._splomGrid.draw(),n=0;n<i.length;n++){var l=i[n];!0===l.visible&&0!==l._length&&("splom"===l.type?t._splomScenes[l.uid].draw():"scattergl"===l.type?a.pushUnique(o,l.xaxis+l.yaxis):"scatterpolargl"===l.type&&a.pushUnique(s,l.subplot))}for(n=0;n<o.length;n++)(r=t._plots[o[n]])._scene&&r._scene.draw();for(n=0;n<s.length;n++)(r=t[s[n]]._subplot)._scene&&r._scene.draw()}},t.doAutoRangeAndConstraints=function(e){for(var t,n=h.list(e,"",!0),r={},i=0;i<n.length;i++)if(!r[(t=n[i])._id]){r[t._id]=1,v(e,t),m(e,t);var o=t._matchGroup;if(o)for(var a in o){var s=h.getFromId(e,a);m(e,s,t.range),r[a]=1}}g(e)},t.finalDraw=function(e){i.getComponentMethod("rangeslider","draw")(e),i.getComponentMethod("rangeselector","draw")(e)},t.drawMarginPushers=function(e){i.getComponentMethod("legend","draw")(e),i.getComponentMethod("rangeselector","draw")(e),i.getComponentMethod("sliders","draw")(e),i.getComponentMethod("updatemenus","draw")(e),i.getComponentMethod("colorbar","draw")(e)}},function(e,t,n){"use strict";e.exports={undo:{width:857.1,height:1e3,path:"m857 350q0-87-34-166t-91-137-137-92-166-34q-96 0-183 41t-147 114q-4 6-4 13t5 11l76 77q6 5 14 5 9-1 13-7 41-53 100-82t126-29q58 0 110 23t92 61 61 91 22 111-22 111-61 91-92 61-110 23q-55 0-105-20t-90-57l77-77q17-16 8-38-10-23-33-23h-250q-15 0-25 11t-11 25v250q0 24 22 33 22 10 39-8l72-72q60 57 137 88t159 31q87 0 166-34t137-92 91-137 34-166z",transform:"matrix(1 0 0 -1 0 850)"},home:{width:928.6,height:1e3,path:"m786 296v-267q0-15-11-26t-25-10h-214v214h-143v-214h-214q-15 0-25 10t-11 26v267q0 1 0 2t0 2l321 264 321-264q1-1 1-4z m124 39l-34-41q-5-5-12-6h-2q-7 0-12 3l-386 322-386-322q-7-4-13-4-7 2-12 7l-35 41q-4 5-3 13t6 12l401 334q18 15 42 15t43-15l136-114v109q0 8 5 13t13 5h107q8 0 13-5t5-13v-227l122-102q5-5 6-12t-4-13z",transform:"matrix(1 0 0 -1 0 850)"},"camera-retro":{width:1e3,height:1e3,path:"m518 386q0 8-5 13t-13 5q-37 0-63-27t-26-63q0-8 5-13t13-5 12 5 5 13q0 23 16 38t38 16q8 0 13 5t5 13z m125-73q0-59-42-101t-101-42-101 42-42 101 42 101 101 42 101-42 42-101z m-572-320h858v71h-858v-71z m643 320q0 89-62 152t-152 62-151-62-63-152 63-151 151-63 152 63 62 151z m-571 358h214v72h-214v-72z m-72-107h858v143h-462l-36-71h-360v-72z m929 143v-714q0-30-21-51t-50-21h-858q-29 0-50 21t-21 51v714q0 30 21 51t50 21h858q29 0 50-21t21-51z",transform:"matrix(1 0 0 -1 0 850)"},zoombox:{width:1e3,height:1e3,path:"m1000-25l-250 251c40 63 63 138 63 218 0 224-182 406-407 406-224 0-406-182-406-406s183-406 407-406c80 0 155 22 218 62l250-250 125 125z m-812 250l0 438 437 0 0-438-437 0z m62 375l313 0 0-312-313 0 0 312z",transform:"matrix(1 0 0 -1 0 850)"},pan:{width:1e3,height:1e3,path:"m1000 350l-187 188 0-125-250 0 0 250 125 0-188 187-187-187 125 0 0-250-250 0 0 125-188-188 186-187 0 125 252 0 0-250-125 0 187-188 188 188-125 0 0 250 250 0 0-126 187 188z",transform:"matrix(1 0 0 -1 0 850)"},zoom_plus:{width:875,height:1e3,path:"m1 787l0-875 875 0 0 875-875 0z m687-500l-187 0 0-187-125 0 0 187-188 0 0 125 188 0 0 187 125 0 0-187 187 0 0-125z",transform:"matrix(1 0 0 -1 0 850)"},zoom_minus:{width:875,height:1e3,path:"m0 788l0-876 875 0 0 876-875 0z m688-500l-500 0 0 125 500 0 0-125z",transform:"matrix(1 0 0 -1 0 850)"},autoscale:{width:1e3,height:1e3,path:"m250 850l-187 0-63 0 0-62 0-188 63 0 0 188 187 0 0 62z m688 0l-188 0 0-62 188 0 0-188 62 0 0 188 0 62-62 0z m-875-938l0 188-63 0 0-188 0-62 63 0 187 0 0 62-187 0z m875 188l0-188-188 0 0-62 188 0 62 0 0 62 0 188-62 0z m-125 188l-1 0-93-94-156 156 156 156 92-93 2 0 0 250-250 0 0-2 93-92-156-156-156 156 94 92 0 2-250 0 0-250 0 0 93 93 157-156-157-156-93 94 0 0 0-250 250 0 0 0-94 93 156 157 156-157-93-93 0 0 250 0 0 250z",transform:"matrix(1 0 0 -1 0 850)"},tooltip_basic:{width:1500,height:1e3,path:"m375 725l0 0-375-375 375-374 0-1 1125 0 0 750-1125 0z",transform:"matrix(1 0 0 -1 0 850)"},tooltip_compare:{width:1125,height:1e3,path:"m187 786l0 2-187-188 188-187 0 0 937 0 0 373-938 0z m0-499l0 1-187-188 188-188 0 0 937 0 0 376-938-1z",transform:"matrix(1 0 0 -1 0 850)"},plotlylogo:{width:1542,height:1e3,path:"m0-10h182v-140h-182v140z m228 146h183v-286h-183v286z m225 714h182v-1000h-182v1000z m225-285h182v-715h-182v715z m225 142h183v-857h-183v857z m231-428h182v-429h-182v429z m225-291h183v-138h-183v138z",transform:"matrix(1 0 0 -1 0 850)"},"z-axis":{width:1e3,height:1e3,path:"m833 5l-17 108v41l-130-65 130-66c0 0 0 38 0 39 0-1 36-14 39-25 4-15-6-22-16-30-15-12-39-16-56-20-90-22-187-23-279-23-261 0-341 34-353 59 3 60 228 110 228 110-140-8-351-35-351-116 0-120 293-142 474-142 155 0 477 22 477 142 0 50-74 79-163 96z m-374 94c-58-5-99-21-99-40 0-24 65-43 144-43 79 0 143 19 143 43 0 19-42 34-98 40v216h87l-132 135-133-135h88v-216z m167 515h-136v1c16 16 31 34 46 52l84 109v54h-230v-71h124v-1c-16-17-28-32-44-51l-89-114v-51h245v72z",transform:"matrix(1 0 0 -1 0 850)"},"3d_rotate":{width:1e3,height:1e3,path:"m922 660c-5 4-9 7-14 11-359 263-580-31-580-31l-102 28 58-400c0 1 1 1 2 2 118 108 351 249 351 249s-62 27-100 42c88 83 222 183 347 122 16-8 30-17 44-27-2 1-4 2-6 4z m36-329c0 0 64 229-88 296-62 27-124 14-175-11 157-78 225-208 249-266 8-19 11-31 11-31 2 5 6 15 11 32-5-13-8-20-8-20z m-775-239c70-31 117-50 198-32-121 80-199 346-199 346l-96-15-58-12c0 0 55-226 155-287z m603 133l-317-139c0 0 4-4 19-14 7-5 24-15 24-15s-177-147-389 4c235-287 536-112 536-112l31-22 100 299-4-1z m-298-153c6-4 14-9 24-15 0 0-17 10-24 15z",transform:"matrix(1 0 0 -1 0 850)"},camera:{width:1e3,height:1e3,path:"m500 450c-83 0-150-67-150-150 0-83 67-150 150-150 83 0 150 67 150 150 0 83-67 150-150 150z m400 150h-120c-16 0-34 13-39 29l-31 93c-6 15-23 28-40 28h-340c-16 0-34-13-39-28l-31-94c-6-15-23-28-40-28h-120c-55 0-100-45-100-100v-450c0-55 45-100 100-100h800c55 0 100 45 100 100v450c0 55-45 100-100 100z m-400-550c-138 0-250 112-250 250 0 138 112 250 250 250 138 0 250-112 250-250 0-138-112-250-250-250z m365 380c-19 0-35 16-35 35 0 19 16 35 35 35 19 0 35-16 35-35 0-19-16-35-35-35z",transform:"matrix(1 0 0 -1 0 850)"},movie:{width:1e3,height:1e3,path:"m938 413l-188-125c0 37-17 71-44 94 64 38 107 107 107 187 0 121-98 219-219 219-121 0-219-98-219-219 0-61 25-117 66-156h-115c30 33 49 76 49 125 0 103-84 187-187 187s-188-84-188-187c0-57 26-107 65-141-38-22-65-62-65-109v-250c0-70 56-126 125-126h500c69 0 125 56 125 126l188-126c34 0 62 28 62 63v375c0 35-28 63-62 63z m-750 0c-69 0-125 56-125 125s56 125 125 125 125-56 125-125-56-125-125-125z m406-1c-87 0-157 70-157 157 0 86 70 156 157 156s156-70 156-156-70-157-156-157z",transform:"matrix(1 0 0 -1 0 850)"},question:{width:857.1,height:1e3,path:"m500 82v107q0 8-5 13t-13 5h-107q-8 0-13-5t-5-13v-107q0-8 5-13t13-5h107q8 0 13 5t5 13z m143 375q0 49-31 91t-77 65-95 23q-136 0-207-119-9-14 4-24l74-55q4-4 10-4 9 0 14 7 30 38 48 51 19 14 48 14 27 0 48-15t21-33q0-21-11-34t-38-25q-35-16-65-48t-29-70v-20q0-8 5-13t13-5h107q8 0 13 5t5 13q0 10 12 27t30 28q18 10 28 16t25 19 25 27 16 34 7 45z m214-107q0-117-57-215t-156-156-215-58-216 58-155 156-58 215 58 215 155 156 216 58 215-58 156-156 57-215z",transform:"matrix(1 0 0 -1 0 850)"},disk:{width:857.1,height:1e3,path:"m214-7h429v214h-429v-214z m500 0h72v500q0 8-6 21t-11 20l-157 156q-5 6-19 12t-22 5v-232q0-22-15-38t-38-16h-322q-22 0-37 16t-16 38v232h-72v-714h72v232q0 22 16 38t37 16h465q22 0 38-16t15-38v-232z m-214 518v178q0 8-5 13t-13 5h-107q-7 0-13-5t-5-13v-178q0-8 5-13t13-5h107q7 0 13 5t5 13z m357-18v-518q0-22-15-38t-38-16h-750q-23 0-38 16t-16 38v750q0 22 16 38t38 16h517q23 0 50-12t42-26l156-157q16-15 27-42t11-49z",transform:"matrix(1 0 0 -1 0 850)"},drawopenpath:{width:70,height:70,path:"M33.21,85.65a7.31,7.31,0,0,1-2.59-.48c-8.16-3.11-9.27-19.8-9.88-41.3-.1-3.58-.19-6.68-.35-9-.15-2.1-.67-3.48-1.43-3.79-2.13-.88-7.91,2.32-12,5.86L3,32.38c1.87-1.64,11.55-9.66,18.27-6.9,2.13.87,4.75,3.14,5.17,9,.17,2.43.26,5.59.36,9.25a224.17,224.17,0,0,0,1.5,23.4c1.54,10.76,4,12.22,4.48,12.4.84.32,2.79-.46,5.76-3.59L43,80.07C41.53,81.57,37.68,85.64,33.21,85.65ZM74.81,69a11.34,11.34,0,0,0,6.09-6.72L87.26,44.5,74.72,32,56.9,38.35c-2.37.86-5.57,3.42-6.61,6L38.65,72.14l8.42,8.43ZM55,46.27a7.91,7.91,0,0,1,3.64-3.17l14.8-5.3,8,8L76.11,60.6l-.06.19a6.37,6.37,0,0,1-3,3.43L48.25,74.59,44.62,71Zm16.57,7.82A6.9,6.9,0,1,0,64.64,61,6.91,6.91,0,0,0,71.54,54.09Zm-4.05,0a2.85,2.85,0,1,1-2.85-2.85A2.86,2.86,0,0,1,67.49,54.09Zm-4.13,5.22L60.5,56.45,44.26,72.7l2.86,2.86ZM97.83,35.67,84.14,22l-8.57,8.57L89.26,44.24Zm-13.69-8,8,8-2.85,2.85-8-8Z",transform:"matrix(1 0 0 1 -15 -15)"},drawclosedpath:{width:90,height:90,path:"M88.41,21.12a26.56,26.56,0,0,0-36.18,0l-2.07,2-2.07-2a26.57,26.57,0,0,0-36.18,0,23.74,23.74,0,0,0,0,34.8L48,90.12a3.22,3.22,0,0,0,4.42,0l36-34.21a23.73,23.73,0,0,0,0-34.79ZM84,51.24,50.16,83.35,16.35,51.25a17.28,17.28,0,0,1,0-25.47,20,20,0,0,1,27.3,0l4.29,4.07a3.23,3.23,0,0,0,4.44,0l4.29-4.07a20,20,0,0,1,27.3,0,17.27,17.27,0,0,1,0,25.46ZM66.76,47.68h-33v6.91h33ZM53.35,35H46.44V68h6.91Z",transform:"matrix(1 0 0 1 -5 -5)"},lasso:{width:1031,height:1e3,path:"m1018 538c-36 207-290 336-568 286-277-48-473-256-436-463 10-57 36-108 76-151-13-66 11-137 68-183 34-28 75-41 114-42l-55-70 0 0c-2-1-3-2-4-3-10-14-8-34 5-45 14-11 34-8 45 4 1 1 2 3 2 5l0 0 113 140c16 11 31 24 45 40 4 3 6 7 8 11 48-3 100 0 151 9 278 48 473 255 436 462z m-624-379c-80 14-149 48-197 96 42 42 109 47 156 9 33-26 47-66 41-105z m-187-74c-19 16-33 37-39 60 50-32 109-55 174-68-42-25-95-24-135 8z m360 75c-34-7-69-9-102-8 8 62-16 128-68 170-73 59-175 54-244-5-9 20-16 40-20 61-28 159 121 317 333 354s407-60 434-217c28-159-121-318-333-355z",transform:"matrix(1 0 0 -1 0 850)"},selectbox:{width:1e3,height:1e3,path:"m0 850l0-143 143 0 0 143-143 0z m286 0l0-143 143 0 0 143-143 0z m285 0l0-143 143 0 0 143-143 0z m286 0l0-143 143 0 0 143-143 0z m-857-286l0-143 143 0 0 143-143 0z m857 0l0-143 143 0 0 143-143 0z m-857-285l0-143 143 0 0 143-143 0z m857 0l0-143 143 0 0 143-143 0z m-857-286l0-143 143 0 0 143-143 0z m286 0l0-143 143 0 0 143-143 0z m285 0l0-143 143 0 0 143-143 0z m286 0l0-143 143 0 0 143-143 0z",transform:"matrix(1 0 0 -1 0 850)"},drawline:{width:70,height:70,path:"M60.64,62.3a11.29,11.29,0,0,0,6.09-6.72l6.35-17.72L60.54,25.31l-17.82,6.4c-2.36.86-5.57,3.41-6.6,6L24.48,65.5l8.42,8.42ZM40.79,39.63a7.89,7.89,0,0,1,3.65-3.17l14.79-5.31,8,8L61.94,54l-.06.19a6.44,6.44,0,0,1-3,3.43L34.07,68l-3.62-3.63Zm16.57,7.81a6.9,6.9,0,1,0-6.89,6.9A6.9,6.9,0,0,0,57.36,47.44Zm-4,0a2.86,2.86,0,1,1-2.85-2.85A2.86,2.86,0,0,1,53.32,47.44Zm-4.13,5.22L46.33,49.8,30.08,66.05l2.86,2.86ZM83.65,29,70,15.34,61.4,23.9,75.09,37.59ZM70,21.06l8,8-2.84,2.85-8-8ZM87,80.49H10.67V87H87Z",transform:"matrix(1 0 0 1 -15 -15)"},drawrect:{width:80,height:80,path:"M78,22V79H21V22H78m9-9H12V88H87V13ZM68,46.22H31V54H68ZM53,32H45.22V69H53Z",transform:"matrix(1 0 0 1 -10 -10)"},drawcircle:{width:80,height:80,path:"M50,84.72C26.84,84.72,8,69.28,8,50.3S26.84,15.87,50,15.87,92,31.31,92,50.3,73.16,84.72,50,84.72Zm0-60.59c-18.6,0-33.74,11.74-33.74,26.17S31.4,76.46,50,76.46,83.74,64.72,83.74,50.3,68.6,24.13,50,24.13Zm17.15,22h-34v7.11h34Zm-13.8-13H46.24v34h7.11Z",transform:"matrix(1 0 0 1 -10 -10)"},eraseshape:{width:80,height:80,path:"M82.77,78H31.85L6,49.57,31.85,21.14H82.77a8.72,8.72,0,0,1,8.65,8.77V69.24A8.72,8.72,0,0,1,82.77,78ZM35.46,69.84H82.77a.57.57,0,0,0,.49-.6V29.91a.57.57,0,0,0-.49-.61H35.46L17,49.57Zm32.68-34.7-24,24,5,5,24-24Zm-19,.53-5,5,24,24,5-5Z",transform:"matrix(1 0 0 1 -10 -10)"},spikeline:{width:1e3,height:1e3,path:"M512 409c0-57-46-104-103-104-57 0-104 47-104 104 0 57 47 103 104 103 57 0 103-46 103-103z m-327-39l92 0 0 92-92 0z m-185 0l92 0 0 92-92 0z m370-186l92 0 0 93-92 0z m0-184l92 0 0 92-92 0z",transform:"matrix(1.5 0 0 -1.5 0 850)"},pencil:{width:1792,height:1792,path:"M491 1536l91-91-235-235-91 91v107h128v128h107zm523-928q0-22-22-22-10 0-17 7l-542 542q-7 7-7 17 0 22 22 22 10 0 17-7l542-542q7-7 7-17zm-54-192l416 416-832 832h-416v-416zm683 96q0 53-37 90l-166 166-416-416 166-165q36-38 90-38 53 0 91 38l235 234q37 39 37 91z",transform:"matrix(1 0 0 1 0 1)"},newplotlylogo:{name:"newplotlylogo",svg:"<svg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 132 132'><defs><style>.cls-1 {fill: #3f4f75;} .cls-2 {fill: #80cfbe;} .cls-3 {fill: #fff;}</style></defs><title>plotly-logomark</title><g id='symbol'><rect class='cls-1' width='132' height='132' rx='6' ry='6'/><circle class='cls-2' cx='78' cy='54' r='6'/><circle class='cls-2' cx='102' cy='30' r='6'/><circle class='cls-2' cx='78' cy='30' r='6'/><circle class='cls-2' cx='54' cy='30' r='6'/><circle class='cls-2' cx='30' cy='30' r='6'/><circle class='cls-2' cx='30' cy='54' r='6'/><path class='cls-3' d='M30,72a6,6,0,0,0-6,6v24a6,6,0,0,0,12,0V78A6,6,0,0,0,30,72Z'/><path class='cls-3' d='M78,72a6,6,0,0,0-6,6v24a6,6,0,0,0,12,0V78A6,6,0,0,0,78,72Z'/><path class='cls-3' d='M54,48a6,6,0,0,0-6,6v48a6,6,0,0,0,12,0V54A6,6,0,0,0,54,48Z'/><path class='cls-3' d='M102,48a6,6,0,0,0-6,6v48a6,6,0,0,0,12,0V54A6,6,0,0,0,102,48Z'/></g></svg>"}}},function(e,t,n){"use strict";e.exports={CIRCLE_SIDES:32,i000:0,i090:8,i180:16,i270:24,cos45:Math.cos(Math.PI/4),sin45:Math.sin(Math.PI/4),SQRT2:Math.sqrt(2)}},function(e,t,n){"use strict";var r=n(10).strTranslate;function i(e,t){switch(e.type){case"log":return e.p2d(t);case"date":return e.p2r(t,0,e.calendar);default:return e.p2r(t)}}e.exports={p2r:i,r2p:function(e,t){switch(e.type){case"log":return e.d2p(t);case"date":return e.r2p(t,0,e.calendar);default:return e.r2p(t)}},axValue:function(e){var t="y"===e._id.charAt(0)?1:0;return function(n){return i(e,n[t])}},getTransform:function(e){return r(e.xaxis._offset,e.yaxis._offset)}}},function(e,t,n){"use strict";e.exports={segmentRE:/[MLHVQCTSZ][^MLHVQCTSZ]*/g,paramRE:/[^\s,]+/g,paramIsX:{M:{0:!0,drawn:0},L:{0:!0,drawn:0},H:{0:!0,drawn:0},V:{},Q:{0:!0,2:!0,drawn:2},C:{0:!0,2:!0,4:!0,drawn:4},T:{0:!0,drawn:0},S:{0:!0,2:!0,drawn:2},Z:{}},paramIsY:{M:{1:!0,drawn:1},L:{1:!0,drawn:1},H:{},V:{0:!0,drawn:0},Q:{1:!0,3:!0,drawn:3},C:{1:!0,3:!0,5:!0,drawn:5},T:{1:!0,drawn:1},S:{1:!0,3:!0,drawn:5},Z:{}},numParams:{M:2,L:2,H:1,V:1,Q:4,C:6,T:2,S:4,Z:0}}},function(e,t,n){"use strict";var r=n(553),i=n(10);t.rangeToShapePosition=function(e){return"log"===e.type?e.r2d:function(e){return e}},t.shapePositionToRange=function(e){return"log"===e.type?e.d2r:function(e){return e}},t.decodeDate=function(e){return function(t){return t.replace&&(t=t.replace("_"," ")),e(t)}},t.encodeDate=function(e){return function(t){return e(t).replace(" ","_")}},t.extractPathCoords=function(e,t){var n=[];return e.match(r.segmentRE).forEach((function(e){var o=t[e.charAt(0)].drawn;if(void 0!==o){var a=e.substr(1).match(r.paramRE);!a||a.length<o||n.push(i.cleanNumber(a[o]))}})),n},t.getDataToPixel=function(e,n,r,i){var o,a=e._fullLayout._size;if(n)if("domain"===i)o=function(e){return n._length*(r?1-e:e)+n._offset};else{var s=t.shapePositionToRange(n);o=function(e){return n._offset+n.r2p(s(e,!0))},"date"===n.type&&(o=t.decodeDate(o))}else o=r?function(e){return a.t+a.h*(1-e)}:function(e){return a.l+a.w*e};return o},t.getPixelToData=function(e,n,r,i){var o,a=e._fullLayout._size;if(n)if("domain"===i)o=function(e){var t=(e-n._offset)/n._length;return r?1-t:t};else{var s=t.rangeToShapePosition(n);o=function(e){return s(n.p2r(e-n._offset))}}else o=r?function(e){return 1-(e-a.t)/a.h}:function(e){return(e-a.l)/a.w};return o},t.roundPositionForSharpStrokeRendering=function(e,t){var n=1===Math.round(t%2),r=Math.round(e);return n?r+.5:r},t.makeOptionsAndPlotinfo=function(e,t){var n=e._fullLayout.shapes[t]||{},r=e._fullLayout._plots[n.xref+n.yref];return!!r?r._hadPlotinfo=!0:(r={},n.xref&&"paper"!==n.xref&&(r.xaxis=e._fullLayout[n.xref+"axis"]),n.yref&&"paper"!==n.yref&&(r.yaxis=e._fullLayout[n.yref+"axis"])),r.xsizemode=n.xsizemode,r.ysizemode=n.ysizemode,r.xanchor=n.xanchor,r.yanchor=n.yanchor,{options:n,plotinfo:r}}},function(e,t,n){"use strict";var r=n(39),i=n(10),o=n(56),a=n(41),s=n(196),l=/"/g,u=new RegExp('("TOBESTRIPPED)|(TOBESTRIPPED")',"g");e.exports=function(e,t,n){var c,d=e._fullLayout,h=d._paper,f=d._toppaper,p=d.width,g=d.height;h.insert("rect",":first-child").call(o.setRect,0,0,p,g).call(a.fill,d.paper_bgcolor);var v=d._basePlotModules||[];for(c=0;c<v.length;c++){var m=v[c];m.toSVG&&m.toSVG(e)}if(f){var y=f.node().childNodes,b=Array.prototype.slice.call(y);for(c=0;c<b.length;c++){var C=b[c];C.childNodes.length&&h.node().appendChild(C)}}if(d._draggers&&d._draggers.remove(),h.node().style.background="",h.selectAll("text").attr({"data-unformatted":null,"data-math":null}).each((function(){var e=r.select(this);if("hidden"!==this.style.visibility&&"none"!==this.style.display){e.style({visibility:null,display:null});var t=this.style.fontFamily;t&&-1!==t.indexOf('"')&&e.style("font-family",t.replace(l,"TOBESTRIPPED"))}else e.remove()})),d._gradientUrlQueryParts){var A=[];for(var _ in d._gradientUrlQueryParts)A.push(_);A.length&&h.selectAll(A.join(",")).each((function(){var e=r.select(this),t=this.style.fill;t&&-1!==t.indexOf("url(")&&e.style("fill",t.replace(l,"TOBESTRIPPED"));var n=this.style.stroke;n&&-1!==n.indexOf("url(")&&e.style("stroke",n.replace(l,"TOBESTRIPPED"))}))}"pdf"!==t&&"eps"!==t||h.selectAll("#MathJax_SVG_glyphs path").attr("stroke-width",0),h.node().setAttributeNS(s.xmlns,"xmlns",s.svg),h.node().setAttributeNS(s.xmlns,"xmlns:xlink",s.xlink),"svg"===t&&n&&(h.attr("width",n*p),h.attr("height",n*g),h.attr("viewBox","0 0 "+p+" "+g));var I=(new window.XMLSerializer).serializeToString(h.node());return I=function(e){var t=r.select("body").append("div").style({display:"none"}).html(""),n=e.replace(/(&[^;]*;)/gi,(function(e){return"&lt;"===e?"&#60;":"&rt;"===e?"&#62;":-1!==e.indexOf("<")||-1!==e.indexOf(">")?"":t.html(e).text()}));return t.remove(),n}(I),I=(I=I.replace(/&(?!\w+;|\#[0-9]+;| \#x[0-9A-F]+;)/g,"&amp;")).replace(u,"'"),i.isIE()&&(I=(I=(I=I.replace(/"/gi,"'")).replace(/(\('#)([^']*)('\))/gi,'("#$2")')).replace(/(\\')/gi,'"')),I}},function(e,t,n){"use strict";var r=n(10),i=n(540).EventEmitter,o=n(256);e.exports=function(e){var t=e.emitter||new i,n=new Promise((function(i,a){var s=window.Image,l=e.svg,u=e.format||"png";if(r.isIE()&&"svg"!==u){var c=new Error(o.MSG_IE_BAD_FORMAT);return a(c),e.promise?n:t.emit("error",c)}var d,h,f=e.canvas,p=e.scale||1,g=e.width||300,v=e.height||150,m=p*g,y=p*v,b=f.getContext("2d"),C=new s;"svg"===u||r.isIE9orBelow()||r.isSafari()?h=o.encodeSVG(l):(d=o.createBlob(l,"svg"),h=o.createObjectURL(d)),f.width=m,f.height=y,C.onload=function(){var n;switch(d=null,o.revokeObjectURL(h),"svg"!==u&&b.drawImage(C,0,0,m,y),u){case"jpeg":n=f.toDataURL("image/jpeg");break;case"png":n=f.toDataURL("image/png");break;case"webp":n=f.toDataURL("image/webp");break;case"svg":n=h;break;default:var r="Image format is not jpeg, png, svg or webp.";if(a(new Error(r)),!e.promise)return t.emit("error",r)}i(n),e.promise||t.emit("success",n)},C.onerror=function(n){if(d=null,o.revokeObjectURL(h),a(n),!e.promise)return t.emit("error",n)},C.src=h}));return e.promise?n:t}},function(e,t,n){"use strict";var r=n(42),i=n(10),o=i.dateTime2ms,a=i.incrementMonth,s=n(72).ONEAVGMONTH;e.exports=function(e,t,n,i){if("date"!==t.type)return i;var l=e[n+"periodalignment"];if(!l)return i;var u,c=e[n+"period"];if(r(c)){if((c=+c)<=0)return i}else if("string"==typeof c&&"M"===c.charAt(0)){var d=+c.substring(1);if(!(d>0&&Math.round(d)===d))return i;u=d}for(var h=t.calendar,f="start"===l,p="end"===l,g=e[n+"period0"],v=o(g,h)||0,m=[],y=i.length,b=0;b<y;b++){var C,A,_,I=i[b];if(u){for(C=Math.round((I-v)/(u*s)),_=a(v,u*C,h);_>I;)_=a(_,-u,h);for(;_<=I;)_=a(_,u,h);A=a(_,-u,h)}else{for(_=v+(C=Math.round((I-v)/c))*c;_>I;)_-=c;for(;_<=I;)_+=c;A=_-c}m[b]=f?A:p?_:(A+_)/2}return m}},function(e,t,n){"use strict";var r=n(10);e.exports=function(e,t){for(var n=0;n<e.length;n++)e[n].i=n;r.mergeArray(t.text,e,"tx"),r.mergeArray(t.texttemplate,e,"txt"),r.mergeArray(t.hovertext,e,"htx"),r.mergeArray(t.customdata,e,"data"),r.mergeArray(t.textposition,e,"tp"),t.textfont&&(r.mergeArrayCastPositive(t.textfont.size,e,"ts"),r.mergeArray(t.textfont.color,e,"tc"),r.mergeArray(t.textfont.family,e,"tf"));var i=t.marker;if(i){r.mergeArrayCastPositive(i.size,e,"ms"),r.mergeArrayCastPositive(i.opacity,e,"mo"),r.mergeArray(i.symbol,e,"mx"),r.mergeArray(i.color,e,"mc");var o=i.line;i.line&&(r.mergeArray(o.color,e,"mlc"),r.mergeArrayCastPositive(o.width,e,"mlw"));var a=i.gradient;a&&"none"!==a.type&&(r.mergeArray(a.type,e,"mgt"),r.mergeArray(a.color,e,"mgc"))}}},function(e,t,n){"use strict";var r=n(42),i=n(27),o=n(10),a=n(178),s=n(134),l=n(542),u=n(988),c=n(543),d=n(1019),h=n(1735),f=n(300),p=n(95).WEEKDAY_PATTERN,g=n(95).HOUR_PATTERN;function v(e,t,n){function i(n,r){return o.coerce(e,t,s.rangebreaks,n,r)}if(i("enabled")){var a=i("bounds");if(a&&a.length>=2){var l,u,c="";if(2===a.length)for(l=0;l<2;l++)if(u=y(a[l])){c=p;break}var d=i("pattern",c);if(d===p)for(l=0;l<2;l++)(u=y(a[l]))&&(t.bounds[l]=a[l]=u-1);if(d)for(l=0;l<2;l++)switch(u=a[l],d){case p:if(!r(u))return void(t.enabled=!1);if((u=+u)!==Math.floor(u)||u<0||u>=7)return void(t.enabled=!1);t.bounds[l]=a[l]=u;break;case g:if(!r(u))return void(t.enabled=!1);if((u=+u)<0||u>24)return void(t.enabled=!1);t.bounds[l]=a[l]=u}if(!1===n.autorange){var h=n.range;if(h[0]<h[1]){if(a[0]<h[0]&&a[1]>h[1])return void(t.enabled=!1)}else if(a[0]>h[0]&&a[1]<h[1])return void(t.enabled=!1)}}else{var f=i("values");if(!f||!f.length)return void(t.enabled=!1);i("dvalue")}}}e.exports=function(e,t,n,r,g){var m,y=r.letter,b=r.font||{},C=r.splomStash||{},A=n("visible",!r.visibleDflt),_=t._template||{},I=t.type||_.type||"-";"date"===I&&(i.getComponentMethod("calendars","handleDefaults")(e,t,"calendar",r.calendar),r.noTicklabelmode||(m=n("ticklabelmode")));r.noTicklabelposition&&"multicategory"!==I||o.coerce(e,t,{ticklabelposition:{valType:"enumerated",dflt:"outside",values:"period"===m?["outside","inside"]:"x"===y?["outside","inside","outside left","inside left","outside right","inside right"]:["outside","inside","outside top","inside top","outside bottom","inside bottom"]}},"ticklabelposition"),f(t,g);var x=!t.isValidRange(e.range);x&&r.reverseDflt&&(x="reversed"),!n("autorange",x)||"linear"!==I&&"-"!==I||n("rangemode"),n("range"),t.cleanRange(),d(e,t,n,r),"category"===I||r.noHover||n("hoverformat");var w=n("color"),M=w!==s.color.dflt?w:b.color,S=C.label||g._dfltTitle[y];if(c(e,t,n,I,r,{pass:1}),!A)return t;n("title.text",S),o.coerceFont(n,"title.font",{family:b.family,size:Math.round(1.2*b.size),color:M}),l(e,t,n,I),c(e,t,n,I,r,{pass:2}),u(e,t,n,r),h(e,t,n,{dfltColor:w,bgColor:r.bgColor,showGrid:r.showGrid,attributes:s}),(t.showline||t.ticks)&&n("mirror"),r.automargin&&n("automargin");var T,E="multicategory"===I;r.noTickson||"category"!==I&&!E||!t.ticks&&!t.showgrid||(E&&(T="boundaries"),"boundaries"===n("tickson",T)&&delete t.ticklabelposition);E&&(n("showdividers")&&(n("dividercolor"),n("dividerwidth")));if("date"===I)if(a(e,t,{name:"rangebreaks",inclusionAttr:"enabled",handleItemDefaults:v}),t.rangebreaks.length){for(var O=0;O<t.rangebreaks.length;O++)if(t.rangebreaks[O].pattern===p){t._hasDayOfWeekBreaks=!0;break}if(f(t,g),g._has("scattergl")||g._has("splom"))for(var k=0;k<r.data.length;k++){var L=r.data[k];"scattergl"!==L.type&&"splom"!==L.type||(L.visible=!1,o.warn(L.type+" traces do not work on axes with rangebreaks. Setting trace "+L.index+" to `visible: false`."))}}else delete t.rangebreaks;return t};var m={sun:1,mon:2,tue:3,wed:4,thu:5,fri:6,sat:7};function y(e){if("string"==typeof e)return m[e.substr(0,3).toLowerCase()]}},function(e,t,n){"use strict";var r=n(39),i=n(27),o=n(82),a=n(10),s=a.strTranslate,l=n(55),u=n(41),c=n(56),d=n(179),h=n(96),f=n(214),p=n(163),g=n(64).arrayEditor,v=n(1739);function m(e,t){var n=e._fullLayout.annotations[t]||{},r=l.getFromId(e,n.xref),i=l.getFromId(e,n.yref);r&&r.setScale(),i&&i.setScale(),b(e,n,t,!1,r,i)}function y(e,t,n,r,i){var o=i[n],a=i[n+"ref"],s=-1!==n.indexOf("y"),u="domain"===l.getRefType(a),c=s?r.h:r.w;return e?u?o+(s?-t:t)/e._length:e.p2r(e.r2p(o)+t):o+(s?-t:t)/c}function b(e,t,n,o,m,b){var C,A,_=e._fullLayout,I=e._fullLayout._size,x=e._context.edits;o?(C="annotation-"+o,A=o+".annotations"):(C="annotation",A="annotations");var w=g(e.layout,A,t),M=w.modifyBase,S=w.modifyItem,T=w.getUpdateObj;_._infolayer.selectAll("."+C+'[data-index="'+n+'"]').remove();var E="clip"+_._uid+"_ann"+n;if(t._input&&!1!==t.visible){var O={x:{},y:{}},k=+t.textangle||0,L=_._infolayer.append("g").classed(C,!0).attr("data-index",String(n)).style("opacity",t.opacity),D=L.append("g").classed("annotation-text-g",!0),P=x[t.showarrow?"annotationTail":"annotationPosition"],N=t.captureevents||x.annotationText||P,j=D.append("g").style("pointer-events",N?"all":null).call(f,"pointer").on("click",(function(){e._dragging=!1,e.emit("plotly_clickannotation",G(r.event))}));t.hovertext&&j.on("mouseover",(function(){var n=t.hoverlabel,r=n.font,i=this.getBoundingClientRect(),o=e.getBoundingClientRect();d.loneHover({x0:i.left-o.left,x1:i.right-o.left,y:(i.top+i.bottom)/2-o.top,text:t.hovertext,color:n.bgcolor,borderColor:n.bordercolor,fontFamily:r.family,fontSize:r.size,fontColor:r.color},{container:_._hoverlayer.node(),outerContainer:_._paper.node(),gd:e})})).on("mouseout",(function(){d.loneUnhover(_._hoverlayer.node())}));var R=t.borderwidth,z=t.borderpad,F=R+z,B=j.append("rect").attr("class","bg").style("stroke-width",R+"px").call(u.stroke,t.bordercolor).call(u.fill,t.bgcolor),H=t.width||t.height,U=_._topclips.selectAll("#"+E).data(H?[0]:[]);U.enter().append("clipPath").classed("annclip",!0).attr("id",E).append("rect"),U.exit().remove();var V=t.font,Y=_._meta?a.templateString(t.text,_._meta):t.text,W=j.append("text").classed("annotation-text",!0).text(Y);x.annotationText?W.call(h.makeEditable,{delegate:j,gd:e}).call(X).on("edit",(function(n){t.text=n,this.call(X),S("text",n),m&&m.autorange&&M(m._name+".autorange",!0),b&&b.autorange&&M(b._name+".autorange",!0),i.call("_guiRelayout",e,T())})):W.call(X)}else r.selectAll("#"+E).remove();function G(e){var r={index:n,annotation:t._input,fullAnnotation:t,event:e};return o&&(r.subplotId=o),r}function X(n){return n.call(c.font,V).attr({"text-anchor":{left:"start",right:"end"}[t.align]||"middle"}),h.convertToTspans(n,e,q),n}function q(){var n=W.selectAll("a");1===n.size()&&n.text()===W.text()&&j.insert("a",":first-child").attr({"xlink:xlink:href":n.attr("xlink:href"),"xlink:xlink:show":n.attr("xlink:show")}).style({cursor:"pointer"}).node().appendChild(B.node());var r=j.select(".annotation-text-math-group"),d=!r.empty(),g=c.bBox((d?r:W).node()),C=g.width,A=g.height,w=t.width||C,N=t.height||A,z=Math.round(w+2*F),V=Math.round(N+2*F);function Y(e,t){return"auto"===t&&(t=e<1/3?"left":e>2/3?"right":"center"),{center:0,middle:0,left:.5,bottom:-.5,right:-.5,top:.5}[t]}for(var X=!1,q=["x","y"],Z=0;Z<q.length;Z++){var Q,K,J,$,ee,te=q[Z],ne=t[te+"ref"]||te,re=t["a"+te+"ref"],ie={x:m,y:b}[te],oe=(k+("x"===te?0:-90))*Math.PI/180,ae=z*Math.cos(oe),se=V*Math.sin(oe),le=Math.abs(ae)+Math.abs(se),ue=t[te+"anchor"],ce=t[te+"shift"]*("x"===te?1:-1),de=O[te],he=l.getRefType(ne);if(ie&&"domain"!==he){var fe=ie.r2fraction(t[te]);(fe<0||fe>1)&&(re===ne?((fe=ie.r2fraction(t["a"+te]))<0||fe>1)&&(X=!0):X=!0),Q=ie._offset+ie.r2p(t[te]),$=.5}else{var pe="domain"===he;"x"===te?(J=t[te],Q=pe?ie._offset+ie._length*J:Q=I.l+I.w*J):(J=1-t[te],Q=pe?ie._offset+ie._length*J:Q=I.t+I.h*J),$=t.showarrow?.5:J}if(t.showarrow){de.head=Q;var ge=t["a"+te];if(ee=ae*Y(.5,t.xanchor)-se*Y(.5,t.yanchor),re===ne){var ve=l.getRefType(re);"domain"===ve?("y"===te&&(ge=1-ge),de.tail=ie._offset+ie._length*ge):"paper"===ve?"y"===te?(ge=1-ge,de.tail=I.t+I.h*ge):de.tail=I.l+I.w*ge:de.tail=ie._offset+ie.r2p(ge),K=ee}else de.tail=Q+ge,K=ee+ge;de.text=de.tail+ee;var me=_["x"===te?"width":"height"];if("paper"===ne&&(de.head=a.constrain(de.head,1,me-1)),"pixel"===re){var ye=-Math.max(de.tail-3,de.text),be=Math.min(de.tail+3,de.text)-me;ye>0?(de.tail+=ye,de.text+=ye):be>0&&(de.tail-=be,de.text-=be)}de.tail+=ce,de.head+=ce}else K=ee=le*Y($,ue),de.text=Q+ee;de.text+=ce,ee+=ce,K+=ce,t["_"+te+"padplus"]=le/2+K,t["_"+te+"padminus"]=le/2-K,t["_"+te+"size"]=le,t["_"+te+"shift"]=ee}if(X)j.remove();else{var Ce=0,Ae=0;if("left"!==t.align&&(Ce=(w-C)*("center"===t.align?.5:1)),"top"!==t.valign&&(Ae=(N-A)*("middle"===t.valign?.5:1)),d)r.select("svg").attr({x:F+Ce-1,y:F+Ae}).call(c.setClipUrl,H?E:null,e);else{var _e=F+Ae-g.top,Ie=F+Ce-g.left;W.call(h.positionText,Ie,_e).call(c.setClipUrl,H?E:null,e)}U.select("rect").call(c.setRect,F,F,w,N),B.call(c.setRect,R/2,R/2,z-R,V-R),j.call(c.setTranslate,Math.round(O.x.text-z/2),Math.round(O.y.text-V/2)),D.attr({transform:"rotate("+k+","+O.x.text+","+O.y.text+")"});var xe,we=function(n,r){L.selectAll(".annotation-arrow-g").remove();var l=O.x.head,d=O.y.head,h=O.x.tail+n,f=O.y.tail+r,g=O.x.text+n,C=O.y.text+r,A=a.rotationXYMatrix(k,g,C),_=a.apply2DTransform(A),w=a.apply2DTransform2(A),E=+B.attr("width"),P=+B.attr("height"),N=g-.5*E,R=N+E,z=C-.5*P,F=z+P,H=[[N,z,N,F],[N,F,R,F],[R,F,R,z],[R,z,N,z]].map(w);if(!H.reduce((function(e,t){return e^!!a.segmentsIntersect(l,d,l+1e6,d+1e6,t[0],t[1],t[2],t[3])}),!1)){H.forEach((function(e){var t=a.segmentsIntersect(h,f,l,d,e[0],e[1],e[2],e[3]);t&&(h=t.x,f=t.y)}));var U=t.arrowwidth,V=t.arrowcolor,Y=t.arrowside,W=L.append("g").style({opacity:u.opacity(V)}).classed("annotation-arrow-g",!0),G=W.append("path").attr("d","M"+h+","+f+"L"+l+","+d).style("stroke-width",U+"px").call(u.stroke,u.rgb(V));if(v(G,Y,t),x.annotationPosition&&G.node().parentNode&&!o){var X=l,q=d;if(t.standoff){var Z=Math.sqrt(Math.pow(l-h,2)+Math.pow(d-f,2));X+=t.standoff*(h-l)/Z,q+=t.standoff*(f-d)/Z}var Q,K,J=W.append("path").classed("annotation-arrow",!0).classed("anndrag",!0).classed("cursor-move",!0).attr({d:"M3,3H-3V-3H3ZM0,0L"+(h-X)+","+(f-q),transform:s(X,q)}).style("stroke-width",U+6+"px").call(u.stroke,"rgba(0,0,0,0)").call(u.fill,"rgba(0,0,0,0)");p.init({element:J.node(),gd:e,prepFn:function(){var e=c.getTranslate(j);Q=e.x,K=e.y,m&&m.autorange&&M(m._name+".autorange",!0),b&&b.autorange&&M(b._name+".autorange",!0)},moveFn:function(e,n){var r=_(Q,K),i=r[0]+e,o=r[1]+n;j.call(c.setTranslate,i,o),S("x",y(m,e,"x",I,t)),S("y",y(b,n,"y",I,t)),t.axref===t.xref&&S("ax",y(m,e,"ax",I,t)),t.ayref===t.yref&&S("ay",y(b,n,"ay",I,t)),W.attr("transform",s(e,n)),D.attr({transform:"rotate("+k+","+i+","+o+")"})},doneFn:function(){i.call("_guiRelayout",e,T());var t=document.querySelector(".js-notes-box-panel");t&&t.redraw(t.selectedObj)}})}}};if(t.showarrow&&we(0,0),P)p.init({element:j.node(),gd:e,prepFn:function(){xe=D.attr("transform")},moveFn:function(e,n){var r="pointer";if(t.showarrow)t.axref===t.xref?S("ax",y(m,e,"ax",I,t)):S("ax",t.ax+e),t.ayref===t.yref?S("ay",y(b,n,"ay",I.w,t)):S("ay",t.ay+n),we(e,n);else{if(o)return;var i,a;if(m)i=y(m,e,"x",I,t);else{var l=t._xsize/I.w,u=t.x+(t._xshift-t.xshift)/I.w-l/2;i=p.align(u+e/I.w,l,0,1,t.xanchor)}if(b)a=y(b,n,"y",I,t);else{var c=t._ysize/I.h,d=t.y-(t._yshift+t.yshift)/I.h-c/2;a=p.align(d-n/I.h,c,0,1,t.yanchor)}S("x",i),S("y",a),m&&b||(r=p.getCursor(m?.5:i,b?.5:a,t.xanchor,t.yanchor))}D.attr({transform:s(e,n)+xe}),f(j,r)},clickFn:function(n,r){t.captureevents&&e.emit("plotly_clickannotation",G(r))},doneFn:function(){f(j),i.call("_guiRelayout",e,T());var t=document.querySelector(".js-notes-box-panel");t&&t.redraw(t.selectedObj)}})}}}e.exports={draw:function(e){var t=e._fullLayout;t._infolayer.selectAll(".annotation").remove();for(var n=0;n<t.annotations.length;n++)t.annotations[n].visible&&m(e,n);return o.previousPromises(e)},drawOne:m,drawRaw:b}},function(e,t,n){"use strict";e.exports={axisRefDescription:function(e,t,n){return["If set to a",e,"axis id (e.g. *"+e+"* or","*"+e+"2*), the `"+e+"` position refers to a",e,"coordinate. If set to *paper*, the `"+e+"`","position refers to the distance from the",t,"of the plotting","area in normalized coordinates where *0* (*1*) corresponds to the",t,"("+n+"). If set to a",e,"axis ID followed by","*domain* (separated by a space), the position behaves like for","*paper*, but refers to the distance in fractions of the domain","length from the",t,"of the domain of that axis: e.g.,","*"+e+"2 domain* refers to the domain of the second",e," axis and a",e,"position of 0.5 refers to the","point between the",t,"and the",n,"of the domain of the","second",e,"axis."].join(" ")}}},function(e,t,n){"use strict";var r=n(27),i=n(10),o=n(83);e.exports=function(e){return function(t,n){var a=t[e];if(Array.isArray(a))for(var s=r.subplotsRegistry.cartesian,l=s.idRegex,u=n._subplots,c=u.xaxis,d=u.yaxis,h=u.cartesian,f=n._has("cartesian")||n._has("gl2d"),p=0;p<a.length;p++){var g=a[p];if(i.isPlainObject(g)){var v=o.cleanId(g.xref,"x",!1),m=o.cleanId(g.yref,"y",!1),y=l.x.test(v),b=l.y.test(m);if(y||b){f||i.pushUnique(n._basePlotModules,s);var C=!1;y&&-1===c.indexOf(v)&&(c.push(v),C=!0),b&&-1===d.indexOf(m)&&(d.push(m),C=!0),C&&y&&b&&h.push(v+m)}}}}}},function(e,t,n){"use strict";var r=n(392),i=n(124).overrideAll,o=n(64).templatedArray;e.exports=i(o("annotation",{visible:r.visible,x:{valType:"any"},y:{valType:"any"},z:{valType:"any"},ax:{valType:"number"},ay:{valType:"number"},xanchor:r.xanchor,xshift:r.xshift,yanchor:r.yanchor,yshift:r.yshift,text:r.text,textangle:r.textangle,font:r.font,width:r.width,height:r.height,opacity:r.opacity,align:r.align,valign:r.valign,bgcolor:r.bgcolor,bordercolor:r.bordercolor,borderpad:r.borderpad,borderwidth:r.borderwidth,showarrow:r.showarrow,arrowcolor:r.arrowcolor,arrowhead:r.arrowhead,startarrowhead:r.startarrowhead,arrowside:r.arrowside,arrowsize:r.arrowsize,startarrowsize:r.startarrowsize,arrowwidth:r.arrowwidth,standoff:r.standoff,startstandoff:r.startstandoff,hovertext:r.hovertext,hoverlabel:r.hoverlabel,captureevents:r.captureevents}),"calc","from-root")},function(e,t,n){"use strict";e.exports={name:"updatemenus",containerClassName:"updatemenu-container",headerGroupClassName:"updatemenu-header-group",headerClassName:"updatemenu-header",headerArrowClassName:"updatemenu-header-arrow",dropdownButtonGroupClassName:"updatemenu-dropdown-button-group",dropdownButtonClassName:"updatemenu-dropdown-button",buttonClassName:"updatemenu-button",itemRectClassName:"updatemenu-item-rect",itemTextClassName:"updatemenu-item-text",menuIndexAttrName:"updatemenu-active-index",autoMarginIdRoot:"updatemenu-",blankHeaderOpts:{label:"  "},minWidth:30,minHeight:30,textPadX:24,arrowPadX:16,rx:2,ry:2,textOffsetX:12,textOffsetY:3,arrowOffsetX:4,gapButtonHeader:5,gapButton:2,activeColor:"#F4FAFF",hoverColor:"#F4FAFF",arrowSymbol:{left:"◄",right:"►",up:"▲",down:"▼"}}},function(e,t,n){"use strict";var r=n(133);e.exports={bgcolor:{valType:"color",dflt:r.background,editType:"plot"},bordercolor:{valType:"color",dflt:r.defaultLine,editType:"plot"},borderwidth:{valType:"integer",dflt:0,min:0,editType:"plot"},autorange:{valType:"boolean",dflt:!0,editType:"calc",impliedEdits:{"range[0]":void 0,"range[1]":void 0}},range:{valType:"info_array",items:[{valType:"any",editType:"calc",impliedEdits:{"^autorange":!1}},{valType:"any",editType:"calc",impliedEdits:{"^autorange":!1}}],editType:"calc",impliedEdits:{autorange:!1}},thickness:{valType:"number",dflt:.15,min:0,max:1,editType:"plot"},visible:{valType:"boolean",dflt:!0,editType:"calc"},editType:"calc"}},function(e,t,n){"use strict";e.exports={name:"rangeslider",containerClassName:"rangeslider-container",bgClassName:"rangeslider-bg",rangePlotClassName:"rangeslider-rangeplot",maskMinClassName:"rangeslider-mask-min",maskMaxClassName:"rangeslider-mask-max",slideBoxClassName:"rangeslider-slidebox",grabberMinClassName:"rangeslider-grabber-min",grabAreaMinClassName:"rangeslider-grabarea-min",handleMinClassName:"rangeslider-handle-min",grabberMaxClassName:"rangeslider-grabber-max",grabAreaMaxClassName:"rangeslider-grabarea-max",handleMaxClassName:"rangeslider-handle-max",maskMinOppAxisClassName:"rangeslider-mask-min-opp-axis",maskMaxOppAxisClassName:"rangeslider-mask-max-opp-axis",maskColor:"rgba(0,0,0,0.4)",maskOppAxisColor:"rgba(0,0,0,0.2)",slideBoxFill:"transparent",slideBoxCursor:"ew-resize",grabAreaFill:"transparent",grabAreaCursor:"col-resize",grabAreaWidth:10,handleWidth:4,handleRadius:1,handleStrokeWidth:1,extraPad:15}},function(e,t,n){"use strict";var r=n(94),i=n(133),o=(0,n(64).templatedArray)("button",{visible:{valType:"boolean",dflt:!0,editType:"plot"},step:{valType:"enumerated",values:["month","year","day","hour","minute","second","all"],dflt:"month",editType:"plot"},stepmode:{valType:"enumerated",values:["backward","todate"],dflt:"backward",editType:"plot"},count:{valType:"number",min:0,dflt:1,editType:"plot"},label:{valType:"string",editType:"plot"},editType:"plot"});e.exports={visible:{valType:"boolean",editType:"plot"},buttons:o,x:{valType:"number",min:-2,max:3,editType:"plot"},xanchor:{valType:"enumerated",values:["auto","left","center","right"],dflt:"left",editType:"plot"},y:{valType:"number",min:-2,max:3,editType:"plot"},yanchor:{valType:"enumerated",values:["auto","top","middle","bottom"],dflt:"bottom",editType:"plot"},font:r({editType:"plot"}),bgcolor:{valType:"color",dflt:i.lightLine,editType:"plot"},activecolor:{valType:"color",editType:"plot"},bordercolor:{valType:"color",dflt:i.defaultLine,editType:"plot"},borderwidth:{valType:"number",min:0,dflt:0,editType:"plot"},editType:"plot"}},function(e,t,n){"use strict";var r=n(63).extendFlat;t.attributes=function(e,t){t=t||{};var n={valType:"info_array",editType:(e=e||{}).editType,items:[{valType:"number",min:0,max:1,editType:e.editType},{valType:"number",min:0,max:1,editType:e.editType}],dflt:[0,1]},i=(e.name&&e.name,e.trace,t.description&&t.description,{x:r({},n,{}),y:r({},n,{}),editType:e.editType});return e.noGridCell||(i.row={valType:"integer",min:0,dflt:0,editType:e.editType},i.column={valType:"integer",min:0,dflt:0,editType:e.editType}),i},t.defaults=function(e,t,n,r){var i=r&&r.x||[0,1],o=r&&r.y||[0,1],a=t.grid;if(a){var s=n("domain.column");void 0!==s&&(s<a.columns?i=a._domains.x[s]:delete e.domain.column);var l=n("domain.row");void 0!==l&&(l<a.rows?o=a._domains.y[l]:delete e.domain.row)}var u=n("domain.x",i),c=n("domain.y",o);u[0]<u[1]||(e.domain.x=i.slice()),c[0]<c[1]||(e.domain.y=o.slice())}},function(e,t,n){"use strict";var r=n(216),i=n(1035),o=n(112);e.exports=function(e){if(arguments.length<=1)throw new Error("gl-texture2d: Missing arguments for texture2d constructor");a||u(e);if("number"==typeof arguments[1])return m(e,arguments[1],arguments[2],arguments[3]||e.RGBA,arguments[4]||e.UNSIGNED_BYTE);if(Array.isArray(arguments[1]))return m(e,0|arguments[1][0],0|arguments[1][1],arguments[2]||e.RGBA,arguments[3]||e.UNSIGNED_BYTE);if("object"==typeof arguments[1]){var t=arguments[1],n=c(t)?t:t.raw;if(n)return y(e,n,0|t.width,0|t.height,arguments[2]||e.RGBA,arguments[3]||e.UNSIGNED_BYTE);if(t.shape&&t.data&&t.stride)return b(e,t)}throw new Error("gl-texture2d: Invalid arguments for texture2d constructor")};var a=null,s=null,l=null;function u(e){a=[e.LINEAR,e.NEAREST_MIPMAP_LINEAR,e.LINEAR_MIPMAP_NEAREST,e.LINEAR_MIPMAP_NEAREST],s=[e.NEAREST,e.LINEAR,e.NEAREST_MIPMAP_NEAREST,e.NEAREST_MIPMAP_LINEAR,e.LINEAR_MIPMAP_NEAREST,e.LINEAR_MIPMAP_LINEAR],l=[e.REPEAT,e.CLAMP_TO_EDGE,e.MIRRORED_REPEAT]}function c(e){return"undefined"!=typeof HTMLCanvasElement&&e instanceof HTMLCanvasElement||"undefined"!=typeof HTMLImageElement&&e instanceof HTMLImageElement||"undefined"!=typeof HTMLVideoElement&&e instanceof HTMLVideoElement||"undefined"!=typeof ImageData&&e instanceof ImageData}var d=function(e,t){i.muls(e,t,255)};function h(e,t,n){var r=e.gl,i=r.getParameter(r.MAX_TEXTURE_SIZE);if(t<0||t>i||n<0||n>i)throw new Error("gl-texture2d: Invalid texture size");return e._shape=[t,n],e.bind(),r.texImage2D(r.TEXTURE_2D,0,e.format,t,n,0,e.format,e.type,null),e._mipLevels=[0],e}function f(e,t,n,r,i,o){this.gl=e,this.handle=t,this.format=i,this.type=o,this._shape=[n,r],this._mipLevels=[0],this._magFilter=e.NEAREST,this._minFilter=e.NEAREST,this._wrapS=e.CLAMP_TO_EDGE,this._wrapT=e.CLAMP_TO_EDGE,this._anisoSamples=1;var a=this,s=[this._wrapS,this._wrapT];Object.defineProperties(s,[{get:function(){return a._wrapS},set:function(e){return a.wrapS=e}},{get:function(){return a._wrapT},set:function(e){return a.wrapT=e}}]),this._wrapVector=s;var l=[this._shape[0],this._shape[1]];Object.defineProperties(l,[{get:function(){return a._shape[0]},set:function(e){return a.width=e}},{get:function(){return a._shape[1]},set:function(e){return a.height=e}}]),this._shapeVector=l}var p=f.prototype;function g(e,t){return 3===e.length?1===t[2]&&t[1]===e[0]*e[2]&&t[0]===e[2]:1===t[0]&&t[1]===e[0]}function v(e){var t=e.createTexture();return e.bindTexture(e.TEXTURE_2D,t),e.texParameteri(e.TEXTURE_2D,e.TEXTURE_MIN_FILTER,e.NEAREST),e.texParameteri(e.TEXTURE_2D,e.TEXTURE_MAG_FILTER,e.NEAREST),e.texParameteri(e.TEXTURE_2D,e.TEXTURE_WRAP_S,e.CLAMP_TO_EDGE),e.texParameteri(e.TEXTURE_2D,e.TEXTURE_WRAP_T,e.CLAMP_TO_EDGE),t}function m(e,t,n,r,i){var o=e.getParameter(e.MAX_TEXTURE_SIZE);if(t<0||t>o||n<0||n>o)throw new Error("gl-texture2d: Invalid texture shape");if(i===e.FLOAT&&!e.getExtension("OES_texture_float"))throw new Error("gl-texture2d: Floating point textures not supported on this platform");var a=v(e);return e.texImage2D(e.TEXTURE_2D,0,r,t,n,0,r,i,null),new f(e,a,t,n,r,i)}function y(e,t,n,r,i,o){var a=v(e);return e.texImage2D(e.TEXTURE_2D,0,i,i,o,t),new f(e,a,n,r,i,o)}function b(e,t){var n=t.dtype,a=t.shape.slice(),s=e.getParameter(e.MAX_TEXTURE_SIZE);if(a[0]<0||a[0]>s||a[1]<0||a[1]>s)throw new Error("gl-texture2d: Invalid texture size");var l=g(a,t.stride.slice()),u=0;"float32"===n?u=e.FLOAT:"float64"===n?(u=e.FLOAT,l=!1,n="float32"):"uint8"===n?u=e.UNSIGNED_BYTE:(u=e.UNSIGNED_BYTE,l=!1,n="uint8");var c,h,p=0;if(2===a.length)p=e.LUMINANCE,a=[a[0],a[1],1],t=r(t.data,a,[t.stride[0],t.stride[1],1],t.offset);else{if(3!==a.length)throw new Error("gl-texture2d: Invalid shape for texture");if(1===a[2])p=e.ALPHA;else if(2===a[2])p=e.LUMINANCE_ALPHA;else if(3===a[2])p=e.RGB;else{if(4!==a[2])throw new Error("gl-texture2d: Invalid shape for pixel coords");p=e.RGBA}}u!==e.FLOAT||e.getExtension("OES_texture_float")||(u=e.UNSIGNED_BYTE,l=!1);var m=t.size;if(l)c=0===t.offset&&t.data.length===m?t.data:t.data.subarray(t.offset,t.offset+m);else{var y=[a[2],a[2]*a[0],1];h=o.malloc(m,n);var b=r(h,a,y,0);"float32"!==n&&"float64"!==n||u!==e.UNSIGNED_BYTE?i.assign(b,t):d(b,t),c=h.subarray(0,m)}var C=v(e);return e.texImage2D(e.TEXTURE_2D,0,p,a[0],a[1],0,p,u,c),l||o.free(h),new f(e,C,a[0],a[1],p,u)}Object.defineProperties(p,{minFilter:{get:function(){return this._minFilter},set:function(e){this.bind();var t=this.gl;if(this.type===t.FLOAT&&a.indexOf(e)>=0&&(t.getExtension("OES_texture_float_linear")||(e=t.NEAREST)),s.indexOf(e)<0)throw new Error("gl-texture2d: Unknown filter mode "+e);return t.texParameteri(t.TEXTURE_2D,t.TEXTURE_MIN_FILTER,e),this._minFilter=e}},magFilter:{get:function(){return this._magFilter},set:function(e){this.bind();var t=this.gl;if(this.type===t.FLOAT&&a.indexOf(e)>=0&&(t.getExtension("OES_texture_float_linear")||(e=t.NEAREST)),s.indexOf(e)<0)throw new Error("gl-texture2d: Unknown filter mode "+e);return t.texParameteri(t.TEXTURE_2D,t.TEXTURE_MAG_FILTER,e),this._magFilter=e}},mipSamples:{get:function(){return this._anisoSamples},set:function(e){var t=this._anisoSamples;if(this._anisoSamples=0|Math.max(e,1),t!==this._anisoSamples){var n=this.gl.getExtension("EXT_texture_filter_anisotropic");n&&this.gl.texParameterf(this.gl.TEXTURE_2D,n.TEXTURE_MAX_ANISOTROPY_EXT,this._anisoSamples)}return this._anisoSamples}},wrapS:{get:function(){return this._wrapS},set:function(e){if(this.bind(),l.indexOf(e)<0)throw new Error("gl-texture2d: Unknown wrap mode "+e);return this.gl.texParameteri(this.gl.TEXTURE_2D,this.gl.TEXTURE_WRAP_S,e),this._wrapS=e}},wrapT:{get:function(){return this._wrapT},set:function(e){if(this.bind(),l.indexOf(e)<0)throw new Error("gl-texture2d: Unknown wrap mode "+e);return this.gl.texParameteri(this.gl.TEXTURE_2D,this.gl.TEXTURE_WRAP_T,e),this._wrapT=e}},wrap:{get:function(){return this._wrapVector},set:function(e){if(Array.isArray(e)||(e=[e,e]),2!==e.length)throw new Error("gl-texture2d: Must specify wrap mode for rows and columns");for(var t=0;t<2;++t)if(l.indexOf(e[t])<0)throw new Error("gl-texture2d: Unknown wrap mode "+e);this._wrapS=e[0],this._wrapT=e[1];var n=this.gl;return this.bind(),n.texParameteri(n.TEXTURE_2D,n.TEXTURE_WRAP_S,this._wrapS),n.texParameteri(n.TEXTURE_2D,n.TEXTURE_WRAP_T,this._wrapT),e}},shape:{get:function(){return this._shapeVector},set:function(e){if(Array.isArray(e)){if(2!==e.length)throw new Error("gl-texture2d: Invalid texture shape")}else e=[0|e,0|e];return h(this,0|e[0],0|e[1]),[0|e[0],0|e[1]]}},width:{get:function(){return this._shape[0]},set:function(e){return h(this,e|=0,this._shape[1]),e}},height:{get:function(){return this._shape[1]},set:function(e){return e|=0,h(this,this._shape[0],e),e}}}),p.bind=function(e){var t=this.gl;return void 0!==e&&t.activeTexture(t.TEXTURE0+(0|e)),t.bindTexture(t.TEXTURE_2D,this.handle),void 0!==e?0|e:t.getParameter(t.ACTIVE_TEXTURE)-t.TEXTURE0},p.dispose=function(){this.gl.deleteTexture(this.handle)},p.generateMipmap=function(){this.bind(),this.gl.generateMipmap(this.gl.TEXTURE_2D);for(var e=Math.min(this._shape[0],this._shape[1]),t=0;e>0;++t,e>>>=1)this._mipLevels.indexOf(t)<0&&this._mipLevels.push(t)},p.setPixels=function(e,t,n,a){var s=this.gl;this.bind(),Array.isArray(t)?(a=n,n=0|t[1],t=0|t[0]):(t=t||0,n=n||0),a=a||0;var l=c(e)?e:e.raw;if(l){this._mipLevels.indexOf(a)<0?(s.texImage2D(s.TEXTURE_2D,0,this.format,this.format,this.type,l),this._mipLevels.push(a)):s.texSubImage2D(s.TEXTURE_2D,a,t,n,this.format,this.type,l)}else{if(!(e.shape&&e.stride&&e.data))throw new Error("gl-texture2d: Unsupported data type");if(e.shape.length<2||t+e.shape[1]>this._shape[1]>>>a||n+e.shape[0]>this._shape[0]>>>a||t<0||n<0)throw new Error("gl-texture2d: Texture dimensions are out of bounds");!function(e,t,n,a,s,l,u,c){var h=c.dtype,f=c.shape.slice();if(f.length<2||f.length>3)throw new Error("gl-texture2d: Invalid ndarray, must be 2d or 3d");var p=0,v=0,m=g(f,c.stride.slice());"float32"===h?p=e.FLOAT:"float64"===h?(p=e.FLOAT,m=!1,h="float32"):"uint8"===h?p=e.UNSIGNED_BYTE:(p=e.UNSIGNED_BYTE,m=!1,h="uint8");if(2===f.length)v=e.LUMINANCE,f=[f[0],f[1],1],c=r(c.data,f,[c.stride[0],c.stride[1],1],c.offset);else{if(3!==f.length)throw new Error("gl-texture2d: Invalid shape for texture");if(1===f[2])v=e.ALPHA;else if(2===f[2])v=e.LUMINANCE_ALPHA;else if(3===f[2])v=e.RGB;else{if(4!==f[2])throw new Error("gl-texture2d: Invalid shape for pixel coords");v=e.RGBA}f[2]}v!==e.LUMINANCE&&v!==e.ALPHA||s!==e.LUMINANCE&&s!==e.ALPHA||(v=s);if(v!==s)throw new Error("gl-texture2d: Incompatible texture format for setPixels");var y=c.size,b=u.indexOf(a)<0;b&&u.push(a);if(p===l&&m)0===c.offset&&c.data.length===y?b?e.texImage2D(e.TEXTURE_2D,a,s,f[0],f[1],0,s,l,c.data):e.texSubImage2D(e.TEXTURE_2D,a,t,n,f[0],f[1],s,l,c.data):b?e.texImage2D(e.TEXTURE_2D,a,s,f[0],f[1],0,s,l,c.data.subarray(c.offset,c.offset+y)):e.texSubImage2D(e.TEXTURE_2D,a,t,n,f[0],f[1],s,l,c.data.subarray(c.offset,c.offset+y));else{var C;C=l===e.FLOAT?o.mallocFloat32(y):o.mallocUint8(y);var A=r(C,f,[f[2],f[2]*f[0],1]);p===e.FLOAT&&l===e.UNSIGNED_BYTE?d(A,c):i.assign(A,c),b?e.texImage2D(e.TEXTURE_2D,a,s,f[0],f[1],0,s,l,C.subarray(0,y)):e.texSubImage2D(e.TEXTURE_2D,a,t,n,f[0],f[1],s,l,C.subarray(0,y)),l===e.FLOAT?o.freeFloat32(C):o.freeUint8(C)}}(s,t,n,a,this.format,this.type,this._mipLevels,e)}}},function(e,t,n){"use strict";var r=n(1844),i=n(1047),o=n(1048),a=n(1846),s=n(303),l=n(1049);e.exports=function e(t,n){if(r(t))return n?l(t,e(n)):[t[0].clone(),t[1].clone()];var u,c,d=0;if(i(t))u=t.clone();else if("string"==typeof t)u=a(t);else{if(0===t)return[o(0),o(1)];if(t===Math.floor(t))u=o(t);else{for(;t!==Math.floor(t);)t*=Math.pow(2,256),d-=256;u=o(t)}}if(r(n))u.mul(n[1]),c=n[0].clone();else if(i(n))c=n.clone();else if("string"==typeof n)c=a(n);else if(n)if(n===Math.floor(n))c=o(n);else{for(;n!==Math.floor(n);)n*=Math.pow(2,256),d+=256;c=o(n)}else c=o(1);d>0?u=u.ushln(d):d<0&&(c=c.ushln(-d));return s(u,c)}},function(e,t,n){(function(t){var n=!1;if("undefined"!=typeof Float64Array){var r=new Float64Array(1),i=new Uint32Array(r.buffer);if(r[0]=1,n=!0,1072693248===i[1]){e.exports=function(e){return r[0]=e,[i[0],i[1]]},e.exports.pack=function(e,t){return i[0]=e,i[1]=t,r[0]},e.exports.lo=function(e){return r[0]=e,i[0]},e.exports.hi=function(e){return r[0]=e,i[1]}}else if(1072693248===i[0]){e.exports=function(e){return r[0]=e,[i[1],i[0]]},e.exports.pack=function(e,t){return i[1]=e,i[0]=t,r[0]},e.exports.lo=function(e){return r[0]=e,i[1]},e.exports.hi=function(e){return r[0]=e,i[0]}}else n=!1}if(!n){var o=new t(8);e.exports=function(e){return o.writeDoubleLE(e,0,!0),[o.readUInt32LE(0,!0),o.readUInt32LE(4,!0)]},e.exports.pack=function(e,t){return o.writeUInt32LE(e,0,!0),o.writeUInt32LE(t,4,!0),o.readDoubleLE(0,!0)},e.exports.lo=function(e){return o.writeDoubleLE(e,0,!0),o.readUInt32LE(0,!0)},e.exports.hi=function(e){return o.writeDoubleLE(e,0,!0),o.readUInt32LE(4,!0)}}e.exports.sign=function(t){return e.exports.hi(t)>>>31},e.exports.exponent=function(t){return(e.exports.hi(t)<<1>>>21)-1023},e.exports.fraction=function(t){var n=e.exports.lo(t),r=e.exports.hi(t),i=1048575&r;return 2146435072&r&&(i+=1<<20),[n,i]},e.exports.denormalized=function(t){return!(2146435072&e.exports.hi(t))}}).call(this,n(1034).Buffer)},function(e,t,n){"use strict";var r=n(397);e.exports=function(e){return e.cmp(new r(0))}},function(e,t,n){"use strict";var r=n(1055),i=n(394);function o(e,t){this.point=e,this.index=t}function a(e,t){for(var n=e.point,r=t.point,i=n.length,o=0;o<i;++o){var a=r[o]-n[o];if(a)return a}return 0}e.exports=function(e,t){var n=e.length;if(0===n)return[];var s=e[0].length;if(s<1)return[];if(1===s)return function(e,t,n){if(1===e)return n?[[-1,0]]:[];var r=t.map((function(e,t){return[e[0],t]}));r.sort((function(e,t){return e[0]-t[0]}));for(var i=new Array(e-1),o=1;o<e;++o){var a=r[o-1],s=r[o];i[o-1]=[a[1],s[1]]}n&&i.push([-1,i[0][1]],[i[e-1][1],-1]);return i}(n,e,t);for(var l=new Array(n),u=1,c=0;c<n;++c){for(var d=e[c],h=new Array(s+1),f=0,p=0;p<s;++p){var g=d[p];h[p]=g,f+=g*g}h[s]=f,l[c]=new o(h,c),u=Math.max(f,u)}i(l,a),n=l.length;var v=new Array(n+s+1),m=new Array(n+s+1),y=(s+1)*(s+1)*u,b=new Array(s+1);for(c=0;c<=s;++c)b[c]=0;b[s]=y,v[0]=b.slice(),m[0]=-1;for(c=0;c<=s;++c){(h=b.slice())[c]=1,v[c+1]=h,m[c+1]=-1}for(c=0;c<n;++c){var C=l[c];v[c+s+1]=C.point,m[c+s+1]=C.index}var A=r(v,!1);A=t?A.filter((function(e){for(var t=0,n=0;n<=s;++n){var r=m[e[n]];if(r<0&&++t>=2)return!1;e[n]=r}return!0})):A.filter((function(e){for(var t=0;t<=s;++t){var n=m[e[t]];if(n<0)return!1;e[t]=n}return!0}));if(1&s)for(c=0;c<A.length;++c){h=(C=A[c])[0];C[0]=C[1],C[1]=h}return A}},function(e,t,n){"use strict";e.exports={circle:"●","circle-open":"○",square:"■","square-open":"□",diamond:"◆","diamond-open":"◇",cross:"+",x:"❌"}},function(e,t){e.exports=function(e,t){var n=t[0],r=t[1],i=t[2],o=n*n+r*r+i*i;o>0&&(o=1/Math.sqrt(o),e[0]=t[0]*o,e[1]=t[1]*o,e[2]=t[2]*o);return e}},function(e,t,n){"use strict";var r=n(217),i=n(218),o=r(["precision highp float;\n#define GLSLIFY 1\n\nattribute vec3 position;\n\nuniform mat4 model, view, projection;\nuniform vec3 offset, majorAxis, minorAxis, screenAxis;\nuniform float lineWidth;\nuniform vec2 screenShape;\n\nvec3 project(vec3 p) {\n  vec4 pp = projection * view * model * vec4(p, 1.0);\n  return pp.xyz / max(pp.w, 0.0001);\n}\n\nvoid main() {\n  vec3 major = position.x * majorAxis;\n  vec3 minor = position.y * minorAxis;\n\n  vec3 vPosition = major + minor + offset;\n  vec3 pPosition = project(vPosition);\n  vec3 offset = project(vPosition + screenAxis * position.z);\n\n  vec2 screen = normalize((offset - pPosition).xy * screenShape) / screenShape;\n\n  gl_Position = vec4(pPosition + vec3(0.5 * screen * lineWidth, 0), 1.0);\n}\n"]),a=r(["precision highp float;\n#define GLSLIFY 1\n\nuniform vec4 color;\nvoid main() {\n  gl_FragColor = color;\n}"]);t.line=function(e){return i(e,o,a,null,[{name:"position",type:"vec3"}])};var s=r(["precision highp float;\n#define GLSLIFY 1\n\nattribute vec3 position;\n\nuniform mat4 model, view, projection;\nuniform vec3 offset, axis, alignDir, alignOpt;\nuniform float scale, angle, pixelScale;\nuniform vec2 resolution;\n\nvec3 project(vec3 p) {\n  vec4 pp = projection * view * model * vec4(p, 1.0);\n  return pp.xyz / max(pp.w, 0.0001);\n}\n\nfloat computeViewAngle(vec3 a, vec3 b) {\n  vec3 A = project(a);\n  vec3 B = project(b);\n\n  return atan(\n    (B.y - A.y) * resolution.y,\n    (B.x - A.x) * resolution.x\n  );\n}\n\nconst float PI = 3.141592;\nconst float TWO_PI = 2.0 * PI;\nconst float HALF_PI = 0.5 * PI;\nconst float ONE_AND_HALF_PI = 1.5 * PI;\n\nint option = int(floor(alignOpt.x + 0.001));\nfloat hv_ratio =       alignOpt.y;\nbool enableAlign =    (alignOpt.z != 0.0);\n\nfloat mod_angle(float a) {\n  return mod(a, PI);\n}\n\nfloat positive_angle(float a) {\n  return mod_angle((a < 0.0) ?\n    a + TWO_PI :\n    a\n  );\n}\n\nfloat look_upwards(float a) {\n  float b = positive_angle(a);\n  return ((b > HALF_PI) && (b <= ONE_AND_HALF_PI)) ?\n    b - PI :\n    b;\n}\n\nfloat look_horizontal_or_vertical(float a, float ratio) {\n  // ratio controls the ratio between being horizontal to (vertical + horizontal)\n  // if ratio is set to 0.5 then it is 50%, 50%.\n  // when using a higher ratio e.g. 0.75 the result would\n  // likely be more horizontal than vertical.\n\n  float b = positive_angle(a);\n\n  return\n    (b < (      ratio) * HALF_PI) ? 0.0 :\n    (b < (2.0 - ratio) * HALF_PI) ? -HALF_PI :\n    (b < (2.0 + ratio) * HALF_PI) ? 0.0 :\n    (b < (4.0 - ratio) * HALF_PI) ? HALF_PI :\n                                    0.0;\n}\n\nfloat roundTo(float a, float b) {\n  return float(b * floor((a + 0.5 * b) / b));\n}\n\nfloat look_round_n_directions(float a, int n) {\n  float b = positive_angle(a);\n  float div = TWO_PI / float(n);\n  float c = roundTo(b, div);\n  return look_upwards(c);\n}\n\nfloat applyAlignOption(float rawAngle, float delta) {\n  return\n    (option >  2) ? look_round_n_directions(rawAngle + delta, option) :       // option 3-n: round to n directions\n    (option == 2) ? look_horizontal_or_vertical(rawAngle + delta, hv_ratio) : // horizontal or vertical\n    (option == 1) ? rawAngle + delta :       // use free angle, and flip to align with one direction of the axis\n    (option == 0) ? look_upwards(rawAngle) : // use free angle, and stay upwards\n    (option ==-1) ? 0.0 :                    // useful for backward compatibility, all texts remains horizontal\n                    rawAngle;                // otherwise return back raw input angle\n}\n\nbool isAxisTitle = (axis.x == 0.0) &&\n                   (axis.y == 0.0) &&\n                   (axis.z == 0.0);\n\nvoid main() {\n  //Compute world offset\n  float axisDistance = position.z;\n  vec3 dataPosition = axisDistance * axis + offset;\n\n  float beta = angle; // i.e. user defined attributes for each tick\n\n  float axisAngle;\n  float clipAngle;\n  float flip;\n\n  if (enableAlign) {\n    axisAngle = (isAxisTitle) ? HALF_PI :\n                      computeViewAngle(dataPosition, dataPosition + axis);\n    clipAngle = computeViewAngle(dataPosition, dataPosition + alignDir);\n\n    axisAngle += (sin(axisAngle) < 0.0) ? PI : 0.0;\n    clipAngle += (sin(clipAngle) < 0.0) ? PI : 0.0;\n\n    flip = (dot(vec2(cos(axisAngle), sin(axisAngle)),\n                vec2(sin(clipAngle),-cos(clipAngle))) > 0.0) ? 1.0 : 0.0;\n\n    beta += applyAlignOption(clipAngle, flip * PI);\n  }\n\n  //Compute plane offset\n  vec2 planeCoord = position.xy * pixelScale;\n\n  mat2 planeXform = scale * mat2(\n     cos(beta), sin(beta),\n    -sin(beta), cos(beta)\n  );\n\n  vec2 viewOffset = 2.0 * planeXform * planeCoord / resolution;\n\n  //Compute clip position\n  vec3 clipPosition = project(dataPosition);\n\n  //Apply text offset in clip coordinates\n  clipPosition += vec3(viewOffset, 0.0);\n\n  //Done\n  gl_Position = vec4(clipPosition, 1.0);\n}"]),l=r(["precision highp float;\n#define GLSLIFY 1\n\nuniform vec4 color;\nvoid main() {\n  gl_FragColor = color;\n}"]);t.text=function(e){return i(e,s,l,null,[{name:"position",type:"vec3"}])};var u=r(["precision highp float;\n#define GLSLIFY 1\n\nattribute vec3 position;\nattribute vec3 normal;\n\nuniform mat4 model, view, projection;\nuniform vec3 enable;\nuniform vec3 bounds[2];\n\nvarying vec3 colorChannel;\n\nvoid main() {\n\n  vec3 signAxis = sign(bounds[1] - bounds[0]);\n\n  vec3 realNormal = signAxis * normal;\n\n  if(dot(realNormal, enable) > 0.0) {\n    vec3 minRange = min(bounds[0], bounds[1]);\n    vec3 maxRange = max(bounds[0], bounds[1]);\n    vec3 nPosition = mix(minRange, maxRange, 0.5 * (position + 1.0));\n    gl_Position = projection * view * model * vec4(nPosition, 1.0);\n  } else {\n    gl_Position = vec4(0,0,0,0);\n  }\n\n  colorChannel = abs(realNormal);\n}"]),c=r(["precision highp float;\n#define GLSLIFY 1\n\nuniform vec4 colors[3];\n\nvarying vec3 colorChannel;\n\nvoid main() {\n  gl_FragColor = colorChannel.x * colors[0] +\n                 colorChannel.y * colors[1] +\n                 colorChannel.z * colors[2];\n}"]);t.bg=function(e){return i(e,u,c,null,[{name:"position",type:"vec3"},{name:"normal",type:"vec3"}])}},function(e,t,n){"use strict";var r=n(39),i=n(41),o=n(56),a=n(10),s=n(27),l=n(1079).resizeText,u=n(398),c=u.textfont,d=u.insidetextfont,h=u.outsidetextfont,f=n(578);function p(e,t,n){o.pointStyle(e.selectAll("path"),t,n),g(e,t,n)}function g(e,t,n){e.selectAll("text").each((function(e){var i=r.select(this),s=a.ensureUniformFontSize(n,v(i,e,t,n));o.font(i,s)}))}function v(e,t,n,r){var i=r._fullLayout.font,o=n.textfont;if(e.classed("bartext-inside")){var a=A(t,n);o=y(n,t.i,i,a)}else e.classed("bartext-outside")&&(o=b(n,t.i,i));return o}function m(e,t,n){return C(c,e.textfont,t,n)}function y(e,t,n,r){var o=m(e,t,n);return(void 0===e._input.textfont||void 0===e._input.textfont.color||Array.isArray(e.textfont.color)&&void 0===e.textfont.color[t])&&(o={color:i.contrast(r),family:o.family,size:o.size}),C(d,e.insidetextfont,t,o)}function b(e,t,n){var r=m(e,t,n);return C(h,e.outsidetextfont,t,r)}function C(e,t,n,r){t=t||{};var i=f.getValue(t.family,n),o=f.getValue(t.size,n),a=f.getValue(t.color,n);return{family:f.coerceString(e.family,i,r.family),size:f.coerceNumber(e.size,o,r.size),color:f.coerceColor(e.color,a,r.color)}}function A(e,t){return"waterfall"===t.type?t[e.dir].marker.color:e.mc||t.marker.color}e.exports={style:function(e){var t=r.select(e).selectAll("g.barlayer").selectAll("g.trace");l(e,t,"bar");var n=t.size(),i=e._fullLayout;t.style("opacity",(function(e){return e[0].trace.opacity})).each((function(e){("stack"===i.barmode&&n>1||0===i.bargap&&0===i.bargroupgap&&!e[0].trace.marker.line.width)&&r.select(this).attr("shape-rendering","crispEdges")})),t.selectAll("g.points").each((function(t){p(r.select(this),t[0].trace,e)})),s.getComponentMethod("errorbars","style")(t)},styleTextPoints:g,styleOnSelect:function(e,t,n){var i=t[0].trace;i.selectedpoints?function(e,t,n){o.selectedPointStyle(e.selectAll("path"),t),function(e,t,n){e.each((function(e){var i,s=r.select(this);if(e.selected){i=a.ensureUniformFontSize(n,v(s,e,t,n));var l=t.selected.textfont&&t.selected.textfont.color;l&&(i.color=l),o.font(s,i)}else o.selectedTextStyle(s,t)}))}(e.selectAll("text"),t,n)}(n,i,e):(p(n,i,e),s.getComponentMethod("errorbars","style")(n))},getInsideTextFont:y,getOutsideTextFont:b,getBarColor:A,resizeText:l}},function(e,t,n){"use strict";var r=n(42),i=n(123),o=n(10).isArrayOrTypedArray;t.coerceString=function(e,t,n){if("string"==typeof t){if(t||!e.noBlank)return t}else if(("number"==typeof t||!0===t)&&!e.strict)return String(t);return void 0!==n?n:e.dflt},t.coerceNumber=function(e,t,n){if(r(t)){t=+t;var i=e.min,o=e.max;if(!(void 0!==i&&t<i||void 0!==o&&t>o))return t}return void 0!==n?n:e.dflt},t.coerceColor=function(e,t,n){return i(t).isValid()?t:void 0!==n?n:e.dflt},t.coerceEnumerated=function(e,t,n){return e.coerceNumber&&(t=+t),-1!==e.values.indexOf(t)?t:void 0!==n?n:e.dflt},t.getValue=function(e,t){var n;return Array.isArray(e)?t<e.length&&(n=e[t]):n=e,n},t.getLineWidth=function(e,t){return 0<t.mlw?t.mlw:o(e.marker.line.width)?0:e.marker.line.width}},function(e,t,n){"use strict";var r=n(94),i=n(1989),o=n(133),a=r({editType:"calc"});a.family.dflt='"Open Sans", verdana, arial, sans-serif',a.size.dflt=12,a.color.dflt=o.defaultLine,e.exports={carpet:{valType:"string",editType:"calc"},x:{valType:"data_array",editType:"calc+clearAxisTypes"},y:{valType:"data_array",editType:"calc+clearAxisTypes"},a:{valType:"data_array",editType:"calc"},a0:{valType:"number",dflt:0,editType:"calc"},da:{valType:"number",dflt:1,editType:"calc"},b:{valType:"data_array",editType:"calc"},b0:{valType:"number",dflt:0,editType:"calc"},db:{valType:"number",dflt:1,editType:"calc"},cheaterslope:{valType:"number",dflt:1,editType:"calc"},aaxis:i,baxis:i,font:a,color:{valType:"color",dflt:o.defaultLine,editType:"plot"},transforms:void 0}},function(e,t,n){"use strict";e.exports={COMPARISON_OPS:["=","!=","<",">=",">","<="],COMPARISON_OPS2:["=","<",">=",">","<="],INTERVAL_OPS:["[]","()","[)","(]","][",")(","](",")["],SET_OPS:["{}","}{"],CONSTRAINT_REDUCTION:{"=":"=","<":"<","<=":"<",">":">",">=":">","[]":"[]","()":"[]","[)":"[]","(]":"[]","][":"][",")(":"][","](":"][",")[":"]["}}},function(e,t,n){"use strict";e.exports=function(e){return e.end+e.size/1e6}},function(e,t,n){!function(e){var t={pairs:"()[]{}''\"\"",closeBefore:")]}'\":;>",triples:"",explode:"[]{}"},n=e.Pos;function r(e,n){return"pairs"==n&&"string"==typeof e?e:"object"==typeof e&&null!=e[n]?e[n]:t[n]}e.defineOption("autoCloseBrackets",!1,(function(t,n,a){a&&a!=e.Init&&(t.removeKeyMap(i),t.state.closeBrackets=null),n&&(o(r(n,"pairs")),t.state.closeBrackets=n,t.addKeyMap(i))}));var i={Backspace:function(t){var i=s(t);if(!i||t.getOption("disableInput"))return e.Pass;for(var o=r(i,"pairs"),a=t.listSelections(),l=0;l<a.length;l++){if(!a[l].empty())return e.Pass;var c=u(t,a[l].head);if(!c||o.indexOf(c)%2!=0)return e.Pass}for(l=a.length-1;l>=0;l--){var d=a[l].head;t.replaceRange("",n(d.line,d.ch-1),n(d.line,d.ch+1),"+delete")}},Enter:function(t){var n=s(t),i=n&&r(n,"explode");if(!i||t.getOption("disableInput"))return e.Pass;for(var o=t.listSelections(),a=0;a<o.length;a++){if(!o[a].empty())return e.Pass;var c=u(t,o[a].head);if(!c||i.indexOf(c)%2!=0)return e.Pass}t.operation((function(){var e=t.lineSeparator()||"\n";t.replaceSelection(e+e,null),l(t,-1),o=t.listSelections();for(var n=0;n<o.length;n++){var r=o[n].head.line;t.indentLine(r,null,!0),t.indentLine(r+1,null,!0)}}))}};function o(e){for(var t=0;t<e.length;t++){var n=e.charAt(t),r="'"+n+"'";i[r]||(i[r]=a(n))}}function a(t){return function(i){return function(t,i){var o=s(t);if(!o||t.getOption("disableInput"))return e.Pass;var a=r(o,"pairs"),u=a.indexOf(i);if(-1==u)return e.Pass;for(var d,h=r(o,"closeBefore"),f=r(o,"triples"),p=a.charAt(u+1)==i,g=t.listSelections(),v=u%2==0,m=0;m<g.length;m++){var y,b=g[m],C=b.head,A=t.getRange(C,n(C.line,C.ch+1));if(v&&!b.empty())y="surround";else if(!p&&v||A!=i)if(p&&C.ch>1&&f.indexOf(i)>=0&&t.getRange(n(C.line,C.ch-2),C)==i+i){if(C.ch>2&&/\bstring/.test(t.getTokenTypeAt(n(C.line,C.ch-2))))return e.Pass;y="addFour"}else if(p){var _=0==C.ch?" ":t.getRange(n(C.line,C.ch-1),C);if(e.isWordChar(A)||_==i||e.isWordChar(_))return e.Pass;y="both"}else{if(!v||!(0===A.length||/\s/.test(A)||h.indexOf(A)>-1))return e.Pass;y="both"}else y=p&&c(t,C)?"both":f.indexOf(i)>=0&&t.getRange(C,n(C.line,C.ch+3))==i+i+i?"skipThree":"skip";if(d){if(d!=y)return e.Pass}else d=y}var I=u%2?a.charAt(u-1):i,x=u%2?i:a.charAt(u+1);t.operation((function(){if("skip"==d)l(t,1);else if("skipThree"==d)l(t,3);else if("surround"==d){for(var r=t.getSelections(),i=0;i<r.length;i++)r[i]=I+r[i]+x;for(t.replaceSelections(r,"around"),r=t.listSelections().slice(),i=0;i<r.length;i++)r[i]=(o=r[i],a=void 0,a=e.cmpPos(o.anchor,o.head)>0,{anchor:new n(o.anchor.line,o.anchor.ch+(a?-1:1)),head:new n(o.head.line,o.head.ch+(a?1:-1))});t.setSelections(r)}else"both"==d?(t.replaceSelection(I+x,null),t.triggerElectric(I+x),l(t,-1)):"addFour"==d&&(t.replaceSelection(I+I+I+I,"before"),l(t,1));var o,a}))}(i,t)}}function s(e){var t=e.state.closeBrackets;return!t||t.override?t:e.getModeAt(e.getCursor()).closeBrackets||t}function l(e,t){for(var n=[],r=e.listSelections(),i=0,o=0;o<r.length;o++){var a=r[o];a.head==e.getCursor()&&(i=o);var s=a.head.ch||t>0?{line:a.head.line,ch:a.head.ch+t}:{line:a.head.line-1};n.push({anchor:s,head:s})}e.setSelections(n,i)}function u(e,t){var r=e.getRange(n(t.line,t.ch-1),n(t.line,t.ch+1));return 2==r.length?r:null}function c(e,t){var r=e.getTokenAt(n(t.line,t.ch+1));return/\bstring/.test(r.type)&&r.start==t.ch&&(0==t.ch||!/\bstring/.test(e.getTokenTypeAt(t)))}o(t.pairs+"`")}(n(122))},function(e,t,n){!function(e){e.defineOption("autoCloseTags",!1,(function(a,s,l){if(l!=e.Init&&l&&a.removeKeyMap("autoCloseTags"),s){var u={name:"autoCloseTags"};"object"==typeof s&&!1===s.whenClosing||(u["'/'"]=function(t){return function(t){return t.getOption("disableInput")?e.Pass:r(t,!0)}(t)}),"object"==typeof s&&!1===s.whenOpening||(u["'>'"]=function(r){return function(r){if(r.getOption("disableInput"))return e.Pass;for(var a=r.listSelections(),s=[],l=r.getOption("autoCloseTags"),u=0;u<a.length;u++){if(!a[u].empty())return e.Pass;var c=a[u].head,d=r.getTokenAt(c),h=e.innerMode(r.getMode(),d.state),f=h.state,p=h.mode.xmlCurrentTag&&h.mode.xmlCurrentTag(f),g=p&&p.name;if(!g)return e.Pass;var v="html"==h.mode.configuration,m="object"==typeof l&&l.dontCloseTags||v&&t,y="object"==typeof l&&l.indentTags||v&&n;d.end>c.ch&&(g=g.slice(0,g.length-d.end+c.ch));var b=g.toLowerCase();if(!g||"string"==d.type&&(d.end!=c.ch||!/[\"\']/.test(d.string.charAt(d.string.length-1))||1==d.string.length)||"tag"==d.type&&p.close||d.string.indexOf("/")==c.ch-d.start-1||m&&i(m,b)>-1||o(r,h.mode.xmlCurrentContext&&h.mode.xmlCurrentContext(f)||[],g,c,!0))return e.Pass;var C="object"==typeof l&&l.emptyTags;if(C&&i(C,g)>-1)s[u]={text:"/>",newPos:e.Pos(c.line,c.ch+2)};else{var A=y&&i(y,b)>-1;s[u]={indent:A,text:">"+(A?"\n\n":"")+"</"+g+">",newPos:A?e.Pos(c.line+1,0):e.Pos(c.line,c.ch+1)}}}var _="object"==typeof l&&l.dontIndentOnAutoClose;for(u=a.length-1;u>=0;u--){var I=s[u];r.replaceRange(I.text,a[u].head,a[u].anchor,"+insert");var x=r.listSelections().slice(0);x[u]={head:I.newPos,anchor:I.newPos},r.setSelections(x),!_&&I.indent&&(r.indentLine(I.newPos.line,null,!0),r.indentLine(I.newPos.line+1,null,!0))}}(r)}),a.addKeyMap(u)}}));var t=["area","base","br","col","command","embed","hr","img","input","keygen","link","meta","param","source","track","wbr"],n=["applet","blockquote","body","button","div","dl","fieldset","form","frameset","h1","h2","h3","h4","h5","h6","head","html","iframe","layer","legend","object","ol","p","select","table","ul"];function r(t,n){for(var r=t.listSelections(),i=[],a=n?"/":"</",s=t.getOption("autoCloseTags"),l="object"==typeof s&&s.dontIndentOnSlash,u=0;u<r.length;u++){if(!r[u].empty())return e.Pass;var c=r[u].head,d=t.getTokenAt(c),h=e.innerMode(t.getMode(),d.state),f=h.state;if(n&&("string"==d.type||"<"!=d.string.charAt(0)||d.start!=c.ch-1))return e.Pass;var p,g="xml"!=h.mode.name&&"htmlmixed"==t.getMode().name;if(g&&"javascript"==h.mode.name)p=a+"script";else if(g&&"css"==h.mode.name)p=a+"style";else{var v=h.mode.xmlCurrentContext&&h.mode.xmlCurrentContext(f),m=v.length?v[v.length-1]:"";if(!v||v.length&&o(t,v,m,c))return e.Pass;p=a+m}">"!=t.getLine(c.line).charAt(d.end)&&(p+=">"),i[u]=p}if(t.replaceSelections(i),r=t.listSelections(),!l)for(u=0;u<r.length;u++)(u==r.length-1||r[u].head.line<r[u+1].head.line)&&t.indentLine(r[u].head.line)}function i(e,t){if(e.indexOf)return e.indexOf(t);for(var n=0,r=e.length;n<r;++n)if(e[n]==t)return n;return-1}function o(t,n,r,i,o){if(!e.scanForClosingTag)return!1;var a=Math.min(t.lastLine()+1,i.line+500),s=e.scanForClosingTag(t,i,null,a);if(!s||s.tag!=r)return!1;for(var l=o?1:0,u=n.length-1;u>=0&&n[u]==r;u--)++l;for(i=s.to,u=1;u<l;u++){var c=e.scanForClosingTag(t,i,null,a);if(!c||c.tag!=r)return!1;i=c.to}return!0}e.commands.closeTag=function(e){return r(e)}}(n(122),n(2030))},function(e,t,n){!function(e){"use strict";e.registerHelper("fold","brace",(function(t,n){var r,i=n.line,o=t.getLine(i);function a(a){for(var s=n.ch,l=0;;){var u=s<=0?-1:o.lastIndexOf(a,s-1);if(-1!=u){if(1==l&&u<n.ch)break;if(r=t.getTokenTypeAt(e.Pos(i,u+1)),!/^(comment|string)/.test(r))return u+1;s=u-1}else{if(1==l)break;l=1,s=o.length}}}var s,l,u,c=a("{"),d=a("[");if(null!=c&&(null==d||d>c))u=c,s="{",l="}";else{if(null==d)return;u=d,s="[",l="]"}var h,f,p=1,g=t.lastLine();e:for(var v=i;v<=g;++v)for(var m=t.getLine(v),y=v==i?u:0;;){var b=m.indexOf(s,y),C=m.indexOf(l,y);if(b<0&&(b=m.length),C<0&&(C=m.length),(y=Math.min(b,C))==m.length)break;if(t.getTokenTypeAt(e.Pos(v,y+1))==r)if(y==b)++p;else if(!--p){h=v,f=y;break e}++y}if(null!=h&&i!=h)return{from:e.Pos(i,u),to:e.Pos(h,f)}})),e.registerHelper("fold","import",(function(t,n){function r(n){if(n<t.firstLine()||n>t.lastLine())return null;var r=t.getTokenAt(e.Pos(n,1));if(/\S/.test(r.string)||(r=t.getTokenAt(e.Pos(n,r.end+1))),"keyword"!=r.type||"import"!=r.string)return null;for(var i=n,o=Math.min(t.lastLine(),n+10);i<=o;++i){var a=t.getLine(i).indexOf(";");if(-1!=a)return{startCh:r.end,end:e.Pos(i,a)}}}var i,o=n.line,a=r(o);if(!a||r(o-1)||(i=r(o-2))&&i.end.line==o-1)return null;for(var s=a.end;;){var l=r(s.line+1);if(null==l)break;s=l.end}return{from:t.clipPos(e.Pos(o,a.startCh+1)),to:s}})),e.registerHelper("fold","include",(function(t,n){function r(n){if(n<t.firstLine()||n>t.lastLine())return null;var r=t.getTokenAt(e.Pos(n,1));return/\S/.test(r.string)||(r=t.getTokenAt(e.Pos(n,r.end+1))),"meta"==r.type&&"#include"==r.string.slice(0,8)?r.start+8:void 0}var i=n.line,o=r(i);if(null==o||null!=r(i-1))return null;for(var a=i;null!=r(a+1);)++a;return{from:e.Pos(i,o+1),to:t.clipPos(e.Pos(a))}}))}(n(122))},function(e,t,n){!function(e){"use strict";e.registerGlobalHelper("fold","comment",(function(e){return e.blockCommentStart&&e.blockCommentEnd}),(function(t,n){var r=t.getModeAt(n),i=r.blockCommentStart,o=r.blockCommentEnd;if(i&&o){for(var a,s=n.line,l=t.getLine(s),u=n.ch,c=0;;){var d=u<=0?-1:l.lastIndexOf(i,u-1);if(-1!=d){if(1==c&&d<n.ch)return;if(/comment/.test(t.getTokenTypeAt(e.Pos(s,d+1)))&&(0==d||l.slice(d-o.length,d)==o||!/comment/.test(t.getTokenTypeAt(e.Pos(s,d))))){a=d+i.length;break}u=d-1}else{if(1==c)return;c=1,u=l.length}}var h,f,p=1,g=t.lastLine();e:for(var v=s;v<=g;++v)for(var m=t.getLine(v),y=v==s?a:0;;){var b=m.indexOf(i,y),C=m.indexOf(o,y);if(b<0&&(b=m.length),C<0&&(C=m.length),(y=Math.min(b,C))==m.length)break;if(y==b)++p;else if(!--p){h=v,f=y;break e}++y}if(null!=h&&(s!=h||f!=a))return{from:e.Pos(s,a),to:e.Pos(h,f)}}}))}(n(122))},function(e,t,n){!function(e){"use strict";e.defineOption("foldGutter",!1,(function(t,r,i){var o;i&&i!=e.Init&&(t.clearGutter(t.state.foldGutter.options.gutter),t.state.foldGutter=null,t.off("gutterClick",l),t.off("changes",u),t.off("viewportChange",c),t.off("fold",d),t.off("unfold",d),t.off("swapDoc",u)),r&&(t.state.foldGutter=new n((!0===(o=r)&&(o={}),null==o.gutter&&(o.gutter="CodeMirror-foldgutter"),null==o.indicatorOpen&&(o.indicatorOpen="CodeMirror-foldgutter-open"),null==o.indicatorFolded&&(o.indicatorFolded="CodeMirror-foldgutter-folded"),o)),s(t),t.on("gutterClick",l),t.on("changes",u),t.on("viewportChange",c),t.on("fold",d),t.on("unfold",d),t.on("swapDoc",u))}));var t=e.Pos;function n(e){this.options=e,this.from=this.to=0}function r(e,n){for(var r=e.findMarks(t(n,0),t(n+1,0)),i=0;i<r.length;++i)if(r[i].__isFold){var o=r[i].find(-1);if(o&&o.line===n)return r[i]}}function i(e){if("string"==typeof e){var t=document.createElement("div");return t.className=e+" CodeMirror-guttermarker-subtle",t}return e.cloneNode(!0)}function o(e,n,o){var s=e.state.foldGutter.options,l=n-1,u=e.foldOption(s,"minFoldSize"),c=e.foldOption(s,"rangeFinder"),d="string"==typeof s.indicatorFolded&&a(s.indicatorFolded),h="string"==typeof s.indicatorOpen&&a(s.indicatorOpen);e.eachLine(n,o,(function(n){++l;var o=null,a=n.gutterMarkers;if(a&&(a=a[s.gutter]),r(e,l)){if(d&&a&&d.test(a.className))return;o=i(s.indicatorFolded)}else{var f=t(l,0),p=c&&c(e,f);if(p&&p.to.line-p.from.line>=u){if(h&&a&&h.test(a.className))return;o=i(s.indicatorOpen)}}(o||a)&&e.setGutterMarker(n,s.gutter,o)}))}function a(e){return new RegExp("(^|\\s)"+e+"(?:$|\\s)\\s*")}function s(e){var t=e.getViewport(),n=e.state.foldGutter;n&&(e.operation((function(){o(e,t.from,t.to)})),n.from=t.from,n.to=t.to)}function l(e,n,i){var o=e.state.foldGutter;if(o){var a=o.options;if(i==a.gutter){var s=r(e,n);s?s.clear():e.foldCode(t(n,0),a)}}}function u(e){var t=e.state.foldGutter;if(t){var n=t.options;t.from=t.to=0,clearTimeout(t.changeUpdate),t.changeUpdate=setTimeout((function(){s(e)}),n.foldOnChangeTimeSpan||600)}}function c(e){var t=e.state.foldGutter;if(t){var n=t.options;clearTimeout(t.changeUpdate),t.changeUpdate=setTimeout((function(){var n=e.getViewport();t.from==t.to||n.from-t.to>20||t.from-n.to>20?s(e):e.operation((function(){n.from<t.from&&(o(e,n.from,t.from),t.from=n.from),n.to>t.to&&(o(e,t.to,n.to),t.to=n.to)}))}),n.updateViewportTimeSpan||400)}}function d(e,t){var n=e.state.foldGutter;if(n){var r=t.line;r>=n.from&&r<n.to&&o(e,r,r+1)}}}(n(122),n(400))},function(e,t,n){var r=n(2031);"string"==typeof r&&(r=[[e.i,r,""]]);var i={insert:"head",singleton:!1};n(147)(r,i);r.locals&&(e.exports=r.locals)},function(e,t,n){!function(e){"use strict";function t(e,t){if(this.cm=e,this.options=t,this.widget=null,this.debounce=0,this.tick=0,this.startPos=this.cm.getCursor("start"),this.startLen=this.cm.getLine(this.startPos.line).length-this.cm.getSelection().length,this.options.updateOnCursorActivity){var n=this;e.on("cursorActivity",this.activityFunc=function(){n.cursorActivity()})}}e.showHint=function(e,t,n){if(!t)return e.showHint(n);n&&n.async&&(t.async=!0);var r={hint:t};if(n)for(var i in n)r[i]=n[i];return e.showHint(r)},e.defineExtension("showHint",(function(n){n=function(e,t,n){var r=e.options.hintOptions,i={};for(var o in l)i[o]=l[o];if(r)for(var o in r)void 0!==r[o]&&(i[o]=r[o]);if(n)for(var o in n)void 0!==n[o]&&(i[o]=n[o]);return i.hint.resolve&&(i.hint=i.hint.resolve(e,t)),i}(this,this.getCursor("start"),n);var r=this.listSelections();if(!(r.length>1)){if(this.somethingSelected()){if(!n.hint.supportsSelection)return;for(var i=0;i<r.length;i++)if(r[i].head.line!=r[i].anchor.line)return}this.state.completionActive&&this.state.completionActive.close();var o=this.state.completionActive=new t(this,n);o.options.hint&&(e.signal(this,"startCompletion",this),o.update(!0))}})),e.defineExtension("closeHint",(function(){this.state.completionActive&&this.state.completionActive.close()}));var n=window.requestAnimationFrame||function(e){return setTimeout(e,1e3/60)},r=window.cancelAnimationFrame||clearTimeout;function i(e){return"string"==typeof e?e:e.text}function o(e,t){for(;t&&t!=e;){if("LI"===t.nodeName.toUpperCase()&&t.parentNode==e)return t;t=t.parentNode}}function a(t,n){this.completion=t,this.data=n,this.picked=!1;var r=this,a=t.cm,s=a.getInputField().ownerDocument,l=s.defaultView||s.parentWindow,u=this.hints=s.createElement("ul"),c=t.cm.options.theme;u.className="CodeMirror-hints "+c,this.selectedHint=n.selectedHint||0;for(var d=n.list,h=0;h<d.length;++h){var f=u.appendChild(s.createElement("li")),p=d[h],g="CodeMirror-hint"+(h!=this.selectedHint?"":" CodeMirror-hint-active");null!=p.className&&(g=p.className+" "+g),f.className=g,p.render?p.render(f,n,p):f.appendChild(s.createTextNode(p.displayText||i(p))),f.hintId=h}var v=t.options.container||s.body,m=a.cursorCoords(t.options.alignWithWord?n.from:null),y=m.left,b=m.bottom,C=!0,A=0,_=0;if(v!==s.body){var I=-1!==["absolute","relative","fixed"].indexOf(l.getComputedStyle(v).position)?v:v.offsetParent,x=I.getBoundingClientRect(),w=s.body.getBoundingClientRect();A=x.left-w.left-I.scrollLeft,_=x.top-w.top-I.scrollTop}u.style.left=y-A+"px",u.style.top=b-_+"px";var M=l.innerWidth||Math.max(s.body.offsetWidth,s.documentElement.offsetWidth),S=l.innerHeight||Math.max(s.body.offsetHeight,s.documentElement.offsetHeight);v.appendChild(u);var T,E=t.options.moveOnOverlap?u.getBoundingClientRect():new DOMRect,O=!!t.options.paddingForScrollbar&&u.scrollHeight>u.clientHeight+1;if(setTimeout((function(){T=a.getScrollInfo()})),E.bottom-S>0){var k=E.bottom-E.top;if(m.top-(m.bottom-E.top)-k>0)u.style.top=(b=m.top-k-_)+"px",C=!1;else if(k>S){u.style.height=S-5+"px",u.style.top=(b=m.bottom-E.top-_)+"px";var L=a.getCursor();n.from.ch!=L.ch&&(m=a.cursorCoords(L),u.style.left=(y=m.left-A)+"px",E=u.getBoundingClientRect())}}var D,P=E.right-M;if(P>0&&(E.right-E.left>M&&(u.style.width=M-5+"px",P-=E.right-E.left-M),u.style.left=(y=m.left-P-A)+"px"),O)for(var N=u.firstChild;N;N=N.nextSibling)N.style.paddingRight=a.display.nativeBarWidth+"px";a.addKeyMap(this.keyMap=function(e,t){var n={Up:function(){t.moveFocus(-1)},Down:function(){t.moveFocus(1)},PageUp:function(){t.moveFocus(1-t.menuSize(),!0)},PageDown:function(){t.moveFocus(t.menuSize()-1,!0)},Home:function(){t.setFocus(0)},End:function(){t.setFocus(t.length-1)},Enter:t.pick,Tab:t.pick,Esc:t.close};/Mac/.test(navigator.platform)&&(n["Ctrl-P"]=function(){t.moveFocus(-1)},n["Ctrl-N"]=function(){t.moveFocus(1)});var r=e.options.customKeys,i=r?{}:n;function o(e,r){var o;o="string"!=typeof r?function(e){return r(e,t)}:n.hasOwnProperty(r)?n[r]:r,i[e]=o}if(r)for(var a in r)r.hasOwnProperty(a)&&o(a,r[a]);var s=e.options.extraKeys;if(s)for(var a in s)s.hasOwnProperty(a)&&o(a,s[a]);return i}(t,{moveFocus:function(e,t){r.changeActive(r.selectedHint+e,t)},setFocus:function(e){r.changeActive(e)},menuSize:function(){return r.screenAmount()},length:d.length,close:function(){t.close()},pick:function(){r.pick()},data:n})),t.options.closeOnUnfocus&&(a.on("blur",this.onBlur=function(){D=setTimeout((function(){t.close()}),100)}),a.on("focus",this.onFocus=function(){clearTimeout(D)})),a.on("scroll",this.onScroll=function(){var e=a.getScrollInfo(),n=a.getWrapperElement().getBoundingClientRect(),r=b+T.top-e.top,i=r-(l.pageYOffset||(s.documentElement||s.body).scrollTop);if(C||(i+=u.offsetHeight),i<=n.top||i>=n.bottom)return t.close();u.style.top=r+"px",u.style.left=y+T.left-e.left+"px"}),e.on(u,"dblclick",(function(e){var t=o(u,e.target||e.srcElement);t&&null!=t.hintId&&(r.changeActive(t.hintId),r.pick())})),e.on(u,"click",(function(e){var n=o(u,e.target||e.srcElement);n&&null!=n.hintId&&(r.changeActive(n.hintId),t.options.completeOnSingleClick&&r.pick())})),e.on(u,"mousedown",(function(){setTimeout((function(){a.focus()}),20)}));var j=this.getSelectedHintRange();return 0===j.from&&0===j.to||this.scrollToActive(),e.signal(n,"select",d[this.selectedHint],u.childNodes[this.selectedHint]),!0}function s(e,t,n,r){if(e.async)e(t,r,n);else{var i=e(t,n);i&&i.then?i.then(r):r(i)}}t.prototype={close:function(){this.active()&&(this.cm.state.completionActive=null,this.tick=null,this.options.updateOnCursorActivity&&this.cm.off("cursorActivity",this.activityFunc),this.widget&&this.data&&e.signal(this.data,"close"),this.widget&&this.widget.close(),e.signal(this.cm,"endCompletion",this.cm))},active:function(){return this.cm.state.completionActive==this},pick:function(t,n){var r=t.list[n],o=this;this.cm.operation((function(){r.hint?r.hint(o.cm,t,r):o.cm.replaceRange(i(r),r.from||t.from,r.to||t.to,"complete"),e.signal(t,"pick",r),o.cm.scrollIntoView()})),this.options.closeOnPick&&this.close()},cursorActivity:function(){this.debounce&&(r(this.debounce),this.debounce=0);var e=this.startPos;this.data&&(e=this.data.from);var t=this.cm.getCursor(),i=this.cm.getLine(t.line);if(t.line!=this.startPos.line||i.length-t.ch!=this.startLen-this.startPos.ch||t.ch<e.ch||this.cm.somethingSelected()||!t.ch||this.options.closeCharacters.test(i.charAt(t.ch-1)))this.close();else{var o=this;this.debounce=n((function(){o.update()})),this.widget&&this.widget.disable()}},update:function(e){if(null!=this.tick){var t=this,n=++this.tick;s(this.options.hint,this.cm,this.options,(function(r){t.tick==n&&t.finishUpdate(r,e)}))}},finishUpdate:function(t,n){this.data&&e.signal(this.data,"update");var r=this.widget&&this.widget.picked||n&&this.options.completeSingle;this.widget&&this.widget.close(),this.data=t,t&&t.list.length&&(r&&1==t.list.length?this.pick(t,0):(this.widget=new a(this,t),e.signal(t,"shown")))}},a.prototype={close:function(){if(this.completion.widget==this){this.completion.widget=null,this.hints.parentNode.removeChild(this.hints),this.completion.cm.removeKeyMap(this.keyMap);var e=this.completion.cm;this.completion.options.closeOnUnfocus&&(e.off("blur",this.onBlur),e.off("focus",this.onFocus)),e.off("scroll",this.onScroll)}},disable:function(){this.completion.cm.removeKeyMap(this.keyMap);var e=this;this.keyMap={Enter:function(){e.picked=!0}},this.completion.cm.addKeyMap(this.keyMap)},pick:function(){this.completion.pick(this.data,this.selectedHint)},changeActive:function(t,n){if(t>=this.data.list.length?t=n?this.data.list.length-1:0:t<0&&(t=n?0:this.data.list.length-1),this.selectedHint!=t){var r=this.hints.childNodes[this.selectedHint];r&&(r.className=r.className.replace(" CodeMirror-hint-active","")),(r=this.hints.childNodes[this.selectedHint=t]).className+=" CodeMirror-hint-active",this.scrollToActive(),e.signal(this.data,"select",this.data.list[this.selectedHint],r)}},scrollToActive:function(){var e=this.getSelectedHintRange(),t=this.hints.childNodes[e.from],n=this.hints.childNodes[e.to],r=this.hints.firstChild;t.offsetTop<this.hints.scrollTop?this.hints.scrollTop=t.offsetTop-r.offsetTop:n.offsetTop+n.offsetHeight>this.hints.scrollTop+this.hints.clientHeight&&(this.hints.scrollTop=n.offsetTop+n.offsetHeight-this.hints.clientHeight+r.offsetTop)},screenAmount:function(){return Math.floor(this.hints.clientHeight/this.hints.firstChild.offsetHeight)||1},getSelectedHintRange:function(){var e=this.completion.options.scrollMargin||0;return{from:Math.max(0,this.selectedHint-e),to:Math.min(this.data.list.length-1,this.selectedHint+e)}}},e.registerHelper("hint","auto",{resolve:function(t,n){var r,i=t.getHelpers(n,"hint");if(i.length){var o=function(e,t,n){var r=function(e,t){if(!e.somethingSelected())return t;for(var n=[],r=0;r<t.length;r++)t[r].supportsSelection&&n.push(t[r]);return n}(e,i);!function i(o){if(o==r.length)return t(null);s(r[o],e,n,(function(e){e&&e.list.length>0?t(e):i(o+1)}))}(0)};return o.async=!0,o.supportsSelection=!0,o}return(r=t.getHelper(t.getCursor(),"hintWords"))?function(t){return e.hint.fromList(t,{words:r})}:e.hint.anyword?function(t,n){return e.hint.anyword(t,n)}:function(){}}}),e.registerHelper("hint","fromList",(function(t,n){var r,i=t.getCursor(),o=t.getTokenAt(i),a=e.Pos(i.line,o.start),s=i;o.start<i.ch&&/\w/.test(o.string.charAt(i.ch-o.start-1))?r=o.string.substr(0,i.ch-o.start):(r="",a=i);for(var l=[],u=0;u<n.words.length;u++){var c=n.words[u];c.slice(0,r.length)==r&&l.push(c)}if(l.length)return{list:l,from:a,to:s}})),e.commands.autocomplete=e.showHint;var l={hint:e.hint.auto,completeSingle:!0,alignWithWord:!0,closeCharacters:/[\s()\[\]{};:>,]/,closeOnPick:!0,closeOnUnfocus:!0,updateOnCursorActivity:!0,completeOnSingleClick:!0,container:null,customKeys:null,extraKeys:null,paddingForScrollbar:!0,moveOnOverlap:!0};e.defineOption("hintOptions",null)}(n(122))},function(e,t,n){var r=n(2032);"string"==typeof r&&(r=[[e.i,r,""]]);var i={insert:"head",singleton:!1};n(147)(r,i);r.locals&&(e.exports=r.locals)},function(e,t,n){var r=n(2033);"string"==typeof r&&(r=[[e.i,r,""]]);var i={insert:"head",singleton:!1};n(147)(r,i);r.locals&&(e.exports=r.locals)},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.Polygon=void 0;const r=n(402),i=n(67);class o{constructor(e=[]){this.points=e}serialize(){return i.map(this.points,e=>[e.x,e.y])}deserialize(e){this.points=i.map(e,e=>new r.Point(e[0],e[1]))}scale(e,t,n){let o=r.Point.createScaleMatrix(e,t,n);i.forEach(this.points,e=>{e.transform(o)})}transform(e){i.forEach(this.points,t=>{t.transform(e)})}setPoints(e){this.points=e}getPoints(){return this.points}rotate(e){this.transform(r.Point.createRotateMatrix(e/(180/Math.PI),this.getOrigin()))}translate(e,t){i.forEach(this.points,n=>{n.translate(e,t)})}doClone(e){this.points=i.map(e.points,e=>e.clone())}clone(){let e=Object.create(this);return e.doClone(this),e}getOrigin(){if(0===this.points.length)return null;let e=this.getBoundingBox();return r.Point.middlePoint(e.getTopLeft(),e.getBottomRight())}static boundingBoxFromPolygons(e){return o.boundingBoxFromPoints(i.flatMap(e,e=>e.getPoints()))}static boundingBoxFromPoints(e){if(0===e.length)return new a.Rectangle(0,0,0,0);let t=e[0].x,n=e[0].x,i=e[0].y,o=e[0].y;for(let r=1;r<e.length;r++)e[r].x<t&&(t=e[r].x),e[r].x>n&&(n=e[r].x),e[r].y<i&&(i=e[r].y),e[r].y>o&&(o=e[r].y);return new a.Rectangle(new r.Point(t,i),new r.Point(n,i),new r.Point(n,o),new r.Point(t,o))}getBoundingBox(){let e=this.points[0].x,t=this.points[0].x,n=this.points[0].y,i=this.points[0].y;for(let r=1;r<this.points.length;r++)this.points[r].x<e&&(e=this.points[r].x),this.points[r].x>t&&(t=this.points[r].x),this.points[r].y<n&&(n=this.points[r].y),this.points[r].y>i&&(i=this.points[r].y);return new a.Rectangle(new r.Point(e,n),new r.Point(t,n),new r.Point(t,i),new r.Point(e,i))}}t.Polygon=o;const a=n(1099)},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.CanvasModel=void 0;const r=n(67),i=n(593);class o extends i.BaseEntity{constructor(e={}){super(Object.assign({zoom:100,gridSize:0,offsetX:0,offsetY:0},e)),this.layers=[]}getSelectionEntities(){return r.flatMap(this.layers,e=>e.getSelectionEntities())}getSelectedEntities(){return r.filter(this.getSelectionEntities(),e=>e.isSelected())}clearSelection(){r.forEach(this.getSelectedEntities(),e=>{e.setSelected(!1)})}getModels(){return r.flatMap(this.layers,e=>r.values(e.getModels()))}addLayer(e){e.setParent(this),e.registerListener({entityRemoved:e=>{}}),this.layers.push(e)}removeLayer(e){const t=this.layers.indexOf(e);return-1!==t&&(this.layers.splice(t,1),!0)}getLayers(){return this.layers}setGridSize(e=0){this.options.gridSize=e,this.fireEvent({size:e},"gridUpdated")}getGridPosition(e){return 0===this.options.gridSize?e:this.options.gridSize*Math.floor((e+this.options.gridSize/2)/this.options.gridSize)}deserializeModel(e,t){const n={},r={},i={},o={data:e,engine:t,registerModel:e=>{n[e.getID()]=e,i[e.getID()]&&i[e.getID()](e)},getModel:e=>n[e]?Promise.resolve(n[e]):(r[e]||(r[e]=new Promise(t=>{i[e]=t})),r[e])};this.deserialize(o)}deserialize(e){super.deserialize(e),this.options.offsetX=e.data.offsetX,this.options.offsetY=e.data.offsetY,this.options.zoom=e.data.zoom,this.options.gridSize=e.data.gridSize,r.forEach(e.data.layers,t=>{const n=e.engine.getFactoryForLayer(t.type).generateModel({initialConfig:t});n.deserialize(Object.assign(Object.assign({},e),{data:t})),this.addLayer(n)})}serialize(){return Object.assign(Object.assign({},super.serialize()),{offsetX:this.options.offsetX,offsetY:this.options.offsetY,zoom:this.options.zoom,gridSize:this.options.gridSize,layers:r.map(this.layers,e=>e.serialize())})}setZoomLevel(e){this.options.zoom=e,this.fireEvent({zoom:e},"zoomUpdated")}setOffset(e,t){this.options.offsetX=e,this.options.offsetY=t,this.fireEvent({offsetX:e,offsetY:t},"offsetUpdated")}setOffsetX(e){this.setOffset(e,this.options.offsetY)}setOffsetY(e){this.setOffset(this.options.offsetX,e)}getOffsetY(){return this.options.offsetY}getOffsetX(){return this.options.offsetX}getZoomLevel(){return this.options.zoom}}t.CanvasModel=o},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.BaseEntity=void 0;const r=n(401),i=n(67),o=n(306);class a extends o.BaseObserver{constructor(e={}){super(),this.options=Object.assign({id:r.Toolkit.UID()},e)}getOptions(){return this.options}getID(){return this.options.id}doClone(e={},t){}clone(e={}){if(e[this.options.id])return e[this.options.id];let t=i.cloneDeep(this);return t.options=Object.assign(Object.assign({},this.options),{id:r.Toolkit.UID()}),t.clearListeners(),e[this.options.id]=t,this.doClone(e,t),t}clearListeners(){this.listeners={}}deserialize(e){this.options.id=e.data.id,this.options.locked=e.data.locked}serialize(){return{id:this.options.id,locked:this.options.locked}}fireEvent(e,t){super.fireEvent(Object.assign({entity:this},e),t)}isLocked(){return this.options.locked}setLocked(e=!0){this.options.locked=e,this.fireEvent({locked:e},"lockChanged")}}t.BaseEntity=a},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.BaseModel=void 0;const r=n(593),i=n(592);class o extends r.BaseEntity{constructor(e){super(e)}performanceTune(){return!0}getParentCanvasModel(){return this.parent?this.parent instanceof i.CanvasModel?this.parent:this.parent instanceof o?this.parent.getParentCanvasModel():null:null}getParent(){return this.parent}setParent(e){this.parent=e}getSelectionEntities(){return[this]}serialize(){return Object.assign(Object.assign({},super.serialize()),{type:this.options.type,selected:this.options.selected,extras:this.options.extras})}deserialize(e){super.deserialize(e),this.options.extras=e.data.extras,this.options.selected=e.data.selected}getType(){return this.options.type}isSelected(){return this.options.selected}isLocked(){return!!super.isLocked()||!!this.parent&&this.parent.isLocked()}setSelected(e=!0){this.options.selected!==e&&(this.options.selected=e,this.fireEvent({isSelected:e},"selectionChanged"))}remove(){this.fireEvent({},"entityRemoved")}}t.BaseModel=o},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.SelectionLayerModel=void 0;const r=n(1110);class i extends r.LayerModel{constructor(){super({transformed:!1,isSvg:!1,type:"selection"})}setBox(e){this.box=e}getChildModelFactoryBank(){return null}}t.SelectionLayerModel=i},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.LinkModel=void 0;const r=n(597),i=n(67),o=n(151),a=n(69);class s extends a.BaseModel{constructor(e){super(e),this.points=[new r.PointModel({link:this}),new r.PointModel({link:this})],this.sourcePort=null,this.targetPort=null,this.renderedPaths=[],this.labels=[]}getBoundingBox(){return o.Polygon.boundingBoxFromPoints(i.map(this.points,e=>e.getPosition()))}getSelectionEntities(){return this.getTargetPort()&&this.getSourcePort()?super.getSelectionEntities().concat(i.slice(this.points,1,this.points.length-1)):this.getSourcePort()?this.getTargetPort()?super.getSelectionEntities().concat(this.points):super.getSelectionEntities().concat(i.slice(this.points,1,this.points.length)):super.getSelectionEntities().concat(i.slice(this.points,0,this.points.length-1))}deserialize(e){super.deserialize(e),this.points=i.map(e.data.points||[],t=>{var n=new r.PointModel({link:this,position:new o.Point(t.x,t.y)});return n.deserialize(Object.assign(Object.assign({},e),{data:t})),n}),i.forEach(e.data.labels||[],t=>{let n=e.engine.getFactoryForLabel(t.type).generateModel({});n.deserialize(Object.assign(Object.assign({},e),{data:t})),this.addLabel(n)}),e.data.target&&e.getModel(e.data.targetPort).then(e=>{this.setTargetPort(e)}),e.data.source&&e.getModel(e.data.sourcePort).then(e=>{this.setSourcePort(e)})}getRenderedPath(){return this.renderedPaths}setRenderedPaths(e){this.renderedPaths=e}serialize(){return Object.assign(Object.assign({},super.serialize()),{source:this.sourcePort?this.sourcePort.getParent().getID():null,sourcePort:this.sourcePort?this.sourcePort.getID():null,target:this.targetPort?this.targetPort.getParent().getID():null,targetPort:this.targetPort?this.targetPort.getID():null,points:i.map(this.points,e=>e.serialize()),labels:i.map(this.labels,e=>e.serialize())})}doClone(e={},t){t.setPoints(i.map(this.getPoints(),t=>t.clone(e))),this.sourcePort&&t.setSourcePort(this.sourcePort.clone(e)),this.targetPort&&t.setTargetPort(this.targetPort.clone(e))}clearPort(e){this.sourcePort===e?this.setSourcePort(null):this.targetPort===e&&this.setTargetPort(null)}remove(){this.sourcePort&&this.sourcePort.removeLink(this),this.targetPort&&this.targetPort.removeLink(this),super.remove()}isLastPoint(e){return this.getPointIndex(e)===this.points.length-1}getPointIndex(e){return this.points.indexOf(e)}getPointModel(e){for(var t=0;t<this.points.length;t++)if(this.points[t].getID()===e)return this.points[t];return null}getPortForPoint(e){return null!==this.sourcePort&&this.getFirstPoint().getID()===e.getID()?this.sourcePort:null!==this.targetPort&&this.getLastPoint().getID()===e.getID()?this.targetPort:null}getPointForPort(e){return null!==this.sourcePort&&this.sourcePort.getID()===e.getID()?this.getFirstPoint():null!==this.targetPort&&this.targetPort.getID()===e.getID()?this.getLastPoint():null}getFirstPoint(){return this.points[0]}getLastPoint(){return this.points[this.points.length-1]}setSourcePort(e){null!==e&&e.addLink(this),null!==this.sourcePort&&this.sourcePort.removeLink(this),this.sourcePort=e,this.fireEvent({port:e},"sourcePortChanged")}getSourcePort(){return this.sourcePort}getTargetPort(){return this.targetPort}setTargetPort(e){null!==e&&e.addLink(this),null!==this.targetPort&&this.targetPort.removeLink(this),this.targetPort=e,this.fireEvent({port:e},"targetPortChanged")}point(e,t,n=1){return this.addPoint(this.generatePoint(e,t),n)}addLabel(e){e.setParent(this),this.labels.push(e)}getPoints(){return this.points}getLabels(){return this.labels}setPoints(e){i.forEach(e,e=>{e.setParent(this)}),this.points=e}removePoint(e){this.points.splice(this.getPointIndex(e),1)}removePointsBefore(e){this.points.splice(0,this.getPointIndex(e))}removePointsAfter(e){this.points.splice(this.getPointIndex(e)+1)}removeMiddlePoints(){this.points.length>2&&this.points.splice(0,this.points.length-2)}addPoint(e,t=1){return e.setParent(this),this.points.splice(t,0,e),e}generatePoint(e=0,t=0){return new r.PointModel({link:this,position:new o.Point(e,t)})}}t.LinkModel=s},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.PointModel=void 0;const r=n(69);class i extends r.BasePositionModel{constructor(e){super(Object.assign(Object.assign({},e),{type:"point"})),this.parent=e.link}isConnectedToPort(){return null!==this.parent.getPortForPoint(this)}getLink(){return this.getParent()}remove(){this.parent&&this.parent.removePoint(this),super.remove()}isLocked(){return super.isLocked()||this.getParent().isLocked()}}t.PointModel=i},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.NodeLayerModel=void 0;const r=n(69),i=n(405);class o extends r.LayerModel{constructor(){super({type:"diagram-nodes",isSvg:!1,transformed:!0})}addModel(e){if(!(e instanceof i.NodeModel))throw new Error("Can only add nodes to this layer");e.registerListener({entityRemoved:()=>{this.getParent().removeNode(e)}}),super.addModel(e)}getChildModelFactoryBank(e){return e.getNodeFactories()}getNodes(){return this.getModels()}}t.NodeLayerModel=o},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.LinkLayerModel=void 0;const r=n(69),i=n(596);class o extends r.LayerModel{constructor(){super({type:"diagram-links",isSvg:!0,transformed:!0})}addModel(e){if(!(e instanceof i.LinkModel))throw new Error("Can only add links to this layer");e.registerListener({entityRemoved:()=>{this.getParent().removeLink(e)}}),super.addModel(e)}getLinks(){return this.getModels()}getChildModelFactoryBank(e){return e.getLinkFactories()}}t.LinkLayerModel=o},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.DefaultLabelModel=void 0;const r=n(152);class i extends r.LabelModel{constructor(e={}){super(Object.assign({offsetY:null==e.offsetY?-23:e.offsetY,type:"default"},e))}setLabel(e){this.options.label=e}deserialize(e){super.deserialize(e),this.options.label=e.data.label}serialize(){return Object.assign(Object.assign({},super.serialize()),{label:this.options.label})}}t.DefaultLabelModel=i},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.DefaultLinkModel=void 0;const r=n(152),i=n(600),o=n(151);class a extends r.LinkModel{constructor(e={}){super(Object.assign({type:"default",width:e.width||3,color:e.color||"gray",selectedColor:e.selectedColor||"rgb(0,192,255)",curvyness:50},e))}calculateControlOffset(e){return e.getOptions().alignment===r.PortModelAlignment.RIGHT?[this.options.curvyness,0]:e.getOptions().alignment===r.PortModelAlignment.LEFT?[-this.options.curvyness,0]:e.getOptions().alignment===r.PortModelAlignment.TOP?[0,-this.options.curvyness]:[0,this.options.curvyness]}getSVGPath(){if(2==this.points.length){const e=new o.BezierCurve;return e.setSource(this.getFirstPoint().getPosition()),e.setTarget(this.getLastPoint().getPosition()),e.setSourceControl(this.getFirstPoint().getPosition().clone()),e.setTargetControl(this.getLastPoint().getPosition().clone()),this.sourcePort&&e.getSourceControl().translate(...this.calculateControlOffset(this.getSourcePort())),this.targetPort&&e.getTargetControl().translate(...this.calculateControlOffset(this.getTargetPort())),e.getSVGCurve()}}serialize(){return Object.assign(Object.assign({},super.serialize()),{width:this.options.width,color:this.options.color,curvyness:this.options.curvyness,selectedColor:this.options.selectedColor})}deserialize(e){super.deserialize(e),this.options.color=e.data.color,this.options.width=e.data.width,this.options.curvyness=e.data.curvyness,this.options.selectedColor=e.data.selectedColor}addLabel(e){if(e instanceof r.LabelModel)return super.addLabel(e);let t=new i.DefaultLabelModel;return t.setLabel(e),super.addLabel(t)}setWidth(e){this.options.width=e,this.fireEvent({width:e},"widthChanged")}setColor(e){this.options.color=e,this.fireEvent({color:e},"colorChanged")}}t.DefaultLinkModel=a},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.DefaultPortModel=void 0;const r=n(152),i=n(601);class o extends r.PortModel{constructor(e,t,n){t&&(e={in:!!e,name:t,label:n}),e=e,super(Object.assign({label:e.label||e.name,alignment:e.in?r.PortModelAlignment.LEFT:r.PortModelAlignment.RIGHT,type:"default"},e))}deserialize(e){super.deserialize(e),this.options.in=e.data.in,this.options.label=e.data.label}serialize(){return Object.assign(Object.assign({},super.serialize()),{in:this.options.in,label:this.options.label})}link(e,t){let n=this.createLinkModel(t);return n.setSourcePort(this),n.setTargetPort(e),n}canLinkToPort(e){return!(e instanceof o)||this.options.in!==e.getOptions().in}createLinkModel(e){let t=super.createLinkModel();return!t&&e?e.generateModel({}):t||new i.DefaultLinkModel}}t.DefaultPortModel=o},function(e,t){e.exports=function(e,t,n){this.x=e,this.y=t,this.walkable=void 0===n||n}},function(e,t,n){var r=n(407),i=n(219),o=n(307),a=n(126);function s(e){e=e||{},this.allowDiagonal=e.allowDiagonal,this.dontCrossCorners=e.dontCrossCorners,this.heuristic=e.heuristic||o.manhattan,this.weight=e.weight||1,this.diagonalMovement=e.diagonalMovement,this.diagonalMovement||(this.allowDiagonal?this.dontCrossCorners?this.diagonalMovement=a.OnlyWhenNoObstacles:this.diagonalMovement=a.IfAtMostOneObstacle:this.diagonalMovement=a.Never),this.diagonalMovement===a.Never?this.heuristic=e.heuristic||o.manhattan:this.heuristic=e.heuristic||o.octile}s.prototype.findPath=function(e,t,n,o,a){var s,l,u,c,d,h,f,p,g=new r((function(e,t){return e.f-t.f})),v=a.getNodeAt(e,t),m=a.getNodeAt(n,o),y=this.heuristic,b=this.diagonalMovement,C=this.weight,A=Math.abs,_=Math.SQRT2;for(v.g=0,v.f=0,g.push(v),v.opened=!0;!g.empty();){if((s=g.pop()).closed=!0,s===m)return i.backtrace(m);for(c=0,d=(l=a.getNeighbors(s,b)).length;c<d;++c)(u=l[c]).closed||(h=u.x,f=u.y,p=s.g+(h-s.x==0||f-s.y==0?1:_),(!u.opened||p<u.g)&&(u.g=p,u.h=u.h||C*y(A(h-n),A(f-o)),u.f=u.g+u.h,u.parent=s,u.opened?g.updateItem(u):(g.push(u),u.opened=!0)))}return[]},e.exports=s},function(e,t,n){var r=n(407),i=n(219),o=n(307),a=n(126);function s(e){e=e||{},this.allowDiagonal=e.allowDiagonal,this.dontCrossCorners=e.dontCrossCorners,this.diagonalMovement=e.diagonalMovement,this.heuristic=e.heuristic||o.manhattan,this.weight=e.weight||1,this.diagonalMovement||(this.allowDiagonal?this.dontCrossCorners?this.diagonalMovement=a.OnlyWhenNoObstacles:this.diagonalMovement=a.IfAtMostOneObstacle:this.diagonalMovement=a.Never),this.diagonalMovement===a.Never?this.heuristic=e.heuristic||o.manhattan:this.heuristic=e.heuristic||o.octile}s.prototype.findPath=function(e,t,n,o,a){var s,l,u,c,d,h,f,p,g=function(e,t){return e.f-t.f},v=new r(g),m=new r(g),y=a.getNodeAt(e,t),b=a.getNodeAt(n,o),C=this.heuristic,A=this.diagonalMovement,_=this.weight,I=Math.abs,x=Math.SQRT2;for(y.g=0,y.f=0,v.push(y),y.opened=1,b.g=0,b.f=0,m.push(b),b.opened=2;!v.empty()&&!m.empty();){for((s=v.pop()).closed=!0,c=0,d=(l=a.getNeighbors(s,A)).length;c<d;++c)if(!(u=l[c]).closed){if(2===u.opened)return i.biBacktrace(s,u);h=u.x,f=u.y,p=s.g+(h-s.x==0||f-s.y==0?1:x),(!u.opened||p<u.g)&&(u.g=p,u.h=u.h||_*C(I(h-n),I(f-o)),u.f=u.g+u.h,u.parent=s,u.opened?v.updateItem(u):(v.push(u),u.opened=1))}for((s=m.pop()).closed=!0,c=0,d=(l=a.getNeighbors(s,A)).length;c<d;++c)if(!(u=l[c]).closed){if(1===u.opened)return i.biBacktrace(u,s);h=u.x,f=u.y,p=s.g+(h-s.x==0||f-s.y==0?1:x),(!u.opened||p<u.g)&&(u.g=p,u.h=u.h||_*C(I(h-e),I(f-t)),u.f=u.g+u.h,u.parent=s,u.opened?m.updateItem(u):(m.push(u),u.opened=2))}}return[]},e.exports=s},function(e,t,n){"use strict";var r=n(135);e.exports=i;function i(e){this._isDirected=!r.has(e,"directed")||e.directed,this._isMultigraph=!!r.has(e,"multigraph")&&e.multigraph,this._isCompound=!!r.has(e,"compound")&&e.compound,this._label=void 0,this._defaultNodeLabelFn=r.constant(void 0),this._defaultEdgeLabelFn=r.constant(void 0),this._nodes={},this._isCompound&&(this._parent={},this._children={},this._children["\0"]={}),this._in={},this._preds={},this._out={},this._sucs={},this._edgeObjs={},this._edgeLabels={}}function o(e,t){e[t]?e[t]++:e[t]=1}function a(e,t){--e[t]||delete e[t]}function s(e,t,n,i){var o=""+t,a=""+n;if(!e&&o>a){var s=o;o=a,a=s}return o+""+a+""+(r.isUndefined(i)?"\0":i)}function l(e,t,n,r){var i=""+t,o=""+n;if(!e&&i>o){var a=i;i=o,o=a}var s={v:i,w:o};return r&&(s.name=r),s}function u(e,t){return s(e,t.v,t.w,t.name)}i.prototype._nodeCount=0,i.prototype._edgeCount=0,i.prototype.isDirected=function(){return this._isDirected},i.prototype.isMultigraph=function(){return this._isMultigraph},i.prototype.isCompound=function(){return this._isCompound},i.prototype.setGraph=function(e){return this._label=e,this},i.prototype.graph=function(){return this._label},i.prototype.setDefaultNodeLabel=function(e){return r.isFunction(e)||(e=r.constant(e)),this._defaultNodeLabelFn=e,this},i.prototype.nodeCount=function(){return this._nodeCount},i.prototype.nodes=function(){return r.keys(this._nodes)},i.prototype.sources=function(){var e=this;return r.filter(this.nodes(),(function(t){return r.isEmpty(e._in[t])}))},i.prototype.sinks=function(){var e=this;return r.filter(this.nodes(),(function(t){return r.isEmpty(e._out[t])}))},i.prototype.setNodes=function(e,t){var n=arguments,i=this;return r.each(e,(function(e){n.length>1?i.setNode(e,t):i.setNode(e)})),this},i.prototype.setNode=function(e,t){return r.has(this._nodes,e)?(arguments.length>1&&(this._nodes[e]=t),this):(this._nodes[e]=arguments.length>1?t:this._defaultNodeLabelFn(e),this._isCompound&&(this._parent[e]="\0",this._children[e]={},this._children["\0"][e]=!0),this._in[e]={},this._preds[e]={},this._out[e]={},this._sucs[e]={},++this._nodeCount,this)},i.prototype.node=function(e){return this._nodes[e]},i.prototype.hasNode=function(e){return r.has(this._nodes,e)},i.prototype.removeNode=function(e){var t=this;if(r.has(this._nodes,e)){var n=function(e){t.removeEdge(t._edgeObjs[e])};delete this._nodes[e],this._isCompound&&(this._removeFromParentsChildList(e),delete this._parent[e],r.each(this.children(e),(function(e){t.setParent(e)})),delete this._children[e]),r.each(r.keys(this._in[e]),n),delete this._in[e],delete this._preds[e],r.each(r.keys(this._out[e]),n),delete this._out[e],delete this._sucs[e],--this._nodeCount}return this},i.prototype.setParent=function(e,t){if(!this._isCompound)throw new Error("Cannot set parent in a non-compound graph");if(r.isUndefined(t))t="\0";else{for(var n=t+="";!r.isUndefined(n);n=this.parent(n))if(n===e)throw new Error("Setting "+t+" as parent of "+e+" would create a cycle");this.setNode(t)}return this.setNode(e),this._removeFromParentsChildList(e),this._parent[e]=t,this._children[t][e]=!0,this},i.prototype._removeFromParentsChildList=function(e){delete this._children[this._parent[e]][e]},i.prototype.parent=function(e){if(this._isCompound){var t=this._parent[e];if("\0"!==t)return t}},i.prototype.children=function(e){if(r.isUndefined(e)&&(e="\0"),this._isCompound){var t=this._children[e];if(t)return r.keys(t)}else{if("\0"===e)return this.nodes();if(this.hasNode(e))return[]}},i.prototype.predecessors=function(e){var t=this._preds[e];if(t)return r.keys(t)},i.prototype.successors=function(e){var t=this._sucs[e];if(t)return r.keys(t)},i.prototype.neighbors=function(e){var t=this.predecessors(e);if(t)return r.union(t,this.successors(e))},i.prototype.isLeaf=function(e){return 0===(this.isDirected()?this.successors(e):this.neighbors(e)).length},i.prototype.filterNodes=function(e){var t=new this.constructor({directed:this._isDirected,multigraph:this._isMultigraph,compound:this._isCompound});t.setGraph(this.graph());var n=this;r.each(this._nodes,(function(n,r){e(r)&&t.setNode(r,n)})),r.each(this._edgeObjs,(function(e){t.hasNode(e.v)&&t.hasNode(e.w)&&t.setEdge(e,n.edge(e))}));var i={};return this._isCompound&&r.each(t.nodes(),(function(e){t.setParent(e,function e(r){var o=n.parent(r);return void 0===o||t.hasNode(o)?(i[r]=o,o):o in i?i[o]:e(o)}(e))})),t},i.prototype.setDefaultEdgeLabel=function(e){return r.isFunction(e)||(e=r.constant(e)),this._defaultEdgeLabelFn=e,this},i.prototype.edgeCount=function(){return this._edgeCount},i.prototype.edges=function(){return r.values(this._edgeObjs)},i.prototype.setPath=function(e,t){var n=this,i=arguments;return r.reduce(e,(function(e,r){return i.length>1?n.setEdge(e,r,t):n.setEdge(e,r),r})),this},i.prototype.setEdge=function(){var e,t,n,i,a=!1,u=arguments[0];"object"==typeof u&&null!==u&&"v"in u?(e=u.v,t=u.w,n=u.name,2===arguments.length&&(i=arguments[1],a=!0)):(e=u,t=arguments[1],n=arguments[3],arguments.length>2&&(i=arguments[2],a=!0)),e=""+e,t=""+t,r.isUndefined(n)||(n=""+n);var c=s(this._isDirected,e,t,n);if(r.has(this._edgeLabels,c))return a&&(this._edgeLabels[c]=i),this;if(!r.isUndefined(n)&&!this._isMultigraph)throw new Error("Cannot set a named edge when isMultigraph = false");this.setNode(e),this.setNode(t),this._edgeLabels[c]=a?i:this._defaultEdgeLabelFn(e,t,n);var d=l(this._isDirected,e,t,n);return e=d.v,t=d.w,Object.freeze(d),this._edgeObjs[c]=d,o(this._preds[t],e),o(this._sucs[e],t),this._in[t][c]=d,this._out[e][c]=d,this._edgeCount++,this},i.prototype.edge=function(e,t,n){var r=1===arguments.length?u(this._isDirected,arguments[0]):s(this._isDirected,e,t,n);return this._edgeLabels[r]},i.prototype.hasEdge=function(e,t,n){var i=1===arguments.length?u(this._isDirected,arguments[0]):s(this._isDirected,e,t,n);return r.has(this._edgeLabels,i)},i.prototype.removeEdge=function(e,t,n){var r=1===arguments.length?u(this._isDirected,arguments[0]):s(this._isDirected,e,t,n),i=this._edgeObjs[r];return i&&(e=i.v,t=i.w,delete this._edgeLabels[r],delete this._edgeObjs[r],a(this._preds[t],e),a(this._sucs[e],t),delete this._in[t][r],delete this._out[e][r],this._edgeCount--),this},i.prototype.inEdges=function(e,t){var n=this._in[e];if(n){var i=r.values(n);return t?r.filter(i,(function(e){return e.v===t})):i}},i.prototype.outEdges=function(e,t){var n=this._out[e];if(n){var i=r.values(n);return t?r.filter(i,(function(e){return e.w===t})):i}},i.prototype.nodeEdges=function(e,t){var n=this.inEdges(e,t);if(n)return n.concat(this.outEdges(e,t))}},function(e,t){e.exports=function(e,t){for(var n=-1,r=null==e?0:e.length;++n<r&&!1!==t(e[n],n,e););return e}},function(e,t,n){var r=n(855);e.exports=function(e){var t=new e.constructor(e.byteLength);return new r(t).set(new r(e)),t}},function(e,t){e.exports=function(e){return function(){return e}}},function(e,t,n){var r=n(611),i=n(189);e.exports=function(e,t){return e&&r(e,t,i)}},function(e,t,n){var r=n(2101)();e.exports=r},function(e,t,n){var r=n(412),i=n(309);e.exports=function(e,t){for(var n=0,o=(t=r(t,e)).length;null!=e&&n<o;)e=e[i(t[n++])];return n&&n==o?e:void 0}},function(e,t,n){var r=n(81),i=n(259),o=/\.|\[(?:[^[\]]*|(["'])(?:(?!\1)[^\\]|\\.)*?\1)\]/,a=/^\w*$/;e.exports=function(e,t){if(r(e))return!1;var n=typeof e;return!("number"!=n&&"symbol"!=n&&"boolean"!=n&&null!=e&&!i(e))||(a.test(e)||!o.test(e)||null!=t&&e in Object(t))}},function(e,t,n){var r=n(495),i=n(2128);e.exports=function e(t,n,o,a,s){var l=-1,u=t.length;for(o||(o=i),s||(s=[]);++l<u;){var c=t[l];n>0&&o(c)?n>1?e(c,n-1,o,a,s):r(s,c):a||(s[s.length]=c)}return s}},function(e,t,n){var r=n(259);e.exports=function(e,t,n){for(var i=-1,o=e.length;++i<o;){var a=e[i],s=t(a);if(null!=s&&(void 0===l?s==s&&!r(s):n(s,l)))var l=s,u=a}return u}},function(e,t,n){"use strict";var r=n(36);Object.defineProperty(t,"__esModule",{value:!0}),Object.defineProperty(t,"MTableAction",{enumerable:!0,get:function(){return i.default}}),Object.defineProperty(t,"MTableActions",{enumerable:!0,get:function(){return o.default}}),Object.defineProperty(t,"MTableBody",{enumerable:!0,get:function(){return a.default}}),Object.defineProperty(t,"MTableBodyRow",{enumerable:!0,get:function(){return s.default}}),Object.defineProperty(t,"MTableGroupbar",{enumerable:!0,get:function(){return l.default}}),Object.defineProperty(t,"MTableGroupRow",{enumerable:!0,get:function(){return u.default}}),Object.defineProperty(t,"MTableCell",{enumerable:!0,get:function(){return c.default}}),Object.defineProperty(t,"MTableEditRow",{enumerable:!0,get:function(){return d.default}}),Object.defineProperty(t,"MTableEditField",{enumerable:!0,get:function(){return h.default}}),Object.defineProperty(t,"MTableFilterRow",{enumerable:!0,get:function(){return f.default}}),Object.defineProperty(t,"MTableHeader",{enumerable:!0,get:function(){return p.default}}),Object.defineProperty(t,"MTablePagination",{enumerable:!0,get:function(){return g.default}}),Object.defineProperty(t,"MTableSteppedPagination",{enumerable:!0,get:function(){return v.default}}),Object.defineProperty(t,"MTableToolbar",{enumerable:!0,get:function(){return m.default}});var i=r(n(2241)),o=r(n(2242)),a=r(n(2243)),s=r(n(2244)),l=r(n(2245)),u=r(n(2246)),c=r(n(2247)),d=r(n(2248)),h=r(n(2249)),f=r(n(2273)),p=r(n(2274)),g=r(n(2294)),v=r(n(2295)),m=r(n(2296))},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){(0,a.default)(1,arguments);var n=(0,i.default)(e,t),s=n.getUTCFullYear(),l=t||{},u=l.locale,c=u&&u.options&&u.options.firstWeekContainsDate,d=null==c?1:(0,r.default)(c),h=null==l.firstWeekContainsDate?d:(0,r.default)(l.firstWeekContainsDate);if(!(h>=1&&h<=7))throw new RangeError("firstWeekContainsDate must be between 1 and 7 inclusively");var f=new Date(0);f.setUTCFullYear(s+1,0,h),f.setUTCHours(0,0,0,0);var p=(0,o.default)(f,t),g=new Date(0);g.setUTCFullYear(s,0,h),g.setUTCHours(0,0,0,0);var v=(0,o.default)(g,t);return n.getTime()>=p.getTime()?s+1:n.getTime()>=v.getTime()?s:s-1};var r=s(n(77)),i=s(n(47)),o=s(n(424)),a=s(n(40));function s(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.hexToRgb=o,t.rgbToHex=function(e){if(0===e.indexOf("#"))return e;var t=s(e).values;return"#".concat(t.map((function(e){return 1===(t=e.toString(16)).length?"0".concat(t):t;var t})).join(""))},t.hslToRgb=a,t.decomposeColor=s,t.recomposeColor=l,t.getContrastRatio=function(e,t){var n=u(e),r=u(t);return(Math.max(n,r)+.05)/(Math.min(n,r)+.05)},t.getLuminance=u,t.emphasize=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:.15;return u(e)>.5?c(e,t):d(e,t)},t.fade=function(e,t){e=s(e),t=i(t),("rgb"===e.type||"hsl"===e.type)&&(e.type+="a");return e.values[3]=t,l(e)},t.darken=c,t.lighten=d;var r=n(31);function i(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:0,n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:1;return Math.min(Math.max(t,e),n)}function o(e){e=e.substr(1);var t=new RegExp(".{1,".concat(e.length>=6?2:1,"}"),"g"),n=e.match(t);return n&&1===n[0].length&&(n=n.map((function(e){return e+e}))),n?"rgb".concat(4===n.length?"a":"","(").concat(n.map((function(e,t){return t<3?parseInt(e,16):Math.round(parseInt(e,16)/255*1e3)/1e3})).join(", "),")"):""}function a(e){var t=(e=s(e)).values,n=t[0],r=t[1]/100,i=t[2]/100,o=r*Math.min(i,1-i),a=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:(e+n/30)%12;return i-o*Math.max(Math.min(t-3,9-t,1),-1)},u="rgb",c=[Math.round(255*a(0)),Math.round(255*a(8)),Math.round(255*a(4))];return"hsla"===e.type&&(u+="a",c.push(t[3])),l({type:u,values:c})}function s(e){if(e.type)return e;if("#"===e.charAt(0))return s(o(e));var t=e.indexOf("("),n=e.substring(0,t);if(-1===["rgb","rgba","hsl","hsla"].indexOf(n))throw new Error((0,r.formatMuiErrorMessage)(3,e));var i=e.substring(t+1,e.length-1).split(",");return{type:n,values:i=i.map((function(e){return parseFloat(e)}))}}function l(e){var t=e.type,n=e.values;return-1!==t.indexOf("rgb")?n=n.map((function(e,t){return t<3?parseInt(e,10):e})):-1!==t.indexOf("hsl")&&(n[1]="".concat(n[1],"%"),n[2]="".concat(n[2],"%")),"".concat(t,"(").concat(n.join(", "),")")}function u(e){var t="hsl"===(e=s(e)).type?s(a(e)).values:e.values;return t=t.map((function(e){return(e/=255)<=.03928?e/12.92:Math.pow((e+.055)/1.055,2.4)})),Number((.2126*t[0]+.7152*t[1]+.0722*t[2]).toFixed(3))}function c(e,t){if(e=s(e),t=i(t),-1!==e.type.indexOf("hsl"))e.values[2]*=1-t;else if(-1!==e.type.indexOf("rgb"))for(var n=0;n<3;n+=1)e.values[n]*=1-t;return l(e)}function d(e,t){if(e=s(e),t=i(t),-1!==e.type.indexOf("hsl"))e.values[2]+=(100-e.values[2])*t;else if(-1!==e.type.indexOf("rgb"))for(var n=0;n<3;n+=1)e.values[n]+=(255-e.values[n])*t;return l(e)}},function(e,t,n){"use strict";n.r(t);var r=n(319);n.d(t,"borders",(function(){return r.h})),n.d(t,"border",(function(){return r.a})),n.d(t,"borderTop",(function(){return r.g})),n.d(t,"borderRight",(function(){return r.f})),n.d(t,"borderBottom",(function(){return r.b})),n.d(t,"borderLeft",(function(){return r.d})),n.d(t,"borderColor",(function(){return r.c})),n.d(t,"borderRadius",(function(){return r.e}));var i=n(264);n.d(t,"breakpoints",(function(){return i.a}));var o=n(85);n.d(t,"compose",(function(){return o.a}));var a=n(627);n.d(t,"css",(function(){return a.a}));var s=n(628);n.d(t,"display",(function(){return s.a}));var l=n(320);n.d(t,"flexbox",(function(){return l.d})),n.d(t,"flexBasis",(function(){return l.f})),n.d(t,"flexDirection",(function(){return l.g})),n.d(t,"flexWrap",(function(){return l.j})),n.d(t,"justifyContent",(function(){return l.k})),n.d(t,"alignItems",(function(){return l.b})),n.d(t,"alignContent",(function(){return l.a})),n.d(t,"order",(function(){return l.n})),n.d(t,"flex",(function(){return l.e})),n.d(t,"flexGrow",(function(){return l.h})),n.d(t,"flexShrink",(function(){return l.i})),n.d(t,"alignSelf",(function(){return l.c})),n.d(t,"justifyItems",(function(){return l.l})),n.d(t,"justifySelf",(function(){return l.m}));var u=n(321);n.d(t,"grid",(function(){return u.a})),n.d(t,"gridGap",(function(){return u.h})),n.d(t,"gridColumnGap",(function(){return u.g})),n.d(t,"gridRowGap",(function(){return u.j})),n.d(t,"gridColumn",(function(){return u.f})),n.d(t,"gridRow",(function(){return u.i})),n.d(t,"gridAutoFlow",(function(){return u.d})),n.d(t,"gridAutoColumns",(function(){return u.c})),n.d(t,"gridAutoRows",(function(){return u.e})),n.d(t,"gridTemplateColumns",(function(){return u.l})),n.d(t,"gridTemplateRows",(function(){return u.m})),n.d(t,"gridTemplateAreas",(function(){return u.k})),n.d(t,"gridArea",(function(){return u.b}));var c=n(323);n.d(t,"palette",(function(){return c.c})),n.d(t,"color",(function(){return c.b})),n.d(t,"bgcolor",(function(){return c.a}));var d=n(322);n.d(t,"positions",(function(){return d.b})),n.d(t,"position",(function(){return d.d})),n.d(t,"zIndex",(function(){return d.g})),n.d(t,"top",(function(){return d.f})),n.d(t,"right",(function(){return d.e})),n.d(t,"bottom",(function(){return d.a})),n.d(t,"left",(function(){return d.c}));var h=n(629);n.d(t,"shadows",(function(){return h.a}));var f=n(324);n.d(t,"sizing",(function(){return f.b})),n.d(t,"width",(function(){return f.j})),n.d(t,"maxWidth",(function(){return f.e})),n.d(t,"minWidth",(function(){return f.g})),n.d(t,"height",(function(){return f.c})),n.d(t,"maxHeight",(function(){return f.d})),n.d(t,"minHeight",(function(){return f.f})),n.d(t,"sizeWidth",(function(){return f.i})),n.d(t,"sizeHeight",(function(){return f.h})),n.d(t,"boxSizing",(function(){return f.a}));var p=n(326);n.d(t,"spacing",(function(){return p.b})),n.d(t,"createUnarySpacing",(function(){return p.a}));var g=n(25);n.d(t,"style",(function(){return g.a}));var v=n(325);n.d(t,"typography",(function(){return v.a})),n.d(t,"fontFamily",(function(){return v.b})),n.d(t,"fontSize",(function(){return v.c})),n.d(t,"fontStyle",(function(){return v.d})),n.d(t,"fontWeight",(function(){return v.e})),n.d(t,"letterSpacing",(function(){return v.f})),n.d(t,"lineHeight",(function(){return v.g})),n.d(t,"textAlign",(function(){return v.h}))},function(e,t,n){var r,i,o=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),a=this&&this.__setModuleDefault||(Object.create?function(e,t){Object.defineProperty(e,"default",{enumerable:!0,value:t})}:function(e,t){e.default=t}),s=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)"default"!==n&&Object.hasOwnProperty.call(e,n)&&o(t,e,n);return a(t,e),t};r=[n,t,n(940),n(176)],void 0===(i=function(e,t,n,r){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.rootReducer=t.initialState=void 0,r=s(r),t.initialState={systemConfig:{},mainState:{panel_0:[!1,"Project_Overview"],panel_1:[!0,"System_Arc"],panel_2:[!1,"Module_Creator"],panel_3:[!1,"Dashboard"],panel_4:[!1,"User_Management"],panel_5:[!1,"Module_Library"]},saveSignal:0,systemArch:{lockStatus:!1,systemTree:{nodeData:[{title:"Root"}],nodePath:[],selectedNode:null},systemData:{mainData:{},portData:{}},systemGraph:{systemGraphData:{Root:{inPort:["inwards"],outPort:["outwards"],connections:[]}},systemList:["Root"],graphJsonData:{},updateData:{}},systemPBS:{},systemPBSUpdated:0},dashboardState:{computing:!1,timeStepList:["None"],data3D:{},variableData:{},portMetaData:{},selectedVariable:{},computedResult:{},recorderData:{},driverData:{}}},t.rootReducer=function(e=t.initialState,i){switch(i.type){case n.Action.RESET_STORE:return t.initialState;case n.Action.SAVE_STATE:return r.saveState_(e);case n.Action.SWITCH_PANEL:return r.mainSwitchPanel_(e,i);case n.Action.LOCK_GRAPH:return Object.assign(Object.assign({},e),{systemArch:Object.assign(Object.assign({},e.systemArch),{lockStatus:i.status})});case n.Action.GET_SERVER_DATA:return r.serverGetData_(e,i);case n.Action.ARCH_SAVE_GRAPH_JSON:return r.archSaveGraphJson_(e,i);case n.Action.ARCH_SAVE_GRAPH_POSITION:return r.archSaveGraphPosition_(e,i);case n.Action.ARCH_UPDATE_CONNECTION_GRAPH:return r.archUpdateConnectionGraph_(e,i);case n.Action.ARCH_FILTER_NODE:return r.archFilterNode_(e,i);case n.Action.SELECT_NODE:return r.archSelectNode_(e,i);case n.Action.ADD_NODE:return r.archAddNode_(e,i);case n.Action.UPDATE_TREE:return r.archUpdateTreeData_(e,i);case n.Action.REMOVE_NODE:return r.archRemoveNode_(e,i);case n.Action.DASHBOARD_ADD_3D_DATA:return r.dashboardAdd3DData_(e,i);case n.Action.DASHBOARD_TOGGLE_RUN:return Object.assign(Object.assign({},e),{dashboardState:Object.assign(Object.assign({},e.dashboardState),{computing:!e.dashboardState.computing})});case n.Action.DASHBOARD_SET_CONTROLLER:return r.dashboardSetController_(e,i);case n.Action.DASHBOARD_ADD_CONTROLLER:return r.dashboardAddController_(e,i);case n.Action.DASHBOARD_REMOVE_CONTROLLER:return r.dashboardRemoveController_(e,i);case n.Action.DASHBOARD_UPDATE_CONTROLLER:return r.dashboardUpdateController_(e,i);case n.Action.DASHBOARD_UPDATE_COMPUTED_RESULT:return r.dashboardUpdateComputedResult_(e,i)}return e}}.apply(t,r))||(e.exports=i)},function(e,t,n){"use strict";n.r(t);var r=n(0),i=n.n(r),o=n(101),a=n(1),s=n(118),l=n(78);function u(e,t){var n=Object(r.useState)((function(){return{inputs:t,result:e()}}))[0],i=Object(r.useRef)(!0),o=Object(r.useRef)(n),a=i.current||Boolean(t&&o.current.inputs&&function(e,t){if(e.length!==t.length)return!1;for(var n=0;n<e.length;n++)if(e[n]!==t[n])return!1;return!0}(t,o.current.inputs))?o.current:{inputs:t,result:e()};return Object(r.useEffect)((function(){i.current=!1,o.current=a}),[a]),a.result}var c=u,d=function(e,t){return u((function(){return e}),t)};var h=function(e,t){if(!e)throw new Error("Invariant failed")},f=function(e){var t=e.top,n=e.right,r=e.bottom,i=e.left;return{top:t,right:n,bottom:r,left:i,width:n-i,height:r-t,x:i,y:t,center:{x:(n+i)/2,y:(r+t)/2}}},p=function(e,t){return{top:e.top-t.top,left:e.left-t.left,bottom:e.bottom+t.bottom,right:e.right+t.right}},g=function(e,t){return{top:e.top+t.top,left:e.left+t.left,bottom:e.bottom-t.bottom,right:e.right-t.right}},v={top:0,right:0,bottom:0,left:0},m=function(e){var t=e.borderBox,n=e.margin,r=void 0===n?v:n,i=e.border,o=void 0===i?v:i,a=e.padding,s=void 0===a?v:a,l=f(p(t,r)),u=f(g(t,o)),c=f(g(u,s));return{marginBox:l,borderBox:f(t),paddingBox:u,contentBox:c,margin:r,border:o,padding:s}},y=function(e){var t=e.slice(0,-2);if("px"!==e.slice(-2))return 0;var n=Number(t);return isNaN(n)&&h(!1),n},b=function(e,t){var n,r,i=e.borderBox,o=e.border,a=e.margin,s=e.padding,l=(r=t,{top:(n=i).top+r.y,left:n.left+r.x,bottom:n.bottom+r.y,right:n.right+r.x});return m({borderBox:l,border:o,margin:a,padding:s})},C=function(e,t){return void 0===t&&(t={x:window.pageXOffset,y:window.pageYOffset}),b(e,t)},A=function(e,t){var n={top:y(t.marginTop),right:y(t.marginRight),bottom:y(t.marginBottom),left:y(t.marginLeft)},r={top:y(t.paddingTop),right:y(t.paddingRight),bottom:y(t.paddingBottom),left:y(t.paddingLeft)},i={top:y(t.borderTopWidth),right:y(t.borderRightWidth),bottom:y(t.borderBottomWidth),left:y(t.borderLeftWidth)};return m({borderBox:e,margin:n,padding:r,border:i})},_=function(e){var t=e.getBoundingClientRect(),n=window.getComputedStyle(e);return A(t,n)};function I(e,t){if(e.length!==t.length)return!1;for(var n=0;n<e.length;n++)if(e[n]!==t[n])return!1;return!0}var x=function(e,t){var n;void 0===t&&(t=I);var r,i=[],o=!1;return function(){for(var a=[],s=0;s<arguments.length;s++)a[s]=arguments[s];return o&&n===this&&t(a,i)||(r=e.apply(this,a),o=!0,n=this,i=a),r}},w=function(e){var t=[],n=null,r=function(){for(var r=arguments.length,i=new Array(r),o=0;o<r;o++)i[o]=arguments[o];t=i,n||(n=requestAnimationFrame((function(){n=null,e.apply(void 0,t)})))};return r.cancel=function(){n&&(cancelAnimationFrame(n),n=null)},r},M=n(22),S=n.n(M);n.d(t,"DragDropContext",(function(){return Pr})),n.d(t,"Draggable",(function(){return vi})),n.d(t,"Droppable",(function(){return Ai})),n.d(t,"resetServerContext",(function(){return Dr}));function T(e,t){}T.bind(null,"warn"),T.bind(null,"error");function E(){}function O(e,t,n){var r=t.map((function(t){var r,i,o=(r=n,i=t.options,Object(a.a)({},r,{},i));return e.addEventListener(t.eventName,t.fn,o),function(){e.removeEventListener(t.eventName,t.fn,o)}}));return function(){r.forEach((function(e){e()}))}}function k(e){this.message=e}function L(e,t){if(!e)throw new k("Invariant failed")}k.prototype.toString=function(){return this.message};var D=function(e){function t(){for(var t,n=arguments.length,r=new Array(n),i=0;i<n;i++)r[i]=arguments[i];return(t=e.call.apply(e,[this].concat(r))||this).callbacks=null,t.unbind=E,t.onWindowError=function(e){var n=t.getCallbacks();n.isDragging()&&n.tryAbort(),e.error instanceof k&&e.preventDefault()},t.getCallbacks=function(){if(!t.callbacks)throw new Error("Unable to find AppCallbacks in <ErrorBoundary/>");return t.callbacks},t.setCallbacks=function(e){t.callbacks=e},t}Object(o.a)(t,e);var n=t.prototype;return n.componentDidMount=function(){this.unbind=O(window,[{eventName:"error",fn:this.onWindowError}])},n.componentWillUnmount=function(){this.unbind()},n.componentDidCatch=function(e){if(!(e instanceof k))throw e;this.setState({})},n.render=function(){return this.props.children(this.setCallbacks)},t}(i.a.Component),P=function(e){return e+1},N=function(e,t){var n=e.droppableId===t.droppableId,r=P(e.index),i=P(t.index);return n?"\n      You have moved the item from position "+r+"\n      to position "+i+"\n    ":"\n    You have moved the item from position "+r+"\n    in list "+e.droppableId+"\n    to list "+t.droppableId+"\n    in position "+i+"\n  "},j=function(e,t,n){return t.droppableId===n.droppableId?"\n      The item "+e+"\n      has been combined with "+n.draggableId:"\n      The item "+e+"\n      in list "+t.droppableId+"\n      has been combined with "+n.draggableId+"\n      in list "+n.droppableId+"\n    "},R=function(e){return"\n  The item has returned to its starting position\n  of "+P(e.index)+"\n"},z="\n  Press space bar to start a drag.\n  When dragging you can use the arrow keys to move the item around and escape to cancel.\n  Some screen readers may require you to be in focus mode or to use your pass through key\n",F=function(e){return"\n  You have lifted an item in position "+P(e.source.index)+"\n"},B=function(e){var t=e.destination;if(t)return N(e.source,t);var n=e.combine;return n?j(e.draggableId,e.source,n):"You are over an area that cannot be dropped on"},H=function(e){if("CANCEL"===e.reason)return"\n      Movement cancelled.\n      "+R(e.source)+"\n    ";var t=e.destination,n=e.combine;return t?"\n      You have dropped the item.\n      "+N(e.source,t)+"\n    ":n?"\n      You have dropped the item.\n      "+j(e.draggableId,e.source,n)+"\n    ":"\n    The item has been dropped while not over a drop area.\n    "+R(e.source)+"\n  "},U={x:0,y:0},V=function(e,t){return{x:e.x+t.x,y:e.y+t.y}},Y=function(e,t){return{x:e.x-t.x,y:e.y-t.y}},W=function(e,t){return e.x===t.x&&e.y===t.y},G=function(e){return{x:0!==e.x?-e.x:0,y:0!==e.y?-e.y:0}},X=function(e,t,n){var r;return void 0===n&&(n=0),(r={})[e]=t,r["x"===e?"y":"x"]=n,r},q=function(e,t){return Math.sqrt(Math.pow(t.x-e.x,2)+Math.pow(t.y-e.y,2))},Z=function(e,t){return Math.min.apply(Math,t.map((function(t){return q(e,t)})))},Q=function(e){return function(t){return{x:e(t.x),y:e(t.y)}}},K=function(e,t){return{top:e.top+t.y,left:e.left+t.x,bottom:e.bottom+t.y,right:e.right+t.x}},J=function(e){return[{x:e.left,y:e.top},{x:e.right,y:e.top},{x:e.left,y:e.bottom},{x:e.right,y:e.bottom}]},$=function(e,t){return t&&t.shouldClipSubject?function(e,t){var n=f({top:Math.max(t.top,e.top),right:Math.min(t.right,e.right),bottom:Math.min(t.bottom,e.bottom),left:Math.max(t.left,e.left)});return n.width<=0||n.height<=0?null:n}(t.pageMarginBox,e):f(e)},ee=function(e){var t=e.page,n=e.withPlaceholder,r=e.axis,i=e.frame,o=function(e,t,n){var r;return n&&n.increasedBy?Object(a.a)({},e,((r={})[t.end]=e[t.end]+n.increasedBy[t.line],r)):e}(function(e,t){return t?K(e,t.scroll.diff.displacement):e}(t.marginBox,i),r,n);return{page:t,withPlaceholder:n,active:$(o,i)}},te=function(e,t){e.frame||L(!1);var n=e.frame,r=Y(t,n.scroll.initial),i=G(r),o=Object(a.a)({},n,{scroll:{initial:n.scroll.initial,current:t,diff:{value:r,displacement:i},max:n.scroll.max}}),s=ee({page:e.subject.page,withPlaceholder:e.subject.withPlaceholder,axis:e.axis,frame:o});return Object(a.a)({},e,{frame:o,subject:s})};function ne(e){return Object.values?Object.values(e):Object.keys(e).map((function(t){return e[t]}))}function re(e,t){if(e.findIndex)return e.findIndex(t);for(var n=0;n<e.length;n++)if(t(e[n]))return n;return-1}function ie(e,t){if(e.find)return e.find(t);var n=re(e,t);return-1!==n?e[n]:void 0}function oe(e){return Array.prototype.slice.call(e)}var ae=x((function(e){return e.reduce((function(e,t){return e[t.descriptor.id]=t,e}),{})})),se=x((function(e){return e.reduce((function(e,t){return e[t.descriptor.id]=t,e}),{})})),le=x((function(e){return ne(e)})),ue=x((function(e){return ne(e)})),ce=x((function(e,t){return ue(t).filter((function(t){return e===t.descriptor.droppableId})).sort((function(e,t){return e.descriptor.index-t.descriptor.index}))}));function de(e){return e.at&&"REORDER"===e.at.type?e.at.destination:null}function he(e){return e.at&&"COMBINE"===e.at.type?e.at.combine:null}var fe=x((function(e,t){return t.filter((function(t){return t.descriptor.id!==e.descriptor.id}))})),pe=function(e,t){return e.descriptor.droppableId===t.descriptor.id},ge={point:U,value:0},ve={invisible:{},visible:{},all:[]},me={displaced:ve,displacedBy:ge,at:null},ye=function(e,t){return function(n){return e<=n&&n<=t}},be=function(e){var t=ye(e.top,e.bottom),n=ye(e.left,e.right);return function(r){if(t(r.top)&&t(r.bottom)&&n(r.left)&&n(r.right))return!0;var i=t(r.top)||t(r.bottom),o=n(r.left)||n(r.right);if(i&&o)return!0;var a=r.top<e.top&&r.bottom>e.bottom,s=r.left<e.left&&r.right>e.right;return!(!a||!s)||(a&&o||s&&i)}},Ce=function(e){var t=ye(e.top,e.bottom),n=ye(e.left,e.right);return function(e){return t(e.top)&&t(e.bottom)&&n(e.left)&&n(e.right)}},Ae={direction:"vertical",line:"y",crossAxisLine:"x",start:"top",end:"bottom",size:"height",crossAxisStart:"left",crossAxisEnd:"right",crossAxisSize:"width"},_e={direction:"horizontal",line:"x",crossAxisLine:"y",start:"left",end:"right",size:"width",crossAxisStart:"top",crossAxisEnd:"bottom",crossAxisSize:"height"},Ie=function(e){var t=e.target,n=e.destination,r=e.viewport,i=e.withDroppableDisplacement,o=e.isVisibleThroughFrameFn,a=i?function(e,t){var n=t.frame?t.frame.scroll.diff.displacement:U;return K(e,n)}(t,n):t;return function(e,t,n){return!!t.subject.active&&n(t.subject.active)(e)}(a,n,o)&&function(e,t,n){return n(t)(e)}(a,r,o)},xe=function(e){return Ie(Object(a.a)({},e,{isVisibleThroughFrameFn:be}))},we=function(e){return Ie(Object(a.a)({},e,{isVisibleThroughFrameFn:Ce}))};function Me(e){var t=e.afterDragging,n=e.destination,r=e.displacedBy,i=e.viewport,o=e.forceShouldAnimate,a=e.last;return t.reduce((function(e,t){var s=function(e,t){var n=e.page.marginBox,r={top:t.point.y,right:0,bottom:0,left:t.point.x};return f(p(n,r))}(t,r),l=t.descriptor.id;if(e.all.push(l),!xe({target:s,destination:n,viewport:i,withDroppableDisplacement:!0}))return e.invisible[t.descriptor.id]=!0,e;var u={draggableId:l,shouldAnimate:function(e,t,n){if("boolean"==typeof n)return n;if(!t)return!0;var r=t.invisible,i=t.visible;if(r[e])return!1;var o=i[e];return!o||o.shouldAnimate}(l,a,o)};return e.visible[l]=u,e}),{all:[],visible:{},invisible:{}})}function Se(e){var t=e.insideDestination,n=e.inHomeList,r=e.displacedBy,i=e.destination,o=function(e,t){if(!e.length)return 0;var n=e[e.length-1].descriptor.index;return t.inHomeList?n:n+1}(t,{inHomeList:n});return{displaced:ve,displacedBy:r,at:{type:"REORDER",destination:{droppableId:i.descriptor.id,index:o}}}}function Te(e){var t=e.draggable,n=e.insideDestination,r=e.destination,i=e.viewport,o=e.displacedBy,a=e.last,s=e.index,l=e.forceShouldAnimate,u=pe(t,r);if(null==s)return Se({insideDestination:n,inHomeList:u,displacedBy:o,destination:r});var c=ie(n,(function(e){return e.descriptor.index===s}));if(!c)return Se({insideDestination:n,inHomeList:u,displacedBy:o,destination:r});var d=fe(t,n),h=n.indexOf(c);return{displaced:Me({afterDragging:d.slice(h),destination:r,displacedBy:o,last:a,viewport:i.frame,forceShouldAnimate:l}),displacedBy:o,at:{type:"REORDER",destination:{droppableId:r.descriptor.id,index:s}}}}function Ee(e,t){return Boolean(t.effected[e])}var Oe=function(e){var t=e.isMovingForward,n=e.isInHomeList,r=e.draggable,i=e.draggables,o=e.destination,a=e.insideDestination,s=e.previousImpact,l=e.viewport,u=e.afterCritical,c=s.at;if(c||L(!1),"REORDER"===c.type){var d=function(e){var t=e.isMovingForward,n=e.isInHomeList,r=e.insideDestination,i=e.location;if(!r.length)return null;var o=i.index,a=t?o+1:o-1,s=r[0].descriptor.index,l=r[r.length-1].descriptor.index;return a<s||a>(n?l:l+1)?null:a}({isMovingForward:t,isInHomeList:n,location:c.destination,insideDestination:a});return null==d?null:Te({draggable:r,insideDestination:a,destination:o,viewport:l,last:s.displaced,displacedBy:s.displacedBy,index:d})}var h=function(e){var t=e.isMovingForward,n=e.destination,r=e.draggables,i=e.combine,o=e.afterCritical;if(!n.isCombineEnabled)return null;var a=i.draggableId,s=r[a].descriptor.index;return Ee(a,o)?t?s:s-1:t?s+1:s}({isMovingForward:t,destination:o,displaced:s.displaced,draggables:i,combine:c.combine,afterCritical:u});return null==h?null:Te({draggable:r,insideDestination:a,destination:o,viewport:l,last:s.displaced,displacedBy:s.displacedBy,index:h})},ke=function(e){var t=e.afterCritical,n=e.impact,r=e.draggables,i=he(n);i||L(!1);var o=i.draggableId,a=r[o].page.borderBox.center,s=function(e){var t=e.displaced,n=e.afterCritical,r=e.combineWith,i=e.displacedBy,o=Boolean(t.visible[r]||t.invisible[r]);return Ee(r,n)?o?U:G(i.point):o?i.point:U}({displaced:n.displaced,afterCritical:t,combineWith:o,displacedBy:n.displacedBy});return V(a,s)},Le=function(e,t){return t.margin[e.start]+t.borderBox[e.size]/2},De=function(e,t,n){return t[e.crossAxisStart]+n.margin[e.crossAxisStart]+n.borderBox[e.crossAxisSize]/2},Pe=function(e){var t=e.axis,n=e.moveRelativeTo,r=e.isMoving;return X(t.line,n.marginBox[t.end]+Le(t,r),De(t,n.marginBox,r))},Ne=function(e){var t=e.axis,n=e.moveRelativeTo,r=e.isMoving;return X(t.line,n.marginBox[t.start]-function(e,t){return t.margin[e.end]+t.borderBox[e.size]/2}(t,r),De(t,n.marginBox,r))},je=function(e){var t=e.impact,n=e.draggable,r=e.draggables,i=e.droppable,o=e.afterCritical,a=ce(i.descriptor.id,r),s=n.page,l=i.axis;if(!a.length)return function(e){var t=e.axis,n=e.moveInto,r=e.isMoving;return X(t.line,n.contentBox[t.start]+Le(t,r),De(t,n.contentBox,r))}({axis:l,moveInto:i.page,isMoving:s});var u=t.displaced,c=t.displacedBy,d=u.all[0];if(d){var h=r[d];if(Ee(d,o))return Ne({axis:l,moveRelativeTo:h.page,isMoving:s});var f=b(h.page,c.point);return Ne({axis:l,moveRelativeTo:f,isMoving:s})}var p=a[a.length-1];if(p.descriptor.id===n.descriptor.id)return s.borderBox.center;if(Ee(p.descriptor.id,o)){var g=b(p.page,G(o.displacedBy.point));return Pe({axis:l,moveRelativeTo:g,isMoving:s})}return Pe({axis:l,moveRelativeTo:p.page,isMoving:s})},Re=function(e,t){var n=e.frame;return n?V(t,n.scroll.diff.displacement):t},ze=function(e){var t=function(e){var t=e.impact,n=e.draggable,r=e.droppable,i=e.draggables,o=e.afterCritical,a=n.page.borderBox.center,s=t.at;return r&&s?"REORDER"===s.type?je({impact:t,draggable:n,draggables:i,droppable:r,afterCritical:o}):ke({impact:t,draggables:i,afterCritical:o}):a}(e),n=e.droppable;return n?Re(n,t):t},Fe=function(e,t){var n=Y(t,e.scroll.initial),r=G(n);return{frame:f({top:t.y,bottom:t.y+e.frame.height,left:t.x,right:t.x+e.frame.width}),scroll:{initial:e.scroll.initial,max:e.scroll.max,current:t,diff:{value:n,displacement:r}}}};function Be(e,t){return e.map((function(e){return t[e]}))}var He=function(e){var t=e.pageBorderBoxCenter,n=e.draggable,r=function(e,t){return V(e.scroll.diff.displacement,t)}(e.viewport,t),i=Y(r,n.page.borderBox.center);return V(n.client.borderBox.center,i)},Ue=function(e){var t=e.draggable,n=e.destination,r=e.newPageBorderBoxCenter,i=e.viewport,o=e.withDroppableDisplacement,s=e.onlyOnMainAxis,l=void 0!==s&&s,u=Y(r,t.page.borderBox.center),c={target:K(t.page.borderBox,u),destination:n,withDroppableDisplacement:o,viewport:i};return l?function(e){return Ie(Object(a.a)({},e,{isVisibleThroughFrameFn:(t=e.destination.axis,function(e){var n=ye(e.top,e.bottom),r=ye(e.left,e.right);return function(e){return t===Ae?n(e.top)&&n(e.bottom):r(e.left)&&r(e.right)}})}));var t}(c):we(c)},Ve=function(e){var t=e.isMovingForward,n=e.draggable,r=e.destination,i=e.draggables,o=e.previousImpact,s=e.viewport,l=e.previousPageBorderBoxCenter,u=e.previousClientSelection,c=e.afterCritical;if(!r.isEnabled)return null;var d=ce(r.descriptor.id,i),h=pe(n,r),f=function(e){var t=e.isMovingForward,n=e.draggable,r=e.destination,i=e.insideDestination,o=e.previousImpact;if(!r.isCombineEnabled)return null;if(!de(o))return null;function s(e){var t={type:"COMBINE",combine:{draggableId:e,droppableId:r.descriptor.id}};return Object(a.a)({},o,{at:t})}var l=o.displaced.all,u=l.length?l[0]:null;if(t)return u?s(u):null;var c=fe(n,i);if(!u)return c.length?s(c[c.length-1].descriptor.id):null;var d=re(c,(function(e){return e.descriptor.id===u}));-1===d&&L(!1);var h=d-1;return h<0?null:s(c[h].descriptor.id)}({isMovingForward:t,draggable:n,destination:r,insideDestination:d,previousImpact:o})||Oe({isMovingForward:t,isInHomeList:h,draggable:n,draggables:i,destination:r,insideDestination:d,previousImpact:o,viewport:s,afterCritical:c});if(!f)return null;var p=ze({impact:f,draggable:n,droppable:r,draggables:i,afterCritical:c});if(Ue({draggable:n,destination:r,newPageBorderBoxCenter:p,viewport:s.frame,withDroppableDisplacement:!1,onlyOnMainAxis:!0}))return{clientSelection:He({pageBorderBoxCenter:p,draggable:n,viewport:s}),impact:f,scrollJumpRequest:null};var g=Y(p,l);return{clientSelection:u,impact:function(e){var t=e.impact,n=e.viewport,r=e.destination,i=e.draggables,o=e.maxScrollChange,s=Fe(n,V(n.scroll.current,o)),l=r.frame?te(r,V(r.frame.scroll.current,o)):r,u=t.displaced,c=Me({afterDragging:Be(u.all,i),destination:r,displacedBy:t.displacedBy,viewport:s.frame,last:u,forceShouldAnimate:!1}),d=Me({afterDragging:Be(u.all,i),destination:l,displacedBy:t.displacedBy,viewport:n.frame,last:u,forceShouldAnimate:!1}),h={},f={},p=[u,c,d];return u.all.forEach((function(e){var t=function(e,t){for(var n=0;n<t.length;n++){var r=t[n].visible[e];if(r)return r}return null}(e,p);t?f[e]=t:h[e]=!0})),Object(a.a)({},t,{displaced:{all:u.all,invisible:h,visible:f}})}({impact:f,viewport:s,destination:r,draggables:i,maxScrollChange:g}),scrollJumpRequest:g}},Ye=function(e){var t=e.subject.active;return t||L(!1),t},We=function(e,t){var n=e.page.borderBox.center;return Ee(e.descriptor.id,t)?Y(n,t.displacedBy.point):n},Ge=function(e,t){var n=e.page.borderBox;return Ee(e.descriptor.id,t)?K(n,G(t.displacedBy.point)):n},Xe=x((function(e,t){var n=t[e.line];return{value:n,point:X(e.line,n)}})),qe=function(e,t){return Object(a.a)({},e,{scroll:Object(a.a)({},e.scroll,{max:t})})},Ze=function(e,t,n){var r=e.frame;pe(t,e)&&L(!1),e.subject.withPlaceholder&&L(!1);var i=Xe(e.axis,t.displaceBy).point,o=function(e,t,n){var r=e.axis;if("virtual"===e.descriptor.mode)return X(r.line,t[r.line]);var i=e.subject.page.contentBox[r.size],o=ce(e.descriptor.id,n).reduce((function(e,t){return e+t.client.marginBox[r.size]}),0)+t[r.line]-i;return o<=0?null:X(r.line,o)}(e,i,n),s={placeholderSize:i,increasedBy:o,oldFrameMaxScroll:e.frame?e.frame.scroll.max:null};if(!r){var l=ee({page:e.subject.page,withPlaceholder:s,axis:e.axis,frame:e.frame});return Object(a.a)({},e,{subject:l})}var u=o?V(r.scroll.max,o):r.scroll.max,c=qe(r,u),d=ee({page:e.subject.page,withPlaceholder:s,axis:e.axis,frame:c});return Object(a.a)({},e,{subject:d,frame:c})},Qe=function(e){var t=e.isMovingForward,n=e.previousPageBorderBoxCenter,r=e.draggable,i=e.isOver,o=e.draggables,a=e.droppables,s=e.viewport,l=e.afterCritical,u=function(e){var t=e.isMovingForward,n=e.pageBorderBoxCenter,r=e.source,i=e.droppables,o=e.viewport,a=r.subject.active;if(!a)return null;var s=r.axis,l=ye(a[s.start],a[s.end]),u=le(i).filter((function(e){return e!==r})).filter((function(e){return e.isEnabled})).filter((function(e){return Boolean(e.subject.active)})).filter((function(e){return be(o.frame)(Ye(e))})).filter((function(e){var n=Ye(e);return t?a[s.crossAxisEnd]<n[s.crossAxisEnd]:n[s.crossAxisStart]<a[s.crossAxisStart]})).filter((function(e){var t=Ye(e),n=ye(t[s.start],t[s.end]);return l(t[s.start])||l(t[s.end])||n(a[s.start])||n(a[s.end])})).sort((function(e,n){var r=Ye(e)[s.crossAxisStart],i=Ye(n)[s.crossAxisStart];return t?r-i:i-r})).filter((function(e,t,n){return Ye(e)[s.crossAxisStart]===Ye(n[0])[s.crossAxisStart]}));if(!u.length)return null;if(1===u.length)return u[0];var c=u.filter((function(e){return ye(Ye(e)[s.start],Ye(e)[s.end])(n[s.line])}));return 1===c.length?c[0]:c.length>1?c.sort((function(e,t){return Ye(e)[s.start]-Ye(t)[s.start]}))[0]:u.sort((function(e,t){var r=Z(n,J(Ye(e))),i=Z(n,J(Ye(t)));return r!==i?r-i:Ye(e)[s.start]-Ye(t)[s.start]}))[0]}({isMovingForward:t,pageBorderBoxCenter:n,source:i,droppables:a,viewport:s});if(!u)return null;var c=ce(u.descriptor.id,o),d=function(e){var t=e.previousPageBorderBoxCenter,n=e.moveRelativeTo,r=e.insideDestination,i=e.draggable,o=e.draggables,a=e.destination,s=e.viewport,l=e.afterCritical;if(!n){if(r.length)return null;var u={displaced:ve,displacedBy:ge,at:{type:"REORDER",destination:{droppableId:a.descriptor.id,index:0}}},c=ze({impact:u,draggable:i,droppable:a,draggables:o,afterCritical:l}),d=pe(i,a)?a:Ze(a,i,o);return Ue({draggable:i,destination:d,newPageBorderBoxCenter:c,viewport:s.frame,withDroppableDisplacement:!1,onlyOnMainAxis:!0})?u:null}var h,f=Boolean(t[a.axis.line]<=n.page.borderBox.center[a.axis.line]),p=(h=n.descriptor.index,n.descriptor.id===i.descriptor.id||f?h:h+1),g=Xe(a.axis,i.displaceBy);return Te({draggable:i,insideDestination:r,destination:a,viewport:s,displacedBy:g,last:ve,index:p})}({previousPageBorderBoxCenter:n,destination:u,draggable:r,draggables:o,moveRelativeTo:function(e){var t=e.pageBorderBoxCenter,n=e.viewport,r=e.destination,i=e.insideDestination,o=e.afterCritical;return i.filter((function(e){return we({target:Ge(e,o),destination:r,viewport:n.frame,withDroppableDisplacement:!0})})).sort((function(e,n){var i=q(t,Re(r,We(e,o))),a=q(t,Re(r,We(n,o)));return i<a?-1:a<i?1:e.descriptor.index-n.descriptor.index}))[0]||null}({pageBorderBoxCenter:n,viewport:s,destination:u,insideDestination:c,afterCritical:l}),insideDestination:c,viewport:s,afterCritical:l});if(!d)return null;var h=ze({impact:d,draggable:r,droppable:u,draggables:o,afterCritical:l});return{clientSelection:He({pageBorderBoxCenter:h,draggable:r,viewport:s}),impact:d,scrollJumpRequest:null}},Ke=function(e){var t=e.at;return t?"REORDER"===t.type?t.destination.droppableId:t.combine.droppableId:null},Je=function(e){var t=e.state,n=e.type,r=function(e,t){var n=Ke(e);return n?t[n]:null}(t.impact,t.dimensions.droppables),i=Boolean(r),o=t.dimensions.droppables[t.critical.droppable.id],a=r||o,s=a.axis.direction,l="vertical"===s&&("MOVE_UP"===n||"MOVE_DOWN"===n)||"horizontal"===s&&("MOVE_LEFT"===n||"MOVE_RIGHT"===n);if(l&&!i)return null;var u="MOVE_DOWN"===n||"MOVE_RIGHT"===n,c=t.dimensions.draggables[t.critical.draggable.id],d=t.current.page.borderBoxCenter,h=t.dimensions,f=h.draggables,p=h.droppables;return l?Ve({isMovingForward:u,previousPageBorderBoxCenter:d,draggable:c,destination:a,draggables:f,viewport:t.viewport,previousClientSelection:t.current.client.selection,previousImpact:t.impact,afterCritical:t.afterCritical}):Qe({isMovingForward:u,previousPageBorderBoxCenter:d,draggable:c,isOver:a,draggables:f,droppables:p,viewport:t.viewport,afterCritical:t.afterCritical})};function $e(e){return"DRAGGING"===e.phase||"COLLECTING"===e.phase}function et(e){var t=ye(e.top,e.bottom),n=ye(e.left,e.right);return function(e){return t(e.y)&&n(e.x)}}function tt(e){var t=e.pageBorderBox,n=e.draggable,r=e.droppables,i=le(r).filter((function(e){if(!e.isEnabled)return!1;var n,r,i=e.subject.active;if(!i)return!1;if(r=i,!((n=t).left<r.right&&n.right>r.left&&n.top<r.bottom&&n.bottom>r.top))return!1;if(et(i)(t.center))return!0;var o=e.axis,a=i.center[o.crossAxisLine],s=t[o.crossAxisStart],l=t[o.crossAxisEnd],u=ye(i[o.crossAxisStart],i[o.crossAxisEnd]),c=u(s),d=u(l);return!c&&!d||(c?s<a:l>a)}));return i.length?1===i.length?i[0].descriptor.id:function(e){var t=e.pageBorderBox,n=e.draggable,r=e.candidates,i=n.page.borderBox.center,o=r.map((function(e){var n=e.axis,r=X(e.axis.line,t.center[n.line],e.page.borderBox.center[n.crossAxisLine]);return{id:e.descriptor.id,distance:q(i,r)}})).sort((function(e,t){return t.distance-e.distance}));return o[0]?o[0].id:null}({pageBorderBox:t,draggable:n,candidates:i}):null}var nt=function(e,t){return f(K(e,t))};function rt(e){var t=e.displaced,n=e.id;return Boolean(t.visible[n]||t.invisible[n])}var it=function(e){var t=e.pageOffset,n=e.draggable,r=e.draggables,i=e.droppables,o=e.previousImpact,a=e.viewport,s=e.afterCritical,l=nt(n.page.borderBox,t),u=tt({pageBorderBox:l,draggable:n,droppables:i});if(!u)return me;var c,d,h=i[u],f=ce(h.descriptor.id,r),p=(c=l,(d=h.frame)?nt(c,d.scroll.diff.value):c);return function(e){var t=e.draggable,n=e.pageBorderBoxWithDroppableScroll,r=e.previousImpact,i=e.destination,o=e.insideDestination,a=e.afterCritical;if(!i.isCombineEnabled)return null;var s=i.axis,l=Xe(i.axis,t.displaceBy),u=l.value,c=n[s.start],d=n[s.end],h=ie(fe(t,o),(function(e){var t=e.descriptor.id,n=e.page.borderBox,i=n[s.size]/4,o=Ee(t,a),l=rt({displaced:r.displaced,id:t});return o?l?d>n[s.start]+i&&d<n[s.end]-i:c>n[s.start]-u+i&&c<n[s.end]-u-i:l?d>n[s.start]+u+i&&d<n[s.end]+u-i:c>n[s.start]+i&&c<n[s.end]-i}));return h?{displacedBy:l,displaced:r.displaced,at:{type:"COMBINE",combine:{draggableId:h.descriptor.id,droppableId:i.descriptor.id}}}:null}({pageBorderBoxWithDroppableScroll:p,draggable:n,previousImpact:o,destination:h,insideDestination:f,afterCritical:s})||function(e){var t=e.pageBorderBoxWithDroppableScroll,n=e.draggable,r=e.destination,i=e.insideDestination,o=e.last,a=e.viewport,s=e.afterCritical,l=r.axis,u=Xe(r.axis,n.displaceBy),c=u.value,d=t[l.start],h=t[l.end],f=function(e){var t=e.draggable,n=e.closest,r=e.inHomeList;return n?r&&n.descriptor.index>t.descriptor.index?n.descriptor.index-1:n.descriptor.index:null}({draggable:n,closest:ie(fe(n,i),(function(e){var t=e.descriptor.id,n=e.page.borderBox.center[l.line],r=Ee(t,s),i=rt({displaced:o,id:t});return r?i?h<=n:d<n-c:i?h<=n+c:d<n})),inHomeList:pe(n,r)});return Te({draggable:n,insideDestination:i,destination:r,viewport:a,last:o,displacedBy:u,index:f})}({pageBorderBoxWithDroppableScroll:p,draggable:n,destination:h,insideDestination:f,last:o.displaced,viewport:a,afterCritical:s})},ot=function(e,t){var n;return Object(a.a)({},e,((n={})[t.descriptor.id]=t,n))},at=function(e){var t=e.previousImpact,n=e.impact,r=e.droppables,i=Ke(t),o=Ke(n);if(!i)return r;if(i===o)return r;var s=r[i];if(!s.subject.withPlaceholder)return r;var l=function(e){var t=e.subject.withPlaceholder;t||L(!1);var n=e.frame;if(!n){var r=ee({page:e.subject.page,axis:e.axis,frame:null,withPlaceholder:null});return Object(a.a)({},e,{subject:r})}var i=t.oldFrameMaxScroll;i||L(!1);var o=qe(n,i),s=ee({page:e.subject.page,axis:e.axis,frame:o,withPlaceholder:null});return Object(a.a)({},e,{subject:s,frame:o})}(s);return ot(r,l)},st=function(e){var t=e.state,n=e.clientSelection,r=e.dimensions,i=e.viewport,o=e.impact,s=e.scrollJumpRequest,l=i||t.viewport,u=r||t.dimensions,c=n||t.current.client.selection,d=Y(c,t.initial.client.selection),h={offset:d,selection:c,borderBoxCenter:V(t.initial.client.borderBoxCenter,d)},f={selection:V(h.selection,l.scroll.current),borderBoxCenter:V(h.borderBoxCenter,l.scroll.current),offset:V(h.offset,l.scroll.diff.value)},p={client:h,page:f};if("COLLECTING"===t.phase)return Object(a.a)({phase:"COLLECTING"},t,{dimensions:u,viewport:l,current:p});var g=u.draggables[t.critical.draggable.id],v=o||it({pageOffset:f.offset,draggable:g,draggables:u.draggables,droppables:u.droppables,previousImpact:t.impact,viewport:l,afterCritical:t.afterCritical}),m=function(e){var t=e.draggable,n=e.draggables,r=e.droppables,i=e.previousImpact,o=e.impact,a=at({previousImpact:i,impact:o,droppables:r}),s=Ke(o);if(!s)return a;var l=r[s];if(pe(t,l))return a;if(l.subject.withPlaceholder)return a;var u=Ze(l,t,n);return ot(a,u)}({draggable:g,impact:v,previousImpact:t.impact,draggables:u.draggables,droppables:u.droppables});return Object(a.a)({},t,{current:p,dimensions:{draggables:u.draggables,droppables:m},impact:v,viewport:l,scrollJumpRequest:s||null,forceShouldAnimate:!s&&null})};var lt=function(e){var t=e.impact,n=e.viewport,r=e.draggables,i=e.destination,o=e.forceShouldAnimate,s=t.displaced,l=Me({afterDragging:function(e,t){return e.map((function(e){return t[e]}))}(s.all,r),destination:i,displacedBy:t.displacedBy,viewport:n.frame,forceShouldAnimate:o,last:s});return Object(a.a)({},t,{displaced:l})},ut=function(e){var t=e.impact,n=e.draggable,r=e.droppable,i=e.draggables,o=e.viewport,a=e.afterCritical,s=ze({impact:t,draggable:n,draggables:i,droppable:r,afterCritical:a});return He({pageBorderBoxCenter:s,draggable:n,viewport:o})},ct=function(e){var t=e.state,n=e.dimensions,r=e.viewport;"SNAP"!==t.movementMode&&L(!1);var i=t.impact,o=r||t.viewport,a=n||t.dimensions,s=a.draggables,l=a.droppables,u=s[t.critical.draggable.id],c=Ke(i);c||L(!1);var d=l[c],h=lt({impact:i,viewport:o,destination:d,draggables:s}),f=ut({impact:h,draggable:u,droppable:d,draggables:s,viewport:o,afterCritical:t.afterCritical});return st({impact:h,clientSelection:f,state:t,dimensions:a,viewport:o})},dt=function(e){var t=e.draggable,n=e.home,r=e.draggables,i=e.viewport,o=Xe(n.axis,t.displaceBy),a=ce(n.descriptor.id,r),s=a.indexOf(t);-1===s&&L(!1);var l,u=a.slice(s+1),c=u.reduce((function(e,t){return e[t.descriptor.id]=!0,e}),{}),d={inVirtualList:"virtual"===n.descriptor.mode,displacedBy:o,effected:c};return{impact:{displaced:Me({afterDragging:u,destination:n,displacedBy:o,last:null,viewport:i.frame,forceShouldAnimate:!1}),displacedBy:o,at:{type:"REORDER",destination:(l=t.descriptor,{index:l.index,droppableId:l.droppableId})}},afterCritical:d}},ht=function(e){0},ft=function(e){0},pt=function(e){var t=e.additions,n=e.updatedDroppables,r=e.viewport,i=r.scroll.diff.value;return t.map((function(e){var t=e.descriptor.droppableId,o=function(e){var t=e.frame;return t||L(!1),t}(n[t]).scroll.diff.value;return function(e){var t=e.draggable,n=e.offset,r=e.initialWindowScroll,i=b(t.client,n),o=C(i,r);return Object(a.a)({},t,{placeholder:Object(a.a)({},t.placeholder,{client:i}),client:i,page:o})}({draggable:e,offset:V(i,o),initialWindowScroll:r.scroll.initial})}))},gt=function(e){return"SNAP"===e.movementMode},vt=function(e,t,n){var r=function(e,t){return{draggables:e.draggables,droppables:ot(e.droppables,t)}}(e.dimensions,t);return!gt(e)||n?st({state:e,dimensions:r}):ct({state:e,dimensions:r})};function mt(e){return e.isDragging&&"SNAP"===e.movementMode?Object(a.a)({phase:"DRAGGING"},e,{scrollJumpRequest:null}):e}var yt={phase:"IDLE",completed:null,shouldFlush:!1},bt=function(e,t){if(void 0===e&&(e=yt),"FLUSH"===t.type)return Object(a.a)({},yt,{shouldFlush:!0});if("INITIAL_PUBLISH"===t.type){"IDLE"!==e.phase&&L(!1);var n=t.payload,r=n.critical,i=n.clientSelection,o=n.viewport,s=n.dimensions,l=n.movementMode,u=s.draggables[r.draggable.id],c=s.droppables[r.droppable.id],d={selection:i,borderBoxCenter:u.client.borderBox.center,offset:U},h={client:d,page:{selection:V(d.selection,o.scroll.initial),borderBoxCenter:V(d.selection,o.scroll.initial),offset:V(d.selection,o.scroll.diff.value)}},f=le(s.droppables).every((function(e){return!e.isFixedOnPage})),p=dt({draggable:u,home:c,draggables:s.draggables,viewport:o}),g=p.impact;return{phase:"DRAGGING",isDragging:!0,critical:r,movementMode:l,dimensions:s,initial:h,current:h,isWindowScrollAllowed:f,impact:g,afterCritical:p.afterCritical,onLiftImpact:g,viewport:o,scrollJumpRequest:null,forceShouldAnimate:null}}if("COLLECTION_STARTING"===t.type)return"COLLECTING"===e.phase||"DROP_PENDING"===e.phase?e:("DRAGGING"!==e.phase&&L(!1),Object(a.a)({phase:"COLLECTING"},e,{phase:"COLLECTING"}));if("PUBLISH_WHILE_DRAGGING"===t.type)return"COLLECTING"!==e.phase&&"DROP_PENDING"!==e.phase&&L(!1),function(e){var t=e.state,n=e.published;ht();var r=n.modified.map((function(e){var n=t.dimensions.droppables[e.droppableId];return te(n,e.scroll)})),i=Object(a.a)({},t.dimensions.droppables,{},ae(r)),o=se(pt({additions:n.additions,updatedDroppables:i,viewport:t.viewport})),s=Object(a.a)({},t.dimensions.draggables,{},o);n.removals.forEach((function(e){delete s[e]}));var l={droppables:i,draggables:s},u=Ke(t.impact),c=u?l.droppables[u]:null,d=l.draggables[t.critical.draggable.id],h=l.droppables[t.critical.droppable.id],f=dt({draggable:d,home:h,draggables:s,viewport:t.viewport}),p=f.impact,g=f.afterCritical,v=c&&c.isCombineEnabled?t.impact:p,m=it({pageOffset:t.current.page.offset,draggable:l.draggables[t.critical.draggable.id],draggables:l.draggables,droppables:l.droppables,previousImpact:v,viewport:t.viewport,afterCritical:g});ft();var y=Object(a.a)({phase:"DRAGGING"},t,{phase:"DRAGGING",impact:m,onLiftImpact:p,dimensions:l,afterCritical:g,forceShouldAnimate:!1});return"COLLECTING"===t.phase?y:Object(a.a)({phase:"DROP_PENDING"},y,{phase:"DROP_PENDING",reason:t.reason,isWaiting:!1})}({state:e,published:t.payload});if("MOVE"===t.type){if("DROP_PENDING"===e.phase)return e;$e(e)||L(!1);var v=t.payload.client;return W(v,e.current.client.selection)?e:st({state:e,clientSelection:v,impact:gt(e)?e.impact:null})}if("UPDATE_DROPPABLE_SCROLL"===t.type){if("DROP_PENDING"===e.phase)return mt(e);if("COLLECTING"===e.phase)return mt(e);$e(e)||L(!1);var m=t.payload,y=m.id,b=m.newScroll,C=e.dimensions.droppables[y];if(!C)return e;var A=te(C,b);return vt(e,A,!1)}if("UPDATE_DROPPABLE_IS_ENABLED"===t.type){if("DROP_PENDING"===e.phase)return e;$e(e)||L(!1);var _=t.payload,I=_.id,x=_.isEnabled,w=e.dimensions.droppables[I];w||L(!1),w.isEnabled===x&&L(!1);var M=Object(a.a)({},w,{isEnabled:x});return vt(e,M,!0)}if("UPDATE_DROPPABLE_IS_COMBINE_ENABLED"===t.type){if("DROP_PENDING"===e.phase)return e;$e(e)||L(!1);var S=t.payload,T=S.id,E=S.isCombineEnabled,O=e.dimensions.droppables[T];O||L(!1),O.isCombineEnabled===E&&L(!1);var k=Object(a.a)({},O,{isCombineEnabled:E});return vt(e,k,!0)}if("MOVE_BY_WINDOW_SCROLL"===t.type){if("DROP_PENDING"===e.phase||"DROP_ANIMATING"===e.phase)return e;$e(e)||L(!1),e.isWindowScrollAllowed||L(!1);var D=t.payload.newScroll;if(W(e.viewport.scroll.current,D))return mt(e);var P=Fe(e.viewport,D);return gt(e)?ct({state:e,viewport:P}):st({state:e,viewport:P})}if("UPDATE_VIEWPORT_MAX_SCROLL"===t.type){if(!$e(e))return e;var N=t.payload.maxScroll;if(W(N,e.viewport.scroll.max))return e;var j=Object(a.a)({},e.viewport,{scroll:Object(a.a)({},e.viewport.scroll,{max:N})});return Object(a.a)({phase:"DRAGGING"},e,{viewport:j})}if("MOVE_UP"===t.type||"MOVE_DOWN"===t.type||"MOVE_LEFT"===t.type||"MOVE_RIGHT"===t.type){if("COLLECTING"===e.phase||"DROP_PENDING"===e.phase)return e;"DRAGGING"!==e.phase&&L(!1);var R=Je({state:e,type:t.type});return R?st({state:e,impact:R.impact,clientSelection:R.clientSelection,scrollJumpRequest:R.scrollJumpRequest}):e}if("DROP_PENDING"===t.type){var z=t.payload.reason;return"COLLECTING"!==e.phase&&L(!1),Object(a.a)({phase:"DROP_PENDING"},e,{phase:"DROP_PENDING",isWaiting:!0,reason:z})}if("DROP_ANIMATE"===t.type){var F=t.payload,B=F.completed,H=F.dropDuration,Y=F.newHomeClientOffset;return"DRAGGING"!==e.phase&&"DROP_PENDING"!==e.phase&&L(!1),{phase:"DROP_ANIMATING",completed:B,dropDuration:H,newHomeClientOffset:Y,dimensions:e.dimensions}}return"DROP_COMPLETE"===t.type?{phase:"IDLE",completed:t.payload.completed,shouldFlush:!1}:e},Ct=function(e){return{type:"PUBLISH_WHILE_DRAGGING",payload:e}},At=function(){return{type:"COLLECTION_STARTING",payload:null}},_t=function(e){return{type:"UPDATE_DROPPABLE_SCROLL",payload:e}},It=function(e){return{type:"UPDATE_DROPPABLE_IS_ENABLED",payload:e}},xt=function(e){return{type:"UPDATE_DROPPABLE_IS_COMBINE_ENABLED",payload:e}},wt=function(e){return{type:"MOVE",payload:e}},Mt=function(){return{type:"MOVE_UP",payload:null}},St=function(){return{type:"MOVE_DOWN",payload:null}},Tt=function(){return{type:"MOVE_RIGHT",payload:null}},Et=function(){return{type:"MOVE_LEFT",payload:null}},Ot=function(e){return{type:"DROP_COMPLETE",payload:e}},kt=function(e){return{type:"DROP",payload:e}},Lt=function(){return{type:"DROP_ANIMATION_FINISHED",payload:null}};var Dt="cubic-bezier(.2,1,.1,1)",Pt={drop:0,combining:.7},Nt={drop:.75},jt=.2+"s "+"cubic-bezier(0.2, 0, 0, 1)",Rt={fluid:"opacity "+jt,snap:"transform "+jt+", opacity "+jt,drop:function(e){var t=e+"s "+Dt;return"transform "+t+", opacity "+t},outOfTheWay:"transform "+jt,placeholder:"height "+jt+", width "+jt+", margin "+jt},zt=function(e){return W(e,U)?null:"translate("+e.x+"px, "+e.y+"px)"},Ft=zt,Bt=function(e,t){var n=zt(e);return n?t?n+" scale("+Nt.drop+")":n:null},Ht=.33,Ut=.55,Vt=Ut-Ht,Yt=function(e){var t=e.getState,n=e.dispatch;return function(e){return function(r){if("DROP"===r.type){var i=t(),o=r.payload.reason;if("COLLECTING"!==i.phase){if("IDLE"!==i.phase){"DROP_PENDING"===i.phase&&i.isWaiting&&L(!1),"DRAGGING"!==i.phase&&"DROP_PENDING"!==i.phase&&L(!1);var s=i.critical,l=i.dimensions,u=l.draggables[i.critical.draggable.id],c=function(e){var t=e.draggables,n=e.reason,r=e.lastImpact,i=e.home,o=e.viewport,s=e.onLiftImpact;return r.at&&"DROP"===n?"REORDER"===r.at.type?{impact:r,didDropInsideDroppable:!0}:{impact:Object(a.a)({},r,{displaced:ve}),didDropInsideDroppable:!0}:{impact:lt({draggables:t,impact:s,destination:i,viewport:o,forceShouldAnimate:!0}),didDropInsideDroppable:!1}}({reason:o,lastImpact:i.impact,afterCritical:i.afterCritical,onLiftImpact:i.onLiftImpact,home:i.dimensions.droppables[i.critical.droppable.id],viewport:i.viewport,draggables:i.dimensions.draggables}),d=c.impact,h=c.didDropInsideDroppable,f=h?de(d):null,p=h?he(d):null,g={index:s.draggable.index,droppableId:s.droppable.id},v={draggableId:u.descriptor.id,type:u.descriptor.type,source:g,reason:o,mode:i.movementMode,destination:f,combine:p},m=function(e){var t=e.impact,n=e.draggable,r=e.dimensions,i=e.viewport,o=e.afterCritical,a=r.draggables,s=r.droppables,l=Ke(t),u=l?s[l]:null,c=s[n.descriptor.droppableId],d=ut({impact:t,draggable:n,draggables:a,afterCritical:o,droppable:u||c,viewport:i});return Y(d,n.client.borderBox.center)}({impact:d,draggable:u,dimensions:l,viewport:i.viewport,afterCritical:i.afterCritical}),y={critical:i.critical,afterCritical:i.afterCritical,result:v,impact:d};if(!W(i.current.client.offset,m)||Boolean(v.combine)){var b=function(e){var t=e.current,n=e.destination,r=e.reason,i=q(t,n);if(i<=0)return Ht;if(i>=1500)return Ut;var o=Ht+Vt*(i/1500);return Number(("CANCEL"===r?.6*o:o).toFixed(2))}({current:i.current.client.offset,destination:m,reason:o});n(function(e){return{type:"DROP_ANIMATE",payload:e}}({newHomeClientOffset:m,dropDuration:b,completed:y}))}else n(Ot({completed:y}))}}else n(function(e){return{type:"DROP_PENDING",payload:e}}({reason:o}))}else e(r)}}},Wt=function(){return{x:window.pageXOffset,y:window.pageYOffset}};function Gt(e){var t=e.onWindowScroll;var n,r=w((function(){t(Wt())})),i=(n=r,{eventName:"scroll",options:{passive:!0,capture:!1},fn:function(e){e.target!==window&&e.target!==window.document||n()}}),o=E;function a(){return o!==E}return{start:function(){a()&&L(!1),o=O(window,[i])},stop:function(){a()||L(!1),r.cancel(),o(),o=E},isActive:a}}var Xt=function(e){var t=Gt({onWindowScroll:function(t){e.dispatch({type:"MOVE_BY_WINDOW_SCROLL",payload:{newScroll:t}})}});return function(e){return function(n){t.isActive()||"INITIAL_PUBLISH"!==n.type||t.start(),t.isActive()&&function(e){return"DROP_COMPLETE"===e.type||"DROP_ANIMATE"===e.type||"FLUSH"===e.type}(n)&&t.stop(),e(n)}}},qt=function(){var e=[];return{add:function(t){var n=setTimeout((function(){return function(t){var n=re(e,(function(e){return e.timerId===t}));-1===n&&L(!1),e.splice(n,1)[0].callback()}(n)})),r={timerId:n,callback:t};e.push(r)},flush:function(){if(e.length){var t=[].concat(e);e.length=0,t.forEach((function(e){clearTimeout(e.timerId),e.callback()}))}}}},Zt=function(e,t){ht(),t(),ft()},Qt=function(e,t){return{draggableId:e.draggable.id,type:e.droppable.type,source:{droppableId:e.droppable.id,index:e.draggable.index},mode:t}},Kt=function(e,t,n,r){if(e){var i=function(e){var t=!1,n=!1,r=setTimeout((function(){n=!0})),i=function(i){t||n||(t=!0,e(i),clearTimeout(r))};return i.wasCalled=function(){return t},i}(n);e(t,{announce:i}),i.wasCalled()||n(r(t))}else n(r(t))},Jt=function(e,t){var n=function(e,t){var n=qt(),r=null,i=function(n){r||L(!1),r=null,Zt(0,(function(){return Kt(e().onDragEnd,n,t,H)}))};return{beforeCapture:function(t,n){r&&L(!1),Zt(0,(function(){var r=e().onBeforeCapture;r&&r({draggableId:t,mode:n})}))},beforeStart:function(t,n){r&&L(!1),Zt(0,(function(){var r=e().onBeforeDragStart;r&&r(Qt(t,n))}))},start:function(i,o){r&&L(!1);var a=Qt(i,o);r={mode:o,lastCritical:i,lastLocation:a.source,lastCombine:null},n.add((function(){Zt(0,(function(){return Kt(e().onDragStart,a,t,F)}))}))},update:function(i,o){var s=de(o),l=he(o);r||L(!1);var u=!function(e,t){if(e===t)return!0;var n=e.draggable.id===t.draggable.id&&e.draggable.droppableId===t.draggable.droppableId&&e.draggable.type===t.draggable.type&&e.draggable.index===t.draggable.index,r=e.droppable.id===t.droppable.id&&e.droppable.type===t.droppable.type;return n&&r}(i,r.lastCritical);u&&(r.lastCritical=i);var c,d,h=(c=r.lastLocation,d=s,!(null==c&&null==d||null!=c&&null!=d&&c.droppableId===d.droppableId&&c.index===d.index));h&&(r.lastLocation=s);var f=!function(e,t){return null==e&&null==t||null!=e&&null!=t&&(e.draggableId===t.draggableId&&e.droppableId===t.droppableId)}(r.lastCombine,l);if(f&&(r.lastCombine=l),u||h||f){var p=Object(a.a)({},Qt(i,r.mode),{combine:l,destination:s});n.add((function(){Zt(0,(function(){return Kt(e().onDragUpdate,p,t,B)}))}))}},flush:function(){r||L(!1),n.flush()},drop:i,abort:function(){if(r){var e=Object(a.a)({},Qt(r.lastCritical,r.mode),{combine:null,destination:null,reason:"CANCEL"});i(e)}}}}(e,t);return function(e){return function(t){return function(r){if("BEFORE_INITIAL_CAPTURE"!==r.type){if("INITIAL_PUBLISH"===r.type){var i=r.payload.critical;return n.beforeStart(i,r.payload.movementMode),t(r),void n.start(i,r.payload.movementMode)}if("DROP_COMPLETE"===r.type){var o=r.payload.completed.result;return n.flush(),t(r),void n.drop(o)}if(t(r),"FLUSH"!==r.type){var a=e.getState();"DRAGGING"===a.phase&&n.update(a.critical,a.impact)}else n.abort()}else n.beforeCapture(r.payload.draggableId,r.payload.movementMode)}}}},$t=function(e){return function(t){return function(n){if("DROP_ANIMATION_FINISHED"===n.type){var r=e.getState();"DROP_ANIMATING"!==r.phase&&L(!1),e.dispatch(Ot({completed:r.completed}))}else t(n)}}},en=function(e){var t=null,n=null;return function(r){return function(i){if("FLUSH"!==i.type&&"DROP_COMPLETE"!==i.type&&"DROP_ANIMATION_FINISHED"!==i.type||(n&&(cancelAnimationFrame(n),n=null),t&&(t(),t=null)),r(i),"DROP_ANIMATE"===i.type){var o={eventName:"scroll",options:{capture:!0,passive:!1,once:!0},fn:function(){"DROP_ANIMATING"===e.getState().phase&&e.dispatch({type:"DROP_ANIMATION_FINISHED",payload:null})}};n=requestAnimationFrame((function(){n=null,t=O(window,[o])}))}}}},tn=function(e){return function(t){return function(n){if(t(n),"PUBLISH_WHILE_DRAGGING"===n.type){var r=e.getState();"DROP_PENDING"===r.phase&&(r.isWaiting||e.dispatch(kt({reason:r.reason})))}}}},nn=s.compose,rn=function(e){var t,n=e.dimensionMarshal,r=e.focusMarshal,i=e.styleMarshal,o=e.getResponders,a=e.announce,l=e.autoScroller;return Object(s.createStore)(bt,nn(Object(s.applyMiddleware)((t=i,function(){return function(e){return function(n){"INITIAL_PUBLISH"===n.type&&t.dragging(),"DROP_ANIMATE"===n.type&&t.dropping(n.payload.completed.result.reason),"FLUSH"!==n.type&&"DROP_COMPLETE"!==n.type||t.resting(),e(n)}}}),function(e){return function(){return function(t){return function(n){"DROP_COMPLETE"!==n.type&&"FLUSH"!==n.type&&"DROP_ANIMATE"!==n.type||e.stopPublishing(),t(n)}}}}(n),function(e){return function(t){var n=t.getState,r=t.dispatch;return function(t){return function(i){if("LIFT"===i.type){var o=i.payload,a=o.id,s=o.clientSelection,l=o.movementMode,u=n();"DROP_ANIMATING"===u.phase&&r(Ot({completed:u.completed})),"IDLE"!==n().phase&&L(!1),r({type:"FLUSH",payload:null}),r({type:"BEFORE_INITIAL_CAPTURE",payload:{draggableId:a,movementMode:l}});var c={draggableId:a,scrollOptions:{shouldPublishImmediately:"SNAP"===l}},d=e.startPublishing(c),h=d.critical,f=d.dimensions,p=d.viewport;r(function(e){return{type:"INITIAL_PUBLISH",payload:e}}({critical:h,dimensions:f,clientSelection:s,movementMode:l,viewport:p}))}else t(i)}}}}(n),Yt,$t,en,tn,function(e){return function(t){return function(n){return function(r){if(function(e){return"DROP_COMPLETE"===e.type||"DROP_ANIMATE"===e.type||"FLUSH"===e.type}(r))return e.stop(),void n(r);if("INITIAL_PUBLISH"===r.type){n(r);var i=t.getState();return"DRAGGING"!==i.phase&&L(!1),void e.start(i)}n(r),e.scroll(t.getState())}}}}(l),Xt,function(e){var t=!1;return function(){return function(n){return function(r){if("INITIAL_PUBLISH"===r.type)return t=!0,e.tryRecordFocus(r.payload.critical.draggable.id),n(r),void e.tryRestoreFocusRecorded();if(n(r),t){if("FLUSH"===r.type)return t=!1,void e.tryRestoreFocusRecorded();if("DROP_COMPLETE"===r.type){t=!1;var i=r.payload.completed.result;i.combine&&e.tryShiftRecord(i.draggableId,i.combine.draggableId),e.tryRestoreFocusRecorded()}}}}}}(r),Jt(o,a))))};var on=function(e){var t=e.scrollHeight,n=e.scrollWidth,r=e.height,i=e.width,o=Y({x:n,y:t},{x:i,y:r});return{x:Math.max(0,o.x),y:Math.max(0,o.y)}},an=function(){var e=document.documentElement;return e||L(!1),e},sn=function(){var e=an();return on({scrollHeight:e.scrollHeight,scrollWidth:e.scrollWidth,width:e.clientWidth,height:e.clientHeight})},ln=function(e){var t=e.critical,n=e.scrollOptions,r=e.registry;ht();var i,o,a,s,l,u,c,d=(i=Wt(),o=sn(),a=i.y,s=i.x,l=an(),u=l.clientWidth,c=l.clientHeight,{frame:f({top:a,left:s,right:s+u,bottom:a+c}),scroll:{initial:i,current:i,max:o,diff:{value:U,displacement:U}}}),h=d.scroll.current,p=t.droppable,g=r.droppable.getAllByType(p.type).map((function(e){return e.callbacks.getDimensionAndWatchScroll(h,n)})),v=r.draggable.getAllByType(t.draggable.type).map((function(e){return e.getDimension(h)})),m={draggables:se(v),droppables:ae(g)};return ft(),{dimensions:m,critical:t,viewport:d}};function un(e,t,n){return n.descriptor.id!==t.id&&(n.descriptor.type===t.type&&"virtual"===e.droppable.getById(n.descriptor.droppableId).descriptor.mode)}var cn,dn,hn=function(e,t){var n=null,r=function(e){var t=e.registry,n=e.callbacks,r={additions:{},removals:{},modified:{}},i=null,o=function(){i||(n.collectionStarting(),i=requestAnimationFrame((function(){i=null,ht();var e=r,o=e.additions,a=e.removals,s=e.modified,l=Object.keys(o).map((function(e){return t.draggable.getById(e).getDimension(U)})).sort((function(e,t){return e.descriptor.index-t.descriptor.index})),u=Object.keys(s).map((function(e){return{droppableId:e,scroll:t.droppable.getById(e).callbacks.getScrollWhileDragging()}})),c={additions:l,removals:Object.keys(a),modified:u};r={additions:{},removals:{},modified:{}},ft(),n.publish(c)})))};return{add:function(e){var t=e.descriptor.id;r.additions[t]=e,r.modified[e.descriptor.droppableId]=!0,r.removals[t]&&delete r.removals[t],o()},remove:function(e){var t=e.descriptor;r.removals[t.id]=!0,r.modified[t.droppableId]=!0,r.additions[t.id]&&delete r.additions[t.id],o()},stop:function(){i&&(cancelAnimationFrame(i),i=null,r={additions:{},removals:{},modified:{}})}}}({callbacks:{publish:t.publishWhileDragging,collectionStarting:t.collectionStarting},registry:e}),i=function(t){n||L(!1);var i=n.critical.draggable;"ADDITION"===t.type&&un(e,i,t.value)&&r.add(t.value),"REMOVAL"===t.type&&un(e,i,t.value)&&r.remove(t.value)};return{updateDroppableIsEnabled:function(r,i){e.droppable.exists(r)||L(!1),n&&t.updateDroppableIsEnabled({id:r,isEnabled:i})},updateDroppableIsCombineEnabled:function(r,i){n&&(e.droppable.exists(r)||L(!1),t.updateDroppableIsCombineEnabled({id:r,isCombineEnabled:i}))},scrollDroppable:function(t,r){n&&e.droppable.getById(t).callbacks.scroll(r)},updateDroppableScroll:function(r,i){n&&(e.droppable.exists(r)||L(!1),t.updateDroppableScroll({id:r,newScroll:i}))},startPublishing:function(t){n&&L(!1);var r=e.draggable.getById(t.draggableId),o=e.droppable.getById(r.descriptor.droppableId),a={draggable:r.descriptor,droppable:o.descriptor},s=e.subscribe(i);return n={critical:a,unsubscribe:s},ln({critical:a,registry:e,scrollOptions:t.scrollOptions})},stopPublishing:function(){if(n){r.stop();var t=n.critical.droppable;e.droppable.getAllByType(t.type).forEach((function(e){return e.callbacks.dragStopped()})),n.unsubscribe(),n=null}}}},fn=function(e,t){return"IDLE"===e.phase||"DROP_ANIMATING"===e.phase&&(e.completed.result.draggableId!==t&&"DROP"===e.completed.result.reason)},pn=function(e){window.scrollBy(e.x,e.y)},gn=x((function(e){return le(e).filter((function(e){return!!e.isEnabled&&!!e.frame}))})),vn=function(e){var t=e.center,n=e.destination,r=e.droppables;if(n){var i=r[n];return i.frame?i:null}return function(e,t){return ie(gn(t),(function(t){return t.frame||L(!1),et(t.frame.pageMarginBox)(e)}))}(t,r)},mn=.25,yn=.05,bn=28,Cn=function(e){return Math.pow(e,2)},An={stopDampeningAt:1200,accelerateAt:360},_n=function(e){var t=e.startOfRange,n=e.endOfRange,r=e.current,i=n-t;return 0===i?0:(r-t)/i},In=An.accelerateAt,xn=An.stopDampeningAt,wn=function(e){var t=e.distanceToEdge,n=e.thresholds,r=e.dragStartTime,i=e.shouldUseTimeDampening,o=function(e,t){if(e>t.startScrollingFrom)return 0;if(e<=t.maxScrollValueAt)return bn;if(e===t.startScrollingFrom)return 1;var n=_n({startOfRange:t.maxScrollValueAt,endOfRange:t.startScrollingFrom,current:e}),r=bn*Cn(1-n);return Math.ceil(r)}(t,n);return 0===o?0:i?Math.max(function(e,t){var n=t,r=xn,i=Date.now()-n;if(i>=xn)return e;if(i<In)return 1;var o=_n({startOfRange:In,endOfRange:r,current:i}),a=e*Cn(o);return Math.ceil(a)}(o,r),1):o},Mn=function(e){var t=e.container,n=e.distanceToEdges,r=e.dragStartTime,i=e.axis,o=e.shouldUseTimeDampening,a=function(e,t){return{startScrollingFrom:e[t.size]*mn,maxScrollValueAt:e[t.size]*yn}}(t,i);return n[i.end]<n[i.start]?wn({distanceToEdge:n[i.end],thresholds:a,dragStartTime:r,shouldUseTimeDampening:o}):-1*wn({distanceToEdge:n[i.start],thresholds:a,dragStartTime:r,shouldUseTimeDampening:o})},Sn=Q((function(e){return 0===e?0:e})),Tn=function(e){var t=e.dragStartTime,n=e.container,r=e.subject,i=e.center,o=e.shouldUseTimeDampening,a={top:i.y-n.top,right:n.right-i.x,bottom:n.bottom-i.y,left:i.x-n.left},s=Mn({container:n,distanceToEdges:a,dragStartTime:t,axis:Ae,shouldUseTimeDampening:o}),l=Mn({container:n,distanceToEdges:a,dragStartTime:t,axis:_e,shouldUseTimeDampening:o}),u=Sn({x:l,y:s});if(W(u,U))return null;var c=function(e){var t=e.container,n=e.subject,r=e.proposedScroll,i=n.height>t.height,o=n.width>t.width;return o||i?o&&i?null:{x:o?0:r.x,y:i?0:r.y}:r}({container:n,subject:r,proposedScroll:u});return c?W(c,U)?null:c:null},En=Q((function(e){return 0===e?0:e>0?1:-1})),On=(cn=function(e,t){return e<0?e:e>t?e-t:0},function(e){var t=e.current,n=e.max,r=e.change,i=V(t,r),o={x:cn(i.x,n.x),y:cn(i.y,n.y)};return W(o,U)?null:o}),kn=function(e){var t=e.max,n=e.current,r=e.change,i={x:Math.max(n.x,t.x),y:Math.max(n.y,t.y)},o=En(r),a=On({max:i,current:n,change:o});return!a||(0!==o.x&&0===a.x||0!==o.y&&0===a.y)},Ln=function(e,t){return kn({current:e.scroll.current,max:e.scroll.max,change:t})},Dn=function(e,t){var n=e.frame;return!!n&&kn({current:n.scroll.current,max:n.scroll.max,change:t})},Pn=function(e){var t=e.state,n=e.dragStartTime,r=e.shouldUseTimeDampening,i=e.scrollWindow,o=e.scrollDroppable,a=t.current.page.borderBoxCenter,s=t.dimensions.draggables[t.critical.draggable.id].page.marginBox;if(t.isWindowScrollAllowed){var l=function(e){var t=e.viewport,n=e.subject,r=e.center,i=e.dragStartTime,o=e.shouldUseTimeDampening,a=Tn({dragStartTime:i,container:t.frame,subject:n,center:r,shouldUseTimeDampening:o});return a&&Ln(t,a)?a:null}({dragStartTime:n,viewport:t.viewport,subject:s,center:a,shouldUseTimeDampening:r});if(l)return void i(l)}var u=vn({center:a,destination:Ke(t.impact),droppables:t.dimensions.droppables});if(u){var c=function(e){var t=e.droppable,n=e.subject,r=e.center,i=e.dragStartTime,o=e.shouldUseTimeDampening,a=t.frame;if(!a)return null;var s=Tn({dragStartTime:i,container:a.pageMarginBox,subject:n,center:r,shouldUseTimeDampening:o});return s&&Dn(t,s)?s:null}({dragStartTime:n,droppable:u,subject:s,center:a,shouldUseTimeDampening:r});c&&o(u.descriptor.id,c)}},Nn=function(e){var t=e.move,n=e.scrollDroppable,r=e.scrollWindow,i=function(e,t){if(!Dn(e,t))return t;var r=function(e,t){var n=e.frame;return n&&Dn(e,t)?On({current:n.scroll.current,max:n.scroll.max,change:t}):null}(e,t);if(!r)return n(e.descriptor.id,t),null;var i=Y(t,r);return n(e.descriptor.id,i),Y(t,i)},o=function(e,t,n){if(!e)return n;if(!Ln(t,n))return n;var i=function(e,t){if(!Ln(e,t))return null;var n=e.scroll.max,r=e.scroll.current;return On({current:r,max:n,change:t})}(t,n);if(!i)return r(n),null;var o=Y(n,i);return r(o),Y(n,o)};return function(e){var n=e.scrollJumpRequest;if(n){var r=Ke(e.impact);r||L(!1);var a=i(e.dimensions.droppables[r],n);if(a){var s=e.viewport,l=o(e.isWindowScrollAllowed,s,a);l&&function(e,n){var r=V(e.current.client.selection,n);t({client:r})}(e,l)}}}},jn=function(e){var t=e.scrollDroppable,n=e.scrollWindow,r=e.move,i=function(e){var t=e.scrollWindow,n=e.scrollDroppable,r=w(t),i=w(n),o=null,a=function(e){o||L(!1);var t=o,n=t.shouldUseTimeDampening,a=t.dragStartTime;Pn({state:e,scrollWindow:r,scrollDroppable:i,dragStartTime:a,shouldUseTimeDampening:n})};return{start:function(e){ht(),o&&L(!1);var t=Date.now(),n=!1,r=function(){n=!0};Pn({state:e,dragStartTime:0,shouldUseTimeDampening:!1,scrollWindow:r,scrollDroppable:r}),o={dragStartTime:t,shouldUseTimeDampening:n},ft(),n&&a(e)},stop:function(){o&&(r.cancel(),i.cancel(),o=null)},scroll:a}}({scrollWindow:n,scrollDroppable:t}),o=Nn({move:r,scrollWindow:n,scrollDroppable:t});return{scroll:function(e){"DRAGGING"===e.phase&&("FLUID"!==e.movementMode?e.scrollJumpRequest&&o(e):i.scroll(e))},start:i.start,stop:i.stop}},Rn={base:dn="data-rbd-drag-handle",draggableId:dn+"-draggable-id",contextId:dn+"-context-id"},zn=function(){var e="data-rbd-draggable";return{base:e,contextId:e+"-context-id",id:e+"-id"}}(),Fn=function(){var e="data-rbd-droppable";return{base:e,contextId:e+"-context-id",id:e+"-id"}}(),Bn={contextId:"data-rbd-scroll-container-context-id"},Hn=function(e,t){return e.map((function(e){var n=e.styles[t];return n?e.selector+" { "+n+" }":""})).join(" ")},Un="undefined"!=typeof window&&void 0!==window.document&&void 0!==window.document.createElement?r.useLayoutEffect:r.useEffect,Vn=function(){var e=document.querySelector("head");return e||L(!1),e},Yn=function(e){var t=document.createElement("style");return e&&t.setAttribute("nonce",e),t.type="text/css",t};function Wn(e,t){var n=c((function(){return function(e){var t,n,r,i=(t=e,function(e){return"["+e+'="'+t+'"]'}),o=(n="\n      cursor: -webkit-grab;\n      cursor: grab;\n    ",{selector:i(Rn.contextId),styles:{always:"\n          -webkit-touch-callout: none;\n          -webkit-tap-highlight-color: rgba(0,0,0,0);\n          touch-action: manipulation;\n        ",resting:n,dragging:"pointer-events: none;",dropAnimating:n}}),a=[(r="\n      transition: "+Rt.outOfTheWay+";\n    ",{selector:i(zn.contextId),styles:{dragging:r,dropAnimating:r,userCancel:r}}),o,{selector:i(Fn.contextId),styles:{always:"overflow-anchor: none;"}},{selector:"body",styles:{dragging:"\n        cursor: grabbing;\n        cursor: -webkit-grabbing;\n        user-select: none;\n        -webkit-user-select: none;\n        -moz-user-select: none;\n        -ms-user-select: none;\n        overflow-anchor: none;\n      "}}];return{always:Hn(a,"always"),resting:Hn(a,"resting"),dragging:Hn(a,"dragging"),dropAnimating:Hn(a,"dropAnimating"),userCancel:Hn(a,"userCancel")}}(e)}),[e]),i=Object(r.useRef)(null),o=Object(r.useRef)(null),a=d(x((function(e){var t=o.current;t||L(!1),t.textContent=e})),[]),s=d((function(e){var t=i.current;t||L(!1),t.textContent=e}),[]);Un((function(){(i.current||o.current)&&L(!1);var r=Yn(t),l=Yn(t);return i.current=r,o.current=l,r.setAttribute("data-rbd-always",e),l.setAttribute("data-rbd-dynamic",e),Vn().appendChild(r),Vn().appendChild(l),s(n.always),a(n.resting),function(){var e=function(e){var t=e.current;t||L(!1),Vn().removeChild(t),e.current=null};e(i),e(o)}}),[t,s,a,n.always,n.resting,e]);var l=d((function(){return a(n.dragging)}),[a,n.dragging]),u=d((function(e){a("DROP"!==e?n.userCancel:n.dropAnimating)}),[a,n.dropAnimating,n.userCancel]),h=d((function(){o.current&&a(n.resting)}),[a,n.resting]);return c((function(){return{dragging:l,dropping:u,resting:h}}),[l,u,h])}var Gn=function(e){return e&&e.ownerDocument?e.ownerDocument.defaultView:window};function Xn(e){return e instanceof Gn(e).HTMLElement}function qn(e,t){var n="["+Rn.contextId+'="'+e+'"]',r=oe(document.querySelectorAll(n));if(!r.length)return null;var i=ie(r,(function(e){return e.getAttribute(Rn.draggableId)===t}));return i&&Xn(i)?i:null}function Zn(){var e={draggables:{},droppables:{}},t=[];function n(e){t.length&&t.forEach((function(t){return t(e)}))}function r(t){return e.draggables[t]||null}function i(t){return e.droppables[t]||null}return{draggable:{register:function(t){e.draggables[t.descriptor.id]=t,n({type:"ADDITION",value:t})},update:function(t,n){var r=e.draggables[n.descriptor.id];r&&r.uniqueId===t.uniqueId&&(delete e.draggables[n.descriptor.id],e.draggables[t.descriptor.id]=t)},unregister:function(t){var i=t.descriptor.id,o=r(i);o&&t.uniqueId===o.uniqueId&&(delete e.draggables[i],n({type:"REMOVAL",value:t}))},getById:function(e){var t=r(e);return t||L(!1),t},findById:r,exists:function(e){return Boolean(r(e))},getAllByType:function(t){return ne(e.draggables).filter((function(e){return e.descriptor.type===t}))}},droppable:{register:function(t){e.droppables[t.descriptor.id]=t},unregister:function(t){var n=i(t.descriptor.id);n&&t.uniqueId===n.uniqueId&&delete e.droppables[t.descriptor.id]},getById:function(e){var t=i(e);return t||L(!1),t},findById:i,exists:function(e){return Boolean(i(e))},getAllByType:function(t){return ne(e.droppables).filter((function(e){return e.descriptor.type===t}))}},subscribe:function(e){return t.push(e),function(){var n=t.indexOf(e);-1!==n&&t.splice(n,1)}},clean:function(){e.draggables={},e.droppables={},t.length=0}}}var Qn=i.a.createContext(null),Kn=function(){var e=document.body;return e||L(!1),e},Jn={position:"absolute",width:"1px",height:"1px",margin:"-1px",border:"0",padding:"0",overflow:"hidden",clip:"rect(0 0 0 0)","clip-path":"inset(100%)"};var $n=0,er={separator:"::"};function tr(e,t){return void 0===t&&(t=er),c((function(){return""+e+t.separator+$n++}),[t.separator,e])}var nr=i.a.createContext(null);function rr(e){0}function ir(e,t){rr()}function or(){ir()}function ar(e){var t=Object(r.useRef)(e);return Object(r.useEffect)((function(){t.current=e})),t}var sr,lr=((sr={})[13]=!0,sr[9]=!0,sr),ur=function(e){lr[e.keyCode]&&e.preventDefault()},cr=function(){var e="visibilitychange";return"undefined"==typeof document?e:ie([e,"ms"+e,"webkit"+e,"moz"+e,"o"+e],(function(e){return"on"+e in document}))||e}();var dr,hr={type:"IDLE"};function fr(e){var t=e.cancel,n=e.completed,r=e.getPhase,i=e.setPhase;return[{eventName:"mousemove",fn:function(e){var t=e.button,n=e.clientX,o=e.clientY;if(0===t){var a={x:n,y:o},s=r();if("DRAGGING"===s.type)return e.preventDefault(),void s.actions.move(a);"PENDING"!==s.type&&L(!1);var l=s.point;if(u=l,c=a,Math.abs(c.x-u.x)>=5||Math.abs(c.y-u.y)>=5){var u,c;e.preventDefault();var d=s.actions.fluidLift(a);i({type:"DRAGGING",actions:d})}}}},{eventName:"mouseup",fn:function(e){var i=r();"DRAGGING"===i.type?(e.preventDefault(),i.actions.drop({shouldBlockNextClick:!0}),n()):t()}},{eventName:"mousedown",fn:function(e){"DRAGGING"===r().type&&e.preventDefault(),t()}},{eventName:"keydown",fn:function(e){if("PENDING"!==r().type)return 27===e.keyCode?(e.preventDefault(),void t()):void ur(e);t()}},{eventName:"resize",fn:t},{eventName:"scroll",options:{passive:!0,capture:!1},fn:function(){"PENDING"===r().type&&t()}},{eventName:"webkitmouseforcedown",fn:function(e){var n=r();"IDLE"===n.type&&L(!1),n.actions.shouldRespectForcePress()?t():e.preventDefault()}},{eventName:cr,fn:t}]}function pr(){}var gr=((dr={})[34]=!0,dr[33]=!0,dr[36]=!0,dr[35]=!0,dr);function vr(e,t){function n(){t(),e.cancel()}return[{eventName:"keydown",fn:function(r){return 27===r.keyCode?(r.preventDefault(),void n()):32===r.keyCode?(r.preventDefault(),t(),void e.drop()):40===r.keyCode?(r.preventDefault(),void e.moveDown()):38===r.keyCode?(r.preventDefault(),void e.moveUp()):39===r.keyCode?(r.preventDefault(),void e.moveRight()):37===r.keyCode?(r.preventDefault(),void e.moveLeft()):void(gr[r.keyCode]?r.preventDefault():ur(r))}},{eventName:"mousedown",fn:n},{eventName:"mouseup",fn:n},{eventName:"click",fn:n},{eventName:"touchstart",fn:n},{eventName:"resize",fn:n},{eventName:"wheel",fn:n,options:{passive:!0}},{eventName:cr,fn:n}]}var mr={type:"IDLE"};var yr={input:!0,button:!0,textarea:!0,select:!0,option:!0,optgroup:!0,video:!0,audio:!0};function br(e,t){var n=t.target;return!!Xn(n)&&function e(t,n){if(null==n)return!1;if(Boolean(yr[n.tagName.toLowerCase()]))return!0;var r=n.getAttribute("contenteditable");return"true"===r||""===r||n!==t&&e(t,n.parentElement)}(e,n)}var Cr=function(e){return f(e.getBoundingClientRect()).center};var Ar="undefined"==typeof document?"matches":ie(["matches","msMatchesSelector","webkitMatchesSelector"],(function(e){return e in Element.prototype}))||"matches";function _r(e,t){return e.closest?e.closest(t):function e(t,n){return null==t?null:t[Ar](n)?t:e(t.parentElement,n)}(e,t)}function Ir(e,t){var n,r=t.target;if(!((n=r)instanceof Gn(n).Element))return null;var i=_r(r,function(e){return"["+Rn.contextId+'="'+e+'"]'}(e));return i&&Xn(i)?i:null}function xr(e){e.preventDefault()}function wr(e){var t=e.expected,n=e.phase,r=e.isLockActive;e.shouldWarn;return!!r()&&t===n}function Mr(e){var t=e.lockAPI,n=e.store,r=e.registry,i=e.draggableId;if(t.isClaimed())return!1;var o=r.draggable.findById(i);return!!o&&(!!o.options.isEnabled&&!!fn(n.getState(),i))}function Sr(e){var t=e.lockAPI,n=e.contextId,r=e.store,i=e.registry,o=e.draggableId,s=e.forceSensorStop,l=e.sourceEvent;if(!Mr({lockAPI:t,store:r,registry:i,draggableId:o}))return null;var u=i.draggable.getById(o),c=function(e,t){var n="["+zn.contextId+'="'+e+'"]',r=ie(oe(document.querySelectorAll(n)),(function(e){return e.getAttribute(zn.id)===t}));return r&&Xn(r)?r:null}(n,u.descriptor.id);if(!c)return null;if(l&&!u.options.canDragInteractiveElements&&br(c,l))return null;var d=t.claim(s||E),h="PRE_DRAG";function f(){return u.options.shouldRespectForcePress}function p(){return t.isActive(d)}var g=function(e,t){wr({expected:e,phase:h,isLockActive:p,shouldWarn:!0})&&r.dispatch(t())}.bind(null,"DRAGGING");function v(e){function n(){t.release(),h="COMPLETED"}function i(t,i){if(void 0===i&&(i={shouldBlockNextClick:!1}),e.cleanup(),i.shouldBlockNextClick){var o=O(window,[{eventName:"click",fn:xr,options:{once:!0,passive:!1,capture:!0}}]);setTimeout(o)}n(),r.dispatch(kt({reason:t}))}return"PRE_DRAG"!==h&&(n(),"PRE_DRAG"!==h&&L(!1)),r.dispatch(function(e){return{type:"LIFT",payload:e}}(e.liftActionArgs)),h="DRAGGING",Object(a.a)({isActive:function(){return wr({expected:"DRAGGING",phase:h,isLockActive:p,shouldWarn:!1})},shouldRespectForcePress:f,drop:function(e){return i("DROP",e)},cancel:function(e){return i("CANCEL",e)}},e.actions)}return{isActive:function(){return wr({expected:"PRE_DRAG",phase:h,isLockActive:p,shouldWarn:!1})},shouldRespectForcePress:f,fluidLift:function(e){var t=w((function(e){g((function(){return wt({client:e})}))})),n=v({liftActionArgs:{id:o,clientSelection:e,movementMode:"FLUID"},cleanup:function(){return t.cancel()},actions:{move:t}});return Object(a.a)({},n,{move:t})},snapLift:function(){var e={moveUp:function(){return g(Mt)},moveRight:function(){return g(Tt)},moveDown:function(){return g(St)},moveLeft:function(){return g(Et)}};return v({liftActionArgs:{id:o,clientSelection:Cr(c),movementMode:"SNAP"},cleanup:E,actions:e})},abort:function(){wr({expected:"PRE_DRAG",phase:h,isLockActive:p,shouldWarn:!0})&&t.release()}}}var Tr=[function(e){var t=Object(r.useRef)(hr),n=Object(r.useRef)(E),i=c((function(){return{eventName:"mousedown",fn:function(t){if(!t.defaultPrevented&&0===t.button&&!(t.ctrlKey||t.metaKey||t.shiftKey||t.altKey)){var r=e.findClosestDraggableId(t);if(r){var i=e.tryGetLock(r,s,{sourceEvent:t});if(i){t.preventDefault();var o={x:t.clientX,y:t.clientY};n.current(),h(i,o)}}}}}}),[e]),o=c((function(){return{eventName:"webkitmouseforcewillbegin",fn:function(t){if(!t.defaultPrevented){var n=e.findClosestDraggableId(t);if(n){var r=e.findOptionsForDraggable(n);r&&(r.shouldRespectForcePress||e.canGetLock(n)&&t.preventDefault())}}}}}),[e]),a=d((function(){n.current=O(window,[o,i],{passive:!1,capture:!0})}),[o,i]),s=d((function(){"IDLE"!==t.current.type&&(t.current=hr,n.current(),a())}),[a]),l=d((function(){var e=t.current;s(),"DRAGGING"===e.type&&e.actions.cancel({shouldBlockNextClick:!0}),"PENDING"===e.type&&e.actions.abort()}),[s]),u=d((function(){var e=fr({cancel:l,completed:s,getPhase:function(){return t.current},setPhase:function(e){t.current=e}});n.current=O(window,e,{capture:!0,passive:!1})}),[l,s]),h=d((function(e,n){"IDLE"!==t.current.type&&L(!1),t.current={type:"PENDING",point:n,actions:e},u()}),[u]);Un((function(){return a(),function(){n.current()}}),[a])},function(e){var t=Object(r.useRef)(pr),n=c((function(){return{eventName:"keydown",fn:function(n){if(!n.defaultPrevented&&32===n.keyCode){var r=e.findClosestDraggableId(n);if(r){var o=e.tryGetLock(r,l,{sourceEvent:n});if(o){n.preventDefault();var a=!0,s=o.snapLift();t.current(),t.current=O(window,vr(s,l),{capture:!0,passive:!1})}}}function l(){a||L(!1),a=!1,t.current(),i()}}}}),[e]),i=d((function(){t.current=O(window,[n],{passive:!1,capture:!0})}),[n]);Un((function(){return i(),function(){t.current()}}),[i])},function(e){var t=Object(r.useRef)(mr),n=Object(r.useRef)(E),i=d((function(){return t.current}),[]),o=d((function(e){t.current=e}),[]),a=c((function(){return{eventName:"touchstart",fn:function(t){if(!t.defaultPrevented){var r=e.findClosestDraggableId(t);if(r){var i=e.tryGetLock(r,l,{sourceEvent:t});if(i){var o=t.touches[0],a={x:o.clientX,y:o.clientY};n.current(),p(i,a)}}}}}}),[e]),s=d((function(){n.current=O(window,[a],{capture:!0,passive:!1})}),[a]),l=d((function(){var e=t.current;"IDLE"!==e.type&&("PENDING"===e.type&&clearTimeout(e.longPressTimerId),o(mr),n.current(),s())}),[s,o]),u=d((function(){var e=t.current;l(),"DRAGGING"===e.type&&e.actions.cancel({shouldBlockNextClick:!0}),"PENDING"===e.type&&e.actions.abort()}),[l]),h=d((function(){var e={capture:!0,passive:!1},t={cancel:u,completed:l,getPhase:i},r=O(window,function(e){var t=e.cancel,n=e.completed,r=e.getPhase;return[{eventName:"touchmove",options:{capture:!1},fn:function(e){var n=r();if("DRAGGING"===n.type){n.hasMoved=!0;var i=e.touches[0],o={x:i.clientX,y:i.clientY};e.preventDefault(),n.actions.move(o)}else t()}},{eventName:"touchend",fn:function(e){var i=r();"DRAGGING"===i.type?(e.preventDefault(),i.actions.drop({shouldBlockNextClick:!0}),n()):t()}},{eventName:"touchcancel",fn:function(e){"DRAGGING"===r().type?(e.preventDefault(),t()):t()}},{eventName:"touchforcechange",fn:function(e){var n=r();"IDLE"===n.type&&L(!1);var i=e.touches[0];if(i&&i.force>=.15){var o=n.actions.shouldRespectForcePress();if("PENDING"!==n.type)return o?n.hasMoved?void e.preventDefault():void t():void e.preventDefault();o&&t()}}},{eventName:cr,fn:t}]}(t),e),o=O(window,function(e){var t=e.cancel,n=e.getPhase;return[{eventName:"orientationchange",fn:t},{eventName:"resize",fn:t},{eventName:"contextmenu",fn:function(e){e.preventDefault()}},{eventName:"keydown",fn:function(e){"DRAGGING"===n().type?(27===e.keyCode&&e.preventDefault(),t()):t()}},{eventName:cr,fn:t}]}(t),e);n.current=function(){r(),o()}}),[u,i,l]),f=d((function(){var e=i();"PENDING"!==e.type&&L(!1);var t=e.actions.fluidLift(e.point);o({type:"DRAGGING",actions:t,hasMoved:!1})}),[i,o]),p=d((function(e,t){"IDLE"!==i().type&&L(!1);var n=setTimeout(f,120);o({type:"PENDING",point:t,actions:e,longPressTimerId:n}),h()}),[h,i,o,f]);Un((function(){return s(),function(){n.current();var e=i();"PENDING"===e.type&&(clearTimeout(e.longPressTimerId),o(mr))}}),[i,s,o]),Un((function(){return O(window,[{eventName:"touchmove",fn:function(){},options:{capture:!1,passive:!1}}])}),[])}];function Er(e){var t=e.contextId,n=e.store,i=e.registry,o=e.customSensors,a=e.enableDefaultSensors,s=[].concat(a?Tr:[],o||[]),l=Object(r.useState)((function(){return function(){var e=null;function t(){e||L(!1),e=null}return{isClaimed:function(){return Boolean(e)},isActive:function(t){return t===e},claim:function(t){e&&L(!1);var n={abandon:t};return e=n,n},release:t,tryAbandon:function(){e&&(e.abandon(),t())}}}()}))[0],u=d((function(e,t){e.isDragging&&!t.isDragging&&l.tryAbandon()}),[l]);Un((function(){var e=n.getState();return n.subscribe((function(){var t=n.getState();u(e,t),e=t}))}),[l,n,u]),Un((function(){return l.tryAbandon}),[l.tryAbandon]);var h=d((function(e){return Mr({lockAPI:l,registry:i,store:n,draggableId:e})}),[l,i,n]),f=d((function(e,r,o){return Sr({lockAPI:l,registry:i,contextId:t,store:n,draggableId:e,forceSensorStop:r,sourceEvent:o&&o.sourceEvent?o.sourceEvent:null})}),[t,l,i,n]),p=d((function(e){return function(e,t){var n=Ir(e,t);return n?n.getAttribute(Rn.draggableId):null}(t,e)}),[t]),g=d((function(e){var t=i.draggable.findById(e);return t?t.options:null}),[i.draggable]),v=d((function(){l.isClaimed()&&(l.tryAbandon(),"IDLE"!==n.getState().phase&&n.dispatch({type:"FLUSH",payload:null}))}),[l,n]),m=d(l.isClaimed,[l]),y=c((function(){return{canGetLock:h,tryGetLock:f,findClosestDraggableId:p,findOptionsForDraggable:g,tryReleaseLock:v,isLockClaimed:m}}),[h,f,p,g,v,m]);rr();for(var b=0;b<s.length;b++)s[b](y)}function Or(e){return e.current||L(!1),e.current}function kr(e){var t=e.contextId,n=e.setCallbacks,o=e.sensors,u=e.nonce,h=e.dragHandleUsageInstructions,f=Object(r.useRef)(null);or();var p=ar(e),g=d((function(){return function(e){return{onBeforeCapture:e.onBeforeCapture,onBeforeDragStart:e.onBeforeDragStart,onDragStart:e.onDragStart,onDragEnd:e.onDragEnd,onDragUpdate:e.onDragUpdate}}(p.current)}),[p]),v=function(e){var t=c((function(){return function(e){return"rbd-announcement-"+e}(e)}),[e]),n=Object(r.useRef)(null);return Object(r.useEffect)((function(){var e=document.createElement("div");return n.current=e,e.id=t,e.setAttribute("aria-live","assertive"),e.setAttribute("aria-atomic","true"),Object(a.a)(e.style,Jn),Kn().appendChild(e),function(){setTimeout((function(){var t=Kn();t.contains(e)&&t.removeChild(e),e===n.current&&(n.current=null)}))}}),[t]),d((function(e){var t=n.current;t&&(t.textContent=e)}),[])}(t),m=function(e){var t=e.contextId,n=e.text,i=tr("hidden-text",{separator:"-"}),o=c((function(){return"rbd-hidden-text-"+(e={contextId:t,uniqueId:i}).contextId+"-"+e.uniqueId;var e}),[i,t]);return Object(r.useEffect)((function(){var e=document.createElement("div");return e.id=o,e.textContent=n,e.style.display="none",Kn().appendChild(e),function(){var t=Kn();t.contains(e)&&t.removeChild(e)}}),[o,n]),o}({contextId:t,text:h}),y=Wn(t,u),b=d((function(e){Or(f).dispatch(e)}),[]),C=c((function(){return Object(s.bindActionCreators)({publishWhileDragging:Ct,updateDroppableScroll:_t,updateDroppableIsEnabled:It,updateDroppableIsCombineEnabled:xt,collectionStarting:At},b)}),[b]),A=function(){var e=c(Zn,[]);return Object(r.useEffect)((function(){return function(){requestAnimationFrame(e.clean)}}),[e]),e}(),_=c((function(){return hn(A,C)}),[A,C]),I=c((function(){return jn(Object(a.a)({scrollWindow:pn,scrollDroppable:_.scrollDroppable},Object(s.bindActionCreators)({move:wt},b)))}),[_.scrollDroppable,b]),x=function(e){var t=Object(r.useRef)({}),n=Object(r.useRef)(null),i=Object(r.useRef)(null),o=Object(r.useRef)(!1),a=d((function(e,n){var r={id:e,focus:n};return t.current[e]=r,function(){var n=t.current;n[e]!==r&&delete n[e]}}),[]),s=d((function(t){var n=qn(e,t);n&&n!==document.activeElement&&n.focus()}),[e]),l=d((function(e,t){n.current===e&&(n.current=t)}),[]),u=d((function(){i.current||o.current&&(i.current=requestAnimationFrame((function(){i.current=null;var e=n.current;e&&s(e)})))}),[s]),h=d((function(e){n.current=null;var t=document.activeElement;t&&t.getAttribute(Rn.draggableId)===e&&(n.current=e)}),[]);return Un((function(){return o.current=!0,function(){o.current=!1;var e=i.current;e&&cancelAnimationFrame(e)}}),[]),c((function(){return{register:a,tryRecordFocus:h,tryRestoreFocusRecorded:u,tryShiftRecord:l}}),[a,h,u,l])}(t),w=c((function(){return rn({announce:v,autoScroller:I,dimensionMarshal:_,focusMarshal:x,getResponders:g,styleMarshal:y})}),[v,I,_,x,g,y]);f.current=w;var M=d((function(){var e=Or(f);"IDLE"!==e.getState().phase&&e.dispatch({type:"FLUSH",payload:null})}),[]),S=d((function(){var e=Or(f).getState();return e.isDragging||"DROP_ANIMATING"===e.phase}),[]);n(c((function(){return{isDragging:S,tryAbort:M}}),[S,M]));var T=d((function(e){return fn(Or(f).getState(),e)}),[]),E=d((function(){return $e(Or(f).getState())}),[]),O=c((function(){return{marshal:_,focus:x,contextId:t,canLift:T,isMovementAllowed:E,dragHandleUsageInstructionsId:m,registry:A}}),[t,_,m,x,T,E,A]);return Er({contextId:t,store:w,registry:A,customSensors:o,enableDefaultSensors:!1!==e.enableDefaultSensors}),Object(r.useEffect)((function(){return M}),[M]),i.a.createElement(nr.Provider,{value:O},i.a.createElement(l.Provider,{context:Qn,store:w},e.children))}var Lr=0;function Dr(){Lr=0,$n=0}function Pr(e){var t=c((function(){return""+Lr++}),[]),n=e.dragHandleUsageInstructions||z;return i.a.createElement(D,null,(function(r){return i.a.createElement(kr,{nonce:e.nonce,contextId:t,setCallbacks:r,dragHandleUsageInstructions:n,enableDefaultSensors:e.enableDefaultSensors,sensors:e.sensors,onBeforeCapture:e.onBeforeCapture,onBeforeDragStart:e.onBeforeDragStart,onDragStart:e.onDragStart,onDragUpdate:e.onDragUpdate,onDragEnd:e.onDragEnd},e.children)}))}var Nr=function(e){return function(t){return e===t}},jr=Nr("scroll"),Rr=Nr("auto"),zr=(Nr("visible"),function(e,t){return t(e.overflowX)||t(e.overflowY)}),Fr=function(e){var t=window.getComputedStyle(e),n={overflowX:t.overflowX,overflowY:t.overflowY};return zr(n,jr)||zr(n,Rr)},Br=function e(t){return null==t||t===document.body||t===document.documentElement?null:Fr(t)?t:e(t.parentElement)},Hr=function(e){return{x:e.scrollLeft,y:e.scrollTop}},Ur=function(e){return{closestScrollable:Br(e),isFixedOnPage:function e(t){return!!t&&("fixed"===window.getComputedStyle(t).position||e(t.parentElement))}(e)}},Vr=function(e){var t=e.ref,n=e.descriptor,r=e.env,i=e.windowScroll,o=e.direction,a=e.isDropDisabled,s=e.isCombineEnabled,l=e.shouldClipSubject,u=r.closestScrollable,c=function(e,t){var n=_(e);if(!t)return n;if(e!==t)return n;var r=n.paddingBox.top-t.scrollTop,i=n.paddingBox.left-t.scrollLeft,o=r+t.scrollHeight,a=i+t.scrollWidth,s=p({top:r,right:a,bottom:o,left:i},n.border);return m({borderBox:s,margin:n.margin,border:n.border,padding:n.padding})}(t,u),d=C(c,i),h=function(){if(!u)return null;var e=_(u),t={scrollHeight:u.scrollHeight,scrollWidth:u.scrollWidth};return{client:e,page:C(e,i),scroll:Hr(u),scrollSize:t,shouldClipSubject:l}}();return function(e){var t=e.descriptor,n=e.isEnabled,r=e.isCombineEnabled,i=e.isFixedOnPage,o=e.direction,a=e.client,s=e.page,l=e.closest,u=function(){if(!l)return null;var e=l.scrollSize,t=l.client,n=on({scrollHeight:e.scrollHeight,scrollWidth:e.scrollWidth,height:t.paddingBox.height,width:t.paddingBox.width});return{pageMarginBox:l.page.marginBox,frameClient:t,scrollSize:e,shouldClipSubject:l.shouldClipSubject,scroll:{initial:l.scroll,current:l.scroll,max:n,diff:{value:U,displacement:U}}}}(),c="vertical"===o?Ae:_e;return{descriptor:t,isCombineEnabled:r,isFixedOnPage:i,axis:c,isEnabled:n,client:a,page:s,frame:u,subject:ee({page:s,withPlaceholder:null,axis:c,frame:u})}}({descriptor:n,isEnabled:!a,isCombineEnabled:s,isFixedOnPage:r.isFixedOnPage,direction:o,client:c,page:d,closest:h})},Yr={passive:!1},Wr={passive:!0},Gr=function(e){return e.shouldPublishImmediately?Yr:Wr};function Xr(e){var t=Object(r.useContext)(e);return t||L(!1),t}var qr=function(e){return e&&e.env.closestScrollable||null};function Zr(){}var Qr={width:0,height:0,margin:{top:0,right:0,bottom:0,left:0}},Kr=function(e){var t=e.isAnimatingOpenOnMount,n=e.placeholder,r=e.animate,i=function(e){var t=e.isAnimatingOpenOnMount,n=e.placeholder,r=e.animate;return t||"close"===r?Qr:{height:n.client.borderBox.height,width:n.client.borderBox.width,margin:n.client.margin}}({isAnimatingOpenOnMount:t,placeholder:n,animate:r});return{display:n.display,boxSizing:"border-box",width:i.width,height:i.height,marginTop:i.margin.top,marginRight:i.margin.right,marginBottom:i.margin.bottom,marginLeft:i.margin.left,flexShrink:"0",flexGrow:"0",pointerEvents:"none",transition:"none"!==r?Rt.placeholder:null}};var Jr=i.a.memo((function(e){var t=Object(r.useRef)(null),n=d((function(){t.current&&(clearTimeout(t.current),t.current=null)}),[]),o=e.animate,a=e.onTransitionEnd,s=e.onClose,l=e.contextId,u=Object(r.useState)("open"===e.animate),c=u[0],h=u[1];Object(r.useEffect)((function(){return c?"open"!==o?(n(),h(!1),Zr):t.current?Zr:(t.current=setTimeout((function(){t.current=null,h(!1)})),n):Zr}),[o,c,n]);var f=d((function(e){"height"===e.propertyName&&(a(),"close"===o&&s())}),[o,s,a]),p=Kr({isAnimatingOpenOnMount:c,animate:e.animate,placeholder:e.placeholder});return i.a.createElement(e.placeholder.tagName,{style:p,"data-rbd-placeholder-context-id":l,onTransitionEnd:f,ref:e.innerRef})})),$r=i.a.createContext(null);var ei=function(e){function t(){for(var t,n=arguments.length,r=new Array(n),i=0;i<n;i++)r[i]=arguments[i];return(t=e.call.apply(e,[this].concat(r))||this).state={isVisible:Boolean(t.props.on),data:t.props.on,animate:t.props.shouldAnimate&&t.props.on?"open":"none"},t.onClose=function(){"close"===t.state.animate&&t.setState({isVisible:!1})},t}return Object(o.a)(t,e),t.getDerivedStateFromProps=function(e,t){return e.shouldAnimate?e.on?{isVisible:!0,data:e.on,animate:"open"}:t.isVisible?{isVisible:!0,data:t.data,animate:"close"}:{isVisible:!1,animate:"close",data:null}:{isVisible:Boolean(e.on),data:e.on,animate:"none"}},t.prototype.render=function(){if(!this.state.isVisible)return null;var e={onClose:this.onClose,data:this.state.data,animate:this.state.animate};return this.props.children(e)},t}(i.a.PureComponent),ti=5e3,ni=4500,ri=function(e,t){return t?Rt.drop(t.duration):e?Rt.snap:Rt.fluid},ii=function(e,t){return e?t?Pt.drop:Pt.combining:null};function oi(e){return"DRAGGING"===e.type?(r=(n=e).dimension.client,i=n.offset,o=n.combineWith,a=n.dropping,s=Boolean(o),l=function(e){return null!=e.forceShouldAnimate?e.forceShouldAnimate:"SNAP"===e.mode}(n),u=Boolean(a),c=u?Bt(i,s):Ft(i),{position:"fixed",top:r.marginBox.top,left:r.marginBox.left,boxSizing:"border-box",width:r.borderBox.width,height:r.borderBox.height,transition:ri(l,a),transform:c,opacity:ii(s,u),zIndex:u?ni:ti,pointerEvents:"none"}):{transform:Ft((t=e).offset),transition:t.shouldAnimateDisplacement?null:"none"};var t,n,r,i,o,a,s,l,u,c}function ai(e){var t=tr("draggable"),n=e.descriptor,i=e.registry,o=e.getDraggableRef,a=e.canDragInteractiveElements,s=e.shouldRespectForcePress,l=e.isEnabled,u=c((function(){return{canDragInteractiveElements:a,shouldRespectForcePress:s,isEnabled:l}}),[a,l,s]),h=d((function(e){var t=o();return t||L(!1),function(e,t,n){void 0===n&&(n=U);var r=window.getComputedStyle(t),i=t.getBoundingClientRect(),o=A(i,r),a=C(o,n);return{descriptor:e,placeholder:{client:o,tagName:t.tagName.toLowerCase(),display:r.display},displaceBy:{x:o.marginBox.width,y:o.marginBox.height},client:o,page:a}}(n,t,e)}),[n,o]),f=c((function(){return{uniqueId:t,descriptor:n,options:u,getDimension:h}}),[n,h,u,t]),p=Object(r.useRef)(f),g=Object(r.useRef)(!0);Un((function(){return i.draggable.register(p.current),function(){return i.draggable.unregister(p.current)}}),[i.draggable]),Un((function(){if(g.current)g.current=!1;else{var e=p.current;p.current=f,i.draggable.update(f,e)}}),[f,i.draggable])}function si(e,t,n){ir()}function li(e){e.preventDefault()}var ui=function(e,t){return e===t},ci=function(e){var t=e.combine,n=e.destination;return n?n.droppableId:t?t.droppableId:null};function di(e){return{isDragging:!1,isDropAnimating:!1,isClone:!1,dropAnimation:null,mode:null,draggingOver:null,combineTargetFor:e,combineWith:null}}var hi={mapped:{type:"SECONDARY",offset:U,combineTargetFor:null,shouldAnimateDisplacement:!0,snapshot:di(null)}};var fi={dropAnimationFinished:Lt},pi=Object(l.connect)((function(){var e,t,n,r=(e=x((function(e,t){return{x:e,y:t}})),t=x((function(e,t,n,r,i){return{isDragging:!0,isClone:t,isDropAnimating:Boolean(i),dropAnimation:i,mode:e,draggingOver:n,combineWith:r,combineTargetFor:null}})),n=x((function(e,n,r,i,o,a,s){return{mapped:{type:"DRAGGING",dropping:null,draggingOver:o,combineWith:a,mode:n,offset:e,dimension:r,forceShouldAnimate:s,snapshot:t(n,i,o,a,null)}}})),function(r,i){if(r.isDragging){if(r.critical.draggable.id!==i.draggableId)return null;var o=r.current.client.offset,a=r.dimensions.draggables[i.draggableId],s=Ke(r.impact),l=(c=r.impact).at&&"COMBINE"===c.at.type?c.at.combine.draggableId:null,u=r.forceShouldAnimate;return n(e(o.x,o.y),r.movementMode,a,i.isClone,s,l,u)}var c;if("DROP_ANIMATING"===r.phase){var d=r.completed;if(d.result.draggableId!==i.draggableId)return null;var h=i.isClone,f=r.dimensions.draggables[i.draggableId],p=d.result,g=p.mode,v=ci(p),m=function(e){return e.combine?e.combine.draggableId:null}(p),y={duration:r.dropDuration,curve:Dt,moveTo:r.newHomeClientOffset,opacity:m?Pt.drop:null,scale:m?Nt.drop:null};return{mapped:{type:"DRAGGING",offset:r.newHomeClientOffset,dimension:f,dropping:y,draggingOver:v,combineWith:m,mode:g,forceShouldAnimate:null,snapshot:t(g,h,v,m,y)}}}return null}),i=function(){var e=x((function(e,t){return{x:e,y:t}})),t=x(di),n=x((function(e,n,r){return void 0===n&&(n=null),{mapped:{type:"SECONDARY",offset:e,combineTargetFor:n,shouldAnimateDisplacement:r,snapshot:t(n)}}})),r=function(e){return e?n(U,e,!0):null},i=function(t,i,o,a){var s=o.displaced.visible[t],l=Boolean(a.inVirtualList&&a.effected[t]),u=he(o),c=u&&u.draggableId===t?i:null;if(!s){if(!l)return r(c);if(o.displaced.invisible[t])return null;var d=G(a.displacedBy.point),h=e(d.x,d.y);return n(h,c,!0)}if(l)return r(c);var f=o.displacedBy.point,p=e(f.x,f.y);return n(p,c,s.shouldAnimate)};return function(e,t){if(e.isDragging)return e.critical.draggable.id===t.draggableId?null:i(t.draggableId,e.critical.draggable.id,e.impact,e.afterCritical);if("DROP_ANIMATING"===e.phase){var n=e.completed;return n.result.draggableId===t.draggableId?null:i(t.draggableId,n.result.draggableId,n.impact,n.afterCritical)}return null}}();return function(e,t){return r(e,t)||i(e,t)||hi}}),fi,null,{context:Qn,pure:!0,areStatePropsEqual:ui})((function(e){var t=Object(r.useRef)(null),n=d((function(e){t.current=e}),[]),i=d((function(){return t.current}),[]),o=Xr(nr),a=o.contextId,s=o.dragHandleUsageInstructionsId,l=o.registry,u=Xr($r),h=u.type,f=u.droppableId,p=c((function(){return{id:e.draggableId,index:e.index,type:h,droppableId:f}}),[e.draggableId,e.index,h,f]),g=e.children,v=e.draggableId,m=e.isEnabled,y=e.shouldRespectForcePress,b=e.canDragInteractiveElements,C=e.isClone,A=e.mapped,_=e.dropAnimationFinished;si(),rr(),C||ai(c((function(){return{descriptor:p,registry:l,getDraggableRef:i,canDragInteractiveElements:b,shouldRespectForcePress:y,isEnabled:m}}),[p,l,i,b,y,m]));var I=c((function(){return m?{tabIndex:0,role:"button","aria-describedby":s,"data-rbd-drag-handle-draggable-id":v,"data-rbd-drag-handle-context-id":a,draggable:!1,onDragStart:li}:null}),[a,s,v,m]),x=d((function(e){"DRAGGING"===A.type&&A.dropping&&"transform"===e.propertyName&&_()}),[_,A]),w=c((function(){var e=oi(A),t="DRAGGING"===A.type&&A.dropping?x:null;return{innerRef:n,draggableProps:{"data-rbd-draggable-context-id":a,"data-rbd-draggable-id":v,style:e,onTransitionEnd:t},dragHandleProps:I}}),[a,I,v,A,x,n]),M=c((function(){return{draggableId:p.id,type:p.type,source:{index:p.index,droppableId:p.droppableId}}}),[p.droppableId,p.id,p.index,p.type]);return g(w,A.snapshot,M)}));function gi(e){return Xr($r).isUsingCloneFor!==e.draggableId||e.isClone?i.a.createElement(pi,e):null}function vi(e){var t="boolean"!=typeof e.isDragDisabled||!e.isDragDisabled,n=Boolean(e.disableInteractiveElementBlocking),r=Boolean(e.shouldRespectForcePress);return i.a.createElement(gi,Object(a.a)({},e,{isClone:!1,isEnabled:t,canDragInteractiveElements:n,shouldRespectForcePress:r}))}var mi=function(e,t){return e===t.droppable.type},yi=function(e,t){return t.draggables[e.draggable.id]},bi={updateViewportMaxScroll:function(e){return{type:"UPDATE_VIEWPORT_MAX_SCROLL",payload:e}}};var Ci={mode:"standard",type:"DEFAULT",direction:"vertical",isDropDisabled:!1,isCombineEnabled:!1,ignoreContainerClipping:!1,renderClone:null,getContainerForClone:function(){return document.body||L(!1),document.body}},Ai=Object(l.connect)((function(){var e={placeholder:null,shouldAnimatePlaceholder:!0,snapshot:{isDraggingOver:!1,draggingOverWith:null,draggingFromThisWith:null,isUsingPlaceholder:!1},useClone:null},t=Object(a.a)({},e,{shouldAnimatePlaceholder:!1}),n=x((function(e){return{draggableId:e.id,type:e.type,source:{index:e.index,droppableId:e.droppableId}}})),r=x((function(r,i,o,a,s,l){var u=s.descriptor.id;if(s.descriptor.droppableId===r){var c=l?{render:l,dragging:n(s.descriptor)}:null,d={isDraggingOver:o,draggingOverWith:o?u:null,draggingFromThisWith:u,isUsingPlaceholder:!0};return{placeholder:s.placeholder,shouldAnimatePlaceholder:!1,snapshot:d,useClone:c}}if(!i)return t;if(!a)return e;var h={isDraggingOver:o,draggingOverWith:u,draggingFromThisWith:null,isUsingPlaceholder:!0};return{placeholder:s.placeholder,shouldAnimatePlaceholder:!0,snapshot:h,useClone:null}}));return function(n,i){var o=i.droppableId,a=i.type,s=!i.isDropDisabled,l=i.renderClone;if(n.isDragging){var u=n.critical;if(!mi(a,u))return t;var c=yi(u,n.dimensions),d=Ke(n.impact)===o;return r(o,s,d,d,c,l)}if("DROP_ANIMATING"===n.phase){var h=n.completed;if(!mi(a,h.critical))return t;var f=yi(h.critical,n.dimensions);return r(o,s,ci(h.result)===o,Ke(h.impact)===o,f,l)}if("IDLE"===n.phase&&n.completed&&!n.shouldFlush){var p=n.completed;if(!mi(a,p.critical))return t;var g=Ke(p.impact)===o,v=Boolean(p.impact.at&&"COMBINE"===p.impact.at.type),m=p.critical.droppable.id===o;return g?v?e:t:m?e:t}return t}}),bi,null,{context:Qn,pure:!0,areStatePropsEqual:ui})((function(e){var t=Object(r.useContext)(nr);t||L(!1);var n=t.contextId,o=t.isMovementAllowed,a=Object(r.useRef)(null),s=Object(r.useRef)(null),l=e.children,u=e.droppableId,h=e.type,f=e.mode,p=e.direction,g=e.ignoreContainerClipping,v=e.isDropDisabled,m=e.isCombineEnabled,y=e.snapshot,b=e.useClone,C=e.updateViewportMaxScroll,A=e.getContainerForClone,_=d((function(){return a.current}),[]),I=d((function(e){a.current=e}),[]),M=(d((function(){return s.current}),[]),d((function(e){s.current=e}),[]));ir();var T=d((function(){o()&&C({maxScroll:sn()})}),[o,C]);!function(e){var t=Object(r.useRef)(null),n=Xr(nr),i=tr("droppable"),o=n.registry,a=n.marshal,s=ar(e),l=c((function(){return{id:e.droppableId,type:e.type,mode:e.mode}}),[e.droppableId,e.mode,e.type]),u=Object(r.useRef)(l),h=c((function(){return x((function(e,n){t.current||L(!1);var r={x:e,y:n};a.updateDroppableScroll(l.id,r)}))}),[l.id,a]),f=d((function(){var e=t.current;return e&&e.env.closestScrollable?Hr(e.env.closestScrollable):U}),[]),p=d((function(){var e=f();h(e.x,e.y)}),[f,h]),g=c((function(){return w(p)}),[p]),v=d((function(){var e=t.current,n=qr(e);e&&n||L(!1),e.scrollOptions.shouldPublishImmediately?p():g()}),[g,p]),m=d((function(e,r){t.current&&L(!1);var i=s.current,o=i.getDroppableRef();o||L(!1);var a=Ur(o),u={ref:o,descriptor:l,env:a,scrollOptions:r};t.current=u;var c=Vr({ref:o,descriptor:l,env:a,windowScroll:e,direction:i.direction,isDropDisabled:i.isDropDisabled,isCombineEnabled:i.isCombineEnabled,shouldClipSubject:!i.ignoreContainerClipping}),d=a.closestScrollable;return d&&(d.setAttribute(Bn.contextId,n.contextId),d.addEventListener("scroll",v,Gr(u.scrollOptions))),c}),[n.contextId,l,v,s]),y=d((function(){var e=t.current,n=qr(e);return e&&n||L(!1),Hr(n)}),[]),b=d((function(){var e=t.current;e||L(!1);var n=qr(e);t.current=null,n&&(g.cancel(),n.removeAttribute(Bn.contextId),n.removeEventListener("scroll",v,Gr(e.scrollOptions)))}),[v,g]),C=d((function(e){var n=t.current;n||L(!1);var r=qr(n);r||L(!1),r.scrollTop+=e.y,r.scrollLeft+=e.x}),[]),A=c((function(){return{getDimensionAndWatchScroll:m,getScrollWhileDragging:y,dragStopped:b,scroll:C}}),[b,m,y,C]),_=c((function(){return{uniqueId:i,descriptor:l,callbacks:A}}),[A,l,i]);Un((function(){return u.current=_.descriptor,o.droppable.register(_),function(){t.current&&b(),o.droppable.unregister(_)}}),[A,l,b,_,a,o.droppable]),Un((function(){t.current&&a.updateDroppableIsEnabled(u.current.id,!e.isDropDisabled)}),[e.isDropDisabled,a]),Un((function(){t.current&&a.updateDroppableIsCombineEnabled(u.current.id,e.isCombineEnabled)}),[e.isCombineEnabled,a])}({droppableId:u,type:h,mode:f,direction:p,isDropDisabled:v,isCombineEnabled:m,ignoreContainerClipping:g,getDroppableRef:_});var E=i.a.createElement(ei,{on:e.placeholder,shouldAnimate:e.shouldAnimatePlaceholder},(function(e){var t=e.onClose,r=e.data,o=e.animate;return i.a.createElement(Jr,{placeholder:r,onClose:t,innerRef:M,animate:o,contextId:n,onTransitionEnd:T})})),O=c((function(){return{innerRef:I,placeholder:E,droppableProps:{"data-rbd-droppable-id":u,"data-rbd-droppable-context-id":n}}}),[n,u,E,I]),k=b?b.dragging.draggableId:null,D=c((function(){return{droppableId:u,type:h,isUsingCloneFor:k}}),[u,k,h]);return i.a.createElement($r.Provider,{value:D},l(O,y),function(){if(!b)return null;var e=b.dragging,t=b.render,n=i.a.createElement(gi,{draggableId:e.draggableId,index:e.source.index,isClone:!0,isEnabled:!0,shouldRespectForcePress:!1,canDragInteractiveElements:!0},(function(n,r){return t(n,r,e)}));return S.a.createPortal(n,A())}())}));Ai.defaultProps=Ci},function(e,t,n){"use strict";n.r(t);var r,i=n(463),o=n(17),a=n(16);class s{constructor(e){this.trusted=!1,this._changed=new a.Signal(this),this._raw={};const t=r.getData(e.value);this._data=new i.a({values:t}),this._rawData=t;const n=e.value;for(const e in n)switch(e){case"data":break;default:this._raw[e]=r.extract(n,e)}}get changed(){return this._changed}dispose(){this._data.dispose(),a.Signal.clearData(this)}get data(){return this._rawData}get metadata(){return{}}setData(e){e.data&&(this._updateObservable(this._data,e.data),this._rawData=e.data),this._changed.emit(void 0)}toJSON(){const e={};for(const t in this._raw)e[t]=r.extract(this._raw,t);return e}_updateObservable(e,t){const n=e.keys(),r=Object.keys(t);for(const t of n)-1===r.indexOf(t)&&e.delete(t);for(const n of r){const r=e.get(n),i=t[n];r!==i&&e.set(n,i)}}}(s||(s={})).getData=function(e){return r.getData(e)},function(e){function t(e){return function(e){const t=Object.create(null);for(const r in e)t[r]=n(e,r);return t}(e)}function n(e,t){const n=e[t];return void 0===n||o.JSONExt.isPrimitive(n)?n:o.JSONExt.deepCopy(n)}e.getData=t,e.getBundleOptions=function(e){return{data:t(e.value)}},e.extract=n}(r||(r={}));var l=n(866),u=n(329),c=n(433),d=n(864),h=n(865),f=n(173);const p=new o.Token("@jupyterlab/rendermime:IRenderMimeRegistry"),g=new o.Token("@jupyterlab/rendermime:ILatexTypesetter");var v=n(171);n.d(t,"AttachmentModel",(function(){return s})),n.d(t,"htmlRendererFactory",(function(){return l.a})),n.d(t,"imageRendererFactory",(function(){return l.b})),n.d(t,"latexRendererFactory",(function(){return l.d})),n.d(t,"markdownRendererFactory",(function(){return l.e})),n.d(t,"svgRendererFactory",(function(){return l.g})),n.d(t,"textRendererFactory",(function(){return l.h})),n.d(t,"javaScriptRendererFactory",(function(){return l.c})),n.d(t,"standardRendererFactories",(function(){return l.f})),n.d(t,"removeMath",(function(){return u.a})),n.d(t,"replaceMath",(function(){return u.b})),n.d(t,"MimeModel",(function(){return c.a})),n.d(t,"OutputModel",(function(){return d.a})),n.d(t,"RenderMimeRegistry",(function(){return h.a})),n.d(t,"renderHTML",(function(){return f.a})),n.d(t,"renderImage",(function(){return f.b})),n.d(t,"renderLatex",(function(){return f.c})),n.d(t,"renderMarkdown",(function(){return f.d})),n.d(t,"renderSVG",(function(){return f.e})),n.d(t,"renderText",(function(){return f.f})),n.d(t,"IRenderMimeRegistry",(function(){return p})),n.d(t,"ILatexTypesetter",(function(){return g})),n.d(t,"RenderedCommon",(function(){return v.a})),n.d(t,"RenderedHTMLCommon",(function(){return v.c})),n.d(t,"RenderedHTML",(function(){return v.b})),n.d(t,"RenderedLatex",(function(){return v.f})),n.d(t,"RenderedImage",(function(){return v.d})),n.d(t,"RenderedMarkdown",(function(){return v.g})),n.d(t,"RenderedSVG",(function(){return v.h})),n.d(t,"RenderedText",(function(){return v.i})),n.d(t,"RenderedJavaScript",(function(){return v.e}))},function(e,t,n){var r=n(208),i=n(364),o=n(158),a=Function.prototype,s=Object.prototype,l=a.toString,u=s.hasOwnProperty,c=l.call(Object);e.exports=function(e){if(!o(e)||"[object Object]"!=r(e))return!1;var t=i(e);if(null===t)return!0;var n=u.call(t,"constructor")&&t.constructor;return"function"==typeof n&&n instanceof n&&l.call(n)==c}},function(e,t,n){"use strict";n.d(t,"a",(function(){return a}));var r=n(339),i=n.n(r);class o{static reg(e){return new RegExp("^"+e+"$","i")}}o.N={integer:"[+-]?[0-9]+",integer_pos:"[+]?[0-9]+",integer_zero_ff:"([0-9]|[1-9][0-9]|1[0-9][0-9]|2[0-4][0-9]|25[0-5])",number:"[+-]?([0-9]*[.])?[0-9]+(e-?[0-9]*)?",number_pos:"[+]?([0-9]*[.])?[0-9]+(e-?[0-9]*)?",number_zero_hundred:"[+]?(([0-9]|[1-9][0-9])([.][0-9]+)?|100)",number_zero_one:"[+]?(1([.][0]+)?|0([.][0-9]+)?)"},o.B={angle:`(${o.N.number}(deg|rad|grad|turn)|0)`,frequency:o.N.number+"(Hz|kHz)",ident:String.raw`-?([_a-z]|[\xA0-\xFF]|\\[0-9a-f]{1,6}(\r\n|[ \t\r\n\f])?|\\[^\r\n\f0-9a-f])([_a-z0-9-]|[\xA0-\xFF]|\\[0-9a-f]{1,6}(\r\n|[ \t\r\n\f])?|\\[^\r\n\f0-9a-f])*`,len_or_perc:`(0|${o.N.number}(px|em|rem|ex|in|cm|mm|pt|pc|%))`,length:`(${o.N.number}(px|em|rem|ex|in|cm|mm|pt|pc)|0)`,length_pos:`(${o.N.number_pos}(px|em|rem|ex|in|cm|mm|pt|pc)|0)`,percentage:o.N.number+"%",percentage_pos:o.N.number_pos+"%",percentage_zero_hundred:o.N.number_zero_hundred+"%",string:String.raw`(\"([^\n\r\f\\"]|\\\n|\r\n|\r|\f|\\[0-9a-f]{1,6}(\r\n|[ \t\r\n\f])?|\\[^\r\n\f0-9a-f])*\")|(\'([^\n\r\f\\']|\\\n|\r\n|\r|\f|\\[0-9a-f]{1,6}(\r\n|[ \t\r\n\f])?|\\[^\r\n\f0-9a-f])*\')`,time:o.N.number+"(s|ms)",url:"url\\(.*?\\)",z_index:"[+-]?[0-9]{1,7}"},o.A={absolute_size:"xx-small|x-small|small|medium|large|x-large|xx-large",attachment:"scroll|fixed|local",bg_origin:"border-box|padding-box|content-box",border_style:"none|hidden|dotted|dashed|solid|double|groove|ridge|inset|outset",box:"border-box|padding-box|content-box",display_inside:"auto|block|table|flex|grid",display_outside:"block-level|inline-level|none|table-row-group|table-header-group|table-footer-group|table-row|table-cell|table-column-group|table-column|table-caption",ending_shape:"circle|ellipse",generic_family:"serif|sans-serif|cursive|fantasy|monospace",generic_voice:"male|female|child",relative_size:"smaller|larger",repeat_style:"repeat-x|repeat-y|((?:repeat|space|round|no-repeat)(?:\\s*(?:repeat|space|round|no-repeat))?)",side_or_corner:"(left|right)?\\s*(top|bottom)?",single_animation_direction:"normal|reverse|alternate|alternate-reverse",single_animation_fill_mode:"none|forwards|backwards|both",single_animation_play_state:"running|paused"},o._COLOR={hex:"\\#(0x)?[0-9a-f]+",name:"aliceblue|antiquewhite|aqua|aquamarine|azure|beige|bisque|black|blanchedalmond|blue|blueviolet|brown|burlywood|cadetblue|chartreuse|chocolate|coral|cornflowerblue|cornsilk|crimson|cyan|darkblue|darkcyan|darkgoldenrod|darkgray|darkgreen|darkkhaki|darkmagenta|darkolivegreen|darkorange|darkorchid|darkred|darksalmon|darkseagreen|darkslateblue|darkslategray|darkturquoise|darkviolet|deeppink|deepskyblue|dimgray|dodgerblue|firebrick|floralwhite|forestgreen|fuchsia|gainsboro|ghostwhite|gold|goldenrod|gray|green|greenyellow|honeydew|hotpink|indianred|indigo|ivory|khaki|lavender|lavenderblush|lawngreen|lemonchiffon|lightblue|lightcoral|lightcyan|lightgoldenrodyellow|lightgreen|lightgrey|lightpink|lightsalmon|lightseagreen|lightskyblue|lightslategray|lightsteelblue|lightyellow|lime|limegreen|linen|magenta|maroon|mediumaquamarine|mediumblue|mediumorchid|mediumpurple|mediumseagreen|mediumslateblue|mediumspringgreen|mediumturquoise|mediumvioletred|midnightblue|mintcream|mistyrose|moccasin|navajowhite|navy|oldlace|olive|olivedrab|orange|orangered|orchid|palegoldenrod|palegreen|paleturquoise|palevioletred|papayawhip|peachpuff|peru|pink|plum|powderblue|purple|red|rosybrown|royalblue|saddlebrown|salmon|sandybrown|seagreen|seashell|sienna|silver|skyblue|slateblue|slategray|snow|springgreen|steelblue|tan|teal|thistle|tomato|turquoise|transparent|violet|wheat|white|whitesmoke|yellow|yellowgreen",rgb:String.raw`rgb\(\s*(\d{1,3})\s*,\s*(\d{1,3})\s*,\s*(\d{1,3})\s*\)`,rgba:String.raw`rgba\(\s*(\d{1,3})\s*,\s*(\d{1,3})\s*,\s*(\d{1,3})\s*,\s*(${o.N.integer_zero_ff}|${o.N.number_zero_one}|${o.B.percentage_zero_hundred})\s*\)`},o._C={alpha:`${o.N.integer_zero_ff}|${o.N.number_zero_one}|${o.B.percentage_zero_hundred}`,alphavalue:o.N.number_zero_one,bg_position:`((${o.B.len_or_perc}|left|center|right|top|bottom)\\s*){1,4}`,bg_size:`(${o.B.length_pos}|${o.B.percentage}|auto){1,2}|cover|contain`,border_width:"thin|medium|thick|"+o.B.length,bottom:o.B.length+"|auto",color:`${o._COLOR.hex}|${o._COLOR.rgb}|${o._COLOR.rgba}|${o._COLOR.name}`,family_name:`${o.B.string}|(${o.B.ident}\\s*)+`,image_decl:o.B.url,left:o.B.length+"|auto",loose_quotable_words:`(${o.B.ident})+`,margin_width:o.B.len_or_perc+"|auto",padding_width:`${o.B.length_pos}|${o.B.percentage_pos}`,page_url:o.B.url,position:`((${o.B.len_or_perc}|left|center|right|top|bottom)\\s*){1,4}`,right:o.B.length+"|auto",shadow:"",size:`closest-side|farthest-side|closest-corner|farthest-corner|${o.B.length}|(${o.B.len_or_perc})\\s+(${o.B.len_or_perc})`,top:o.B.length+"|auto"},o._C1={image_list:`image\\(\\s*(${o.B.url})*\\s*(${o.B.url}|${o._C.color})\\s*\\)`,shadow:`((${o._C.color})\\s+((${o.B.length})\\s*){2,4}(s+inset)?)|((inset\\s+)?((${o.B.length})\\s*){2,4}\\s*(${o._C.color})?)`},o._C2={bg_image:`(${o.B.url}|${o._C1.image_list})|none`,image:`${o.B.url}|${o._C1.image_list}`,shape:`rect\\(\\s*(${o._C.top})\\s*,\\s*(${o._C.right})\\s*,\\s*(${o._C.bottom})\\s*,\\s*(${o._C.left})\\s*\\)`},o.C=Object.assign(Object.assign(Object.assign({},o._C),o._C1),o._C2),o.AP={border_collapse:"collapse|separate",box:"normal|none|contents",box_sizing:"content-box|padding-box|border-box",caption_side:"top|bottom",clear:"none|left|right|both",direction:"ltr|rtl",empty_cells:"show|hide",float:"left|right|none",font_stretch:"normal|wider|narrower|ultra-condensed|extra-condensed|condensed|semi-condensed|semi-expanded|expanded|extra-expanded|ultra-expanded",font_style:"normal|italic|oblique",font_variant:"normal|small-caps",font_weight:"normal|bold|bolder|lighter|100|200|300|400|500|600|700|800|900",list_style_position:"inside|outside",list_style_type:"disc|circle|square|decimal|decimal-leading-zero|lower-roman|upper-roman|lower-greek|lower-latin|upper-latin|armenian|georgian|lower-alpha|upper-alpha|none",overflow:"visible|hidden|scroll|auto",overflow_wrap:"normal|break-word",overflow_x:"visible|hidden|scroll|auto|no-display|no-content",page_break_after:"auto|always|avoid|left|right",page_break_before:"auto|always|avoid|left|right",page_break_inside:"avoid|auto",position:"static|relative|absolute",resize:"none|both|horizontal|vertical",speak:"normal|none|spell-out",speak_header:"once|always",speak_numeral:"digits|continuous",speak_punctuation:"code|none",table_layout:"auto|fixed",text_align:"left|right|center|justify",text_decoration:"none|((underline|overline|line-through|blink)\\s*)+",text_transform:"capitalize|uppercase|lowercase|none",text_wrap:"normal|unrestricted|none|suppress",unicode_bidi:"normal|embed|bidi-override",visibility:"visible|hidden|collapse",white_space:"normal|pre|nowrap|pre-wrap|pre-line",word_break:"normal|keep-all|break-all"},o._CP={background_attachment:`${o.A.attachment}(,\\s*${o.A.attachment})*`,background_color:o.C.color,background_origin:`${o.A.box}(,\\s*${o.A.box})*`,background_repeat:`${o.A.repeat_style}(,\\s*${o.A.repeat_style})*`,border:`((${o.C.border_width}|${o.A.border_style}|${o.C.color})\\s*){1,3}`,border_radius:`((${o.B.len_or_perc})\\s*){1,4}(\\/\\s*((${o.B.len_or_perc})\\s*){1,4})?`,border_spacing:`${o.B.length}\\s*(${o.B.length})?`,border_top_color:o.C.color,border_top_style:o.A.border_style,border_width:`((${o.C.border_width})\\s*){1,4}`,color:o.C.color,cursor:`(${o.B.url}(\\s*,\\s*)?)*(auto|crosshair|default|pointer|move|e-resize|ne-resize|nw-resize|n-resize|se-resize|sw-resize|s-resize|w-resize|text|wait|help|progress|all-scroll|col-resize|hand|no-drop|not-allowed|row-resize|vertical-text)`,display:`inline|block|list-item|run-in|inline-list-item|inline-block|table|inline-table|table-cell|table-caption|flex|inline-flex|grid|inline-grid|${o.A.display_inside}|${o.A.display_outside}|inherit|inline-box|inline-stack`,display_outside:o.A.display_outside,elevation:o.B.angle+"|below|level|above|higher|lower",font_family:`(${o.C.family_name}|${o.A.generic_family})(,\\s*(${o.C.family_name}|${o.A.generic_family}))*`,height:`${o.B.length}|${o.B.percentage}|auto`,letter_spacing:"normal|"+o.B.length,list_style_image:o.C.image+"|none",margin_right:o.C.margin_width,max_height:`${o.B.length_pos}|${o.B.percentage_pos}|none|auto`,min_height:`${o.B.length_pos}|${o.B.percentage_pos}|auto`,opacity:o.C.alphavalue,outline_color:o.C.color+"|invert",outline_width:o.C.border_width,padding:`((${o.C.padding_width})\\s*){1,4}`,padding_top:o.C.padding_width,pitch_range:o.N.number,right:`${o.B.length}|${o.B.percentage}|auto`,stress:o.N.number,text_indent:`${o.B.length}|${o.B.percentage}`,text_shadow:`none|${o.C.shadow}(,\\s*(${o.C.shadow}))*`,volume:`${o.N.number_pos}|${o.B.percentage_pos}|silent|x-soft|soft|medium|loud|x-loud`,word_wrap:o.AP.overflow_wrap,zoom:`normal|${o.N.number_pos}|${o.B.percentage_pos}`,backface_visibility:o.AP.visibility,background_clip:`${o.A.box}(,\\s*(${o.A.box}))*`,background_position:`${o.C.bg_position}(,\\s*(${o.C.bg_position}))*`,border_bottom_color:o.C.color,border_bottom_style:o.A.border_style,border_color:`((${o.C.color})\\s*){1,4}`,border_left_color:o.C.color,border_right_color:o.C.color,border_style:`((${o.A.border_style})\\s*){1,4}`,border_top_left_radius:`(${o.B.length}|${o.B.percentage})(\\s*(${o.B.length}|${o.B.percentage}))?`,border_top_width:o.C.border_width,box_shadow:`none|${o.C.shadow}(,\\s*(${o.C.shadow}))*`,clip:o.C.shape+"|auto",display_inside:o.A.display_inside,font_size:`${o.A.absolute_size}|${o.A.relative_size}|${o.B.length_pos}|${o.B.percentage_pos}`,line_height:`normal|${o.N.number_pos}|${o.B.length_pos}|${o.B.percentage_pos}`,margin_left:o.C.margin_width,max_width:`${o.B.length_pos}|${o.B.percentage_pos}|none|auto`,outline_style:o.A.border_style,padding_bottom:o.C.padding_width,padding_right:o.C.padding_width,perspective:"none|"+o.B.length,richness:o.N.number,text_overflow:`((clip|ellipsis|${o.B.string})\\s*){1,2}`,top:`${o.B.length}|${o.B.percentage}|auto`,width:`${o.B.length_pos}|${o.B.percentage_pos}|auto`,z_index:"auto|"+o.B.z_index,background:`(((${o.C.bg_position}\\s*(\\/\\s*${o.C.bg_size})?)|(${o.A.repeat_style})|(${o.A.attachment})|(${o.A.bg_origin})|(${o.C.bg_image})|(${o.C.color}))\\s*)+`,background_size:`${o.C.bg_size}(,\\s*${o.C.bg_size})*`,border_bottom_left_radius:`(${o.B.length}|${o.B.percentage})(\\s*(${o.B.length}|${o.B.percentage}))?`,border_bottom_width:o.C.border_width,border_left_style:o.A.border_style,border_right_style:o.A.border_style,border_top:`((${o.C.border_width}|${o.A.border_style}|${o.C.color})\\s*){1,3}`,bottom:o.B.len_or_perc+"|auto",list_style:`((${o.AP.list_style_type}|${o.AP.list_style_position}|${o.C.image}|none})\\s*){1,3}`,margin_top:o.C.margin_width,outline:`((${o.C.color}|invert|${o.A.border_style}|${o.C.border_width})\\s*){1,3}`,overflow_y:o.AP.overflow_x,pitch:o.B.frequency+"|x-low|low|medium|high|x-high",vertical_align:"baseline|sub|super|top|text-top|middle|bottom|text-bottom|"+o.B.len_or_perc,word_spacing:"normal|"+o.B.length,background_image:`${o.C.bg_image}(,\\s*${o.C.bg_image})*`,border_bottom_right_radius:`(${o.B.length}|${o.B.percentage})(\\s*(${o.B.length}|${o.B.percentage}))?`,border_left_width:o.C.border_width,border_right_width:o.C.border_width,left:o.B.len_or_perc+"|auto",margin_bottom:o.C.margin_width,pause_after:`${o.B.time}|${o.B.percentage}`,speech_rate:o.N.number+"|x-slow|slow|medium|fast|x-fast|faster|slower",transition_duration:`${o.B.time}(,\\s*${o.B.time})*`,border_bottom:`((${o.C.border_width}|${o.A.border_style}|${o.C.color})\\s*){1,3}`,border_right:`((${o.C.border_width}|${o.A.border_style}|${o.C.color})\\s*){1,3}`,margin:`((${o.C.margin_width})\\s*){1,4}`,padding_left:o.C.padding_width,border_left:`((${o.C.border_width}|${o.A.border_style}|${o.C.color})\\s*){1,3}`,quotes:`(${o.B.string}\\s*${o.B.string})+|none`,border_top_right_radius:`(${o.B.length}|${o.B.percentage})(\\s*(${o.B.length}|${o.B.percentage}))?`,min_width:`${o.B.length_pos}|${o.B.percentage_pos}|auto`},o._CP1={font:`(((((${o.AP.font_style}|${o.AP.font_variant}|${o.AP.font_weight})\\s*){1,3})?\\s*(${o._CP.font_size})\\s*(\\/\\s*(${o._CP.line_height}))?\\s+(${o._CP.font_family}))|caption|icon|menu|message-box|small-caption|status-bar)`},o.CP=Object.assign(Object.assign({},o._CP),o._CP1),o.BORDER_COLLAPSE=o.reg(o.AP.border_collapse),o.BOX=o.reg(o.AP.box),o.BOX_SIZING=o.reg(o.AP.box_sizing),o.CAPTION_SIDE=o.reg(o.AP.caption_side),o.CLEAR=o.reg(o.AP.clear),o.DIRECTION=o.reg(o.AP.direction),o.EMPTY_CELLS=o.reg(o.AP.empty_cells),o.FLOAT=o.reg(o.AP.float),o.FONT_STRETCH=o.reg(o.AP.font_stretch),o.FONT_STYLE=o.reg(o.AP.font_style),o.FONT_VARIANT=o.reg(o.AP.font_variant),o.FONT_WEIGHT=o.reg(o.AP.font_weight),o.LIST_STYLE_POSITION=o.reg(o.AP.list_style_position),o.LIST_STYLE_TYPE=o.reg(o.AP.list_style_type),o.OVERFLOW=o.reg(o.AP.overflow),o.OVERFLOW_WRAP=o.reg(o.AP.overflow_wrap),o.OVERFLOW_X=o.reg(o.AP.overflow_x),o.PAGE_BREAK_AFTER=o.reg(o.AP.page_break_after),o.PAGE_BREAK_BEFORE=o.reg(o.AP.page_break_before),o.PAGE_BREAK_INSIDE=o.reg(o.AP.page_break_inside),o.POSITION=o.reg(o.AP.position),o.RESIZE=o.reg(o.AP.resize),o.SPEAK=o.reg(o.AP.speak),o.SPEAK_HEADER=o.reg(o.AP.speak_header),o.SPEAK_NUMERAL=o.reg(o.AP.speak_numeral),o.SPEAK_PUNCTUATION=o.reg(o.AP.speak_punctuation),o.TABLE_LAYOUT=o.reg(o.AP.table_layout),o.TEXT_ALIGN=o.reg(o.AP.text_align),o.TEXT_DECORATION=o.reg(o.AP.text_decoration),o.TEXT_TRANSFORM=o.reg(o.AP.text_transform),o.TEXT_WRAP=o.reg(o.AP.text_wrap),o.UNICODE_BIDI=o.reg(o.AP.unicode_bidi),o.VISIBILITY=o.reg(o.AP.visibility),o.WHITE_SPACE=o.reg(o.AP.white_space),o.WORD_BREAK=o.reg(o.AP.word_break),o.BACKGROUND_ATTACHMENT=o.reg(o.CP.background_attachment),o.BACKGROUND_COLOR=o.reg(o.CP.background_color),o.BACKGROUND_ORIGIN=o.reg(o.CP.background_origin),o.BACKGROUND_REPEAT=o.reg(o.CP.background_repeat),o.BORDER=o.reg(o.CP.border),o.BORDER_RADIUS=o.reg(o.CP.border_radius),o.BORDER_SPACING=o.reg(o.CP.border_spacing),o.BORDER_TOP_COLOR=o.reg(o.CP.border_top_color),o.BORDER_TOP_STYLE=o.reg(o.CP.border_top_style),o.BORDER_WIDTH=o.reg(o.CP.border_width),o.COLOR=o.reg(o.CP.color),o.CURSOR=o.reg(o.CP.cursor),o.DISPLAY=o.reg(o.CP.display),o.DISPLAY_OUTSIDE=o.reg(o.CP.display_outside),o.ELEVATION=o.reg(o.CP.elevation),o.FONT_FAMILY=o.reg(o.CP.font_family),o.HEIGHT=o.reg(o.CP.height),o.LETTER_SPACING=o.reg(o.CP.letter_spacing),o.LIST_STYLE_IMAGE=o.reg(o.CP.list_style_image),o.MARGIN_RIGHT=o.reg(o.CP.margin_right),o.MAX_HEIGHT=o.reg(o.CP.max_height),o.MIN_HEIGHT=o.reg(o.CP.min_height),o.OPACITY=o.reg(o.CP.opacity),o.OUTLINE_COLOR=o.reg(o.CP.outline_color),o.OUTLINE_WIDTH=o.reg(o.CP.outline_width),o.PADDING=o.reg(o.CP.padding),o.PADDING_TOP=o.reg(o.CP.padding_top),o.PITCH_RANGE=o.reg(o.CP.pitch_range),o.RIGHT=o.reg(o.CP.right),o.STRESS=o.reg(o.CP.stress),o.TEXT_INDENT=o.reg(o.CP.text_indent),o.TEXT_SHADOW=o.reg(o.CP.text_shadow),o.VOLUME=o.reg(o.CP.volume),o.WORD_WRAP=o.reg(o.CP.word_wrap),o.ZOOM=o.reg(o.CP.zoom),o.BACKFACE_VISIBILITY=o.reg(o.CP.backface_visibility),o.BACKGROUND_CLIP=o.reg(o.CP.background_clip),o.BACKGROUND_POSITION=o.reg(o.CP.background_position),o.BORDER_BOTTOM_COLOR=o.reg(o.CP.border_bottom_color),o.BORDER_BOTTOM_STYLE=o.reg(o.CP.border_bottom_style),o.BORDER_COLOR=o.reg(o.CP.border_color),o.BORDER_LEFT_COLOR=o.reg(o.CP.border_left_color),o.BORDER_RIGHT_COLOR=o.reg(o.CP.border_right_color),o.BORDER_STYLE=o.reg(o.CP.border_style),o.BORDER_TOP_LEFT_RADIUS=o.reg(o.CP.border_top_left_radius),o.BORDER_TOP_WIDTH=o.reg(o.CP.border_top_width),o.BOX_SHADOW=o.reg(o.CP.box_shadow),o.CLIP=o.reg(o.CP.clip),o.DISPLAY_INSIDE=o.reg(o.CP.display_inside),o.FONT_SIZE=o.reg(o.CP.font_size),o.LINE_HEIGHT=o.reg(o.CP.line_height),o.MARGIN_LEFT=o.reg(o.CP.margin_left),o.MAX_WIDTH=o.reg(o.CP.max_width),o.OUTLINE_STYLE=o.reg(o.CP.outline_style),o.PADDING_BOTTOM=o.reg(o.CP.padding_bottom),o.PADDING_RIGHT=o.reg(o.CP.padding_right),o.PERSPECTIVE=o.reg(o.CP.perspective),o.RICHNESS=o.reg(o.CP.richness),o.TEXT_OVERFLOW=o.reg(o.CP.text_overflow),o.TOP=o.reg(o.CP.top),o.WIDTH=o.reg(o.CP.width),o.Z_INDEX=o.reg(o.CP.z_index),o.BACKGROUND=o.reg(o.CP.background),o.BACKGROUND_SIZE=o.reg(o.CP.background_size),o.BORDER_BOTTOM_LEFT_RADIUS=o.reg(o.CP.border_bottom_left_radius),o.BORDER_BOTTOM_WIDTH=o.reg(o.CP.border_bottom_width),o.BORDER_LEFT_STYLE=o.reg(o.CP.border_left_style),o.BORDER_RIGHT_STYLE=o.reg(o.CP.border_right_style),o.BORDER_TOP=o.reg(o.CP.border_top),o.BOTTOM=o.reg(o.CP.bottom),o.LIST_STYLE=o.reg(o.CP.list_style),o.MARGIN_TOP=o.reg(o.CP.margin_top),o.OUTLINE=o.reg(o.CP.outline),o.OVERFLOW_Y=o.reg(o.CP.overflow_y),o.PITCH=o.reg(o.CP.pitch),o.VERTICAL_ALIGN=o.reg(o.CP.vertical_align),o.WORD_SPACING=o.reg(o.CP.word_spacing),o.BACKGROUND_IMAGE=o.reg(o.CP.background_image),o.BORDER_BOTTOM_RIGHT_RADIUS=o.reg(o.CP.border_bottom_right_radius),o.BORDER_LEFT_WIDTH=o.reg(o.CP.border_left_width),o.BORDER_RIGHT_WIDTH=o.reg(o.CP.border_right_width),o.LEFT=o.reg(o.CP.left),o.MARGIN_BOTTOM=o.reg(o.CP.margin_bottom),o.PAUSE_AFTER=o.reg(o.CP.pause_after),o.SPEECH_RATE=o.reg(o.CP.speech_rate),o.TRANSITION_DURATION=o.reg(o.CP.transition_duration),o.BORDER_BOTTOM=o.reg(o.CP.border_bottom),o.BORDER_RIGHT=o.reg(o.CP.border_right),o.MARGIN=o.reg(o.CP.margin),o.PADDING_LEFT=o.reg(o.CP.padding_left),o.BORDER_LEFT=o.reg(o.CP.border_left),o.FONT=o.reg(o.CP.font),o.QUOTES=o.reg(o.CP.quotes),o.BORDER_TOP_RIGHT_RADIUS=o.reg(o.CP.border_top_right_radius),o.MIN_WIDTH=o.reg(o.CP.min_width);const a=new class{constructor(){this._options={allowedTags:["a","abbr","acronym","address","area","article","aside","audio","b","bdi","bdo","big","blockquote","br","button","canvas","caption","center","cite","code","col","colgroup","colspan","command","data","datalist","dd","del","details","dfn","dir","div","dl","dt","em","fieldset","figcaption","figure","font","footer","form","h1","h2","h3","h4","h5","h6","header","hgroup","hr","i","img","input","ins","kbd","label","legend","li","map","mark","menu","meter","nav","nobr","ol","optgroup","option","output","p","pre","progress","q","rowspan","s","samp","section","select","small","source","span","strike","strong","sub","summary","sup","table","tbody","td","textarea","tfoot","th","thead","time","tr","track","tt","u","ul","var","video","wbr"],allowedAttributes:{"*":["class","dir","draggable","hidden","id","inert","itemprop","itemref","itemscope","lang","spellcheck","style","title","translate"],a:["accesskey","coords","href","hreflang","name","rel","shape","tabindex","target","type"],area:["accesskey","alt","coords","href","nohref","shape","tabindex"],audio:["autoplay","controls","loop","mediagroup","muted","preload","src"],bdo:["dir"],blockquote:["cite"],br:["clear"],button:["accesskey","disabled","name","tabindex","type","value"],canvas:["height","width"],caption:["align"],col:["align","char","charoff","span","valign","width"],colgroup:["align","char","charoff","span","valign","width"],command:["checked","command","disabled","icon","label","radiogroup","type"],data:["value"],del:["cite","datetime"],details:["open"],dir:["compact"],div:["align"],dl:["compact"],fieldset:["disabled"],font:["color","face","size"],form:["accept","action","autocomplete","enctype","method","name","novalidate"],h1:["align"],h2:["align"],h3:["align"],h4:["align"],h5:["align"],h6:["align"],hr:["align","noshade","size","width"],iframe:["align","frameborder","height","marginheight","marginwidth","width"],img:["align","alt","border","height","hspace","ismap","name","src","usemap","vspace","width"],input:["accept","accesskey","align","alt","autocomplete","checked","disabled","inputmode","ismap","list","max","maxlength","min","multiple","name","placeholder","readonly","required","size","src","step","tabindex","type","usemap","value"],ins:["cite","datetime"],label:["accesskey","for"],legend:["accesskey","align"],li:["type","value"],map:["name"],menu:["compact","label","type"],meter:["high","low","max","min","value"],ol:["compact","reversed","start","type"],optgroup:["disabled","label"],option:["disabled","label","selected","value"],output:["for","name"],p:["align"],pre:["width"],progress:["max","min","value"],q:["cite"],select:["autocomplete","disabled","multiple","name","required","size","tabindex"],source:["type"],table:["align","bgcolor","border","cellpadding","cellspacing","frame","rules","summary","width"],tbody:["align","char","charoff","valign"],td:["abbr","align","axis","bgcolor","char","charoff","colspan","headers","height","nowrap","rowspan","scope","valign","width"],textarea:["accesskey","autocomplete","cols","disabled","inputmode","name","placeholder","readonly","required","rows","tabindex","wrap"],tfoot:["align","char","charoff","valign"],th:["abbr","align","axis","bgcolor","char","charoff","colspan","headers","height","nowrap","rowspan","scope","valign","width"],thead:["align","char","charoff","valign"],tr:["align","bgcolor","char","charoff","valign"],track:["default","kind","label","srclang"],ul:["compact","type"],video:["autoplay","controls","height","loop","mediagroup","muted","poster","preload","src","width"]},allowedStyles:{"*":{"backface-visibility":[o.BACKFACE_VISIBILITY],background:[o.BACKGROUND],"background-attachment":[o.BACKGROUND_ATTACHMENT],"background-clip":[o.BACKGROUND_CLIP],"background-color":[o.BACKGROUND_COLOR],"background-image":[o.BACKGROUND_IMAGE],"background-origin":[o.BACKGROUND_ORIGIN],"background-position":[o.BACKGROUND_POSITION],"background-repeat":[o.BACKGROUND_REPEAT],"background-size":[o.BACKGROUND_SIZE],border:[o.BORDER],"border-bottom":[o.BORDER_BOTTOM],"border-bottom-color":[o.BORDER_BOTTOM_COLOR],"border-bottom-left-radius":[o.BORDER_BOTTOM_LEFT_RADIUS],"border-bottom-right-radius":[o.BORDER_BOTTOM_RIGHT_RADIUS],"border-bottom-style":[o.BORDER_BOTTOM_STYLE],"border-bottom-width":[o.BORDER_BOTTOM_WIDTH],"border-collapse":[o.BORDER_COLLAPSE],"border-color":[o.BORDER_COLOR],"border-left":[o.BORDER_LEFT],"border-left-color":[o.BORDER_LEFT_COLOR],"border-left-style":[o.BORDER_LEFT_STYLE],"border-left-width":[o.BORDER_LEFT_WIDTH],"border-radius":[o.BORDER_RADIUS],"border-right":[o.BORDER_RIGHT],"border-right-color":[o.BORDER_RIGHT_COLOR],"border-right-style":[o.BORDER_RIGHT_STYLE],"border-right-width":[o.BORDER_RIGHT_WIDTH],"border-spacing":[o.BORDER_SPACING],"border-style":[o.BORDER_STYLE],"border-top":[o.BORDER_TOP],"border-top-color":[o.BORDER_TOP_COLOR],"border-top-left-radius":[o.BORDER_TOP_LEFT_RADIUS],"border-top-right-radius":[o.BORDER_TOP_RIGHT_RADIUS],"border-top-style":[o.BORDER_TOP_STYLE],"border-top-width":[o.BORDER_TOP_WIDTH],"border-width":[o.BORDER_WIDTH],bottom:[o.BOTTOM],box:[o.BOX],"box-shadow":[o.BOX_SHADOW],"box-sizing":[o.BOX_SIZING],"caption-side":[o.CAPTION_SIDE],clear:[o.CLEAR],clip:[o.CLIP],color:[o.COLOR],cursor:[o.CURSOR],direction:[o.DIRECTION],display:[o.DISPLAY],"display-inside":[o.DISPLAY_INSIDE],"display-outside":[o.DISPLAY_OUTSIDE],elevation:[o.ELEVATION],"empty-cells":[o.EMPTY_CELLS],float:[o.FLOAT],font:[o.FONT],"font-family":[o.FONT_FAMILY],"font-size":[o.FONT_SIZE],"font-stretch":[o.FONT_STRETCH],"font-style":[o.FONT_STYLE],"font-variant":[o.FONT_VARIANT],"font-weight":[o.FONT_WEIGHT],height:[o.HEIGHT],left:[o.LEFT],"letter-spacing":[o.LETTER_SPACING],"line-height":[o.LINE_HEIGHT],"list-style":[o.LIST_STYLE],"list-style-image":[o.LIST_STYLE_IMAGE],"list-style-position":[o.LIST_STYLE_POSITION],"list-style-type":[o.LIST_STYLE_TYPE],margin:[o.MARGIN],"margin-bottom":[o.MARGIN_BOTTOM],"margin-left":[o.MARGIN_LEFT],"margin-right":[o.MARGIN_RIGHT],"margin-top":[o.MARGIN_TOP],"max-height":[o.MAX_HEIGHT],"max-width":[o.MAX_WIDTH],"min-height":[o.MIN_HEIGHT],"min-width":[o.MIN_WIDTH],opacity:[o.OPACITY],outline:[o.OUTLINE],"outline-color":[o.OUTLINE_COLOR],"outline-style":[o.OUTLINE_STYLE],"outline-width":[o.OUTLINE_WIDTH],overflow:[o.OVERFLOW],"overflow-wrap":[o.OVERFLOW_WRAP],"overflow-x":[o.OVERFLOW_X],"overflow-y":[o.OVERFLOW_Y],padding:[o.PADDING],"padding-bottom":[o.PADDING_BOTTOM],"padding-left":[o.PADDING_LEFT],"padding-right":[o.PADDING_RIGHT],"padding-top":[o.PADDING_TOP],"page-break-after":[o.PAGE_BREAK_AFTER],"page-break-before":[o.PAGE_BREAK_BEFORE],"page-break-inside":[o.PAGE_BREAK_INSIDE],"pause-after":[o.PAUSE_AFTER],perspective:[o.PERSPECTIVE],pitch:[o.PITCH],"pitch-range":[o.PITCH_RANGE],position:[o.POSITION],quotes:[o.QUOTES],resize:[o.RESIZE],richness:[o.RICHNESS],right:[o.RIGHT],speak:[o.SPEAK],"speak-header":[o.SPEAK_HEADER],"speak-numeral":[o.SPEAK_NUMERAL],"speak-punctuation":[o.SPEAK_PUNCTUATION],"speech-rate":[o.SPEECH_RATE],stress:[o.STRESS],"table-layout":[o.TABLE_LAYOUT],"text-align":[o.TEXT_ALIGN],"text-decoration":[o.TEXT_DECORATION],"text-indent":[o.TEXT_INDENT],"text-overflow":[o.TEXT_OVERFLOW],"text-shadow":[o.TEXT_SHADOW],"text-transform":[o.TEXT_TRANSFORM],"text-wrap":[o.TEXT_WRAP],top:[o.TOP],"unicode-bidi":[o.UNICODE_BIDI],"vertical-align":[o.VERTICAL_ALIGN],visibility:[o.VISIBILITY],volume:[o.VOLUME],"white-space":[o.WHITE_SPACE],width:[o.WIDTH],"word-break":[o.WORD_BREAK],"word-spacing":[o.WORD_SPACING],"word-wrap":[o.WORD_WRAP],"z-index":[o.Z_INDEX],zoom:[o.ZOOM]}},transformTags:{a:i.a.simpleTransform("a",{rel:"nofollow"}),input:i.a.simpleTransform("input",{disabled:"disabled"})},allowedSchemesByTag:{img:i.a.defaults.allowedSchemes.concat(["attachment"])},allowedSchemesAppliedToAttributes:["href","cite"]}}sanitize(e,t){return i()(e,Object.assign(Object.assign({},this._options),t||{}))}}},function(e,t,n){"use strict";var r=function(){};e.exports=r},function(e,t){e.exports=function(e,t){if(null==e)return{};var n,r,i={},o=Object.keys(e);for(r=0;r<o.length;r++)n=o[r],t.indexOf(n)>=0||(i[n]=e[n]);return i}},function(e,t,n){"use strict";var r=n(74),i=n(1),o=(n(3),n(204));t.a=function(e){var t=function(t){var n=e(t);return t.css?Object(i.a)({},Object(o.a)(n,e(Object(i.a)({theme:t.theme},t.css))),function(e,t){var n={};return Object.keys(e).forEach((function(r){-1===t.indexOf(r)&&(n[r]=e[r])})),n}(t.css,[e.filterProps])):n};return t.propTypes={},t.filterProps=["css"].concat(Object(r.a)(e.filterProps)),t}},function(e,t,n){"use strict";var r=n(25),i=n(85),o=Object(r.a)({prop:"displayPrint",cssProperty:!1,transform:function(e){return{"@media print":{display:e}}}}),a=Object(r.a)({prop:"display"}),s=Object(r.a)({prop:"overflow"}),l=Object(r.a)({prop:"textOverflow"}),u=Object(r.a)({prop:"visibility"}),c=Object(r.a)({prop:"whiteSpace"});t.a=Object(i.a)(o,a,s,l,u,c)},function(e,t,n){"use strict";var r=n(25),i=Object(r.a)({prop:"boxShadow",themeKey:"shadows"});t.a=i},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(4)),s=n(227),l=n(7),u=n(15),c=n(141),d=n(169),h=o.forwardRef((function(e,t){var n=e.children,l=e.classes,h=e.className,f=e.color,p=void 0===f?"primary":f,g=e.component,v=void 0===g?"div":g,m=e.disabled,y=void 0!==m&&m,b=e.error,C=void 0!==b&&b,A=e.fullWidth,_=void 0!==A&&A,I=e.focused,x=e.hiddenLabel,w=void 0!==x&&x,M=e.margin,S=void 0===M?"none":M,T=e.required,E=void 0!==T&&T,O=e.size,k=e.variant,L=void 0===k?"standard":k,D=Object(i.a)(e,["children","classes","className","color","component","disabled","error","fullWidth","focused","hiddenLabel","margin","required","size","variant"]),P=o.useState((function(){var e=!1;return n&&o.Children.forEach(n,(function(t){if(Object(c.a)(t,["Input","Select"])){var n=Object(c.a)(t,["Select"])?t.props.input:t;n&&Object(s.a)(n.props)&&(e=!0)}})),e})),N=P[0],j=P[1],R=o.useState((function(){var e=!1;return n&&o.Children.forEach(n,(function(t){Object(c.a)(t,["Input","Select"])&&Object(s.b)(t.props,!0)&&(e=!0)})),e})),z=R[0],F=R[1],B=o.useState(!1),H=B[0],U=B[1],V=void 0!==I?I:H;y&&V&&U(!1);var Y=o.useCallback((function(){F(!0)}),[]),W={adornedStart:N,setAdornedStart:j,color:p,disabled:y,error:C,filled:z,focused:V,fullWidth:_,hiddenLabel:w,margin:("small"===O?"dense":void 0)||S,onBlur:function(){U(!1)},onEmpty:o.useCallback((function(){F(!1)}),[]),onFilled:Y,onFocus:function(){U(!0)},registerEffect:void 0,required:E,variant:L};return o.createElement(d.a.Provider,{value:W},o.createElement(v,Object(r.a)({className:Object(a.a)(l.root,h,"none"!==S&&l["margin".concat(Object(u.a)(S))],_&&l.fullWidth),ref:t},D),n))}));t.a=Object(l.a)({root:{display:"inline-flex",flexDirection:"column",position:"relative",minWidth:0,padding:0,margin:0,border:0,verticalAlign:"top"},marginNormal:{marginTop:16,marginBottom:8},marginDense:{marginTop:8,marginBottom:4},fullWidth:{width:"100%"}},{name:"MuiFormControl"})(h)},function(e,t,n){"use strict";var r=n(5),i=n(1),o=n(0),a=(n(3),n(4)),s=n(142),l=n(106),u=n(15),c=n(7),d=o.forwardRef((function(e,t){var n=e.children,c=e.classes,d=e.className,h=(e.color,e.component),f=void 0===h?"label":h,p=(e.disabled,e.error,e.filled,e.focused,e.required,Object(r.a)(e,["children","classes","className","color","component","disabled","error","filled","focused","required"])),g=Object(l.a)(),v=Object(s.a)({props:e,muiFormControl:g,states:["color","required","focused","disabled","error","filled"]});return o.createElement(f,Object(i.a)({className:Object(a.a)(c.root,c["color".concat(Object(u.a)(v.color||"primary"))],d,v.disabled&&c.disabled,v.error&&c.error,v.filled&&c.filled,v.focused&&c.focused,v.required&&c.required),ref:t},p),n,v.required&&o.createElement("span",{"aria-hidden":!0,className:Object(a.a)(c.asterisk,v.error&&c.error)}," ","*"))}));t.a=Object(c.a)((function(e){return{root:Object(i.a)({color:e.palette.text.secondary},e.typography.body1,{lineHeight:1,padding:0,"&$focused":{color:e.palette.primary.main},"&$disabled":{color:e.palette.text.disabled},"&$error":{color:e.palette.error.main}}),colorSecondary:{"&$focused":{color:e.palette.secondary.main}},focused:{},disabled:{},error:{},filled:{},required:{},asterisk:{"&$error":{color:e.palette.error.main}}}}),{name:"MuiFormLabel"})(d)},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(4)),s=n(142),l=n(106),u=n(7),c=n(631),d=o.forwardRef((function(e,t){var n=e.classes,u=e.className,d=e.disableAnimation,h=void 0!==d&&d,f=(e.margin,e.shrink),p=(e.variant,Object(i.a)(e,["classes","className","disableAnimation","margin","shrink","variant"])),g=Object(l.a)(),v=f;void 0===v&&g&&(v=g.filled||g.focused||g.adornedStart);var m=Object(s.a)({props:e,muiFormControl:g,states:["margin","variant"]});return o.createElement(c.a,Object(r.a)({"data-shrink":v,className:Object(a.a)(n.root,u,g&&n.formControl,!h&&n.animated,v&&n.shrink,"dense"===m.margin&&n.marginDense,{filled:n.filled,outlined:n.outlined}[m.variant]),classes:{focused:n.focused,disabled:n.disabled,error:n.error,required:n.required,asterisk:n.asterisk},ref:t},p))}));t.a=Object(u.a)((function(e){return{root:{display:"block",transformOrigin:"top left"},focused:{},disabled:{},error:{},required:{},asterisk:{},formControl:{position:"absolute",left:0,top:0,transform:"translate(0, 24px) scale(1)"},marginDense:{transform:"translate(0, 21px) scale(1)"},shrink:{transform:"translate(0, 1.5px) scale(0.75)",transformOrigin:"top left"},animated:{transition:e.transitions.create(["color","transform"],{duration:e.transitions.duration.shorter,easing:e.transitions.easing.easeOut})},filled:{zIndex:1,pointerEvents:"none",transform:"translate(12px, 20px) scale(1)","&$marginDense":{transform:"translate(12px, 17px) scale(1)"},"&$shrink":{transform:"translate(12px, 10px) scale(0.75)","&$marginDense":{transform:"translate(12px, 7px) scale(0.75)"}}},outlined:{zIndex:1,pointerEvents:"none",transform:"translate(14px, 20px) scale(1)","&$marginDense":{transform:"translate(14px, 12px) scale(1)"},"&$shrink":{transform:"translate(14px, -6px) scale(0.75)"}}}}),{name:"MuiInputLabel"})(d)},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(22)),s=n(105),l=n(4),u=n(59),c=n(202),d=n(107),h=n(7),f=n(462),p=n(1312),g=n(336);function v(e,t){var n=0;return"number"==typeof t?n=t:"center"===t?n=e.height/2:"bottom"===t&&(n=e.height),n}function m(e,t){var n=0;return"number"==typeof t?n=t:"center"===t?n=e.width/2:"right"===t&&(n=e.width),n}function y(e){return[e.horizontal,e.vertical].map((function(e){return"number"==typeof e?"".concat(e,"px"):e})).join(" ")}function b(e){return"function"==typeof e?e():e}var C=o.forwardRef((function(e,t){var n=e.action,h=e.anchorEl,C=e.anchorOrigin,A=void 0===C?{vertical:"top",horizontal:"left"}:C,_=e.anchorPosition,I=e.anchorReference,x=void 0===I?"anchorEl":I,w=e.children,M=e.classes,S=e.className,T=e.container,E=e.elevation,O=void 0===E?8:E,k=e.getContentAnchorEl,L=e.marginThreshold,D=void 0===L?16:L,P=e.onEnter,N=e.onEntered,j=e.onEntering,R=e.onExit,z=e.onExited,F=e.onExiting,B=e.open,H=e.PaperProps,U=void 0===H?{}:H,V=e.transformOrigin,Y=void 0===V?{vertical:"top",horizontal:"left"}:V,W=e.TransitionComponent,G=void 0===W?p.a:W,X=e.transitionDuration,q=void 0===X?"auto":X,Z=e.TransitionProps,Q=void 0===Z?{}:Z,K=Object(i.a)(e,["action","anchorEl","anchorOrigin","anchorPosition","anchorReference","children","classes","className","container","elevation","getContentAnchorEl","marginThreshold","onEnter","onEntered","onEntering","onExit","onExited","onExiting","open","PaperProps","transformOrigin","TransitionComponent","transitionDuration","TransitionProps"]),J=o.useRef(),$=o.useCallback((function(e){if("anchorPosition"===x)return _;var t=b(h),n=(t&&1===t.nodeType?t:Object(u.a)(J.current).body).getBoundingClientRect(),r=0===e?A.vertical:"center";return{top:n.top+v(n,r),left:n.left+m(n,A.horizontal)}}),[h,A.horizontal,A.vertical,_,x]),ee=o.useCallback((function(e){var t=0;if(k&&"anchorEl"===x){var n=k(e);if(n&&e.contains(n)){var r=function(e,t){for(var n=t,r=0;n&&n!==e;)r+=(n=n.parentElement).scrollTop;return r}(e,n);t=n.offsetTop+n.clientHeight/2-r||0}0}return t}),[A.vertical,x,k]),te=o.useCallback((function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:0;return{vertical:v(e,Y.vertical)+t,horizontal:m(e,Y.horizontal)}}),[Y.horizontal,Y.vertical]),ne=o.useCallback((function(e){var t=ee(e),n={width:e.offsetWidth,height:e.offsetHeight},r=te(n,t);if("none"===x)return{top:null,left:null,transformOrigin:y(r)};var i=$(t),o=i.top-r.vertical,a=i.left-r.horizontal,s=o+n.height,l=a+n.width,u=Object(c.a)(b(h)),d=u.innerHeight-D,f=u.innerWidth-D;if(o<D){var p=o-D;o-=p,r.vertical+=p}else if(s>d){var g=s-d;o-=g,r.vertical+=g}if(a<D){var v=a-D;a-=v,r.horizontal+=v}else if(l>f){var m=l-f;a-=m,r.horizontal+=m}return{top:"".concat(Math.round(o),"px"),left:"".concat(Math.round(a),"px"),transformOrigin:y(r)}}),[h,x,$,ee,te,D]),re=o.useCallback((function(){var e=J.current;if(e){var t=ne(e);null!==t.top&&(e.style.top=t.top),null!==t.left&&(e.style.left=t.left),e.style.transformOrigin=t.transformOrigin}}),[ne]),ie=o.useCallback((function(e){J.current=a.findDOMNode(e)}),[]);o.useEffect((function(){B&&re()})),o.useImperativeHandle(n,(function(){return B?{updatePosition:function(){re()}}:null}),[B,re]),o.useEffect((function(){if(B){var e=Object(s.a)((function(){re()}));return window.addEventListener("resize",e),function(){e.clear(),window.removeEventListener("resize",e)}}}),[B,re]);var oe=q;"auto"!==q||G.muiSupportAuto||(oe=void 0);var ae=T||(h?Object(u.a)(b(h)).body:void 0);return o.createElement(f.a,Object(r.a)({container:ae,open:B,ref:t,BackdropProps:{invisible:!0},className:Object(l.a)(M.root,S)},K),o.createElement(G,Object(r.a)({appear:!0,in:B,onEnter:P,onEntered:N,onExit:R,onExited:z,onExiting:F,timeout:oe},Q,{onEntering:Object(d.a)((function(e,t){j&&j(e,t),re()}),Q.onEntering)}),o.createElement(g.a,Object(r.a)({elevation:O,ref:ie},U,{className:Object(l.a)(M.paper,U.className)}),w)))}));t.a=Object(h.a)({root:{},paper:{position:"absolute",overflowY:"auto",overflowX:"hidden",minWidth:16,minHeight:16,maxWidth:"calc(100% - 32px)",maxHeight:"calc(100% - 32px)",outline:0}},{name:"MuiPopover"})(C)},function(e,t,n){var r=n(1583),i=n(1584),o=n(942),a=n(1585);e.exports=function(e,t){return r(e)||i(e,t)||o(e,t)||a()}},function(e,t,n){"use strict";var r=n(1616),i=[],o=[],a=r.makeRequestCallFromTimer((function(){if(o.length)throw o.shift()}));function s(e){var t;(t=i.length?i.pop():new l).task=e,r(t)}function l(){this.task=null}e.exports=s,l.prototype.call=function(){try{this.task.call()}catch(e){s.onerror?s.onerror(e):(o.push(e),a())}finally{this.task=null,i[i.length]=this}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){(0,i.default)(1,arguments);var t=(0,r.default)(e);return!isNaN(t)};var r=o(n(47)),i=o(n(40));function o(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t,n){(0,h.default)(2,arguments);var f=String(t),v=n||{},m=v.locale||i.default,C=m.options&&m.options.firstWeekContainsDate,A=null==C?1:(0,d.default)(C),_=null==v.firstWeekContainsDate?A:(0,d.default)(v.firstWeekContainsDate);if(!(_>=1&&_<=7))throw new RangeError("firstWeekContainsDate must be between 1 and 7 inclusively");var I=m.options&&m.options.weekStartsOn,x=null==I?0:(0,d.default)(I),w=null==v.weekStartsOn?x:(0,d.default)(v.weekStartsOn);if(!(w>=0&&w<=6))throw new RangeError("weekStartsOn must be between 0 and 6 inclusively");if(!m.localize)throw new RangeError("locale must contain localize property");if(!m.formatLong)throw new RangeError("locale must contain formatLong property");var M=(0,a.default)(e);if(!(0,r.default)(M))throw new RangeError("Invalid time value");var S=(0,u.default)(M),T=(0,o.default)(M,S),E={firstWeekContainsDate:_,weekStartsOn:w,locale:m,_originalDate:M},O=f.match(g).map((function(e){var t=e[0];return"p"===t||"P"===t?(0,l.default[t])(e,m.formatLong,E):e})).join("").match(p).map((function(n){if("''"===n)return"'";var r=n[0];if("'"===r)return b(n);var i=s.default[r];if(i)return!v.useAdditionalWeekYearTokens&&(0,c.isProtectedWeekYearToken)(n)&&(0,c.throwProtectedError)(n,t,e),!v.useAdditionalDayOfYearTokens&&(0,c.isProtectedDayOfYearToken)(n)&&(0,c.throwProtectedError)(n,t,e),i(T,n,m.localize,E);if(r.match(y))throw new RangeError("Format string contains an unescaped latin alphabet character `"+r+"`");return n})).join("");return O};var r=f(n(636)),i=f(n(1179)),o=f(n(1180)),a=f(n(47)),s=f(n(2269)),l=f(n(1181)),u=f(n(1182)),c=n(1183),d=f(n(77)),h=f(n(40));function f(e){return e&&e.__esModule?e:{default:e}}var p=/[yYQqMLwIdDecihHKkms]o|(\w)\1*|''|'(''|[^'])+('|$)|./g,g=/P+p+|P+|p+|''|'(''|[^'])+('|$)|./g,v=/^'([^]*?)'?$/,m=/''/g,y=/[a-zA-Z]/;function b(e){return e.match(v)[1].replace(m,"'")}e.exports=t.default},function(e,t,n){"use strict";n.r(t),n.d(t,"LineGeometry",(function(){return i}));var r=n(230),i=function(){r.a.call(this),this.type="LineGeometry"};i.prototype=Object.assign(Object.create(r.a.prototype),{constructor:i,isLineGeometry:!0,setPositions:function(e){for(var t=e.length-3,n=new Float32Array(2*t),i=0;i<t;i+=3)n[2*i]=e[i],n[2*i+1]=e[i+1],n[2*i+2]=e[i+2],n[2*i+3]=e[i+3],n[2*i+4]=e[i+4],n[2*i+5]=e[i+5];return r.a.prototype.setPositions.call(this,n),this},setColors:function(e){for(var t=e.length-3,n=new Float32Array(2*t),i=0;i<t;i+=3)n[2*i]=e[i],n[2*i+1]=e[i+1],n[2*i+2]=e[i+2],n[2*i+3]=e[i+3],n[2*i+4]=e[i+4],n[2*i+5]=e[i+5];return r.a.prototype.setColors.call(this,n),this},fromLine:function(e){var t=e.geometry;return t.isGeometry?this.setPositions(t.vertices):t.isBufferGeometry&&this.setPositions(t.attributes.position.array),this},copy:function(){return this}})},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(2373)),s=n(49),l=n(140),u=n(334),c=(n(116),n(4)),d=n(59),h=n(15),f=n(436),p=n(227),g=n(37),v=n(100);function m(e,t){return"object"===Object(l.a)(t)&&null!==t?e===t:String(e)===String(t)}var y=o.forwardRef((function(e,t){var n=e["aria-label"],a=e.autoFocus,l=e.autoWidth,y=e.children,b=e.classes,C=e.className,A=e.defaultValue,_=e.disabled,I=e.displayEmpty,x=e.IconComponent,w=e.inputRef,M=e.labelId,S=e.MenuProps,T=void 0===S?{}:S,E=e.multiple,O=e.name,k=e.onBlur,L=e.onChange,D=e.onClose,P=e.onFocus,N=e.onOpen,j=e.open,R=e.readOnly,z=e.renderValue,F=e.SelectDisplayProps,B=void 0===F?{}:F,H=e.tabIndex,U=(e.type,e.value),V=e.variant,Y=void 0===V?"standard":V,W=Object(i.a)(e,["aria-label","autoFocus","autoWidth","children","classes","className","defaultValue","disabled","displayEmpty","IconComponent","inputRef","labelId","MenuProps","multiple","name","onBlur","onChange","onClose","onFocus","onOpen","open","readOnly","renderValue","SelectDisplayProps","tabIndex","type","value","variant"]),G=Object(v.a)({controlled:U,default:A,name:"Select"}),X=Object(s.a)(G,2),q=X[0],Z=X[1],Q=o.useRef(null),K=o.useState(null),J=K[0],$=K[1],ee=o.useRef(null!=j).current,te=o.useState(),ne=te[0],re=te[1],ie=o.useState(!1),oe=ie[0],ae=ie[1],se=Object(g.a)(t,w);o.useImperativeHandle(se,(function(){return{focus:function(){J.focus()},node:Q.current,value:q}}),[J,q]),o.useEffect((function(){a&&J&&J.focus()}),[a,J]),o.useEffect((function(){if(J){var e=Object(d.a)(J).getElementById(M);if(e){var t=function(){getSelection().isCollapsed&&J.focus()};return e.addEventListener("click",t),function(){e.removeEventListener("click",t)}}}}),[M,J]);var le,ue,ce=function(e,t){e?N&&N(t):D&&D(t),ee||(re(l?null:J.clientWidth),ae(e))},de=o.Children.toArray(y),he=function(e){return function(t){var n;if(E||ce(!1,t),E){n=Array.isArray(q)?q.slice():[];var r=q.indexOf(e.props.value);-1===r?n.push(e.props.value):n.splice(r,1)}else n=e.props.value;e.props.onClick&&e.props.onClick(t),q!==n&&(Z(n),L&&(t.persist(),Object.defineProperty(t,"target",{writable:!0,value:{value:n,name:O}}),L(t,e)))}},fe=null!==J&&(ee?j:oe);delete W["aria-invalid"];var pe=[],ge=!1;(Object(p.b)({value:q})||I)&&(z?le=z(q):ge=!0);var ve=de.map((function(e){if(!o.isValidElement(e))return null;var t;if(E){if(!Array.isArray(q))throw new Error(Object(u.a)(2));(t=q.some((function(t){return m(t,e.props.value)})))&&ge&&pe.push(e.props.children)}else(t=m(q,e.props.value))&&ge&&(ue=e.props.children);return t&&!0,o.cloneElement(e,{"aria-selected":t?"true":void 0,onClick:he(e),onKeyUp:function(t){" "===t.key&&t.preventDefault(),e.props.onKeyUp&&e.props.onKeyUp(t)},role:"option",selected:t,value:void 0,"data-value":e.props.value})}));ge&&(le=E?pe.join(", "):ue);var me,ye=ne;!l&&ee&&J&&(ye=J.clientWidth),me=void 0!==H?H:_?null:0;var be=B.id||(O?"mui-component-select-".concat(O):void 0);return o.createElement(o.Fragment,null,o.createElement("div",Object(r.a)({className:Object(c.a)(b.root,b.select,b.selectMenu,b[Y],C,_&&b.disabled),ref:$,tabIndex:me,role:"button","aria-disabled":_?"true":void 0,"aria-expanded":fe?"true":void 0,"aria-haspopup":"listbox","aria-label":n,"aria-labelledby":[M,be].filter(Boolean).join(" ")||void 0,onKeyDown:function(e){if(!R){-1!==[" ","ArrowUp","ArrowDown","Enter"].indexOf(e.key)&&(e.preventDefault(),ce(!0,e))}},onMouseDown:_||R?null:function(e){0===e.button&&(e.preventDefault(),J.focus(),ce(!0,e))},onBlur:function(e){!fe&&k&&(e.persist(),Object.defineProperty(e,"target",{writable:!0,value:{value:q,name:O}}),k(e))},onFocus:P},B,{id:be}),function(e){return null==e||"string"==typeof e&&!e.trim()}(le)?o.createElement("span",{dangerouslySetInnerHTML:{__html:"&#8203;"}}):le),o.createElement("input",Object(r.a)({value:Array.isArray(q)?q.join(","):q,name:O,ref:Q,"aria-hidden":!0,onChange:function(e){var t=de.map((function(e){return e.props.value})).indexOf(e.target.value);if(-1!==t){var n=de[t];Z(n.props.value),L&&L(e,n)}},tabIndex:-1,className:b.nativeInput,autoFocus:a},W)),o.createElement(x,{className:Object(c.a)(b.icon,b["icon".concat(Object(h.a)(Y))],fe&&b.iconOpen,_&&b.disabled)}),o.createElement(f.a,Object(r.a)({id:"menu-".concat(O||""),anchorEl:J,open:fe,onClose:function(e){ce(!1,e)}},T,{MenuListProps:Object(r.a)({"aria-labelledby":M,role:"listbox",disableListWrap:!0},T.MenuListProps),PaperProps:Object(r.a)({},T.PaperProps,{style:Object(r.a)({minWidth:ye},null!=T.PaperProps?T.PaperProps.style:null)})}),ve))})),b=n(142),C=n(106),A=n(7),_=n(455),I=n(454),x=n(1286),w=n(456),M=n(2355),S=n(2363),T=x.b,E=o.createElement(I.a,null),O=o.createElement(M.a,null),k=o.forwardRef((function e(t,n){var s=t.autoWidth,l=void 0!==s&&s,u=t.children,c=t.classes,d=t.displayEmpty,h=void 0!==d&&d,f=t.IconComponent,p=void 0===f?_.a:f,g=t.id,v=t.input,m=t.inputProps,A=t.label,I=t.labelId,x=t.labelWidth,M=void 0===x?0:x,T=t.MenuProps,k=t.multiple,L=void 0!==k&&k,D=t.native,P=void 0!==D&&D,N=t.onClose,j=t.onOpen,R=t.open,z=t.renderValue,F=t.SelectDisplayProps,B=t.variant,H=void 0===B?"standard":B,U=Object(i.a)(t,["autoWidth","children","classes","displayEmpty","IconComponent","id","input","inputProps","label","labelId","labelWidth","MenuProps","multiple","native","onClose","onOpen","open","renderValue","SelectDisplayProps","variant"]),V=P?w.a:y,Y=Object(C.a)(),W=Object(b.a)({props:t,muiFormControl:Y,states:["variant"]}).variant||H,G=v||{standard:E,outlined:o.createElement(S.a,{label:A,labelWidth:M}),filled:O}[W];return o.cloneElement(G,Object(r.a)({inputComponent:V,inputProps:Object(r.a)({children:u,IconComponent:p,variant:W,type:void 0,multiple:L},P?{id:g}:{autoWidth:l,displayEmpty:h,labelId:I,MenuProps:T,onClose:N,onOpen:j,open:R,renderValue:z,SelectDisplayProps:Object(r.a)({id:g},F)},m,{classes:m?Object(a.a)({baseClasses:c,newClasses:m.classes,Component:e}):c},v?v.props.inputProps:{}),ref:n},U))}));k.muiName="Select";t.a=Object(A.a)(T,{name:"MuiSelect"})(k)},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.InputGroup=void 0;var r=n(2),i=r.__importDefault(n(11)),o=r.__importStar(n(0)),a=n(13),s=n(239),l=r.__importStar(n(504)),u=n(209),c=n(873),d=n(1470),h=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.state={},t.leftElement=null,t.rightElement=null,t.refHandlers={leftElement:function(e){return t.leftElement=e},rightElement:function(e){return t.rightElement=e}},t}return r.__extends(t,e),t.prototype.render=function(){var e,t=this.props,n=t.asyncControl,a=void 0!==n&&n,l=t.className,c=t.disabled,h=t.fill,f=t.inputRef,p=t.intent,g=t.large,v=t.small,m=t.round,y=i.default(s.Classes.INPUT_GROUP,s.Classes.intentClass(p),((e={})[s.Classes.DISABLED]=c,e[s.Classes.FILL]=h,e[s.Classes.LARGE]=g,e[s.Classes.SMALL]=v,e[s.Classes.ROUND]=m,e),l),b=r.__assign(r.__assign({},this.props.style),{paddingLeft:this.state.leftElementWidth,paddingRight:this.state.rightElementWidth}),C=r.__assign(r.__assign({type:"text"},u.removeNonHTMLProps(this.props)),{className:s.Classes.INPUT,style:b});return o.createElement("div",{className:y},this.maybeRenderLeftElement(),a?o.createElement(d.AsyncControllableInput,r.__assign({},C,{inputRef:f})):o.createElement("input",r.__assign({},C,{ref:f})),this.maybeRenderRightElement())},t.prototype.componentDidMount=function(){this.updateInputWidth()},t.prototype.componentDidUpdate=function(e){var t=this.props,n=t.leftElement,r=t.rightElement;e.leftElement===n&&e.rightElement===r||this.updateInputWidth()},t.prototype.validateProps=function(e){null!=e.leftElement&&null!=e.leftIcon&&console.warn(l.INPUT_WARN_LEFT_ELEMENT_LEFT_ICON_MUTEX)},t.prototype.maybeRenderLeftElement=function(){var e=this.props,t=e.leftElement,n=e.leftIcon;return null!=t?o.createElement("span",{className:s.Classes.INPUT_LEFT_CONTAINER,ref:this.refHandlers.leftElement},t):null!=n?o.createElement(c.Icon,{icon:n}):void 0},t.prototype.maybeRenderRightElement=function(){var e=this.props.rightElement;if(null!=e)return o.createElement("span",{className:s.Classes.INPUT_ACTION,ref:this.refHandlers.rightElement},e)},t.prototype.updateInputWidth=function(){var e=this.state,t=e.leftElementWidth,n=e.rightElementWidth;if(null!=this.leftElement){var r=this.leftElement.clientWidth;(void 0===t||Math.abs(r-t)>2)&&this.setState({leftElementWidth:r})}else this.setState({leftElementWidth:void 0});if(null!=this.rightElement){r=this.rightElement.clientWidth;(void 0===n||Math.abs(r-n)>2)&&this.setState({rightElementWidth:r})}else this.setState({rightElementWidth:void 0})},t.displayName=u.DISPLAYNAME_PREFIX+".InputGroup",t=r.__decorate([a.polyfill],t)}(s.AbstractPureComponent2);t.InputGroup=h},function(e,t,n){"use strict";t.__esModule=!0;var r=o(n(0)),i=o(n(1476));function o(e){return e&&e.__esModule?e:{default:e}}t.default=r.default.createContext||i.default,e.exports=t.default},function(e,t,n){"use strict";(function(e){
/**!
 * @fileOverview Kickass library to create and place poppers near their reference elements.
 * @version 1.16.1
 * @license
 * Copyright (c) 2016 Federico Zivolo and contributors
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
var n="undefined"!=typeof window&&"undefined"!=typeof document&&"undefined"!=typeof navigator,r=function(){for(var e=["Edge","Trident","Firefox"],t=0;t<e.length;t+=1)if(n&&navigator.userAgent.indexOf(e[t])>=0)return 1;return 0}();var i=n&&window.Promise?function(e){var t=!1;return function(){t||(t=!0,window.Promise.resolve().then((function(){t=!1,e()})))}}:function(e){var t=!1;return function(){t||(t=!0,setTimeout((function(){t=!1,e()}),r))}};function o(e){return e&&"[object Function]"==={}.toString.call(e)}function a(e,t){if(1!==e.nodeType)return[];var n=e.ownerDocument.defaultView.getComputedStyle(e,null);return t?n[t]:n}function s(e){return"HTML"===e.nodeName?e:e.parentNode||e.host}function l(e){if(!e)return document.body;switch(e.nodeName){case"HTML":case"BODY":return e.ownerDocument.body;case"#document":return e.body}var t=a(e),n=t.overflow,r=t.overflowX,i=t.overflowY;return/(auto|scroll|overlay)/.test(n+i+r)?e:l(s(e))}function u(e){return e&&e.referenceNode?e.referenceNode:e}var c=n&&!(!window.MSInputMethodContext||!document.documentMode),d=n&&/MSIE 10/.test(navigator.userAgent);function h(e){return 11===e?c:10===e?d:c||d}function f(e){if(!e)return document.documentElement;for(var t=h(10)?document.body:null,n=e.offsetParent||null;n===t&&e.nextElementSibling;)n=(e=e.nextElementSibling).offsetParent;var r=n&&n.nodeName;return r&&"BODY"!==r&&"HTML"!==r?-1!==["TH","TD","TABLE"].indexOf(n.nodeName)&&"static"===a(n,"position")?f(n):n:e?e.ownerDocument.documentElement:document.documentElement}function p(e){return null!==e.parentNode?p(e.parentNode):e}function g(e,t){if(!(e&&e.nodeType&&t&&t.nodeType))return document.documentElement;var n=e.compareDocumentPosition(t)&Node.DOCUMENT_POSITION_FOLLOWING,r=n?e:t,i=n?t:e,o=document.createRange();o.setStart(r,0),o.setEnd(i,0);var a,s,l=o.commonAncestorContainer;if(e!==l&&t!==l||r.contains(i))return"BODY"===(s=(a=l).nodeName)||"HTML"!==s&&f(a.firstElementChild)!==a?f(l):l;var u=p(e);return u.host?g(u.host,t):g(e,p(t).host)}function v(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:"top",n="top"===t?"scrollTop":"scrollLeft",r=e.nodeName;if("BODY"===r||"HTML"===r){var i=e.ownerDocument.documentElement,o=e.ownerDocument.scrollingElement||i;return o[n]}return e[n]}function m(e,t){var n=arguments.length>2&&void 0!==arguments[2]&&arguments[2],r=v(t,"top"),i=v(t,"left"),o=n?-1:1;return e.top+=r*o,e.bottom+=r*o,e.left+=i*o,e.right+=i*o,e}function y(e,t){var n="x"===t?"Left":"Top",r="Left"===n?"Right":"Bottom";return parseFloat(e["border"+n+"Width"])+parseFloat(e["border"+r+"Width"])}function b(e,t,n,r){return Math.max(t["offset"+e],t["scroll"+e],n["client"+e],n["offset"+e],n["scroll"+e],h(10)?parseInt(n["offset"+e])+parseInt(r["margin"+("Height"===e?"Top":"Left")])+parseInt(r["margin"+("Height"===e?"Bottom":"Right")]):0)}function C(e){var t=e.body,n=e.documentElement,r=h(10)&&getComputedStyle(n);return{height:b("Height",t,n,r),width:b("Width",t,n,r)}}var A=function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")},_=function(){function e(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}return function(t,n,r){return n&&e(t.prototype,n),r&&e(t,r),t}}(),I=function(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e},x=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e};function w(e){return x({},e,{right:e.left+e.width,bottom:e.top+e.height})}function M(e){var t={};try{if(h(10)){t=e.getBoundingClientRect();var n=v(e,"top"),r=v(e,"left");t.top+=n,t.left+=r,t.bottom+=n,t.right+=r}else t=e.getBoundingClientRect()}catch(e){}var i={left:t.left,top:t.top,width:t.right-t.left,height:t.bottom-t.top},o="HTML"===e.nodeName?C(e.ownerDocument):{},s=o.width||e.clientWidth||i.width,l=o.height||e.clientHeight||i.height,u=e.offsetWidth-s,c=e.offsetHeight-l;if(u||c){var d=a(e);u-=y(d,"x"),c-=y(d,"y"),i.width-=u,i.height-=c}return w(i)}function S(e,t){var n=arguments.length>2&&void 0!==arguments[2]&&arguments[2],r=h(10),i="HTML"===t.nodeName,o=M(e),s=M(t),u=l(e),c=a(t),d=parseFloat(c.borderTopWidth),f=parseFloat(c.borderLeftWidth);n&&i&&(s.top=Math.max(s.top,0),s.left=Math.max(s.left,0));var p=w({top:o.top-s.top-d,left:o.left-s.left-f,width:o.width,height:o.height});if(p.marginTop=0,p.marginLeft=0,!r&&i){var g=parseFloat(c.marginTop),v=parseFloat(c.marginLeft);p.top-=d-g,p.bottom-=d-g,p.left-=f-v,p.right-=f-v,p.marginTop=g,p.marginLeft=v}return(r&&!n?t.contains(u):t===u&&"BODY"!==u.nodeName)&&(p=m(p,t)),p}function T(e){var t=arguments.length>1&&void 0!==arguments[1]&&arguments[1],n=e.ownerDocument.documentElement,r=S(e,n),i=Math.max(n.clientWidth,window.innerWidth||0),o=Math.max(n.clientHeight,window.innerHeight||0),a=t?0:v(n),s=t?0:v(n,"left"),l={top:a-r.top+r.marginTop,left:s-r.left+r.marginLeft,width:i,height:o};return w(l)}function E(e){var t=e.nodeName;if("BODY"===t||"HTML"===t)return!1;if("fixed"===a(e,"position"))return!0;var n=s(e);return!!n&&E(n)}function O(e){if(!e||!e.parentElement||h())return document.documentElement;for(var t=e.parentElement;t&&"none"===a(t,"transform");)t=t.parentElement;return t||document.documentElement}function k(e,t,n,r){var i=arguments.length>4&&void 0!==arguments[4]&&arguments[4],o={top:0,left:0},a=i?O(e):g(e,u(t));if("viewport"===r)o=T(a,i);else{var c=void 0;"scrollParent"===r?"BODY"===(c=l(s(t))).nodeName&&(c=e.ownerDocument.documentElement):c="window"===r?e.ownerDocument.documentElement:r;var d=S(c,a,i);if("HTML"!==c.nodeName||E(a))o=d;else{var h=C(e.ownerDocument),f=h.height,p=h.width;o.top+=d.top-d.marginTop,o.bottom=f+d.top,o.left+=d.left-d.marginLeft,o.right=p+d.left}}var v="number"==typeof(n=n||0);return o.left+=v?n:n.left||0,o.top+=v?n:n.top||0,o.right-=v?n:n.right||0,o.bottom-=v?n:n.bottom||0,o}function L(e){return e.width*e.height}function D(e,t,n,r,i){var o=arguments.length>5&&void 0!==arguments[5]?arguments[5]:0;if(-1===e.indexOf("auto"))return e;var a=k(n,r,o,i),s={top:{width:a.width,height:t.top-a.top},right:{width:a.right-t.right,height:a.height},bottom:{width:a.width,height:a.bottom-t.bottom},left:{width:t.left-a.left,height:a.height}},l=Object.keys(s).map((function(e){return x({key:e},s[e],{area:L(s[e])})})).sort((function(e,t){return t.area-e.area})),u=l.filter((function(e){var t=e.width,r=e.height;return t>=n.clientWidth&&r>=n.clientHeight})),c=u.length>0?u[0].key:l[0].key,d=e.split("-")[1];return c+(d?"-"+d:"")}function P(e,t,n){var r=arguments.length>3&&void 0!==arguments[3]?arguments[3]:null,i=r?O(t):g(t,u(n));return S(n,i,r)}function N(e){var t=e.ownerDocument.defaultView.getComputedStyle(e),n=parseFloat(t.marginTop||0)+parseFloat(t.marginBottom||0),r=parseFloat(t.marginLeft||0)+parseFloat(t.marginRight||0);return{width:e.offsetWidth+r,height:e.offsetHeight+n}}function j(e){var t={left:"right",right:"left",bottom:"top",top:"bottom"};return e.replace(/left|right|bottom|top/g,(function(e){return t[e]}))}function R(e,t,n){n=n.split("-")[0];var r=N(e),i={width:r.width,height:r.height},o=-1!==["right","left"].indexOf(n),a=o?"top":"left",s=o?"left":"top",l=o?"height":"width",u=o?"width":"height";return i[a]=t[a]+t[l]/2-r[l]/2,i[s]=n===s?t[s]-r[u]:t[j(s)],i}function z(e,t){return Array.prototype.find?e.find(t):e.filter(t)[0]}function F(e,t,n){return(void 0===n?e:e.slice(0,function(e,t,n){if(Array.prototype.findIndex)return e.findIndex((function(e){return e[t]===n}));var r=z(e,(function(e){return e[t]===n}));return e.indexOf(r)}(e,"name",n))).forEach((function(e){e.function&&console.warn("`modifier.function` is deprecated, use `modifier.fn`!");var n=e.function||e.fn;e.enabled&&o(n)&&(t.offsets.popper=w(t.offsets.popper),t.offsets.reference=w(t.offsets.reference),t=n(t,e))})),t}function B(){if(!this.state.isDestroyed){var e={instance:this,styles:{},arrowStyles:{},attributes:{},flipped:!1,offsets:{}};e.offsets.reference=P(this.state,this.popper,this.reference,this.options.positionFixed),e.placement=D(this.options.placement,e.offsets.reference,this.popper,this.reference,this.options.modifiers.flip.boundariesElement,this.options.modifiers.flip.padding),e.originalPlacement=e.placement,e.positionFixed=this.options.positionFixed,e.offsets.popper=R(this.popper,e.offsets.reference,e.placement),e.offsets.popper.position=this.options.positionFixed?"fixed":"absolute",e=F(this.modifiers,e),this.state.isCreated?this.options.onUpdate(e):(this.state.isCreated=!0,this.options.onCreate(e))}}function H(e,t){return e.some((function(e){var n=e.name;return e.enabled&&n===t}))}function U(e){for(var t=[!1,"ms","Webkit","Moz","O"],n=e.charAt(0).toUpperCase()+e.slice(1),r=0;r<t.length;r++){var i=t[r],o=i?""+i+n:e;if(void 0!==document.body.style[o])return o}return null}function V(){return this.state.isDestroyed=!0,H(this.modifiers,"applyStyle")&&(this.popper.removeAttribute("x-placement"),this.popper.style.position="",this.popper.style.top="",this.popper.style.left="",this.popper.style.right="",this.popper.style.bottom="",this.popper.style.willChange="",this.popper.style[U("transform")]=""),this.disableEventListeners(),this.options.removeOnDestroy&&this.popper.parentNode.removeChild(this.popper),this}function Y(e){var t=e.ownerDocument;return t?t.defaultView:window}function W(e,t,n,r){n.updateBound=r,Y(e).addEventListener("resize",n.updateBound,{passive:!0});var i=l(e);return function e(t,n,r,i){var o="BODY"===t.nodeName,a=o?t.ownerDocument.defaultView:t;a.addEventListener(n,r,{passive:!0}),o||e(l(a.parentNode),n,r,i),i.push(a)}(i,"scroll",n.updateBound,n.scrollParents),n.scrollElement=i,n.eventsEnabled=!0,n}function G(){this.state.eventsEnabled||(this.state=W(this.reference,this.options,this.state,this.scheduleUpdate))}function X(){var e,t;this.state.eventsEnabled&&(cancelAnimationFrame(this.scheduleUpdate),this.state=(e=this.reference,t=this.state,Y(e).removeEventListener("resize",t.updateBound),t.scrollParents.forEach((function(e){e.removeEventListener("scroll",t.updateBound)})),t.updateBound=null,t.scrollParents=[],t.scrollElement=null,t.eventsEnabled=!1,t))}function q(e){return""!==e&&!isNaN(parseFloat(e))&&isFinite(e)}function Z(e,t){Object.keys(t).forEach((function(n){var r="";-1!==["width","height","top","right","bottom","left"].indexOf(n)&&q(t[n])&&(r="px"),e.style[n]=t[n]+r}))}var Q=n&&/Firefox/i.test(navigator.userAgent);function K(e,t,n){var r=z(e,(function(e){return e.name===t})),i=!!r&&e.some((function(e){return e.name===n&&e.enabled&&e.order<r.order}));if(!i){var o="`"+t+"`",a="`"+n+"`";console.warn(a+" modifier is required by "+o+" modifier in order to work, be sure to include it before "+o+"!")}return i}var J=["auto-start","auto","auto-end","top-start","top","top-end","right-start","right","right-end","bottom-end","bottom","bottom-start","left-end","left","left-start"],$=J.slice(3);function ee(e){var t=arguments.length>1&&void 0!==arguments[1]&&arguments[1],n=$.indexOf(e),r=$.slice(n+1).concat($.slice(0,n));return t?r.reverse():r}var te="flip",ne="clockwise",re="counterclockwise";function ie(e,t,n,r){var i=[0,0],o=-1!==["right","left"].indexOf(r),a=e.split(/(\+|\-)/).map((function(e){return e.trim()})),s=a.indexOf(z(a,(function(e){return-1!==e.search(/,|\s/)})));a[s]&&-1===a[s].indexOf(",")&&console.warn("Offsets separated by white space(s) are deprecated, use a comma (,) instead.");var l=/\s*,\s*|\s+/,u=-1!==s?[a.slice(0,s).concat([a[s].split(l)[0]]),[a[s].split(l)[1]].concat(a.slice(s+1))]:[a];return(u=u.map((function(e,r){var i=(1===r?!o:o)?"height":"width",a=!1;return e.reduce((function(e,t){return""===e[e.length-1]&&-1!==["+","-"].indexOf(t)?(e[e.length-1]=t,a=!0,e):a?(e[e.length-1]+=t,a=!1,e):e.concat(t)}),[]).map((function(e){return function(e,t,n,r){var i=e.match(/((?:\-|\+)?\d*\.?\d*)(.*)/),o=+i[1],a=i[2];if(!o)return e;if(0===a.indexOf("%")){var s=void 0;switch(a){case"%p":s=n;break;case"%":case"%r":default:s=r}return w(s)[t]/100*o}if("vh"===a||"vw"===a){return("vh"===a?Math.max(document.documentElement.clientHeight,window.innerHeight||0):Math.max(document.documentElement.clientWidth,window.innerWidth||0))/100*o}return o}(e,i,t,n)}))}))).forEach((function(e,t){e.forEach((function(n,r){q(n)&&(i[t]+=n*("-"===e[r-1]?-1:1))}))})),i}var oe={placement:"bottom",positionFixed:!1,eventsEnabled:!0,removeOnDestroy:!1,onCreate:function(){},onUpdate:function(){},modifiers:{shift:{order:100,enabled:!0,fn:function(e){var t=e.placement,n=t.split("-")[0],r=t.split("-")[1];if(r){var i=e.offsets,o=i.reference,a=i.popper,s=-1!==["bottom","top"].indexOf(n),l=s?"left":"top",u=s?"width":"height",c={start:I({},l,o[l]),end:I({},l,o[l]+o[u]-a[u])};e.offsets.popper=x({},a,c[r])}return e}},offset:{order:200,enabled:!0,fn:function(e,t){var n=t.offset,r=e.placement,i=e.offsets,o=i.popper,a=i.reference,s=r.split("-")[0],l=void 0;return l=q(+n)?[+n,0]:ie(n,o,a,s),"left"===s?(o.top+=l[0],o.left-=l[1]):"right"===s?(o.top+=l[0],o.left+=l[1]):"top"===s?(o.left+=l[0],o.top-=l[1]):"bottom"===s&&(o.left+=l[0],o.top+=l[1]),e.popper=o,e},offset:0},preventOverflow:{order:300,enabled:!0,fn:function(e,t){var n=t.boundariesElement||f(e.instance.popper);e.instance.reference===n&&(n=f(n));var r=U("transform"),i=e.instance.popper.style,o=i.top,a=i.left,s=i[r];i.top="",i.left="",i[r]="";var l=k(e.instance.popper,e.instance.reference,t.padding,n,e.positionFixed);i.top=o,i.left=a,i[r]=s,t.boundaries=l;var u=t.priority,c=e.offsets.popper,d={primary:function(e){var n=c[e];return c[e]<l[e]&&!t.escapeWithReference&&(n=Math.max(c[e],l[e])),I({},e,n)},secondary:function(e){var n="right"===e?"left":"top",r=c[n];return c[e]>l[e]&&!t.escapeWithReference&&(r=Math.min(c[n],l[e]-("right"===e?c.width:c.height))),I({},n,r)}};return u.forEach((function(e){var t=-1!==["left","top"].indexOf(e)?"primary":"secondary";c=x({},c,d[t](e))})),e.offsets.popper=c,e},priority:["left","right","top","bottom"],padding:5,boundariesElement:"scrollParent"},keepTogether:{order:400,enabled:!0,fn:function(e){var t=e.offsets,n=t.popper,r=t.reference,i=e.placement.split("-")[0],o=Math.floor,a=-1!==["top","bottom"].indexOf(i),s=a?"right":"bottom",l=a?"left":"top",u=a?"width":"height";return n[s]<o(r[l])&&(e.offsets.popper[l]=o(r[l])-n[u]),n[l]>o(r[s])&&(e.offsets.popper[l]=o(r[s])),e}},arrow:{order:500,enabled:!0,fn:function(e,t){var n;if(!K(e.instance.modifiers,"arrow","keepTogether"))return e;var r=t.element;if("string"==typeof r){if(!(r=e.instance.popper.querySelector(r)))return e}else if(!e.instance.popper.contains(r))return console.warn("WARNING: `arrow.element` must be child of its popper element!"),e;var i=e.placement.split("-")[0],o=e.offsets,s=o.popper,l=o.reference,u=-1!==["left","right"].indexOf(i),c=u?"height":"width",d=u?"Top":"Left",h=d.toLowerCase(),f=u?"left":"top",p=u?"bottom":"right",g=N(r)[c];l[p]-g<s[h]&&(e.offsets.popper[h]-=s[h]-(l[p]-g)),l[h]+g>s[p]&&(e.offsets.popper[h]+=l[h]+g-s[p]),e.offsets.popper=w(e.offsets.popper);var v=l[h]+l[c]/2-g/2,m=a(e.instance.popper),y=parseFloat(m["margin"+d]),b=parseFloat(m["border"+d+"Width"]),C=v-e.offsets.popper[h]-y-b;return C=Math.max(Math.min(s[c]-g,C),0),e.arrowElement=r,e.offsets.arrow=(I(n={},h,Math.round(C)),I(n,f,""),n),e},element:"[x-arrow]"},flip:{order:600,enabled:!0,fn:function(e,t){if(H(e.instance.modifiers,"inner"))return e;if(e.flipped&&e.placement===e.originalPlacement)return e;var n=k(e.instance.popper,e.instance.reference,t.padding,t.boundariesElement,e.positionFixed),r=e.placement.split("-")[0],i=j(r),o=e.placement.split("-")[1]||"",a=[];switch(t.behavior){case te:a=[r,i];break;case ne:a=ee(r);break;case re:a=ee(r,!0);break;default:a=t.behavior}return a.forEach((function(s,l){if(r!==s||a.length===l+1)return e;r=e.placement.split("-")[0],i=j(r);var u=e.offsets.popper,c=e.offsets.reference,d=Math.floor,h="left"===r&&d(u.right)>d(c.left)||"right"===r&&d(u.left)<d(c.right)||"top"===r&&d(u.bottom)>d(c.top)||"bottom"===r&&d(u.top)<d(c.bottom),f=d(u.left)<d(n.left),p=d(u.right)>d(n.right),g=d(u.top)<d(n.top),v=d(u.bottom)>d(n.bottom),m="left"===r&&f||"right"===r&&p||"top"===r&&g||"bottom"===r&&v,y=-1!==["top","bottom"].indexOf(r),b=!!t.flipVariations&&(y&&"start"===o&&f||y&&"end"===o&&p||!y&&"start"===o&&g||!y&&"end"===o&&v),C=!!t.flipVariationsByContent&&(y&&"start"===o&&p||y&&"end"===o&&f||!y&&"start"===o&&v||!y&&"end"===o&&g),A=b||C;(h||m||A)&&(e.flipped=!0,(h||m)&&(r=a[l+1]),A&&(o=function(e){return"end"===e?"start":"start"===e?"end":e}(o)),e.placement=r+(o?"-"+o:""),e.offsets.popper=x({},e.offsets.popper,R(e.instance.popper,e.offsets.reference,e.placement)),e=F(e.instance.modifiers,e,"flip"))})),e},behavior:"flip",padding:5,boundariesElement:"viewport",flipVariations:!1,flipVariationsByContent:!1},inner:{order:700,enabled:!1,fn:function(e){var t=e.placement,n=t.split("-")[0],r=e.offsets,i=r.popper,o=r.reference,a=-1!==["left","right"].indexOf(n),s=-1===["top","left"].indexOf(n);return i[a?"left":"top"]=o[n]-(s?i[a?"width":"height"]:0),e.placement=j(t),e.offsets.popper=w(i),e}},hide:{order:800,enabled:!0,fn:function(e){if(!K(e.instance.modifiers,"hide","preventOverflow"))return e;var t=e.offsets.reference,n=z(e.instance.modifiers,(function(e){return"preventOverflow"===e.name})).boundaries;if(t.bottom<n.top||t.left>n.right||t.top>n.bottom||t.right<n.left){if(!0===e.hide)return e;e.hide=!0,e.attributes["x-out-of-boundaries"]=""}else{if(!1===e.hide)return e;e.hide=!1,e.attributes["x-out-of-boundaries"]=!1}return e}},computeStyle:{order:850,enabled:!0,fn:function(e,t){var n=t.x,r=t.y,i=e.offsets.popper,o=z(e.instance.modifiers,(function(e){return"applyStyle"===e.name})).gpuAcceleration;void 0!==o&&console.warn("WARNING: `gpuAcceleration` option moved to `computeStyle` modifier and will not be supported in future versions of Popper.js!");var a=void 0!==o?o:t.gpuAcceleration,s=f(e.instance.popper),l=M(s),u={position:i.position},c=function(e,t){var n=e.offsets,r=n.popper,i=n.reference,o=Math.round,a=Math.floor,s=function(e){return e},l=o(i.width),u=o(r.width),c=-1!==["left","right"].indexOf(e.placement),d=-1!==e.placement.indexOf("-"),h=t?c||d||l%2==u%2?o:a:s,f=t?o:s;return{left:h(l%2==1&&u%2==1&&!d&&t?r.left-1:r.left),top:f(r.top),bottom:f(r.bottom),right:h(r.right)}}(e,window.devicePixelRatio<2||!Q),d="bottom"===n?"top":"bottom",h="right"===r?"left":"right",p=U("transform"),g=void 0,v=void 0;if(v="bottom"===d?"HTML"===s.nodeName?-s.clientHeight+c.bottom:-l.height+c.bottom:c.top,g="right"===h?"HTML"===s.nodeName?-s.clientWidth+c.right:-l.width+c.right:c.left,a&&p)u[p]="translate3d("+g+"px, "+v+"px, 0)",u[d]=0,u[h]=0,u.willChange="transform";else{var m="bottom"===d?-1:1,y="right"===h?-1:1;u[d]=v*m,u[h]=g*y,u.willChange=d+", "+h}var b={"x-placement":e.placement};return e.attributes=x({},b,e.attributes),e.styles=x({},u,e.styles),e.arrowStyles=x({},e.offsets.arrow,e.arrowStyles),e},gpuAcceleration:!0,x:"bottom",y:"right"},applyStyle:{order:900,enabled:!0,fn:function(e){var t,n;return Z(e.instance.popper,e.styles),t=e.instance.popper,n=e.attributes,Object.keys(n).forEach((function(e){!1!==n[e]?t.setAttribute(e,n[e]):t.removeAttribute(e)})),e.arrowElement&&Object.keys(e.arrowStyles).length&&Z(e.arrowElement,e.arrowStyles),e},onLoad:function(e,t,n,r,i){var o=P(i,t,e,n.positionFixed),a=D(n.placement,o,t,e,n.modifiers.flip.boundariesElement,n.modifiers.flip.padding);return t.setAttribute("x-placement",a),Z(t,{position:n.positionFixed?"fixed":"absolute"}),n},gpuAcceleration:void 0}}},ae=function(){function e(t,n){var r=this,a=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};A(this,e),this.scheduleUpdate=function(){return requestAnimationFrame(r.update)},this.update=i(this.update.bind(this)),this.options=x({},e.Defaults,a),this.state={isDestroyed:!1,isCreated:!1,scrollParents:[]},this.reference=t&&t.jquery?t[0]:t,this.popper=n&&n.jquery?n[0]:n,this.options.modifiers={},Object.keys(x({},e.Defaults.modifiers,a.modifiers)).forEach((function(t){r.options.modifiers[t]=x({},e.Defaults.modifiers[t]||{},a.modifiers?a.modifiers[t]:{})})),this.modifiers=Object.keys(this.options.modifiers).map((function(e){return x({name:e},r.options.modifiers[e])})).sort((function(e,t){return e.order-t.order})),this.modifiers.forEach((function(e){e.enabled&&o(e.onLoad)&&e.onLoad(r.reference,r.popper,r.options,e,r.state)})),this.update();var s=this.options.eventsEnabled;s&&this.enableEventListeners(),this.state.eventsEnabled=s}return _(e,[{key:"update",value:function(){return B.call(this)}},{key:"destroy",value:function(){return V.call(this)}},{key:"enableEventListeners",value:function(){return G.call(this)}},{key:"disableEventListeners",value:function(){return X.call(this)}}]),e}();ae.Utils=("undefined"!=typeof window?window:e).PopperUtils,ae.placements=J,ae.Defaults=oe,t.a=ae}).call(this,n(61))},function(e,t,n){"use strict";(function(e,r){var i,o=n(1289);i="undefined"!=typeof self?self:"undefined"!=typeof window?window:void 0!==e?e:r;var a=Object(o.a)(i);t.a=a}).call(this,n(61),n(1578)(e))},function(e,t,n){(function(e,n){var r="[object Arguments]",i="[object Map]",o="[object Object]",a="[object Set]",s=/^\[object .+?Constructor\]$/,l=/^(?:0|[1-9]\d*)$/,u={};u["[object Float32Array]"]=u["[object Float64Array]"]=u["[object Int8Array]"]=u["[object Int16Array]"]=u["[object Int32Array]"]=u["[object Uint8Array]"]=u["[object Uint8ClampedArray]"]=u["[object Uint16Array]"]=u["[object Uint32Array]"]=!0,u[r]=u["[object Array]"]=u["[object ArrayBuffer]"]=u["[object Boolean]"]=u["[object DataView]"]=u["[object Date]"]=u["[object Error]"]=u["[object Function]"]=u[i]=u["[object Number]"]=u[o]=u["[object RegExp]"]=u[a]=u["[object String]"]=u["[object WeakMap]"]=!1;var c="object"==typeof e&&e&&e.Object===Object&&e,d="object"==typeof self&&self&&self.Object===Object&&self,h=c||d||Function("return this")(),f=t&&!t.nodeType&&t,p=f&&"object"==typeof n&&n&&!n.nodeType&&n,g=p&&p.exports===f,v=g&&c.process,m=function(){try{return v&&v.binding&&v.binding("util")}catch(e){}}(),y=m&&m.isTypedArray;function b(e,t){for(var n=-1,r=null==e?0:e.length;++n<r;)if(t(e[n],n,e))return!0;return!1}function C(e){var t=-1,n=Array(e.size);return e.forEach((function(e,r){n[++t]=[r,e]})),n}function A(e){var t=-1,n=Array(e.size);return e.forEach((function(e){n[++t]=e})),n}var _,I,x,w=Array.prototype,M=Function.prototype,S=Object.prototype,T=h["__core-js_shared__"],E=M.toString,O=S.hasOwnProperty,k=(_=/[^.]+$/.exec(T&&T.keys&&T.keys.IE_PROTO||""))?"Symbol(src)_1."+_:"",L=S.toString,D=RegExp("^"+E.call(O).replace(/[\\^$.*+?()[\]{}|]/g,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$"),P=g?h.Buffer:void 0,N=h.Symbol,j=h.Uint8Array,R=S.propertyIsEnumerable,z=w.splice,F=N?N.toStringTag:void 0,B=Object.getOwnPropertySymbols,H=P?P.isBuffer:void 0,U=(I=Object.keys,x=Object,function(e){return I(x(e))}),V=me(h,"DataView"),Y=me(h,"Map"),W=me(h,"Promise"),G=me(h,"Set"),X=me(h,"WeakMap"),q=me(Object,"create"),Z=Ae(V),Q=Ae(Y),K=Ae(W),J=Ae(G),$=Ae(X),ee=N?N.prototype:void 0,te=ee?ee.valueOf:void 0;function ne(e){var t=-1,n=null==e?0:e.length;for(this.clear();++t<n;){var r=e[t];this.set(r[0],r[1])}}function re(e){var t=-1,n=null==e?0:e.length;for(this.clear();++t<n;){var r=e[t];this.set(r[0],r[1])}}function ie(e){var t=-1,n=null==e?0:e.length;for(this.clear();++t<n;){var r=e[t];this.set(r[0],r[1])}}function oe(e){var t=-1,n=null==e?0:e.length;for(this.__data__=new ie;++t<n;)this.add(e[t])}function ae(e){var t=this.__data__=new re(e);this.size=t.size}function se(e,t){var n=xe(e),r=!n&&Ie(e),i=!n&&!r&&we(e),o=!n&&!r&&!i&&Oe(e),a=n||r||i||o,s=a?function(e,t){for(var n=-1,r=Array(e);++n<e;)r[n]=t(n);return r}(e.length,String):[],l=s.length;for(var u in e)!t&&!O.call(e,u)||a&&("length"==u||i&&("offset"==u||"parent"==u)||o&&("buffer"==u||"byteLength"==u||"byteOffset"==u)||Ce(u,l))||s.push(u);return s}function le(e,t){for(var n=e.length;n--;)if(_e(e[n][0],t))return n;return-1}function ue(e){return null==e?void 0===e?"[object Undefined]":"[object Null]":F&&F in Object(e)?function(e){var t=O.call(e,F),n=e[F];try{e[F]=void 0;var r=!0}catch(e){}var i=L.call(e);r&&(t?e[F]=n:delete e[F]);return i}(e):function(e){return L.call(e)}(e)}function ce(e){return Ee(e)&&ue(e)==r}function de(e,t,n,s,l){return e===t||(null==e||null==t||!Ee(e)&&!Ee(t)?e!=e&&t!=t:function(e,t,n,s,l,u){var c=xe(e),d=xe(t),h=c?"[object Array]":be(e),f=d?"[object Array]":be(t),p=(h=h==r?o:h)==o,g=(f=f==r?o:f)==o,v=h==f;if(v&&we(e)){if(!we(t))return!1;c=!0,p=!1}if(v&&!p)return u||(u=new ae),c||Oe(e)?pe(e,t,n,s,l,u):function(e,t,n,r,o,s,l){switch(n){case"[object DataView]":if(e.byteLength!=t.byteLength||e.byteOffset!=t.byteOffset)return!1;e=e.buffer,t=t.buffer;case"[object ArrayBuffer]":return!(e.byteLength!=t.byteLength||!s(new j(e),new j(t)));case"[object Boolean]":case"[object Date]":case"[object Number]":return _e(+e,+t);case"[object Error]":return e.name==t.name&&e.message==t.message;case"[object RegExp]":case"[object String]":return e==t+"";case i:var u=C;case a:var c=1&r;if(u||(u=A),e.size!=t.size&&!c)return!1;var d=l.get(e);if(d)return d==t;r|=2,l.set(e,t);var h=pe(u(e),u(t),r,o,s,l);return l.delete(e),h;case"[object Symbol]":if(te)return te.call(e)==te.call(t)}return!1}(e,t,h,n,s,l,u);if(!(1&n)){var m=p&&O.call(e,"__wrapped__"),y=g&&O.call(t,"__wrapped__");if(m||y){var b=m?e.value():e,_=y?t.value():t;return u||(u=new ae),l(b,_,n,s,u)}}if(!v)return!1;return u||(u=new ae),function(e,t,n,r,i,o){var a=1&n,s=ge(e),l=s.length,u=ge(t).length;if(l!=u&&!a)return!1;var c=l;for(;c--;){var d=s[c];if(!(a?d in t:O.call(t,d)))return!1}var h=o.get(e);if(h&&o.get(t))return h==t;var f=!0;o.set(e,t),o.set(t,e);var p=a;for(;++c<l;){d=s[c];var g=e[d],v=t[d];if(r)var m=a?r(v,g,d,t,e,o):r(g,v,d,e,t,o);if(!(void 0===m?g===v||i(g,v,n,r,o):m)){f=!1;break}p||(p="constructor"==d)}if(f&&!p){var y=e.constructor,b=t.constructor;y==b||!("constructor"in e)||!("constructor"in t)||"function"==typeof y&&y instanceof y&&"function"==typeof b&&b instanceof b||(f=!1)}return o.delete(e),o.delete(t),f}(e,t,n,s,l,u)}(e,t,n,s,de,l))}function he(e){return!(!Te(e)||function(e){return!!k&&k in e}(e))&&(Me(e)?D:s).test(Ae(e))}function fe(e){if(n=(t=e)&&t.constructor,r="function"==typeof n&&n.prototype||S,t!==r)return U(e);var t,n,r,i=[];for(var o in Object(e))O.call(e,o)&&"constructor"!=o&&i.push(o);return i}function pe(e,t,n,r,i,o){var a=1&n,s=e.length,l=t.length;if(s!=l&&!(a&&l>s))return!1;var u=o.get(e);if(u&&o.get(t))return u==t;var c=-1,d=!0,h=2&n?new oe:void 0;for(o.set(e,t),o.set(t,e);++c<s;){var f=e[c],p=t[c];if(r)var g=a?r(p,f,c,t,e,o):r(f,p,c,e,t,o);if(void 0!==g){if(g)continue;d=!1;break}if(h){if(!b(t,(function(e,t){if(a=t,!h.has(a)&&(f===e||i(f,e,n,r,o)))return h.push(t);var a}))){d=!1;break}}else if(f!==p&&!i(f,p,n,r,o)){d=!1;break}}return o.delete(e),o.delete(t),d}function ge(e){return function(e,t,n){var r=t(e);return xe(e)?r:function(e,t){for(var n=-1,r=t.length,i=e.length;++n<r;)e[i+n]=t[n];return e}(r,n(e))}(e,ke,ye)}function ve(e,t){var n,r,i=e.__data__;return("string"==(r=typeof(n=t))||"number"==r||"symbol"==r||"boolean"==r?"__proto__"!==n:null===n)?i["string"==typeof t?"string":"hash"]:i.map}function me(e,t){var n=function(e,t){return null==e?void 0:e[t]}(e,t);return he(n)?n:void 0}ne.prototype.clear=function(){this.__data__=q?q(null):{},this.size=0},ne.prototype.delete=function(e){var t=this.has(e)&&delete this.__data__[e];return this.size-=t?1:0,t},ne.prototype.get=function(e){var t=this.__data__;if(q){var n=t[e];return"__lodash_hash_undefined__"===n?void 0:n}return O.call(t,e)?t[e]:void 0},ne.prototype.has=function(e){var t=this.__data__;return q?void 0!==t[e]:O.call(t,e)},ne.prototype.set=function(e,t){var n=this.__data__;return this.size+=this.has(e)?0:1,n[e]=q&&void 0===t?"__lodash_hash_undefined__":t,this},re.prototype.clear=function(){this.__data__=[],this.size=0},re.prototype.delete=function(e){var t=this.__data__,n=le(t,e);return!(n<0)&&(n==t.length-1?t.pop():z.call(t,n,1),--this.size,!0)},re.prototype.get=function(e){var t=this.__data__,n=le(t,e);return n<0?void 0:t[n][1]},re.prototype.has=function(e){return le(this.__data__,e)>-1},re.prototype.set=function(e,t){var n=this.__data__,r=le(n,e);return r<0?(++this.size,n.push([e,t])):n[r][1]=t,this},ie.prototype.clear=function(){this.size=0,this.__data__={hash:new ne,map:new(Y||re),string:new ne}},ie.prototype.delete=function(e){var t=ve(this,e).delete(e);return this.size-=t?1:0,t},ie.prototype.get=function(e){return ve(this,e).get(e)},ie.prototype.has=function(e){return ve(this,e).has(e)},ie.prototype.set=function(e,t){var n=ve(this,e),r=n.size;return n.set(e,t),this.size+=n.size==r?0:1,this},oe.prototype.add=oe.prototype.push=function(e){return this.__data__.set(e,"__lodash_hash_undefined__"),this},oe.prototype.has=function(e){return this.__data__.has(e)},ae.prototype.clear=function(){this.__data__=new re,this.size=0},ae.prototype.delete=function(e){var t=this.__data__,n=t.delete(e);return this.size=t.size,n},ae.prototype.get=function(e){return this.__data__.get(e)},ae.prototype.has=function(e){return this.__data__.has(e)},ae.prototype.set=function(e,t){var n=this.__data__;if(n instanceof re){var r=n.__data__;if(!Y||r.length<199)return r.push([e,t]),this.size=++n.size,this;n=this.__data__=new ie(r)}return n.set(e,t),this.size=n.size,this};var ye=B?function(e){return null==e?[]:(e=Object(e),function(e,t){for(var n=-1,r=null==e?0:e.length,i=0,o=[];++n<r;){var a=e[n];t(a,n,e)&&(o[i++]=a)}return o}(B(e),(function(t){return R.call(e,t)})))}:function(){return[]},be=ue;function Ce(e,t){return!!(t=null==t?9007199254740991:t)&&("number"==typeof e||l.test(e))&&e>-1&&e%1==0&&e<t}function Ae(e){if(null!=e){try{return E.call(e)}catch(e){}try{return e+""}catch(e){}}return""}function _e(e,t){return e===t||e!=e&&t!=t}(V&&"[object DataView]"!=be(new V(new ArrayBuffer(1)))||Y&&be(new Y)!=i||W&&"[object Promise]"!=be(W.resolve())||G&&be(new G)!=a||X&&"[object WeakMap]"!=be(new X))&&(be=function(e){var t=ue(e),n=t==o?e.constructor:void 0,r=n?Ae(n):"";if(r)switch(r){case Z:return"[object DataView]";case Q:return i;case K:return"[object Promise]";case J:return a;case $:return"[object WeakMap]"}return t});var Ie=ce(function(){return arguments}())?ce:function(e){return Ee(e)&&O.call(e,"callee")&&!R.call(e,"callee")},xe=Array.isArray;var we=H||function(){return!1};function Me(e){if(!Te(e))return!1;var t=ue(e);return"[object Function]"==t||"[object GeneratorFunction]"==t||"[object AsyncFunction]"==t||"[object Proxy]"==t}function Se(e){return"number"==typeof e&&e>-1&&e%1==0&&e<=9007199254740991}function Te(e){var t=typeof e;return null!=e&&("object"==t||"function"==t)}function Ee(e){return null!=e&&"object"==typeof e}var Oe=y?function(e){return function(t){return e(t)}}(y):function(e){return Ee(e)&&Se(e.length)&&!!u[ue(e)]};function ke(e){return null!=(t=e)&&Se(t.length)&&!Me(t)?se(e):fe(e);var t}n.exports=function(e,t){return de(e,t)}}).call(this,n(61),n(188)(e))},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){(0,o.default)(2,arguments);var n=(0,i.default)(e),a=(0,r.default)(t);if(isNaN(a))return new Date(NaN);if(!a)return n;return n.setDate(n.getDate()+a),n};var r=a(n(77)),i=a(n(47)),o=a(n(40));function a(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){(0,i.default)(1,arguments);var t=(0,r.default)(e);return t.setHours(23,59,59,999),t};var r=o(n(47)),i=o(n(40));function o(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){(0,i.default)(1,arguments);var t=(0,r.default)(e);return t.setDate(1),t.setHours(0,0,0,0),t};var r=o(n(47)),i=o(n(40));function o(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){(0,i.default)(1,arguments);var t=(0,r.default)(e),n=t.getMonth();return t.setFullYear(t.getFullYear(),n+1,0),t.setHours(23,59,59,999),t};var r=o(n(47)),i=o(n(40));function o(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){(0,i.default)(1,arguments);var t=(0,r.default)(e),n=t.getFullYear();return t.setFullYear(n+1,0,0),t.setHours(23,59,59,999),t};var r=o(n(47)),i=o(n(40));function o(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){(0,o.default)(1,arguments);var n=t||{},a=n.locale,s=a&&a.options&&a.options.weekStartsOn,l=null==s?0:(0,i.default)(s),u=null==n.weekStartsOn?l:(0,i.default)(n.weekStartsOn);if(!(u>=0&&u<=6))throw new RangeError("weekStartsOn must be between 0 and 6 inclusively");var c=(0,r.default)(e),d=c.getDay(),h=(d<u?7:0)+d-u;return c.setDate(c.getDate()-h),c.setHours(0,0,0,0),c};var r=a(n(47)),i=a(n(77)),o=a(n(40));function a(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){(0,o.default)(1,arguments);var n=t||{},a=n.locale,s=a&&a.options&&a.options.weekStartsOn,l=null==s?0:(0,i.default)(s),u=null==n.weekStartsOn?l:(0,i.default)(n.weekStartsOn);if(!(u>=0&&u<=6))throw new RangeError("weekStartsOn must be between 0 and 6 inclusively");var c=(0,r.default)(e),d=c.getDay(),h=6+(d<u?-7:0)-(d-u);return c.setDate(c.getDate()+h),c.setHours(23,59,59,999),c};var r=a(n(47)),i=a(n(77)),o=a(n(40));function a(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";var r=n(0),i=n(118),o="dnd-core/INIT_COORDS",a="dnd-core/BEGIN_DRAG",s="dnd-core/PUBLISH_DRAG_SOURCE",l="dnd-core/HOVER",u="dnd-core/DROP",c="dnd-core/END_DRAG",d=function(e,t){return e===t};function h(e,t){return!e&&!t||!(!e||!t)&&(e.x===t.x&&e.y===t.y)}function f(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:d;if(e.length!==t.length)return!1;for(var r=0;r<e.length;++r)if(!n(e[r],t[r]))return!1;return!0}function p(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),n.push.apply(n,r)}return n}function g(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?p(Object(n),!0).forEach((function(t){v(e,t,n[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):p(Object(n)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))}))}return e}function v(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}var m={initialSourceClientOffset:null,initialClientOffset:null,clientOffset:null};function y(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:m,t=arguments.length>1?arguments[1]:void 0,n=t.payload;switch(t.type){case o:case a:return{initialSourceClientOffset:n.sourceClientOffset,initialClientOffset:n.clientOffset,clientOffset:n.clientOffset};case l:return h(e.clientOffset,n.clientOffset)?e:g({},e,{clientOffset:n.clientOffset});case c:case u:return m;default:return e}}var b="dnd-core/ADD_SOURCE",C="dnd-core/ADD_TARGET",A="dnd-core/REMOVE_SOURCE",_="dnd-core/REMOVE_TARGET";function I(e){return(I="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function x(e,t,n){return t.split(".").reduce((function(e,t){return e&&e[t]?e[t]:n||null}),e)}function w(e,t){return e.filter((function(e){return e!==t}))}function M(e){return"object"===I(e)}function S(e,t){var n=new Map,r=function(e){return n.set(e,n.has(e)?n.get(e)+1:1)};e.forEach(r),t.forEach(r);var i=[];return n.forEach((function(e,t){1===e&&i.push(t)})),i}function T(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),n.push.apply(n,r)}return n}function E(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?T(Object(n),!0).forEach((function(t){O(e,t,n[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):T(Object(n)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))}))}return e}function O(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}var k={itemType:null,item:null,sourceId:null,targetIds:[],dropResult:null,didDrop:!1,isSourcePublic:null};function L(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:k,t=arguments.length>1?arguments[1]:void 0,n=t.payload;switch(t.type){case a:return E({},e,{itemType:n.itemType,item:n.item,sourceId:n.sourceId,isSourcePublic:n.isSourcePublic,dropResult:null,didDrop:!1});case s:return E({},e,{isSourcePublic:!0});case l:return E({},e,{targetIds:n.targetIds});case _:return-1===e.targetIds.indexOf(n.targetId)?e:E({},e,{targetIds:w(e.targetIds,n.targetId)});case u:return E({},e,{dropResult:n.dropResult,didDrop:!0,targetIds:[]});case c:return E({},e,{itemType:null,item:null,sourceId:null,dropResult:null,didDrop:!1,isSourcePublic:null,targetIds:[]});default:return e}}function D(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:0,t=arguments.length>1?arguments[1]:void 0;switch(t.type){case b:case C:return e+1;case A:case _:return e-1;default:return e}}var P=[],N=[];function j(e,t){return e!==P&&(e===N||void 0===t||(n=e,t.filter((function(e){return n.indexOf(e)>-1}))).length>0);var n}function R(){var e=arguments.length>1?arguments[1]:void 0;switch(e.type){case l:break;case b:case C:case _:case A:return P;case a:case s:case c:case u:default:return N}var t=e.payload,n=t.targetIds,r=void 0===n?[]:n,i=t.prevTargetIds,o=void 0===i?[]:i,d=S(r,o),h=d.length>0||!f(r,o);if(!h)return P;var p=o[o.length-1],g=r[r.length-1];return p!==g&&(p&&d.push(p),g&&d.push(g)),d}function z(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:0;return e+1}function F(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),n.push.apply(n,r)}return n}function B(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?F(Object(n),!0).forEach((function(t){H(e,t,n[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):F(Object(n)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))}))}return e}function H(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function U(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},t=arguments.length>1?arguments[1]:void 0;return{dirtyHandlerIds:R(e.dirtyHandlerIds,{type:t.type,payload:B({},t.payload,{prevTargetIds:x(e,"dragOperation.targetIds",[])})}),dragOffset:y(e.dragOffset,t),refCount:D(e.refCount,t),dragOperation:L(e.dragOperation,t),stateId:z(e.stateId)}}P.__IS_NONE__=!0,N.__IS_ALL__=!0;var V=n(19),Y=n.n(V);function W(e,t){return{type:o,payload:{sourceClientOffset:t||null,clientOffset:e||null}}}var G={type:o,payload:{clientOffset:null,sourceClientOffset:null}};function X(e){return function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:[],n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{publishSource:!0},r=n.publishSource,i=void 0===r||r,o=n.clientOffset,s=n.getSourceClientOffset,l=e.getMonitor(),u=e.getRegistry();e.dispatch(W(o)),q(t,l,u);var c=K(t,l);if(null!==c){var d=null;o&&(Z(s),d=s(c)),e.dispatch(W(o,d));var h=u.getSource(c),f=h.beginDrag(l,c);Q(f),u.pinSource(c);var p=u.getSourceType(c);return{type:a,payload:{itemType:p,item:f,sourceId:c,clientOffset:o||null,sourceClientOffset:d||null,isSourcePublic:!!i}}}e.dispatch(G)}}function q(e,t,n){Y()(!t.isDragging(),"Cannot call beginDrag while dragging."),e.forEach((function(e){Y()(n.getSource(e),"Expected sourceIds to be registered.")}))}function Z(e){Y()("function"==typeof e,"When clientOffset is provided, getSourceClientOffset must be a function.")}function Q(e){Y()(M(e),"Item must be an object.")}function K(e,t){for(var n=null,r=e.length-1;r>=0;r--)if(t.canDragSource(e[r])){n=e[r];break}return n}function J(e){return function(){if(e.getMonitor().isDragging())return{type:s}}}function $(e,t){return null===t?null===e:Array.isArray(e)?e.some((function(e){return e===t})):e===t}function ee(e){return function(t){var n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},r=n.clientOffset;te(t);var i=t.slice(0),o=e.getMonitor(),a=e.getRegistry();ne(i,o,a);var s=o.getItemType();return re(i,a,s),ie(i,o,a),{type:l,payload:{targetIds:i,clientOffset:r||null}}}}function te(e){Y()(Array.isArray(e),"Expected targetIds to be an array.")}function ne(e,t,n){Y()(t.isDragging(),"Cannot call hover while not dragging."),Y()(!t.didDrop(),"Cannot call hover after drop.");for(var r=0;r<e.length;r++){var i=e[r];Y()(e.lastIndexOf(i)===r,"Expected targetIds to be unique in the passed array.");var o=n.getTarget(i);Y()(o,"Expected targetIds to be registered.")}}function re(e,t,n){for(var r=e.length-1;r>=0;r--){var i=e[r];$(t.getTargetType(i),n)||e.splice(r,1)}}function ie(e,t,n){e.forEach((function(e){n.getTarget(e).hover(t,e)}))}function oe(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),n.push.apply(n,r)}return n}function ae(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?oe(Object(n),!0).forEach((function(t){se(e,t,n[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):oe(Object(n)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))}))}return e}function se(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function le(e){return function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},n=e.getMonitor(),r=e.getRegistry();ue(n);var i=de(n);i.forEach((function(i,o){var a=ce(i,o,r,n),s={type:u,payload:{dropResult:ae({},t,{},a)}};e.dispatch(s)}))}}function ue(e){Y()(e.isDragging(),"Cannot call drop while not dragging."),Y()(!e.didDrop(),"Cannot call drop twice during one drag operation.")}function ce(e,t,n,r){var i=n.getTarget(e),o=i?i.drop(r,e):void 0;return function(e){Y()(void 0===e||M(e),"Drop result must either be an object or undefined.")}(o),void 0===o&&(o=0===t?{}:r.getDropResult()),o}function de(e){var t=e.getTargetIds().filter(e.canDropOnTarget,e);return t.reverse(),t}function he(e){return function(){var t=e.getMonitor(),n=e.getRegistry();!function(e){Y()(e.isDragging(),"Cannot call endDrag while not dragging.")}(t);var r=t.getSourceId();return n.getSource(r,!0).endDrag(t,r),n.unpinSource(),{type:c}}}function fe(e,t){return{x:e.x-t.x,y:e.y-t.y}}function pe(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}var ge,ve=function(){function e(t,n){!function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,e),this.store=t,this.registry=n}var t,n,r;return t=e,(n=[{key:"subscribeToStateChange",value:function(e){var t=this,n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{handlerIds:void 0},r=n.handlerIds;Y()("function"==typeof e,"listener must be a function."),Y()(void 0===r||Array.isArray(r),"handlerIds, when specified, must be an array of strings.");var i=this.store.getState().stateId,o=function(){var n=t.store.getState(),o=n.stateId;try{o===i||o===i+1&&!j(n.dirtyHandlerIds,r)||e()}finally{i=o}};return this.store.subscribe(o)}},{key:"subscribeToOffsetChange",value:function(e){var t=this;Y()("function"==typeof e,"listener must be a function.");var n=this.store.getState().dragOffset;return this.store.subscribe((function(){var r=t.store.getState().dragOffset;r!==n&&(n=r,e())}))}},{key:"canDragSource",value:function(e){if(!e)return!1;var t=this.registry.getSource(e);return Y()(t,"Expected to find a valid source."),!this.isDragging()&&t.canDrag(this,e)}},{key:"canDropOnTarget",value:function(e){if(!e)return!1;var t=this.registry.getTarget(e);return Y()(t,"Expected to find a valid target."),!(!this.isDragging()||this.didDrop())&&$(this.registry.getTargetType(e),this.getItemType())&&t.canDrop(this,e)}},{key:"isDragging",value:function(){return Boolean(this.getItemType())}},{key:"isDraggingSource",value:function(e){if(!e)return!1;var t=this.registry.getSource(e,!0);return Y()(t,"Expected to find a valid source."),!(!this.isDragging()||!this.isSourcePublic())&&this.registry.getSourceType(e)===this.getItemType()&&t.isDragging(this,e)}},{key:"isOverTarget",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{shallow:!1};if(!e)return!1;var n=t.shallow;if(!this.isDragging())return!1;var r=this.registry.getTargetType(e),i=this.getItemType();if(i&&!$(r,i))return!1;var o=this.getTargetIds();if(!o.length)return!1;var a=o.indexOf(e);return n?a===o.length-1:a>-1}},{key:"getItemType",value:function(){return this.store.getState().dragOperation.itemType}},{key:"getItem",value:function(){return this.store.getState().dragOperation.item}},{key:"getSourceId",value:function(){return this.store.getState().dragOperation.sourceId}},{key:"getTargetIds",value:function(){return this.store.getState().dragOperation.targetIds}},{key:"getDropResult",value:function(){return this.store.getState().dragOperation.dropResult}},{key:"didDrop",value:function(){return this.store.getState().dragOperation.didDrop}},{key:"isSourcePublic",value:function(){return this.store.getState().dragOperation.isSourcePublic}},{key:"getInitialClientOffset",value:function(){return this.store.getState().dragOffset.initialClientOffset}},{key:"getInitialSourceClientOffset",value:function(){return this.store.getState().dragOffset.initialSourceClientOffset}},{key:"getClientOffset",value:function(){return this.store.getState().dragOffset.clientOffset}},{key:"getSourceClientOffset",value:function(){return e=this.store.getState().dragOffset,r=e.clientOffset,i=e.initialClientOffset,o=e.initialSourceClientOffset,r&&i&&o?fe((n=o,{x:(t=r).x+n.x,y:t.y+n.y}),i):null;var e,t,n,r,i,o}},{key:"getDifferenceFromInitialOffset",value:function(){return e=this.store.getState().dragOffset,t=e.clientOffset,n=e.initialClientOffset,t&&n?fe(t,n):null;var e,t,n}}])&&pe(t.prototype,n),r&&pe(t,r),e}(),me=n(635),ye=n.n(me),be=0;function Ce(e){return(Ce="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function Ae(e,t){t&&Array.isArray(e)?e.forEach((function(e){return Ae(e,!1)})):Y()("string"==typeof e||"symbol"===Ce(e),t?"Type can only be a string, a symbol, or an array of either.":"Type can only be a string or a symbol.")}function _e(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}function Ie(e,t){return function(e){if(Array.isArray(e))return e}(e)||function(e,t){if(!(Symbol.iterator in Object(e))&&"[object Arguments]"!==Object.prototype.toString.call(e))return;var n=[],r=!0,i=!1,o=void 0;try{for(var a,s=e[Symbol.iterator]();!(r=(a=s.next()).done)&&(n.push(a.value),!t||n.length!==t);r=!0);}catch(e){i=!0,o=e}finally{try{r||null==s.return||s.return()}finally{if(i)throw o}}return n}(e,t)||function(){throw new TypeError("Invalid attempt to destructure non-iterable instance")}()}function xe(e){var t=(be++).toString();switch(e){case ge.SOURCE:return"S".concat(t);case ge.TARGET:return"T".concat(t);default:throw new Error("Unknown Handler Role: ".concat(e))}}function we(e){switch(e[0]){case"S":return ge.SOURCE;case"T":return ge.TARGET;default:Y()(!1,"Cannot parse handler ID: ".concat(e))}}function Me(e,t){var n=e.entries(),r=!1;do{var i=n.next(),o=i.done;if(Ie(i.value,2)[1]===t)return!0;r=!!o}while(!r);return!1}!function(e){e.SOURCE="SOURCE",e.TARGET="TARGET"}(ge||(ge={}));var Se=function(){function e(t){!function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,e),this.types=new Map,this.dragSources=new Map,this.dropTargets=new Map,this.pinnedSourceId=null,this.pinnedSource=null,this.store=t}var t,n,r;return t=e,(n=[{key:"addSource",value:function(e,t){Ae(e),function(e){Y()("function"==typeof e.canDrag,"Expected canDrag to be a function."),Y()("function"==typeof e.beginDrag,"Expected beginDrag to be a function."),Y()("function"==typeof e.endDrag,"Expected endDrag to be a function.")}(t);var n=this.addHandler(ge.SOURCE,e,t);return this.store.dispatch(function(e){return{type:b,payload:{sourceId:e}}}(n)),n}},{key:"addTarget",value:function(e,t){Ae(e,!0),function(e){Y()("function"==typeof e.canDrop,"Expected canDrop to be a function."),Y()("function"==typeof e.hover,"Expected hover to be a function."),Y()("function"==typeof e.drop,"Expected beginDrag to be a function.")}(t);var n=this.addHandler(ge.TARGET,e,t);return this.store.dispatch(function(e){return{type:C,payload:{targetId:e}}}(n)),n}},{key:"containsHandler",value:function(e){return Me(this.dragSources,e)||Me(this.dropTargets,e)}},{key:"getSource",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]&&arguments[1];Y()(this.isSourceId(e),"Expected a valid source ID.");var n=t&&e===this.pinnedSourceId,r=n?this.pinnedSource:this.dragSources.get(e);return r}},{key:"getTarget",value:function(e){return Y()(this.isTargetId(e),"Expected a valid target ID."),this.dropTargets.get(e)}},{key:"getSourceType",value:function(e){return Y()(this.isSourceId(e),"Expected a valid source ID."),this.types.get(e)}},{key:"getTargetType",value:function(e){return Y()(this.isTargetId(e),"Expected a valid target ID."),this.types.get(e)}},{key:"isSourceId",value:function(e){return we(e)===ge.SOURCE}},{key:"isTargetId",value:function(e){return we(e)===ge.TARGET}},{key:"removeSource",value:function(e){var t=this;Y()(this.getSource(e),"Expected an existing source."),this.store.dispatch(function(e){return{type:A,payload:{sourceId:e}}}(e)),ye()((function(){t.dragSources.delete(e),t.types.delete(e)}))}},{key:"removeTarget",value:function(e){Y()(this.getTarget(e),"Expected an existing target."),this.store.dispatch(function(e){return{type:_,payload:{targetId:e}}}(e)),this.dropTargets.delete(e),this.types.delete(e)}},{key:"pinSource",value:function(e){var t=this.getSource(e);Y()(t,"Expected an existing source."),this.pinnedSourceId=e,this.pinnedSource=t}},{key:"unpinSource",value:function(){Y()(this.pinnedSource,"No source is pinned at the time."),this.pinnedSourceId=null,this.pinnedSource=null}},{key:"addHandler",value:function(e,t,n){var r=xe(e);return this.types.set(r,t),e===ge.SOURCE?this.dragSources.set(r,n):e===ge.TARGET&&this.dropTargets.set(r,n),r}}])&&_e(t.prototype,n),r&&_e(t,r),e}();function Te(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function Ee(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}function Oe(e){var t="undefined"!=typeof window&&window.__REDUX_DEVTOOLS_EXTENSION__;return Object(i.createStore)(U,e&&t&&t({name:"dnd-core",instanceId:"dnd-core"}))}var ke=function(){function e(){var t=this,n=arguments.length>0&&void 0!==arguments[0]&&arguments[0];Te(this,e),this.isSetUp=!1,this.handleRefCountChange=function(){var e=t.store.getState().refCount>0;t.backend&&(e&&!t.isSetUp?(t.backend.setup(),t.isSetUp=!0):!e&&t.isSetUp&&(t.backend.teardown(),t.isSetUp=!1))};var r=Oe(n);this.store=r,this.monitor=new ve(r,new Se(r)),r.subscribe(this.handleRefCountChange)}var t,n,r;return t=e,(n=[{key:"receiveBackend",value:function(e){this.backend=e}},{key:"getMonitor",value:function(){return this.monitor}},{key:"getBackend",value:function(){return this.backend}},{key:"getRegistry",value:function(){return this.monitor.registry}},{key:"getActions",value:function(){var e=this,t=this.store.dispatch,n=function(e){return{beginDrag:X(e),publishDragSource:J(e),hover:ee(e),drop:le(e),endDrag:he(e)}}(this);return Object.keys(n).reduce((function(r,i){var o,a=n[i];return r[i]=(o=a,function(){for(var n=arguments.length,r=new Array(n),i=0;i<n;i++)r[i]=arguments[i];var a=o.apply(e,r);void 0!==a&&t(a)}),r}),{})}},{key:"dispatch",value:function(e){this.store.dispatch(e)}}])&&Ee(t.prototype,n),r&&Ee(t,r),e}();function Le(e,t,n,r){var i=new ke(r),o=e(i,t,n);return i.receiveBackend(o),i}n.d(t,"a",(function(){return De})),n.d(t,"b",(function(){return Pe}));var De=r.createContext({dragDropManager:void 0});function Pe(e,t,n,r){return{dragDropManager:Le(e,t,n,r)}}},function(e,t,n){"use strict";function r(e){var t=e.theme,n=e.name,r=e.props;if(!t||!t.props||!t.props[n])return r;var i,o=t.props[n];for(i in o)void 0===r[i]&&(r[i]=o[i]);return r}n.d(t,"a",(function(){return r}))},function(e,t,n){"use strict";var r=n(5),i=n(1),o=n(0),a=n.n(o),s=n(89),l=n(2373),u={set:function(e,t,n,r){var i=e.get(t);i||(i=new Map,e.set(t,i)),i.set(n,r)},get:function(e,t,n){var r=e.get(t);return r?r.get(n):void 0},delete:function(e,t,n){e.get(t).delete(n)}},c=n(277),d=n(318),h=-1e9;function f(){return h+=1}n(140);var p=n(442);function g(e){var t="function"==typeof e;return{create:function(n,r){var o;try{o=t?e(n):e}catch(e){throw e}if(!r||!n.overrides||!n.overrides[r])return o;var a=n.overrides[r],s=Object(i.a)({},o);return Object.keys(a).forEach((function(e){s[e]=Object(p.a)(s[e],a[e])})),s},options:{}}}var v={};function m(e,t,n){var r=e.state;if(e.stylesOptions.disableGeneration)return t||{};r.cacheClasses||(r.cacheClasses={value:null,lastProp:null,lastJSS:{}});var i=!1;return r.classes!==r.cacheClasses.lastJSS&&(r.cacheClasses.lastJSS=r.classes,i=!0),t!==r.cacheClasses.lastProp&&(r.cacheClasses.lastProp=t,i=!0),i&&(r.cacheClasses.value=Object(l.a)({baseClasses:r.cacheClasses.lastJSS,newClasses:t,Component:n})),r.cacheClasses.value}function y(e,t){var n=e.state,r=e.theme,o=e.stylesOptions,a=e.stylesCreator,c=e.name;if(!o.disableGeneration){var d=u.get(o.sheetsManager,a,r);d||(d={refs:0,staticSheet:null,dynamicStyles:null},u.set(o.sheetsManager,a,r,d));var h=Object(i.a)({},a.options,o,{theme:r,flip:"boolean"==typeof o.flip?o.flip:"rtl"===r.direction});h.generateId=h.serverGenerateClassName||h.generateClassName;var f=o.sheetsRegistry;if(0===d.refs){var p;o.sheetsCache&&(p=u.get(o.sheetsCache,a,r));var g=a.create(r,c);p||((p=o.jss.createStyleSheet(g,Object(i.a)({link:!1},h))).attach(),o.sheetsCache&&u.set(o.sheetsCache,a,r,p)),f&&f.add(p),d.staticSheet=p,d.dynamicStyles=Object(s.e)(g)}if(d.dynamicStyles){var v=o.jss.createStyleSheet(d.dynamicStyles,Object(i.a)({link:!0},h));v.update(t),v.attach(),n.dynamicSheet=v,n.classes=Object(l.a)({baseClasses:d.staticSheet.classes,newClasses:v.classes}),f&&f.add(v)}else n.classes=d.staticSheet.classes;d.refs+=1}}function b(e,t){var n=e.state;n.dynamicSheet&&n.dynamicSheet.update(t)}function C(e){var t=e.state,n=e.theme,r=e.stylesOptions,i=e.stylesCreator;if(!r.disableGeneration){var o=u.get(r.sheetsManager,i,n);o.refs-=1;var a=r.sheetsRegistry;0===o.refs&&(u.delete(r.sheetsManager,i,n),r.jss.removeStyleSheet(o.staticSheet),a&&a.remove(o.staticSheet)),t.dynamicSheet&&(r.jss.removeStyleSheet(t.dynamicSheet),a&&a.remove(t.dynamicSheet))}}function A(e,t){var n,r=a.a.useRef([]),i=a.a.useMemo((function(){return{}}),t);r.current!==i&&(r.current=i,n=e()),a.a.useEffect((function(){return function(){n&&n()}}),[i])}function _(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=t.name,o=t.classNamePrefix,s=t.Component,l=t.defaultTheme,u=void 0===l?v:l,h=Object(r.a)(t,["name","classNamePrefix","Component","defaultTheme"]),p=g(e),_=n||o||"makeStyles";p.options={index:f(),name:n,meta:_,classNamePrefix:_};var I=function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},t=Object(c.a)()||u,r=Object(i.a)({},a.a.useContext(d.a),h),o=a.a.useRef(),l=a.a.useRef();A((function(){var i={name:n,state:{},stylesCreator:p,stylesOptions:r,theme:t};return y(i,e),l.current=!1,o.current=i,function(){C(i)}}),[t,p]),a.a.useEffect((function(){l.current&&b(o.current,e),l.current=!0}));var f=m(o.current,e.classes,s);return f};return I}n.d(t,"a",(function(){return _}))},function(e,t,n){!function(e){"use strict";e.defineMode("markdown",(function(t,n){var r=e.getMode(t,"text/html"),i="null"==r.name;void 0===n.highlightFormatting&&(n.highlightFormatting=!1),void 0===n.maxBlockquoteDepth&&(n.maxBlockquoteDepth=0),void 0===n.taskLists&&(n.taskLists=!1),void 0===n.strikethrough&&(n.strikethrough=!1),void 0===n.emoji&&(n.emoji=!1),void 0===n.fencedCodeBlockHighlighting&&(n.fencedCodeBlockHighlighting=!0),void 0===n.fencedCodeBlockDefaultMode&&(n.fencedCodeBlockDefaultMode="text/plain"),void 0===n.xml&&(n.xml=!0),void 0===n.tokenTypeOverrides&&(n.tokenTypeOverrides={});var o={header:"header",code:"comment",quote:"quote",list1:"variable-2",list2:"variable-3",list3:"keyword",hr:"hr",image:"image",imageAltText:"image-alt-text",imageMarker:"image-marker",formatting:"formatting",linkInline:"link",linkEmail:"link",linkText:"link",linkHref:"string",em:"em",strong:"strong",strikethrough:"strikethrough",emoji:"builtin"};for(var a in o)o.hasOwnProperty(a)&&n.tokenTypeOverrides[a]&&(o[a]=n.tokenTypeOverrides[a]);var s=/^([*\-_])(?:\s*\1){2,}\s*$/,l=/^(?:[*\-+]|^[0-9]+([.)]))\s+/,u=/^\[(x| )\](?=\s)/i,c=n.allowAtxHeaderWithoutSpace?/^(#+)/:/^(#+)(?: |$)/,d=/^ {0,3}(?:\={1,}|-{2,})\s*$/,h=/^[^#!\[\]*_\\<>` "'(~:]+/,f=/^(~~~+|```+)[ \t]*([\w\/+#-]*)[^\n`]*$/,p=/^\s*\[[^\]]+?\]:.*$/,g=/[!"#$%&'()*+,\-.\/:;<=>?@\[\\\]^_`{|}~\xA1\xA7\xAB\xB6\xB7\xBB\xBF\u037E\u0387\u055A-\u055F\u0589\u058A\u05BE\u05C0\u05C3\u05C6\u05F3\u05F4\u0609\u060A\u060C\u060D\u061B\u061E\u061F\u066A-\u066D\u06D4\u0700-\u070D\u07F7-\u07F9\u0830-\u083E\u085E\u0964\u0965\u0970\u0AF0\u0DF4\u0E4F\u0E5A\u0E5B\u0F04-\u0F12\u0F14\u0F3A-\u0F3D\u0F85\u0FD0-\u0FD4\u0FD9\u0FDA\u104A-\u104F\u10FB\u1360-\u1368\u1400\u166D\u166E\u169B\u169C\u16EB-\u16ED\u1735\u1736\u17D4-\u17D6\u17D8-\u17DA\u1800-\u180A\u1944\u1945\u1A1E\u1A1F\u1AA0-\u1AA6\u1AA8-\u1AAD\u1B5A-\u1B60\u1BFC-\u1BFF\u1C3B-\u1C3F\u1C7E\u1C7F\u1CC0-\u1CC7\u1CD3\u2010-\u2027\u2030-\u2043\u2045-\u2051\u2053-\u205E\u207D\u207E\u208D\u208E\u2308-\u230B\u2329\u232A\u2768-\u2775\u27C5\u27C6\u27E6-\u27EF\u2983-\u2998\u29D8-\u29DB\u29FC\u29FD\u2CF9-\u2CFC\u2CFE\u2CFF\u2D70\u2E00-\u2E2E\u2E30-\u2E42\u3001-\u3003\u3008-\u3011\u3014-\u301F\u3030\u303D\u30A0\u30FB\uA4FE\uA4FF\uA60D-\uA60F\uA673\uA67E\uA6F2-\uA6F7\uA874-\uA877\uA8CE\uA8CF\uA8F8-\uA8FA\uA8FC\uA92E\uA92F\uA95F\uA9C1-\uA9CD\uA9DE\uA9DF\uAA5C-\uAA5F\uAADE\uAADF\uAAF0\uAAF1\uABEB\uFD3E\uFD3F\uFE10-\uFE19\uFE30-\uFE52\uFE54-\uFE61\uFE63\uFE68\uFE6A\uFE6B\uFF01-\uFF03\uFF05-\uFF0A\uFF0C-\uFF0F\uFF1A\uFF1B\uFF1F\uFF20\uFF3B-\uFF3D\uFF3F\uFF5B\uFF5D\uFF5F-\uFF65]|\uD800[\uDD00-\uDD02\uDF9F\uDFD0]|\uD801\uDD6F|\uD802[\uDC57\uDD1F\uDD3F\uDE50-\uDE58\uDE7F\uDEF0-\uDEF6\uDF39-\uDF3F\uDF99-\uDF9C]|\uD804[\uDC47-\uDC4D\uDCBB\uDCBC\uDCBE-\uDCC1\uDD40-\uDD43\uDD74\uDD75\uDDC5-\uDDC9\uDDCD\uDDDB\uDDDD-\uDDDF\uDE38-\uDE3D\uDEA9]|\uD805[\uDCC6\uDDC1-\uDDD7\uDE41-\uDE43\uDF3C-\uDF3E]|\uD809[\uDC70-\uDC74]|\uD81A[\uDE6E\uDE6F\uDEF5\uDF37-\uDF3B\uDF44]|\uD82F\uDC9F|\uD836[\uDE87-\uDE8B]/;function v(e,t,n){return t.f=t.inline=n,n(e,t)}function m(e,t,n){return t.f=t.block=n,n(e,t)}function y(t){if(t.linkTitle=!1,t.linkHref=!1,t.linkText=!1,t.em=!1,t.strong=!1,t.strikethrough=!1,t.quote=0,t.indentedCode=!1,t.f==C){var n=i;if(!n){var o=e.innerMode(r,t.htmlState);n="xml"==o.mode.name&&null===o.state.tagStart&&!o.state.context&&o.state.tokenize.isInText}n&&(t.f=x,t.block=b,t.htmlState=null)}return t.trailingSpace=0,t.trailingSpaceNewLine=!1,t.prevLine=t.thisLine,t.thisLine={stream:null},null}function b(r,i){var a,h=r.column()===i.indentation,g=!(a=i.prevLine.stream)||!/\S/.test(a.string),m=i.indentedCode,y=i.prevLine.hr,b=!1!==i.list,C=(i.listStack[i.listStack.length-1]||0)+3;i.indentedCode=!1;var I=i.indentation;if(null===i.indentationDiff&&(i.indentationDiff=i.indentation,b)){for(i.list=null;I<i.listStack[i.listStack.length-1];)i.listStack.pop(),i.listStack.length?i.indentation=i.listStack[i.listStack.length-1]:i.list=!1;!1!==i.list&&(i.indentationDiff=I-i.listStack[i.listStack.length-1])}var x=!(g||y||i.prevLine.header||b&&m||i.prevLine.fencedCodeEnd),w=(!1===i.list||y||g)&&i.indentation<=C&&r.match(s),M=null;if(i.indentationDiff>=4&&(m||i.prevLine.fencedCodeEnd||i.prevLine.header||g))return r.skipToEnd(),i.indentedCode=!0,o.code;if(r.eatSpace())return null;if(h&&i.indentation<=C&&(M=r.match(c))&&M[1].length<=6)return i.quote=0,i.header=M[1].length,i.thisLine.header=!0,n.highlightFormatting&&(i.formatting="header"),i.f=i.inline,_(i);if(i.indentation<=C&&r.eat(">"))return i.quote=h?1:i.quote+1,n.highlightFormatting&&(i.formatting="quote"),r.eatSpace(),_(i);if(!w&&!i.setext&&h&&i.indentation<=C&&(M=r.match(l))){var S=M[1]?"ol":"ul";return i.indentation=I+r.current().length,i.list=!0,i.quote=0,i.listStack.push(i.indentation),i.em=!1,i.strong=!1,i.code=!1,i.strikethrough=!1,n.taskLists&&r.match(u,!1)&&(i.taskList=!0),i.f=i.inline,n.highlightFormatting&&(i.formatting=["list","list-"+S]),_(i)}return h&&i.indentation<=C&&(M=r.match(f,!0))?(i.quote=0,i.fencedEndRE=new RegExp(M[1]+"+ *$"),i.localMode=n.fencedCodeBlockHighlighting&&function(n){if(e.findModeByName){var r=e.findModeByName(n);r&&(n=r.mime||r.mimes[0])}var i=e.getMode(t,n);return"null"==i.name?null:i}(M[2]||n.fencedCodeBlockDefaultMode),i.localMode&&(i.localState=e.startState(i.localMode)),i.f=i.block=A,n.highlightFormatting&&(i.formatting="code-block"),i.code=-1,_(i)):i.setext||!(x&&b||i.quote||!1!==i.list||i.code||w||p.test(r.string))&&(M=r.lookAhead(1))&&(M=M.match(d))?(i.setext?(i.header=i.setext,i.setext=0,r.skipToEnd(),n.highlightFormatting&&(i.formatting="header")):(i.header="="==M[0].charAt(0)?1:2,i.setext=i.header),i.thisLine.header=!0,i.f=i.inline,_(i)):w?(r.skipToEnd(),i.hr=!0,i.thisLine.hr=!0,o.hr):"["===r.peek()?v(r,i,T):v(r,i,i.inline)}function C(t,n){var o=r.token(t,n.htmlState);if(!i){var a=e.innerMode(r,n.htmlState);("xml"==a.mode.name&&null===a.state.tagStart&&!a.state.context&&a.state.tokenize.isInText||n.md_inside&&t.current().indexOf(">")>-1)&&(n.f=x,n.block=b,n.htmlState=null)}return o}function A(e,t){var r,i=t.listStack[t.listStack.length-1]||0,a=t.indentation<i,s=i+3;return t.fencedEndRE&&t.indentation<=s&&(a||e.match(t.fencedEndRE))?(n.highlightFormatting&&(t.formatting="code-block"),a||(r=_(t)),t.localMode=t.localState=null,t.block=b,t.f=x,t.fencedEndRE=null,t.code=0,t.thisLine.fencedCodeEnd=!0,a?m(e,t,t.block):r):t.localMode?t.localMode.token(e,t.localState):(e.skipToEnd(),o.code)}function _(e){var t=[];if(e.formatting){t.push(o.formatting),"string"==typeof e.formatting&&(e.formatting=[e.formatting]);for(var r=0;r<e.formatting.length;r++)t.push(o.formatting+"-"+e.formatting[r]),"header"===e.formatting[r]&&t.push(o.formatting+"-"+e.formatting[r]+"-"+e.header),"quote"===e.formatting[r]&&(!n.maxBlockquoteDepth||n.maxBlockquoteDepth>=e.quote?t.push(o.formatting+"-"+e.formatting[r]+"-"+e.quote):t.push("error"))}if(e.taskOpen)return t.push("meta"),t.length?t.join(" "):null;if(e.taskClosed)return t.push("property"),t.length?t.join(" "):null;if(e.linkHref?t.push(o.linkHref,"url"):(e.strong&&t.push(o.strong),e.em&&t.push(o.em),e.strikethrough&&t.push(o.strikethrough),e.emoji&&t.push(o.emoji),e.linkText&&t.push(o.linkText),e.code&&t.push(o.code),e.image&&t.push(o.image),e.imageAltText&&t.push(o.imageAltText,"link"),e.imageMarker&&t.push(o.imageMarker)),e.header&&t.push(o.header,o.header+"-"+e.header),e.quote&&(t.push(o.quote),!n.maxBlockquoteDepth||n.maxBlockquoteDepth>=e.quote?t.push(o.quote+"-"+e.quote):t.push(o.quote+"-"+n.maxBlockquoteDepth)),!1!==e.list){var i=(e.listStack.length-1)%3;i?1===i?t.push(o.list2):t.push(o.list3):t.push(o.list1)}return e.trailingSpaceNewLine?t.push("trailing-space-new-line"):e.trailingSpace&&t.push("trailing-space-"+(e.trailingSpace%2?"a":"b")),t.length?t.join(" "):null}function I(e,t){if(e.match(h,!0))return _(t)}function x(t,i){var a=i.text(t,i);if(void 0!==a)return a;if(i.list)return i.list=null,_(i);if(i.taskList)return" "===t.match(u,!0)[1]?i.taskOpen=!0:i.taskClosed=!0,n.highlightFormatting&&(i.formatting="task"),i.taskList=!1,_(i);if(i.taskOpen=!1,i.taskClosed=!1,i.header&&t.match(/^#+$/,!0))return n.highlightFormatting&&(i.formatting="header"),_(i);var s=t.next();if(i.linkTitle){i.linkTitle=!1;var l=s;"("===s&&(l=")");var c="^\\s*(?:[^"+(l=(l+"").replace(/([.?*+^\[\]\\(){}|-])/g,"\\$1"))+"\\\\]+|\\\\\\\\|\\\\.)"+l;if(t.match(new RegExp(c),!0))return o.linkHref}if("`"===s){var d=i.formatting;n.highlightFormatting&&(i.formatting="code"),t.eatWhile("`");var h=t.current().length;if(0!=i.code||i.quote&&1!=h){if(h==i.code){var f=_(i);return i.code=0,f}return i.formatting=d,_(i)}return i.code=h,_(i)}if(i.code)return _(i);if("\\"===s&&(t.next(),n.highlightFormatting)){var p=_(i),v=o.formatting+"-escape";return p?p+" "+v:v}if("!"===s&&t.match(/\[[^\]]*\] ?(?:\(|\[)/,!1))return i.imageMarker=!0,i.image=!0,n.highlightFormatting&&(i.formatting="image"),_(i);if("["===s&&i.imageMarker&&t.match(/[^\]]*\](\(.*?\)| ?\[.*?\])/,!1))return i.imageMarker=!1,i.imageAltText=!0,n.highlightFormatting&&(i.formatting="image"),_(i);if("]"===s&&i.imageAltText){n.highlightFormatting&&(i.formatting="image");var p=_(i);return i.imageAltText=!1,i.image=!1,i.inline=i.f=M,p}if("["===s&&!i.image)return i.linkText&&t.match(/^.*?\]/)||(i.linkText=!0,n.highlightFormatting&&(i.formatting="link")),_(i);if("]"===s&&i.linkText){n.highlightFormatting&&(i.formatting="link");var p=_(i);return i.linkText=!1,i.inline=i.f=t.match(/\(.*?\)| ?\[.*?\]/,!1)?M:x,p}if("<"===s&&t.match(/^(https?|ftps?):\/\/(?:[^\\>]|\\.)+>/,!1))return i.f=i.inline=w,n.highlightFormatting&&(i.formatting="link"),(p=_(i))?p+=" ":p="",p+o.linkInline;if("<"===s&&t.match(/^[^> \\]+@(?:[^\\>]|\\.)+>/,!1))return i.f=i.inline=w,n.highlightFormatting&&(i.formatting="link"),(p=_(i))?p+=" ":p="",p+o.linkEmail;if(n.xml&&"<"===s&&t.match(/^(!--|\?|!\[CDATA\[|[a-z][a-z0-9-]*(?:\s+[a-z_:.\-]+(?:\s*=\s*[^>]+)?)*\s*(?:>|$))/i,!1)){var y=t.string.indexOf(">",t.pos);if(-1!=y){var b=t.string.substring(t.start,y);/markdown\s*=\s*('|"){0,1}1('|"){0,1}/.test(b)&&(i.md_inside=!0)}return t.backUp(1),i.htmlState=e.startState(r),m(t,i,C)}if(n.xml&&"<"===s&&t.match(/^\/\w*?>/))return i.md_inside=!1,"tag";if("*"===s||"_"===s){for(var A=1,I=1==t.pos?" ":t.string.charAt(t.pos-2);A<3&&t.eat(s);)A++;var S=t.peek()||" ",T=!/\s/.test(S)&&(!g.test(S)||/\s/.test(I)||g.test(I)),E=!/\s/.test(I)&&(!g.test(I)||/\s/.test(S)||g.test(S)),O=null,k=null;if(A%2&&(i.em||!T||"*"!==s&&E&&!g.test(I)?i.em!=s||!E||"*"!==s&&T&&!g.test(S)||(O=!1):O=!0),A>1&&(i.strong||!T||"*"!==s&&E&&!g.test(I)?i.strong!=s||!E||"*"!==s&&T&&!g.test(S)||(k=!1):k=!0),null!=k||null!=O)return n.highlightFormatting&&(i.formatting=null==O?"strong":null==k?"em":"strong em"),!0===O&&(i.em=s),!0===k&&(i.strong=s),f=_(i),!1===O&&(i.em=!1),!1===k&&(i.strong=!1),f}else if(" "===s&&(t.eat("*")||t.eat("_"))){if(" "===t.peek())return _(i);t.backUp(1)}if(n.strikethrough)if("~"===s&&t.eatWhile(s)){if(i.strikethrough)return n.highlightFormatting&&(i.formatting="strikethrough"),f=_(i),i.strikethrough=!1,f;if(t.match(/^[^\s]/,!1))return i.strikethrough=!0,n.highlightFormatting&&(i.formatting="strikethrough"),_(i)}else if(" "===s&&t.match(/^~~/,!0)){if(" "===t.peek())return _(i);t.backUp(2)}if(n.emoji&&":"===s&&t.match(/^(?:[a-z_\d+][a-z_\d+-]*|\-[a-z_\d+][a-z_\d+-]*):/)){i.emoji=!0,n.highlightFormatting&&(i.formatting="emoji");var L=_(i);return i.emoji=!1,L}return" "===s&&(t.match(/^ +$/,!1)?i.trailingSpace++:i.trailingSpace&&(i.trailingSpaceNewLine=!0)),_(i)}function w(e,t){if(">"===e.next()){t.f=t.inline=x,n.highlightFormatting&&(t.formatting="link");var r=_(t);return r?r+=" ":r="",r+o.linkInline}return e.match(/^[^>]+/,!0),o.linkInline}function M(e,t){if(e.eatSpace())return null;var r,i=e.next();return"("===i||"["===i?(t.f=t.inline=(r="("===i?")":"]",function(e,t){if(e.next()===r){t.f=t.inline=x,n.highlightFormatting&&(t.formatting="link-string");var i=_(t);return t.linkHref=!1,i}return e.match(S[r]),t.linkHref=!0,_(t)}),n.highlightFormatting&&(t.formatting="link-string"),t.linkHref=!0,_(t)):"error"}var S={")":/^(?:[^\\\(\)]|\\.|\((?:[^\\\(\)]|\\.)*\))*?(?=\))/,"]":/^(?:[^\\\[\]]|\\.|\[(?:[^\\\[\]]|\\.)*\])*?(?=\])/};function T(e,t){return e.match(/^([^\]\\]|\\.)*\]:/,!1)?(t.f=E,e.next(),n.highlightFormatting&&(t.formatting="link"),t.linkText=!0,_(t)):v(e,t,x)}function E(e,t){if(e.match(/^\]:/,!0)){t.f=t.inline=O,n.highlightFormatting&&(t.formatting="link");var r=_(t);return t.linkText=!1,r}return e.match(/^([^\]\\]|\\.)+/,!0),o.linkText}function O(e,t){return e.eatSpace()?null:(e.match(/^[^\s]+/,!0),void 0===e.peek()?t.linkTitle=!0:e.match(/^(?:\s+(?:"(?:[^"\\]|\\\\|\\.)+"|'(?:[^'\\]|\\\\|\\.)+'|\((?:[^)\\]|\\\\|\\.)+\)))?/,!0),t.f=t.inline=x,o.linkHref+" url")}var k={startState:function(){return{f:b,prevLine:{stream:null},thisLine:{stream:null},block:b,htmlState:null,indentation:0,inline:x,text:I,formatting:!1,linkText:!1,linkHref:!1,linkTitle:!1,code:0,em:!1,strong:!1,header:0,setext:0,hr:!1,taskList:!1,list:!1,listStack:[],quote:0,trailingSpace:0,trailingSpaceNewLine:!1,strikethrough:!1,emoji:!1,fencedEndRE:null}},copyState:function(t){return{f:t.f,prevLine:t.prevLine,thisLine:t.thisLine,block:t.block,htmlState:t.htmlState&&e.copyState(r,t.htmlState),indentation:t.indentation,localMode:t.localMode,localState:t.localMode?e.copyState(t.localMode,t.localState):null,inline:t.inline,text:t.text,formatting:!1,linkText:t.linkText,linkTitle:t.linkTitle,linkHref:t.linkHref,code:t.code,em:t.em,strong:t.strong,strikethrough:t.strikethrough,emoji:t.emoji,header:t.header,setext:t.setext,hr:t.hr,taskList:t.taskList,list:t.list,listStack:t.listStack.slice(0),quote:t.quote,indentedCode:t.indentedCode,trailingSpace:t.trailingSpace,trailingSpaceNewLine:t.trailingSpaceNewLine,md_inside:t.md_inside,fencedEndRE:t.fencedEndRE}},token:function(e,t){if(t.formatting=!1,e!=t.thisLine.stream){if(t.header=0,t.hr=!1,e.match(/^\s*$/,!0))return y(t),null;if(t.prevLine=t.thisLine,t.thisLine={stream:e},t.taskList=!1,t.trailingSpace=0,t.trailingSpaceNewLine=!1,!t.localState&&(t.f=t.block,t.f!=C)){var n=e.match(/^\s*/,!0)[0].replace(/\t/g,"    ").length;if(t.indentation=n,t.indentationDiff=null,n>0)return null}}return t.f(e,t)},innerMode:function(e){return e.block==C?{state:e.htmlState,mode:r}:e.localState?{state:e.localState,mode:e.localMode}:{state:e,mode:k}},indent:function(t,n,i){return t.block==C&&r.indent?r.indent(t.htmlState,n,i):t.localState&&t.localMode.indent?t.localMode.indent(t.localState,n,i):e.Pass},blankLine:y,getType:_,blockCommentStart:"\x3c!--",blockCommentEnd:"--\x3e",closeBrackets:"()[]{}''\"\"``",fold:"markdown"};return k}),"xml"),e.defineMIME("text/markdown","markdown"),e.defineMIME("text/x-markdown","markdown")}(n(24),n(1317),n(500))},,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,function(e,t,n){"use strict";var r=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),i=this&&this.__setModuleDefault||(Object.create?function(e,t){Object.defineProperty(e,"default",{enumerable:!0,value:t})}:function(e,t){e.default=t}),o=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)"default"!==n&&Object.prototype.hasOwnProperty.call(e,n)&&r(t,e,n);return i(t,e),t},a=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};Object.defineProperty(t,"__esModule",{value:!0}),t.URLExt=void 0;const s=a(n(1348)),l=o(n(486));!function(e){function t(e){if("undefined"!=typeof document&&document){const t=document.createElement("a");return t.href=e,t}return s.default(e)}function n(...e){const t=s.default(e[0],{}),n=`${t.protocol}${t.slashes?"//":""}${t.auth}${t.auth?"@":""}${t.host}`,r=l.join(`${n&&"/"!==t.pathname[0]?"/":""}${t.pathname}`,...e.slice(1));return`${n}${"."===r?"":r}`}e.parse=t,e.normalize=function(e){return e&&t(e).toString()},e.join=n,e.encodeParts=function(e){return n(...e.split("/").map(encodeURIComponent))},e.objectToQueryString=function(e){const t=Object.keys(e).filter(e=>e.length>0);return t.length?"?"+t.map(t=>{const n=encodeURIComponent(String(e[t]));return t+(n?"="+n:"")}).join("&"):""},e.queryStringToObject=function(e){return e.replace(/^\?/,"").split("&").reduce((e,t)=>{const[n,r]=t.split("=");return n.length>0&&(e[n]=decodeURIComponent(r||"")),e},{})},e.isLocal=function(e){const{protocol:n}=t(e);return(!n||0!==e.toLowerCase().indexOf(n))&&0!==e.indexOf("/")}}(t.URLExt||(t.URLExt={}))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("af",{months:"Januarie_Februarie_Maart_April_Mei_Junie_Julie_Augustus_September_Oktober_November_Desember".split("_"),monthsShort:"Jan_Feb_Mrt_Apr_Mei_Jun_Jul_Aug_Sep_Okt_Nov_Des".split("_"),weekdays:"Sondag_Maandag_Dinsdag_Woensdag_Donderdag_Vrydag_Saterdag".split("_"),weekdaysShort:"Son_Maa_Din_Woe_Don_Vry_Sat".split("_"),weekdaysMin:"So_Ma_Di_Wo_Do_Vr_Sa".split("_"),meridiemParse:/vm|nm/i,isPM:function(e){return/^nm$/i.test(e)},meridiem:function(e,t,n){return e<12?n?"vm":"VM":n?"nm":"NM"},longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Vandag om] LT",nextDay:"[Môre om] LT",nextWeek:"dddd [om] LT",lastDay:"[Gister om] LT",lastWeek:"[Laas] dddd [om] LT",sameElse:"L"},relativeTime:{future:"oor %s",past:"%s gelede",s:"'n paar sekondes",ss:"%d sekondes",m:"'n minuut",mm:"%d minute",h:"'n uur",hh:"%d ure",d:"'n dag",dd:"%d dae",M:"'n maand",MM:"%d maande",y:"'n jaar",yy:"%d jaar"},dayOfMonthOrdinalParse:/\d{1,2}(ste|de)/,ordinal:function(e){return e+(1===e||8===e||e>=20?"ste":"de")},week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t={1:"١",2:"٢",3:"٣",4:"٤",5:"٥",6:"٦",7:"٧",8:"٨",9:"٩",0:"٠"},n={"١":"1","٢":"2","٣":"3","٤":"4","٥":"5","٦":"6","٧":"7","٨":"8","٩":"9","٠":"0"},r=function(e){return 0===e?0:1===e?1:2===e?2:e%100>=3&&e%100<=10?3:e%100>=11?4:5},i={s:["أقل من ثانية","ثانية واحدة",["ثانيتان","ثانيتين"],"%d ثوان","%d ثانية","%d ثانية"],m:["أقل من دقيقة","دقيقة واحدة",["دقيقتان","دقيقتين"],"%d دقائق","%d دقيقة","%d دقيقة"],h:["أقل من ساعة","ساعة واحدة",["ساعتان","ساعتين"],"%d ساعات","%d ساعة","%d ساعة"],d:["أقل من يوم","يوم واحد",["يومان","يومين"],"%d أيام","%d يومًا","%d يوم"],M:["أقل من شهر","شهر واحد",["شهران","شهرين"],"%d أشهر","%d شهرا","%d شهر"],y:["أقل من عام","عام واحد",["عامان","عامين"],"%d أعوام","%d عامًا","%d عام"]},o=function(e){return function(t,n,o,a){var s=r(t),l=i[e][r(t)];return 2===s&&(l=l[n?0:1]),l.replace(/%d/i,t)}},a=["يناير","فبراير","مارس","أبريل","مايو","يونيو","يوليو","أغسطس","سبتمبر","أكتوبر","نوفمبر","ديسمبر"];e.defineLocale("ar",{months:a,monthsShort:a,weekdays:"الأحد_الإثنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت".split("_"),weekdaysShort:"أحد_إثنين_ثلاثاء_أربعاء_خميس_جمعة_سبت".split("_"),weekdaysMin:"ح_ن_ث_ر_خ_ج_س".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"D/‏M/‏YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},meridiemParse:/ص|م/,isPM:function(e){return"م"===e},meridiem:function(e,t,n){return e<12?"ص":"م"},calendar:{sameDay:"[اليوم عند الساعة] LT",nextDay:"[غدًا عند الساعة] LT",nextWeek:"dddd [عند الساعة] LT",lastDay:"[أمس عند الساعة] LT",lastWeek:"dddd [عند الساعة] LT",sameElse:"L"},relativeTime:{future:"بعد %s",past:"منذ %s",s:o("s"),ss:o("s"),m:o("m"),mm:o("m"),h:o("h"),hh:o("h"),d:o("d"),dd:o("d"),M:o("M"),MM:o("M"),y:o("y"),yy:o("y")},preparse:function(e){return e.replace(/[١٢٣٤٥٦٧٨٩٠]/g,(function(e){return n[e]})).replace(/،/g,",")},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]})).replace(/,/g,"،")},week:{dow:6,doy:12}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t=function(e){return 0===e?0:1===e?1:2===e?2:e%100>=3&&e%100<=10?3:e%100>=11?4:5},n={s:["أقل من ثانية","ثانية واحدة",["ثانيتان","ثانيتين"],"%d ثوان","%d ثانية","%d ثانية"],m:["أقل من دقيقة","دقيقة واحدة",["دقيقتان","دقيقتين"],"%d دقائق","%d دقيقة","%d دقيقة"],h:["أقل من ساعة","ساعة واحدة",["ساعتان","ساعتين"],"%d ساعات","%d ساعة","%d ساعة"],d:["أقل من يوم","يوم واحد",["يومان","يومين"],"%d أيام","%d يومًا","%d يوم"],M:["أقل من شهر","شهر واحد",["شهران","شهرين"],"%d أشهر","%d شهرا","%d شهر"],y:["أقل من عام","عام واحد",["عامان","عامين"],"%d أعوام","%d عامًا","%d عام"]},r=function(e){return function(r,i,o,a){var s=t(r),l=n[e][t(r)];return 2===s&&(l=l[i?0:1]),l.replace(/%d/i,r)}},i=["جانفي","فيفري","مارس","أفريل","ماي","جوان","جويلية","أوت","سبتمبر","أكتوبر","نوفمبر","ديسمبر"];e.defineLocale("ar-dz",{months:i,monthsShort:i,weekdays:"الأحد_الإثنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت".split("_"),weekdaysShort:"أحد_إثنين_ثلاثاء_أربعاء_خميس_جمعة_سبت".split("_"),weekdaysMin:"ح_ن_ث_ر_خ_ج_س".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"D/‏M/‏YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},meridiemParse:/ص|م/,isPM:function(e){return"م"===e},meridiem:function(e,t,n){return e<12?"ص":"م"},calendar:{sameDay:"[اليوم عند الساعة] LT",nextDay:"[غدًا عند الساعة] LT",nextWeek:"dddd [عند الساعة] LT",lastDay:"[أمس عند الساعة] LT",lastWeek:"dddd [عند الساعة] LT",sameElse:"L"},relativeTime:{future:"بعد %s",past:"منذ %s",s:r("s"),ss:r("s"),m:r("m"),mm:r("m"),h:r("h"),hh:r("h"),d:r("d"),dd:r("d"),M:r("M"),MM:r("M"),y:r("y"),yy:r("y")},postformat:function(e){return e.replace(/,/g,"،")},week:{dow:0,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("ar-kw",{months:"يناير_فبراير_مارس_أبريل_ماي_يونيو_يوليوز_غشت_شتنبر_أكتوبر_نونبر_دجنبر".split("_"),monthsShort:"يناير_فبراير_مارس_أبريل_ماي_يونيو_يوليوز_غشت_شتنبر_أكتوبر_نونبر_دجنبر".split("_"),weekdays:"الأحد_الإتنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت".split("_"),weekdaysShort:"احد_اتنين_ثلاثاء_اربعاء_خميس_جمعة_سبت".split("_"),weekdaysMin:"ح_ن_ث_ر_خ_ج_س".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[اليوم على الساعة] LT",nextDay:"[غدا على الساعة] LT",nextWeek:"dddd [على الساعة] LT",lastDay:"[أمس على الساعة] LT",lastWeek:"dddd [على الساعة] LT",sameElse:"L"},relativeTime:{future:"في %s",past:"منذ %s",s:"ثوان",ss:"%d ثانية",m:"دقيقة",mm:"%d دقائق",h:"ساعة",hh:"%d ساعات",d:"يوم",dd:"%d أيام",M:"شهر",MM:"%d أشهر",y:"سنة",yy:"%d سنوات"},week:{dow:0,doy:12}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t={1:"1",2:"2",3:"3",4:"4",5:"5",6:"6",7:"7",8:"8",9:"9",0:"0"},n=function(e){return 0===e?0:1===e?1:2===e?2:e%100>=3&&e%100<=10?3:e%100>=11?4:5},r={s:["أقل من ثانية","ثانية واحدة",["ثانيتان","ثانيتين"],"%d ثوان","%d ثانية","%d ثانية"],m:["أقل من دقيقة","دقيقة واحدة",["دقيقتان","دقيقتين"],"%d دقائق","%d دقيقة","%d دقيقة"],h:["أقل من ساعة","ساعة واحدة",["ساعتان","ساعتين"],"%d ساعات","%d ساعة","%d ساعة"],d:["أقل من يوم","يوم واحد",["يومان","يومين"],"%d أيام","%d يومًا","%d يوم"],M:["أقل من شهر","شهر واحد",["شهران","شهرين"],"%d أشهر","%d شهرا","%d شهر"],y:["أقل من عام","عام واحد",["عامان","عامين"],"%d أعوام","%d عامًا","%d عام"]},i=function(e){return function(t,i,o,a){var s=n(t),l=r[e][n(t)];return 2===s&&(l=l[i?0:1]),l.replace(/%d/i,t)}},o=["يناير","فبراير","مارس","أبريل","مايو","يونيو","يوليو","أغسطس","سبتمبر","أكتوبر","نوفمبر","ديسمبر"];e.defineLocale("ar-ly",{months:o,monthsShort:o,weekdays:"الأحد_الإثنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت".split("_"),weekdaysShort:"أحد_إثنين_ثلاثاء_أربعاء_خميس_جمعة_سبت".split("_"),weekdaysMin:"ح_ن_ث_ر_خ_ج_س".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"D/‏M/‏YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},meridiemParse:/ص|م/,isPM:function(e){return"م"===e},meridiem:function(e,t,n){return e<12?"ص":"م"},calendar:{sameDay:"[اليوم عند الساعة] LT",nextDay:"[غدًا عند الساعة] LT",nextWeek:"dddd [عند الساعة] LT",lastDay:"[أمس عند الساعة] LT",lastWeek:"dddd [عند الساعة] LT",sameElse:"L"},relativeTime:{future:"بعد %s",past:"منذ %s",s:i("s"),ss:i("s"),m:i("m"),mm:i("m"),h:i("h"),hh:i("h"),d:i("d"),dd:i("d"),M:i("M"),MM:i("M"),y:i("y"),yy:i("y")},preparse:function(e){return e.replace(/،/g,",")},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]})).replace(/,/g,"،")},week:{dow:6,doy:12}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("ar-ma",{months:"يناير_فبراير_مارس_أبريل_ماي_يونيو_يوليوز_غشت_شتنبر_أكتوبر_نونبر_دجنبر".split("_"),monthsShort:"يناير_فبراير_مارس_أبريل_ماي_يونيو_يوليوز_غشت_شتنبر_أكتوبر_نونبر_دجنبر".split("_"),weekdays:"الأحد_الإثنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت".split("_"),weekdaysShort:"احد_اثنين_ثلاثاء_اربعاء_خميس_جمعة_سبت".split("_"),weekdaysMin:"ح_ن_ث_ر_خ_ج_س".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[اليوم على الساعة] LT",nextDay:"[غدا على الساعة] LT",nextWeek:"dddd [على الساعة] LT",lastDay:"[أمس على الساعة] LT",lastWeek:"dddd [على الساعة] LT",sameElse:"L"},relativeTime:{future:"في %s",past:"منذ %s",s:"ثوان",ss:"%d ثانية",m:"دقيقة",mm:"%d دقائق",h:"ساعة",hh:"%d ساعات",d:"يوم",dd:"%d أيام",M:"شهر",MM:"%d أشهر",y:"سنة",yy:"%d سنوات"},week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t={1:"١",2:"٢",3:"٣",4:"٤",5:"٥",6:"٦",7:"٧",8:"٨",9:"٩",0:"٠"},n={"١":"1","٢":"2","٣":"3","٤":"4","٥":"5","٦":"6","٧":"7","٨":"8","٩":"9","٠":"0"};e.defineLocale("ar-sa",{months:"يناير_فبراير_مارس_أبريل_مايو_يونيو_يوليو_أغسطس_سبتمبر_أكتوبر_نوفمبر_ديسمبر".split("_"),monthsShort:"يناير_فبراير_مارس_أبريل_مايو_يونيو_يوليو_أغسطس_سبتمبر_أكتوبر_نوفمبر_ديسمبر".split("_"),weekdays:"الأحد_الإثنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت".split("_"),weekdaysShort:"أحد_إثنين_ثلاثاء_أربعاء_خميس_جمعة_سبت".split("_"),weekdaysMin:"ح_ن_ث_ر_خ_ج_س".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},meridiemParse:/ص|م/,isPM:function(e){return"م"===e},meridiem:function(e,t,n){return e<12?"ص":"م"},calendar:{sameDay:"[اليوم على الساعة] LT",nextDay:"[غدا على الساعة] LT",nextWeek:"dddd [على الساعة] LT",lastDay:"[أمس على الساعة] LT",lastWeek:"dddd [على الساعة] LT",sameElse:"L"},relativeTime:{future:"في %s",past:"منذ %s",s:"ثوان",ss:"%d ثانية",m:"دقيقة",mm:"%d دقائق",h:"ساعة",hh:"%d ساعات",d:"يوم",dd:"%d أيام",M:"شهر",MM:"%d أشهر",y:"سنة",yy:"%d سنوات"},preparse:function(e){return e.replace(/[١٢٣٤٥٦٧٨٩٠]/g,(function(e){return n[e]})).replace(/،/g,",")},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]})).replace(/,/g,"،")},week:{dow:0,doy:6}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("ar-tn",{months:"جانفي_فيفري_مارس_أفريل_ماي_جوان_جويلية_أوت_سبتمبر_أكتوبر_نوفمبر_ديسمبر".split("_"),monthsShort:"جانفي_فيفري_مارس_أفريل_ماي_جوان_جويلية_أوت_سبتمبر_أكتوبر_نوفمبر_ديسمبر".split("_"),weekdays:"الأحد_الإثنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت".split("_"),weekdaysShort:"أحد_إثنين_ثلاثاء_أربعاء_خميس_جمعة_سبت".split("_"),weekdaysMin:"ح_ن_ث_ر_خ_ج_س".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[اليوم على الساعة] LT",nextDay:"[غدا على الساعة] LT",nextWeek:"dddd [على الساعة] LT",lastDay:"[أمس على الساعة] LT",lastWeek:"dddd [على الساعة] LT",sameElse:"L"},relativeTime:{future:"في %s",past:"منذ %s",s:"ثوان",ss:"%d ثانية",m:"دقيقة",mm:"%d دقائق",h:"ساعة",hh:"%d ساعات",d:"يوم",dd:"%d أيام",M:"شهر",MM:"%d أشهر",y:"سنة",yy:"%d سنوات"},week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t={1:"-inci",5:"-inci",8:"-inci",70:"-inci",80:"-inci",2:"-nci",7:"-nci",20:"-nci",50:"-nci",3:"-üncü",4:"-üncü",100:"-üncü",6:"-ncı",9:"-uncu",10:"-uncu",30:"-uncu",60:"-ıncı",90:"-ıncı"};e.defineLocale("az",{months:"yanvar_fevral_mart_aprel_may_iyun_iyul_avqust_sentyabr_oktyabr_noyabr_dekabr".split("_"),monthsShort:"yan_fev_mar_apr_may_iyn_iyl_avq_sen_okt_noy_dek".split("_"),weekdays:"Bazar_Bazar ertəsi_Çərşənbə axşamı_Çərşənbə_Cümə axşamı_Cümə_Şənbə".split("_"),weekdaysShort:"Baz_BzE_ÇAx_Çər_CAx_Cüm_Şən".split("_"),weekdaysMin:"Bz_BE_ÇA_Çə_CA_Cü_Şə".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[bugün saat] LT",nextDay:"[sabah saat] LT",nextWeek:"[gələn həftə] dddd [saat] LT",lastDay:"[dünən] LT",lastWeek:"[keçən həftə] dddd [saat] LT",sameElse:"L"},relativeTime:{future:"%s sonra",past:"%s əvvəl",s:"bir neçə saniyə",ss:"%d saniyə",m:"bir dəqiqə",mm:"%d dəqiqə",h:"bir saat",hh:"%d saat",d:"bir gün",dd:"%d gün",M:"bir ay",MM:"%d ay",y:"bir il",yy:"%d il"},meridiemParse:/gecə|səhər|gündüz|axşam/,isPM:function(e){return/^(gündüz|axşam)$/.test(e)},meridiem:function(e,t,n){return e<4?"gecə":e<12?"səhər":e<17?"gündüz":"axşam"},dayOfMonthOrdinalParse:/\d{1,2}-(ıncı|inci|nci|üncü|ncı|uncu)/,ordinal:function(e){if(0===e)return e+"-ıncı";var n=e%10;return e+(t[n]||t[e%100-n]||t[e>=100?100:null])},week:{dow:1,doy:7}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
function t(e,t,n){var r,i;return"m"===n?t?"хвіліна":"хвіліну":"h"===n?t?"гадзіна":"гадзіну":e+" "+(r=+e,i={ss:t?"секунда_секунды_секунд":"секунду_секунды_секунд",mm:t?"хвіліна_хвіліны_хвілін":"хвіліну_хвіліны_хвілін",hh:t?"гадзіна_гадзіны_гадзін":"гадзіну_гадзіны_гадзін",dd:"дзень_дні_дзён",MM:"месяц_месяцы_месяцаў",yy:"год_гады_гадоў"}[n].split("_"),r%10==1&&r%100!=11?i[0]:r%10>=2&&r%10<=4&&(r%100<10||r%100>=20)?i[1]:i[2])}e.defineLocale("be",{months:{format:"студзеня_лютага_сакавіка_красавіка_траўня_чэрвеня_ліпеня_жніўня_верасня_кастрычніка_лістапада_снежня".split("_"),standalone:"студзень_люты_сакавік_красавік_травень_чэрвень_ліпень_жнівень_верасень_кастрычнік_лістапад_снежань".split("_")},monthsShort:"студ_лют_сак_крас_трав_чэрв_ліп_жнів_вер_каст_ліст_снеж".split("_"),weekdays:{format:"нядзелю_панядзелак_аўторак_сераду_чацвер_пятніцу_суботу".split("_"),standalone:"нядзеля_панядзелак_аўторак_серада_чацвер_пятніца_субота".split("_"),isFormat:/\[ ?[Ууў] ?(?:мінулую|наступную)? ?\] ?dddd/},weekdaysShort:"нд_пн_ат_ср_чц_пт_сб".split("_"),weekdaysMin:"нд_пн_ат_ср_чц_пт_сб".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY г.",LLL:"D MMMM YYYY г., HH:mm",LLLL:"dddd, D MMMM YYYY г., HH:mm"},calendar:{sameDay:"[Сёння ў] LT",nextDay:"[Заўтра ў] LT",lastDay:"[Учора ў] LT",nextWeek:function(){return"[У] dddd [ў] LT"},lastWeek:function(){switch(this.day()){case 0:case 3:case 5:case 6:return"[У мінулую] dddd [ў] LT";case 1:case 2:case 4:return"[У мінулы] dddd [ў] LT"}},sameElse:"L"},relativeTime:{future:"праз %s",past:"%s таму",s:"некалькі секунд",m:t,mm:t,h:t,hh:t,d:"дзень",dd:t,M:"месяц",MM:t,y:"год",yy:t},meridiemParse:/ночы|раніцы|дня|вечара/,isPM:function(e){return/^(дня|вечара)$/.test(e)},meridiem:function(e,t,n){return e<4?"ночы":e<12?"раніцы":e<17?"дня":"вечара"},dayOfMonthOrdinalParse:/\d{1,2}-(і|ы|га)/,ordinal:function(e,t){switch(t){case"M":case"d":case"DDD":case"w":case"W":return e%10!=2&&e%10!=3||e%100==12||e%100==13?e+"-ы":e+"-і";case"D":return e+"-га";default:return e}},week:{dow:1,doy:7}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("bg",{months:"януари_февруари_март_април_май_юни_юли_август_септември_октомври_ноември_декември".split("_"),monthsShort:"яну_фев_мар_апр_май_юни_юли_авг_сеп_окт_ное_дек".split("_"),weekdays:"неделя_понеделник_вторник_сряда_четвъртък_петък_събота".split("_"),weekdaysShort:"нед_пон_вто_сря_чет_пет_съб".split("_"),weekdaysMin:"нд_пн_вт_ср_чт_пт_сб".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"D.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY H:mm",LLLL:"dddd, D MMMM YYYY H:mm"},calendar:{sameDay:"[Днес в] LT",nextDay:"[Утре в] LT",nextWeek:"dddd [в] LT",lastDay:"[Вчера в] LT",lastWeek:function(){switch(this.day()){case 0:case 3:case 6:return"[Миналата] dddd [в] LT";case 1:case 2:case 4:case 5:return"[Миналия] dddd [в] LT"}},sameElse:"L"},relativeTime:{future:"след %s",past:"преди %s",s:"няколко секунди",ss:"%d секунди",m:"минута",mm:"%d минути",h:"час",hh:"%d часа",d:"ден",dd:"%d дена",w:"седмица",ww:"%d седмици",M:"месец",MM:"%d месеца",y:"година",yy:"%d години"},dayOfMonthOrdinalParse:/\d{1,2}-(ев|ен|ти|ви|ри|ми)/,ordinal:function(e){var t=e%10,n=e%100;return 0===e?e+"-ев":0===n?e+"-ен":n>10&&n<20?e+"-ти":1===t?e+"-ви":2===t?e+"-ри":7===t||8===t?e+"-ми":e+"-ти"},week:{dow:1,doy:7}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("bm",{months:"Zanwuyekalo_Fewuruyekalo_Marisikalo_Awirilikalo_Mɛkalo_Zuwɛnkalo_Zuluyekalo_Utikalo_Sɛtanburukalo_ɔkutɔburukalo_Nowanburukalo_Desanburukalo".split("_"),monthsShort:"Zan_Few_Mar_Awi_Mɛ_Zuw_Zul_Uti_Sɛt_ɔku_Now_Des".split("_"),weekdays:"Kari_Ntɛnɛn_Tarata_Araba_Alamisa_Juma_Sibiri".split("_"),weekdaysShort:"Kar_Ntɛ_Tar_Ara_Ala_Jum_Sib".split("_"),weekdaysMin:"Ka_Nt_Ta_Ar_Al_Ju_Si".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"MMMM [tile] D [san] YYYY",LLL:"MMMM [tile] D [san] YYYY [lɛrɛ] HH:mm",LLLL:"dddd MMMM [tile] D [san] YYYY [lɛrɛ] HH:mm"},calendar:{sameDay:"[Bi lɛrɛ] LT",nextDay:"[Sini lɛrɛ] LT",nextWeek:"dddd [don lɛrɛ] LT",lastDay:"[Kunu lɛrɛ] LT",lastWeek:"dddd [tɛmɛnen lɛrɛ] LT",sameElse:"L"},relativeTime:{future:"%s kɔnɔ",past:"a bɛ %s bɔ",s:"sanga dama dama",ss:"sekondi %d",m:"miniti kelen",mm:"miniti %d",h:"lɛrɛ kelen",hh:"lɛrɛ %d",d:"tile kelen",dd:"tile %d",M:"kalo kelen",MM:"kalo %d",y:"san kelen",yy:"san %d"},week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t={1:"১",2:"২",3:"৩",4:"৪",5:"৫",6:"৬",7:"৭",8:"৮",9:"৯",0:"০"},n={"১":"1","২":"2","৩":"3","৪":"4","৫":"5","৬":"6","৭":"7","৮":"8","৯":"9","০":"0"};e.defineLocale("bn",{months:"জানুয়ারি_ফেব্রুয়ারি_মার্চ_এপ্রিল_মে_জুন_জুলাই_আগস্ট_সেপ্টেম্বর_অক্টোবর_নভেম্বর_ডিসেম্বর".split("_"),monthsShort:"জানু_ফেব্রু_মার্চ_এপ্রিল_মে_জুন_জুলাই_আগস্ট_সেপ্ট_অক্টো_নভে_ডিসে".split("_"),weekdays:"রবিবার_সোমবার_মঙ্গলবার_বুধবার_বৃহস্পতিবার_শুক্রবার_শনিবার".split("_"),weekdaysShort:"রবি_সোম_মঙ্গল_বুধ_বৃহস্পতি_শুক্র_শনি".split("_"),weekdaysMin:"রবি_সোম_মঙ্গল_বুধ_বৃহ_শুক্র_শনি".split("_"),longDateFormat:{LT:"A h:mm সময়",LTS:"A h:mm:ss সময়",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm সময়",LLLL:"dddd, D MMMM YYYY, A h:mm সময়"},calendar:{sameDay:"[আজ] LT",nextDay:"[আগামীকাল] LT",nextWeek:"dddd, LT",lastDay:"[গতকাল] LT",lastWeek:"[গত] dddd, LT",sameElse:"L"},relativeTime:{future:"%s পরে",past:"%s আগে",s:"কয়েক সেকেন্ড",ss:"%d সেকেন্ড",m:"এক মিনিট",mm:"%d মিনিট",h:"এক ঘন্টা",hh:"%d ঘন্টা",d:"এক দিন",dd:"%d দিন",M:"এক মাস",MM:"%d মাস",y:"এক বছর",yy:"%d বছর"},preparse:function(e){return e.replace(/[১২৩৪৫৬৭৮৯০]/g,(function(e){return n[e]}))},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]}))},meridiemParse:/রাত|সকাল|দুপুর|বিকাল|রাত/,meridiemHour:function(e,t){return 12===e&&(e=0),"রাত"===t&&e>=4||"দুপুর"===t&&e<5||"বিকাল"===t?e+12:e},meridiem:function(e,t,n){return e<4?"রাত":e<10?"সকাল":e<17?"দুপুর":e<20?"বিকাল":"রাত"},week:{dow:0,doy:6}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t={1:"১",2:"২",3:"৩",4:"৪",5:"৫",6:"৬",7:"৭",8:"৮",9:"৯",0:"০"},n={"১":"1","২":"2","৩":"3","৪":"4","৫":"5","৬":"6","৭":"7","৮":"8","৯":"9","০":"0"};e.defineLocale("bn-bd",{months:"জানুয়ারি_ফেব্রুয়ারি_মার্চ_এপ্রিল_মে_জুন_জুলাই_আগস্ট_সেপ্টেম্বর_অক্টোবর_নভেম্বর_ডিসেম্বর".split("_"),monthsShort:"জানু_ফেব্রু_মার্চ_এপ্রিল_মে_জুন_জুলাই_আগস্ট_সেপ্ট_অক্টো_নভে_ডিসে".split("_"),weekdays:"রবিবার_সোমবার_মঙ্গলবার_বুধবার_বৃহস্পতিবার_শুক্রবার_শনিবার".split("_"),weekdaysShort:"রবি_সোম_মঙ্গল_বুধ_বৃহস্পতি_শুক্র_শনি".split("_"),weekdaysMin:"রবি_সোম_মঙ্গল_বুধ_বৃহ_শুক্র_শনি".split("_"),longDateFormat:{LT:"A h:mm সময়",LTS:"A h:mm:ss সময়",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm সময়",LLLL:"dddd, D MMMM YYYY, A h:mm সময়"},calendar:{sameDay:"[আজ] LT",nextDay:"[আগামীকাল] LT",nextWeek:"dddd, LT",lastDay:"[গতকাল] LT",lastWeek:"[গত] dddd, LT",sameElse:"L"},relativeTime:{future:"%s পরে",past:"%s আগে",s:"কয়েক সেকেন্ড",ss:"%d সেকেন্ড",m:"এক মিনিট",mm:"%d মিনিট",h:"এক ঘন্টা",hh:"%d ঘন্টা",d:"এক দিন",dd:"%d দিন",M:"এক মাস",MM:"%d মাস",y:"এক বছর",yy:"%d বছর"},preparse:function(e){return e.replace(/[১২৩৪৫৬৭৮৯০]/g,(function(e){return n[e]}))},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]}))},meridiemParse:/রাত|ভোর|সকাল|দুপুর|বিকাল|সন্ধ্যা|রাত/,meridiemHour:function(e,t){return 12===e&&(e=0),"রাত"===t?e<4?e:e+12:"ভোর"===t||"সকাল"===t?e:"দুপুর"===t?e>=3?e:e+12:"বিকাল"===t||"সন্ধ্যা"===t?e+12:void 0},meridiem:function(e,t,n){return e<4?"রাত":e<6?"ভোর":e<12?"সকাল":e<15?"দুপুর":e<18?"বিকাল":e<20?"সন্ধ্যা":"রাত"},week:{dow:0,doy:6}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t={1:"༡",2:"༢",3:"༣",4:"༤",5:"༥",6:"༦",7:"༧",8:"༨",9:"༩",0:"༠"},n={"༡":"1","༢":"2","༣":"3","༤":"4","༥":"5","༦":"6","༧":"7","༨":"8","༩":"9","༠":"0"};e.defineLocale("bo",{months:"ཟླ་བ་དང་པོ_ཟླ་བ་གཉིས་པ_ཟླ་བ་གསུམ་པ_ཟླ་བ་བཞི་པ_ཟླ་བ་ལྔ་པ_ཟླ་བ་དྲུག་པ_ཟླ་བ་བདུན་པ_ཟླ་བ་བརྒྱད་པ_ཟླ་བ་དགུ་པ_ཟླ་བ་བཅུ་པ_ཟླ་བ་བཅུ་གཅིག་པ_ཟླ་བ་བཅུ་གཉིས་པ".split("_"),monthsShort:"ཟླ་1_ཟླ་2_ཟླ་3_ཟླ་4_ཟླ་5_ཟླ་6_ཟླ་7_ཟླ་8_ཟླ་9_ཟླ་10_ཟླ་11_ཟླ་12".split("_"),monthsShortRegex:/^(ཟླ་\d{1,2})/,monthsParseExact:!0,weekdays:"གཟའ་ཉི་མ་_གཟའ་ཟླ་བ་_གཟའ་མིག་དམར་_གཟའ་ལྷག་པ་_གཟའ་ཕུར་བུ_གཟའ་པ་སངས་_གཟའ་སྤེན་པ་".split("_"),weekdaysShort:"ཉི་མ་_ཟླ་བ་_མིག་དམར་_ལྷག་པ་_ཕུར་བུ_པ་སངས་_སྤེན་པ་".split("_"),weekdaysMin:"ཉི_ཟླ_མིག_ལྷག_ཕུར_སངས_སྤེན".split("_"),longDateFormat:{LT:"A h:mm",LTS:"A h:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm",LLLL:"dddd, D MMMM YYYY, A h:mm"},calendar:{sameDay:"[དི་རིང] LT",nextDay:"[སང་ཉིན] LT",nextWeek:"[བདུན་ཕྲག་རྗེས་མ], LT",lastDay:"[ཁ་སང] LT",lastWeek:"[བདུན་ཕྲག་མཐའ་མ] dddd, LT",sameElse:"L"},relativeTime:{future:"%s ལ་",past:"%s སྔན་ལ",s:"ལམ་སང",ss:"%d སྐར་ཆ།",m:"སྐར་མ་གཅིག",mm:"%d སྐར་མ",h:"ཆུ་ཚོད་གཅིག",hh:"%d ཆུ་ཚོད",d:"ཉིན་གཅིག",dd:"%d ཉིན་",M:"ཟླ་བ་གཅིག",MM:"%d ཟླ་བ",y:"ལོ་གཅིག",yy:"%d ལོ"},preparse:function(e){return e.replace(/[༡༢༣༤༥༦༧༨༩༠]/g,(function(e){return n[e]}))},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]}))},meridiemParse:/མཚན་མོ|ཞོགས་ཀས|ཉིན་གུང|དགོང་དག|མཚན་མོ/,meridiemHour:function(e,t){return 12===e&&(e=0),"མཚན་མོ"===t&&e>=4||"ཉིན་གུང"===t&&e<5||"དགོང་དག"===t?e+12:e},meridiem:function(e,t,n){return e<4?"མཚན་མོ":e<10?"ཞོགས་ཀས":e<17?"ཉིན་གུང":e<20?"དགོང་དག":"མཚན་མོ"},week:{dow:0,doy:6}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
function t(e,t,n){return e+" "+function(e,t){return 2===t?function(e){var t={m:"v",b:"v",d:"z"};return void 0===t[e.charAt(0)]?e:t[e.charAt(0)]+e.substring(1)}(e):e}({mm:"munutenn",MM:"miz",dd:"devezh"}[n],e)}var n=[/^gen/i,/^c[ʼ\']hwe/i,/^meu/i,/^ebr/i,/^mae/i,/^(mez|eve)/i,/^gou/i,/^eos/i,/^gwe/i,/^her/i,/^du/i,/^ker/i],r=/^(genver|c[ʼ\']hwevrer|meurzh|ebrel|mae|mezheven|gouere|eost|gwengolo|here|du|kerzu|gen|c[ʼ\']hwe|meu|ebr|mae|eve|gou|eos|gwe|her|du|ker)/i,i=[/^Su/i,/^Lu/i,/^Me([^r]|$)/i,/^Mer/i,/^Ya/i,/^Gw/i,/^Sa/i];e.defineLocale("br",{months:"Genver_Cʼhwevrer_Meurzh_Ebrel_Mae_Mezheven_Gouere_Eost_Gwengolo_Here_Du_Kerzu".split("_"),monthsShort:"Gen_Cʼhwe_Meu_Ebr_Mae_Eve_Gou_Eos_Gwe_Her_Du_Ker".split("_"),weekdays:"Sul_Lun_Meurzh_Mercʼher_Yaou_Gwener_Sadorn".split("_"),weekdaysShort:"Sul_Lun_Meu_Mer_Yao_Gwe_Sad".split("_"),weekdaysMin:"Su_Lu_Me_Mer_Ya_Gw_Sa".split("_"),weekdaysParse:i,fullWeekdaysParse:[/^sul/i,/^lun/i,/^meurzh/i,/^merc[ʼ\']her/i,/^yaou/i,/^gwener/i,/^sadorn/i],shortWeekdaysParse:[/^Sul/i,/^Lun/i,/^Meu/i,/^Mer/i,/^Yao/i,/^Gwe/i,/^Sad/i],minWeekdaysParse:i,monthsRegex:r,monthsShortRegex:r,monthsStrictRegex:/^(genver|c[ʼ\']hwevrer|meurzh|ebrel|mae|mezheven|gouere|eost|gwengolo|here|du|kerzu)/i,monthsShortStrictRegex:/^(gen|c[ʼ\']hwe|meu|ebr|mae|eve|gou|eos|gwe|her|du|ker)/i,monthsParse:n,longMonthsParse:n,shortMonthsParse:n,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D [a viz] MMMM YYYY",LLL:"D [a viz] MMMM YYYY HH:mm",LLLL:"dddd, D [a viz] MMMM YYYY HH:mm"},calendar:{sameDay:"[Hiziv da] LT",nextDay:"[Warcʼhoazh da] LT",nextWeek:"dddd [da] LT",lastDay:"[Decʼh da] LT",lastWeek:"dddd [paset da] LT",sameElse:"L"},relativeTime:{future:"a-benn %s",past:"%s ʼzo",s:"un nebeud segondennoù",ss:"%d eilenn",m:"ur vunutenn",mm:t,h:"un eur",hh:"%d eur",d:"un devezh",dd:t,M:"ur miz",MM:t,y:"ur bloaz",yy:function(e){switch(function e(t){return t>9?e(t%10):t}(e)){case 1:case 3:case 4:case 5:case 9:return e+" bloaz";default:return e+" vloaz"}}},dayOfMonthOrdinalParse:/\d{1,2}(añ|vet)/,ordinal:function(e){return e+(1===e?"añ":"vet")},week:{dow:1,doy:4},meridiemParse:/a.m.|g.m./,isPM:function(e){return"g.m."===e},meridiem:function(e,t,n){return e<12?"a.m.":"g.m."}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
function t(e,t,n){var r=e+" ";switch(n){case"ss":return r+=1===e?"sekunda":2===e||3===e||4===e?"sekunde":"sekundi";case"m":return t?"jedna minuta":"jedne minute";case"mm":return r+=1===e?"minuta":2===e||3===e||4===e?"minute":"minuta";case"h":return t?"jedan sat":"jednog sata";case"hh":return r+=1===e?"sat":2===e||3===e||4===e?"sata":"sati";case"dd":return r+=1===e?"dan":"dana";case"MM":return r+=1===e?"mjesec":2===e||3===e||4===e?"mjeseca":"mjeseci";case"yy":return r+=1===e?"godina":2===e||3===e||4===e?"godine":"godina"}}e.defineLocale("bs",{months:"januar_februar_mart_april_maj_juni_juli_august_septembar_oktobar_novembar_decembar".split("_"),monthsShort:"jan._feb._mar._apr._maj._jun._jul._aug._sep._okt._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"nedjelja_ponedjeljak_utorak_srijeda_četvrtak_petak_subota".split("_"),weekdaysShort:"ned._pon._uto._sri._čet._pet._sub.".split("_"),weekdaysMin:"ne_po_ut_sr_če_pe_su".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd, D. MMMM YYYY H:mm"},calendar:{sameDay:"[danas u] LT",nextDay:"[sutra u] LT",nextWeek:function(){switch(this.day()){case 0:return"[u] [nedjelju] [u] LT";case 3:return"[u] [srijedu] [u] LT";case 6:return"[u] [subotu] [u] LT";case 1:case 2:case 4:case 5:return"[u] dddd [u] LT"}},lastDay:"[jučer u] LT",lastWeek:function(){switch(this.day()){case 0:case 3:return"[prošlu] dddd [u] LT";case 6:return"[prošle] [subote] [u] LT";case 1:case 2:case 4:case 5:return"[prošli] dddd [u] LT"}},sameElse:"L"},relativeTime:{future:"za %s",past:"prije %s",s:"par sekundi",ss:t,m:t,mm:t,h:t,hh:t,d:"dan",dd:t,M:"mjesec",MM:t,y:"godinu",yy:t},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:7}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("ca",{months:{standalone:"gener_febrer_març_abril_maig_juny_juliol_agost_setembre_octubre_novembre_desembre".split("_"),format:"de gener_de febrer_de març_d'abril_de maig_de juny_de juliol_d'agost_de setembre_d'octubre_de novembre_de desembre".split("_"),isFormat:/D[oD]?(\s)+MMMM/},monthsShort:"gen._febr._març_abr._maig_juny_jul._ag._set._oct._nov._des.".split("_"),monthsParseExact:!0,weekdays:"diumenge_dilluns_dimarts_dimecres_dijous_divendres_dissabte".split("_"),weekdaysShort:"dg._dl._dt._dc._dj._dv._ds.".split("_"),weekdaysMin:"dg_dl_dt_dc_dj_dv_ds".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM [de] YYYY",ll:"D MMM YYYY",LLL:"D MMMM [de] YYYY [a les] H:mm",lll:"D MMM YYYY, H:mm",LLLL:"dddd D MMMM [de] YYYY [a les] H:mm",llll:"ddd D MMM YYYY, H:mm"},calendar:{sameDay:function(){return"[avui a "+(1!==this.hours()?"les":"la")+"] LT"},nextDay:function(){return"[demà a "+(1!==this.hours()?"les":"la")+"] LT"},nextWeek:function(){return"dddd [a "+(1!==this.hours()?"les":"la")+"] LT"},lastDay:function(){return"[ahir a "+(1!==this.hours()?"les":"la")+"] LT"},lastWeek:function(){return"[el] dddd [passat a "+(1!==this.hours()?"les":"la")+"] LT"},sameElse:"L"},relativeTime:{future:"d'aquí %s",past:"fa %s",s:"uns segons",ss:"%d segons",m:"un minut",mm:"%d minuts",h:"una hora",hh:"%d hores",d:"un dia",dd:"%d dies",M:"un mes",MM:"%d mesos",y:"un any",yy:"%d anys"},dayOfMonthOrdinalParse:/\d{1,2}(r|n|t|è|a)/,ordinal:function(e,t){var n=1===e?"r":2===e?"n":3===e?"r":4===e?"t":"è";return"w"!==t&&"W"!==t||(n="a"),e+n},week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t="leden_únor_březen_duben_květen_červen_červenec_srpen_září_říjen_listopad_prosinec".split("_"),n="led_úno_bře_dub_kvě_čvn_čvc_srp_zář_říj_lis_pro".split("_"),r=[/^led/i,/^úno/i,/^bře/i,/^dub/i,/^kvě/i,/^(čvn|červen$|června)/i,/^(čvc|červenec|července)/i,/^srp/i,/^zář/i,/^říj/i,/^lis/i,/^pro/i],i=/^(leden|únor|březen|duben|květen|červenec|července|červen|června|srpen|září|říjen|listopad|prosinec|led|úno|bře|dub|kvě|čvn|čvc|srp|zář|říj|lis|pro)/i;function o(e){return e>1&&e<5&&1!=~~(e/10)}function a(e,t,n,r){var i=e+" ";switch(n){case"s":return t||r?"pár sekund":"pár sekundami";case"ss":return t||r?i+(o(e)?"sekundy":"sekund"):i+"sekundami";case"m":return t?"minuta":r?"minutu":"minutou";case"mm":return t||r?i+(o(e)?"minuty":"minut"):i+"minutami";case"h":return t?"hodina":r?"hodinu":"hodinou";case"hh":return t||r?i+(o(e)?"hodiny":"hodin"):i+"hodinami";case"d":return t||r?"den":"dnem";case"dd":return t||r?i+(o(e)?"dny":"dní"):i+"dny";case"M":return t||r?"měsíc":"měsícem";case"MM":return t||r?i+(o(e)?"měsíce":"měsíců"):i+"měsíci";case"y":return t||r?"rok":"rokem";case"yy":return t||r?i+(o(e)?"roky":"let"):i+"lety"}}e.defineLocale("cs",{months:t,monthsShort:n,monthsRegex:i,monthsShortRegex:i,monthsStrictRegex:/^(leden|ledna|února|únor|březen|března|duben|dubna|květen|května|červenec|července|červen|června|srpen|srpna|září|říjen|října|listopadu|listopad|prosinec|prosince)/i,monthsShortStrictRegex:/^(led|úno|bře|dub|kvě|čvn|čvc|srp|zář|říj|lis|pro)/i,monthsParse:r,longMonthsParse:r,shortMonthsParse:r,weekdays:"neděle_pondělí_úterý_středa_čtvrtek_pátek_sobota".split("_"),weekdaysShort:"ne_po_út_st_čt_pá_so".split("_"),weekdaysMin:"ne_po_út_st_čt_pá_so".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd D. MMMM YYYY H:mm",l:"D. M. YYYY"},calendar:{sameDay:"[dnes v] LT",nextDay:"[zítra v] LT",nextWeek:function(){switch(this.day()){case 0:return"[v neděli v] LT";case 1:case 2:return"[v] dddd [v] LT";case 3:return"[ve středu v] LT";case 4:return"[ve čtvrtek v] LT";case 5:return"[v pátek v] LT";case 6:return"[v sobotu v] LT"}},lastDay:"[včera v] LT",lastWeek:function(){switch(this.day()){case 0:return"[minulou neděli v] LT";case 1:case 2:return"[minulé] dddd [v] LT";case 3:return"[minulou středu v] LT";case 4:case 5:return"[minulý] dddd [v] LT";case 6:return"[minulou sobotu v] LT"}},sameElse:"L"},relativeTime:{future:"za %s",past:"před %s",s:a,ss:a,m:a,mm:a,h:a,hh:a,d:a,dd:a,M:a,MM:a,y:a,yy:a},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("cv",{months:"кӑрлач_нарӑс_пуш_ака_май_ҫӗртме_утӑ_ҫурла_авӑн_юпа_чӳк_раштав".split("_"),monthsShort:"кӑр_нар_пуш_ака_май_ҫӗр_утӑ_ҫур_авн_юпа_чӳк_раш".split("_"),weekdays:"вырсарникун_тунтикун_ытларикун_юнкун_кӗҫнерникун_эрнекун_шӑматкун".split("_"),weekdaysShort:"выр_тун_ытл_юн_кӗҫ_эрн_шӑм".split("_"),weekdaysMin:"вр_тн_ыт_юн_кҫ_эр_шм".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD-MM-YYYY",LL:"YYYY [ҫулхи] MMMM [уйӑхӗн] D[-мӗшӗ]",LLL:"YYYY [ҫулхи] MMMM [уйӑхӗн] D[-мӗшӗ], HH:mm",LLLL:"dddd, YYYY [ҫулхи] MMMM [уйӑхӗн] D[-мӗшӗ], HH:mm"},calendar:{sameDay:"[Паян] LT [сехетре]",nextDay:"[Ыран] LT [сехетре]",lastDay:"[Ӗнер] LT [сехетре]",nextWeek:"[Ҫитес] dddd LT [сехетре]",lastWeek:"[Иртнӗ] dddd LT [сехетре]",sameElse:"L"},relativeTime:{future:function(e){return e+(/сехет$/i.exec(e)?"рен":/ҫул$/i.exec(e)?"тан":"ран")},past:"%s каялла",s:"пӗр-ик ҫеккунт",ss:"%d ҫеккунт",m:"пӗр минут",mm:"%d минут",h:"пӗр сехет",hh:"%d сехет",d:"пӗр кун",dd:"%d кун",M:"пӗр уйӑх",MM:"%d уйӑх",y:"пӗр ҫул",yy:"%d ҫул"},dayOfMonthOrdinalParse:/\d{1,2}-мӗш/,ordinal:"%d-мӗш",week:{dow:1,doy:7}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("cy",{months:"Ionawr_Chwefror_Mawrth_Ebrill_Mai_Mehefin_Gorffennaf_Awst_Medi_Hydref_Tachwedd_Rhagfyr".split("_"),monthsShort:"Ion_Chwe_Maw_Ebr_Mai_Meh_Gor_Aws_Med_Hyd_Tach_Rhag".split("_"),weekdays:"Dydd Sul_Dydd Llun_Dydd Mawrth_Dydd Mercher_Dydd Iau_Dydd Gwener_Dydd Sadwrn".split("_"),weekdaysShort:"Sul_Llun_Maw_Mer_Iau_Gwe_Sad".split("_"),weekdaysMin:"Su_Ll_Ma_Me_Ia_Gw_Sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Heddiw am] LT",nextDay:"[Yfory am] LT",nextWeek:"dddd [am] LT",lastDay:"[Ddoe am] LT",lastWeek:"dddd [diwethaf am] LT",sameElse:"L"},relativeTime:{future:"mewn %s",past:"%s yn ôl",s:"ychydig eiliadau",ss:"%d eiliad",m:"munud",mm:"%d munud",h:"awr",hh:"%d awr",d:"diwrnod",dd:"%d diwrnod",M:"mis",MM:"%d mis",y:"blwyddyn",yy:"%d flynedd"},dayOfMonthOrdinalParse:/\d{1,2}(fed|ain|af|il|ydd|ed|eg)/,ordinal:function(e){var t="";return e>20?t=40===e||50===e||60===e||80===e||100===e?"fed":"ain":e>0&&(t=["","af","il","ydd","ydd","ed","ed","ed","fed","fed","fed","eg","fed","eg","eg","fed","eg","eg","fed","eg","fed"][e]),e+t},week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("da",{months:"januar_februar_marts_april_maj_juni_juli_august_september_oktober_november_december".split("_"),monthsShort:"jan_feb_mar_apr_maj_jun_jul_aug_sep_okt_nov_dec".split("_"),weekdays:"søndag_mandag_tirsdag_onsdag_torsdag_fredag_lørdag".split("_"),weekdaysShort:"søn_man_tir_ons_tor_fre_lør".split("_"),weekdaysMin:"sø_ma_ti_on_to_fr_lø".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY HH:mm",LLLL:"dddd [d.] D. MMMM YYYY [kl.] HH:mm"},calendar:{sameDay:"[i dag kl.] LT",nextDay:"[i morgen kl.] LT",nextWeek:"på dddd [kl.] LT",lastDay:"[i går kl.] LT",lastWeek:"[i] dddd[s kl.] LT",sameElse:"L"},relativeTime:{future:"om %s",past:"%s siden",s:"få sekunder",ss:"%d sekunder",m:"et minut",mm:"%d minutter",h:"en time",hh:"%d timer",d:"en dag",dd:"%d dage",M:"en måned",MM:"%d måneder",y:"et år",yy:"%d år"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
function t(e,t,n,r){var i={m:["eine Minute","einer Minute"],h:["eine Stunde","einer Stunde"],d:["ein Tag","einem Tag"],dd:[e+" Tage",e+" Tagen"],w:["eine Woche","einer Woche"],M:["ein Monat","einem Monat"],MM:[e+" Monate",e+" Monaten"],y:["ein Jahr","einem Jahr"],yy:[e+" Jahre",e+" Jahren"]};return t?i[n][0]:i[n][1]}e.defineLocale("de",{months:"Januar_Februar_März_April_Mai_Juni_Juli_August_September_Oktober_November_Dezember".split("_"),monthsShort:"Jan._Feb._März_Apr._Mai_Juni_Juli_Aug._Sep._Okt._Nov._Dez.".split("_"),monthsParseExact:!0,weekdays:"Sonntag_Montag_Dienstag_Mittwoch_Donnerstag_Freitag_Samstag".split("_"),weekdaysShort:"So._Mo._Di._Mi._Do._Fr._Sa.".split("_"),weekdaysMin:"So_Mo_Di_Mi_Do_Fr_Sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY HH:mm",LLLL:"dddd, D. MMMM YYYY HH:mm"},calendar:{sameDay:"[heute um] LT [Uhr]",sameElse:"L",nextDay:"[morgen um] LT [Uhr]",nextWeek:"dddd [um] LT [Uhr]",lastDay:"[gestern um] LT [Uhr]",lastWeek:"[letzten] dddd [um] LT [Uhr]"},relativeTime:{future:"in %s",past:"vor %s",s:"ein paar Sekunden",ss:"%d Sekunden",m:t,mm:"%d Minuten",h:t,hh:"%d Stunden",d:t,dd:t,w:t,ww:"%d Wochen",M:t,MM:t,y:t,yy:t},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
function t(e,t,n,r){var i={m:["eine Minute","einer Minute"],h:["eine Stunde","einer Stunde"],d:["ein Tag","einem Tag"],dd:[e+" Tage",e+" Tagen"],w:["eine Woche","einer Woche"],M:["ein Monat","einem Monat"],MM:[e+" Monate",e+" Monaten"],y:["ein Jahr","einem Jahr"],yy:[e+" Jahre",e+" Jahren"]};return t?i[n][0]:i[n][1]}e.defineLocale("de-at",{months:"Jänner_Februar_März_April_Mai_Juni_Juli_August_September_Oktober_November_Dezember".split("_"),monthsShort:"Jän._Feb._März_Apr._Mai_Juni_Juli_Aug._Sep._Okt._Nov._Dez.".split("_"),monthsParseExact:!0,weekdays:"Sonntag_Montag_Dienstag_Mittwoch_Donnerstag_Freitag_Samstag".split("_"),weekdaysShort:"So._Mo._Di._Mi._Do._Fr._Sa.".split("_"),weekdaysMin:"So_Mo_Di_Mi_Do_Fr_Sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY HH:mm",LLLL:"dddd, D. MMMM YYYY HH:mm"},calendar:{sameDay:"[heute um] LT [Uhr]",sameElse:"L",nextDay:"[morgen um] LT [Uhr]",nextWeek:"dddd [um] LT [Uhr]",lastDay:"[gestern um] LT [Uhr]",lastWeek:"[letzten] dddd [um] LT [Uhr]"},relativeTime:{future:"in %s",past:"vor %s",s:"ein paar Sekunden",ss:"%d Sekunden",m:t,mm:"%d Minuten",h:t,hh:"%d Stunden",d:t,dd:t,w:t,ww:"%d Wochen",M:t,MM:t,y:t,yy:t},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
function t(e,t,n,r){var i={m:["eine Minute","einer Minute"],h:["eine Stunde","einer Stunde"],d:["ein Tag","einem Tag"],dd:[e+" Tage",e+" Tagen"],w:["eine Woche","einer Woche"],M:["ein Monat","einem Monat"],MM:[e+" Monate",e+" Monaten"],y:["ein Jahr","einem Jahr"],yy:[e+" Jahre",e+" Jahren"]};return t?i[n][0]:i[n][1]}e.defineLocale("de-ch",{months:"Januar_Februar_März_April_Mai_Juni_Juli_August_September_Oktober_November_Dezember".split("_"),monthsShort:"Jan._Feb._März_Apr._Mai_Juni_Juli_Aug._Sep._Okt._Nov._Dez.".split("_"),monthsParseExact:!0,weekdays:"Sonntag_Montag_Dienstag_Mittwoch_Donnerstag_Freitag_Samstag".split("_"),weekdaysShort:"So_Mo_Di_Mi_Do_Fr_Sa".split("_"),weekdaysMin:"So_Mo_Di_Mi_Do_Fr_Sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY HH:mm",LLLL:"dddd, D. MMMM YYYY HH:mm"},calendar:{sameDay:"[heute um] LT [Uhr]",sameElse:"L",nextDay:"[morgen um] LT [Uhr]",nextWeek:"dddd [um] LT [Uhr]",lastDay:"[gestern um] LT [Uhr]",lastWeek:"[letzten] dddd [um] LT [Uhr]"},relativeTime:{future:"in %s",past:"vor %s",s:"ein paar Sekunden",ss:"%d Sekunden",m:t,mm:"%d Minuten",h:t,hh:"%d Stunden",d:t,dd:t,w:t,ww:"%d Wochen",M:t,MM:t,y:t,yy:t},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t=["ޖެނުއަރީ","ފެބްރުއަރީ","މާރިޗު","އޭޕްރީލު","މޭ","ޖޫން","ޖުލައި","އޯގަސްޓު","ސެޕްޓެމްބަރު","އޮކްޓޯބަރު","ނޮވެމްބަރު","ޑިސެމްބަރު"],n=["އާދިއްތަ","ހޯމަ","އަންގާރަ","ބުދަ","ބުރާސްފަތި","ހުކުރު","ހޮނިހިރު"];e.defineLocale("dv",{months:t,monthsShort:t,weekdays:n,weekdaysShort:n,weekdaysMin:"އާދި_ހޯމަ_އަން_ބުދަ_ބުރާ_ހުކު_ހޮނި".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"D/M/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},meridiemParse:/މކ|މފ/,isPM:function(e){return"މފ"===e},meridiem:function(e,t,n){return e<12?"މކ":"މފ"},calendar:{sameDay:"[މިއަދު] LT",nextDay:"[މާދަމާ] LT",nextWeek:"dddd LT",lastDay:"[އިއްޔެ] LT",lastWeek:"[ފާއިތުވި] dddd LT",sameElse:"L"},relativeTime:{future:"ތެރޭގައި %s",past:"ކުރިން %s",s:"ސިކުންތުކޮޅެއް",ss:"d% ސިކުންތު",m:"މިނިޓެއް",mm:"މިނިޓު %d",h:"ގަޑިއިރެއް",hh:"ގަޑިއިރު %d",d:"ދުވަހެއް",dd:"ދުވަސް %d",M:"މަހެއް",MM:"މަސް %d",y:"އަހަރެއް",yy:"އަހަރު %d"},preparse:function(e){return e.replace(/،/g,",")},postformat:function(e){return e.replace(/,/g,"،")},week:{dow:7,doy:12}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("el",{monthsNominativeEl:"Ιανουάριος_Φεβρουάριος_Μάρτιος_Απρίλιος_Μάιος_Ιούνιος_Ιούλιος_Αύγουστος_Σεπτέμβριος_Οκτώβριος_Νοέμβριος_Δεκέμβριος".split("_"),monthsGenitiveEl:"Ιανουαρίου_Φεβρουαρίου_Μαρτίου_Απριλίου_Μαΐου_Ιουνίου_Ιουλίου_Αυγούστου_Σεπτεμβρίου_Οκτωβρίου_Νοεμβρίου_Δεκεμβρίου".split("_"),months:function(e,t){return e?"string"==typeof t&&/D/.test(t.substring(0,t.indexOf("MMMM")))?this._monthsGenitiveEl[e.month()]:this._monthsNominativeEl[e.month()]:this._monthsNominativeEl},monthsShort:"Ιαν_Φεβ_Μαρ_Απρ_Μαϊ_Ιουν_Ιουλ_Αυγ_Σεπ_Οκτ_Νοε_Δεκ".split("_"),weekdays:"Κυριακή_Δευτέρα_Τρίτη_Τετάρτη_Πέμπτη_Παρασκευή_Σάββατο".split("_"),weekdaysShort:"Κυρ_Δευ_Τρι_Τετ_Πεμ_Παρ_Σαβ".split("_"),weekdaysMin:"Κυ_Δε_Τρ_Τε_Πε_Πα_Σα".split("_"),meridiem:function(e,t,n){return e>11?n?"μμ":"ΜΜ":n?"πμ":"ΠΜ"},isPM:function(e){return"μ"===(e+"").toLowerCase()[0]},meridiemParse:/[ΠΜ]\.?Μ?\.?/i,longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY h:mm A",LLLL:"dddd, D MMMM YYYY h:mm A"},calendarEl:{sameDay:"[Σήμερα {}] LT",nextDay:"[Αύριο {}] LT",nextWeek:"dddd [{}] LT",lastDay:"[Χθες {}] LT",lastWeek:function(){switch(this.day()){case 6:return"[το προηγούμενο] dddd [{}] LT";default:return"[την προηγούμενη] dddd [{}] LT"}},sameElse:"L"},calendar:function(e,t){var n,r=this._calendarEl[e],i=t&&t.hours();return n=r,("undefined"!=typeof Function&&n instanceof Function||"[object Function]"===Object.prototype.toString.call(n))&&(r=r.apply(t)),r.replace("{}",i%12==1?"στη":"στις")},relativeTime:{future:"σε %s",past:"%s πριν",s:"λίγα δευτερόλεπτα",ss:"%d δευτερόλεπτα",m:"ένα λεπτό",mm:"%d λεπτά",h:"μία ώρα",hh:"%d ώρες",d:"μία μέρα",dd:"%d μέρες",M:"ένας μήνας",MM:"%d μήνες",y:"ένας χρόνος",yy:"%d χρόνια"},dayOfMonthOrdinalParse:/\d{1,2}η/,ordinal:"%dη",week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("en-au",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY h:mm A",LLLL:"dddd, D MMMM YYYY h:mm A"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(e){var t=e%10;return e+(1==~~(e%100/10)?"th":1===t?"st":2===t?"nd":3===t?"rd":"th")},week:{dow:0,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("en-ca",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"YYYY-MM-DD",LL:"MMMM D, YYYY",LLL:"MMMM D, YYYY h:mm A",LLLL:"dddd, MMMM D, YYYY h:mm A"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(e){var t=e%10;return e+(1==~~(e%100/10)?"th":1===t?"st":2===t?"nd":3===t?"rd":"th")}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("en-gb",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(e){var t=e%10;return e+(1==~~(e%100/10)?"th":1===t?"st":2===t?"nd":3===t?"rd":"th")},week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("en-ie",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(e){var t=e%10;return e+(1==~~(e%100/10)?"th":1===t?"st":2===t?"nd":3===t?"rd":"th")},week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("en-il",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(e){var t=e%10;return e+(1==~~(e%100/10)?"th":1===t?"st":2===t?"nd":3===t?"rd":"th")}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("en-in",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY h:mm A",LLLL:"dddd, D MMMM YYYY h:mm A"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(e){var t=e%10;return e+(1==~~(e%100/10)?"th":1===t?"st":2===t?"nd":3===t?"rd":"th")},week:{dow:0,doy:6}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("en-nz",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY h:mm A",LLLL:"dddd, D MMMM YYYY h:mm A"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(e){var t=e%10;return e+(1==~~(e%100/10)?"th":1===t?"st":2===t?"nd":3===t?"rd":"th")},week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("en-sg",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(e){var t=e%10;return e+(1==~~(e%100/10)?"th":1===t?"st":2===t?"nd":3===t?"rd":"th")},week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("eo",{months:"januaro_februaro_marto_aprilo_majo_junio_julio_aŭgusto_septembro_oktobro_novembro_decembro".split("_"),monthsShort:"jan_feb_mart_apr_maj_jun_jul_aŭg_sept_okt_nov_dec".split("_"),weekdays:"dimanĉo_lundo_mardo_merkredo_ĵaŭdo_vendredo_sabato".split("_"),weekdaysShort:"dim_lun_mard_merk_ĵaŭ_ven_sab".split("_"),weekdaysMin:"di_lu_ma_me_ĵa_ve_sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"[la] D[-an de] MMMM, YYYY",LLL:"[la] D[-an de] MMMM, YYYY HH:mm",LLLL:"dddd[n], [la] D[-an de] MMMM, YYYY HH:mm",llll:"ddd, [la] D[-an de] MMM, YYYY HH:mm"},meridiemParse:/[ap]\.t\.m/i,isPM:function(e){return"p"===e.charAt(0).toLowerCase()},meridiem:function(e,t,n){return e>11?n?"p.t.m.":"P.T.M.":n?"a.t.m.":"A.T.M."},calendar:{sameDay:"[Hodiaŭ je] LT",nextDay:"[Morgaŭ je] LT",nextWeek:"dddd[n je] LT",lastDay:"[Hieraŭ je] LT",lastWeek:"[pasintan] dddd[n je] LT",sameElse:"L"},relativeTime:{future:"post %s",past:"antaŭ %s",s:"kelkaj sekundoj",ss:"%d sekundoj",m:"unu minuto",mm:"%d minutoj",h:"unu horo",hh:"%d horoj",d:"unu tago",dd:"%d tagoj",M:"unu monato",MM:"%d monatoj",y:"unu jaro",yy:"%d jaroj"},dayOfMonthOrdinalParse:/\d{1,2}a/,ordinal:"%da",week:{dow:1,doy:7}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t="ene._feb._mar._abr._may._jun._jul._ago._sep._oct._nov._dic.".split("_"),n="ene_feb_mar_abr_may_jun_jul_ago_sep_oct_nov_dic".split("_"),r=[/^ene/i,/^feb/i,/^mar/i,/^abr/i,/^may/i,/^jun/i,/^jul/i,/^ago/i,/^sep/i,/^oct/i,/^nov/i,/^dic/i],i=/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre|ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i;e.defineLocale("es",{months:"enero_febrero_marzo_abril_mayo_junio_julio_agosto_septiembre_octubre_noviembre_diciembre".split("_"),monthsShort:function(e,r){return e?/-MMM-/.test(r)?n[e.month()]:t[e.month()]:t},monthsRegex:i,monthsShortRegex:i,monthsStrictRegex:/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre)/i,monthsShortStrictRegex:/^(ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i,monthsParse:r,longMonthsParse:r,shortMonthsParse:r,weekdays:"domingo_lunes_martes_miércoles_jueves_viernes_sábado".split("_"),weekdaysShort:"dom._lun._mar._mié._jue._vie._sáb.".split("_"),weekdaysMin:"do_lu_ma_mi_ju_vi_sá".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD/MM/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY H:mm",LLLL:"dddd, D [de] MMMM [de] YYYY H:mm"},calendar:{sameDay:function(){return"[hoy a la"+(1!==this.hours()?"s":"")+"] LT"},nextDay:function(){return"[mañana a la"+(1!==this.hours()?"s":"")+"] LT"},nextWeek:function(){return"dddd [a la"+(1!==this.hours()?"s":"")+"] LT"},lastDay:function(){return"[ayer a la"+(1!==this.hours()?"s":"")+"] LT"},lastWeek:function(){return"[el] dddd [pasado a la"+(1!==this.hours()?"s":"")+"] LT"},sameElse:"L"},relativeTime:{future:"en %s",past:"hace %s",s:"unos segundos",ss:"%d segundos",m:"un minuto",mm:"%d minutos",h:"una hora",hh:"%d horas",d:"un día",dd:"%d días",w:"una semana",ww:"%d semanas",M:"un mes",MM:"%d meses",y:"un año",yy:"%d años"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,doy:4},invalidDate:"Fecha inválida"})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t="ene._feb._mar._abr._may._jun._jul._ago._sep._oct._nov._dic.".split("_"),n="ene_feb_mar_abr_may_jun_jul_ago_sep_oct_nov_dic".split("_"),r=[/^ene/i,/^feb/i,/^mar/i,/^abr/i,/^may/i,/^jun/i,/^jul/i,/^ago/i,/^sep/i,/^oct/i,/^nov/i,/^dic/i],i=/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre|ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i;e.defineLocale("es-do",{months:"enero_febrero_marzo_abril_mayo_junio_julio_agosto_septiembre_octubre_noviembre_diciembre".split("_"),monthsShort:function(e,r){return e?/-MMM-/.test(r)?n[e.month()]:t[e.month()]:t},monthsRegex:i,monthsShortRegex:i,monthsStrictRegex:/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre)/i,monthsShortStrictRegex:/^(ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i,monthsParse:r,longMonthsParse:r,shortMonthsParse:r,weekdays:"domingo_lunes_martes_miércoles_jueves_viernes_sábado".split("_"),weekdaysShort:"dom._lun._mar._mié._jue._vie._sáb.".split("_"),weekdaysMin:"do_lu_ma_mi_ju_vi_sá".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY h:mm A",LLLL:"dddd, D [de] MMMM [de] YYYY h:mm A"},calendar:{sameDay:function(){return"[hoy a la"+(1!==this.hours()?"s":"")+"] LT"},nextDay:function(){return"[mañana a la"+(1!==this.hours()?"s":"")+"] LT"},nextWeek:function(){return"dddd [a la"+(1!==this.hours()?"s":"")+"] LT"},lastDay:function(){return"[ayer a la"+(1!==this.hours()?"s":"")+"] LT"},lastWeek:function(){return"[el] dddd [pasado a la"+(1!==this.hours()?"s":"")+"] LT"},sameElse:"L"},relativeTime:{future:"en %s",past:"hace %s",s:"unos segundos",ss:"%d segundos",m:"un minuto",mm:"%d minutos",h:"una hora",hh:"%d horas",d:"un día",dd:"%d días",w:"una semana",ww:"%d semanas",M:"un mes",MM:"%d meses",y:"un año",yy:"%d años"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t="ene._feb._mar._abr._may._jun._jul._ago._sep._oct._nov._dic.".split("_"),n="ene_feb_mar_abr_may_jun_jul_ago_sep_oct_nov_dic".split("_"),r=[/^ene/i,/^feb/i,/^mar/i,/^abr/i,/^may/i,/^jun/i,/^jul/i,/^ago/i,/^sep/i,/^oct/i,/^nov/i,/^dic/i],i=/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre|ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i;e.defineLocale("es-mx",{months:"enero_febrero_marzo_abril_mayo_junio_julio_agosto_septiembre_octubre_noviembre_diciembre".split("_"),monthsShort:function(e,r){return e?/-MMM-/.test(r)?n[e.month()]:t[e.month()]:t},monthsRegex:i,monthsShortRegex:i,monthsStrictRegex:/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre)/i,monthsShortStrictRegex:/^(ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i,monthsParse:r,longMonthsParse:r,shortMonthsParse:r,weekdays:"domingo_lunes_martes_miércoles_jueves_viernes_sábado".split("_"),weekdaysShort:"dom._lun._mar._mié._jue._vie._sáb.".split("_"),weekdaysMin:"do_lu_ma_mi_ju_vi_sá".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD/MM/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY H:mm",LLLL:"dddd, D [de] MMMM [de] YYYY H:mm"},calendar:{sameDay:function(){return"[hoy a la"+(1!==this.hours()?"s":"")+"] LT"},nextDay:function(){return"[mañana a la"+(1!==this.hours()?"s":"")+"] LT"},nextWeek:function(){return"dddd [a la"+(1!==this.hours()?"s":"")+"] LT"},lastDay:function(){return"[ayer a la"+(1!==this.hours()?"s":"")+"] LT"},lastWeek:function(){return"[el] dddd [pasado a la"+(1!==this.hours()?"s":"")+"] LT"},sameElse:"L"},relativeTime:{future:"en %s",past:"hace %s",s:"unos segundos",ss:"%d segundos",m:"un minuto",mm:"%d minutos",h:"una hora",hh:"%d horas",d:"un día",dd:"%d días",w:"una semana",ww:"%d semanas",M:"un mes",MM:"%d meses",y:"un año",yy:"%d años"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:0,doy:4},invalidDate:"Fecha inválida"})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t="ene._feb._mar._abr._may._jun._jul._ago._sep._oct._nov._dic.".split("_"),n="ene_feb_mar_abr_may_jun_jul_ago_sep_oct_nov_dic".split("_"),r=[/^ene/i,/^feb/i,/^mar/i,/^abr/i,/^may/i,/^jun/i,/^jul/i,/^ago/i,/^sep/i,/^oct/i,/^nov/i,/^dic/i],i=/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre|ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i;e.defineLocale("es-us",{months:"enero_febrero_marzo_abril_mayo_junio_julio_agosto_septiembre_octubre_noviembre_diciembre".split("_"),monthsShort:function(e,r){return e?/-MMM-/.test(r)?n[e.month()]:t[e.month()]:t},monthsRegex:i,monthsShortRegex:i,monthsStrictRegex:/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre)/i,monthsShortStrictRegex:/^(ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i,monthsParse:r,longMonthsParse:r,shortMonthsParse:r,weekdays:"domingo_lunes_martes_miércoles_jueves_viernes_sábado".split("_"),weekdaysShort:"dom._lun._mar._mié._jue._vie._sáb.".split("_"),weekdaysMin:"do_lu_ma_mi_ju_vi_sá".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"MM/DD/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY h:mm A",LLLL:"dddd, D [de] MMMM [de] YYYY h:mm A"},calendar:{sameDay:function(){return"[hoy a la"+(1!==this.hours()?"s":"")+"] LT"},nextDay:function(){return"[mañana a la"+(1!==this.hours()?"s":"")+"] LT"},nextWeek:function(){return"dddd [a la"+(1!==this.hours()?"s":"")+"] LT"},lastDay:function(){return"[ayer a la"+(1!==this.hours()?"s":"")+"] LT"},lastWeek:function(){return"[el] dddd [pasado a la"+(1!==this.hours()?"s":"")+"] LT"},sameElse:"L"},relativeTime:{future:"en %s",past:"hace %s",s:"unos segundos",ss:"%d segundos",m:"un minuto",mm:"%d minutos",h:"una hora",hh:"%d horas",d:"un día",dd:"%d días",w:"una semana",ww:"%d semanas",M:"un mes",MM:"%d meses",y:"un año",yy:"%d años"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:0,doy:6}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
function t(e,t,n,r){var i={s:["mõne sekundi","mõni sekund","paar sekundit"],ss:[e+"sekundi",e+"sekundit"],m:["ühe minuti","üks minut"],mm:[e+" minuti",e+" minutit"],h:["ühe tunni","tund aega","üks tund"],hh:[e+" tunni",e+" tundi"],d:["ühe päeva","üks päev"],M:["kuu aja","kuu aega","üks kuu"],MM:[e+" kuu",e+" kuud"],y:["ühe aasta","aasta","üks aasta"],yy:[e+" aasta",e+" aastat"]};return t?i[n][2]?i[n][2]:i[n][1]:r?i[n][0]:i[n][1]}e.defineLocale("et",{months:"jaanuar_veebruar_märts_aprill_mai_juuni_juuli_august_september_oktoober_november_detsember".split("_"),monthsShort:"jaan_veebr_märts_apr_mai_juuni_juuli_aug_sept_okt_nov_dets".split("_"),weekdays:"pühapäev_esmaspäev_teisipäev_kolmapäev_neljapäev_reede_laupäev".split("_"),weekdaysShort:"P_E_T_K_N_R_L".split("_"),weekdaysMin:"P_E_T_K_N_R_L".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd, D. MMMM YYYY H:mm"},calendar:{sameDay:"[Täna,] LT",nextDay:"[Homme,] LT",nextWeek:"[Järgmine] dddd LT",lastDay:"[Eile,] LT",lastWeek:"[Eelmine] dddd LT",sameElse:"L"},relativeTime:{future:"%s pärast",past:"%s tagasi",s:t,ss:t,m:t,mm:t,h:t,hh:t,d:t,dd:"%d päeva",M:t,MM:t,y:t,yy:t},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("eu",{months:"urtarrila_otsaila_martxoa_apirila_maiatza_ekaina_uztaila_abuztua_iraila_urria_azaroa_abendua".split("_"),monthsShort:"urt._ots._mar._api._mai._eka._uzt._abu._ira._urr._aza._abe.".split("_"),monthsParseExact:!0,weekdays:"igandea_astelehena_asteartea_asteazkena_osteguna_ostirala_larunbata".split("_"),weekdaysShort:"ig._al._ar._az._og._ol._lr.".split("_"),weekdaysMin:"ig_al_ar_az_og_ol_lr".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"YYYY[ko] MMMM[ren] D[a]",LLL:"YYYY[ko] MMMM[ren] D[a] HH:mm",LLLL:"dddd, YYYY[ko] MMMM[ren] D[a] HH:mm",l:"YYYY-M-D",ll:"YYYY[ko] MMM D[a]",lll:"YYYY[ko] MMM D[a] HH:mm",llll:"ddd, YYYY[ko] MMM D[a] HH:mm"},calendar:{sameDay:"[gaur] LT[etan]",nextDay:"[bihar] LT[etan]",nextWeek:"dddd LT[etan]",lastDay:"[atzo] LT[etan]",lastWeek:"[aurreko] dddd LT[etan]",sameElse:"L"},relativeTime:{future:"%s barru",past:"duela %s",s:"segundo batzuk",ss:"%d segundo",m:"minutu bat",mm:"%d minutu",h:"ordu bat",hh:"%d ordu",d:"egun bat",dd:"%d egun",M:"hilabete bat",MM:"%d hilabete",y:"urte bat",yy:"%d urte"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:7}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t={1:"۱",2:"۲",3:"۳",4:"۴",5:"۵",6:"۶",7:"۷",8:"۸",9:"۹",0:"۰"},n={"۱":"1","۲":"2","۳":"3","۴":"4","۵":"5","۶":"6","۷":"7","۸":"8","۹":"9","۰":"0"};e.defineLocale("fa",{months:"ژانویه_فوریه_مارس_آوریل_مه_ژوئن_ژوئیه_اوت_سپتامبر_اکتبر_نوامبر_دسامبر".split("_"),monthsShort:"ژانویه_فوریه_مارس_آوریل_مه_ژوئن_ژوئیه_اوت_سپتامبر_اکتبر_نوامبر_دسامبر".split("_"),weekdays:"یک‌شنبه_دوشنبه_سه‌شنبه_چهارشنبه_پنج‌شنبه_جمعه_شنبه".split("_"),weekdaysShort:"یک‌شنبه_دوشنبه_سه‌شنبه_چهارشنبه_پنج‌شنبه_جمعه_شنبه".split("_"),weekdaysMin:"ی_د_س_چ_پ_ج_ش".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},meridiemParse:/قبل از ظهر|بعد از ظهر/,isPM:function(e){return/بعد از ظهر/.test(e)},meridiem:function(e,t,n){return e<12?"قبل از ظهر":"بعد از ظهر"},calendar:{sameDay:"[امروز ساعت] LT",nextDay:"[فردا ساعت] LT",nextWeek:"dddd [ساعت] LT",lastDay:"[دیروز ساعت] LT",lastWeek:"dddd [پیش] [ساعت] LT",sameElse:"L"},relativeTime:{future:"در %s",past:"%s پیش",s:"چند ثانیه",ss:"%d ثانیه",m:"یک دقیقه",mm:"%d دقیقه",h:"یک ساعت",hh:"%d ساعت",d:"یک روز",dd:"%d روز",M:"یک ماه",MM:"%d ماه",y:"یک سال",yy:"%d سال"},preparse:function(e){return e.replace(/[۰-۹]/g,(function(e){return n[e]})).replace(/،/g,",")},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]})).replace(/,/g,"،")},dayOfMonthOrdinalParse:/\d{1,2}م/,ordinal:"%dم",week:{dow:6,doy:12}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t="nolla yksi kaksi kolme neljä viisi kuusi seitsemän kahdeksan yhdeksän".split(" "),n=["nolla","yhden","kahden","kolmen","neljän","viiden","kuuden",t[7],t[8],t[9]];function r(e,r,i,o){var a="";switch(i){case"s":return o?"muutaman sekunnin":"muutama sekunti";case"ss":a=o?"sekunnin":"sekuntia";break;case"m":return o?"minuutin":"minuutti";case"mm":a=o?"minuutin":"minuuttia";break;case"h":return o?"tunnin":"tunti";case"hh":a=o?"tunnin":"tuntia";break;case"d":return o?"päivän":"päivä";case"dd":a=o?"päivän":"päivää";break;case"M":return o?"kuukauden":"kuukausi";case"MM":a=o?"kuukauden":"kuukautta";break;case"y":return o?"vuoden":"vuosi";case"yy":a=o?"vuoden":"vuotta"}return a=function(e,r){return e<10?r?n[e]:t[e]:e}(e,o)+" "+a}e.defineLocale("fi",{months:"tammikuu_helmikuu_maaliskuu_huhtikuu_toukokuu_kesäkuu_heinäkuu_elokuu_syyskuu_lokakuu_marraskuu_joulukuu".split("_"),monthsShort:"tammi_helmi_maalis_huhti_touko_kesä_heinä_elo_syys_loka_marras_joulu".split("_"),weekdays:"sunnuntai_maanantai_tiistai_keskiviikko_torstai_perjantai_lauantai".split("_"),weekdaysShort:"su_ma_ti_ke_to_pe_la".split("_"),weekdaysMin:"su_ma_ti_ke_to_pe_la".split("_"),longDateFormat:{LT:"HH.mm",LTS:"HH.mm.ss",L:"DD.MM.YYYY",LL:"Do MMMM[ta] YYYY",LLL:"Do MMMM[ta] YYYY, [klo] HH.mm",LLLL:"dddd, Do MMMM[ta] YYYY, [klo] HH.mm",l:"D.M.YYYY",ll:"Do MMM YYYY",lll:"Do MMM YYYY, [klo] HH.mm",llll:"ddd, Do MMM YYYY, [klo] HH.mm"},calendar:{sameDay:"[tänään] [klo] LT",nextDay:"[huomenna] [klo] LT",nextWeek:"dddd [klo] LT",lastDay:"[eilen] [klo] LT",lastWeek:"[viime] dddd[na] [klo] LT",sameElse:"L"},relativeTime:{future:"%s päästä",past:"%s sitten",s:r,ss:r,m:r,mm:r,h:r,hh:r,d:r,dd:r,M:r,MM:r,y:r,yy:r},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("fil",{months:"Enero_Pebrero_Marso_Abril_Mayo_Hunyo_Hulyo_Agosto_Setyembre_Oktubre_Nobyembre_Disyembre".split("_"),monthsShort:"Ene_Peb_Mar_Abr_May_Hun_Hul_Ago_Set_Okt_Nob_Dis".split("_"),weekdays:"Linggo_Lunes_Martes_Miyerkules_Huwebes_Biyernes_Sabado".split("_"),weekdaysShort:"Lin_Lun_Mar_Miy_Huw_Biy_Sab".split("_"),weekdaysMin:"Li_Lu_Ma_Mi_Hu_Bi_Sab".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"MM/D/YYYY",LL:"MMMM D, YYYY",LLL:"MMMM D, YYYY HH:mm",LLLL:"dddd, MMMM DD, YYYY HH:mm"},calendar:{sameDay:"LT [ngayong araw]",nextDay:"[Bukas ng] LT",nextWeek:"LT [sa susunod na] dddd",lastDay:"LT [kahapon]",lastWeek:"LT [noong nakaraang] dddd",sameElse:"L"},relativeTime:{future:"sa loob ng %s",past:"%s ang nakalipas",s:"ilang segundo",ss:"%d segundo",m:"isang minuto",mm:"%d minuto",h:"isang oras",hh:"%d oras",d:"isang araw",dd:"%d araw",M:"isang buwan",MM:"%d buwan",y:"isang taon",yy:"%d taon"},dayOfMonthOrdinalParse:/\d{1,2}/,ordinal:function(e){return e},week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("fo",{months:"januar_februar_mars_apríl_mai_juni_juli_august_september_oktober_november_desember".split("_"),monthsShort:"jan_feb_mar_apr_mai_jun_jul_aug_sep_okt_nov_des".split("_"),weekdays:"sunnudagur_mánadagur_týsdagur_mikudagur_hósdagur_fríggjadagur_leygardagur".split("_"),weekdaysShort:"sun_mán_týs_mik_hós_frí_ley".split("_"),weekdaysMin:"su_má_tý_mi_hó_fr_le".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D. MMMM, YYYY HH:mm"},calendar:{sameDay:"[Í dag kl.] LT",nextDay:"[Í morgin kl.] LT",nextWeek:"dddd [kl.] LT",lastDay:"[Í gjár kl.] LT",lastWeek:"[síðstu] dddd [kl] LT",sameElse:"L"},relativeTime:{future:"um %s",past:"%s síðani",s:"fá sekund",ss:"%d sekundir",m:"ein minuttur",mm:"%d minuttir",h:"ein tími",hh:"%d tímar",d:"ein dagur",dd:"%d dagar",M:"ein mánaður",MM:"%d mánaðir",y:"eitt ár",yy:"%d ár"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t=/(janv\.?|févr\.?|mars|avr\.?|mai|juin|juil\.?|août|sept\.?|oct\.?|nov\.?|déc\.?|janvier|février|mars|avril|mai|juin|juillet|août|septembre|octobre|novembre|décembre)/i,n=[/^janv/i,/^févr/i,/^mars/i,/^avr/i,/^mai/i,/^juin/i,/^juil/i,/^août/i,/^sept/i,/^oct/i,/^nov/i,/^déc/i];e.defineLocale("fr",{months:"janvier_février_mars_avril_mai_juin_juillet_août_septembre_octobre_novembre_décembre".split("_"),monthsShort:"janv._févr._mars_avr._mai_juin_juil._août_sept._oct._nov._déc.".split("_"),monthsRegex:t,monthsShortRegex:t,monthsStrictRegex:/^(janvier|février|mars|avril|mai|juin|juillet|août|septembre|octobre|novembre|décembre)/i,monthsShortStrictRegex:/(janv\.?|févr\.?|mars|avr\.?|mai|juin|juil\.?|août|sept\.?|oct\.?|nov\.?|déc\.?)/i,monthsParse:n,longMonthsParse:n,shortMonthsParse:n,weekdays:"dimanche_lundi_mardi_mercredi_jeudi_vendredi_samedi".split("_"),weekdaysShort:"dim._lun._mar._mer._jeu._ven._sam.".split("_"),weekdaysMin:"di_lu_ma_me_je_ve_sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[Aujourd’hui à] LT",nextDay:"[Demain à] LT",nextWeek:"dddd [à] LT",lastDay:"[Hier à] LT",lastWeek:"dddd [dernier à] LT",sameElse:"L"},relativeTime:{future:"dans %s",past:"il y a %s",s:"quelques secondes",ss:"%d secondes",m:"une minute",mm:"%d minutes",h:"une heure",hh:"%d heures",d:"un jour",dd:"%d jours",w:"une semaine",ww:"%d semaines",M:"un mois",MM:"%d mois",y:"un an",yy:"%d ans"},dayOfMonthOrdinalParse:/\d{1,2}(er|)/,ordinal:function(e,t){switch(t){case"D":return e+(1===e?"er":"");default:case"M":case"Q":case"DDD":case"d":return e+(1===e?"er":"e");case"w":case"W":return e+(1===e?"re":"e")}},week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("fr-ca",{months:"janvier_février_mars_avril_mai_juin_juillet_août_septembre_octobre_novembre_décembre".split("_"),monthsShort:"janv._févr._mars_avr._mai_juin_juil._août_sept._oct._nov._déc.".split("_"),monthsParseExact:!0,weekdays:"dimanche_lundi_mardi_mercredi_jeudi_vendredi_samedi".split("_"),weekdaysShort:"dim._lun._mar._mer._jeu._ven._sam.".split("_"),weekdaysMin:"di_lu_ma_me_je_ve_sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[Aujourd’hui à] LT",nextDay:"[Demain à] LT",nextWeek:"dddd [à] LT",lastDay:"[Hier à] LT",lastWeek:"dddd [dernier à] LT",sameElse:"L"},relativeTime:{future:"dans %s",past:"il y a %s",s:"quelques secondes",ss:"%d secondes",m:"une minute",mm:"%d minutes",h:"une heure",hh:"%d heures",d:"un jour",dd:"%d jours",M:"un mois",MM:"%d mois",y:"un an",yy:"%d ans"},dayOfMonthOrdinalParse:/\d{1,2}(er|e)/,ordinal:function(e,t){switch(t){default:case"M":case"Q":case"D":case"DDD":case"d":return e+(1===e?"er":"e");case"w":case"W":return e+(1===e?"re":"e")}}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("fr-ch",{months:"janvier_février_mars_avril_mai_juin_juillet_août_septembre_octobre_novembre_décembre".split("_"),monthsShort:"janv._févr._mars_avr._mai_juin_juil._août_sept._oct._nov._déc.".split("_"),monthsParseExact:!0,weekdays:"dimanche_lundi_mardi_mercredi_jeudi_vendredi_samedi".split("_"),weekdaysShort:"dim._lun._mar._mer._jeu._ven._sam.".split("_"),weekdaysMin:"di_lu_ma_me_je_ve_sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[Aujourd’hui à] LT",nextDay:"[Demain à] LT",nextWeek:"dddd [à] LT",lastDay:"[Hier à] LT",lastWeek:"dddd [dernier à] LT",sameElse:"L"},relativeTime:{future:"dans %s",past:"il y a %s",s:"quelques secondes",ss:"%d secondes",m:"une minute",mm:"%d minutes",h:"une heure",hh:"%d heures",d:"un jour",dd:"%d jours",M:"un mois",MM:"%d mois",y:"un an",yy:"%d ans"},dayOfMonthOrdinalParse:/\d{1,2}(er|e)/,ordinal:function(e,t){switch(t){default:case"M":case"Q":case"D":case"DDD":case"d":return e+(1===e?"er":"e");case"w":case"W":return e+(1===e?"re":"e")}},week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t="jan._feb._mrt._apr._mai_jun._jul._aug._sep._okt._nov._des.".split("_"),n="jan_feb_mrt_apr_mai_jun_jul_aug_sep_okt_nov_des".split("_");e.defineLocale("fy",{months:"jannewaris_febrewaris_maart_april_maaie_juny_july_augustus_septimber_oktober_novimber_desimber".split("_"),monthsShort:function(e,r){return e?/-MMM-/.test(r)?n[e.month()]:t[e.month()]:t},monthsParseExact:!0,weekdays:"snein_moandei_tiisdei_woansdei_tongersdei_freed_sneon".split("_"),weekdaysShort:"si._mo._ti._wo._to._fr._so.".split("_"),weekdaysMin:"Si_Mo_Ti_Wo_To_Fr_So".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD-MM-YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[hjoed om] LT",nextDay:"[moarn om] LT",nextWeek:"dddd [om] LT",lastDay:"[juster om] LT",lastWeek:"[ôfrûne] dddd [om] LT",sameElse:"L"},relativeTime:{future:"oer %s",past:"%s lyn",s:"in pear sekonden",ss:"%d sekonden",m:"ien minút",mm:"%d minuten",h:"ien oere",hh:"%d oeren",d:"ien dei",dd:"%d dagen",M:"ien moanne",MM:"%d moannen",y:"ien jier",yy:"%d jierren"},dayOfMonthOrdinalParse:/\d{1,2}(ste|de)/,ordinal:function(e){return e+(1===e||8===e||e>=20?"ste":"de")},week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("ga",{months:["Eanáir","Feabhra","Márta","Aibreán","Bealtaine","Meitheamh","Iúil","Lúnasa","Meán Fómhair","Deireadh Fómhair","Samhain","Nollaig"],monthsShort:["Ean","Feabh","Márt","Aib","Beal","Meith","Iúil","Lún","M.F.","D.F.","Samh","Noll"],monthsParseExact:!0,weekdays:["Dé Domhnaigh","Dé Luain","Dé Máirt","Dé Céadaoin","Déardaoin","Dé hAoine","Dé Sathairn"],weekdaysShort:["Domh","Luan","Máirt","Céad","Déar","Aoine","Sath"],weekdaysMin:["Do","Lu","Má","Cé","Dé","A","Sa"],longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Inniu ag] LT",nextDay:"[Amárach ag] LT",nextWeek:"dddd [ag] LT",lastDay:"[Inné ag] LT",lastWeek:"dddd [seo caite] [ag] LT",sameElse:"L"},relativeTime:{future:"i %s",past:"%s ó shin",s:"cúpla soicind",ss:"%d soicind",m:"nóiméad",mm:"%d nóiméad",h:"uair an chloig",hh:"%d uair an chloig",d:"lá",dd:"%d lá",M:"mí",MM:"%d míonna",y:"bliain",yy:"%d bliain"},dayOfMonthOrdinalParse:/\d{1,2}(d|na|mh)/,ordinal:function(e){return e+(1===e?"d":e%10==2?"na":"mh")},week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("gd",{months:["Am Faoilleach","An Gearran","Am Màrt","An Giblean","An Cèitean","An t-Ògmhios","An t-Iuchar","An Lùnastal","An t-Sultain","An Dàmhair","An t-Samhain","An Dùbhlachd"],monthsShort:["Faoi","Gear","Màrt","Gibl","Cèit","Ògmh","Iuch","Lùn","Sult","Dàmh","Samh","Dùbh"],monthsParseExact:!0,weekdays:["Didòmhnaich","Diluain","Dimàirt","Diciadain","Diardaoin","Dihaoine","Disathairne"],weekdaysShort:["Did","Dil","Dim","Dic","Dia","Dih","Dis"],weekdaysMin:["Dò","Lu","Mà","Ci","Ar","Ha","Sa"],longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[An-diugh aig] LT",nextDay:"[A-màireach aig] LT",nextWeek:"dddd [aig] LT",lastDay:"[An-dè aig] LT",lastWeek:"dddd [seo chaidh] [aig] LT",sameElse:"L"},relativeTime:{future:"ann an %s",past:"bho chionn %s",s:"beagan diogan",ss:"%d diogan",m:"mionaid",mm:"%d mionaidean",h:"uair",hh:"%d uairean",d:"latha",dd:"%d latha",M:"mìos",MM:"%d mìosan",y:"bliadhna",yy:"%d bliadhna"},dayOfMonthOrdinalParse:/\d{1,2}(d|na|mh)/,ordinal:function(e){return e+(1===e?"d":e%10==2?"na":"mh")},week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("gl",{months:"xaneiro_febreiro_marzo_abril_maio_xuño_xullo_agosto_setembro_outubro_novembro_decembro".split("_"),monthsShort:"xan._feb._mar._abr._mai._xuñ._xul._ago._set._out._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"domingo_luns_martes_mércores_xoves_venres_sábado".split("_"),weekdaysShort:"dom._lun._mar._mér._xov._ven._sáb.".split("_"),weekdaysMin:"do_lu_ma_mé_xo_ve_sá".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD/MM/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY H:mm",LLLL:"dddd, D [de] MMMM [de] YYYY H:mm"},calendar:{sameDay:function(){return"[hoxe "+(1!==this.hours()?"ás":"á")+"] LT"},nextDay:function(){return"[mañá "+(1!==this.hours()?"ás":"á")+"] LT"},nextWeek:function(){return"dddd ["+(1!==this.hours()?"ás":"a")+"] LT"},lastDay:function(){return"[onte "+(1!==this.hours()?"á":"a")+"] LT"},lastWeek:function(){return"[o] dddd [pasado "+(1!==this.hours()?"ás":"a")+"] LT"},sameElse:"L"},relativeTime:{future:function(e){return 0===e.indexOf("un")?"n"+e:"en "+e},past:"hai %s",s:"uns segundos",ss:"%d segundos",m:"un minuto",mm:"%d minutos",h:"unha hora",hh:"%d horas",d:"un día",dd:"%d días",M:"un mes",MM:"%d meses",y:"un ano",yy:"%d anos"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
function t(e,t,n,r){var i={s:["थोडया सॅकंडांनी","थोडे सॅकंड"],ss:[e+" सॅकंडांनी",e+" सॅकंड"],m:["एका मिणटान","एक मिनूट"],mm:[e+" मिणटांनी",e+" मिणटां"],h:["एका वरान","एक वर"],hh:[e+" वरांनी",e+" वरां"],d:["एका दिसान","एक दीस"],dd:[e+" दिसांनी",e+" दीस"],M:["एका म्हयन्यान","एक म्हयनो"],MM:[e+" म्हयन्यानी",e+" म्हयने"],y:["एका वर्सान","एक वर्स"],yy:[e+" वर्सांनी",e+" वर्सां"]};return r?i[n][0]:i[n][1]}e.defineLocale("gom-deva",{months:{standalone:"जानेवारी_फेब्रुवारी_मार्च_एप्रील_मे_जून_जुलय_ऑगस्ट_सप्टेंबर_ऑक्टोबर_नोव्हेंबर_डिसेंबर".split("_"),format:"जानेवारीच्या_फेब्रुवारीच्या_मार्चाच्या_एप्रीलाच्या_मेयाच्या_जूनाच्या_जुलयाच्या_ऑगस्टाच्या_सप्टेंबराच्या_ऑक्टोबराच्या_नोव्हेंबराच्या_डिसेंबराच्या".split("_"),isFormat:/MMMM(\s)+D[oD]?/},monthsShort:"जाने._फेब्रु._मार्च_एप्री._मे_जून_जुल._ऑग._सप्टें._ऑक्टो._नोव्हें._डिसें.".split("_"),monthsParseExact:!0,weekdays:"आयतार_सोमार_मंगळार_बुधवार_बिरेस्तार_सुक्रार_शेनवार".split("_"),weekdaysShort:"आयत._सोम._मंगळ._बुध._ब्रेस्त._सुक्र._शेन.".split("_"),weekdaysMin:"आ_सो_मं_बु_ब्रे_सु_शे".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"A h:mm [वाजतां]",LTS:"A h:mm:ss [वाजतां]",L:"DD-MM-YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY A h:mm [वाजतां]",LLLL:"dddd, MMMM Do, YYYY, A h:mm [वाजतां]",llll:"ddd, D MMM YYYY, A h:mm [वाजतां]"},calendar:{sameDay:"[आयज] LT",nextDay:"[फाल्यां] LT",nextWeek:"[फुडलो] dddd[,] LT",lastDay:"[काल] LT",lastWeek:"[फाटलो] dddd[,] LT",sameElse:"L"},relativeTime:{future:"%s",past:"%s आदीं",s:t,ss:t,m:t,mm:t,h:t,hh:t,d:t,dd:t,M:t,MM:t,y:t,yy:t},dayOfMonthOrdinalParse:/\d{1,2}(वेर)/,ordinal:function(e,t){switch(t){case"D":return e+"वेर";default:case"M":case"Q":case"DDD":case"d":case"w":case"W":return e}},week:{dow:0,doy:3},meridiemParse:/राती|सकाळीं|दनपारां|सांजे/,meridiemHour:function(e,t){return 12===e&&(e=0),"राती"===t?e<4?e:e+12:"सकाळीं"===t?e:"दनपारां"===t?e>12?e:e+12:"सांजे"===t?e+12:void 0},meridiem:function(e,t,n){return e<4?"राती":e<12?"सकाळीं":e<16?"दनपारां":e<20?"सांजे":"राती"}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
function t(e,t,n,r){var i={s:["thoddea sekondamni","thodde sekond"],ss:[e+" sekondamni",e+" sekond"],m:["eka mintan","ek minut"],mm:[e+" mintamni",e+" mintam"],h:["eka voran","ek vor"],hh:[e+" voramni",e+" voram"],d:["eka disan","ek dis"],dd:[e+" disamni",e+" dis"],M:["eka mhoinean","ek mhoino"],MM:[e+" mhoineamni",e+" mhoine"],y:["eka vorsan","ek voros"],yy:[e+" vorsamni",e+" vorsam"]};return r?i[n][0]:i[n][1]}e.defineLocale("gom-latn",{months:{standalone:"Janer_Febrer_Mars_Abril_Mai_Jun_Julai_Agost_Setembr_Otubr_Novembr_Dezembr".split("_"),format:"Janerachea_Febrerachea_Marsachea_Abrilachea_Maiachea_Junachea_Julaiachea_Agostachea_Setembrachea_Otubrachea_Novembrachea_Dezembrachea".split("_"),isFormat:/MMMM(\s)+D[oD]?/},monthsShort:"Jan._Feb._Mars_Abr._Mai_Jun_Jul._Ago._Set._Otu._Nov._Dez.".split("_"),monthsParseExact:!0,weekdays:"Aitar_Somar_Mongllar_Budhvar_Birestar_Sukrar_Son'var".split("_"),weekdaysShort:"Ait._Som._Mon._Bud._Bre._Suk._Son.".split("_"),weekdaysMin:"Ai_Sm_Mo_Bu_Br_Su_Sn".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"A h:mm [vazta]",LTS:"A h:mm:ss [vazta]",L:"DD-MM-YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY A h:mm [vazta]",LLLL:"dddd, MMMM Do, YYYY, A h:mm [vazta]",llll:"ddd, D MMM YYYY, A h:mm [vazta]"},calendar:{sameDay:"[Aiz] LT",nextDay:"[Faleam] LT",nextWeek:"[Fuddlo] dddd[,] LT",lastDay:"[Kal] LT",lastWeek:"[Fattlo] dddd[,] LT",sameElse:"L"},relativeTime:{future:"%s",past:"%s adim",s:t,ss:t,m:t,mm:t,h:t,hh:t,d:t,dd:t,M:t,MM:t,y:t,yy:t},dayOfMonthOrdinalParse:/\d{1,2}(er)/,ordinal:function(e,t){switch(t){case"D":return e+"er";default:case"M":case"Q":case"DDD":case"d":case"w":case"W":return e}},week:{dow:0,doy:3},meridiemParse:/rati|sokallim|donparam|sanje/,meridiemHour:function(e,t){return 12===e&&(e=0),"rati"===t?e<4?e:e+12:"sokallim"===t?e:"donparam"===t?e>12?e:e+12:"sanje"===t?e+12:void 0},meridiem:function(e,t,n){return e<4?"rati":e<12?"sokallim":e<16?"donparam":e<20?"sanje":"rati"}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t={1:"૧",2:"૨",3:"૩",4:"૪",5:"૫",6:"૬",7:"૭",8:"૮",9:"૯",0:"૦"},n={"૧":"1","૨":"2","૩":"3","૪":"4","૫":"5","૬":"6","૭":"7","૮":"8","૯":"9","૦":"0"};e.defineLocale("gu",{months:"જાન્યુઆરી_ફેબ્રુઆરી_માર્ચ_એપ્રિલ_મે_જૂન_જુલાઈ_ઑગસ્ટ_સપ્ટેમ્બર_ઑક્ટ્બર_નવેમ્બર_ડિસેમ્બર".split("_"),monthsShort:"જાન્યુ._ફેબ્રુ._માર્ચ_એપ્રિ._મે_જૂન_જુલા._ઑગ._સપ્ટે._ઑક્ટ્._નવે._ડિસે.".split("_"),monthsParseExact:!0,weekdays:"રવિવાર_સોમવાર_મંગળવાર_બુધ્વાર_ગુરુવાર_શુક્રવાર_શનિવાર".split("_"),weekdaysShort:"રવિ_સોમ_મંગળ_બુધ્_ગુરુ_શુક્ર_શનિ".split("_"),weekdaysMin:"ર_સો_મં_બુ_ગુ_શુ_શ".split("_"),longDateFormat:{LT:"A h:mm વાગ્યે",LTS:"A h:mm:ss વાગ્યે",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm વાગ્યે",LLLL:"dddd, D MMMM YYYY, A h:mm વાગ્યે"},calendar:{sameDay:"[આજ] LT",nextDay:"[કાલે] LT",nextWeek:"dddd, LT",lastDay:"[ગઇકાલે] LT",lastWeek:"[પાછલા] dddd, LT",sameElse:"L"},relativeTime:{future:"%s મા",past:"%s પહેલા",s:"અમુક પળો",ss:"%d સેકંડ",m:"એક મિનિટ",mm:"%d મિનિટ",h:"એક કલાક",hh:"%d કલાક",d:"એક દિવસ",dd:"%d દિવસ",M:"એક મહિનો",MM:"%d મહિનો",y:"એક વર્ષ",yy:"%d વર્ષ"},preparse:function(e){return e.replace(/[૧૨૩૪૫૬૭૮૯૦]/g,(function(e){return n[e]}))},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]}))},meridiemParse:/રાત|બપોર|સવાર|સાંજ/,meridiemHour:function(e,t){return 12===e&&(e=0),"રાત"===t?e<4?e:e+12:"સવાર"===t?e:"બપોર"===t?e>=10?e:e+12:"સાંજ"===t?e+12:void 0},meridiem:function(e,t,n){return e<4?"રાત":e<10?"સવાર":e<17?"બપોર":e<20?"સાંજ":"રાત"},week:{dow:0,doy:6}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("he",{months:"ינואר_פברואר_מרץ_אפריל_מאי_יוני_יולי_אוגוסט_ספטמבר_אוקטובר_נובמבר_דצמבר".split("_"),monthsShort:"ינו׳_פבר׳_מרץ_אפר׳_מאי_יוני_יולי_אוג׳_ספט׳_אוק׳_נוב׳_דצמ׳".split("_"),weekdays:"ראשון_שני_שלישי_רביעי_חמישי_שישי_שבת".split("_"),weekdaysShort:"א׳_ב׳_ג׳_ד׳_ה׳_ו׳_ש׳".split("_"),weekdaysMin:"א_ב_ג_ד_ה_ו_ש".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D [ב]MMMM YYYY",LLL:"D [ב]MMMM YYYY HH:mm",LLLL:"dddd, D [ב]MMMM YYYY HH:mm",l:"D/M/YYYY",ll:"D MMM YYYY",lll:"D MMM YYYY HH:mm",llll:"ddd, D MMM YYYY HH:mm"},calendar:{sameDay:"[היום ב־]LT",nextDay:"[מחר ב־]LT",nextWeek:"dddd [בשעה] LT",lastDay:"[אתמול ב־]LT",lastWeek:"[ביום] dddd [האחרון בשעה] LT",sameElse:"L"},relativeTime:{future:"בעוד %s",past:"לפני %s",s:"מספר שניות",ss:"%d שניות",m:"דקה",mm:"%d דקות",h:"שעה",hh:function(e){return 2===e?"שעתיים":e+" שעות"},d:"יום",dd:function(e){return 2===e?"יומיים":e+" ימים"},M:"חודש",MM:function(e){return 2===e?"חודשיים":e+" חודשים"},y:"שנה",yy:function(e){return 2===e?"שנתיים":e%10==0&&10!==e?e+" שנה":e+" שנים"}},meridiemParse:/אחה"צ|לפנה"צ|אחרי הצהריים|לפני הצהריים|לפנות בוקר|בבוקר|בערב/i,isPM:function(e){return/^(אחה"צ|אחרי הצהריים|בערב)$/.test(e)},meridiem:function(e,t,n){return e<5?"לפנות בוקר":e<10?"בבוקר":e<12?n?'לפנה"צ':"לפני הצהריים":e<18?n?'אחה"צ':"אחרי הצהריים":"בערב"}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t={1:"१",2:"२",3:"३",4:"४",5:"५",6:"६",7:"७",8:"८",9:"९",0:"०"},n={"१":"1","२":"2","३":"3","४":"4","५":"5","६":"6","७":"7","८":"8","९":"9","०":"0"},r=[/^जन/i,/^फ़र|फर/i,/^मार्च/i,/^अप्रै/i,/^मई/i,/^जून/i,/^जुल/i,/^अग/i,/^सितं|सित/i,/^अक्टू/i,/^नव|नवं/i,/^दिसं|दिस/i];e.defineLocale("hi",{months:{format:"जनवरी_फ़रवरी_मार्च_अप्रैल_मई_जून_जुलाई_अगस्त_सितम्बर_अक्टूबर_नवम्बर_दिसम्बर".split("_"),standalone:"जनवरी_फरवरी_मार्च_अप्रैल_मई_जून_जुलाई_अगस्त_सितंबर_अक्टूबर_नवंबर_दिसंबर".split("_")},monthsShort:"जन._फ़र._मार्च_अप्रै._मई_जून_जुल._अग._सित._अक्टू._नव._दिस.".split("_"),weekdays:"रविवार_सोमवार_मंगलवार_बुधवार_गुरूवार_शुक्रवार_शनिवार".split("_"),weekdaysShort:"रवि_सोम_मंगल_बुध_गुरू_शुक्र_शनि".split("_"),weekdaysMin:"र_सो_मं_बु_गु_शु_श".split("_"),longDateFormat:{LT:"A h:mm बजे",LTS:"A h:mm:ss बजे",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm बजे",LLLL:"dddd, D MMMM YYYY, A h:mm बजे"},monthsParse:r,longMonthsParse:r,shortMonthsParse:[/^जन/i,/^फ़र/i,/^मार्च/i,/^अप्रै/i,/^मई/i,/^जून/i,/^जुल/i,/^अग/i,/^सित/i,/^अक्टू/i,/^नव/i,/^दिस/i],monthsRegex:/^(जनवरी|जन\.?|फ़रवरी|फरवरी|फ़र\.?|मार्च?|अप्रैल|अप्रै\.?|मई?|जून?|जुलाई|जुल\.?|अगस्त|अग\.?|सितम्बर|सितंबर|सित\.?|अक्टूबर|अक्टू\.?|नवम्बर|नवंबर|नव\.?|दिसम्बर|दिसंबर|दिस\.?)/i,monthsShortRegex:/^(जनवरी|जन\.?|फ़रवरी|फरवरी|फ़र\.?|मार्च?|अप्रैल|अप्रै\.?|मई?|जून?|जुलाई|जुल\.?|अगस्त|अग\.?|सितम्बर|सितंबर|सित\.?|अक्टूबर|अक्टू\.?|नवम्बर|नवंबर|नव\.?|दिसम्बर|दिसंबर|दिस\.?)/i,monthsStrictRegex:/^(जनवरी?|फ़रवरी|फरवरी?|मार्च?|अप्रैल?|मई?|जून?|जुलाई?|अगस्त?|सितम्बर|सितंबर|सित?\.?|अक्टूबर|अक्टू\.?|नवम्बर|नवंबर?|दिसम्बर|दिसंबर?)/i,monthsShortStrictRegex:/^(जन\.?|फ़र\.?|मार्च?|अप्रै\.?|मई?|जून?|जुल\.?|अग\.?|सित\.?|अक्टू\.?|नव\.?|दिस\.?)/i,calendar:{sameDay:"[आज] LT",nextDay:"[कल] LT",nextWeek:"dddd, LT",lastDay:"[कल] LT",lastWeek:"[पिछले] dddd, LT",sameElse:"L"},relativeTime:{future:"%s में",past:"%s पहले",s:"कुछ ही क्षण",ss:"%d सेकंड",m:"एक मिनट",mm:"%d मिनट",h:"एक घंटा",hh:"%d घंटे",d:"एक दिन",dd:"%d दिन",M:"एक महीने",MM:"%d महीने",y:"एक वर्ष",yy:"%d वर्ष"},preparse:function(e){return e.replace(/[१२३४५६७८९०]/g,(function(e){return n[e]}))},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]}))},meridiemParse:/रात|सुबह|दोपहर|शाम/,meridiemHour:function(e,t){return 12===e&&(e=0),"रात"===t?e<4?e:e+12:"सुबह"===t?e:"दोपहर"===t?e>=10?e:e+12:"शाम"===t?e+12:void 0},meridiem:function(e,t,n){return e<4?"रात":e<10?"सुबह":e<17?"दोपहर":e<20?"शाम":"रात"},week:{dow:0,doy:6}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
function t(e,t,n){var r=e+" ";switch(n){case"ss":return r+=1===e?"sekunda":2===e||3===e||4===e?"sekunde":"sekundi";case"m":return t?"jedna minuta":"jedne minute";case"mm":return r+=1===e?"minuta":2===e||3===e||4===e?"minute":"minuta";case"h":return t?"jedan sat":"jednog sata";case"hh":return r+=1===e?"sat":2===e||3===e||4===e?"sata":"sati";case"dd":return r+=1===e?"dan":"dana";case"MM":return r+=1===e?"mjesec":2===e||3===e||4===e?"mjeseca":"mjeseci";case"yy":return r+=1===e?"godina":2===e||3===e||4===e?"godine":"godina"}}e.defineLocale("hr",{months:{format:"siječnja_veljače_ožujka_travnja_svibnja_lipnja_srpnja_kolovoza_rujna_listopada_studenoga_prosinca".split("_"),standalone:"siječanj_veljača_ožujak_travanj_svibanj_lipanj_srpanj_kolovoz_rujan_listopad_studeni_prosinac".split("_")},monthsShort:"sij._velj._ožu._tra._svi._lip._srp._kol._ruj._lis._stu._pro.".split("_"),monthsParseExact:!0,weekdays:"nedjelja_ponedjeljak_utorak_srijeda_četvrtak_petak_subota".split("_"),weekdaysShort:"ned._pon._uto._sri._čet._pet._sub.".split("_"),weekdaysMin:"ne_po_ut_sr_če_pe_su".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"Do MMMM YYYY",LLL:"Do MMMM YYYY H:mm",LLLL:"dddd, Do MMMM YYYY H:mm"},calendar:{sameDay:"[danas u] LT",nextDay:"[sutra u] LT",nextWeek:function(){switch(this.day()){case 0:return"[u] [nedjelju] [u] LT";case 3:return"[u] [srijedu] [u] LT";case 6:return"[u] [subotu] [u] LT";case 1:case 2:case 4:case 5:return"[u] dddd [u] LT"}},lastDay:"[jučer u] LT",lastWeek:function(){switch(this.day()){case 0:return"[prošlu] [nedjelju] [u] LT";case 3:return"[prošlu] [srijedu] [u] LT";case 6:return"[prošle] [subote] [u] LT";case 1:case 2:case 4:case 5:return"[prošli] dddd [u] LT"}},sameElse:"L"},relativeTime:{future:"za %s",past:"prije %s",s:"par sekundi",ss:t,m:t,mm:t,h:t,hh:t,d:"dan",dd:t,M:"mjesec",MM:t,y:"godinu",yy:t},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:7}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t="vasárnap hétfőn kedden szerdán csütörtökön pénteken szombaton".split(" ");function n(e,t,n,r){var i=e;switch(n){case"s":return r||t?"néhány másodperc":"néhány másodperce";case"ss":return i+(r||t)?" másodperc":" másodperce";case"m":return"egy"+(r||t?" perc":" perce");case"mm":return i+(r||t?" perc":" perce");case"h":return"egy"+(r||t?" óra":" órája");case"hh":return i+(r||t?" óra":" órája");case"d":return"egy"+(r||t?" nap":" napja");case"dd":return i+(r||t?" nap":" napja");case"M":return"egy"+(r||t?" hónap":" hónapja");case"MM":return i+(r||t?" hónap":" hónapja");case"y":return"egy"+(r||t?" év":" éve");case"yy":return i+(r||t?" év":" éve")}return""}function r(e){return(e?"":"[múlt] ")+"["+t[this.day()]+"] LT[-kor]"}e.defineLocale("hu",{months:"január_február_március_április_május_június_július_augusztus_szeptember_október_november_december".split("_"),monthsShort:"jan._feb._márc._ápr._máj._jún._júl._aug._szept._okt._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"vasárnap_hétfő_kedd_szerda_csütörtök_péntek_szombat".split("_"),weekdaysShort:"vas_hét_kedd_sze_csüt_pén_szo".split("_"),weekdaysMin:"v_h_k_sze_cs_p_szo".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"YYYY.MM.DD.",LL:"YYYY. MMMM D.",LLL:"YYYY. MMMM D. H:mm",LLLL:"YYYY. MMMM D., dddd H:mm"},meridiemParse:/de|du/i,isPM:function(e){return"u"===e.charAt(1).toLowerCase()},meridiem:function(e,t,n){return e<12?!0===n?"de":"DE":!0===n?"du":"DU"},calendar:{sameDay:"[ma] LT[-kor]",nextDay:"[holnap] LT[-kor]",nextWeek:function(){return r.call(this,!0)},lastDay:"[tegnap] LT[-kor]",lastWeek:function(){return r.call(this,!1)},sameElse:"L"},relativeTime:{future:"%s múlva",past:"%s",s:n,ss:n,m:n,mm:n,h:n,hh:n,d:n,dd:n,M:n,MM:n,y:n,yy:n},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("hy-am",{months:{format:"հունվարի_փետրվարի_մարտի_ապրիլի_մայիսի_հունիսի_հուլիսի_օգոստոսի_սեպտեմբերի_հոկտեմբերի_նոյեմբերի_դեկտեմբերի".split("_"),standalone:"հունվար_փետրվար_մարտ_ապրիլ_մայիս_հունիս_հուլիս_օգոստոս_սեպտեմբեր_հոկտեմբեր_նոյեմբեր_դեկտեմբեր".split("_")},monthsShort:"հնվ_փտր_մրտ_ապր_մյս_հնս_հլս_օգս_սպտ_հկտ_նմբ_դկտ".split("_"),weekdays:"կիրակի_երկուշաբթի_երեքշաբթի_չորեքշաբթի_հինգշաբթի_ուրբաթ_շաբաթ".split("_"),weekdaysShort:"կրկ_երկ_երք_չրք_հնգ_ուրբ_շբթ".split("_"),weekdaysMin:"կրկ_երկ_երք_չրք_հնգ_ուրբ_շբթ".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY թ.",LLL:"D MMMM YYYY թ., HH:mm",LLLL:"dddd, D MMMM YYYY թ., HH:mm"},calendar:{sameDay:"[այսօր] LT",nextDay:"[վաղը] LT",lastDay:"[երեկ] LT",nextWeek:function(){return"dddd [օրը ժամը] LT"},lastWeek:function(){return"[անցած] dddd [օրը ժամը] LT"},sameElse:"L"},relativeTime:{future:"%s հետո",past:"%s առաջ",s:"մի քանի վայրկյան",ss:"%d վայրկյան",m:"րոպե",mm:"%d րոպե",h:"ժամ",hh:"%d ժամ",d:"օր",dd:"%d օր",M:"ամիս",MM:"%d ամիս",y:"տարի",yy:"%d տարի"},meridiemParse:/գիշերվա|առավոտվա|ցերեկվա|երեկոյան/,isPM:function(e){return/^(ցերեկվա|երեկոյան)$/.test(e)},meridiem:function(e){return e<4?"գիշերվա":e<12?"առավոտվա":e<17?"ցերեկվա":"երեկոյան"},dayOfMonthOrdinalParse:/\d{1,2}|\d{1,2}-(ին|րդ)/,ordinal:function(e,t){switch(t){case"DDD":case"w":case"W":case"DDDo":return 1===e?e+"-ին":e+"-րդ";default:return e}},week:{dow:1,doy:7}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("id",{months:"Januari_Februari_Maret_April_Mei_Juni_Juli_Agustus_September_Oktober_November_Desember".split("_"),monthsShort:"Jan_Feb_Mar_Apr_Mei_Jun_Jul_Agt_Sep_Okt_Nov_Des".split("_"),weekdays:"Minggu_Senin_Selasa_Rabu_Kamis_Jumat_Sabtu".split("_"),weekdaysShort:"Min_Sen_Sel_Rab_Kam_Jum_Sab".split("_"),weekdaysMin:"Mg_Sn_Sl_Rb_Km_Jm_Sb".split("_"),longDateFormat:{LT:"HH.mm",LTS:"HH.mm.ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY [pukul] HH.mm",LLLL:"dddd, D MMMM YYYY [pukul] HH.mm"},meridiemParse:/pagi|siang|sore|malam/,meridiemHour:function(e,t){return 12===e&&(e=0),"pagi"===t?e:"siang"===t?e>=11?e:e+12:"sore"===t||"malam"===t?e+12:void 0},meridiem:function(e,t,n){return e<11?"pagi":e<15?"siang":e<19?"sore":"malam"},calendar:{sameDay:"[Hari ini pukul] LT",nextDay:"[Besok pukul] LT",nextWeek:"dddd [pukul] LT",lastDay:"[Kemarin pukul] LT",lastWeek:"dddd [lalu pukul] LT",sameElse:"L"},relativeTime:{future:"dalam %s",past:"%s yang lalu",s:"beberapa detik",ss:"%d detik",m:"semenit",mm:"%d menit",h:"sejam",hh:"%d jam",d:"sehari",dd:"%d hari",M:"sebulan",MM:"%d bulan",y:"setahun",yy:"%d tahun"},week:{dow:0,doy:6}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
function t(e){return e%100==11||e%10!=1}function n(e,n,r,i){var o=e+" ";switch(r){case"s":return n||i?"nokkrar sekúndur":"nokkrum sekúndum";case"ss":return t(e)?o+(n||i?"sekúndur":"sekúndum"):o+"sekúnda";case"m":return n?"mínúta":"mínútu";case"mm":return t(e)?o+(n||i?"mínútur":"mínútum"):n?o+"mínúta":o+"mínútu";case"hh":return t(e)?o+(n||i?"klukkustundir":"klukkustundum"):o+"klukkustund";case"d":return n?"dagur":i?"dag":"degi";case"dd":return t(e)?n?o+"dagar":o+(i?"daga":"dögum"):n?o+"dagur":o+(i?"dag":"degi");case"M":return n?"mánuður":i?"mánuð":"mánuði";case"MM":return t(e)?n?o+"mánuðir":o+(i?"mánuði":"mánuðum"):n?o+"mánuður":o+(i?"mánuð":"mánuði");case"y":return n||i?"ár":"ári";case"yy":return t(e)?o+(n||i?"ár":"árum"):o+(n||i?"ár":"ári")}}e.defineLocale("is",{months:"janúar_febrúar_mars_apríl_maí_júní_júlí_ágúst_september_október_nóvember_desember".split("_"),monthsShort:"jan_feb_mar_apr_maí_jún_júl_ágú_sep_okt_nóv_des".split("_"),weekdays:"sunnudagur_mánudagur_þriðjudagur_miðvikudagur_fimmtudagur_föstudagur_laugardagur".split("_"),weekdaysShort:"sun_mán_þri_mið_fim_fös_lau".split("_"),weekdaysMin:"Su_Má_Þr_Mi_Fi_Fö_La".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY [kl.] H:mm",LLLL:"dddd, D. MMMM YYYY [kl.] H:mm"},calendar:{sameDay:"[í dag kl.] LT",nextDay:"[á morgun kl.] LT",nextWeek:"dddd [kl.] LT",lastDay:"[í gær kl.] LT",lastWeek:"[síðasta] dddd [kl.] LT",sameElse:"L"},relativeTime:{future:"eftir %s",past:"fyrir %s síðan",s:n,ss:n,m:n,mm:n,h:"klukkustund",hh:n,d:n,dd:n,M:n,MM:n,y:n,yy:n},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("it",{months:"gennaio_febbraio_marzo_aprile_maggio_giugno_luglio_agosto_settembre_ottobre_novembre_dicembre".split("_"),monthsShort:"gen_feb_mar_apr_mag_giu_lug_ago_set_ott_nov_dic".split("_"),weekdays:"domenica_lunedì_martedì_mercoledì_giovedì_venerdì_sabato".split("_"),weekdaysShort:"dom_lun_mar_mer_gio_ven_sab".split("_"),weekdaysMin:"do_lu_ma_me_gi_ve_sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:function(){return"[Oggi a"+(this.hours()>1?"lle ":0===this.hours()?" ":"ll'")+"]LT"},nextDay:function(){return"[Domani a"+(this.hours()>1?"lle ":0===this.hours()?" ":"ll'")+"]LT"},nextWeek:function(){return"dddd [a"+(this.hours()>1?"lle ":0===this.hours()?" ":"ll'")+"]LT"},lastDay:function(){return"[Ieri a"+(this.hours()>1?"lle ":0===this.hours()?" ":"ll'")+"]LT"},lastWeek:function(){switch(this.day()){case 0:return"[La scorsa] dddd [a"+(this.hours()>1?"lle ":0===this.hours()?" ":"ll'")+"]LT";default:return"[Lo scorso] dddd [a"+(this.hours()>1?"lle ":0===this.hours()?" ":"ll'")+"]LT"}},sameElse:"L"},relativeTime:{future:"tra %s",past:"%s fa",s:"alcuni secondi",ss:"%d secondi",m:"un minuto",mm:"%d minuti",h:"un'ora",hh:"%d ore",d:"un giorno",dd:"%d giorni",w:"una settimana",ww:"%d settimane",M:"un mese",MM:"%d mesi",y:"un anno",yy:"%d anni"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("it-ch",{months:"gennaio_febbraio_marzo_aprile_maggio_giugno_luglio_agosto_settembre_ottobre_novembre_dicembre".split("_"),monthsShort:"gen_feb_mar_apr_mag_giu_lug_ago_set_ott_nov_dic".split("_"),weekdays:"domenica_lunedì_martedì_mercoledì_giovedì_venerdì_sabato".split("_"),weekdaysShort:"dom_lun_mar_mer_gio_ven_sab".split("_"),weekdaysMin:"do_lu_ma_me_gi_ve_sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[Oggi alle] LT",nextDay:"[Domani alle] LT",nextWeek:"dddd [alle] LT",lastDay:"[Ieri alle] LT",lastWeek:function(){switch(this.day()){case 0:return"[la scorsa] dddd [alle] LT";default:return"[lo scorso] dddd [alle] LT"}},sameElse:"L"},relativeTime:{future:function(e){return(/^[0-9].+$/.test(e)?"tra":"in")+" "+e},past:"%s fa",s:"alcuni secondi",ss:"%d secondi",m:"un minuto",mm:"%d minuti",h:"un'ora",hh:"%d ore",d:"un giorno",dd:"%d giorni",M:"un mese",MM:"%d mesi",y:"un anno",yy:"%d anni"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("ja",{eras:[{since:"2019-05-01",offset:1,name:"令和",narrow:"㋿",abbr:"R"},{since:"1989-01-08",until:"2019-04-30",offset:1,name:"平成",narrow:"㍻",abbr:"H"},{since:"1926-12-25",until:"1989-01-07",offset:1,name:"昭和",narrow:"㍼",abbr:"S"},{since:"1912-07-30",until:"1926-12-24",offset:1,name:"大正",narrow:"㍽",abbr:"T"},{since:"1873-01-01",until:"1912-07-29",offset:6,name:"明治",narrow:"㍾",abbr:"M"},{since:"0001-01-01",until:"1873-12-31",offset:1,name:"西暦",narrow:"AD",abbr:"AD"},{since:"0000-12-31",until:-1/0,offset:1,name:"紀元前",narrow:"BC",abbr:"BC"}],eraYearOrdinalRegex:/(元|\d+)年/,eraYearOrdinalParse:function(e,t){return"元"===t[1]?1:parseInt(t[1]||e,10)},months:"1月_2月_3月_4月_5月_6月_7月_8月_9月_10月_11月_12月".split("_"),monthsShort:"1月_2月_3月_4月_5月_6月_7月_8月_9月_10月_11月_12月".split("_"),weekdays:"日曜日_月曜日_火曜日_水曜日_木曜日_金曜日_土曜日".split("_"),weekdaysShort:"日_月_火_水_木_金_土".split("_"),weekdaysMin:"日_月_火_水_木_金_土".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY/MM/DD",LL:"YYYY年M月D日",LLL:"YYYY年M月D日 HH:mm",LLLL:"YYYY年M月D日 dddd HH:mm",l:"YYYY/MM/DD",ll:"YYYY年M月D日",lll:"YYYY年M月D日 HH:mm",llll:"YYYY年M月D日(ddd) HH:mm"},meridiemParse:/午前|午後/i,isPM:function(e){return"午後"===e},meridiem:function(e,t,n){return e<12?"午前":"午後"},calendar:{sameDay:"[今日] LT",nextDay:"[明日] LT",nextWeek:function(e){return e.week()!==this.week()?"[来週]dddd LT":"dddd LT"},lastDay:"[昨日] LT",lastWeek:function(e){return this.week()!==e.week()?"[先週]dddd LT":"dddd LT"},sameElse:"L"},dayOfMonthOrdinalParse:/\d{1,2}日/,ordinal:function(e,t){switch(t){case"y":return 1===e?"元年":e+"年";case"d":case"D":case"DDD":return e+"日";default:return e}},relativeTime:{future:"%s後",past:"%s前",s:"数秒",ss:"%d秒",m:"1分",mm:"%d分",h:"1時間",hh:"%d時間",d:"1日",dd:"%d日",M:"1ヶ月",MM:"%dヶ月",y:"1年",yy:"%d年"}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("jv",{months:"Januari_Februari_Maret_April_Mei_Juni_Juli_Agustus_September_Oktober_Nopember_Desember".split("_"),monthsShort:"Jan_Feb_Mar_Apr_Mei_Jun_Jul_Ags_Sep_Okt_Nop_Des".split("_"),weekdays:"Minggu_Senen_Seloso_Rebu_Kemis_Jemuwah_Septu".split("_"),weekdaysShort:"Min_Sen_Sel_Reb_Kem_Jem_Sep".split("_"),weekdaysMin:"Mg_Sn_Sl_Rb_Km_Jm_Sp".split("_"),longDateFormat:{LT:"HH.mm",LTS:"HH.mm.ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY [pukul] HH.mm",LLLL:"dddd, D MMMM YYYY [pukul] HH.mm"},meridiemParse:/enjing|siyang|sonten|ndalu/,meridiemHour:function(e,t){return 12===e&&(e=0),"enjing"===t?e:"siyang"===t?e>=11?e:e+12:"sonten"===t||"ndalu"===t?e+12:void 0},meridiem:function(e,t,n){return e<11?"enjing":e<15?"siyang":e<19?"sonten":"ndalu"},calendar:{sameDay:"[Dinten puniko pukul] LT",nextDay:"[Mbenjang pukul] LT",nextWeek:"dddd [pukul] LT",lastDay:"[Kala wingi pukul] LT",lastWeek:"dddd [kepengker pukul] LT",sameElse:"L"},relativeTime:{future:"wonten ing %s",past:"%s ingkang kepengker",s:"sawetawis detik",ss:"%d detik",m:"setunggal menit",mm:"%d menit",h:"setunggal jam",hh:"%d jam",d:"sedinten",dd:"%d dinten",M:"sewulan",MM:"%d wulan",y:"setaun",yy:"%d taun"},week:{dow:1,doy:7}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("ka",{months:"იანვარი_თებერვალი_მარტი_აპრილი_მაისი_ივნისი_ივლისი_აგვისტო_სექტემბერი_ოქტომბერი_ნოემბერი_დეკემბერი".split("_"),monthsShort:"იან_თებ_მარ_აპრ_მაი_ივნ_ივლ_აგვ_სექ_ოქტ_ნოე_დეკ".split("_"),weekdays:{standalone:"კვირა_ორშაბათი_სამშაბათი_ოთხშაბათი_ხუთშაბათი_პარასკევი_შაბათი".split("_"),format:"კვირას_ორშაბათს_სამშაბათს_ოთხშაბათს_ხუთშაბათს_პარასკევს_შაბათს".split("_"),isFormat:/(წინა|შემდეგ)/},weekdaysShort:"კვი_ორშ_სამ_ოთხ_ხუთ_პარ_შაბ".split("_"),weekdaysMin:"კვ_ორ_სა_ოთ_ხუ_პა_შა".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[დღეს] LT[-ზე]",nextDay:"[ხვალ] LT[-ზე]",lastDay:"[გუშინ] LT[-ზე]",nextWeek:"[შემდეგ] dddd LT[-ზე]",lastWeek:"[წინა] dddd LT-ზე",sameElse:"L"},relativeTime:{future:function(e){return e.replace(/(წამ|წუთ|საათ|წელ|დღ|თვ)(ი|ე)/,(function(e,t,n){return"ი"===n?t+"ში":t+n+"ში"}))},past:function(e){return/(წამი|წუთი|საათი|დღე|თვე)/.test(e)?e.replace(/(ი|ე)$/,"ის წინ"):/წელი/.test(e)?e.replace(/წელი$/,"წლის წინ"):e},s:"რამდენიმე წამი",ss:"%d წამი",m:"წუთი",mm:"%d წუთი",h:"საათი",hh:"%d საათი",d:"დღე",dd:"%d დღე",M:"თვე",MM:"%d თვე",y:"წელი",yy:"%d წელი"},dayOfMonthOrdinalParse:/0|1-ლი|მე-\d{1,2}|\d{1,2}-ე/,ordinal:function(e){return 0===e?e:1===e?e+"-ლი":e<20||e<=100&&e%20==0||e%100==0?"მე-"+e:e+"-ე"},week:{dow:1,doy:7}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t={0:"-ші",1:"-ші",2:"-ші",3:"-ші",4:"-ші",5:"-ші",6:"-шы",7:"-ші",8:"-ші",9:"-шы",10:"-шы",20:"-шы",30:"-шы",40:"-шы",50:"-ші",60:"-шы",70:"-ші",80:"-ші",90:"-шы",100:"-ші"};e.defineLocale("kk",{months:"қаңтар_ақпан_наурыз_сәуір_мамыр_маусым_шілде_тамыз_қыркүйек_қазан_қараша_желтоқсан".split("_"),monthsShort:"қаң_ақп_нау_сәу_мам_мау_шіл_там_қыр_қаз_қар_жел".split("_"),weekdays:"жексенбі_дүйсенбі_сейсенбі_сәрсенбі_бейсенбі_жұма_сенбі".split("_"),weekdaysShort:"жек_дүй_сей_сәр_бей_жұм_сен".split("_"),weekdaysMin:"жк_дй_сй_ср_бй_жм_сн".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Бүгін сағат] LT",nextDay:"[Ертең сағат] LT",nextWeek:"dddd [сағат] LT",lastDay:"[Кеше сағат] LT",lastWeek:"[Өткен аптаның] dddd [сағат] LT",sameElse:"L"},relativeTime:{future:"%s ішінде",past:"%s бұрын",s:"бірнеше секунд",ss:"%d секунд",m:"бір минут",mm:"%d минут",h:"бір сағат",hh:"%d сағат",d:"бір күн",dd:"%d күн",M:"бір ай",MM:"%d ай",y:"бір жыл",yy:"%d жыл"},dayOfMonthOrdinalParse:/\d{1,2}-(ші|шы)/,ordinal:function(e){return e+(t[e]||t[e%10]||t[e>=100?100:null])},week:{dow:1,doy:7}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t={1:"១",2:"២",3:"៣",4:"៤",5:"៥",6:"៦",7:"៧",8:"៨",9:"៩",0:"០"},n={"១":"1","២":"2","៣":"3","៤":"4","៥":"5","៦":"6","៧":"7","៨":"8","៩":"9","០":"0"};e.defineLocale("km",{months:"មករា_កុម្ភៈ_មីនា_មេសា_ឧសភា_មិថុនា_កក្កដា_សីហា_កញ្ញា_តុលា_វិច្ឆិកា_ធ្នូ".split("_"),monthsShort:"មករា_កុម្ភៈ_មីនា_មេសា_ឧសភា_មិថុនា_កក្កដា_សីហា_កញ្ញា_តុលា_វិច្ឆិកា_ធ្នូ".split("_"),weekdays:"អាទិត្យ_ច័ន្ទ_អង្គារ_ពុធ_ព្រហស្បតិ៍_សុក្រ_សៅរ៍".split("_"),weekdaysShort:"អា_ច_អ_ព_ព្រ_សុ_ស".split("_"),weekdaysMin:"អា_ច_អ_ព_ព្រ_សុ_ស".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},meridiemParse:/ព្រឹក|ល្ងាច/,isPM:function(e){return"ល្ងាច"===e},meridiem:function(e,t,n){return e<12?"ព្រឹក":"ល្ងាច"},calendar:{sameDay:"[ថ្ងៃនេះ ម៉ោង] LT",nextDay:"[ស្អែក ម៉ោង] LT",nextWeek:"dddd [ម៉ោង] LT",lastDay:"[ម្សិលមិញ ម៉ោង] LT",lastWeek:"dddd [សប្តាហ៍មុន] [ម៉ោង] LT",sameElse:"L"},relativeTime:{future:"%sទៀត",past:"%sមុន",s:"ប៉ុន្មានវិនាទី",ss:"%d វិនាទី",m:"មួយនាទី",mm:"%d នាទី",h:"មួយម៉ោង",hh:"%d ម៉ោង",d:"មួយថ្ងៃ",dd:"%d ថ្ងៃ",M:"មួយខែ",MM:"%d ខែ",y:"មួយឆ្នាំ",yy:"%d ឆ្នាំ"},dayOfMonthOrdinalParse:/ទី\d{1,2}/,ordinal:"ទី%d",preparse:function(e){return e.replace(/[១២៣៤៥៦៧៨៩០]/g,(function(e){return n[e]}))},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]}))},week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t={1:"೧",2:"೨",3:"೩",4:"೪",5:"೫",6:"೬",7:"೭",8:"೮",9:"೯",0:"೦"},n={"೧":"1","೨":"2","೩":"3","೪":"4","೫":"5","೬":"6","೭":"7","೮":"8","೯":"9","೦":"0"};e.defineLocale("kn",{months:"ಜನವರಿ_ಫೆಬ್ರವರಿ_ಮಾರ್ಚ್_ಏಪ್ರಿಲ್_ಮೇ_ಜೂನ್_ಜುಲೈ_ಆಗಸ್ಟ್_ಸೆಪ್ಟೆಂಬರ್_ಅಕ್ಟೋಬರ್_ನವೆಂಬರ್_ಡಿಸೆಂಬರ್".split("_"),monthsShort:"ಜನ_ಫೆಬ್ರ_ಮಾರ್ಚ್_ಏಪ್ರಿಲ್_ಮೇ_ಜೂನ್_ಜುಲೈ_ಆಗಸ್ಟ್_ಸೆಪ್ಟೆಂ_ಅಕ್ಟೋ_ನವೆಂ_ಡಿಸೆಂ".split("_"),monthsParseExact:!0,weekdays:"ಭಾನುವಾರ_ಸೋಮವಾರ_ಮಂಗಳವಾರ_ಬುಧವಾರ_ಗುರುವಾರ_ಶುಕ್ರವಾರ_ಶನಿವಾರ".split("_"),weekdaysShort:"ಭಾನು_ಸೋಮ_ಮಂಗಳ_ಬುಧ_ಗುರು_ಶುಕ್ರ_ಶನಿ".split("_"),weekdaysMin:"ಭಾ_ಸೋ_ಮಂ_ಬು_ಗು_ಶು_ಶ".split("_"),longDateFormat:{LT:"A h:mm",LTS:"A h:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm",LLLL:"dddd, D MMMM YYYY, A h:mm"},calendar:{sameDay:"[ಇಂದು] LT",nextDay:"[ನಾಳೆ] LT",nextWeek:"dddd, LT",lastDay:"[ನಿನ್ನೆ] LT",lastWeek:"[ಕೊನೆಯ] dddd, LT",sameElse:"L"},relativeTime:{future:"%s ನಂತರ",past:"%s ಹಿಂದೆ",s:"ಕೆಲವು ಕ್ಷಣಗಳು",ss:"%d ಸೆಕೆಂಡುಗಳು",m:"ಒಂದು ನಿಮಿಷ",mm:"%d ನಿಮಿಷ",h:"ಒಂದು ಗಂಟೆ",hh:"%d ಗಂಟೆ",d:"ಒಂದು ದಿನ",dd:"%d ದಿನ",M:"ಒಂದು ತಿಂಗಳು",MM:"%d ತಿಂಗಳು",y:"ಒಂದು ವರ್ಷ",yy:"%d ವರ್ಷ"},preparse:function(e){return e.replace(/[೧೨೩೪೫೬೭೮೯೦]/g,(function(e){return n[e]}))},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]}))},meridiemParse:/ರಾತ್ರಿ|ಬೆಳಿಗ್ಗೆ|ಮಧ್ಯಾಹ್ನ|ಸಂಜೆ/,meridiemHour:function(e,t){return 12===e&&(e=0),"ರಾತ್ರಿ"===t?e<4?e:e+12:"ಬೆಳಿಗ್ಗೆ"===t?e:"ಮಧ್ಯಾಹ್ನ"===t?e>=10?e:e+12:"ಸಂಜೆ"===t?e+12:void 0},meridiem:function(e,t,n){return e<4?"ರಾತ್ರಿ":e<10?"ಬೆಳಿಗ್ಗೆ":e<17?"ಮಧ್ಯಾಹ್ನ":e<20?"ಸಂಜೆ":"ರಾತ್ರಿ"},dayOfMonthOrdinalParse:/\d{1,2}(ನೇ)/,ordinal:function(e){return e+"ನೇ"},week:{dow:0,doy:6}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("ko",{months:"1월_2월_3월_4월_5월_6월_7월_8월_9월_10월_11월_12월".split("_"),monthsShort:"1월_2월_3월_4월_5월_6월_7월_8월_9월_10월_11월_12월".split("_"),weekdays:"일요일_월요일_화요일_수요일_목요일_금요일_토요일".split("_"),weekdaysShort:"일_월_화_수_목_금_토".split("_"),weekdaysMin:"일_월_화_수_목_금_토".split("_"),longDateFormat:{LT:"A h:mm",LTS:"A h:mm:ss",L:"YYYY.MM.DD.",LL:"YYYY년 MMMM D일",LLL:"YYYY년 MMMM D일 A h:mm",LLLL:"YYYY년 MMMM D일 dddd A h:mm",l:"YYYY.MM.DD.",ll:"YYYY년 MMMM D일",lll:"YYYY년 MMMM D일 A h:mm",llll:"YYYY년 MMMM D일 dddd A h:mm"},calendar:{sameDay:"오늘 LT",nextDay:"내일 LT",nextWeek:"dddd LT",lastDay:"어제 LT",lastWeek:"지난주 dddd LT",sameElse:"L"},relativeTime:{future:"%s 후",past:"%s 전",s:"몇 초",ss:"%d초",m:"1분",mm:"%d분",h:"한 시간",hh:"%d시간",d:"하루",dd:"%d일",M:"한 달",MM:"%d달",y:"일 년",yy:"%d년"},dayOfMonthOrdinalParse:/\d{1,2}(일|월|주)/,ordinal:function(e,t){switch(t){case"d":case"D":case"DDD":return e+"일";case"M":return e+"월";case"w":case"W":return e+"주";default:return e}},meridiemParse:/오전|오후/,isPM:function(e){return"오후"===e},meridiem:function(e,t,n){return e<12?"오전":"오후"}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t={1:"١",2:"٢",3:"٣",4:"٤",5:"٥",6:"٦",7:"٧",8:"٨",9:"٩",0:"٠"},n={"١":"1","٢":"2","٣":"3","٤":"4","٥":"5","٦":"6","٧":"7","٨":"8","٩":"9","٠":"0"},r=["کانونی دووەم","شوبات","ئازار","نیسان","ئایار","حوزەیران","تەمموز","ئاب","ئەیلوول","تشرینی یەكەم","تشرینی دووەم","كانونی یەکەم"];e.defineLocale("ku",{months:r,monthsShort:r,weekdays:"یه‌كشه‌ممه‌_دووشه‌ممه‌_سێشه‌ممه‌_چوارشه‌ممه‌_پێنجشه‌ممه‌_هه‌ینی_شه‌ممه‌".split("_"),weekdaysShort:"یه‌كشه‌م_دووشه‌م_سێشه‌م_چوارشه‌م_پێنجشه‌م_هه‌ینی_شه‌ممه‌".split("_"),weekdaysMin:"ی_د_س_چ_پ_ه_ش".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},meridiemParse:/ئێواره‌|به‌یانی/,isPM:function(e){return/ئێواره‌/.test(e)},meridiem:function(e,t,n){return e<12?"به‌یانی":"ئێواره‌"},calendar:{sameDay:"[ئه‌مرۆ كاتژمێر] LT",nextDay:"[به‌یانی كاتژمێر] LT",nextWeek:"dddd [كاتژمێر] LT",lastDay:"[دوێنێ كاتژمێر] LT",lastWeek:"dddd [كاتژمێر] LT",sameElse:"L"},relativeTime:{future:"له‌ %s",past:"%s",s:"چه‌ند چركه‌یه‌ك",ss:"چركه‌ %d",m:"یه‌ك خوله‌ك",mm:"%d خوله‌ك",h:"یه‌ك كاتژمێر",hh:"%d كاتژمێر",d:"یه‌ك ڕۆژ",dd:"%d ڕۆژ",M:"یه‌ك مانگ",MM:"%d مانگ",y:"یه‌ك ساڵ",yy:"%d ساڵ"},preparse:function(e){return e.replace(/[١٢٣٤٥٦٧٨٩٠]/g,(function(e){return n[e]})).replace(/،/g,",")},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]})).replace(/,/g,"،")},week:{dow:6,doy:12}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t={0:"-чү",1:"-чи",2:"-чи",3:"-чү",4:"-чү",5:"-чи",6:"-чы",7:"-чи",8:"-чи",9:"-чу",10:"-чу",20:"-чы",30:"-чу",40:"-чы",50:"-чү",60:"-чы",70:"-чи",80:"-чи",90:"-чу",100:"-чү"};e.defineLocale("ky",{months:"январь_февраль_март_апрель_май_июнь_июль_август_сентябрь_октябрь_ноябрь_декабрь".split("_"),monthsShort:"янв_фев_март_апр_май_июнь_июль_авг_сен_окт_ноя_дек".split("_"),weekdays:"Жекшемби_Дүйшөмбү_Шейшемби_Шаршемби_Бейшемби_Жума_Ишемби".split("_"),weekdaysShort:"Жек_Дүй_Шей_Шар_Бей_Жум_Ише".split("_"),weekdaysMin:"Жк_Дй_Шй_Шр_Бй_Жм_Иш".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Бүгүн саат] LT",nextDay:"[Эртең саат] LT",nextWeek:"dddd [саат] LT",lastDay:"[Кечээ саат] LT",lastWeek:"[Өткөн аптанын] dddd [күнү] [саат] LT",sameElse:"L"},relativeTime:{future:"%s ичинде",past:"%s мурун",s:"бирнече секунд",ss:"%d секунд",m:"бир мүнөт",mm:"%d мүнөт",h:"бир саат",hh:"%d саат",d:"бир күн",dd:"%d күн",M:"бир ай",MM:"%d ай",y:"бир жыл",yy:"%d жыл"},dayOfMonthOrdinalParse:/\d{1,2}-(чи|чы|чү|чу)/,ordinal:function(e){return e+(t[e]||t[e%10]||t[e>=100?100:null])},week:{dow:1,doy:7}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
function t(e,t,n,r){var i={m:["eng Minutt","enger Minutt"],h:["eng Stonn","enger Stonn"],d:["een Dag","engem Dag"],M:["ee Mount","engem Mount"],y:["ee Joer","engem Joer"]};return t?i[n][0]:i[n][1]}function n(e){if(e=parseInt(e,10),isNaN(e))return!1;if(e<0)return!0;if(e<10)return 4<=e&&e<=7;if(e<100){var t=e%10;return n(0===t?e/10:t)}if(e<1e4){for(;e>=10;)e/=10;return n(e)}return n(e/=1e3)}e.defineLocale("lb",{months:"Januar_Februar_Mäerz_Abrëll_Mee_Juni_Juli_August_September_Oktober_November_Dezember".split("_"),monthsShort:"Jan._Febr._Mrz._Abr._Mee_Jun._Jul._Aug._Sept._Okt._Nov._Dez.".split("_"),monthsParseExact:!0,weekdays:"Sonndeg_Méindeg_Dënschdeg_Mëttwoch_Donneschdeg_Freideg_Samschdeg".split("_"),weekdaysShort:"So._Mé._Dë._Më._Do._Fr._Sa.".split("_"),weekdaysMin:"So_Mé_Dë_Më_Do_Fr_Sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm [Auer]",LTS:"H:mm:ss [Auer]",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm [Auer]",LLLL:"dddd, D. MMMM YYYY H:mm [Auer]"},calendar:{sameDay:"[Haut um] LT",sameElse:"L",nextDay:"[Muer um] LT",nextWeek:"dddd [um] LT",lastDay:"[Gëschter um] LT",lastWeek:function(){switch(this.day()){case 2:case 4:return"[Leschten] dddd [um] LT";default:return"[Leschte] dddd [um] LT"}}},relativeTime:{future:function(e){return n(e.substr(0,e.indexOf(" ")))?"a "+e:"an "+e},past:function(e){return n(e.substr(0,e.indexOf(" ")))?"viru "+e:"virun "+e},s:"e puer Sekonnen",ss:"%d Sekonnen",m:t,mm:"%d Minutten",h:t,hh:"%d Stonnen",d:t,dd:"%d Deeg",M:t,MM:"%d Méint",y:t,yy:"%d Joer"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("lo",{months:"ມັງກອນ_ກຸມພາ_ມີນາ_ເມສາ_ພຶດສະພາ_ມິຖຸນາ_ກໍລະກົດ_ສິງຫາ_ກັນຍາ_ຕຸລາ_ພະຈິກ_ທັນວາ".split("_"),monthsShort:"ມັງກອນ_ກຸມພາ_ມີນາ_ເມສາ_ພຶດສະພາ_ມິຖຸນາ_ກໍລະກົດ_ສິງຫາ_ກັນຍາ_ຕຸລາ_ພະຈິກ_ທັນວາ".split("_"),weekdays:"ອາທິດ_ຈັນ_ອັງຄານ_ພຸດ_ພະຫັດ_ສຸກ_ເສົາ".split("_"),weekdaysShort:"ທິດ_ຈັນ_ອັງຄານ_ພຸດ_ພະຫັດ_ສຸກ_ເສົາ".split("_"),weekdaysMin:"ທ_ຈ_ອຄ_ພ_ພຫ_ສກ_ສ".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"ວັນdddd D MMMM YYYY HH:mm"},meridiemParse:/ຕອນເຊົ້າ|ຕອນແລງ/,isPM:function(e){return"ຕອນແລງ"===e},meridiem:function(e,t,n){return e<12?"ຕອນເຊົ້າ":"ຕອນແລງ"},calendar:{sameDay:"[ມື້ນີ້ເວລາ] LT",nextDay:"[ມື້ອື່ນເວລາ] LT",nextWeek:"[ວັນ]dddd[ໜ້າເວລາ] LT",lastDay:"[ມື້ວານນີ້ເວລາ] LT",lastWeek:"[ວັນ]dddd[ແລ້ວນີ້ເວລາ] LT",sameElse:"L"},relativeTime:{future:"ອີກ %s",past:"%sຜ່ານມາ",s:"ບໍ່ເທົ່າໃດວິນາທີ",ss:"%d ວິນາທີ",m:"1 ນາທີ",mm:"%d ນາທີ",h:"1 ຊົ່ວໂມງ",hh:"%d ຊົ່ວໂມງ",d:"1 ມື້",dd:"%d ມື້",M:"1 ເດືອນ",MM:"%d ເດືອນ",y:"1 ປີ",yy:"%d ປີ"},dayOfMonthOrdinalParse:/(ທີ່)\d{1,2}/,ordinal:function(e){return"ທີ່"+e}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t={ss:"sekundė_sekundžių_sekundes",m:"minutė_minutės_minutę",mm:"minutės_minučių_minutes",h:"valanda_valandos_valandą",hh:"valandos_valandų_valandas",d:"diena_dienos_dieną",dd:"dienos_dienų_dienas",M:"mėnuo_mėnesio_mėnesį",MM:"mėnesiai_mėnesių_mėnesius",y:"metai_metų_metus",yy:"metai_metų_metus"};function n(e,t,n,r){return t?i(n)[0]:r?i(n)[1]:i(n)[2]}function r(e){return e%10==0||e>10&&e<20}function i(e){return t[e].split("_")}function o(e,t,o,a){var s=e+" ";return 1===e?s+n(0,t,o[0],a):t?s+(r(e)?i(o)[1]:i(o)[0]):a?s+i(o)[1]:s+(r(e)?i(o)[1]:i(o)[2])}e.defineLocale("lt",{months:{format:"sausio_vasario_kovo_balandžio_gegužės_birželio_liepos_rugpjūčio_rugsėjo_spalio_lapkričio_gruodžio".split("_"),standalone:"sausis_vasaris_kovas_balandis_gegužė_birželis_liepa_rugpjūtis_rugsėjis_spalis_lapkritis_gruodis".split("_"),isFormat:/D[oD]?(\[[^\[\]]*\]|\s)+MMMM?|MMMM?(\[[^\[\]]*\]|\s)+D[oD]?/},monthsShort:"sau_vas_kov_bal_geg_bir_lie_rgp_rgs_spa_lap_grd".split("_"),weekdays:{format:"sekmadienį_pirmadienį_antradienį_trečiadienį_ketvirtadienį_penktadienį_šeštadienį".split("_"),standalone:"sekmadienis_pirmadienis_antradienis_trečiadienis_ketvirtadienis_penktadienis_šeštadienis".split("_"),isFormat:/dddd HH:mm/},weekdaysShort:"Sek_Pir_Ant_Tre_Ket_Pen_Šeš".split("_"),weekdaysMin:"S_P_A_T_K_Pn_Š".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"YYYY [m.] MMMM D [d.]",LLL:"YYYY [m.] MMMM D [d.], HH:mm [val.]",LLLL:"YYYY [m.] MMMM D [d.], dddd, HH:mm [val.]",l:"YYYY-MM-DD",ll:"YYYY [m.] MMMM D [d.]",lll:"YYYY [m.] MMMM D [d.], HH:mm [val.]",llll:"YYYY [m.] MMMM D [d.], ddd, HH:mm [val.]"},calendar:{sameDay:"[Šiandien] LT",nextDay:"[Rytoj] LT",nextWeek:"dddd LT",lastDay:"[Vakar] LT",lastWeek:"[Praėjusį] dddd LT",sameElse:"L"},relativeTime:{future:"po %s",past:"prieš %s",s:function(e,t,n,r){return t?"kelios sekundės":r?"kelių sekundžių":"kelias sekundes"},ss:o,m:n,mm:o,h:n,hh:o,d:n,dd:o,M:n,MM:o,y:n,yy:o},dayOfMonthOrdinalParse:/\d{1,2}-oji/,ordinal:function(e){return e+"-oji"},week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t={ss:"sekundes_sekundēm_sekunde_sekundes".split("_"),m:"minūtes_minūtēm_minūte_minūtes".split("_"),mm:"minūtes_minūtēm_minūte_minūtes".split("_"),h:"stundas_stundām_stunda_stundas".split("_"),hh:"stundas_stundām_stunda_stundas".split("_"),d:"dienas_dienām_diena_dienas".split("_"),dd:"dienas_dienām_diena_dienas".split("_"),M:"mēneša_mēnešiem_mēnesis_mēneši".split("_"),MM:"mēneša_mēnešiem_mēnesis_mēneši".split("_"),y:"gada_gadiem_gads_gadi".split("_"),yy:"gada_gadiem_gads_gadi".split("_")};function n(e,t,n){return n?t%10==1&&t%100!=11?e[2]:e[3]:t%10==1&&t%100!=11?e[0]:e[1]}function r(e,r,i){return e+" "+n(t[i],e,r)}function i(e,r,i){return n(t[i],e,r)}e.defineLocale("lv",{months:"janvāris_februāris_marts_aprīlis_maijs_jūnijs_jūlijs_augusts_septembris_oktobris_novembris_decembris".split("_"),monthsShort:"jan_feb_mar_apr_mai_jūn_jūl_aug_sep_okt_nov_dec".split("_"),weekdays:"svētdiena_pirmdiena_otrdiena_trešdiena_ceturtdiena_piektdiena_sestdiena".split("_"),weekdaysShort:"Sv_P_O_T_C_Pk_S".split("_"),weekdaysMin:"Sv_P_O_T_C_Pk_S".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY.",LL:"YYYY. [gada] D. MMMM",LLL:"YYYY. [gada] D. MMMM, HH:mm",LLLL:"YYYY. [gada] D. MMMM, dddd, HH:mm"},calendar:{sameDay:"[Šodien pulksten] LT",nextDay:"[Rīt pulksten] LT",nextWeek:"dddd [pulksten] LT",lastDay:"[Vakar pulksten] LT",lastWeek:"[Pagājušā] dddd [pulksten] LT",sameElse:"L"},relativeTime:{future:"pēc %s",past:"pirms %s",s:function(e,t){return t?"dažas sekundes":"dažām sekundēm"},ss:r,m:i,mm:r,h:i,hh:r,d:i,dd:r,M:i,MM:r,y:i,yy:r},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t={words:{ss:["sekund","sekunda","sekundi"],m:["jedan minut","jednog minuta"],mm:["minut","minuta","minuta"],h:["jedan sat","jednog sata"],hh:["sat","sata","sati"],dd:["dan","dana","dana"],MM:["mjesec","mjeseca","mjeseci"],yy:["godina","godine","godina"]},correctGrammaticalCase:function(e,t){return 1===e?t[0]:e>=2&&e<=4?t[1]:t[2]},translate:function(e,n,r){var i=t.words[r];return 1===r.length?n?i[0]:i[1]:e+" "+t.correctGrammaticalCase(e,i)}};e.defineLocale("me",{months:"januar_februar_mart_april_maj_jun_jul_avgust_septembar_oktobar_novembar_decembar".split("_"),monthsShort:"jan._feb._mar._apr._maj_jun_jul_avg._sep._okt._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"nedjelja_ponedjeljak_utorak_srijeda_četvrtak_petak_subota".split("_"),weekdaysShort:"ned._pon._uto._sri._čet._pet._sub.".split("_"),weekdaysMin:"ne_po_ut_sr_če_pe_su".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd, D. MMMM YYYY H:mm"},calendar:{sameDay:"[danas u] LT",nextDay:"[sjutra u] LT",nextWeek:function(){switch(this.day()){case 0:return"[u] [nedjelju] [u] LT";case 3:return"[u] [srijedu] [u] LT";case 6:return"[u] [subotu] [u] LT";case 1:case 2:case 4:case 5:return"[u] dddd [u] LT"}},lastDay:"[juče u] LT",lastWeek:function(){return["[prošle] [nedjelje] [u] LT","[prošlog] [ponedjeljka] [u] LT","[prošlog] [utorka] [u] LT","[prošle] [srijede] [u] LT","[prošlog] [četvrtka] [u] LT","[prošlog] [petka] [u] LT","[prošle] [subote] [u] LT"][this.day()]},sameElse:"L"},relativeTime:{future:"za %s",past:"prije %s",s:"nekoliko sekundi",ss:t.translate,m:t.translate,mm:t.translate,h:t.translate,hh:t.translate,d:"dan",dd:t.translate,M:"mjesec",MM:t.translate,y:"godinu",yy:t.translate},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:7}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("mi",{months:"Kohi-tāte_Hui-tanguru_Poutū-te-rangi_Paenga-whāwhā_Haratua_Pipiri_Hōngoingoi_Here-turi-kōkā_Mahuru_Whiringa-ā-nuku_Whiringa-ā-rangi_Hakihea".split("_"),monthsShort:"Kohi_Hui_Pou_Pae_Hara_Pipi_Hōngoi_Here_Mahu_Whi-nu_Whi-ra_Haki".split("_"),monthsRegex:/(?:['a-z\u0101\u014D\u016B]+\-?){1,3}/i,monthsStrictRegex:/(?:['a-z\u0101\u014D\u016B]+\-?){1,3}/i,monthsShortRegex:/(?:['a-z\u0101\u014D\u016B]+\-?){1,3}/i,monthsShortStrictRegex:/(?:['a-z\u0101\u014D\u016B]+\-?){1,2}/i,weekdays:"Rātapu_Mane_Tūrei_Wenerei_Tāite_Paraire_Hātarei".split("_"),weekdaysShort:"Ta_Ma_Tū_We_Tāi_Pa_Hā".split("_"),weekdaysMin:"Ta_Ma_Tū_We_Tāi_Pa_Hā".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY [i] HH:mm",LLLL:"dddd, D MMMM YYYY [i] HH:mm"},calendar:{sameDay:"[i teie mahana, i] LT",nextDay:"[apopo i] LT",nextWeek:"dddd [i] LT",lastDay:"[inanahi i] LT",lastWeek:"dddd [whakamutunga i] LT",sameElse:"L"},relativeTime:{future:"i roto i %s",past:"%s i mua",s:"te hēkona ruarua",ss:"%d hēkona",m:"he meneti",mm:"%d meneti",h:"te haora",hh:"%d haora",d:"he ra",dd:"%d ra",M:"he marama",MM:"%d marama",y:"he tau",yy:"%d tau"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("mk",{months:"јануари_февруари_март_април_мај_јуни_јули_август_септември_октомври_ноември_декември".split("_"),monthsShort:"јан_фев_мар_апр_мај_јун_јул_авг_сеп_окт_ное_дек".split("_"),weekdays:"недела_понеделник_вторник_среда_четврток_петок_сабота".split("_"),weekdaysShort:"нед_пон_вто_сре_чет_пет_саб".split("_"),weekdaysMin:"нe_пo_вт_ср_че_пе_сa".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"D.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY H:mm",LLLL:"dddd, D MMMM YYYY H:mm"},calendar:{sameDay:"[Денес во] LT",nextDay:"[Утре во] LT",nextWeek:"[Во] dddd [во] LT",lastDay:"[Вчера во] LT",lastWeek:function(){switch(this.day()){case 0:case 3:case 6:return"[Изминатата] dddd [во] LT";case 1:case 2:case 4:case 5:return"[Изминатиот] dddd [во] LT"}},sameElse:"L"},relativeTime:{future:"за %s",past:"пред %s",s:"неколку секунди",ss:"%d секунди",m:"една минута",mm:"%d минути",h:"еден час",hh:"%d часа",d:"еден ден",dd:"%d дена",M:"еден месец",MM:"%d месеци",y:"една година",yy:"%d години"},dayOfMonthOrdinalParse:/\d{1,2}-(ев|ен|ти|ви|ри|ми)/,ordinal:function(e){var t=e%10,n=e%100;return 0===e?e+"-ев":0===n?e+"-ен":n>10&&n<20?e+"-ти":1===t?e+"-ви":2===t?e+"-ри":7===t||8===t?e+"-ми":e+"-ти"},week:{dow:1,doy:7}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("ml",{months:"ജനുവരി_ഫെബ്രുവരി_മാർച്ച്_ഏപ്രിൽ_മേയ്_ജൂൺ_ജൂലൈ_ഓഗസ്റ്റ്_സെപ്റ്റംബർ_ഒക്ടോബർ_നവംബർ_ഡിസംബർ".split("_"),monthsShort:"ജനു._ഫെബ്രു._മാർ._ഏപ്രി._മേയ്_ജൂൺ_ജൂലൈ._ഓഗ._സെപ്റ്റ._ഒക്ടോ._നവം._ഡിസം.".split("_"),monthsParseExact:!0,weekdays:"ഞായറാഴ്ച_തിങ്കളാഴ്ച_ചൊവ്വാഴ്ച_ബുധനാഴ്ച_വ്യാഴാഴ്ച_വെള്ളിയാഴ്ച_ശനിയാഴ്ച".split("_"),weekdaysShort:"ഞായർ_തിങ്കൾ_ചൊവ്വ_ബുധൻ_വ്യാഴം_വെള്ളി_ശനി".split("_"),weekdaysMin:"ഞാ_തി_ചൊ_ബു_വ്യാ_വെ_ശ".split("_"),longDateFormat:{LT:"A h:mm -നു",LTS:"A h:mm:ss -നു",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm -നു",LLLL:"dddd, D MMMM YYYY, A h:mm -നു"},calendar:{sameDay:"[ഇന്ന്] LT",nextDay:"[നാളെ] LT",nextWeek:"dddd, LT",lastDay:"[ഇന്നലെ] LT",lastWeek:"[കഴിഞ്ഞ] dddd, LT",sameElse:"L"},relativeTime:{future:"%s കഴിഞ്ഞ്",past:"%s മുൻപ്",s:"അൽപ നിമിഷങ്ങൾ",ss:"%d സെക്കൻഡ്",m:"ഒരു മിനിറ്റ്",mm:"%d മിനിറ്റ്",h:"ഒരു മണിക്കൂർ",hh:"%d മണിക്കൂർ",d:"ഒരു ദിവസം",dd:"%d ദിവസം",M:"ഒരു മാസം",MM:"%d മാസം",y:"ഒരു വർഷം",yy:"%d വർഷം"},meridiemParse:/രാത്രി|രാവിലെ|ഉച്ച കഴിഞ്ഞ്|വൈകുന്നേരം|രാത്രി/i,meridiemHour:function(e,t){return 12===e&&(e=0),"രാത്രി"===t&&e>=4||"ഉച്ച കഴിഞ്ഞ്"===t||"വൈകുന്നേരം"===t?e+12:e},meridiem:function(e,t,n){return e<4?"രാത്രി":e<12?"രാവിലെ":e<17?"ഉച്ച കഴിഞ്ഞ്":e<20?"വൈകുന്നേരം":"രാത്രി"}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
function t(e,t,n,r){switch(n){case"s":return t?"хэдхэн секунд":"хэдхэн секундын";case"ss":return e+(t?" секунд":" секундын");case"m":case"mm":return e+(t?" минут":" минутын");case"h":case"hh":return e+(t?" цаг":" цагийн");case"d":case"dd":return e+(t?" өдөр":" өдрийн");case"M":case"MM":return e+(t?" сар":" сарын");case"y":case"yy":return e+(t?" жил":" жилийн");default:return e}}e.defineLocale("mn",{months:"Нэгдүгээр сар_Хоёрдугаар сар_Гуравдугаар сар_Дөрөвдүгээр сар_Тавдугаар сар_Зургадугаар сар_Долдугаар сар_Наймдугаар сар_Есдүгээр сар_Аравдугаар сар_Арван нэгдүгээр сар_Арван хоёрдугаар сар".split("_"),monthsShort:"1 сар_2 сар_3 сар_4 сар_5 сар_6 сар_7 сар_8 сар_9 сар_10 сар_11 сар_12 сар".split("_"),monthsParseExact:!0,weekdays:"Ням_Даваа_Мягмар_Лхагва_Пүрэв_Баасан_Бямба".split("_"),weekdaysShort:"Ням_Дав_Мяг_Лха_Пүр_Баа_Бям".split("_"),weekdaysMin:"Ня_Да_Мя_Лх_Пү_Ба_Бя".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"YYYY оны MMMMын D",LLL:"YYYY оны MMMMын D HH:mm",LLLL:"dddd, YYYY оны MMMMын D HH:mm"},meridiemParse:/ҮӨ|ҮХ/i,isPM:function(e){return"ҮХ"===e},meridiem:function(e,t,n){return e<12?"ҮӨ":"ҮХ"},calendar:{sameDay:"[Өнөөдөр] LT",nextDay:"[Маргааш] LT",nextWeek:"[Ирэх] dddd LT",lastDay:"[Өчигдөр] LT",lastWeek:"[Өнгөрсөн] dddd LT",sameElse:"L"},relativeTime:{future:"%s дараа",past:"%s өмнө",s:t,ss:t,m:t,mm:t,h:t,hh:t,d:t,dd:t,M:t,MM:t,y:t,yy:t},dayOfMonthOrdinalParse:/\d{1,2} өдөр/,ordinal:function(e,t){switch(t){case"d":case"D":case"DDD":return e+" өдөр";default:return e}}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t={1:"१",2:"२",3:"३",4:"४",5:"५",6:"६",7:"७",8:"८",9:"९",0:"०"},n={"१":"1","२":"2","३":"3","४":"4","५":"5","६":"6","७":"7","८":"8","९":"9","०":"0"};function r(e,t,n,r){var i="";if(t)switch(n){case"s":i="काही सेकंद";break;case"ss":i="%d सेकंद";break;case"m":i="एक मिनिट";break;case"mm":i="%d मिनिटे";break;case"h":i="एक तास";break;case"hh":i="%d तास";break;case"d":i="एक दिवस";break;case"dd":i="%d दिवस";break;case"M":i="एक महिना";break;case"MM":i="%d महिने";break;case"y":i="एक वर्ष";break;case"yy":i="%d वर्षे"}else switch(n){case"s":i="काही सेकंदां";break;case"ss":i="%d सेकंदां";break;case"m":i="एका मिनिटा";break;case"mm":i="%d मिनिटां";break;case"h":i="एका तासा";break;case"hh":i="%d तासां";break;case"d":i="एका दिवसा";break;case"dd":i="%d दिवसां";break;case"M":i="एका महिन्या";break;case"MM":i="%d महिन्यां";break;case"y":i="एका वर्षा";break;case"yy":i="%d वर्षां"}return i.replace(/%d/i,e)}e.defineLocale("mr",{months:"जानेवारी_फेब्रुवारी_मार्च_एप्रिल_मे_जून_जुलै_ऑगस्ट_सप्टेंबर_ऑक्टोबर_नोव्हेंबर_डिसेंबर".split("_"),monthsShort:"जाने._फेब्रु._मार्च._एप्रि._मे._जून._जुलै._ऑग._सप्टें._ऑक्टो._नोव्हें._डिसें.".split("_"),monthsParseExact:!0,weekdays:"रविवार_सोमवार_मंगळवार_बुधवार_गुरूवार_शुक्रवार_शनिवार".split("_"),weekdaysShort:"रवि_सोम_मंगळ_बुध_गुरू_शुक्र_शनि".split("_"),weekdaysMin:"र_सो_मं_बु_गु_शु_श".split("_"),longDateFormat:{LT:"A h:mm वाजता",LTS:"A h:mm:ss वाजता",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm वाजता",LLLL:"dddd, D MMMM YYYY, A h:mm वाजता"},calendar:{sameDay:"[आज] LT",nextDay:"[उद्या] LT",nextWeek:"dddd, LT",lastDay:"[काल] LT",lastWeek:"[मागील] dddd, LT",sameElse:"L"},relativeTime:{future:"%sमध्ये",past:"%sपूर्वी",s:r,ss:r,m:r,mm:r,h:r,hh:r,d:r,dd:r,M:r,MM:r,y:r,yy:r},preparse:function(e){return e.replace(/[१२३४५६७८९०]/g,(function(e){return n[e]}))},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]}))},meridiemParse:/पहाटे|सकाळी|दुपारी|सायंकाळी|रात्री/,meridiemHour:function(e,t){return 12===e&&(e=0),"पहाटे"===t||"सकाळी"===t?e:"दुपारी"===t||"सायंकाळी"===t||"रात्री"===t?e>=12?e:e+12:void 0},meridiem:function(e,t,n){return e>=0&&e<6?"पहाटे":e<12?"सकाळी":e<17?"दुपारी":e<20?"सायंकाळी":"रात्री"},week:{dow:0,doy:6}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("ms",{months:"Januari_Februari_Mac_April_Mei_Jun_Julai_Ogos_September_Oktober_November_Disember".split("_"),monthsShort:"Jan_Feb_Mac_Apr_Mei_Jun_Jul_Ogs_Sep_Okt_Nov_Dis".split("_"),weekdays:"Ahad_Isnin_Selasa_Rabu_Khamis_Jumaat_Sabtu".split("_"),weekdaysShort:"Ahd_Isn_Sel_Rab_Kha_Jum_Sab".split("_"),weekdaysMin:"Ah_Is_Sl_Rb_Km_Jm_Sb".split("_"),longDateFormat:{LT:"HH.mm",LTS:"HH.mm.ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY [pukul] HH.mm",LLLL:"dddd, D MMMM YYYY [pukul] HH.mm"},meridiemParse:/pagi|tengahari|petang|malam/,meridiemHour:function(e,t){return 12===e&&(e=0),"pagi"===t?e:"tengahari"===t?e>=11?e:e+12:"petang"===t||"malam"===t?e+12:void 0},meridiem:function(e,t,n){return e<11?"pagi":e<15?"tengahari":e<19?"petang":"malam"},calendar:{sameDay:"[Hari ini pukul] LT",nextDay:"[Esok pukul] LT",nextWeek:"dddd [pukul] LT",lastDay:"[Kelmarin pukul] LT",lastWeek:"dddd [lepas pukul] LT",sameElse:"L"},relativeTime:{future:"dalam %s",past:"%s yang lepas",s:"beberapa saat",ss:"%d saat",m:"seminit",mm:"%d minit",h:"sejam",hh:"%d jam",d:"sehari",dd:"%d hari",M:"sebulan",MM:"%d bulan",y:"setahun",yy:"%d tahun"},week:{dow:1,doy:7}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("ms-my",{months:"Januari_Februari_Mac_April_Mei_Jun_Julai_Ogos_September_Oktober_November_Disember".split("_"),monthsShort:"Jan_Feb_Mac_Apr_Mei_Jun_Jul_Ogs_Sep_Okt_Nov_Dis".split("_"),weekdays:"Ahad_Isnin_Selasa_Rabu_Khamis_Jumaat_Sabtu".split("_"),weekdaysShort:"Ahd_Isn_Sel_Rab_Kha_Jum_Sab".split("_"),weekdaysMin:"Ah_Is_Sl_Rb_Km_Jm_Sb".split("_"),longDateFormat:{LT:"HH.mm",LTS:"HH.mm.ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY [pukul] HH.mm",LLLL:"dddd, D MMMM YYYY [pukul] HH.mm"},meridiemParse:/pagi|tengahari|petang|malam/,meridiemHour:function(e,t){return 12===e&&(e=0),"pagi"===t?e:"tengahari"===t?e>=11?e:e+12:"petang"===t||"malam"===t?e+12:void 0},meridiem:function(e,t,n){return e<11?"pagi":e<15?"tengahari":e<19?"petang":"malam"},calendar:{sameDay:"[Hari ini pukul] LT",nextDay:"[Esok pukul] LT",nextWeek:"dddd [pukul] LT",lastDay:"[Kelmarin pukul] LT",lastWeek:"dddd [lepas pukul] LT",sameElse:"L"},relativeTime:{future:"dalam %s",past:"%s yang lepas",s:"beberapa saat",ss:"%d saat",m:"seminit",mm:"%d minit",h:"sejam",hh:"%d jam",d:"sehari",dd:"%d hari",M:"sebulan",MM:"%d bulan",y:"setahun",yy:"%d tahun"},week:{dow:1,doy:7}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("mt",{months:"Jannar_Frar_Marzu_April_Mejju_Ġunju_Lulju_Awwissu_Settembru_Ottubru_Novembru_Diċembru".split("_"),monthsShort:"Jan_Fra_Mar_Apr_Mej_Ġun_Lul_Aww_Set_Ott_Nov_Diċ".split("_"),weekdays:"Il-Ħadd_It-Tnejn_It-Tlieta_L-Erbgħa_Il-Ħamis_Il-Ġimgħa_Is-Sibt".split("_"),weekdaysShort:"Ħad_Tne_Tli_Erb_Ħam_Ġim_Sib".split("_"),weekdaysMin:"Ħa_Tn_Tl_Er_Ħa_Ġi_Si".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Illum fil-]LT",nextDay:"[Għada fil-]LT",nextWeek:"dddd [fil-]LT",lastDay:"[Il-bieraħ fil-]LT",lastWeek:"dddd [li għadda] [fil-]LT",sameElse:"L"},relativeTime:{future:"f’ %s",past:"%s ilu",s:"ftit sekondi",ss:"%d sekondi",m:"minuta",mm:"%d minuti",h:"siegħa",hh:"%d siegħat",d:"ġurnata",dd:"%d ġranet",M:"xahar",MM:"%d xhur",y:"sena",yy:"%d sni"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t={1:"၁",2:"၂",3:"၃",4:"၄",5:"၅",6:"၆",7:"၇",8:"၈",9:"၉",0:"၀"},n={"၁":"1","၂":"2","၃":"3","၄":"4","၅":"5","၆":"6","၇":"7","၈":"8","၉":"9","၀":"0"};e.defineLocale("my",{months:"ဇန်နဝါရီ_ဖေဖော်ဝါရီ_မတ်_ဧပြီ_မေ_ဇွန်_ဇူလိုင်_သြဂုတ်_စက်တင်ဘာ_အောက်တိုဘာ_နိုဝင်ဘာ_ဒီဇင်ဘာ".split("_"),monthsShort:"ဇန်_ဖေ_မတ်_ပြီ_မေ_ဇွန်_လိုင်_သြ_စက်_အောက်_နို_ဒီ".split("_"),weekdays:"တနင်္ဂနွေ_တနင်္လာ_အင်္ဂါ_ဗုဒ္ဓဟူး_ကြာသပတေး_သောကြာ_စနေ".split("_"),weekdaysShort:"နွေ_လာ_ဂါ_ဟူး_ကြာ_သော_နေ".split("_"),weekdaysMin:"နွေ_လာ_ဂါ_ဟူး_ကြာ_သော_နေ".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[ယနေ.] LT [မှာ]",nextDay:"[မနက်ဖြန်] LT [မှာ]",nextWeek:"dddd LT [မှာ]",lastDay:"[မနေ.က] LT [မှာ]",lastWeek:"[ပြီးခဲ့သော] dddd LT [မှာ]",sameElse:"L"},relativeTime:{future:"လာမည့် %s မှာ",past:"လွန်ခဲ့သော %s က",s:"စက္ကန်.အနည်းငယ်",ss:"%d စက္ကန့်",m:"တစ်မိနစ်",mm:"%d မိနစ်",h:"တစ်နာရီ",hh:"%d နာရီ",d:"တစ်ရက်",dd:"%d ရက်",M:"တစ်လ",MM:"%d လ",y:"တစ်နှစ်",yy:"%d နှစ်"},preparse:function(e){return e.replace(/[၁၂၃၄၅၆၇၈၉၀]/g,(function(e){return n[e]}))},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]}))},week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("nb",{months:"januar_februar_mars_april_mai_juni_juli_august_september_oktober_november_desember".split("_"),monthsShort:"jan._feb._mars_apr._mai_juni_juli_aug._sep._okt._nov._des.".split("_"),monthsParseExact:!0,weekdays:"søndag_mandag_tirsdag_onsdag_torsdag_fredag_lørdag".split("_"),weekdaysShort:"sø._ma._ti._on._to._fr._lø.".split("_"),weekdaysMin:"sø_ma_ti_on_to_fr_lø".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY [kl.] HH:mm",LLLL:"dddd D. MMMM YYYY [kl.] HH:mm"},calendar:{sameDay:"[i dag kl.] LT",nextDay:"[i morgen kl.] LT",nextWeek:"dddd [kl.] LT",lastDay:"[i går kl.] LT",lastWeek:"[forrige] dddd [kl.] LT",sameElse:"L"},relativeTime:{future:"om %s",past:"%s siden",s:"noen sekunder",ss:"%d sekunder",m:"ett minutt",mm:"%d minutter",h:"en time",hh:"%d timer",d:"en dag",dd:"%d dager",w:"en uke",ww:"%d uker",M:"en måned",MM:"%d måneder",y:"ett år",yy:"%d år"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t={1:"१",2:"२",3:"३",4:"४",5:"५",6:"६",7:"७",8:"८",9:"९",0:"०"},n={"१":"1","२":"2","३":"3","४":"4","५":"5","६":"6","७":"7","८":"8","९":"9","०":"0"};e.defineLocale("ne",{months:"जनवरी_फेब्रुवरी_मार्च_अप्रिल_मई_जुन_जुलाई_अगष्ट_सेप्टेम्बर_अक्टोबर_नोभेम्बर_डिसेम्बर".split("_"),monthsShort:"जन._फेब्रु._मार्च_अप्रि._मई_जुन_जुलाई._अग._सेप्ट._अक्टो._नोभे._डिसे.".split("_"),monthsParseExact:!0,weekdays:"आइतबार_सोमबार_मङ्गलबार_बुधबार_बिहिबार_शुक्रबार_शनिबार".split("_"),weekdaysShort:"आइत._सोम._मङ्गल._बुध._बिहि._शुक्र._शनि.".split("_"),weekdaysMin:"आ._सो._मं._बु._बि._शु._श.".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"Aको h:mm बजे",LTS:"Aको h:mm:ss बजे",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, Aको h:mm बजे",LLLL:"dddd, D MMMM YYYY, Aको h:mm बजे"},preparse:function(e){return e.replace(/[१२३४५६७८९०]/g,(function(e){return n[e]}))},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]}))},meridiemParse:/राति|बिहान|दिउँसो|साँझ/,meridiemHour:function(e,t){return 12===e&&(e=0),"राति"===t?e<4?e:e+12:"बिहान"===t?e:"दिउँसो"===t?e>=10?e:e+12:"साँझ"===t?e+12:void 0},meridiem:function(e,t,n){return e<3?"राति":e<12?"बिहान":e<16?"दिउँसो":e<20?"साँझ":"राति"},calendar:{sameDay:"[आज] LT",nextDay:"[भोलि] LT",nextWeek:"[आउँदो] dddd[,] LT",lastDay:"[हिजो] LT",lastWeek:"[गएको] dddd[,] LT",sameElse:"L"},relativeTime:{future:"%sमा",past:"%s अगाडि",s:"केही क्षण",ss:"%d सेकेण्ड",m:"एक मिनेट",mm:"%d मिनेट",h:"एक घण्टा",hh:"%d घण्टा",d:"एक दिन",dd:"%d दिन",M:"एक महिना",MM:"%d महिना",y:"एक बर्ष",yy:"%d बर्ष"},week:{dow:0,doy:6}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t="jan._feb._mrt._apr._mei_jun._jul._aug._sep._okt._nov._dec.".split("_"),n="jan_feb_mrt_apr_mei_jun_jul_aug_sep_okt_nov_dec".split("_"),r=[/^jan/i,/^feb/i,/^maart|mrt.?$/i,/^apr/i,/^mei$/i,/^jun[i.]?$/i,/^jul[i.]?$/i,/^aug/i,/^sep/i,/^okt/i,/^nov/i,/^dec/i],i=/^(januari|februari|maart|april|mei|ju[nl]i|augustus|september|oktober|november|december|jan\.?|feb\.?|mrt\.?|apr\.?|ju[nl]\.?|aug\.?|sep\.?|okt\.?|nov\.?|dec\.?)/i;e.defineLocale("nl",{months:"januari_februari_maart_april_mei_juni_juli_augustus_september_oktober_november_december".split("_"),monthsShort:function(e,r){return e?/-MMM-/.test(r)?n[e.month()]:t[e.month()]:t},monthsRegex:i,monthsShortRegex:i,monthsStrictRegex:/^(januari|februari|maart|april|mei|ju[nl]i|augustus|september|oktober|november|december)/i,monthsShortStrictRegex:/^(jan\.?|feb\.?|mrt\.?|apr\.?|mei|ju[nl]\.?|aug\.?|sep\.?|okt\.?|nov\.?|dec\.?)/i,monthsParse:r,longMonthsParse:r,shortMonthsParse:r,weekdays:"zondag_maandag_dinsdag_woensdag_donderdag_vrijdag_zaterdag".split("_"),weekdaysShort:"zo._ma._di._wo._do._vr._za.".split("_"),weekdaysMin:"zo_ma_di_wo_do_vr_za".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD-MM-YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[vandaag om] LT",nextDay:"[morgen om] LT",nextWeek:"dddd [om] LT",lastDay:"[gisteren om] LT",lastWeek:"[afgelopen] dddd [om] LT",sameElse:"L"},relativeTime:{future:"over %s",past:"%s geleden",s:"een paar seconden",ss:"%d seconden",m:"één minuut",mm:"%d minuten",h:"één uur",hh:"%d uur",d:"één dag",dd:"%d dagen",w:"één week",ww:"%d weken",M:"één maand",MM:"%d maanden",y:"één jaar",yy:"%d jaar"},dayOfMonthOrdinalParse:/\d{1,2}(ste|de)/,ordinal:function(e){return e+(1===e||8===e||e>=20?"ste":"de")},week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t="jan._feb._mrt._apr._mei_jun._jul._aug._sep._okt._nov._dec.".split("_"),n="jan_feb_mrt_apr_mei_jun_jul_aug_sep_okt_nov_dec".split("_"),r=[/^jan/i,/^feb/i,/^maart|mrt.?$/i,/^apr/i,/^mei$/i,/^jun[i.]?$/i,/^jul[i.]?$/i,/^aug/i,/^sep/i,/^okt/i,/^nov/i,/^dec/i],i=/^(januari|februari|maart|april|mei|ju[nl]i|augustus|september|oktober|november|december|jan\.?|feb\.?|mrt\.?|apr\.?|ju[nl]\.?|aug\.?|sep\.?|okt\.?|nov\.?|dec\.?)/i;e.defineLocale("nl-be",{months:"januari_februari_maart_april_mei_juni_juli_augustus_september_oktober_november_december".split("_"),monthsShort:function(e,r){return e?/-MMM-/.test(r)?n[e.month()]:t[e.month()]:t},monthsRegex:i,monthsShortRegex:i,monthsStrictRegex:/^(januari|februari|maart|april|mei|ju[nl]i|augustus|september|oktober|november|december)/i,monthsShortStrictRegex:/^(jan\.?|feb\.?|mrt\.?|apr\.?|mei|ju[nl]\.?|aug\.?|sep\.?|okt\.?|nov\.?|dec\.?)/i,monthsParse:r,longMonthsParse:r,shortMonthsParse:r,weekdays:"zondag_maandag_dinsdag_woensdag_donderdag_vrijdag_zaterdag".split("_"),weekdaysShort:"zo._ma._di._wo._do._vr._za.".split("_"),weekdaysMin:"zo_ma_di_wo_do_vr_za".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[vandaag om] LT",nextDay:"[morgen om] LT",nextWeek:"dddd [om] LT",lastDay:"[gisteren om] LT",lastWeek:"[afgelopen] dddd [om] LT",sameElse:"L"},relativeTime:{future:"over %s",past:"%s geleden",s:"een paar seconden",ss:"%d seconden",m:"één minuut",mm:"%d minuten",h:"één uur",hh:"%d uur",d:"één dag",dd:"%d dagen",M:"één maand",MM:"%d maanden",y:"één jaar",yy:"%d jaar"},dayOfMonthOrdinalParse:/\d{1,2}(ste|de)/,ordinal:function(e){return e+(1===e||8===e||e>=20?"ste":"de")},week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("nn",{months:"januar_februar_mars_april_mai_juni_juli_august_september_oktober_november_desember".split("_"),monthsShort:"jan._feb._mars_apr._mai_juni_juli_aug._sep._okt._nov._des.".split("_"),monthsParseExact:!0,weekdays:"sundag_måndag_tysdag_onsdag_torsdag_fredag_laurdag".split("_"),weekdaysShort:"su._må._ty._on._to._fr._lau.".split("_"),weekdaysMin:"su_må_ty_on_to_fr_la".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY [kl.] H:mm",LLLL:"dddd D. MMMM YYYY [kl.] HH:mm"},calendar:{sameDay:"[I dag klokka] LT",nextDay:"[I morgon klokka] LT",nextWeek:"dddd [klokka] LT",lastDay:"[I går klokka] LT",lastWeek:"[Føregåande] dddd [klokka] LT",sameElse:"L"},relativeTime:{future:"om %s",past:"%s sidan",s:"nokre sekund",ss:"%d sekund",m:"eit minutt",mm:"%d minutt",h:"ein time",hh:"%d timar",d:"ein dag",dd:"%d dagar",w:"ei veke",ww:"%d veker",M:"ein månad",MM:"%d månader",y:"eit år",yy:"%d år"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("oc-lnc",{months:{standalone:"genièr_febrièr_març_abril_mai_junh_julhet_agost_setembre_octòbre_novembre_decembre".split("_"),format:"de genièr_de febrièr_de març_d'abril_de mai_de junh_de julhet_d'agost_de setembre_d'octòbre_de novembre_de decembre".split("_"),isFormat:/D[oD]?(\s)+MMMM/},monthsShort:"gen._febr._març_abr._mai_junh_julh._ago._set._oct._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"dimenge_diluns_dimars_dimècres_dijòus_divendres_dissabte".split("_"),weekdaysShort:"dg._dl._dm._dc._dj._dv._ds.".split("_"),weekdaysMin:"dg_dl_dm_dc_dj_dv_ds".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM [de] YYYY",ll:"D MMM YYYY",LLL:"D MMMM [de] YYYY [a] H:mm",lll:"D MMM YYYY, H:mm",LLLL:"dddd D MMMM [de] YYYY [a] H:mm",llll:"ddd D MMM YYYY, H:mm"},calendar:{sameDay:"[uèi a] LT",nextDay:"[deman a] LT",nextWeek:"dddd [a] LT",lastDay:"[ièr a] LT",lastWeek:"dddd [passat a] LT",sameElse:"L"},relativeTime:{future:"d'aquí %s",past:"fa %s",s:"unas segondas",ss:"%d segondas",m:"una minuta",mm:"%d minutas",h:"una ora",hh:"%d oras",d:"un jorn",dd:"%d jorns",M:"un mes",MM:"%d meses",y:"un an",yy:"%d ans"},dayOfMonthOrdinalParse:/\d{1,2}(r|n|t|è|a)/,ordinal:function(e,t){var n=1===e?"r":2===e?"n":3===e?"r":4===e?"t":"è";return"w"!==t&&"W"!==t||(n="a"),e+n},week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t={1:"੧",2:"੨",3:"੩",4:"੪",5:"੫",6:"੬",7:"੭",8:"੮",9:"੯",0:"੦"},n={"੧":"1","੨":"2","੩":"3","੪":"4","੫":"5","੬":"6","੭":"7","੮":"8","੯":"9","੦":"0"};e.defineLocale("pa-in",{months:"ਜਨਵਰੀ_ਫ਼ਰਵਰੀ_ਮਾਰਚ_ਅਪ੍ਰੈਲ_ਮਈ_ਜੂਨ_ਜੁਲਾਈ_ਅਗਸਤ_ਸਤੰਬਰ_ਅਕਤੂਬਰ_ਨਵੰਬਰ_ਦਸੰਬਰ".split("_"),monthsShort:"ਜਨਵਰੀ_ਫ਼ਰਵਰੀ_ਮਾਰਚ_ਅਪ੍ਰੈਲ_ਮਈ_ਜੂਨ_ਜੁਲਾਈ_ਅਗਸਤ_ਸਤੰਬਰ_ਅਕਤੂਬਰ_ਨਵੰਬਰ_ਦਸੰਬਰ".split("_"),weekdays:"ਐਤਵਾਰ_ਸੋਮਵਾਰ_ਮੰਗਲਵਾਰ_ਬੁਧਵਾਰ_ਵੀਰਵਾਰ_ਸ਼ੁੱਕਰਵਾਰ_ਸ਼ਨੀਚਰਵਾਰ".split("_"),weekdaysShort:"ਐਤ_ਸੋਮ_ਮੰਗਲ_ਬੁਧ_ਵੀਰ_ਸ਼ੁਕਰ_ਸ਼ਨੀ".split("_"),weekdaysMin:"ਐਤ_ਸੋਮ_ਮੰਗਲ_ਬੁਧ_ਵੀਰ_ਸ਼ੁਕਰ_ਸ਼ਨੀ".split("_"),longDateFormat:{LT:"A h:mm ਵਜੇ",LTS:"A h:mm:ss ਵਜੇ",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm ਵਜੇ",LLLL:"dddd, D MMMM YYYY, A h:mm ਵਜੇ"},calendar:{sameDay:"[ਅਜ] LT",nextDay:"[ਕਲ] LT",nextWeek:"[ਅਗਲਾ] dddd, LT",lastDay:"[ਕਲ] LT",lastWeek:"[ਪਿਛਲੇ] dddd, LT",sameElse:"L"},relativeTime:{future:"%s ਵਿੱਚ",past:"%s ਪਿਛਲੇ",s:"ਕੁਝ ਸਕਿੰਟ",ss:"%d ਸਕਿੰਟ",m:"ਇਕ ਮਿੰਟ",mm:"%d ਮਿੰਟ",h:"ਇੱਕ ਘੰਟਾ",hh:"%d ਘੰਟੇ",d:"ਇੱਕ ਦਿਨ",dd:"%d ਦਿਨ",M:"ਇੱਕ ਮਹੀਨਾ",MM:"%d ਮਹੀਨੇ",y:"ਇੱਕ ਸਾਲ",yy:"%d ਸਾਲ"},preparse:function(e){return e.replace(/[੧੨੩੪੫੬੭੮੯੦]/g,(function(e){return n[e]}))},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]}))},meridiemParse:/ਰਾਤ|ਸਵੇਰ|ਦੁਪਹਿਰ|ਸ਼ਾਮ/,meridiemHour:function(e,t){return 12===e&&(e=0),"ਰਾਤ"===t?e<4?e:e+12:"ਸਵੇਰ"===t?e:"ਦੁਪਹਿਰ"===t?e>=10?e:e+12:"ਸ਼ਾਮ"===t?e+12:void 0},meridiem:function(e,t,n){return e<4?"ਰਾਤ":e<10?"ਸਵੇਰ":e<17?"ਦੁਪਹਿਰ":e<20?"ਸ਼ਾਮ":"ਰਾਤ"},week:{dow:0,doy:6}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t="styczeń_luty_marzec_kwiecień_maj_czerwiec_lipiec_sierpień_wrzesień_październik_listopad_grudzień".split("_"),n="stycznia_lutego_marca_kwietnia_maja_czerwca_lipca_sierpnia_września_października_listopada_grudnia".split("_"),r=[/^sty/i,/^lut/i,/^mar/i,/^kwi/i,/^maj/i,/^cze/i,/^lip/i,/^sie/i,/^wrz/i,/^paź/i,/^lis/i,/^gru/i];function i(e){return e%10<5&&e%10>1&&~~(e/10)%10!=1}function o(e,t,n){var r=e+" ";switch(n){case"ss":return r+(i(e)?"sekundy":"sekund");case"m":return t?"minuta":"minutę";case"mm":return r+(i(e)?"minuty":"minut");case"h":return t?"godzina":"godzinę";case"hh":return r+(i(e)?"godziny":"godzin");case"ww":return r+(i(e)?"tygodnie":"tygodni");case"MM":return r+(i(e)?"miesiące":"miesięcy");case"yy":return r+(i(e)?"lata":"lat")}}e.defineLocale("pl",{months:function(e,r){return e?/D MMMM/.test(r)?n[e.month()]:t[e.month()]:t},monthsShort:"sty_lut_mar_kwi_maj_cze_lip_sie_wrz_paź_lis_gru".split("_"),monthsParse:r,longMonthsParse:r,shortMonthsParse:r,weekdays:"niedziela_poniedziałek_wtorek_środa_czwartek_piątek_sobota".split("_"),weekdaysShort:"ndz_pon_wt_śr_czw_pt_sob".split("_"),weekdaysMin:"Nd_Pn_Wt_Śr_Cz_Pt_So".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Dziś o] LT",nextDay:"[Jutro o] LT",nextWeek:function(){switch(this.day()){case 0:return"[W niedzielę o] LT";case 2:return"[We wtorek o] LT";case 3:return"[W środę o] LT";case 6:return"[W sobotę o] LT";default:return"[W] dddd [o] LT"}},lastDay:"[Wczoraj o] LT",lastWeek:function(){switch(this.day()){case 0:return"[W zeszłą niedzielę o] LT";case 3:return"[W zeszłą środę o] LT";case 6:return"[W zeszłą sobotę o] LT";default:return"[W zeszły] dddd [o] LT"}},sameElse:"L"},relativeTime:{future:"za %s",past:"%s temu",s:"kilka sekund",ss:o,m:o,mm:o,h:o,hh:o,d:"1 dzień",dd:"%d dni",w:"tydzień",ww:o,M:"miesiąc",MM:o,y:"rok",yy:o},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("pt",{months:"janeiro_fevereiro_março_abril_maio_junho_julho_agosto_setembro_outubro_novembro_dezembro".split("_"),monthsShort:"jan_fev_mar_abr_mai_jun_jul_ago_set_out_nov_dez".split("_"),weekdays:"Domingo_Segunda-feira_Terça-feira_Quarta-feira_Quinta-feira_Sexta-feira_Sábado".split("_"),weekdaysShort:"Dom_Seg_Ter_Qua_Qui_Sex_Sáb".split("_"),weekdaysMin:"Do_2ª_3ª_4ª_5ª_6ª_Sá".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY HH:mm",LLLL:"dddd, D [de] MMMM [de] YYYY HH:mm"},calendar:{sameDay:"[Hoje às] LT",nextDay:"[Amanhã às] LT",nextWeek:"dddd [às] LT",lastDay:"[Ontem às] LT",lastWeek:function(){return 0===this.day()||6===this.day()?"[Último] dddd [às] LT":"[Última] dddd [às] LT"},sameElse:"L"},relativeTime:{future:"em %s",past:"há %s",s:"segundos",ss:"%d segundos",m:"um minuto",mm:"%d minutos",h:"uma hora",hh:"%d horas",d:"um dia",dd:"%d dias",w:"uma semana",ww:"%d semanas",M:"um mês",MM:"%d meses",y:"um ano",yy:"%d anos"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("pt-br",{months:"janeiro_fevereiro_março_abril_maio_junho_julho_agosto_setembro_outubro_novembro_dezembro".split("_"),monthsShort:"jan_fev_mar_abr_mai_jun_jul_ago_set_out_nov_dez".split("_"),weekdays:"domingo_segunda-feira_terça-feira_quarta-feira_quinta-feira_sexta-feira_sábado".split("_"),weekdaysShort:"dom_seg_ter_qua_qui_sex_sáb".split("_"),weekdaysMin:"do_2ª_3ª_4ª_5ª_6ª_sá".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY [às] HH:mm",LLLL:"dddd, D [de] MMMM [de] YYYY [às] HH:mm"},calendar:{sameDay:"[Hoje às] LT",nextDay:"[Amanhã às] LT",nextWeek:"dddd [às] LT",lastDay:"[Ontem às] LT",lastWeek:function(){return 0===this.day()||6===this.day()?"[Último] dddd [às] LT":"[Última] dddd [às] LT"},sameElse:"L"},relativeTime:{future:"em %s",past:"há %s",s:"poucos segundos",ss:"%d segundos",m:"um minuto",mm:"%d minutos",h:"uma hora",hh:"%d horas",d:"um dia",dd:"%d dias",M:"um mês",MM:"%d meses",y:"um ano",yy:"%d anos"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",invalidDate:"Data inválida"})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
function t(e,t,n){var r=" ";return(e%100>=20||e>=100&&e%100==0)&&(r=" de "),e+r+{ss:"secunde",mm:"minute",hh:"ore",dd:"zile",ww:"săptămâni",MM:"luni",yy:"ani"}[n]}e.defineLocale("ro",{months:"ianuarie_februarie_martie_aprilie_mai_iunie_iulie_august_septembrie_octombrie_noiembrie_decembrie".split("_"),monthsShort:"ian._feb._mart._apr._mai_iun._iul._aug._sept._oct._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"duminică_luni_marți_miercuri_joi_vineri_sâmbătă".split("_"),weekdaysShort:"Dum_Lun_Mar_Mie_Joi_Vin_Sâm".split("_"),weekdaysMin:"Du_Lu_Ma_Mi_Jo_Vi_Sâ".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY H:mm",LLLL:"dddd, D MMMM YYYY H:mm"},calendar:{sameDay:"[azi la] LT",nextDay:"[mâine la] LT",nextWeek:"dddd [la] LT",lastDay:"[ieri la] LT",lastWeek:"[fosta] dddd [la] LT",sameElse:"L"},relativeTime:{future:"peste %s",past:"%s în urmă",s:"câteva secunde",ss:t,m:"un minut",mm:t,h:"o oră",hh:t,d:"o zi",dd:t,w:"o săptămână",ww:t,M:"o lună",MM:t,y:"un an",yy:t},week:{dow:1,doy:7}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
function t(e,t,n){var r,i;return"m"===n?t?"минута":"минуту":e+" "+(r=+e,i={ss:t?"секунда_секунды_секунд":"секунду_секунды_секунд",mm:t?"минута_минуты_минут":"минуту_минуты_минут",hh:"час_часа_часов",dd:"день_дня_дней",ww:"неделя_недели_недель",MM:"месяц_месяца_месяцев",yy:"год_года_лет"}[n].split("_"),r%10==1&&r%100!=11?i[0]:r%10>=2&&r%10<=4&&(r%100<10||r%100>=20)?i[1]:i[2])}var n=[/^янв/i,/^фев/i,/^мар/i,/^апр/i,/^ма[йя]/i,/^июн/i,/^июл/i,/^авг/i,/^сен/i,/^окт/i,/^ноя/i,/^дек/i];e.defineLocale("ru",{months:{format:"января_февраля_марта_апреля_мая_июня_июля_августа_сентября_октября_ноября_декабря".split("_"),standalone:"январь_февраль_март_апрель_май_июнь_июль_август_сентябрь_октябрь_ноябрь_декабрь".split("_")},monthsShort:{format:"янв._февр._мар._апр._мая_июня_июля_авг._сент._окт._нояб._дек.".split("_"),standalone:"янв._февр._март_апр._май_июнь_июль_авг._сент._окт._нояб._дек.".split("_")},weekdays:{standalone:"воскресенье_понедельник_вторник_среда_четверг_пятница_суббота".split("_"),format:"воскресенье_понедельник_вторник_среду_четверг_пятницу_субботу".split("_"),isFormat:/\[ ?[Вв] ?(?:прошлую|следующую|эту)? ?] ?dddd/},weekdaysShort:"вс_пн_вт_ср_чт_пт_сб".split("_"),weekdaysMin:"вс_пн_вт_ср_чт_пт_сб".split("_"),monthsParse:n,longMonthsParse:n,shortMonthsParse:n,monthsRegex:/^(январ[ья]|янв\.?|феврал[ья]|февр?\.?|марта?|мар\.?|апрел[ья]|апр\.?|ма[йя]|июн[ья]|июн\.?|июл[ья]|июл\.?|августа?|авг\.?|сентябр[ья]|сент?\.?|октябр[ья]|окт\.?|ноябр[ья]|нояб?\.?|декабр[ья]|дек\.?)/i,monthsShortRegex:/^(январ[ья]|янв\.?|феврал[ья]|февр?\.?|марта?|мар\.?|апрел[ья]|апр\.?|ма[йя]|июн[ья]|июн\.?|июл[ья]|июл\.?|августа?|авг\.?|сентябр[ья]|сент?\.?|октябр[ья]|окт\.?|ноябр[ья]|нояб?\.?|декабр[ья]|дек\.?)/i,monthsStrictRegex:/^(январ[яь]|феврал[яь]|марта?|апрел[яь]|ма[яй]|июн[яь]|июл[яь]|августа?|сентябр[яь]|октябр[яь]|ноябр[яь]|декабр[яь])/i,monthsShortStrictRegex:/^(янв\.|февр?\.|мар[т.]|апр\.|ма[яй]|июн[ья.]|июл[ья.]|авг\.|сент?\.|окт\.|нояб?\.|дек\.)/i,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY г.",LLL:"D MMMM YYYY г., H:mm",LLLL:"dddd, D MMMM YYYY г., H:mm"},calendar:{sameDay:"[Сегодня, в] LT",nextDay:"[Завтра, в] LT",lastDay:"[Вчера, в] LT",nextWeek:function(e){if(e.week()===this.week())return 2===this.day()?"[Во] dddd, [в] LT":"[В] dddd, [в] LT";switch(this.day()){case 0:return"[В следующее] dddd, [в] LT";case 1:case 2:case 4:return"[В следующий] dddd, [в] LT";case 3:case 5:case 6:return"[В следующую] dddd, [в] LT"}},lastWeek:function(e){if(e.week()===this.week())return 2===this.day()?"[Во] dddd, [в] LT":"[В] dddd, [в] LT";switch(this.day()){case 0:return"[В прошлое] dddd, [в] LT";case 1:case 2:case 4:return"[В прошлый] dddd, [в] LT";case 3:case 5:case 6:return"[В прошлую] dddd, [в] LT"}},sameElse:"L"},relativeTime:{future:"через %s",past:"%s назад",s:"несколько секунд",ss:t,m:t,mm:t,h:"час",hh:t,d:"день",dd:t,w:"неделя",ww:t,M:"месяц",MM:t,y:"год",yy:t},meridiemParse:/ночи|утра|дня|вечера/i,isPM:function(e){return/^(дня|вечера)$/.test(e)},meridiem:function(e,t,n){return e<4?"ночи":e<12?"утра":e<17?"дня":"вечера"},dayOfMonthOrdinalParse:/\d{1,2}-(й|го|я)/,ordinal:function(e,t){switch(t){case"M":case"d":case"DDD":return e+"-й";case"D":return e+"-го";case"w":case"W":return e+"-я";default:return e}},week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t=["جنوري","فيبروري","مارچ","اپريل","مئي","جون","جولاءِ","آگسٽ","سيپٽمبر","آڪٽوبر","نومبر","ڊسمبر"],n=["آچر","سومر","اڱارو","اربع","خميس","جمع","ڇنڇر"];e.defineLocale("sd",{months:t,monthsShort:t,weekdays:n,weekdaysShort:n,weekdaysMin:n,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd، D MMMM YYYY HH:mm"},meridiemParse:/صبح|شام/,isPM:function(e){return"شام"===e},meridiem:function(e,t,n){return e<12?"صبح":"شام"},calendar:{sameDay:"[اڄ] LT",nextDay:"[سڀاڻي] LT",nextWeek:"dddd [اڳين هفتي تي] LT",lastDay:"[ڪالهه] LT",lastWeek:"[گزريل هفتي] dddd [تي] LT",sameElse:"L"},relativeTime:{future:"%s پوء",past:"%s اڳ",s:"چند سيڪنڊ",ss:"%d سيڪنڊ",m:"هڪ منٽ",mm:"%d منٽ",h:"هڪ ڪلاڪ",hh:"%d ڪلاڪ",d:"هڪ ڏينهن",dd:"%d ڏينهن",M:"هڪ مهينو",MM:"%d مهينا",y:"هڪ سال",yy:"%d سال"},preparse:function(e){return e.replace(/،/g,",")},postformat:function(e){return e.replace(/,/g,"،")},week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("se",{months:"ođđajagemánnu_guovvamánnu_njukčamánnu_cuoŋománnu_miessemánnu_geassemánnu_suoidnemánnu_borgemánnu_čakčamánnu_golggotmánnu_skábmamánnu_juovlamánnu".split("_"),monthsShort:"ođđj_guov_njuk_cuo_mies_geas_suoi_borg_čakč_golg_skáb_juov".split("_"),weekdays:"sotnabeaivi_vuossárga_maŋŋebárga_gaskavahkku_duorastat_bearjadat_lávvardat".split("_"),weekdaysShort:"sotn_vuos_maŋ_gask_duor_bear_láv".split("_"),weekdaysMin:"s_v_m_g_d_b_L".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"MMMM D. [b.] YYYY",LLL:"MMMM D. [b.] YYYY [ti.] HH:mm",LLLL:"dddd, MMMM D. [b.] YYYY [ti.] HH:mm"},calendar:{sameDay:"[otne ti] LT",nextDay:"[ihttin ti] LT",nextWeek:"dddd [ti] LT",lastDay:"[ikte ti] LT",lastWeek:"[ovddit] dddd [ti] LT",sameElse:"L"},relativeTime:{future:"%s geažes",past:"maŋit %s",s:"moadde sekunddat",ss:"%d sekunddat",m:"okta minuhta",mm:"%d minuhtat",h:"okta diimmu",hh:"%d diimmut",d:"okta beaivi",dd:"%d beaivvit",M:"okta mánnu",MM:"%d mánut",y:"okta jahki",yy:"%d jagit"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("si",{months:"ජනවාරි_පෙබරවාරි_මාර්තු_අප්‍රේල්_මැයි_ජූනි_ජූලි_අගෝස්තු_සැප්තැම්බර්_ඔක්තෝබර්_නොවැම්බර්_දෙසැම්බර්".split("_"),monthsShort:"ජන_පෙබ_මාර්_අප්_මැයි_ජූනි_ජූලි_අගෝ_සැප්_ඔක්_නොවැ_දෙසැ".split("_"),weekdays:"ඉරිදා_සඳුදා_අඟහරුවාදා_බදාදා_බ්‍රහස්පතින්දා_සිකුරාදා_සෙනසුරාදා".split("_"),weekdaysShort:"ඉරි_සඳු_අඟ_බදා_බ්‍රහ_සිකු_සෙන".split("_"),weekdaysMin:"ඉ_ස_අ_බ_බ්‍ර_සි_සෙ".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"a h:mm",LTS:"a h:mm:ss",L:"YYYY/MM/DD",LL:"YYYY MMMM D",LLL:"YYYY MMMM D, a h:mm",LLLL:"YYYY MMMM D [වැනි] dddd, a h:mm:ss"},calendar:{sameDay:"[අද] LT[ට]",nextDay:"[හෙට] LT[ට]",nextWeek:"dddd LT[ට]",lastDay:"[ඊයේ] LT[ට]",lastWeek:"[පසුගිය] dddd LT[ට]",sameElse:"L"},relativeTime:{future:"%sකින්",past:"%sකට පෙර",s:"තත්පර කිහිපය",ss:"තත්පර %d",m:"මිනිත්තුව",mm:"මිනිත්තු %d",h:"පැය",hh:"පැය %d",d:"දිනය",dd:"දින %d",M:"මාසය",MM:"මාස %d",y:"වසර",yy:"වසර %d"},dayOfMonthOrdinalParse:/\d{1,2} වැනි/,ordinal:function(e){return e+" වැනි"},meridiemParse:/පෙර වරු|පස් වරු|පෙ.ව|ප.ව./,isPM:function(e){return"ප.ව."===e||"පස් වරු"===e},meridiem:function(e,t,n){return e>11?n?"ප.ව.":"පස් වරු":n?"පෙ.ව.":"පෙර වරු"}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t="január_február_marec_apríl_máj_jún_júl_august_september_október_november_december".split("_"),n="jan_feb_mar_apr_máj_jún_júl_aug_sep_okt_nov_dec".split("_");function r(e){return e>1&&e<5}function i(e,t,n,i){var o=e+" ";switch(n){case"s":return t||i?"pár sekúnd":"pár sekundami";case"ss":return t||i?o+(r(e)?"sekundy":"sekúnd"):o+"sekundami";case"m":return t?"minúta":i?"minútu":"minútou";case"mm":return t||i?o+(r(e)?"minúty":"minút"):o+"minútami";case"h":return t?"hodina":i?"hodinu":"hodinou";case"hh":return t||i?o+(r(e)?"hodiny":"hodín"):o+"hodinami";case"d":return t||i?"deň":"dňom";case"dd":return t||i?o+(r(e)?"dni":"dní"):o+"dňami";case"M":return t||i?"mesiac":"mesiacom";case"MM":return t||i?o+(r(e)?"mesiace":"mesiacov"):o+"mesiacmi";case"y":return t||i?"rok":"rokom";case"yy":return t||i?o+(r(e)?"roky":"rokov"):o+"rokmi"}}e.defineLocale("sk",{months:t,monthsShort:n,weekdays:"nedeľa_pondelok_utorok_streda_štvrtok_piatok_sobota".split("_"),weekdaysShort:"ne_po_ut_st_št_pi_so".split("_"),weekdaysMin:"ne_po_ut_st_št_pi_so".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd D. MMMM YYYY H:mm"},calendar:{sameDay:"[dnes o] LT",nextDay:"[zajtra o] LT",nextWeek:function(){switch(this.day()){case 0:return"[v nedeľu o] LT";case 1:case 2:return"[v] dddd [o] LT";case 3:return"[v stredu o] LT";case 4:return"[vo štvrtok o] LT";case 5:return"[v piatok o] LT";case 6:return"[v sobotu o] LT"}},lastDay:"[včera o] LT",lastWeek:function(){switch(this.day()){case 0:return"[minulú nedeľu o] LT";case 1:case 2:return"[minulý] dddd [o] LT";case 3:return"[minulú stredu o] LT";case 4:case 5:return"[minulý] dddd [o] LT";case 6:return"[minulú sobotu o] LT"}},sameElse:"L"},relativeTime:{future:"za %s",past:"pred %s",s:i,ss:i,m:i,mm:i,h:i,hh:i,d:i,dd:i,M:i,MM:i,y:i,yy:i},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
function t(e,t,n,r){var i=e+" ";switch(n){case"s":return t||r?"nekaj sekund":"nekaj sekundami";case"ss":return i+=1===e?t?"sekundo":"sekundi":2===e?t||r?"sekundi":"sekundah":e<5?t||r?"sekunde":"sekundah":"sekund";case"m":return t?"ena minuta":"eno minuto";case"mm":return i+=1===e?t?"minuta":"minuto":2===e?t||r?"minuti":"minutama":e<5?t||r?"minute":"minutami":t||r?"minut":"minutami";case"h":return t?"ena ura":"eno uro";case"hh":return i+=1===e?t?"ura":"uro":2===e?t||r?"uri":"urama":e<5?t||r?"ure":"urami":t||r?"ur":"urami";case"d":return t||r?"en dan":"enim dnem";case"dd":return i+=1===e?t||r?"dan":"dnem":2===e?t||r?"dni":"dnevoma":t||r?"dni":"dnevi";case"M":return t||r?"en mesec":"enim mesecem";case"MM":return i+=1===e?t||r?"mesec":"mesecem":2===e?t||r?"meseca":"mesecema":e<5?t||r?"mesece":"meseci":t||r?"mesecev":"meseci";case"y":return t||r?"eno leto":"enim letom";case"yy":return i+=1===e?t||r?"leto":"letom":2===e?t||r?"leti":"letoma":e<5?t||r?"leta":"leti":t||r?"let":"leti"}}e.defineLocale("sl",{months:"januar_februar_marec_april_maj_junij_julij_avgust_september_oktober_november_december".split("_"),monthsShort:"jan._feb._mar._apr._maj._jun._jul._avg._sep._okt._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"nedelja_ponedeljek_torek_sreda_četrtek_petek_sobota".split("_"),weekdaysShort:"ned._pon._tor._sre._čet._pet._sob.".split("_"),weekdaysMin:"ne_po_to_sr_če_pe_so".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD. MM. YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd, D. MMMM YYYY H:mm"},calendar:{sameDay:"[danes ob] LT",nextDay:"[jutri ob] LT",nextWeek:function(){switch(this.day()){case 0:return"[v] [nedeljo] [ob] LT";case 3:return"[v] [sredo] [ob] LT";case 6:return"[v] [soboto] [ob] LT";case 1:case 2:case 4:case 5:return"[v] dddd [ob] LT"}},lastDay:"[včeraj ob] LT",lastWeek:function(){switch(this.day()){case 0:return"[prejšnjo] [nedeljo] [ob] LT";case 3:return"[prejšnjo] [sredo] [ob] LT";case 6:return"[prejšnjo] [soboto] [ob] LT";case 1:case 2:case 4:case 5:return"[prejšnji] dddd [ob] LT"}},sameElse:"L"},relativeTime:{future:"čez %s",past:"pred %s",s:t,ss:t,m:t,mm:t,h:t,hh:t,d:t,dd:t,M:t,MM:t,y:t,yy:t},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:7}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("sq",{months:"Janar_Shkurt_Mars_Prill_Maj_Qershor_Korrik_Gusht_Shtator_Tetor_Nëntor_Dhjetor".split("_"),monthsShort:"Jan_Shk_Mar_Pri_Maj_Qer_Kor_Gus_Sht_Tet_Nën_Dhj".split("_"),weekdays:"E Diel_E Hënë_E Martë_E Mërkurë_E Enjte_E Premte_E Shtunë".split("_"),weekdaysShort:"Die_Hën_Mar_Mër_Enj_Pre_Sht".split("_"),weekdaysMin:"D_H_Ma_Më_E_P_Sh".split("_"),weekdaysParseExact:!0,meridiemParse:/PD|MD/,isPM:function(e){return"M"===e.charAt(0)},meridiem:function(e,t,n){return e<12?"PD":"MD"},longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Sot në] LT",nextDay:"[Nesër në] LT",nextWeek:"dddd [në] LT",lastDay:"[Dje në] LT",lastWeek:"dddd [e kaluar në] LT",sameElse:"L"},relativeTime:{future:"në %s",past:"%s më parë",s:"disa sekonda",ss:"%d sekonda",m:"një minutë",mm:"%d minuta",h:"një orë",hh:"%d orë",d:"një ditë",dd:"%d ditë",M:"një muaj",MM:"%d muaj",y:"një vit",yy:"%d vite"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t={words:{ss:["sekunda","sekunde","sekundi"],m:["jedan minut","jedne minute"],mm:["minut","minute","minuta"],h:["jedan sat","jednog sata"],hh:["sat","sata","sati"],dd:["dan","dana","dana"],MM:["mesec","meseca","meseci"],yy:["godina","godine","godina"]},correctGrammaticalCase:function(e,t){return 1===e?t[0]:e>=2&&e<=4?t[1]:t[2]},translate:function(e,n,r){var i=t.words[r];return 1===r.length?n?i[0]:i[1]:e+" "+t.correctGrammaticalCase(e,i)}};e.defineLocale("sr",{months:"januar_februar_mart_april_maj_jun_jul_avgust_septembar_oktobar_novembar_decembar".split("_"),monthsShort:"jan._feb._mar._apr._maj_jun_jul_avg._sep._okt._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"nedelja_ponedeljak_utorak_sreda_četvrtak_petak_subota".split("_"),weekdaysShort:"ned._pon._uto._sre._čet._pet._sub.".split("_"),weekdaysMin:"ne_po_ut_sr_če_pe_su".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"D. M. YYYY.",LL:"D. MMMM YYYY.",LLL:"D. MMMM YYYY. H:mm",LLLL:"dddd, D. MMMM YYYY. H:mm"},calendar:{sameDay:"[danas u] LT",nextDay:"[sutra u] LT",nextWeek:function(){switch(this.day()){case 0:return"[u] [nedelju] [u] LT";case 3:return"[u] [sredu] [u] LT";case 6:return"[u] [subotu] [u] LT";case 1:case 2:case 4:case 5:return"[u] dddd [u] LT"}},lastDay:"[juče u] LT",lastWeek:function(){return["[prošle] [nedelje] [u] LT","[prošlog] [ponedeljka] [u] LT","[prošlog] [utorka] [u] LT","[prošle] [srede] [u] LT","[prošlog] [četvrtka] [u] LT","[prošlog] [petka] [u] LT","[prošle] [subote] [u] LT"][this.day()]},sameElse:"L"},relativeTime:{future:"za %s",past:"pre %s",s:"nekoliko sekundi",ss:t.translate,m:t.translate,mm:t.translate,h:t.translate,hh:t.translate,d:"dan",dd:t.translate,M:"mesec",MM:t.translate,y:"godinu",yy:t.translate},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:7}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t={words:{ss:["секунда","секунде","секунди"],m:["један минут","једне минуте"],mm:["минут","минуте","минута"],h:["један сат","једног сата"],hh:["сат","сата","сати"],dd:["дан","дана","дана"],MM:["месец","месеца","месеци"],yy:["година","године","година"]},correctGrammaticalCase:function(e,t){return 1===e?t[0]:e>=2&&e<=4?t[1]:t[2]},translate:function(e,n,r){var i=t.words[r];return 1===r.length?n?i[0]:i[1]:e+" "+t.correctGrammaticalCase(e,i)}};e.defineLocale("sr-cyrl",{months:"јануар_фебруар_март_април_мај_јун_јул_август_септембар_октобар_новембар_децембар".split("_"),monthsShort:"јан._феб._мар._апр._мај_јун_јул_авг._сеп._окт._нов._дец.".split("_"),monthsParseExact:!0,weekdays:"недеља_понедељак_уторак_среда_четвртак_петак_субота".split("_"),weekdaysShort:"нед._пон._уто._сре._чет._пет._суб.".split("_"),weekdaysMin:"не_по_ут_ср_че_пе_су".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"D. M. YYYY.",LL:"D. MMMM YYYY.",LLL:"D. MMMM YYYY. H:mm",LLLL:"dddd, D. MMMM YYYY. H:mm"},calendar:{sameDay:"[данас у] LT",nextDay:"[сутра у] LT",nextWeek:function(){switch(this.day()){case 0:return"[у] [недељу] [у] LT";case 3:return"[у] [среду] [у] LT";case 6:return"[у] [суботу] [у] LT";case 1:case 2:case 4:case 5:return"[у] dddd [у] LT"}},lastDay:"[јуче у] LT",lastWeek:function(){return["[прошле] [недеље] [у] LT","[прошлог] [понедељка] [у] LT","[прошлог] [уторка] [у] LT","[прошле] [среде] [у] LT","[прошлог] [четвртка] [у] LT","[прошлог] [петка] [у] LT","[прошле] [суботе] [у] LT"][this.day()]},sameElse:"L"},relativeTime:{future:"за %s",past:"пре %s",s:"неколико секунди",ss:t.translate,m:t.translate,mm:t.translate,h:t.translate,hh:t.translate,d:"дан",dd:t.translate,M:"месец",MM:t.translate,y:"годину",yy:t.translate},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:7}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("ss",{months:"Bhimbidvwane_Indlovana_Indlov'lenkhulu_Mabasa_Inkhwekhweti_Inhlaba_Kholwane_Ingci_Inyoni_Imphala_Lweti_Ingongoni".split("_"),monthsShort:"Bhi_Ina_Inu_Mab_Ink_Inh_Kho_Igc_Iny_Imp_Lwe_Igo".split("_"),weekdays:"Lisontfo_Umsombuluko_Lesibili_Lesitsatfu_Lesine_Lesihlanu_Umgcibelo".split("_"),weekdaysShort:"Lis_Umb_Lsb_Les_Lsi_Lsh_Umg".split("_"),weekdaysMin:"Li_Us_Lb_Lt_Ls_Lh_Ug".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY h:mm A",LLLL:"dddd, D MMMM YYYY h:mm A"},calendar:{sameDay:"[Namuhla nga] LT",nextDay:"[Kusasa nga] LT",nextWeek:"dddd [nga] LT",lastDay:"[Itolo nga] LT",lastWeek:"dddd [leliphelile] [nga] LT",sameElse:"L"},relativeTime:{future:"nga %s",past:"wenteka nga %s",s:"emizuzwana lomcane",ss:"%d mzuzwana",m:"umzuzu",mm:"%d emizuzu",h:"lihora",hh:"%d emahora",d:"lilanga",dd:"%d emalanga",M:"inyanga",MM:"%d tinyanga",y:"umnyaka",yy:"%d iminyaka"},meridiemParse:/ekuseni|emini|entsambama|ebusuku/,meridiem:function(e,t,n){return e<11?"ekuseni":e<15?"emini":e<19?"entsambama":"ebusuku"},meridiemHour:function(e,t){return 12===e&&(e=0),"ekuseni"===t?e:"emini"===t?e>=11?e:e+12:"entsambama"===t||"ebusuku"===t?0===e?0:e+12:void 0},dayOfMonthOrdinalParse:/\d{1,2}/,ordinal:"%d",week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("sv",{months:"januari_februari_mars_april_maj_juni_juli_augusti_september_oktober_november_december".split("_"),monthsShort:"jan_feb_mar_apr_maj_jun_jul_aug_sep_okt_nov_dec".split("_"),weekdays:"söndag_måndag_tisdag_onsdag_torsdag_fredag_lördag".split("_"),weekdaysShort:"sön_mån_tis_ons_tor_fre_lör".split("_"),weekdaysMin:"sö_må_ti_on_to_fr_lö".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"D MMMM YYYY",LLL:"D MMMM YYYY [kl.] HH:mm",LLLL:"dddd D MMMM YYYY [kl.] HH:mm",lll:"D MMM YYYY HH:mm",llll:"ddd D MMM YYYY HH:mm"},calendar:{sameDay:"[Idag] LT",nextDay:"[Imorgon] LT",lastDay:"[Igår] LT",nextWeek:"[På] dddd LT",lastWeek:"[I] dddd[s] LT",sameElse:"L"},relativeTime:{future:"om %s",past:"för %s sedan",s:"några sekunder",ss:"%d sekunder",m:"en minut",mm:"%d minuter",h:"en timme",hh:"%d timmar",d:"en dag",dd:"%d dagar",M:"en månad",MM:"%d månader",y:"ett år",yy:"%d år"},dayOfMonthOrdinalParse:/\d{1,2}(\:e|\:a)/,ordinal:function(e){var t=e%10;return e+(1==~~(e%100/10)?":e":1===t||2===t?":a":":e")},week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("sw",{months:"Januari_Februari_Machi_Aprili_Mei_Juni_Julai_Agosti_Septemba_Oktoba_Novemba_Desemba".split("_"),monthsShort:"Jan_Feb_Mac_Apr_Mei_Jun_Jul_Ago_Sep_Okt_Nov_Des".split("_"),weekdays:"Jumapili_Jumatatu_Jumanne_Jumatano_Alhamisi_Ijumaa_Jumamosi".split("_"),weekdaysShort:"Jpl_Jtat_Jnne_Jtan_Alh_Ijm_Jmos".split("_"),weekdaysMin:"J2_J3_J4_J5_Al_Ij_J1".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"hh:mm A",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[leo saa] LT",nextDay:"[kesho saa] LT",nextWeek:"[wiki ijayo] dddd [saat] LT",lastDay:"[jana] LT",lastWeek:"[wiki iliyopita] dddd [saat] LT",sameElse:"L"},relativeTime:{future:"%s baadaye",past:"tokea %s",s:"hivi punde",ss:"sekunde %d",m:"dakika moja",mm:"dakika %d",h:"saa limoja",hh:"masaa %d",d:"siku moja",dd:"siku %d",M:"mwezi mmoja",MM:"miezi %d",y:"mwaka mmoja",yy:"miaka %d"},week:{dow:1,doy:7}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t={1:"௧",2:"௨",3:"௩",4:"௪",5:"௫",6:"௬",7:"௭",8:"௮",9:"௯",0:"௦"},n={"௧":"1","௨":"2","௩":"3","௪":"4","௫":"5","௬":"6","௭":"7","௮":"8","௯":"9","௦":"0"};e.defineLocale("ta",{months:"ஜனவரி_பிப்ரவரி_மார்ச்_ஏப்ரல்_மே_ஜூன்_ஜூலை_ஆகஸ்ட்_செப்டெம்பர்_அக்டோபர்_நவம்பர்_டிசம்பர்".split("_"),monthsShort:"ஜனவரி_பிப்ரவரி_மார்ச்_ஏப்ரல்_மே_ஜூன்_ஜூலை_ஆகஸ்ட்_செப்டெம்பர்_அக்டோபர்_நவம்பர்_டிசம்பர்".split("_"),weekdays:"ஞாயிற்றுக்கிழமை_திங்கட்கிழமை_செவ்வாய்கிழமை_புதன்கிழமை_வியாழக்கிழமை_வெள்ளிக்கிழமை_சனிக்கிழமை".split("_"),weekdaysShort:"ஞாயிறு_திங்கள்_செவ்வாய்_புதன்_வியாழன்_வெள்ளி_சனி".split("_"),weekdaysMin:"ஞா_தி_செ_பு_வி_வெ_ச".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, HH:mm",LLLL:"dddd, D MMMM YYYY, HH:mm"},calendar:{sameDay:"[இன்று] LT",nextDay:"[நாளை] LT",nextWeek:"dddd, LT",lastDay:"[நேற்று] LT",lastWeek:"[கடந்த வாரம்] dddd, LT",sameElse:"L"},relativeTime:{future:"%s இல்",past:"%s முன்",s:"ஒரு சில விநாடிகள்",ss:"%d விநாடிகள்",m:"ஒரு நிமிடம்",mm:"%d நிமிடங்கள்",h:"ஒரு மணி நேரம்",hh:"%d மணி நேரம்",d:"ஒரு நாள்",dd:"%d நாட்கள்",M:"ஒரு மாதம்",MM:"%d மாதங்கள்",y:"ஒரு வருடம்",yy:"%d ஆண்டுகள்"},dayOfMonthOrdinalParse:/\d{1,2}வது/,ordinal:function(e){return e+"வது"},preparse:function(e){return e.replace(/[௧௨௩௪௫௬௭௮௯௦]/g,(function(e){return n[e]}))},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]}))},meridiemParse:/யாமம்|வைகறை|காலை|நண்பகல்|எற்பாடு|மாலை/,meridiem:function(e,t,n){return e<2?" யாமம்":e<6?" வைகறை":e<10?" காலை":e<14?" நண்பகல்":e<18?" எற்பாடு":e<22?" மாலை":" யாமம்"},meridiemHour:function(e,t){return 12===e&&(e=0),"யாமம்"===t?e<2?e:e+12:"வைகறை"===t||"காலை"===t||"நண்பகல்"===t&&e>=10?e:e+12},week:{dow:0,doy:6}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("te",{months:"జనవరి_ఫిబ్రవరి_మార్చి_ఏప్రిల్_మే_జూన్_జులై_ఆగస్టు_సెప్టెంబర్_అక్టోబర్_నవంబర్_డిసెంబర్".split("_"),monthsShort:"జన._ఫిబ్ర._మార్చి_ఏప్రి._మే_జూన్_జులై_ఆగ._సెప్._అక్టో._నవ._డిసె.".split("_"),monthsParseExact:!0,weekdays:"ఆదివారం_సోమవారం_మంగళవారం_బుధవారం_గురువారం_శుక్రవారం_శనివారం".split("_"),weekdaysShort:"ఆది_సోమ_మంగళ_బుధ_గురు_శుక్ర_శని".split("_"),weekdaysMin:"ఆ_సో_మం_బు_గు_శు_శ".split("_"),longDateFormat:{LT:"A h:mm",LTS:"A h:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm",LLLL:"dddd, D MMMM YYYY, A h:mm"},calendar:{sameDay:"[నేడు] LT",nextDay:"[రేపు] LT",nextWeek:"dddd, LT",lastDay:"[నిన్న] LT",lastWeek:"[గత] dddd, LT",sameElse:"L"},relativeTime:{future:"%s లో",past:"%s క్రితం",s:"కొన్ని క్షణాలు",ss:"%d సెకన్లు",m:"ఒక నిమిషం",mm:"%d నిమిషాలు",h:"ఒక గంట",hh:"%d గంటలు",d:"ఒక రోజు",dd:"%d రోజులు",M:"ఒక నెల",MM:"%d నెలలు",y:"ఒక సంవత్సరం",yy:"%d సంవత్సరాలు"},dayOfMonthOrdinalParse:/\d{1,2}వ/,ordinal:"%dవ",meridiemParse:/రాత్రి|ఉదయం|మధ్యాహ్నం|సాయంత్రం/,meridiemHour:function(e,t){return 12===e&&(e=0),"రాత్రి"===t?e<4?e:e+12:"ఉదయం"===t?e:"మధ్యాహ్నం"===t?e>=10?e:e+12:"సాయంత్రం"===t?e+12:void 0},meridiem:function(e,t,n){return e<4?"రాత్రి":e<10?"ఉదయం":e<17?"మధ్యాహ్నం":e<20?"సాయంత్రం":"రాత్రి"},week:{dow:0,doy:6}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("tet",{months:"Janeiru_Fevereiru_Marsu_Abril_Maiu_Juñu_Jullu_Agustu_Setembru_Outubru_Novembru_Dezembru".split("_"),monthsShort:"Jan_Fev_Mar_Abr_Mai_Jun_Jul_Ago_Set_Out_Nov_Dez".split("_"),weekdays:"Domingu_Segunda_Tersa_Kuarta_Kinta_Sesta_Sabadu".split("_"),weekdaysShort:"Dom_Seg_Ters_Kua_Kint_Sest_Sab".split("_"),weekdaysMin:"Do_Seg_Te_Ku_Ki_Ses_Sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Ohin iha] LT",nextDay:"[Aban iha] LT",nextWeek:"dddd [iha] LT",lastDay:"[Horiseik iha] LT",lastWeek:"dddd [semana kotuk] [iha] LT",sameElse:"L"},relativeTime:{future:"iha %s",past:"%s liuba",s:"segundu balun",ss:"segundu %d",m:"minutu ida",mm:"minutu %d",h:"oras ida",hh:"oras %d",d:"loron ida",dd:"loron %d",M:"fulan ida",MM:"fulan %d",y:"tinan ida",yy:"tinan %d"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(e){var t=e%10;return e+(1==~~(e%100/10)?"th":1===t?"st":2===t?"nd":3===t?"rd":"th")},week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t={0:"-ум",1:"-ум",2:"-юм",3:"-юм",4:"-ум",5:"-ум",6:"-ум",7:"-ум",8:"-ум",9:"-ум",10:"-ум",12:"-ум",13:"-ум",20:"-ум",30:"-юм",40:"-ум",50:"-ум",60:"-ум",70:"-ум",80:"-ум",90:"-ум",100:"-ум"};e.defineLocale("tg",{months:{format:"январи_феврали_марти_апрели_майи_июни_июли_августи_сентябри_октябри_ноябри_декабри".split("_"),standalone:"январ_феврал_март_апрел_май_июн_июл_август_сентябр_октябр_ноябр_декабр".split("_")},monthsShort:"янв_фев_мар_апр_май_июн_июл_авг_сен_окт_ноя_дек".split("_"),weekdays:"якшанбе_душанбе_сешанбе_чоршанбе_панҷшанбе_ҷумъа_шанбе".split("_"),weekdaysShort:"яшб_дшб_сшб_чшб_пшб_ҷум_шнб".split("_"),weekdaysMin:"яш_дш_сш_чш_пш_ҷм_шб".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Имрӯз соати] LT",nextDay:"[Фардо соати] LT",lastDay:"[Дирӯз соати] LT",nextWeek:"dddd[и] [ҳафтаи оянда соати] LT",lastWeek:"dddd[и] [ҳафтаи гузашта соати] LT",sameElse:"L"},relativeTime:{future:"баъди %s",past:"%s пеш",s:"якчанд сония",m:"як дақиқа",mm:"%d дақиқа",h:"як соат",hh:"%d соат",d:"як рӯз",dd:"%d рӯз",M:"як моҳ",MM:"%d моҳ",y:"як сол",yy:"%d сол"},meridiemParse:/шаб|субҳ|рӯз|бегоҳ/,meridiemHour:function(e,t){return 12===e&&(e=0),"шаб"===t?e<4?e:e+12:"субҳ"===t?e:"рӯз"===t?e>=11?e:e+12:"бегоҳ"===t?e+12:void 0},meridiem:function(e,t,n){return e<4?"шаб":e<11?"субҳ":e<16?"рӯз":e<19?"бегоҳ":"шаб"},dayOfMonthOrdinalParse:/\d{1,2}-(ум|юм)/,ordinal:function(e){return e+(t[e]||t[e%10]||t[e>=100?100:null])},week:{dow:1,doy:7}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("th",{months:"มกราคม_กุมภาพันธ์_มีนาคม_เมษายน_พฤษภาคม_มิถุนายน_กรกฎาคม_สิงหาคม_กันยายน_ตุลาคม_พฤศจิกายน_ธันวาคม".split("_"),monthsShort:"ม.ค._ก.พ._มี.ค._เม.ย._พ.ค._มิ.ย._ก.ค._ส.ค._ก.ย._ต.ค._พ.ย._ธ.ค.".split("_"),monthsParseExact:!0,weekdays:"อาทิตย์_จันทร์_อังคาร_พุธ_พฤหัสบดี_ศุกร์_เสาร์".split("_"),weekdaysShort:"อาทิตย์_จันทร์_อังคาร_พุธ_พฤหัส_ศุกร์_เสาร์".split("_"),weekdaysMin:"อา._จ._อ._พ._พฤ._ศ._ส.".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY เวลา H:mm",LLLL:"วันddddที่ D MMMM YYYY เวลา H:mm"},meridiemParse:/ก่อนเที่ยง|หลังเที่ยง/,isPM:function(e){return"หลังเที่ยง"===e},meridiem:function(e,t,n){return e<12?"ก่อนเที่ยง":"หลังเที่ยง"},calendar:{sameDay:"[วันนี้ เวลา] LT",nextDay:"[พรุ่งนี้ เวลา] LT",nextWeek:"dddd[หน้า เวลา] LT",lastDay:"[เมื่อวานนี้ เวลา] LT",lastWeek:"[วัน]dddd[ที่แล้ว เวลา] LT",sameElse:"L"},relativeTime:{future:"อีก %s",past:"%sที่แล้ว",s:"ไม่กี่วินาที",ss:"%d วินาที",m:"1 นาที",mm:"%d นาที",h:"1 ชั่วโมง",hh:"%d ชั่วโมง",d:"1 วัน",dd:"%d วัน",w:"1 สัปดาห์",ww:"%d สัปดาห์",M:"1 เดือน",MM:"%d เดือน",y:"1 ปี",yy:"%d ปี"}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t={1:"'inji",5:"'inji",8:"'inji",70:"'inji",80:"'inji",2:"'nji",7:"'nji",20:"'nji",50:"'nji",3:"'ünji",4:"'ünji",100:"'ünji",6:"'njy",9:"'unjy",10:"'unjy",30:"'unjy",60:"'ynjy",90:"'ynjy"};e.defineLocale("tk",{months:"Ýanwar_Fewral_Mart_Aprel_Maý_Iýun_Iýul_Awgust_Sentýabr_Oktýabr_Noýabr_Dekabr".split("_"),monthsShort:"Ýan_Few_Mar_Apr_Maý_Iýn_Iýl_Awg_Sen_Okt_Noý_Dek".split("_"),weekdays:"Ýekşenbe_Duşenbe_Sişenbe_Çarşenbe_Penşenbe_Anna_Şenbe".split("_"),weekdaysShort:"Ýek_Duş_Siş_Çar_Pen_Ann_Şen".split("_"),weekdaysMin:"Ýk_Dş_Sş_Çr_Pn_An_Şn".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[bugün sagat] LT",nextDay:"[ertir sagat] LT",nextWeek:"[indiki] dddd [sagat] LT",lastDay:"[düýn] LT",lastWeek:"[geçen] dddd [sagat] LT",sameElse:"L"},relativeTime:{future:"%s soň",past:"%s öň",s:"birnäçe sekunt",m:"bir minut",mm:"%d minut",h:"bir sagat",hh:"%d sagat",d:"bir gün",dd:"%d gün",M:"bir aý",MM:"%d aý",y:"bir ýyl",yy:"%d ýyl"},ordinal:function(e,n){switch(n){case"d":case"D":case"Do":case"DD":return e;default:if(0===e)return e+"'unjy";var r=e%10;return e+(t[r]||t[e%100-r]||t[e>=100?100:null])}},week:{dow:1,doy:7}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("tl-ph",{months:"Enero_Pebrero_Marso_Abril_Mayo_Hunyo_Hulyo_Agosto_Setyembre_Oktubre_Nobyembre_Disyembre".split("_"),monthsShort:"Ene_Peb_Mar_Abr_May_Hun_Hul_Ago_Set_Okt_Nob_Dis".split("_"),weekdays:"Linggo_Lunes_Martes_Miyerkules_Huwebes_Biyernes_Sabado".split("_"),weekdaysShort:"Lin_Lun_Mar_Miy_Huw_Biy_Sab".split("_"),weekdaysMin:"Li_Lu_Ma_Mi_Hu_Bi_Sab".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"MM/D/YYYY",LL:"MMMM D, YYYY",LLL:"MMMM D, YYYY HH:mm",LLLL:"dddd, MMMM DD, YYYY HH:mm"},calendar:{sameDay:"LT [ngayong araw]",nextDay:"[Bukas ng] LT",nextWeek:"LT [sa susunod na] dddd",lastDay:"LT [kahapon]",lastWeek:"LT [noong nakaraang] dddd",sameElse:"L"},relativeTime:{future:"sa loob ng %s",past:"%s ang nakalipas",s:"ilang segundo",ss:"%d segundo",m:"isang minuto",mm:"%d minuto",h:"isang oras",hh:"%d oras",d:"isang araw",dd:"%d araw",M:"isang buwan",MM:"%d buwan",y:"isang taon",yy:"%d taon"},dayOfMonthOrdinalParse:/\d{1,2}/,ordinal:function(e){return e},week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t="pagh_wa’_cha’_wej_loS_vagh_jav_Soch_chorgh_Hut".split("_");function n(e,n,r,i){var o=function(e){var n=Math.floor(e%1e3/100),r=Math.floor(e%100/10),i=e%10,o="";return n>0&&(o+=t[n]+"vatlh"),r>0&&(o+=(""!==o?" ":"")+t[r]+"maH"),i>0&&(o+=(""!==o?" ":"")+t[i]),""===o?"pagh":o}(e);switch(r){case"ss":return o+" lup";case"mm":return o+" tup";case"hh":return o+" rep";case"dd":return o+" jaj";case"MM":return o+" jar";case"yy":return o+" DIS"}}e.defineLocale("tlh",{months:"tera’ jar wa’_tera’ jar cha’_tera’ jar wej_tera’ jar loS_tera’ jar vagh_tera’ jar jav_tera’ jar Soch_tera’ jar chorgh_tera’ jar Hut_tera’ jar wa’maH_tera’ jar wa’maH wa’_tera’ jar wa’maH cha’".split("_"),monthsShort:"jar wa’_jar cha’_jar wej_jar loS_jar vagh_jar jav_jar Soch_jar chorgh_jar Hut_jar wa’maH_jar wa’maH wa’_jar wa’maH cha’".split("_"),monthsParseExact:!0,weekdays:"lojmItjaj_DaSjaj_povjaj_ghItlhjaj_loghjaj_buqjaj_ghInjaj".split("_"),weekdaysShort:"lojmItjaj_DaSjaj_povjaj_ghItlhjaj_loghjaj_buqjaj_ghInjaj".split("_"),weekdaysMin:"lojmItjaj_DaSjaj_povjaj_ghItlhjaj_loghjaj_buqjaj_ghInjaj".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[DaHjaj] LT",nextDay:"[wa’leS] LT",nextWeek:"LLL",lastDay:"[wa’Hu’] LT",lastWeek:"LLL",sameElse:"L"},relativeTime:{future:function(e){var t=e;return t=-1!==e.indexOf("jaj")?t.slice(0,-3)+"leS":-1!==e.indexOf("jar")?t.slice(0,-3)+"waQ":-1!==e.indexOf("DIS")?t.slice(0,-3)+"nem":t+" pIq"},past:function(e){var t=e;return t=-1!==e.indexOf("jaj")?t.slice(0,-3)+"Hu’":-1!==e.indexOf("jar")?t.slice(0,-3)+"wen":-1!==e.indexOf("DIS")?t.slice(0,-3)+"ben":t+" ret"},s:"puS lup",ss:n,m:"wa’ tup",mm:n,h:"wa’ rep",hh:n,d:"wa’ jaj",dd:n,M:"wa’ jar",MM:n,y:"wa’ DIS",yy:n},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t={1:"'inci",5:"'inci",8:"'inci",70:"'inci",80:"'inci",2:"'nci",7:"'nci",20:"'nci",50:"'nci",3:"'üncü",4:"'üncü",100:"'üncü",6:"'ncı",9:"'uncu",10:"'uncu",30:"'uncu",60:"'ıncı",90:"'ıncı"};e.defineLocale("tr",{months:"Ocak_Şubat_Mart_Nisan_Mayıs_Haziran_Temmuz_Ağustos_Eylül_Ekim_Kasım_Aralık".split("_"),monthsShort:"Oca_Şub_Mar_Nis_May_Haz_Tem_Ağu_Eyl_Eki_Kas_Ara".split("_"),weekdays:"Pazar_Pazartesi_Salı_Çarşamba_Perşembe_Cuma_Cumartesi".split("_"),weekdaysShort:"Paz_Pts_Sal_Çar_Per_Cum_Cts".split("_"),weekdaysMin:"Pz_Pt_Sa_Ça_Pe_Cu_Ct".split("_"),meridiem:function(e,t,n){return e<12?n?"öö":"ÖÖ":n?"ös":"ÖS"},meridiemParse:/öö|ÖÖ|ös|ÖS/,isPM:function(e){return"ös"===e||"ÖS"===e},longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[bugün saat] LT",nextDay:"[yarın saat] LT",nextWeek:"[gelecek] dddd [saat] LT",lastDay:"[dün] LT",lastWeek:"[geçen] dddd [saat] LT",sameElse:"L"},relativeTime:{future:"%s sonra",past:"%s önce",s:"birkaç saniye",ss:"%d saniye",m:"bir dakika",mm:"%d dakika",h:"bir saat",hh:"%d saat",d:"bir gün",dd:"%d gün",w:"bir hafta",ww:"%d hafta",M:"bir ay",MM:"%d ay",y:"bir yıl",yy:"%d yıl"},ordinal:function(e,n){switch(n){case"d":case"D":case"Do":case"DD":return e;default:if(0===e)return e+"'ıncı";var r=e%10;return e+(t[r]||t[e%100-r]||t[e>=100?100:null])}},week:{dow:1,doy:7}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
function t(e,t,n,r){var i={s:["viensas secunds","'iensas secunds"],ss:[e+" secunds",e+" secunds"],m:["'n míut","'iens míut"],mm:[e+" míuts",e+" míuts"],h:["'n þora","'iensa þora"],hh:[e+" þoras",e+" þoras"],d:["'n ziua","'iensa ziua"],dd:[e+" ziuas",e+" ziuas"],M:["'n mes","'iens mes"],MM:[e+" mesen",e+" mesen"],y:["'n ar","'iens ar"],yy:[e+" ars",e+" ars"]};return r||t?i[n][0]:i[n][1]}e.defineLocale("tzl",{months:"Januar_Fevraglh_Març_Avrïu_Mai_Gün_Julia_Guscht_Setemvar_Listopäts_Noemvar_Zecemvar".split("_"),monthsShort:"Jan_Fev_Mar_Avr_Mai_Gün_Jul_Gus_Set_Lis_Noe_Zec".split("_"),weekdays:"Súladi_Lúneçi_Maitzi_Márcuri_Xhúadi_Viénerçi_Sáturi".split("_"),weekdaysShort:"Súl_Lún_Mai_Már_Xhú_Vié_Sát".split("_"),weekdaysMin:"Sú_Lú_Ma_Má_Xh_Vi_Sá".split("_"),longDateFormat:{LT:"HH.mm",LTS:"HH.mm.ss",L:"DD.MM.YYYY",LL:"D. MMMM [dallas] YYYY",LLL:"D. MMMM [dallas] YYYY HH.mm",LLLL:"dddd, [li] D. MMMM [dallas] YYYY HH.mm"},meridiemParse:/d\'o|d\'a/i,isPM:function(e){return"d'o"===e.toLowerCase()},meridiem:function(e,t,n){return e>11?n?"d'o":"D'O":n?"d'a":"D'A"},calendar:{sameDay:"[oxhi à] LT",nextDay:"[demà à] LT",nextWeek:"dddd [à] LT",lastDay:"[ieiri à] LT",lastWeek:"[sür el] dddd [lasteu à] LT",sameElse:"L"},relativeTime:{future:"osprei %s",past:"ja%s",s:t,ss:t,m:t,mm:t,h:t,hh:t,d:t,dd:t,M:t,MM:t,y:t,yy:t},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("tzm",{months:"ⵉⵏⵏⴰⵢⵔ_ⴱⵕⴰⵢⵕ_ⵎⴰⵕⵚ_ⵉⴱⵔⵉⵔ_ⵎⴰⵢⵢⵓ_ⵢⵓⵏⵢⵓ_ⵢⵓⵍⵢⵓⵣ_ⵖⵓⵛⵜ_ⵛⵓⵜⴰⵏⴱⵉⵔ_ⴽⵟⵓⴱⵕ_ⵏⵓⵡⴰⵏⴱⵉⵔ_ⴷⵓⵊⵏⴱⵉⵔ".split("_"),monthsShort:"ⵉⵏⵏⴰⵢⵔ_ⴱⵕⴰⵢⵕ_ⵎⴰⵕⵚ_ⵉⴱⵔⵉⵔ_ⵎⴰⵢⵢⵓ_ⵢⵓⵏⵢⵓ_ⵢⵓⵍⵢⵓⵣ_ⵖⵓⵛⵜ_ⵛⵓⵜⴰⵏⴱⵉⵔ_ⴽⵟⵓⴱⵕ_ⵏⵓⵡⴰⵏⴱⵉⵔ_ⴷⵓⵊⵏⴱⵉⵔ".split("_"),weekdays:"ⴰⵙⴰⵎⴰⵙ_ⴰⵢⵏⴰⵙ_ⴰⵙⵉⵏⴰⵙ_ⴰⴽⵔⴰⵙ_ⴰⴽⵡⴰⵙ_ⴰⵙⵉⵎⵡⴰⵙ_ⴰⵙⵉⴹⵢⴰⵙ".split("_"),weekdaysShort:"ⴰⵙⴰⵎⴰⵙ_ⴰⵢⵏⴰⵙ_ⴰⵙⵉⵏⴰⵙ_ⴰⴽⵔⴰⵙ_ⴰⴽⵡⴰⵙ_ⴰⵙⵉⵎⵡⴰⵙ_ⴰⵙⵉⴹⵢⴰⵙ".split("_"),weekdaysMin:"ⴰⵙⴰⵎⴰⵙ_ⴰⵢⵏⴰⵙ_ⴰⵙⵉⵏⴰⵙ_ⴰⴽⵔⴰⵙ_ⴰⴽⵡⴰⵙ_ⴰⵙⵉⵎⵡⴰⵙ_ⴰⵙⵉⴹⵢⴰⵙ".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[ⴰⵙⴷⵅ ⴴ] LT",nextDay:"[ⴰⵙⴽⴰ ⴴ] LT",nextWeek:"dddd [ⴴ] LT",lastDay:"[ⴰⵚⴰⵏⵜ ⴴ] LT",lastWeek:"dddd [ⴴ] LT",sameElse:"L"},relativeTime:{future:"ⴷⴰⴷⵅ ⵙ ⵢⴰⵏ %s",past:"ⵢⴰⵏ %s",s:"ⵉⵎⵉⴽ",ss:"%d ⵉⵎⵉⴽ",m:"ⵎⵉⵏⵓⴺ",mm:"%d ⵎⵉⵏⵓⴺ",h:"ⵙⴰⵄⴰ",hh:"%d ⵜⴰⵙⵙⴰⵄⵉⵏ",d:"ⴰⵙⵙ",dd:"%d oⵙⵙⴰⵏ",M:"ⴰⵢoⵓⵔ",MM:"%d ⵉⵢⵢⵉⵔⵏ",y:"ⴰⵙⴳⴰⵙ",yy:"%d ⵉⵙⴳⴰⵙⵏ"},week:{dow:6,doy:12}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("tzm-latn",{months:"innayr_brˤayrˤ_marˤsˤ_ibrir_mayyw_ywnyw_ywlywz_ɣwšt_šwtanbir_ktˤwbrˤ_nwwanbir_dwjnbir".split("_"),monthsShort:"innayr_brˤayrˤ_marˤsˤ_ibrir_mayyw_ywnyw_ywlywz_ɣwšt_šwtanbir_ktˤwbrˤ_nwwanbir_dwjnbir".split("_"),weekdays:"asamas_aynas_asinas_akras_akwas_asimwas_asiḍyas".split("_"),weekdaysShort:"asamas_aynas_asinas_akras_akwas_asimwas_asiḍyas".split("_"),weekdaysMin:"asamas_aynas_asinas_akras_akwas_asimwas_asiḍyas".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[asdkh g] LT",nextDay:"[aska g] LT",nextWeek:"dddd [g] LT",lastDay:"[assant g] LT",lastWeek:"dddd [g] LT",sameElse:"L"},relativeTime:{future:"dadkh s yan %s",past:"yan %s",s:"imik",ss:"%d imik",m:"minuḍ",mm:"%d minuḍ",h:"saɛa",hh:"%d tassaɛin",d:"ass",dd:"%d ossan",M:"ayowr",MM:"%d iyyirn",y:"asgas",yy:"%d isgasn"},week:{dow:6,doy:12}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("ug-cn",{months:"يانۋار_فېۋرال_مارت_ئاپرېل_ماي_ئىيۇن_ئىيۇل_ئاۋغۇست_سېنتەبىر_ئۆكتەبىر_نويابىر_دېكابىر".split("_"),monthsShort:"يانۋار_فېۋرال_مارت_ئاپرېل_ماي_ئىيۇن_ئىيۇل_ئاۋغۇست_سېنتەبىر_ئۆكتەبىر_نويابىر_دېكابىر".split("_"),weekdays:"يەكشەنبە_دۈشەنبە_سەيشەنبە_چارشەنبە_پەيشەنبە_جۈمە_شەنبە".split("_"),weekdaysShort:"يە_دۈ_سە_چا_پە_جۈ_شە".split("_"),weekdaysMin:"يە_دۈ_سە_چا_پە_جۈ_شە".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"YYYY-يىلىM-ئاينىڭD-كۈنى",LLL:"YYYY-يىلىM-ئاينىڭD-كۈنى، HH:mm",LLLL:"dddd، YYYY-يىلىM-ئاينىڭD-كۈنى، HH:mm"},meridiemParse:/يېرىم كېچە|سەھەر|چۈشتىن بۇرۇن|چۈش|چۈشتىن كېيىن|كەچ/,meridiemHour:function(e,t){return 12===e&&(e=0),"يېرىم كېچە"===t||"سەھەر"===t||"چۈشتىن بۇرۇن"===t?e:"چۈشتىن كېيىن"===t||"كەچ"===t?e+12:e>=11?e:e+12},meridiem:function(e,t,n){var r=100*e+t;return r<600?"يېرىم كېچە":r<900?"سەھەر":r<1130?"چۈشتىن بۇرۇن":r<1230?"چۈش":r<1800?"چۈشتىن كېيىن":"كەچ"},calendar:{sameDay:"[بۈگۈن سائەت] LT",nextDay:"[ئەتە سائەت] LT",nextWeek:"[كېلەركى] dddd [سائەت] LT",lastDay:"[تۆنۈگۈن] LT",lastWeek:"[ئالدىنقى] dddd [سائەت] LT",sameElse:"L"},relativeTime:{future:"%s كېيىن",past:"%s بۇرۇن",s:"نەچچە سېكونت",ss:"%d سېكونت",m:"بىر مىنۇت",mm:"%d مىنۇت",h:"بىر سائەت",hh:"%d سائەت",d:"بىر كۈن",dd:"%d كۈن",M:"بىر ئاي",MM:"%d ئاي",y:"بىر يىل",yy:"%d يىل"},dayOfMonthOrdinalParse:/\d{1,2}(-كۈنى|-ئاي|-ھەپتە)/,ordinal:function(e,t){switch(t){case"d":case"D":case"DDD":return e+"-كۈنى";case"w":case"W":return e+"-ھەپتە";default:return e}},preparse:function(e){return e.replace(/،/g,",")},postformat:function(e){return e.replace(/,/g,"،")},week:{dow:1,doy:7}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
function t(e,t,n){var r,i;return"m"===n?t?"хвилина":"хвилину":"h"===n?t?"година":"годину":e+" "+(r=+e,i={ss:t?"секунда_секунди_секунд":"секунду_секунди_секунд",mm:t?"хвилина_хвилини_хвилин":"хвилину_хвилини_хвилин",hh:t?"година_години_годин":"годину_години_годин",dd:"день_дні_днів",MM:"місяць_місяці_місяців",yy:"рік_роки_років"}[n].split("_"),r%10==1&&r%100!=11?i[0]:r%10>=2&&r%10<=4&&(r%100<10||r%100>=20)?i[1]:i[2])}function n(e){return function(){return e+"о"+(11===this.hours()?"б":"")+"] LT"}}e.defineLocale("uk",{months:{format:"січня_лютого_березня_квітня_травня_червня_липня_серпня_вересня_жовтня_листопада_грудня".split("_"),standalone:"січень_лютий_березень_квітень_травень_червень_липень_серпень_вересень_жовтень_листопад_грудень".split("_")},monthsShort:"січ_лют_бер_квіт_трав_черв_лип_серп_вер_жовт_лист_груд".split("_"),weekdays:function(e,t){var n={nominative:"неділя_понеділок_вівторок_середа_четвер_п’ятниця_субота".split("_"),accusative:"неділю_понеділок_вівторок_середу_четвер_п’ятницю_суботу".split("_"),genitive:"неділі_понеділка_вівторка_середи_четверга_п’ятниці_суботи".split("_")};return!0===e?n.nominative.slice(1,7).concat(n.nominative.slice(0,1)):e?n[/(\[[ВвУу]\]) ?dddd/.test(t)?"accusative":/\[?(?:минулої|наступної)? ?\] ?dddd/.test(t)?"genitive":"nominative"][e.day()]:n.nominative},weekdaysShort:"нд_пн_вт_ср_чт_пт_сб".split("_"),weekdaysMin:"нд_пн_вт_ср_чт_пт_сб".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY р.",LLL:"D MMMM YYYY р., HH:mm",LLLL:"dddd, D MMMM YYYY р., HH:mm"},calendar:{sameDay:n("[Сьогодні "),nextDay:n("[Завтра "),lastDay:n("[Вчора "),nextWeek:n("[У] dddd ["),lastWeek:function(){switch(this.day()){case 0:case 3:case 5:case 6:return n("[Минулої] dddd [").call(this);case 1:case 2:case 4:return n("[Минулого] dddd [").call(this)}},sameElse:"L"},relativeTime:{future:"за %s",past:"%s тому",s:"декілька секунд",ss:t,m:t,mm:t,h:"годину",hh:t,d:"день",dd:t,M:"місяць",MM:t,y:"рік",yy:t},meridiemParse:/ночі|ранку|дня|вечора/,isPM:function(e){return/^(дня|вечора)$/.test(e)},meridiem:function(e,t,n){return e<4?"ночі":e<12?"ранку":e<17?"дня":"вечора"},dayOfMonthOrdinalParse:/\d{1,2}-(й|го)/,ordinal:function(e,t){switch(t){case"M":case"d":case"DDD":case"w":case"W":return e+"-й";case"D":return e+"-го";default:return e}},week:{dow:1,doy:7}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
var t=["جنوری","فروری","مارچ","اپریل","مئی","جون","جولائی","اگست","ستمبر","اکتوبر","نومبر","دسمبر"],n=["اتوار","پیر","منگل","بدھ","جمعرات","جمعہ","ہفتہ"];e.defineLocale("ur",{months:t,monthsShort:t,weekdays:n,weekdaysShort:n,weekdaysMin:n,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd، D MMMM YYYY HH:mm"},meridiemParse:/صبح|شام/,isPM:function(e){return"شام"===e},meridiem:function(e,t,n){return e<12?"صبح":"شام"},calendar:{sameDay:"[آج بوقت] LT",nextDay:"[کل بوقت] LT",nextWeek:"dddd [بوقت] LT",lastDay:"[گذشتہ روز بوقت] LT",lastWeek:"[گذشتہ] dddd [بوقت] LT",sameElse:"L"},relativeTime:{future:"%s بعد",past:"%s قبل",s:"چند سیکنڈ",ss:"%d سیکنڈ",m:"ایک منٹ",mm:"%d منٹ",h:"ایک گھنٹہ",hh:"%d گھنٹے",d:"ایک دن",dd:"%d دن",M:"ایک ماہ",MM:"%d ماہ",y:"ایک سال",yy:"%d سال"},preparse:function(e){return e.replace(/،/g,",")},postformat:function(e){return e.replace(/,/g,"،")},week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("uz",{months:"январ_феврал_март_апрел_май_июн_июл_август_сентябр_октябр_ноябр_декабр".split("_"),monthsShort:"янв_фев_мар_апр_май_июн_июл_авг_сен_окт_ноя_дек".split("_"),weekdays:"Якшанба_Душанба_Сешанба_Чоршанба_Пайшанба_Жума_Шанба".split("_"),weekdaysShort:"Якш_Душ_Сеш_Чор_Пай_Жум_Шан".split("_"),weekdaysMin:"Як_Ду_Се_Чо_Па_Жу_Ша".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"D MMMM YYYY, dddd HH:mm"},calendar:{sameDay:"[Бугун соат] LT [да]",nextDay:"[Эртага] LT [да]",nextWeek:"dddd [куни соат] LT [да]",lastDay:"[Кеча соат] LT [да]",lastWeek:"[Утган] dddd [куни соат] LT [да]",sameElse:"L"},relativeTime:{future:"Якин %s ичида",past:"Бир неча %s олдин",s:"фурсат",ss:"%d фурсат",m:"бир дакика",mm:"%d дакика",h:"бир соат",hh:"%d соат",d:"бир кун",dd:"%d кун",M:"бир ой",MM:"%d ой",y:"бир йил",yy:"%d йил"},week:{dow:1,doy:7}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("uz-latn",{months:"Yanvar_Fevral_Mart_Aprel_May_Iyun_Iyul_Avgust_Sentabr_Oktabr_Noyabr_Dekabr".split("_"),monthsShort:"Yan_Fev_Mar_Apr_May_Iyun_Iyul_Avg_Sen_Okt_Noy_Dek".split("_"),weekdays:"Yakshanba_Dushanba_Seshanba_Chorshanba_Payshanba_Juma_Shanba".split("_"),weekdaysShort:"Yak_Dush_Sesh_Chor_Pay_Jum_Shan".split("_"),weekdaysMin:"Ya_Du_Se_Cho_Pa_Ju_Sha".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"D MMMM YYYY, dddd HH:mm"},calendar:{sameDay:"[Bugun soat] LT [da]",nextDay:"[Ertaga] LT [da]",nextWeek:"dddd [kuni soat] LT [da]",lastDay:"[Kecha soat] LT [da]",lastWeek:"[O'tgan] dddd [kuni soat] LT [da]",sameElse:"L"},relativeTime:{future:"Yaqin %s ichida",past:"Bir necha %s oldin",s:"soniya",ss:"%d soniya",m:"bir daqiqa",mm:"%d daqiqa",h:"bir soat",hh:"%d soat",d:"bir kun",dd:"%d kun",M:"bir oy",MM:"%d oy",y:"bir yil",yy:"%d yil"},week:{dow:1,doy:7}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("vi",{months:"tháng 1_tháng 2_tháng 3_tháng 4_tháng 5_tháng 6_tháng 7_tháng 8_tháng 9_tháng 10_tháng 11_tháng 12".split("_"),monthsShort:"Thg 01_Thg 02_Thg 03_Thg 04_Thg 05_Thg 06_Thg 07_Thg 08_Thg 09_Thg 10_Thg 11_Thg 12".split("_"),monthsParseExact:!0,weekdays:"chủ nhật_thứ hai_thứ ba_thứ tư_thứ năm_thứ sáu_thứ bảy".split("_"),weekdaysShort:"CN_T2_T3_T4_T5_T6_T7".split("_"),weekdaysMin:"CN_T2_T3_T4_T5_T6_T7".split("_"),weekdaysParseExact:!0,meridiemParse:/sa|ch/i,isPM:function(e){return/^ch$/i.test(e)},meridiem:function(e,t,n){return e<12?n?"sa":"SA":n?"ch":"CH"},longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM [năm] YYYY",LLL:"D MMMM [năm] YYYY HH:mm",LLLL:"dddd, D MMMM [năm] YYYY HH:mm",l:"DD/M/YYYY",ll:"D MMM YYYY",lll:"D MMM YYYY HH:mm",llll:"ddd, D MMM YYYY HH:mm"},calendar:{sameDay:"[Hôm nay lúc] LT",nextDay:"[Ngày mai lúc] LT",nextWeek:"dddd [tuần tới lúc] LT",lastDay:"[Hôm qua lúc] LT",lastWeek:"dddd [tuần trước lúc] LT",sameElse:"L"},relativeTime:{future:"%s tới",past:"%s trước",s:"vài giây",ss:"%d giây",m:"một phút",mm:"%d phút",h:"một giờ",hh:"%d giờ",d:"một ngày",dd:"%d ngày",w:"một tuần",ww:"%d tuần",M:"một tháng",MM:"%d tháng",y:"một năm",yy:"%d năm"},dayOfMonthOrdinalParse:/\d{1,2}/,ordinal:function(e){return e},week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("x-pseudo",{months:"J~áñúá~rý_F~ébrú~árý_~Márc~h_Áp~ríl_~Máý_~Júñé~_Júl~ý_Áú~gúst~_Sép~témb~ér_Ó~ctób~ér_Ñ~óvém~bér_~Décé~mbér".split("_"),monthsShort:"J~áñ_~Féb_~Már_~Ápr_~Máý_~Júñ_~Júl_~Áúg_~Sép_~Óct_~Ñóv_~Déc".split("_"),monthsParseExact:!0,weekdays:"S~úñdá~ý_Mó~ñdáý~_Túé~sdáý~_Wéd~ñésd~áý_T~húrs~dáý_~Fríd~áý_S~átúr~dáý".split("_"),weekdaysShort:"S~úñ_~Móñ_~Túé_~Wéd_~Thú_~Frí_~Sát".split("_"),weekdaysMin:"S~ú_Mó~_Tú_~Wé_T~h_Fr~_Sá".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[T~ódá~ý át] LT",nextDay:"[T~ómó~rró~w át] LT",nextWeek:"dddd [át] LT",lastDay:"[Ý~ést~érdá~ý át] LT",lastWeek:"[L~ást] dddd [át] LT",sameElse:"L"},relativeTime:{future:"í~ñ %s",past:"%s á~gó",s:"á ~féw ~sécó~ñds",ss:"%d s~écóñ~ds",m:"á ~míñ~úté",mm:"%d m~íñú~tés",h:"á~ñ hó~úr",hh:"%d h~óúrs",d:"á ~dáý",dd:"%d d~áýs",M:"á ~móñ~th",MM:"%d m~óñt~hs",y:"á ~ýéár",yy:"%d ý~éárs"},dayOfMonthOrdinalParse:/\d{1,2}(th|st|nd|rd)/,ordinal:function(e){var t=e%10;return e+(1==~~(e%100/10)?"th":1===t?"st":2===t?"nd":3===t?"rd":"th")},week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("yo",{months:"Sẹ́rẹ́_Èrèlè_Ẹrẹ̀nà_Ìgbé_Èbibi_Òkùdu_Agẹmo_Ògún_Owewe_Ọ̀wàrà_Bélú_Ọ̀pẹ̀̀".split("_"),monthsShort:"Sẹ́r_Èrl_Ẹrn_Ìgb_Èbi_Òkù_Agẹ_Ògú_Owe_Ọ̀wà_Bél_Ọ̀pẹ̀̀".split("_"),weekdays:"Àìkú_Ajé_Ìsẹ́gun_Ọjọ́rú_Ọjọ́bọ_Ẹtì_Àbámẹ́ta".split("_"),weekdaysShort:"Àìk_Ajé_Ìsẹ́_Ọjr_Ọjb_Ẹtì_Àbá".split("_"),weekdaysMin:"Àì_Aj_Ìs_Ọr_Ọb_Ẹt_Àb".split("_"),longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY h:mm A",LLLL:"dddd, D MMMM YYYY h:mm A"},calendar:{sameDay:"[Ònì ni] LT",nextDay:"[Ọ̀la ni] LT",nextWeek:"dddd [Ọsẹ̀ tón'bọ] [ni] LT",lastDay:"[Àna ni] LT",lastWeek:"dddd [Ọsẹ̀ tólọ́] [ni] LT",sameElse:"L"},relativeTime:{future:"ní %s",past:"%s kọjá",s:"ìsẹjú aayá die",ss:"aayá %d",m:"ìsẹjú kan",mm:"ìsẹjú %d",h:"wákati kan",hh:"wákati %d",d:"ọjọ́ kan",dd:"ọjọ́ %d",M:"osù kan",MM:"osù %d",y:"ọdún kan",yy:"ọdún %d"},dayOfMonthOrdinalParse:/ọjọ́\s\d{1,2}/,ordinal:"ọjọ́ %d",week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("zh-cn",{months:"一月_二月_三月_四月_五月_六月_七月_八月_九月_十月_十一月_十二月".split("_"),monthsShort:"1月_2月_3月_4月_5月_6月_7月_8月_9月_10月_11月_12月".split("_"),weekdays:"星期日_星期一_星期二_星期三_星期四_星期五_星期六".split("_"),weekdaysShort:"周日_周一_周二_周三_周四_周五_周六".split("_"),weekdaysMin:"日_一_二_三_四_五_六".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY/MM/DD",LL:"YYYY年M月D日",LLL:"YYYY年M月D日Ah点mm分",LLLL:"YYYY年M月D日ddddAh点mm分",l:"YYYY/M/D",ll:"YYYY年M月D日",lll:"YYYY年M月D日 HH:mm",llll:"YYYY年M月D日dddd HH:mm"},meridiemParse:/凌晨|早上|上午|中午|下午|晚上/,meridiemHour:function(e,t){return 12===e&&(e=0),"凌晨"===t||"早上"===t||"上午"===t?e:"下午"===t||"晚上"===t?e+12:e>=11?e:e+12},meridiem:function(e,t,n){var r=100*e+t;return r<600?"凌晨":r<900?"早上":r<1130?"上午":r<1230?"中午":r<1800?"下午":"晚上"},calendar:{sameDay:"[今天]LT",nextDay:"[明天]LT",nextWeek:function(e){return e.week()!==this.week()?"[下]dddLT":"[本]dddLT"},lastDay:"[昨天]LT",lastWeek:function(e){return this.week()!==e.week()?"[上]dddLT":"[本]dddLT"},sameElse:"L"},dayOfMonthOrdinalParse:/\d{1,2}(日|月|周)/,ordinal:function(e,t){switch(t){case"d":case"D":case"DDD":return e+"日";case"M":return e+"月";case"w":case"W":return e+"周";default:return e}},relativeTime:{future:"%s后",past:"%s前",s:"几秒",ss:"%d 秒",m:"1 分钟",mm:"%d 分钟",h:"1 小时",hh:"%d 小时",d:"1 天",dd:"%d 天",w:"1 周",ww:"%d 周",M:"1 个月",MM:"%d 个月",y:"1 年",yy:"%d 年"},week:{dow:1,doy:4}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("zh-hk",{months:"一月_二月_三月_四月_五月_六月_七月_八月_九月_十月_十一月_十二月".split("_"),monthsShort:"1月_2月_3月_4月_5月_6月_7月_8月_9月_10月_11月_12月".split("_"),weekdays:"星期日_星期一_星期二_星期三_星期四_星期五_星期六".split("_"),weekdaysShort:"週日_週一_週二_週三_週四_週五_週六".split("_"),weekdaysMin:"日_一_二_三_四_五_六".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY/MM/DD",LL:"YYYY年M月D日",LLL:"YYYY年M月D日 HH:mm",LLLL:"YYYY年M月D日dddd HH:mm",l:"YYYY/M/D",ll:"YYYY年M月D日",lll:"YYYY年M月D日 HH:mm",llll:"YYYY年M月D日dddd HH:mm"},meridiemParse:/凌晨|早上|上午|中午|下午|晚上/,meridiemHour:function(e,t){return 12===e&&(e=0),"凌晨"===t||"早上"===t||"上午"===t?e:"中午"===t?e>=11?e:e+12:"下午"===t||"晚上"===t?e+12:void 0},meridiem:function(e,t,n){var r=100*e+t;return r<600?"凌晨":r<900?"早上":r<1200?"上午":1200===r?"中午":r<1800?"下午":"晚上"},calendar:{sameDay:"[今天]LT",nextDay:"[明天]LT",nextWeek:"[下]ddddLT",lastDay:"[昨天]LT",lastWeek:"[上]ddddLT",sameElse:"L"},dayOfMonthOrdinalParse:/\d{1,2}(日|月|週)/,ordinal:function(e,t){switch(t){case"d":case"D":case"DDD":return e+"日";case"M":return e+"月";case"w":case"W":return e+"週";default:return e}},relativeTime:{future:"%s後",past:"%s前",s:"幾秒",ss:"%d 秒",m:"1 分鐘",mm:"%d 分鐘",h:"1 小時",hh:"%d 小時",d:"1 天",dd:"%d 天",M:"1 個月",MM:"%d 個月",y:"1 年",yy:"%d 年"}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("zh-mo",{months:"一月_二月_三月_四月_五月_六月_七月_八月_九月_十月_十一月_十二月".split("_"),monthsShort:"1月_2月_3月_4月_5月_6月_7月_8月_9月_10月_11月_12月".split("_"),weekdays:"星期日_星期一_星期二_星期三_星期四_星期五_星期六".split("_"),weekdaysShort:"週日_週一_週二_週三_週四_週五_週六".split("_"),weekdaysMin:"日_一_二_三_四_五_六".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"YYYY年M月D日",LLL:"YYYY年M月D日 HH:mm",LLLL:"YYYY年M月D日dddd HH:mm",l:"D/M/YYYY",ll:"YYYY年M月D日",lll:"YYYY年M月D日 HH:mm",llll:"YYYY年M月D日dddd HH:mm"},meridiemParse:/凌晨|早上|上午|中午|下午|晚上/,meridiemHour:function(e,t){return 12===e&&(e=0),"凌晨"===t||"早上"===t||"上午"===t?e:"中午"===t?e>=11?e:e+12:"下午"===t||"晚上"===t?e+12:void 0},meridiem:function(e,t,n){var r=100*e+t;return r<600?"凌晨":r<900?"早上":r<1130?"上午":r<1230?"中午":r<1800?"下午":"晚上"},calendar:{sameDay:"[今天] LT",nextDay:"[明天] LT",nextWeek:"[下]dddd LT",lastDay:"[昨天] LT",lastWeek:"[上]dddd LT",sameElse:"L"},dayOfMonthOrdinalParse:/\d{1,2}(日|月|週)/,ordinal:function(e,t){switch(t){case"d":case"D":case"DDD":return e+"日";case"M":return e+"月";case"w":case"W":return e+"週";default:return e}},relativeTime:{future:"%s內",past:"%s前",s:"幾秒",ss:"%d 秒",m:"1 分鐘",mm:"%d 分鐘",h:"1 小時",hh:"%d 小時",d:"1 天",dd:"%d 天",M:"1 個月",MM:"%d 個月",y:"1 年",yy:"%d 年"}})}(n(12))},function(e,t,n){!function(e){"use strict";
//! moment.js locale configuration
e.defineLocale("zh-tw",{months:"一月_二月_三月_四月_五月_六月_七月_八月_九月_十月_十一月_十二月".split("_"),monthsShort:"1月_2月_3月_4月_5月_6月_7月_8月_9月_10月_11月_12月".split("_"),weekdays:"星期日_星期一_星期二_星期三_星期四_星期五_星期六".split("_"),weekdaysShort:"週日_週一_週二_週三_週四_週五_週六".split("_"),weekdaysMin:"日_一_二_三_四_五_六".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY/MM/DD",LL:"YYYY年M月D日",LLL:"YYYY年M月D日 HH:mm",LLLL:"YYYY年M月D日dddd HH:mm",l:"YYYY/M/D",ll:"YYYY年M月D日",lll:"YYYY年M月D日 HH:mm",llll:"YYYY年M月D日dddd HH:mm"},meridiemParse:/凌晨|早上|上午|中午|下午|晚上/,meridiemHour:function(e,t){return 12===e&&(e=0),"凌晨"===t||"早上"===t||"上午"===t?e:"中午"===t?e>=11?e:e+12:"下午"===t||"晚上"===t?e+12:void 0},meridiem:function(e,t,n){var r=100*e+t;return r<600?"凌晨":r<900?"早上":r<1130?"上午":r<1230?"中午":r<1800?"下午":"晚上"},calendar:{sameDay:"[今天] LT",nextDay:"[明天] LT",nextWeek:"[下]dddd LT",lastDay:"[昨天] LT",lastWeek:"[上]dddd LT",sameElse:"L"},dayOfMonthOrdinalParse:/\d{1,2}(日|月|週)/,ordinal:function(e,t){switch(t){case"d":case"D":case"DDD":return e+"日";case"M":return e+"月";case"w":case"W":return e+"週";default:return e}},relativeTime:{future:"%s後",past:"%s前",s:"幾秒",ss:"%d 秒",m:"1 分鐘",mm:"%d 分鐘",h:"1 小時",hh:"%d 小時",d:"1 天",dd:"%d 天",M:"1 個月",MM:"%d 個月",y:"1 年",yy:"%d 年"}})}(n(12))},function(e,t,n){"use strict";var r=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),i=this&&this.__setModuleDefault||(Object.create?function(e,t){Object.defineProperty(e,"default",{enumerable:!0,value:t})}:function(e,t){e.default=t}),o=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)"default"!==n&&Object.prototype.hasOwnProperty.call(e,n)&&r(t,e,n);return i(t,e),t};Object.defineProperty(t,"__esModule",{value:!0}),t.Drive=t.ContentsManager=t.Contents=void 0;const a=n(48),s=n(6),l=n(16),u=n(84),c=o(n(1355));var d;!function(e){e.validateContentsModel=function(e){c.validateContentsModel(e)},e.validateCheckpointModel=function(e){c.validateCheckpointModel(e)}}(t.Contents||(t.Contents={}));t.ContentsManager=class{constructor(e={}){var t,n;this._isDisposed=!1,this._additionalDrives=new Map,this._fileChanged=new l.Signal(this);const r=this.serverSettings=null!==(t=e.serverSettings)&&void 0!==t?t:u.ServerConnection.makeSettings();this._defaultDrive=null!==(n=e.defaultDrive)&&void 0!==n?n:new h({serverSettings:r}),this._defaultDrive.fileChanged.connect(this._onFileChanged,this)}get fileChanged(){return this._fileChanged}get isDisposed(){return this._isDisposed}dispose(){this.isDisposed||(this._isDisposed=!0,l.Signal.clearData(this))}addDrive(e){this._additionalDrives.set(e.name,e),e.fileChanged.connect(this._onFileChanged,this)}getModelDBFactory(e){var t;const[n]=this._driveForPath(e);return null!==(t=null==n?void 0:n.modelDBFactory)&&void 0!==t?t:null}localPath(e){const t=e.split("/"),n=t[0].split(":");return 1!==n.length&&this._additionalDrives.has(n[0])?a.PathExt.join(n.slice(1).join(":"),...t.slice(1)):a.PathExt.removeSlash(e)}normalize(e){const t=e.split(":");return 1===t.length?a.PathExt.normalize(e):`${t[0]}:${a.PathExt.normalize(t.slice(1).join(":"))}`}resolvePath(e,t){const n=this.driveName(e),r=this.localPath(e),i=a.PathExt.resolve("/",r,t);return n?`${n}:${i}`:i}driveName(e){const t=e.split("/")[0].split(":");return 1===t.length?"":this._additionalDrives.has(t[0])?t[0]:""}get(e,t){const[n,r]=this._driveForPath(e);return n.get(r,t).then(e=>{const t=[];return"directory"===e.type&&e.content?(s.each(e.content,e=>{t.push(Object.assign(Object.assign({},e),{path:this._toGlobalPath(n,e.path)}))}),Object.assign(Object.assign({},e),{path:this._toGlobalPath(n,r),content:t})):Object.assign(Object.assign({},e),{path:this._toGlobalPath(n,r)})})}getDownloadUrl(e){const[t,n]=this._driveForPath(e);return t.getDownloadUrl(n)}newUntitled(e={}){if(e.path){const t=this.normalize(e.path),[n,r]=this._driveForPath(t);return n.newUntitled(Object.assign(Object.assign({},e),{path:r})).then(e=>Object.assign(Object.assign({},e),{path:a.PathExt.join(t,e.name)}))}return this._defaultDrive.newUntitled(e)}delete(e){const[t,n]=this._driveForPath(e);return t.delete(n)}rename(e,t){const[n,r]=this._driveForPath(e),[i,o]=this._driveForPath(t);if(n!==i)throw Error("ContentsManager: renaming files must occur within a Drive");return n.rename(r,o).then(e=>Object.assign(Object.assign({},e),{path:this._toGlobalPath(n,o)}))}save(e,t={}){const n=this.normalize(e),[r,i]=this._driveForPath(e);return r.save(i,Object.assign(Object.assign({},t),{path:i})).then(e=>Object.assign(Object.assign({},e),{path:n}))}copy(e,t){const[n,r]=this._driveForPath(e),[i,o]=this._driveForPath(t);if(n===i)return n.copy(r,o).then(e=>Object.assign(Object.assign({},e),{path:this._toGlobalPath(n,e.path)}));throw Error("Copying files between drives is not currently implemented")}createCheckpoint(e){const[t,n]=this._driveForPath(e);return t.createCheckpoint(n)}listCheckpoints(e){const[t,n]=this._driveForPath(e);return t.listCheckpoints(n)}restoreCheckpoint(e,t){const[n,r]=this._driveForPath(e);return n.restoreCheckpoint(r,t)}deleteCheckpoint(e,t){const[n,r]=this._driveForPath(e);return n.deleteCheckpoint(r,t)}_toGlobalPath(e,t){return e===this._defaultDrive?a.PathExt.removeSlash(t):`${e.name}:${a.PathExt.removeSlash(t)}`}_driveForPath(e){const t=this.driveName(e),n=this.localPath(e);return t?[this._additionalDrives.get(t),n]:[this._defaultDrive,n]}_onFileChanged(e,t){var n,r;if(e===this._defaultDrive)this._fileChanged.emit(t);else{let i=null,o=null;(null===(n=t.newValue)||void 0===n?void 0:n.path)&&(i=Object.assign(Object.assign({},t.newValue),{path:this._toGlobalPath(e,t.newValue.path)})),(null===(r=t.oldValue)||void 0===r?void 0:r.path)&&(o=Object.assign(Object.assign({},t.oldValue),{path:this._toGlobalPath(e,t.oldValue.path)})),this._fileChanged.emit({type:t.type,newValue:i,oldValue:o})}}};class h{constructor(e={}){var t,n,r;this._isDisposed=!1,this._fileChanged=new l.Signal(this),this.name=null!==(t=e.name)&&void 0!==t?t:"Default",this._apiEndpoint=null!==(n=e.apiEndpoint)&&void 0!==n?n:"api/contents",this.serverSettings=null!==(r=e.serverSettings)&&void 0!==r?r:u.ServerConnection.makeSettings()}get fileChanged(){return this._fileChanged}get isDisposed(){return this._isDisposed}dispose(){this.isDisposed||(this._isDisposed=!0,l.Signal.clearData(this))}async get(e,t){let n=this._getUrl(e);if(t){"notebook"===t.type&&delete t.format;const e=t.content?"1":"0",r=Object.assign(Object.assign({},t),{content:e});n+=a.URLExt.objectToQueryString(r)}const r=this.serverSettings,i=await u.ServerConnection.makeRequest(n,{},r);if(200!==i.status){throw await u.ServerConnection.ResponseError.create(i)}const o=await i.json();return c.validateContentsModel(o),o}getDownloadUrl(e){const t=this.serverSettings.baseUrl;let n=a.URLExt.join(t,"files",a.URLExt.encodeParts(e));const r=document.cookie.match("\\b_xsrf=([^;]*)\\b");if(r){const e=new URL(n);e.searchParams.append("_xsrf",r[1]),n=e.toString()}return Promise.resolve(n)}async newUntitled(e={}){var t;let n="{}";e&&(e.ext&&(e.ext=d.normalizeExtension(e.ext)),n=JSON.stringify(e));const r=this.serverSettings,i=this._getUrl(null!==(t=e.path)&&void 0!==t?t:""),o={method:"POST",body:n},a=await u.ServerConnection.makeRequest(i,o,r);if(201!==a.status){throw await u.ServerConnection.ResponseError.create(a)}const s=await a.json();return c.validateContentsModel(s),this._fileChanged.emit({type:"new",oldValue:null,newValue:s}),s}async delete(e){const t=this._getUrl(e),n=this.serverSettings,r=await u.ServerConnection.makeRequest(t,{method:"DELETE"},n);if(204!==r.status){throw await u.ServerConnection.ResponseError.create(r)}this._fileChanged.emit({type:"delete",oldValue:{path:e},newValue:null})}async rename(e,t){const n=this.serverSettings,r=this._getUrl(e),i={method:"PATCH",body:JSON.stringify({path:t})},o=await u.ServerConnection.makeRequest(r,i,n);if(200!==o.status){throw await u.ServerConnection.ResponseError.create(o)}const a=await o.json();return c.validateContentsModel(a),this._fileChanged.emit({type:"rename",oldValue:{path:e},newValue:a}),a}async save(e,t={}){const n=this.serverSettings,r=this._getUrl(e),i={method:"PUT",body:JSON.stringify(t)},o=await u.ServerConnection.makeRequest(r,i,n);if(200!==o.status&&201!==o.status){throw await u.ServerConnection.ResponseError.create(o)}const a=await o.json();return c.validateContentsModel(a),this._fileChanged.emit({type:"save",oldValue:null,newValue:a}),a}async copy(e,t){const n=this.serverSettings,r=this._getUrl(t),i={method:"POST",body:JSON.stringify({copy_from:e})},o=await u.ServerConnection.makeRequest(r,i,n);if(201!==o.status){throw await u.ServerConnection.ResponseError.create(o)}const a=await o.json();return c.validateContentsModel(a),this._fileChanged.emit({type:"new",oldValue:null,newValue:a}),a}async createCheckpoint(e){const t=this._getUrl(e,"checkpoints"),n=await u.ServerConnection.makeRequest(t,{method:"POST"},this.serverSettings);if(201!==n.status){throw await u.ServerConnection.ResponseError.create(n)}const r=await n.json();return c.validateCheckpointModel(r),r}async listCheckpoints(e){const t=this._getUrl(e,"checkpoints"),n=await u.ServerConnection.makeRequest(t,{},this.serverSettings);if(200!==n.status){throw await u.ServerConnection.ResponseError.create(n)}const r=await n.json();if(!Array.isArray(r))throw new Error("Invalid Checkpoint list");for(let e=0;e<r.length;e++)c.validateCheckpointModel(r[e]);return r}async restoreCheckpoint(e,t){const n=this._getUrl(e,"checkpoints",t),r=await u.ServerConnection.makeRequest(n,{method:"POST"},this.serverSettings);if(204!==r.status){throw await u.ServerConnection.ResponseError.create(r)}}async deleteCheckpoint(e,t){const n=this._getUrl(e,"checkpoints",t),r=await u.ServerConnection.makeRequest(n,{method:"DELETE"},this.serverSettings);if(204!==r.status){throw await u.ServerConnection.ResponseError.create(r)}}_getUrl(...e){const t=e.map(e=>a.URLExt.encodeParts(e)),n=this.serverSettings.baseUrl;return a.URLExt.join(n,this._apiEndpoint,...t)}}t.Drive=h,function(e){e.normalizeExtension=function(e){return e.length>0&&0!==e.indexOf(".")&&(e="."+e),e}}(d||(d={}))},function(e,t,n){"use strict";var r=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),i=this&&this.__setModuleDefault||(Object.create?function(e,t){Object.defineProperty(e,"default",{enumerable:!0,value:t})}:function(e,t){e.default=t}),o=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)"default"!==n&&Object.prototype.hasOwnProperty.call(e,n)&&r(t,e,n);return i(t,e),t},a=this&&this.__exportStar||function(e,t){for(var n in e)"default"===n||Object.prototype.hasOwnProperty.call(t,n)||r(t,e,n)};Object.defineProperty(t,"__esModule",{value:!0}),t.KernelAPI=t.KernelMessage=t.Kernel=void 0;const s=o(n(1356));t.Kernel=s;const l=o(n(353));t.KernelMessage=l;const u=o(n(487));t.KernelAPI=u,a(n(1359),t)},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.getSpecs=void 0;const r=n(129),i=n(1365),o=n(48);t.getSpecs=async function(e=r.ServerConnection.makeSettings()){const t=o.URLExt.join(e.baseUrl,"api/kernelspecs"),n=await r.ServerConnection.makeRequest(t,{},e);if(200!==n.status){throw await r.ServerConnection.ResponseError.create(n)}const a=await n.json();return i.validateSpecModels(a)}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.NbConvertManager=void 0;const r=n(48),i=n(129);t.NbConvertManager=class{constructor(e={}){var t;this.serverSettings=null!==(t=e.serverSettings)&&void 0!==t?t:i.ServerConnection.makeSettings()}async getExportFormats(){const e=this.serverSettings.baseUrl,t=r.URLExt.join(e,"api/nbconvert"),{serverSettings:n}=this,o=await i.ServerConnection.makeRequest(t,{},n);if(200!==o.status){throw await i.ServerConnection.ResponseError.create(o)}const a=await o.json(),s={};return Object.keys(a).forEach((function(e){const t=a[e].output_mimetype;s[e]={output_mimetype:t}})),s}}},function(e,t,n){"use strict";var r=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),i=this&&this.__setModuleDefault||(Object.create?function(e,t){Object.defineProperty(e,"default",{enumerable:!0,value:t})}:function(e,t){e.default=t}),o=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)"default"!==n&&Object.prototype.hasOwnProperty.call(e,n)&&r(t,e,n);return i(t,e),t},a=this&&this.__exportStar||function(e,t){for(var n in e)"default"===n||Object.prototype.hasOwnProperty.call(t,n)||r(t,e,n)};Object.defineProperty(t,"__esModule",{value:!0}),t.SessionAPI=t.Session=void 0;const s=o(n(1369));t.Session=s;const l=o(n(490));t.SessionAPI=l,a(n(1371),t)},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.SettingManager=void 0;const r=n(48),i=n(263),o=n(129);class a extends i.DataConnector{constructor(e={}){var t;super(),this.serverSettings=null!==(t=e.serverSettings)&&void 0!==t?t:o.ServerConnection.makeSettings()}async fetch(e){if(!e)throw new Error("Plugin `id` parameter is required for settings fetch.");const{serverSettings:t}=this,{baseUrl:n,appUrl:r}=t,{makeRequest:i,ResponseError:a}=o.ServerConnection,l=n+r,u=s.url(l,e),c=await i(u,{},t);if(200!==c.status){throw await a.create(c)}return c.json()}async list(){var e,t;const{serverSettings:n}=this,{baseUrl:r,appUrl:i}=n,{makeRequest:a,ResponseError:l}=o.ServerConnection,u=r+i,c=s.url(u,""),d=await a(c,{},n);if(200!==d.status)throw new l(d);const h=await d.json(),f=null!==(t=null===(e=null==h?void 0:h.settings)||void 0===e?void 0:e.map(e=>(e.data={composite:{},user:{}},e)))&&void 0!==t?t:[];return{ids:f.map(e=>e.id),values:f}}async save(e,t){const{serverSettings:n}=this,{baseUrl:r,appUrl:i}=n,{makeRequest:a,ResponseError:l}=o.ServerConnection,u=r+i,c=s.url(u,e),d={body:JSON.stringify({raw:t}),method:"PUT"},h=await a(c,d,n);if(204!==h.status)throw new l(h)}}var s;t.SettingManager=a,function(e){e.url=function(e,t){return r.URLExt.join(e,"api/settings",t)}}(s||(s={}))},function(e,t,n){"use strict";var r=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),i=this&&this.__setModuleDefault||(Object.create?function(e,t){Object.defineProperty(e,"default",{enumerable:!0,value:t})}:function(e,t){e.default=t}),o=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)"default"!==n&&Object.prototype.hasOwnProperty.call(e,n)&&r(t,e,n);return i(t,e),t},a=this&&this.__exportStar||function(e,t){for(var n in e)"default"===n||Object.prototype.hasOwnProperty.call(t,n)||r(t,e,n)};Object.defineProperty(t,"__esModule",{value:!0}),t.TerminalAPI=t.Terminal=void 0;const s=o(n(1378));t.Terminal=s;const l=o(n(355));t.TerminalAPI=l,a(n(1379),t)},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.WorkspaceManager=void 0;const r=n(48),i=n(263),o=n(129);class a extends i.DataConnector{constructor(e={}){var t;super(),this.serverSettings=null!==(t=e.serverSettings)&&void 0!==t?t:o.ServerConnection.makeSettings()}async fetch(e){const{serverSettings:t}=this,{baseUrl:n,appUrl:r}=t,{makeRequest:i,ResponseError:a}=o.ServerConnection,l=n+r,u=s.url(l,e),c=await i(u,{},t);if(200!==c.status){throw await a.create(c)}return c.json()}async list(){const{serverSettings:e}=this,{baseUrl:t,appUrl:n}=e,{makeRequest:r,ResponseError:i}=o.ServerConnection,a=t+n,l=s.url(a,""),u=await r(l,{},e);if(200!==u.status){throw await i.create(u)}return(await u.json()).workspaces}async remove(e){const{serverSettings:t}=this,{baseUrl:n,appUrl:r}=t,{makeRequest:i,ResponseError:a}=o.ServerConnection,l=n+r,u=s.url(l,e),c=await i(u,{method:"DELETE"},t);if(204!==c.status){throw await a.create(c)}}async save(e,t){const{serverSettings:n}=this,{baseUrl:r,appUrl:i}=n,{makeRequest:a,ResponseError:l}=o.ServerConnection,u=r+i,c=s.url(u,e),d={body:JSON.stringify(t),method:"PUT"},h=await a(c,d,n);if(204!==h.status){throw await l.create(h)}}}var s;t.WorkspaceManager=a,function(e){e.url=function(e,t){return r.URLExt.join(e,"api/workspaces",t)}}(s||(s={}))},function(e,t,n){var r;void 0===(r=function(e,t){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(){const e=window.location.href.split("?")[0].split("/module/");let t=e[0],n=null;return e.length>1&&(n=e[1].replace("/","")),t.endsWith("/")||(t+="/"),{BASEURL:t,COSAPP_MODULE:n}}}.apply(t,[n,t]))||(e.exports=r)},function(e,t,n){(function(t){var n="object"==typeof t&&t&&t.Object===Object&&t;e.exports=n}).call(this,n(61))},function(e,t){var n=Function.prototype.toString;e.exports=function(e){if(null!=e){try{return n.call(e)}catch(e){}try{return e+""}catch(e){}}return""}},function(e,t,n){var r=n(853),i=n(1413),o=n(854);e.exports=function(e,t,n,a,s,l){var u=1&n,c=e.length,d=t.length;if(c!=d&&!(u&&d>c))return!1;var h=l.get(e);if(h&&l.get(t))return h==t;var f=-1,p=!0,g=2&n?new r:void 0;for(l.set(e,t),l.set(t,e);++f<c;){var v=e[f],m=t[f];if(a)var y=u?a(m,v,f,t,e,l):a(v,m,f,e,t,l);if(void 0!==y){if(y)continue;p=!1;break}if(g){if(!i(t,(function(e,t){if(!o(g,t)&&(v===e||s(v,e,n,a,l)))return g.push(t)}))){p=!1;break}}else if(v!==m&&!s(v,m,n,a,l)){p=!1;break}}return l.delete(e),l.delete(t),p}},function(e,t,n){var r=n(493),i=n(1411),o=n(1412);function a(e){var t=-1,n=null==e?0:e.length;for(this.__data__=new r;++t<n;)this.add(e[t])}a.prototype.add=a.prototype.push=i,a.prototype.has=o,e.exports=a},function(e,t){e.exports=function(e,t){return e.has(t)}},function(e,t,n){var r=n(146).Uint8Array;e.exports=r},function(e,t,n){var r=n(857),i=n(496),o=n(189);e.exports=function(e){return r(e,o,i)}},function(e,t,n){var r=n(495),i=n(81);e.exports=function(e,t,n){var o=t(e);return i(e)?o:r(o,n(e))}},function(e,t){e.exports=function(e,t){for(var n=-1,r=null==e?0:e.length,i=0,o=[];++n<r;){var a=e[n];t(a,n,e)&&(o[i++]=a)}return o}},function(e,t){e.exports=function(){return[]}},function(e,t,n){var r=n(1417),i=n(282),o=n(81),a=n(237),s=n(361),l=n(283),u=Object.prototype.hasOwnProperty;e.exports=function(e,t){var n=o(e),c=!n&&i(e),d=!n&&!c&&a(e),h=!n&&!c&&!d&&l(e),f=n||c||d||h,p=f?r(e.length,String):[],g=p.length;for(var v in e)!t&&!u.call(e,v)||f&&("length"==v||d&&("offset"==v||"parent"==v)||h&&("buffer"==v||"byteLength"==v||"byteOffset"==v)||s(v,g))||p.push(v);return p}},function(e,t){e.exports=function(e,t){return function(n){return e(t(n))}}},function(e,t,n){var r=n(207)(n(146),"Set");e.exports=r},function(e,t,n){var r,i,o;
/*!
 * jQuery UI Widget 1.12.1
 * http://jqueryui.com
 *
 * Copyright jQuery Foundation and other contributors
 * Released under the MIT license.
 * http://jquery.org/license
 */i=[n(71),n(284)],void 0===(o="function"==typeof(r=function(e){var t,n=0,r=Array.prototype.slice;return e.cleanData=(t=e.cleanData,function(n){var r,i,o;for(o=0;null!=(i=n[o]);o++)try{(r=e._data(i,"events"))&&r.remove&&e(i).triggerHandler("remove")}catch(e){}t(n)}),e.widget=function(t,n,r){var i,o,a,s={},l=t.split(".")[0],u=l+"-"+(t=t.split(".")[1]);return r||(r=n,n=e.Widget),e.isArray(r)&&(r=e.extend.apply(null,[{}].concat(r))),e.expr[":"][u.toLowerCase()]=function(t){return!!e.data(t,u)},e[l]=e[l]||{},i=e[l][t],o=e[l][t]=function(e,t){if(!this._createWidget)return new o(e,t);arguments.length&&this._createWidget(e,t)},e.extend(o,i,{version:r.version,_proto:e.extend({},r),_childConstructors:[]}),(a=new n).options=e.widget.extend({},a.options),e.each(r,(function(t,r){e.isFunction(r)?s[t]=function(){function e(){return n.prototype[t].apply(this,arguments)}function i(e){return n.prototype[t].apply(this,e)}return function(){var t,n=this._super,o=this._superApply;return this._super=e,this._superApply=i,t=r.apply(this,arguments),this._super=n,this._superApply=o,t}}():s[t]=r})),o.prototype=e.widget.extend(a,{widgetEventPrefix:i&&a.widgetEventPrefix||t},s,{constructor:o,namespace:l,widgetName:t,widgetFullName:u}),i?(e.each(i._childConstructors,(function(t,n){var r=n.prototype;e.widget(r.namespace+"."+r.widgetName,o,n._proto)})),delete i._childConstructors):n._childConstructors.push(o),e.widget.bridge(t,o),o},e.widget.extend=function(t){for(var n,i,o=r.call(arguments,1),a=0,s=o.length;a<s;a++)for(n in o[a])i=o[a][n],o[a].hasOwnProperty(n)&&void 0!==i&&(e.isPlainObject(i)?t[n]=e.isPlainObject(t[n])?e.widget.extend({},t[n],i):e.widget.extend({},i):t[n]=i);return t},e.widget.bridge=function(t,n){var i=n.prototype.widgetFullName||t;e.fn[t]=function(o){var a="string"==typeof o,s=r.call(arguments,1),l=this;return a?this.length||"instance"!==o?this.each((function(){var n,r=e.data(this,i);return"instance"===o?(l=r,!1):r?e.isFunction(r[o])&&"_"!==o.charAt(0)?(n=r[o].apply(r,s))!==r&&void 0!==n?(l=n&&n.jquery?l.pushStack(n.get()):n,!1):void 0:e.error("no such method '"+o+"' for "+t+" widget instance"):e.error("cannot call methods on "+t+" prior to initialization; attempted to call method '"+o+"'")})):l=void 0:(s.length&&(o=e.widget.extend.apply(null,[o].concat(s))),this.each((function(){var t=e.data(this,i);t?(t.option(o||{}),t._init&&t._init()):e.data(this,i,new n(o,this))}))),l}},e.Widget=function(){},e.Widget._childConstructors=[],e.Widget.prototype={widgetName:"widget",widgetEventPrefix:"",defaultElement:"<div>",options:{classes:{},disabled:!1,create:null},_createWidget:function(t,r){r=e(r||this.defaultElement||this)[0],this.element=e(r),this.uuid=n++,this.eventNamespace="."+this.widgetName+this.uuid,this.bindings=e(),this.hoverable=e(),this.focusable=e(),this.classesElementLookup={},r!==this&&(e.data(r,this.widgetFullName,this),this._on(!0,this.element,{remove:function(e){e.target===r&&this.destroy()}}),this.document=e(r.style?r.ownerDocument:r.document||r),this.window=e(this.document[0].defaultView||this.document[0].parentWindow)),this.options=e.widget.extend({},this.options,this._getCreateOptions(),t),this._create(),this.options.disabled&&this._setOptionDisabled(this.options.disabled),this._trigger("create",null,this._getCreateEventData()),this._init()},_getCreateOptions:function(){return{}},_getCreateEventData:e.noop,_create:e.noop,_init:e.noop,destroy:function(){var t=this;this._destroy(),e.each(this.classesElementLookup,(function(e,n){t._removeClass(n,e)})),this.element.off(this.eventNamespace).removeData(this.widgetFullName),this.widget().off(this.eventNamespace).removeAttr("aria-disabled"),this.bindings.off(this.eventNamespace)},_destroy:e.noop,widget:function(){return this.element},option:function(t,n){var r,i,o,a=t;if(0===arguments.length)return e.widget.extend({},this.options);if("string"==typeof t)if(a={},r=t.split("."),t=r.shift(),r.length){for(i=a[t]=e.widget.extend({},this.options[t]),o=0;o<r.length-1;o++)i[r[o]]=i[r[o]]||{},i=i[r[o]];if(t=r.pop(),1===arguments.length)return void 0===i[t]?null:i[t];i[t]=n}else{if(1===arguments.length)return void 0===this.options[t]?null:this.options[t];a[t]=n}return this._setOptions(a),this},_setOptions:function(e){var t;for(t in e)this._setOption(t,e[t]);return this},_setOption:function(e,t){return"classes"===e&&this._setOptionClasses(t),this.options[e]=t,"disabled"===e&&this._setOptionDisabled(t),this},_setOptionClasses:function(t){var n,r,i;for(n in t)i=this.classesElementLookup[n],t[n]!==this.options.classes[n]&&i&&i.length&&(r=e(i.get()),this._removeClass(i,n),r.addClass(this._classes({element:r,keys:n,classes:t,add:!0})))},_setOptionDisabled:function(e){this._toggleClass(this.widget(),this.widgetFullName+"-disabled",null,!!e),e&&(this._removeClass(this.hoverable,null,"ui-state-hover"),this._removeClass(this.focusable,null,"ui-state-focus"))},enable:function(){return this._setOptions({disabled:!1})},disable:function(){return this._setOptions({disabled:!0})},_classes:function(t){var n=[],r=this;function i(i,o){var a,s;for(s=0;s<i.length;s++)a=r.classesElementLookup[i[s]]||e(),a=t.add?e(e.unique(a.get().concat(t.element.get()))):e(a.not(t.element).get()),r.classesElementLookup[i[s]]=a,n.push(i[s]),o&&t.classes[i[s]]&&n.push(t.classes[i[s]])}return t=e.extend({element:this.element,classes:this.options.classes||{}},t),this._on(t.element,{remove:"_untrackClassesElement"}),t.keys&&i(t.keys.match(/\S+/g)||[],!0),t.extra&&i(t.extra.match(/\S+/g)||[]),n.join(" ")},_untrackClassesElement:function(t){var n=this;e.each(n.classesElementLookup,(function(r,i){-1!==e.inArray(t.target,i)&&(n.classesElementLookup[r]=e(i.not(t.target).get()))}))},_removeClass:function(e,t,n){return this._toggleClass(e,t,n,!1)},_addClass:function(e,t,n){return this._toggleClass(e,t,n,!0)},_toggleClass:function(e,t,n,r){r="boolean"==typeof r?r:n;var i="string"==typeof e||null===e,o={extra:i?t:n,keys:i?e:t,element:i?this.element:e,add:r};return o.element.toggleClass(this._classes(o),r),this},_on:function(t,n,r){var i,o=this;"boolean"!=typeof t&&(r=n,n=t,t=!1),r?(n=i=e(n),this.bindings=this.bindings.add(n)):(r=n,n=this.element,i=this.widget()),e.each(r,(function(r,a){function s(){if(t||!0!==o.options.disabled&&!e(this).hasClass("ui-state-disabled"))return("string"==typeof a?o[a]:a).apply(o,arguments)}"string"!=typeof a&&(s.guid=a.guid=a.guid||s.guid||e.guid++);var l=r.match(/^([\w:-]*)\s*(.*)$/),u=l[1]+o.eventNamespace,c=l[2];c?i.on(u,c,s):n.on(u,s)}))},_off:function(t,n){n=(n||"").split(" ").join(this.eventNamespace+" ")+this.eventNamespace,t.off(n).off(n),this.bindings=e(this.bindings.not(t).get()),this.focusable=e(this.focusable.not(t).get()),this.hoverable=e(this.hoverable.not(t).get())},_delay:function(e,t){var n=this;return setTimeout((function(){return("string"==typeof e?n[e]:e).apply(n,arguments)}),t||0)},_hoverable:function(t){this.hoverable=this.hoverable.add(t),this._on(t,{mouseenter:function(t){this._addClass(e(t.currentTarget),null,"ui-state-hover")},mouseleave:function(t){this._removeClass(e(t.currentTarget),null,"ui-state-hover")}})},_focusable:function(t){this.focusable=this.focusable.add(t),this._on(t,{focusin:function(t){this._addClass(e(t.currentTarget),null,"ui-state-focus")},focusout:function(t){this._removeClass(e(t.currentTarget),null,"ui-state-focus")}})},_trigger:function(t,n,r){var i,o,a=this.options[t];if(r=r||{},(n=e.Event(n)).type=(t===this.widgetEventPrefix?t:this.widgetEventPrefix+t).toLowerCase(),n.target=this.element[0],o=n.originalEvent)for(i in o)i in n||(n[i]=o[i]);return this.element.trigger(n,r),!(e.isFunction(a)&&!1===a.apply(this.element[0],[n].concat(r))||n.isDefaultPrevented())}},e.each({show:"fadeIn",hide:"fadeOut"},(function(t,n){e.Widget.prototype["_"+t]=function(r,i,o){var a;"string"==typeof i&&(i={effect:i});var s=i?!0===i||"number"==typeof i?n:i.effect||n:t;"number"==typeof(i=i||{})&&(i={duration:i}),a=!e.isEmptyObject(i),i.complete=o,i.delay&&r.delay(i.delay),a&&e.effects&&e.effects.effect[s]?r[t](i):s!==t&&r[s]?r[s](i.duration,i.easing,o):r.queue((function(n){e(this)[t](),o&&o.call(r[0]),n()}))}})),e.widget})?r.apply(t,i):r)||(e.exports=o)},function(e,t,n){"use strict";n.d(t,"a",(function(){return l}));var r,i=n(17),o=n(16),a=n(119),s=n(463);class l{constructor(e){this._changed=new o.Signal(this),this._raw={};const{data:t,metadata:n,trusted:i}=r.getBundleOptions(e);this._data=new s.a({values:t}),this._rawData=t,this._metadata=new s.a({values:n}),this._rawMetadata=n,this.trusted=i;const l=e.value;for(const e in l)switch(e){case"data":case"metadata":break;default:this._raw[e]=r.extract(l,e)}this.type=l.output_type,a.d(l)?this.executionCount=l.execution_count:this.executionCount=null}get changed(){return this._changed}dispose(){this._data.dispose(),this._metadata.dispose(),o.Signal.clearData(this)}get data(){return this._rawData}get metadata(){return this._rawMetadata}setData(e){e.data&&(this._updateObservable(this._data,e.data),this._rawData=e.data),e.metadata&&(this._updateObservable(this._metadata,e.metadata),this._rawMetadata=e.metadata),this._changed.emit(void 0)}toJSON(){const e={};for(const t in this._raw)e[t]=r.extract(this._raw,t);switch(this.type){case"display_data":case"execute_result":case"update_display_data":e.data=this.data,e.metadata=this.metadata}return delete e.transient,e}_updateObservable(e,t){const n=e.keys(),r=Object.keys(t);for(const t of n)-1===r.indexOf(t)&&e.delete(t);for(const n of r){const r=e.get(n),i=t[n];r!==i&&e.set(n,i)}}}!function(e){e.getData=function(e){return r.getData(e)},e.getMetadata=function(e){return r.getMetadata(e)}}(l||(l={})),function(e){function t(e){let t={};if(a.d(e)||a.a(e)||a.b(e))t=e.data;else if(a.e(e))"stderr"===e.name?t["application/vnd.jupyter.stderr"]=e.text:t["application/vnd.jupyter.stdout"]=e.text;else if(a.c(e)){t["application/vnd.jupyter.error"]=e;const n=e.traceback.join("\n");t["application/vnd.jupyter.stderr"]=n||`${e.ename}: ${e.evalue}`}return function(e){const t=Object.create(null);for(const n in e)t[n]=r(e,n);return t}(t)}function n(e){const t=Object.create(null);if(a.d(e)||a.a(e))for(const n in e.metadata)t[n]=r(e.metadata,n);return t}function r(e,t){const n=e[t];return void 0===n||i.JSONExt.isPrimitive(n)?n:JSON.parse(JSON.stringify(n))}e.getData=t,e.getMetadata=n,e.getBundleOptions=function(e){return{data:t(e.value),metadata:n(e.value),trusted:!!e.trusted}},e.extract=r}(r||(r={}))},function(e,t,n){"use strict";n.d(t,"a",(function(){return l}));var r,i=n(48),o=n(624),a=n(51),s=n(433);class l{constructor(e={}){if(this._id=0,this._ranks={},this._types=null,this._factories={},this.translator=e.translator||a.a,this.resolver=e.resolver||null,this.linkHandler=e.linkHandler||null,this.latexTypesetter=e.latexTypesetter||null,this.sanitizer=e.sanitizer||o.a,e.initialFactories)for(const t of e.initialFactories)this.addFactory(t)}get mimeTypes(){return this._types||(this._types=r.sortedTypes(this._ranks))}preferredMimeType(e,t="ensure"){if("ensure"===t||"prefer"===t)for(const t of this.mimeTypes)if(t in e&&this._factories[t].safe)return t;if("ensure"!==t)for(const t of this.mimeTypes)if(t in e)return t}createRenderer(e){if(!(e in this._factories))throw new Error(`No factory for mime type: '${e}'`);return this._factories[e].createRenderer({mimeType:e,resolver:this.resolver,sanitizer:this.sanitizer,linkHandler:this.linkHandler,latexTypesetter:this.latexTypesetter,translator:this.translator})}createModel(e={}){return new s.a(e)}clone(e={}){const t=new l({resolver:e.resolver||this.resolver||void 0,sanitizer:e.sanitizer||this.sanitizer||void 0,linkHandler:e.linkHandler||this.linkHandler||void 0,latexTypesetter:e.latexTypesetter||this.latexTypesetter||void 0,translator:this.translator});return t._factories=Object.assign({},this._factories),t._ranks=Object.assign({},this._ranks),t._id=this._id,t}getFactory(e){return this._factories[e]}addFactory(e,t){void 0===t&&void 0===(t=e.defaultRank)&&(t=100);for(const n of e.mimeTypes)this._factories[n]=e,this._ranks[n]={rank:t,id:this._id++};this._types=null}removeMimeType(e){delete this._factories[e],delete this._ranks[e],this._types=null}getRank(e){const t=this._ranks[e];return t&&t.rank}setRank(e,t){if(!this._ranks[e])return;const n=this._id++;this._ranks[e]={rank:t,id:n},this._types=null}}!function(e){e.UrlResolver=class{constructor(e){if(e.path)this._path=e.path;else{if(!e.session)throw new Error("Either 'path' or 'session' must be given as a constructor option");this._session=e.session}this._contents=e.contents}get path(){var e;return null!==(e=this._path)&&void 0!==e?e:this._session.path}set path(e){this._path=e}async resolveUrl(e){if(this.isLocal(e)){const t=encodeURI(i.PathExt.dirname(this.path));e=i.PathExt.resolve(t,e)}return e}async getDownloadUrl(e){return this.isLocal(e)?this._contents.getDownloadUrl(decodeURI(e)):e}isLocal(e){const t=decodeURI(e);return i.URLExt.isLocal(e)||!!this._contents.driveName(t)}}}(l||(l={})),function(e){e.sortedTypes=function(e){return Object.keys(e).sort((t,n)=>{const r=e[t],i=e[n];return r.rank!==i.rank?r.rank-i.rank:r.id-i.id})},e.sessionConnection=function(e){return e.sessionChanged?e.session:e}}(r||(r={}))},function(e,t,n){"use strict";n.d(t,"a",(function(){return i})),n.d(t,"b",(function(){return o})),n.d(t,"d",(function(){return a})),n.d(t,"e",(function(){return s})),n.d(t,"g",(function(){return l})),n.d(t,"h",(function(){return u})),n.d(t,"c",(function(){return c})),n.d(t,"f",(function(){return d}));var r=n(171);const i={safe:!0,mimeTypes:["text/html"],defaultRank:50,createRenderer:e=>new r.b(e)},o={safe:!0,mimeTypes:["image/bmp","image/png","image/jpeg","image/gif"],defaultRank:90,createRenderer:e=>new r.d(e)},a={safe:!0,mimeTypes:["text/latex"],defaultRank:70,createRenderer:e=>new r.f(e)},s={safe:!0,mimeTypes:["text/markdown"],defaultRank:60,createRenderer:e=>new r.g(e)},l={safe:!1,mimeTypes:["image/svg+xml"],defaultRank:80,createRenderer:e=>new r.h(e)},u={safe:!0,mimeTypes:["text/plain","application/vnd.jupyter.stdout","application/vnd.jupyter.stderr"],defaultRank:120,createRenderer:e=>new r.i(e)},c={safe:!1,mimeTypes:["text/javascript","application/javascript"],defaultRank:110,createRenderer:e=>new r.e(e)},d=[i,s,a,l,o,c,u]},function(e,t,n){!function(e){var t=/MSIE \d/.test(navigator.userAgent)&&(null==document.documentMode||document.documentMode<8),n=e.Pos,r={"(":")>",")":"(<","[":"]>","]":"[<","{":"}>","}":"{<","<":">>",">":"<<"};function i(e){return e&&e.bracketRegex||/[(){}[\]]/}function o(e,t,o){var s=e.getLineHandle(t.line),l=t.ch-1,u=o&&o.afterCursor;null==u&&(u=/(^| )cm-fat-cursor($| )/.test(e.getWrapperElement().className));var c=i(o),d=!u&&l>=0&&c.test(s.text.charAt(l))&&r[s.text.charAt(l)]||c.test(s.text.charAt(l+1))&&r[s.text.charAt(++l)];if(!d)return null;var h=">"==d.charAt(1)?1:-1;if(o&&o.strict&&h>0!=(l==t.ch))return null;var f=e.getTokenTypeAt(n(t.line,l+1)),p=a(e,n(t.line,l+(h>0?1:0)),h,f||null,o);return null==p?null:{from:n(t.line,l),to:p&&p.pos,match:p&&p.ch==d.charAt(0),forward:h>0}}function a(e,t,o,a,s){for(var l=s&&s.maxScanLineLength||1e4,u=s&&s.maxScanLines||1e3,c=[],d=i(s),h=o>0?Math.min(t.line+u,e.lastLine()+1):Math.max(e.firstLine()-1,t.line-u),f=t.line;f!=h;f+=o){var p=e.getLine(f);if(p){var g=o>0?0:p.length-1,v=o>0?p.length:-1;if(!(p.length>l))for(f==t.line&&(g=t.ch-(o<0?1:0));g!=v;g+=o){var m=p.charAt(g);if(d.test(m)&&(void 0===a||e.getTokenTypeAt(n(f,g+1))==a)){var y=r[m];if(y&&">"==y.charAt(1)==o>0)c.push(m);else{if(!c.length)return{pos:n(f,g),ch:m};c.pop()}}}}}return f-o!=(o>0?e.lastLine():e.firstLine())&&null}function s(e,r,i){for(var a=e.state.matchBrackets.maxHighlightLineLength||1e3,s=[],l=e.listSelections(),u=0;u<l.length;u++){var c=l[u].empty()&&o(e,l[u].head,i);if(c&&e.getLine(c.from.line).length<=a){var d=c.match?"CodeMirror-matchingbracket":"CodeMirror-nonmatchingbracket";s.push(e.markText(c.from,n(c.from.line,c.from.ch+1),{className:d})),c.to&&e.getLine(c.to.line).length<=a&&s.push(e.markText(c.to,n(c.to.line,c.to.ch+1),{className:d}))}}if(s.length){t&&e.state.focused&&e.focus();var h=function(){e.operation((function(){for(var e=0;e<s.length;e++)s[e].clear()}))};if(!r)return h;setTimeout(h,800)}}function l(e){e.operation((function(){e.state.matchBrackets.currentlyHighlighted&&(e.state.matchBrackets.currentlyHighlighted(),e.state.matchBrackets.currentlyHighlighted=null),e.state.matchBrackets.currentlyHighlighted=s(e,!1,e.state.matchBrackets)}))}e.defineOption("matchBrackets",!1,(function(t,n,r){function i(e){e.state.matchBrackets&&e.state.matchBrackets.currentlyHighlighted&&(e.state.matchBrackets.currentlyHighlighted(),e.state.matchBrackets.currentlyHighlighted=null)}r&&r!=e.Init&&(t.off("cursorActivity",l),t.off("focus",l),t.off("blur",i),i(t)),n&&(t.state.matchBrackets="object"==typeof n?n:{},t.on("cursorActivity",l),t.on("focus",l),t.on("blur",i))})),e.defineExtension("matchBrackets",(function(){s(this,!0)})),e.defineExtension("findMatchingBracket",(function(e,t,n){return(n||"boolean"==typeof t)&&(n?(n.strict=t,t=n):t=t?{strict:!0}:null),o(this,e,t)})),e.defineExtension("scanForBracket",(function(e,t,n,r){return a(this,e,t,n,r)}))}(n(24))},function(e,t,n){!function(e){"use strict";function t(t,n,i,o){if(i&&i.call){var a=i;i=null}else a=r(t,i,"rangeFinder");"number"==typeof n&&(n=e.Pos(n,0));var s=r(t,i,"minFoldSize");function l(e){var r=a(t,n);if(!r||r.to.line-r.from.line<s)return null;for(var i=t.findMarksAt(r.from),l=0;l<i.length;++l)if(i[l].__isFold&&"fold"!==o){if(!e)return null;r.cleared=!0,i[l].clear()}return r}var u=l(!0);if(r(t,i,"scanUp"))for(;!u&&n.line>t.firstLine();)n=e.Pos(n.line-1,0),u=l(!1);if(u&&!u.cleared&&"unfold"!==o){var c=function(e,t,n){var i=r(e,t,"widget");if("function"==typeof i&&(i=i(n.from,n.to)),"string"==typeof i){var o=document.createTextNode(i);(i=document.createElement("span")).appendChild(o),i.className="CodeMirror-foldmarker"}else i&&(i=i.cloneNode(!0));return i}(t,i,u);e.on(c,"mousedown",(function(t){d.clear(),e.e_preventDefault(t)}));var d=t.markText(u.from,u.to,{replacedWith:c,clearOnEnter:r(t,i,"clearOnEnter"),__isFold:!0});d.on("clear",(function(n,r){e.signal(t,"unfold",t,n,r)})),e.signal(t,"fold",t,u.from,u.to)}}e.newFoldFunction=function(e,n){return function(r,i){t(r,i,{rangeFinder:e,widget:n})}},e.defineExtension("foldCode",(function(e,n,r){t(this,e,n,r)})),e.defineExtension("isFolded",(function(e){for(var t=this.findMarksAt(e),n=0;n<t.length;++n)if(t[n].__isFold)return!0})),e.commands.toggleFold=function(e){e.foldCode(e.getCursor())},e.commands.fold=function(e){e.foldCode(e.getCursor(),null,"fold")},e.commands.unfold=function(e){e.foldCode(e.getCursor(),null,"unfold")},e.commands.foldAll=function(t){t.operation((function(){for(var n=t.firstLine(),r=t.lastLine();n<=r;n++)t.foldCode(e.Pos(n,0),null,"fold")}))},e.commands.unfoldAll=function(t){t.operation((function(){for(var n=t.firstLine(),r=t.lastLine();n<=r;n++)t.foldCode(e.Pos(n,0),null,"unfold")}))},e.registerHelper("fold","combine",(function(){var e=Array.prototype.slice.call(arguments,0);return function(t,n){for(var r=0;r<e.length;++r){var i=e[r](t,n);if(i)return i}}})),e.registerHelper("fold","auto",(function(e,t){for(var n=e.getHelpers(t,"fold"),r=0;r<n.length;r++){var i=n[r](e,t);if(i)return i}}));var n={rangeFinder:e.fold.auto,widget:"↔",minFoldSize:0,scanUp:!1,clearOnEnter:!0};function r(e,t,r){if(t&&void 0!==t[r])return t[r];var i=e.options.foldOptions;return i&&void 0!==i[r]?i[r]:n[r]}e.defineOption("foldOptions",null),e.defineExtension("foldOption",(function(e,t){return r(this,e,t)}))}(n(24))},function(e,t,n){!function(e){function t(t,n,r){var i,o=t.getWrapperElement();return(i=o.appendChild(document.createElement("div"))).className=r?"CodeMirror-dialog CodeMirror-dialog-bottom":"CodeMirror-dialog CodeMirror-dialog-top","string"==typeof n?i.innerHTML=n:i.appendChild(n),e.addClass(o,"dialog-opened"),i}function n(e,t){e.state.currentNotificationClose&&e.state.currentNotificationClose(),e.state.currentNotificationClose=t}e.defineExtension("openDialog",(function(r,i,o){o||(o={}),n(this,null);var a=t(this,r,o.bottom),s=!1,l=this;function u(t){if("string"==typeof t)d.value=t;else{if(s)return;s=!0,e.rmClass(a.parentNode,"dialog-opened"),a.parentNode.removeChild(a),l.focus(),o.onClose&&o.onClose(a)}}var c,d=a.getElementsByTagName("input")[0];return d?(d.focus(),o.value&&(d.value=o.value,!1!==o.selectValueOnOpen&&d.select()),o.onInput&&e.on(d,"input",(function(e){o.onInput(e,d.value,u)})),o.onKeyUp&&e.on(d,"keyup",(function(e){o.onKeyUp(e,d.value,u)})),e.on(d,"keydown",(function(t){o&&o.onKeyDown&&o.onKeyDown(t,d.value,u)||((27==t.keyCode||!1!==o.closeOnEnter&&13==t.keyCode)&&(d.blur(),e.e_stop(t),u()),13==t.keyCode&&i(d.value,t))})),!1!==o.closeOnBlur&&e.on(a,"focusout",(function(e){null!==e.relatedTarget&&u()}))):(c=a.getElementsByTagName("button")[0])&&(e.on(c,"click",(function(){u(),l.focus()})),!1!==o.closeOnBlur&&e.on(c,"blur",u),c.focus()),u})),e.defineExtension("openConfirm",(function(r,i,o){n(this,null);var a=t(this,r,o&&o.bottom),s=a.getElementsByTagName("button"),l=!1,u=this,c=1;function d(){l||(l=!0,e.rmClass(a.parentNode,"dialog-opened"),a.parentNode.removeChild(a),u.focus())}s[0].focus();for(var h=0;h<s.length;++h){var f=s[h];!function(t){e.on(f,"click",(function(n){e.e_preventDefault(n),d(),t&&t(u)}))}(i[h]),e.on(f,"blur",(function(){--c,setTimeout((function(){c<=0&&d()}),200)})),e.on(f,"focus",(function(){++c}))}})),e.defineExtension("openNotification",(function(r,i){n(this,u);var o,a=t(this,r,i&&i.bottom),s=!1,l=i&&void 0!==i.duration?i.duration:5e3;function u(){s||(s=!0,clearTimeout(o),e.rmClass(a.parentNode,"dialog-opened"),a.parentNode.removeChild(a))}return e.on(a,"click",(function(t){e.e_preventDefault(t),u()})),l&&(o=setTimeout(u,l)),u}))}(n(24))},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.Alignment=void 0,t.Alignment={CENTER:"center",LEFT:"left",RIGHT:"right"}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.Elevation=void 0,t.Elevation={ZERO:0,ONE:1,TWO:2,THREE:3,FOUR:4}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.getPositionIgnoreAngles=t.isPositionVertical=t.isPositionHorizontal=t.Position=void 0,t.Position={BOTTOM:"bottom",BOTTOM_LEFT:"bottom-left",BOTTOM_RIGHT:"bottom-right",LEFT:"left",LEFT_BOTTOM:"left-bottom",LEFT_TOP:"left-top",RIGHT:"right",RIGHT_BOTTOM:"right-bottom",RIGHT_TOP:"right-top",TOP:"top",TOP_LEFT:"top-left",TOP_RIGHT:"top-right"},t.isPositionHorizontal=function(e){return e===t.Position.TOP||e===t.Position.TOP_LEFT||e===t.Position.TOP_RIGHT||e===t.Position.BOTTOM||e===t.Position.BOTTOM_LEFT||e===t.Position.BOTTOM_RIGHT},t.isPositionVertical=function(e){return e===t.Position.LEFT||e===t.Position.LEFT_TOP||e===t.Position.LEFT_BOTTOM||e===t.Position.RIGHT||e===t.Position.RIGHT_TOP||e===t.Position.RIGHT_BOTTOM},t.getPositionIgnoreAngles=function(e){return e===t.Position.TOP||e===t.Position.TOP_LEFT||e===t.Position.TOP_RIGHT?t.Position.TOP:e===t.Position.BOTTOM||e===t.Position.BOTTOM_LEFT||e===t.Position.BOTTOM_RIGHT?t.Position.BOTTOM:e===t.Position.LEFT||e===t.Position.LEFT_TOP||e===t.Position.LEFT_BOTTOM?t.Position.LEFT:t.Position.RIGHT}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.Icon=void 0;var r=n(2),i=r.__importDefault(n(11)),o=r.__importStar(n(0)),a=n(13),s=n(1195),l=n(239),u=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}var n;return r.__extends(t,e),n=t,t.prototype.render=function(){var e=this.props.icon;if(null==e||"boolean"==typeof e)return null;if("string"!=typeof e)return e;var t=this.props,a=t.className,s=t.color,u=t.htmlTitle,c=t.iconSize,d=void 0===c?n.SIZE_STANDARD:c,h=t.intent,f=t.title,p=void 0===f?e:f,g=t.tagName,v=void 0===g?"span":g,m=r.__rest(t,["className","color","htmlTitle","iconSize","intent","title","tagName"]),y=d>=n.SIZE_LARGE?n.SIZE_LARGE:n.SIZE_STANDARD,b=this.renderSvgPaths(y,e),C=i.default(l.Classes.ICON,l.Classes.iconClass(e),l.Classes.intentClass(h),a),A="0 0 "+y+" "+y;return o.createElement(v,r.__assign(r.__assign({},m),{className:C,title:u}),o.createElement("svg",{fill:s,"data-icon":e,width:d,height:d,viewBox:A},p&&o.createElement("desc",null,p),b))},t.prototype.renderSvgPaths=function(e,t){var r=(e===n.SIZE_STANDARD?s.IconSvgPaths16:s.IconSvgPaths20)[t];return null==r?null:r.map((function(e,t){return o.createElement("path",{key:t,d:e,fillRule:"evenodd"})}))},t.displayName=l.DISPLAYNAME_PREFIX+".Icon",t.SIZE_STANDARD=16,t.SIZE_LARGE=20,t=n=r.__decorate([a.polyfill],t)}(l.AbstractPureComponent2);t.Icon=u},function(e,t,n){"use strict";function r(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];for(var n={},i=0,o=e;i<o.length;i++){var a=o[i];if(null!=a&&!1!==a)for(var s in a){var l=a[s];(l||0===l)&&("$nest"===s&&l?n[s]=n.$nest?r(n.$nest,l):l:-1!==s.indexOf("&")||0===s.indexOf("@media")?n[s]=n[s]?r(n[s],l):l:n[s]=l)}}return n}Object.defineProperty(t,"__esModule",{value:!0}),t.raf="undefined"==typeof requestAnimationFrame?function(e){return setTimeout(e)}:"undefined"==typeof window?requestAnimationFrame:requestAnimationFrame.bind(window),t.classes=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return e.map((function(e){return e&&"object"==typeof e?Object.keys(e).map((function(t){return!!e[t]&&t})):[e]})).reduce((function(e,t){return e.concat(t)}),[]).filter((function(e){return!!e})).join(" ")},t.extend=r,t.media=function(e){for(var t,n=[],o=1;o<arguments.length;o++)n[o-1]=arguments[o];var a=[];e.type&&a.push(e.type),e.orientation&&a.push("(orientation: "+e.orientation+")"),e.minWidth&&a.push("(min-width: "+i(e.minWidth)+")"),e.maxWidth&&a.push("(max-width: "+i(e.maxWidth)+")"),e.minHeight&&a.push("(min-height: "+i(e.minHeight)+")"),e.maxHeight&&a.push("(max-height: "+i(e.maxHeight)+")");var s="@media "+a.join(" and "),l={$nest:(t={},t[s]=r.apply(void 0,n),t)};return l};var i=function(e){return"string"==typeof e?e:e+"px"}},function(e,t,n){"use strict";(function(e){n.d(t,"e",(function(){return i})),n.d(t,"b",(function(){return o})),n.d(t,"a",(function(){return a})),n.d(t,"c",(function(){return s})),n.d(t,"d",(function(){return l}));var r=n(29);function i(t){return void 0!==e&&e.env&&"production"===t}function o(e,t){return void 0===e&&(e=[]),void 0===t&&(t=[]),e.length-t.length}function a(e,t,n){return void 0===n&&(n=1e-5),Math.abs(e-t)<=n}function s(e,t,n){if(null==e)return e;if(n<t)throw new Error(r.d);return Math.min(Math.max(e,t),n)}function l(e){if(!isFinite(e))return 0;for(var t=1,n=0;Math.round(e*t)/t!==e;)t*=10,n++;return n}}).call(this,n(145))},function(e,t,n){"use strict";"undefined"!=typeof window&&"undefined"!=typeof document&&n(1475)},function(e,t){function n(t,r){return e.exports=n=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e},n(t,r)}e.exports=n},function(e,t,n){"use strict";t.__esModule=!0,t.default=t.EXITING=t.ENTERED=t.ENTERING=t.EXITED=t.UNMOUNTED=void 0;var r=function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)if(Object.prototype.hasOwnProperty.call(e,n)){var r=Object.defineProperty&&Object.getOwnPropertyDescriptor?Object.getOwnPropertyDescriptor(e,n):{};r.get||r.set?Object.defineProperty(t,n,r):t[n]=e[n]}return t.default=e,t}(n(3)),i=s(n(0)),o=s(n(22)),a=n(13);n(879);function s(e){return e&&e.__esModule?e:{default:e}}t.UNMOUNTED="unmounted";t.EXITED="exited";t.ENTERING="entering";t.ENTERED="entered";t.EXITING="exiting";var l=function(e){var t,n;function r(t,n){var r;r=e.call(this,t,n)||this;var i,o=n.transitionGroup,a=o&&!o.isMounting?t.enter:t.appear;return r.appearStatus=null,t.in?a?(i="exited",r.appearStatus="entering"):i="entered":i=t.unmountOnExit||t.mountOnEnter?"unmounted":"exited",r.state={status:i},r.nextCallback=null,r}n=e,(t=r).prototype=Object.create(n.prototype),t.prototype.constructor=t,t.__proto__=n;var a=r.prototype;return a.getChildContext=function(){return{transitionGroup:null}},r.getDerivedStateFromProps=function(e,t){return e.in&&"unmounted"===t.status?{status:"exited"}:null},a.componentDidMount=function(){this.updateStatus(!0,this.appearStatus)},a.componentDidUpdate=function(e){var t=null;if(e!==this.props){var n=this.state.status;this.props.in?"entering"!==n&&"entered"!==n&&(t="entering"):"entering"!==n&&"entered"!==n||(t="exiting")}this.updateStatus(!1,t)},a.componentWillUnmount=function(){this.cancelNextCallback()},a.getTimeouts=function(){var e,t,n,r=this.props.timeout;return e=t=n=r,null!=r&&"number"!=typeof r&&(e=r.exit,t=r.enter,n=void 0!==r.appear?r.appear:t),{exit:e,enter:t,appear:n}},a.updateStatus=function(e,t){if(void 0===e&&(e=!1),null!==t){this.cancelNextCallback();var n=o.default.findDOMNode(this);"entering"===t?this.performEnter(n,e):this.performExit(n)}else this.props.unmountOnExit&&"exited"===this.state.status&&this.setState({status:"unmounted"})},a.performEnter=function(e,t){var n=this,r=this.props.enter,i=this.context.transitionGroup?this.context.transitionGroup.isMounting:t,o=this.getTimeouts(),a=i?o.appear:o.enter;t||r?(this.props.onEnter(e,i),this.safeSetState({status:"entering"},(function(){n.props.onEntering(e,i),n.onTransitionEnd(e,a,(function(){n.safeSetState({status:"entered"},(function(){n.props.onEntered(e,i)}))}))}))):this.safeSetState({status:"entered"},(function(){n.props.onEntered(e)}))},a.performExit=function(e){var t=this,n=this.props.exit,r=this.getTimeouts();n?(this.props.onExit(e),this.safeSetState({status:"exiting"},(function(){t.props.onExiting(e),t.onTransitionEnd(e,r.exit,(function(){t.safeSetState({status:"exited"},(function(){t.props.onExited(e)}))}))}))):this.safeSetState({status:"exited"},(function(){t.props.onExited(e)}))},a.cancelNextCallback=function(){null!==this.nextCallback&&(this.nextCallback.cancel(),this.nextCallback=null)},a.safeSetState=function(e,t){t=this.setNextCallback(t),this.setState(e,t)},a.setNextCallback=function(e){var t=this,n=!0;return this.nextCallback=function(r){n&&(n=!1,t.nextCallback=null,e(r))},this.nextCallback.cancel=function(){n=!1},this.nextCallback},a.onTransitionEnd=function(e,t,n){this.setNextCallback(n);var r=null==t&&!this.props.addEndListener;e&&!r?(this.props.addEndListener&&this.props.addEndListener(e,this.nextCallback),null!=t&&setTimeout(this.nextCallback,t)):setTimeout(this.nextCallback,0)},a.render=function(){var e=this.state.status;if("unmounted"===e)return null;var t=this.props,n=t.children,r=function(e,t){if(null==e)return{};var n,r,i={},o=Object.keys(e);for(r=0;r<o.length;r++)n=o[r],t.indexOf(n)>=0||(i[n]=e[n]);return i}(t,["children"]);if(delete r.in,delete r.mountOnEnter,delete r.unmountOnExit,delete r.appear,delete r.enter,delete r.exit,delete r.timeout,delete r.addEndListener,delete r.onEnter,delete r.onEntering,delete r.onEntered,delete r.onExit,delete r.onExiting,delete r.onExited,"function"==typeof n)return n(e,r);var o=i.default.Children.only(n);return i.default.cloneElement(o,r)},r}(i.default.Component);function u(){}l.contextTypes={transitionGroup:r.object},l.childContextTypes={transitionGroup:function(){}},l.propTypes={},l.defaultProps={in:!1,mountOnEnter:!1,unmountOnExit:!1,appear:!1,enter:!0,exit:!0,onEnter:u,onEntering:u,onEntered:u,onExit:u,onExiting:u,onExited:u},l.UNMOUNTED=0,l.EXITED=1,l.ENTERING=2,l.ENTERED=3,l.EXITING=4;var c=(0,a.polyfill)(l);t.default=c},function(e,t,n){"use strict";t.__esModule=!0,t.classNamesShape=t.timeoutsShape=void 0;var r;(r=n(3))&&r.__esModule;t.timeoutsShape=null;t.classNamesShape=null},function(e,t,n){"use strict";t.__esModule=!0,t.default=void 0;var r=s(n(3)),i=s(n(0)),o=n(13),a=n(1485);function s(e){return e&&e.__esModule?e:{default:e}}function l(){return(l=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e}).apply(this,arguments)}function u(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}var c=Object.values||function(e){return Object.keys(e).map((function(t){return e[t]}))},d=function(e){var t,n;function r(t,n){var r,i=(r=e.call(this,t,n)||this).handleExited.bind(u(u(r)));return r.state={handleExited:i,firstRender:!0},r}n=e,(t=r).prototype=Object.create(n.prototype),t.prototype.constructor=t,t.__proto__=n;var o=r.prototype;return o.getChildContext=function(){return{transitionGroup:{isMounting:!this.appeared}}},o.componentDidMount=function(){this.appeared=!0,this.mounted=!0},o.componentWillUnmount=function(){this.mounted=!1},r.getDerivedStateFromProps=function(e,t){var n=t.children,r=t.handleExited;return{children:t.firstRender?(0,a.getInitialChildMapping)(e,r):(0,a.getNextChildMapping)(e,n,r),firstRender:!1}},o.handleExited=function(e,t){var n=(0,a.getChildMapping)(this.props.children);e.key in n||(e.props.onExited&&e.props.onExited(t),this.mounted&&this.setState((function(t){var n=l({},t.children);return delete n[e.key],{children:n}})))},o.render=function(){var e=this.props,t=e.component,n=e.childFactory,r=function(e,t){if(null==e)return{};var n,r,i={},o=Object.keys(e);for(r=0;r<o.length;r++)n=o[r],t.indexOf(n)>=0||(i[n]=e[n]);return i}(e,["component","childFactory"]),o=c(this.state.children).map(n);return delete r.appear,delete r.enter,delete r.exit,null===t?o:i.default.createElement(t,r,o)},r}(i.default.Component);d.childContextTypes={transitionGroup:r.default.object.isRequired},d.propTypes={},d.defaultProps={component:"div",childFactory:function(e){return e}};var h=(0,o.polyfill)(d);t.default=h,e.exports=t.default},function(e,t,n){"use strict";var r=Array.prototype.slice,i=n(882),o=Object.keys,a=o?function(e){return o(e)}:n(1486),s=Object.keys;a.shim=function(){Object.keys?function(){var e=Object.keys(arguments);return e&&e.length===arguments.length}(1,2)||(Object.keys=function(e){return i(e)?s(r.call(e)):s(e)}):Object.keys=a;return Object.keys||a},e.exports=a},function(e,t,n){"use strict";var r=Object.prototype.toString;e.exports=function(e){var t=r.call(e),n="[object Arguments]"===t;return n||(n="[object Array]"!==t&&null!==e&&"object"==typeof e&&"number"==typeof e.length&&e.length>=0&&"[object Function]"===r.call(e.callee)),n}},function(e,t,n){"use strict";var r=n(884),i=n(508),o=i(r("String.prototype.indexOf"));e.exports=function(e,t){var n=r(e,!!t);return"function"==typeof n&&o(e,".prototype.")>-1?i(n):n}},function(e,t,n){"use strict";var r=SyntaxError,i=Function,o=TypeError,a=function(e){try{return i('"use strict"; return ('+e+").constructor;")()}catch(e){}},s=Object.getOwnPropertyDescriptor;if(s)try{s({},"")}catch(e){s=null}var l=function(){throw new o},u=s?function(){try{return l}catch(e){try{return s(arguments,"callee").get}catch(e){return l}}}():l,c=n(885)(),d=Object.getPrototypeOf||function(e){return e.__proto__},h={},f="undefined"==typeof Uint8Array?void 0:d(Uint8Array),p={"%AggregateError%":"undefined"==typeof AggregateError?void 0:AggregateError,"%Array%":Array,"%ArrayBuffer%":"undefined"==typeof ArrayBuffer?void 0:ArrayBuffer,"%ArrayIteratorPrototype%":c?d([][Symbol.iterator]()):void 0,"%AsyncFromSyncIteratorPrototype%":void 0,"%AsyncFunction%":h,"%AsyncGenerator%":h,"%AsyncGeneratorFunction%":h,"%AsyncIteratorPrototype%":h,"%Atomics%":"undefined"==typeof Atomics?void 0:Atomics,"%BigInt%":"undefined"==typeof BigInt?void 0:BigInt,"%Boolean%":Boolean,"%DataView%":"undefined"==typeof DataView?void 0:DataView,"%Date%":Date,"%decodeURI%":decodeURI,"%decodeURIComponent%":decodeURIComponent,"%encodeURI%":encodeURI,"%encodeURIComponent%":encodeURIComponent,"%Error%":Error,"%eval%":eval,"%EvalError%":EvalError,"%Float32Array%":"undefined"==typeof Float32Array?void 0:Float32Array,"%Float64Array%":"undefined"==typeof Float64Array?void 0:Float64Array,"%FinalizationRegistry%":"undefined"==typeof FinalizationRegistry?void 0:FinalizationRegistry,"%Function%":i,"%GeneratorFunction%":h,"%Int8Array%":"undefined"==typeof Int8Array?void 0:Int8Array,"%Int16Array%":"undefined"==typeof Int16Array?void 0:Int16Array,"%Int32Array%":"undefined"==typeof Int32Array?void 0:Int32Array,"%isFinite%":isFinite,"%isNaN%":isNaN,"%IteratorPrototype%":c?d(d([][Symbol.iterator]())):void 0,"%JSON%":"object"==typeof JSON?JSON:void 0,"%Map%":"undefined"==typeof Map?void 0:Map,"%MapIteratorPrototype%":"undefined"!=typeof Map&&c?d((new Map)[Symbol.iterator]()):void 0,"%Math%":Math,"%Number%":Number,"%Object%":Object,"%parseFloat%":parseFloat,"%parseInt%":parseInt,"%Promise%":"undefined"==typeof Promise?void 0:Promise,"%Proxy%":"undefined"==typeof Proxy?void 0:Proxy,"%RangeError%":RangeError,"%ReferenceError%":ReferenceError,"%Reflect%":"undefined"==typeof Reflect?void 0:Reflect,"%RegExp%":RegExp,"%Set%":"undefined"==typeof Set?void 0:Set,"%SetIteratorPrototype%":"undefined"!=typeof Set&&c?d((new Set)[Symbol.iterator]()):void 0,"%SharedArrayBuffer%":"undefined"==typeof SharedArrayBuffer?void 0:SharedArrayBuffer,"%String%":String,"%StringIteratorPrototype%":c?d(""[Symbol.iterator]()):void 0,"%Symbol%":c?Symbol:void 0,"%SyntaxError%":r,"%ThrowTypeError%":u,"%TypedArray%":f,"%TypeError%":o,"%Uint8Array%":"undefined"==typeof Uint8Array?void 0:Uint8Array,"%Uint8ClampedArray%":"undefined"==typeof Uint8ClampedArray?void 0:Uint8ClampedArray,"%Uint16Array%":"undefined"==typeof Uint16Array?void 0:Uint16Array,"%Uint32Array%":"undefined"==typeof Uint32Array?void 0:Uint32Array,"%URIError%":URIError,"%WeakMap%":"undefined"==typeof WeakMap?void 0:WeakMap,"%WeakRef%":"undefined"==typeof WeakRef?void 0:WeakRef,"%WeakSet%":"undefined"==typeof WeakSet?void 0:WeakSet},g={"%ArrayBufferPrototype%":["ArrayBuffer","prototype"],"%ArrayPrototype%":["Array","prototype"],"%ArrayProto_entries%":["Array","prototype","entries"],"%ArrayProto_forEach%":["Array","prototype","forEach"],"%ArrayProto_keys%":["Array","prototype","keys"],"%ArrayProto_values%":["Array","prototype","values"],"%AsyncFunctionPrototype%":["AsyncFunction","prototype"],"%AsyncGenerator%":["AsyncGeneratorFunction","prototype"],"%AsyncGeneratorPrototype%":["AsyncGeneratorFunction","prototype","prototype"],"%BooleanPrototype%":["Boolean","prototype"],"%DataViewPrototype%":["DataView","prototype"],"%DatePrototype%":["Date","prototype"],"%ErrorPrototype%":["Error","prototype"],"%EvalErrorPrototype%":["EvalError","prototype"],"%Float32ArrayPrototype%":["Float32Array","prototype"],"%Float64ArrayPrototype%":["Float64Array","prototype"],"%FunctionPrototype%":["Function","prototype"],"%Generator%":["GeneratorFunction","prototype"],"%GeneratorPrototype%":["GeneratorFunction","prototype","prototype"],"%Int8ArrayPrototype%":["Int8Array","prototype"],"%Int16ArrayPrototype%":["Int16Array","prototype"],"%Int32ArrayPrototype%":["Int32Array","prototype"],"%JSONParse%":["JSON","parse"],"%JSONStringify%":["JSON","stringify"],"%MapPrototype%":["Map","prototype"],"%NumberPrototype%":["Number","prototype"],"%ObjectPrototype%":["Object","prototype"],"%ObjProto_toString%":["Object","prototype","toString"],"%ObjProto_valueOf%":["Object","prototype","valueOf"],"%PromisePrototype%":["Promise","prototype"],"%PromiseProto_then%":["Promise","prototype","then"],"%Promise_all%":["Promise","all"],"%Promise_reject%":["Promise","reject"],"%Promise_resolve%":["Promise","resolve"],"%RangeErrorPrototype%":["RangeError","prototype"],"%ReferenceErrorPrototype%":["ReferenceError","prototype"],"%RegExpPrototype%":["RegExp","prototype"],"%SetPrototype%":["Set","prototype"],"%SharedArrayBufferPrototype%":["SharedArrayBuffer","prototype"],"%StringPrototype%":["String","prototype"],"%SymbolPrototype%":["Symbol","prototype"],"%SyntaxErrorPrototype%":["SyntaxError","prototype"],"%TypedArrayPrototype%":["TypedArray","prototype"],"%TypeErrorPrototype%":["TypeError","prototype"],"%Uint8ArrayPrototype%":["Uint8Array","prototype"],"%Uint8ClampedArrayPrototype%":["Uint8ClampedArray","prototype"],"%Uint16ArrayPrototype%":["Uint16Array","prototype"],"%Uint32ArrayPrototype%":["Uint32Array","prototype"],"%URIErrorPrototype%":["URIError","prototype"],"%WeakMapPrototype%":["WeakMap","prototype"],"%WeakSetPrototype%":["WeakSet","prototype"]},v=n(507),m=n(1490),y=v.call(Function.call,Array.prototype.concat),b=v.call(Function.apply,Array.prototype.splice),C=v.call(Function.call,String.prototype.replace),A=v.call(Function.call,String.prototype.slice),_=/[^%.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\\]|\\.)*?)\2)\]|(?=(?:\.|\[\])(?:\.|\[\]|%$))/g,I=/\\(\\)?/g,x=function(e){var t=A(e,0,1),n=A(e,-1);if("%"===t&&"%"!==n)throw new r("invalid intrinsic syntax, expected closing `%`");if("%"===n&&"%"!==t)throw new r("invalid intrinsic syntax, expected opening `%`");var i=[];return C(e,_,(function(e,t,n,r){i[i.length]=n?C(r,I,"$1"):t||e})),i},w=function(e,t){var n,i=e;if(m(g,i)&&(i="%"+(n=g[i])[0]+"%"),m(p,i)){var s=p[i];if(s===h&&(s=function e(t){var n;if("%AsyncFunction%"===t)n=a("async function () {}");else if("%GeneratorFunction%"===t)n=a("function* () {}");else if("%AsyncGeneratorFunction%"===t)n=a("async function* () {}");else if("%AsyncGenerator%"===t){var r=e("%AsyncGeneratorFunction%");r&&(n=r.prototype)}else if("%AsyncIteratorPrototype%"===t){var i=e("%AsyncGenerator%");i&&(n=d(i.prototype))}return p[t]=n,n}(i)),void 0===s&&!t)throw new o("intrinsic "+e+" exists, but is not available. Please file an issue!");return{alias:n,name:i,value:s}}throw new r("intrinsic "+e+" does not exist!")};e.exports=function(e,t){if("string"!=typeof e||0===e.length)throw new o("intrinsic name must be a non-empty string");if(arguments.length>1&&"boolean"!=typeof t)throw new o('"allowMissing" argument must be a boolean');var n=x(e),i=n.length>0?n[0]:"",a=w("%"+i+"%",t),l=a.name,u=a.value,c=!1,d=a.alias;d&&(i=d[0],b(n,y([0,1],d)));for(var h=1,f=!0;h<n.length;h+=1){var g=n[h],v=A(g,0,1),C=A(g,-1);if(('"'===v||"'"===v||"`"===v||'"'===C||"'"===C||"`"===C)&&v!==C)throw new r("property names with quotes must have matching quotes");if("constructor"!==g&&f||(c=!0),m(p,l="%"+(i+="."+g)+"%"))u=p[l];else if(null!=u){if(!(g in u)){if(!t)throw new o("base intrinsic for "+e+" exists, but the property is not available.");return}if(s&&h+1>=n.length){var _=s(u,g);u=(f=!!_)&&"get"in _&&!("originalValue"in _.get)?_.get:u[g]}else f=m(u,g),u=u[g];f&&!c&&(p[l]=u)}}return u}},function(e,t,n){"use strict";(function(t){var r=t.Symbol,i=n(1488);e.exports=function(){return"function"==typeof r&&("function"==typeof Symbol&&("symbol"==typeof r("foo")&&("symbol"==typeof Symbol("bar")&&i())))}}).call(this,n(61))},function(e,t,n){"use strict";var r=function(e){return e!=e};e.exports=function(e,t){return 0===e&&0===t?1/e==1/t:e===t||!(!r(e)||!r(t))}},function(e,t,n){"use strict";var r=n(886);e.exports=function(){return"function"==typeof Object.is?Object.is:r}},function(e,t,n){"use strict";var r=Object,i=TypeError;e.exports=function(){if(null!=this&&this!==r(this))throw new i("RegExp.prototype.flags getter called on non-object");var e="";return this.global&&(e+="g"),this.ignoreCase&&(e+="i"),this.multiline&&(e+="m"),this.dotAll&&(e+="s"),this.unicode&&(e+="u"),this.sticky&&(e+="y"),e}},function(e,t,n){"use strict";var r=n(888),i=n(285).supportsDescriptors,o=Object.getOwnPropertyDescriptor,a=TypeError;e.exports=function(){if(!i)throw new a("RegExp.prototype.flags requires a true ES5 environment that supports property descriptors");if("gim"===/a/gim.flags){var e=o(RegExp.prototype,"flags");if(e&&"function"==typeof e.get&&"boolean"==typeof/a/.dotAll)return e.get}return r}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.Classes=void 0;var r=n(506),i=r.__importStar(n(1497));t.Classes=i,r.__exportStar(n(1498),t),r.__exportStar(n(1499),t),r.__exportStar(n(1500),t),r.__exportStar(n(1501),t),r.__exportStar(n(1502),t)},function(e,t,n){"use strict";n.d(t,"a",(function(){return r}));var r;!function(e){e.setGeometry=function(e){const{anchor:t,host:n,node:r,privilege:i}=e;if(r.classList.add("jp-HoverBox"),r.classList.add("jp-mod-outofview"),!n.contains(document.elementFromPoint(t.left,t.top)))return;r.style.maxHeight="",r.style.marginTop="",r.classList.remove("jp-mod-outofview");const o=e.style||window.getComputedStyle(r),a=window.innerHeight,s=t.top,l=a-t.bottom,u=parseInt(o.marginTop,10)||0,c=parseInt(o.minHeight,10)||e.minHeight;let d=parseInt(o.maxHeight,10)||e.maxHeight;const h="forceAbove"!==i&&("forceBelow"===i||("above"===i?s<d&&s<l:l>=d||l>=s));if(h?d=Math.min(l-u,d):(d=Math.min(s,d),r.style.marginTop="0px"),r.style.maxHeight=d+"px",!(d>c&&(l>=c||s>=c)))return void r.classList.add("jp-mod-outofview");const f=e.offset&&e.offset.vertical&&e.offset.vertical.above||0,p=e.offset&&e.offset.vertical&&e.offset.vertical.below||0,g=h?a-l+p:s-r.getBoundingClientRect().height+f;r.style.top=Math.floor(g)+"px";const v=e.offset&&e.offset.horizontal||0;let m=t.left+v;r.style.left=Math.ceil(m)+"px",r.style.width="auto",r.scrollHeight>=d&&(r.style.width=""+(2*r.offsetWidth-r.clientWidth),r.scrollTop=0);const y=r.getBoundingClientRect().right;y>window.innerWidth&&(m-=y-window.innerWidth,r.style.left=Math.ceil(m)+"px")}}(r||(r={}))},function(e,t,n){"use strict";var r=n(241);e.exports=function(e){r.copy(e,this)}},function(e,t,n){"use strict";e.exports=function(e,t){t||(t={}),"function"==typeof t&&(t={cmp:t});var n,r="boolean"==typeof t.cycles&&t.cycles,i=t.cmp&&(n=t.cmp,function(e){return function(t,r){var i={key:t,value:e[t]},o={key:r,value:e[r]};return n(i,o)}}),o=[];return function e(t){if(t&&t.toJSON&&"function"==typeof t.toJSON&&(t=t.toJSON()),void 0!==t){if("number"==typeof t)return isFinite(t)?""+t:"null";if("object"!=typeof t)return JSON.stringify(t);var n,a;if(Array.isArray(t)){for(a="[",n=0;n<t.length;n++)n&&(a+=","),a+=e(t[n])||"null";return a+"]"}if(null===t)return"null";if(-1!==o.indexOf(t)){if(r)return JSON.stringify("__cycle__");throw new TypeError("Converting circular structure to JSON")}var s=o.push(t)-1,l=Object.keys(t).sort(i&&i(t));for(a="",n=0;n<l.length;n++){var u=l[n],c=e(t[u]);c&&(a&&(a+=","),a+=JSON.stringify(u)+":"+c)}return o.splice(s,1),"{"+a+"}"}}(e)}},function(e,t,n){"use strict";e.exports=function(e,t,n){var r="",i=!0===e.schema.$async,o=e.util.schemaHasRulesExcept(e.schema,e.RULES.all,"$ref"),a=e.self._getId(e.schema);if(e.opts.strictKeywords){var s=e.util.schemaUnknownRules(e.schema,e.RULES.keywords);if(s){var l="unknown keyword: "+s;if("log"!==e.opts.strictKeywords)throw new Error(l);e.logger.warn(l)}}if(e.isTop&&(r+=" var validate = ",i&&(e.async=!0,r+="async "),r+="function(data, dataPath, parentData, parentDataProperty, rootData) { 'use strict'; ",a&&(e.opts.sourceCode||e.opts.processCode)&&(r+=" /*# sourceURL="+a+" */ ")),"boolean"==typeof e.schema||!o&&!e.schema.$ref){var u=e.level,c=e.dataLevel,d=e.schema["false schema"],h=e.schemaPath+e.util.getProperty("false schema"),f=e.errSchemaPath+"/false schema",p=!e.opts.allErrors,g="data"+(c||""),v="valid"+u;if(!1===e.schema){e.isTop?p=!0:r+=" var "+v+" = false; ",(Q=Q||[]).push(r),r="",!1!==e.createErrors?(r+=" { keyword: 'false schema' , dataPath: (dataPath || '') + "+e.errorPath+" , schemaPath: "+e.util.toQuotedString(f)+" , params: {} ",!1!==e.opts.messages&&(r+=" , message: 'boolean schema is false' "),e.opts.verbose&&(r+=" , schema: false , parentSchema: validate.schema"+e.schemaPath+" , data: "+g+" "),r+=" } "):r+=" {} ";var m=r;r=Q.pop(),!e.compositeRule&&p?e.async?r+=" throw new ValidationError(["+m+"]); ":r+=" validate.errors = ["+m+"]; return false; ":r+=" var err = "+m+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; "}else e.isTop?r+=i?" return data; ":" validate.errors = null; return true; ":r+=" var "+v+" = true; ";return e.isTop&&(r+=" }; return validate; "),r}if(e.isTop){var y=e.isTop;u=e.level=0,c=e.dataLevel=0,g="data";if(e.rootId=e.resolve.fullPath(e.self._getId(e.root.schema)),e.baseId=e.baseId||e.rootId,delete e.isTop,e.dataPathArr=[""],void 0!==e.schema.default&&e.opts.useDefaults&&e.opts.strictDefaults){var b="default is ignored in the schema root";if("log"!==e.opts.strictDefaults)throw new Error(b);e.logger.warn(b)}r+=" var vErrors = null; ",r+=" var errors = 0;     ",r+=" if (rootData === undefined) rootData = data; "}else{u=e.level,g="data"+((c=e.dataLevel)||"");if(a&&(e.baseId=e.resolve.url(e.baseId,a)),i&&!e.async)throw new Error("async schema in sync schema");r+=" var errs_"+u+" = errors;"}v="valid"+u,p=!e.opts.allErrors;var C="",A="",_=e.schema.type,I=Array.isArray(_);if(_&&e.opts.nullable&&!0===e.schema.nullable&&(I?-1==_.indexOf("null")&&(_=_.concat("null")):"null"!=_&&(_=[_,"null"],I=!0)),I&&1==_.length&&(_=_[0],I=!1),e.schema.$ref&&o){if("fail"==e.opts.extendRefs)throw new Error('$ref: validation keywords used in schema at path "'+e.errSchemaPath+'" (see option extendRefs)');!0!==e.opts.extendRefs&&(o=!1,e.logger.warn('$ref: keywords ignored in schema at path "'+e.errSchemaPath+'"'))}if(e.schema.$comment&&e.opts.$comment&&(r+=" "+e.RULES.all.$comment.code(e,"$comment")),_){if(e.opts.coerceTypes)var x=e.util.coerceToTypes(e.opts.coerceTypes,_);var w=e.RULES.types[_];if(x||I||!0===w||w&&!K(w)){h=e.schemaPath+".type",f=e.errSchemaPath+"/type",h=e.schemaPath+".type",f=e.errSchemaPath+"/type";var M=I?"checkDataTypes":"checkDataType";if(r+=" if ("+e.util[M](_,g,e.opts.strictNumbers,!0)+") { ",x){var S="dataType"+u,T="coerced"+u;r+=" var "+S+" = typeof "+g+"; var "+T+" = undefined; ","array"==e.opts.coerceTypes&&(r+=" if ("+S+" == 'object' && Array.isArray("+g+") && "+g+".length == 1) { "+g+" = "+g+"[0]; "+S+" = typeof "+g+"; if ("+e.util.checkDataType(e.schema.type,g,e.opts.strictNumbers)+") "+T+" = "+g+"; } "),r+=" if ("+T+" !== undefined) ; ";var E=x;if(E)for(var O,k=-1,L=E.length-1;k<L;)"string"==(O=E[k+=1])?r+=" else if ("+S+" == 'number' || "+S+" == 'boolean') "+T+" = '' + "+g+"; else if ("+g+" === null) "+T+" = ''; ":"number"==O||"integer"==O?(r+=" else if ("+S+" == 'boolean' || "+g+" === null || ("+S+" == 'string' && "+g+" && "+g+" == +"+g+" ","integer"==O&&(r+=" && !("+g+" % 1)"),r+=")) "+T+" = +"+g+"; "):"boolean"==O?r+=" else if ("+g+" === 'false' || "+g+" === 0 || "+g+" === null) "+T+" = false; else if ("+g+" === 'true' || "+g+" === 1) "+T+" = true; ":"null"==O?r+=" else if ("+g+" === '' || "+g+" === 0 || "+g+" === false) "+T+" = null; ":"array"==e.opts.coerceTypes&&"array"==O&&(r+=" else if ("+S+" == 'string' || "+S+" == 'number' || "+S+" == 'boolean' || "+g+" == null) "+T+" = ["+g+"]; ");r+=" else {   ",(Q=Q||[]).push(r),r="",!1!==e.createErrors?(r+=" { keyword: 'type' , dataPath: (dataPath || '') + "+e.errorPath+" , schemaPath: "+e.util.toQuotedString(f)+" , params: { type: '",r+=I?""+_.join(","):""+_,r+="' } ",!1!==e.opts.messages&&(r+=" , message: 'should be ",r+=I?""+_.join(","):""+_,r+="' "),e.opts.verbose&&(r+=" , schema: validate.schema"+h+" , parentSchema: validate.schema"+e.schemaPath+" , data: "+g+" "),r+=" } "):r+=" {} ";m=r;r=Q.pop(),!e.compositeRule&&p?e.async?r+=" throw new ValidationError(["+m+"]); ":r+=" validate.errors = ["+m+"]; return false; ":r+=" var err = "+m+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",r+=" } if ("+T+" !== undefined) {  ";var D=c?"data"+(c-1||""):"parentData";r+=" "+g+" = "+T+"; ",c||(r+="if ("+D+" !== undefined)"),r+=" "+D+"["+(c?e.dataPathArr[c]:"parentDataProperty")+"] = "+T+"; } "}else{(Q=Q||[]).push(r),r="",!1!==e.createErrors?(r+=" { keyword: 'type' , dataPath: (dataPath || '') + "+e.errorPath+" , schemaPath: "+e.util.toQuotedString(f)+" , params: { type: '",r+=I?""+_.join(","):""+_,r+="' } ",!1!==e.opts.messages&&(r+=" , message: 'should be ",r+=I?""+_.join(","):""+_,r+="' "),e.opts.verbose&&(r+=" , schema: validate.schema"+h+" , parentSchema: validate.schema"+e.schemaPath+" , data: "+g+" "),r+=" } "):r+=" {} ";m=r;r=Q.pop(),!e.compositeRule&&p?e.async?r+=" throw new ValidationError(["+m+"]); ":r+=" validate.errors = ["+m+"]; return false; ":r+=" var err = "+m+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; "}r+=" } "}}if(e.schema.$ref&&!o)r+=" "+e.RULES.all.$ref.code(e,"$ref")+" ",p&&(r+=" } if (errors === ",r+=y?"0":"errs_"+u,r+=") { ",A+="}");else{var P=e.RULES;if(P)for(var N=-1,j=P.length-1;N<j;)if(K(w=P[N+=1])){if(w.type&&(r+=" if ("+e.util.checkDataType(w.type,g,e.opts.strictNumbers)+") { "),e.opts.useDefaults)if("object"==w.type&&e.schema.properties){d=e.schema.properties;var R=Object.keys(d);if(R)for(var z,F=-1,B=R.length-1;F<B;){if(void 0!==(V=d[z=R[F+=1]]).default){var H=g+e.util.getProperty(z);if(e.compositeRule){if(e.opts.strictDefaults){b="default is ignored for: "+H;if("log"!==e.opts.strictDefaults)throw new Error(b);e.logger.warn(b)}}else r+=" if ("+H+" === undefined ","empty"==e.opts.useDefaults&&(r+=" || "+H+" === null || "+H+" === '' "),r+=" ) "+H+" = ","shared"==e.opts.useDefaults?r+=" "+e.useDefault(V.default)+" ":r+=" "+JSON.stringify(V.default)+" ",r+="; "}}}else if("array"==w.type&&Array.isArray(e.schema.items)){var U=e.schema.items;if(U){k=-1;for(var V,Y=U.length-1;k<Y;)if(void 0!==(V=U[k+=1]).default){H=g+"["+k+"]";if(e.compositeRule){if(e.opts.strictDefaults){b="default is ignored for: "+H;if("log"!==e.opts.strictDefaults)throw new Error(b);e.logger.warn(b)}}else r+=" if ("+H+" === undefined ","empty"==e.opts.useDefaults&&(r+=" || "+H+" === null || "+H+" === '' "),r+=" ) "+H+" = ","shared"==e.opts.useDefaults?r+=" "+e.useDefault(V.default)+" ":r+=" "+JSON.stringify(V.default)+" ",r+="; "}}}var W=w.rules;if(W)for(var G,X=-1,q=W.length-1;X<q;)if(J(G=W[X+=1])){var Z=G.code(e,G.keyword,w.type);Z&&(r+=" "+Z+" ",p&&(C+="}"))}if(p&&(r+=" "+C+" ",C=""),w.type&&(r+=" } ",_&&_===w.type&&!x)){r+=" else { ";var Q;h=e.schemaPath+".type",f=e.errSchemaPath+"/type";(Q=Q||[]).push(r),r="",!1!==e.createErrors?(r+=" { keyword: 'type' , dataPath: (dataPath || '') + "+e.errorPath+" , schemaPath: "+e.util.toQuotedString(f)+" , params: { type: '",r+=I?""+_.join(","):""+_,r+="' } ",!1!==e.opts.messages&&(r+=" , message: 'should be ",r+=I?""+_.join(","):""+_,r+="' "),e.opts.verbose&&(r+=" , schema: validate.schema"+h+" , parentSchema: validate.schema"+e.schemaPath+" , data: "+g+" "),r+=" } "):r+=" {} ";m=r;r=Q.pop(),!e.compositeRule&&p?e.async?r+=" throw new ValidationError(["+m+"]); ":r+=" validate.errors = ["+m+"]; return false; ":r+=" var err = "+m+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",r+=" } "}p&&(r+=" if (errors === ",r+=y?"0":"errs_"+u,r+=") { ",A+="}")}}function K(e){for(var t=e.rules,n=0;n<t.length;n++)if(J(t[n]))return!0}function J(t){return void 0!==e.schema[t.keyword]||t.implements&&function(t){for(var n=t.implements,r=0;r<n.length;r++)if(void 0!==e.schema[n[r]])return!0}(t)}return p&&(r+=" "+A+" "),y?(i?(r+=" if (errors === 0) return data;           ",r+=" else throw new ValidationError(vErrors); "):(r+=" validate.errors = vErrors; ",r+=" return errors === 0;       "),r+=" }; return validate;"):r+=" var "+v+" = errors === errs_"+u+";",r}},function(e,t,n){"use strict";e.exports=function(e,t,n){var r,i=" ",o=e.level,a=e.dataLevel,s=e.schema[t],l=e.schemaPath+e.util.getProperty(t),u=e.errSchemaPath+"/"+t,c=!e.opts.allErrors,d="data"+(a||""),h=e.opts.$data&&s&&s.$data;h?(i+=" var schema"+o+" = "+e.util.getData(s.$data,a,e.dataPathArr)+"; ",r="schema"+o):r=s;var f="maximum"==t,p=f?"exclusiveMaximum":"exclusiveMinimum",g=e.schema[p],v=e.opts.$data&&g&&g.$data,m=f?"<":">",y=f?">":"<",b=void 0;if(!h&&"number"!=typeof s&&void 0!==s)throw new Error(t+" must be number");if(!v&&void 0!==g&&"number"!=typeof g&&"boolean"!=typeof g)throw new Error(p+" must be number or boolean");if(v){var C=e.util.getData(g.$data,a,e.dataPathArr),A="exclusive"+o,_="exclType"+o,I="exclIsNumber"+o,x="' + "+(S="op"+o)+" + '";i+=" var schemaExcl"+o+" = "+C+"; ",i+=" var "+A+"; var "+_+" = typeof "+(C="schemaExcl"+o)+"; if ("+_+" != 'boolean' && "+_+" != 'undefined' && "+_+" != 'number') { ";var w;b=p;(w=w||[]).push(i),i="",!1!==e.createErrors?(i+=" { keyword: '"+(b||"_exclusiveLimit")+"' , dataPath: (dataPath || '') + "+e.errorPath+" , schemaPath: "+e.util.toQuotedString(u)+" , params: {} ",!1!==e.opts.messages&&(i+=" , message: '"+p+" should be boolean' "),e.opts.verbose&&(i+=" , schema: validate.schema"+l+" , parentSchema: validate.schema"+e.schemaPath+" , data: "+d+" "),i+=" } "):i+=" {} ";var M=i;i=w.pop(),!e.compositeRule&&c?e.async?i+=" throw new ValidationError(["+M+"]); ":i+=" validate.errors = ["+M+"]; return false; ":i+=" var err = "+M+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",i+=" } else if ( ",h&&(i+=" ("+r+" !== undefined && typeof "+r+" != 'number') || "),i+=" "+_+" == 'number' ? ( ("+A+" = "+r+" === undefined || "+C+" "+m+"= "+r+") ? "+d+" "+y+"= "+C+" : "+d+" "+y+" "+r+" ) : ( ("+A+" = "+C+" === true) ? "+d+" "+y+"= "+r+" : "+d+" "+y+" "+r+" ) || "+d+" !== "+d+") { var op"+o+" = "+A+" ? '"+m+"' : '"+m+"='; ",void 0===s&&(b=p,u=e.errSchemaPath+"/"+p,r=C,h=v)}else{x=m;if((I="number"==typeof g)&&h){var S="'"+x+"'";i+=" if ( ",h&&(i+=" ("+r+" !== undefined && typeof "+r+" != 'number') || "),i+=" ( "+r+" === undefined || "+g+" "+m+"= "+r+" ? "+d+" "+y+"= "+g+" : "+d+" "+y+" "+r+" ) || "+d+" !== "+d+") { "}else{I&&void 0===s?(A=!0,b=p,u=e.errSchemaPath+"/"+p,r=g,y+="="):(I&&(r=Math[f?"min":"max"](g,s)),g===(!I||r)?(A=!0,b=p,u=e.errSchemaPath+"/"+p,y+="="):(A=!1,x+="="));S="'"+x+"'";i+=" if ( ",h&&(i+=" ("+r+" !== undefined && typeof "+r+" != 'number') || "),i+=" "+d+" "+y+" "+r+" || "+d+" !== "+d+") { "}}b=b||t,(w=w||[]).push(i),i="",!1!==e.createErrors?(i+=" { keyword: '"+(b||"_limit")+"' , dataPath: (dataPath || '') + "+e.errorPath+" , schemaPath: "+e.util.toQuotedString(u)+" , params: { comparison: "+S+", limit: "+r+", exclusive: "+A+" } ",!1!==e.opts.messages&&(i+=" , message: 'should be "+x+" ",i+=h?"' + "+r:r+"'"),e.opts.verbose&&(i+=" , schema:  ",i+=h?"validate.schema"+l:""+s,i+="         , parentSchema: validate.schema"+e.schemaPath+" , data: "+d+" "),i+=" } "):i+=" {} ";M=i;return i=w.pop(),!e.compositeRule&&c?e.async?i+=" throw new ValidationError(["+M+"]); ":i+=" validate.errors = ["+M+"]; return false; ":i+=" var err = "+M+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",i+=" } ",c&&(i+=" else { "),i}},function(e,t,n){"use strict";e.exports=function(e,t,n){var r,i=" ",o=e.level,a=e.dataLevel,s=e.schema[t],l=e.schemaPath+e.util.getProperty(t),u=e.errSchemaPath+"/"+t,c=!e.opts.allErrors,d="data"+(a||""),h=e.opts.$data&&s&&s.$data;if(h?(i+=" var schema"+o+" = "+e.util.getData(s.$data,a,e.dataPathArr)+"; ",r="schema"+o):r=s,!h&&"number"!=typeof s)throw new Error(t+" must be number");i+="if ( ",h&&(i+=" ("+r+" !== undefined && typeof "+r+" != 'number') || "),i+=" "+d+".length "+("maxItems"==t?">":"<")+" "+r+") { ";var f=t,p=p||[];p.push(i),i="",!1!==e.createErrors?(i+=" { keyword: '"+(f||"_limitItems")+"' , dataPath: (dataPath || '') + "+e.errorPath+" , schemaPath: "+e.util.toQuotedString(u)+" , params: { limit: "+r+" } ",!1!==e.opts.messages&&(i+=" , message: 'should NOT have ",i+="maxItems"==t?"more":"fewer",i+=" than ",i+=h?"' + "+r+" + '":""+s,i+=" items' "),e.opts.verbose&&(i+=" , schema:  ",i+=h?"validate.schema"+l:""+s,i+="         , parentSchema: validate.schema"+e.schemaPath+" , data: "+d+" "),i+=" } "):i+=" {} ";var g=i;return i=p.pop(),!e.compositeRule&&c?e.async?i+=" throw new ValidationError(["+g+"]); ":i+=" validate.errors = ["+g+"]; return false; ":i+=" var err = "+g+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",i+="} ",c&&(i+=" else { "),i}},function(e,t,n){"use strict";e.exports=function(e,t,n){var r,i=" ",o=e.level,a=e.dataLevel,s=e.schema[t],l=e.schemaPath+e.util.getProperty(t),u=e.errSchemaPath+"/"+t,c=!e.opts.allErrors,d="data"+(a||""),h=e.opts.$data&&s&&s.$data;if(h?(i+=" var schema"+o+" = "+e.util.getData(s.$data,a,e.dataPathArr)+"; ",r="schema"+o):r=s,!h&&"number"!=typeof s)throw new Error(t+" must be number");var f="maxLength"==t?">":"<";i+="if ( ",h&&(i+=" ("+r+" !== undefined && typeof "+r+" != 'number') || "),!1===e.opts.unicode?i+=" "+d+".length ":i+=" ucs2length("+d+") ",i+=" "+f+" "+r+") { ";var p=t,g=g||[];g.push(i),i="",!1!==e.createErrors?(i+=" { keyword: '"+(p||"_limitLength")+"' , dataPath: (dataPath || '') + "+e.errorPath+" , schemaPath: "+e.util.toQuotedString(u)+" , params: { limit: "+r+" } ",!1!==e.opts.messages&&(i+=" , message: 'should NOT be ",i+="maxLength"==t?"longer":"shorter",i+=" than ",i+=h?"' + "+r+" + '":""+s,i+=" characters' "),e.opts.verbose&&(i+=" , schema:  ",i+=h?"validate.schema"+l:""+s,i+="         , parentSchema: validate.schema"+e.schemaPath+" , data: "+d+" "),i+=" } "):i+=" {} ";var v=i;return i=g.pop(),!e.compositeRule&&c?e.async?i+=" throw new ValidationError(["+v+"]); ":i+=" validate.errors = ["+v+"]; return false; ":i+=" var err = "+v+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",i+="} ",c&&(i+=" else { "),i}},function(e,t,n){"use strict";e.exports=function(e,t,n){var r,i=" ",o=e.level,a=e.dataLevel,s=e.schema[t],l=e.schemaPath+e.util.getProperty(t),u=e.errSchemaPath+"/"+t,c=!e.opts.allErrors,d="data"+(a||""),h=e.opts.$data&&s&&s.$data;if(h?(i+=" var schema"+o+" = "+e.util.getData(s.$data,a,e.dataPathArr)+"; ",r="schema"+o):r=s,!h&&"number"!=typeof s)throw new Error(t+" must be number");i+="if ( ",h&&(i+=" ("+r+" !== undefined && typeof "+r+" != 'number') || "),i+=" Object.keys("+d+").length "+("maxProperties"==t?">":"<")+" "+r+") { ";var f=t,p=p||[];p.push(i),i="",!1!==e.createErrors?(i+=" { keyword: '"+(f||"_limitProperties")+"' , dataPath: (dataPath || '') + "+e.errorPath+" , schemaPath: "+e.util.toQuotedString(u)+" , params: { limit: "+r+" } ",!1!==e.opts.messages&&(i+=" , message: 'should NOT have ",i+="maxProperties"==t?"more":"fewer",i+=" than ",i+=h?"' + "+r+" + '":""+s,i+=" properties' "),e.opts.verbose&&(i+=" , schema:  ",i+=h?"validate.schema"+l:""+s,i+="         , parentSchema: validate.schema"+e.schemaPath+" , data: "+d+" "),i+=" } "):i+=" {} ";var g=i;return i=p.pop(),!e.compositeRule&&c?e.async?i+=" throw new ValidationError(["+g+"]); ":i+=" validate.errors = ["+g+"]; return false; ":i+=" var err = "+g+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",i+="} ",c&&(i+=" else { "),i}},function(e){e.exports=JSON.parse('{"$schema":"http://json-schema.org/draft-07/schema#","$id":"http://json-schema.org/draft-07/schema#","title":"Core schema meta-schema","definitions":{"schemaArray":{"type":"array","minItems":1,"items":{"$ref":"#"}},"nonNegativeInteger":{"type":"integer","minimum":0},"nonNegativeIntegerDefault0":{"allOf":[{"$ref":"#/definitions/nonNegativeInteger"},{"default":0}]},"simpleTypes":{"enum":["array","boolean","integer","null","number","object","string"]},"stringArray":{"type":"array","items":{"type":"string"},"uniqueItems":true,"default":[]}},"type":["object","boolean"],"properties":{"$id":{"type":"string","format":"uri-reference"},"$schema":{"type":"string","format":"uri"},"$ref":{"type":"string","format":"uri-reference"},"$comment":{"type":"string"},"title":{"type":"string"},"description":{"type":"string"},"default":true,"readOnly":{"type":"boolean","default":false},"examples":{"type":"array","items":true},"multipleOf":{"type":"number","exclusiveMinimum":0},"maximum":{"type":"number"},"exclusiveMaximum":{"type":"number"},"minimum":{"type":"number"},"exclusiveMinimum":{"type":"number"},"maxLength":{"$ref":"#/definitions/nonNegativeInteger"},"minLength":{"$ref":"#/definitions/nonNegativeIntegerDefault0"},"pattern":{"type":"string","format":"regex"},"additionalItems":{"$ref":"#"},"items":{"anyOf":[{"$ref":"#"},{"$ref":"#/definitions/schemaArray"}],"default":true},"maxItems":{"$ref":"#/definitions/nonNegativeInteger"},"minItems":{"$ref":"#/definitions/nonNegativeIntegerDefault0"},"uniqueItems":{"type":"boolean","default":false},"contains":{"$ref":"#"},"maxProperties":{"$ref":"#/definitions/nonNegativeInteger"},"minProperties":{"$ref":"#/definitions/nonNegativeIntegerDefault0"},"required":{"$ref":"#/definitions/stringArray"},"additionalProperties":{"$ref":"#"},"definitions":{"type":"object","additionalProperties":{"$ref":"#"},"default":{}},"properties":{"type":"object","additionalProperties":{"$ref":"#"},"default":{}},"patternProperties":{"type":"object","additionalProperties":{"$ref":"#"},"propertyNames":{"format":"regex"},"default":{}},"dependencies":{"type":"object","additionalProperties":{"anyOf":[{"$ref":"#"},{"$ref":"#/definitions/stringArray"}]}},"propertyNames":{"$ref":"#"},"const":true,"enum":{"type":"array","items":true,"minItems":1,"uniqueItems":true},"type":{"anyOf":[{"$ref":"#/definitions/simpleTypes"},{"type":"array","items":{"$ref":"#/definitions/simpleTypes"},"minItems":1,"uniqueItems":true}]},"format":{"type":"string"},"contentMediaType":{"type":"string"},"contentEncoding":{"type":"string"},"if":{"$ref":"#"},"then":{"$ref":"#"},"else":{"$ref":"#"},"allOf":{"$ref":"#/definitions/schemaArray"},"anyOf":{"$ref":"#/definitions/schemaArray"},"oneOf":{"$ref":"#/definitions/schemaArray"},"not":{"$ref":"#"}},"default":true}')},function(e,t,n){e.exports={v1:{state:n(1556),view:n(1557)},v2:{state:n(1558),view:n(1559)}}},function(e,t,n){var r,i,o=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),a=this&&this.__setModuleDefault||(Object.create?function(e,t){Object.defineProperty(e,"default",{enumerable:!0,value:t})}:function(e,t){e.default=t}),s=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)"default"!==n&&Object.hasOwnProperty.call(e,n)&&o(t,e,n);return a(t,e),t},l=this&&this.__awaiter||function(e,t,n,r){return new(n||(n=Promise))((function(i,o){function a(e){try{l(r.next(e))}catch(e){o(e)}}function s(e){try{l(r.throw(e))}catch(e){o(e)}}function l(e){var t;e.done?i(e.value):(t=e.value,t instanceof n?t:new n((function(e){e(t)}))).then(a,s)}l((r=r.apply(e,t||[])).next())}))},u=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};r=[n,t,n(84),n(90),n(75),n(108),n(109),n(110),n(175),n(103),n(288),n(120),n(88),n(121),n(1565),n(514),n(0),n(78),n(849),n(1579),n(1636),n(2040),n(2214),n(2222),n(2306),n(2311),n(2328),n(2330),n(2334),n(176),n(305)],void 0===(i=function(e,t,n,r,i,o,a,c,d,h,f,p,g,v,m,y,b,C,A,_,I,x,w,M,S,T,E,O,k,L){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.SysExplorerElement=void 0,i=u(i),o=u(o),a=u(a),c=u(c),d=u(d),h=u(h),f=u(f),p=u(p),g=u(g),v=u(v),m=u(m),y=s(y),b=s(b),A=u(A),_=u(_),I=u(I),x=u(x),w=u(w),M=u(M),S=u(S),T=u(T),E=u(E),O=u(O),k=u(k),L=s(L);const D={global:{tabEnableRename:!0},layout:{type:"row",id:"#1",children:[{type:"tabset",id:"#2",children:[],active:!0}]},borders:[]},P={global:{tabEnableRename:!0,tabSetTabLocation:"bottom"},layout:{type:"row",id:"#1",children:[{type:"tabset",id:"#2",children:[{type:"tab",id:"#3",name:"New section ",component:"sub",config:{model:{global:{},layout:{type:"row",id:"#1",children:[{type:"tabset",id:"#3",children:[],active:!0}]},borders:[]}}}],active:!0}]},borders:[]};class N extends b.Component{constructor(e){super(e),this.layoutRef=b.default.createRef(),this.get_python_code=e=>{const t={};let n;n="plotJson"in e?e.plotJson:{};for(const[e,r]of Object.entries(n)){const{traceConfig:n={}}=r;"pythonCode"in n&&"widgetName"in n&&n.widgetName.length>0&&(t[n.widgetName+e]={pythonCode:n.pythonCode,id:e,widgetName:n.widgetName})}return t},this.get_document_source=e=>{const t={};let n;n="plotJson"in e?e.plotJson:{};for(const[e,r]of Object.entries(n)){const{traceConfig:n={}}=r;"documentSource"in n&&"widgetName"in n&&n.widgetName.length>0&&(t[n.widgetName+e]={documentSource:n.documentSource,id:e,widgetName:n.widgetName})}return t},this.on_msg=(e,t)=>{const{type:n,payload:r}=e;switch(n){case"ChartElement::update_signal":this.computedUpdate(r);break;case"SysExplorer::update_save_path":this.templatePath=r.templatePath}},this.componentWillUnmount=()=>{this.props.model.stopListening(this.props.model,"msg:custom",this.on_msg)},this.computedUpdate=e=>{const t=e.computed_data,n=e.recorder_data,r=e.driver_data;this.props.updateComputedResult({variable:t,recorder:n,driver:r})},this.loadChartFromJson=e=>{const t=e.target.files[0],n=new FileReader;n.onload=()=>{const{modelJson:e,plotJson:t,selectedVariable:r}=JSON.parse(n.result);r&&this.props.setControllerState(r),this.initialStateAll=JSON.parse(JSON.stringify(t)),this.setState(t=>Object.assign(Object.assign({},t),{model:y.Model.fromJson(e)}))},n.readAsText(t)},this.addGraph=(e,t=null,n=null)=>{t&&t in this.innerlayoutRef&&this.innerlayoutRef[t].current.addTabToTabSet(n,{component:e,name:N.COMPONENT_DICT[e],config:{layoutID:t}})},this.onRenderTabSet=(e,t,n)=>{const r=e.getId();t.buttons.push(function(e,t,n){const[r,o]=b.default.useState(null),a=()=>{o(null)},s="tab_settings_popover_"+t+n,l=Object.keys(N.COMPONENT_DICT).map(r=>b.default.createElement(p.default,{key:r,onClick:()=>{e(r,t,n),a()}},N.COMPONENT_DICT[r]));return b.default.createElement("div",{key:s},b.default.createElement(i.default,{"aria-controls":"simple-menu","aria-haspopup":"true",onClick:e=>{o(e.currentTarget)}},b.default.createElement(m.default,{style:{color:"darkgreen"}})),b.default.createElement(f.default,{id:"simple-menu",anchorEl:r,keepMounted:!0,open:Boolean(r),onClose:a},l))}(this.addGraph,n,r))},this.prepare_data_for_factory=(e,t)=>{let n="";e&&"layoutID"in e&&(n=e.layoutID);const r=`${n}@${t}`;let i;return this.divRef[r]=b.default.createRef(),r in this.initialStateAll?(i=JSON.parse(JSON.stringify(this.initialStateAll[r])),delete this.initialStateAll[r],"traceConfig"in i&&this.correctKey(i.traceConfig,"add")):i=null,{id:r,data:i}},this.factory=e=>{const t=e.getComponent(),n=e.getConfig(),r=e.getId();e.getName();switch(Object.values(N.COMPONENT_DICT).push("Section"),t){case"grid":{const{id:e,data:t}=this.prepare_data_for_factory(n,r);return b.default.createElement(I.default,{ref:this.divRef[e],model:this.props.model,send_msg:this.props.send_msg,id:e,classes:{},initialState:t})}case"controller":return b.default.createElement(w.default,{model:this.props.model,send_msg:this.props.send_msg});case"3Dview":return b.default.createElement(S.default,{model:this.props.model,send_msg:this.props.send_msg});case"structureView":{const{id:e,data:t}=this.prepare_data_for_factory(n,r);return b.default.createElement(T.default,{ref:this.divRef[e],model:this.props.model,send_msg:this.props.send_msg,initialState:t})}case"PBS":return b.default.createElement(_.default,null);case"connectionView":return b.default.createElement(x.default,null);case"infoView":return b.default.createElement(E.default,{model:this.props.model,send_msg:this.props.send_msg});case"dataView":{const{id:e,data:t}=this.prepare_data_for_factory(n,r);return b.default.createElement(M.default,{ref:this.divRef[e],model:this.props.model,send_msg:this.props.send_msg,id:e,initialState:t})}case"widgetView":{const{id:e,data:t}=this.prepare_data_for_factory(n,r);return b.default.createElement(k.default,{ref:this.divRef[e],model:this.props.model,send_msg:this.props.send_msg,id:e,initialState:t,availableWidget:this.availableWidget})}case"documentView":{const{id:e,data:t}=this.prepare_data_for_factory(n,r);return b.default.createElement(O.default,{ref:this.divRef[e],model:this.props.model,send_msg:this.props.send_msg,id:e,initialState:t,availableDocument:this.availableDocument})}case"sub":return this.generateSection(e,r)}return null},this.generateSection=(e,t)=>{let n,r=e.getExtraData().model;return this.innerlayoutRef[t]=b.default.createRef(),n=e.getConfig()&&e.getConfig().model?e.getConfig().model:D,r||(e.getExtraData().model=y.Model.fromJson(n),r=e.getExtraData().model,e.setEventListener("save",n=>{this.state.model.doAction(y.Actions.updateNodeAttributes(t,{config:{model:e.getExtraData().model.toJson()}}))})),b.default.createElement(y.Layout,{ref:this.innerlayoutRef[t],classNameMapper:e=>("flexlayout__tabset-selected"===e?e="inner__flexlayout__tabset-selected flexlayout__tabset-selected":"flexlayout__tabset"===e?e="inner__flexlayout__tabset flexlayout__tabset":"flexlayout__tab"===e&&(e="inner__flexlayout__tab flexlayout__tab"),e),model:r,factory:this.factory,onRenderTabSet:(e,n)=>{this.onRenderTabSet(e,n,t)},onAction:e=>this.innerOnAction(t,e)})},this.onAddRow=()=>{this.layoutRef.current.addTabToActiveTabSet({component:"sub",name:"New section"})},this.onAdd=()=>{this.layoutRef.current.addTabToActiveTabSet({component:"grid",name:"New chart"})},this.onAddController=()=>{this.layoutRef.current.addTabToActiveTabSet({component:"controller",name:"New controller"})},this.onAction=e=>{if("FlexLayout_MoveNode"!==e.type&&"FlexLayout_AdjustSplit"!==e.type&&"FlexLayout_DeleteTab"!==e.type&&"FlexLayout_MaximizeToggle"!==e.type&&"FlexLayout_SelectTab"!==e.type||window.dispatchEvent(new Event("resize")),"FlexLayout_DeleteTab"===e.type){const t=e.data.node;for(const e in this.divRef)e.split("@")[0]===t&&delete this.divRef[e];for(const e in this.initialStateAll)e.split("@")[0]===t&&delete this.initialStateAll[e]}return e},this.innerOnAction=(e,t)=>{if("FlexLayout_MoveNode"!==t.type&&"FlexLayout_AdjustSplit"!==t.type&&"FlexLayout_DeleteTab"!==t.type&&"FlexLayout_MaximizeToggle"!==t.type||window.dispatchEvent(new Event("resize")),"FlexLayout_DeleteTab"===t.type){const n=`${e}@${t.data.node}`;n in this.divRef&&delete this.divRef[n],n in this.initialStateAll&&delete this.initialStateAll[n]}return t},this.onModelChange=()=>{console.log()},this.toggleSaveDialog=()=>{this.setState(e=>e.saveDialog?Object.assign(Object.assign({},e),{saveDialog:!e.saveDialog}):Object.assign(Object.assign({},e),{saveDialog:!e.saveDialog,jsonName:this.templatePath}))},this.handleJsonName=e=>{const t=e.target.value;this.setState(e=>Object.assign(Object.assign({},e),{jsonName:t}))},this.correctKey=(e,t)=>{const n=["systemNameX","systemNameY","systemNameZ","systemName"],r=this.props.systemConfig.root_name;Object.keys(e).forEach(i=>{if("systemName"===i&&"string"==typeof e[i])"add"===t?""===e[i]?e[i]=r:e[i]=`${r}.${e[i]}`:e[i]===r?e[i]="":e[i]=e[i].replace(r+".","");else if("object"==typeof e[i]&&null!==e[i])for(const o of n)o in e[i]&&("add"===t?""===e[i][o]?e[i][o]=r:e[i][o]=`${r}.${e[i][o]}`:e[i][o]===r?e[i][o]="":e[i][o]=e[i][o].replace(r+".",""))})},this.saveChartToJson=()=>{const e=this.state.jsonName,t=this.state.model.toJson(),n={};for(const[e,t]of Object.entries(this.divRef)){const r=JSON.parse(JSON.stringify(t.current.state.traceConfig));this.correctKey(r,"remove"),n[e]={traceConfig:r,plotLayout:t.current.state.plotLayout,advancedTraceDataConfig:t.current.state.advancedTraceDataConfig}}for(const e in this.initialStateAll)e in n||(n[e]=this.initialStateAll[e]);const r={},i=this.props.systemConfig.root_name;for(const e in this.props.selectedVariable){r[e.replace(i+".","")]=this.props.selectedVariable[e]}this.props.send_msg({action:"SysExplorer::chartViewerSaveJson",payload:{jsonName:e,jsonData:{modelJson:t,plotJson:n,selectedVariable:r}}}),this.toggleSaveDialog()},this.toggleLockTab=()=>{this.state.model.doAction(y.Actions.updateModelAttributes({tabEnableClose:this.state.enableClose}));for(const e in this.innerlayoutRef)this.innerlayoutRef[e].current.props.model.doAction(y.Actions.updateModelAttributes({tabEnableClose:this.state.enableClose}))},this.initialStateAll={},this.divRef={},this.innerlayoutRef={},this.layoutRef=b.default.createRef(),this.tabProps={},this.tabUpdateSignal={},e.model.listenTo(e.model,"msg:custom",this.on_msg);const{chart_template:t={},template_path:n=null}=this.props.model.get("chart_template");if(this.chartTemplate=t,this.templatePath=n,this.availableWidget=this.get_python_code(this.chartTemplate),this.availableDocument=this.get_document_source(this.chartTemplate),"selectedVariable"in this.chartTemplate){const t={};for(const e in this.chartTemplate.selectedVariable)t[`${this.props.systemConfig.root_name}.${e}`]=this.chartTemplate.selectedVariable[e];e.setControllerState(t)}D.global.tabEnableClose=this.props.systemConfig.enableEdit,P.global.tabEnableClose=this.props.systemConfig.enableEdit,this.state={model:y.Model.fromJson(P),selectedTab:"",saveDialog:!1,jsonName:this.templatePath?this.templatePath:"",enableClose:this.props.systemConfig.enableEdit}}componentDidMount(){"modelJson"in this.chartTemplate&&"plotJson"in this.chartTemplate&&(this.initialStateAll=JSON.parse(JSON.stringify(this.chartTemplate.plotJson)),this.setState(e=>Object.assign(Object.assign({},e),{model:y.Model.fromJson(this.chartTemplate.modelJson)})))}restartKernel(){return l(this,void 0,void 0,(function*(){if(!confirm("Do you want to restart kernel an refresh page?"))return;const{BASEURL:e,COSAPP_MODULE:t}=A.default(),r=window.sessionStorage.getItem("adso_kernel_id");if(r){const t=`${e}api/kernels/${r}`,i=n.ServerConnection.makeSettings(),o={method:"DELETE"};let a;try{a=yield n.ServerConnection.makeRequest(t,o,i)}catch(e){return void console.log("error",e)}(yield a).ok&&(window.sessionStorage.removeItem("adso_kernel_id"),window.sessionStorage.removeItem("adso_kernel_name"),location.reload())}}))}render(){const{classes:e}=this.props,t=b.default.createElement(i.default,{onClick:this.toggleSaveDialog,classes:{root:e.textColor}},"save"),n=b.default.createElement("div",null,b.default.createElement("input",{onChange:e=>this.loadChartFromJson(e),onClick:e=>{e.currentTarget.value=null},style:{display:"None"},id:"load-chart-from-json-input",type:"file"}),b.default.createElement("label",{htmlFor:"load-chart-from-json-input"},b.default.createElement(i.default,{classes:{root:e.textColor},component:"span"},"load")));return b.default.createElement("div",{style:{height:"100%"}},b.default.createElement(o.default,{open:this.state.saveDialog,"aria-labelledby":"draggable-dialog-title",fullWidth:!0,maxWidth:"sm"},b.default.createElement(d.default,{style:{cursor:"move"},className:"draggable-dialog-title"},"Save configuration"),b.default.createElement(c.default,null,b.default.createElement(h.default,{className:this.props.classes.formControl},b.default.createElement(g.default,{value:this.state.jsonName,onChange:this.handleJsonName,label:"File name"}))),b.default.createElement(a.default,null,b.default.createElement(i.default,{autoFocus:!0,onClick:this.toggleSaveDialog,color:"primary"},"Close"),b.default.createElement(i.default,{autoFocus:!0,onClick:this.saveChartToJson,color:"primary"},"Save"))),b.default.createElement("div",{style:{width:"100%",height:"calc(100% - 36px)",background:"radial-gradient(#efeded, #8f9091)"}},b.default.createElement(y.Layout,{ref:this.layoutRef,model:this.state.model,factory:this.factory,classNameMapper:e=>("flexlayout__layout"===e?e="chartviewer__flexlayout__layout flexlayout__layout ":"flexlayout__tabset-selected"===e&&(e="outer__flexlayout__tabset-selected flexlayout__tabset-selected "),e),onAction:this.onAction})),b.default.createElement(v.default,{variant:"dense",classes:{dense:e.toolbarHeigt}},b.default.createElement(i.default,{onClick:this.onAddRow,classes:{root:e.textColor}},"Add section"),this.props.systemConfig.enableEdit?t:b.default.createElement("div",null),this.props.systemConfig.enableEdit?n:b.default.createElement("div",null),this.props.systemConfig.enableEdit?b.default.createElement("div",null):b.default.createElement(i.default,{onClick:this.restartKernel,classes:{root:e.textColor}},"Restart"),b.default.createElement(i.default,{onClick:()=>{this.setState(e=>Object.assign(Object.assign({},e),{enableClose:!e.enableClose}),this.toggleLockTab)},classes:{root:e.textColor}},this.state.enableClose?"lock":"unlock")))}}t.SysExplorerElement=N,N.COMPONENT_DICT={grid:"Chart widget",dataView:"Data widget",controller:"Controller widget","3Dview":"3D widget",structureView:"Structure widget",infoView:"System info widget",PBS:"PBS widget",connectionView:"Connection widget",documentView:"Document widget",widgetView:"Custom widget"},t.default=r.withStyles(e=>({textSizeSmall:{},toolbarHeigt:{minHeight:36,background:"rgb(50, 50, 50)"},viewSelector:{minWidth:120,color:"rgb(250, 250, 250)"},bgSelector:{minWidth:60,color:"rgb(250, 250, 250)"},textColor:{color:"rgb(250, 250, 250)",background:"#525354",margin:"0px 5px"},backGround:{color:"rgb(50, 50, 50)"},formControl:{padding:e.spacing(1),width:"90%"}}))(C.connect(e=>(e=>({computingState:e.dashboardState.computing,selectedVariable:e.dashboardState.selectedVariable,systemConfig:e.systemConfig}))(e),e=>({updateComputedResult:t=>e(L.dashboardUpdateComputedResult(t)),toggleComputing:()=>e(L.dashboardToggleRun()),setControllerState:t=>e(L.dashboardSetController(t))}))(N))}.apply(t,r))||(e.exports=i)},function(e,t,n){"use strict";n.d(t,"a",(function(){return i}));var r=n(2349);function i(e){return Object(r.a)(e)}},function(e,t,n){"use strict";var r=n(1),i=n(654),o=n(167);t.a=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};return Object(i.a)(e,Object(r.a)({defaultTheme:o.a},t))}},function(e,t,n){"use strict";var r=n(1),i=n(450),o=n(49),a=n(5),s=n(0),l=(n(116),n(3),n(4)),u=n(2352),c=n(336),d=n(7),h=n(451),f=n(100),p=s.forwardRef((function(e,t){var n=e.children,d=e.classes,p=e.className,g=e.defaultExpanded,v=void 0!==g&&g,m=e.disabled,y=void 0!==m&&m,b=e.expanded,C=e.onChange,A=e.square,_=void 0!==A&&A,I=e.TransitionComponent,x=void 0===I?u.a:I,w=e.TransitionProps,M=Object(a.a)(e,["children","classes","className","defaultExpanded","disabled","expanded","onChange","square","TransitionComponent","TransitionProps"]),S=Object(f.a)({controlled:b,default:v,name:"Accordion",state:"expanded"}),T=Object(o.a)(S,2),E=T[0],O=T[1],k=s.useCallback((function(e){O(!E),C&&C(e,!E)}),[E,C,O]),L=s.Children.toArray(n),D=Object(i.a)(L),P=D[0],N=D.slice(1),j=s.useMemo((function(){return{expanded:E,disabled:y,toggle:k}}),[E,y,k]);return s.createElement(c.a,Object(r.a)({className:Object(l.a)(d.root,p,E&&d.expanded,y&&d.disabled,!_&&d.rounded),ref:t,square:_},M),s.createElement(h.a.Provider,{value:j},P),s.createElement(x,Object(r.a)({in:E,timeout:"auto"},w),s.createElement("div",{"aria-labelledby":P.props.id,id:P.props["aria-controls"],role:"region"},N)))}));t.a=Object(d.a)((function(e){var t={duration:e.transitions.duration.shortest};return{root:{position:"relative",transition:e.transitions.create(["margin"],t),"&:before":{position:"absolute",left:0,top:-1,right:0,height:1,content:'""',opacity:1,backgroundColor:e.palette.divider,transition:e.transitions.create(["opacity","background-color"],t)},"&:first-child":{"&:before":{display:"none"}},"&$expanded":{margin:"16px 0","&:first-child":{marginTop:0},"&:last-child":{marginBottom:0},"&:before":{opacity:0}},"&$expanded + &":{"&:before":{display:"none"}},"&$disabled":{backgroundColor:e.palette.action.disabledBackground}},rounded:{borderRadius:0,"&:first-child":{borderTopLeftRadius:e.shape.borderRadius,borderTopRightRadius:e.shape.borderRadius},"&:last-child":{borderBottomLeftRadius:e.shape.borderRadius,borderBottomRightRadius:e.shape.borderRadius,"@supports (-ms-ime-align: auto)":{borderBottomLeftRadius:0,borderBottomRightRadius:0}}},expanded:{},disabled:{}}}),{name:"MuiAccordion"})(p)},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(4)),s=n(7),l=o.forwardRef((function(e,t){var n=e.classes,s=e.className,l=Object(i.a)(e,["classes","className"]);return o.createElement("div",Object(r.a)({className:Object(a.a)(n.root,s),ref:t},l))}));t.a=Object(s.a)((function(e){return{root:{display:"flex",padding:e.spacing(1,2,2)}}}),{name:"MuiAccordionDetails"})(l)},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(4)),s=n(231),l=n(452),u=n(7),c=n(451),d=o.forwardRef((function(e,t){var n=e.children,u=e.classes,d=e.className,h=e.expandIcon,f=e.IconButtonProps,p=e.onBlur,g=e.onClick,v=e.onFocusVisible,m=Object(i.a)(e,["children","classes","className","expandIcon","IconButtonProps","onBlur","onClick","onFocusVisible"]),y=o.useState(!1),b=y[0],C=y[1],A=o.useContext(c.a),_=A.disabled,I=void 0!==_&&_,x=A.expanded,w=A.toggle;return o.createElement(s.a,Object(r.a)({focusRipple:!1,disableRipple:!0,disabled:I,component:"div","aria-expanded":x,className:Object(a.a)(u.root,d,I&&u.disabled,x&&u.expanded,b&&u.focused),onFocusVisible:function(e){C(!0),v&&v(e)},onBlur:function(e){C(!1),p&&p(e)},onClick:function(e){w&&w(e),g&&g(e)},ref:t},m),o.createElement("div",{className:Object(a.a)(u.content,x&&u.expanded)},n),h&&o.createElement(l.a,Object(r.a)({className:Object(a.a)(u.expandIcon,x&&u.expanded),edge:"end",component:"div",tabIndex:null,role:null,"aria-hidden":!0},f),h))}));t.a=Object(u.a)((function(e){var t={duration:e.transitions.duration.shortest};return{root:{display:"flex",minHeight:48,transition:e.transitions.create(["min-height","background-color"],t),padding:e.spacing(0,2),"&:hover:not($disabled)":{cursor:"pointer"},"&$expanded":{minHeight:64},"&$focused":{backgroundColor:e.palette.action.focus},"&$disabled":{opacity:e.palette.action.disabledOpacity}},expanded:{},focused:{},disabled:{},content:{display:"flex",flexGrow:1,transition:e.transitions.create(["margin"],t),margin:"12px 0","&$expanded":{margin:"20px 0"}},expandIcon:{transform:"rotate(0deg)",transition:e.transitions.create("transform",t),"&:hover":{backgroundColor:"transparent"},"&$expanded":{transform:"rotate(180deg)"}}}}),{name:"MuiAccordionSummary"})(d)},function(e,t,n){"use strict";var r=n(5),i=n(1),o=n(0),a=(n(3),n(4)),s=n(7),l=n(26),u=n(231),c=n(15),d=o.forwardRef((function(e,t){var n=e.children,s=e.classes,l=e.className,d=e.color,h=void 0===d?"default":d,f=e.component,p=void 0===f?"button":f,g=e.disabled,v=void 0!==g&&g,m=e.disableElevation,y=void 0!==m&&m,b=e.disableFocusRipple,C=void 0!==b&&b,A=e.endIcon,_=e.focusVisibleClassName,I=e.fullWidth,x=void 0!==I&&I,w=e.size,M=void 0===w?"medium":w,S=e.startIcon,T=e.type,E=void 0===T?"button":T,O=e.variant,k=void 0===O?"text":O,L=Object(r.a)(e,["children","classes","className","color","component","disabled","disableElevation","disableFocusRipple","endIcon","focusVisibleClassName","fullWidth","size","startIcon","type","variant"]),D=S&&o.createElement("span",{className:Object(a.a)(s.startIcon,s["iconSize".concat(Object(c.a)(M))])},S),P=A&&o.createElement("span",{className:Object(a.a)(s.endIcon,s["iconSize".concat(Object(c.a)(M))])},A);return o.createElement(u.a,Object(i.a)({className:Object(a.a)(s.root,s[k],l,"inherit"===h?s.colorInherit:"default"!==h&&s["".concat(k).concat(Object(c.a)(h))],"medium"!==M&&[s["".concat(k,"Size").concat(Object(c.a)(M))],s["size".concat(Object(c.a)(M))]],y&&s.disableElevation,v&&s.disabled,x&&s.fullWidth),component:p,disabled:v,focusRipple:!C,focusVisibleClassName:Object(a.a)(s.focusVisible,_),ref:t,type:E},L),o.createElement("span",{className:s.label},D,n,P))}));t.a=Object(s.a)((function(e){return{root:Object(i.a)({},e.typography.button,{boxSizing:"border-box",minWidth:64,padding:"6px 16px",borderRadius:e.shape.borderRadius,color:e.palette.text.primary,transition:e.transitions.create(["background-color","box-shadow","border"],{duration:e.transitions.duration.short}),"&:hover":{textDecoration:"none",backgroundColor:Object(l.d)(e.palette.text.primary,e.palette.action.hoverOpacity),"@media (hover: none)":{backgroundColor:"transparent"},"&$disabled":{backgroundColor:"transparent"}},"&$disabled":{color:e.palette.action.disabled}}),label:{width:"100%",display:"inherit",alignItems:"inherit",justifyContent:"inherit"},text:{padding:"6px 8px"},textPrimary:{color:e.palette.primary.main,"&:hover":{backgroundColor:Object(l.d)(e.palette.primary.main,e.palette.action.hoverOpacity),"@media (hover: none)":{backgroundColor:"transparent"}}},textSecondary:{color:e.palette.secondary.main,"&:hover":{backgroundColor:Object(l.d)(e.palette.secondary.main,e.palette.action.hoverOpacity),"@media (hover: none)":{backgroundColor:"transparent"}}},outlined:{padding:"5px 15px",border:"1px solid ".concat("light"===e.palette.type?"rgba(0, 0, 0, 0.23)":"rgba(255, 255, 255, 0.23)"),"&$disabled":{border:"1px solid ".concat(e.palette.action.disabledBackground)}},outlinedPrimary:{color:e.palette.primary.main,border:"1px solid ".concat(Object(l.d)(e.palette.primary.main,.5)),"&:hover":{border:"1px solid ".concat(e.palette.primary.main),backgroundColor:Object(l.d)(e.palette.primary.main,e.palette.action.hoverOpacity),"@media (hover: none)":{backgroundColor:"transparent"}}},outlinedSecondary:{color:e.palette.secondary.main,border:"1px solid ".concat(Object(l.d)(e.palette.secondary.main,.5)),"&:hover":{border:"1px solid ".concat(e.palette.secondary.main),backgroundColor:Object(l.d)(e.palette.secondary.main,e.palette.action.hoverOpacity),"@media (hover: none)":{backgroundColor:"transparent"}},"&$disabled":{border:"1px solid ".concat(e.palette.action.disabled)}},contained:{color:e.palette.getContrastText(e.palette.grey[300]),backgroundColor:e.palette.grey[300],boxShadow:e.shadows[2],"&:hover":{backgroundColor:e.palette.grey.A100,boxShadow:e.shadows[4],"@media (hover: none)":{boxShadow:e.shadows[2],backgroundColor:e.palette.grey[300]},"&$disabled":{backgroundColor:e.palette.action.disabledBackground}},"&$focusVisible":{boxShadow:e.shadows[6]},"&:active":{boxShadow:e.shadows[8]},"&$disabled":{color:e.palette.action.disabled,boxShadow:e.shadows[0],backgroundColor:e.palette.action.disabledBackground}},containedPrimary:{color:e.palette.primary.contrastText,backgroundColor:e.palette.primary.main,"&:hover":{backgroundColor:e.palette.primary.dark,"@media (hover: none)":{backgroundColor:e.palette.primary.main}}},containedSecondary:{color:e.palette.secondary.contrastText,backgroundColor:e.palette.secondary.main,"&:hover":{backgroundColor:e.palette.secondary.dark,"@media (hover: none)":{backgroundColor:e.palette.secondary.main}}},disableElevation:{boxShadow:"none","&:hover":{boxShadow:"none"},"&$focusVisible":{boxShadow:"none"},"&:active":{boxShadow:"none"},"&$disabled":{boxShadow:"none"}},focusVisible:{},disabled:{},colorInherit:{color:"inherit",borderColor:"currentColor"},textSizeSmall:{padding:"4px 5px",fontSize:e.typography.pxToRem(13)},textSizeLarge:{padding:"8px 11px",fontSize:e.typography.pxToRem(15)},outlinedSizeSmall:{padding:"3px 9px",fontSize:e.typography.pxToRem(13)},outlinedSizeLarge:{padding:"7px 21px",fontSize:e.typography.pxToRem(15)},containedSizeSmall:{padding:"4px 10px",fontSize:e.typography.pxToRem(13)},containedSizeLarge:{padding:"8px 22px",fontSize:e.typography.pxToRem(15)},sizeSmall:{},sizeLarge:{},fullWidth:{width:"100%"},startIcon:{display:"inherit",marginRight:8,marginLeft:-4,"&$iconSizeSmall":{marginLeft:-2}},endIcon:{display:"inherit",marginRight:-4,marginLeft:8,"&$iconSizeSmall":{marginRight:-2}},iconSizeSmall:{"& > *:first-child":{fontSize:18}},iconSizeMedium:{"& > *:first-child":{fontSize:20}},iconSizeLarge:{"& > *:first-child":{fontSize:22}}}}),{name:"MuiButton"})(d)},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(4)),s=n(7),l=n(15),u=o.forwardRef((function(e,t){var n=e.classes,s=e.className,u=e.color,c=void 0===u?"primary":u,d=e.disableShrink,h=void 0!==d&&d,f=e.size,p=void 0===f?40:f,g=e.style,v=e.thickness,m=void 0===v?3.6:v,y=e.value,b=void 0===y?0:y,C=e.variant,A=void 0===C?"indeterminate":C,_=Object(i.a)(e,["classes","className","color","disableShrink","size","style","thickness","value","variant"]),I={},x={},w={};if("determinate"===A||"static"===A){var M=2*Math.PI*((44-m)/2);I.strokeDasharray=M.toFixed(3),w["aria-valuenow"]=Math.round(b),I.strokeDashoffset="".concat(((100-b)/100*M).toFixed(3),"px"),x.transform="rotate(-90deg)"}return o.createElement("div",Object(r.a)({className:Object(a.a)(n.root,s,"inherit"!==c&&n["color".concat(Object(l.a)(c))],{determinate:n.determinate,indeterminate:n.indeterminate,static:n.static}[A]),style:Object(r.a)({width:p,height:p},x,g),ref:t,role:"progressbar"},w,_),o.createElement("svg",{className:n.svg,viewBox:"".concat(22," ").concat(22," ").concat(44," ").concat(44)},o.createElement("circle",{className:Object(a.a)(n.circle,h&&n.circleDisableShrink,{determinate:n.circleDeterminate,indeterminate:n.circleIndeterminate,static:n.circleStatic}[A]),style:I,cx:44,cy:44,r:(44-m)/2,fill:"none",strokeWidth:m})))}));t.a=Object(s.a)((function(e){return{root:{display:"inline-block"},static:{transition:e.transitions.create("transform")},indeterminate:{animation:"$circular-rotate 1.4s linear infinite"},determinate:{transition:e.transitions.create("transform")},colorPrimary:{color:e.palette.primary.main},colorSecondary:{color:e.palette.secondary.main},svg:{display:"block"},circle:{stroke:"currentColor"},circleStatic:{transition:e.transitions.create("stroke-dashoffset")},circleIndeterminate:{animation:"$circular-dash 1.4s ease-in-out infinite",strokeDasharray:"80px, 200px",strokeDashoffset:"0px"},circleDeterminate:{transition:e.transitions.create("stroke-dashoffset")},"@keyframes circular-rotate":{"0%":{transformOrigin:"50% 50%"},"100%":{transform:"rotate(360deg)"}},"@keyframes circular-dash":{"0%":{strokeDasharray:"1px, 200px",strokeDashoffset:"0px"},"50%":{strokeDasharray:"100px, 200px",strokeDashoffset:"-15px"},"100%":{strokeDasharray:"100px, 200px",strokeDashoffset:"-125px"}},circleDisableShrink:{animation:"none"}}}),{name:"MuiCircularProgress",flip:!1})(u)},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(21),a=n(0),s=(n(3),n(4)),l=n(7),u=n(15),c=n(462),d=n(2353),h=n(1310),f=n(65),p=n(336),g={enter:f.b.enteringScreen,exit:f.b.leavingScreen},v=a.forwardRef((function(e,t){var n=e.BackdropProps,o=e.children,l=e.classes,f=e.className,v=e.disableBackdropClick,m=void 0!==v&&v,y=e.disableEscapeKeyDown,b=void 0!==y&&y,C=e.fullScreen,A=void 0!==C&&C,_=e.fullWidth,I=void 0!==_&&_,x=e.maxWidth,w=void 0===x?"sm":x,M=e.onBackdropClick,S=e.onClose,T=e.onEnter,E=e.onEntered,O=e.onEntering,k=e.onEscapeKeyDown,L=e.onExit,D=e.onExited,P=e.onExiting,N=e.open,j=e.PaperComponent,R=void 0===j?p.a:j,z=e.PaperProps,F=void 0===z?{}:z,B=e.scroll,H=void 0===B?"paper":B,U=e.TransitionComponent,V=void 0===U?h.a:U,Y=e.transitionDuration,W=void 0===Y?g:Y,G=e.TransitionProps,X=e["aria-describedby"],q=e["aria-labelledby"],Z=Object(i.a)(e,["BackdropProps","children","classes","className","disableBackdropClick","disableEscapeKeyDown","fullScreen","fullWidth","maxWidth","onBackdropClick","onClose","onEnter","onEntered","onEntering","onEscapeKeyDown","onExit","onExited","onExiting","open","PaperComponent","PaperProps","scroll","TransitionComponent","transitionDuration","TransitionProps","aria-describedby","aria-labelledby"]),Q=a.useRef();return a.createElement(c.a,Object(r.a)({className:Object(s.a)(l.root,f),BackdropComponent:d.a,BackdropProps:Object(r.a)({transitionDuration:W},n),closeAfterTransition:!0,disableBackdropClick:m,disableEscapeKeyDown:b,onEscapeKeyDown:k,onClose:S,open:N,ref:t},Z),a.createElement(V,Object(r.a)({appear:!0,in:N,timeout:W,onEnter:T,onEntering:O,onEntered:E,onExit:L,onExiting:P,onExited:D,role:"none presentation"},G),a.createElement("div",{className:Object(s.a)(l.container,l["scroll".concat(Object(u.a)(H))]),onMouseUp:function(e){e.target===e.currentTarget&&e.target===Q.current&&(Q.current=null,M&&M(e),!m&&S&&S(e,"backdropClick"))},onMouseDown:function(e){Q.current=e.target}},a.createElement(R,Object(r.a)({elevation:24,role:"dialog","aria-describedby":X,"aria-labelledby":q},F,{className:Object(s.a)(l.paper,l["paperScroll".concat(Object(u.a)(H))],l["paperWidth".concat(Object(u.a)(String(w)))],F.className,A&&l.paperFullScreen,I&&l.paperFullWidth)}),o))))}));t.a=Object(l.a)((function(e){return{root:{"@media print":{position:"absolute !important"}},scrollPaper:{display:"flex",justifyContent:"center",alignItems:"center"},scrollBody:{overflowY:"auto",overflowX:"hidden",textAlign:"center","&:after":{content:'""',display:"inline-block",verticalAlign:"middle",height:"100%",width:"0"}},container:{height:"100%","@media print":{height:"auto"},outline:0},paper:{margin:32,position:"relative",overflowY:"auto","@media print":{overflowY:"visible",boxShadow:"none"}},paperScrollPaper:{display:"flex",flexDirection:"column",maxHeight:"calc(100% - 64px)"},paperScrollBody:{display:"inline-block",verticalAlign:"middle",textAlign:"left"},paperWidthFalse:{maxWidth:"calc(100% - 64px)"},paperWidthXs:{maxWidth:Math.max(e.breakpoints.values.xs,444),"&$paperScrollBody":Object(o.a)({},e.breakpoints.down(Math.max(e.breakpoints.values.xs,444)+64),{maxWidth:"calc(100% - 64px)"})},paperWidthSm:{maxWidth:e.breakpoints.values.sm,"&$paperScrollBody":Object(o.a)({},e.breakpoints.down(e.breakpoints.values.sm+64),{maxWidth:"calc(100% - 64px)"})},paperWidthMd:{maxWidth:e.breakpoints.values.md,"&$paperScrollBody":Object(o.a)({},e.breakpoints.down(e.breakpoints.values.md+64),{maxWidth:"calc(100% - 64px)"})},paperWidthLg:{maxWidth:e.breakpoints.values.lg,"&$paperScrollBody":Object(o.a)({},e.breakpoints.down(e.breakpoints.values.lg+64),{maxWidth:"calc(100% - 64px)"})},paperWidthXl:{maxWidth:e.breakpoints.values.xl,"&$paperScrollBody":Object(o.a)({},e.breakpoints.down(e.breakpoints.values.xl+64),{maxWidth:"calc(100% - 64px)"})},paperFullWidth:{width:"calc(100% - 64px)"},paperFullScreen:{margin:0,width:"100%",maxWidth:"100%",height:"100%",maxHeight:"none",borderRadius:0,"&$paperScrollBody":{margin:0,maxWidth:"100%"}}}}),{name:"MuiDialog"})(v)},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(4)),s=n(7),l=o.forwardRef((function(e,t){var n=e.disableSpacing,s=void 0!==n&&n,l=e.classes,u=e.className,c=Object(i.a)(e,["disableSpacing","classes","className"]);return o.createElement("div",Object(r.a)({className:Object(a.a)(l.root,u,!s&&l.spacing),ref:t},c))}));t.a=Object(s.a)({root:{display:"flex",alignItems:"center",padding:8,justifyContent:"flex-end",flex:"0 0 auto"},spacing:{"& > :not(:first-child)":{marginLeft:8}}},{name:"MuiDialogActions"})(l)},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(4)),s=n(7),l=o.forwardRef((function(e,t){var n=e.classes,s=e.className,l=e.dividers,u=void 0!==l&&l,c=Object(i.a)(e,["classes","className","dividers"]);return o.createElement("div",Object(r.a)({className:Object(a.a)(n.root,s,u&&n.dividers),ref:t},c))}));t.a=Object(s.a)((function(e){return{root:{flex:"1 1 auto",WebkitOverflowScrolling:"touch",overflowY:"auto",padding:"8px 24px","&:first-child":{paddingTop:20}},dividers:{padding:"16px 24px",borderTop:"1px solid ".concat(e.palette.divider),borderBottom:"1px solid ".concat(e.palette.divider)}}}),{name:"MuiDialogContent"})(l)},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(4)),s=n(7),l=n(187),u=o.forwardRef((function(e,t){var n=e.children,s=e.classes,u=e.className,c=e.disableTypography,d=void 0!==c&&c,h=Object(i.a)(e,["children","classes","className","disableTypography"]);return o.createElement("div",Object(r.a)({className:Object(a.a)(s.root,u),ref:t},h),d?n:o.createElement(l.a,{component:"h2",variant:"h6"},n))}));t.a=Object(s.a)({root:{margin:0,padding:"16px 24px",flex:"0 0 auto"}},{name:"MuiDialogTitle"})(u)},function(e,t,n){"use strict";var r=n(5),i=n(1),o=n(0),a=(n(3),n(4)),s=n(7),l=n(231),u=n(15),c=o.forwardRef((function(e,t){var n=e.children,s=e.classes,c=e.className,d=e.color,h=void 0===d?"default":d,f=e.component,p=void 0===f?"button":f,g=e.disabled,v=void 0!==g&&g,m=e.disableFocusRipple,y=void 0!==m&&m,b=e.focusVisibleClassName,C=e.size,A=void 0===C?"large":C,_=e.variant,I=void 0===_?"round":_,x=Object(r.a)(e,["children","classes","className","color","component","disabled","disableFocusRipple","focusVisibleClassName","size","variant"]);return o.createElement(l.a,Object(i.a)({className:Object(a.a)(s.root,c,"round"!==I&&s.extended,"large"!==A&&s["size".concat(Object(u.a)(A))],v&&s.disabled,{primary:s.primary,secondary:s.secondary,inherit:s.colorInherit}[h]),component:p,disabled:v,focusRipple:!y,focusVisibleClassName:Object(a.a)(s.focusVisible,b),ref:t},x),o.createElement("span",{className:s.label},n))}));t.a=Object(s.a)((function(e){return{root:Object(i.a)({},e.typography.button,{boxSizing:"border-box",minHeight:36,transition:e.transitions.create(["background-color","box-shadow","border"],{duration:e.transitions.duration.short}),borderRadius:"50%",padding:0,minWidth:0,width:56,height:56,boxShadow:e.shadows[6],"&:active":{boxShadow:e.shadows[12]},color:e.palette.getContrastText(e.palette.grey[300]),backgroundColor:e.palette.grey[300],"&:hover":{backgroundColor:e.palette.grey.A100,"@media (hover: none)":{backgroundColor:e.palette.grey[300]},"&$disabled":{backgroundColor:e.palette.action.disabledBackground},textDecoration:"none"},"&$focusVisible":{boxShadow:e.shadows[6]},"&$disabled":{color:e.palette.action.disabled,boxShadow:e.shadows[0],backgroundColor:e.palette.action.disabledBackground}}),label:{width:"100%",display:"inherit",alignItems:"inherit",justifyContent:"inherit"},primary:{color:e.palette.primary.contrastText,backgroundColor:e.palette.primary.main,"&:hover":{backgroundColor:e.palette.primary.dark,"@media (hover: none)":{backgroundColor:e.palette.primary.main}}},secondary:{color:e.palette.secondary.contrastText,backgroundColor:e.palette.secondary.main,"&:hover":{backgroundColor:e.palette.secondary.dark,"@media (hover: none)":{backgroundColor:e.palette.secondary.main}}},extended:{borderRadius:24,padding:"0 16px",width:"auto",minHeight:"auto",minWidth:48,height:48,"&$sizeSmall":{width:"auto",padding:"0 8px",borderRadius:17,minWidth:34,height:34},"&$sizeMedium":{width:"auto",padding:"0 16px",borderRadius:20,minWidth:40,height:40}},focusVisible:{},disabled:{},colorInherit:{color:"inherit"},sizeSmall:{width:40,height:40},sizeMedium:{width:48,height:48}}}),{name:"MuiFab"})(c)},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(4)),s=n(106),l=n(7),u=n(187),c=n(15),d=o.forwardRef((function(e,t){e.checked;var n=e.classes,l=e.className,d=e.control,h=e.disabled,f=(e.inputRef,e.label),p=e.labelPlacement,g=void 0===p?"end":p,v=(e.name,e.onChange,e.value,Object(i.a)(e,["checked","classes","className","control","disabled","inputRef","label","labelPlacement","name","onChange","value"])),m=Object(s.a)(),y=h;void 0===y&&void 0!==d.props.disabled&&(y=d.props.disabled),void 0===y&&m&&(y=m.disabled);var b={disabled:y};return["checked","name","onChange","value","inputRef"].forEach((function(t){void 0===d.props[t]&&void 0!==e[t]&&(b[t]=e[t])})),o.createElement("label",Object(r.a)({className:Object(a.a)(n.root,l,"end"!==g&&n["labelPlacement".concat(Object(c.a)(g))],y&&n.disabled),ref:t},v),o.cloneElement(d,b),o.createElement(u.a,{component:"span",className:Object(a.a)(n.label,y&&n.disabled)},f))}));t.a=Object(l.a)((function(e){return{root:{display:"inline-flex",alignItems:"center",cursor:"pointer",verticalAlign:"middle",WebkitTapHighlightColor:"transparent",marginLeft:-11,marginRight:16,"&$disabled":{cursor:"default"}},labelPlacementStart:{flexDirection:"row-reverse",marginLeft:16,marginRight:-11},labelPlacementTop:{flexDirection:"column-reverse",marginLeft:16},labelPlacementBottom:{flexDirection:"column",marginLeft:16},disabled:{},label:{"&$disabled":{color:e.palette.text.disabled}}}}),{name:"MuiFormControlLabel"})(d)},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(4)),s=n(7),l=o.forwardRef((function(e,t){var n=e.classes,s=e.className,l=e.row,u=void 0!==l&&l,c=Object(i.a)(e,["classes","className","row"]);return o.createElement("div",Object(r.a)({className:Object(a.a)(n.root,s,u&&n.row),ref:t},c))}));t.a=Object(s.a)({root:{display:"flex",flexDirection:"column",flexWrap:"wrap"},row:{flexDirection:"row"}},{name:"MuiFormGroup"})(l)},function(e,t,n){"use strict";var r=n(5),i=n(1),o=n(0),a=(n(3),n(4)),s=n(7),l=[0,1,2,3,4,5,6,7,8,9,10],u=["auto",!0,1,2,3,4,5,6,7,8,9,10,11,12];function c(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:1,n=parseFloat(e);return"".concat(n/t).concat(String(e).replace(String(n),"")||"px")}var d=o.forwardRef((function(e,t){var n=e.alignContent,s=void 0===n?"stretch":n,l=e.alignItems,u=void 0===l?"stretch":l,c=e.classes,d=e.className,h=e.component,f=void 0===h?"div":h,p=e.container,g=void 0!==p&&p,v=e.direction,m=void 0===v?"row":v,y=e.item,b=void 0!==y&&y,C=e.justify,A=void 0===C?"flex-start":C,_=e.lg,I=void 0!==_&&_,x=e.md,w=void 0!==x&&x,M=e.sm,S=void 0!==M&&M,T=e.spacing,E=void 0===T?0:T,O=e.wrap,k=void 0===O?"wrap":O,L=e.xl,D=void 0!==L&&L,P=e.xs,N=void 0!==P&&P,j=e.zeroMinWidth,R=void 0!==j&&j,z=Object(r.a)(e,["alignContent","alignItems","classes","className","component","container","direction","item","justify","lg","md","sm","spacing","wrap","xl","xs","zeroMinWidth"]),F=Object(a.a)(c.root,d,g&&[c.container,0!==E&&c["spacing-xs-".concat(String(E))]],b&&c.item,R&&c.zeroMinWidth,"row"!==m&&c["direction-xs-".concat(String(m))],"wrap"!==k&&c["wrap-xs-".concat(String(k))],"stretch"!==u&&c["align-items-xs-".concat(String(u))],"stretch"!==s&&c["align-content-xs-".concat(String(s))],"flex-start"!==A&&c["justify-xs-".concat(String(A))],!1!==N&&c["grid-xs-".concat(String(N))],!1!==S&&c["grid-sm-".concat(String(S))],!1!==w&&c["grid-md-".concat(String(w))],!1!==I&&c["grid-lg-".concat(String(I))],!1!==D&&c["grid-xl-".concat(String(D))]);return o.createElement(f,Object(i.a)({className:F,ref:t},z))})),h=Object(s.a)((function(e){return Object(i.a)({root:{},container:{boxSizing:"border-box",display:"flex",flexWrap:"wrap",width:"100%"},item:{boxSizing:"border-box",margin:"0"},zeroMinWidth:{minWidth:0},"direction-xs-column":{flexDirection:"column"},"direction-xs-column-reverse":{flexDirection:"column-reverse"},"direction-xs-row-reverse":{flexDirection:"row-reverse"},"wrap-xs-nowrap":{flexWrap:"nowrap"},"wrap-xs-wrap-reverse":{flexWrap:"wrap-reverse"},"align-items-xs-center":{alignItems:"center"},"align-items-xs-flex-start":{alignItems:"flex-start"},"align-items-xs-flex-end":{alignItems:"flex-end"},"align-items-xs-baseline":{alignItems:"baseline"},"align-content-xs-center":{alignContent:"center"},"align-content-xs-flex-start":{alignContent:"flex-start"},"align-content-xs-flex-end":{alignContent:"flex-end"},"align-content-xs-space-between":{alignContent:"space-between"},"align-content-xs-space-around":{alignContent:"space-around"},"justify-xs-center":{justifyContent:"center"},"justify-xs-flex-end":{justifyContent:"flex-end"},"justify-xs-space-between":{justifyContent:"space-between"},"justify-xs-space-around":{justifyContent:"space-around"},"justify-xs-space-evenly":{justifyContent:"space-evenly"}},function(e,t){var n={};return l.forEach((function(r){var i=e.spacing(r);0!==i&&(n["spacing-".concat(t,"-").concat(r)]={margin:"-".concat(c(i,2)),width:"calc(100% + ".concat(c(i),")"),"& > $item":{padding:c(i,2)}})})),n}(e,"xs"),e.breakpoints.keys.reduce((function(t,n){return function(e,t,n){var r={};u.forEach((function(e){var t="grid-".concat(n,"-").concat(e);if(!0!==e)if("auto"!==e){var i="".concat(Math.round(e/12*1e8)/1e6,"%");r[t]={flexBasis:i,flexGrow:0,maxWidth:i}}else r[t]={flexBasis:"auto",flexGrow:0,maxWidth:"none"};else r[t]={flexBasis:0,flexGrow:1,maxWidth:"100%"}})),"xs"===n?Object(i.a)(e,r):e[t.breakpoints.up(n)]=r}(t,e,n),t}),{}))}),{name:"MuiGrid"})(d);t.a=h},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(4)),s=n(7),l=n(15),u=o.forwardRef((function(e,t){var n=e.classes,s=e.className,u=e.color,c=void 0===u?"inherit":u,d=e.component,h=void 0===d?"span":d,f=e.fontSize,p=void 0===f?"default":f,g=Object(i.a)(e,["classes","className","color","component","fontSize"]);return o.createElement(h,Object(r.a)({className:Object(a.a)("material-icons",n.root,s,"inherit"!==c&&n["color".concat(Object(l.a)(c))],"default"!==p&&n["fontSize".concat(Object(l.a)(p))]),"aria-hidden":!0,ref:t},g))}));u.muiName="Icon",t.a=Object(s.a)((function(e){return{root:{userSelect:"none",fontSize:e.typography.pxToRem(24),width:"1em",height:"1em",overflow:"hidden",flexShrink:0},colorPrimary:{color:e.palette.primary.main},colorSecondary:{color:e.palette.secondary.main},colorAction:{color:e.palette.action.active},colorError:{color:e.palette.error.main},colorDisabled:{color:e.palette.action.disabled},fontSizeInherit:{fontSize:"inherit"},fontSizeSmall:{fontSize:e.typography.pxToRem(20)},fontSizeLarge:{fontSize:e.typography.pxToRem(36)}}}),{name:"MuiIcon"})(u)},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(4)),s=n(187),l=n(7),u=n(169),c=o.forwardRef((function(e,t){var n=e.children,l=e.classes,c=e.className,d=e.component,h=void 0===d?"div":d,f=e.disablePointerEvents,p=void 0!==f&&f,g=e.disableTypography,v=void 0!==g&&g,m=e.position,y=e.variant,b=Object(i.a)(e,["children","classes","className","component","disablePointerEvents","disableTypography","position","variant"]),C=Object(u.b)()||{},A=y;return y&&C.variant,C&&!A&&(A=C.variant),o.createElement(u.a.Provider,{value:null},o.createElement(h,Object(r.a)({className:Object(a.a)(l.root,c,p&&l.disablePointerEvents,C.hiddenLabel&&l.hiddenLabel,"filled"===A&&l.filled,{start:l.positionStart,end:l.positionEnd}[m],"dense"===C.margin&&l.marginDense),ref:t},b),"string"!=typeof n||v?n:o.createElement(s.a,{color:"textSecondary"},n)))}));t.a=Object(l.a)({root:{display:"flex",height:"0.01em",maxHeight:"2em",alignItems:"center",whiteSpace:"nowrap"},filled:{"&$positionStart:not($hiddenLabel)":{marginTop:16}},positionStart:{marginRight:8},positionEnd:{marginLeft:8},disablePointerEvents:{pointerEvents:"none"},hiddenLabel:{},marginDense:{}},{name:"MuiInputAdornment"})(c)},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(4)),s=n(15),l=n(7),u=n(26),c=n(52),d=o.forwardRef((function(e,t){var n=e.classes,l=e.className,u=e.color,d=void 0===u?"primary":u,h=e.value,f=e.valueBuffer,p=e.variant,g=void 0===p?"indeterminate":p,v=Object(i.a)(e,["classes","className","color","value","valueBuffer","variant"]),m=Object(c.a)(),y={},b={bar1:{},bar2:{}};if("determinate"===g||"buffer"===g)if(void 0!==h){y["aria-valuenow"]=Math.round(h),y["aria-valuemin"]=0,y["aria-valuemax"]=100;var C=h-100;"rtl"===m.direction&&(C=-C),b.bar1.transform="translateX(".concat(C,"%)")}else 0;if("buffer"===g)if(void 0!==f){var A=(f||0)-100;"rtl"===m.direction&&(A=-A),b.bar2.transform="translateX(".concat(A,"%)")}else 0;return o.createElement("div",Object(r.a)({className:Object(a.a)(n.root,n["color".concat(Object(s.a)(d))],l,{determinate:n.determinate,indeterminate:n.indeterminate,buffer:n.buffer,query:n.query}[g]),role:"progressbar"},y,{ref:t},v),"buffer"===g?o.createElement("div",{className:Object(a.a)(n.dashed,n["dashedColor".concat(Object(s.a)(d))])}):null,o.createElement("div",{className:Object(a.a)(n.bar,n["barColor".concat(Object(s.a)(d))],("indeterminate"===g||"query"===g)&&n.bar1Indeterminate,{determinate:n.bar1Determinate,buffer:n.bar1Buffer}[g]),style:b.bar1}),"determinate"===g?null:o.createElement("div",{className:Object(a.a)(n.bar,("indeterminate"===g||"query"===g)&&n.bar2Indeterminate,"buffer"===g?[n["color".concat(Object(s.a)(d))],n.bar2Buffer]:n["barColor".concat(Object(s.a)(d))]),style:b.bar2}))}));t.a=Object(l.a)((function(e){var t=function(t){return"light"===e.palette.type?Object(u.i)(t,.62):Object(u.a)(t,.5)},n=t(e.palette.primary.main),r=t(e.palette.secondary.main);return{root:{position:"relative",overflow:"hidden",height:4,"@media print":{colorAdjust:"exact"}},colorPrimary:{backgroundColor:n},colorSecondary:{backgroundColor:r},determinate:{},indeterminate:{},buffer:{backgroundColor:"transparent"},query:{transform:"rotate(180deg)"},dashed:{position:"absolute",marginTop:0,height:"100%",width:"100%",animation:"$buffer 3s infinite linear"},dashedColorPrimary:{backgroundImage:"radial-gradient(".concat(n," 0%, ").concat(n," 16%, transparent 42%)"),backgroundSize:"10px 10px",backgroundPosition:"0 -23px"},dashedColorSecondary:{backgroundImage:"radial-gradient(".concat(r," 0%, ").concat(r," 16%, transparent 42%)"),backgroundSize:"10px 10px",backgroundPosition:"0 -23px"},bar:{width:"100%",position:"absolute",left:0,bottom:0,top:0,transition:"transform 0.2s linear",transformOrigin:"left"},barColorPrimary:{backgroundColor:e.palette.primary.main},barColorSecondary:{backgroundColor:e.palette.secondary.main},bar1Indeterminate:{width:"auto",animation:"$indeterminate1 2.1s cubic-bezier(0.65, 0.815, 0.735, 0.395) infinite"},bar1Determinate:{transition:"transform .".concat(4,"s linear")},bar1Buffer:{zIndex:1,transition:"transform .".concat(4,"s linear")},bar2Indeterminate:{width:"auto",animation:"$indeterminate2 2.1s cubic-bezier(0.165, 0.84, 0.44, 1) 1.15s infinite"},bar2Buffer:{transition:"transform .".concat(4,"s linear")},"@keyframes indeterminate1":{"0%":{left:"-35%",right:"100%"},"60%":{left:"100%",right:"-90%"},"100%":{left:"100%",right:"-90%"}},"@keyframes indeterminate2":{"0%":{left:"-200%",right:"100%"},"60%":{left:"107%",right:"-8%"},"100%":{left:"107%",right:"-8%"}},"@keyframes buffer":{"0%":{opacity:1,backgroundPosition:"0 -23px"},"50%":{opacity:0,backgroundPosition:"0 -23px"},"100%":{opacity:1,backgroundPosition:"-200px -23px"}}}}),{name:"MuiLinearProgress"})(d)},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(4)),s=n(7),l=n(187),u=n(143),c=o.forwardRef((function(e,t){var n=e.children,s=e.classes,c=e.className,d=e.disableTypography,h=void 0!==d&&d,f=e.inset,p=void 0!==f&&f,g=e.primary,v=e.primaryTypographyProps,m=e.secondary,y=e.secondaryTypographyProps,b=Object(i.a)(e,["children","classes","className","disableTypography","inset","primary","primaryTypographyProps","secondary","secondaryTypographyProps"]),C=o.useContext(u.a).dense,A=null!=g?g:n;null==A||A.type===l.a||h||(A=o.createElement(l.a,Object(r.a)({variant:C?"body2":"body1",className:s.primary,component:"span",display:"block"},v),A));var _=m;return null==_||_.type===l.a||h||(_=o.createElement(l.a,Object(r.a)({variant:"body2",className:s.secondary,color:"textSecondary",display:"block"},y),_)),o.createElement("div",Object(r.a)({className:Object(a.a)(s.root,c,C&&s.dense,p&&s.inset,A&&_&&s.multiline),ref:t},b),A,_)}));t.a=Object(s.a)({root:{flex:"1 1 auto",minWidth:0,marginTop:4,marginBottom:4},multiline:{marginTop:6,marginBottom:6},dense:{},inset:{paddingLeft:56},primary:{},secondary:{}},{name:"MuiListItemText"})(c)},function(e,t,n){"use strict";var r=n(5),i=n(21),o=n(1),a=n(0),s=(n(3),n(4)),l=n(7),u=n(2360),c=a.forwardRef((function(e,t){var n,i=e.classes,l=e.className,c=e.component,d=void 0===c?"li":c,h=e.disableGutters,f=void 0!==h&&h,p=e.ListItemClasses,g=e.role,v=void 0===g?"menuitem":g,m=e.selected,y=e.tabIndex,b=Object(r.a)(e,["classes","className","component","disableGutters","ListItemClasses","role","selected","tabIndex"]);return e.disabled||(n=void 0!==y?y:-1),a.createElement(u.a,Object(o.a)({button:!0,role:v,tabIndex:n,component:d,selected:m,disableGutters:f,classes:Object(o.a)({dense:i.dense},p),className:Object(s.a)(i.root,l,m&&i.selected,!f&&i.gutters),ref:t},b))}));t.a=Object(l.a)((function(e){return{root:Object(o.a)({},e.typography.body1,Object(i.a)({minHeight:48,paddingTop:6,paddingBottom:6,boxSizing:"border-box",width:"auto",overflow:"hidden",whiteSpace:"nowrap"},e.breakpoints.up("sm"),{minHeight:"auto"})),gutters:{},selected:{},dense:Object(o.a)({},e.typography.body2,{minHeight:"auto"})}}),{name:"MuiMenuItem"})(c)},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(4)),s=n(7),l=n(26),u=n(15),c=n(274),d=o.forwardRef((function(e,t){var n=e.classes,s=e.className,l=e.color,d=void 0===l?"secondary":l,h=e.edge,f=void 0!==h&&h,p=e.size,g=void 0===p?"medium":p,v=Object(i.a)(e,["classes","className","color","edge","size"]),m=o.createElement("span",{className:n.thumb});return o.createElement("span",{className:Object(a.a)(n.root,s,{start:n.edgeStart,end:n.edgeEnd}[f],"small"===g&&n["size".concat(Object(u.a)(g))])},o.createElement(c.a,Object(r.a)({type:"checkbox",icon:m,checkedIcon:m,classes:{root:Object(a.a)(n.switchBase,n["color".concat(Object(u.a)(d))]),input:n.input,checked:n.checked,disabled:n.disabled},ref:t},v)),o.createElement("span",{className:n.track}))}));t.a=Object(s.a)((function(e){return{root:{display:"inline-flex",width:58,height:38,overflow:"hidden",padding:12,boxSizing:"border-box",position:"relative",flexShrink:0,zIndex:0,verticalAlign:"middle","@media print":{colorAdjust:"exact"}},edgeStart:{marginLeft:-8},edgeEnd:{marginRight:-8},switchBase:{position:"absolute",top:0,left:0,zIndex:1,color:"light"===e.palette.type?e.palette.grey[50]:e.palette.grey[400],transition:e.transitions.create(["left","transform"],{duration:e.transitions.duration.shortest}),"&$checked":{transform:"translateX(20px)"},"&$disabled":{color:"light"===e.palette.type?e.palette.grey[400]:e.palette.grey[800]},"&$checked + $track":{opacity:.5},"&$disabled + $track":{opacity:"light"===e.palette.type?.12:.1}},colorPrimary:{"&$checked":{color:e.palette.primary.main,"&:hover":{backgroundColor:Object(l.d)(e.palette.primary.main,e.palette.action.hoverOpacity),"@media (hover: none)":{backgroundColor:"transparent"}}},"&$disabled":{color:"light"===e.palette.type?e.palette.grey[400]:e.palette.grey[800]},"&$checked + $track":{backgroundColor:e.palette.primary.main},"&$disabled + $track":{backgroundColor:"light"===e.palette.type?e.palette.common.black:e.palette.common.white}},colorSecondary:{"&$checked":{color:e.palette.secondary.main,"&:hover":{backgroundColor:Object(l.d)(e.palette.secondary.main,e.palette.action.hoverOpacity),"@media (hover: none)":{backgroundColor:"transparent"}}},"&$disabled":{color:"light"===e.palette.type?e.palette.grey[400]:e.palette.grey[800]},"&$checked + $track":{backgroundColor:e.palette.secondary.main},"&$disabled + $track":{backgroundColor:"light"===e.palette.type?e.palette.common.black:e.palette.common.white}},sizeSmall:{width:40,height:24,padding:7,"& $thumb":{width:16,height:16},"& $switchBase":{padding:4,"&$checked":{transform:"translateX(16px)"}}},checked:{},disabled:{},input:{left:"-100%",width:"300%"},thumb:{boxShadow:e.shadows[1],backgroundColor:"currentColor",width:20,height:20,borderRadius:"50%"},track:{height:"100%",width:"100%",borderRadius:7,zIndex:-1,transition:e.transitions.create(["opacity","background-color"],{duration:e.transitions.duration.shortest}),backgroundColor:"light"===e.palette.type?e.palette.common.black:e.palette.common.white,opacity:"light"===e.palette.type?.38:.3}}}),{name:"MuiSwitch"})(d)},function(e,t,n){"use strict";var r=n(5),i=n(1),o=n(0),a=(n(3),n(4)),s=n(7),l=n(457),u=o.forwardRef((function(e,t){var n=e.classes,s=e.className,u=e.component,c=void 0===u?"table":u,d=e.padding,h=void 0===d?"default":d,f=e.size,p=void 0===f?"medium":f,g=e.stickyHeader,v=void 0!==g&&g,m=Object(r.a)(e,["classes","className","component","padding","size","stickyHeader"]),y=o.useMemo((function(){return{padding:h,size:p,stickyHeader:v}}),[h,p,v]);return o.createElement(l.a.Provider,{value:y},o.createElement(c,Object(i.a)({role:"table"===c?null:"table",ref:t,className:Object(a.a)(n.root,s,v&&n.stickyHeader)},m)))}));t.a=Object(s.a)((function(e){return{root:{display:"table",width:"100%",borderCollapse:"collapse",borderSpacing:0,"& caption":Object(i.a)({},e.typography.body2,{padding:e.spacing(2),color:e.palette.text.secondary,textAlign:"left",captionSide:"bottom"})},stickyHeader:{borderCollapse:"separate"}}}),{name:"MuiTable"})(u)},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(4)),s=n(7),l=n(168),u={variant:"body"},c=o.forwardRef((function(e,t){var n=e.classes,s=e.className,c=e.component,d=void 0===c?"tbody":c,h=Object(i.a)(e,["classes","className","component"]);return o.createElement(l.a.Provider,{value:u},o.createElement(d,Object(r.a)({className:Object(a.a)(n.root,s),ref:t,role:"tbody"===d?null:"rowgroup"},h)))}));t.a=Object(s.a)({root:{display:"table-row-group"}},{name:"MuiTableBody"})(c)},function(e,t,n){"use strict";var r=n(5),i=n(1),o=n(0),a=(n(3),n(4)),s=n(7),l=n(15),u=n(26),c=n(457),d=n(168),h=o.forwardRef((function(e,t){var n,s,u=e.align,h=void 0===u?"inherit":u,f=e.classes,p=e.className,g=e.component,v=e.padding,m=e.scope,y=e.size,b=e.sortDirection,C=e.variant,A=Object(r.a)(e,["align","classes","className","component","padding","scope","size","sortDirection","variant"]),_=o.useContext(c.a),I=o.useContext(d.a),x=I&&"head"===I.variant;g?(s=g,n=x?"columnheader":"cell"):s=x?"th":"td";var w=m;!w&&x&&(w="col");var M=v||(_&&_.padding?_.padding:"default"),S=y||(_&&_.size?_.size:"medium"),T=C||I&&I.variant,E=null;return b&&(E="asc"===b?"ascending":"descending"),o.createElement(s,Object(i.a)({ref:t,className:Object(a.a)(f.root,f[T],p,"inherit"!==h&&f["align".concat(Object(l.a)(h))],"default"!==M&&f["padding".concat(Object(l.a)(M))],"medium"!==S&&f["size".concat(Object(l.a)(S))],"head"===T&&_&&_.stickyHeader&&f.stickyHeader),"aria-sort":E,role:n,scope:w},A))}));t.a=Object(s.a)((function(e){return{root:Object(i.a)({},e.typography.body2,{display:"table-cell",verticalAlign:"inherit",borderBottom:"1px solid\n    ".concat("light"===e.palette.type?Object(u.i)(Object(u.d)(e.palette.divider,1),.88):Object(u.a)(Object(u.d)(e.palette.divider,1),.68)),textAlign:"left",padding:16}),head:{color:e.palette.text.primary,lineHeight:e.typography.pxToRem(24),fontWeight:e.typography.fontWeightMedium},body:{color:e.palette.text.primary},footer:{color:e.palette.text.secondary,lineHeight:e.typography.pxToRem(21),fontSize:e.typography.pxToRem(12)},sizeSmall:{padding:"6px 24px 6px 16px","&:last-child":{paddingRight:16},"&$paddingCheckbox":{width:24,padding:"0 12px 0 16px","&:last-child":{paddingLeft:12,paddingRight:16},"& > *":{padding:0}}},paddingCheckbox:{width:48,padding:"0 0 0 4px","&:last-child":{paddingLeft:0,paddingRight:4}},paddingNone:{padding:0,"&:last-child":{padding:0}},alignLeft:{textAlign:"left"},alignCenter:{textAlign:"center"},alignRight:{textAlign:"right",flexDirection:"row-reverse"},alignJustify:{textAlign:"justify"},stickyHeader:{position:"sticky",top:0,left:0,zIndex:2,backgroundColor:e.palette.background.default}}}),{name:"MuiTableCell"})(h)},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(4)),s=n(7),l=n(168),u={variant:"footer"},c=o.forwardRef((function(e,t){var n=e.classes,s=e.className,c=e.component,d=void 0===c?"tfoot":c,h=Object(i.a)(e,["classes","className","component"]);return o.createElement(l.a.Provider,{value:u},o.createElement(d,Object(r.a)({className:Object(a.a)(n.root,s),ref:t,role:"tfoot"===d?null:"rowgroup"},h)))}));t.a=Object(s.a)({root:{display:"table-footer-group"}},{name:"MuiTableFooter"})(c)},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(4)),s=n(7),l=n(168),u={variant:"head"},c=o.forwardRef((function(e,t){var n=e.classes,s=e.className,c=e.component,d=void 0===c?"thead":c,h=Object(i.a)(e,["classes","className","component"]);return o.createElement(l.a.Provider,{value:u},o.createElement(d,Object(r.a)({className:Object(a.a)(n.root,s),ref:t,role:"thead"===d?null:"rowgroup"},h)))}));t.a=Object(s.a)({root:{display:"table-header-group"}},{name:"MuiTableHead"})(c)},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(21),a=n(0),s=(n(3),n(4)),l=n(7),u=a.forwardRef((function(e,t){var n=e.classes,o=e.className,l=e.component,u=void 0===l?"div":l,c=e.disableGutters,d=void 0!==c&&c,h=e.variant,f=void 0===h?"regular":h,p=Object(i.a)(e,["classes","className","component","disableGutters","variant"]);return a.createElement(u,Object(r.a)({className:Object(s.a)(n.root,n[f],o,!d&&n.gutters),ref:t},p))}));t.a=Object(l.a)((function(e){return{root:{position:"relative",display:"flex",alignItems:"center"},gutters:Object(o.a)({paddingLeft:e.spacing(2),paddingRight:e.spacing(2)},e.breakpoints.up("sm"),{paddingLeft:e.spacing(3),paddingRight:e.spacing(3)}),regular:e.mixins.toolbar,dense:{minHeight:48}}}),{name:"MuiToolbar"})(u)},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(4)),s=n(7),l=n(168),u=n(26),c=o.forwardRef((function(e,t){var n=e.classes,s=e.className,u=e.component,c=void 0===u?"tr":u,d=e.hover,h=void 0!==d&&d,f=e.selected,p=void 0!==f&&f,g=Object(i.a)(e,["classes","className","component","hover","selected"]),v=o.useContext(l.a);return o.createElement(c,Object(r.a)({ref:t,className:Object(a.a)(n.root,s,v&&{head:n.head,footer:n.footer}[v.variant],h&&n.hover,p&&n.selected),role:"tr"===c?null:"row"},g))}));t.a=Object(s.a)((function(e){return{root:{color:"inherit",display:"table-row",verticalAlign:"middle",outline:0,"&$hover:hover":{backgroundColor:e.palette.action.hover},"&$selected, &$selected:hover":{backgroundColor:Object(u.d)(e.palette.secondary.main,e.palette.action.selectedOpacity)}},selected:{},hover:{},head:{},footer:{}}}),{name:"MuiTableRow"})(c)},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(4)),s=n(454),l=n(2355),u=n(2363),c=n(632),d=n(630),h=n(2357),f=n(639),p=n(7),g={standard:s.a,filled:l.a,outlined:u.a},v=o.forwardRef((function(e,t){var n=e.autoComplete,s=e.autoFocus,l=void 0!==s&&s,u=e.children,p=e.classes,v=e.className,m=e.color,y=void 0===m?"primary":m,b=e.defaultValue,C=e.disabled,A=void 0!==C&&C,_=e.error,I=void 0!==_&&_,x=e.FormHelperTextProps,w=e.fullWidth,M=void 0!==w&&w,S=e.helperText,T=e.hiddenLabel,E=e.id,O=e.InputLabelProps,k=e.inputProps,L=e.InputProps,D=e.inputRef,P=e.label,N=e.multiline,j=void 0!==N&&N,R=e.name,z=e.onBlur,F=e.onChange,B=e.onFocus,H=e.placeholder,U=e.required,V=void 0!==U&&U,Y=e.rows,W=e.rowsMax,G=e.select,X=void 0!==G&&G,q=e.SelectProps,Z=e.type,Q=e.value,K=e.variant,J=void 0===K?"standard":K,$=Object(i.a)(e,["autoComplete","autoFocus","children","classes","className","color","defaultValue","disabled","error","FormHelperTextProps","fullWidth","helperText","hiddenLabel","id","InputLabelProps","inputProps","InputProps","inputRef","label","multiline","name","onBlur","onChange","onFocus","placeholder","required","rows","rowsMax","select","SelectProps","type","value","variant"]);var ee={};if("outlined"===J&&(O&&void 0!==O.shrink&&(ee.notched=O.shrink),P)){var te,ne=null!==(te=null==O?void 0:O.required)&&void 0!==te?te:V;ee.label=o.createElement(o.Fragment,null,P,ne&&" *")}X&&(q&&q.native||(ee.id=void 0),ee["aria-describedby"]=void 0);var re=S&&E?"".concat(E,"-helper-text"):void 0,ie=P&&E?"".concat(E,"-label"):void 0,oe=g[J],ae=o.createElement(oe,Object(r.a)({"aria-describedby":re,autoComplete:n,autoFocus:l,defaultValue:b,fullWidth:M,multiline:j,name:R,rows:Y,rowsMax:W,type:Z,value:Q,id:E,inputRef:D,onBlur:z,onChange:F,onFocus:B,placeholder:H,inputProps:k},ee,L));return o.createElement(d.a,Object(r.a)({className:Object(a.a)(p.root,v),disabled:A,error:I,fullWidth:M,hiddenLabel:T,ref:t,required:V,color:y,variant:J},$),P&&o.createElement(c.a,Object(r.a)({htmlFor:E,id:ie},O),P),X?o.createElement(f.a,Object(r.a)({"aria-describedby":re,id:E,labelId:ie,value:Q,input:ae},q),u):ae,S&&o.createElement(h.a,Object(r.a)({id:re},x),S))}));t.a=Object(p.a)({root:{}},{name:"MuiTextField"})(v)},function(e,t,n){"use strict";var r=n(1),i=n(49),o=n(5),a=n(21),s=n(0),l=n(22),u=(n(3),n(4)),c=n(442),d=n(26),h=n(7),f=n(15),p=n(1312),g=n(2362),v=n(37),m=n(186),y=n(87),b=n(166),C=n(100),A=n(52);function _(e){return Math.round(1e5*e)/1e5}var I=!1,x=null;var w=s.forwardRef((function(e,t){var n=e.arrow,a=void 0!==n&&n,d=e.children,h=e.classes,_=e.disableFocusListener,w=void 0!==_&&_,M=e.disableHoverListener,S=void 0!==M&&M,T=e.disableTouchListener,E=void 0!==T&&T,O=e.enterDelay,k=void 0===O?100:O,L=e.enterNextDelay,D=void 0===L?0:L,P=e.enterTouchDelay,N=void 0===P?700:P,j=e.id,R=e.interactive,z=void 0!==R&&R,F=e.leaveDelay,B=void 0===F?0:F,H=e.leaveTouchDelay,U=void 0===H?1500:H,V=e.onClose,Y=e.onOpen,W=e.open,G=e.placement,X=void 0===G?"bottom":G,q=e.PopperComponent,Z=void 0===q?g.a:q,Q=e.PopperProps,K=e.title,J=e.TransitionComponent,$=void 0===J?p.a:J,ee=e.TransitionProps,te=Object(o.a)(e,["arrow","children","classes","disableFocusListener","disableHoverListener","disableTouchListener","enterDelay","enterNextDelay","enterTouchDelay","id","interactive","leaveDelay","leaveTouchDelay","onClose","onOpen","open","placement","PopperComponent","PopperProps","title","TransitionComponent","TransitionProps"]),ne=Object(A.a)(),re=s.useState(),ie=re[0],oe=re[1],ae=s.useState(null),se=ae[0],le=ae[1],ue=s.useRef(!1),ce=s.useRef(),de=s.useRef(),he=s.useRef(),fe=s.useRef(),pe=Object(C.a)({controlled:W,default:!1,name:"Tooltip",state:"open"}),ge=Object(i.a)(pe,2),ve=ge[0],me=ge[1],ye=ve,be=Object(m.a)(j);s.useEffect((function(){return function(){clearTimeout(ce.current),clearTimeout(de.current),clearTimeout(he.current),clearTimeout(fe.current)}}),[]);var Ce=function(e){clearTimeout(x),I=!0,me(!0),Y&&Y(e)},Ae=function(){var e=!(arguments.length>0&&void 0!==arguments[0])||arguments[0];return function(t){var n=d.props;"mouseover"===t.type&&n.onMouseOver&&e&&n.onMouseOver(t),ue.current&&"touchstart"!==t.type||(ie&&ie.removeAttribute("title"),clearTimeout(de.current),clearTimeout(he.current),k||I&&D?(t.persist(),de.current=setTimeout((function(){Ce(t)}),I?D:k)):Ce(t))}},_e=Object(b.a)(),Ie=_e.isFocusVisible,xe=_e.onBlurVisible,we=_e.ref,Me=s.useState(!1),Se=Me[0],Te=Me[1],Ee=function(){Se&&(Te(!1),xe())},Oe=function(){var e=!(arguments.length>0&&void 0!==arguments[0])||arguments[0];return function(t){ie||oe(t.currentTarget),Ie(t)&&(Te(!0),Ae()(t));var n=d.props;n.onFocus&&e&&n.onFocus(t)}},ke=function(e){clearTimeout(x),x=setTimeout((function(){I=!1}),800+B),me(!1),V&&V(e),clearTimeout(ce.current),ce.current=setTimeout((function(){ue.current=!1}),ne.transitions.duration.shortest)},Le=function(){var e=!(arguments.length>0&&void 0!==arguments[0])||arguments[0];return function(t){var n=d.props;"blur"===t.type&&(n.onBlur&&e&&n.onBlur(t),Ee()),"mouseleave"===t.type&&n.onMouseLeave&&t.currentTarget===ie&&n.onMouseLeave(t),clearTimeout(de.current),clearTimeout(he.current),t.persist(),he.current=setTimeout((function(){ke(t)}),B)}},De=function(e){ue.current=!0;var t=d.props;t.onTouchStart&&t.onTouchStart(e)},Pe=Object(v.a)(oe,t),Ne=Object(v.a)(we,Pe),je=s.useCallback((function(e){Object(y.a)(Ne,l.findDOMNode(e))}),[Ne]),Re=Object(v.a)(d.ref,je);""===K&&(ye=!1);var ze=!ye&&!S,Fe=Object(r.a)({"aria-describedby":ye?be:null,title:ze&&"string"==typeof K?K:null},te,d.props,{className:Object(u.a)(te.className,d.props.className),onTouchStart:De,ref:Re}),Be={};E||(Fe.onTouchStart=function(e){De(e),clearTimeout(he.current),clearTimeout(ce.current),clearTimeout(fe.current),e.persist(),fe.current=setTimeout((function(){Ae()(e)}),N)},Fe.onTouchEnd=function(e){d.props.onTouchEnd&&d.props.onTouchEnd(e),clearTimeout(fe.current),clearTimeout(he.current),e.persist(),he.current=setTimeout((function(){ke(e)}),U)}),S||(Fe.onMouseOver=Ae(),Fe.onMouseLeave=Le(),z&&(Be.onMouseOver=Ae(!1),Be.onMouseLeave=Le(!1))),w||(Fe.onFocus=Oe(),Fe.onBlur=Le(),z&&(Be.onFocus=Oe(!1),Be.onBlur=Le(!1)));var He=s.useMemo((function(){return Object(c.a)({popperOptions:{modifiers:{arrow:{enabled:Boolean(se),element:se}}}},Q)}),[se,Q]);return s.createElement(s.Fragment,null,s.cloneElement(d,Fe),s.createElement(Z,Object(r.a)({className:Object(u.a)(h.popper,z&&h.popperInteractive,a&&h.popperArrow),placement:X,anchorEl:ie,open:!!ie&&ye,id:Fe["aria-describedby"],transition:!0},Be,He),(function(e){var t=e.placement,n=e.TransitionProps;return s.createElement($,Object(r.a)({timeout:ne.transitions.duration.shorter},n,ee),s.createElement("div",{className:Object(u.a)(h.tooltip,h["tooltipPlacement".concat(Object(f.a)(t.split("-")[0]))],ue.current&&h.touch,a&&h.tooltipArrow)},K,a?s.createElement("span",{className:h.arrow,ref:le}):null))})))}));t.a=Object(h.a)((function(e){return{popper:{zIndex:e.zIndex.tooltip,pointerEvents:"none"},popperInteractive:{pointerEvents:"auto"},popperArrow:{'&[x-placement*="bottom"] $arrow':{top:0,left:0,marginTop:"-0.71em",marginLeft:4,marginRight:4,"&::before":{transformOrigin:"0 100%"}},'&[x-placement*="top"] $arrow':{bottom:0,left:0,marginBottom:"-0.71em",marginLeft:4,marginRight:4,"&::before":{transformOrigin:"100% 0"}},'&[x-placement*="right"] $arrow':{left:0,marginLeft:"-0.71em",height:"1em",width:"0.71em",marginTop:4,marginBottom:4,"&::before":{transformOrigin:"100% 100%"}},'&[x-placement*="left"] $arrow':{right:0,marginRight:"-0.71em",height:"1em",width:"0.71em",marginTop:4,marginBottom:4,"&::before":{transformOrigin:"0 0"}}},tooltip:{backgroundColor:Object(d.d)(e.palette.grey[700],.9),borderRadius:e.shape.borderRadius,color:e.palette.common.white,fontFamily:e.typography.fontFamily,padding:"4px 8px",fontSize:e.typography.pxToRem(10),lineHeight:"".concat(_(1.4),"em"),maxWidth:300,wordWrap:"break-word",fontWeight:e.typography.fontWeightMedium},tooltipArrow:{position:"relative",margin:"0"},arrow:{overflow:"hidden",position:"absolute",width:"1em",height:"0.71em",boxSizing:"border-box",color:Object(d.d)(e.palette.grey[700],.9),"&::before":{content:'""',margin:"auto",display:"block",width:"100%",height:"100%",backgroundColor:"currentColor",transform:"rotate(45deg)"}},touch:{padding:"8px 16px",fontSize:e.typography.pxToRem(14),lineHeight:"".concat(_(16/14),"em"),fontWeight:e.typography.fontWeightRegular},tooltipPlacementLeft:Object(a.a)({transformOrigin:"right center",margin:"0 24px "},e.breakpoints.up("sm"),{margin:"0 14px"}),tooltipPlacementRight:Object(a.a)({transformOrigin:"left center",margin:"0 24px"},e.breakpoints.up("sm"),{margin:"0 14px"}),tooltipPlacementTop:Object(a.a)({transformOrigin:"center bottom",margin:"24px 0"},e.breakpoints.up("sm"),{margin:"14px 0"}),tooltipPlacementBottom:Object(a.a)({transformOrigin:"center top",margin:"24px 0"},e.breakpoints.up("sm"),{margin:"14px 0"})}}),{name:"MuiTooltip",flip:!1})(w)},function(e,t){function n(t){return"function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?e.exports=n=function(e){return typeof e}:e.exports=n=function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e},n(t)}e.exports=n},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=function(e,t){this.type=e,this.data=t};t.default=r},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.adjustSelectedIndex=void 0;var r=n(191),i=n(243);t.adjustSelectedIndex=function(e,t){if(void 0!==e&&(e.getType()===r.default.TYPE||e.getType()===i.default.TYPE)){var n=e.getSelected();-1!==n&&(t===n&&e.getChildren().length>0?t>=e.getChildren().length&&e._setSelected(e.getChildren().length-1):t<n?e._setSelected(n-1):t>n||e._setSelected(-1))}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(111),i=n(243),o=function(){function e(e){this._model=e,this._borders=[]}return e._fromJson=function(t,n){var r=new e(n);return r._borders=t.map((function(e){return i.default._fromJson(e,n)})),r},e.prototype.getBorders=function(){return this._borders},e.prototype._forEachNode=function(e){this._borders.forEach((function(t){e(t,0),t.getChildren().forEach((function(t){t._forEachNode(e,1)}))}))},e.prototype._toJson=function(){return this._borders.map((function(e){return e._toJson()}))},e.prototype._layoutBorder=function(e,t){for(var n=e.outer,i=n.height,o=n.width,a=0,s=0,l=0,u=0,c=this._borders.filter((function(e){return e.isShowing()})),d=0,h=c;d<h.length;d++){if((g=h[d]).isShowing()){g._setAdjustedSize(g.getSize());var f=-1!==g.getSelected();g.getLocation().getOrientation()===r.default.HORZ?(s+=g.getBorderBarSize()+this._model.getSplitterSize(),f&&(s+=g.getSize(),u+=g.getSize())):(a+=g.getBorderBarSize()+this._model.getSplitterSize(),f&&(a+=g.getSize(),l+=g.getSize()))}}for(var p=0;s>o&&u>0||a>i&&l>0;){var g;if(-1!==(g=c[p]).getSelected()){var v=g._getAdjustedSize();s>o&&u>0&&g.getLocation().getOrientation()===r.default.HORZ&&v>0?(g._setAdjustedSize(v-1),s--,u--):a>i&&l>0&&g.getLocation().getOrientation()===r.default.VERT&&v>0&&(g._setAdjustedSize(v-1),a--,l--)}p=(p+1)%c.length}return c.forEach((function(n){e.outer=n._layoutBorderOuter(e.outer,t)})),e.inner=e.outer,c.forEach((function(n){e.inner=n._layoutBorderInner(e.inner,t)})),e},e.prototype._findDropTargetNode=function(e,t,n){for(var r=0,i=this._borders;r<i.length;r++){var o=i[r];if(o.isShowing()){var a=o.canDrop(e,t,n);if(void 0!==a)return a}}},e}();t.default=o},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.ICloseType=void 0,function(e){e[e.Visible=1]="Visible",e[e.Always=2]="Always",e[e.Selected=3]="Selected"}(t.ICloseType||(t.ICloseType={}))},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.showPopup=void 0;var r=n(0),i=n(22);t.showPopup=function(e,t,n,a,s){var l=t.ownerDocument,u=t.getBoundingClientRect(),c=e.getBoundingClientRect(),d=l.createElement("div");d.className=s("flexlayout__popup_menu_container"),u.left<c.left+c.width/2?d.style.left=u.left-c.left+"px":d.style.right=c.right-u.right+"px",u.top<c.top+c.height/2?d.style.top=u.top-c.top+"px":d.style.bottom=c.bottom-u.bottom+"px",e.appendChild(d);var h=function(){e.removeChild(d),i.unmountComponentAtNode(d),d.removeEventListener("mouseup",f),l.removeEventListener("mouseup",p)},f=function(e){e.stopPropagation()},p=function(e){h()};d.addEventListener("mouseup",f),l.addEventListener("mouseup",p),i.render(r.createElement(o,{currentDocument:l,onSelect:a,onHide:h,items:n,classNameMapper:s}),d)};var o=function(e){var t=e.items,n=e.onHide,i=e.onSelect,o=e.classNameMapper,a=t.map((function(e){return r.createElement("div",{key:e.index,className:o("flexlayout__popup_menu_item"),onClick:function(t){return function(e,t){i(e),n(),t.stopPropagation()}(e,t)}},e.node._getRenderedName())}));return r.createElement("div",{className:o("flexlayout__popup_menu")},a)}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.useTabOverflow=void 0;var r=n(0),i=n(131),o=n(191),a=n(111);t.useTabOverflow=function(e,t,n){var s=r.useRef(!0),l=r.useRef(new i.default(0,0,0,0)),u=r.useRef(null),c=r.useState(0),d=c[0],h=c[1],f=r.useRef(!1),p=r.useState([]),g=p[0],v=p[1];r.useLayoutEffect((function(){f.current=!1}),[e.getSelectedNode(),e.getRect().width,e.getRect().height]),r.useLayoutEffect((function(){C()})),r.useEffect((function(){var e=u.current;return e.addEventListener("wheel",m),function(){e.removeEventListener("wheel",m)}}),[]);var m=function(e){e.preventDefault()},y=function(e){return t===a.default.HORZ?e.x:e.y},b=function(e){return t===a.default.HORZ?e.getRight():e.getBottom()},C=function(){var r,i=e instanceof o.default?e.getRect():e.getTabHeaderRect(),u=e.getChildren()[e.getChildren().length-1];if(!0===s.current||i.width!==l.current.width||i.height!==l.current.height){l.current=i;var c=!(e instanceof o.default)||!0===e.isEnableTabStrip(),p=b(i)-(r=n.current.getBoundingClientRect(),t===a.default.HORZ?r.width:r.height);if(c&&e.getChildren().length>0){if(0===g.length&&0===d&&b(u.getTabRect())+2<p)return;p-=g.length>0?t===a.default.HORZ?10:0:45;var m=0,C=e.getSelectedNode();if(C&&!f.current){var A=C.getTabRect(),_=y(A)-2,I=b(A)+2;(I>p||_<y(i))&&(_<y(i)&&(m=y(i)-_),I+m>p&&(m=p-I))}for(var x=Math.max(0,p-(b(u.getTabRect())+2+m)),w=Math.min(0,d+m+x),M=w-d,S=[],T=0;T<e.getChildren().length;T++){var E=e.getChildren()[T];(y(E.getTabRect())+M<y(i)||b(E.getTabRect())+M>p)&&S.push({node:E,index:T})}s.current=!1,v(S),h(w)}}else s.current=!0};return{selfRef:u,position:d,userControlledLeft:f,hiddenTabs:g,onMouseWheel:function(e){var t=0;t=Math.abs(e.deltaX)>Math.abs(e.deltaY)?e.deltaX:e.deltaY,1===e.deltaMode&&(t*=40),h(d+t),f.current=!0,e.stopPropagation()}}}},function(e,t,n){"use strict";var r,i=this&&this.__extends||(r=function(e,t){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(e[n]=t[n])})(e,t)},function(e,t){function n(){this.constructor=e}r(e,t),e.prototype=null===t?Object.create(t):(n.prototype=t.prototype,new n)});Object.defineProperty(t,"__esModule",{value:!0}),t.ErrorBoundary=void 0;var o=n(0),a=n(148),s=function(e){function t(t){var n=e.call(this,t)||this;return n.state={hasError:!1},n}return i(t,e),t.getDerivedStateFromError=function(e){return{hasError:!0}},t.prototype.componentDidCatch=function(e,t){console.debug(e),console.debug(t)},t.prototype.render=function(){return this.state.hasError?o.createElement("div",{className:a.CLASSES.FLEXLAYOUT__ERROR_BOUNDARY_CONTAINER},o.createElement("div",{className:a.CLASSES.FLEXLAYOUT__ERROR_BOUNDARY_CONTENT},this.props.message)):this.props.children},t}(o.Component);t.ErrorBoundary=s},function(e,t,n){var r;void 0===(r=function(e,t){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.Action=void 0,function(e){e.SAVE_STATE="SAVE_STATE",e.RESET_STORE="RESET_STORE",e.SWITCH_PANEL="SWITCH_PANEL",e.SELECT_NODE="SELECT_NODE",e.ADD_NODE="ADD_NODE",e.REMOVE_NODE="REMOVE_NODE",e.UPDATE_TREE="UPDATE_TREE",e.ARCH_SAVE_GRAPH_JSON="ARCH_SAVE_GRAPH_JSON",e.ARCH_SAVE_GRAPH_POSITION="ARCH_SAVE_GRAPH_POSITION",e.ARCH_UPDATE_CONNECTION_GRAPH="ARCH_UPDATE_CONNECTION_GRAPH",e.ARCH_FILTER_NODE="ARCH_FILTER_NODE",e.LOCK_GRAPH="LOCK_GRAPH",e.DASHBOARD_ADD_3D_DATA="DASHBOARD_ADD_3D_DATA",e.GET_SERVER_DATA="GET_SERVER_DATA",e.DASHBOARD_TOGGLE_RUN="DASHBOARD_TOGGLE_RUN",e.DASHBOARD_ADD_CONTROLLER="DASHBOARD_ADD_CONTROLLER",e.DASHBOARD_SET_CONTROLLER="DASHBOARD_SET_CONTROLLER",e.DASHBOARD_REMOVE_CONTROLLER="DASHBOARD_REMOVE_CONTROLLER",e.DASHBOARD_UPDATE_CONTROLLER="DASHBOARD_UPDATE_CONTROLLER",e.DASHBOARD_UPDATE_COMPUTED_RESULT="DASHBOARD_UPDATE_COMPUTED_RESULT",e.DASHBOARD_UPDATE_RECORDER_DATA="DASHBOARD_UPDATE_RECORDER_DATA",e.DASHBOARD_UPDATE_DRIVER_DATA="DASHBOARD_UPDATE_DRIVER_DATA"}(t.Action||(t.Action={}))}.apply(t,[n,t]))||(e.exports=r)},function(e,t){e.exports=function(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,r=new Array(t);n<t;n++)r[n]=e[n];return r}},function(e,t,n){var r=n(941);e.exports=function(e,t){if(e){if("string"==typeof e)return r(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?r(e,t):void 0}}},function(e,t,n){"use strict";e.exports=n(1586)},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),function(e){e.SOURCE="SOURCE",e.TARGET="TARGET"}(t.HandlerRole||(t.HandlerRole={}))},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){return null===t?null===e:Array.isArray(e)?e.some((function(e){return e===t})):e===t}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.strictEquality=function(e,t){return e===t},t.areCoordsEqual=function(e,t){return!e&&!t||!(!e||!t)&&(e.x===t.x&&e.y===t.y)},t.areArraysEqual=function(e,n,r){if(void 0===r&&(r=t.strictEquality),e.length!==n.length)return!1;for(var i=0;i<e.length;++i)if(!r(e[i],n[i]))return!1;return!0}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(246);t.NONE=[],t.ALL=[],t.NONE.__IS_NONE__=!0,t.ALL.__IS_ALL__=!0,t.areDirty=function(e,n){return e!==t.NONE&&(e===t.ALL||void 0===n||r.intersection(n,e).length>0)}},function(e,t,n){"use strict";var r,i=this&&this.__extends||(r=function(e,t){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(e,t)},function(e,t){function n(){this.constructor=e}r(e,t),e.prototype=null===t?Object.create(t):(n.prototype=t.prototype,new n)}),o=this&&this.__assign||function(){return(o=Object.assign||function(e){for(var t,n=1,r=arguments.length;n<r;n++)for(var i in t=arguments[n])Object.prototype.hasOwnProperty.call(t,i)&&(e[i]=t[i]);return e}).apply(this,arguments)};Object.defineProperty(t,"__esModule",{value:!0});var a=n(0),s=n(372),l=n(1619),u=n(517),c=(n(212),n(19)),d=n(91),h=n(117);t.default=function(e){var t=e.DecoratedComponent,n=e.createHandler,r=e.createMonitor,f=e.createConnector,p=e.registerHandler,g=e.containerDisplayName,v=e.getType,m=e.collect,y=e.options.arePropsEqual,b=void 0===y?h:y,C=t,A=t.displayName||t.name||"Component",_=function(e){function d(t){var n=e.call(this,t)||this;return n.decoratedRef=a.createRef(),n.handleChange=function(){var e=n.getCurrentState();h(e,n.state)||n.setState(e)},n.disposable=new l.SerialDisposable,n.receiveProps(t),n.dispose(),n}return i(d,e),d.prototype.getHandlerId=function(){return this.handlerId},d.prototype.getDecoratedComponentInstance=function(){return c(this.decoratedRef.current,"In order to access an instance of the decorated component, it must either be a class component or use React.forwardRef()"),this.decoratedRef.current},d.prototype.shouldComponentUpdate=function(e,t){return!b(e,this.props)||!h(t,this.state)},d.prototype.componentDidMount=function(){this.disposable=new l.SerialDisposable,this.currentType=void 0,this.receiveProps(this.props),this.handleChange()},d.prototype.componentDidUpdate=function(e){b(this.props,e)||(this.receiveProps(this.props),this.handleChange())},d.prototype.componentWillUnmount=function(){this.dispose()},d.prototype.receiveProps=function(e){this.handler&&(this.handler.receiveProps(e),this.receiveType(v(e)))},d.prototype.receiveType=function(e){if(this.handlerMonitor&&this.manager&&this.handlerConnector&&e!==this.currentType){this.currentType=e;var t=p(e,this.handler,this.manager),n=t[0],r=t[1];this.handlerId=n,this.handlerMonitor.receiveHandlerId(n),this.handlerConnector.receiveHandlerId(n);var i=this.manager.getMonitor().subscribeToStateChange(this.handleChange,{handlerIds:[n]});this.disposable.setDisposable(new l.CompositeDisposable(new l.Disposable(i),new l.Disposable(r)))}},d.prototype.dispose=function(){this.disposable.dispose(),this.handlerConnector&&this.handlerConnector.receiveHandlerId(null)},d.prototype.getCurrentState=function(){return this.handlerConnector?m(this.handlerConnector.hooks,this.handlerMonitor,this.props):{}},d.prototype.render=function(){var e=this;return a.createElement(s.Consumer,null,(function(t){var n=t.dragDropManager;return e.receiveDragDropManager(n),"undefined"!=typeof requestAnimationFrame&&requestAnimationFrame((function(){return e.handlerConnector.reconnect()})),a.createElement(C,o({},e.props,e.getCurrentState(),{ref:u.isRefable(C)?e.decoratedRef:null}))}))},d.prototype.receiveDragDropManager=function(e){void 0===this.manager&&(c(void 0!==e,"Could not find the drag and drop manager in the context of %s. Make sure to wrap the top-level component of your app with DragDropContext. Read more: http://react-dnd.github.io/react-dnd/docs/troubleshooting#could-not-find-the-drag-and-drop-manager-in-the-context",A,A),void 0!==e&&(this.manager=e,this.handlerMonitor=r(e),this.handlerConnector=f(e.getBackend()),this.handler=n(this.handlerMonitor,this.decoratedRef)))},d.DecoratedComponent=t,d.displayName=g+"("+A+")",d}(a.Component);return d(_,t)}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t,n){var r=n.getRegistry(),i=r.addSource(e,t);return[i,function(){return r.removeSource(i)}]}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.getDecoratedComponent=function(e){var t=e.current;return null==t?null:t.decoratedRef?t.decoratedRef.current:t}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(19),i=!1,o=!1,a=function(){function e(e){this.sourceId=null,this.internalMonitor=e.getMonitor()}return e.prototype.receiveHandlerId=function(e){this.sourceId=e},e.prototype.getHandlerId=function(){return this.sourceId},e.prototype.canDrag=function(){r(!i,"You may not call monitor.canDrag() inside your canDrag() implementation. Read more: http://react-dnd.github.io/react-dnd/docs/api/drag-source-monitor");try{return i=!0,this.internalMonitor.canDragSource(this.sourceId)}finally{i=!1}},e.prototype.isDragging=function(){if(!this.sourceId)return!1;r(!o,"You may not call monitor.isDragging() inside your isDragging() implementation. Read more: http://react-dnd.github.io/react-dnd/docs/api/drag-source-monitor");try{return o=!0,this.internalMonitor.isDraggingSource(this.sourceId)}finally{o=!1}},e.prototype.subscribeToStateChange=function(e,t){return this.internalMonitor.subscribeToStateChange(e,t)},e.prototype.isDraggingSource=function(e){return this.internalMonitor.isDraggingSource(e)},e.prototype.isOverTarget=function(e,t){return this.internalMonitor.isOverTarget(e,t)},e.prototype.getTargetIds=function(){return this.internalMonitor.getTargetIds()},e.prototype.isSourcePublic=function(){return this.internalMonitor.isSourcePublic()},e.prototype.getSourceId=function(){return this.internalMonitor.getSourceId()},e.prototype.subscribeToOffsetChange=function(e){return this.internalMonitor.subscribeToOffsetChange(e)},e.prototype.canDragSource=function(e){return this.internalMonitor.canDragSource(e)},e.prototype.canDropOnTarget=function(e){return this.internalMonitor.canDropOnTarget(e)},e.prototype.getItemType=function(){return this.internalMonitor.getItemType()},e.prototype.getItem=function(){return this.internalMonitor.getItem()},e.prototype.getDropResult=function(){return this.internalMonitor.getDropResult()},e.prototype.didDrop=function(){return this.internalMonitor.didDrop()},e.prototype.getInitialClientOffset=function(){return this.internalMonitor.getInitialClientOffset()},e.prototype.getInitialSourceClientOffset=function(){return this.internalMonitor.getInitialSourceClientOffset()},e.prototype.getSourceClientOffset=function(){return this.internalMonitor.getSourceClientOffset()},e.prototype.getClientOffset=function(){return this.internalMonitor.getClientOffset()},e.prototype.getDifferenceFromInitialOffset=function(){return this.internalMonitor.getDifferenceFromInitialOffset()},e}();t.default=a},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(953),i=n(954),o=n(117),a=function(){function e(e){var t=this;this.backend=e,this.hooks=r.default({dragSource:function(e,n){t.dragSourceOptions=n||null,i.isRef(e)?t.dragSourceRef=e:t.dragSourceNode=e,t.reconnectDragSource()},dragPreview:function(e,n){t.dragPreviewOptions=n||null,i.isRef(e)?t.dragPreviewRef=e:t.dragPreviewNode=e,t.reconnectDragPreview()}}),this.handlerId=null,this.dragSourceRef=null,this.dragSourceOptionsInternal=null,this.dragPreviewRef=null,this.dragPreviewOptionsInternal=null,this.lastConnectedHandlerId=null,this.lastConnectedDragSource=null,this.lastConnectedDragSourceOptions=null,this.lastConnectedDragPreview=null,this.lastConnectedDragPreviewOptions=null}return e.prototype.receiveHandlerId=function(e){this.handlerId!==e&&(this.handlerId=e,this.reconnect())},Object.defineProperty(e.prototype,"connectTarget",{get:function(){return this.dragSource},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"dragSourceOptions",{get:function(){return this.dragSourceOptionsInternal},set:function(e){this.dragSourceOptionsInternal=e},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"dragPreviewOptions",{get:function(){return this.dragPreviewOptionsInternal},set:function(e){this.dragPreviewOptionsInternal=e},enumerable:!0,configurable:!0}),e.prototype.reconnect=function(){this.reconnectDragSource(),this.reconnectDragPreview()},e.prototype.reconnectDragSource=function(){var e=this.didHandlerIdChange()||this.didConnectedDragSourceChange()||this.didDragSourceOptionsChange();e&&this.disconnectDragSource();var t=this.dragSource;this.handlerId&&(t?e&&(this.lastConnectedHandlerId=this.handlerId,this.lastConnectedDragSource=t,this.lastConnectedDragSourceOptions=this.dragSourceOptions,this.dragSourceUnsubscribe=this.backend.connectDragSource(this.handlerId,t,this.dragSourceOptions)):this.lastConnectedDragSource=t)},e.prototype.reconnectDragPreview=function(){var e=this.didHandlerIdChange()||this.didConnectedDragPreviewChange()||this.didDragPreviewOptionsChange();e&&this.disconnectDragPreview();var t=this.dragPreview;this.handlerId&&t&&e&&(this.lastConnectedHandlerId=this.handlerId,this.lastConnectedDragPreview=t,this.lastConnectedDragPreviewOptions=this.dragPreviewOptions,this.dragPreviewUnsubscribe=this.backend.connectDragPreview(this.handlerId,t,this.dragPreviewOptions))},e.prototype.didHandlerIdChange=function(){return this.lastConnectedHandlerId!==this.handlerId},e.prototype.didConnectedDragSourceChange=function(){return this.lastConnectedDragSource!==this.dragSource},e.prototype.didConnectedDragPreviewChange=function(){return this.lastConnectedDragPreview!==this.dragPreview},e.prototype.didDragSourceOptionsChange=function(){return!o(this.lastConnectedDragSourceOptions,this.dragSourceOptions)},e.prototype.didDragPreviewOptionsChange=function(){return!o(this.lastConnectedDragPreviewOptions,this.dragPreviewOptions)},e.prototype.disconnectDragSource=function(){this.dragSourceUnsubscribe&&(this.dragSourceUnsubscribe(),this.dragSourceUnsubscribe=void 0,this.dragPreviewNode=null,this.dragPreviewRef=null)},e.prototype.disconnectDragPreview=function(){this.dragPreviewUnsubscribe&&(this.dragPreviewUnsubscribe(),this.dragPreviewUnsubscribe=void 0,this.dragPreviewNode=null,this.dragPreviewRef=null)},Object.defineProperty(e.prototype,"dragSource",{get:function(){return this.dragSourceNode||this.dragSourceRef&&this.dragSourceRef.current},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"dragPreview",{get:function(){return this.dragPreviewNode||this.dragPreviewRef&&this.dragPreviewRef.current},enumerable:!0,configurable:!0}),e}();t.default=a},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(0),i=n(1624);function o(e){return function(t,n){if(void 0===t&&(t=null),void 0===n&&(n=null),!r.isValidElement(t)){var o=t;return e(o,n),o}var a=t;!function(e){if("string"!=typeof e.type){var t=e.type.displayName||e.type.name||"the component";throw new Error("Only native element nodes can now be passed to React DnD connectors.You can either wrap "+t+" into a <div>, or turn it into a drag source or a drop target itself.")}}(a);var s=n?function(t){return e(t,n)}:e;return i.default(a,s)}}n(1625),t.default=function(e){var t={};return Object.keys(e).forEach((function(n){var r=e[n];if(n.endsWith("Ref"))t[n]=e[n];else{var i=o(r);t[n]=function(){return i}}})),t}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.isRef=function(e){return null!==e&&"object"==typeof e&&e.hasOwnProperty("current")}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function e(t,n){return"string"==typeof t||"symbol"==typeof t||!!n&&Array.isArray(t)&&t.every((function(t){return e(t,!1)}))}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t,n){var r=n.getRegistry(),i=r.addTarget(e,t);return[i,function(){return r.removeTarget(i)}]}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(19),i=!1,o=function(){function e(e){this.targetId=null,this.internalMonitor=e.getMonitor()}return e.prototype.receiveHandlerId=function(e){this.targetId=e},e.prototype.getHandlerId=function(){return this.targetId},e.prototype.subscribeToStateChange=function(e,t){return this.internalMonitor.subscribeToStateChange(e,t)},e.prototype.canDrop=function(){if(!this.targetId)return!1;r(!i,"You may not call monitor.canDrop() inside your canDrop() implementation. Read more: http://react-dnd.github.io/react-dnd/docs/api/drop-target-monitor");try{return i=!0,this.internalMonitor.canDropOnTarget(this.targetId)}finally{i=!1}},e.prototype.isOver=function(e){return!!this.targetId&&this.internalMonitor.isOverTarget(this.targetId,e)},e.prototype.getItemType=function(){return this.internalMonitor.getItemType()},e.prototype.getItem=function(){return this.internalMonitor.getItem()},e.prototype.getDropResult=function(){return this.internalMonitor.getDropResult()},e.prototype.didDrop=function(){return this.internalMonitor.didDrop()},e.prototype.getInitialClientOffset=function(){return this.internalMonitor.getInitialClientOffset()},e.prototype.getInitialSourceClientOffset=function(){return this.internalMonitor.getInitialSourceClientOffset()},e.prototype.getSourceClientOffset=function(){return this.internalMonitor.getSourceClientOffset()},e.prototype.getClientOffset=function(){return this.internalMonitor.getClientOffset()},e.prototype.getDifferenceFromInitialOffset=function(){return this.internalMonitor.getDifferenceFromInitialOffset()},e}();t.default=o},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(953),i=n(954),o=n(117),a=function(){function e(e){var t=this;this.backend=e,this.hooks=r.default({dropTarget:function(e,n){t.dropTargetOptions=n,i.isRef(e)?t.dropTargetRef=e:t.dropTargetNode=e,t.reconnect()}}),this.handlerId=null,this.dropTargetRef=null,this.dropTargetOptionsInternal=null,this.lastConnectedHandlerId=null,this.lastConnectedDropTarget=null,this.lastConnectedDropTargetOptions=null}return Object.defineProperty(e.prototype,"connectTarget",{get:function(){return this.dropTarget},enumerable:!0,configurable:!0}),e.prototype.reconnect=function(){var e=this.didHandlerIdChange()||this.didDropTargetChange()||this.didOptionsChange();e&&this.disconnectDropTarget();var t=this.dropTarget;this.handlerId&&(t?e&&(this.lastConnectedHandlerId=this.handlerId,this.lastConnectedDropTarget=t,this.lastConnectedDropTargetOptions=this.dropTargetOptions,this.unsubscribeDropTarget=this.backend.connectDropTarget(this.handlerId,t,this.dropTargetOptions)):this.lastConnectedDropTarget=t)},e.prototype.receiveHandlerId=function(e){e!==this.handlerId&&(this.handlerId=e,this.reconnect())},Object.defineProperty(e.prototype,"dropTargetOptions",{get:function(){return this.dropTargetOptionsInternal},set:function(e){this.dropTargetOptionsInternal=e},enumerable:!0,configurable:!0}),e.prototype.didHandlerIdChange=function(){return this.lastConnectedHandlerId!==this.handlerId},e.prototype.didDropTargetChange=function(){return this.lastConnectedDropTarget!==this.dropTarget},e.prototype.didOptionsChange=function(){return!o(this.lastConnectedDropTargetOptions,this.dropTargetOptions)},e.prototype.disconnectDropTarget=function(){this.unsubscribeDropTarget&&(this.unsubscribeDropTarget(),this.unsubscribeDropTarget=void 0)},Object.defineProperty(e.prototype,"dropTarget",{get:function(){return this.dropTargetNode||this.dropTargetRef&&this.dropTargetRef.current},enumerable:!0,configurable:!0}),e}();t.default=a},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(0),i=n(960);t.useMonitorOutput=function(e,t,n){var o=i.useCollector(e,t,n),a=o[0],s=o[1];return r.useEffect((function(){var t=e.getHandlerId();if(null!=t)return e.subscribeToStateChange(s,{handlerIds:[t]})}),[e,s]),a}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(117),i=n(0);t.useCollector=function(e,t,n){var o=i.useState((function(){return t(e)})),a=o[0],s=o[1],l=i.useCallback((function(){var i=t(e);r(a,i)||(s(i),n&&n())}),[a,e,n]);return[a,l]}},function(e,t,n){window,e.exports=function(e){var t={};function n(r){if(t[r])return t[r].exports;var i=t[r]={i:r,l:!1,exports:{}};return e[r].call(i.exports,i,i.exports,n),i.l=!0,i.exports}return n.m=e,n.c=t,n.d=function(e,t,r){n.o(e,t)||Object.defineProperty(e,t,{enumerable:!0,get:r})},n.r=function(e){"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})},n.t=function(e,t){if(1&t&&(e=n(e)),8&t)return e;if(4&t&&"object"==typeof e&&e&&e.__esModule)return e;var r=Object.create(null);if(n.r(r),Object.defineProperty(r,"default",{enumerable:!0,value:e}),2&t&&"string"!=typeof e)for(var i in e)n.d(r,i,function(t){return e[t]}.bind(null,i));return r},n.n=function(e){var t=e&&e.__esModule?function(){return e.default}:function(){return e};return n.d(t,"a",t),t},n.o=function(e,t){return Object.prototype.hasOwnProperty.call(e,t)},n.p="",n(n.s=556)}([function(e,t,n){"use strict";n.r(t),n.d(t,"contains",(function(){return i})),n.d(t,"includes",(function(){return i})),n.d(t,"difference",(function(){return a})),n.d(t,"find",(function(){return C})),n.d(t,"findIndex",(function(){return A})),n.d(t,"firstValue",(function(){return _})),n.d(t,"flatten",(function(){return I})),n.d(t,"flattenDeep",(function(){return x})),n.d(t,"getRange",(function(){return w})),n.d(t,"pull",(function(){return E})),n.d(t,"pullAt",(function(){return k})),n.d(t,"reduce",(function(){return L})),n.d(t,"remove",(function(){return D})),n.d(t,"sortBy",(function(){return N})),n.d(t,"union",(function(){return R})),n.d(t,"uniq",(function(){return j})),n.d(t,"valuesOfKey",(function(){return z})),n.d(t,"head",(function(){return F})),n.d(t,"last",(function(){return B})),n.d(t,"startsWith",(function(){return H})),n.d(t,"endsWith",(function(){return U})),n.d(t,"filter",(function(){return o})),n.d(t,"every",(function(){return V})),n.d(t,"some",(function(){return Y})),n.d(t,"group",(function(){return q})),n.d(t,"groupBy",(function(){return G})),n.d(t,"groupToMap",(function(){return X})),n.d(t,"getWrapBehavior",(function(){return Z})),n.d(t,"wrapBehavior",(function(){return Q})),n.d(t,"number2color",(function(){return J})),n.d(t,"parseRadius",(function(){return $})),n.d(t,"clamp",(function(){return ee})),n.d(t,"fixedBase",(function(){return te})),n.d(t,"isDecimal",(function(){return re})),n.d(t,"isEven",(function(){return ie})),n.d(t,"isInteger",(function(){return oe})),n.d(t,"isNegative",(function(){return ae})),n.d(t,"isNumberEqual",(function(){return se})),n.d(t,"isOdd",(function(){return le})),n.d(t,"isPositive",(function(){return ue})),n.d(t,"maxBy",(function(){return ce})),n.d(t,"minBy",(function(){return de})),n.d(t,"mod",(function(){return he})),n.d(t,"toDegree",(function(){return pe})),n.d(t,"toInteger",(function(){return ge})),n.d(t,"toRadian",(function(){return me})),n.d(t,"forIn",(function(){return ye})),n.d(t,"has",(function(){return be})),n.d(t,"hasKey",(function(){return Ce})),n.d(t,"hasValue",(function(){return _e})),n.d(t,"keys",(function(){return g})),n.d(t,"isMatch",(function(){return v})),n.d(t,"values",(function(){return Ae})),n.d(t,"lowerCase",(function(){return xe})),n.d(t,"lowerFirst",(function(){return we})),n.d(t,"substitute",(function(){return Me})),n.d(t,"upperCase",(function(){return Se})),n.d(t,"upperFirst",(function(){return Te})),n.d(t,"getType",(function(){return Oe})),n.d(t,"isArguments",(function(){return ke})),n.d(t,"isArray",(function(){return d})),n.d(t,"isArrayLike",(function(){return r})),n.d(t,"isBoolean",(function(){return Le})),n.d(t,"isDate",(function(){return De})),n.d(t,"isError",(function(){return Pe})),n.d(t,"isFunction",(function(){return u})),n.d(t,"isFinite",(function(){return Ne})),n.d(t,"isNil",(function(){return c})),n.d(t,"isNull",(function(){return je})),n.d(t,"isNumber",(function(){return ne})),n.d(t,"isObject",(function(){return f})),n.d(t,"isObjectLike",(function(){return y})),n.d(t,"isPlainObject",(function(){return b})),n.d(t,"isPrototype",(function(){return ze})),n.d(t,"isRegExp",(function(){return Fe})),n.d(t,"isString",(function(){return P})),n.d(t,"isType",(function(){return l})),n.d(t,"isUndefined",(function(){return Be})),n.d(t,"isElement",(function(){return He})),n.d(t,"requestAnimationFrame",(function(){return Ue})),n.d(t,"clearAnimationFrame",(function(){return Ve})),n.d(t,"augment",(function(){return Ge})),n.d(t,"clone",(function(){return qe})),n.d(t,"debounce",(function(){return Ze})),n.d(t,"memoize",(function(){return Qe})),n.d(t,"deepMix",(function(){return Je})),n.d(t,"each",(function(){return p})),n.d(t,"extend",(function(){return $e})),n.d(t,"indexOf",(function(){return et})),n.d(t,"isEmpty",(function(){return nt})),n.d(t,"isEqual",(function(){return rt})),n.d(t,"isEqualWith",(function(){return it})),n.d(t,"map",(function(){return ot})),n.d(t,"mapValues",(function(){return st})),n.d(t,"mix",(function(){return We})),n.d(t,"assign",(function(){return We})),n.d(t,"get",(function(){return lt})),n.d(t,"set",(function(){return ut})),n.d(t,"pick",(function(){return dt})),n.d(t,"throttle",(function(){return ht})),n.d(t,"toArray",(function(){return ft})),n.d(t,"toString",(function(){return Ie})),n.d(t,"uniqueId",(function(){return gt})),n.d(t,"noop",(function(){return vt})),n.d(t,"identity",(function(){return mt})),n.d(t,"size",(function(){return yt})),n.d(t,"Cache",(function(){return bt}));var r=function(e){return null!==e&&"function"!=typeof e&&isFinite(e.length)},i=function(e,t){return!!r(e)&&e.indexOf(t)>-1},o=function(e,t){if(!r(e))return e;for(var n=[],i=0;i<e.length;i++){var o=e[i];t(o,i)&&n.push(o)}return n},a=function(e,t){return void 0===t&&(t=[]),o(e,(function(e){return!i(t,e)}))},s={}.toString,l=function(e,t){return s.call(e)==="[object "+t+"]"},u=function(e){return l(e,"Function")},c=function(e){return null==e},d=function(e){return Array.isArray?Array.isArray(e):l(e,"Array")};function h(e){return(h="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}var f=function(e){var t=h(e);return null!==e&&"object"===t||"function"===t},p=function(e,t){if(e)if(d(e))for(var n=0,r=e.length;n<r&&!1!==t(e[n],n);n++);else if(f(e))for(var i in e)if(e.hasOwnProperty(i)&&!1===t(e[i],i))break},g=Object.keys?function(e){return Object.keys(e)}:function(e){var t=[];return p(e,(function(n,r){u(e)&&"prototype"===r||t.push(r)})),t},v=function(e,t){var n=g(t),r=n.length;if(c(e))return!r;for(var i=0;i<r;i+=1){var o=n[i];if(t[o]!==e[o]||!(o in e))return!1}return!0};function m(e){return(m="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}var y=function(e){return"object"===m(e)&&null!==e},b=function(e){if(!y(e)||!l(e,"Object"))return!1;if(null===Object.getPrototypeOf(e))return!0;for(var t=e;null!==Object.getPrototypeOf(t);)t=Object.getPrototypeOf(t);return Object.getPrototypeOf(e)===t},C=function(e,t){if(!d(e))return null;var n;if(u(t)&&(n=t),b(t)&&(n=function(e){return v(e,t)}),n)for(var r=0;r<e.length;r+=1)if(n(e[r]))return e[r];return null},A=function(e,t,n){void 0===n&&(n=0);for(var r=n;r<e.length;r++)if(t(e[r],r))return r;return-1},_=function(e,t){for(var n=null,r=0;r<e.length;r++){var i=e[r][t];if(!c(i)){n=d(i)?i[0]:i;break}}return n},I=function(e){if(!d(e))return[];for(var t=[],n=0;n<e.length;n++)t=t.concat(e[n]);return t},x=function e(t,n){if(void 0===n&&(n=[]),d(t))for(var r=0;r<t.length;r+=1)e(t[r],n);else n.push(t);return n},w=function(e){var t=e.filter((function(e){return!isNaN(e)}));if(!t.length)return{min:0,max:0};if(d(e[0])){for(var n=[],r=0;r<e.length;r++)n=n.concat(e[r]);t=n}var i=Math.max.apply(null,t);return{min:Math.min.apply(null,t),max:i}},M=Array.prototype,S=M.splice,T=M.indexOf,E=function(e){for(var t=[],n=1;n<arguments.length;n++)t[n-1]=arguments[n];for(var r=0;r<t.length;r++)for(var i=t[r],o=-1;(o=T.call(e,i))>-1;)S.call(e,o,1);return e},O=Array.prototype.splice,k=function(e,t){if(!r(e))return[];for(var n=e?t.length:0,i=n-1;n--;){var o=void 0,a=t[n];n!==i&&a===o||(o=a,O.call(e,a,1))}return e},L=function(e,t,n){if(!d(e)&&!b(e))return e;var r=n;return p(e,(function(e,n){r=t(r,e,n)})),r},D=function(e,t){var n=[];if(!r(e))return n;for(var i=-1,o=[],a=e.length;++i<a;){var s=e[i];t(s,i,e)&&(n.push(s),o.push(i))}return k(e,o),n},P=function(e){return l(e,"String")},N=function(e,t){var n;if(u(t))n=function(e,n){return t(e)-t(n)};else{var r=[];P(t)?r.push(t):d(t)&&(r=t),n=function(e,t){for(var n=0;n<r.length;n+=1){var i=r[n];if(e[i]>t[i])return 1;if(e[i]<t[i])return-1}return 0}}return e.sort(n),e},j=function(e){var t=[];return p(e,(function(e){i(t,e)||t.push(e)})),t},R=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return j([].concat.apply([],e))},z=function(e,t){for(var n=[],r={},i=0;i<e.length;i++){var o=e[i][t];if(!c(o)){d(o)||(o=[o]);for(var a=0;a<o.length;a++){var s=o[a];r[s]||(n.push(s),r[s]=!0)}}}return n};function F(e){if(r(e))return e[0]}function B(e){if(r(e))return e[e.length-1]}var H=function(e,t){return!(!d(e)&&!P(e))&&e[0]===t},U=function(e,t){return!(!d(e)&&!P(e))&&e[e.length-1]===t},V=function(e,t){for(var n=0;n<e.length;n++)if(!t(e[n],n))return!1;return!0},Y=function(e,t){for(var n=0;n<e.length;n++)if(t(e[n],n))return!0;return!1},W=Object.prototype.hasOwnProperty,G=function(e,t){if(!t||!d(e))return{};for(var n,r={},i=u(t)?t:function(e){return e[t]},o=0;o<e.length;o++){var a=e[o];n=i(a),W.call(r,n)?r[n].push(a):r[n]=[a]}return r},X=function(e,t){if(!t)return{0:e};if(!u(t)){var n=d(t)?t:t.replace(/\s+/g,"").split("*");t=function(e){for(var t="_",r=0,i=n.length;r<i;r++)t+=e[n[r]]&&e[n[r]].toString();return t}}return G(e,t)},q=function(e,t){if(!t)return[e];var n=X(e,t),r=[];for(var i in n)r.push(n[i]);return r},Z=function(e,t){return e["_wrap_"+t]},Q=function(e,t){if(e["_wrap_"+t])return e["_wrap_"+t];var n=function(n){e[t](n)};return e["_wrap_"+t]=n,n},K={},J=function(e){var t=K[e];if(!t){for(var n=e.toString(16),r=n.length;r<6;r++)n="0"+n;t="#"+n,K[e]=t}return t},$=function(e){var t=0,n=0,r=0,i=0;return d(e)?1===e.length?t=n=r=i=e[0]:2===e.length?(t=r=e[0],n=i=e[1]):3===e.length?(t=e[0],n=i=e[1],r=e[2]):(t=e[0],n=e[1],r=e[2],i=e[3]):t=n=r=i=e,{r1:t,r2:n,r3:r,r4:i}},ee=function(e,t,n){return e<t?t:e>n?n:e},te=function(e,t){var n=t.toString(),r=n.indexOf(".");if(-1===r)return Math.round(e);var i=n.substr(r+1).length;return i>20&&(i=20),parseFloat(e.toFixed(i))},ne=function(e){return l(e,"Number")},re=function(e){return ne(e)&&e%1!=0},ie=function(e){return ne(e)&&e%2==0},oe=Number.isInteger?Number.isInteger:function(e){return ne(e)&&e%1==0},ae=function(e){return ne(e)&&e<0};function se(e,t,n){return void 0===n&&(n=1e-5),Math.abs(e-t)<n}var le=function(e){return ne(e)&&e%2!=0},ue=function(e){return ne(e)&&e>0},ce=function(e,t){if(d(e)){var n,r,i=e[0];return n=u(t)?t(e[0]):e[0][t],p(e,(function(e){(r=u(t)?t(e):e[t])>n&&(i=e,n=r)})),i}},de=function(e,t){if(d(e)){var n,r,i=e[0];return n=u(t)?t(e[0]):e[0][t],p(e,(function(e){(r=u(t)?t(e):e[t])<n&&(i=e,n=r)})),i}},he=function(e,t){return(e%t+t)%t},fe=180/Math.PI,pe=function(e){return fe*e},ge=parseInt,ve=Math.PI/180,me=function(e){return ve*e},ye=p,be=function(e,t){return e.hasOwnProperty(t)},Ce=be,Ae=Object.values?function(e){return Object.values(e)}:function(e){var t=[];return p(e,(function(n,r){u(e)&&"prototype"===r||t.push(n)})),t},_e=function(e,t){return i(Ae(e),t)},Ie=function(e){return c(e)?"":e.toString()},xe=function(e){return Ie(e).toLowerCase()},we=function(e){var t=Ie(e);return t.charAt(0).toLowerCase()+t.substring(1)},Me=function(e,t){return e&&t?e.replace(/\\?\{([^{}]+)\}/g,(function(e,n){return"\\"===e.charAt(0)?e.slice(1):void 0===t[n]?"":t[n]})):e},Se=function(e){return Ie(e).toUpperCase()},Te=function(e){var t=Ie(e);return t.charAt(0).toUpperCase()+t.substring(1)},Ee={}.toString,Oe=function(e){return Ee.call(e).replace(/^\[object /,"").replace(/]$/,"")},ke=function(e){return l(e,"Arguments")},Le=function(e){return l(e,"Boolean")},De=function(e){return l(e,"Date")},Pe=function(e){return l(e,"Error")},Ne=function(e){return ne(e)&&isFinite(e)},je=function(e){return null===e},Re=Object.prototype,ze=function(e){var t=e&&e.constructor;return e===("function"==typeof t&&t.prototype||Re)},Fe=function(e){return l(e,"RegExp")},Be=function(e){return void 0===e},He=function(e){return e instanceof Element||e instanceof HTMLDocument};function Ue(e){return(window.requestAnimationFrame||window.webkitRequestAnimationFrame||window.mozRequestAnimationFrame||window.msRequestAnimationFrame||function(e){return setTimeout(e,16)})(e)}function Ve(e){(window.cancelAnimationFrame||window.webkitCancelAnimationFrame||window.mozCancelAnimationFrame||window.msCancelAnimationFrame||clearTimeout)(e)}function Ye(e,t){for(var n in t)t.hasOwnProperty(n)&&"constructor"!==n&&void 0!==t[n]&&(e[n]=t[n])}function We(e,t,n,r){return t&&Ye(e,t),n&&Ye(e,n),r&&Ye(e,r),e}var Ge=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];for(var n=e[0],r=1;r<e.length;r++){var i=e[r];u(i)&&(i=i.prototype),We(n.prototype,i)}};function Xe(e){return(Xe="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}var qe=function e(t){if("object"!==Xe(t)||null===t)return t;var n;if(d(t)){n=[];for(var r=0,i=t.length;r<i;r++)"object"===Xe(t[r])&&null!=t[r]?n[r]=e(t[r]):n[r]=t[r]}else for(var o in n={},t)"object"===Xe(t[o])&&null!=t[o]?n[o]=e(t[o]):n[o]=t[o];return n},Ze=function(e,t,n){var r;return function(){var i=this,o=arguments,a=function(){r=null,n||e.apply(i,o)},s=n&&!r;clearTimeout(r),r=setTimeout(a,t),s&&e.apply(i,o)}},Qe=function(e,t){if(!u(e))throw new TypeError("Expected a function");var n=function n(){for(var r=[],i=0;i<arguments.length;i++)r[i]=arguments[i];var o=t?t.apply(this,r):r[0],a=n.cache;if(a.has(o))return a.get(o);var s=e.apply(this,r);return a.set(o,s),s};return n.cache=new Map,n};function Ke(e,t,n,r){for(var i in n=n||0,r=r||5,t)if(t.hasOwnProperty(i)){var o=t[i];null!==o&&b(o)?(b(e[i])||(e[i]={}),n<r?Ke(e[i],o,n+1,r):e[i]=t[i]):d(o)?(e[i]=[],e[i]=e[i].concat(o)):void 0!==o&&(e[i]=o)}}var Je=function(e){for(var t=[],n=1;n<arguments.length;n++)t[n-1]=arguments[n];for(var r=0;r<t.length;r+=1)Ke(e,t[r]);return e},$e=function(e,t,n,r){u(t)||(n=t,t=e,e=function(){});var i=Object.create?function(e,t){return Object.create(e,{constructor:{value:t}})}:function(e,t){function n(){}n.prototype=e;var r=new n;return r.constructor=t,r},o=i(t.prototype,e);return e.prototype=We(o,e.prototype),e.superclass=i(t.prototype,t),We(o,n),We(e,r),e},et=function(e,t){if(!r(e))return-1;var n=Array.prototype.indexOf;if(n)return n.call(e,t);for(var i=-1,o=0;o<e.length;o++)if(e[o]===t){i=o;break}return i},tt=Object.prototype.hasOwnProperty,nt=function(e){if(c(e))return!0;if(r(e))return!e.length;var t=Oe(e);if("Map"===t||"Set"===t)return!e.size;if(ze(e))return!Object.keys(e).length;for(var n in e)if(tt.call(e,n))return!1;return!0},rt=function e(t,n){if(t===n)return!0;if(!t||!n)return!1;if(P(t)||P(n))return!1;if(r(t)||r(n)){if(t.length!==n.length)return!1;for(var i=!0,o=0;o<t.length&&(i=e(t[o],n[o]));o++);return i}if(y(t)||y(n)){var a=Object.keys(t),s=Object.keys(n);if(a.length!==s.length)return!1;for(i=!0,o=0;o<a.length&&(i=e(t[a[o]],n[a[o]]));o++);return i}return!1},it=function(e,t,n){return u(n)?!!n(e,t):rt(e,t)},ot=function(e,t){if(!r(e))return e;for(var n=[],i=0;i<e.length;i++){var o=e[i];n.push(t(o,i))}return n},at=function(e){return e},st=function(e,t){void 0===t&&(t=at);var n={};return f(e)&&!c(e)&&Object.keys(e).forEach((function(r){n[r]=t(e[r],r)})),n},lt=function(e,t,n){for(var r=0,i=P(t)?t.split("."):t;e&&r<i.length;)e=e[i[r++]];return void 0===e||r<i.length?n:e},ut=function(e,t,n){var r=e,i=P(t)?t.split("."):t;return i.forEach((function(e,t){t<i.length-1?(f(r[e])||(r[e]=ne(i[t+1])?[]:{}),r=r[e]):r[e]=n})),e},ct=Object.prototype.hasOwnProperty,dt=function(e,t){if(null===e||!b(e))return{};var n={};return p(t,(function(t){ct.call(e,t)&&(n[t]=e[t])})),n},ht=function(e,t,n){var r,i,o,a,s=0;n||(n={});var l=function(){s=!1===n.leading?0:Date.now(),r=null,a=e.apply(i,o),r||(i=o=null)},u=function(){var u=Date.now();s||!1!==n.leading||(s=u);var c=t-(u-s);return i=this,o=arguments,c<=0||c>t?(r&&(clearTimeout(r),r=null),s=u,a=e.apply(i,o),r||(i=o=null)):r||!1===n.trailing||(r=setTimeout(l,c)),a};return u.cancel=function(){clearTimeout(r),s=0,r=i=o=null},u},ft=function(e){return r(e)?Array.prototype.slice.call(e):[]},pt={},gt=function(e){return pt[e=e||"g"]?pt[e]+=1:pt[e]=1,e+pt[e]},vt=function(){},mt=function(e){return e};function yt(e){return c(e)?0:r(e)?e.length:Object.keys(e).length}var bt=function(){function e(){this.map={}}return e.prototype.has=function(e){return void 0!==this.map[e]},e.prototype.get=function(e,t){var n=this.map[e];return void 0===n?t:n},e.prototype.set=function(e,t){this.map[e]=t},e.prototype.clear=function(){this.map={}},e.prototype.delete=function(e){delete this.map[e]},e.prototype.size=function(){return Object.keys(this.map).length},e}()},function(e,t,n){"use strict";function r(e){return(r="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}
/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */n.r(t),n.d(t,"__extends",(function(){return o})),n.d(t,"__assign",(function(){return a})),n.d(t,"__rest",(function(){return s})),n.d(t,"__decorate",(function(){return l})),n.d(t,"__param",(function(){return u})),n.d(t,"__metadata",(function(){return c})),n.d(t,"__awaiter",(function(){return d})),n.d(t,"__generator",(function(){return h})),n.d(t,"__createBinding",(function(){return f})),n.d(t,"__exportStar",(function(){return p})),n.d(t,"__values",(function(){return g})),n.d(t,"__read",(function(){return v})),n.d(t,"__spread",(function(){return m})),n.d(t,"__spreadArrays",(function(){return y})),n.d(t,"__await",(function(){return b})),n.d(t,"__asyncGenerator",(function(){return C})),n.d(t,"__asyncDelegator",(function(){return A})),n.d(t,"__asyncValues",(function(){return _})),n.d(t,"__makeTemplateObject",(function(){return I})),n.d(t,"__importStar",(function(){return w})),n.d(t,"__importDefault",(function(){return M})),n.d(t,"__classPrivateFieldGet",(function(){return S})),n.d(t,"__classPrivateFieldSet",(function(){return T}));var i=function(e,t){return(i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(e[n]=t[n])})(e,t)};function o(e,t){function n(){this.constructor=e}i(e,t),e.prototype=null===t?Object.create(t):(n.prototype=t.prototype,new n)}var a=function(){return(a=Object.assign||function(e){for(var t,n=1,r=arguments.length;n<r;n++)for(var i in t=arguments[n])Object.prototype.hasOwnProperty.call(t,i)&&(e[i]=t[i]);return e}).apply(this,arguments)};function s(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(null!=e&&"function"==typeof Object.getOwnPropertySymbols){var i=0;for(r=Object.getOwnPropertySymbols(e);i<r.length;i++)t.indexOf(r[i])<0&&Object.prototype.propertyIsEnumerable.call(e,r[i])&&(n[r[i]]=e[r[i]])}return n}function l(e,t,n,i){var o,a=arguments.length,s=a<3?t:null===i?i=Object.getOwnPropertyDescriptor(t,n):i;if("object"===("undefined"==typeof Reflect?"undefined":r(Reflect))&&"function"==typeof Reflect.decorate)s=Reflect.decorate(e,t,n,i);else for(var l=e.length-1;l>=0;l--)(o=e[l])&&(s=(a<3?o(s):a>3?o(t,n,s):o(t,n))||s);return a>3&&s&&Object.defineProperty(t,n,s),s}function u(e,t){return function(n,r){t(n,r,e)}}function c(e,t){if("object"===("undefined"==typeof Reflect?"undefined":r(Reflect))&&"function"==typeof Reflect.metadata)return Reflect.metadata(e,t)}function d(e,t,n,r){return new(n||(n=Promise))((function(i,o){function a(e){try{l(r.next(e))}catch(e){o(e)}}function s(e){try{l(r.throw(e))}catch(e){o(e)}}function l(e){var t;e.done?i(e.value):(t=e.value,t instanceof n?t:new n((function(e){e(t)}))).then(a,s)}l((r=r.apply(e,t||[])).next())}))}function h(e,t){var n,r,i,o,a={label:0,sent:function(){if(1&i[0])throw i[1];return i[1]},trys:[],ops:[]};return o={next:s(0),throw:s(1),return:s(2)},"function"==typeof Symbol&&(o[Symbol.iterator]=function(){return this}),o;function s(o){return function(s){return function(o){if(n)throw new TypeError("Generator is already executing.");for(;a;)try{if(n=1,r&&(i=2&o[0]?r.return:o[0]?r.throw||((i=r.return)&&i.call(r),0):r.next)&&!(i=i.call(r,o[1])).done)return i;switch(r=0,i&&(o=[2&o[0],i.value]),o[0]){case 0:case 1:i=o;break;case 4:return a.label++,{value:o[1],done:!1};case 5:a.label++,r=o[1],o=[0];continue;case 7:o=a.ops.pop(),a.trys.pop();continue;default:if(!((i=(i=a.trys).length>0&&i[i.length-1])||6!==o[0]&&2!==o[0])){a=0;continue}if(3===o[0]&&(!i||o[1]>i[0]&&o[1]<i[3])){a.label=o[1];break}if(6===o[0]&&a.label<i[1]){a.label=i[1],i=o;break}if(i&&a.label<i[2]){a.label=i[2],a.ops.push(o);break}i[2]&&a.ops.pop(),a.trys.pop();continue}o=t.call(e,a)}catch(e){o=[6,e],r=0}finally{n=i=0}if(5&o[0])throw o[1];return{value:o[0]?o[1]:void 0,done:!0}}([o,s])}}}var f=Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]};function p(e,t){for(var n in e)"default"===n||Object.prototype.hasOwnProperty.call(t,n)||f(t,e,n)}function g(e){var t="function"==typeof Symbol&&Symbol.iterator,n=t&&e[t],r=0;if(n)return n.call(e);if(e&&"number"==typeof e.length)return{next:function(){return e&&r>=e.length&&(e=void 0),{value:e&&e[r++],done:!e}}};throw new TypeError(t?"Object is not iterable.":"Symbol.iterator is not defined.")}function v(e,t){var n="function"==typeof Symbol&&e[Symbol.iterator];if(!n)return e;var r,i,o=n.call(e),a=[];try{for(;(void 0===t||t-- >0)&&!(r=o.next()).done;)a.push(r.value)}catch(e){i={error:e}}finally{try{r&&!r.done&&(n=o.return)&&n.call(o)}finally{if(i)throw i.error}}return a}function m(){for(var e=[],t=0;t<arguments.length;t++)e=e.concat(v(arguments[t]));return e}function y(){for(var e=0,t=0,n=arguments.length;t<n;t++)e+=arguments[t].length;var r=Array(e),i=0;for(t=0;t<n;t++)for(var o=arguments[t],a=0,s=o.length;a<s;a++,i++)r[i]=o[a];return r}function b(e){return this instanceof b?(this.v=e,this):new b(e)}function C(e,t,n){if(!Symbol.asyncIterator)throw new TypeError("Symbol.asyncIterator is not defined.");var r,i=n.apply(e,t||[]),o=[];return r={},a("next"),a("throw"),a("return"),r[Symbol.asyncIterator]=function(){return this},r;function a(e){i[e]&&(r[e]=function(t){return new Promise((function(n,r){o.push([e,t,n,r])>1||s(e,t)}))})}function s(e,t){try{(n=i[e](t)).value instanceof b?Promise.resolve(n.value.v).then(l,u):c(o[0][2],n)}catch(e){c(o[0][3],e)}var n}function l(e){s("next",e)}function u(e){s("throw",e)}function c(e,t){e(t),o.shift(),o.length&&s(o[0][0],o[0][1])}}function A(e){var t,n;return t={},r("next"),r("throw",(function(e){throw e})),r("return"),t[Symbol.iterator]=function(){return this},t;function r(r,i){t[r]=e[r]?function(t){return(n=!n)?{value:b(e[r](t)),done:"return"===r}:i?i(t):t}:i}}function _(e){if(!Symbol.asyncIterator)throw new TypeError("Symbol.asyncIterator is not defined.");var t,n=e[Symbol.asyncIterator];return n?n.call(e):(e=g(e),t={},r("next"),r("throw"),r("return"),t[Symbol.asyncIterator]=function(){return this},t);function r(n){t[n]=e[n]&&function(t){return new Promise((function(r,i){!function(e,t,n,r){Promise.resolve(r).then((function(t){e({value:t,done:n})}),t)}(r,i,(t=e[n](t)).done,t.value)}))}}}function I(e,t){return Object.defineProperty?Object.defineProperty(e,"raw",{value:t}):e.raw=t,e}var x=Object.create?function(e,t){Object.defineProperty(e,"default",{enumerable:!0,value:t})}:function(e,t){e.default=t};function w(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)"default"!==n&&Object.prototype.hasOwnProperty.call(e,n)&&f(t,e,n);return x(t,e),t}function M(e){return e&&e.__esModule?e:{default:e}}function S(e,t){if(!t.has(e))throw new TypeError("attempted to get private field on non-instance");return t.get(e)}function T(e,t,n){if(!t.has(e))throw new TypeError("attempted to set private field on non-instance");return t.set(e,n),n}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(4),i=n(35);t.default=function(e,t){if(e)if(r.default(e))for(var n=0,o=e.length;n<o&&!1!==t(e[n],n);n++);else if(i.default(e))for(var a in e)if(e.hasOwnProperty(a)&&!1===t(e[a],a))break}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(27);t.default=function(e){return r.default(e,"String")}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(27);t.default=function(e){return Array.isArray?Array.isArray(e):r.default(e,"Array")}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){if(e)for(var n in t)t.hasOwnProperty(n)&&(e.style[n]=t[n]);return e}},function(e,t,n){"use strict";n.d(t,"b",(function(){return r})),n.d(t,"a",(function(){return i})),n.d(t,"c",(function(){return o}));var r=1e-6,i="undefined"!=typeof Float32Array?Float32Array:Array,o=Math.random;Math.PI,Math.hypot||(Math.hypot=function(){for(var e=0,t=arguments.length;t--;)e+=arguments[t]*arguments[t];return Math.sqrt(e)})},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(72);t.contains=r.default,t.includes=r.default;var i=n(242);t.difference=i.default;var o=n(243);t.find=o.default;var a=n(244);t.findIndex=a.default;var s=n(245);t.firstValue=s.default;var l=n(246);t.flatten=l.default;var u=n(247);t.flattenDeep=u.default;var c=n(248);t.getRange=c.default;var d=n(249);t.pull=d.default;var h=n(140);t.pullAt=h.default;var f=n(250);t.reduce=f.default;var p=n(251);t.remove=p.default;var g=n(252);t.sortBy=g.default;var v=n(253);t.union=v.default;var m=n(141);t.uniq=m.default;var y=n(254);t.valuesOfKey=y.default;var b=n(255);t.head=b.default;var C=n(256);t.last=C.default;var A=n(257);t.startsWith=A.default;var _=n(258);t.endsWith=_.default;var I=n(137);t.filter=I.default;var x=n(259);t.every=x.default;var w=n(260);t.some=w.default;var M=n(261);t.group=M.default;var S=n(69);t.groupBy=S.default;var T=n(142);t.groupToMap=T.default;var E=n(262);t.getWrapBehavior=E.default;var O=n(50);t.wrapBehavior=O.default;var k=n(263);t.number2color=k.default;var L=n(264);t.parseRadius=L.default;var D=n(265);t.clamp=D.default;var P=n(266);t.fixedBase=P.default;var N=n(267);t.isDecimal=N.default;var j=n(268);t.isEven=j.default;var R=n(269);t.isInteger=R.default;var z=n(270);t.isNegative=z.default;var F=n(103);t.isNumberEqual=F.default;var B=n(271);t.isOdd=B.default;var H=n(272);t.isPositive=H.default;var U=n(273);t.maxBy=U.default;var V=n(274);t.minBy=V.default;var Y=n(67);t.mod=Y.default;var W=n(275);t.toDegree=W.default;var G=n(276);t.toInteger=G.default;var X=n(68);t.toRadian=X.default;var q=n(277);t.forIn=q.default;var Z=n(143);t.has=Z.default;var Q=n(278);t.hasKey=Q.default;var K=n(279);t.hasValue=K.default;var J=n(139);t.keys=J.default;var $=n(138);t.isMatch=$.default;var ee=n(144);t.values=ee.default;var te=n(280);t.lowerCase=te.default;var ne=n(281);t.lowerFirst=ne.default;var re=n(282);t.substitute=re.default;var ie=n(283);t.upperCase=ie.default;var oe=n(49);t.upperFirst=oe.default;var ae=n(145);t.getType=ae.default;var se=n(284);t.isArguments=se.default;var le=n(4);t.isArray=le.default;var ue=n(21);t.isArrayLike=ue.default;var ce=n(285);t.isBoolean=ce.default;var de=n(286);t.isDate=de.default;var he=n(287);t.isError=he.default;var fe=n(14);t.isFunction=fe.default;var pe=n(288);t.isFinite=pe.default;var ge=n(10);t.isNil=ge.default;var ve=n(289);t.isNull=ve.default;var me=n(11);t.isNumber=me.default;var ye=n(35);t.isObject=ye.default;var be=n(102);t.isObjectLike=be.default;var Ce=n(58);t.isPlainObject=Ce.default;var Ae=n(146);t.isPrototype=Ae.default;var _e=n(290);t.isRegExp=_e.default;var Ie=n(3);t.isString=Ie.default;var xe=n(27);t.isType=xe.default;var we=n(291);t.isUndefined=we.default;var Me=n(292);t.isElement=Me.default;var Se=n(147);t.requestAnimationFrame=Se.default;var Te=n(148);t.clearAnimationFrame=Te.default;var Ee=n(293);t.augment=Ee.default;var Oe=n(98);t.clone=Oe.default;var ke=n(294);t.debounce=ke.default;var Le=n(295);t.memoize=Le.default;var De=n(12);t.deepMix=De.default;var Pe=n(2);t.each=Pe.default;var Ne=n(296);t.extend=Ne.default;var je=n(297);t.indexOf=je.default;var Re=n(298);t.isEmpty=Re.default;var ze=n(95);t.isEqual=ze.default;var Fe=n(299);t.isEqualWith=Fe.default;var Be=n(300);t.map=Be.default;var He=n(301);t.mapValues=He.default;var Ue=n(17);t.mix=Ue.default,t.assign=Ue.default;var Ve=n(130);t.get=Ve.default;var Ye=n(302);t.set=Ye.default;var We=n(303);t.pick=We.default;var Ge=n(304);t.throttle=Ge.default;var Xe=n(305);t.toArray=Xe.default;var qe=n(59);t.toString=qe.default;var Ze=n(306);t.uniqueId=Ze.default;var Qe=n(307);t.noop=Qe.default;var Ke=n(308);t.identity=Ke.default;var Je=n(309);t.size=Je.default;var $e=n(310);t.Cache=$e.default},function(e,t,n){"use strict";var r,i,o,a;Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){r||(r=document.createElement("table"),i=document.createElement("tr"),o=/^\s*<(\w+|!)[^>]*>/,a={tr:document.createElement("tbody"),tbody:r,thead:r,tfoot:r,td:i,th:i,"*":document.createElement("div")});var t=o.test(e)&&RegExp.$1;t&&t in a||(t="*");var n=a[t];e=e.replace(/(^\s*)|(\s*$)/g,""),n.innerHTML=""+e;var s=n.childNodes[0];return n.removeChild(s),s}},function(e,t,n){"use strict";n.d(t,"j",(function(){return m})),n.d(t,"c",(function(){return y})),n.d(t,"g",(function(){return b})),n.d(t,"b",(function(){return C}));var r=n(10),i=n.n(r);n.d(t,"e",(function(){return i.a}));var o=n(14),a=n.n(o);n.d(t,"d",(function(){return a.a}));var s=n(3),l=n.n(s);n.d(t,"h",(function(){return l.a}));var u=n(35),c=n.n(u);n.d(t,"f",(function(){return c.a})),n(4);var d=n(17),h=n.n(d);n.d(t,"i",(function(){return h.a}));var f=n(2),p=n.n(f);n.d(t,"a",(function(){return p.a}));var g=n(49),v=n.n(g);function m(e,t){var n=e.indexOf(t);-1!==n&&e.splice(n,1)}n.d(t,"k",(function(){return v.a}));var y="undefined"!=typeof window&&void 0!==window.document;function b(e,t){if(e.isCanvas())return!0;for(var n=t.getParent(),r=!1;n;){if(n===e){r=!0;break}n=n.getParent()}return r}function C(e){return e.cfg.visible&&e.cfg.capture}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){return null==e}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(27);t.default=function(e){return r.default(e,"Number")}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(4),i=n(58);function o(e,t,n,a){for(var s in n=n||0,a=a||5,t)if(t.hasOwnProperty(s)){var l=t[s];null!==l&&i.default(l)?(i.default(e[s])||(e[s]={}),n<a?o(e[s],l,n+1,a):e[s]=t[s]):r.default(l)?(e[s]=[],e[s]=e[s].concat(l)):void 0!==l&&(e[s]=l)}}t.default=function(e){for(var t=[],n=1;n<arguments.length;n++)t[n-1]=arguments[n];for(var r=0;r<t.length;r+=1)o(e,t[r]);return e}},function(e,t,n){var r;try{r={cloneDeep:n(486),constant:n(122),defaults:n(487),each:n(191),filter:n(194),find:n(488),flatten:n(221),forEach:n(192),forIn:n(493),has:n(205),isUndefined:n(206),last:n(494),map:n(207),mapValues:n(495),max:n(496),merge:n(498),min:n(504),minBy:n(505),now:n(506),pick:n(507),range:n(512),reduce:n(209),sortBy:n(515),uniqueId:n(520),values:n(214),zipObject:n(521)}}catch(e){}r||(r=window._),e.exports=r},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(27);t.default=function(e){return r.default(e,"Function")}},function(e,t){var n=Array.isArray;e.exports=n},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.sub=t.mul=void 0,t.create=function(){var e=new r.ARRAY_TYPE(9);return r.ARRAY_TYPE!=Float32Array&&(e[1]=0,e[2]=0,e[3]=0,e[5]=0,e[6]=0,e[7]=0),e[0]=1,e[4]=1,e[8]=1,e},t.fromMat4=function(e,t){return e[0]=t[0],e[1]=t[1],e[2]=t[2],e[3]=t[4],e[4]=t[5],e[5]=t[6],e[6]=t[8],e[7]=t[9],e[8]=t[10],e},t.clone=function(e){var t=new r.ARRAY_TYPE(9);return t[0]=e[0],t[1]=e[1],t[2]=e[2],t[3]=e[3],t[4]=e[4],t[5]=e[5],t[6]=e[6],t[7]=e[7],t[8]=e[8],t},t.copy=function(e,t){return e[0]=t[0],e[1]=t[1],e[2]=t[2],e[3]=t[3],e[4]=t[4],e[5]=t[5],e[6]=t[6],e[7]=t[7],e[8]=t[8],e},t.fromValues=function(e,t,n,i,o,a,s,l,u){var c=new r.ARRAY_TYPE(9);return c[0]=e,c[1]=t,c[2]=n,c[3]=i,c[4]=o,c[5]=a,c[6]=s,c[7]=l,c[8]=u,c},t.set=function(e,t,n,r,i,o,a,s,l,u){return e[0]=t,e[1]=n,e[2]=r,e[3]=i,e[4]=o,e[5]=a,e[6]=s,e[7]=l,e[8]=u,e},t.identity=function(e){return e[0]=1,e[1]=0,e[2]=0,e[3]=0,e[4]=1,e[5]=0,e[6]=0,e[7]=0,e[8]=1,e},t.transpose=function(e,t){if(e===t){var n=t[1],r=t[2],i=t[5];e[1]=t[3],e[2]=t[6],e[3]=n,e[5]=t[7],e[6]=r,e[7]=i}else e[0]=t[0],e[1]=t[3],e[2]=t[6],e[3]=t[1],e[4]=t[4],e[5]=t[7],e[6]=t[2],e[7]=t[5],e[8]=t[8];return e},t.invert=function(e,t){var n=t[0],r=t[1],i=t[2],o=t[3],a=t[4],s=t[5],l=t[6],u=t[7],c=t[8],d=c*a-s*u,h=-c*o+s*l,f=u*o-a*l,p=n*d+r*h+i*f;return p?(p=1/p,e[0]=d*p,e[1]=(-c*r+i*u)*p,e[2]=(s*r-i*a)*p,e[3]=h*p,e[4]=(c*n-i*l)*p,e[5]=(-s*n+i*o)*p,e[6]=f*p,e[7]=(-u*n+r*l)*p,e[8]=(a*n-r*o)*p,e):null},t.adjoint=function(e,t){var n=t[0],r=t[1],i=t[2],o=t[3],a=t[4],s=t[5],l=t[6],u=t[7],c=t[8];return e[0]=a*c-s*u,e[1]=i*u-r*c,e[2]=r*s-i*a,e[3]=s*l-o*c,e[4]=n*c-i*l,e[5]=i*o-n*s,e[6]=o*u-a*l,e[7]=r*l-n*u,e[8]=n*a-r*o,e},t.determinant=function(e){var t=e[0],n=e[1],r=e[2],i=e[3],o=e[4],a=e[5],s=e[6],l=e[7],u=e[8];return t*(u*o-a*l)+n*(-u*i+a*s)+r*(l*i-o*s)},t.multiply=i,t.translate=function(e,t,n){var r=t[0],i=t[1],o=t[2],a=t[3],s=t[4],l=t[5],u=t[6],c=t[7],d=t[8],h=n[0],f=n[1];return e[0]=r,e[1]=i,e[2]=o,e[3]=a,e[4]=s,e[5]=l,e[6]=h*r+f*a+u,e[7]=h*i+f*s+c,e[8]=h*o+f*l+d,e},t.rotate=function(e,t,n){var r=t[0],i=t[1],o=t[2],a=t[3],s=t[4],l=t[5],u=t[6],c=t[7],d=t[8],h=Math.sin(n),f=Math.cos(n);return e[0]=f*r+h*a,e[1]=f*i+h*s,e[2]=f*o+h*l,e[3]=f*a-h*r,e[4]=f*s-h*i,e[5]=f*l-h*o,e[6]=u,e[7]=c,e[8]=d,e},t.scale=function(e,t,n){var r=n[0],i=n[1];return e[0]=r*t[0],e[1]=r*t[1],e[2]=r*t[2],e[3]=i*t[3],e[4]=i*t[4],e[5]=i*t[5],e[6]=t[6],e[7]=t[7],e[8]=t[8],e},t.fromTranslation=function(e,t){return e[0]=1,e[1]=0,e[2]=0,e[3]=0,e[4]=1,e[5]=0,e[6]=t[0],e[7]=t[1],e[8]=1,e},t.fromRotation=function(e,t){var n=Math.sin(t),r=Math.cos(t);return e[0]=r,e[1]=n,e[2]=0,e[3]=-n,e[4]=r,e[5]=0,e[6]=0,e[7]=0,e[8]=1,e},t.fromScaling=function(e,t){return e[0]=t[0],e[1]=0,e[2]=0,e[3]=0,e[4]=t[1],e[5]=0,e[6]=0,e[7]=0,e[8]=1,e},t.fromMat2d=function(e,t){return e[0]=t[0],e[1]=t[1],e[2]=0,e[3]=t[2],e[4]=t[3],e[5]=0,e[6]=t[4],e[7]=t[5],e[8]=1,e},t.fromQuat=function(e,t){var n=t[0],r=t[1],i=t[2],o=t[3],a=n+n,s=r+r,l=i+i,u=n*a,c=r*a,d=r*s,h=i*a,f=i*s,p=i*l,g=o*a,v=o*s,m=o*l;return e[0]=1-d-p,e[3]=c-m,e[6]=h+v,e[1]=c+m,e[4]=1-u-p,e[7]=f-g,e[2]=h-v,e[5]=f+g,e[8]=1-u-d,e},t.normalFromMat4=function(e,t){var n=t[0],r=t[1],i=t[2],o=t[3],a=t[4],s=t[5],l=t[6],u=t[7],c=t[8],d=t[9],h=t[10],f=t[11],p=t[12],g=t[13],v=t[14],m=t[15],y=n*s-r*a,b=n*l-i*a,C=n*u-o*a,A=r*l-i*s,_=r*u-o*s,I=i*u-o*l,x=c*g-d*p,w=c*v-h*p,M=c*m-f*p,S=d*v-h*g,T=d*m-f*g,E=h*m-f*v,O=y*E-b*T+C*S+A*M-_*w+I*x;return O?(O=1/O,e[0]=(s*E-l*T+u*S)*O,e[1]=(l*M-a*E-u*w)*O,e[2]=(a*T-s*M+u*x)*O,e[3]=(i*T-r*E-o*S)*O,e[4]=(n*E-i*M+o*w)*O,e[5]=(r*M-n*T-o*x)*O,e[6]=(g*I-v*_+m*A)*O,e[7]=(v*C-p*I-m*b)*O,e[8]=(p*_-g*C+m*y)*O,e):null},t.projection=function(e,t,n){return e[0]=2/t,e[1]=0,e[2]=0,e[3]=0,e[4]=-2/n,e[5]=0,e[6]=-1,e[7]=1,e[8]=1,e},t.str=function(e){return"mat3("+e[0]+", "+e[1]+", "+e[2]+", "+e[3]+", "+e[4]+", "+e[5]+", "+e[6]+", "+e[7]+", "+e[8]+")"},t.frob=function(e){return Math.sqrt(Math.pow(e[0],2)+Math.pow(e[1],2)+Math.pow(e[2],2)+Math.pow(e[3],2)+Math.pow(e[4],2)+Math.pow(e[5],2)+Math.pow(e[6],2)+Math.pow(e[7],2)+Math.pow(e[8],2))},t.add=function(e,t,n){return e[0]=t[0]+n[0],e[1]=t[1]+n[1],e[2]=t[2]+n[2],e[3]=t[3]+n[3],e[4]=t[4]+n[4],e[5]=t[5]+n[5],e[6]=t[6]+n[6],e[7]=t[7]+n[7],e[8]=t[8]+n[8],e},t.subtract=o,t.multiplyScalar=function(e,t,n){return e[0]=t[0]*n,e[1]=t[1]*n,e[2]=t[2]*n,e[3]=t[3]*n,e[4]=t[4]*n,e[5]=t[5]*n,e[6]=t[6]*n,e[7]=t[7]*n,e[8]=t[8]*n,e},t.multiplyScalarAndAdd=function(e,t,n,r){return e[0]=t[0]+n[0]*r,e[1]=t[1]+n[1]*r,e[2]=t[2]+n[2]*r,e[3]=t[3]+n[3]*r,e[4]=t[4]+n[4]*r,e[5]=t[5]+n[5]*r,e[6]=t[6]+n[6]*r,e[7]=t[7]+n[7]*r,e[8]=t[8]+n[8]*r,e},t.exactEquals=function(e,t){return e[0]===t[0]&&e[1]===t[1]&&e[2]===t[2]&&e[3]===t[3]&&e[4]===t[4]&&e[5]===t[5]&&e[6]===t[6]&&e[7]===t[7]&&e[8]===t[8]},t.equals=function(e,t){var n=e[0],i=e[1],o=e[2],a=e[3],s=e[4],l=e[5],u=e[6],c=e[7],d=e[8],h=t[0],f=t[1],p=t[2],g=t[3],v=t[4],m=t[5],y=t[6],b=t[7],C=t[8];return Math.abs(n-h)<=r.EPSILON*Math.max(1,Math.abs(n),Math.abs(h))&&Math.abs(i-f)<=r.EPSILON*Math.max(1,Math.abs(i),Math.abs(f))&&Math.abs(o-p)<=r.EPSILON*Math.max(1,Math.abs(o),Math.abs(p))&&Math.abs(a-g)<=r.EPSILON*Math.max(1,Math.abs(a),Math.abs(g))&&Math.abs(s-v)<=r.EPSILON*Math.max(1,Math.abs(s),Math.abs(v))&&Math.abs(l-m)<=r.EPSILON*Math.max(1,Math.abs(l),Math.abs(m))&&Math.abs(u-y)<=r.EPSILON*Math.max(1,Math.abs(u),Math.abs(y))&&Math.abs(c-b)<=r.EPSILON*Math.max(1,Math.abs(c),Math.abs(b))&&Math.abs(d-C)<=r.EPSILON*Math.max(1,Math.abs(d),Math.abs(C))};var r=function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)Object.prototype.hasOwnProperty.call(e,n)&&(t[n]=e[n]);return t.default=e,t}(n(104));function i(e,t,n){var r=t[0],i=t[1],o=t[2],a=t[3],s=t[4],l=t[5],u=t[6],c=t[7],d=t[8],h=n[0],f=n[1],p=n[2],g=n[3],v=n[4],m=n[5],y=n[6],b=n[7],C=n[8];return e[0]=h*r+f*a+p*u,e[1]=h*i+f*s+p*c,e[2]=h*o+f*l+p*d,e[3]=g*r+v*a+m*u,e[4]=g*i+v*s+m*c,e[5]=g*o+v*l+m*d,e[6]=y*r+b*a+C*u,e[7]=y*i+b*s+C*c,e[8]=y*o+b*l+C*d,e}function o(e,t,n){return e[0]=t[0]-n[0],e[1]=t[1]-n[1],e[2]=t[2]-n[2],e[3]=t[3]-n[3],e[4]=t[4]-n[4],e[5]=t[5]-n[5],e[6]=t[6]-n[6],e[7]=t[7]-n[7],e[8]=t[8]-n[8],e}t.mul=i,t.sub=o},function(e,t,n){"use strict";function r(e,t){for(var n in t)t.hasOwnProperty(n)&&"constructor"!==n&&void 0!==t[n]&&(e[n]=t[n])}Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t,n,i){return t&&r(e,t),n&&r(e,n),i&&r(e,i),e}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.isSamePoint=t.mergeRegion=t.intersectRect=t.inBox=t.distance=t.getPixelRatio=void 0,t.getPixelRatio=function(){return window?window.devicePixelRatio:1},t.distance=function(e,t,n,r){var i=e-n,o=t-r;return Math.sqrt(i*i+o*o)},t.inBox=function(e,t,n,r,i,o){return i>=e&&i<=e+n&&o>=t&&o<=t+r},t.intersectRect=function(e,t){return!(t.minX>e.maxX||t.maxX<e.minX||t.minY>e.maxY||t.maxY<e.minY)},t.mergeRegion=function(e,t){return e&&t?{minX:Math.min(e.minX,t.minX),minY:Math.min(e.minY,t.minY),maxX:Math.max(e.maxX,t.maxX),maxY:Math.max(e.maxY,t.maxY)}:e||t},t.isSamePoint=function(e,t){return e[0]===t[0]&&e[1]===t[1]};var r=n(10);Object.defineProperty(t,"isNil",{enumerable:!0,get:function(){return r.default}});var i=n(3);Object.defineProperty(t,"isString",{enumerable:!0,get:function(){return i.default}});var o=n(14);Object.defineProperty(t,"isFunction",{enumerable:!0,get:function(){return o.default}});var a=n(4);Object.defineProperty(t,"isArray",{enumerable:!0,get:function(){return a.default}});var s=n(2);Object.defineProperty(t,"each",{enumerable:!0,get:function(){return s.default}});var l=n(68);Object.defineProperty(t,"toRadian",{enumerable:!0,get:function(){return l.default}});var u=n(67);Object.defineProperty(t,"mod",{enumerable:!0,get:function(){return u.default}});var c=n(103);Object.defineProperty(t,"isNumberEqual",{enumerable:!0,get:function(){return c.default}});var d=n(147);Object.defineProperty(t,"requestAnimationFrame",{enumerable:!0,get:function(){return d.default}});var h=n(148);Object.defineProperty(t,"clearAnimationFrame",{enumerable:!0,get:function(){return h.default}})},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.EVENTS=t.SVG_ATTR_MAP=t.SHAPE_TO_TAGS=void 0,t.SHAPE_TO_TAGS={rect:"path",circle:"circle",line:"line",path:"path",marker:"path",text:"text",polyline:"polyline",polygon:"polygon",image:"image",ellipse:"ellipse",dom:"foreignObject"},t.SVG_ATTR_MAP={opacity:"opacity",fillStyle:"fill",fill:"fill",fillOpacity:"fill-opacity",strokeStyle:"stroke",strokeOpacity:"stroke-opacity",stroke:"stroke",x:"x",y:"y",r:"r",rx:"rx",ry:"ry",width:"width",height:"height",x1:"x1",x2:"x2",y1:"y1",y2:"y2",lineCap:"stroke-linecap",lineJoin:"stroke-linejoin",lineWidth:"stroke-width",lineDash:"stroke-dasharray",lineDashOffset:"stroke-dashoffset",miterLimit:"stroke-miterlimit",font:"font",fontSize:"font-size",fontStyle:"font-style",fontVariant:"font-variant",fontWeight:"font-weight",fontFamily:"font-family",startArrow:"marker-start",endArrow:"marker-end",path:"d",class:"class",id:"id",style:"style",preserveAspectRatio:"preserveAspectRatio"},t.EVENTS=["click","mousedown","mouseup","dblclick","contextmenu","mouseenter","mouseleave","mouseover","mouseout","mousemove","wheel"]},function(e,t,n){"use strict";var r=n(13),i=n(30).Graph;function o(e,t,n,i){var o;do{o=r.uniqueId(i)}while(e.hasNode(o));return n.dummy=t,e.setNode(o,n),o}function a(e){return r.max(r.map(e.nodes(),(function(t){var n=e.node(t).rank;if(!r.isUndefined(n))return n})))}e.exports={addDummyNode:o,simplify:function(e){var t=(new i).setGraph(e.graph());return r.forEach(e.nodes(),(function(n){t.setNode(n,e.node(n))})),r.forEach(e.edges(),(function(n){var r=t.edge(n.v,n.w)||{weight:0,minlen:1},i=e.edge(n);t.setEdge(n.v,n.w,{weight:r.weight+i.weight,minlen:Math.max(r.minlen,i.minlen)})})),t},asNonCompoundGraph:function(e){var t=new i({multigraph:e.isMultigraph()}).setGraph(e.graph());return r.forEach(e.nodes(),(function(n){e.children(n).length||t.setNode(n,e.node(n))})),r.forEach(e.edges(),(function(n){t.setEdge(n,e.edge(n))})),t},successorWeights:function(e){var t=r.map(e.nodes(),(function(t){var n={};return r.forEach(e.outEdges(t),(function(t){n[t.w]=(n[t.w]||0)+e.edge(t).weight})),n}));return r.zipObject(e.nodes(),t)},predecessorWeights:function(e){var t=r.map(e.nodes(),(function(t){var n={};return r.forEach(e.inEdges(t),(function(t){n[t.v]=(n[t.v]||0)+e.edge(t).weight})),n}));return r.zipObject(e.nodes(),t)},intersectRect:function(e,t){var n,r,i=e.x,o=e.y,a=t.x-i,s=t.y-o,l=e.width/2,u=e.height/2;if(!a&&!s)throw new Error("Not possible to find intersection inside of the rectangle");return Math.abs(s)*l>Math.abs(a)*u?(s<0&&(u=-u),n=u*a/s,r=u):(a<0&&(l=-l),n=l,r=l*s/a),{x:i+n,y:o+r}},buildLayerMatrix:function(e){var t=r.map(r.range(a(e)+1),(function(){return[]}));return r.forEach(e.nodes(),(function(n){var i=e.node(n),o=i.rank;r.isUndefined(o)||(t[o][i.order]=n)})),t},normalizeRanks:function(e){var t=r.min(r.map(e.nodes(),(function(t){return e.node(t).rank})));r.forEach(e.nodes(),(function(n){var i=e.node(n);r.has(i,"rank")&&(i.rank-=t)}))},removeEmptyRanks:function(e){var t=r.min(r.map(e.nodes(),(function(t){return e.node(t).rank}))),n=[];r.forEach(e.nodes(),(function(r){var i=e.node(r).rank-t;n[i]||(n[i]=[]),n[i].push(r)}));var i=0,o=e.graph().nodeRankFactor;r.forEach(n,(function(t,n){r.isUndefined(t)&&n%o!=0?--i:i&&r.forEach(t,(function(t){e.node(t).rank+=i}))}))},addBorderNode:function(e,t,n,r){var i={width:0,height:0};return arguments.length>=4&&(i.rank=n,i.order=r),o(e,"border",i,t)},maxRank:a,partition:function(e,t){var n={lhs:[],rhs:[]};return r.forEach(e,(function(e){t(e)?n.lhs.push(e):n.rhs.push(e)})),n},time:function(e,t){var n=r.now();try{return t()}finally{console.log(e+" time: "+(r.now()-n)+"ms")}},notime:function(e,t){return t()}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){return null!==e&&"function"!=typeof e&&isFinite(e.length)}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(172);t.mat3=r.default;var i=n(368);t.vec2=i.default;var o=n(369);t.vec3=o.default;var a=n(370);t.transform=a.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(1),i=n(43),o=n(61),a=n(29),s=n(109),l=n(19),u=n(110),c=n(171),d=n(157),h=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.type="svg",t.canFill=!1,t.canStroke=!1,t}return r.__extends(t,e),t.prototype.getDefaultAttrs=function(){var t=e.prototype.getDefaultAttrs.call(this);return r.__assign(r.__assign({},t),{lineWidth:1,lineAppendWidth:0,strokeOpacity:1,fillOpacity:1})},t.prototype.afterAttrsChange=function(t){e.prototype.afterAttrsChange.call(this,t);var n=this.get("canvas");if(n&&n.get("autoDraw")){var r=n.get("context");this.draw(r,t)}},t.prototype.getShapeBase=function(){return u},t.prototype.getGroupBase=function(){return c.default},t.prototype.onCanvasChange=function(e){s.refreshElement(this,e)},t.prototype.calculateBBox=function(){var e=this.get("el"),t=null;if(e)t=e.getBBox();else{var n=d.getBBoxMethod(this.get("type"));n&&(t=n(this))}if(t){var r=t.x,i=t.y,o=t.width,a=t.height,s=this.getHitLineWidth(),l=s/2,u=r-l,c=i-l;return{x:u,y:c,minX:u,minY:c,maxX:r+o+l,maxY:i+a+l,width:o+s,height:a+s}}return{x:0,y:0,minX:0,minY:0,maxX:0,maxY:0,width:0,height:0}},t.prototype.isFill=function(){var e=this.attr(),t=e.fill,n=e.fillStyle;return(t||n||this.isClipShape())&&this.canFill},t.prototype.isStroke=function(){var e=this.attr(),t=e.stroke,n=e.strokeStyle;return(t||n)&&this.canStroke},t.prototype.draw=function(e,t){var n=this.get("el");this.get("destroyed")?n&&n.parentNode.removeChild(n):(n||a.createDom(this),o.setClip(this,e),this.createPath(e,t),this.shadow(e,t),this.strokeAndFill(e,t),this.transform(t))},t.prototype.createPath=function(e,t){},t.prototype.strokeAndFill=function(e,t){var n=t||this.attr(),r=n.fill,i=n.fillStyle,o=n.stroke,a=n.strokeStyle,s=n.fillOpacity,u=n.strokeOpacity,c=n.lineWidth,d=this.get("el");this.canFill&&(t?"fill"in n?this._setColor(e,"fill",r):"fillStyle"in n&&this._setColor(e,"fill",i):this._setColor(e,"fill",r||i),s&&d.setAttribute(l.SVG_ATTR_MAP.fillOpacity,s)),this.canStroke&&c>0&&(t?"stroke"in n?this._setColor(e,"stroke",o):"strokeStyle"in n&&this._setColor(e,"stroke",a):this._setColor(e,"stroke",o||a),u&&d.setAttribute(l.SVG_ATTR_MAP.strokeOpacity,u),c&&d.setAttribute(l.SVG_ATTR_MAP.lineWidth,c))},t.prototype._setColor=function(e,t,n){var r=this.get("el");if(n)if(n=n.trim(),/^[r,R,L,l]{1}[\s]*\(/.test(n))(i=e.find("gradient",n))||(i=e.addGradient(n)),r.setAttribute(l.SVG_ATTR_MAP[t],"url(#"+i+")");else if(/^[p,P]{1}[\s]*\(/.test(n)){var i;(i=e.find("pattern",n))||(i=e.addPattern(n)),r.setAttribute(l.SVG_ATTR_MAP[t],"url(#"+i+")")}else r.setAttribute(l.SVG_ATTR_MAP[t],n);else r.setAttribute(l.SVG_ATTR_MAP[t],"none")},t.prototype.shadow=function(e,t){var n=this.attr(),r=t||n,i=r.shadowOffsetX,a=r.shadowOffsetY,s=r.shadowBlur,l=r.shadowColor;(i||a||s||l)&&o.setShadow(this,e)},t.prototype.transform=function(e){var t=this.attr();(e||t).matrix&&o.setTransform(this)},t.prototype.isInShape=function(e,t){return this.isPointInPath(e,t)},t.prototype.isPointInPath=function(e,t){var n=this.get("el"),r=this.get("canvas").get("el").getBoundingClientRect(),i=e+r.left,o=t+r.top,a=document.elementFromPoint(i,o);return!(!a||!a.isEqualNode(n))},t.prototype.getHitLineWidth=function(){var e=this.attrs,t=e.lineWidth,n=e.lineAppendWidth;return this.isStroke()?t+n:0},t}(i.AbstractShape);t.default=h},function(e,t,n){var r;try{r={clone:n(373),constant:n(122),each:n(191),filter:n(194),has:n(205),isArray:n(15),isEmpty:n(450),isFunction:n(62),isUndefined:n(206),keys:n(41),map:n(207),reduce:n(209),size:n(453),transform:n(459),union:n(460),values:n(214)}}catch(e){}r||(r=window._),e.exports=r},function(e,t){function n(e){return(n="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}e.exports=function(e){var t=n(e);return null!=e&&("object"==t||"function"==t)}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.forEach=t.sqrLen=t.sqrDist=t.dist=t.div=t.mul=t.sub=t.len=void 0,t.create=o,t.clone=function(e){var t=new i.ARRAY_TYPE(2);return t[0]=e[0],t[1]=e[1],t},t.fromValues=function(e,t){var n=new i.ARRAY_TYPE(2);return n[0]=e,n[1]=t,n},t.copy=function(e,t){return e[0]=t[0],e[1]=t[1],e},t.set=function(e,t,n){return e[0]=t,e[1]=n,e},t.add=function(e,t,n){return e[0]=t[0]+n[0],e[1]=t[1]+n[1],e},t.subtract=a,t.multiply=s,t.divide=l,t.ceil=function(e,t){return e[0]=Math.ceil(t[0]),e[1]=Math.ceil(t[1]),e},t.floor=function(e,t){return e[0]=Math.floor(t[0]),e[1]=Math.floor(t[1]),e},t.min=function(e,t,n){return e[0]=Math.min(t[0],n[0]),e[1]=Math.min(t[1],n[1]),e},t.max=function(e,t,n){return e[0]=Math.max(t[0],n[0]),e[1]=Math.max(t[1],n[1]),e},t.round=function(e,t){return e[0]=Math.round(t[0]),e[1]=Math.round(t[1]),e},t.scale=function(e,t,n){return e[0]=t[0]*n,e[1]=t[1]*n,e},t.scaleAndAdd=function(e,t,n,r){return e[0]=t[0]+n[0]*r,e[1]=t[1]+n[1]*r,e},t.distance=u,t.squaredDistance=c,t.length=d,t.squaredLength=h,t.negate=function(e,t){return e[0]=-t[0],e[1]=-t[1],e},t.inverse=function(e,t){return e[0]=1/t[0],e[1]=1/t[1],e},t.normalize=function(e,t){var n=t[0],r=t[1],i=n*n+r*r;return i>0&&(i=1/Math.sqrt(i),e[0]=t[0]*i,e[1]=t[1]*i),e},t.dot=function(e,t){return e[0]*t[0]+e[1]*t[1]},t.cross=function(e,t,n){var r=t[0]*n[1]-t[1]*n[0];return e[0]=e[1]=0,e[2]=r,e},t.lerp=function(e,t,n,r){var i=t[0],o=t[1];return e[0]=i+r*(n[0]-i),e[1]=o+r*(n[1]-o),e},t.random=function(e,t){t=t||1;var n=2*i.RANDOM()*Math.PI;return e[0]=Math.cos(n)*t,e[1]=Math.sin(n)*t,e},t.transformMat2=function(e,t,n){var r=t[0],i=t[1];return e[0]=n[0]*r+n[2]*i,e[1]=n[1]*r+n[3]*i,e},t.transformMat2d=function(e,t,n){var r=t[0],i=t[1];return e[0]=n[0]*r+n[2]*i+n[4],e[1]=n[1]*r+n[3]*i+n[5],e},t.transformMat3=function(e,t,n){var r=t[0],i=t[1];return e[0]=n[0]*r+n[3]*i+n[6],e[1]=n[1]*r+n[4]*i+n[7],e},t.transformMat4=function(e,t,n){var r=t[0],i=t[1];return e[0]=n[0]*r+n[4]*i+n[12],e[1]=n[1]*r+n[5]*i+n[13],e},t.rotate=function(e,t,n,r){var i=t[0]-n[0],o=t[1]-n[1],a=Math.sin(r),s=Math.cos(r);return e[0]=i*s-o*a+n[0],e[1]=i*a+o*s+n[1],e},t.angle=function(e,t){var n=e[0],r=e[1],i=t[0],o=t[1],a=n*n+r*r;a>0&&(a=1/Math.sqrt(a));var s=i*i+o*o;s>0&&(s=1/Math.sqrt(s));var l=(n*i+r*o)*a*s;return l>1?0:l<-1?Math.PI:Math.acos(l)},t.str=function(e){return"vec2("+e[0]+", "+e[1]+")"},t.exactEquals=function(e,t){return e[0]===t[0]&&e[1]===t[1]},t.equals=function(e,t){var n=e[0],r=e[1],o=t[0],a=t[1];return Math.abs(n-o)<=i.EPSILON*Math.max(1,Math.abs(n),Math.abs(o))&&Math.abs(r-a)<=i.EPSILON*Math.max(1,Math.abs(r),Math.abs(a))};var r,i=function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)Object.prototype.hasOwnProperty.call(e,n)&&(t[n]=e[n]);return t.default=e,t}(n(104));function o(){var e=new i.ARRAY_TYPE(2);return i.ARRAY_TYPE!=Float32Array&&(e[0]=0,e[1]=0),e}function a(e,t,n){return e[0]=t[0]-n[0],e[1]=t[1]-n[1],e}function s(e,t,n){return e[0]=t[0]*n[0],e[1]=t[1]*n[1],e}function l(e,t,n){return e[0]=t[0]/n[0],e[1]=t[1]/n[1],e}function u(e,t){var n=t[0]-e[0],r=t[1]-e[1];return Math.sqrt(n*n+r*r)}function c(e,t){var n=t[0]-e[0],r=t[1]-e[1];return n*n+r*r}function d(e){var t=e[0],n=e[1];return Math.sqrt(t*t+n*n)}function h(e){var t=e[0],n=e[1];return t*t+n*n}t.len=d,t.sub=a,t.mul=s,t.div=l,t.dist=u,t.sqrDist=c,t.sqrLen=h,t.forEach=(r=o(),function(e,t,n,i,o,a){var s,l=void 0;for(t||(t=2),n||(n=0),s=i?Math.min(i*t+n,e.length):e.length,l=n;l<s;l+=t)r[0]=e[l],r[1]=e[l+1],o(r,r,a),e[l]=r[0],e[l+1]=r[1];return e})},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r={}.toString;t.default=function(e,t){return r.call(e)==="[object "+t+"]"}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(1),i=n(43),o=n(18),a=n(60),s=n(157),l=n(73),u=n(164),c=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return r.__extends(t,e),t.prototype.getDefaultAttrs=function(){var t=e.prototype.getDefaultAttrs.call(this);return r.__assign(r.__assign({},t),{lineWidth:1,lineAppendWidth:0,strokeOpacity:1,fillOpacity:1})},t.prototype.getShapeBase=function(){return l},t.prototype.getGroupBase=function(){return u.default},t.prototype.onCanvasChange=function(e){a.refreshElement(this,e)},t.prototype.calculateBBox=function(){var e=this.get("type"),t=this.getHitLineWidth(),n=s.getBBoxMethod(e)(this),r=t/2,i=n.x-r,o=n.y-r,a=n.x+n.width+r,l=n.y+n.height+r;return{x:i,minX:i,y:o,minY:o,width:n.width+t,height:n.height+t,maxX:a,maxY:l}},t.prototype.isFill=function(){return!!this.attrs.fill||this.isClipShape()},t.prototype.isStroke=function(){return!!this.attrs.stroke},t.prototype._applyClip=function(e,t){t&&(e.save(),a.applyAttrsToContext(e,t),t.createPath(e),e.restore(),e.clip(),t._afterDraw())},t.prototype.draw=function(e,t){var n=this.cfg.clipShape;if(t){if(!1===this.cfg.refresh)return void this.set("hasChanged",!1);var r=this.getCanvasBBox();if(!o.intersectRect(t,r))return this.set("hasChanged",!1),void(this.cfg.isInView&&this._afterDraw())}e.save(),a.applyAttrsToContext(e,this),this._applyClip(e,n),this.drawPath(e),e.restore(),this._afterDraw()},t.prototype.getCanvasViewBox=function(){var e=this.cfg.canvas;return e?e.getViewRange():null},t.prototype.cacheCanvasBBox=function(){var e=this.getCanvasViewBox();if(e){var t=this.getCanvasBBox(),n=o.intersectRect(t,e);this.set("isInView",n),n?this.set("cacheCanvasBBox",t):this.set("cacheCanvasBBox",null)}},t.prototype._afterDraw=function(){this.cacheCanvasBBox(),this.set("hasChanged",!1),this.set("refresh",null)},t.prototype.skipDraw=function(){this.set("cacheCanvasBBox",null),this.set("isInView",null),this.set("hasChanged",!1)},t.prototype.drawPath=function(e){this.createPath(e),this.strokeAndFill(e),this.afterDrawPath(e)},t.prototype.fill=function(e){e.fill()},t.prototype.stroke=function(e){e.stroke()},t.prototype.strokeAndFill=function(e){var t=this.attrs,n=t.lineWidth,r=t.opacity,i=t.strokeOpacity,a=t.fillOpacity;this.isFill()&&(o.isNil(a)||1===a?this.fill(e):(e.globalAlpha=a,this.fill(e),e.globalAlpha=r)),this.isStroke()&&n>0&&(o.isNil(i)||1===i||(e.globalAlpha=i),this.stroke(e)),this.afterDrawPath(e)},t.prototype.createPath=function(e){},t.prototype.afterDrawPath=function(e){},t.prototype.isInShape=function(e,t){var n=this.isStroke(),r=this.isFill(),i=this.getHitLineWidth();return this.isInStrokeOrPath(e,t,n,r,i)},t.prototype.isInStrokeOrPath=function(e,t,n,r,i){return!1},t.prototype.getHitLineWidth=function(){if(!this.isStroke())return 0;var e=this.attrs;return e.lineWidth+e.lineAppendWidth},t}(i.AbstractShape);t.default=c},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.moveTo=t.sortDom=t.createDom=t.createSVGElement=void 0;var r=n(0),i=n(19);function o(e){return document.createElementNS("http://www.w3.org/2000/svg",e)}t.createSVGElement=o,t.createDom=function(e){var t=i.SHAPE_TO_TAGS[e.type],n=e.getParent();if(!t)throw new Error("the type "+e.type+" is not supported by svg");var r=o(t);if(e.get("id")&&(r.id=e.get("id")),e.set("el",r),e.set("attrs",{}),n){var a=n.get("el");a||(a=n.createDom(),n.set("el",a)),a.appendChild(r)}return r},t.sortDom=function(e,t){var n=e.get("el"),i=r.toArray(n.children).sort(t),o=document.createDocumentFragment();i.forEach((function(e){o.appendChild(e)})),n.appendChild(o)},t.moveTo=function(e,t){var n=e.parentNode,r=Array.from(n.childNodes).filter((function(e){return 1===e.nodeType&&"defs"!==e.nodeName.toLowerCase()})),i=r[t],o=r.indexOf(e);if(i){if(o>t)n.insertBefore(e,i);else if(o<t){var a=r[t+1];a?n.insertBefore(e,a):n.appendChild(e)}}else n.appendChild(e)}},function(e,t,n){var r;try{r=n(371)}catch(e){}r||(r=window.graphlib),e.exports=r},function(e,t,n){function r(e){return(r="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}var i=n(174),o="object"==("undefined"==typeof self?"undefined":r(self))&&self&&self.Object===Object&&self,a=i||o||Function("return this")();e.exports=a},function(e,t,n){"use strict";function r(e,t){var n=[],r=e[0],i=e[1],o=e[2],a=e[3],s=e[4],l=e[5],u=e[6],c=e[7],d=e[8],h=t[0],f=t[1],p=t[2],g=t[3],v=t[4],m=t[5],y=t[6],b=t[7],C=t[8];return n[0]=h*r+f*a+p*u,n[1]=h*i+f*s+p*c,n[2]=h*o+f*l+p*d,n[3]=g*r+v*a+m*u,n[4]=g*i+v*s+m*c,n[5]=g*o+v*l+m*d,n[6]=y*r+b*a+C*u,n[7]=y*i+b*s+C*c,n[8]=y*o+b*l+C*d,n}function i(e,t){var n=[],r=t[0],i=t[1];return n[0]=e[0]*r+e[3]*i+e[6],n[1]=e[1]*r+e[4]*i+e[7],n}function o(e){var t=[],n=e[0],r=e[1],i=e[2],o=e[3],a=e[4],s=e[5],l=e[6],u=e[7],c=e[8],d=c*a-s*u,h=-c*o+s*l,f=u*o-a*l,p=n*d+r*h+i*f;return p?(p=1/p,t[0]=d*p,t[1]=(-c*r+i*u)*p,t[2]=(s*r-i*a)*p,t[3]=h*p,t[4]=(c*n-i*l)*p,t[5]=(-s*n+i*o)*p,t[6]=f*p,t[7]=(-u*n+r*l)*p,t[8]=(a*n-r*o)*p,t):null}n.d(t,"b",(function(){return r})),n.d(t,"c",(function(){return i})),n.d(t,"a",(function(){return o}))},function(e,t,n){"use strict";function r(e){return Math.min.apply(null,e)}function i(e){return Math.max.apply(null,e)}Object.defineProperty(t,"__esModule",{value:!0}),t.piMod=t.getBBoxRange=t.getBBoxByArray=t.isNumberEqual=t.distance=void 0,t.distance=function(e,t,n,r){var i=e-n,o=t-r;return Math.sqrt(i*i+o*o)},t.isNumberEqual=function(e,t){return Math.abs(e-t)<.001},t.getBBoxByArray=function(e,t){var n=r(e),o=r(t);return{x:n,y:o,width:i(e)-n,height:i(t)-o}},t.getBBoxRange=function(e,t,n,o){return{minX:r([e,n]),maxX:i([e,n]),minY:r([t,o]),maxY:i([t,o])}},t.piMod=function(e){return(e+2*Math.PI)%(2*Math.PI)}},function(e,t){function n(e){return(n="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}e.exports=function(e){return null!=e&&"object"==n(e)}},function(e,t,n){"use strict";function r(e){return(r="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){var t=r(e);return null!==e&&"object"===t||"function"===t}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(1),i=n(43),o=n(319),a=n(73),s=n(164),l=n(18),u=n(60),c=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return r.__extends(t,e),t.prototype.getDefaultCfg=function(){var t=e.prototype.getDefaultCfg.call(this);return t.renderer="canvas",t.autoDraw=!0,t.localRefresh=!0,t.refreshElements=[],t.clipView=!0,t.quickHit=!1,t},t.prototype.onCanvasChange=function(e){"attr"!==e&&"sort"!==e&&"changeSize"!==e||(this.set("refreshElements",[this]),this.draw())},t.prototype.getShapeBase=function(){return a},t.prototype.getGroupBase=function(){return s.default},t.prototype.getPixelRatio=function(){var e=this.get("pixelRatio")||l.getPixelRatio();return e>=1?Math.ceil(e):1},t.prototype.getViewRange=function(){return{minX:0,minY:0,maxX:this.cfg.width,maxY:this.cfg.height}},t.prototype.createDom=function(){var e=document.createElement("canvas"),t=e.getContext("2d");return this.set("context",t),e},t.prototype.setDOMSize=function(t,n){e.prototype.setDOMSize.call(this,t,n);var r=this.get("context"),i=this.get("el"),o=this.getPixelRatio();i.width=o*t,i.height=o*n,o>1&&r.scale(o,o)},t.prototype.clear=function(){e.prototype.clear.call(this),this._clearFrame();var t=this.get("context"),n=this.get("el");t.clearRect(0,0,n.width,n.height)},t.prototype.getShape=function(t,n){return this.get("quickHit")?o.getShape(this,t,n):e.prototype.getShape.call(this,t,n,null)},t.prototype._getRefreshRegion=function(){var e,t=this.get("refreshElements"),n=this.getViewRange();return t.length&&t[0]===this?e=n:(e=u.getMergedRegion(t))&&(e.minX=Math.floor(e.minX),e.minY=Math.floor(e.minY),e.maxX=Math.ceil(e.maxX),e.maxY=Math.ceil(e.maxY),e.maxY+=1,this.get("clipView")&&(e=u.mergeView(e,n))),e},t.prototype.refreshElement=function(e){this.get("refreshElements").push(e)},t.prototype._clearFrame=function(){var e=this.get("drawFrame");e&&(l.clearAnimationFrame(e),this.set("drawFrame",null),this.set("refreshElements",[]))},t.prototype.draw=function(){var e=this.get("drawFrame");this.get("autoDraw")&&e||this._startDraw()},t.prototype._drawAll=function(){var e=this.get("context"),t=this.get("el"),n=this.getChildren();e.clearRect(0,0,t.width,t.height),u.applyAttrsToContext(e,this),u.drawChildren(e,n),this.set("refreshElements",[])},t.prototype._drawRegion=function(){var e=this.get("context"),t=this.get("refreshElements"),n=this.getChildren(),r=this._getRefreshRegion();r?(e.clearRect(r.minX,r.minY,r.maxX-r.minX,r.maxY-r.minY),e.save(),e.beginPath(),e.rect(r.minX,r.minY,r.maxX-r.minX,r.maxY-r.minY),e.clip(),u.applyAttrsToContext(e,this),u.checkRefresh(this,n,r),u.drawChildren(e,n,r),e.restore()):t.length&&u.clearChanged(t),l.each(t,(function(e){e.get("hasChanged")&&e.set("hasChanged",!1)})),this.set("refreshElements",[])},t.prototype._startDraw=function(){var e=this,t=this.get("drawFrame");t||(t=l.requestAnimationFrame((function(){e.get("localRefresh")?e._drawRegion():e._drawAll(),e.set("drawFrame",null)})),this.set("drawFrame",t))},t.prototype.skipDraw=function(){},t}(i.AbstractCanvas);t.default=c},function(e,t,n){"use strict";n.r(t),n.d(t,"catmullRomToBezier",(function(){return l})),n.d(t,"fillPath",(function(){return E})),n.d(t,"fillPathByDiff",(function(){return L})),n.d(t,"formatPath",(function(){return N})),n.d(t,"intersection",(function(){return S})),n.d(t,"parsePathArray",(function(){return v})),n.d(t,"parsePathString",(function(){return s})),n.d(t,"pathToAbsolute",(function(){return c})),n.d(t,"pathToCurve",(function(){return p})),n.d(t,"rectPath",(function(){return _}));var r=n(0),i="\t\n\v\f\r   ᠎             　\u2028\u2029",o=new RegExp("([a-z])["+i+",]*((-?\\d*\\.?\\d*(?:e[\\-+]?\\d+)?["+i+"]*,?["+i+"]*)+)","ig"),a=new RegExp("(-?\\d*\\.?\\d*(?:e[\\-+]?\\d+)?)["+i+"]*,?["+i+"]*","ig"),s=function(e){if(!e)return null;if(Object(r.isArray)(e))return e;var t={a:7,c:6,o:2,h:1,l:2,m:2,r:4,q:4,s:4,t:2,v:1,u:3,z:0},n=[];return String(e).replace(o,(function(r,i,o){var s=[],l=i.toLowerCase();if(o.replace(a,(function(e,t){t&&s.push(+t)})),"m"===l&&s.length>2&&(n.push([i].concat(s.splice(0,2))),l="l",i="m"===i?"l":"L"),"o"===l&&1===s.length&&n.push([i,s[0]]),"r"===l)n.push([i].concat(s));else for(;s.length>=t[l]&&(n.push([i].concat(s.splice(0,t[l]))),t[l]););return e})),n},l=function(e,t){for(var n=[],r=0,i=e.length;i-2*!t>r;r+=2){var o=[{x:+e[r-2],y:+e[r-1]},{x:+e[r],y:+e[r+1]},{x:+e[r+2],y:+e[r+3]},{x:+e[r+4],y:+e[r+5]}];t?r?i-4===r?o[3]={x:+e[0],y:+e[1]}:i-2===r&&(o[2]={x:+e[0],y:+e[1]},o[3]={x:+e[2],y:+e[3]}):o[0]={x:+e[i-2],y:+e[i-1]}:i-4===r?o[3]=o[2]:r||(o[0]={x:+e[r],y:+e[r+1]}),n.push(["C",(-o[0].x+6*o[1].x+o[2].x)/6,(-o[0].y+6*o[1].y+o[2].y)/6,(o[1].x+6*o[2].x-o[3].x)/6,(o[1].y+6*o[2].y-o[3].y)/6,o[2].x,o[2].y])}return n},u=function(e,t,n,r,i){var o=[];if(null===i&&null===r&&(r=n),e=+e,t=+t,n=+n,r=+r,null!==i){var a=Math.PI/180,s=e+n*Math.cos(-r*a),l=e+n*Math.cos(-i*a);o=[["M",s,t+n*Math.sin(-r*a)],["A",n,n,0,+(i-r>180),0,l,t+n*Math.sin(-i*a)]]}else o=[["M",e,t],["m",0,-r],["a",n,r,0,1,1,0,2*r],["a",n,r,0,1,1,0,-2*r],["z"]];return o},c=function(e){if(!(e=s(e))||!e.length)return[["M",0,0]];var t,n,r=[],i=0,o=0,a=0,c=0,d=0;"M"===e[0][0]&&(a=i=+e[0][1],c=o=+e[0][2],d++,r[0]=["M",i,o]);for(var h=3===e.length&&"M"===e[0][0]&&"R"===e[1][0].toUpperCase()&&"Z"===e[2][0].toUpperCase(),f=void 0,p=void 0,g=d,v=e.length;g<v;g++){if(r.push(f=[]),(t=(p=e[g])[0])!==t.toUpperCase())switch(f[0]=t.toUpperCase(),f[0]){case"A":f[1]=p[1],f[2]=p[2],f[3]=p[3],f[4]=p[4],f[5]=p[5],f[6]=+p[6]+i,f[7]=+p[7]+o;break;case"V":f[1]=+p[1]+o;break;case"H":f[1]=+p[1]+i;break;case"R":for(var m=2,y=(n=[i,o].concat(p.slice(1))).length;m<y;m++)n[m]=+n[m]+i,n[++m]=+n[m]+o;r.pop(),r=r.concat(l(n,h));break;case"O":r.pop(),(n=u(i,o,p[1],p[2])).push(n[0]),r=r.concat(n);break;case"U":r.pop(),r=r.concat(u(i,o,p[1],p[2],p[3])),f=["U"].concat(r[r.length-1].slice(-2));break;case"M":a=+p[1]+i,c=+p[2]+o;break;default:for(m=1,y=p.length;m<y;m++)f[m]=+p[m]+(m%2?i:o)}else if("R"===t)n=[i,o].concat(p.slice(1)),r.pop(),r=r.concat(l(n,h)),f=["R"].concat(p.slice(-2));else if("O"===t)r.pop(),(n=u(i,o,p[1],p[2])).push(n[0]),r=r.concat(n);else if("U"===t)r.pop(),r=r.concat(u(i,o,p[1],p[2],p[3])),f=["U"].concat(r[r.length-1].slice(-2));else for(var b=0,C=p.length;b<C;b++)f[b]=p[b];if("O"!==(t=t.toUpperCase()))switch(f[0]){case"Z":i=+a,o=+c;break;case"H":i=f[1];break;case"V":o=f[1];break;case"M":a=f[f.length-2],c=f[f.length-1];break;default:i=f[f.length-2],o=f[f.length-1]}}return r},d=function(e,t,n,r){return[e,t,n,r,n,r]},h=function(e,t,n,r,i,o){return[1/3*e+2/3*n,1/3*t+2/3*r,1/3*i+2/3*n,1/3*o+2/3*r,i,o]},f=function e(t,n,r,i,o,a,s,l,u,c){r===i&&(r+=1);var d,h,f,p,g,v=120*Math.PI/180,m=Math.PI/180*(+o||0),y=[],b=function(e,t,n){return{x:e*Math.cos(n)-t*Math.sin(n),y:e*Math.sin(n)+t*Math.cos(n)}};if(c)h=c[0],f=c[1],p=c[2],g=c[3];else{t=(d=b(t,n,-m)).x,n=d.y,l=(d=b(l,u,-m)).x,u=d.y,t===l&&n===u&&(l+=1,u+=1);var C=(t-l)/2,A=(n-u)/2,_=C*C/(r*r)+A*A/(i*i);_>1&&(r*=_=Math.sqrt(_),i*=_);var I=r*r,x=i*i,w=(a===s?-1:1)*Math.sqrt(Math.abs((I*x-I*A*A-x*C*C)/(I*A*A+x*C*C)));p=w*r*A/i+(t+l)/2,g=w*-i*C/r+(n+u)/2,h=Math.asin(((n-g)/i).toFixed(9)),f=Math.asin(((u-g)/i).toFixed(9)),h=t<p?Math.PI-h:h,f=l<p?Math.PI-f:f,h<0&&(h=2*Math.PI+h),f<0&&(f=2*Math.PI+f),s&&h>f&&(h-=2*Math.PI),!s&&f>h&&(f-=2*Math.PI)}var M=f-h;if(Math.abs(M)>v){var S=f,T=l,E=u;f=h+v*(s&&f>h?1:-1),y=e(l=p+r*Math.cos(f),u=g+i*Math.sin(f),r,i,o,0,s,T,E,[f,S,p,g])}M=f-h;var O=Math.cos(h),k=Math.sin(h),L=Math.cos(f),D=Math.sin(f),P=Math.tan(M/4),N=4/3*r*P,j=4/3*i*P,R=[t,n],z=[t+N*k,n-j*O],F=[l+N*D,u-j*L],B=[l,u];if(z[0]=2*R[0]-z[0],z[1]=2*R[1]-z[1],c)return[z,F,B].concat(y);for(var H=[],U=0,V=(y=[z,F,B].concat(y).join().split(",")).length;U<V;U++)H[U]=U%2?b(y[U-1],y[U],m).y:b(y[U],y[U+1],m).x;return H},p=function(e,t){var n,r=c(e),i=t&&c(t),o={x:0,y:0,bx:0,by:0,X:0,Y:0,qx:null,qy:null},a={x:0,y:0,bx:0,by:0,X:0,Y:0,qx:null,qy:null},s=[],l=[],u="",p="",g=function(e,t,n){var r,i;if(!e)return["C",t.x,t.y,t.x,t.y,t.x,t.y];switch(!(e[0]in{T:1,Q:1})&&(t.qx=t.qy=null),e[0]){case"M":t.X=e[1],t.Y=e[2];break;case"A":e=["C"].concat(f.apply(0,[t.x,t.y].concat(e.slice(1))));break;case"S":"C"===n||"S"===n?(r=2*t.x-t.bx,i=2*t.y-t.by):(r=t.x,i=t.y),e=["C",r,i].concat(e.slice(1));break;case"T":"Q"===n||"T"===n?(t.qx=2*t.x-t.qx,t.qy=2*t.y-t.qy):(t.qx=t.x,t.qy=t.y),e=["C"].concat(h(t.x,t.y,t.qx,t.qy,e[1],e[2]));break;case"Q":t.qx=e[1],t.qy=e[2],e=["C"].concat(h(t.x,t.y,e[1],e[2],e[3],e[4]));break;case"L":e=["C"].concat(d(t.x,t.y,e[1],e[2]));break;case"H":e=["C"].concat(d(t.x,t.y,e[1],t.y));break;case"V":e=["C"].concat(d(t.x,t.y,t.x,e[1]));break;case"Z":e=["C"].concat(d(t.x,t.y,t.X,t.Y))}return e},v=function(e,t){if(e[t].length>7){e[t].shift();for(var o=e[t];o.length;)s[t]="A",i&&(l[t]="A"),e.splice(t++,0,["C"].concat(o.splice(0,6)));e.splice(t,1),n=Math.max(r.length,i&&i.length||0)}},m=function(e,t,o,a,s){e&&t&&"M"===e[s][0]&&"M"!==t[s][0]&&(t.splice(s,0,["M",a.x,a.y]),o.bx=0,o.by=0,o.x=e[s][1],o.y=e[s][2],n=Math.max(r.length,i&&i.length||0))};n=Math.max(r.length,i&&i.length||0);for(var y=0;y<n;y++){r[y]&&(u=r[y][0]),"C"!==u&&(s[y]=u,y&&(p=s[y-1])),r[y]=g(r[y],o,p),"A"!==s[y]&&"C"===u&&(s[y]="C"),v(r,y),i&&(i[y]&&(u=i[y][0]),"C"!==u&&(l[y]=u,y&&(p=l[y-1])),i[y]=g(i[y],a,p),"A"!==l[y]&&"C"===u&&(l[y]="C"),v(i,y)),m(r,i,o,a,y),m(i,r,a,o,y);var b=r[y],C=i&&i[y],A=b.length,_=i&&C.length;o.x=b[A-2],o.y=b[A-1],o.bx=parseFloat(b[A-4])||o.x,o.by=parseFloat(b[A-3])||o.y,a.bx=i&&(parseFloat(C[_-4])||a.x),a.by=i&&(parseFloat(C[_-3])||a.y),a.x=i&&C[_-2],a.y=i&&C[_-1]}return i?[r,i]:r},g=/,?([a-z]),?/gi,v=function(e){return e.join(",").replace(g,"$1")},m=function(e,t,n,r,i){return e*(e*(-3*t+9*n-9*r+3*i)+6*t-12*n+6*r)-3*t+3*n},y=function(e,t,n,r,i,o,a,s,l){null===l&&(l=1);for(var u=(l=l>1?1:l<0?0:l)/2,c=[-.1252,.1252,-.3678,.3678,-.5873,.5873,-.7699,.7699,-.9041,.9041,-.9816,.9816],d=[.2491,.2491,.2335,.2335,.2032,.2032,.1601,.1601,.1069,.1069,.0472,.0472],h=0,f=0;f<12;f++){var p=u*c[f]+u,g=m(p,e,n,i,a),v=m(p,t,r,o,s),y=g*g+v*v;h+=d[f]*Math.sqrt(y)}return u*h},b=function(e,t,n,r,i,o,a,s){for(var l,u,c,d,h=[],f=[[],[]],p=0;p<2;++p)if(0===p?(u=6*e-12*n+6*i,l=-3*e+9*n-9*i+3*a,c=3*n-3*e):(u=6*t-12*r+6*o,l=-3*t+9*r-9*o+3*s,c=3*r-3*t),Math.abs(l)<1e-12){if(Math.abs(u)<1e-12)continue;(d=-c/u)>0&&d<1&&h.push(d)}else{var g=u*u-4*c*l,v=Math.sqrt(g);if(!(g<0)){var m=(-u+v)/(2*l);m>0&&m<1&&h.push(m);var y=(-u-v)/(2*l);y>0&&y<1&&h.push(y)}}for(var b,C=h.length,A=C;C--;)b=1-(d=h[C]),f[0][C]=b*b*b*e+3*b*b*d*n+3*b*d*d*i+d*d*d*a,f[1][C]=b*b*b*t+3*b*b*d*r+3*b*d*d*o+d*d*d*s;return f[0][A]=e,f[1][A]=t,f[0][A+1]=a,f[1][A+1]=s,f[0].length=f[1].length=A+2,{min:{x:Math.min.apply(0,f[0]),y:Math.min.apply(0,f[1])},max:{x:Math.max.apply(0,f[0]),y:Math.max.apply(0,f[1])}}},C=function(e,t,n,r,i,o,a,s){if(!(Math.max(e,n)<Math.min(i,a)||Math.min(e,n)>Math.max(i,a)||Math.max(t,r)<Math.min(o,s)||Math.min(t,r)>Math.max(o,s))){var l=(e-n)*(o-s)-(t-r)*(i-a);if(l){var u=((e*r-t*n)*(i-a)-(e-n)*(i*s-o*a))/l,c=((e*r-t*n)*(o-s)-(t-r)*(i*s-o*a))/l,d=+u.toFixed(2),h=+c.toFixed(2);if(!(d<+Math.min(e,n).toFixed(2)||d>+Math.max(e,n).toFixed(2)||d<+Math.min(i,a).toFixed(2)||d>+Math.max(i,a).toFixed(2)||h<+Math.min(t,r).toFixed(2)||h>+Math.max(t,r).toFixed(2)||h<+Math.min(o,s).toFixed(2)||h>+Math.max(o,s).toFixed(2)))return{x:u,y:c}}}},A=function(e,t,n){return t>=e.x&&t<=e.x+e.width&&n>=e.y&&n<=e.y+e.height},_=function(e,t,n,r,i){if(i)return[["M",+e+ +i,t],["l",n-2*i,0],["a",i,i,0,0,1,i,i],["l",0,r-2*i],["a",i,i,0,0,1,-i,i],["l",2*i-n,0],["a",i,i,0,0,1,-i,-i],["l",0,2*i-r],["a",i,i,0,0,1,i,-i],["z"]];var o=[["M",e,t],["l",n,0],["l",0,r],["l",-n,0],["z"]];return o.parsePathArray=v,o},I=function(e,t,n,r){return null===e&&(e=t=n=r=0),null===t&&(t=e.y,n=e.width,r=e.height,e=e.x),{x:e,y:t,width:n,w:n,height:r,h:r,x2:e+n,y2:t+r,cx:e+n/2,cy:t+r/2,r1:Math.min(n,r)/2,r2:Math.max(n,r)/2,r0:Math.sqrt(n*n+r*r)/2,path:_(e,t,n,r),vb:[e,t,n,r].join(" ")}},x=function(e,t,n,i,o,a,s,l){Object(r.isArray)(e)||(e=[e,t,n,i,o,a,s,l]);var u=b.apply(null,e);return I(u.min.x,u.min.y,u.max.x-u.min.x,u.max.y-u.min.y)},w=function(e,t,n,r,i,o,a,s,l){var u=1-l,c=Math.pow(u,3),d=Math.pow(u,2),h=l*l,f=h*l,p=e+2*l*(n-e)+h*(i-2*n+e),g=t+2*l*(r-t)+h*(o-2*r+t),v=n+2*l*(i-n)+h*(a-2*i+n),m=r+2*l*(o-r)+h*(s-2*o+r);return{x:c*e+3*d*l*n+3*u*l*l*i+f*a,y:c*t+3*d*l*r+3*u*l*l*o+f*s,m:{x:p,y:g},n:{x:v,y:m},start:{x:u*e+l*n,y:u*t+l*r},end:{x:u*i+l*a,y:u*o+l*s},alpha:90-180*Math.atan2(p-v,g-m)/Math.PI}},M=function(e,t,n){if(!function(e,t){return e=I(e),t=I(t),A(t,e.x,e.y)||A(t,e.x2,e.y)||A(t,e.x,e.y2)||A(t,e.x2,e.y2)||A(e,t.x,t.y)||A(e,t.x2,t.y)||A(e,t.x,t.y2)||A(e,t.x2,t.y2)||(e.x<t.x2&&e.x>t.x||t.x<e.x2&&t.x>e.x)&&(e.y<t.y2&&e.y>t.y||t.y<e.y2&&t.y>e.y)}(x(e),x(t)))return n?0:[];for(var r=~~(y.apply(0,e)/8),i=~~(y.apply(0,t)/8),o=[],a=[],s={},l=n?0:[],u=0;u<r+1;u++){var c=w.apply(0,e.concat(u/r));o.push({x:c.x,y:c.y,t:u/r})}for(u=0;u<i+1;u++)c=w.apply(0,t.concat(u/i)),a.push({x:c.x,y:c.y,t:u/i});for(u=0;u<r;u++)for(var d=0;d<i;d++){var h=o[u],f=o[u+1],p=a[d],g=a[d+1],v=Math.abs(f.x-h.x)<.001?"y":"x",m=Math.abs(g.x-p.x)<.001?"y":"x",b=C(h.x,h.y,f.x,f.y,p.x,p.y,g.x,g.y);if(b){if(s[b.x.toFixed(4)]===b.y.toFixed(4))continue;s[b.x.toFixed(4)]=b.y.toFixed(4);var _=h.t+Math.abs((b[v]-h[v])/(f[v]-h[v]))*(f.t-h.t),M=p.t+Math.abs((b[m]-p[m])/(g[m]-p[m]))*(g.t-p.t);_>=0&&_<=1&&M>=0&&M<=1&&(n?l+=1:l.push({x:b.x,y:b.y,t1:_,t2:M}))}}return l},S=function(e,t){return function(e,t,n){var r,i,o,a,s,l,u,c,d,h;e=p(e),t=p(t);for(var f=[],g=0,v=e.length;g<v;g++){var m=e[g];if("M"===m[0])r=s=m[1],i=l=m[2];else{"C"===m[0]?(d=[r,i].concat(m.slice(1)),r=d[6],i=d[7]):(d=[r,i,r,i,s,l,s,l],r=s,i=l);for(var y=0,b=t.length;y<b;y++){var C=t[y];if("M"===C[0])o=u=C[1],a=c=C[2];else{"C"===C[0]?(h=[o,a].concat(C.slice(1)),o=h[6],a=h[7]):(h=[o,a,o,a,u,c,u,c],o=u,a=c);for(var A=M(d,h,n),_=0,I=A.length;_<I;_++)A[_].segment1=g,A[_].segment2=y,A[_].bez1=d,A[_].bez2=h;f=f.concat(A)}}}}return f}(e,t)};function T(e,t){var n=[],r=[];return e.length&&function e(t,i){if(1===t.length)n.push(t[0]),r.push(t[0]);else{for(var o=[],a=0;a<t.length-1;a++)0===a&&n.push(t[0]),a===t.length-2&&r.push(t[a+1]),o[a]=[(1-i)*t[a][0]+i*t[a+1][0],(1-i)*t[a][1]+i*t[a+1][1]];e(o,i)}}(e,t),{left:n,right:r.reverse()}}var E=function(e,t){if(1===e.length)return e;var n=e.length-1,r=t.length-1,i=n/r,o=[];if(1===e.length&&"M"===e[0][0]){for(var a=0;a<r-n;a++)e.push(e[0]);return e}for(a=0;a<r;a++){var s=Math.floor(i*a);o[s]=(o[s]||0)+1}var l=o.reduce((function(t,r,i){return i===n?t.concat(e[n]):t.concat(function(e,t,n){if(1===n)return[[].concat(e)];var r=[];if("L"===t[0]||"C"===t[0]||"Q"===t[0])r=r.concat(function(e,t,n){var r=[[e[1],e[2]]];n=n||2;var i=[];"A"===t[0]?(r.push(t[6]),r.push(t[7])):"C"===t[0]?(r.push([t[1],t[2]]),r.push([t[3],t[4]]),r.push([t[5],t[6]])):"S"===t[0]||"Q"===t[0]?(r.push([t[1],t[2]]),r.push([t[3],t[4]])):r.push([t[1],t[2]]);for(var o=r,a=1/n,s=0;s<n-1;s++){var l=T(o,a/(1-a*s));i.push(l.left),o=l.right}return i.push(o),i.map((function(e){var t=[];return 4===e.length&&(t.push("C"),t=t.concat(e[2])),e.length>=3&&(3===e.length&&t.push("Q"),t=t.concat(e[1])),2===e.length&&t.push("L"),t.concat(e[e.length-1])}))}(e,t,n));else{var i=[].concat(e);"M"===i[0]&&(i[0]="L");for(var o=0;o<=n-1;o++)r.push(i)}return r}(e[i],e[i+1],r))}),[]);return l.unshift(e[0]),"Z"!==t[r]&&"z"!==t[r]||l.push("Z"),l},O=function(e,t){if(e.length!==t.length)return!1;var n=!0;return Object(r.each)(e,(function(e,r){if(e!==t[r])return n=!1,!1})),n};function k(e,t,n){var r=null,i=n;return t<i&&(i=t,r="add"),e<i&&(i=e,r="del"),{type:r,min:i}}var L=function(e,t){var n=function(e,t){var n,r,i=e.length,o=t.length,a=0;if(0===i||0===o)return null;for(var s=[],l=0;l<=i;l++)s[l]=[],s[l][0]={min:l};for(var u=0;u<=o;u++)s[0][u]={min:u};for(l=1;l<=i;l++)for(n=e[l-1],u=1;u<=o;u++){r=t[u-1],a=O(n,r)?0:1;var c=s[l-1][u].min+1,d=s[l][u-1].min+1,h=s[l-1][u-1].min+a;s[l][u]=k(c,d,h)}return s}(e,t),r=e.length,i=t.length,o=[],a=1,s=1;if(n[r][i].min!==r){for(var l=1;l<=r;l++){var u=n[l][l].min;s=l;for(var c=a;c<=i;c++)n[l][c].min<u&&(u=n[l][c].min,s=c);a=s,n[l][a].type&&o.push({index:l-1,type:n[l][a].type})}for(l=o.length-1;l>=0;l--)a=o[l].index,"add"===o[l].type?e.splice(a,0,[].concat(e[a])):e.splice(a,1)}var d=i-(r=e.length);if(r<i)for(l=0;l<d;l++)"z"===e[r-1][0]||"Z"===e[r-1][0]?e.splice(r-2,0,e[r-2]):e.push(e[r-1]),r+=1;return e};function D(e,t,n){for(var r,i=[].concat(e),o=1/(n+1),a=P(t)[0],s=1;s<=n;s++)o*=s,0===(r=Math.floor(e.length*o))?i.unshift([a[0]*o+e[r][0]*(1-o),a[1]*o+e[r][1]*(1-o)]):i.splice(r,0,[a[0]*o+e[r][0]*(1-o),a[1]*o+e[r][1]*(1-o)]);return i}function P(e){var t=[];switch(e[0]){case"M":case"L":t.push([e[1],e[2]]);break;case"A":t.push([e[6],e[7]]);break;case"Q":t.push([e[3],e[4]]),t.push([e[1],e[2]]);break;case"T":t.push([e[1],e[2]]);break;case"C":t.push([e[5],e[6]]),t.push([e[1],e[2]]),t.push([e[3],e[4]]);break;case"S":t.push([e[3],e[4]]),t.push([e[1],e[2]]);break;case"H":case"V":t.push([e[1],e[1]])}return t}var N=function(e,t){if(e.length<=1)return e;for(var n,r=0;r<t.length;r++)if(e[r][0]!==t[r][0])switch(n=P(e[r]),t[r][0]){case"M":e[r]=["M"].concat(n[0]);break;case"L":e[r]=["L"].concat(n[0]);break;case"A":e[r]=[].concat(t[r]),e[r][6]=n[0][0],e[r][7]=n[0][1];break;case"Q":if(n.length<2){if(!(r>0)){e[r]=t[r];break}n=D(n,e[r-1],1)}e[r]=["Q"].concat(n.reduce((function(e,t){return e.concat(t)}),[]));break;case"T":e[r]=["T"].concat(n[0]);break;case"C":if(n.length<3){if(!(r>0)){e[r]=t[r];break}n=D(n,e[r-1],2)}e[r]=["C"].concat(n.reduce((function(e,t){return e.concat(t)}),[]));break;case"S":if(n.length<2){if(!(r>0)){e[r]=t[r];break}n=D(n,e[r-1],1)}e[r]=["S"].concat(n.reduce((function(e,t){return e.concat(t)}),[]));break;default:e[r]=t[r]}return e}},function(e,t,n){var r=n(62),i=n(116);e.exports=function(e){return null!=e&&i(e.length)&&!r(e)}},function(e,t,n){function r(e){return(r="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}var i=n(430),o=n(440),a=n(47),s=n(15),l=n(447);e.exports=function(e){return"function"==typeof e?e:null==e?a:"object"==r(e)?s(e)?o(e[0],e[1]):i(e):l(e)}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(33),i=n(105);t.default={box:function(e,t,n,i){return r.getBBoxByArray([e,n],[t,i])},length:function(e,t,n,i){return r.distance(e,t,n,i)},pointAt:function(e,t,n,r,i){return{x:(1-i)*e+i*n,y:(1-i)*t+i*r}},pointDistance:function(e,t,n,i,o,a){var s=(n-e)*(o-e)+(i-t)*(a-t);return s<0?r.distance(e,t,o,a):s>(n-e)*(n-e)+(i-t)*(i-t)?r.distance(n,i,o,a):this.pointToLine(e,t,n,i,o,a)},pointToLine:function(e,t,n,r,o,a){var s=[n-e,r-t];if(i.exactEquals(s,[0,0]))return Math.sqrt((o-e)*(o-e)+(a-t)*(a-t));var l=[-s[1],s[0]];i.normalize(l,l);var u=[o-e,a-t];return Math.abs(i.dot(u,l))},tangentAngle:function(e,t,n,r){return Math.atan2(r-t,n-e)}}},function(e,t,n){var r=n(177),i=n(118),o=n(38);e.exports=function(e){return o(e)?r(e):i(e)}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(1),i=n(43),o=n(19),a=n(109),s=n(61),l=n(29),u=n(110),c=n(171),d=n(362),h=function(e){function t(t){return e.call(this,r.__assign(r.__assign({},t),{autoDraw:!0,renderer:"svg"}))||this}return r.__extends(t,e),t.prototype.getShapeBase=function(){return u},t.prototype.getGroupBase=function(){return c.default},t.prototype.getShape=function(e,t,n){var r=n.target||n.srcElement;if(!o.SHAPE_TO_TAGS[r.tagName]){for(var i=r.parentNode;i&&!o.SHAPE_TO_TAGS[i.tagName];)i=i.parentNode;r=i}return this.find((function(e){return e.get("el")===r}))},t.prototype.createDom=function(){var e=l.createSVGElement("svg"),t=new d.default(e);return e.setAttribute("width",""+this.get("width")),e.setAttribute("height",""+this.get("height")),this.set("context",t),e},t.prototype.onCanvasChange=function(e){var t=this.get("context"),n=this.get("el");if("sort"===e){var r=this.get("children");r&&r.length&&l.sortDom(this,(function(e,t){return r.indexOf(e)-r.indexOf(t)?1:0}))}else if("clear"===e){if(n){n.innerHTML="";var i=t.el;i.innerHTML="",n.appendChild(i)}}else"matrix"===e?s.setTransform(this):"clip"===e?s.setClip(this,t):"changeSize"===e&&(n.setAttribute("width",""+this.get("width")),n.setAttribute("height",""+this.get("height")))},t.prototype.draw=function(){var e=this.get("context"),t=this.getChildren();s.setClip(this,e),t.length&&a.drawChildren(e,t)},t}(i.AbstractCanvas);t.default=h},function(e,t,n){"use strict";n.r(t),n.d(t,"version",(function(){return h}));var r=n(37);n.d(t,"PathUtil",(function(){return r}));var i=n(150);for(var o in i)["default","version","Event","Base","AbstractCanvas","AbstractGroup","AbstractShape","PathUtil"].indexOf(o)<0&&function(e){n.d(t,e,(function(){return i[e]}))}(o);var a=n(151);for(var o in a)["default","version","Event","Base","AbstractCanvas","AbstractGroup","AbstractShape","PathUtil"].indexOf(o)<0&&function(e){n.d(t,e,(function(){return a[e]}))}(o);var s=n(96);n.d(t,"Event",(function(){return s.a}));var l=n(97);n.d(t,"Base",(function(){return l.a}));var u=n(228);n.d(t,"AbstractCanvas",(function(){return u.a}));var c=n(153);n.d(t,"AbstractGroup",(function(){return c.a}));var d=n(154);n.d(t,"AbstractShape",(function(){return d.a}));var h=n(318).version},function(e,t,n){var r=n(384),i=n(390);e.exports=function(e,t){var n=i(e,t);return r(n)?n:void 0}},function(e,t,n){var r=n(52),i=n(386),o=n(387),a=r?r.toStringTag:void 0;e.exports=function(e){return null==e?void 0===e?"[object Undefined]":"[object Null]":a&&a in Object(e)?i(e):o(e)}},function(e,t,n){var r=n(177),i=n(410),o=n(38);e.exports=function(e){return o(e)?r(e,!0):i(e)}},function(e,t){e.exports=function(e){return e}},function(e,t,n){var r=n(0).mix;e.exports={assign:r}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(59);t.default=function(e){var t=r.default(e);return t.charAt(0).toUpperCase()+t.substring(1)}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){if(e["_wrap_"+t])return e["_wrap_"+t];var n=function(n){e[t](n)};return e["_wrap_"+t]=n,n}},function(e,t){e.exports=function(e,t){return e===t||e!=e&&t!=t}},function(e,t,n){var r=n(31).Symbol;e.exports=r},function(e,t,n){(function(e){function r(e){return(r="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}var i=n(31),o=n(406),a="object"==r(t)&&t&&!t.nodeType&&t,s=a&&"object"==r(e)&&e&&!e.nodeType&&e,l=s&&s.exports===a?i.Buffer:void 0,u=(l?l.isBuffer:void 0)||o;e.exports=u}).call(this,n(115)(e))},function(e,t,n){var r=n(415),i=n(112),o=n(416),a=n(186),s=n(417),l=n(45),u=n(175),c=u(r),d=u(i),h=u(o),f=u(a),p=u(s),g=l;(r&&"[object DataView]"!=g(new r(new ArrayBuffer(1)))||i&&"[object Map]"!=g(new i)||o&&"[object Promise]"!=g(o.resolve())||a&&"[object Set]"!=g(new a)||s&&"[object WeakMap]"!=g(new s))&&(g=function(e){var t=l(e),n="[object Object]"==t?e.constructor:void 0,r=n?u(n):"";if(r)switch(r){case c:return"[object DataView]";case d:return"[object Map]";case h:return"[object Promise]";case f:return"[object Set]";case p:return"[object WeakMap]"}return t}),e.exports=g},function(e,t,n){function r(e){return(r="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}var i=n(45),o=n(34);e.exports=function(e){return"symbol"==r(e)||o(e)&&"[object Symbol]"==i(e)}},function(e,t){var n=[],r=[];function i(e,t){if(t=t||{},void 0===e)throw new Error("insert-css: You need to provide a CSS string. Usage: insertCss(cssString[, options]).");var i,o=!0===t.prepend?"prepend":"append",a=void 0!==t.container?t.container:document.querySelector("head"),s=n.indexOf(a);return-1===s&&(s=n.push(a)-1,r[s]={}),void 0!==r[s]&&void 0!==r[s][o]?i=r[s][o]:(i=r[s][o]=function(){var e=document.createElement("style");return e.setAttribute("type","text/css"),e}(),"prepend"===o?a.insertBefore(i,a.childNodes[0]):a.appendChild(i)),65279===e.charCodeAt(0)&&(e=e.substr(1,e.length)),i.styleSheet?i.styleSheet.cssText+=e:i.textContent+=e,i}e.exports=i,e.exports.insertCss=i},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t,n){if(e){if("function"==typeof e.addEventListener)return e.addEventListener(t,n,!1),{remove:function(){e.removeEventListener(t,n,!1)}};if("function"==typeof e.attachEvent)return e.attachEvent("on"+t,n),{remove:function(){e.detachEvent("on"+t,n)}}}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(102),i=n(27);t.default=function(e){if(!r.default(e)||!i.default(e,"Object"))return!1;if(null===Object.getPrototypeOf(e))return!0;for(var t=e;null!==Object.getPrototypeOf(t);)t=Object.getPrototypeOf(t);return Object.getPrototypeOf(e)===t}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(10);t.default=function(e){return r.default(e)?"":e.toString()}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.mergeView=t.getMergedRegion=t.getRefreshRegion=t.refreshElement=t.drawPath=t.clearChanged=t.checkChildrenRefresh=t.checkRefresh=t.drawChildren=t.applyAttrsToContext=void 0;var r=n(0),i=n(156),o=n(321),a=n(18),s=n(74),l={fill:"fillStyle",stroke:"strokeStyle",opacity:"globalAlpha"};function u(e,t){for(var n=0;n<e.length;n++){var r=e[n];if(r.cfg.visible)if(r.cfg.hasChanged)r.cfg.refresh=!0,r.isGroup()&&c(r.cfg.children,t);else if(r.cfg.refresh)r.isGroup()&&u(r.cfg.children,t);else{var i=d(r,t);r.cfg.refresh=i,i&&r.isGroup()&&u(r.cfg.children,t)}}}function c(e,t){for(var n=0;n<e.length;n++){var r=e[n];r.cfg.refresh=!0,r.isGroup()&&c(r.get("children"),t)}}function d(e,t){var n=e.cfg.cacheCanvasBBox;return e.cfg.isInView&&n&&a.intersectRect(n,t)}function h(e){var t;if(e.destroyed)t=e._cacheCanvasBBox;else{var n=e.get("cacheCanvasBBox"),r=n&&!(!n.width||!n.height),i=e.getCanvasBBox(),o=i&&!(!i.width||!i.height);r&&o?t=a.mergeRegion(n,i):r?t=n:o&&(t=i)}return t}t.applyAttrsToContext=function(e,t){var n=t.attr();for(var o in n){var a=n[o],s=l[o]?l[o]:o;"matrix"===s&&a?e.transform(a[0],a[1],a[3],a[4],a[6],a[7]):"lineDash"===s&&e.setLineDash?r.isArray(a)&&e.setLineDash(a):("strokeStyle"===s||"fillStyle"===s?a=i.parseStyle(e,t,a):"globalAlpha"===s&&(a*=e.globalAlpha),e[s]=a)}},t.drawChildren=function(e,t,n){for(var r=0;r<t.length;r++){var i=t[r];i.cfg.visible?i.draw(e,n):i.skipDraw()}},t.checkRefresh=function(e,t,n){var i=e.get("refreshElements");r.each(i,(function(t){if(t!==e)for(var n=t.cfg.parent;n&&n!==e&&!n.cfg.refresh;)n.cfg.refresh=!0,n=n.cfg.parent})),i[0]===e?c(t,n):u(t,n)},t.checkChildrenRefresh=u,t.clearChanged=function e(t){for(var n=0;n<t.length;n++){var r=t[n];r.cfg.hasChanged=!1,r.isGroup()&&!r.destroyed&&e(r.cfg.children)}},t.drawPath=function(e,t,n,r){var i=n.path,a=n.startArrow,l=n.endArrow,u=[0,0],c=[0,0],d={dx:0,dy:0};t.beginPath();for(var h=0;h<i.length;h++){var f=i[h],p=f[0];if(0===h&&a&&a.d){var g=e.getStartTangent();d=s.getShortenOffset(g[0][0],g[0][1],g[1][0],g[1][1],a.d)}else h===i.length-2&&"Z"===i[h+1][0]&&l&&l.d?"Z"===i[h+1][0]&&(g=e.getEndTangent(),d=s.getShortenOffset(g[0][0],g[0][1],g[1][0],g[1][1],l.d)):h===i.length-1&&l&&l.d&&"Z"!==i[0]&&(g=e.getEndTangent(),d=s.getShortenOffset(g[0][0],g[0][1],g[1][0],g[1][1],l.d));var v=d.dx,m=d.dy;switch(p){case"M":t.moveTo(f[1]-v,f[2]-m),c=[f[1],f[2]];break;case"L":t.lineTo(f[1]-v,f[2]-m);break;case"Q":t.quadraticCurveTo(f[1],f[2],f[3]-v,f[4]-m);break;case"C":t.bezierCurveTo(f[1],f[2],f[3],f[4],f[5]-v,f[6]-m);break;case"A":var y=void 0;r?(y=r[h])||(y=o.default(u,f),r[h]=y):y=o.default(u,f);var b=y.cx,C=y.cy,A=y.rx,_=y.ry,I=y.startAngle,x=y.endAngle,w=y.xRotation,M=y.sweepFlag;if(t.ellipse)t.ellipse(b,C,A,_,w,I,x,1-M);else{var S=A>_?A:_,T=A>_?1:A/_,E=A>_?_/A:1;t.translate(b,C),t.rotate(w),t.scale(T,E),t.arc(0,0,S,I,x,1-M),t.scale(1/T,1/E),t.rotate(-w),t.translate(-b,-C)}break;case"Z":t.closePath()}if("Z"===p)u=c;else{var O=f.length;u=[f[O-2],f[O-1]]}}},t.refreshElement=function(e,t){var n=e.get("canvas");n&&("remove"===t&&(e._cacheCanvasBBox=e.get("cacheCanvasBBox")),e.get("hasChanged")||(e.set("hasChanged",!0),e.cfg.parent&&e.cfg.parent.get("hasChanged")||(n.refreshElement(e,t,n),n.get("autoDraw")&&n.draw())))},t.getRefreshRegion=h,t.getMergedRegion=function(e){if(!e.length)return null;var t=[],n=[],i=[],o=[];return r.each(e,(function(e){var r=h(e);r&&(t.push(r.minX),n.push(r.minY),i.push(r.maxX),o.push(r.maxY))})),{minX:Math.min.apply(null,t),minY:Math.min.apply(null,n),maxX:Math.max.apply(null,i),maxY:Math.max.apply(null,o)}},t.mergeView=function(e,t){return e&&t&&a.intersectRect(e,t)?{minX:Math.max(e.minX,t.minX),minY:Math.max(e.minY,t.minY),maxX:Math.min(e.maxX,t.maxX),maxY:Math.min(e.maxY,t.maxY)}:null}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.setClip=t.setTransform=t.setShadow=void 0;var r=n(29);t.setShadow=function(e,t){var n=e.cfg.el,r=e.attr(),i={dx:r.shadowOffsetX,dy:r.shadowOffsetY,blur:r.shadowBlur,color:r.shadowColor};if(i.dx||i.dy||i.blur||i.color){var o=t.find("filter",i);o||(o=t.addShadow(i)),n.setAttribute("filter","url(#"+o+")")}else n.removeAttribute("filter")},t.setTransform=function(e){var t=e.attr().matrix;if(t){for(var n=e.cfg.el,r=[],i=0;i<9;i+=3)r.push(t[i]+","+t[i+1]);-1===(r=r.join(",")).indexOf("NaN")?n.setAttribute("transform","matrix("+r+")"):console.warn("invalid matrix:",t)}},t.setClip=function(e,t){var n=e.getClip(),i=e.get("el");if(n){if(n&&!i.hasAttribute("clip-path")){r.createDom(n),n.createPath(t);var o=t.addClip(n);i.setAttribute("clip-path","url(#"+o+")")}}else i.removeAttribute("clip-path")}},function(e,t,n){var r=n(45),i=n(25);e.exports=function(e){if(!i(e))return!1;var t=r(e);return"[object Function]"==t||"[object GeneratorFunction]"==t||"[object AsyncFunction]"==t||"[object Proxy]"==t}},function(e,t,n){var r=n(81),i=n(82);e.exports=function(e,t,n,o){var a=!n;n||(n={});for(var s=-1,l=t.length;++s<l;){var u=t[s],c=o?o(n[u],e[u],u,n,e):void 0;void 0===c&&(c=e[u]),a?i(n,u,c):r(n,u,c)}return n}},function(e,t,n){var r=n(405),i=n(34),o=Object.prototype,a=o.hasOwnProperty,s=o.propertyIsEnumerable,l=r(function(){return arguments}())?r:function(e){return i(e)&&a.call(e,"callee")&&!s.call(e,"callee")};e.exports=l},function(e,t,n){var r=n(407),i=n(84),o=n(117),a=o&&o.isTypedArray,s=a?i(a):r;e.exports=s},function(e,t,n){var r=n(55);e.exports=function(e){if("string"==typeof e||r(e))return e;var t=e+"";return"0"==t&&1/e==-1/0?"-0":t}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){return(e%t+t)%t}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=Math.PI/180;t.default=function(e){return r*e}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(4),i=n(14),o=Object.prototype.hasOwnProperty;t.default=function(e,t){if(!t||!r.default(e))return{};for(var n,a={},s=i.default(t)?t:function(e){return e[t]},l=0;l<e.length;l++){var u=e[l];n=s(u),o.call(a,n)?a[n].push(u):a[n]=[u]}return a}},function(e,t,n){"use strict";function r(e){return(r="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}var i=n(311),o=n(315),a=[].slice,s=["keyword","gray","hex"],l={};Object.keys(o).forEach((function(e){l[a.call(o[e].labels).sort().join("")]=e}));var u={};function c(e,t){if(!(this instanceof c))return new c(e,t);if(t&&t in s&&(t=null),t&&!(t in o))throw new Error("Unknown model: "+t);var n,r;if(null==e)this.model="rgb",this.color=[0,0,0],this.valpha=1;else if(e instanceof c)this.model=e.model,this.color=e.color.slice(),this.valpha=e.valpha;else if("string"==typeof e){var d=i.get(e);if(null===d)throw new Error("Unable to parse color from string: "+e);this.model=d.model,r=o[this.model].channels,this.color=d.value.slice(0,r),this.valpha="number"==typeof d.value[r]?d.value[r]:1}else if(e.length){this.model=t||"rgb",r=o[this.model].channels;var h=a.call(e,0,r);this.color=p(h,r),this.valpha="number"==typeof e[r]?e[r]:1}else if("number"==typeof e)e&=16777215,this.model="rgb",this.color=[e>>16&255,e>>8&255,255&e],this.valpha=1;else{this.valpha=1;var f=Object.keys(e);"alpha"in e&&(f.splice(f.indexOf("alpha"),1),this.valpha="number"==typeof e.alpha?e.alpha:0);var g=f.sort().join("");if(!(g in l))throw new Error("Unable to parse color from object: "+JSON.stringify(e));this.model=l[g];var v=o[this.model].labels,m=[];for(n=0;n<v.length;n++)m.push(e[v[n]]);this.color=p(m)}if(u[this.model])for(r=o[this.model].channels,n=0;n<r;n++){var y=u[this.model][n];y&&(this.color[n]=y(this.color[n]))}this.valpha=Math.max(0,Math.min(1,this.valpha)),Object.freeze&&Object.freeze(this)}function d(e,t,n){return(e=Array.isArray(e)?e:[e]).forEach((function(e){(u[e]||(u[e]=[]))[t]=n})),e=e[0],function(r){var i;return arguments.length?(n&&(r=n(r)),(i=this[e]()).color[t]=r,i):(i=this[e]().color[t],n&&(i=n(i)),i)}}function h(e){return function(t){return Math.max(0,Math.min(e,t))}}function f(e){return Array.isArray(e)?e:[e]}function p(e,t){for(var n=0;n<t;n++)"number"!=typeof e[n]&&(e[n]=0);return e}c.prototype={toString:function(){return this.string()},toJSON:function(){return this[this.model]()},string:function(e){var t=this.model in i.to?this:this.rgb(),n=1===(t=t.round("number"==typeof e?e:1)).valpha?t.color:t.color.concat(this.valpha);return i.to[t.model](n)},percentString:function(e){var t=this.rgb().round("number"==typeof e?e:1),n=1===t.valpha?t.color:t.color.concat(this.valpha);return i.to.rgb.percent(n)},array:function(){return 1===this.valpha?this.color.slice():this.color.concat(this.valpha)},object:function(){for(var e={},t=o[this.model].channels,n=o[this.model].labels,r=0;r<t;r++)e[n[r]]=this.color[r];return 1!==this.valpha&&(e.alpha=this.valpha),e},unitArray:function(){var e=this.rgb().color;return e[0]/=255,e[1]/=255,e[2]/=255,1!==this.valpha&&e.push(this.valpha),e},unitObject:function(){var e=this.rgb().object();return e.r/=255,e.g/=255,e.b/=255,1!==this.valpha&&(e.alpha=this.valpha),e},round:function(e){return e=Math.max(e||0,0),new c(this.color.map(function(e){return function(t){return function(e,t){return Number(e.toFixed(t))}(t,e)}}(e)).concat(this.valpha),this.model)},alpha:function(e){return arguments.length?new c(this.color.concat(Math.max(0,Math.min(1,e))),this.model):this.valpha},red:d("rgb",0,h(255)),green:d("rgb",1,h(255)),blue:d("rgb",2,h(255)),hue:d(["hsl","hsv","hsl","hwb","hcg"],0,(function(e){return(e%360+360)%360})),saturationl:d("hsl",1,h(100)),lightness:d("hsl",2,h(100)),saturationv:d("hsv",1,h(100)),value:d("hsv",2,h(100)),chroma:d("hcg",1,h(100)),gray:d("hcg",2,h(100)),white:d("hwb",1,h(100)),wblack:d("hwb",2,h(100)),cyan:d("cmyk",0,h(100)),magenta:d("cmyk",1,h(100)),yellow:d("cmyk",2,h(100)),black:d("cmyk",3,h(100)),x:d("xyz",0,h(100)),y:d("xyz",1,h(100)),z:d("xyz",2,h(100)),l:d("lab",0,h(100)),a:d("lab",1),b:d("lab",2),keyword:function(e){return arguments.length?new c(e):o[this.model].keyword(this.color)},hex:function(e){return arguments.length?new c(e):i.to.hex(this.rgb().round().color)},rgbNumber:function(){var e=this.rgb().color;return(255&e[0])<<16|(255&e[1])<<8|255&e[2]},luminosity:function(){for(var e=this.rgb().color,t=[],n=0;n<e.length;n++){var r=e[n]/255;t[n]=r<=.03928?r/12.92:Math.pow((r+.055)/1.055,2.4)}return.2126*t[0]+.7152*t[1]+.0722*t[2]},contrast:function(e){var t=this.luminosity(),n=e.luminosity();return t>n?(t+.05)/(n+.05):(n+.05)/(t+.05)},level:function(e){var t=this.contrast(e);return t>=7.1?"AAA":t>=4.5?"AA":""},isDark:function(){var e=this.rgb().color;return(299*e[0]+587*e[1]+114*e[2])/1e3<128},isLight:function(){return!this.isDark()},negate:function(){for(var e=this.rgb(),t=0;t<3;t++)e.color[t]=255-e.color[t];return e},lighten:function(e){var t=this.hsl();return t.color[2]+=t.color[2]*e,t},darken:function(e){var t=this.hsl();return t.color[2]-=t.color[2]*e,t},saturate:function(e){var t=this.hsl();return t.color[1]+=t.color[1]*e,t},desaturate:function(e){var t=this.hsl();return t.color[1]-=t.color[1]*e,t},whiten:function(e){var t=this.hwb();return t.color[1]+=t.color[1]*e,t},blacken:function(e){var t=this.hwb();return t.color[2]+=t.color[2]*e,t},grayscale:function(){var e=this.rgb().color,t=.3*e[0]+.59*e[1]+.11*e[2];return c.rgb(t,t,t)},fade:function(e){return this.alpha(this.valpha-this.valpha*e)},opaquer:function(e){return this.alpha(this.valpha+this.valpha*e)},rotate:function(e){var t=this.hsl(),n=t.color[0];return n=(n=(n+e)%360)<0?360+n:n,t.color[0]=n,t},mix:function(e,t){if(!e||!e.rgb)throw new Error('Argument to "mix" was not a Color instance, but rather an instance of '+r(e));var n=e.rgb(),i=this.rgb(),o=void 0===t?.5:t,a=2*o-1,s=n.alpha()-i.alpha(),l=((a*s==-1?a:(a+s)/(1+a*s))+1)/2,u=1-l;return c.rgb(l*n.red()+u*i.red(),l*n.green()+u*i.green(),l*n.blue()+u*i.blue(),n.alpha()*o+i.alpha()*(1-o))}},Object.keys(o).forEach((function(e){if(-1===s.indexOf(e)){var t=o[e].channels;c.prototype[e]=function(){if(this.model===e)return new c(this);if(arguments.length)return new c(arguments,e);var n="number"==typeof arguments[t]?t:this.valpha;return new c(f(o[this.model][e].raw(this.color)).concat(n),e)},c[e]=function(n){return"number"==typeof n&&(n=p(a.call(arguments),t)),new c(n,e)}}})),e.exports=c},function(e,t,n){var r;function i(e){return(i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}!function(o){var a=/^\s+/,s=/\s+$/,l=0,u=o.round,c=o.min,d=o.max,h=o.random;function f(e,t){if(t=t||{},(e=e||"")instanceof f)return e;if(!(this instanceof f))return new f(e,t);var n=function(e){var t,n,r,l={r:0,g:0,b:0},u=1,h=null,f=null,p=null,g=!1,v=!1;return"string"==typeof e&&(e=function(e){e=e.replace(a,"").replace(s,"").toLowerCase();var t,n=!1;if(k[e])e=k[e],n=!0;else if("transparent"==e)return{r:0,g:0,b:0,a:0,format:"name"};return(t=Y.rgb.exec(e))?{r:t[1],g:t[2],b:t[3]}:(t=Y.rgba.exec(e))?{r:t[1],g:t[2],b:t[3],a:t[4]}:(t=Y.hsl.exec(e))?{h:t[1],s:t[2],l:t[3]}:(t=Y.hsla.exec(e))?{h:t[1],s:t[2],l:t[3],a:t[4]}:(t=Y.hsv.exec(e))?{h:t[1],s:t[2],v:t[3]}:(t=Y.hsva.exec(e))?{h:t[1],s:t[2],v:t[3],a:t[4]}:(t=Y.hex8.exec(e))?{r:j(t[1]),g:j(t[2]),b:j(t[3]),a:B(t[4]),format:n?"name":"hex8"}:(t=Y.hex6.exec(e))?{r:j(t[1]),g:j(t[2]),b:j(t[3]),format:n?"name":"hex"}:(t=Y.hex4.exec(e))?{r:j(t[1]+""+t[1]),g:j(t[2]+""+t[2]),b:j(t[3]+""+t[3]),a:B(t[4]+""+t[4]),format:n?"name":"hex8"}:!!(t=Y.hex3.exec(e))&&{r:j(t[1]+""+t[1]),g:j(t[2]+""+t[2]),b:j(t[3]+""+t[3]),format:n?"name":"hex"}}(e)),"object"==i(e)&&(W(e.r)&&W(e.g)&&W(e.b)?(t=e.r,n=e.g,r=e.b,l={r:255*P(t,255),g:255*P(n,255),b:255*P(r,255)},g=!0,v="%"===String(e.r).substr(-1)?"prgb":"rgb"):W(e.h)&&W(e.s)&&W(e.v)?(h=z(e.s),f=z(e.v),l=function(e,t,n){e=6*P(e,360),t=P(t,100),n=P(n,100);var r=o.floor(e),i=e-r,a=n*(1-t),s=n*(1-i*t),l=n*(1-(1-i)*t),u=r%6;return{r:255*[n,s,a,a,l,n][u],g:255*[l,n,n,s,a,a][u],b:255*[a,a,l,n,n,s][u]}}(e.h,h,f),g=!0,v="hsv"):W(e.h)&&W(e.s)&&W(e.l)&&(h=z(e.s),p=z(e.l),l=function(e,t,n){var r,i,o;function a(e,t,n){return n<0&&(n+=1),n>1&&(n-=1),n<1/6?e+6*(t-e)*n:n<.5?t:n<2/3?e+(t-e)*(2/3-n)*6:e}if(e=P(e,360),t=P(t,100),n=P(n,100),0===t)r=i=o=n;else{var s=n<.5?n*(1+t):n+t-n*t,l=2*n-s;r=a(l,s,e+1/3),i=a(l,s,e),o=a(l,s,e-1/3)}return{r:255*r,g:255*i,b:255*o}}(e.h,h,p),g=!0,v="hsl"),e.hasOwnProperty("a")&&(u=e.a)),u=D(u),{ok:g,format:e.format||v,r:c(255,d(l.r,0)),g:c(255,d(l.g,0)),b:c(255,d(l.b,0)),a:u}}(e);this._originalInput=e,this._r=n.r,this._g=n.g,this._b=n.b,this._a=n.a,this._roundA=u(100*this._a)/100,this._format=t.format||n.format,this._gradientType=t.gradientType,this._r<1&&(this._r=u(this._r)),this._g<1&&(this._g=u(this._g)),this._b<1&&(this._b=u(this._b)),this._ok=n.ok,this._tc_id=l++}function p(e,t,n){e=P(e,255),t=P(t,255),n=P(n,255);var r,i,o=d(e,t,n),a=c(e,t,n),s=(o+a)/2;if(o==a)r=i=0;else{var l=o-a;switch(i=s>.5?l/(2-o-a):l/(o+a),o){case e:r=(t-n)/l+(t<n?6:0);break;case t:r=(n-e)/l+2;break;case n:r=(e-t)/l+4}r/=6}return{h:r,s:i,l:s}}function g(e,t,n){e=P(e,255),t=P(t,255),n=P(n,255);var r,i,o=d(e,t,n),a=c(e,t,n),s=o,l=o-a;if(i=0===o?0:l/o,o==a)r=0;else{switch(o){case e:r=(t-n)/l+(t<n?6:0);break;case t:r=(n-e)/l+2;break;case n:r=(e-t)/l+4}r/=6}return{h:r,s:i,v:s}}function v(e,t,n,r){var i=[R(u(e).toString(16)),R(u(t).toString(16)),R(u(n).toString(16))];return r&&i[0].charAt(0)==i[0].charAt(1)&&i[1].charAt(0)==i[1].charAt(1)&&i[2].charAt(0)==i[2].charAt(1)?i[0].charAt(0)+i[1].charAt(0)+i[2].charAt(0):i.join("")}function m(e,t,n,r){return[R(F(r)),R(u(e).toString(16)),R(u(t).toString(16)),R(u(n).toString(16))].join("")}function y(e,t){t=0===t?0:t||10;var n=f(e).toHsl();return n.s-=t/100,n.s=N(n.s),f(n)}function b(e,t){t=0===t?0:t||10;var n=f(e).toHsl();return n.s+=t/100,n.s=N(n.s),f(n)}function C(e){return f(e).desaturate(100)}function A(e,t){t=0===t?0:t||10;var n=f(e).toHsl();return n.l+=t/100,n.l=N(n.l),f(n)}function _(e,t){t=0===t?0:t||10;var n=f(e).toRgb();return n.r=d(0,c(255,n.r-u(-t/100*255))),n.g=d(0,c(255,n.g-u(-t/100*255))),n.b=d(0,c(255,n.b-u(-t/100*255))),f(n)}function I(e,t){t=0===t?0:t||10;var n=f(e).toHsl();return n.l-=t/100,n.l=N(n.l),f(n)}function x(e,t){var n=f(e).toHsl(),r=(n.h+t)%360;return n.h=r<0?360+r:r,f(n)}function w(e){var t=f(e).toHsl();return t.h=(t.h+180)%360,f(t)}function M(e){var t=f(e).toHsl(),n=t.h;return[f(e),f({h:(n+120)%360,s:t.s,l:t.l}),f({h:(n+240)%360,s:t.s,l:t.l})]}function S(e){var t=f(e).toHsl(),n=t.h;return[f(e),f({h:(n+90)%360,s:t.s,l:t.l}),f({h:(n+180)%360,s:t.s,l:t.l}),f({h:(n+270)%360,s:t.s,l:t.l})]}function T(e){var t=f(e).toHsl(),n=t.h;return[f(e),f({h:(n+72)%360,s:t.s,l:t.l}),f({h:(n+216)%360,s:t.s,l:t.l})]}function E(e,t,n){t=t||6,n=n||30;var r=f(e).toHsl(),i=360/n,o=[f(e)];for(r.h=(r.h-(i*t>>1)+720)%360;--t;)r.h=(r.h+i)%360,o.push(f(r));return o}function O(e,t){t=t||6;for(var n=f(e).toHsv(),r=n.h,i=n.s,o=n.v,a=[],s=1/t;t--;)a.push(f({h:r,s:i,v:o})),o=(o+s)%1;return a}f.prototype={isDark:function(){return this.getBrightness()<128},isLight:function(){return!this.isDark()},isValid:function(){return this._ok},getOriginalInput:function(){return this._originalInput},getFormat:function(){return this._format},getAlpha:function(){return this._a},getBrightness:function(){var e=this.toRgb();return(299*e.r+587*e.g+114*e.b)/1e3},getLuminance:function(){var e,t,n,r=this.toRgb();return e=r.r/255,t=r.g/255,n=r.b/255,.2126*(e<=.03928?e/12.92:o.pow((e+.055)/1.055,2.4))+.7152*(t<=.03928?t/12.92:o.pow((t+.055)/1.055,2.4))+.0722*(n<=.03928?n/12.92:o.pow((n+.055)/1.055,2.4))},setAlpha:function(e){return this._a=D(e),this._roundA=u(100*this._a)/100,this},toHsv:function(){var e=g(this._r,this._g,this._b);return{h:360*e.h,s:e.s,v:e.v,a:this._a}},toHsvString:function(){var e=g(this._r,this._g,this._b),t=u(360*e.h),n=u(100*e.s),r=u(100*e.v);return 1==this._a?"hsv("+t+", "+n+"%, "+r+"%)":"hsva("+t+", "+n+"%, "+r+"%, "+this._roundA+")"},toHsl:function(){var e=p(this._r,this._g,this._b);return{h:360*e.h,s:e.s,l:e.l,a:this._a}},toHslString:function(){var e=p(this._r,this._g,this._b),t=u(360*e.h),n=u(100*e.s),r=u(100*e.l);return 1==this._a?"hsl("+t+", "+n+"%, "+r+"%)":"hsla("+t+", "+n+"%, "+r+"%, "+this._roundA+")"},toHex:function(e){return v(this._r,this._g,this._b,e)},toHexString:function(e){return"#"+this.toHex(e)},toHex8:function(e){return function(e,t,n,r,i){var o=[R(u(e).toString(16)),R(u(t).toString(16)),R(u(n).toString(16)),R(F(r))];return i&&o[0].charAt(0)==o[0].charAt(1)&&o[1].charAt(0)==o[1].charAt(1)&&o[2].charAt(0)==o[2].charAt(1)&&o[3].charAt(0)==o[3].charAt(1)?o[0].charAt(0)+o[1].charAt(0)+o[2].charAt(0)+o[3].charAt(0):o.join("")}(this._r,this._g,this._b,this._a,e)},toHex8String:function(e){return"#"+this.toHex8(e)},toRgb:function(){return{r:u(this._r),g:u(this._g),b:u(this._b),a:this._a}},toRgbString:function(){return 1==this._a?"rgb("+u(this._r)+", "+u(this._g)+", "+u(this._b)+")":"rgba("+u(this._r)+", "+u(this._g)+", "+u(this._b)+", "+this._roundA+")"},toPercentageRgb:function(){return{r:u(100*P(this._r,255))+"%",g:u(100*P(this._g,255))+"%",b:u(100*P(this._b,255))+"%",a:this._a}},toPercentageRgbString:function(){return 1==this._a?"rgb("+u(100*P(this._r,255))+"%, "+u(100*P(this._g,255))+"%, "+u(100*P(this._b,255))+"%)":"rgba("+u(100*P(this._r,255))+"%, "+u(100*P(this._g,255))+"%, "+u(100*P(this._b,255))+"%, "+this._roundA+")"},toName:function(){return 0===this._a?"transparent":!(this._a<1)&&(L[v(this._r,this._g,this._b,!0)]||!1)},toFilter:function(e){var t="#"+m(this._r,this._g,this._b,this._a),n=t,r=this._gradientType?"GradientType = 1, ":"";if(e){var i=f(e);n="#"+m(i._r,i._g,i._b,i._a)}return"progid:DXImageTransform.Microsoft.gradient("+r+"startColorstr="+t+",endColorstr="+n+")"},toString:function(e){var t=!!e;e=e||this._format;var n=!1,r=this._a<1&&this._a>=0;return t||!r||"hex"!==e&&"hex6"!==e&&"hex3"!==e&&"hex4"!==e&&"hex8"!==e&&"name"!==e?("rgb"===e&&(n=this.toRgbString()),"prgb"===e&&(n=this.toPercentageRgbString()),"hex"!==e&&"hex6"!==e||(n=this.toHexString()),"hex3"===e&&(n=this.toHexString(!0)),"hex4"===e&&(n=this.toHex8String(!0)),"hex8"===e&&(n=this.toHex8String()),"name"===e&&(n=this.toName()),"hsl"===e&&(n=this.toHslString()),"hsv"===e&&(n=this.toHsvString()),n||this.toHexString()):"name"===e&&0===this._a?this.toName():this.toRgbString()},clone:function(){return f(this.toString())},_applyModification:function(e,t){var n=e.apply(null,[this].concat([].slice.call(t)));return this._r=n._r,this._g=n._g,this._b=n._b,this.setAlpha(n._a),this},lighten:function(){return this._applyModification(A,arguments)},brighten:function(){return this._applyModification(_,arguments)},darken:function(){return this._applyModification(I,arguments)},desaturate:function(){return this._applyModification(y,arguments)},saturate:function(){return this._applyModification(b,arguments)},greyscale:function(){return this._applyModification(C,arguments)},spin:function(){return this._applyModification(x,arguments)},_applyCombination:function(e,t){return e.apply(null,[this].concat([].slice.call(t)))},analogous:function(){return this._applyCombination(E,arguments)},complement:function(){return this._applyCombination(w,arguments)},monochromatic:function(){return this._applyCombination(O,arguments)},splitcomplement:function(){return this._applyCombination(T,arguments)},triad:function(){return this._applyCombination(M,arguments)},tetrad:function(){return this._applyCombination(S,arguments)}},f.fromRatio=function(e,t){if("object"==i(e)){var n={};for(var r in e)e.hasOwnProperty(r)&&(n[r]="a"===r?e[r]:z(e[r]));e=n}return f(e,t)},f.equals=function(e,t){return!(!e||!t)&&f(e).toRgbString()==f(t).toRgbString()},f.random=function(){return f.fromRatio({r:h(),g:h(),b:h()})},f.mix=function(e,t,n){n=0===n?0:n||50;var r=f(e).toRgb(),i=f(t).toRgb(),o=n/100;return f({r:(i.r-r.r)*o+r.r,g:(i.g-r.g)*o+r.g,b:(i.b-r.b)*o+r.b,a:(i.a-r.a)*o+r.a})},f.readability=function(e,t){var n=f(e),r=f(t);return(o.max(n.getLuminance(),r.getLuminance())+.05)/(o.min(n.getLuminance(),r.getLuminance())+.05)},f.isReadable=function(e,t,n){var r,i,o=f.readability(e,t);switch(i=!1,(r=function(e){var t,n;return"AA"!==(t=((e=e||{level:"AA",size:"small"}).level||"AA").toUpperCase())&&"AAA"!==t&&(t="AA"),"small"!==(n=(e.size||"small").toLowerCase())&&"large"!==n&&(n="small"),{level:t,size:n}}(n)).level+r.size){case"AAsmall":case"AAAlarge":i=o>=4.5;break;case"AAlarge":i=o>=3;break;case"AAAsmall":i=o>=7}return i},f.mostReadable=function(e,t,n){var r,i,o,a,s=null,l=0;i=(n=n||{}).includeFallbackColors,o=n.level,a=n.size;for(var u=0;u<t.length;u++)(r=f.readability(e,t[u]))>l&&(l=r,s=f(t[u]));return f.isReadable(e,s,{level:o,size:a})||!i?s:(n.includeFallbackColors=!1,f.mostReadable(e,["#fff","#000"],n))};var k=f.names={aliceblue:"f0f8ff",antiquewhite:"faebd7",aqua:"0ff",aquamarine:"7fffd4",azure:"f0ffff",beige:"f5f5dc",bisque:"ffe4c4",black:"000",blanchedalmond:"ffebcd",blue:"00f",blueviolet:"8a2be2",brown:"a52a2a",burlywood:"deb887",burntsienna:"ea7e5d",cadetblue:"5f9ea0",chartreuse:"7fff00",chocolate:"d2691e",coral:"ff7f50",cornflowerblue:"6495ed",cornsilk:"fff8dc",crimson:"dc143c",cyan:"0ff",darkblue:"00008b",darkcyan:"008b8b",darkgoldenrod:"b8860b",darkgray:"a9a9a9",darkgreen:"006400",darkgrey:"a9a9a9",darkkhaki:"bdb76b",darkmagenta:"8b008b",darkolivegreen:"556b2f",darkorange:"ff8c00",darkorchid:"9932cc",darkred:"8b0000",darksalmon:"e9967a",darkseagreen:"8fbc8f",darkslateblue:"483d8b",darkslategray:"2f4f4f",darkslategrey:"2f4f4f",darkturquoise:"00ced1",darkviolet:"9400d3",deeppink:"ff1493",deepskyblue:"00bfff",dimgray:"696969",dimgrey:"696969",dodgerblue:"1e90ff",firebrick:"b22222",floralwhite:"fffaf0",forestgreen:"228b22",fuchsia:"f0f",gainsboro:"dcdcdc",ghostwhite:"f8f8ff",gold:"ffd700",goldenrod:"daa520",gray:"808080",green:"008000",greenyellow:"adff2f",grey:"808080",honeydew:"f0fff0",hotpink:"ff69b4",indianred:"cd5c5c",indigo:"4b0082",ivory:"fffff0",khaki:"f0e68c",lavender:"e6e6fa",lavenderblush:"fff0f5",lawngreen:"7cfc00",lemonchiffon:"fffacd",lightblue:"add8e6",lightcoral:"f08080",lightcyan:"e0ffff",lightgoldenrodyellow:"fafad2",lightgray:"d3d3d3",lightgreen:"90ee90",lightgrey:"d3d3d3",lightpink:"ffb6c1",lightsalmon:"ffa07a",lightseagreen:"20b2aa",lightskyblue:"87cefa",lightslategray:"789",lightslategrey:"789",lightsteelblue:"b0c4de",lightyellow:"ffffe0",lime:"0f0",limegreen:"32cd32",linen:"faf0e6",magenta:"f0f",maroon:"800000",mediumaquamarine:"66cdaa",mediumblue:"0000cd",mediumorchid:"ba55d3",mediumpurple:"9370db",mediumseagreen:"3cb371",mediumslateblue:"7b68ee",mediumspringgreen:"00fa9a",mediumturquoise:"48d1cc",mediumvioletred:"c71585",midnightblue:"191970",mintcream:"f5fffa",mistyrose:"ffe4e1",moccasin:"ffe4b5",navajowhite:"ffdead",navy:"000080",oldlace:"fdf5e6",olive:"808000",olivedrab:"6b8e23",orange:"ffa500",orangered:"ff4500",orchid:"da70d6",palegoldenrod:"eee8aa",palegreen:"98fb98",paleturquoise:"afeeee",palevioletred:"db7093",papayawhip:"ffefd5",peachpuff:"ffdab9",peru:"cd853f",pink:"ffc0cb",plum:"dda0dd",powderblue:"b0e0e6",purple:"800080",rebeccapurple:"663399",red:"f00",rosybrown:"bc8f8f",royalblue:"4169e1",saddlebrown:"8b4513",salmon:"fa8072",sandybrown:"f4a460",seagreen:"2e8b57",seashell:"fff5ee",sienna:"a0522d",silver:"c0c0c0",skyblue:"87ceeb",slateblue:"6a5acd",slategray:"708090",slategrey:"708090",snow:"fffafa",springgreen:"00ff7f",steelblue:"4682b4",tan:"d2b48c",teal:"008080",thistle:"d8bfd8",tomato:"ff6347",turquoise:"40e0d0",violet:"ee82ee",wheat:"f5deb3",white:"fff",whitesmoke:"f5f5f5",yellow:"ff0",yellowgreen:"9acd32"},L=f.hexNames=function(e){var t={};for(var n in e)e.hasOwnProperty(n)&&(t[e[n]]=n);return t}(k);function D(e){return e=parseFloat(e),(isNaN(e)||e<0||e>1)&&(e=1),e}function P(e,t){(function(e){return"string"==typeof e&&-1!=e.indexOf(".")&&1===parseFloat(e)})(e)&&(e="100%");var n=function(e){return"string"==typeof e&&-1!=e.indexOf("%")}(e);return e=c(t,d(0,parseFloat(e))),n&&(e=parseInt(e*t,10)/100),o.abs(e-t)<1e-6?1:e%t/parseFloat(t)}function N(e){return c(1,d(0,e))}function j(e){return parseInt(e,16)}function R(e){return 1==e.length?"0"+e:""+e}function z(e){return e<=1&&(e=100*e+"%"),e}function F(e){return o.round(255*parseFloat(e)).toString(16)}function B(e){return j(e)/255}var H,U,V,Y=(U="[\\s|\\(]+("+(H="(?:[-\\+]?\\d*\\.\\d+%?)|(?:[-\\+]?\\d+%?)")+")[,|\\s]+("+H+")[,|\\s]+("+H+")\\s*\\)?",V="[\\s|\\(]+("+H+")[,|\\s]+("+H+")[,|\\s]+("+H+")[,|\\s]+("+H+")\\s*\\)?",{CSS_UNIT:new RegExp(H),rgb:new RegExp("rgb"+U),rgba:new RegExp("rgba"+V),hsl:new RegExp("hsl"+U),hsla:new RegExp("hsla"+V),hsv:new RegExp("hsv"+U),hsva:new RegExp("hsva"+V),hex3:/^#?([0-9a-fA-F]{1})([0-9a-fA-F]{1})([0-9a-fA-F]{1})$/,hex6:/^#?([0-9a-fA-F]{2})([0-9a-fA-F]{2})([0-9a-fA-F]{2})$/,hex4:/^#?([0-9a-fA-F]{1})([0-9a-fA-F]{1})([0-9a-fA-F]{1})([0-9a-fA-F]{1})$/,hex8:/^#?([0-9a-fA-F]{2})([0-9a-fA-F]{2})([0-9a-fA-F]{2})([0-9a-fA-F]{2})$/});function W(e){return!!Y.CSS_UNIT.exec(e)}e.exports?e.exports=f:void 0===(r=function(){return f}.call(t,n,t,e))||(e.exports=r)}(Math)},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(21);t.default=function(e,t){return!!r.default(e)&&e.indexOf(t)>-1}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(28);Object.defineProperty(t,"Base",{enumerable:!0,get:function(){return r.default}});var i=n(334);Object.defineProperty(t,"Circle",{enumerable:!0,get:function(){return i.default}});var o=n(335);Object.defineProperty(t,"Ellipse",{enumerable:!0,get:function(){return o.default}});var a=n(336);Object.defineProperty(t,"Image",{enumerable:!0,get:function(){return a.default}});var s=n(337);Object.defineProperty(t,"Line",{enumerable:!0,get:function(){return s.default}});var l=n(338);Object.defineProperty(t,"Marker",{enumerable:!0,get:function(){return l.default}});var u=n(340);Object.defineProperty(t,"Path",{enumerable:!0,get:function(){return u.default}});var c=n(342);Object.defineProperty(t,"Polygon",{enumerable:!0,get:function(){return c.default}});var d=n(343);Object.defineProperty(t,"Polyline",{enumerable:!0,get:function(){return d.default}});var h=n(345);Object.defineProperty(t,"Rect",{enumerable:!0,get:function(){return h.default}});var f=n(348);Object.defineProperty(t,"Text",{enumerable:!0,get:function(){return f.default}})},function(e,t,n){"use strict";function r(e){return(r="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}Object.defineProperty(t,"__esModule",{value:!0}),t.addEndArrow=t.addStartArrow=t.getShortenOffset=void 0;var i=n(1),o=n(73),a=Math.sin,s=Math.cos,l=Math.atan2,u=Math.PI;function c(e,t,n,r,i,c,d){var h=t.stroke,f=t.lineWidth,p=l(r-c,n-i),g=new o.Path({type:"path",canvas:e.get("canvas"),isArrowShape:!0,attrs:{path:"M"+10*s(u/6)+","+10*a(u/6)+" L0,0 L"+10*s(u/6)+",-"+10*a(u/6),stroke:h,lineWidth:f}});g.translate(i,c),g.rotateAtPoint(i,c,p),e.set(d?"startArrowShape":"endArrowShape",g)}function d(e,t,n,r,u,c,d){var h=t.startArrow,f=t.endArrow,p=t.stroke,g=t.lineWidth,v=d?h:f,m=v.d,y=v.fill,b=v.stroke,C=v.lineWidth,A=i.__rest(v,["d","fill","stroke","lineWidth"]),_=l(r-c,n-u);m&&(u-=s(_)*m,c-=a(_)*m);var I=new o.Path({type:"path",canvas:e.get("canvas"),isArrowShape:!0,attrs:i.__assign(i.__assign({},A),{stroke:b||p,lineWidth:C||g,fill:y})});I.translate(u,c),I.rotateAtPoint(u,c,_),e.set(d?"startArrowShape":"endArrowShape",I)}t.getShortenOffset=function(e,t,n,r,i){var o=l(r-t,n-e);return{dx:s(o)*i,dy:a(o)*i}},t.addStartArrow=function(e,t,n,i,o,a){"object"===r(t.startArrow)?d(e,t,n,i,o,a,!0):t.startArrow?c(e,t,n,i,o,a,!0):e.set("startArrowShape",null)},t.addEndArrow=function(e,t,n,i,o,a){"object"===r(t.endArrow)?d(e,t,n,i,o,a,!1):t.endArrow?c(e,t,n,i,o,a,!1):e.set("startArrowShape",null)}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(40);t.default=function(e,t,n,i,o,a,s){var l=Math.min(e,n),u=Math.max(e,n),c=Math.min(t,i),d=Math.max(t,i),h=o/2;return a>=l-h&&a<=u+h&&s>=c-h&&s<=d+h&&r.default.pointToLine(e,t,n,i,a,s)<=o/2}},function(e,t,n){var r=n(77),i=n(379),o=n(380),a=n(381),s=n(382),l=n(383);function u(e){var t=this.__data__=new r(e);this.size=t.size}u.prototype.clear=i,u.prototype.delete=o,u.prototype.get=a,u.prototype.has=s,u.prototype.set=l,e.exports=u},function(e,t,n){var r=n(374),i=n(375),o=n(376),a=n(377),s=n(378);function l(e){var t=-1,n=null==e?0:e.length;for(this.clear();++t<n;){var r=e[t];this.set(r[0],r[1])}}l.prototype.clear=r,l.prototype.delete=i,l.prototype.get=o,l.prototype.has=a,l.prototype.set=s,e.exports=l},function(e,t,n){var r=n(51);e.exports=function(e,t){for(var n=e.length;n--;)if(r(e[n][0],t))return n;return-1}},function(e,t,n){var r=n(44)(Object,"create");e.exports=r},function(e,t,n){var r=n(399);e.exports=function(e,t){var n=e.__data__;return r(t)?n["string"==typeof t?"string":"hash"]:n.map}},function(e,t,n){var r=n(82),i=n(51),o=Object.prototype.hasOwnProperty;e.exports=function(e,t,n){var a=e[t];o.call(e,t)&&i(a,n)&&(void 0!==n||t in e)||r(e,t,n)}},function(e,t,n){var r=n(176);e.exports=function(e,t,n){"__proto__"==t&&r?r(e,t,{configurable:!0,enumerable:!0,value:n,writable:!0}):e[t]=n}},function(e,t){function n(e){return(n="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}var r=/^(?:0|[1-9]\d*)$/;e.exports=function(e,t){var i=n(e);return!!(t=null==t?9007199254740991:t)&&("number"==i||"symbol"!=i&&r.test(e))&&e>-1&&e%1==0&&e<t}},function(e,t){e.exports=function(e){return function(t){return e(t)}}},function(e,t){var n=Object.prototype;e.exports=function(e){var t=e&&e.constructor;return e===("function"==typeof t&&t.prototype||n)}},function(e,t,n){var r=n(178)(Object.getPrototypeOf,Object);e.exports=r},function(e,t,n){var r=n(123),i=n(428)(r);e.exports=i},function(e,t,n){var r=n(89),i=n(66);e.exports=function(e,t){for(var n=0,o=(t=r(t,e)).length;null!=e&&n<o;)e=e[i(t[n++])];return n&&n==o?e:void 0}},function(e,t,n){var r=n(15),i=n(126),o=n(442),a=n(201);e.exports=function(e,t){return r(e)?e:i(e,t)?[e]:o(a(e))}},function(e,t){e.exports=function(e,t){for(var n=-1,r=null==e?0:e.length,i=Array(r);++n<r;)i[n]=t(e[n],n,e);return i}},function(e,t,n){var r=n(47),i=n(210),o=n(211);e.exports=function(e,t){return o(i(e,t,r),e+"")}},function(e,t,n){function r(e){return(r="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}var i=n(51),o=n(38),a=n(83),s=n(25);e.exports=function(e,t,n){if(!s(n))return!1;var l=r(t);return!!("number"==l?o(n)&&a(t,n.length):"string"==l&&t in n)&&i(n[t],e)}},function(e,t,n){"use strict";var r=n(13);e.exports={longestPath:function(e){var t={};r.forEach(e.sources(),(function n(i){var o=e.node(i);if(r.has(t,i))return o.rank;t[i]=!0;var a=r.min(r.map(e.outEdges(i),(function(t){return n(t.w)-e.edge(t).minlen})));return a!==Number.POSITIVE_INFINITY&&null!=a||(a=0),o.rank=a}))},slack:function(e,t){return e.node(t.w).rank-e.node(t.v).rank-e.edge(t).minlen}}},function(e,t,n){var r=n(226),i=function(){function e(e,t){void 0===t&&(t={}),this.options=t,this.rootNode=r(e,t)}return e.prototype.execute=function(){throw new Error("please override this method")},e}();e.exports=i},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(102),i=n(21),o=n(3);t.default=function e(t,n){if(t===n)return!0;if(!t||!n)return!1;if(o.default(t)||o.default(n))return!1;if(i.default(t)||i.default(n)){if(t.length!==n.length)return!1;for(var a=!0,s=0;s<t.length&&(a=e(t[s],n[s]));s++);return a}if(r.default(t)||r.default(n)){var l=Object.keys(t),u=Object.keys(n);if(l.length!==u.length)return!1;for(a=!0,s=0;s<l.length&&(a=e(t[l[s]],n[l[s]]));s++);return a}return!1}},function(e,t,n){"use strict";var r=function(){function e(e,t){this.bubbles=!0,this.target=null,this.currentTarget=null,this.delegateTarget=null,this.delegateObject=null,this.defaultPrevented=!1,this.propagationStopped=!1,this.shape=null,this.fromShape=null,this.toShape=null,this.propagationPath=[],this.type=e,this.name=e,this.originalEvent=t,this.timeStamp=t.timeStamp}return e.prototype.preventDefault=function(){this.defaultPrevented=!0,this.originalEvent.preventDefault&&this.originalEvent.preventDefault()},e.prototype.stopPropagation=function(){this.propagationStopped=!0},e.prototype.toString=function(){return"[Event (type="+this.type+")]"},e.prototype.save=function(){},e.prototype.restore=function(){},e}();t.a=r},function(e,t,n){"use strict";var r=n(1),i=n(99),o=n(9),a=function(e){function t(t){var n=e.call(this)||this;n.destroyed=!1;var r=n.getDefaultCfg();return n.cfg=Object(o.i)(r,t),n}return Object(r.__extends)(t,e),t.prototype.getDefaultCfg=function(){return{}},t.prototype.get=function(e){return this.cfg[e]},t.prototype.set=function(e,t){this.cfg[e]=t},t.prototype.destroy=function(){this.cfg={destroyed:!0},this.off(),this.destroyed=!0},t}(i.a);t.a=a},function(e,t,n){"use strict";function r(e){return(r="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}Object.defineProperty(t,"__esModule",{value:!0});var i=n(4);t.default=function e(t){if("object"!==r(t)||null===t)return t;var n;if(i.default(t)){n=[];for(var o=0,a=t.length;o<a;o++)"object"===r(t[o])&&null!=t[o]?n[o]=e(t[o]):n[o]=t[o]}else for(var s in n={},t)"object"===r(t[s])&&null!=t[s]?n[s]=e(t[s]):n[s]=t[s];return n}},function(e,t,n){"use strict";var r=function(){function e(){this._events={}}return e.prototype.on=function(e,t,n){return this._events[e]||(this._events[e]=[]),this._events[e].push({callback:t,once:!!n}),this},e.prototype.once=function(e,t){return this.on(e,t,!0),this},e.prototype.emit=function(e){for(var t=this,n=[],r=1;r<arguments.length;r++)n[r-1]=arguments[r];var i=this._events[e]||[],o=this._events["*"]||[],a=function(r){for(var i=r.length,o=0;o<i;o++)if(r[o]){var a=r[o],s=a.callback;a.once&&(r.splice(o,1),0===r.length&&delete t._events[e],i--,o--),s.apply(t,n)}};a(i),a(o)},e.prototype.off=function(e,t){if(e)if(t){for(var n=this._events[e]||[],r=n.length,i=0;i<r;i++)n[i].callback===t&&(n.splice(i,1),r--,i--);0===n.length&&delete this._events[e]}else delete this._events[e];else this._events={};return this},e.prototype.getEvents=function(){return this._events},e}();t.a=r},function(e,t,n){"use strict";var r=n(1),i=n(101),o=n(9),a={},s=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(r.__extends)(t,e),t.prototype.isCanvas=function(){return!1},t.prototype.getBBox=function(){var e=1/0,t=-1/0,n=1/0,r=-1/0,i=[],a=[],s=this.getChildren().filter((function(e){return e.get("visible")&&(!e.isGroup()||e.isGroup()&&e.getChildren().length>0)}));return s.length>0?(Object(o.a)(s,(function(e){var t=e.getBBox();i.push(t.minX,t.maxX),a.push(t.minY,t.maxY)})),e=Math.min.apply(null,i),t=Math.max.apply(null,i),n=Math.min.apply(null,a),r=Math.max.apply(null,a)):(e=0,t=0,n=0,r=0),{x:e,y:n,minX:e,minY:n,maxX:t,maxY:r,width:t-e,height:r-n}},t.prototype.getCanvasBBox=function(){var e=1/0,t=-1/0,n=1/0,r=-1/0,i=[],a=[],s=this.getChildren().filter((function(e){return e.get("visible")&&(!e.isGroup()||e.isGroup()&&e.getChildren().length>0)}));return s.length>0?(Object(o.a)(s,(function(e){var t=e.getCanvasBBox();i.push(t.minX,t.maxX),a.push(t.minY,t.maxY)})),e=Math.min.apply(null,i),t=Math.max.apply(null,i),n=Math.min.apply(null,a),r=Math.max.apply(null,a)):(e=0,t=0,n=0,r=0),{x:e,y:n,minX:e,minY:n,maxX:t,maxY:r,width:t-e,height:r-n}},t.prototype.getDefaultCfg=function(){var t=e.prototype.getDefaultCfg.call(this);return t.children=[],t},t.prototype.onAttrChange=function(t,n,r){if(e.prototype.onAttrChange.call(this,t,n,r),"matrix"===t){var i=this.getTotalMatrix();this._applyChildrenMarix(i)}},t.prototype.applyMatrix=function(t){var n=this.getTotalMatrix();e.prototype.applyMatrix.call(this,t);var r=this.getTotalMatrix();r!==n&&this._applyChildrenMarix(r)},t.prototype._applyChildrenMarix=function(e){var t=this.getChildren();Object(o.a)(t,(function(t){t.applyMatrix(e)}))},t.prototype.addShape=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];var n=e[0],r=e[1];Object(o.f)(n)?r=n:r.type=n;var i=a[r.type];i||(i=Object(o.k)(r.type),a[r.type]=i);var s=this.getShapeBase(),l=new s[i](r);return this.add(l),l},t.prototype.addGroup=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];var n,r=e[0],i=e[1];if(Object(o.d)(r))n=new r(i||{parent:this});else{var a=r||{},s=this.getGroupBase();n=new s(a)}return this.add(n),n},t.prototype.getCanvas=function(){return this.isCanvas()?this:this.get("canvas")},t.prototype.getShape=function(e,t,n){if(!Object(o.b)(this))return null;var r,i=this.getChildren();if(this.isCanvas())r=this._findShape(i,e,t,n);else{var a=[e,t,1];a=this.invertFromMatrix(a),this.isClipped(a[0],a[1])||(r=this._findShape(i,a[0],a[1],n))}return r},t.prototype._findShape=function(e,t,n,r){for(var i=null,a=e.length-1;a>=0;a--){var s=e[a];if(Object(o.b)(s)&&(s.isGroup()?i=s.getShape(t,n,r):s.isHit(t,n)&&(i=s)),i)break}return i},t.prototype.add=function(e){var t=this.getCanvas(),n=this.getChildren(),r=this.get("timeline"),i=e.getParent();i&&function(e,t,n){void 0===n&&(n=!0),n?t.destroy():(t.set("parent",null),t.set("canvas",null)),Object(o.j)(e.getChildren(),t)}(i,e,!1),e.set("parent",this),t&&function e(t,n){if(t.set("canvas",n),t.isGroup()){var r=t.get("children");r.length&&r.forEach((function(t){e(t,n)}))}}(e,t),r&&function e(t,n){if(t.set("timeline",n),t.isGroup()){var r=t.get("children");r.length&&r.forEach((function(t){e(t,n)}))}}(e,r),n.push(e),e.onCanvasChange("add"),this._applyElementMatrix(e)},t.prototype._applyElementMatrix=function(e){var t=this.getTotalMatrix();t&&e.applyMatrix(t)},t.prototype.getChildren=function(){return this.get("children")},t.prototype.sort=function(){var e,t=this.getChildren();Object(o.a)(t,(function(e,t){return e._INDEX=t,e})),t.sort((e=function(e,t){return e.get("zIndex")-t.get("zIndex")},function(t,n){var r=e(t,n);return 0===r?t._INDEX-n._INDEX:r})),this.onCanvasChange("sort")},t.prototype.clear=function(){if(this.set("clearing",!0),!this.destroyed){for(var e=this.getChildren(),t=e.length-1;t>=0;t--)e[t].destroy();this.set("children",[]),this.onCanvasChange("clear"),this.set("clearing",!1)}},t.prototype.destroy=function(){this.get("destroyed")||(this.clear(),e.prototype.destroy.call(this))},t.prototype.getFirst=function(){return this.getChildByIndex(0)},t.prototype.getLast=function(){var e=this.getChildren();return this.getChildByIndex(e.length-1)},t.prototype.getChildByIndex=function(e){return this.getChildren()[e]},t.prototype.getCount=function(){return this.getChildren().length},t.prototype.contain=function(e){return this.getChildren().indexOf(e)>-1},t.prototype.removeChild=function(e,t){void 0===t&&(t=!0),this.contain(e)&&e.remove(t)},t.prototype.findAll=function(e){var t=[],n=this.getChildren();return Object(o.a)(n,(function(n){e(n)&&t.push(n),n.isGroup()&&(t=t.concat(n.findAll(e)))})),t},t.prototype.find=function(e){var t=null,n=this.getChildren();return Object(o.a)(n,(function(n){if(e(n)?t=n:n.isGroup()&&(t=n.find(e)),t)return!1})),t},t.prototype.findById=function(e){return this.find((function(t){return t.get("id")===e}))},t.prototype.findByClassName=function(e){return this.find((function(t){return t.get("className")===e}))},t.prototype.findAllByName=function(e){return this.findAll((function(t){return t.get("name")===e}))},t}(i.a);t.a=s},function(e,t,n){"use strict";var r=n(1),i=n(0),o=n(133),a=n(9),s=n(32),l=n(97),u=o.ext.transform,c=["zIndex","capture","visible","type"],d=["repeat"];function h(e,t){var n={},r=t.attrs;for(var i in e)n[i]=r[i];return n}function f(e,t){var n={},r=t.attr();return Object(i.each)(e,(function(e,t){-1!==d.indexOf(t)||Object(i.isEqual)(r[t],e)||(n[t]=e)})),n}function p(e,t){if(t.onFrame)return e;var n=t.startTime,r=t.delay,o=t.duration,a=Object.prototype.hasOwnProperty;return Object(i.each)(e,(function(e){n+r<e.startTime+e.delay+e.duration&&o>e.delay&&Object(i.each)(t.toAttrs,(function(t,n){a.call(e.toAttrs,n)&&(delete e.toAttrs[n],delete e.fromAttrs[n])}))})),e}var g=function(e){function t(t){var n=e.call(this,t)||this;n.attrs={};var r=n.getDefaultAttrs();return Object(i.mix)(r,t.attrs),n.attrs=r,n.initAttrs(r),n.initAnimate(),n}return Object(r.__extends)(t,e),t.prototype.getDefaultCfg=function(){return{visible:!0,capture:!0,zIndex:0}},t.prototype.getDefaultAttrs=function(){return{matrix:this.getDefaultMatrix(),opacity:1}},t.prototype.onCanvasChange=function(e){},t.prototype.initAttrs=function(e){},t.prototype.initAnimate=function(){this.set("animable",!0),this.set("animating",!1)},t.prototype.isGroup=function(){return!1},t.prototype.getParent=function(){return this.get("parent")},t.prototype.getCanvas=function(){return this.get("canvas")},t.prototype.attr=function(){for(var e,t=[],n=0;n<arguments.length;n++)t[n]=arguments[n];var r=t[0],o=t[1];if(!r)return this.attrs;if(Object(i.isObject)(r)){for(var a in r)this.setAttr(a,r[a]);return this.afterAttrsChange(r),this}return 2===t.length?(this.setAttr(r,o),this.afterAttrsChange(((e={})[r]=o,e)),this):this.attrs[r]},t.prototype.isClipped=function(e,t){var n=this.getClip();return n&&!n.isHit(e,t)},t.prototype.setAttr=function(e,t){var n=this.attrs[e];n!==t&&(this.attrs[e]=t,this.onAttrChange(e,t,n))},t.prototype.onAttrChange=function(e,t,n){"matrix"===e&&this.set("totalMatrix",null)},t.prototype.afterAttrsChange=function(e){if(this.cfg.isClipShape){var t=this.cfg.applyTo;t&&t.onCanvasChange("clip")}else this.onCanvasChange("attr")},t.prototype.show=function(){return this.set("visible",!0),this.onCanvasChange("show"),this},t.prototype.hide=function(){return this.set("visible",!1),this.onCanvasChange("hide"),this},t.prototype.setZIndex=function(e){this.set("zIndex",e);var t=this.getParent();return t&&t.sort(),this},t.prototype.toFront=function(){var e=this.getParent();if(e){var t=e.getChildren(),n=(this.get("el"),t.indexOf(this));t.splice(n,1),t.push(this),this.onCanvasChange("zIndex")}},t.prototype.toBack=function(){var e=this.getParent();if(e){var t=e.getChildren(),n=(this.get("el"),t.indexOf(this));t.splice(n,1),t.unshift(this),this.onCanvasChange("zIndex")}},t.prototype.remove=function(e){void 0===e&&(e=!0);var t=this.getParent();t?(Object(a.j)(t.getChildren(),this),t.get("clearing")||this.onCanvasChange("remove")):this.onCanvasChange("remove"),e&&this.destroy()},t.prototype.resetMatrix=function(){this.attr("matrix",this.getDefaultMatrix()),this.onCanvasChange("matrix")},t.prototype.getMatrix=function(){return this.attr("matrix")},t.prototype.setMatrix=function(e){this.attr("matrix",e),this.onCanvasChange("matrix")},t.prototype.getTotalMatrix=function(){var e=this.cfg.totalMatrix;if(!e){var t=this.attr("matrix"),n=this.cfg.parentMatrix;e=n&&t?Object(s.b)(n,t):t||n,this.set("totalMatrix",e)}return e},t.prototype.applyMatrix=function(e){var t,n=this.attr("matrix");t=e&&n?Object(s.b)(e,n):n||e,this.set("totalMatrix",t),this.set("parentMatrix",e)},t.prototype.getDefaultMatrix=function(){return null},t.prototype.applyToMatrix=function(e){var t=this.attr("matrix");return t?Object(s.c)(t,e):e},t.prototype.invertFromMatrix=function(e){var t=this.attr("matrix");if(t){var n=Object(s.a)(t);if(n)return Object(s.c)(n,e)}return e},t.prototype.setClip=function(e){var t=this.getCanvas(),n=null;if(e){var r=this.getShapeBase()[Object(i.upperFirst)(e.type)];r&&(n=new r({type:e.type,isClipShape:!0,applyTo:this,attrs:e.attrs,canvas:t}))}return this.set("clipShape",n),this.onCanvasChange("clip"),n},t.prototype.getClip=function(){return this.cfg.clipShape||null},t.prototype.clone=function(){var e=this,t=this.attrs,n={};Object(i.each)(t,(function(e,r){Object(i.isArray)(t[r])?n[r]=function(e){for(var t=[],n=0;n<e.length;n++)Object(i.isArray)(e[n])?t.push([].concat(e[n])):t.push(e[n]);return t}(t[r]):n[r]=t[r]}));var r=new(0,this.constructor)({attrs:n});return Object(i.each)(c,(function(t){r.set(t,e.get(t))})),r},t.prototype.destroy=function(){this.destroyed||(this.attrs={},e.prototype.destroy.call(this))},t.prototype.isAnimatePaused=function(){return this.get("_pause").isPaused},t.prototype.animate=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];this.set("animating",!0);var n=this.get("timeline");n||(n=this.get("canvas").get("timeline"),this.set("timeline",n));var r=this.get("animations")||[];n.timer||n.initTimer();var o,a,s,l,u,c=e[0],d=e[1],g=e[2],v=void 0===g?"easeLinear":g,m=e[3],y=void 0===m?i.noop:m,b=e[4],C=void 0===b?0:b;Object(i.isFunction)(c)?(o=c,c={}):Object(i.isObject)(c)&&c.onFrame&&(o=c.onFrame,a=c.repeat),Object(i.isObject)(d)?(d=(u=d).duration,v=u.easing||"easeLinear",C=u.delay||0,a=u.repeat||a||!1,y=u.callback||i.noop,s=u.pauseCallback||i.noop,l=u.resumeCallback||i.noop):(Object(i.isNumber)(y)&&(C=y,y=null),Object(i.isFunction)(v)?(y=v,v="easeLinear"):v=v||"easeLinear");var A=f(c,this),_={fromAttrs:h(A,this),toAttrs:A,duration:d,easing:v,repeat:a,callback:y,pauseCallback:s,resumeCallback:l,delay:C,startTime:n.getTime(),id:Object(i.uniqueId)(),onFrame:o,pathFormatted:!1};r.length>0?r=p(r,_):n.addAnimator(this),r.push(_),this.set("animations",r),this.set("_pause",{isPaused:!1})},t.prototype.stopAnimate=function(e){var t=this;void 0===e&&(e=!0);var n=this.get("animations");Object(i.each)(n,(function(n){e&&(n.onFrame?t.attr(n.onFrame(1)):t.attr(n.toAttrs)),n.callback&&n.callback()})),this.set("animating",!1),this.set("animations",[])},t.prototype.pauseAnimate=function(){var e=this.get("timeline"),t=this.get("animations"),n=e.getTime();return Object(i.each)(t,(function(e){e._paused=!0,e._pauseTime=n,e.pauseCallback&&e.pauseCallback()})),this.set("_pause",{isPaused:!0,pauseTime:n}),this},t.prototype.resumeAnimate=function(){var e=this.get("timeline").getTime(),t=this.get("animations"),n=this.get("_pause").pauseTime;return Object(i.each)(t,(function(t){t.startTime=t.startTime+(e-n),t._paused=!1,t._pauseTime=null,t.resumeCallback&&t.resumeCallback()})),this.set("_pause",{isPaused:!1}),this.set("animations",t),this},t.prototype.emitDelegation=function(e,t){var n,r=this,o=t.propagationPath;this.getEvents(),"mouseenter"===e?n=t.fromShape:"mouseleave"===e&&(n=t.toShape);for(var s=function(e){var s=o[e],u=s.get("name");if(u){if((s.isGroup()||s.isCanvas&&s.isCanvas())&&n&&Object(a.g)(s,n))return"break";Object(i.isArray)(u)?Object(i.each)(u,(function(e){r.emitDelegateEvent(s,e,t)})):l.emitDelegateEvent(s,u,t)}},l=this,u=0;u<o.length&&"break"!==s(u);u++);},t.prototype.emitDelegateEvent=function(e,t,n){var r=this.getEvents(),i=t+":"+n.type;(r[i]||r["*"])&&(n.name=i,n.currentTarget=e,n.delegateTarget=this,n.delegateObject=e.get("delegateObject"),this.emit(i,n))},t.prototype.translate=function(e,t){void 0===e&&(e=0),void 0===t&&(t=0);var n=this.getMatrix(),r=u(n,[["t",e,t]]);return this.setMatrix(r),this},t.prototype.move=function(e,t){var n=this.attr("x")||0,r=this.attr("y")||0;return this.translate(e-n,t-r),this},t.prototype.moveTo=function(e,t){return this.move(e,t)},t.prototype.scale=function(e,t){var n=this.getMatrix(),r=u(n,[["s",e,t||e]]);return this.setMatrix(r),this},t.prototype.rotate=function(e){var t=this.getMatrix(),n=u(t,[["r",e]]);return this.setMatrix(n),this},t.prototype.rotateAtStart=function(e){var t=this.attr(),n=t.x,r=t.y,i=this.getMatrix(),o=u(i,[["t",-n,-r],["r",e],["t",n,r]]);return this.setMatrix(o),this},t.prototype.rotateAtPoint=function(e,t,n){var r=this.getMatrix(),i=u(r,[["t",-e,-t],["r",n],["t",e,t]]);return this.setMatrix(i),this},t}(l.a);t.a=g},function(e,t,n){"use strict";function r(e){return(r="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){return"object"===r(e)&&null!==e}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t,n){return void 0===n&&(n=1e-5),Math.abs(e-t)<n}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.setMatrixArrayType=function(e){t.ARRAY_TYPE=e},t.toRadian=function(e){return e*i},t.equals=function(e,t){return Math.abs(e-t)<=r*Math.max(1,Math.abs(e),Math.abs(t))};var r=t.EPSILON=1e-6;t.ARRAY_TYPE="undefined"!=typeof Float32Array?Float32Array:Array,t.RANDOM=Math.random;var i=Math.PI/180},function(e,t,n){"use strict";n.r(t),n.d(t,"create",(function(){return i})),n.d(t,"clone",(function(){return o})),n.d(t,"fromValues",(function(){return a})),n.d(t,"copy",(function(){return s})),n.d(t,"set",(function(){return l})),n.d(t,"add",(function(){return u})),n.d(t,"subtract",(function(){return c})),n.d(t,"multiply",(function(){return d})),n.d(t,"divide",(function(){return h})),n.d(t,"ceil",(function(){return f})),n.d(t,"floor",(function(){return p})),n.d(t,"min",(function(){return g})),n.d(t,"max",(function(){return v})),n.d(t,"round",(function(){return m})),n.d(t,"scale",(function(){return y})),n.d(t,"scaleAndAdd",(function(){return b})),n.d(t,"distance",(function(){return C})),n.d(t,"squaredDistance",(function(){return A})),n.d(t,"length",(function(){return _})),n.d(t,"squaredLength",(function(){return I})),n.d(t,"negate",(function(){return x})),n.d(t,"inverse",(function(){return w})),n.d(t,"normalize",(function(){return M})),n.d(t,"dot",(function(){return S})),n.d(t,"cross",(function(){return T})),n.d(t,"lerp",(function(){return E})),n.d(t,"random",(function(){return O})),n.d(t,"transformMat2",(function(){return k})),n.d(t,"transformMat2d",(function(){return L})),n.d(t,"transformMat3",(function(){return D})),n.d(t,"transformMat4",(function(){return P})),n.d(t,"rotate",(function(){return N})),n.d(t,"angle",(function(){return j})),n.d(t,"zero",(function(){return R})),n.d(t,"str",(function(){return z})),n.d(t,"exactEquals",(function(){return F})),n.d(t,"equals",(function(){return B})),n.d(t,"len",(function(){return U})),n.d(t,"sub",(function(){return V})),n.d(t,"mul",(function(){return Y})),n.d(t,"div",(function(){return W})),n.d(t,"dist",(function(){return G})),n.d(t,"sqrDist",(function(){return X})),n.d(t,"sqrLen",(function(){return q})),n.d(t,"forEach",(function(){return Z}));var r=n(6);function i(){var e=new r.a(2);return r.a!=Float32Array&&(e[0]=0,e[1]=0),e}function o(e){var t=new r.a(2);return t[0]=e[0],t[1]=e[1],t}function a(e,t){var n=new r.a(2);return n[0]=e,n[1]=t,n}function s(e,t){return e[0]=t[0],e[1]=t[1],e}function l(e,t,n){return e[0]=t,e[1]=n,e}function u(e,t,n){return e[0]=t[0]+n[0],e[1]=t[1]+n[1],e}function c(e,t,n){return e[0]=t[0]-n[0],e[1]=t[1]-n[1],e}function d(e,t,n){return e[0]=t[0]*n[0],e[1]=t[1]*n[1],e}function h(e,t,n){return e[0]=t[0]/n[0],e[1]=t[1]/n[1],e}function f(e,t){return e[0]=Math.ceil(t[0]),e[1]=Math.ceil(t[1]),e}function p(e,t){return e[0]=Math.floor(t[0]),e[1]=Math.floor(t[1]),e}function g(e,t,n){return e[0]=Math.min(t[0],n[0]),e[1]=Math.min(t[1],n[1]),e}function v(e,t,n){return e[0]=Math.max(t[0],n[0]),e[1]=Math.max(t[1],n[1]),e}function m(e,t){return e[0]=Math.round(t[0]),e[1]=Math.round(t[1]),e}function y(e,t,n){return e[0]=t[0]*n,e[1]=t[1]*n,e}function b(e,t,n,r){return e[0]=t[0]+n[0]*r,e[1]=t[1]+n[1]*r,e}function C(e,t){var n=t[0]-e[0],r=t[1]-e[1];return Math.hypot(n,r)}function A(e,t){var n=t[0]-e[0],r=t[1]-e[1];return n*n+r*r}function _(e){var t=e[0],n=e[1];return Math.hypot(t,n)}function I(e){var t=e[0],n=e[1];return t*t+n*n}function x(e,t){return e[0]=-t[0],e[1]=-t[1],e}function w(e,t){return e[0]=1/t[0],e[1]=1/t[1],e}function M(e,t){var n=t[0],r=t[1],i=n*n+r*r;return i>0&&(i=1/Math.sqrt(i)),e[0]=t[0]*i,e[1]=t[1]*i,e}function S(e,t){return e[0]*t[0]+e[1]*t[1]}function T(e,t,n){var r=t[0]*n[1]-t[1]*n[0];return e[0]=e[1]=0,e[2]=r,e}function E(e,t,n,r){var i=t[0],o=t[1];return e[0]=i+r*(n[0]-i),e[1]=o+r*(n[1]-o),e}function O(e,t){t=t||1;var n=2*r.c()*Math.PI;return e[0]=Math.cos(n)*t,e[1]=Math.sin(n)*t,e}function k(e,t,n){var r=t[0],i=t[1];return e[0]=n[0]*r+n[2]*i,e[1]=n[1]*r+n[3]*i,e}function L(e,t,n){var r=t[0],i=t[1];return e[0]=n[0]*r+n[2]*i+n[4],e[1]=n[1]*r+n[3]*i+n[5],e}function D(e,t,n){var r=t[0],i=t[1];return e[0]=n[0]*r+n[3]*i+n[6],e[1]=n[1]*r+n[4]*i+n[7],e}function P(e,t,n){var r=t[0],i=t[1];return e[0]=n[0]*r+n[4]*i+n[12],e[1]=n[1]*r+n[5]*i+n[13],e}function N(e,t,n,r){var i=t[0]-n[0],o=t[1]-n[1],a=Math.sin(r),s=Math.cos(r);return e[0]=i*s-o*a+n[0],e[1]=i*a+o*s+n[1],e}function j(e,t){var n=e[0],r=e[1],i=t[0],o=t[1],a=Math.sqrt(n*n+r*r)*Math.sqrt(i*i+o*o),s=a&&(n*i+r*o)/a;return Math.acos(Math.min(Math.max(s,-1),1))}function R(e){return e[0]=0,e[1]=0,e}function z(e){return"vec2("+e[0]+", "+e[1]+")"}function F(e,t){return e[0]===t[0]&&e[1]===t[1]}function B(e,t){var n=e[0],i=e[1],o=t[0],a=t[1];return Math.abs(n-o)<=r.b*Math.max(1,Math.abs(n),Math.abs(o))&&Math.abs(i-a)<=r.b*Math.max(1,Math.abs(i),Math.abs(a))}var H,U=_,V=c,Y=d,W=h,G=C,X=A,q=I,Z=(H=i(),function(e,t,n,r,i,o){var a,s;for(t||(t=2),n||(n=0),s=r?Math.min(r*t+n,e.length):e.length,a=n;a<s;a+=t)H[0]=e[a],H[1]=e[a+1],i(H,H,o),e[a]=H[0],e[a+1]=H[1];return e})},function(e,t,n){"use strict";n.r(t),n.d(t,"create",(function(){return i})),n.d(t,"clone",(function(){return o})),n.d(t,"length",(function(){return a})),n.d(t,"fromValues",(function(){return s})),n.d(t,"copy",(function(){return l})),n.d(t,"set",(function(){return u})),n.d(t,"add",(function(){return c})),n.d(t,"subtract",(function(){return d})),n.d(t,"multiply",(function(){return h})),n.d(t,"divide",(function(){return f})),n.d(t,"ceil",(function(){return p})),n.d(t,"floor",(function(){return g})),n.d(t,"min",(function(){return v})),n.d(t,"max",(function(){return m})),n.d(t,"round",(function(){return y})),n.d(t,"scale",(function(){return b})),n.d(t,"scaleAndAdd",(function(){return C})),n.d(t,"distance",(function(){return A})),n.d(t,"squaredDistance",(function(){return _})),n.d(t,"squaredLength",(function(){return I})),n.d(t,"negate",(function(){return x})),n.d(t,"inverse",(function(){return w})),n.d(t,"normalize",(function(){return M})),n.d(t,"dot",(function(){return S})),n.d(t,"cross",(function(){return T})),n.d(t,"lerp",(function(){return E})),n.d(t,"hermite",(function(){return O})),n.d(t,"bezier",(function(){return k})),n.d(t,"random",(function(){return L})),n.d(t,"transformMat4",(function(){return D})),n.d(t,"transformMat3",(function(){return P})),n.d(t,"transformQuat",(function(){return N})),n.d(t,"rotateX",(function(){return j})),n.d(t,"rotateY",(function(){return R})),n.d(t,"rotateZ",(function(){return z})),n.d(t,"angle",(function(){return F})),n.d(t,"zero",(function(){return B})),n.d(t,"str",(function(){return H})),n.d(t,"exactEquals",(function(){return U})),n.d(t,"equals",(function(){return V})),n.d(t,"sub",(function(){return W})),n.d(t,"mul",(function(){return G})),n.d(t,"div",(function(){return X})),n.d(t,"dist",(function(){return q})),n.d(t,"sqrDist",(function(){return Z})),n.d(t,"len",(function(){return Q})),n.d(t,"sqrLen",(function(){return K})),n.d(t,"forEach",(function(){return J}));var r=n(6);function i(){var e=new r.a(3);return r.a!=Float32Array&&(e[0]=0,e[1]=0,e[2]=0),e}function o(e){var t=new r.a(3);return t[0]=e[0],t[1]=e[1],t[2]=e[2],t}function a(e){var t=e[0],n=e[1],r=e[2];return Math.hypot(t,n,r)}function s(e,t,n){var i=new r.a(3);return i[0]=e,i[1]=t,i[2]=n,i}function l(e,t){return e[0]=t[0],e[1]=t[1],e[2]=t[2],e}function u(e,t,n,r){return e[0]=t,e[1]=n,e[2]=r,e}function c(e,t,n){return e[0]=t[0]+n[0],e[1]=t[1]+n[1],e[2]=t[2]+n[2],e}function d(e,t,n){return e[0]=t[0]-n[0],e[1]=t[1]-n[1],e[2]=t[2]-n[2],e}function h(e,t,n){return e[0]=t[0]*n[0],e[1]=t[1]*n[1],e[2]=t[2]*n[2],e}function f(e,t,n){return e[0]=t[0]/n[0],e[1]=t[1]/n[1],e[2]=t[2]/n[2],e}function p(e,t){return e[0]=Math.ceil(t[0]),e[1]=Math.ceil(t[1]),e[2]=Math.ceil(t[2]),e}function g(e,t){return e[0]=Math.floor(t[0]),e[1]=Math.floor(t[1]),e[2]=Math.floor(t[2]),e}function v(e,t,n){return e[0]=Math.min(t[0],n[0]),e[1]=Math.min(t[1],n[1]),e[2]=Math.min(t[2],n[2]),e}function m(e,t,n){return e[0]=Math.max(t[0],n[0]),e[1]=Math.max(t[1],n[1]),e[2]=Math.max(t[2],n[2]),e}function y(e,t){return e[0]=Math.round(t[0]),e[1]=Math.round(t[1]),e[2]=Math.round(t[2]),e}function b(e,t,n){return e[0]=t[0]*n,e[1]=t[1]*n,e[2]=t[2]*n,e}function C(e,t,n,r){return e[0]=t[0]+n[0]*r,e[1]=t[1]+n[1]*r,e[2]=t[2]+n[2]*r,e}function A(e,t){var n=t[0]-e[0],r=t[1]-e[1],i=t[2]-e[2];return Math.hypot(n,r,i)}function _(e,t){var n=t[0]-e[0],r=t[1]-e[1],i=t[2]-e[2];return n*n+r*r+i*i}function I(e){var t=e[0],n=e[1],r=e[2];return t*t+n*n+r*r}function x(e,t){return e[0]=-t[0],e[1]=-t[1],e[2]=-t[2],e}function w(e,t){return e[0]=1/t[0],e[1]=1/t[1],e[2]=1/t[2],e}function M(e,t){var n=t[0],r=t[1],i=t[2],o=n*n+r*r+i*i;return o>0&&(o=1/Math.sqrt(o)),e[0]=t[0]*o,e[1]=t[1]*o,e[2]=t[2]*o,e}function S(e,t){return e[0]*t[0]+e[1]*t[1]+e[2]*t[2]}function T(e,t,n){var r=t[0],i=t[1],o=t[2],a=n[0],s=n[1],l=n[2];return e[0]=i*l-o*s,e[1]=o*a-r*l,e[2]=r*s-i*a,e}function E(e,t,n,r){var i=t[0],o=t[1],a=t[2];return e[0]=i+r*(n[0]-i),e[1]=o+r*(n[1]-o),e[2]=a+r*(n[2]-a),e}function O(e,t,n,r,i,o){var a=o*o,s=a*(2*o-3)+1,l=a*(o-2)+o,u=a*(o-1),c=a*(3-2*o);return e[0]=t[0]*s+n[0]*l+r[0]*u+i[0]*c,e[1]=t[1]*s+n[1]*l+r[1]*u+i[1]*c,e[2]=t[2]*s+n[2]*l+r[2]*u+i[2]*c,e}function k(e,t,n,r,i,o){var a=1-o,s=a*a,l=o*o,u=s*a,c=3*o*s,d=3*l*a,h=l*o;return e[0]=t[0]*u+n[0]*c+r[0]*d+i[0]*h,e[1]=t[1]*u+n[1]*c+r[1]*d+i[1]*h,e[2]=t[2]*u+n[2]*c+r[2]*d+i[2]*h,e}function L(e,t){t=t||1;var n=2*r.c()*Math.PI,i=2*r.c()-1,o=Math.sqrt(1-i*i)*t;return e[0]=Math.cos(n)*o,e[1]=Math.sin(n)*o,e[2]=i*t,e}function D(e,t,n){var r=t[0],i=t[1],o=t[2],a=n[3]*r+n[7]*i+n[11]*o+n[15];return a=a||1,e[0]=(n[0]*r+n[4]*i+n[8]*o+n[12])/a,e[1]=(n[1]*r+n[5]*i+n[9]*o+n[13])/a,e[2]=(n[2]*r+n[6]*i+n[10]*o+n[14])/a,e}function P(e,t,n){var r=t[0],i=t[1],o=t[2];return e[0]=r*n[0]+i*n[3]+o*n[6],e[1]=r*n[1]+i*n[4]+o*n[7],e[2]=r*n[2]+i*n[5]+o*n[8],e}function N(e,t,n){var r=n[0],i=n[1],o=n[2],a=n[3],s=t[0],l=t[1],u=t[2],c=i*u-o*l,d=o*s-r*u,h=r*l-i*s,f=i*h-o*d,p=o*c-r*h,g=r*d-i*c,v=2*a;return c*=v,d*=v,h*=v,f*=2,p*=2,g*=2,e[0]=s+c+f,e[1]=l+d+p,e[2]=u+h+g,e}function j(e,t,n,r){var i=[],o=[];return i[0]=t[0]-n[0],i[1]=t[1]-n[1],i[2]=t[2]-n[2],o[0]=i[0],o[1]=i[1]*Math.cos(r)-i[2]*Math.sin(r),o[2]=i[1]*Math.sin(r)+i[2]*Math.cos(r),e[0]=o[0]+n[0],e[1]=o[1]+n[1],e[2]=o[2]+n[2],e}function R(e,t,n,r){var i=[],o=[];return i[0]=t[0]-n[0],i[1]=t[1]-n[1],i[2]=t[2]-n[2],o[0]=i[2]*Math.sin(r)+i[0]*Math.cos(r),o[1]=i[1],o[2]=i[2]*Math.cos(r)-i[0]*Math.sin(r),e[0]=o[0]+n[0],e[1]=o[1]+n[1],e[2]=o[2]+n[2],e}function z(e,t,n,r){var i=[],o=[];return i[0]=t[0]-n[0],i[1]=t[1]-n[1],i[2]=t[2]-n[2],o[0]=i[0]*Math.cos(r)-i[1]*Math.sin(r),o[1]=i[0]*Math.sin(r)+i[1]*Math.cos(r),o[2]=i[2],e[0]=o[0]+n[0],e[1]=o[1]+n[1],e[2]=o[2]+n[2],e}function F(e,t){var n=e[0],r=e[1],i=e[2],o=t[0],a=t[1],s=t[2],l=Math.sqrt(n*n+r*r+i*i)*Math.sqrt(o*o+a*a+s*s),u=l&&S(e,t)/l;return Math.acos(Math.min(Math.max(u,-1),1))}function B(e){return e[0]=0,e[1]=0,e[2]=0,e}function H(e){return"vec3("+e[0]+", "+e[1]+", "+e[2]+")"}function U(e,t){return e[0]===t[0]&&e[1]===t[1]&&e[2]===t[2]}function V(e,t){var n=e[0],i=e[1],o=e[2],a=t[0],s=t[1],l=t[2];return Math.abs(n-a)<=r.b*Math.max(1,Math.abs(n),Math.abs(a))&&Math.abs(i-s)<=r.b*Math.max(1,Math.abs(i),Math.abs(s))&&Math.abs(o-l)<=r.b*Math.max(1,Math.abs(o),Math.abs(l))}var Y,W=d,G=h,X=f,q=A,Z=_,Q=a,K=I,J=(Y=i(),function(e,t,n,r,i,o){var a,s;for(t||(t=3),n||(n=0),s=r?Math.min(r*t+n,e.length):e.length,a=n;a<s;a+=t)Y[0]=e[a],Y[1]=e[a+1],Y[2]=e[a+2],i(Y,Y,o),e[a]=Y[0],e[a+1]=Y[1],e[a+2]=Y[2];return e})},function(e,t,n){"use strict";function r(e,t){return e&&t?{minX:Math.min(e.minX,t.minX),minY:Math.min(e.minY,t.minY),maxX:Math.max(e.maxX,t.maxX),maxY:Math.max(e.maxY,t.maxY)}:e||t}Object.defineProperty(t,"__esModule",{value:!0}),t.mergeArrowBBox=t.mergeBBox=void 0,t.mergeBBox=r,t.mergeArrowBBox=function(e,t){var n=e.get("startArrowShape"),i=e.get("endArrowShape");return n&&(t=r(t,n.getCanvasBBox())),i&&(t=r(t,i.getCanvasBBox())),t}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(33),i=n(40),o=n(161);function a(e,t,n,r,i){var o=1-i;return o*o*o*e+3*t*i*o*o+3*n*i*i*o+r*i*i*i}function s(e,t,n,r,i){var o=1-i;return 3*(o*o*(t-e)+2*o*i*(n-t)+i*i*(r-n))}function l(e,t,n,i){var o,a,s,l=-3*e+9*t-9*n+3*i,u=6*e-12*t+6*n,c=3*t-3*e,d=[];if(r.isNumberEqual(l,0))r.isNumberEqual(u,0)||(o=-c/u)>=0&&o<=1&&d.push(o);else{var h=u*u-4*l*c;r.isNumberEqual(h,0)?d.push(-u/(2*l)):h>0&&(a=(-u-(s=Math.sqrt(h)))/(2*l),(o=(-u+s)/(2*l))>=0&&o<=1&&d.push(o),a>=0&&a<=1&&d.push(a))}return d}function u(e,t,n,r,o,s,l,u,c){var d=a(e,n,o,l,c),h=a(t,r,s,u,c),f=i.default.pointAt(e,t,n,r,c),p=i.default.pointAt(n,r,o,s,c),g=i.default.pointAt(o,s,l,u,c),v=i.default.pointAt(f.x,f.y,p.x,p.y,c),m=i.default.pointAt(p.x,p.y,g.x,g.y,c);return[[e,t,f.x,f.y,v.x,v.y,d,h],[d,h,m.x,m.y,g.x,g.y,l,u]]}t.default={extrema:l,box:function(e,t,n,i,o,s,u,c){for(var d=[e,u],h=[t,c],f=l(e,n,o,u),p=l(t,i,s,c),g=0;g<f.length;g++)d.push(a(e,n,o,u,f[g]));for(g=0;g<p.length;g++)h.push(a(t,i,s,c,p[g]));return r.getBBoxByArray(d,h)},length:function(e,t,n,r,i,a,s,l){return function e(t,n,r,i,a,s,l,c,d){if(0===d)return o.snapLength([t,r,a,l],[n,i,s,c]);var h=u(t,n,r,i,a,s,l,c,.5),f=h[0],p=h[1];return f.push(d-1),p.push(d-1),e.apply(null,f)+e.apply(null,p)}(e,t,n,r,i,a,s,l,3)},nearestPoint:function(e,t,n,r,i,s,l,u,c,d,h){return o.nearestPoint([e,n,i,l],[t,r,s,u],c,d,a,h)},pointDistance:function(e,t,n,i,o,a,s,l,u,c,d){var h=this.nearestPoint(e,t,n,i,o,a,s,l,u,c,d);return r.distance(h.x,h.y,u,c)},interpolationAt:a,pointAt:function(e,t,n,r,i,o,s,l,u){return{x:a(e,n,i,s,u),y:a(t,r,o,l,u)}},divide:function(e,t,n,r,i,o,a,s,l){return u(e,t,n,r,i,o,a,s,l)},tangentAngle:function(e,t,n,i,o,a,l,u,c){var d=s(e,n,o,l,c),h=s(t,i,a,u,c);return r.piMod(Math.atan2(h,d))}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.refreshElement=t.drawChildren=void 0;var r=n(61),i=n(29);t.drawChildren=function(e,t){t.forEach((function(t){t.draw(e)}))},t.refreshElement=function(e,t){var n=e.get("canvas");if(n&&n.get("autoDraw")){var o=n.get("context"),a=e.getParent(),s=a?a.getChildren():[n],l=e.get("el");if("remove"===t)if(e.get("isClipShape")){var u=l&&l.parentNode,c=u&&u.parentNode;u&&c&&c.removeChild(u)}else l&&l.parentNode&&l.parentNode.removeChild(l);else if("show"===t)l.setAttribute("visibility","visible");else if("hide"===t)l.setAttribute("visibility","hidden");else if("zIndex"===t)i.moveTo(l,s.indexOf(e));else if("sort"===t){var d=e.get("children");d&&d.length&&i.sortDom(e,(function(e,t){return d.indexOf(e)-d.indexOf(t)?1:0}))}else"clear"===t?l&&(l.innerHTML=""):"matrix"===t?r.setTransform(e):"clip"===t?r.setClip(e,o):"attr"===t||"add"===t&&e.draw(o)}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(23);Object.defineProperty(t,"Base",{enumerable:!0,get:function(){return r.default}});var i=n(349);Object.defineProperty(t,"Circle",{enumerable:!0,get:function(){return i.default}});var o=n(350);Object.defineProperty(t,"Dom",{enumerable:!0,get:function(){return o.default}});var a=n(351);Object.defineProperty(t,"Ellipse",{enumerable:!0,get:function(){return a.default}});var s=n(352);Object.defineProperty(t,"Image",{enumerable:!0,get:function(){return s.default}});var l=n(353);Object.defineProperty(t,"Line",{enumerable:!0,get:function(){return l.default}});var u=n(354);Object.defineProperty(t,"Marker",{enumerable:!0,get:function(){return u.default}});var c=n(356);Object.defineProperty(t,"Path",{enumerable:!0,get:function(){return c.default}});var d=n(357);Object.defineProperty(t,"Polygon",{enumerable:!0,get:function(){return d.default}});var h=n(358);Object.defineProperty(t,"Polyline",{enumerable:!0,get:function(){return h.default}});var f=n(359);Object.defineProperty(t,"Rect",{enumerable:!0,get:function(){return f.default}});var p=n(361);Object.defineProperty(t,"Text",{enumerable:!0,get:function(){return p.default}})},function(e,t,n){"use strict";function r(e){return(r="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}var i=n(24);function o(e){this._isDirected=!i.has(e,"directed")||e.directed,this._isMultigraph=!!i.has(e,"multigraph")&&e.multigraph,this._isCompound=!!i.has(e,"compound")&&e.compound,this._label=void 0,this._defaultNodeLabelFn=i.constant(void 0),this._defaultEdgeLabelFn=i.constant(void 0),this._nodes={},this._isCompound&&(this._parent={},this._children={},this._children["\0"]={}),this._in={},this._preds={},this._out={},this._sucs={},this._edgeObjs={},this._edgeLabels={}}function a(e,t){e[t]?e[t]++:e[t]=1}function s(e,t){--e[t]||delete e[t]}function l(e,t,n,r){var o=""+t,a=""+n;if(!e&&o>a){var s=o;o=a,a=s}return o+""+a+""+(i.isUndefined(r)?"\0":r)}function u(e,t,n,r){var i=""+t,o=""+n;if(!e&&i>o){var a=i;i=o,o=a}var s={v:i,w:o};return r&&(s.name=r),s}function c(e,t){return l(e,t.v,t.w,t.name)}e.exports=o,o.prototype._nodeCount=0,o.prototype._edgeCount=0,o.prototype.isDirected=function(){return this._isDirected},o.prototype.isMultigraph=function(){return this._isMultigraph},o.prototype.isCompound=function(){return this._isCompound},o.prototype.setGraph=function(e){return this._label=e,this},o.prototype.graph=function(){return this._label},o.prototype.setDefaultNodeLabel=function(e){return i.isFunction(e)||(e=i.constant(e)),this._defaultNodeLabelFn=e,this},o.prototype.nodeCount=function(){return this._nodeCount},o.prototype.nodes=function(){return i.keys(this._nodes)},o.prototype.sources=function(){var e=this;return i.filter(this.nodes(),(function(t){return i.isEmpty(e._in[t])}))},o.prototype.sinks=function(){var e=this;return i.filter(this.nodes(),(function(t){return i.isEmpty(e._out[t])}))},o.prototype.setNodes=function(e,t){var n=arguments,r=this;return i.each(e,(function(e){n.length>1?r.setNode(e,t):r.setNode(e)})),this},o.prototype.setNode=function(e,t){return i.has(this._nodes,e)?(arguments.length>1&&(this._nodes[e]=t),this):(this._nodes[e]=arguments.length>1?t:this._defaultNodeLabelFn(e),this._isCompound&&(this._parent[e]="\0",this._children[e]={},this._children["\0"][e]=!0),this._in[e]={},this._preds[e]={},this._out[e]={},this._sucs[e]={},++this._nodeCount,this)},o.prototype.node=function(e){return this._nodes[e]},o.prototype.hasNode=function(e){return i.has(this._nodes,e)},o.prototype.removeNode=function(e){var t=this;if(i.has(this._nodes,e)){var n=function(e){t.removeEdge(t._edgeObjs[e])};delete this._nodes[e],this._isCompound&&(this._removeFromParentsChildList(e),delete this._parent[e],i.each(this.children(e),(function(e){t.setParent(e)})),delete this._children[e]),i.each(i.keys(this._in[e]),n),delete this._in[e],delete this._preds[e],i.each(i.keys(this._out[e]),n),delete this._out[e],delete this._sucs[e],--this._nodeCount}return this},o.prototype.setParent=function(e,t){if(!this._isCompound)throw new Error("Cannot set parent in a non-compound graph");if(i.isUndefined(t))t="\0";else{for(var n=t+="";!i.isUndefined(n);n=this.parent(n))if(n===e)throw new Error("Setting "+t+" as parent of "+e+" would create a cycle");this.setNode(t)}return this.setNode(e),this._removeFromParentsChildList(e),this._parent[e]=t,this._children[t][e]=!0,this},o.prototype._removeFromParentsChildList=function(e){delete this._children[this._parent[e]][e]},o.prototype.parent=function(e){if(this._isCompound){var t=this._parent[e];if("\0"!==t)return t}},o.prototype.children=function(e){if(i.isUndefined(e)&&(e="\0"),this._isCompound){var t=this._children[e];if(t)return i.keys(t)}else{if("\0"===e)return this.nodes();if(this.hasNode(e))return[]}},o.prototype.predecessors=function(e){var t=this._preds[e];if(t)return i.keys(t)},o.prototype.successors=function(e){var t=this._sucs[e];if(t)return i.keys(t)},o.prototype.neighbors=function(e){var t=this.predecessors(e);if(t)return i.union(t,this.successors(e))},o.prototype.isLeaf=function(e){return 0===(this.isDirected()?this.successors(e):this.neighbors(e)).length},o.prototype.filterNodes=function(e){var t=new this.constructor({directed:this._isDirected,multigraph:this._isMultigraph,compound:this._isCompound});t.setGraph(this.graph());var n=this;i.each(this._nodes,(function(n,r){e(r)&&t.setNode(r,n)})),i.each(this._edgeObjs,(function(e){t.hasNode(e.v)&&t.hasNode(e.w)&&t.setEdge(e,n.edge(e))}));var r={};return this._isCompound&&i.each(t.nodes(),(function(e){t.setParent(e,function e(i){var o=n.parent(i);return void 0===o||t.hasNode(o)?(r[i]=o,o):o in r?r[o]:e(o)}(e))})),t},o.prototype.setDefaultEdgeLabel=function(e){return i.isFunction(e)||(e=i.constant(e)),this._defaultEdgeLabelFn=e,this},o.prototype.edgeCount=function(){return this._edgeCount},o.prototype.edges=function(){return i.values(this._edgeObjs)},o.prototype.setPath=function(e,t){var n=this,r=arguments;return i.reduce(e,(function(e,i){return r.length>1?n.setEdge(e,i,t):n.setEdge(e,i),i})),this},o.prototype.setEdge=function(){var e,t,n,o,s=!1,c=arguments[0];"object"===r(c)&&null!==c&&"v"in c?(e=c.v,t=c.w,n=c.name,2===arguments.length&&(o=arguments[1],s=!0)):(e=c,t=arguments[1],n=arguments[3],arguments.length>2&&(o=arguments[2],s=!0)),e=""+e,t=""+t,i.isUndefined(n)||(n=""+n);var d=l(this._isDirected,e,t,n);if(i.has(this._edgeLabels,d))return s&&(this._edgeLabels[d]=o),this;if(!i.isUndefined(n)&&!this._isMultigraph)throw new Error("Cannot set a named edge when isMultigraph = false");this.setNode(e),this.setNode(t),this._edgeLabels[d]=s?o:this._defaultEdgeLabelFn(e,t,n);var h=u(this._isDirected,e,t,n);return e=h.v,t=h.w,Object.freeze(h),this._edgeObjs[d]=h,a(this._preds[t],e),a(this._sucs[e],t),this._in[t][d]=h,this._out[e][d]=h,this._edgeCount++,this},o.prototype.edge=function(e,t,n){var r=1===arguments.length?c(this._isDirected,arguments[0]):l(this._isDirected,e,t,n);return this._edgeLabels[r]},o.prototype.hasEdge=function(e,t,n){var r=1===arguments.length?c(this._isDirected,arguments[0]):l(this._isDirected,e,t,n);return i.has(this._edgeLabels,r)},o.prototype.removeEdge=function(e,t,n){var r=1===arguments.length?c(this._isDirected,arguments[0]):l(this._isDirected,e,t,n),i=this._edgeObjs[r];return i&&(e=i.v,t=i.w,delete this._edgeLabels[r],delete this._edgeObjs[r],s(this._preds[t],e),s(this._sucs[e],t),delete this._in[t][r],delete this._out[e][r],this._edgeCount--),this},o.prototype.inEdges=function(e,t){var n=this._in[e];if(n){var r=i.values(n);return t?i.filter(r,(function(e){return e.v===t})):r}},o.prototype.outEdges=function(e,t){var n=this._out[e];if(n){var r=i.values(n);return t?i.filter(r,(function(e){return e.w===t})):r}},o.prototype.nodeEdges=function(e,t){var n=this.inEdges(e,t);if(n)return n.concat(this.outEdges(e,t))}},function(e,t,n){var r=n(44)(n(31),"Map");e.exports=r},function(e,t,n){var r=n(391),i=n(398),o=n(400),a=n(401),s=n(402);function l(e){var t=-1,n=null==e?0:e.length;for(this.clear();++t<n;){var r=e[t];this.set(r[0],r[1])}}l.prototype.clear=r,l.prototype.delete=i,l.prototype.get=o,l.prototype.has=a,l.prototype.set=s,e.exports=l},function(e,t){e.exports=function(e,t){for(var n=-1,r=null==e?0:e.length;++n<r&&!1!==t(e[n],n,e););return e}},function(e,t){e.exports=function(e){return e.webpackPolyfill||(e.deprecate=function(){},e.paths=[],e.children||(e.children=[]),Object.defineProperty(e,"loaded",{enumerable:!0,get:function(){return e.l}}),Object.defineProperty(e,"id",{enumerable:!0,get:function(){return e.i}}),e.webpackPolyfill=1),e}},function(e,t){e.exports=function(e){return"number"==typeof e&&e>-1&&e%1==0&&e<=9007199254740991}},function(e,t,n){(function(e){function r(e){return(r="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}var i=n(174),o="object"==r(t)&&t&&!t.nodeType&&t,a=o&&"object"==r(e)&&e&&!e.nodeType&&e,s=a&&a.exports===o&&i.process,l=function(){try{return a&&a.require&&a.require("util").types||s&&s.binding&&s.binding("util")}catch(e){}}();e.exports=l}).call(this,n(115)(e))},function(e,t,n){var r=n(85),i=n(408),o=Object.prototype.hasOwnProperty;e.exports=function(e){if(!r(e))return i(e);var t=[];for(var n in Object(e))o.call(e,n)&&"constructor"!=n&&t.push(n);return t}},function(e,t,n){var r=n(181),i=n(182),o=Object.prototype.propertyIsEnumerable,a=Object.getOwnPropertySymbols,s=a?function(e){return null==e?[]:(e=Object(e),r(a(e),(function(t){return o.call(e,t)})))}:i;e.exports=s},function(e,t){e.exports=function(e,t){for(var n=-1,r=t.length,i=e.length;++n<r;)e[i+n]=t[n];return e}},function(e,t,n){var r=n(187);e.exports=function(e){var t=new e.constructor(e.byteLength);return new r(t).set(new r(e)),t}},function(e,t){e.exports=function(e){return function(){return e}}},function(e,t,n){var r=n(124),i=n(41);e.exports=function(e,t){return e&&r(e,t,i)}},function(e,t,n){var r=n(427)();e.exports=r},function(e,t){e.exports=function(e){var t=-1,n=Array(e.size);return e.forEach((function(e){n[++t]=e})),n}},function(e,t,n){function r(e){return(r="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}var i=n(15),o=n(55),a=/\.|\[(?:[^[\]]*|(["'])(?:(?!\1)[^\\]|\\.)*?\1)\]/,s=/^\w*$/;e.exports=function(e,t){if(i(e))return!1;var n=r(e);return!("number"!=n&&"symbol"!=n&&"boolean"!=n&&null!=e&&!o(e))||s.test(e)||!a.test(e)||null!=t&&e in Object(t)}},function(e,t,n){var r=n(120),i=n(461);e.exports=function e(t,n,o,a,s){var l=-1,u=t.length;for(o||(o=i),s||(s=[]);++l<u;){var c=t[l];n>0&&o(c)?n>1?e(c,n-1,o,a,s):r(s,c):a||(s[s.length]=c)}return s}},function(e,t,n){var r=n(55);e.exports=function(e,t,n){for(var i=-1,o=e.length;++i<o;){var a=e[i],s=t(a);if(null!=s&&(void 0===l?s==s&&!r(s):n(s,l)))var l=s,u=a}return u}},function(e,t,n){var r=n(227),i=["LR","RL","TB","BT","H","V"],o=["LR","RL","H"],a=i[0];e.exports=function(e,t,n){var s=t.direction||a;if(t.isHorizontal=function(e){return o.indexOf(e)>-1}(s),s&&-1===i.indexOf(s))throw new TypeError("Invalid direction: "+s);if(s===i[0])n(e,t);else if(s===i[1])n(e,t),e.right2left();else if(s===i[2])n(e,t);else if(s===i[3])n(e,t),e.bottom2top();else if(s===i[4]||s===i[5]){var l=r(e,t),u=l.left,c=l.right;n(u,t),n(c,t),t.isHorizontal?u.right2left():u.bottom2top(),c.translate(u.x-c.x,u.y-c.y),e.x=u.x,e.y=c.y;var d=e.getBoundingBox();t.isHorizontal?d.top<0&&e.translate(0,-d.top):d.left<0&&e.translate(-d.left,0)}var h=t.fixedRoot;return void 0===h&&(h=!0),h&&e.translate(-(e.x+e.width/2+e.hgap),-(e.y+e.height/2+e.vgap)),e}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(3);t.default=function(e,t,n){for(var i=0,o=r.default(t)?t.split("."):t;e&&i<o.length;)e=e[o[i++]];return void 0===e||i<o.length?n:e}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.forEach=t.sqrLen=t.len=t.sqrDist=t.dist=t.div=t.mul=t.sub=void 0,t.create=o,t.clone=function(e){var t=new i.ARRAY_TYPE(3);return t[0]=e[0],t[1]=e[1],t[2]=e[2],t},t.length=a,t.fromValues=s,t.copy=function(e,t){return e[0]=t[0],e[1]=t[1],e[2]=t[2],e},t.set=function(e,t,n,r){return e[0]=t,e[1]=n,e[2]=r,e},t.add=function(e,t,n){return e[0]=t[0]+n[0],e[1]=t[1]+n[1],e[2]=t[2]+n[2],e},t.subtract=l,t.multiply=u,t.divide=c,t.ceil=function(e,t){return e[0]=Math.ceil(t[0]),e[1]=Math.ceil(t[1]),e[2]=Math.ceil(t[2]),e},t.floor=function(e,t){return e[0]=Math.floor(t[0]),e[1]=Math.floor(t[1]),e[2]=Math.floor(t[2]),e},t.min=function(e,t,n){return e[0]=Math.min(t[0],n[0]),e[1]=Math.min(t[1],n[1]),e[2]=Math.min(t[2],n[2]),e},t.max=function(e,t,n){return e[0]=Math.max(t[0],n[0]),e[1]=Math.max(t[1],n[1]),e[2]=Math.max(t[2],n[2]),e},t.round=function(e,t){return e[0]=Math.round(t[0]),e[1]=Math.round(t[1]),e[2]=Math.round(t[2]),e},t.scale=function(e,t,n){return e[0]=t[0]*n,e[1]=t[1]*n,e[2]=t[2]*n,e},t.scaleAndAdd=function(e,t,n,r){return e[0]=t[0]+n[0]*r,e[1]=t[1]+n[1]*r,e[2]=t[2]+n[2]*r,e},t.distance=d,t.squaredDistance=h,t.squaredLength=f,t.negate=function(e,t){return e[0]=-t[0],e[1]=-t[1],e[2]=-t[2],e},t.inverse=function(e,t){return e[0]=1/t[0],e[1]=1/t[1],e[2]=1/t[2],e},t.normalize=p,t.dot=g,t.cross=function(e,t,n){var r=t[0],i=t[1],o=t[2],a=n[0],s=n[1],l=n[2];return e[0]=i*l-o*s,e[1]=o*a-r*l,e[2]=r*s-i*a,e},t.lerp=function(e,t,n,r){var i=t[0],o=t[1],a=t[2];return e[0]=i+r*(n[0]-i),e[1]=o+r*(n[1]-o),e[2]=a+r*(n[2]-a),e},t.hermite=function(e,t,n,r,i,o){var a=o*o,s=a*(2*o-3)+1,l=a*(o-2)+o,u=a*(o-1),c=a*(3-2*o);return e[0]=t[0]*s+n[0]*l+r[0]*u+i[0]*c,e[1]=t[1]*s+n[1]*l+r[1]*u+i[1]*c,e[2]=t[2]*s+n[2]*l+r[2]*u+i[2]*c,e},t.bezier=function(e,t,n,r,i,o){var a=1-o,s=a*a,l=o*o,u=s*a,c=3*o*s,d=3*l*a,h=l*o;return e[0]=t[0]*u+n[0]*c+r[0]*d+i[0]*h,e[1]=t[1]*u+n[1]*c+r[1]*d+i[1]*h,e[2]=t[2]*u+n[2]*c+r[2]*d+i[2]*h,e},t.random=function(e,t){t=t||1;var n=2*i.RANDOM()*Math.PI,r=2*i.RANDOM()-1,o=Math.sqrt(1-r*r)*t;return e[0]=Math.cos(n)*o,e[1]=Math.sin(n)*o,e[2]=r*t,e},t.transformMat4=function(e,t,n){var r=t[0],i=t[1],o=t[2],a=n[3]*r+n[7]*i+n[11]*o+n[15];return a=a||1,e[0]=(n[0]*r+n[4]*i+n[8]*o+n[12])/a,e[1]=(n[1]*r+n[5]*i+n[9]*o+n[13])/a,e[2]=(n[2]*r+n[6]*i+n[10]*o+n[14])/a,e},t.transformMat3=function(e,t,n){var r=t[0],i=t[1],o=t[2];return e[0]=r*n[0]+i*n[3]+o*n[6],e[1]=r*n[1]+i*n[4]+o*n[7],e[2]=r*n[2]+i*n[5]+o*n[8],e},t.transformQuat=function(e,t,n){var r=n[0],i=n[1],o=n[2],a=n[3],s=t[0],l=t[1],u=t[2],c=i*u-o*l,d=o*s-r*u,h=r*l-i*s,f=i*h-o*d,p=o*c-r*h,g=r*d-i*c,v=2*a;return c*=v,d*=v,h*=v,f*=2,p*=2,g*=2,e[0]=s+c+f,e[1]=l+d+p,e[2]=u+h+g,e},t.rotateX=function(e,t,n,r){var i=[],o=[];return i[0]=t[0]-n[0],i[1]=t[1]-n[1],i[2]=t[2]-n[2],o[0]=i[0],o[1]=i[1]*Math.cos(r)-i[2]*Math.sin(r),o[2]=i[1]*Math.sin(r)+i[2]*Math.cos(r),e[0]=o[0]+n[0],e[1]=o[1]+n[1],e[2]=o[2]+n[2],e},t.rotateY=function(e,t,n,r){var i=[],o=[];return i[0]=t[0]-n[0],i[1]=t[1]-n[1],i[2]=t[2]-n[2],o[0]=i[2]*Math.sin(r)+i[0]*Math.cos(r),o[1]=i[1],o[2]=i[2]*Math.cos(r)-i[0]*Math.sin(r),e[0]=o[0]+n[0],e[1]=o[1]+n[1],e[2]=o[2]+n[2],e},t.rotateZ=function(e,t,n,r){var i=[],o=[];return i[0]=t[0]-n[0],i[1]=t[1]-n[1],i[2]=t[2]-n[2],o[0]=i[0]*Math.cos(r)-i[1]*Math.sin(r),o[1]=i[0]*Math.sin(r)+i[1]*Math.cos(r),o[2]=i[2],e[0]=o[0]+n[0],e[1]=o[1]+n[1],e[2]=o[2]+n[2],e},t.angle=function(e,t){var n=s(e[0],e[1],e[2]),r=s(t[0],t[1],t[2]);p(n,n),p(r,r);var i=g(n,r);return i>1?0:i<-1?Math.PI:Math.acos(i)},t.str=function(e){return"vec3("+e[0]+", "+e[1]+", "+e[2]+")"},t.exactEquals=function(e,t){return e[0]===t[0]&&e[1]===t[1]&&e[2]===t[2]},t.equals=function(e,t){var n=e[0],r=e[1],o=e[2],a=t[0],s=t[1],l=t[2];return Math.abs(n-a)<=i.EPSILON*Math.max(1,Math.abs(n),Math.abs(a))&&Math.abs(r-s)<=i.EPSILON*Math.max(1,Math.abs(r),Math.abs(s))&&Math.abs(o-l)<=i.EPSILON*Math.max(1,Math.abs(o),Math.abs(l))};var r,i=function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)Object.prototype.hasOwnProperty.call(e,n)&&(t[n]=e[n]);return t.default=e,t}(n(104));function o(){var e=new i.ARRAY_TYPE(3);return i.ARRAY_TYPE!=Float32Array&&(e[0]=0,e[1]=0,e[2]=0),e}function a(e){var t=e[0],n=e[1],r=e[2];return Math.sqrt(t*t+n*n+r*r)}function s(e,t,n){var r=new i.ARRAY_TYPE(3);return r[0]=e,r[1]=t,r[2]=n,r}function l(e,t,n){return e[0]=t[0]-n[0],e[1]=t[1]-n[1],e[2]=t[2]-n[2],e}function u(e,t,n){return e[0]=t[0]*n[0],e[1]=t[1]*n[1],e[2]=t[2]*n[2],e}function c(e,t,n){return e[0]=t[0]/n[0],e[1]=t[1]/n[1],e[2]=t[2]/n[2],e}function d(e,t){var n=t[0]-e[0],r=t[1]-e[1],i=t[2]-e[2];return Math.sqrt(n*n+r*r+i*i)}function h(e,t){var n=t[0]-e[0],r=t[1]-e[1],i=t[2]-e[2];return n*n+r*r+i*i}function f(e){var t=e[0],n=e[1],r=e[2];return t*t+n*n+r*r}function p(e,t){var n=t[0],r=t[1],i=t[2],o=n*n+r*r+i*i;return o>0&&(o=1/Math.sqrt(o),e[0]=t[0]*o,e[1]=t[1]*o,e[2]=t[2]*o),e}function g(e,t){return e[0]*t[0]+e[1]*t[1]+e[2]*t[2]}t.sub=l,t.mul=u,t.div=c,t.dist=d,t.sqrDist=h,t.len=a,t.sqrLen=f,t.forEach=(r=o(),function(e,t,n,i,o,a){var s,l=void 0;for(t||(t=3),n||(n=0),s=i?Math.min(i*t+n,e.length):e.length,l=n;l<s;l+=t)r[0]=e[l],r[1]=e[l+1],r[2]=e[l+2],o(r,r,a),e[l]=r[0],e[l+1]=r[1],e[l+2]=r[2];return e})},function(e,t,n){"use strict";n.r(t),function(e){n.d(t,"BrowserInfo",(function(){return r})),n.d(t,"NodeInfo",(function(){return i})),n.d(t,"SearchBotDeviceInfo",(function(){return o})),n.d(t,"BotInfo",(function(){return a})),n.d(t,"ReactNativeInfo",(function(){return s})),n.d(t,"detect",(function(){return d})),n.d(t,"browserName",(function(){return f})),n.d(t,"parseUserAgent",(function(){return p})),n.d(t,"detectOS",(function(){return g})),n.d(t,"getNodeVersion",(function(){return v}));var r=function(e,t,n){this.name=e,this.version=t,this.os=n,this.type="browser"},i=function(t){this.version=t,this.type="node",this.name="node",this.os=e.platform},o=function(e,t,n,r){this.name=e,this.version=t,this.os=n,this.bot=r,this.type="bot-device"},a=function(){this.type="bot",this.bot=!0,this.name="bot",this.version=null,this.os=null},s=function(){this.type="react-native",this.name="react-native",this.version=null,this.os=null},l=/(nuhk|Googlebot|Yammybot|Openbot|Slurp|MSNBot|Ask\ Jeeves\/Teoma|ia_archiver)/,u=[["aol",/AOLShield\/([0-9\._]+)/],["edge",/Edge\/([0-9\._]+)/],["edge-ios",/EdgiOS\/([0-9\._]+)/],["yandexbrowser",/YaBrowser\/([0-9\._]+)/],["kakaotalk",/KAKAOTALK\s([0-9\.]+)/],["samsung",/SamsungBrowser\/([0-9\.]+)/],["silk",/\bSilk\/([0-9._-]+)\b/],["miui",/MiuiBrowser\/([0-9\.]+)$/],["beaker",/BeakerBrowser\/([0-9\.]+)/],["edge-chromium",/EdgA?\/([0-9\.]+)/],["chromium-webview",/(?!Chrom.*OPR)wv\).*Chrom(?:e|ium)\/([0-9\.]+)(:?\s|$)/],["chrome",/(?!Chrom.*OPR)Chrom(?:e|ium)\/([0-9\.]+)(:?\s|$)/],["phantomjs",/PhantomJS\/([0-9\.]+)(:?\s|$)/],["crios",/CriOS\/([0-9\.]+)(:?\s|$)/],["firefox",/Firefox\/([0-9\.]+)(?:\s|$)/],["fxios",/FxiOS\/([0-9\.]+)/],["opera-mini",/Opera Mini.*Version\/([0-9\.]+)/],["opera",/Opera\/([0-9\.]+)(?:\s|$)/],["opera",/OPR\/([0-9\.]+)(:?\s|$)/],["ie",/Trident\/7\.0.*rv\:([0-9\.]+).*\).*Gecko$/],["ie",/MSIE\s([0-9\.]+);.*Trident\/[4-7].0/],["ie",/MSIE\s(7\.0)/],["bb10",/BB10;\sTouch.*Version\/([0-9\.]+)/],["android",/Android\s([0-9\.]+)/],["ios",/Version\/([0-9\._]+).*Mobile.*Safari.*/],["safari",/Version\/([0-9\._]+).*Safari/],["facebook",/FBAV\/([0-9\.]+)/],["instagram",/Instagram\s([0-9\.]+)/],["ios-webview",/AppleWebKit\/([0-9\.]+).*Mobile/],["ios-webview",/AppleWebKit\/([0-9\.]+).*Gecko\)$/],["searchbot",/alexa|bot|crawl(er|ing)|facebookexternalhit|feedburner|google web preview|nagios|postrank|pingdom|slurp|spider|yahoo!|yandex/]],c=[["iOS",/iP(hone|od|ad)/],["Android OS",/Android/],["BlackBerry OS",/BlackBerry|BB10/],["Windows Mobile",/IEMobile/],["Amazon OS",/Kindle/],["Windows 3.11",/Win16/],["Windows 95",/(Windows 95)|(Win95)|(Windows_95)/],["Windows 98",/(Windows 98)|(Win98)/],["Windows 2000",/(Windows NT 5.0)|(Windows 2000)/],["Windows XP",/(Windows NT 5.1)|(Windows XP)/],["Windows Server 2003",/(Windows NT 5.2)/],["Windows Vista",/(Windows NT 6.0)/],["Windows 7",/(Windows NT 6.1)/],["Windows 8",/(Windows NT 6.2)/],["Windows 8.1",/(Windows NT 6.3)/],["Windows 10",/(Windows NT 10.0)/],["Windows ME",/Windows ME/],["Open BSD",/OpenBSD/],["Sun OS",/SunOS/],["Chrome OS",/CrOS/],["Linux",/(Linux)|(X11)/],["Mac OS",/(Mac_PowerPC)|(Macintosh)/],["QNX",/QNX/],["BeOS",/BeOS/],["OS/2",/OS\/2/]];function d(e){return e?p(e):"undefined"==typeof document&&"undefined"!=typeof navigator&&"ReactNative"===navigator.product?new s:"undefined"!=typeof navigator?p(navigator.userAgent):v()}function h(e){return""!==e&&u.reduce((function(t,n){var r=n[0],i=n[1];if(t)return t;var o=i.exec(e);return!!o&&[r,o]}),!1)}function f(e){var t=h(e);return t?t[0]:null}function p(e){var t=h(e);if(!t)return null;var n=t[0],i=t[1];if("searchbot"===n)return new a;var s=i[1]&&i[1].split(/[._]/).slice(0,3);s?s.length<3&&(s=function(){for(var e=0,t=0,n=arguments.length;t<n;t++)e+=arguments[t].length;var r=Array(e),i=0;for(t=0;t<n;t++)for(var o=arguments[t],a=0,s=o.length;a<s;a++,i++)r[i]=o[a];return r}(s,function(e){for(var t=[],n=0;n<e;n++)t.push("0");return t}(3-s.length))):s=[];var u=s.join("."),c=g(e),d=l.exec(e);return d&&d[1]?new o(n,u,c,d[1]):new r(n,u,c)}function g(e){for(var t=0,n=c.length;t<n;t++){var r=c[t],i=r[0];if(r[1].exec(e))return i}return null}function v(){return void 0!==e&&e.version?new i(e.version.slice(1)):null}}.call(this,n(152))},function(e,t,n){"use strict";n.r(t),n.d(t,"mat3",(function(){return r})),n.d(t,"vec2",(function(){return z})),n.d(t,"vec3",(function(){return F})),n.d(t,"ext",(function(){return i}));var r={};n.r(r),n.d(r,"create",(function(){return a})),n.d(r,"fromMat4",(function(){return s})),n.d(r,"clone",(function(){return l})),n.d(r,"copy",(function(){return u})),n.d(r,"fromValues",(function(){return c})),n.d(r,"set",(function(){return d})),n.d(r,"identity",(function(){return h})),n.d(r,"transpose",(function(){return f})),n.d(r,"invert",(function(){return p})),n.d(r,"adjoint",(function(){return g})),n.d(r,"determinant",(function(){return v})),n.d(r,"multiply",(function(){return m})),n.d(r,"translate",(function(){return y})),n.d(r,"rotate",(function(){return b})),n.d(r,"scale",(function(){return C})),n.d(r,"fromTranslation",(function(){return A})),n.d(r,"fromRotation",(function(){return _})),n.d(r,"fromScaling",(function(){return I})),n.d(r,"fromMat2d",(function(){return x})),n.d(r,"fromQuat",(function(){return w})),n.d(r,"normalFromMat4",(function(){return M})),n.d(r,"projection",(function(){return S})),n.d(r,"str",(function(){return T})),n.d(r,"frob",(function(){return E})),n.d(r,"add",(function(){return O})),n.d(r,"subtract",(function(){return k})),n.d(r,"multiplyScalar",(function(){return L})),n.d(r,"multiplyScalarAndAdd",(function(){return D})),n.d(r,"exactEquals",(function(){return P})),n.d(r,"equals",(function(){return N})),n.d(r,"mul",(function(){return j})),n.d(r,"sub",(function(){return R}));var i={};n.r(i),n.d(i,"leftTranslate",(function(){return B})),n.d(i,"leftRotate",(function(){return H})),n.d(i,"leftScale",(function(){return U})),n.d(i,"transform",(function(){return V})),n.d(i,"direction",(function(){return Y})),n.d(i,"angleTo",(function(){return W})),n.d(i,"vertical",(function(){return G}));var o=n(6);function a(){var e=new o.a(9);return o.a!=Float32Array&&(e[1]=0,e[2]=0,e[3]=0,e[5]=0,e[6]=0,e[7]=0),e[0]=1,e[4]=1,e[8]=1,e}function s(e,t){return e[0]=t[0],e[1]=t[1],e[2]=t[2],e[3]=t[4],e[4]=t[5],e[5]=t[6],e[6]=t[8],e[7]=t[9],e[8]=t[10],e}function l(e){var t=new o.a(9);return t[0]=e[0],t[1]=e[1],t[2]=e[2],t[3]=e[3],t[4]=e[4],t[5]=e[5],t[6]=e[6],t[7]=e[7],t[8]=e[8],t}function u(e,t){return e[0]=t[0],e[1]=t[1],e[2]=t[2],e[3]=t[3],e[4]=t[4],e[5]=t[5],e[6]=t[6],e[7]=t[7],e[8]=t[8],e}function c(e,t,n,r,i,a,s,l,u){var c=new o.a(9);return c[0]=e,c[1]=t,c[2]=n,c[3]=r,c[4]=i,c[5]=a,c[6]=s,c[7]=l,c[8]=u,c}function d(e,t,n,r,i,o,a,s,l,u){return e[0]=t,e[1]=n,e[2]=r,e[3]=i,e[4]=o,e[5]=a,e[6]=s,e[7]=l,e[8]=u,e}function h(e){return e[0]=1,e[1]=0,e[2]=0,e[3]=0,e[4]=1,e[5]=0,e[6]=0,e[7]=0,e[8]=1,e}function f(e,t){if(e===t){var n=t[1],r=t[2],i=t[5];e[1]=t[3],e[2]=t[6],e[3]=n,e[5]=t[7],e[6]=r,e[7]=i}else e[0]=t[0],e[1]=t[3],e[2]=t[6],e[3]=t[1],e[4]=t[4],e[5]=t[7],e[6]=t[2],e[7]=t[5],e[8]=t[8];return e}function p(e,t){var n=t[0],r=t[1],i=t[2],o=t[3],a=t[4],s=t[5],l=t[6],u=t[7],c=t[8],d=c*a-s*u,h=-c*o+s*l,f=u*o-a*l,p=n*d+r*h+i*f;return p?(p=1/p,e[0]=d*p,e[1]=(-c*r+i*u)*p,e[2]=(s*r-i*a)*p,e[3]=h*p,e[4]=(c*n-i*l)*p,e[5]=(-s*n+i*o)*p,e[6]=f*p,e[7]=(-u*n+r*l)*p,e[8]=(a*n-r*o)*p,e):null}function g(e,t){var n=t[0],r=t[1],i=t[2],o=t[3],a=t[4],s=t[5],l=t[6],u=t[7],c=t[8];return e[0]=a*c-s*u,e[1]=i*u-r*c,e[2]=r*s-i*a,e[3]=s*l-o*c,e[4]=n*c-i*l,e[5]=i*o-n*s,e[6]=o*u-a*l,e[7]=r*l-n*u,e[8]=n*a-r*o,e}function v(e){var t=e[0],n=e[1],r=e[2],i=e[3],o=e[4],a=e[5],s=e[6],l=e[7],u=e[8];return t*(u*o-a*l)+n*(-u*i+a*s)+r*(l*i-o*s)}function m(e,t,n){var r=t[0],i=t[1],o=t[2],a=t[3],s=t[4],l=t[5],u=t[6],c=t[7],d=t[8],h=n[0],f=n[1],p=n[2],g=n[3],v=n[4],m=n[5],y=n[6],b=n[7],C=n[8];return e[0]=h*r+f*a+p*u,e[1]=h*i+f*s+p*c,e[2]=h*o+f*l+p*d,e[3]=g*r+v*a+m*u,e[4]=g*i+v*s+m*c,e[5]=g*o+v*l+m*d,e[6]=y*r+b*a+C*u,e[7]=y*i+b*s+C*c,e[8]=y*o+b*l+C*d,e}function y(e,t,n){var r=t[0],i=t[1],o=t[2],a=t[3],s=t[4],l=t[5],u=t[6],c=t[7],d=t[8],h=n[0],f=n[1];return e[0]=r,e[1]=i,e[2]=o,e[3]=a,e[4]=s,e[5]=l,e[6]=h*r+f*a+u,e[7]=h*i+f*s+c,e[8]=h*o+f*l+d,e}function b(e,t,n){var r=t[0],i=t[1],o=t[2],a=t[3],s=t[4],l=t[5],u=t[6],c=t[7],d=t[8],h=Math.sin(n),f=Math.cos(n);return e[0]=f*r+h*a,e[1]=f*i+h*s,e[2]=f*o+h*l,e[3]=f*a-h*r,e[4]=f*s-h*i,e[5]=f*l-h*o,e[6]=u,e[7]=c,e[8]=d,e}function C(e,t,n){var r=n[0],i=n[1];return e[0]=r*t[0],e[1]=r*t[1],e[2]=r*t[2],e[3]=i*t[3],e[4]=i*t[4],e[5]=i*t[5],e[6]=t[6],e[7]=t[7],e[8]=t[8],e}function A(e,t){return e[0]=1,e[1]=0,e[2]=0,e[3]=0,e[4]=1,e[5]=0,e[6]=t[0],e[7]=t[1],e[8]=1,e}function _(e,t){var n=Math.sin(t),r=Math.cos(t);return e[0]=r,e[1]=n,e[2]=0,e[3]=-n,e[4]=r,e[5]=0,e[6]=0,e[7]=0,e[8]=1,e}function I(e,t){return e[0]=t[0],e[1]=0,e[2]=0,e[3]=0,e[4]=t[1],e[5]=0,e[6]=0,e[7]=0,e[8]=1,e}function x(e,t){return e[0]=t[0],e[1]=t[1],e[2]=0,e[3]=t[2],e[4]=t[3],e[5]=0,e[6]=t[4],e[7]=t[5],e[8]=1,e}function w(e,t){var n=t[0],r=t[1],i=t[2],o=t[3],a=n+n,s=r+r,l=i+i,u=n*a,c=r*a,d=r*s,h=i*a,f=i*s,p=i*l,g=o*a,v=o*s,m=o*l;return e[0]=1-d-p,e[3]=c-m,e[6]=h+v,e[1]=c+m,e[4]=1-u-p,e[7]=f-g,e[2]=h-v,e[5]=f+g,e[8]=1-u-d,e}function M(e,t){var n=t[0],r=t[1],i=t[2],o=t[3],a=t[4],s=t[5],l=t[6],u=t[7],c=t[8],d=t[9],h=t[10],f=t[11],p=t[12],g=t[13],v=t[14],m=t[15],y=n*s-r*a,b=n*l-i*a,C=n*u-o*a,A=r*l-i*s,_=r*u-o*s,I=i*u-o*l,x=c*g-d*p,w=c*v-h*p,M=c*m-f*p,S=d*v-h*g,T=d*m-f*g,E=h*m-f*v,O=y*E-b*T+C*S+A*M-_*w+I*x;return O?(O=1/O,e[0]=(s*E-l*T+u*S)*O,e[1]=(l*M-a*E-u*w)*O,e[2]=(a*T-s*M+u*x)*O,e[3]=(i*T-r*E-o*S)*O,e[4]=(n*E-i*M+o*w)*O,e[5]=(r*M-n*T-o*x)*O,e[6]=(g*I-v*_+m*A)*O,e[7]=(v*C-p*I-m*b)*O,e[8]=(p*_-g*C+m*y)*O,e):null}function S(e,t,n){return e[0]=2/t,e[1]=0,e[2]=0,e[3]=0,e[4]=-2/n,e[5]=0,e[6]=-1,e[7]=1,e[8]=1,e}function T(e){return"mat3("+e[0]+", "+e[1]+", "+e[2]+", "+e[3]+", "+e[4]+", "+e[5]+", "+e[6]+", "+e[7]+", "+e[8]+")"}function E(e){return Math.hypot(e[0],e[1],e[2],e[3],e[4],e[5],e[6],e[7],e[8])}function O(e,t,n){return e[0]=t[0]+n[0],e[1]=t[1]+n[1],e[2]=t[2]+n[2],e[3]=t[3]+n[3],e[4]=t[4]+n[4],e[5]=t[5]+n[5],e[6]=t[6]+n[6],e[7]=t[7]+n[7],e[8]=t[8]+n[8],e}function k(e,t,n){return e[0]=t[0]-n[0],e[1]=t[1]-n[1],e[2]=t[2]-n[2],e[3]=t[3]-n[3],e[4]=t[4]-n[4],e[5]=t[5]-n[5],e[6]=t[6]-n[6],e[7]=t[7]-n[7],e[8]=t[8]-n[8],e}function L(e,t,n){return e[0]=t[0]*n,e[1]=t[1]*n,e[2]=t[2]*n,e[3]=t[3]*n,e[4]=t[4]*n,e[5]=t[5]*n,e[6]=t[6]*n,e[7]=t[7]*n,e[8]=t[8]*n,e}function D(e,t,n,r){return e[0]=t[0]+n[0]*r,e[1]=t[1]+n[1]*r,e[2]=t[2]+n[2]*r,e[3]=t[3]+n[3]*r,e[4]=t[4]+n[4]*r,e[5]=t[5]+n[5]*r,e[6]=t[6]+n[6]*r,e[7]=t[7]+n[7]*r,e[8]=t[8]+n[8]*r,e}function P(e,t){return e[0]===t[0]&&e[1]===t[1]&&e[2]===t[2]&&e[3]===t[3]&&e[4]===t[4]&&e[5]===t[5]&&e[6]===t[6]&&e[7]===t[7]&&e[8]===t[8]}function N(e,t){var n=e[0],r=e[1],i=e[2],a=e[3],s=e[4],l=e[5],u=e[6],c=e[7],d=e[8],h=t[0],f=t[1],p=t[2],g=t[3],v=t[4],m=t[5],y=t[6],b=t[7],C=t[8];return Math.abs(n-h)<=o.b*Math.max(1,Math.abs(n),Math.abs(h))&&Math.abs(r-f)<=o.b*Math.max(1,Math.abs(r),Math.abs(f))&&Math.abs(i-p)<=o.b*Math.max(1,Math.abs(i),Math.abs(p))&&Math.abs(a-g)<=o.b*Math.max(1,Math.abs(a),Math.abs(g))&&Math.abs(s-v)<=o.b*Math.max(1,Math.abs(s),Math.abs(v))&&Math.abs(l-m)<=o.b*Math.max(1,Math.abs(l),Math.abs(m))&&Math.abs(u-y)<=o.b*Math.max(1,Math.abs(u),Math.abs(y))&&Math.abs(c-b)<=o.b*Math.max(1,Math.abs(c),Math.abs(b))&&Math.abs(d-C)<=o.b*Math.max(1,Math.abs(d),Math.abs(C))}var j=m,R=k,z=n(105),F=n(106);function B(e,t,n){var i=[0,0,0,0,0,0,0,0,0];return r.fromTranslation(i,n),r.multiply(e,i,t)}function H(e,t,n){var i=[0,0,0,0,0,0,0,0,0];return r.fromRotation(i,n),r.multiply(e,i,t)}function U(e,t,n){var i=[0,0,0,0,0,0,0,0,0];return r.fromScaling(i,n),r.multiply(e,i,t)}function V(e,t){for(var n,i,o,a=e?[].concat(e):[1,0,0,0,1,0,0,0,1],s=0,l=t.length;s<l;s++){var u=t[s];switch(u[0]){case"t":B(a,a,[u[1],u[2]]);break;case"s":U(a,a,[u[1],u[2]]);break;case"r":H(a,a,u[1]);break;case"m":n=a,i=a,o=u[1],r.multiply(n,o,i)}}return a}function Y(e,t){return e[0]*t[1]-t[0]*e[1]}function W(e,t,n){var r=z.angle(e,t),i=Y(e,t)>=0;return n?i?2*Math.PI-r:r:i?r:2*Math.PI-r}function G(e,t,n){return n?(e[0]=t[1],e[1]=-1*t[0]):(e[0]=-1*t[1],e[1]=t[0]),e}},function(e,t,n){e.exports={graphlib:n(30),layout:n(485),debug:n(544),util:{time:n(20).time,notime:n(20).notime},version:n(545)}},function(e,t,n){var r={compactBox:n(546),dendrogram:n(548),indented:n(550),mindmap:n(552)};e.exports=r},function(e,t,n){(function(e){function r(e){return(r="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}var i=Object.getOwnPropertyDescriptors||function(e){for(var t=Object.keys(e),n={},r=0;r<t.length;r++)n[t[r]]=Object.getOwnPropertyDescriptor(e,t[r]);return n},o=/%[sdj%]/g;t.format=function(e){if(!y(e)){for(var t=[],n=0;n<arguments.length;n++)t.push(l(arguments[n]));return t.join(" ")}n=1;for(var r=arguments,i=r.length,a=String(e).replace(o,(function(e){if("%%"===e)return"%";if(n>=i)return e;switch(e){case"%s":return String(r[n++]);case"%d":return Number(r[n++]);case"%j":try{return JSON.stringify(r[n++])}catch(e){return"[Circular]"}default:return e}})),s=r[n];n<i;s=r[++n])v(s)||!A(s)?a+=" "+s:a+=" "+l(s);return a},t.deprecate=function(n,r){if(void 0!==e&&!0===e.noDeprecation)return n;if(void 0===e)return function(){return t.deprecate(n,r).apply(this,arguments)};var i=!1;return function(){if(!i){if(e.throwDeprecation)throw new Error(r);e.traceDeprecation?console.trace(r):console.error(r),i=!0}return n.apply(this,arguments)}};var a,s={};function l(e,n){var r={seen:[],stylize:c};return arguments.length>=3&&(r.depth=arguments[2]),arguments.length>=4&&(r.colors=arguments[3]),g(n)?r.showHidden=n:n&&t._extend(r,n),b(r.showHidden)&&(r.showHidden=!1),b(r.depth)&&(r.depth=2),b(r.colors)&&(r.colors=!1),b(r.customInspect)&&(r.customInspect=!0),r.colors&&(r.stylize=u),d(r,e,r.depth)}function u(e,t){var n=l.styles[t];return n?"["+l.colors[n][0]+"m"+e+"["+l.colors[n][1]+"m":e}function c(e,t){return e}function d(e,n,r){if(e.customInspect&&n&&x(n.inspect)&&n.inspect!==t.inspect&&(!n.constructor||n.constructor.prototype!==n)){var i=n.inspect(r,e);return y(i)||(i=d(e,i,r)),i}var o=function(e,t){if(b(t))return e.stylize("undefined","undefined");if(y(t)){var n="'"+JSON.stringify(t).replace(/^"|"$/g,"").replace(/'/g,"\\'").replace(/\\"/g,'"')+"'";return e.stylize(n,"string")}return m(t)?e.stylize(""+t,"number"):g(t)?e.stylize(""+t,"boolean"):v(t)?e.stylize("null","null"):void 0}(e,n);if(o)return o;var a=Object.keys(n),s=function(e){var t={};return e.forEach((function(e,n){t[e]=!0})),t}(a);if(e.showHidden&&(a=Object.getOwnPropertyNames(n)),I(n)&&(a.indexOf("message")>=0||a.indexOf("description")>=0))return h(n);if(0===a.length){if(x(n)){var l=n.name?": "+n.name:"";return e.stylize("[Function"+l+"]","special")}if(C(n))return e.stylize(RegExp.prototype.toString.call(n),"regexp");if(_(n))return e.stylize(Date.prototype.toString.call(n),"date");if(I(n))return h(n)}var u,c="",A=!1,w=["{","}"];return p(n)&&(A=!0,w=["[","]"]),x(n)&&(c=" [Function"+(n.name?": "+n.name:"")+"]"),C(n)&&(c=" "+RegExp.prototype.toString.call(n)),_(n)&&(c=" "+Date.prototype.toUTCString.call(n)),I(n)&&(c=" "+h(n)),0!==a.length||A&&0!=n.length?r<0?C(n)?e.stylize(RegExp.prototype.toString.call(n),"regexp"):e.stylize("[Object]","special"):(e.seen.push(n),u=A?function(e,t,n,r,i){for(var o=[],a=0,s=t.length;a<s;++a)E(t,String(a))?o.push(f(e,t,n,r,String(a),!0)):o.push("");return i.forEach((function(i){i.match(/^\d+$/)||o.push(f(e,t,n,r,i,!0))})),o}(e,n,r,s,a):a.map((function(t){return f(e,n,r,s,t,A)})),e.seen.pop(),function(e,t,n){return e.reduce((function(e,t){return t.indexOf("\n"),e+t.replace(/\u001b\[\d\d?m/g,"").length+1}),0)>60?n[0]+(""===t?"":t+"\n ")+" "+e.join(",\n  ")+" "+n[1]:n[0]+t+" "+e.join(", ")+" "+n[1]}(u,c,w)):w[0]+c+w[1]}function h(e){return"["+Error.prototype.toString.call(e)+"]"}function f(e,t,n,r,i,o){var a,s,l;if((l=Object.getOwnPropertyDescriptor(t,i)||{value:t[i]}).get?s=l.set?e.stylize("[Getter/Setter]","special"):e.stylize("[Getter]","special"):l.set&&(s=e.stylize("[Setter]","special")),E(r,i)||(a="["+i+"]"),s||(e.seen.indexOf(l.value)<0?(s=v(n)?d(e,l.value,null):d(e,l.value,n-1)).indexOf("\n")>-1&&(s=o?s.split("\n").map((function(e){return"  "+e})).join("\n").substr(2):"\n"+s.split("\n").map((function(e){return"   "+e})).join("\n")):s=e.stylize("[Circular]","special")),b(a)){if(o&&i.match(/^\d+$/))return s;(a=JSON.stringify(""+i)).match(/^"([a-zA-Z_][a-zA-Z_0-9]*)"$/)?(a=a.substr(1,a.length-2),a=e.stylize(a,"name")):(a=a.replace(/'/g,"\\'").replace(/\\"/g,'"').replace(/(^"|"$)/g,"'"),a=e.stylize(a,"string"))}return a+": "+s}function p(e){return Array.isArray(e)}function g(e){return"boolean"==typeof e}function v(e){return null===e}function m(e){return"number"==typeof e}function y(e){return"string"==typeof e}function b(e){return void 0===e}function C(e){return A(e)&&"[object RegExp]"===w(e)}function A(e){return"object"===r(e)&&null!==e}function _(e){return A(e)&&"[object Date]"===w(e)}function I(e){return A(e)&&("[object Error]"===w(e)||e instanceof Error)}function x(e){return"function"==typeof e}function w(e){return Object.prototype.toString.call(e)}function M(e){return e<10?"0"+e.toString(10):e.toString(10)}t.debuglog=function(n){if(b(a)&&(a=e.env.NODE_DEBUG||""),n=n.toUpperCase(),!s[n])if(new RegExp("\\b"+n+"\\b","i").test(a)){var r=e.pid;s[n]=function(){var e=t.format.apply(t,arguments);console.error("%s %d: %s",n,r,e)}}else s[n]=function(){};return s[n]},t.inspect=l,l.colors={bold:[1,22],italic:[3,23],underline:[4,24],inverse:[7,27],white:[37,39],grey:[90,39],black:[30,39],blue:[34,39],cyan:[36,39],green:[32,39],magenta:[35,39],red:[31,39],yellow:[33,39]},l.styles={special:"cyan",number:"yellow",boolean:"yellow",undefined:"grey",null:"bold",string:"green",date:"magenta",regexp:"red"},t.isArray=p,t.isBoolean=g,t.isNull=v,t.isNullOrUndefined=function(e){return null==e},t.isNumber=m,t.isString=y,t.isSymbol=function(e){return"symbol"===r(e)},t.isUndefined=b,t.isRegExp=C,t.isObject=A,t.isDate=_,t.isError=I,t.isFunction=x,t.isPrimitive=function(e){return null===e||"boolean"==typeof e||"number"==typeof e||"string"==typeof e||"symbol"===r(e)||void 0===e},t.isBuffer=n(554);var S=["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"];function T(){var e=new Date,t=[M(e.getHours()),M(e.getMinutes()),M(e.getSeconds())].join(":");return[e.getDate(),S[e.getMonth()],t].join(" ")}function E(e,t){return Object.prototype.hasOwnProperty.call(e,t)}t.log=function(){console.log("%s - %s",T(),t.format.apply(t,arguments))},t.inherits=n(555),t._extend=function(e,t){if(!t||!A(t))return e;for(var n=Object.keys(t),r=n.length;r--;)e[n[r]]=t[n[r]];return e};var O="undefined"!=typeof Symbol?Symbol("util.promisify.custom"):void 0;function k(e,t){if(!e){var n=new Error("Promise was rejected with a falsy value");n.reason=e,e=n}return t(e)}t.promisify=function(e){if("function"!=typeof e)throw new TypeError('The "original" argument must be of type Function');if(O&&e[O]){var t;if("function"!=typeof(t=e[O]))throw new TypeError('The "util.promisify.custom" argument must be of type Function');return Object.defineProperty(t,O,{value:t,enumerable:!1,writable:!1,configurable:!0}),t}function t(){for(var t,n,r=new Promise((function(e,r){t=e,n=r})),i=[],o=0;o<arguments.length;o++)i.push(arguments[o]);i.push((function(e,r){e?n(e):t(r)}));try{e.apply(this,i)}catch(e){n(e)}return r}return Object.setPrototypeOf(t,Object.getPrototypeOf(e)),O&&Object.defineProperty(t,O,{value:t,enumerable:!1,writable:!1,configurable:!0}),Object.defineProperties(t,i(e))},t.promisify.custom=O,t.callbackify=function(t){if("function"!=typeof t)throw new TypeError('The "original" argument must be of type Function');function n(){for(var n=[],r=0;r<arguments.length;r++)n.push(arguments[r]);var i=n.pop();if("function"!=typeof i)throw new TypeError("The last argument must be of type Function");var o=this,a=function(){return i.apply(o,arguments)};t.apply(this,n).then((function(t){e.nextTick(a,null,t)}),(function(t){e.nextTick(k,t,a)}))}return Object.setPrototypeOf(n,Object.getPrototypeOf(t)),Object.defineProperties(n,i(t)),n}}).call(this,n(152))},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(21);t.default=function(e,t){if(!r.default(e))return e;for(var n=[],i=0;i<e.length;i++){var o=e[i];t(o,i)&&n.push(o)}return n}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(10),i=n(139);t.default=function(e,t){var n=i.default(t),o=n.length;if(r.default(e))return!o;for(var a=0;a<o;a+=1){var s=n[a];if(t[s]!==e[s]||!(s in e))return!1}return!0}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(2),i=n(14),o=Object.keys?function(e){return Object.keys(e)}:function(e){var t=[];return r.default(e,(function(n,r){i.default(e)&&"prototype"===r||t.push(r)})),t};t.default=o},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(21),i=Array.prototype.splice;t.default=function(e,t){if(!r.default(e))return[];for(var n=e?t.length:0,o=n-1;n--;){var a=void 0,s=t[n];n!==o&&s===a||(a=s,i.call(e,s,1))}return e}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(72),i=n(2);t.default=function(e){var t=[];return i.default(e,(function(e){r.default(t,e)||t.push(e)})),t}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(4),i=n(14),o=n(69);t.default=function(e,t){if(!t)return{0:e};if(!i.default(t)){var n=r.default(t)?t:t.replace(/\s+/g,"").split("*");t=function(e){for(var t="_",r=0,i=n.length;r<i;r++)t+=e[n[r]]&&e[n[r]].toString();return t}}return o.default(e,t)}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){return e.hasOwnProperty(t)}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(2),i=n(14),o=Object.values?function(e){return Object.values(e)}:function(e){var t=[];return r.default(e,(function(n,r){i.default(e)&&"prototype"===r||t.push(n)})),t};t.default=o},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r={}.toString;t.default=function(e){return r.call(e).replace(/^\[object /,"").replace(/]$/,"")}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=Object.prototype;t.default=function(e){var t=e&&e.constructor;return e===("function"==typeof t&&t.prototype||r)}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){return(window.requestAnimationFrame||window.webkitRequestAnimationFrame||window.mozRequestAnimationFrame||window.msRequestAnimationFrame||function(e){return setTimeout(e,16)})(e)}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){(window.cancelAnimationFrame||window.webkitCancelAnimationFrame||window.mozCancelAnimationFrame||window.msCancelAnimationFrame||clearTimeout)(e)}},function(e,t,n){var r=n(316),i={};for(var o in r)r.hasOwnProperty(o)&&(i[r[o]]=o);var a=e.exports={rgb:{channels:3,labels:"rgb"},hsl:{channels:3,labels:"hsl"},hsv:{channels:3,labels:"hsv"},hwb:{channels:3,labels:"hwb"},cmyk:{channels:4,labels:"cmyk"},xyz:{channels:3,labels:"xyz"},lab:{channels:3,labels:"lab"},lch:{channels:3,labels:"lch"},hex:{channels:1,labels:["hex"]},keyword:{channels:1,labels:["keyword"]},ansi16:{channels:1,labels:["ansi16"]},ansi256:{channels:1,labels:["ansi256"]},hcg:{channels:3,labels:["h","c","g"]},apple:{channels:3,labels:["r16","g16","b16"]},gray:{channels:1,labels:["gray"]}};for(var s in a)if(a.hasOwnProperty(s)){if(!("channels"in a[s]))throw new Error("missing channels property: "+s);if(!("labels"in a[s]))throw new Error("missing channel labels property: "+s);if(a[s].labels.length!==a[s].channels)throw new Error("channel and label counts mismatch: "+s);var l=a[s].channels,u=a[s].labels;delete a[s].channels,delete a[s].labels,Object.defineProperty(a[s],"channels",{value:l}),Object.defineProperty(a[s],"labels",{value:u})}a.rgb.hsl=function(e){var t,n,r=e[0]/255,i=e[1]/255,o=e[2]/255,a=Math.min(r,i,o),s=Math.max(r,i,o),l=s-a;return s===a?t=0:r===s?t=(i-o)/l:i===s?t=2+(o-r)/l:o===s&&(t=4+(r-i)/l),(t=Math.min(60*t,360))<0&&(t+=360),n=(a+s)/2,[t,100*(s===a?0:n<=.5?l/(s+a):l/(2-s-a)),100*n]},a.rgb.hsv=function(e){var t,n,r,i,o,a=e[0]/255,s=e[1]/255,l=e[2]/255,u=Math.max(a,s,l),c=u-Math.min(a,s,l),d=function(e){return(u-e)/6/c+.5};return 0===c?i=o=0:(o=c/u,t=d(a),n=d(s),r=d(l),a===u?i=r-n:s===u?i=1/3+t-r:l===u&&(i=2/3+n-t),i<0?i+=1:i>1&&(i-=1)),[360*i,100*o,100*u]},a.rgb.hwb=function(e){var t=e[0],n=e[1],r=e[2];return[a.rgb.hsl(e)[0],1/255*Math.min(t,Math.min(n,r))*100,100*(r=1-1/255*Math.max(t,Math.max(n,r)))]},a.rgb.cmyk=function(e){var t,n=e[0]/255,r=e[1]/255,i=e[2]/255;return[100*((1-n-(t=Math.min(1-n,1-r,1-i)))/(1-t)||0),100*((1-r-t)/(1-t)||0),100*((1-i-t)/(1-t)||0),100*t]},a.rgb.keyword=function(e){var t=i[e];if(t)return t;var n,o,a,s=1/0;for(var l in r)if(r.hasOwnProperty(l)){var u=(o=e,a=r[l],Math.pow(o[0]-a[0],2)+Math.pow(o[1]-a[1],2)+Math.pow(o[2]-a[2],2));u<s&&(s=u,n=l)}return n},a.keyword.rgb=function(e){return r[e]},a.rgb.xyz=function(e){var t=e[0]/255,n=e[1]/255,r=e[2]/255;return[100*(.4124*(t=t>.04045?Math.pow((t+.055)/1.055,2.4):t/12.92)+.3576*(n=n>.04045?Math.pow((n+.055)/1.055,2.4):n/12.92)+.1805*(r=r>.04045?Math.pow((r+.055)/1.055,2.4):r/12.92)),100*(.2126*t+.7152*n+.0722*r),100*(.0193*t+.1192*n+.9505*r)]},a.rgb.lab=function(e){var t=a.rgb.xyz(e),n=t[0],r=t[1],i=t[2];return r/=100,i/=108.883,n=(n/=95.047)>.008856?Math.pow(n,1/3):7.787*n+16/116,[116*(r=r>.008856?Math.pow(r,1/3):7.787*r+16/116)-16,500*(n-r),200*(r-(i=i>.008856?Math.pow(i,1/3):7.787*i+16/116))]},a.hsl.rgb=function(e){var t,n,r,i,o,a=e[0]/360,s=e[1]/100,l=e[2]/100;if(0===s)return[o=255*l,o,o];t=2*l-(n=l<.5?l*(1+s):l+s-l*s),i=[0,0,0];for(var u=0;u<3;u++)(r=a+1/3*-(u-1))<0&&r++,r>1&&r--,o=6*r<1?t+6*(n-t)*r:2*r<1?n:3*r<2?t+(n-t)*(2/3-r)*6:t,i[u]=255*o;return i},a.hsl.hsv=function(e){var t=e[0],n=e[1]/100,r=e[2]/100,i=n,o=Math.max(r,.01);return n*=(r*=2)<=1?r:2-r,i*=o<=1?o:2-o,[t,100*(0===r?2*i/(o+i):2*n/(r+n)),(r+n)/2*100]},a.hsv.rgb=function(e){var t=e[0]/60,n=e[1]/100,r=e[2]/100,i=Math.floor(t)%6,o=t-Math.floor(t),a=255*r*(1-n),s=255*r*(1-n*o),l=255*r*(1-n*(1-o));switch(r*=255,i){case 0:return[r,l,a];case 1:return[s,r,a];case 2:return[a,r,l];case 3:return[a,s,r];case 4:return[l,a,r];case 5:return[r,a,s]}},a.hsv.hsl=function(e){var t,n,r,i=e[0],o=e[1]/100,a=e[2]/100,s=Math.max(a,.01);return r=(2-o)*a,n=o*s,[i,100*(n=(n/=(t=(2-o)*s)<=1?t:2-t)||0),100*(r/=2)]},a.hwb.rgb=function(e){var t,n,r,i,o,a,s,l=e[0]/360,u=e[1]/100,c=e[2]/100,d=u+c;switch(d>1&&(u/=d,c/=d),r=6*l-(t=Math.floor(6*l)),0!=(1&t)&&(r=1-r),i=u+r*((n=1-c)-u),t){default:case 6:case 0:o=n,a=i,s=u;break;case 1:o=i,a=n,s=u;break;case 2:o=u,a=n,s=i;break;case 3:o=u,a=i,s=n;break;case 4:o=i,a=u,s=n;break;case 5:o=n,a=u,s=i}return[255*o,255*a,255*s]},a.cmyk.rgb=function(e){var t=e[0]/100,n=e[1]/100,r=e[2]/100,i=e[3]/100;return[255*(1-Math.min(1,t*(1-i)+i)),255*(1-Math.min(1,n*(1-i)+i)),255*(1-Math.min(1,r*(1-i)+i))]},a.xyz.rgb=function(e){var t,n,r,i=e[0]/100,o=e[1]/100,a=e[2]/100;return n=-.9689*i+1.8758*o+.0415*a,r=.0557*i+-.204*o+1.057*a,t=(t=3.2406*i+-1.5372*o+-.4986*a)>.0031308?1.055*Math.pow(t,1/2.4)-.055:12.92*t,n=n>.0031308?1.055*Math.pow(n,1/2.4)-.055:12.92*n,r=r>.0031308?1.055*Math.pow(r,1/2.4)-.055:12.92*r,[255*(t=Math.min(Math.max(0,t),1)),255*(n=Math.min(Math.max(0,n),1)),255*(r=Math.min(Math.max(0,r),1))]},a.xyz.lab=function(e){var t=e[0],n=e[1],r=e[2];return n/=100,r/=108.883,t=(t/=95.047)>.008856?Math.pow(t,1/3):7.787*t+16/116,[116*(n=n>.008856?Math.pow(n,1/3):7.787*n+16/116)-16,500*(t-n),200*(n-(r=r>.008856?Math.pow(r,1/3):7.787*r+16/116))]},a.lab.xyz=function(e){var t,n,r,i=e[0];t=e[1]/500+(n=(i+16)/116),r=n-e[2]/200;var o=Math.pow(n,3),a=Math.pow(t,3),s=Math.pow(r,3);return n=o>.008856?o:(n-16/116)/7.787,t=a>.008856?a:(t-16/116)/7.787,r=s>.008856?s:(r-16/116)/7.787,[t*=95.047,n*=100,r*=108.883]},a.lab.lch=function(e){var t,n=e[0],r=e[1],i=e[2];return(t=360*Math.atan2(i,r)/2/Math.PI)<0&&(t+=360),[n,Math.sqrt(r*r+i*i),t]},a.lch.lab=function(e){var t,n=e[0],r=e[1];return t=e[2]/360*2*Math.PI,[n,r*Math.cos(t),r*Math.sin(t)]},a.rgb.ansi16=function(e){var t=e[0],n=e[1],r=e[2],i=1 in arguments?arguments[1]:a.rgb.hsv(e)[2];if(0===(i=Math.round(i/50)))return 30;var o=30+(Math.round(r/255)<<2|Math.round(n/255)<<1|Math.round(t/255));return 2===i&&(o+=60),o},a.hsv.ansi16=function(e){return a.rgb.ansi16(a.hsv.rgb(e),e[2])},a.rgb.ansi256=function(e){var t=e[0],n=e[1],r=e[2];return t===n&&n===r?t<8?16:t>248?231:Math.round((t-8)/247*24)+232:16+36*Math.round(t/255*5)+6*Math.round(n/255*5)+Math.round(r/255*5)},a.ansi16.rgb=function(e){var t=e%10;if(0===t||7===t)return e>50&&(t+=3.5),[t=t/10.5*255,t,t];var n=.5*(1+~~(e>50));return[(1&t)*n*255,(t>>1&1)*n*255,(t>>2&1)*n*255]},a.ansi256.rgb=function(e){if(e>=232){var t=10*(e-232)+8;return[t,t,t]}var n;return e-=16,[Math.floor(e/36)/5*255,Math.floor((n=e%36)/6)/5*255,n%6/5*255]},a.rgb.hex=function(e){var t=(((255&Math.round(e[0]))<<16)+((255&Math.round(e[1]))<<8)+(255&Math.round(e[2]))).toString(16).toUpperCase();return"000000".substring(t.length)+t},a.hex.rgb=function(e){var t=e.toString(16).match(/[a-f0-9]{6}|[a-f0-9]{3}/i);if(!t)return[0,0,0];var n=t[0];3===t[0].length&&(n=n.split("").map((function(e){return e+e})).join(""));var r=parseInt(n,16);return[r>>16&255,r>>8&255,255&r]},a.rgb.hcg=function(e){var t,n=e[0]/255,r=e[1]/255,i=e[2]/255,o=Math.max(Math.max(n,r),i),a=Math.min(Math.min(n,r),i),s=o-a;return t=s<=0?0:o===n?(r-i)/s%6:o===r?2+(i-n)/s:4+(n-r)/s+4,t/=6,[360*(t%=1),100*s,100*(s<1?a/(1-s):0)]},a.hsl.hcg=function(e){var t,n=e[1]/100,r=e[2]/100,i=0;return(t=r<.5?2*n*r:2*n*(1-r))<1&&(i=(r-.5*t)/(1-t)),[e[0],100*t,100*i]},a.hsv.hcg=function(e){var t=e[1]/100,n=e[2]/100,r=t*n,i=0;return r<1&&(i=(n-r)/(1-r)),[e[0],100*r,100*i]},a.hcg.rgb=function(e){var t=e[0]/360,n=e[1]/100,r=e[2]/100;if(0===n)return[255*r,255*r,255*r];var i,o=[0,0,0],a=t%1*6,s=a%1,l=1-s;switch(Math.floor(a)){case 0:o[0]=1,o[1]=s,o[2]=0;break;case 1:o[0]=l,o[1]=1,o[2]=0;break;case 2:o[0]=0,o[1]=1,o[2]=s;break;case 3:o[0]=0,o[1]=l,o[2]=1;break;case 4:o[0]=s,o[1]=0,o[2]=1;break;default:o[0]=1,o[1]=0,o[2]=l}return i=(1-n)*r,[255*(n*o[0]+i),255*(n*o[1]+i),255*(n*o[2]+i)]},a.hcg.hsv=function(e){var t=e[1]/100,n=t+e[2]/100*(1-t),r=0;return n>0&&(r=t/n),[e[0],100*r,100*n]},a.hcg.hsl=function(e){var t=e[1]/100,n=e[2]/100*(1-t)+.5*t,r=0;return n>0&&n<.5?r=t/(2*n):n>=.5&&n<1&&(r=t/(2*(1-n))),[e[0],100*r,100*n]},a.hcg.hwb=function(e){var t=e[1]/100,n=t+e[2]/100*(1-t);return[e[0],100*(n-t),100*(1-n)]},a.hwb.hcg=function(e){var t=e[1]/100,n=1-e[2]/100,r=n-t,i=0;return r<1&&(i=(n-r)/(1-r)),[e[0],100*r,100*i]},a.apple.rgb=function(e){return[e[0]/65535*255,e[1]/65535*255,e[2]/65535*255]},a.rgb.apple=function(e){return[e[0]/255*65535,e[1]/255*65535,e[2]/255*65535]},a.gray.rgb=function(e){return[e[0]/100*255,e[0]/100*255,e[0]/100*255]},a.gray.hsl=a.gray.hsv=function(e){return[0,0,e[0]]},a.gray.hwb=function(e){return[0,100,e[0]]},a.gray.cmyk=function(e){return[0,0,0,e[0]]},a.gray.lab=function(e){return[e[0],0,0]},a.gray.hex=function(e){var t=255&Math.round(e[0]/100*255),n=((t<<16)+(t<<8)+t).toString(16).toUpperCase();return"000000".substring(n.length)+n},a.rgb.gray=function(e){return[(e[0]+e[1]+e[2])/3/255*100]}},function(e,t){},function(e,t){},function(e,t){var n,r,i=e.exports={};function o(){throw new Error("setTimeout has not been defined")}function a(){throw new Error("clearTimeout has not been defined")}function s(e){if(n===setTimeout)return setTimeout(e,0);if((n===o||!n)&&setTimeout)return n=setTimeout,setTimeout(e,0);try{return n(e,0)}catch(t){try{return n.call(null,e,0)}catch(t){return n.call(this,e,0)}}}!function(){try{n="function"==typeof setTimeout?setTimeout:o}catch(e){n=o}try{r="function"==typeof clearTimeout?clearTimeout:a}catch(e){r=a}}();var l,u=[],c=!1,d=-1;function h(){c&&l&&(c=!1,l.length?u=l.concat(u):d=-1,u.length&&f())}function f(){if(!c){var e=s(h);c=!0;for(var t=u.length;t;){for(l=u,u=[];++d<t;)l&&l[d].run();d=-1,t=u.length}l=null,c=!1,function(e){if(r===clearTimeout)return clearTimeout(e);if((r===a||!r)&&clearTimeout)return r=clearTimeout,clearTimeout(e);try{r(e)}catch(t){try{return r.call(null,e)}catch(t){return r.call(this,e)}}}(e)}}function p(e,t){this.fun=e,this.array=t}function g(){}i.nextTick=function(e){var t=new Array(arguments.length-1);if(arguments.length>1)for(var n=1;n<arguments.length;n++)t[n-1]=arguments[n];u.push(new p(e,t)),1!==u.length||c||s(f)},p.prototype.run=function(){this.fun.apply(null,this.array)},i.title="browser",i.browser=!0,i.env={},i.argv=[],i.version="",i.versions={},i.on=g,i.addListener=g,i.once=g,i.off=g,i.removeListener=g,i.removeAllListeners=g,i.emit=g,i.prependListener=g,i.prependOnceListener=g,i.listeners=function(e){return[]},i.binding=function(e){throw new Error("process.binding is not supported")},i.cwd=function(){return"/"},i.chdir=function(e){throw new Error("process.chdir is not supported")},i.umask=function(){return 0}},function(e,t,n){"use strict";var r=n(1),i=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(r.__extends)(t,e),t.prototype.isGroup=function(){return!0},t.prototype.isEntityGroup=function(){return!1},t.prototype.clone=function(){for(var t=e.prototype.clone.call(this),n=this.getChildren(),r=0;r<n.length;r++){var i=n[r];t.add(i.clone())}return t},t}(n(100).a);t.a=i},function(e,t,n){"use strict";var r=n(1),i=n(101),o=n(32),a=function(e){function t(t){return e.call(this,t)||this}return Object(r.__extends)(t,e),t.prototype._isInBBox=function(e,t){var n=this.getBBox();return n.minX<=e&&n.maxX>=e&&n.minY<=t&&n.maxY>=t},t.prototype.afterAttrsChange=function(t){e.prototype.afterAttrsChange.call(this,t),this.clearCacheBBox()},t.prototype.getBBox=function(){var e=this.cfg.bbox;return e||(e=this.calculateBBox(),this.set("bbox",e)),e},t.prototype.getCanvasBBox=function(){var e=this.cfg.canvasBBox;return e||(e=this.calculateCanvasBBox(),this.set("canvasBBox",e)),e},t.prototype.applyMatrix=function(t){e.prototype.applyMatrix.call(this,t),this.set("canvasBBox",null)},t.prototype.calculateCanvasBBox=function(){var e=this.getBBox(),t=this.getTotalMatrix(),n=e.minX,r=e.minY,i=e.maxX,a=e.maxY;if(t){var s=Object(o.c)(t,[e.minX,e.minY]),l=Object(o.c)(t,[e.maxX,e.minY]),u=Object(o.c)(t,[e.minX,e.maxY]),c=Object(o.c)(t,[e.maxX,e.maxY]);n=Math.min(s[0],l[0],u[0],c[0]),i=Math.max(s[0],l[0],u[0],c[0]),r=Math.min(s[1],l[1],u[1],c[1]),a=Math.max(s[1],l[1],u[1],c[1])}var d=this.attrs;if(d.shadowColor){var h=d.shadowBlur,f=void 0===h?0:h,p=d.shadowOffsetX,g=void 0===p?0:p,v=d.shadowOffsetY,m=void 0===v?0:v,y=n-f+g,b=i+f+g,C=r-f+m,A=a+f+m;n=Math.min(n,y),i=Math.max(i,b),r=Math.min(r,C),a=Math.max(a,A)}return{x:n,y:r,minX:n,minY:r,maxX:i,maxY:a,width:i-n,height:a-r}},t.prototype.clearCacheBBox=function(){this.set("bbox",null),this.set("canvasBBox",null)},t.prototype.isClipShape=function(){return this.get("isClipShape")},t.prototype.isInShape=function(e,t){return!1},t.prototype.isOnlyHitBox=function(){return!1},t.prototype.isHit=function(e,t){var n=this.get("startArrowShape"),r=this.get("endArrowShape"),i=[e,t,1],o=(i=this.invertFromMatrix(i))[0],a=i[1],s=this._isInBBox(o,a);if(this.isOnlyHitBox())return s;if(s&&!this.isClipped(o,a)){if(this.isInShape(o,a))return!0;if(n&&n.isHit(o,a))return!0;if(r&&r.isHit(o,a))return!0}return!1},t}(i.a);t.a=a},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.isAllowCapture=t.isParent=t.isBrowser=t.removeFromArray=void 0,t.removeFromArray=function(e,t){var n=e.indexOf(t);-1!==n&&e.splice(n,1)},t.isBrowser="undefined"!=typeof window&&void 0!==window.document;var r=n(10);Object.defineProperty(t,"isNil",{enumerable:!0,get:function(){return r.default}});var i=n(14);Object.defineProperty(t,"isFunction",{enumerable:!0,get:function(){return i.default}});var o=n(3);Object.defineProperty(t,"isString",{enumerable:!0,get:function(){return o.default}});var a=n(35);Object.defineProperty(t,"isObject",{enumerable:!0,get:function(){return a.default}});var s=n(4);Object.defineProperty(t,"isArray",{enumerable:!0,get:function(){return s.default}});var l=n(17);Object.defineProperty(t,"mix",{enumerable:!0,get:function(){return l.default}});var u=n(2);Object.defineProperty(t,"each",{enumerable:!0,get:function(){return u.default}});var c=n(49);Object.defineProperty(t,"upperFirst",{enumerable:!0,get:function(){return c.default}}),t.isParent=function(e,t){if(e.isCanvas())return!0;for(var n=t.getParent(),r=!1;n;){if(n===e){r=!0;break}n=n.getParent()}return r},t.isAllowCapture=function(e){return e.cfg.visible&&e.cfg.capture}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.parseRadius=t.parseStyle=t.parsePattern=t.parseRadialGradient=t.parseLineGradient=void 0;var r=n(18),i=/^l\s*\(\s*([\d.]+)\s*\)\s*(.*)/i,o=/^r\s*\(\s*([\d.]+)\s*,\s*([\d.]+)\s*,\s*([\d.]+)\s*\)\s*(.*)/i,a=/^p\s*\(\s*([axyn])\s*\)\s*(.*)/i,s=/[\d.]+:(#[^\s]+|[^\)]+\))/gi;function l(e,t){var n=e.match(s);r.each(n,(function(e){var n=e.split(":");t.addColorStop(n[0],n[1])}))}function u(e,t,n){var r,o,a=i.exec(n),s=parseFloat(a[1])%360*(Math.PI/180),u=a[2],c=t.getBBox();s>=0&&s<.5*Math.PI?(r={x:c.minX,y:c.minY},o={x:c.maxX,y:c.maxY}):.5*Math.PI<=s&&s<Math.PI?(r={x:c.maxX,y:c.minY},o={x:c.minX,y:c.maxY}):Math.PI<=s&&s<1.5*Math.PI?(r={x:c.maxX,y:c.maxY},o={x:c.minX,y:c.minY}):(r={x:c.minX,y:c.maxY},o={x:c.maxX,y:c.minY});var d=Math.tan(s),h=d*d,f=(o.x-r.x+d*(o.y-r.y))/(h+1)+r.x,p=d*(o.x-r.x+d*(o.y-r.y))/(h+1)+r.y,g=e.createLinearGradient(r.x,r.y,f,p);return l(u,g),g}function c(e,t,n){var r=o.exec(n),i=parseFloat(r[1]),a=parseFloat(r[2]),u=parseFloat(r[3]),c=r[4];if(0===u){var d=c.match(s);return d[d.length-1].split(":")[1]}var h=t.getBBox(),f=h.maxX-h.minX,p=h.maxY-h.minY,g=Math.sqrt(f*f+p*p)/2,v=e.createRadialGradient(h.minX+f*i,h.minY+p*a,0,h.minX+f/2,h.minY+p/2,u*g);return l(c,v),v}function d(e,t,n){if(t.get("patternSource")&&t.get("patternSource")===n)return t.get("pattern");var r,i,o=a.exec(n),s=o[1],l=o[2];function u(){r=e.createPattern(i,s),t.set("pattern",r),t.set("patternSource",n)}switch(s){case"a":s="repeat";break;case"x":s="repeat-x";break;case"y":s="repeat-y";break;case"n":s="no-repeat";break;default:s="no-repeat"}return i=new Image,l.match(/^data:/i)||(i.crossOrigin="Anonymous"),i.src=l,i.complete?u():(i.onload=u,i.src=i.src),r}t.parseLineGradient=u,t.parseRadialGradient=c,t.parsePattern=d,t.parseStyle=function(e,t,n){if(r.isString(n)){if("("===n[1]||"("===n[2]){if("l"===n[0])return u(e,t,n);if("r"===n[0])return c(e,t,n);if("p"===n[0])return d(e,t,n)}return n}},t.parseRadius=function(e){var t=0,n=0,i=0,o=0;return r.isArray(e)?1===e.length?t=n=i=o=e[0]:2===e.length?(t=i=e[0],n=o=e[1]):3===e.length?(t=e[0],n=o=e[1],i=e[2]):(t=e[0],n=e[1],i=e[2],o=e[3]):t=n=i=o=e,[t,n,i,o]}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.getBBoxMethod=void 0;var r=n(322);Object.defineProperty(t,"getBBoxMethod",{enumerable:!0,get:function(){return r.getMethod}});var i=n(323),o=n(324),a=n(325),s=n(326),l=n(327),u=n(328),c=n(332),d=n(333);r.register("rect",i.default),r.register("image",i.default),r.register("circle",o.default),r.register("marker",o.default),r.register("polyline",a.default),r.register("polygon",s.default),r.register("text",l.default),r.register("path",u.default),r.register("line",c.default),r.register("ellipse",d.default)},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.assembleFont=t.getTextWidth=t.getLineSpaceing=t.getTextHeight=void 0;var r=n(155),i=n(159);function o(e,t){return t?t-e:.14*e}t.getTextHeight=function(e,t,n){var i=1;return r.isString(e)&&(i=e.split("\n").length),i>1?t*i+o(t,n)*(i-1):t},t.getLineSpaceing=o,t.getTextWidth=function(e,t){var n=i.getOffScreenContext(),o=0;if(r.isNil(e)||""===e)return o;if(n.save(),n.font=t,r.isString(e)&&e.includes("\n")){var a=e.split("\n");r.each(a,(function(e){var t=n.measureText(e).width;o<t&&(o=t)}))}else o=n.measureText(e).width;return n.restore(),o},t.assembleFont=function(e){var t=e.fontSize,n=e.fontFamily,r=e.fontWeight;return[e.fontStyle,e.fontVariant,r,t+"px",n].join(" ").trim()}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.getOffScreenContext=void 0;var r=null;t.getOffScreenContext=function(){if(!r){var e=document.createElement("canvas");e.width=1,e.height=1,r=e.getContext("2d")}return r}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(40),i=n(33),o=n(161);function a(e,t,n,r){var i=1-r;return i*i*e+2*r*i*t+r*r*n}function s(e,t,n){var r=e+n-2*t;if(i.isNumberEqual(r,0))return[.5];var o=(e-t)/r;return o<=1&&o>=0?[o]:[]}function l(e,t,n,r){return 2*(1-r)*(t-e)+2*r*(n-t)}function u(e,t,n,i,o,s,l){var u=a(e,n,o,l),c=a(t,i,s,l),d=r.default.pointAt(e,t,n,i,l),h=r.default.pointAt(n,i,o,s,l);return[[e,t,d.x,d.y,u,c],[u,c,h.x,h.y,o,s]]}t.default={box:function(e,t,n,r,o,l){var u=s(e,n,o)[0],c=s(t,r,l)[0],d=[e,o],h=[t,l];return void 0!==u&&d.push(a(e,n,o,u)),void 0!==c&&h.push(a(t,r,l,c)),i.getBBoxByArray(d,h)},length:function(e,t,n,r,o,a){return function e(t,n,r,o,a,s,l){if(0===l)return(i.distance(t,n,r,o)+i.distance(r,o,a,s)+i.distance(t,n,a,s))/2;var c=u(t,n,r,o,a,s,.5),d=c[0],h=c[1];return d.push(l-1),h.push(l-1),e.apply(null,d)+e.apply(null,h)}(e,t,n,r,o,a,3)},nearestPoint:function(e,t,n,r,i,s,l,u){return o.nearestPoint([e,n,i],[t,r,s],l,u,a)},pointDistance:function(e,t,n,r,o,a,s,l){var u=this.nearestPoint(e,t,n,r,o,a,s,l);return i.distance(u.x,u.y,s,l)},interpolationAt:a,pointAt:function(e,t,n,r,i,o,s){return{x:a(e,n,i,s),y:a(t,r,o,s)}},divide:function(e,t,n,r,i,o,a){return u(e,t,n,r,i,o,a)},tangentAngle:function(e,t,n,r,o,a,s){var u=l(e,n,o,s),c=l(t,r,a,s),d=Math.atan2(c,u);return i.piMod(d)}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.snapLength=t.nearestPoint=void 0;var r=n(33);t.nearestPoint=function(e,t,n,i,o,a){var s,l=1/0,u=[n,i],c=20;a&&a>200&&(c=a/10);for(var d=1/c,h=d/10,f=0;f<=c;f++){var p=f*d,g=[o.apply(null,e.concat([p])),o.apply(null,t.concat([p]))];(b=r.distance(u[0],u[1],g[0],g[1]))<l&&(s=p,l=b)}if(0===s)return{x:e[0],y:t[0]};if(1===s){var v=e.length;return{x:e[v-1],y:t[v-1]}}for(l=1/0,f=0;f<32&&!(h<1e-4);f++){var m=s-h,y=s+h,b=(g=[o.apply(null,e.concat([m])),o.apply(null,t.concat([m]))],r.distance(u[0],u[1],g[0],g[1]));if(m>=0&&b<l)s=m,l=b;else{var C=[o.apply(null,e.concat([y])),o.apply(null,t.concat([y]))],A=r.distance(u[0],u[1],C[0],C[1]);y<=1&&A<l?(s=y,l=A):h*=.5}}return{x:o.apply(null,e.concat([s])),y:o.apply(null,t.concat([s]))}},t.snapLength=function(e,t){for(var n=0,i=e.length,o=0;o<i;o++){var a=e[o],s=t[o],l=e[(o+1)%i],u=t[(o+1)%i];n+=r.distance(a,s,l,u)}return n/2}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(163),i=n(163),o=n(331);function a(e,t){return[t[0]+(t[0]-e[0]),t[1]+(t[1]-e[1])]}t.default=function(e){for(var t=[],n=null,s=null,l=null,u=0,c=(e=o.default(e)).length,d=0;d<c;d++){var h=e[d];s=e[d+1];var f=h[0],p={command:f,prePoint:n,params:h,startTangent:null,endTangent:null};switch(f){case"M":l=[h[1],h[2]],u=d;break;case"A":var g=r.default(n,h);p.arcParams=g}if("Z"===f)n=l,s=e[u+1];else{var v=h.length;n=[h[v-2],h[v-1]]}s&&"Z"===s[0]&&(s=e[u],t[u]&&(t[u].prePoint=n)),p.currentPoint=n,t[u]&&i.isSamePoint(n,t[u].currentPoint)&&(t[u].prePoint=p.prePoint);var m=s?[s[s.length-2],s[s.length-1]]:null;p.nextPoint=m;var y=p.prePoint;if(["L","H","V"].includes(f))p.startTangent=[y[0]-n[0],y[1]-n[1]],p.endTangent=[n[0]-y[0],n[1]-y[1]];else if("Q"===f){var b=[h[1],h[2]];p.startTangent=[y[0]-b[0],y[1]-b[1]],p.endTangent=[n[0]-b[0],n[1]-b[1]]}else if("T"===f)b=a((_=t[d-1]).currentPoint,y),"Q"===_.command?(p.command="Q",p.startTangent=[y[0]-b[0],y[1]-b[1]],p.endTangent=[n[0]-b[0],n[1]-b[1]]):(p.command="TL",p.startTangent=[y[0]-n[0],y[1]-n[1]],p.endTangent=[n[0]-y[0],n[1]-y[1]]);else if("C"===f){var C=[h[1],h[2]],A=[h[3],h[4]];p.startTangent=[y[0]-C[0],y[1]-C[1]],p.endTangent=[n[0]-A[0],n[1]-A[1]]}else if("S"===f){var _;C=a((_=t[d-1]).currentPoint,y),A=[h[1],h[2]],"C"===_.command?(p.command="C",p.startTangent=[y[0]-C[0],y[1]-C[1]],p.endTangent=[n[0]-A[0],n[1]-A[1]]):(p.command="SQ",p.startTangent=[y[0]-A[0],y[1]-A[1]],p.endTangent=[n[0]-A[0],n[1]-A[1]])}else if("A"===f){var I=.001,x=p.arcParams||{},w=x.cx,M=void 0===w?0:w,S=x.cy,T=void 0===S?0:S,E=x.rx,O=void 0===E?0:E,k=x.ry,L=void 0===k?0:k,D=x.sweepFlag,P=void 0===D?0:D,N=x.startAngle,j=void 0===N?0:N,R=x.endAngle,z=void 0===R?0:R;0===P&&(I*=-1);var F=O*Math.cos(j-I)+M,B=L*Math.sin(j-I)+T;p.startTangent=[F-l[0],B-l[1]];var H=O*Math.cos(j+z+I)+M,U=L*Math.sin(j+z-I)+T;p.endTangent=[y[0]-H,y[1]-U]}t.push(p)}return t}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(67),i=n(68);function o(e){return Math.sqrt(e[0]*e[0]+e[1]*e[1])}function a(e,t){return o(e)*o(t)?(e[0]*t[0]+e[1]*t[1])/(o(e)*o(t)):1}function s(e,t){return(e[0]*t[1]<e[1]*t[0]?-1:1)*Math.acos(a(e,t))}function l(e,t){return e[0]===t[0]&&e[1]===t[1]}t.isSamePoint=l,t.default=function(e,t){var n=t[1],o=t[2],u=r.default(i.default(t[3]),2*Math.PI),c=t[4],d=t[5],h=e[0],f=e[1],p=t[6],g=t[7],v=Math.cos(u)*(h-p)/2+Math.sin(u)*(f-g)/2,m=-1*Math.sin(u)*(h-p)/2+Math.cos(u)*(f-g)/2,y=v*v/(n*n)+m*m/(o*o);y>1&&(n*=Math.sqrt(y),o*=Math.sqrt(y));var b=n*n*(m*m)+o*o*(v*v),C=b?Math.sqrt((n*n*(o*o)-b)/b):1;c===d&&(C*=-1),isNaN(C)&&(C=0);var A=o?C*n*m/o:0,_=n?C*-o*v/n:0,I=(h+p)/2+Math.cos(u)*A-Math.sin(u)*_,x=(f+g)/2+Math.sin(u)*A+Math.cos(u)*_,w=[(v-A)/n,(m-_)/o],M=[(-1*v-A)/n,(-1*m-_)/o],S=s([1,0],w),T=s(w,M);return a(w,M)<=-1&&(T=Math.PI),a(w,M)>=1&&(T=0),0===d&&T>0&&(T-=2*Math.PI),1===d&&T<0&&(T+=2*Math.PI),{cx:I,cy:x,rx:l(e,[p,g])?0:n,ry:l(e,[p,g])?0:o,startAngle:S,endAngle:S+T,xRotation:u,arcFlag:c,sweepFlag:d}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(1),i=n(43),o=n(73),a=n(60),s=n(0),l=n(18),u=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return r.__extends(t,e),t.prototype.onCanvasChange=function(e){a.refreshElement(this,e)},t.prototype.getShapeBase=function(){return o},t.prototype.getGroupBase=function(){return t},t.prototype._applyClip=function(e,t){t&&(e.save(),a.applyAttrsToContext(e,t),t.createPath(e),e.restore(),e.clip(),t._afterDraw())},t.prototype.cacheCanvasBBox=function(){var e=this.cfg.children,t=[],n=[];s.each(e,(function(e){var r=e.cfg.cacheCanvasBBox;r&&e.cfg.isInView&&(t.push(r.minX,r.maxX),n.push(r.minY,r.maxY))}));var r=null;if(t.length){var i=Math.min.apply(null,t),o=Math.max.apply(null,t),a=Math.min.apply(null,n),u=Math.max.apply(null,n);r={minX:i,minY:a,x:i,y:a,maxX:o,maxY:u,width:o-i,height:u-a};var c=this.cfg.canvas;if(c){var d=c.getViewRange();this.set("isInView",l.intersectRect(r,d))}}else this.set("isInView",!1);this.set("cacheCanvasBBox",r)},t.prototype.draw=function(e,t){var n=this.cfg.children,r=!t||this.cfg.refresh;n.length&&r&&(e.save(),a.applyAttrsToContext(e,this),this._applyClip(e,this.getClip()),a.drawChildren(e,n,t),e.restore(),this.cacheCanvasBBox()),this.cfg.refresh=null,this.set("hasChanged",!1)},t.prototype.skipDraw=function(){this.set("cacheCanvasBBox",null),this.set("hasChanged",!1)},t}(i.AbstractGroup);t.default=u},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(339),i=/[a-z]/;function o(e,t){return[t[0]+(t[0]-e[0]),t[1]+(t[1]-e[1])]}t.default=function(e){var t=r.default(e);if(!t||!t.length)return[["M",0,0]];for(var n=!1,a=0;a<t.length;a++){var s=t[a][0];if(i.test(s)||["V","H","T","S"].indexOf(s)>=0){n=!0;break}}if(!n)return t;var l=[],u=0,c=0,d=0,h=0,f=0,p=t[0];"M"!==p[0]&&"m"!==p[0]||(d=u=+p[1],h=c=+p[2],f++,l[0]=["M",u,c]),a=f;for(var g=t.length;a<g;a++){var v=t[a],m=l[a-1],y=[],b=(s=v[0]).toUpperCase();if(s!==b)switch(y[0]=b,b){case"A":y[1]=v[1],y[2]=v[2],y[3]=v[3],y[4]=v[4],y[5]=v[5],y[6]=+v[6]+u,y[7]=+v[7]+c;break;case"V":y[1]=+v[1]+c;break;case"H":y[1]=+v[1]+u;break;case"M":d=+v[1]+u,h=+v[2]+c;break;default:for(var C=1,A=v.length;C<A;C++)y[C]=+v[C]+(C%2?u:c)}else y=t[a];switch(b){case"Z":u=+d,c=+h;break;case"H":y=["L",u=y[1],c];break;case"V":y=["L",u,c=y[1]];break;case"T":u=y[1],c=y[2];var _=o([m[1],m[2]],[m[3],m[4]]);y=["Q",_[0],_[1],u,c];break;case"S":u=y[y.length-2],c=y[y.length-1];var I=m.length,x=o([m[I-4],m[I-3]],[m[I-2],m[I-1]]);y=["C",x[0],x[1],y[1],y[2],u,c];break;case"M":d=y[y.length-2],h=y[y.length-1];break;default:u=y[y.length-2],c=y[y.length-1]}l.push(y)}return l}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(159);t.default=function(e,t,n){var i=r.getOffScreenContext();return e.createPath(i),i.isPointInPath(t,n)}},function(e,t,n){"use strict";function r(e){return Math.abs(e)<1e-6?0:e<0?-1:1}function i(e,t,n){return(n[0]-e[0])*(t[1]-e[1])==(t[0]-e[0])*(n[1]-e[1])&&Math.min(e[0],t[0])<=n[0]&&n[0]<=Math.max(e[0],t[0])&&Math.min(e[1],t[1])<=n[1]&&n[1]<=Math.max(e[1],t[1])}Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t,n){var o=!1,a=e.length;if(a<=2)return!1;for(var s=0;s<a;s++){var l=e[s],u=e[(s+1)%a];if(i(l,u,[t,n]))return!0;r(l[1]-n)>0!=r(u[1]-n)>0&&r(t-(n-l[1])*(l[0]-u[0])/(l[1]-u[1])-l[0])<0&&(o=!o)}return o}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(18);t.default=function(e,t,n,i,o,a,s,l){var u=(Math.atan2(l-t,s-e)+2*Math.PI)%(2*Math.PI);if(u<i||u>o)return!1;var c={x:e+n*Math.cos(u),y:t+n*Math.sin(u)};return r.distance(c.x,c.y,s,l)<=a/2}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(75);t.default=function(e,t,n,i,o){var a=e.length;if(a<2)return!1;for(var s=0;s<a-1;s++){var l=e[s][0],u=e[s][1],c=e[s+1][0],d=e[s+1][1];if(r.default(l,u,c,d,t,n,i))return!0}if(o){var h=e[0],f=e[a-1];if(r.default(h[0],h[1],f[0],f[1],t,n,i))return!0}return!1}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(344),i=n(33);t.default={box:function(e){for(var t=[],n=[],r=0;r<e.length;r++){var o=e[r];t.push(o[0]),n.push(o[1])}return i.getBBoxByArray(t,n)},length:function(e){return r.lengthOfSegment(e)},pointAt:function(e,t){return r.pointAtSegments(e,t)},pointDistance:function(e,t,n){return r.distanceAtSegment(e,t,n)},tangentAngle:function(e,t){return r.angleAtSegments(e,t)}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(1),i=n(43),o=n(0),a=n(110),s=n(109),l=n(61),u=n(19),c=n(29),d=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return r.__extends(t,e),t.prototype.isEntityGroup=function(){return!0},t.prototype.createDom=function(){var e=c.createSVGElement("g");this.set("el",e);var t=this.getParent();if(t){var n=t.get("el");n||(n=t.createDom(),t.set("el",n)),n.appendChild(e)}return e},t.prototype.afterAttrsChange=function(t){e.prototype.afterAttrsChange.call(this,t);var n=this.get("canvas");if(n&&n.get("autoDraw")){var r=n.get("context");this.createPath(r,t)}},t.prototype.onCanvasChange=function(e){s.refreshElement(this,e)},t.prototype.getShapeBase=function(){return a},t.prototype.getGroupBase=function(){return t},t.prototype.draw=function(e){var t=this.getChildren(),n=this.get("el");this.get("destroyed")?n&&n.parentNode.removeChild(n):(n||this.createDom(),l.setClip(this,e),this.createPath(e),t.length&&s.drawChildren(e,t))},t.prototype.createPath=function(e,t){var n=this.attr(),r=this.get("el");o.each(t||n,(function(e,t){u.SVG_ATTR_MAP[t]&&r.setAttribute(u.SVG_ATTR_MAP[t],e)})),l.setTransform(this)},t}(i.AbstractGroup);t.default=d},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(16);r.translate=function(e,t,n){var i=new Array(9);return r.fromTranslation(i,n),r.multiply(e,i,t)},r.rotate=function(e,t,n){var i=new Array(9);return r.fromRotation(i,n),r.multiply(e,i,t)},r.scale=function(e,t,n){var i=new Array(9);return r.fromScaling(i,n),r.multiply(e,i,t)},r.transform=function(e,t){for(var n=[].concat(e),i=0,o=t.length;i<o;i++){var a=t[i];switch(a[0]){case"t":r.translate(n,n,[a[1],a[2]]);break;case"s":r.scale(n,n,[a[1],a[2]]);break;case"r":r.rotate(n,n,a[1])}}return n},t.default=r},function(e,t,n){var r=n(76),i=n(114),o=n(81),a=n(403),s=n(409),l=n(179),u=n(180),c=n(412),d=n(413),h=n(184),f=n(414),p=n(54),g=n(418),v=n(419),m=n(189),y=n(15),b=n(53),C=n(423),A=n(25),_=n(425),I=n(41),x=n(46),w={};w["[object Arguments]"]=w["[object Array]"]=w["[object ArrayBuffer]"]=w["[object DataView]"]=w["[object Boolean]"]=w["[object Date]"]=w["[object Float32Array]"]=w["[object Float64Array]"]=w["[object Int8Array]"]=w["[object Int16Array]"]=w["[object Int32Array]"]=w["[object Map]"]=w["[object Number]"]=w["[object Object]"]=w["[object RegExp]"]=w["[object Set]"]=w["[object String]"]=w["[object Symbol]"]=w["[object Uint8Array]"]=w["[object Uint8ClampedArray]"]=w["[object Uint16Array]"]=w["[object Uint32Array]"]=!0,w["[object Error]"]=w["[object Function]"]=w["[object WeakMap]"]=!1,e.exports=function e(t,n,M,S,T,E){var O,k=1&n,L=2&n,D=4&n;if(M&&(O=T?M(t,S,T,E):M(t)),void 0!==O)return O;if(!A(t))return t;var P=y(t);if(P){if(O=g(t),!k)return u(t,O)}else{var N=p(t),j="[object Function]"==N||"[object GeneratorFunction]"==N;if(b(t))return l(t,k);if("[object Object]"==N||"[object Arguments]"==N||j&&!T){if(O=L||j?{}:m(t),!k)return L?d(t,s(O,t)):c(t,a(O,t))}else{if(!w[N])return T?t:{};O=v(t,N,k)}}E||(E=new r);var R=E.get(t);if(R)return R;E.set(t,O),_(t)?t.forEach((function(r){O.add(e(r,n,M,r,t,E))})):C(t)&&t.forEach((function(r,i){O.set(i,e(r,n,M,i,t,E))}));var z=P?void 0:(D?L?f:h:L?x:I)(t);return i(z||t,(function(r,i){z&&(r=t[i=r]),o(O,i,e(r,n,M,i,t,E))})),O}},function(e,t,n){(function(t){function n(e){return(n="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}var r="object"==(void 0===t?"undefined":n(t))&&t&&t.Object===Object&&t;e.exports=r}).call(this,n(385))},function(e,t){var n=Function.prototype.toString;e.exports=function(e){if(null!=e){try{return n.call(e)}catch(e){}try{return e+""}catch(e){}}return""}},function(e,t,n){var r=n(44),i=function(){try{var e=r(Object,"defineProperty");return e({},"",{}),e}catch(e){}}();e.exports=i},function(e,t,n){var r=n(404),i=n(64),o=n(15),a=n(53),s=n(83),l=n(65),u=Object.prototype.hasOwnProperty;e.exports=function(e,t){var n=o(e),c=!n&&i(e),d=!n&&!c&&a(e),h=!n&&!c&&!d&&l(e),f=n||c||d||h,p=f?r(e.length,String):[],g=p.length;for(var v in e)!t&&!u.call(e,v)||f&&("length"==v||d&&("offset"==v||"parent"==v)||h&&("buffer"==v||"byteLength"==v||"byteOffset"==v)||s(v,g))||p.push(v);return p}},function(e,t){e.exports=function(e,t){return function(n){return e(t(n))}}},function(e,t,n){(function(e){function r(e){return(r="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}var i=n(31),o="object"==r(t)&&t&&!t.nodeType&&t,a=o&&"object"==r(e)&&e&&!e.nodeType&&e,s=a&&a.exports===o?i.Buffer:void 0,l=s?s.allocUnsafe:void 0;e.exports=function(e,t){if(t)return e.slice();var n=e.length,r=l?l(n):new e.constructor(n);return e.copy(r),r}}).call(this,n(115)(e))},function(e,t){e.exports=function(e,t){var n=-1,r=e.length;for(t||(t=Array(r));++n<r;)t[n]=e[n];return t}},function(e,t){e.exports=function(e,t){for(var n=-1,r=null==e?0:e.length,i=0,o=[];++n<r;){var a=e[n];t(a,n,e)&&(o[i++]=a)}return o}},function(e,t){e.exports=function(){return[]}},function(e,t,n){var r=n(120),i=n(86),o=n(119),a=n(182),s=Object.getOwnPropertySymbols?function(e){for(var t=[];e;)r(t,o(e)),e=i(e);return t}:a;e.exports=s},function(e,t,n){var r=n(185),i=n(119),o=n(41);e.exports=function(e){return r(e,o,i)}},function(e,t,n){var r=n(120),i=n(15);e.exports=function(e,t,n){var o=t(e);return i(e)?o:r(o,n(e))}},function(e,t,n){var r=n(44)(n(31),"Set");e.exports=r},function(e,t,n){var r=n(31).Uint8Array;e.exports=r},function(e,t,n){var r=n(121);e.exports=function(e,t){var n=t?r(e.buffer):e.buffer;return new e.constructor(n,e.byteOffset,e.length)}},function(e,t,n){var r=n(190),i=n(86),o=n(85);e.exports=function(e){return"function"!=typeof e.constructor||o(e)?{}:r(i(e))}},function(e,t,n){var r=n(25),i=Object.create,o=function(){function e(){}return function(t){if(!r(t))return{};if(i)return i(t);e.prototype=t;var n=new e;return e.prototype=void 0,n}}();e.exports=o},function(e,t,n){e.exports=n(192)},function(e,t,n){var r=n(114),i=n(87),o=n(193),a=n(15);e.exports=function(e,t){return(a(e)?r:i)(e,o(t))}},function(e,t,n){var r=n(47);e.exports=function(e){return"function"==typeof e?e:r}},function(e,t,n){var r=n(181),i=n(429),o=n(39),a=n(15);e.exports=function(e,t){return(a(e)?r:i)(e,o(t,3))}},function(e,t,n){var r=n(432),i=n(34);e.exports=function e(t,n,o,a,s){return t===n||(null==t||null==n||!i(t)&&!i(n)?t!=t&&n!=n:r(t,n,o,a,e,s))}},function(e,t,n){var r=n(197),i=n(435),o=n(198);e.exports=function(e,t,n,a,s,l){var u=1&n,c=e.length,d=t.length;if(c!=d&&!(u&&d>c))return!1;var h=l.get(e),f=l.get(t);if(h&&f)return h==t&&f==e;var p=-1,g=!0,v=2&n?new r:void 0;for(l.set(e,t),l.set(t,e);++p<c;){var m=e[p],y=t[p];if(a)var b=u?a(y,m,p,t,e,l):a(m,y,p,e,t,l);if(void 0!==b){if(b)continue;g=!1;break}if(v){if(!i(t,(function(e,t){if(!o(v,t)&&(m===e||s(m,e,n,a,l)))return v.push(t)}))){g=!1;break}}else if(m!==y&&!s(m,y,n,a,l)){g=!1;break}}return l.delete(e),l.delete(t),g}},function(e,t,n){var r=n(113),i=n(433),o=n(434);function a(e){var t=-1,n=null==e?0:e.length;for(this.__data__=new r;++t<n;)this.add(e[t])}a.prototype.add=a.prototype.push=i,a.prototype.has=o,e.exports=a},function(e,t){e.exports=function(e,t){return e.has(t)}},function(e,t,n){var r=n(25);e.exports=function(e){return e==e&&!r(e)}},function(e,t){e.exports=function(e,t){return function(n){return null!=n&&n[e]===t&&(void 0!==t||e in Object(n))}}},function(e,t,n){var r=n(445);e.exports=function(e){return null==e?"":r(e)}},function(e,t,n){var r=n(446),i=n(203);e.exports=function(e,t){return null!=e&&i(e,t,r)}},function(e,t,n){var r=n(89),i=n(64),o=n(15),a=n(83),s=n(116),l=n(66);e.exports=function(e,t,n){for(var u=-1,c=(t=r(t,e)).length,d=!1;++u<c;){var h=l(t[u]);if(!(d=null!=e&&n(e,h)))break;e=e[h]}return d||++u!=c?d:!!(c=null==e?0:e.length)&&s(c)&&a(h,c)&&(o(e)||i(e))}},function(e,t){e.exports=function(e){return function(t){return null==t?void 0:t[e]}}},function(e,t,n){var r=n(449),i=n(203);e.exports=function(e,t){return null!=e&&i(e,t,r)}},function(e,t){e.exports=function(e){return void 0===e}},function(e,t,n){var r=n(90),i=n(39),o=n(208),a=n(15);e.exports=function(e,t){return(a(e)?r:o)(e,i(t,3))}},function(e,t,n){var r=n(87),i=n(38);e.exports=function(e,t){var n=-1,o=i(e)?Array(e.length):[];return r(e,(function(e,r,i){o[++n]=t(e,r,i)})),o}},function(e,t,n){var r=n(451),i=n(87),o=n(39),a=n(452),s=n(15);e.exports=function(e,t,n){var l=s(e)?r:a,u=arguments.length<3;return l(e,o(t,4),n,u,i)}},function(e,t,n){var r=n(462),i=Math.max;e.exports=function(e,t,n){return t=i(void 0===t?e.length-1:t,0),function(){for(var o=arguments,a=-1,s=i(o.length-t,0),l=Array(s);++a<s;)l[a]=o[t+a];a=-1;for(var u=Array(t+1);++a<t;)u[a]=o[a];return u[t]=n(l),r(e,this,u)}}},function(e,t,n){var r=n(463),i=n(464)(r);e.exports=i},function(e,t){e.exports=function(e,t,n,r){for(var i=e.length,o=n+(r?1:-1);r?o--:++o<i;)if(t(e[o],o,e))return o;return-1}},function(e,t,n){var r=n(38),i=n(34);e.exports=function(e){return i(e)&&r(e)}},function(e,t,n){var r=n(473),i=n(41);e.exports=function(e){return null==e?[]:r(e,i(e))}},function(e,t,n){var r=n(24),i=n(216);e.exports=function(e,t,n,r){return function(e,t,n,r){var o,a,s={},l=new i,u=function(e){var t=e.v!==o?e.v:e.w,r=s[t],i=n(e),u=a.distance+i;if(i<0)throw new Error("dijkstra does not allow negative edge weights. Bad edge: "+e+" Weight: "+i);u<r.distance&&(r.distance=u,r.predecessor=o,l.decrease(t,u))};for(e.nodes().forEach((function(e){var n=e===t?0:Number.POSITIVE_INFINITY;s[e]={distance:n},l.add(e,n)}));l.size()>0&&(o=l.removeMin(),(a=s[o]).distance!==Number.POSITIVE_INFINITY);)r(o).forEach(u);return s}(e,String(t),n||o,r||function(t){return e.outEdges(t)})};var o=r.constant(1)},function(e,t,n){var r=n(24);function i(){this._arr=[],this._keyIndices={}}e.exports=i,i.prototype.size=function(){return this._arr.length},i.prototype.keys=function(){return this._arr.map((function(e){return e.key}))},i.prototype.has=function(e){return r.has(this._keyIndices,e)},i.prototype.priority=function(e){var t=this._keyIndices[e];if(void 0!==t)return this._arr[t].priority},i.prototype.min=function(){if(0===this.size())throw new Error("Queue underflow");return this._arr[0].key},i.prototype.add=function(e,t){var n=this._keyIndices;if(e=String(e),!r.has(n,e)){var i=this._arr,o=i.length;return n[e]=o,i.push({key:e,priority:t}),this._decrease(o),!0}return!1},i.prototype.removeMin=function(){this._swap(0,this._arr.length-1);var e=this._arr.pop();return delete this._keyIndices[e.key],this._heapify(0),e.key},i.prototype.decrease=function(e,t){var n=this._keyIndices[e];if(t>this._arr[n].priority)throw new Error("New priority is greater than current priority. Key: "+e+" Old: "+this._arr[n].priority+" New: "+t);this._arr[n].priority=t,this._decrease(n)},i.prototype._heapify=function(e){var t=this._arr,n=2*e,r=n+1,i=e;n<t.length&&(i=t[n].priority<t[i].priority?n:i,r<t.length&&(i=t[r].priority<t[i].priority?r:i),i!==e&&(this._swap(e,i),this._heapify(i)))},i.prototype._decrease=function(e){for(var t,n=this._arr,r=n[e].priority;0!==e&&!(n[t=e>>1].priority<r);)this._swap(e,t),e=t},i.prototype._swap=function(e,t){var n=this._arr,r=this._keyIndices,i=n[e],o=n[t];n[e]=o,n[t]=i,r[o.key]=e,r[i.key]=t}},function(e,t,n){var r=n(24);e.exports=function(e){var t=0,n=[],i={},o=[];return e.nodes().forEach((function(a){r.has(i,a)||function a(s){var l=i[s]={onStack:!0,lowlink:t,index:t++};if(n.push(s),e.successors(s).forEach((function(e){r.has(i,e)?i[e].onStack&&(l.lowlink=Math.min(l.lowlink,i[e].index)):(a(e),l.lowlink=Math.min(l.lowlink,i[e].lowlink))})),l.lowlink===l.index){var u,c=[];do{u=n.pop(),i[u].onStack=!1,c.push(u)}while(s!==u);o.push(c)}}(a)})),o}},function(e,t,n){var r=n(24);function i(e){var t={},n={},i=[];if(r.each(e.sinks(),(function a(s){if(r.has(n,s))throw new o;r.has(t,s)||(n[s]=!0,t[s]=!0,r.each(e.predecessors(s),a),delete n[s],i.push(s))})),r.size(t)!==e.nodeCount())throw new o;return i}function o(){}e.exports=i,i.CycleException=o,o.prototype=new Error},function(e,t,n){var r=n(24);e.exports=function(e,t,n){r.isArray(t)||(t=[t]);var i=(e.isDirected()?e.successors:e.neighbors).bind(e),o=[],a={};return r.each(t,(function(t){if(!e.hasNode(t))throw new Error("Graph does not have node: "+t);!function e(t,n,i,o,a,s){r.has(o,n)||(o[n]=!0,i||s.push(n),r.each(a(n),(function(n){e(t,n,i,o,a,s)})),i&&s.push(n))}(e,t,"post"===n,a,i,o)})),o}},function(e,t,n){var r=n(492);e.exports=function(e){return e?(e=r(e))===1/0||e===-1/0?17976931348623157e292*(e<0?-1:1):e==e?e:0:0===e?e:0}},function(e,t,n){var r=n(127);e.exports=function(e){return null!=e&&e.length?r(e,1):[]}},function(e,t,n){var r=n(82),i=n(51);e.exports=function(e,t,n){(void 0!==n&&!i(e[t],n)||void 0===n&&!(t in e))&&r(e,t,n)}},function(e,t){e.exports=function(e,t){if(("constructor"!==t||"function"!=typeof e[t])&&"__proto__"!=t)return e[t]}},function(e,t){e.exports=function(e,t){return e<t}},function(e,t,n){"use strict";var r=n(13),i=n(30).Graph,o=n(93).slack;function a(e,t){return r.forEach(e.nodes(),(function n(i){r.forEach(t.nodeEdges(i),(function(r){var a=r.v,s=i===a?r.w:a;e.hasNode(s)||o(t,r)||(e.setNode(s,{}),e.setEdge(i,s,{}),n(s))}))})),e.nodeCount()}function s(e,t){return r.minBy(t.edges(),(function(n){if(e.hasNode(n.v)!==e.hasNode(n.w))return o(t,n)}))}function l(e,t,n){r.forEach(e.nodes(),(function(e){t.node(e).rank+=n}))}e.exports=function(e){var t,n,r=new i({directed:!1}),u=e.nodes()[0],c=e.nodeCount();for(r.setNode(u,{});a(r,e)<c;)t=s(r,e),n=r.hasNode(t.v)?o(e,t):-o(e,t),l(r,e,n);return r}},function(e,t,n){var r=n(48),i={getId:function(e){return e.id||e.name},getHGap:function(e){return e.hgap||18},getVGap:function(e){return e.vgap||18},getChildren:function(e){return e.children},getHeight:function(e){return e.height||36},getWidth:function(e){var t=e.label||" ";return e.width||18*t.split("").length}};function o(e,t){var n=this;if(n.vgap=n.hgap=0,e instanceof o)return e;n.data=e;var r=t.getHGap(e),i=t.getVGap(e);return n.width=t.getWidth(e),n.height=t.getHeight(e),n.id=t.getId(e),n.x=n.y=0,n.depth=0,n.children||(n.children=[]),n.addGap(r,i),n}r.assign(o.prototype,{isRoot:function(){return 0===this.depth},isLeaf:function(){return 0===this.children.length},addGap:function(e,t){this.hgap+=e,this.vgap+=t,this.width+=2*e,this.height+=2*t},eachNode:function(e){for(var t,n=[this];t=n.shift();)e(t),n=t.children.concat(n)},DFTraverse:function(e){this.eachNode(e)},BFTraverse:function(e){for(var t,n=[this];t=n.shift();)e(t),n=n.concat(t.children)},getBoundingBox:function(){var e={left:Number.MAX_VALUE,top:Number.MAX_VALUE,width:0,height:0};return this.eachNode((function(t){e.left=Math.min(e.left,t.x),e.top=Math.min(e.top,t.y),e.width=Math.max(e.width,t.x+t.width),e.height=Math.max(e.height,t.y+t.height)})),e},translate:function(e,t){void 0===e&&(e=0),void 0===t&&(t=0),this.eachNode((function(n){n.x+=e,n.y+=t}))},right2left:function(){var e=this.getBoundingBox();this.eachNode((function(t){t.x=t.x-2*(t.x-e.left)-t.width})),this.translate(e.width,0)},bottom2top:function(){var e=this.getBoundingBox();this.eachNode((function(t){t.y=t.y-2*(t.y-e.top)-t.height})),this.translate(0,e.height)}}),e.exports=function(e,t,n){void 0===t&&(t={});var a,s=new o(e,t=r.assign({},i,t)),l=[s];if(!n&&!e.collapsed)for(;a=l.shift();)if(!a.data.collapsed){var u=t.getChildren(a.data),c=u?u.length:0;if(a.children=new Array(c),u&&c)for(var d=0;d<c;d++){var h=new o(u[d],t);a.children[d]=h,l.push(h),h.parent=a,h.depth=a.depth+1}}return s}},function(e,t,n){var r=n(226);e.exports=function(e,t){for(var n=r(e.data,t,!0),i=r(e.data,t,!0),o=e.children.length,a=Math.round(o/2),s=t.getSide||function(e,t){return t<a?"right":"left"},l=0;l<o;l++){var u=e.children[l];"right"===s(u,l)?i.children.push(u):n.children.push(u)}return n.eachNode((function(e){e.isRoot()||(e.side="left")})),i.eachNode((function(e){e.isRoot()||(e.side="right")})),{left:n,right:i}}},function(e,t,n){"use strict";var r={};n.r(r),n.d(r,"easeLinear",(function(){return S})),n.d(r,"easeQuad",(function(){return O})),n.d(r,"easeQuadIn",(function(){return T})),n.d(r,"easeQuadOut",(function(){return E})),n.d(r,"easeQuadInOut",(function(){return O})),n.d(r,"easeCubic",(function(){return D})),n.d(r,"easeCubicIn",(function(){return k})),n.d(r,"easeCubicOut",(function(){return L})),n.d(r,"easeCubicInOut",(function(){return D})),n.d(r,"easePoly",(function(){return j})),n.d(r,"easePolyIn",(function(){return P})),n.d(r,"easePolyOut",(function(){return N})),n.d(r,"easePolyInOut",(function(){return j})),n.d(r,"easeSin",(function(){return H})),n.d(r,"easeSinIn",(function(){return F})),n.d(r,"easeSinOut",(function(){return B})),n.d(r,"easeSinInOut",(function(){return H})),n.d(r,"easeExp",(function(){return W})),n.d(r,"easeExpIn",(function(){return V})),n.d(r,"easeExpOut",(function(){return Y})),n.d(r,"easeExpInOut",(function(){return W})),n.d(r,"easeCircle",(function(){return q})),n.d(r,"easeCircleIn",(function(){return G})),n.d(r,"easeCircleOut",(function(){return X})),n.d(r,"easeCircleInOut",(function(){return q})),n.d(r,"easeBounce",(function(){return K})),n.d(r,"easeBounceIn",(function(){return Q})),n.d(r,"easeBounceOut",(function(){return K})),n.d(r,"easeBounceInOut",(function(){return J})),n.d(r,"easeBack",(function(){return te})),n.d(r,"easeBackIn",(function(){return $})),n.d(r,"easeBackOut",(function(){return ee})),n.d(r,"easeBackInOut",(function(){return te})),n.d(r,"easeElastic",(function(){return ie})),n.d(r,"easeElasticIn",(function(){return re})),n.d(r,"easeElasticOut",(function(){return ie})),n.d(r,"easeElasticInOut",(function(){return oe}));var i=n(1),o=n(132),a=n(100),s=n(9),l=n(0);function u(e){return(u="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}var c,d,h=0,f=0,p=0,g=0,v=0,m=0,y="object"===("undefined"==typeof performance?"undefined":u(performance))&&performance.now?performance:Date,b="object"===("undefined"==typeof window?"undefined":u(window))&&window.requestAnimationFrame?window.requestAnimationFrame.bind(window):function(e){setTimeout(e,17)};function C(){return v||(b(A),v=y.now()+m)}function A(){v=0}function _(){this._call=this._time=this._next=null}function I(e,t,n){var r=new _;return r.restart(e,t,n),r}function x(){v=(g=y.now())+m,h=f=0;try{!function(){C(),++h;for(var e,t=c;t;)(e=v-t._time)>=0&&t._call.call(null,e),t=t._next;--h}()}finally{h=0,function(){for(var e,t,n=c,r=1/0;n;)n._call?(r>n._time&&(r=n._time),e=n,n=n._next):(t=n._next,n._next=null,n=e?e._next=t:c=t);d=e,M(r)}(),v=0}}function w(){var e=y.now(),t=e-g;t>1e3&&(m-=t,g=e)}function M(e){h||(f&&(f=clearTimeout(f)),e-v>24?(e<1/0&&(f=setTimeout(x,e-y.now()-m)),p&&(p=clearInterval(p))):(p||(g=y.now(),p=setInterval(w,1e3)),h=1,b(x)))}function S(e){return+e}function T(e){return e*e}function E(e){return e*(2-e)}function O(e){return((e*=2)<=1?e*e:--e*(2-e)+1)/2}function k(e){return e*e*e}function L(e){return--e*e*e+1}function D(e){return((e*=2)<=1?e*e*e:(e-=2)*e*e+2)/2}_.prototype=I.prototype={constructor:_,restart:function(e,t,n){if("function"!=typeof e)throw new TypeError("callback is not a function");n=(null==n?C():+n)+(null==t?0:+t),this._next||d===this||(d?d._next=this:c=this,d=this),this._call=e,this._time=n,M()},stop:function(){this._call&&(this._call=null,this._time=1/0,M())}};var P=function e(t){function n(e){return Math.pow(e,t)}return t=+t,n.exponent=e,n}(3),N=function e(t){function n(e){return 1-Math.pow(1-e,t)}return t=+t,n.exponent=e,n}(3),j=function e(t){function n(e){return((e*=2)<=1?Math.pow(e,t):2-Math.pow(2-e,t))/2}return t=+t,n.exponent=e,n}(3),R=Math.PI,z=R/2;function F(e){return 1==+e?1:1-Math.cos(e*z)}function B(e){return Math.sin(e*z)}function H(e){return(1-Math.cos(R*e))/2}function U(e){return 1.0009775171065494*(Math.pow(2,-10*e)-.0009765625)}function V(e){return U(1-+e)}function Y(e){return 1-U(e)}function W(e){return((e*=2)<=1?U(1-e):2-U(e-1))/2}function G(e){return 1-Math.sqrt(1-e*e)}function X(e){return Math.sqrt(1- --e*e)}function q(e){return((e*=2)<=1?1-Math.sqrt(1-e*e):Math.sqrt(1-(e-=2)*e)+1)/2}var Z=7.5625;function Q(e){return 1-K(1-e)}function K(e){return(e=+e)<4/11?Z*e*e:e<8/11?Z*(e-=6/11)*e+3/4:e<10/11?Z*(e-=9/11)*e+15/16:Z*(e-=21/22)*e+63/64}function J(e){return((e*=2)<=1?1-K(1-e):K(e-1)+1)/2}var $=function e(t){function n(e){return(e=+e)*e*(t*(e-1)+e)}return t=+t,n.overshoot=e,n}(1.70158),ee=function e(t){function n(e){return--e*e*((e+1)*t+e)+1}return t=+t,n.overshoot=e,n}(1.70158),te=function e(t){function n(e){return((e*=2)<1?e*e*((t+1)*e-t):(e-=2)*e*((t+1)*e+t)+2)/2}return t=+t,n.overshoot=e,n}(1.70158),ne=2*Math.PI,re=function e(t,n){var r=Math.asin(1/(t=Math.max(1,t)))*(n/=ne);function i(e){return t*U(- --e)*Math.sin((r-e)/n)}return i.amplitude=function(t){return e(t,n*ne)},i.period=function(n){return e(t,n)},i}(1,.3),ie=function e(t,n){var r=Math.asin(1/(t=Math.max(1,t)))*(n/=ne);function i(e){return 1-t*U(e=+e)*Math.sin((e+r)/n)}return i.amplitude=function(t){return e(t,n*ne)},i.period=function(n){return e(t,n)},i}(1,.3),oe=function e(t,n){var r=Math.asin(1/(t=Math.max(1,t)))*(n/=ne);function i(e){return((e=2*e-1)<0?t*U(-e)*Math.sin((r-e)/n):2-t*U(e)*Math.sin((r+e)/n))/2}return i.amplitude=function(t){return e(t,n*ne)},i.period=function(n){return e(t,n)},i}(1,.3),ae=function(e,t,n){e.prototype=t.prototype=n,n.constructor=e};function se(e,t){var n=Object.create(e.prototype);for(var r in t)n[r]=t[r];return n}function le(){}var ue="\\s*([+-]?\\d+)\\s*",ce="\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)\\s*",de="\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)%\\s*",he=/^#([0-9a-f]{3,8})$/,fe=new RegExp("^rgb\\("+[ue,ue,ue]+"\\)$"),pe=new RegExp("^rgb\\("+[de,de,de]+"\\)$"),ge=new RegExp("^rgba\\("+[ue,ue,ue,ce]+"\\)$"),ve=new RegExp("^rgba\\("+[de,de,de,ce]+"\\)$"),me=new RegExp("^hsl\\("+[ce,de,de]+"\\)$"),ye=new RegExp("^hsla\\("+[ce,de,de,ce]+"\\)$"),be={aliceblue:15792383,antiquewhite:16444375,aqua:65535,aquamarine:8388564,azure:15794175,beige:16119260,bisque:16770244,black:0,blanchedalmond:16772045,blue:255,blueviolet:9055202,brown:10824234,burlywood:14596231,cadetblue:6266528,chartreuse:8388352,chocolate:13789470,coral:16744272,cornflowerblue:6591981,cornsilk:16775388,crimson:14423100,cyan:65535,darkblue:139,darkcyan:35723,darkgoldenrod:12092939,darkgray:11119017,darkgreen:25600,darkgrey:11119017,darkkhaki:12433259,darkmagenta:9109643,darkolivegreen:5597999,darkorange:16747520,darkorchid:10040012,darkred:9109504,darksalmon:15308410,darkseagreen:9419919,darkslateblue:4734347,darkslategray:3100495,darkslategrey:3100495,darkturquoise:52945,darkviolet:9699539,deeppink:16716947,deepskyblue:49151,dimgray:6908265,dimgrey:6908265,dodgerblue:2003199,firebrick:11674146,floralwhite:16775920,forestgreen:2263842,fuchsia:16711935,gainsboro:14474460,ghostwhite:16316671,gold:16766720,goldenrod:14329120,gray:8421504,green:32768,greenyellow:11403055,grey:8421504,honeydew:15794160,hotpink:16738740,indianred:13458524,indigo:4915330,ivory:16777200,khaki:15787660,lavender:15132410,lavenderblush:16773365,lawngreen:8190976,lemonchiffon:16775885,lightblue:11393254,lightcoral:15761536,lightcyan:14745599,lightgoldenrodyellow:16448210,lightgray:13882323,lightgreen:9498256,lightgrey:13882323,lightpink:16758465,lightsalmon:16752762,lightseagreen:2142890,lightskyblue:8900346,lightslategray:7833753,lightslategrey:7833753,lightsteelblue:11584734,lightyellow:16777184,lime:65280,limegreen:3329330,linen:16445670,magenta:16711935,maroon:8388608,mediumaquamarine:6737322,mediumblue:205,mediumorchid:12211667,mediumpurple:9662683,mediumseagreen:3978097,mediumslateblue:8087790,mediumspringgreen:64154,mediumturquoise:4772300,mediumvioletred:13047173,midnightblue:1644912,mintcream:16121850,mistyrose:16770273,moccasin:16770229,navajowhite:16768685,navy:128,oldlace:16643558,olive:8421376,olivedrab:7048739,orange:16753920,orangered:16729344,orchid:14315734,palegoldenrod:15657130,palegreen:10025880,paleturquoise:11529966,palevioletred:14381203,papayawhip:16773077,peachpuff:16767673,peru:13468991,pink:16761035,plum:14524637,powderblue:11591910,purple:8388736,rebeccapurple:6697881,red:16711680,rosybrown:12357519,royalblue:4286945,saddlebrown:9127187,salmon:16416882,sandybrown:16032864,seagreen:3050327,seashell:16774638,sienna:10506797,silver:12632256,skyblue:8900331,slateblue:6970061,slategray:7372944,slategrey:7372944,snow:16775930,springgreen:65407,steelblue:4620980,tan:13808780,teal:32896,thistle:14204888,tomato:16737095,turquoise:4251856,violet:15631086,wheat:16113331,white:16777215,whitesmoke:16119285,yellow:16776960,yellowgreen:10145074};function Ce(){return this.rgb().formatHex()}function Ae(){return this.rgb().formatRgb()}function _e(e){var t,n;return e=(e+"").trim().toLowerCase(),(t=he.exec(e))?(n=t[1].length,t=parseInt(t[1],16),6===n?Ie(t):3===n?new Se(t>>8&15|t>>4&240,t>>4&15|240&t,(15&t)<<4|15&t,1):8===n?xe(t>>24&255,t>>16&255,t>>8&255,(255&t)/255):4===n?xe(t>>12&15|t>>8&240,t>>8&15|t>>4&240,t>>4&15|240&t,((15&t)<<4|15&t)/255):null):(t=fe.exec(e))?new Se(t[1],t[2],t[3],1):(t=pe.exec(e))?new Se(255*t[1]/100,255*t[2]/100,255*t[3]/100,1):(t=ge.exec(e))?xe(t[1],t[2],t[3],t[4]):(t=ve.exec(e))?xe(255*t[1]/100,255*t[2]/100,255*t[3]/100,t[4]):(t=me.exec(e))?ke(t[1],t[2]/100,t[3]/100,1):(t=ye.exec(e))?ke(t[1],t[2]/100,t[3]/100,t[4]):be.hasOwnProperty(e)?Ie(be[e]):"transparent"===e?new Se(NaN,NaN,NaN,0):null}function Ie(e){return new Se(e>>16&255,e>>8&255,255&e,1)}function xe(e,t,n,r){return r<=0&&(e=t=n=NaN),new Se(e,t,n,r)}function we(e){return e instanceof le||(e=_e(e)),e?new Se((e=e.rgb()).r,e.g,e.b,e.opacity):new Se}function Me(e,t,n,r){return 1===arguments.length?we(e):new Se(e,t,n,null==r?1:r)}function Se(e,t,n,r){this.r=+e,this.g=+t,this.b=+n,this.opacity=+r}function Te(){return"#"+Oe(this.r)+Oe(this.g)+Oe(this.b)}function Ee(){var e=this.opacity;return(1===(e=isNaN(e)?1:Math.max(0,Math.min(1,e)))?"rgb(":"rgba(")+Math.max(0,Math.min(255,Math.round(this.r)||0))+", "+Math.max(0,Math.min(255,Math.round(this.g)||0))+", "+Math.max(0,Math.min(255,Math.round(this.b)||0))+(1===e?")":", "+e+")")}function Oe(e){return((e=Math.max(0,Math.min(255,Math.round(e)||0)))<16?"0":"")+e.toString(16)}function ke(e,t,n,r){return r<=0?e=t=n=NaN:n<=0||n>=1?e=t=NaN:t<=0&&(e=NaN),new De(e,t,n,r)}function Le(e){if(e instanceof De)return new De(e.h,e.s,e.l,e.opacity);if(e instanceof le||(e=_e(e)),!e)return new De;if(e instanceof De)return e;var t=(e=e.rgb()).r/255,n=e.g/255,r=e.b/255,i=Math.min(t,n,r),o=Math.max(t,n,r),a=NaN,s=o-i,l=(o+i)/2;return s?(a=t===o?(n-r)/s+6*(n<r):n===o?(r-t)/s+2:(t-n)/s+4,s/=l<.5?o+i:2-o-i,a*=60):s=l>0&&l<1?0:a,new De(a,s,l,e.opacity)}function De(e,t,n,r){this.h=+e,this.s=+t,this.l=+n,this.opacity=+r}function Pe(e,t,n){return 255*(e<60?t+(n-t)*e/60:e<180?n:e<240?t+(n-t)*(240-e)/60:t)}function Ne(e,t,n,r,i){var o=e*e,a=o*e;return((1-3*e+3*o-a)*t+(4-6*o+3*a)*n+(1+3*e+3*o-3*a)*r+a*i)/6}ae(le,_e,{copy:function(e){return Object.assign(new this.constructor,this,e)},displayable:function(){return this.rgb().displayable()},hex:Ce,formatHex:Ce,formatHsl:function(){return Le(this).formatHsl()},formatRgb:Ae,toString:Ae}),ae(Se,Me,se(le,{brighter:function(e){return e=null==e?1/.7:Math.pow(1/.7,e),new Se(this.r*e,this.g*e,this.b*e,this.opacity)},darker:function(e){return e=null==e?.7:Math.pow(.7,e),new Se(this.r*e,this.g*e,this.b*e,this.opacity)},rgb:function(){return this},displayable:function(){return-.5<=this.r&&this.r<255.5&&-.5<=this.g&&this.g<255.5&&-.5<=this.b&&this.b<255.5&&0<=this.opacity&&this.opacity<=1},hex:Te,formatHex:Te,formatRgb:Ee,toString:Ee})),ae(De,(function(e,t,n,r){return 1===arguments.length?Le(e):new De(e,t,n,null==r?1:r)}),se(le,{brighter:function(e){return e=null==e?1/.7:Math.pow(1/.7,e),new De(this.h,this.s,this.l*e,this.opacity)},darker:function(e){return e=null==e?.7:Math.pow(.7,e),new De(this.h,this.s,this.l*e,this.opacity)},rgb:function(){var e=this.h%360+360*(this.h<0),t=isNaN(e)||isNaN(this.s)?0:this.s,n=this.l,r=n+(n<.5?n:1-n)*t,i=2*n-r;return new Se(Pe(e>=240?e-240:e+120,i,r),Pe(e,i,r),Pe(e<120?e+240:e-120,i,r),this.opacity)},displayable:function(){return(0<=this.s&&this.s<=1||isNaN(this.s))&&0<=this.l&&this.l<=1&&0<=this.opacity&&this.opacity<=1},formatHsl:function(){var e=this.opacity;return(1===(e=isNaN(e)?1:Math.max(0,Math.min(1,e)))?"hsl(":"hsla(")+(this.h||0)+", "+100*(this.s||0)+"%, "+100*(this.l||0)+"%"+(1===e?")":", "+e+")")}}));var je=function(e){return function(){return e}};function Re(e,t){var n=t-e;return n?function(e,t){return function(n){return e+n*t}}(e,n):je(isNaN(e)?t:e)}var ze=function e(t){var n=function(e){return 1==(e=+e)?Re:function(t,n){return n-t?function(e,t,n){return e=Math.pow(e,n),t=Math.pow(t,n)-e,n=1/n,function(r){return Math.pow(e+r*t,n)}}(t,n,e):je(isNaN(t)?n:t)}}(t);function r(e,t){var r=n((e=Me(e)).r,(t=Me(t)).r),i=n(e.g,t.g),o=n(e.b,t.b),a=Re(e.opacity,t.opacity);return function(t){return e.r=r(t),e.g=i(t),e.b=o(t),e.opacity=a(t),e+""}}return r.gamma=e,r}(1);function Fe(e){return function(t){var n,r,i=t.length,o=new Array(i),a=new Array(i),s=new Array(i);for(n=0;n<i;++n)r=Me(t[n]),o[n]=r.r||0,a[n]=r.g||0,s[n]=r.b||0;return o=e(o),a=e(a),s=e(s),r.opacity=1,function(e){return r.r=o(e),r.g=a(e),r.b=s(e),r+""}}}Fe((function(e){var t=e.length-1;return function(n){var r=n<=0?n=0:n>=1?(n=1,t-1):Math.floor(n*t),i=e[r],o=e[r+1],a=r>0?e[r-1]:2*i-o,s=r<t-1?e[r+2]:2*o-i;return Ne((n-r/t)*t,a,i,o,s)}})),Fe((function(e){var t=e.length;return function(n){var r=Math.floor(((n%=1)<0?++n:n)*t),i=e[(r+t-1)%t],o=e[r%t],a=e[(r+1)%t],s=e[(r+2)%t];return Ne((n-r/t)*t,i,o,a,s)}}));var Be=function(e,t){t||(t=[]);var n,r=e?Math.min(t.length,e.length):0,i=t.slice();return function(o){for(n=0;n<r;++n)i[n]=e[n]*(1-o)+t[n]*o;return i}};function He(e){return ArrayBuffer.isView(e)&&!(e instanceof DataView)}function Ue(e,t){var n,r=t?t.length:0,i=e?Math.min(r,e.length):0,o=new Array(i),a=new Array(r);for(n=0;n<i;++n)o[n]=Ke(e[n],t[n]);for(;n<r;++n)a[n]=t[n];return function(e){for(n=0;n<i;++n)a[n]=o[n](e);return a}}var Ve=function(e,t){var n=new Date;return e=+e,t=+t,function(r){return n.setTime(e*(1-r)+t*r),n}},Ye=function(e,t){return e=+e,t=+t,function(n){return e*(1-n)+t*n}};function We(e){return(We="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}var Ge=function(e,t){var n,r={},i={};for(n in null!==e&&"object"===We(e)||(e={}),null!==t&&"object"===We(t)||(t={}),t)n in e?r[n]=Ke(e[n],t[n]):i[n]=t[n];return function(e){for(n in r)i[n]=r[n](e);return i}},Xe=/[-+]?(?:\d+\.?\d*|\.?\d+)(?:[eE][-+]?\d+)?/g,qe=new RegExp(Xe.source,"g"),Ze=function(e,t){var n,r,i,o=Xe.lastIndex=qe.lastIndex=0,a=-1,s=[],l=[];for(e+="",t+="";(n=Xe.exec(e))&&(r=qe.exec(t));)(i=r.index)>o&&(i=t.slice(o,i),s[a]?s[a]+=i:s[++a]=i),(n=n[0])===(r=r[0])?s[a]?s[a]+=r:s[++a]=r:(s[++a]=null,l.push({i:a,x:Ye(n,r)})),o=qe.lastIndex;return o<t.length&&(i=t.slice(o),s[a]?s[a]+=i:s[++a]=i),s.length<2?l[0]?function(e){return function(t){return e(t)+""}}(l[0].x):function(e){return function(){return e}}(t):(t=l.length,function(e){for(var n,r=0;r<t;++r)s[(n=l[r]).i]=n.x(e);return s.join("")})};function Qe(e){return(Qe="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}var Ke=function(e,t){var n,r=Qe(t);return null==t||"boolean"===r?je(t):("number"===r?Ye:"string"===r?(n=_e(t))?(t=n,ze):Ze:t instanceof _e?ze:t instanceof Date?Ve:He(t)?Be:Array.isArray(t)?Ue:"function"!=typeof t.valueOf&&"function"!=typeof t.toString||isNaN(t)?Ge:Ye)(e,t)},Je=n(37),$e=[1,0,0,0,1,0,0,0,1];function et(e,t,n){var i,o=t.startTime;if(n<o+t.delay||t._paused)return!1;var a=t.duration,s=t.easing;if(n=n-o-t.delay,t.repeat)i=n%a/a,i=r[s](i);else{if(!((i=n/a)<1))return t.onFrame?e.attr(t.onFrame(1)):e.attr(t.toAttrs),!0;i=r[s](i)}if(t.onFrame){var u=t.onFrame(i);e.attr(u)}else!function(e,t,n){var r={},i=t.fromAttrs,o=t.toAttrs;if(!e.destroyed){var a,s,u,c;for(var d in o)if(!Object(l.isEqual)(i[d],o[d]))if("path"===d){var h=o[d],f=i[d];h.length>f.length?(h=Je.parsePathString(o[d]),f=Je.parsePathString(i[d]),f=Je.fillPathByDiff(f,h),f=Je.formatPath(f,h),t.fromAttrs.path=f,t.toAttrs.path=h):t.pathFormatted||(h=Je.parsePathString(o[d]),f=Je.parsePathString(i[d]),f=Je.formatPath(f,h),t.fromAttrs.path=f,t.toAttrs.path=h,t.pathFormatted=!0),r[d]=[];for(var p=0;p<h.length;p++){for(var g=h[p],v=f[p],m=[],y=0;y<g.length;y++)Object(l.isNumber)(g[y])&&v&&Object(l.isNumber)(v[y])?(a=Ke(v[y],g[y]),m.push(a(n))):m.push(g[y]);r[d].push(m)}}else if("matrix"===d){var b=(u=i[d]||$e,(He(c=o[d]||$e)?Be:Ue)(u,c))(n);r[d]=b}else["fill","stroke","fillStyle","strokeStyle"].includes(d)&&(s=o[d],/^[r,R,L,l]{1}[\s]*\(/.test(s))?r[d]=o[d]:Object(l.isFunction)(o[d])||(a=Ke(i[d],o[d]),r[d]=a(n));e.attr(r)}}(e,t,i);return!1}var tt=function(){function e(e){this.animators=[],this.current=0,this.timer=null,this.canvas=e}return e.prototype.initTimer=function(){var e,t,n,r=this;this.timer=I((function(i){if(r.current=i,r.animators.length>0){for(var o=r.animators.length-1;o>=0;o--)if((e=r.animators[o]).destroyed)r.removeAnimator(o);else{if(!e.isAnimatePaused())for(var a=(t=e.get("animations")).length-1;a>=0;a--)n=t[a],et(e,n,i)&&(t.splice(a,1),n.callback&&n.callback());0===t.length&&r.removeAnimator(o)}r.canvas.get("autoDraw")||r.canvas.draw()}}))},e.prototype.addAnimator=function(e){this.animators.push(e)},e.prototype.removeAnimator=function(e){this.animators.splice(e,1)},e.prototype.isAnimating=function(){return!!this.animators.length},e.prototype.stop=function(){this.timer&&this.timer.stop()},e.prototype.stopAllAnimations=function(e){void 0===e&&(e=!0),this.animators.forEach((function(t){t.stopAnimate(e)})),this.animators=[],this.canvas.draw()},e.prototype.getTime=function(){return this.current},e}(),nt=n(96),rt=["mousedown","mouseup","dblclick","mouseout","mouseover","mousemove","mouseleave","mouseenter","touchstart","touchmove","touchend","dragenter","dragover","dragleave","drop","contextmenu","mousewheel"];function it(e,t,n){n.name=t,n.target=e,n.currentTarget=e,n.delegateTarget=e,e.emit(t,n)}function ot(e,t,n){if(n.bubbles){var r=void 0,i=!1;if("mouseenter"===t?(r=n.fromShape,i=!0):"mouseleave"===t&&(i=!0,r=n.toShape),e.isCanvas()&&i)return;if(r&&Object(s.g)(e,r))return void(n.bubbles=!1);n.name=t,n.currentTarget=e,n.delegateTarget=e,e.emit(t,n)}}var at=function(){function e(e){var t=this;this.draggingShape=null,this.dragging=!1,this.currentShape=null,this.mousedownShape=null,this.mousedownPoint=null,this._eventCallback=function(e){var n=e.type;t._triggerEvent(n,e)},this._onDocumentMove=function(e){if(t.canvas.get("el")!==e.target&&(t.dragging||t.currentShape)){var n=t._getPointInfo(e);t.dragging&&t._emitEvent("drag",e,n,t.draggingShape)}},this._onDocumentMouseUp=function(e){if(t.canvas.get("el")!==e.target&&t.dragging){var n=t._getPointInfo(e);t.draggingShape&&t._emitEvent("drop",e,n,null),t._emitEvent("dragend",e,n,t.draggingShape),t._afterDrag(t.draggingShape,n,e)}},this.canvas=e.canvas}return e.prototype.init=function(){this._bindEvents()},e.prototype._bindEvents=function(){var e=this,t=this.canvas.get("el");Object(s.a)(rt,(function(n){t.addEventListener(n,e._eventCallback)})),document&&(document.addEventListener("mousemove",this._onDocumentMove),document.addEventListener("mouseup",this._onDocumentMouseUp))},e.prototype._clearEvents=function(){var e=this,t=this.canvas.get("el");Object(s.a)(rt,(function(n){t.removeEventListener(n,e._eventCallback)})),document&&(document.removeEventListener("mousemove",this._onDocumentMove),document.removeEventListener("mouseup",this._onDocumentMouseUp))},e.prototype._getEventObj=function(e,t,n,r,i,o){var a=new nt.a(e,t);return a.fromShape=i,a.toShape=o,a.x=n.x,a.y=n.y,a.clientX=n.clientX,a.clientY=n.clientY,a.propagationPath.push(r),a},e.prototype._getShape=function(e,t){return this.canvas.getShape(e.x,e.y,t)},e.prototype._getPointInfo=function(e){var t=this.canvas,n=t.getClientByEvent(e),r=t.getPointByEvent(e);return{x:r.x,y:r.y,clientX:n.x,clientY:n.y}},e.prototype._triggerEvent=function(e,t){var n=this._getPointInfo(t),r=this._getShape(n,t),i=this["_on"+e],o=!1;if(i)i.call(this,n,r,t);else{var a=this.currentShape;"mouseenter"===e||"dragenter"===e||"mouseover"===e?(this._emitEvent(e,t,n,null,null,r),r&&this._emitEvent(e,t,n,r,null,r),"mouseenter"===e&&this.draggingShape&&this._emitEvent("dragenter",t,n,null)):"mouseleave"===e||"dragleave"===e||"mouseout"===e?(o=!0,a&&this._emitEvent(e,t,n,a,a,null),this._emitEvent(e,t,n,null,a,null),"mouseleave"===e&&this.draggingShape&&this._emitEvent("dragleave",t,n,null)):this._emitEvent(e,t,n,r,null,null)}if(o||(this.currentShape=r),r&&!r.get("destroyed")){var s=this.canvas;s.get("el").style.cursor=r.attr("cursor")||s.get("cursor")}},e.prototype._onmousedown=function(e,t,n){0===n.button&&(this.mousedownShape=t,this.mousedownPoint=e,this.mousedownTimeStamp=n.timeStamp),this._emitEvent("mousedown",n,e,t,null,null)},e.prototype._emitMouseoverEvents=function(e,t,n,r){var i=this.canvas.get("el");n!==r&&(n&&(this._emitEvent("mouseout",e,t,n,n,r),this._emitEvent("mouseleave",e,t,n,n,r),r&&!r.get("destroyed")||(i.style.cursor=this.canvas.get("cursor"))),r&&(this._emitEvent("mouseover",e,t,r,n,r),this._emitEvent("mouseenter",e,t,r,n,r)))},e.prototype._emitDragoverEvents=function(e,t,n,r,i){r?(r!==n&&(n&&this._emitEvent("dragleave",e,t,n,n,r),this._emitEvent("dragenter",e,t,r,n,r)),i||this._emitEvent("dragover",e,t,r)):n&&this._emitEvent("dragleave",e,t,n,n,r),i&&this._emitEvent("dragover",e,t,r)},e.prototype._afterDrag=function(e,t,n){e&&(e.set("capture",!0),this.draggingShape=null),this.dragging=!1;var r=this._getShape(t,n);r!==e&&this._emitMouseoverEvents(n,t,e,r),this.currentShape=r},e.prototype._onmouseup=function(e,t,n){if(0===n.button){var r=this.draggingShape;this.dragging?(r&&this._emitEvent("drop",n,e,t),this._emitEvent("dragend",n,e,r),this._afterDrag(r,e,n)):(this._emitEvent("mouseup",n,e,t),t===this.mousedownShape&&this._emitEvent("click",n,e,t),this.mousedownShape=null,this.mousedownPoint=null)}},e.prototype._ondragover=function(e,t,n){n.preventDefault();var r=this.currentShape;this._emitDragoverEvents(n,e,r,t,!0)},e.prototype._onmousemove=function(e,t,n){var r=this.canvas,i=this.currentShape,o=this.draggingShape;if(this.dragging)o&&this._emitDragoverEvents(n,e,i,t,!1),this._emitEvent("drag",n,e,o);else{var a=this.mousedownPoint;if(a){var s=this.mousedownShape,l=n.timeStamp-this.mousedownTimeStamp,u=a.clientX-e.clientX,c=a.clientY-e.clientY;l>120||u*u+c*c>40?s&&s.get("draggable")?((o=this.mousedownShape).set("capture",!1),this.draggingShape=o,this.dragging=!0,this._emitEvent("dragstart",n,e,o),this.mousedownShape=null,this.mousedownPoint=null):!s&&r.get("draggable")?(this.dragging=!0,this._emitEvent("dragstart",n,e,null),this.mousedownShape=null,this.mousedownPoint=null):(this._emitMouseoverEvents(n,e,i,t),this._emitEvent("mousemove",n,e,t)):(this._emitMouseoverEvents(n,e,i,t),this._emitEvent("mousemove",n,e,t))}else this._emitMouseoverEvents(n,e,i,t),this._emitEvent("mousemove",n,e,t)}},e.prototype._emitEvent=function(e,t,n,r,i,o){var a=this._getEventObj(e,t,n,r,i,o);if(r){a.shape=r,it(r,e,a);for(var s=r.getParent();s;)s.emitDelegation(e,a),a.propagationStopped||ot(s,e,a),a.propagationPath.push(s),s=s.getParent()}else it(this.canvas,e,a)},e.prototype.destroy=function(){this._clearEvents(),this.canvas=null,this.currentShape=null,this.draggingShape=null,this.mousedownPoint=null,this.mousedownShape=null,this.mousedownTimeStamp=null},e}(),st=Object(o.detect)(),lt=st&&"firefox"===st.name,ut=function(e){function t(t){var n=e.call(this,t)||this;return n.initContainer(),n.initDom(),n.initEvents(),n.initTimeline(),n}return Object(i.__extends)(t,e),t.prototype.getDefaultCfg=function(){var t=e.prototype.getDefaultCfg.call(this);return t.cursor="default",t.supportCSSTransform=!1,t},t.prototype.initContainer=function(){var e=this.get("container");Object(s.h)(e)&&(e=document.getElementById(e),this.set("container",e))},t.prototype.initDom=function(){var e=this.createDom();this.set("el",e),this.get("container").appendChild(e),this.setDOMSize(this.get("width"),this.get("height"))},t.prototype.initEvents=function(){var e=new at({canvas:this});e.init(),this.set("eventController",e)},t.prototype.initTimeline=function(){var e=new tt(this);this.set("timeline",e)},t.prototype.setDOMSize=function(e,t){var n=this.get("el");s.c&&(n.style.width=e+"px",n.style.height=t+"px")},t.prototype.changeSize=function(e,t){this.setDOMSize(e,t),this.set("width",e),this.set("height",t),this.onCanvasChange("changeSize")},t.prototype.getRenderer=function(){return this.get("renderer")},t.prototype.getCursor=function(){return this.get("cursor")},t.prototype.setCursor=function(e){this.set("cursor",e);var t=this.get("el");s.c&&t&&(t.style.cursor=e)},t.prototype.getPointByEvent=function(e){if(this.get("supportCSSTransform")){if(lt&&!Object(s.e)(e.layerX)&&e.layerX!==e.offsetX)return{x:e.layerX,y:e.layerY};if(!Object(s.e)(e.offsetX))return{x:e.offsetX,y:e.offsetY}}var t=this.getClientByEvent(e),n=t.x,r=t.y;return this.getPointByClient(n,r)},t.prototype.getClientByEvent=function(e){var t=e;return e.touches&&(t="touchend"===e.type?e.changedTouches[0]:e.touches[0]),{x:t.clientX,y:t.clientY}},t.prototype.getPointByClient=function(e,t){var n=this.get("el").getBoundingClientRect();return{x:e-n.left,y:t-n.top}},t.prototype.getClientByPoint=function(e,t){var n=this.get("el").getBoundingClientRect();return{x:e+n.left,y:t+n.top}},t.prototype.draw=function(){},t.prototype.removeDom=function(){var e=this.get("el");e.parentNode.removeChild(e)},t.prototype.clearEvents=function(){this.get("eventController").destroy()},t.prototype.isCanvas=function(){return!0},t.prototype.getParent=function(){return null},t.prototype.destroy=function(){var t=this.get("timeline");this.get("destroyed")||(this.clear(),t&&t.stop(),this.clearEvents(),this.removeDom(),e.prototype.destroy.call(this))},t}(a.a);t.a=ut},function(e){e.exports=JSON.parse('{"a":"3.8.5"}')},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=function(){function e(e,t){this.bubbles=!0,this.target=null,this.currentTarget=null,this.delegateTarget=null,this.delegateObject=null,this.defaultPrevented=!1,this.propagationStopped=!1,this.shape=null,this.fromShape=null,this.toShape=null,this.propagationPath=[],this.type=e,this.name=e,this.originalEvent=t,this.timeStamp=t.timeStamp}return e.prototype.preventDefault=function(){this.defaultPrevented=!0,this.originalEvent.preventDefault&&this.originalEvent.preventDefault()},e.prototype.stopPropagation=function(){this.propagationStopped=!0},e.prototype.toString=function(){return"[Event (type="+this.type+")]"},e.prototype.save=function(){},e.prototype.restore=function(){},e}();t.default=r},function(e,t,n){"use strict";e.exports=function(e,t,n,r){try{try{var i;try{i=new window.Blob([e])}catch(t){(i=new(window.BlobBuilder||window.WebKitBlobBuilder||window.MozBlobBuilder||window.MSBlobBuilder)).append(e),i=i.getBlob()}var o=window.URL||window.webkitURL,a=o.createObjectURL(i),s=new window[t](a,n);return o.revokeObjectURL(a),s}catch(r){return new window[t]("data:application/javascript,".concat(encodeURIComponent(e)),n)}}catch(e){if(!r)throw Error("Inline worker is not supported");return new window[t](r,n)}}},,,,,,,,,,,function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(137),i=n(72);t.default=function(e,t){return void 0===t&&(t=[]),r.default(e,(function(e){return!i.default(t,e)}))}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(14),i=n(138),o=n(4),a=n(58);t.default=function(e,t){if(!o.default(e))return null;var n;if(r.default(t)&&(n=t),a.default(t)&&(n=function(e){return i.default(e,t)}),n)for(var s=0;s<e.length;s+=1)if(n(e[s]))return e[s];return null}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t,n){void 0===n&&(n=0);for(var r=n;r<e.length;r++)if(t(e[r],r))return r;return-1}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(10),i=n(4);t.default=function(e,t){for(var n=null,o=0;o<e.length;o++){var a=e[o][t];if(!r.default(a)){n=i.default(a)?a[0]:a;break}}return n}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(4);t.default=function(e){if(!r.default(e))return[];for(var t=[],n=0;n<e.length;n++)t=t.concat(e[n]);return t}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(4);t.default=function e(t,n){if(void 0===n&&(n=[]),r.default(t))for(var i=0;i<t.length;i+=1)e(t[i],n);else n.push(t);return n}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(4);t.default=function(e){var t=e.filter((function(e){return!isNaN(e)}));if(!t.length)return{min:0,max:0};if(r.default(e[0])){for(var n=[],i=0;i<e.length;i++)n=n.concat(e[i]);t=n}var o=Math.max.apply(null,t);return{min:Math.min.apply(null,t),max:o}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=Array.prototype,i=r.splice,o=r.indexOf;t.default=function(e){for(var t=[],n=1;n<arguments.length;n++)t[n-1]=arguments[n];for(var r=0;r<t.length;r++)for(var a=t[r],s=-1;(s=o.call(e,a))>-1;)i.call(e,s,1);return e}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(2),i=n(4),o=n(58);t.default=function(e,t,n){if(!i.default(e)&&!o.default(e))return e;var a=n;return r.default(e,(function(e,n){a=t(a,e,n)})),a}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(21),i=n(140);t.default=function(e,t){var n=[];if(!r.default(e))return n;for(var o=-1,a=[],s=e.length;++o<s;){var l=e[o];t(l,o,e)&&(n.push(l),a.push(o))}return i.default(e,a),n}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(4),i=n(3),o=n(14);t.default=function(e,t){var n;if(o.default(t))n=function(e,n){return t(e)-t(n)};else{var a=[];i.default(t)?a.push(t):r.default(t)&&(a=t),n=function(e,t){for(var n=0;n<a.length;n+=1){var r=a[n];if(e[r]>t[r])return 1;if(e[r]<t[r])return-1}return 0}}return e.sort(n),e}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(141);t.default=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return r.default([].concat.apply([],e))}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(4),i=n(10);t.default=function(e,t){for(var n=[],o={},a=0;a<e.length;a++){var s=e[a][t];if(!i.default(s)){r.default(s)||(s=[s]);for(var l=0;l<s.length;l++){var u=s[l];o[u]||(n.push(u),o[u]=!0)}}}return n}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(21);t.default=function(e){if(r.default(e))return e[0]}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(21);t.default=function(e){if(r.default(e))return e[e.length-1]}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(4),i=n(3);t.default=function(e,t){return!(!r.default(e)&&!i.default(e))&&e[0]===t}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(4),i=n(3);t.default=function(e,t){return!(!r.default(e)&&!i.default(e))&&e[e.length-1]===t}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){for(var n=0;n<e.length;n++)if(!t(e[n],n))return!1;return!0}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){for(var n=0;n<e.length;n++)if(t(e[n],n))return!0;return!1}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(142);t.default=function(e,t){if(!t)return[e];var n=r.default(e,t),i=[];for(var o in n)i.push(n[o]);return i}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){return e["_wrap_"+t]}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r={};t.default=function(e){var t=r[e];if(!t){for(var n=e.toString(16),i=n.length;i<6;i++)n="0"+n;t="#"+n,r[e]=t}return t}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(4);t.default=function(e){var t=0,n=0,i=0,o=0;return r.default(e)?1===e.length?t=n=i=o=e[0]:2===e.length?(t=i=e[0],n=o=e[1]):3===e.length?(t=e[0],n=o=e[1],i=e[2]):(t=e[0],n=e[1],i=e[2],o=e[3]):t=n=i=o=e,{r1:t,r2:n,r3:i,r4:o}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t,n){return e<t?t:e>n?n:e}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){var n=t.toString(),r=n.indexOf(".");if(-1===r)return Math.round(e);var i=n.substr(r+1).length;return i>20&&(i=20),parseFloat(e.toFixed(i))}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(11);t.default=function(e){return r.default(e)&&e%1!=0}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(11);t.default=function(e){return r.default(e)&&e%2==0}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(11),i=Number.isInteger?Number.isInteger:function(e){return r.default(e)&&e%1==0};t.default=i},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(11);t.default=function(e){return r.default(e)&&e<0}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(11);t.default=function(e){return r.default(e)&&e%2!=0}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(11);t.default=function(e){return r.default(e)&&e>0}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(2),i=n(4),o=n(14);t.default=function(e,t){if(i.default(e)){var n,a,s=e[0];return n=o.default(t)?t(e[0]):e[0][t],r.default(e,(function(e){(a=o.default(t)?t(e):e[t])>n&&(s=e,n=a)})),s}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(2),i=n(4),o=n(14);t.default=function(e,t){if(i.default(e)){var n,a,s=e[0];return n=o.default(t)?t(e[0]):e[0][t],r.default(e,(function(e){(a=o.default(t)?t(e):e[t])<n&&(s=e,n=a)})),s}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=180/Math.PI;t.default=function(e){return r*e}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=parseInt},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(2);t.default=r.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(143);t.default=r.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(72),i=n(144);t.default=function(e,t){return r.default(i.default(e),t)}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(59);t.default=function(e){return r.default(e).toLowerCase()}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(59);t.default=function(e){var t=r.default(e);return t.charAt(0).toLowerCase()+t.substring(1)}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){return e&&t?e.replace(/\\?\{([^{}]+)\}/g,(function(e,n){return"\\"===e.charAt(0)?e.slice(1):void 0===t[n]?"":t[n]})):e}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(59);t.default=function(e){return r.default(e).toUpperCase()}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(27);t.default=function(e){return r.default(e,"Arguments")}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(27);t.default=function(e){return r.default(e,"Boolean")}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(27);t.default=function(e){return r.default(e,"Date")}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(27);t.default=function(e){return r.default(e,"Error")}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(11);t.default=function(e){return r.default(e)&&isFinite(e)}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){return null===e}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(27);t.default=function(e){return r.default(e,"RegExp")}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){return void 0===e}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){return e instanceof Element||e instanceof HTMLDocument}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(17),i=n(14);t.default=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];for(var n=e[0],o=1;o<e.length;o++){var a=e[o];i.default(a)&&(a=a.prototype),r.default(n.prototype,a)}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t,n){var r;return function(){var i=this,o=arguments,a=function(){r=null,n||e.apply(i,o)},s=n&&!r;clearTimeout(r),r=setTimeout(a,t),s&&e.apply(i,o)}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(14);t.default=function(e,t){if(!r.default(e))throw new TypeError("Expected a function");var n=function n(){for(var r=[],i=0;i<arguments.length;i++)r[i]=arguments[i];var o=t?t.apply(this,r):r[0],a=n.cache;if(a.has(o))return a.get(o);var s=e.apply(this,r);return a.set(o,s),s};return n.cache=new Map,n}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(17),i=n(14);t.default=function(e,t,n,o){i.default(t)||(n=t,t=e,e=function(){});var a=Object.create?function(e,t){return Object.create(e,{constructor:{value:t}})}:function(e,t){function n(){}n.prototype=e;var r=new n;return r.constructor=t,r},s=a(t.prototype,e);return e.prototype=r.default(s,e.prototype),e.superclass=a(t.prototype,t),r.default(s,n),r.default(e,o),e}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(21);t.default=function(e,t){if(!r.default(e))return-1;var n=Array.prototype.indexOf;if(n)return n.call(e,t);for(var i=-1,o=0;o<e.length;o++)if(e[o]===t){i=o;break}return i}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(10),i=n(21),o=n(145),a=n(146),s=Object.prototype.hasOwnProperty;t.default=function(e){if(r.default(e))return!0;if(i.default(e))return!e.length;var t=o.default(e);if("Map"===t||"Set"===t)return!e.size;if(a.default(e))return!Object.keys(e).length;for(var n in e)if(s.call(e,n))return!1;return!0}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(14),i=n(95);t.default=function(e,t,n){return r.default(n)?!!n(e,t):i.default(e,t)}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(21);t.default=function(e,t){if(!r.default(e))return e;for(var n=[],i=0;i<e.length;i++){var o=e[i];n.push(t(o,i))}return n}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(10),i=n(35),o=function(e){return e};t.default=function(e,t){void 0===t&&(t=o);var n={};return i.default(e)&&!r.default(e)&&Object.keys(e).forEach((function(r){n[r]=t(e[r],r)})),n}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(35),i=n(3),o=n(11);t.default=function(e,t,n){var a=e,s=i.default(t)?t.split("."):t;return s.forEach((function(e,t){t<s.length-1?(r.default(a[e])||(a[e]=o.default(s[t+1])?[]:{}),a=a[e]):a[e]=n})),e}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(2),i=n(58),o=Object.prototype.hasOwnProperty;t.default=function(e,t){if(null===e||!i.default(e))return{};var n={};return r.default(t,(function(t){o.call(e,t)&&(n[t]=e[t])})),n}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t,n){var r,i,o,a,s=0;n||(n={});var l=function(){s=!1===n.leading?0:Date.now(),r=null,a=e.apply(i,o),r||(i=o=null)},u=function(){var u=Date.now();s||!1!==n.leading||(s=u);var c=t-(u-s);return i=this,o=arguments,c<=0||c>t?(r&&(clearTimeout(r),r=null),s=u,a=e.apply(i,o),r||(i=o=null)):r||!1===n.trailing||(r=setTimeout(l,c)),a};return u.cancel=function(){clearTimeout(r),s=0,r=i=o=null},u}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(21);t.default=function(e){return r.default(e)?Array.prototype.slice.call(e):[]}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r={};t.default=function(e){return r[e=e||"g"]?r[e]+=1:r[e]=1,e+r[e]}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(){}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){return e}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(10),i=n(21);t.default=function(e){return r.default(e)?0:i.default(e)?e.length:Object.keys(e).length}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=function(){function e(){this.map={}}return e.prototype.has=function(e){return void 0!==this.map[e]},e.prototype.get=function(e,t){var n=this.map[e];return void 0===n?t:n},e.prototype.set=function(e,t){this.map[e]=t},e.prototype.clear=function(){this.map={}},e.prototype.delete=function(e){delete this.map[e]},e.prototype.size=function(){return Object.keys(this.map).length},e}();t.default=r},function(e,t,n){var r=n(312),i=n(313),o={};for(var a in r)r.hasOwnProperty(a)&&(o[r[a]]=a);var s=e.exports={to:{},get:{}};function l(e,t,n){return Math.min(Math.max(t,e),n)}function u(e){var t=e.toString(16).toUpperCase();return t.length<2?"0"+t:t}s.get=function(e){var t,n;switch(e.substring(0,3).toLowerCase()){case"hsl":t=s.get.hsl(e),n="hsl";break;case"hwb":t=s.get.hwb(e),n="hwb";break;default:t=s.get.rgb(e),n="rgb"}return t?{model:n,value:t}:null},s.get.rgb=function(e){if(!e)return null;var t,n,i,o=[0,0,0,1];if(t=e.match(/^#([a-f0-9]{6})([a-f0-9]{2})?$/i)){for(i=t[2],t=t[1],n=0;n<3;n++){var a=2*n;o[n]=parseInt(t.slice(a,a+2),16)}i&&(o[3]=parseInt(i,16)/255)}else if(t=e.match(/^#([a-f0-9]{3,4})$/i)){for(i=(t=t[1])[3],n=0;n<3;n++)o[n]=parseInt(t[n]+t[n],16);i&&(o[3]=parseInt(i+i,16)/255)}else if(t=e.match(/^rgba?\(\s*([+-]?\d+)\s*,\s*([+-]?\d+)\s*,\s*([+-]?\d+)\s*(?:,\s*([+-]?[\d\.]+)\s*)?\)$/)){for(n=0;n<3;n++)o[n]=parseInt(t[n+1],0);t[4]&&(o[3]=parseFloat(t[4]))}else{if(!(t=e.match(/^rgba?\(\s*([+-]?[\d\.]+)\%\s*,\s*([+-]?[\d\.]+)\%\s*,\s*([+-]?[\d\.]+)\%\s*(?:,\s*([+-]?[\d\.]+)\s*)?\)$/)))return(t=e.match(/(\D+)/))?"transparent"===t[1]?[0,0,0,0]:(o=r[t[1]])?(o[3]=1,o):null:null;for(n=0;n<3;n++)o[n]=Math.round(2.55*parseFloat(t[n+1]));t[4]&&(o[3]=parseFloat(t[4]))}for(n=0;n<3;n++)o[n]=l(o[n],0,255);return o[3]=l(o[3],0,1),o},s.get.hsl=function(e){if(!e)return null;var t=e.match(/^hsla?\(\s*([+-]?(?:\d*\.)?\d+)(?:deg)?\s*,\s*([+-]?[\d\.]+)%\s*,\s*([+-]?[\d\.]+)%\s*(?:,\s*([+-]?[\d\.]+)\s*)?\)$/);if(t){var n=parseFloat(t[4]);return[(parseFloat(t[1])+360)%360,l(parseFloat(t[2]),0,100),l(parseFloat(t[3]),0,100),l(isNaN(n)?1:n,0,1)]}return null},s.get.hwb=function(e){if(!e)return null;var t=e.match(/^hwb\(\s*([+-]?\d*[\.]?\d+)(?:deg)?\s*,\s*([+-]?[\d\.]+)%\s*,\s*([+-]?[\d\.]+)%\s*(?:,\s*([+-]?[\d\.]+)\s*)?\)$/);if(t){var n=parseFloat(t[4]);return[(parseFloat(t[1])%360+360)%360,l(parseFloat(t[2]),0,100),l(parseFloat(t[3]),0,100),l(isNaN(n)?1:n,0,1)]}return null},s.to.hex=function(){var e=i(arguments);return"#"+u(e[0])+u(e[1])+u(e[2])+(e[3]<1?u(Math.round(255*e[3])):"")},s.to.rgb=function(){var e=i(arguments);return e.length<4||1===e[3]?"rgb("+Math.round(e[0])+", "+Math.round(e[1])+", "+Math.round(e[2])+")":"rgba("+Math.round(e[0])+", "+Math.round(e[1])+", "+Math.round(e[2])+", "+e[3]+")"},s.to.rgb.percent=function(){var e=i(arguments),t=Math.round(e[0]/255*100),n=Math.round(e[1]/255*100),r=Math.round(e[2]/255*100);return e.length<4||1===e[3]?"rgb("+t+"%, "+n+"%, "+r+"%)":"rgba("+t+"%, "+n+"%, "+r+"%, "+e[3]+")"},s.to.hsl=function(){var e=i(arguments);return e.length<4||1===e[3]?"hsl("+e[0]+", "+e[1]+"%, "+e[2]+"%)":"hsla("+e[0]+", "+e[1]+"%, "+e[2]+"%, "+e[3]+")"},s.to.hwb=function(){var e=i(arguments),t="";return e.length>=4&&1!==e[3]&&(t=", "+e[3]),"hwb("+e[0]+", "+e[1]+"%, "+e[2]+"%"+t+")"},s.to.keyword=function(e){return o[e.slice(0,3)]}},function(e,t,n){"use strict";e.exports={aliceblue:[240,248,255],antiquewhite:[250,235,215],aqua:[0,255,255],aquamarine:[127,255,212],azure:[240,255,255],beige:[245,245,220],bisque:[255,228,196],black:[0,0,0],blanchedalmond:[255,235,205],blue:[0,0,255],blueviolet:[138,43,226],brown:[165,42,42],burlywood:[222,184,135],cadetblue:[95,158,160],chartreuse:[127,255,0],chocolate:[210,105,30],coral:[255,127,80],cornflowerblue:[100,149,237],cornsilk:[255,248,220],crimson:[220,20,60],cyan:[0,255,255],darkblue:[0,0,139],darkcyan:[0,139,139],darkgoldenrod:[184,134,11],darkgray:[169,169,169],darkgreen:[0,100,0],darkgrey:[169,169,169],darkkhaki:[189,183,107],darkmagenta:[139,0,139],darkolivegreen:[85,107,47],darkorange:[255,140,0],darkorchid:[153,50,204],darkred:[139,0,0],darksalmon:[233,150,122],darkseagreen:[143,188,143],darkslateblue:[72,61,139],darkslategray:[47,79,79],darkslategrey:[47,79,79],darkturquoise:[0,206,209],darkviolet:[148,0,211],deeppink:[255,20,147],deepskyblue:[0,191,255],dimgray:[105,105,105],dimgrey:[105,105,105],dodgerblue:[30,144,255],firebrick:[178,34,34],floralwhite:[255,250,240],forestgreen:[34,139,34],fuchsia:[255,0,255],gainsboro:[220,220,220],ghostwhite:[248,248,255],gold:[255,215,0],goldenrod:[218,165,32],gray:[128,128,128],green:[0,128,0],greenyellow:[173,255,47],grey:[128,128,128],honeydew:[240,255,240],hotpink:[255,105,180],indianred:[205,92,92],indigo:[75,0,130],ivory:[255,255,240],khaki:[240,230,140],lavender:[230,230,250],lavenderblush:[255,240,245],lawngreen:[124,252,0],lemonchiffon:[255,250,205],lightblue:[173,216,230],lightcoral:[240,128,128],lightcyan:[224,255,255],lightgoldenrodyellow:[250,250,210],lightgray:[211,211,211],lightgreen:[144,238,144],lightgrey:[211,211,211],lightpink:[255,182,193],lightsalmon:[255,160,122],lightseagreen:[32,178,170],lightskyblue:[135,206,250],lightslategray:[119,136,153],lightslategrey:[119,136,153],lightsteelblue:[176,196,222],lightyellow:[255,255,224],lime:[0,255,0],limegreen:[50,205,50],linen:[250,240,230],magenta:[255,0,255],maroon:[128,0,0],mediumaquamarine:[102,205,170],mediumblue:[0,0,205],mediumorchid:[186,85,211],mediumpurple:[147,112,219],mediumseagreen:[60,179,113],mediumslateblue:[123,104,238],mediumspringgreen:[0,250,154],mediumturquoise:[72,209,204],mediumvioletred:[199,21,133],midnightblue:[25,25,112],mintcream:[245,255,250],mistyrose:[255,228,225],moccasin:[255,228,181],navajowhite:[255,222,173],navy:[0,0,128],oldlace:[253,245,230],olive:[128,128,0],olivedrab:[107,142,35],orange:[255,165,0],orangered:[255,69,0],orchid:[218,112,214],palegoldenrod:[238,232,170],palegreen:[152,251,152],paleturquoise:[175,238,238],palevioletred:[219,112,147],papayawhip:[255,239,213],peachpuff:[255,218,185],peru:[205,133,63],pink:[255,192,203],plum:[221,160,221],powderblue:[176,224,230],purple:[128,0,128],rebeccapurple:[102,51,153],red:[255,0,0],rosybrown:[188,143,143],royalblue:[65,105,225],saddlebrown:[139,69,19],salmon:[250,128,114],sandybrown:[244,164,96],seagreen:[46,139,87],seashell:[255,245,238],sienna:[160,82,45],silver:[192,192,192],skyblue:[135,206,235],slateblue:[106,90,205],slategray:[112,128,144],slategrey:[112,128,144],snow:[255,250,250],springgreen:[0,255,127],steelblue:[70,130,180],tan:[210,180,140],teal:[0,128,128],thistle:[216,191,216],tomato:[255,99,71],turquoise:[64,224,208],violet:[238,130,238],wheat:[245,222,179],white:[255,255,255],whitesmoke:[245,245,245],yellow:[255,255,0],yellowgreen:[154,205,50]}},function(e,t,n){"use strict";var r=n(314),i=Array.prototype.concat,o=Array.prototype.slice,a=e.exports=function(e){for(var t=[],n=0,a=e.length;n<a;n++){var s=e[n];r(s)?t=i.call(t,o.call(s)):t.push(s)}return t};a.wrap=function(e){return function(){return e(a(arguments))}}},function(e,t){e.exports=function(e){return!(!e||"string"==typeof e)&&(e instanceof Array||Array.isArray(e)||e.length>=0&&(e.splice instanceof Function||Object.getOwnPropertyDescriptor(e,e.length-1)&&"String"!==e.constructor.name))}},function(e,t,n){function r(e){return(r="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}var i=n(149),o=n(317),a={};Object.keys(i).forEach((function(e){a[e]={},Object.defineProperty(a[e],"channels",{value:i[e].channels}),Object.defineProperty(a[e],"labels",{value:i[e].labels});var t=o(e);Object.keys(t).forEach((function(n){var i=t[n];a[e][n]=function(e){var t=function(t){if(null==t)return t;arguments.length>1&&(t=Array.prototype.slice.call(arguments));var n=e(t);if("object"===r(n))for(var i=n.length,o=0;o<i;o++)n[o]=Math.round(n[o]);return n};return"conversion"in e&&(t.conversion=e.conversion),t}(i),a[e][n].raw=function(e){var t=function(t){return null==t?t:(arguments.length>1&&(t=Array.prototype.slice.call(arguments)),e(t))};return"conversion"in e&&(t.conversion=e.conversion),t}(i)}))})),e.exports=a},function(e,t,n){"use strict";e.exports={aliceblue:[240,248,255],antiquewhite:[250,235,215],aqua:[0,255,255],aquamarine:[127,255,212],azure:[240,255,255],beige:[245,245,220],bisque:[255,228,196],black:[0,0,0],blanchedalmond:[255,235,205],blue:[0,0,255],blueviolet:[138,43,226],brown:[165,42,42],burlywood:[222,184,135],cadetblue:[95,158,160],chartreuse:[127,255,0],chocolate:[210,105,30],coral:[255,127,80],cornflowerblue:[100,149,237],cornsilk:[255,248,220],crimson:[220,20,60],cyan:[0,255,255],darkblue:[0,0,139],darkcyan:[0,139,139],darkgoldenrod:[184,134,11],darkgray:[169,169,169],darkgreen:[0,100,0],darkgrey:[169,169,169],darkkhaki:[189,183,107],darkmagenta:[139,0,139],darkolivegreen:[85,107,47],darkorange:[255,140,0],darkorchid:[153,50,204],darkred:[139,0,0],darksalmon:[233,150,122],darkseagreen:[143,188,143],darkslateblue:[72,61,139],darkslategray:[47,79,79],darkslategrey:[47,79,79],darkturquoise:[0,206,209],darkviolet:[148,0,211],deeppink:[255,20,147],deepskyblue:[0,191,255],dimgray:[105,105,105],dimgrey:[105,105,105],dodgerblue:[30,144,255],firebrick:[178,34,34],floralwhite:[255,250,240],forestgreen:[34,139,34],fuchsia:[255,0,255],gainsboro:[220,220,220],ghostwhite:[248,248,255],gold:[255,215,0],goldenrod:[218,165,32],gray:[128,128,128],green:[0,128,0],greenyellow:[173,255,47],grey:[128,128,128],honeydew:[240,255,240],hotpink:[255,105,180],indianred:[205,92,92],indigo:[75,0,130],ivory:[255,255,240],khaki:[240,230,140],lavender:[230,230,250],lavenderblush:[255,240,245],lawngreen:[124,252,0],lemonchiffon:[255,250,205],lightblue:[173,216,230],lightcoral:[240,128,128],lightcyan:[224,255,255],lightgoldenrodyellow:[250,250,210],lightgray:[211,211,211],lightgreen:[144,238,144],lightgrey:[211,211,211],lightpink:[255,182,193],lightsalmon:[255,160,122],lightseagreen:[32,178,170],lightskyblue:[135,206,250],lightslategray:[119,136,153],lightslategrey:[119,136,153],lightsteelblue:[176,196,222],lightyellow:[255,255,224],lime:[0,255,0],limegreen:[50,205,50],linen:[250,240,230],magenta:[255,0,255],maroon:[128,0,0],mediumaquamarine:[102,205,170],mediumblue:[0,0,205],mediumorchid:[186,85,211],mediumpurple:[147,112,219],mediumseagreen:[60,179,113],mediumslateblue:[123,104,238],mediumspringgreen:[0,250,154],mediumturquoise:[72,209,204],mediumvioletred:[199,21,133],midnightblue:[25,25,112],mintcream:[245,255,250],mistyrose:[255,228,225],moccasin:[255,228,181],navajowhite:[255,222,173],navy:[0,0,128],oldlace:[253,245,230],olive:[128,128,0],olivedrab:[107,142,35],orange:[255,165,0],orangered:[255,69,0],orchid:[218,112,214],palegoldenrod:[238,232,170],palegreen:[152,251,152],paleturquoise:[175,238,238],palevioletred:[219,112,147],papayawhip:[255,239,213],peachpuff:[255,218,185],peru:[205,133,63],pink:[255,192,203],plum:[221,160,221],powderblue:[176,224,230],purple:[128,0,128],rebeccapurple:[102,51,153],red:[255,0,0],rosybrown:[188,143,143],royalblue:[65,105,225],saddlebrown:[139,69,19],salmon:[250,128,114],sandybrown:[244,164,96],seagreen:[46,139,87],seashell:[255,245,238],sienna:[160,82,45],silver:[192,192,192],skyblue:[135,206,235],slateblue:[106,90,205],slategray:[112,128,144],slategrey:[112,128,144],snow:[255,250,250],springgreen:[0,255,127],steelblue:[70,130,180],tan:[210,180,140],teal:[0,128,128],thistle:[216,191,216],tomato:[255,99,71],turquoise:[64,224,208],violet:[238,130,238],wheat:[245,222,179],white:[255,255,255],whitesmoke:[245,245,245],yellow:[255,255,0],yellowgreen:[154,205,50]}},function(e,t,n){var r=n(149);function i(e){var t=function(){for(var e={},t=Object.keys(r),n=t.length,i=0;i<n;i++)e[t[i]]={distance:-1,parent:null};return e}(),n=[e];for(t[e].distance=0;n.length;)for(var i=n.pop(),o=Object.keys(r[i]),a=o.length,s=0;s<a;s++){var l=o[s],u=t[l];-1===u.distance&&(u.distance=t[i].distance+1,u.parent=i,n.unshift(l))}return t}function o(e,t){return function(n){return t(e(n))}}function a(e,t){for(var n=[t[e].parent,e],i=r[t[e].parent][e],a=t[e].parent;t[a].parent;)n.unshift(t[a].parent),i=o(r[t[a].parent][a],i),a=t[a].parent;return i.conversion=n,i}e.exports=function(e){for(var t=i(e),n={},r=Object.keys(t),o=r.length,s=0;s<o;s++){var l=r[s];null!==t[l].parent&&(n[l]=a(l,t))}return n}},function(e){e.exports=JSON.parse('{"name":"@antv/g-base","version":"0.5.1","description":"A common util collection for antv projects","main":"lib/index.js","module":"esm/index.js","types":"lib/index.d.ts","files":["package.json","esm","lib","LICENSE","README.md"],"scripts":{"build":"npm run clean && run-p build:*","build:esm":"tsc -p tsconfig.json --target ES5 --module ESNext --outDir esm","build:cjs":"tsc -p tsconfig.json --target ES5 --module commonjs --outDir lib","clean":"rm -rf esm lib","watch:cjs":"tsc-watch -p tsconfig.json --target ES5 --module commonjs --outDir lib --compiler typescript/bin/tsc","coverage":"npm run coverage-generator && npm run coverage-viewer","coverage-generator":"torch --coverage --compile --source-pattern src/*.js,src/**/*.js --opts tests/mocha.opts","coverage-viewer":"torch-coverage","test":"torch --renderer --compile --opts tests/mocha.opts","test-live":"torch --compile --interactive tests/unit","tsc":"tsc --noEmit","typecheck":"tsc --noEmit"},"repository":{"type":"git","url":"git+https://github.com/antvis/util.git"},"keywords":["util","antv","g"],"publishConfig":{"access":"public"},"author":"https://github.com/orgs/antvis/people","license":"ISC","bugs":{"url":"https://github.com/antvis/util/issues"},"devDependencies":{"@antv/torch":"^1.0.0","gl-matrix":"^3.0.0","less":"^3.9.0","npm-run-all":"^4.1.5","tsc-watch":"^4.0.0"},"homepage":"https://github.com/antvis/util#readme","dependencies":{"@antv/event-emitter":"^0.1.1","@antv/g-math":"^0.1.5","@antv/matrix-util":"^3.1.0-beta.1","@antv/path-util":"~2.0.5","@antv/util":"~2.0.0","@types/d3-timer":"^1.0.9","d3-ease":"^1.0.5","d3-interpolate":"^1.3.2","d3-timer":"^1.0.9","detect-browser":"^5.1.0"},"__npminstall_done":true,"gitHead":"ba7ecf8b8deac37682414ac0b7fc2c3e903398bd","_from":"@antv/g-base@0.5.1","_resolved":"https://registry.npm.alibaba-inc.com/@antv/g-base/download/@antv/g-base-0.5.1.tgz"}')},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.getShape=void 0;var r=n(155),i=n(320);function o(e,t,n){var r=e.getTotalMatrix();if(r){var o=function(e,t){if(t){var n=i.invert(t);return i.multiplyVec2(n,e)}return e}([t,n,1],r);return[o[0],o[1]]}return[t,n]}function a(e,t,n){if(e.isCanvas&&e.isCanvas())return!0;if(!r.isAllowCapture(e)||!1===e.cfg.isInView)return!1;if(e.cfg.clipShape){var i=o(e,t,n),a=i[0],s=i[1];if(e.isClipped(a,s))return!1}var l=e.cfg.cacheCanvasBBox||e.getCanvasBBox();return t>=l.minX&&t<=l.maxX&&n>=l.minY&&n<=l.maxY}t.getShape=function e(t,n,r){if(!a(t,n,r))return null;for(var i=null,s=t.getChildren(),l=s.length-1;l>=0;l--){var u=s[l];if(u.isGroup())i=e(u,n,r);else if(a(u,n,r)){var c=u,d=o(u,n,r),h=d[0],f=d[1];c.isInShape(h,f)&&(i=u)}if(i)break}return i}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.invert=t.multiplyVec2=t.multiplyMatrix=void 0,t.multiplyMatrix=function(e,t){var n=[],r=e[0],i=e[1],o=e[2],a=e[3],s=e[4],l=e[5],u=e[6],c=e[7],d=e[8],h=t[0],f=t[1],p=t[2],g=t[3],v=t[4],m=t[5],y=t[6],b=t[7],C=t[8];return n[0]=h*r+f*a+p*u,n[1]=h*i+f*s+p*c,n[2]=h*o+f*l+p*d,n[3]=g*r+v*a+m*u,n[4]=g*i+v*s+m*c,n[5]=g*o+v*l+m*d,n[6]=y*r+b*a+C*u,n[7]=y*i+b*s+C*c,n[8]=y*o+b*l+C*d,n},t.multiplyVec2=function(e,t){var n=[],r=t[0],i=t[1];return n[0]=e[0]*r+e[3]*i+e[6],n[1]=e[1]*r+e[4]*i+e[7],n},t.invert=function(e){var t=[],n=e[0],r=e[1],i=e[2],o=e[3],a=e[4],s=e[5],l=e[6],u=e[7],c=e[8],d=c*a-s*u,h=-c*o+s*l,f=u*o-a*l,p=n*d+r*h+i*f;return p?(p=1/p,t[0]=d*p,t[1]=(-c*r+i*u)*p,t[2]=(s*r-i*a)*p,t[3]=h*p,t[4]=(c*n-i*l)*p,t[5]=(-s*n+i*o)*p,t[6]=f*p,t[7]=(-u*n+r*l)*p,t[8]=(a*n-r*o)*p,t):null}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(18);function i(e){return Math.sqrt(e[0]*e[0]+e[1]*e[1])}function o(e,t){return i(e)*i(t)?(e[0]*t[0]+e[1]*t[1])/(i(e)*i(t)):1}function a(e,t){return(e[0]*t[1]<e[1]*t[0]?-1:1)*Math.acos(o(e,t))}t.default=function(e,t){var n=t[1],i=t[2],s=r.mod(r.toRadian(t[3]),2*Math.PI),l=t[4],u=t[5],c=e[0],d=e[1],h=t[6],f=t[7],p=Math.cos(s)*(c-h)/2+Math.sin(s)*(d-f)/2,g=-1*Math.sin(s)*(c-h)/2+Math.cos(s)*(d-f)/2,v=p*p/(n*n)+g*g/(i*i);v>1&&(n*=Math.sqrt(v),i*=Math.sqrt(v));var m=n*n*(g*g)+i*i*(p*p),y=m?Math.sqrt((n*n*(i*i)-m)/m):1;l===u&&(y*=-1),isNaN(y)&&(y=0);var b=i?y*n*g/i:0,C=n?y*-i*p/n:0,A=(c+h)/2+Math.cos(s)*b-Math.sin(s)*C,_=(d+f)/2+Math.sin(s)*b+Math.cos(s)*C,I=[(p-b)/n,(g-C)/i],x=[(-1*p-b)/n,(-1*g-C)/i],w=a([1,0],I),M=a(I,x);return o(I,x)<=-1&&(M=Math.PI),o(I,x)>=1&&(M=0),0===u&&M>0&&(M-=2*Math.PI),1===u&&M<0&&(M+=2*Math.PI),{cx:A,cy:_,rx:r.isSamePoint(e,[h,f])?0:n,ry:r.isSamePoint(e,[h,f])?0:i,startAngle:w,endAngle:w+M,xRotation:s,arcFlag:l,sweepFlag:u}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.getMethod=t.register=void 0;var r=new Map;t.register=function(e,t){r.set(e,t)},t.getMethod=function(e){return r.get(e)}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){var t=e.attr();return{x:t.x,y:t.y,width:t.width,height:t.height}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){var t=e.attr(),n=t.x,r=t.y,i=t.r;return{x:n-i,y:r-i,width:2*i,height:2*i}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(33),i=n(107);t.default=function(e){for(var t=e.attr().points,n=[],o=[],a=0;a<t.length;a++){var s=t[a];n.push(s[0]),o.push(s[1])}var l=r.getBBoxByArray(n,o),u=l.x,c=l.y,d={minX:u,minY:c,maxX:u+l.width,maxY:c+l.height};return{x:(d=i.mergeArrowBBox(e,d)).minX,y:d.minY,width:d.maxX-d.minX,height:d.maxY-d.minY}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(33);t.default=function(e){for(var t=e.attr().points,n=[],i=[],o=0;o<t.length;o++){var a=t[o];n.push(a[0]),i.push(a[1])}return r.getBBoxByArray(n,i)}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(158);t.default=function(e){var t=e.attr(),n=t.x,i=t.y,o=t.text,a=t.fontSize,s=t.lineHeight,l=t.font;l||(l=r.assembleFont(t));var u,c=r.getTextWidth(o,l);if(c){var d=t.textAlign,h=t.textBaseline,f=r.getTextHeight(o,a,s),p={x:n,y:i-f};d&&("end"===d||"right"===d?p.x-=c:"center"===d&&(p.x-=c/2)),h&&("top"===h?p.y+=f:"middle"===h&&(p.y+=f/2)),u={x:p.x,y:p.y,width:c,height:f}}else u={x:n,y:i,width:0,height:0};return u}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(160),i=n(108),o=n(329),a=n(162),s=n(103),l=n(107);function u(e,t){var n=e.prePoint,r=e.currentPoint,i=e.nextPoint,o=Math.pow(r[0]-n[0],2)+Math.pow(r[1]-n[1],2),a=Math.pow(r[0]-i[0],2)+Math.pow(r[1]-i[1],2),l=Math.pow(n[0]-i[0],2)+Math.pow(n[1]-i[1],2),u=Math.acos((o+a-l)/(2*Math.sqrt(o)*Math.sqrt(a)));if(!u||0===Math.sin(u)||s.default(u,0))return{xExtra:0,yExtra:0};var c=Math.abs(Math.atan2(i[1]-r[1],i[0]-r[0])),d=Math.abs(Math.atan2(i[0]-r[0],i[1]-r[1]));return c=c>Math.PI/2?Math.PI-c:c,d=d>Math.PI/2?Math.PI-d:d,{xExtra:Math.cos(u/2-c)*(t/2*(1/Math.sin(u/2)))-t/2||0,yExtra:Math.cos(d-u/2)*(t/2*(1/Math.sin(u/2)))-t/2||0}}t.default=function(e){var t=e.attr(),n=t.path,s=t.stroke?t.lineWidth:0,c=function(e,t){for(var n=[],a=[],s=[],l=0;l<e.length;l++){var c=(b=e[l]).currentPoint,d=b.params,h=b.prePoint,f=void 0;switch(b.command){case"Q":f=r.default.box(h[0],h[1],d[1],d[2],d[3],d[4]);break;case"C":f=i.default.box(h[0],h[1],d[1],d[2],d[3],d[4],d[5],d[6]);break;case"A":var p=b.arcParams;f=o.default.box(p.cx,p.cy,p.rx,p.ry,p.xRotation,p.startAngle,p.endAngle);break;default:n.push(c[0]),a.push(c[1])}f&&(b.box=f,n.push(f.x,f.x+f.width),a.push(f.y,f.y+f.height)),t&&("L"===b.command||"M"===b.command)&&b.prePoint&&b.nextPoint&&s.push(b)}n=n.filter((function(e){return!Number.isNaN(e)})),a=a.filter((function(e){return!Number.isNaN(e)}));var g=Math.min.apply(null,n),v=Math.min.apply(null,a),m=Math.max.apply(null,n),y=Math.max.apply(null,a);if(0===s.length)return{x:g,y:v,width:m-g,height:y-v};for(l=0;l<s.length;l++){var b;(c=(b=s[l]).currentPoint)[0]===g?g-=u(b,t).xExtra:c[0]===m&&(m+=u(b,t).xExtra),c[1]===v?v-=u(b,t).yExtra:c[1]===y&&(y+=u(b,t).yExtra)}return{x:g,y:v,width:m-g,height:y-v}}(e.get("segments")||a.default(n),s),d=c.x,h=c.y,f={minX:d,minY:h,maxX:d+c.width,maxY:h+c.height};return{x:(f=l.mergeArrowBBox(e,f)).minX,y:f.minY,width:f.maxX-f.minX,height:f.maxY-f.minY}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(33),i=n(330);function o(e,t,n,r,i,o){return n*Math.cos(i)*Math.cos(o)-r*Math.sin(i)*Math.sin(o)+e}function a(e,t,n,r,i,o){return n*Math.sin(i)*Math.cos(o)+r*Math.cos(i)*Math.sin(o)+t}function s(e,t,n){return{x:e*Math.cos(n),y:t*Math.sin(n)}}function l(e,t,n){var r=Math.cos(n),i=Math.sin(n);return[e*r-t*i,e*i+t*r]}t.default={box:function(e,t,n,r,i,s,l){for(var u=function(e,t,n){return Math.atan(-t/e*Math.tan(n))}(n,r,i),c=1/0,d=-1/0,h=[s,l],f=2*-Math.PI;f<=2*Math.PI;f+=Math.PI){var p=u+f;s<l?s<p&&p<l&&h.push(p):l<p&&p<s&&h.push(p)}for(f=0;f<h.length;f++){var g=o(e,0,n,r,i,h[f]);g<c&&(c=g),g>d&&(d=g)}var v=function(e,t,n){return Math.atan(t/(e*Math.tan(n)))}(n,r,i),m=1/0,y=-1/0,b=[s,l];for(f=2*-Math.PI;f<=2*Math.PI;f+=Math.PI){var C=v+f;s<l?s<C&&C<l&&b.push(C):l<C&&C<s&&b.push(C)}for(f=0;f<b.length;f++){var A=a(0,t,n,r,i,b[f]);A<m&&(m=A),A>y&&(y=A)}return{x:c,y:m,width:d-c,height:y-m}},length:function(e,t,n,r,i,o,a){},nearestPoint:function(e,t,n,r,o,a,u,c,d){var h=l(c-e,d-t,-o),f=h[0],p=h[1],g=i.default.nearestPoint(0,0,n,r,f,p),v=function(e,t,n,r){return(Math.atan2(r*e,n*t)+2*Math.PI)%(2*Math.PI)}(n,r,g.x,g.y);v<a?g=s(n,r,a):v>u&&(g=s(n,r,u));var m=l(g.x,g.y,o);return{x:m[0]+e,y:m[1]+t}},pointDistance:function(e,t,n,i,o,a,s,l,u){var c=this.nearestPoint(e,t,n,i,l,u);return r.distance(c.x,c.y,l,u)},pointAt:function(e,t,n,r,i,s,l,u){var c=(l-s)*u+s;return{x:o(e,0,n,r,i,c),y:a(0,t,n,r,i,c)}},tangentAngle:function(e,t,n,i,o,a,s,l){var u=(s-a)*l+a,c=function(e,t,n,r,i,o,a,s){return-1*n*Math.cos(i)*Math.sin(s)-r*Math.sin(i)*Math.cos(s)}(0,0,n,i,o,0,0,u),d=function(e,t,n,r,i,o,a,s){return-1*n*Math.sin(i)*Math.sin(s)+r*Math.cos(i)*Math.cos(s)}(0,0,n,i,o,0,0,u);return r.piMod(Math.atan2(d,c))}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(33);function i(e,t){var n=Math.abs(e);return t>0?n:-1*n}t.default={box:function(e,t,n,r){return{x:e-n,y:t-r,width:2*n,height:2*r}},length:function(e,t,n,r){return Math.PI*(3*(n+r)-Math.sqrt((3*n+r)*(n+3*r)))},nearestPoint:function(e,t,n,r,o,a){var s=n,l=r;if(0===s||0===l)return{x:e,y:t};for(var u,c,d=o-e,h=a-t,f=Math.abs(d),p=Math.abs(h),g=s*s,v=l*l,m=Math.PI/4,y=0;y<4;y++){u=s*Math.cos(m),c=l*Math.sin(m);var b=(g-v)*Math.pow(Math.cos(m),3)/s,C=(v-g)*Math.pow(Math.sin(m),3)/l,A=u-b,_=c-C,I=f-b,x=p-C,w=Math.hypot(_,A),M=Math.hypot(x,I);m+=w*Math.asin((A*x-_*I)/(w*M))/Math.sqrt(g+v-u*u-c*c),m=Math.min(Math.PI/2,Math.max(0,m))}return{x:e+i(u,d),y:t+i(c,h)}},pointDistance:function(e,t,n,i,o,a){var s=this.nearestPoint(e,t,n,i,o,a);return r.distance(s.x,s.y,o,a)},pointAt:function(e,t,n,r,i){var o=2*Math.PI*i;return{x:e+n*Math.cos(o),y:t+r*Math.sin(o)}},tangentAngle:function(e,t,n,i,o){var a=2*Math.PI*o,s=Math.atan2(i*Math.cos(a),-n*Math.sin(a));return r.piMod(s)}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(2),i=n(4),o=n(3),a=/[MLHVQTCSAZ]([^MLHVQTCSAZ]*)/gi,s=/[^\s\,]+/gi;t.default=function(e){var t=e||[];return i.default(t)?t:o.default(t)?(t=t.match(a),r.default(t,(function(e,n){if((e=e.match(s))[0].length>1){var i=e[0].charAt(0);e.splice(1,0,e[0].substr(1)),e[0]=i}r.default(e,(function(t,n){isNaN(t)||(e[n]=+t)})),t[n]=e})),t):void 0}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(107);t.default=function(e){var t=e.attr(),n=t.x1,i=t.y1,o=t.x2,a=t.y2,s={minX:Math.min(n,o),maxX:Math.max(n,o),minY:Math.min(i,a),maxY:Math.max(i,a)};return{x:(s=r.mergeArrowBBox(e,s)).minX,y:s.minY,width:s.maxX-s.minX,height:s.maxY-s.minY}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){var t=e.attr(),n=t.x,r=t.y,i=t.rx,o=t.ry;return{x:n-i,y:r-o,width:2*i,height:2*o}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(1),i=n(28),o=n(18),a=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return r.__extends(t,e),t.prototype.getDefaultAttrs=function(){var t=e.prototype.getDefaultAttrs.call(this);return r.__assign(r.__assign({},t),{x:0,y:0,r:0})},t.prototype.isInStrokeOrPath=function(e,t,n,r,i){var a=this.attr(),s=a.x,l=a.y,u=a.r,c=i/2,d=o.distance(s,l,e,t);return r&&n?d<=u+c:r?d<=u:!!n&&d>=u-c&&d<=u+c},t.prototype.createPath=function(e){var t=this.attr(),n=t.x,r=t.y,i=t.r;e.beginPath(),e.arc(n,r,i,0,2*Math.PI,!1),e.closePath()},t}(i.default);t.default=a},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(1);function i(e,t,n,r){return e/(n*n)+t/(r*r)}var o=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return r.__extends(t,e),t.prototype.getDefaultAttrs=function(){var t=e.prototype.getDefaultAttrs.call(this);return r.__assign(r.__assign({},t),{x:0,y:0,rx:0,ry:0})},t.prototype.isInStrokeOrPath=function(e,t,n,r,o){var a=this.attr(),s=o/2,l=a.x,u=a.y,c=a.rx,d=a.ry,h=(e-l)*(e-l),f=(t-u)*(t-u);return r&&n?i(h,f,c+s,d+s)<=1:r?i(h,f,c,d)<=1:!!n&&i(h,f,c-s,d-s)>=1&&i(h,f,c+s,d+s)<=1},t.prototype.createPath=function(e){var t=this.attr(),n=t.x,r=t.y,i=t.rx,o=t.ry;if(e.beginPath(),e.ellipse)e.ellipse(n,r,i,o,0,0,2*Math.PI,!1);else{var a=i>o?i:o,s=i>o?1:i/o,l=i>o?o/i:1;e.save(),e.translate(n,r),e.scale(s,l),e.arc(0,0,a,0,2*Math.PI),e.restore(),e.closePath()}},t}(n(28).default);t.default=o},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(1),i=n(28),o=n(18);function a(e){return e instanceof HTMLElement&&o.isString(e.nodeName)&&"CANVAS"===e.nodeName.toUpperCase()}var s=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return r.__extends(t,e),t.prototype.getDefaultAttrs=function(){var t=e.prototype.getDefaultAttrs.call(this);return r.__assign(r.__assign({},t),{x:0,y:0,width:0,height:0})},t.prototype.initAttrs=function(e){this._setImage(e.img)},t.prototype.isStroke=function(){return!1},t.prototype.isOnlyHitBox=function(){return!0},t.prototype._afterLoading=function(){if(!0===this.get("toDraw")){var e=this.get("canvas");e?e.draw():this.createPath(this.get("context"))}},t.prototype._setImage=function(e){var t=this,n=this.attrs;if(o.isString(e)){var r=new Image;r.onload=function(){if(t.destroyed)return!1;t.attr("img",r),t.set("loading",!1),t._afterLoading();var e=t.get("callback");e&&e.call(t)},r.crossOrigin="Anonymous",r.src=e,this.set("loading",!0)}else e instanceof Image?(n.width||(n.width=e.width),n.height||(n.height=e.height)):a(e)&&(n.width||(n.width=Number(e.getAttribute("width"))),n.height||(n.height,Number(e.getAttribute("height"))))},t.prototype.onAttrChange=function(t,n,r){e.prototype.onAttrChange.call(this,t,n,r),"img"===t&&this._setImage(n)},t.prototype.createPath=function(e){if(this.get("loading"))return this.set("toDraw",!0),void this.set("context",e);var t=this.attr(),n=t.x,r=t.y,i=t.width,s=t.height,l=t.sx,u=t.sy,c=t.swidth,d=t.sheight,h=t.img;(h instanceof Image||a(h))&&(o.isNil(l)||o.isNil(u)||o.isNil(c)||o.isNil(d)?e.drawImage(h,n,r,i,s):e.drawImage(h,l,u,c,d,n,r,i,s))},t}(i.default);t.default=s},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(1),i=n(40),o=n(28),a=n(75),s=n(74),l=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return r.__extends(t,e),t.prototype.getDefaultAttrs=function(){var t=e.prototype.getDefaultAttrs.call(this);return r.__assign(r.__assign({},t),{x1:0,y1:0,x2:0,y2:0,startArrow:!1,endArrow:!1})},t.prototype.initAttrs=function(e){this.setArrow()},t.prototype.onAttrChange=function(t,n,r){e.prototype.onAttrChange.call(this,t,n,r),this.setArrow()},t.prototype.setArrow=function(){var e=this.attr(),t=e.x1,n=e.y1,r=e.x2,i=e.y2,o=e.startArrow,a=e.endArrow;o&&s.addStartArrow(this,e,r,i,t,n),a&&s.addEndArrow(this,e,t,n,r,i)},t.prototype.isInStrokeOrPath=function(e,t,n,r,i){if(!n||!i)return!1;var o=this.attr(),s=o.x1,l=o.y1,u=o.x2,c=o.y2;return a.default(s,l,u,c,i,e,t)},t.prototype.createPath=function(e){var t=this.attr(),n=t.x1,r=t.y1,i=t.x2,o=t.y2,a=t.startArrow,l=t.endArrow,u={dx:0,dy:0},c={dx:0,dy:0};a&&a.d&&(u=s.getShortenOffset(n,r,i,o,t.startArrow.d)),l&&l.d&&(c=s.getShortenOffset(n,r,i,o,t.endArrow.d)),e.beginPath(),e.moveTo(n+u.dx,r+u.dy),e.lineTo(i-c.dx,o-c.dy)},t.prototype.afterDrawPath=function(e){var t=this.get("startArrowShape"),n=this.get("endArrowShape");t&&t.draw(e),n&&n.draw(e)},t.prototype.getTotalLength=function(){var e=this.attr(),t=e.x1,n=e.y1,r=e.x2,o=e.y2;return i.default.length(t,n,r,o)},t.prototype.getPoint=function(e){var t=this.attr(),n=t.x1,r=t.y1,o=t.x2,a=t.y2;return i.default.pointAt(n,r,o,a,e)},t}(o.default);t.default=l},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(1),i=n(0),o=n(165),a=n(28),s=n(18),l=n(60),u={circle:function(e,t,n){return[["M",e-n,t],["A",n,n,0,1,0,e+n,t],["A",n,n,0,1,0,e-n,t]]},square:function(e,t,n){return[["M",e-n,t-n],["L",e+n,t-n],["L",e+n,t+n],["L",e-n,t+n],["Z"]]},diamond:function(e,t,n){return[["M",e-n,t],["L",e,t-n],["L",e+n,t],["L",e,t+n],["Z"]]},triangle:function(e,t,n){var r=n*Math.sin(1/3*Math.PI);return[["M",e-n,t+r],["L",e,t-r],["L",e+n,t+r],["Z"]]},"triangle-down":function(e,t,n){var r=n*Math.sin(1/3*Math.PI);return[["M",e-n,t-r],["L",e+n,t-r],["L",e,t+r],["Z"]]}},c=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return r.__extends(t,e),t.prototype.initAttrs=function(e){this._resetParamsCache()},t.prototype._resetParamsCache=function(){this.set("paramsCache",{})},t.prototype.onAttrChange=function(t,n,r){e.prototype.onAttrChange.call(this,t,n,r),-1!==["symbol","x","y","r","radius"].indexOf(t)&&this._resetParamsCache()},t.prototype.isOnlyHitBox=function(){return!0},t.prototype._getR=function(e){return i.isNil(e.r)?e.radius:e.r},t.prototype._getPath=function(){var e,n,r=this.attr(),i=r.x,a=r.y,l=r.symbol||"circle",u=this._getR(r);return s.isFunction(l)?(n=(e=l)(i,a,u),n=o.default(n)):n=(e=t.Symbols[l])(i,a,u),e?n:(console.warn(l+" marker is not supported."),null)},t.prototype.createPath=function(e){var t=this._getPath(),n=this.get("paramsCache");l.drawPath(this,e,{path:t},n)},t.Symbols=u,t}(a.default);t.default=c},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(4),i="\t\n\v\f\r   ᠎             　\u2028\u2029",o=new RegExp("([a-z])["+i+",]*((-?\\d*\\.?\\d*(?:e[\\-+]?\\d+)?["+i+"]*,?["+i+"]*)+)","ig"),a=new RegExp("(-?\\d*\\.?\\d*(?:e[\\-+]?\\d+)?)["+i+"]*,?["+i+"]*","ig");t.default=function(e){if(!e)return null;if(r.default(e))return e;var t={a:7,c:6,o:2,h:1,l:2,m:2,r:4,q:4,s:4,t:2,v:1,u:3,z:0},n=[];return String(e).replace(o,(function(e,r,i){var o=[],s=r.toLowerCase();if(i.replace(a,(function(e,t){t&&o.push(+t)})),"m"===s&&o.length>2&&(n.push([r].concat(o.splice(0,2))),s="l",r="m"===r?"l":"L"),"o"===s&&1===o.length&&n.push([r,o[0]]),"r"===s)n.push([r].concat(o));else for(;o.length>=t[s]&&(n.push([r].concat(o.splice(0,t[s]))),t[s]););return""})),n}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(1),i=n(108),o=n(0),a=n(28),s=n(165),l=n(162),u=n(60),c=n(166),d=n(167),h=n(341),f=n(74);function p(e,t,n){for(var r=!1,i=0;i<e.length;i++){var o=e[i];if(r=d.default(o,t,n))break}return r}var g=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return r.__extends(t,e),t.prototype.getDefaultAttrs=function(){var t=e.prototype.getDefaultAttrs.call(this);return r.__assign(r.__assign({},t),{startArrow:!1,endArrow:!1})},t.prototype.initAttrs=function(e){this._setPathArr(e.path),this.setArrow()},t.prototype.onAttrChange=function(t,n,r){e.prototype.onAttrChange.call(this,t,n,r),"path"===t&&this._setPathArr(n),this.setArrow()},t.prototype._setPathArr=function(e){this.attrs.path=s.default(e);var t=h.default.hasArc(e);this.set("hasArc",t),this.set("paramsCache",{}),this.set("segments",null),this.set("curve",null),this.set("tCache",null),this.set("totalLength",null)},t.prototype.getSegments=function(){var e=this.get("segements");return e||(e=l.default(this.attr("path")),this.set("segments",e)),e},t.prototype.setArrow=function(){var e=this.attr(),t=e.startArrow,n=e.endArrow;if(t){var r=this.getStartTangent();f.addStartArrow(this,e,r[0][0],r[0][1],r[1][0],r[1][1])}n&&(r=this.getEndTangent(),f.addEndArrow(this,e,r[0][0],r[0][1],r[1][0],r[1][1]))},t.prototype.isInStrokeOrPath=function(e,t,n,r,i){var o=this.getSegments(),a=this.get("hasArc"),s=!1;if(n){var l=this.getTotalLength();s=h.default.isPointInStroke(o,i,e,t,l)}if(!s&&r)if(a)s=c.default(this,e,t);else{var u=this.attr("path"),d=h.default.extractPolygons(u);s=p(d.polygons,e,t)||p(d.polylines,e,t)}return s},t.prototype.createPath=function(e){var t=this.attr(),n=this.get("paramsCache");u.drawPath(this,e,t,n)},t.prototype.afterDrawPath=function(e){var t=this.get("startArrowShape"),n=this.get("endArrowShape");t&&t.draw(e),n&&n.draw(e)},t.prototype.getTotalLength=function(){var e=this.get("totalLength");return o.isNil(e)?(this._calculateCurve(),this._setTcache(),this.get("totalLength")):e},t.prototype.getPoint=function(e){var t,n,r=this.get("tCache");r||(this._calculateCurve(),this._setTcache(),r=this.get("tCache"));var a=this.get("curve");if(!r||0===r.length)return a?{x:a[0][1],y:a[0][2]}:null;o.each(r,(function(r,i){e>=r[0]&&e<=r[1]&&(t=(e-r[0])/(r[1]-r[0]),n=i)}));var s=a[n];if(o.isNil(s)||o.isNil(n))return null;var l=s.length,u=a[n+1];return i.default.pointAt(s[l-2],s[l-1],u[1],u[2],u[3],u[4],u[5],u[6],t)},t.prototype._calculateCurve=function(){var e=this.attr().path;this.set("curve",h.default.pathToCurve(e))},t.prototype._setTcache=function(){var e,t,n,r,a=0,s=0,l=[],u=this.get("curve");u&&(o.each(u,(function(e,t){n=u[t+1],r=e.length,n&&(a+=i.default.length(e[r-2],e[r-1],n[1],n[2],n[3],n[4],n[5],n[6])||0)})),this.set("totalLength",a),0!==a?(o.each(u,(function(o,c){n=u[c+1],r=o.length,n&&((e=[])[0]=s/a,t=i.default.length(o[r-2],o[r-1],n[1],n[2],n[3],n[4],n[5],n[6]),s+=t||0,e[1]=s/a,l.push(e))})),this.set("tCache",l)):this.set("tCache",[]))},t.prototype.getStartTangent=function(){var e,t=this.getSegments();if(t.length>1){var n=t[0].currentPoint,r=t[1].currentPoint,i=t[1].startTangent;e=[],i?(e.push([n[0]-i[0],n[1]-i[1]]),e.push([n[0],n[1]])):(e.push([r[0],r[1]]),e.push([n[0],n[1]]))}return e},t.prototype.getEndTangent=function(){var e,t=this.getSegments(),n=t.length;if(n>1){var r=t[n-2].currentPoint,i=t[n-1].currentPoint,o=t[n-1].endTangent;e=[],o?(e.push([i[0]-o[0],i[1]-o[1]]),e.push([i[0],i[1]])):(e.push([r[0],r[1]]),e.push([i[0],i[1]]))}return e},t}(a.default);t.default=g},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(1),i=n(43),o=n(160),a=n(108),s=n(18),l=n(75),u=n(168),c=n(133),d=n(106),h=c.ext.transform;t.default=r.__assign({hasArc:function(e){for(var t=!1,n=e.length,r=0;r<n;r++){var i=e[r][0];if("C"===i||"A"===i||"Q"===i){t=!0;break}}return t},extractPolygons:function(e){for(var t=e.length,n=[],r=[],i=[],o=0;o<t;o++){var a=e[o],s=a[0];"M"===s?(i.length&&(r.push(i),i=[]),i.push([a[1],a[2]])):"Z"===s?i.length&&(n.push(i),i=[]):i.push([a[1],a[2]])}return i.length>0&&r.push(i),{polygons:n,polylines:r}},isPointInStroke:function(e,t,n,r,i){for(var c=!1,f=t/2,p=0;p<e.length;p++){var g=e[p],v=g.currentPoint,m=g.params,y=g.prePoint,b=g.box;if(!b||s.inBox(b.x-f,b.y-f,b.width+t,b.height+t,n,r)){switch(g.command){case"L":case"Z":c=l.default(y[0],y[1],v[0],v[1],t,n,r);break;case"Q":c=o.default.pointDistance(y[0],y[1],m[1],m[2],m[3],m[4],n,r)<=t/2;break;case"C":c=a.default.pointDistance(y[0],y[1],m[1],m[2],m[3],m[4],m[5],m[6],n,r,i)<=t/2;break;case"A":var C=g.arcParams,A=C.cx,_=C.cy,I=C.rx,x=C.ry,w=C.startAngle,M=C.endAngle,S=C.xRotation,T=[n,r,1],E=I>x?I:x,O=h(null,[["t",-A,-_],["r",-S],["s",1/(I>x?1:I/x),1/(I>x?x/I:1)]]);d.transformMat3(T,T,O),c=u.default(0,0,E,w,M,t,T[0],T[1])}if(c)break}}return c}},i.PathUtil)},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(1),i=n(28),o=n(169),a=n(167),s=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return r.__extends(t,e),t.prototype.isInStrokeOrPath=function(e,t,n,r,i){var s=this.attr().points,l=!1;return n&&(l=o.default(s,i,e,t,!0)),!l&&r&&(l=a.default(s,e,t)),l},t.prototype.createPath=function(e){var t=this.attr().points;if(!(t.length<2)){e.beginPath();for(var n=0;n<t.length;n++){var r=t[n];0===n?e.moveTo(r[0],r[1]):e.lineTo(r[0],r[1])}e.closePath()}},t}(i.default);t.default=s},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(1),i=n(40),o=n(170),a=n(0),s=n(28),l=n(169),u=n(74),c=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return r.__extends(t,e),t.prototype.getDefaultAttrs=function(){var t=e.prototype.getDefaultAttrs.call(this);return r.__assign(r.__assign({},t),{startArrow:!1,endArrow:!1})},t.prototype.initAttrs=function(e){this.setArrow()},t.prototype.onAttrChange=function(t,n,r){e.prototype.onAttrChange.call(this,t,n,r),this.setArrow(),-1!==["points"].indexOf(t)&&this._resetCache()},t.prototype._resetCache=function(){this.set("totalLength",null),this.set("tCache",null)},t.prototype.setArrow=function(){var e=this.attr(),t=this.attrs,n=t.points,r=t.startArrow,i=t.endArrow,o=n.length,a=n[0][0],s=n[0][1],l=n[o-1][0],c=n[o-1][1];r&&u.addStartArrow(this,e,n[1][0],n[1][1],a,s),i&&u.addEndArrow(this,e,n[o-2][0],n[o-2][1],l,c)},t.prototype.isFill=function(){return!1},t.prototype.isInStrokeOrPath=function(e,t,n,r,i){if(!n||!i)return!1;var o=this.attr().points;return l.default(o,i,e,t,!1)},t.prototype.isStroke=function(){return!0},t.prototype.createPath=function(e){var t=this.attr(),n=t.points,r=t.startArrow,i=t.endArrow,o=n.length;if(!(n.length<2)){var a,s=n[0][0],l=n[0][1],c=n[o-1][0],d=n[o-1][1];r&&r.d&&(s+=(a=u.getShortenOffset(s,l,n[1][0],n[1][1],r.d)).dx,l+=a.dy),i&&i.d&&(c-=(a=u.getShortenOffset(n[o-2][0],n[o-2][1],c,d,i.d)).dx,d-=a.dy),e.beginPath(),e.moveTo(s,l);for(var h=0;h<o-1;h++){var f=n[h];e.lineTo(f[0],f[1])}e.lineTo(c,d)}},t.prototype.afterDrawPath=function(e){var t=this.get("startArrowShape"),n=this.get("endArrowShape");t&&t.draw(e),n&&n.draw(e)},t.prototype.getTotalLength=function(){var e=this.attr().points,t=this.get("totalLength");return a.isNil(t)?(this.set("totalLength",o.default.length(e)),this.get("totalLength")):t},t.prototype.getPoint=function(e){var t,n,r=this.attr().points,o=this.get("tCache");return o||(this._setTcache(),o=this.get("tCache")),a.each(o,(function(r,i){e>=r[0]&&e<=r[1]&&(t=(e-r[0])/(r[1]-r[0]),n=i)})),i.default.pointAt(r[n][0],r[n][1],r[n+1][0],r[n+1][1],t)},t.prototype._setTcache=function(){var e=this.attr().points;if(e&&0!==e.length){var t=this.getTotalLength();if(!(t<=0)){var n,r,o=0,s=[];a.each(e,(function(a,l){e[l+1]&&((n=[])[0]=o/t,r=i.default.length(a[0],a[1],e[l+1][0],e[l+1][1]),o+=r,n[1]=o/t,s.push(n))})),this.set("tCache",s)}}},t.prototype.getStartTangent=function(){var e=this.attr().points,t=[];return t.push([e[1][0],e[1][1]]),t.push([e[0][0],e[0][1]]),t},t.prototype.getEndTangent=function(){var e=this.attr().points,t=e.length-1,n=[];return n.push([e[t-1][0],e[t-1][1]]),n.push([e[t][0],e[t][1]]),n},t}(s.default);t.default=c},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.distanceAtSegment=t.angleAtSegments=t.pointAtSegments=t.lengthOfSegment=void 0;var r=n(40),i=n(33);function o(e){for(var t=0,n=[],r=0;r<e.length-1;r++){var o=e[r],a=e[r+1],s=i.distance(o[0],o[1],a[0],a[1]),l={from:o,to:a,length:s};n.push(l),t+=s}return{segments:n,totalLength:t}}t.lengthOfSegment=function(e){if(e.length<2)return 0;for(var t=0,n=0;n<e.length-1;n++){var r=e[n],o=e[n+1];t+=i.distance(r[0],r[1],o[0],o[1])}return t},t.pointAtSegments=function(e,t){if(t>1||t<0||e.length<2)return null;var n=o(e),i=n.segments,a=n.totalLength;if(0===a)return{x:e[0][0],y:e[0][1]};for(var s=0,l=null,u=0;u<i.length;u++){var c=i[u],d=c.from,h=c.to,f=c.length/a;if(t>=s&&t<=s+f){var p=(t-s)/f;l=r.default.pointAt(d[0],d[1],h[0],h[1],p);break}s+=f}return l},t.angleAtSegments=function(e,t){if(t>1||t<0||e.length<2)return 0;for(var n=o(e),r=n.segments,i=n.totalLength,a=0,s=0,l=0;l<r.length;l++){var u=r[l],c=u.from,d=u.to,h=u.length/i;if(t>=a&&t<=a+h){s=Math.atan2(d[1]-c[1],d[0]-c[0]);break}a+=h}return s},t.distanceAtSegment=function(e,t,n){for(var i=1/0,o=0;o<e.length-1;o++){var a=e[o],s=e[o+1],l=r.default.pointDistance(a[0],a[1],s[0],s[1],t,n);l<i&&(i=l)}return i}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(1),i=n(28),o=n(156),a=n(18),s=n(346),l=n(347),u=n(166),c=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return r.__extends(t,e),t.prototype.getDefaultAttrs=function(){var t=e.prototype.getDefaultAttrs.call(this);return r.__assign(r.__assign({},t),{x:0,y:0,width:0,height:0,radius:0})},t.prototype.isInStrokeOrPath=function(e,t,n,r,i){var o=this.attr(),c=o.x,d=o.y,h=o.width,f=o.height,p=o.radius;if(p){var g=!1;return n&&(g=l.default(c,d,h,f,p,i,e,t)),!g&&r&&(g=u.default(this,e,t)),g}var v=i/2;return r&&n?a.inBox(c-v,d-v,h+v,f+v,e,t):r?a.inBox(c,d,h,f,e,t):n?s.default(c,d,h,f,i,e,t):void 0},t.prototype.createPath=function(e){var t=this.attr(),n=t.x,r=t.y,i=t.width,a=t.height,s=t.radius;if(e.beginPath(),0===s)e.rect(n,r,i,a);else{var l=o.parseRadius(s),u=l[0],c=l[1],d=l[2],h=l[3];e.moveTo(n+u,r),e.lineTo(n+i-c,r),0!==c&&e.arc(n+i-c,r+c,c,-Math.PI/2,0),e.lineTo(n+i,r+a-d),0!==d&&e.arc(n+i-d,r+a-d,d,0,Math.PI/2),e.lineTo(n+h,r+a),0!==h&&e.arc(n+h,r+a-h,h,Math.PI/2,Math.PI),e.lineTo(n,r+u),0!==u&&e.arc(n+u,r+u,u,Math.PI,1.5*Math.PI),e.closePath()}},t}(i.default);t.default=c},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(18);t.default=function(e,t,n,i,o,a,s){var l=o/2;return r.inBox(e-l,t-l,n,o,a,s)||r.inBox(e+n-l,t-l,o,i,a,s)||r.inBox(e+l,t+i-l,n,o,a,s)||r.inBox(e-l,t+l,o,i,a,s)}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(75),i=n(168);t.default=function(e,t,n,o,a,s,l,u){return r.default(e+a,t,e+n-a,t,s,l,u)||r.default(e+n,t+a,e+n,t+o-a,s,l,u)||r.default(e+n-a,t+o,e+a,t+o,s,l,u)||r.default(e,t+o-a,e,t+a,s,l,u)||i.default(e+n-a,t+a,a,1.5*Math.PI,2*Math.PI,s,l,u)||i.default(e+n-a,t+o-a,a,0,.5*Math.PI,s,l,u)||i.default(e+a,t+o-a,a,.5*Math.PI,Math.PI,s,l,u)||i.default(e+a,t+a,a,Math.PI,1.5*Math.PI,s,l,u)}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(1),i=n(28),o=n(18),a=n(158),s=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return r.__extends(t,e),t.prototype.getDefaultAttrs=function(){var t=e.prototype.getDefaultAttrs.call(this);return r.__assign(r.__assign({},t),{x:0,y:0,text:null,fontSize:12,fontFamily:"sans-serif",fontStyle:"normal",fontWeight:"normal",fontVariant:"normal",textAlign:"start",textBaseline:"bottom"})},t.prototype.isOnlyHitBox=function(){return!0},t.prototype.initAttrs=function(e){this._assembleFont(),e.text&&this._setText(e.text)},t.prototype._assembleFont=function(){var e=this.attrs;e.font=a.assembleFont(e)},t.prototype._setText=function(e){var t=null;o.isString(e)&&-1!==e.indexOf("\n")&&(t=e.split("\n")),this.set("textArr",t)},t.prototype.onAttrChange=function(t,n,r){e.prototype.onAttrChange.call(this,t,n,r),t.startsWith("font")&&this._assembleFont(),"text"===t&&this._setText(n)},t.prototype._getSpaceingY=function(){var e=this.attrs,t=e.lineHeight,n=1*e.fontSize;return t?t-n:.14*n},t.prototype._drawTextArr=function(e,t,n){var r,i=this.attrs,s=i.textBaseline,l=i.x,u=i.y,c=1*i.fontSize,d=this._getSpaceingY(),h=a.getTextHeight(i.text,i.fontSize,i.lineHeight);o.each(t,(function(t,i){r=u+i*(d+c)-h+c,"middle"===s&&(r+=h-c-(h-c)/2),"top"===s&&(r+=h-c),n?e.fillText(t,l,r):e.strokeText(t,l,r)}))},t.prototype._drawText=function(e,t){var n=this.attr(),r=n.x,i=n.y,o=this.get("textArr");if(o)this._drawTextArr(e,o,t);else{var a=n.text;t?e.fillText(a,r,i):e.strokeText(a,r,i)}},t.prototype.strokeAndFill=function(e){var t=this.attrs,n=t.lineWidth,r=t.opacity,i=t.strokeOpacity,a=t.fillOpacity;this.isStroke()&&n>0&&(o.isNil(i)||1===i||(e.globalAlpha=r),this.stroke(e)),this.isFill()&&(o.isNil(a)||1===a?this.fill(e):(e.globalAlpha=a,this.fill(e),e.globalAlpha=r)),this.afterDrawPath(e)},t.prototype.fill=function(e){this._drawText(e,!0)},t.prototype.stroke=function(e){this._drawText(e,!1)},t}(i.default);t.default=s},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(1),i=n(0),o=n(19),a=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.type="circle",t.canFill=!0,t.canStroke=!0,t}return r.__extends(t,e),t.prototype.getDefaultAttrs=function(){var t=e.prototype.getDefaultAttrs.call(this);return r.__assign(r.__assign({},t),{x:0,y:0,r:0})},t.prototype.createPath=function(e,t){var n=this.attr(),r=this.get("el");i.each(t||n,(function(e,t){"x"===t||"y"===t?r.setAttribute("c"+t,e):o.SVG_ATTR_MAP[t]&&r.setAttribute(o.SVG_ATTR_MAP[t],e)}))},t}(n(23).default);t.default=a},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(1),i=n(0),o=n(19),a=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.type="dom",t.canFill=!1,t.canStroke=!1,t}return r.__extends(t,e),t.prototype.createPath=function(e,t){var n=this.attr(),r=this.get("el");if(i.each(t||n,(function(e,t){o.SVG_ATTR_MAP[t]&&r.setAttribute(o.SVG_ATTR_MAP[t],e)})),"function"==typeof n.html){var a=n.html.call(this,n);if(a instanceof Element||a instanceof HTMLDocument){for(var s=r.childNodes,l=s.length-1;l>=0;l--)r.removeChild(s[l]);r.appendChild(a)}else r.innerHTML=a}else r.innerHTML=n.html},t}(n(23).default);t.default=a},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(1),i=n(0),o=n(19),a=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.type="ellipse",t.canFill=!0,t.canStroke=!0,t}return r.__extends(t,e),t.prototype.getDefaultAttrs=function(){var t=e.prototype.getDefaultAttrs.call(this);return r.__assign(r.__assign({},t),{x:0,y:0,rx:0,ry:0})},t.prototype.createPath=function(e,t){var n=this.attr(),r=this.get("el");i.each(t||n,(function(e,t){"x"===t||"y"===t?r.setAttribute("c"+t,e):o.SVG_ATTR_MAP[t]&&r.setAttribute(o.SVG_ATTR_MAP[t],e)}))},t}(n(23).default);t.default=a},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(1),i=n(0),o=n(19),a=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.type="image",t.canFill=!1,t.canStroke=!1,t}return r.__extends(t,e),t.prototype.getDefaultAttrs=function(){var t=e.prototype.getDefaultAttrs.call(this);return r.__assign(r.__assign({},t),{x:0,y:0,width:0,height:0})},t.prototype.createPath=function(e,t){var n=this,r=this.attr(),a=this.get("el");i.each(t||r,(function(e,t){"img"===t?n._setImage(r.img):o.SVG_ATTR_MAP[t]&&a.setAttribute(o.SVG_ATTR_MAP[t],e)}))},t.prototype.setAttr=function(e,t){this.attrs[e]=t,"img"===e&&this._setImage(t)},t.prototype._setImage=function(e){var t=this.attr(),n=this.get("el");if(i.isString(e))n.setAttribute("href",e);else if(e instanceof window.Image)t.width||(n.setAttribute("width",e.width),this.attr("width",e.width)),t.height||(n.setAttribute("height",e.height),this.attr("height",e.height)),n.setAttribute("href",e.src);else if(e instanceof HTMLElement&&i.isString(e.nodeName)&&"CANVAS"===e.nodeName.toUpperCase())n.setAttribute("href",e.toDataURL());else if(e instanceof ImageData){var r=document.createElement("canvas");r.setAttribute("width",""+e.width),r.setAttribute("height",""+e.height),r.getContext("2d").putImageData(e,0,0),t.width||(n.setAttribute("width",""+e.width),this.attr("width",e.width)),t.height||(n.setAttribute("height",""+e.height),this.attr("height",e.height)),n.setAttribute("href",r.toDataURL())}},t}(n(23).default);t.default=a},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(1),i=n(40),o=n(0),a=n(19),s=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.type="line",t.canFill=!1,t.canStroke=!0,t}return r.__extends(t,e),t.prototype.getDefaultAttrs=function(){var t=e.prototype.getDefaultAttrs.call(this);return r.__assign(r.__assign({},t),{x1:0,y1:0,x2:0,y2:0,startArrow:!1,endArrow:!1})},t.prototype.createPath=function(e,t){var n=this.attr(),r=this.get("el");o.each(t||n,(function(t,i){if("startArrow"===i||"endArrow"===i)if(t){var s=o.isObject(t)?e.addArrow(n,a.SVG_ATTR_MAP[i]):e.getDefaultArrow(n,a.SVG_ATTR_MAP[i]);r.setAttribute(a.SVG_ATTR_MAP[i],"url(#"+s+")")}else r.removeAttribute(a.SVG_ATTR_MAP[i]);else a.SVG_ATTR_MAP[i]&&r.setAttribute(a.SVG_ATTR_MAP[i],t)}))},t.prototype.getTotalLength=function(){var e=this.attr(),t=e.x1,n=e.y1,r=e.x2,o=e.y2;return i.default.length(t,n,r,o)},t.prototype.getPoint=function(e){var t=this.attr(),n=t.x1,r=t.y1,o=t.x2,a=t.y2;return i.default.pointAt(n,r,o,a,e)},t}(n(23).default);t.default=s},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(1),i=n(0),o=n(23),a=n(355),s=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.type="marker",t.canFill=!0,t.canStroke=!0,t}return r.__extends(t,e),t.prototype.createPath=function(e){this.get("el").setAttribute("d",this._assembleMarker())},t.prototype._assembleMarker=function(){var e=this._getPath();return i.isArray(e)?e.map((function(e){return e.join(" ")})).join(""):e},t.prototype._getPath=function(){var e,t=this.attr(),n=t.x,r=t.y,o=t.r||t.radius,s=t.symbol||"circle";return(e=i.isFunction(s)?s:a.default.get(s))?e(n,r,o):(console.warn(e+" symbol is not exist."),null)},t.symbolsFactory=a.default,t}(o.default);t.default=s},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r={circle:function(e,t,n){return[["M",e,t],["m",-n,0],["a",n,n,0,1,0,2*n,0],["a",n,n,0,1,0,2*-n,0]]},square:function(e,t,n){return[["M",e-n,t-n],["L",e+n,t-n],["L",e+n,t+n],["L",e-n,t+n],["Z"]]},diamond:function(e,t,n){return[["M",e-n,t],["L",e,t-n],["L",e+n,t],["L",e,t+n],["Z"]]},triangle:function(e,t,n){var r=n*Math.sin(1/3*Math.PI);return[["M",e-n,t+r],["L",e,t-r],["L",e+n,t+r],["z"]]},triangleDown:function(e,t,n){var r=n*Math.sin(1/3*Math.PI);return[["M",e-n,t-r],["L",e+n,t-r],["L",e,t+r],["Z"]]}};t.default={get:function(e){return r[e]},register:function(e,t){r[e]=t},remove:function(e){delete r[e]},getAll:function(){return r}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(1),i=n(0),o=n(19),a=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.type="path",t.canFill=!0,t.canStroke=!0,t}return r.__extends(t,e),t.prototype.getDefaultAttrs=function(){var t=e.prototype.getDefaultAttrs.call(this);return r.__assign(r.__assign({},t),{startArrow:!1,endArrow:!1})},t.prototype.createPath=function(e,t){var n=this,r=this.attr(),a=this.get("el");i.each(t||r,(function(t,s){if("path"===s&&i.isArray(t))a.setAttribute("d",n._formatPath(t));else if("startArrow"===s||"endArrow"===s)if(t){var l=i.isObject(t)?e.addArrow(r,o.SVG_ATTR_MAP[s]):e.getDefaultArrow(r,o.SVG_ATTR_MAP[s]);a.setAttribute(o.SVG_ATTR_MAP[s],"url(#"+l+")")}else a.removeAttribute(o.SVG_ATTR_MAP[s]);else o.SVG_ATTR_MAP[s]&&a.setAttribute(o.SVG_ATTR_MAP[s],t)}))},t.prototype._formatPath=function(e){var t=e.map((function(e){return e.join(" ")})).join("");return~t.indexOf("NaN")?"":t},t.prototype.getTotalLength=function(){var e=this.get("el");return e?e.getTotalLength():null},t.prototype.getPoint=function(e){var t=this.get("el"),n=this.getTotalLength();if(0===n)return null;var r=t?t.getPointAtLength(e*n):null;return r?{x:r.x,y:r.y}:null},t}(n(23).default);t.default=a},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(1),i=n(0),o=n(19),a=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.type="polygon",t.canFill=!0,t.canStroke=!0,t}return r.__extends(t,e),t.prototype.createPath=function(e,t){var n=this.attr(),r=this.get("el");i.each(t||n,(function(e,t){"points"===t&&i.isArray(e)&&e.length>=2?r.setAttribute("points",e.map((function(e){return e[0]+","+e[1]})).join(" ")):o.SVG_ATTR_MAP[t]&&r.setAttribute(o.SVG_ATTR_MAP[t],e)}))},t}(n(23).default);t.default=a},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(1),i=n(40),o=n(170),a=n(0),s=n(19),l=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.type="polyline",t.canFill=!0,t.canStroke=!0,t}return r.__extends(t,e),t.prototype.getDefaultAttrs=function(){var t=e.prototype.getDefaultAttrs.call(this);return r.__assign(r.__assign({},t),{startArrow:!1,endArrow:!1})},t.prototype.onAttrChange=function(t,n,r){e.prototype.onAttrChange.call(this,t,n,r),-1!==["points"].indexOf(t)&&this._resetCache()},t.prototype._resetCache=function(){this.set("totalLength",null),this.set("tCache",null)},t.prototype.createPath=function(e,t){var n=this.attr(),r=this.get("el");a.each(t||n,(function(e,t){"points"===t&&a.isArray(e)&&e.length>=2?r.setAttribute("points",e.map((function(e){return e[0]+","+e[1]})).join(" ")):s.SVG_ATTR_MAP[t]&&r.setAttribute(s.SVG_ATTR_MAP[t],e)}))},t.prototype.getTotalLength=function(){var e=this.attr().points,t=this.get("totalLength");return a.isNil(t)?(this.set("totalLength",o.default.length(e)),this.get("totalLength")):t},t.prototype.getPoint=function(e){var t,n,r=this.attr().points,o=this.get("tCache");return o||(this._setTcache(),o=this.get("tCache")),a.each(o,(function(r,i){e>=r[0]&&e<=r[1]&&(t=(e-r[0])/(r[1]-r[0]),n=i)})),i.default.pointAt(r[n][0],r[n][1],r[n+1][0],r[n+1][1],t)},t.prototype._setTcache=function(){var e=this.attr().points;if(e&&0!==e.length){var t=this.getTotalLength();if(!(t<=0)){var n,r,o=0,s=[];a.each(e,(function(a,l){e[l+1]&&((n=[])[0]=o/t,r=i.default.length(a[0],a[1],e[l+1][0],e[l+1][1]),o+=r,n[1]=o/t,s.push(n))})),this.set("tCache",s)}}},t.prototype.getStartTangent=function(){var e=this.attr().points,t=[];return t.push([e[1][0],e[1][1]]),t.push([e[0][0],e[0][1]]),t},t.prototype.getEndTangent=function(){var e=this.attr().points,t=e.length-1,n=[];return n.push([e[t-1][0],e[t-1][1]]),n.push([e[t][0],e[t][1]]),n},t}(n(23).default);t.default=l},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(1),i=n(0),o=n(23),a=n(19),s=n(360),l=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.type="rect",t.canFill=!0,t.canStroke=!0,t}return r.__extends(t,e),t.prototype.getDefaultAttrs=function(){var t=e.prototype.getDefaultAttrs.call(this);return r.__assign(r.__assign({},t),{x:0,y:0,width:0,height:0,radius:0})},t.prototype.createPath=function(e,t){var n=this,r=this.attr(),o=this.get("el"),s=!1,l=["x","y","width","height","radius"];i.each(t||r,(function(e,t){-1===l.indexOf(t)||s?-1===l.indexOf(t)&&a.SVG_ATTR_MAP[t]&&o.setAttribute(a.SVG_ATTR_MAP[t],e):(o.setAttribute("d",n._assembleRect(r)),s=!0)}))},t.prototype._assembleRect=function(e){var t=e.x,n=e.y,r=e.width,o=e.height,a=e.radius;if(!a)return"M "+t+","+n+" l "+r+",0 l 0,"+o+" l"+-r+" 0 z";var l=s.parseRadius(a);return i.isArray(a)?1===a.length?l.r1=l.r2=l.r3=l.r4=a[0]:2===a.length?(l.r1=l.r3=a[0],l.r2=l.r4=a[1]):3===a.length?(l.r1=a[0],l.r2=l.r4=a[1],l.r3=a[2]):(l.r1=a[0],l.r2=a[1],l.r3=a[2],l.r4=a[3]):l.r1=l.r2=l.r3=l.r4=a,[["M "+(t+l.r1)+","+n],["l "+(r-l.r1-l.r2)+",0"],["a "+l.r2+","+l.r2+",0,0,1,"+l.r2+","+l.r2],["l 0,"+(o-l.r2-l.r3)],["a "+l.r3+","+l.r3+",0,0,1,"+-l.r3+","+l.r3],["l "+(l.r3+l.r4-r)+",0"],["a "+l.r4+","+l.r4+",0,0,1,"+-l.r4+","+-l.r4],["l 0,"+(l.r4+l.r1-o)],["a "+l.r1+","+l.r1+",0,0,1,"+l.r1+","+-l.r1],["z"]].join(" ")},t}(o.default);t.default=l},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.parsePath=t.parseRadius=void 0;var r=n(0),i=/[MLHVQTCSAZ]([^MLHVQTCSAZ]*)/gi,o=/[^\s\,]+/gi;t.parseRadius=function(e){var t=0,n=0,i=0,o=0;return r.isArray(e)?1===e.length?t=n=i=o=e[0]:2===e.length?(t=i=e[0],n=o=e[1]):3===e.length?(t=e[0],n=o=e[1],i=e[2]):(t=e[0],n=e[1],i=e[2],o=e[3]):t=n=i=o=e,{r1:t,r2:n,r3:i,r4:o}},t.parsePath=function(e){return e=e||[],r.isArray(e)?e:r.isString(e)?(e=e.match(i),r.each(e,(function(t,n){if((t=t.match(o))[0].length>1){var i=t[0].charAt(0);t.splice(1,0,t[0].substr(1)),t[0]=i}r.each(t,(function(e,n){isNaN(e)||(t[n]=+e)})),e[n]=t})),e):void 0}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(1),i=n(0),o=n(132),a=n(61),s=n(19),l=n(23),u={top:"before-edge",middle:"central",bottom:"after-edge",alphabetic:"baseline",hanging:"hanging"},c={top:"text-before-edge",middle:"central",bottom:"text-after-edge",alphabetic:"alphabetic",hanging:"hanging"},d={left:"left",start:"left",center:"middle",right:"end",end:"end"},h=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.type="text",t.canFill=!0,t.canStroke=!0,t}return r.__extends(t,e),t.prototype.getDefaultAttrs=function(){var t=e.prototype.getDefaultAttrs.call(this);return r.__assign(r.__assign({},t),{x:0,y:0,text:null,fontSize:12,fontFamily:"sans-serif",fontStyle:"normal",fontWeight:"normal",fontVariant:"normal",textAlign:"start",textBaseline:"bottom"})},t.prototype.createPath=function(e,t){var n=this,r=this.attr(),o=this.get("el");this._setFont(),i.each(t||r,(function(e,t){"text"===t?n._setText(""+e):"matrix"===t&&e?a.setTransform(n):s.SVG_ATTR_MAP[t]&&o.setAttribute(s.SVG_ATTR_MAP[t],e)})),o.setAttribute("paint-order","stroke"),o.setAttribute("style","stroke-linecap:butt; stroke-linejoin:miter;")},t.prototype._setFont=function(){var e=this.get("el"),t=this.attr(),n=t.textBaseline,r=t.textAlign,i=o.detect();i&&"firefox"===i.name?e.setAttribute("dominant-baseline",c[n]||"alphabetic"):e.setAttribute("alignment-baseline",u[n]||"baseline"),e.setAttribute("text-anchor",d[r]||"left")},t.prototype._setText=function(e){var t=this.get("el"),n=this.attr(),r=n.x,o=n.textBaseline,a=void 0===o?"bottom":o;if(e)if(~e.indexOf("\n")){var s=e.split("\n"),l=s.length-1,u="";i.each(s,(function(e,t){0===t?"alphabetic"===a?u+='<tspan x="'+r+'" dy="'+-l+'em">'+e+"</tspan>":"top"===a?u+='<tspan x="'+r+'" dy="0.9em">'+e+"</tspan>":"middle"===a?u+='<tspan x="'+r+'" dy="'+-(l-1)/2+'em">'+e+"</tspan>":"bottom"===a?u+='<tspan x="'+r+'" dy="-'+(l+.3)+'em">'+e+"</tspan>":"hanging"===a&&(u+='<tspan x="'+r+'" dy="'+(-(l-1)-.3)+'em">'+e+"</tspan>"):u+='<tspan x="'+r+'" dy="1em">'+e+"</tspan>"})),t.innerHTML=u}else t.innerHTML=e;else t.innerHTML=""},t}(l.default);t.default=h},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(0),i=n(363),o=n(364),a=n(365),s=n(366),l=n(367),u=n(29),c=function(){function e(e){var t=u.createSVGElement("defs"),n=r.uniqueId("defs_");t.id=n,e.appendChild(t),this.children=[],this.defaultArrow={},this.el=t,this.canvas=e}return e.prototype.find=function(e,t){for(var n=this.children,r=null,i=0;i<n.length;i++)if(n[i].match(e,t)){r=n[i].id;break}return r},e.prototype.findById=function(e){for(var t=this.children,n=null,r=0;r<t.length;r++)if(t[r].id===e){n=t[r];break}return n},e.prototype.add=function(e){this.children.push(e),e.canvas=this.canvas,e.parent=this},e.prototype.getDefaultArrow=function(e,t){var n=e.stroke||e.strokeStyle;if(this.defaultArrow[n])return this.defaultArrow[n].id;var r=new a.default(e,t);return this.defaultArrow[n]=r,this.el.appendChild(r.el),this.add(r),r.id},e.prototype.addGradient=function(e){var t=new i.default(e);return this.el.appendChild(t.el),this.add(t),t.id},e.prototype.addArrow=function(e,t){var n=new a.default(e,t);return this.el.appendChild(n.el),this.add(n),n.id},e.prototype.addShadow=function(e){var t=new o.default(e);return this.el.appendChild(t.el),this.add(t),t.id},e.prototype.addPattern=function(e){var t=new l.default(e);return this.el.appendChild(t.el),this.add(t),t.id},e.prototype.addClip=function(e){var t=new s.default(e);return this.el.appendChild(t.el),this.add(t),t.id},e}();t.default=c},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(0),i=n(29),o=/^l\s*\(\s*([\d.]+)\s*\)\s*(.*)/i,a=/^r\s*\(\s*([\d.]+)\s*,\s*([\d.]+)\s*,\s*([\d.]+)\s*\)\s*(.*)/i,s=/[\d.]+:(#[^\s]+|[^\)]+\))/gi;function l(e){var t=e.match(s);if(!t)return"";var n="";return t.sort((function(e,t){return e=e.split(":"),t=t.split(":"),Number(e[0])-Number(t[0])})),r.each(t,(function(e){e=e.split(":"),n+='<stop offset="'+e[0]+'" stop-color="'+e[1]+'"></stop>'})),n}var u=function(){function e(e){this.cfg={};var t=null,n=r.uniqueId("gradient_");return"l"===e.toLowerCase()[0]?function(e,t){var n,i,a=o.exec(e),s=r.mod(r.toRadian(parseFloat(a[1])),2*Math.PI),u=a[2];s>=0&&s<.5*Math.PI?(n={x:0,y:0},i={x:1,y:1}):.5*Math.PI<=s&&s<Math.PI?(n={x:1,y:0},i={x:0,y:1}):Math.PI<=s&&s<1.5*Math.PI?(n={x:1,y:1},i={x:0,y:0}):(n={x:0,y:1},i={x:1,y:0});var c=Math.tan(s),d=c*c,h=(i.x-n.x+c*(i.y-n.y))/(d+1)+n.x,f=c*(i.x-n.x+c*(i.y-n.y))/(d+1)+n.y;t.setAttribute("x1",n.x),t.setAttribute("y1",n.y),t.setAttribute("x2",h),t.setAttribute("y2",f),t.innerHTML=l(u)}(e,t=i.createSVGElement("linearGradient")):function(e,t){var n=a.exec(e),r=parseFloat(n[1]),i=parseFloat(n[2]),o=parseFloat(n[3]),s=n[4];t.setAttribute("cx",r),t.setAttribute("cy",i),t.setAttribute("r",o),t.innerHTML=l(s)}(e,t=i.createSVGElement("radialGradient")),t.setAttribute("id",n),this.el=t,this.id=n,this.cfg=e,this}return e.prototype.match=function(e,t){return this.cfg===t},e}();t.default=u},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(0),i=n(29),o={shadowColor:"color",shadowOpacity:"opacity",shadowBlur:"blur",shadowOffsetX:"dx",shadowOffsetY:"dy"},a={x:"-40%",y:"-40%",width:"200%",height:"200%"},s=function(){function e(e){this.type="filter",this.cfg={},this.type="filter";var t=i.createSVGElement("filter");return r.each(a,(function(e,n){t.setAttribute(n,e)})),this.el=t,this.id=r.uniqueId("filter_"),this.el.id=this.id,this.cfg=e,this._parseShadow(e,t),this}return e.prototype.match=function(e,t){if(this.type!==e)return!1;var n=!0,i=this.cfg;return r.each(Object.keys(i),(function(e){if(i[e]!==t[e])return n=!1,!1})),n},e.prototype.update=function(e,t){var n=this.cfg;return n[o[e]]=t,this._parseShadow(n,this.el),this},e.prototype._parseShadow=function(e,t){var n='<feDropShadow\n      dx="'+(e.dx||0)+'"\n      dy="'+(e.dy||0)+'"\n      stdDeviation="'+(e.blur?e.blur/10:0)+'"\n      flood-color="'+(e.color?e.color:"#000")+'"\n      flood-opacity="'+(e.opacity?e.opacity:1)+'"\n      />';t.innerHTML=n},e}();t.default=s},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(0),i=n(29),o=function(){function e(e,t){this.cfg={};var n=i.createSVGElement("marker"),o=r.uniqueId("marker_");n.setAttribute("id",o);var a=i.createSVGElement("path");a.setAttribute("stroke",e.stroke||"none"),a.setAttribute("fill",e.fill||"none"),n.appendChild(a),n.setAttribute("overflow","visible"),n.setAttribute("orient","auto-start-reverse"),this.el=n,this.child=a,this.id=o;var s=e["marker-start"===t?"startArrow":"endArrow"];return this.stroke=e.stroke||"#000",!0===s?this._setDefaultPath(t,a):(this.cfg=s,this._setMarker(e.lineWidth,a)),this}return e.prototype.match=function(){return!1},e.prototype._setDefaultPath=function(e,t){var n=this.el;t.setAttribute("d","M0,0 L"+10*Math.cos(Math.PI/6)+",5 L0,10"),n.setAttribute("refX",""+10*Math.cos(Math.PI/6)),n.setAttribute("refY","5")},e.prototype._setMarker=function(e,t){var n=this.el,i=this.cfg.path,o=this.cfg.d;r.isArray(i)&&(i=i.map((function(e){return e.join(" ")})).join("")),t.setAttribute("d",i),n.appendChild(t),o&&n.setAttribute("refX",""+o/e)},e.prototype.update=function(e){var t=this.child;t.attr?t.attr("fill",e):t.setAttribute("fill",e)},e}();t.default=o},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(0),i=n(29),o=function(){function e(e){this.type="clip",this.cfg={};var t=i.createSVGElement("clipPath");this.el=t,this.id=r.uniqueId("clip_"),t.id=this.id;var n=e.cfg.el;return t.appendChild(n),this.cfg=e,this}return e.prototype.match=function(){return!1},e.prototype.remove=function(){var e=this.el;e.parentNode.removeChild(e)},e}();t.default=o},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(0),i=n(29),o=/^p\s*\(\s*([axyn])\s*\)\s*(.*)/i,a=function(){function e(e){this.cfg={};var t=i.createSVGElement("pattern");t.setAttribute("patternUnits","userSpaceOnUse");var n=i.createSVGElement("image");t.appendChild(n);var a=r.uniqueId("pattern_");t.id=a,this.el=t,this.id=a,this.cfg=e;var s=o.exec(e)[2];n.setAttribute("href",s);var l=new Image;function u(){t.setAttribute("width",""+l.width),t.setAttribute("height",""+l.height)}return s.match(/^data:/i)||(l.crossOrigin="Anonymous"),l.src=s,l.complete?u():(l.onload=u,l.src=l.src),this}return e.prototype.match=function(e,t){return this.cfg===t},e}();t.default=a},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(26),i=n(0);r.angle=function(e,t){var n=r.dot(e,t)/(r.length(e)*r.length(t));return Math.acos(i.clamp(n,-1,1))},r.direction=function(e,t){return e[0]*t[1]-t[0]*e[1]},r.angleTo=function(e,t,n){var i=r.angle(e,t),o=r.direction(e,t)>=0;return n?o?2*Math.PI-i:i:o?i:2*Math.PI-i},r.vertical=function(e,t,n){return n?(e[0]=t[1],e[1]=-1*t[0]):(e[0]=-1*t[1],e[1]=t[0]),e},t.default=r},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(131);t.default=r},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(0),i=n(172);t.default=function(e,t){var n=e?r.clone(e):[1,0,0,0,1,0,0,0,1];return r.each(t,(function(e){switch(e[0]){case"t":i.default.translate(n,n,[e[1],e[2]]);break;case"s":i.default.scale(n,n,[e[1],e[2]]);break;case"r":i.default.rotate(n,n,e[1]);break;case"m":i.default.multiply(n,n,e[1]);break;default:return!1}})),n}},function(e,t,n){var r=n(372);e.exports={Graph:r.Graph,json:n(475),alg:n(476),version:r.version}},function(e,t,n){e.exports={Graph:n(111),version:n(474)}},function(e,t,n){var r=n(173);e.exports=function(e){return r(e,4)}},function(e,t){e.exports=function(){this.__data__=[],this.size=0}},function(e,t,n){var r=n(78),i=Array.prototype.splice;e.exports=function(e){var t=this.__data__,n=r(t,e);return!(n<0||(n==t.length-1?t.pop():i.call(t,n,1),--this.size,0))}},function(e,t,n){var r=n(78);e.exports=function(e){var t=this.__data__,n=r(t,e);return n<0?void 0:t[n][1]}},function(e,t,n){var r=n(78);e.exports=function(e){return r(this.__data__,e)>-1}},function(e,t,n){var r=n(78);e.exports=function(e,t){var n=this.__data__,i=r(n,e);return i<0?(++this.size,n.push([e,t])):n[i][1]=t,this}},function(e,t,n){var r=n(77);e.exports=function(){this.__data__=new r,this.size=0}},function(e,t){e.exports=function(e){var t=this.__data__,n=t.delete(e);return this.size=t.size,n}},function(e,t){e.exports=function(e){return this.__data__.get(e)}},function(e,t){e.exports=function(e){return this.__data__.has(e)}},function(e,t,n){var r=n(77),i=n(112),o=n(113);e.exports=function(e,t){var n=this.__data__;if(n instanceof r){var a=n.__data__;if(!i||a.length<199)return a.push([e,t]),this.size=++n.size,this;n=this.__data__=new o(a)}return n.set(e,t),this.size=n.size,this}},function(e,t,n){var r=n(62),i=n(388),o=n(25),a=n(175),s=/^\[object .+?Constructor\]$/,l=Function.prototype,u=Object.prototype,c=l.toString,d=u.hasOwnProperty,h=RegExp("^"+c.call(d).replace(/[\\^$.*+?()[\]{}|]/g,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$");e.exports=function(e){return!(!o(e)||i(e))&&(r(e)?h:s).test(a(e))}},function(e,t){function n(e){return(n="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}var r;r=function(){return this}();try{r=r||new Function("return this")()}catch(e){"object"===("undefined"==typeof window?"undefined":n(window))&&(r=window)}e.exports=r},function(e,t,n){var r=n(52),i=Object.prototype,o=i.hasOwnProperty,a=i.toString,s=r?r.toStringTag:void 0;e.exports=function(e){var t=o.call(e,s),n=e[s];try{e[s]=void 0;var r=!0}catch(e){}var i=a.call(e);return r&&(t?e[s]=n:delete e[s]),i}},function(e,t){var n=Object.prototype.toString;e.exports=function(e){return n.call(e)}},function(e,t,n){var r,i=n(389),o=(r=/[^.]+$/.exec(i&&i.keys&&i.keys.IE_PROTO||""))?"Symbol(src)_1."+r:"";e.exports=function(e){return!!o&&o in e}},function(e,t,n){var r=n(31)["__core-js_shared__"];e.exports=r},function(e,t){e.exports=function(e,t){return null==e?void 0:e[t]}},function(e,t,n){var r=n(392),i=n(77),o=n(112);e.exports=function(){this.size=0,this.__data__={hash:new r,map:new(o||i),string:new r}}},function(e,t,n){var r=n(393),i=n(394),o=n(395),a=n(396),s=n(397);function l(e){var t=-1,n=null==e?0:e.length;for(this.clear();++t<n;){var r=e[t];this.set(r[0],r[1])}}l.prototype.clear=r,l.prototype.delete=i,l.prototype.get=o,l.prototype.has=a,l.prototype.set=s,e.exports=l},function(e,t,n){var r=n(79);e.exports=function(){this.__data__=r?r(null):{},this.size=0}},function(e,t){e.exports=function(e){var t=this.has(e)&&delete this.__data__[e];return this.size-=t?1:0,t}},function(e,t,n){var r=n(79),i=Object.prototype.hasOwnProperty;e.exports=function(e){var t=this.__data__;if(r){var n=t[e];return"__lodash_hash_undefined__"===n?void 0:n}return i.call(t,e)?t[e]:void 0}},function(e,t,n){var r=n(79),i=Object.prototype.hasOwnProperty;e.exports=function(e){var t=this.__data__;return r?void 0!==t[e]:i.call(t,e)}},function(e,t,n){var r=n(79);e.exports=function(e,t){var n=this.__data__;return this.size+=this.has(e)?0:1,n[e]=r&&void 0===t?"__lodash_hash_undefined__":t,this}},function(e,t,n){var r=n(80);e.exports=function(e){var t=r(this,e).delete(e);return this.size-=t?1:0,t}},function(e,t){function n(e){return(n="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}e.exports=function(e){var t=n(e);return"string"==t||"number"==t||"symbol"==t||"boolean"==t?"__proto__"!==e:null===e}},function(e,t,n){var r=n(80);e.exports=function(e){return r(this,e).get(e)}},function(e,t,n){var r=n(80);e.exports=function(e){return r(this,e).has(e)}},function(e,t,n){var r=n(80);e.exports=function(e,t){var n=r(this,e),i=n.size;return n.set(e,t),this.size+=n.size==i?0:1,this}},function(e,t,n){var r=n(63),i=n(41);e.exports=function(e,t){return e&&r(t,i(t),e)}},function(e,t){e.exports=function(e,t){for(var n=-1,r=Array(e);++n<e;)r[n]=t(n);return r}},function(e,t,n){var r=n(45),i=n(34);e.exports=function(e){return i(e)&&"[object Arguments]"==r(e)}},function(e,t){e.exports=function(){return!1}},function(e,t,n){var r=n(45),i=n(116),o=n(34),a={};a["[object Float32Array]"]=a["[object Float64Array]"]=a["[object Int8Array]"]=a["[object Int16Array]"]=a["[object Int32Array]"]=a["[object Uint8Array]"]=a["[object Uint8ClampedArray]"]=a["[object Uint16Array]"]=a["[object Uint32Array]"]=!0,a["[object Arguments]"]=a["[object Array]"]=a["[object ArrayBuffer]"]=a["[object Boolean]"]=a["[object DataView]"]=a["[object Date]"]=a["[object Error]"]=a["[object Function]"]=a["[object Map]"]=a["[object Number]"]=a["[object Object]"]=a["[object RegExp]"]=a["[object Set]"]=a["[object String]"]=a["[object WeakMap]"]=!1,e.exports=function(e){return o(e)&&i(e.length)&&!!a[r(e)]}},function(e,t,n){var r=n(178)(Object.keys,Object);e.exports=r},function(e,t,n){var r=n(63),i=n(46);e.exports=function(e,t){return e&&r(t,i(t),e)}},function(e,t,n){var r=n(25),i=n(85),o=n(411),a=Object.prototype.hasOwnProperty;e.exports=function(e){if(!r(e))return o(e);var t=i(e),n=[];for(var s in e)("constructor"!=s||!t&&a.call(e,s))&&n.push(s);return n}},function(e,t){e.exports=function(e){var t=[];if(null!=e)for(var n in Object(e))t.push(n);return t}},function(e,t,n){var r=n(63),i=n(119);e.exports=function(e,t){return r(e,i(e),t)}},function(e,t,n){var r=n(63),i=n(183);e.exports=function(e,t){return r(e,i(e),t)}},function(e,t,n){var r=n(185),i=n(183),o=n(46);e.exports=function(e){return r(e,o,i)}},function(e,t,n){var r=n(44)(n(31),"DataView");e.exports=r},function(e,t,n){var r=n(44)(n(31),"Promise");e.exports=r},function(e,t,n){var r=n(44)(n(31),"WeakMap");e.exports=r},function(e,t){var n=Object.prototype.hasOwnProperty;e.exports=function(e){var t=e.length,r=new e.constructor(t);return t&&"string"==typeof e[0]&&n.call(e,"index")&&(r.index=e.index,r.input=e.input),r}},function(e,t,n){var r=n(121),i=n(420),o=n(421),a=n(422),s=n(188);e.exports=function(e,t,n){var l=e.constructor;switch(t){case"[object ArrayBuffer]":return r(e);case"[object Boolean]":case"[object Date]":return new l(+e);case"[object DataView]":return i(e,n);case"[object Float32Array]":case"[object Float64Array]":case"[object Int8Array]":case"[object Int16Array]":case"[object Int32Array]":case"[object Uint8Array]":case"[object Uint8ClampedArray]":case"[object Uint16Array]":case"[object Uint32Array]":return s(e,n);case"[object Map]":return new l;case"[object Number]":case"[object String]":return new l(e);case"[object RegExp]":return o(e);case"[object Set]":return new l;case"[object Symbol]":return a(e)}}},function(e,t,n){var r=n(121);e.exports=function(e,t){var n=t?r(e.buffer):e.buffer;return new e.constructor(n,e.byteOffset,e.byteLength)}},function(e,t){var n=/\w*$/;e.exports=function(e){var t=new e.constructor(e.source,n.exec(e));return t.lastIndex=e.lastIndex,t}},function(e,t,n){var r=n(52),i=r?r.prototype:void 0,o=i?i.valueOf:void 0;e.exports=function(e){return o?Object(o.call(e)):{}}},function(e,t,n){var r=n(424),i=n(84),o=n(117),a=o&&o.isMap,s=a?i(a):r;e.exports=s},function(e,t,n){var r=n(54),i=n(34);e.exports=function(e){return i(e)&&"[object Map]"==r(e)}},function(e,t,n){var r=n(426),i=n(84),o=n(117),a=o&&o.isSet,s=a?i(a):r;e.exports=s},function(e,t,n){var r=n(54),i=n(34);e.exports=function(e){return i(e)&&"[object Set]"==r(e)}},function(e,t){e.exports=function(e){return function(t,n,r){for(var i=-1,o=Object(t),a=r(t),s=a.length;s--;){var l=a[e?s:++i];if(!1===n(o[l],l,o))break}return t}}},function(e,t,n){var r=n(38);e.exports=function(e,t){return function(n,i){if(null==n)return n;if(!r(n))return e(n,i);for(var o=n.length,a=t?o:-1,s=Object(n);(t?a--:++a<o)&&!1!==i(s[a],a,s););return n}}},function(e,t,n){var r=n(87);e.exports=function(e,t){var n=[];return r(e,(function(e,r,i){t(e,r,i)&&n.push(e)})),n}},function(e,t,n){var r=n(431),i=n(439),o=n(200);e.exports=function(e){var t=i(e);return 1==t.length&&t[0][2]?o(t[0][0],t[0][1]):function(n){return n===e||r(n,e,t)}}},function(e,t,n){var r=n(76),i=n(195);e.exports=function(e,t,n,o){var a=n.length,s=a,l=!o;if(null==e)return!s;for(e=Object(e);a--;){var u=n[a];if(l&&u[2]?u[1]!==e[u[0]]:!(u[0]in e))return!1}for(;++a<s;){var c=(u=n[a])[0],d=e[c],h=u[1];if(l&&u[2]){if(void 0===d&&!(c in e))return!1}else{var f=new r;if(o)var p=o(d,h,c,e,t,f);if(!(void 0===p?i(h,d,3,o,f):p))return!1}}return!0}},function(e,t,n){var r=n(76),i=n(196),o=n(436),a=n(438),s=n(54),l=n(15),u=n(53),c=n(65),d="[object Object]",h=Object.prototype.hasOwnProperty;e.exports=function(e,t,n,f,p,g){var v=l(e),m=l(t),y=v?"[object Array]":s(e),b=m?"[object Array]":s(t),C=(y="[object Arguments]"==y?d:y)==d,A=(b="[object Arguments]"==b?d:b)==d,_=y==b;if(_&&u(e)){if(!u(t))return!1;v=!0,C=!1}if(_&&!C)return g||(g=new r),v||c(e)?i(e,t,n,f,p,g):o(e,t,y,n,f,p,g);if(!(1&n)){var I=C&&h.call(e,"__wrapped__"),x=A&&h.call(t,"__wrapped__");if(I||x){var w=I?e.value():e,M=x?t.value():t;return g||(g=new r),p(w,M,n,f,g)}}return!!_&&(g||(g=new r),a(e,t,n,f,p,g))}},function(e,t){e.exports=function(e){return this.__data__.set(e,"__lodash_hash_undefined__"),this}},function(e,t){e.exports=function(e){return this.__data__.has(e)}},function(e,t){e.exports=function(e,t){for(var n=-1,r=null==e?0:e.length;++n<r;)if(t(e[n],n,e))return!0;return!1}},function(e,t,n){var r=n(52),i=n(187),o=n(51),a=n(196),s=n(437),l=n(125),u=r?r.prototype:void 0,c=u?u.valueOf:void 0;e.exports=function(e,t,n,r,u,d,h){switch(n){case"[object DataView]":if(e.byteLength!=t.byteLength||e.byteOffset!=t.byteOffset)return!1;e=e.buffer,t=t.buffer;case"[object ArrayBuffer]":return!(e.byteLength!=t.byteLength||!d(new i(e),new i(t)));case"[object Boolean]":case"[object Date]":case"[object Number]":return o(+e,+t);case"[object Error]":return e.name==t.name&&e.message==t.message;case"[object RegExp]":case"[object String]":return e==t+"";case"[object Map]":var f=s;case"[object Set]":var p=1&r;if(f||(f=l),e.size!=t.size&&!p)return!1;var g=h.get(e);if(g)return g==t;r|=2,h.set(e,t);var v=a(f(e),f(t),r,u,d,h);return h.delete(e),v;case"[object Symbol]":if(c)return c.call(e)==c.call(t)}return!1}},function(e,t){e.exports=function(e){var t=-1,n=Array(e.size);return e.forEach((function(e,r){n[++t]=[r,e]})),n}},function(e,t,n){var r=n(184),i=Object.prototype.hasOwnProperty;e.exports=function(e,t,n,o,a,s){var l=1&n,u=r(e),c=u.length;if(c!=r(t).length&&!l)return!1;for(var d=c;d--;){var h=u[d];if(!(l?h in t:i.call(t,h)))return!1}var f=s.get(e),p=s.get(t);if(f&&p)return f==t&&p==e;var g=!0;s.set(e,t),s.set(t,e);for(var v=l;++d<c;){var m=e[h=u[d]],y=t[h];if(o)var b=l?o(y,m,h,t,e,s):o(m,y,h,e,t,s);if(!(void 0===b?m===y||a(m,y,n,o,s):b)){g=!1;break}v||(v="constructor"==h)}if(g&&!v){var C=e.constructor,A=t.constructor;C==A||!("constructor"in e)||!("constructor"in t)||"function"==typeof C&&C instanceof C&&"function"==typeof A&&A instanceof A||(g=!1)}return s.delete(e),s.delete(t),g}},function(e,t,n){var r=n(199),i=n(41);e.exports=function(e){for(var t=i(e),n=t.length;n--;){var o=t[n],a=e[o];t[n]=[o,a,r(a)]}return t}},function(e,t,n){var r=n(195),i=n(441),o=n(202),a=n(126),s=n(199),l=n(200),u=n(66);e.exports=function(e,t){return a(e)&&s(t)?l(u(e),t):function(n){var a=i(n,e);return void 0===a&&a===t?o(n,e):r(t,a,3)}}},function(e,t,n){var r=n(88);e.exports=function(e,t,n){var i=null==e?void 0:r(e,t);return void 0===i?n:i}},function(e,t,n){var r=n(443),i=/[^.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\\]|\\.)*?)\2)\]|(?=(?:\.|\[\])(?:\.|\[\]|$))/g,o=/\\(\\)?/g,a=r((function(e){var t=[];return 46===e.charCodeAt(0)&&t.push(""),e.replace(i,(function(e,n,r,i){t.push(r?i.replace(o,"$1"):n||e)})),t}));e.exports=a},function(e,t,n){var r=n(444);e.exports=function(e){var t=r(e,(function(e){return 500===n.size&&n.clear(),e})),n=t.cache;return t}},function(e,t,n){var r=n(113);function i(e,t){if("function"!=typeof e||null!=t&&"function"!=typeof t)throw new TypeError("Expected a function");var n=function n(){var r=arguments,i=t?t.apply(this,r):r[0],o=n.cache;if(o.has(i))return o.get(i);var a=e.apply(this,r);return n.cache=o.set(i,a)||o,a};return n.cache=new(i.Cache||r),n}i.Cache=r,e.exports=i},function(e,t,n){var r=n(52),i=n(90),o=n(15),a=n(55),s=r?r.prototype:void 0,l=s?s.toString:void 0;e.exports=function e(t){if("string"==typeof t)return t;if(o(t))return i(t,e)+"";if(a(t))return l?l.call(t):"";var n=t+"";return"0"==n&&1/t==-1/0?"-0":n}},function(e,t){e.exports=function(e,t){return null!=e&&t in Object(e)}},function(e,t,n){var r=n(204),i=n(448),o=n(126),a=n(66);e.exports=function(e){return o(e)?r(a(e)):i(e)}},function(e,t,n){var r=n(88);e.exports=function(e){return function(t){return r(t,e)}}},function(e,t){var n=Object.prototype.hasOwnProperty;e.exports=function(e,t){return null!=e&&n.call(e,t)}},function(e,t,n){var r=n(118),i=n(54),o=n(64),a=n(15),s=n(38),l=n(53),u=n(85),c=n(65),d=Object.prototype.hasOwnProperty;e.exports=function(e){if(null==e)return!0;if(s(e)&&(a(e)||"string"==typeof e||"function"==typeof e.splice||l(e)||c(e)||o(e)))return!e.length;var t=i(e);if("[object Map]"==t||"[object Set]"==t)return!e.size;if(u(e))return!r(e).length;for(var n in e)if(d.call(e,n))return!1;return!0}},function(e,t){e.exports=function(e,t,n,r){var i=-1,o=null==e?0:e.length;for(r&&o&&(n=e[++i]);++i<o;)n=t(n,e[i],i,e);return n}},function(e,t){e.exports=function(e,t,n,r,i){return i(e,(function(e,i,o){n=r?(r=!1,e):t(n,e,i,o)})),n}},function(e,t,n){var r=n(118),i=n(54),o=n(38),a=n(454),s=n(455);e.exports=function(e){if(null==e)return 0;if(o(e))return a(e)?s(e):e.length;var t=i(e);return"[object Map]"==t||"[object Set]"==t?e.size:r(e).length}},function(e,t,n){var r=n(45),i=n(15),o=n(34);e.exports=function(e){return"string"==typeof e||!i(e)&&o(e)&&"[object String]"==r(e)}},function(e,t,n){var r=n(456),i=n(457),o=n(458);e.exports=function(e){return i(e)?o(e):r(e)}},function(e,t,n){var r=n(204)("length");e.exports=r},function(e,t){var n=RegExp("[\\u200d\\ud800-\\udfff\\u0300-\\u036f\\ufe20-\\ufe2f\\u20d0-\\u20ff\\ufe0e\\ufe0f]");e.exports=function(e){return n.test(e)}},function(e,t){var n="[\\u0300-\\u036f\\ufe20-\\ufe2f\\u20d0-\\u20ff]",r="\\ud83c[\\udffb-\\udfff]",i="[^\\ud800-\\udfff]",o="(?:\\ud83c[\\udde6-\\uddff]){2}",a="[\\ud800-\\udbff][\\udc00-\\udfff]",s="(?:"+n+"|"+r+")?",l="[\\ufe0e\\ufe0f]?"+s+"(?:\\u200d(?:"+[i,o,a].join("|")+")[\\ufe0e\\ufe0f]?"+s+")*",u="(?:"+[i+n+"?",n,o,a,"[\\ud800-\\udfff]"].join("|")+")",c=RegExp(r+"(?="+r+")|"+u+l,"g");e.exports=function(e){for(var t=c.lastIndex=0;c.test(e);)++t;return t}},function(e,t,n){var r=n(114),i=n(190),o=n(123),a=n(39),s=n(86),l=n(15),u=n(53),c=n(62),d=n(25),h=n(65);e.exports=function(e,t,n){var f=l(e),p=f||u(e)||h(e);if(t=a(t,4),null==n){var g=e&&e.constructor;n=p?f?new g:[]:d(e)&&c(g)?i(s(e)):{}}return(p?r:o)(e,(function(e,r,i){return t(n,e,r,i)})),n}},function(e,t,n){var r=n(127),i=n(91),o=n(465),a=n(213),s=i((function(e){return o(r(e,1,a,!0))}));e.exports=s},function(e,t,n){var r=n(52),i=n(64),o=n(15),a=r?r.isConcatSpreadable:void 0;e.exports=function(e){return o(e)||i(e)||!!(a&&e&&e[a])}},function(e,t){e.exports=function(e,t,n){switch(n.length){case 0:return e.call(t);case 1:return e.call(t,n[0]);case 2:return e.call(t,n[0],n[1]);case 3:return e.call(t,n[0],n[1],n[2])}return e.apply(t,n)}},function(e,t,n){var r=n(122),i=n(176),o=n(47),a=i?function(e,t){return i(e,"toString",{configurable:!0,enumerable:!1,value:r(t),writable:!0})}:o;e.exports=a},function(e,t){var n=Date.now;e.exports=function(e){var t=0,r=0;return function(){var i=n(),o=16-(i-r);if(r=i,o>0){if(++t>=800)return arguments[0]}else t=0;return e.apply(void 0,arguments)}}},function(e,t,n){var r=n(197),i=n(466),o=n(470),a=n(198),s=n(471),l=n(125);e.exports=function(e,t,n){var u=-1,c=i,d=e.length,h=!0,f=[],p=f;if(n)h=!1,c=o;else if(d>=200){var g=t?null:s(e);if(g)return l(g);h=!1,c=a,p=new r}else p=t?[]:f;e:for(;++u<d;){var v=e[u],m=t?t(v):v;if(v=n||0!==v?v:0,h&&m==m){for(var y=p.length;y--;)if(p[y]===m)continue e;t&&p.push(m),f.push(v)}else c(p,m,n)||(p!==f&&p.push(m),f.push(v))}return f}},function(e,t,n){var r=n(467);e.exports=function(e,t){return!(null==e||!e.length)&&r(e,t,0)>-1}},function(e,t,n){var r=n(212),i=n(468),o=n(469);e.exports=function(e,t,n){return t==t?o(e,t,n):r(e,i,n)}},function(e,t){e.exports=function(e){return e!=e}},function(e,t){e.exports=function(e,t,n){for(var r=n-1,i=e.length;++r<i;)if(e[r]===t)return r;return-1}},function(e,t){e.exports=function(e,t,n){for(var r=-1,i=null==e?0:e.length;++r<i;)if(n(t,e[r]))return!0;return!1}},function(e,t,n){var r=n(186),i=n(472),o=n(125),a=r&&1/o(new r([,-0]))[1]==1/0?function(e){return new r(e)}:i;e.exports=a},function(e,t){e.exports=function(){}},function(e,t,n){var r=n(90);e.exports=function(e,t){return r(t,(function(t){return e[t]}))}},function(e,t){e.exports="2.1.8"},function(e,t,n){var r=n(24),i=n(111);function o(e){return r.map(e.nodes(),(function(t){var n=e.node(t),i=e.parent(t),o={v:t};return r.isUndefined(n)||(o.value=n),r.isUndefined(i)||(o.parent=i),o}))}function a(e){return r.map(e.edges(),(function(t){var n=e.edge(t),i={v:t.v,w:t.w};return r.isUndefined(t.name)||(i.name=t.name),r.isUndefined(n)||(i.value=n),i}))}e.exports={write:function(e){var t={options:{directed:e.isDirected(),multigraph:e.isMultigraph(),compound:e.isCompound()},nodes:o(e),edges:a(e)};return r.isUndefined(e.graph())||(t.value=r.clone(e.graph())),t},read:function(e){var t=new i(e.options).setGraph(e.value);return r.each(e.nodes,(function(e){t.setNode(e.v,e.value),e.parent&&t.setParent(e.v,e.parent)})),r.each(e.edges,(function(e){t.setEdge({v:e.v,w:e.w,name:e.name},e.value)})),t}}},function(e,t,n){e.exports={components:n(477),dijkstra:n(215),dijkstraAll:n(478),findCycles:n(479),floydWarshall:n(480),isAcyclic:n(481),postorder:n(482),preorder:n(483),prim:n(484),tarjan:n(217),topsort:n(218)}},function(e,t,n){var r=n(24);e.exports=function(e){var t,n={},i=[];function o(i){r.has(n,i)||(n[i]=!0,t.push(i),r.each(e.successors(i),o),r.each(e.predecessors(i),o))}return r.each(e.nodes(),(function(e){t=[],o(e),t.length&&i.push(t)})),i}},function(e,t,n){var r=n(215),i=n(24);e.exports=function(e,t,n){return i.transform(e.nodes(),(function(i,o){i[o]=r(e,o,t,n)}),{})}},function(e,t,n){var r=n(24),i=n(217);e.exports=function(e){return r.filter(i(e),(function(t){return t.length>1||1===t.length&&e.hasEdge(t[0],t[0])}))}},function(e,t,n){var r=n(24);e.exports=function(e,t,n){return function(e,t,n){var r={},i=e.nodes();return i.forEach((function(e){r[e]={},r[e][e]={distance:0},i.forEach((function(t){e!==t&&(r[e][t]={distance:Number.POSITIVE_INFINITY})})),n(e).forEach((function(n){var i=n.v===e?n.w:n.v,o=t(n);r[e][i]={distance:o,predecessor:e}}))})),i.forEach((function(e){var t=r[e];i.forEach((function(n){var o=r[n];i.forEach((function(n){var r=o[e],i=t[n],a=o[n],s=r.distance+i.distance;s<a.distance&&(a.distance=s,a.predecessor=i.predecessor)}))}))})),r}(e,t||i,n||function(t){return e.outEdges(t)})};var i=r.constant(1)},function(e,t,n){var r=n(218);e.exports=function(e){try{r(e)}catch(e){if(e instanceof r.CycleException)return!1;throw e}return!0}},function(e,t,n){var r=n(219);e.exports=function(e,t){return r(e,t,"post")}},function(e,t,n){var r=n(219);e.exports=function(e,t){return r(e,t,"pre")}},function(e,t,n){var r=n(24),i=n(111),o=n(216);e.exports=function(e,t){var n,a=new i,s={},l=new o;function u(e){var r=e.v===n?e.w:e.v,i=l.priority(r);if(void 0!==i){var o=t(e);o<i&&(s[r]=n,l.decrease(r,o))}}if(0===e.nodeCount())return a;r.each(e.nodes(),(function(e){l.add(e,Number.POSITIVE_INFINITY),a.setNode(e)})),l.decrease(e.nodes()[0],0);for(var c=!1;l.size()>0;){if(n=l.removeMin(),r.has(s,n))a.setEdge(n,s[n]);else{if(c)throw new Error("Input graph is not connected: "+e);c=!0}e.nodeEdges(n).forEach(u)}return a}},function(e,t,n){"use strict";var r=n(13),i=n(523),o=n(526),a=n(527),s=n(20).normalizeRanks,l=n(529),u=n(20).removeEmptyRanks,c=n(530),d=n(531),h=n(532),f=n(533),p=n(542),g=n(20),v=n(30).Graph;e.exports=function(e,t){var n=t&&t.debugTiming?g.time:g.notime;n("layout",(function(){var t=n("  buildLayoutGraph",(function(){return function(e){var t=new v({multigraph:!0,compound:!0}),n=M(e.graph());return t.setGraph(r.merge({},y,w(n,m),r.pick(n,b))),r.forEach(e.nodes(),(function(n){var i=M(e.node(n));t.setNode(n,r.defaults(w(i,C),A)),t.setParent(n,e.parent(n))})),r.forEach(e.edges(),(function(n){var i=M(e.edge(n));t.setEdge(n,r.merge({},I,w(i,_),r.pick(i,x)))})),t}(e)}));n("  runLayout",(function(){!function(e,t){t("    makeSpaceForEdgeLabels",(function(){!function(e){var t=e.graph();t.ranksep/=2,r.forEach(e.edges(),(function(n){var r=e.edge(n);r.minlen*=2,"c"!==r.labelpos.toLowerCase()&&("TB"===t.rankdir||"BT"===t.rankdir?r.width+=r.labeloffset:r.height+=r.labeloffset)}))}(e)})),t("    removeSelfEdges",(function(){!function(e){r.forEach(e.edges(),(function(t){if(t.v===t.w){var n=e.node(t.v);n.selfEdges||(n.selfEdges=[]),n.selfEdges.push({e:t,label:e.edge(t)}),e.removeEdge(t)}}))}(e)})),t("    acyclic",(function(){i.run(e)})),t("    nestingGraph.run",(function(){c.run(e)})),t("    rank",(function(){a(g.asNonCompoundGraph(e))})),t("    injectEdgeLabelProxies",(function(){!function(e){r.forEach(e.edges(),(function(t){var n=e.edge(t);if(n.width&&n.height){var r=e.node(t.v),i={rank:(e.node(t.w).rank-r.rank)/2+r.rank,e:t};g.addDummyNode(e,"edge-proxy",i,"_ep")}}))}(e)})),t("    removeEmptyRanks",(function(){u(e)})),t("    nestingGraph.cleanup",(function(){c.cleanup(e)})),t("    normalizeRanks",(function(){s(e)})),t("    assignRankMinMax",(function(){!function(e){var t=0;r.forEach(e.nodes(),(function(n){var i=e.node(n);i.borderTop&&(i.minRank=e.node(i.borderTop).rank,i.maxRank=e.node(i.borderBottom).rank,t=r.max(t,i.maxRank))})),e.graph().maxRank=t}(e)})),t("    removeEdgeLabelProxies",(function(){!function(e){r.forEach(e.nodes(),(function(t){var n=e.node(t);"edge-proxy"===n.dummy&&(e.edge(n.e).labelRank=n.rank,e.removeNode(t))}))}(e)})),t("    normalize.run",(function(){o.run(e)})),t("    parentDummyChains",(function(){l(e)})),t("    addBorderSegments",(function(){d(e)})),t("    order",(function(){f(e)})),t("    insertSelfEdges",(function(){!function(e){var t=g.buildLayerMatrix(e);r.forEach(t,(function(t){var n=0;r.forEach(t,(function(t,i){var o=e.node(t);o.order=i+n,r.forEach(o.selfEdges,(function(t){g.addDummyNode(e,"selfedge",{width:t.label.width,height:t.label.height,rank:o.rank,order:i+ ++n,e:t.e,label:t.label},"_se")})),delete o.selfEdges}))}))}(e)})),t("    adjustCoordinateSystem",(function(){h.adjust(e)})),t("    position",(function(){p(e)})),t("    positionSelfEdges",(function(){!function(e){r.forEach(e.nodes(),(function(t){var n=e.node(t);if("selfedge"===n.dummy){var r=e.node(n.e.v),i=r.x+r.width/2,o=r.y,a=n.x-i,s=r.height/2;e.setEdge(n.e,n.label),e.removeNode(t),n.label.points=[{x:i+2*a/3,y:o-s},{x:i+5*a/6,y:o-s},{x:i+a,y:o},{x:i+5*a/6,y:o+s},{x:i+2*a/3,y:o+s}],n.label.x=n.x,n.label.y=n.y}}))}(e)})),t("    removeBorderNodes",(function(){!function(e){r.forEach(e.nodes(),(function(t){if(e.children(t).length){var n=e.node(t),i=e.node(n.borderTop),o=e.node(n.borderBottom),a=e.node(r.last(n.borderLeft)),s=e.node(r.last(n.borderRight));n.width=Math.abs(s.x-a.x),n.height=Math.abs(o.y-i.y),n.x=a.x+n.width/2,n.y=i.y+n.height/2}})),r.forEach(e.nodes(),(function(t){"border"===e.node(t).dummy&&e.removeNode(t)}))}(e)})),t("    normalize.undo",(function(){o.undo(e)})),t("    fixupEdgeLabelCoords",(function(){!function(e){r.forEach(e.edges(),(function(t){var n=e.edge(t);if(r.has(n,"x"))switch("l"!==n.labelpos&&"r"!==n.labelpos||(n.width-=n.labeloffset),n.labelpos){case"l":n.x-=n.width/2+n.labeloffset;break;case"r":n.x+=n.width/2+n.labeloffset}}))}(e)})),t("    undoCoordinateSystem",(function(){h.undo(e)})),t("    translateGraph",(function(){!function(e){var t=Number.POSITIVE_INFINITY,n=0,i=Number.POSITIVE_INFINITY,o=0,a=e.graph(),s=a.marginx||0,l=a.marginy||0;function u(e){var r=e.x,a=e.y,s=e.width,l=e.height;t=Math.min(t,r-s/2),n=Math.max(n,r+s/2),i=Math.min(i,a-l/2),o=Math.max(o,a+l/2)}r.forEach(e.nodes(),(function(t){u(e.node(t))})),r.forEach(e.edges(),(function(t){var n=e.edge(t);r.has(n,"x")&&u(n)})),t-=s,i-=l,r.forEach(e.nodes(),(function(n){var r=e.node(n);r.x-=t,r.y-=i})),r.forEach(e.edges(),(function(n){var o=e.edge(n);r.forEach(o.points,(function(e){e.x-=t,e.y-=i})),r.has(o,"x")&&(o.x-=t),r.has(o,"y")&&(o.y-=i)})),a.width=n-t+s,a.height=o-i+l}(e)})),t("    assignNodeIntersects",(function(){!function(e){r.forEach(e.edges(),(function(t){var n,r,i=e.edge(t),o=e.node(t.v),a=e.node(t.w);i.points?(n=i.points[0],r=i.points[i.points.length-1]):(i.points=[],n=a,r=o),i.points.unshift(g.intersectRect(o,n)),i.points.push(g.intersectRect(a,r))}))}(e)})),t("    reversePoints",(function(){!function(e){r.forEach(e.edges(),(function(t){var n=e.edge(t);n.reversed&&n.points.reverse()}))}(e)})),t("    acyclic.undo",(function(){i.undo(e)}))}(t,n)})),n("  updateInputGraph",(function(){!function(e,t){r.forEach(e.nodes(),(function(n){var r=e.node(n),i=t.node(n);r&&(r.x=i.x,r.y=i.y,t.children(n).length&&(r.width=i.width,r.height=i.height))})),r.forEach(e.edges(),(function(n){var i=e.edge(n),o=t.edge(n);i.points=o.points,r.has(o,"x")&&(i.x=o.x,i.y=o.y)})),e.graph().width=t.graph().width,e.graph().height=t.graph().height}(e,t)}))}))};var m=["nodesep","edgesep","ranksep","marginx","marginy"],y={ranksep:50,edgesep:20,nodesep:50,rankdir:"tb"},b=["acyclicer","ranker","rankdir","align"],C=["width","height"],A={width:0,height:0},_=["minlen","weight","width","height","labeloffset"],I={minlen:1,weight:1,width:0,height:0,labeloffset:10,labelpos:"r"},x=["labelpos"];function w(e,t){return r.mapValues(r.pick(e,t),Number)}function M(e){var t={};return r.forEach(e,(function(e,n){t[n.toLowerCase()]=e})),t}},function(e,t,n){var r=n(173);e.exports=function(e){return r(e,5)}},function(e,t,n){var r=n(91),i=n(51),o=n(92),a=n(46),s=Object.prototype,l=s.hasOwnProperty,u=r((function(e,t){e=Object(e);var n=-1,r=t.length,u=r>2?t[2]:void 0;for(u&&o(t[0],t[1],u)&&(r=1);++n<r;)for(var c=t[n],d=a(c),h=-1,f=d.length;++h<f;){var p=d[h],g=e[p];(void 0===g||i(g,s[p])&&!l.call(e,p))&&(e[p]=c[p])}return e}));e.exports=u},function(e,t,n){var r=n(489)(n(490));e.exports=r},function(e,t,n){var r=n(39),i=n(38),o=n(41);e.exports=function(e){return function(t,n,a){var s=Object(t);if(!i(t)){var l=r(n,3);t=o(t),n=function(e){return l(s[e],e,s)}}var u=e(t,n,a);return u>-1?s[l?t[u]:u]:void 0}}},function(e,t,n){var r=n(212),i=n(39),o=n(491),a=Math.max;e.exports=function(e,t,n){var s=null==e?0:e.length;if(!s)return-1;var l=null==n?0:o(n);return l<0&&(l=a(s+l,0)),r(e,i(t,3),l)}},function(e,t,n){var r=n(220);e.exports=function(e){var t=r(e),n=t%1;return t==t?n?t-n:t:0}},function(e,t,n){var r=n(25),i=n(55),o=/^\s+|\s+$/g,a=/^[-+]0x[0-9a-f]+$/i,s=/^0b[01]+$/i,l=/^0o[0-7]+$/i,u=parseInt;e.exports=function(e){if("number"==typeof e)return e;if(i(e))return NaN;if(r(e)){var t="function"==typeof e.valueOf?e.valueOf():e;e=r(t)?t+"":t}if("string"!=typeof e)return 0===e?e:+e;e=e.replace(o,"");var n=s.test(e);return n||l.test(e)?u(e.slice(2),n?2:8):a.test(e)?NaN:+e}},function(e,t,n){var r=n(124),i=n(193),o=n(46);e.exports=function(e,t){return null==e?e:r(e,i(t),o)}},function(e,t){e.exports=function(e){var t=null==e?0:e.length;return t?e[t-1]:void 0}},function(e,t,n){var r=n(82),i=n(123),o=n(39);e.exports=function(e,t){var n={};return t=o(t,3),i(e,(function(e,i,o){r(n,i,t(e,i,o))})),n}},function(e,t,n){var r=n(128),i=n(497),o=n(47);e.exports=function(e){return e&&e.length?r(e,o,i):void 0}},function(e,t){e.exports=function(e,t){return e>t}},function(e,t,n){var r=n(499),i=n(503)((function(e,t,n){r(e,t,n)}));e.exports=i},function(e,t,n){var r=n(76),i=n(222),o=n(124),a=n(500),s=n(25),l=n(46),u=n(223);e.exports=function e(t,n,c,d,h){t!==n&&o(n,(function(o,l){if(h||(h=new r),s(o))a(t,n,l,c,e,d,h);else{var f=d?d(u(t,l),o,l+"",t,n,h):void 0;void 0===f&&(f=o),i(t,l,f)}}),l)}},function(e,t,n){var r=n(222),i=n(179),o=n(188),a=n(180),s=n(189),l=n(64),u=n(15),c=n(213),d=n(53),h=n(62),f=n(25),p=n(501),g=n(65),v=n(223),m=n(502);e.exports=function(e,t,n,y,b,C,A){var _=v(e,n),I=v(t,n),x=A.get(I);if(x)r(e,n,x);else{var w=C?C(_,I,n+"",e,t,A):void 0,M=void 0===w;if(M){var S=u(I),T=!S&&d(I),E=!S&&!T&&g(I);w=I,S||T||E?u(_)?w=_:c(_)?w=a(_):T?(M=!1,w=i(I,!0)):E?(M=!1,w=o(I,!0)):w=[]:p(I)||l(I)?(w=_,l(_)?w=m(_):f(_)&&!h(_)||(w=s(I))):M=!1}M&&(A.set(I,w),b(w,I,y,C,A),A.delete(I)),r(e,n,w)}}},function(e,t,n){var r=n(45),i=n(86),o=n(34),a=Function.prototype,s=Object.prototype,l=a.toString,u=s.hasOwnProperty,c=l.call(Object);e.exports=function(e){if(!o(e)||"[object Object]"!=r(e))return!1;var t=i(e);if(null===t)return!0;var n=u.call(t,"constructor")&&t.constructor;return"function"==typeof n&&n instanceof n&&l.call(n)==c}},function(e,t,n){var r=n(63),i=n(46);e.exports=function(e){return r(e,i(e))}},function(e,t,n){var r=n(91),i=n(92);e.exports=function(e){return r((function(t,n){var r=-1,o=n.length,a=o>1?n[o-1]:void 0,s=o>2?n[2]:void 0;for(a=e.length>3&&"function"==typeof a?(o--,a):void 0,s&&i(n[0],n[1],s)&&(a=o<3?void 0:a,o=1),t=Object(t);++r<o;){var l=n[r];l&&e(t,l,r,a)}return t}))}},function(e,t,n){var r=n(128),i=n(224),o=n(47);e.exports=function(e){return e&&e.length?r(e,o,i):void 0}},function(e,t,n){var r=n(128),i=n(39),o=n(224);e.exports=function(e,t){return e&&e.length?r(e,i(t,2),o):void 0}},function(e,t,n){var r=n(31);e.exports=function(){return r.Date.now()}},function(e,t,n){var r=n(508),i=n(511)((function(e,t){return null==e?{}:r(e,t)}));e.exports=i},function(e,t,n){var r=n(509),i=n(202);e.exports=function(e,t){return r(e,t,(function(t,n){return i(e,n)}))}},function(e,t,n){var r=n(88),i=n(510),o=n(89);e.exports=function(e,t,n){for(var a=-1,s=t.length,l={};++a<s;){var u=t[a],c=r(e,u);n(c,u)&&i(l,o(u,e),c)}return l}},function(e,t,n){var r=n(81),i=n(89),o=n(83),a=n(25),s=n(66);e.exports=function(e,t,n,l){if(!a(e))return e;for(var u=-1,c=(t=i(t,e)).length,d=c-1,h=e;null!=h&&++u<c;){var f=s(t[u]),p=n;if("__proto__"===f||"constructor"===f||"prototype"===f)return e;if(u!=d){var g=h[f];void 0===(p=l?l(g,f,h):void 0)&&(p=a(g)?g:o(t[u+1])?[]:{})}r(h,f,p),h=h[f]}return e}},function(e,t,n){var r=n(221),i=n(210),o=n(211);e.exports=function(e){return o(i(e,void 0,r),e+"")}},function(e,t,n){var r=n(513)();e.exports=r},function(e,t,n){var r=n(514),i=n(92),o=n(220);e.exports=function(e){return function(t,n,a){return a&&"number"!=typeof a&&i(t,n,a)&&(n=a=void 0),t=o(t),void 0===n?(n=t,t=0):n=o(n),a=void 0===a?t<n?1:-1:o(a),r(t,n,a,e)}}},function(e,t){var n=Math.ceil,r=Math.max;e.exports=function(e,t,i,o){for(var a=-1,s=r(n((t-e)/(i||1)),0),l=Array(s);s--;)l[o?s:++a]=e,e+=i;return l}},function(e,t,n){var r=n(127),i=n(516),o=n(91),a=n(92),s=o((function(e,t){if(null==e)return[];var n=t.length;return n>1&&a(e,t[0],t[1])?t=[]:n>2&&a(t[0],t[1],t[2])&&(t=[t[0]]),i(e,r(t,1),[])}));e.exports=s},function(e,t,n){var r=n(90),i=n(88),o=n(39),a=n(208),s=n(517),l=n(84),u=n(518),c=n(47),d=n(15);e.exports=function(e,t,n){t=t.length?r(t,(function(e){return d(e)?function(t){return i(t,1===e.length?e[0]:e)}:e})):[c];var h=-1;t=r(t,l(o));var f=a(e,(function(e,n,i){return{criteria:r(t,(function(t){return t(e)})),index:++h,value:e}}));return s(f,(function(e,t){return u(e,t,n)}))}},function(e,t){e.exports=function(e,t){var n=e.length;for(e.sort(t);n--;)e[n]=e[n].value;return e}},function(e,t,n){var r=n(519);e.exports=function(e,t,n){for(var i=-1,o=e.criteria,a=t.criteria,s=o.length,l=n.length;++i<s;){var u=r(o[i],a[i]);if(u)return i>=l?u:u*("desc"==n[i]?-1:1)}return e.index-t.index}},function(e,t,n){var r=n(55);e.exports=function(e,t){if(e!==t){var n=void 0!==e,i=null===e,o=e==e,a=r(e),s=void 0!==t,l=null===t,u=t==t,c=r(t);if(!l&&!c&&!a&&e>t||a&&s&&u&&!l&&!c||i&&s&&u||!n&&u||!o)return 1;if(!i&&!a&&!c&&e<t||c&&n&&o&&!i&&!a||l&&n&&o||!s&&o||!u)return-1}return 0}},function(e,t,n){var r=n(201),i=0;e.exports=function(e){var t=++i;return r(e)+t}},function(e,t,n){var r=n(81),i=n(522);e.exports=function(e,t){return i(e||[],t||[],r)}},function(e,t){e.exports=function(e,t,n){for(var r=-1,i=e.length,o=t.length,a={};++r<i;){var s=r<o?t[r]:void 0;n(a,e[r],s)}return a}},function(e,t,n){"use strict";var r=n(13),i=n(524);e.exports={run:function(e){var t="greedy"===e.graph().acyclicer?i(e,function(e){return function(t){return e.edge(t).weight}}(e)):function(e){var t=[],n={},i={};return r.forEach(e.nodes(),(function o(a){r.has(i,a)||(i[a]=!0,n[a]=!0,r.forEach(e.outEdges(a),(function(e){r.has(n,e.w)?t.push(e):o(e.w)})),delete n[a])})),t}(e);r.forEach(t,(function(t){var n=e.edge(t);e.removeEdge(t),n.forwardName=t.name,n.reversed=!0,e.setEdge(t.w,t.v,n,r.uniqueId("rev"))}))},undo:function(e){r.forEach(e.edges(),(function(t){var n=e.edge(t);if(n.reversed){e.removeEdge(t);var r=n.forwardName;delete n.reversed,delete n.forwardName,e.setEdge(t.w,t.v,n,r)}}))}}},function(e,t,n){var r=n(13),i=n(30).Graph,o=n(525);e.exports=function(e,t){if(e.nodeCount()<=1)return[];var n=function(e,t){var n=new i,a=0,s=0;r.forEach(e.nodes(),(function(e){n.setNode(e,{v:e,in:0,out:0})})),r.forEach(e.edges(),(function(e){var r=n.edge(e.v,e.w)||0,i=t(e),o=r+i;n.setEdge(e.v,e.w,o),s=Math.max(s,n.node(e.v).out+=i),a=Math.max(a,n.node(e.w).in+=i)}));var u=r.range(s+a+3).map((function(){return new o})),c=a+1;return r.forEach(n.nodes(),(function(e){l(u,c,n.node(e))})),{graph:n,buckets:u,zeroIdx:c}}(e,t||a),u=function(e,t,n){for(var r,i=[],o=t[t.length-1],a=t[0];e.nodeCount();){for(;r=a.dequeue();)s(e,t,n,r);for(;r=o.dequeue();)s(e,t,n,r);if(e.nodeCount())for(var l=t.length-2;l>0;--l)if(r=t[l].dequeue()){i=i.concat(s(e,t,n,r,!0));break}}return i}(n.graph,n.buckets,n.zeroIdx);return r.flatten(r.map(u,(function(t){return e.outEdges(t.v,t.w)})),!0)};var a=r.constant(1);function s(e,t,n,i,o){var a=o?[]:void 0;return r.forEach(e.inEdges(i.v),(function(r){var i=e.edge(r),s=e.node(r.v);o&&a.push({v:r.v,w:r.w}),s.out-=i,l(t,n,s)})),r.forEach(e.outEdges(i.v),(function(r){var i=e.edge(r),o=r.w,a=e.node(o);a.in-=i,l(t,n,a)})),e.removeNode(i.v),a}function l(e,t,n){n.out?n.in?e[n.out-n.in+t].enqueue(n):e[e.length-1].enqueue(n):e[0].enqueue(n)}},function(e,t){function n(){var e={};e._next=e._prev=e,this._sentinel=e}function r(e){e._prev._next=e._next,e._next._prev=e._prev,delete e._next,delete e._prev}function i(e,t){if("_next"!==e&&"_prev"!==e)return t}e.exports=n,n.prototype.dequeue=function(){var e=this._sentinel,t=e._prev;if(t!==e)return r(t),t},n.prototype.enqueue=function(e){var t=this._sentinel;e._prev&&e._next&&r(e),e._next=t._next,t._next._prev=e,t._next=e,e._prev=t},n.prototype.toString=function(){for(var e=[],t=this._sentinel,n=t._prev;n!==t;)e.push(JSON.stringify(n,i)),n=n._prev;return"["+e.join(", ")+"]"}},function(e,t,n){"use strict";var r=n(13),i=n(20);e.exports={run:function(e){e.graph().dummyChains=[],r.forEach(e.edges(),(function(t){!function(e,t){var n,r,o,a=t.v,s=e.node(a).rank,l=t.w,u=e.node(l).rank,c=t.name,d=e.edge(t),h=d.labelRank;if(u!==s+1){for(e.removeEdge(t),o=0,++s;s<u;++o,++s)d.points=[],r={width:0,height:0,edgeLabel:d,edgeObj:t,rank:s},n=i.addDummyNode(e,"edge",r,"_d"),s===h&&(r.width=d.width,r.height=d.height,r.dummy="edge-label",r.labelpos=d.labelpos),e.setEdge(a,n,{weight:d.weight},c),0===o&&e.graph().dummyChains.push(n),a=n;e.setEdge(a,l,{weight:d.weight},c)}}(e,t)}))},undo:function(e){r.forEach(e.graph().dummyChains,(function(t){var n,r=e.node(t),i=r.edgeLabel;for(e.setEdge(r.edgeObj,i);r.dummy;)n=e.successors(t)[0],e.removeNode(t),i.points.push({x:r.x,y:r.y}),"edge-label"===r.dummy&&(i.x=r.x,i.y=r.y,i.width=r.width,i.height=r.height),t=n,r=e.node(t)}))}}},function(e,t,n){"use strict";var r=n(93).longestPath,i=n(225),o=n(528);e.exports=function(e){switch(e.graph().ranker){case"network-simplex":s(e);break;case"tight-tree":!function(e){r(e),i(e)}(e);break;case"longest-path":a(e);break;default:s(e)}};var a=r;function s(e){o(e)}},function(e,t,n){"use strict";var r=n(13),i=n(225),o=n(93).slack,a=n(93).longestPath,s=n(30).alg.preorder,l=n(30).alg.postorder,u=n(20).simplify;function c(e){e=u(e),a(e);var t,n=i(e);for(f(n),d(n,e);t=g(n);)m(n,e,t,v(n,e,t))}function d(e,t){var n=l(e,e.nodes());n=n.slice(0,n.length-1),r.forEach(n,(function(n){!function(e,t,n){var r=e.node(n).parent;e.edge(n,r).cutvalue=h(e,t,n)}(e,t,n)}))}function h(e,t,n){var i=e.node(n).parent,o=!0,a=t.edge(n,i),s=0;return a||(o=!1,a=t.edge(i,n)),s=a.weight,r.forEach(t.nodeEdges(n),(function(r){var a,l,u=r.v===n,c=u?r.w:r.v;if(c!==i){var d=u===o,h=t.edge(r).weight;if(s+=d?h:-h,a=n,l=c,e.hasEdge(a,l)){var f=e.edge(n,c).cutvalue;s+=d?-f:f}}})),s}function f(e,t){arguments.length<2&&(t=e.nodes()[0]),p(e,{},1,t)}function p(e,t,n,i,o){var a=n,s=e.node(i);return t[i]=!0,r.forEach(e.neighbors(i),(function(o){r.has(t,o)||(n=p(e,t,n,o,i))})),s.low=a,s.lim=n++,o?s.parent=o:delete s.parent,n}function g(e){return r.find(e.edges(),(function(t){return e.edge(t).cutvalue<0}))}function v(e,t,n){var i=n.v,a=n.w;t.hasEdge(i,a)||(i=n.w,a=n.v);var s=e.node(i),l=e.node(a),u=s,c=!1;s.lim>l.lim&&(u=l,c=!0);var d=r.filter(t.edges(),(function(t){return c===y(0,e.node(t.v),u)&&c!==y(0,e.node(t.w),u)}));return r.minBy(d,(function(e){return o(t,e)}))}function m(e,t,n,i){var o=n.v,a=n.w;e.removeEdge(o,a),e.setEdge(i.v,i.w,{}),f(e),d(e,t),function(e,t){var n=r.find(e.nodes(),(function(e){return!t.node(e).parent})),i=s(e,n);i=i.slice(1),r.forEach(i,(function(n){var r=e.node(n).parent,i=t.edge(n,r),o=!1;i||(i=t.edge(r,n),o=!0),t.node(n).rank=t.node(r).rank+(o?i.minlen:-i.minlen)}))}(e,t)}function y(e,t,n){return n.low<=t.lim&&t.lim<=n.lim}e.exports=c,c.initLowLimValues=f,c.initCutValues=d,c.calcCutValue=h,c.leaveEdge=g,c.enterEdge=v,c.exchangeEdges=m},function(e,t,n){var r=n(13);e.exports=function(e){var t=function(e){var t={},n=0;return r.forEach(e.children(),(function i(o){var a=n;r.forEach(e.children(o),i),t[o]={low:a,lim:n++}})),t}(e);r.forEach(e.graph().dummyChains,(function(n){for(var r=e.node(n),i=r.edgeObj,o=function(e,t,n,r){var i,o,a=[],s=[],l=Math.min(t[n].low,t[r].low),u=Math.max(t[n].lim,t[r].lim);i=n;do{i=e.parent(i),a.push(i)}while(i&&(t[i].low>l||u>t[i].lim));for(o=i,i=r;(i=e.parent(i))!==o;)s.push(i);return{path:a.concat(s.reverse()),lca:o}}(e,t,i.v,i.w),a=o.path,s=o.lca,l=0,u=a[l],c=!0;n!==i.w;){if(r=e.node(n),c){for(;(u=a[l])!==s&&e.node(u).maxRank<r.rank;)l++;u===s&&(c=!1)}if(!c){for(;l<a.length-1&&e.node(u=a[l+1]).minRank<=r.rank;)l++;u=a[l]}e.setParent(n,u),n=e.successors(n)[0]}}))}},function(e,t,n){var r=n(13),i=n(20);e.exports={run:function(e){var t=i.addDummyNode(e,"root",{},"_root"),n=function(e){var t={};return r.forEach(e.children(),(function(n){!function n(i,o){var a=e.children(i);a&&a.length&&r.forEach(a,(function(e){n(e,o+1)})),t[i]=o}(n,1)})),t}(e),o=r.max(r.values(n))-1,a=2*o+1;e.graph().nestingRoot=t,r.forEach(e.edges(),(function(t){e.edge(t).minlen*=a}));var s=function(e){return r.reduce(e.edges(),(function(t,n){return t+e.edge(n).weight}),0)}(e)+1;r.forEach(e.children(),(function(l){!function e(t,n,o,a,s,l,u){var c=t.children(u);if(c.length){var d=i.addBorderNode(t,"_bt"),h=i.addBorderNode(t,"_bb"),f=t.node(u);t.setParent(d,u),f.borderTop=d,t.setParent(h,u),f.borderBottom=h,r.forEach(c,(function(r){e(t,n,o,a,s,l,r);var i=t.node(r),c=i.borderTop?i.borderTop:r,f=i.borderBottom?i.borderBottom:r,p=i.borderTop?a:2*a,g=c!==f?1:s-l[u]+1;t.setEdge(d,c,{weight:p,minlen:g,nestingEdge:!0}),t.setEdge(f,h,{weight:p,minlen:g,nestingEdge:!0})})),t.parent(u)||t.setEdge(n,d,{weight:0,minlen:s+l[u]})}else u!==n&&t.setEdge(n,u,{weight:0,minlen:o})}(e,t,a,s,o,n,l)})),e.graph().nodeRankFactor=a},cleanup:function(e){var t=e.graph();e.removeNode(t.nestingRoot),delete t.nestingRoot,r.forEach(e.edges(),(function(t){e.edge(t).nestingEdge&&e.removeEdge(t)}))}}},function(e,t,n){var r=n(13),i=n(20);function o(e,t,n,r,o,a){var s={width:0,height:0,rank:a,borderType:t},l=o[t][a-1],u=i.addDummyNode(e,"border",s,n);o[t][a]=u,e.setParent(u,r),l&&e.setEdge(l,u,{weight:1})}e.exports=function(e){r.forEach(e.children(),(function t(n){var i=e.children(n),a=e.node(n);if(i.length&&r.forEach(i,t),r.has(a,"minRank")){a.borderLeft=[],a.borderRight=[];for(var s=a.minRank,l=a.maxRank+1;s<l;++s)o(e,"borderLeft","_bl",n,a,s),o(e,"borderRight","_br",n,a,s)}}))}},function(e,t,n){"use strict";var r=n(13);function i(e){r.forEach(e.nodes(),(function(t){o(e.node(t))})),r.forEach(e.edges(),(function(t){o(e.edge(t))}))}function o(e){var t=e.width;e.width=e.height,e.height=t}function a(e){e.y=-e.y}function s(e){var t=e.x;e.x=e.y,e.y=t}e.exports={adjust:function(e){var t=e.graph().rankdir.toLowerCase();"lr"!==t&&"rl"!==t||i(e)},undo:function(e){var t=e.graph().rankdir.toLowerCase();"bt"!==t&&"rl"!==t||function(e){r.forEach(e.nodes(),(function(t){a(e.node(t))})),r.forEach(e.edges(),(function(t){var n=e.edge(t);r.forEach(n.points,a),r.has(n,"y")&&a(n)}))}(e),"lr"!==t&&"rl"!==t||(function(e){r.forEach(e.nodes(),(function(t){s(e.node(t))})),r.forEach(e.edges(),(function(t){var n=e.edge(t);r.forEach(n.points,s),r.has(n,"x")&&s(n)}))}(e),i(e))}}},function(e,t,n){"use strict";var r=n(13),i=n(534),o=n(535),a=n(536),s=n(540),l=n(541),u=n(30).Graph,c=n(20);function d(e,t,n){return r.map(t,(function(t){return s(e,t,n)}))}function h(e,t){var n=new u;r.forEach(e,(function(e){var i=e.graph().root,o=a(e,i,n,t);r.forEach(o.vs,(function(t,n){e.node(t).order=n})),l(e,n,o.vs)}))}function f(e,t){r.forEach(t,(function(t){r.forEach(t,(function(t,n){e.node(t).order=n}))}))}e.exports=function(e){var t=c.maxRank(e),n=d(e,r.range(1,t+1),"inEdges"),a=d(e,r.range(t-1,-1,-1),"outEdges"),s=i(e);f(e,s);for(var l,u=Number.POSITIVE_INFINITY,p=0,g=0;g<4;++p,++g){h(p%2?n:a,p%4>=2),s=c.buildLayerMatrix(e);var v=o(e,s);v<u&&(g=0,l=r.cloneDeep(s),u=v)}f(e,l)}},function(e,t,n){"use strict";var r=n(13);e.exports=function(e){var t={},n=r.filter(e.nodes(),(function(t){return!e.children(t).length})),i=r.max(r.map(n,(function(t){return e.node(t).rank}))),o=r.map(r.range(i+1),(function(){return[]})),a=r.sortBy(n,(function(t){return e.node(t).rank}));return r.forEach(a,(function n(i){if(!r.has(t,i)){t[i]=!0;var a=e.node(i);o[a.rank].push(i),r.forEach(e.successors(i),n)}})),o}},function(e,t,n){"use strict";var r=n(13);function i(e,t,n){for(var i=r.zipObject(n,r.map(n,(function(e,t){return t}))),o=r.flatten(r.map(t,(function(t){return r.sortBy(r.map(e.outEdges(t),(function(t){return{pos:i[t.w],weight:e.edge(t).weight}})),"pos")})),!0),a=1;a<n.length;)a<<=1;var s=2*a-1;a-=1;var l=r.map(new Array(s),(function(){return 0})),u=0;return r.forEach(o.forEach((function(e){var t=e.pos+a;l[t]+=e.weight;for(var n=0;t>0;)t%2&&(n+=l[t+1]),l[t=t-1>>1]+=e.weight;u+=e.weight*n}))),u}e.exports=function(e,t){for(var n=0,r=1;r<t.length;++r)n+=i(e,t[r-1],t[r]);return n}},function(e,t,n){var r=n(13),i=n(537),o=n(538),a=n(539);e.exports=function e(t,n,s,l){var u=t.children(n),c=t.node(n),d=c?c.borderLeft:void 0,h=c?c.borderRight:void 0,f={};d&&(u=r.filter(u,(function(e){return e!==d&&e!==h})));var p=i(t,u);r.forEach(p,(function(n){if(t.children(n.v).length){var i=e(t,n.v,s,l);f[n.v]=i,r.has(i,"barycenter")&&(o=n,a=i,r.isUndefined(o.barycenter)?(o.barycenter=a.barycenter,o.weight=a.weight):(o.barycenter=(o.barycenter*o.weight+a.barycenter*a.weight)/(o.weight+a.weight),o.weight+=a.weight))}var o,a}));var g=o(p,s);!function(e,t){r.forEach(e,(function(e){e.vs=r.flatten(e.vs.map((function(e){return t[e]?t[e].vs:e})),!0)}))}(g,f);var v=a(g,l);if(d&&(v.vs=r.flatten([d,v.vs,h],!0),t.predecessors(d).length)){var m=t.node(t.predecessors(d)[0]),y=t.node(t.predecessors(h)[0]);r.has(v,"barycenter")||(v.barycenter=0,v.weight=0),v.barycenter=(v.barycenter*v.weight+m.order+y.order)/(v.weight+2),v.weight+=2}return v}},function(e,t,n){var r=n(13);e.exports=function(e,t){return r.map(t,(function(t){var n=e.inEdges(t);if(n.length){var i=r.reduce(n,(function(t,n){var r=e.edge(n),i=e.node(n.v);return{sum:t.sum+r.weight*i.order,weight:t.weight+r.weight}}),{sum:0,weight:0});return{v:t,barycenter:i.sum/i.weight,weight:i.weight}}return{v:t}}))}},function(e,t,n){"use strict";var r=n(13);e.exports=function(e,t){var n={};return r.forEach(e,(function(e,t){var i=n[e.v]={indegree:0,in:[],out:[],vs:[e.v],i:t};r.isUndefined(e.barycenter)||(i.barycenter=e.barycenter,i.weight=e.weight)})),r.forEach(t.edges(),(function(e){var t=n[e.v],i=n[e.w];r.isUndefined(t)||r.isUndefined(i)||(i.indegree++,t.out.push(n[e.w]))})),function(e){var t=[];function n(e){return function(t){t.merged||(r.isUndefined(t.barycenter)||r.isUndefined(e.barycenter)||t.barycenter>=e.barycenter)&&function(e,t){var n=0,r=0;e.weight&&(n+=e.barycenter*e.weight,r+=e.weight),t.weight&&(n+=t.barycenter*t.weight,r+=t.weight),e.vs=t.vs.concat(e.vs),e.barycenter=n/r,e.weight=r,e.i=Math.min(t.i,e.i),t.merged=!0}(e,t)}}function i(t){return function(n){n.in.push(t),0==--n.indegree&&e.push(n)}}for(;e.length;){var o=e.pop();t.push(o),r.forEach(o.in.reverse(),n(o)),r.forEach(o.out,i(o))}return r.map(r.filter(t,(function(e){return!e.merged})),(function(e){return r.pick(e,["vs","i","barycenter","weight"])}))}(r.filter(n,(function(e){return!e.indegree})))}},function(e,t,n){var r=n(13),i=n(20);function o(e,t,n){for(var i;t.length&&(i=r.last(t)).i<=n;)t.pop(),e.push(i.vs),n++;return n}e.exports=function(e,t){var n,a=i.partition(e,(function(e){return r.has(e,"barycenter")})),s=a.lhs,l=r.sortBy(a.rhs,(function(e){return-e.i})),u=[],c=0,d=0,h=0;s.sort((n=!!t,function(e,t){return e.barycenter<t.barycenter?-1:e.barycenter>t.barycenter?1:n?t.i-e.i:e.i-t.i})),h=o(u,l,h),r.forEach(s,(function(e){h+=e.vs.length,u.push(e.vs),c+=e.barycenter*e.weight,d+=e.weight,h=o(u,l,h)}));var f={vs:r.flatten(u,!0)};return d&&(f.barycenter=c/d,f.weight=d),f}},function(e,t,n){var r=n(13),i=n(30).Graph;e.exports=function(e,t,n){var o=function(e){for(var t;e.hasNode(t=r.uniqueId("_root")););return t}(e),a=new i({compound:!0}).setGraph({root:o}).setDefaultNodeLabel((function(t){return e.node(t)}));return r.forEach(e.nodes(),(function(i){var s=e.node(i),l=e.parent(i);(s.rank===t||s.minRank<=t&&t<=s.maxRank)&&(a.setNode(i),a.setParent(i,l||o),r.forEach(e[n](i),(function(t){var n=t.v===i?t.w:t.v,o=a.edge(n,i),s=r.isUndefined(o)?0:o.weight;a.setEdge(n,i,{weight:e.edge(t).weight+s})})),r.has(s,"minRank")&&a.setNode(i,{borderLeft:s.borderLeft[t],borderRight:s.borderRight[t]}))})),a}},function(e,t,n){var r=n(13);e.exports=function(e,t,n){var i,o={};r.forEach(n,(function(n){for(var r,a,s=e.parent(n);s;){if((r=e.parent(s))?(a=o[r],o[r]=s):(a=i,i=s),a&&a!==s)return void t.setEdge(a,s);s=r}}))}},function(e,t,n){"use strict";var r=n(13),i=n(20),o=n(543).positionX;e.exports=function(e){(function(e){var t=i.buildLayerMatrix(e),n=e.graph().ranksep,o=0;r.forEach(t,(function(t){var i=r.max(r.map(t,(function(t){return e.node(t).height})));r.forEach(t,(function(t){e.node(t).y=o+i/2})),o+=i+n}))})(e=i.asNonCompoundGraph(e)),r.forEach(o(e),(function(t,n){e.node(n).x=t}))}},function(e,t,n){"use strict";var r=n(13),i=n(30).Graph,o=n(20);function a(e,t){var n={};return r.reduce(t,(function(t,i){var o=0,a=0,s=t.length,u=r.last(i);return r.forEach(i,(function(t,c){var d=function(e,t){if(e.node(t).dummy)return r.find(e.predecessors(t),(function(t){return e.node(t).dummy}))}(e,t),h=d?e.node(d).order:s;(d||t===u)&&(r.forEach(i.slice(a,c+1),(function(t){r.forEach(e.predecessors(t),(function(r){var i=e.node(r),a=i.order;!(a<o||h<a)||i.dummy&&e.node(t).dummy||l(n,r,t)}))})),a=c+1,o=h)})),i})),n}function s(e,t){var n={};function i(t,i,o,a,s){var u;r.forEach(r.range(i,o),(function(i){u=t[i],e.node(u).dummy&&r.forEach(e.predecessors(u),(function(t){var r=e.node(t);r.dummy&&(r.order<a||r.order>s)&&l(n,t,u)}))}))}return r.reduce(t,(function(t,n){var o,a=-1,s=0;return r.forEach(n,(function(r,l){if("border"===e.node(r).dummy){var u=e.predecessors(r);u.length&&(o=e.node(u[0]).order,i(n,s,l,a,o),s=l,a=o)}i(n,s,n.length,o,t.length)})),n})),n}function l(e,t,n){if(t>n){var r=t;t=n,n=r}var i=e[t];i||(e[t]=i={}),i[n]=!0}function u(e,t,n){if(t>n){var i=t;t=n,n=i}return r.has(e[t],n)}function c(e,t,n,i){var o={},a={},s={};return r.forEach(t,(function(e){r.forEach(e,(function(e,t){o[e]=e,a[e]=e,s[e]=t}))})),r.forEach(t,(function(e){var t=-1;r.forEach(e,(function(e){var l=i(e);if(l.length)for(var c=((l=r.sortBy(l,(function(e){return s[e]}))).length-1)/2,d=Math.floor(c),h=Math.ceil(c);d<=h;++d){var f=l[d];a[e]===e&&t<s[f]&&!u(n,e,f)&&(a[f]=e,a[e]=o[e]=o[f],t=s[f])}}))})),{root:o,align:a}}function d(e,t,n,o,a){var s={},l=function(e,t,n,o){var a=new i,s=e.graph(),l=function(e,t,n){return function(i,o,a){var s,l=i.node(o),u=i.node(a),c=0;if(c+=l.width/2,r.has(l,"labelpos"))switch(l.labelpos.toLowerCase()){case"l":s=-l.width/2;break;case"r":s=l.width/2}if(s&&(c+=n?s:-s),s=0,c+=(l.dummy?t:e)/2,c+=(u.dummy?t:e)/2,c+=u.width/2,r.has(u,"labelpos"))switch(u.labelpos.toLowerCase()){case"l":s=u.width/2;break;case"r":s=-u.width/2}return s&&(c+=n?s:-s),s=0,c}}(s.nodesep,s.edgesep,o);return r.forEach(t,(function(t){var i;r.forEach(t,(function(t){var r=n[t];if(a.setNode(r),i){var o=n[i],s=a.edge(o,r);a.setEdge(o,r,Math.max(l(e,t,i),s||0))}i=t}))})),a}(e,t,n,a),u=a?"borderLeft":"borderRight";function c(e,t){for(var n=l.nodes(),r=n.pop(),i={};r;)i[r]?e(r):(i[r]=!0,n.push(r),n=n.concat(t(r))),r=n.pop()}return c((function(e){s[e]=l.inEdges(e).reduce((function(e,t){return Math.max(e,s[t.v]+l.edge(t))}),0)}),l.predecessors.bind(l)),c((function(t){var n=l.outEdges(t).reduce((function(e,t){return Math.min(e,s[t.w]-l.edge(t))}),Number.POSITIVE_INFINITY),r=e.node(t);n!==Number.POSITIVE_INFINITY&&r.borderType!==u&&(s[t]=Math.max(s[t],n))}),l.successors.bind(l)),r.forEach(o,(function(e){s[e]=s[n[e]]})),s}function h(e,t){return r.minBy(r.values(t),(function(t){var n=Number.NEGATIVE_INFINITY,i=Number.POSITIVE_INFINITY;return r.forIn(t,(function(t,r){var o=function(e,t){return e.node(t).width}(e,r)/2;n=Math.max(t+o,n),i=Math.min(t-o,i)})),n-i}))}function f(e,t){var n=r.values(t),i=r.min(n),o=r.max(n);r.forEach(["u","d"],(function(n){r.forEach(["l","r"],(function(a){var s,l=n+a,u=e[l];if(u!==t){var c=r.values(u);(s="l"===a?i-r.min(c):o-r.max(c))&&(e[l]=r.mapValues(u,(function(e){return e+s})))}}))}))}function p(e,t){return r.mapValues(e.ul,(function(n,i){if(t)return e[t.toLowerCase()][i];var o=r.sortBy(r.map(e,i));return(o[1]+o[2])/2}))}e.exports={positionX:function(e){var t,n=o.buildLayerMatrix(e),i=r.merge(a(e,n),s(e,n)),l={};r.forEach(["u","d"],(function(o){t="u"===o?n:r.values(n).reverse(),r.forEach(["l","r"],(function(n){"r"===n&&(t=r.map(t,(function(e){return r.values(e).reverse()})));var a=("u"===o?e.predecessors:e.successors).bind(e),s=c(0,t,i,a),u=d(e,t,s.root,s.align,"r"===n);"r"===n&&(u=r.mapValues(u,(function(e){return-e}))),l[o+n]=u}))}));var u=h(e,l);return f(l,u),p(l,e.graph().align)},findType1Conflicts:a,findType2Conflicts:s,addConflict:l,hasConflict:u,verticalAlignment:c,horizontalCompaction:d,alignCoordinates:f,findSmallestWidthAlignment:h,balance:p}},function(e,t,n){var r=n(13),i=n(20),o=n(30).Graph;e.exports={debugOrdering:function(e){var t=i.buildLayerMatrix(e),n=new o({compound:!0,multigraph:!0}).setGraph({});return r.forEach(e.nodes(),(function(t){n.setNode(t,{label:t}),n.setParent(t,"layer"+e.node(t).rank)})),r.forEach(e.edges(),(function(e){n.setEdge(e.v,e.w,{},e.name)})),r.forEach(t,(function(e,t){var i="layer"+t;n.setNode(i,{rank:"same"}),r.reduce(e,(function(e,t){return n.setEdge(e,t,{style:"invis"}),t}))})),n}}},function(e,t){e.exports="0.8.5"},function(e,t,n){var r=n(94),i=n(547),o=n(129),a=n(48),s=function(e){var t,n;function r(){return e.apply(this,arguments)||this}return n=e,(t=r).prototype=Object.create(n.prototype),t.prototype.constructor=t,t.__proto__=n,r.prototype.execute=function(){return o(this.rootNode,this.options,i)},r}(r),l={};e.exports=function(e,t){return t=a.assign({},l,t),new s(e,t).execute()}},function(e,t){function n(e,t,n,r){void 0===r&&(r=[]);var i=this;i.w=e||0,i.h=t||0,i.y=n||0,i.x=0,i.c=r||[],i.cs=r.length,i.prelim=0,i.mod=0,i.shift=0,i.change=0,i.tl=null,i.tr=null,i.el=null,i.er=null,i.msel=0,i.mser=0}n.fromNode=function(e,t){if(!e)return null;var r=[];return e.children.forEach((function(e){r.push(n.fromNode(e,t))})),t?new n(e.height,e.width,e.x,r):new n(e.width,e.height,e.y,r)},e.exports=function(e,t){void 0===t&&(t={});var r=t.isHorizontal;function i(e){0===e.cs?(e.el=e,e.er=e,e.msel=e.mser=0):(e.el=e.c[0].el,e.msel=e.c[0].msel,e.er=e.c[e.cs-1].er,e.mser=e.c[e.cs-1].mser)}function o(e,t,n){for(var r=e.c[t-1],i=r.mod,o=e.c[t],c=o.mod;null!==r&&null!==o;){u(r)>n.low&&(n=n.nxt);var d=i+r.prelim+r.w-(c+o.prelim);d>0&&(c+=d,a(e,t,n.index,d));var h=u(r),f=u(o);h<=f&&null!==(r=l(r))&&(i+=r.mod),h>=f&&null!==(o=s(o))&&(c+=o.mod)}!r&&o?function(e,t,n,r){var i=e.c[0].el;i.tl=n;var o=r-n.mod-e.c[0].msel;i.mod+=o,i.prelim-=o,e.c[0].el=e.c[t].el,e.c[0].msel=e.c[t].msel}(e,t,o,c):r&&!o&&function(e,t,n,r){var i=e.c[t].er;i.tr=n;var o=r-n.mod-e.c[t].mser;i.mod+=o,i.prelim-=o,e.c[t].er=e.c[t-1].er,e.c[t].mser=e.c[t-1].mser}(e,t,r,i)}function a(e,t,n,r){e.c[t].mod+=r,e.c[t].msel+=r,e.c[t].mser+=r,function(e,t,n,r){if(n!==t-1){var i=t-n;e.c[n+1].shift+=r/i,e.c[t].shift-=r/i,e.c[t].change-=r-r/i}}(e,t,n,r)}function s(e){return 0===e.cs?e.tl:e.c[0]}function l(e){return 0===e.cs?e.tr:e.c[e.cs-1]}function u(e){return e.y+e.h}function c(e,t,n){for(;null!==n&&e>=n.low;)n=n.nxt;return{low:e,index:t,nxt:n}}!function e(t,n,r){void 0===r&&(r=0),n?(t.x=r,r+=t.width):(t.y=r,r+=t.height),t.children.forEach((function(t){e(t,n,r)}))}(e,r);var d=n.fromNode(e,r);return function e(t){if(0!==t.cs){e(t.c[0]);for(var n=c(u(t.c[0].el),0,null),r=1;r<t.cs;++r){e(t.c[r]);var a=u(t.c[r].er);o(t,r,n),n=c(a,r,n)}!function(e){e.prelim=(e.c[0].prelim+e.c[0].mod+e.c[e.cs-1].mod+e.c[e.cs-1].prelim+e.c[e.cs-1].w)/2-e.w/2}(t),i(t)}else i(t)}(d),function e(t,n){n+=t.mod,t.x=t.prelim+n,function(e){for(var t=0,n=0,r=0;r<e.cs;r++)n+=(t+=e.c[r].shift)+e.c[r].change,e.c[r].mod+=n}(t);for(var r=0;r<t.cs;r++)e(t.c[r],n)}(d,0),function e(t,n,r){r?n.y=t.x:n.x=t.x,t.c.forEach((function(t,i){e(t,n.children[i],r)}))}(d,e,r),function(e,t){var n=function e(t,n){var r=n?t.y:t.x;return t.children.forEach((function(t){r=Math.min(e(t,n),r)})),r}(e,t);!function e(t,n,r){r?t.y+=n:t.x+=n,t.children.forEach((function(t){e(t,n,r)}))}(e,-n,t)}(e,r),e}},function(e,t,n){var r=n(94),i=n(549),o=n(129),a=n(48),s=function(e){var t,n;function r(){return e.apply(this,arguments)||this}return n=e,(t=r).prototype=Object.create(n.prototype),t.prototype.constructor=t,t.__proto__=n,r.prototype.execute=function(){return this.rootNode.width=0,o(this.rootNode,this.options,i)},r}(r),l={};e.exports=function(e,t){return t=a.assign({},l,t),new s(e,t).execute()}},function(e,t,n){var r=n(48);function i(e,t){void 0===e&&(e=0),void 0===t&&(t=[]);var n=this;n.x=n.y=0,n.leftChild=n.rightChild=null,n.height=0,n.children=t}var o={isHorizontal:!0,nodeSep:20,nodeSize:20,rankSep:200,subTreeSep:10};e.exports=function(e,t){void 0===t&&(t={}),t=r.assign({},o,t);var n,a=0,s=function e(t){if(!t)return null;t.width=0,t.depth&&t.depth>a&&(a=t.depth);var n=t.children,r=n.length,o=new i(t.height,[]);return n.forEach((function(t,n){var i=e(t);o.children.push(i),0===n&&(o.leftChild=i),n===r-1&&(o.rightChild=i)})),o.originNode=t,o.isLeaf=t.isLeaf(),o}(e);return function e(t){if(t.isLeaf||0===t.children.length)t.drawingDepth=a;else{var n=t.children.map((function(t){return e(t)})),r=Math.min.apply(null,n);t.drawingDepth=r-1}return t.drawingDepth}(s),function e(r){r.x=r.drawingDepth*t.rankSep,r.isLeaf?(r.y=0,n&&(r.y=n.y+n.height+t.nodeSep,r.originNode.parent!==n.originNode.parent&&(r.y+=t.subTreeSep)),n=r):(r.children.forEach((function(t){e(t)})),r.y=(r.leftChild.y+r.rightChild.y)/2)}(s),function e(t,n,r){r?(n.x=t.x,n.y=t.y):(n.x=t.y,n.y=t.x),t.children.forEach((function(t,i){e(t,n.children[i],r)}))}(s,e,t.isHorizontal),e}},function(e,t,n){var r=n(94),i=n(551),o=n(227),a=n(48),s=["LR","RL","H"],l=s[0],u=function(e){var t,n;function r(){return e.apply(this,arguments)||this}return n=e,(t=r).prototype=Object.create(n.prototype),t.prototype.constructor=t,t.__proto__=n,r.prototype.execute=function(){var e=this.options,t=this.rootNode;e.isHorizontal=!0;var n=e.indent,r=void 0===n?20:n,a=e.dropCap,u=void 0===a||a,c=e.direction||l;if(c&&-1===s.indexOf(c))throw new TypeError("Invalid direction: "+c);if(c===s[0])i(t,r,u);else if(c===s[1])i(t,r,u),t.right2left();else if(c===s[2]){var d=o(t,e),h=d.left,f=d.right;i(h,r,u),h.right2left(),i(f,r,u);var p=h.getBoundingBox();f.translate(p.width,0),t.x=f.x-t.width/2}return t},r}(r),c={};e.exports=function(e,t){return t=a.assign({},c,t),new u(e,t).execute()}},function(e,t){e.exports=function(e,t,n){var r=null;e.eachNode((function(e){!function(e,t,n,r){if(!r)try{if(e.id===e.parent.children[0].id)return e.x+=n*e.depth,void(e.y=t?t.y:0)}catch(e){}e.x+=n*e.depth,e.y=t?t.y+t.height:0}(e,r,t,n),r=e}))}},function(e,t,n){var r=n(94),i=n(553),o=n(129),a=n(48),s=function(e){var t,n;function r(){return e.apply(this,arguments)||this}return n=e,(t=r).prototype=Object.create(n.prototype),t.prototype.constructor=t,t.__proto__=n,r.prototype.execute=function(){return o(this.rootNode,this.options,i)},r}(r),l={};e.exports=function(e,t){return t=a.assign({},l,t),new s(e,t).execute()}},function(e,t,n){var r=n(48),i={getSubTreeSep:function(){return 0}};e.exports=function(e,t){void 0===t&&(t={}),t=r.assign({},i,t),e.parent={x:0,width:0,height:0,y:0},e.BFTraverse((function(e){e.x=e.parent.x+e.parent.width})),e.parent=null,function e(t,n){var r=0;return t.children.length?t.children.forEach((function(t){r+=e(t,n)})):r=t.height,t._subTreeSep=n.getSubTreeSep(t.data),t.totalHeight=Math.max(t.height,r)+2*t._subTreeSep,t.totalHeight}(e,t),e.startY=0,e.y=e.totalHeight/2-e.height/2,e.eachNode((function(e){var t=e.children,n=t.length;if(n){var r=t[0];if(r.startY=e.startY+e._subTreeSep,1===n)r.y=e.y+e.height/2-r.height/2;else{r.y=r.startY+r.totalHeight/2-r.height/2;for(var i=1;i<n;i++){var o=t[i];o.startY=t[i-1].startY+t[i-1].totalHeight,o.y=o.startY+o.totalHeight/2-o.height/2}}}})),function e(t){var n=t.children,r=n.length;if(r){n.forEach((function(t){e(t)}));var i=n[0],o=n[r-1],a=o.y-i.y+o.height,s=0;if(n.forEach((function(e){s+=e.totalHeight})),a>t.height)t.y=i.y+a/2-t.height/2;else if(1!==n.length||t.height>s){var l=t.y+(t.height-a)/2-i.y;n.forEach((function(e){e.translate(0,l)}))}else t.y=(i.y+i.height/2+o.y+o.height/2)/2-t.height/2}}(e)}},function(e,t){function n(e){return(n="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}e.exports=function(e){return e&&"object"===n(e)&&"function"==typeof e.copy&&"function"==typeof e.fill&&"function"==typeof e.readUInt8}},function(e,t){"function"==typeof Object.create?e.exports=function(e,t){e.super_=t,e.prototype=Object.create(t.prototype,{constructor:{value:e,enumerable:!1,writable:!0,configurable:!0}})}:e.exports=function(e,t){e.super_=t;var n=function(){};n.prototype=t.prototype,e.prototype=new n,e.prototype.constructor=e}},function(e,t,n){"use strict";n.r(t),n.d(t,"registerNode",(function(){return Zl})),n.d(t,"registerCombo",(function(){return Kl})),n.d(t,"Graph",(function(){return Po})),n.d(t,"TreeGraph",(function(){return Ro})),n.d(t,"Util",(function(){return ba})),n.d(t,"registerEdge",(function(){return Ql})),n.d(t,"Layout",(function(){return ro})),n.d(t,"Global",(function(){return J})),n.d(t,"registerLayout",(function(){return $l})),n.d(t,"Minimap",(function(){return eu})),n.d(t,"Grid",(function(){return tu})),n.d(t,"Bundling",(function(){return nu})),n.d(t,"Menu",(function(){return ru})),n.d(t,"Fisheye",(function(){return iu})),n.d(t,"registerBehavior",(function(){return Jl})),n.d(t,"Algorithm",(function(){return u})),n.d(t,"ToolBar",(function(){return ou})),n.d(t,"Tooltip",(function(){return au})),n.d(t,"TimeBar",(function(){return su})),n.d(t,"ImageMinimap",(function(){return lu})),n.d(t,"EdgeFilterLens",(function(){return uu}));var r={};n.r(r),n.d(r,"formatPadding",(function(){return T})),n.d(r,"cloneEvent",(function(){return E})),n.d(r,"isViewportChanged",(function(){return O})),n.d(r,"isNaN",(function(){return k})),n.d(r,"calculationItemsBBox",(function(){return L})),n.d(r,"processParallelEdges",(function(){return D}));var i={};n.r(i),n.d(i,"mixColor",(function(){return Z})),n.d(i,"getColorsWithSubjectColor",(function(){return Q}));var o={};n.r(o),n.d(o,"getLineIntersect",(function(){return je})),n.d(o,"getRectIntersectByPoint",(function(){return Re})),n.d(o,"getCircleIntersectByPoint",(function(){return ze})),n.d(o,"getEllipseIntersectByPoint",(function(){return Fe})),n.d(o,"applyMatrix",(function(){return Be})),n.d(o,"invertMatrix",(function(){return He})),n.d(o,"getCircleCenterByPoints",(function(){return Ue})),n.d(o,"distance",(function(){return Ve})),n.d(o,"scaleMatrix",(function(){return Ye})),n.d(o,"floydWarshall",(function(){return We})),n.d(o,"getAdjMatrix",(function(){return Ge})),n.d(o,"translate",(function(){return Xe})),n.d(o,"move",(function(){return qe})),n.d(o,"scale",(function(){return Ze})),n.d(o,"rotate",(function(){return Qe})),n.d(o,"getDegree",(function(){return Ke})),n.d(o,"isPointInPolygon",(function(){return $e})),n.d(o,"intersectBBox",(function(){return et})),n.d(o,"isPolygonsIntersect",(function(){return tt})),n.d(o,"Line",(function(){return nt})),n.d(o,"getBBoxBoundLine",(function(){return rt})),n.d(o,"itemIntersectByLine",(function(){return ot})),n.d(o,"fractionToLine",(function(){return at})),n.d(o,"getPointsCenter",(function(){return st})),n.d(o,"squareDist",(function(){return lt})),n.d(o,"pointLineSquareDist",(function(){return ut})),n.d(o,"isPointsOverlap",(function(){return ct})),n.d(o,"pointRectSquareDist",(function(){return dt}));var a={};n.r(a),n.d(a,"getSpline",(function(){return mt})),n.d(a,"getControlPoint",(function(){return yt})),n.d(a,"pointsToPolygon",(function(){return bt})),n.d(a,"pathToPoints",(function(){return Ct})),n.d(a,"getClosedSpline",(function(){return At})),n.d(a,"roundedHull",(function(){return wt})),n.d(a,"paddedHull",(function(){return Mt}));var s={};n.r(s),n.d(s,"getBBox",(function(){return un})),n.d(s,"getLoopCfgs",(function(){return cn})),n.d(s,"getLabelPosition",(function(){return dn})),n.d(s,"traverseTree",(function(){return pn})),n.d(s,"traverseTreeUp",(function(){return gn})),n.d(s,"radialLayout",(function(){return vn})),n.d(s,"getLetterWidth",(function(){return mn})),n.d(s,"getTextSize",(function(){return yn})),n.d(s,"plainCombosToTrees",(function(){return bn})),n.d(s,"reconstructTree",(function(){return Cn})),n.d(s,"getComboBBox",(function(){return An})),n.d(s,"getChartRegion",(function(){return _n}));var l={};n.r(l),n.d(l,"default",(function(){return cs})),n.d(l,"assign",(function(){return Ya})),n.d(l,"format",(function(){return ls})),n.d(l,"parse",(function(){return us})),n.d(l,"defaultI18n",(function(){return qa})),n.d(l,"setGlobalDateI18n",(function(){return Qa})),n.d(l,"setGlobalDateMasks",(function(){return ss}));var u={};n.r(u),n.d(u,"depthFirstSearch",(function(){return kl})),n.d(u,"breadthFirstSearch",(function(){return Dl})),n.d(u,"detectDirectedCycle",(function(){return Fl})),n.d(u,"degree",(function(){return yo})),n.d(u,"adjMatrix",(function(){return Io})),n.d(u,"floydWarshall",(function(){return xo})),n.d(u,"getConnectedComponents",(function(){return Nl})),n.d(u,"detectAllCycles",(function(){return zl})),n.d(u,"detectAllDirectedCycle",(function(){return Rl})),n.d(u,"detectAllUndirectedCycle",(function(){return jl})),n.d(u,"findShortestPath",(function(){return Bl})),n.d(u,"findAllPath",(function(){return Hl})),n.d(u,"minimumSpanningTree",(function(){return Xl})),n.d(u,"getPageRank",(function(){return ql}));var c,d=n(229),h=n(2),f=n.n(h),p=n(7),g={getDefaultCfg:function(){return{}},getEvents:function(){return{}},shouldBegin:function(){return!0},shouldUpdate:function(){return!0},shouldEnd:function(){return!0},bind:function(e){var t=this,n=this.events;this.graph=e,"drag-canvas"!==this.type&&"brush-select"!==this.type&&"lasso-select"!==this.type||e.get("canvas").set("draggable",!0),f()(n,(function(t,n){e.on(n,t)})),document.addEventListener("visibilitychange",(function(){t.keydown=!1}))},unbind:function(e){var t=this.events;"drag-canvas"!==this.type&&"brush-select"!==this.type&&"lasso-select"!==this.type||e.get("canvas").set("draggable",!1),f()(t,(function(t,n){e.off(n,t)}))},get:function(e){return this[e]},set:function(e,t){return this[e]=t,this}},v=function(){function e(){}return e.registerBehavior=function(t,n){if(!n)throw new Error("please specify handler for this behavior: "+t);var r=Object(p.clone)(g);Object.assign(r,n);var i=function(e){var t=this;Object.assign(this,this.getDefaultCfg(),e);var n=this.getEvents();this.events=null;var r={};n&&(Object(p.each)(n,(function(e,n){r[n]=Object(p.wrapBehavior)(t,e)})),this.events=r)};i.prototype=r,e.types[t]=i},e.hasBehavior=function(t){return!!e.types[t]},e.getBehavior=function(t){return e.types[t]},e.types={},e}();!function(e){e.CLICK="click",e.DBLCLICK="dblclick",e.MOUSEDOWN="mousedown",e.MOUDEUP="mouseup",e.CONTEXTMENU="contextmenu",e.MOUSEENTER="mouseenter",e.MOUSEOUT="mouseout",e.MOUSEOVER="mouseover",e.MOUSEMOVE="mousemove",e.MOUSELEAVE="mouseleave",e.DRAGSTART="dragstart",e.DRAGEND="dragend",e.DRAG="drag",e.DRAGENTER="dragenter",e.DRAGLEAVE="dragleave",e.DRAGOVER="dragover",e.DRAGOUT="dragout",e.DDROP="drop",e.KEYUP="keyup",e.KEYDOWN="keydown",e.WHEEL="wheel",e.FOCUS="focus",e.BLUR="blur",e.TOUCHSTART="touchstart",e.TOUCHMOVE="touchmove",e.TOUCHEND="touchend",e.NODE_CONTEXTMENU="node:contextmenu",e.NODE_CLICK="node:click",e.NODE_DBLCLICK="node:dblclick",e.NODE_MOUSEDOWN="node:mousedown",e.NODE_MOUSEUP="node:mouseup",e.NODE_MOUSEENTER="node:mouseenter",e.NODE_MOUSELEAVE="node:mouseleave",e.NODE_MOUSEMOVE="node:mousemove",e.NODE_MOUSEOUT="node:mouseout",e.NODE_MOUSEOVER="node:mouseover",e.NODE_DROP="node:drop",e.NODE_DRAGOVER="node:dragover",e.NODE_DRAGENTER="node:dragenter",e.NODE_DRAGLEAVE="node:dragleave",e.NODE_DRAGSTART="node:dragstart",e.NODE_DRAG="node:drag",e.NODE_DRAGEND="node:dragend",e.COMBO_CONTEXTMENU="combo:contextmenu",e.COMBO_CLICK="combo:click",e.COMBO_DBLCLICK="combo:dblclick",e.COMBO_MOUSEDOWN="combo:mousedown",e.COMBO_MOUSEUP="combo:mouseup",e.COMBO_MOUSEENTER="combo:mouseenter",e.COMBO_MOUSELEAVE="combo:mouseleave",e.COMBO_MOUSEMOVE="combo:mousemove",e.COMBO_MOUSEOUT="combo:mouseout",e.COMBO_MOUSEOVER="combo:mouseover",e.COMBO_DROP="combo:drop",e.COMBO_DRAGOVER="combo:dragover",e.COMBO_DRAGENTER="combo:dragenter",e.COMBO_DRAGLEAVE="combo:dragleave",e.COMBO_DRAGSTART="combo:dragstart",e.COMBO_DRAG="combo:drag",e.COMBO_DRAGEND="combo:dragend",e.EDGE_CONTEXTMENU="edge:contextmenu",e.EDGE_CLICK="edge:click",e.EDGE_DBLCLICK="edge:dblclick",e.EDGE_MOUSEDOWN="edge:mousedown",e.EDGE_MOUSEUP="edge:mouseup",e.EDGE_MOUSEENTER="edge:mouseenter",e.EDGE_MOUSELEAVE="edge:mouseleave",e.EDGE_MOUSEMOVE="edge:mousemove",e.EDGE_MOUSEOUT="edge:mouseout",e.EDGE_MOUSEOVER="edge:mouseover",e.EDGE_DROP="edge:drop",e.EDGE_DRAGOVER="edge:dragover",e.EDGE_DRAGENTER="edge:dragenter",e.EDGE_DRAGLEAVE="edge:dragleave",e.CANVAS_CONTEXTMENU="canvas:contextmenu",e.CANVAS_CLICK="canvas:click",e.CANVAS_DBLCLICK="canvas:dblclick",e.CANVAS_MOUSEDOWN="canvas:mousedown",e.CANVAS_MOUSEUP="canvas:mouseup",e.CANVAS_MOUSEENTER="canvas:mouseenter",e.CANVAS_MOUSELEAVE="canvas:mouseleave",e.CANVAS_MOUSEMOVE="canvas:mousemove",e.CANVAS_MOUSEOUT="canvas:mouseout",e.CANVAS_MOUSEOVER="canvas:mouseover",e.CANVAS_DROP="canvas:drop",e.CANVAS_DRAGENTER="canvas:dragenter",e.CANVAS_DRAGLEAVE="canvas:dragleave",e.CANVAS_DRAGSTART="canvas:dragstart",e.CANVAS_DRAG="canvas:drag",e.CANVAS_DRAGEND="canvas:dragend",e.BEFORERENDER="beforerender",e.AFTERRENDER="afterrender",e.BEFOREADDITEM="beforeadditem",e.AFTERADDITEM="afteradditem",e.BEFOREREMOVEITEM="beforeremoveitem",e.AFTERREMOVEITEM="afterremoveitem",e.BEFOREUPDATEITEM="beforeupdateitem",e.AFTERUPDATEITEM="afterupdateitem",e.BEFOREITEMVISIBILITYCHANGE="beforeitemvisibilitychange",e.AFTERITEMVISIBILITYCHANGE="afteritemvisibilitychange",e.BEFOREITEMSTATECHANGE="beforeitemstatechange",e.AFTERITEMSTATECHANGE="afteritemstatechange",e.BEFOREITEMREFRESH="beforeitemrefresh",e.AFTERITEMREFRESH="afteritemrefresh",e.BEFOREITEMSTATESCLEAR="beforeitemstatesclear",e.AFTERITEMSTATESCLEAR="afteritemstatesclear",e.BEFOREMODECHANGE="beforemodechange",e.AFTERMODECHANGE="aftermodechange",e.BEFORELAYOUT="beforelayout",e.AFTERLAYOUT="afterlayout",e.BEFORECREATEEDGE="beforecreateedge",e.AFTERCREATEEDGE="aftercreateedge",e.BEFOREGRAPHREFRESHPOSITION="beforegraphrefreshposition",e.AFTERGRAPHREFRESHPOSITION="aftergraphrefreshposition",e.BEFOREGRAPHREFRESH="beforegraphrefresh",e.AFTERGRAPHREFRESH="aftergraphrefresh",e.BEFOREANIMATE="beforeanimate",e.AFTERANIMATE="afteranimate",e.BEFOREPAINT="beforepaint",e.AFTERPAINT="afterpaint",e.GRAPHSTATECHANGE="graphstatechange",e.AFTERACTIVATERELATIONS="afteractivaterelations",e.NODESELECTCHANGE="nodeselectchange",e.TOOLTIPCHANGE="tooltipchange",e.WHEELZOOM="wheelzoom",e.VIEWPORTCHANGE="viewportchange",e.DRAGNODEEND="dragnodeend",e.STACKCHANGE="stackchange"}(c||(c={}));var m=n(4),y=n.n(m),b=n(10),C=n.n(b),A=n(11),_=n.n(A),I=n(3),x=n.n(I),w=n(1),M=n(230),S=function(e){function t(t,n){var r=e.call(this,t,n)||this;return r.item=n.item,r.canvasX=n.canvasX,r.canvasY=n.canvasY,r.wheelDelta=n.wheelDelta,r.detail=n.detail,r}return Object(w.__extends)(t,e),t}(n.n(M).a),T=function(e){var t=0,n=0,r=0,i=0;return _()(e)?t=n=r=i=e:x()(e)?t=n=r=i=parseInt(e,10):y()(e)&&(t=e[0],r=C()(e[1])?e[0]:e[1],i=C()(e[2])?e[0]:e[2],n=C()(e[3])?r:e[3]),[t,r,i,n]},E=function(e){var t=new S(e.type,e);return t.clientX=e.clientX,t.clientY=e.clientY,t.x=e.x,t.y=e.y,t.target=e.target,t.currentTarget=e.currentTarget,t.bubbles=!0,t.item=e.item,t},O=function(e){if(!e)return!1;for(var t=[1,0,0,0,1,0,0,0,1],n=0;n<9;n++)if(e[n]!==t[n])return!0;return!1},k=function(e){return Number.isNaN(Number(e))},L=function(e){for(var t=1/0,n=-1/0,r=1/0,i=-1/0,o=0;o<e.length;o++){var a=e[o].getBBox(),s=a.minX,l=a.minY,u=a.maxX,c=a.maxY;s<t&&(t=s),l<r&&(r=l),u>n&&(n=u),c>i&&(i=c)}return{x:Math.floor(t),y:Math.floor(r),width:Math.ceil(n)-Math.floor(t),height:Math.ceil(i)-Math.floor(r),minX:t,minY:r,maxX:n,maxY:i}},D=function(e,t){void 0===t&&(t=15);for(var n=e.length,r=2*t,i=["top","top-right","right","bottom-right","bottom","bottom-left","left","top-left"],o={},a=[],s={},l=0;l<n;l++){var u=e[l],c=u.source,d=u.target,h=c+"-"+d;if(!a[l]){o[h]||(o[h]=[]),a[l]=!0,o[h].push(u);for(var f=0;f<n;f++)if(l!==f){var p=e[f],g=p.source,v=p.target;a[f]||(c===v&&d===g?(o[h].push(p),a[f]=!0,s[g+"|"+v+"|"+(o[h].length-1)]=!0):c===g&&d===v&&(o[h].push(p),a[f]=!0))}}}for(var m in o)for(var y=o[m],b=y.length,C=0;C<b;C++){var A=y[C];if(A.source!==A.target){A.type="quadratic";var _=(C%2==0?1:-1)*(s[A.source+"|"+A.target+"|"+C]?-1:1);A.curveOffset=b%2==1?_*Math.ceil(C/2)*r:_*(Math.floor(C/2)*r+t)}else A.type="loop",A.loopCfg={position:i[C%8],dist:20*Math.floor(C/8)+50}}return e},P=Math.abs,N=["shift","ctrl","alt","control"],j={getDefaultCfg:function(){return{direction:"both",enableOptimize:!1,scalableRange:0}},getEvents:function(){return{dragstart:"onMouseDown",mousedown:"onMouseDown",drag:"onMouseMove",dragend:"onMouseUp",mouseup:"onMouseUp","canvas:click":"onMouseUp",keyup:"onKeyUp",focus:"onKeyUp",keydown:"onKeyDown",touchstart:"onMouseDown",touchmove:"onMouseMove",touchend:"onMouseUp"}},updateViewport:function(e){var t=this.origin,n=+e.clientX,r=+e.clientY;if(!k(n)&&!k(r)){var i=n-t.x,o=r-t.y;"x"===this.get("direction")?o=0:"y"===this.get("direction")&&(i=0),this.origin={x:n,y:r};var a=this.graph.get("width"),s=this.graph.get("height"),l=this.graph.get("canvas").getCanvasBBox();(l.minX<=a+this.scalableRange&&l.minX+i>a+this.scalableRange||l.maxX+this.scalableRange>=0&&l.maxX+this.scalableRange+i<0)&&(i=0),(l.minY<=s+this.scalableRange&&l.minY+o>s+this.scalableRange||l.maxY+this.scalableRange>=0&&l.maxY+this.scalableRange+o<0)&&(o=0),this.graph.translate(i,o)}},onMouseDown:function(e){var t=e.originalEvent;if((!t||0===t.button)&&(e.name===c.TOUCHSTART||!window||!window.event||"undefined"==typeof window||window.event.buttons||window.event.button)&&!this.keydown&&!e.shape&&(this.origin={x:e.clientX,y:e.clientY},this.dragging=!1,this.enableOptimize)){for(var n=this.graph,r=n.getEdges(),i=0,o=r.length;i<o;i++){var a=r[i].get("group").get("children");a&&a.forEach((function(e){e.hide()}))}for(var s=n.getNodes(),l=0,u=s.length;l<u;l++)for(var d=0,h=s[l].getContainer().get("children");d<h.length;d++){var f=h[d];f.get("isKeyShape")||f.hide()}}},onMouseMove:function(e){var t=this.graph;if(!this.keydown&&!e.shape&&(e=E(e),this.origin)){if(this.dragging)e.type="drag",t.emit("canvas:drag",e);else{if(P(this.origin.x-e.clientX)+P(this.origin.y-e.clientY)<10)return;this.shouldBegin.call(this,e)&&(e.type="dragstart",t.emit("canvas:dragstart",e),this.dragging=!0)}this.shouldUpdate.call(this,e)&&this.updateViewport(e)}},onMouseUp:function(e){var t=this.graph;if(!this.keydown&&!e.shape){if(this.enableOptimize){for(var n=t.getEdges(),r=0,i=n.length;r<i;r++){var o=n[r].get("group").get("children");o&&o.forEach((function(e){e.show()}))}for(var a=t.getNodes(),s=0,l=a.length;s<l;s++)for(var u=0,c=a[s].getContainer().get("children");u<c.length;u++){var d=c[u];d.get("isKeyShape")||d.show()}}this.dragging?(e=E(e),this.shouldEnd.call(this,e)&&this.updateViewport(e),e.type="dragend",t.emit("canvas:dragend",e),this.endDrag()):this.origin=null}},endDrag:function(){this.origin=null,this.dragging=!1,this.dragbegin=!1},onKeyDown:function(e){var t=e.key;t&&(N.indexOf(t.toLowerCase())>-1?this.keydown=!0:this.keydown=!1)},onKeyUp:function(){this.keydown=!1,this.origin=null,this.dragging=!1,this.dragbegin=!1}},R=n(70),z=n.n(R),F=n(71),B=n.n(F),H=[{index:7,opacity:.15},{index:6,opacity:.25},{index:5,opacity:.3},{index:5,opacity:.45},{index:5,opacity:.65},{index:5,opacity:.85},{index:4,opacity:.9},{index:3,opacity:.95},{index:2,opacity:.97},{index:1,opacity:.98}];function U(e,t,n){var r;return(r=Math.round(e.h)>=60&&Math.round(e.h)<=240?n?Math.round(e.h)-2*t:Math.round(e.h)+2*t:n?Math.round(e.h)+2*t:Math.round(e.h)-2*t)<0?r+=360:r>=360&&(r-=360),r}function V(e,t,n){return 0===e.h&&0===e.s?e.s:((r=n?e.s-.16*t:4===t?e.s+.16:e.s+.05*t)>1&&(r=1),n&&5===t&&r>.1&&(r=.1),r<.06&&(r=.06),Number(r.toFixed(2)));var r}function Y(e,t,n){var r;return(r=n?e.v+.05*t:e.v-.15*t)>1&&(r=1),Number(r.toFixed(2))}function W(e){for(var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=[],r=B()(e),i=5;i>0;i-=1){var o=r.toHsv(),a=B()({h:U(o,i,!0),s:V(o,i,!0),v:Y(o,i,!0)}).toHexString();n.push(a)}n.push(r.toHexString());for(var s=1;s<=4;s+=1){var l=r.toHsv(),u=B()({h:U(l,s),s:V(l,s),v:Y(l,s)}).toHexString();n.push(u)}return"dark"===t.theme?H.map((function(e){var r=e.index,i=e.opacity;return B.a.mix(t.backgroundColor||"#141414",n[r],100*i).toHexString()})):n}var G={red:"#F5222D",volcano:"#FA541C",orange:"#FA8C16",gold:"#FAAD14",yellow:"#FADB14",lime:"#A0D911",green:"#52C41A",cyan:"#13C2C2",blue:"#1890FF",geekblue:"#2F54EB",purple:"#722ED1",magenta:"#EB2F96",grey:"#666666"},X={},q={};Object.keys(G).forEach((function(e){X[e]=W(G[e]),X[e].primary=X[e][5],q[e]=W(G[e],{theme:"dark",backgroundColor:"#141414"}),q[e].primary=q[e][5]})),X.red,X.volcano,X.gold,X.orange,X.yellow,X.lime,X.green,X.cyan,X.blue,X.geekblue,X.purple,X.magenta,X.grey;var Z=function(e,t,n){var r=z()(e),i=z()(t);return z()([(1-n)*r.color[0]+n*i.color[0],(1-n)*r.color[1]+n*i.color[1],(1-n)*r.color[2]+n*i.color[2]]).rgb()},Q=function(e,t,n,r){void 0===t&&(t="#fff"),void 0===n&&(n="default"),void 0===r&&(r="rgb(150, 150, 150)");var i=Z(t,e,.05).rgb().toString(),o=Z(t,r,.02).rgb().toString(),a=Z(t,r,.05).rgb().toString(),s=Z(t,r,.1).rgb().toString(),l=Z(t,r,.2).rgb().toString(),u=Z(t,r,.3).rgb().toString(),c=W(e,{theme:n,backgroundColor:t}),d=z()(e).hex().toLowerCase(),h=c.indexOf(d),f=e;return-1!==h&&(f=c[h+1]),{mainStroke:e,mainFill:Z(t,e,.1).rgb().toString(),activeStroke:e,activeFill:i,inactiveStroke:Z(t,e,.4).rgb().toString(),inactiveFill:i,selectedStroke:e,selectedFill:t,highlightStroke:f,highlightFill:Z(t,e,.2).rgb().toString(),disableStroke:u,disableFill:a,edgeMainStroke:u,edgeActiveStroke:e,edgeInactiveStroke:l,edgeSelectedStroke:e,edgeHighlightStroke:e,edgeDisableStroke:s,comboMainStroke:u,comboMainFill:o,comboActiveStroke:e,comboActiveFill:i,comboInactiveStroke:u,comboInactiveFill:o,comboSelectedStroke:e,comboSelectedFill:o,comboHighlightStroke:f,comboHighlightFill:o,comboDisableStroke:l,comboDisableFill:a}},K=Q("rgb(95, 149, 255)","rgb(255, 255, 255)"),J={version:"3.8.5",rootContainerClassName:"root-container",nodeContainerClassName:"node-container",edgeContainerClassName:"edge-container",comboContainerClassName:"combo-container",customGroupContainerClassName:"custom-group-container",delegateContainerClassName:"delegate-container",defaultLoopPosition:"top",nodeLabel:{style:{fill:"#000",fontSize:12,textAlign:"center",textBaseline:"middle"},offset:4},defaultNode:{type:"circle",style:{lineWidth:1,stroke:K.mainStroke,fill:K.mainFill},size:20,color:K.mainStroke,linkPoints:{size:8,lineWidth:1,fill:K.activeFill,stroke:K.activeStroke}},nodeStateStyles:{active:{fill:K.activeFill,stroke:K.activeStroke,lineWidth:2,shadowColor:K.mainStroke,shadowBlur:10},selected:{fill:K.selectedFill,stroke:K.selectedStroke,lineWidth:4,shadowColor:K.selectedStroke,shadowBlur:10,"text-shape":{fontWeight:500}},highlight:{fill:K.highlightFill,stroke:K.highlightStroke,lineWidth:2,"text-shape":{fontWeight:500}},inactive:{fill:K.inactiveFill,stroke:K.inactiveStroke,lineWidth:1},disable:{fill:K.disableFill,stroke:K.disableStroke,lineWidth:1}},edgeLabel:{style:{fill:"rgb(0, 0, 0)",textAlign:"center",textBaseline:"middle",fontSize:12}},defaultEdge:{type:"line",size:1,style:{stroke:K.edgeMainStroke,lineAppendWidth:2},color:K.edgeMainStroke},edgeStateStyles:{active:{stroke:K.edgeActiveStroke,lineWidth:1},selected:{stroke:K.edgeSelectedStroke,lineWidth:2,shadowColor:K.edgeSelectedStroke,shadowBlur:10,"text-shape":{fontWeight:500}},highlight:{stroke:K.edgeHighlightStroke,lineWidth:2,"text-shape":{fontWeight:500}},inactive:{stroke:K.edgeInactiveStroke,lineWidth:1},disable:{stroke:K.edgeDisableStroke,lineWidth:1}},comboLabel:{style:{fill:"rgb(0, 0, 0)",textBaseline:"middle",fontSize:12},refY:10,refX:10},defaultCombo:{type:"circle",style:{fill:K.comboMainFill,lineWidth:1,stroke:K.comboMainStroke,r:5,width:20,height:10},size:[20,5],color:K.comboMainStroke,padding:[25,20,15,20]},comboStateStyles:{active:{stroke:K.comboActiveStroke,lineWidth:1,fill:K.comboActiveFill},selected:{stroke:K.comboSelectedStroke,lineWidth:2,fill:K.comboSelectedFill,shadowColor:K.comboSelectedStroke,shadowBlur:10,"text-shape":{fontWeight:500}},highlight:{stroke:K.comboHighlightStroke,lineWidth:2,fill:K.comboHighlightFill,"text-shape":{fontWeight:500}},inactive:{stroke:K.comboInactiveStroke,fill:K.comboInactiveFill,lineWidth:1},disable:{stroke:K.comboDisableStroke,fill:K.comboDisableFill,lineWidth:1}},delegateStyle:{fill:"#F3F9FF",fillOpacity:.5,stroke:"#1890FF",strokeOpacity:.9,lineDash:[5,5]},textWaterMarkerConfig:{width:150,height:100,compatible:!1,text:{x:0,y:60,lineHeight:20,rotate:20,fontSize:14,fontFamily:"Microsoft YaHei",fill:"rgba(0, 0, 0, 0.1)",baseline:"Middle"}},imageWaterMarkerConfig:{width:150,height:130,compatible:!1,image:{x:0,y:0,width:30,height:20,rotate:0}},waterMarkerImage:"https://gw.alipayobjects.com/os/s/prod/antv/assets/image/logo-with-text-73b8a.svg"},$={getDefaultCfg:function(){return{updateEdge:!0,delegateStyle:{},enableDelegate:!1,onlyChangeComboSize:!1,comboActiveState:"",selectedState:"selected"}},getEvents:function(){return{"node:dragstart":"onDragStart","node:drag":"onDrag","node:dragend":"onDragEnd","combo:dragenter":"onDragEnter","combo:dragleave":"onDragLeave","combo:drop":"onDropCombo","node:drop":"onDropNode","canvas:drop":"onDropCanvas"}},validationCombo:function(e){return!(!this.origin||!e||e.destroyed)&&"combo"===e.getType()},onDragStart:function(e){var t=this;if(this.shouldBegin.call(this,e)){var n=e.item;if(n&&!n.destroyed&&!n.hasLocked()){n.getContainer().set("capture",!1);var r=e.target;if(r&&r.get("isAnchorPoint"))return;var i=this.graph;this.targets=[],this.targetCombo=null;var o=i.findAllByState("node",this.selectedState),a=n.get("id");0===o.filter((function(e){var t=e.get("id");return a===t})).length?this.targets.push(n):o.length>1?o.forEach((function(e){e.hasLocked()||t.targets.push(e)})):this.targets.push(n);var s=[];this.targets.forEach((function(e){s.push(Object(p.clone)(e.getModel()))})),this.set("beforeDragNodes",s),this.origin={x:e.x,y:e.y},this.point={},this.originPoint={}}}},onDrag:function(e){var t=this;this.origin&&this.shouldUpdate(this,e)&&(this.get("enableDelegate")?this.updateDelegate(e):this.targets.map((function(n){t.update(n,e)})))},onDragEnd:function(e){if(this.origin&&this.shouldEnd.call(this,e)){var t=e.item;t&&t.getContainer().set("capture",!0),this.delegateRect&&(this.delegateRect.remove(),this.delegateRect=null),this.updatePositions(e);var n=this.graph;if(n.get("enabledStack")){var r={before:{nodes:this.get("beforeDragNodes"),edges:[],combos:[]},after:{nodes:[],edges:[],combos:[]}};this.targets.forEach((function(e){r.after.nodes.push(e.getModel())})),n.pushStack("update",Object(p.clone)(r))}n.emit("dragnodeend",{items:this.targets,targetItem:null}),this.point={},this.origin=null,this.originPoint={},this.targets.length=0,this.targetCombo=null}},onDropCombo:function(e){var t=e.item;if(this.validationCombo(t)){this.updatePositions(e);var n=this.graph;if(this.comboActiveState&&n.setItemState(t,this.comboActiveState,!1),this.targetCombo=t,this.onlyChangeComboSize)n.updateCombos();else{var r=t.getModel();this.targets.map((function(e){e.getModel().comboId!==r.id&&n.updateComboTree(e,r.id)})),n.updateCombo(t)}n.emit("dragnodeend",{items:this.targets,targetItem:this.targetCombo})}},onDropCanvas:function(e){var t=this.graph;this.targets&&0!==this.targets.length&&(this.updatePositions(e),this.onlyChangeComboSize?t.updateCombos():this.targets.map((function(e){e.getModel().comboId&&t.updateComboTree(e)})))},onDropNode:function(e){if(this.targets&&0!==this.targets.length){var t=e.item;this.updatePositions(e);var n=this.graph,r=t.getModel().comboId;if(r){var i=n.findById(r);this.comboActiveState&&n.setItemState(i,this.comboActiveState,!1),this.targets.map((function(e){var t=e.getModel();r!==t.comboId&&n.updateComboTree(e,r)})),n.updateCombo(i)}else this.targets.map((function(e){e.getModel().comboId&&n.updateComboTree(e)}));n.emit("dragnodeend",{items:this.targets,targetItem:t})}},onDragEnter:function(e){var t=e.item;if(this.validationCombo(t)){var n=this.graph;this.comboActiveState&&n.setItemState(t,this.comboActiveState,!0)}},onDragLeave:function(e){var t=e.item;if(this.validationCombo(t)){var n=this.graph;this.comboActiveState&&n.setItemState(t,this.comboActiveState,!1)}},updatePositions:function(e){var t=this;this.targets&&0!==this.targets.length&&this.get("enableDelegate")&&this.targets.map((function(n){return t.update(n,e)}))},update:function(e,t){var n=this.origin,r=e.get("model"),i=e.get("id");this.point[i]||(this.point[i]={x:r.x||0,y:r.y||0});var o={x:t.x-n.x+this.point[i].x,y:t.y-n.y+this.point[i].y};this.get("updateEdge")?this.graph.updateItem(e,o,!1):e.updatePosition(o)},updateDelegate:function(e){if(this.graph,this.delegateRect){var t=e.x-this.origin.x+this.originPoint.minX,n=e.y-this.origin.y+this.originPoint.minY;this.delegateRect.attr({x:t,y:n})}else{var r=this.graph.get("group"),i=Object(p.deepMix)({},J.delegateStyle,this.delegateStyle),o=this.calculationGroupPosition(e),a=o.x,s=o.y,l=o.width,u=o.height,c=o.minX,d=o.minY;this.originPoint={x:a,y:s,width:l,height:u,minX:c,minY:d},this.delegateRect=r.addShape("rect",{attrs:Object(w.__assign)({width:l,height:u,x:a,y:s},i),name:"rect-delegate-shape"}),this.delegateRect.set("capture",!1)}},calculationGroupPosition:function(e){var t=this.graph.findAllByState("node",this.selectedState);0===t.length&&t.push(e.item);for(var n=1/0,r=-1/0,i=1/0,o=-1/0,a=0;a<t.length;a++){var s=t[a].getBBox(),l=s.minX,u=s.minY,c=s.maxX,d=s.maxY;l<n&&(n=l),u<i&&(i=u),c>r&&(r=c),d>o&&(o=d)}return{x:Math.floor(n),y:Math.floor(i),width:Math.ceil(r)-Math.floor(n),height:Math.ceil(o)-Math.floor(i),minX:n,minY:i}}},ee=Math.min,te=Math.max,ne=Math.abs,re=["drag","shift","ctrl","alt","control"],ie={getDefaultCfg:function(){return{brushStyle:{fill:"#EEF6FF",fillOpacity:.4,stroke:"#DDEEFE",lineWidth:1},onSelect:function(){},onDeselect:function(){},selectedState:"selected",trigger:"shift",includeEdges:!0,selectedEdges:[],selectedNodes:[]}},getEvents:function(){return re.indexOf(this.trigger.toLowerCase())>-1||(this.trigger="shift",console.warn("Behavior brush-select 的 trigger 参数不合法，请输入 'drag'、'shift'、'ctrl' 或 'alt'")),"drag"===this.trigger?{dragstart:"onMouseDown",drag:"onMouseMove",dragend:"onMouseUp","canvas:click":"clearStates"}:{dragstart:"onMouseDown",drag:"onMouseMove",dragend:"onMouseUp","canvas:click":"clearStates",keyup:"onKeyUp",keydown:"onKeyDown"}},onMouseDown:function(e){var t=e.item,n=this.brush;t||("drag"===this.trigger||this.keydown)&&(this.selectedNodes&&0!==this.selectedNodes.length&&this.clearStates(),n||(n=this.createBrush()),this.originPoint={x:e.canvasX,y:e.canvasY},n.attr({width:0,height:0}),n.show(),this.dragging=!0)},onMouseMove:function(e){this.dragging&&("drag"===this.trigger||this.keydown)&&this.updateBrush(e)},onMouseUp:function(e){this.graph,(this.brush||this.dragging)&&("drag"===this.trigger||this.keydown)&&(this.brush.remove(!0),this.brush=null,this.getSelectedNodes(e),this.dragging=!1)},clearStates:function(){var e=this.graph,t=this.selectedState,n=e.findAllByState("node",t),r=e.findAllByState("edge",t);n.forEach((function(n){return e.setItemState(n,t,!1)})),r.forEach((function(n){return e.setItemState(n,t,!1)})),this.selectedNodes=[],this.selectedEdges=[],this.onDeselect&&this.onDeselect(this.selectedNodes,this.selectedEdges),e.emit("nodeselectchange",{selectedItems:{nodes:[],edges:[]},select:!1})},getSelectedNodes:function(e){var t=this,n=this.graph,r=this.originPoint,i=this.shouldUpdate,o=this.selectedState,a={x:e.x,y:e.y},s=n.getPointByCanvas(r.x,r.y),l=ee(a.x,s.x),u=te(a.x,s.x),c=ee(a.y,s.y),d=te(a.y,s.y),h=[],f=[];n.getNodes().forEach((function(e){var t=e.getBBox();if(t.centerX>=l&&t.centerX<=u&&t.centerY>=c&&t.centerY<=d&&i(e,"select")){h.push(e);var r=e.getModel();f.push(r.id),n.setItemState(e,o,!0)}}));var p=[];this.includeEdges&&h.forEach((function(e){e.getOutEdges().forEach((function(e){var r=e.getModel(),o=r.source,a=r.target;f.includes(o)&&f.includes(a)&&i(e,"select")&&(p.push(e),n.setItemState(e,t.selectedState,!0))}))})),this.selectedEdges=p,this.selectedNodes=h,this.onSelect&&this.onSelect(h,p),n.emit("nodeselectchange",{selectedItems:{nodes:h,edges:p},select:!0})},createBrush:function(){var e=this.graph.get("canvas").addShape("rect",{attrs:this.brushStyle,capture:!1,name:"brush-shape"});return this.brush=e,e},updateBrush:function(e){var t=this.originPoint;this.brush.attr({width:ne(e.canvasX-t.x),height:ne(e.canvasY-t.y),x:ee(e.canvasX,t.x),y:ee(e.canvasY,t.y)})},onKeyDown:function(e){var t=e.key;if(t){var n=this.trigger.toLowerCase(),r=t.toLowerCase();this.keydown=r===n||"control"===r&&"ctrl"===n||"ctrl"===r&&"control"===n}},onKeyUp:function(){this.brush&&(this.brush.remove(!0),this.brush=null,this.dragging=!1),this.keydown=!1}},oe=["shift","ctrl","alt","control"],ae={getDefaultCfg:function(){return{multiple:!0,trigger:"shift",selectedState:"selected"}},getEvents:function(){return oe.indexOf(this.trigger.toLowerCase())>-1||(this.trigger="shift",console.warn("Behavior brush-select 的 trigger 参数不合法，请输入 'drag'、'shift'、'ctrl' 或 'alt'")),this.multiple?{"node:click":"onClick","combo:click":"onClick","canvas:click":"onCanvasClick",keyup:"onKeyUp",keydown:"onKeyDown"}:{"node:click":"onClick","combo:click":"onClick","canvas:click":"onCanvasClick"}},onClick:function(e){var t=this,n=e.item;if(n&&!n.destroyed){var r=n.getType(),i=this,o=i.graph,a=i.keydown,s=i.multiple,l=i.shouldUpdate;if(i.shouldBegin.call(this,e)){if(!a||!s){var u=o.findAllByState(r,this.selectedState);f()(u,(function(e){e!==n&&o.setItemState(e,t.selectedState,!1)}))}if(n.hasState(this.selectedState)){l.call(this,e)&&o.setItemState(n,this.selectedState,!1);var c=o.findAllByState("node",this.selectedState),d=o.findAllByState("combo",this.selectedState);o.emit("nodeselectchange",{target:n,selectedItems:{nodes:c,combos:d},select:!1})}else l.call(this,e)&&o.setItemState(n,this.selectedState,!0),c=o.findAllByState("node",this.selectedState),d=o.findAllByState("combo",this.selectedState),o.emit("nodeselectchange",{target:n,selectedItems:{nodes:c,combos:d},select:!0})}}},onCanvasClick:function(){var e=this,t=this.graph,n=t.findAllByState("node",this.selectedState);f()(n,(function(n){t.setItemState(n,e.selectedState,!1)}));var r=t.findAllByState("combo",this.selectedState);f()(r,(function(n){t.setItemState(n,e.selectedState,!1)})),t.emit("nodeselectchange",{selectedItems:{nodes:[],edges:[],combos:[]},select:!1})},onKeyDown:function(e){var t=e.key;t&&(t.toLowerCase()===this.trigger.toLowerCase()||"control"===t.toLowerCase()?this.keydown=!0:this.keydown=!1)},onKeyUp:function(){this.keydown=!1}},se=n(16);se.translate=function(e,t,n){var r=new Array(9);return se.fromTranslation(r,n),se.multiply(e,r,t)},se.rotate=function(e,t,n){var r=new Array(9);return se.fromRotation(r,n),se.multiply(e,r,t)},se.scale=function(e,t,n){var r=new Array(9);return se.fromScaling(r,n),se.multiply(e,r,t)},se.transform=function(e,t){for(var n=[].concat(e),r=0,i=t.length;r<i;r++){var o=t[r];switch(o[0]){case"t":se.translate(n,n,[o[1],o[2]]);break;case"s":se.scale(n,n,[o[1],o[2]]);break;case"r":se.rotate(n,n,o[1])}}return n};var le=se,ue=n(26),ce=n(0);ue.angle=function(e,t){var n=ue.dot(e,t)/(ue.length(e)*ue.length(t));return Math.acos(Object(ce.clamp)(n,-1,1))},ue.direction=function(e,t){return e[0]*t[1]-t[0]*e[1]},ue.angleTo=function(e,t,n){var r=ue.angle(e,t),i=ue.direction(e,t)>=0;return n?i?2*Math.PI-r:r:i?r:2*Math.PI-r},ue.vertical=function(e,t,n){return n?(e[0]=t[1],e[1]=-1*t[0]):(e[0]=-1*t[1],e[1]=t[0]),e};var de=ue,he=n(131),fe=function(e,t){var n=e?Object(ce.clone)(e):[1,0,0,0,1,0,0,0,1];return Object(ce.each)(t,(function(e){switch(e[0]){case"t":le.translate(n,n,[e[1],e[2]]);break;case"s":le.scale(n,n,[e[1],e[2]]);break;case"r":le.rotate(n,n,e[1]);break;case"m":le.multiply(n,n,e[1]);break;default:return!1}})),n},pe={getDefaultCfg:function(){return{sensitivity:2,minZoom:void 0,maxZoom:void 0,enableOptimize:!1,optimizeZoom:.7,fixSelectedItems:{fixAll:!1,fixLineWidth:!1,fixLabel:!1,fixState:"selected"}}},getEvents:function(){var e=this.fixSelectedItems;return e.fixState||(e.fixState="selected"),e.fixAll&&(e.fixLineWidth=!0,e.fixLabel=!0),{wheel:"onWheel",touchstart:"onTouchStart",touchmove:"onTouchMove",touchend:"onTouchEnd"}},onTouchStart:function(e){var t=e.originalEvent.touches,n=t[0],r=t[1];e.preventDefault(),r&&(this.startPoint={pageX:n.pageX,pageY:n.pageY},this.moveable=!0,r&&(this.endPoint={pageX:r.pageX,pageY:r.pageY}),this.originScale=this.currentScale||1)},onTouchMove:function(e){if(this.moveable){e.preventDefault();var t=e.originalEvent.touches,n=t[0],r=t[1];if(r){this.endPoint||(this.endPoint={pageX:r.pageX,pageY:r.pageY});var i=function(e,t){return Math.hypot(t.x-e.x,t.y-e.y)},o=i({x:n.pageX,y:n.pageY},{x:r.pageX,y:r.pageY})/i({x:this.startPoint.pageX,y:this.startPoint.pageY},{x:this.endPoint.pageX,y:this.endPoint.pageY}),a=this.originScale*o;this.currentScale=a;var s=this.get("minZoom")||this.graph.get("minZoom");if(!(a>(this.get("maxZoom")||this.graph.get("maxZoom"))||a<s)){var l=this.graph.get("canvas").getPointByClient(e.clientX,e.clientY);this.graph.zoomTo(a,{x:l.x,y:l.y}),this.graph.emit("wheelzoom",e)}}}},onTouchEnd:function(){this.moveable=!1,this.endPoint=null},onWheel:function(e){var t=this,n=this.graph,r=this.fixSelectedItems;if(this.shouldUpdate.call(this,e)){e.preventDefault();var i,o=n.get("canvas").getPointByClient(e.clientX,e.clientY),a=this.get("sensitivity"),s=n.getZoom();i=s*(e.wheelDelta<0?1-.05*a:1/(1-.05*a));var l=this.get("minZoom")||n.get("minZoom");if(!(i>(this.get("maxZoom")||n.get("maxZoom"))||i<l)){if(this.get("enableOptimize")){var u=this.get("optimizeZoom"),c=this.get("optimized"),d=n.getNodes(),h=n.getEdges(),f=d.length,p=h.length;if(!c){for(var g=0;g<f;g++)if(!(w=d[g]).destroyed)for(var v=(X=w.get("group").get("children")).length,m=0;m<v;m++)(N=X[m]).destoryed||N.get("isKeyShape")||N.hide();for(var y=0;y<p;y++)for(v=(X=(G=h[y]).get("group").get("children")).length,m=0;m<v;m++)(N=X[m]).hide();this.set("optimized",!0)}clearTimeout(this.get("timeout"));var b=setTimeout((function(){var e=n.getZoom();if(t.get("optimized")){t.set("optimized",!1);for(var r=0;r<f;r++){var i=d[r],o=(l=i.get("group").get("children")).length;if(e<u)i.getKeyShape().show();else for(var a=0;a<o;a++)(g=l[a]).get("visible")||g.show()}for(var s=0;s<p;s++){var l,c=h[s];if(o=(l=c.get("group").get("children")).length,e<u)c.getKeyShape().show();else for(a=0;a<o;a++){var g;(g=l[a]).get("visible")||g.show()}}}}),500);this.set("timeout",b)}if(s<=1){var C=void 0,A=void 0;if(r.fixAll||r.fixLineWidth||r.fixLabel){C=n.findAllByState("node",r.fixState),A=n.findAllByState("edge",r.fixState);for(var _=s/i,I=C.length,x=0;x<I;x++){var w,M=(w=C[x]).getContainer(),S=w.getModel(),T=w.getOriginStyle(),E=w.getStateStyle(r.fixState),O=w.get("shapeFactory").getShape(S.shape||S.type).getStateStyle(r.fixState,w)[r.fixState];if(r.fixAll){if(i<=1){var k=Object(ce.clone)(M.getMatrix());k||(k=[1,0,0,0,1,0,0,0,1]);var L=w.getModel(),D=L.x,P=L.y;le.translate(k,k,[-D,-P]),le.scale(k,k,[_,_]),le.translate(k,k,[D,P]),M.setMatrix(k)}}else for(v=(X=M.get("children")).length,m=0;m<v;m++){var N=X[m],j=void 0,R=void 0;if(r.fixLabel&&"text"===N.get("type")){j=N.attr("fontSize")||12;var z=E[N.get("name")],F=O[N.get("name")],B=z?z.fontSize:12,H=F?F.fontSize:12,U=B||H||12;if(i<=1&&N.attr("fontSize",U/i),R)break}if(r.fixLineWidth&&N.get("isKeyShape")){R=N.attr("lineWidth")||0;var V=E.lineWidth||O.lineWidth||T.lineWidth||0;if(i<=1&&N.attr("lineWidth",V/i),j)break}}}for(var Y=A.length,W=0;W<Y;W++){var G,X=(M=(G=A[W]).getContainer()).get("children");for(S=G.getModel(),E=G.getStateStyle(r.fixState),O=G.get("shapeFactory").getShape(S.shape||S.type).getStateStyle(r.fixState,G)[r.fixState],v=X.length,m=0;m<v&&(N=X[m],j=void 0,R=void 0,!r.fixLabel&&!r.fixAll||"text"!==N.get("type")||(j=N.attr("fontSize")||12,z=E[N.get("name")],F=O[N.get("name")],B=z?z.fontSize:12,H=F?F.fontSize:12,U=B||H||12,i<=1&&N.attr("fontSize",U/i),!R))&&(!r.fixLineWidth&&!r.fixAll||!N.get("isKeyShape")||(R=N.attr("lineWidth")||0,V=E.lineWidth||O.lineWidth||1,i<=1&&N.attr("lineWidth",V/i),!j));m++);}}}n.zoomTo(i,{x:o.x,y:o.y}),n.emit("wheelzoom",e)}}}},ge=n(5),ve=n.n(ge),me=n(8),ye=n.n(me),be={onMouseEnter:function(e){var t=e.item;this.currentTarget=t,this.showTooltip(e),this.graph.emit("tooltipchange",{item:e.item,action:"show"})},onMouseMove:function(e){this.shouldUpdate(e)?this.currentTarget&&e.item===this.currentTarget&&this.updatePosition(e):this.hideTooltip()},onMouseLeave:function(e){this.shouldEnd(e)&&(this.hideTooltip(),this.graph.emit("tooltipchange",{item:this.currentTarget,action:"hide"}),this.currentTarget=null)},showTooltip:function(e){var t=this.container;if(e.item&&!e.item.destroyed){t||(t=this.createTooltip(this.graph.get("canvas")),this.container=t);var n=this.formatText(e.item.get("model"),e);t.innerHTML=n,ve()(this.container,{visibility:"visible"}),this.updatePosition(e)}},hideTooltip:function(){ve()(this.container,{visibility:"hidden"})},updatePosition:function(e){var t=this.get("shouldBegin"),n=this.width,r=this.height,i=this.container,o=this.graph;if(t(e)){var a=o.getPointByClient(e.clientX,e.clientY),s=o.getCanvasByPoint(a.x,a.y),l=s.x,u=s.y,c=i.getBoundingClientRect();l>n/2?l-=c.width:l+=this.offset,u>r/2?u-=c.height:u+=this.offset;var d=l+"px",h=u+"px";ve()(this.container,{left:d,top:h,visibility:"visible"})}else ve()(i,{visibility:"hidden"})},createTooltip:function(e){var t=e.get("el");t.style.position="relative";var n=ye()('<div class="g6-tooltip g6-'+this.item+'-tooltip"></div>');return t.parentNode.appendChild(n),ve()(n,{position:"absolute",visibility:"visible"}),this.width=e.get("width"),this.height=e.get("height"),this.container=n,this.graph.get("tooltips").push(n),n}},Ce=Object(w.__assign)({getDefaultCfg:function(){return{item:"node",offset:12,formatText:function(e){return e.label}}},getEvents:function(){return{"node:mouseenter":"onMouseEnter","node:mouseleave":"onMouseLeave","node:mousemove":"onMouseMove",afterremoveitem:"onMouseLeave"}}},be),Ae=Object(w.__assign)({getDefaultCfg:function(){return{item:"edge",offset:12,formatText:function(e){return"source: "+e.source+" target: "+e.target}}},getEvents:function(){return{"edge:mouseenter":"onMouseEnter","edge:mouseleave":"onMouseLeave","edge:mousemove":"onMouseMove",afterremoveitem:"onMouseLeave"}}},be),_e=n(12),Ie=n.n(_e),xe={getDefaultCfg:function(){return{delegate:!0,delegateStyle:{},delegateShapes:{},delegateShapeBBoxs:{}}},getEvents:function(){return{dragstart:"onDragStart",drag:"onDrag",dragend:"onDragEnd","canvas:mouseleave":"onOutOfRange"}},onDragStart:function(e){var t=this.graph,n=e.target.get("groupId");if(n){var r=t.get("customGroupControll"),i=r.customGroup,o=i[n].nodeGroup;this.targetGroup=o,this.mouseOrigin={x:e.x,y:e.y};for(var a=t.save().groups,s=null,l=0;l<a.length;l++){var u=a[l];if(n===u.id){s=u.parentId;break}}if(s){var c=i[s].nodeGroup;r.setGroupStyle(c.get("keyShape"),"hover")}}},onDrag:function(e){this.mouseOrigin&&this.updateDelegate(e)},onDragEnd:function(e){var t=this.graph,n=e.target.get("groupId");if(this.delegateShapes[n]&&(this.delegateShapeBBox=this.delegateShapes[n].getBBox(),this.delegateShapes[n].remove(),delete this.delegateShapes[n]),this.delegateShapeBBox){var r=t.get("customGroupControll"),i=this.delegateShapeBBoxs[n];r.updateGroup(n,i,this.mouseOrigin),this.mouseOrigin=null,this.shapeOrigin=null,r.resetNodePoint(),this.delegateShapeBBox=null}},updateDelegate:function(e){var t=this.graph,n=e.target.get("groupId"),r=this.targetGroup.get("keyShape"),i=this.delegateShapes[n],o=r.getBBox(),a=r.get("type");if(i){var s=this.mouseOrigin,l=this.shapeOrigin,u=e.x-s.x,c=e.y-s.y,d=u+l.x,h=c+l.y;i.attr({x:d,y:h}),this.delegateShapeBBoxs[n]=i.getBBox()}else{var f=t.get("delegateGroup"),p=o.x,g=o.y,v=o.width,m=o.height,y=Object(w.__assign)({width:v,height:m},Ie()({},J.delegateStyle,this.delegateStyle));if("circle"===a){var b=v>m?v/2:m/2,C=p+b,A=g+b;i=f.addShape("circle",{attrs:Object(w.__assign)({x:C,y:A,r:b},y),name:"circle-delegate-shape"}),this.shapeOrigin={x:C,y:A}}else i=f.addShape("rect",{attrs:Object(w.__assign)({x:p,y:g},y),name:"rect-delegate-shape"}),this.shapeOrigin={x:p,y:g};this.delegateShapes[n]=i,this.delegateShapeBBoxs[n]=i.getBBox()}},onOutOfRange:function(e){var t=this,n=this.graph.get("canvas").get("el"),r=function(i){i.target!==n&&(t.onDragEnd(e),document.body.removeEventListener("mouseup",r,!0))};this.mouseOrigin&&document.body.addEventListener("mouseup",r,!0)}},we=document.body,Me={getDefaultCfg:function(){return{updateEdge:!0,delegate:!0,delegateStyle:{},maxMultiple:1.1,minMultiple:1}},getEvents:function(){return{"node:dragstart":"onDragStart","node:drag":"onDrag","node:dragend":"onDragEnd","canvas:mouseleave":"onOutOfRange",dragover:"onDragOver",dragleave:"onDragLeave"}},onDragOver:function(e){var t=this.graph,n=e.target.get("groupId");if(n&&this.origin){var r=t.get("customGroupControll"),i=r.getDeletageGroupById(n);if(i){var o=i.nodeGroup.get("keyShape");this.inGroupId=n,r.setGroupStyle(o,"hover")}}},onDragLeave:function(e){var t=this.graph,n=e.target.get("groupId");if(n&&this.origin){var r=t.get("customGroupControll"),i=r.getDeletageGroupById(n);if(i){var o=i.nodeGroup.get("keyShape");r.setGroupStyle(o,"default")}}n||(this.inGroupId=null)},onDragStart:function(e){var t=this.graph;if(this.shouldBegin.call(this,e)){var n=e.item;this.target=n;var r=n.getModel().groupId;if(r){var i=t.get("customGroupControll"),o=i.getDeletageGroupById(r);if(o){var a=o.nodeGroup.get("keyShape");i.setGroupStyle(a,"hover"),this.inGroupId=r}}this.origin={x:e.x,y:e.y},this.point={},this.originPoint={}}},onDrag:function(e){if(this.origin&&this.get("shouldUpdate").call(this,e)){this.update(this.target,e,!0);var t=e.item,n=this.graph,r=t.getModel().groupId;if(r){var i=n.get("customGroupControll"),o=i.getDeletageGroupById(r);if(o){var a=o.nodeGroup.get("keyShape");this.inGroupId!==r?i.setGroupStyle(a,"default"):i.setGroupStyle(a,"hover")}}}},onDragEnd:function(e){if(this.origin&&this.shouldEnd.call(this,e)){if(this.shape&&(this.shape.remove(),this.shape=null),this.target){var t=this.target.get("delegateShape");t&&(t.remove(),this.target.set("delegateShape",null))}this.target&&this.update(this.target,e),this.point={},this.origin=null,this.originPoint={},this.target=null,this.setCurrentGroupStyle(e)}},setCurrentGroupStyle:function(e){var t=this.graph,n=e.item,r=n.getModel(),i=r.groupId,o=r.id,a=t.get("customGroupControll"),s=a.customGroup,l=t.get("groupNodes");if(this.inGroupId&&i){if(!(A=s[i].nodeGroup))return;var u=A.get("keyShape"),c=n.getBBox(),d=u.getBBox(),h=c.centerX,f=c.centerY,p=d.minX,g=d.minY,v=d.maxX,m=d.maxY;if(!(h<v*this.maxMultiple&&h>p*this.minMultiple&&f<m*this.maxMultiple&&f>g*this.minMultiple)||this.inGroupId!==i){var y=l[i];l[i]=y.filter((function(e){return e!==o})),a.dynamicChangeGroupSize(e,A,u),delete r.groupId}if(this.inGroupId!==i){if(!(C=s[this.inGroupId].nodeGroup))return;var b=C.get("keyShape");-1===l[this.inGroupId].indexOf(o)&&l[this.inGroupId].push(o),r.groupId=this.inGroupId,a.dynamicChangeGroupSize(e,C,b)}a.setGroupStyle(u,"default")}else if(this.inGroupId&&!i){var C;if(!(C=s[this.inGroupId].nodeGroup))return;u=C.get("keyShape"),-1===l[this.inGroupId].indexOf(o)&&l[this.inGroupId].push(o),r.groupId=this.inGroupId,a.dynamicChangeGroupSize(e,C,u)}else if(!this.inGroupId&&i){var A;if(Object.keys(l).forEach((function(e){var t=l[e];l[e]=t.filter((function(e){return e!==o}))})),!(A=s[i].nodeGroup))return;u=A.get("keyShape"),a.dynamicChangeGroupSize(e,A,u),delete r.groupId}this.inGroupId=null},onOutOfRange:function(e){var t=this,n=t.graph.get("canvas").get("el");t.origin&&we.addEventListener("mouseup",(function r(i){i.target!==n&&(e.item=t.target,t.onDragEnd(e),document.body.removeEventListener("mouseup",r,!0))}),!0)},update:function(e,t,n){var r=this.origin,i=e.get("model"),o=e.get("id");this.point[o]||(this.point[o]={x:i.x,y:i.y});var a=t.x-r.x+this.point[o].x,s=t.y-r.y+this.point[o].y;if(n)this.updateDelegate(t,a,s);else{var l={x:a,y:s};this.get("updateEdge")?this.graph.updateItem(e,l):(e.updatePosition(l),this.graph.paint())}},updateDelegate:function(e,t,n){var r=this.graph,i=e.item,o=r.get("groupType"),a=i.get("keyShape").getBBox();if(!this.shape){var s=r.get("delegateGroup"),l=Ie()({},J.delegateStyle,this.delegateStyle);this.target&&(this.shape=s.addShape("rect",{attrs:Object(w.__assign)({width:a.width,height:a.height,x:t-a.width/2,y:n-a.height/2},l),name:"delegate-shape"}),this.target.set("delegateShape",this.shape)),this.shape.set("capture",!1)}this.target&&("circle"===o?this.shape.attr({x:t-a.width/2,y:n-a.height/2}):"rect"===o&&this.shape.attr({x:t,y:n}))}},Se=["click","dblclick"],Te={getDefaultCfg:function(){return{trigger:"dblclick"}},getEvents:function(){var e,t;return Se.includes(this.trigger)?t=this.trigger:(t="dblclick",console.warn("Behavior collapse-expand-group 的 trigger 参数不合法，请输入 'click' 或 'dblclick'")),(e={})[""+t]="onGroupClick",e},onGroupClick:function(e){var t=e.target,n=this.graph,r=t.get("groupId");r&&n.get("customGroupControll").collapseExpandGroup(r)}},Ee=["click","dblclick"],Oe={getDefaultCfg:function(){return{trigger:"click",onChange:function(){}}},getEvents:function(){var e,t;return Ee.includes(this.trigger)?t=this.trigger:(t="click",console.warn("Behavior collapse-expand 的 trigger 参数不合法，请输入 'click' 或 'dblclick'")),(e={})["node:"+t]="onNodeClick",e.touchstart="onNodeClick",e},onNodeClick:function(e){var t=e.item,n=this.graph.findDataById(t.get("id"));if(n){var r=n.children;if(r&&0!==r.length){var i=!n.collapsed;if(this.shouldBegin(e,i)&&(n.collapsed=i,t.getModel().collapsed=i,this.graph.emit("itemcollapsed",{item:e.item,collapsed:i}),this.shouldUpdate(e,i))){try{this.onChange(t,i)}catch(e){console.warn("G6 自 3.0.4 版本支持直接从 item.getModel() 获取源数据(临时通知，将在3.2.0版本中清除)",e)}this.graph.layout()}}}}},ke=function(e,t){if(!1!==t(e)&&e){var n=e.get("combos");if(0===n.length)return!1;Object(p.each)(n,(function(e){ke(e,t)}))}},Le={getDefaultCfg:function(){return{enableDelegate:!1,delegateStyle:{},onlyChangeComboSize:!1,activeState:"",selectedState:"selected"}},getEvents:function(){return{"combo:dragstart":"onDragStart","combo:drag":"onDrag","combo:dragend":"onDragEnd","combo:drop":"onDrop","node:drop":"onNodeDrop","combo:dragenter":"onDragEnter","combo:dragleave":"onDragLeave"}},validationCombo:function(e){var t=e.item;return!(!t||t.destroyed||!this.shouldUpdate(this,e)||"combo"!==t.getType())},onDragStart:function(e){var t=this,n=this.graph,r=e.item;if(this.validationCombo(e)){this.targets=[];var i=n.findAllByState("combo",this.selectedState),o=r.get("id");0===i.filter((function(e){var t=e.get("id");return o===t})).length?this.targets.push(r):this.targets=i,this.activeState&&this.targets.map((function(e){var r=e.getModel();if(r.parentId){var i=n.findById(r.parentId);i&&n.setItemState(i,t.activeState,!0)}})),this.point={},this.originPoint={},this.origin={x:e.x,y:e.y},this.currentItemChildCombos=[],ke(r,(function(e){if(e.destroyed)return!1;var n=e.getModel();return t.currentItemChildCombos.push(n.id),!0}))}},onDrag:function(e){var t=this;if(this.origin&&this.validationCombo(e))if(this.enableDelegate)this.updateDelegate(e);else{if(this.activeState){var n=this.graph,r=e.item,i=r.getModel(),o=n.getCombos(),a=r.getBBox(),s=a.centerX,l=a.centerY,u=a.width;o.filter((function(e){var n=e.getModel();return i.parentId,n.id!==i.id&&!t.currentItemChildCombos.includes(n.id)})).map((function(e){var r=e.getBBox(),i=r.centerX,o=r.centerY,a=r.width,c=s-i,d=l-o,h=2*Math.sqrt(c*c+d*d);u+a-h>.8*u?n.setItemState(e,t.activeState,!0):n.setItemState(e,t.activeState,!1)}))}Object(p.each)(this.targets,(function(n){t.updateCombo(n,e)}))}},updatePositions:function(e){var t=this;this.enableDelegate&&Object(p.each)(this.targets,(function(n){t.updateCombo(n,e)}))},onDrop:function(e){var t=this,n=e.item;if(n&&this.targets&&!n.destroyed){this.updatePositions(e);var r=this.graph,i=n.getModel();this.targets.map((function(e){e.getModel().parentId!==i.id?(t.activeState&&r.setItemState(n,t.activeState,!1),t.onlyChangeComboSize?r.updateCombo(e):r.updateComboTree(e,i.id)):r.updateCombo(n)})),this.end(n,e),this.endComparison=!0}},onNodeDrop:function(e){var t=this;if(this.targets&&0!==this.targets.length){this.updatePositions(e);var n,r=this.graph,i=e.item.getModel().comboId;if(i){if(this.activeState){var o=r.findById(i);r.setItemState(o,this.activeState,!1)}this.targets.map((function(e){t.onlyChangeComboSize?r.updateCombo(e):i!==e.getID()&&(n=r.findById(i),i!==e.getModel().parentId&&r.updateComboTree(e,i))}))}else this.targets.map((function(e){t.onlyChangeComboSize?r.updateCombo(e):e.getModel().comboId&&r.updateComboTree(e)}));this.endComparison=!0,this.end(n,e)}},onDragEnter:function(e){if(this.origin&&this.validationCombo(e)){var t=e.item,n=this.graph;this.activeState&&n.setItemState(t,this.activeState,!0)}},onDragLeave:function(e){if(this.origin&&this.validationCombo(e)){var t=e.item,n=this.graph;this.activeState&&n.setItemState(t,this.activeState,!1)}},onDragEnd:function(e){if(this.targets&&0!==this.targets.length){var t=e.item;this.updatePositions(e);var n=this.getParentCombo(t.getModel().parentId),r=this.graph;n&&this.activeState&&r.setItemState(n,this.activeState,!1),this.end(void 0,e)}},end:function(e,t){var n=this;if(this.origin){var r=this.graph;this.delegateShape&&(r.get("delegateGroup").clear(),this.delegateShape=null),e&&this.activeState&&r.setItemState(e,this.activeState,!1),e||this.targets.map((function(e){n.onlyChangeComboSize?r.updateCombo(e):r.updateComboTree(e)})),this.point=[],this.origin=null,this.originPoint=null,this.targets.length=0}},traverse:function(e,t){var n=this;if(!1!==t(e)&&e){var r=e.get("combos");Object(p.each)(r,(function(e){n.traverse(e,t)}));var i=e.get("nodes");Object(p.each)(i,(function(e){n.traverse(e,t)}))}},updateCombo:function(e,t){var n=this;this.traverse(e,(function(e){return!e.destroyed&&(n.updateSignleItem(e,t),!0)}))},updateSignleItem:function(e,t){var n=this.origin,r=this.graph,i=e.getModel(),o=e.get("id");this.point[o]||(this.point[o]={x:i.x,y:i.y});var a=t.x-n.x+this.point[o].x,s=t.y-n.y+this.point[o].y;r.updateItem(e,{x:a,y:s})},getParentCombo:function(e){var t=this.graph;if(e){var n=t.findById(e);if(n)return n}},updateDelegate:function(e){var t=this.graph;if(this.delegateShape){var n=e.x-this.origin.x+this.originPoint.minX,r=e.y-this.origin.y+this.originPoint.minY;this.delegateShape.attr({x:n,y:r})}else{var i,o=t.get("delegateGroup"),a=(i=this.targets.length>1?L(this.targets):this.targets[0].getBBox()).x,s=i.y,l=i.width,u=i.height,c=i.minX,d=i.minY;this.originPoint={x:a,y:s,width:l,height:u,minX:c,minY:d};var h=Object(w.__assign)(Object(w.__assign)({},J.delegateStyle),this.delegateStyle);this.delegateShape=o.addShape("rect",{attrs:Object(w.__assign)({width:i.width,height:i.height,x:i.x,y:i.y},h),name:"combo-delegate-shape"})}}},De=["click","dblclick"],Pe={getDefaultCfg:function(){return{trigger:"dblclick",relayout:!0}},getEvents:function(){var e,t;return De.includes(this.trigger)?t=this.trigger:(t="dblclick",console.warn("Behavior collapse-expand-group 的 trigger 参数不合法，请输入 'click' 或 'dblclick'")),(e={})["combo:"+t]="onComboClick",e},onComboClick:function(e){var t=e.item,n=this.graph,r=this.relayout;if(t&&!t.destroyed&&"combo"===t.getType()){var i=t.getModel().id;i&&(n.collapseExpandCombo(i),r&&n.get("layout")?n.layout():n.refreshPositions())}}},Ne=function(e,t,n){return e>=t&&e<=n},je=function(e,t,n,r){var i=n.x-e.x,o=n.y-e.y,a=t.x-e.x,s=t.y-e.y,l=r.x-n.x,u=r.y-n.y,c=a*u-s*l,d=null;if(c*c>.001*(a*a+s*s)*(l*l+u*u)){var h=(i*u-o*l)/c,f=(i*s-o*a)/c;Ne(h,0,1)&&Ne(f,0,1)&&(d={x:e.x+h*a,y:e.y+h*s})}return d},Re=function(e,t){var n=e.x,r=e.y,i=e.width,o=e.height,a=[],s={x:n+i/2,y:r+o/2};a.push({x:n,y:r}),a.push({x:n+i,y:r}),a.push({x:n+i,y:r+o}),a.push({x:n,y:r+o}),a.push({x:n,y:r});for(var l=null,u=1;u<a.length&&!(l=je(a[u-1],a[u],s,t));u++);return l},ze=function(e,t){var n=e.x,r=e.y,i=e.r,o=t.x-n,a=t.y-r;if(Math.sqrt(o*o+a*a)<i)return null;var s=Math.sign(o),l=Math.sign(a),u=Math.atan(a/o);return{x:n+Math.abs(i*Math.cos(u))*s,y:r+Math.abs(i*Math.sin(u))*l}},Fe=function(e,t){var n=e.rx,r=e.ry,i=e.x,o=e.y,a=t.x-i,s=t.y-o,l=Math.atan2(s/r,a/n);return l<0&&(l+=2*Math.PI),{x:i+n*Math.cos(l),y:o+r*Math.sin(l)}},Be=function(e,t,n){void 0===n&&(n=1);var r=[e.x,e.y,n];return t&&!isNaN(t[0])||(t=[1,0,0,0,1,0,0,0,1]),he.transformMat3(r,r,t),{x:r[0],y:r[1]}},He=function(e,t,n){void 0===n&&(n=1),t&&!isNaN(t[0])||(t=[1,0,0,0,1,0,0,0,1]);var r=le.invert([],t);r||(r=[1,0,0,0,1,0,0,0,1]);var i=[e.x,e.y,n];return he.transformMat3(i,i,r),{x:i[0],y:i[1]}},Ue=function(e,t,n){var r=e.x-t.x,i=e.y-t.y,o=e.x-n.x,a=e.y-n.y,s=(e.x*e.x-t.x*t.x-t.y*t.y+e.y*e.y)/2,l=(e.x*e.x-n.x*n.x-n.y*n.y+e.y*e.y)/2,u=i*o-r*a;return{x:-(a*s-i*l)/u,y:-(r*l-o*s)/u}},Ve=function(e,t){var n=e.x-t.x,r=e.y-t.y;return Math.sqrt(n*n+r*r)},Ye=function(e,t){var n=[];return e.forEach((function(e){var r=[];e.forEach((function(e){r.push(e*t)})),n.push(r)})),n},We=function(e){for(var t=[],n=e.length,r=0;r<n;r+=1){t[r]=[];for(var i=0;i<n;i+=1)r===i?t[r][i]=0:0!==e[r][i]&&e[r][i]?t[r][i]=e[r][i]:t[r][i]=1/0}for(var o=0;o<n;o+=1)for(r=0;r<n;r+=1)for(i=0;i<n;i+=1)t[r][i]>t[r][o]+t[o][i]&&(t[r][i]=t[r][o]+t[o][i]);return t},Ge=function(e,t){var n=e.nodes,r=e.edges,i=[],o={};if(!n)throw new Error("invalid nodes data!");return n&&n.forEach((function(e,t){o[e.id]=t,i.push([])})),r&&r.forEach((function(e){var n=e.source,r=e.target,a=o[n],s=o[r];i[a][s]=1,t||(i[s][a]=1)})),i},Xe=function(e,t){e.translate(t.x,t.y)},qe=function(e,t){var n=e.getMatrix();n||(n=[1,0,0,0,1,0,0,0,1]);var r=e.getCanvasBBox(),i=t.x-r.minX,o=t.y-r.minY,a=fe(n,[["t",i,o]]);e.setMatrix(a)},Ze=function(e,t){var n=e.getMatrix();n||(n=[1,0,0,0,1,0,0,0,1]);var r=t;y()(t)||(r=[t,t]),y()(t)&&1===t.length&&(r=[t[0],t[0]]),n=fe(n,[["s",r[0],r[1]]]),e.setMatrix(n)},Qe=function(e,t){var n=e.getMatrix();n||(n=[1,0,0,0,1,0,0,0,1]),n=fe(n,[["r",t]]),e.setMatrix(n)},Ke=function(e,t,n){for(var r=[],i=0;i<e;i++)r[i]=0;return n.forEach((function(e){e.source&&(r[t[e.source]]+=1),e.target&&(r[t[e.target]]+=1)})),r};function Je(e,t,n){return(n[0]-e[0])*(t[1]-e[1])==(t[0]-e[0])*(n[1]-e[1])&&Math.min(e[0],t[0])<=n[0]&&n[0]<=Math.max(e[0],t[0])&&Math.min(e[1],t[1])<=n[1]&&n[1]<=Math.max(e[1],t[1])}var $e=function(e,t,n){var r=!1,i=e.length;function o(e){return Math.abs(e)<1e-6?0:e<0?-1:1}if(i<=2)return!1;for(var a=0;a<i;a++){var s=e[a],l=e[(a+1)%i];if(Je(s,l,[t,n]))return!0;o(s[1]-n)>0!=o(l[1]-n)>0&&o(t-(n-s[1])*(s[0]-l[0])/(s[1]-l[1])-s[0])<0&&(r=!r)}return r},et=function(e,t){return!(t.minX>e.maxX||t.maxX<e.minX||t.minY>e.maxY||t.maxY<e.minY)},tt=function(e,t){var n=function(e){var t=e.map((function(e){return e[0]})),n=e.map((function(e){return e[1]}));return{minX:Math.min.apply(null,t),maxX:Math.max.apply(null,t),minY:Math.min.apply(null,n),maxY:Math.max.apply(null,n)}},r=function(e){for(var t=[],n=e.length,r=0;r<n-1;r++){var i=e[r],o=e[r+1];t.push({from:{x:i[0],y:i[1]},to:{x:o[0],y:o[1]}})}if(t.length>1){var a=e[0],s=e[n-1];t.push({from:{x:s[0],y:s[1]},to:{x:a[0],y:a[1]}})}return t};if(e.length<2||t.length<2)return!1;var i=n(e),o=n(t);if(!et(i,o))return!1;var a=!1;if(Object(ce.each)(t,(function(t){if($e(e,t[0],t[1]))return a=!0,!1})),a)return!0;if(Object(ce.each)(e,(function(e){if($e(t,e[0],e[1]))return a=!0,!1})),a)return!0;var s=r(e),l=r(t),u=!1;return Object(ce.each)(l,(function(e){if(function(e,t){var n=!1;return Object(ce.each)(e,(function(e){if(je(e.from,e.to,t.from,t.to))return n=!0,!1})),n}(s,e))return u=!0,!1})),u},nt=function(){function e(e,t,n,r){this.x1=e,this.y1=t,this.x2=n,this.y2=r}return e.prototype.getBBox=function(){var e=Math.min(this.x1,this.x2),t=Math.min(this.y1,this.y2),n=Math.max(this.x1,this.x2),r=Math.max(this.y1,this.y2);return{x:e,y:t,minX:e,minY:t,maxX:n,maxY:r,width:n-e,height:r-t}},e}(),rt=function(e,t){return{top:[e.minX,e.minY,e.maxX,e.minY],left:[e.minX,e.minY,e.minX,e.maxY],bottom:[e.minX,e.maxY,e.maxX,e.maxY],right:[e.maxX,e.minY,e.maxX,e.maxY]}[t]},it=function(e,t){var n=(t.x2-t.x1)*(e.y1-t.y1)-(t.y2-t.y1)*(e.x1-t.x1),r=(e.x2-e.x1)*(e.y1-t.y1)-(e.y2-e.y1)*(e.x1-t.x1),i=(t.y2-t.y1)*(e.x2-e.x1)-(t.x2-t.x1)*(e.y2-e.y1);if(i){var o=n/i,a=r/i;if(o>=0&&o<=1&&a>=0&&a<=1)return o}return Number.POSITIVE_INFINITY},ot=function(e,t){for(var n=["top","left","bottom","right"],r=e.getBBox(),i=0,o=[],a=0;a<4;a++){var s=rt(r,n[a]),l=s[0],u=s[1],c=s[2],d=s[3];o[a]=je({x:t.x1,y:t.y1},{x:t.x2,y:t.y2},{x:l,y:u},{x:c,y:d}),o[a]&&(i+=1)}return[o,i]},at=function(e,t){for(var n=["top","left","bottom","right"],r=e.getBBox(),i=Number.POSITIVE_INFINITY,o=0,a=0;a<4;a++){var s=rt(r,n[a]),l=s[0],u=s[1],c=s[2],d=s[3],h=it(t,new nt(l,u,c,d));(h=Math.abs(h-.5))>=0&&h<=1&&(o+=1,i=h<i?h:i)}return 0===o?-1:i},st=function(e){var t=0,n=0;if(e.length>0){for(var r=0,i=e;r<i.length;r++){var o=i[r];t+=o.x,n+=o.y}t/=e.length,n/=e.length}return{x:t,y:n}},lt=function(e,t){return Math.pow(e.x-t.x,2)+Math.pow(e.y-t.y,2)},ut=function(e,t){var n,r=t.x1,i=t.y1,o=t.x2-r,a=t.y2-i,s=e.x-r,l=e.y-i,u=s*o+l*a;n=u<=0||(u=(s=o-s)*o+(l=a-l)*a)<=0?0:u*u/(o*o+a*a);var c=s*s+l*l-n;return c<0&&(c=0),c},ct=function(e,t,n){return void 0===n&&(n=.001),Math.pow(e.x-t.x,2)+Math.pow(e.y-t.y,2)<Math.pow(n,2)},dt=function(e,t){var n=e.x<t.x,r=e.x>t.x+t.width,i=e.y>t.y+t.height,o=e.y<t.y;if(!(n||r||i||o))return 0;if(i&&!n&&!r)return Math.pow(t.y+t.height-e.y,2);if(o&&!n&&!r)return Math.pow(e.y-t.y,2);if(n&&!i&&!o)return Math.pow(t.x-e.x,2);if(r&&!i&&!o)return Math.pow(t.x+t.width-e.x,2);var a=Math.min(Math.abs(t.x-e.x),Math.abs(t.x+t.width-e.x)),s=Math.min(Math.abs(t.y-e.y),Math.abs(t.y+t.height-e.y));return a*a+s*s};function ht(e,t){for(var n=[],r=0,i=e.length;i-2*!t>r;r+=2){var o=[{x:+e[r-2],y:+e[r-1]},{x:+e[r],y:+e[r+1]},{x:+e[r+2],y:+e[r+3]},{x:+e[r+4],y:+e[r+5]}];t?r?i-4===r?o[3]={x:+e[0],y:+e[1]}:i-2===r&&(o[2]={x:+e[0],y:+e[1]},o[3]={x:+e[2],y:+e[3]}):o[0]={x:+e[i-2],y:+e[i-1]}:i-4===r?o[3]=o[2]:r||(o[0]={x:+e[r],y:+e[r+1]}),n.push(["C",(-o[0].x+6*o[1].x+o[2].x)/6,(-o[0].y+6*o[1].y+o[2].y)/6,(o[1].x+6*o[2].x-o[3].x)/6,(o[1].y+6*o[2].y-o[3].y)/6,o[2].x,o[2].y])}return n}n(95);var ft="\t\n\v\f\r   ᠎             　\u2028\u2029",pt=new RegExp("([a-z])["+ft+",]*((-?\\d*\\.?\\d*(?:e[\\-+]?\\d+)?["+ft+"]*,?["+ft+"]*)+)","ig"),gt=new RegExp("(-?\\d*\\.?\\d*(?:e[\\-+]?\\d+)?)["+ft+"]*,?["+ft+"]*","ig");n(67),n(68);var vt=function(e,t){return e&&t?e.replace(/\\?\{([^{}]+)\}/g,(function(e,n){if("\\"===e.charAt(0))return e.slice(1);var r=t[n];return 0===r&&(r="0"),r||""})):e},mt=function(e){var t=[];if(e.length<2)throw new Error("point length must largn than 2, now it's "+e.length);for(var n=0,r=e;n<r.length;n++){var i=r[n],o=i.x,a=i.y;t.push(o),t.push(a)}var s=ht(t);return s.unshift(["M",e[0].x,e[0].y]),s},yt=function(e,t,n,r){void 0===n&&(n=0),void 0===r&&(r=0);var i={x:(1-n)*e.x+n*t.x,y:(1-n)*e.y+n*t.y},o=[];de.normalize(o,[t.x-e.x,t.y-e.y]),0===o.length&&(o=[0,0]);var a=[-o[1]*r,o[0]*r];return i.x+=a[0],i.y+=a[1],i},bt=function(e,t){var n=e.length;if(!n)return"";for(var r="",i=0;i<n;i++){var o=e[i];r+=vt(0===i?"M{x} {y}":"L{x} {y}",o)}return t&&(r+="Z"),r},Ct=function(e){var t=[];return e.forEach((function(e){if("A"!==e[0])for(var n=1;n<e.length;n+=2)t.push([e[n],e[n+1]]);else{var r=e.length;t.push([e[r-2],e[r-1]])}})),t},At=function(e){if(e.length<2)throw new Error("point length must largn than 2, now it's "+e.length);var t=e[0],n=e[1],r=e[e.length-1],i=e[e.length-2];e.unshift(r),e.unshift(i),e.push(t),e.push(n);for(var o=[],a=1;a<e.length-2;a+=1){var s=e[a-1].x,l=e[a-1].y,u=e[a].x,c=e[a].y,d=e[a+1].x,h=e[a+1].y,f=u+(d-s)/6,p=c+(h-l)/6,g=d-((a!==e.length-2?e[a+2].x:d)-u)/6,v=h-((a!==e.length-2?e[a+2].y:h)-c)/6;o.push(["C",f,p,g,v,d,h])}return o.unshift(["M",r.x,r.y]),o},_t=function(e,t){return de.scale([],de.normalize([],e),t)},It=function(e,t){var n=[e[1]-t[1],t[0]-e[0]],r=Math.sqrt(n[0]*n[0]+n[1]*n[1]);if(0===r)throw new Error("p0 should not be equal to p1");return[n[0]/r,n[1]/r]},xt=function(e,t){return[t[0]-e[0],t[1]-e[1]]};function wt(e,t){if(!e||e.length<1)return"";if(1===e.length)return function(e){var n=[e[0][0],e[0][1]-t],r=[e[0][0],e[0][1]+t];return"M "+n+" A "+t+","+t+",0,0,0,"+r+" A "+t+","+t+",0,0,0,"+n}(e);if(2===e.length)return function(e){var n=de.scale([],It(e[0],e[1]),t),r=de.scale([],n,-1),i=de.add([],e[0],n),o=de.add([],e[1],n),a=de.add([],e[1],r),s=de.add([],e[0],r);return"M "+i+" L "+o+" A "+[t,t,"0,0,0",a].join(",")+" L "+s+" A "+[t,t,"0,0,0",i].join(",")}(e);for(var n=new Array(e.length),r=0;r<n.length;++r){var i=0===r?e[e.length-1]:e[r-1],o=e[r],a=de.scale([],It(i,o),t);n[r]=[de.add([],i,a),de.add([],o,a)]}var s="A "+[t,t,"0,0,0,"].join(",");return(n=n.map((function(e,t){var r="";return 0===t&&(r="M "+n[n.length-1][1]+" "),r+(s+e[0]+" L ")+e[1]}))).join(" ")}function Mt(e,t){var n,r,i,o=e.length;if(!e||o<1)return"";if(1===o)return r=[(n=e)[0][0],n[0][1]-t],i=[n[0][0],n[0][1]+t],"M "+r+" A "+[t,t,"0,0,0",i].join(",")+" A "+[t,t,"0,0,0",r].join(",");if(2===o)return function(e){var n=xt(e[0],e[1]),r=_t(n,t),i=de.add([],e[0],de.scale([],r,-1)),o=de.add([],e[1],r),a=1.2*t,s=_t(de.normalize([],n),a),l=de.scale([],s,-1),u=de.add([],i,l),c=de.add([],o,l),d=de.add([],i,s);return"M "+i+" C "+[u,c,o].join(",")+" S "+[d,i].join(",")+" Z"}(e);for(var a=e.map((function(t,n){var r=e[(n+1)%o];return{p:t,v:de.normalize([],xt(t,r))}})),s=0;s<a.length;++s){var l=s>0?s-1:o-1,u=de.normalize([],de.add([],a[l].v,de.scale([],a[s].v,-1)));a[s].p=de.add([],a[s].p,de.scale([],u,t))}return a.map((function(e){var t=e.p;return{x:t[0],y:t[1]}}))}var St=["drag","shift","ctrl","alt","control"],Tt=["click","drag"],Et=["shift","ctrl","control","alt","meta",void 0],Ot=["shift","ctrl","alt","control"],kt={"drag-canvas":j,"zoom-canvas":pe,"drag-node":$,"activate-relations":{getDefaultCfg:function(){return{trigger:"mouseenter",activeState:"active",inactiveState:"inactive",resetSelected:!1,shouldUpdate:function(){return!0}}},getEvents:function(){return"mouseenter"===this.get("trigger")?{"node:mouseenter":"setAllItemStates","node:mouseleave":"clearActiveState"}:{"node:click":"setAllItemStates","canvas:click":"clearAllItemStates"}},setAllItemStates:function(e){var t=e.item,n=this.graph;if(this.item=t,this.shouldUpdate(e.item,{event:e,action:"activate"})){for(var r=this.activeState,i=this.inactiveState,o=n.getNodes(),a=n.getEdges(),s=o.length,l=a.length,u=0;u<s;u++){var c=o[u],d=c.hasState("selected");this.resetSelected&&d&&n.setItemState(c,"selected",!1),n.setItemState(c,r,!1),i&&n.setItemState(c,i,!0)}for(u=0;u<l;u++){var h=a[u];n.setItemState(h,r,!1),i&&n.setItemState(h,i,!0)}i&&n.setItemState(t,i,!1),n.setItemState(t,r,!0);var f=t.getEdges(),p=f.length;for(u=0;u<p;u++){var g;g=(h=f[u]).getSource()===t?h.getTarget():h.getSource(),i&&n.setItemState(g,i,!1),n.setItemState(g,r,!0),n.setItemState(h,i,!1),n.setItemState(h,r,!0),h.toFront()}n.emit("afteractivaterelations",{item:e.item,action:"activate"})}},clearActiveState:function(e){var t=this.get("graph");if(this.shouldUpdate(e.item,{event:e,action:"deactivate"})){var n=this.activeState,r=this.inactiveState,i=t.get("autoPaint");t.setAutoPaint(!1);for(var o=t.getNodes(),a=t.getEdges(),s=o.length,l=a.length,u=0;u<s;u++){var c=o[u];t.clearItemStates(c,[n,r])}for(u=0;u<l;u++){var d=a[u];t.clearItemStates(d,[n,r,"deactivate"])}t.paint(),t.setAutoPaint(i),t.emit("afteractivaterelations",{item:e.item||this.get("item"),action:"deactivate"})}},clearAllItemStates:function(e){var t=this.graph;if(this.shouldUpdate(e.item,{event:e,action:"deactivate"})){for(var n=this.activeState,r=this.inactiveState,i=t.getNodes(),o=t.getEdges(),a=i.length,s=o.length,l=0;l<a;l++){var u=i[l];t.clearItemStates(u,[n,r])}for(l=0;l<s;l++){var c=o[l];t.clearItemStates(c,[n,r,"deactivate"])}t.emit("afteractivaterelations",{item:e.item||this.get("item"),action:"deactivate"})}}},"brush-select":ie,"click-select":ae,"lasso-select":{getDefaultCfg:function(){return{delegateStyle:{fill:"#EEF6FF",fillOpacity:.4,stroke:"#DDEEFE",lineWidth:1},onSelect:function(){},onDeselect:function(){},selectedState:"selected",trigger:"shift",includeEdges:!0,selectedEdges:[],selectedNodes:[]}},getEvents:function(){return St.indexOf(this.trigger.toLowerCase())>-1||(this.trigger="shift",console.warn("Behavior lasso-select 的 trigger 参数不合法，请输入 'drag'、'shift'、'ctrl' 或 'alt'")),"drag"===this.trigger?{dragstart:"onDragStart",drag:"onDragMove",dragend:"onDragEnd","canvas:click":"clearStates"}:{dragstart:"onDragStart",drag:"onDragMove",dragend:"onDragEnd",keyup:"onKeyUp",keydown:"onKeyDown","canvas:click":"clearStates"}},onDragStart:function(e){var t=this.lasso;e.item||("drag"===this.trigger||this.keydown)&&(this.selectedNodes&&0!==this.selectedNodes.length&&this.clearStates(),t||(t=this.createLasso()),this.dragging=!0,this.originPoint={x:e.x,y:e.y},this.points.push(this.originPoint),t.show())},onDragMove:function(e){this.dragging&&("drag"===this.trigger||this.keydown)&&(this.points.push({x:e.x,y:e.y}),this.updateLasso(e))},onDragEnd:function(e){(this.lasso||this.dragging)&&("drag"===this.trigger||this.keydown)&&(this.points.push(this.originPoint),this.getSelectedItems(),this.lasso.remove(!0),this.lasso=null,this.points=[],this.dragging=!1)},getLassoPath:function(){var e=this.points,t=[];return e.length&&(e.forEach((function(e,n){0===n?t.push(["M",e.x,e.y]):t.push(["L",e.x,e.y])})),t.push(["L",e[0].x,e[0].y])),t},clearStates:function(){var e=this.graph,t=this.selectedState,n=e.findAllByState("node",t),r=e.findAllByState("edge",t);n.forEach((function(n){return e.setItemState(n,t,!1)})),r.forEach((function(n){return e.setItemState(n,t,!1)})),this.onDeselect&&this.onDeselect(this.selectedNodes,this.selectedEdges),this.selectedNodes=[],this.selectedEdges=[],e.emit("nodeselectchange",{selectedItems:{nodes:[],edges:[]},select:!1})},getSelectedItems:function(){var e=this,t=this.graph,n=this.shouldUpdate,r=this.points.map((function(e){return[t.getCanvasByPoint(e.x,e.y).x,t.getCanvasByPoint(e.x,e.y).y]})),i=this.selectedState,o=[],a=[];t.getNodes().forEach((function(e){if(function(e,t){var n,r=e.getKeyShape();if("path"===e.get("type"))n=Ct(r.attr("path"));else{var i=r.getCanvasBBox();n=[[i.minX,i.minY],[i.maxX,i.minY],[i.maxX,i.maxY],[i.minX,i.maxY]]}return tt(t,n)}(e,r)&&n(e,"select")){o.push(e);var s=e.getModel();a.push(s.id),t.setItemState(e,i,!0)}}));var s=[];this.includeEdges&&o.forEach((function(r){r.getOutEdges().forEach((function(r){var i=r.getModel(),o=i.source,l=i.target;a.includes(o)&&a.includes(l)&&n(r,"select")&&(s.push(r),t.setItemState(r,e.selectedState,!0))}))})),this.selectedEdges=s,this.selectedNodes=o,this.onSelect&&this.onSelect(o,s),t.emit("nodeselectchange",{selectedItems:{nodes:o,edges:s},select:!0})},createLasso:function(){var e=this.graph.get("delegateGroup").addShape("path",{attrs:Object(w.__assign)({path:[]},this.delegateStyle),capture:!1,name:"lasso-shape"});return this.lasso=e,this.points=[],e},updateLasso:function(e){this.lasso.attr({path:this.getLassoPath()})},onKeyDown:function(e){var t=e.key;t&&(t.toLowerCase()===this.trigger.toLowerCase()?this.keydown=!0:this.keydown=!1)},onKeyUp:function(){this.lasso&&(this.lasso.remove(!0),this.lasso=null,this.points=[],this.dragging=!1),this.keydown=!1}},tooltip:Ce,"edge-tooltip":Ae,"drag-group":xe,"drag-node-with-group":Me,"collapse-expand-group":Te,"collapse-expand":Oe,"drag-combo":Le,"collapse-expand-combo":Pe,"create-edge":{getDefaultCfg:function(){return{trigger:"click",key:void 0,edgeConfig:{}}},getEvents:function(){var e;return Tt.indexOf(this.trigger.toLowerCase())>-1||(this.trigger="click",console.warn("Behavior create-edge 的 trigger 参数不合法，请输入 'click'，'drag'")),this.key&&-1===Et.indexOf(this.key.toLowerCase())&&(this.trigger=void 0,console.warn("Behavior create-edge 的 key 参数不合法，请输入 'shift'，'ctrl'，'alt'，'control'，或 undefined")),"drag"===this.trigger?e={"node:dragstart":"onClick","combo:dragstart":"onClick",drag:"updateEndPoint","node:drop":"onClick","combo:drop":"onClick",dragend:"onDragEnd"}:"click"===this.trigger&&(e={"node:click":"onClick",mousemove:"updateEndPoint","edge:click":"cancelCreating","canvas:click":"cancelCreating","combo:click":"onClick"}),this.key&&(e.keydown="onKeyDown",e.keyup="onKeyUp"),e},onDragEnd:function(e){if(!this.key||this.keydown){var t=e.item;t&&t.getID()!==this.source&&"node"===t.getType()||this.cancelCreating({item:this.edge,x:e.x,y:e.y})}},onClick:function(e){if(!this.key||this.keydown){var t=e.item,n=this.graph,r=t.getModel();if(this.addingEdge&&this.edge){if(!this.shouldEnd.call(this,e))return;var i={target:r.id};this.source===r.id&&(i.type="loop"),n.emit("beforecreateedge",{}),n.updateItem(this.edge,i),n.emit("aftercreateedge",{edge:this.edge}),this.edge.getKeyShape().set("capture",!0),this.edge=null,this.addingEdge=!1}else{if(!this.shouldBegin.call(this,e))return;this.edge=n.addItem("edge",Object(w.__assign)({source:r.id,target:r.id},this.edgeConfig),!1),this.source=r.id,this.addingEdge=!0,this.edge.getKeyShape().set("capture",!1)}}},updateEndPoint:function(e){if(!this.key||this.keydown){var t={x:e.x,y:e.y};this.graph.findById(this.source)?this.addingEdge&&this.edge&&this.graph.updateItem(this.edge,{target:t},!1):this.addingEdge=!1}},cancelCreating:function(e){if(!this.key||this.keydown){var t=this.graph,n=e.item;if(this.addingEdge&&e.target&&e.target.isCanvas&&e.target.isCanvas())return t.removeItem(this.edge,!1),this.edge=null,void(this.addingEdge=!1);if(this.addingEdge&&this.edge===n){var r=!0;if(!t.get("groupByTypes"))for(var i=e.x,o=e.y,a=t.getNodes(),s=a.length,l=0;l<s;l++){var u=a[l],c=u.getModel(),d=u.getBBox();if(i<=d.maxX&&i>=d.minX&&o<=d.maxY&&o>=d.minY){if(!this.shouldEnd.call(this,{x:e.x,y:e.y,canvasX:e.canvasX,canvasY:e.canvasY,clientX:e.clientX,clientY:e.clientY,item:u}))return;t.emit("beforecreateedge",{}),t.updateItem(this.edge,{target:c.id}),t.emit("aftercreateedge",{edge:this.edge}),r=!1;break}}r&&t.removeItem(this.edge,!1),this.edge=null,this.addingEdge=!1}}},onKeyDown:function(e){var t=e.key;t&&(t.toLowerCase()===this.key.toLowerCase()?this.keydown=!0:this.keydown=!1)},onKeyUp:function(){this.addingEdge&&this.edge&&(this.graph.removeItem(this.edge,!1),this.addingEdge=!1,this.edge=null),this.keydown=!1}},"shortcuts-call":{getDefaultCfg:function(){return{trigger:"ctrl",combinedKey:"1",functionName:"fitView",functionParams:[]}},getEvents:function(){return Ot.indexOf(this.trigger.toLowerCase())>-1||(this.trigger="ctrl",console.warn("Behavior shortcuts-fit-view 的 trigger 参数 '"+this.trigger+"' 不合法，请输入 'drag'、'shift'、'ctrl' 或 'alt'")),this.combinedKey===this.trigger&&(this.combinedKey=void 0),{keyup:"onKeyUp",keydown:"onKeyDown"}},onKeyDown:function(e){var t=e.key;if(t){var n=this.trigger.toLowerCase(),r=t.toLowerCase();this.triggerKeydown||(this.triggerKeydown=r===n||"control"===r&&"ctrl"===n||"ctrl"===r&&"control"===n);var i=this.graph;if(!i[this.functionName])return console.warn("Behavior shortcuts-fit-view 的 functionName 参数 '"+this.functionName+"' 不合法，它不是 Graph 的一个函数名"),{};if(!this.triggerKeydown||this.combinedKey){var o=this.combinedKey.toLowerCase();this.triggerKeydown&&(r===o||"control"===r&&"ctrl"===o||"ctrl"===r&&"control"===o)&&(this.functionParams&&this.functionParams.length?i[this.functionName].apply(i,this.functionParams):i[this.functionName]())}else this.functionParams&&this.functionParams.length?i[this.functionName].apply(i,this.functionParams):i[this.functionName]()}},onKeyUp:function(){this.brush&&(this.brush.remove(!0),this.brush=null,this.dragging=!1),this.triggerKeydown=!1}}};f()(kt,(function(e,t){v.registerBehavior(t,e)}));var Lt=v,Dt=n(99),Pt=n(36),Nt=n.n(Pt),jt=n(42),Rt=n.n(jt),zt=n(22),Ft=n(69),Bt=n.n(Ft),Ht=function(){function e(e){this.graph=e,this.destroyed=!1,this.modes=e.get("modes")||{default:[]},this.formatModes(),this.mode=e.get("defaultMode")||"default",this.currentBehaves=[],this.setMode(this.mode)}return e.prototype.formatModes=function(){var e=this.modes;f()(e,(function(e){f()(e,(function(t,n){x()(t)&&(e[n]={type:t})}))}))},e.prototype.setBehaviors=function(e){var t,n=this.graph,r=this.modes[e],i=[];f()(r||[],(function(e){var r=v.getBehavior(e.type);r&&(t=new r(e))&&(t.bind(n),i.push(t))})),this.currentBehaves=i},e.mergeBehaviors=function(e,t){return f()(t,(function(t){e.indexOf(t)<0&&(x()(t)&&(t={type:t}),e.push(t))})),e},e.filterBehaviors=function(e,t){var n=[];return e.forEach((function(e){var r;r=x()(e)?e:e.type,t.indexOf(r)<0&&n.push(e)})),n},e.prototype.setMode=function(e){var t=this.modes,n=this.graph,r=e;t[r]&&(n.emit("beforemodechange",{mode:e}),f()(this.currentBehaves,(function(e){e.unbind(n)})),this.setBehaviors(r),n.emit("aftermodechange",{mode:e}),this.mode=e)},e.prototype.getMode=function(){return this.mode},e.prototype.manipulateBehaviors=function(t,n,r){var i,o=this;if(i=y()(t)?t:[t],y()(n))return f()(n,(function(t){o.modes[t]?o.modes[t]=r?e.mergeBehaviors(o.modes[t]||[],i):e.filterBehaviors(o.modes[t]||[],i):r&&(o.modes[t]=i)})),this;var a=n;return n||(a=this.mode),this.modes[a]||r&&(this.modes[a]=i),this.modes[a]=r?e.mergeBehaviors(this.modes[a]||[],i):e.filterBehaviors(this.modes[a]||[],i),this.setMode(this.mode),this},e.prototype.destroy=function(){this.graph=null,this.modes=null,this.currentBehaves=null,this.destroyed=!0},e}(),Ut=function(){function e(e){this.destroyed=!1,this.graph=e,this.destroyed=!1}return e.prototype.getViewCenter=function(){var e=this.getFormatPadding(),t=this.graph,n=this.graph.get("width"),r=t.get("height");return{x:(n-e[1]-e[3])/2+e[3],y:(r-e[0]-e[2])/2+e[0]}},e.prototype.fitCenter=function(){var e=this.graph,t=e.get("group");t.resetMatrix();var n=t.getCanvasBBox();if(0!==n.width&&0!==n.height){var r=this.getViewCenter(),i=n.x+n.width/2,o=n.y+n.height/2;e.translate(r.x-i,r.y-o)}},e.prototype.fitView=function(){var e=this.graph,t=this.getFormatPadding(),n=e.get("width"),r=e.get("height"),i=e.get("group");i.resetMatrix();var o=i.getCanvasBBox();if(0!==o.width&&0!==o.height){var a=this.getViewCenter(),s=o.x+o.width/2,l=o.y+o.height/2;e.translate(a.x-s,a.y-l);var u=(n-t[1]-t[3])/o.width,c=(r-t[0]-t[2])/o.height,d=u;u>c&&(d=c),e.zoom(d,a)}},e.prototype.getFormatPadding=function(){var e=this.graph.get("fitViewPadding");return T(e)},e.prototype.focusPoint=function(e,t,n){var r=this,i=this.getViewCenter(),o=this.getPointByCanvas(i.x,i.y),a=this.graph.get("group").getMatrix();if(a||(a=[1,0,0,0,1,0,0,0,1]),t){var s=(o.x-e.x)*a[0],l=(o.y-e.y)*a[4],u=0,c=0,d=0,h=0;this.graph.get("canvas").animate((function(e){d=s*e,h=l*e,r.graph.translate(d-u,h-c),u=d,c=h}),Object(w.__assign)({},n))}else this.graph.translate((o.x-e.x)*a[0],(o.y-e.y)*a[4])},e.prototype.getPointByCanvas=function(e,t){var n=this.graph.get("group").getMatrix();return n||(n=[1,0,0,0,1,0,0,0,1]),He({x:e,y:t},n)},e.prototype.getPointByClient=function(e,t){var n=this.graph.get("canvas").getPointByClient(e,t);return this.getPointByCanvas(n.x,n.y)},e.prototype.getClientByPoint=function(e,t){var n=this.graph.get("canvas"),r=this.getCanvasByPoint(e,t),i=n.getClientByPoint(r.x,r.y);return{x:i.x,y:i.y}},e.prototype.getCanvasByPoint=function(e,t){var n=this.graph.get("group").getMatrix();return n||(n=[1,0,0,0,1,0,0,0,1]),Be({x:e,y:t},n)},e.prototype.focus=function(e,t,n){x()(e)&&(e=this.graph.findById(e));var r=e.get("group").getMatrix();r||(r=[1,0,0,0,1,0,0,0,1]),e&&this.focusPoint({x:r[6],y:r[7]},t,n)},e.prototype.changeSize=function(e,t){var n=this.graph;if(!_()(e)||!_()(t))throw Error("invalid canvas width & height, please make sure width & height type is number");n.set({width:e,height:t}),n.get("canvas").changeSize(e,t),n.get("plugins").forEach((function(r){if(r.get("gridContainer")){var i=n.get("minZoom");ve()(r.get("container"),{width:e+"px",height:t+"px"}),ve()(r.get("gridContainer"),{width:e/i+"px",height:t/i+"px",left:0,top:0})}}))},e.prototype.destroy=function(){this.graph=null,this.destroyed=!1},e}(),Vt=n(57),Yt=n.n(Vt),Wt=n(50),Gt=n.n(Wt),Xt=function(){function e(e){this.preItem=null,this.graph=e,this.extendEvents=[],this.dragging=!1,this.destroyed=!1,this.initEvents()}return e.prototype.initEvents=function(){var e=this.graph,t=this.extendEvents,n=e.get("canvas"),r=n.get("el"),i=Gt()(this,"onCanvasEvents"),o=Gt()(this,"onExtendEvents"),a=Gt()(this,"onWheelEvent");n.on("*",i),this.canvasHandler=i,t.push(Yt()(r,"DOMMouseScroll",a)),t.push(Yt()(r,"mousewheel",a)),"undefined"!=typeof window&&(t.push(Yt()(window,"keydown",o)),t.push(Yt()(window,"keyup",o)),t.push(Yt()(window,"focus",o)))},e.getItemRoot=function(e){for(;e&&!e.get("item");)e=e.get("parent");return e},e.prototype.onCanvasEvents=function(t){var n=this.graph,r=n.get("canvas"),i=t.target,o=t.type;t.canvasX=t.x,t.canvasY=t.y;var a={x:t.canvasX,y:t.canvasY},s=n.get("group").getMatrix();if(s||(s=[1,0,0,0,1,0,0,0,1]),O(s)&&(a=n.getPointByClient(t.clientX,t.clientY)),t.x=a.x,t.y=a.y,t.currentTarget=n,i===r)return"mousemove"===o&&this.handleMouseMove(t,"canvas"),t.target=r,t.item=null,n.emit(o,t),void n.emit("canvas:"+o,t);var l=e.getItemRoot(i);if(l){var u=l.get("item");if(!u.destroyed){var c=u.getType();if(t.target=i,t.item=u,t.canvasX===t.x&&t.canvasY===t.y){var d=n.getCanvasByPoint(t.x,t.y);t.canvasX=d.x,t.canvasY=d.y}n.emit(o,t),t.name&&!t.name.includes(":")?n.emit(c+":"+o,t):n.emit(t.name,t),"dragstart"===o&&(this.dragging=!0),"dragend"===o&&(this.dragging=!1),"mousemove"===o&&this.handleMouseMove(t,c)}}else n.emit(o,t)},e.prototype.onExtendEvents=function(e){this.graph.emit(e.type,e)},e.prototype.onWheelEvent=function(e){C()(e.wheelDelta)&&(e.wheelDelta=-e.detail),this.graph.emit("wheel",e)},e.prototype.handleMouseMove=function(e,t){var n=this.graph,r=this.preItem,i=n.get("canvas"),o=e.target===i?null:e.item;e=E(e),r&&r!==o&&!r.destroyed&&(e.item=r,this.emitCustomEvent(r.getType(),"mouseleave",e),this.dragging&&this.emitCustomEvent(r.getType(),"dragleave",e)),o&&r!==o&&(e.item=o,this.emitCustomEvent(t,"mouseenter",e),this.dragging&&this.emitCustomEvent(t,"dragenter",e)),this.preItem=o},e.prototype.emitCustomEvent=function(e,t,n){n.type=t,this.graph.emit(e+":"+t,n)},e.prototype.destroy=function(){var e=this.graph,t=this.canvasHandler,n=this.extendEvents;e.get("canvas").off("*",t),f()(n,(function(e){e.remove()})),this.dragging=!1,this.preItem=null,this.extendEvents.length=0,this.canvasHandler=null,this.destroyed=!0},e}(),qt=n(98),Zt=n.n(qt),Qt=n(35),Kt=n.n(Qt),Jt=n(49),$t=n.n(Jt),en=n(130),tn=n.n(en),nn={" ":.3329986572265625,a:.5589996337890625,A:.6569992065429687,b:.58599853515625,B:.6769989013671875,c:.5469985961914062,C:.7279998779296875,d:.58599853515625,D:.705999755859375,e:.554998779296875,E:.63699951171875,f:.37299957275390627,F:.5769989013671875,g:.5909988403320312,G:.7479995727539063,h:.555999755859375,H:.7199996948242188,i:.255999755859375,I:.23699951171875,j:.26699981689453123,J:.5169998168945312,k:.5289993286132812,K:.6899993896484375,l:.23499908447265624,L:.5879989624023437,m:.854998779296875,M:.8819992065429687,n:.5589996337890625,N:.7189987182617188,o:.58599853515625,O:.7669998168945312,p:.58599853515625,P:.6419998168945312,q:.58599853515625,Q:.7669998168945312,r:.3649993896484375,R:.6759994506835938,s:.504998779296875,S:.6319992065429687,t:.354998779296875,T:.6189987182617187,u:.5599990844726562,U:.7139999389648437,v:.48199920654296874,V:.6389999389648438,w:.754998779296875,W:.929998779296875,x:.5089996337890625,X:.63699951171875,y:.4959991455078125,Y:.66199951171875,z:.48699951171875,Z:.6239990234375,0:.6,1:.40099945068359377,2:.6,3:.6,4:.6,5:.6,6:.6,7:.5469985961914062,8:.6,9:.6,"[":.3329986572265625,"]":.3329986572265625,",":.26399993896484375,".":.26399993896484375,";":.26399993896484375,":":.26399993896484375,"{":.3329986572265625,"}":.3329986572265625,"\\":.5,"|":.19499969482421875,"=":.604998779296875,"+":.604998779296875,"-":.604998779296875,_:.5,"`":.3329986572265625," ~":.8329986572265625,"!":.3329986572265625,"@":.8579986572265625,"#":.6,$:.6,"%":.9699996948242188,"^":.517999267578125,"&":.7259994506835937,"*":.505999755859375,"(":.3329986572265625,")":.3329986572265625,"<":.604998779296875,">":.604998779296875,"/":.5,"?":.53699951171875},rn=Math.PI,on=Math.sin,an=Math.cos,sn=on(rn/8),ln=an(rn/8),un=function(e,t){var n=e.getBBox(),r={x:n.minX,y:n.minY},i={x:n.maxX,y:n.maxY};if(t){var o=t.getMatrix();o||(o=[1,0,0,0,1,0,0,0,1]),r=Be(r,o),i=Be(i,o)}var a=r.x,s=r.y,l=i.x,u=i.y;return{x:a,y:s,minX:a,minY:s,maxX:l,maxY:u,width:l-a,height:u-s}},cn=function(e){var t=e.sourceNode||e.targetNode,n=t.get("group").getMatrix();n||(n=[1,0,0,0,1,0,0,0,1]);var r=t.getKeyShape().getBBox(),i=e.loopCfg||{},o=i.dist||2*Math.max(r.width,r.height),a=i.position||J.defaultLoopPosition,s=[n[6],n[7]],l=[e.startPoint.x,e.startPoint.y],u=[e.endPoint.x,e.endPoint.y],c=r.height/2,d=r.height/2,h=c*sn,f=c*ln,p=d*sn,g=d*ln;if(l[0]===u[0]&&l[1]===u[1]){switch(a){case"top":l=[s[0]-h,s[1]-f],u=[s[0]+p,s[1]-g];break;case"top-right":c=r.height/2,d=r.width/2,l=[s[0]+(h=c*sn),s[1]-(f=c*ln)],u=[s[0]+(g=d*ln),s[1]-(p=d*sn)];break;case"right":c=r.width/2,d=r.width/2,l=[s[0]+(f=c*ln),s[1]-(h=c*sn)],u=[s[0]+(g=d*ln),s[1]+(p=d*sn)];break;case"bottom-right":c=r.width/2,d=r.height/2,l=[s[0]+(f=c*ln),s[1]+(h=c*sn)],u=[s[0]+(p=d*sn),s[1]+(g=d*ln)];break;case"bottom":c=r.height/2,d=r.height/2,l=[s[0]+(h=c*sn),s[1]+(f=c*ln)],u=[s[0]-(p=d*sn),s[1]+(g=d*ln)];break;case"bottom-left":c=r.height/2,d=r.width/2,l=[s[0]-(h=c*sn),s[1]+(f=c*ln)],u=[s[0]-(g=d*ln),s[1]+(p=d*sn)];break;case"left":c=r.width/2,d=r.width/2,l=[s[0]-(f=c*ln),s[1]+(h=c*sn)],u=[s[0]-(g=d*ln),s[1]-(p=d*sn)];break;case"top-left":c=r.width/2,d=r.height/2,l=[s[0]-(f=c*ln),s[1]-(h=c*sn)],u=[s[0]-(p=d*sn),s[1]-(g=d*ln)];break;default:c=r.width/2,d=r.width/2,l=[s[0]-(h=c*sn),s[1]-(f=c*ln)],u=[s[0]+(p=d*sn),s[1]-(g=d*ln)]}if(!1===i.clockwise){var v=[l[0],l[1]];l=[u[0],u[1]],u=[v[0],v[1]]}}var m=[l[0]-s[0],l[1]-s[1]],y=(c+o)/c,b=(d+o)/d;!1===i.clockwise&&(y=(d+o)/d,b=(c+o)/c);var C=de.scale([],m,y),A=[s[0]+C[0],s[1]+C[1]],_=[u[0]-s[0],u[1]-s[1]],I=de.scale([],_,b),x=[s[0]+I[0],s[1]+I[1]];return e.startPoint={x:l[0],y:l[1]},e.endPoint={x:u[0],y:u[1]},e.controlPoints=[{x:A[0],y:A[1]},{x:x[0],y:x[1]}],e},dn=function(e,t,n,r,i){var o=[],a=e.getPoint(t);if(null===a)return{x:0,y:0,angle:0};if(t<1e-4)o=e.getStartTangent().reverse();else if(t>.9999)o=e.getEndTangent();else{var s=e.getPoint(t+1e-4);o.push([a.x,a.y]),o.push([s.x,s.y])}var l=Math.atan2(o[1][1]-o[0][1],o[1][0]-o[0][0]);if(l<0&&(l+=2*rn),n&&(a.x+=an(l)*n,a.y+=on(l)*n),r){var u=l-rn/2;l>.5*rn&&l<1.5*rn&&(u-=rn),a.x+=an(u)*r,a.y+=on(u)*r}var c={x:a.x,y:a.y,angle:l};return i?(l>.5*rn&&l<1.5*rn&&(l-=rn),Object(w.__assign)({rotate:l},c)):c},hn=function(e,t){if(!1===t(e))return!1;if(e&&e.children)for(var n=e.children.length-1;n>=0;n--)if(!hn(e.children[n],t))return!1;return!0},fn=function(e,t){if(e&&e.children)for(var n=e.children.length-1;n>=0;n--)if(!fn(e.children[n],t))return;return!1!==t(e)},pn=function(e,t){"function"==typeof t&&hn(e,t)},gn=function(e,t){"function"==typeof t&&fn(e,t)},vn=function(e,t){var n={x:1/0,y:1/0},r={x:-1/0,y:-1/0},i="x",o="y";t&&["V","TB","BT"].indexOf(t)>=0&&(o="x",i="y");var a=0;pn(e,(function(e){return a++,e.x>r.x&&(r.x=e.x),e.x<n.x&&(n.x=e.x),e.y>r.y&&(r.y=e.y),e.y<n.y&&(n.y=e.y),!0}));var s=2*rn/a,l=r[o]-n[o];return 0===l||pn(e,(function(t){var r=(t[o]-n[o])/l*(2*rn-s)+s,a=Math.abs("x"===i?t.x-e.x:t.y-e.y);return t.x=a*Math.cos(r),t.y=a*Math.sin(r),!0})),e},mn=function(e,t){return t*(nn[e]||1)},yn=function(e,t){var n=0,r=new RegExp("[一-龥]+");return e.split("").forEach((function(e){r.test(e)?n+=t:n+=mn(e,t)})),[n,t]},bn=function(e,t){var n=[],r={},i={};e.forEach((function(e){i[e.id]=e})),e.forEach((function(e,t){var o=Object(ce.clone)(e);o.itemType="combo",o.children=void 0,o.parentId===o.id?(console.warn("The parentId for combo "+o.id+" can not be the same as the combo's id"),delete o.parentId):o.parentId&&!i[o.parentId]&&(console.warn("The parent combo for combo "+o.id+" does not exist!"),delete o.parentId);var a=r[o.id];if(a){if(o.children=a.children,r[o.id]=o,!(a=o).parentId)return void n.push(a);var s=r[a.parentId];if(s)s.children?s.children.push(o):s.children=[o];else{var l={id:a.parentId,children:[a]};r[a.parentId]=l,r[o.id]=o}}else if(Object(ce.isString)(e.parentId))if(l=r[e.parentId])l.children?l.children.push(o):l.children=[o],r[o.id]=o;else{var u={id:e.parentId,children:[o]};r[u.id]=u,r[o.id]=o}else n.push(o),r[o.id]=o}));var o={};return(t||[]).forEach((function(e){o[e.id]=e;var t=r[e.comboId];if(t){var n={id:e.id,comboId:e.comboId};t.children?t.children.push(n):t.children=[n],n.itemType="node",r[e.id]=n}})),n.forEach((function(e){e.depth=0,hn(e,(function(e){var t,n=r[e.id].itemType;t="node"===n?r[e.comboId]:r[e.parentId],e.depth=t?"node"===n?t.depth+1:t.depth+2:0;var i=o[e.id];return i&&(i.depth=e.depth),!0}))})),n},Cn=function(e,t,n){var r,i=e,o={root:{children:e}},a=!1,s="root";(e||[]).forEach((function(e){if(!a)return e.id===t?(r=e,"combo"===e.itemType?r.parentId=n:r.comboId=n,void(a=!0)):void pn(e,(function(e){return o[e.id]={children:e.children},i=o[e.parentId||e.comboId||"root"].children,!e||!e.removed&&t!==e.id||!i||(s=e.parentId||e.comboId||"root",r=e,"combo"===e.itemType?r.parentId=n:r.comboId=n,a=!0,!1)}))}));var l=(i=o[s].children)?i.indexOf(r):-1;if(l>-1&&i.splice(l,1),a||(r={id:t,itemType:"node",comboId:n},o[t]={children:void 0}),t){var u=!1;if(n){var c=0;(e||[]).forEach((function(e){u||pn(e,(function(e){return n!==e.id||(u=!0,e.children?e.children.push(r):e.children=[r],c=e.depth,"node"===r.itemType?r.depth=c+2:r.depth=c+1,!1)}))}))}else n&&u||"node"===r.itemType||e.push(r);var d=r.depth;pn(r,(function(e){return"node"===e.itemType?d+=2:d+=1,e.depth=d,!0}))}return e},An=function(e,t){var n={minX:1/0,minY:1/0,maxX:-1/0,maxY:-1/0,x:void 0,y:void 0,width:void 0,height:void 0,centerX:void 0,centerY:void 0};return e&&0!==e.length?(e.forEach((function(e){var r=t.findById(e.id);if(r&&r.isVisible()){r.set("bboxCanvasCache",void 0);var i=r.getCanvasBBox();i.x&&n.minX>i.minX&&(n.minX=i.minX),i.y&&n.minY>i.minY&&(n.minY=i.minY),i.x&&n.maxX<i.maxX&&(n.maxX=i.maxX),i.y&&n.maxY<i.maxY&&(n.maxY=i.maxY)}})),n.x=(n.minX+n.maxX)/2,n.y=(n.minY+n.maxY)/2,n.width=n.maxX-n.minX,n.height=n.maxY-n.minY,n.centerX=(n.minX+n.maxX)/2,n.centerY=(n.minY+n.maxY)/2,Object.keys(n).forEach((function(e){n[e]!==1/0&&n[e]!==-1/0||(n[e]=void 0)})),n):n},_n=function(e){var t=e.group,n=e.height,r=e.width,i=e.x,o=e.y,a=t.get("canvas"),s=a.get("width"),l=a.get("height"),u={start:{x:0,y:0},end:{x:0,y:0}};return u.start.x=i/s,u.start.y=o/l,u.end.x=(i+r)/s,u.end.y=(o+n)/l,u};function In(e){if("string"!=typeof e)return e;var t=function(e){if("string"!=typeof e)return e;try{return JSON.parse(e.trim())}catch(t){return e.trim()}},n=t(e);if("string"!=typeof n)return n;for(var r=function(e){return e[e.length-1]},i=e.trim(),o=[],a=[],s=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return e.some((function(e){return r(a)===e}))},l=null,u=0,c="";u<i.length;){var d=i[u],h=s('"',"'");if(h||d.trim()){var f="\\"===i[u-1],p=s("}"),g=s("]"),v=s(","),m=r(o);if(h)if(r(a)!==d||f)c+=d;else{a.pop();var y=t(c);m.push(y),l=y,c=""}else if(g&&","===d)c&&(m.push(t(c)),c="");else if(p&&":"===d)a.push(","),c&&(m.push(c),c="");else if(v&&","===d)c&&(m.push(t(c)),c=""),a.pop();else if("}"===d&&(p||v)){c&&(m.push(t(c)),c=""),v&&a.pop();for(var b={},C=1;C<m.length;C+=2)b[m[C-1]]=m[C];o.pop(),o.length&&r(o).push(b),a.pop(),l=b}else"]"===d&&g?(c&&(m.push(t(c)),c=""),o.pop(),o.length&&r(o).push(m),a.pop(),l=m):"{"===d?(o.push([]),a.push("}")):"["===d?(o.push([]),a.push("]")):'"'===d?a.push('"'):"'"===d?a.push("'"):c+=d;u+=1}else u+=1}return l||c}function xn(e){var t={},n=function(t){var n=function(e){return function(t){for(var n=e.length,r=[],i=0,o="";i<n;)if("{"===e[i]&&"{"===e[i+1])r.push(o),o="",i+=2;else if("}"===e[i]&&"}"===e[i+1]){if(r.length){var a=r.pop();o=tn()(t,o,a.endsWith("=")?'"{'+o+'}"':o),r.push(a+o)}i+=2,o=""}else o+=e[i],i+=1;return r.push(o),r.map((function(e,t){return r[t-1]&&r[t-1].endsWith("=")?'"{'+e+'}"':e})).join("")}}("function"==typeof e?e(t):e)(t),r=document.createElement("div");r.innerHTML=n;var i=function e(t,n){var r;void 0===n&&(n={x:0,y:0});var i=Object(w.__assign)({x:0,y:0,width:0,height:0},n);if(null===(r=t.children)||void 0===r?void 0:r.length){var o=t.attrs,a=void 0===o?{}:o,s=a.marginTop,l=Object(w.__assign)({},n);s&&(l.y+=s);for(var u=0;u<t.children.length;u++){t.children[u].attrs.key=(a.key||"root")+" -"+u+" ";var c=e(t.children[u],l);if(c.bbox){var d=c.bbox;"inline"===c.attrs.next?l.x+=c.bbox.width:l.y+=c.bbox.height,d.width+d.x>i.width&&(i.width=d.width+d.x),d.height+d.y>i.height&&(i.height=d.height+d.y)}}}return t.bbox=function(e,t,n){var r,i,o=e.attrs,a=void 0===o?{}:o,s={x:t.x||0,y:t.y||0,width:n.width||0,height:n.height||0};switch(e.type){case"maker":case"circle":a.r&&(i=2*a.r,r=2*a.r);break;case"text":a.text&&(i=yn(a.text,a.fontSize||12)[0],r=16,s.y+=r,s.height=r,s.width=i,e.attrs=Object(w.__assign)({fontSize:12,fill:"#000"},a));break;default:a.width&&(i=a.width),a.height&&(r=a.height)}return r>=0&&(s.height=r),i>=0&&(s.width=i),a.marginTop&&(s.y+=a.marginTop),a.marginLeft&&(s.x+=a.marginLeft),s}(t,n,i),t.attrs=Object(w.__assign)(Object(w.__assign)({},t.attrs),t.bbox),t}(function e(t,n){var r={},i=t.getAttributeNames&&t.getAttributeNames()||[],o=t.children&&Array.from(t.children).map((function(t){return e(t,n)})),a={},s=t.tagName?t.tagName.toLowerCase():"group";return"text"===s&&(r.text=t.innerText),a.type=s,"img"===s&&(a.type="image"),Array.from(i).forEach((function(e){var n=e.split("-").reduce((function(e,t){return e+t.charAt(0).toUpperCase()+t.slice(1)})),i=t.getAttribute(e);try{if("style"===n||"attrs"===n){var o=In(i);r=Object(w.__assign)(Object(w.__assign)({},r),o)}else a[n]=In(i)}catch(e){if("style"===n)throw e;a[n]=i}})),a.attrs=r,n&&n.style&&a.name&&"object"==typeof n.style[a.name]&&(a.attrs=Object(w.__assign)(Object(w.__assign)({},a.attrs),n.style[a.name])),n&&n.style&&a.keyshape&&(a.attrs=Object(w.__assign)(Object(w.__assign)({},a.attrs),n.style)),o.length&&(a.children=o),a}(r.children[0],t));return r.remove(),i};return{draw:function(e,r){var i=n(e),o=r,a=function(e){var t=e.attrs,n=void 0===t?{}:t,i=e.bbox,s=e.type,l=e.children,u=Object(w.__rest)(e,["attrs","bbox","type","children"]);if("group"!==e.type){var c=r.addShape(e.type,Object(w.__assign)({attrs:n,origin:{bbox:i,type:s,children:l}},u));e.keyshape&&(o=c)}e.children&&e.children.forEach((function(e){return a(e)}))};return a(i),t[e.id]=[i],o},update:function(e,r){t[e.id]||(t[e.id]=[]);var i=r.getContainer(),o=i.get("children"),a=n(e),s=function e(t,n){var r,i,o,a,s=(t||{}).type,l=((null==n?void 0:n.attrs)||{}).key;if(l&&t&&(t.attrs.key=l),!t&&n)return{action:"delete",val:n,type:s,key:l};if(t&&!n)return{action:"add",val:t,type:s};if(!t&&!n)return{action:"same",type:s};var u=[];if((null===(r=t.children)||void 0===r?void 0:r.length)>0||(null===(i=n.children)||void 0===i?void 0:i.length)>0)for(var c=Math.max(null===(o=t.children)||void 0===o?void 0:o.length,null===(a=n.children)||void 0===a?void 0:a.length),d=n.children||[],h=t.children||[],f=0;f<c;f+=1)u.push(e(h[f],d[f]));var p=Object.keys(n.attrs),g=Object.keys(t.attrs);return n.type!==t.type?{action:"restructure",nowTarget:t,formerTarget:n,key:l,children:u}:p.filter((function(e){return"children"!==e})).some((function(e){return t.attrs[e]!==n.attrs[e]||!g.includes(e)}))?{action:"change",val:t,children:u,type:s,key:l}:{action:"same",children:u,type:s,key:l}}(a,t[e.id].pop()),l=function(e){var t;"group"!==e.type&&i.addShape(e.type,{attrs:e.attrs}),(null===(t=e.children)||void 0===t?void 0:t.length)&&e.children.map((function(e){return l(e)}))},u=function(e){var t,n=o.find((function(t){return t.attrs.key===e.attrs.key}));n&&i.removeChild(n),(null===(t=e.children)||void 0===t?void 0:t.length)&&e.children.map((function(e){return u(e)}))},c=function(e){var t=e.key;if("group"!==e.type){var n=o.find((function(e){return e.attrs.key===t}));switch(e.action){case"change":if(n){var i=e.val.keyshape?r.getOriginStyle():{};n.attr(Object(w.__assign)(Object(w.__assign)({},i),e.val.attrs))}break;case"add":l(e.val);break;case"delete":u(e.val);break;case"restructure":u(e.formerTarget),l(e.nowTarget)}}e.children&&e.children.forEach((function(e){return c(e)}))};c(s),t[e.id].push(a)},getAnchorPoints:function(){return[[0,.5],[1,.5],[.5,1],[.5,0]]}}}var wn={};function Mn(e){return wn[e]||(wn[e]=Object(ce.upperFirst)(e)),wn[e]}var Sn={defaultShapeType:"defaultType",className:null,getShape:function(e){return this[e]||this[this.defaultShapeType]},draw:function(e,t,n){var r=this.getShape(e),i=r.draw(t,n);return r.afterDraw&&r.afterDraw(t,n,i),i},baseUpdate:function(e,t,n){var r=this.getShape(e);r.update&&r.update(t,n),r.afterUpdate&&r.afterUpdate(t,n)},setState:function(e,t,n,r){this.getShape(e).setState(t,n,r)},shouldUpdate:function(e){return!!this.getShape(e).update},getControlPoints:function(e,t){return this.getShape(e).getControlPoints(t)},getAnchorPoints:function(e,t){return this.getShape(e).getAnchorPoints(t)}},Tn={options:{},draw:function(e,t){return this.drawShape(e,t)},drawShape:function(){},afterDraw:function(){},afterUpdate:function(){},setState:function(){},getControlPoints:function(e){return e.controlPoints},getAnchorPoints:function(e){var t=this.options.anchorPoints;return e.anchorPoints||t}},En=function(){function e(){}return e.registerFactory=function(t,n){var r=Mn(t),i=Sn,o=Object(w.__assign)(Object(w.__assign)({},i),n);return e[r]=o,o.className=r,o},e.getFactory=function(t){return e[Mn(t)]},e.registerNode=function(t,n,r){var i,o=e.Node;if("string"==typeof n||"function"==typeof n){var a=xn(n);i=Object(w.__assign)(Object(w.__assign)({},o.getShape("single-node")),a)}else if(n.jsx)a=xn(n.jsx),i=Object(w.__assign)(Object(w.__assign)(Object(w.__assign)({},o.getShape("single-node")),a),n);else{var s=r?o.getShape(r):Tn;i=Object(w.__assign)(Object(w.__assign)({},s),n)}return i.type=t,i.itemType="node",o[t]=i,i},e.registerEdge=function(t,n,r){var i=e.Edge,o=r?i.getShape(r):Tn,a=Object(w.__assign)(Object(w.__assign)({},o),n);return a.type=t,a.itemType="edge",i[t]=a,a},e.registerCombo=function(t,n,r){var i=e.Combo,o=r?i.getShape(r):Tn,a=Object(w.__assign)(Object(w.__assign)({},o),n);return a.type=t,a.itemType="combo",i[t]=a,a},e}(),On=En;En.registerFactory("node",{defaultShapeType:"circle"}),En.registerFactory("edge",{defaultShapeType:"line"}),En.registerFactory("combo",{defaultShapeType:"circle"});var kn=function(){function e(e){this._cfg={},this.destroyed=!1,this._cfg=Object.assign({id:void 0,type:"item",model:{},group:void 0,animate:!1,visible:!0,locked:!1,event:!0,keyShape:void 0,states:[]},this.getDefaultCfg(),e);var t=this.get("model"),n=t.id,r=this.get("type");n||(n=Object(ce.uniqueId)(r),this.get("model").id=n),this.set("id",n);var i=e.group;i&&(i.set("item",this),i.set("id",n)),this.init(),this.draw();var o=t.shape||t.type||("edge"===r?"line":"circle"),a=this.get("shapeFactory");if(a&&a[o]){var s=a[o].options;if(s&&s.stateStyles){var l=this.get("styles")||t.stateStyles;l=Object(ce.deepMix)({},s.stateStyles,l),this.set("styles",l)}}}return e.prototype.calculateBBox=function(){var e=this.get("keyShape"),t=this.get("group"),n=un(e,t);return n.x=n.minX,n.y=n.minY,n.width=n.maxX-n.minX,n.height=n.maxY-n.minY,n.centerX=(n.minX+n.maxX)/2,n.centerY=(n.minY+n.maxY)/2,n},e.prototype.calculateCanvasBBox=function(){var e=this.get("keyShape"),t=this.get("group"),n=un(e,t);return n.x=n.minX,n.y=n.minY,n.width=n.maxX-n.minX,n.height=n.maxY-n.minY,n.centerX=(n.minX+n.maxX)/2,n.centerY=(n.minY+n.maxY)/2,n},e.prototype.drawInner=function(){var e=this.get("shapeFactory"),t=this.get("group"),n=this.get("model");t.clear();var r=n.visible;if(void 0===r||r||this.changeVisibility(r),e){this.updatePosition(n);var i=this.getShapeCfg(n),o=i.shape||i.type,a=e.draw(o,i,t);a&&(this.set("keyShape",a),a.set("isKeyShape",!0),a.set("draggable",!0)),this.setOriginStyle(),this.set("currentShape",o),this.restoreStates(e,o)}},e.prototype.setOriginStyle=function(e){var t={},n=this.get("group").get("children"),r=this.getKeyShape(),i=this;Object(ce.each)(n,(function(e){var n=e.get("name");if(n)t[n]=i.getShapeStyleByName(n);else{var o=r.get("name"),a=i.getShapeStyleByName();o?t[o]=a:Object.assign(t,a)}}));var o,a=this.getOriginStyle();o=e?Object(ce.deepMix)({},a,t,e.style,{labelCfg:e.labelCfg}):Object(ce.deepMix)({},a,t),i.set("originStyle",o)},e.prototype.restoreStates=function(e,t){var n=this,r=n.get("states");Object(ce.each)(r,(function(r){e.setState(t,r,!0,n)}))},e.prototype.init=function(){var e=On.getFactory(this.get("type"));this.set("shapeFactory",e)},e.prototype.get=function(e){return this._cfg[e]},e.prototype.set=function(e,t){Object(ce.isPlainObject)(e)?this._cfg=Object(w.__assign)(Object(w.__assign)({},this._cfg),e):this._cfg[e]=t},e.prototype.getDefaultCfg=function(){return{}},e.prototype.clearCache=function(){this.set("bboxCache",null),this.set("bboxCanvasCache",null)},e.prototype.beforeDraw=function(){},e.prototype.afterDraw=function(){},e.prototype.afterUpdate=function(){},e.prototype.draw=function(){this.beforeDraw(),this.drawInner(),this.afterDraw()},e.prototype.getShapeStyleByName=function(e){var t=this.get("group"),n=this.getKeyShape();if(e&&(n=t.find((function(t){return t.get("name")===e}))),n){var r={};return Object(ce.each)(n.attr(),(function(e,t){"img"!==t&&(r[t]=e)})),r}return{}},e.prototype.getShapeCfg=function(e){var t=this.get("styles");if(t){var n=e;return n.style=Object(w.__assign)(Object(w.__assign)({},t),e.style),n}return e},e.prototype.getStateStyle=function(e){var t=this.get("styles");return t&&t[e]},e.prototype.getOriginStyle=function(){return this.get("originStyle")},e.prototype.getCurrentStatesStyle=function(){var e=this,t={};return Object(ce.each)(e.getStates(),(function(n){t=Object.assign(t,e.getStateStyle(n))})),t},e.prototype.setState=function(e,t){var n=this.get("states"),r=this.get("shapeFactory"),i=e,o=e;Object(ce.isString)(t)&&(i=e+":"+t,o=e+":");var a=n;if(Object(ce.isBoolean)(t)){var s=n.indexOf(o);if(t){if(s>-1)return;n.push(i)}else s>-1&&n.splice(s,1)}else if(Object(ce.isString)(t)){var l=n.filter((function(e){return e.includes(o)}));l.length>0&&this.clearStates(l),(a=a.filter((function(e){return!e.includes(o)}))).push(i),this.set("states",a)}if(r){var u=this.get("model"),c=u.shape||u.type;r.setState(c,e,t,this)}},e.prototype.clearStates=function(e){var t=this,n=t.getStates(),r=t.get("shapeFactory"),i=t.get("model"),o=i.shape||i.type;e||(e=n),Object(ce.isString)(e)&&(e=[e]);var a=n.filter((function(t){return-1===e.indexOf(t)}));t.set("states",a),e.forEach((function(e){r.setState(o,e,!1,t)}))},e.prototype.getContainer=function(){return this.get("group")},e.prototype.getKeyShape=function(){return this.get("keyShape")},e.prototype.getModel=function(){return this.get("model")},e.prototype.getType=function(){return this.get("type")},e.prototype.getID=function(){return this.get("id")},e.prototype.isItem=function(){return!0},e.prototype.getStates=function(){return this.get("states")},e.prototype.hasState=function(e){return this.getStates().indexOf(e)>=0},e.prototype.refresh=function(){var e=this.get("model");this.updatePosition(e),this.updateShape(),this.afterUpdate(),this.clearCache()},e.prototype.isOnlyMove=function(e){return!1},e.prototype.update=function(e){var t=this.get("model"),n=t.visible,r=e.visible;n!==r&&void 0!==r&&this.changeVisibility(r);var i=t.x,o=t.y;e.x=isNaN(e.x)?t.x:e.x,e.y=isNaN(e.y)?t.y:e.y;var a=this.get("styles");if(e.stateStyles){var s=e.stateStyles;Object(ce.mix)(a,s),delete e.stateStyles}Object.assign(t,e),this.isOnlyMove(e)?this.updatePosition(e):(i===e.x&&o===e.y||this.updatePosition(e),this.updateShape()),this.afterUpdate(),this.clearCache()},e.prototype.updateShape=function(){var e=this.get("shapeFactory"),t=this.get("model"),n=t.shape||t.type;if(e.shouldUpdate(n)&&n===this.get("currentShape")){var r=this.getShapeCfg(t);e.baseUpdate(n,r,this)}else this.draw();this.setOriginStyle(t),this.restoreStates(e,n)},e.prototype.updatePosition=function(e){var t=this.get("model"),n=Object(ce.isNil)(e.x)?t.x:e.x,r=Object(ce.isNil)(e.y)?t.y:e.y,i=this.get("group");Object(ce.isNil)(n)||Object(ce.isNil)(r)||(i.resetMatrix(),Xe(i,{x:n,y:r}),t.x=n,t.y=r,this.clearCache())},e.prototype.getBBox=function(){var e=this.get("bboxCache");return e||(e=this.calculateBBox(),this.set("bboxCache",e)),e},e.prototype.getCanvasBBox=function(){var e=this.get("bboxCanvasCache");return e||(e=this.calculateCanvasBBox(),this.set("bboxCanvasCache",e)),e},e.prototype.toFront=function(){this.get("group").toFront()},e.prototype.toBack=function(){this.get("group").toBack()},e.prototype.show=function(){this.changeVisibility(!0)},e.prototype.hide=function(){this.changeVisibility(!1)},e.prototype.changeVisibility=function(e){var t=this.get("group");e?t.show():t.hide(),this.set("visible",e)},e.prototype.isVisible=function(){return this.get("visible")},e.prototype.enableCapture=function(e){var t=this.get("group");t&&t.set("capture",e)},e.prototype.destroy=function(){if(!this.destroyed){var e=this.get("animate"),t=this.get("group");e&&t.stopAnimate(),this.clearCache(),t.remove(),this._cfg=null,this.destroyed=!0}},e}(),Ln={source:"start",target:"end"},Dn=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(w.__extends)(t,e),t.prototype.getDefaultCfg=function(){return{type:"edge",sourceNode:null,targetNode:null,startPoint:null,endPoint:null,linkCenter:!1}},t.prototype.setEnd=function(e,t){var n=Ln[e]+"Point",r=e+"Node",i=this.get(r);i&&!i.destroyed&&i.removeEdge(this),Object(ce.isPlainObject)(t)?(this.set(n,t),this.set(r,null)):(t.addEdge(this),this.set(r,t),this.set(n,null))},t.prototype.getLinkPoint=function(e,t,n){var r=Ln[e]+"Point",i=e+"Node",o=this.get(r);if(!o){var a=this.get(i),s=e+"Anchor",l=this.getPrePoint(e,n),u=t[s];Object(ce.isNil)(u)||(o=a.getLinkPointByAnchor(u)),o=o||a.getLinkPoint(l),Object(ce.isNil)(o.index)||this.set(e+"AnchorIndex",o.index)}return o},t.prototype.getPrePoint=function(e,t){if(t&&t.length)return t["source"===e?0:t.length-1];var n="source"===e?"target":"source";return this.getEndPoint(n)},t.prototype.getEndPoint=function(e){var t=e+"Node",n=Ln[e]+"Point",r=this.get(t);return r?r.get("model"):this.get(n)},t.prototype.getControlPointsByCenter=function(e){var t=this.getEndPoint("source"),n=this.getEndPoint("target"),r=this.get("shapeFactory"),i=e.shape||e.type;return r.getControlPoints(i,{startPoint:t,endPoint:n})},t.prototype.getEndCenter=function(e){var t=e+"Node",n=Ln[e]+"Point",r=this.get(t);if(r){var i=r.getBBox();return{x:i.centerX,y:i.centerY}}return this.get(n)},t.prototype.init=function(){e.prototype.init.call(this),this.setSource(this.get("source")),this.setTarget(this.get("target"))},t.prototype.getShapeCfg=function(t){var n=this.get("linkCenter"),r=e.prototype.getShapeCfg.call(this,t);if(n)r.startPoint=this.getEndCenter("source"),r.endPoint=this.getEndCenter("target");else{var i=r.controlPoints||this.getControlPointsByCenter(r);r.startPoint=this.getLinkPoint("source",t,i),r.endPoint=this.getLinkPoint("target",t,i)}return r.sourceNode=this.get("sourceNode"),r.targetNode=this.get("targetNode"),r},t.prototype.getModel=function(){var e=this.get("model"),t=this.get("sourceNode"),n=this.get("targetNode");return t?delete e.sourceNode:e.source=this.get("startPoint"),n?delete e.targetNode:e.target=this.get("endPoint"),Object(ce.isString)(e.source)||Object(ce.isPlainObject)(e.source)||(e.source=e.source.getID()),Object(ce.isString)(e.target)||Object(ce.isPlainObject)(e.target)||(e.target=e.target.getID()),e},t.prototype.setSource=function(e){this.setEnd("source",e),this.set("source",e)},t.prototype.setTarget=function(e){this.setEnd("target",e),this.set("target",e)},t.prototype.getSource=function(){return this.get("source")},t.prototype.getTarget=function(){return this.get("target")},t.prototype.updatePosition=function(){},t.prototype.update=function(e){var t=this.get("model"),n=t.visible,r=e.visible;n!==r&&void 0!==r&&this.changeVisibility(r);var i=this.get("styles");if(e.stateStyles){var o=e.stateStyles;Object(ce.mix)(i,o),delete e.stateStyles}Object.assign(t,e),this.updateShape(),this.afterUpdate(),this.clearCache()},t.prototype.destroy=function(){var t=this.get("sourceNode"),n=this.get("targetNode");t&&!t.destroyed&&t.removeEdge(this),n&&!n.destroyed&&n.removeEdge(this),e.prototype.destroy.call(this)},t}(kn),Pn=n(17),Nn=n.n(Pn),jn=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(w.__extends)(t,e),t.prototype.getNearestPoint=function(e,t){for(var n=0,r=e[0],i=Ve(e[0],t),o=0;o<e.length;o++){var a=e[o],s=Ve(a,t);s<i&&(r=a,i=s,n=o)}return r.anchorIndex=n,r},t.prototype.getDefaultCfg=function(){return{type:"node",edges:[]}},t.prototype.getEdges=function(){return this.get("edges")},t.prototype.getInEdges=function(){var e=this;return this.get("edges").filter((function(t){return t.get("target")===e}))},t.prototype.getOutEdges=function(){var e=this;return this.get("edges").filter((function(t){return t.get("source")===e}))},t.prototype.getNeighbors=function(e){var t=this,n=this.get("edges");return"target"===e?n.filter((function(e){return e.getSource()===t})).map((function(e){return e.getTarget()})):"source"===e?n.filter((function(e){return e.getTarget()===t})).map((function(e){return e.getSource()})):n.map((function(e){return e.getSource()===t?e.getTarget():e.getSource()}))},t.prototype.getLinkPointByAnchor=function(e){return this.getAnchorPoints()[e]},t.prototype.getLinkPoint=function(e){var t,n,r=this.get("keyShape").get("type"),i=this.get("type"),o=this.getBBox();"combo"===i?(t=(o.maxX+o.minX)/2,n=(o.maxY+o.minY)/2):(t=o.centerX,n=o.centerY);var a,s=this.getAnchorPoints();switch(r){case"circle":a=ze({x:t,y:n,r:o.width/2},e);break;case"ellipse":a=Fe({x:t,y:n,rx:o.width/2,ry:o.height/2},e);break;default:a=Re(o,e)}var l=a;return s.length&&(l||(l=e),l=this.getNearestPoint(s,l)),l||(l={x:t,y:n}),l},t.prototype.getAnchorPoints=function(){var e=this.get("anchorPointsCache");if(!e){e=[];var t=this.get("shapeFactory"),n=this.getBBox(),r=this.get("model"),i=this.getShapeCfg(r),o=r.shape||r.type,a=t.getAnchorPoints(o,i)||[];f()(a,(function(t,r){var i=Nn()({x:n.minX+t[0]*n.width,y:n.minY+t[1]*n.height},t[2],{index:r});e.push(i)})),this.set("anchorPointsCache",e)}return e},t.prototype.addEdge=function(e){this.get("edges").push(e)},t.prototype.lock=function(){this.set("locked",!0)},t.prototype.unlock=function(){this.set("locked",!1)},t.prototype.hasLocked=function(){return this.get("locked")},t.prototype.removeEdge=function(e){var t=this.getEdges(),n=t.indexOf(e);n>-1&&t.splice(n,1)},t.prototype.clearCache=function(){this.set("bboxCache",null),this.set("anchorPointsCache",null)},t.prototype.isOnlyMove=function(e){if(!e)return!1;var t=!C()(e.x),n=!C()(e.y),r=Object.keys(e);return 1===r.length&&(t||n)||2===r.length&&t&&n},t}(kn),Rn=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(w.__extends)(t,e),t.prototype.getDefaultCfg=function(){return{type:"combo",nodes:[],edges:[],combos:[]}},t.prototype.getShapeCfg=function(e){var t=this.get("styles"),n=this.get("bbox");if(t&&n){var r=e,i={r:Math.hypot(n.height,n.width)/2||J.defaultCombo.size[0]/2,width:n.width||J.defaultCombo.size[0],height:n.height||J.defaultCombo.size[1]};r.style=Object(w.__assign)(Object(w.__assign)(Object(w.__assign)({},t),e.style),i);var o=e.padding||J.defaultCombo.padding;return _()(o)?(i.r+=o,i.width+=2*o,i.height+=2*o):(i.r+=o[0],i.width+=o[1]+o[3]||2*o[1],i.height+=o[0]+o[2]||2*o[0]),this.set("sizeCache",i),r}return e},t.prototype.calculateCanvasBBox=function(){var e=this.get("keyShape"),t=this.get("group"),n=un(e,t);n.x=n.minX,n.y=n.minY,n.centerX=(n.minX+n.maxX)/2,n.centerY=(n.minY+n.maxY)/2;var r=this.get("sizeCache");return r?("circle"===e.get("type")?(n.width=2*r.r,n.height=2*r.r):(n.width=r.width,n.height=r.height),n.minX=n.centerX-n.width/2,n.minY=n.centerY-n.height/2,n.maxX=n.centerX+n.width/2,n.maxY=n.centerY+n.height/2):(n.width=n.maxX-n.minX,n.height=n.maxY-n.minY,n.centerX=(n.minX+n.maxX)/2,n.centerY=(n.minY+n.maxY)/2),n},t.prototype.getChildren=function(){return{nodes:this.getNodes(),combos:this.getCombos()}},t.prototype.getNodes=function(){return this.get("nodes")},t.prototype.getCombos=function(){return this.get("combos")},t.prototype.addChild=function(e){switch(e.getType()){case"node":this.addNode(e);break;case"combo":this.addCombo(e);break;default:return console.warn("Only node or combo items are allowed to be added into a combo"),!1}return!0},t.prototype.addCombo=function(e){return this.get("combos").push(e),!0},t.prototype.addNode=function(e){return this.get("nodes").push(e),!0},t.prototype.removeChild=function(e){switch(e.getType()){case"node":this.removeNode(e);break;case"combo":this.removeCombo(e);break;default:return console.warn("Only node or combo items are allowed to be added into a combo"),!1}return!0},t.prototype.removeCombo=function(e){if(e){var t=this.getCombos(),n=t.indexOf(e);return n>-1&&(t.splice(n,1),!0)}},t.prototype.removeNode=function(e){if(e){var t=this.getNodes(),n=t.indexOf(e);return n>-1&&(t.splice(n,1),!0)}},t.prototype.isOnlyMove=function(e){return!1},t.prototype.getBBox=function(){return this.set("bboxCanvasCache",null),this.calculateCanvasBBox()},t.prototype.clearCache=function(){this.set("bboxCache",null),this.set("bboxCanvasCache",null),this.set("anchorPointsCache",null)},t.prototype.destroy=function(){if(!this.destroyed){var e=this.get("animate"),t=this.get("group");e&&t.stopAnimate(),this.clearCache(),this.set("sizeCache",null),this.set("bbox",null),t.remove(),this._cfg=null,this.destroyed=!0}},t}(jn),zn=Object.hasOwnProperty,Fn=function(){function e(e){this.graph=e,this.destroyed=!1}return e.prototype.addItem=function(e,t){var n=this,r=this.graph,i="vedge"===e?"edge":e,o=r.get(i+"Group")||r.get("group"),a=$t()(i),s=null,l=r.get(i+$t()("stateStyles"))||{},u=r.get("default"+a);t.stateStyles&&(l=t.stateStyles);var c=r.get(i+"Mapper");if(c){var d=c(t);d.stateStyles&&(l=d.stateStyles,delete d.stateStyles),t=Ie()({},u,t,d)}else u&&f()(u,(function(e,n){zn.call(t,n)||(Kt()(e)?t[n]=Zt()(e):t[n]=u[n])}));if(t.shape&&!t.type&&console.warn("shape 字段即将被废弃，请使用 type 代替"),r.emit("beforeadditem",{type:e,model:t}),"edge"===e||"vedge"===e){var h=void 0,p=void 0;if(h=t.source,p=t.target,h&&x()(h)&&(h=r.findById(h)),p&&x()(p)&&(p=r.findById(p)),!h||!p)return void console.warn("The source or target node of edge "+t.id+" does not exist!");h.getType&&"combo"===h.getType()&&(t.isComboEdge=!0,r.updateCombo(h)),p.getType&&"combo"===p.getType()&&(t.isComboEdge=!0,r.updateCombo(p)),s=new Dn({model:t,source:h,target:p,styles:l,linkCenter:r.get("linkCenter"),group:o.addGroup()})}else if("node"===e)s=new jn({model:t,styles:l,group:o.addGroup()});else if("combo"===e){var g=t.children,v=An(g,r);t.x=v.x||t.x||100*Math.random(),t.y=v.y||t.y||100*Math.random();var m=o.addGroup();m.setZIndex(t.depth);var y=(s=new Rn({model:t,styles:l,bbox:v,group:m})).getModel();(g||[]).forEach((function(e){var t=r.findById(e.id);s.addChild(t),e.depth=y.depth+2})),t.collapsed&&setTimeout((function(){r.collapseCombo(s),n.updateCombo(s,[])}),250)}if(s)return r.get(e+"s").push(s),r.get("itemMap")[s.get("id")]=s,r.emit("afteradditem",{item:s,model:t}),s},e.prototype.updateItem=function(e,t){var n=this.graph;if(x()(e)&&(e=n.findById(e)),e&&!e.destroyed){var r="";e.getType&&(r=e.getType());var i=n.get(r+"Mapper"),o=e.getModel();if(i){var a=i(Ie()({},o,t)),s=Ie()({},o,a,t);a.stateStyles&&(e.set("styles",s.stateStyles),delete s.stateStyles),f()(s,(function(e,n){t[n]=e}))}else f()(t,(function(e,n){o[n]&&Kt()(e)&&!y()(e)&&(t[n]=Object(w.__assign)(Object(w.__assign)({},o[n]),t[n]))}));if(n.emit("beforeupdateitem",{item:e,cfg:t}),"edge"===r){if(t.source){var l=t.source;x()(l)&&(l=n.findById(l)),e.setSource(l)}if(t.target){var u=t.target;x()(u)&&(u=n.findById(u)),e.setTarget(u)}}if(e.update(t),"node"===r||"combo"===r){var c=e.getEdges();f()(c,(function(e){e.refresh()}))}n.emit("afterupdateitem",{item:e,cfg:t})}},e.prototype.updateCombo=function(e,t){var n=this.graph;if(x()(e)&&(e=n.findById(e)),e&&!e.destroyed){var r=An(t,n);e.set("bbox",r),e.update({x:r.x,y:r.y});for(var i=e.getEdges()||[],o=i.length,a=0;a<o;a++){var s=i[a];null==s||s.refresh()}}},e.prototype.collapseCombo=function(e){var t=this.graph;x()(e)&&(e=t.findById(e));var n=e.getChildren();n.nodes.forEach((function(e){t.hideItem(e)})),n.combos.forEach((function(e){t.hideItem(e)}))},e.prototype.expandCombo=function(e){var t=this.graph;x()(e)&&(e=t.findById(e));var n=e.getChildren();n.nodes.forEach((function(e){t.showItem(e)})),n.combos.forEach((function(e){e.getModel().collapsed?e.show():t.showItem(e)}))},e.prototype.removeItem=function(e){var t=this,n=this.graph;if(x()(e)&&(e=n.findById(e)),e&&!e.destroyed){var r=Zt()(e.getModel());n.emit("beforeremoveitem",{item:r});var i="";e.getType&&(i=e.getType());var o=n.get(i+"s"),a=o.indexOf(e);if(a>-1&&o.splice(a,1),"edge"===i){var s=n.get("v"+i+"s"),l=s.indexOf(e);l>-1&&s.splice(l,1)}var u=e.get("id");delete n.get("itemMap")[u];var c=n.get("comboTrees"),d=e.get("id");if("node"===i){var h=e.getModel().comboId;if(c){var f=c,p=!1;c.forEach((function(e){p||pn(e,(function(e){if(e.id===d&&f){var t=f.indexOf(e);return f.splice(t,1),p=!0,!1}return f=e.children,!0}))}))}for(var g=(m=e.getEdges()).length-1;g>=0;g--)n.removeItem(m[g],!1);h&&n.updateCombo(h)}else if("combo"===i){var v,m,y=e.getModel().parentId,b=!1;for((c||[]).forEach((function(e){b||pn(e,(function(e){return e.id!==d||(v=e,b=!0,!1)}))})),v.removed=!0,v&&v.children&&v.children.forEach((function(e){t.removeItem(e.id)})),g=(m=e.getEdges()).length;g>=0;g--)n.removeItem(m[g],!1);y&&n.updateCombo(y)}e.destroy(),n.emit("afterremoveitem",{item:r})}},e.prototype.setItemState=function(e,t,n){var r=this.graph,i=t;x()(n)&&(i=t+":"+n),e.hasState(i)===n||x()(n)&&e.hasState(i)||(r.emit("beforeitemstatechange",{item:e,state:i,enabled:n}),e.setState(t,n),r.autoPaint(),r.emit("afteritemstatechange",{item:e,state:i,enabled:n}))},e.prototype.priorityState=function(e,t){var n=this.graph,r=e;x()(e)&&(r=n.findById(e)),this.setItemState(r,t,!1),this.setItemState(r,t,!0)},e.prototype.clearItemStates=function(e,t){var n=this.graph;x()(e)&&(e=n.findById(e)),n.emit("beforeitemstatesclear",{item:e,states:t}),e.clearStates(t),n.emit("afteritemstatesclear",{item:e,states:t})},e.prototype.refreshItem=function(e){var t=this.graph;x()(e)&&(e=t.findById(e)),t.emit("beforeitemrefresh",{item:e}),e.refresh(),t.emit("afteritemrefresh",{item:e})},e.prototype.addCombos=function(e,t){var n=this,r=this.graph;(e||[]).forEach((function(e){gn(e,(function(e){var r;return t.forEach((function(t){t.id===e.id&&(t.children=e.children,t.depth=e.depth,r=t)})),r&&n.addItem("combo",r),!0}))}));var i=r.get("comboGroup");i&&i.sort()},e.prototype.changeItemVisibility=function(e,t){var n=this,r=this.graph;if(x()(e)&&(e=r.findById(e)),e){if(r.emit("beforeitemvisibilitychange",{item:e,visible:t}),e.changeVisibility(t),e.getType&&"node"===e.getType()){var i=e.getEdges();f()(i,(function(e){(!t||e.get("source").isVisible()&&e.get("target").isVisible())&&n.changeItemVisibility(e,t)}))}else if(e.getType&&"combo"===e.getType()){var o=r.get("comboTrees"),a=e.get("id"),s=[],l=!1;(o||[]).forEach((function(e){l||e.children&&0!==e.children.length&&pn(e,(function(e){return e.id!==a||(s=e.children,l=!0,!1)}))})),s.forEach((function(e){var i=r.findById(e.id);n.changeItemVisibility(i,t)})),i=e.getEdges(),f()(i,(function(e){(!t||e.get("source").isVisible()&&e.get("target").isVisible())&&n.changeItemVisibility(e,t)}))}return r.emit("afteritemvisibilitychange",{item:e,visible:t}),e}console.warn("The item to be shown or hidden does not exist!")},e.prototype.destroy=function(){this.graph=null,this.destroyed=!0},e}(),Bn=function(){function e(){this.nodes=[],this.edges=[],this.combos=[],this.positions=[],this.destroyed=!1}return e.prototype.init=function(e){this.nodes=e.nodes||[],this.edges=e.edges||[],this.combos=e.combos||[]},e.prototype.execute=function(){},e.prototype.layout=function(e){this.init(e),this.execute()},e.prototype.getDefaultCfg=function(){return{}},e.prototype.updateCfg=function(e){Nn()(this,e)},e.prototype.destroy=function(){this.positions=null,this.nodes=null,this.edges=null,this.destroyed=!0},e}(),Hn={registerLayout:function(e,t,n){if(void 0===n&&(n=Bn),!t)throw new Error("please specify handler for this layout: "+e);var r=function(e){function n(n){var r=e.call(this)||this,i=r,o={},a=i.getDefaultCfg();return Nn()(o,a,t,n),f()(o,(function(e,t){i[t]=e})),r}return Object(w.__extends)(n,e),n}(n);Hn[e]=r}},Un=Hn;function Vn(e,t){return e.degree<t.degree?-1:e.degree>t.degree?1:0}var Yn=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.center=[0,0],t.radius=null,t.startRadius=null,t.endRadius=null,t.startAngle=0,t.endAngle=2*Math.PI,t.clockwise=!0,t.divisions=1,t.ordering=null,t.angleRatio=1,t.nodes=[],t.edges=[],t.nodeMap={},t.degrees=[],t.width=300,t.height=300,t}return Object(w.__extends)(t,e),t.prototype.getDefaultCfg=function(){return{center:[0,0],radius:null,startRadius:null,endRadius:null,startAngle:0,endAngle:2*Math.PI,clockwise:!0,divisions:1,ordering:null,angleRatio:1}},t.prototype.execute=function(){var e=this.nodes,t=this.edges,n=e.length,r=this.center;if(0!==n){if(1===n)return e[0].x=r[0],void(e[0].y=r[1]);var i=this.radius,o=this.startRadius,a=this.endRadius,s=this.divisions,l=this.startAngle,u=this.endAngle,c=(u-l)/n,d={};e.forEach((function(e,t){d[e.id]=t})),this.nodeMap=d;var h=Ke(e.length,d,t);this.degrees=h,this.width||"undefined"==typeof window||(this.width=window.innerWidth),this.height||"undefined"==typeof window||(this.height=window.innerHeight),i||o||a?!o&&a?o=a:o&&!a&&(a=o):i=this.height>this.width?this.width/2:this.height/2;var f=c*this.angleRatio;this.astep=f;var p=this.ordering,g=[];g="topology"===p?this.topologyOrdering():"topology-directed"===p?this.topologyOrdering(!0):"degree"===p?this.degreeOrdering():e;for(var v=this.clockwise,m=Math.ceil(n/s),y=0;y<n;++y){var b=i;b||null===o||null===a||(b=o+y*(a-o)/(n-1)),b||(b=10+100*y/(n-1));var C=l+y%m*f+2*Math.PI/s*Math.floor(y/m);v||(C=u-y%m*f-2*Math.PI/s*Math.floor(y/m)),g[y].x=r[0]+Math.cos(C)*b,g[y].y=r[1]+Math.sin(C)*b,g[y].weight=h[y]}}},t.prototype.topologyOrdering=function(e){void 0===e&&(e=!1);var t=this.degrees,n=this.edges,r=this.nodes,i=Object(ce.clone)(r),o=this.nodeMap,a=[i[0]],s=[r[0]],l=[],u=r.length;l[0]=!0,function(e,t,n,r){e.forEach((function(t,n){e[n].children=[],e[n].parent=[]})),r?t.forEach((function(t){var r=0;t.source&&(r=n[t.source]);var i=0;t.target&&(i=n[t.target]),e[r].children.push(e[i].id),e[i].parent.push(e[r].id)})):t.forEach((function(t){var r=0;t.source&&(r=n[t.source]);var i=0;t.target&&(i=n[t.target]),e[r].children.push(e[i].id),e[i].children.push(e[r].id)}))}(i,n,o,e);var c=0;return i.forEach((function(e,d){if(0!==d)if(d!==u-1&&t[d]===t[d+1]&&!function(e,t,n){for(var r=n.length,i=0;i<r;i++)if(e.id===n[i].source&&t.id===n[i].target||t.id===n[i].source&&e.id===n[i].target)return!0;return!1}(a[c],e,n)||!0===l[d]){for(var h=a[c].children,f=!1,p=0;p<h.length;p++){var g=o[h[p]];if(t[g]===t[d]&&!0!==l[g]){a.push(i[g]),s.push(r[o[i[g].id]]),l[g]=!0,f=!0;break}}for(var v=0;!f&&(l[v]||(a.push(i[v]),s.push(r[o[i[v].id]]),l[v]=!0,f=!0),++v!==u););}else a.push(e),s.push(r[o[e.id]]),l[d]=!0,c++})),s},t.prototype.degreeOrdering=function(){var e=this.nodes,t=[],n=this.degrees;return e.forEach((function(e,r){e.degree=n[r],t.push(e)})),t.sort(Vn),t},t}(Bn),Wn=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.center=[0,0],t.nodeSize=30,t.minNodeSpacing=10,t.preventOverlap=!1,t.equidistant=!1,t.startAngle=1.5*Math.PI,t.clockwise=!0,t.sortBy="degree",t.nodes=[],t.edges=[],t.width=300,t.height=300,t}return Object(w.__extends)(t,e),t.prototype.getDefaultCfg=function(){return{center:[0,0],nodeSize:30,minNodeSpacing:10,preventOverlap:!1,sweep:void 0,equidistant:!1,startAngle:1.5*Math.PI,clockwise:!0,maxLevelDiff:void 0,sortBy:"degree"}},t.prototype.execute=function(){var e=this,t=e.nodes,n=e.edges,r=t.length,i=e.center;if(0!==r){if(1===r)return t[0].x=i[0],void(t[0].y=i[1]);var o,a=[];o=y()(e.nodeSize)?Math.max(e.nodeSize[0],e.nodeSize[1]):e.nodeSize,t.forEach((function(e){a.push(e);var t=o;y()(e.size)?t=Math.max(e.size[0],e.size[1]):Object(ce.isNumber)(e.size)&&(t=e.size),o=Math.max(o,t)})),e.width||"undefined"==typeof window||(e.width=window.innerWidth),e.height||"undefined"==typeof window||(e.height=window.innerHeight),e.clockwise=void 0!==e.counterclockwise?!e.counterclockwise:e.clockwise;var s={},l={};if(a.forEach((function(e,t){s[e.id]=e,l[e.id]=t})),!("degree"!==e.sortBy&&x()(e.sortBy)&&void 0!==a[0][e.sortBy]||(e.sortBy="degree",Object(ce.isNumber)(t[0].degree)))){var u=Ke(t.length,l,n);a.forEach((function(e,t){e.degree=u[t]}))}a.sort((function(t,n){return n[e.sortBy]-t[e.sortBy]})),e.maxValueNode=a[0],e.maxLevelDiff=e.maxLevelDiff||e.maxValueNode[e.sortBy]/4;var c=[[]],d=c[0];a.forEach((function(t){if(d.length>0){var n=Math.abs(d[0][e.sortBy]-t[e.sortBy]);e.maxLevelDiff&&n>=e.maxLevelDiff&&(d=[],c.push(d))}d.push(t)}));var h=o+e.minNodeSpacing;if(!e.preventOverlap){var f=c.length>0&&c[0].length>1,p=(Math.min(e.width,e.height)/2-h)/(c.length+(f?1:0));h=Math.min(h,p)}var g=0;if(c.forEach((function(t){var n=e.sweep;void 0===n&&(n=2*Math.PI-2*Math.PI/t.length);var r=t.dTheta=n/Math.max(1,t.length-1);if(t.length>1&&e.preventOverlap){var i=Math.cos(r)-Math.cos(0),o=Math.sin(r)-Math.sin(0),a=Math.sqrt(h*h/(i*i+o*o));g=Math.max(a,g)}t.r=g,g+=h})),e.equidistant){for(var v=0,m=0,b=0;b<c.length;b++){var C=c[b].r-m;v=Math.max(v,C)}m=0,c.forEach((function(e,t){0===t&&(m=e.r),e.r=m,m+=v}))}c.forEach((function(t){var n=t.dTheta,r=t.r;t.forEach((function(t,o){var a=e.startAngle+(e.clockwise?1:-1)*n*o;t.x=i[0]+r*Math.cos(a),t.y=i[1]+r*Math.sin(a)}))}))}},t}(Bn),Gn=n(134),Xn=n.n(Gn),qn=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.rankdir="TB",t.nodesep=50,t.ranksep=50,t.controlPoints=!1,t.sortByCombo=!1,t}return Object(w.__extends)(t,e),t.prototype.getDefaultCfg=function(){return{rankdir:"TB",align:void 0,nodeSize:void 0,nodesepFunc:void 0,ranksepFunc:void 0,nodesep:50,ranksep:50,controlPoints:!1}},t.prototype.execute=function(){var e=this,t=e.nodes,n=e.nodeSize,r=e.rankdir;if(t){var i,o=e.edges||[],a=new Xn.a.graphlib.Graph;i=n?y()(n)?function(){return n}:function(){return[n,n]}:function(e){return e.size?y()(e.size)?e.size:[e.size,e.size]:[40,40]};var s,l=Zn(e.nodesepFunc,e.nodesep,50),u=Zn(e.ranksepFunc,e.ranksep,50);"LR"!==r&&"RL"!==r||(l=Zn(e.ranksepFunc,e.ranksep,50),u=Zn(e.nodesepFunc,e.nodesep,50)),a.setDefaultEdgeLabel((function(){return{}})),a.setGraph(e),t.forEach((function(e){var t=i(e),n=u(e),r=l(e),o=t[0]+2*r,s=t[1]+2*n;a.setNode(e.id,{width:o,height:s})})),o.forEach((function(e){a.setEdge(e.source,e.target,{weight:e.weight||1})})),Xn.a.layout(a),a.nodes().forEach((function(e){s=a.node(e);var n=t.findIndex((function(t){return t.id===e}));t[n].x=s.x,t[n].y=s.y})),a.edges().forEach((function(t){s=a.edge(t);var n=o.findIndex((function(e){return e.source===t.v&&e.target===t.w}));e.controlPoints&&"loop"!==o[n].type&&"loop"!==o[n].shape&&(o[n].controlPoints=s.points.slice(1,s.points.length-1))})),e.sortByCombo&&e.sortLevel("comboId")}},t.prototype.sortLevel=function(e){var t=this.nodes,n={};t.forEach((function(e){n[e.y]||(n[e.y]={y:e.y,nodes:[]}),n[e.y].nodes.push(e)})),Object.keys(n).forEach((function(t){var r=n[t].nodes,i=r.length,o={};if(r.forEach((function(e){var t=e.comboId;o[t]||(o[t]={x:0,y:0,count:0}),o[t].x+=e.x,o[t].y+=e.y,o[t].count++})),Object.keys(o).forEach((function(e){o[e].x/=o[e].count,o[e].y/=o[e].count})),1!==i){var a=r.sort((function(e,t){return e.x-t.x})),s=a[0].x,l=(a[i-1].x-s)/(i-1);r.sort((function(t,n){var r=t[e]||"undefined",i=n[e]||"undefined";return r<i?-1:r>i?1:0})).forEach((function(e,t){e.x=s+t*l}))}}))},t}(Bn);function Zn(e,t,n){return e||(Object(ce.isNumber)(t)?function(){return t}:function(){return n})}function Qn(e,t,n,r){if(isNaN(t)||isNaN(n))return e;var i,o,a,s,l,u,c,d,h,f=e._root,p={data:r},g=e._x0,v=e._y0,m=e._x1,y=e._y1;if(!f)return e._root=p,e;for(;f.length;)if((u=t>=(o=(g+m)/2))?g=o:m=o,(c=n>=(a=(v+y)/2))?v=a:y=a,i=f,!(f=f[d=c<<1|u]))return i[d]=p,e;if(s=+e._x.call(null,f.data),l=+e._y.call(null,f.data),t===s&&n===l)return p.next=f,i?i[d]=p:e._root=p,e;do{i=i?i[d]=new Array(4):e._root=new Array(4),(u=t>=(o=(g+m)/2))?g=o:m=o,(c=n>=(a=(v+y)/2))?v=a:y=a}while((d=c<<1|u)==(h=(l>=a)<<1|s>=o));return i[h]=f,i[d]=p,e}var Kn=function(e,t,n,r,i){this.node=e,this.x0=t,this.y0=n,this.x1=r,this.y1=i};function Jn(e){return e[0]}function $n(e){return e[1]}function er(e,t,n){var r=new tr(null==t?Jn:t,null==n?$n:n,NaN,NaN,NaN,NaN);return null==e?r:r.addAll(e)}function tr(e,t,n,r,i,o){this._x=e,this._y=t,this._x0=n,this._y0=r,this._x1=i,this._y1=o,this._root=void 0}function nr(e){for(var t={data:e.data},n=t;e=e.next;)n=n.next={data:e.data};return t}var rr=er.prototype=tr.prototype;rr.copy=function(){var e,t,n=new tr(this._x,this._y,this._x0,this._y0,this._x1,this._y1),r=this._root;if(!r)return n;if(!r.length)return n._root=nr(r),n;for(e=[{source:r,target:n._root=new Array(4)}];r=e.pop();)for(var i=0;i<4;++i)(t=r.source[i])&&(t.length?e.push({source:t,target:r.target[i]=new Array(4)}):r.target[i]=nr(t));return n},rr.add=function(e){var t=+this._x.call(null,e),n=+this._y.call(null,e);return Qn(this.cover(t,n),t,n,e)},rr.addAll=function(e){var t,n,r,i,o=e.length,a=new Array(o),s=new Array(o),l=1/0,u=1/0,c=-1/0,d=-1/0;for(n=0;n<o;++n)isNaN(r=+this._x.call(null,t=e[n]))||isNaN(i=+this._y.call(null,t))||(a[n]=r,s[n]=i,r<l&&(l=r),r>c&&(c=r),i<u&&(u=i),i>d&&(d=i));if(l>c||u>d)return this;for(this.cover(l,u).cover(c,d),n=0;n<o;++n)Qn(this,a[n],s[n],e[n]);return this},rr.cover=function(e,t){if(isNaN(e=+e)||isNaN(t=+t))return this;var n=this._x0,r=this._y0,i=this._x1,o=this._y1;if(isNaN(n))i=(n=Math.floor(e))+1,o=(r=Math.floor(t))+1;else{for(var a,s,l=i-n||1,u=this._root;n>e||e>=i||r>t||t>=o;)switch(s=(t<r)<<1|e<n,(a=new Array(4))[s]=u,u=a,l*=2,s){case 0:i=n+l,o=r+l;break;case 1:n=i-l,o=r+l;break;case 2:i=n+l,r=o-l;break;case 3:n=i-l,r=o-l}this._root&&this._root.length&&(this._root=u)}return this._x0=n,this._y0=r,this._x1=i,this._y1=o,this},rr.data=function(){var e=[];return this.visit((function(t){if(!t.length)do{e.push(t.data)}while(t=t.next)})),e},rr.extent=function(e){return arguments.length?this.cover(+e[0][0],+e[0][1]).cover(+e[1][0],+e[1][1]):isNaN(this._x0)?void 0:[[this._x0,this._y0],[this._x1,this._y1]]},rr.find=function(e,t,n){var r,i,o,a,s,l,u,c=this._x0,d=this._y0,h=this._x1,f=this._y1,p=[],g=this._root;for(g&&p.push(new Kn(g,c,d,h,f)),null==n?n=1/0:(c=e-n,d=t-n,h=e+n,f=t+n,n*=n);l=p.pop();)if(!(!(g=l.node)||(i=l.x0)>h||(o=l.y0)>f||(a=l.x1)<c||(s=l.y1)<d))if(g.length){var v=(i+a)/2,m=(o+s)/2;p.push(new Kn(g[3],v,m,a,s),new Kn(g[2],i,m,v,s),new Kn(g[1],v,o,a,m),new Kn(g[0],i,o,v,m)),(u=(t>=m)<<1|e>=v)&&(l=p[p.length-1],p[p.length-1]=p[p.length-1-u],p[p.length-1-u]=l)}else{var y=e-+this._x.call(null,g.data),b=t-+this._y.call(null,g.data),C=y*y+b*b;if(C<n){var A=Math.sqrt(n=C);c=e-A,d=t-A,h=e+A,f=t+A,r=g.data}}return r},rr.remove=function(e){if(isNaN(o=+this._x.call(null,e))||isNaN(a=+this._y.call(null,e)))return this;var t,n,r,i,o,a,s,l,u,c,d,h,f=this._root,p=this._x0,g=this._y0,v=this._x1,m=this._y1;if(!f)return this;if(f.length)for(;;){if((u=o>=(s=(p+v)/2))?p=s:v=s,(c=a>=(l=(g+m)/2))?g=l:m=l,t=f,!(f=f[d=c<<1|u]))return this;if(!f.length)break;(t[d+1&3]||t[d+2&3]||t[d+3&3])&&(n=t,h=d)}for(;f.data!==e;)if(r=f,!(f=f.next))return this;return(i=f.next)&&delete f.next,r?(i?r.next=i:delete r.next,this):t?(i?t[d]=i:delete t[d],(f=t[0]||t[1]||t[2]||t[3])&&f===(t[3]||t[2]||t[1]||t[0])&&!f.length&&(n?n[h]=f:this._root=f),this):(this._root=i,this)},rr.removeAll=function(e){for(var t=0,n=e.length;t<n;++t)this.remove(e[t]);return this},rr.root=function(){return this._root},rr.size=function(){var e=0;return this.visit((function(t){if(!t.length)do{++e}while(t=t.next)})),e},rr.visit=function(e){var t,n,r,i,o,a,s=[],l=this._root;for(l&&s.push(new Kn(l,this._x0,this._y0,this._x1,this._y1));t=s.pop();)if(!e(l=t.node,r=t.x0,i=t.y0,o=t.x1,a=t.y1)&&l.length){var u=(r+o)/2,c=(i+a)/2;(n=l[3])&&s.push(new Kn(n,u,c,o,a)),(n=l[2])&&s.push(new Kn(n,r,c,u,a)),(n=l[1])&&s.push(new Kn(n,u,i,o,c)),(n=l[0])&&s.push(new Kn(n,r,i,u,c))}return this},rr.visitAfter=function(e){var t,n=[],r=[];for(this._root&&n.push(new Kn(this._root,this._x0,this._y0,this._x1,this._y1));t=n.pop();){var i=t.node;if(i.length){var o,a=t.x0,s=t.y0,l=t.x1,u=t.y1,c=(a+l)/2,d=(s+u)/2;(o=i[0])&&n.push(new Kn(o,a,s,c,d)),(o=i[1])&&n.push(new Kn(o,c,s,l,d)),(o=i[2])&&n.push(new Kn(o,a,d,c,u)),(o=i[3])&&n.push(new Kn(o,c,d,l,u))}r.push(t)}for(;t=r.pop();)e(t.node,t.x0,t.y0,t.x1,t.y1);return this},rr.x=function(e){return arguments.length?(this._x=e,this):this._x},rr.y=function(e){return arguments.length?(this._y=e,this):this._y};var ir=function(e){return function(){return e}},or=function(e){return 1e-6*(e()-.5)},ar={value:function(){}};function sr(){for(var e,t=0,n=arguments.length,r={};t<n;++t){if(!(e=arguments[t]+"")||e in r||/[\s.]/.test(e))throw new Error("illegal type: "+e);r[e]=[]}return new lr(r)}function lr(e){this._=e}function ur(e,t){return e.trim().split(/^|\s+/).map((function(e){var n="",r=e.indexOf(".");if(r>=0&&(n=e.slice(r+1),e=e.slice(0,r)),e&&!t.hasOwnProperty(e))throw new Error("unknown type: "+e);return{type:e,name:n}}))}function cr(e,t){for(var n,r=0,i=e.length;r<i;++r)if((n=e[r]).name===t)return n.value}function dr(e,t,n){for(var r=0,i=e.length;r<i;++r)if(e[r].name===t){e[r]=ar,e=e.slice(0,r).concat(e.slice(r+1));break}return null!=n&&e.push({name:t,value:n}),e}lr.prototype=sr.prototype={constructor:lr,on:function(e,t){var n,r=this._,i=ur(e+"",r),o=-1,a=i.length;if(!(arguments.length<2)){if(null!=t&&"function"!=typeof t)throw new Error("invalid callback: "+t);for(;++o<a;)if(n=(e=i[o]).type)r[n]=dr(r[n],e.name,t);else if(null==t)for(n in r)r[n]=dr(r[n],e.name,null);return this}for(;++o<a;)if((n=(e=i[o]).type)&&(n=cr(r[n],e.name)))return n},copy:function(){var e={},t=this._;for(var n in t)e[n]=t[n].slice();return new lr(e)},call:function(e,t){if((n=arguments.length-2)>0)for(var n,r,i=new Array(n),o=0;o<n;++o)i[o]=arguments[o+2];if(!this._.hasOwnProperty(e))throw new Error("unknown type: "+e);for(o=0,n=(r=this._[e]).length;o<n;++o)r[o].value.apply(t,i)},apply:function(e,t,n){if(!this._.hasOwnProperty(e))throw new Error("unknown type: "+e);for(var r=this._[e],i=0,o=r.length;i<o;++i)r[i].value.apply(t,n)}};var hr=sr;function fr(e){return(fr="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}var pr,gr,vr=0,mr=0,yr=0,br=0,Cr=0,Ar=0,_r="object"===("undefined"==typeof performance?"undefined":fr(performance))&&performance.now?performance:Date,Ir="object"===("undefined"==typeof window?"undefined":fr(window))&&window.requestAnimationFrame?window.requestAnimationFrame.bind(window):function(e){setTimeout(e,17)};function xr(){return Cr||(Ir(wr),Cr=_r.now()+Ar)}function wr(){Cr=0}function Mr(){this._call=this._time=this._next=null}function Sr(e,t,n){var r=new Mr;return r.restart(e,t,n),r}function Tr(){Cr=(br=_r.now())+Ar,vr=mr=0;try{!function(){xr(),++vr;for(var e,t=pr;t;)(e=Cr-t._time)>=0&&t._call.call(null,e),t=t._next;--vr}()}finally{vr=0,function(){for(var e,t,n=pr,r=1/0;n;)n._call?(r>n._time&&(r=n._time),e=n,n=n._next):(t=n._next,n._next=null,n=e?e._next=t:pr=t);gr=e,Or(r)}(),Cr=0}}function Er(){var e=_r.now(),t=e-br;t>1e3&&(Ar-=t,br=e)}function Or(e){vr||(mr&&(mr=clearTimeout(mr)),e-Cr>24?(e<1/0&&(mr=setTimeout(Tr,e-_r.now()-Ar)),yr&&(yr=clearInterval(yr))):(yr||(br=_r.now(),yr=setInterval(Er,1e3)),vr=1,Ir(Tr)))}function kr(e){return e.x}function Lr(e){return e.y}Mr.prototype=Sr.prototype={constructor:Mr,restart:function(e,t,n){if("function"!=typeof e)throw new TypeError("callback is not a function");n=(null==n?xr():+n)+(null==t?0:+t),this._next||gr===this||(gr?gr._next=this:pr=this,gr=this),this._call=e,this._time=n,Or()},stop:function(){this._call&&(this._call=null,this._time=1/0,Or())}};var Dr=Math.PI*(3-Math.sqrt(5)),Pr=function(e){var t,n,r=1,i=.001,o=1-Math.pow(i,1/300),a=0,s=.6,l=new Map,u=Sr(h),c=hr("tick","end"),d=(n=1,function(){return(n=(1664525*n+1013904223)%4294967296)/4294967296});function h(){f(),c.call("tick",t),r<i&&(u.stop(),c.call("end",t))}function f(n){var i,u,c=e.length;void 0===n&&(n=1);for(var d=0;d<n;++d)for(r+=(a-r)*o,l.forEach((function(e){e(r)})),i=0;i<c;++i)null==(u=e[i]).fx?u.x+=u.vx*=s:(u.x=u.fx,u.vx=0),null==u.fy?u.y+=u.vy*=s:(u.y=u.fy,u.vy=0);return t}function p(){for(var t,n=0,r=e.length;n<r;++n){if((t=e[n]).index=n,null!=t.fx&&(t.x=t.fx),null!=t.fy&&(t.y=t.fy),isNaN(t.x)||isNaN(t.y)){var i=10*Math.sqrt(.5+n),o=n*Dr;t.x=i*Math.cos(o),t.y=i*Math.sin(o)}(isNaN(t.vx)||isNaN(t.vy))&&(t.vx=t.vy=0)}}function g(t){return t.initialize&&t.initialize(e,d),t}return null==e&&(e=[]),p(),t={tick:f,restart:function(){return u.restart(h),t},stop:function(){return u.stop(),t},nodes:function(n){return arguments.length?(e=n,p(),l.forEach(g),t):e},alpha:function(e){return arguments.length?(r=+e,t):r},alphaMin:function(e){return arguments.length?(i=+e,t):i},alphaDecay:function(e){return arguments.length?(o=+e,t):+o},alphaTarget:function(e){return arguments.length?(a=+e,t):a},velocityDecay:function(e){return arguments.length?(s=1-e,t):1-s},randomSource:function(e){return arguments.length?(d=e,l.forEach(g),t):d},force:function(e,n){return arguments.length>1?(null==n?l.delete(e):l.set(e,g(n)),t):l.get(e)},find:function(t,n,r){var i,o,a,s,l,u=0,c=e.length;for(null==r?r=1/0:r*=r,u=0;u<c;++u)(a=(i=t-(s=e[u]).x)*i+(o=n-s.y)*o)<r&&(l=s,r=a);return l},on:function(e,n){return arguments.length>1?(c.on(e,n),t):c.on(e)}}},Nr=function(){var e,t,n,r,i,o=ir(-30),a=1,s=1/0,l=.81;function u(n){var i,o=e.length,a=er(e,kr,Lr).visitAfter(d);for(r=n,i=0;i<o;++i)t=e[i],a.visit(h)}function c(){if(e){var t,n,r=e.length;for(i=new Array(r),t=0;t<r;++t)n=e[t],i[n.index]=+o(n,t,e)}}function d(e){var t,n,r,o,a,s=0,l=0;if(e.length){for(r=o=a=0;a<4;++a)(t=e[a])&&(n=Math.abs(t.value))&&(s+=t.value,l+=n,r+=n*t.x,o+=n*t.y);e.x=r/l,e.y=o/l}else{(t=e).x=t.data.x,t.y=t.data.y;do{s+=i[t.data.index]}while(t=t.next)}e.value=s}function h(e,o,u,c){if(!e.value)return!0;var d=e.x-t.x,h=e.y-t.y,f=c-o,p=d*d+h*h;if(f*f/l<p)return p<s&&(0===d&&(p+=(d=or(n))*d),0===h&&(p+=(h=or(n))*h),p<a&&(p=Math.sqrt(a*p)),t.vx+=d*e.value*r/p,t.vy+=h*e.value*r/p),!0;if(!(e.length||p>=s)){(e.data!==t||e.next)&&(0===d&&(p+=(d=or(n))*d),0===h&&(p+=(h=or(n))*h),p<a&&(p=Math.sqrt(a*p)));do{e.data!==t&&(f=i[e.data.index]*r/p,t.vx+=d*f,t.vy+=h*f)}while(e=e.next)}}return u.initialize=function(t,r){e=t,n=r,c()},u.strength=function(e){return arguments.length?(o="function"==typeof e?e:ir(+e),c(),u):o},u.distanceMin=function(e){return arguments.length?(a=e*e,u):Math.sqrt(a)},u.distanceMax=function(e){return arguments.length?(s=e*e,u):Math.sqrt(s)},u.theta=function(e){return arguments.length?(l=e*e,u):Math.sqrt(l)},u};function jr(e){return(jr="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function Rr(e){return e.index}function zr(e,t){var n=e.get(t);if(!n)throw new Error("node not found: "+t);return n}var Fr=function(e){var t,n,r,i,o,a,s=Rr,l=function(e){return 1/Math.min(i[e.source.index],i[e.target.index])},u=ir(30),c=1;function d(r){for(var i=0,s=e.length;i<c;++i)for(var l,u,d,h,f,p,g,v=0;v<s;++v)u=(l=e[v]).source,h=(d=l.target).x+d.vx-u.x-u.vx||or(a),f=d.y+d.vy-u.y-u.vy||or(a),h*=p=((p=Math.sqrt(h*h+f*f))-n[v])/p*r*t[v],f*=p,d.vx-=h*(g=o[v]),d.vy-=f*g,u.vx+=h*(g=1-g),u.vy+=f*g}function h(){if(r){var a,l,u=r.length,c=e.length,d=new Map(r.map((function(e,t){return[s(e,t,r),e]})));for(a=0,i=new Array(u);a<c;++a)(l=e[a]).index=a,"object"!==jr(l.source)&&(l.source=zr(d,l.source)),"object"!==jr(l.target)&&(l.target=zr(d,l.target)),i[l.source.index]=(i[l.source.index]||0)+1,i[l.target.index]=(i[l.target.index]||0)+1;for(a=0,o=new Array(c);a<c;++a)l=e[a],o[a]=i[l.source.index]/(i[l.source.index]+i[l.target.index]);t=new Array(c),f(),n=new Array(c),p()}}function f(){if(r)for(var n=0,i=e.length;n<i;++n)t[n]=+l(e[n],n,e)}function p(){if(r)for(var t=0,i=e.length;t<i;++t)n[t]=+u(e[t],t,e)}return null==e&&(e=[]),d.initialize=function(e,t){r=e,a=t,h()},d.links=function(t){return arguments.length?(e=t,h(),d):e},d.id=function(e){return arguments.length?(s=e,d):s},d.iterations=function(e){return arguments.length?(c=+e,d):c},d.strength=function(e){return arguments.length?(l="function"==typeof e?e:ir(+e),f(),d):l},d.distance=function(e){return arguments.length?(u="function"==typeof e?e:ir(+e),p(),d):u},d};function Br(e){return e.x+e.vx}function Hr(e){return e.y+e.vy}var Ur=function(e){var t,n,r,i=1,o=1;function a(){for(var e,a,l,u,c,d,h,f=t.length,p=0;p<o;++p)for(a=er(t,Br,Hr).visitAfter(s),e=0;e<f;++e)l=t[e],d=n[l.index],h=d*d,u=l.x+l.vx,c=l.y+l.vy,a.visit(g);function g(e,t,n,o,a){var s=e.data,f=e.r,p=d+f;if(!s)return t>u+p||o<u-p||n>c+p||a<c-p;if(s.index>l.index){var g=u-s.x-s.vx,v=c-s.y-s.vy,m=g*g+v*v;m<p*p&&(0===g&&(m+=(g=or(r))*g),0===v&&(m+=(v=or(r))*v),m=(p-(m=Math.sqrt(m)))/m*i,l.vx+=(g*=m)*(p=(f*=f)/(h+f)),l.vy+=(v*=m)*p,s.vx-=g*(p=1-p),s.vy-=v*p)}}}function s(e){if(e.data)return e.r=n[e.data.index];for(var t=e.r=0;t<4;++t)e[t]&&e[t].r>e.r&&(e.r=e[t].r)}function l(){if(t){var r,i,o=t.length;for(n=new Array(o),r=0;r<o;++r)i=t[r],n[i.index]=+e(i,r,t)}}return"function"!=typeof e&&(e=ir(null==e?1:+e)),a.initialize=function(e,n){t=e,r=n,l()},a.iterations=function(e){return arguments.length?(o=+e,a):o},a.strength=function(e){return arguments.length?(i=+e,a):i},a.radius=function(t){return arguments.length?(e="function"==typeof t?t:ir(+t),l(),a):e},a},Vr=n(14),Yr=n.n(Vr),Wr="LAYOUT_TICK",Gr=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.center=[0,0],t.nodeStrength=null,t.edgeStrength=null,t.preventOverlap=!1,t.clusterNodeStrength=null,t.clusterEdgeStrength=null,t.clusterEdgeDistance=null,t.clusterNodeSize=null,t.clusterFociStrength=null,t.linkDistance=50,t.alphaDecay=.028,t.alphaMin=.001,t.alpha=.3,t.collideStrength=1,t.workerEnabled=!1,t.tick=function(){},t.onLayoutEnd=function(){},t.onTick=function(){},t.ticking=void 0,t}return Object(w.__extends)(t,e),t.prototype.getDefaultCfg=function(){return{center:[0,0],nodeStrength:null,edgeStrength:null,preventOverlap:!1,nodeSize:void 0,nodeSpacing:void 0,linkDistance:50,forceSimulation:null,alphaDecay:.028,alphaMin:.001,alpha:.3,collideStrength:1,clustering:!1,clusterNodeStrength:-1,clusterEdgeStrength:.1,clusterEdgeDistance:100,clusterFociStrength:.8,clusterNodeSize:10,tick:function(){},onLayoutEnd:function(){},onTick:function(){},workerEnabled:!1}},t.prototype.init=function(e){this.nodes=e.nodes||[];var t=e.edges||[];this.edges=t.map((function(e){var t={},n=["targetNode","sourceNode","startPoint","endPoint"];return Object.keys(e).forEach((function(r){n.indexOf(r)>-1||(t[r]=e[r])})),t})),this.ticking=!1},t.prototype.execute=function(e){var t=this,n=t.nodes,r=t.edges;if(!t.ticking){var i=t.forceSimulation,o=t.alphaMin,a=t.alphaDecay,s=t.alpha;if(i)e&&(t.clustering&&t.clusterForce&&(t.clusterForce.nodes(n),t.clusterForce.links(r)),i.nodes(n),t.edgeForce.links(r)),t.preventOverlap&&t.overlapProcess(i),i.alpha(s).restart(),this.ticking=!0;else try{var l=Nr();if(t.nodeStrength&&l.strength(t.nodeStrength),i=Pr().nodes(n),t.clustering){var u=function(){function e(e){return function(){return e}}var t,n=function(e){return e.cluster},r=e(1),i=e(-1),o=e(100),a=e(.1),s=[0,0],l=[],u={},c=[],d=100,h=100,f={none:{x:0,y:0}},p=[],g="force",v=!0,m=.1;function y(e){if(!v)return y;t.tick(),C();for(var r=0,i=l.length,o=void 0,a=e*m;r<i;++r)(o=l[r]).vx+=(f[n(o)].x-o.x)*a,o.vy+=(f[n(o)].y-o.y)*a}function b(){l&&function(){if(l&&l.length){if(void 0===n(l[0]))throw Error("Couldnt find the grouping attribute for the nodes. Make sure to set it up with forceInABox.groupBy('clusterAttr') before calling .links()");var e,s,f,g,v,m=(s=[],f=[],g={},v=[],e=function(e){var t={};return e.forEach((function(e){var r=n(e);t[r]||(t[r]={count:0,sumforceNodeSize:0})})),e.forEach((function(e){var i=n(e),o=r(e),a=t[i];a.count=a.count+1,a.sumforceNodeSize=a.sumforceNodeSize+Math.PI*(o*o)*1.3,t[i]=a})),t}(l),v=function(e){var t={},r=[];return e.forEach((function(e){var r=function(e){var t=n(u[e.source]),r=n(u[e.target]);return t<=r?t+"~"+r:r+"~"+t}(e),i=0;void 0!==t[r]&&(i=t[r]),i+=1,t[r]=i})),Object.entries(t).forEach((function(e){var t=e[0],n=e[1],i=t.split("~")[0],o=t.split("~")[1];void 0!==i&&void 0!==o&&r.push({source:i,target:o,count:n})})),r}(c),Object.keys(e).forEach((function(t,n){var r=e[t];s.push({id:t,size:r.count,r:Math.sqrt(r.sumforceNodeSize/Math.PI)}),g[t]=n})),v.forEach((function(e){var t=g[e.source],n=g[e.target];void 0!==t&&void 0!==n&&f.push({source:t,target:n,count:e.count})})),{nodes:s,links:f});t=Pr(m.nodes).force("x",function(e){var t,n,r,i=ir(.1);function o(e){for(var i,o=0,a=t.length;o<a;++o)(i=t[o]).vx+=(r[o]-i.x)*n[o]*e}function a(){if(t){var o,a=t.length;for(n=new Array(a),r=new Array(a),o=0;o<a;++o)n[o]=isNaN(r[o]=+e(t[o],o,t))?0:+i(t[o],o,t)}}return"function"!=typeof e&&(e=ir(null==e?0:+e)),o.initialize=function(e){t=e,a()},o.strength=function(e){return arguments.length?(i="function"==typeof e?e:ir(+e),a(),o):i},o.x=function(t){return arguments.length?(e="function"==typeof t?t:ir(+t),a(),o):e},o}(d).strength(.1)).force("y",function(e){var t,n,r,i=ir(.1);function o(e){for(var i,o=0,a=t.length;o<a;++o)(i=t[o]).vy+=(r[o]-i.y)*n[o]*e}function a(){if(t){var o,a=t.length;for(n=new Array(a),r=new Array(a),o=0;o<a;++o)n[o]=isNaN(r[o]=+e(t[o],o,t))?0:+i(t[o],o,t)}}return"function"!=typeof e&&(e=ir(null==e?0:+e)),o.initialize=function(e){t=e,a()},o.strength=function(e){return arguments.length?(i="function"==typeof e?e:ir(+e),a(),o):i},o.y=function(t){return arguments.length?(e="function"==typeof t?t:ir(+t),a(),o):e},o}(h).strength(.1)).force("collide",Ur((function(e){return e.r})).iterations(4)).force("charge",Nr().strength(i)).force("links",Fr(m.nodes.length?m.links:[]).distance(o).strength(a)),p=t.nodes(),C()}}()}function C(){return f={none:{x:0,y:0}},p.forEach((function(e){f[e.id]={x:e.x-s[0],y:e.y-s[1]}})),f}function A(e){u={},e.forEach((function(e){u[e.id]=e}))}return y.initialize=function(e){l=e,b()},y.template=function(e){return arguments.length?(g=e,b(),y):g},y.groupBy=function(e){return arguments.length?"string"==typeof e?(n=function(t){return t[e]},y):(n=e,y):n},y.enableGrouping=function(e){return arguments.length?(v=e,y):v},y.strength=function(e){return arguments.length?(m=e,y):m},y.centerX=function(e){return arguments.length?(d=e,y):d},y.centerY=function(e){return arguments.length?(h=e,y):h},y.nodes=function(e){return arguments.length?(A(e||[]),l=e||[],y):l},y.links=function(e){return arguments.length?(c=e||[],b(),y):c},y.forceNodeSize=function(t){return arguments.length?(r="function"==typeof t?t:e(+t),b(),y):r},y.nodeSize=y.forceNodeSize,y.forceCharge=function(t){return arguments.length?(i="function"==typeof t?t:e(+t),b(),y):i},y.forceLinkDistance=function(t){return arguments.length?(o="function"==typeof t?t:e(+t),b(),y):o},y.forceLinkStrength=function(t){return arguments.length?(a="function"==typeof t?t:e(+t),b(),y):a},y.offset=function(e){return arguments.length?(s=e,y):s},y.getFocis=C,y}();u.centerX(t.center[0]).centerY(t.center[1]).template("force").strength(t.clusterFociStrength),r&&u.links(r),n&&u.nodes(n),u.forceLinkDistance(t.clusterEdgeDistance).forceLinkStrength(t.clusterEdgeStrength).forceCharge(t.clusterNodeStrength).forceNodeSize(t.clusterNodeSize),t.clusterForce=u,i.force("group",u)}if(i.force("center",function(e,t){var n,r=1;function i(){var i,o,a=n.length,s=0,l=0;for(i=0;i<a;++i)s+=(o=n[i]).x,l+=o.y;for(s=(s/a-e)*r,l=(l/a-t)*r,i=0;i<a;++i)(o=n[i]).x-=s,o.y-=l}return null==e&&(e=0),null==t&&(t=0),i.initialize=function(e){n=e},i.x=function(t){return arguments.length?(e=+t,i):e},i.y=function(e){return arguments.length?(t=+e,i):t},i.strength=function(e){return arguments.length?(r=+e,i):r},i}(t.center[0],t.center[1])).force("charge",l).alpha(s).alphaDecay(a).alphaMin(o),t.preventOverlap&&t.overlapProcess(i),r){var c=Fr().id((function(e){return e.id})).links(r);t.edgeStrength&&c.strength(t.edgeStrength),t.linkDistance&&c.distance(t.linkDistance),t.edgeForce=c,i.force("link",c)}if(t.workerEnabled&&!("undefined"!=typeof WorkerGlobalScope&&self instanceof WorkerGlobalScope)&&(t.workerEnabled=!1,console.warn("workerEnabled option is only supported when running in web worker.")),t.workerEnabled){i.stop();for(var d=function(e){var t=e.alphaMin(),n=e.alphaTarget(),r=e.alpha(),i=Math.log((t-n)/(r-n))/Math.log(1-e.alphaDecay());return Math.ceil(i)}(i),h=1;h<=d;h++)i.tick(),postMessage({type:Wr,currentTick:h,totalTicks:d,nodes:n},void 0);t.ticking=!1}else i.on("tick",(function(){t.tick()})).on("end",(function(){t.ticking=!1,t.onLayoutEnd&&t.onLayoutEnd()})),t.ticking=!0;t.forceSimulation=i,t.ticking=!0}catch(e){t.ticking=!1,console.warn(e)}}},t.prototype.overlapProcess=function(e){var t,n,r=this.nodeSize,i=this.nodeSpacing,o=this.collideStrength;if(n=_()(i)?function(){return i}:Yr()(i)?i:function(){return 0},r)if(Yr()(r))t=function(e){return r(e)+n(e)};else if(y()(r)){var a=(r[0]>r[1]?r[0]:r[1])/2;t=function(e){return a+n(e)}}else if(_()(r)){var s=r/2;t=function(e){return s+n(e)}}else t=function(){return 10};else t=function(e){return e.size?y()(e.size)?(e.size[0]>e.size[1]?e.size[0]:e.size[1])/2+n(e):e.size/2+n(e):10+n(e)};e.force("collisionForce",Ur(t).strength(o))},t.prototype.updateCfg=function(e){this.ticking&&(this.forceSimulation.stop(),this.ticking=!1),this.forceSimulation=null,Nn()(this,e)},t.prototype.destroy=function(){this.ticking&&(this.forceSimulation.stop(),this.ticking=!1),this.nodes=null,this.edges=null,this.destroyed=!0},t}(Bn),Xr=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.center=[0,0],t.maxIteration=500,t.gravity=10,t.clustering=!1,t.clusterGravity=10,t.linkDistance=50,t.alpha=1,t.alphaMin=.001,t.alphaDecay=1-Math.pow(t.alphaMin,1/300),t.alphaTarget=0,t.velocityDecay=.6,t.linkStrength=1,t.nodeStrength=30,t.preventOverlap=!1,t.collideStrength=1,t.optimizeRangeFactor=1,t.tick=function(){},t.nodes=[],t.edges=[],t.width=300,t.height=300,t.bias=[],t.nodeMap={},t.nodeIdxMap={},t}return Object(w.__extends)(t,e),t.prototype.getDefaultCfg=function(){return{maxIteration:1e3,center:[0,0],gravity:10,clustering:!1,clusterGravity:10,preventOverlap:!1,nodeSpacing:void 0,collideStrength:10}},t.prototype.execute=function(){var e=this.nodes,t=this.center;if(e&&0!==e.length){if(1===e.length)return e[0].x=t[0],void(e[0].y=t[1]);var n={},r={};e.forEach((function(e,t){n[e.id]=e,r[e.id]=t})),this.nodeMap=n,this.nodeIdxMap=r,this.run()}},t.prototype.run=function(){var e=this,t=e.nodes,n=e.edges,r=e.maxIteration;e.width||"undefined"==typeof window||(e.width=window.innerWidth),e.height||"undefined"==typeof window||(e.height=window.innerHeight);var i,o=e.center,a=e.velocityDecay,s=e.clustering;e.initVals(),s&&(i=e.getClusterMap());for(var l=function(r){var o=[];t.forEach((function(e,t){o[t]={x:0,y:0}})),e.applyCalculate(t,n,o),s&&e.applyClusterForce(i,o),t.forEach((function(e,t){Object(ce.isNumber)(e.x)&&Object(ce.isNumber)(e.y)&&(e.x+=o[t].x*a,e.y+=o[t].y*a)})),u.alpha+=(u.alphaTarget-u.alpha)*u.alphaDecay,e.tick()},u=this,c=0;c<r;c++)l();var d=[0,0];t.forEach((function(e){Object(ce.isNumber)(e.x)&&Object(ce.isNumber)(e.y)&&(d[0]+=e.x,d[1]+=e.y)})),d[0]/=t.length,d[1]/=t.length;var h=[o[0]-d[0],o[1]-d[1]];t.forEach((function(e,t){Object(ce.isNumber)(e.x)&&Object(ce.isNumber)(e.y)&&(e.x+=h[0],e.y+=h[1])}))},t.prototype.initVals=function(){for(var e=this.edges,t={},n=0;n<e.length;++n)t[e[n].source]?t[e[n].source]++:t[e[n].source]=1,t[e[n].target]?t[e[n].target]++:t[e[n].target]=1;var r=[];for(n=0;n<e.length;++n)r[n]=t[e[n].source]/(t[e[n].source]+t[e[n].target]);this.bias=r;var i,o,a=this.nodeSize,s=this.nodeSpacing;if(o=Object(ce.isNumber)(s)?function(){return s}:Object(ce.isFunction)(s)?s:function(){return 0},a)if(Object(ce.isFunction)(a))i=function(e){return a(e)+o(e)};else if(Object(ce.isArray)(a)){var l=(a[0]>a[1]?a[0]:a[1])/2;i=function(e){return l+o(e)}}else if(Object(ce.isNumber)(a)){var u=a/2;i=function(e){return u+o(e)}}else i=function(){return 10};else i=function(e){return e.size?Object(ce.isArray)(e.size)?(e.size[0]>e.size[1]?e.size[0]:e.size[1])/2+o(e):e.size/2+o(e):10+o(e)};this.nodeSize=i;var c,d=this.linkDistance;d||(d=50),Object(ce.isNumber)(d)&&(c=function(e){return d}),this.linkDistance=c;var h,f=this.linkStrength;f||(f=1),Object(ce.isNumber)(f)&&(h=function(e){return f}),this.linkStrength=h;var p,g=this.nodeStrength;g||(g=30),Object(ce.isNumber)(g)&&(p=function(e){return g}),this.nodeStrength=p},t.prototype.getClusterMap=function(){var e=this.nodes,t={};for(var n in e.forEach((function(e){if(void 0===t[e.cluster]){var n={name:e.cluster,cx:0,cy:0,count:0};t[e.cluster]=n}var r=t[e.cluster];Object(ce.isNumber)(e.x)&&(r.cx+=e.x),Object(ce.isNumber)(e.y)&&(r.cy+=e.y),r.count++})),t)t[n].cx/=t[n].count,t[n].cy/=t[n].count;return t},t.prototype.applyClusterForce=function(e,t){var n=this.gravity,r=this.nodes,i=this.clusterGravity||n,o=this.alpha;for(var a in r.forEach((function(n,r){if(Object(ce.isNumber)(n.x)&&Object(ce.isNumber)(n.y)){var a=e[n.cluster],s=n.x-a.cx,l=n.y-a.cy,u=Math.sqrt(s*s+l*l);t[r].x-=s*i*o/u,t[r].y-=l*i*o/u}})),e)e[a].cx=0,e[a].cy=0,e[a].count=0;for(var a in r.forEach((function(t){var n=e[t.cluster];Object(ce.isNumber)(t.x)&&(n.cx+=t.x),Object(ce.isNumber)(t.y)&&(n.cy+=t.y),n.count++})),e)e[a].cx/=e[a].count,e[a].cy/=e[a].count},t.prototype.applyCalculate=function(e,t,n){var r={};e.forEach((function(t,i){n[i]={x:0,y:0},e.forEach((function(e,n){if(!(i<n)){var o=t.x-e.x,a=t.y-e.y,s=o*o+a*a;s<1&&(s=Math.sqrt(s)),0===o&&(s+=(o=.01*Math.random())*o),0===a&&(s+=(a=.01*Math.random())*a);var l=Math.sqrt(s);r[t.id+"-"+e.id]={vx:o,vy:a,vl:s,sqrtVl:l},r[e.id+"-"+t.id]={vx:-o,vy:-a,vl:s,sqrtVl:l}}}))})),this.calRepulsive(e,n,r),this.calAttractive(t,n,r)},t.prototype.calRepulsive=function(e,t,n){var r=this.width*this.optimizeRangeFactor*this.width*this.optimizeRangeFactor,i=this.nodeStrength,o=this.alpha,a=this.collideStrength,s=this.preventOverlap,l=this.nodeSize;e.forEach((function(u,c){e.forEach((function(e,d){if(c!==d&&Object(ce.isNumber)(u.x)&&Object(ce.isNumber)(e.x)&&Object(ce.isNumber)(u.y)&&Object(ce.isNumber)(e.y)){var h=n[u.id+"-"+e.id],f=h.vl,p=h.vx,g=h.vy;if(!(f>r)&&(t[c].x+=p*i(e)*o/f,t[c].y+=g*i(e)*o/f,s&&c<d)){var v=l(u),m=l(e),y=v+m;if(f<y*y){var b=n[u.id+"-"+e.id].sqrtVl,C=(y-b)/b*a,A=m*m/(v*v+m*m),_=p*C,I=g*C;t[c].x+=_*A,t[c].y+=I*A,A=1-A,t[d].x-=_*A,t[d].y-=I*A}}}}))}))},t.prototype.calAttractive=function(e,t,n){var r=this,i=this.linkDistance,o=this.alpha,a=this.linkStrength,s=this.bias;e.forEach((function(e,l){if(e.source&&e.target){var u=r.nodeIdxMap[e.source],c=r.nodeIdxMap[e.target];if(u!==c){var d=r.nodeMap[e.source],h=r.nodeMap[e.target];if(Object(ce.isNumber)(h.x)&&Object(ce.isNumber)(d.x)&&Object(ce.isNumber)(h.y)&&Object(ce.isNumber)(d.y)){var f=n[e.target+"-"+e.source],p=(f.vl,f.sqrtVl),g=f.vx,v=f.vy,m=(p-i(e))/p*o*a(e),y=g*m,b=v*m,C=s[l];t[c].x-=y*C,t[c].y-=b*C,t[u].x+=y*(1-C),t[u].y+=b*(1-C)}}}}))},t}(Bn),qr=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.center=[0,0],t.maxIteration=1e3,t.gravity=10,t.speed=1,t.clustering=!1,t.clusterGravity=10,t.nodes=[],t.edges=[],t.width=300,t.height=300,t.nodeMap={},t.nodeIdxMap={},t}return Object(w.__extends)(t,e),t.prototype.getDefaultCfg=function(){return{maxIteration:1e3,center:[0,0],gravity:10,speed:1,clustering:!1,clusterGravity:10}},t.prototype.execute=function(){var e=this,t=this.nodes,n=this.center;if(t&&0!==t.length){if(1===t.length)return t[0].x=n[0],void(t[0].y=n[1]);var r={},i={};t.forEach((function(t,n){Object(ce.isNumber)(t.x)||(t.x=Math.random()*e.width),Object(ce.isNumber)(t.y)||(t.y=Math.random()*e.height),r[t.id]=t,i[t.id]=n})),this.nodeMap=r,this.nodeIdxMap=i,this.run()}},t.prototype.run=function(){var e=this,t=e.nodes,n=e.edges,r=e.maxIteration;e.width||"undefined"==typeof window||(e.width=window.innerWidth),e.height||"undefined"==typeof window||(e.height=window.innerHeight);var i=e.center,o=e.width/10,a=Math.sqrt(e.width*e.height/(t.length+1)),s=e.gravity,l=e.speed,u=e.clustering,c={};if(u)for(var d in t.forEach((function(e){if(void 0===c[e.cluster]){var t={name:e.cluster,cx:0,cy:0,count:0};c[e.cluster]=t}var n=c[e.cluster];Object(ce.isNumber)(e.x)&&(n.cx+=e.x),Object(ce.isNumber)(e.y)&&(n.cy+=e.y),n.count++})),c)c[d].cx/=c[d].count,c[d].cy/=c[d].count;for(var h=function(r){var d=[];if(t.forEach((function(e,t){d[t]={x:0,y:0}})),e.applyCalculate(t,n,d,a),u){var h=e.clusterGravity||s;for(var f in t.forEach((function(e,t){if(Object(ce.isNumber)(e.x)&&Object(ce.isNumber)(e.y)){var n=c[e.cluster],r=Math.sqrt((e.x-n.cx)*(e.x-n.cx)+(e.y-n.cy)*(e.y-n.cy)),i=a*h;d[t].x-=i*(e.x-n.cx)/r,d[t].y-=i*(e.y-n.cy)/r}})),c)c[f].cx=0,c[f].cy=0,c[f].count=0;for(var f in t.forEach((function(e){var t=c[e.cluster];Object(ce.isNumber)(e.x)&&(t.cx+=e.x),Object(ce.isNumber)(e.y)&&(t.cy+=e.y),t.count++})),c)c[f].cx/=c[f].count,c[f].cy/=c[f].count}t.forEach((function(e,t){if(Object(ce.isNumber)(e.x)&&Object(ce.isNumber)(e.y)){var n=.01*a*s;d[t].x-=n*(e.x-i[0]),d[t].y-=n*(e.y-i[1])}})),t.forEach((function(e,t){if(Object(ce.isNumber)(e.x)&&Object(ce.isNumber)(e.y)){var n=Math.sqrt(d[t].x*d[t].x+d[t].y*d[t].y);if(n>0){var r=Math.min(o*(l/800),n);e.x+=d[t].x/n*r,e.y+=d[t].y/n*r}}}))},f=0;f<r;f++)h()},t.prototype.applyCalculate=function(e,t,n,r){this.calRepulsive(e,n,r),this.calAttractive(t,n,r)},t.prototype.calRepulsive=function(e,t,n){e.forEach((function(r,i){t[i]={x:0,y:0},e.forEach((function(e,o){if(i!==o&&Object(ce.isNumber)(r.x)&&Object(ce.isNumber)(e.x)&&Object(ce.isNumber)(r.y)&&Object(ce.isNumber)(e.y)){var a=r.x-e.x,s=r.y-e.y,l=a*a+s*s;if(0===l){l=1;var u=i>o?1:-1;a=.01*u,s=.01*u}var c=n*n/l;t[i].x+=a*c,t[i].y+=s*c}}))}))},t.prototype.calAttractive=function(e,t,n){var r=this;e.forEach((function(e){if(e.source&&e.target){var i=r.nodeIdxMap[e.source],o=r.nodeIdxMap[e.target];if(i!==o){var a=r.nodeMap[e.source],s=r.nodeMap[e.target];if(Object(ce.isNumber)(s.x)&&Object(ce.isNumber)(a.x)&&Object(ce.isNumber)(s.y)&&Object(ce.isNumber)(a.y)){var l=s.x-a.x,u=s.y-a.y,c=Math.sqrt(l*l+u*u),d=c*c/n;t[o].x-=l/c*d,t[o].y-=u/c*d,t[i].x+=l/c*d,t[i].y+=u/c*d}}}}))},t}(Bn),Zr=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.begin=[0,0],t.preventOverlap=!0,t.preventOverlapPadding=10,t.condense=!1,t.sortBy="degree",t.nodeSize=30,t.nodes=[],t.edges=[],t.center=[0,0],t.width=300,t.height=300,t.row=0,t.col=0,t.cellWidth=0,t.cellHeight=0,t.cellUsed={},t.id2manPos={},t}return Object(w.__extends)(t,e),t.prototype.getDefaultCfg=function(){return{begin:[0,0],preventOverlap:!0,preventOverlapPadding:10,condense:!1,rows:void 0,cols:void 0,position:void 0,sortBy:"degree",nodeSize:30}},t.prototype.execute=function(){var e=this,t=e.nodes,n=t.length,r=e.center;if(0!==n){if(1===n)return t[0].x=r[0],void(t[0].y=r[1]);var i=e.edges,o=[];t.forEach((function(e){o.push(e)}));var a={};if(o.forEach((function(e,t){a[e.id]=t})),("degree"===e.sortBy||!x()(e.sortBy)||void 0===o[0][e.sortBy])&&(e.sortBy="degree",k(t[0].degree))){var s=Ke(o.length,a,i);o.forEach((function(e,t){e.degree=s[t]}))}o.sort((function(t,n){return n[e.sortBy]-t[e.sortBy]})),e.width||"undefined"==typeof window||(e.width=window.innerWidth),e.height||"undefined"==typeof window||(e.height=window.innerHeight);var l=e.rows,u=null!=e.cols?e.cols:e.columns;if(e.cells=n,null!=l&&null!=u?(e.rows=l,e.cols=u):null!=l&&null==u?(e.rows=l,e.cols=Math.ceil(e.cells/e.rows)):null==l&&null!=u?(e.cols=u,e.rows=Math.ceil(e.cells/e.cols)):(e.splits=Math.sqrt(e.cells*e.height/e.width),e.rows=Math.round(e.splits),e.cols=Math.round(e.width/e.height*e.splits)),e.cols*e.rows>e.cells)((d=e.small())-1)*(c=e.large())>=e.cells?e.small(d-1):(c-1)*d>=e.cells&&e.large(c-1);else for(;e.cols*e.rows<e.cells;){var c,d=e.small();((c=e.large())+1)*d>=e.cells?e.large(c+1):e.small(d+1)}e.cellWidth=e.width/e.cols,e.cellHeight=e.height/e.rows,e.condense&&(e.cellWidth=0,e.cellHeight=0),e.preventOverlap&&o.forEach((function(t){var n,r;t.x&&t.y||(t.x=0,t.y=0),Object(ce.isArray)(t.size)?(n=t.size[0],r=t.size[1]):Object(ce.isNumber)(t.size)&&(n=t.size,r=t.size),void 0!==n&&void 0!==r||(Object(ce.isArray)(e.nodeSize)?(n=e.nodeSize[0],r=e.nodeSize[1]):Object(ce.isNumber)(e.nodeSize)?(n=e.nodeSize,r=e.nodeSize):(n=30,r=30));var i=e.preventOverlapPadding,o=n+i,a=r+i;e.cellWidth=Math.max(e.cellWidth,o),e.cellHeight=Math.max(e.cellHeight,a)})),e.cellUsed={},e.row=0,e.col=0,e.id2manPos={};for(var h=0;h<o.length;h++){var f=o[h],p=void 0;if(e.position&&(p=e.position(f)),p&&(void 0!==p.row||void 0!==p.col)){var g={row:p.row,col:p.col};if(void 0===g.col)for(g.col=0;e.used(g.row,g.col);)g.col++;else if(void 0===g.row)for(g.row=0;e.used(g.row,g.col);)g.row++;e.id2manPos[f.id]=g,e.use(g.row,g.col)}e.getPos(f)}}},t.prototype.small=function(e){var t,n=this.rows||5,r=this.cols||5;return null==e?t=Math.min(n,r):Math.min(n,r)===this.rows?this.rows=e:this.cols=e,t},t.prototype.large=function(e){var t,n=this.rows||5,r=this.cols||5;return null==e?t=Math.max(n,r):Math.max(n,r)===this.rows?this.rows=e:this.cols=e,t},t.prototype.used=function(e,t){return this.cellUsed["c-"+e+"-"+t]||!1},t.prototype.use=function(e,t){this.cellUsed["c-"+e+"-"+t]=!0},t.prototype.moveToNextCell=function(){var e=this.cols||5;this.col++,this.col>=e&&(this.col=0,this.row++)},t.prototype.getPos=function(e){var t,n,r=this.begin,i=this.cellWidth,o=this.cellHeight,a=this.id2manPos[e.id];if(a)t=a.col*i+i/2+r[0],n=a.row*o+o/2+r[1];else{for(;this.used(this.row,this.col);)this.moveToNextCell();t=this.col*i+i/2+r[0],n=this.row*o+o/2+r[1],this.use(this.row,this.col),this.moveToNextCell()}e.x=t,e.y=n},t}(Bn),Qr=Object.prototype.toString;function Kr(e){return Qr.call(e).endsWith("Array]")}var Jr=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};if(!Kr(e))throw new TypeError("input must be an array");if(0===e.length)throw new TypeError("input must not be empty");var n=t.fromIndex,r=void 0===n?0:n,i=t.toIndex,o=void 0===i?e.length:i;if(r<0||r>=e.length||!Number.isInteger(r))throw new Error("fromIndex must be a positive integer smaller than length");if(o<=r||o>e.length||!Number.isInteger(o))throw new Error("toIndex must be an integer greater than fromIndex and at most equal to length");for(var a=e[r],s=r+1;s<o;s++)e[s]>a&&(a=e[s]);return a},$r=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};if(!Kr(e))throw new TypeError("input must be an array");if(0===e.length)throw new TypeError("input must not be empty");var n=t.fromIndex,r=void 0===n?0:n,i=t.toIndex,o=void 0===i?e.length:i;if(r<0||r>=e.length||!Number.isInteger(r))throw new Error("fromIndex must be a positive integer smaller than length");if(o<=r||o>e.length||!Number.isInteger(o))throw new Error("toIndex must be an integer greater than fromIndex and at most equal to length");for(var a=e[r],s=r+1;s<o;s++)e[s]<a&&(a=e[s]);return a},ei=function(e){var t,n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};if(!Kr(e))throw new TypeError("input must be an array");if(0===e.length)throw new TypeError("input must not be empty");if(void 0!==n.output){if(!Kr(n.output))throw new TypeError("output option must be an array if specified");t=n.output}else t=new Array(e.length);var r=$r(e),i=Jr(e);if(r===i)throw new RangeError("minimum and maximum input values are equal. Cannot rescale a constant array");var o=n.min,a=void 0===o?n.autoMinMax?r:0:o,s=n.max,l=void 0===s?n.autoMinMax?i:1:s;if(a>=l)throw new RangeError("min option must be smaller than max option");for(var u=(l-a)/(i-r),c=0;c<e.length;c++)t[c]=(e[c]-r)*u+a;return t},ti=" ".repeat(2),ni=" ".repeat(4);function ri(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=t.maxRows,r=void 0===n?15:n,i=t.maxColumns,o=void 0===i?10:i,a=t.maxNumSize,s=void 0===a?8:a;return"".concat(e.constructor.name," {\n").concat(ti,"[\n").concat(ni).concat(ii(e,r,o,s),"\n").concat(ti,"]\n").concat(ti,"rows: ").concat(e.rows,"\n").concat(ti,"columns: ").concat(e.columns,"\n}")}function ii(e,t,n,r){for(var i=e.rows,o=e.columns,a=Math.min(i,t),s=Math.min(o,n),l=[],u=0;u<a;u++){for(var c=[],d=0;d<s;d++)c.push(oi(e.get(u,d),r));l.push("".concat(c.join(" ")))}return s!==o&&(l[l.length-1]+=" ... ".concat(o-n," more columns")),a!==i&&l.push("... ".concat(i-t," more rows")),l.join("\n".concat(ni))}function oi(e,t){var n=String(e);if(n.length<=t)return n.padEnd(t," ");var r=e.toPrecision(t-2);if(r.length<=t)return r;var i=e.toExponential(t-2),o=i.indexOf("e"),a=i.slice(o);return i.slice(0,t-a.length)+a}function ai(e){return(ai="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function si(e,t,n){var r=n?e.rows:e.rows-1;if(t<0||t>r)throw new RangeError("Row index out of range")}function li(e,t,n){var r=n?e.columns:e.columns-1;if(t<0||t>r)throw new RangeError("Column index out of range")}function ui(e,t){if(t.to1DArray&&(t=t.to1DArray()),t.length!==e.columns)throw new RangeError("vector size must be the same as the number of columns");return t}function ci(e,t){if(t.to1DArray&&(t=t.to1DArray()),t.length!==e.rows)throw new RangeError("vector size must be the same as the number of rows");return t}function di(e,t){if("object"!==ai(t))throw new TypeError("unexpected type for row indices");if(t.some((function(t){return t<0||t>=e.rows})))throw new RangeError("row indices are out of range");return Array.isArray(t)||(t=Array.from(t)),t}function hi(e,t){if("object"!==ai(t))throw new TypeError("unexpected type for column indices");if(t.some((function(t){return t<0||t>=e.columns})))throw new RangeError("column indices are out of range");return Array.isArray(t)||(t=Array.from(t)),t}function fi(e,t,n,r,i){if(5!==arguments.length)throw new RangeError("expected 4 arguments");if(gi("startRow",t),gi("endRow",n),gi("startColumn",r),gi("endColumn",i),t>n||r>i||t<0||t>=e.rows||n<0||n>=e.rows||r<0||r>=e.columns||i<0||i>=e.columns)throw new RangeError("Submatrix indices are out of range")}function pi(e){for(var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:0,n=[],r=0;r<e;r++)n.push(t);return n}function gi(e,t){if("number"!=typeof t)throw new TypeError("".concat(e," must be a number"))}function vi(e,t,n){for(var r=e.rows,i=e.columns,o=[],a=0;a<r;a++){for(var s=0,l=0,u=0,c=0;c<i;c++)s+=u=e.get(a,c)-n[a],l+=u*u;t?o.push((l-s*s/i)/(i-1)):o.push((l-s*s/i)/i)}return o}function mi(e,t,n){for(var r=e.rows,i=e.columns,o=[],a=0;a<i;a++){for(var s=0,l=0,u=0,c=0;c<r;c++)s+=u=e.get(c,a)-n[a],l+=u*u;t?o.push((l-s*s/r)/(r-1)):o.push((l-s*s/r)/r)}return o}function yi(e,t,n){for(var r=e.rows,i=e.columns,o=r*i,a=0,s=0,l=0,u=0;u<r;u++)for(var c=0;c<i;c++)a+=l=e.get(u,c)-n,s+=l*l;return t?(s-a*a/o)/(o-1):(s-a*a/o)/o}function bi(e,t){for(var n=0;n<e.rows;n++)for(var r=0;r<e.columns;r++)e.set(n,r,e.get(n,r)-t[n])}function Ci(e,t){for(var n=0;n<e.rows;n++)for(var r=0;r<e.columns;r++)e.set(n,r,e.get(n,r)-t[r])}function Ai(e,t){for(var n=0;n<e.rows;n++)for(var r=0;r<e.columns;r++)e.set(n,r,e.get(n,r)-t)}function _i(e){for(var t=[],n=0;n<e.rows;n++){for(var r=0,i=0;i<e.columns;i++)r+=Math.pow(e.get(n,i),2)/(e.columns-1);t.push(Math.sqrt(r))}return t}function Ii(e,t){for(var n=0;n<e.rows;n++)for(var r=0;r<e.columns;r++)e.set(n,r,e.get(n,r)/t[n])}function xi(e){for(var t=[],n=0;n<e.columns;n++){for(var r=0,i=0;i<e.rows;i++)r+=Math.pow(e.get(i,n),2)/(e.rows-1);t.push(Math.sqrt(r))}return t}function wi(e,t){for(var n=0;n<e.rows;n++)for(var r=0;r<e.columns;r++)e.set(n,r,e.get(n,r)/t[r])}function Mi(e){for(var t=e.size-1,n=0,r=0;r<e.columns;r++)for(var i=0;i<e.rows;i++)n+=Math.pow(e.get(i,r),2)/t;return Math.sqrt(n)}function Si(e,t){for(var n=0;n<e.rows;n++)for(var r=0;r<e.columns;r++)e.set(n,r,e.get(n,r)/t)}function Ti(e,t){return(Ti=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function Ei(e,t){return!t||"object"!==ki(t)&&"function"!=typeof t?function(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}(e):t}function Oi(e){return(Oi=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}function ki(e){return(ki="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function Li(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function Di(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}function Pi(e,t,n){return t&&Di(e.prototype,t),n&&Di(e,n),e}var Ni=function(){function e(){Li(this,e)}return Pi(e,[{key:"apply",value:function(e){if("function"!=typeof e)throw new TypeError("callback must be a function");for(var t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)e.call(this,t,n);return this}},{key:"to1DArray",value:function(){for(var e=[],t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)e.push(this.get(t,n));return e}},{key:"to2DArray",value:function(){for(var e=[],t=0;t<this.rows;t++){e.push([]);for(var n=0;n<this.columns;n++)e[t].push(this.get(t,n))}return e}},{key:"toJSON",value:function(){return this.to2DArray()}},{key:"isRowVector",value:function(){return 1===this.rows}},{key:"isColumnVector",value:function(){return 1===this.columns}},{key:"isVector",value:function(){return 1===this.rows||1===this.columns}},{key:"isSquare",value:function(){return this.rows===this.columns}},{key:"isSymmetric",value:function(){if(this.isSquare()){for(var e=0;e<this.rows;e++)for(var t=0;t<=e;t++)if(this.get(e,t)!==this.get(t,e))return!1;return!0}return!1}},{key:"isEchelonForm",value:function(){for(var e=0,t=0,n=-1,r=!0,i=!1;e<this.rows&&r;){for(t=0,i=!1;t<this.columns&&!1===i;)0===this.get(e,t)?t++:1===this.get(e,t)&&t>n?(i=!0,n=t):(r=!1,i=!0);e++}return r}},{key:"isReducedEchelonForm",value:function(){for(var e=0,t=0,n=-1,r=!0,i=!1;e<this.rows&&r;){for(t=0,i=!1;t<this.columns&&!1===i;)0===this.get(e,t)?t++:1===this.get(e,t)&&t>n?(i=!0,n=t):(r=!1,i=!0);for(var o=t+1;o<this.rows;o++)0!==this.get(e,o)&&(r=!1);e++}return r}},{key:"echelonForm",value:function(){for(var e=this.clone(),t=0,n=0;t<e.rows&&n<e.columns;){for(var r=t,i=t;i<e.rows;i++)e.get(i,n)>e.get(r,n)&&(r=i);if(0===e.get(r,n))n++;else{e.swapRows(t,r);for(var o=e.get(t,n),a=n;a<e.columns;a++)e.set(t,a,e.get(t,a)/o);for(var s=t+1;s<e.rows;s++){var l=e.get(s,n)/e.get(t,n);e.set(s,n,0);for(var u=n+1;u<e.columns;u++)e.set(s,u,e.get(s,u)-e.get(t,u)*l)}t++,n++}}return e}},{key:"reducedEchelonForm",value:function(){for(var e=this.echelonForm(),t=e.columns,n=e.rows,r=n-1;r>=0;)if(0===e.maxRow(r))r--;else{for(var i=0,o=!1;i<n&&!1===o;)1===e.get(r,i)?o=!0:i++;for(var a=0;a<r;a++)for(var s=e.get(a,i),l=i;l<t;l++){var u=e.get(a,l)-s*e.get(r,l);e.set(a,l,u)}r--}return e}},{key:"set",value:function(){throw new Error("set method is unimplemented")}},{key:"get",value:function(){throw new Error("get method is unimplemented")}},{key:"repeat",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};if("object"!==ki(e))throw new TypeError("options must be an object");var t=e.rows,n=void 0===t?1:t,r=e.columns,i=void 0===r?1:r;if(!Number.isInteger(n)||n<=0)throw new TypeError("rows must be a positive integer");if(!Number.isInteger(i)||i<=0)throw new TypeError("columns must be a positive integer");for(var o=new Fi(this.rows*n,this.columns*i),a=0;a<n;a++)for(var s=0;s<i;s++)o.setSubMatrix(this,this.rows*a,this.columns*s);return o}},{key:"fill",value:function(e){for(var t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)this.set(t,n,e);return this}},{key:"neg",value:function(){return this.mulS(-1)}},{key:"getRow",value:function(e){si(this,e);for(var t=[],n=0;n<this.columns;n++)t.push(this.get(e,n));return t}},{key:"getRowVector",value:function(e){return Fi.rowVector(this.getRow(e))}},{key:"setRow",value:function(e,t){si(this,e),t=ui(this,t);for(var n=0;n<this.columns;n++)this.set(e,n,t[n]);return this}},{key:"swapRows",value:function(e,t){si(this,e),si(this,t);for(var n=0;n<this.columns;n++){var r=this.get(e,n);this.set(e,n,this.get(t,n)),this.set(t,n,r)}return this}},{key:"getColumn",value:function(e){li(this,e);for(var t=[],n=0;n<this.rows;n++)t.push(this.get(n,e));return t}},{key:"getColumnVector",value:function(e){return Fi.columnVector(this.getColumn(e))}},{key:"setColumn",value:function(e,t){li(this,e),t=ci(this,t);for(var n=0;n<this.rows;n++)this.set(n,e,t[n]);return this}},{key:"swapColumns",value:function(e,t){li(this,e),li(this,t);for(var n=0;n<this.rows;n++){var r=this.get(n,e);this.set(n,e,this.get(n,t)),this.set(n,t,r)}return this}},{key:"addRowVector",value:function(e){e=ui(this,e);for(var t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)this.set(t,n,this.get(t,n)+e[n]);return this}},{key:"subRowVector",value:function(e){e=ui(this,e);for(var t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)this.set(t,n,this.get(t,n)-e[n]);return this}},{key:"mulRowVector",value:function(e){e=ui(this,e);for(var t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)this.set(t,n,this.get(t,n)*e[n]);return this}},{key:"divRowVector",value:function(e){e=ui(this,e);for(var t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)this.set(t,n,this.get(t,n)/e[n]);return this}},{key:"addColumnVector",value:function(e){e=ci(this,e);for(var t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)this.set(t,n,this.get(t,n)+e[t]);return this}},{key:"subColumnVector",value:function(e){e=ci(this,e);for(var t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)this.set(t,n,this.get(t,n)-e[t]);return this}},{key:"mulColumnVector",value:function(e){e=ci(this,e);for(var t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)this.set(t,n,this.get(t,n)*e[t]);return this}},{key:"divColumnVector",value:function(e){e=ci(this,e);for(var t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)this.set(t,n,this.get(t,n)/e[t]);return this}},{key:"mulRow",value:function(e,t){si(this,e);for(var n=0;n<this.columns;n++)this.set(e,n,this.get(e,n)*t);return this}},{key:"mulColumn",value:function(e,t){li(this,e);for(var n=0;n<this.rows;n++)this.set(n,e,this.get(n,e)*t);return this}},{key:"max",value:function(){for(var e=this.get(0,0),t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)this.get(t,n)>e&&(e=this.get(t,n));return e}},{key:"maxIndex",value:function(){for(var e=this.get(0,0),t=[0,0],n=0;n<this.rows;n++)for(var r=0;r<this.columns;r++)this.get(n,r)>e&&(e=this.get(n,r),t[0]=n,t[1]=r);return t}},{key:"min",value:function(){for(var e=this.get(0,0),t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)this.get(t,n)<e&&(e=this.get(t,n));return e}},{key:"minIndex",value:function(){for(var e=this.get(0,0),t=[0,0],n=0;n<this.rows;n++)for(var r=0;r<this.columns;r++)this.get(n,r)<e&&(e=this.get(n,r),t[0]=n,t[1]=r);return t}},{key:"maxRow",value:function(e){si(this,e);for(var t=this.get(e,0),n=1;n<this.columns;n++)this.get(e,n)>t&&(t=this.get(e,n));return t}},{key:"maxRowIndex",value:function(e){si(this,e);for(var t=this.get(e,0),n=[e,0],r=1;r<this.columns;r++)this.get(e,r)>t&&(t=this.get(e,r),n[1]=r);return n}},{key:"minRow",value:function(e){si(this,e);for(var t=this.get(e,0),n=1;n<this.columns;n++)this.get(e,n)<t&&(t=this.get(e,n));return t}},{key:"minRowIndex",value:function(e){si(this,e);for(var t=this.get(e,0),n=[e,0],r=1;r<this.columns;r++)this.get(e,r)<t&&(t=this.get(e,r),n[1]=r);return n}},{key:"maxColumn",value:function(e){li(this,e);for(var t=this.get(0,e),n=1;n<this.rows;n++)this.get(n,e)>t&&(t=this.get(n,e));return t}},{key:"maxColumnIndex",value:function(e){li(this,e);for(var t=this.get(0,e),n=[0,e],r=1;r<this.rows;r++)this.get(r,e)>t&&(t=this.get(r,e),n[0]=r);return n}},{key:"minColumn",value:function(e){li(this,e);for(var t=this.get(0,e),n=1;n<this.rows;n++)this.get(n,e)<t&&(t=this.get(n,e));return t}},{key:"minColumnIndex",value:function(e){li(this,e);for(var t=this.get(0,e),n=[0,e],r=1;r<this.rows;r++)this.get(r,e)<t&&(t=this.get(r,e),n[0]=r);return n}},{key:"diag",value:function(){for(var e=Math.min(this.rows,this.columns),t=[],n=0;n<e;n++)t.push(this.get(n,n));return t}},{key:"norm",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:"frobenius",t=0;if("max"===e)return this.max();if("frobenius"===e){for(var n=0;n<this.rows;n++)for(var r=0;r<this.columns;r++)t+=this.get(n,r)*this.get(n,r);return Math.sqrt(t)}throw new RangeError("unknown norm type: ".concat(e))}},{key:"cumulativeSum",value:function(){for(var e=0,t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)e+=this.get(t,n),this.set(t,n,e);return this}},{key:"dot",value:function(t){e.isMatrix(t)&&(t=t.to1DArray());var n=this.to1DArray();if(n.length!==t.length)throw new RangeError("vectors do not have the same size");for(var r=0,i=0;i<n.length;i++)r+=n[i]*t[i];return r}},{key:"mmul",value:function(e){e=Fi.checkMatrix(e);for(var t=this.rows,n=this.columns,r=e.columns,i=new Fi(t,r),o=new Float64Array(n),a=0;a<r;a++){for(var s=0;s<n;s++)o[s]=e.get(s,a);for(var l=0;l<t;l++){for(var u=0,c=0;c<n;c++)u+=this.get(l,c)*o[c];i.set(l,a,u)}}return i}},{key:"strassen2x2",value:function(e){e=Fi.checkMatrix(e);var t=new Fi(2,2),n=this.get(0,0),r=e.get(0,0),i=this.get(0,1),o=e.get(0,1),a=this.get(1,0),s=e.get(1,0),l=this.get(1,1),u=e.get(1,1),c=(n+l)*(r+u),d=(a+l)*r,h=n*(o-u),f=l*(s-r),p=(n+i)*u,g=c+f-p+(i-l)*(s+u),v=h+p,m=d+f,y=c-d+h+(a-n)*(r+o);return t.set(0,0,g),t.set(0,1,v),t.set(1,0,m),t.set(1,1,y),t}},{key:"strassen3x3",value:function(e){e=Fi.checkMatrix(e);var t=new Fi(3,3),n=this.get(0,0),r=this.get(0,1),i=this.get(0,2),o=this.get(1,0),a=this.get(1,1),s=this.get(1,2),l=this.get(2,0),u=this.get(2,1),c=this.get(2,2),d=e.get(0,0),h=e.get(0,1),f=e.get(0,2),p=e.get(1,0),g=e.get(1,1),v=e.get(1,2),m=e.get(2,0),y=e.get(2,1),b=e.get(2,2),C=(n-o)*(-h+g),A=(-n+o+a)*(d-h+g),_=(o+a)*(-d+h),I=n*d,x=(-n+l+u)*(d-f+v),w=(-n+l)*(f-v),M=(l+u)*(-d+f),S=(-i+u+c)*(g+m-y),T=(i-c)*(g-y),E=i*m,O=(u+c)*(-m+y),k=(-i+a+s)*(v+m-b),L=(i-s)*(v-b),D=(a+s)*(-m+b),P=I+E+r*p,N=(n+r+i-o-a-u-c)*g+A+_+I+S+E+O,j=I+x+M+(n+r+i-a-s-l-u)*v+E+k+D,R=C+a*(-d+h+p-g-v-m+b)+A+I+E+k+L,z=C+A+_+I+s*y,F=E+k+L+D+o*f,B=I+x+w+u*(-d+f+p-g-v-m+y)+S+T+E,H=S+T+E+O+l*h,U=I+x+w+M+c*b;return t.set(0,0,P),t.set(0,1,N),t.set(0,2,j),t.set(1,0,R),t.set(1,1,z),t.set(1,2,F),t.set(2,0,B),t.set(2,1,H),t.set(2,2,U),t}},{key:"mmulStrassen",value:function(t){t=Fi.checkMatrix(t);var n=this.clone(),r=n.rows,i=n.columns,o=t.rows,a=t.columns;function s(t,n,r){var i=t.rows,o=t.columns;if(i===n&&o===r)return t;var a=e.zeros(n,r);return a.setSubMatrix(t,0,0)}i!==o&&console.warn("Multiplying ".concat(r," x ").concat(i," and ").concat(o," x ").concat(a," matrix: dimensions do not match."));var l=Math.max(r,o),u=Math.max(i,a);return function t(n,r,i,o){if(i<=512||o<=512)return n.mmul(r);i%2==1&&o%2==1?(n=s(n,i+1,o+1),r=s(r,i+1,o+1)):i%2==1?(n=s(n,i+1,o),r=s(r,i+1,o)):o%2==1&&(n=s(n,i,o+1),r=s(r,i,o+1));var a=parseInt(n.rows/2,10),l=parseInt(n.columns/2,10),u=n.subMatrix(0,a-1,0,l-1),c=r.subMatrix(0,a-1,0,l-1),d=n.subMatrix(0,a-1,l,n.columns-1),h=r.subMatrix(0,a-1,l,r.columns-1),f=n.subMatrix(a,n.rows-1,0,l-1),p=r.subMatrix(a,r.rows-1,0,l-1),g=n.subMatrix(a,n.rows-1,l,n.columns-1),v=r.subMatrix(a,r.rows-1,l,r.columns-1),m=t(e.add(u,g),e.add(c,v),a,l),y=t(e.add(f,g),c,a,l),b=t(u,e.sub(h,v),a,l),C=t(g,e.sub(p,c),a,l),A=t(e.add(u,d),v,a,l),_=t(e.sub(f,u),e.add(c,h),a,l),I=t(e.sub(d,g),e.add(p,v),a,l),x=e.add(m,C);x.sub(A),x.add(I);var w=e.add(b,A),M=e.add(y,C),S=e.sub(m,y);S.add(b),S.add(_);var T=e.zeros(2*x.rows,2*x.columns);return(T=(T=(T=(T=T.setSubMatrix(x,0,0)).setSubMatrix(w,x.rows,0)).setSubMatrix(M,0,x.columns)).setSubMatrix(S,x.rows,x.columns)).subMatrix(0,i-1,0,o-1)}(n=s(n,l,u),t=s(t,l,u),l,u)}},{key:"scaleRows",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};if("object"!==ki(e))throw new TypeError("options must be an object");var t=e.min,n=void 0===t?0:t,r=e.max,i=void 0===r?1:r;if(!Number.isFinite(n))throw new TypeError("min must be a number");if(!Number.isFinite(i))throw new TypeError("max must be a number");if(n>=i)throw new RangeError("min must be smaller than max");for(var o=new Fi(this.rows,this.columns),a=0;a<this.rows;a++){var s=this.getRow(a);ei(s,{min:n,max:i,output:s}),o.setRow(a,s)}return o}},{key:"scaleColumns",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};if("object"!==ki(e))throw new TypeError("options must be an object");var t=e.min,n=void 0===t?0:t,r=e.max,i=void 0===r?1:r;if(!Number.isFinite(n))throw new TypeError("min must be a number");if(!Number.isFinite(i))throw new TypeError("max must be a number");if(n>=i)throw new RangeError("min must be smaller than max");for(var o=new Fi(this.rows,this.columns),a=0;a<this.columns;a++){var s=this.getColumn(a);ei(s,{min:n,max:i,output:s}),o.setColumn(a,s)}return o}},{key:"flipRows",value:function(){for(var e=Math.ceil(this.columns/2),t=0;t<this.rows;t++)for(var n=0;n<e;n++){var r=this.get(t,n),i=this.get(t,this.columns-1-n);this.set(t,n,i),this.set(t,this.columns-1-n,r)}return this}},{key:"flipColumns",value:function(){for(var e=Math.ceil(this.rows/2),t=0;t<this.columns;t++)for(var n=0;n<e;n++){var r=this.get(n,t),i=this.get(this.rows-1-n,t);this.set(n,t,i),this.set(this.rows-1-n,t,r)}return this}},{key:"kroneckerProduct",value:function(e){e=Fi.checkMatrix(e);for(var t=this.rows,n=this.columns,r=e.rows,i=e.columns,o=new Fi(t*r,n*i),a=0;a<t;a++)for(var s=0;s<n;s++)for(var l=0;l<r;l++)for(var u=0;u<i;u++)o.set(r*a+l,i*s+u,this.get(a,s)*e.get(l,u));return o}},{key:"transpose",value:function(){for(var e=new Fi(this.columns,this.rows),t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)e.set(n,t,this.get(t,n));return e}},{key:"sortRows",value:function(){for(var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:ji,t=0;t<this.rows;t++)this.setRow(t,this.getRow(t).sort(e));return this}},{key:"sortColumns",value:function(){for(var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:ji,t=0;t<this.columns;t++)this.setColumn(t,this.getColumn(t).sort(e));return this}},{key:"subMatrix",value:function(e,t,n,r){fi(this,e,t,n,r);for(var i=new Fi(t-e+1,r-n+1),o=e;o<=t;o++)for(var a=n;a<=r;a++)i.set(o-e,a-n,this.get(o,a));return i}},{key:"subMatrixRow",value:function(e,t,n){if(void 0===t&&(t=0),void 0===n&&(n=this.columns-1),t>n||t<0||t>=this.columns||n<0||n>=this.columns)throw new RangeError("Argument out of range");for(var r=new Fi(e.length,n-t+1),i=0;i<e.length;i++)for(var o=t;o<=n;o++){if(e[i]<0||e[i]>=this.rows)throw new RangeError("Row index out of range: ".concat(e[i]));r.set(i,o-t,this.get(e[i],o))}return r}},{key:"subMatrixColumn",value:function(e,t,n){if(void 0===t&&(t=0),void 0===n&&(n=this.rows-1),t>n||t<0||t>=this.rows||n<0||n>=this.rows)throw new RangeError("Argument out of range");for(var r=new Fi(n-t+1,e.length),i=0;i<e.length;i++)for(var o=t;o<=n;o++){if(e[i]<0||e[i]>=this.columns)throw new RangeError("Column index out of range: ".concat(e[i]));r.set(o-t,i,this.get(o,e[i]))}return r}},{key:"setSubMatrix",value:function(e,t,n){fi(this,t,t+(e=Fi.checkMatrix(e)).rows-1,n,n+e.columns-1);for(var r=0;r<e.rows;r++)for(var i=0;i<e.columns;i++)this.set(t+r,n+i,e.get(r,i));return this}},{key:"selection",value:function(e,t){for(var n=function(e,t,n){return{row:di(e,t),column:hi(e,n)}}(this,e,t),r=new Fi(e.length,t.length),i=0;i<n.row.length;i++)for(var o=n.row[i],a=0;a<n.column.length;a++){var s=n.column[a];r.set(i,a,this.get(o,s))}return r}},{key:"trace",value:function(){for(var e=Math.min(this.rows,this.columns),t=0,n=0;n<e;n++)t+=this.get(n,n);return t}},{key:"clone",value:function(){for(var e=new Fi(this.rows,this.columns),t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)e.set(t,n,this.get(t,n));return e}},{key:"sum",value:function(e){switch(e){case"row":return function(e){for(var t=pi(e.rows),n=0;n<e.rows;++n)for(var r=0;r<e.columns;++r)t[n]+=e.get(n,r);return t}(this);case"column":return function(e){for(var t=pi(e.columns),n=0;n<e.rows;++n)for(var r=0;r<e.columns;++r)t[r]+=e.get(n,r);return t}(this);case void 0:return function(e){for(var t=0,n=0;n<e.rows;n++)for(var r=0;r<e.columns;r++)t+=e.get(n,r);return t}(this);default:throw new Error("invalid option: ".concat(e))}}},{key:"product",value:function(e){switch(e){case"row":return function(e){for(var t=pi(e.rows,1),n=0;n<e.rows;++n)for(var r=0;r<e.columns;++r)t[n]*=e.get(n,r);return t}(this);case"column":return function(e){for(var t=pi(e.columns,1),n=0;n<e.rows;++n)for(var r=0;r<e.columns;++r)t[r]*=e.get(n,r);return t}(this);case void 0:return function(e){for(var t=1,n=0;n<e.rows;n++)for(var r=0;r<e.columns;r++)t*=e.get(n,r);return t}(this);default:throw new Error("invalid option: ".concat(e))}}},{key:"mean",value:function(e){var t=this.sum(e);switch(e){case"row":for(var n=0;n<this.rows;n++)t[n]/=this.columns;return t;case"column":for(var r=0;r<this.columns;r++)t[r]/=this.rows;return t;case void 0:return t/this.size;default:throw new Error("invalid option: ".concat(e))}}},{key:"variance",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};if("object"===ki(e)&&(t=e,e=void 0),"object"!==ki(t))throw new TypeError("options must be an object");var n=t,r=n.unbiased,i=void 0===r||r,o=n.mean,a=void 0===o?this.mean(e):o;if("boolean"!=typeof i)throw new TypeError("unbiased must be a boolean");switch(e){case"row":if(!Array.isArray(a))throw new TypeError("mean must be an array");return vi(this,i,a);case"column":if(!Array.isArray(a))throw new TypeError("mean must be an array");return mi(this,i,a);case void 0:if("number"!=typeof a)throw new TypeError("mean must be a number");return yi(this,i,a);default:throw new Error("invalid option: ".concat(e))}}},{key:"standardDeviation",value:function(e,t){"object"===ki(e)&&(t=e,e=void 0);var n=this.variance(e,t);if(void 0===e)return Math.sqrt(n);for(var r=0;r<n.length;r++)n[r]=Math.sqrt(n[r]);return n}},{key:"center",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};if("object"===ki(e)&&(t=e,e=void 0),"object"!==ki(t))throw new TypeError("options must be an object");var n=t,r=n.center,i=void 0===r?this.mean(e):r;switch(e){case"row":if(!Array.isArray(i))throw new TypeError("center must be an array");return bi(this,i),this;case"column":if(!Array.isArray(i))throw new TypeError("center must be an array");return Ci(this,i),this;case void 0:if("number"!=typeof i)throw new TypeError("center must be a number");return Ai(this,i),this;default:throw new Error("invalid option: ".concat(e))}}},{key:"scale",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};if("object"===ki(e)&&(t=e,e=void 0),"object"!==ki(t))throw new TypeError("options must be an object");var n=t.scale;switch(e){case"row":if(void 0===n)n=_i(this);else if(!Array.isArray(n))throw new TypeError("scale must be an array");return Ii(this,n),this;case"column":if(void 0===n)n=xi(this);else if(!Array.isArray(n))throw new TypeError("scale must be an array");return wi(this,n),this;case void 0:if(void 0===n)n=Mi(this);else if("number"!=typeof n)throw new TypeError("scale must be a number");return Si(this,n),this;default:throw new Error("invalid option: ".concat(e))}}},{key:"toString",value:function(e){return ri(this,e)}},{key:"size",get:function(){return this.rows*this.columns}}],[{key:"from1DArray",value:function(e,t,n){if(e*t!==n.length)throw new RangeError("data length does not match given dimensions");for(var r=new Fi(e,t),i=0;i<e;i++)for(var o=0;o<t;o++)r.set(i,o,n[i*t+o]);return r}},{key:"rowVector",value:function(e){for(var t=new Fi(1,e.length),n=0;n<e.length;n++)t.set(0,n,e[n]);return t}},{key:"columnVector",value:function(e){for(var t=new Fi(e.length,1),n=0;n<e.length;n++)t.set(n,0,e[n]);return t}},{key:"zeros",value:function(e,t){return new Fi(e,t)}},{key:"ones",value:function(e,t){return new Fi(e,t).fill(1)}},{key:"rand",value:function(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};if("object"!==ki(n))throw new TypeError("options must be an object");for(var r=n.random,i=void 0===r?Math.random:r,o=new Fi(e,t),a=0;a<e;a++)for(var s=0;s<t;s++)o.set(a,s,i());return o}},{key:"randInt",value:function(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};if("object"!==ki(n))throw new TypeError("options must be an object");var r=n.min,i=void 0===r?0:r,o=n.max,a=void 0===o?1e3:o,s=n.random,l=void 0===s?Math.random:s;if(!Number.isInteger(i))throw new TypeError("min must be an integer");if(!Number.isInteger(a))throw new TypeError("max must be an integer");if(i>=a)throw new RangeError("min must be smaller than max");for(var u=a-i,c=new Fi(e,t),d=0;d<e;d++)for(var h=0;h<t;h++){var f=i+Math.round(l()*u);c.set(d,h,f)}return c}},{key:"eye",value:function(e,t,n){void 0===t&&(t=e),void 0===n&&(n=1);for(var r=Math.min(e,t),i=this.zeros(e,t),o=0;o<r;o++)i.set(o,o,n);return i}},{key:"diag",value:function(e,t,n){var r=e.length;void 0===t&&(t=r),void 0===n&&(n=t);for(var i=Math.min(r,t,n),o=this.zeros(t,n),a=0;a<i;a++)o.set(a,a,e[a]);return o}},{key:"min",value:function(e,t){e=this.checkMatrix(e),t=this.checkMatrix(t);for(var n=e.rows,r=e.columns,i=new Fi(n,r),o=0;o<n;o++)for(var a=0;a<r;a++)i.set(o,a,Math.min(e.get(o,a),t.get(o,a)));return i}},{key:"max",value:function(e,t){e=this.checkMatrix(e),t=this.checkMatrix(t);for(var n=e.rows,r=e.columns,i=new this(n,r),o=0;o<n;o++)for(var a=0;a<r;a++)i.set(o,a,Math.max(e.get(o,a),t.get(o,a)));return i}},{key:"checkMatrix",value:function(t){return e.isMatrix(t)?t:new Fi(t)}},{key:"isMatrix",value:function(e){return null!=e&&"Matrix"===e.klass}}]),e}();function ji(e,t){return e-t}Ni.prototype.klass="Matrix","undefined"!=typeof Symbol&&(Ni.prototype[Symbol.for("nodejs.util.inspect.custom")]=function(){return ri(this)}),Ni.random=Ni.rand,Ni.randomInt=Ni.randInt,Ni.diagonal=Ni.diag,Ni.prototype.diagonal=Ni.prototype.diag,Ni.identity=Ni.eye,Ni.prototype.negate=Ni.prototype.neg,Ni.prototype.tensorProduct=Ni.prototype.kroneckerProduct;var Ri,zi,Fi=function(e){!function(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&Ti(e,t)}(n,e);var t=function(e){var t=function(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],(function(){}))),!0}catch(e){return!1}}();return function(){var n,r=Oi(e);if(t){var i=Oi(this).constructor;n=Reflect.construct(r,arguments,i)}else n=r.apply(this,arguments);return Ei(this,n)}}(n);function n(e,r){var i;if(Li(this,n),i=t.call(this),n.isMatrix(e))return Ei(i,e.clone());if(Number.isInteger(e)&&e>0){if(i.data=[],!(Number.isInteger(r)&&r>0))throw new TypeError("nColumns must be a positive integer");for(var o=0;o<e;o++)i.data.push(new Float64Array(r))}else{if(!Array.isArray(e))throw new TypeError("First argument must be a positive number or an array");var a=e;if(e=a.length,"number"!=typeof(r=a[0].length)||0===r)throw new TypeError("Data must be a 2D array with at least one element");i.data=[];for(var s=0;s<e;s++){if(a[s].length!==r)throw new RangeError("Inconsistent array dimensions");i.data.push(Float64Array.from(a[s]))}}return i.rows=e,i.columns=r,i}return Pi(n,[{key:"set",value:function(e,t,n){return this.data[e][t]=n,this}},{key:"get",value:function(e,t){return this.data[e][t]}},{key:"removeRow",value:function(e){if(si(this,e),1===this.rows)throw new RangeError("A matrix cannot have less than one row");return this.data.splice(e,1),this.rows-=1,this}},{key:"addRow",value:function(e,t){return void 0===t&&(t=e,e=this.rows),si(this,e,!0),t=Float64Array.from(ui(this,t)),this.data.splice(e,0,t),this.rows+=1,this}},{key:"removeColumn",value:function(e){if(li(this,e),1===this.columns)throw new RangeError("A matrix cannot have less than one column");for(var t=0;t<this.rows;t++){for(var n=new Float64Array(this.columns-1),r=0;r<e;r++)n[r]=this.data[t][r];for(var i=e+1;i<this.columns;i++)n[i-1]=this.data[t][i];this.data[t]=n}return this.columns-=1,this}},{key:"addColumn",value:function(e,t){void 0===t&&(t=e,e=this.columns),li(this,e,!0),t=ci(this,t);for(var n=0;n<this.rows;n++){for(var r=new Float64Array(this.columns+1),i=0;i<e;i++)r[i]=this.data[n][i];for(r[i++]=t[n];i<this.columns+1;i++)r[i]=this.data[n][i-1];this.data[n]=r}return this.columns+=1,this}}]),n}(Ni);function Bi(e){return(Bi="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function Hi(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}function Ui(e,t){return(Ui=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function Vi(e,t){return!t||"object"!==Bi(t)&&"function"!=typeof t?function(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}(e):t}function Yi(e){return(Yi=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}zi=Fi,(Ri=Ni).prototype.add=function(e){return"number"==typeof e?this.addS(e):this.addM(e)},Ri.prototype.addS=function(e){for(var t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)this.set(t,n,this.get(t,n)+e);return this},Ri.prototype.addM=function(e){if(e=zi.checkMatrix(e),this.rows!==e.rows||this.columns!==e.columns)throw new RangeError("Matrices dimensions must be equal");for(var t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)this.set(t,n,this.get(t,n)+e.get(t,n));return this},Ri.add=function(e,t){return new zi(e).add(t)},Ri.prototype.sub=function(e){return"number"==typeof e?this.subS(e):this.subM(e)},Ri.prototype.subS=function(e){for(var t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)this.set(t,n,this.get(t,n)-e);return this},Ri.prototype.subM=function(e){if(e=zi.checkMatrix(e),this.rows!==e.rows||this.columns!==e.columns)throw new RangeError("Matrices dimensions must be equal");for(var t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)this.set(t,n,this.get(t,n)-e.get(t,n));return this},Ri.sub=function(e,t){return new zi(e).sub(t)},Ri.prototype.subtract=Ri.prototype.sub,Ri.prototype.subtractS=Ri.prototype.subS,Ri.prototype.subtractM=Ri.prototype.subM,Ri.subtract=Ri.sub,Ri.prototype.mul=function(e){return"number"==typeof e?this.mulS(e):this.mulM(e)},Ri.prototype.mulS=function(e){for(var t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)this.set(t,n,this.get(t,n)*e);return this},Ri.prototype.mulM=function(e){if(e=zi.checkMatrix(e),this.rows!==e.rows||this.columns!==e.columns)throw new RangeError("Matrices dimensions must be equal");for(var t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)this.set(t,n,this.get(t,n)*e.get(t,n));return this},Ri.mul=function(e,t){return new zi(e).mul(t)},Ri.prototype.multiply=Ri.prototype.mul,Ri.prototype.multiplyS=Ri.prototype.mulS,Ri.prototype.multiplyM=Ri.prototype.mulM,Ri.multiply=Ri.mul,Ri.prototype.div=function(e){return"number"==typeof e?this.divS(e):this.divM(e)},Ri.prototype.divS=function(e){for(var t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)this.set(t,n,this.get(t,n)/e);return this},Ri.prototype.divM=function(e){if(e=zi.checkMatrix(e),this.rows!==e.rows||this.columns!==e.columns)throw new RangeError("Matrices dimensions must be equal");for(var t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)this.set(t,n,this.get(t,n)/e.get(t,n));return this},Ri.div=function(e,t){return new zi(e).div(t)},Ri.prototype.divide=Ri.prototype.div,Ri.prototype.divideS=Ri.prototype.divS,Ri.prototype.divideM=Ri.prototype.divM,Ri.divide=Ri.div,Ri.prototype.mod=function(e){return"number"==typeof e?this.modS(e):this.modM(e)},Ri.prototype.modS=function(e){for(var t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)this.set(t,n,this.get(t,n)%e);return this},Ri.prototype.modM=function(e){if(e=zi.checkMatrix(e),this.rows!==e.rows||this.columns!==e.columns)throw new RangeError("Matrices dimensions must be equal");for(var t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)this.set(t,n,this.get(t,n)%e.get(t,n));return this},Ri.mod=function(e,t){return new zi(e).mod(t)},Ri.prototype.modulus=Ri.prototype.mod,Ri.prototype.modulusS=Ri.prototype.modS,Ri.prototype.modulusM=Ri.prototype.modM,Ri.modulus=Ri.mod,Ri.prototype.and=function(e){return"number"==typeof e?this.andS(e):this.andM(e)},Ri.prototype.andS=function(e){for(var t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)this.set(t,n,this.get(t,n)&e);return this},Ri.prototype.andM=function(e){if(e=zi.checkMatrix(e),this.rows!==e.rows||this.columns!==e.columns)throw new RangeError("Matrices dimensions must be equal");for(var t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)this.set(t,n,this.get(t,n)&e.get(t,n));return this},Ri.and=function(e,t){return new zi(e).and(t)},Ri.prototype.or=function(e){return"number"==typeof e?this.orS(e):this.orM(e)},Ri.prototype.orS=function(e){for(var t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)this.set(t,n,this.get(t,n)|e);return this},Ri.prototype.orM=function(e){if(e=zi.checkMatrix(e),this.rows!==e.rows||this.columns!==e.columns)throw new RangeError("Matrices dimensions must be equal");for(var t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)this.set(t,n,this.get(t,n)|e.get(t,n));return this},Ri.or=function(e,t){return new zi(e).or(t)},Ri.prototype.xor=function(e){return"number"==typeof e?this.xorS(e):this.xorM(e)},Ri.prototype.xorS=function(e){for(var t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)this.set(t,n,this.get(t,n)^e);return this},Ri.prototype.xorM=function(e){if(e=zi.checkMatrix(e),this.rows!==e.rows||this.columns!==e.columns)throw new RangeError("Matrices dimensions must be equal");for(var t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)this.set(t,n,this.get(t,n)^e.get(t,n));return this},Ri.xor=function(e,t){return new zi(e).xor(t)},Ri.prototype.leftShift=function(e){return"number"==typeof e?this.leftShiftS(e):this.leftShiftM(e)},Ri.prototype.leftShiftS=function(e){for(var t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)this.set(t,n,this.get(t,n)<<e);return this},Ri.prototype.leftShiftM=function(e){if(e=zi.checkMatrix(e),this.rows!==e.rows||this.columns!==e.columns)throw new RangeError("Matrices dimensions must be equal");for(var t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)this.set(t,n,this.get(t,n)<<e.get(t,n));return this},Ri.leftShift=function(e,t){return new zi(e).leftShift(t)},Ri.prototype.signPropagatingRightShift=function(e){return"number"==typeof e?this.signPropagatingRightShiftS(e):this.signPropagatingRightShiftM(e)},Ri.prototype.signPropagatingRightShiftS=function(e){for(var t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)this.set(t,n,this.get(t,n)>>e);return this},Ri.prototype.signPropagatingRightShiftM=function(e){if(e=zi.checkMatrix(e),this.rows!==e.rows||this.columns!==e.columns)throw new RangeError("Matrices dimensions must be equal");for(var t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)this.set(t,n,this.get(t,n)>>e.get(t,n));return this},Ri.signPropagatingRightShift=function(e,t){return new zi(e).signPropagatingRightShift(t)},Ri.prototype.rightShift=function(e){return"number"==typeof e?this.rightShiftS(e):this.rightShiftM(e)},Ri.prototype.rightShiftS=function(e){for(var t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)this.set(t,n,this.get(t,n)>>>e);return this},Ri.prototype.rightShiftM=function(e){if(e=zi.checkMatrix(e),this.rows!==e.rows||this.columns!==e.columns)throw new RangeError("Matrices dimensions must be equal");for(var t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)this.set(t,n,this.get(t,n)>>>e.get(t,n));return this},Ri.rightShift=function(e,t){return new zi(e).rightShift(t)},Ri.prototype.zeroFillRightShift=Ri.prototype.rightShift,Ri.prototype.zeroFillRightShiftS=Ri.prototype.rightShiftS,Ri.prototype.zeroFillRightShiftM=Ri.prototype.rightShiftM,Ri.zeroFillRightShift=Ri.rightShift,Ri.prototype.not=function(){for(var e=0;e<this.rows;e++)for(var t=0;t<this.columns;t++)this.set(e,t,~this.get(e,t));return this},Ri.not=function(e){return new zi(e).not()},Ri.prototype.abs=function(){for(var e=0;e<this.rows;e++)for(var t=0;t<this.columns;t++)this.set(e,t,Math.abs(this.get(e,t)));return this},Ri.abs=function(e){return new zi(e).abs()},Ri.prototype.acos=function(){for(var e=0;e<this.rows;e++)for(var t=0;t<this.columns;t++)this.set(e,t,Math.acos(this.get(e,t)));return this},Ri.acos=function(e){return new zi(e).acos()},Ri.prototype.acosh=function(){for(var e=0;e<this.rows;e++)for(var t=0;t<this.columns;t++)this.set(e,t,Math.acosh(this.get(e,t)));return this},Ri.acosh=function(e){return new zi(e).acosh()},Ri.prototype.asin=function(){for(var e=0;e<this.rows;e++)for(var t=0;t<this.columns;t++)this.set(e,t,Math.asin(this.get(e,t)));return this},Ri.asin=function(e){return new zi(e).asin()},Ri.prototype.asinh=function(){for(var e=0;e<this.rows;e++)for(var t=0;t<this.columns;t++)this.set(e,t,Math.asinh(this.get(e,t)));return this},Ri.asinh=function(e){return new zi(e).asinh()},Ri.prototype.atan=function(){for(var e=0;e<this.rows;e++)for(var t=0;t<this.columns;t++)this.set(e,t,Math.atan(this.get(e,t)));return this},Ri.atan=function(e){return new zi(e).atan()},Ri.prototype.atanh=function(){for(var e=0;e<this.rows;e++)for(var t=0;t<this.columns;t++)this.set(e,t,Math.atanh(this.get(e,t)));return this},Ri.atanh=function(e){return new zi(e).atanh()},Ri.prototype.cbrt=function(){for(var e=0;e<this.rows;e++)for(var t=0;t<this.columns;t++)this.set(e,t,Math.cbrt(this.get(e,t)));return this},Ri.cbrt=function(e){return new zi(e).cbrt()},Ri.prototype.ceil=function(){for(var e=0;e<this.rows;e++)for(var t=0;t<this.columns;t++)this.set(e,t,Math.ceil(this.get(e,t)));return this},Ri.ceil=function(e){return new zi(e).ceil()},Ri.prototype.clz32=function(){for(var e=0;e<this.rows;e++)for(var t=0;t<this.columns;t++)this.set(e,t,Math.clz32(this.get(e,t)));return this},Ri.clz32=function(e){return new zi(e).clz32()},Ri.prototype.cos=function(){for(var e=0;e<this.rows;e++)for(var t=0;t<this.columns;t++)this.set(e,t,Math.cos(this.get(e,t)));return this},Ri.cos=function(e){return new zi(e).cos()},Ri.prototype.cosh=function(){for(var e=0;e<this.rows;e++)for(var t=0;t<this.columns;t++)this.set(e,t,Math.cosh(this.get(e,t)));return this},Ri.cosh=function(e){return new zi(e).cosh()},Ri.prototype.exp=function(){for(var e=0;e<this.rows;e++)for(var t=0;t<this.columns;t++)this.set(e,t,Math.exp(this.get(e,t)));return this},Ri.exp=function(e){return new zi(e).exp()},Ri.prototype.expm1=function(){for(var e=0;e<this.rows;e++)for(var t=0;t<this.columns;t++)this.set(e,t,Math.expm1(this.get(e,t)));return this},Ri.expm1=function(e){return new zi(e).expm1()},Ri.prototype.floor=function(){for(var e=0;e<this.rows;e++)for(var t=0;t<this.columns;t++)this.set(e,t,Math.floor(this.get(e,t)));return this},Ri.floor=function(e){return new zi(e).floor()},Ri.prototype.fround=function(){for(var e=0;e<this.rows;e++)for(var t=0;t<this.columns;t++)this.set(e,t,Math.fround(this.get(e,t)));return this},Ri.fround=function(e){return new zi(e).fround()},Ri.prototype.log=function(){for(var e=0;e<this.rows;e++)for(var t=0;t<this.columns;t++)this.set(e,t,Math.log(this.get(e,t)));return this},Ri.log=function(e){return new zi(e).log()},Ri.prototype.log1p=function(){for(var e=0;e<this.rows;e++)for(var t=0;t<this.columns;t++)this.set(e,t,Math.log1p(this.get(e,t)));return this},Ri.log1p=function(e){return new zi(e).log1p()},Ri.prototype.log10=function(){for(var e=0;e<this.rows;e++)for(var t=0;t<this.columns;t++)this.set(e,t,Math.log10(this.get(e,t)));return this},Ri.log10=function(e){return new zi(e).log10()},Ri.prototype.log2=function(){for(var e=0;e<this.rows;e++)for(var t=0;t<this.columns;t++)this.set(e,t,Math.log2(this.get(e,t)));return this},Ri.log2=function(e){return new zi(e).log2()},Ri.prototype.round=function(){for(var e=0;e<this.rows;e++)for(var t=0;t<this.columns;t++)this.set(e,t,Math.round(this.get(e,t)));return this},Ri.round=function(e){return new zi(e).round()},Ri.prototype.sign=function(){for(var e=0;e<this.rows;e++)for(var t=0;t<this.columns;t++)this.set(e,t,Math.sign(this.get(e,t)));return this},Ri.sign=function(e){return new zi(e).sign()},Ri.prototype.sin=function(){for(var e=0;e<this.rows;e++)for(var t=0;t<this.columns;t++)this.set(e,t,Math.sin(this.get(e,t)));return this},Ri.sin=function(e){return new zi(e).sin()},Ri.prototype.sinh=function(){for(var e=0;e<this.rows;e++)for(var t=0;t<this.columns;t++)this.set(e,t,Math.sinh(this.get(e,t)));return this},Ri.sinh=function(e){return new zi(e).sinh()},Ri.prototype.sqrt=function(){for(var e=0;e<this.rows;e++)for(var t=0;t<this.columns;t++)this.set(e,t,Math.sqrt(this.get(e,t)));return this},Ri.sqrt=function(e){return new zi(e).sqrt()},Ri.prototype.tan=function(){for(var e=0;e<this.rows;e++)for(var t=0;t<this.columns;t++)this.set(e,t,Math.tan(this.get(e,t)));return this},Ri.tan=function(e){return new zi(e).tan()},Ri.prototype.tanh=function(){for(var e=0;e<this.rows;e++)for(var t=0;t<this.columns;t++)this.set(e,t,Math.tanh(this.get(e,t)));return this},Ri.tanh=function(e){return new zi(e).tanh()},Ri.prototype.trunc=function(){for(var e=0;e<this.rows;e++)for(var t=0;t<this.columns;t++)this.set(e,t,Math.trunc(this.get(e,t)));return this},Ri.trunc=function(e){return new zi(e).trunc()},Ri.pow=function(e,t){return new zi(e).pow(t)},Ri.prototype.pow=function(e){return"number"==typeof e?this.powS(e):this.powM(e)},Ri.prototype.powS=function(e){for(var t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)this.set(t,n,Math.pow(this.get(t,n),e));return this},Ri.prototype.powM=function(e){if(e=zi.checkMatrix(e),this.rows!==e.rows||this.columns!==e.columns)throw new RangeError("Matrices dimensions must be equal");for(var t=0;t<this.rows;t++)for(var n=0;n<this.columns;n++)this.set(t,n,Math.pow(this.get(t,n),e.get(t,n)));return this};var Wi=function(e){!function(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&Ui(e,t)}(i,e);var t,n,r=function(e){var t=function(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],(function(){}))),!0}catch(e){return!1}}();return function(){var n,r=Yi(e);if(t){var i=Yi(this).constructor;n=Reflect.construct(r,arguments,i)}else n=r.apply(this,arguments);return Vi(this,n)}}(i);function i(e){var t;return function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,i),(t=r.call(this)).data=e,t.rows=e.length,t.columns=e[0].length,t}return t=i,(n=[{key:"set",value:function(e,t,n){return this.data[e][t]=n,this}},{key:"get",value:function(e,t){return this.data[e][t]}}])&&Hi(t.prototype,n),i}(Ni);function Gi(e,t){var n=0;return Math.abs(e)>Math.abs(t)?(n=t/e,Math.abs(e)*Math.sqrt(1+n*n)):0!==t?(n=e/t,Math.abs(t)*Math.sqrt(1+n*n)):0}function Xi(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function qi(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}var Zi=function(){function e(t){var n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};Xi(this,e);var r,i=(t=Wi.checkMatrix(t)).rows,o=t.columns,a=n.computeLeftSingularVectors,s=void 0===a||a,l=n.computeRightSingularVectors,u=void 0===l||l,c=n.autoTranspose,d=void 0!==c&&c,h=Boolean(s),f=Boolean(u),p=!1;if(i<o)if(d){i=(r=t.transpose()).rows,o=r.columns,p=!0;var g=h;h=f,f=g}else r=t.clone(),console.warn("Computing SVD on a matrix with more columns than rows. Consider enabling autoTranspose");else r=t.clone();for(var v=Math.min(i,o),m=Math.min(i+1,o),y=new Float64Array(m),b=new Fi(i,v),C=new Fi(o,o),A=new Float64Array(o),_=new Float64Array(i),I=new Float64Array(m),x=0;x<m;x++)I[x]=x;for(var w=Math.min(i-1,o),M=Math.max(0,Math.min(o-2,i)),S=Math.max(w,M),T=0;T<S;T++){if(T<w){y[T]=0;for(var E=T;E<i;E++)y[T]=Gi(y[T],r.get(E,T));if(0!==y[T]){r.get(T,T)<0&&(y[T]=-y[T]);for(var O=T;O<i;O++)r.set(O,T,r.get(O,T)/y[T]);r.set(T,T,r.get(T,T)+1)}y[T]=-y[T]}for(var k=T+1;k<o;k++){if(T<w&&0!==y[T]){for(var L=0,D=T;D<i;D++)L+=r.get(D,T)*r.get(D,k);L=-L/r.get(T,T);for(var P=T;P<i;P++)r.set(P,k,r.get(P,k)+L*r.get(P,T))}A[k]=r.get(T,k)}if(h&&T<w)for(var N=T;N<i;N++)b.set(N,T,r.get(N,T));if(T<M){A[T]=0;for(var j=T+1;j<o;j++)A[T]=Gi(A[T],A[j]);if(0!==A[T]){A[T+1]<0&&(A[T]=0-A[T]);for(var R=T+1;R<o;R++)A[R]/=A[T];A[T+1]+=1}if(A[T]=-A[T],T+1<i&&0!==A[T]){for(var z=T+1;z<i;z++)_[z]=0;for(var F=T+1;F<i;F++)for(var B=T+1;B<o;B++)_[F]+=A[B]*r.get(F,B);for(var H=T+1;H<o;H++)for(var U=-A[H]/A[T+1],V=T+1;V<i;V++)r.set(V,H,r.get(V,H)+U*_[V])}if(f)for(var Y=T+1;Y<o;Y++)C.set(Y,T,A[Y])}}var W=Math.min(o,i+1);if(w<o&&(y[w]=r.get(w,w)),i<W&&(y[W-1]=0),M+1<W&&(A[M]=r.get(M,W-1)),A[W-1]=0,h){for(var G=w;G<v;G++){for(var X=0;X<i;X++)b.set(X,G,0);b.set(G,G,1)}for(var q=w-1;q>=0;q--)if(0!==y[q]){for(var Z=q+1;Z<v;Z++){for(var Q=0,K=q;K<i;K++)Q+=b.get(K,q)*b.get(K,Z);Q=-Q/b.get(q,q);for(var J=q;J<i;J++)b.set(J,Z,b.get(J,Z)+Q*b.get(J,q))}for(var $=q;$<i;$++)b.set($,q,-b.get($,q));b.set(q,q,1+b.get(q,q));for(var ee=0;ee<q-1;ee++)b.set(ee,q,0)}else{for(var te=0;te<i;te++)b.set(te,q,0);b.set(q,q,1)}}if(f)for(var ne=o-1;ne>=0;ne--){if(ne<M&&0!==A[ne])for(var re=ne+1;re<o;re++){for(var ie=0,oe=ne+1;oe<o;oe++)ie+=C.get(oe,ne)*C.get(oe,re);ie=-ie/C.get(ne+1,ne);for(var ae=ne+1;ae<o;ae++)C.set(ae,re,C.get(ae,re)+ie*C.get(ae,ne))}for(var se=0;se<o;se++)C.set(se,ne,0);C.set(ne,ne,1)}for(var le=W-1,ue=Number.EPSILON;W>0;){var ce=void 0,de=void 0;for(ce=W-2;ce>=-1&&-1!==ce;ce--){var he=Number.MIN_VALUE+ue*Math.abs(y[ce]+Math.abs(y[ce+1]));if(Math.abs(A[ce])<=he||Number.isNaN(A[ce])){A[ce]=0;break}}if(ce===W-2)de=4;else{var fe=void 0;for(fe=W-1;fe>=ce&&fe!==ce;fe--){var pe=(fe!==W?Math.abs(A[fe]):0)+(fe!==ce+1?Math.abs(A[fe-1]):0);if(Math.abs(y[fe])<=ue*pe){y[fe]=0;break}}fe===ce?de=3:fe===W-1?de=1:(de=2,ce=fe)}switch(ce++,de){case 1:var ge=A[W-2];A[W-2]=0;for(var ve=W-2;ve>=ce;ve--){var me=Gi(y[ve],ge),ye=y[ve]/me,be=ge/me;if(y[ve]=me,ve!==ce&&(ge=-be*A[ve-1],A[ve-1]=ye*A[ve-1]),f)for(var Ce=0;Ce<o;Ce++)me=ye*C.get(Ce,ve)+be*C.get(Ce,W-1),C.set(Ce,W-1,-be*C.get(Ce,ve)+ye*C.get(Ce,W-1)),C.set(Ce,ve,me)}break;case 2:var Ae=A[ce-1];A[ce-1]=0;for(var _e=ce;_e<W;_e++){var Ie=Gi(y[_e],Ae),xe=y[_e]/Ie,we=Ae/Ie;if(y[_e]=Ie,Ae=-we*A[_e],A[_e]=xe*A[_e],h)for(var Me=0;Me<i;Me++)Ie=xe*b.get(Me,_e)+we*b.get(Me,ce-1),b.set(Me,ce-1,-we*b.get(Me,_e)+xe*b.get(Me,ce-1)),b.set(Me,_e,Ie)}break;case 3:var Se=Math.max(Math.abs(y[W-1]),Math.abs(y[W-2]),Math.abs(A[W-2]),Math.abs(y[ce]),Math.abs(A[ce])),Te=y[W-1]/Se,Ee=y[W-2]/Se,Oe=A[W-2]/Se,ke=y[ce]/Se,Le=A[ce]/Se,De=((Ee+Te)*(Ee-Te)+Oe*Oe)/2,Pe=Te*Oe*(Te*Oe),Ne=0;0===De&&0===Pe||(Ne=Pe/(De+(Ne=De<0?0-Math.sqrt(De*De+Pe):Math.sqrt(De*De+Pe))));for(var je=(ke+Te)*(ke-Te)+Ne,Re=ke*Le,ze=ce;ze<W-1;ze++){var Fe=Gi(je,Re);0===Fe&&(Fe=Number.MIN_VALUE);var Be=je/Fe,He=Re/Fe;if(ze!==ce&&(A[ze-1]=Fe),je=Be*y[ze]+He*A[ze],A[ze]=Be*A[ze]-He*y[ze],Re=He*y[ze+1],y[ze+1]=Be*y[ze+1],f)for(var Ue=0;Ue<o;Ue++)Fe=Be*C.get(Ue,ze)+He*C.get(Ue,ze+1),C.set(Ue,ze+1,-He*C.get(Ue,ze)+Be*C.get(Ue,ze+1)),C.set(Ue,ze,Fe);if(0===(Fe=Gi(je,Re))&&(Fe=Number.MIN_VALUE),Be=je/Fe,He=Re/Fe,y[ze]=Fe,je=Be*A[ze]+He*y[ze+1],y[ze+1]=-He*A[ze]+Be*y[ze+1],Re=He*A[ze+1],A[ze+1]=Be*A[ze+1],h&&ze<i-1)for(var Ve=0;Ve<i;Ve++)Fe=Be*b.get(Ve,ze)+He*b.get(Ve,ze+1),b.set(Ve,ze+1,-He*b.get(Ve,ze)+Be*b.get(Ve,ze+1)),b.set(Ve,ze,Fe)}A[W-2]=je;break;case 4:if(y[ce]<=0&&(y[ce]=y[ce]<0?-y[ce]:0,f))for(var Ye=0;Ye<=le;Ye++)C.set(Ye,ce,-C.get(Ye,ce));for(;ce<le&&!(y[ce]>=y[ce+1]);){var We=y[ce];if(y[ce]=y[ce+1],y[ce+1]=We,f&&ce<o-1)for(var Ge=0;Ge<o;Ge++)We=C.get(Ge,ce+1),C.set(Ge,ce+1,C.get(Ge,ce)),C.set(Ge,ce,We);if(h&&ce<i-1)for(var Xe=0;Xe<i;Xe++)We=b.get(Xe,ce+1),b.set(Xe,ce+1,b.get(Xe,ce)),b.set(Xe,ce,We);ce++}W--}}if(p){var qe=C;C=b,b=qe}this.m=i,this.n=o,this.s=y,this.U=b,this.V=C}var t,n;return t=e,(n=[{key:"solve",value:function(e){for(var t=e,n=this.threshold,r=this.s.length,i=Fi.zeros(r,r),o=0;o<r;o++)Math.abs(this.s[o])<=n?i.set(o,o,0):i.set(o,o,1/this.s[o]);for(var a=this.U,s=this.rightSingularVectors,l=s.mmul(i),u=s.rows,c=a.rows,d=Fi.zeros(u,c),h=0;h<u;h++)for(var f=0;f<c;f++){for(var p=0,g=0;g<r;g++)p+=l.get(h,g)*a.get(f,g);d.set(h,f,p)}return d.mmul(t)}},{key:"solveForDiagonal",value:function(e){return this.solve(Fi.diag(e))}},{key:"inverse",value:function(){for(var e=this.V,t=this.threshold,n=e.rows,r=e.columns,i=new Fi(n,this.s.length),o=0;o<n;o++)for(var a=0;a<r;a++)Math.abs(this.s[a])>t&&i.set(o,a,e.get(o,a)/this.s[a]);for(var s=this.U,l=s.rows,u=s.columns,c=new Fi(n,l),d=0;d<n;d++)for(var h=0;h<l;h++){for(var f=0,p=0;p<u;p++)f+=i.get(d,p)*s.get(h,p);c.set(d,h,f)}return c}},{key:"condition",get:function(){return this.s[0]/this.s[Math.min(this.m,this.n)-1]}},{key:"norm2",get:function(){return this.s[0]}},{key:"rank",get:function(){for(var e=Math.max(this.m,this.n)*this.s[0]*Number.EPSILON,t=0,n=this.s,r=0,i=n.length;r<i;r++)n[r]>e&&t++;return t}},{key:"diagonal",get:function(){return Array.from(this.s)}},{key:"threshold",get:function(){return Number.EPSILON/2*Math.max(this.m,this.n)*this.s[0]}},{key:"leftSingularVectors",get:function(){return this.U}},{key:"rightSingularVectors",get:function(){return this.V}},{key:"diagonalMatrix",get:function(){return Fi.diag(this.s)}}])&&qi(t.prototype,n),e}(),Qi=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.center=[0,0],t.linkDistance=50,t.scaledDistances=null,t}return Object(w.__extends)(t,e),t.prototype.getDefaultCfg=function(){return{center:[0,0],linkDistance:50}},t.prototype.execute=function(){var e=this.nodes,t=this.edges,n=void 0===t?[]:t,r=this.center;if(e&&0!==e.length){if(1===e.length)return e[0].x=r[0],void(e[0].y=r[1]);var i=this.linkDistance,o=Ge({nodes:e,edges:n},!1),a=We(o);this.handleInfinity(a);var s=Ye(a,i);this.scaledDistances=s;var l=this.runMDS();this.positions=l,l.forEach((function(t,n){e[n].x=t[0]+r[0],e[n].y=t[1]+r[1]}))}},t.prototype.runMDS=function(){var e=this.scaledDistances,t=Fi.mul(Fi.pow(e,2),-.5),n=t.mean("row"),r=t.mean("column"),i=t.mean();t.add(i).subRowVector(n).subColumnVector(r);var o=new Zi(t),a=Fi.sqrt(o.diagonalMatrix).diagonal();return o.leftSingularVectors.toJSON().map((function(e){return Fi.mul([e],[a]).toJSON()[0].splice(0,2)}))},t.prototype.handleInfinity=function(e){var t=-999999;e.forEach((function(e){e.forEach((function(e){e!==1/0&&t<e&&(t=e)}))})),e.forEach((function(n,r){n.forEach((function(n,i){n===1/0&&(e[r][i]=t)}))}))},t}(Bn),Ki=function(){function e(e){this.distances=e.distances,this.dimension=e.dimension||2,this.linkDistance=e.linkDistance}return e.prototype.layout=function(){var e=this.dimension,t=this.distances,n=this.linkDistance;try{var r=Fi.mul(Fi.pow(t,2),-.5),i=r.mean("row"),o=r.mean("column"),a=r.mean();r.add(a).subRowVector(i).subColumnVector(o);var s=new Zi(r),l=Fi.sqrt(s.diagonalMatrix).diagonal();return s.leftSingularVectors.toJSON().map((function(t){return Fi.mul([t],[l]).toJSON()[0].splice(0,e)}))}catch(e){for(var u=[],c=0;c<t.length;c++){var d=Math.random()*n,h=Math.random()*n;u.push([d,h])}return u}},e}(),Ji=function(){function e(e){this.disp=[],this.positions=e.positions,this.adjMatrix=e.adjMatrix,this.focusID=e.focusID,this.radii=e.radii,this.iterations=e.iterations||10,this.height=e.height||10,this.width=e.width||10,this.speed=e.speed||100,this.gravity=e.gravity||10,this.nodeSizeFunc=e.nodeSizeFunc,this.k=e.k||5,this.strictRadial=e.strictRadial,this.nodes=e.nodes}return e.prototype.layout=function(){var e=this.positions,t=[],n=this.iterations,r=this.width/10;this.maxDisplace=r,this.disp=t;for(var i=0;i<n;i++)e.forEach((function(e,n){t[n]={x:0,y:0}})),this.getRepulsion(),this.updatePositions();return e},e.prototype.getRepulsion=function(){var e=this,t=e.positions,n=e.nodes,r=e.disp,i=e.k,o=e.radii||[];t.forEach((function(a,s){r[s]={x:0,y:0},t.forEach((function(t,l){if(s!==l&&o[s]===o[l]){var u=a[0]-t[0],c=a[1]-t[1],d=Math.sqrt(u*u+c*c);if(0===d){d=1;var h=s>l?1:-1;u=.01*h,c=.01*h}if(d<e.nodeSizeFunc(n[s])/2+e.nodeSizeFunc(n[l])/2){var f=i*i/d;r[s].x+=u/d*f,r[s].y+=c/d*f}}}))}))},e.prototype.updatePositions=function(){var e=this.positions,t=this.disp,n=this.speed,r=this.strictRadial,i=this.focusID,o=this.maxDisplace||this.width/10;r&&t.forEach((function(t,n){var r=e[n][0]-e[i][0],o=e[n][1]-e[i][1],a=Math.sqrt(r*r+o*o),s=o/a,l=-r/a,u=Math.sqrt(t.x*t.x+t.y*t.y),c=Math.acos((s*t.x+l*t.y)/u);c>Math.PI/2&&(c-=Math.PI/2,s*=-1,l*=-1);var d=Math.cos(c)*u;t.x=s*d,t.y=l*d}));var a=this.radii;e.forEach((function(s,l){if(l!==i){var u=Math.sqrt(t[l].x*t[l].x+t[l].y*t[l].y);if(u>0&&l!==i){var c=Math.min(o*(n/800),u);if(s[0]+=t[l].x/u*c,s[1]+=t[l].y/u*c,r){var d=s[0]-e[i][0],h=s[1]-e[i][1],f=Math.sqrt(d*d+h*h);d=d/f*a[l],h=h/f*a[l],s[0]=e[i][0]+d,s[1]=e[i][1]+h}}}}))},e}();function $i(e,t){return Math.sqrt((e[0]-t[0])*(e[0]-t[0])+(e[1]-t[1])*(e[1]-t[1]))}var eo=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.center=[0,0],t.maxIteration=1e3,t.focusNode=null,t.unitRadius=null,t.linkDistance=50,t.preventOverlap=!1,t.strictRadial=!0,t.maxPreventOverlapIteration=200,t.sortStrength=10,t}return Object(w.__extends)(t,e),t.prototype.getDefaultCfg=function(){return{center:[0,0],maxIteration:1e3,focusNode:null,unitRadius:null,linkDistance:50,preventOverlap:!1,nodeSize:void 0,nodeSpacing:void 0,strictRadial:!0,maxPreventOverlapIteration:200,sortBy:void 0,sortStrength:10}},t.prototype.execute=function(){var e=this,t=e.nodes,n=e.edges||[],r=e.center;if(t&&0!==t.length){if(1===t.length)return t[0].x=r[0],void(t[0].y=r[1]);var i=e.linkDistance,o=null;if(x()(e.focusNode)){for(var a=!1,s=0;s<t.length;s++)t[s].id===e.focusNode&&(o=t[s],e.focusNode=o,a=!0,s=t.length);a||(o=null)}else o=e.focusNode;o||(o=t[0],e.focusNode=o);var l,u,c,d=(l=t,u=o.id,c=-1,l.forEach((function(e,t){e.id===u&&(c=t)})),c);e.focusIndex=d;var h=Ge({nodes:t,edges:n},!1),f=We(h),p=e.maxToFocus(f,d);e.handleInfinity(f,d,p+1),e.distances=f;var g=f[d];e.width||"undefined"==typeof window||(e.width=window.innerWidth),e.height||"undefined"==typeof window||(e.height=window.innerHeight);var v=e.width||500,m=e.height||500,b=v-r[0]>r[0]?r[0]:v-r[0],C=m-r[1]>r[1]?r[1]:m-r[1];0===b&&(b=v/2),0===C&&(C=m/2);var A=C>b?b:C,I=Math.max.apply(Math,g),w=[];g.forEach((function(t,n){e.unitRadius||(e.unitRadius=A/I),w[n]=t*e.unitRadius})),e.radii=w;var M=e.eIdealDisMatrix();e.eIdealDistances=M;var S=function(e){for(var t=e.length,n=e[0].length,r=[],i=0;i<t;i++){for(var o=[],a=0;a<n;a++)0!==e[i][a]?o.push(1/(e[i][a]*e[i][a])):o.push(0);r.push(o)}return r}(M);e.weights=S;var T=new Ki({distances:M,linkDistance:i}).layout();T.forEach((function(e){k(e[0])&&(e[0]=Math.random()*i),k(e[1])&&(e[1]=Math.random()*i)})),e.positions=T,T.forEach((function(e,n){t[n].x=e[0]+r[0],t[n].y=e[1]+r[1]})),T.forEach((function(e){e[0]-=T[d][0],e[1]-=T[d][1]})),e.run();var E=e.preventOverlap,O=e.nodeSize,L=e.strictRadial;if(E){var D,P=e.nodeSpacing;D=_()(P)?function(){return P}:Yr()(P)?P:function(){return 0};var N={nodeSizeFunc:O?y()(O)?function(e){return(O[0]>O[1]?O[0]:O[1])+D(e)}:function(e){return O+D(e)}:function(e){return e.size?y()(e.size)?(e.size[0]>e.size[1]?e.size[0]:e.size[1])+D(e):e.size+D(e):10+D(e)},adjMatrix:h,positions:T,radii:w,height:m,width:v,strictRadial:L,focusID:d,iterations:e.maxPreventOverlapIteration||200,k:T.length/4.5,nodes:t},j=new Ji(N);T=j.layout()}T.forEach((function(e,n){t[n].x=e[0]+r[0],t[n].y=e[1]+r[1]}))}},t.prototype.run=function(){for(var e=this.maxIteration,t=this.positions||[],n=this.weights||[],r=this.eIdealDistances||[],i=this.radii||[],o=0;o<=e;o++){var a=o/e;this.oneIteration(a,t,i,r,n)}},t.prototype.oneIteration=function(e,t,n,r,i){var o=1-e,a=this.focusIndex;t.forEach((function(s,l){var u=$i(s,[0,0]),c=0===u?0:1/u;if(l!==a){var d=0,h=0,f=0;t.forEach((function(e,t){if(l!==t){var n=$i(s,e),o=0===n?0:1/n,a=r[t][l];f+=i[l][t],d+=i[l][t]*(e[0]+a*(s[0]-e[0])*o),h+=i[l][t]*(e[1]+a*(s[1]-e[1])*o)}}));var p=0===n[l]?0:1/n[l];f*=o,f+=e*p*p,d*=o,d+=e*p*s[0]*c,s[0]=d/f,h*=o,h+=e*p*s[1]*c,s[1]=h/f}}))},t.prototype.eIdealDisMatrix=function(){var e=this,t=e.nodes;if(!t)return[];var n=e.distances,r=e.linkDistance,i=e.radii||[],o=e.unitRadius||50,a=[];return n&&n.forEach((function(n,s){var l=[];n.forEach((function(n,a){if(s===a)l.push(0);else if(i[s]===i[a])if("data"===e.sortBy)l.push(n*(Math.abs(s-a)*e.sortStrength)/(i[s]/o));else if(e.sortBy){var u=t[s][e.sortBy]||0,c=t[a][e.sortBy]||0;x()(u)&&(u=u.charCodeAt(0)),x()(c)&&(c=c.charCodeAt(0)),l.push(n*(Math.abs(u-c)*e.sortStrength)/(i[s]/o))}else l.push(n*r/(i[s]/o));else{var d=(r+o)/2;l.push(n*d)}})),a.push(l)})),a},t.prototype.handleInfinity=function(e,t,n){for(var r=e.length,i=0;i<r;i++)if(e[t][i]===1/0){e[t][i]=n,e[i][t]=n;for(var o=0;o<r;o++)e[i][o]!==1/0&&e[t][o]===1/0&&(e[t][o]=n+e[i][o],e[o][t]=n+e[i][o])}for(i=0;i<r;i++)if(i!==t)for(o=0;o<r;o++)if(e[i][o]===1/0){var a=Math.abs(e[t][i]-e[t][o]);a=0===a?1:a,e[i][o]=a}},t.prototype.maxToFocus=function(e,t){for(var n=0,r=0;r<e[t].length;r++)e[t][r]!==1/0&&(n=e[t][r]>n?e[t][r]:n);return n},t}(Bn),to=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.center=[0,0],t.width=300,t.height=300,t}return Object(w.__extends)(t,e),t.prototype.getDefaultCfg=function(){return{center:[0,0],width:300,height:300}},t.prototype.execute=function(){var e=this,t=e.nodes,n=e.center;e.width||"undefined"==typeof window||(e.width=window.innerWidth),e.height||"undefined"==typeof window||(e.height=window.innerHeight),t&&t.forEach((function(t){t.x=.9*(Math.random()-.5)*e.width+n[0],t.y=.9*(Math.random()-.5)*e.height+n[1]}))},t}(Bn),no={circular:Yn,concentric:Wn,dagre:qn,force:Gr,g6force:Xr,comboForce:function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.center=[0,0],t.maxIteration=100,t.gravity=10,t.comboGravity=10,t.linkDistance=10,t.alpha=1,t.alphaMin=.001,t.alphaDecay=1-Math.pow(t.alphaMin,1/300),t.alphaTarget=0,t.velocityDecay=.6,t.edgeStrength=.2,t.nodeStrength=30,t.preventOverlap=!1,t.preventNodeOverlap=!1,t.preventComboOverlap=!1,t.collideStrength=void 0,t.nodeCollideStrength=void 0,t.comboCollideStrength=void 0,t.optimizeRangeFactor=1,t.onTick=function(){},t.onLayoutEnd=function(){},t.depthAttractiveForceScale=.5,t.depthRepulsiveForceScale=2,t.nodes=[],t.edges=[],t.combos=[],t.comboTrees=[],t.width=300,t.height=300,t.bias=[],t.nodeMap={},t.oriComboMap={},t.nodeIdxMap={},t.comboMap={},t.previousLayouted=!1,t}return Object(w.__extends)(t,e),t.prototype.getDefaultCfg=function(){return{maxIteration:100,center:[0,0],gravity:10,speed:1,comboGravity:30,preventOverlap:!1,preventComboOverlap:!0,preventNodeOverlap:!0,nodeSpacing:void 0,collideStrength:void 0,nodeCollideStrength:.5,comboCollideStrength:.5,comboSpacing:20,comboPadding:10,edgeStrength:.2,nodeStrength:30,linkDistance:10}},t.prototype.execute=function(){var e=this.nodes,t=this.center;if(this.comboTree={id:"comboTreeRoot",depth:-1,children:this.comboTrees},e&&0!==e.length){if(1===e.length)return e[0].x=t[0],void(e[0].y=t[1]);this.initVals(),this.run(),this.onLayoutEnd()}},t.prototype.run=function(){var e=this,t=e.nodes,n=e.previousLayouted?e.maxIteration/5:e.maxIteration;e.width||"undefined"==typeof window||(e.width=window.innerWidth),e.height||"undefined"==typeof window||(e.height=window.innerHeight);var r=e.center,i=e.velocityDecay,o=e.comboMap;e.previousLayouted||e.initPos(o);for(var a=function(n){var r=[];t.forEach((function(e,t){r[t]={x:0,y:0}})),e.applyCalculate(r),e.applyComboCenterForce(r),t.forEach((function(e,t){Object(ce.isNumber)(e.x)&&Object(ce.isNumber)(e.y)&&(e.x+=r[t].x*i,e.y+=r[t].y*i)})),e.alpha+=(e.alphaTarget-e.alpha)*e.alphaDecay,e.onTick()},s=0;s<n;s++)a();var l=[0,0];t.forEach((function(e){Object(ce.isNumber)(e.x)&&Object(ce.isNumber)(e.y)&&(l[0]+=e.x,l[1]+=e.y)})),l[0]/=t.length,l[1]/=t.length;var u=[r[0]-l[0],r[1]-l[1]];t.forEach((function(e,t){Object(ce.isNumber)(e.x)&&Object(ce.isNumber)(e.y)&&(e.x+=u[0],e.y+=u[1])})),e.combos.forEach((function(e){var t=o[e.id];t&&t.empty&&(e.x=t.cx||e.x,e.y=t.cy||e.y)})),e.previousLayouted=!0},t.prototype.initVals=function(){var e=this.edges,t=this.nodes,n=this.combos,r={},i={},o={};t.forEach((function(e,t){i[e.id]=e,o[e.id]=t})),this.nodeMap=i,this.nodeIdxMap=o;var a={};n.forEach((function(e){a[e.id]=e})),this.oriComboMap=a,this.comboMap=this.getComboMap();var s=this.preventOverlap;this.preventComboOverlap=this.preventComboOverlap||s,this.preventNodeOverlap=this.preventNodeOverlap||s;var l=this.collideStrength;l&&(this.comboCollideStrength=l,this.nodeCollideStrength=l);for(var u=0;u<e.length;++u)r[e[u].source]?r[e[u].source]++:r[e[u].source]=1,r[e[u].target]?r[e[u].target]++:r[e[u].target]=1;var c=[];for(u=0;u<e.length;++u)c[u]=r[e[u].source]/(r[e[u].source]+r[e[u].target]);this.bias=c;var d,h,f=this.nodeSize,p=this.nodeSpacing;if(h=Object(ce.isNumber)(p)?function(){return p}:Object(ce.isFunction)(p)?p:function(){return 0},this.nodeSpacing=h,f)if(Object(ce.isFunction)(f))d=function(e){return f(e)};else if(Object(ce.isArray)(f)){var g=(f[0]>f[1]?f[0]:f[1])/2;d=function(e){return g}}else{var v=f/2;d=function(e){return v}}else d=function(e){return e.size?Object(ce.isArray)(e.size)?(e.size[0]>e.size[1]?e.size[0]:e.size[1])/2:e.size/2:10};this.nodeSize=d;var m,y=this.comboSpacing;m=Object(ce.isNumber)(y)?function(){return y}:Object(ce.isFunction)(y)?y:function(){return 0},this.comboSpacing=m;var b,C=this.comboPadding;b=Object(ce.isNumber)(C)?function(){return C}:Object(ce.isArray)(C)?function(){return Math.max.apply(null,C)}:Object(ce.isFunction)(C)?C:function(){return 0},this.comboPadding=b;var A,_=this.linkDistance;_||(_=10),A=Object(ce.isNumber)(_)?function(e){return _}:_,this.linkDistance=A;var I,x=this.edgeStrength;x||(x=1),I=Object(ce.isNumber)(x)?function(e){return x}:x,this.edgeStrength=I;var w,M=this.nodeStrength;M||(M=30),w=Object(ce.isNumber)(M)?function(e){return M}:M,this.nodeStrength=w},t.prototype.initPos=function(e){this.nodes.forEach((function(t,n){if(t.comboId){var r=e[t.comboId];t.x=r.cx+100/(n+1),t.y=r.cy+100/(n+1)}else t.x=100/(n+1),t.y=100/(n+1)}))},t.prototype.getComboMap=function(){var e=this,t=e.nodeMap,n=e.nodeIdxMap,r=e.comboTrees,i=e.oriComboMap,o={};return(r||[]).forEach((function(r){var a=[];gn(r,(function(r){if("node"===r.itemType)return!0;if(!i[r.id])return!0;if(void 0===o[r.id]){var s={name:r.id,cx:0,cy:0,count:0,depth:e.oriComboMap[r.id].depth,children:[]};o[r.id]=s}var l=r.children;l&&l.forEach((function(e){if(!o[e.id]&&!t[e.id])return!0;a.push(e)}));var u=o[r.id];if(u.cx=0,u.cy=0,0===a.length){u.empty=!0;var c=i[r.id],d=Object.keys(t).length,h=r.id+"-visual-child-"+d,f={id:h,x:c.x,y:c.y,depth:u.depth+1,itemType:"node"};e.nodes.push(f),t[h]=f,n[h]=d,u.cx=c.x,u.cy=c.y,a.push(f)}return a.forEach((function(e){if(u.count++,"node"!==e.itemType){var n=o[e.id];return Object(ce.isNumber)(n.cx)&&(u.cx+=n.cx),void(Object(ce.isNumber)(n.cy)&&(u.cy+=n.cy))}var r=t[e.id];r&&(Object(ce.isNumber)(r.x)&&(u.cx+=r.x),Object(ce.isNumber)(r.y)&&(u.cy+=r.y))})),u.cx/=u.count,u.cy/=u.count,u.children=a,!0}))})),o},t.prototype.applyComboCenterForce=function(e){var t=this.gravity,n=this.comboGravity||t,r=this.alpha,i=this.comboTrees,o=this.nodeIdxMap,a=this.nodeMap,s=this.comboMap;(i||[]).forEach((function(t){gn(t,(function(t){if("node"===t.itemType)return!0;if(!s[t.id])return!0;var i=s[t.id],l=.5*(i.depth+1),u=i.cx,c=i.cy;return i.cx=0,i.cy=0,i.children.forEach((function(t){if("node"!==t.itemType){var d=s[t.id];return d&&Object(ce.isNumber)(d.cx)&&(i.cx+=d.cx),void(d&&Object(ce.isNumber)(d.cy)&&(i.cy+=d.cy))}var h=a[t.id],f=h.x-u||.005,p=h.y-c||.005,g=Math.sqrt(f*f+p*p),v=o[h.id],m=n*r/g*l;e[v].x-=f*m,e[v].y-=p*m,Object(ce.isNumber)(h.x)&&(i.cx+=h.x),Object(ce.isNumber)(h.y)&&(i.cy+=h.y)})),i.cx/=i.count,i.cy/=i.count,!0}))}))},t.prototype.applyCalculate=function(e){var t=this.comboMap,n=this.nodes,r={};n.forEach((function(e,t){n.forEach((function(n,i){if(!(t<i)){var o=e.x-n.x||.005,a=e.y-n.y||.005,s=o*o+a*a,l=Math.sqrt(s);s<1&&(s=l),r[e.id+"-"+n.id]={vx:o,vy:a,vl2:s,vl:l},r[n.id+"-"+e.id]={vx:-o,vy:-a,vl2:s,vl:l}}}))})),this.updateComboSizes(t),this.calRepulsive(e,r),this.calAttractive(e,r),this.preventComboOverlap&&this.comboNonOverlapping(e,t)},t.prototype.updateComboSizes=function(e){var t=this,n=t.comboTrees,r=t.nodeMap,i=t.nodeSize,o=t.comboSpacing,a=t.comboPadding;(n||[]).forEach((function(n){var s=[];gn(n,(function(n){if("node"===n.itemType)return!0;var l=e[n.id];if(l){var u=n.children;u&&u.forEach((function(t){(e[t.id]||r[t.id])&&s.push(t)})),l.minX=1/0,l.minY=1/0,l.maxX=-1/0,l.maxY=-1/0,s.forEach((function(e){if("node"!==e.itemType)return!0;var t=r[e.id];if(!t)return!0;var n=i(t),o=t.x-n,a=t.y-n,s=t.x+n,u=t.y+n;l.minX>o&&(l.minX=o),l.minY>a&&(l.minY=a),l.maxX<s&&(l.maxX=s),l.maxY<u&&(l.maxY=u)}));var c=t.oriComboMap[n.id].size||J.defaultCombo.size;Object(ce.isArray)(c)&&(c=c[0]);var d=Math.max(l.maxX-l.minX,l.maxY-l.minY,c);return l.r=d/2+o(l)/2+a(l),!0}}))}))},t.prototype.comboNonOverlapping=function(e,t){var n=this.comboTree,r=this.comboCollideStrength,i=this.nodeIdxMap,o=this.nodeMap;gn(n,(function(n){if(t[n.id]||o[n.id]||"comboTreeRoot"===n.id){var a=n.children;return a&&a.length>1&&a.forEach((function(n,s){if("node"!==n.itemType){var l=t[n.id];l&&a.forEach((function(a,u){if(!(s<=u)&&"node"!==a.itemType){var c=t[a.id];if(c){var d=l.cx-c.cx||.005,h=l.cy-c.cy||.005,f=d*d+h*h,p=l.r,g=c.r,v=p+g,m=g*g,y=p*p;if(f<v*v){var b=n.children;if(!b||0===b.length)return;var C=a.children;if(!C||0===C.length)return;var A=Math.sqrt(f),_=(v-A)/A*r,I=d*_,x=h*_,w=m/(y+m),M=1-w;b.forEach((function(t){if("node"===t.itemType&&o[t.id]){var n=i[t.id];C.forEach((function(t){if("node"===t.itemType&&o[t.id]){var r=i[t.id];e[n].x+=I*w,e[n].y+=x*w,e[r].x-=I*M,e[r].y-=x*M}}))}}))}}}}))}})),!0}}))},t.prototype.calRepulsive=function(e,t){var n=this,r=n.nodes,i=n.width*n.optimizeRangeFactor,o=n.nodeStrength,a=n.alpha,s=n.nodeCollideStrength,l=n.preventNodeOverlap,u=n.nodeSize,c=n.nodeSpacing,d=n.depthRepulsiveForceScale,h=n.center;r.forEach((function(f,p){if(f.x&&f.y){if(h){var g=n.gravity,v=f.x-h[0]||.005,m=f.y-h[1]||.005,y=Math.sqrt(v*v+m*m);e[p].x-=v*g*a/y,e[p].y-=m*g*a/y}r.forEach((function(n,r){if(p!==r&&n.x&&n.y){var h=t[f.id+"-"+n.id],g=h.vl2,v=h.vl;if(!(v>i)){var m=t[f.id+"-"+n.id],y=m.vx,b=m.vy,C=Math.abs(n.depth-f.depth)+1||1;n.comboId!==f.comboId&&C++;var A=C?Math.pow(d,C):1,_=o(n)*a/g*A;if(e[p].x+=y*_,e[p].y+=b*_,p<r&&l){var I=u(f)+c(f),x=u(n)+c(n),w=I+x;if(g<w*w){var M=(w-v)/v*s,S=x*x,T=S/(I*I+S),E=y*M,O=b*M;e[p].x+=E*T,e[p].y+=O*T,T=1-T,e[r].x-=E*T,e[r].y-=O*T}}}}}))}}))},t.prototype.calAttractive=function(e,t){var n=this,r=n.edges,i=n.linkDistance,o=n.alpha,a=n.edgeStrength,s=n.bias,l=n.depthAttractiveForceScale;r.forEach((function(r,u){if(r.source&&r.target&&r.source!==r.target){var c=n.nodeIdxMap[r.source],d=n.nodeIdxMap[r.target],h=n.nodeMap[r.source],f=n.nodeMap[r.target],p=Math.abs(h.depth-f.depth);h.comboId===f.comboId&&(p/=2);var g=p?Math.pow(l,p):1;if(h.comboId!==f.comboId&&1===g?g=l/2:h.comboId===f.comboId&&(g=2),Object(ce.isNumber)(f.x)&&Object(ce.isNumber)(h.x)&&Object(ce.isNumber)(f.y)&&Object(ce.isNumber)(h.y)){var v=t[r.target+"-"+r.source],m=v.vl,y=v.vx,b=v.vy,C=(m-i(r))/m*o*a(r)*g,A=y*C,_=b*C,I=s[u];e[d].x-=A*I,e[d].y-=_*I,e[c].x+=A*(1-I),e[c].y+=_*(1-I)}}}))},t}(Bn),fruchterman:qr,grid:Zr,mds:Qi,radial:eo,random:to};f()(no,(function(e,t){Un.registerLayout(t,{},e)}));var ro=Un,io=n(231),oo=n.n(io),ao=function(){return oo()('!function(t){var e={};function r(n){if(e[n])return e[n].exports;var o=e[n]={i:n,l:!1,exports:{}};return t[n].call(o.exports,o,o.exports,r),o.l=!0,o.exports}r.m=t,r.c=e,r.d=function(t,e,n){r.o(t,e)||Object.defineProperty(t,e,{enumerable:!0,get:n})},r.r=function(t){"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(t,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(t,"__esModule",{value:!0})},r.t=function(t,e){if(1&e&&(t=r(t)),8&e)return t;if(4&e&&"object"==typeof t&&t&&t.__esModule)return t;var n=Object.create(null);if(r.r(n),Object.defineProperty(n,"default",{enumerable:!0,value:t}),2&e&&"string"!=typeof t)for(var o in t)r.d(n,o,function(e){return t[e]}.bind(null,o));return n},r.n=function(t){var e=t&&t.__esModule?function(){return t.default}:function(){return t};return r.d(e,"a",e),e},r.o=function(t,e){return Object.prototype.hasOwnProperty.call(t,e)},r.p="",r(r.s=317)}([function(t,e,r){var n;try{n={cloneDeep:r(250),constant:r(67),defaults:r(251),each:r(93),filter:r(96),find:r(252),flatten:r(123),forEach:r(94),forIn:r(257),has:r(107),isUndefined:r(108),last:r(258),map:r(109),mapValues:r(259),max:r(260),merge:r(262),min:r(268),minBy:r(269),now:r(270),pick:r(271),range:r(276),reduce:r(111),sortBy:r(279),uniqueId:r(284),values:r(116),zipObject:r(285)}}catch(t){}n||(n=window._),t.exports=n},function(t,e){var r=Array.isArray;t.exports=r},function(t,e,r){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var n=r(34);e.default=function(t){return Array.isArray?Array.isArray(t):n.default(t,"Array")}},function(t,e,r){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),e.sub=e.mul=void 0,e.create=function(){var t=new n.ARRAY_TYPE(9);n.ARRAY_TYPE!=Float32Array&&(t[1]=0,t[2]=0,t[3]=0,t[5]=0,t[6]=0,t[7]=0);return t[0]=1,t[4]=1,t[8]=1,t},e.fromMat4=function(t,e){return t[0]=e[0],t[1]=e[1],t[2]=e[2],t[3]=e[4],t[4]=e[5],t[5]=e[6],t[6]=e[8],t[7]=e[9],t[8]=e[10],t},e.clone=function(t){var e=new n.ARRAY_TYPE(9);return e[0]=t[0],e[1]=t[1],e[2]=t[2],e[3]=t[3],e[4]=t[4],e[5]=t[5],e[6]=t[6],e[7]=t[7],e[8]=t[8],e},e.copy=function(t,e){return t[0]=e[0],t[1]=e[1],t[2]=e[2],t[3]=e[3],t[4]=e[4],t[5]=e[5],t[6]=e[6],t[7]=e[7],t[8]=e[8],t},e.fromValues=function(t,e,r,o,i,a,u,s,c){var f=new n.ARRAY_TYPE(9);return f[0]=t,f[1]=e,f[2]=r,f[3]=o,f[4]=i,f[5]=a,f[6]=u,f[7]=s,f[8]=c,f},e.set=function(t,e,r,n,o,i,a,u,s,c){return t[0]=e,t[1]=r,t[2]=n,t[3]=o,t[4]=i,t[5]=a,t[6]=u,t[7]=s,t[8]=c,t},e.identity=function(t){return t[0]=1,t[1]=0,t[2]=0,t[3]=0,t[4]=1,t[5]=0,t[6]=0,t[7]=0,t[8]=1,t},e.transpose=function(t,e){if(t===e){var r=e[1],n=e[2],o=e[5];t[1]=e[3],t[2]=e[6],t[3]=r,t[5]=e[7],t[6]=n,t[7]=o}else t[0]=e[0],t[1]=e[3],t[2]=e[6],t[3]=e[1],t[4]=e[4],t[5]=e[7],t[6]=e[2],t[7]=e[5],t[8]=e[8];return t},e.invert=function(t,e){var r=e[0],n=e[1],o=e[2],i=e[3],a=e[4],u=e[5],s=e[6],c=e[7],f=e[8],h=f*a-u*c,l=-f*i+u*s,d=c*i-a*s,p=r*h+n*l+o*d;if(!p)return null;return p=1/p,t[0]=h*p,t[1]=(-f*n+o*c)*p,t[2]=(u*n-o*a)*p,t[3]=l*p,t[4]=(f*r-o*s)*p,t[5]=(-u*r+o*i)*p,t[6]=d*p,t[7]=(-c*r+n*s)*p,t[8]=(a*r-n*i)*p,t},e.adjoint=function(t,e){var r=e[0],n=e[1],o=e[2],i=e[3],a=e[4],u=e[5],s=e[6],c=e[7],f=e[8];return t[0]=a*f-u*c,t[1]=o*c-n*f,t[2]=n*u-o*a,t[3]=u*s-i*f,t[4]=r*f-o*s,t[5]=o*i-r*u,t[6]=i*c-a*s,t[7]=n*s-r*c,t[8]=r*a-n*i,t},e.determinant=function(t){var e=t[0],r=t[1],n=t[2],o=t[3],i=t[4],a=t[5],u=t[6],s=t[7],c=t[8];return e*(c*i-a*s)+r*(-c*o+a*u)+n*(s*o-i*u)},e.multiply=o,e.translate=function(t,e,r){var n=e[0],o=e[1],i=e[2],a=e[3],u=e[4],s=e[5],c=e[6],f=e[7],h=e[8],l=r[0],d=r[1];return t[0]=n,t[1]=o,t[2]=i,t[3]=a,t[4]=u,t[5]=s,t[6]=l*n+d*a+c,t[7]=l*o+d*u+f,t[8]=l*i+d*s+h,t},e.rotate=function(t,e,r){var n=e[0],o=e[1],i=e[2],a=e[3],u=e[4],s=e[5],c=e[6],f=e[7],h=e[8],l=Math.sin(r),d=Math.cos(r);return t[0]=d*n+l*a,t[1]=d*o+l*u,t[2]=d*i+l*s,t[3]=d*a-l*n,t[4]=d*u-l*o,t[5]=d*s-l*i,t[6]=c,t[7]=f,t[8]=h,t},e.scale=function(t,e,r){var n=r[0],o=r[1];return t[0]=n*e[0],t[1]=n*e[1],t[2]=n*e[2],t[3]=o*e[3],t[4]=o*e[4],t[5]=o*e[5],t[6]=e[6],t[7]=e[7],t[8]=e[8],t},e.fromTranslation=function(t,e){return t[0]=1,t[1]=0,t[2]=0,t[3]=0,t[4]=1,t[5]=0,t[6]=e[0],t[7]=e[1],t[8]=1,t},e.fromRotation=function(t,e){var r=Math.sin(e),n=Math.cos(e);return t[0]=n,t[1]=r,t[2]=0,t[3]=-r,t[4]=n,t[5]=0,t[6]=0,t[7]=0,t[8]=1,t},e.fromScaling=function(t,e){return t[0]=e[0],t[1]=0,t[2]=0,t[3]=0,t[4]=e[1],t[5]=0,t[6]=0,t[7]=0,t[8]=1,t},e.fromMat2d=function(t,e){return t[0]=e[0],t[1]=e[1],t[2]=0,t[3]=e[2],t[4]=e[3],t[5]=0,t[6]=e[4],t[7]=e[5],t[8]=1,t},e.fromQuat=function(t,e){var r=e[0],n=e[1],o=e[2],i=e[3],a=r+r,u=n+n,s=o+o,c=r*a,f=n*a,h=n*u,l=o*a,d=o*u,p=o*s,v=i*a,g=i*u,y=i*s;return t[0]=1-h-p,t[3]=f-y,t[6]=l+g,t[1]=f+y,t[4]=1-c-p,t[7]=d-v,t[2]=l-g,t[5]=d+v,t[8]=1-c-h,t},e.normalFromMat4=function(t,e){var r=e[0],n=e[1],o=e[2],i=e[3],a=e[4],u=e[5],s=e[6],c=e[7],f=e[8],h=e[9],l=e[10],d=e[11],p=e[12],v=e[13],g=e[14],y=e[15],m=r*u-n*a,b=r*s-o*a,w=r*c-i*a,x=n*s-o*u,M=n*c-i*u,k=o*c-i*s,_=f*v-h*p,S=f*g-l*p,E=f*y-d*p,j=h*g-l*v,A=h*y-d*v,O=l*y-d*g,N=m*O-b*A+w*j+x*E-M*S+k*_;if(!N)return null;return N=1/N,t[0]=(u*O-s*A+c*j)*N,t[1]=(s*E-a*O-c*S)*N,t[2]=(a*A-u*E+c*_)*N,t[3]=(o*A-n*O-i*j)*N,t[4]=(r*O-o*E+i*S)*N,t[5]=(n*E-r*A-i*_)*N,t[6]=(v*k-g*M+y*x)*N,t[7]=(g*w-p*k-y*b)*N,t[8]=(p*M-v*w+y*m)*N,t},e.projection=function(t,e,r){return t[0]=2/e,t[1]=0,t[2]=0,t[3]=0,t[4]=-2/r,t[5]=0,t[6]=-1,t[7]=1,t[8]=1,t},e.str=function(t){return"mat3("+t[0]+", "+t[1]+", "+t[2]+", "+t[3]+", "+t[4]+", "+t[5]+", "+t[6]+", "+t[7]+", "+t[8]+")"},e.frob=function(t){return Math.sqrt(Math.pow(t[0],2)+Math.pow(t[1],2)+Math.pow(t[2],2)+Math.pow(t[3],2)+Math.pow(t[4],2)+Math.pow(t[5],2)+Math.pow(t[6],2)+Math.pow(t[7],2)+Math.pow(t[8],2))},e.add=function(t,e,r){return t[0]=e[0]+r[0],t[1]=e[1]+r[1],t[2]=e[2]+r[2],t[3]=e[3]+r[3],t[4]=e[4]+r[4],t[5]=e[5]+r[5],t[6]=e[6]+r[6],t[7]=e[7]+r[7],t[8]=e[8]+r[8],t},e.subtract=i,e.multiplyScalar=function(t,e,r){return t[0]=e[0]*r,t[1]=e[1]*r,t[2]=e[2]*r,t[3]=e[3]*r,t[4]=e[4]*r,t[5]=e[5]*r,t[6]=e[6]*r,t[7]=e[7]*r,t[8]=e[8]*r,t},e.multiplyScalarAndAdd=function(t,e,r,n){return t[0]=e[0]+r[0]*n,t[1]=e[1]+r[1]*n,t[2]=e[2]+r[2]*n,t[3]=e[3]+r[3]*n,t[4]=e[4]+r[4]*n,t[5]=e[5]+r[5]*n,t[6]=e[6]+r[6]*n,t[7]=e[7]+r[7]*n,t[8]=e[8]+r[8]*n,t},e.exactEquals=function(t,e){return t[0]===e[0]&&t[1]===e[1]&&t[2]===e[2]&&t[3]===e[3]&&t[4]===e[4]&&t[5]===e[5]&&t[6]===e[6]&&t[7]===e[7]&&t[8]===e[8]},e.equals=function(t,e){var r=t[0],o=t[1],i=t[2],a=t[3],u=t[4],s=t[5],c=t[6],f=t[7],h=t[8],l=e[0],d=e[1],p=e[2],v=e[3],g=e[4],y=e[5],m=e[6],b=e[7],w=e[8];return Math.abs(r-l)<=n.EPSILON*Math.max(1,Math.abs(r),Math.abs(l))&&Math.abs(o-d)<=n.EPSILON*Math.max(1,Math.abs(o),Math.abs(d))&&Math.abs(i-p)<=n.EPSILON*Math.max(1,Math.abs(i),Math.abs(p))&&Math.abs(a-v)<=n.EPSILON*Math.max(1,Math.abs(a),Math.abs(v))&&Math.abs(u-g)<=n.EPSILON*Math.max(1,Math.abs(u),Math.abs(g))&&Math.abs(s-y)<=n.EPSILON*Math.max(1,Math.abs(s),Math.abs(y))&&Math.abs(c-m)<=n.EPSILON*Math.max(1,Math.abs(c),Math.abs(m))&&Math.abs(f-b)<=n.EPSILON*Math.max(1,Math.abs(f),Math.abs(b))&&Math.abs(h-w)<=n.EPSILON*Math.max(1,Math.abs(h),Math.abs(w))};var n=function(t){if(t&&t.__esModule)return t;var e={};if(null!=t)for(var r in t)Object.prototype.hasOwnProperty.call(t,r)&&(e[r]=t[r]);return e.default=t,e}(r(55));function o(t,e,r){var n=e[0],o=e[1],i=e[2],a=e[3],u=e[4],s=e[5],c=e[6],f=e[7],h=e[8],l=r[0],d=r[1],p=r[2],v=r[3],g=r[4],y=r[5],m=r[6],b=r[7],w=r[8];return t[0]=l*n+d*a+p*c,t[1]=l*o+d*u+p*f,t[2]=l*i+d*s+p*h,t[3]=v*n+g*a+y*c,t[4]=v*o+g*u+y*f,t[5]=v*i+g*s+y*h,t[6]=m*n+b*a+w*c,t[7]=m*o+b*u+w*f,t[8]=m*i+b*s+w*h,t}function i(t,e,r){return t[0]=e[0]-r[0],t[1]=e[1]-r[1],t[2]=e[2]-r[2],t[3]=e[3]-r[3],t[4]=e[4]-r[4],t[5]=e[5]-r[5],t[6]=e[6]-r[6],t[7]=e[7]-r[7],t[8]=e[8]-r[8],t}e.mul=o,e.sub=i},function(t,e,r){"use strict";var n=r(0),o=r(7).Graph;function i(t,e,r,o){var i;do{i=n.uniqueId(o)}while(t.hasNode(i));return r.dummy=e,t.setNode(i,r),i}function a(t){return n.max(n.map(t.nodes(),(function(e){var r=t.node(e).rank;if(!n.isUndefined(r))return r})))}t.exports={addDummyNode:i,simplify:function(t){var e=(new o).setGraph(t.graph());return n.forEach(t.nodes(),(function(r){e.setNode(r,t.node(r))})),n.forEach(t.edges(),(function(r){var n=e.edge(r.v,r.w)||{weight:0,minlen:1},o=t.edge(r);e.setEdge(r.v,r.w,{weight:n.weight+o.weight,minlen:Math.max(n.minlen,o.minlen)})})),e},asNonCompoundGraph:function(t){var e=new o({multigraph:t.isMultigraph()}).setGraph(t.graph());return n.forEach(t.nodes(),(function(r){t.children(r).length||e.setNode(r,t.node(r))})),n.forEach(t.edges(),(function(r){e.setEdge(r,t.edge(r))})),e},successorWeights:function(t){var e=n.map(t.nodes(),(function(e){var r={};return n.forEach(t.outEdges(e),(function(e){r[e.w]=(r[e.w]||0)+t.edge(e).weight})),r}));return n.zipObject(t.nodes(),e)},predecessorWeights:function(t){var e=n.map(t.nodes(),(function(e){var r={};return n.forEach(t.inEdges(e),(function(e){r[e.v]=(r[e.v]||0)+t.edge(e).weight})),r}));return n.zipObject(t.nodes(),e)},intersectRect:function(t,e){var r,n,o=t.x,i=t.y,a=e.x-o,u=e.y-i,s=t.width/2,c=t.height/2;if(!a&&!u)throw new Error("Not possible to find intersection inside of the rectangle");Math.abs(u)*s>Math.abs(a)*c?(u<0&&(c=-c),r=c*a/u,n=c):(a<0&&(s=-s),r=s,n=s*u/a);return{x:o+r,y:i+n}},buildLayerMatrix:function(t){var e=n.map(n.range(a(t)+1),(function(){return[]}));return n.forEach(t.nodes(),(function(r){var o=t.node(r),i=o.rank;n.isUndefined(i)||(e[i][o.order]=r)})),e},normalizeRanks:function(t){var e=n.min(n.map(t.nodes(),(function(e){return t.node(e).rank})));n.forEach(t.nodes(),(function(r){var o=t.node(r);n.has(o,"rank")&&(o.rank-=e)}))},removeEmptyRanks:function(t){var e=n.min(n.map(t.nodes(),(function(e){return t.node(e).rank}))),r=[];n.forEach(t.nodes(),(function(n){var o=t.node(n).rank-e;r[o]||(r[o]=[]),r[o].push(n)}));var o=0,i=t.graph().nodeRankFactor;n.forEach(r,(function(e,r){n.isUndefined(e)&&r%i!=0?--o:o&&n.forEach(e,(function(e){t.node(e).rank+=o}))}))},addBorderNode:function(t,e,r,n){var o={width:0,height:0};arguments.length>=4&&(o.rank=r,o.order=n);return i(t,"border",o,e)},maxRank:a,partition:function(t,e){var r={lhs:[],rhs:[]};return n.forEach(t,(function(t){e(t)?r.lhs.push(t):r.rhs.push(t)})),r},time:function(t,e){var r=n.now();try{return e()}finally{console.log(t+" time: "+(n.now()-r)+"ms")}},notime:function(t,e){return e()}}},function(t,e,r){var n;try{n={clone:r(137),constant:r(67),each:r(93),filter:r(96),has:r(107),isArray:r(1),isEmpty:r(214),isFunction:r(25),isUndefined:r(108),keys:r(14),map:r(109),reduce:r(111),size:r(217),transform:r(223),union:r(224),values:r(116)}}catch(t){}n||(n=window._),t.exports=n},function(t,e){function r(t){return(r="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(t){return typeof t}:function(t){return t&&"function"==typeof Symbol&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t})(t)}t.exports=function(t){var e=r(t);return null!=t&&("object"==e||"function"==e)}},function(t,e,r){var n;try{n=r(135)}catch(t){}n||(n=window.graphlib),t.exports=n},function(t,e,r){function n(t){return(n="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(t){return typeof t}:function(t){return t&&"function"==typeof Symbol&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t})(t)}var o=r(76),i="object"==("undefined"==typeof self?"undefined":n(self))&&self&&self.Object===Object&&self,a=o||i||Function("return this")();t.exports=a},function(t,e,r){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),e.forEach=e.sqrLen=e.sqrDist=e.dist=e.div=e.mul=e.sub=e.len=void 0,e.create=o,e.clone=function(t){var e=new n.ARRAY_TYPE(2);return e[0]=t[0],e[1]=t[1],e},e.fromValues=function(t,e){var r=new n.ARRAY_TYPE(2);return r[0]=t,r[1]=e,r},e.copy=function(t,e){return t[0]=e[0],t[1]=e[1],t},e.set=function(t,e,r){return t[0]=e,t[1]=r,t},e.add=function(t,e,r){return t[0]=e[0]+r[0],t[1]=e[1]+r[1],t},e.subtract=i,e.multiply=a,e.divide=u,e.ceil=function(t,e){return t[0]=Math.ceil(e[0]),t[1]=Math.ceil(e[1]),t},e.floor=function(t,e){return t[0]=Math.floor(e[0]),t[1]=Math.floor(e[1]),t},e.min=function(t,e,r){return t[0]=Math.min(e[0],r[0]),t[1]=Math.min(e[1],r[1]),t},e.max=function(t,e,r){return t[0]=Math.max(e[0],r[0]),t[1]=Math.max(e[1],r[1]),t},e.round=function(t,e){return t[0]=Math.round(e[0]),t[1]=Math.round(e[1]),t},e.scale=function(t,e,r){return t[0]=e[0]*r,t[1]=e[1]*r,t},e.scaleAndAdd=function(t,e,r,n){return t[0]=e[0]+r[0]*n,t[1]=e[1]+r[1]*n,t},e.distance=s,e.squaredDistance=c,e.length=f,e.squaredLength=h,e.negate=function(t,e){return t[0]=-e[0],t[1]=-e[1],t},e.inverse=function(t,e){return t[0]=1/e[0],t[1]=1/e[1],t},e.normalize=function(t,e){var r=e[0],n=e[1],o=r*r+n*n;o>0&&(o=1/Math.sqrt(o),t[0]=e[0]*o,t[1]=e[1]*o);return t},e.dot=function(t,e){return t[0]*e[0]+t[1]*e[1]},e.cross=function(t,e,r){var n=e[0]*r[1]-e[1]*r[0];return t[0]=t[1]=0,t[2]=n,t},e.lerp=function(t,e,r,n){var o=e[0],i=e[1];return t[0]=o+n*(r[0]-o),t[1]=i+n*(r[1]-i),t},e.random=function(t,e){e=e||1;var r=2*n.RANDOM()*Math.PI;return t[0]=Math.cos(r)*e,t[1]=Math.sin(r)*e,t},e.transformMat2=function(t,e,r){var n=e[0],o=e[1];return t[0]=r[0]*n+r[2]*o,t[1]=r[1]*n+r[3]*o,t},e.transformMat2d=function(t,e,r){var n=e[0],o=e[1];return t[0]=r[0]*n+r[2]*o+r[4],t[1]=r[1]*n+r[3]*o+r[5],t},e.transformMat3=function(t,e,r){var n=e[0],o=e[1];return t[0]=r[0]*n+r[3]*o+r[6],t[1]=r[1]*n+r[4]*o+r[7],t},e.transformMat4=function(t,e,r){var n=e[0],o=e[1];return t[0]=r[0]*n+r[4]*o+r[12],t[1]=r[1]*n+r[5]*o+r[13],t},e.rotate=function(t,e,r,n){var o=e[0]-r[0],i=e[1]-r[1],a=Math.sin(n),u=Math.cos(n);return t[0]=o*u-i*a+r[0],t[1]=o*a+i*u+r[1],t},e.angle=function(t,e){var r=t[0],n=t[1],o=e[0],i=e[1],a=r*r+n*n;a>0&&(a=1/Math.sqrt(a));var u=o*o+i*i;u>0&&(u=1/Math.sqrt(u));var s=(r*o+n*i)*a*u;return s>1?0:s<-1?Math.PI:Math.acos(s)},e.str=function(t){return"vec2("+t[0]+", "+t[1]+")"},e.exactEquals=function(t,e){return t[0]===e[0]&&t[1]===e[1]},e.equals=function(t,e){var r=t[0],o=t[1],i=e[0],a=e[1];return Math.abs(r-i)<=n.EPSILON*Math.max(1,Math.abs(r),Math.abs(i))&&Math.abs(o-a)<=n.EPSILON*Math.max(1,Math.abs(o),Math.abs(a))};var n=function(t){if(t&&t.__esModule)return t;var e={};if(null!=t)for(var r in t)Object.prototype.hasOwnProperty.call(t,r)&&(e[r]=t[r]);return e.default=t,e}(r(55));function o(){var t=new n.ARRAY_TYPE(2);return n.ARRAY_TYPE!=Float32Array&&(t[0]=0,t[1]=0),t}function i(t,e,r){return t[0]=e[0]-r[0],t[1]=e[1]-r[1],t}function a(t,e,r){return t[0]=e[0]*r[0],t[1]=e[1]*r[1],t}function u(t,e,r){return t[0]=e[0]/r[0],t[1]=e[1]/r[1],t}function s(t,e){var r=e[0]-t[0],n=e[1]-t[1];return Math.sqrt(r*r+n*n)}function c(t,e){var r=e[0]-t[0],n=e[1]-t[1];return r*r+n*n}function f(t){var e=t[0],r=t[1];return Math.sqrt(e*e+r*r)}function h(t){var e=t[0],r=t[1];return e*e+r*r}var l;e.len=f,e.sub=i,e.mul=a,e.div=u,e.dist=s,e.sqrDist=c,e.sqrLen=h,e.forEach=(l=o(),function(t,e,r,n,o,i){var a=void 0,u=void 0;for(e||(e=2),r||(r=0),u=n?Math.min(n*e+r,t.length):t.length,a=r;a<u;a+=e)l[0]=t[a],l[1]=t[a+1],o(l,l,i),t[a]=l[0],t[a+1]=l[1];return t})},function(t,e){function r(t){return(r="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(t){return typeof t}:function(t){return t&&"function"==typeof Symbol&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t})(t)}t.exports=function(t){return null!=t&&"object"==r(t)}},function(t,e,r){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var n=r(34);e.default=function(t){return n.default(t,"String")}},function(t,e,r){var n=r(25),o=r(61);t.exports=function(t){return null!=t&&o(t.length)&&!n(t)}},function(t,e,r){function n(t){return(n="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(t){return typeof t}:function(t){return t&&"function"==typeof Symbol&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t})(t)}var o=r(194),i=r(204),a=r(18),u=r(1),s=r(211);t.exports=function(t){return"function"==typeof t?t:null==t?a:"object"==n(t)?u(t)?i(t[0],t[1]):o(t):s(t)}},function(t,e,r){var n=r(79),o=r(63),i=r(12);t.exports=function(t){return i(t)?n(t):o(t)}},function(t,e,r){var n=r(148),o=r(154);t.exports=function(t,e){var r=o(t,e);return n(r)?r:void 0}},function(t,e,r){var n=r(21),o=r(150),i=r(151),a=n?n.toStringTag:void 0;t.exports=function(t){return null==t?void 0===t?"[object Undefined]":"[object Null]":a&&a in Object(t)?o(t):i(t)}},function(t,e,r){var n=r(79),o=r(174),i=r(12);t.exports=function(t){return i(t)?n(t,!0):o(t)}},function(t,e){t.exports=function(t){return t}},function(t,e,r){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var n=r(34);e.default=function(t){return n.default(t,"Number")}},function(t,e){t.exports=function(t,e){return t===e||t!=t&&e!=e}},function(t,e,r){var n=r(8).Symbol;t.exports=n},function(t,e,r){(function(t){function n(t){return(n="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(t){return typeof t}:function(t){return t&&"function"==typeof Symbol&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t})(t)}var o=r(8),i=r(170),a="object"==n(e)&&e&&!e.nodeType&&e,u=a&&"object"==n(t)&&t&&!t.nodeType&&t,s=u&&u.exports===a?o.Buffer:void 0,c=(s?s.isBuffer:void 0)||i;t.exports=c}).call(this,r(60)(t))},function(t,e,r){var n=r(179),o=r(57),i=r(180),a=r(88),u=r(181),s=r(16),c=r(77),f=c(n),h=c(o),l=c(i),d=c(a),p=c(u),v=s;(n&&"[object DataView]"!=v(new n(new ArrayBuffer(1)))||o&&"[object Map]"!=v(new o)||i&&"[object Promise]"!=v(i.resolve())||a&&"[object Set]"!=v(new a)||u&&"[object WeakMap]"!=v(new u))&&(v=function(t){var e=s(t),r="[object Object]"==e?t.constructor:void 0,n=r?c(r):"";if(n)switch(n){case f:return"[object DataView]";case h:return"[object Map]";case l:return"[object Promise]";case d:return"[object Set]";case p:return"[object WeakMap]"}return e}),t.exports=v},function(t,e,r){function n(t){return(n="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(t){return typeof t}:function(t){return t&&"function"==typeof Symbol&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t})(t)}var o=r(16),i=r(10);t.exports=function(t){return"symbol"==n(t)||i(t)&&"[object Symbol]"==o(t)}},function(t,e,r){var n=r(16),o=r(6);t.exports=function(t){if(!o(t))return!1;var e=n(t);return"[object Function]"==e||"[object GeneratorFunction]"==e||"[object AsyncFunction]"==e||"[object Proxy]"==e}},function(t,e,r){var n=r(40),o=r(41);t.exports=function(t,e,r,i){var a=!r;r||(r={});for(var u=-1,s=e.length;++u<s;){var c=e[u],f=i?i(r[c],t[c],c,r,t):void 0;void 0===f&&(f=t[c]),a?o(r,c,f):n(r,c,f)}return r}},function(t,e,r){var n=r(169),o=r(10),i=Object.prototype,a=i.hasOwnProperty,u=i.propertyIsEnumerable,s=n(function(){return arguments}())?n:function(t){return o(t)&&a.call(t,"callee")&&!u.call(t,"callee")};t.exports=s},function(t,e,r){var n=r(171),o=r(43),i=r(62),a=i&&i.isTypedArray,u=a?o(a):n;t.exports=u},function(t,e,r){var n=r(24);t.exports=function(t){if("string"==typeof t||n(t))return t;var e=t+"";return"0"==e&&1/t==-1/0?"-0":e}},function(t,e,r){"use strict";function n(t,e){for(var r in e)e.hasOwnProperty(r)&&"constructor"!==r&&void 0!==e[r]&&(t[r]=e[r])}Object.defineProperty(e,"__esModule",{value:!0}),e.default=function(t,e,r,o){return e&&n(t,e),r&&n(t,r),o&&n(t,o),t}},function(t,e,r){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var n=r(34);e.default=function(t){return n.default(t,"Function")}},function(t,e,r){"use strict";function n(t){return(n="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(t){return typeof t}:function(t){return t&&"function"==typeof Symbol&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t})(t)}var o=r(310),i=r(314),a=[].slice,u=["keyword","gray","hex"],s={};Object.keys(i).forEach((function(t){s[a.call(i[t].labels).sort().join("")]=t}));var c={};function f(t,e){if(!(this instanceof f))return new f(t,e);if(e&&e in u&&(e=null),e&&!(e in i))throw new Error("Unknown model: "+e);var r,n;if(null==t)this.model="rgb",this.color=[0,0,0],this.valpha=1;else if(t instanceof f)this.model=t.model,this.color=t.color.slice(),this.valpha=t.valpha;else if("string"==typeof t){var h=o.get(t);if(null===h)throw new Error("Unable to parse color from string: "+t);this.model=h.model,n=i[this.model].channels,this.color=h.value.slice(0,n),this.valpha="number"==typeof h.value[n]?h.value[n]:1}else if(t.length){this.model=e||"rgb",n=i[this.model].channels;var l=a.call(t,0,n);this.color=p(l,n),this.valpha="number"==typeof t[n]?t[n]:1}else if("number"==typeof t)t&=16777215,this.model="rgb",this.color=[t>>16&255,t>>8&255,255&t],this.valpha=1;else{this.valpha=1;var d=Object.keys(t);"alpha"in t&&(d.splice(d.indexOf("alpha"),1),this.valpha="number"==typeof t.alpha?t.alpha:0);var v=d.sort().join("");if(!(v in s))throw new Error("Unable to parse color from object: "+JSON.stringify(t));this.model=s[v];var g=i[this.model].labels,y=[];for(r=0;r<g.length;r++)y.push(t[g[r]]);this.color=p(y)}if(c[this.model])for(n=i[this.model].channels,r=0;r<n;r++){var m=c[this.model][r];m&&(this.color[r]=m(this.color[r]))}this.valpha=Math.max(0,Math.min(1,this.valpha)),Object.freeze&&Object.freeze(this)}function h(t,e,r){return(t=Array.isArray(t)?t:[t]).forEach((function(t){(c[t]||(c[t]=[]))[e]=r})),t=t[0],function(n){var o;return arguments.length?(r&&(n=r(n)),(o=this[t]()).color[e]=n,o):(o=this[t]().color[e],r&&(o=r(o)),o)}}function l(t){return function(e){return Math.max(0,Math.min(t,e))}}function d(t){return Array.isArray(t)?t:[t]}function p(t,e){for(var r=0;r<e;r++)"number"!=typeof t[r]&&(t[r]=0);return t}f.prototype={toString:function(){return this.string()},toJSON:function(){return this[this.model]()},string:function(t){var e=this.model in o.to?this:this.rgb(),r=1===(e=e.round("number"==typeof t?t:1)).valpha?e.color:e.color.concat(this.valpha);return o.to[e.model](r)},percentString:function(t){var e=this.rgb().round("number"==typeof t?t:1),r=1===e.valpha?e.color:e.color.concat(this.valpha);return o.to.rgb.percent(r)},array:function(){return 1===this.valpha?this.color.slice():this.color.concat(this.valpha)},object:function(){for(var t={},e=i[this.model].channels,r=i[this.model].labels,n=0;n<e;n++)t[r[n]]=this.color[n];return 1!==this.valpha&&(t.alpha=this.valpha),t},unitArray:function(){var t=this.rgb().color;return t[0]/=255,t[1]/=255,t[2]/=255,1!==this.valpha&&t.push(this.valpha),t},unitObject:function(){var t=this.rgb().object();return t.r/=255,t.g/=255,t.b/=255,1!==this.valpha&&(t.alpha=this.valpha),t},round:function(t){return t=Math.max(t||0,0),new f(this.color.map(function(t){return function(e){return function(t,e){return Number(t.toFixed(e))}(e,t)}}(t)).concat(this.valpha),this.model)},alpha:function(t){return arguments.length?new f(this.color.concat(Math.max(0,Math.min(1,t))),this.model):this.valpha},red:h("rgb",0,l(255)),green:h("rgb",1,l(255)),blue:h("rgb",2,l(255)),hue:h(["hsl","hsv","hsl","hwb","hcg"],0,(function(t){return(t%360+360)%360})),saturationl:h("hsl",1,l(100)),lightness:h("hsl",2,l(100)),saturationv:h("hsv",1,l(100)),value:h("hsv",2,l(100)),chroma:h("hcg",1,l(100)),gray:h("hcg",2,l(100)),white:h("hwb",1,l(100)),wblack:h("hwb",2,l(100)),cyan:h("cmyk",0,l(100)),magenta:h("cmyk",1,l(100)),yellow:h("cmyk",2,l(100)),black:h("cmyk",3,l(100)),x:h("xyz",0,l(100)),y:h("xyz",1,l(100)),z:h("xyz",2,l(100)),l:h("lab",0,l(100)),a:h("lab",1),b:h("lab",2),keyword:function(t){return arguments.length?new f(t):i[this.model].keyword(this.color)},hex:function(t){return arguments.length?new f(t):o.to.hex(this.rgb().round().color)},rgbNumber:function(){var t=this.rgb().color;return(255&t[0])<<16|(255&t[1])<<8|255&t[2]},luminosity:function(){for(var t=this.rgb().color,e=[],r=0;r<t.length;r++){var n=t[r]/255;e[r]=n<=.03928?n/12.92:Math.pow((n+.055)/1.055,2.4)}return.2126*e[0]+.7152*e[1]+.0722*e[2]},contrast:function(t){var e=this.luminosity(),r=t.luminosity();return e>r?(e+.05)/(r+.05):(r+.05)/(e+.05)},level:function(t){var e=this.contrast(t);return e>=7.1?"AAA":e>=4.5?"AA":""},isDark:function(){var t=this.rgb().color;return(299*t[0]+587*t[1]+114*t[2])/1e3<128},isLight:function(){return!this.isDark()},negate:function(){for(var t=this.rgb(),e=0;e<3;e++)t.color[e]=255-t.color[e];return t},lighten:function(t){var e=this.hsl();return e.color[2]+=e.color[2]*t,e},darken:function(t){var e=this.hsl();return e.color[2]-=e.color[2]*t,e},saturate:function(t){var e=this.hsl();return e.color[1]+=e.color[1]*t,e},desaturate:function(t){var e=this.hsl();return e.color[1]-=e.color[1]*t,e},whiten:function(t){var e=this.hwb();return e.color[1]+=e.color[1]*t,e},blacken:function(t){var e=this.hwb();return e.color[2]+=e.color[2]*t,e},grayscale:function(){var t=this.rgb().color,e=.3*t[0]+.59*t[1]+.11*t[2];return f.rgb(e,e,e)},fade:function(t){return this.alpha(this.valpha-this.valpha*t)},opaquer:function(t){return this.alpha(this.valpha+this.valpha*t)},rotate:function(t){var e=this.hsl(),r=e.color[0];return r=(r=(r+t)%360)<0?360+r:r,e.color[0]=r,e},mix:function(t,e){if(!t||!t.rgb)throw new Error(\'Argument to "mix" was not a Color instance, but rather an instance of \'+n(t));var r=t.rgb(),o=this.rgb(),i=void 0===e?.5:e,a=2*i-1,u=r.alpha()-o.alpha(),s=((a*u==-1?a:(a+u)/(1+a*u))+1)/2,c=1-s;return f.rgb(s*r.red()+c*o.red(),s*r.green()+c*o.green(),s*r.blue()+c*o.blue(),r.alpha()*i+o.alpha()*(1-i))}},Object.keys(i).forEach((function(t){if(-1===u.indexOf(t)){var e=i[t].channels;f.prototype[t]=function(){if(this.model===t)return new f(this);if(arguments.length)return new f(arguments,t);var r="number"==typeof arguments[e]?e:this.valpha;return new f(d(i[this.model][t].raw(this.color)).concat(r),t)},f[t]=function(r){return"number"==typeof r&&(r=p(a.call(arguments),e)),new f(r,t)}}})),t.exports=f},function(t,e,r){var n;function o(t){return(o="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(t){return typeof t}:function(t){return t&&"function"==typeof Symbol&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t})(t)}!function(i){var a=/^\\s+/,u=/\\s+$/,s=0,c=i.round,f=i.min,h=i.max,l=i.random;function d(t,e){if(e=e||{},(t=t||"")instanceof d)return t;if(!(this instanceof d))return new d(t,e);var r=function(t){var e={r:0,g:0,b:0},r=1,n=null,s=null,c=null,l=!1,d=!1;"string"==typeof t&&(t=function(t){t=t.replace(a,"").replace(u,"").toLowerCase();var e,r=!1;if(I[t])t=I[t],r=!0;else if("transparent"==t)return{r:0,g:0,b:0,a:0,format:"name"};if(e=Y.rgb.exec(t))return{r:e[1],g:e[2],b:e[3]};if(e=Y.rgba.exec(t))return{r:e[1],g:e[2],b:e[3],a:e[4]};if(e=Y.hsl.exec(t))return{h:e[1],s:e[2],l:e[3]};if(e=Y.hsla.exec(t))return{h:e[1],s:e[2],l:e[3],a:e[4]};if(e=Y.hsv.exec(t))return{h:e[1],s:e[2],v:e[3]};if(e=Y.hsva.exec(t))return{h:e[1],s:e[2],v:e[3],a:e[4]};if(e=Y.hex8.exec(t))return{r:F(e[1]),g:F(e[2]),b:F(e[3]),a:L(e[4]),format:r?"name":"hex8"};if(e=Y.hex6.exec(t))return{r:F(e[1]),g:F(e[2]),b:F(e[3]),format:r?"name":"hex"};if(e=Y.hex4.exec(t))return{r:F(e[1]+""+e[1]),g:F(e[2]+""+e[2]),b:F(e[3]+""+e[3]),a:L(e[4]+""+e[4]),format:r?"name":"hex8"};if(e=Y.hex3.exec(t))return{r:F(e[1]+""+e[1]),g:F(e[2]+""+e[2]),b:F(e[3]+""+e[3]),format:r?"name":"hex"};return!1}(t));"object"==o(t)&&(H(t.r)&&H(t.g)&&H(t.b)?(p=t.r,v=t.g,g=t.b,e={r:255*C(p,255),g:255*C(v,255),b:255*C(g,255)},l=!0,d="%"===String(t.r).substr(-1)?"prgb":"rgb"):H(t.h)&&H(t.s)&&H(t.v)?(n=D(t.s),s=D(t.v),e=function(t,e,r){t=6*C(t,360),e=C(e,100),r=C(r,100);var n=i.floor(t),o=t-n,a=r*(1-e),u=r*(1-o*e),s=r*(1-(1-o)*e),c=n%6;return{r:255*[r,u,a,a,s,r][c],g:255*[s,r,r,u,a,a][c],b:255*[a,a,s,r,r,u][c]}}(t.h,n,s),l=!0,d="hsv"):H(t.h)&&H(t.s)&&H(t.l)&&(n=D(t.s),c=D(t.l),e=function(t,e,r){var n,o,i;function a(t,e,r){return r<0&&(r+=1),r>1&&(r-=1),r<1/6?t+6*(e-t)*r:r<.5?e:r<2/3?t+(e-t)*(2/3-r)*6:t}if(t=C(t,360),e=C(e,100),r=C(r,100),0===e)n=o=i=r;else{var u=r<.5?r*(1+e):r+e-r*e,s=2*r-u;n=a(s,u,t+1/3),o=a(s,u,t),i=a(s,u,t-1/3)}return{r:255*n,g:255*o,b:255*i}}(t.h,n,c),l=!0,d="hsl"),t.hasOwnProperty("a")&&(r=t.a));var p,v,g;return r=P(r),{ok:l,format:t.format||d,r:f(255,h(e.r,0)),g:f(255,h(e.g,0)),b:f(255,h(e.b,0)),a:r}}(t);this._originalInput=t,this._r=r.r,this._g=r.g,this._b=r.b,this._a=r.a,this._roundA=c(100*this._a)/100,this._format=e.format||r.format,this._gradientType=e.gradientType,this._r<1&&(this._r=c(this._r)),this._g<1&&(this._g=c(this._g)),this._b<1&&(this._b=c(this._b)),this._ok=r.ok,this._tc_id=s++}function p(t,e,r){t=C(t,255),e=C(e,255),r=C(r,255);var n,o,i=h(t,e,r),a=f(t,e,r),u=(i+a)/2;if(i==a)n=o=0;else{var s=i-a;switch(o=u>.5?s/(2-i-a):s/(i+a),i){case t:n=(e-r)/s+(e<r?6:0);break;case e:n=(r-t)/s+2;break;case r:n=(t-e)/s+4}n/=6}return{h:n,s:o,l:u}}function v(t,e,r){t=C(t,255),e=C(e,255),r=C(r,255);var n,o,i=h(t,e,r),a=f(t,e,r),u=i,s=i-a;if(o=0===i?0:s/i,i==a)n=0;else{switch(i){case t:n=(e-r)/s+(e<r?6:0);break;case e:n=(r-t)/s+2;break;case r:n=(t-e)/s+4}n/=6}return{h:n,s:o,v:u}}function g(t,e,r,n){var o=[T(c(t).toString(16)),T(c(e).toString(16)),T(c(r).toString(16))];return n&&o[0].charAt(0)==o[0].charAt(1)&&o[1].charAt(0)==o[1].charAt(1)&&o[2].charAt(0)==o[2].charAt(1)?o[0].charAt(0)+o[1].charAt(0)+o[2].charAt(0):o.join("")}function y(t,e,r,n){return[T(q(n)),T(c(t).toString(16)),T(c(e).toString(16)),T(c(r).toString(16))].join("")}function m(t,e){e=0===e?0:e||10;var r=d(t).toHsl();return r.s-=e/100,r.s=z(r.s),d(r)}function b(t,e){e=0===e?0:e||10;var r=d(t).toHsl();return r.s+=e/100,r.s=z(r.s),d(r)}function w(t){return d(t).desaturate(100)}function x(t,e){e=0===e?0:e||10;var r=d(t).toHsl();return r.l+=e/100,r.l=z(r.l),d(r)}function M(t,e){e=0===e?0:e||10;var r=d(t).toRgb();return r.r=h(0,f(255,r.r-c(-e/100*255))),r.g=h(0,f(255,r.g-c(-e/100*255))),r.b=h(0,f(255,r.b-c(-e/100*255))),d(r)}function k(t,e){e=0===e?0:e||10;var r=d(t).toHsl();return r.l-=e/100,r.l=z(r.l),d(r)}function _(t,e){var r=d(t).toHsl(),n=(r.h+e)%360;return r.h=n<0?360+n:n,d(r)}function S(t){var e=d(t).toHsl();return e.h=(e.h+180)%360,d(e)}function E(t){var e=d(t).toHsl(),r=e.h;return[d(t),d({h:(r+120)%360,s:e.s,l:e.l}),d({h:(r+240)%360,s:e.s,l:e.l})]}function j(t){var e=d(t).toHsl(),r=e.h;return[d(t),d({h:(r+90)%360,s:e.s,l:e.l}),d({h:(r+180)%360,s:e.s,l:e.l}),d({h:(r+270)%360,s:e.s,l:e.l})]}function A(t){var e=d(t).toHsl(),r=e.h;return[d(t),d({h:(r+72)%360,s:e.s,l:e.l}),d({h:(r+216)%360,s:e.s,l:e.l})]}function O(t,e,r){e=e||6,r=r||30;var n=d(t).toHsl(),o=360/r,i=[d(t)];for(n.h=(n.h-(o*e>>1)+720)%360;--e;)n.h=(n.h+o)%360,i.push(d(n));return i}function N(t,e){e=e||6;for(var r=d(t).toHsv(),n=r.h,o=r.s,i=r.v,a=[],u=1/e;e--;)a.push(d({h:n,s:o,v:i})),i=(i+u)%1;return a}d.prototype={isDark:function(){return this.getBrightness()<128},isLight:function(){return!this.isDark()},isValid:function(){return this._ok},getOriginalInput:function(){return this._originalInput},getFormat:function(){return this._format},getAlpha:function(){return this._a},getBrightness:function(){var t=this.toRgb();return(299*t.r+587*t.g+114*t.b)/1e3},getLuminance:function(){var t,e,r,n=this.toRgb();return t=n.r/255,e=n.g/255,r=n.b/255,.2126*(t<=.03928?t/12.92:i.pow((t+.055)/1.055,2.4))+.7152*(e<=.03928?e/12.92:i.pow((e+.055)/1.055,2.4))+.0722*(r<=.03928?r/12.92:i.pow((r+.055)/1.055,2.4))},setAlpha:function(t){return this._a=P(t),this._roundA=c(100*this._a)/100,this},toHsv:function(){var t=v(this._r,this._g,this._b);return{h:360*t.h,s:t.s,v:t.v,a:this._a}},toHsvString:function(){var t=v(this._r,this._g,this._b),e=c(360*t.h),r=c(100*t.s),n=c(100*t.v);return 1==this._a?"hsv("+e+", "+r+"%, "+n+"%)":"hsva("+e+", "+r+"%, "+n+"%, "+this._roundA+")"},toHsl:function(){var t=p(this._r,this._g,this._b);return{h:360*t.h,s:t.s,l:t.l,a:this._a}},toHslString:function(){var t=p(this._r,this._g,this._b),e=c(360*t.h),r=c(100*t.s),n=c(100*t.l);return 1==this._a?"hsl("+e+", "+r+"%, "+n+"%)":"hsla("+e+", "+r+"%, "+n+"%, "+this._roundA+")"},toHex:function(t){return g(this._r,this._g,this._b,t)},toHexString:function(t){return"#"+this.toHex(t)},toHex8:function(t){return function(t,e,r,n,o){var i=[T(c(t).toString(16)),T(c(e).toString(16)),T(c(r).toString(16)),T(q(n))];if(o&&i[0].charAt(0)==i[0].charAt(1)&&i[1].charAt(0)==i[1].charAt(1)&&i[2].charAt(0)==i[2].charAt(1)&&i[3].charAt(0)==i[3].charAt(1))return i[0].charAt(0)+i[1].charAt(0)+i[2].charAt(0)+i[3].charAt(0);return i.join("")}(this._r,this._g,this._b,this._a,t)},toHex8String:function(t){return"#"+this.toHex8(t)},toRgb:function(){return{r:c(this._r),g:c(this._g),b:c(this._b),a:this._a}},toRgbString:function(){return 1==this._a?"rgb("+c(this._r)+", "+c(this._g)+", "+c(this._b)+")":"rgba("+c(this._r)+", "+c(this._g)+", "+c(this._b)+", "+this._roundA+")"},toPercentageRgb:function(){return{r:c(100*C(this._r,255))+"%",g:c(100*C(this._g,255))+"%",b:c(100*C(this._b,255))+"%",a:this._a}},toPercentageRgbString:function(){return 1==this._a?"rgb("+c(100*C(this._r,255))+"%, "+c(100*C(this._g,255))+"%, "+c(100*C(this._b,255))+"%)":"rgba("+c(100*C(this._r,255))+"%, "+c(100*C(this._g,255))+"%, "+c(100*C(this._b,255))+"%, "+this._roundA+")"},toName:function(){return 0===this._a?"transparent":!(this._a<1)&&(R[g(this._r,this._g,this._b,!0)]||!1)},toFilter:function(t){var e="#"+y(this._r,this._g,this._b,this._a),r=e,n=this._gradientType?"GradientType = 1, ":"";if(t){var o=d(t);r="#"+y(o._r,o._g,o._b,o._a)}return"progid:DXImageTransform.Microsoft.gradient("+n+"startColorstr="+e+",endColorstr="+r+")"},toString:function(t){var e=!!t;t=t||this._format;var r=!1,n=this._a<1&&this._a>=0;return e||!n||"hex"!==t&&"hex6"!==t&&"hex3"!==t&&"hex4"!==t&&"hex8"!==t&&"name"!==t?("rgb"===t&&(r=this.toRgbString()),"prgb"===t&&(r=this.toPercentageRgbString()),"hex"!==t&&"hex6"!==t||(r=this.toHexString()),"hex3"===t&&(r=this.toHexString(!0)),"hex4"===t&&(r=this.toHex8String(!0)),"hex8"===t&&(r=this.toHex8String()),"name"===t&&(r=this.toName()),"hsl"===t&&(r=this.toHslString()),"hsv"===t&&(r=this.toHsvString()),r||this.toHexString()):"name"===t&&0===this._a?this.toName():this.toRgbString()},clone:function(){return d(this.toString())},_applyModification:function(t,e){var r=t.apply(null,[this].concat([].slice.call(e)));return this._r=r._r,this._g=r._g,this._b=r._b,this.setAlpha(r._a),this},lighten:function(){return this._applyModification(x,arguments)},brighten:function(){return this._applyModification(M,arguments)},darken:function(){return this._applyModification(k,arguments)},desaturate:function(){return this._applyModification(m,arguments)},saturate:function(){return this._applyModification(b,arguments)},greyscale:function(){return this._applyModification(w,arguments)},spin:function(){return this._applyModification(_,arguments)},_applyCombination:function(t,e){return t.apply(null,[this].concat([].slice.call(e)))},analogous:function(){return this._applyCombination(O,arguments)},complement:function(){return this._applyCombination(S,arguments)},monochromatic:function(){return this._applyCombination(N,arguments)},splitcomplement:function(){return this._applyCombination(A,arguments)},triad:function(){return this._applyCombination(E,arguments)},tetrad:function(){return this._applyCombination(j,arguments)}},d.fromRatio=function(t,e){if("object"==o(t)){var r={};for(var n in t)t.hasOwnProperty(n)&&(r[n]="a"===n?t[n]:D(t[n]));t=r}return d(t,e)},d.equals=function(t,e){return!(!t||!e)&&d(t).toRgbString()==d(e).toRgbString()},d.random=function(){return d.fromRatio({r:l(),g:l(),b:l()})},d.mix=function(t,e,r){r=0===r?0:r||50;var n=d(t).toRgb(),o=d(e).toRgb(),i=r/100;return d({r:(o.r-n.r)*i+n.r,g:(o.g-n.g)*i+n.g,b:(o.b-n.b)*i+n.b,a:(o.a-n.a)*i+n.a})},d.readability=function(t,e){var r=d(t),n=d(e);return(i.max(r.getLuminance(),n.getLuminance())+.05)/(i.min(r.getLuminance(),n.getLuminance())+.05)},d.isReadable=function(t,e,r){var n,o,i=d.readability(t,e);switch(o=!1,(n=function(t){var e,r;e=((t=t||{level:"AA",size:"small"}).level||"AA").toUpperCase(),r=(t.size||"small").toLowerCase(),"AA"!==e&&"AAA"!==e&&(e="AA");"small"!==r&&"large"!==r&&(r="small");return{level:e,size:r}}(r)).level+n.size){case"AAsmall":case"AAAlarge":o=i>=4.5;break;case"AAlarge":o=i>=3;break;case"AAAsmall":o=i>=7}return o},d.mostReadable=function(t,e,r){var n,o,i,a,u=null,s=0;o=(r=r||{}).includeFallbackColors,i=r.level,a=r.size;for(var c=0;c<e.length;c++)(n=d.readability(t,e[c]))>s&&(s=n,u=d(e[c]));return d.isReadable(t,u,{level:i,size:a})||!o?u:(r.includeFallbackColors=!1,d.mostReadable(t,["#fff","#000"],r))};var I=d.names={aliceblue:"f0f8ff",antiquewhite:"faebd7",aqua:"0ff",aquamarine:"7fffd4",azure:"f0ffff",beige:"f5f5dc",bisque:"ffe4c4",black:"000",blanchedalmond:"ffebcd",blue:"00f",blueviolet:"8a2be2",brown:"a52a2a",burlywood:"deb887",burntsienna:"ea7e5d",cadetblue:"5f9ea0",chartreuse:"7fff00",chocolate:"d2691e",coral:"ff7f50",cornflowerblue:"6495ed",cornsilk:"fff8dc",crimson:"dc143c",cyan:"0ff",darkblue:"00008b",darkcyan:"008b8b",darkgoldenrod:"b8860b",darkgray:"a9a9a9",darkgreen:"006400",darkgrey:"a9a9a9",darkkhaki:"bdb76b",darkmagenta:"8b008b",darkolivegreen:"556b2f",darkorange:"ff8c00",darkorchid:"9932cc",darkred:"8b0000",darksalmon:"e9967a",darkseagreen:"8fbc8f",darkslateblue:"483d8b",darkslategray:"2f4f4f",darkslategrey:"2f4f4f",darkturquoise:"00ced1",darkviolet:"9400d3",deeppink:"ff1493",deepskyblue:"00bfff",dimgray:"696969",dimgrey:"696969",dodgerblue:"1e90ff",firebrick:"b22222",floralwhite:"fffaf0",forestgreen:"228b22",fuchsia:"f0f",gainsboro:"dcdcdc",ghostwhite:"f8f8ff",gold:"ffd700",goldenrod:"daa520",gray:"808080",green:"008000",greenyellow:"adff2f",grey:"808080",honeydew:"f0fff0",hotpink:"ff69b4",indianred:"cd5c5c",indigo:"4b0082",ivory:"fffff0",khaki:"f0e68c",lavender:"e6e6fa",lavenderblush:"fff0f5",lawngreen:"7cfc00",lemonchiffon:"fffacd",lightblue:"add8e6",lightcoral:"f08080",lightcyan:"e0ffff",lightgoldenrodyellow:"fafad2",lightgray:"d3d3d3",lightgreen:"90ee90",lightgrey:"d3d3d3",lightpink:"ffb6c1",lightsalmon:"ffa07a",lightseagreen:"20b2aa",lightskyblue:"87cefa",lightslategray:"789",lightslategrey:"789",lightsteelblue:"b0c4de",lightyellow:"ffffe0",lime:"0f0",limegreen:"32cd32",linen:"faf0e6",magenta:"f0f",maroon:"800000",mediumaquamarine:"66cdaa",mediumblue:"0000cd",mediumorchid:"ba55d3",mediumpurple:"9370db",mediumseagreen:"3cb371",mediumslateblue:"7b68ee",mediumspringgreen:"00fa9a",mediumturquoise:"48d1cc",mediumvioletred:"c71585",midnightblue:"191970",mintcream:"f5fffa",mistyrose:"ffe4e1",moccasin:"ffe4b5",navajowhite:"ffdead",navy:"000080",oldlace:"fdf5e6",olive:"808000",olivedrab:"6b8e23",orange:"ffa500",orangered:"ff4500",orchid:"da70d6",palegoldenrod:"eee8aa",palegreen:"98fb98",paleturquoise:"afeeee",palevioletred:"db7093",papayawhip:"ffefd5",peachpuff:"ffdab9",peru:"cd853f",pink:"ffc0cb",plum:"dda0dd",powderblue:"b0e0e6",purple:"800080",rebeccapurple:"663399",red:"f00",rosybrown:"bc8f8f",royalblue:"4169e1",saddlebrown:"8b4513",salmon:"fa8072",sandybrown:"f4a460",seagreen:"2e8b57",seashell:"fff5ee",sienna:"a0522d",silver:"c0c0c0",skyblue:"87ceeb",slateblue:"6a5acd",slategray:"708090",slategrey:"708090",snow:"fffafa",springgreen:"00ff7f",steelblue:"4682b4",tan:"d2b48c",teal:"008080",thistle:"d8bfd8",tomato:"ff6347",turquoise:"40e0d0",violet:"ee82ee",wheat:"f5deb3",white:"fff",whitesmoke:"f5f5f5",yellow:"ff0",yellowgreen:"9acd32"},R=d.hexNames=function(t){var e={};for(var r in t)t.hasOwnProperty(r)&&(e[t[r]]=r);return e}(I);function P(t){return t=parseFloat(t),(isNaN(t)||t<0||t>1)&&(t=1),t}function C(t,e){(function(t){return"string"==typeof t&&-1!=t.indexOf(".")&&1===parseFloat(t)})(t)&&(t="100%");var r=function(t){return"string"==typeof t&&-1!=t.indexOf("%")}(t);return t=f(e,h(0,parseFloat(t))),r&&(t=parseInt(t*e,10)/100),i.abs(t-e)<1e-6?1:t%e/parseFloat(e)}function z(t){return f(1,h(0,t))}function F(t){return parseInt(t,16)}function T(t){return 1==t.length?"0"+t:""+t}function D(t){return t<=1&&(t=100*t+"%"),t}function q(t){return i.round(255*parseFloat(t)).toString(16)}function L(t){return F(t)/255}var B,V,U,Y=(V="[\\\\s|\\\\(]+("+(B="(?:[-\\\\+]?\\\\d*\\\\.\\\\d+%?)|(?:[-\\\\+]?\\\\d+%?)")+")[,|\\\\s]+("+B+")[,|\\\\s]+("+B+")\\\\s*\\\\)?",U="[\\\\s|\\\\(]+("+B+")[,|\\\\s]+("+B+")[,|\\\\s]+("+B+")[,|\\\\s]+("+B+")\\\\s*\\\\)?",{CSS_UNIT:new RegExp(B),rgb:new RegExp("rgb"+V),rgba:new RegExp("rgba"+U),hsl:new RegExp("hsl"+V),hsla:new RegExp("hsla"+U),hsv:new RegExp("hsv"+V),hsva:new RegExp("hsva"+U),hex3:/^#?([0-9a-fA-F]{1})([0-9a-fA-F]{1})([0-9a-fA-F]{1})$/,hex6:/^#?([0-9a-fA-F]{2})([0-9a-fA-F]{2})([0-9a-fA-F]{2})$/,hex4:/^#?([0-9a-fA-F]{1})([0-9a-fA-F]{1})([0-9a-fA-F]{1})([0-9a-fA-F]{1})$/,hex8:/^#?([0-9a-fA-F]{2})([0-9a-fA-F]{2})([0-9a-fA-F]{2})([0-9a-fA-F]{2})$/});function H(t){return!!Y.CSS_UNIT.exec(t)}t.exports?t.exports=d:void 0===(n=function(){return d}.call(e,r,e,t))||(t.exports=n)}(Math)},function(t,e,r){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var n={}.toString;e.default=function(t,e){return n.call(t)==="[object "+e+"]"}},function(t,e,r){var n=r(36),o=r(143),i=r(144),a=r(145),u=r(146),s=r(147);function c(t){var e=this.__data__=new n(t);this.size=e.size}c.prototype.clear=o,c.prototype.delete=i,c.prototype.get=a,c.prototype.has=u,c.prototype.set=s,t.exports=c},function(t,e,r){var n=r(138),o=r(139),i=r(140),a=r(141),u=r(142);function s(t){var e=-1,r=null==t?0:t.length;for(this.clear();++e<r;){var n=t[e];this.set(n[0],n[1])}}s.prototype.clear=n,s.prototype.delete=o,s.prototype.get=i,s.prototype.has=a,s.prototype.set=u,t.exports=s},function(t,e,r){var n=r(20);t.exports=function(t,e){for(var r=t.length;r--;)if(n(t[r][0],e))return r;return-1}},function(t,e,r){var n=r(15)(Object,"create");t.exports=n},function(t,e,r){var n=r(163);t.exports=function(t,e){var r=t.__data__;return n(e)?r["string"==typeof e?"string":"hash"]:r.map}},function(t,e,r){var n=r(41),o=r(20),i=Object.prototype.hasOwnProperty;t.exports=function(t,e,r){var a=t[e];i.call(t,e)&&o(a,r)&&(void 0!==r||e in t)||n(t,e,r)}},function(t,e,r){var n=r(78);t.exports=function(t,e,r){"__proto__"==e&&n?n(t,e,{configurable:!0,enumerable:!0,value:r,writable:!0}):t[e]=r}},function(t,e){function r(t){return(r="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(t){return typeof t}:function(t){return t&&"function"==typeof Symbol&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t})(t)}var n=/^(?:0|[1-9]\\d*)$/;t.exports=function(t,e){var o=r(t);return!!(e=null==e?9007199254740991:e)&&("number"==o||"symbol"!=o&&n.test(t))&&t>-1&&t%1==0&&t<e}},function(t,e){t.exports=function(t){return function(e){return t(e)}}},function(t,e){var r=Object.prototype;t.exports=function(t){var e=t&&t.constructor;return t===("function"==typeof e&&e.prototype||r)}},function(t,e,r){var n=r(80)(Object.getPrototypeOf,Object);t.exports=n},function(t,e,r){var n=r(68),o=r(192)(n);t.exports=o},function(t,e,r){var n=r(48),o=r(29);t.exports=function(t,e){for(var r=0,i=(e=n(e,t)).length;null!=t&&r<i;)t=t[o(e[r++])];return r&&r==i?t:void 0}},function(t,e,r){var n=r(1),o=r(71),i=r(206),a=r(103);t.exports=function(t,e){return n(t)?t:o(t,e)?[t]:i(a(t))}},function(t,e){t.exports=function(t,e){for(var r=-1,n=null==t?0:t.length,o=Array(n);++r<n;)o[r]=e(t[r],r,t);return o}},function(t,e,r){var n=r(18),o=r(112),i=r(113);t.exports=function(t,e){return i(o(t,e,n),t+"")}},function(t,e,r){function n(t){return(n="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(t){return typeof t}:function(t){return t&&"function"==typeof Symbol&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t})(t)}var o=r(20),i=r(12),a=r(42),u=r(6);t.exports=function(t,e,r){if(!u(r))return!1;var s=n(e);return!!("number"==s?i(r)&&a(e,r.length):"string"==s&&e in r)&&o(r[e],t)}},function(t,e,r){"use strict";var n=r(0);t.exports={longestPath:function(t){var e={};n.forEach(t.sources(),(function r(o){var i=t.node(o);if(n.has(e,o))return i.rank;e[o]=!0;var a=n.min(n.map(t.outEdges(o),(function(e){return r(e.w)-t.edge(e).minlen})));return a!==Number.POSITIVE_INFINITY&&null!=a||(a=0),i.rank=a}))},slack:function(t,e){return t.node(e.w).rank-t.node(e.v).rank-t.edge(e).minlen}}},function(t,e,r){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var n=r(2),o=r(134);e.default=function(t,e){if(t)if(n.default(t))for(var r=0,i=t.length;r<i&&!1!==e(t[r],r);r++);else if(o.default(t))for(var a in t)if(t.hasOwnProperty(a)&&!1===e(t[a],a))break}},function(t,e,r){"use strict";Object.defineProperty(e,"__esModule",{value:!0});e.default=function(t){return null==t}},function(t,e,r){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),e.setMatrixArrayType=function(t){e.ARRAY_TYPE=t},e.toRadian=function(t){return t*o},e.equals=function(t,e){return Math.abs(t-e)<=n*Math.max(1,Math.abs(t),Math.abs(e))};var n=e.EPSILON=1e-6;e.ARRAY_TYPE="undefined"!=typeof Float32Array?Float32Array:Array,e.RANDOM=Math.random;var o=Math.PI/180},function(t,e,r){"use strict";function n(t){return(n="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(t){return typeof t}:function(t){return t&&"function"==typeof Symbol&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t})(t)}var o=r(5);t.exports=i;function i(t){this._isDirected=!o.has(t,"directed")||t.directed,this._isMultigraph=!!o.has(t,"multigraph")&&t.multigraph,this._isCompound=!!o.has(t,"compound")&&t.compound,this._label=void 0,this._defaultNodeLabelFn=o.constant(void 0),this._defaultEdgeLabelFn=o.constant(void 0),this._nodes={},this._isCompound&&(this._parent={},this._children={},this._children["\\0"]={}),this._in={},this._preds={},this._out={},this._sucs={},this._edgeObjs={},this._edgeLabels={}}function a(t,e){t[e]?t[e]++:t[e]=1}function u(t,e){--t[e]||delete t[e]}function s(t,e,r,n){var i=""+e,a=""+r;if(!t&&i>a){var u=i;i=a,a=u}return i+""+a+""+(o.isUndefined(n)?"\\0":n)}function c(t,e,r,n){var o=""+e,i=""+r;if(!t&&o>i){var a=o;o=i,i=a}var u={v:o,w:i};return n&&(u.name=n),u}function f(t,e){return s(t,e.v,e.w,e.name)}i.prototype._nodeCount=0,i.prototype._edgeCount=0,i.prototype.isDirected=function(){return this._isDirected},i.prototype.isMultigraph=function(){return this._isMultigraph},i.prototype.isCompound=function(){return this._isCompound},i.prototype.setGraph=function(t){return this._label=t,this},i.prototype.graph=function(){return this._label},i.prototype.setDefaultNodeLabel=function(t){return o.isFunction(t)||(t=o.constant(t)),this._defaultNodeLabelFn=t,this},i.prototype.nodeCount=function(){return this._nodeCount},i.prototype.nodes=function(){return o.keys(this._nodes)},i.prototype.sources=function(){var t=this;return o.filter(this.nodes(),(function(e){return o.isEmpty(t._in[e])}))},i.prototype.sinks=function(){var t=this;return o.filter(this.nodes(),(function(e){return o.isEmpty(t._out[e])}))},i.prototype.setNodes=function(t,e){var r=arguments,n=this;return o.each(t,(function(t){r.length>1?n.setNode(t,e):n.setNode(t)})),this},i.prototype.setNode=function(t,e){return o.has(this._nodes,t)?(arguments.length>1&&(this._nodes[t]=e),this):(this._nodes[t]=arguments.length>1?e:this._defaultNodeLabelFn(t),this._isCompound&&(this._parent[t]="\\0",this._children[t]={},this._children["\\0"][t]=!0),this._in[t]={},this._preds[t]={},this._out[t]={},this._sucs[t]={},++this._nodeCount,this)},i.prototype.node=function(t){return this._nodes[t]},i.prototype.hasNode=function(t){return o.has(this._nodes,t)},i.prototype.removeNode=function(t){var e=this;if(o.has(this._nodes,t)){var r=function(t){e.removeEdge(e._edgeObjs[t])};delete this._nodes[t],this._isCompound&&(this._removeFromParentsChildList(t),delete this._parent[t],o.each(this.children(t),(function(t){e.setParent(t)})),delete this._children[t]),o.each(o.keys(this._in[t]),r),delete this._in[t],delete this._preds[t],o.each(o.keys(this._out[t]),r),delete this._out[t],delete this._sucs[t],--this._nodeCount}return this},i.prototype.setParent=function(t,e){if(!this._isCompound)throw new Error("Cannot set parent in a non-compound graph");if(o.isUndefined(e))e="\\0";else{for(var r=e+="";!o.isUndefined(r);r=this.parent(r))if(r===t)throw new Error("Setting "+e+" as parent of "+t+" would create a cycle");this.setNode(e)}return this.setNode(t),this._removeFromParentsChildList(t),this._parent[t]=e,this._children[e][t]=!0,this},i.prototype._removeFromParentsChildList=function(t){delete this._children[this._parent[t]][t]},i.prototype.parent=function(t){if(this._isCompound){var e=this._parent[t];if("\\0"!==e)return e}},i.prototype.children=function(t){if(o.isUndefined(t)&&(t="\\0"),this._isCompound){var e=this._children[t];if(e)return o.keys(e)}else{if("\\0"===t)return this.nodes();if(this.hasNode(t))return[]}},i.prototype.predecessors=function(t){var e=this._preds[t];if(e)return o.keys(e)},i.prototype.successors=function(t){var e=this._sucs[t];if(e)return o.keys(e)},i.prototype.neighbors=function(t){var e=this.predecessors(t);if(e)return o.union(e,this.successors(t))},i.prototype.isLeaf=function(t){return 0===(this.isDirected()?this.successors(t):this.neighbors(t)).length},i.prototype.filterNodes=function(t){var e=new this.constructor({directed:this._isDirected,multigraph:this._isMultigraph,compound:this._isCompound});e.setGraph(this.graph());var r=this;o.each(this._nodes,(function(r,n){t(n)&&e.setNode(n,r)})),o.each(this._edgeObjs,(function(t){e.hasNode(t.v)&&e.hasNode(t.w)&&e.setEdge(t,r.edge(t))}));var n={};return this._isCompound&&o.each(e.nodes(),(function(t){e.setParent(t,function t(o){var i=r.parent(o);return void 0===i||e.hasNode(i)?(n[o]=i,i):i in n?n[i]:t(i)}(t))})),e},i.prototype.setDefaultEdgeLabel=function(t){return o.isFunction(t)||(t=o.constant(t)),this._defaultEdgeLabelFn=t,this},i.prototype.edgeCount=function(){return this._edgeCount},i.prototype.edges=function(){return o.values(this._edgeObjs)},i.prototype.setPath=function(t,e){var r=this,n=arguments;return o.reduce(t,(function(t,o){return n.length>1?r.setEdge(t,o,e):r.setEdge(t,o),o})),this},i.prototype.setEdge=function(){var t,e,r,i,u=!1,f=arguments[0];"object"===n(f)&&null!==f&&"v"in f?(t=f.v,e=f.w,r=f.name,2===arguments.length&&(i=arguments[1],u=!0)):(t=f,e=arguments[1],r=arguments[3],arguments.length>2&&(i=arguments[2],u=!0)),t=""+t,e=""+e,o.isUndefined(r)||(r=""+r);var h=s(this._isDirected,t,e,r);if(o.has(this._edgeLabels,h))return u&&(this._edgeLabels[h]=i),this;if(!o.isUndefined(r)&&!this._isMultigraph)throw new Error("Cannot set a named edge when isMultigraph = false");this.setNode(t),this.setNode(e),this._edgeLabels[h]=u?i:this._defaultEdgeLabelFn(t,e,r);var l=c(this._isDirected,t,e,r);return t=l.v,e=l.w,Object.freeze(l),this._edgeObjs[h]=l,a(this._preds[e],t),a(this._sucs[t],e),this._in[e][h]=l,this._out[t][h]=l,this._edgeCount++,this},i.prototype.edge=function(t,e,r){var n=1===arguments.length?f(this._isDirected,arguments[0]):s(this._isDirected,t,e,r);return this._edgeLabels[n]},i.prototype.hasEdge=function(t,e,r){var n=1===arguments.length?f(this._isDirected,arguments[0]):s(this._isDirected,t,e,r);return o.has(this._edgeLabels,n)},i.prototype.removeEdge=function(t,e,r){var n=1===arguments.length?f(this._isDirected,arguments[0]):s(this._isDirected,t,e,r),o=this._edgeObjs[n];return o&&(t=o.v,e=o.w,delete this._edgeLabels[n],delete this._edgeObjs[n],u(this._preds[e],t),u(this._sucs[t],e),delete this._in[e][n],delete this._out[t][n],this._edgeCount--),this},i.prototype.inEdges=function(t,e){var r=this._in[t];if(r){var n=o.values(r);return e?o.filter(n,(function(t){return t.v===e})):n}},i.prototype.outEdges=function(t,e){var r=this._out[t];if(r){var n=o.values(r);return e?o.filter(n,(function(t){return t.w===e})):n}},i.prototype.nodeEdges=function(t,e){var r=this.inEdges(t,e);if(r)return r.concat(this.outEdges(t,e))}},function(t,e,r){var n=r(15)(r(8),"Map");t.exports=n},function(t,e,r){var n=r(155),o=r(162),i=r(164),a=r(165),u=r(166);function s(t){var e=-1,r=null==t?0:t.length;for(this.clear();++e<r;){var n=t[e];this.set(n[0],n[1])}}s.prototype.clear=n,s.prototype.delete=o,s.prototype.get=i,s.prototype.has=a,s.prototype.set=u,t.exports=s},function(t,e){t.exports=function(t,e){for(var r=-1,n=null==t?0:t.length;++r<n&&!1!==e(t[r],r,t););return t}},function(t,e){t.exports=function(t){return t.webpackPolyfill||(t.deprecate=function(){},t.paths=[],t.children||(t.children=[]),Object.defineProperty(t,"loaded",{enumerable:!0,get:function(){return t.l}}),Object.defineProperty(t,"id",{enumerable:!0,get:function(){return t.i}}),t.webpackPolyfill=1),t}},function(t,e){t.exports=function(t){return"number"==typeof t&&t>-1&&t%1==0&&t<=9007199254740991}},function(t,e,r){(function(t){function n(t){return(n="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(t){return typeof t}:function(t){return t&&"function"==typeof Symbol&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t})(t)}var o=r(76),i="object"==n(e)&&e&&!e.nodeType&&e,a=i&&"object"==n(t)&&t&&!t.nodeType&&t,u=a&&a.exports===i&&o.process,s=function(){try{var t=a&&a.require&&a.require("util").types;return t||u&&u.binding&&u.binding("util")}catch(t){}}();t.exports=s}).call(this,r(60)(t))},function(t,e,r){var n=r(44),o=r(172),i=Object.prototype.hasOwnProperty;t.exports=function(t){if(!n(t))return o(t);var e=[];for(var r in Object(t))i.call(t,r)&&"constructor"!=r&&e.push(r);return e}},function(t,e,r){var n=r(83),o=r(84),i=Object.prototype.propertyIsEnumerable,a=Object.getOwnPropertySymbols,u=a?function(t){return null==t?[]:(t=Object(t),n(a(t),(function(e){return i.call(t,e)})))}:o;t.exports=u},function(t,e){t.exports=function(t,e){for(var r=-1,n=e.length,o=t.length;++r<n;)t[o+r]=e[r];return t}},function(t,e,r){var n=r(89);t.exports=function(t){var e=new t.constructor(t.byteLength);return new n(e).set(new n(t)),e}},function(t,e){t.exports=function(t){return function(){return t}}},function(t,e,r){var n=r(69),o=r(14);t.exports=function(t,e){return t&&n(t,e,o)}},function(t,e,r){var n=r(191)();t.exports=n},function(t,e){t.exports=function(t){var e=-1,r=Array(t.size);return t.forEach((function(t){r[++e]=t})),r}},function(t,e,r){function n(t){return(n="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(t){return typeof t}:function(t){return t&&"function"==typeof Symbol&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t})(t)}var o=r(1),i=r(24),a=/\\.|\\[(?:[^[\\]]*|(["\'])(?:(?!\\1)[^\\\\]|\\\\.)*?\\1)\\]/,u=/^\\w*$/;t.exports=function(t,e){if(o(t))return!1;var r=n(t);return!("number"!=r&&"symbol"!=r&&"boolean"!=r&&null!=t&&!i(t))||(u.test(t)||!a.test(t)||null!=e&&t in Object(e))}},function(t,e,r){var n=r(65),o=r(225);t.exports=function t(e,r,i,a,u){var s=-1,c=e.length;for(i||(i=o),u||(u=[]);++s<c;){var f=e[s];r>0&&i(f)?r>1?t(f,r-1,i,a,u):n(u,f):a||(u[u.length]=f)}return u}},function(t,e,r){var n=r(24);t.exports=function(t,e,r){for(var o=-1,i=t.length;++o<i;){var a=t[o],u=e(a);if(null!=u&&(void 0===s?u==u&&!n(u):r(u,s)))var s=u,c=a}return c}},function(t,e,r){t.exports={graphlib:r(7),layout:r(249),debug:r(308),util:{time:r(4).time,notime:r(4).notime},version:r(309)}},function(t,e,r){var n=r(35),o=r(59),i=r(40),a=r(167),u=r(173),s=r(81),c=r(82),f=r(176),h=r(177),l=r(86),d=r(178),p=r(23),v=r(182),g=r(183),y=r(91),m=r(1),b=r(22),w=r(187),x=r(6),M=r(189),k=r(14),_=r(17),S={};S["[object Arguments]"]=S["[object Array]"]=S["[object ArrayBuffer]"]=S["[object DataView]"]=S["[object Boolean]"]=S["[object Date]"]=S["[object Float32Array]"]=S["[object Float64Array]"]=S["[object Int8Array]"]=S["[object Int16Array]"]=S["[object Int32Array]"]=S["[object Map]"]=S["[object Number]"]=S["[object Object]"]=S["[object RegExp]"]=S["[object Set]"]=S["[object String]"]=S["[object Symbol]"]=S["[object Uint8Array]"]=S["[object Uint8ClampedArray]"]=S["[object Uint16Array]"]=S["[object Uint32Array]"]=!0,S["[object Error]"]=S["[object Function]"]=S["[object WeakMap]"]=!1,t.exports=function t(e,r,E,j,A,O){var N,I=1&r,R=2&r,P=4&r;if(E&&(N=A?E(e,j,A,O):E(e)),void 0!==N)return N;if(!x(e))return e;var C=m(e);if(C){if(N=v(e),!I)return c(e,N)}else{var z=p(e),F="[object Function]"==z||"[object GeneratorFunction]"==z;if(b(e))return s(e,I);if("[object Object]"==z||"[object Arguments]"==z||F&&!A){if(N=R||F?{}:y(e),!I)return R?h(e,u(N,e)):f(e,a(N,e))}else{if(!S[z])return A?e:{};N=g(e,z,I)}}O||(O=new n);var T=O.get(e);if(T)return T;O.set(e,N),M(e)?e.forEach((function(n){N.add(t(n,r,E,n,e,O))})):w(e)&&e.forEach((function(n,o){N.set(o,t(n,r,E,o,e,O))}));var D=C?void 0:(P?R?d:l:R?_:k)(e);return o(D||e,(function(n,o){D&&(n=e[o=n]),i(N,o,t(n,r,E,o,e,O))})),N}},function(t,e,r){(function(e){function r(t){return(r="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(t){return typeof t}:function(t){return t&&"function"==typeof Symbol&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t})(t)}var n="object"==(void 0===e?"undefined":r(e))&&e&&e.Object===Object&&e;t.exports=n}).call(this,r(149))},function(t,e){var r=Function.prototype.toString;t.exports=function(t){if(null!=t){try{return r.call(t)}catch(t){}try{return t+""}catch(t){}}return""}},function(t,e,r){var n=r(15),o=function(){try{var t=n(Object,"defineProperty");return t({},"",{}),t}catch(t){}}();t.exports=o},function(t,e,r){var n=r(168),o=r(27),i=r(1),a=r(22),u=r(42),s=r(28),c=Object.prototype.hasOwnProperty;t.exports=function(t,e){var r=i(t),f=!r&&o(t),h=!r&&!f&&a(t),l=!r&&!f&&!h&&s(t),d=r||f||h||l,p=d?n(t.length,String):[],v=p.length;for(var g in t)!e&&!c.call(t,g)||d&&("length"==g||h&&("offset"==g||"parent"==g)||l&&("buffer"==g||"byteLength"==g||"byteOffset"==g)||u(g,v))||p.push(g);return p}},function(t,e){t.exports=function(t,e){return function(r){return t(e(r))}}},function(t,e,r){(function(t){function n(t){return(n="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(t){return typeof t}:function(t){return t&&"function"==typeof Symbol&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t})(t)}var o=r(8),i="object"==n(e)&&e&&!e.nodeType&&e,a=i&&"object"==n(t)&&t&&!t.nodeType&&t,u=a&&a.exports===i?o.Buffer:void 0,s=u?u.allocUnsafe:void 0;t.exports=function(t,e){if(e)return t.slice();var r=t.length,n=s?s(r):new t.constructor(r);return t.copy(n),n}}).call(this,r(60)(t))},function(t,e){t.exports=function(t,e){var r=-1,n=t.length;for(e||(e=Array(n));++r<n;)e[r]=t[r];return e}},function(t,e){t.exports=function(t,e){for(var r=-1,n=null==t?0:t.length,o=0,i=[];++r<n;){var a=t[r];e(a,r,t)&&(i[o++]=a)}return i}},function(t,e){t.exports=function(){return[]}},function(t,e,r){var n=r(65),o=r(45),i=r(64),a=r(84),u=Object.getOwnPropertySymbols?function(t){for(var e=[];t;)n(e,i(t)),t=o(t);return e}:a;t.exports=u},function(t,e,r){var n=r(87),o=r(64),i=r(14);t.exports=function(t){return n(t,i,o)}},function(t,e,r){var n=r(65),o=r(1);t.exports=function(t,e,r){var i=e(t);return o(t)?i:n(i,r(t))}},function(t,e,r){var n=r(15)(r(8),"Set");t.exports=n},function(t,e,r){var n=r(8).Uint8Array;t.exports=n},function(t,e,r){var n=r(66);t.exports=function(t,e){var r=e?n(t.buffer):t.buffer;return new t.constructor(r,t.byteOffset,t.length)}},function(t,e,r){var n=r(92),o=r(45),i=r(44);t.exports=function(t){return"function"!=typeof t.constructor||i(t)?{}:n(o(t))}},function(t,e,r){var n=r(6),o=Object.create,i=function(){function t(){}return function(e){if(!n(e))return{};if(o)return o(e);t.prototype=e;var r=new t;return t.prototype=void 0,r}}();t.exports=i},function(t,e,r){t.exports=r(94)},function(t,e,r){var n=r(59),o=r(46),i=r(95),a=r(1);t.exports=function(t,e){return(a(t)?n:o)(t,i(e))}},function(t,e,r){var n=r(18);t.exports=function(t){return"function"==typeof t?t:n}},function(t,e,r){var n=r(83),o=r(193),i=r(13),a=r(1);t.exports=function(t,e){return(a(t)?n:o)(t,i(e,3))}},function(t,e,r){var n=r(196),o=r(10);t.exports=function t(e,r,i,a,u){return e===r||(null==e||null==r||!o(e)&&!o(r)?e!=e&&r!=r:n(e,r,i,a,t,u))}},function(t,e,r){var n=r(99),o=r(199),i=r(100);t.exports=function(t,e,r,a,u,s){var c=1&r,f=t.length,h=e.length;if(f!=h&&!(c&&h>f))return!1;var l=s.get(t),d=s.get(e);if(l&&d)return l==e&&d==t;var p=-1,v=!0,g=2&r?new n:void 0;for(s.set(t,e),s.set(e,t);++p<f;){var y=t[p],m=e[p];if(a)var b=c?a(m,y,p,e,t,s):a(y,m,p,t,e,s);if(void 0!==b){if(b)continue;v=!1;break}if(g){if(!o(e,(function(t,e){if(!i(g,e)&&(y===t||u(y,t,r,a,s)))return g.push(e)}))){v=!1;break}}else if(y!==m&&!u(y,m,r,a,s)){v=!1;break}}return s.delete(t),s.delete(e),v}},function(t,e,r){var n=r(58),o=r(197),i=r(198);function a(t){var e=-1,r=null==t?0:t.length;for(this.__data__=new n;++e<r;)this.add(t[e])}a.prototype.add=a.prototype.push=o,a.prototype.has=i,t.exports=a},function(t,e){t.exports=function(t,e){return t.has(e)}},function(t,e,r){var n=r(6);t.exports=function(t){return t==t&&!n(t)}},function(t,e){t.exports=function(t,e){return function(r){return null!=r&&(r[t]===e&&(void 0!==e||t in Object(r)))}}},function(t,e,r){var n=r(209);t.exports=function(t){return null==t?"":n(t)}},function(t,e,r){var n=r(210),o=r(105);t.exports=function(t,e){return null!=t&&o(t,e,n)}},function(t,e,r){var n=r(48),o=r(27),i=r(1),a=r(42),u=r(61),s=r(29);t.exports=function(t,e,r){for(var c=-1,f=(e=n(e,t)).length,h=!1;++c<f;){var l=s(e[c]);if(!(h=null!=t&&r(t,l)))break;t=t[l]}return h||++c!=f?h:!!(f=null==t?0:t.length)&&u(f)&&a(l,f)&&(i(t)||o(t))}},function(t,e){t.exports=function(t){return function(e){return null==e?void 0:e[t]}}},function(t,e,r){var n=r(213),o=r(105);t.exports=function(t,e){return null!=t&&o(t,e,n)}},function(t,e){t.exports=function(t){return void 0===t}},function(t,e,r){var n=r(49),o=r(13),i=r(110),a=r(1);t.exports=function(t,e){return(a(t)?n:i)(t,o(e,3))}},function(t,e,r){var n=r(46),o=r(12);t.exports=function(t,e){var r=-1,i=o(t)?Array(t.length):[];return n(t,(function(t,n,o){i[++r]=e(t,n,o)})),i}},function(t,e,r){var n=r(215),o=r(46),i=r(13),a=r(216),u=r(1);t.exports=function(t,e,r){var s=u(t)?n:a,c=arguments.length<3;return s(t,i(e,4),r,c,o)}},function(t,e,r){var n=r(226),o=Math.max;t.exports=function(t,e,r){return e=o(void 0===e?t.length-1:e,0),function(){for(var i=arguments,a=-1,u=o(i.length-e,0),s=Array(u);++a<u;)s[a]=i[e+a];a=-1;for(var c=Array(e+1);++a<e;)c[a]=i[a];return c[e]=r(s),n(t,this,c)}}},function(t,e,r){var n=r(227),o=r(228)(n);t.exports=o},function(t,e){t.exports=function(t,e,r,n){for(var o=t.length,i=r+(n?1:-1);n?i--:++i<o;)if(e(t[i],i,t))return i;return-1}},function(t,e,r){var n=r(12),o=r(10);t.exports=function(t){return o(t)&&n(t)}},function(t,e,r){var n=r(237),o=r(14);t.exports=function(t){return null==t?[]:n(t,o(t))}},function(t,e,r){var n=r(5),o=r(118);t.exports=function(t,e,r,n){return function(t,e,r,n){var i,a,u={},s=new o,c=function(t){var e=t.v!==i?t.v:t.w,n=u[e],o=r(t),c=a.distance+o;if(o<0)throw new Error("dijkstra does not allow negative edge weights. Bad edge: "+t+" Weight: "+o);c<n.distance&&(n.distance=c,n.predecessor=i,s.decrease(e,c))};t.nodes().forEach((function(t){var r=t===e?0:Number.POSITIVE_INFINITY;u[t]={distance:r},s.add(t,r)}));for(;s.size()>0&&(i=s.removeMin(),(a=u[i]).distance!==Number.POSITIVE_INFINITY);)n(i).forEach(c);return u}(t,String(e),r||i,n||function(e){return t.outEdges(e)})};var i=n.constant(1)},function(t,e,r){var n=r(5);function o(){this._arr=[],this._keyIndices={}}t.exports=o,o.prototype.size=function(){return this._arr.length},o.prototype.keys=function(){return this._arr.map((function(t){return t.key}))},o.prototype.has=function(t){return n.has(this._keyIndices,t)},o.prototype.priority=function(t){var e=this._keyIndices[t];if(void 0!==e)return this._arr[e].priority},o.prototype.min=function(){if(0===this.size())throw new Error("Queue underflow");return this._arr[0].key},o.prototype.add=function(t,e){var r=this._keyIndices;if(t=String(t),!n.has(r,t)){var o=this._arr,i=o.length;return r[t]=i,o.push({key:t,priority:e}),this._decrease(i),!0}return!1},o.prototype.removeMin=function(){this._swap(0,this._arr.length-1);var t=this._arr.pop();return delete this._keyIndices[t.key],this._heapify(0),t.key},o.prototype.decrease=function(t,e){var r=this._keyIndices[t];if(e>this._arr[r].priority)throw new Error("New priority is greater than current priority. Key: "+t+" Old: "+this._arr[r].priority+" New: "+e);this._arr[r].priority=e,this._decrease(r)},o.prototype._heapify=function(t){var e=this._arr,r=2*t,n=r+1,o=t;r<e.length&&(o=e[r].priority<e[o].priority?r:o,n<e.length&&(o=e[n].priority<e[o].priority?n:o),o!==t&&(this._swap(t,o),this._heapify(o)))},o.prototype._decrease=function(t){for(var e,r=this._arr,n=r[t].priority;0!==t&&!(r[e=t>>1].priority<n);)this._swap(t,e),t=e},o.prototype._swap=function(t,e){var r=this._arr,n=this._keyIndices,o=r[t],i=r[e];r[t]=i,r[e]=o,n[i.key]=t,n[o.key]=e}},function(t,e,r){var n=r(5);t.exports=function(t){var e=0,r=[],o={},i=[];return t.nodes().forEach((function(a){n.has(o,a)||function a(u){var s=o[u]={onStack:!0,lowlink:e,index:e++};if(r.push(u),t.successors(u).forEach((function(t){n.has(o,t)?o[t].onStack&&(s.lowlink=Math.min(s.lowlink,o[t].index)):(a(t),s.lowlink=Math.min(s.lowlink,o[t].lowlink))})),s.lowlink===s.index){var c,f=[];do{c=r.pop(),o[c].onStack=!1,f.push(c)}while(u!==c);i.push(f)}}(a)})),i}},function(t,e,r){var n=r(5);function o(t){var e={},r={},o=[];if(n.each(t.sinks(),(function a(u){if(n.has(r,u))throw new i;n.has(e,u)||(r[u]=!0,e[u]=!0,n.each(t.predecessors(u),a),delete r[u],o.push(u))})),n.size(e)!==t.nodeCount())throw new i;return o}function i(){}t.exports=o,o.CycleException=i,i.prototype=new Error},function(t,e,r){var n=r(5);t.exports=function(t,e,r){n.isArray(e)||(e=[e]);var o=(t.isDirected()?t.successors:t.neighbors).bind(t),i=[],a={};return n.each(e,(function(e){if(!t.hasNode(e))throw new Error("Graph does not have node: "+e);!function t(e,r,o,i,a,u){n.has(i,r)||(i[r]=!0,o||u.push(r),n.each(a(r),(function(r){t(e,r,o,i,a,u)})),o&&u.push(r))}(t,e,"post"===r,a,o,i)})),i}},function(t,e,r){var n=r(256);t.exports=function(t){return t?(t=n(t))===1/0||t===-1/0?17976931348623157e292*(t<0?-1:1):t==t?t:0:0===t?t:0}},function(t,e,r){var n=r(72);t.exports=function(t){return(null==t?0:t.length)?n(t,1):[]}},function(t,e,r){var n=r(41),o=r(20);t.exports=function(t,e,r){(void 0!==r&&!o(t[e],r)||void 0===r&&!(e in t))&&n(t,e,r)}},function(t,e){t.exports=function(t,e){if(("constructor"!==e||"function"!=typeof t[e])&&"__proto__"!=e)return t[e]}},function(t,e){t.exports=function(t,e){return t<e}},function(t,e,r){"use strict";var n=r(0),o=r(7).Graph,i=r(52).slack;function a(t,e){return n.forEach(t.nodes(),(function r(o){n.forEach(e.nodeEdges(o),(function(n){var a=n.v,u=o===a?n.w:a;t.hasNode(u)||i(e,n)||(t.setNode(u,{}),t.setEdge(o,u,{}),r(u))}))})),t.nodeCount()}function u(t,e){return n.minBy(e.edges(),(function(r){if(t.hasNode(r.v)!==t.hasNode(r.w))return i(e,r)}))}function s(t,e,r){n.forEach(t.nodes(),(function(t){e.node(t).rank+=r}))}t.exports=function(t){var e,r,n=new o({directed:!1}),c=t.nodes()[0],f=t.nodeCount();n.setNode(c,{});for(;a(n,t)<f;)e=u(n,t),r=n.hasNode(e.v)?i(t,e):-i(t,e),s(n,t,r);return n}},function(t,e,r){var n=r(315),o={};for(var i in n)n.hasOwnProperty(i)&&(o[n[i]]=i);var a=t.exports={rgb:{channels:3,labels:"rgb"},hsl:{channels:3,labels:"hsl"},hsv:{channels:3,labels:"hsv"},hwb:{channels:3,labels:"hwb"},cmyk:{channels:4,labels:"cmyk"},xyz:{channels:3,labels:"xyz"},lab:{channels:3,labels:"lab"},lch:{channels:3,labels:"lch"},hex:{channels:1,labels:["hex"]},keyword:{channels:1,labels:["keyword"]},ansi16:{channels:1,labels:["ansi16"]},ansi256:{channels:1,labels:["ansi256"]},hcg:{channels:3,labels:["h","c","g"]},apple:{channels:3,labels:["r16","g16","b16"]},gray:{channels:1,labels:["gray"]}};for(var u in a)if(a.hasOwnProperty(u)){if(!("channels"in a[u]))throw new Error("missing channels property: "+u);if(!("labels"in a[u]))throw new Error("missing channel labels property: "+u);if(a[u].labels.length!==a[u].channels)throw new Error("channel and label counts mismatch: "+u);var s=a[u].channels,c=a[u].labels;delete a[u].channels,delete a[u].labels,Object.defineProperty(a[u],"channels",{value:s}),Object.defineProperty(a[u],"labels",{value:c})}a.rgb.hsl=function(t){var e,r,n=t[0]/255,o=t[1]/255,i=t[2]/255,a=Math.min(n,o,i),u=Math.max(n,o,i),s=u-a;return u===a?e=0:n===u?e=(o-i)/s:o===u?e=2+(i-n)/s:i===u&&(e=4+(n-o)/s),(e=Math.min(60*e,360))<0&&(e+=360),r=(a+u)/2,[e,100*(u===a?0:r<=.5?s/(u+a):s/(2-u-a)),100*r]},a.rgb.hsv=function(t){var e,r,n,o,i,a=t[0]/255,u=t[1]/255,s=t[2]/255,c=Math.max(a,u,s),f=c-Math.min(a,u,s),h=function(t){return(c-t)/6/f+.5};return 0===f?o=i=0:(i=f/c,e=h(a),r=h(u),n=h(s),a===c?o=n-r:u===c?o=1/3+e-n:s===c&&(o=2/3+r-e),o<0?o+=1:o>1&&(o-=1)),[360*o,100*i,100*c]},a.rgb.hwb=function(t){var e=t[0],r=t[1],n=t[2];return[a.rgb.hsl(t)[0],100*(1/255*Math.min(e,Math.min(r,n))),100*(n=1-1/255*Math.max(e,Math.max(r,n)))]},a.rgb.cmyk=function(t){var e,r=t[0]/255,n=t[1]/255,o=t[2]/255;return[100*((1-r-(e=Math.min(1-r,1-n,1-o)))/(1-e)||0),100*((1-n-e)/(1-e)||0),100*((1-o-e)/(1-e)||0),100*e]},a.rgb.keyword=function(t){var e=o[t];if(e)return e;var r,i,a,u=1/0;for(var s in n)if(n.hasOwnProperty(s)){var c=n[s],f=(i=t,a=c,Math.pow(i[0]-a[0],2)+Math.pow(i[1]-a[1],2)+Math.pow(i[2]-a[2],2));f<u&&(u=f,r=s)}return r},a.keyword.rgb=function(t){return n[t]},a.rgb.xyz=function(t){var e=t[0]/255,r=t[1]/255,n=t[2]/255;return[100*(.4124*(e=e>.04045?Math.pow((e+.055)/1.055,2.4):e/12.92)+.3576*(r=r>.04045?Math.pow((r+.055)/1.055,2.4):r/12.92)+.1805*(n=n>.04045?Math.pow((n+.055)/1.055,2.4):n/12.92)),100*(.2126*e+.7152*r+.0722*n),100*(.0193*e+.1192*r+.9505*n)]},a.rgb.lab=function(t){var e=a.rgb.xyz(t),r=e[0],n=e[1],o=e[2];return n/=100,o/=108.883,r=(r/=95.047)>.008856?Math.pow(r,1/3):7.787*r+16/116,[116*(n=n>.008856?Math.pow(n,1/3):7.787*n+16/116)-16,500*(r-n),200*(n-(o=o>.008856?Math.pow(o,1/3):7.787*o+16/116))]},a.hsl.rgb=function(t){var e,r,n,o,i,a=t[0]/360,u=t[1]/100,s=t[2]/100;if(0===u)return[i=255*s,i,i];e=2*s-(r=s<.5?s*(1+u):s+u-s*u),o=[0,0,0];for(var c=0;c<3;c++)(n=a+1/3*-(c-1))<0&&n++,n>1&&n--,i=6*n<1?e+6*(r-e)*n:2*n<1?r:3*n<2?e+(r-e)*(2/3-n)*6:e,o[c]=255*i;return o},a.hsl.hsv=function(t){var e=t[0],r=t[1]/100,n=t[2]/100,o=r,i=Math.max(n,.01);return r*=(n*=2)<=1?n:2-n,o*=i<=1?i:2-i,[e,100*(0===n?2*o/(i+o):2*r/(n+r)),100*((n+r)/2)]},a.hsv.rgb=function(t){var e=t[0]/60,r=t[1]/100,n=t[2]/100,o=Math.floor(e)%6,i=e-Math.floor(e),a=255*n*(1-r),u=255*n*(1-r*i),s=255*n*(1-r*(1-i));switch(n*=255,o){case 0:return[n,s,a];case 1:return[u,n,a];case 2:return[a,n,s];case 3:return[a,u,n];case 4:return[s,a,n];case 5:return[n,a,u]}},a.hsv.hsl=function(t){var e,r,n,o=t[0],i=t[1]/100,a=t[2]/100,u=Math.max(a,.01);return n=(2-i)*a,r=i*u,[o,100*(r=(r/=(e=(2-i)*u)<=1?e:2-e)||0),100*(n/=2)]},a.hwb.rgb=function(t){var e,r,n,o,i,a,u,s=t[0]/360,c=t[1]/100,f=t[2]/100,h=c+f;switch(h>1&&(c/=h,f/=h),n=6*s-(e=Math.floor(6*s)),0!=(1&e)&&(n=1-n),o=c+n*((r=1-f)-c),e){default:case 6:case 0:i=r,a=o,u=c;break;case 1:i=o,a=r,u=c;break;case 2:i=c,a=r,u=o;break;case 3:i=c,a=o,u=r;break;case 4:i=o,a=c,u=r;break;case 5:i=r,a=c,u=o}return[255*i,255*a,255*u]},a.cmyk.rgb=function(t){var e=t[0]/100,r=t[1]/100,n=t[2]/100,o=t[3]/100;return[255*(1-Math.min(1,e*(1-o)+o)),255*(1-Math.min(1,r*(1-o)+o)),255*(1-Math.min(1,n*(1-o)+o))]},a.xyz.rgb=function(t){var e,r,n,o=t[0]/100,i=t[1]/100,a=t[2]/100;return r=-.9689*o+1.8758*i+.0415*a,n=.0557*o+-.204*i+1.057*a,e=(e=3.2406*o+-1.5372*i+-.4986*a)>.0031308?1.055*Math.pow(e,1/2.4)-.055:12.92*e,r=r>.0031308?1.055*Math.pow(r,1/2.4)-.055:12.92*r,n=n>.0031308?1.055*Math.pow(n,1/2.4)-.055:12.92*n,[255*(e=Math.min(Math.max(0,e),1)),255*(r=Math.min(Math.max(0,r),1)),255*(n=Math.min(Math.max(0,n),1))]},a.xyz.lab=function(t){var e=t[0],r=t[1],n=t[2];return r/=100,n/=108.883,e=(e/=95.047)>.008856?Math.pow(e,1/3):7.787*e+16/116,[116*(r=r>.008856?Math.pow(r,1/3):7.787*r+16/116)-16,500*(e-r),200*(r-(n=n>.008856?Math.pow(n,1/3):7.787*n+16/116))]},a.lab.xyz=function(t){var e,r,n,o=t[0];e=t[1]/500+(r=(o+16)/116),n=r-t[2]/200;var i=Math.pow(r,3),a=Math.pow(e,3),u=Math.pow(n,3);return r=i>.008856?i:(r-16/116)/7.787,e=a>.008856?a:(e-16/116)/7.787,n=u>.008856?u:(n-16/116)/7.787,[e*=95.047,r*=100,n*=108.883]},a.lab.lch=function(t){var e,r=t[0],n=t[1],o=t[2];return(e=360*Math.atan2(o,n)/2/Math.PI)<0&&(e+=360),[r,Math.sqrt(n*n+o*o),e]},a.lch.lab=function(t){var e,r=t[0],n=t[1];return e=t[2]/360*2*Math.PI,[r,n*Math.cos(e),n*Math.sin(e)]},a.rgb.ansi16=function(t){var e=t[0],r=t[1],n=t[2],o=1 in arguments?arguments[1]:a.rgb.hsv(t)[2];if(0===(o=Math.round(o/50)))return 30;var i=30+(Math.round(n/255)<<2|Math.round(r/255)<<1|Math.round(e/255));return 2===o&&(i+=60),i},a.hsv.ansi16=function(t){return a.rgb.ansi16(a.hsv.rgb(t),t[2])},a.rgb.ansi256=function(t){var e=t[0],r=t[1],n=t[2];return e===r&&r===n?e<8?16:e>248?231:Math.round((e-8)/247*24)+232:16+36*Math.round(e/255*5)+6*Math.round(r/255*5)+Math.round(n/255*5)},a.ansi16.rgb=function(t){var e=t%10;if(0===e||7===e)return t>50&&(e+=3.5),[e=e/10.5*255,e,e];var r=.5*(1+~~(t>50));return[(1&e)*r*255,(e>>1&1)*r*255,(e>>2&1)*r*255]},a.ansi256.rgb=function(t){if(t>=232){var e=10*(t-232)+8;return[e,e,e]}var r;return t-=16,[Math.floor(t/36)/5*255,Math.floor((r=t%36)/6)/5*255,r%6/5*255]},a.rgb.hex=function(t){var e=(((255&Math.round(t[0]))<<16)+((255&Math.round(t[1]))<<8)+(255&Math.round(t[2]))).toString(16).toUpperCase();return"000000".substring(e.length)+e},a.hex.rgb=function(t){var e=t.toString(16).match(/[a-f0-9]{6}|[a-f0-9]{3}/i);if(!e)return[0,0,0];var r=e[0];3===e[0].length&&(r=r.split("").map((function(t){return t+t})).join(""));var n=parseInt(r,16);return[n>>16&255,n>>8&255,255&n]},a.rgb.hcg=function(t){var e,r=t[0]/255,n=t[1]/255,o=t[2]/255,i=Math.max(Math.max(r,n),o),a=Math.min(Math.min(r,n),o),u=i-a;return e=u<=0?0:i===r?(n-o)/u%6:i===n?2+(o-r)/u:4+(r-n)/u+4,e/=6,[360*(e%=1),100*u,100*(u<1?a/(1-u):0)]},a.hsl.hcg=function(t){var e=t[1]/100,r=t[2]/100,n=1,o=0;return(n=r<.5?2*e*r:2*e*(1-r))<1&&(o=(r-.5*n)/(1-n)),[t[0],100*n,100*o]},a.hsv.hcg=function(t){var e=t[1]/100,r=t[2]/100,n=e*r,o=0;return n<1&&(o=(r-n)/(1-n)),[t[0],100*n,100*o]},a.hcg.rgb=function(t){var e=t[0]/360,r=t[1]/100,n=t[2]/100;if(0===r)return[255*n,255*n,255*n];var o,i=[0,0,0],a=e%1*6,u=a%1,s=1-u;switch(Math.floor(a)){case 0:i[0]=1,i[1]=u,i[2]=0;break;case 1:i[0]=s,i[1]=1,i[2]=0;break;case 2:i[0]=0,i[1]=1,i[2]=u;break;case 3:i[0]=0,i[1]=s,i[2]=1;break;case 4:i[0]=u,i[1]=0,i[2]=1;break;default:i[0]=1,i[1]=0,i[2]=s}return o=(1-r)*n,[255*(r*i[0]+o),255*(r*i[1]+o),255*(r*i[2]+o)]},a.hcg.hsv=function(t){var e=t[1]/100,r=e+t[2]/100*(1-e),n=0;return r>0&&(n=e/r),[t[0],100*n,100*r]},a.hcg.hsl=function(t){var e=t[1]/100,r=t[2]/100*(1-e)+.5*e,n=0;return r>0&&r<.5?n=e/(2*r):r>=.5&&r<1&&(n=e/(2*(1-r))),[t[0],100*n,100*r]},a.hcg.hwb=function(t){var e=t[1]/100,r=e+t[2]/100*(1-e);return[t[0],100*(r-e),100*(1-r)]},a.hwb.hcg=function(t){var e=t[1]/100,r=1-t[2]/100,n=r-e,o=0;return n<1&&(o=(r-n)/(1-n)),[t[0],100*n,100*o]},a.apple.rgb=function(t){return[t[0]/65535*255,t[1]/65535*255,t[2]/65535*255]},a.rgb.apple=function(t){return[t[0]/255*65535,t[1]/255*65535,t[2]/255*65535]},a.gray.rgb=function(t){return[t[0]/100*255,t[0]/100*255,t[0]/100*255]},a.gray.hsl=a.gray.hsv=function(t){return[0,0,t[0]]},a.gray.hwb=function(t){return[0,100,t[0]]},a.gray.cmyk=function(t){return[0,0,0,t[0]]},a.gray.lab=function(t){return[t[0],0,0]},a.gray.hex=function(t){var e=255&Math.round(t[0]/100*255),r=((e<<16)+(e<<8)+e).toString(16).toUpperCase();return"000000".substring(r.length)+r},a.rgb.gray=function(t){return[(t[0]+t[1]+t[2])/3/255*100]}},function(t,e,r){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),e.forEach=e.sqrLen=e.len=e.sqrDist=e.dist=e.div=e.mul=e.sub=void 0,e.create=o,e.clone=function(t){var e=new n.ARRAY_TYPE(3);return e[0]=t[0],e[1]=t[1],e[2]=t[2],e},e.length=i,e.fromValues=a,e.copy=function(t,e){return t[0]=e[0],t[1]=e[1],t[2]=e[2],t},e.set=function(t,e,r,n){return t[0]=e,t[1]=r,t[2]=n,t},e.add=function(t,e,r){return t[0]=e[0]+r[0],t[1]=e[1]+r[1],t[2]=e[2]+r[2],t},e.subtract=u,e.multiply=s,e.divide=c,e.ceil=function(t,e){return t[0]=Math.ceil(e[0]),t[1]=Math.ceil(e[1]),t[2]=Math.ceil(e[2]),t},e.floor=function(t,e){return t[0]=Math.floor(e[0]),t[1]=Math.floor(e[1]),t[2]=Math.floor(e[2]),t},e.min=function(t,e,r){return t[0]=Math.min(e[0],r[0]),t[1]=Math.min(e[1],r[1]),t[2]=Math.min(e[2],r[2]),t},e.max=function(t,e,r){return t[0]=Math.max(e[0],r[0]),t[1]=Math.max(e[1],r[1]),t[2]=Math.max(e[2],r[2]),t},e.round=function(t,e){return t[0]=Math.round(e[0]),t[1]=Math.round(e[1]),t[2]=Math.round(e[2]),t},e.scale=function(t,e,r){return t[0]=e[0]*r,t[1]=e[1]*r,t[2]=e[2]*r,t},e.scaleAndAdd=function(t,e,r,n){return t[0]=e[0]+r[0]*n,t[1]=e[1]+r[1]*n,t[2]=e[2]+r[2]*n,t},e.distance=f,e.squaredDistance=h,e.squaredLength=l,e.negate=function(t,e){return t[0]=-e[0],t[1]=-e[1],t[2]=-e[2],t},e.inverse=function(t,e){return t[0]=1/e[0],t[1]=1/e[1],t[2]=1/e[2],t},e.normalize=d,e.dot=p,e.cross=function(t,e,r){var n=e[0],o=e[1],i=e[2],a=r[0],u=r[1],s=r[2];return t[0]=o*s-i*u,t[1]=i*a-n*s,t[2]=n*u-o*a,t},e.lerp=function(t,e,r,n){var o=e[0],i=e[1],a=e[2];return t[0]=o+n*(r[0]-o),t[1]=i+n*(r[1]-i),t[2]=a+n*(r[2]-a),t},e.hermite=function(t,e,r,n,o,i){var a=i*i,u=a*(2*i-3)+1,s=a*(i-2)+i,c=a*(i-1),f=a*(3-2*i);return t[0]=e[0]*u+r[0]*s+n[0]*c+o[0]*f,t[1]=e[1]*u+r[1]*s+n[1]*c+o[1]*f,t[2]=e[2]*u+r[2]*s+n[2]*c+o[2]*f,t},e.bezier=function(t,e,r,n,o,i){var a=1-i,u=a*a,s=i*i,c=u*a,f=3*i*u,h=3*s*a,l=s*i;return t[0]=e[0]*c+r[0]*f+n[0]*h+o[0]*l,t[1]=e[1]*c+r[1]*f+n[1]*h+o[1]*l,t[2]=e[2]*c+r[2]*f+n[2]*h+o[2]*l,t},e.random=function(t,e){e=e||1;var r=2*n.RANDOM()*Math.PI,o=2*n.RANDOM()-1,i=Math.sqrt(1-o*o)*e;return t[0]=Math.cos(r)*i,t[1]=Math.sin(r)*i,t[2]=o*e,t},e.transformMat4=function(t,e,r){var n=e[0],o=e[1],i=e[2],a=r[3]*n+r[7]*o+r[11]*i+r[15];return a=a||1,t[0]=(r[0]*n+r[4]*o+r[8]*i+r[12])/a,t[1]=(r[1]*n+r[5]*o+r[9]*i+r[13])/a,t[2]=(r[2]*n+r[6]*o+r[10]*i+r[14])/a,t},e.transformMat3=function(t,e,r){var n=e[0],o=e[1],i=e[2];return t[0]=n*r[0]+o*r[3]+i*r[6],t[1]=n*r[1]+o*r[4]+i*r[7],t[2]=n*r[2]+o*r[5]+i*r[8],t},e.transformQuat=function(t,e,r){var n=r[0],o=r[1],i=r[2],a=r[3],u=e[0],s=e[1],c=e[2],f=o*c-i*s,h=i*u-n*c,l=n*s-o*u,d=o*l-i*h,p=i*f-n*l,v=n*h-o*f,g=2*a;return f*=g,h*=g,l*=g,d*=2,p*=2,v*=2,t[0]=u+f+d,t[1]=s+h+p,t[2]=c+l+v,t},e.rotateX=function(t,e,r,n){var o=[],i=[];return o[0]=e[0]-r[0],o[1]=e[1]-r[1],o[2]=e[2]-r[2],i[0]=o[0],i[1]=o[1]*Math.cos(n)-o[2]*Math.sin(n),i[2]=o[1]*Math.sin(n)+o[2]*Math.cos(n),t[0]=i[0]+r[0],t[1]=i[1]+r[1],t[2]=i[2]+r[2],t},e.rotateY=function(t,e,r,n){var o=[],i=[];return o[0]=e[0]-r[0],o[1]=e[1]-r[1],o[2]=e[2]-r[2],i[0]=o[2]*Math.sin(n)+o[0]*Math.cos(n),i[1]=o[1],i[2]=o[2]*Math.cos(n)-o[0]*Math.sin(n),t[0]=i[0]+r[0],t[1]=i[1]+r[1],t[2]=i[2]+r[2],t},e.rotateZ=function(t,e,r,n){var o=[],i=[];return o[0]=e[0]-r[0],o[1]=e[1]-r[1],o[2]=e[2]-r[2],i[0]=o[0]*Math.cos(n)-o[1]*Math.sin(n),i[1]=o[0]*Math.sin(n)+o[1]*Math.cos(n),i[2]=o[2],t[0]=i[0]+r[0],t[1]=i[1]+r[1],t[2]=i[2]+r[2],t},e.angle=function(t,e){var r=a(t[0],t[1],t[2]),n=a(e[0],e[1],e[2]);d(r,r),d(n,n);var o=p(r,n);return o>1?0:o<-1?Math.PI:Math.acos(o)},e.str=function(t){return"vec3("+t[0]+", "+t[1]+", "+t[2]+")"},e.exactEquals=function(t,e){return t[0]===e[0]&&t[1]===e[1]&&t[2]===e[2]},e.equals=function(t,e){var r=t[0],o=t[1],i=t[2],a=e[0],u=e[1],s=e[2];return Math.abs(r-a)<=n.EPSILON*Math.max(1,Math.abs(r),Math.abs(a))&&Math.abs(o-u)<=n.EPSILON*Math.max(1,Math.abs(o),Math.abs(u))&&Math.abs(i-s)<=n.EPSILON*Math.max(1,Math.abs(i),Math.abs(s))};var n=function(t){if(t&&t.__esModule)return t;var e={};if(null!=t)for(var r in t)Object.prototype.hasOwnProperty.call(t,r)&&(e[r]=t[r]);return e.default=t,e}(r(55));function o(){var t=new n.ARRAY_TYPE(3);return n.ARRAY_TYPE!=Float32Array&&(t[0]=0,t[1]=0,t[2]=0),t}function i(t){var e=t[0],r=t[1],n=t[2];return Math.sqrt(e*e+r*r+n*n)}function a(t,e,r){var o=new n.ARRAY_TYPE(3);return o[0]=t,o[1]=e,o[2]=r,o}function u(t,e,r){return t[0]=e[0]-r[0],t[1]=e[1]-r[1],t[2]=e[2]-r[2],t}function s(t,e,r){return t[0]=e[0]*r[0],t[1]=e[1]*r[1],t[2]=e[2]*r[2],t}function c(t,e,r){return t[0]=e[0]/r[0],t[1]=e[1]/r[1],t[2]=e[2]/r[2],t}function f(t,e){var r=e[0]-t[0],n=e[1]-t[1],o=e[2]-t[2];return Math.sqrt(r*r+n*n+o*o)}function h(t,e){var r=e[0]-t[0],n=e[1]-t[1],o=e[2]-t[2];return r*r+n*n+o*o}function l(t){var e=t[0],r=t[1],n=t[2];return e*e+r*r+n*n}function d(t,e){var r=e[0],n=e[1],o=e[2],i=r*r+n*n+o*o;return i>0&&(i=1/Math.sqrt(i),t[0]=e[0]*i,t[1]=e[1]*i,t[2]=e[2]*i),t}function p(t,e){return t[0]*e[0]+t[1]*e[1]+t[2]*e[2]}var v;e.sub=u,e.mul=s,e.div=c,e.dist=f,e.sqrDist=h,e.len=i,e.sqrLen=l,e.forEach=(v=o(),function(t,e,r,n,o,i){var a=void 0,u=void 0;for(e||(e=3),r||(r=0),u=n?Math.min(n*e+r,t.length):t.length,a=r;a<u;a+=e)v[0]=t[a],v[1]=t[a+1],v[2]=t[a+2],o(v,v,i),t[a]=v[0],t[a+1]=v[1],t[a+2]=v[2];return t})},function(t,e,r){"use strict";Object.defineProperty(e,"__esModule",{value:!0});var n=function(){function t(t,e){this.bubbles=!0,this.target=null,this.currentTarget=null,this.delegateTarget=null,this.delegateObject=null,this.defaultPrevented=!1,this.propagationStopped=!1,this.shape=null,this.fromShape=null,this.toShape=null,this.propagationPath=[],this.type=t,this.name=t,this.originalEvent=e,this.timeStamp=e.timeStamp}return t.prototype.preventDefault=function(){this.defaultPrevented=!0,this.originalEvent.preventDefault&&this.originalEvent.preventDefault()},t.prototype.stopPropagation=function(){this.propagationStopped=!0},t.prototype.toString=function(){return"[Event (type="+this.type+")]"},t.prototype.save=function(){},t.prototype.restore=function(){},t}();e.default=n},,,,function(t,e,r){"use strict";function n(t){return(n="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(t){return typeof t}:function(t){return t&&"function"==typeof Symbol&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t})(t)}Object.defineProperty(e,"__esModule",{value:!0}),e.default=function(t){var e=n(t);return null!==t&&"object"===e||"function"===e}},function(t,e,r){var n=r(136);t.exports={Graph:n.Graph,json:r(239),alg:r(240),version:n.version}},function(t,e,r){t.exports={Graph:r(56),version:r(238)}},function(t,e,r){var n=r(75);t.exports=function(t){return n(t,4)}},function(t,e){t.exports=function(){this.__data__=[],this.size=0}},function(t,e,r){var n=r(37),o=Array.prototype.splice;t.exports=function(t){var e=this.__data__,r=n(e,t);return!(r<0)&&(r==e.length-1?e.pop():o.call(e,r,1),--this.size,!0)}},function(t,e,r){var n=r(37);t.exports=function(t){var e=this.__data__,r=n(e,t);return r<0?void 0:e[r][1]}},function(t,e,r){var n=r(37);t.exports=function(t){return n(this.__data__,t)>-1}},function(t,e,r){var n=r(37);t.exports=function(t,e){var r=this.__data__,o=n(r,t);return o<0?(++this.size,r.push([t,e])):r[o][1]=e,this}},function(t,e,r){var n=r(36);t.exports=function(){this.__data__=new n,this.size=0}},function(t,e){t.exports=function(t){var e=this.__data__,r=e.delete(t);return this.size=e.size,r}},function(t,e){t.exports=function(t){return this.__data__.get(t)}},function(t,e){t.exports=function(t){return this.__data__.has(t)}},function(t,e,r){var n=r(36),o=r(57),i=r(58);t.exports=function(t,e){var r=this.__data__;if(r instanceof n){var a=r.__data__;if(!o||a.length<199)return a.push([t,e]),this.size=++r.size,this;r=this.__data__=new i(a)}return r.set(t,e),this.size=r.size,this}},function(t,e,r){var n=r(25),o=r(152),i=r(6),a=r(77),u=/^\\[object .+?Constructor\\]$/,s=Function.prototype,c=Object.prototype,f=s.toString,h=c.hasOwnProperty,l=RegExp("^"+f.call(h).replace(/[\\\\^$.*+?()[\\]{}|]/g,"\\\\$&").replace(/hasOwnProperty|(function).*?(?=\\\\\\()| for .+?(?=\\\\\\])/g,"$1.*?")+"$");t.exports=function(t){return!(!i(t)||o(t))&&(n(t)?l:u).test(a(t))}},function(t,e){function r(t){return(r="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(t){return typeof t}:function(t){return t&&"function"==typeof Symbol&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t})(t)}var n;n=function(){return this}();try{n=n||new Function("return this")()}catch(t){"object"===("undefined"==typeof window?"undefined":r(window))&&(n=window)}t.exports=n},function(t,e,r){var n=r(21),o=Object.prototype,i=o.hasOwnProperty,a=o.toString,u=n?n.toStringTag:void 0;t.exports=function(t){var e=i.call(t,u),r=t[u];try{t[u]=void 0;var n=!0}catch(t){}var o=a.call(t);return n&&(e?t[u]=r:delete t[u]),o}},function(t,e){var r=Object.prototype.toString;t.exports=function(t){return r.call(t)}},function(t,e,r){var n,o=r(153),i=(n=/[^.]+$/.exec(o&&o.keys&&o.keys.IE_PROTO||""))?"Symbol(src)_1."+n:"";t.exports=function(t){return!!i&&i in t}},function(t,e,r){var n=r(8)["__core-js_shared__"];t.exports=n},function(t,e){t.exports=function(t,e){return null==t?void 0:t[e]}},function(t,e,r){var n=r(156),o=r(36),i=r(57);t.exports=function(){this.size=0,this.__data__={hash:new n,map:new(i||o),string:new n}}},function(t,e,r){var n=r(157),o=r(158),i=r(159),a=r(160),u=r(161);function s(t){var e=-1,r=null==t?0:t.length;for(this.clear();++e<r;){var n=t[e];this.set(n[0],n[1])}}s.prototype.clear=n,s.prototype.delete=o,s.prototype.get=i,s.prototype.has=a,s.prototype.set=u,t.exports=s},function(t,e,r){var n=r(38);t.exports=function(){this.__data__=n?n(null):{},this.size=0}},function(t,e){t.exports=function(t){var e=this.has(t)&&delete this.__data__[t];return this.size-=e?1:0,e}},function(t,e,r){var n=r(38),o=Object.prototype.hasOwnProperty;t.exports=function(t){var e=this.__data__;if(n){var r=e[t];return"__lodash_hash_undefined__"===r?void 0:r}return o.call(e,t)?e[t]:void 0}},function(t,e,r){var n=r(38),o=Object.prototype.hasOwnProperty;t.exports=function(t){var e=this.__data__;return n?void 0!==e[t]:o.call(e,t)}},function(t,e,r){var n=r(38);t.exports=function(t,e){var r=this.__data__;return this.size+=this.has(t)?0:1,r[t]=n&&void 0===e?"__lodash_hash_undefined__":e,this}},function(t,e,r){var n=r(39);t.exports=function(t){var e=n(this,t).delete(t);return this.size-=e?1:0,e}},function(t,e){function r(t){return(r="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(t){return typeof t}:function(t){return t&&"function"==typeof Symbol&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t})(t)}t.exports=function(t){var e=r(t);return"string"==e||"number"==e||"symbol"==e||"boolean"==e?"__proto__"!==t:null===t}},function(t,e,r){var n=r(39);t.exports=function(t){return n(this,t).get(t)}},function(t,e,r){var n=r(39);t.exports=function(t){return n(this,t).has(t)}},function(t,e,r){var n=r(39);t.exports=function(t,e){var r=n(this,t),o=r.size;return r.set(t,e),this.size+=r.size==o?0:1,this}},function(t,e,r){var n=r(26),o=r(14);t.exports=function(t,e){return t&&n(e,o(e),t)}},function(t,e){t.exports=function(t,e){for(var r=-1,n=Array(t);++r<t;)n[r]=e(r);return n}},function(t,e,r){var n=r(16),o=r(10);t.exports=function(t){return o(t)&&"[object Arguments]"==n(t)}},function(t,e){t.exports=function(){return!1}},function(t,e,r){var n=r(16),o=r(61),i=r(10),a={};a["[object Float32Array]"]=a["[object Float64Array]"]=a["[object Int8Array]"]=a["[object Int16Array]"]=a["[object Int32Array]"]=a["[object Uint8Array]"]=a["[object Uint8ClampedArray]"]=a["[object Uint16Array]"]=a["[object Uint32Array]"]=!0,a["[object Arguments]"]=a["[object Array]"]=a["[object ArrayBuffer]"]=a["[object Boolean]"]=a["[object DataView]"]=a["[object Date]"]=a["[object Error]"]=a["[object Function]"]=a["[object Map]"]=a["[object Number]"]=a["[object Object]"]=a["[object RegExp]"]=a["[object Set]"]=a["[object String]"]=a["[object WeakMap]"]=!1,t.exports=function(t){return i(t)&&o(t.length)&&!!a[n(t)]}},function(t,e,r){var n=r(80)(Object.keys,Object);t.exports=n},function(t,e,r){var n=r(26),o=r(17);t.exports=function(t,e){return t&&n(e,o(e),t)}},function(t,e,r){var n=r(6),o=r(44),i=r(175),a=Object.prototype.hasOwnProperty;t.exports=function(t){if(!n(t))return i(t);var e=o(t),r=[];for(var u in t)("constructor"!=u||!e&&a.call(t,u))&&r.push(u);return r}},function(t,e){t.exports=function(t){var e=[];if(null!=t)for(var r in Object(t))e.push(r);return e}},function(t,e,r){var n=r(26),o=r(64);t.exports=function(t,e){return n(t,o(t),e)}},function(t,e,r){var n=r(26),o=r(85);t.exports=function(t,e){return n(t,o(t),e)}},function(t,e,r){var n=r(87),o=r(85),i=r(17);t.exports=function(t){return n(t,i,o)}},function(t,e,r){var n=r(15)(r(8),"DataView");t.exports=n},function(t,e,r){var n=r(15)(r(8),"Promise");t.exports=n},function(t,e,r){var n=r(15)(r(8),"WeakMap");t.exports=n},function(t,e){var r=Object.prototype.hasOwnProperty;t.exports=function(t){var e=t.length,n=new t.constructor(e);return e&&"string"==typeof t[0]&&r.call(t,"index")&&(n.index=t.index,n.input=t.input),n}},function(t,e,r){var n=r(66),o=r(184),i=r(185),a=r(186),u=r(90);t.exports=function(t,e,r){var s=t.constructor;switch(e){case"[object ArrayBuffer]":return n(t);case"[object Boolean]":case"[object Date]":return new s(+t);case"[object DataView]":return o(t,r);case"[object Float32Array]":case"[object Float64Array]":case"[object Int8Array]":case"[object Int16Array]":case"[object Int32Array]":case"[object Uint8Array]":case"[object Uint8ClampedArray]":case"[object Uint16Array]":case"[object Uint32Array]":return u(t,r);case"[object Map]":return new s;case"[object Number]":case"[object String]":return new s(t);case"[object RegExp]":return i(t);case"[object Set]":return new s;case"[object Symbol]":return a(t)}}},function(t,e,r){var n=r(66);t.exports=function(t,e){var r=e?n(t.buffer):t.buffer;return new t.constructor(r,t.byteOffset,t.byteLength)}},function(t,e){var r=/\\w*$/;t.exports=function(t){var e=new t.constructor(t.source,r.exec(t));return e.lastIndex=t.lastIndex,e}},function(t,e,r){var n=r(21),o=n?n.prototype:void 0,i=o?o.valueOf:void 0;t.exports=function(t){return i?Object(i.call(t)):{}}},function(t,e,r){var n=r(188),o=r(43),i=r(62),a=i&&i.isMap,u=a?o(a):n;t.exports=u},function(t,e,r){var n=r(23),o=r(10);t.exports=function(t){return o(t)&&"[object Map]"==n(t)}},function(t,e,r){var n=r(190),o=r(43),i=r(62),a=i&&i.isSet,u=a?o(a):n;t.exports=u},function(t,e,r){var n=r(23),o=r(10);t.exports=function(t){return o(t)&&"[object Set]"==n(t)}},function(t,e){t.exports=function(t){return function(e,r,n){for(var o=-1,i=Object(e),a=n(e),u=a.length;u--;){var s=a[t?u:++o];if(!1===r(i[s],s,i))break}return e}}},function(t,e,r){var n=r(12);t.exports=function(t,e){return function(r,o){if(null==r)return r;if(!n(r))return t(r,o);for(var i=r.length,a=e?i:-1,u=Object(r);(e?a--:++a<i)&&!1!==o(u[a],a,u););return r}}},function(t,e,r){var n=r(46);t.exports=function(t,e){var r=[];return n(t,(function(t,n,o){e(t,n,o)&&r.push(t)})),r}},function(t,e,r){var n=r(195),o=r(203),i=r(102);t.exports=function(t){var e=o(t);return 1==e.length&&e[0][2]?i(e[0][0],e[0][1]):function(r){return r===t||n(r,t,e)}}},function(t,e,r){var n=r(35),o=r(97);t.exports=function(t,e,r,i){var a=r.length,u=a,s=!i;if(null==t)return!u;for(t=Object(t);a--;){var c=r[a];if(s&&c[2]?c[1]!==t[c[0]]:!(c[0]in t))return!1}for(;++a<u;){var f=(c=r[a])[0],h=t[f],l=c[1];if(s&&c[2]){if(void 0===h&&!(f in t))return!1}else{var d=new n;if(i)var p=i(h,l,f,t,e,d);if(!(void 0===p?o(l,h,3,i,d):p))return!1}}return!0}},function(t,e,r){var n=r(35),o=r(98),i=r(200),a=r(202),u=r(23),s=r(1),c=r(22),f=r(28),h="[object Object]",l=Object.prototype.hasOwnProperty;t.exports=function(t,e,r,d,p,v){var g=s(t),y=s(e),m=g?"[object Array]":u(t),b=y?"[object Array]":u(e),w=(m="[object Arguments]"==m?h:m)==h,x=(b="[object Arguments]"==b?h:b)==h,M=m==b;if(M&&c(t)){if(!c(e))return!1;g=!0,w=!1}if(M&&!w)return v||(v=new n),g||f(t)?o(t,e,r,d,p,v):i(t,e,m,r,d,p,v);if(!(1&r)){var k=w&&l.call(t,"__wrapped__"),_=x&&l.call(e,"__wrapped__");if(k||_){var S=k?t.value():t,E=_?e.value():e;return v||(v=new n),p(S,E,r,d,v)}}return!!M&&(v||(v=new n),a(t,e,r,d,p,v))}},function(t,e){t.exports=function(t){return this.__data__.set(t,"__lodash_hash_undefined__"),this}},function(t,e){t.exports=function(t){return this.__data__.has(t)}},function(t,e){t.exports=function(t,e){for(var r=-1,n=null==t?0:t.length;++r<n;)if(e(t[r],r,t))return!0;return!1}},function(t,e,r){var n=r(21),o=r(89),i=r(20),a=r(98),u=r(201),s=r(70),c=n?n.prototype:void 0,f=c?c.valueOf:void 0;t.exports=function(t,e,r,n,c,h,l){switch(r){case"[object DataView]":if(t.byteLength!=e.byteLength||t.byteOffset!=e.byteOffset)return!1;t=t.buffer,e=e.buffer;case"[object ArrayBuffer]":return!(t.byteLength!=e.byteLength||!h(new o(t),new o(e)));case"[object Boolean]":case"[object Date]":case"[object Number]":return i(+t,+e);case"[object Error]":return t.name==e.name&&t.message==e.message;case"[object RegExp]":case"[object String]":return t==e+"";case"[object Map]":var d=u;case"[object Set]":var p=1&n;if(d||(d=s),t.size!=e.size&&!p)return!1;var v=l.get(t);if(v)return v==e;n|=2,l.set(t,e);var g=a(d(t),d(e),n,c,h,l);return l.delete(t),g;case"[object Symbol]":if(f)return f.call(t)==f.call(e)}return!1}},function(t,e){t.exports=function(t){var e=-1,r=Array(t.size);return t.forEach((function(t,n){r[++e]=[n,t]})),r}},function(t,e,r){var n=r(86),o=Object.prototype.hasOwnProperty;t.exports=function(t,e,r,i,a,u){var s=1&r,c=n(t),f=c.length;if(f!=n(e).length&&!s)return!1;for(var h=f;h--;){var l=c[h];if(!(s?l in e:o.call(e,l)))return!1}var d=u.get(t),p=u.get(e);if(d&&p)return d==e&&p==t;var v=!0;u.set(t,e),u.set(e,t);for(var g=s;++h<f;){var y=t[l=c[h]],m=e[l];if(i)var b=s?i(m,y,l,e,t,u):i(y,m,l,t,e,u);if(!(void 0===b?y===m||a(y,m,r,i,u):b)){v=!1;break}g||(g="constructor"==l)}if(v&&!g){var w=t.constructor,x=e.constructor;w==x||!("constructor"in t)||!("constructor"in e)||"function"==typeof w&&w instanceof w&&"function"==typeof x&&x instanceof x||(v=!1)}return u.delete(t),u.delete(e),v}},function(t,e,r){var n=r(101),o=r(14);t.exports=function(t){for(var e=o(t),r=e.length;r--;){var i=e[r],a=t[i];e[r]=[i,a,n(a)]}return e}},function(t,e,r){var n=r(97),o=r(205),i=r(104),a=r(71),u=r(101),s=r(102),c=r(29);t.exports=function(t,e){return a(t)&&u(e)?s(c(t),e):function(r){var a=o(r,t);return void 0===a&&a===e?i(r,t):n(e,a,3)}}},function(t,e,r){var n=r(47);t.exports=function(t,e,r){var o=null==t?void 0:n(t,e);return void 0===o?r:o}},function(t,e,r){var n=r(207),o=/[^.[\\]]+|\\[(?:(-?\\d+(?:\\.\\d+)?)|(["\'])((?:(?!\\2)[^\\\\]|\\\\.)*?)\\2)\\]|(?=(?:\\.|\\[\\])(?:\\.|\\[\\]|$))/g,i=/\\\\(\\\\)?/g,a=n((function(t){var e=[];return 46===t.charCodeAt(0)&&e.push(""),t.replace(o,(function(t,r,n,o){e.push(n?o.replace(i,"$1"):r||t)})),e}));t.exports=a},function(t,e,r){var n=r(208);t.exports=function(t){var e=n(t,(function(t){return 500===r.size&&r.clear(),t})),r=e.cache;return e}},function(t,e,r){var n=r(58);function o(t,e){if("function"!=typeof t||null!=e&&"function"!=typeof e)throw new TypeError("Expected a function");var r=function r(){var n=arguments,o=e?e.apply(this,n):n[0],i=r.cache;if(i.has(o))return i.get(o);var a=t.apply(this,n);return r.cache=i.set(o,a)||i,a};return r.cache=new(o.Cache||n),r}o.Cache=n,t.exports=o},function(t,e,r){var n=r(21),o=r(49),i=r(1),a=r(24),u=n?n.prototype:void 0,s=u?u.toString:void 0;t.exports=function t(e){if("string"==typeof e)return e;if(i(e))return o(e,t)+"";if(a(e))return s?s.call(e):"";var r=e+"";return"0"==r&&1/e==-1/0?"-0":r}},function(t,e){t.exports=function(t,e){return null!=t&&e in Object(t)}},function(t,e,r){var n=r(106),o=r(212),i=r(71),a=r(29);t.exports=function(t){return i(t)?n(a(t)):o(t)}},function(t,e,r){var n=r(47);t.exports=function(t){return function(e){return n(e,t)}}},function(t,e){var r=Object.prototype.hasOwnProperty;t.exports=function(t,e){return null!=t&&r.call(t,e)}},function(t,e,r){var n=r(63),o=r(23),i=r(27),a=r(1),u=r(12),s=r(22),c=r(44),f=r(28),h=Object.prototype.hasOwnProperty;t.exports=function(t){if(null==t)return!0;if(u(t)&&(a(t)||"string"==typeof t||"function"==typeof t.splice||s(t)||f(t)||i(t)))return!t.length;var e=o(t);if("[object Map]"==e||"[object Set]"==e)return!t.size;if(c(t))return!n(t).length;for(var r in t)if(h.call(t,r))return!1;return!0}},function(t,e){t.exports=function(t,e,r,n){var o=-1,i=null==t?0:t.length;for(n&&i&&(r=t[++o]);++o<i;)r=e(r,t[o],o,t);return r}},function(t,e){t.exports=function(t,e,r,n,o){return o(t,(function(t,o,i){r=n?(n=!1,t):e(r,t,o,i)})),r}},function(t,e,r){var n=r(63),o=r(23),i=r(12),a=r(218),u=r(219);t.exports=function(t){if(null==t)return 0;if(i(t))return a(t)?u(t):t.length;var e=o(t);return"[object Map]"==e||"[object Set]"==e?t.size:n(t).length}},function(t,e,r){var n=r(16),o=r(1),i=r(10);t.exports=function(t){return"string"==typeof t||!o(t)&&i(t)&&"[object String]"==n(t)}},function(t,e,r){var n=r(220),o=r(221),i=r(222);t.exports=function(t){return o(t)?i(t):n(t)}},function(t,e,r){var n=r(106)("length");t.exports=n},function(t,e){var r=RegExp("[\\\\u200d\\\\ud800-\\\\udfff\\\\u0300-\\\\u036f\\\\ufe20-\\\\ufe2f\\\\u20d0-\\\\u20ff\\\\ufe0e\\\\ufe0f]");t.exports=function(t){return r.test(t)}},function(t,e){var r="[\\\\ud800-\\\\udfff]",n="[\\\\u0300-\\\\u036f\\\\ufe20-\\\\ufe2f\\\\u20d0-\\\\u20ff]",o="\\\\ud83c[\\\\udffb-\\\\udfff]",i="[^\\\\ud800-\\\\udfff]",a="(?:\\\\ud83c[\\\\udde6-\\\\uddff]){2}",u="[\\\\ud800-\\\\udbff][\\\\udc00-\\\\udfff]",s="(?:"+n+"|"+o+")"+"?",c="[\\\\ufe0e\\\\ufe0f]?"+s+("(?:\\\\u200d(?:"+[i,a,u].join("|")+")[\\\\ufe0e\\\\ufe0f]?"+s+")*"),f="(?:"+[i+n+"?",n,a,u,r].join("|")+")",h=RegExp(o+"(?="+o+")|"+f+c,"g");t.exports=function(t){for(var e=h.lastIndex=0;h.test(t);)++e;return e}},function(t,e,r){var n=r(59),o=r(92),i=r(68),a=r(13),u=r(45),s=r(1),c=r(22),f=r(25),h=r(6),l=r(28);t.exports=function(t,e,r){var d=s(t),p=d||c(t)||l(t);if(e=a(e,4),null==r){var v=t&&t.constructor;r=p?d?new v:[]:h(t)&&f(v)?o(u(t)):{}}return(p?n:i)(t,(function(t,n,o){return e(r,t,n,o)})),r}},function(t,e,r){var n=r(72),o=r(50),i=r(229),a=r(115),u=o((function(t){return i(n(t,1,a,!0))}));t.exports=u},function(t,e,r){var n=r(21),o=r(27),i=r(1),a=n?n.isConcatSpreadable:void 0;t.exports=function(t){return i(t)||o(t)||!!(a&&t&&t[a])}},function(t,e){t.exports=function(t,e,r){switch(r.length){case 0:return t.call(e);case 1:return t.call(e,r[0]);case 2:return t.call(e,r[0],r[1]);case 3:return t.call(e,r[0],r[1],r[2])}return t.apply(e,r)}},function(t,e,r){var n=r(67),o=r(78),i=r(18),a=o?function(t,e){return o(t,"toString",{configurable:!0,enumerable:!1,value:n(e),writable:!0})}:i;t.exports=a},function(t,e){var r=Date.now;t.exports=function(t){var e=0,n=0;return function(){var o=r(),i=16-(o-n);if(n=o,i>0){if(++e>=800)return arguments[0]}else e=0;return t.apply(void 0,arguments)}}},function(t,e,r){var n=r(99),o=r(230),i=r(234),a=r(100),u=r(235),s=r(70);t.exports=function(t,e,r){var c=-1,f=o,h=t.length,l=!0,d=[],p=d;if(r)l=!1,f=i;else if(h>=200){var v=e?null:u(t);if(v)return s(v);l=!1,f=a,p=new n}else p=e?[]:d;t:for(;++c<h;){var g=t[c],y=e?e(g):g;if(g=r||0!==g?g:0,l&&y==y){for(var m=p.length;m--;)if(p[m]===y)continue t;e&&p.push(y),d.push(g)}else f(p,y,r)||(p!==d&&p.push(y),d.push(g))}return d}},function(t,e,r){var n=r(231);t.exports=function(t,e){return!!(null==t?0:t.length)&&n(t,e,0)>-1}},function(t,e,r){var n=r(114),o=r(232),i=r(233);t.exports=function(t,e,r){return e==e?i(t,e,r):n(t,o,r)}},function(t,e){t.exports=function(t){return t!=t}},function(t,e){t.exports=function(t,e,r){for(var n=r-1,o=t.length;++n<o;)if(t[n]===e)return n;return-1}},function(t,e){t.exports=function(t,e,r){for(var n=-1,o=null==t?0:t.length;++n<o;)if(r(e,t[n]))return!0;return!1}},function(t,e,r){var n=r(88),o=r(236),i=r(70),a=n&&1/i(new n([,-0]))[1]==1/0?function(t){return new n(t)}:o;t.exports=a},function(t,e){t.exports=function(){}},function(t,e,r){var n=r(49);t.exports=function(t,e){return n(e,(function(e){return t[e]}))}},function(t,e){t.exports="2.1.8"},function(t,e,r){var n=r(5),o=r(56);function i(t){return n.map(t.nodes(),(function(e){var r=t.node(e),o=t.parent(e),i={v:e};return n.isUndefined(r)||(i.value=r),n.isUndefined(o)||(i.parent=o),i}))}function a(t){return n.map(t.edges(),(function(e){var r=t.edge(e),o={v:e.v,w:e.w};return n.isUndefined(e.name)||(o.name=e.name),n.isUndefined(r)||(o.value=r),o}))}t.exports={write:function(t){var e={options:{directed:t.isDirected(),multigraph:t.isMultigraph(),compound:t.isCompound()},nodes:i(t),edges:a(t)};n.isUndefined(t.graph())||(e.value=n.clone(t.graph()));return e},read:function(t){var e=new o(t.options).setGraph(t.value);return n.each(t.nodes,(function(t){e.setNode(t.v,t.value),t.parent&&e.setParent(t.v,t.parent)})),n.each(t.edges,(function(t){e.setEdge({v:t.v,w:t.w,name:t.name},t.value)})),e}}},function(t,e,r){t.exports={components:r(241),dijkstra:r(117),dijkstraAll:r(242),findCycles:r(243),floydWarshall:r(244),isAcyclic:r(245),postorder:r(246),preorder:r(247),prim:r(248),tarjan:r(119),topsort:r(120)}},function(t,e,r){var n=r(5);t.exports=function(t){var e,r={},o=[];function i(o){n.has(r,o)||(r[o]=!0,e.push(o),n.each(t.successors(o),i),n.each(t.predecessors(o),i))}return n.each(t.nodes(),(function(t){e=[],i(t),e.length&&o.push(e)})),o}},function(t,e,r){var n=r(117),o=r(5);t.exports=function(t,e,r){return o.transform(t.nodes(),(function(o,i){o[i]=n(t,i,e,r)}),{})}},function(t,e,r){var n=r(5),o=r(119);t.exports=function(t){return n.filter(o(t),(function(e){return e.length>1||1===e.length&&t.hasEdge(e[0],e[0])}))}},function(t,e,r){var n=r(5);t.exports=function(t,e,r){return function(t,e,r){var n={},o=t.nodes();return o.forEach((function(t){n[t]={},n[t][t]={distance:0},o.forEach((function(e){t!==e&&(n[t][e]={distance:Number.POSITIVE_INFINITY})})),r(t).forEach((function(r){var o=r.v===t?r.w:r.v,i=e(r);n[t][o]={distance:i,predecessor:t}}))})),o.forEach((function(t){var e=n[t];o.forEach((function(r){var i=n[r];o.forEach((function(r){var n=i[t],o=e[r],a=i[r],u=n.distance+o.distance;u<a.distance&&(a.distance=u,a.predecessor=o.predecessor)}))}))})),n}(t,e||o,r||function(e){return t.outEdges(e)})};var o=n.constant(1)},function(t,e,r){var n=r(120);t.exports=function(t){try{n(t)}catch(t){if(t instanceof n.CycleException)return!1;throw t}return!0}},function(t,e,r){var n=r(121);t.exports=function(t,e){return n(t,e,"post")}},function(t,e,r){var n=r(121);t.exports=function(t,e){return n(t,e,"pre")}},function(t,e,r){var n=r(5),o=r(56),i=r(118);t.exports=function(t,e){var r,a=new o,u={},s=new i;function c(t){var n=t.v===r?t.w:t.v,o=s.priority(n);if(void 0!==o){var i=e(t);i<o&&(u[n]=r,s.decrease(n,i))}}if(0===t.nodeCount())return a;n.each(t.nodes(),(function(t){s.add(t,Number.POSITIVE_INFINITY),a.setNode(t)})),s.decrease(t.nodes()[0],0);var f=!1;for(;s.size()>0;){if(r=s.removeMin(),n.has(u,r))a.setEdge(r,u[r]);else{if(f)throw new Error("Input graph is not connected: "+t);f=!0}t.nodeEdges(r).forEach(c)}return a}},function(t,e,r){"use strict";var n=r(0),o=r(287),i=r(290),a=r(291),u=r(4).normalizeRanks,s=r(293),c=r(4).removeEmptyRanks,f=r(294),h=r(295),l=r(296),d=r(297),p=r(306),v=r(4),g=r(7).Graph;t.exports=function(t,e){var r=e&&e.debugTiming?v.time:v.notime;r("layout",(function(){var e=r("  buildLayoutGraph",(function(){return function(t){var e=new g({multigraph:!0,compound:!0}),r=E(t.graph());return e.setGraph(n.merge({},m,S(r,y),n.pick(r,b))),n.forEach(t.nodes(),(function(r){var o=E(t.node(r));e.setNode(r,n.defaults(S(o,w),x)),e.setParent(r,t.parent(r))})),n.forEach(t.edges(),(function(r){var o=E(t.edge(r));e.setEdge(r,n.merge({},k,S(o,M),n.pick(o,_)))})),e}(t)}));r("  runLayout",(function(){!function(t,e){e("    makeSpaceForEdgeLabels",(function(){!function(t){var e=t.graph();e.ranksep/=2,n.forEach(t.edges(),(function(r){var n=t.edge(r);n.minlen*=2,"c"!==n.labelpos.toLowerCase()&&("TB"===e.rankdir||"BT"===e.rankdir?n.width+=n.labeloffset:n.height+=n.labeloffset)}))}(t)})),e("    removeSelfEdges",(function(){!function(t){n.forEach(t.edges(),(function(e){if(e.v===e.w){var r=t.node(e.v);r.selfEdges||(r.selfEdges=[]),r.selfEdges.push({e:e,label:t.edge(e)}),t.removeEdge(e)}}))}(t)})),e("    acyclic",(function(){o.run(t)})),e("    nestingGraph.run",(function(){f.run(t)})),e("    rank",(function(){a(v.asNonCompoundGraph(t))})),e("    injectEdgeLabelProxies",(function(){!function(t){n.forEach(t.edges(),(function(e){var r=t.edge(e);if(r.width&&r.height){var n=t.node(e.v),o={rank:(t.node(e.w).rank-n.rank)/2+n.rank,e:e};v.addDummyNode(t,"edge-proxy",o,"_ep")}}))}(t)})),e("    removeEmptyRanks",(function(){c(t)})),e("    nestingGraph.cleanup",(function(){f.cleanup(t)})),e("    normalizeRanks",(function(){u(t)})),e("    assignRankMinMax",(function(){!function(t){var e=0;n.forEach(t.nodes(),(function(r){var o=t.node(r);o.borderTop&&(o.minRank=t.node(o.borderTop).rank,o.maxRank=t.node(o.borderBottom).rank,e=n.max(e,o.maxRank))})),t.graph().maxRank=e}(t)})),e("    removeEdgeLabelProxies",(function(){!function(t){n.forEach(t.nodes(),(function(e){var r=t.node(e);"edge-proxy"===r.dummy&&(t.edge(r.e).labelRank=r.rank,t.removeNode(e))}))}(t)})),e("    normalize.run",(function(){i.run(t)})),e("    parentDummyChains",(function(){s(t)})),e("    addBorderSegments",(function(){h(t)})),e("    order",(function(){d(t)})),e("    insertSelfEdges",(function(){!function(t){var e=v.buildLayerMatrix(t);n.forEach(e,(function(e){var r=0;n.forEach(e,(function(e,o){var i=t.node(e);i.order=o+r,n.forEach(i.selfEdges,(function(e){v.addDummyNode(t,"selfedge",{width:e.label.width,height:e.label.height,rank:i.rank,order:o+ ++r,e:e.e,label:e.label},"_se")})),delete i.selfEdges}))}))}(t)})),e("    adjustCoordinateSystem",(function(){l.adjust(t)})),e("    position",(function(){p(t)})),e("    positionSelfEdges",(function(){!function(t){n.forEach(t.nodes(),(function(e){var r=t.node(e);if("selfedge"===r.dummy){var n=t.node(r.e.v),o=n.x+n.width/2,i=n.y,a=r.x-o,u=n.height/2;t.setEdge(r.e,r.label),t.removeNode(e),r.label.points=[{x:o+2*a/3,y:i-u},{x:o+5*a/6,y:i-u},{x:o+a,y:i},{x:o+5*a/6,y:i+u},{x:o+2*a/3,y:i+u}],r.label.x=r.x,r.label.y=r.y}}))}(t)})),e("    removeBorderNodes",(function(){!function(t){n.forEach(t.nodes(),(function(e){if(t.children(e).length){var r=t.node(e),o=t.node(r.borderTop),i=t.node(r.borderBottom),a=t.node(n.last(r.borderLeft)),u=t.node(n.last(r.borderRight));r.width=Math.abs(u.x-a.x),r.height=Math.abs(i.y-o.y),r.x=a.x+r.width/2,r.y=o.y+r.height/2}})),n.forEach(t.nodes(),(function(e){"border"===t.node(e).dummy&&t.removeNode(e)}))}(t)})),e("    normalize.undo",(function(){i.undo(t)})),e("    fixupEdgeLabelCoords",(function(){!function(t){n.forEach(t.edges(),(function(e){var r=t.edge(e);if(n.has(r,"x"))switch("l"!==r.labelpos&&"r"!==r.labelpos||(r.width-=r.labeloffset),r.labelpos){case"l":r.x-=r.width/2+r.labeloffset;break;case"r":r.x+=r.width/2+r.labeloffset}}))}(t)})),e("    undoCoordinateSystem",(function(){l.undo(t)})),e("    translateGraph",(function(){!function(t){var e=Number.POSITIVE_INFINITY,r=0,o=Number.POSITIVE_INFINITY,i=0,a=t.graph(),u=a.marginx||0,s=a.marginy||0;function c(t){var n=t.x,a=t.y,u=t.width,s=t.height;e=Math.min(e,n-u/2),r=Math.max(r,n+u/2),o=Math.min(o,a-s/2),i=Math.max(i,a+s/2)}n.forEach(t.nodes(),(function(e){c(t.node(e))})),n.forEach(t.edges(),(function(e){var r=t.edge(e);n.has(r,"x")&&c(r)})),e-=u,o-=s,n.forEach(t.nodes(),(function(r){var n=t.node(r);n.x-=e,n.y-=o})),n.forEach(t.edges(),(function(r){var i=t.edge(r);n.forEach(i.points,(function(t){t.x-=e,t.y-=o})),n.has(i,"x")&&(i.x-=e),n.has(i,"y")&&(i.y-=o)})),a.width=r-e+u,a.height=i-o+s}(t)})),e("    assignNodeIntersects",(function(){!function(t){n.forEach(t.edges(),(function(e){var r,n,o=t.edge(e),i=t.node(e.v),a=t.node(e.w);o.points?(r=o.points[0],n=o.points[o.points.length-1]):(o.points=[],r=a,n=i),o.points.unshift(v.intersectRect(i,r)),o.points.push(v.intersectRect(a,n))}))}(t)})),e("    reversePoints",(function(){!function(t){n.forEach(t.edges(),(function(e){var r=t.edge(e);r.reversed&&r.points.reverse()}))}(t)})),e("    acyclic.undo",(function(){o.undo(t)}))}(e,r)})),r("  updateInputGraph",(function(){!function(t,e){n.forEach(t.nodes(),(function(r){var n=t.node(r),o=e.node(r);n&&(n.x=o.x,n.y=o.y,e.children(r).length&&(n.width=o.width,n.height=o.height))})),n.forEach(t.edges(),(function(r){var o=t.edge(r),i=e.edge(r);o.points=i.points,n.has(i,"x")&&(o.x=i.x,o.y=i.y)})),t.graph().width=e.graph().width,t.graph().height=e.graph().height}(t,e)}))}))};var y=["nodesep","edgesep","ranksep","marginx","marginy"],m={ranksep:50,edgesep:20,nodesep:50,rankdir:"tb"},b=["acyclicer","ranker","rankdir","align"],w=["width","height"],x={width:0,height:0},M=["minlen","weight","width","height","labeloffset"],k={minlen:1,weight:1,width:0,height:0,labeloffset:10,labelpos:"r"},_=["labelpos"];function S(t,e){return n.mapValues(n.pick(t,e),Number)}function E(t){var e={};return n.forEach(t,(function(t,r){e[r.toLowerCase()]=t})),e}},function(t,e,r){var n=r(75);t.exports=function(t){return n(t,5)}},function(t,e,r){var n=r(50),o=r(20),i=r(51),a=r(17),u=Object.prototype,s=u.hasOwnProperty,c=n((function(t,e){t=Object(t);var r=-1,n=e.length,c=n>2?e[2]:void 0;for(c&&i(e[0],e[1],c)&&(n=1);++r<n;)for(var f=e[r],h=a(f),l=-1,d=h.length;++l<d;){var p=h[l],v=t[p];(void 0===v||o(v,u[p])&&!s.call(t,p))&&(t[p]=f[p])}return t}));t.exports=c},function(t,e,r){var n=r(253)(r(254));t.exports=n},function(t,e,r){var n=r(13),o=r(12),i=r(14);t.exports=function(t){return function(e,r,a){var u=Object(e);if(!o(e)){var s=n(r,3);e=i(e),r=function(t){return s(u[t],t,u)}}var c=t(e,r,a);return c>-1?u[s?e[c]:c]:void 0}}},function(t,e,r){var n=r(114),o=r(13),i=r(255),a=Math.max;t.exports=function(t,e,r){var u=null==t?0:t.length;if(!u)return-1;var s=null==r?0:i(r);return s<0&&(s=a(u+s,0)),n(t,o(e,3),s)}},function(t,e,r){var n=r(122);t.exports=function(t){var e=n(t),r=e%1;return e==e?r?e-r:e:0}},function(t,e,r){var n=r(6),o=r(24),i=/^\\s+|\\s+$/g,a=/^[-+]0x[0-9a-f]+$/i,u=/^0b[01]+$/i,s=/^0o[0-7]+$/i,c=parseInt;t.exports=function(t){if("number"==typeof t)return t;if(o(t))return NaN;if(n(t)){var e="function"==typeof t.valueOf?t.valueOf():t;t=n(e)?e+"":e}if("string"!=typeof t)return 0===t?t:+t;t=t.replace(i,"");var r=u.test(t);return r||s.test(t)?c(t.slice(2),r?2:8):a.test(t)?NaN:+t}},function(t,e,r){var n=r(69),o=r(95),i=r(17);t.exports=function(t,e){return null==t?t:n(t,o(e),i)}},function(t,e){t.exports=function(t){var e=null==t?0:t.length;return e?t[e-1]:void 0}},function(t,e,r){var n=r(41),o=r(68),i=r(13);t.exports=function(t,e){var r={};return e=i(e,3),o(t,(function(t,o,i){n(r,o,e(t,o,i))})),r}},function(t,e,r){var n=r(73),o=r(261),i=r(18);t.exports=function(t){return t&&t.length?n(t,i,o):void 0}},function(t,e){t.exports=function(t,e){return t>e}},function(t,e,r){var n=r(263),o=r(267)((function(t,e,r){n(t,e,r)}));t.exports=o},function(t,e,r){var n=r(35),o=r(124),i=r(69),a=r(264),u=r(6),s=r(17),c=r(125);t.exports=function t(e,r,f,h,l){e!==r&&i(r,(function(i,s){if(l||(l=new n),u(i))a(e,r,s,f,t,h,l);else{var d=h?h(c(e,s),i,s+"",e,r,l):void 0;void 0===d&&(d=i),o(e,s,d)}}),s)}},function(t,e,r){var n=r(124),o=r(81),i=r(90),a=r(82),u=r(91),s=r(27),c=r(1),f=r(115),h=r(22),l=r(25),d=r(6),p=r(265),v=r(28),g=r(125),y=r(266);t.exports=function(t,e,r,m,b,w,x){var M=g(t,r),k=g(e,r),_=x.get(k);if(_)n(t,r,_);else{var S=w?w(M,k,r+"",t,e,x):void 0,E=void 0===S;if(E){var j=c(k),A=!j&&h(k),O=!j&&!A&&v(k);S=k,j||A||O?c(M)?S=M:f(M)?S=a(M):A?(E=!1,S=o(k,!0)):O?(E=!1,S=i(k,!0)):S=[]:p(k)||s(k)?(S=M,s(M)?S=y(M):d(M)&&!l(M)||(S=u(k))):E=!1}E&&(x.set(k,S),b(S,k,m,w,x),x.delete(k)),n(t,r,S)}}},function(t,e,r){var n=r(16),o=r(45),i=r(10),a=Function.prototype,u=Object.prototype,s=a.toString,c=u.hasOwnProperty,f=s.call(Object);t.exports=function(t){if(!i(t)||"[object Object]"!=n(t))return!1;var e=o(t);if(null===e)return!0;var r=c.call(e,"constructor")&&e.constructor;return"function"==typeof r&&r instanceof r&&s.call(r)==f}},function(t,e,r){var n=r(26),o=r(17);t.exports=function(t){return n(t,o(t))}},function(t,e,r){var n=r(50),o=r(51);t.exports=function(t){return n((function(e,r){var n=-1,i=r.length,a=i>1?r[i-1]:void 0,u=i>2?r[2]:void 0;for(a=t.length>3&&"function"==typeof a?(i--,a):void 0,u&&o(r[0],r[1],u)&&(a=i<3?void 0:a,i=1),e=Object(e);++n<i;){var s=r[n];s&&t(e,s,n,a)}return e}))}},function(t,e,r){var n=r(73),o=r(126),i=r(18);t.exports=function(t){return t&&t.length?n(t,i,o):void 0}},function(t,e,r){var n=r(73),o=r(13),i=r(126);t.exports=function(t,e){return t&&t.length?n(t,o(e,2),i):void 0}},function(t,e,r){var n=r(8);t.exports=function(){return n.Date.now()}},function(t,e,r){var n=r(272),o=r(275)((function(t,e){return null==t?{}:n(t,e)}));t.exports=o},function(t,e,r){var n=r(273),o=r(104);t.exports=function(t,e){return n(t,e,(function(e,r){return o(t,r)}))}},function(t,e,r){var n=r(47),o=r(274),i=r(48);t.exports=function(t,e,r){for(var a=-1,u=e.length,s={};++a<u;){var c=e[a],f=n(t,c);r(f,c)&&o(s,i(c,t),f)}return s}},function(t,e,r){var n=r(40),o=r(48),i=r(42),a=r(6),u=r(29);t.exports=function(t,e,r,s){if(!a(t))return t;for(var c=-1,f=(e=o(e,t)).length,h=f-1,l=t;null!=l&&++c<f;){var d=u(e[c]),p=r;if("__proto__"===d||"constructor"===d||"prototype"===d)return t;if(c!=h){var v=l[d];void 0===(p=s?s(v,d,l):void 0)&&(p=a(v)?v:i(e[c+1])?[]:{})}n(l,d,p),l=l[d]}return t}},function(t,e,r){var n=r(123),o=r(112),i=r(113);t.exports=function(t){return i(o(t,void 0,n),t+"")}},function(t,e,r){var n=r(277)();t.exports=n},function(t,e,r){var n=r(278),o=r(51),i=r(122);t.exports=function(t){return function(e,r,a){return a&&"number"!=typeof a&&o(e,r,a)&&(r=a=void 0),e=i(e),void 0===r?(r=e,e=0):r=i(r),a=void 0===a?e<r?1:-1:i(a),n(e,r,a,t)}}},function(t,e){var r=Math.ceil,n=Math.max;t.exports=function(t,e,o,i){for(var a=-1,u=n(r((e-t)/(o||1)),0),s=Array(u);u--;)s[i?u:++a]=t,t+=o;return s}},function(t,e,r){var n=r(72),o=r(280),i=r(50),a=r(51),u=i((function(t,e){if(null==t)return[];var r=e.length;return r>1&&a(t,e[0],e[1])?e=[]:r>2&&a(e[0],e[1],e[2])&&(e=[e[0]]),o(t,n(e,1),[])}));t.exports=u},function(t,e,r){var n=r(49),o=r(47),i=r(13),a=r(110),u=r(281),s=r(43),c=r(282),f=r(18),h=r(1);t.exports=function(t,e,r){e=e.length?n(e,(function(t){return h(t)?function(e){return o(e,1===t.length?t[0]:t)}:t})):[f];var l=-1;e=n(e,s(i));var d=a(t,(function(t,r,o){return{criteria:n(e,(function(e){return e(t)})),index:++l,value:t}}));return u(d,(function(t,e){return c(t,e,r)}))}},function(t,e){t.exports=function(t,e){var r=t.length;for(t.sort(e);r--;)t[r]=t[r].value;return t}},function(t,e,r){var n=r(283);t.exports=function(t,e,r){for(var o=-1,i=t.criteria,a=e.criteria,u=i.length,s=r.length;++o<u;){var c=n(i[o],a[o]);if(c)return o>=s?c:c*("desc"==r[o]?-1:1)}return t.index-e.index}},function(t,e,r){var n=r(24);t.exports=function(t,e){if(t!==e){var r=void 0!==t,o=null===t,i=t==t,a=n(t),u=void 0!==e,s=null===e,c=e==e,f=n(e);if(!s&&!f&&!a&&t>e||a&&u&&c&&!s&&!f||o&&u&&c||!r&&c||!i)return 1;if(!o&&!a&&!f&&t<e||f&&r&&i&&!o&&!a||s&&r&&i||!u&&i||!c)return-1}return 0}},function(t,e,r){var n=r(103),o=0;t.exports=function(t){var e=++o;return n(t)+e}},function(t,e,r){var n=r(40),o=r(286);t.exports=function(t,e){return o(t||[],e||[],n)}},function(t,e){t.exports=function(t,e,r){for(var n=-1,o=t.length,i=e.length,a={};++n<o;){var u=n<i?e[n]:void 0;r(a,t[n],u)}return a}},function(t,e,r){"use strict";var n=r(0),o=r(288);t.exports={run:function(t){var e="greedy"===t.graph().acyclicer?o(t,function(t){return function(e){return t.edge(e).weight}}(t)):function(t){var e=[],r={},o={};function i(a){n.has(o,a)||(o[a]=!0,r[a]=!0,n.forEach(t.outEdges(a),(function(t){n.has(r,t.w)?e.push(t):i(t.w)})),delete r[a])}return n.forEach(t.nodes(),i),e}(t);n.forEach(e,(function(e){var r=t.edge(e);t.removeEdge(e),r.forwardName=e.name,r.reversed=!0,t.setEdge(e.w,e.v,r,n.uniqueId("rev"))}))},undo:function(t){n.forEach(t.edges(),(function(e){var r=t.edge(e);if(r.reversed){t.removeEdge(e);var n=r.forwardName;delete r.reversed,delete r.forwardName,t.setEdge(e.w,e.v,r,n)}}))}}},function(t,e,r){var n=r(0),o=r(7).Graph,i=r(289);t.exports=function(t,e){if(t.nodeCount()<=1)return[];var r=function(t,e){var r=new o,a=0,u=0;n.forEach(t.nodes(),(function(t){r.setNode(t,{v:t,in:0,out:0})})),n.forEach(t.edges(),(function(t){var n=r.edge(t.v,t.w)||0,o=e(t),i=n+o;r.setEdge(t.v,t.w,i),u=Math.max(u,r.node(t.v).out+=o),a=Math.max(a,r.node(t.w).in+=o)}));var c=n.range(u+a+3).map((function(){return new i})),f=a+1;return n.forEach(r.nodes(),(function(t){s(c,f,r.node(t))})),{graph:r,buckets:c,zeroIdx:f}}(t,e||a),c=function(t,e,r){var n,o=[],i=e[e.length-1],a=e[0];for(;t.nodeCount();){for(;n=a.dequeue();)u(t,e,r,n);for(;n=i.dequeue();)u(t,e,r,n);if(t.nodeCount())for(var s=e.length-2;s>0;--s)if(n=e[s].dequeue()){o=o.concat(u(t,e,r,n,!0));break}}return o}(r.graph,r.buckets,r.zeroIdx);return n.flatten(n.map(c,(function(e){return t.outEdges(e.v,e.w)})),!0)};var a=n.constant(1);function u(t,e,r,o,i){var a=i?[]:void 0;return n.forEach(t.inEdges(o.v),(function(n){var o=t.edge(n),u=t.node(n.v);i&&a.push({v:n.v,w:n.w}),u.out-=o,s(e,r,u)})),n.forEach(t.outEdges(o.v),(function(n){var o=t.edge(n),i=n.w,a=t.node(i);a.in-=o,s(e,r,a)})),t.removeNode(o.v),a}function s(t,e,r){r.out?r.in?t[r.out-r.in+e].enqueue(r):t[t.length-1].enqueue(r):t[0].enqueue(r)}},function(t,e){function r(){var t={};t._next=t._prev=t,this._sentinel=t}function n(t){t._prev._next=t._next,t._next._prev=t._prev,delete t._next,delete t._prev}function o(t,e){if("_next"!==t&&"_prev"!==t)return e}t.exports=r,r.prototype.dequeue=function(){var t=this._sentinel,e=t._prev;if(e!==t)return n(e),e},r.prototype.enqueue=function(t){var e=this._sentinel;t._prev&&t._next&&n(t),t._next=e._next,e._next._prev=t,e._next=t,t._prev=e},r.prototype.toString=function(){for(var t=[],e=this._sentinel,r=e._prev;r!==e;)t.push(JSON.stringify(r,o)),r=r._prev;return"["+t.join(", ")+"]"}},function(t,e,r){"use strict";var n=r(0),o=r(4);t.exports={run:function(t){t.graph().dummyChains=[],n.forEach(t.edges(),(function(e){!function(t,e){var r,n,i,a=e.v,u=t.node(a).rank,s=e.w,c=t.node(s).rank,f=e.name,h=t.edge(e),l=h.labelRank;if(c===u+1)return;for(t.removeEdge(e),i=0,++u;u<c;++i,++u)h.points=[],n={width:0,height:0,edgeLabel:h,edgeObj:e,rank:u},r=o.addDummyNode(t,"edge",n,"_d"),u===l&&(n.width=h.width,n.height=h.height,n.dummy="edge-label",n.labelpos=h.labelpos),t.setEdge(a,r,{weight:h.weight},f),0===i&&t.graph().dummyChains.push(r),a=r;t.setEdge(a,s,{weight:h.weight},f)}(t,e)}))},undo:function(t){n.forEach(t.graph().dummyChains,(function(e){var r,n=t.node(e),o=n.edgeLabel;for(t.setEdge(n.edgeObj,o);n.dummy;)r=t.successors(e)[0],t.removeNode(e),o.points.push({x:n.x,y:n.y}),"edge-label"===n.dummy&&(o.x=n.x,o.y=n.y,o.width=n.width,o.height=n.height),e=r,n=t.node(e)}))}}},function(t,e,r){"use strict";var n=r(52).longestPath,o=r(127),i=r(292);t.exports=function(t){switch(t.graph().ranker){case"network-simplex":u(t);break;case"tight-tree":!function(t){n(t),o(t)}(t);break;case"longest-path":a(t);break;default:u(t)}};var a=n;function u(t){i(t)}},function(t,e,r){"use strict";var n=r(0),o=r(127),i=r(52).slack,a=r(52).longestPath,u=r(7).alg.preorder,s=r(7).alg.postorder,c=r(4).simplify;function f(t){t=c(t),a(t);var e,r=o(t);for(d(r),h(r,t);e=v(r);)y(r,t,e,g(r,t,e))}function h(t,e){var r=s(t,t.nodes());r=r.slice(0,r.length-1),n.forEach(r,(function(r){!function(t,e,r){var n=t.node(r).parent;t.edge(r,n).cutvalue=l(t,e,r)}(t,e,r)}))}function l(t,e,r){var o=t.node(r).parent,i=!0,a=e.edge(r,o),u=0;return a||(i=!1,a=e.edge(o,r)),u=a.weight,n.forEach(e.nodeEdges(r),(function(n){var a,s,c=n.v===r,f=c?n.w:n.v;if(f!==o){var h=c===i,l=e.edge(n).weight;if(u+=h?l:-l,a=r,s=f,t.hasEdge(a,s)){var d=t.edge(r,f).cutvalue;u+=h?-d:d}}})),u}function d(t,e){arguments.length<2&&(e=t.nodes()[0]),p(t,{},1,e)}function p(t,e,r,o,i){var a=r,u=t.node(o);return e[o]=!0,n.forEach(t.neighbors(o),(function(i){n.has(e,i)||(r=p(t,e,r,i,o))})),u.low=a,u.lim=r++,i?u.parent=i:delete u.parent,r}function v(t){return n.find(t.edges(),(function(e){return t.edge(e).cutvalue<0}))}function g(t,e,r){var o=r.v,a=r.w;e.hasEdge(o,a)||(o=r.w,a=r.v);var u=t.node(o),s=t.node(a),c=u,f=!1;u.lim>s.lim&&(c=s,f=!0);var h=n.filter(e.edges(),(function(e){return f===m(t,t.node(e.v),c)&&f!==m(t,t.node(e.w),c)}));return n.minBy(h,(function(t){return i(e,t)}))}function y(t,e,r,o){var i=r.v,a=r.w;t.removeEdge(i,a),t.setEdge(o.v,o.w,{}),d(t),h(t,e),function(t,e){var r=n.find(t.nodes(),(function(t){return!e.node(t).parent})),o=u(t,r);o=o.slice(1),n.forEach(o,(function(r){var n=t.node(r).parent,o=e.edge(r,n),i=!1;o||(o=e.edge(n,r),i=!0),e.node(r).rank=e.node(n).rank+(i?o.minlen:-o.minlen)}))}(t,e)}function m(t,e,r){return r.low<=e.lim&&e.lim<=r.lim}t.exports=f,f.initLowLimValues=d,f.initCutValues=h,f.calcCutValue=l,f.leaveEdge=v,f.enterEdge=g,f.exchangeEdges=y},function(t,e,r){var n=r(0);t.exports=function(t){var e=function(t){var e={},r=0;function o(i){var a=r;n.forEach(t.children(i),o),e[i]={low:a,lim:r++}}return n.forEach(t.children(),o),e}(t);n.forEach(t.graph().dummyChains,(function(r){for(var n=t.node(r),o=n.edgeObj,i=function(t,e,r,n){var o,i,a=[],u=[],s=Math.min(e[r].low,e[n].low),c=Math.max(e[r].lim,e[n].lim);o=r;do{o=t.parent(o),a.push(o)}while(o&&(e[o].low>s||c>e[o].lim));i=o,o=n;for(;(o=t.parent(o))!==i;)u.push(o);return{path:a.concat(u.reverse()),lca:i}}(t,e,o.v,o.w),a=i.path,u=i.lca,s=0,c=a[s],f=!0;r!==o.w;){if(n=t.node(r),f){for(;(c=a[s])!==u&&t.node(c).maxRank<n.rank;)s++;c===u&&(f=!1)}if(!f){for(;s<a.length-1&&t.node(c=a[s+1]).minRank<=n.rank;)s++;c=a[s]}t.setParent(r,c),r=t.successors(r)[0]}}))}},function(t,e,r){var n=r(0),o=r(4);t.exports={run:function(t){var e=o.addDummyNode(t,"root",{},"_root"),r=function(t){var e={};return n.forEach(t.children(),(function(r){!function r(o,i){var a=t.children(o);a&&a.length&&n.forEach(a,(function(t){r(t,i+1)}));e[o]=i}(r,1)})),e}(t),i=n.max(n.values(r))-1,a=2*i+1;t.graph().nestingRoot=e,n.forEach(t.edges(),(function(e){t.edge(e).minlen*=a}));var u=function(t){return n.reduce(t.edges(),(function(e,r){return e+t.edge(r).weight}),0)}(t)+1;n.forEach(t.children(),(function(s){!function t(e,r,i,a,u,s,c){var f=e.children(c);if(!f.length)return void(c!==r&&e.setEdge(r,c,{weight:0,minlen:i}));var h=o.addBorderNode(e,"_bt"),l=o.addBorderNode(e,"_bb"),d=e.node(c);e.setParent(h,c),d.borderTop=h,e.setParent(l,c),d.borderBottom=l,n.forEach(f,(function(n){t(e,r,i,a,u,s,n);var o=e.node(n),f=o.borderTop?o.borderTop:n,d=o.borderBottom?o.borderBottom:n,p=o.borderTop?a:2*a,v=f!==d?1:u-s[c]+1;e.setEdge(h,f,{weight:p,minlen:v,nestingEdge:!0}),e.setEdge(d,l,{weight:p,minlen:v,nestingEdge:!0})})),e.parent(c)||e.setEdge(r,h,{weight:0,minlen:u+s[c]})}(t,e,a,u,i,r,s)})),t.graph().nodeRankFactor=a},cleanup:function(t){var e=t.graph();t.removeNode(e.nestingRoot),delete e.nestingRoot,n.forEach(t.edges(),(function(e){t.edge(e).nestingEdge&&t.removeEdge(e)}))}}},function(t,e,r){var n=r(0),o=r(4);function i(t,e,r,n,i,a){var u={width:0,height:0,rank:a,borderType:e},s=i[e][a-1],c=o.addDummyNode(t,"border",u,r);i[e][a]=c,t.setParent(c,n),s&&t.setEdge(s,c,{weight:1})}t.exports=function(t){n.forEach(t.children(),(function e(r){var o=t.children(r),a=t.node(r);if(o.length&&n.forEach(o,e),n.has(a,"minRank")){a.borderLeft=[],a.borderRight=[];for(var u=a.minRank,s=a.maxRank+1;u<s;++u)i(t,"borderLeft","_bl",r,a,u),i(t,"borderRight","_br",r,a,u)}}))}},function(t,e,r){"use strict";var n=r(0);function o(t){n.forEach(t.nodes(),(function(e){i(t.node(e))})),n.forEach(t.edges(),(function(e){i(t.edge(e))}))}function i(t){var e=t.width;t.width=t.height,t.height=e}function a(t){t.y=-t.y}function u(t){var e=t.x;t.x=t.y,t.y=e}t.exports={adjust:function(t){var e=t.graph().rankdir.toLowerCase();"lr"!==e&&"rl"!==e||o(t)},undo:function(t){var e=t.graph().rankdir.toLowerCase();"bt"!==e&&"rl"!==e||function(t){n.forEach(t.nodes(),(function(e){a(t.node(e))})),n.forEach(t.edges(),(function(e){var r=t.edge(e);n.forEach(r.points,a),n.has(r,"y")&&a(r)}))}(t);"lr"!==e&&"rl"!==e||(!function(t){n.forEach(t.nodes(),(function(e){u(t.node(e))})),n.forEach(t.edges(),(function(e){var r=t.edge(e);n.forEach(r.points,u),n.has(r,"x")&&u(r)}))}(t),o(t))}}},function(t,e,r){"use strict";var n=r(0),o=r(298),i=r(299),a=r(300),u=r(304),s=r(305),c=r(7).Graph,f=r(4);function h(t,e,r){return n.map(e,(function(e){return u(t,e,r)}))}function l(t,e){var r=new c;n.forEach(t,(function(t){var o=t.graph().root,i=a(t,o,r,e);n.forEach(i.vs,(function(e,r){t.node(e).order=r})),s(t,r,i.vs)}))}function d(t,e){n.forEach(e,(function(e){n.forEach(e,(function(e,r){t.node(e).order=r}))}))}t.exports=function(t){var e=f.maxRank(t),r=h(t,n.range(1,e+1),"inEdges"),a=h(t,n.range(e-1,-1,-1),"outEdges"),u=o(t);d(t,u);for(var s,c=Number.POSITIVE_INFINITY,p=0,v=0;v<4;++p,++v){l(p%2?r:a,p%4>=2),u=f.buildLayerMatrix(t);var g=i(t,u);g<c&&(v=0,s=n.cloneDeep(u),c=g)}d(t,s)}},function(t,e,r){"use strict";var n=r(0);t.exports=function(t){var e={},r=n.filter(t.nodes(),(function(e){return!t.children(e).length})),o=n.max(n.map(r,(function(e){return t.node(e).rank}))),i=n.map(n.range(o+1),(function(){return[]}));var a=n.sortBy(r,(function(e){return t.node(e).rank}));return n.forEach(a,(function r(o){if(n.has(e,o))return;e[o]=!0;var a=t.node(o);i[a.rank].push(o),n.forEach(t.successors(o),r)})),i}},function(t,e,r){"use strict";var n=r(0);function o(t,e,r){for(var o=n.zipObject(r,n.map(r,(function(t,e){return e}))),i=n.flatten(n.map(e,(function(e){return n.sortBy(n.map(t.outEdges(e),(function(e){return{pos:o[e.w],weight:t.edge(e).weight}})),"pos")})),!0),a=1;a<r.length;)a<<=1;var u=2*a-1;a-=1;var s=n.map(new Array(u),(function(){return 0})),c=0;return n.forEach(i.forEach((function(t){var e=t.pos+a;s[e]+=t.weight;for(var r=0;e>0;)e%2&&(r+=s[e+1]),s[e=e-1>>1]+=t.weight;c+=t.weight*r}))),c}t.exports=function(t,e){for(var r=0,n=1;n<e.length;++n)r+=o(t,e[n-1],e[n]);return r}},function(t,e,r){var n=r(0),o=r(301),i=r(302),a=r(303);t.exports=function t(e,r,u,s){var c=e.children(r),f=e.node(r),h=f?f.borderLeft:void 0,l=f?f.borderRight:void 0,d={};h&&(c=n.filter(c,(function(t){return t!==h&&t!==l})));var p=o(e,c);n.forEach(p,(function(r){if(e.children(r.v).length){var o=t(e,r.v,u,s);d[r.v]=o,n.has(o,"barycenter")&&(i=r,a=o,n.isUndefined(i.barycenter)?(i.barycenter=a.barycenter,i.weight=a.weight):(i.barycenter=(i.barycenter*i.weight+a.barycenter*a.weight)/(i.weight+a.weight),i.weight+=a.weight))}var i,a}));var v=i(p,u);!function(t,e){n.forEach(t,(function(t){t.vs=n.flatten(t.vs.map((function(t){return e[t]?e[t].vs:t})),!0)}))}(v,d);var g=a(v,s);if(h&&(g.vs=n.flatten([h,g.vs,l],!0),e.predecessors(h).length)){var y=e.node(e.predecessors(h)[0]),m=e.node(e.predecessors(l)[0]);n.has(g,"barycenter")||(g.barycenter=0,g.weight=0),g.barycenter=(g.barycenter*g.weight+y.order+m.order)/(g.weight+2),g.weight+=2}return g}},function(t,e,r){var n=r(0);t.exports=function(t,e){return n.map(e,(function(e){var r=t.inEdges(e);if(r.length){var o=n.reduce(r,(function(e,r){var n=t.edge(r),o=t.node(r.v);return{sum:e.sum+n.weight*o.order,weight:e.weight+n.weight}}),{sum:0,weight:0});return{v:e,barycenter:o.sum/o.weight,weight:o.weight}}return{v:e}}))}},function(t,e,r){"use strict";var n=r(0);t.exports=function(t,e){var r={};return n.forEach(t,(function(t,e){var o=r[t.v]={indegree:0,in:[],out:[],vs:[t.v],i:e};n.isUndefined(t.barycenter)||(o.barycenter=t.barycenter,o.weight=t.weight)})),n.forEach(e.edges(),(function(t){var e=r[t.v],o=r[t.w];n.isUndefined(e)||n.isUndefined(o)||(o.indegree++,e.out.push(r[t.w]))})),function(t){var e=[];function r(t){return function(e){e.merged||(n.isUndefined(e.barycenter)||n.isUndefined(t.barycenter)||e.barycenter>=t.barycenter)&&function(t,e){var r=0,n=0;t.weight&&(r+=t.barycenter*t.weight,n+=t.weight);e.weight&&(r+=e.barycenter*e.weight,n+=e.weight);t.vs=e.vs.concat(t.vs),t.barycenter=r/n,t.weight=n,t.i=Math.min(e.i,t.i),e.merged=!0}(t,e)}}function o(e){return function(r){r.in.push(e),0==--r.indegree&&t.push(r)}}for(;t.length;){var i=t.pop();e.push(i),n.forEach(i.in.reverse(),r(i)),n.forEach(i.out,o(i))}return n.map(n.filter(e,(function(t){return!t.merged})),(function(t){return n.pick(t,["vs","i","barycenter","weight"])}))}(n.filter(r,(function(t){return!t.indegree})))}},function(t,e,r){var n=r(0),o=r(4);function i(t,e,r){for(var o;e.length&&(o=n.last(e)).i<=r;)e.pop(),t.push(o.vs),r++;return r}t.exports=function(t,e){var r=o.partition(t,(function(t){return n.has(t,"barycenter")})),a=r.lhs,u=n.sortBy(r.rhs,(function(t){return-t.i})),s=[],c=0,f=0,h=0;a.sort((l=!!e,function(t,e){return t.barycenter<e.barycenter?-1:t.barycenter>e.barycenter?1:l?e.i-t.i:t.i-e.i})),h=i(s,u,h),n.forEach(a,(function(t){h+=t.vs.length,s.push(t.vs),c+=t.barycenter*t.weight,f+=t.weight,h=i(s,u,h)}));var l;var d={vs:n.flatten(s,!0)};f&&(d.barycenter=c/f,d.weight=f);return d}},function(t,e,r){var n=r(0),o=r(7).Graph;t.exports=function(t,e,r){var i=function(t){var e;for(;t.hasNode(e=n.uniqueId("_root")););return e}(t),a=new o({compound:!0}).setGraph({root:i}).setDefaultNodeLabel((function(e){return t.node(e)}));return n.forEach(t.nodes(),(function(o){var u=t.node(o),s=t.parent(o);(u.rank===e||u.minRank<=e&&e<=u.maxRank)&&(a.setNode(o),a.setParent(o,s||i),n.forEach(t[r](o),(function(e){var r=e.v===o?e.w:e.v,i=a.edge(r,o),u=n.isUndefined(i)?0:i.weight;a.setEdge(r,o,{weight:t.edge(e).weight+u})})),n.has(u,"minRank")&&a.setNode(o,{borderLeft:u.borderLeft[e],borderRight:u.borderRight[e]}))})),a}},function(t,e,r){var n=r(0);t.exports=function(t,e,r){var o,i={};n.forEach(r,(function(r){for(var n,a,u=t.parent(r);u;){if((n=t.parent(u))?(a=i[n],i[n]=u):(a=o,o=u),a&&a!==u)return void e.setEdge(a,u);u=n}}))}},function(t,e,r){"use strict";var n=r(0),o=r(4),i=r(307).positionX;t.exports=function(t){(function(t){var e=o.buildLayerMatrix(t),r=t.graph().ranksep,i=0;n.forEach(e,(function(e){var o=n.max(n.map(e,(function(e){return t.node(e).height})));n.forEach(e,(function(e){t.node(e).y=i+o/2})),i+=o+r}))})(t=o.asNonCompoundGraph(t)),n.forEach(i(t),(function(e,r){t.node(r).x=e}))}},function(t,e,r){"use strict";var n=r(0),o=r(7).Graph,i=r(4);function a(t,e){var r={};return n.reduce(e,(function(e,o){var i=0,a=0,u=e.length,c=n.last(o);return n.forEach(o,(function(e,f){var h=function(t,e){if(t.node(e).dummy)return n.find(t.predecessors(e),(function(e){return t.node(e).dummy}))}(t,e),l=h?t.node(h).order:u;(h||e===c)&&(n.forEach(o.slice(a,f+1),(function(e){n.forEach(t.predecessors(e),(function(n){var o=t.node(n),a=o.order;!(a<i||l<a)||o.dummy&&t.node(e).dummy||s(r,n,e)}))})),a=f+1,i=l)})),o})),r}function u(t,e){var r={};function o(e,o,i,a,u){var c;n.forEach(n.range(o,i),(function(o){c=e[o],t.node(c).dummy&&n.forEach(t.predecessors(c),(function(e){var n=t.node(e);n.dummy&&(n.order<a||n.order>u)&&s(r,e,c)}))}))}return n.reduce(e,(function(e,r){var i,a=-1,u=0;return n.forEach(r,(function(n,s){if("border"===t.node(n).dummy){var c=t.predecessors(n);c.length&&(i=t.node(c[0]).order,o(r,u,s,a,i),u=s,a=i)}o(r,u,r.length,i,e.length)})),r})),r}function s(t,e,r){if(e>r){var n=e;e=r,r=n}var o=t[e];o||(t[e]=o={}),o[r]=!0}function c(t,e,r){if(e>r){var o=e;e=r,r=o}return n.has(t[e],r)}function f(t,e,r,o){var i={},a={},u={};return n.forEach(e,(function(t){n.forEach(t,(function(t,e){i[t]=t,a[t]=t,u[t]=e}))})),n.forEach(e,(function(t){var e=-1;n.forEach(t,(function(t){var s=o(t);if(s.length)for(var f=((s=n.sortBy(s,(function(t){return u[t]}))).length-1)/2,h=Math.floor(f),l=Math.ceil(f);h<=l;++h){var d=s[h];a[t]===t&&e<u[d]&&!c(r,t,d)&&(a[d]=t,a[t]=i[t]=i[d],e=u[d])}}))})),{root:i,align:a}}function h(t,e,r,i,a){var u={},s=function(t,e,r,i){var a=new o,u=t.graph(),s=function(t,e,r){return function(o,i,a){var u,s=o.node(i),c=o.node(a),f=0;if(f+=s.width/2,n.has(s,"labelpos"))switch(s.labelpos.toLowerCase()){case"l":u=-s.width/2;break;case"r":u=s.width/2}if(u&&(f+=r?u:-u),u=0,f+=(s.dummy?e:t)/2,f+=(c.dummy?e:t)/2,f+=c.width/2,n.has(c,"labelpos"))switch(c.labelpos.toLowerCase()){case"l":u=c.width/2;break;case"r":u=-c.width/2}return u&&(f+=r?u:-u),u=0,f}}(u.nodesep,u.edgesep,i);return n.forEach(e,(function(e){var o;n.forEach(e,(function(e){var n=r[e];if(a.setNode(n),o){var i=r[o],u=a.edge(i,n);a.setEdge(i,n,Math.max(s(t,e,o),u||0))}o=e}))})),a}(t,e,r,a),c=a?"borderLeft":"borderRight";function f(t,e){for(var r=s.nodes(),n=r.pop(),o={};n;)o[n]?t(n):(o[n]=!0,r.push(n),r=r.concat(e(n))),n=r.pop()}return f((function(t){u[t]=s.inEdges(t).reduce((function(t,e){return Math.max(t,u[e.v]+s.edge(e))}),0)}),s.predecessors.bind(s)),f((function(e){var r=s.outEdges(e).reduce((function(t,e){return Math.min(t,u[e.w]-s.edge(e))}),Number.POSITIVE_INFINITY),n=t.node(e);r!==Number.POSITIVE_INFINITY&&n.borderType!==c&&(u[e]=Math.max(u[e],r))}),s.successors.bind(s)),n.forEach(i,(function(t){u[t]=u[r[t]]})),u}function l(t,e){return n.minBy(n.values(e),(function(e){var r=Number.NEGATIVE_INFINITY,o=Number.POSITIVE_INFINITY;return n.forIn(e,(function(e,n){var i=function(t,e){return t.node(e).width}(t,n)/2;r=Math.max(e+i,r),o=Math.min(e-i,o)})),r-o}))}function d(t,e){var r=n.values(e),o=n.min(r),i=n.max(r);n.forEach(["u","d"],(function(r){n.forEach(["l","r"],(function(a){var u,s=r+a,c=t[s];if(c!==e){var f=n.values(c);(u="l"===a?o-n.min(f):i-n.max(f))&&(t[s]=n.mapValues(c,(function(t){return t+u})))}}))}))}function p(t,e){return n.mapValues(t.ul,(function(r,o){if(e)return t[e.toLowerCase()][o];var i=n.sortBy(n.map(t,o));return(i[1]+i[2])/2}))}t.exports={positionX:function(t){var e,r=i.buildLayerMatrix(t),o=n.merge(a(t,r),u(t,r)),s={};n.forEach(["u","d"],(function(i){e="u"===i?r:n.values(r).reverse(),n.forEach(["l","r"],(function(r){"r"===r&&(e=n.map(e,(function(t){return n.values(t).reverse()})));var a=("u"===i?t.predecessors:t.successors).bind(t),u=f(t,e,o,a),c=h(t,e,u.root,u.align,"r"===r);"r"===r&&(c=n.mapValues(c,(function(t){return-t}))),s[i+r]=c}))}));var c=l(t,s);return d(s,c),p(s,t.graph().align)},findType1Conflicts:a,findType2Conflicts:u,addConflict:s,hasConflict:c,verticalAlignment:f,horizontalCompaction:h,alignCoordinates:d,findSmallestWidthAlignment:l,balance:p}},function(t,e,r){var n=r(0),o=r(4),i=r(7).Graph;t.exports={debugOrdering:function(t){var e=o.buildLayerMatrix(t),r=new i({compound:!0,multigraph:!0}).setGraph({});return n.forEach(t.nodes(),(function(e){r.setNode(e,{label:e}),r.setParent(e,"layer"+t.node(e).rank)})),n.forEach(t.edges(),(function(t){r.setEdge(t.v,t.w,{},t.name)})),n.forEach(e,(function(t,e){var o="layer"+e;r.setNode(o,{rank:"same"}),n.reduce(t,(function(t,e){return r.setEdge(t,e,{style:"invis"}),e}))})),r}}},function(t,e){t.exports="0.8.5"},function(t,e,r){var n=r(311),o=r(312),i={};for(var a in n)n.hasOwnProperty(a)&&(i[n[a]]=a);var u=t.exports={to:{},get:{}};function s(t,e,r){return Math.min(Math.max(e,t),r)}function c(t){var e=t.toString(16).toUpperCase();return e.length<2?"0"+e:e}u.get=function(t){var e,r;switch(t.substring(0,3).toLowerCase()){case"hsl":e=u.get.hsl(t),r="hsl";break;case"hwb":e=u.get.hwb(t),r="hwb";break;default:e=u.get.rgb(t),r="rgb"}return e?{model:r,value:e}:null},u.get.rgb=function(t){if(!t)return null;var e,r,o,i=[0,0,0,1];if(e=t.match(/^#([a-f0-9]{6})([a-f0-9]{2})?$/i)){for(o=e[2],e=e[1],r=0;r<3;r++){var a=2*r;i[r]=parseInt(e.slice(a,a+2),16)}o&&(i[3]=parseInt(o,16)/255)}else if(e=t.match(/^#([a-f0-9]{3,4})$/i)){for(o=(e=e[1])[3],r=0;r<3;r++)i[r]=parseInt(e[r]+e[r],16);o&&(i[3]=parseInt(o+o,16)/255)}else if(e=t.match(/^rgba?\\(\\s*([+-]?\\d+)\\s*,\\s*([+-]?\\d+)\\s*,\\s*([+-]?\\d+)\\s*(?:,\\s*([+-]?[\\d\\.]+)\\s*)?\\)$/)){for(r=0;r<3;r++)i[r]=parseInt(e[r+1],0);e[4]&&(i[3]=parseFloat(e[4]))}else{if(!(e=t.match(/^rgba?\\(\\s*([+-]?[\\d\\.]+)\\%\\s*,\\s*([+-]?[\\d\\.]+)\\%\\s*,\\s*([+-]?[\\d\\.]+)\\%\\s*(?:,\\s*([+-]?[\\d\\.]+)\\s*)?\\)$/)))return(e=t.match(/(\\D+)/))?"transparent"===e[1]?[0,0,0,0]:(i=n[e[1]])?(i[3]=1,i):null:null;for(r=0;r<3;r++)i[r]=Math.round(2.55*parseFloat(e[r+1]));e[4]&&(i[3]=parseFloat(e[4]))}for(r=0;r<3;r++)i[r]=s(i[r],0,255);return i[3]=s(i[3],0,1),i},u.get.hsl=function(t){if(!t)return null;var e=t.match(/^hsla?\\(\\s*([+-]?(?:\\d*\\.)?\\d+)(?:deg)?\\s*,\\s*([+-]?[\\d\\.]+)%\\s*,\\s*([+-]?[\\d\\.]+)%\\s*(?:,\\s*([+-]?[\\d\\.]+)\\s*)?\\)$/);if(e){var r=parseFloat(e[4]);return[(parseFloat(e[1])+360)%360,s(parseFloat(e[2]),0,100),s(parseFloat(e[3]),0,100),s(isNaN(r)?1:r,0,1)]}return null},u.get.hwb=function(t){if(!t)return null;var e=t.match(/^hwb\\(\\s*([+-]?\\d*[\\.]?\\d+)(?:deg)?\\s*,\\s*([+-]?[\\d\\.]+)%\\s*,\\s*([+-]?[\\d\\.]+)%\\s*(?:,\\s*([+-]?[\\d\\.]+)\\s*)?\\)$/);if(e){var r=parseFloat(e[4]);return[(parseFloat(e[1])%360+360)%360,s(parseFloat(e[2]),0,100),s(parseFloat(e[3]),0,100),s(isNaN(r)?1:r,0,1)]}return null},u.to.hex=function(){var t=o(arguments);return"#"+c(t[0])+c(t[1])+c(t[2])+(t[3]<1?c(Math.round(255*t[3])):"")},u.to.rgb=function(){var t=o(arguments);return t.length<4||1===t[3]?"rgb("+Math.round(t[0])+", "+Math.round(t[1])+", "+Math.round(t[2])+")":"rgba("+Math.round(t[0])+", "+Math.round(t[1])+", "+Math.round(t[2])+", "+t[3]+")"},u.to.rgb.percent=function(){var t=o(arguments),e=Math.round(t[0]/255*100),r=Math.round(t[1]/255*100),n=Math.round(t[2]/255*100);return t.length<4||1===t[3]?"rgb("+e+"%, "+r+"%, "+n+"%)":"rgba("+e+"%, "+r+"%, "+n+"%, "+t[3]+")"},u.to.hsl=function(){var t=o(arguments);return t.length<4||1===t[3]?"hsl("+t[0]+", "+t[1]+"%, "+t[2]+"%)":"hsla("+t[0]+", "+t[1]+"%, "+t[2]+"%, "+t[3]+")"},u.to.hwb=function(){var t=o(arguments),e="";return t.length>=4&&1!==t[3]&&(e=", "+t[3]),"hwb("+t[0]+", "+t[1]+"%, "+t[2]+"%"+e+")"},u.to.keyword=function(t){return i[t.slice(0,3)]}},function(t,e,r){"use strict";t.exports={aliceblue:[240,248,255],antiquewhite:[250,235,215],aqua:[0,255,255],aquamarine:[127,255,212],azure:[240,255,255],beige:[245,245,220],bisque:[255,228,196],black:[0,0,0],blanchedalmond:[255,235,205],blue:[0,0,255],blueviolet:[138,43,226],brown:[165,42,42],burlywood:[222,184,135],cadetblue:[95,158,160],chartreuse:[127,255,0],chocolate:[210,105,30],coral:[255,127,80],cornflowerblue:[100,149,237],cornsilk:[255,248,220],crimson:[220,20,60],cyan:[0,255,255],darkblue:[0,0,139],darkcyan:[0,139,139],darkgoldenrod:[184,134,11],darkgray:[169,169,169],darkgreen:[0,100,0],darkgrey:[169,169,169],darkkhaki:[189,183,107],darkmagenta:[139,0,139],darkolivegreen:[85,107,47],darkorange:[255,140,0],darkorchid:[153,50,204],darkred:[139,0,0],darksalmon:[233,150,122],darkseagreen:[143,188,143],darkslateblue:[72,61,139],darkslategray:[47,79,79],darkslategrey:[47,79,79],darkturquoise:[0,206,209],darkviolet:[148,0,211],deeppink:[255,20,147],deepskyblue:[0,191,255],dimgray:[105,105,105],dimgrey:[105,105,105],dodgerblue:[30,144,255],firebrick:[178,34,34],floralwhite:[255,250,240],forestgreen:[34,139,34],fuchsia:[255,0,255],gainsboro:[220,220,220],ghostwhite:[248,248,255],gold:[255,215,0],goldenrod:[218,165,32],gray:[128,128,128],green:[0,128,0],greenyellow:[173,255,47],grey:[128,128,128],honeydew:[240,255,240],hotpink:[255,105,180],indianred:[205,92,92],indigo:[75,0,130],ivory:[255,255,240],khaki:[240,230,140],lavender:[230,230,250],lavenderblush:[255,240,245],lawngreen:[124,252,0],lemonchiffon:[255,250,205],lightblue:[173,216,230],lightcoral:[240,128,128],lightcyan:[224,255,255],lightgoldenrodyellow:[250,250,210],lightgray:[211,211,211],lightgreen:[144,238,144],lightgrey:[211,211,211],lightpink:[255,182,193],lightsalmon:[255,160,122],lightseagreen:[32,178,170],lightskyblue:[135,206,250],lightslategray:[119,136,153],lightslategrey:[119,136,153],lightsteelblue:[176,196,222],lightyellow:[255,255,224],lime:[0,255,0],limegreen:[50,205,50],linen:[250,240,230],magenta:[255,0,255],maroon:[128,0,0],mediumaquamarine:[102,205,170],mediumblue:[0,0,205],mediumorchid:[186,85,211],mediumpurple:[147,112,219],mediumseagreen:[60,179,113],mediumslateblue:[123,104,238],mediumspringgreen:[0,250,154],mediumturquoise:[72,209,204],mediumvioletred:[199,21,133],midnightblue:[25,25,112],mintcream:[245,255,250],mistyrose:[255,228,225],moccasin:[255,228,181],navajowhite:[255,222,173],navy:[0,0,128],oldlace:[253,245,230],olive:[128,128,0],olivedrab:[107,142,35],orange:[255,165,0],orangered:[255,69,0],orchid:[218,112,214],palegoldenrod:[238,232,170],palegreen:[152,251,152],paleturquoise:[175,238,238],palevioletred:[219,112,147],papayawhip:[255,239,213],peachpuff:[255,218,185],peru:[205,133,63],pink:[255,192,203],plum:[221,160,221],powderblue:[176,224,230],purple:[128,0,128],rebeccapurple:[102,51,153],red:[255,0,0],rosybrown:[188,143,143],royalblue:[65,105,225],saddlebrown:[139,69,19],salmon:[250,128,114],sandybrown:[244,164,96],seagreen:[46,139,87],seashell:[255,245,238],sienna:[160,82,45],silver:[192,192,192],skyblue:[135,206,235],slateblue:[106,90,205],slategray:[112,128,144],slategrey:[112,128,144],snow:[255,250,250],springgreen:[0,255,127],steelblue:[70,130,180],tan:[210,180,140],teal:[0,128,128],thistle:[216,191,216],tomato:[255,99,71],turquoise:[64,224,208],violet:[238,130,238],wheat:[245,222,179],white:[255,255,255],whitesmoke:[245,245,245],yellow:[255,255,0],yellowgreen:[154,205,50]}},function(t,e,r){"use strict";var n=r(313),o=Array.prototype.concat,i=Array.prototype.slice,a=t.exports=function(t){for(var e=[],r=0,a=t.length;r<a;r++){var u=t[r];n(u)?e=o.call(e,i.call(u)):e.push(u)}return e};a.wrap=function(t){return function(){return t(a(arguments))}}},function(t,e){t.exports=function(t){return!(!t||"string"==typeof t)&&(t instanceof Array||Array.isArray(t)||t.length>=0&&(t.splice instanceof Function||Object.getOwnPropertyDescriptor(t,t.length-1)&&"String"!==t.constructor.name))}},function(t,e,r){function n(t){return(n="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(t){return typeof t}:function(t){return t&&"function"==typeof Symbol&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t})(t)}var o=r(128),i=r(316),a={};Object.keys(o).forEach((function(t){a[t]={},Object.defineProperty(a[t],"channels",{value:o[t].channels}),Object.defineProperty(a[t],"labels",{value:o[t].labels});var e=i(t);Object.keys(e).forEach((function(r){var o=e[r];a[t][r]=function(t){var e=function(e){if(null==e)return e;arguments.length>1&&(e=Array.prototype.slice.call(arguments));var r=t(e);if("object"===n(r))for(var o=r.length,i=0;i<o;i++)r[i]=Math.round(r[i]);return r};return"conversion"in t&&(e.conversion=t.conversion),e}(o),a[t][r].raw=function(t){var e=function(e){return null==e?e:(arguments.length>1&&(e=Array.prototype.slice.call(arguments)),t(e))};return"conversion"in t&&(e.conversion=t.conversion),e}(o)}))})),t.exports=a},function(t,e,r){"use strict";t.exports={aliceblue:[240,248,255],antiquewhite:[250,235,215],aqua:[0,255,255],aquamarine:[127,255,212],azure:[240,255,255],beige:[245,245,220],bisque:[255,228,196],black:[0,0,0],blanchedalmond:[255,235,205],blue:[0,0,255],blueviolet:[138,43,226],brown:[165,42,42],burlywood:[222,184,135],cadetblue:[95,158,160],chartreuse:[127,255,0],chocolate:[210,105,30],coral:[255,127,80],cornflowerblue:[100,149,237],cornsilk:[255,248,220],crimson:[220,20,60],cyan:[0,255,255],darkblue:[0,0,139],darkcyan:[0,139,139],darkgoldenrod:[184,134,11],darkgray:[169,169,169],darkgreen:[0,100,0],darkgrey:[169,169,169],darkkhaki:[189,183,107],darkmagenta:[139,0,139],darkolivegreen:[85,107,47],darkorange:[255,140,0],darkorchid:[153,50,204],darkred:[139,0,0],darksalmon:[233,150,122],darkseagreen:[143,188,143],darkslateblue:[72,61,139],darkslategray:[47,79,79],darkslategrey:[47,79,79],darkturquoise:[0,206,209],darkviolet:[148,0,211],deeppink:[255,20,147],deepskyblue:[0,191,255],dimgray:[105,105,105],dimgrey:[105,105,105],dodgerblue:[30,144,255],firebrick:[178,34,34],floralwhite:[255,250,240],forestgreen:[34,139,34],fuchsia:[255,0,255],gainsboro:[220,220,220],ghostwhite:[248,248,255],gold:[255,215,0],goldenrod:[218,165,32],gray:[128,128,128],green:[0,128,0],greenyellow:[173,255,47],grey:[128,128,128],honeydew:[240,255,240],hotpink:[255,105,180],indianred:[205,92,92],indigo:[75,0,130],ivory:[255,255,240],khaki:[240,230,140],lavender:[230,230,250],lavenderblush:[255,240,245],lawngreen:[124,252,0],lemonchiffon:[255,250,205],lightblue:[173,216,230],lightcoral:[240,128,128],lightcyan:[224,255,255],lightgoldenrodyellow:[250,250,210],lightgray:[211,211,211],lightgreen:[144,238,144],lightgrey:[211,211,211],lightpink:[255,182,193],lightsalmon:[255,160,122],lightseagreen:[32,178,170],lightskyblue:[135,206,250],lightslategray:[119,136,153],lightslategrey:[119,136,153],lightsteelblue:[176,196,222],lightyellow:[255,255,224],lime:[0,255,0],limegreen:[50,205,50],linen:[250,240,230],magenta:[255,0,255],maroon:[128,0,0],mediumaquamarine:[102,205,170],mediumblue:[0,0,205],mediumorchid:[186,85,211],mediumpurple:[147,112,219],mediumseagreen:[60,179,113],mediumslateblue:[123,104,238],mediumspringgreen:[0,250,154],mediumturquoise:[72,209,204],mediumvioletred:[199,21,133],midnightblue:[25,25,112],mintcream:[245,255,250],mistyrose:[255,228,225],moccasin:[255,228,181],navajowhite:[255,222,173],navy:[0,0,128],oldlace:[253,245,230],olive:[128,128,0],olivedrab:[107,142,35],orange:[255,165,0],orangered:[255,69,0],orchid:[218,112,214],palegoldenrod:[238,232,170],palegreen:[152,251,152],paleturquoise:[175,238,238],palevioletred:[219,112,147],papayawhip:[255,239,213],peachpuff:[255,218,185],peru:[205,133,63],pink:[255,192,203],plum:[221,160,221],powderblue:[176,224,230],purple:[128,0,128],rebeccapurple:[102,51,153],red:[255,0,0],rosybrown:[188,143,143],royalblue:[65,105,225],saddlebrown:[139,69,19],salmon:[250,128,114],sandybrown:[244,164,96],seagreen:[46,139,87],seashell:[255,245,238],sienna:[160,82,45],silver:[192,192,192],skyblue:[135,206,235],slateblue:[106,90,205],slategray:[112,128,144],slategrey:[112,128,144],snow:[255,250,250],springgreen:[0,255,127],steelblue:[70,130,180],tan:[210,180,140],teal:[0,128,128],thistle:[216,191,216],tomato:[255,99,71],turquoise:[64,224,208],violet:[238,130,238],wheat:[245,222,179],white:[255,255,255],whitesmoke:[245,245,245],yellow:[255,255,0],yellowgreen:[154,205,50]}},function(t,e,r){var n=r(128);function o(t){var e=function(){for(var t={},e=Object.keys(n),r=e.length,o=0;o<r;o++)t[e[o]]={distance:-1,parent:null};return t}(),r=[t];for(e[t].distance=0;r.length;)for(var o=r.pop(),i=Object.keys(n[o]),a=i.length,u=0;u<a;u++){var s=i[u],c=e[s];-1===c.distance&&(c.distance=e[o].distance+1,c.parent=o,r.unshift(s))}return e}function i(t,e){return function(r){return e(t(r))}}function a(t,e){for(var r=[e[t].parent,t],o=n[e[t].parent][t],a=e[t].parent;e[a].parent;)r.unshift(e[a].parent),o=i(n[e[a].parent][a],o),a=e[a].parent;return o.conversion=r,o}t.exports=function(t){for(var e=o(t),r={},n=Object.keys(e),i=n.length,u=0;u<i;u++){var s=n[u];null!==e[s].parent&&(r[s]=a(s,e))}return r}},function(t,e,r){"use strict";r.r(e);var n=r(53),o=r.n(n);\n/*! *****************************************************************************\nCopyright (c) Microsoft Corporation.\n\nPermission to use, copy, modify, and/or distribute this software for any\npurpose with or without fee is hereby granted.\n\nTHE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH\nREGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY\nAND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,\nINDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM\nLOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR\nOTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR\nPERFORMANCE OF THIS SOFTWARE.\n***************************************************************************** */\nvar i=function(t,e){return(i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(t,e){t.__proto__=e}||function(t,e){for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&(t[r]=e[r])})(t,e)};function a(t,e){function r(){this.constructor=t}i(t,e),t.prototype=null===e?Object.create(e):(r.prototype=e.prototype,new r)}Object.create;Object.create;var u=r(30),s=r.n(u),c=function(){function t(){this.nodes=[],this.edges=[],this.combos=[],this.positions=[],this.destroyed=!1}return t.prototype.init=function(t){this.nodes=t.nodes||[],this.edges=t.edges||[],this.combos=t.combos||[]},t.prototype.execute=function(){},t.prototype.layout=function(t){this.init(t),this.execute()},t.prototype.getDefaultCfg=function(){return{}},t.prototype.updateCfg=function(t){s()(this,t)},t.prototype.destroy=function(){this.positions=null,this.nodes=null,this.edges=null,this.destroyed=!0},t}(),f={registerLayout:function(t,e,r){if(void 0===r&&(r=c),!e)throw new Error("please specify handler for this layout: "+t);var n=function(t){function r(r){var n=t.call(this)||this,i=n,a={},u=i.getDefaultCfg();return s()(a,u,e,r),o()(a,(function(t,e){i[e]=t})),n}return a(r,t),r}(r);f[t]=n}},h=f,l=r(3);l.translate=function(t,e,r){var n=new Array(9);return l.fromTranslation(n,r),l.multiply(t,n,e)},l.rotate=function(t,e,r){var n=new Array(9);return l.fromRotation(n,r),l.multiply(t,n,e)},l.scale=function(t,e,r){var n=new Array(9);return l.fromScaling(n,r),l.multiply(t,n,e)},l.transform=function(t,e){for(var r=[].concat(t),n=0,o=e.length;n<o;n++){var i=e[n];switch(i[0]){case"t":l.translate(r,r,[i[1],i[2]]);break;case"s":l.scale(r,r,[i[1],i[2]]);break;case"r":l.rotate(r,r,i[1])}}return r};var d=r(9),p={}.toString,v=function(t,e){return p.call(t)==="[object "+e+"]"},g=function(t){return v(t,"Function")},y=function(t){return Array.isArray?Array.isArray(t):v(t,"Array")};function m(t){return(m="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(t){return typeof t}:function(t){return t&&"function"==typeof Symbol&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t})(t)}var b=function(t){var e=m(t);return null!==t&&"object"===e||"function"===e};var w=function(t,e){if(t)if(y(t))for(var r=0,n=t.length;r<n&&!1!==e(t[r],r);r++);else if(b(t))for(var o in t)if(t.hasOwnProperty(o)&&!1===e(t[o],o))break};Object.keys;var x=Array.prototype;x.splice,x.indexOf,Array.prototype.splice;Object.prototype.hasOwnProperty;var M=function(t,e,r){return t<e?e:t>r?r:t},k=function(t){return v(t,"Number")};Number.isInteger&&Number.isInteger;Math.PI,parseInt,Math.PI,Object.values;Object.prototype;function _(t){return(_="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(t){return typeof t}:function(t){return t&&"function"==typeof Symbol&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t})(t)}var S=function t(e){if("object"!==_(e)||null===e)return e;var r;if(y(e)){r=[];for(var n=0,o=e.length;n<o;n++)"object"===_(e[n])&&null!=e[n]?r[n]=t(e[n]):r[n]=e[n]}else for(var i in r={},e)"object"===_(e[i])&&null!=e[i]?r[i]=t(e[i]):r[i]=e[i];return r};Object.prototype.hasOwnProperty;Object.prototype.hasOwnProperty;!function(){function t(){this.map={}}t.prototype.has=function(t){return void 0!==this.map[t]},t.prototype.get=function(t,e){var r=this.map[t];return void 0===r?e:r},t.prototype.set=function(t,e){this.map[t]=e},t.prototype.clear=function(){this.map={}},t.prototype.delete=function(t){delete this.map[t]},t.prototype.size=function(){return Object.keys(this.map).length}}();d.angle=function(t,e){var r=d.dot(t,e)/(d.length(t)*d.length(e));return Math.acos(M(r,-1,1))},d.direction=function(t,e){return t[0]*e[1]-e[0]*t[1]},d.angleTo=function(t,e,r){var n=d.angle(t,e),o=d.direction(t,e)>=0;return r?o?2*Math.PI-n:n:o?n:2*Math.PI-n},d.vertical=function(t,e,r){return r?(t[0]=e[1],t[1]=-1*e[0]):(t[0]=-1*e[1],t[1]=e[0]),t};r(129);var E=r(2),j=r.n(E),A=function(t){for(var e=[],r=t.length,n=0;n<r;n+=1){e[n]=[];for(var o=0;o<r;o+=1)n===o?e[n][o]=0:0!==t[n][o]&&t[n][o]?e[n][o]=t[n][o]:e[n][o]=1/0}for(var i=0;i<r;i+=1)for(n=0;n<r;n+=1)for(o=0;o<r;o+=1)e[n][o]>e[n][i]+e[i][o]&&(e[n][o]=e[n][i]+e[i][o]);return e},O=function(t,e){var r=t.nodes,n=t.edges,o=[],i={};if(!r)throw new Error("invalid nodes data!");return r&&r.forEach((function(t,e){i[t.id]=e;o.push([])})),n&&n.forEach((function(t){var r=t.source,n=t.target,a=i[r],u=i[n];o[a][u]=1,e||(o[u][a]=1)})),o},N=function(t,e,r){for(var n=[],o=0;o<t;o++)n[o]=0;return r.forEach((function(t){t.source&&(n[e[t.source]]+=1),t.target&&(n[e[t.target]]+=1)})),n};!function(){function t(t,e,r,n){this.x1=t,this.y1=e,this.x2=r,this.y2=n}t.prototype.getBBox=function(){var t=Math.min(this.x1,this.x2),e=Math.min(this.y1,this.y2),r=Math.max(this.x1,this.x2),n=Math.max(this.y1,this.y2);return{x:t,y:e,minX:t,minY:e,maxX:r,maxY:n,width:r-t,height:n-e}}}();function I(t,e){return t.degree<e.degree?-1:t.degree>e.degree?1:0}var R=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.center=[0,0],e.radius=null,e.startRadius=null,e.endRadius=null,e.startAngle=0,e.endAngle=2*Math.PI,e.clockwise=!0,e.divisions=1,e.ordering=null,e.angleRatio=1,e.nodes=[],e.edges=[],e.nodeMap={},e.degrees=[],e.width=300,e.height=300,e}return a(e,t),e.prototype.getDefaultCfg=function(){return{center:[0,0],radius:null,startRadius:null,endRadius:null,startAngle:0,endAngle:2*Math.PI,clockwise:!0,divisions:1,ordering:null,angleRatio:1}},e.prototype.execute=function(){var t=this.nodes,e=this.edges,r=t.length,n=this.center;if(0!==r){if(1===r)return t[0].x=n[0],void(t[0].y=n[1]);var o=this.radius,i=this.startRadius,a=this.endRadius,u=this.divisions,s=this.startAngle,c=this.endAngle,f=(c-s)/r,h={};t.forEach((function(t,e){h[t.id]=e})),this.nodeMap=h;var l=N(t.length,h,e);this.degrees=l,this.width||"undefined"==typeof window||(this.width=window.innerWidth),this.height||"undefined"==typeof window||(this.height=window.innerHeight),o||i||a?!i&&a?i=a:i&&!a&&(a=i):o=this.height>this.width?this.width/2:this.height/2;var d=f*this.angleRatio;this.astep=d;var p=this.ordering,v=[];v="topology"===p?this.topologyOrdering():"topology-directed"===p?this.topologyOrdering(!0):"degree"===p?this.degreeOrdering():t;for(var g=this.clockwise,y=Math.ceil(r/u),m=0;m<r;++m){var b=o;b||null===i||null===a||(b=i+m*(a-i)/(r-1)),b||(b=10+100*m/(r-1));var w=s+m%y*d+2*Math.PI/u*Math.floor(m/y);g||(w=c-m%y*d-2*Math.PI/u*Math.floor(m/y)),v[m].x=n[0]+Math.cos(w)*b,v[m].y=n[1]+Math.sin(w)*b,v[m].weight=l[m]}}},e.prototype.topologyOrdering=function(t){void 0===t&&(t=!1);var e=this.degrees,r=this.edges,n=this.nodes,o=S(n),i=this.nodeMap,a=[o[0]],u=[n[0]],s=[],c=n.length;s[0]=!0,function(t,e,r,n){t.forEach((function(e,r){t[r].children=[],t[r].parent=[]})),n?e.forEach((function(e){var n=0;e.source&&(n=r[e.source]);var o=0;e.target&&(o=r[e.target]),t[n].children.push(t[o].id),t[o].parent.push(t[n].id)})):e.forEach((function(e){var n=0;e.source&&(n=r[e.source]);var o=0;e.target&&(o=r[e.target]),t[n].children.push(t[o].id),t[o].children.push(t[n].id)}))}(o,r,i,t);var f=0;return o.forEach((function(t,h){if(0!==h)if(h!==c-1&&e[h]===e[h+1]&&!function(t,e,r){for(var n=r.length,o=0;o<n;o++)if(t.id===r[o].source&&e.id===r[o].target||e.id===r[o].source&&t.id===r[o].target)return!0;return!1}(a[f],t,r)||!0===s[h]){for(var l=a[f].children,d=!1,p=0;p<l.length;p++){var v=i[l[p]];if(e[v]===e[h]&&!0!==s[v]){a.push(o[v]),u.push(n[i[o[v].id]]),s[v]=!0,d=!0;break}}for(var g=0;!d&&(s[g]||(a.push(o[g]),u.push(n[i[o[g].id]]),s[g]=!0,d=!0),++g!==c););}else a.push(t),u.push(n[i[t.id]]),s[h]=!0,f++})),u},e.prototype.degreeOrdering=function(){var t=this.nodes,e=[],r=this.degrees;return t.forEach((function(t,n){t.degree=r[n],e.push(t)})),e.sort(I),e},e}(c),P=r(11),C=r.n(P),z=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.center=[0,0],e.nodeSize=30,e.minNodeSpacing=10,e.preventOverlap=!1,e.equidistant=!1,e.startAngle=1.5*Math.PI,e.clockwise=!0,e.sortBy="degree",e.nodes=[],e.edges=[],e.width=300,e.height=300,e}return a(e,t),e.prototype.getDefaultCfg=function(){return{center:[0,0],nodeSize:30,minNodeSpacing:10,preventOverlap:!1,sweep:void 0,equidistant:!1,startAngle:1.5*Math.PI,clockwise:!0,maxLevelDiff:void 0,sortBy:"degree"}},e.prototype.execute=function(){var t=this,e=t.nodes,r=t.edges,n=e.length,o=t.center;if(0!==n){if(1===n)return e[0].x=o[0],void(e[0].y=o[1]);var i,a=[];i=j()(t.nodeSize)?Math.max(t.nodeSize[0],t.nodeSize[1]):t.nodeSize,e.forEach((function(t){a.push(t);var e=i;j()(t.size)?e=Math.max(t.size[0],t.size[1]):k(t.size)&&(e=t.size),i=Math.max(i,e)})),t.width||"undefined"==typeof window||(t.width=window.innerWidth),t.height||"undefined"==typeof window||(t.height=window.innerHeight),t.clockwise=void 0!==t.counterclockwise?!t.counterclockwise:t.clockwise;var u={},s={};if(a.forEach((function(t,e){u[t.id]=t,s[t.id]=e})),!("degree"!==t.sortBy&&C()(t.sortBy)&&void 0!==a[0][t.sortBy]||(t.sortBy="degree",k(e[0].degree)))){var c=N(e.length,s,r);a.forEach((function(t,e){t.degree=c[e]}))}a.sort((function(e,r){return r[t.sortBy]-e[t.sortBy]})),t.maxValueNode=a[0],t.maxLevelDiff=t.maxLevelDiff||t.maxValueNode[t.sortBy]/4;var f=[[]],h=f[0];a.forEach((function(e){if(h.length>0){var r=Math.abs(h[0][t.sortBy]-e[t.sortBy]);t.maxLevelDiff&&r>=t.maxLevelDiff&&(h=[],f.push(h))}h.push(e)}));var l=i+t.minNodeSpacing;if(!t.preventOverlap){var d=f.length>0&&f[0].length>1,p=(Math.min(t.width,t.height)/2-l)/(f.length+(d?1:0));l=Math.min(l,p)}var v=0;if(f.forEach((function(e){var r=t.sweep;void 0===r&&(r=2*Math.PI-2*Math.PI/e.length);var n=e.dTheta=r/Math.max(1,e.length-1);if(e.length>1&&t.preventOverlap){var o=Math.cos(n)-Math.cos(0),i=Math.sin(n)-Math.sin(0),a=Math.sqrt(l*l/(o*o+i*i));v=Math.max(a,v)}e.r=v,v+=l})),t.equidistant){for(var g=0,y=0,m=0;m<f.length;m++){var b=f[m].r-y;g=Math.max(g,b)}y=0,f.forEach((function(t,e){0===e&&(y=t.r),t.r=y,y+=g}))}f.forEach((function(e){var r=e.dTheta,n=e.r;e.forEach((function(e,i){var a=t.startAngle+(t.clockwise?1:-1)*r*i;e.x=o[0]+n*Math.cos(a),e.y=o[1]+n*Math.sin(a)}))}))}},e}(c),F=r(74),T=r.n(F),D=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.rankdir="TB",e.nodesep=50,e.ranksep=50,e.controlPoints=!1,e.sortByCombo=!1,e}return a(e,t),e.prototype.getDefaultCfg=function(){return{rankdir:"TB",align:void 0,nodeSize:void 0,nodesepFunc:void 0,ranksepFunc:void 0,nodesep:50,ranksep:50,controlPoints:!1}},e.prototype.execute=function(){var t=this,e=t.nodes,r=t.nodeSize,n=t.rankdir;if(e){var o,i=t.edges||[],a=new T.a.graphlib.Graph;o=r?j()(r)?function(){return r}:function(){return[r,r]}:function(t){return t.size?j()(t.size)?t.size:[t.size,t.size]:[40,40]};var u,s=q(t.nodesepFunc,t.nodesep,50),c=q(t.ranksepFunc,t.ranksep,50);"LR"!==n&&"RL"!==n||(s=q(t.ranksepFunc,t.ranksep,50),c=q(t.nodesepFunc,t.nodesep,50)),a.setDefaultEdgeLabel((function(){return{}})),a.setGraph(t),e.forEach((function(t){var e=o(t),r=c(t),n=s(t),i=e[0]+2*n,u=e[1]+2*r;a.setNode(t.id,{width:i,height:u})})),i.forEach((function(t){a.setEdge(t.source,t.target,{weight:t.weight||1})})),T.a.layout(a),a.nodes().forEach((function(t){u=a.node(t);var r=e.findIndex((function(e){return e.id===t}));e[r].x=u.x,e[r].y=u.y})),a.edges().forEach((function(e){u=a.edge(e);var r=i.findIndex((function(t){return t.source===e.v&&t.target===e.w}));t.controlPoints&&"loop"!==i[r].type&&"loop"!==i[r].shape&&(i[r].controlPoints=u.points.slice(1,u.points.length-1))})),t.sortByCombo&&t.sortLevel("comboId")}},e.prototype.sortLevel=function(t){var e=this.nodes,r={};e.forEach((function(t){r[t.y]||(r[t.y]={y:t.y,nodes:[]}),r[t.y].nodes.push(t)})),Object.keys(r).forEach((function(e){var n=r[e].nodes,o=n.length,i={};if(n.forEach((function(t){var e=t.comboId;i[e]||(i[e]={x:0,y:0,count:0}),i[e].x+=t.x,i[e].y+=t.y,i[e].count++})),Object.keys(i).forEach((function(t){i[t].x/=i[t].count,i[t].y/=i[t].count})),1!==o){var a=n.sort((function(t,e){return t.x-e.x})),u=a[0].x,s=(a[o-1].x-u)/(o-1);n.sort((function(e,r){var n=e[t]||"undefined",o=r[t]||"undefined";return n<o?-1:n>o?1:0})).forEach((function(t,e){t.x=u+e*s}))}}))},e}(c);function q(t,e,r){return t||(k(e)?function(){return e}:function(){return r})}function L(t,e,r,n){if(isNaN(e)||isNaN(r))return t;var o,i,a,u,s,c,f,h,l,d=t._root,p={data:n},v=t._x0,g=t._y0,y=t._x1,m=t._y1;if(!d)return t._root=p,t;for(;d.length;)if((c=e>=(i=(v+y)/2))?v=i:y=i,(f=r>=(a=(g+m)/2))?g=a:m=a,o=d,!(d=d[h=f<<1|c]))return o[h]=p,t;if(u=+t._x.call(null,d.data),s=+t._y.call(null,d.data),e===u&&r===s)return p.next=d,o?o[h]=p:t._root=p,t;do{o=o?o[h]=new Array(4):t._root=new Array(4),(c=e>=(i=(v+y)/2))?v=i:y=i,(f=r>=(a=(g+m)/2))?g=a:m=a}while((h=f<<1|c)==(l=(s>=a)<<1|u>=i));return o[l]=d,o[h]=p,t}var B=function(t,e,r,n,o){this.node=t,this.x0=e,this.y0=r,this.x1=n,this.y1=o};function V(t){return t[0]}function U(t){return t[1]}function Y(t,e,r){var n=new H(null==e?V:e,null==r?U:r,NaN,NaN,NaN,NaN);return null==t?n:n.addAll(t)}function H(t,e,r,n,o,i){this._x=t,this._y=e,this._x0=r,this._y0=n,this._x1=o,this._y1=i,this._root=void 0}function W(t){for(var e={data:t.data},r=e;t=t.next;)r=r.next={data:t.data};return e}var G=Y.prototype=H.prototype;G.copy=function(){var t,e,r=new H(this._x,this._y,this._x0,this._y0,this._x1,this._y1),n=this._root;if(!n)return r;if(!n.length)return r._root=W(n),r;for(t=[{source:n,target:r._root=new Array(4)}];n=t.pop();)for(var o=0;o<4;++o)(e=n.source[o])&&(e.length?t.push({source:e,target:n.target[o]=new Array(4)}):n.target[o]=W(e));return r},G.add=function(t){var e=+this._x.call(null,t),r=+this._y.call(null,t);return L(this.cover(e,r),e,r,t)},G.addAll=function(t){var e,r,n,o,i=t.length,a=new Array(i),u=new Array(i),s=1/0,c=1/0,f=-1/0,h=-1/0;for(r=0;r<i;++r)isNaN(n=+this._x.call(null,e=t[r]))||isNaN(o=+this._y.call(null,e))||(a[r]=n,u[r]=o,n<s&&(s=n),n>f&&(f=n),o<c&&(c=o),o>h&&(h=o));if(s>f||c>h)return this;for(this.cover(s,c).cover(f,h),r=0;r<i;++r)L(this,a[r],u[r],t[r]);return this},G.cover=function(t,e){if(isNaN(t=+t)||isNaN(e=+e))return this;var r=this._x0,n=this._y0,o=this._x1,i=this._y1;if(isNaN(r))o=(r=Math.floor(t))+1,i=(n=Math.floor(e))+1;else{for(var a,u,s=o-r||1,c=this._root;r>t||t>=o||n>e||e>=i;)switch(u=(e<n)<<1|t<r,(a=new Array(4))[u]=c,c=a,s*=2,u){case 0:o=r+s,i=n+s;break;case 1:r=o-s,i=n+s;break;case 2:o=r+s,n=i-s;break;case 3:r=o-s,n=i-s}this._root&&this._root.length&&(this._root=c)}return this._x0=r,this._y0=n,this._x1=o,this._y1=i,this},G.data=function(){var t=[];return this.visit((function(e){if(!e.length)do{t.push(e.data)}while(e=e.next)})),t},G.extent=function(t){return arguments.length?this.cover(+t[0][0],+t[0][1]).cover(+t[1][0],+t[1][1]):isNaN(this._x0)?void 0:[[this._x0,this._y0],[this._x1,this._y1]]},G.find=function(t,e,r){var n,o,i,a,u,s,c,f=this._x0,h=this._y0,l=this._x1,d=this._y1,p=[],v=this._root;for(v&&p.push(new B(v,f,h,l,d)),null==r?r=1/0:(f=t-r,h=e-r,l=t+r,d=e+r,r*=r);s=p.pop();)if(!(!(v=s.node)||(o=s.x0)>l||(i=s.y0)>d||(a=s.x1)<f||(u=s.y1)<h))if(v.length){var g=(o+a)/2,y=(i+u)/2;p.push(new B(v[3],g,y,a,u),new B(v[2],o,y,g,u),new B(v[1],g,i,a,y),new B(v[0],o,i,g,y)),(c=(e>=y)<<1|t>=g)&&(s=p[p.length-1],p[p.length-1]=p[p.length-1-c],p[p.length-1-c]=s)}else{var m=t-+this._x.call(null,v.data),b=e-+this._y.call(null,v.data),w=m*m+b*b;if(w<r){var x=Math.sqrt(r=w);f=t-x,h=e-x,l=t+x,d=e+x,n=v.data}}return n},G.remove=function(t){if(isNaN(i=+this._x.call(null,t))||isNaN(a=+this._y.call(null,t)))return this;var e,r,n,o,i,a,u,s,c,f,h,l,d=this._root,p=this._x0,v=this._y0,g=this._x1,y=this._y1;if(!d)return this;if(d.length)for(;;){if((c=i>=(u=(p+g)/2))?p=u:g=u,(f=a>=(s=(v+y)/2))?v=s:y=s,e=d,!(d=d[h=f<<1|c]))return this;if(!d.length)break;(e[h+1&3]||e[h+2&3]||e[h+3&3])&&(r=e,l=h)}for(;d.data!==t;)if(n=d,!(d=d.next))return this;return(o=d.next)&&delete d.next,n?(o?n.next=o:delete n.next,this):e?(o?e[h]=o:delete e[h],(d=e[0]||e[1]||e[2]||e[3])&&d===(e[3]||e[2]||e[1]||e[0])&&!d.length&&(r?r[l]=d:this._root=d),this):(this._root=o,this)},G.removeAll=function(t){for(var e=0,r=t.length;e<r;++e)this.remove(t[e]);return this},G.root=function(){return this._root},G.size=function(){var t=0;return this.visit((function(e){if(!e.length)do{++t}while(e=e.next)})),t},G.visit=function(t){var e,r,n,o,i,a,u=[],s=this._root;for(s&&u.push(new B(s,this._x0,this._y0,this._x1,this._y1));e=u.pop();)if(!t(s=e.node,n=e.x0,o=e.y0,i=e.x1,a=e.y1)&&s.length){var c=(n+i)/2,f=(o+a)/2;(r=s[3])&&u.push(new B(r,c,f,i,a)),(r=s[2])&&u.push(new B(r,n,f,c,a)),(r=s[1])&&u.push(new B(r,c,o,i,f)),(r=s[0])&&u.push(new B(r,n,o,c,f))}return this},G.visitAfter=function(t){var e,r=[],n=[];for(this._root&&r.push(new B(this._root,this._x0,this._y0,this._x1,this._y1));e=r.pop();){var o=e.node;if(o.length){var i,a=e.x0,u=e.y0,s=e.x1,c=e.y1,f=(a+s)/2,h=(u+c)/2;(i=o[0])&&r.push(new B(i,a,u,f,h)),(i=o[1])&&r.push(new B(i,f,u,s,h)),(i=o[2])&&r.push(new B(i,a,h,f,c)),(i=o[3])&&r.push(new B(i,f,h,s,c))}n.push(e)}for(;e=n.pop();)t(e.node,e.x0,e.y0,e.x1,e.y1);return this},G.x=function(t){return arguments.length?(this._x=t,this):this._x},G.y=function(t){return arguments.length?(this._y=t,this):this._y};var $=function(t){return function(){return t}},X=function(t){return 1e-6*(t()-.5)},J={value:function(){}};function Q(){for(var t,e=0,r=arguments.length,n={};e<r;++e){if(!(t=arguments[e]+"")||t in n||/[\\s.]/.test(t))throw new Error("illegal type: "+t);n[t]=[]}return new K(n)}function K(t){this._=t}function Z(t,e){return t.trim().split(/^|\\s+/).map((function(t){var r="",n=t.indexOf(".");if(n>=0&&(r=t.slice(n+1),t=t.slice(0,n)),t&&!e.hasOwnProperty(t))throw new Error("unknown type: "+t);return{type:t,name:r}}))}function tt(t,e){for(var r,n=0,o=t.length;n<o;++n)if((r=t[n]).name===e)return r.value}function et(t,e,r){for(var n=0,o=t.length;n<o;++n)if(t[n].name===e){t[n]=J,t=t.slice(0,n).concat(t.slice(n+1));break}return null!=r&&t.push({name:e,value:r}),t}K.prototype=Q.prototype={constructor:K,on:function(t,e){var r,n=this._,o=Z(t+"",n),i=-1,a=o.length;if(!(arguments.length<2)){if(null!=e&&"function"!=typeof e)throw new Error("invalid callback: "+e);for(;++i<a;)if(r=(t=o[i]).type)n[r]=et(n[r],t.name,e);else if(null==e)for(r in n)n[r]=et(n[r],t.name,null);return this}for(;++i<a;)if((r=(t=o[i]).type)&&(r=tt(n[r],t.name)))return r},copy:function(){var t={},e=this._;for(var r in e)t[r]=e[r].slice();return new K(t)},call:function(t,e){if((r=arguments.length-2)>0)for(var r,n,o=new Array(r),i=0;i<r;++i)o[i]=arguments[i+2];if(!this._.hasOwnProperty(t))throw new Error("unknown type: "+t);for(i=0,r=(n=this._[t]).length;i<r;++i)n[i].value.apply(e,o)},apply:function(t,e,r){if(!this._.hasOwnProperty(t))throw new Error("unknown type: "+t);for(var n=this._[t],o=0,i=n.length;o<i;++o)n[o].value.apply(e,r)}};var rt=Q;function nt(t){return(nt="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(t){return typeof t}:function(t){return t&&"function"==typeof Symbol&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t})(t)}var ot,it,at=0,ut=0,st=0,ct=0,ft=0,ht=0,lt="object"===("undefined"==typeof performance?"undefined":nt(performance))&&performance.now?performance:Date,dt="object"===("undefined"==typeof window?"undefined":nt(window))&&window.requestAnimationFrame?window.requestAnimationFrame.bind(window):function(t){setTimeout(t,17)};function pt(){return ft||(dt(vt),ft=lt.now()+ht)}function vt(){ft=0}function gt(){this._call=this._time=this._next=null}function yt(t,e,r){var n=new gt;return n.restart(t,e,r),n}function mt(){ft=(ct=lt.now())+ht,at=ut=0;try{!function(){pt(),++at;for(var t,e=ot;e;)(t=ft-e._time)>=0&&e._call.call(null,t),e=e._next;--at}()}finally{at=0,function(){var t,e,r=ot,n=1/0;for(;r;)r._call?(n>r._time&&(n=r._time),t=r,r=r._next):(e=r._next,r._next=null,r=t?t._next=e:ot=e);it=t,wt(n)}(),ft=0}}function bt(){var t=lt.now(),e=t-ct;e>1e3&&(ht-=e,ct=t)}function wt(t){at||(ut&&(ut=clearTimeout(ut)),t-ft>24?(t<1/0&&(ut=setTimeout(mt,t-lt.now()-ht)),st&&(st=clearInterval(st))):(st||(ct=lt.now(),st=setInterval(bt,1e3)),at=1,dt(mt)))}gt.prototype=yt.prototype={constructor:gt,restart:function(t,e,r){if("function"!=typeof t)throw new TypeError("callback is not a function");r=(null==r?pt():+r)+(null==e?0:+e),this._next||it===this||(it?it._next=this:ot=this,it=this),this._call=t,this._time=r,wt()},stop:function(){this._call&&(this._call=null,this._time=1/0,wt())}};function xt(t){return t.x}function Mt(t){return t.y}var kt=Math.PI*(3-Math.sqrt(5)),_t=function(t){var e,r,n=1,o=.001,i=1-Math.pow(o,1/300),a=0,u=.6,s=new Map,c=yt(l),f=rt("tick","end"),h=(r=1,function(){return(r=(1664525*r+1013904223)%4294967296)/4294967296});function l(){d(),f.call("tick",e),n<o&&(c.stop(),f.call("end",e))}function d(r){var o,c,f=t.length;void 0===r&&(r=1);for(var h=0;h<r;++h)for(n+=(a-n)*i,s.forEach((function(t){t(n)})),o=0;o<f;++o)null==(c=t[o]).fx?c.x+=c.vx*=u:(c.x=c.fx,c.vx=0),null==c.fy?c.y+=c.vy*=u:(c.y=c.fy,c.vy=0);return e}function p(){for(var e,r=0,n=t.length;r<n;++r){if((e=t[r]).index=r,null!=e.fx&&(e.x=e.fx),null!=e.fy&&(e.y=e.fy),isNaN(e.x)||isNaN(e.y)){var o=10*Math.sqrt(.5+r),i=r*kt;e.x=o*Math.cos(i),e.y=o*Math.sin(i)}(isNaN(e.vx)||isNaN(e.vy))&&(e.vx=e.vy=0)}}function v(e){return e.initialize&&e.initialize(t,h),e}return null==t&&(t=[]),p(),e={tick:d,restart:function(){return c.restart(l),e},stop:function(){return c.stop(),e},nodes:function(r){return arguments.length?(t=r,p(),s.forEach(v),e):t},alpha:function(t){return arguments.length?(n=+t,e):n},alphaMin:function(t){return arguments.length?(o=+t,e):o},alphaDecay:function(t){return arguments.length?(i=+t,e):+i},alphaTarget:function(t){return arguments.length?(a=+t,e):a},velocityDecay:function(t){return arguments.length?(u=1-t,e):1-u},randomSource:function(t){return arguments.length?(h=t,s.forEach(v),e):h},force:function(t,r){return arguments.length>1?(null==r?s.delete(t):s.set(t,v(r)),e):s.get(t)},find:function(e,r,n){var o,i,a,u,s,c=0,f=t.length;for(null==n?n=1/0:n*=n,c=0;c<f;++c)(a=(o=e-(u=t[c]).x)*o+(i=r-u.y)*i)<n&&(s=u,n=a);return s},on:function(t,r){return arguments.length>1?(f.on(t,r),e):f.on(t)}}},St=function(){var t,e,r,n,o,i=$(-30),a=1,u=1/0,s=.81;function c(r){var o,i=t.length,a=Y(t,xt,Mt).visitAfter(h);for(n=r,o=0;o<i;++o)e=t[o],a.visit(l)}function f(){if(t){var e,r,n=t.length;for(o=new Array(n),e=0;e<n;++e)r=t[e],o[r.index]=+i(r,e,t)}}function h(t){var e,r,n,i,a,u=0,s=0;if(t.length){for(n=i=a=0;a<4;++a)(e=t[a])&&(r=Math.abs(e.value))&&(u+=e.value,s+=r,n+=r*e.x,i+=r*e.y);t.x=n/s,t.y=i/s}else{(e=t).x=e.data.x,e.y=e.data.y;do{u+=o[e.data.index]}while(e=e.next)}t.value=u}function l(t,i,c,f){if(!t.value)return!0;var h=t.x-e.x,l=t.y-e.y,d=f-i,p=h*h+l*l;if(d*d/s<p)return p<u&&(0===h&&(p+=(h=X(r))*h),0===l&&(p+=(l=X(r))*l),p<a&&(p=Math.sqrt(a*p)),e.vx+=h*t.value*n/p,e.vy+=l*t.value*n/p),!0;if(!(t.length||p>=u)){(t.data!==e||t.next)&&(0===h&&(p+=(h=X(r))*h),0===l&&(p+=(l=X(r))*l),p<a&&(p=Math.sqrt(a*p)));do{t.data!==e&&(d=o[t.data.index]*n/p,e.vx+=h*d,e.vy+=l*d)}while(t=t.next)}}return c.initialize=function(e,n){t=e,r=n,f()},c.strength=function(t){return arguments.length?(i="function"==typeof t?t:$(+t),f(),c):i},c.distanceMin=function(t){return arguments.length?(a=t*t,c):Math.sqrt(a)},c.distanceMax=function(t){return arguments.length?(u=t*t,c):Math.sqrt(u)},c.theta=function(t){return arguments.length?(s=t*t,c):Math.sqrt(s)},c};function Et(t){return(Et="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(t){return typeof t}:function(t){return t&&"function"==typeof Symbol&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t})(t)}function jt(t){return t.index}function At(t,e){var r=t.get(e);if(!r)throw new Error("node not found: "+e);return r}var Ot=function(t){var e,r,n,o,i,a,u=jt,s=function(t){return 1/Math.min(o[t.source.index],o[t.target.index])},c=$(30),f=1;function h(n){for(var o=0,u=t.length;o<f;++o)for(var s,c,h,l,d,p,v,g=0;g<u;++g)c=(s=t[g]).source,l=(h=s.target).x+h.vx-c.x-c.vx||X(a),d=h.y+h.vy-c.y-c.vy||X(a),l*=p=((p=Math.sqrt(l*l+d*d))-r[g])/p*n*e[g],d*=p,h.vx-=l*(v=i[g]),h.vy-=d*v,c.vx+=l*(v=1-v),c.vy+=d*v}function l(){if(n){var a,s,c=n.length,f=t.length,h=new Map(n.map((function(t,e){return[u(t,e,n),t]})));for(a=0,o=new Array(c);a<f;++a)(s=t[a]).index=a,"object"!==Et(s.source)&&(s.source=At(h,s.source)),"object"!==Et(s.target)&&(s.target=At(h,s.target)),o[s.source.index]=(o[s.source.index]||0)+1,o[s.target.index]=(o[s.target.index]||0)+1;for(a=0,i=new Array(f);a<f;++a)s=t[a],i[a]=o[s.source.index]/(o[s.source.index]+o[s.target.index]);e=new Array(f),d(),r=new Array(f),p()}}function d(){if(n)for(var r=0,o=t.length;r<o;++r)e[r]=+s(t[r],r,t)}function p(){if(n)for(var e=0,o=t.length;e<o;++e)r[e]=+c(t[e],e,t)}return null==t&&(t=[]),h.initialize=function(t,e){n=t,a=e,l()},h.links=function(e){return arguments.length?(t=e,l(),h):t},h.id=function(t){return arguments.length?(u=t,h):u},h.iterations=function(t){return arguments.length?(f=+t,h):f},h.strength=function(t){return arguments.length?(s="function"==typeof t?t:$(+t),d(),h):s},h.distance=function(t){return arguments.length?(c="function"==typeof t?t:$(+t),p(),h):c},h};function Nt(t){return t.x+t.vx}function It(t){return t.y+t.vy}var Rt=function(t){var e,r,n,o=1,i=1;function a(){for(var t,a,s,c,f,h,l,d=e.length,p=0;p<i;++p)for(a=Y(e,Nt,It).visitAfter(u),t=0;t<d;++t)s=e[t],h=r[s.index],l=h*h,c=s.x+s.vx,f=s.y+s.vy,a.visit(v);function v(t,e,r,i,a){var u=t.data,d=t.r,p=h+d;if(!u)return e>c+p||i<c-p||r>f+p||a<f-p;if(u.index>s.index){var v=c-u.x-u.vx,g=f-u.y-u.vy,y=v*v+g*g;y<p*p&&(0===v&&(y+=(v=X(n))*v),0===g&&(y+=(g=X(n))*g),y=(p-(y=Math.sqrt(y)))/y*o,s.vx+=(v*=y)*(p=(d*=d)/(l+d)),s.vy+=(g*=y)*p,u.vx-=v*(p=1-p),u.vy-=g*p)}}}function u(t){if(t.data)return t.r=r[t.data.index];for(var e=t.r=0;e<4;++e)t[e]&&t[e].r>t.r&&(t.r=t[e].r)}function s(){if(e){var n,o,i=e.length;for(r=new Array(i),n=0;n<i;++n)o=e[n],r[o.index]=+t(o,n,e)}}return"function"!=typeof t&&(t=$(null==t?1:+t)),a.initialize=function(t,r){e=t,n=r,s()},a.iterations=function(t){return arguments.length?(i=+t,a):i},a.strength=function(t){return arguments.length?(o=+t,a):o},a.radius=function(e){return arguments.length?(t="function"==typeof e?e:$(+e),s(),a):t},a};function Pt(){function t(t){return function(){return t}}var e,r=function(t){return t.cluster},n=t(1),o=t(-1),i=t(100),a=t(.1),u=[0,0],s=[],c={},f=[],h=100,l=100,d={none:{x:0,y:0}},p=[],v="force",g=!0,y=.1;function m(t){if(!g)return m;e.tick(),w();for(var n=0,o=s.length,i=void 0,a=t*y;n<o;++n)(i=s[n]).vx+=(d[r(i)].x-i.x)*a,i.vy+=(d[r(i)].y-i.y)*a}function b(){s&&function(){if(!s||!s.length)return;if(void 0===r(s[0]))throw Error("Couldnt find the grouping attribute for the nodes. Make sure to set it up with forceInABox.groupBy(\'clusterAttr\') before calling .links()");var t=(d=[],v=[],g={},y=[],u=function(t){var e={};return t.forEach((function(t){var n=r(t);e[n]||(e[n]={count:0,sumforceNodeSize:0})})),t.forEach((function(t){var o=r(t),i=n(t),a=e[o];a.count=a.count+1,a.sumforceNodeSize=a.sumforceNodeSize+Math.PI*(i*i)*1.3,e[o]=a})),e}(s),y=function(t){var e={},n=[];return t.forEach((function(t){var n=function(t){var e=r(c[t.source]),n=r(c[t.target]);return e<=n?e+"~"+n:n+"~"+e}(t),o=0;void 0!==e[n]&&(o=e[n]),o+=1,e[n]=o})),Object.entries(e).forEach((function(t){var e=t[0],r=t[1],o=e.split("~")[0],i=e.split("~")[1];void 0!==o&&void 0!==i&&n.push({source:o,target:i,count:r})})),n}(f),Object.keys(u).forEach((function(t,e){var r=u[t];d.push({id:t,size:r.count,r:Math.sqrt(r.sumforceNodeSize/Math.PI)}),g[t]=e})),y.forEach((function(t){var e=g[t.source],r=g[t.target];void 0!==e&&void 0!==r&&v.push({source:e,target:r,count:t.count})})),{nodes:d,links:v});var u,d,v,g,y;e=_t(t.nodes).force("x",function(t){var e,r,n,o=$(.1);function i(t){for(var o,i=0,a=e.length;i<a;++i)(o=e[i]).vx+=(n[i]-o.x)*r[i]*t}function a(){if(e){var i,a=e.length;for(r=new Array(a),n=new Array(a),i=0;i<a;++i)r[i]=isNaN(n[i]=+t(e[i],i,e))?0:+o(e[i],i,e)}}return"function"!=typeof t&&(t=$(null==t?0:+t)),i.initialize=function(t){e=t,a()},i.strength=function(t){return arguments.length?(o="function"==typeof t?t:$(+t),a(),i):o},i.x=function(e){return arguments.length?(t="function"==typeof e?e:$(+e),a(),i):t},i}(h).strength(.1)).force("y",function(t){var e,r,n,o=$(.1);function i(t){for(var o,i=0,a=e.length;i<a;++i)(o=e[i]).vy+=(n[i]-o.y)*r[i]*t}function a(){if(e){var i,a=e.length;for(r=new Array(a),n=new Array(a),i=0;i<a;++i)r[i]=isNaN(n[i]=+t(e[i],i,e))?0:+o(e[i],i,e)}}return"function"!=typeof t&&(t=$(null==t?0:+t)),i.initialize=function(t){e=t,a()},i.strength=function(t){return arguments.length?(o="function"==typeof t?t:$(+t),a(),i):o},i.y=function(e){return arguments.length?(t="function"==typeof e?e:$(+e),a(),i):t},i}(l).strength(.1)).force("collide",Rt((function(t){return t.r})).iterations(4)).force("charge",St().strength(o)).force("links",Ot(t.nodes.length?t.links:[]).distance(i).strength(a)),p=e.nodes(),w()}()}function w(){return d={none:{x:0,y:0}},p.forEach((function(t){d[t.id]={x:t.x-u[0],y:t.y-u[1]}})),d}function x(t){c={},t.forEach((function(t){c[t.id]=t}))}return m.initialize=function(t){s=t,b()},m.template=function(t){return arguments.length?(v=t,b(),m):v},m.groupBy=function(t){return arguments.length?"string"==typeof t?(r=function(e){return e[t]},m):(r=t,m):r},m.enableGrouping=function(t){return arguments.length?(g=t,m):g},m.strength=function(t){return arguments.length?(y=t,m):y},m.centerX=function(t){return arguments.length?(h=t,m):h},m.centerY=function(t){return arguments.length?(l=t,m):l},m.nodes=function(t){return arguments.length?(x(t||[]),s=t||[],m):s},m.links=function(t){return arguments.length?(f=t||[],b(),m):f},m.forceNodeSize=function(e){return arguments.length?(n="function"==typeof e?e:t(+e),b(),m):n},m.nodeSize=m.forceNodeSize,m.forceCharge=function(e){return arguments.length?(o="function"==typeof e?e:t(+e),b(),m):o},m.forceLinkDistance=function(e){return arguments.length?(i="function"==typeof e?e:t(+e),b(),m):i},m.forceLinkStrength=function(e){return arguments.length?(a="function"==typeof e?e:t(+e),b(),m):a},m.offset=function(t){return arguments.length?(u=t,m):u},m.getFocis=w,m}var Ct=r(31),zt=r.n(Ct),Ft=r(19),Tt=r.n(Ft),Dt="LAYOUT_RUN",qt="LAYOUT_END",Lt="LAYOUT_ERROR",Bt="LAYOUT_TICK",Vt=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.center=[0,0],e.nodeStrength=null,e.edgeStrength=null,e.preventOverlap=!1,e.clusterNodeStrength=null,e.clusterEdgeStrength=null,e.clusterEdgeDistance=null,e.clusterNodeSize=null,e.clusterFociStrength=null,e.linkDistance=50,e.alphaDecay=.028,e.alphaMin=.001,e.alpha=.3,e.collideStrength=1,e.workerEnabled=!1,e.tick=function(){},e.onLayoutEnd=function(){},e.onTick=function(){},e.ticking=void 0,e}return a(e,t),e.prototype.getDefaultCfg=function(){return{center:[0,0],nodeStrength:null,edgeStrength:null,preventOverlap:!1,nodeSize:void 0,nodeSpacing:void 0,linkDistance:50,forceSimulation:null,alphaDecay:.028,alphaMin:.001,alpha:.3,collideStrength:1,clustering:!1,clusterNodeStrength:-1,clusterEdgeStrength:.1,clusterEdgeDistance:100,clusterFociStrength:.8,clusterNodeSize:10,tick:function(){},onLayoutEnd:function(){},onTick:function(){},workerEnabled:!1}},e.prototype.init=function(t){this.nodes=t.nodes||[];var e=t.edges||[];this.edges=e.map((function(t){var e={},r=["targetNode","sourceNode","startPoint","endPoint"];return Object.keys(t).forEach((function(n){r.indexOf(n)>-1||(e[n]=t[n])})),e})),this.ticking=!1},e.prototype.execute=function(t){var e=this,r=e.nodes,n=e.edges;if(!e.ticking){var o=e.forceSimulation,i=e.alphaMin,a=e.alphaDecay,u=e.alpha;if(o)t&&(e.clustering&&e.clusterForce&&(e.clusterForce.nodes(r),e.clusterForce.links(n)),o.nodes(r),e.edgeForce.links(n)),e.preventOverlap&&e.overlapProcess(o),o.alpha(u).restart(),this.ticking=!0;else try{var s=St();if(e.nodeStrength&&s.strength(e.nodeStrength),o=_t().nodes(r),e.clustering){var c=Pt();c.centerX(e.center[0]).centerY(e.center[1]).template("force").strength(e.clusterFociStrength),n&&c.links(n),r&&c.nodes(r),c.forceLinkDistance(e.clusterEdgeDistance).forceLinkStrength(e.clusterEdgeStrength).forceCharge(e.clusterNodeStrength).forceNodeSize(e.clusterNodeSize),e.clusterForce=c,o.force("group",c)}if(o.force("center",function(t,e){var r,n=1;function o(){var o,i,a=r.length,u=0,s=0;for(o=0;o<a;++o)u+=(i=r[o]).x,s+=i.y;for(u=(u/a-t)*n,s=(s/a-e)*n,o=0;o<a;++o)(i=r[o]).x-=u,i.y-=s}return null==t&&(t=0),null==e&&(e=0),o.initialize=function(t){r=t},o.x=function(e){return arguments.length?(t=+e,o):t},o.y=function(t){return arguments.length?(e=+t,o):e},o.strength=function(t){return arguments.length?(n=+t,o):n},o}(e.center[0],e.center[1])).force("charge",s).alpha(u).alphaDecay(a).alphaMin(i),e.preventOverlap&&e.overlapProcess(o),n){var f=Ot().id((function(t){return t.id})).links(n);e.edgeStrength&&f.strength(e.edgeStrength),e.linkDistance&&f.distance(e.linkDistance),e.edgeForce=f,o.force("link",f)}if(e.workerEnabled&&!Ut()&&(e.workerEnabled=!1,console.warn("workerEnabled option is only supported when running in web worker.")),e.workerEnabled){o.stop();for(var h=function(t){var e=t.alphaMin(),r=t.alphaTarget(),n=t.alpha(),o=Math.log((e-r)/(n-r))/Math.log(1-t.alphaDecay());return Math.ceil(o)}(o),l=1;l<=h;l++)o.tick(),postMessage({type:Bt,currentTick:l,totalTicks:h,nodes:r},void 0);e.ticking=!1}else o.on("tick",(function(){e.tick()})).on("end",(function(){e.ticking=!1,e.onLayoutEnd&&e.onLayoutEnd()})),e.ticking=!0;e.forceSimulation=o,e.ticking=!0}catch(t){e.ticking=!1,console.warn(t)}}},e.prototype.overlapProcess=function(t){var e,r,n=this.nodeSize,o=this.nodeSpacing,i=this.collideStrength;if(r=Tt()(o)?function(){return o}:zt()(o)?o:function(){return 0},n)if(zt()(n))e=function(t){return n(t)+r(t)};else if(j()(n)){var a=(n[0]>n[1]?n[0]:n[1])/2;e=function(t){return a+r(t)}}else if(Tt()(n)){var u=n/2;e=function(t){return u+r(t)}}else e=function(){return 10};else e=function(t){return t.size?j()(t.size)?(t.size[0]>t.size[1]?t.size[0]:t.size[1])/2+r(t):t.size/2+r(t):10+r(t)};t.force("collisionForce",Rt(e).strength(i))},e.prototype.updateCfg=function(t){this.ticking&&(this.forceSimulation.stop(),this.ticking=!1),this.forceSimulation=null,s()(this,t)},e.prototype.destroy=function(){this.ticking&&(this.forceSimulation.stop(),this.ticking=!1),this.nodes=null,this.edges=null,this.destroyed=!0},e}(c);function Ut(){return"undefined"!=typeof WorkerGlobalScope&&self instanceof WorkerGlobalScope}var Yt=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.center=[0,0],e.maxIteration=500,e.gravity=10,e.clustering=!1,e.clusterGravity=10,e.linkDistance=50,e.alpha=1,e.alphaMin=.001,e.alphaDecay=1-Math.pow(e.alphaMin,1/300),e.alphaTarget=0,e.velocityDecay=.6,e.linkStrength=1,e.nodeStrength=30,e.preventOverlap=!1,e.collideStrength=1,e.optimizeRangeFactor=1,e.tick=function(){},e.nodes=[],e.edges=[],e.width=300,e.height=300,e.bias=[],e.nodeMap={},e.nodeIdxMap={},e}return a(e,t),e.prototype.getDefaultCfg=function(){return{maxIteration:1e3,center:[0,0],gravity:10,clustering:!1,clusterGravity:10,preventOverlap:!1,nodeSpacing:void 0,collideStrength:10}},e.prototype.execute=function(){var t=this.nodes,e=this.center;if(t&&0!==t.length){if(1===t.length)return t[0].x=e[0],void(t[0].y=e[1]);var r={},n={};t.forEach((function(t,e){r[t.id]=t,n[t.id]=e})),this.nodeMap=r,this.nodeIdxMap=n,this.run()}},e.prototype.run=function(){var t=this,e=t.nodes,r=t.edges,n=t.maxIteration;t.width||"undefined"==typeof window||(t.width=window.innerWidth),t.height||"undefined"==typeof window||(t.height=window.innerHeight);var o,i=t.center,a=t.velocityDecay,u=t.clustering;t.initVals(),u&&(o=t.getClusterMap());for(var s=function(n){var i=[];e.forEach((function(t,e){i[e]={x:0,y:0}})),t.applyCalculate(e,r,i),u&&t.applyClusterForce(o,i),e.forEach((function(t,e){k(t.x)&&k(t.y)&&(t.x+=i[e].x*a,t.y+=i[e].y*a)})),c.alpha+=(c.alphaTarget-c.alpha)*c.alphaDecay,t.tick()},c=this,f=0;f<n;f++)s();var h=[0,0];e.forEach((function(t){k(t.x)&&k(t.y)&&(h[0]+=t.x,h[1]+=t.y)})),h[0]/=e.length,h[1]/=e.length;var l=[i[0]-h[0],i[1]-h[1]];e.forEach((function(t,e){k(t.x)&&k(t.y)&&(t.x+=l[0],t.y+=l[1])}))},e.prototype.initVals=function(){for(var t=this.edges,e={},r=0;r<t.length;++r)e[t[r].source]?e[t[r].source]++:e[t[r].source]=1,e[t[r].target]?e[t[r].target]++:e[t[r].target]=1;var n=[];for(r=0;r<t.length;++r)n[r]=e[t[r].source]/(e[t[r].source]+e[t[r].target]);this.bias=n;var o,i,a=this.nodeSize,u=this.nodeSpacing;if(i=k(u)?function(){return u}:g(u)?u:function(){return 0},a)if(g(a))o=function(t){return a(t)+i(t)};else if(y(a)){var s=(a[0]>a[1]?a[0]:a[1])/2;o=function(t){return s+i(t)}}else if(k(a)){var c=a/2;o=function(t){return c+i(t)}}else o=function(){return 10};else o=function(t){return t.size?y(t.size)?(t.size[0]>t.size[1]?t.size[0]:t.size[1])/2+i(t):t.size/2+i(t):10+i(t)};this.nodeSize=o;var f,h=this.linkDistance;h||(h=50),k(h)&&(f=function(t){return h}),this.linkDistance=f;var l,d=this.linkStrength;d||(d=1),k(d)&&(l=function(t){return d}),this.linkStrength=l;var p,v=this.nodeStrength;v||(v=30),k(v)&&(p=function(t){return v}),this.nodeStrength=p},e.prototype.getClusterMap=function(){var t=this.nodes,e={};for(var r in t.forEach((function(t){if(void 0===e[t.cluster]){var r={name:t.cluster,cx:0,cy:0,count:0};e[t.cluster]=r}var n=e[t.cluster];k(t.x)&&(n.cx+=t.x),k(t.y)&&(n.cy+=t.y),n.count++})),e)e[r].cx/=e[r].count,e[r].cy/=e[r].count;return e},e.prototype.applyClusterForce=function(t,e){var r=this.gravity,n=this.nodes,o=this.clusterGravity||r,i=this.alpha;for(var a in n.forEach((function(r,n){if(k(r.x)&&k(r.y)){var a=t[r.cluster],u=r.x-a.cx,s=r.y-a.cy,c=Math.sqrt(u*u+s*s);e[n].x-=u*o*i/c,e[n].y-=s*o*i/c}})),t)t[a].cx=0,t[a].cy=0,t[a].count=0;for(var a in n.forEach((function(e){var r=t[e.cluster];k(e.x)&&(r.cx+=e.x),k(e.y)&&(r.cy+=e.y),r.count++})),t)t[a].cx/=t[a].count,t[a].cy/=t[a].count},e.prototype.applyCalculate=function(t,e,r){var n={};t.forEach((function(e,o){r[o]={x:0,y:0},t.forEach((function(t,r){if(!(o<r)){var i=e.x-t.x,a=e.y-t.y,u=i*i+a*a;u<1&&(u=Math.sqrt(u)),0===i&&(u+=(i=.01*Math.random())*i),0===a&&(u+=(a=.01*Math.random())*a);var s=Math.sqrt(u);n[e.id+"-"+t.id]={vx:i,vy:a,vl:u,sqrtVl:s},n[t.id+"-"+e.id]={vx:-i,vy:-a,vl:u,sqrtVl:s}}}))})),this.calRepulsive(t,r,n),this.calAttractive(e,r,n)},e.prototype.calRepulsive=function(t,e,r){var n=this.width*this.optimizeRangeFactor*this.width*this.optimizeRangeFactor,o=this.nodeStrength,i=this.alpha,a=this.collideStrength,u=this.preventOverlap,s=this.nodeSize;t.forEach((function(c,f){t.forEach((function(t,h){if(f!==h&&k(c.x)&&k(t.x)&&k(c.y)&&k(t.y)){var l=r[c.id+"-"+t.id],d=l.vl,p=l.vx,v=l.vy;if(!(d>n)&&(e[f].x+=p*o(t)*i/d,e[f].y+=v*o(t)*i/d,u&&f<h)){var g=s(c),y=s(t),m=g+y;if(d<m*m){var b=r[c.id+"-"+t.id].sqrtVl,w=(m-b)/b*a,x=y*y/(g*g+y*y),M=p*w,_=v*w;e[f].x+=M*x,e[f].y+=_*x,x=1-x,e[h].x-=M*x,e[h].y-=_*x}}}}))}))},e.prototype.calAttractive=function(t,e,r){var n=this,o=this.linkDistance,i=this.alpha,a=this.linkStrength,u=this.bias;t.forEach((function(t,s){if(t.source&&t.target){var c=n.nodeIdxMap[t.source],f=n.nodeIdxMap[t.target];if(c!==f){var h=n.nodeMap[t.source],l=n.nodeMap[t.target];if(k(l.x)&&k(h.x)&&k(l.y)&&k(h.y)){var d=r[t.target+"-"+t.source],p=(d.vl,d.sqrtVl),v=d.vx,g=d.vy,y=(p-o(t))/p*i*a(t),m=v*y,b=g*y,w=u[s];e[f].x-=m*w,e[f].y-=b*w,e[c].x+=m*(1-w),e[c].y+=b*(1-w)}}}}))},e}(c),Ht=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.center=[0,0],e.maxIteration=1e3,e.gravity=10,e.speed=1,e.clustering=!1,e.clusterGravity=10,e.nodes=[],e.edges=[],e.width=300,e.height=300,e.nodeMap={},e.nodeIdxMap={},e}return a(e,t),e.prototype.getDefaultCfg=function(){return{maxIteration:1e3,center:[0,0],gravity:10,speed:1,clustering:!1,clusterGravity:10}},e.prototype.execute=function(){var t=this,e=this.nodes,r=this.center;if(e&&0!==e.length){if(1===e.length)return e[0].x=r[0],void(e[0].y=r[1]);var n={},o={};e.forEach((function(e,r){k(e.x)||(e.x=Math.random()*t.width),k(e.y)||(e.y=Math.random()*t.height),n[e.id]=e,o[e.id]=r})),this.nodeMap=n,this.nodeIdxMap=o,this.run()}},e.prototype.run=function(){var t=this,e=t.nodes,r=t.edges,n=t.maxIteration;t.width||"undefined"==typeof window||(t.width=window.innerWidth),t.height||"undefined"==typeof window||(t.height=window.innerHeight);var o=t.center,i=t.width/10,a=Math.sqrt(t.width*t.height/(e.length+1)),u=t.gravity,s=t.speed,c=t.clustering,f={};if(c)for(var h in e.forEach((function(t){if(void 0===f[t.cluster]){var e={name:t.cluster,cx:0,cy:0,count:0};f[t.cluster]=e}var r=f[t.cluster];k(t.x)&&(r.cx+=t.x),k(t.y)&&(r.cy+=t.y),r.count++})),f)f[h].cx/=f[h].count,f[h].cy/=f[h].count;for(var l=function(n){var h=[];if(e.forEach((function(t,e){h[e]={x:0,y:0}})),t.applyCalculate(e,r,h,a),c){var l=t.clusterGravity||u;for(var d in e.forEach((function(t,e){if(k(t.x)&&k(t.y)){var r=f[t.cluster],n=Math.sqrt((t.x-r.cx)*(t.x-r.cx)+(t.y-r.cy)*(t.y-r.cy)),o=a*l;h[e].x-=o*(t.x-r.cx)/n,h[e].y-=o*(t.y-r.cy)/n}})),f)f[d].cx=0,f[d].cy=0,f[d].count=0;for(var d in e.forEach((function(t){var e=f[t.cluster];k(t.x)&&(e.cx+=t.x),k(t.y)&&(e.cy+=t.y),e.count++})),f)f[d].cx/=f[d].count,f[d].cy/=f[d].count}e.forEach((function(t,e){if(k(t.x)&&k(t.y)){var r=.01*a*u;h[e].x-=r*(t.x-o[0]),h[e].y-=r*(t.y-o[1])}})),e.forEach((function(t,e){if(k(t.x)&&k(t.y)){var r=Math.sqrt(h[e].x*h[e].x+h[e].y*h[e].y);if(r>0){var n=Math.min(i*(s/800),r);t.x+=h[e].x/r*n,t.y+=h[e].y/r*n}}}))},d=0;d<n;d++)l()},e.prototype.applyCalculate=function(t,e,r,n){this.calRepulsive(t,r,n),this.calAttractive(e,r,n)},e.prototype.calRepulsive=function(t,e,r){t.forEach((function(n,o){e[o]={x:0,y:0},t.forEach((function(t,i){if(o!==i&&k(n.x)&&k(t.x)&&k(n.y)&&k(t.y)){var a=n.x-t.x,u=n.y-t.y,s=a*a+u*u;if(0===s){s=1;var c=o>i?1:-1;a=.01*c,u=.01*c}var f=r*r/s;e[o].x+=a*f,e[o].y+=u*f}}))}))},e.prototype.calAttractive=function(t,e,r){var n=this;t.forEach((function(t){if(t.source&&t.target){var o=n.nodeIdxMap[t.source],i=n.nodeIdxMap[t.target];if(o!==i){var a=n.nodeMap[t.source],u=n.nodeMap[t.target];if(k(u.x)&&k(a.x)&&k(u.y)&&k(a.y)){var s=u.x-a.x,c=u.y-a.y,f=Math.sqrt(s*s+c*c),h=f*f/r;e[i].x-=s/f*h,e[i].y-=c/f*h,e[o].x+=s/f*h,e[o].y+=c/f*h}}}}))},e}(c),Wt=(r(54),r(130)),Gt=(function(t){function e(e,r){var n=t.call(this,e,r)||this;return n.item=r.item,n.canvasX=r.canvasX,n.canvasY=r.canvasY,n.wheelDelta=r.wheelDelta,n.detail=r.detail,n}a(e,t)}(r.n(Wt).a),function(t){return Number.isNaN(Number(t))}),$t=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.begin=[0,0],e.preventOverlap=!0,e.preventOverlapPadding=10,e.condense=!1,e.sortBy="degree",e.nodeSize=30,e.nodes=[],e.edges=[],e.center=[0,0],e.width=300,e.height=300,e.row=0,e.col=0,e.cellWidth=0,e.cellHeight=0,e.cellUsed={},e.id2manPos={},e}return a(e,t),e.prototype.getDefaultCfg=function(){return{begin:[0,0],preventOverlap:!0,preventOverlapPadding:10,condense:!1,rows:void 0,cols:void 0,position:void 0,sortBy:"degree",nodeSize:30}},e.prototype.execute=function(){var t=this,e=t.nodes,r=e.length,n=t.center;if(0!==r){if(1===r)return e[0].x=n[0],void(e[0].y=n[1]);var o=t.edges,i=[];e.forEach((function(t){i.push(t)}));var a={};if(i.forEach((function(t,e){a[t.id]=e})),("degree"===t.sortBy||!C()(t.sortBy)||void 0===i[0][t.sortBy])&&(t.sortBy="degree",Gt(e[0].degree))){var u=N(i.length,a,o);i.forEach((function(t,e){t.degree=u[e]}))}i.sort((function(e,r){return r[t.sortBy]-e[t.sortBy]})),t.width||"undefined"==typeof window||(t.width=window.innerWidth),t.height||"undefined"==typeof window||(t.height=window.innerHeight);var s=t.rows,c=null!=t.cols?t.cols:t.columns;if(t.cells=r,null!=s&&null!=c?(t.rows=s,t.cols=c):null!=s&&null==c?(t.rows=s,t.cols=Math.ceil(t.cells/t.rows)):null==s&&null!=c?(t.cols=c,t.rows=Math.ceil(t.cells/t.cols)):(t.splits=Math.sqrt(t.cells*t.height/t.width),t.rows=Math.round(t.splits),t.cols=Math.round(t.width/t.height*t.splits)),t.cols*t.rows>t.cells)((h=t.small())-1)*(f=t.large())>=t.cells?t.small(h-1):(f-1)*h>=t.cells&&t.large(f-1);else for(;t.cols*t.rows<t.cells;){var f,h=t.small();((f=t.large())+1)*h>=t.cells?t.large(f+1):t.small(h+1)}t.cellWidth=t.width/t.cols,t.cellHeight=t.height/t.rows,t.condense&&(t.cellWidth=0,t.cellHeight=0),t.preventOverlap&&i.forEach((function(e){var r,n;e.x&&e.y||(e.x=0,e.y=0),y(e.size)?(r=e.size[0],n=e.size[1]):k(e.size)&&(r=e.size,n=e.size),void 0!==r&&void 0!==n||(y(t.nodeSize)?(r=t.nodeSize[0],n=t.nodeSize[1]):k(t.nodeSize)?(r=t.nodeSize,n=t.nodeSize):(r=30,n=30));var o=t.preventOverlapPadding,i=r+o,a=n+o;t.cellWidth=Math.max(t.cellWidth,i),t.cellHeight=Math.max(t.cellHeight,a)})),t.cellUsed={},t.row=0,t.col=0,t.id2manPos={};for(var l=0;l<i.length;l++){var d=i[l],p=void 0;if(t.position&&(p=t.position(d)),p&&(void 0!==p.row||void 0!==p.col)){var v={row:p.row,col:p.col};if(void 0===v.col)for(v.col=0;t.used(v.row,v.col);)v.col++;else if(void 0===v.row)for(v.row=0;t.used(v.row,v.col);)v.row++;t.id2manPos[d.id]=v,t.use(v.row,v.col)}t.getPos(d)}}},e.prototype.small=function(t){var e,r=this.rows||5,n=this.cols||5;null==t?e=Math.min(r,n):Math.min(r,n)===this.rows?this.rows=t:this.cols=t;return e},e.prototype.large=function(t){var e,r=this.rows||5,n=this.cols||5;null==t?e=Math.max(r,n):Math.max(r,n)===this.rows?this.rows=t:this.cols=t;return e},e.prototype.used=function(t,e){return this.cellUsed["c-"+t+"-"+e]||!1},e.prototype.use=function(t,e){this.cellUsed["c-"+t+"-"+e]=!0},e.prototype.moveToNextCell=function(){var t=this.cols||5;this.col++,this.col>=t&&(this.col=0,this.row++)},e.prototype.getPos=function(t){var e,r,n=this.begin,o=this.cellWidth,i=this.cellHeight,a=this.id2manPos[t.id];if(a)e=a.col*o+o/2+n[0],r=a.row*i+i/2+n[1];else{for(;this.used(this.row,this.col);)this.moveToNextCell();e=this.col*o+o/2+n[0],r=this.row*i+i/2+n[1],this.use(this.row,this.col),this.moveToNextCell()}t.x=e,t.y=r},e}(c),Xt=Object.prototype.toString;function Jt(t){return Xt.call(t).endsWith("Array]")}var Qt=function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};if(!Jt(t))throw new TypeError("input must be an array");if(0===t.length)throw new TypeError("input must not be empty");var r=e.fromIndex,n=void 0===r?0:r,o=e.toIndex,i=void 0===o?t.length:o;if(n<0||n>=t.length||!Number.isInteger(n))throw new Error("fromIndex must be a positive integer smaller than length");if(i<=n||i>t.length||!Number.isInteger(i))throw new Error("toIndex must be an integer greater than fromIndex and at most equal to length");for(var a=t[n],u=n+1;u<i;u++)t[u]>a&&(a=t[u]);return a};var Kt=function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};if(!Jt(t))throw new TypeError("input must be an array");if(0===t.length)throw new TypeError("input must not be empty");var r=e.fromIndex,n=void 0===r?0:r,o=e.toIndex,i=void 0===o?t.length:o;if(n<0||n>=t.length||!Number.isInteger(n))throw new Error("fromIndex must be a positive integer smaller than length");if(i<=n||i>t.length||!Number.isInteger(i))throw new Error("toIndex must be an integer greater than fromIndex and at most equal to length");for(var a=t[n],u=n+1;u<i;u++)t[u]<a&&(a=t[u]);return a};var Zt=function(t){var e,r=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};if(!Jt(t))throw new TypeError("input must be an array");if(0===t.length)throw new TypeError("input must not be empty");if(void 0!==r.output){if(!Jt(r.output))throw new TypeError("output option must be an array if specified");e=r.output}else e=new Array(t.length);var n=Kt(t),o=Qt(t);if(n===o)throw new RangeError("minimum and maximum input values are equal. Cannot rescale a constant array");var i=r.min,a=void 0===i?r.autoMinMax?n:0:i,u=r.max,s=void 0===u?r.autoMinMax?o:1:u;if(a>=s)throw new RangeError("min option must be smaller than max option");for(var c=(s-a)/(o-n),f=0;f<t.length;f++)e[f]=(t[f]-n)*c+a;return e},te=" ".repeat(2),ee=" ".repeat(4);function re(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},r=e.maxRows,n=void 0===r?15:r,o=e.maxColumns,i=void 0===o?10:o,a=e.maxNumSize,u=void 0===a?8:a;return"".concat(t.constructor.name," {\\n").concat(te,"[\\n").concat(ee).concat(ne(t,n,i,u),"\\n").concat(te,"]\\n").concat(te,"rows: ").concat(t.rows,"\\n").concat(te,"columns: ").concat(t.columns,"\\n}")}function ne(t,e,r,n){for(var o=t.rows,i=t.columns,a=Math.min(o,e),u=Math.min(i,r),s=[],c=0;c<a;c++){for(var f=[],h=0;h<u;h++)f.push(oe(t.get(c,h),n));s.push("".concat(f.join(" ")))}return u!==i&&(s[s.length-1]+=" ... ".concat(i-r," more columns")),a!==o&&s.push("... ".concat(o-e," more rows")),s.join("\\n".concat(ee))}function oe(t,e){var r=String(t);if(r.length<=e)return r.padEnd(e," ");var n=t.toPrecision(e-2);if(n.length<=e)return n;var o=t.toExponential(e-2),i=o.indexOf("e"),a=o.slice(i);return o.slice(0,e-a.length)+a}function ie(t){return(ie="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(t){return typeof t}:function(t){return t&&"function"==typeof Symbol&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t})(t)}function ae(t,e,r){var n=r?t.rows:t.rows-1;if(e<0||e>n)throw new RangeError("Row index out of range")}function ue(t,e,r){var n=r?t.columns:t.columns-1;if(e<0||e>n)throw new RangeError("Column index out of range")}function se(t,e){if(e.to1DArray&&(e=e.to1DArray()),e.length!==t.columns)throw new RangeError("vector size must be the same as the number of columns");return e}function ce(t,e){if(e.to1DArray&&(e=e.to1DArray()),e.length!==t.rows)throw new RangeError("vector size must be the same as the number of rows");return e}function fe(t,e){if("object"!==ie(e))throw new TypeError("unexpected type for row indices");if(e.some((function(e){return e<0||e>=t.rows})))throw new RangeError("row indices are out of range");return Array.isArray(e)||(e=Array.from(e)),e}function he(t,e){if("object"!==ie(e))throw new TypeError("unexpected type for column indices");if(e.some((function(e){return e<0||e>=t.columns})))throw new RangeError("column indices are out of range");return Array.isArray(e)||(e=Array.from(e)),e}function le(t,e,r,n,o){if(5!==arguments.length)throw new RangeError("expected 4 arguments");if(pe("startRow",e),pe("endRow",r),pe("startColumn",n),pe("endColumn",o),e>r||n>o||e<0||e>=t.rows||r<0||r>=t.rows||n<0||n>=t.columns||o<0||o>=t.columns)throw new RangeError("Submatrix indices are out of range")}function de(t){for(var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:0,r=[],n=0;n<t;n++)r.push(e);return r}function pe(t,e){if("number"!=typeof e)throw new TypeError("".concat(t," must be a number"))}function ve(t,e,r){for(var n=t.rows,o=t.columns,i=[],a=0;a<n;a++){for(var u=0,s=0,c=0,f=0;f<o;f++)u+=c=t.get(a,f)-r[a],s+=c*c;e?i.push((s-u*u/o)/(o-1)):i.push((s-u*u/o)/o)}return i}function ge(t,e,r){for(var n=t.rows,o=t.columns,i=[],a=0;a<o;a++){for(var u=0,s=0,c=0,f=0;f<n;f++)u+=c=t.get(f,a)-r[a],s+=c*c;e?i.push((s-u*u/n)/(n-1)):i.push((s-u*u/n)/n)}return i}function ye(t,e,r){for(var n=t.rows,o=t.columns,i=n*o,a=0,u=0,s=0,c=0;c<n;c++)for(var f=0;f<o;f++)a+=s=t.get(c,f)-r,u+=s*s;return e?(u-a*a/i)/(i-1):(u-a*a/i)/i}function me(t,e){for(var r=0;r<t.rows;r++)for(var n=0;n<t.columns;n++)t.set(r,n,t.get(r,n)-e[r])}function be(t,e){for(var r=0;r<t.rows;r++)for(var n=0;n<t.columns;n++)t.set(r,n,t.get(r,n)-e[n])}function we(t,e){for(var r=0;r<t.rows;r++)for(var n=0;n<t.columns;n++)t.set(r,n,t.get(r,n)-e)}function xe(t){for(var e=[],r=0;r<t.rows;r++){for(var n=0,o=0;o<t.columns;o++)n+=Math.pow(t.get(r,o),2)/(t.columns-1);e.push(Math.sqrt(n))}return e}function Me(t,e){for(var r=0;r<t.rows;r++)for(var n=0;n<t.columns;n++)t.set(r,n,t.get(r,n)/e[r])}function ke(t){for(var e=[],r=0;r<t.columns;r++){for(var n=0,o=0;o<t.rows;o++)n+=Math.pow(t.get(o,r),2)/(t.rows-1);e.push(Math.sqrt(n))}return e}function _e(t,e){for(var r=0;r<t.rows;r++)for(var n=0;n<t.columns;n++)t.set(r,n,t.get(r,n)/e[n])}function Se(t){for(var e=t.size-1,r=0,n=0;n<t.columns;n++)for(var o=0;o<t.rows;o++)r+=Math.pow(t.get(o,n),2)/e;return Math.sqrt(r)}function Ee(t,e){for(var r=0;r<t.rows;r++)for(var n=0;n<t.columns;n++)t.set(r,n,t.get(r,n)/e)}function je(t,e){return(je=Object.setPrototypeOf||function(t,e){return t.__proto__=e,t})(t,e)}function Ae(t){var e=function(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],(function(){}))),!0}catch(t){return!1}}();return function(){var r,n=Ne(t);if(e){var o=Ne(this).constructor;r=Reflect.construct(n,arguments,o)}else r=n.apply(this,arguments);return Oe(this,r)}}function Oe(t,e){return!e||"object"!==Ie(e)&&"function"!=typeof e?function(t){if(void 0===t)throw new ReferenceError("this hasn\'t been initialised - super() hasn\'t been called");return t}(t):e}function Ne(t){return(Ne=Object.setPrototypeOf?Object.getPrototypeOf:function(t){return t.__proto__||Object.getPrototypeOf(t)})(t)}function Ie(t){return(Ie="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(t){return typeof t}:function(t){return t&&"function"==typeof Symbol&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t})(t)}function Re(t,e){if(!(t instanceof e))throw new TypeError("Cannot call a class as a function")}function Pe(t,e){for(var r=0;r<e.length;r++){var n=e[r];n.enumerable=n.enumerable||!1,n.configurable=!0,"value"in n&&(n.writable=!0),Object.defineProperty(t,n.key,n)}}function Ce(t,e,r){return e&&Pe(t.prototype,e),r&&Pe(t,r),t}var ze=function(){function t(){Re(this,t)}return Ce(t,[{key:"apply",value:function(t){if("function"!=typeof t)throw new TypeError("callback must be a function");for(var e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)t.call(this,e,r);return this}},{key:"to1DArray",value:function(){for(var t=[],e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)t.push(this.get(e,r));return t}},{key:"to2DArray",value:function(){for(var t=[],e=0;e<this.rows;e++){t.push([]);for(var r=0;r<this.columns;r++)t[e].push(this.get(e,r))}return t}},{key:"toJSON",value:function(){return this.to2DArray()}},{key:"isRowVector",value:function(){return 1===this.rows}},{key:"isColumnVector",value:function(){return 1===this.columns}},{key:"isVector",value:function(){return 1===this.rows||1===this.columns}},{key:"isSquare",value:function(){return this.rows===this.columns}},{key:"isSymmetric",value:function(){if(this.isSquare()){for(var t=0;t<this.rows;t++)for(var e=0;e<=t;e++)if(this.get(t,e)!==this.get(e,t))return!1;return!0}return!1}},{key:"isEchelonForm",value:function(){for(var t=0,e=0,r=-1,n=!0,o=!1;t<this.rows&&n;){for(e=0,o=!1;e<this.columns&&!1===o;)0===this.get(t,e)?e++:1===this.get(t,e)&&e>r?(o=!0,r=e):(n=!1,o=!0);t++}return n}},{key:"isReducedEchelonForm",value:function(){for(var t=0,e=0,r=-1,n=!0,o=!1;t<this.rows&&n;){for(e=0,o=!1;e<this.columns&&!1===o;)0===this.get(t,e)?e++:1===this.get(t,e)&&e>r?(o=!0,r=e):(n=!1,o=!0);for(var i=e+1;i<this.rows;i++)0!==this.get(t,i)&&(n=!1);t++}return n}},{key:"echelonForm",value:function(){for(var t=this.clone(),e=0,r=0;e<t.rows&&r<t.columns;){for(var n=e,o=e;o<t.rows;o++)t.get(o,r)>t.get(n,r)&&(n=o);if(0===t.get(n,r))r++;else{t.swapRows(e,n);for(var i=t.get(e,r),a=r;a<t.columns;a++)t.set(e,a,t.get(e,a)/i);for(var u=e+1;u<t.rows;u++){var s=t.get(u,r)/t.get(e,r);t.set(u,r,0);for(var c=r+1;c<t.columns;c++)t.set(u,c,t.get(u,c)-t.get(e,c)*s)}e++,r++}}return t}},{key:"reducedEchelonForm",value:function(){for(var t=this.echelonForm(),e=t.columns,r=t.rows,n=r-1;n>=0;)if(0===t.maxRow(n))n--;else{for(var o=0,i=!1;o<r&&!1===i;)1===t.get(n,o)?i=!0:o++;for(var a=0;a<n;a++)for(var u=t.get(a,o),s=o;s<e;s++){var c=t.get(a,s)-u*t.get(n,s);t.set(a,s,c)}n--}return t}},{key:"set",value:function(){throw new Error("set method is unimplemented")}},{key:"get",value:function(){throw new Error("get method is unimplemented")}},{key:"repeat",value:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};if("object"!==Ie(t))throw new TypeError("options must be an object");var e=t.rows,r=void 0===e?1:e,n=t.columns,o=void 0===n?1:n;if(!Number.isInteger(r)||r<=0)throw new TypeError("rows must be a positive integer");if(!Number.isInteger(o)||o<=0)throw new TypeError("columns must be a positive integer");for(var i=new qe(this.rows*r,this.columns*o),a=0;a<r;a++)for(var u=0;u<o;u++)i.setSubMatrix(this,this.rows*a,this.columns*u);return i}},{key:"fill",value:function(t){for(var e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)this.set(e,r,t);return this}},{key:"neg",value:function(){return this.mulS(-1)}},{key:"getRow",value:function(t){ae(this,t);for(var e=[],r=0;r<this.columns;r++)e.push(this.get(t,r));return e}},{key:"getRowVector",value:function(t){return qe.rowVector(this.getRow(t))}},{key:"setRow",value:function(t,e){ae(this,t),e=se(this,e);for(var r=0;r<this.columns;r++)this.set(t,r,e[r]);return this}},{key:"swapRows",value:function(t,e){ae(this,t),ae(this,e);for(var r=0;r<this.columns;r++){var n=this.get(t,r);this.set(t,r,this.get(e,r)),this.set(e,r,n)}return this}},{key:"getColumn",value:function(t){ue(this,t);for(var e=[],r=0;r<this.rows;r++)e.push(this.get(r,t));return e}},{key:"getColumnVector",value:function(t){return qe.columnVector(this.getColumn(t))}},{key:"setColumn",value:function(t,e){ue(this,t),e=ce(this,e);for(var r=0;r<this.rows;r++)this.set(r,t,e[r]);return this}},{key:"swapColumns",value:function(t,e){ue(this,t),ue(this,e);for(var r=0;r<this.rows;r++){var n=this.get(r,t);this.set(r,t,this.get(r,e)),this.set(r,e,n)}return this}},{key:"addRowVector",value:function(t){t=se(this,t);for(var e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)this.set(e,r,this.get(e,r)+t[r]);return this}},{key:"subRowVector",value:function(t){t=se(this,t);for(var e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)this.set(e,r,this.get(e,r)-t[r]);return this}},{key:"mulRowVector",value:function(t){t=se(this,t);for(var e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)this.set(e,r,this.get(e,r)*t[r]);return this}},{key:"divRowVector",value:function(t){t=se(this,t);for(var e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)this.set(e,r,this.get(e,r)/t[r]);return this}},{key:"addColumnVector",value:function(t){t=ce(this,t);for(var e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)this.set(e,r,this.get(e,r)+t[e]);return this}},{key:"subColumnVector",value:function(t){t=ce(this,t);for(var e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)this.set(e,r,this.get(e,r)-t[e]);return this}},{key:"mulColumnVector",value:function(t){t=ce(this,t);for(var e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)this.set(e,r,this.get(e,r)*t[e]);return this}},{key:"divColumnVector",value:function(t){t=ce(this,t);for(var e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)this.set(e,r,this.get(e,r)/t[e]);return this}},{key:"mulRow",value:function(t,e){ae(this,t);for(var r=0;r<this.columns;r++)this.set(t,r,this.get(t,r)*e);return this}},{key:"mulColumn",value:function(t,e){ue(this,t);for(var r=0;r<this.rows;r++)this.set(r,t,this.get(r,t)*e);return this}},{key:"max",value:function(){for(var t=this.get(0,0),e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)this.get(e,r)>t&&(t=this.get(e,r));return t}},{key:"maxIndex",value:function(){for(var t=this.get(0,0),e=[0,0],r=0;r<this.rows;r++)for(var n=0;n<this.columns;n++)this.get(r,n)>t&&(t=this.get(r,n),e[0]=r,e[1]=n);return e}},{key:"min",value:function(){for(var t=this.get(0,0),e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)this.get(e,r)<t&&(t=this.get(e,r));return t}},{key:"minIndex",value:function(){for(var t=this.get(0,0),e=[0,0],r=0;r<this.rows;r++)for(var n=0;n<this.columns;n++)this.get(r,n)<t&&(t=this.get(r,n),e[0]=r,e[1]=n);return e}},{key:"maxRow",value:function(t){ae(this,t);for(var e=this.get(t,0),r=1;r<this.columns;r++)this.get(t,r)>e&&(e=this.get(t,r));return e}},{key:"maxRowIndex",value:function(t){ae(this,t);for(var e=this.get(t,0),r=[t,0],n=1;n<this.columns;n++)this.get(t,n)>e&&(e=this.get(t,n),r[1]=n);return r}},{key:"minRow",value:function(t){ae(this,t);for(var e=this.get(t,0),r=1;r<this.columns;r++)this.get(t,r)<e&&(e=this.get(t,r));return e}},{key:"minRowIndex",value:function(t){ae(this,t);for(var e=this.get(t,0),r=[t,0],n=1;n<this.columns;n++)this.get(t,n)<e&&(e=this.get(t,n),r[1]=n);return r}},{key:"maxColumn",value:function(t){ue(this,t);for(var e=this.get(0,t),r=1;r<this.rows;r++)this.get(r,t)>e&&(e=this.get(r,t));return e}},{key:"maxColumnIndex",value:function(t){ue(this,t);for(var e=this.get(0,t),r=[0,t],n=1;n<this.rows;n++)this.get(n,t)>e&&(e=this.get(n,t),r[0]=n);return r}},{key:"minColumn",value:function(t){ue(this,t);for(var e=this.get(0,t),r=1;r<this.rows;r++)this.get(r,t)<e&&(e=this.get(r,t));return e}},{key:"minColumnIndex",value:function(t){ue(this,t);for(var e=this.get(0,t),r=[0,t],n=1;n<this.rows;n++)this.get(n,t)<e&&(e=this.get(n,t),r[0]=n);return r}},{key:"diag",value:function(){for(var t=Math.min(this.rows,this.columns),e=[],r=0;r<t;r++)e.push(this.get(r,r));return e}},{key:"norm",value:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:"frobenius",e=0;if("max"===t)return this.max();if("frobenius"===t){for(var r=0;r<this.rows;r++)for(var n=0;n<this.columns;n++)e+=this.get(r,n)*this.get(r,n);return Math.sqrt(e)}throw new RangeError("unknown norm type: ".concat(t))}},{key:"cumulativeSum",value:function(){for(var t=0,e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)t+=this.get(e,r),this.set(e,r,t);return this}},{key:"dot",value:function(e){t.isMatrix(e)&&(e=e.to1DArray());var r=this.to1DArray();if(r.length!==e.length)throw new RangeError("vectors do not have the same size");for(var n=0,o=0;o<r.length;o++)n+=r[o]*e[o];return n}},{key:"mmul",value:function(t){t=qe.checkMatrix(t);for(var e=this.rows,r=this.columns,n=t.columns,o=new qe(e,n),i=new Float64Array(r),a=0;a<n;a++){for(var u=0;u<r;u++)i[u]=t.get(u,a);for(var s=0;s<e;s++){for(var c=0,f=0;f<r;f++)c+=this.get(s,f)*i[f];o.set(s,a,c)}}return o}},{key:"strassen2x2",value:function(t){t=qe.checkMatrix(t);var e=new qe(2,2),r=this.get(0,0),n=t.get(0,0),o=this.get(0,1),i=t.get(0,1),a=this.get(1,0),u=t.get(1,0),s=this.get(1,1),c=t.get(1,1),f=(r+s)*(n+c),h=(a+s)*n,l=r*(i-c),d=s*(u-n),p=(r+o)*c,v=f+d-p+(o-s)*(u+c),g=l+p,y=h+d,m=f-h+l+(a-r)*(n+i);return e.set(0,0,v),e.set(0,1,g),e.set(1,0,y),e.set(1,1,m),e}},{key:"strassen3x3",value:function(t){t=qe.checkMatrix(t);var e=new qe(3,3),r=this.get(0,0),n=this.get(0,1),o=this.get(0,2),i=this.get(1,0),a=this.get(1,1),u=this.get(1,2),s=this.get(2,0),c=this.get(2,1),f=this.get(2,2),h=t.get(0,0),l=t.get(0,1),d=t.get(0,2),p=t.get(1,0),v=t.get(1,1),g=t.get(1,2),y=t.get(2,0),m=t.get(2,1),b=t.get(2,2),w=(r-i)*(-l+v),x=(-r+i+a)*(h-l+v),M=(i+a)*(-h+l),k=r*h,_=(-r+s+c)*(h-d+g),S=(-r+s)*(d-g),E=(s+c)*(-h+d),j=(-o+c+f)*(v+y-m),A=(o-f)*(v-m),O=o*y,N=(c+f)*(-y+m),I=(-o+a+u)*(g+y-b),R=(o-u)*(g-b),P=(a+u)*(-y+b),C=k+O+n*p,z=(r+n+o-i-a-c-f)*v+x+M+k+j+O+N,F=k+_+E+(r+n+o-a-u-s-c)*g+O+I+P,T=w+a*(-h+l+p-v-g-y+b)+x+k+O+I+R,D=w+x+M+k+u*m,q=O+I+R+P+i*d,L=k+_+S+c*(-h+d+p-v-g-y+m)+j+A+O,B=j+A+O+N+s*l,V=k+_+S+E+f*b;return e.set(0,0,C),e.set(0,1,z),e.set(0,2,F),e.set(1,0,T),e.set(1,1,D),e.set(1,2,q),e.set(2,0,L),e.set(2,1,B),e.set(2,2,V),e}},{key:"mmulStrassen",value:function(e){e=qe.checkMatrix(e);var r=this.clone(),n=r.rows,o=r.columns,i=e.rows,a=e.columns;function u(e,r,n){var o=e.rows,i=e.columns;if(o===r&&i===n)return e;var a=t.zeros(r,n);return a=a.setSubMatrix(e,0,0)}o!==i&&console.warn("Multiplying ".concat(n," x ").concat(o," and ").concat(i," x ").concat(a," matrix: dimensions do not match."));var s=Math.max(n,i),c=Math.max(o,a);return function e(r,n,o,i){if(o<=512||i<=512)return r.mmul(n);o%2==1&&i%2==1?(r=u(r,o+1,i+1),n=u(n,o+1,i+1)):o%2==1?(r=u(r,o+1,i),n=u(n,o+1,i)):i%2==1&&(r=u(r,o,i+1),n=u(n,o,i+1));var a=parseInt(r.rows/2,10),s=parseInt(r.columns/2,10),c=r.subMatrix(0,a-1,0,s-1),f=n.subMatrix(0,a-1,0,s-1),h=r.subMatrix(0,a-1,s,r.columns-1),l=n.subMatrix(0,a-1,s,n.columns-1),d=r.subMatrix(a,r.rows-1,0,s-1),p=n.subMatrix(a,n.rows-1,0,s-1),v=r.subMatrix(a,r.rows-1,s,r.columns-1),g=n.subMatrix(a,n.rows-1,s,n.columns-1),y=e(t.add(c,v),t.add(f,g),a,s),m=e(t.add(d,v),f,a,s),b=e(c,t.sub(l,g),a,s),w=e(v,t.sub(p,f),a,s),x=e(t.add(c,h),g,a,s),M=e(t.sub(d,c),t.add(f,l),a,s),k=e(t.sub(h,v),t.add(p,g),a,s),_=t.add(y,w);_.sub(x),_.add(k);var S=t.add(b,x),E=t.add(m,w),j=t.sub(y,m);j.add(b),j.add(M);var A=t.zeros(2*_.rows,2*_.columns);return(A=(A=(A=(A=A.setSubMatrix(_,0,0)).setSubMatrix(S,_.rows,0)).setSubMatrix(E,0,_.columns)).setSubMatrix(j,_.rows,_.columns)).subMatrix(0,o-1,0,i-1)}(r=u(r,s,c),e=u(e,s,c),s,c)}},{key:"scaleRows",value:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};if("object"!==Ie(t))throw new TypeError("options must be an object");var e=t.min,r=void 0===e?0:e,n=t.max,o=void 0===n?1:n;if(!Number.isFinite(r))throw new TypeError("min must be a number");if(!Number.isFinite(o))throw new TypeError("max must be a number");if(r>=o)throw new RangeError("min must be smaller than max");for(var i=new qe(this.rows,this.columns),a=0;a<this.rows;a++){var u=this.getRow(a);Zt(u,{min:r,max:o,output:u}),i.setRow(a,u)}return i}},{key:"scaleColumns",value:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};if("object"!==Ie(t))throw new TypeError("options must be an object");var e=t.min,r=void 0===e?0:e,n=t.max,o=void 0===n?1:n;if(!Number.isFinite(r))throw new TypeError("min must be a number");if(!Number.isFinite(o))throw new TypeError("max must be a number");if(r>=o)throw new RangeError("min must be smaller than max");for(var i=new qe(this.rows,this.columns),a=0;a<this.columns;a++){var u=this.getColumn(a);Zt(u,{min:r,max:o,output:u}),i.setColumn(a,u)}return i}},{key:"flipRows",value:function(){for(var t=Math.ceil(this.columns/2),e=0;e<this.rows;e++)for(var r=0;r<t;r++){var n=this.get(e,r),o=this.get(e,this.columns-1-r);this.set(e,r,o),this.set(e,this.columns-1-r,n)}return this}},{key:"flipColumns",value:function(){for(var t=Math.ceil(this.rows/2),e=0;e<this.columns;e++)for(var r=0;r<t;r++){var n=this.get(r,e),o=this.get(this.rows-1-r,e);this.set(r,e,o),this.set(this.rows-1-r,e,n)}return this}},{key:"kroneckerProduct",value:function(t){t=qe.checkMatrix(t);for(var e=this.rows,r=this.columns,n=t.rows,o=t.columns,i=new qe(e*n,r*o),a=0;a<e;a++)for(var u=0;u<r;u++)for(var s=0;s<n;s++)for(var c=0;c<o;c++)i.set(n*a+s,o*u+c,this.get(a,u)*t.get(s,c));return i}},{key:"transpose",value:function(){for(var t=new qe(this.columns,this.rows),e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)t.set(r,e,this.get(e,r));return t}},{key:"sortRows",value:function(){for(var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:Fe,e=0;e<this.rows;e++)this.setRow(e,this.getRow(e).sort(t));return this}},{key:"sortColumns",value:function(){for(var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:Fe,e=0;e<this.columns;e++)this.setColumn(e,this.getColumn(e).sort(t));return this}},{key:"subMatrix",value:function(t,e,r,n){le(this,t,e,r,n);for(var o=new qe(e-t+1,n-r+1),i=t;i<=e;i++)for(var a=r;a<=n;a++)o.set(i-t,a-r,this.get(i,a));return o}},{key:"subMatrixRow",value:function(t,e,r){if(void 0===e&&(e=0),void 0===r&&(r=this.columns-1),e>r||e<0||e>=this.columns||r<0||r>=this.columns)throw new RangeError("Argument out of range");for(var n=new qe(t.length,r-e+1),o=0;o<t.length;o++)for(var i=e;i<=r;i++){if(t[o]<0||t[o]>=this.rows)throw new RangeError("Row index out of range: ".concat(t[o]));n.set(o,i-e,this.get(t[o],i))}return n}},{key:"subMatrixColumn",value:function(t,e,r){if(void 0===e&&(e=0),void 0===r&&(r=this.rows-1),e>r||e<0||e>=this.rows||r<0||r>=this.rows)throw new RangeError("Argument out of range");for(var n=new qe(r-e+1,t.length),o=0;o<t.length;o++)for(var i=e;i<=r;i++){if(t[o]<0||t[o]>=this.columns)throw new RangeError("Column index out of range: ".concat(t[o]));n.set(i-e,o,this.get(i,t[o]))}return n}},{key:"setSubMatrix",value:function(t,e,r){le(this,e,e+(t=qe.checkMatrix(t)).rows-1,r,r+t.columns-1);for(var n=0;n<t.rows;n++)for(var o=0;o<t.columns;o++)this.set(e+n,r+o,t.get(n,o));return this}},{key:"selection",value:function(t,e){for(var r=function(t,e,r){return{row:fe(t,e),column:he(t,r)}}(this,t,e),n=new qe(t.length,e.length),o=0;o<r.row.length;o++)for(var i=r.row[o],a=0;a<r.column.length;a++){var u=r.column[a];n.set(o,a,this.get(i,u))}return n}},{key:"trace",value:function(){for(var t=Math.min(this.rows,this.columns),e=0,r=0;r<t;r++)e+=this.get(r,r);return e}},{key:"clone",value:function(){for(var t=new qe(this.rows,this.columns),e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)t.set(e,r,this.get(e,r));return t}},{key:"sum",value:function(t){switch(t){case"row":return function(t){for(var e=de(t.rows),r=0;r<t.rows;++r)for(var n=0;n<t.columns;++n)e[r]+=t.get(r,n);return e}(this);case"column":return function(t){for(var e=de(t.columns),r=0;r<t.rows;++r)for(var n=0;n<t.columns;++n)e[n]+=t.get(r,n);return e}(this);case void 0:return function(t){for(var e=0,r=0;r<t.rows;r++)for(var n=0;n<t.columns;n++)e+=t.get(r,n);return e}(this);default:throw new Error("invalid option: ".concat(t))}}},{key:"product",value:function(t){switch(t){case"row":return function(t){for(var e=de(t.rows,1),r=0;r<t.rows;++r)for(var n=0;n<t.columns;++n)e[r]*=t.get(r,n);return e}(this);case"column":return function(t){for(var e=de(t.columns,1),r=0;r<t.rows;++r)for(var n=0;n<t.columns;++n)e[n]*=t.get(r,n);return e}(this);case void 0:return function(t){for(var e=1,r=0;r<t.rows;r++)for(var n=0;n<t.columns;n++)e*=t.get(r,n);return e}(this);default:throw new Error("invalid option: ".concat(t))}}},{key:"mean",value:function(t){var e=this.sum(t);switch(t){case"row":for(var r=0;r<this.rows;r++)e[r]/=this.columns;return e;case"column":for(var n=0;n<this.columns;n++)e[n]/=this.rows;return e;case void 0:return e/this.size;default:throw new Error("invalid option: ".concat(t))}}},{key:"variance",value:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};if("object"===Ie(t)&&(e=t,t=void 0),"object"!==Ie(e))throw new TypeError("options must be an object");var r=e,n=r.unbiased,o=void 0===n||n,i=r.mean,a=void 0===i?this.mean(t):i;if("boolean"!=typeof o)throw new TypeError("unbiased must be a boolean");switch(t){case"row":if(!Array.isArray(a))throw new TypeError("mean must be an array");return ve(this,o,a);case"column":if(!Array.isArray(a))throw new TypeError("mean must be an array");return ge(this,o,a);case void 0:if("number"!=typeof a)throw new TypeError("mean must be a number");return ye(this,o,a);default:throw new Error("invalid option: ".concat(t))}}},{key:"standardDeviation",value:function(t,e){"object"===Ie(t)&&(e=t,t=void 0);var r=this.variance(t,e);if(void 0===t)return Math.sqrt(r);for(var n=0;n<r.length;n++)r[n]=Math.sqrt(r[n]);return r}},{key:"center",value:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};if("object"===Ie(t)&&(e=t,t=void 0),"object"!==Ie(e))throw new TypeError("options must be an object");var r=e,n=r.center,o=void 0===n?this.mean(t):n;switch(t){case"row":if(!Array.isArray(o))throw new TypeError("center must be an array");return me(this,o),this;case"column":if(!Array.isArray(o))throw new TypeError("center must be an array");return be(this,o),this;case void 0:if("number"!=typeof o)throw new TypeError("center must be a number");return we(this,o),this;default:throw new Error("invalid option: ".concat(t))}}},{key:"scale",value:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};if("object"===Ie(t)&&(e=t,t=void 0),"object"!==Ie(e))throw new TypeError("options must be an object");var r=e.scale;switch(t){case"row":if(void 0===r)r=xe(this);else if(!Array.isArray(r))throw new TypeError("scale must be an array");return Me(this,r),this;case"column":if(void 0===r)r=ke(this);else if(!Array.isArray(r))throw new TypeError("scale must be an array");return _e(this,r),this;case void 0:if(void 0===r)r=Se(this);else if("number"!=typeof r)throw new TypeError("scale must be a number");return Ee(this,r),this;default:throw new Error("invalid option: ".concat(t))}}},{key:"toString",value:function(t){return re(this,t)}},{key:"size",get:function(){return this.rows*this.columns}}],[{key:"from1DArray",value:function(t,e,r){if(t*e!==r.length)throw new RangeError("data length does not match given dimensions");for(var n=new qe(t,e),o=0;o<t;o++)for(var i=0;i<e;i++)n.set(o,i,r[o*e+i]);return n}},{key:"rowVector",value:function(t){for(var e=new qe(1,t.length),r=0;r<t.length;r++)e.set(0,r,t[r]);return e}},{key:"columnVector",value:function(t){for(var e=new qe(t.length,1),r=0;r<t.length;r++)e.set(r,0,t[r]);return e}},{key:"zeros",value:function(t,e){return new qe(t,e)}},{key:"ones",value:function(t,e){return new qe(t,e).fill(1)}},{key:"rand",value:function(t,e){var r=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};if("object"!==Ie(r))throw new TypeError("options must be an object");for(var n=r.random,o=void 0===n?Math.random:n,i=new qe(t,e),a=0;a<t;a++)for(var u=0;u<e;u++)i.set(a,u,o());return i}},{key:"randInt",value:function(t,e){var r=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};if("object"!==Ie(r))throw new TypeError("options must be an object");var n=r.min,o=void 0===n?0:n,i=r.max,a=void 0===i?1e3:i,u=r.random,s=void 0===u?Math.random:u;if(!Number.isInteger(o))throw new TypeError("min must be an integer");if(!Number.isInteger(a))throw new TypeError("max must be an integer");if(o>=a)throw new RangeError("min must be smaller than max");for(var c=a-o,f=new qe(t,e),h=0;h<t;h++)for(var l=0;l<e;l++){var d=o+Math.round(s()*c);f.set(h,l,d)}return f}},{key:"eye",value:function(t,e,r){void 0===e&&(e=t),void 0===r&&(r=1);for(var n=Math.min(t,e),o=this.zeros(t,e),i=0;i<n;i++)o.set(i,i,r);return o}},{key:"diag",value:function(t,e,r){var n=t.length;void 0===e&&(e=n),void 0===r&&(r=e);for(var o=Math.min(n,e,r),i=this.zeros(e,r),a=0;a<o;a++)i.set(a,a,t[a]);return i}},{key:"min",value:function(t,e){t=this.checkMatrix(t),e=this.checkMatrix(e);for(var r=t.rows,n=t.columns,o=new qe(r,n),i=0;i<r;i++)for(var a=0;a<n;a++)o.set(i,a,Math.min(t.get(i,a),e.get(i,a)));return o}},{key:"max",value:function(t,e){t=this.checkMatrix(t),e=this.checkMatrix(e);for(var r=t.rows,n=t.columns,o=new this(r,n),i=0;i<r;i++)for(var a=0;a<n;a++)o.set(i,a,Math.max(t.get(i,a),e.get(i,a)));return o}},{key:"checkMatrix",value:function(e){return t.isMatrix(e)?e:new qe(e)}},{key:"isMatrix",value:function(t){return null!=t&&"Matrix"===t.klass}}]),t}();function Fe(t,e){return t-e}ze.prototype.klass="Matrix","undefined"!=typeof Symbol&&(ze.prototype[Symbol.for("nodejs.util.inspect.custom")]=function(){return re(this)}),ze.random=ze.rand,ze.randomInt=ze.randInt,ze.diagonal=ze.diag,ze.prototype.diagonal=ze.prototype.diag,ze.identity=ze.eye,ze.prototype.negate=ze.prototype.neg,ze.prototype.tensorProduct=ze.prototype.kroneckerProduct;var Te,De,qe=function(t){!function(t,e){if("function"!=typeof e&&null!==e)throw new TypeError("Super expression must either be null or a function");t.prototype=Object.create(e&&e.prototype,{constructor:{value:t,writable:!0,configurable:!0}}),e&&je(t,e)}(r,t);var e=Ae(r);function r(t,n){var o;if(Re(this,r),o=e.call(this),r.isMatrix(t))return Oe(o,t.clone());if(Number.isInteger(t)&&t>0){if(o.data=[],!(Number.isInteger(n)&&n>0))throw new TypeError("nColumns must be a positive integer");for(var i=0;i<t;i++)o.data.push(new Float64Array(n))}else{if(!Array.isArray(t))throw new TypeError("First argument must be a positive number or an array");var a=t;if(t=a.length,"number"!=typeof(n=a[0].length)||0===n)throw new TypeError("Data must be a 2D array with at least one element");o.data=[];for(var u=0;u<t;u++){if(a[u].length!==n)throw new RangeError("Inconsistent array dimensions");o.data.push(Float64Array.from(a[u]))}}return o.rows=t,o.columns=n,o}return Ce(r,[{key:"set",value:function(t,e,r){return this.data[t][e]=r,this}},{key:"get",value:function(t,e){return this.data[t][e]}},{key:"removeRow",value:function(t){if(ae(this,t),1===this.rows)throw new RangeError("A matrix cannot have less than one row");return this.data.splice(t,1),this.rows-=1,this}},{key:"addRow",value:function(t,e){return void 0===e&&(e=t,t=this.rows),ae(this,t,!0),e=Float64Array.from(se(this,e)),this.data.splice(t,0,e),this.rows+=1,this}},{key:"removeColumn",value:function(t){if(ue(this,t),1===this.columns)throw new RangeError("A matrix cannot have less than one column");for(var e=0;e<this.rows;e++){for(var r=new Float64Array(this.columns-1),n=0;n<t;n++)r[n]=this.data[e][n];for(var o=t+1;o<this.columns;o++)r[o-1]=this.data[e][o];this.data[e]=r}return this.columns-=1,this}},{key:"addColumn",value:function(t,e){void 0===e&&(e=t,t=this.columns),ue(this,t,!0),e=ce(this,e);for(var r=0;r<this.rows;r++){for(var n=new Float64Array(this.columns+1),o=0;o<t;o++)n[o]=this.data[r][o];for(n[o++]=e[r];o<this.columns+1;o++)n[o]=this.data[r][o-1];this.data[r]=n}return this.columns+=1,this}}]),r}(ze);function Le(t){return(Le="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(t){return typeof t}:function(t){return t&&"function"==typeof Symbol&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t})(t)}function Be(t,e){for(var r=0;r<e.length;r++){var n=e[r];n.enumerable=n.enumerable||!1,n.configurable=!0,"value"in n&&(n.writable=!0),Object.defineProperty(t,n.key,n)}}function Ve(t,e){return(Ve=Object.setPrototypeOf||function(t,e){return t.__proto__=e,t})(t,e)}function Ue(t){var e=function(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],(function(){}))),!0}catch(t){return!1}}();return function(){var r,n=He(t);if(e){var o=He(this).constructor;r=Reflect.construct(n,arguments,o)}else r=n.apply(this,arguments);return Ye(this,r)}}function Ye(t,e){return!e||"object"!==Le(e)&&"function"!=typeof e?function(t){if(void 0===t)throw new ReferenceError("this hasn\'t been initialised - super() hasn\'t been called");return t}(t):e}function He(t){return(He=Object.setPrototypeOf?Object.getPrototypeOf:function(t){return t.__proto__||Object.getPrototypeOf(t)})(t)}De=qe,(Te=ze).prototype.add=function(t){return"number"==typeof t?this.addS(t):this.addM(t)},Te.prototype.addS=function(t){for(var e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)this.set(e,r,this.get(e,r)+t);return this},Te.prototype.addM=function(t){if(t=De.checkMatrix(t),this.rows!==t.rows||this.columns!==t.columns)throw new RangeError("Matrices dimensions must be equal");for(var e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)this.set(e,r,this.get(e,r)+t.get(e,r));return this},Te.add=function(t,e){return new De(t).add(e)},Te.prototype.sub=function(t){return"number"==typeof t?this.subS(t):this.subM(t)},Te.prototype.subS=function(t){for(var e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)this.set(e,r,this.get(e,r)-t);return this},Te.prototype.subM=function(t){if(t=De.checkMatrix(t),this.rows!==t.rows||this.columns!==t.columns)throw new RangeError("Matrices dimensions must be equal");for(var e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)this.set(e,r,this.get(e,r)-t.get(e,r));return this},Te.sub=function(t,e){return new De(t).sub(e)},Te.prototype.subtract=Te.prototype.sub,Te.prototype.subtractS=Te.prototype.subS,Te.prototype.subtractM=Te.prototype.subM,Te.subtract=Te.sub,Te.prototype.mul=function(t){return"number"==typeof t?this.mulS(t):this.mulM(t)},Te.prototype.mulS=function(t){for(var e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)this.set(e,r,this.get(e,r)*t);return this},Te.prototype.mulM=function(t){if(t=De.checkMatrix(t),this.rows!==t.rows||this.columns!==t.columns)throw new RangeError("Matrices dimensions must be equal");for(var e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)this.set(e,r,this.get(e,r)*t.get(e,r));return this},Te.mul=function(t,e){return new De(t).mul(e)},Te.prototype.multiply=Te.prototype.mul,Te.prototype.multiplyS=Te.prototype.mulS,Te.prototype.multiplyM=Te.prototype.mulM,Te.multiply=Te.mul,Te.prototype.div=function(t){return"number"==typeof t?this.divS(t):this.divM(t)},Te.prototype.divS=function(t){for(var e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)this.set(e,r,this.get(e,r)/t);return this},Te.prototype.divM=function(t){if(t=De.checkMatrix(t),this.rows!==t.rows||this.columns!==t.columns)throw new RangeError("Matrices dimensions must be equal");for(var e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)this.set(e,r,this.get(e,r)/t.get(e,r));return this},Te.div=function(t,e){return new De(t).div(e)},Te.prototype.divide=Te.prototype.div,Te.prototype.divideS=Te.prototype.divS,Te.prototype.divideM=Te.prototype.divM,Te.divide=Te.div,Te.prototype.mod=function(t){return"number"==typeof t?this.modS(t):this.modM(t)},Te.prototype.modS=function(t){for(var e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)this.set(e,r,this.get(e,r)%t);return this},Te.prototype.modM=function(t){if(t=De.checkMatrix(t),this.rows!==t.rows||this.columns!==t.columns)throw new RangeError("Matrices dimensions must be equal");for(var e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)this.set(e,r,this.get(e,r)%t.get(e,r));return this},Te.mod=function(t,e){return new De(t).mod(e)},Te.prototype.modulus=Te.prototype.mod,Te.prototype.modulusS=Te.prototype.modS,Te.prototype.modulusM=Te.prototype.modM,Te.modulus=Te.mod,Te.prototype.and=function(t){return"number"==typeof t?this.andS(t):this.andM(t)},Te.prototype.andS=function(t){for(var e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)this.set(e,r,this.get(e,r)&t);return this},Te.prototype.andM=function(t){if(t=De.checkMatrix(t),this.rows!==t.rows||this.columns!==t.columns)throw new RangeError("Matrices dimensions must be equal");for(var e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)this.set(e,r,this.get(e,r)&t.get(e,r));return this},Te.and=function(t,e){return new De(t).and(e)},Te.prototype.or=function(t){return"number"==typeof t?this.orS(t):this.orM(t)},Te.prototype.orS=function(t){for(var e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)this.set(e,r,this.get(e,r)|t);return this},Te.prototype.orM=function(t){if(t=De.checkMatrix(t),this.rows!==t.rows||this.columns!==t.columns)throw new RangeError("Matrices dimensions must be equal");for(var e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)this.set(e,r,this.get(e,r)|t.get(e,r));return this},Te.or=function(t,e){return new De(t).or(e)},Te.prototype.xor=function(t){return"number"==typeof t?this.xorS(t):this.xorM(t)},Te.prototype.xorS=function(t){for(var e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)this.set(e,r,this.get(e,r)^t);return this},Te.prototype.xorM=function(t){if(t=De.checkMatrix(t),this.rows!==t.rows||this.columns!==t.columns)throw new RangeError("Matrices dimensions must be equal");for(var e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)this.set(e,r,this.get(e,r)^t.get(e,r));return this},Te.xor=function(t,e){return new De(t).xor(e)},Te.prototype.leftShift=function(t){return"number"==typeof t?this.leftShiftS(t):this.leftShiftM(t)},Te.prototype.leftShiftS=function(t){for(var e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)this.set(e,r,this.get(e,r)<<t);return this},Te.prototype.leftShiftM=function(t){if(t=De.checkMatrix(t),this.rows!==t.rows||this.columns!==t.columns)throw new RangeError("Matrices dimensions must be equal");for(var e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)this.set(e,r,this.get(e,r)<<t.get(e,r));return this},Te.leftShift=function(t,e){return new De(t).leftShift(e)},Te.prototype.signPropagatingRightShift=function(t){return"number"==typeof t?this.signPropagatingRightShiftS(t):this.signPropagatingRightShiftM(t)},Te.prototype.signPropagatingRightShiftS=function(t){for(var e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)this.set(e,r,this.get(e,r)>>t);return this},Te.prototype.signPropagatingRightShiftM=function(t){if(t=De.checkMatrix(t),this.rows!==t.rows||this.columns!==t.columns)throw new RangeError("Matrices dimensions must be equal");for(var e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)this.set(e,r,this.get(e,r)>>t.get(e,r));return this},Te.signPropagatingRightShift=function(t,e){return new De(t).signPropagatingRightShift(e)},Te.prototype.rightShift=function(t){return"number"==typeof t?this.rightShiftS(t):this.rightShiftM(t)},Te.prototype.rightShiftS=function(t){for(var e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)this.set(e,r,this.get(e,r)>>>t);return this},Te.prototype.rightShiftM=function(t){if(t=De.checkMatrix(t),this.rows!==t.rows||this.columns!==t.columns)throw new RangeError("Matrices dimensions must be equal");for(var e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)this.set(e,r,this.get(e,r)>>>t.get(e,r));return this},Te.rightShift=function(t,e){return new De(t).rightShift(e)},Te.prototype.zeroFillRightShift=Te.prototype.rightShift,Te.prototype.zeroFillRightShiftS=Te.prototype.rightShiftS,Te.prototype.zeroFillRightShiftM=Te.prototype.rightShiftM,Te.zeroFillRightShift=Te.rightShift,Te.prototype.not=function(){for(var t=0;t<this.rows;t++)for(var e=0;e<this.columns;e++)this.set(t,e,~this.get(t,e));return this},Te.not=function(t){return new De(t).not()},Te.prototype.abs=function(){for(var t=0;t<this.rows;t++)for(var e=0;e<this.columns;e++)this.set(t,e,Math.abs(this.get(t,e)));return this},Te.abs=function(t){return new De(t).abs()},Te.prototype.acos=function(){for(var t=0;t<this.rows;t++)for(var e=0;e<this.columns;e++)this.set(t,e,Math.acos(this.get(t,e)));return this},Te.acos=function(t){return new De(t).acos()},Te.prototype.acosh=function(){for(var t=0;t<this.rows;t++)for(var e=0;e<this.columns;e++)this.set(t,e,Math.acosh(this.get(t,e)));return this},Te.acosh=function(t){return new De(t).acosh()},Te.prototype.asin=function(){for(var t=0;t<this.rows;t++)for(var e=0;e<this.columns;e++)this.set(t,e,Math.asin(this.get(t,e)));return this},Te.asin=function(t){return new De(t).asin()},Te.prototype.asinh=function(){for(var t=0;t<this.rows;t++)for(var e=0;e<this.columns;e++)this.set(t,e,Math.asinh(this.get(t,e)));return this},Te.asinh=function(t){return new De(t).asinh()},Te.prototype.atan=function(){for(var t=0;t<this.rows;t++)for(var e=0;e<this.columns;e++)this.set(t,e,Math.atan(this.get(t,e)));return this},Te.atan=function(t){return new De(t).atan()},Te.prototype.atanh=function(){for(var t=0;t<this.rows;t++)for(var e=0;e<this.columns;e++)this.set(t,e,Math.atanh(this.get(t,e)));return this},Te.atanh=function(t){return new De(t).atanh()},Te.prototype.cbrt=function(){for(var t=0;t<this.rows;t++)for(var e=0;e<this.columns;e++)this.set(t,e,Math.cbrt(this.get(t,e)));return this},Te.cbrt=function(t){return new De(t).cbrt()},Te.prototype.ceil=function(){for(var t=0;t<this.rows;t++)for(var e=0;e<this.columns;e++)this.set(t,e,Math.ceil(this.get(t,e)));return this},Te.ceil=function(t){return new De(t).ceil()},Te.prototype.clz32=function(){for(var t=0;t<this.rows;t++)for(var e=0;e<this.columns;e++)this.set(t,e,Math.clz32(this.get(t,e)));return this},Te.clz32=function(t){return new De(t).clz32()},Te.prototype.cos=function(){for(var t=0;t<this.rows;t++)for(var e=0;e<this.columns;e++)this.set(t,e,Math.cos(this.get(t,e)));return this},Te.cos=function(t){return new De(t).cos()},Te.prototype.cosh=function(){for(var t=0;t<this.rows;t++)for(var e=0;e<this.columns;e++)this.set(t,e,Math.cosh(this.get(t,e)));return this},Te.cosh=function(t){return new De(t).cosh()},Te.prototype.exp=function(){for(var t=0;t<this.rows;t++)for(var e=0;e<this.columns;e++)this.set(t,e,Math.exp(this.get(t,e)));return this},Te.exp=function(t){return new De(t).exp()},Te.prototype.expm1=function(){for(var t=0;t<this.rows;t++)for(var e=0;e<this.columns;e++)this.set(t,e,Math.expm1(this.get(t,e)));return this},Te.expm1=function(t){return new De(t).expm1()},Te.prototype.floor=function(){for(var t=0;t<this.rows;t++)for(var e=0;e<this.columns;e++)this.set(t,e,Math.floor(this.get(t,e)));return this},Te.floor=function(t){return new De(t).floor()},Te.prototype.fround=function(){for(var t=0;t<this.rows;t++)for(var e=0;e<this.columns;e++)this.set(t,e,Math.fround(this.get(t,e)));return this},Te.fround=function(t){return new De(t).fround()},Te.prototype.log=function(){for(var t=0;t<this.rows;t++)for(var e=0;e<this.columns;e++)this.set(t,e,Math.log(this.get(t,e)));return this},Te.log=function(t){return new De(t).log()},Te.prototype.log1p=function(){for(var t=0;t<this.rows;t++)for(var e=0;e<this.columns;e++)this.set(t,e,Math.log1p(this.get(t,e)));return this},Te.log1p=function(t){return new De(t).log1p()},Te.prototype.log10=function(){for(var t=0;t<this.rows;t++)for(var e=0;e<this.columns;e++)this.set(t,e,Math.log10(this.get(t,e)));return this},Te.log10=function(t){return new De(t).log10()},Te.prototype.log2=function(){for(var t=0;t<this.rows;t++)for(var e=0;e<this.columns;e++)this.set(t,e,Math.log2(this.get(t,e)));return this},Te.log2=function(t){return new De(t).log2()},Te.prototype.round=function(){for(var t=0;t<this.rows;t++)for(var e=0;e<this.columns;e++)this.set(t,e,Math.round(this.get(t,e)));return this},Te.round=function(t){return new De(t).round()},Te.prototype.sign=function(){for(var t=0;t<this.rows;t++)for(var e=0;e<this.columns;e++)this.set(t,e,Math.sign(this.get(t,e)));return this},Te.sign=function(t){return new De(t).sign()},Te.prototype.sin=function(){for(var t=0;t<this.rows;t++)for(var e=0;e<this.columns;e++)this.set(t,e,Math.sin(this.get(t,e)));return this},Te.sin=function(t){return new De(t).sin()},Te.prototype.sinh=function(){for(var t=0;t<this.rows;t++)for(var e=0;e<this.columns;e++)this.set(t,e,Math.sinh(this.get(t,e)));return this},Te.sinh=function(t){return new De(t).sinh()},Te.prototype.sqrt=function(){for(var t=0;t<this.rows;t++)for(var e=0;e<this.columns;e++)this.set(t,e,Math.sqrt(this.get(t,e)));return this},Te.sqrt=function(t){return new De(t).sqrt()},Te.prototype.tan=function(){for(var t=0;t<this.rows;t++)for(var e=0;e<this.columns;e++)this.set(t,e,Math.tan(this.get(t,e)));return this},Te.tan=function(t){return new De(t).tan()},Te.prototype.tanh=function(){for(var t=0;t<this.rows;t++)for(var e=0;e<this.columns;e++)this.set(t,e,Math.tanh(this.get(t,e)));return this},Te.tanh=function(t){return new De(t).tanh()},Te.prototype.trunc=function(){for(var t=0;t<this.rows;t++)for(var e=0;e<this.columns;e++)this.set(t,e,Math.trunc(this.get(t,e)));return this},Te.trunc=function(t){return new De(t).trunc()},Te.pow=function(t,e){return new De(t).pow(e)},Te.prototype.pow=function(t){return"number"==typeof t?this.powS(t):this.powM(t)},Te.prototype.powS=function(t){for(var e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)this.set(e,r,Math.pow(this.get(e,r),t));return this},Te.prototype.powM=function(t){if(t=De.checkMatrix(t),this.rows!==t.rows||this.columns!==t.columns)throw new RangeError("Matrices dimensions must be equal");for(var e=0;e<this.rows;e++)for(var r=0;r<this.columns;r++)this.set(e,r,Math.pow(this.get(e,r),t.get(e,r)));return this};var We=function(t){!function(t,e){if("function"!=typeof e&&null!==e)throw new TypeError("Super expression must either be null or a function");t.prototype=Object.create(e&&e.prototype,{constructor:{value:t,writable:!0,configurable:!0}}),e&&Ve(t,e)}(i,t);var e,r,n,o=Ue(i);function i(t){var e;return function(t,e){if(!(t instanceof e))throw new TypeError("Cannot call a class as a function")}(this,i),(e=o.call(this)).data=t,e.rows=t.length,e.columns=t[0].length,e}return e=i,(r=[{key:"set",value:function(t,e,r){return this.data[t][e]=r,this}},{key:"get",value:function(t,e){return this.data[t][e]}}])&&Be(e.prototype,r),n&&Be(e,n),i}(ze);function Ge(t,e){var r=0;return Math.abs(t)>Math.abs(e)?(r=e/t,Math.abs(t)*Math.sqrt(1+r*r)):0!==e?(r=t/e,Math.abs(e)*Math.sqrt(1+r*r)):0}function $e(t,e){if(!(t instanceof e))throw new TypeError("Cannot call a class as a function")}function Xe(t,e){for(var r=0;r<e.length;r++){var n=e[r];n.enumerable=n.enumerable||!1,n.configurable=!0,"value"in n&&(n.writable=!0),Object.defineProperty(t,n.key,n)}}var Je=function(){function t(e){var r=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};$e(this,t);var n,o=(e=We.checkMatrix(e)).rows,i=e.columns,a=r.computeLeftSingularVectors,u=void 0===a||a,s=r.computeRightSingularVectors,c=void 0===s||s,f=r.autoTranspose,h=void 0!==f&&f,l=Boolean(u),d=Boolean(c),p=!1;if(o<i)if(h){o=(n=e.transpose()).rows,i=n.columns,p=!0;var v=l;l=d,d=v}else n=e.clone(),console.warn("Computing SVD on a matrix with more columns than rows. Consider enabling autoTranspose");else n=e.clone();for(var g=Math.min(o,i),y=Math.min(o+1,i),m=new Float64Array(y),b=new qe(o,g),w=new qe(i,i),x=new Float64Array(i),M=new Float64Array(o),k=new Float64Array(y),_=0;_<y;_++)k[_]=_;for(var S=Math.min(o-1,i),E=Math.max(0,Math.min(i-2,o)),j=Math.max(S,E),A=0;A<j;A++){if(A<S){m[A]=0;for(var O=A;O<o;O++)m[A]=Ge(m[A],n.get(O,A));if(0!==m[A]){n.get(A,A)<0&&(m[A]=-m[A]);for(var N=A;N<o;N++)n.set(N,A,n.get(N,A)/m[A]);n.set(A,A,n.get(A,A)+1)}m[A]=-m[A]}for(var I=A+1;I<i;I++){if(A<S&&0!==m[A]){for(var R=0,P=A;P<o;P++)R+=n.get(P,A)*n.get(P,I);R=-R/n.get(A,A);for(var C=A;C<o;C++)n.set(C,I,n.get(C,I)+R*n.get(C,A))}x[I]=n.get(A,I)}if(l&&A<S)for(var z=A;z<o;z++)b.set(z,A,n.get(z,A));if(A<E){x[A]=0;for(var F=A+1;F<i;F++)x[A]=Ge(x[A],x[F]);if(0!==x[A]){x[A+1]<0&&(x[A]=0-x[A]);for(var T=A+1;T<i;T++)x[T]/=x[A];x[A+1]+=1}if(x[A]=-x[A],A+1<o&&0!==x[A]){for(var D=A+1;D<o;D++)M[D]=0;for(var q=A+1;q<o;q++)for(var L=A+1;L<i;L++)M[q]+=x[L]*n.get(q,L);for(var B=A+1;B<i;B++)for(var V=-x[B]/x[A+1],U=A+1;U<o;U++)n.set(U,B,n.get(U,B)+V*M[U])}if(d)for(var Y=A+1;Y<i;Y++)w.set(Y,A,x[Y])}}var H=Math.min(i,o+1);if(S<i&&(m[S]=n.get(S,S)),o<H&&(m[H-1]=0),E+1<H&&(x[E]=n.get(E,H-1)),x[H-1]=0,l){for(var W=S;W<g;W++){for(var G=0;G<o;G++)b.set(G,W,0);b.set(W,W,1)}for(var $=S-1;$>=0;$--)if(0!==m[$]){for(var X=$+1;X<g;X++){for(var J=0,Q=$;Q<o;Q++)J+=b.get(Q,$)*b.get(Q,X);J=-J/b.get($,$);for(var K=$;K<o;K++)b.set(K,X,b.get(K,X)+J*b.get(K,$))}for(var Z=$;Z<o;Z++)b.set(Z,$,-b.get(Z,$));b.set($,$,1+b.get($,$));for(var tt=0;tt<$-1;tt++)b.set(tt,$,0)}else{for(var et=0;et<o;et++)b.set(et,$,0);b.set($,$,1)}}if(d)for(var rt=i-1;rt>=0;rt--){if(rt<E&&0!==x[rt])for(var nt=rt+1;nt<i;nt++){for(var ot=0,it=rt+1;it<i;it++)ot+=w.get(it,rt)*w.get(it,nt);ot=-ot/w.get(rt+1,rt);for(var at=rt+1;at<i;at++)w.set(at,nt,w.get(at,nt)+ot*w.get(at,rt))}for(var ut=0;ut<i;ut++)w.set(ut,rt,0);w.set(rt,rt,1)}for(var st=H-1,ct=Number.EPSILON;H>0;){var ft=void 0,ht=void 0;for(ft=H-2;ft>=-1&&-1!==ft;ft--){var lt=Number.MIN_VALUE+ct*Math.abs(m[ft]+Math.abs(m[ft+1]));if(Math.abs(x[ft])<=lt||Number.isNaN(x[ft])){x[ft]=0;break}}if(ft===H-2)ht=4;else{var dt=void 0;for(dt=H-1;dt>=ft&&dt!==ft;dt--){var pt=(dt!==H?Math.abs(x[dt]):0)+(dt!==ft+1?Math.abs(x[dt-1]):0);if(Math.abs(m[dt])<=ct*pt){m[dt]=0;break}}dt===ft?ht=3:dt===H-1?ht=1:(ht=2,ft=dt)}switch(ft++,ht){case 1:var vt=x[H-2];x[H-2]=0;for(var gt=H-2;gt>=ft;gt--){var yt=Ge(m[gt],vt),mt=m[gt]/yt,bt=vt/yt;if(m[gt]=yt,gt!==ft&&(vt=-bt*x[gt-1],x[gt-1]=mt*x[gt-1]),d)for(var wt=0;wt<i;wt++)yt=mt*w.get(wt,gt)+bt*w.get(wt,H-1),w.set(wt,H-1,-bt*w.get(wt,gt)+mt*w.get(wt,H-1)),w.set(wt,gt,yt)}break;case 2:var xt=x[ft-1];x[ft-1]=0;for(var Mt=ft;Mt<H;Mt++){var kt=Ge(m[Mt],xt),_t=m[Mt]/kt,St=xt/kt;if(m[Mt]=kt,xt=-St*x[Mt],x[Mt]=_t*x[Mt],l)for(var Et=0;Et<o;Et++)kt=_t*b.get(Et,Mt)+St*b.get(Et,ft-1),b.set(Et,ft-1,-St*b.get(Et,Mt)+_t*b.get(Et,ft-1)),b.set(Et,Mt,kt)}break;case 3:var jt=Math.max(Math.abs(m[H-1]),Math.abs(m[H-2]),Math.abs(x[H-2]),Math.abs(m[ft]),Math.abs(x[ft])),At=m[H-1]/jt,Ot=m[H-2]/jt,Nt=x[H-2]/jt,It=m[ft]/jt,Rt=x[ft]/jt,Pt=((Ot+At)*(Ot-At)+Nt*Nt)/2,Ct=At*Nt*(At*Nt),zt=0;0===Pt&&0===Ct||(zt=Ct/(Pt+(zt=Pt<0?0-Math.sqrt(Pt*Pt+Ct):Math.sqrt(Pt*Pt+Ct))));for(var Ft=(It+At)*(It-At)+zt,Tt=It*Rt,Dt=ft;Dt<H-1;Dt++){var qt=Ge(Ft,Tt);0===qt&&(qt=Number.MIN_VALUE);var Lt=Ft/qt,Bt=Tt/qt;if(Dt!==ft&&(x[Dt-1]=qt),Ft=Lt*m[Dt]+Bt*x[Dt],x[Dt]=Lt*x[Dt]-Bt*m[Dt],Tt=Bt*m[Dt+1],m[Dt+1]=Lt*m[Dt+1],d)for(var Vt=0;Vt<i;Vt++)qt=Lt*w.get(Vt,Dt)+Bt*w.get(Vt,Dt+1),w.set(Vt,Dt+1,-Bt*w.get(Vt,Dt)+Lt*w.get(Vt,Dt+1)),w.set(Vt,Dt,qt);if(0===(qt=Ge(Ft,Tt))&&(qt=Number.MIN_VALUE),Lt=Ft/qt,Bt=Tt/qt,m[Dt]=qt,Ft=Lt*x[Dt]+Bt*m[Dt+1],m[Dt+1]=-Bt*x[Dt]+Lt*m[Dt+1],Tt=Bt*x[Dt+1],x[Dt+1]=Lt*x[Dt+1],l&&Dt<o-1)for(var Ut=0;Ut<o;Ut++)qt=Lt*b.get(Ut,Dt)+Bt*b.get(Ut,Dt+1),b.set(Ut,Dt+1,-Bt*b.get(Ut,Dt)+Lt*b.get(Ut,Dt+1)),b.set(Ut,Dt,qt)}x[H-2]=Ft,1;break;case 4:if(m[ft]<=0&&(m[ft]=m[ft]<0?-m[ft]:0,d))for(var Yt=0;Yt<=st;Yt++)w.set(Yt,ft,-w.get(Yt,ft));for(;ft<st&&!(m[ft]>=m[ft+1]);){var Ht=m[ft];if(m[ft]=m[ft+1],m[ft+1]=Ht,d&&ft<i-1)for(var Wt=0;Wt<i;Wt++)Ht=w.get(Wt,ft+1),w.set(Wt,ft+1,w.get(Wt,ft)),w.set(Wt,ft,Ht);if(l&&ft<o-1)for(var Gt=0;Gt<o;Gt++)Ht=b.get(Gt,ft+1),b.set(Gt,ft+1,b.get(Gt,ft)),b.set(Gt,ft,Ht);ft++}0,H--}}if(p){var $t=w;w=b,b=$t}this.m=o,this.n=i,this.s=m,this.U=b,this.V=w}var e,r,n;return e=t,(r=[{key:"solve",value:function(t){for(var e=t,r=this.threshold,n=this.s.length,o=qe.zeros(n,n),i=0;i<n;i++)Math.abs(this.s[i])<=r?o.set(i,i,0):o.set(i,i,1/this.s[i]);for(var a=this.U,u=this.rightSingularVectors,s=u.mmul(o),c=u.rows,f=a.rows,h=qe.zeros(c,f),l=0;l<c;l++)for(var d=0;d<f;d++){for(var p=0,v=0;v<n;v++)p+=s.get(l,v)*a.get(d,v);h.set(l,d,p)}return h.mmul(e)}},{key:"solveForDiagonal",value:function(t){return this.solve(qe.diag(t))}},{key:"inverse",value:function(){for(var t=this.V,e=this.threshold,r=t.rows,n=t.columns,o=new qe(r,this.s.length),i=0;i<r;i++)for(var a=0;a<n;a++)Math.abs(this.s[a])>e&&o.set(i,a,t.get(i,a)/this.s[a]);for(var u=this.U,s=u.rows,c=u.columns,f=new qe(r,s),h=0;h<r;h++)for(var l=0;l<s;l++){for(var d=0,p=0;p<c;p++)d+=o.get(h,p)*u.get(l,p);f.set(h,l,d)}return f}},{key:"condition",get:function(){return this.s[0]/this.s[Math.min(this.m,this.n)-1]}},{key:"norm2",get:function(){return this.s[0]}},{key:"rank",get:function(){for(var t=Math.max(this.m,this.n)*this.s[0]*Number.EPSILON,e=0,r=this.s,n=0,o=r.length;n<o;n++)r[n]>t&&e++;return e}},{key:"diagonal",get:function(){return Array.from(this.s)}},{key:"threshold",get:function(){return Number.EPSILON/2*Math.max(this.m,this.n)*this.s[0]}},{key:"leftSingularVectors",get:function(){return this.U}},{key:"rightSingularVectors",get:function(){return this.V}},{key:"diagonalMatrix",get:function(){return qe.diag(this.s)}}])&&Xe(e.prototype,r),n&&Xe(e,n),t}(),Qe=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.center=[0,0],e.linkDistance=50,e.scaledDistances=null,e}return a(e,t),e.prototype.getDefaultCfg=function(){return{center:[0,0],linkDistance:50}},e.prototype.execute=function(){var t=this.nodes,e=this.edges,r=void 0===e?[]:e,n=this.center;if(t&&0!==t.length){if(1===t.length)return t[0].x=n[0],void(t[0].y=n[1]);var o=this.linkDistance,i=O({nodes:t,edges:r},!1),a=A(i);this.handleInfinity(a);var u,s,c=(u=o,s=[],a.forEach((function(t){var e=[];t.forEach((function(t){e.push(t*u)})),s.push(e)})),s);this.scaledDistances=c;var f=this.runMDS();this.positions=f,f.forEach((function(e,r){t[r].x=e[0]+n[0],t[r].y=e[1]+n[1]}))}},e.prototype.runMDS=function(){var t=this.scaledDistances,e=qe.mul(qe.pow(t,2),-.5),r=e.mean("row"),n=e.mean("column"),o=e.mean();e.add(o).subRowVector(r).subColumnVector(n);var i=new Je(e),a=qe.sqrt(i.diagonalMatrix).diagonal();return i.leftSingularVectors.toJSON().map((function(t){return qe.mul([t],[a]).toJSON()[0].splice(0,2)}))},e.prototype.handleInfinity=function(t){var e=-999999;t.forEach((function(t){t.forEach((function(t){t!==1/0&&e<t&&(e=t)}))})),t.forEach((function(r,n){r.forEach((function(r,o){r===1/0&&(t[n][o]=e)}))}))},e}(c),Ke=function(){function t(t){this.distances=t.distances,this.dimension=t.dimension||2,this.linkDistance=t.linkDistance}return t.prototype.layout=function(){var t=this.dimension,e=this.distances,r=this.linkDistance;try{var n=qe.mul(qe.pow(e,2),-.5),o=n.mean("row"),i=n.mean("column"),a=n.mean();n.add(a).subRowVector(o).subColumnVector(i);var u=new Je(n),s=qe.sqrt(u.diagonalMatrix).diagonal();return u.leftSingularVectors.toJSON().map((function(e){return qe.mul([e],[s]).toJSON()[0].splice(0,t)}))}catch(t){for(var c=[],f=0;f<e.length;f++){var h=Math.random()*r,l=Math.random()*r;c.push([h,l])}return c}},t}(),Ze=function(){function t(t){this.disp=[],this.positions=t.positions,this.adjMatrix=t.adjMatrix,this.focusID=t.focusID,this.radii=t.radii,this.iterations=t.iterations||10,this.height=t.height||10,this.width=t.width||10,this.speed=t.speed||100,this.gravity=t.gravity||10,this.nodeSizeFunc=t.nodeSizeFunc,this.k=t.k||5,this.strictRadial=t.strictRadial,this.nodes=t.nodes}return t.prototype.layout=function(){var t=this.positions,e=[],r=this.iterations,n=this.width/10;this.maxDisplace=n,this.disp=e;for(var o=0;o<r;o++)t.forEach((function(t,r){e[r]={x:0,y:0}})),this.getRepulsion(),this.updatePositions();return t},t.prototype.getRepulsion=function(){var t=this,e=t.positions,r=t.nodes,n=t.disp,o=t.k,i=t.radii||[];e.forEach((function(a,u){n[u]={x:0,y:0},e.forEach((function(e,s){if(u!==s&&i[u]===i[s]){var c=a[0]-e[0],f=a[1]-e[1],h=Math.sqrt(c*c+f*f);if(0===h){h=1;var l=u>s?1:-1;c=.01*l,f=.01*l}if(h<t.nodeSizeFunc(r[u])/2+t.nodeSizeFunc(r[s])/2){var d=o*o/h;n[u].x+=c/h*d,n[u].y+=f/h*d}}}))}))},t.prototype.updatePositions=function(){var t=this.positions,e=this.disp,r=this.speed,n=this.strictRadial,o=this.focusID,i=this.maxDisplace||this.width/10;n&&e.forEach((function(e,r){var n=t[r][0]-t[o][0],i=t[r][1]-t[o][1],a=Math.sqrt(n*n+i*i),u=i/a,s=-n/a,c=Math.sqrt(e.x*e.x+e.y*e.y),f=Math.acos((u*e.x+s*e.y)/c);f>Math.PI/2&&(f-=Math.PI/2,u*=-1,s*=-1);var h=Math.cos(f)*c;e.x=u*h,e.y=s*h}));var a=this.radii;t.forEach((function(u,s){if(s!==o){var c=Math.sqrt(e[s].x*e[s].x+e[s].y*e[s].y);if(c>0&&s!==o){var f=Math.min(i*(r/800),c);if(u[0]+=e[s].x/c*f,u[1]+=e[s].y/c*f,n){var h=u[0]-t[o][0],l=u[1]-t[o][1],d=Math.sqrt(h*h+l*l);h=h/d*a[s],l=l/d*a[s],u[0]=t[o][0]+h,u[1]=t[o][1]+l}}}}))},t}();function tr(t,e){return Math.sqrt((t[0]-e[0])*(t[0]-e[0])+(t[1]-e[1])*(t[1]-e[1]))}var er=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.center=[0,0],e.maxIteration=1e3,e.focusNode=null,e.unitRadius=null,e.linkDistance=50,e.preventOverlap=!1,e.strictRadial=!0,e.maxPreventOverlapIteration=200,e.sortStrength=10,e}return a(e,t),e.prototype.getDefaultCfg=function(){return{center:[0,0],maxIteration:1e3,focusNode:null,unitRadius:null,linkDistance:50,preventOverlap:!1,nodeSize:void 0,nodeSpacing:void 0,strictRadial:!0,maxPreventOverlapIteration:200,sortBy:void 0,sortStrength:10}},e.prototype.execute=function(){var t=this,e=t.nodes,r=t.edges||[],n=t.center;if(e&&0!==e.length){if(1===e.length)return e[0].x=n[0],void(e[0].y=n[1]);var o=t.linkDistance,i=null;if(C()(t.focusNode)){for(var a=!1,u=0;u<e.length;u++)e[u].id===t.focusNode&&(i=e[u],t.focusNode=i,a=!0,u=e.length);a||(i=null)}else i=t.focusNode;i||(i=e[0],t.focusNode=i);var s,c,f,h=(s=e,c=i.id,f=-1,s.forEach((function(t,e){t.id===c&&(f=e)})),f);t.focusIndex=h;var l=O({nodes:e,edges:r},!1),d=A(l),p=t.maxToFocus(d,h);t.handleInfinity(d,h,p+1),t.distances=d;var v=d[h];t.width||"undefined"==typeof window||(t.width=window.innerWidth),t.height||"undefined"==typeof window||(t.height=window.innerHeight);var g=t.width||500,y=t.height||500,m=g-n[0]>n[0]?n[0]:g-n[0],b=y-n[1]>n[1]?n[1]:y-n[1];0===m&&(m=g/2),0===b&&(b=y/2);var w=b>m?m:b,x=Math.max.apply(Math,v),M=[];v.forEach((function(e,r){t.unitRadius||(t.unitRadius=w/x),M[r]=e*t.unitRadius})),t.radii=M;var k=t.eIdealDisMatrix();t.eIdealDistances=k;var _=function(t){for(var e=t.length,r=t[0].length,n=[],o=0;o<e;o++){for(var i=[],a=0;a<r;a++)0!==t[o][a]?i.push(1/(t[o][a]*t[o][a])):i.push(0);n.push(i)}return n}(k);t.weights=_;var S=new Ke({distances:k,linkDistance:o}).layout();S.forEach((function(t){Gt(t[0])&&(t[0]=Math.random()*o),Gt(t[1])&&(t[1]=Math.random()*o)})),t.positions=S,S.forEach((function(t,r){e[r].x=t[0]+n[0],e[r].y=t[1]+n[1]})),S.forEach((function(t){t[0]-=S[h][0],t[1]-=S[h][1]})),t.run();var E=t.preventOverlap,N=t.nodeSize,I=t.strictRadial;if(E){var R,P=t.nodeSpacing;R=Tt()(P)?function(){return P}:zt()(P)?P:function(){return 0};var z={nodeSizeFunc:N?j()(N)?function(t){return(N[0]>N[1]?N[0]:N[1])+R(t)}:function(t){return N+R(t)}:function(t){return t.size?j()(t.size)?(t.size[0]>t.size[1]?t.size[0]:t.size[1])+R(t):t.size+R(t):10+R(t)},adjMatrix:l,positions:S,radii:M,height:y,width:g,strictRadial:I,focusID:h,iterations:t.maxPreventOverlapIteration||200,k:S.length/4.5,nodes:e},F=new Ze(z);S=F.layout()}S.forEach((function(t,r){e[r].x=t[0]+n[0],e[r].y=t[1]+n[1]}))}},e.prototype.run=function(){for(var t=this.maxIteration,e=this.positions||[],r=this.weights||[],n=this.eIdealDistances||[],o=this.radii||[],i=0;i<=t;i++){var a=i/t;this.oneIteration(a,e,o,n,r)}},e.prototype.oneIteration=function(t,e,r,n,o){var i=1-t,a=this.focusIndex;e.forEach((function(u,s){var c=tr(u,[0,0]),f=0===c?0:1/c;if(s!==a){var h=0,l=0,d=0;e.forEach((function(t,e){if(s!==e){var r=tr(u,t),i=0===r?0:1/r,a=n[e][s];d+=o[s][e],h+=o[s][e]*(t[0]+a*(u[0]-t[0])*i),l+=o[s][e]*(t[1]+a*(u[1]-t[1])*i)}}));var p=0===r[s]?0:1/r[s];d*=i,d+=t*p*p,h*=i,h+=t*p*u[0]*f,u[0]=h/d,l*=i,l+=t*p*u[1]*f,u[1]=l/d}}))},e.prototype.eIdealDisMatrix=function(){var t=this,e=t.nodes;if(!e)return[];var r=t.distances,n=t.linkDistance,o=t.radii||[],i=t.unitRadius||50,a=[];return r&&r.forEach((function(r,u){var s=[];r.forEach((function(r,a){if(u===a)s.push(0);else if(o[u]===o[a])if("data"===t.sortBy)s.push(r*(Math.abs(u-a)*t.sortStrength)/(o[u]/i));else if(t.sortBy){var c=e[u][t.sortBy]||0,f=e[a][t.sortBy]||0;C()(c)&&(c=c.charCodeAt(0)),C()(f)&&(f=f.charCodeAt(0)),s.push(r*(Math.abs(c-f)*t.sortStrength)/(o[u]/i))}else s.push(r*n/(o[u]/i));else{var h=(n+i)/2;s.push(r*h)}})),a.push(s)})),a},e.prototype.handleInfinity=function(t,e,r){for(var n=t.length,o=0;o<n;o++)if(t[e][o]===1/0){t[e][o]=r,t[o][e]=r;for(var i=0;i<n;i++)t[o][i]!==1/0&&t[e][i]===1/0&&(t[e][i]=r+t[o][i],t[i][e]=r+t[o][i])}for(o=0;o<n;o++)if(o!==e)for(i=0;i<n;i++)if(t[o][i]===1/0){var a=Math.abs(t[e][o]-t[e][i]);a=0===a?1:a,t[o][i]=a}},e.prototype.maxToFocus=function(t,e){for(var r=0,n=0;n<t[e].length;n++)t[e][n]!==1/0&&(r=t[e][n]>r?t[e][n]:r);return r},e}(c),rr=function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.center=[0,0],e.width=300,e.height=300,e}return a(e,t),e.prototype.getDefaultCfg=function(){return{center:[0,0],width:300,height:300}},e.prototype.execute=function(){var t=this,e=t.nodes,r=t.center;t.width||"undefined"==typeof window||(t.width=window.innerWidth),t.height||"undefined"==typeof window||(t.height=window.innerHeight),e&&e.forEach((function(e){e.x=.9*(Math.random()-.5)*t.width+r[0],e.y=.9*(Math.random()-.5)*t.height+r[1]}))},e}(c),nr=r(32),or=r.n(nr),ir=r(33),ar=r.n(ir),ur=[{index:7,opacity:.15},{index:6,opacity:.25},{index:5,opacity:.3},{index:5,opacity:.45},{index:5,opacity:.65},{index:5,opacity:.85},{index:4,opacity:.9},{index:3,opacity:.95},{index:2,opacity:.97},{index:1,opacity:.98}];function sr(t,e,r){var n;return(n=Math.round(t.h)>=60&&Math.round(t.h)<=240?r?Math.round(t.h)-2*e:Math.round(t.h)+2*e:r?Math.round(t.h)+2*e:Math.round(t.h)-2*e)<0?n+=360:n>=360&&(n-=360),n}function cr(t,e,r){return 0===t.h&&0===t.s?t.s:((n=r?t.s-.16*e:4===e?t.s+.16:t.s+.05*e)>1&&(n=1),r&&5===e&&n>.1&&(n=.1),n<.06&&(n=.06),Number(n.toFixed(2)));var n}function fr(t,e,r){var n;return(n=r?t.v+.05*e:t.v-.15*e)>1&&(n=1),Number(n.toFixed(2))}function hr(t){for(var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},r=[],n=ar()(t),o=5;o>0;o-=1){var i=n.toHsv(),a=ar()({h:sr(i,o,!0),s:cr(i,o,!0),v:fr(i,o,!0)}).toHexString();r.push(a)}r.push(n.toHexString());for(var u=1;u<=4;u+=1){var s=n.toHsv(),c=ar()({h:sr(s,u),s:cr(s,u),v:fr(s,u)}).toHexString();r.push(c)}return"dark"===e.theme?ur.map((function(t){var n=t.index,o=t.opacity;return ar.a.mix(e.backgroundColor||"#141414",r[n],100*o).toHexString()})):r}var lr={red:"#F5222D",volcano:"#FA541C",orange:"#FA8C16",gold:"#FAAD14",yellow:"#FADB14",lime:"#A0D911",green:"#52C41A",cyan:"#13C2C2",blue:"#1890FF",geekblue:"#2F54EB",purple:"#722ED1",magenta:"#EB2F96",grey:"#666666"},dr={},pr={};Object.keys(lr).forEach((function(t){dr[t]=hr(lr[t]),dr[t].primary=dr[t][5],pr[t]=hr(lr[t],{theme:"dark",backgroundColor:"#141414"}),pr[t].primary=pr[t][5]}));dr.red,dr.volcano,dr.gold,dr.orange,dr.yellow,dr.lime,dr.green,dr.cyan,dr.blue,dr.geekblue,dr.purple,dr.magenta,dr.grey;var vr=function(t,e,r){var n=or()(t),o=or()(e);return or()([(1-r)*n.color[0]+r*o.color[0],(1-r)*n.color[1]+r*o.color[1],(1-r)*n.color[2]+r*o.color[2]]).rgb()},gr=function(t,e,r,n){void 0===e&&(e="#fff"),void 0===r&&(r="default"),void 0===n&&(n="rgb(150, 150, 150)");var o=vr(e,t,.05).rgb().toString(),i=vr(e,n,.02).rgb().toString(),a=vr(e,n,.05).rgb().toString(),u=vr(e,n,.1).rgb().toString(),s=vr(e,n,.2).rgb().toString(),c=vr(e,n,.3).rgb().toString(),f=hr(t,{theme:r,backgroundColor:e}),h=or()(t).hex().toLowerCase(),l=f.indexOf(h),d=t;return-1!==l&&(d=f[l+1]),{mainStroke:t,mainFill:vr(e,t,.1).rgb().toString(),activeStroke:t,activeFill:o,inactiveStroke:vr(e,t,.4).rgb().toString(),inactiveFill:o,selectedStroke:t,selectedFill:e,highlightStroke:d,highlightFill:vr(e,t,.2).rgb().toString(),disableStroke:c,disableFill:a,edgeMainStroke:c,edgeActiveStroke:t,edgeInactiveStroke:s,edgeSelectedStroke:t,edgeHighlightStroke:t,edgeDisableStroke:u,comboMainStroke:c,comboMainFill:i,comboActiveStroke:t,comboActiveFill:o,comboInactiveStroke:c,comboInactiveFill:i,comboSelectedStroke:t,comboSelectedFill:i,comboHighlightStroke:d,comboHighlightFill:i,comboDisableStroke:s,comboDisableFill:a}}("rgb(95, 149, 255)","rgb(255, 255, 255)"),yr={version:"3.8.5",rootContainerClassName:"root-container",nodeContainerClassName:"node-container",edgeContainerClassName:"edge-container",comboContainerClassName:"combo-container",customGroupContainerClassName:"custom-group-container",delegateContainerClassName:"delegate-container",defaultLoopPosition:"top",nodeLabel:{style:{fill:"#000",fontSize:12,textAlign:"center",textBaseline:"middle"},offset:4},defaultNode:{type:"circle",style:{lineWidth:1,stroke:gr.mainStroke,fill:gr.mainFill},size:20,color:gr.mainStroke,linkPoints:{size:8,lineWidth:1,fill:gr.activeFill,stroke:gr.activeStroke}},nodeStateStyles:{active:{fill:gr.activeFill,stroke:gr.activeStroke,lineWidth:2,shadowColor:gr.mainStroke,shadowBlur:10},selected:{fill:gr.selectedFill,stroke:gr.selectedStroke,lineWidth:4,shadowColor:gr.selectedStroke,shadowBlur:10,"text-shape":{fontWeight:500}},highlight:{fill:gr.highlightFill,stroke:gr.highlightStroke,lineWidth:2,"text-shape":{fontWeight:500}},inactive:{fill:gr.inactiveFill,stroke:gr.inactiveStroke,lineWidth:1},disable:{fill:gr.disableFill,stroke:gr.disableStroke,lineWidth:1}},edgeLabel:{style:{fill:"rgb(0, 0, 0)",textAlign:"center",textBaseline:"middle",fontSize:12}},defaultEdge:{type:"line",size:1,style:{stroke:gr.edgeMainStroke,lineAppendWidth:2},color:gr.edgeMainStroke},edgeStateStyles:{active:{stroke:gr.edgeActiveStroke,lineWidth:1},selected:{stroke:gr.edgeSelectedStroke,lineWidth:2,shadowColor:gr.edgeSelectedStroke,shadowBlur:10,"text-shape":{fontWeight:500}},highlight:{stroke:gr.edgeHighlightStroke,lineWidth:2,"text-shape":{fontWeight:500}},inactive:{stroke:gr.edgeInactiveStroke,lineWidth:1},disable:{stroke:gr.edgeDisableStroke,lineWidth:1}},comboLabel:{style:{fill:"rgb(0, 0, 0)",textBaseline:"middle",fontSize:12},refY:10,refX:10},defaultCombo:{type:"circle",style:{fill:gr.comboMainFill,lineWidth:1,stroke:gr.comboMainStroke,r:5,width:20,height:10},size:[20,5],color:gr.comboMainStroke,padding:[25,20,15,20]},comboStateStyles:{active:{stroke:gr.comboActiveStroke,lineWidth:1,fill:gr.comboActiveFill},selected:{stroke:gr.comboSelectedStroke,lineWidth:2,fill:gr.comboSelectedFill,shadowColor:gr.comboSelectedStroke,shadowBlur:10,"text-shape":{fontWeight:500}},highlight:{stroke:gr.comboHighlightStroke,lineWidth:2,fill:gr.comboHighlightFill,"text-shape":{fontWeight:500}},inactive:{stroke:gr.comboInactiveStroke,fill:gr.comboInactiveFill,lineWidth:1},disable:{stroke:gr.comboDisableStroke,fill:gr.comboDisableFill,lineWidth:1}},delegateStyle:{fill:"#F3F9FF",fillOpacity:.5,stroke:"#1890FF",strokeOpacity:.9,lineDash:[5,5]},textWaterMarkerConfig:{width:150,height:100,compatible:!1,text:{x:0,y:60,lineHeight:20,rotate:20,fontSize:14,fontFamily:"Microsoft YaHei",fill:"rgba(0, 0, 0, 0.1)",baseline:"Middle"}},imageWaterMarkerConfig:{width:150,height:130,compatible:!1,image:{x:0,y:0,width:30,height:20,rotate:0}},waterMarkerImage:"https://gw.alipayobjects.com/os/s/prod/antv/assets/image/logo-with-text-73b8a.svg"},mr=Math.PI,br=Math.sin,wr=Math.cos,xr=(br(mr/8),wr(mr/8),function(t,e){if(t&&t.children)for(var r=t.children.length-1;r>=0;r--)if(!xr(t.children[r],e))return;return!1!==e(t)}),Mr=function(t,e){"function"==typeof e&&xr(t,e)},kr={circular:R,concentric:z,dagre:D,force:Vt,g6force:Yt,comboForce:function(t){function e(){var e=null!==t&&t.apply(this,arguments)||this;return e.center=[0,0],e.maxIteration=100,e.gravity=10,e.comboGravity=10,e.linkDistance=10,e.alpha=1,e.alphaMin=.001,e.alphaDecay=1-Math.pow(e.alphaMin,1/300),e.alphaTarget=0,e.velocityDecay=.6,e.edgeStrength=.2,e.nodeStrength=30,e.preventOverlap=!1,e.preventNodeOverlap=!1,e.preventComboOverlap=!1,e.collideStrength=void 0,e.nodeCollideStrength=void 0,e.comboCollideStrength=void 0,e.optimizeRangeFactor=1,e.onTick=function(){},e.onLayoutEnd=function(){},e.depthAttractiveForceScale=.5,e.depthRepulsiveForceScale=2,e.nodes=[],e.edges=[],e.combos=[],e.comboTrees=[],e.width=300,e.height=300,e.bias=[],e.nodeMap={},e.oriComboMap={},e.nodeIdxMap={},e.comboMap={},e.previousLayouted=!1,e}return a(e,t),e.prototype.getDefaultCfg=function(){return{maxIteration:100,center:[0,0],gravity:10,speed:1,comboGravity:30,preventOverlap:!1,preventComboOverlap:!0,preventNodeOverlap:!0,nodeSpacing:void 0,collideStrength:void 0,nodeCollideStrength:.5,comboCollideStrength:.5,comboSpacing:20,comboPadding:10,edgeStrength:.2,nodeStrength:30,linkDistance:10}},e.prototype.execute=function(){var t=this.nodes,e=this.center;if(this.comboTree={id:"comboTreeRoot",depth:-1,children:this.comboTrees},t&&0!==t.length){if(1===t.length)return t[0].x=e[0],void(t[0].y=e[1]);this.initVals(),this.run(),this.onLayoutEnd()}},e.prototype.run=function(){var t=this,e=t.nodes,r=t.previousLayouted?t.maxIteration/5:t.maxIteration;t.width||"undefined"==typeof window||(t.width=window.innerWidth),t.height||"undefined"==typeof window||(t.height=window.innerHeight);var n=t.center,o=t.velocityDecay,i=t.comboMap;t.previousLayouted||t.initPos(i);for(var a=function(r){var n=[];e.forEach((function(t,e){n[e]={x:0,y:0}})),t.applyCalculate(n),t.applyComboCenterForce(n),e.forEach((function(t,e){k(t.x)&&k(t.y)&&(t.x+=n[e].x*o,t.y+=n[e].y*o)})),t.alpha+=(t.alphaTarget-t.alpha)*t.alphaDecay,t.onTick()},u=0;u<r;u++)a();var s=[0,0];e.forEach((function(t){k(t.x)&&k(t.y)&&(s[0]+=t.x,s[1]+=t.y)})),s[0]/=e.length,s[1]/=e.length;var c=[n[0]-s[0],n[1]-s[1]];e.forEach((function(t,e){k(t.x)&&k(t.y)&&(t.x+=c[0],t.y+=c[1])})),t.combos.forEach((function(t){var e=i[t.id];e&&e.empty&&(t.x=e.cx||t.x,t.y=e.cy||t.y)})),t.previousLayouted=!0},e.prototype.initVals=function(){var t=this.edges,e=this.nodes,r=this.combos,n={},o={},i={};e.forEach((function(t,e){o[t.id]=t,i[t.id]=e})),this.nodeMap=o,this.nodeIdxMap=i;var a={};r.forEach((function(t){a[t.id]=t})),this.oriComboMap=a,this.comboMap=this.getComboMap();var u=this.preventOverlap;this.preventComboOverlap=this.preventComboOverlap||u,this.preventNodeOverlap=this.preventNodeOverlap||u;var s=this.collideStrength;s&&(this.comboCollideStrength=s,this.nodeCollideStrength=s);for(var c=0;c<t.length;++c)n[t[c].source]?n[t[c].source]++:n[t[c].source]=1,n[t[c].target]?n[t[c].target]++:n[t[c].target]=1;var f=[];for(c=0;c<t.length;++c)f[c]=n[t[c].source]/(n[t[c].source]+n[t[c].target]);this.bias=f;var h,l,d=this.nodeSize,p=this.nodeSpacing;if(l=k(p)?function(){return p}:g(p)?p:function(){return 0},this.nodeSpacing=l,d)if(g(d))h=function(t){return d(t)};else if(y(d)){var v=(d[0]>d[1]?d[0]:d[1])/2;h=function(t){return v}}else{var m=d/2;h=function(t){return m}}else h=function(t){return t.size?y(t.size)?(t.size[0]>t.size[1]?t.size[0]:t.size[1])/2:t.size/2:10};this.nodeSize=h;var b,w=this.comboSpacing;b=k(w)?function(){return w}:g(w)?w:function(){return 0},this.comboSpacing=b;var x,M=this.comboPadding;x=k(M)?function(){return M}:y(M)?function(){return Math.max.apply(null,M)}:g(M)?M:function(){return 0},this.comboPadding=x;var _,S=this.linkDistance;S||(S=10),_=k(S)?function(t){return S}:S,this.linkDistance=_;var E,j=this.edgeStrength;j||(j=1),E=k(j)?function(t){return j}:j,this.edgeStrength=E;var A,O=this.nodeStrength;O||(O=30),A=k(O)?function(t){return O}:O,this.nodeStrength=A},e.prototype.initPos=function(t){this.nodes.forEach((function(e,r){if(e.comboId){var n=t[e.comboId];e.x=n.cx+100/(r+1),e.y=n.cy+100/(r+1)}else e.x=100/(r+1),e.y=100/(r+1)}))},e.prototype.getComboMap=function(){var t=this,e=t.nodeMap,r=t.nodeIdxMap,n=t.comboTrees,o=t.oriComboMap,i={};return(n||[]).forEach((function(n){var a=[];Mr(n,(function(n){if("node"===n.itemType)return!0;if(!o[n.id])return!0;if(void 0===i[n.id]){var u={name:n.id,cx:0,cy:0,count:0,depth:t.oriComboMap[n.id].depth,children:[]};i[n.id]=u}var s=n.children;s&&s.forEach((function(t){if(!i[t.id]&&!e[t.id])return!0;a.push(t)}));var c=i[n.id];if(c.cx=0,c.cy=0,0===a.length){c.empty=!0;var f=o[n.id],h=Object.keys(e).length,l=n.id+"-visual-child-"+h,d={id:l,x:f.x,y:f.y,depth:c.depth+1,itemType:"node"};t.nodes.push(d),e[l]=d,r[l]=h,c.cx=f.x,c.cy=f.y,a.push(d)}return a.forEach((function(t){if(c.count++,"node"!==t.itemType){var r=i[t.id];return k(r.cx)&&(c.cx+=r.cx),void(k(r.cy)&&(c.cy+=r.cy))}var n=e[t.id];n&&(k(n.x)&&(c.cx+=n.x),k(n.y)&&(c.cy+=n.y))})),c.cx/=c.count,c.cy/=c.count,c.children=a,!0}))})),i},e.prototype.applyComboCenterForce=function(t){var e=this.gravity,r=this.comboGravity||e,n=this.alpha,o=this.comboTrees,i=this.nodeIdxMap,a=this.nodeMap,u=this.comboMap;(o||[]).forEach((function(e){Mr(e,(function(e){if("node"===e.itemType)return!0;if(!u[e.id])return!0;var o=u[e.id],s=.5*(o.depth+1),c=o.cx,f=o.cy;return o.cx=0,o.cy=0,o.children.forEach((function(e){if("node"!==e.itemType){var h=u[e.id];return h&&k(h.cx)&&(o.cx+=h.cx),void(h&&k(h.cy)&&(o.cy+=h.cy))}var l=a[e.id],d=l.x-c||.005,p=l.y-f||.005,v=Math.sqrt(d*d+p*p),g=i[l.id],y=r*n/v*s;t[g].x-=d*y,t[g].y-=p*y,k(l.x)&&(o.cx+=l.x),k(l.y)&&(o.cy+=l.y)})),o.cx/=o.count,o.cy/=o.count,!0}))}))},e.prototype.applyCalculate=function(t){var e=this.comboMap,r=this.nodes,n={};r.forEach((function(t,e){r.forEach((function(r,o){if(!(e<o)){var i=t.x-r.x||.005,a=t.y-r.y||.005,u=i*i+a*a,s=Math.sqrt(u);u<1&&(u=s),n[t.id+"-"+r.id]={vx:i,vy:a,vl2:u,vl:s},n[r.id+"-"+t.id]={vx:-i,vy:-a,vl2:u,vl:s}}}))})),this.updateComboSizes(e),this.calRepulsive(t,n),this.calAttractive(t,n),this.preventComboOverlap&&this.comboNonOverlapping(t,e)},e.prototype.updateComboSizes=function(t){var e=this,r=e.comboTrees,n=e.nodeMap,o=e.nodeSize,i=e.comboSpacing,a=e.comboPadding;(r||[]).forEach((function(r){var u=[];Mr(r,(function(r){if("node"===r.itemType)return!0;var s=t[r.id];if(s){var c=r.children;c&&c.forEach((function(e){(t[e.id]||n[e.id])&&u.push(e)})),s.minX=1/0,s.minY=1/0,s.maxX=-1/0,s.maxY=-1/0,u.forEach((function(t){if("node"!==t.itemType)return!0;var e=n[t.id];if(!e)return!0;var r=o(e),i=e.x-r,a=e.y-r,u=e.x+r,c=e.y+r;s.minX>i&&(s.minX=i),s.minY>a&&(s.minY=a),s.maxX<u&&(s.maxX=u),s.maxY<c&&(s.maxY=c)}));var f=e.oriComboMap[r.id].size||yr.defaultCombo.size;y(f)&&(f=f[0]);var h=Math.max(s.maxX-s.minX,s.maxY-s.minY,f);return s.r=h/2+i(s)/2+a(s),!0}}))}))},e.prototype.comboNonOverlapping=function(t,e){var r=this.comboTree,n=this.comboCollideStrength,o=this.nodeIdxMap,i=this.nodeMap;Mr(r,(function(r){if(e[r.id]||i[r.id]||"comboTreeRoot"===r.id){var a=r.children;return a&&a.length>1&&a.forEach((function(r,u){if("node"!==r.itemType){var s=e[r.id];s&&a.forEach((function(a,c){if(!(u<=c)&&"node"!==a.itemType){var f=e[a.id];if(f){var h=s.cx-f.cx||.005,l=s.cy-f.cy||.005,d=h*h+l*l,p=s.r,v=f.r,g=p+v,y=v*v,m=p*p;if(d<g*g){var b=r.children;if(!b||0===b.length)return;var w=a.children;if(!w||0===w.length)return;var x=Math.sqrt(d),M=(g-x)/x*n,k=h*M,_=l*M,S=y/(m+y),E=1-S;b.forEach((function(e){if("node"===e.itemType&&i[e.id]){var r=o[e.id];w.forEach((function(e){if("node"===e.itemType&&i[e.id]){var n=o[e.id];t[r].x+=k*S,t[r].y+=_*S,t[n].x-=k*E,t[n].y-=_*E}}))}}))}}}}))}})),!0}}))},e.prototype.calRepulsive=function(t,e){var r=this,n=r.nodes,o=r.width*r.optimizeRangeFactor,i=r.nodeStrength,a=r.alpha,u=r.nodeCollideStrength,s=r.preventNodeOverlap,c=r.nodeSize,f=r.nodeSpacing,h=r.depthRepulsiveForceScale,l=r.center;n.forEach((function(d,p){if(d.x&&d.y){if(l){var v=r.gravity,g=d.x-l[0]||.005,y=d.y-l[1]||.005,m=Math.sqrt(g*g+y*y);t[p].x-=g*v*a/m,t[p].y-=y*v*a/m}n.forEach((function(r,n){if(p!==n&&r.x&&r.y){var l=e[d.id+"-"+r.id],v=l.vl2,g=l.vl;if(!(g>o)){var y=e[d.id+"-"+r.id],m=y.vx,b=y.vy,w=Math.abs(r.depth-d.depth)+1||1;r.comboId!==d.comboId&&w++;var x=w?Math.pow(h,w):1,M=i(r)*a/v*x;if(t[p].x+=m*M,t[p].y+=b*M,p<n&&s){var k=c(d)+f(d),_=c(r)+f(r),S=k+_;if(v<S*S){var E=(S-g)/g*u,j=_*_,A=j/(k*k+j),O=m*E,N=b*E;t[p].x+=O*A,t[p].y+=N*A,A=1-A,t[n].x-=O*A,t[n].y-=N*A}}}}}))}}))},e.prototype.calAttractive=function(t,e){var r=this,n=r.edges,o=r.linkDistance,i=r.alpha,a=r.edgeStrength,u=r.bias,s=r.depthAttractiveForceScale;n.forEach((function(n,c){if(n.source&&n.target&&n.source!==n.target){var f=r.nodeIdxMap[n.source],h=r.nodeIdxMap[n.target],l=r.nodeMap[n.source],d=r.nodeMap[n.target],p=Math.abs(l.depth-d.depth);l.comboId===d.comboId&&(p/=2);var v=p?Math.pow(s,p):1;if(l.comboId!==d.comboId&&1===v?v=s/2:l.comboId===d.comboId&&(v=2),k(d.x)&&k(l.x)&&k(d.y)&&k(l.y)){var g=e[n.target+"-"+n.source],y=g.vl,m=g.vx,b=g.vy,w=(y-o(n))/y*i*a(n)*v,x=m*w,M=b*w,_=u[c];t[h].x-=x*_,t[h].y-=M*_,t[f].x+=x*(1-_),t[f].y+=M*(1-_)}}}))},e}(c),fruchterman:Ht,grid:$t,mds:Qe,radial:er,random:rr};o()(kr,(function(t,e){h.registerLayout(e,{},t)}));var _r=h,Sr=self;Sr.onmessage=function(t){(function(t){return t.data.type===Dt})(t)&&function(t){switch(t.data.type){case Dt:var e=t.data,r=e.nodes,n=e.edges,o=e.layoutCfg,i=void 0===o?{}:o,a=i.type,u=_r[a];if(!u){Sr.postMessage({type:Lt,message:"layout "+a+" not found"});break}var s=new u(i);s.init({nodes:r,edges:n}),s.execute(),Sr.postMessage({type:qt,nodes:r}),s.destroy()}}(t)};e.default=null}]);\n//# sourceMappingURL=g6Layout.worker.js.map',"Worker",void 0,n.p+"g6Layout.worker.js")},so=function(e){return setTimeout(e,16)},lo=function(e){return clearTimeout(e)},uo=function(e){return(window.requestAnimationFrame||window.webkitRequestAnimationFrame||so)(e)},co=function(e){return(window.cancelAnimationFrame||window.webkitCancelAnimationFrame||lo)(e)},ho=function(){function e(e){this.graph=e,this.layoutCfg=e.get("layout")||{},this.layoutType=this.layoutCfg.type,this.worker=null,this.workerData={},this.initLayout()}return e.prototype.initLayout=function(){},e.prototype.getWorker=function(){return this.worker||("undefined"==typeof Worker?(console.warn("Web worker is not supported in current browser."),this.worker=null):this.worker=new ao),this.worker},e.prototype.stopWorker=function(){var e=this.workerData;this.worker&&(this.worker.terminate(),this.worker=null,e.requestId&&(co(e.requestId),e.requestId=null),e.requestId2&&(co(e.requestId2),e.requestId2=null))},e.prototype.getLayoutType=function(){return this.layoutCfg.type},e.prototype.layout=function(e){var t=this.graph;this.data=this.setDataFromGraph();var n=this.data.nodes;if(!n)return!1;var r=t.get("width"),i=t.get("height"),o={};Object.assign(o,{width:r,height:i,center:[r/2,i/2]},this.layoutCfg),this.layoutCfg=o;var a=!!this.layoutType,s=this.layoutMethod;s&&s.destroy(),t.emit("beforelayout");var l=this.initPositions(o.center,n);if(this.stopWorker(),o.workerEnabled&&this.layoutWithWorker(this.data,e))return!0;if("force"===this.layoutType||"g6force"===this.layoutType){var u=o.onTick,c=function(){u&&u(),t.refreshPositions()};o.tick=c;var d=o.onLayoutEnd;o.onLayoutEnd=function(){d&&d(),t.emit("afterlayout")}}else"comboForce"===this.layoutType&&(o.comboTrees=t.get("comboTrees"));var h=!1;if(void 0!==this.layoutType){try{s=new ro[this.layoutType](o)}catch(e){return console.warn("The layout method: "+this.layoutType+" does not exist! Please specify it first."),!1}if(h=s.enableTick){var f=o.onTick,p=o.onLayoutEnd;c=function(){f&&f(),t.refreshPositions()},s.tick=c,s.onLayoutEnd=function(){p&&p(),t.emit("afterlayout")}}s.init(this.data),a&&s.execute(),this.layoutMethod=s}return!a&&l||"force"===this.layoutType||h||(t.emit("afterlayout"),this.refreshLayout()),!1},e.prototype.layoutWithWorker=function(e,t){var n=this,r=e.nodes,i=e.edges,o=this.layoutCfg,a=(this.graph,this.getWorker()),s=this.workerData;if(!a)return!1;s.requestId=null,s.requestId2=null,s.currentTick=null,s.currentTickData=null;var l=function(e,t){var n={};return e&&"object"==typeof e?(Object.keys(e).forEach((function(t){e.hasOwnProperty(t)&&function(e){return"function"!=typeof e}(e[t])&&(n[t]=e[t])})),n):e}(o);return a.postMessage({type:"LAYOUT_RUN",nodes:r,edges:i,layoutCfg:l}),a.onmessage=function(r){n.handleWorkerMessage(r,e,t)},!0},e.prototype.handleWorkerMessage=function(e,t,n){var r=this.graph,i=this.workerData,o=this.layoutCfg,a=e.data,s=a.type,l=function(){o.onTick&&o.onTick()},u=function(){o.onLayoutEnd&&o.onLayoutEnd(),r.emit("afterlayout")};switch(s){case Wr:i.currentTick=a.currentTick,i.currentTickData=a,i.requestId||(i.requestId=uo((function(){fo(t,a),r.refreshPositions(),l(),1===a.currentTick&&n&&n(),a.currentTick===a.totalTicks?u():i.currentTick===a.totalTicks&&(i.requestId2=uo((function(){fo(t,i.currentTickData),r.refreshPositions(),i.requestId2=null,l(),u()}))),i.requestId=null})));break;case"LAYOUT_END":null==i.currentTick&&(fo(t,a),this.refreshLayout(),n&&n(),r.emit("afterlayout"))}},e.prototype.refreshLayout=function(){var e=this.graph;e.get("animate")?e.positionsAnimate():e.refreshPositions()},e.prototype.updateLayoutCfg=function(e){var t=this.graph,n=this.layoutMethod;this.layoutType=e.type,n?(this.data=this.setDataFromGraph(),this.stopWorker(),e.workerEnabled&&this.layoutWithWorker(this.data,null)||(n.init(this.data),n.updateCfg(e),t.emit("beforelayout"),n.execute(),"force"===this.layoutType||n.enableTick||t.emit("afterlayout"),this.refreshLayout())):console.warn("You did not assign any layout type and the graph has no previous layout method!")},e.prototype.changeLayout=function(e){var t=this.graph,n=this.layoutMethod;this.layoutType=e,this.layoutCfg=t.get("layout")||{},this.layoutCfg.type=e,n&&n.destroy(),this.layout()},e.prototype.changeData=function(){var e=this.layoutMethod;e&&e.destroy(),this.layout()},e.prototype.destroyLayout=function(){var e=this.layoutMethod,t=this.graph;e&&e.destroy(),t.set("layout",void 0),this.layoutCfg=void 0,this.layoutType=void 0,this.layoutMethod=void 0},e.prototype.setDataFromGraph=function(){var e=[],t=[],n=[],r=this.graph.getNodes(),i=this.graph.getEdges(),o=this.graph.getCombos();return r.forEach((function(t){if(t.isVisible()){var n=t.getModel();e.push(n)}})),i.forEach((function(e){if(!e.destroyed&&e.isVisible()){var n=e.getModel();n.isComboEdge||t.push(n)}})),o.forEach((function(e){if(!e.destroyed&&e.isVisible()){var t=e.getModel();n.push(t)}})),{nodes:e,edges:t,combos:n}},e.prototype.relayout=function(e){var t=this.graph,n=this.layoutMethod,r=this.layoutCfg;if(e){this.data=this.setDataFromGraph();var i=this.data.nodes;if(!i)return!1;this.initPositions(r.center,i),n.init(this.data)}"force"===this.layoutType&&(n.ticking=!1,n.forceSimulation.stop()),t.emit("beforelayout"),n.execute(e),"force"===this.layoutType||n.enableTick||t.emit("afterlayout"),this.refreshLayout()},e.prototype.layoutAnimate=function(){},e.prototype.moveToZero=function(){var e=this.graph.get("data").nodes;if(void 0!==e[0].x&&null!==e[0].x&&!k(e[0].x)){var t=[0,0];e.forEach((function(e){t[0]+=e.x,t[1]+=e.y})),t[0]/=e.length,t[1]/=e.length,e.forEach((function(e){e.x-=t[0],e.y-=t[1]}))}},e.prototype.initPositions=function(e,t){var n=this.graph;if(!t)return!1;var r=!0,i=.85*n.get("width"),o=.85*n.get("height"),a=t.length,s=Math.ceil(Math.sqrt(a)*(i/o)),l=Math.ceil(a/s),u=i/(s-1),c=o/(l-1);isFinite(u)&&u||(u=0),isFinite(c)&&u||(c=0);var d=e[0]-i/2,h=e[1]-o/2;return t.forEach((function(e,t){k(e.x)&&(r=!1,e.x=t%s*u+d),k(e.y)&&(r=!1,e.y=Math.floor(t/s)*c+h)})),r},e.prototype.destroy=function(){var e=this.layoutMethod;this.graph=null,e&&e.destroy();var t=this.worker;t&&(t.terminate(),this.worker=null),this.destroyed=!0},e}();function fo(e,t){var n=e.nodes,r=t.nodes;n.forEach((function(e,t){e.x=r[t].x,e.y=r[t].y}))}var po=null,go=function(){function e(e){this.graph=e,this.cachedStates={enabled:{},disabled:{}},this.destroyed=!1}return e.checkCache=function(e,t,n){if(n[t]){var r=n[t].indexOf(e);r>=0&&n[t].splice(r,1)}},e.cacheState=function(e,t,n){n[t]||(n[t]=[]),n[t].push(e)},e.prototype.updateState=function(t,n,r){var i=this,o=e.checkCache,a=e.cacheState;if(!t.destroyed){var s=this.cachedStates,l=s.enabled,u=s.disabled;r?(o(t,n,u),a(t,n,l)):(o(t,n,l),a(t,n,u)),po&&clearTimeout(po),po=setTimeout((function(){po=null,i.updateGraphStates()}),16)}},e.prototype.updateStates=function(e,t,n){var r=this;x()(t)?this.updateState(e,t,n):t.forEach((function(t){r.updateState(e,t,n)}))},e.prototype.updateGraphStates=function(){var e=this.graph.get("states"),t=this.cachedStates;f()(t.disabled,(function(t,n){e[n]&&(e[n]=e[n].filter((function(e){return t.indexOf(e)<0&&!t.destroyed})))})),f()(t.enabled,(function(t,n){if(e[n]){var r={};e[n].forEach((function(e){e.destroyed||(r[e.get("id")]=!0)})),t.forEach((function(t){if(!t.destroyed){var i=t.get("id");r[i]||(r[i]=!0,e[n].push(t))}}))}else e[n]=t})),this.graph.emit("graphstatechange",{states:e}),this.cachedStates={enabled:{},disabled:{}}},e.prototype.destroy=function(){this.graph=null,this.cachedStates=null,po&&clearTimeout(po),po=null,this.destroyed=!0},e}(),vo=null,mo=function(){function e(t){this.graph=t;var n=t.get("groupStyle");this.styles=Ie()({},e.getDefaultCfg(),n),this.customGroup={},this.delegateInGroup={},this.nodePoint=[],this.destroyed=!1}return e.getDefaultCfg=function(){return{default:{lineWidth:1,stroke:"#A3B1BF",strokeOpacity:.9,fill:"#F3F9FF",fillOpacity:.8,opacity:.8,disCoefficient:.6,minDis:30,maxDis:30},hover:{stroke:"#faad14",fill:"#ffe58f",fillOpacity:.3,opacity:.3,lineWidth:3},collapse:{r:30,width:80,height:40,stroke:"#A3B1BF",lineWidth:3,fill:"#F3F9FF",offsetX:-15,offsetY:5},icon:"https://gw.alipayobjects.com/zos/rmsportal/MXXetJAxlqrbisIuZxDO.svg",operatorBtn:{collapse:{img:"https://gw.alipayobjects.com/zos/rmsportal/uZVdwjJGqDooqKLKtvGA.svg",width:16,height:16},expand:{width:16,height:16,img:"https://gw.alipayobjects.com/zos/rmsportal/MXXetJAxlqrbisIuZxDO.svg"}},visible:!1}},e.prototype.create=function(e,t,n,r,i,o){void 0===n&&(n="circle"),void 0===r&&(r=0),void 0===i&&(i=!1),void 0===o&&(o={});var a=this.graph,s=a.get("customGroup");if(s.get("children").map((function(e){return e.get("id")})).indexOf(e)>-1)console.warn("已经存在ID为 "+e+" 的分组，请重新设置分组ID！");else{var l=s.addGroup({id:e,zIndex:r}),u=this.styles.default,c=this.calculationGroupPosition(t),d=c.x,h=c.y,f=c.width,p=c.height,g=c.maxX,v=this.getGroupPadding(e);a.get("groupBBoxs")[e]={x:d,y:h,width:f,height:p,maxX:g};var m=null;if(i){var y=a.get("groups");t.forEach((function(t){var n=a.findById(t).getModel();n.groupId||(n.groupId=e)})),y.find((function(t){return t.id===e}))||(y.push({id:e,title:o}),a.set({groups:y}))}var b=a.get("groups").filter((function(t){return t.id===e}));b&&b.length>0&&(m=b[0].title);var C=0,A=0,_=null;if("circle"===n){var I=(f+2*d)/2,x=(p+2*h)/2,M=(f>p?f/2:p/2)+v;_=l.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},u),{x:I,y:x,r:M}),draggable:!0,capture:!0,zIndex:r,groupId:e,name:"circle-group-shape"}),C=I,A=x-M,this.setDeletageGroupByStyle(e,l,{width:f,height:p,x:I,y:x,r:M})}else{var S=v*u.disCoefficient;_=l.addShape("rect",{attrs:Object(w.__assign)(Object(w.__assign)({},u),{x:d-S,y:h-S,width:f+2*S,height:p+2*S}),draggable:!0,capture:!0,zIndex:r,groupId:e,name:"rect-group-shape"}),C=d-S+15,A=h-S+15,this.setDeletageGroupByStyle(e,l,{x:d-S,y:h-S,width:f+S,height:p+S,btnOffset:g-3})}if(m){var T=m.offsetX,E=void 0===T?0:T,O=m.offsetY,k=void 0===O?0:O,L=m.text,D=void 0===L?m:L,P=Object(w.__rest)(m,["offsetX","offsetY","text"]);l.addShape("text",{attrs:Object(w.__assign)({text:D,stroke:"#444",x:C+E,y:A+k},P),className:"group-title",name:"group-title-shape"}).set("capture",!1)}l.set("keyShape",_),a.get("groupNodes")[e]=t}},e.prototype.setGroupStyle=function(e,t){if(e&&!e.get("destroyed")){var n={},r=this.styles,i=r.hover,o=r.default;x()(t)?"default"===t?n=Ie()({},o):"hover"===t&&(n=Ie()({},i)):n=Ie()({},o,t),Object.keys(n).forEach((function(t){e.attr(t,n[t])}))}},e.prototype.calculationGroupPosition=function(e,t){if(void 0===t&&(t={x:100,y:100}),0===e.length)return{x:t.x,y:t.y,width:100,height:100};var n=this.graph,r=1/0,i=-1/0,o=1/0,a=-1/0;e.forEach((function(e){var t=(x()(e)?n.findById(e):e).getBBox(),s=t.minX,l=t.minY,u=t.maxX,c=t.maxY;s<r&&(r=s),l<o&&(o=l),u>i&&(i=u),c>a&&(a=c)}));var s=Math.floor(r),l=Math.floor(o);return{x:s,y:l,width:Math.ceil(i)-s,height:Math.ceil(a)-l,maxX:Math.ceil(i)}},e.prototype.flatToTree=function(e,t,n){void 0===t&&(t="id"),void 0===n&&(n="parentId");var r=[],i=[];return e.forEach((function(e){r[e[t]]=e})),e.forEach((function(e){var t=r[e[n]];t?(t.children=t.children||[],t.children.push(e)):i.push(e)})),i},e.prototype.getGroupPadding=function(e){var t=this.graph,n=this.styles.default,r=t.get("groups"),i=r.filter((function(t){return t.parentId===e})),o=1;return i.length>0&&(vo||(vo=this.flatToTree(r)),pn(vo[0],(function(t){if(t.parentId===e&&t.children)return o+=t.children.length,!0}))),r.filter((function(t){return t.id===e&&!t.parentId})).length>0&&(o+=1),r.filter((function(t){return t.parentId===e})).length>0?n.maxDis+30*(o>1?o/2:1):n.minDis},e.prototype.setDeletageGroupByStyle=function(e,t,n){var r=n.width,i=n.height,o=n.x,a=n.y,s=n.r,l=n.btnOffset,u=this.customGroup[e];if(u){var c=u.groupStyle,d=Ie()({},c,n);this.customGroup[e]={nodeGroup:t,groupStyle:d}}else this.customGroup[e]={nodeGroup:t,groupStyle:{width:r,height:i,x:o,y:a,r:s,btnOffset:l}}},e.prototype.getDeletageGroupById=function(e){return this.customGroup[e]},e.prototype.collapseExpandGroup=function(e){var t=this.getDeletageGroupById(e).nodeGroup;t.get("hasHidden")?(t.set("hasHidden",!1),this.expandGroup(e)):(t.set("hasHidden",!0),this.collapseGroup(e))},e.prototype.setGroupTmpNode=function(e,t){var n=this.graph,r=n.get("groupNodes"),i=n.get("groups");r[e].indexOf(t)<0&&r[e].push(t);var o=i.filter((function(t){return t.id===e})),a=null;o.length>0&&(a=o[0].parentId),a&&this.setGroupTmpNode(a,t)},e.prototype.collapseGroup=function(e){var t=this,n=this.getDeletageGroupById(e).nodeGroup,r=this.styles.collapse,i=this.graph,o=i.get("groupType"),a=i.get("groupNodes")[e],s=n.get("keyShape"),l=r.r,u=r.width,c=r.height,d=r.offsetX,h=r.offsetY,f=Object(w.__rest)(r,["r","width","height","offsetX","offsetY"]);Object.keys(f).forEach((function(e){s.attr(e,f[e])}));var p={groupId:e,id:e+"-custom-node",x:s.attr("x"),y:s.attr("y"),style:{r:l},type:"circle"},g=n.find((function(e){return"group-title"===e.get("className")}));"circle"===o?(s.animate({r:l},{duration:500,easing:"easeCubic"}),g&&g.attr({x:s.attr("x")+d,y:s.attr("y")+h})):"rect"===o&&(s.animate({width:u,height:c},{duration:500,easing:"easeCubic"}),g&&g.attr({x:s.attr("x")+10,y:s.attr("y")+c/2+5}),p={groupId:e,id:e+"-custom-node",x:s.attr("x")+u/2,y:s.attr("y")+c/2,size:[u,c],type:"rect"});var v=i.getEdges(),m=v.filter((function(e){var t=e.getModel();return!a.includes(t.source)&&a.includes(t.target)})),y=v.filter((function(e){var t=e.getModel();return a.includes(t.source)&&!a.includes(t.target)})),b=v.filter((function(e){var t=e.getModel();return a.includes(t.source)&&a.includes(t.target)}));if(a.forEach((function(n){var r=i.findById(n),o=r.getModel().groupId;o&&o!==e&&t.getDeletageGroupById(o).nodeGroup.hide(),r.hide()})),b.forEach((function(e){var t=e.getSource(),n=e.getTarget();t.isVisible()&&n.isVisible()?e.show():e.hide()})),m.length>0||y.length>0){var C=i.add("node",p);C.set("capture",!1),C.hide(),this.delegateInGroup[e]={delegateNode:C},this.setGroupTmpNode(e,e+"-custom-node"),this.updateEdgeInGroupLinks(e,m,y)}},e.prototype.updateEdgeInGroupLinks=function(e,t,n){var r=this.graph,i={};t.map((function(t){var n=t.getModel(),o=t.get("id"),a=n.target;return i[o]=a,r.updateItem(t,{target:e+"-custom-node"}),!0}));var o={};n.map((function(t){var n=t.getModel(),i=t.get("id"),a=n.source;return o[i]=a,r.updateItem(t,{source:e+"-custom-node"}),!0})),this.delegateInGroup[e]=Ie()({sourceOutTargetInEdges:t,sourceInTargetOutEdges:n,edgesOuts:i,edgesIn:o},this.delegateInGroup[e])},e.prototype.expandGroup=function(e){var t=this,n=this.graph,r=n.get("groupType"),i=n.get("groupNodes")[e],o=i.filter((function(e){return-1===e.indexOf("custom-node")})),a=this.calculationGroupPosition(o),s=a.width,l=a.height,u=this.getDeletageGroupById(e).nodeGroup,c=u.get("keyShape"),d=this.styles,h=d.default,f=d.collapse;Object.keys(h).forEach((function(e){c.attr(e,h[e])}));var p=u.find((function(e){return"group-title"===e.get("className")})),g=this.getGroupPadding(e);if("circle"===r){var v=s>l?s/2:l/2;c.animate({r:v+g},{duration:500,easing:"easeCubic"})}else if("rect"===r){var m=f.width,y=f.height;c.animate({width:m+s+g*h.disCoefficient*2,height:y+l+g*h.disCoefficient*2},{duration:500,easing:"easeCubic"})}if(p){var b=null,C=n.get("groups").filter((function(t){return t.id===e}));C&&C.length>0&&(b=C[0].title);var A=b.offsetX,_=void 0===A?0:A,I=b.offsetY,x=void 0===I?0:I;"circle"===r?p.animate({x:c.attr("x")+_,y:c.attr("y")-c.attr("r")+x},{duration:600,easing:"easeCubic"}):"rect"===r&&p.animate({x:c.attr("x")+(15+_),y:c.attr("y")+(15+x)},{duration:600,easing:"easeCubic"})}setTimeout((function(){i.forEach((function(r){var i=n.findById(r),o=i.getModel().groupId;if(o&&o!==e){var a=t.getDeletageGroupById(o).nodeGroup;a.show(),a.get("hasHidden")||i.show()}else i.show()})),n.getEdges().filter((function(e){var t=e.getModel();return i.includes(t.source)||i.includes(t.target)})).forEach((function(e){var t=e.getSource(),n=e.getTarget();t.isVisible()&&n.isVisible()&&e.show()}))}),300);var w=this.delegateInGroup[e];if(w){var M=w.sourceOutTargetInEdges,S=w.sourceInTargetOutEdges,T=w.edgesOuts,E=w.edgesIn,O=w.delegateNode;M.map((function(e){var t=e.get("id"),r=T[t];return n.updateItem(e,{target:r}),!0})),S.map((function(e){var t=e.get("id"),r=E[t];return n.updateItem(e,{source:r}),!0}));var k=O.getModel();this.deleteTmpNode(e,k.id),n.remove(O),delete this.delegateInGroup[e]}},e.prototype.deleteTmpNode=function(e,t){var n=this.graph,r=n.get("groups"),i=n.get("groupNodes")[e],o=i.indexOf(t);i.splice(o,1);var a=r.filter((function(t){return t.id===e})),s=null;a.length>0&&(s=a[0].parentId),s&&this.deleteTmpNode(s,t)},e.prototype.remove=function(e){var t=this.graph,n=this.getDeletageGroupById(e);if(n){var r=n.nodeGroup,i=t.get("groupNodes"),o=i[e];o.forEach((function(n){var r=t.findById(n),i=r.getModel(),o=i.groupId;o&&e===o&&(delete i.groupId,t.updateItem(r,i))})),r.destroy(),delete this.customGroup[e];var a=t.get("groups");if(a.length>0){var s=a.filter((function(t){return t.id!==e}));t.set("groups",s)}for(var l=null,u=null,c=0;c<a.length;c++){var d=a[c];if(e===d.id){l=d.parentId,u=d;break}}u&&delete u.parentId,delete i[e],l&&(i[l]=i[l].filter((function(e){return!o.includes(e)})))}},e.prototype.updateGroup=function(e,t,n){var r=this.graph,i=r.get("groupType");this.updateItemInGroup(e,t,n);for(var o=r.save().groups,a=null,s=null,l=0;l<o.length;l++){var u=o[l];if(e===u.id){a=u.parentId,s=u;break}}if(a){var c=this.getDeletageGroupById(a).nodeGroup,d=c.get("keyShape");this.setGroupStyle(d,"default");var h=d.getBBox(),f=h.minX,p=h.minY,g=h.maxX,v=h.maxY,m=this.getDeletageGroupById(e).nodeGroup.get("keyShape").getBBox(),y=m.x,b=m.y;if(!(y<g&&y>f&&b<v&&b>p)){delete s.parentId;var C=r.get("groupNodes"),A=C[e],_=C[a];C[a]=_.filter((function(e){return-1===A.indexOf(e)}));var I=this.calculationGroupPosition(C[a]),x=I.x,w=I.y,M=I.width,S=I.height,T=this.getGroupPadding(a),E=c.find((function(e){return"group-title"===e.get("className")})),O=0,k=0;if("circle"===i){var L=M>S?M/2:S/2,D=x,P=w;d.attr({r:L+T,x:D,y:P}),O=D,k=P-d.attr("r")}else if("rect"===i){var N=T*this.styles.default.disCoefficient;d.attr({x:x-N,y:w-N}),O=x-N+15,k=w-N+15}if(E){var j=s.title,R=0,z=0;j&&(R=j.offsetX,z=j.offsetY),E.attr({x:O+R,y:k+z})}}}},e.prototype.updateItemInGroup=function(e,t,n){var r=this,i=this.graph,o=i.get("groupType"),a=i.get("groupNodes"),s=a[e],l=this.getDeletageGroupById(e).nodeGroup.getBBox(),u=[];s.forEach((function(e,n){var o=i.findById(e),a=o.getModel(),s=a.groupId;s&&!u.includes(s)&&u.push(s),r.nodePoint[n]||(r.nodePoint[n]={x:a.x,y:a.y});var c=t.x-l.x+r.nodePoint[n].x,d=t.y-l.y+r.nodePoint[n].y;r.nodePoint[n]={x:c,y:d},i.updateItem(o,{x:c,y:d})})),u.includes(e)||u.push(e),u.forEach((function(e){var n=r.getDeletageGroupById(e).nodeGroup,i=n.get("keyShape"),s=a[e].filter((function(e){return-1===e.indexOf("custom-node")})),l=r.calculationGroupPosition(s,t),u=l.x,c=l.y,d=l.width,h=l.height,f=0,p=0;if("circle"===o){var g=(d+2*u)/2,v=(h+2*c)/2;i.attr({x:g,y:v}),f=g,p=v-i.attr("r")}else if("rect"===o){var m=n.get("hasHidden"),y={},b=r.getGroupPadding(e)*r.styles.default.disCoefficient;f=u-b+15,p=c-b+15,m?(y={x:u-b,y:c-b},p+=10):y={x:u-b,y:c-b,width:d+2*b,height:h+2*b},i.attr(y)}r.updateGroupTitle(n,e,f,p)}))},e.prototype.updateGroupTitle=function(e,t,n,r){var i=this.graph,o=e.find((function(e){return"group-title"===e.get("className")}));if(o){var a=null,s=i.get("groups").filter((function(e){return e.id===t}));s&&s.length>0&&(a=s[0].title);var l=0,u=0;a&&(l=a.offsetX||0,u=a.offsetY||0),o.attr({x:n+l,y:r+u})}},e.prototype.dynamicChangeGroupSize=function(e,t,n){var r=e.item.getModel().groupId,i=this.graph,o=i.get("groupType"),a=i.get("groupNodes"),s=a[r];if(0===s.length){delete a[r];var l=i.get("groups");i.set("groups",l.filter((function(e){return e.id!==r}))),t.remove()}else{var u=this.calculationGroupPosition(s),c=u.x,d=u.y,h=u.width,f=u.height,p=this.getGroupPadding(r),g=0,v=0;if("circle"===o){var m=h>f?h/2:f/2,y=(h+2*c)/2,b=(f+2*d)/2;n.attr({r:m+p,x:y,y:b}),g=y,v=b-n.attr("r")}else if("rect"===o){var C=p*this.styles.default.disCoefficient;n.attr({x:c-C,y:d-C,width:h+2*C,height:f+2*C}),g=c-C+15,v=d-C+15}this.updateGroupTitle(t,r,g,v)}this.setGroupStyle(n,"default")},e.prototype.resetNodePoint=function(){this.nodePoint.length=0},e.prototype.destroy=function(){this.graph=null,this.styles={},this.customGroup={},this.delegateInGroup={},this.resetNodePoint(),this.destroyed=!0},e}(),yo=function(e){var t={};return e.getNodes().forEach((function(e){t[e.getID()]={degree:0,inDegree:0,outDegree:0}})),e.getEdges().forEach((function(e){t[e.getSource().getID()].degree++,t[e.getSource().getID()].outDegree++,t[e.getTarget().getID()].degree++,t[e.getTarget().getID()].inDegree++})),t},bo=function(){function e(t){this.compare=t||e.defaultCompareFunction}return e.defaultCompareFunction=function(e,t){return e===t?0:e<t?-1:1},e.prototype.equal=function(e,t){return 0===this.compare(e,t)},e.prototype.lessThan=function(e,t){return this.compare(e,t)<0},e.prototype.greaterThan=function(e,t){return this.compare(e,t)>0},e.prototype.lessThanOrEqual=function(e,t){return this.lessThan(e,t)||this.equal(e,t)},e.prototype.greaterThanOrEqual=function(e,t){return this.greaterThan(e,t)||this.equal(e,t)},e.prototype.reverse=function(){var e=this.compare;this.compare=function(t,n){return e(n,t)}},e}(),Co=function(){function e(e,t){void 0===t&&(t=null),this.value=e,this.next=t}return e.prototype.toString=function(e){return e?e(this.value):""+this.value},e}(),Ao=function(){function e(e){void 0===e&&(e=null),this.head=null,this.tail=null,this.compare=new bo(e)}return e.prototype.prepend=function(e){var t=new Co(e,this.head);return this.head=t,this.tail||(this.tail=t),this},e.prototype.append=function(e){var t=new Co(e);return this.head?(this.tail.next=t,this.tail=t,this):(this.head=t,this.tail=t,this)},e.prototype.delete=function(e){if(!this.head)return null;for(var t=null;this.head&&this.compare.equal(this.head.value,e);)t=this.head,this.head=this.head.next;var n=this.head;if(null!==n)for(;n.next;)this.compare.equal(n.next.value,e)?(t=n.next,n.next=n.next.next):n=n.next;return this.compare.equal(this.tail.value,e)&&(this.tail=n),t},e.prototype.find=function(e){var t=e.value,n=void 0===t?void 0:t,r=e.callback,i=void 0===r?void 0:r;if(!this.head)return null;for(var o=this.head;o;){if(i&&i(o.value))return o;if(void 0!==n&&this.compare.equal(o.value,n))return o;o=o.next}return null},e.prototype.deleteTail=function(){var e=this.tail;if(this.head===this.tail)return this.head=null,this.tail=null,e;for(var t=this.head;t.next;)t.next.next?t=t.next:t.next=null;return this.tail=t,e},e.prototype.deleteHead=function(){if(!this.head)return null;var e=this.head;return this.head.next?this.head=this.head.next:(this.head=null,this.tail=null),e},e.prototype.fromArray=function(e){var t=this;return e.forEach((function(e){return t.append(e)})),this},e.prototype.toArray=function(){for(var e=[],t=this.head;t;)e.push(t),t=t.next;return e},e.prototype.reverse=function(){for(var e=this.head,t=null,n=null;e;)n=e.next,e.next=t,t=e,e=n;this.tail=this.head,this.head=t},e.prototype.toString=function(e){return void 0===e&&(e=void 0),this.toArray().map((function(t){return t.toString(e)})).toString()},e}(),_o=function(){function e(e){void 0===e&&(e=10),this.linkedList=new Ao,this.maxStep=e}return Object.defineProperty(e.prototype,"length",{get:function(){return this.linkedList.toArray().length},enumerable:!1,configurable:!0}),e.prototype.isEmpty=function(){return!this.linkedList.head},e.prototype.isMaxStack=function(){return this.toArray().length>=this.maxStep},e.prototype.peek=function(){return this.isEmpty()?null:this.linkedList.head.value},e.prototype.push=function(e){this.linkedList.prepend(e),this.length>this.maxStep&&this.linkedList.deleteTail()},e.prototype.pop=function(){var e=this.linkedList.deleteHead();return e?e.value:null},e.prototype.toArray=function(){return this.linkedList.toArray().map((function(e){return e.value}))},e.prototype.clear=function(){for(;!this.isEmpty();)this.pop()},e}(),Io=function(e,t){var n=e.getNodes(),r=e.getEdges(),i=[],o={};if(!n)throw new Error("invalid nodes data!");return n&&n.forEach((function(e,t){o[e.getID()]=t,i.push([])})),r&&r.forEach((function(e){var n=e.getModel(),r=n.source,a=n.target,s=o[r],l=o[a];i[s][l]=1,t||(i[l][s]=1)})),i},xo=function(e,t){var n=e.get("adjMatrix");n||(n=Io(e,t));for(var r=[],i=n.length,o=0;o<i;o+=1){r[o]=[];for(var a=0;a<i;a+=1)o===a?r[o][a]=0:0!==n[o][a]&&n[o][a]?r[o][a]=n[o][a]:r[o][a]=1/0}for(var s=0;s<i;s+=1)for(o=0;o<i;o+=1)for(a=0;a<i;a+=1)r[o][a]>r[o][s]+r[s][a]&&(r[o][a]=r[o][s]+r[s][a]);return r},wo=function(e,t,n){return(e.y-n.y)*(t.x-n.x)-(e.x-n.x)*(t.y-n.y)},Mo=function(e){var t=e.map((function(e){return{x:e.getModel().x,y:e.getModel().y}}));t.sort((function(e,t){return e.x===t.x?e.y-t.y:e.x-t.x}));for(var n=[],r=0;r<t.length;r++){for(;n.length>=2&&wo(n[n.length-2],n[n.length-1],t[r])<=0;)n.pop();n.push(t[r])}var i=[];for(r=t.length-1;r>=0;r--){for(;i.length>=2&&wo(i[i.length-2],i[i.length-1],t[r])<=0;)i.pop();i.push(t[r])}return i.pop(),n.pop(),n.concat(i)},So={maxRoutingIterations:100,maxMarchingIterations:100,pixelGroupSize:2,edgeR0:10,edgeR1:10,nodeR0:5,nodeR1:10,morphBuffer:5,threshold:.001,skip:16,nodeInfluenceFactor:1,edgeInfluenceFactor:1,negativeNodeInfluenceFactor:-.5};function To(e,t,n){var r=!1,i=function(e,n){return t.cells[e+n*t.width]},o=function(e,t){var r=0;return i(e-1,t-1)>=n&&(r+=1),i(e,t-1)>n&&(r+=2),i(e-1,t)>n&&(r+=4),i(e,t)>n&&(r+=8),r},a=function(n,r){for(var i,a,s=n,l=r,u=0;u<t.width*t.height;u++){if(i=s,a=l,e.findIndex((function(e){return e.x===s&&e.y===l}))>-1){if(e[0].x===s&&e[0].y===l)return!0}else e.push({x:s,y:l});var c=o(s,l);switch(c){case-1:return console.warn("Marched out of bounds"),!0;case 0:case 3:case 2:case 7:s++;break;case 12:case 14:case 4:s--;break;case 6:0===i&&(-1===a?s-=1:s+=1);break;case 1:case 13:case 5:l--;break;case 9:1===i&&(0===a?l-=1:l+=1);break;case 10:case 8:case 11:l++;break;default:return console.warn("Marching squares invalid state: "+c),!0}}};this.march=function(){for(var e=0;e<t.width&&!r;e+=1)for(var s=0;s<t.height&&!r;s+=1)i(e,s)>n&&15!==o(e,s)&&(r=a(e,s));return r}}var Eo=function(e,t){var n=Number.POSITIVE_INFINITY,r=null;return e.forEach((function(e){var i=at(e,t);i>=0&&i<n&&(r=e,n=i)})),r},Oo=function(e,t,n){var r=Object.assign(So,n),i=st(e.map((function(e){return{x:e.getModel().x,y:e.getModel().y}})));e=e.sort((function(e,t){return lt({x:e.getModel().x,y:e.getModel().y},i)-lt({x:t.getModel().x,y:t.getModel().y},i)}));var o=[],a=[];e.forEach((function(e){(function(e,t,n,r,i){var o=function(e,t,n){var r=null,i=Number.POSITIVE_INFINITY;return t.forEach((function(t){var o={x:e.getModel().x,y:e.getModel().y},a={x:t.getModel().x,y:t.getModel().y},s=lt(o,a),l=new nt(o.x,o.y,a.x,a.y),u=n.reduce((function(e,t){return at(t,l)>0?e+1:e}),0);s*Math.pow(u+1,2)<i&&(r=t,i=s*Math.pow(u+1,2))})),r}(e,n,t);return null===o?[]:function(e){for(var n=[];e.length>0;){var r=e.pop();if(0===e.length){n.push(r);break}var i=e.pop(),o=new nt(r.x1,r.y1,i.x2,i.y2);Eo(t,o)?(n.push(r),e.push(i)):e.push(o)}return n}(function(e,t,n,r){var i=[],o=[];o.push(e);for(var a=!0,s=0,l=function(e,t){var n=!1;return t.forEach((function(t){n||(ct(e,{x:t.x1,y:t.y1})||ct(e,{x:t.x2,y:t.y2}))&&(n=!0)})),n},u=function(e,t){for(var n=0,r=t;n<r.length;n++){var i=r[n].getBBox(),o=[[i.x,i.y],[i.x+i.width,i.y],[i.x,i.y+i.height],[i.x+i.width,i.y+i.height]];if($e(o,e.x,e.y))return!0}return!1};a&&s<n;){a=!1;for(var c=function(){var e=o.pop(),n=Eo(t,e);if(n){var c=ot(n,e),d=c[0];if(2===c[1]){var h=function(s){for(var c=r,h=Lo(n,c,d,s),f=l(h,o)||l(h,i),p=u(h,t);!f&&p&&c>=1;)h=Lo(n,c/=1.5,d,s),f=l(h,o)||l(h,i),p=u(h,t);!h||f||s&&p||(o.push(new nt(e.x1,e.y1,h.x,h.y)),o.push(new nt(h.x,h.y,e.x2,e.y2)),a=!0)};h(!0),a||h(!1)}}a||i.push(e),s+=1};!a&&o.length;)c()}for(;o.length;)i.push(o.pop());return i}(new nt(e.getModel().x,e.getModel().y,o.getModel().x,o.getModel().y),t,r,i))})(e,t,o,r.maxRoutingIterations,r.morphBuffer).forEach((function(e){a.push(e)})),o.push(e)}));for(var s,l,u,c,d,h=function(e,t,n){var r={minX:Number.POSITIVE_INFINITY,minY:Number.POSITIVE_INFINITY,maxX:Number.NEGATIVE_INFINITY,maxY:Number.NEGATIVE_INFINITY,width:0,height:0,x:0,y:0},i=[];e.forEach((function(e){i.push(e.getBBox())})),t.forEach((function(e){i.push(e.getBBox())}));for(var o=0,a=i;o<a.length;o++){var s=a[o];r.minX=(s.minX<r.minX?s.minX:r.minX)-n,r.minY=(s.minY<r.minY?s.minY:r.minY)-n,r.maxX=(s.maxX>r.maxX?s.maxX:r.maxX)+n,r.maxY=(s.maxY>r.maxY?s.maxY:r.maxY)+n}return r.width=r.maxX-r.minX,r.height=r.maxY-r.minY,r.x=r.minX,r.y=r.minY,r}(e,a,r.nodeR0),f=(s=h.width,l=h.height,u=r.pixelGroupSize,c=Math.ceil(s/u),d=Math.ceil(l/u),{cells:new Float32Array(Math.max(0,c*d)).fill(0),width:c,height:d}),p=[],g=[],v=0;v<r.maxMarchingIterations;v++)if(ko(e,t,a,h,f,r),g=[],new To(p=[],f,r.threshold).march()){var m=p.map((function(e){return{x:Math.round(e.x*r.pixelGroupSize+h.minX),y:Math.round(e.y*r.pixelGroupSize+h.minY)}}));if(m){var y=m.length;if(r.skip>1)for(y=Math.floor(m.length/r.skip);y<3&&r.skip>1;)r.skip-=1,y=Math.floor(m.length/r.skip);for(var b=0,C=0;C<y;C+=1,b+=r.skip)g.push({x:m[b].x,y:m[b].y})}if(g&&function(){for(var t=0,n=e;t<n.length;t++){var r=n[t],i=g.map((function(e){return[e.x,e.y]}));if(!$e(i,r.getBBox().centerX,r.getBBox().centerY))return!1}return!0}())return g;if(r.threshold*=.9,v<=.5*r.maxMarchingIterations)r.memberInfluenceFactor*=1.2,r.edgeInfluenceFactor*=1.2;else{if(!(0!==r.nonMemberInfluenceFactor&&t.length>0))break;r.nonMemberInfluenceFactor*=.8}}return g};function ko(e,t,n,r,i,o){function a(e,t){var n=Math.floor((e-t)/o.pixelGroupSize);return n<0?0:n}function s(e,t){return e*o.pixelGroupSize+t}var l=(o.nodeR0-o.nodeR1)*(o.nodeR0-o.nodeR1),u=(o.edgeR0-o.edgeR1)*(o.edgeR0-o.edgeR1),c=function(e,t){return[Math.min(a(e.minX,t+r.minX),i.width),Math.min(a(e.minY,t+r.minY),i.height),Math.min(a(e.maxX,-t+r.minX),i.width),Math.min(a(e.maxY,-t+r.minY),i.height)]},d=function(e,t){for(var n=e.getBBox(),a=c(n,o.nodeR1),l=a[0],u=a[1],d=a[2],h=a[3],f=u;f<h;f+=1)for(var p=l;p<d;p+=1)if(!(t<0&&i[p+f*i.width]<=0)){var g=s(p,r.minX),v=s(f,r.minY),m=dt({x:g,y:v},{x:n.minX,y:n.minY,width:n.width,height:n.height});if(m<Math.pow(o.nodeR1,2)){var y=Math.sqrt(m)-o.nodeR1;i.cells[p+f*i.width]+=t*y*y}}};o.nodeInfluenceFactor&&e.forEach((function(e){d(e,o.nodeInfluenceFactor/l)})),o.edgeInfluenceFactor&&n.forEach((function(e){!function(e,t){for(var n=e.getBBox(),a=c(n,o.edgeR1),l=a[0],u=a[1],d=a[2],h=a[3],f=u;f<h;f+=1)for(var p=l;p<d;p+=1)if(!(t<0&&i.cells[p+f*i.width]<=0)){var g=s(p,r.minX),v=s(f,r.minY),m=ut({x:g,y:v},e);if(m<Math.pow(o.edgeR1,2)){var y=Math.sqrt(m)-o.edgeR1;i.cells[p+f*i.width]+=t*y*y}}}(e,o.edgeInfluenceFactor/u)})),o.negativeNodeInfluenceFactor&&t.forEach((function(e){d(e,o.negativeNodeInfluenceFactor/l)}))}function Lo(e,t,n,r){var i=e.getBBox(),o=n[0],a=n[1],s=n[2],l=n[3],u={topLeft:{x:i.minX-t,y:i.minY-t},topRight:{x:i.maxX+t,y:i.minY-t},bottomLeft:{x:i.minX-t,y:i.maxY+t},bottomRight:{x:i.maxX+t,y:i.maxY+t}},c=i.height*i.width;function d(e,t){return i.width*(.5*(e.y-i.minY+(t.y-i.minY)))}if(a)return o?r?u.topLeft:u.bottomRight:s?r?u.bottomLeft:u.topRight:d(a,l)<.5*c?a.y>l.y?r?u.topLeft:u.bottomRight:r?u.topRight:u.bottomLeft:a.y<l.y?r?u.bottomLeft:u.topRight:r?u.bottomRight:u.topLeft;if(l){if(o)return r?u.topRight:u.bottomLeft;if(s)return r?u.bottomRight:u.topLeft}return d(o,s)<.5*c?o.x>s.x?r?u.topLeft:u.bottomRight:r?u.bottomLeft:u.topRight:o.x<s.x?r?u.topRight:u.bottomLeft:r?u.bottomRight:u.topLeft}var Do=function(){function e(e,t){this.cfg=Object(ce.deepMix)(this.getDefaultCfg(),t),this.graph=e,this.id=this.cfg.id,this.group=this.cfg.group,this.members=this.cfg.members.map((function(t){return Object(ce.isString)(t)?e.findById(t):t})),this.nonMembers=this.cfg.nonMembers.map((function(t){return Object(ce.isString)(t)?e.findById(t):t})),this.setPadding(),this.setType(),this.path=this.calcPath(this.members,this.nonMembers),this.render()}return e.prototype.getDefaultCfg=function(){return{id:"g6-hull",type:"round-convex",members:[],nonMembers:[],style:{fill:"lightblue",stroke:"blue",opacity:.2},padding:10}},e.prototype.setPadding=function(){var e=this.members.length&&this.members[0].getKeyShape().getCanvasBBox().width/2;this.padding=this.cfg.padding>0?this.cfg.padding+e:10+e,this.cfg.bubbleCfg={nodeR0:this.padding-e,nodeR1:this.padding-e,morphBuffer:this.padding-e}},e.prototype.setType=function(){this.type=this.cfg.type,this.members.length<3&&(this.type="round-convex"),"round-convex"!==this.type&&"smooth-convex"!==this.type&&"bubble"!==this.type&&(console.warn("The hull type should be either round-convex, smooth-convex or bubble, round-convex is used by default."),this.type="round-convex")},e.prototype.calcPath=function(e,t){var n,r,i;switch(this.type){case"round-convex":r=function(e){if(!e)return null;if(y()(e))return e;var t={a:7,c:6,o:2,h:1,l:2,m:2,r:4,q:4,s:4,t:2,v:1,u:3,z:0},n=[];return String(e).replace(pt,(function(e,r,i){var o=[],a=r.toLowerCase();if(i.replace(gt,(function(e,t){t&&o.push(+t)})),"m"===a&&o.length>2&&(n.push([r].concat(o.splice(0,2))),a="l",r="m"===r?"l":"L"),"o"===a&&1===o.length&&n.push([r,o[0]]),"r"===a)n.push([r].concat(o));else for(;o.length>=t[a]&&(n.push([r].concat(o.splice(0,t[a]))),t[a]););return""})),n}(i=wt((n=Mo(e)).map((function(e){return[e.x,e.y]})),this.padding));break;case"smooth-convex":i=Mt((n=Mo(e)).map((function(e){return[e.x,e.y]})),this.padding),r=n.length>=2&&At(i);break;case"bubble":r=(n=Oo(e,t,this.cfg.bubbleCfg)).length>=2&&At(n)}return r},e.prototype.render=function(){this.group.addShape("path",{attrs:Object(w.__assign)({path:this.path},this.cfg.style),id:this.id,name:this.cfg.id}),this.group.toBack()},e.prototype.addMember=function(e){if(e){Object(ce.isString)(e)&&(e=this.graph.findById(e)),this.members.push(e);var t=this.nonMembers.indexOf(e);return t>-1&&this.nonMembers.splice(t,1),this.updateData(this.members,this.nonMembers),!0}},e.prototype.addNonMember=function(e){if(e){Object(ce.isString)(e)&&(e=this.graph.findById(e)),this.nonMembers.push(e);var t=this.members.indexOf(e);return t>-1&&this.members.splice(t,1),this.updateData(this.members,this.nonMembers),!0}},e.prototype.removeMember=function(e){if(e){Object(ce.isString)(e)&&(e=this.graph.findById(e));var t=this.members.indexOf(e);return t>-1&&(this.members.splice(t,1),this.updateData(this.members,this.nonMembers),!0)}},e.prototype.removeNonMember=function(e){if(e){Object(ce.isString)(e)&&(e=this.graph.findById(e));var t=this.nonMembers.indexOf(e);return t>-1&&(this.nonMembers.splice(t,1),this.updateData(this.members,this.nonMembers),!0)}},e.prototype.updateData=function(e,t){var n=this;this.group.findById(this.id).remove(),e&&(this.members=e.map((function(e){return Object(ce.isString)(e)?n.graph.findById(e):e}))),t&&(this.nonMembers=t.map((function(e){return Object(ce.isString)(e)?n.graph.findById(e):e}))),this.path=this.calcPath(this.members,this.nonMembers),this.render()},e.prototype.updateStyle=function(e){this.group.findById(this.id).attr(Object(w.__assign)({},e))},e.prototype.updateCfg=function(e){var t=this;this.cfg=Object(ce.deepMix)(this.cfg,e),this.id=this.cfg.id,this.group=this.cfg.group,e.members&&(this.members=this.cfg.members.map((function(e){return Object(ce.isString)(e)?t.graph.findById(e):e}))),e.nonMembers&&(this.nonMembers=this.cfg.nonMembers.map((function(e){return Object(ce.isString)(e)?t.graph.findById(e):e}))),this.setPadding(),this.setType(),this.path=this.calcPath(this.members,this.nonMembers),this.render()},e.prototype.contain=function(e){var t,n,r=this,i=(t=Object(ce.isString)(e)?this.graph.findById(e):e).getKeyShape();if("path"===t.get("type"))n=Ct(i.attr("path"));else{var o=i.getCanvasBBox();n=[[o.minX,o.minY],[o.maxX,o.minY],[o.maxX,o.maxY],[o.minX,o.maxY]]}return n=n.map((function(e){var t=r.graph.getPointByCanvas(e[0],e[1]);return[t.x,t.y]})),tt(n,Ct(this.path))},e.prototype.destroy=function(){this.group.remove(),this.cfg=null},e}(),Po=function(e){function t(t){var n=e.call(this)||this;return n.cfg=Object(ce.deepMix)(n.getDefaultCfg(),t),n.init(),n.animating=!1,n.destroyed=!1,n.cfg.enabledStack&&(n.undoStack=new _o(n.cfg.maxStep),n.redoStack=new _o(n.cfg.maxStep)),n}return Object(w.__extends)(t,e),t.prototype.init=function(){this.initCanvas();var e=new Xt(this),t=new Ut(this),n=new Ht(this),r=new Fn(this),i=new ho(this),o=new go(this),a=new mo(this);this.set({eventController:e,viewController:t,modeController:n,itemController:r,layoutController:i,stateController:o,customGroupControll:a}),this.initPlugin()},t.prototype.initCanvas=function(){var e=this.get("container");if(Object(ce.isString)(e)&&(e=document.getElementById(e),this.set("container",e)),!e)throw new Error("invalid container");var t,n=this.get("width"),r=this.get("height");if("svg"===this.get("renderer"))t=new Rt.a({container:e,width:n,height:r});else{var i={container:e,width:n,height:r},o=this.get("pixelRatio");o&&(i.pixelRatio=o),t=new Nt.a(i)}this.set("canvas",t),this.initGroups()},t.prototype.initPlugin=function(){var e=this;Object(ce.each)(e.get("plugins"),(function(t){!t.destroyed&&t.initPlugin&&t.initPlugin(e)}))},t.prototype.initGroups=function(){var e=this.get("canvas"),t=this.get("canvas").get("el").id,n=e.addGroup({id:t+"-root",className:J.rootContainerClassName});if(this.get("groupByTypes")){var r=n.addGroup({id:t+"-edge",className:J.edgeContainerClassName}),i=n.addGroup({id:t+"-node",className:J.nodeContainerClassName}),o=n.addGroup({id:t+"-combo",className:J.comboContainerClassName}),a=n.addGroup({id:t+"-group",className:J.customGroupContainerClassName});a.toBack(),o.toBack(),this.set({nodeGroup:i,edgeGroup:r,customGroup:a,comboGroup:o})}var s=n.addGroup({id:t+"-delegate",className:J.delegateContainerClassName});this.set({delegateGroup:s}),this.set("group",n)},t.prototype.getDefaultCfg=function(){return{container:void 0,width:void 0,height:void 0,renderer:"canvas",modes:{},plugins:[],data:{},fitViewPadding:10,minZoom:.2,maxZoom:10,event:!0,groupByTypes:!0,directed:!1,autoPaint:!0,nodes:[],edges:[],combos:[],vedges:[],itemMap:{},linkCenter:!1,defaultNode:{},defaultEdge:{},nodeStateStyles:{},edgeStateStyles:{},states:{},animate:!1,animateCfg:{onFrame:void 0,duration:500,easing:"easeLinear"},callback:void 0,groupType:"circle",groupBBoxs:{},groupNodes:{},groups:[],groupStyle:{},enabledStack:!1,maxStep:10,tooltips:[]}},t.prototype.set=function(e,t){return Object(ce.isPlainObject)(e)?this.cfg=Object(w.__assign)(Object(w.__assign)({},this.cfg),e):this.cfg[e]=t,this},t.prototype.get=function(e){return this.cfg[e]},t.prototype.getGroup=function(){return this.get("group")},t.prototype.getContainer=function(){return this.get("container")},t.prototype.getMinZoom=function(){return this.get("minZoom")},t.prototype.setMinZoom=function(e){return this.set("minZoom",e)},t.prototype.getMaxZoom=function(){return this.get("maxZoom")},t.prototype.setMaxZoom=function(e){return this.set("maxZoom",e)},t.prototype.getWidth=function(){return this.get("width")},t.prototype.getHeight=function(){return this.get("height")},t.prototype.clearItemStates=function(e,t){Object(ce.isString)(e)&&(e=this.findById(e));var n=this.get("itemController");t||(t=e.get("states")),n.clearItemStates(e,t),this.get("stateController").updateStates(e,t,!1)},t.prototype.node=function(e){"function"==typeof e&&this.set("nodeMapper",e)},t.prototype.edge=function(e){"function"==typeof e&&this.set("edgeMapper",e)},t.prototype.combo=function(e){"function"==typeof e&&this.set("comboMapper",e)},t.prototype.findById=function(e){return this.get("itemMap")[e]},t.prototype.find=function(e,t){var n,r=this.get(e+"s");return Object(ce.each)(r,(function(e,r){if(t(e,r))return n=e})),n},t.prototype.findAll=function(e,t){var n=[];return Object(ce.each)(this.get(e+"s"),(function(e,r){t(e,r)&&n.push(e)})),n},t.prototype.findAllByState=function(e,t){return this.findAll(e,(function(e){return e.hasState(t)}))},t.prototype.translate=function(e,t){var n=this.get("group"),r=Object(ce.clone)(n.getMatrix());r||(r=[1,0,0,0,1,0,0,0,1]),zt.mat3.translate(r,r,[e,t]),n.setMatrix(r),this.emit("viewportchange",{action:"translate",matrix:n.getMatrix()}),this.autoPaint()},t.prototype.moveTo=function(e,t){var n=this.get("group");qe(n,{x:e,y:t}),this.emit("viewportchange",{action:"move",matrix:n.getMatrix()})},t.prototype.fitView=function(e){e&&this.set("fitViewPadding",e),this.get("viewController").fitView(),this.autoPaint()},t.prototype.fitCenter=function(){this.get("viewController").fitCenter(),this.autoPaint()},t.prototype.addBehaviors=function(e,t){return this.get("modeController").manipulateBehaviors(e,t,!0),this},t.prototype.removeBehaviors=function(e,t){return this.get("modeController").manipulateBehaviors(e,t,!1),this},t.prototype.zoom=function(e,t){var n=this.get("group"),r=Object(ce.clone)(n.getMatrix()),i=this.get("minZoom"),o=this.get("maxZoom");r||(r=[1,0,0,0,1,0,0,0,1]),t?(zt.mat3.translate(r,r,[-t.x,-t.y]),zt.mat3.scale(r,r,[e,e]),zt.mat3.translate(r,r,[t.x,t.y])):zt.mat3.scale(r,r,[e,e]),i&&r[0]<i||o&&r[0]>o||(n.setMatrix(r),this.emit("viewportchange",{action:"zoom",matrix:r}),this.autoPaint())},t.prototype.zoomTo=function(e,t){var n=e/this.getZoom();this.zoom(n,t)},t.prototype.focusItem=function(e,t,n){var r=this.get("viewController"),i=!1;t?i=!0:void 0===t&&(i=this.get("animate"));var o={};n?o=n:void 0===n&&(o=this.get("animateCfg")),r.focus(e,i,o),this.autoPaint()},t.prototype.autoPaint=function(){this.get("autoPaint")&&this.paint()},t.prototype.paint=function(){this.emit("beforepaint"),this.get("canvas").draw(),this.emit("afterpaint")},t.prototype.getPointByClient=function(e,t){return this.get("viewController").getPointByClient(e,t)},t.prototype.getClientByPoint=function(e,t){return this.get("viewController").getClientByPoint(e,t)},t.prototype.getPointByCanvas=function(e,t){return this.get("viewController").getPointByCanvas(e,t)},t.prototype.getCanvasByPoint=function(e,t){return this.get("viewController").getCanvasByPoint(e,t)},t.prototype.showItem=function(e,t){void 0===t&&(t=!0);var n=this.get("itemController").changeItemVisibility(e,!0);if(t&&this.get("enabledStack")){var r=n.getID(),i={},o={};switch(n.getType()){case"node":i.nodes=[{id:r,visible:!1}],o.nodes=[{id:r,visible:!0}];break;case"edge":i.nodes=[{id:r,visible:!1}],o.edges=[{id:r,visible:!0}];break;case"combo":i.nodes=[{id:r,visible:!1}],o.combos=[{id:r,visible:!0}]}this.pushStack("visible",{before:i,after:o})}},t.prototype.hideItem=function(e,t){void 0===t&&(t=!0);var n=this.get("itemController").changeItemVisibility(e,!1);if(t&&this.get("enabledStack")){var r=n.getID(),i={},o={};switch(n.getType()){case"node":i.nodes=[{id:r,visible:!0}],o.nodes=[{id:r,visible:!1}];break;case"edge":i.nodes=[{id:r,visible:!0}],o.edges=[{id:r,visible:!1}];break;case"combo":i.nodes=[{id:r,visible:!0}],o.combos=[{id:r,visible:!1}]}this.pushStack("visible",{before:i,after:o})}},t.prototype.refreshItem=function(e){this.get("itemController").refreshItem(e)},t.prototype.setAutoPaint=function(e){this.set("autoPaint",e),this.get("canvas").set("autoDraw",e)},t.prototype.remove=function(e,t){void 0===t&&(t=!0),this.removeItem(e,t)},t.prototype.removeItem=function(e,t){void 0===t&&(t=!0);var n=e;if(Object(ce.isString)(e)&&(n=this.findById(e)),!n&&Object(ce.isString)(e))console.warn("The item to be removed does not exist!"),this.get("customGroupControll").remove(e);else if(n){var r="";if(n.getType&&(r=n.getType()),t&&this.get("enabledStack")){var i=Object(w.__assign)(Object(w.__assign)({},n.getModel()),{itemType:r}),o={};switch(r){case"node":o.nodes=[i],o.edges=[];for(var a=n.getEdges(),s=a.length-1;s>=0;s--)o.edges.push(Object(w.__assign)(Object(w.__assign)({},a[s].getModel()),{itemType:"edge"}));break;case"edge":o.edges=[i];break;case"combo":o.combos=[i]}this.pushStack("delete",{before:o,after:{}})}if(this.get("itemController").removeItem(e),"combo"===r){var l=Cn(this.get("comboTrees"));this.set("comboTrees",l)}}},t.prototype.addItem=function(e,t,n,r){void 0===n&&(n=!0),void 0===r&&(r=!0);var i=this.get("comboSorted");this.set("comboSorted",i&&!r);var o=this.get("itemController");if("group"===e){var a=t.groupId,s=t.nodes,l=t.type,u=t.zIndex,c=t.title,d=c;return Object(ce.isString)(c)&&(d={text:c}),this.get("customGroupControll").create(a,s,l,u,!0,d)}if(!t.id||!this.findById(t.id)){var h,f=this.get("comboTrees");if(f||(f=[]),"combo"===e){var p=this.get("itemMap"),g=!1;if(f.forEach((function(n){g||gn(n,(function(n){if(t.parentId===n.id){g=!0;var r=Object(w.__assign)({id:t.id,depth:n.depth+2},t);n.children?n.children.push(r):n.children=[r],t.depth=r.depth,h=o.addItem(e,t)}var i=p[n.id];return g&&i&&i.getType&&"combo"===i.getType()&&o.updateCombo(i,n.children),!0}))})),!g){var v=Object(w.__assign)({id:t.id,depth:0},t);t.depth=v.depth,f.push(v),h=o.addItem(e,t)}this.set("comboTrees",f)}else if("node"===e&&Object(ce.isString)(t.comboId)&&f){var m;if((m=this.findById(t.comboId))&&m.getType&&"combo"!==m.getType())return void console.warn("'"+t.comboId+"' is a not id of a combo in the graph, the node will be added without combo.");h=o.addItem(e,t);var y=this.get("itemMap"),b=!1,C=!1;(f||[]).forEach((function(e){C||b||gn(e,(function(e){if(e.id===t.id)return C=!0,!1;if(t.comboId===e.id&&!C){b=!0;var n=Object(ce.clone)(t);n.itemType="node",e.children?e.children.push(n):e.children=[n],t.depth=e.depth+1}return b&&y[e.id].getType&&"combo"===y[e.id].getType()&&o.updateCombo(y[e.id],e.children),!0}))}))}else h=o.addItem(e,t);("node"===e&&t.comboId||"combo"===e&&t.parentId)&&(m=this.findById(t.comboId||t.parentId))&&m.addChild(h);var A=this.get("combos");if(A&&A.length>0&&this.sortCombos(),this.autoPaint(),n&&this.get("enabledStack")){var _=Object(w.__assign)(Object(w.__assign)({},h.getModel()),{itemType:e}),I={};switch(e){case"node":I.nodes=[_];break;case"edge":I.edges=[_];break;case"combo":I.combos=[_]}this.pushStack("add",{before:{},after:I})}return h}console.warn("This item exists already. Be sure the id %c"+t.id+"%c is unique.","font-size: 20px; color: red;","")},t.prototype.add=function(e,t,n,r){return void 0===n&&(n=!0),void 0===r&&(r=!0),this.addItem(e,t,n,r)},t.prototype.updateItem=function(e,t,n){var r=this;void 0===n&&(n=!0);var i,o=this.get("itemController");i=Object(ce.isString)(e)?this.findById(e):e;var a=Object(ce.clone)(i.getModel()),s="";i.getType&&(s=i.getType());var l=Object(w.__spreadArrays)(i.getStates());if("combo"===s&&Object(ce.each)(l,(function(e){return r.setItemState(i,e,!1)})),o.updateItem(i,t),"combo"===s&&Object(ce.each)(l,(function(e){return r.setItemState(i,e,!0)})),n&&this.get("enabledStack")){var u={nodes:[],edges:[],combos:[]},c={nodes:[],edges:[],combos:[]},d=Object(w.__assign)({id:a.id},t);switch(s){case"node":u.nodes.push(a),c.nodes.push(d);break;case"edge":u.edges.push(a),c.edges.push(d);break;case"combo":u.combos.push(a),c.combos.push(d)}"node"===s&&u.nodes.push(a),this.pushStack("update",{before:u,after:c})}},t.prototype.update=function(e,t,n){void 0===n&&(n=!0),this.updateItem(e,t,n)},t.prototype.setItemState=function(e,t,n){Object(ce.isString)(e)&&(e=this.findById(e)),this.get("itemController").setItemState(e,t,n);var r=this.get("stateController");Object(ce.isString)(n)?r.updateState(e,t+":"+n,!0):r.updateState(e,t,n)},t.prototype.priorityState=function(e,t){this.get("itemController").priorityState(e,t)},t.prototype.data=function(e){this.set("data",e)},t.prototype.render=function(){var e=this;this.set("comboSorted",!1);var t=this.get("data");if(this.get("enabledStack")&&this.clearStack(),!t)throw new Error("data must be defined first");var n=t.nodes,r=void 0===n?[]:n,i=t.edges,o=void 0===i?[]:i,a=t.combos,s=void 0===a?[]:a;if(this.clear(),this.emit("beforerender"),Object(ce.each)(r,(function(t){e.add("node",t,!1,!1)})),s&&0!==s.length){var l=bn(s,r);this.set("comboTrees",l),e.addCombos(s)}Object(ce.each)(o,(function(t){e.add("edge",t,!1,!1)}));var u=e.get("animate");function c(){e.get("fitView")?e.fitView():e.get("fitCenter")&&e.fitCenter(),e.autoPaint(),e.emit("afterrender"),(e.get("fitView")||e.get("fitCenter"))&&e.set("animate",u)}if((e.get("fitView")||e.get("fitCenter"))&&e.set("animate",!1),e.get("layoutController").layout(c)||c(),this.get("groupByTypes")||(s&&0!==s.length?this.sortCombos():t.nodes&&t.edges&&t.nodes.length<t.edges.length?this.getNodes().forEach((function(e){e.toFront()})):this.getEdges().forEach((function(e){e.toBack()}))),t.nodes&&t.nodes.filter((function(e){return e.groupId})).length>0){var d=e.get("groupType");this.renderCustomGroup(t,d)}this.get("enabledStack")&&this.pushStack("render")},t.prototype.read=function(e){this.data(e),this.render()},t.prototype.diffItems=function(e,t,n){var r,i=this,o=this.get("itemMap");Object(ce.each)(n,(function(n){if(r=o[n.id]){if(i.get("animate")&&"node"===e){var a=r.getContainer().getMatrix();a||(a=[1,0,0,0,1,0,0,0,1]),r.set("originAttrs",{x:a[6],y:a[7]})}i.updateItem(r,n,!1)}else r=i.addItem(e,n,!1);t[e+"s"].push(r)}))},t.prototype.changeData=function(e,t){void 0===t&&(t=!0);var n=this;if(!e)return this;t&&this.get("enabledStack")&&this.pushStack("changedata",{before:n.save(),after:e}),this.set("comboSorted",!1),this.getNodes().map((function(e){return n.clearItemStates(e)})),this.getEdges().map((function(e){return n.clearItemStates(e)}));var r=this.get("canvas"),i=r.get("localRefresh");r.set("localRefresh",!1),n.get("data")||(n.data(e),n.render());var o=this.get("itemMap"),a={nodes:[],edges:[]},s=e.combos;if(s){var l=bn(s,e.nodes);this.set("comboTrees",l)}this.diffItems("node",a,e.nodes),Object(ce.each)(o,(function(e,t){o[t].getModel().depth=0,e.getType&&"edge"===e.getType()||(e.getType&&"combo"===e.getType()?(delete o[t],e.destroy()):a.nodes.indexOf(e)<0&&(delete o[t],n.remove(e,!1)))}));for(var u=this.getCombos(),c=u.length-1;c>=0;c--)u[c].destroyed&&u.splice(c,1);s&&(n.addCombos(s),this.get("groupByTypes")||this.sortCombos()),this.diffItems("edge",a,e.edges),Object(ce.each)(o,(function(e,t){(!e.getType||"node"!==e.getType()&&"combo"!==e.getType())&&a.edges.indexOf(e)<0&&(delete o[t],n.remove(e,!1))})),this.set({nodes:a.nodes,edges:a.edges});var d=this.get("layoutController");return d.changeData(),n.get("animate")&&!d.getLayoutType()?n.positionsAnimate():n.autoPaint(),setTimeout((function(){r.set("localRefresh",i)}),16),this},t.prototype.addCombos=function(e){var t=this.get("comboTrees");this.get("itemController").addCombos(t,e)},t.prototype.createCombo=function(e,t){var n=this;this.set("comboSorted",!1);var r,i="";if(e){if(Object(ce.isString)(e))i=e,r={id:e};else{if(!(i=e.id))return void console.warn("Create combo failed. Please assign a unique string id for the adding combo.");r=e}var o=t.map((function(e){var t=n.findById(e),r="";t.getType&&(r=t.getType());var o={id:t.getID(),itemType:r};return"combo"===r?o.parentId=i:"node"===r&&(o.comboId=i),o}));r.children=o,this.addItem("combo",r,!1),this.set("comboSorted",!1);var a=this.get("comboTrees");(a||[]).forEach((function(e){gn(e,(function(e){return e.id!==i||(e.itemType="combo",e.children=o,!1)}))})),a&&this.sortCombos()}},t.prototype.uncombo=function(e){var t,n=this,r=e;if(Object(ce.isString)(e)&&(r=this.findById(e)),!r||r.getType&&"combo"!==r.getType())console.warn("The item is not a combo!");else{var i=r.getModel().parentId,o=this.get("comboTrees");o||(o=[]);var a,s=this.get("itemMap"),l=r.get("id"),u=[],c=this.get("combos"),d=this.findById(i);if(o.forEach((function(t){a||gn(t,(function(t){var o;if(t.id===l){a=t,r.getEdges().forEach((function(e){n.removeItem(e,!1)}));var h=c.indexOf(e);c.splice(h,1),delete s[l],r.destroy()}return!i||!a||t.id!==i||(d.removeCombo(r),-1!==(h=(u=t.children).indexOf(a))&&u.splice(h,1),null===(o=a.children)||void 0===o||o.forEach((function(e){var t=n.findById(e.id),r=t.getModel();t.getType&&"combo"===t.getType()?(e.parentId=i,delete e.comboId,r.parentId=i,delete r.comboId):t.getType&&"node"===t.getType()&&(e.comboId=i,r.comboId=i),d.addChild(t),u.push(e)})),!1)}))})),!i&&a){var h=o.indexOf(a);o.splice(h,1),null===(t=a.children)||void 0===t||t.forEach((function(e){e.parentId=void 0,n.findById(e.id).getModel().parentId=void 0,"node"!==e.itemType&&o.push(e)}))}}},t.prototype.updateCombos=function(){var e=this,t=this.get("comboTrees"),n=this.get("itemController"),r=this.get("itemMap");(t||[]).forEach((function(t){gn(t,(function(t){if(!t)return!0;var i=r[t.id];if(i&&i.getType&&"combo"===i.getType()){var o=Object(w.__spreadArrays)(i.getStates());Object(ce.each)(o,(function(t){return e.setItemState(i,t,!1)})),n.updateCombo(i,t.children),Object(ce.each)(o,(function(t){return e.setItemState(i,t,!0)}))}return!0}))})),this.sortCombos()},t.prototype.updateCombo=function(e){var t,n=this,r=e;if(Object(ce.isString)(e)&&(r=this.findById(e)),!r||r.getType&&"combo"!==r.getType())console.warn("The item to be updated is not a combo!");else{t=r.get("id");var i=this.get("comboTrees"),o=this.get("itemController"),a=this.get("itemMap");(i||[]).forEach((function(e){gn(e,(function(e){if(!e)return!0;var r=a[e.id];if(t===e.id&&r&&r.getType&&"combo"===r.getType()){var i=Object(w.__spreadArrays)(r.getStates());Object(ce.each)(i,(function(e){r.getStateStyle(e)&&n.setItemState(r,e,!1)})),o.updateCombo(r,e.children),Object(ce.each)(i,(function(e){r.getStateStyle(e)&&n.setItemState(r,e,!0)})),t&&(t=e.parentId)}return!0}))}))}},t.prototype.updateComboTree=function(e,t){var n;this.set("comboSorted",!1);var r=(n=Object(ce.isString)(e)?this.findById(e):e).getModel(),i=r.comboId||r.parentId;if(t&&n.getType&&"combo"===n.getType()){var o,a=this.get("comboTrees"),s=!0;if((a||[]).forEach((function(e){o||pn(e,(function(e){if(!o)return e.id===n.getID()&&(o=e),!0}))})),pn(o,(function(e){return e.id!==t||(s=!1,!1)})),!s)return void console.warn("Failed to update the combo tree! The parentId points to a descendant of the combo!")}if(r.parentId||r.comboId){var l=this.findById(r.parentId||r.comboId);l&&l.removeChild(n)}var u,c="";n.getType&&(c=n.getType()),"combo"===c?r.parentId=t:"node"===c&&(r.comboId=t),t&&(u=this.findById(t))&&u.addChild(n),i&&(u=this.findById(i))&&u.removeChild(n);var d=Cn(this.get("comboTrees"),r.id,t);this.set("comboTrees",d),this.updateCombos()},t.prototype.renderCustomGroup=function(e,t){var n=this,r=e.groups,i=e.nodes,o=void 0===i?[]:i,a=10;if(r){this.set({groups:r});var s=function(e){var t=Bt()(e.groups,"id"),n=Bt()(e.groups,"parentId"),r={};for(var i in n)if(i){var o=n[i],a=t[i];if(a&&o){var s=Object(w.__spreadArrays)(o,a);r[i]=s}else o&&(r[i]=o)}var l=Object(w.__assign)(Object(w.__assign)({},t),r),u={};for(var c in l)c&&"undefined"!==c&&(o=l[c].map((function(e){return e.id})),u[c]=o);var d={},h=function(t){if(!t||"undefined"===t)return"continue";for(var n=u[t],r=[],i=0,o=n;i<o.length;i++){var a=o[i],s=l[a].map((function(e){return e.id}));r.push.apply(r,s)}var c=e.nodes?e.nodes.filter((function(e){return r.indexOf(e.groupId)>-1||r.indexOf(e.parentId)>-1})).map((function(e){return e.id})):[];d[t]=c};for(var c in u)h(c);return d}(e);Object.keys(s).forEach((function(e){var r=s[e];n.get("customGroupControll").create(e,r,t,a),a--})),this.get("customGroup").sort()}else{var l=o.filter((function(e){return e.groupId})),u=[],c=Object(ce.groupBy)(l,"groupId");Object.keys(c).forEach((function(e){var r=c[e].map((function(e){return e.id}));n.get("customGroupControll").create(e,r,t,a),a--,u.find((function(t){return t.id===e}))||u.push({id:e})})),this.set({groups:u})}},t.prototype.save=function(){var e=[],t=[],n=[];return Object(ce.each)(this.get("nodes"),(function(t){e.push(t.getModel())})),Object(ce.each)(this.get("edges"),(function(e){t.push(e.getModel())})),Object(ce.each)(this.get("combos"),(function(e){n.push(e.getModel())})),{nodes:e,edges:t,combos:n,groups:this.get("groups")}},t.prototype.changeSize=function(e,t){return this.get("viewController").changeSize(e,t),this},t.prototype.refresh=function(){if(this.emit("beforegraphrefresh"),this.get("animate"))this.positionsAnimate();else{var e=this.get("nodes"),t=this.get("edges"),n=this.get("edges");Object(ce.each)(e,(function(e){e.refresh()})),Object(ce.each)(t,(function(e){e.refresh()})),Object(ce.each)(n,(function(e){e.refresh()}))}this.emit("aftergraphrefresh"),this.autoPaint()},t.prototype.getNodes=function(){return this.get("nodes")},t.prototype.getEdges=function(){return this.get("edges")},t.prototype.getCombos=function(){return this.get("combos")},t.prototype.getComboChildren=function(e){if(Object(ce.isString)(e)&&(e=this.findById(e)),e&&(!e.getType||"combo"===e.getType()))return e.getChildren();console.warn("The combo does not exist!")},t.prototype.positionsAnimate=function(){var e=this;e.emit("beforeanimate");var t=e.get("animateCfg"),n=t.onFrame,r=e.getNodes(),i=r.map((function(e){var t=e.getModel();return{id:t.id,x:t.x,y:t.y}}));e.isAnimating()&&e.stopAnimate(),e.get("canvas").animate((function(t){Object(ce.each)(i,(function(r){var i=e.findById(r.id);if(i&&!i.destroyed){var o=i.get("originAttrs"),a=i.get("model");if(!o){var s=i.getContainer().getMatrix();s||(s=[1,0,0,0,1,0,0,0,1]),o={x:s[6],y:s[7]},i.set("originAttrs",o)}if(n){var l=n(i,t,r,o);i.set("model",Object.assign(a,l))}else a.x=o.x+(r.x-o.x)*t,a.y=o.y+(r.y-o.y)*t}})),e.refreshPositions()}),{duration:t.duration,easing:t.easing,callback:function(){Object(ce.each)(r,(function(e){e.set("originAttrs",null)})),t.callback&&t.callback(),e.emit("afteranimate"),e.animating=!1}})},t.prototype.refreshPositions=function(){this.emit("beforegraphrefreshposition");var e,t=this.get("nodes"),n=this.get("edges"),r=this.get("vedges"),i=this.get("combos"),o={};Object(ce.each)(t,(function(t){e=t.getModel();var n=t.get("originAttrs");n&&e.x===n.x&&e.y===n.y||(t.updatePosition({x:e.x,y:e.y}),o[e.id]=!0,e.comboId&&(o[e.comboId]=!0))})),i&&0!==i.length&&this.updateCombos(),Object(ce.each)(n,(function(e){var t=e.getSource().getModel(),n=e.getTarget().getModel();(o[t.id]||o[n.id]||e.getModel().isComboEdge)&&e.refresh()})),Object(ce.each)(r,(function(e){e.refresh()})),this.emit("aftergraphrefreshposition"),this.autoPaint()},t.prototype.stopAnimate=function(){this.get("canvas").stopAnimate()},t.prototype.isAnimating=function(){return this.animating},t.prototype.getZoom=function(){var e=this.get("group").getMatrix();return e?e[0]:1},t.prototype.getCurrentMode=function(){return this.get("modeController").getMode()},t.prototype.setMode=function(e){return this.get("modeController").setMode(e),this},t.prototype.clear=function(){return this.get("canvas").clear(),this.initGroups(),this.set({itemMap:{},nodes:[],edges:[],groups:[],combos:[],comboTrees:[]}),this.emit("afterrender"),this},t.prototype.toDataURL=function(e,t){var n=this.get("canvas"),r=n.getRenderer(),i=n.get("el");e||(e="image/png");var o="";if("svg"===r){var a=i.cloneNode(!0),s=document.implementation.createDocumentType("svg","-//W3C//DTD SVG 1.1//EN","http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd"),l=document.implementation.createDocument("http://www.w3.org/2000/svg","svg",s);l.replaceChild(a,l.documentElement);var u=(new XMLSerializer).serializeToString(l);o="data:image/svg+xml;charset=utf8,"+encodeURIComponent(u)}else{var c=void 0,d=i.getContext("2d"),h=this.get("width"),f=this.get("height"),p=void 0;if(t){var g=window.devicePixelRatio;c=d.getImageData(0,0,h*g,f*g),p=d.globalCompositeOperation,d.globalCompositeOperation="destination-over",d.fillStyle=t,d.fillRect(0,0,h,f)}o=i.toDataURL(e),t&&(d.clearRect(0,0,h,f),d.putImageData(c,0,0),d.globalCompositeOperation=p)}return o},t.prototype.toFullDataURL=function(e,t,n){var r=this.get("group").getCanvasBBox(),i=r.height,o=r.width,a=this.get("renderer"),s=ye()('<id="virtual-image"></div>'),l=n?n.backgroundColor:void 0,u=n?n.padding:void 0;u?Object(ce.isNumber)(u)&&(u=[u,u,u,u]):u=[0,0,0,0];var c=i+u[0]+u[2],d=o+u[1]+u[3],h={container:s,height:c,width:d,quickHit:!0},f="svg"===a?new Rt.a(h):new Nt.a(h),p=this.get("group").clone(),g=Object(ce.clone)(p.getMatrix());g||(g=[1,0,0,0,1,0,0,0,1]);var v=(r.maxX+r.minX)/2,m=(r.maxY+r.minY)/2;zt.mat3.translate(g,g,[-v,-m]),zt.mat3.translate(g,g,[o/2+u[3],i/2+u[0]]),p.resetMatrix(),p.setMatrix(g),f.add(p);var y=f.get("el"),b="";t||(t="image/png"),setTimeout((function(){if("svg"===a){var n=y.cloneNode(!0),r=document.implementation.createDocumentType("svg","-//W3C//DTD SVG 1.1//EN","http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd"),i=document.implementation.createDocument("http://www.w3.org/2000/svg","svg",r);i.replaceChild(n,i.documentElement);var o=(new XMLSerializer).serializeToString(i);b="data:image/svg+xml;charset=utf8,"+encodeURIComponent(o)}else{var s=void 0,u=y.getContext("2d"),h=void 0;if(l){var f=window.devicePixelRatio;s=u.getImageData(0,0,d*f,c*f),h=u.globalCompositeOperation,u.globalCompositeOperation="destination-over",u.fillStyle=l,u.fillRect(0,0,d,c)}b=y.toDataURL(t),l&&(u.clearRect(0,0,d,c),u.putImageData(s,0,0),u.globalCompositeOperation=h)}null==e||e(b)}),16)},t.prototype.downloadFullImage=function(e,t,n){var r=this,i=this.get("group").getCanvasBBox(),o=i.height,a=i.width,s=this.get("renderer"),l=ye()('<id="virtual-image"></div>'),u=n?n.backgroundColor:void 0,c=n?n.padding:void 0;c?Object(ce.isNumber)(c)&&(c=[c,c,c,c]):c=[0,0,0,0];var d=o+c[0]+c[2],h=a+c[1]+c[3],f={container:l,height:d,width:h},p="svg"===s?new Rt.a(f):new Nt.a(f),g=this.get("group").clone(),v=Object(ce.clone)(g.getMatrix());v||(v=[1,0,0,0,1,0,0,0,1]);var m=(i.maxX+i.minX)/2,y=(i.maxY+i.minY)/2;zt.mat3.translate(v,v,[-m,-y]),zt.mat3.translate(v,v,[a/2+c[3],o/2+c[0]]),g.resetMatrix(),g.setMatrix(v),p.add(g);var b=p.get("el");t||(t="image/png"),setTimeout((function(){var n="";if("svg"===s){var i=b.cloneNode(!0),o=document.implementation.createDocumentType("svg","-//W3C//DTD SVG 1.1//EN","http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd"),a=document.implementation.createDocument("http://www.w3.org/2000/svg","svg",o);a.replaceChild(i,a.documentElement);var l=(new XMLSerializer).serializeToString(a);n="data:image/svg+xml;charset=utf8,"+encodeURIComponent(l)}else{var c=void 0,f=b.getContext("2d"),p=void 0;if(u){var g=window.devicePixelRatio;c=f.getImageData(0,0,h*g,d*g),p=f.globalCompositeOperation,f.globalCompositeOperation="destination-over",f.fillStyle=u,f.fillRect(0,0,h,d)}n=b.toDataURL(t),u&&(f.clearRect(0,0,h,d),f.putImageData(c,0,0),f.globalCompositeOperation=p)}var v=document.createElement("a"),m=(e||"graph")+("svg"===s?".svg":"."+t.split("/")[1]);r.dataURLToImage(n,s,v,m);var y=document.createEvent("MouseEvents");y.initEvent("click",!1,!1),v.dispatchEvent(y)}),16)},t.prototype.downloadImage=function(e,t,n){var r=this,i=this;i.isAnimating()&&i.stopAnimate();var o=i.get("canvas").getRenderer();t||(t="image/png");var a=(e||"graph")+("svg"===o?".svg":t.split("/")[1]),s=document.createElement("a");setTimeout((function(){var e=i.toDataURL(t,n);r.dataURLToImage(e,o,s,a);var l=document.createEvent("MouseEvents");l.initEvent("click",!1,!1),s.dispatchEvent(l)}),16)},t.prototype.dataURLToImage=function(e,t,n,r){if("undefined"!=typeof window)if(window.Blob&&window.URL&&"svg"!==t){var i=e.split(","),o="";if(i&&i.length>0){var a=i[0].match(/:(.*?);/);a&&a.length>=2&&(o=a[1])}for(var s=atob(i[1]),l=s.length,u=new Uint8Array(l);l--;)u[l]=s.charCodeAt(l);var c=new Blob([u],{type:o});window.navigator.msSaveBlob?window.navigator.msSaveBlob(c,r):n.addEventListener("click",(function(){n.download=r,n.href=window.URL.createObjectURL(c)}))}else n.addEventListener("click",(function(){n.download=r,n.href=e}))},t.prototype.updateLayout=function(e){var t,n=this.get("layoutController");Object(ce.isString)(e)?e={type:t=e}:t=e.type;var r=this.get("layout"),i=r?r.type:void 0;if(t&&i!==t)this.set("layout",e),n.changeLayout(t);else{var o={};Object.assign(o,r,e),o.type=i||"random",this.set("layout",o),n.updateLayoutCfg(o)}},t.prototype.destroyLayout=function(){this.get("layoutController").destroyLayout()},t.prototype.layout=function(){var e=this.get("layoutController"),t=this.get("layout");t&&(t.workerEnabled?e.layout():e.layoutMethod?e.relayout(!0):e.layout())},t.prototype.collapseCombo=function(e){var t=this;if(Object(ce.isString)(e)&&(e=this.findById(e)),e){var n=e.getModel();this.get("itemController").collapseCombo(e),n.collapsed=!0;var r=this.getEdges().concat(this.get("vedges")),i=[],o=[],a=this.get("comboTrees"),s=!1,l={};(a||[]).forEach((function(e){l[e.id]=e})),(a||[]).forEach((function(e){s||pn(e,(function(e){if(s&&l[e.id])return!1;if(n.parentId===e.id?(l={},e.children.forEach((function(e){l[e.id]=e}))):n.id===e.id&&(s=!0),s){var r=t.findById(e.id);r&&r.getType&&"combo"===r.getType()&&(i=i.concat(r.getNodes()),o=o.concat(r.getCombos()))}return!0}))}));var u={},c=[];r.forEach((function(e){if(!e.isVisible()||e.getModel().isVEdge){var r=e.getSource(),a=e.getTarget();if((i.includes(r)||o.includes(r))&&!i.includes(a)&&!o.includes(a)||r.getModel().id===n.id){if((h=e.getModel()).isVEdge)return void t.removeItem(e,!1);for(var s=a.getModel();!a.isVisible();){if(!(a=t.findById(s.parentId||s.comboId))||!s.parentId&&!s.comboId)return;s=a.getModel()}var l=s.id;if(u[n.id+"-"+l])return void(u[n.id+"-"+l]+=h.size||1);var d=t.addItem("vedge",{source:n.id,target:l,isVEdge:!0},!1);u[n.id+"-"+l]=h.size||1,c.push(d)}else if(!i.includes(r)&&!o.includes(r)&&(i.includes(a)||o.includes(a))||a.getModel().id===n.id){var h;if((h=e.getModel()).isVEdge)return void t.removeItem(e,!1);for(var f=r.getModel();!r.isVisible();){if(!(r=t.findById(f.parentId||f.comboId))||!f.parentId&&!f.comboId)return;f=r.getModel()}var p=f.id;if(u[p+"-"+n.id])return void(u[p+"-"+n.id]+=h.size||1);d=t.addItem("vedge",{target:n.id,source:p,isVEdge:!0},!1),u[p+"-"+n.id]=h.size||1,c.push(d)}}})),c.forEach((function(e){var n=e.getModel();t.updateItem(e,{size:u[n.source+"-"+n.target]},!1)}))}else console.warn("The combo to be collapsed does not exist!")},t.prototype.expandCombo=function(e){var t=this;if(Object(ce.isString)(e)&&(e=this.findById(e)),!e||e.getType&&"combo"!==e.getType())console.warn("The combo to be collapsed does not exist!");else{var n=e.getModel();this.get("itemController").expandCombo(e),n.collapsed=!1;var r=this.getEdges().concat(this.get("vedges")),i=[],o=[],a=this.get("comboTrees"),s=!1,l={};(a||[]).forEach((function(e){l[e.id]=e})),(a||[]).forEach((function(e){s||pn(e,(function(e){if(s&&l[e.id])return!1;if(n.parentId===e.id?(l={},e.children.forEach((function(e){l[e.id]=e}))):n.id===e.id&&(s=!0),s){var r=t.findById(e.id);r&&r.getType&&"combo"===r.getType()&&(i=i.concat(r.getNodes()),o=o.concat(r.getCombos()))}return!0}))}));var u={},c={};r.forEach((function(e){if(!e.isVisible()||e.getModel().isVEdge){var r=e.getSource(),a=e.getTarget(),s=r.get("id"),l=a.get("id");if((i.includes(r)||o.includes(r))&&!i.includes(a)&&!o.includes(a)||s===n.id){if(e.getModel().isVEdge)return void t.removeItem(e,!1);for(var d=a.getModel();!a.isVisible();){if(!(a=t.findById(d.comboId||d.parentId))||!d.parentId&&!d.comboId)return;d=a.getModel()}l=d.id;for(var h=r.getModel();!r.isVisible();){if(!(r=t.findById(h.comboId||h.parentId))||!h.parentId&&!h.comboId)return;if(h.comboId===n.id||h.parentId===n.id)break;h=r.getModel()}if(s=h.id,l){if(u[p=s+"-"+l])return u[p]+=e.getModel().size||1,void t.updateItem(c[p],{size:u[p]},!1);var f=t.addItem("vedge",{source:s,target:l,isVEdge:!0},!1);u[p]=e.getModel().size||1,c[p]=f}}else if(!i.includes(r)&&!o.includes(r)&&(i.includes(a)||o.includes(a))||l===n.id){if(e.getModel().isVEdge)return void t.removeItem(e,!1);for(h=r.getModel();!r.isVisible();){if(!(r=t.findById(h.comboId||h.parentId))||!h.parentId&&!h.comboId)return;h=r.getModel()}for(s=h.id,d=a.getModel();!a.isVisible();){if(!(a=t.findById(d.comboId||d.parentId))||!d.parentId&&!d.comboId)return;if(d.comboId===n.id||d.parentId===n.id)break;d=a.getModel()}if(l=d.id,s){var p;if(u[p=s+"-"+l])return u[p]+=e.getModel().size||1,void t.updateItem(c[p],{size:u[p]},!1);f=t.addItem("vedge",{target:l,source:s,isVEdge:!0},!1),u[p]=e.getModel().size||1,c[p]=f}}else(i.includes(r)||o.includes(r))&&(i.includes(a)||o.includes(a))&&r.isVisible()&&a.isVisible()&&e.show()}}))}},t.prototype.collapseExpandCombo=function(e){if(Object(ce.isString)(e)&&(e=this.findById(e)),!e.getType||"combo"===e.getType()){for(var t=e.getModel(),n=this.findById(t.parentId);n;){var r=n.getModel();if(r.collapsed)return console.warn("Fail to expand the combo since it's ancestor combo is collapsed."),void(n=void 0);n=this.findById(r.parentId)}t.collapsed?this.expandCombo(e):this.collapseCombo(e),this.updateCombo(e)}},t.prototype.collapseGroup=function(e){this.get("customGroupControll").collapseGroup(e)},t.prototype.expandGroup=function(e){this.get("customGroupControll").expandGroup(e)},t.prototype.addPlugin=function(e){e.destroyed||(this.get("plugins").push(e),e.initPlugin(this))},t.prototype.removePlugin=function(e){var t=this.get("plugins"),n=t.indexOf(e);n>=0&&(e.destroyPlugin(),t.splice(n,1))},t.prototype.sortCombos=function(){var e=this;if(!this.get("comboSorted")){this.set("comboSorted",!0);var t=[],n={};(this.get("comboTrees")||[]).forEach((function(e){pn(e,(function(e){return t[e.depth]?t[e.depth].push(e.id):t[e.depth]=[e.id],n[e.id]=e.depth,!0}))})),(this.getEdges().concat(this.get("vedges"))||[]).forEach((function(e){var r=e.getModel(),i=n[r.source]||0,o=n[r.target]||0,a=Math.max(i,o);t[a]?t[a].push(r.id):t[a]=[r.id]})),t.forEach((function(t){if(t&&t.length)for(var n=t.length-1;n>=0;n--){var r=e.findById(t[n]);null==r||r.toFront()}}))}},t.prototype.getNeighbors=function(e,t){var n=e;return Object(ce.isString)(e)&&(n=this.findById(e)),n.getNeighbors(t)},t.prototype.getNodeDegree=function(e,t){void 0===t&&(t=void 0);var n=e;Object(ce.isString)(e)&&(n=this.findById(e));var r=this.get("degrees");r||(r=yo(this)),this.set("degees",r);var i,o=r[n.getID()];switch(t){case"in":i=o.inDegree;break;case"out":i=o.outDegree;break;case"all":i=o;break;default:i=o.degree}return i},t.prototype.getUndoStack=function(){return this.undoStack},t.prototype.getRedoStack=function(){return this.redoStack},t.prototype.getStackData=function(){return this.get("enabledStack")?{undoStack:this.undoStack.toArray(),redoStack:this.redoStack.toArray()}:null},t.prototype.clearStack=function(){this.get("enabledStack")&&(this.undoStack.clear(),this.redoStack.clear())},t.prototype.pushStack=function(e,t,n){if(void 0===e&&(e="update"),void 0===n&&(n="undo"),this.get("enabledStack")){var r=t?Object(ce.clone)(t):{before:{},after:Object(ce.clone)(this.save())};"redo"===n?this.redoStack.push({action:e,data:r}):this.undoStack.push({action:e,data:r}),this.emit("stackchange",{undoStack:this.undoStack,redoStack:this.redoStack})}else console.warn("请先启用 undo & redo 功能，在实例化 Graph 时候配置 enabledStack: true !")},t.prototype.getAdjMatrix=function(e,t){void 0===e&&(e=!0),void 0===t&&(t=this.get("directed"));var n=this.get("adjMatrix");return n&&e||(n=Io(this,t),this.set("adjMatrix",n)),n},t.prototype.getShortestPathMatrix=function(e,t){void 0===e&&(e=!0),void 0===t&&(t=this.get("directed"));var n=this.get("adjMatrix"),r=this.get("shortestPathMatrix");return n&&e||(n=Io(this,t),this.set("adjMatrix",n)),r&&e||(r=xo(this,t),this.set("shortestPathMatrix",r)),r},t.prototype.destroy=function(){this.clear(),this.clearStack(),Object(ce.each)(this.get("plugins"),(function(e){e.destroyPlugin()}));var e=this.get("tooltips");if(e)for(var t=0;t<e.length;t++){var n=e[t];if(n){var r=n.parentElement;r&&r.removeChild(n)}}this.get("eventController").destroy(),this.get("itemController").destroy(),this.get("modeController").destroy(),this.get("viewController").destroy(),this.get("stateController").destroy(),this.get("layoutController").destroy(),this.get("customGroupControll").destroy(),this.get("canvas").destroy(),this.get("graphWaterMarker")&&this.get("graphWaterMarker").destroy(),document.querySelector(".g6-graph-watermarker")&&document.querySelector(".g6-graph-watermarker").remove(),this.cfg=null,this.destroyed=!0,this.redoStack=null,this.undoStack=null},t.prototype.createHull=function(e){var t=this.get("hullGroup"),n=this.get("hullMap");if(n||(n={},this.set("hullMap",n)),t||((t=this.get("group").addGroup({id:"hullGroup"})).toBack(),this.set("hullGroup",t)),n[e.id])return console.warn("Existed hull id."),n[e.id];var r=t.addGroup({id:e.id+"-container"}),i=new Do(this,Object(w.__assign)(Object(w.__assign)({},e),{group:r}));return n[i.id]=i,i},t.prototype.getHulls=function(){return this.get("hullMap")},t.prototype.getHullById=function(e){return this.get("hullMap")[e]},t.prototype.removeHull=function(e){var t;t=Object(ce.isString)(e)?this.getHullById(e):e,delete this.get("hullMap")[t.id],t.destroy()},t.prototype.setImageWaterMarker=function(e,t){void 0===e&&(e=J.waterMarkerImage);var n=this.get("container");Object(ce.isString)(n)&&(n=document.getElementById(n)),n.style.position||(n.style.position="relative");var r=this.get("graphWaterMarker"),i=Object(ce.deepMix)({},J.imageWaterMarkerConfig,t),o=i.width,a=i.height,s=i.compatible,l=i.image;if(!r){var u={container:n,width:o,height:a,capture:!1},c=this.get("pixelRatio");c&&(u.pixelRatio=c),r=new Nt.a(u),this.set("graphWaterMarker",r)}r.get("el").style.display="none";var d=r.get("context"),h=l.rotate,f=l.x,p=l.y;d.rotate(-h*Math.PI/180);var g=new Image;g.crossOrigin="anonymous",g.src=e,g.onload=function(){if(d.drawImage(g,f,p,l.width,l.height),d.rotate(h*Math.PI/180),s)n.style.cssText="background-image: url("+r.get("el").toDataURL("image/png")+");background-repeat:repeat;";else{var e=document.querySelector(".g6-graph-watermarker");e||((e=document.createElement("div")).className="g6-graph-watermarker"),e.className="g6-graph-watermarker",r.destroyed||(e.style.cssText="background-image: url("+r.get("el").toDataURL("image/png")+");background-repeat:repeat;position:absolute;top:0;bottom:0;left:0;right:0;pointer-events:none;z-index:-1;",n.appendChild(e))}}},t.prototype.setTextWaterMarker=function(e,t){var n=this.get("container");Object(ce.isString)(n)&&(n=document.getElementById(n)),n.style.position||(n.style.position="relative");var r=this.get("graphWaterMarker"),i=Object(ce.deepMix)({},J.textWaterMarkerConfig,t),o=i.width,a=i.height,s=i.compatible,l=i.text;if(!r){var u={container:n,width:o,height:a,capture:!1},c=this.get("pixelRatio");c&&(u.pixelRatio=c),r=new Nt.a(u),this.set("graphWaterMarker",r)}r.get("el").style.display="none";var d=r.get("context"),h=l.rotate,f=l.fill,p=l.fontFamily,g=l.fontSize,v=l.baseline,m=l.x,y=l.y,b=l.lineHeight;d.rotate(-h*Math.PI/180),d.font=g+"px "+p,d.fillStyle=f,d.textBaseline=v;for(var C=e.length-1;C>=0;C--)d.fillText(e[C],m,y+C*b);if(d.rotate(h*Math.PI/180),s)n.style.cssText="background-image: url("+r.get("el").toDataURL("image/png")+");background-repeat:repeat;";else{var A=document.querySelector(".g6-graph-watermarker");A||((A=document.createElement("div")).className="g6-graph-watermarker"),A.style.cssText="background-image: url("+r.get("el").toDataURL("image/png")+");background-repeat:repeat;position:absolute;top:0;bottom:0;left:0;right:0;pointer-events:none;z-index:99;",n.appendChild(A)}},t}(Dt.a),No=n(135),jo=n.n(No),Ro=function(e){function t(t){var n=e.call(this,t)||this;return n.layoutAnimating=!1,n.set("removeList",[]),n.set("layoutMethod",n.getLayout()),n}return Object(w.__extends)(t,e),t.prototype.getLayout=function(){var e=this.get("layout");return e?"function"==typeof e?e:(e.type||(e.type="dendrogram"),e.direction||(e.direction="TB"),e.radial?function(t){var n=jo.a[e.type](t,e);return vn(n),n}:function(t){return jo.a[e.type](t,e)}):null},t.indexOfChild=function(e,t){var n=-1;return Object(p.each)(e,(function(e,r){if(t===e.id)return n=r,!1})),n},t.prototype.getDefaultCfg=function(){var t=e.prototype.getDefaultCfg.call(this);return t.animate=!0,t},t.prototype.innerAddChild=function(e,t,n){var r=this,i=e.data;i&&(i.x=e.x,i.y=e.y,i.depth=e.depth);var o=r.addItem("node",i,!1);if(t){if(o.set("parent",t),n){var a=t.get("originAttrs");if(a)o.set("originAttrs",a);else{var s=t.getModel();o.set("originAttrs",{x:s.x,y:s.y})}}var l=t.get("children");l?l.push(o):t.set("children",[o]),r.addItem("edge",{source:t,target:o,id:t.get("id")+":"+o.get("id")},!1)}return Object(p.each)(e.children||[],(function(e){r.innerAddChild(e,o,n)})),r.emit("afteraddchild",{item:o,parent:t}),o},t.prototype.innerUpdateChild=function(e,n,r){var i=this,o=i.findById(e.id);if(o){Object(p.each)(e.children||[],(function(e){i.innerUpdateChild(e,o,r)}));var a,s,l=o.get("children");if(l&&l.length>0)for(var u=l.length-1;u>=0;u--){var c=l[u].getModel();-1===t.indexOfChild(e.children||[],c.id)&&(i.innerRemoveChild(c.id,{x:e.x,y:e.y},r),l.splice(u,1))}o.get("originAttrs")&&(a=o.get("originAttrs").x,s=o.get("originAttrs").y);var d=o.getModel();r&&o.set("originAttrs",{x:d.x,y:d.y}),o.set("model",e.data),a===e.x&&s===e.y||o.updatePosition({x:e.x,y:e.y})}else i.innerAddChild(e,n,r)},t.prototype.innerRemoveChild=function(e,t,n){var r=this,i=r.findById(e);if(i)if(Object(p.each)(i.get("children"),(function(e){r.innerRemoveChild(e.getModel().id,t,n)})),n){var o=i.getModel();i.set("to",t),i.set("originAttrs",{x:o.x,y:o.y}),r.get("removeList").push(i)}else r.removeItem(i)},t.prototype.changeData=function(e){var t=this;this.getNodes().map((function(e){return t.clearItemStates(e)})),this.getEdges().map((function(e){return t.clearItemStates(e)})),e?(t.data(e),t.render()):t.layout(this.get("fitView"))},t.prototype.changeLayout=function(e){console.warn("Please call updateLayout instead of changeLayout. changeLayout will be discarded soon"),this.updateLayout(e)},t.prototype.updateLayout=function(e){e?(this.set("layout",e),this.set("layoutMethod",this.getLayout()),this.layout()):console.warn("layout cannot be null")},t.prototype.refreshLayout=function(e){console.warn("Please call layout instead of refreshLayout. refreshLayout will be discarded soon"),this.layout(e)},t.prototype.layout=function(e){var t=this.get("data"),n=this.get("layoutMethod")(t,this.get("layout")),r=this.get("animate");this.emit("beforerefreshlayout",{data:t,layoutData:n}),this.emit("beforelayout"),this.innerUpdateChild(n,void 0,r),e&&this.get("viewController").fitView(),r?this.layoutAnimate(n):(this.refresh(),this.paint()),this.emit("afterrefreshlayout",{data:t,layoutData:n}),this.emit("afterlayout")},t.prototype.addChild=function(e,t){this.emit("beforeaddchild",{model:e,parent:t}),Object(p.isString)(t)||(t=t.get("id"));var n=this.findDataById(t);n&&(n.children||(n.children=[]),n.children.push(e),this.changeData())},t.prototype.updateChild=function(e,n){if(n&&this.findById(n)){var r=this.findById(n).getModel(),i=this.findById(e.id);if(r.children||(r.children=[]),i){var o=t.indexOfChild(r.children,e.id);r.children[o]=e}else r.children.push(e);this.changeData()}else this.changeData(e)},t.prototype.removeChild=function(e){var n=this.findById(e);if(n){var r=n.get("parent");if(r&&!r.destroyed){var i=this.findDataById(r.get("id")),o=i&&i.children||[],a=n.getModel(),s=t.indexOfChild(o,a.id);o.splice(s,1)}this.changeData()}},t.prototype.findDataById=function(e,t){var n=this;if(t||(t=n.get("data")),e===t.id)return t;var r=null;return Object(p.each)(t.children||[],(function(t){return t.id===e?(r=t,!1):!(r=n.findDataById(e,t))&&void 0})),r},t.prototype.layoutAnimate=function(e,t){var n=this,r=this.get("animateCfg");n.emit("beforeanimate",{data:e}),n.getEdges().forEach((function(e){var t=e.get("model");t.sourceAnchor||(t.sourceAnchor=e.get("sourceAnchorIndex"))})),this.get("canvas").animate((function(r){pn(e,(function(i){var o=n.findById(i.id);if(o){var a=o.get("originAttrs"),s=o.get("model");if(a||(a={x:s.x,y:s.y},o.set("originAttrs",a)),t){var l=t(o,r,a,e);o.set("model",Object.assign(s,l))}else s.x=a.x+(i.x-a.x)*r,s.y=a.y+(i.y-a.y)*r}return!0})),Object(p.each)(n.get("removeList"),(function(e){var t=e.getModel(),n=e.get("originAttrs"),i=e.get("to");t.x=n.x+(i.x-n.x)*r,t.y=n.y+(i.y-n.y)*r})),n.refreshPositions()}),{duration:r.duration,easing:r.ease,callback:function(){Object(p.each)(n.getNodes(),(function(e){e.set("originAttrs",null)})),Object(p.each)(n.get("removeList"),(function(e){n.removeItem(e)})),n.set("removeList",[]),r.callback&&r.callback(),n.emit("afteranimate",{data:e})},delay:r.delay})},t.prototype.stopLayoutAnimate=function(){this.get("canvas").stopAnimate(),this.emit("layoutanimateend",{data:this.get("data")}),this.layoutAnimating=!1},t.prototype.isLayoutAnimating=function(){return this.layoutAnimating},t.prototype.render=function(){if(!this.get("data"))throw new Error("data must be defined first");this.clear(),this.emit("beforerender"),this.layout(this.get("fitView")),this.emit("afterrender")},t.prototype.save=function(){return this.get("data")},t}(Po),zo=["startArrow","endArrow"],Fo={options:{},itemType:"",type:"",getCustomConfig:function(e){return{}},getOptions:function(e){return Object(ce.deepMix)({labelCfg:{style:{fontFamily:"undefined"!=typeof window&&window.getComputedStyle(document.body,null).getPropertyValue("font-family")||"Arial, sans-serif"}},descriptionCfg:{style:{fontFamily:"undefined"!=typeof window&&window.getComputedStyle(document.body,null).getPropertyValue("font-family")||"Arial, sans-serif"}}},this.options,this.getCustomConfig(e)||{},e)},draw:function(e,t){var n=this.drawShape(e,t);return n.set("className",this.itemType+"-shape"),e.label&&this.drawLabel(e,t).set("className",this.itemType+"-label"),n},afterDraw:function(e,t,n){},drawShape:function(e,t){return null},drawLabel:function(e,t){var n=this.getOptions(e).labelCfg||{},r=this.getLabelStyle(e,n,t),i=r.rotate;delete r.rotate;var o=t.addShape("text",{attrs:r,draggable:!0,className:"text-shape",name:"text-shape"});if(i){var a=o.getBBox(),s=o.getMatrix();if(s||(s=[1,0,0,0,1,0,0,0,1]),r.rotateCenter)switch(r.rotateCenter){case"center":s=fe(s,[["t",-a.width/2,-a.height/2],["r",i],["t",a.width/2,a.height/2]]);break;case"lefttop":s=fe(s,[["t",-r.x,-r.y],["r",i],["t",r.x,r.y]]);break;case"leftcenter":s=fe(s,[["t",-r.x,-r.y-a.height/2],["r",i],["t",r.x,r.y+a.height/2]]);break;default:s=fe(s,[["t",-a.width/2,-a.height/2],["r",i],["t",a.width/2,a.height/2]])}else s=fe(s,[["t",-r.x,-r.y-a.height/2],["r",i],["t",r.x,r.y+a.height/2]]);o.setMatrix(s)}if(r.background){var l=this.drawLabelBg(e,t,o),u=this.itemType+"-label-bg";l.set("classname",u),o.toFront()}return o},drawLabelBg:function(e,t,n){var r=this.options.labelCfg,i=Object(ce.mix)({},r,e.labelCfg),o=this.getLabelBgStyleByPosition(n,e,i,t);return t.addShape("rect",{name:"text-bg-shape",attrs:o})},getLabelStyleByPosition:function(e,t,n){return{text:e.label}},getLabelBgStyleByPosition:function(e,t,n,r){return{}},getLabelStyle:function(e,t,n){var r=this.getLabelStyleByPosition(e,t,n),i=this.itemType+"Label",o=J[i]?J[i].style:null;return Object(w.__assign)(Object(w.__assign)(Object(w.__assign)({},o),r),t.style)},getShapeStyle:function(e){return e.style},update:function(e,t){this.updateShapeStyle(e,t),this.updateLabel(e,t)},updateShapeStyle:function(e,t){var n=t.getContainer(),r=t.getKeyShape(),i=Object(ce.mix)({},r.attr(),e.style),o=function(e){var t,o=i[e];if(Object(ce.isPlainObject)(o)){var a=n.find((function(t){return t.get("name")===e}));a&&a.attr(o)}else r.attr(((t={})[e]=o,t))};for(var a in i)o(a)},updateLabel:function(e,t){var n=t.getContainer(),r=this.getOptions({}).labelCfg,i=this.itemType+"-label",o=n.find((function(e){return e.get("className")===i})),a=this.itemType+"-label-bg",s=n.find((function(e){return e.get("classname")===a}));if(e.label||""===e.label)if(o){var l={};t.getModel&&(l=t.getModel().labelCfg);var u=Object(ce.deepMix)({},r,l,e.labelCfg),c=this.getLabelStyleByPosition(e,u,n),d=e.labelCfg?e.labelCfg.style:void 0,h=u.style&&u.style.background,f=Object(w.__assign)(Object(w.__assign)(Object(w.__assign)({},o.attr()),c),d),p=f.rotate;if(delete f.rotate,p){var g=[1,0,0,0,1,0,0,0,1];g=fe(g,[["t",-f.x,-f.y],["r",p],["t",f.x,f.y]]),o.resetMatrix(),o.attr(Object(w.__assign)(Object(w.__assign)({},f),{matrix:g}))}else o.resetMatrix(),o.attr(f);if(s)if(f.background){var v=this.getLabelBgStyleByPosition(o,e,u,n),m=Object(w.__assign)(Object(w.__assign)({},v),h);s.resetMatrix(),p&&s.rotateAtStart(p),s.attr(m)}else n.removeChild(s);else f.background&&((s=this.drawLabelBg(e,n,o)).set("classname",a),o.toFront())}else this.drawLabel(e,n).set("className",i)},afterUpdate:function(e,t){},setState:function(e,t,n){var r,i,o=n.get("keyShape");if(o){var a=n.getType(),s=Object(ce.isBoolean)(t)?e:e+":"+t,l=this.getStateStyle(s,n),u=n.getStateStyle(s);if(u||l){var c=Object(ce.mix)({},u||l),d=n.getContainer();if(t){var h=function(e){var t,n=c[e];if(Object(ce.isPlainObject)(n)&&!zo.includes(e)){var r=d.find((function(t){return t.get("name")===e}));r&&r.attr(n)}else o.attr(((t={})[e]=n,t))};for(var f in c)h(f)}else{var p=Object(ce.clone)(n.getCurrentStatesStyle()),g=n.getModel(),v=Object(ce.mix)({},g.style,Object(ce.clone)(n.getOriginStyle())),m=o.get("name"),y=o.attr(),b={},C=function(e){var t=c[e];if(Object(ce.isPlainObject)(t)&&!zo.includes(e)){var n=d.find((function(t){return t.get("name")===e}));if(n){var r=n.attr();Object(ce.each)(t,(function(e,t){(r[t]||0===r[t])&&delete r[t]})),b[e]=r}}else y[e]&&!{x:1,y:1,cx:1,cy:1}[e]&&delete y[e]};for(var A in c)C(A);for(var f in m?b[m]=y:Object(ce.mix)(b,y),p){var _=p[f];Object(ce.isPlainObject)(_)&&!zo.includes(f)||(m?(Object(ce.mix)(v[m],((i={})[f]=_,i)),delete v[f]):Object(ce.mix)(v,((r={})[f]=_,r)),delete p[f])}var I={};Object(ce.deepMix)(I,v,b,p);var x=function(e){var t,n,r=I[e];if(Object(ce.isPlainObject)(r)&&!zo.includes(e)){var i=d.find((function(t){return t.get("name")===e}));i&&i.attr(r)}else"combo"===a?m||o.attr(((t={})[e]=r,t)):o.attr(((n={})[e]=r,n))};for(var w in I)x(w)}}}},getStateStyle:function(e,t){var n=t.getModel(),r=t.getType(),i=this.getOptions(n),o=i.stateStyles,a=i.style,s=void 0===a?{}:a,l=n.stateStyles?n.stateStyles[e]:o&&o[e];return"combo"===r?Object(ce.clone)(l):Object(ce.mix)({},s,l)},getControlPoints:function(e){return e.controlPoints},getAnchorPoints:function(e){return this.getOptions(e).anchorPoints}},Bo={itemType:"node",shapeType:"single-node",labelPosition:"center",offset:J.nodeLabel.offset,getSize:function(e){var t=e.size||this.getOptions({}).size||J.defaultNode.size;return Object(ce.isArray)(t)&&1===t.length&&(t=[t[0],t[0]]),Object(ce.isArray)(t)||(t=[t,t]),t},getLabelStyleByPosition:function(e,t){var n=t.position||this.labelPosition;if("center"===n)return{x:0,y:0,text:e.label};var r=t.offset;Object(ce.isNil)(r)&&(r=this.offset);var i,o=this.getSize(e),a=o[0],s=o[1];switch(n){case"top":i={x:0,y:0-s/2-r,textBaseline:"bottom"};break;case"bottom":i={x:0,y:s/2+r,textBaseline:"top"};break;case"left":i={x:0-a/2-r,y:0,textAlign:"right"};break;default:i={x:a/2+r,y:0,textAlign:"left"}}return i.text=e.label,i},getLabelBgStyleByPosition:function(e,t,n,r){if(!e)return{};var i=e.getBBox(),o=n.style&&n.style.background;if(!o)return{};var a,s=T(o.padding),l=i.width+s[1]+s[3],u=i.height+s[0]+s[2],c=n.offset;return Object(ce.isNil)(c)&&(c=this.offset),a={x:i.minX-s[3],y:i.minY-s[0]},Object(w.__assign)(Object(w.__assign)(Object(w.__assign)({},a),o),{width:l,height:u})},drawShape:function(e,t){var n=this.shapeType,r=this.getShapeStyle(e);return t.addShape(n,{attrs:r,draggable:!0,name:"node-shape"})},updateLinkPoints:function(e,t){var n,r=this.getOptions(e).linkPoints,i=t.find((function(e){return"link-point-left"===e.get("className")})),o=t.find((function(e){return"link-point-right"===e.get("className")})),a=t.find((function(e){return"link-point-top"===e.get("className")})),s=t.find((function(e){return"link-point-bottom"===e.get("className")}));i&&(n=i.attr()),o&&!n&&(n=o.attr()),a&&!n&&(n=a.attr()),s&&!n&&(n=s.attr()),n||(n=r);var l=Object(ce.mix)({},n,e.linkPoints),u=l.fill,c=l.stroke,d=l.lineWidth,h=l.size/2;h||(h=l.r);var f=e.linkPoints?e.linkPoints:{left:void 0,right:void 0,top:void 0,bottom:void 0},p=f.left,g=f.right,v=f.top,m=f.bottom,y=this.getSize(e),b=y[0],C=y[1],A={r:h,fill:u,stroke:c,lineWidth:d};i?p||void 0===p?i.attr(Object(w.__assign)(Object(w.__assign)({},A),{x:-b/2,y:0})):i.remove():p&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},A),{x:-b/2,y:0}),className:"link-point-left",name:"link-point-left",isAnchorPoint:!0}),o?(g||void 0===g||o.remove(),o.attr(Object(w.__assign)(Object(w.__assign)({},A),{x:b/2,y:0}))):g&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},A),{x:b/2,y:0}),className:"link-point-right",name:"link-point-right",isAnchorPoint:!0}),a?(v||void 0===v||a.remove(),a.attr(Object(w.__assign)(Object(w.__assign)({},A),{x:0,y:-C/2}))):v&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},A),{x:0,y:-C/2}),className:"link-point-top",name:"link-point-top",isAnchorPoint:!0}),s?m||void 0===m?s.attr(Object(w.__assign)(Object(w.__assign)({},A),{x:0,y:C/2})):s.remove():m&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},A),{x:0,y:C/2}),className:"link-point-bottom",name:"link-point-bottom",isAnchorPoint:!0})},updateShape:function(e,t,n,r){t.get("keyShape").attr(Object(w.__assign)({},n)),this.updateLabel(e,t),r&&this.updateIcon(e,t)},updateIcon:function(e,t){var n=this,r=t.getContainer(),i=this.getOptions(e).icon,o=(e.icon?e.icon:{show:void 0}).show,a=r.find((function(e){return e.get("className")===n.type+"-icon"}));if(a)if(o||void 0===o){var s=Object(ce.mix)({},a.attr(),i),l=s.width,u=s.height;a.attr(Object(w.__assign)(Object(w.__assign)({},s),{x:-l/2,y:-u/2}))}else a.remove();else if(o){l=i.width,u=i.height,r.addShape("image",{attrs:Object(w.__assign)(Object(w.__assign)({},i),{x:-l/2,y:-u/2}),className:this.type+"-icon",name:this.type+"-icon"});var c=r.find((function(e){return"node-label"===e.get("className")}));c&&c.toFront()}}},Ho=Object(w.__assign)(Object(w.__assign)({},Fo),Bo);On.registerNode("single-node",Ho);var Uo={itemType:"edge",labelPosition:"center",refX:0,refY:0,labelAutoRotate:!1,options:{size:J.defaultEdge.size,style:{x:0,y:0,stroke:J.defaultEdge.style.stroke,lineAppendWidth:J.defaultEdge.style.lineAppendWidth},labelCfg:{style:{fill:J.edgeLabel.style.fill,fontSize:J.edgeLabel.style.fontSize}},stateStyles:Object(w.__assign)({},J.edgeStateStyles)},getPath:function(e){var t=[];return Object(ce.each)(e,(function(e,n){0===n?t.push(["M",e.x,e.y]):t.push(["L",e.x,e.y])})),t},getShapeStyle:function(e){var t=this.options.style,n={stroke:e.color},r=Object(ce.mix)({},t,n,e.style),i=e.size||J.defaultEdge.size,o=(e=this.getPathPoints(e)).startPoint,a=e.endPoint,s=this.getControlPoints(e),l=[o];s&&(l=l.concat(s)),l.push(a);var u=this.getPath(l);return Object(ce.mix)({},J.defaultEdge.style,{stroke:J.defaultEdge.color,lineWidth:i,path:u},r)},updateShapeStyle:function(e,t){var n=t.getContainer(),r={stroke:e.color},i=n.find((function(e){return"edge-shape"===e.get("className")}))||t.getKeyShape(),o=e.size,a=(e=this.getPathPoints(e)).startPoint,s=e.endPoint,l=this.getControlPoints(e),u=[a];l&&(u=u.concat(l)),u.push(s);var c=Object(ce.mix)({},r,i.attr(),e.style),d=e.sourceNode,h=e.targetNode,f={radius:c.radius};l||(f={source:d,target:h,offset:c.offset,radius:c.radius});var p=this.getPath(u,f),g=Object(ce.mix)(r,i.attr(),{lineWidth:o,path:p},e.style);i&&i.attr(g)},getLabelStyleByPosition:function(e,t,n){var r,i=t.position||this.labelPosition,o={},a=n&&n.find((function(e){return"edge-shape"===e.get("className")}));r="start"===i?0:"end"===i?1:.5;var s=t.refX||this.refX,l=t.refY||this.refY;if(e.startPoint.x===e.endPoint.x&&e.startPoint.y===e.endPoint.y)return o.x=e.startPoint.x+s,o.y=e.startPoint.y+l,o.text=e.label,o;var u=Object(ce.isNil)(t.autoRotate)?this.labelAutoRotate:t.autoRotate,c=dn(a,r,s,l,u);return o.x=c.x,o.y=c.y,o.rotate=c.rotate,o.textAlign=this._getTextAlign(i,c.angle),o.text=e.label,o},getLabelBgStyleByPosition:function(e,t,n,r){if(!e)return{};var i=e.getBBox(),o=n.style&&n.style.background;if(!o)return{};var a,s=o.padding,l=i.width+s[1]+s[3],u=i.height+s[0]+s[2],c=n.position||this.labelPosition,d=Object(w.__assign)(Object(w.__assign)({},o),{width:l,height:u,x:i.minX-s[2],y:i.minY-s[0],rotate:0}),h=Object(ce.isNil)(n.autoRotate)?this.labelAutoRotate:n.autoRotate,f=r&&r.find((function(e){return"edge-shape"===e.get("className")}));a="start"===c?0:"end"===c?1:.5;var p=n.refX||this.refX,g=n.refY||this.refY;if(t.startPoint.x===t.endPoint.x&&t.startPoint.y===t.endPoint.y)return d.x=t.startPoint.x+p,d.y=t.startPoint.y+g,d;var v=dn(f,a,p-l/2,g+u/2,h),m=v.angle;return m>.5*Math.PI&&m<1.5*Math.PI&&(v=dn(f,a,p+l/2,g+u/2,h)),h&&(d.x=v.x,d.y=v.y),d.rotate=v.rotate,d},_getTextAlign:function(e,t){var n="center";return t?(t%=2*Math.PI,"center"!==e&&(n=t>=0&&t<=Math.PI/2||t>=1.5*Math.PI&&t<2*Math.PI?e:function(e){var t=e;return"start"===e?t="end":"end"===e&&(t="start"),t}(e)),n):e},getControlPoints:function(e){return e.controlPoints},getPathPoints:function(e){return e},drawShape:function(e,t){var n=this.getShapeStyle(e);return t.addShape("path",{className:"edge-shape",name:"edge-shape",attrs:n})},drawLabel:function(e,t){var n=this.options.labelCfg,r=Object(ce.deepMix)({fontFamily:"undefined"!=typeof window&&window.getComputedStyle(document.body,null).getPropertyValue("font-family")||"Arial, sans-serif"},n,e.labelCfg),i=this.getLabelStyle(e,r,t),o=i.rotate;delete i.rotate;var a=t.addShape("text",{attrs:i,name:"text-shape"});if(o&&a.rotateAtStart(o),i.background){var s=this.drawLabelBg(e,t,a),l=this.itemType+"-label-bg";s.set("classname",l),a.toFront()}return a},drawLabelBg:function(e,t,n){var r=this.options.labelCfg,i=Object(ce.deepMix)({},r,e.labelCfg),o=this.getLabelStyle(e,i,t).rotate,a=this.getLabelBgStyleByPosition(n,e,i,t);delete a.rotate;var s=t.addShape("rect",{name:"text-bg-shape",attrs:a});return o&&s.rotateAtStart(o),s}},Vo=Object(w.__assign)(Object(w.__assign)({},Fo),Uo);On.registerEdge("single-edge",Vo),On.registerEdge("line",{getControlPoints:function(){}},"single-edge"),On.registerEdge("spline",{getPath:function(e){return mt(e)}},"single-edge"),On.registerEdge("arc",{curveOffset:20,clockwise:1,getControlPoints:function(e){var t,n,r=e.startPoint,i=e.endPoint,o=(r.x+i.x)/2,a=(r.y+i.y)/2;if(void 0!==e.controlPoints){if(n=e.controlPoints[0],t=Ue(r,n,i),r.x<=i.x&&r.y>i.y?this.clockwise=t.x>n.x?0:1:r.x<=i.x&&r.y<i.y?this.clockwise=t.x>n.x?1:0:r.x>i.x&&r.y<=i.y?this.clockwise=t.y<n.y?0:1:this.clockwise=t.y<n.y?1:0,(n.x-r.x)/(n.y-r.y)==(i.x-r.x)/(i.y-r.y))return[]}else{void 0===e.curveOffset&&(e.curveOffset=this.curveOffset),y()(e.curveOffset)&&(e.curveOffset=e.curveOffset[0]),e.curveOffset<0?this.clockwise=0:this.clockwise=1;var s={x:i.x-r.x,y:i.y-r.y},l=Math.atan2(s.y,s.x);n={x:e.curveOffset*Math.cos(-Math.PI/2+l)+o,y:e.curveOffset*Math.sin(-Math.PI/2+l)+a},t=Ue(r,n,i)}var u=Ve(r,t);return[{x:u,y:u}]},getPath:function(e){var t=[];return t.push(["M",e[0].x,e[0].y]),2===e.length?t.push(["L",e[1].x,e[1].y]):t.push(["A",e[1].x,e[1].y,0,0,this.clockwise,e[2].x,e[2].y]),t}},"single-edge"),On.registerEdge("quadratic",{curvePosition:.5,curveOffset:-20,getControlPoints:function(e){var t=e.controlPoints;if(!t||!t.length){var n=e.startPoint,r=e.endPoint;void 0===e.curveOffset&&(e.curveOffset=this.curveOffset),void 0===e.curvePosition&&(e.curvePosition=this.curvePosition),y()(this.curveOffset)&&(e.curveOffset=e.curveOffset[0]),y()(this.curvePosition)&&(e.curvePosition=e.curveOffset[0]),t=[yt(n,r,e.curvePosition,e.curveOffset)]}return t},getPath:function(e){var t=[];return t.push(["M",e[0].x,e[0].y]),t.push(["Q",e[1].x,e[1].y,e[2].x,e[2].y]),t}},"single-edge"),On.registerEdge("cubic",{curvePosition:[.5,.5],curveOffset:[-20,20],getControlPoints:function(e){var t=e.controlPoints;if(void 0===e.curveOffset&&(e.curveOffset=this.curveOffset),void 0===e.curvePosition&&(e.curvePosition=this.curvePosition),_()(e.curveOffset)&&(e.curveOffset=[e.curveOffset,-e.curveOffset]),_()(e.curvePosition)&&(e.curvePosition=[e.curvePosition,1-e.curvePosition]),!t||!t.length||t.length<2){var n=e.startPoint,r=e.endPoint;t=[yt(n,r,e.curvePosition[0],e.curveOffset[0]),yt(n,r,e.curvePosition[1],e.curveOffset[1])]}return t},getPath:function(e){var t=[];return t.push(["M",e[0].x,e[0].y]),t.push(["C",e[1].x,e[1].y,e[2].x,e[2].y,e[3].x,e[3].y]),t}},"single-edge"),On.registerEdge("cubic-vertical",{curvePosition:[.5,.5],getControlPoints:function(e){var t=e.startPoint,n=e.endPoint;return void 0!==e.curvePosition&&(this.curvePosition=e.curvePosition),_()(this.curvePosition)&&(this.curvePosition=[this.curvePosition,1-this.curvePosition]),[{x:t.x,y:(n.y-t.y)*this.curvePosition[0]+t.y},{x:n.x,y:(n.y-t.y)*this.curvePosition[1]+t.y}]}},"cubic"),On.registerEdge("cubic-horizontal",{curvePosition:[.5,.5],getControlPoints:function(e){var t=e.startPoint,n=e.endPoint;return void 0!==e.curvePosition&&(this.curvePosition=e.curvePosition),_()(this.curvePosition)&&(this.curvePosition=[this.curvePosition,1-this.curvePosition]),[{x:(n.x-t.x)*this.curvePosition[0]+t.x,y:t.y},{x:(n.x-t.x)*this.curvePosition[1]+t.x,y:n.y}]}},"cubic"),On.registerEdge("loop",{getPathPoints:function(e){return cn(e)},getControlPoints:function(e){return e.controlPoints},afterDraw:function(e){e.controlPoints=void 0},afterUpdate:function(e){e.controlPoints=void 0}},"cubic");var Yo={itemType:"combo",shapeType:"single-combo",labelPosition:"top",refX:J.comboLabel.refX,refY:J.comboLabel.refY,options:{style:{stroke:J.defaultCombo.style.stroke,fill:J.defaultCombo.style.fill,lineWidth:J.defaultCombo.style.lineWidth},labelCfg:{style:{fill:J.comboLabel.style.fill,fontSize:J.comboLabel.style.fontSize}},stateStyles:Object(w.__assign)({},J.comboStateStyles)},getSize:function(e){var t=Object(ce.clone)(e.size||this.options.size||J.defaultCombo.size);return Object(ce.isArray)(t)&&1===t.length&&(t=[t[0],t[0]]),Object(ce.isArray)(t)||(t=[t,t]),t},getLabelStyleByPosition:function(e,t){var n=t.position||this.labelPosition,r=e.style,i=e.padding||this.options.padding;Object(ce.isArray)(i)&&(i=i[0]);var o=t.refX,a=t.refY;Object(ce.isNil)(o)&&(o=this.refX),Object(ce.isNil)(a)&&(a=this.refY);var s,l=this.getSize(e),u=(Math.max(r.r,l[0]/2)||l[0]/2)+i;switch(n){case"top":s={x:0,y:-u-a,textBaseline:"bottom",textAlign:"center"};break;case"bottom":s={x:0,y:u+a,textBaseline:"bottom",textAlign:"center"};break;case"left":s={x:-u+o,y:0,textAlign:"left"};break;case"center":s={x:0,y:0,text:e.label,textAlign:"center"};break;default:s={x:u+o,y:0,textAlign:"right"}}return s.text=e.label,s},drawShape:function(e,t){var n=this.shapeType,r=this.getShapeStyle(e);return t.addShape(n,{attrs:r,draggable:!0,name:"combo-shape"})},updateShape:function(e,t,n){var r=t.get("keyShape");(void 0===e.animate?this.options.animate:e.animate)&&r.animate?r.animate(n,{duration:200,easing:"easeLinear"}):r.attr(Object(w.__assign)({},n)),this.updateLabel(e,t)}},Wo=Object(w.__assign)(Object(w.__assign)({},Fo),Yo);On.registerCombo("single-combo",Wo),On.registerCombo("circle",{options:{size:[J.defaultCombo.size[0],J.defaultCombo.size[0]],padding:J.defaultCombo.padding[0],animate:!0,style:{stroke:J.defaultCombo.style.stroke,fill:J.defaultCombo.style.fill,lineWidth:J.defaultCombo.style.lineWidth},labelCfg:{style:{fill:J.comboLabel.style.fill,fontSize:J.comboLabel.style.fontSize},refX:0,refY:0},stateStyles:Object(w.__assign)({},J.comboStateStyles)},shapeType:"circle",labelPosition:"top",drawShape:function(e,t){var n=this.getShapeStyle(e);return delete n.height,delete n.width,t.addShape("circle",{attrs:n,className:"circle-combo",name:"circle-combo",draggable:!0})},getShapeStyle:function(e){var t=this.options.style,n=e.padding||this.options.padding;Object(ce.isArray)(n)&&(n=n[0]);var r,i={stroke:e.color},o=Object(ce.mix)({},t,i,e.style),a=this.getSize(e);r=!Object(ce.isNumber)(o.r)||isNaN(o.r)?a[0]/2||J.defaultCombo.style.r:Math.max(o.r,a[0]/2)||a[0]/2,o.r=r+n;var s=Object(w.__assign)({x:0,y:0},o);return e.style?e.style.r=r:e.style={r:r},s},update:function(e,t){var n=this.getSize(e),r=e.padding||this.options.padding;Object(ce.isArray)(r)&&(r=r[0]);var i=Object(ce.clone)(e.style),o=Math.max(i.r,n[0]/2)||n[0]/2;i.r=o+r;var a=t.get("sizeCache");a&&(a.r=i.r);var s={stroke:e.color},l=t.get("keyShape"),u=Object(ce.mix)({},l.attr(),s,i);e.style?e.style.r=o:e.style={r:o},this.updateShape(e,t,u,!0)}},"single-combo"),On.registerCombo("rect",{options:{size:[40,5],padding:[25,20,15,20],animate:!0,style:{radius:0,stroke:J.defaultCombo.style.stroke,fill:J.defaultCombo.style.fill,lineWidth:J.defaultCombo.style.lineWidth},labelCfg:{style:{fill:J.comboLabel.style.fill,fontSize:J.comboLabel.style.fontSize}},anchorPoints:[[0,.5],[1,.5]],stateStyles:Object(w.__assign)({},J.comboStateStyles)},shapeType:"rect",labelPosition:"top",drawShape:function(e,t){var n=this.getShapeStyle(e);return t.addShape("rect",{attrs:n,className:"rect-combo",name:"rect-combo",draggable:!0})},getLabelStyleByPosition:function(e,t){var n=t.position||this.labelPosition,r=e.style,i=e.padding||this.options.padding;Object(ce.isNumber)(i)&&(i=[i,i,i,i]);var o=t.refX,a=t.refY;Object(ce.isNil)(o)&&(o=this.refX),Object(ce.isNil)(a)&&(a=this.refY);var s,l=r.width/2+i[3],u=r.height/2+i[0];switch(n){case"top":s={x:0-l+o,y:0-u+a,textBaseline:"top",textAlign:"left"};break;case"bottom":s={x:0,y:u+a,textBaseline:"bottom",textAlign:"center"};break;case"left":s={x:0-l+a,y:0,textAlign:"left"};break;case"center":s={x:0,y:0,text:e.label,textAlign:"center"};break;default:s={x:l+o,y:0,textAlign:"right"}}return s.text=e.label,s},getShapeStyle:function(e){var t=this.options.style,n=e.padding||this.options.padding;Object(ce.isNumber)(n)&&(n=[n,n,n,n]);var r,i,o={stroke:e.color},a=Object(ce.mix)({},t,o,e.style),s=this.getSize(e);r=!Object(ce.isNumber)(a.width)||isNaN(a.width)?s[0]||J.defaultCombo.style.width:Math.max(a.width,s[0])||s[0],i=!Object(ce.isNumber)(a.height)||isNaN(a.height)?s[1]||J.defaultCombo.style.height:Math.max(a.height,s[1])||s[1];var l=-r/2-n[3],u=-i/2-n[0];a.width=r+n[1]+n[3],a.height=i+n[0]+n[2];var c=Object(w.__assign)({x:l,y:u},a);return e.style?(e.style.width=r,e.style.height=i):e.style={width:r,height:i},c},update:function(e,t){var n=this.getSize(e),r=e.padding||this.options.padding;Object(ce.isNumber)(r)&&(r=[r,r,r,r]);var i=Object(ce.clone)(e.style),o=Math.max(i.width,n[0])||n[0],a=Math.max(i.height,n[1])||n[1];i.width=o+r[1]+r[3],i.height=a+r[0]+r[2];var s=t.get("sizeCache");s&&(s.width=i.width,s.height=i.height),i.x=-o/2-r[3],i.y=-a/2-r[0];var l={stroke:e.color},u=t.get("keyShape"),c=Object(ce.mix)({},u.attr(),l,i);e.style?(e.style.width=o,e.style.height=a):e.style={width:o,height:a},this.updateShape(e,t,c,!1)},updateShape:function(e,t,n){var r=t.get("keyShape");(void 0===e.animate?this.options.animate:e.animate)&&r.animate?r.animate(n,{duration:200,easing:"easeLinear"}):r.attr(Object(w.__assign)({},n)),this.updateLabel(e,t)}},"single-combo"),On.registerNode("circle",{options:{size:J.defaultNode.size,style:{x:0,y:0,stroke:J.defaultNode.style.stroke,fill:J.defaultNode.style.fill,lineWidth:J.defaultNode.style.lineWidth},labelCfg:{style:{fill:J.nodeLabel.style.fill,fontSize:J.nodeLabel.style.fontSize}},linkPoints:{top:!1,right:!1,bottom:!1,left:!1,size:J.defaultNode.linkPoints.size,lineWidth:J.defaultNode.linkPoints.lineWidth,fill:J.defaultNode.linkPoints.fill,stroke:J.defaultNode.linkPoints.stroke},icon:{show:!1,img:"https://gw.alipayobjects.com/zos/bmw-prod/5d015065-8505-4e7a-baec-976f81e3c41d.svg",width:20,height:20},stateStyles:Object(w.__assign)({},J.nodeStateStyles)},shapeType:"circle",labelPosition:"center",drawShape:function(e,t){var n=this.getOptions(e).icon,r=void 0===n?{}:n,i=this.getShapeStyle(e),o=Ie()({},r,e.icon),a=t.addShape("circle",{attrs:i,className:this.type+"-keyShape",draggable:!0}),s=o.width,l=o.height;return o.show&&t.addShape("image",{attrs:Object(w.__assign)({x:-s/2,y:-l/2},o),className:this.type+"-icon",name:this.type+"-icon",draggable:!0}),this.drawLinkPoints(e,t),a},drawLinkPoints:function(e,t){var n=this.getOptions(e).linkPoints,r=void 0===n?{}:n,i=r.top,o=r.left,a=r.right,s=r.bottom,l=r.size,u=r.r,c=Object(w.__rest)(r,["top","left","right","bottom","size","r"]),d=this.getSize(e)[0]/2;o&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},c),{x:-d,y:0,r:l/2||u||5}),className:"link-point-left",name:"link-point-left",isAnchorPoint:!0}),a&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},c),{x:d,y:0,r:l/2||u||5}),className:"link-point-right",name:"link-point-right",isAnchorPoint:!0}),i&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},c),{x:0,y:-d,r:l/2||u||5}),className:"link-point-top",name:"link-point-top",isAnchorPoint:!0}),s&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},c),{x:0,y:d,r:l/2||u||5}),className:"link-point-bottom",name:"link-point-bottom",isAnchorPoint:!0})},getShapeStyle:function(e){var t=this.getOptions(e).style,n={stroke:e.color},r=Ie()({},t,n),i=this.getSize(e)[0]/2;return Object(w.__assign)({x:0,y:0,r:i},r)},update:function(e,t){var n=t.getContainer(),r=this.getSize(e),i={stroke:e.color,r:r[0]/2},o=t.get("keyShape"),a=Ie()({},o.attr(),i,e.style);this.updateShape(e,t,a,!0),this.updateLinkPoints(e,n)}},"single-node"),On.registerNode("rect",{options:{size:[100,30],style:{radius:0,stroke:J.defaultNode.style.stroke,fill:J.defaultNode.style.fill,lineWidth:J.defaultNode.style.lineWidth},labelCfg:{style:{fill:J.nodeLabel.style.fill,fontSize:J.nodeLabel.style.fontSize}},linkPoints:{top:!1,right:!1,bottom:!1,left:!1,size:J.defaultNode.linkPoints.size,lineWidth:J.defaultNode.linkPoints.lineWidth,fill:J.defaultNode.linkPoints.fill,stroke:J.defaultNode.linkPoints.stroke},icon:{show:!1,img:"https://gw.alipayobjects.com/zos/bmw-prod/5d015065-8505-4e7a-baec-976f81e3c41d.svg",width:20,height:20},anchorPoints:[[0,.5],[1,.5]],stateStyles:Object(w.__assign)({},J.nodeStateStyles)},shapeType:"rect",labelPosition:"center",drawShape:function(e,t){var n=this.getShapeStyle(e),r=t.addShape("rect",{attrs:n,className:this.type+"-keyShape",name:this.type+"-keyShape",draggable:!0});return this.drawLinkPoints(e,t),r},drawLinkPoints:function(e,t){var n=this.getOptions(e).linkPoints,r=void 0===n?{}:n,i=r.top,o=r.left,a=r.right,s=r.bottom,l=r.size,u=r.r,c=Object(w.__rest)(r,["top","left","right","bottom","size","r"]),d=this.getSize(e),h=d[0],f=d[1];o&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},c),{x:-h/2,y:0,r:l/2||u||5}),className:"link-point-left",name:"link-point-left",isAnchorPoint:!0}),a&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},c),{x:h/2,y:0,r:l/2||u||5}),className:"link-point-right",name:"link-point-right",isAnchorPoint:!0}),i&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},c),{x:0,y:-f/2,r:l/2||u||5}),className:"link-point-top",name:"link-point-top",isAnchorPoint:!0}),s&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},c),{x:0,y:f/2,r:l/2||u||5}),className:"link-point-bottom",name:"link-point-bottom",isAnchorPoint:!0})},getShapeStyle:function(e){var t=this.getOptions(e).style,n={stroke:e.color},r=Object(ce.mix)({},t,n),i=this.getSize(e),o=r.width||i[0],a=r.height||i[1];return Object(w.__assign)({x:-o/2,y:-a/2,width:o,height:a},r)},update:function(e,t){var n=t.getContainer(),r=this.getOptions({}).style,i=this.getSize(e),o=t.get("keyShape");e.size||(i[0]=o.attr("width")||r.width,i[1]=o.attr("height")||r.height);var a={stroke:e.color,x:-i[0]/2,y:-i[1]/2,width:i[0],height:i[1]},s=Object(ce.mix)({},r,o.attr(),a);s=Object(ce.mix)(s,e.style),this.updateShape(e,t,s,!1),this.updateLinkPoints(e,n)}},"single-node"),On.registerNode("ellipse",{options:{size:[80,40],style:{x:0,y:0,stroke:J.defaultNode.style.stroke,fill:J.defaultNode.style.fill,lineWidth:J.defaultNode.style.lineWidth},labelCfg:{style:{fill:J.nodeLabel.style.fill,fontSize:J.nodeLabel.style.fontSize}},linkPoints:{top:!1,right:!1,bottom:!1,left:!1,size:J.defaultNode.linkPoints.size,lineWidth:J.defaultNode.linkPoints.lineWidth,fill:J.defaultNode.linkPoints.fill,stroke:J.defaultNode.linkPoints.stroke},icon:{show:!1,img:"https://gw.alipayobjects.com/zos/bmw-prod/5d015065-8505-4e7a-baec-976f81e3c41d.svg",width:20,height:20},stateStyles:Object(w.__assign)({},J.nodeStateStyles)},shapeType:"ellipse",labelPosition:"center",drawShape:function(e,t){var n=this.getOptions(e).icon,r=void 0===n?{}:n,i=this.getShapeStyle(e),o=t.addShape("ellipse",{attrs:i,className:"ellipse-keyShape",name:"ellipse-keyShape",draggable:!0}),a=r.width,s=r.height;return r.show&&t.addShape("image",{attrs:Object(w.__assign)({x:-a/2,y:-s/2},r),className:this.type+"-icon",name:this.type+"-icon",draggable:!0}),this.drawLinkPoints(e,t),o},drawLinkPoints:function(e,t){var n=this.getOptions(e).linkPoints,r=void 0===n?{}:n,i=r.top,o=r.left,a=r.right,s=r.bottom,l=r.size,u=r.r,c=Object(w.__rest)(r,["top","left","right","bottom","size","r"]),d=this.getSize(e),h=d[0]/2,f=d[1]/2;o&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},c),{x:-h,y:0,r:l/2||u||5}),className:"link-point-left",name:"link-point-left",isAnchorPoint:!0}),a&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},c),{x:h,y:0,r:l/2||u||5}),className:"link-point-right",name:"link-point-right",isAnchorPoint:!0}),i&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},c),{x:0,y:-f,r:l/2||u||5}),className:"link-point-top",name:"link-point-top",isAnchorPoint:!0}),s&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},c),{x:0,y:f,r:l/2||u||5}),className:"link-point-bottom",name:"link-point-bottom",isAnchorPoint:!0})},getShapeStyle:function(e){var t=this.getOptions(e).style,n={stroke:e.color},r=Object(ce.mix)({},t,n),i=this.getSize(e),o=i[0]/2,a=i[1]/2;return Object(w.__assign)({x:0,y:0,rx:o,ry:a},r)},update:function(e,t){var n=t.getContainer(),r=this.getOptions({}).style,i=this.getSize(e),o={stroke:e.color,rx:i[0]/2,ry:i[1]/2},a=t.get("keyShape"),s=Object(ce.mix)({},r,a.attr(),o);s=Object(ce.mix)(s,e.style),this.updateShape(e,t,s,!0),this.updateLinkPoints(e,n)}},"single-node"),On.registerNode("diamond",{options:{size:[80,80],style:{stroke:J.defaultNode.style.stroke,fill:J.defaultNode.style.fill,lineWidth:J.defaultNode.style.lineWidth},labelCfg:{style:{fill:J.nodeLabel.style.fill,fontSize:J.nodeLabel.style.fontSize}},linkPoints:{top:!1,right:!1,bottom:!1,left:!1,size:J.defaultNode.linkPoints.size,lineWidth:J.defaultNode.linkPoints.lineWidth,fill:J.defaultNode.linkPoints.fill,stroke:J.defaultNode.linkPoints.stroke},icon:{show:!1,img:"https://gw.alipayobjects.com/zos/bmw-prod/5d015065-8505-4e7a-baec-976f81e3c41d.svg",width:20,height:20},stateStyles:Object(w.__assign)({},J.nodeStateStyles)},shapeType:"diamond",labelPosition:"center",drawShape:function(e,t){var n=this.getOptions(e).icon,r=void 0===n?{}:n,i=this.getShapeStyle(e),o=t.addShape("path",{attrs:i,className:this.type+"-keyShape",name:this.type+"-keyShape",draggable:!0}),a=r.width,s=r.height;return r.show&&t.addShape("image",{attrs:Object(w.__assign)({x:-a/2,y:-s/2},r),className:this.type+"-icon",name:this.type+"-icon",draggable:!0}),this.drawLinkPoints(e,t),o},drawLinkPoints:function(e,t){var n=this.getOptions(e).linkPoints,r=void 0===n?{}:n,i=r.top,o=r.left,a=r.right,s=r.bottom,l=r.size,u=r.r,c=Object(w.__rest)(r,["top","left","right","bottom","size","r"]),d=this.getSize(e),h=d[0],f=d[1];o&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},c),{x:-h/2,y:0,r:l/2||u||5}),className:"link-point-left",name:"link-point-left",isAnchorPoint:!0}),a&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},c),{x:h/2,y:0,r:l/2||u||5}),className:"link-point-right",name:"link-point-right",isAnchorPoint:!0}),i&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},c),{x:0,y:-f/2,r:l/2||u||5}),className:"link-point-top",name:"link-point-top",isAnchorPoint:!0}),s&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},c),{x:0,y:f/2,r:l/2||u||5}),className:"link-point-bottom",name:"link-point-bottom",isAnchorPoint:!0})},getPath:function(e){var t=this.getSize(e),n=t[0],r=t[1];return[["M",0,-r/2],["L",n/2,0],["L",0,r/2],["L",-n/2,0],["Z"]]},getShapeStyle:function(e){var t=this.getOptions(e).style,n={stroke:e.color},r=Object(ce.mix)({},t,n),i=this.getPath(e);return Object(w.__assign)({path:i},r)},update:function(e,t){var n=t.getContainer(),r=this.getOptions({}).style,i=this.getPath(e),o={stroke:e.color,path:i},a=t.get("keyShape"),s=Object(ce.mix)({},r,a.attr(),o);s=Object(ce.mix)(s,e.style),this.updateShape(e,t,s,!0),this.updateLinkPoints(e,n)}},"single-node"),On.registerNode("triangle",{options:{size:40,direction:"up",style:{stroke:J.defaultNode.style.stroke,fill:J.defaultNode.style.fill,lineWidth:J.defaultNode.style.lineWidth},labelCfg:{style:{fill:J.nodeLabel.style.fill,fontSize:J.nodeLabel.style.fontSize},offset:15},linkPoints:{top:!1,right:!1,bottom:!1,left:!1,size:J.defaultNode.linkPoints.size,lineWidth:J.defaultNode.linkPoints.lineWidth,fill:J.defaultNode.linkPoints.fill,stroke:J.defaultNode.linkPoints.stroke},icon:{show:!1,img:"https://gw.alipayobjects.com/zos/bmw-prod/5d015065-8505-4e7a-baec-976f81e3c41d.svg",width:20,height:20,offset:6},stateStyles:Object(w.__assign)({},J.nodeStateStyles)},shapeType:"triangle",labelPosition:"bottom",drawShape:function(e,t){var n=this.getOptions(e),r=n.icon,i=void 0===r?{}:r,o=n.direction,a=this.getShapeStyle(e),s=e.direction||o,l=t.addShape("path",{attrs:a,className:this.type+"-keyShape",name:this.type+"-keyShape",draggable:!0}),u=i.width,c=i.height,d=i.show,h=i.offset;if(d){var f=-u/2,p=-c/2;"up"!==s&&"down"!==s||(p+=h),"left"!==s&&"right"!==s||(f+=h),t.addShape("image",{attrs:Object(w.__assign)({x:f,y:p},i),className:this.type+"-icon",name:this.type+"-icon",draggable:!0})}return this.drawLinkPoints(e,t),l},drawLinkPoints:function(e,t){var n=this.getOptions(e),r=n.linkPoints,i=void 0===r?{}:r,o=n.direction,a=e.direction||o,s=i.top,l=i.left,u=i.right,c=i.bottom,d=i.size,h=i.r,f=Object(w.__rest)(i,["top","left","right","bottom","size","r"]),p=this.getSize(e)[0];if(l){var g=null,v=p*Math.sin(1/3*Math.PI),m=p*Math.sin(1/3*Math.PI);"up"===a?g=[-m,v]:"down"===a?g=[-m,-v]:"left"===a&&(g=[-m,m-v]),g&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},f),{x:g[0],y:g[1],r:d/2||h||5}),className:"link-point-left",name:"link-point-left"})}if(u){var y=null;v=p*Math.sin(1/3*Math.PI),m=p*Math.sin(1/3*Math.PI),"up"===a?y=[m,v]:"down"===a?y=[m,-v]:"right"===a&&(y=[m,m-v]),y&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},f),{x:y[0],y:y[1],r:d/2||h||5}),className:"link-point-right",name:"link-point-right"})}if(s){var b=null;v=p*Math.sin(1/3*Math.PI),m=p*Math.sin(1/3*Math.PI),"up"===a?b=[m-v,-v]:"left"===a?b=[m,-v]:"right"===a&&(b=[-m,-v]),b&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},f),{x:b[0],y:b[1],r:d/2||h||5}),className:"link-point-top",name:"link-point-top"})}if(c){var C=null;v=p*Math.sin(1/3*Math.PI),m=p*Math.sin(1/3*Math.PI),"down"===a?C=[-m+v,v]:"left"===a?C=[m,v]:"right"===a&&(C=[-m,v]),C&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},f),{x:C[0],y:C[1],r:d/2||h||5}),className:"link-point-bottom",name:"link-point-bottom"})}},getPath:function(e){var t=this.getOptions(e).direction,n=e.direction||t,r=this.getSize(e)[0],i=r*Math.sin(1/3*Math.PI),o=r*Math.sin(1/3*Math.PI),a=[["M",-o,i],["L",0,-i],["L",o,i],["Z"]];return"down"===n?a=[["M",-o,-i],["L",o,-i],["L",0,i],["Z"]]:"left"===n?a=[["M",-o,o-i],["L",o,-o],["L",o,o],["Z"]]:"right"===n&&(a=[["M",o,o-i],["L",-o,o],["L",-o,-o],["Z"]]),a},getShapeStyle:function(e){var t=this.getOptions(e).style,n={stroke:e.color},r=Object(ce.mix)({},t,n),i=this.getPath(e);return Object(w.__assign)({path:i},r)},update:function(e,t){var n=t.getContainer(),r=this.getOptions({}).style,i=this.getPath(e),o={stroke:e.color,path:i},a=t.get("keyShape"),s=Object(ce.mix)({},r,a.attr(),o);s=Object(ce.mix)(s,e.style),this.updateShape(e,t,s,!0),this.updateLinkPoints(e,n)},updateLinkPoints:function(e,t){var n=this.getOptions({}),r=n.linkPoints,i=n.direction,o=e.direction||i,a=t.find((function(e){return"link-point-left"===e.get("className")})),s=t.find((function(e){return"link-point-right"===e.get("className")})),l=t.find((function(e){return"link-point-top"===e.get("className")})),u=t.find((function(e){return"link-point-bottom"===e.get("className")})),c=r,d=a||s||l||u;d&&(c=d.attr());var h=Object(ce.mix)({},c,e.linkPoints),f=h.fill,p=h.stroke,g=h.lineWidth,v=h.size/2;v||(v=h.r);var m=e.linkPoints?e.linkPoints:{left:void 0,right:void 0,top:void 0,bottom:void 0},y=m.left,b=m.right,C=m.top,A=m.bottom,_=this.getSize(e)[0],I={r:v,fill:f,stroke:p,lineWidth:g},x=null,M=_*Math.sin(1/3*Math.PI),S=_*Math.sin(1/3*Math.PI);"up"===o?x=[-S,M]:"down"===o?x=[-S,-M]:"left"===o&&(x=[-S,S-M]),x&&(a?y||void 0===y?a.attr(Object(w.__assign)(Object(w.__assign)({},I),{x:x[0],y:x[1]})):a.remove():y&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},I),{x:x[0],y:x[1]}),className:"link-point-left",name:"link-point-left",isAnchorPoint:!0}));var T=null;"up"===o?T=[S,M]:"down"===o?T=[S,-M]:"right"===o&&(T=[S,S-M]),T&&(s?b||void 0===b?s.attr(Object(w.__assign)(Object(w.__assign)({},I),{x:T[0],y:T[1]})):s.remove():b&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},I),{x:T[0],y:T[1]}),className:"link-point-right",name:"link-point-right",isAnchorPoint:!0}));var E=null;"up"===o?E=[S-M,-M]:"left"===o?E=[S,-M]:"right"===o&&(E=[-S,-M]),E&&(l?C||void 0===C?l.attr(Object(w.__assign)(Object(w.__assign)({},I),{x:E[0],y:E[1]})):l.remove():C&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},I),{x:E[0],y:E[1]}),className:"link-point-top",name:"link-point-top",isAnchorPoint:!0}));var O=null;"down"===o?O=[-S+M,M]:"left"===o?O=[S,M]:"right"===o&&(O=[-S,M]),O&&(u?A||void 0===A?u.attr(Object(w.__assign)(Object(w.__assign)({},I),{x:O[0],y:O[1]})):u.remove():A&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},I),{x:O[0],y:O[1]}),className:"link-point-bottom",name:"link-point-bottom",isAnchorPoint:!0}))}},"single-node"),On.registerNode("modelRect",{options:{size:[185,70],style:{radius:5,stroke:"#69c0ff",fill:"#ffffff",lineWidth:J.defaultNode.style.lineWidth,fillOpacity:1},labelCfg:{style:{fill:"#595959",fontSize:14},offset:30},descriptionCfg:{style:{fontSize:12,fill:"#bfbfbf"},paddingTop:0},preRect:{show:!0,width:4,fill:"#40a9ff",radius:2},linkPoints:{top:!1,right:!1,bottom:!1,left:!1,size:10,lineWidth:1,fill:"#72CC4A",stroke:"#72CC4A"},logoIcon:{show:!0,x:0,y:0,img:"https://gw.alipayobjects.com/zos/basement_prod/4f81893c-1806-4de4-aff3-9a6b266bc8a2.svg",width:16,height:16,offset:0},stateIcon:{show:!0,x:0,y:0,img:"https://gw.alipayobjects.com/zos/basement_prod/300a2523-67e0-4cbf-9d4a-67c077b40395.svg",width:16,height:16,offset:-5},anchorPoints:[[0,.5],[1,.5]]},shapeType:"modelRect",drawShape:function(e,t){var n=this.getOptions(e).preRect,r=void 0===n?{}:n,i=this.getShapeStyle(e),o=this.getSize(e),a=o[0],s=o[1],l=t.addShape("rect",{attrs:i,className:this.type+"-keyShape",name:this.type+"-keyShape",draggable:!0}),u=r.show,c=Object(w.__rest)(r,["show"]);return u&&t.addShape("rect",{attrs:Object(w.__assign)({x:-a/2,y:-s/2,height:s},c),className:"pre-rect",name:"pre-rect",draggable:!0}),this.drawLogoIcon(e,t),this.drawStateIcon(e,t),this.drawLinkPoints(e,t),l},drawLogoIcon:function(e,t){var n=this.getOptions(e).logoIcon,r=void 0===n?{}:n,i=this.getSize(e)[0];if(r.show){var o=r.width,a=r.height,s=r.x,l=r.y,u=r.offset,c=Object(w.__rest)(r,["width","height","x","y","offset"]);t.addShape("image",{attrs:Object(w.__assign)(Object(w.__assign)({},c),{x:s||-i/2+o+u,y:l||-a/2,width:o,height:a}),className:"rect-logo-icon",name:"rect-logo-icon",draggable:!0})}},drawStateIcon:function(e,t){var n=this.getOptions(e).stateIcon,r=void 0===n?{}:n,i=this.getSize(e)[0];if(r.show){var o=r.width,a=r.height,s=r.x,l=r.y,u=r.offset,c=Object(w.__rest)(r,["width","height","x","y","offset"]);t.addShape("image",{attrs:Object(w.__assign)(Object(w.__assign)({},c),{x:s||i/2-o+u,y:l||-a/2,width:o,height:a}),className:"rect-state-icon",name:"rect-state-icon",draggable:!0})}},drawLinkPoints:function(e,t){var n=this.getOptions(e).linkPoints,r=void 0===n?{}:n,i=r.top,o=r.left,a=r.right,s=r.bottom,l=r.size,u=r.r,c=Object(w.__rest)(r,["top","left","right","bottom","size","r"]),d=this.getSize(e),h=d[0],f=d[1];o&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},c),{x:-h/2,y:0,r:l/2||u||5}),className:"link-point-left",name:"link-point-left",isAnchorPoint:!0}),a&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},c),{x:h/2,y:0,r:l/2||u||5}),className:"link-point-right",name:"link-point-right",isAnchorPoint:!0}),i&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},c),{x:0,y:-f/2,r:l/2||u||5}),className:"link-point-top",name:"link-point-top",isAnchorPoint:!0}),s&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},c),{x:0,y:f/2,r:l/2||u||5}),className:"link-point-bottom",name:"link-point-bottom",isAnchorPoint:!0})},drawLabel:function(e,t){var n=this.getOptions(e),r=n.labelCfg,i=void 0===r?{}:r,o=n.logoIcon,a=void 0===o?{}:o,s=n.descriptionCfg,l=void 0===s?{}:s,u=this.getSize(e)[0],c=null,d=a.show,h=a.width,f=-u/2+i.offset;d&&(f=-u/2+h+i.offset);var p=i.style,g=l.style,v=l.paddingTop;return Object(ce.isString)(e.description)?(c=t.addShape("text",{attrs:Object(w.__assign)(Object(w.__assign)({},p),{x:f,y:-5,text:e.label}),className:"text-shape",name:"text-shape",draggable:!0}),t.addShape("text",{attrs:Object(w.__assign)(Object(w.__assign)({},g),{x:f,y:17+(v||0),text:e.description}),className:"rect-description",name:"rect-description",draggable:!0})):c=t.addShape("text",{attrs:Object(w.__assign)(Object(w.__assign)({},p),{x:f,y:7,text:e.label}),draggable:!0}),c},getShapeStyle:function(e){var t=this.getOptions(e).style,n={stroke:e.color},r=Object(ce.mix)({},t,n),i=this.getSize(e),o=r.width||i[0],a=r.height||i[1];return Object(w.__assign)({x:-o/2,y:-a/2,width:o,height:a},r)},update:function(e,t){var n=this.getOptions(e),r=n.style,i=void 0===r?{}:r,o=n.labelCfg,a=void 0===o?{}:o,s=n.descriptionCfg,l=void 0===s?{}:s,u=this.getSize(e),c=u[0],d=u[1];t.get("keyShape").attr(Object(w.__assign)(Object(w.__assign)({},i),{x:-c/2,y:-d/2,width:c,height:d}));var h=t.getContainer(),f=h.find((function(e){return"rect-logo-icon"===e.get("className")})),p=f?f.attr():{},g=Object(ce.mix)({},p,e.logoIcon),v=g.width;void 0===v&&(v=this.options.logoIcon.width);var m=e.logoIcon?e.logoIcon.show:void 0,y=a.offset,b=-c/2+v+y;m||void 0===m||(b=-c/2+y);var C=h.find((function(e){return"node-label"===e.get("className")})),A=h.find((function(e){return"rect-description"===e.get("className")}));if(e.label)if(C){var _=e.labelCfg?e.labelCfg.style:{},I=Object(ce.mix)({},C.attr(),_);e.label&&(I.text=e.label),I.x=b,Object(ce.isString)(e.description)&&(I.y=-5),A&&(A.resetMatrix(),A.attr({x:b})),C.resetMatrix(),C.attr(I)}else h.addShape("text",{attrs:Object(w.__assign)(Object(w.__assign)({},a.style),{x:b,y:e.description?-5:7,text:e.label}),className:"node-label",name:"node-label",draggable:!0});if(Object(ce.isString)(e.description)){var x=l.paddingTop;if(A){_=e.descriptionCfg?e.descriptionCfg.style:{};var M=Object(ce.mix)({},A.attr(),_);Object(ce.isString)(e.description)&&(M.text=e.description),M.x=b,A.resetMatrix(),A.attr(Object(w.__assign)(Object(w.__assign)({},M),{y:17+(x||0)}))}else h.addShape("text",{attrs:Object(w.__assign)(Object(w.__assign)({},l.style),{x:b,y:17+(x||0),text:e.description}),className:"rect-description",name:"rect-description",draggable:!0})}var S=h.find((function(e){return"pre-rect"===e.get("className")}));if(S){var T=Object(ce.mix)({},S.attr(),e.preRect);S.attr(Object(w.__assign)(Object(w.__assign)({},T),{x:-c/2,y:-d/2,height:d}))}if(f)if(m||void 0===m){var E=g.width,O=g.height,k=g.x,L=g.y,D=g.offset,P=Object(w.__rest)(g,["width","height","x","y","offset"]);f.attr(Object(w.__assign)(Object(w.__assign)({},P),{x:k||-c/2+E+D,y:L||-O/2,width:E,height:O}))}else f.remove();else m&&this.drawLogoIcon(e,h);var N=h.find((function(e){return"rect-state-icon"===e.get("className")})),j=N?N.attr():{},R=Object(ce.mix)({},j,e.stateIcon);if(N){R.show||void 0===R.show||N.remove();var z=R.width,F=(O=R.height,k=R.x,L=R.y,R.offset),B=Object(w.__rest)(R,["width","height","x","y","offset"]);N.attr(Object(w.__assign)(Object(w.__assign)({},B),{x:k||c/2-z+F,y:L||-O/2,width:z,height:O}))}else R.show&&this.drawStateIcon(e,h);this.updateLinkPoints(e,h)}},"single-node"),On.registerNode("star",{options:{size:60,style:{stroke:J.defaultNode.style.stroke,fill:J.defaultNode.style.fill,lineWidth:J.defaultNode.style.lineWidth},labelCfg:{style:{fill:J.nodeLabel.style.fill,fontSize:J.nodeLabel.style.fontSize}},linkPoints:{top:!1,right:!1,bottom:!1,left:!1,size:J.defaultNode.linkPoints.size,lineWidth:J.defaultNode.linkPoints.lineWidth,fill:J.defaultNode.linkPoints.fill,stroke:J.defaultNode.linkPoints.stroke},icon:{show:!1,img:"https://gw.alipayobjects.com/zos/bmw-prod/5d015065-8505-4e7a-baec-976f81e3c41d.svg",width:20,height:20},stateStyles:Object(w.__assign)({},J.nodeStateStyles)},shapeType:"star",labelPosition:"center",drawShape:function(e,t){var n=this.getOptions(e).icon,r=void 0===n?{}:n,i=this.getShapeStyle(e),o=t.addShape("path",{attrs:i,className:this.type+"-keyShape",name:this.type+"-keyShape",draggable:!0}),a=r.width,s=r.height;return r.show&&t.addShape("image",{attrs:Object(w.__assign)({x:-a/2,y:-s/2},r),className:this.type+"-icon",name:this.type+"-icon",draggable:!0}),this.drawLinkPoints(e,t),o},drawLinkPoints:function(e,t){var n=this.getOptions(e).linkPoints,r=void 0===n?{}:n,i=r.top,o=r.left,a=r.right,s=r.leftBottom,l=r.rightBottom,u=r.size,c=r.r,d=Object(w.__rest)(r,["top","left","right","leftBottom","rightBottom","size","r"]),h=this.getSize(e)[0];if(a){var f=Math.cos(.1*Math.PI)*h,p=Math.sin(.1*Math.PI)*h;t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},d),{x:f,y:-p,r:u/2||c||5}),className:"link-point-right",name:"link-point-right"})}i&&(f=Math.cos(.5*Math.PI)*h,p=Math.sin(.5*Math.PI)*h,t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},d),{x:f,y:-p,r:u/2||c||5}),className:"link-point-top",name:"link-point-top"})),o&&(f=Math.cos(.9*Math.PI)*h,p=Math.sin(.9*Math.PI)*h,t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},d),{x:f,y:-p,r:u/2||c||5}),className:"link-point-left",name:"link-point-left"})),s&&(f=Math.cos(1.3*Math.PI)*h,p=Math.sin(1.3*Math.PI)*h,t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},d),{x:f,y:-p,r:u/2||c||5}),className:"link-point-left-bottom",name:"link-point-left-bottom"})),l&&(f=Math.cos(1.7*Math.PI)*h,p=Math.sin(1.7*Math.PI)*h,t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},d),{x:f,y:-p,r:u/2||c||5}),className:"link-point-right-bottom",name:"link-point-right-bottom"}))},getPath:function(e){for(var t=this.getSize(e)[0],n=3*t/8,r=e.innerR||n,i=[],o=0;o<5;o++){var a=Math.cos((18+72*o)/180*Math.PI)*t,s=Math.sin((18+72*o)/180*Math.PI)*t,l=Math.cos((54+72*o)/180*Math.PI)*r,u=Math.sin((54+72*o)/180*Math.PI)*r;0===o?i.push(["M",a,-s]):i.push(["L",a,-s]),i.push(["L",l,-u])}return i.push(["Z"]),i},getShapeStyle:function(e){var t=this.getOptions(e).style,n={stroke:e.color},r=Object(ce.mix)({},t,n),i=this.getPath(e);return Object(w.__assign)({path:i},r)},update:function(e,t){var n=t.getContainer(),r=this.getOptions({}).style,i=this.getPath(e),o={stroke:e.color,path:i},a=t.get("keyShape"),s=Object(ce.mix)({},r,a.attr(),o);s=Object(ce.mix)(s,e.style),this.updateShape(e,t,s,!0),this.updateLinkPoints(e,n)},updateLinkPoints:function(e,t){var n=this.getOptions({}).linkPoints,r=t.find((function(e){return"link-point-left"===e.get("className")})),i=t.find((function(e){return"link-point-right"===e.get("className")})),o=t.find((function(e){return"link-point-top"===e.get("className")})),a=t.find((function(e){return"link-point-left-bottom"===e.get("className")})),s=t.find((function(e){return"link-point-right-bottom"===e.get("className")})),l=n,u=r||i||o||a||s;u&&(l=u.attr());var c=Object(ce.mix)({},l,e.linkPoints),d=c.fill,h=c.stroke,f=c.lineWidth,p=c.size/2;p||(p=c.r);var g=e.linkPoints?e.linkPoints:{left:void 0,right:void 0,top:void 0,leftBottom:void 0,rightBottom:void 0},v=g.left,m=g.right,y=g.top,b=g.leftBottom,C=g.rightBottom,A=this.getSize(e)[0],_={r:p,fill:d,stroke:h,lineWidth:f},I=Math.cos(.1*Math.PI)*A,x=Math.sin(.1*Math.PI)*A;i?m||void 0===m?i.attr(Object(w.__assign)(Object(w.__assign)({},_),{x:I,y:-x})):i.remove():m&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},_),{x:I,y:-x}),className:"link-point-right",name:"link-point-right",isAnchorPoint:!0}),I=Math.cos(.5*Math.PI)*A,x=Math.sin(.5*Math.PI)*A,o?y||void 0===y?o.attr(Object(w.__assign)(Object(w.__assign)({},_),{x:I,y:-x})):o.remove():y&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},_),{x:I,y:-x}),className:"link-point-top",name:"link-point-top",isAnchorPoint:!0}),I=Math.cos(.9*Math.PI)*A,x=Math.sin(.9*Math.PI)*A,r?v||void 0===v?r.attr(Object(w.__assign)(Object(w.__assign)({},_),{x:I,y:-x})):r.remove():v&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},_),{x:I,y:-x}),className:"link-point-left",name:"link-point-left",isAnchorPoint:!0}),I=Math.cos(1.3*Math.PI)*A,x=Math.sin(1.3*Math.PI)*A,a?b||void 0===b?a.attr(Object(w.__assign)(Object(w.__assign)({},_),{x:I,y:-x})):a.remove():b&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},_),{x:I,y:-x}),className:"link-point-left-bottom",name:"link-point-left-bottom",isAnchorPoint:!0}),I=Math.cos(1.7*Math.PI)*A,x=Math.sin(1.7*Math.PI)*A,s?C||void 0===C?s.attr(Object(w.__assign)(Object(w.__assign)({},_),{x:I,y:-x})):a.remove():C&&t.addShape("circle",{attrs:Object(w.__assign)(Object(w.__assign)({},_),{x:I,y:-x}),className:"link-point-right-bottom",name:"link-point-right-bottom",isAnchorPoint:!0})}},"single-node"),On.registerNode("image",{options:{img:"https://gw.alipayobjects.com/mdn/rms_f8c6a0/afts/img/A*eD7nT6tmYgAAAAAAAAAAAABkARQnAQ",size:200,clipCfg:{show:!1,type:"circle",r:50,rx:50,ry:35,width:50,height:35,points:[[30,12],[12,30],[30,48],[48,30]],path:[["M",25,25],["L",50,25],["A",12.5,12.5,0,1,1,50,50],["A",12.5,12.5,0,1,0,50,50],["L",25,75],["Z"]],x:0,y:0}},shapeType:"image",labelPosition:"bottom",drawShape:function(e,t){var n=this.shapeType,r=this.getShapeStyle(e);delete r.fill;var i=t.addShape(n,{attrs:r,className:this.type+"-keyShape",name:this.type+"-keyShape",draggable:!0});return this.drawClip(e,i),i},drawClip:function(e,t){var n=this.getOptions(e).clipCfg;if(n.show){var r=n.type,i=n.x,o=n.y,a=n.style;if("circle"===r){var s=n.r;t.setClip({type:"circle",attrs:Object(w.__assign)({r:s,x:i,y:o},a)})}else if("rect"===r){var l=n.width,u=n.height,c=i-l/2,d=o-u/2;t.setClip({type:"rect",attrs:Object(w.__assign)({x:c,y:d,width:l,height:u},a)})}else if("ellipse"===r){var h=n.rx,f=n.ry;t.setClip({type:"ellipse",attrs:Object(w.__assign)({x:i,y:o,rx:h,ry:f},a)})}else if("polygon"===r){var p=n.points;t.setClip({type:"polygon",attrs:Object(w.__assign)({points:p},a)})}else if("path"===r){var g=n.path;t.setClip({type:"path",attrs:Object(w.__assign)({path:g},a)})}}},getShapeStyle:function(e){var t=this.getOptions(e).style,n=this.getSize(e),r=this.getOptions(e).img,i=n[0],o=n[1];return t&&(i=t.width||n[0],o=t.height||n[1]),Object(w.__assign)({x:-i/2,y:-o/2,width:i,height:o,img:r},t)},updateShapeStyle:function(e,t){var n=t.getContainer(),r=this.itemType+"-shape",i=n.find((function(e){return e.get("className")===r}))||t.getKeyShape(),o=this.getShapeStyle(e);i&&i.attr(o)}},"single-node");var Go=function(e){var t=e.x,n=e.y;return{x:t,y:n,centerX:t,centerY:n,minX:t,minY:n,maxX:t,maxY:n,height:0,width:0}},Xo=function(e,t){return 2*Math.abs(e.centerX-t.centerX)<e.width+t.width&&2*Math.abs(e.centerY-t.centerY)<e.height+t.height},qo=function(e){var t=[],n={};return e.forEach((function(e){var t=e.x+"-"+e.y;e.id=t,n[t]=e})),Object(ce.each)(n,(function(e){t.push(e)})),t},Zo=function(e){return qo(e)},Qo=function(e,t){return[e,{x:e.x,y:t.y},t]},Ko=function(e,t){return 0===e.width&&0===e.height?e:{centerX:e.centerX,centerY:e.centerY,minX:e.minX-t,minY:e.minY-t,maxX:e.maxX+t,maxY:e.maxY+t,height:e.height+2*t,width:e.width+2*t}},Jo=function(e,t){return function(e,t){var n=Math.abs(e.x-t.centerX),r=Math.abs(e.y-t.centerY);return n/t.width>r/t.height}(t,e)?{x:t.x>e.centerX?e.maxX:e.minX,y:t.y}:{x:t.x,y:t.y>e.centerY?e.maxY:e.minY}},$o=function(e,t){var n=Math.min(e.minX,t.minX),r=Math.min(e.minY,t.minY),i=Math.max(e.maxX,t.maxX),o=Math.max(e.maxY,t.maxY);return{centerX:(n+i)/2,centerY:(r+o)/2,minX:n,minY:r,maxX:i,maxY:o,height:o-r,width:i-n}},ea=function(e){var t=e.minX,n=e.minY,r=e.maxX,i=e.maxY;return[{x:t,y:n},{x:r,y:n},{x:r,y:i},{x:t,y:i}]},ta=function(e,t){var n=e.x,r=e.y;return n<t.minX||n>t.maxX||r<t.minY||r>t.maxY},na=function(e,t){return Math.abs(e.x-t.x)+Math.abs(e.y-t.y)},ra=function(e,t,n,r,i){return na(e,t)+na(e,n)+function(e,t){var n=0;return t.forEach((function(t){t&&(e.x===t.x&&(n+=-2),e.y===t.y&&(n+=-2))})),n}(e,[t,n,r,i])},ia=function(e,t,n,r,i){void 0===i&&(i=0),e.unshift(t[r]),n[r]&&n[r]!==r&&i<=100&&ia(e,t,n,n[r],i+1)},oa=function(e,t,n,r){var i=t.x-e.x,o=t.y-e.y,a=r.x-n.x,s=r.y-n.y,l=(-o*(e.x-n.x)+i*(e.y-n.y))/(-a*o+i*s),u=(a*(e.y-n.y)-s*(e.x-n.x))/(-a*o+i*s);return l>=0&&l<=1&&u>=0&&u<=1},aa=function(e,t,n){if(0===n.width&&0===n.height)return!1;var r=ea(n),i=r[0],o=r[1],a=r[2],s=r[3];return oa(e,t,i,o)||oa(e,t,i,s)||oa(e,t,o,a)||oa(e,t,a,s)},sa=function(e,t){var n=[],r=e[0];return n.push("M"+r.x+" "+r.y),e.forEach((function(r,i){var o=e[i+1],a=e[i+2];if(o&&a)if(function(e,t,n){return!(e.x===t.x&&t.x===n.x||e.y===t.y&&t.y===n.y)}(r,o,a)){var s=function(e,t,n,r){var i=na(e,t),o=na(n,t);return i<r&&(r=i),o<r&&(r=o),[{x:t.x-r/i*(t.x-e.x),y:t.y-r/i*(t.y-e.y)},{x:t.x-r/o*(t.x-n.x),y:t.y-r/o*(t.y-n.y)}]}(r,o,a,t),l=s[0],u=s[1];n.push("L"+l.x+" "+l.y),n.push("Q"+o.x+" "+o.y+" "+u.x+" "+u.y),n.push("L"+u.x+" "+u.y)}else n.push("L"+o.x+" "+o.y);else o&&n.push("L"+o.x+" "+o.y)})),n.join("")},la=function(e,t,n,r,i){var o,a;if(n&&n.getType()?"combo"===n.getType()?((o=n.getKeyShape().getCanvasBBox()||Go(e)).centerX=(o.minX+o.maxX)/2,o.centerY=(o.minY+o.maxY)/2):o=n.getBBox():o=Go(e),r&&r.getType()?"combo"===r.getType()?((a=r.getKeyShape().getCanvasBBox()||Go(t)).centerX=(a.minX+a.maxX)/2,a.centerY=(a.minY+a.maxY)/2):a=r&&r.getBBox():a=Go(t),Xo(o,a))return Zo(Qo(e,t));var s=Ko(o,i),l=Ko(a,i);if(Xo(s,l))return Zo(Qo(e,t));var u=Jo(s,e),c=Jo(l,t),d=function(e){void 0===e&&(e=[]);var t=[],n=[];e.forEach((function(e){t.push(e.x),n.push(e.y)}));var r=Math.min.apply(Math,t),i=Math.max.apply(Math,t),o=Math.min.apply(Math,n),a=Math.max.apply(Math,n);return{centerX:(r+i)/2,centerY:(o+a)/2,maxX:i,maxY:a,minX:r,minY:o,height:a-o,width:i-r}}([u,c]),h=($o(s,l),$o(s,d)),f=$o(l,d),p=[];p=(p=p.concat(ea(h))).concat(ea(f));var g={x:(e.x+t.x)/2,y:(e.y+t.y)/2};[d,h,f].forEach((function(e){p=p.concat(function(e,t){return function(e,t){return t<e.minX||t>e.maxX?[]:[{x:t,y:e.minY},{x:t,y:e.maxY}]}(e,t.x).concat(function(e,t){return t<e.minY||t>e.maxY?[]:[{x:e.minX,y:t},{x:e.maxX,y:t}]}(e,t.y))}(e,g).filter((function(e){return ta(e,s)&&ta(e,l)})))})),[{x:u.x,y:c.y},{x:c.x,y:u.y}].forEach((function(e){ta(e,s)&&ta(e,l)&&p.push(e)})),p.unshift(u),p.push(c);var v=function(e,t,n,r,i,o,a){var s=[],l=[t],u={},c={},d={};c[t.id]=0,d[t.id]=ra(t,n,t);var h={};e.forEach((function(e){h[e.id]=e}));for(var f=function(){var f,p,g,v,m=1/0;if(l.forEach((function(e){d[e.id]<m&&(m=d[e.id],f=e)})),f===n){var y=[];return ia(y,h,u,n.id),{value:y}}g=f,(v=(p=l).indexOf(g))>-1&&p.splice(v,1),s.push(f),function(e,t,n,r){var i=[];return e.forEach((function(e){e!==t&&(e.x!==t.x&&e.y!==t.y||aa(e,t,n)||aa(e,t,r)||i.push(e))})),qo(i)}(e,f,r,i).forEach((function(e){if(-1===s.indexOf(e)){-1===l.indexOf(e)&&l.push(e);var r=d[f.id]+na(f,e);c[e.id]&&r>=c[e.id]||(u[e.id]=f.id,c[e.id]=r,d[e.id]=c[e.id]+ra(e,n,t,o,a))}}))};l.length;){var p=f();if("object"==typeof p)return p.value}return[t,n]}(p=qo(p),u,c,o,a,e,t);return v.unshift(e),v.push(t),Zo(v)},ua={offset:20,maxAllowedDirectionChange:90,maximumLoops:2e3,gridSize:10,directions:[{stepX:1,stepY:0},{stepX:-1,stepY:0},{stepX:0,stepY:1},{stepX:0,stepY:-1}],get penalties(){return{0:0,45:this.gridSize/2,90:this.gridSize/2}},distFunc:function(e,t){return Math.abs(e.x-t.x)+Math.abs(e.y-t.y)},fallbackRoute:function(e,t,n,r,i){return Zo(la(e,t,n,r,i.offset))}},ca=function(e,t){var n=Math.floor(e/t);return n<0?0:n},da=function(e,t){var n=t.x-e.x,r=t.y-e.y;return n||r?(360+180*Math.atan2(r,n)/Math.PI)%360:0},ha=function(e,t){var n=Math.abs(e-t);return n>180?360-n:n},fa=function(e,t,n){for(var r=1/0,i=0,o=t.length;i<o;i++){var a=n(e,t[i]);a<r&&(r=a)}return r},pa=function(e,t,n){var r=[];if(!t)return r;var i=n.directions,o=n.offset,a=t.getBBox(),s=Ko(t.getBBox(),o);for(var l in s)s[l]=ca(s[l],n.gridSize);if(e.x>ca(a.minX,n.gridSize)&&e.x<ca(a.maxX,n.gridSize)&&e.y>ca(a.minY,n.gridSize)&&e.y<ca(a.maxY,n.gridSize))for(var u=0,c=i;u<c.length;u++){var d=c[u],h=[[{x:s.minX,y:s.minY},{x:s.maxX,y:s.minY}],[{x:s.minX,y:s.minY},{x:s.minX,y:s.maxY}],[{x:s.maxX,y:s.minY},{x:s.maxX,y:s.maxY}],[{x:s.minX,y:s.maxY},{x:s.maxX,y:s.maxY}]];for(l=0;l<4;l++){var f,p=h[l];(f=je(e,{x:e.x+d.stepX*s.width,y:e.y+d.stepY*s.height},p[0],p[1]))&&!aa(e,f,t.getBBox())&&(f.id=f.x+"-"+f.y,r.push(f))}}else(f=Jo(s,e)).id=f.x+"-"+f.y,r.push(f);return r},ga=function(e,t,n,r,i){var o=Ie()(ua,i);o.obstacles=o.obstacles||[];var a=function(e,t,n){var r={};return e.forEach((function(e){if(e)for(var i=Ko(e.getBBox(),n),o=ca(i.minX,t);o<=ca(i.maxX,t);o+=1)for(var a=ca(i.minY,t);a<=ca(i.maxY,t);a+=1)r[o+"-"+a]=!0})),r}(o.obstacles.concat([n,r]),o.gridSize,o.offset),s={x:ca(e.x,o.gridSize),y:ca(e.y,o.gridSize)},l={x:ca(t.x,o.gridSize),y:ca(t.y,o.gridSize)};e.id=s.x+"-"+s.y,t.id=l.x+"-"+l.y;var u=pa(s,n,o),c=pa(l,r,o);u.forEach((function(e){delete a[e.id]})),c.forEach((function(e){delete a[e.id]}));for(var d={},h={},f={},p={},g={},v=0;v<u.length;v++){var m=u[v];d[m.id]=m,p[m.id]=0,g[m.id]=fa(m,c,o.distFunc)}for(var y=function(e,t){var n,r=da(e,t);if(f[e.id]){var i=da({x:parseFloat(f[e.id].split("-")[0]),y:parseFloat(f[e.id].split("-")[1])},e);n=ha(i,r)}else{var o=da(s,e);n=ha(o,r)}return n},b=o.maximumLoops,C=function(){var n,r=1/0;for(var i in d)g[i]<=r&&(r=g[i],n=d[i]);if(!n)return"break";if(c.findIndex((function(e){return e.x===n.x&&e.y===n.y}))>-1)return{value:function(n){var r=[t],i={x:parseFloat(n.split("-")[0]),y:parseFloat(n.split("-")[1]),id:n};for(y(i,l)&&r.unshift({x:i.x===l.x?t.x:i.x*o.gridSize,y:i.y===l.y?t.y:i.y*o.gridSize});f[n]&&f[n]!==n;){var a={x:parseFloat(n.split("-")[0]),y:parseFloat(n.split("-")[1]),id:n},u=f[n],c={x:parseFloat(u.split("-")[0]),y:parseFloat(u.split("-")[1]),id:u};y(c,a)&&r.unshift({x:c.x===a.x?r[0].x:c.x*o.gridSize,y:c.y===a.y?r[0].y:c.y*o.gridSize}),n=u}var d=parseFloat(n.split("-")[0]),h=parseFloat(n.split("-")[1]);return r[0].x=d===s.x?e.x:r[0].x,r[0].y=h===s.y?e.y:r[0].y,r.unshift(e),r}(n.id)};delete d[n.id],h[n.id]=!0;for(var u=0;u<o.directions.length;u++){var v=o.directions[u],m={x:n.x+v.stepX,y:n.y+v.stepY,id:n.x+v.stepX+"-"+(n.y+v.stepY)};if(!h[m.id]){var C=y(n,m);if(!(C>o.maxAllowedDirectionChange||a[m.id])){d[m.id]||(d[m.id]=m);var A=o.distFunc(n,m)+(isNaN(o.penalties[C])?o.gridSize:o.penalties[C]),_=p[n.id]+A;p[m.id]&&_>=p[m.id]||(f[m.id]=n.id,p[m.id]=_,g[m.id]=_+fa(m,c,o.distFunc))}}}b-=1};Object.keys(d).length>0&&b>0;){var A=C();if("object"==typeof A)return A.value;if("break"===A)break}return o.fallbackRoute(e,t,n,r,o)};On.registerEdge("polyline",{options:{color:J.defaultEdge.color,size:J.defaultEdge.size,style:{radius:0,offset:15,x:0,y:0,stroke:J.defaultEdge.style.stroke,lineAppendWidth:J.defaultEdge.style.lineAppendWidth},labelCfg:{style:{fill:J.edgeLabel.style.fill,fontSize:J.edgeLabel.style.fontSize}},routeCfg:{obstacles:[],maxAllowedDirectionChange:90,maximumLoops:1e3,gridSize:10},stateStyles:Object(w.__assign)({},J.edgeStateStyles)},shapeType:"polyline",labelPosition:"center",drawShape:function(e,t){var n=this.getShapeStyle(e);return 0===n.radius&&delete n.radius,t.addShape("path",{className:"edge-shape",name:"edge-shape",attrs:n})},getShapeStyle:function(e){var t=this.options.style,n={stroke:e.color},r=Object(ce.mix)({},t,n,e.style);e=this.getPathPoints(e),this.radius=r.radius,this.offset=r.offset;var i=e.startPoint,o=e.endPoint,a=this.getControlPoints(e),s=[i];a&&(s=s.concat(a)),s.push(o);var l=e.sourceNode,u=e.targetNode,c=r.radius,d=this.options.routeCfg;d.offset=r.offset;var h=this.getPath(s,l,u,c,d);return(Object(ce.isArray)(h)&&h.length<=1||Object(ce.isString)(h)&&-1===h.indexOf("L"))&&(h="M0 0, L0 0"),(isNaN(i.x)||isNaN(i.y)||isNaN(o.x)||isNaN(o.y))&&(h="M0 0, L0 0"),Object(ce.mix)({},J.defaultEdge.style,r,{lineWidth:e.size,path:h})},updateShapeStyle:function(e,t){var n=t.getContainer(),r={stroke:e.color},i=n.find((function(e){return"edge-shape"===e.get("className")}))||t.getKeyShape(),o=e.size,a=(e=this.getPathPoints(e)).startPoint,s=e.endPoint,l=this.getControlPoints(e),u=[a];l&&(u=u.concat(l)),u.push(s);var c=Object(ce.mix)({},r,i.attr(),e.style),d=e.sourceNode,h=e.targetNode,f=c.radius,p=this.options.routeCfg;p.offset=c.offset;var g=this.getPath(u,d,h,f,p);(Object(ce.isArray)(g)&&g.length<=1||Object(ce.isString)(g)&&-1===g.indexOf("L"))&&(g="M0 0, L0 0"),(isNaN(a.x)||isNaN(a.y)||isNaN(s.x)||isNaN(s.y))&&(g="M0 0, L0 0");var v=Object(ce.mix)(r,i.attr(),{lineWidth:o,path:g},e.style);i&&i.attr(v)},getPath:function(e,t,n,r,i){var o;if(!i.offset||e.length>2){if(r)return sa(e,r);var a=[];return Object(ce.each)(e,(function(e,t){0===t?a.push(["M",e.x,e.y]):a.push(["L",e.x,e.y])})),a}return r?(o=ga(e[0],e[e.length-1],t,n,i),sa(o,r)):(o=ga(e[0],e[e.length-1],t,n,i),bt(o))}},"single-edge");var va={collapse:function(e,t,n){return[["M",e-n,t],["a",n,n,0,1,0,2*n,0],["a",n,n,0,1,0,2*-n,0],["M",e-n+4,t],["L",e+n-4,t]]},expand:function(e,t,n){return[["M",e-n,t],["a",n,n,0,1,0,2*n,0],["a",n,n,0,1,0,2*-n,0],["M",e-n+4,t],["L",e-n+2*n-4,t],["M",e-n+n,t-n+4],["L",e,t+n-4]]},upTriangle:function(e,t,n){var r=n*Math.cos(Math.PI/6),i=n*Math.sin(Math.PI/6);return[["M",e-r,t+i],["L",e+r,t+i],["L",e,t-n],["Z"]]},downTriangle:function(e,t,n){var r=n*Math.cos(Math.PI/6),i=n*Math.sin(Math.PI/6);return[["M",e-r,t-i],["L",e+r,t-i],["L",e,t+n],["Z"]]}},ma=On,ya={mat3:zt.mat3,mix:Nn.a,deepMix:Ie.a,transform:zt.transform},ba=Object(w.__assign)(Object(w.__assign)(Object(w.__assign)(Object(w.__assign)(Object(w.__assign)(Object(w.__assign)({},ya),r),s),a),o),i),Ca=function(){function e(e){this._cfgs=Ie()(this.getDefaultCfgs(),e),this._events={},this.destroyed=!1}return e.prototype.getDefaultCfgs=function(){return{}},e.prototype.initPlugin=function(e){var t=this;t.set("graph",e);var n=t.getEvents(),r={};f()(n,(function(n,i){var o=Gt()(t,n);r[i]=o,e.on(i,o)})),this._events=r,this.init()},e.prototype.init=function(){},e.prototype.getEvents=function(){return{}},e.prototype.get=function(e){return this._cfgs[e]},e.prototype.set=function(e,t){this._cfgs[e]=t},e.prototype.destroy=function(){},e.prototype.destroyPlugin=function(){this.destroy();var e=this.get("graph"),t=this._events;f()(t,(function(t,n){e.off(n,t)})),this._events=null,this._cfgs=null,this.destroyed=!0},e}(),Aa="url(data:image/svg+xml;base64,PHN2ZyB3aWR0aD0iNDAiIGhlaWdodD0iNDAiIHhtbG5zPSJodHRwOi8vd3d3LnczLm9yZy8yMDAwL3N2ZyI+PGRlZnM+PHBhdHRlcm4gaWQ9ImdyaWQiIHdpZHRoPSI0MCIgaGVpZ2h0PSI0MCIgcGF0dGVyblVuaXRzPSJ1c2VyU3BhY2VPblVzZSI+PHBhdGggZD0iTSAwIDEwIEwgNDAgMTAgTSAxMCAwIEwgMTAgNDAgTSAwIDIwIEwgNDAgMjAgTSAyMCAwIEwgMjAgNDAgTSAwIDMwIEwgNDAgMzAgTSAzMCAwIEwgMzAgNDAiIGZpbGw9Im5vbmUiIHN0cm9rZT0iI2UwZTBlMCIgb3BhY2l0eT0iMC4yIiBzdHJva2Utd2lkdGg9IjEiLz48cGF0aCBkPSJNIDQwIDAgTCAwIDAgMCA0MCIgZmlsbD0ibm9uZSIgc3Ryb2tlPSIjZTBlMGUwIiBzdHJva2Utd2lkdGg9IjEiLz48L3BhdHRlcm4+PC9kZWZzPjxyZWN0IHdpZHRoPSIxMDAlIiBoZWlnaHQ9IjEwMCUiIGZpbGw9InVybCgjZ3JpZCkiLz48L3N2Zz4=)",_a=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(w.__extends)(t,e),t.prototype.getDefaultCfgs=function(){return{img:Aa}},t.prototype.init=function(){var e=this.get("graph"),t=e.get("minZoom"),n=e.get("container"),r=e.get("canvas").get("el"),i=e.get("width"),o=e.get("height"),a=this.get("img")||Aa,s=ye()("<div class='g6-grid-container' style=\"position: absolute; left:0;top:0;right:0;bottom:0;overflow: hidden;z-index: -1;\"></div>"),l=ye()("<div \n        class='g6-grid' \n        style='position:absolute;\n        transform-origin: 0% 0% 0px;\n        background-image: "+a+";\n        user-select: none\n        '></div>");s.appendChild(l),ve()(s,{width:i+"px",height:o+"px",left:n.offsetLeft+"px",top:n.offsetTop+"px"}),ve()(l,{width:i/t+"px",height:o/t+"px",left:"0px",top:"0px"}),n.insertBefore(s,r),this.set("container",s),this.set("gridContainer",l)},t.prototype.getEvents=function(){return{viewportchange:"updateGrid"}},t.prototype.updateGrid=function(e){var t=this.get("gridContainer"),n=e.matrix;n||(n=[1,0,0,0,1,0,0,0,1]);var r="matrix("+n[0]+", "+n[1]+", "+n[3]+", "+n[4]+", 0, 0)";ve()(t,{transform:r})},t.prototype.getContainer=function(){return this.get("container")},t.prototype.destroy=function(){var e=this.get("graph").get("container"),t=this.get("container");e.removeChild(t)},t}(Ca),Ia=n(56),xa=n.n(Ia);xa()("\n  .g6-component-contextmenu {\n    border: 1px solid #e2e2e2;\n    border-radius: 4px;\n    font-size: 12px;\n    color: #545454;\n    background-color: rgba(255, 255, 255, 0.9);\n    padding: 10px 8px;\n    box-shadow: rgb(174, 174, 174) 0px 0px 10px;\n  }\n  .g6-contextmenu-ul {\n    padding: 0;\n    margin: 0;\n    list-style: none;\n  }\n\n");var wa=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(w.__extends)(t,e),t.prototype.getDefaultCfgs=function(){return{offsetX:6,offsetY:6,handleMenuClick:void 0,getContent:function(e){return"\n          <ul class='g6-contextmenu-ul'>\n            <li>菜单项1</li>\n            <li>菜单项2</li>\n          </ul>\n        "},shouldBegin:function(e){return!0},onHide:function(){return!0},itemTypes:["node","edge","combo"]}},t.prototype.getEvents=function(){return{contextmenu:"onMenuShow"}},t.prototype.init=function(){var e=this.get("className"),t=ye()("<div class="+(e||"g6-component-contextmenu")+"></div>");ve()(t,{top:"0px",position:"absolute",visibility:"hidden"});var n=this.get("container");n||(n=this.get("graph").get("container")),n.appendChild(t),this.set("menu",t)},t.prototype.onMenuShow=function(e){var t=this;if(e.preventDefault(),e.stopPropagation(),e.item&&this.get("shouldBegin")(e)){var n=this.get("itemTypes");if(e.item&&e.item.getType&&-1===n.indexOf(e.item.getType()))t.onMenuHide();else if(e.item){var r=this.get("menu"),i=this.get("getContent"),o=this.get("graph"),a=i(e,o);x()(a)?r.innerHTML=a:r.innerHTML=a.outerHTML,this.removeMenuEventListener();var s=this.get("handleMenuClick");if(s){var l=function(t){s(t.target,e.item,o)};this.set("handleMenuClickWrapper",l),r.addEventListener("click",l)}var u=o.get("width"),c=o.get("height"),d=r.getBoundingClientRect(),h=this.get("offsetX")||0,f=this.get("offsetY")||0,p=o.getContainer().offsetTop,g=o.getContainer().offsetLeft,v=e.canvasX+g+h,m=e.canvasY+p+f;v+d.width>u&&(v=e.canvasX-d.width-h+g),m+d.height>c&&(m=e.canvasY-d.height-f+p),ve()(r,{top:m+"px",left:v+"px",visibility:"visible"});var y=function(e){t.onMenuHide()};document.body.addEventListener("click",y),this.set("handler",y)}}},t.prototype.removeMenuEventListener=function(){var e=this.get("handleMenuClickWrapper"),t=this.get("handler");e&&(this.get("menu").removeEventListener("click",e),this.set("handleMenuClickWrapper",null)),t&&document.body.removeEventListener("click",t)},t.prototype.onMenuHide=function(){var e=this.get("menu");e&&ve()(e,{visibility:"hidden"}),this.removeMenuEventListener()},t.prototype.destroy=function(){var e=this.get("menu");if(this.removeMenuEventListener(),e){var t=this.get("container");t||(t=this.get("graph").get("container")),t.removeChild(e)}},t}(Ca),Ma=Math.max,Sa=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.handleUpdateCanvas=Object(ce.debounce)((function(e){var n=t;n.destroyed||n.updateCanvas()}),100,!1),t}return Object(w.__extends)(t,e),t.prototype.getDefaultCfgs=function(){return{container:null,className:"g6-minimap",viewportClassName:"g6-minimap-viewport",type:"default",padding:50,size:[200,120],delegateStyle:{fill:"#40a9ff",stroke:"#096dd9"},refresh:!0}},t.prototype.getEvents=function(){return{beforepaint:"updateViewport",beforeanimate:"disableRefresh",afteranimate:"enableRefresh",viewportchange:"disableOneRefresh"}},t.prototype.disableRefresh=function(){this.set("refresh",!1)},t.prototype.enableRefresh=function(){this.set("refresh",!0),this.updateCanvas()},t.prototype.disableOneRefresh=function(){this.set("viewportChange",!0)},t.prototype.initViewport=function(){var e=this,t=this._cfgs,n=t.size,r=t.graph;if(!this.destroyed){var i=this.get("canvas").get("container"),o=ye()("\n      <div\n        class="+t.viewportClassName+"\n        style='position:absolute;\n          left:0;\n          top:0;\n          box-sizing:border-box;\n          border: 2px solid #1980ff'\n        draggable=true>\n      </div>"),a=0,s=0,l=!1,u=0,c=0,d=0,h=0,f=0,p=0;o.addEventListener("dragstart",(function(i){if(i.dataTransfer){var g=new Image;g.src="data:image/svg+xml,%3Csvg xmlns='http://www.w3.org/2000/svg' %3E%3Cpath /%3E%3C/svg%3E",i.dataTransfer.setDragImage(g,0,0),i.dataTransfer.setData("text","view-port-minimap")}if(t.refresh=!1,i.target===o){var v=o.style;u=parseInt(v.left,10),c=parseInt(v.top,10),d=parseInt(v.width,10),h=parseInt(v.height,10),d>n[0]||h>n[1]||(p=r.getZoom(),f=e.get("ratio"),l=!0,a=i.clientX,s=i.clientY)}}),!1),o.addEventListener("drag",(function(e){if(l&&!C()(e.clientX)&&!C()(e.clientY)){var t=a-e.clientX,i=s-e.clientY;(u-t<0||u-t+d>=n[0])&&(t=0),(c-i<0||c-i+h>=n[1])&&(i=0),u-=t,c-=i,ve()(o,{left:u+"px",top:c+"px"}),r.translate(t*p/f,i*p/f),a=e.clientX,s=e.clientY}}),!1),o.addEventListener("dragend",(function(){l=!1,t.refresh=!0}),!1),this.set("viewport",o),i.appendChild(o)}},t.prototype.updateViewport=function(){if(!this.destroyed){var e=this.get("ratio"),t=(this.get("dx"),this.get("dy"),this.get("totaldx")),n=this.get("totaldy"),r=this.get("graph"),i=this.get("size"),o=r.get("width"),a=r.get("height"),s=r.getPointByCanvas(0,0),l=r.getPointByCanvas(o,a),u=this.get("viewport");u||this.initViewport(),r.getZoom();var c=(l.x-s.x)*e,d=(l.y-s.y)*e,h=s.x*e+t,f=s.y*e+n,p=h+c,g=f+d;h<0&&(c+=h,h=0),p>i[0]&&(c-=p-i[0]),f<0&&(d+=f,f=0),g>i[1]&&(d-=g-i[1]),this.set("ratio",e);var v=h+"px",m=f+"px";ve()(u,{left:v,top:m,width:c+"px",height:d+"px"})}},t.prototype.updateGraphShapes=function(){var e=this._cfgs.graph,t=this.get("canvas"),n=e.get("group");if(!n.destroyed){var r=n.clone();r.resetMatrix(),t.clear(),t.add(r),"svg"===e.get("renderer")&&this.updateVisible(r)}},t.prototype.updateVisible=function(e){var t=this;if(e.isGroup()||e.get("visible")){var n=e.get("children");if(!n||!n.length)return;n.forEach((function(e){e.get("visible")||e.hide(),t.updateVisible(e)}))}else e.hide()},t.prototype.updateKeyShapes=function(){var e=this,t=this._cfgs.graph;f()(t.getEdges(),(function(t){e.updateOneEdgeKeyShape(t)})),f()(t.getNodes(),(function(t){e.updateOneNodeKeyShape(t)})),this.clearDestroyedShapes()},t.prototype.updateOneNodeKeyShape=function(e){var t=this.get("canvas"),n=t.get("children")[0]||t.addGroup(),r=this.get("itemMap")||{},i=r[e.get("id")],o=e.getBBox(),a=e.get("keyShape").clone(),s=a.attr(),l={x:o.centerX,y:o.centerY};i?l=Object.assign(s,l):(i=a,n.add(i));var u=i.get("type");"rect"!==u&&"image"!==u||(l.x=o.minX,l.y=o.minY),i.attr(l),e.isVisible()||i.hide(),i.exist=!0,r[e.get("id")]=i,this.set("itemMap",r)},t.prototype.updateDelegateShapes=function(){var e=this,t=this._cfgs.graph;f()(t.getEdges(),(function(t){e.updateOneEdgeKeyShape(t)})),f()(t.getNodes(),(function(t){e.updateOneNodeDelegateShape(t)})),this.clearDestroyedShapes()},t.prototype.clearDestroyedShapes=function(){var e=this.get("itemMap")||{},t=Object.keys(e);if(t&&0!==t.length)for(var n=t.length-1;n>=0;n--){var r=e[t[n]],i=r.exist;r.exist=!1,i||(r.remove(),delete e[t[n]])}},t.prototype.updateOneEdgeKeyShape=function(e){var t=this.get("canvas"),n=t.get("children")[0]||t.addGroup(),r=this.get("itemMap")||{},i=r[e.get("id")];if(i){var o=e.get("keyShape").attr("path");i.attr("path",o)}else i=e.get("keyShape").clone(),n.add(i),i.toBack();e.isVisible()||i.hide(),i.exist=!0,r[e.get("id")]=i,this.set("itemMap",r)},t.prototype.updateOneNodeDelegateShape=function(e){var t=this.get("canvas"),n=t.get("children")[0]||t.addGroup(),r=this.get("delegateStyle"),i=this.get("itemMap")||{},o=i[e.get("id")],a=e.getBBox();if(o){var s={x:a.minX,y:a.minY,width:a.width,height:a.height};o.attr(s)}else o=n.addShape("rect",{attrs:Object(w.__assign)({x:a.minX,y:a.minY,width:a.width,height:a.height},r),name:"minimap-node-shape"});e.isVisible()||o.hide(),o.exist=!0,i[e.get("id")]=o,this.set("itemMap",i)},t.prototype.init=function(){this.initContainer(),this.get("graph").on("afterupdateitem",this.handleUpdateCanvas),this.get("graph").on("afteritemstatechange",this.handleUpdateCanvas),this.get("graph").on("afteradditem",this.handleUpdateCanvas),this.get("graph").on("afterremoveitem",this.handleUpdateCanvas),this.get("graph").on("afterrender",this.handleUpdateCanvas),this.get("graph").on("afterlayout",this.handleUpdateCanvas)},t.prototype.initContainer=function(){var e=this.get("graph"),t=this.get("size"),n=this.get("className"),r=this.get("container"),i=ye()("<div class='"+n+"' style='width: "+t[0]+"px; height: "+t[1]+"px; overflow: hidden'></div>");x()(r)&&(r=document.getElementById(r)),r?r.appendChild(i):e.get("container").appendChild(i),this.set("container",i);var o,a=ye()('<div class="g6-minimap-container" style="position: relative;"></div>');i.appendChild(a),o="svg"===e.get("renderer")?new Rt.a({container:a,width:t[0],height:t[1]}):new Nt.a({container:a,width:t[0],height:t[1]}),this.set("canvas",o),this.updateCanvas()},t.prototype.updateCanvas=function(){if(this.get("refresh")){var e=this.get("graph");if(!e.get("destroyed")){this.get("viewportChange")&&(this.set("viewportChange",!1),this.updateViewport());var t=this.get("size"),n=this.get("canvas"),r=this.get("type"),i=this.get("padding");if(!n.destroyed){switch(r){case"default":this.updateGraphShapes();break;case"keyShape":this.updateKeyShapes();break;case"delegate":this.updateDelegateShapes()}var o=n.get("children")[0];if(o){o.resetMatrix();var a=o.getCanvasBBox(),s=e.get("canvas").getBBox(),l=s.width,u=s.height;Number.isFinite(a.width)&&(l=Ma(a.width,l),u=Ma(a.height,u)),l+=2*i,u+=2*i;var c=Math.min(t[0]/l,t[1]/u),d=[1,0,0,0,1,0,0,0,1],h=0,f=0;Number.isFinite(a.minX)&&(h=-a.minX),Number.isFinite(a.minY)&&(f=-a.minY);var p=(t[0]-(l-2*i)*c)/2,g=(t[1]-(u-2*i)*c)/2;d=fe(d,[["t",h,f],["s",c,c],["t",p,g]]),o.setMatrix(d),this.set("ratio",c),this.set("totaldx",p+h*c),this.set("totaldy",g+f*c),this.set("dx",p),this.set("dy",g),this.updateViewport()}}}}},t.prototype.getCanvas=function(){return this.get("canvas")},t.prototype.getViewport=function(){return this.get("viewport")},t.prototype.getContainer=function(){return this.get("container")},t.prototype.destroy=function(){this.get("canvas").destroy();var e=this.get("container");e.parentNode.removeChild(e)},t}(Ca);function Ta(e,t,n){var r=e.x-t.x,i=e.y-t.y;return!n||Math.abs(r)>n||Math.abs(i)>n?Math.sqrt(r*r+i*i):n}function Ea(e,t){var n=(t.source.y-t.target.y)/(t.source.x-t.target.x),r=(n*n*t.source.x+n*(e.y-t.source.y)+e.x)/(n*n+1);return{x:r,y:n*(r-t.source.x)+t.source.y}}var Oa=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(w.__extends)(t,e),t.prototype.getDefaultCfgs=function(){return{edgeBundles:[],edgePoints:[],K:.1,lambda:.1,divisions:1,divRate:2,cycles:6,iterations:90,iterRate:.6666667,bundleThreshold:.6,eps:1e-6,onLayoutEnd:function(){},onTick:function(){}}},t.prototype.init=function(){var e=this.get("graph"),t=this.get("onTick");this.set("tick",(function(){t&&t(),e.refreshPositions()}))},t.prototype.bundling=function(e){var t=this;if(t.set("data",e),!t.isTicking()){var n=e.edges||[],r=e.nodes||[],i={},o=!1;if(r.forEach((function(e){null!==e.x&&null!==!e.y&&void 0!==e.x&&void 0!==!e.y||(o=!0),i[e.id]=e})),o)throw new Error("please layout the graph or assign x and y for nodes first");t.set("nodeIdMap",i);var a=t.get("divisions"),s=t.get("divRate"),l=t.divideEdges(a);t.set("edgePoints",l);var u=t.getEdgeBundles();t.set("edgeBundles",u);for(var c=t.get("cycles"),d=t.get("iterations"),h=t.get("iterRate"),f=t.get("lambda"),p=0;p<c;p++){for(var g=function(e){var r=[];n.forEach((function(e,n){if(e.source!==e.target){var o=i[e.source],s=i[e.target];r[n]=t.getEdgeForces({source:o,target:s},n,a,f);for(var u=0;u<a+1;u++)l[n][u].x+=r[n][u].x,l[n][u].y+=r[n][u].y}}))},v=0;v<d;v++)g();f/=2,a*=s,d*=h,l=t.divideEdges(a),t.set("edgePoints",l)}n.forEach((function(e,t){e.source!==e.target&&(e.shape="polyline",e.type="polyline",e.controlPoints=l[t].slice(1,l[t].length-1))})),t.get("graph").refresh()}},t.prototype.updateBundling=function(e){var t=this,n=e.data;if(n&&t.set("data",n),t.get("ticking")&&t.set("ticking",!1),Object.keys(e).forEach((function(n){t.set(n,e[n])})),e.onTick){var r=this.get("graph");t.set("tick",(function(){e.onTick(),r.refresh()}))}t.bundling(n)},t.prototype.divideEdges=function(e){var t=this,n=t.get("data").edges,r=t.get("nodeIdMap"),i=t.get("edgePoints");return i&&void 0!==i||(i=[]),n.forEach((function(n,o){i[o]&&void 0!==i[o]||(i[o]=[]);var a=r[n.source],s=r[n.target];if(1===e)i[o].push({x:a.x,y:a.y}),i[o].push({x:.5*(a.x+s.x),y:.5*(a.y+s.y)}),i[o].push({x:s.x,y:s.y});else{var l=(i[o]&&i[o]!==[]?t.getEdgeLength(i[o]):Ta({x:a.x,y:a.y},{x:s.x,y:s.y}))/(e+1),u=l,c=[{x:a.x,y:a.y}];i[o].forEach((function(e,t){if(0!==t){for(var n=Ta(e,i[o][t-1]);n>u;){var r=u/n,a={x:i[o][t-1].x,y:i[o][t-1].y};a.x+=r*(e.x-i[o][t-1].x),a.y+=r*(e.y-i[o][t-1].y),c.push(a),n-=u,u=l}u-=n}})),c.push({x:s.x,y:s.y}),i[o]=c}})),i},t.prototype.getEdgeLength=function(e){var t=0;return e.forEach((function(n,r){0!==r&&(t+=Ta(n,e[r-1]))})),t},t.prototype.getEdgeBundles=function(){var e=this,t=e.get("data").edges||[],n=e.get("bundleThreshold"),r=e.get("nodeIdMap"),i=e.get("edgeBundles");return i||(i=[]),t.forEach((function(e,t){i[t]&&void 0!==i[t]||(i[t]=[])})),t.forEach((function(o,a){var s=r[o.source],l=r[o.target];t.forEach((function(t,o){if(!(o<=a)){var u=r[t.source],c=r[t.target];e.getBundleScore({source:s,target:l},{source:u,target:c})>=n&&(i[a].push(o),i[o].push(a))}}))})),i},t.prototype.getBundleScore=function(e,t){return e.vx=e.target.x-e.source.x,e.vy=e.target.y-e.source.y,t.vx=t.target.x-t.source.x,t.vy=t.target.y-t.source.y,e.length=Ta({x:e.source.x,y:e.source.y},{x:e.target.x,y:e.target.y}),t.length=Ta({x:t.source.x,y:t.source.y},{x:t.target.x,y:t.target.y}),this.getAngleScore(e,t)*this.getScaleScore(e,t)*this.getPositionScore(e,t)*this.getVisibilityScore(e,t)},t.prototype.getAngleScore=function(e,t){return function(e,t){return e.x*t.x+e.y*t.y}({x:e.vx,y:e.vy},{x:t.vx,y:t.vy})/(e.length*t.length)},t.prototype.getScaleScore=function(e,t){var n=(e.length+t.length)/2;return 2/(n/Math.min(e.length,t.length)+Math.max(e.length,t.length)/n)},t.prototype.getPositionScore=function(e,t){var n=(e.length+t.length)/2;return n/(n+Ta({x:(e.source.x+e.target.x)/2,y:(e.source.y+e.target.y)/2},{x:(t.source.x+t.target.x)/2,y:(t.source.y+t.target.y)/2}))},t.prototype.getVisibilityScore=function(e,t){var n=this.getEdgeVisibility(e,t),r=this.getEdgeVisibility(t,e);return n<r?n:r},t.prototype.getEdgeVisibility=function(e,t){var n=Ea(t.source,e),r=Ea(t.target,e),i={x:(n.x+r.x)/2,y:(n.y+r.y)/2},o={x:(e.source.x+e.target.x)/2,y:(e.source.y+e.target.y)/2};return Math.max(0,1-2*Ta(i,o)/Ta(n,r))},t.prototype.getEdgeForces=function(e,t,n,r){for(var i=this.get("edgePoints"),o=this.get("K")/(Ta(e.source,e.target)*(n+1)),a=[{x:0,y:0}],s=1;s<n;s++){var l={x:0,y:0},u=this.getSpringForce({pre:i[t][s-1],cur:i[t][s],next:i[t][s+1]},o),c=this.getElectrostaticForce(s,t);l.x=r*(u.x+c.x),l.y=r*(u.y+c.y),a.push(l)}return a.push({x:0,y:0}),a},t.prototype.getSpringForce=function(e,t){var n=e.pre.x+e.next.x-2*e.cur.x,r=e.pre.y+e.next.y-2*e.cur.y;return{x:n*=t,y:r*=t}},t.prototype.getElectrostaticForce=function(e,t){var n=this.get("eps"),r=this.get("edgeBundles"),i=this.get("edgePoints"),o=r[t],a={x:0,y:0};return o.forEach((function(r){var o={x:i[r][e].x-i[t][e].x,y:i[r][e].y-i[t][e].y};if(Math.abs(o.x)>n||Math.abs(o.y)>n){var s=1/Ta(i[r][e],i[t][e]);a.x+=o.x*s,a.y+=o.y*s}})),a},t.prototype.isTicking=function(){return this.get("ticking")},t.prototype.getSimulation=function(){return this.get("forceSimulation")},t.prototype.destroy=function(){this.get("ticking")&&this.getSimulation().stop(),e.prototype.destroy.call(this)},t}(Ca),ka={stroke:"#000",strokeOpacity:.8,lineWidth:2,fillOpacity:.1,fill:"#ccc"},La=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(w.__extends)(t,e),t.prototype.getDefaultCfgs=function(){return{trigger:"mousemove",d:1.5,r:300,delegateStyle:Object(p.clone)(ka),showLabel:!1,maxD:5,minD:0,scaleRBy:"unset",scaleDBy:"unset",showDPercent:!0}},t.prototype.getEvents=function(){var e;switch(this.get("trigger")){case"click":e={click:"magnify"};break;case"drag":e={click:"createDelegate"};break;default:e={mousemove:"magnify"}}return e},t.prototype.init=function(){var e=this.get("r");this.set("cachedMagnifiedModels",[]),this.set("cachedOriginPositions",{}),this.set("r2",e*e);var t=this.get("d");this.set("molecularParam",(t+1)*e)},t.prototype.createDelegate=function(e){var t=this,n=this,r=n.get("delegate");r&&!r.destroyed||(n.magnify(e),(r=n.get("delegate")).on("dragstart",(function(e){n.set("delegateCenterDiff",{x:r.attr("x")-e.x,y:r.attr("y")-e.y})})),r.on("drag",(function(e){n.magnify(e)})),"wheel"===this.get("scaleDBy")&&r.on("mousewheel",(function(e){t.scaleDByWheel(e)})),"wheel"===this.get("scaleRBy")&&r.on("mousewheel",(function(e){n.scaleRByWheel(e)})))},t.prototype.scaleRByWheel=function(e){if(e&&e.originalEvent){e.preventDefault&&e.preventDefault();var t,n=this.get("graph"),r=this.get("delegate"),i=(r?{x:r.attr("x"),y:r.attr("y")}:void 0)||n.getPointByClient(e.clientX,e.clientY);t=e.originalEvent.wheelDelta<0?.95:1/.95;var o=this.get("maxR"),a=this.get("minR"),s=this.get("r");(s>(o||n.get("height"))&&t>1||s<(a||.05*n.get("height"))&&t<1)&&(t=1),s*=t,this.set("r",s),this.set("r2",s*s);var l=this.get("d");this.set("molecularParam",(l+1)*s),this.set("delegateCenterDiff",void 0),this.magnify(e,i)}},t.prototype.scaleRByDrag=function(e){if(e){var t,n=this.get("dragPrePos"),r=this.get("graph"),i=r.getPointByClient(e.clientX,e.clientY);t=e.x-n.x<0?.95:1/.95;var o=this.get("maxR"),a=this.get("minR"),s=this.get("r");(s>(o||r.get("height"))&&t>1||s<(a||.05*r.get("height"))&&t<1)&&(t=1),s*=t,this.set("r",s),this.set("r2",s*s);var l=this.get("d");this.set("molecularParam",(l+1)*s),this.magnify(e,i),this.set("dragPrePos",{x:e.x,y:e.y})}},t.prototype.scaleDByWheel=function(e){if(e||e.originalEvent){var t;e.preventDefault&&e.preventDefault(),t=e.originalEvent.wheelDelta<0?-.1:.1;var n=this.get("d")+t,r=this.get("maxD"),i=this.get("minD");if(n<r&&n>i){this.set("d",n);var o=this.get("r");this.set("molecularParam",(n+1)*o);var a=this.get("delegate"),s=a?{x:a.attr("x"),y:a.attr("y")}:void 0;this.set("delegateCenterDiff",void 0),this.magnify(e,s)}}},t.prototype.scaleDByDrag=function(e){var t=this.get("dragPrePos"),n=e.x-t.x>0?.1:-.1,r=this.get("d")+n,i=this.get("maxD"),o=this.get("minD");if(r<i&&r>o){this.set("d",r);var a=this.get("r");this.set("molecularParam",(r+1)*a),this.magnify(e)}this.set("dragPrePos",{x:e.x,y:e.y})},t.prototype.magnify=function(e,t){this.restoreCache();var n=this.get("graph"),r=this.get("cachedMagnifiedModels"),i=this.get("cachedOriginPositions"),o=this.get("showLabel"),a=this.get("r"),s=this.get("r2"),l=this.get("d"),u=this.get("molecularParam"),c=n.getNodes(),d=c.length,h=t?{x:t.x,y:t.y}:{x:e.x,y:e.y};!this.get("dragging")||"mousemove"!==this.get("trigger")&&"click"!==this.get("trigger")||(h=this.get("cacheCenter"));var f=this.get("delegateCenterDiff");f&&(h.x+=f.x,h.y+=f.y),this.updateDelegate(h,a);for(var p=0;p<d;p++){var g=c[p].getModel(),v=g.x,m=g.y;if(!isNaN(v)&&!isNaN(m)){var y=(v-h.x)*(v-h.x)+(m-h.y)*(m-h.y);if(!isNaN(y)&&y<s&&0!==y){var b=Math.sqrt(y),C=u*b/(l*b+a),A=(v-h.x)/b,_=(m-h.y)/b;if(g.x=A*C+h.x,g.y=_*C+h.y,i[g.id]||(i[g.id]={x:v,y:m,texts:[]}),r.push(g),o&&2*b<a)for(var I=c[p].getContainer().getChildren(),x=I.length,w=0;w<x;w++){var M=I[w];"text"===M.get("type")&&(i[g.id].texts.push({visible:M.get("visible"),shape:M}),M.set("visible",!0))}}}}n.refreshPositions()},t.prototype.restoreCache=function(){for(var e=this.get("cachedMagnifiedModels"),t=this.get("cachedOriginPositions"),n=e.length,r=0;r<n;r++){var i=e[r],o=t[i.id];i.x=o.x,i.y=o.y;for(var a=o.texts.length,s=0;s<a;s++){var l=o.texts[s];l.shape.set("visible",l.visible)}}this.set("cachedMagnifiedModels",[]),this.set("cachedOriginPositions",{})},t.prototype.updateParams=function(e){var t=e.r,n=e.d,r=e.trigger,i=e.minD,o=e.maxD,a=e.minR,s=e.maxR,l=e.scaleDBy,u=e.scaleRBy;isNaN(e.r)||(this.set("r",t),this.set("r2",t*t)),isNaN(n)||this.set("d",n),isNaN(o)||this.set("maxD",o),isNaN(i)||this.set("minD",i),isNaN(s)||this.set("maxR",s),isNaN(a)||this.set("minR",a);var c,d=this.get("d"),h=this.get("r");this.set("molecularParam",(d+1)*h),"mousemove"!==r&&"click"!==r&&"drag"!==r||this.set("trigger",r),("drag"===l||"wheel"===l||"unset"===l)&&(this.set("scaleDBy",l),this.get("delegate").remove(),this.get("delegate").destroy(),(c=this.get("dPercentText"))&&(c.remove(),c.destroy())),"drag"!==u&&"wheel"!==u&&"unset"!==u||(this.set("scaleRBy",u),this.get("delegate").remove(),this.get("delegate").destroy(),(c=this.get("dPercentText"))&&(c.remove(),c.destroy()))},t.prototype.updateDelegate=function(e,t){var n=this,r=this,i=r.get("graph"),o=r.get("delegate");if(!o||o.destroyed){var a=i.get("group"),s=r.get("delegateStyle")||ka;o=a.addShape("circle",{attrs:Object(w.__assign)({r:t/1.5,x:e.x,y:e.y},s),name:"lens-shape",draggable:!0}),"drag"!==this.get("trigger")&&("wheel"===this.get("scaleRBy")?o.on("mousewheel",(function(e){r.scaleRByWheel(e)})):"drag"===this.get("scaleRBy")&&(o.on("dragstart",(function(e){r.set("dragging",!0),r.set("cacheCenter",{x:e.x,y:e.y}),r.set("dragPrePos",{x:e.x,y:e.y})})),o.on("drag",(function(e){r.scaleRByDrag(e)})),o.on("dragend",(function(e){r.set("dragging",!1)}))),"wheel"===this.get("scaleDBy")?o.on("mousewheel",(function(e){n.scaleDByWheel(e)})):"drag"===this.get("scaleDBy")&&(o.on("dragstart",(function(e){r.set("dragging",!0),r.set("cacheCenter",{x:e.x,y:e.y}),r.set("dragPrePos",{x:e.x,y:e.y})})),o.on("drag",(function(e){n.scaleDByDrag(e)})),o.on("dragend",(function(e){r.set("dragging",!1)}))))}else o.attr({x:e.x,y:e.y,r:t/1.5});if(r.get("showDPercent")){var l=Math.round((r.get("d")-r.get("minD"))/(r.get("maxD")-r.get("minD"))*100),u=r.get("dPercentText"),c=e.y+t/1.5+16;!u||u.destroyed?(u=(a=i.get("group")).addShape("text",{attrs:{text:l+"%",x:e.x,y:c,fill:"#aaa",stroke:"#fff",lineWidth:1,fontSize:12}}),r.set("dPercentText",u)):u.attr({text:l+"%",x:e.x,y:c})}r.set("delegate",o)},t.prototype.clear=function(){var e=this.get("graph");this.restoreCache(),e.refreshPositions();var t=this.get("delegate");t&&!t.destroyed&&(t.remove(),t.destroy());var n=this.get("dPercentText");n&&!n.destroyed&&(n.remove(),n.destroy())},t.prototype.destroy=function(){this.clear()},t}(Ca);xa()("\n  .g6-component-toolbar {\n    position: absolute;\n    list-style-type: none;\n    padding: 6px;\n    left: 0px;\n    top: 0px;\n    background-color: rgba(255, 255, 255, 0.9);\n    border: 1px solid #e2e2e2;\n    border-radius: 4px;\n    font-size: 12px;\n    color: #545454;\n    margin: 0;\n  }\n  .g6-component-toolbar li {\n    float: left;\n    text-align: center;\n    width: 35px;\n    height: 24px;\n    cursor: pointer;\n\t\tlist-style-type:none;\n    list-style: none;\n    margin-left: 0px;\n  }\n  .g6-component-toolbar li .icon {\n    opacity: 0.7;\n  }\n  .g6-component-toolbar li .icon:hover {\n    opacity: 1;\n  }\n");var Da=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(w.__extends)(t,e),t.prototype.getDefaultCfgs=function(){return{handleClick:void 0,getContent:function(e){return'\n          <ul class=\'g6-component-toolbar\'>\n            <li code=\'redo\'>\n              <svg class="icon" viewBox="0 0 1024 1024" version="1.1" xmlns="http://www.w3.org/2000/svg" width="24" height="24">\n                <path d="M256 682.666667c0-102.741333 66.730667-213.333333 213.333333-213.333334 107.008 0 190.762667 56.576 230.570667 125.354667L611.968 682.666667H853.333333v-241.365334l-91.562666 91.562667C704.768 448.469333 601.130667 384 469.333333 384c-196.096 0-298.666667 150.229333-298.666666 298.666667h85.333333z" fill="" p-id="2041"></path>\n              </svg>\n            </li>\n            <li code=\'undo\'>\n              <svg class="icon" viewBox="0 0 1024 1024" version="1.1" xmlns="http://www.w3.org/2000/svg" width="24" height="24">\n                <path d="M170.666667 682.666667h241.365333l-87.936-87.978667C363.904 525.909333 447.658667 469.333333 554.666667 469.333333c146.602667 0 213.333333 110.592 213.333333 213.333334h85.333333c0-148.437333-102.570667-298.666667-298.666666-298.666667-131.797333 0-235.392 64.469333-292.48 148.821333L170.666667 441.301333V682.666667z" fill="" p-id="2764"></path>\n              </svg>\n            </li>\n            <li  code=\'zoomOut\'>\n              <svg class="icon" viewBox="0 0 1024 1024" version="1.1" xmlns="http://www.w3.org/2000/svg" width="24" height="24">\n                <path d="M658.432 428.736a33.216 33.216 0 0 1-33.152 33.152H525.824v99.456a33.216 33.216 0 0 1-66.304 0V461.888H360.064a33.152 33.152 0 0 1 0-66.304H459.52V296.128a33.152 33.152 0 0 1 66.304 0V395.52H625.28c18.24 0 33.152 14.848 33.152 33.152z m299.776 521.792a43.328 43.328 0 0 1-60.864-6.912l-189.248-220.992a362.368 362.368 0 0 1-215.36 70.848 364.8 364.8 0 1 1 364.8-364.736 363.072 363.072 0 0 1-86.912 235.968l192.384 224.64a43.392 43.392 0 0 1-4.8 61.184z m-465.536-223.36a298.816 298.816 0 0 0 298.432-298.432 298.816 298.816 0 0 0-298.432-298.432A298.816 298.816 0 0 0 194.24 428.8a298.816 298.816 0 0 0 298.432 298.432z"></path>\n              </svg>\n            </li>\n            <li code=\'zoomIn\'>\n              <svg class="icon" viewBox="0 0 1024 1024" version="1.1" xmlns="http://www.w3.org/2000/svg" width="24" height="24">\n                <path d="M639.936 416a32 32 0 0 1-32 32h-256a32 32 0 0 1 0-64h256a32 32 0 0 1 32 32z m289.28 503.552a41.792 41.792 0 0 1-58.752-6.656l-182.656-213.248A349.76 349.76 0 0 1 480 768 352 352 0 1 1 832 416a350.4 350.4 0 0 1-83.84 227.712l185.664 216.768a41.856 41.856 0 0 1-4.608 59.072zM479.936 704c158.784 0 288-129.216 288-288S638.72 128 479.936 128a288.32 288.32 0 0 0-288 288c0 158.784 129.216 288 288 288z" p-id="3853"></path>\n              </svg>\n            </li>\n            <li code=\'realZoom\'>\n              <svg class="icon" viewBox="0 0 1024 1024" version="1.1" xmlns="http://www.w3.org/2000/svg" width="20" height="24">\n                <path d="M384 320v384H320V320h64z m256 0v384H576V320h64zM512 576v64H448V576h64z m0-192v64H448V384h64z m355.968 576H92.032A28.16 28.16 0 0 1 64 931.968V28.032C64 12.608 76.608 0 95.168 0h610.368L896 192v739.968a28.16 28.16 0 0 1-28.032 28.032zM704 64v128h128l-128-128z m128 192h-190.464V64H128v832h704V256z"></path>\n              </svg>\n            </li>\n            <li code=\'autoZoom\'>\n              <svg class="icon" viewBox="0 0 1024 1024" version="1.1" xmlns="http://www.w3.org/2000/svg" width="20" height="24">\n                <path d="M684.288 305.28l0.128-0.64-0.128-0.64V99.712c0-19.84 15.552-35.904 34.496-35.712a35.072 35.072 0 0 1 34.56 35.776v171.008h170.944c19.648 0 35.84 15.488 35.712 34.432a35.072 35.072 0 0 1-35.84 34.496h-204.16l-0.64-0.128a32.768 32.768 0 0 1-20.864-7.552c-1.344-1.024-2.816-1.664-3.968-2.816-0.384-0.32-0.512-0.768-0.832-1.088a33.472 33.472 0 0 1-9.408-22.848zM305.28 64a35.072 35.072 0 0 0-34.56 35.776v171.008H99.776A35.072 35.072 0 0 0 64 305.216c0 18.944 15.872 34.496 35.84 34.496h204.16l0.64-0.128a32.896 32.896 0 0 0 20.864-7.552c1.344-1.024 2.816-1.664 3.904-2.816 0.384-0.32 0.512-0.768 0.768-1.088a33.024 33.024 0 0 0 9.536-22.848l-0.128-0.64 0.128-0.704V99.712A35.008 35.008 0 0 0 305.216 64z m618.944 620.288h-204.16l-0.64 0.128-0.512-0.128c-7.808 0-14.72 3.2-20.48 7.68-1.28 1.024-2.752 1.664-3.84 2.752-0.384 0.32-0.512 0.768-0.832 1.088a33.664 33.664 0 0 0-9.408 22.912l0.128 0.64-0.128 0.704v204.288c0 19.712 15.552 35.904 34.496 35.712a35.072 35.072 0 0 0 34.56-35.776V753.28h170.944c19.648 0 35.84-15.488 35.712-34.432a35.072 35.072 0 0 0-35.84-34.496z m-593.92 11.52c-0.256-0.32-0.384-0.768-0.768-1.088-1.088-1.088-2.56-1.728-3.84-2.688a33.088 33.088 0 0 0-20.48-7.68l-0.512 0.064-0.64-0.128H99.84a35.072 35.072 0 0 0-35.84 34.496 35.072 35.072 0 0 0 35.712 34.432H270.72v171.008c0 19.84 15.552 35.84 34.56 35.776a35.008 35.008 0 0 0 34.432-35.712V720l-0.128-0.64 0.128-0.704a33.344 33.344 0 0 0-9.472-22.848zM512 374.144a137.92 137.92 0 1 0 0.128 275.84A137.92 137.92 0 0 0 512 374.08z"></path>\n              </svg>\n            </li>\n          </ul>\n        '}}},t.prototype.init=function(){var e=this,t=this.get("graph"),n=this.get("getContent")(t),r=n;Object(p.isString)(n)&&(r=ye()(n));var i=this.get("className");r.setAttribute("class",i||"g6-component-toolbar");var o=this.get("container");o||(o=this.get("graph").get("container")),o.appendChild(r),this.set("toolBar",r);var a=this.get("handleClick");r.addEventListener("click",(function(n){var r=n.path.filter((function(e){return"LI"===e.nodeName}));if(0!==r.length){var i=r[0].getAttribute("code");i&&(a?a(i,t):e.handleDefaultOperator(i,t))}}));var s=this.get("position");s&&ve()(r,{top:s.y+"px",left:s.x+"px"}),this.bindUndoRedo()},t.prototype.bindUndoRedo=function(){var e=this.get("graph"),t=document.querySelector('.g6-component-toolbar li[code="undo"]'),n=document.querySelector('.g6-component-toolbar li[code="undo"] svg'),r=document.querySelector('.g6-component-toolbar li[code="redo"]'),i=document.querySelector('.g6-component-toolbar li[code="redo"] svg');t&&n&&r&&i&&e.on("stackchange",(function(e){var o=e.undoStack,a=e.redoStack,s=o.length,l=a.length;1===s?(t.setAttribute("style","cursor: not-allowed"),n.setAttribute("style","opacity: 0.4")):(t.removeAttribute("style"),n.removeAttribute("style")),0===l?(r.setAttribute("style","cursor: not-allowed"),i.setAttribute("style","opacity: 0.4")):(r.removeAttribute("style"),i.removeAttribute("style"))}))},t.prototype.undo=function(){var e=this.get("graph"),t=e.getUndoStack();if(t&&0!==t.length){var n=t.pop();if(n){var r=n.action;e.pushStack(r,Object(p.clone)(n.data),"redo");var i=n.data.before;if("add"===r&&(i=n.data.after),!i)return;switch(r){case"visible":Object.keys(i).forEach((function(t){var n=i[t];n&&n.forEach((function(t){var n=e.findById(t.id);t.visible?e.showItem(n,!1):e.hideItem(n,!1)}))}));break;case"render":case"update":Object.keys(i).forEach((function(t){var n=i[t];n&&n.forEach((function(t){e.updateItem(t.id,t,!1)}))}));break;case"changedata":e.changeData(i,!1);break;case"delete":Object.keys(i).forEach((function(t){var n=i[t];n&&n.forEach((function(t){var n=t.itemType;delete t.itemType,e.addItem(n,t,!1)}))}));break;case"add":Object.keys(i).forEach((function(t){var n=i[t];n&&n.forEach((function(t){e.removeItem(t.id,!1)}))}))}}}},t.prototype.redo=function(){var e=this.get("graph"),t=e.getRedoStack();if(t&&0!==t.length){var n=t.pop();if(n){var r=n.action,i=n.data.after;if(e.pushStack(r,Object(p.clone)(n.data)),"delete"===r&&(i=n.data.before),!i)return;switch(r){case"visible":Object.keys(i).forEach((function(t){var n=i[t];n&&n.forEach((function(t){var n=e.findById(t.id);t.visible?e.showItem(n,!1):e.hideItem(n,!1)}))}));break;case"render":case"update":Object.keys(i).forEach((function(t){var n=i[t];n&&n.forEach((function(t){e.updateItem(t.id,t,!1)}))}));break;case"changedata":e.changeData(i,!1);break;case"delete":i.edges&&i.edges.forEach((function(t){e.removeItem(t.id,!1)})),i.nodes&&i.nodes.forEach((function(t){e.removeItem(t.id,!1)})),i.combos&&i.combos.forEach((function(t){e.removeItem(t.id,!1)}));break;case"add":Object.keys(i).forEach((function(t){var n=i[t];n&&n.forEach((function(t){var n=t.itemType;delete t.itemType,e.addItem(n,t,!1)}))}))}}}},t.prototype.handleDefaultOperator=function(e,t){var n=t.getZoom();switch(e){case"redo":this.redo();break;case"undo":this.undo();break;case"zoomOut":if(1.25*n>5)return;t.zoomTo(1.1*n);break;case"zoomIn":if(.75*n<.3)return;t.zoomTo(.9*n);break;case"realZoom":t.zoomTo(1);break;case"autoZoom":t.fitView([20,20])}},t.prototype.destroy=function(){var e=this.get("toolBar");if(e){var t=this.get("container");t||(t=this.get("graph").get("container")),t.removeChild(e)}var n=this.get("handleClick");n&&e.removeEventListener("click",n)},t}(Ca);xa()("\n  .g6-component-tooltip {\n    border: 1px solid #e2e2e2;\n    border-radius: 4px;\n    font-size: 12px;\n    color: #545454;\n    background-color: rgba(255, 255, 255, 0.9);\n    padding: 10px 8px;\n    box-shadow: rgb(174, 174, 174) 0px 0px 10px;\n  }\n  .tooltip-type {\n    padding: 0;\n    margin: 0;\n  }\n  .tooltip-id {\n    color: #531dab;\n  }\n");var Pa=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(w.__extends)(t,e),t.prototype.getDefaultCfgs=function(){return{offsetX:6,offsetY:6,getContent:function(e){return"\n          <h4 class='tooltip-type'>类型："+e.item.getType()+"</h4>\n          <span class='tooltip-id'>ID："+e.item.getID()+"</span>\n        "},shouldBegin:function(e){return!0},itemTypes:["node","edge","combo"]}},t.prototype.getEvents=function(){return{"node:mouseenter":"onMouseEnter","node:mouseleave":"onMouseLeave","node:mousemove":"onMouseMove","edge:mouseenter":"onMouseEnter","edge:mouseleave":"onMouseLeave","edge:mousemove":"onMouseMove",afterremoveitem:"onMouseLeave",contextmenu:"onMouseLeave","node:drag":"onMouseLeave"}},t.prototype.init=function(){var e=this.get("className")||"g6-component-tooltip",t=ye()("<div class="+e+"></div>"),n=this.get("container");n||(n=this.get("graph").get("container")),ve()(t,{position:"absolute",visibility:"hidden"}),n.appendChild(t),this.set("tooltip",t)},t.prototype.onMouseEnter=function(e){var t=this.get("itemTypes");if(!e.item||!e.item.getType||-1!==t.indexOf(e.item.getType())){var n=e.item,r=this.get("graph");this.currentTarget=n,this.showTooltip(e),r.emit("tooltipchange",{item:e.item,action:"show"})}},t.prototype.onMouseMove=function(e){var t=this.get("itemTypes");e.item&&e.item.getType&&-1===t.indexOf(e.item.getType())||this.currentTarget&&e.item===this.currentTarget&&this.showTooltip(e)},t.prototype.onMouseLeave=function(){this.hideTooltip(),this.get("graph").emit("tooltipchange",{item:this.currentTarget,action:"hide"}),this.currentTarget=null},t.prototype.showTooltip=function(e){if(e.item){var t=this.get("itemTypes");if(!e.item.getType||-1!==t.indexOf(e.item.getType())){var n=this.get("tooltip"),r=this.get("getContent")(e);x()(r)?n.innerHTML=r:n.innerHTML=r.outerHTML,this.updatePosition(e)}}},t.prototype.hideTooltip=function(){var e=this.get("tooltip");e&&ve()(e,{visibility:"hidden"})},t.prototype.updatePosition=function(e){var t=this.get("shouldBegin"),n=this.get("tooltip");if(t(e)){var r=this.get("graph"),i=r.get("width"),o=r.get("height"),a=this.get("offsetX")||0,s=this.get("offsetY")||0,l=r.getPointByClient(e.clientX,e.clientY),u=r.getCanvasByPoint(l.x,l.y),c=u.x,d=u.y;c+=a,d+=s;var h=n.getBoundingClientRect();c+h.width>i&&(c=c-h.width-a),d+h.height>o&&(d=d-h.height-s),ve()(n,{left:c+"px",top:d+"px",visibility:"visible"})}else ve()(n,{visibility:"hidden"})},t.prototype.hide=function(){this.onMouseLeave()},t.prototype.destroy=function(){var e=this.get("tooltip");if(e){var t=this.get("container");t||(t=this.get("graph").get("container")),t.removeChild(e)}},t}(Ca),Na={};function ja(e,t){Na[e]=t}var Ra=function(){function e(e){this.type="base",this.isCategory=!1,this.isLinear=!1,this.isContinuous=!1,this.isIdentity=!1,this.values=[],this.range=[0,1],this.ticks=[],this.__cfg__=e,this.initCfg(),this.init()}return e.prototype.translate=function(e){return e},e.prototype.change=function(e){Object(ce.assign)(this.__cfg__,e),this.init()},e.prototype.clone=function(){return this.constructor(this.__cfg__)},e.prototype.getTicks=function(){var e=this;return Object(ce.map)(this.ticks,(function(t,n){return Object(ce.isObject)(t)?t:{text:e.getText(t,n),tickValue:t,value:e.scale(t)}}))},e.prototype.getText=function(e,t){var n=this.formatter,r=n?n(e,t):e;return Object(ce.isNil)(r)||!Object(ce.isFunction)(r.toString)?"":r.toString()},e.prototype.getConfig=function(e){return this.__cfg__[e]},e.prototype.init=function(){Object(ce.assign)(this,this.__cfg__),this.setDomain(),Object(ce.isEmpty)(this.getConfig("ticks"))&&(this.ticks=this.calculateTicks())},e.prototype.initCfg=function(){},e.prototype.setDomain=function(){},e.prototype.calculateTicks=function(){var e=this.tickMethod,t=[];if(Object(ce.isString)(e)){var n=Na[e];if(!n)throw new Error("There is no method to to calculate ticks!");t=n(this)}else Object(ce.isFunction)(e)&&(t=e(this));return t},e.prototype.rangeMin=function(){return Object(ce.head)(this.range)},e.prototype.rangeMax=function(){return Object(ce.last)(this.range)},e.prototype.calcPercent=function(e,t,n){return Object(ce.isNumber)(e)?(e-t)/(n-t):NaN},e.prototype.calcValue=function(e,t,n){return t+e*(n-t)},e}(),za=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.type="cat",t.isCategory=!0,t}return Object(w.__extends)(t,e),t.prototype.translate=function(e){var t=Object(ce.indexOf)(this.values,e);return-1===t?Object(ce.isNumber)(e)?e:NaN:t},t.prototype.scale=function(e){var t=this.translate(e),n=this.calcPercent(t,this.min,this.max);return this.calcValue(n,this.rangeMin(),this.rangeMax())},t.prototype.invert=function(e){var t=this.max-this.min,n=this.calcPercent(e,this.rangeMin(),this.rangeMax()),r=Math.round(t*n)+this.min;return r<this.min||r>this.max?NaN:this.values[r]},t.prototype.getText=function(t){for(var n=[],r=1;r<arguments.length;r++)n[r-1]=arguments[r];var i=t;return Object(ce.isNumber)(t)&&!this.values.includes(t)&&(i=this.values[i]),e.prototype.getText.apply(this,Object(w.__spreadArrays)([i],n))},t.prototype.initCfg=function(){this.tickMethod="cat"},t.prototype.setDomain=function(){if(Object(ce.isNil)(this.getConfig("min"))&&(this.min=0),Object(ce.isNil)(this.getConfig("max"))){var e=this.values.length;this.max=e>1?e-1:e}},t}(Ra),Fa=/d{1,4}|M{1,4}|YY(?:YY)?|S{1,3}|Do|ZZ|Z|([HhMsDm])\1?|[aA]|"[^"]*"|'[^']*'/g,Ba="[^\\s]+",Ha=/\[([^]*?)\]/gm;function Ua(e,t){for(var n=[],r=0,i=e.length;r<i;r++)n.push(e[r].substr(0,t));return n}var Va=function(e){return function(t,n){var r=n[e].map((function(e){return e.toLowerCase()})).indexOf(t.toLowerCase());return r>-1?r:null}};function Ya(e){for(var t=[],n=1;n<arguments.length;n++)t[n-1]=arguments[n];for(var r=0,i=t;r<i.length;r++){var o=i[r];for(var a in o)e[a]=o[a]}return e}var Wa=["Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"],Ga=["January","February","March","April","May","June","July","August","September","October","November","December"],Xa=Ua(Ga,3),qa={dayNamesShort:Ua(Wa,3),dayNames:Wa,monthNamesShort:Xa,monthNames:Ga,amPm:["am","pm"],DoFn:function(e){return e+["th","st","nd","rd"][e%10>3?0:(e-e%10!=10?1:0)*e%10]}},Za=Ya({},qa),Qa=function(e){return Za=Ya(Za,e)},Ka=function(e){return e.replace(/[|\\{()[^$+*?.-]/g,"\\$&")},Ja=function(e,t){for(void 0===t&&(t=2),e=String(e);e.length<t;)e="0"+e;return e},$a={D:function(e){return String(e.getDate())},DD:function(e){return Ja(e.getDate())},Do:function(e,t){return t.DoFn(e.getDate())},d:function(e){return String(e.getDay())},dd:function(e){return Ja(e.getDay())},ddd:function(e,t){return t.dayNamesShort[e.getDay()]},dddd:function(e,t){return t.dayNames[e.getDay()]},M:function(e){return String(e.getMonth()+1)},MM:function(e){return Ja(e.getMonth()+1)},MMM:function(e,t){return t.monthNamesShort[e.getMonth()]},MMMM:function(e,t){return t.monthNames[e.getMonth()]},YY:function(e){return Ja(String(e.getFullYear()),4).substr(2)},YYYY:function(e){return Ja(e.getFullYear(),4)},h:function(e){return String(e.getHours()%12||12)},hh:function(e){return Ja(e.getHours()%12||12)},H:function(e){return String(e.getHours())},HH:function(e){return Ja(e.getHours())},m:function(e){return String(e.getMinutes())},mm:function(e){return Ja(e.getMinutes())},s:function(e){return String(e.getSeconds())},ss:function(e){return Ja(e.getSeconds())},S:function(e){return String(Math.round(e.getMilliseconds()/100))},SS:function(e){return Ja(Math.round(e.getMilliseconds()/10),2)},SSS:function(e){return Ja(e.getMilliseconds(),3)},a:function(e,t){return e.getHours()<12?t.amPm[0]:t.amPm[1]},A:function(e,t){return e.getHours()<12?t.amPm[0].toUpperCase():t.amPm[1].toUpperCase()},ZZ:function(e){var t=e.getTimezoneOffset();return(t>0?"-":"+")+Ja(100*Math.floor(Math.abs(t)/60)+Math.abs(t)%60,4)},Z:function(e){var t=e.getTimezoneOffset();return(t>0?"-":"+")+Ja(Math.floor(Math.abs(t)/60),2)+":"+Ja(Math.abs(t)%60,2)}},es=function(e){return+e-1},ts=[null,"[1-9]\\d?"],ns=[null,Ba],rs=["isPm",Ba,function(e,t){var n=e.toLowerCase();return n===t.amPm[0]?0:n===t.amPm[1]?1:null}],is=["timezoneOffset","[^\\s]*?[\\+\\-]\\d\\d:?\\d\\d|[^\\s]*?Z?",function(e){var t=(e+"").match(/([+-]|\d\d)/gi);if(t){var n=60*+t[1]+parseInt(t[2],10);return"+"===t[0]?n:-n}return 0}],os={D:["day","[1-9]\\d?"],DD:["day","\\d\\d"],Do:["day","[1-9]\\d?"+Ba,function(e){return parseInt(e,10)}],M:["month","[1-9]\\d?",es],MM:["month","\\d\\d",es],YY:["year","\\d\\d",function(e){var t=+(""+(new Date).getFullYear()).substr(0,2);return+(""+(+e>68?t-1:t)+e)}],h:["hour","[1-9]\\d?",void 0,"isPm"],hh:["hour","\\d\\d",void 0,"isPm"],H:["hour","[1-9]\\d?"],HH:["hour","\\d\\d"],m:["minute","[1-9]\\d?"],mm:["minute","\\d\\d"],s:["second","[1-9]\\d?"],ss:["second","\\d\\d"],YYYY:["year","\\d{4}"],S:["millisecond","\\d",function(e){return 100*+e}],SS:["millisecond","\\d\\d",function(e){return 10*+e}],SSS:["millisecond","\\d{3}"],d:ts,dd:ts,ddd:ns,dddd:ns,MMM:["month",Ba,Va("monthNamesShort")],MMMM:["month",Ba,Va("monthNames")],a:rs,A:rs,ZZ:is,Z:is},as={default:"ddd MMM DD YYYY HH:mm:ss",shortDate:"M/D/YY",mediumDate:"MMM D, YYYY",longDate:"MMMM D, YYYY",fullDate:"dddd, MMMM D, YYYY",isoDate:"YYYY-MM-DD",isoDateTime:"YYYY-MM-DDTHH:mm:ssZ",shortTime:"HH:mm",mediumTime:"HH:mm:ss",longTime:"HH:mm:ss.SSS"},ss=function(e){return Ya(as,e)},ls=function(e,t,n){if(void 0===t&&(t=as.default),void 0===n&&(n={}),"number"==typeof e&&(e=new Date(e)),"[object Date]"!==Object.prototype.toString.call(e)||isNaN(e.getTime()))throw new Error("Invalid Date pass to format");var r=[];t=(t=as[t]||t).replace(Ha,(function(e,t){return r.push(t),"@@@"}));var i=Ya(Ya({},Za),n);return(t=t.replace(Fa,(function(t){return $a[t](e,i)}))).replace(/@@@/g,(function(){return r.shift()}))};function us(e,t,n){if(void 0===n&&(n={}),"string"!=typeof t)throw new Error("Invalid format in fecha parse");if(t=as[t]||t,e.length>1e3)return null;var r={year:(new Date).getFullYear(),month:0,day:1,hour:0,minute:0,second:0,millisecond:0,isPm:null,timezoneOffset:null},i=[],o=[],a=t.replace(Ha,(function(e,t){return o.push(Ka(t)),"@@@"})),s={},l={};a=Ka(a).replace(Fa,(function(e){var t=os[e],n=t[0],r=t[1],o=t[3];if(s[n])throw new Error("Invalid format. "+n+" specified twice in format");return s[n]=!0,o&&(l[o]=!0),i.push(t),"("+r+")"})),Object.keys(l).forEach((function(e){if(!s[e])throw new Error("Invalid format. "+e+" is required in specified format")})),a=a.replace(/@@@/g,(function(){return o.shift()}));var u=e.match(new RegExp(a,"i"));if(!u)return null;for(var c=Ya(Ya({},Za),n),d=1;d<u.length;d++){var h=i[d-1],f=h[0],p=h[2],g=p?p(u[d],c):+u[d];if(null==g)return null;r[f]=g}1===r.isPm&&null!=r.hour&&12!=+r.hour?r.hour=+r.hour+12:0===r.isPm&&12==+r.hour&&(r.hour=0);for(var v=new Date(r.year,r.month,r.day,r.hour,r.minute,r.second,r.millisecond),m=[["month","getMonth"],["day","getDate"],["hour","getHours"],["minute","getMinutes"],["second","getSeconds"]],y=(d=0,m.length);d<y;d++)if(s[m[d][0]]&&r[m[d][0]]!==v[m[d][1]]())return null;return null==r.timezoneOffset?v:new Date(Date.UTC(r.year,r.month,r.day,r.hour,r.minute-r.timezoneOffset,r.second,r.millisecond))}var cs={format:ls,parse:us,defaultI18n:qa,setGlobalDateI18n:Qa,setGlobalDateMasks:ss};function ds(e,t){return(l.format||cs.format)(e,t)}function hs(e){return Object(ce.isString)(e)&&(e=e.indexOf("T")>0?new Date(e).getTime():new Date(e.replace(/-/gi,"/")).getTime()),Object(ce.isDate)(e)&&(e=e.getTime()),e}var fs=36e5,ps=24*fs,gs=31*ps,vs=[["HH:mm:ss",1e3],["HH:mm:ss",1e4],["HH:mm:ss",3e4],["HH:mm",6e4],["HH:mm",6e5],["HH:mm",18e5],["HH",fs],["HH",6*fs],["HH",12*fs],["YYYY-MM-DD",ps],["YYYY-MM-DD",4*ps],["YYYY-WW",7*ps],["YYYY-MM",gs],["YYYY-MM",4*gs],["YYYY-MM",6*gs],["YYYY",380*ps]],ms=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.type="timeCat",t}return Object(w.__extends)(t,e),t.prototype.translate=function(e){e=hs(e);var t=this.values.indexOf(e);return-1===t&&(t=Object(ce.isNumber)(e)&&e<this.values.length?e:NaN),t},t.prototype.getText=function(e,t){var n=this.translate(e);if(n>-1){var r=this.values[n],i=this.formatter;return i?i(r,t):ds(r,this.mask)}return e},t.prototype.initCfg=function(){this.tickMethod="time-cat",this.mask="YYYY-MM-DD",this.tickCount=7},t.prototype.setDomain=function(){var t=this.values;Object(ce.each)(t,(function(e,n){t[n]=hs(e)})),t.sort((function(e,t){return e-t})),e.prototype.setDomain.call(this)},t}(za),ys=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.isContinuous=!0,t}return Object(w.__extends)(t,e),t.prototype.scale=function(e){if(Object(ce.isNil)(e))return NaN;var t=this.rangeMin(),n=this.rangeMax();return this.max===this.min?t:t+this.getScalePercent(e)*(n-t)},t.prototype.init=function(){e.prototype.init.call(this);var t=this.ticks,n=Object(ce.head)(t),r=Object(ce.last)(t);n<this.min&&(this.min=n),r>this.max&&(this.max=r),Object(ce.isNil)(this.minLimit)||(this.min=n),Object(ce.isNil)(this.maxLimit)||(this.max=r)},t.prototype.setDomain=function(){var e=Object(ce.getRange)(this.values),t=e.min,n=e.max;Object(ce.isNil)(this.min)&&(this.min=t),Object(ce.isNil)(this.max)&&(this.max=n),this.min>this.max&&(this.min=t,this.max=n)},t.prototype.calculateTicks=function(){var t=this,n=e.prototype.calculateTicks.call(this);return this.nice||(n=Object(ce.filter)(n,(function(e){return e>=t.min&&e<=t.max}))),n},t.prototype.getScalePercent=function(e){var t=this.max,n=this.min;return(e-n)/(t-n)},t.prototype.getInvertPercent=function(e){return(e-this.rangeMin())/(this.rangeMax()-this.rangeMin())},t}(Ra),bs=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.type="linear",t.isLinear=!0,t}return Object(w.__extends)(t,e),t.prototype.invert=function(e){var t=this.getInvertPercent(e);return this.min+t*(this.max-this.min)},t.prototype.initCfg=function(){this.tickMethod="wilkinson-extended",this.nice=!1},t}(ys);function Cs(e,t){var n=Math.E;return t>=0?Math.pow(n,Math.log(t)/e):-1*Math.pow(n,Math.log(-t)/e)}function As(e,t){return 1===e?1:Math.log(t)/Math.log(e)}function _s(e,t,n){Object(ce.isNil)(n)&&(n=Math.max.apply(null,e));var r=n;return Object(ce.each)(e,(function(e){e>0&&e<r&&(r=e)})),r===n&&(r=n/t),r>1&&(r=1),r}var Is=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.type="log",t}return Object(w.__extends)(t,e),t.prototype.invert=function(e){var t,n=this.base,r=As(n,this.max),i=this.rangeMin(),o=this.rangeMax()-i,a=this.positiveMin;if(a){if(0===e)return 0;var s=1/(r-(t=As(n,a/n)))*o;if(e<s)return e/s*a}else t=As(n,this.min);var l=(e-i)/o*(r-t)+t;return Math.pow(n,l)},t.prototype.initCfg=function(){this.tickMethod="log",this.base=10,this.tickCount=6,this.nice=!0},t.prototype.setDomain=function(){e.prototype.setDomain.call(this);var t=this.min;if(t<0)throw new Error("When you use log scale, the minimum value must be greater than zero!");0===t&&(this.positiveMin=_s(this.values,this.base,this.max))},t.prototype.getScalePercent=function(e){var t=this.max,n=this.min;if(t===n)return 0;if(e<=0)return 0;var r=this.base,i=this.positiveMin;return i&&(n=1*i/r),e<i?e/i/(As(r,t)-As(r,n)):(As(r,e)-As(r,n))/(As(r,t)-As(r,n))},t}(ys),xs=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.type="pow",t}return Object(w.__extends)(t,e),t.prototype.invert=function(e){var t=this.getInvertPercent(e),n=this.exponent,r=Cs(n,this.max),i=Cs(n,this.min),o=t*(r-i)+i,a=o>=0?1:-1;return Math.pow(o,n)*a},t.prototype.initCfg=function(){this.tickMethod="pow",this.exponent=2,this.tickCount=5,this.nice=!0},t.prototype.getScalePercent=function(e){var t=this.max,n=this.min;if(t===n)return 0;var r=this.exponent;return(Cs(r,e)-Cs(r,n))/(Cs(r,t)-Cs(r,n))},t}(ys),ws=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.type="time",t}return Object(w.__extends)(t,e),t.prototype.getText=function(e,t){var n=this.translate(e),r=this.formatter;return r?r(n,t):ds(n,this.mask)},t.prototype.scale=function(t){var n=t;return(Object(ce.isString)(n)||Object(ce.isDate)(n))&&(n=this.translate(n)),e.prototype.scale.call(this,n)},t.prototype.translate=function(e){return hs(e)},t.prototype.initCfg=function(){this.tickMethod="time-pretty",this.mask="YYYY-MM-DD",this.tickCount=7,this.nice=!1},t.prototype.setDomain=function(){var e=this.values,t=this.getConfig("min"),n=this.getConfig("max");if(Object(ce.isNil)(t)&&Object(ce.isNumber)(t)||(this.min=this.translate(this.min)),Object(ce.isNil)(n)&&Object(ce.isNumber)(n)||(this.max=this.translate(this.max)),e&&e.length){var r=[],i=1/0,o=i,a=0;Object(ce.each)(e,(function(e){var t=hs(e);if(isNaN(t))throw new TypeError("Invalid Time: "+e+" in time scale!");i>t?(o=i,i=t):o>t&&(o=t),a<t&&(a=t),r.push(t)})),e.length>1&&(this.minTickInterval=o-i),Object(ce.isNil)(t)&&(this.min=i),Object(ce.isNil)(n)&&(this.max=a)}},t}(bs),Ms=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.type="quantize",t}return Object(w.__extends)(t,e),t.prototype.invert=function(e){var t=this.ticks,n=t.length,r=this.getInvertPercent(e),i=Math.floor(r*(n-1));if(i>=n-1)return Object(ce.last)(t);if(i<0)return Object(ce.head)(t);var o=t[i],a=i/(n-1);return o+(r-a)/((i+1)/(n-1)-a)*(t[i+1]-o)},t.prototype.initCfg=function(){this.tickMethod="r-pretty",this.tickCount=5,this.nice=!0},t.prototype.calculateTicks=function(){var t=e.prototype.calculateTicks.call(this);return this.nice||(Object(ce.last)(t)!==this.max&&t.push(this.max),Object(ce.head)(t)!==this.min&&t.unshift(this.min)),t},t.prototype.getScalePercent=function(e){var t=this.ticks;if(e<Object(ce.head)(t))return 0;if(e>Object(ce.last)(t))return 1;var n=0;return Object(ce.each)(t,(function(t,r){if(!(e>=t))return!1;n=r})),n/(t.length-1)},t}(ys),Ss=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.type="quantile",t}return Object(w.__extends)(t,e),t.prototype.initCfg=function(){this.tickMethod="quantile",this.tickCount=5,this.nice=!0},t}(Ms),Ts={};function Es(e,t){if(function(e){return Ts[e]}(e))throw new Error("type '"+e+"' existed.");Ts[e]=t}var Os=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.type="identity",t.isIdentity=!0,t}return Object(w.__extends)(t,e),t.prototype.calculateTicks=function(){return this.values},t.prototype.scale=function(e){return this.values[0]!==e&&Object(ce.isNumber)(e)?e:this.range[0]},t.prototype.invert=function(e){var t=this.range;return e<t[0]||e>t[1]?NaN:this.values[0]},t}(Ra),ks=[1,5,2,2.5,4,3],Ls=100*Number.EPSILON;function Ds(e,t,n,r,i,o){var a=Object(ce.size)(t),s=Object(ce.indexOf)(t,e),l=0,u=function(e,t){return(e%t+t)%t}(r,o);return(u<Ls||o-u<Ls)&&r<=0&&i>=0&&(l=1),1-s/(a-1)-n+l}function Ps(e,t,n){var r=Object(ce.size)(t);return 1-Object(ce.indexOf)(t,e)/(r-1)-n+1}function Ns(e,t,n,r,i,o){var a=(e-1)/(o-i),s=(t-1)/(Math.max(o,r)-Math.min(n,i));return 2-Math.max(a/s,s/a)}function js(e,t){return e>=t?2-(e-1)/(t-1):1}function Rs(e,t,n,r){var i=t-e;return 1-.5*(Math.pow(t-r,2)+Math.pow(e-n,2))/Math.pow(.1*i,2)}function zs(e,t,n){var r=t-e;if(n>r){var i=(n-r)/2;return 1-Math.pow(i,2)/Math.pow(.1*r,2)}return 1}function Fs(e,t,n,r,i,o){if(void 0===n&&(n=5),void 0===r&&(r=!0),void 0===i&&(i=ks),void 0===o&&(o=[.25,.2,.5,.05]),"number"!=typeof e||"number"!=typeof t)return{min:0,max:0,ticks:[]};if(e===t||1===n)return{min:e,max:t,ticks:[e]};for(var a={score:-2,lmin:0,lmax:0,lstep:0},s=1;s<1/0;){for(var l=0,u=i;l<u.length;l++){var c=u[l],d=Ps(c,i,s);if(Number.isNaN(d))throw new Error("NaN");if(o[0]*d+o[1]+o[2]+o[3]<a.score){s=1/0;break}for(var h=2;h<1/0;){var f=js(h,n);if(o[0]*d+o[1]+o[2]*f+o[3]<a.score)break;for(var p=(t-e)/(h+1)/s/c,g=Math.ceil(Math.log10(p));g<1/0;){var v=s*c*Math.pow(10,g),m=zs(e,t,v*(h-1));if(o[0]*d+o[1]*m+o[2]*f+o[3]<a.score)break;var y=Math.floor(t/v)*s-(h-1)*s,b=Math.ceil(e/v)*s;if(y>b)g+=1;else{for(var C=y;C<=b;C+=1){var A=C*(v/s),_=A+v*(h-1),I=v,x=Ds(c,i,s,A,_,I),w=Rs(e,t,A,_),M=Ns(h,n,e,t,A,_),S=o[0]*x+o[1]*w+o[2]*M+1*o[3];S>a.score&&(!r||A<=e&&_>=t)&&(a.lmin=A,a.lmax=_,a.lstep=I,a.score=S)}g+=1}}h+=1}}s+=1}for(var T=Number.isInteger(a.lstep)?0:Math.ceil(Math.abs(Math.log10(a.lstep))),E=[],O=a.lmin;O<=a.lmax;O+=a.lstep)E.push(O);var k=T?Object(ce.map)(E,(function(e){return Number.parseFloat(e.toFixed(T))})):E;return{min:Math.min(e,Object(ce.head)(k)),max:Math.max(t,Object(ce.last)(k)),ticks:k}}function Bs(e){var t=e.values,n=e.tickInterval,r=e.tickCount,i=t;if(Object(ce.isNumber)(n))return Object(ce.filter)(i,(function(e,t){return t%n==0}));var o=e.min,a=e.max;if(Object(ce.isNil)(o)&&(o=0),Object(ce.isNil)(a)&&(a=t.length-1),Object(ce.isNumber)(r)&&r<a-o){var s=Fs(o,a,r,!1,[1,2,5,3,4,7,6,8,9]).ticks;return Object(ce.filter)(s,(function(e){return e>=o&&e<=a})).map((function(e){return t[e]}))}return t.slice(o,a+1)}var Hs=Math.sqrt(50),Us=Math.sqrt(10),Vs=Math.sqrt(2),Ys=function(){function e(){this._domain=[0,1]}return e.prototype.domain=function(e){return e?(this._domain=Array.from(e,Number),this):this._domain.slice()},e.prototype.nice=function(e){var t,n;void 0===e&&(e=5);var r,i=this._domain.slice(),o=0,a=this._domain.length-1,s=this._domain[o],l=this._domain[a];return l<s&&(s=(t=[l,s])[0],l=t[1],o=(n=[a,o])[0],a=n[1]),(r=Ws(s,l,e))>0?r=Ws(s=Math.floor(s/r)*r,l=Math.ceil(l/r)*r,e):r<0&&(r=Ws(s=Math.ceil(s*r)/r,l=Math.floor(l*r)/r,e)),r>0?(i[o]=Math.floor(s/r)*r,i[a]=Math.ceil(l/r)*r,this.domain(i)):r<0&&(i[o]=Math.ceil(s*r)/r,i[a]=Math.floor(l*r)/r,this.domain(i)),this},e.prototype.ticks=function(e){return void 0===e&&(e=5),function(e,t,n){var r,i,o,a,s=-1;if(n=+n,(e=+e)==(t=+t)&&n>0)return[e];if((r=t<e)&&(i=e,e=t,t=i),0===(a=Ws(e,t,n))||!isFinite(a))return[];if(a>0)for(e=Math.ceil(e/a),t=Math.floor(t/a),o=new Array(i=Math.ceil(t-e+1));++s<i;)o[s]=(e+s)*a;else for(e=Math.floor(e*a),t=Math.ceil(t*a),o=new Array(i=Math.ceil(e-t+1));++s<i;)o[s]=(e-s)/a;return r&&o.reverse(),o}(this._domain[0],this._domain[this._domain.length-1],e||5)},e}();function Ws(e,t,n){var r=(t-e)/Math.max(0,n),i=Math.floor(Math.log(r)/Math.LN10),o=r/Math.pow(10,i);return i>=0?(o>=Hs?10:o>=Us?5:o>=Vs?2:1)*Math.pow(10,i):-Math.pow(10,-i)/(o>=Hs?10:o>=Us?5:o>=Vs?2:1)}function Gs(e,t,n){return("ceil"===n?Math.ceil(e/t):"floor"===n?Math.floor(e/t):Math.round(e/t))*t}function Xs(e,t,n){var r=Gs(e,n,"floor"),i=Gs(t,n,"ceil");r=Object(ce.fixedBase)(r,n),i=Object(ce.fixedBase)(i,n);for(var o=[],a=r;a<=i;a+=n){var s=Object(ce.fixedBase)(a,n);o.push(s)}return{min:r,max:i,ticks:o}}function qs(e,t,n){var r,i=e.minLimit,o=e.maxLimit,a=e.min,s=e.max,l=e.tickCount,u=void 0===l?5:l,c=Object(ce.isNil)(i)?Object(ce.isNil)(t)?a:t:i,d=Object(ce.isNil)(o)?Object(ce.isNil)(n)?s:n:o;if(c>d&&(d=(r=[c,d])[0],c=r[1]),u<=2)return[c,d];for(var h=(d-c)/(u-1),f=[],p=0;p<u;p++)f.push(c+h*p);return f}function Zs(e,t,n){void 0===n&&(n=5);var r={max:0,min:0,ticks:[]};if(e===t)return{max:t,min:e,ticks:[e]};var i=(t-e)/n,o=Math.pow(10,Math.floor(Math.log10(i))),a=o<1?Math.ceil(Math.abs(Math.log10(o))):0,s=o;2*o-i<1.5*(i-s)&&5*o-i<2.75*(i-(s=2*o))&&10*o-i<1.5*(i-(s=5*o))&&(s=10*o);var l=Math.ceil(t/s),u=Math.floor(e/s);r.max=Math.max(l*s,t),r.min=Math.min(u*s,e);for(var c=Number.parseFloat((u*s).toFixed(a));c<t;)r.ticks.push(c),c+=s,a&&(c=Number.parseFloat(c.toFixed(a)));return r.ticks.push(c),r}function Qs(e,t){var n=e.length*t;return 1===t?e[e.length-1]:0===t?e[0]:n%1!=0?e[Math.ceil(n)-1]:e.length%2==0?(e[n-1]+e[n])/2:e[n]}function Ks(e){return new Date(e).getFullYear()}function Js(e){return new Date(e,0,1).getTime()}function $s(e){return new Date(e).getMonth()}function el(e,t){return new Date(e,t,1).getTime()}ja("cat",Bs),ja("time-cat",(function(e){var t=Bs(e),n=Object(ce.last)(e.values);return n!==Object(ce.last)(t)&&t.push(n),t})),ja("wilkinson-extended",(function(e){var t=e.min,n=e.max,r=e.tickCount,i=e.nice,o=e.tickInterval,a=e.minLimit,s=e.maxLimit,l=Fs(t,n,r,i).ticks;return Object(ce.isNil)(a)&&Object(ce.isNil)(s)?o?Xs(t,n,o).ticks:l:qs(e,Object(ce.head)(l),Object(ce.last)(l))})),ja("r-pretty",(function(e){var t=e.min,n=e.max,r=e.tickCount,i=e.tickInterval,o=e.minLimit,a=e.maxLimit,s=Zs(t,n,r).ticks;return Object(ce.isNil)(o)&&Object(ce.isNil)(a)?i?Xs(t,n,i).ticks:s:qs(e,Object(ce.head)(s),Object(ce.last)(s))})),ja("time",(function(e){var t=e.min,n=e.max,r=e.minTickInterval,i=e.tickInterval,o=e.tickCount;if(i)o=Math.ceil((n-t)/i);else{var a=(n-t)/(i=function(e,t,n){var r,i=(r=function(e){return e[1]},function(e,t,n,i){for(var o=Object(ce.isNil)(n)?0:n,a=Object(ce.isNil)(i)?e.length:i;o<a;){var s=o+a>>>1;r(e[s])>t?a=s:o=s+1}return o})(vs,(t-e)/n)-1,o=vs[i];return i<0?o=vs[0]:i>=vs.length&&(o=Object(ce.last)(vs)),o}(t,n,o)[1])/o;a>1&&(i*=Math.ceil(a)),r&&i<r&&(i=r)}for(var s=[],l=t;l<n+i;l+=i)s.push(l);return s})),ja("time-pretty",(function(e){var t=e.min,n=e.max,r=e.minTickInterval,i=e.tickCount,o=e.tickInterval,a=[];o||(o=(n-t)/i,r&&o<r&&(o=r));var s=Ks(t);if(o>31536e6)for(var l=Ks(n),u=Math.ceil(o/31536e6),c=s;c<=l+u;c+=u)a.push(Js(c));else if(o>gs){var d=Math.ceil(o/gs),h=$s(t),f=function(e,t){var n=Ks(e),r=Ks(t),i=$s(e);return 12*(r-n)+($s(t)-i)%12}(t,n);for(c=0;c<=f+d;c+=d)a.push(el(s,c+h))}else if(o>ps){var p=(b=new Date(t)).getFullYear(),g=b.getMonth(),v=b.getDate(),m=Math.ceil(o/ps),y=function(e,t){return Math.ceil((t-e)/ps)}(t,n);for(c=0;c<y+m;c+=m)a.push(new Date(p,g,v+c).getTime())}else if(o>fs){p=(b=new Date(t)).getFullYear(),g=b.getMonth(),m=b.getDate();var b,C=b.getHours(),A=Math.ceil(o/fs),_=function(e,t){return Math.ceil((t-e)/fs)}(t,n);for(c=0;c<=_+A;c+=A)a.push(new Date(p,g,m,C+c).getTime())}else if(o>6e4){var I=function(e,t){return Math.ceil((t-e)/6e4)}(t,n),x=Math.ceil(o/6e4);for(c=0;c<=I+x;c+=x)a.push(t+6e4*c)}else{var w=o;w<1e3&&(w=1e3);var M=1e3*Math.floor(t/1e3),S=Math.ceil((n-t)/1e3),T=Math.ceil(w/1e3);for(c=0;c<S+T;c+=T)a.push(M+1e3*c)}return a.length>=512&&console.warn("Notice: current ticks length("+a.length+') >= 512, may cause performance issues, even out of memory. Because of the configure "tickInterval"(in milliseconds, current is '+o+") is too small, increase the value to solve the problem!"),a})),ja("log",(function(e){var t,n=e.base,r=e.tickCount,i=e.min,o=e.max,a=e.values,s=As(n,o);if(i>0)t=Math.floor(As(n,i));else{var l=_s(a,n,o);t=Math.floor(As(n,l))}for(var u=s-t,c=Math.ceil(u/r),d=[],h=t;h<s+c;h+=c)d.push(Math.pow(n,h));return i<=0&&d.unshift(0),d})),ja("pow",(function(e){var t=e.exponent,n=e.tickCount,r=Math.ceil(Cs(t,e.max));return Zs(Math.floor(Cs(t,e.min)),r,n).ticks.map((function(e){var n=e>=0?1:-1;return Math.pow(e,t)*n}))})),ja("quantile",(function(e){var t=e.tickCount,n=e.values;if(!n||!n.length)return[];for(var r=n.slice().sort((function(e,t){return e-t})),i=[],o=0;o<t;o++){var a=o/(t-1);i.push(Qs(r,a))}return i})),ja("d3-linear",(function(e){var t=e.min,n=e.max,r=e.tickInterval,i=e.minLimit,o=e.maxLimit,a=function(e){var t=e.min,n=e.max,r=e.nice,i=e.tickCount,o=new Ys;return o.domain([t,n]),r&&o.nice(i),o.ticks(i)}(e);return Object(ce.isNil)(i)&&Object(ce.isNil)(o)?r?Xs(t,n,r).ticks:a:qs(e,Object(ce.head)(a),Object(ce.last)(a))})),Es("cat",za),Es("category",za),Es("identity",Os),Es("linear",bs),Es("log",Is),Es("pow",xs),Es("time",ws),Es("timeCat",ms),Es("quantize",Ms),Es("quantile",Ss);var tl=n(136);function nl(e){return function(e){return Object(ce.map)(e,(function(e,t){return[0===t?"M":"L",e[0],e[1]]}))}(e)}function rl(e,t,n){for(var r=new bs({values:e}),i=new za({values:Object(ce.map)(e,(function(e,t){return t}))}),o=Object(ce.map)(e,(function(e,o){return[i.scale(o)*t,n-r.scale(e)*n]})),a=[],s=0;s<o.length;s++){var l=o[s],u=il({x:l[0],y:l[1],y0:n,size:5});a.push.apply(a,u)}return function(e,t){void 0===t&&(t=!0);var n=[],r=e[0];n.push(["M",r.x,r.y]);for(var i=1,o=e.length;i<o;i++)n.push(["L",e[i].x,e[i].y]);return t&&(n.push(["L",r.x,r.y]),n.push(["z"])),n}(a)}function il(e){var t,n,r,i,o=e.x,a=e.y,s=e.y0,l=e.size;Object(tl.isArray)(a)?(t=a[0],n=a[1]):(t=s,n=a),Object(tl.isArray)(o)?(r=o[0],i=o[1]):(r=o-l/2,i=o+l/2);var u=[{x:r,y:t},{x:r,y:n}];return u.push({x:i,y:n},{x:i,y:t}),u}var ol={stroke:"#C5C5C5",strokeOpacity:.85},al={fill:"#CACED4",opacity:.85},sl=function(){function e(e){var t=e.x,n=void 0===t?0:t,r=e.y,i=void 0===r?0:r,o=e.width,a=void 0===o?200:o,s=e.height,l=void 0===s?26:s,u=e.smooth,c=void 0===u||u,d=e.isArea,h=void 0!==d&&d,f=e.data,p=void 0===f?[]:f,g=e.lineStyle,v=e.areaStyle,m=e.group,y=e.interval,b=void 0===y?null:y;this.group=m,this.x=n,this.y=i,this.width=a,this.height=l,this.data=p,this.smooth=c,this.isArea=h,this.lineStyle=Object.assign({},ol,g),this.areaStyle=Object.assign({},al,v),this.intervalConfig=b,this.renderLine()}return e.prototype.renderLine=function(){var e=this,t=e.x,n=e.y,r=e.width,i=e.height,o=e.data,a=e.smooth,s=e.isArea,l=(e.backgroundStyle,e.lineStyle),u=e.areaStyle,c=this.group.addGroup({name:"trend-group"});if(o){var d=function(e,t,n,r){void 0===r&&(r=!0);var i=new bs({values:e}),o=new za({values:Object(ce.map)(e,(function(e,t){return t}))}),a=Object(ce.map)(e,(function(e,r){return[o.scale(r)*t,n-i.scale(e)*n]}));return r?function(e){if(e.length<=2)return nl(e);var t=[];Object(ce.each)(e,(function(e){Object(ce.isEqual)(e,t.slice(t.length-2))||t.push(e[0],e[1])}));var n=ht(t,!1),r=Object(ce.head)(e),i=r[0],o=r[1];return n.unshift(["M",i,o]),n}(a):nl(a)}(o,r,i,a);if(c.addShape("path",{attrs:Object(w.__assign)({path:d},l)}),s){var h=function(e,t,n,r){var i=Object(w.__spreadArrays)(e),o=function(e,t){var n=new bs({values:e}),r=Math.max(0,n.min);return t-n.scale(r)*t}(r,n);return i.push(["L",t,o]),i.push(["L",0,o]),i.push(["Z"]),i}(d,r,i,o);c.addShape("path",{attrs:Object(w.__assign)({path:h},u)})}}this.intervalConfig&&c.addShape("path",{attrs:Object(w.__assign)({path:rl(this.intervalConfig.data,r,i)},this.intervalConfig.style)}),c.move(t,n)},e.prototype.destory=function(){this.group.destroy()},e}(),ll={fill:"#1890ff",stroke:"#1890ff",type:"trend",radius:2,opacity:1,cursor:"ew-resize",highLightFill:"#0050b3"},ul={fill:"#fff",stroke:"#1890ff",radius:2,opacity:1,cursor:"ew-resize"},cl=function(){function e(e){var t=e.group,n=e.name,r=e.type,i=e.x,o=void 0===i?0:i,a=e.y,s=void 0===a?0:a,l=e.width,u=void 0===l?2:l,c=e.height,d=void 0===c?24:c,h=e.style,f=void 0===h?{}:h;this.group=t,this.name=n,this.handleType=r,this.x=o,this.y=s,this.width=u,this.height=d,"trend"===r?this.style=Object(w.__assign)(Object(w.__assign)({},ll),f):"simple"===r&&(this.style=Object(w.__assign)(Object(w.__assign)({},ul),f)),this.renderHandle()}return e.prototype.setX=function(e){this.setXY(e,void 0)},e.prototype.setY=function(e){this.setXY(void 0,e)},e.prototype.setXY=function(e,t){Object(ce.isNumber)(e)&&(this.x=e),Object(ce.isNumber)(t)&&(this.y=t),this.updateXY()},e.prototype.renderHandle=function(){var e=this.width,t=this.height,n=this.style,r=this.name,i=n.fill,o=n.stroke,a=n.radius,s=n.opacity,l=n.cursor;this.handleGroup=this.group.addGroup(),"trend"===this.handleType?(this.background=this.handleGroup.addShape("rect",{attrs:{x:0,y:0,width:e,height:t,fill:i,stroke:o,radius:a,opacity:s,cursor:l},name:r+"-handler"}),this.handleGroup.addShape("circle",{attrs:{x:e/2,y:0,r:2*e,fill:i,stroke:o,radius:a,opacity:s,cursor:l},name:r+"-handler"}),this.handleGroup.addShape("circle",{attrs:{x:e/2,y:t,r:2*e,fill:i,stroke:o,radius:a,opacity:s,cursor:l},name:r+"-handler"})):"simple"===this.handleType&&(this.handleGroup.addShape("circle",{attrs:{x:e/2,y:t/2,r:2*e,fill:i,stroke:o,radius:a,opacity:s,cursor:l},name:r+"-handler"}),this.handleGroup.addShape("circle",{attrs:{x:e/2,y:t/2,r:2*e,fill:i,stroke:o,radius:a,opacity:s,cursor:l},name:r+"-handler"})),this.updateXY(),"trend"===this.handleType&&this.bindEvents()},e.prototype.bindEvents=function(){var e=this,t=this.name;this.handleGroup.on(t+"-handler:mouseenter",(function(){var t=e.style.highLightFill;e.background.attr("fill",t)})),this.handleGroup.on(t+"-handler:mouseleave",(function(){var t=e.style.fill;e.background.attr("fill",t)}))},e.prototype.show=function(){this.handleGroup.show()},e.prototype.hide=function(){this.handleGroup.hide()},e.prototype.updateXY=function(){this.handleGroup.setMatrix([1,0,0,0,1,0,this.x,this.y,1])},e}(),dl=function(){function e(e){this.config=Object(ce.deepMix)({},e),this.init()}return e.prototype.update=function(e){this.config=Object(ce.deepMix)({},this.config,e),this.updateElement(),this.renderMarker()},e.prototype.init=function(){this.initElement(),this.renderMarker()},e.prototype.initElement=function(){var e=this.config,t=e.group,n=e.style;this.circle=t.addShape("circle",{attrs:Object(w.__assign)({x:this.config.x,y:this.config.y,r:this.config.r},n)}),this.startMarker=t.addShape("path",{attrs:{path:this.getStartMarkerPath(),fill:"#ccc"},name:"playPauseBtn"}),this.pauseGroupMarker=t.addGroup();var r=1/4*this.config.r,i=.5*this.config.r*Math.sqrt(3);this.pauseGroupMarker.addShape("rect",{attrs:{x:this.config.x-.375*this.config.r,y:this.config.y-i/2,width:2.5*r,height:i,fill:"#fff"},name:"playPauseBtn"}),this.pauseLeftMarker=this.pauseGroupMarker.addShape("rect",{attrs:{x:this.config.x-.375*this.config.r,y:this.config.y-i/2,width:r,height:i,fill:"#ccc"},name:"playPauseBtn"}),this.pauseRightMarker=this.pauseGroupMarker.addShape("rect",{attrs:{x:this.config.x+1/8*this.config.r,y:this.config.y-i/2,width:r,height:i,fill:"#ccc"},name:"playPauseBtn"})},e.prototype.updateElement=function(){this.circle.attr("x",this.config.x),this.circle.attr("y",this.config.y),this.circle.attr("r",this.config.r),this.startMarker.attr("path",this.getStartMarkerPath());var e=1/4*this.config.r,t=.5*this.config.r*Math.sqrt(3);this.pauseLeftMarker.attr("x",this.config.x-.375*this.config.r),this.pauseLeftMarker.attr("y",this.config.y-t/2),this.pauseLeftMarker.attr("width",e),this.pauseLeftMarker.attr("height",t),this.pauseRightMarker.attr("x",this.config.x+1/8*this.config.r),this.pauseRightMarker.attr("y",this.config.y-t/2),this.pauseRightMarker.attr("width",e),this.pauseRightMarker.attr("height",t)},e.prototype.renderMarker=function(){this.config.isPlay?(this.startMarker.hide(),this.pauseGroupMarker.show()):(this.startMarker.show(),this.pauseGroupMarker.hide())},e.prototype.getStartMarkerPath=function(){var e=.5*this.config.r*Math.sqrt(3);return[["M",this.config.x-e/Math.sqrt(3)/2,this.config.y-e/2],["L",this.config.x+e/Math.sqrt(3),this.config.y],["L",this.config.x-e/Math.sqrt(3)/2,this.config.y+e/2]]},e}(),hl={fill:"#607889",stroke:"#607889"},fl={fill:"#fff"},pl={fill:"green"},gl={speed:2,loop:!1,fill:"#fff",stroke:"#fff",hiddleToggle:!1,preBtnStyle:{fill:"#ccc",stroke:"#ccc"},nextBtnStyle:{fill:"#ccc",stroke:"#ccc"},playBtnStyle:{fill:"#fff",stroke:"#ccc"}},vl=function(){function e(e){this.controllerCfg=Object(ce.deepMix)({},gl,e),this.group=e.group,this.speedAxisY=[],this.currentSpeed=this.controllerCfg.speed,this.currentType="range",this.init()}return e.prototype.init=function(){this.renderPlayButton()},e.prototype.getNextMarkerPath=function(e,t,n){return[["M",e,t-n],["L",e+n,t],["L",e,t+n]]},e.prototype.getPreMarkerPath=function(e,t,n){return[["M",e,t-n],["L",e-n,t],["L",e,t+n]]},e.prototype.renderPlayButton=function(){var e=this.controllerCfg,t=e.width,n=e.height,r=e.x,i=e.y,o=e.hiddleToggle,a=e.fill,s=void 0===a?"#ccc":a,l=e.stroke,u=void 0===l?"green":l,c=e.playBtnStyle,d=void 0===c?hl:c,h=e.preBtnStyle,f=void 0===h?fl:h,p=e.nextBtnStyle,g=void 0===p?pl:p,v=n/2-5,m=i+10,y=this.group.addShape("rect",{attrs:{x:r,y:m,width:t,height:n,stroke:u,fill:s},name:"container-rect"});this.playButton?this.playButton.update({x:t/2,y:m,r:v}):this.playButton=new dl({group:this.group,x:t/2,y:m+v+5,r:v,isPlay:this.isPlay,style:d}),this.group.addShape("path",{attrs:Object(w.__assign)({path:this.getPreMarkerPath(t/2-5*v,m+v+5,.5*v)},f),name:"preStepBtn"}),this.group.addShape("path",{attrs:Object(w.__assign)({path:this.getPreMarkerPath(t/2-4.5*v,m+v+5,.5*v)},f),name:"preStepBtn"}),this.group.addShape("path",{attrs:Object(w.__assign)({path:this.getNextMarkerPath(t/2+5*v,m+v+5,.5*v)},g),name:"nextStepBtn"}),this.group.addShape("path",{attrs:Object(w.__assign)({path:this.getNextMarkerPath(t/2+4.5*v,m+v+5,.5*v)},g),name:"nextStepBtn"}),y.toBack(),this.renderSpeedBtn(),o||this.renderToggleTime(),this.bindEvent()},e.prototype.renderSpeedBtn=function(){var e=this.controllerCfg,t=e.y,n=e.width,r=e.hiddleToggle,i=this.group.addGroup({name:"speed-group"});this.speedGroup=i;for(var o=1,a=[],s=9,l=0;l<5;l++){var u=t+15+l*(l+1)+o;i.addShape("rect",{attrs:{x:n-(r?50:110),y:u,width:15,height:2,fill:"#ccc"},speed:s,name:"speed-rect"}),this.speedAxisY.push(u),a.push(s),s-=2,o++}for(l=0;l<4;l++)i.addShape("rect",{attrs:{x:n-(r?50:110),y:this.speedAxisY[l]+2,width:15,height:2*l+1,fill:"#fff",opacity:.3},speed:a[l]-1,name:"speed-rect"});this.speedText=i.addShape("text",{attrs:{x:n-(r?50:110)+20,y:this.speedAxisY[1]+15,text:"1.0X",fill:"#ccc"}}),this.speedPoint=i.addShape("path",{attrs:{path:this.getPath(n-(r?50:110),this.speedAxisY[4]),fill:"#ccc"}})},e.prototype.getPath=function(e,t){return[["M",e,t],["L",e-12,t-6],["L",e-12,t+6],["Z"]]},e.prototype.renderToggleTime=function(){var e=this.controllerCfg.width;this.toggleGroup=this.group.addGroup({name:"toggle-group"}),this.toggleGroup.addShape("rect",{attrs:{width:14,height:14,x:e-50,y:this.speedAxisY[1],fill:"#fff",stroke:"#ccc",lineWidth:2,radius:3},isChecked:!1,name:"toggle-model"}),this.checkedIcon=this.toggleGroup.addShape("path",{attrs:{path:[["M",e-50+3,this.speedAxisY[1]+6],["L",e-50+7,this.speedAxisY[1]+10],["L",e-50+12,this.speedAxisY[1]+4]],stroke:"green",lineWidth:3},capture:!1}),this.checkedIcon.hide(),this.checkedText=this.toggleGroup.addShape("text",{attrs:{text:"单一时间",x:e-50+20,y:this.speedAxisY[1]+15,fill:"#ccc"}})},e.prototype.bindEvent=function(){var e=this,t=this.controllerCfg,n=t.width,r=t.hiddleToggle;this.speedGroup.on("speed-rect:click",(function(t){e.speedPoint.attr("path",e.getPath(n-(r?50:110),t.y)),e.currentSpeed=t.target.get("speed"),e.speedText.attr("text",e.currentSpeed+".0X"),e.group.emit("timebarConfigChanged",{speed:e.currentSpeed,type:e.currentType})})),this.toggleGroup&&this.toggleGroup.on("toggle-model:click",(function(t){var n=t.target.get("isChecked");n?(e.checkedIcon.hide(),e.checkedText.attr("text","单一时间"),e.currentType="range"):(e.checkedIcon.show(),e.checkedText.attr("text","时间范围"),e.currentType="single"),t.target.set("isChecked",!n),e.group.emit("timebarConfigChanged",{type:e.currentType,speed:e.currentSpeed})}))},e.prototype.destroy=function(){this.speedGroup.off("speed-rect:click"),this.toggleGroup&&(this.toggleGroup.off("toggle-model:click"),this.toggleGroup.destroy()),this.speedGroup.destroy()},e}(),ml={fill:"#416180",opacity:.05},yl={fill:"#416180",opacity:.15,radius:5},bl={fill:"#5B8FF9",opacity:.3,cursor:"move"},Cl={width:2,height:24},Al={textBaseline:"middle",fill:"#000",opacity:.45},_l=function(){function e(e){var t=this;this.prevX=0,this.onMouseDown=function(e){return function(n){t.currentHandler=e;var r=n.originalEvent;r.stopPropagation(),r.preventDefault(),t.prevX=Object(ce.get)(r,"touches.0.pageX",r.pageX);var i=t.canvas.get("container");i.addEventListener("mousemove",t.onMouseMove),i.addEventListener("mouseup",t.onMouseUp),i.addEventListener("mouseleave",t.onMouseUp),i.addEventListener("touchmove",t.onMouseMove),i.addEventListener("touchend",t.onMouseUp),i.addEventListener("touchcancel",t.onMouseUp)}},this.onMouseMove=function(e){e.stopPropagation(),e.preventDefault();var n=Object(ce.get)(e,"touches.0.pageX",e.pageX),r=n-t.prevX,i=t.adjustOffsetRange(r/t.width);t.updateStartEnd(i),t.updateUI(),t.prevX=n},this.onMouseUp=function(){t.currentHandler&&(t.currentHandler=void 0);var e=t.canvas.get("container");e&&(e.removeEventListener("mousemove",t.onMouseMove),e.removeEventListener("mouseup",t.onMouseUp),e.removeEventListener("mouseleave",t.onMouseUp),e.removeEventListener("touchmove",t.onMouseMove),e.removeEventListener("touchend",t.onMouseUp),e.removeEventListener("touchcancel",t.onMouseUp))};var n=e.x,r=void 0===n?0:n,i=e.y,o=void 0===i?0:i,a=e.width,s=void 0===a?100:a,l=e.height,u=e.padding,c=void 0===u?10:u,d=e.trendCfg,h=e.controllerCfg,f=void 0===h?{speed:1}:h,p=e.backgroundStyle,g=void 0===p?{}:p,v=e.foregroundStyle,m=void 0===v?{}:v,y=e.handlerStyle,b=void 0===y?{}:y,C=e.textStyle,A=void 0===C?{}:C,_=e.start,I=void 0===_?0:_,x=e.end,M=void 0===x?1:x,S=e.minText,T=void 0===S?"":S,E=e.maxText,O=void 0===E?"":E,k=e.group,L=e.graph,D=e.canvas,P=e.ticks,N=e.type;this.graph=L,this.canvas=D,this.group=k,this.timeBarType=N,this.x=r,this.y=o,this.width=s,this.height=l,this.padding=c,this.ticks=P,this.trendCfg=d,this.controllerCfg=f,this.currentSpeed=f.speed,"trend"===N?this.backgroundStyle=Object(w.__assign)(Object(w.__assign)({},ml),g):"simple"===N&&(this.backgroundStyle=Object(w.__assign)(Object(w.__assign)({},yl),g)),this.foregroundStyle=Object(w.__assign)(Object(w.__assign)({},bl),m),this.handlerStyle=Object(w.__assign)(Object(w.__assign)({},Cl),b),this.textStyle=Object(w.__assign)(Object(w.__assign)({},Al),A),this.currentMode="range",this.start=I,this.end=M,this.minText=T,this.maxText=O,this.renderSlider()}return e.prototype.update=function(e){var t=e.x,n=e.y,r=e.width,i=e.height,o=e.minText,a=e.maxText,s=e.start,l=e.end;this.start=Math.min(1,Math.max(s,0)),this.end=Math.min(1,Math.max(l,0)),Object(ce.assign)(this,{x:t,y:n,width:r,height:i,minText:o,maxText:a}),this.updateUI()},e.prototype.setText=function(e,t){this.minTextShape.attr("text",e),this.maxTextShape.attr("text",t)},e.prototype.renderSlider=function(){var e=this,t=this.width,n=this.height,r=this.timeBarType;if("trend"===r&&Object(ce.size)(Object(ce.get)(this.trendCfg,"data"))){var i=new sl(Object(w.__assign)(Object(w.__assign)({x:this.x,y:this.y,width:t,height:n},this.trendCfg),{group:this.group}));this.trendComponent=i}var o=this.group.addGroup({name:"slider-group"});o.addShape("rect",{attrs:Object(w.__assign)({x:0,y:0,width:t,height:n},this.backgroundStyle)});var a=this.group.addGroup();"trend"===r?(this.minTextShape=a.addShape("text",{attrs:Object(w.__assign)({x:0,y:n/2+this.y,textAlign:"right",text:this.minText,silent:!1},this.textStyle),capture:!1}),this.maxTextShape=a.addShape("text",{attrs:Object(w.__assign)({y:n/2+this.y,textAlign:"left",text:this.maxText,silent:!1},this.textStyle),capture:!1})):(this.minTextShape=a.addShape("text",{attrs:Object(w.__assign)({x:0,y:this.y-10,textAlign:"center",text:this.minText,silent:!1},this.textStyle),capture:!1}),this.maxTextShape=a.addShape("text",{attrs:Object(w.__assign)({y:this.y-10,textAlign:"center",text:this.maxText,silent:!1},this.textStyle),capture:!1})),this.foregroundShape=this.group.addGroup().addShape("rect",{attrs:Object(w.__assign)({x:0,y:this.y,height:n},this.foregroundStyle)});var s=Object(ce.get)(this.handlerStyle,"width",2),l=Object(ce.get)(this.handlerStyle,"height",24),u=this.group.addGroup({name:"minHandlerShape"});this.minHandlerShape=new cl(Object(w.__assign)({name:"minHandlerShape",group:u,type:r,x:this.x,y:this.y,width:s,height:l,cursor:"ew-resize"},this.handlerStyle));var c=this.group.addGroup({name:"maxHandlerShape"});this.maxHandlerShape=new cl(Object(w.__assign)({name:"maxHandlerShape",group:c,type:r,x:this.x,y:this.y,width:s,height:l,cursor:"ew-resize"},this.handlerStyle));var d=this.ticks,h=t/(d.length-1);this.tickPosList=[],this.textList&&this.textList.length&&this.textList.forEach((function(e){e.destroy()}));var f=-1/0;this.textList=d.map((function(t,r){e.tickPosList.push(e.x+r*h);var i=e.group.addShape("text",{attrs:{x:e.x+r*h,y:e.y+n+5,text:t,textAlign:"center",textBaseline:"top",fill:"#607889",opacity:.35}}),o=e.group.addShape("line",{attrs:{x1:e.x+r*h,y1:e.y+n+2,x2:e.x+r*h,y2:e.y+n+6,lineWidth:1,stroke:"#ccc"}}),a=i.getBBox();return a.minX>f?(i.show(),o.show(),f=a.minX+a.width+10):(i.hide(),o.hide()),i})),this.controllerBtnGroup=new vl({group:this.group,x:this.x,y:this.y+n+25,width:t,height:40}),this.updateUI(),o.move(this.x,this.y),this.bindEvents()},e.prototype.bindEvents=function(){var e=this,t=this.group.find((function(e){return"minHandlerShape"===e.get("name")}));t&&(t.on("minHandlerShape-handler:mousedown",this.onMouseDown(this.minHandlerShape)),t.on("minHandlerShape-handler:touchstart",this.onMouseDown(this.minHandlerShape)));var n=this.group.find((function(e){return"maxHandlerShape"===e.get("name")}));n&&(n.on("maxHandlerShape-handler:mousedown",this.onMouseDown(this.maxHandlerShape)),n.on("maxHandlerShape-handler:touchstart",this.onMouseDown(this.maxHandlerShape))),this.foregroundShape.on("mousedown",this.onMouseDown(this.foregroundShape)),this.foregroundShape.on("touchstart",this.onMouseDown(this.foregroundShape)),this.group.on("playPauseBtn:click",(function(){e.isPlay=!e.isPlay,e.currentHandler=e.maxHandlerShape,e.changePlayStatus()})),this.group.on("nextStepBtn:click",(function(){e.currentHandler=e.maxHandlerShape,e.updateStartEnd(.01),e.updateUI()})),this.group.on("preStepBtn:click",(function(){e.currentHandler=e.maxHandlerShape,e.updateStartEnd(-.01),e.updateUI()})),this.group.on("timebarConfigChanged",(function(t){var n=t.type,r=t.speed;e.currentSpeed=r,e.currentMode=n,"single"===n?(e.minHandlerShape.hide(),e.foregroundShape.hide(),e.minTextShape.hide()):"range"===n&&(e.minHandlerShape.show(),e.foregroundShape.show(),e.minTextShape.show())}))},e.prototype.adjustTickIndex=function(e){for(var t=0;t<this.tickPosList.length-1;t++)if(this.tickPosList[t]<=e&&e<=this.tickPosList[t+1])return Math.abs(this.tickPosList[t]-e)<Math.abs(e-this.tickPosList[t+1])?t:t+1},e.prototype.adjustOffsetRange=function(e){switch(this.currentHandler){case this.minHandlerShape:var t=0-this.start,n=1-this.start;return Math.min(n,Math.max(t,e));case this.maxHandlerShape:return t=0-this.end,n=1-this.end,Math.min(n,Math.max(t,e));case this.foregroundShape:return t=0-this.start,n=1-this.end,Math.min(n,Math.max(t,e));default:return 0}},e.prototype.updateStartEnd=function(e){switch(this.currentHandler){case this.minHandlerShape:this.maxText=this.maxTextShape.attr("text"),this.start+=e,this.minText=this.ticks[this.adjustTickIndex(this.start*this.width)];break;case this.maxHandlerShape:this.minText=this.minTextShape.attr("text"),this.end+=e,this.maxText=this.ticks[this.adjustTickIndex(this.end*this.width)];break;case this.foregroundShape:this.start+=e,this.end+=e,this.minText=this.ticks[this.adjustTickIndex(this.start*this.width)],this.maxText=this.ticks[this.adjustTickIndex(this.end*this.width)]}},e.prototype.updateUI=function(){var e=this;this.start<0&&(this.start=0),this.end>1&&(this.end=1);var t=this.start*this.width,n=this.end*this.width;this.foregroundShape.attr("x",t),this.foregroundShape.attr("width",n-t);var r=Object(ce.get)(this.handlerStyle,"width",2);this.setText(this.minText,this.maxText);var i=this.dodgeText([t,n]),o=i[0],a=i[1];this.minHandlerShape.setX(t-r/2),Object(ce.each)(o,(function(t,n){return e.minTextShape.attr(n,t)})),this.maxHandlerShape.setX(n-r/2),Object(ce.each)(a,(function(t,n){return e.maxTextShape.attr(n,t)})),"range"===this.currentMode?this.graph.emit("valueChange",{value:[this.start,this.end].sort()}):"single"===this.currentMode&&this.graph.emit("valueChange",{value:[this.end,this.end]})},e.prototype.dodgeText=function(e){var t,n,r=Object(ce.get)(this.handlerStyle,"width",2),i=this.minTextShape,o=this.maxTextShape,a=e[0],s=e[1],l=!1;a>s&&(a=(t=[s,a])[0],s=t[1],i=(n=[o,i])[0],o=n[1],l=!0);var u=i.getBBox(),c=o.getBBox(),d=null,h=null;return"trend"===this.timeBarType?(d=u.width>a-2?{x:a+r/2+2,textAlign:"left"}:{x:a-r/2-2,textAlign:"right"},h=c.width>this.width-s-2?{x:s-r/2-2,textAlign:"right"}:{x:s+r/2+2,textAlign:"left"}):"simple"===this.timeBarType&&(d=u.width>a-2?{x:a+r/2+2,textAlign:"center"}:{x:a-r/2-2,textAlign:"center"},h=c.width>this.width-s-2?{x:s-r/2-2,textAlign:"center"}:{x:s+r/2+2,textAlign:"center"}),l?[h,d]:[d,h]},e.prototype.startPlay=function(){var e=this;return window.requestAnimationFrame((function(){var t=e,n=t.ticks,r=t.width,i=e.currentSpeed,o=r/n.length/(1e3*(10-i)/60),a=e.adjustOffsetRange(o/e.width);e.updateStartEnd(a),e.updateUI(),e.isPlay&&(e.playHandler=e.startPlay())}))},e.prototype.changePlayStatus=function(e){void 0===e&&(e=!0),this.controllerBtnGroup.playButton.update({isPlay:this.isPlay}),this.isPlay?(this.playHandler=this.startPlay(),this.graph.emit("timelinestart",null)):this.playHandler&&(window.cancelAnimationFrame(this.playHandler),e&&this.graph.emit("timelineend",null))},e.prototype.destory=function(){this.graph.off("valueChange");var e=this.group,t=e.find((function(e){return"minHandlerShape"===e.get("name")}));t&&(t.off("minHandlerShape-handler:mousedown"),t.off("minHandlerShape-handler:touchstart"),t.destroy());var n=e.find((function(e){return"maxHandlerShape"===e.get("name")}));n&&(n.off("maxHandlerShape-handler:mousedown"),n.off("maxHandlerShape-handler:touchstart"),n.destroy()),this.foregroundShape.off("mousedown"),this.foregroundShape.off("touchstart"),this.foregroundShape.destroy(),e.off("playPauseBtn:click"),e.off("nextStepBtn:click"),e.off("preStepBtn:click"),e.off("timebarConfigChanged"),e.destroy(),this.trendComponent&&this.trendComponent.destory()},e}(),Il=function(){function e(e){var t=e.x,n=void 0===t?0:t,r=e.y,i=void 0===r?0:r,o=e.container,a=e.text,s=e.padding,l=void 0===s?[4,4,4,4]:s,u=e.className,c=void 0===u?"g6-component-timebar-tooltip":u,d=e.backgroundColor,h=void 0===d?"#000":d,f=e.textColor,p=void 0===f?"#fff":f,g=e.opacity,v=void 0===g?.8:g,m=e.fontSize,y=void 0===m?12:m;this.container=o,this.className=c,this.backgroundColor=h,this.textColor=p,this.x=n,this.y=i,this.text=a,this.padding=l,this.opacity=v,this.fontSize=y,this.render()}return e.prototype.render=function(){var e=this.className,t=(this.x,this.y,this.backgroundColor),n=this.textColor,r=this.text,i=this.padding,o=this.opacity,a=this.fontSize,s=this.container,l=ye()("<div class='"+e+"' style=\"position: absolute; width: fit-content; height: fit-content; opacity: "+o+'"></div>');Object(ce.isString)(s)&&(s=document.getElementById(s)),s.appendChild(l),this.parentHeight=s.offsetHeight,this.parentWidth=s.offsetWidth,ve()(l,{visibility:"hidden",top:0,left:0});var u=ye()("\n      <div style='position: absolute; white-space:nowrap; background-color: "+t+"; font-size: "+a+"px; border-radius: 4px; width: fit-content; height: fit-content; color: "+n+"; padding: "+i[0]+"px "+i[1]+"px "+i[2]+"px "+i[3]+"px'></div>");u.innerHTML=r,l.appendChild(u),this.backgroundDOM=u;var c=ye()("<div style='position: absolute; width: 0px; height: 0px; border-left: 5px solid transparent; border-right: 5px solid transparent; border-top: 10px solid "+t+"'></div>");l.appendChild(c),this.arrowDOM=c,this.container=l},e.prototype.show=function(e){var t=e.text,n=e.x;e.y,e.clientX,e.clientY,this.backgroundDOM.innerHTML=t;var r=this.backgroundDOM.offsetWidth,i=this.backgroundDOM.offsetHeight,o=this.arrowDOM.offsetWidth,a=this.arrowDOM.offsetHeight;ve()(this.container,{top:-i-a+"px",left:n+"px",visibility:"visible"}),ve()(this.backgroundDOM,{marginLeft:-r/2+"px"}),ve()(this.arrowDOM,{marginLeft:-o/2+"px",top:i+"px"});var s=n-r/2,l=n+r/2;s<0?ve()(this.backgroundDOM,{marginLeft:-r/2-s+"px"}):l>this.parentWidth&&ve()(this.backgroundDOM,{marginLeft:-r/2-l+this.parentWidth+12+"px"})},e.prototype.hide=function(){ve()(this.container,{top:0,left:0,visibility:"hidden"})},e}(),xl={fill:"#5B8FF9"},wl={fill:"#e6e8e9"},Ml=function(){function e(e){this.frameCount=0;var t=e.graph,n=e.canvas,r=e.group,i=e.width,o=e.height,a=e.padding,s=e.data,l=e.start,u=e.end,c=e.x,d=void 0===c?0:c,h=e.y,f=void 0===h?0:h,p=e.tickLabelFormatter,g=e.selectedTickStyle,v=void 0===g?xl:g,m=e.unselectedTickStyle,y=void 0===m?wl:m,b=e.tooltipBackgroundColor,C=e.tooltipFomatter;this.graph=t,this.group=r,this.sliceGroup=r.addGroup({name:"slice-group"}),this.canvas=n,this.width=i,this.height=o,this.padding=a,this.data=s,this.start=l,this.end=u,this.tickLabelFormatter=p,this.selectedTickStyle=v,this.unselectedTickStyle=y,this.x=d,this.y=f,this.tooltipBackgroundColor=b,this.tooltipFomatter=C,this.renderSlices(),this.initEvent()}return e.prototype.renderSlices=function(){var e=this,t=e.width,n=e.height,r=e.padding,i=e.data,o=e.start,a=e.end,s=e.tickLabelFormatter,l=e.selectedTickStyle,u=e.unselectedTickStyle,c=t-2*r,d=n-(3*r+4+10)-2*r,h=i.length,f=(c-2*(h-1))/h;this.tickWidth=f;var p=this.sliceGroup,g=[],v=[],m=Math.round(h*o),y=Math.round(h*a);this.startTickRectId=m,this.endTickRectId=y,i.forEach((function(e,n){var i=n>=m&&n<=y?l:u,o=p.addShape("rect",{attrs:Object(w.__assign)({x:r+n*(f+2),y:r,width:f,height:d},i),draggable:!0,name:"tick-rect-"+n}),a=p.addShape("rect",{attrs:{x:r+n*f+2*(2*n-1)/2,y:r,width:0===n||n===h-1?f+1:f+2,height:d,fill:"#fff",opacity:0},draggable:!0,name:"pick-rect-"+n});a.toFront();var c,b=o.getBBox(),C=(b.minX+b.maxX)/2;if(g.push({rect:o,pickRect:a,value:e.date,x:C,y:b.minY}),s?(c=s(e),!Object(ce.isString)(c)&&c&&(c=e.date)):n%Math.round(h/10)==0&&(c=e.date),c){v.push(c);var A=b.maxY+2*r;p.addShape("line",{attrs:{stroke:"#BFBFBF",x1:C,y1:A,x2:C,y2:A+4}});var _=A+4+r,I=p.addShape("text",{attrs:{fill:"#8c8c8c",stroke:"#fff",lineWidth:1,x:C,y:_,textAlign:"center",text:c,textBaseline:"top",fontSize:10},capture:!1}),x=I.getBBox();x.maxX>t?I.attr("textAlign","right"):x.minX<0&&I.attr("textAlign","left")}})),this.tickRects=g;var b=this.group;this.currentSpeed=1,this.controllerBtnGroup=new vl({group:b,x:this.x,y:this.y+n+5,width:t,height:40,hiddleToggle:!0,speed:this.currentSpeed})},e.prototype.initEvent=function(){var e=this,t=this.sliceGroup;t.on("click",(function(t){var n=t.target;if("rect"===n.get("type")&&n.get("name")){var r=parseInt(n.get("name").split("-")[2],10);if(!isNaN(r)){var i=e.tickRects,o=e.unselectedTickStyle;i.forEach((function(e){e.rect.attr(o)}));var a=e.selectedTickStyle;i[r].rect.attr(a),e.startTickRectId=r,e.endTickRectId=r;var s=r/i.length;e.graph.emit("valueChange",{value:[s,s]})}}})),t.on("dragstart",(function(t){var n=e.tickRects,r=e.unselectedTickStyle;n.forEach((function(e){e.rect.attr(r)}));var i=t.target,o=parseInt(i.get("name").split("-")[2],10),a=e.selectedTickStyle;n[o].rect.attr(a),e.startTickRectId=o;var s=o/n.length;e.graph.emit("valueChange",{value:[s,s]}),e.dragging=!0})),t.on("dragover",(function(t){if(e.dragging&&"rect"===t.target.get("type")){for(var n=parseInt(t.target.get("name").split("-")[2],10),r=e.startTickRectId,i=e.tickRects,o=e.selectedTickStyle,a=e.unselectedTickStyle,s=0;s<i.length;s++){var l=s>=r&&s<=n?o:a;i[s].rect.attr(l)}var u=i.length;e.endTickRectId=n;var c=r/u,d=n/u;e.graph.emit("valueChange",{value:[c,d]})}})),t.on("drop",(function(t){if(e.dragging&&(e.dragging=!1,"rect"===t.target.get("type"))){var n=e.startTickRectId,r=parseInt(t.target.get("name").split("-")[2],10);if(!(r<n)){var i=e.selectedTickStyle,o=e.tickRects;o[r].rect.attr(i),e.endTickRectId=r;var a=o.length,s=n/a,l=r/a;e.graph.emit("valueChange",{value:[s,l]})}}}));var n=this.tooltipBackgroundColor,r=this.tooltipFomatter,i=this.canvas,o=new Il({container:i.get("container"),backgroundColor:n}),a=this.tickRects;a.forEach((function(e){var t=e.pickRect;t.on("mouseenter",(function(e){var t=e.target;if("rect"===t.get("type")){var n=parseInt(t.get("name").split("-")[2],10),s=i.getClientByPoint(a[n].x,a[n].y);o.show({x:a[n].x,y:a[n].y,clientX:s.x,clientY:s.y,text:r?r(a[n].value):a[n].value})}})),t.on("mouseleave",(function(e){o.hide()}))}));var s=this.group;s.on("playPauseBtn:click",(function(){e.isPlay=!e.isPlay,e.changePlayStatus()})),s.on("nextStepBtn:click",(function(){e.updateStartEnd(1)})),s.on("preStepBtn:click",(function(){e.updateStartEnd(-1)})),s.on("timebarConfigChanged",(function(t){t.type;var n=t.speed;e.currentSpeed=n}))},e.prototype.changePlayStatus=function(e){void 0===e&&(e=!0),this.controllerBtnGroup.playButton.update({isPlay:this.isPlay}),this.isPlay?(this.playHandler=this.startPlay(),this.graph.emit("timelinestart",null)):this.playHandler&&(window.cancelAnimationFrame(this.playHandler),e&&this.graph.emit("timelineend",null))},e.prototype.startPlay=function(){var e=this;return window.requestAnimationFrame((function(){var t=e.currentSpeed;e.frameCount%(60/t)==0&&(e.frameCount=0,e.updateStartEnd(1)),e.frameCount++,e.isPlay&&(e.playHandler=e.startPlay())}))},e.prototype.updateStartEnd=function(e){var t=this.tickRects,n=t.length,r=this.unselectedTickStyle,i=this.selectedTickStyle,o=this.endTickRectId;if(e>0?this.endTickRectId++:(t[this.endTickRectId].rect.attr(r),this.endTickRectId--),o!==this.startTickRectId)this.endTickRectId<this.startTickRectId&&(this.startTickRectId=this.endTickRectId);else{for(var a=this.startTickRectId;a<=this.endTickRectId-1;a++)t[a].rect.attr(r);this.startTickRectId=this.endTickRectId}if(t[this.endTickRectId]){t[this.endTickRectId].rect.attr(i);var s=this.startTickRectId/n,l=this.endTickRectId/n;this.graph.emit("valueChange",{value:[s,l]})}},e.prototype.destory=function(){this.graph.off("valueChange");var e=this.sliceGroup;e.off("click"),e.off("dragstart"),e.off("dragover"),e.off("drop"),this.tickRects.forEach((function(e){var t=e.pickRect;t.off("mouseenter"),t.off("mouseleave")})),this.tickRects.length=0,e.off("playPauseBtn:click"),e.off("nextStepBtn:click"),e.off("preStepBtn:click"),e.off("timebarConfigChanged"),this.sliceGroup.destroy()},e}(),Sl=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(w.__extends)(t,e),t.prototype.getDefaultCfgs=function(){return{container:null,className:"g6-component-timebar",padding:10,type:"trend",trend:{data:[],isArea:!1,smooth:!0},controllerCfg:{speed:2,loop:!1},slider:{start:.1,end:.9,minText:"min",maxText:"max"},tick:{start:.1,end:.9,data:[]}}},t.prototype.initContainer=function(){var e,t,n=this.get("graph"),r=this._cfgs,i=r.width,o=r.height,a=this.get("className")||"g6-component-timebar",s=this.get("container"),l=this.get("graph").get("container");s?e=s:(e=ye()("<div class='"+a+"'></div>"),ve()(e,{position:"relative"})),l.appendChild(e),this.set("timeBarContainer",e),t="SVG"===n.get("renderer")?new Rt.a({container:e,width:i,height:o}):new Nt.a({container:e,width:i,height:o}),this.set("canvas",t)},t.prototype.init=function(){this.initContainer();var e=this.get("canvas").addGroup({name:"timebar-group"});this.set("timeBarGroup",e),this.renderTrend(),this.initEvent()},t.prototype.renderTrend=function(){var e=this._cfgs,t=e.width,n=e.x,r=e.y,i=e.padding,o=e.type,a=e.trend,s=e.slider,l=e.controllerCfg,u=a.data,c=Object(w.__rest)(a,["data"]),d=t-2*i,h="trend"===o?26:8,f=this.get("graph"),p=this.get("timeBarGroup"),g=this.get("canvas"),v=null;if("trend"===o||"simple"===o)v=new _l(Object(w.__assign)(Object(w.__assign)({graph:f,canvas:g,group:p,type:o,x:n+i,y:"trend"===o?r+i:r+i+15,width:d,height:h,padding:i,trendCfg:Object(w.__assign)(Object(w.__assign)({},c),{data:u.map((function(e){return e.value}))})},s),{ticks:u.map((function(e){return e.date})),handlerStyle:Object(w.__assign)(Object(w.__assign)({},s.handlerStyle),{height:s.height||h}),controllerCfg:l}));else if("tick"===o){var m=this._cfgs.tick;v=new Ml(Object(w.__assign)({graph:f,canvas:g,group:p,x:n+i,y:r+i},m))}this.set("timebar",v)},t.prototype.filterData=function(e){var t=e.value,n=null,r=this._cfgs.type;if("trend"===r||"simple"===r?n=this._cfgs.trend.data:"tick"===r&&(n=this._cfgs.tick.data),n&&0!==n.length){var i=this.get("rangeChange"),o=this.get("graph"),a=Math.round(n.length*t[0]),s=Math.round(n.length*t[1]);s=s>=n.length?n.length-1:s;var l=n[a].date,u=n[s].date;if("tick"!==r&&this.get("timebar").setText(l,u),i)i(o,l,u);else{var c=o.save();(!this.cacheGraphData||this.cacheGraphData.nodes&&0===this.cacheGraphData.nodes.length)&&(this.cacheGraphData=c);var d=this.cacheGraphData.nodes.filter((function(e){return e.date>=l&&e.date<=u})),h=d.map((function(e){return e.id})),f=this.cacheGraphData.edges.filter((function(e){return h.includes(e.source)&&h.includes(e.target)}));o.changeData({nodes:d,edges:f})}}else console.warn("请配置 TimeBar 组件的数据")},t.prototype.initEvent=function(){var e=this,t=0,n=0,r=this._cfgs.type;r&&"trend"!==r&&"simple"!==r?"tick"===r&&(t=this._cfgs.tick.start,n=this._cfgs.tick.end):(t=this._cfgs.slider.start,n=this._cfgs.slider.end);var i=this.get("graph");i.on("afterrender",(function(){e.filterData({value:[t,n]})})),i.on("valueChange",(function(t){e.filterData(t)}))},t.prototype.destroy=function(){var t=this.get("timebar");t&&t.destory&&t.destory(),e.prototype.destroy.call(this);var n=this.get("timeBarContainer");if(n){var r=this.get("container");r||(r=this.get("graph").get("container")),r.removeChild(n)}},t}(Ca),Tl=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(w.__extends)(t,e),t.prototype.getDefaultCfgs=function(){return{container:null,className:"g6-minimap",viewportClassName:"g6-minimap-viewport",width:200,delegateStyle:{fill:"#40a9ff",stroke:"#096dd9"},refresh:!0}},t.prototype.getEvents=function(){return{beforepaint:"updateViewport",beforeanimate:"disableRefresh",afteranimate:"enableRefresh",viewportchange:"disableOneRefresh"}},t.prototype.disableRefresh=function(){this.set("refresh",!1)},t.prototype.enableRefresh=function(){this.set("refresh",!0),this.updateCanvas()},t.prototype.disableOneRefresh=function(){this.set("viewportChange",!0)},t.prototype.initViewport=function(){var e=this,t=this._cfgs,n=t.graph;if(!this.destroyed){var r=this.get("container"),i=ye()("<div class="+t.viewportClassName+"\n      style='position:absolute;\n        left:0;\n        top:0;\n        box-sizing:border-box;\n        border: 2px solid #1980ff'>\n      </div>"),o=0,a=0,s=!1,l=0,u=0,c=0,d=0,h=0,f=0;r.addEventListener("mousedown",(function(r){if(t.refresh=!1,r.target===i){var l=i.style;c=parseInt(l.width,10),d=parseInt(l.height,10);var u=e.get("width"),p=e.get("height");c>u||d>p||(f=n.getZoom(),h=e.get("ratio"),s=!0,o=r.clientX,a=r.clientY)}}),!1),r.addEventListener("mousemove",(function(t){if(s&&!C()(t.clientX)&&!C()(t.clientY)){var r=e.get("width"),p=e.get("height"),g=i.style;l=parseInt(g.left,10),u=parseInt(g.top,10),c=parseInt(g.width,10),d=parseInt(g.height,10);var v=o-t.clientX,m=a-t.clientY;l-v<0?v=l:l-v+c>=r&&(v=0),u-m<0?m=u:u-m+d>=p&&(m=0),l-=v,u-=m,ve()(i,{left:l+"px",top:u+"px"}),n.translate(v*f/h,m*f/h),o=t.clientX,a=t.clientY}}),!1),r.addEventListener("mouseleave",(function(){s=!1,t.refresh=!0}),!1),r.addEventListener("mouseup",(function(){s=!1,t.refresh=!0}),!1),this.set("viewport",i),r.appendChild(i)}},t.prototype.updateViewport=function(){if(!this.destroyed){var e=this.get("ratio"),t=this.get("width"),n=this.get("height"),r=this.get("graph"),i=r.get("width"),o=i/r.get("height"),a=r.getGroup(),s=a.getCanvasBBox(),l=[(s.minX+s.maxX)/2,(s.minY+s.maxY)/2],u=[s.maxX-s.minX,s.maxY-s.minY],c={centerX:l[0],centerY:l[1],width:0,height:0,minX:0,minY:0};s[0]/s[1]>o?(c.width=u[0],c.height=c.width/o):(c.height=u[1],c.width=c.height*o),c.minX=l[0]-c.width/2,c.minY=l[1]-c.height/2;var d=a.getMatrix();d||(d=[1,0,0,0,1,0,0,0,1]);var h=le.invert([],d),f=Be({x:c.minX,y:c.minY},h),p=r.getCanvasByPoint(f.x,f.y),g=this.get("viewport");g||this.initViewport();var v=i/c.width,m=v*t,y=v*n,b=t*-p.x/c.width,C=n*-p.y/c.height,A=b+m,_=C+y;b<0&&(m+=b,b=0),A>t&&(m-=A-t),C<0&&(y+=C,C=0),_>n&&(y-=_-n),this.set("ratio",e);var I=b+"px",x=C+"px";ve()(g,{left:I,top:x,width:m+"px",height:y+"px"})}},t.prototype.init=function(){this.initContainer()},t.prototype.initContainer=function(){var e=this.get("graph"),t=e.get("width"),n=e.get("height")/t,r=this.get("className"),i=this.get("container"),o=this.get("width"),a=this.get("height");o||a||(o=200),o?(a=n*o,this.set("height",a)):(o=1/n*a,this.set("width",o));var s=ye()("<div class='"+r+"' style='width: "+o+"px; height: "+a+"px; overflow: hidden; position: relative;'></div>");x()(i)&&(i=document.getElementById(i)),i?i.appendChild(s):e.get("container").appendChild(s),this.set("container",s);var l=ye()('<div class="g6-minimap-container" style="position: relative; width: 100%; height: 100%; text-align: center; display: table;"></div>');s.appendChild(l);var u=ye()('<span style="display: table-cell; vertical-align: middle; "></span>');l.appendChild(u),this.set("containerDOM",l),this.set("containerSpan",u);var c=ye()('<img alt="" src="'+this.get("graphImg")+'" style="display: inline-block;" ondragstart="return false;" onselectstart="return false;"/>');this.set("imgDOM",c),this.updateImgSize(),u.appendChild(c),this.updateCanvas()},t.prototype.updateImgSize=function(){var e=this.get("imgDOM"),t=this.get("width"),n=this.get("height");e.onload=function(){var r=function(e,t){var n,r;if(e.naturalWidth)n=e.naturalWidth,r=e.naturalHeight;else{var i=new Image;i.src=e.src,i.onload=function(){}}return[n,r]}(e);r[0]>r[1]?e.width=t:e.height=n}},t.prototype.updateCanvas=function(){if(this.get("refresh")){var e=this.get("graph");if(!e.get("destroyed")){this.get("viewportChange")&&(this.set("viewportChange",!1),this.updateViewport());var t=this.get("width")/e.get("canvas").getCanvasBBox().width;this.set("ratio",t),this.updateViewport()}}},t.prototype.getViewport=function(){return this.get("viewport")},t.prototype.getContainer=function(){return this.get("container")},t.prototype.updateGraphImg=function(e){this.get("imgDOM").remove(),this.set("graphImg",e);var t=ye()('<img alt="" src="'+e+'" style="display: inline-block;" ondragstart="return false;" onselectstart="return false;"/>');this.set("imgDOM",t),t.src=e,this.updateImgSize(),this.get("containerSpan").appendChild(t),this.updateCanvas()},t.prototype.destroy=function(){var e=this.get("container");e.parentNode.removeChild(e)},t}(Ca),El={stroke:"#000",strokeOpacity:.8,lineWidth:2,fillOpacity:1,fill:"#fff"},Ol={Menu:wa,Grid:_a,Minimap:Sa,Bundling:Oa,ToolBar:Da,Tooltip:Pa,Fisheye:La,TimeBar:Sl,ImageMinimap:Tl,EdgeFilterLens:function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(w.__extends)(t,e),t.prototype.getDefaultCfgs=function(){return{type:"both",trigger:"mousemove",r:60,delegateStyle:Object(p.clone)(El),showLabel:"edge",scaleRBy:"wheel"}},t.prototype.getEvents=function(){var e;switch(this.get("trigger")){case"click":e={click:"filter"};break;case"drag":e={click:"createDelegate"};break;default:e={mousemove:"filter"}}return e},t.prototype.init=function(){var e=this.get("showLabel"),t="node"===e||"both"===e,n="edge"===e||"both"===e;this.set("showNodeLabel",t),this.set("showEdgeLabel",n),this.get("shouldShow")||this.set("shouldShow",(function(){return!0}))},t.prototype.createDelegate=function(e){var t=this,n=t.get("delegate");n&&!n.destroyed||(t.filter(e),(n=t.get("delegate")).on("dragstart",(function(e){})),n.on("drag",(function(e){t.filter(e)})),"wheel"===this.get("scaleRBy")&&n.on("mousewheel",(function(e){t.scaleRByWheel(e)})))},t.prototype.scaleRByWheel=function(e){if(e&&e.originalEvent){e.preventDefault&&e.preventDefault();var t,n=this.get("graph"),r=this.get("delegate");(r?(r.attr("x"),r.attr("y"),1):void 0)||n.getPointByClient(e.clientX,e.clientY),t=e.originalEvent.wheelDelta<0?.95:1/.95;var i=this.get("maxR"),o=this.get("minR"),a=this.get("r");(a>(i||n.get("height"))&&t>1||a<(o||.05*n.get("height"))&&t<1)&&(t=1),a*=t,this.set("r",a),this.filter(e)}},t.prototype.filter=function(e){var t=this.get("graph"),n=t.getNodes(),r={},i=this.get("r"),o=this.get("type"),a={x:e.x,y:e.y};this.updateDelegate(a,i);var s=this.get("shouldShow"),l=this.get("vShapes");l&&l.forEach((function(e){e.remove(),e.destroy()})),l=[],n.forEach((function(e){var t=e.getModel(),n=t.x,o=t.y;Ve({x:n,y:o},a)<i&&(r[t.id]=e)}));var u=t.getEdges(),c=[];u.forEach((function(e){var t=e.getModel(),n=t.source,i=t.target;s(t)&&("only-source"===o||"one"===o?r[n]&&!r[i]&&c.push(e):"only-target"===o||"one"===o?r[i]&&!r[n]&&c.push(e):"both"===o&&r[n]&&r[i]&&c.push(e))}));var d=this.get("showNodeLabel"),h=this.get("showEdgelabel"),f=t.get("group");c.forEach((function(e){e.get("group").get("children").forEach((function(e){var t=e.get("type"),n=f.addShape(t,{attrs:e.attr()});l.push(n),d&&"text"===t&&n.set("visible",!0)}))})),Object.keys(r).forEach((function(e){var t=r[e].get("group").clone();if(f.add(t),l.push(t),h)for(var n=t.get("children"),i=0;i<n.length;i++){var o=n[i];"text"===o.get("type")&&o.set("visible",!0)}})),this.set("vShapes",l)},t.prototype.updateParams=function(e){var t=e.r,n=e.trigger,r=e.minR,i=e.maxR,o=e.scaleRBy,a=e.showLabel,s=e.shouldShow;if(isNaN(e.r)||this.set("r",t),isNaN(i)||this.set("maxR",i),isNaN(r)||this.set("minR",r),"mousemove"!==n&&"click"!==n||this.set("trigger",n),"wheel"===o||"unset"===o){this.set("scaleRBy",o),this.get("delegate").remove(),this.get("delegate").destroy();var l=this.get("dPercentText");l&&(l.remove(),l.destroy())}"node"!==a&&"both"!==a||this.set("showNodeLabel",!0),"edge"!==a&&"both"!==a||this.set("showEdgeLabel",!0),s&&this.set("shouldShow",s)},t.prototype.updateDelegate=function(e,t){var n=this,r=n.get("graph"),i=n.get("delegate");if(!i||i.destroyed){var o=r.get("group"),a=n.get("delegateStyle")||El;i=o.addShape("circle",{attrs:Object(w.__assign)({r:t,x:e.x,y:e.y},a),name:"lens-shape",draggable:!0}),"drag"!==this.get("trigger")&&"wheel"===this.get("scaleRBy")&&i.on("mousewheel",(function(e){n.scaleRByWheel(e)}))}else i.attr({x:e.x,y:e.y,r:t});n.set("delegate",i)},t.prototype.clear=function(){var e=this.get("vShapes");e&&e.forEach((function(e){e.remove(),e.destroy()})),e=[],this.set("vShapes",e);var t=this.get("delegate");t&&!t.destroyed&&(t.remove(),t.destroy())},t.prototype.destroy=function(){this.clear()},t}(Ca)};function kl(e,t,n){var r=e.findById(t);!function e(t,n,r,i){i.enter({current:n,previous:r}),t.getNeighbors(n,"target").forEach((function(o){i.allowTraversal({previous:r,current:n,next:o})&&e(t,o,n,i)})),i.leave({current:n,previous:r})}(e,r,null,function(e){void 0===e&&(e={});var t,n=e,r=function(){},i=(t={},function(e){var n=e.next;return!t[n.get("id")]&&(t[n.get("id")]=!0,!0)});return n.allowTraversal=e.allowTraversal||i,n.enter=e.enter||r,n.leave=e.leave||r,n}(n))}var Ll=function(){function e(){this.linkedList=new Ao}return e.prototype.isEmpty=function(){return!this.linkedList.head},e.prototype.peek=function(){return this.linkedList.head?this.linkedList.head.value:null},e.prototype.enqueue=function(e){this.linkedList.append(e)},e.prototype.dequeue=function(){var e=this.linkedList.deleteHead();return e?e.value:null},e.prototype.toString=function(e){return this.linkedList.toString(e)},e}(),Dl=function(e,t,n){var r=function(e){void 0===e&&(e={});var t,n=e,r=function(){},i=(t={},function(e){var n=e.next.get("id");return!t[n]&&(t[n]=!0,!0)});return n.allowTraversal=e.allowTraversal||i,n.enter=e.enter||r,n.leave=e.leave||r,n}(n),i=new Ll,o=e.findById(t);i.enqueue(o);for(var a=null,s=function(){var t=i.dequeue();r.enter({current:t,previous:a}),e.getNeighbors(t,"target").forEach((function(e){r.allowTraversal({previous:a,current:t,next:e})&&i.enqueue(e)})),r.leave({current:t,previous:a}),a=t};!i.isEmpty();)s()};function Pl(e){for(var t=[],n={},r={},i={},o=[],a=0,s=function(l){r[l.get("id")]=a,i[l.get("id")]=a,a+=1,t.push(l),n[l.get("id")]=!0;for(var u=l.getNeighbors("target").filter((function(t){return e.indexOf(t)>-1})),c=0;c<u.length;c++){var d=u[c];r[d.get("id")]||0===r[d.get("id")]?n[d.get("id")]&&(i[l.get("id")]=Math.min(i[l.get("id")],r[d.get("id")])):(s(d),i[l.get("id")]=Math.min(i[l.get("id")],i[d.get("id")]))}if(i[l.get("id")]===r[l.get("id")]){for(var h=[];t.length>0;){var f=t.pop();if(n[f.get("id")]=!1,h.push(f),f===l)break}h.length>0&&o.push(h)}},l=0,u=e;l<u.length;l++){var c=u[l];r[c.get("id")]||0===r[c.get("id")]||s(c)}return o}function Nl(e,t){var n=void 0===t?e.get("directed"):t,r=e.getNodes();return n?Pl(r):function(e){for(var t=[],n={},r=[],i=function(e){r.push(e),n[e.get("id")]=!0;for(var t=e.getNeighbors(),o=0;o<t.length;++o){var a=t[o];n[a.get("id")]||i(a)}},o=0;o<e.length;o++){var a=e[o];if(!n[a.get("id")]){i(a);for(var s=[];r.length>0;)s.push(r.pop());t.push(s)}}return t}(r)}var jl=function(e,t,n){var r,i,o;void 0===n&&(n=!0);for(var a=[],s=0,l=Nl(e,!1);s<l.length;s++){var u=l[s];if(u.length)for(var c=u[0],d=c.get("id"),h=[c],f=((r={})[d]=c,r),p=((i={})[d]=new Set,i);h.length>0;)for(var g=h.pop(),v=g.get("id"),m=g.getNeighbors(),y=0;y<m.length;y+=1){var b=m[y],C=b.get("id");if(C===v)a.push(((o={})[b.getID()]=g,o));else if(C in p){if(!p[v].has(b)){for(var A=!0,_=[b,g],I=f[v];p[C].size&&!p[C].has(I)&&(_.push(I),I!==f[I.getID()]);)I=f[I.getID()];if(_.push(I),t&&n?(A=!1,_.findIndex((function(e){return t.indexOf(e.get("id"))>-1}))>-1&&(A=!0)):t&&!n&&_.findIndex((function(e){return t.indexOf(e.get("id"))>-1}))>-1&&(A=!1),A){for(var x={},w=1;w<_.length;w+=1)x[_[w-1].getID()]=_[w];_.length&&(x[_[_.length-1].getID()]=_[0]),a.push(x)}p[C].add(g)}}else f[C]=g,h.push(b),p[C]=new Set([g])}}return a},Rl=function(e,t,n){void 0===n&&(n=!0);for(var r=[],i=new Set,o=[],a=[],s={},l={},u=function(e,l,c){var d=!1;if(t&&!1===n&&t.indexOf(e.get("id"))>-1)return d;r.push(e),i.add(e);for(var h=c[e.getID()],f=0;f<h.length;f+=1)if((v=s[h[f]])===l){for(var p={},g=1;g<r.length;g+=1)p[r[g-1].getID()]=r[g];r.length&&(p[r[r.length-1].getID()]=r[0]),a.push(p),d=!0}else i.has(v)||u(v,l,c)&&(d=!0);if(d)!function(e){for(var t=[e];t.length>0;){var n=t.pop();i.has(n)&&(i.delete(n),o[n.get("id")].forEach((function(e){t.push(e)})),o[n.get("id")].clear())}}(e);else for(f=0;f<h.length;f+=1){var v=s[h[f]];o[v.get("id")].has(e)||o[v.get("id")].add(e)}return r.pop(),d},c=e.getNodes(),d=0;d<c.length;d+=1){var h=c[d],f=h.getID();l[f]=d,s[d]=h}if(t&&n)for(d=0;d<t.length;d++)f=t[d],l[c[d].getID()]=l[f],l[f]=0,s[0]=e.findById(f),s[l[c[d].getID()]]=c[d];for(var p=function(e){for(var r,i,o=1/0,s=0;s<e.length;s+=1)for(var u=e[s],c=0;c<u.length;c++){var d=l[u[c].getID()];d<o&&(o=d,i=s)}var h=e[i],f=[];for(s=0;s<h.length;s+=1){var p=h[s];f[p.getID()]=[];for(var g=0,v=p.getNeighbors("target").filter((function(e){return h.indexOf(e)>-1}));g<v.length;g++){var m=v[g];m!==p||!1===n&&t.indexOf(p.getId())>-1?f[p.getID()].push(l[m.getID()]):a.push(((r={})[p.getID()]=p,r))}}return{component:h,adjList:f,minIdx:o}},g=0;g<c.length;){var v=Pl(c.filter((function(e){return l[e.getID()]>=g}))).filter((function(e){return e.length>1}));if(0===v.length)break;var m=p(v),y=m.minIdx,b=m.adjList,C=m.component;if(!(C.length>1))break;C.forEach((function(e){o[e.get("id")]=new Set}));var A=s[y];if(t&&n&&-1===t.indexOf(A.get("id")))return a;u(A,A,b),g=y+1}return a},zl=function(e,t,n,r){return void 0===r&&(r=!0),void 0===t&&(t=e.get("directed")),t?Rl(e,n,r):jl(e,n,r)},Fl=function(e){var t=null,n={},r={},i={},o={};e.getNodes().forEach((function(e){r[e.getID()]=e}));for(var a={enter:function(e){var o=e.current,a=e.previous;if(i[o.getID()]){t={};for(var s=o,l=a;l.getID()!==o.getID();)t[s.getID()]=l,s=l,l=n[l.getID()];t[s.getID()]=l}else i[o.getID()]=o,delete r[o.getID()],n[o.getID()]=a},leave:function(e){var t=e.current;o[t.getID()]=t,delete i[t.getID()]},allowTraversal:function(e){var n=e.next;return!t&&!o[n.getID()]}};Object.keys(r).length;)kl(e,Object.keys(r)[0],a);return t},Bl=function(e,t,n,r,i){Object(ce.isString)(t)&&(t=e.findById(t)),Object(ce.isString)(n)&&(n=e.findById(n));var o=function(e,t,n,r){var i=e.getNodes(),o=[],a={},s={},l={};i.forEach((function(e,n){var r=e.getID();o.push(r),s[r]=1/0,r===t&&(s[r]=0)}));for(var u=i.length,c=function(e){var t=function(e,t,n){for(var r,i=1/0,o=0;o<t.length;o++){var a=t[o].get("id");!n[a]&&e[a]<=i&&(i=e[a],r=t[o])}return r}(s,i,a),o=t.get("id");if(a[o]=!0,s[o]===1/0)return"continue";(n?t.getOutEdges():t.getEdges()).forEach((function(e){var n=e.getTarget().getID(),i=e.getSource().getID(),a=n===o?i:n,u=r&&e.getModel()[r]?e.getModel()[r]:1;s[a]>s[t.get("id")]+u&&(s[a]=s[t.get("id")]+u,l[a]=t.get("id"))}))},d=0;d<u;d++)c();var h={};for(var f in s){h[f]=[f];for(var p=l[f];void 0!==p;)h[f].unshift(p),p=l[p]}return{length:s,path:h}}(e,t.get("id"),r,i),a=o.length,s=o.path;return{length:a[n.get("id")],path:s[n.get("id")]}},Hl=function(e,t,n,r){var i;if(Object(ce.isString)(t)&&(t=e.findById(t)),Object(ce.isString)(n)&&(n=e.findById(n)),void 0===r&&(r=e.get("directed")),t===n)return[[t.get("id")]];var o=[t],a=((i={})[t.get("id")]=!0,i),s=[],l=[],u=r?t.getNeighbors("target"):t.getNeighbors();for(s.push(u);o.length>0&&s.length>0;){var c=s[s.length-1];if(c.length){var d=c.shift();if(d&&(o.push(d),a[d.get("id")]=!0,u=r?d.getNeighbors("target"):d.getNeighbors(),s.push(u.filter((function(e){return!a[e.get("id")]})))),o[o.length-1]===n){var h=o.map((function(e){return e.get("id")}));l.push(h),f=o.pop(),a[f.get("id")]=!1,s.pop()}}else{var f=o.pop();a[f.get("id")]=!1,s.pop()}}return l},Ul=function(){function e(e){this.count=e.length,this.parent={};for(var t=0,n=e;t<n.length;t++){var r=n[t];this.parent[r]=r}}return e.prototype.find=function(e){for(;this.parent[e]!==e;)e=this.parent[e];return e},e.prototype.union=function(e,t){var n=this.find(e),r=this.find(t);n!==r&&(n<r?(this.parent[t]!==t&&this.union(this.parent[t],e),this.parent[t]=this.parent[e]):(this.parent[e]!==e&&this.union(this.parent[e],t),this.parent[e]=this.parent[t]))},e.prototype.connected=function(e,t){return this.find(e)===this.find(t)},e}(),Vl=function(e,t){return e-t},Yl=function(){function e(e){void 0===e&&(e=Vl),this.compareFn=e,this.list=[]}return e.prototype.getLeft=function(e){return 2*e+1},e.prototype.getRight=function(e){return 2*e+2},e.prototype.getParent=function(e){return 0===e?null:Math.floor((e-1)/2)},e.prototype.isEmpty=function(){return this.list.length<=0},e.prototype.top=function(){return this.isEmpty()?void 0:this.list[0]},e.prototype.delMin=function(){var e=this.top(),t=this.list.pop();return this.list.length>0&&(this.list[0]=t,this.moveDown(0)),e},e.prototype.insert=function(e){if(null!==e){this.list.push(e);var t=this.list.length-1;return this.moveUp(t),!0}return!1},e.prototype.moveUp=function(e){for(var t=this.getParent(e);e&&e>0&&this.compareFn(this.list[t],this.list[e])>0;){var n=this.list[t];this.list[t]=this.list[e],this.list[e]=n,e=t,t=this.getParent(e)}},e.prototype.moveDown=function(e){var t,n=e,r=this.getLeft(e),i=this.getRight(e),o=this.list.length;null!==r&&r<o&&this.compareFn(this.list[n],this.list[r])>0?n=r:null!==i&&i<o&&this.compareFn(this.list[n],this.list[i])>0&&(n=i),e!==n&&(t=[this.list[n],this.list[e]],this.list[e]=t[0],this.list[n]=t[1],this.moveDown(n))},e}(),Wl=function(e,t){var n=[],r=e.getNodes();if(0===r.length)return n;var i=r[0],o=new Set;o.add(i);var a=new Yl((function(e,n){return t?e.getModel()[t]-n.getModel()[t]:0}));for(i.getEdges().forEach((function(e){a.insert(e)}));!a.isEmpty();){var s=a.delMin(),l=s.getSource(),u=s.getTarget();o.has(l)&&o.has(u)||(n.push(s),o.has(l)||(o.add(l),l.getEdges().forEach((function(e){a.insert(e)}))),o.has(u)||(o.add(u),u.getEdges().forEach((function(e){a.insert(e)}))))}return n},Gl=function(e,t){var n=[];if(0===e.getNodes().length)return n;var r=e.getEdges().map((function(e){return e}));t&&r.sort((function(e,n){return e.getModel()[t]-n.getModel()[t]}));for(var i=new Ul(e.getNodes().map((function(e){return e.get("id")})));r.length>0;){var o=r.shift(),a=o.getSource().get("id"),s=o.getTarget().get("id");i.connected(a,s)||(n.push(o),i.union(a,s))}return n};function Xl(e,t,n){return n?{prim:Wl,kruskal:Gl}[n](e,t):Gl(e,t)}var ql=function(e,t,n){"number"!=typeof t&&(t=1e-6),"number"!=typeof n&&(n=.85);for(var r,i=1,o=0,a=1e3,s=e.getNodes(),l=s.length,u={},c={},d=0;d<l;++d)u[h=(m=s[d]).get("id")]=1/l,c[h]=1/l;for(;a>0&&i>t;){for(o=0,d=0;d<l;++d){var h=(m=s[d]).get("id");if(r=0,0===e.getNodeDegree(m,"in"))u[h]=0;else{for(var f=m.getNeighbors("source"),p=0;p<f.length;++p){var g=f[p],v=e.getNodeDegree(g,"out");v>0&&(r+=c[g.get("id")]/v)}u[h]=n*r,o+=u[h]}}for(o=(1-o)/l,i=0,d=0;d<l;++d){var m;r=u[h=(m=s[d]).get("id")]+o,i+=Math.abs(r-c[h]),c[h]=r}a-=1}return c},Zl=ma.registerNode,Ql=ma.registerEdge,Kl=ma.registerCombo,Jl=Lt.registerBehavior,$l=ro.registerLayout,eu=Ol.Minimap,tu=Ol.Grid,nu=Ol.Bundling,ru=Ol.Menu,iu=Ol.Fisheye,ou=Ol.ToolBar,au=Ol.Tooltip,su=Ol.TimeBar,lu=Ol.ImageMinimap,uu=Ol.EdgeFilterLens;t.default={version:d.a,Graph:Po,TreeGraph:Ro,Util:ba,registerNode:ma.registerNode,registerEdge:ma.registerEdge,registerCombo:ma.registerCombo,registerBehavior:Lt.registerBehavior,registerLayout:ro.registerLayout,Layout:ro,Global:J,Minimap:Ol.Minimap,Grid:Ol.Grid,Bundling:Ol.Bundling,Menu:Ol.Menu,ToolBar:Ol.ToolBar,Tooltip:Ol.Tooltip,TimeBar:su,Fisheye:iu,ImageMinimap:lu,EdgeFilterLens:uu,Algorithm:u,Arrow:{triangle:function(e,t,n){void 0===e&&(e=10),void 0===t&&(t=15),void 0===n&&(n=0);var r=2*n;return"M "+r+",0 L "+(r+t)+",-"+e/2+" L "+(r+t)+","+e/2+" Z"},vee:function(e,t,n){void 0===e&&(e=15),void 0===t&&(t=20),void 0===n&&(n=0);var r=2*n;return"M "+r+",0 L "+(r+t)+",-"+e/2+"\n        L "+(r+2*t/3)+",0 L "+(r+t)+","+e/2+" Z"},circle:function(e,t){return void 0===e&&(e=5),void 0===t&&(t=0),"M "+2*t+", 0\n            a "+e+","+e+" 0 1,0 "+2*e+",0\n            a "+e+","+e+" 0 1,0 "+2*-e+",0"},rect:function(e,t,n){void 0===e&&(e=10),void 0===t&&(t=10),void 0===n&&(n=0);var r=2*n;return"M "+r+","+-e/2+" \n        L "+(r+t)+","+-e/2+" \n        L "+(r+t)+","+e/2+" \n        L "+r+","+e/2+" Z"},diamond:function(e,t,n){void 0===e&&(e=15),void 0===t&&(t=15),void 0===n&&(n=0);var r=2*n;return"M "+r+",0 \n        L "+(r+t/2)+","+-e/2+" \n        L "+(r+t)+",0 \n        L "+(r+t/2)+","+e/2+" Z"},triangleRect:function(e,t,n,r,i,o){void 0===e&&(e=15),void 0===t&&(t=15),void 0===n&&(n=15),void 0===r&&(r=3),void 0===i&&(i=5),void 0===o&&(o=0);var a=2*o,s=a+t+i;return"M "+a+",0 L "+(a+t)+",-"+e/2+" L "+(a+t)+","+e/2+" Z\n            M "+s+", -"+n/2+"\n            L "+(s+r)+" -"+n/2+"\n            L "+(s+r)+" "+n/2+"\n            L "+s+" "+n/2+"\n            Z"}},Marker:va}}]).default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.getBoundPosition=function(e,t,n){if(!e.props.bounds)return[t,n];var a=e.props.bounds;a="string"==typeof a?a:function(e){return{left:e.left,top:e.top,right:e.right,bottom:e.bottom}}(a);var s=o(e);if("string"==typeof a){var l,u=s.ownerDocument,c=u.defaultView;if(!((l="parent"===a?s.parentNode:u.querySelector(a))instanceof c.HTMLElement))throw new Error('Bounds selector "'+a+'" could not find an element.');var d=c.getComputedStyle(s),h=c.getComputedStyle(l);a={left:-s.offsetLeft+(0,r.int)(h.paddingLeft)+(0,r.int)(d.marginLeft),top:-s.offsetTop+(0,r.int)(h.paddingTop)+(0,r.int)(d.marginTop),right:(0,i.innerWidth)(l)-(0,i.outerWidth)(s)-s.offsetLeft+(0,r.int)(h.paddingRight)-(0,r.int)(d.marginRight),bottom:(0,i.innerHeight)(l)-(0,i.outerHeight)(s)-s.offsetTop+(0,r.int)(h.paddingBottom)-(0,r.int)(d.marginBottom)}}(0,r.isNum)(a.right)&&(t=Math.min(t,a.right));(0,r.isNum)(a.bottom)&&(n=Math.min(n,a.bottom));(0,r.isNum)(a.left)&&(t=Math.max(t,a.left));(0,r.isNum)(a.top)&&(n=Math.max(n,a.top));return[t,n]},t.snapToGrid=function(e,t,n){var r=Math.round(t/e[0])*e[0],i=Math.round(n/e[1])*e[1];return[r,i]},t.canDragX=function(e){return"both"===e.props.axis||"x"===e.props.axis},t.canDragY=function(e){return"both"===e.props.axis||"y"===e.props.axis},t.getControlPosition=function(e,t,n){var r="number"==typeof t?(0,i.getTouch)(e,t):null;if("number"==typeof t&&!r)return null;var a=o(n),s=n.props.offsetParent||a.offsetParent||a.ownerDocument.body;return(0,i.offsetXYFromParent)(r||e,s,n.props.scale)},t.createCoreData=function(e,t,n){var i=e.state,a=!(0,r.isNum)(i.lastX),s=o(e);return a?{node:s,deltaX:0,deltaY:0,lastX:t,lastY:n,x:t,y:n}:{node:s,deltaX:t-i.lastX,deltaY:n-i.lastY,lastX:i.lastX,lastY:i.lastY,x:t,y:n}},t.createDraggableData=function(e,t){var n=e.props.scale;return{node:t.node,x:e.state.x+t.deltaX/n,y:e.state.y+t.deltaY/n,deltaX:t.deltaX/n,deltaY:t.deltaY/n,lastX:e.state.x,lastY:e.state.y}};var r=n(380),i=n(521);function o(e){var t=e.findDOMNode();if(!t)throw new Error("<DraggableCore>: Unmounted during event!");return t}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(){void 0}},function(e,t,n){"use strict";t.version="1.58.4"},function(e,t,n){"use strict";e.exports=function(e){for(var t,n=e.length,r=0;r<n;r++)if(((t=e.charCodeAt(r))<9||t>13)&&32!==t&&133!==t&&160!==t&&5760!==t&&6158!==t&&(t<8192||t>8205)&&8232!==t&&8233!==t&&8239!==t&&8287!==t&&8288!==t&&12288!==t&&65279!==t)return!1;return!0}},function(e,t,n){"use strict";var r=n(39),i=n(42),o=[];e.exports=function(e,t){if(-1===o.indexOf(e)){o.push(e);var n=1e3;i(t)?n=t:"long"===t&&(n=3e3);var a=r.select("body").selectAll(".plotly-notifier").data([0]);a.enter().append("div").classed("plotly-notifier",!0),a.selectAll(".notifier-note").data(o).enter().append("div").classed("notifier-note",!0).style("opacity",0).each((function(e){var i=r.select(this);i.append("button").classed("notifier-close",!0).html("&times;").on("click",(function(){i.transition().call(s)}));for(var o=i.append("p"),a=e.split(/<br\s*\/?>/g),l=0;l<a.length;l++)l&&o.append("br"),o.append("span").text(a[l]);"stick"===t?i.transition().duration(350).style("opacity",1):i.transition().duration(700).style("opacity",1).transition().delay(n).call(s)}))}function s(e){e.duration(700).style("opacity",0).each("end",(function(e){var t=o.indexOf(e);-1!==t&&o.splice(t,1),r.select(this).remove()}))}}},function(e,t,n){"use strict";e.exports=function(e,t){if(t instanceof RegExp){for(var n=t.toString(),r=0;r<e.length;r++)if(e[r]instanceof RegExp&&e[r].toString()===n)return e;e.push(t)}else!t&&0!==t||-1!==e.indexOf(t)||e.push(t);return e}},function(e,t,n){e.exports={create:n(529),clone:n(969),copy:n(1652),identity:n(530),transpose:n(531),invert:n(254),adjoint:n(1653),determinant:n(532),multiply:n(255),translate:n(533),scale:n(534),rotate:n(970),rotateX:n(971),rotateY:n(972),rotateZ:n(973),fromRotation:n(1654),fromRotationTranslation:n(974),fromScaling:n(1655),fromTranslation:n(1656),fromXRotation:n(1657),fromYRotation:n(1658),fromZRotation:n(1659),fromQuat:n(535),frustum:n(1660),perspective:n(975),perspectiveFromFieldOfView:n(1661),ortho:n(976),lookAt:n(536),str:n(1662)}},function(e,t){e.exports=function(e){var t=new Float32Array(16);return t[0]=e[0],t[1]=e[1],t[2]=e[2],t[3]=e[3],t[4]=e[4],t[5]=e[5],t[6]=e[6],t[7]=e[7],t[8]=e[8],t[9]=e[9],t[10]=e[10],t[11]=e[11],t[12]=e[12],t[13]=e[13],t[14]=e[14],t[15]=e[15],t}},function(e,t){e.exports=function(e,t,n,r){var i,o,a,s,l,u,c,d,h,f,p,g,v,m,y,b,C,A,_,I,x,w,M,S,T=r[0],E=r[1],O=r[2],k=Math.sqrt(T*T+E*E+O*O);if(Math.abs(k)<1e-6)return null;T*=k=1/k,E*=k,O*=k,i=Math.sin(n),o=Math.cos(n),a=1-o,s=t[0],l=t[1],u=t[2],c=t[3],d=t[4],h=t[5],f=t[6],p=t[7],g=t[8],v=t[9],m=t[10],y=t[11],b=T*T*a+o,C=E*T*a+O*i,A=O*T*a-E*i,_=T*E*a-O*i,I=E*E*a+o,x=O*E*a+T*i,w=T*O*a+E*i,M=E*O*a-T*i,S=O*O*a+o,e[0]=s*b+d*C+g*A,e[1]=l*b+h*C+v*A,e[2]=u*b+f*C+m*A,e[3]=c*b+p*C+y*A,e[4]=s*_+d*I+g*x,e[5]=l*_+h*I+v*x,e[6]=u*_+f*I+m*x,e[7]=c*_+p*I+y*x,e[8]=s*w+d*M+g*S,e[9]=l*w+h*M+v*S,e[10]=u*w+f*M+m*S,e[11]=c*w+p*M+y*S,t!==e&&(e[12]=t[12],e[13]=t[13],e[14]=t[14],e[15]=t[15]);return e}},function(e,t){e.exports=function(e,t,n){var r=Math.sin(n),i=Math.cos(n),o=t[4],a=t[5],s=t[6],l=t[7],u=t[8],c=t[9],d=t[10],h=t[11];t!==e&&(e[0]=t[0],e[1]=t[1],e[2]=t[2],e[3]=t[3],e[12]=t[12],e[13]=t[13],e[14]=t[14],e[15]=t[15]);return e[4]=o*i+u*r,e[5]=a*i+c*r,e[6]=s*i+d*r,e[7]=l*i+h*r,e[8]=u*i-o*r,e[9]=c*i-a*r,e[10]=d*i-s*r,e[11]=h*i-l*r,e}},function(e,t){e.exports=function(e,t,n){var r=Math.sin(n),i=Math.cos(n),o=t[0],a=t[1],s=t[2],l=t[3],u=t[8],c=t[9],d=t[10],h=t[11];t!==e&&(e[4]=t[4],e[5]=t[5],e[6]=t[6],e[7]=t[7],e[12]=t[12],e[13]=t[13],e[14]=t[14],e[15]=t[15]);return e[0]=o*i-u*r,e[1]=a*i-c*r,e[2]=s*i-d*r,e[3]=l*i-h*r,e[8]=o*r+u*i,e[9]=a*r+c*i,e[10]=s*r+d*i,e[11]=l*r+h*i,e}},function(e,t){e.exports=function(e,t,n){var r=Math.sin(n),i=Math.cos(n),o=t[0],a=t[1],s=t[2],l=t[3],u=t[4],c=t[5],d=t[6],h=t[7];t!==e&&(e[8]=t[8],e[9]=t[9],e[10]=t[10],e[11]=t[11],e[12]=t[12],e[13]=t[13],e[14]=t[14],e[15]=t[15]);return e[0]=o*i+u*r,e[1]=a*i+c*r,e[2]=s*i+d*r,e[3]=l*i+h*r,e[4]=u*i-o*r,e[5]=c*i-a*r,e[6]=d*i-s*r,e[7]=h*i-l*r,e}},function(e,t){e.exports=function(e,t,n){var r=t[0],i=t[1],o=t[2],a=t[3],s=r+r,l=i+i,u=o+o,c=r*s,d=r*l,h=r*u,f=i*l,p=i*u,g=o*u,v=a*s,m=a*l,y=a*u;return e[0]=1-(f+g),e[1]=d+y,e[2]=h-m,e[3]=0,e[4]=d-y,e[5]=1-(c+g),e[6]=p+v,e[7]=0,e[8]=h+m,e[9]=p-v,e[10]=1-(c+f),e[11]=0,e[12]=n[0],e[13]=n[1],e[14]=n[2],e[15]=1,e}},function(e,t){e.exports=function(e,t,n,r,i){var o=1/Math.tan(t/2),a=1/(r-i);return e[0]=o/n,e[1]=0,e[2]=0,e[3]=0,e[4]=0,e[5]=o,e[6]=0,e[7]=0,e[8]=0,e[9]=0,e[10]=(i+r)*a,e[11]=-1,e[12]=0,e[13]=0,e[14]=2*i*r*a,e[15]=0,e}},function(e,t){e.exports=function(e,t,n,r,i,o,a){var s=1/(t-n),l=1/(r-i),u=1/(o-a);return e[0]=-2*s,e[1]=0,e[2]=0,e[3]=0,e[4]=0,e[5]=-2*l,e[6]=0,e[7]=0,e[8]=0,e[9]=0,e[10]=2*u,e[11]=0,e[12]=(t+n)*s,e[13]=(i+r)*l,e[14]=(a+o)*u,e[15]=1,e}},function(e,t,n){"use strict";e.exports=function(e){return e}},function(e,t,n){"use strict";e.exports=o,e.exports.isMobile=o,e.exports.default=o;var r=/(android|bb\d+|meego).+mobile|avantgo|bada\/|blackberry|blazer|compal|elaine|fennec|hiptop|iemobile|ip(hone|od)|iris|kindle|lge |maemo|midp|mmp|mobile.+firefox|netfront|opera m(ob|in)i|palm( os)?|phone|p(ixi|re)\/|plucker|pocket|psp|series[46]0|symbian|treo|up\.(browser|link)|vodafone|wap|windows (ce|phone)|xda|xiino/i,i=/(android|bb\d+|meego).+mobile|avantgo|bada\/|blackberry|blazer|compal|elaine|fennec|hiptop|iemobile|ip(hone|od)|iris|kindle|lge |maemo|midp|mmp|mobile.+firefox|netfront|opera m(ob|in)i|palm( os)?|phone|p(ixi|re)\/|plucker|pocket|psp|series[46]0|symbian|treo|up\.(browser|link)|vodafone|wap|windows (ce|phone)|xda|xiino|android|ipad|playbook|silk/i;function o(e){e||(e={});var t=e.ua;if(t||"undefined"==typeof navigator||(t=navigator.userAgent),t&&t.headers&&"string"==typeof t.headers["user-agent"]&&(t=t.headers["user-agent"]),"string"!=typeof t)return!1;var n=e.tablet?i.test(t):r.test(t);return!n&&e.tablet&&e.featureDetect&&navigator&&navigator.maxTouchPoints>1&&-1!==t.indexOf("Macintosh")&&-1!==t.indexOf("Safari")&&(n=!0),n}},function(e,t,n){"use strict";var r=n(39),i=n(42),o=n(980),a=n(10),s=a.nestedProperty,l=n(383),u=n(982),c=n(27),d=n(297),h=n(82),f=n(1684),p=n(55),g=n(56),v=n(41),m=n(991).initInteractions,y=n(196),b=n(96),C=n(391).clearSelect,A=n(253).dfltConfig,_=n(1713),I=n(1715),x=n(549),w=n(124),M=n(95).AX_NAME_PATTERN,S=0;function T(e){var t=e._fullLayout;t._redrawFromAutoMarginCount?t._redrawFromAutoMarginCount--:e.emit("plotly_afterplot")}function E(e,t){try{e._fullLayout._paper.style("background",t)}catch(e){a.error(e)}}function O(e,t){E(e,v.combine(t,"white"))}function k(e,t){if(!e._context){e._context=a.extendDeep({},A);var n=r.select("base");e._context._baseUrl=n.size()&&n.attr("href")?window.location.href.split("#")[0]:""}var i,s,l,u=e._context;if(t){for(s=Object.keys(t),i=0;i<s.length;i++)"editable"!==(l=s[i])&&"edits"!==l&&l in u&&("setBackground"===l&&"opaque"===t[l]?u[l]=O:u[l]=t[l]);t.plot3dPixelRatio&&!u.plotGlPixelRatio&&(u.plotGlPixelRatio=u.plot3dPixelRatio);var c=t.editable;if(void 0!==c)for(u.editable=c,s=Object.keys(u.edits),i=0;i<s.length;i++)u.edits[s[i]]=c;if(t.edits)for(s=Object.keys(t.edits),i=0;i<s.length;i++)(l=s[i])in u.edits&&(u.edits[l]=t.edits[l]);u._exportedPlot=t._exportedPlot}u.staticPlot&&(u.editable=!1,u.edits={},u.autosizable=!1,u.scrollZoom=!1,u.doubleClick=!1,u.showTips=!1,u.showLink=!1,u.displayModeBar=!1),"hover"!==u.displayModeBar||o||(u.displayModeBar=!0),"transparent"!==u.setBackground&&"function"==typeof u.setBackground||(u.setBackground=E),u._hasZeroHeight=u._hasZeroHeight||0===e.clientHeight,u._hasZeroWidth=u._hasZeroWidth||0===e.clientWidth;var d=u.scrollZoom,h=u._scrollZoom={};if(!0===d)h.cartesian=1,h.gl3d=1,h.geo=1,h.mapbox=1;else if("string"==typeof d){var f=d.split("+");for(i=0;i<f.length;i++)h[f[i]]=1}else!1!==d&&(h.gl3d=1,h.geo=1,h.mapbox=1)}function L(e,t){var n,r,i=t+1,o=[];for(n=0;n<e.length;n++)(r=e[n])<0?o.push(i+r):o.push(r);return o}function D(e,t,n){var r,i;for(r=0;r<t.length;r++){if((i=t[r])!==parseInt(i,10))throw new Error("all values in "+n+" must be integers");if(i>=e.data.length||i<-e.data.length)throw new Error(n+" must be valid indices for gd.data.");if(t.indexOf(i,r+1)>-1||i>=0&&t.indexOf(-e.data.length+i)>-1||i<0&&t.indexOf(e.data.length+i)>-1)throw new Error("each index in "+n+" must be unique.")}}function P(e,t,n){if(!Array.isArray(e.data))throw new Error("gd.data must be an array.");if(void 0===t)throw new Error("currentIndices is a required argument.");if(Array.isArray(t)||(t=[t]),D(e,t,"currentIndices"),void 0===n||Array.isArray(n)||(n=[n]),void 0!==n&&D(e,n,"newIndices"),void 0!==n&&t.length!==n.length)throw new Error("current and new indices must be of equal length.")}function N(e,t,n,r,o){!function(e,t,n,r){var i=a.isPlainObject(r);if(!Array.isArray(e.data))throw new Error("gd.data must be an array");if(!a.isPlainObject(t))throw new Error("update must be a key:value object");if(void 0===n)throw new Error("indices must be an integer or array of integers");for(var o in D(e,n,"indices"),t){if(!Array.isArray(t[o])||t[o].length!==n.length)throw new Error("attribute "+o+" must be an array of length equal to indices array length");if(i&&(!(o in r)||!Array.isArray(r[o])||r[o].length!==t[o].length))throw new Error("when maxPoints is set as a key:value object it must contain a 1:1 corrispondence with the keys and number of traces in the update object")}}(e,t,n,r);for(var l=function(e,t,n,r){var o,l,u,c,d,h=a.isPlainObject(r),f=[];for(var p in Array.isArray(n)||(n=[n]),n=L(n,e.data.length-1),t)for(var g=0;g<n.length;g++){if(o=e.data[n[g]],l=(u=s(o,p)).get(),c=t[p][g],!a.isArrayOrTypedArray(c))throw new Error("attribute: "+p+" index: "+g+" must be an array");if(!a.isArrayOrTypedArray(l))throw new Error("cannot extend missing or non-array attribute: "+p);if(l.constructor!==c.constructor)throw new Error("cannot extend array with an array of a different type: "+p);d=h?r[p][g]:r,i(d)||(d=-1),f.push({prop:u,target:l,insert:c,maxp:Math.floor(d)})}return f}(e,t,n,r),u={},c={},d=0;d<l.length;d++){var h=l[d].prop,f=l[d].maxp,p=o(l[d].target,l[d].insert,f);h.set(p[0]),Array.isArray(u[h.astr])||(u[h.astr]=[]),u[h.astr].push(p[1]),Array.isArray(c[h.astr])||(c[h.astr]=[]),c[h.astr].push(l[d].target.length)}return{update:u,maxPoints:c}}function j(e,t){var n=new e.constructor(e.length+t.length);return n.set(e),n.set(t,e.length),n}function R(e,n,r,i){e=a.getGraphDiv(e),I.clearPromiseQueue(e);var o={};if("string"==typeof n)o[n]=r;else{if(!a.isPlainObject(n))return a.warn("Restyle fail.",n,r,i),Promise.reject();o=a.extendFlat({},n),void 0===i&&(i=r)}Object.keys(o).length&&(e.changed=!0);var s=I.coerceTraceIndices(e,i),l=H(e,o,s),c=l.flags;c.calc&&(e.calcdata=void 0),c.clearAxisTypes&&I.clearAxisTypes(e,s,{});var d=[];c.fullReplot?d.push(t.plot):(d.push(h.previousPromises),h.supplyDefaults(e),c.markerSize&&(h.doCalcdata(e),W(d)),c.style&&d.push(x.doTraceStyle),c.colorbars&&d.push(x.doColorBars),d.push(T)),d.push(h.rehover,h.redrag),u.add(e,R,[e,l.undoit,l.traces],R,[e,l.redoit,l.traces]);var f=a.syncOrAsync(d,e);return f&&f.then||(f=Promise.resolve()),f.then((function(){return e.emit("plotly_restyle",l.eventData),e}))}function z(e){return void 0===e?null:e}function F(e,t){return t?function(t,n,r){var i=s(t,n),o=i.set;return i.set=function(t){B((r||"")+n,i.get(),t,e),o(t)},i}:s}function B(e,t,n,r){if(Array.isArray(t)||Array.isArray(n))for(var i=Array.isArray(t)?t:[],o=Array.isArray(n)?n:[],s=Math.max(i.length,o.length),l=0;l<s;l++)B(e+"["+l+"]",i[l],o[l],r);else if(a.isPlainObject(t)||a.isPlainObject(n)){var u=a.isPlainObject(t)?t:{},c=a.isPlainObject(n)?n:{},d=a.extendFlat({},u,c);for(var h in d)B(e+"."+h,u[h],c[h],r)}else void 0===r[e]&&(r[e]=z(t))}function H(e,t,n){var r,i=e._fullLayout,o=e._fullData,l=e.data,u=i._guiEditing,f=F(i._preGUI,u),g=a.extendDeepAll({},t);U(t);var v,m=w.traceFlags(),y={},b={};function C(){return n.map((function(){}))}function A(e){var t=p.id2name(e);-1===v.indexOf(t)&&v.push(t)}function _(e){return"LAYOUT"+e+".autorange"}function x(e){return"LAYOUT"+e+".range"}function M(e){for(var t=e;t<o.length;t++)if(o[t]._input===l[e])return o[t]}function S(r,o,a){if(Array.isArray(r))r.forEach((function(e){S(e,o,a)}));else if(!(r in t)&&!I.hasParent(t,r)){var s;if("LAYOUT"===r.substr(0,6))s=f(e.layout,r.replace("LAYOUT",""));else{var c=n[a];s=F(i._tracePreGUI[M(c)._fullInput.uid],u)(l[c],r)}r in b||(b[r]=C()),void 0===b[r][a]&&(b[r][a]=z(s.get())),void 0!==o&&s.set(o)}}function T(e){return function(t){return o[t][e]}}function E(e){return function(t,r){return!1===t?o[n[r]][e]:null}}for(var O in t){if(I.hasParent(t,O))throw new Error("cannot set "+O+" and a parent attribute simultaneously");var k,L,D,P,N,j,R=t[O];if("autobinx"!==O&&"autobiny"!==O||(O=O.charAt(O.length-1)+"bins",R=Array.isArray(R)?R.map(E(O)):!1===R?n.map(T(O)):null),y[O]=R,"LAYOUT"!==O.substr(0,6)){for(b[O]=C(),r=0;r<n.length;r++){if(k=l[n[r]],L=M(n[r]),P=(D=F(i._tracePreGUI[L._fullInput.uid],u)(k,O)).get(),void 0!==(N=Array.isArray(R)?R[r%R.length]:R)){var B=D.parts[D.parts.length-1],H=O.substr(0,O.length-B.length-1),V=H?H+".":"",Y=H?s(L,H).get():L;if((j=d.getTraceValObject(L,D.parts))&&j.impliedEdits&&null!==N)for(var W in j.impliedEdits)S(a.relativeAttr(O,W),j.impliedEdits[W],r);else if("thicknessmode"!==B&&"lenmode"!==B||P===N||"fraction"!==N&&"pixels"!==N||!Y){if("type"===O&&("pie"===N!=("pie"===P)||"funnelarea"===N!=("funnelarea"===P))){var G="x",X="y";"bar"!==N&&"bar"!==P||"h"!==k.orientation||(G="y",X="x"),a.swapAttrs(k,["?","?src"],"labels",G),a.swapAttrs(k,["d?","?0"],"label",G),a.swapAttrs(k,["?","?src"],"values",X),"pie"===P||"funnelarea"===P?(s(k,"marker.color").set(s(k,"marker.colors").get()),i._pielayer.selectAll("g.trace").remove()):c.traceIs(k,"cartesian")&&s(k,"marker.colors").set(s(k,"marker.color").get())}}else{var q=i._size,Z=Y.orient,Q="top"===Z||"bottom"===Z;if("thicknessmode"===B){var K=Q?q.h:q.w;S(V+"thickness",Y.thickness*("fraction"===N?1/K:K),r)}else{var J=Q?q.w:q.h;S(V+"len",Y.len*("fraction"===N?1/J:J),r)}}b[O][r]=z(P);if(-1!==["swapxy","swapxyaxes","orientation","orientationaxes"].indexOf(O)){if("orientation"===O){D.set(N);var $=k.x&&!k.y?"h":"v";if((D.get()||$)===L.orientation)continue}else"orientationaxes"===O&&(k.orientation={v:"h",h:"v"}[L.orientation]);I.swapXYData(k),m.calc=m.clearAxisTypes=!0}else-1!==h.dataArrayContainers.indexOf(D.parts[0])?(I.manageArrayContainers(D,N,b),m.calc=!0):(j?j.arrayOk&&!c.traceIs(L,"regl")&&(a.isArrayOrTypedArray(N)||a.isArrayOrTypedArray(P))?m.calc=!0:w.update(m,j):m.calc=!0,D.set(N))}}if(-1!==["swapxyaxes","orientationaxes"].indexOf(O)&&p.swap(e,n),"orientationaxes"===O){var ee=s(e.layout,"hovermode"),te=ee.get();"x"===te?ee.set("y"):"y"===te?ee.set("x"):"x unified"===te?ee.set("y unified"):"y unified"===te&&ee.set("x unified")}if(-1!==["orientation","type"].indexOf(O)){for(v=[],r=0;r<n.length;r++){var ne=l[n[r]];c.traceIs(ne,"cartesian")&&(A(ne.xaxis||"x"),A(ne.yaxis||"y"))}S(v.map(_),!0,0),S(v.map(x),[0,1],0)}}else D=f(e.layout,O.replace("LAYOUT","")),b[O]=[z(D.get())],D.set(Array.isArray(R)?R[0]:R),m.calc=!0}return(m.calc||m.plot)&&(m.fullReplot=!0),{flags:m,undoit:b,redoit:y,traces:n,eventData:a.extendDeepNoArrays([],[g,n])}}function U(e){var t,n,r,i=a.counterRegex("axis",".title",!1,!1),o=/colorbar\.title$/,s=Object.keys(e);for(t=0;t<s.length;t++)n=s[t],r=e[n],"title"!==n&&!i.test(n)&&!o.test(n)||"string"!=typeof r&&"number"!=typeof r?n.indexOf("titlefont")>-1?l(n,n.replace("titlefont","title.font")):n.indexOf("titleposition")>-1?l(n,n.replace("titleposition","title.position")):n.indexOf("titleside")>-1?l(n,n.replace("titleside","title.side")):n.indexOf("titleoffset")>-1&&l(n,n.replace("titleoffset","title.offset")):l(n,n.replace("title","title.text"));function l(t,n){e[n]=e[t],delete e[t]}}function V(e,t,n){if(e=a.getGraphDiv(e),I.clearPromiseQueue(e),e.framework&&e.framework.isPolar)return Promise.resolve(e);var r={};if("string"==typeof t)r[t]=n;else{if(!a.isPlainObject(t))return a.warn("Relayout fail.",t,n),Promise.reject();r=a.extendFlat({},t)}Object.keys(r).length&&(e.changed=!0);var i=Z(e,r),o=i.flags;o.calc&&(e.calcdata=void 0);var s=[h.previousPromises];o.layoutReplot?s.push(x.layoutReplot):Object.keys(r).length&&(Y(e,o,i)||h.supplyDefaults(e),o.legend&&s.push(x.doLegend),o.layoutstyle&&s.push(x.layoutStyles),o.axrange&&W(s,i.rangesAltered),o.ticks&&s.push(x.doTicksRelayout),o.modebar&&s.push(x.doModeBar),o.camera&&s.push(x.doCamera),o.colorbars&&s.push(x.doColorBars),s.push(T)),s.push(h.rehover,h.redrag),u.add(e,V,[e,i.undoit],V,[e,i.redoit]);var l=a.syncOrAsync(s,e);return l&&l.then||(l=Promise.resolve(e)),l.then((function(){return e.emit("plotly_relayout",i.eventData),e}))}function Y(e,t,n){var r=e._fullLayout;if(!t.axrange)return!1;for(var i in t)if("axrange"!==i&&t[i])return!1;for(var o in n.rangesAltered){var a=p.id2name(o),s=e.layout[a],l=r[a];if(l.autorange=s.autorange,s.range&&(l.range=s.range.slice()),l.cleanRange(),l._matchGroup)for(var u in l._matchGroup)if(u!==o){var c=r[p.id2name(u)];c.autorange=l.autorange,c.range=l.range.slice(),c._input.range=l.range.slice()}}return!0}function W(e,t){var n=t?function(e){var n=[],r=!0;for(var i in t){var o=p.getFromId(e,i);if(n.push(i),-1!==(o.ticklabelposition||"").indexOf("inside")&&o._anchorAxis&&n.push(o._anchorAxis._id),o._matchGroup)for(var a in o._matchGroup)t[a]||n.push(a);o.automargin&&(r=!1)}return p.draw(e,n,{skipTitle:r})}:function(e){return p.draw(e,"redraw")};e.push(C,x.doAutoRangeAndConstraints,n,x.drawData,x.finalDraw)}var G=/^[xyz]axis[0-9]*\.range(\[[0|1]\])?$/,X=/^[xyz]axis[0-9]*\.autorange$/,q=/^[xyz]axis[0-9]*\.domain(\[[0|1]\])?$/;function Z(e,t){var n,r,i,o=e.layout,l=e._fullLayout,u=l._guiEditing,h=F(l._preGUI,u),f=Object.keys(t),g=p.list(e),v=a.extendDeepAll({},t),m={};for(U(t),f=Object.keys(t),r=0;r<f.length;r++)if(0===f[r].indexOf("allaxes")){for(i=0;i<g.length;i++){var y=g[i]._id.substr(1),b=-1!==y.indexOf("scene")?y+".":"",C=f[r].replace("allaxes",b+g[i]._name);t[C]||(t[C]=t[f[r]])}delete t[f[r]]}var A=w.layoutFlags(),x={},S={};function T(e,n){if(Array.isArray(e))e.forEach((function(e){T(e,n)}));else if(!(e in t)&&!I.hasParent(t,e)){var r=h(o,e);e in S||(S[e]=z(r.get())),void 0!==n&&r.set(n)}}var E,O={};function k(e){var t=p.name2id(e.split(".")[0]);return O[t]=1,t}for(var L in t){if(I.hasParent(t,L))throw new Error("cannot set "+L+" and a parent attribute simultaneously");for(var D=h(o,L),P=t[L],N=D.parts.length-1;N>0&&"string"!=typeof D.parts[N];)N--;var j=D.parts[N],R=D.parts[N-1]+"."+j,B=D.parts.slice(0,N).join("."),H=s(e.layout,B).get(),V=s(l,B).get(),Y=D.get();if(void 0!==P){x[L]=P,S[L]="reverse"===j?P:z(Y);var W=d.getLayoutValObject(l,D.parts);if(W&&W.impliedEdits&&null!==P)for(var Z in W.impliedEdits)T(a.relativeAttr(L,Z),W.impliedEdits[Z]);if(-1!==["width","height"].indexOf(L))if(P){T("autosize",null);var K="height"===L?"width":"height";T(K,l[K])}else l[L]=e._initialAutoSize[L];else if("autosize"===L)T("width",P?null:l.width),T("height",P?null:l.height);else if(R.match(G))k(R),s(l,B+"._inputRange").set(null);else if(R.match(X)){k(R),s(l,B+"._inputRange").set(null);var J=s(l,B).get();J._inputDomain&&(J._input.domain=J._inputDomain.slice())}else R.match(q)&&s(l,B+"._inputDomain").set(null);if("type"===j){E=H;var $="linear"===V.type&&"log"===P,ee="log"===V.type&&"linear"===P;if($||ee){if(E&&E.range)if(V.autorange)$&&(E.range=E.range[1]>E.range[0]?[1,2]:[2,1]);else{var te=E.range[0],ne=E.range[1];$?(te<=0&&ne<=0&&T(B+".autorange",!0),te<=0?te=ne/1e6:ne<=0&&(ne=te/1e6),T(B+".range[0]",Math.log(te)/Math.LN10),T(B+".range[1]",Math.log(ne)/Math.LN10)):(T(B+".range[0]",Math.pow(10,te)),T(B+".range[1]",Math.pow(10,ne)))}else T(B+".autorange",!0);Array.isArray(l._subplots.polar)&&l._subplots.polar.length&&l[D.parts[0]]&&"radialaxis"===D.parts[1]&&delete l[D.parts[0]]._subplot.viewInitial["radialaxis.range"],c.getComponentMethod("annotations","convertCoords")(e,V,P,T),c.getComponentMethod("images","convertCoords")(e,V,P,T)}else T(B+".autorange",!0),T(B+".range",null);s(l,B+"._inputRange").set(null)}else if(j.match(M)){var re=s(l,L).get(),ie=(P||{}).type;ie&&"-"!==ie||(ie="linear"),c.getComponentMethod("annotations","convertCoords")(e,re,ie,T),c.getComponentMethod("images","convertCoords")(e,re,ie,T)}var oe=_.containerArrayMatch(L);if(oe){n=oe.array,r=oe.index;var ae=oe.property,se=W||{editType:"calc"};""!==r&&""===ae&&(_.isAddVal(P)?S[L]=null:_.isRemoveVal(P)?S[L]=(s(o,n).get()||[])[r]:a.warn("unrecognized full object value",t)),w.update(A,se),m[n]||(m[n]={});var le=m[n][r];le||(le=m[n][r]={}),le[ae]=P,delete t[L]}else"reverse"===j?(H.range?H.range.reverse():(T(B+".autorange",!0),H.range=[1,0]),V.autorange?A.calc=!0:A.plot=!0):(l._has("scatter-like")&&l._has("regl")&&"dragmode"===L&&("lasso"===P||"select"===P)&&"lasso"!==Y&&"select"!==Y||l._has("gl2d")?A.plot=!0:W?w.update(A,W):A.calc=!0,D.set(P))}}for(n in m){_.applyContainerArrayChanges(e,h(o,n),m[n],A,h)||(A.plot=!0)}for(var ue in O){var ce=(E=p.getFromId(e,ue))&&E._constraintGroup;if(ce)for(var de in A.calc=!0,ce)O[de]||(p.getFromId(e,de)._constraintShrinkable=!0)}return(Q(e)||t.height||t.width)&&(A.plot=!0),(A.plot||A.calc)&&(A.layoutReplot=!0),{flags:A,rangesAltered:O,undoit:S,redoit:x,eventData:v}}function Q(e){var t=e._fullLayout,n=t.width,r=t.height;return e.layout.autosize&&h.plotAutoSize(e,e.layout,t),t.width!==n||t.height!==r}function K(e,n,r,i){if(e=a.getGraphDiv(e),I.clearPromiseQueue(e),e.framework&&e.framework.isPolar)return Promise.resolve(e);a.isPlainObject(n)||(n={}),a.isPlainObject(r)||(r={}),Object.keys(n).length&&(e.changed=!0),Object.keys(r).length&&(e.changed=!0);var o=I.coerceTraceIndices(e,i),s=H(e,a.extendFlat({},n),o),l=s.flags,c=Z(e,a.extendFlat({},r)),d=c.flags;(l.calc||d.calc)&&(e.calcdata=void 0),l.clearAxisTypes&&I.clearAxisTypes(e,o,r);var f=[];d.layoutReplot?f.push(x.layoutReplot):l.fullReplot?f.push(t.plot):(f.push(h.previousPromises),Y(e,d,c)||h.supplyDefaults(e),l.style&&f.push(x.doTraceStyle),(l.colorbars||d.colorbars)&&f.push(x.doColorBars),d.legend&&f.push(x.doLegend),d.layoutstyle&&f.push(x.layoutStyles),d.axrange&&W(f,c.rangesAltered),d.ticks&&f.push(x.doTicksRelayout),d.modebar&&f.push(x.doModeBar),d.camera&&f.push(x.doCamera),f.push(T)),f.push(h.rehover,h.redrag),u.add(e,K,[e,s.undoit,c.undoit,s.traces],K,[e,s.redoit,c.redoit,s.traces]);var p=a.syncOrAsync(f,e);return p&&p.then||(p=Promise.resolve(e)),p.then((function(){return e.emit("plotly_update",{data:s.eventData,layout:c.eventData}),e}))}function J(e){return function(t){t._fullLayout._guiEditing=!0;var n=e.apply(null,arguments);return t._fullLayout._guiEditing=!1,n}}var $=[{pattern:/^hiddenlabels/,attr:"legend.uirevision"},{pattern:/^((x|y)axis\d*)\.((auto)?range|title\.text)/},{pattern:/axis\d*\.showspikes$/,attr:"modebar.uirevision"},{pattern:/(hover|drag)mode$/,attr:"modebar.uirevision"},{pattern:/^(scene\d*)\.camera/},{pattern:/^(geo\d*)\.(projection|center|fitbounds)/},{pattern:/^(ternary\d*\.[abc]axis)\.(min|title\.text)$/},{pattern:/^(polar\d*\.radialaxis)\.((auto)?range|angle|title\.text)/},{pattern:/^(polar\d*\.angularaxis)\.rotation/},{pattern:/^(mapbox\d*)\.(center|zoom|bearing|pitch)/},{pattern:/^legend\.(x|y)$/,attr:"editrevision"},{pattern:/^(shapes|annotations)/,attr:"editrevision"},{pattern:/^title\.text$/,attr:"editrevision"}],ee=[{pattern:/^selectedpoints$/,attr:"selectionrevision"},{pattern:/(^|value\.)visible$/,attr:"legend.uirevision"},{pattern:/^dimensions\[\d+\]\.constraintrange/},{pattern:/^node\.(x|y|groups)/},{pattern:/^level$/},{pattern:/(^|value\.)name$/},{pattern:/colorbar\.title\.text$/},{pattern:/colorbar\.(x|y)$/,attr:"editrevision"}];function te(e,t){for(var n=0;n<t.length;n++){var r=t[n],i=e.match(r.pattern);if(i)return{head:i[1],attr:r.attr}}}function ne(e,t){var n=s(t,e).get();if(void 0!==n)return n;var r=e.split(".");for(r.pop();r.length>1;)if(r.pop(),void 0!==(n=s(t,r.join(".")+".uirevision").get()))return n;return t.uirevision}function re(e,t){for(var n=0;n<t.length;n++)if(t[n]._fullInput.uid===e)return n;return-1}function ie(e,t,n){for(var r=0;r<t.length;r++)if(t[r].uid===e)return r;return!t[n]||t[n].uid?-1:n}function oe(e,t){var n=a.isPlainObject(e),r=Array.isArray(e);return n||r?(n&&a.isPlainObject(t)||r&&Array.isArray(t))&&JSON.stringify(e)===JSON.stringify(t):e===t}function ae(e,t,n,r){var i,o,l,u=r.getValObject,c=r.flags,d=r.immutable,h=r.inArray,f=r.arrayIndex;function p(){var e=i.editType;h&&-1!==e.indexOf("arraydraw")?a.pushUnique(c.arrays[h],f):(w.update(c,i),"none"!==e&&c.nChanges++,r.transition&&i.anim&&c.nChangesAnim++,(G.test(l)||X.test(l))&&(c.rangesAltered[n[0]]=1),q.test(l)&&s(t,"_inputDomain").set(null),"datarevision"===o&&(c.newDataRevision=1))}function g(e){return"data_array"===e.valType||e.arrayOk}for(o in e){if(c.calc&&!r.transition)return;var v=e[o],m=t[o],y=n.concat(o);if(l=y.join("."),"_"!==o.charAt(0)&&"function"!=typeof v&&v!==m){if(("tick0"===o||"dtick"===o)&&"geo"!==n[0]){var b=t.tickmode;if("auto"===b||"array"===b||!b)continue}if(("range"!==o||!t.autorange)&&("zmin"!==o&&"zmax"!==o||"contourcarpet"!==t.type)&&(i=u(y))&&(!i._compareAsJSON||JSON.stringify(v)!==JSON.stringify(m))){var C,A=i.valType,_=g(i),I=Array.isArray(v),x=Array.isArray(m);if(I&&x){var M="_input_"+o,S=e[M],T=t[M];if(Array.isArray(S)&&S===T)continue}if(void 0===m)_&&I?c.calc=!0:p();else if(i._isLinkedToArray){var E=[],O=!1;h||(c.arrays[o]=E);var k=Math.min(v.length,m.length),L=Math.max(v.length,m.length);if(k!==L){if("arraydraw"!==i.editType){p();continue}O=!0}for(C=0;C<k;C++)ae(v[C],m[C],y.concat(C),a.extendFlat({inArray:o,arrayIndex:C},r));if(O)for(C=k;C<L;C++)E.push(C)}else!A&&a.isPlainObject(v)?ae(v,m,y,r):_?I&&x?(d&&(c.calc=!0),(d||r.newDataRevision)&&p()):I!==x?c.calc=!0:p():I&&x&&v.length===m.length&&String(v)===String(m)||p()}}}for(o in t)if(!(o in e)&&"_"!==o.charAt(0)&&"function"!=typeof t[o]){if(g(i=u(n.concat(o)))&&Array.isArray(t[o]))return void(c.calc=!0);p()}}function se(e){var t=e._fullLayout,n=e.getBoundingClientRect();if(!a.equalDomRects(n,t._lastBBox)){var r=t._invTransform=a.inverseTransformMatrix(a.getFullTransformMatrix(e));t._invScaleX=Math.sqrt(r[0][0]*r[0][0]+r[0][1]*r[0][1]+r[0][2]*r[0][2]),t._invScaleY=Math.sqrt(r[1][0]*r[1][0]+r[1][1]*r[1][1]+r[1][2]*r[1][2]),t._lastBBox=n}}function le(e){var t=r.select(e),n=e._fullLayout;if(n._calcInverseTransform=se,n._calcInverseTransform(e),n._container=t.selectAll(".plot-container").data([0]),n._container.enter().insert("div",":first-child").classed("plot-container",!0).classed("plotly",!0),n._paperdiv=n._container.selectAll(".svg-container").data([0]),n._paperdiv.enter().append("div").classed("user-select-none",!0).classed("svg-container",!0).style("position","relative"),n._glcontainer=n._paperdiv.selectAll(".gl-container").data([{}]),n._glcontainer.enter().append("div").classed("gl-container",!0),n._paperdiv.selectAll(".main-svg").remove(),n._paperdiv.select(".modebar-container").remove(),n._paper=n._paperdiv.insert("svg",":first-child").classed("main-svg",!0),n._toppaper=n._paperdiv.append("svg").classed("main-svg",!0),n._modebardiv=n._paperdiv.append("div"),delete n._modeBar,n._hoverpaper=n._paperdiv.append("svg").classed("main-svg",!0),!n._uid){var i={};r.selectAll("defs").each((function(){this.id&&(i[this.id.split("-")[1]]=1)})),n._uid=a.randstr(i)}n._paperdiv.selectAll(".main-svg").attr(y.svgAttrs),n._defs=n._paper.append("defs").attr("id","defs-"+n._uid),n._clips=n._defs.append("g").classed("clips",!0),n._topdefs=n._toppaper.append("defs").attr("id","topdefs-"+n._uid),n._topclips=n._topdefs.append("g").classed("clips",!0),n._bgLayer=n._paper.append("g").classed("bglayer",!0),n._draggers=n._paper.append("g").classed("draglayer",!0);var o=n._paper.append("g").classed("layer-below",!0);n._imageLowerLayer=o.append("g").classed("imagelayer",!0),n._shapeLowerLayer=o.append("g").classed("shapelayer",!0),n._cartesianlayer=n._paper.append("g").classed("cartesianlayer",!0),n._polarlayer=n._paper.append("g").classed("polarlayer",!0),n._ternarylayer=n._paper.append("g").classed("ternarylayer",!0),n._geolayer=n._paper.append("g").classed("geolayer",!0),n._funnelarealayer=n._paper.append("g").classed("funnelarealayer",!0),n._pielayer=n._paper.append("g").classed("pielayer",!0),n._treemaplayer=n._paper.append("g").classed("treemaplayer",!0),n._sunburstlayer=n._paper.append("g").classed("sunburstlayer",!0),n._indicatorlayer=n._toppaper.append("g").classed("indicatorlayer",!0),n._glimages=n._paper.append("g").classed("glimages",!0);var s=n._toppaper.append("g").classed("layer-above",!0);n._imageUpperLayer=s.append("g").classed("imagelayer",!0),n._shapeUpperLayer=s.append("g").classed("shapelayer",!0),n._infolayer=n._toppaper.append("g").classed("infolayer",!0),n._menulayer=n._toppaper.append("g").classed("menulayer",!0),n._zoomlayer=n._toppaper.append("g").classed("zoomlayer",!0),n._hoverlayer=n._hoverpaper.append("g").classed("hoverlayer",!0),n._modebardiv.classed("modebar-container",!0).style("position","absolute").style("top","0px").style("right","0px"),e.emit("plotly_framework")}t.animate=function(e,t,n){if(e=a.getGraphDiv(e),!a.isPlotDiv(e))throw new Error("This element is not a Plotly plot: "+e+". It's likely that you've failed to create a plot before animating it. For more details, see https://plotly.com/javascript/animations/");var r=e._transitionData;r._frameQueue||(r._frameQueue=[]);var i=(n=h.supplyAnimationDefaults(n)).transition,o=n.frame;function s(e){return Array.isArray(i)?e>=i.length?i[0]:i[e]:i}function l(e){return Array.isArray(o)?e>=o.length?o[0]:o[e]:o}function u(e,t){var n=0;return function(){if(e&&++n===t)return e()}}return void 0===r._frameWaitingCnt&&(r._frameWaitingCnt=0),new Promise((function(o,c){function d(){r._currentFrame&&r._currentFrame.onComplete&&r._currentFrame.onComplete();var t=r._currentFrame=r._frameQueue.shift();if(t){var n=t.name?t.name.toString():null;e._fullLayout._currentFrame=n,r._lastFrameAt=Date.now(),r._timeToNext=t.frameOpts.duration,h.transition(e,t.frame.data,t.frame.layout,I.coerceTraceIndices(e,t.frame.traces),t.frameOpts,t.transitionOpts).then((function(){t.onComplete&&t.onComplete()})),e.emit("plotly_animatingframe",{name:n,frame:t.frame,animation:{frame:t.frameOpts,transition:t.transitionOpts}})}else e.emit("plotly_animated"),window.cancelAnimationFrame(r._animationRaf),r._animationRaf=null}function f(){e.emit("plotly_animating"),r._lastFrameAt=-1/0,r._timeToNext=0,r._runningTransitions=0,r._currentFrame=null;var t=function(){r._animationRaf=window.requestAnimationFrame(t),Date.now()-r._lastFrameAt>r._timeToNext&&d()};t()}var p,g,v=0;function m(e){return Array.isArray(i)?v>=i.length?e.transitionOpts=i[v]:e.transitionOpts=i[0]:e.transitionOpts=i,v++,e}var y=[],b=null==t,C=Array.isArray(t);if(!b&&!C&&a.isPlainObject(t))y.push({type:"object",data:m(a.extendFlat({},t))});else if(b||-1!==["string","number"].indexOf(typeof t))for(p=0;p<r._frames.length;p++)(g=r._frames[p])&&(b||String(g.group)===String(t))&&y.push({type:"byname",name:String(g.name),data:m({name:g.name})});else if(C)for(p=0;p<t.length;p++){var A=t[p];-1!==["number","string"].indexOf(typeof A)?(A=String(A),y.push({type:"byname",name:A,data:m({name:A})})):a.isPlainObject(A)&&y.push({type:"object",data:m(a.extendFlat({},A))})}for(p=0;p<y.length;p++)if("byname"===(g=y[p]).type&&!r._frameHash[g.data.name])return a.warn('animate failure: frame not found: "'+g.data.name+'"'),void c();-1!==["next","immediate"].indexOf(n.mode)&&function(){if(0!==r._frameQueue.length){for(;r._frameQueue.length;){var t=r._frameQueue.pop();t.onInterrupt&&t.onInterrupt()}e.emit("plotly_animationinterrupted",[])}}(),"reverse"===n.direction&&y.reverse();var _=e._fullLayout._currentFrame;if(_&&n.fromcurrent){var x=-1;for(p=0;p<y.length;p++)if("byname"===(g=y[p]).type&&g.name===_){x=p;break}if(x>0&&x<y.length-1){var w=[];for(p=0;p<y.length;p++)g=y[p],("byname"!==y[p].type||p>x)&&w.push(g);y=w}}y.length>0?function(t){if(0!==t.length){for(var i=0;i<t.length;i++){var a;a="byname"===t[i].type?h.computeFrame(e,t[i].name):t[i].data;var d=l(i),p=s(i);p.duration=Math.min(p.duration,d.duration);var g={frame:a,name:t[i].name,frameOpts:d,transitionOpts:p};i===t.length-1&&(g.onComplete=u(o,2),g.onInterrupt=c),r._frameQueue.push(g)}"immediate"===n.mode&&(r._lastFrameAt=-1/0),r._animationRaf||f()}}(y):(e.emit("plotly_animated"),o())}))},t.addFrames=function(e,t,n){if(e=a.getGraphDiv(e),null==t)return Promise.resolve();if(!a.isPlotDiv(e))throw new Error("This element is not a Plotly plot: "+e+". It's likely that you've failed to create a plot before adding frames. For more details, see https://plotly.com/javascript/animations/");var r,i,o,s,l=e._transitionData._frames,c=e._transitionData._frameHash;if(!Array.isArray(t))throw new Error("addFrames failure: frameList must be an Array of frame definitions"+t);var d=l.length+2*t.length,f=[],p={};for(r=t.length-1;r>=0;r--)if(a.isPlainObject(t[r])){var g=t[r].name,v=(c[g]||p[g]||{}).name,m=t[r].name,y=c[v]||p[v];v&&m&&"number"==typeof m&&y&&S<5&&(S++,a.warn('addFrames: overwriting frame "'+(c[v]||p[v]).name+'" with a frame whose name of type "number" also equates to "'+v+'". This is valid but may potentially lead to unexpected behavior since all plotly.js frame names are stored internally as strings.'),5===S&&a.warn("addFrames: This API call has yielded too many of these warnings. For the rest of this call, further warnings about numeric frame names will be suppressed.")),p[g]={name:g},f.push({frame:h.supplyFrameDefaults(t[r]),index:n&&void 0!==n[r]&&null!==n[r]?n[r]:d+r})}f.sort((function(e,t){return e.index>t.index?-1:e.index<t.index?1:0}));var b=[],C=[],A=l.length;for(r=f.length-1;r>=0;r--){if("number"==typeof(i=f[r].frame).name&&a.warn("Warning: addFrames accepts frames with numeric names, but the numbers areimplicitly cast to strings"),!i.name)for(;c[i.name="frame "+e._transitionData._counter++];);if(c[i.name]){for(o=0;o<l.length&&(l[o]||{}).name!==i.name;o++);b.push({type:"replace",index:o,value:i}),C.unshift({type:"replace",index:o,value:l[o]})}else s=Math.max(0,Math.min(f[r].index,A)),b.push({type:"insert",index:s,value:i}),C.unshift({type:"delete",index:s}),A++}var _=h.modifyFrames,I=h.modifyFrames,x=[e,C],w=[e,b];return u&&u.add(e,_,x,I,w),h.modifyFrames(e,b)},t.deleteFrames=function(e,t){if(e=a.getGraphDiv(e),!a.isPlotDiv(e))throw new Error("This element is not a Plotly plot: "+e);var n,r,i=e._transitionData._frames,o=[],s=[];if(!t)for(t=[],n=0;n<i.length;n++)t.push(n);for((t=t.slice()).sort(),n=t.length-1;n>=0;n--)r=t[n],o.push({type:"delete",index:r}),s.unshift({type:"insert",index:r,value:i[r]});var l=h.modifyFrames,c=h.modifyFrames,d=[e,s],f=[e,o];return u&&u.add(e,l,d,c,f),h.modifyFrames(e,o)},t.addTraces=function e(n,r,i){n=a.getGraphDiv(n);var o,s,l=[],c=t.deleteTraces,d=e,h=[n,l],f=[n,r];for(function(e,t,n){var r,i;if(!Array.isArray(e.data))throw new Error("gd.data must be an array.");if(void 0===t)throw new Error("traces must be defined.");for(Array.isArray(t)||(t=[t]),r=0;r<t.length;r++)if("object"!=typeof(i=t[r])||Array.isArray(i)||null===i)throw new Error("all values in traces array must be non-array objects");if(void 0===n||Array.isArray(n)||(n=[n]),void 0!==n&&n.length!==t.length)throw new Error("if indices is specified, traces.length must equal indices.length")}(n,r,i),Array.isArray(r)||(r=[r]),r=r.map((function(e){return a.extendFlat({},e)})),I.cleanData(r),o=0;o<r.length;o++)n.data.push(r[o]);for(o=0;o<r.length;o++)l.push(-r.length+o);if(void 0===i)return s=t.redraw(n),u.add(n,c,h,d,f),s;Array.isArray(i)||(i=[i]);try{P(n,l,i)}catch(e){throw n.data.splice(n.data.length-r.length,r.length),e}return u.startSequence(n),u.add(n,c,h,d,f),s=t.moveTraces(n,l,i),u.stopSequence(n),s},t.deleteTraces=function e(n,r){n=a.getGraphDiv(n);var i,o,s=[],l=t.addTraces,c=e,d=[n,s,r],h=[n,r];if(void 0===r)throw new Error("indices must be an integer or array of integers.");for(Array.isArray(r)||(r=[r]),D(n,r,"indices"),(r=L(r,n.data.length-1)).sort(a.sorterDes),i=0;i<r.length;i+=1)o=n.data.splice(r[i],1)[0],s.push(o);var f=t.redraw(n);return u.add(n,l,d,c,h),f},t.extendTraces=function e(n,r,i,o){function s(e,t,n){var r,i;if(a.isTypedArray(e))if(n<0){var o=new e.constructor(0),s=j(e,t);n<0?(r=s,i=o):(r=o,i=s)}else if(r=new e.constructor(n),i=new e.constructor(e.length+t.length-n),n===t.length)r.set(t),i.set(e);else if(n<t.length){var l=t.length-n;r.set(t.subarray(l)),i.set(e),i.set(t.subarray(0,l),e.length)}else{var u=n-t.length,c=e.length-u;r.set(e.subarray(c)),r.set(t,u),i.set(e.subarray(0,c))}else r=e.concat(t),i=n>=0&&n<r.length?r.splice(0,r.length-n):[];return[r,i]}var l=N(n=a.getGraphDiv(n),r,i,o,s),c=t.redraw(n),d=[n,l.update,i,l.maxPoints];return u.add(n,t.prependTraces,d,e,arguments),c},t.moveTraces=function e(n,r,i){var o,s=[],l=[],c=e,d=e,h=[n=a.getGraphDiv(n),i,r],f=[n,r,i];if(P(n,r,i),r=Array.isArray(r)?r:[r],void 0===i)for(i=[],o=0;o<r.length;o++)i.push(-r.length+o);for(i=Array.isArray(i)?i:[i],r=L(r,n.data.length-1),i=L(i,n.data.length-1),o=0;o<n.data.length;o++)-1===r.indexOf(o)&&s.push(n.data[o]);for(o=0;o<r.length;o++)l.push({newIndex:i[o],trace:n.data[r[o]]});for(l.sort((function(e,t){return e.newIndex-t.newIndex})),o=0;o<l.length;o+=1)s.splice(l[o].newIndex,0,l[o].trace);n.data=s;var p=t.redraw(n);return u.add(n,c,h,d,f),p},t.prependTraces=function e(n,r,i,o){function s(e,t,n){var r,i;if(a.isTypedArray(e))if(n<=0){var o=new e.constructor(0),s=j(t,e);n<0?(r=s,i=o):(r=o,i=s)}else if(r=new e.constructor(n),i=new e.constructor(e.length+t.length-n),n===t.length)r.set(t),i.set(e);else if(n<t.length){var l=t.length-n;r.set(t.subarray(0,l)),i.set(t.subarray(l)),i.set(e,l)}else{var u=n-t.length;r.set(t),r.set(e.subarray(0,u),t.length),i.set(e.subarray(u))}else r=t.concat(e),i=n>=0&&n<r.length?r.splice(n,r.length):[];return[r,i]}var l=N(n=a.getGraphDiv(n),r,i,o,s),c=t.redraw(n),d=[n,l.update,i,l.maxPoints];return u.add(n,t.extendTraces,d,e,arguments),c},t.newPlot=function(e,n,r,i){return e=a.getGraphDiv(e),h.cleanPlot([],{},e._fullData||[],e._fullLayout||{}),h.purge(e),t.plot(e,n,r,i)},t.plot=function(e,n,i,o){var s;if(e=a.getGraphDiv(e),l.init(e),a.isPlainObject(n)){var u=n;n=u.data,i=u.layout,o=u.config,s=u.frames}if(!1===l.triggerHandler(e,"plotly_beforeplot",[n,i,o]))return Promise.reject();n||i||a.isPlotDiv(e)||a.warn("Calling Plotly.plot as if redrawing but this container doesn't yet have a plot.",e),k(e,o),i||(i={}),r.select(e).classed("js-plotly-plot",!0),g.makeTester(),Array.isArray(e._promises)||(e._promises=[]);var d=0===(e.data||[]).length&&Array.isArray(n);Array.isArray(n)&&(I.cleanData(n),d?e.data=n:e.data.push.apply(e.data,n),e.empty=!1),e.layout&&!d||(e.layout=I.cleanLayout(i)),h.supplyDefaults(e);var v=e._fullLayout,y=v._has("cartesian");if(!v._has("polar")&&n&&n[0]&&n[0].r)return a.log("Legacy polar charts are deprecated!"),function(e,t,n){var i=r.select(e).selectAll(".plot-container").data([0]);i.enter().insert("div",":first-child").classed("plot-container plotly",!0);var o=i.selectAll(".svg-container").data([0]);o.enter().append("div").classed("svg-container",!0).style("position","relative"),o.html(""),t&&(e.data=t);n&&(e.layout=n);f.manager.fillLayout(e),o.style({width:e._fullLayout.width+"px",height:e._fullLayout.height+"px"}),e.framework=f.manager.framework(e),e.framework({data:e.data,layout:e.layout},o.node()),e.framework.setUndoPoint();var s=e.framework.svg(),l=1,u=e._fullLayout.title?e._fullLayout.title.text:"";""!==u&&u||(l=0);var c=function(){this.call(b.convertToTspans,e)},d=s.select(".title-group text").call(c);if(e._context.edits.titleText){var p=a._(e,"Click to enter Plot title");u&&u!==p||(l=.2,d.attr({"data-unformatted":p}).text(p).style({opacity:l}).on("mouseover.opacity",(function(){r.select(this).transition().duration(100).style("opacity",1)})).on("mouseout.opacity",(function(){r.select(this).transition().duration(1e3).style("opacity",0)})));var g=function(){this.call(b.makeEditable,{gd:e}).on("edit",(function(t){e.framework({layout:{title:{text:t}}}),this.text(t).call(c),this.call(g)})).on("cancel",(function(){var e=this.attr("data-unformatted");this.text(e).call(c)}))};d.call(g)}return e._context.setBackground(e,e._fullLayout.paper_bgcolor),h.addLinks(e),Promise.resolve()}(e,n,i);v._replotting=!0,(d||v._shouldCreateBgLayer)&&(le(e),v._shouldCreateBgLayer&&delete v._shouldCreateBgLayer),e.framework!==le&&(e.framework=le,le(e)),g.initGradients(e),d&&p.saveShowSpikeInitial(e);var C=!e.calcdata||e.calcdata.length!==(e._fullData||[]).length;C&&h.doCalcdata(e);for(var A=0;A<e.calcdata.length;A++)e.calcdata[A][0].trace=e._fullData[A];e._context.responsive?e._responsiveChartHandler||(e._responsiveChartHandler=function(){a.isHidden(e)||h.resize(e)},window.addEventListener("resize",e._responsiveChartHandler)):a.clearResponsive(e);var _=a.extendFlat({},v._size),w=0;function M(){if(h.clearAutoMarginIds(e),x.drawMarginPushers(e),p.allowAutoMargin(e),v._has("pie"))for(var t=e._fullData,n=0;n<t.length;n++){var r=t[n];"pie"===r.type&&r.automargin&&h.allowAutoMargin(e,"pie."+r.uid+".automargin")}return h.doAutoMargin(e),h.previousPromises(e)}function S(){e._transitioning||(x.doAutoRangeAndConstraints(e),d&&p.saveRangeInitial(e),c.getComponentMethod("rangeslider","calcAutorange")(e))}var E=[h.previousPromises,function(){if(s)return t.addFrames(e,s)},function t(){for(var n=v._basePlotModules,r=0;r<n.length;r++)n[r].drawFramework&&n[r].drawFramework(e);if(!v._glcanvas&&v._has("gl")&&(v._glcanvas=v._glcontainer.selectAll(".gl-canvas").data([{key:"contextLayer",context:!0,pick:!1},{key:"focusLayer",context:!1,pick:!1},{key:"pickLayer",context:!1,pick:!0}],(function(e){return e.key})),v._glcanvas.enter().append("canvas").attr("class",(function(e){return"gl-canvas gl-canvas-"+e.key.replace("Layer","")})).style({position:"absolute",top:0,left:0,overflow:"visible","pointer-events":"none"})),v._glcanvas){v._glcanvas.attr("width",v.width).attr("height",v.height);var i=v._glcanvas.data()[0].regl;if(i&&(Math.floor(v.width)!==i._gl.drawingBufferWidth||Math.floor(v.height)!==i._gl.drawingBufferHeight)){var o="WebGL context buffer and canvas dimensions do not match due to browser/WebGL bug.";if(!w)return a.log(o+" Clearing graph and plotting again."),h.cleanPlot([],{},e._fullData,v),h.supplyDefaults(e),v=e._fullLayout,h.doCalcdata(e),w++,t();a.error(o)}}return"h"===v.modebar.orientation?v._modebardiv.style("height",null).style("width","100%"):v._modebardiv.style("width",null).style("height",v.height+"px"),h.previousPromises(e)},M,function(){if(h.didMarginChange(_,v._size))return a.syncOrAsync([M,x.layoutStyles],e)}];y&&E.push((function(){if(C)return a.syncOrAsync([c.getComponentMethod("shapes","calcAutorange"),c.getComponentMethod("annotations","calcAutorange"),S],e);S()})),E.push(x.layoutStyles),y&&E.push((function(){return p.draw(e,d?"":"redraw")}),(function(e){e._fullLayout._insideTickLabelsAutorange&&V(e,e._fullLayout._insideTickLabelsAutorange).then((function(){e._fullLayout._insideTickLabelsAutorange=void 0}))})),E.push(x.drawData,x.finalDraw,m,h.addLinks,h.rehover,h.redrag,h.doAutoMargin,(function(e){e._fullLayout._insideTickLabelsAutorange&&d&&p.saveRangeInitial(e,!0)}),h.previousPromises);var O=a.syncOrAsync(E,e);return O&&O.then||(O=Promise.resolve()),O.then((function(){return T(e),e}))},t.purge=function(e){var t=(e=a.getGraphDiv(e))._fullLayout||{},n=e._fullData||[];return h.cleanPlot([],{},n,t),h.purge(e),l.purge(e),t._container&&t._container.remove(),delete e._context,e},t.react=function(e,n,r,i){var o,l;e=a.getGraphDiv(e),I.clearPromiseQueue(e);var u=e._fullData,f=e._fullLayout;if(a.isPlotDiv(e)&&u&&f){if(a.isPlainObject(n)){var p=n;n=p.data,r=p.layout,i=p.config,o=p.frames}var g=!1;if(i){var v=a.extendDeep({},e._context);e._context=void 0,k(e,i),g=function e(t,n){var r;for(r in t)if("_"!==r.charAt(0)){var i=t[r],o=n[r];if(i!==o)if(a.isPlainObject(i)&&a.isPlainObject(o)){if(e(i,o))return!0}else{if(!Array.isArray(i)||!Array.isArray(o))return!0;if(i.length!==o.length)return!0;for(var s=0;s<i.length;s++)if(i[s]!==o[s]){if(!a.isPlainObject(i[s])||!a.isPlainObject(o[s]))return!0;if(e(i[s],o[s]))return!0}}}}(v,e._context)}e.data=n||[],I.cleanData(e.data),e.layout=r||{},I.cleanLayout(e.layout),function(e,t,n,r){var i,o,l,u,c,d,h,f,p=r._preGUI,g=[],v={};for(i in p){if(c=te(i,$)){if(o=c.attr||c.head+".uirevision",(u=(l=s(r,o).get())&&ne(o,t))&&u===l&&(null===(d=p[i])&&(d=void 0),oe(f=(h=s(t,i)).get(),d))){void 0===f&&"autorange"===i.substr(i.length-9)&&g.push(i.substr(0,i.length-10)),h.set(z(s(r,i).get()));continue}}else a.warn("unrecognized GUI edit: "+i);delete p[i],"range["===i.substr(i.length-8,6)&&(v[i.substr(0,i.length-9)]=1)}for(var m=0;m<g.length;m++){var y=g[m];if(v[y]){var b=s(t,y).get();b&&delete b.autorange}}var C=r._tracePreGUI;for(var A in C){var _,I=C[A],x=null;for(i in I){if(!x){var w=re(A,n);if(w<0){delete C[A];break}var M=ie(A,e,(_=n[w]._fullInput).index);if(M<0){delete C[A];break}x=e[M]}if(c=te(i,ee)){if(c.attr?u=(l=s(r,c.attr).get())&&ne(c.attr,t):(l=_.uirevision,void 0===(u=x.uirevision)&&(u=t.uirevision)),u&&u===l&&(null===(d=I[i])&&(d=void 0),oe(f=(h=s(x,i)).get(),d))){h.set(z(s(_,i).get()));continue}}else a.warn("unrecognized GUI edit: "+i+" in trace uid "+A);delete I[i]}}}(e.data,e.layout,u,f),h.supplyDefaults(e,{skipUpdateCalc:!0});var m=e._fullData,y=e._fullLayout,b=void 0===y.datarevision,C=y.transition,A=function(e,t,n,r,i){var o=w.layoutFlags();function a(e){return d.getLayoutValObject(n,e)}o.arrays={},o.rangesAltered={},o.nChanges=0,o.nChangesAnim=0,ae(t,n,[],{getValObject:a,flags:o,immutable:r,transition:i,gd:e}),(o.plot||o.calc)&&(o.layoutReplot=!0);i&&o.nChanges&&o.nChangesAnim&&(o.anim=o.nChanges===o.nChangesAnim?"all":"some");return o}(e,f,y,b,C),_=A.newDataRevision,M=function(e,t,n,r,i,o){var a=t.length===n.length;if(!i&&!a)return{fullReplot:!0,calc:!0};var s,l,u=w.traceFlags();u.arrays={},u.nChanges=0,u.nChangesAnim=0;var c={getValObject:function(e){var t=d.getTraceValObject(l,e);return!l._module.animatable&&t.anim&&(t.anim=!1),t},flags:u,immutable:r,transition:i,newDataRevision:o,gd:e},f={};for(s=0;s<t.length;s++)if(n[s]){if(l=n[s]._fullInput,h.hasMakesDataTransform(l)&&(l=n[s]),f[l.uid])continue;f[l.uid]=1,ae(t[s]._fullInput,l,[],c)}(u.calc||u.plot)&&(u.fullReplot=!0);i&&u.nChanges&&u.nChangesAnim&&(u.anim=u.nChanges===u.nChangesAnim&&a?"all":"some");return u}(e,u,m,b,C,_);if(Q(e)&&(A.layoutReplot=!0),M.calc||A.calc){e.calcdata=void 0;for(var S=Object.getOwnPropertyNames(y),E=0;E<S.length;E++){var O=S[E],L=O.substring(0,5);if("xaxis"===L||"yaxis"===L){var D=y[O]._emptyCategories;D&&D()}}}else h.supplyDefaultsUpdateCalc(e.calcdata,m);var P=[];if(o&&(e._transitionData={},h.createTransitionData(e),P.push((function(){return t.addFrames(e,o)}))),y.transition&&!g&&(M.anim||A.anim))A.ticks&&P.push(x.doTicksRelayout),h.doCalcdata(e),x.doAutoRangeAndConstraints(e),P.push((function(){return h.transitionFromReact(e,M,A,f)}));else if(M.fullReplot||A.layoutReplot||g)e._fullLayout._skipDefaults=!0,P.push(t.plot);else{for(var N in A.arrays){var j=A.arrays[N];if(j.length){var R=c.getComponentMethod(N,"drawOne");if(R!==a.noop)for(var F=0;F<j.length;F++)R(e,j[F]);else{var B=c.getComponentMethod(N,"draw");if(B===a.noop)throw new Error("cannot draw components: "+N);B(e)}}}P.push(h.previousPromises),M.style&&P.push(x.doTraceStyle),(M.colorbars||A.colorbars)&&P.push(x.doColorBars),A.legend&&P.push(x.doLegend),A.layoutstyle&&P.push(x.layoutStyles),A.axrange&&W(P),A.ticks&&P.push(x.doTicksRelayout),A.modebar&&P.push(x.doModeBar),A.camera&&P.push(x.doCamera),P.push(T)}P.push(h.rehover,h.redrag),(l=a.syncOrAsync(P,e))&&l.then||(l=Promise.resolve(e))}else l=t.newPlot(e,n,r,i);return l.then((function(){return e.emit("plotly_react",{data:n,layout:r}),e}))},t.redraw=function(e){if(e=a.getGraphDiv(e),!a.isPlotDiv(e))throw new Error("This element is not a Plotly plot: "+e);return I.cleanData(e.data),I.cleanLayout(e.layout),e.calcdata=void 0,t.plot(e).then((function(){return e.emit("plotly_redraw"),e}))},t.relayout=V,t.restyle=R,t.setPlotConfig=function(e){return a.extendFlat(A,e)},t.update=K,t._guiRelayout=J(V),t._guiRestyle=J(R),t._guiUpdate=J(K),t._storeDirectGUIEdit=function(e,t,n){for(var r in n){B(r,s(e,r).get(),n[r],t)}}},function(e,t,n){"use strict";(function(t){var r,i=n(981);r="function"==typeof t.matchMedia?!t.matchMedia("(hover: none)").matches:i,e.exports=r}).call(this,n(61))},function(e,t){e.exports=!0},function(e,t,n){"use strict";var r=n(10),i=n(253).dfltConfig;var o={add:function(e,t,n,r,o){var a,s;e.undoQueue=e.undoQueue||{index:0,queue:[],sequence:!1},s=e.undoQueue.index,e.autoplay?e.undoQueue.inSequence||(e.autoplay=!1):(!e.undoQueue.sequence||e.undoQueue.beginSequence?(a={undo:{calls:[],args:[]},redo:{calls:[],args:[]}},e.undoQueue.queue.splice(s,e.undoQueue.queue.length-s,a),e.undoQueue.index+=1):a=e.undoQueue.queue[s-1],e.undoQueue.beginSequence=!1,a&&(a.undo.calls.unshift(t),a.undo.args.unshift(n),a.redo.calls.push(r),a.redo.args.push(o)),e.undoQueue.queue.length>i.queueLength&&(e.undoQueue.queue.shift(),e.undoQueue.index--))},startSequence:function(e){e.undoQueue=e.undoQueue||{index:0,queue:[],sequence:!1},e.undoQueue.sequence=!0,e.undoQueue.beginSequence=!0},stopSequence:function(e){e.undoQueue=e.undoQueue||{index:0,queue:[],sequence:!1},e.undoQueue.sequence=!1,e.undoQueue.beginSequence=!1},undo:function(e){var t,n;if(e.framework&&e.framework.isPolar)e.framework.undo();else if(!(void 0===e.undoQueue||isNaN(e.undoQueue.index)||e.undoQueue.index<=0)){for(e.undoQueue.index--,t=e.undoQueue.queue[e.undoQueue.index],e.undoQueue.inSequence=!0,n=0;n<t.undo.calls.length;n++)o.plotDo(e,t.undo.calls[n],t.undo.args[n]);e.undoQueue.inSequence=!1,e.autoplay=!1}},redo:function(e){var t,n;if(e.framework&&e.framework.isPolar)e.framework.redo();else if(!(void 0===e.undoQueue||isNaN(e.undoQueue.index)||e.undoQueue.index>=e.undoQueue.queue.length)){for(t=e.undoQueue.queue[e.undoQueue.index],e.undoQueue.inSequence=!0,n=0;n<t.redo.calls.length;n++)o.plotDo(e,t.redo.calls[n],t.redo.args[n]);e.undoQueue.inSequence=!1,e.autoplay=!1,e.undoQueue.index++}}};o.plotDo=function(e,t,n){e.autoplay=!0,n=function(e,t){for(var n,i=[],o=0;o<t.length;o++)n=t[o],i[o]=n===e?n:"object"==typeof n?Array.isArray(n)?r.extendDeep([],n):r.extendDeepAll({},n):n;return i}(e,n),t.apply(null,n)},e.exports=o},function(e,t,n){"use strict";e.exports={_isLinkedToArray:"frames_entry",group:{valType:"string"},name:{valType:"string"},traces:{valType:"any"},baseframe:{valType:"string"},data:{valType:"any"},layout:{valType:"any"}}},function(e,t,n){"use strict";var r=n(63).extendFlat,i=n(161),o=n(251).scales;e.exports={editType:"calc",colorscale:{editType:"calc",sequential:{valType:"colorscale",dflt:o.Reds,editType:"calc"},sequentialminus:{valType:"colorscale",dflt:o.Blues,editType:"calc"},diverging:{valType:"colorscale",dflt:o.RdBu,editType:"calc"}},coloraxis:r({_isSubplotObj:!0,editType:"calc"},i("",{colorAttr:"corresponding trace color array(s)",noColorAxis:!0,showScaleDflt:!0}))}},function(e,t,n){"use strict";var r=n(10);e.exports=function(e){return r.isPlainObject(e.colorbar)}},function(e,t,n){"use strict";var r=n(10),i=n(64),o=n(542),a=n(988),s=n(543),l=n(541);e.exports=function(e,t,n){var u=i.newContainer(t,"colorbar"),c=e.colorbar||{};function d(e,t){return r.coerce(c,u,l,e,t)}var h=d("thicknessmode");d("thickness","fraction"===h?30/(n.width-n.margin.l-n.margin.r):30);var f=d("lenmode");d("len","fraction"===f?1:n.height-n.margin.t-n.margin.b),d("x"),d("xanchor"),d("xpad"),d("y"),d("yanchor"),d("ypad"),r.noneOrAll(c,u,["x","y"]),d("outlinecolor"),d("outlinewidth"),d("bordercolor"),d("borderwidth"),d("bgcolor");var p=d("ticklabelposition");o(c,u,d,"linear");var g={outerTicks:!1,font:n.font};-1!==p.indexOf("inside")&&(g.bgColor="black"),s(c,u,d,"linear",g),a(c,u,d,"linear",g),d("title.text",n._dfltTitle.colorbar),r.coerceFont(d,"title.font",n.font),d("title.side")}},function(e,t,n){"use strict";var r=n(42),i=n(10),o=n(72),a=o.ONEDAY,s=o.ONEWEEK;t.dtick=function(e,t){var n="log"===t,i="date"===t,o="category"===t,s=i?a:1;if(!e)return s;if(r(e))return(e=Number(e))<=0?s:o?Math.max(1,Math.round(e)):i?Math.max(.1,e):e;if("string"!=typeof e||!i&&!n)return s;var l=e.charAt(0),u=e.substr(1);return(u=r(u)?Number(u):0)<=0||!(i&&"M"===l&&u===Math.round(u)||n&&"L"===l||n&&"D"===l&&(1===u||2===u))?s:e},t.tick0=function(e,t,n,o){return"date"===t?i.cleanDate(e,i.dateTick0(n,o%s==0?1:0)):"D1"!==o&&"D2"!==o?r(e)?Number(e):0:void 0}},function(e,t,n){"use strict";var r=n(10),i=n(134);e.exports=function(e,t,n,o){var a=r.coerce2(e,t,i,"ticklen"),s=r.coerce2(e,t,i,"tickwidth"),l=r.coerce2(e,t,i,"tickcolor",t.color);n("ticks",o.outerTicks||a||s||l?"outside":"")||(delete t.ticklen,delete t.tickwidth,delete t.tickcolor)}},function(e,t,n){"use strict";var r=n(42);e.exports=function(e){var t=e.marker,n=t.sizeref||1,i=t.sizemin||0,o="area"===t.sizemode?function(e){return Math.sqrt(e/n)}:function(e){return e/n};return function(e){var t=o(e/2);return r(t)&&t>0?Math.max(t,i):0}}},function(e,t,n){var r=n(39),i=n(10).extendDeepAll,o=n(97).MID_SHIFT,a=e.exports={version:"0.2.2"};a.Axis=function(){var e,t,n,s,l={data:[],layout:{}},u={},c={},d=r.dispatch("hover"),h={};return h.render=function(u){return function(u){t=u||t;var d=l.data,h=l.layout;("string"==typeof t||t.nodeName)&&(t=r.select(t)),t.datum(d).each((function(t,l){var u=t.slice();c={data:a.util.cloneJson(u),layout:a.util.cloneJson(h)};var d=0;u.forEach((function(e,t){e.color||(e.color=h.defaultColorRange[d],d=(d+1)%h.defaultColorRange.length),e.strokeColor||(e.strokeColor="LinePlot"===e.geometry?e.color:r.rgb(e.color).darker().toString()),c.data[t].color=e.color,c.data[t].strokeColor=e.strokeColor,c.data[t].strokeDash=e.strokeDash,c.data[t].strokeSize=e.strokeSize}));var f=u.filter((function(e,t){var n=e.visible;return void 0===n||!0===n})),p=!1,g=f.map((function(e,t){return p=p||void 0!==e.groupId,e}));if(p){var v=r.nest().key((function(e,t){return void 0!==e.groupId?e.groupId:"unstacked"})).entries(g),m=[],y=v.map((function(e,t){if("unstacked"===e.key)return e.values;var n=e.values[0].r.map((function(e,t){return 0}));return e.values.forEach((function(e,t,r){e.yStack=[n],m.push(n),n=a.util.sumArrays(e.r,n)})),e.values}));f=r.merge(y)}f.forEach((function(e,t){e.t=Array.isArray(e.t[0])?e.t:[e.t],e.r=Array.isArray(e.r[0])?e.r:[e.r]}));var b=Math.min(h.width-h.margin.left-h.margin.right,h.height-h.margin.top-h.margin.bottom)/2;b=Math.max(10,b);var C,A=[h.margin.left+b,h.margin.top+b];p?C=[0,r.max(a.util.sumArrays(a.util.arrayLast(f).r[0],a.util.arrayLast(m)))]:C=r.extent(a.util.flattenArray(f.map((function(e,t){return e.r}))));h.radialAxis.domain!=a.DATAEXTENT&&(C[0]=0),n=r.scale.linear().domain(h.radialAxis.domain!=a.DATAEXTENT&&h.radialAxis.domain?h.radialAxis.domain:C).range([0,b]),c.layout.radialAxis.domain=n.domain();var _,I=a.util.flattenArray(f.map((function(e,t){return e.t}))),x="string"==typeof I[0];x&&(I=a.util.deduplicate(I),_=I.slice(),I=r.range(I.length),f=f.map((function(e,t){var n=e;return e.t=[I],p&&(n.yStack=e.yStack),n})));var w=f.filter((function(e,t){return"LinePlot"===e.geometry||"DotPlot"===e.geometry})).length===f.length,M=null===h.needsEndSpacing?x||!w:h.needsEndSpacing,S=h.angularAxis.domain&&h.angularAxis.domain!=a.DATAEXTENT&&!x&&h.angularAxis.domain[0]>=0?h.angularAxis.domain:r.extent(I),T=Math.abs(I[1]-I[0]);w&&!x&&(T=0);var E=S.slice();M&&x&&(E[1]+=T);var O=h.angularAxis.ticksCount||4;O>8&&(O=O/(O/8)+O%8),h.angularAxis.ticksStep&&(O=(E[1]-E[0])/O);var k=h.angularAxis.ticksStep||(E[1]-E[0])/(O*(h.minorTicks+1));_&&(k=Math.max(Math.round(k),1)),E[2]||(E[2]=k);var L=r.range.apply(this,E);if(L=L.map((function(e,t){return parseFloat(e.toPrecision(12))})),s=r.scale.linear().domain(E.slice(0,2)).range("clockwise"===h.direction?[0,360]:[360,0]),c.layout.angularAxis.domain=s.domain(),c.layout.angularAxis.endPadding=M?T:0,void 0===(e=r.select(this).select("svg.chart-root"))||e.empty()){var D=(new DOMParser).parseFromString("<svg xmlns='http://www.w3.org/2000/svg' class='chart-root'>' + '<g class='outer-group'>' + '<g class='chart-group'>' + '<circle class='background-circle'></circle>' + '<g class='geometry-group'></g>' + '<g class='radial axis-group'>' + '<circle class='outside-circle'></circle>' + '</g>' + '<g class='angular axis-group'></g>' + '<g class='guides-group'><line></line><circle r='0'></circle></g>' + '</g>' + '<g class='legend-group'></g>' + '<g class='tooltips-group'></g>' + '<g class='title-group'><text></text></g>' + '</g>' + '</svg>","application/xml"),P=this.appendChild(this.ownerDocument.importNode(D.documentElement,!0));e=r.select(P)}e.select(".guides-group").style({"pointer-events":"none"}),e.select(".angular.axis-group").style({"pointer-events":"none"}),e.select(".radial.axis-group").style({"pointer-events":"none"});var N,j=e.select(".chart-group"),R={fill:"none",stroke:h.tickColor},z={"font-size":h.font.size,"font-family":h.font.family,fill:h.font.color,"text-shadow":["-1px 0px","1px -1px","-1px 1px","1px 1px"].map((function(e,t){return" "+e+" 0 "+h.font.outlineColor})).join(",")};if(h.showLegend){N=e.select(".legend-group").attr({transform:"translate("+[b,h.margin.top]+")"}).style({display:"block"});var F=f.map((function(e,t){var n=a.util.cloneJson(e);return n.symbol="DotPlot"===e.geometry?e.dotType||"circle":"LinePlot"!=e.geometry?"square":"line",n.visibleInLegend=void 0===e.visibleInLegend||e.visibleInLegend,n.color="LinePlot"===e.geometry?e.strokeColor:e.color,n}));a.Legend().config({data:f.map((function(e,t){return e.name||"Element"+t})),legendConfig:i({},a.Legend.defaultConfig().legendConfig,{container:N,elements:F,reverseOrder:h.legend.reverseOrder})})();var B=N.node().getBBox();b=Math.min(h.width-B.width-h.margin.left-h.margin.right,h.height-h.margin.top-h.margin.bottom)/2,b=Math.max(10,b),A=[h.margin.left+b,h.margin.top+b],n.range([0,b]),c.layout.radialAxis.domain=n.domain(),N.attr("transform","translate("+[A[0]+b,A[1]-b]+")")}else N=e.select(".legend-group").style({display:"none"});e.attr({width:h.width,height:h.height}).style({opacity:h.opacity}),j.attr("transform","translate("+A+")").style({cursor:"crosshair"});var H=[(h.width-(h.margin.left+h.margin.right+2*b+(B?B.width:0)))/2,(h.height-(h.margin.top+h.margin.bottom+2*b))/2];if(H[0]=Math.max(0,H[0]),H[1]=Math.max(0,H[1]),e.select(".outer-group").attr("transform","translate("+H+")"),h.title&&h.title.text){var U=e.select("g.title-group text").style(z).text(h.title.text),V=U.node().getBBox();U.attr({x:A[0]-V.width/2,y:A[1]-b-20})}var Y=e.select(".radial.axis-group");if(h.radialAxis.gridLinesVisible){var W=Y.selectAll("circle.grid-circle").data(n.ticks(5));W.enter().append("circle").attr({class:"grid-circle"}).style(R),W.attr("r",n),W.exit().remove()}Y.select("circle.outside-circle").attr({r:b}).style(R);var G=e.select("circle.background-circle").attr({r:b}).style({fill:h.backgroundColor,stroke:h.stroke});function X(e,t){return s(e)%360+h.orientation}if(h.radialAxis.visible){var q=r.svg.axis().scale(n).ticks(5).tickSize(5);Y.call(q).attr({transform:"rotate("+h.radialAxis.orientation+")"}),Y.selectAll(".domain").style(R),Y.selectAll("g>text").text((function(e,t){return this.textContent+h.radialAxis.ticksSuffix})).style(z).style({"text-anchor":"start"}).attr({x:0,y:0,dx:0,dy:0,transform:function(e,t){return"horizontal"===h.radialAxis.tickOrientation?"rotate("+-h.radialAxis.orientation+") translate("+[0,z["font-size"]]+")":"translate("+[0,z["font-size"]]+")"}}),Y.selectAll("g>line").style({stroke:"black"})}var Z=e.select(".angular.axis-group").selectAll("g.angular-tick").data(L),Q=Z.enter().append("g").classed("angular-tick",!0);Z.attr({transform:function(e,t){return"rotate("+X(e)+")"}}).style({display:h.angularAxis.visible?"block":"none"}),Z.exit().remove(),Q.append("line").classed("grid-line",!0).classed("major",(function(e,t){return t%(h.minorTicks+1)==0})).classed("minor",(function(e,t){return!(t%(h.minorTicks+1)==0)})).style(R),Q.selectAll(".minor").style({stroke:h.minorTickColor}),Z.select("line.grid-line").attr({x1:h.tickLength?b-h.tickLength:0,x2:b}).style({display:h.angularAxis.gridLinesVisible?"block":"none"}),Q.append("text").classed("axis-text",!0).style(z);var K=Z.select("text.axis-text").attr({x:b+h.labelOffset,dy:o+"em",transform:function(e,t){var n=X(e),r=b+h.labelOffset,i=h.angularAxis.tickOrientation;return"horizontal"==i?"rotate("+-n+" "+r+" 0)":"radial"==i?n<270&&n>90?"rotate(180 "+r+" 0)":null:"rotate("+(n<=180&&n>0?-90:90)+" "+r+" 0)"}}).style({"text-anchor":"middle",display:h.angularAxis.labelsVisible?"block":"none"}).text((function(e,t){return t%(h.minorTicks+1)!=0?"":_?_[e]+h.angularAxis.ticksSuffix:e+h.angularAxis.ticksSuffix})).style(z);h.angularAxis.rewriteTicks&&K.text((function(e,t){return t%(h.minorTicks+1)!=0?"":h.angularAxis.rewriteTicks(this.textContent,t)}));var J=r.max(j.selectAll(".angular-tick text")[0].map((function(e,t){return e.getCTM().e+e.getBBox().width})));N.attr({transform:"translate("+[b+J,h.margin.top]+")"});var $=e.select("g.geometry-group").selectAll("g").size()>0,ee=e.select("g.geometry-group").selectAll("g.geometry").data(f);if(ee.enter().append("g").attr({class:function(e,t){return"geometry geometry"+t}}),ee.exit().remove(),f[0]||$){var te=[];f.forEach((function(e,t){var r={};r.radialScale=n,r.angularScale=s,r.container=ee.filter((function(e,n){return n==t})),r.geometry=e.geometry,r.orientation=h.orientation,r.direction=h.direction,r.index=t,te.push({data:e,geometryConfig:r})}));var ne=r.nest().key((function(e,t){return void 0!==e.data.groupId||"unstacked"})).entries(te),re=[];ne.forEach((function(e,t){"unstacked"===e.key?re=re.concat(e.values.map((function(e,t){return[e]}))):re.push(e.values)})),re.forEach((function(e,t){var n;n=Array.isArray(e)?e[0].geometryConfig.geometry:e.geometryConfig.geometry;var r=e.map((function(e,t){return i(a[n].defaultConfig(),e)}));a[n]().config(r)()}))}var ie,oe,ae=e.select(".guides-group"),se=e.select(".tooltips-group"),le=a.tooltipPanel().config({container:se,fontSize:8})(),ue=a.tooltipPanel().config({container:se,fontSize:8})(),ce=a.tooltipPanel().config({container:se,hasTick:!0})();if(!x){var de=ae.select("line").attr({x1:0,y1:0,y2:0}).style({stroke:"grey","pointer-events":"none"});j.on("mousemove.angular-guide",(function(e,t){var n=a.util.getMousePos(G).angle;de.attr({x2:-b,transform:"rotate("+n+")"}).style({opacity:.5});var r=(n+180+360-h.orientation)%360;ie=s.invert(r);var i=a.util.convertToCartesian(b+12,n+180);le.text(a.util.round(ie)).move([i[0]+A[0],i[1]+A[1]])})).on("mouseout.angular-guide",(function(e,t){ae.select("line").style({opacity:0})}))}var he=ae.select("circle").style({stroke:"grey",fill:"none"});j.on("mousemove.radial-guide",(function(e,t){var r=a.util.getMousePos(G).radius;he.attr({r:r}).style({opacity:.5}),oe=n.invert(a.util.getMousePos(G).radius);var i=a.util.convertToCartesian(r,h.radialAxis.orientation);ue.text(a.util.round(oe)).move([i[0]+A[0],i[1]+A[1]])})).on("mouseout.radial-guide",(function(e,t){he.style({opacity:0}),ce.hide(),le.hide(),ue.hide()})),e.selectAll(".geometry-group .mark").on("mouseover.tooltip",(function(t,n){var i=r.select(this),o=this.style.fill,s="black",l=this.style.opacity||1;if(i.attr({"data-opacity":l}),o&&"none"!==o){i.attr({"data-fill":o}),s=r.hsl(o).darker().toString(),i.style({fill:s,opacity:1});var u={t:a.util.round(t[0]),r:a.util.round(t[1])};x&&(u.t=_[t[0]]);var c="t: "+u.t+", r: "+u.r,d=this.getBoundingClientRect(),h=e.node().getBoundingClientRect(),f=[d.left+d.width/2-H[0]-h.left,d.top+d.height/2-H[1]-h.top];ce.config({color:s}).text(c),ce.move(f)}else o=this.style.stroke||"black",i.attr({"data-stroke":o}),s=r.hsl(o).darker().toString(),i.style({stroke:s,opacity:1})})).on("mousemove.tooltip",(function(e,t){if(0!=r.event.which)return!1;r.select(this).attr("data-fill")&&ce.show()})).on("mouseout.tooltip",(function(e,t){ce.hide();var n=r.select(this),i=n.attr("data-fill");i?n.style({fill:i,opacity:n.attr("data-opacity")}):n.style({stroke:n.attr("data-stroke"),opacity:n.attr("data-opacity")})}))}))}(u),this},h.config=function(e){if(!arguments.length)return l;var t=a.util.cloneJson(e);return t.data.forEach((function(e,t){l.data[t]||(l.data[t]={}),i(l.data[t],a.Axis.defaultConfig().data[0]),i(l.data[t],e)})),i(l.layout,a.Axis.defaultConfig().layout),i(l.layout,t.layout),this},h.getLiveConfig=function(){return c},h.getinputConfig=function(){return u},h.radialScale=function(e){return n},h.angularScale=function(e){return s},h.svg=function(){return e},r.rebind(h,d,"on"),h},a.Axis.defaultConfig=function(e,t){return{data:[{t:[1,2,3,4],r:[10,11,12,13],name:"Line1",geometry:"LinePlot",color:null,strokeDash:"solid",strokeColor:null,strokeSize:"1",visibleInLegend:!0,opacity:1}],layout:{defaultColorRange:r.scale.category10().range(),title:null,height:450,width:500,margin:{top:40,right:40,bottom:40,left:40},font:{size:12,color:"gray",outlineColor:"white",family:"Tahoma, sans-serif"},direction:"clockwise",orientation:0,labelOffset:10,radialAxis:{domain:null,orientation:-45,ticksSuffix:"",visible:!0,gridLinesVisible:!0,tickOrientation:"horizontal",rewriteTicks:null},angularAxis:{domain:[0,360],ticksSuffix:"",visible:!0,gridLinesVisible:!0,labelsVisible:!0,tickOrientation:"horizontal",rewriteTicks:null,ticksCount:null,ticksStep:null},minorTicks:0,tickLength:null,tickColor:"silver",minorTickColor:"#eee",backgroundColor:"none",needsEndSpacing:null,showLegend:!0,legend:{reverseOrder:!1},opacity:1}}},a.util={},a.DATAEXTENT="dataExtent",a.AREA="AreaChart",a.LINE="LinePlot",a.DOT="DotPlot",a.BAR="BarChart",a.util._override=function(e,t){for(var n in e)n in t&&(t[n]=e[n])},a.util._extend=function(e,t){for(var n in e)t[n]=e[n]},a.util._rndSnd=function(){return 2*Math.random()-1+(2*Math.random()-1)+(2*Math.random()-1)},a.util.dataFromEquation2=function(e,t){var n=t||6;return r.range(0,360+n,n).map((function(t,n){var r=t*Math.PI/180;return[t,e(r)]}))},a.util.dataFromEquation=function(e,t,n){var i=t||6,o=[],a=[];r.range(0,360+i,i).forEach((function(t,n){var r=t*Math.PI/180,i=e(r);o.push(t),a.push(i)}));var s={t:o,r:a};return n&&(s.name=n),s},a.util.ensureArray=function(e,t){if(void 0===e)return null;var n=[].concat(e);return r.range(t).map((function(e,t){return n[t]||n[0]}))},a.util.fillArrays=function(e,t,n){return t.forEach((function(t,r){e[t]=a.util.ensureArray(e[t],n)})),e},a.util.cloneJson=function(e){return JSON.parse(JSON.stringify(e))},a.util.validateKeys=function(e,t){"string"==typeof t&&(t=t.split("."));var n=t.shift();return e[n]&&(!t.length||objHasKeys(e[n],t))},a.util.sumArrays=function(e,t){return r.zip(e,t).map((function(e,t){return r.sum(e)}))},a.util.arrayLast=function(e){return e[e.length-1]},a.util.arrayEqual=function(e,t){for(var n=Math.max(e.length,t.length,1);n-- >=0&&e[n]===t[n];);return-2===n},a.util.flattenArray=function(e){for(var t=[];!a.util.arrayEqual(t,e);)t=e,e=[].concat.apply([],e);return e},a.util.deduplicate=function(e){return e.filter((function(e,t,n){return n.indexOf(e)==t}))},a.util.convertToCartesian=function(e,t){var n=t*Math.PI/180;return[e*Math.cos(n),e*Math.sin(n)]},a.util.round=function(e,t){var n=t||2,r=Math.pow(10,n);return Math.round(e*r)/r},a.util.getMousePos=function(e){var t=r.mouse(e.node()),n=t[0],i=t[1],o={};return o.x=n,o.y=i,o.pos=t,o.angle=180*(Math.atan2(i,n)+Math.PI)/Math.PI,o.radius=Math.sqrt(n*n+i*i),o},a.util.duplicatesCount=function(e){for(var t,n={},r={},i=0,o=e.length;i<o;i++)(t=e[i])in n?(n[t]++,r[t]=n[t]):n[t]=1;return r},a.util.duplicates=function(e){return Object.keys(a.util.duplicatesCount(e))},a.util.translator=function(e,t,n,r){if(r){var i=n.slice();n=t,t=i}var o=t.reduce((function(e,t){if(void 0!==e)return e[t]}),e);void 0!==o&&(t.reduce((function(e,n,r){if(void 0!==e)return r===t.length-1&&delete e[n],e[n]}),e),n.reduce((function(e,t,r){return void 0===e[t]&&(e[t]={}),r===n.length-1&&(e[t]=o),e[t]}),e))},a.PolyChart=function(){var e=[a.PolyChart.defaultConfig()],t=r.dispatch("hover"),n={solid:"none",dash:[5,2],dot:[2,5]};function o(){var t=e[0].geometryConfig,i=t.container;"string"==typeof i&&(i=r.select(i)),i.datum(e).each((function(e,i){var o=!!e[0].data.yStack,a=e.map((function(e,t){return o?r.zip(e.data.t[0],e.data.r[0],e.data.yStack[0]):r.zip(e.data.t[0],e.data.r[0])})),s=t.angularScale,l=t.radialScale.domain()[0],u={bar:function(n,i,o){var a=e[o].data,l=t.radialScale(n[1])-t.radialScale(0),u=t.radialScale(n[2]||0),c=a.barWidth;r.select(this).attr({class:"mark bar",d:"M"+[[l+u,-c/2],[l+u,c/2],[u,c/2],[u,-c/2]].join("L")+"Z",transform:function(e,n){return"rotate("+(t.orientation+s(e[0]))+")"}})}};u.dot=function(n,i,o){var a=n[2]?[n[0],n[1]+n[2]]:n,s=r.svg.symbol().size(e[o].data.dotSize).type(e[o].data.dotType)(n,i);r.select(this).attr({class:"mark dot",d:s,transform:function(e,n){var r,i,o,s=(r=function(e,n){var r=t.radialScale(e[1]),i=(t.angularScale(e[0])+t.orientation)*Math.PI/180;return{r:r,t:i}}(a),i=r.r*Math.cos(r.t),o=r.r*Math.sin(r.t),{x:i,y:o});return"translate("+[s.x,s.y]+")"}})};var c=r.svg.line.radial().interpolate(e[0].data.lineInterpolation).radius((function(e){return t.radialScale(e[1])})).angle((function(e){return t.angularScale(e[0])*Math.PI/180}));u.line=function(n,i,o){var s=n[2]?a[o].map((function(e,t){return[e[0],e[1]+e[2]]})):a[o];if(r.select(this).each(u.dot).style({opacity:function(t,n){return+e[o].data.dotVisible},fill:p.stroke(n,i,o)}).attr({class:"mark dot"}),!(i>0)){var l=r.select(this.parentNode).selectAll("path.line").data([0]);l.enter().insert("path"),l.attr({class:"line",d:c(s),transform:function(e,n){return"rotate("+(t.orientation+90)+")"},"pointer-events":"none"}).style({fill:function(e,t){return p.fill(n,i,o)},"fill-opacity":0,stroke:function(e,t){return p.stroke(n,i,o)},"stroke-width":function(e,t){return p["stroke-width"](n,i,o)},"stroke-dasharray":function(e,t){return p["stroke-dasharray"](n,i,o)},opacity:function(e,t){return p.opacity(n,i,o)},display:function(e,t){return p.display(n,i,o)}})}};var d=t.angularScale.range(),h=Math.abs(d[1]-d[0])/a[0].length*Math.PI/180,f=r.svg.arc().startAngle((function(e){return-h/2})).endAngle((function(e){return h/2})).innerRadius((function(e){return t.radialScale(l+(e[2]||0))})).outerRadius((function(e){return t.radialScale(l+(e[2]||0))+t.radialScale(e[1])}));u.arc=function(e,n,i){r.select(this).attr({class:"mark arc",d:f,transform:function(e,n){return"rotate("+(t.orientation+s(e[0])+90)+")"}})};var p={fill:function(t,n,r){return e[r].data.color},stroke:function(t,n,r){return e[r].data.strokeColor},"stroke-width":function(t,n,r){return e[r].data.strokeSize+"px"},"stroke-dasharray":function(t,r,i){return n[e[i].data.strokeDash]},opacity:function(t,n,r){return e[r].data.opacity},display:function(t,n,r){return void 0===e[r].data.visible||e[r].data.visible?"block":"none"}},g=r.select(this).selectAll("g.layer").data(a);g.enter().append("g").attr({class:"layer"});var v=g.selectAll("path.mark").data((function(e,t){return e}));v.enter().append("path").attr({class:"mark"}),v.style(p).each(u[t.geometryType]),v.exit().remove(),g.exit().remove()}))}return o.config=function(t){return arguments.length?(t.forEach((function(t,n){e[n]||(e[n]={}),i(e[n],a.PolyChart.defaultConfig()),i(e[n],t)})),this):e},o.getColorScale=function(){},r.rebind(o,t,"on"),o},a.PolyChart.defaultConfig=function(){return{data:{name:"geom1",t:[[1,2,3,4]],r:[[1,2,3,4]],dotType:"circle",dotSize:64,dotVisible:!1,barWidth:20,color:"#ffa500",strokeSize:1,strokeColor:"silver",strokeDash:"solid",opacity:1,index:0,visible:!0,visibleInLegend:!0},geometryConfig:{geometry:"LinePlot",geometryType:"arc",direction:"clockwise",orientation:0,container:"body",radialScale:null,angularScale:null,colorScale:r.scale.category20()}}},a.BarChart=function(){return a.PolyChart()},a.BarChart.defaultConfig=function(){return{geometryConfig:{geometryType:"bar"}}},a.AreaChart=function(){return a.PolyChart()},a.AreaChart.defaultConfig=function(){return{geometryConfig:{geometryType:"arc"}}},a.DotPlot=function(){return a.PolyChart()},a.DotPlot.defaultConfig=function(){return{geometryConfig:{geometryType:"dot",dotType:"circle"}}},a.LinePlot=function(){return a.PolyChart()},a.LinePlot.defaultConfig=function(){return{geometryConfig:{geometryType:"line"}}},a.Legend=function(){var e=a.Legend.defaultConfig(),t=r.dispatch("hover");function n(){var t=e.legendConfig,o=e.data.map((function(e,n){return[].concat(e).map((function(e,r){var o=i({},t.elements[n]);return o.name=e,o.color=[].concat(t.elements[n].color)[r],o}))})),a=r.merge(o);a=a.filter((function(e,n){return t.elements[n]&&(t.elements[n].visibleInLegend||void 0===t.elements[n].visibleInLegend)})),t.reverseOrder&&(a=a.reverse());var s=t.container;("string"==typeof s||s.nodeName)&&(s=r.select(s));var l=a.map((function(e,t){return e.color})),u=t.fontSize,c=null==t.isContinuous?"number"==typeof a[0]:t.isContinuous,d=c?t.height:u*a.length,h=s.classed("legend-group",!0).selectAll("svg").data([0]),f=h.enter().append("svg").attr({width:300,height:d+u,xmlns:"http://www.w3.org/2000/svg","xmlns:xlink":"http://www.w3.org/1999/xlink",version:"1.1"});f.append("g").classed("legend-axis",!0),f.append("g").classed("legend-marks",!0);var p=r.range(a.length),g=r.scale[c?"linear":"ordinal"]().domain(p).range(l),v=r.scale[c?"linear":"ordinal"]().domain(p)[c?"range":"rangePoints"]([0,d]);if(c){var m=h.select(".legend-marks").append("defs").append("linearGradient").attr({id:"grad1",x1:"0%",y1:"0%",x2:"0%",y2:"100%"}).selectAll("stop").data(l);m.enter().append("stop"),m.attr({offset:function(e,t){return t/(l.length-1)*100+"%"}}).style({"stop-color":function(e,t){return e}}),h.append("rect").classed("legend-mark",!0).attr({height:t.height,width:t.colorBandWidth,fill:"url(#grad1)"})}else{var y=h.select(".legend-marks").selectAll("path.legend-mark").data(a);y.enter().append("path").classed("legend-mark",!0),y.attr({transform:function(e,t){return"translate("+[u/2,v(t)+u/2]+")"},d:function(e,t){var n,i,o,a=e.symbol;return o=3*(i=u),"line"===(n=a)?"M"+[[-i/2,-i/12],[i/2,-i/12],[i/2,i/12],[-i/2,i/12]]+"Z":-1!=r.svg.symbolTypes.indexOf(n)?r.svg.symbol().type(n).size(o)():r.svg.symbol().type("square").size(o)()},fill:function(e,t){return g(t)}}),y.exit().remove()}var b=r.svg.axis().scale(v).orient("right"),C=h.select("g.legend-axis").attr({transform:"translate("+[c?t.colorBandWidth:u,u/2]+")"}).call(b);return C.selectAll(".domain").style({fill:"none",stroke:"none"}),C.selectAll("line").style({fill:"none",stroke:c?t.textColor:"none"}),C.selectAll("text").style({fill:t.textColor,"font-size":t.fontSize}).text((function(e,t){return a[t].name})),n}return n.config=function(t){return arguments.length?(i(e,t),this):e},r.rebind(n,t,"on"),n},a.Legend.defaultConfig=function(e,t){return{data:["a","b","c"],legendConfig:{elements:[{symbol:"line",color:"red"},{symbol:"square",color:"yellow"},{symbol:"diamond",color:"limegreen"}],height:150,colorBandWidth:30,fontSize:12,container:"body",isContinuous:null,textColor:"grey",reverseOrder:!1}}},a.tooltipPanel=function(){var e,t,n,o={container:null,hasTick:!1,fontSize:12,color:"white",padding:5},s="tooltip-"+a.tooltipPanel.uid++,l=10,u=function(){var r=(e=o.container.selectAll("g."+s).data([0])).enter().append("g").classed(s,!0).style({"pointer-events":"none",display:"none"});return n=r.append("path").style({fill:"white","fill-opacity":.9}).attr({d:"M0 0"}),t=r.append("text").attr({dx:o.padding+l,dy:.3*+o.fontSize}),u};return u.text=function(i){var a=r.hsl(o.color).l,s=a>=.5?"#aaa":"white",c=a>=.5?"black":"white",d=i||"";t.style({fill:c,"font-size":o.fontSize+"px"}).text(d);var h=o.padding,f=t.node().getBBox(),p={fill:o.color,stroke:s,"stroke-width":"2px"},g=f.width+2*h+l,v=f.height+2*h;return n.attr({d:"M"+[[l,-v/2],[l,-v/4],[o.hasTick?0:l,0],[l,v/4],[l,v/2],[g,v/2],[g,-v/2]].join("L")+"Z"}).style(p),e.attr({transform:"translate("+[l,-v/2+2*h]+")"}),e.style({display:"block"}),u},u.move=function(t){if(e)return e.attr({transform:"translate("+[t[0],t[1]]+")"}).style({display:"block"}),u},u.hide=function(){if(e)return e.style({display:"none"}),u},u.show=function(){if(e)return e.style({display:"block"}),u},u.config=function(e){return i(o,e),u},u},a.tooltipPanel.uid=1,a.adapter={},a.adapter.plotly=function(){var e={convert:function(e,t){var n={};if(e.data&&(n.data=e.data.map((function(e,n){var r=i({},e);return[[r,["marker","color"],["color"]],[r,["marker","opacity"],["opacity"]],[r,["marker","line","color"],["strokeColor"]],[r,["marker","line","dash"],["strokeDash"]],[r,["marker","line","width"],["strokeSize"]],[r,["marker","symbol"],["dotType"]],[r,["marker","size"],["dotSize"]],[r,["marker","barWidth"],["barWidth"]],[r,["line","interpolation"],["lineInterpolation"]],[r,["showlegend"],["visibleInLegend"]]].forEach((function(e,n){a.util.translator.apply(null,e.concat(t))})),t||delete r.marker,t&&delete r.groupId,t?("LinePlot"===r.geometry?(r.type="scatter",!0===r.dotVisible?(delete r.dotVisible,r.mode="lines+markers"):r.mode="lines"):"DotPlot"===r.geometry?(r.type="scatter",r.mode="markers"):"AreaChart"===r.geometry?r.type="area":"BarChart"===r.geometry&&(r.type="bar"),delete r.geometry):("scatter"===r.type?"lines"===r.mode?r.geometry="LinePlot":"markers"===r.mode?r.geometry="DotPlot":"lines+markers"===r.mode&&(r.geometry="LinePlot",r.dotVisible=!0):"area"===r.type?r.geometry="AreaChart":"bar"===r.type&&(r.geometry="BarChart"),delete r.mode,delete r.type),r})),!t&&e.layout&&"stack"===e.layout.barmode)){var o=a.util.duplicates(n.data.map((function(e,t){return e.geometry})));n.data.forEach((function(e,t){var r=o.indexOf(e.geometry);-1!=r&&(n.data[t].groupId=r)}))}if(e.layout){var s=i({},e.layout);if([[s,["plot_bgcolor"],["backgroundColor"]],[s,["showlegend"],["showLegend"]],[s,["radialaxis"],["radialAxis"]],[s,["angularaxis"],["angularAxis"]],[s.angularaxis,["showline"],["gridLinesVisible"]],[s.angularaxis,["showticklabels"],["labelsVisible"]],[s.angularaxis,["nticks"],["ticksCount"]],[s.angularaxis,["tickorientation"],["tickOrientation"]],[s.angularaxis,["ticksuffix"],["ticksSuffix"]],[s.angularaxis,["range"],["domain"]],[s.angularaxis,["endpadding"],["endPadding"]],[s.radialaxis,["showline"],["gridLinesVisible"]],[s.radialaxis,["tickorientation"],["tickOrientation"]],[s.radialaxis,["ticksuffix"],["ticksSuffix"]],[s.radialaxis,["range"],["domain"]],[s.angularAxis,["showline"],["gridLinesVisible"]],[s.angularAxis,["showticklabels"],["labelsVisible"]],[s.angularAxis,["nticks"],["ticksCount"]],[s.angularAxis,["tickorientation"],["tickOrientation"]],[s.angularAxis,["ticksuffix"],["ticksSuffix"]],[s.angularAxis,["range"],["domain"]],[s.angularAxis,["endpadding"],["endPadding"]],[s.radialAxis,["showline"],["gridLinesVisible"]],[s.radialAxis,["tickorientation"],["tickOrientation"]],[s.radialAxis,["ticksuffix"],["ticksSuffix"]],[s.radialAxis,["range"],["domain"]],[s.font,["outlinecolor"],["outlineColor"]],[s.legend,["traceorder"],["reverseOrder"]],[s,["labeloffset"],["labelOffset"]],[s,["defaultcolorrange"],["defaultColorRange"]]].forEach((function(e,n){a.util.translator.apply(null,e.concat(t))})),t?(void 0!==s.tickLength&&(s.angularaxis.ticklen=s.tickLength,delete s.tickLength),s.tickColor&&(s.angularaxis.tickcolor=s.tickColor,delete s.tickColor)):(s.angularAxis&&void 0!==s.angularAxis.ticklen&&(s.tickLength=s.angularAxis.ticklen),s.angularAxis&&void 0!==s.angularAxis.tickcolor&&(s.tickColor=s.angularAxis.tickcolor)),s.legend&&"boolean"!=typeof s.legend.reverseOrder&&(s.legend.reverseOrder="normal"!=s.legend.reverseOrder),s.legend&&"boolean"==typeof s.legend.traceorder&&(s.legend.traceorder=s.legend.traceorder?"reversed":"normal",delete s.legend.reverseOrder),s.margin&&void 0!==s.margin.t){var l=["t","r","b","l","pad"],u=["top","right","bottom","left","pad"],c={};r.entries(s.margin).forEach((function(e,t){c[u[l.indexOf(e.key)]]=e.value})),s.margin=c}t&&(delete s.needsEndSpacing,delete s.minorTickColor,delete s.minorTicks,delete s.angularaxis.ticksCount,delete s.angularaxis.ticksCount,delete s.angularaxis.ticksStep,delete s.angularaxis.rewriteTicks,delete s.angularaxis.nticks,delete s.radialaxis.ticksCount,delete s.radialaxis.ticksCount,delete s.radialaxis.ticksStep,delete s.radialaxis.rewriteTicks,delete s.radialaxis.nticks),n.layout=s}return n}};return e}},function(e,t,n){"use strict";var r=n(39),i=n(179),o=n(163),a=n(214),s=n(1699).makeDragBox,l=n(95).DRAGGERSIZE;t.initInteractions=function(e){var n=e._fullLayout;if(e._context.staticPlot)r.select(e).selectAll(".drag").remove();else if(n._has("cartesian")||n._has("splom")){Object.keys(n._plots||{}).sort((function(e,t){if((n._plots[e].mainplot&&!0)===(n._plots[t].mainplot&&!0)){var r=e.split("y"),i=t.split("y");return r[0]===i[0]?Number(r[1]||1)-Number(i[1]||1):Number(r[0]||1)-Number(i[0]||1)}return n._plots[e].mainplot?1:-1})).forEach((function(t){var r=n._plots[t],a=r.xaxis,u=r.yaxis;if(!r.mainplot){var c=s(e,r,a._offset,u._offset,a._length,u._length,"ns","ew");c.onmousemove=function(n){e._fullLayout._rehover=function(){e._fullLayout._hoversubplot===t&&e._fullLayout._plots[t]&&i.hover(e,n,t)},i.hover(e,n,t),e._fullLayout._lasthover=c,e._fullLayout._hoversubplot=t},c.onmouseout=function(t){e._dragging||(e._fullLayout._hoversubplot=null,o.unhover(e,t))},e._context.showAxisDragHandles&&(s(e,r,a._offset-l,u._offset-l,l,l,"n","w"),s(e,r,a._offset+a._length,u._offset-l,l,l,"n","e"),s(e,r,a._offset-l,u._offset+u._length,l,l,"s","w"),s(e,r,a._offset+a._length,u._offset+u._length,l,l,"s","e"))}if(e._context.showAxisDragHandles){if(t===a._mainSubplot){var d=a._mainLinePosition;"top"===a.side&&(d-=l),s(e,r,a._offset+.1*a._length,d,.8*a._length,l,"","ew"),s(e,r,a._offset,d,.1*a._length,l,"","w"),s(e,r,a._offset+.9*a._length,d,.1*a._length,l,"","e")}if(t===u._mainSubplot){var h=u._mainLinePosition;"right"!==u.side&&(h-=l),s(e,r,h,u._offset+.1*u._length,l,.8*u._length,"ns",""),s(e,r,h,u._offset+.9*u._length,l,.1*u._length,"s",""),s(e,r,h,u._offset,l,.1*u._length,"n","")}}}));var a=n._hoverlayer.node();a.onmousemove=function(t){t.target=e._fullLayout._lasthover,i.hover(e,t,n._hoversubplot)},a.onclick=function(t){t.target=e._fullLayout._lasthover,i.click(e,t)},a.onmousedown=function(t){e._fullLayout._lasthover.onmousedown(t)},t.updateFx(e)}},t.updateFx=function(e){var t=e._fullLayout,n="pan"===t.dragmode?"move":"crosshair";a(t._draggers,n)}},function(e,t){var n={left:0,top:0};e.exports=function(e,t,r){t=t||e.currentTarget||e.srcElement,Array.isArray(r)||(r=[0,0]);var i=e.clientX||0,o=e.clientY||0,a=(s=t,s===window||s===document||s===document.body?n:s.getBoundingClientRect());var s;return r[0]=i-a.left,r[1]=o-a.top,r}},function(e,t,n){"use strict";var r=n(39),i=n(42),o=n(123),a=n(10),s=a.strTranslate,l=a.strRotate,u=n(383),c=n(96),d=n(1690),h=n(56),f=n(41),p=n(163),g=n(55),v=n(27),m=n(162),y=n(381),b=n(994),C=n(996),A=y.YANGLE,_=Math.PI*A/180,I=1/Math.sin(_),x=Math.cos(_),w=Math.sin(_),M=y.HOVERARROWSIZE,S=y.HOVERTEXTPAD;function T(e){return[e.trace.index,e.index,e.x0,e.y0,e.name,e.attr,e.xa,e.ya||""].join(",")}t.hover=function(e,t,n,o){e=a.getGraphDiv(e),a.throttle(e._fullLayout._uid+y.HOVERID,y.HOVERMINTIME,(function(){!function(e,t,n,o){n||(n="xy");var s=Array.isArray(n)?n:[n],l=e._fullLayout,c=l._plots||[],h=c[n],g=l._has("cartesian");if(h){var y=h.overlays.map((function(e){return e.id}));s=s.concat(y)}for(var b=s.length,C=new Array(b),A=new Array(b),_=!1,x=0;x<b;x++){var w=s[x];if(c[w])_=!0,C[x]=c[w].xaxis,A[x]=c[w].yaxis;else{if(!l[w]||!l[w]._subplot)return void a.warn("Unrecognized subplot: "+w);var M=l[w]._subplot;C[x]=M.xaxis,A[x]=M.yaxis}}var S=t.hovermode||l.hovermode;S&&!_&&(S="closest");if(-1===["x","y","closest","x unified","y unified"].indexOf(S)||!e.calcdata||e.querySelector(".zoombox")||e._dragging)return p.unhoverRaw(e,t);var E,k,j,R,z,F,B,H,U,V,Y,W,G,X=-1===l.hoverdistance?1/0:l.hoverdistance,q=-1===l.spikedistance?1/0:l.spikedistance,Z=[],Q=[],K={hLinePoint:null,vLinePoint:null},J=!1;if(Array.isArray(t))for(S="array",j=0;j<t.length;j++)(z=e.calcdata[t[j].curveNumber||0])&&(F=z[0].trace,"skip"!==z[0].trace.hoverinfo&&(Q.push(z),"h"===F.orientation&&(J=!0)));else{for(R=0;R<e.calcdata.length;R++)z=e.calcdata[R],"skip"!==(F=z[0].trace).hoverinfo&&m.isTraceInSubplots(F,s)&&(Q.push(z),"h"===F.orientation&&(J=!0));var $,ee;if(!t.target)$="xpx"in t?t.xpx:C[0]._length/2,ee="ypx"in t?t.ypx:A[0]._length/2;else{if(!1===u.triggerHandler(e,"plotly_beforehover",t))return;var te=t.target.getBoundingClientRect();$=t.clientX-te.left,ee=t.clientY-te.top,l._calcInverseTransform(e);var ne=a.apply3DTransform(l._invTransform)($,ee);if($=ne[0],ee=ne[1],$<0||$>C[0]._length||ee<0||ee>A[0]._length)return p.unhoverRaw(e,t)}if(t.pointerX=$+C[0]._offset,t.pointerY=ee+A[0]._offset,E="xval"in t?m.flat(s,t.xval):m.p2c(C,$),k="yval"in t?m.flat(s,t.yval):m.p2c(A,ee),!i(E[0])||!i(k[0]))return a.warn("Fx.hover failed",t,e),p.unhoverRaw(e,t)}var re=1/0;function ie(e,n){for(R=0;R<Q.length;R++)if((z=Q[R])&&z[0]&&z[0].trace&&!0===(F=z[0].trace).visible&&0!==F._length&&-1===["carpet","contourcarpet"].indexOf(F._module.name)){if("splom"===F.type?B=s[H=0]:(B=m.getSubplot(F),H=s.indexOf(B)),U=S,m.isUnifiedHover(U)&&(U=U.charAt(0)),W={cd:z,trace:F,xa:C[H],ya:A[H],maxHoverDistance:X,maxSpikeDistance:q,index:!1,distance:Math.min(re,X),spikeDistance:1/0,xSpike:void 0,ySpike:void 0,color:f.defaultLine,name:F.name,x0:void 0,x1:void 0,y0:void 0,y1:void 0,xLabelVal:void 0,yLabelVal:void 0,zLabelVal:void 0,text:void 0},l[B]&&(W.subplot=l[B]._subplot),l._splomScenes&&l._splomScenes[F.uid]&&(W.scene=l._splomScenes[F.uid]),G=Z.length,"array"===U){var r=t[R];"pointNumber"in r?(W.index=r.pointNumber,U="closest"):(U="","xval"in r&&(V=r.xval,U="x"),"yval"in r&&(Y=r.yval,U=U?"closest":"y"))}else void 0!==e&&void 0!==n?(V=e,Y=n):(V=E[H],Y=k[H]);if(0!==X)if(F._module&&F._module.hoverPoints){var o=F._module.hoverPoints(W,V,Y,U,l._hoverlayer);if(o)for(var u,c=0;c<o.length;c++)u=o[c],i(u.x0)&&i(u.y0)&&Z.push(D(u,S))}else a.log("Unrecognized trace type in hover:",F);if("closest"===S&&Z.length>G&&(Z.splice(0,G),re=Z[0].distance),g&&0!==q&&0===Z.length){W.distance=q,W.index=!1;var d=F._module.hoverPoints(W,V,Y,"closest",l._hoverlayer);if(d&&(d=d.filter((function(e){return e.spikeDistance<=q}))),d&&d.length){var h,p=d.filter((function(e){return e.xa.showspikes&&"hovered data"!==e.xa.spikesnap}));if(p.length){var v=p[0];i(v.x0)&&i(v.y0)&&(h=ae(v),(!K.vLinePoint||K.vLinePoint.spikeDistance>h.spikeDistance)&&(K.vLinePoint=h))}var y=d.filter((function(e){return e.ya.showspikes&&"hovered data"!==e.ya.spikesnap}));if(y.length){var b=y[0];i(b.x0)&&i(b.y0)&&(h=ae(b),(!K.hLinePoint||K.hLinePoint.spikeDistance>h.spikeDistance)&&(K.hLinePoint=h))}}}}}function oe(e,t){for(var n,r=null,i=1/0,o=0;o<e.length;o++)(n=e[o].spikeDistance)<=i&&n<=t&&(r=e[o],i=n);return r}function ae(e){return e?{xa:e.xa,ya:e.ya,x:void 0!==e.xSpike?e.xSpike:(e.x0+e.x1)/2,y:void 0!==e.ySpike?e.ySpike:(e.y0+e.y1)/2,distance:e.distance,spikeDistance:e.spikeDistance,curveNumber:e.trace.index,color:e.color,pointNumber:e.index}:null}ie();var se={fullLayout:l,container:l._hoverlayer,outerContainer:l._paperdiv,event:t},le=e._spikepoints,ue={vLinePoint:K.vLinePoint,hLinePoint:K.hLinePoint};if(e._spikepoints=ue,g&&0!==q&&0!==Z.length){var ce=oe(Z.filter((function(e){return e.ya.showspikes})),q);K.hLinePoint=ae(ce);var de=oe(Z.filter((function(e){return e.xa.showspikes})),q);K.vLinePoint=ae(de)}if(0===Z.length){var he=p.unhoverRaw(e,t);return!g||null===K.hLinePoint&&null===K.vLinePoint||N(le)&&P(e,K,se),he}g&&N(le)&&P(e,K,se);if(Z.sort((function(e,t){return e.distance-t.distance})),m.isXYhover(U)&&0!==Z[0].length&&"splom"!==Z[0].trace.type){var fe=Z[0],pe=fe.cd[fe.index],ge="group"===l.boxmode||"group"===l.violinmode,ve=fe.xVal,me=fe.xa;"category"===me.type&&(ve=me._categoriesMap[ve]),"date"===me.type&&(ve=me.d2c(ve)),pe&&pe.t&&pe.t.posLetter===me._id&&ge&&(ve+=pe.t.dPos);var ye=fe.yVal;"category"===(me=fe.ya).type&&(ye=me._categoriesMap[ye]),"date"===me.type&&(ye=me.d2c(ye)),pe&&pe.t&&pe.t.posLetter===me._id&&ge&&(ye+=pe.t.dPos),ie(ve,ye);var be={};Z=Z.filter((function(e){var t=T(e);if(!be[t])return be[t]=!0,be[t]}))}var Ce=e._hoverdata,Ae=[];for(j=0;j<Z.length;j++){var _e=Z[j],Ie=m.makeEventData(_e,_e.trace,_e.cd);if(!1!==_e.hovertemplate){var xe=!1;_e.cd[_e.index]&&_e.cd[_e.index].ht&&(xe=_e.cd[_e.index].ht),_e.hovertemplate=xe||_e.trace.hovertemplate||!1}_e.eventData=[Ie],Ae.push(Ie)}e._hoverdata=Ae;var we="y"===S&&(Q.length>1||Z.length>1)||"closest"===S&&J&&Z.length>1,Me=f.combine(l.plot_bgcolor||f.background,l.paper_bgcolor),Se={hovermode:S,rotateLabels:we,bgColor:Me,container:l._hoverlayer,outerContainer:l._paperdiv,commonLabelOpts:l.hoverlabel,hoverdistance:l.hoverdistance},Te=O(Z,Se,e);m.isUnifiedHover(S)||(!function(e,t,n){var r,i,o,a,s,l,u,c=0,d=1,h=e.size(),f=new Array(h),p=0;function g(e){var t=e[0],n=e[e.length-1];if(i=t.pmin-t.pos-t.dp+t.size,o=n.pos+n.dp+n.size-t.pmax,i>.01){for(s=e.length-1;s>=0;s--)e[s].dp+=i;r=!1}if(!(o<.01)){if(i<-.01){for(s=e.length-1;s>=0;s--)e[s].dp-=o;r=!1}if(r){var u=0;for(a=0;a<e.length;a++)(l=e[a]).pos+l.dp+l.size>t.pmax&&u++;for(a=e.length-1;a>=0&&!(u<=0);a--)(l=e[a]).pos>t.pmax-1&&(l.del=!0,u--);for(a=0;a<e.length&&!(u<=0);a++)if((l=e[a]).pos<t.pmin+1)for(l.del=!0,u--,o=2*l.size,s=e.length-1;s>=0;s--)e[s].dp-=o;for(a=e.length-1;a>=0&&!(u<=0);a--)(l=e[a]).pos+l.dp+l.size>t.pmax&&(l.del=!0,u--)}}}e.each((function(e){var r=e[t],i="x"===r._id.charAt(0),o=r.range;0===p&&o&&o[0]>o[1]!==i&&(d=-1),f[p++]=[{datum:e,traceIndex:e.trace.index,dp:0,pos:e.pos,posref:e.posref,size:e.by*(i?I:1)/2,pmin:0,pmax:i?n.width:n.height}]})),f.sort((function(e,t){return e[0].posref-t[0].posref||d*(t[0].traceIndex-e[0].traceIndex)}));for(;!r&&c<=h;){for(c++,r=!0,a=0;a<f.length-1;){var v=f[a],m=f[a+1],y=v[v.length-1],b=m[0];if((i=y.pos+y.dp+y.size-b.pos-b.dp+b.size)>.01&&y.pmin===b.pmin&&y.pmax===b.pmax){for(s=m.length-1;s>=0;s--)m[s].dp+=i;for(v.push.apply(v,m),f.splice(a+1,1),u=0,s=v.length-1;s>=0;s--)u+=v[s].dp;for(o=u/v.length,s=v.length-1;s>=0;s--)v[s].dp-=o;r=!1}else a++}f.forEach(g)}for(a=f.length-1;a>=0;a--){var C=f[a];for(s=C.length-1;s>=0;s--){var A=C[s],_=A.datum;_.offset=A.dp,_.del=A.del}}}(Te,we?"xa":"ya",l),L(Te,we,l._invScaleX,l._invScaleY));if(t.target&&t.target.tagName){var Ee=v.getComponentMethod("annotations","hasClickToShow")(e,Ae);d(r.select(t.target),Ee?"pointer":"")}if(!t.target||o||!function(e,t,n){if(!n||n.length!==e._hoverdata.length)return!0;for(var r=n.length-1;r>=0;r--){var i=n[r],o=e._hoverdata[r];if(i.curveNumber!==o.curveNumber||String(i.pointNumber)!==String(o.pointNumber)||String(i.pointNumbers)!==String(o.pointNumbers))return!0}return!1}(e,0,Ce))return;Ce&&e.emit("plotly_unhover",{event:t,points:Ce});e.emit("plotly_hover",{event:t,points:e._hoverdata,xaxes:C,yaxes:A,xvals:E,yvals:k})}(e,t,n,o)}))},t.loneHover=function(e,t){var n=!0;Array.isArray(e)||(n=!1,e=[e]);var i=e.map((function(e){return{color:e.color||f.defaultLine,x0:e.x0||e.x||0,x1:e.x1||e.x||0,y0:e.y0||e.y||0,y1:e.y1||e.y||0,xLabel:e.xLabel,yLabel:e.yLabel,zLabel:e.zLabel,text:e.text,name:e.name,idealAlign:e.idealAlign,borderColor:e.borderColor,fontFamily:e.fontFamily,fontSize:e.fontSize,fontColor:e.fontColor,nameLength:e.nameLength,textAlign:e.textAlign,trace:e.trace||{index:0,hoverinfo:""},xa:{_offset:0},ya:{_offset:0},index:0,hovertemplate:e.hovertemplate||!1,eventData:e.eventData||!1,hovertemplateLabels:e.hovertemplateLabels||!1}})),o=r.select(t.container),a=t.outerContainer?r.select(t.outerContainer):o,s={hovermode:"closest",rotateLabels:!1,bgColor:t.bgColor||f.background,container:o,outerContainer:a},l=O(i,s,t.gd),u=0,c=0;l.sort((function(e,t){return e.y0-t.y0})).each((function(e,n){var r=e.y0-e.by/2;e.offset=r-5<u?u-r+5:0,u=r+e.by+e.offset,n===t.anchorIndex&&(c=e.offset)})).each((function(e){e.offset-=c}));var d=t.gd._fullLayout._invScaleX,h=t.gd._fullLayout._invScaleY;return L(l,s.rotateLabels,d,h),n?l:l.node()};var E=/<extra>([\s\S]*)<\/extra>/;function O(e,t,n){var i=n._fullLayout,o=t.hovermode,u=t.rotateLabels,d=t.bgColor,p=t.container,g=t.outerContainer,v=t.commonLabelOpts||{},_=t.fontFamily||y.HOVERFONT,I=t.fontSize||y.HOVERFONTSIZE,x=e[0],w=x.xa,E=x.ya,O="y"===o.charAt(0)?"yLabel":"xLabel",L=x[O],D=(String(L)||"").split(" ")[0],P=g.node().getBoundingClientRect(),N=P.top,j=P.width,R=P.height,z=void 0!==L&&x.distance<=t.hoverdistance&&("x"===o||"y"===o);if(z){var F,B,H=!0;for(F=0;F<e.length;F++)if(H&&void 0===e[F].zLabel&&(H=!1),B=e[F].hoverinfo||e[F].trace.hoverinfo){var U=Array.isArray(B)?B:B.split("+");if(-1===U.indexOf("all")&&-1===U.indexOf(o)){z=!1;break}}H&&(z=!1)}var V=p.selectAll("g.axistext").data(z?[0]:[]);function Y(e){return e.filter((function(e){return void 0!==e.zLabelVal||(e[O]||"").split(" ")[0]===D}))}if(V.enter().append("g").classed("axistext",!0),V.exit().remove(),V.each((function(){var t=r.select(this),l=a.ensureSingle(t,"path","",(function(e){e.style({"stroke-width":"1px"})})),u=a.ensureSingle(t,"text","",(function(e){e.attr("data-notex",1)})),d=v.bgcolor||f.defaultLine,p=v.bordercolor||f.contrast(d),g=f.contrast(d),m={family:v.font.family||_,size:v.font.size||I,color:v.font.color||g};l.style({fill:d,stroke:p}),u.text(L).call(h.font,m).call(c.positionText,0,0).call(c.convertToTspans,n),t.attr("transform","");var y,b,C=u.node().getBoundingClientRect();if("x"===o){var A="top"===w.side?"-":"";u.attr("text-anchor","middle").call(c.positionText,0,"top"===w.side?N-C.bottom-M-S:N-C.top+M+S),y=w._offset+(x.x0+x.x1)/2,b=E._offset+("top"===w.side?0:E._length);var T=C.width/2+S;y<T?(y=T,l.attr("d","M-"+(T-M)+",0L-"+(T-2*M)+","+A+M+"H"+(S+C.width/2)+"v"+A+(2*S+C.height)+"H-"+T+"V"+A+M+"Z")):y>i.width-T?(y=i.width-T,l.attr("d","M"+(T-M)+",0L"+T+","+A+M+"v"+A+(2*S+C.height)+"H-"+T+"V"+A+M+"H"+(T-2*M)+"Z")):l.attr("d","M0,0L"+M+","+A+M+"H"+(S+C.width/2)+"v"+A+(2*S+C.height)+"H-"+(S+C.width/2)+"V"+A+M+"H-"+M+"Z")}else{var O,k,D;"right"===E.side?(O="start",k=1,D="",y=w._offset+w._length):(O="end",k=-1,D="-",y=w._offset),b=E._offset+(x.y0+x.y1)/2,u.attr("text-anchor",O),l.attr("d","M0,0L"+D+M+","+M+"V"+(S+C.height/2)+"h"+D+(2*S+C.width)+"V-"+(S+C.height/2)+"H"+D+M+"V-"+M+"Z");var P,j=C.height/2,R=N-C.top-j,z="clip"+i._uid+"commonlabel"+E._id;if(y<C.width+2*S+M){P="M-"+(M+S)+"-"+j+"h-"+(C.width-S)+"V"+j+"h"+(C.width-S)+"Z";var F=C.width-y+S;c.positionText(u,F,R),"end"===O&&u.selectAll("tspan").each((function(){var e=r.select(this),t=h.tester.append("text").text(e.text()).call(h.font,m),n=t.node().getBoundingClientRect();Math.round(n.width)<Math.round(C.width)&&e.attr("x",F-n.width),t.remove()}))}else c.positionText(u,k*(S+M),R),P=null;var B=i._topclips.selectAll("#"+z).data(P?[0]:[]);B.enter().append("clipPath").attr("id",z).append("path"),B.exit().remove(),B.select("path").attr("d",P),h.setClipUrl(u,P?z:null,n)}t.attr("transform",s(y,b)),e=Y(e)})),m.isUnifiedHover(o)){if(p.selectAll("g.hovertext").remove(),void 0!==L&&x.distance<=t.hoverdistance&&(e=Y(e)),0===e.length)return;var W={showlegend:!0,legend:{title:{text:L,font:i.hoverlabel.font},font:i.hoverlabel.font,bgcolor:i.hoverlabel.bgcolor,bordercolor:i.hoverlabel.bordercolor,borderwidth:1,tracegroupgap:7,traceorder:i.legend?i.legend.traceorder:void 0,orientation:"v"}},G={};b(W,G,n._fullData);var X=G.legend;X.entries=[];for(var q=0;q<e.length;q++){var Z=k(e[q],!0,o,i,L),Q=Z[0],K=Z[1],J=e[q];J.name=K,J.text=""!==K?K+" : "+Q:Q;var $=J.cd[J.index];$&&($.mc&&(J.mc=$.mc),$.mcc&&(J.mc=$.mcc),$.mlc&&(J.mlc=$.mlc),$.mlcc&&(J.mlc=$.mlcc),$.mlw&&(J.mlw=$.mlw),$.mrc&&(J.mrc=$.mrc),$.dir&&(J.dir=$.dir)),J._distinct=!0,X.entries.push([J])}X.entries.sort((function(e,t){return e[0].trace.index-t[0].trace.index})),X.layer=p,C(n,X);var ee=a.mean(e.map((function(e){return(e.y0+e.y1)/2}))),te=a.mean(e.map((function(e){return(e.x0+e.x1)/2}))),ne=p.select("g.legend"),re=ne.node().getBoundingClientRect();te+=w._offset,ee+=E._offset-re.height/2;var ie=re.width+2*S;!(te+ie<=j)&&te-ie>=0?te-=ie:te+=2*S;var oe=re.height+2*S,ae=ee+oe>=R;return oe<=R&&(ee<=N?ee=E._offset+2*S:ae&&(ee=R-oe)),ne.attr("transform",s(te,ee)),ne}var se=p.selectAll("g.hovertext").data(e,(function(e){return T(e)}));return se.enter().append("g").classed("hovertext",!0).each((function(){var e=r.select(this);e.append("rect").call(f.fill,f.addOpacity(d,.8)),e.append("text").classed("name",!0),e.append("path").style("stroke-width","1px"),e.append("text").classed("nums",!0).call(h.font,_,I)})),se.exit().remove(),se.each((function(e){var t=r.select(this).attr("transform",""),a=e.color;Array.isArray(a)&&(a=a[e.eventData[0].pointNumber]);var p=e.bgcolor||a,g=f.combine(f.opacity(p)?p:f.defaultLine,d),v=f.combine(f.opacity(a)?a:f.defaultLine,d),m=e.borderColor||f.contrast(g),y=k(e,z,o,i,L,t),b=y[0],C=y[1],x=t.select("text.nums").call(h.font,e.fontFamily||_,e.fontSize||I,e.fontColor||m).text(b).attr("data-notex",1).call(c.positionText,0,0).call(c.convertToTspans,n),w=t.select("text.name"),T=0,E=0;if(C&&C!==b){w.call(h.font,e.fontFamily||_,e.fontSize||I,v).text(C).attr("data-notex",1).call(c.positionText,0,0).call(c.convertToTspans,n);var O=w.node().getBoundingClientRect();T=O.width+2*S,E=O.height+2*S}else w.remove(),t.select("rect").remove();t.select("path").style({fill:g,stroke:m});var D,P,F=x.node().getBoundingClientRect(),B=e.xa._offset+(e.x0+e.x1)/2,H=e.ya._offset+(e.y0+e.y1)/2,U=Math.abs(e.x1-e.x0),V=Math.abs(e.y1-e.y0),Y=F.width+M+S+T;if(e.ty0=N-F.top,e.bx=F.width+2*S,e.by=Math.max(F.height+2*S,E),e.anchor="start",e.txwidth=F.width,e.tx2width=T,e.offset=0,u)e.pos=B,D=H+V/2+Y<=R,P=H-V/2-Y>=0,"top"!==e.idealAlign&&D||!P?D?(H+=V/2,e.anchor="start"):e.anchor="middle":(H-=V/2,e.anchor="end");else if(e.pos=H,D=B+U/2+Y<=j,P=B-U/2-Y>=0,"left"!==e.idealAlign&&D||!P)if(D)B+=U/2,e.anchor="start";else{e.anchor="middle";var W=Y/2,G=B+W-j,X=B-W;G>0&&(B-=G),X<0&&(B+=-X)}else B-=U/2,e.anchor="end";x.attr("text-anchor",e.anchor),T&&w.attr("text-anchor",e.anchor),t.attr("transform",s(B,H)+(u?l(A):""))})),se}function k(e,t,n,r,i,o){var s="",l="";void 0!==e.nameOverride&&(e.name=e.nameOverride),e.name&&(e.trace._meta&&(e.name=a.templateString(e.name,e.trace._meta)),s=j(e.name,e.nameLength)),void 0!==e.zLabel?(void 0!==e.xLabel&&(l+="x: "+e.xLabel+"<br>"),void 0!==e.yLabel&&(l+="y: "+e.yLabel+"<br>"),"choropleth"!==e.trace.type&&"choroplethmapbox"!==e.trace.type&&(l+=(l?"z: ":"")+e.zLabel)):t&&e[n.charAt(0)+"Label"]===i?l=e[("x"===n.charAt(0)?"y":"x")+"Label"]||"":void 0===e.xLabel?void 0!==e.yLabel&&"scattercarpet"!==e.trace.type&&(l=e.yLabel):l=void 0===e.yLabel?e.xLabel:"("+e.xLabel+", "+e.yLabel+")",!e.text&&0!==e.text||Array.isArray(e.text)||(l+=(l?"<br>":"")+e.text),void 0!==e.extraText&&(l+=(l?"<br>":"")+e.extraText),o&&""===l&&!e.hovertemplate&&(""===s&&o.remove(),l=s);var u=r._d3locale,c=e.hovertemplate||!1,d=e.hovertemplateLabels||e,h=e.eventData[0]||{};return c&&(l=(l=a.hovertemplateString(c,d,u,h,e.trace._meta)).replace(E,(function(t,n){return s=j(n,e.nameLength),""}))),[l,s]}function L(e,t,n,i){var o=function(e){return e*n},a=function(e){return e*i};e.each((function(e){var n=r.select(this);if(e.del)return n.remove();var i=n.select("text.nums"),s=e.anchor,l="end"===s?-1:1,u={start:1,end:-1,middle:0}[s],d=u*(M+S),f=d+u*(e.txwidth+S),p=0,g=e.offset,v="middle"===s;v&&(d-=e.tx2width/2,f+=e.txwidth/2+S),t&&(g*=-w,p=e.offset*x),n.select("path").attr("d",v?"M-"+o(e.bx/2+e.tx2width/2)+","+a(g-e.by/2)+"h"+o(e.bx)+"v"+a(e.by)+"h-"+o(e.bx)+"Z":"M0,0L"+o(l*M+p)+","+a(M+g)+"v"+a(e.by/2-M)+"h"+o(l*e.bx)+"v-"+a(e.by)+"H"+o(l*M+p)+"V"+a(g-M)+"Z");var m=p+d,y=g+e.ty0-e.by/2+S,b=e.textAlign||"auto";"auto"!==b&&("left"===b&&"start"!==s?(i.attr("text-anchor","start"),m=v?-e.bx/2-e.tx2width/2+S:-e.bx-S):"right"===b&&"end"!==s&&(i.attr("text-anchor","end"),m=v?e.bx/2-e.tx2width/2-S:e.bx+S)),i.call(c.positionText,o(m),a(y)),e.tx2width&&(n.select("text.name").call(c.positionText,o(f+u*S+p),a(g+e.ty0-e.by/2+S)),n.select("rect").call(h.setRect,o(f+(u-1)*e.tx2width/2+p),a(g-e.by/2-1),o(e.tx2width),a(e.by+2)))}))}function D(e,t){var n=e.index,r=e.trace||{},o=e.cd[0],s=e.cd[n]||{};function l(e){return e||i(e)&&0===e}var u=Array.isArray(n)?function(e,t){var i=a.castOption(o,n,e);return l(i)?i:a.extractOption({},r,"",t)}:function(e,t){return a.extractOption(s,r,e,t)};function c(t,n,r){var i=u(n,r);l(i)&&(e[t]=i)}if(c("hoverinfo","hi","hoverinfo"),c("bgcolor","hbg","hoverlabel.bgcolor"),c("borderColor","hbc","hoverlabel.bordercolor"),c("fontFamily","htf","hoverlabel.font.family"),c("fontSize","hts","hoverlabel.font.size"),c("fontColor","htc","hoverlabel.font.color"),c("nameLength","hnl","hoverlabel.namelength"),c("textAlign","hta","hoverlabel.align"),e.posref="y"===t||"closest"===t&&"h"===r.orientation?e.xa._offset+(e.x0+e.x1)/2:e.ya._offset+(e.y0+e.y1)/2,e.x0=a.constrain(e.x0,0,e.xa._length),e.x1=a.constrain(e.x1,0,e.xa._length),e.y0=a.constrain(e.y0,0,e.ya._length),e.y1=a.constrain(e.y1,0,e.ya._length),void 0!==e.xLabelVal&&(e.xLabel="xLabel"in e?e.xLabel:g.hoverLabelText(e.xa,e.xLabelVal),e.xVal=e.xa.c2d(e.xLabelVal)),void 0!==e.yLabelVal&&(e.yLabel="yLabel"in e?e.yLabel:g.hoverLabelText(e.ya,e.yLabelVal),e.yVal=e.ya.c2d(e.yLabelVal)),void 0!==e.zLabelVal&&void 0===e.zLabel&&(e.zLabel=String(e.zLabelVal)),!(isNaN(e.xerr)||"log"===e.xa.type&&e.xerr<=0)){var d=g.tickText(e.xa,e.xa.c2l(e.xerr),"hover").text;void 0!==e.xerrneg?e.xLabel+=" +"+d+" / -"+g.tickText(e.xa,e.xa.c2l(e.xerrneg),"hover").text:e.xLabel+=" ± "+d,"x"===t&&(e.distance+=1)}if(!(isNaN(e.yerr)||"log"===e.ya.type&&e.yerr<=0)){var h=g.tickText(e.ya,e.ya.c2l(e.yerr),"hover").text;void 0!==e.yerrneg?e.yLabel+=" +"+h+" / -"+g.tickText(e.ya,e.ya.c2l(e.yerrneg),"hover").text:e.yLabel+=" ± "+h,"y"===t&&(e.distance+=1)}var f=e.hoverinfo||e.trace.hoverinfo;return f&&"all"!==f&&(-1===(f=Array.isArray(f)?f:f.split("+")).indexOf("x")&&(e.xLabel=void 0),-1===f.indexOf("y")&&(e.yLabel=void 0),-1===f.indexOf("z")&&(e.zLabel=void 0),-1===f.indexOf("text")&&(e.text=void 0),-1===f.indexOf("name")&&(e.name=void 0)),e}function P(e,t,n){var r,i,a=n.container,s=n.fullLayout,l=s._size,u=n.event,c=!!t.hLinePoint,d=!!t.vLinePoint;if(a.selectAll(".spikeline").remove(),d||c){var p=f.combine(s.plot_bgcolor,s.paper_bgcolor);if(c){var v,m,y=t.hLinePoint;r=y&&y.xa,"cursor"===(i=y&&y.ya).spikesnap?(v=u.pointerX,m=u.pointerY):(v=r._offset+y.x,m=i._offset+y.y);var b,C,A=o.readability(y.color,p)<1.5?f.contrast(p):y.color,_=i.spikemode,I=i.spikethickness,x=i.spikecolor||A,w=g.getPxPosition(e,i);if(-1!==_.indexOf("toaxis")||-1!==_.indexOf("across")){if(-1!==_.indexOf("toaxis")&&(b=w,C=v),-1!==_.indexOf("across")){var M=i._counterDomainMin,S=i._counterDomainMax;"free"===i.anchor&&(M=Math.min(M,i.position),S=Math.max(S,i.position)),b=l.l+M*l.w,C=l.l+S*l.w}a.insert("line",":first-child").attr({x1:b,x2:C,y1:m,y2:m,"stroke-width":I,stroke:x,"stroke-dasharray":h.dashStyle(i.spikedash,I)}).classed("spikeline",!0).classed("crisp",!0),a.insert("line",":first-child").attr({x1:b,x2:C,y1:m,y2:m,"stroke-width":I+2,stroke:p}).classed("spikeline",!0).classed("crisp",!0)}-1!==_.indexOf("marker")&&a.insert("circle",":first-child").attr({cx:w+("right"!==i.side?I:-I),cy:m,r:I,fill:x}).classed("spikeline",!0)}if(d){var T,E,O=t.vLinePoint;r=O&&O.xa,i=O&&O.ya,"cursor"===r.spikesnap?(T=u.pointerX,E=u.pointerY):(T=r._offset+O.x,E=i._offset+O.y);var k,L,D=o.readability(O.color,p)<1.5?f.contrast(p):O.color,P=r.spikemode,N=r.spikethickness,j=r.spikecolor||D,R=g.getPxPosition(e,r);if(-1!==P.indexOf("toaxis")||-1!==P.indexOf("across")){if(-1!==P.indexOf("toaxis")&&(k=R,L=E),-1!==P.indexOf("across")){var z=r._counterDomainMin,F=r._counterDomainMax;"free"===r.anchor&&(z=Math.min(z,r.position),F=Math.max(F,r.position)),k=l.t+(1-F)*l.h,L=l.t+(1-z)*l.h}a.insert("line",":first-child").attr({x1:T,x2:T,y1:k,y2:L,"stroke-width":N,stroke:j,"stroke-dasharray":h.dashStyle(r.spikedash,N)}).classed("spikeline",!0).classed("crisp",!0),a.insert("line",":first-child").attr({x1:T,x2:T,y1:k,y2:L,"stroke-width":N+2,stroke:p}).classed("spikeline",!0).classed("crisp",!0)}-1!==P.indexOf("marker")&&a.insert("circle",":first-child").attr({cx:T,cy:R-("top"!==r.side?N:-N),r:N,fill:j}).classed("spikeline",!0)}}}function N(e,t){return!t||(t.vLinePoint!==e._spikepoints.vLinePoint||t.hLinePoint!==e._spikepoints.hLinePoint)}function j(e,t){return c.plainText(e||"",{len:t,allowedTags:["br","sub","sup","b","i","em"]})}},function(e,t,n){"use strict";var r=n(27),i=n(10),o=n(64),a=n(995),s=n(295),l=n(546);e.exports=function(e,t,n){for(var u=e.legend||{},c=0,d=!1,h="normal",f=0;f<n.length;f++){var p=n[f];p.visible&&((p.showlegend||p._dfltShowLegend&&!(p._module&&p._module.attributes&&p._module.attributes.showlegend&&!1===p._module.attributes.showlegend.dflt))&&(c++,p.showlegend&&(d=!0,(r.traceIs(p,"pie-like")||!0===p._input.showlegend)&&c++)),(r.traceIs(p,"bar")&&"stack"===t.barmode||-1!==["tonextx","tonexty"].indexOf(p.fill))&&(h=l.isGrouped({traceorder:h})?"grouped+reversed":"reversed"),void 0!==p.legendgroup&&""!==p.legendgroup&&(h=l.isReversed({traceorder:h})?"reversed+grouped":"grouped"))}var g=i.coerce(e,t,s,"showlegend",d&&c>1);if(!1!==g||u.uirevision){var v=o.newContainer(t,"legend");if(A("uirevision",t.uirevision),!1!==g){A("bgcolor",t.paper_bgcolor),A("bordercolor"),A("borderwidth"),i.coerceFont(A,"font",t.font);var m,y,b,C=A("orientation");"h"===C?(m=0,r.getComponentMethod("rangeslider","isVisible")(e.xaxis)?(y=1.1,b="bottom"):(y=-.1,b="top")):(m=1.02,y=1,b="auto"),A("traceorder",h),l.isGrouped(t.legend)&&A("tracegroupgap"),A("itemsizing"),A("itemwidth"),A("itemclick"),A("itemdoubleclick"),A("x",m),A("xanchor"),A("y",y),A("yanchor",b),A("valign"),i.noneOrAll(u,v,["x","y"]),A("title.text")&&(A("title.side","h"===C?"left":"top"),i.coerceFont(A,"title.font",t.font))}}function A(e,t){return i.coerce(u,v,a,e,t)}}},function(e,t,n){"use strict";var r=n(94),i=n(133);e.exports={bgcolor:{valType:"color",editType:"legend"},bordercolor:{valType:"color",dflt:i.defaultLine,editType:"legend"},borderwidth:{valType:"number",min:0,dflt:0,editType:"legend"},font:r({editType:"legend"}),orientation:{valType:"enumerated",values:["v","h"],dflt:"v",editType:"legend"},traceorder:{valType:"flaglist",flags:["reversed","grouped"],extras:["normal"],editType:"legend"},tracegroupgap:{valType:"number",min:0,dflt:10,editType:"legend"},itemsizing:{valType:"enumerated",values:["trace","constant"],dflt:"trace",editType:"legend"},itemwidth:{valType:"number",min:30,dflt:30,editType:"legend"},itemclick:{valType:"enumerated",values:["toggle","toggleothers",!1],dflt:"toggle",editType:"legend"},itemdoubleclick:{valType:"enumerated",values:["toggle","toggleothers",!1],dflt:"toggleothers",editType:"legend"},x:{valType:"number",min:-2,max:3,editType:"legend"},xanchor:{valType:"enumerated",values:["auto","left","center","right"],dflt:"left",editType:"legend"},y:{valType:"number",min:-2,max:3,editType:"legend"},yanchor:{valType:"enumerated",values:["auto","top","middle","bottom"],editType:"legend"},uirevision:{valType:"any",editType:"none"},valign:{valType:"enumerated",values:["top","middle","bottom"],dflt:"middle",editType:"legend"},title:{text:{valType:"string",dflt:"",editType:"legend"},font:r({editType:"legend"}),side:{valType:"enumerated",values:["top","left","top left"],editType:"legend"},editType:"legend"},editType:"legend"}},function(e,t,n){"use strict";var r=n(39),i=n(10),o=n(82),a=n(27),s=n(383),l=n(163),u=n(56),c=n(41),d=n(96),h=n(1691),f=n(997),p=n(97),g=p.LINE_SPACING,v=p.FROM_TL,m=p.FROM_BR,y=n(1692),b=n(998),C=n(546);function A(e,t,n,r,i){var o=n.data()[0][0].trace,l={event:i,node:n.node(),curveNumber:o.index,expandedIndex:o._expandedIndex,data:e.data,layout:e.layout,frames:e._transitionData._frames,config:e._context,fullData:e._fullData,fullLayout:e._fullLayout};if(o._group&&(l.group=o._group),a.traceIs(o,"pie-like")&&(l.label=n.datum()[0].label),!1!==s.triggerHandler(e,"plotly_legendclick",l))if(1===r)t._clickTimeout=setTimeout((function(){h(n,e,r)}),e._context.doubleClickDelay);else if(2===r){t._clickTimeout&&clearTimeout(t._clickTimeout),e._legendMouseDownTime=0,!1!==s.triggerHandler(e,"plotly_legenddoubleclick",l)&&h(n,e,r)}}function _(e,t,n){var r,o=e.data()[0][0],s=o.trace,l=a.traceIs(s,"pie-like"),c=s.index,h=n._main&&t._context.edits.legendText&&!l,p=n._maxNameLength;n.entries?r=o.text:(r=l?o.label:s.name,s._meta&&(r=i.templateString(r,s._meta)));var g=i.ensureSingle(e,"text","legendtext");g.attr("text-anchor","start").call(u.font,n.font).text(h?I(r,p):r);var v=n.itemwidth+2*f.itemGap;d.positionText(g,v,0),h?g.call(d.makeEditable,{gd:t,text:r}).call(w,e,t,n).on("edit",(function(r){this.text(I(r,p)).call(w,e,t,n);var s=o.trace._fullInput||{},l={};if(a.hasTransform(s,"groupby")){var u=a.getTransformIndices(s,"groupby"),d=u[u.length-1],h=i.keyedContainer(s,"transforms["+d+"].styles","target","value.name");h.set(o.trace._group,r),l=h.constructUpdate()}else l.name=r;return a.call("_guiRestyle",t,l,c)})):w(g,e,t,n)}function I(e,t){var n=Math.max(4,t);if(e&&e.trim().length>=n/2)return e;for(var r=n-(e=e||"").length;r>0;r--)e+=" ";return e}function x(e,t){var n,o=t._context.doubleClickDelay,a=1,s=i.ensureSingle(e,"rect","legendtoggle",(function(e){t._context.staticPlot||e.style("cursor","pointer").attr("pointer-events","all"),e.call(c.fill,"rgba(0,0,0,0)")}));t._context.staticPlot||(s.on("mousedown",(function(){(n=(new Date).getTime())-t._legendMouseDownTime<o?a+=1:(a=1,t._legendMouseDownTime=n)})),s.on("mouseup",(function(){if(!t._dragged&&!t._editing){var n=t._fullLayout.legend;(new Date).getTime()-t._legendMouseDownTime>o&&(a=Math.max(a-1,1)),A(t,n,e,a,r.event)}})))}function w(e,t,n,r){r._main||e.attr("data-notex",!0),d.convertToTspans(e,n,(function(){!function(e,t,n){var r=e.data()[0][0];if(n._main&&r&&!r.trace.showlegend)return void e.remove();var i=e.select("g[class*=math-group]"),o=i.node();n||(n=t._fullLayout.legend);var a,s,l=n.borderwidth,c=(r?n:n.title).font.size*g;if(o){var h=u.bBox(o);a=h.height,s=h.width,r?u.setTranslate(i,0,.25*a):u.setTranslate(i,l,.75*a+l)}else{var p=e.select(r?".legendtext":".legendtitletext"),v=d.lineCount(p),m=p.node();a=c*v,s=m?u.bBox(m).width:0;var y=c*((v-1)/2-.3);if(r){var b=n.itemwidth+2*f.itemGap;d.positionText(p,b,-y)}else d.positionText(p,f.titlePad+l,c+l)}r?(r.lineHeight=c,r.height=Math.max(a,16)+3,r.width=s):(n._titleWidth=s,n._titleHeight=a)}(t,n,r)}))}function M(e){return i.isRightAnchor(e)?"right":i.isCenterAnchor(e)?"center":"left"}function S(e){return i.isBottomAnchor(e)?"bottom":i.isMiddleAnchor(e)?"middle":"top"}e.exports=function(e,t){var n,s=e._fullLayout,d="legend"+s._uid;if(t?(n=t.layer,d+="-hover"):((t=s.legend||{})._main=!0,n=s._infolayer),n){var h;if(e._legendMouseDownTime||(e._legendMouseDownTime=0),t._main){if(!e.calcdata)return;h=s.showlegend&&y(e.calcdata,t)}else{if(!t.entries)return;h=y(t.entries,t)}var p=s.hiddenlabels||[];if(t._main&&(!s.showlegend||!h.length))return n.selectAll(".legend").remove(),s._topdefs.select("#"+d).remove(),o.autoMargin(e,"legend");var g=i.ensureSingle(n,"g","legend",(function(e){t._main&&e.attr("pointer-events","all")})),I=i.ensureSingleById(s._topdefs,"clipPath",d,(function(e){e.append("rect")})),T=i.ensureSingle(g,"rect","bg",(function(e){e.attr("shape-rendering","crispEdges")}));T.call(c.stroke,t.bordercolor).call(c.fill,t.bgcolor).style("stroke-width",t.borderwidth+"px");var E=i.ensureSingle(g,"g","scrollbox"),O=t.title;if(t._titleWidth=0,t._titleHeight=0,O.text){var k=i.ensureSingle(E,"text","legendtitletext");k.attr("text-anchor","start").call(u.font,O.font).text(O.text),w(k,E,e,t)}else E.selectAll(".legendtitletext").remove();var L=i.ensureSingle(g,"rect","scrollbar",(function(e){e.attr(f.scrollBarEnterAttrs).call(c.fill,f.scrollBarColor)})),D=E.selectAll("g.groups").data(h);D.enter().append("g").attr("class","groups"),D.exit().remove();var P=D.selectAll("g.traces").data(i.identity);P.enter().append("g").attr("class","traces"),P.exit().remove(),P.style("opacity",(function(e){var t=e[0].trace;return a.traceIs(t,"pie-like")?-1!==p.indexOf(e[0].label)?.5:1:"legendonly"===t.visible?.5:1})).each((function(){r.select(this).call(_,e,t)})).call(b,e,t).each((function(){t._main&&r.select(this).call(x,e)})),i.syncOrAsync([o.previousPromises,function(){return function(e,t,n,i){var o=e._fullLayout;i||(i=o.legend);var a=o._size,s=C.isVertical(i),l=C.isGrouped(i),c=i.borderwidth,d=2*c,h=f.itemGap,p=i.itemwidth+2*h,g=2*(c+h),v=S(i),m=i.y<0||0===i.y&&"top"===v,y=i.y>1||1===i.y&&"bottom"===v;i._maxHeight=Math.max(m||y?o.height/2:a.h,30);var b=0;i._width=0,i._height=0;var A=function(e){var t=0,n=0,r=e.title.side;r&&(-1!==r.indexOf("left")&&(t=e._titleWidth),-1!==r.indexOf("top")&&(n=e._titleHeight));return[t,n]}(i);if(s)n.each((function(e){var t=e[0].height;u.setTranslate(this,c+A[0],c+A[1]+i._height+t/2+h),i._height+=t,i._width=Math.max(i._width,e[0].width)})),b=p+i._width,i._width+=h+p+d,i._height+=g,l&&(t.each((function(e,t){u.setTranslate(this,0,t*i.tracegroupgap)})),i._height+=(i._lgroupsLength-1)*i.tracegroupgap);else{var _=M(i),I=i.x<0||0===i.x&&"right"===_,x=i.x>1||1===i.x&&"left"===_,w=y||m,T=o.width/2;i._maxWidth=Math.max(I?w&&"left"===_?a.l+a.w:T:x?w&&"right"===_?a.r+a.w:T:a.w,2*p);var E=0,O=0;n.each((function(e){var t=e[0].width+p;E=Math.max(E,t),O+=t})),b=null;var k=0;if(l){var L=0,D=0,P=0;t.each((function(){var e=0,t=0;r.select(this).selectAll("g.traces").each((function(n){var r=n[0].height;u.setTranslate(this,A[0],A[1]+c+h+r/2+t),t+=r,e=Math.max(e,p+n[0].width)})),L=Math.max(L,t);var n=e+h;n+c+D>i._maxWidth&&(k=Math.max(k,D),D=0,P+=L+i.tracegroupgap,L=t),u.setTranslate(this,D,P),D+=n})),i._width=Math.max(k,D)+c,i._height=P+L+g}else{var N=n.size(),j=O+d+(N-1)*h<i._maxWidth,R=0,z=0,F=0,B=0;n.each((function(e){var t=e[0].height,n=p+e[0].width,r=(j?n:E)+h;r+c+z-h>=i._maxWidth&&(k=Math.max(k,B),z=0,F+=R,i._height+=R,R=0),u.setTranslate(this,A[0]+c+z,A[1]+c+F+t/2+h),B=z+n+h,z+=r,R=Math.max(R,t)})),j?(i._width=z+d,i._height=R+g):(i._width=Math.max(k,B)+d,i._height+=R+g)}}i._width=Math.ceil(Math.max(i._width+A[0],i._titleWidth+2*(c+f.titlePad))),i._height=Math.ceil(Math.max(i._height+A[1],i._titleHeight+2*(c+f.itemGap))),i._effHeight=Math.min(i._height,i._maxHeight);var H=e._context.edits,U=H.legendText||H.legendPosition;n.each((function(e){var t=r.select(this).select(".legendtoggle"),n=e[0].height,i=U?p:b||p+e[0].width;s||(i+=h/2),u.setRect(t,0,-n/2,i,n)}))}(e,D,P,t)},function(){if(!t._main||!function(e){var t=e._fullLayout.legend,n=M(t),r=S(t);return o.autoMargin(e,"legend",{x:t.x,y:t.y,l:t._width*v[n],r:t._width*m[n],b:t._effHeight*m[r],t:t._effHeight*v[r]})}(e)){var c,h,p,y,b=s._size,C=t.borderwidth,_=b.l+b.w*t.x-v[M(t)]*t._width,x=b.t+b.h*(1-t.y)-v[S(t)]*t._effHeight;if(t._main&&s.margin.autoexpand){var w=_,O=x;_=i.constrain(_,0,s.width-t._width),x=i.constrain(x,0,s.height-t._effHeight),_!==w&&i.log("Constrain legend.x to make legend fit inside graph"),x!==O&&i.log("Constrain legend.y to make legend fit inside graph")}if(t._main&&u.setTranslate(g,_,x),L.on(".drag",null),g.on("wheel",null),!t._main||t._height<=t._maxHeight||e._context.staticPlot){var k=t._effHeight;t._main||(k=t._height),T.attr({width:t._width-C,height:k-C,x:C/2,y:C/2}),u.setTranslate(E,0,0),I.select("rect").attr({width:t._width-2*C,height:k-2*C,x:C,y:C}),u.setClipUrl(E,d,e),u.setRect(L,0,0,0,0),delete t._scrollY}else{var D,P,N,j=Math.max(f.scrollBarMinHeight,t._effHeight*t._effHeight/t._height),R=t._effHeight-j-2*f.scrollBarMargin,z=t._height-t._effHeight,F=R/z,B=Math.min(t._scrollY||0,z);T.attr({width:t._width-2*C+f.scrollBarWidth+f.scrollBarMargin,height:t._effHeight-C,x:C/2,y:C/2}),I.select("rect").attr({width:t._width-2*C+f.scrollBarWidth+f.scrollBarMargin,height:t._effHeight-2*C,x:C,y:C+B}),u.setClipUrl(E,d,e),V(B,j,F),g.on("wheel",(function(){V(B=i.constrain(t._scrollY+r.event.deltaY/R*z,0,z),j,F),0!==B&&B!==z&&r.event.preventDefault()}));var H=r.behavior.drag().on("dragstart",(function(){var e=r.event.sourceEvent;D="touchstart"===e.type?e.changedTouches[0].clientY:e.clientY,N=B})).on("drag",(function(){var e=r.event.sourceEvent;2===e.buttons||e.ctrlKey||(P="touchmove"===e.type?e.changedTouches[0].clientY:e.clientY,V(B=function(e,t,n){var r=(n-t)/F+e;return i.constrain(r,0,z)}(N,D,P),j,F))}));L.call(H);var U=r.behavior.drag().on("dragstart",(function(){var e=r.event.sourceEvent;"touchstart"===e.type&&(D=e.changedTouches[0].clientY,N=B)})).on("drag",(function(){var e=r.event.sourceEvent;"touchmove"===e.type&&(P=e.changedTouches[0].clientY,V(B=function(e,t,n){var r=(t-n)/F+e;return i.constrain(r,0,z)}(N,D,P),j,F))}));E.call(U)}if(e._context.edits.legendPosition)g.classed("cursor-move",!0),l.init({element:g.node(),gd:e,prepFn:function(){var e=u.getTranslate(g);p=e.x,y=e.y},moveFn:function(e,n){var r=p+e,i=y+n;u.setTranslate(g,r,i),c=l.align(r,0,b.l,b.l+b.w,t.xanchor),h=l.align(i,0,b.t+b.h,b.t,t.yanchor)},doneFn:function(){void 0!==c&&void 0!==h&&a.call("_guiRelayout",e,{"legend.x":c,"legend.y":h})},clickFn:function(t,r){var i=n.selectAll("g.traces").filter((function(){var e=this.getBoundingClientRect();return r.clientX>=e.left&&r.clientX<=e.right&&r.clientY>=e.top&&r.clientY<=e.bottom}));i.size()>0&&A(e,g,i,t,r)}})}function V(n,r,i){t._scrollY=e._fullLayout.legend._scrollY=n,u.setTranslate(E,0,-n),u.setRect(L,t._width,f.scrollBarMargin+n*i,f.scrollBarWidth,r),I.select("rect").attr("y",C+n)}}],e)}}},function(e,t,n){"use strict";e.exports={scrollBarWidth:6,scrollBarMinHeight:20,scrollBarColor:"#808BA4",scrollBarMargin:4,scrollBarEnterAttrs:{rx:20,ry:3,width:0,height:0},titlePad:2,itemGap:5}},function(e,t,n){"use strict";var r=n(39),i=n(27),o=n(10),a=o.strTranslate,s=n(56),l=n(41),u=n(149).extractOpts,c=n(125),d=n(1693),h=n(999).castOption,f=n(997);function p(e,t){return(t?"radial":"horizontal")+(e?"":"reversed")}e.exports=function(e,t,n){var g=t._fullLayout;n||(n=g.legend);var v="constant"===n.itemsizing,m=n.itemwidth,y=(m+2*f.itemGap)/2,b=a(y,0),C=function(e,t,n,r){var i;if(e+1)i=e;else{if(!(t&&t.width>0))return 0;i=t.width}return v?r:Math.min(i,n)};function A(e,t,n){var o=e[0].trace,a=o.marker||{},s=a.line||{},u=n?o.visible&&o.type===n:i.traceIs(o,"bar"),c=r.select(t).select("g.legendpoints").selectAll("path.legend"+n).data(u?[e]:[]);c.enter().append("path").classed("legend"+n,!0).attr("d","M6,6H-6V-6H6Z").attr("transform",b),c.exit().remove(),c.each((function(e){var t=r.select(this),n=e[0],i=C(n.mlw,a.line,5,2);t.style("stroke-width",i+"px").call(l.fill,n.mc||a.color),i&&l.stroke(t,n.mlc||s.color)}))}function _(e,t,n){var a=e[0],s=a.trace,l=n?s.visible&&s.type===n:i.traceIs(s,n),u=r.select(t).select("g.legendpoints").selectAll("path.legend"+n).data(l?[e]:[]);if(u.enter().append("path").classed("legend"+n,!0).attr("d","M6,6H-6V-6H6Z").attr("transform",b),u.exit().remove(),u.size()){var c=(s.marker||{}).line,f=C(h(c.width,a.pts),c,5,2),p=o.minExtend(s,{marker:{line:{width:f}}});p.marker.line.color=c.color;var g=o.minExtend(a,{trace:p});d(u,g,p)}}e.each((function(e){var t=r.select(this),i=o.ensureSingle(t,"g","layers");i.style("opacity",e[0].trace.opacity);var s=n.valign,l=e[0].lineHeight,u=e[0].height;if("middle"!==s&&l&&u){var c={top:1,bottom:-1}[s]*(.5*(l-u+3));i.attr("transform",a(0,c))}else i.attr("transform",null);i.selectAll("g.legendfill").data([e]).enter().append("g").classed("legendfill",!0),i.selectAll("g.legendlines").data([e]).enter().append("g").classed("legendlines",!0);var d=i.selectAll("g.legendsymbols").data([e]);d.enter().append("g").classed("legendsymbols",!0),d.selectAll("g.legendpoints").data([e]).enter().append("g").classed("legendpoints",!0)})).each((function(e){var n,i=e[0].trace,a=[];if(i.visible)switch(i.type){case"histogram2d":case"heatmap":a=[["M-15,-2V4H15V-2Z"]],n=!0;break;case"choropleth":case"choroplethmapbox":a=[["M-6,-6V6H6V-6Z"]],n=!0;break;case"densitymapbox":a=[["M-6,0 a6,6 0 1,0 12,0 a 6,6 0 1,0 -12,0"]],n="radial";break;case"cone":a=[["M-6,2 A2,2 0 0,0 -6,6 V6L6,4Z"],["M-6,-6 A2,2 0 0,0 -6,-2 L6,-4Z"],["M-6,-2 A2,2 0 0,0 -6,2 L6,0Z"]],n=!1;break;case"streamtube":a=[["M-6,2 A2,2 0 0,0 -6,6 H6 A2,2 0 0,1 6,2 Z"],["M-6,-6 A2,2 0 0,0 -6,-2 H6 A2,2 0 0,1 6,-6 Z"],["M-6,-2 A2,2 0 0,0 -6,2 H6 A2,2 0 0,1 6,-2 Z"]],n=!1;break;case"surface":a=[["M-6,-6 A2,3 0 0,0 -6,0 H6 A2,3 0 0,1 6,-6 Z"],["M-6,1 A2,3 0 0,1 -6,6 H6 A2,3 0 0,0 6,0 Z"]],n=!0;break;case"mesh3d":a=[["M-6,6H0L-6,-6Z"],["M6,6H0L6,-6Z"],["M-6,-6H6L0,6Z"]],n=!1;break;case"volume":a=[["M-6,6H0L-6,-6Z"],["M6,6H0L6,-6Z"],["M-6,-6H6L0,6Z"]],n=!0;break;case"isosurface":a=[["M-6,6H0L-6,-6Z"],["M6,6H0L6,-6Z"],["M-6,-6 A12,24 0 0,0 6,-6 L0,6Z"]],n=!1}var c=r.select(this).select("g.legendpoints").selectAll("path.legend3dandfriends").data(a);c.enter().append("path").classed("legend3dandfriends",!0).attr("transform",b).style("stroke-miterlimit",1),c.exit().remove(),c.each((function(e,a){var c,d=r.select(this),h=u(i),f=h.colorscale,g=h.reversescale;if(f){if(!n){var v=f.length;c=0===a?f[g?v-1:0][1]:1===a?f[g?0:v-1][1]:f[Math.floor((v-1)/2)][1]}}else{var m=i.vertexcolor||i.facecolor||i.color;c=o.isArrayOrTypedArray(m)?m[a]||m[0]:m}d.attr("d",e[0]),c?d.call(l.fill,c):d.call((function(e){if(e.size()){var r="legendfill-"+i.uid;s.gradient(e,t,r,p(g,"radial"===n),f,"fill")}}))}))})).each((function(e){var t=e[0].trace,n="waterfall"===t.type;if(e[0]._distinct&&n){var i=e[0].trace[e[0].dir].marker;return e[0].mc=i.color,e[0].mlw=i.line.width,e[0].mlc=i.line.color,A(e,this,"waterfall")}var o=[];t.visible&&n&&(o=e[0].hasTotals?[["increasing","M-6,-6V6H0Z"],["totals","M6,6H0L-6,-6H-0Z"],["decreasing","M6,6V-6H0Z"]]:[["increasing","M-6,-6V6H6Z"],["decreasing","M6,6V-6H-6Z"]]);var a=r.select(this).select("g.legendpoints").selectAll("path.legendwaterfall").data(o);a.enter().append("path").classed("legendwaterfall",!0).attr("transform",b).style("stroke-miterlimit",1),a.exit().remove(),a.each((function(e){var n=r.select(this),i=t[e[0]].marker,o=C(void 0,i.line,5,2);n.attr("d",e[1]).style("stroke-width",o+"px").call(l.fill,i.color),o&&n.call(l.stroke,i.line.color)}))})).each((function(e){A(e,this,"funnel")})).each((function(e){A(e,this)})).each((function(e){var n=e[0].trace,a=r.select(this).select("g.legendpoints").selectAll("path.legendbox").data(n.visible&&i.traceIs(n,"box-violin")?[e]:[]);a.enter().append("path").classed("legendbox",!0).attr("d","M6,6H-6V-6H6Z").attr("transform",b),a.exit().remove(),a.each((function(){var e=r.select(this);if("all"!==n.boxpoints&&"all"!==n.points||0!==l.opacity(n.fillcolor)||0!==l.opacity((n.line||{}).color)){var i=C(void 0,n.line,5,2);e.style("stroke-width",i+"px").call(l.fill,n.fillcolor),i&&l.stroke(e,n.line.color)}else{var u=o.minExtend(n,{marker:{size:v?12:o.constrain(n.marker.size,2,16),sizeref:1,sizemin:1,sizemode:"diameter"}});a.call(s.pointStyle,u,t)}}))})).each((function(e){_(e,this,"funnelarea")})).each((function(e){_(e,this,"pie")})).each((function(e){var n,i,a=e[0],l=a.trace,d=l.visible&&l.fill&&"none"!==l.fill,h=c.hasLines(l),f=l.contours,g=!1,v=!1,y=u(l),b=y.colorscale,A=y.reversescale;if(f){var _=f.coloring;"lines"===_?g=!0:h="none"===_||"heatmap"===_||f.showlines,"constraint"===f.type?d="="!==f._operation:"fill"!==_&&"heatmap"!==_||(v=!0)}var I=c.hasMarkers(l)||c.hasText(l),x=d||v,w=h||g,M=I||!x?"M5,0":w?"M5,-2":"M5,-3",S=r.select(this),T=S.select(".legendfill").selectAll("path").data(d||v?[e]:[]);if(T.enter().append("path").classed("js-fill",!0),T.exit().remove(),T.attr("d",M+"h"+m+"v6h-"+m+"z").call(d?s.fillGroupStyle:function(e){if(e.size()){var n="legendfill-"+l.uid;s.gradient(e,t,n,p(A),b,"fill")}}),h||g){var E=C(void 0,l.line,10,5);i=o.minExtend(l,{line:{width:E}}),n=[o.minExtend(a,{trace:i})]}var O=S.select(".legendlines").selectAll("path").data(h||g?[n]:[]);O.enter().append("path").classed("js-line",!0),O.exit().remove(),O.attr("d",M+(g?"l"+m+",0.0001":"h"+m)).call(h?s.lineGroupStyle:function(e){if(e.size()){var n="legendline-"+l.uid;s.lineGroupStyle(e),s.gradient(e,t,n,p(A),b,"stroke")}})})).each((function(e){var n,i,a=e[0],l=a.trace,u=c.hasMarkers(l),d=c.hasText(l),h=c.hasLines(l);function f(e,t,n,r){var i=o.nestedProperty(l,e).get(),a=o.isArrayOrTypedArray(i)&&t?t(i):i;if(v&&a&&void 0!==r&&(a=r),n){if(a<n[0])return n[0];if(a>n[1])return n[1]}return a}function p(e){return a._distinct&&a.index&&e[a.index]?e[a.index]:e[0]}if(u||d||h){var g={},m={};if(u){g.mc=f("marker.color",p),g.mx=f("marker.symbol",p),g.mo=f("marker.opacity",o.mean,[.2,1]),g.mlc=f("marker.line.color",p),g.mlw=f("marker.line.width",o.mean,[0,5],2),m.marker={sizeref:1,sizemin:1,sizemode:"diameter"};var y=f("marker.size",o.mean,[2,16],12);g.ms=y,m.marker.size=y}h&&(m.line={width:f("line.width",p,[0,10],5)}),d&&(g.tx="Aa",g.tp=f("textposition",p),g.ts=10,g.tc=f("textfont.color",p),g.tf=f("textfont.family",p)),n=[o.minExtend(a,g)],(i=o.minExtend(l,m)).selectedpoints=null,i.texttemplate=null}var C=r.select(this).select("g.legendpoints"),A=C.selectAll("path.scatterpts").data(u?n:[]);A.enter().insert("path",":first-child").classed("scatterpts",!0).attr("transform",b),A.exit().remove(),A.call(s.pointStyle,i,t),u&&(n[0].mrc=3);var _=C.selectAll("g.pointtext").data(d?n:[]);_.enter().append("g").classed("pointtext",!0).append("text").attr("transform",b),_.exit().remove(),_.selectAll("text").call(s.textPointStyle,i,t)})).each((function(e){var t=e[0].trace,n=r.select(this).select("g.legendpoints").selectAll("path.legendcandle").data(t.visible&&"candlestick"===t.type?[e,e]:[]);n.enter().append("path").classed("legendcandle",!0).attr("d",(function(e,t){return t?"M-15,0H-8M-8,6V-6H8Z":"M15,0H8M8,-6V6H-8Z"})).attr("transform",b).style("stroke-miterlimit",1),n.exit().remove(),n.each((function(e,n){var i=r.select(this),o=t[n?"increasing":"decreasing"],a=C(void 0,o.line,5,2);i.style("stroke-width",a+"px").call(l.fill,o.fillcolor),a&&l.stroke(i,o.line.color)}))})).each((function(e){var t=e[0].trace,n=r.select(this).select("g.legendpoints").selectAll("path.legendohlc").data(t.visible&&"ohlc"===t.type?[e,e]:[]);n.enter().append("path").classed("legendohlc",!0).attr("d",(function(e,t){return t?"M-15,0H0M-8,-6V0":"M15,0H0M8,6V0"})).attr("transform",b).style("stroke-miterlimit",1),n.exit().remove(),n.each((function(e,n){var i=r.select(this),o=t[n?"increasing":"decreasing"],a=C(void 0,o.line,5,2);i.style("fill","none").call(s.dashLine,o.line.dash,a),a&&l.stroke(i,o.line.color)}))}))}},function(e,t,n){"use strict";var r=n(10);function i(e){return-1!==e.indexOf("e")?e.replace(/[.]?0+e/,"e"):-1!==e.indexOf(".")?e.replace(/[.]?0+$/,""):e}t.formatPiePercent=function(e,t){var n=i((100*e).toPrecision(3));return r.numSeparate(n,t)+"%"},t.formatPieValue=function(e,t){var n=i(e.toPrecision(10));return r.numSeparate(n,t)},t.getFirstFilled=function(e,t){if(Array.isArray(e))for(var n=0;n<t.length;n++){var r=e[t[n]];if(r||0===r||""===r)return r}},t.castOption=function(e,n){return Array.isArray(e)?t.getFirstFilled(e,n):e||void 0},t.getRotationAngle=function(e){return("auto"===e?0:e)*Math.PI/180}},function(e,t,n){"use strict";var r=n(10),i=n(250);e.exports=function(e,t,n){function o(n,o){return void 0!==t[n]?t[n]:r.coerce(e,t,i,n,o)}var a,s=o("clickmode");return t._has("cartesian")?s.indexOf("select")>-1?a="closest":(t._isHoriz=function(e,t){for(var n=t._scatterStackOpts||{},r=0;r<e.length;r++){var i=e[r],o=i.xaxis+i.yaxis,a=(n[o]||{})[i.stackgroup]||{};if("h"!==i.orientation&&"h"!==a.orientation)return!1}return!0}(n,t),a=t._isHoriz?"y":"x"):a="closest",o("hovermode",a)}},function(e,t,n){"use strict";var r=n(27),i=n(10),o=n(55),a=n(389).readPaths,s=n(1002),l=n(299).clearOutlineControllers,u=n(41),c=n(56),d=n(64).arrayEditor,h=n(163),f=n(214),p=n(553),g=n(554);function v(e){var t=e._fullLayout;for(var n in t._shapeUpperLayer.selectAll("path").remove(),t._shapeLowerLayer.selectAll("path").remove(),t._plots){var r=t._plots[n].shapelayer;r&&r.selectAll("path").remove()}for(var i=0;i<t.shapes.length;i++)t.shapes[i].visible&&b(e,i)}function m(e){return!!e._fullLayout._drawing}function y(e){return!e._context.edits.shapePosition}function b(e,t){e._fullLayout._paperdiv.selectAll('.shapelayer [data-index="'+t+'"]').remove();var n=g.makeOptionsAndPlotinfo(e,t),l=n.options,b=n.plotinfo;if(l._input&&!1!==l.visible)if("below"!==l.layer)x(e._fullLayout._shapeUpperLayer);else if("paper"===l.xref||"paper"===l.yref)x(e._fullLayout._shapeLowerLayer);else{if(b._hadPlotinfo)x((b.mainplotinfo||b).shapelayer);else x(e._fullLayout._shapeLowerLayer)}function x(n){var x=A(e,l),w={"data-index":t,"fill-rule":l.fillrule,d:x},M=l.opacity,S=l.fillcolor,T=l.line.width?l.line.color:"rgba(0,0,0,0)",E=l.line.width,O=l.line.dash;E||!0!==l.editable||(E=5,O="solid");var k="Z"!==x[x.length-1],L=y(e)&&l.editable&&e._fullLayout._activeShapeIndex===t;L&&(S=k?"rgba(0,0,0,0)":e._fullLayout.activeshape.fillcolor,M=e._fullLayout.activeshape.opacity);var D,P=n.append("path").attr(w).style("opacity",M).call(u.stroke,T).call(u.fill,S).call(c.dashLine,O,E);if(C(P,e,l),(L||e._context.edits.shapePosition)&&(D=d(e.layout,"shapes",l)),L){P.style({cursor:"move"});var N={element:P.node(),plotinfo:b,gd:e,editHelpers:D,isActiveShape:!0},j=a(x,e);s(j,P,N)}else e._context.edits.shapePosition?function(e,t,n,a,s,l){var u,d,v,y,b,I,x,w,M,S,T,E,O,k,L,D,P="pixel"===n.xsizemode,N="pixel"===n.ysizemode,j="line"===n.type,R="path"===n.type,z=l.modifyItem,F=o.getFromId(e,n.xref),B=o.getRefType(n.xref),H=o.getFromId(e,n.yref),U=o.getRefType(n.yref),V=g.getDataToPixel(e,F,!1,B),Y=g.getDataToPixel(e,H,!0,U),W=g.getPixelToData(e,F,!1,B),G=g.getPixelToData(e,H,!0,U),X=j?function(){var e=Math.max(n.line.width,10),r=s.append("g").attr("data-index",a);r.append("path").attr("d",t.attr("d")).style({cursor:"move","stroke-width":e,"stroke-opacity":"0"});var i={"fill-opacity":"0"},o=Math.max(e/2,10);return r.append("circle").attr({"data-line-point":"start-point",cx:P?V(n.xanchor)+n.x0:V(n.x0),cy:N?Y(n.yanchor)-n.y0:Y(n.y0),r:o}).style(i).classed("cursor-grab",!0),r.append("circle").attr({"data-line-point":"end-point",cx:P?V(n.xanchor)+n.x1:V(n.x1),cy:N?Y(n.yanchor)-n.y1:Y(n.y1),r:o}).style(i).classed("cursor-grab",!0),r}():t,q={element:X.node(),gd:e,prepFn:function(r){if(m(e))return;P&&(b=V(n.xanchor));N&&(I=Y(n.yanchor));"path"===n.type?L=n.path:(u=P?n.x0:V(n.x0),d=N?n.y0:Y(n.y0),v=P?n.x1:V(n.x1),y=N?n.y1:Y(n.y1));u<v?(M=u,O="x0",S=v,k="x1"):(M=v,O="x1",S=u,k="x0");!N&&d<y||N&&d>y?(x=d,T="y0",w=y,E="y1"):(x=y,T="y1",w=d,E="y0");Z(r),J(s,n),function(e,t,n){var r=t.xref,i=t.yref,a=o.getFromId(n,r),s=o.getFromId(n,i),l="";"paper"===r||a.autorange||(l+=r);"paper"===i||s.autorange||(l+=i);c.setClipUrl(e,l?"clip"+n._fullLayout._uid+l:null,n)}(t,n,e),q.moveFn="move"===D?Q:K,q.altKey=r.altKey},doneFn:function(){if(m(e))return;f(t),$(s),C(t,e,n),r.call("_guiRelayout",e,l.getUpdateObj())},clickFn:function(){if(m(e))return;$(s)}};function Z(n){if(m(e))D=null;else if(j)D="path"===n.target.tagName?"move":"start-point"===n.target.attributes["data-line-point"].value?"resize-over-start-point":"resize-over-end-point";else{var r=q.element.getBoundingClientRect(),i=r.right-r.left,o=r.bottom-r.top,a=n.clientX-r.left,s=n.clientY-r.top,l=!R&&i>10&&o>10&&!n.shiftKey?h.getCursor(a/i,1-s/o):"move";f(t,l),D=l.split("-")[0]}}function Q(r,i){if("path"===n.type){var o=function(e){return e},a=o,l=o;P?z("xanchor",n.xanchor=W(b+r)):(a=function(e){return W(V(e)+r)},F&&"date"===F.type&&(a=g.encodeDate(a))),N?z("yanchor",n.yanchor=G(I+i)):(l=function(e){return G(Y(e)+i)},H&&"date"===H.type&&(l=g.encodeDate(l))),z("path",n.path=_(L,a,l))}else P?z("xanchor",n.xanchor=W(b+r)):(z("x0",n.x0=W(u+r)),z("x1",n.x1=W(v+r))),N?z("yanchor",n.yanchor=G(I+i)):(z("y0",n.y0=G(d+i)),z("y1",n.y1=G(y+i)));t.attr("d",A(e,n)),J(s,n)}function K(r,i){if(R){var o=function(e){return e},a=o,l=o;P?z("xanchor",n.xanchor=W(b+r)):(a=function(e){return W(V(e)+r)},F&&"date"===F.type&&(a=g.encodeDate(a))),N?z("yanchor",n.yanchor=G(I+i)):(l=function(e){return G(Y(e)+i)},H&&"date"===H.type&&(l=g.encodeDate(l))),z("path",n.path=_(L,a,l))}else if(j){if("resize-over-start-point"===D){var c=u+r,h=N?d-i:d+i;z("x0",n.x0=P?c:W(c)),z("y0",n.y0=N?h:G(h))}else if("resize-over-end-point"===D){var f=v+r,p=N?y-i:y+i;z("x1",n.x1=P?f:W(f)),z("y1",n.y1=N?p:G(p))}}else{var m=function(e){return-1!==D.indexOf(e)},C=m("n"),B=m("s"),U=m("w"),X=m("e"),q=C?x+i:x,Z=B?w+i:w,Q=U?M+r:M,K=X?S+r:S;N&&(C&&(q=x-i),B&&(Z=w-i)),(!N&&Z-q>10||N&&q-Z>10)&&(z(T,n[T]=N?q:G(q)),z(E,n[E]=N?Z:G(Z))),K-Q>10&&(z(O,n[O]=P?Q:W(Q)),z(k,n[k]=P?K:W(K)))}t.attr("d",A(e,n)),J(s,n)}function J(e,t){(P||N)&&function(){var n="path"!==t.type,r=e.selectAll(".visual-cue").data([0]);r.enter().append("path").attr({fill:"#fff","fill-rule":"evenodd",stroke:"#000","stroke-width":1}).classed("visual-cue",!0);var o=V(P?t.xanchor:i.midRange(n?[t.x0,t.x1]:g.extractPathCoords(t.path,p.paramIsX))),a=Y(N?t.yanchor:i.midRange(n?[t.y0,t.y1]:g.extractPathCoords(t.path,p.paramIsY)));if(o=g.roundPositionForSharpStrokeRendering(o,1),a=g.roundPositionForSharpStrokeRendering(a,1),P&&N){var s="M"+(o-1-1)+","+(a-1-1)+"h-8v2h8 v8h2v-8 h8v-2h-8 v-8h-2 Z";r.attr("d",s)}else if(P){var l="M"+(o-1-1)+","+(a-9-1)+"v18 h2 v-18 Z";r.attr("d",l)}else{var u="M"+(o-9-1)+","+(a-1-1)+"h18 v2 h-18 Z";r.attr("d",u)}}()}function $(e){e.selectAll(".visual-cue").remove()}h.init(q),X.node().onmousemove=Z}(e,P,l,t,n,D):!0===l.editable&&P.style("pointer-events",k||u.opacity(S)*M<=.5?"stroke":"all");P.node().addEventListener("click",(function(){return function(e,t){if(!y(e))return;var n=+t.node().getAttribute("data-index");if(n>=0){if(n===e._fullLayout._activeShapeIndex)return void I(e);e._fullLayout._activeShapeIndex=n,e._fullLayout._deactivateShape=I,v(e)}}(e,P)}))}}function C(e,t,n){var r=(n.xref+n.yref).replace(/paper/g,"").replace(/[xyz][1-9]* *domain/g,"");c.setClipUrl(e,r?"clip"+t._fullLayout._uid+r:null,t)}function A(e,t){var n,r,a,s,l,u,c,d,h=t.type,f=o.getRefType(t.xref),v=o.getRefType(t.yref),m=o.getFromId(e,t.xref),y=o.getFromId(e,t.yref),b=e._fullLayout._size;if(m?"domain"===f?r=function(e){return m._offset+m._length*e}:(n=g.shapePositionToRange(m),r=function(e){return m._offset+m.r2p(n(e,!0))}):r=function(e){return b.l+b.w*e},y?"domain"===v?s=function(e){return y._offset+y._length*(1-e)}:(a=g.shapePositionToRange(y),s=function(e){return y._offset+y.r2p(a(e,!0))}):s=function(e){return b.t+b.h*(1-e)},"path"===h)return m&&"date"===m.type&&(r=g.decodeDate(r)),y&&"date"===y.type&&(s=g.decodeDate(s)),function(e,t,n){var r=e.path,o=e.xsizemode,a=e.ysizemode,s=e.xanchor,l=e.yanchor;return r.replace(p.segmentRE,(function(e){var r=0,u=e.charAt(0),c=p.paramIsX[u],d=p.paramIsY[u],h=p.numParams[u],f=e.substr(1).replace(p.paramRE,(function(e){return c[r]?e="pixel"===o?t(s)+Number(e):t(e):d[r]&&(e="pixel"===a?n(l)-Number(e):n(e)),++r>h&&(e="X"),e}));return r>h&&(f=f.replace(/[\s,]*X.*/,""),i.log("Ignoring extra params in segment "+e)),u+f}))}(t,r,s);if("pixel"===t.xsizemode){var C=r(t.xanchor);l=C+t.x0,u=C+t.x1}else l=r(t.x0),u=r(t.x1);if("pixel"===t.ysizemode){var A=s(t.yanchor);c=A-t.y0,d=A-t.y1}else c=s(t.y0),d=s(t.y1);if("line"===h)return"M"+l+","+c+"L"+u+","+d;if("rect"===h)return"M"+l+","+c+"H"+u+"V"+d+"H"+l+"Z";var _=(l+u)/2,I=(c+d)/2,x=Math.abs(_-l),w=Math.abs(I-c),M="A"+x+","+w,S=_+x+","+I;return"M"+S+M+" 0 1,1 "+(_+","+(I-w))+M+" 0 0,1 "+S+"Z"}function _(e,t,n){return e.replace(p.segmentRE,(function(e){var r=0,i=e.charAt(0),o=p.paramIsX[i],a=p.paramIsY[i],s=p.numParams[i];return i+e.substr(1).replace(p.paramRE,(function(e){return r>=s||(o[r]?e=t(e):a[r]&&(e=n(e)),r++),e}))}))}function I(e){y(e)&&(e._fullLayout._activeShapeIndex>=0&&(l(e),delete e._fullLayout._activeShapeIndex,v(e)))}e.exports={draw:v,drawOne:b,eraseActiveShape:function(e){if(!y(e))return;l(e);var t=e._fullLayout._activeShapeIndex,n=(e.layout||{}).shapes||[];if(t<n.length){for(var i=[],o=0;o<n.length;o++)o!==t&&i.push(n[o]);delete e._fullLayout._activeShapeIndex,r.call("_guiRelayout",e,{shapes:i})}}}},function(e,t,n){"use strict";var r=n(163),i=n(388).drawMode,o=n(27),a=n(551),s=a.i000,l=a.i090,u=a.i180,c=a.i270,d=n(299).clearOutlineControllers,h=n(389),f=h.pointsShapeRectangle,p=h.pointsShapeEllipse,g=h.writePaths,v=n(1003);e.exports=function e(t,n,a,h){h||(h=0);var m=a.gd;function y(){e(t,n,a,h++),p(t[0])&&b({redrawing:!0})}function b(e){a.isActiveShape=!1;var t=v(n,a);Object.keys(t).length&&o.call((e||{}).redrawing?"relayout":"_guiRelayout",m,t)}var C,A,_,I,x,w=a.isActiveShape,M=m._fullLayout._zoomlayer,S=a.dragmode;(i(S)?m._fullLayout._drawing=!0:m._fullLayout._activeShapeIndex>=0&&d(m),n.attr("d",g(t)),w&&!h)&&(x=function(e,t){for(var n=0;n<t.length;n++){var r=t[n];e[n]=[];for(var i=0;i<r.length;i++){e[n][i]=[];for(var o=0;o<r[i].length;o++)e[n][i][o]=r[i][o]}}return e}([],t),function(e){C=[];for(var n=0;n<t.length;n++){var i=t[n],o=!f(i)&&p(i);C[n]=[];for(var a=0;a<i.length;a++)if("Z"!==i[a][0]&&(!o||a===s||a===l||a===u||a===c)){var d=i[a][1],h=i[a][2],g=e.append("circle").classed("cursor-grab",!0).attr("data-i",n).attr("data-j",a).attr("cx",d).attr("cy",h).attr("r",4).style({"mix-blend-mode":"luminosity",fill:"black",stroke:"white","stroke-width":1});C[n][a]={element:g.node(),gd:m,prepFn:T,doneFn:O,clickFn:k},r.init(C[n][a])}}}(M.append("g").attr("class","outline-controllers")),function(){if(A=[],!t.length)return;A[0]={element:n[0][0],gd:m,prepFn:D,doneFn:P},r.init(A[0])}());function T(e){_=+e.srcElement.getAttribute("data-i"),I=+e.srcElement.getAttribute("data-j"),C[_][I].moveFn=E}function E(e,n){if(t.length){var r=x[_][I][1],i=x[_][I][2],o=t[_],a=o.length;if(f(o)){for(var s=0;s<a;s++)if(s!==I){var l=o[s];l[1]===o[I][1]&&(l[1]=r+e),l[2]===o[I][2]&&(l[2]=i+n)}if(o[I][1]=r+e,o[I][2]=i+n,!f(o))for(var u=0;u<a;u++)for(var c=0;c<o[u].length;c++)o[u][c]=x[_][u][c]}else o[I][1]=r+e,o[I][2]=i+n;y()}}function O(){b()}function k(e,n){if(2===e){_=+n.srcElement.getAttribute("data-i"),I=+n.srcElement.getAttribute("data-j");var r=t[_];f(r)||p(r)||function(){if(t.length&&t[_]&&t[_].length){for(var e=[],n=0;n<t[_].length;n++)n!==I&&e.push(t[_][n]);e.length>1&&(2!==e.length||"Z"!==e[1][0])&&(0===I&&(e[0][0]="M"),t[_]=e,y(),b())}}()}}function L(e,n){!function(e,n){if(t.length)for(var r=0;r<t.length;r++)for(var i=0;i<t[r].length;i++)for(var o=0;o+2<t[r][i].length;o+=2)t[r][i][o+1]=x[r][i][o+1]+e,t[r][i][o+2]=x[r][i][o+2]+n}(e,n),y()}function D(e){(_=+e.srcElement.getAttribute("data-i"))||(_=0),A[_].moveFn=L}function P(){b()}}},function(e,t,n){"use strict";var r=n(388),i=r.drawMode,o=r.openMode,a=n(551),s=a.i000,l=a.i090,u=a.i180,c=a.i270,d=a.cos45,h=a.sin45,f=n(552),p=f.p2r,g=f.r2p,v=n(299).clearSelect,m=n(389),y=m.readPaths,b=m.writePaths,C=m.ellipseOver;e.exports=function(e,t){if(e.length){var n=e[0][0];if(n){var r=n.getAttribute("d"),a=t.gd,f=a._fullLayout.newshape,m=t.plotinfo,A=m.xaxis,_=m.yaxis,I=!!m.domain||!m.xaxis,x=!!m.domain||!m.yaxis,w=t.isActiveShape,M=t.dragmode,S=(a.layout||{}).shapes||[];if(!i(M)&&void 0!==w){var T=a._fullLayout._activeShapeIndex;if(T<S.length)switch(a._fullLayout.shapes[T].type){case"rect":M="drawrect";break;case"circle":M="drawcircle";break;case"line":M="drawline";break;case"path":var E=S[T].path||"";M="Z"===E[E.length-1]?"drawclosedpath":"drawopenpath"}}var O,k=o(M),L=y(r,a,m,w),D={editable:!0,xref:I?"paper":A._id,yref:x?"paper":_._id,layer:f.layer,opacity:f.opacity,line:{color:f.line.color,width:f.line.width,dash:f.line.dash}};if(k||(D.fillcolor=f.fillcolor,D.fillrule=f.fillrule),1===L.length&&(O=L[0]),O&&"drawrect"===M)D.type="rect",D.x0=O[0][1],D.y0=O[0][2],D.x1=O[2][1],D.y1=O[2][2];else if(O&&"drawline"===M)D.type="line",D.x0=O[0][1],D.y0=O[0][2],D.x1=O[1][1],D.y1=O[1][2];else if(O&&"drawcircle"===M){D.type="circle";var P=O[s][1],N=O[l][1],j=O[u][1],R=O[c][1],z=O[s][2],F=O[l][2],B=O[u][2],H=O[c][2],U=m.xaxis&&("date"===m.xaxis.type||"log"===m.xaxis.type),V=m.yaxis&&("date"===m.yaxis.type||"log"===m.yaxis.type);U&&(P=g(m.xaxis,P),N=g(m.xaxis,N),j=g(m.xaxis,j),R=g(m.xaxis,R)),V&&(z=g(m.yaxis,z),F=g(m.yaxis,F),B=g(m.yaxis,B),H=g(m.yaxis,H));var Y=(N+R)/2,W=(z+B)/2,G=C({x0:Y,y0:W,x1:Y+(R-N+j-P)/2*d,y1:W+(H-F+B-z)/2*h});U&&(G.x0=p(m.xaxis,G.x0),G.x1=p(m.xaxis,G.x1)),V&&(G.y0=p(m.yaxis,G.y0),G.y1=p(m.yaxis,G.y1)),D.x0=G.x0,D.y0=G.y0,D.x1=G.x1,D.y1=G.y1}else D.type="path",A&&_&&function(e,t,n){var r="date"===t.type,i="date"===n.type;if(!r&&!i)return e;for(var o=0;o<e.length;o++)for(var a=0;a<e[o].length;a++)for(var s=0;s+2<e[o][a].length;s+=2)r&&(e[o][a][s+1]=e[o][a][s+1].replace(" ","_")),i&&(e[o][a][s+2]=e[o][a][s+2].replace(" ","_"))}(L,A,_),D.path=b(L),O=null;v(a);for(var X=t.editHelpers,q=(X||{}).modifyItem,Z=[],Q=0;Q<S.length;Q++){var K=a._fullLayout.shapes[Q];if(Z[Q]=K._input,void 0!==w&&Q===a._fullLayout._activeShapeIndex){var J=D;switch(K.type){case"line":case"rect":case"circle":q("x0",J.x0),q("x1",J.x1),q("y0",J.y0),q("y1",J.y1);break;case"path":q("path",J.path)}}}return void 0===w?(Z.push(D),Z):X?X.getUpdateObj():{}}}}},function(e,t,n){"use strict";var r=n(97).FROM_BL;e.exports=function(e,t,n){void 0===n&&(n=r[e.constraintoward||"center"]);var i=[e.r2l(e.range[0]),e.r2l(e.range[1])],o=i[0]+(i[1]-i[0])*n;e.range=e._input.range=[e.l2r(o+(i[0]-o)*t),e.l2r(o+(i[1]-o)*t)],e.setScale()}},function(e,t,n){"use strict";var r=n(528).dot,i=n(72).BADNUM,o=e.exports={};o.tester=function(e){var t,n=e.slice(),r=n[0][0],o=r,a=n[0][1],s=a;for(n.push(n[0]),t=1;t<n.length;t++)r=Math.min(r,n[t][0]),o=Math.max(o,n[t][0]),a=Math.min(a,n[t][1]),s=Math.max(s,n[t][1]);var l,u=!1;5===n.length&&(n[0][0]===n[1][0]?n[2][0]===n[3][0]&&n[0][1]===n[3][1]&&n[1][1]===n[2][1]&&(u=!0,l=function(e){return e[0]===n[0][0]}):n[0][1]===n[1][1]&&n[2][1]===n[3][1]&&n[0][0]===n[3][0]&&n[1][0]===n[2][0]&&(u=!0,l=function(e){return e[1]===n[0][1]}));var c=!0,d=n[0];for(t=1;t<n.length;t++)if(d[0]!==n[t][0]||d[1]!==n[t][1]){c=!1;break}return{xmin:r,xmax:o,ymin:a,ymax:s,pts:n,contains:u?function(e,t){var n=e[0],u=e[1];return!(n===i||n<r||n>o||u===i||u<a||u>s)&&(!t||!l(e))}:function(e,t){var l=e[0],u=e[1];if(l===i||l<r||l>o||u===i||u<a||u>s)return!1;var c,d,h,f,p,g=n.length,v=n[0][0],m=n[0][1],y=0;for(c=1;c<g;c++)if(d=v,h=m,v=n[c][0],m=n[c][1],!(l<(f=Math.min(d,v))||l>Math.max(d,v)||u>Math.max(h,m)))if(u<Math.min(h,m))l!==f&&y++;else{if(u===(p=v===d?u:h+(l-d)*(m-h)/(v-d)))return 1!==c||!t;u<=p&&l!==f&&y++}return y%2==1},isRect:u,degenerate:c}},o.isSegmentBent=function(e,t,n,i){var o,a,s,l=e[t],u=[e[n][0]-l[0],e[n][1]-l[1]],c=r(u,u),d=Math.sqrt(c),h=[-u[1]/d,u[0]/d];for(o=t+1;o<n;o++)if(a=[e[o][0]-l[0],e[o][1]-l[1]],(s=r(a,u))<0||s>c||Math.abs(r(a,h))>i)return!0;return!1},o.filter=function(e,t){var n=[e[0]],r=0,i=0;function a(a){e.push(a);var s=n.length,l=r;n.splice(i+1);for(var u=l+1;u<e.length;u++)(u===e.length-1||o.isSegmentBent(e,l,u+1,t))&&(n.push(e[u]),n.length<s-2&&(r=u,i=n.length-1),l=u)}e.length>1&&a(e.pop());return{addPt:a,raw:e,filtered:n}}},function(e,t,n){"use strict";var r=n(42),i=n(979),o=n(82),a=n(10),s=n(256),l=n(555),u=n(556),c=n(964).version,d={format:{valType:"enumerated",values:["png","jpeg","webp","svg","full-json"],dflt:"png"},width:{valType:"number",min:1},height:{valType:"number",min:1},scale:{valType:"number",min:0,dflt:1},setBackground:{valType:"any",dflt:!1},imageDataOnly:{valType:"boolean",dflt:!1}};e.exports=function(e,t){var n,h,f,p;function g(e){return!(e in t)||a.validate(t[e],d[e])}if(t=t||{},a.isPlainObject(e)?(n=e.data||[],h=e.layout||{},f=e.config||{},p={}):(e=a.getGraphDiv(e),n=a.extendDeep([],e.data),h=a.extendDeep({},e.layout),f=e._context,p=e._fullLayout||{}),!g("width")&&null!==t.width||!g("height")&&null!==t.height)throw new Error("Height and width should be pixel values.");if(!g("format"))throw new Error("Export format is not "+a.join2(d.format.values,", "," or ")+".");var v={};function m(e,n){return a.coerce(t,v,d,e,n)}var y=m("format"),b=m("width"),C=m("height"),A=m("scale"),_=m("setBackground"),I=m("imageDataOnly"),x=document.createElement("div");x.style.position="absolute",x.style.left="-5000px",document.body.appendChild(x);var w=a.extendFlat({},h);b?w.width=b:null===t.width&&r(p.width)&&(w.width=p.width),C?w.height=C:null===t.height&&r(p.height)&&(w.height=p.height);var M=a.extendFlat({},f,{_exportedPlot:!0,staticPlot:!0,setBackground:_}),S=s.getRedrawFunc(x);function T(){return new Promise((function(e){setTimeout(e,s.getDelay(x._fullLayout))}))}function E(){return new Promise((function(e,t){var n=l(x,y,A),r=x._fullLayout.width,d=x._fullLayout.height;function h(){i.purge(x),document.body.removeChild(x)}if("full-json"===y){var f=o.graphJson(x,!1,"keepdata","object",!0,!0);return f.version=c,f=JSON.stringify(f),h(),e(I?f:s.encodeJSON(f))}if(h(),"svg"===y)return e(I?n:s.encodeSVG(n));var p=document.createElement("canvas");p.id=a.randstr(),u({format:y,width:r,height:d,scale:A,canvas:p,svg:n,promise:!0}).then(e).catch(t)}))}return new Promise((function(e,t){i.plot(x,n,w,M).then(S).then(T).then(E).then((function(t){e(function(e){return I?e.replace(s.IMAGE_URL_PREFIX,""):e}(t))})).catch((function(e){t(e)}))}))}},function(e,t,n){"use strict";var r=n(10),i=n(1006),o=n(1717),a=n(256);e.exports=function(e,t){var n;return r.isPlainObject(e)||(n=r.getGraphDiv(e)),(t=t||{}).format=t.format||"png",t.width=t.width||null,t.height=t.height||null,t.imageDataOnly=!0,new Promise((function(s,l){n&&n._snapshotInProgress&&l(new Error("Snapshotting already in progress.")),r.isIE()&&"svg"!==t.format&&l(new Error(a.MSG_IE_BAD_FORMAT)),n&&(n._snapshotInProgress=!0);var u=i(e,t),c=t.filename||e.fn||"newplot";c+="."+t.format.replace("-","."),u.then((function(e){return n&&(n._snapshotInProgress=!1),o(e,c,t.format)})).then((function(e){s(e)})).catch((function(e){n&&(n._snapshotInProgress=!1),l(e)}))}))}},function(e,t,n){"use strict";var r=n(10),i=n(27);e.exports=function(e,t,n,o){var a,s=o("x"),l=o("y");if(i.getComponentMethod("calendars","handleTraceDefaults")(e,t,["x","y"],n),s){var u=r.minRowLength(s);l?a=Math.min(u,r.minRowLength(l)):(a=u,o("y0"),o("dy"))}else{if(!l)return 0;a=r.minRowLength(l),o("x0"),o("dx")}return t._length=a,a}},function(e,t,n){"use strict";var r=n(10).dateTick0,i=n(72).ONEWEEK;function o(e,t){return r(t,e%i==0?1:0)}e.exports=function(e,t,n,r,i){if(i||(i={x:!0,y:!0}),i.x){var a=r("xperiod");a&&(r("xperiod0",o(a,t.xcalendar)),r("xperiodalignment"))}if(i.y){var s=r("yperiod");s&&(r("yperiod0",o(s,t.ycalendar)),r("yperiodalignment"))}}},function(e,t,n){"use strict";var r=n(41),i=n(149).hasColorscale,o=n(213),a=n(125);e.exports=function(e,t,n,s,l,u){var c=a.isBubble(e),d=(e.line||{}).color;(u=u||{},d&&(n=d),l("marker.symbol"),l("marker.opacity",c?.7:1),l("marker.size"),l("marker.color",n),i(e,"marker")&&o(e,t,s,l,{prefix:"marker.",cLetter:"c"}),u.noSelect||(l("selected.marker.color"),l("unselected.marker.color"),l("selected.marker.size"),l("unselected.marker.size")),u.noLine||(l("marker.line.color",d&&!Array.isArray(d)&&t.marker.color!==d?d:c?r.background:r.defaultLine),i(e,"marker.line")&&o(e,t,s,l,{prefix:"marker.line.",cLetter:"c"}),l("marker.line.width",c?1:0)),c&&(l("marker.sizeref"),l("marker.sizemin"),l("marker.sizemode")),u.gradient)&&("none"!==l("marker.gradient.type")&&l("marker.gradient.color"))}},function(e,t,n){"use strict";var r=n(10).isArrayOrTypedArray,i=n(149).hasColorscale,o=n(213);e.exports=function(e,t,n,a,s,l){var u=(e.marker||{}).color;(s("line.color",n),i(e,"line"))?o(e,t,a,s,{prefix:"line.",cLetter:"c"}):s("line.color",!r(u)&&u||n);s("line.width"),(l||{}).noDash||s("line.dash")}},function(e,t,n){"use strict";var r=n(10);e.exports=function(e,t,n,i,o){o=o||{},i("textposition"),r.coerceFont(i,"textfont",n.font),o.noSelect||(i("selected.textfont.color"),i("unselected.textfont.color"))}},function(e,t,n){"use strict";var r=n(42),i=n(10),o=n(55),a=n(557),s=n(72).BADNUM,l=n(125),u=n(1014),c=n(558),d=n(1015);function h(e,t,n,r,i,a,s){var u=t._length,c=e._fullLayout,d=n._id,h=r._id,f=c._firstScatter[g(t)]===t.uid,p=(v(t,c,n,r)||{}).orientation,m=t.fill;n._minDtick=0,r._minDtick=0;var y={padded:!0},b={padded:!0};s&&(y.ppad=b.ppad=s);var C=u<2||i[0]!==i[u-1]||a[0]!==a[u-1];C&&("tozerox"===m||"tonextx"===m&&(f||"h"===p))?y.tozero=!0:(t.error_y||{}).visible||"tonexty"!==m&&"tozeroy"!==m&&(l.hasMarkers(t)||l.hasText(t))||(y.padded=!1,y.ppad=0),C&&("tozeroy"===m||"tonexty"===m&&(f||"v"===p))?b.tozero=!0:"tonextx"!==m&&"tozerox"!==m||(b.padded=!1),d&&(t._extremes[d]=o.findExtremes(n,i,y)),h&&(t._extremes[h]=o.findExtremes(r,a,b))}function f(e,t){if(l.hasMarkers(e)){var n,r=e.marker,a=1.6*(e.marker.sizeref||1);if(n="area"===e.marker.sizemode?function(e){return Math.max(Math.sqrt((e||0)/a),3)}:function(e){return Math.max((e||0)/a,3)},i.isArrayOrTypedArray(r.size)){var s={type:"linear"};o.setConvert(s);for(var u=s.makeCalcdata(e.marker,"size"),c=new Array(t),d=0;d<t;d++)c[d]=n(u[d]);return c}return n(r.size)}}function p(e,t){var n=g(t),r=e._firstScatter;r[n]||(r[n]=t.uid)}function g(e){var t=e.stackgroup;return e.xaxis+e.yaxis+e.type+(t?"-"+t:"")}function v(e,t,n,r){var i=e.stackgroup;if(i){var o=t._scatterStackOpts[n._id+r._id][i],a="v"===o.orientation?r:n;return"linear"===a.type||"log"===a.type?o:void 0}}e.exports={calc:function(e,t){var n,l,g,m,y,b,C=e._fullLayout,A=o.getFromId(e,t.xaxis||"x"),_=o.getFromId(e,t.yaxis||"y"),I=A.makeCalcdata(t,"x"),x=_.makeCalcdata(t,"y"),w=a(t,A,"x",I),M=a(t,_,"y",x),S=t._length,T=new Array(S),E=t.ids,O=v(t,C,A,_),k=!1;p(C,t);var L,D="x",P="y";O?(i.pushUnique(O.traceIndices,t._expandedIndex),(n="v"===O.orientation)?(P="s",L="x"):(D="s",L="y"),y="interpolate"===O.stackgaps):h(e,t,A,_,w,M,f(t,S));var N=!!t.xperiodalignment,j=!!t.yperiodalignment;for(l=0;l<S;l++){var R=T[l]={},z=r(w[l]),F=r(M[l]);z&&F?(R[D]=w[l],R[P]=M[l],N&&(R.orig_x=I[l]),j&&(R.orig_y=x[l])):O&&(n?z:F)?(R[L]=n?w[l]:M[l],R.gap=!0,y?(R.s=s,k=!0):R.s=0):R[D]=R[P]=s,E&&(R.id=String(E[l]))}if(c(T,t),u(e,t),d(T,t),O){for(l=0;l<T.length;)T[l][L]===s?T.splice(l,1):l++;if(i.sort(T,(function(e,t){return e[L]-t[L]||e.i-t.i})),k){for(l=0;l<T.length-1&&T[l].gap;)l++;for((b=T[l].s)||(b=T[l].s=0),g=0;g<l;g++)T[g].s=b;for(m=T.length-1;m>l&&T[m].gap;)m--;for(b=T[m].s,g=T.length-1;g>m;g--)T[g].s=b;for(;l<m;)if(T[++l].gap){for(g=l+1;T[g].gap;)g++;for(var B=T[l-1][L],H=T[l-1].s,U=(T[g].s-H)/(T[g][L]-B);l<g;)T[l].s=H+(T[l][L]-B)*U,l++}}}return T},calcMarkerSize:f,calcAxisExpansion:h,setFirstScatter:p,getStackOpts:v}},function(e,t,n){"use strict";var r=n(149).hasColorscale,i=n(298),o=n(125);e.exports=function(e,t){o.hasLines(t)&&r(t,"line")&&i(e,t,{vals:t.line.color,containerStr:"line",cLetter:"c"}),o.hasMarkers(t)&&(r(t,"marker")&&i(e,t,{vals:t.marker.color,containerStr:"marker",cLetter:"c"}),r(t,"marker.line")&&i(e,t,{vals:t.marker.line.color,containerStr:"marker.line",cLetter:"c"}))}},function(e,t,n){"use strict";var r=n(10);e.exports=function(e,t){r.isArrayOrTypedArray(t.selectedpoints)&&r.tagSelected(e,t)}},function(e,t,n){"use strict";e.exports={container:"marker",min:"cmin",max:"cmax"}},function(e,t,n){"use strict";var r=n(39),i=n(56),o=n(27);function a(e,t,n){i.pointStyle(e.selectAll("path.point"),t,n)}function s(e,t,n){i.textPointStyle(e.selectAll("text"),t,n)}e.exports={style:function(e){var t=r.select(e).selectAll("g.trace.scatter");t.style("opacity",(function(e){return e[0].trace.opacity})),t.selectAll("g.points").each((function(t){a(r.select(this),t.trace||t[0].trace,e)})),t.selectAll("g.text").each((function(t){s(r.select(this),t.trace||t[0].trace,e)})),t.selectAll("g.trace path.js-line").call(i.lineGroupStyle),t.selectAll("g.trace path.js-fill").call(i.fillGroupStyle),o.getComponentMethod("errorbars","style")(t)},stylePoints:a,styleText:s,styleOnSelect:function(e,t,n){var r=t[0].trace;r.selectedpoints?(i.selectedPointStyle(n.selectAll("path.point"),r),i.selectedTextStyle(n.selectAll("text"),r)):(a(n,r,e),s(n,r,e))}}},function(e,t,n){"use strict";var r=n(27).traceIs,i=n(545);function o(e){return{v:"x",h:"y"}[e.orientation||"v"]}function a(e,t){var n=o(e),i=r(e,"box-violin"),a=r(e._fullInput||{},"candlestick");return i&&!a&&t===n&&void 0===e[n]&&void 0===e[n+"0"]}e.exports=function(e,t,n,s){n("autotypenumbers",s.autotypenumbersDflt),"-"===n("type",(s.splomStash||{}).type)&&(!function(e,t){if("-"!==e.type)return;var n,s=e._id,l=s.charAt(0);-1!==s.indexOf("scene")&&(s=l);var u=function(e,t,n){for(var r=0;r<e.length;r++){var i=e[r];if("splom"===i.type&&i._length>0&&(i["_"+n+"axes"]||{})[t])return i;if((i[n+"axis"]||n)===t){if(a(i,n))return i;if((i[n]||[]).length||i[n+"0"])return i}}}(t,s,l);if(!u)return;if("histogram"===u.type&&l==={v:"y",h:"x"}[u.orientation||"v"])return void(e.type="linear");var c=l+"calendar",d=u[c],h={noMultiCategory:!r(u,"cartesian")||r(u,"noMultiCategory")};"box"===u.type&&u._hasPreCompStats&&l==={h:"x",v:"y"}[u.orientation||"v"]&&(h.noMultiCategory=!0);if(h.autotypenumbers=e.autotypenumbers,a(u,l)){var f=o(u),p=[];for(n=0;n<t.length;n++){var g=t[n];r(g,"box-violin")&&(g[l+"axis"]||l)===s&&(void 0!==g[f]?p.push(g[f][0]):void 0!==g.name?p.push(g.name):p.push("text"),g[c]!==d&&(d=void 0))}e.type=i(p,d,h)}else if("splom"===u.type){var v=u.dimensions[u._axesDim[s]];v.visible&&(e.type=i(v.values,d,h))}else e.type=i(u[l]||[u[l+"0"]],d,h)}(t,s.data),"-"===t.type?t.type="linear":e.type=t.type)}},function(e,t,n){"use strict";e.exports=function(e,t,n,r){if("category"===t.type){var i,o=e.categoryarray,a=Array.isArray(o)&&o.length>0;a&&(i="array");var s,l=n("categoryorder",i);"array"===l&&(s=n("categoryarray")),a||"array"!==l||(l=t.categoryorder="trace"),"trace"===l?t._initialCategories=[]:"array"===l?t._initialCategories=s.slice():(s=function(e,t){var n,r,i,o=t.dataAttr||e._id.charAt(0),a={};if(t.axData)n=t.axData;else for(n=[],r=0;r<t.data.length;r++){var s=t.data[r];s[o+"axis"]===e._id&&n.push(s)}for(r=0;r<n.length;r++){var l=n[r][o];for(i=0;i<l.length;i++){var u=l[i];null!=u&&(a[u]=1)}}return Object.keys(a)}(t,r).sort(),"category ascending"===l?t._initialCategories=s:"category descending"===l&&(t._initialCategories=s.reverse()))}}},function(e,t,n){"use strict";var r=n(42),i=n(10);e.exports=function(e,t,n,o){var a,s,l,u,c=o.counterAxes||[],d=o.overlayableAxes||[],h=o.letter,f=o.grid;f&&(s=f._domains[h][f._axisMap[t._id]],a=f._anchors[t._id],s&&(l=f[h+"side"].split(" ")[0],u=f.domain[h]["right"===l||"top"===l?1:0])),s=s||[0,1],a=a||(r(e.position)?"free":c[0]||"free"),l=l||("x"===h?"bottom":"left"),u=u||0,"free"===i.coerce(e,t,{anchor:{valType:"enumerated",values:["free"].concat(c),dflt:a}},"anchor")&&n("position",u),i.coerce(e,t,{side:{valType:"enumerated",values:"x"===h?["bottom","top"]:["left","right"],dflt:l}},"side");var p=!1;if(d.length&&(p=i.coerce(e,t,{overlaying:{valType:"enumerated",values:[!1].concat(d),dflt:!1}},"overlaying")),!p){var g=n("domain",s);g[0]>g[1]-1/4096&&(t.domain=s),i.noneOrAll(e.domain,t.domain,s)}return n("layer"),t}},function(e,t,n){"use strict";e.exports=[{path:"",backoff:0},{path:"M-2.4,-3V3L0.6,0Z",backoff:.6},{path:"M-3.7,-2.5V2.5L1.3,0Z",backoff:1.3},{path:"M-4.45,-3L-1.65,-0.2V0.2L-4.45,3L1.55,0Z",backoff:1.55},{path:"M-2.2,-2.2L-0.2,-0.2V0.2L-2.2,2.2L-1.4,3L1.6,0L-1.4,-3Z",backoff:1.6},{path:"M-4.4,-2.1L-0.6,-0.2V0.2L-4.4,2.1L-4,3L2,0L-4,-3Z",backoff:2},{path:"M2,0A2,2 0 1,1 0,-2A2,2 0 0,1 2,0Z",backoff:0,noRotate:!0},{path:"M2,2V-2H-2V2Z",backoff:0,noRotate:!0}]},function(e,t,n){"use strict";var r=n(10),i=n(41);e.exports=function(e,t,n,o){o("opacity");var a=o("bgcolor"),s=o("bordercolor"),l=i.opacity(s);o("borderpad");var u=o("borderwidth"),c=o("showarrow");if(o("text",c?" ":n._dfltTitle.annotation),o("textangle"),r.coerceFont(o,"font",n.font),o("width"),o("align"),o("height")&&o("valign"),c){var d,h,f=o("arrowside");-1!==f.indexOf("end")&&(d=o("arrowhead"),h=o("arrowsize")),-1!==f.indexOf("start")&&(o("startarrowhead",d),o("startarrowsize",h)),o("arrowcolor",l?t.bordercolor:i.defaultLine),o("arrowwidth",2*(l&&u||1)),o("standoff"),o("startstandoff")}var p=o("hovertext"),g=n.hoverlabel||{};if(p){var v=o("hoverlabel.bgcolor",g.bgcolor||(i.opacity(a)?i.rgb(a):i.defaultLine)),m=o("hoverlabel.bordercolor",g.bordercolor||i.contrast(v));r.coerceFont(o,"hoverlabel.font",{family:g.font.family,size:g.font.size,color:g.font.color||m})}o("captureevents",!!p)}},function(e,t,n){"use strict";function r(e,t){var n,r,i=[0,0,0,0];for(n=0;n<4;++n)for(r=0;r<4;++r)i[r]+=e[4*n+r]*t[n];return i}e.exports=function(e,t){return r(e.projection,r(e.view,r(e.model,[t[0],t[1],t[2],1])))}},function(e,t,n){"use strict";var r=n(392),i=n(193).line,o=n(296).dash,a=n(63).extendFlat,s=n(64).templatedArray;n(561);e.exports=s("shape",{visible:{valType:"boolean",dflt:!0,editType:"calc+arraydraw"},type:{valType:"enumerated",values:["circle","rect","path","line"],editType:"calc+arraydraw"},layer:{valType:"enumerated",values:["below","above"],dflt:"above",editType:"arraydraw"},xref:a({},r.xref,{}),xsizemode:{valType:"enumerated",values:["scaled","pixel"],dflt:"scaled",editType:"calc+arraydraw"},xanchor:{valType:"any",editType:"calc+arraydraw"},x0:{valType:"any",editType:"calc+arraydraw"},x1:{valType:"any",editType:"calc+arraydraw"},yref:a({},r.yref,{}),ysizemode:{valType:"enumerated",values:["scaled","pixel"],dflt:"scaled",editType:"calc+arraydraw"},yanchor:{valType:"any",editType:"calc+arraydraw"},y0:{valType:"any",editType:"calc+arraydraw"},y1:{valType:"any",editType:"calc+arraydraw"},path:{valType:"string",editType:"calc+arraydraw"},opacity:{valType:"number",min:0,max:1,dflt:1,editType:"arraydraw"},line:{color:a({},i.color,{editType:"arraydraw"}),width:a({},i.width,{editType:"calc+arraydraw"}),dash:a({},o,{editType:"arraydraw"}),editType:"calc+arraydraw"},fillcolor:{valType:"color",dflt:"rgba(0,0,0,0)",editType:"arraydraw"},fillrule:{valType:"enumerated",values:["evenodd","nonzero"],dflt:"evenodd",editType:"arraydraw"},editable:{valType:"boolean",dflt:!1,editType:"calc+arraydraw"},editType:"arraydraw"})},function(e,t,n){"use strict";var r=n(95),i=n(64).templatedArray;n(561);e.exports=i("image",{visible:{valType:"boolean",dflt:!0,editType:"arraydraw"},source:{valType:"string",editType:"arraydraw"},layer:{valType:"enumerated",values:["below","above"],dflt:"above",editType:"arraydraw"},sizex:{valType:"number",dflt:0,editType:"arraydraw"},sizey:{valType:"number",dflt:0,editType:"arraydraw"},sizing:{valType:"enumerated",values:["fill","contain","stretch"],dflt:"contain",editType:"arraydraw"},opacity:{valType:"number",min:0,max:1,dflt:1,editType:"arraydraw"},x:{valType:"any",dflt:0,editType:"arraydraw"},y:{valType:"any",dflt:0,editType:"arraydraw"},xanchor:{valType:"enumerated",values:["left","center","right"],dflt:"left",editType:"arraydraw"},yanchor:{valType:"enumerated",values:["top","middle","bottom"],dflt:"top",editType:"arraydraw"},xref:{valType:"enumerated",values:["paper",r.idRegex.x.toString()],dflt:"paper",editType:"arraydraw"},yref:{valType:"enumerated",values:["paper",r.idRegex.y.toString()],dflt:"paper",editType:"arraydraw"},editType:"arraydraw"})},function(e,t,n){"use strict";var r=n(94),i=n(133),o=n(63).extendFlat,a=n(124).overrideAll,s=n(537),l=n(64).templatedArray,u=l("button",{visible:{valType:"boolean"},method:{valType:"enumerated",values:["restyle","relayout","animate","update","skip"],dflt:"restyle"},args:{valType:"info_array",freeLength:!0,items:[{valType:"any"},{valType:"any"},{valType:"any"}]},args2:{valType:"info_array",freeLength:!0,items:[{valType:"any"},{valType:"any"},{valType:"any"}]},label:{valType:"string",dflt:""},execute:{valType:"boolean",dflt:!0}});e.exports=a(l("updatemenu",{_arrayAttrRegexps:[/^updatemenus\[(0|[1-9][0-9]+)\]\.buttons/],visible:{valType:"boolean"},type:{valType:"enumerated",values:["dropdown","buttons"],dflt:"dropdown"},direction:{valType:"enumerated",values:["left","right","up","down"],dflt:"down"},active:{valType:"integer",min:-1,dflt:0},showactive:{valType:"boolean",dflt:!0},buttons:u,x:{valType:"number",min:-2,max:3,dflt:-.05},xanchor:{valType:"enumerated",values:["auto","left","center","right"],dflt:"right"},y:{valType:"number",min:-2,max:3,dflt:1},yanchor:{valType:"enumerated",values:["auto","top","middle","bottom"],dflt:"top"},pad:o(s({editType:"arraydraw"}),{}),font:r({}),bgcolor:{valType:"color"},bordercolor:{valType:"color",dflt:i.borderLine},borderwidth:{valType:"number",min:0,dflt:1,editType:"arraydraw"}}),"arraydraw","from-root")},function(e,t,n){"use strict";var r=n(94),i=n(537),o=n(63).extendDeepAll,a=n(124).overrideAll,s=n(382),l=n(64).templatedArray,u=n(393),c=l("step",{visible:{valType:"boolean",dflt:!0},method:{valType:"enumerated",values:["restyle","relayout","animate","update","skip"],dflt:"restyle"},args:{valType:"info_array",freeLength:!0,items:[{valType:"any"},{valType:"any"},{valType:"any"}]},label:{valType:"string"},value:{valType:"string"},execute:{valType:"boolean",dflt:!0}});e.exports=a(l("slider",{visible:{valType:"boolean",dflt:!0},active:{valType:"number",min:0,dflt:0},steps:c,lenmode:{valType:"enumerated",values:["fraction","pixels"],dflt:"fraction"},len:{valType:"number",min:0,dflt:1},x:{valType:"number",min:-2,max:3,dflt:0},pad:o(i({editType:"arraydraw"}),{},{t:{dflt:20}}),xanchor:{valType:"enumerated",values:["auto","left","center","right"],dflt:"left"},y:{valType:"number",min:-2,max:3,dflt:0},yanchor:{valType:"enumerated",values:["auto","top","middle","bottom"],dflt:"top"},transition:{duration:{valType:"number",min:0,dflt:150},easing:{valType:"enumerated",values:s.transition.easing.values,dflt:"cubic-in-out"}},currentvalue:{visible:{valType:"boolean",dflt:!0},xanchor:{valType:"enumerated",values:["left","center","right"],dflt:"left"},offset:{valType:"number",dflt:10},prefix:{valType:"string"},suffix:{valType:"string"},font:r({})},font:r({}),activebgcolor:{valType:"color",dflt:u.gripBgActiveColor},bgcolor:{valType:"color",dflt:u.railBgColor},bordercolor:{valType:"color",dflt:u.railBorderColor},borderwidth:{valType:"number",min:0,dflt:u.railBorderWidth},ticklen:{valType:"number",min:0,dflt:u.tickLength},tickcolor:{valType:"color",dflt:u.tickColor},tickwidth:{valType:"number",min:0,dflt:1},minorticklen:{valType:"number",min:0,dflt:u.minorTickLength}}),"arraydraw","from-root")},function(e,t,n){"use strict";e.exports={_isSubplotObj:!0,rangemode:{valType:"enumerated",values:["auto","fixed","match"],dflt:"match",editType:"calc"},range:{valType:"info_array",items:[{valType:"any",editType:"plot"},{valType:"any",editType:"plot"}],editType:"plot"},editType:"calc"}},function(e,t,n){"use strict";e.exports={yPad:.02,minButtonWidth:30,rx:3,ry:3,lightAmount:25,darkAmount:10}},function(e,t,n){"use strict";e.exports={visible:{valType:"boolean",editType:"calc"},type:{valType:"enumerated",values:["percent","constant","sqrt","data"],editType:"calc"},symmetric:{valType:"boolean",editType:"calc"},array:{valType:"data_array",editType:"calc"},arrayminus:{valType:"data_array",editType:"calc"},value:{valType:"number",min:0,dflt:10,editType:"calc"},valueminus:{valType:"number",min:0,dflt:10,editType:"calc"},traceref:{valType:"integer",min:0,dflt:0,editType:"style"},tracerefminus:{valType:"integer",min:0,dflt:0,editType:"style"},copy_ystyle:{valType:"boolean",editType:"plot"},copy_zstyle:{valType:"boolean",editType:"style"},color:{valType:"color",editType:"style"},thickness:{valType:"number",min:0,dflt:2,editType:"style"},width:{valType:"number",min:0,editType:"plot"},editType:"calc",_deprecated:{opacity:{valType:"number",editType:"style"}}}},function(e,t,n){"use strict";function r(e,t){return"percent"===e?function(e){return Math.abs(e*t/100)}:"constant"===e?function(){return Math.abs(t)}:"sqrt"===e?function(e){return Math.sqrt(Math.abs(e))}:void 0}e.exports=function(e){var t=e.type,n=e.symmetric;if("data"===t){var i=e.array||[];if(n)return function(e,t){var n=+i[t];return[n,n]};var o=e.arrayminus||[];return function(e,t){var n=+i[t],r=+o[t];return isNaN(n)&&isNaN(r)?[NaN,NaN]:[r||0,n||0]}}var a=r(t,e.value),s=r(t,e.valueminus);return n||void 0===e.valueminus?function(e){var t=a(e);return[t,t]}:function(e){return[s(e),a(e)]}}},function(e,t,n){"use strict";var r=n(27),i=n(10),o=i.extendFlat,a=i.extendDeep;function s(e){var t;switch(e){case"themes__thumb":t={autosize:!0,width:150,height:150,title:{text:""},showlegend:!1,margin:{l:5,r:5,t:5,b:5,pad:0},annotations:[]};break;case"thumbnail":t={title:{text:""},hidesources:!0,showlegend:!1,borderwidth:0,bordercolor:"",margin:{l:1,r:1,t:1,b:1,pad:0},annotations:[]};break;default:t={}}return t}e.exports=function(e,t){var n;e.framework&&e.framework.isPolar&&(e=e.framework.getConfig());var i,l=e.data,u=e.layout,c=a([],l),d=a({},u,s(t.tileClass)),h=e._context||{};if(t.width&&(d.width=t.width),t.height&&(d.height=t.height),"thumbnail"===t.tileClass||"themes__thumb"===t.tileClass){d.annotations=[];var f=Object.keys(d);for(n=0;n<f.length;n++)i=f[n],["xaxis","yaxis","zaxis"].indexOf(i.slice(0,5))>-1&&(d[f[n]].title={text:""});for(n=0;n<c.length;n++){var p=c[n];p.showscale=!1,p.marker&&(p.marker.showscale=!1),r.traceIs(p,"pie-like")&&(p.textposition="none")}}if(Array.isArray(t.annotations))for(n=0;n<t.annotations.length;n++)d.annotations.push(t.annotations[n]);var g=Object.keys(d).filter((function(e){return e.match(/^scene\d*$/)}));if(g.length){var v={};for("thumbnail"===t.tileClass&&(v={title:{text:""},showaxeslabels:!1,showticklabels:!1,linetickenable:!1}),n=0;n<g.length;n++){var m=d[g[n]];m.xaxis||(m.xaxis={}),m.yaxis||(m.yaxis={}),m.zaxis||(m.zaxis={}),o(m.xaxis,v),o(m.yaxis,v),o(m.zaxis,v),m._scene=null}}var y=document.createElement("div");t.tileClass&&(y.className=t.tileClass);var b={gd:y,td:y,layout:d,data:c,config:{staticPlot:void 0===t.staticPlot||t.staticPlot,plotGlPixelRatio:void 0===t.plotGlPixelRatio?2:t.plotGlPixelRatio,displaylogo:t.displaylogo||!1,showLink:t.showLink||!1,showTips:t.showTips||!1,mapboxAccessToken:h.mapboxAccessToken}};return"transparent"!==t.setBackground&&(b.config.setBackground=t.setBackground||"opaque"),b.gd.defaultLayout=s(t.tileClass),b}},function(e,t,n){"use strict";e.exports=function(e,t){switch(void 0===t&&(t=0),typeof e){case"number":if(e>0)return function(e,t){var n,r;for(n=new Array(e),r=0;r<e;++r)n[r]=t;return n}(0|e,t);break;case"object":if("number"==typeof e.length)return function e(t,n,r){var i=0|t[r];if(i<=0)return[];var o,a=new Array(i);if(r===t.length-1)for(o=0;o<i;++o)a[o]=n;else for(o=0;o<i;++o)a[o]=e(t,n,r+1);return a}(e,t,0)}return[]}},function(e,t,n){"use strict";(function(e){
/*!
 * The buffer module from node.js, for the browser.
 *
 * @author   Feross Aboukhadijeh <http://feross.org>
 * @license  MIT
 */
var r=n(437),i=n(1790),o=n(1791);function a(){return l.TYPED_ARRAY_SUPPORT?2147483647:1073741823}function s(e,t){if(a()<t)throw new RangeError("Invalid typed array length");return l.TYPED_ARRAY_SUPPORT?(e=new Uint8Array(t)).__proto__=l.prototype:(null===e&&(e=new l(t)),e.length=t),e}function l(e,t,n){if(!(l.TYPED_ARRAY_SUPPORT||this instanceof l))return new l(e,t,n);if("number"==typeof e){if("string"==typeof t)throw new Error("If encoding is specified then the first argument must be a string");return d(this,e)}return u(this,e,t,n)}function u(e,t,n,r){if("number"==typeof t)throw new TypeError('"value" argument must not be a number');return"undefined"!=typeof ArrayBuffer&&t instanceof ArrayBuffer?function(e,t,n,r){if(t.byteLength,n<0||t.byteLength<n)throw new RangeError("'offset' is out of bounds");if(t.byteLength<n+(r||0))throw new RangeError("'length' is out of bounds");t=void 0===n&&void 0===r?new Uint8Array(t):void 0===r?new Uint8Array(t,n):new Uint8Array(t,n,r);l.TYPED_ARRAY_SUPPORT?(e=t).__proto__=l.prototype:e=h(e,t);return e}(e,t,n,r):"string"==typeof t?function(e,t,n){"string"==typeof n&&""!==n||(n="utf8");if(!l.isEncoding(n))throw new TypeError('"encoding" must be a valid string encoding');var r=0|p(t,n),i=(e=s(e,r)).write(t,n);i!==r&&(e=e.slice(0,i));return e}(e,t,n):function(e,t){if(l.isBuffer(t)){var n=0|f(t.length);return 0===(e=s(e,n)).length||t.copy(e,0,0,n),e}if(t){if("undefined"!=typeof ArrayBuffer&&t.buffer instanceof ArrayBuffer||"length"in t)return"number"!=typeof t.length||(r=t.length)!=r?s(e,0):h(e,t);if("Buffer"===t.type&&o(t.data))return h(e,t.data)}var r;throw new TypeError("First argument must be a string, Buffer, ArrayBuffer, Array, or array-like object.")}(e,t)}function c(e){if("number"!=typeof e)throw new TypeError('"size" argument must be a number');if(e<0)throw new RangeError('"size" argument must not be negative')}function d(e,t){if(c(t),e=s(e,t<0?0:0|f(t)),!l.TYPED_ARRAY_SUPPORT)for(var n=0;n<t;++n)e[n]=0;return e}function h(e,t){var n=t.length<0?0:0|f(t.length);e=s(e,n);for(var r=0;r<n;r+=1)e[r]=255&t[r];return e}function f(e){if(e>=a())throw new RangeError("Attempt to allocate Buffer larger than maximum size: 0x"+a().toString(16)+" bytes");return 0|e}function p(e,t){if(l.isBuffer(e))return e.length;if("undefined"!=typeof ArrayBuffer&&"function"==typeof ArrayBuffer.isView&&(ArrayBuffer.isView(e)||e instanceof ArrayBuffer))return e.byteLength;"string"!=typeof e&&(e=""+e);var n=e.length;if(0===n)return 0;for(var r=!1;;)switch(t){case"ascii":case"latin1":case"binary":return n;case"utf8":case"utf-8":case void 0:return B(e).length;case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return 2*n;case"hex":return n>>>1;case"base64":return H(e).length;default:if(r)return B(e).length;t=(""+t).toLowerCase(),r=!0}}function g(e,t,n){var r=!1;if((void 0===t||t<0)&&(t=0),t>this.length)return"";if((void 0===n||n>this.length)&&(n=this.length),n<=0)return"";if((n>>>=0)<=(t>>>=0))return"";for(e||(e="utf8");;)switch(e){case"hex":return E(this,t,n);case"utf8":case"utf-8":return M(this,t,n);case"ascii":return S(this,t,n);case"latin1":case"binary":return T(this,t,n);case"base64":return w(this,t,n);case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return O(this,t,n);default:if(r)throw new TypeError("Unknown encoding: "+e);e=(e+"").toLowerCase(),r=!0}}function v(e,t,n){var r=e[t];e[t]=e[n],e[n]=r}function m(e,t,n,r,i){if(0===e.length)return-1;if("string"==typeof n?(r=n,n=0):n>2147483647?n=2147483647:n<-2147483648&&(n=-2147483648),n=+n,isNaN(n)&&(n=i?0:e.length-1),n<0&&(n=e.length+n),n>=e.length){if(i)return-1;n=e.length-1}else if(n<0){if(!i)return-1;n=0}if("string"==typeof t&&(t=l.from(t,r)),l.isBuffer(t))return 0===t.length?-1:y(e,t,n,r,i);if("number"==typeof t)return t&=255,l.TYPED_ARRAY_SUPPORT&&"function"==typeof Uint8Array.prototype.indexOf?i?Uint8Array.prototype.indexOf.call(e,t,n):Uint8Array.prototype.lastIndexOf.call(e,t,n):y(e,[t],n,r,i);throw new TypeError("val must be string, number or Buffer")}function y(e,t,n,r,i){var o,a=1,s=e.length,l=t.length;if(void 0!==r&&("ucs2"===(r=String(r).toLowerCase())||"ucs-2"===r||"utf16le"===r||"utf-16le"===r)){if(e.length<2||t.length<2)return-1;a=2,s/=2,l/=2,n/=2}function u(e,t){return 1===a?e[t]:e.readUInt16BE(t*a)}if(i){var c=-1;for(o=n;o<s;o++)if(u(e,o)===u(t,-1===c?0:o-c)){if(-1===c&&(c=o),o-c+1===l)return c*a}else-1!==c&&(o-=o-c),c=-1}else for(n+l>s&&(n=s-l),o=n;o>=0;o--){for(var d=!0,h=0;h<l;h++)if(u(e,o+h)!==u(t,h)){d=!1;break}if(d)return o}return-1}function b(e,t,n,r){n=Number(n)||0;var i=e.length-n;r?(r=Number(r))>i&&(r=i):r=i;var o=t.length;if(o%2!=0)throw new TypeError("Invalid hex string");r>o/2&&(r=o/2);for(var a=0;a<r;++a){var s=parseInt(t.substr(2*a,2),16);if(isNaN(s))return a;e[n+a]=s}return a}function C(e,t,n,r){return U(B(t,e.length-n),e,n,r)}function A(e,t,n,r){return U(function(e){for(var t=[],n=0;n<e.length;++n)t.push(255&e.charCodeAt(n));return t}(t),e,n,r)}function _(e,t,n,r){return A(e,t,n,r)}function I(e,t,n,r){return U(H(t),e,n,r)}function x(e,t,n,r){return U(function(e,t){for(var n,r,i,o=[],a=0;a<e.length&&!((t-=2)<0);++a)n=e.charCodeAt(a),r=n>>8,i=n%256,o.push(i),o.push(r);return o}(t,e.length-n),e,n,r)}function w(e,t,n){return 0===t&&n===e.length?r.fromByteArray(e):r.fromByteArray(e.slice(t,n))}function M(e,t,n){n=Math.min(e.length,n);for(var r=[],i=t;i<n;){var o,a,s,l,u=e[i],c=null,d=u>239?4:u>223?3:u>191?2:1;if(i+d<=n)switch(d){case 1:u<128&&(c=u);break;case 2:128==(192&(o=e[i+1]))&&(l=(31&u)<<6|63&o)>127&&(c=l);break;case 3:o=e[i+1],a=e[i+2],128==(192&o)&&128==(192&a)&&(l=(15&u)<<12|(63&o)<<6|63&a)>2047&&(l<55296||l>57343)&&(c=l);break;case 4:o=e[i+1],a=e[i+2],s=e[i+3],128==(192&o)&&128==(192&a)&&128==(192&s)&&(l=(15&u)<<18|(63&o)<<12|(63&a)<<6|63&s)>65535&&l<1114112&&(c=l)}null===c?(c=65533,d=1):c>65535&&(c-=65536,r.push(c>>>10&1023|55296),c=56320|1023&c),r.push(c),i+=d}return function(e){var t=e.length;if(t<=4096)return String.fromCharCode.apply(String,e);var n="",r=0;for(;r<t;)n+=String.fromCharCode.apply(String,e.slice(r,r+=4096));return n}(r)}t.Buffer=l,t.SlowBuffer=function(e){+e!=e&&(e=0);return l.alloc(+e)},t.INSPECT_MAX_BYTES=50,l.TYPED_ARRAY_SUPPORT=void 0!==e.TYPED_ARRAY_SUPPORT?e.TYPED_ARRAY_SUPPORT:function(){try{var e=new Uint8Array(1);return e.__proto__={__proto__:Uint8Array.prototype,foo:function(){return 42}},42===e.foo()&&"function"==typeof e.subarray&&0===e.subarray(1,1).byteLength}catch(e){return!1}}(),t.kMaxLength=a(),l.poolSize=8192,l._augment=function(e){return e.__proto__=l.prototype,e},l.from=function(e,t,n){return u(null,e,t,n)},l.TYPED_ARRAY_SUPPORT&&(l.prototype.__proto__=Uint8Array.prototype,l.__proto__=Uint8Array,"undefined"!=typeof Symbol&&Symbol.species&&l[Symbol.species]===l&&Object.defineProperty(l,Symbol.species,{value:null,configurable:!0})),l.alloc=function(e,t,n){return function(e,t,n,r){return c(t),t<=0?s(e,t):void 0!==n?"string"==typeof r?s(e,t).fill(n,r):s(e,t).fill(n):s(e,t)}(null,e,t,n)},l.allocUnsafe=function(e){return d(null,e)},l.allocUnsafeSlow=function(e){return d(null,e)},l.isBuffer=function(e){return!(null==e||!e._isBuffer)},l.compare=function(e,t){if(!l.isBuffer(e)||!l.isBuffer(t))throw new TypeError("Arguments must be Buffers");if(e===t)return 0;for(var n=e.length,r=t.length,i=0,o=Math.min(n,r);i<o;++i)if(e[i]!==t[i]){n=e[i],r=t[i];break}return n<r?-1:r<n?1:0},l.isEncoding=function(e){switch(String(e).toLowerCase()){case"hex":case"utf8":case"utf-8":case"ascii":case"latin1":case"binary":case"base64":case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return!0;default:return!1}},l.concat=function(e,t){if(!o(e))throw new TypeError('"list" argument must be an Array of Buffers');if(0===e.length)return l.alloc(0);var n;if(void 0===t)for(t=0,n=0;n<e.length;++n)t+=e[n].length;var r=l.allocUnsafe(t),i=0;for(n=0;n<e.length;++n){var a=e[n];if(!l.isBuffer(a))throw new TypeError('"list" argument must be an Array of Buffers');a.copy(r,i),i+=a.length}return r},l.byteLength=p,l.prototype._isBuffer=!0,l.prototype.swap16=function(){var e=this.length;if(e%2!=0)throw new RangeError("Buffer size must be a multiple of 16-bits");for(var t=0;t<e;t+=2)v(this,t,t+1);return this},l.prototype.swap32=function(){var e=this.length;if(e%4!=0)throw new RangeError("Buffer size must be a multiple of 32-bits");for(var t=0;t<e;t+=4)v(this,t,t+3),v(this,t+1,t+2);return this},l.prototype.swap64=function(){var e=this.length;if(e%8!=0)throw new RangeError("Buffer size must be a multiple of 64-bits");for(var t=0;t<e;t+=8)v(this,t,t+7),v(this,t+1,t+6),v(this,t+2,t+5),v(this,t+3,t+4);return this},l.prototype.toString=function(){var e=0|this.length;return 0===e?"":0===arguments.length?M(this,0,e):g.apply(this,arguments)},l.prototype.equals=function(e){if(!l.isBuffer(e))throw new TypeError("Argument must be a Buffer");return this===e||0===l.compare(this,e)},l.prototype.inspect=function(){var e="",n=t.INSPECT_MAX_BYTES;return this.length>0&&(e=this.toString("hex",0,n).match(/.{2}/g).join(" "),this.length>n&&(e+=" ... ")),"<Buffer "+e+">"},l.prototype.compare=function(e,t,n,r,i){if(!l.isBuffer(e))throw new TypeError("Argument must be a Buffer");if(void 0===t&&(t=0),void 0===n&&(n=e?e.length:0),void 0===r&&(r=0),void 0===i&&(i=this.length),t<0||n>e.length||r<0||i>this.length)throw new RangeError("out of range index");if(r>=i&&t>=n)return 0;if(r>=i)return-1;if(t>=n)return 1;if(this===e)return 0;for(var o=(i>>>=0)-(r>>>=0),a=(n>>>=0)-(t>>>=0),s=Math.min(o,a),u=this.slice(r,i),c=e.slice(t,n),d=0;d<s;++d)if(u[d]!==c[d]){o=u[d],a=c[d];break}return o<a?-1:a<o?1:0},l.prototype.includes=function(e,t,n){return-1!==this.indexOf(e,t,n)},l.prototype.indexOf=function(e,t,n){return m(this,e,t,n,!0)},l.prototype.lastIndexOf=function(e,t,n){return m(this,e,t,n,!1)},l.prototype.write=function(e,t,n,r){if(void 0===t)r="utf8",n=this.length,t=0;else if(void 0===n&&"string"==typeof t)r=t,n=this.length,t=0;else{if(!isFinite(t))throw new Error("Buffer.write(string, encoding, offset[, length]) is no longer supported");t|=0,isFinite(n)?(n|=0,void 0===r&&(r="utf8")):(r=n,n=void 0)}var i=this.length-t;if((void 0===n||n>i)&&(n=i),e.length>0&&(n<0||t<0)||t>this.length)throw new RangeError("Attempt to write outside buffer bounds");r||(r="utf8");for(var o=!1;;)switch(r){case"hex":return b(this,e,t,n);case"utf8":case"utf-8":return C(this,e,t,n);case"ascii":return A(this,e,t,n);case"latin1":case"binary":return _(this,e,t,n);case"base64":return I(this,e,t,n);case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return x(this,e,t,n);default:if(o)throw new TypeError("Unknown encoding: "+r);r=(""+r).toLowerCase(),o=!0}},l.prototype.toJSON=function(){return{type:"Buffer",data:Array.prototype.slice.call(this._arr||this,0)}};function S(e,t,n){var r="";n=Math.min(e.length,n);for(var i=t;i<n;++i)r+=String.fromCharCode(127&e[i]);return r}function T(e,t,n){var r="";n=Math.min(e.length,n);for(var i=t;i<n;++i)r+=String.fromCharCode(e[i]);return r}function E(e,t,n){var r=e.length;(!t||t<0)&&(t=0),(!n||n<0||n>r)&&(n=r);for(var i="",o=t;o<n;++o)i+=F(e[o]);return i}function O(e,t,n){for(var r=e.slice(t,n),i="",o=0;o<r.length;o+=2)i+=String.fromCharCode(r[o]+256*r[o+1]);return i}function k(e,t,n){if(e%1!=0||e<0)throw new RangeError("offset is not uint");if(e+t>n)throw new RangeError("Trying to access beyond buffer length")}function L(e,t,n,r,i,o){if(!l.isBuffer(e))throw new TypeError('"buffer" argument must be a Buffer instance');if(t>i||t<o)throw new RangeError('"value" argument is out of bounds');if(n+r>e.length)throw new RangeError("Index out of range")}function D(e,t,n,r){t<0&&(t=65535+t+1);for(var i=0,o=Math.min(e.length-n,2);i<o;++i)e[n+i]=(t&255<<8*(r?i:1-i))>>>8*(r?i:1-i)}function P(e,t,n,r){t<0&&(t=4294967295+t+1);for(var i=0,o=Math.min(e.length-n,4);i<o;++i)e[n+i]=t>>>8*(r?i:3-i)&255}function N(e,t,n,r,i,o){if(n+r>e.length)throw new RangeError("Index out of range");if(n<0)throw new RangeError("Index out of range")}function j(e,t,n,r,o){return o||N(e,0,n,4),i.write(e,t,n,r,23,4),n+4}function R(e,t,n,r,o){return o||N(e,0,n,8),i.write(e,t,n,r,52,8),n+8}l.prototype.slice=function(e,t){var n,r=this.length;if((e=~~e)<0?(e+=r)<0&&(e=0):e>r&&(e=r),(t=void 0===t?r:~~t)<0?(t+=r)<0&&(t=0):t>r&&(t=r),t<e&&(t=e),l.TYPED_ARRAY_SUPPORT)(n=this.subarray(e,t)).__proto__=l.prototype;else{var i=t-e;n=new l(i,void 0);for(var o=0;o<i;++o)n[o]=this[o+e]}return n},l.prototype.readUIntLE=function(e,t,n){e|=0,t|=0,n||k(e,t,this.length);for(var r=this[e],i=1,o=0;++o<t&&(i*=256);)r+=this[e+o]*i;return r},l.prototype.readUIntBE=function(e,t,n){e|=0,t|=0,n||k(e,t,this.length);for(var r=this[e+--t],i=1;t>0&&(i*=256);)r+=this[e+--t]*i;return r},l.prototype.readUInt8=function(e,t){return t||k(e,1,this.length),this[e]},l.prototype.readUInt16LE=function(e,t){return t||k(e,2,this.length),this[e]|this[e+1]<<8},l.prototype.readUInt16BE=function(e,t){return t||k(e,2,this.length),this[e]<<8|this[e+1]},l.prototype.readUInt32LE=function(e,t){return t||k(e,4,this.length),(this[e]|this[e+1]<<8|this[e+2]<<16)+16777216*this[e+3]},l.prototype.readUInt32BE=function(e,t){return t||k(e,4,this.length),16777216*this[e]+(this[e+1]<<16|this[e+2]<<8|this[e+3])},l.prototype.readIntLE=function(e,t,n){e|=0,t|=0,n||k(e,t,this.length);for(var r=this[e],i=1,o=0;++o<t&&(i*=256);)r+=this[e+o]*i;return r>=(i*=128)&&(r-=Math.pow(2,8*t)),r},l.prototype.readIntBE=function(e,t,n){e|=0,t|=0,n||k(e,t,this.length);for(var r=t,i=1,o=this[e+--r];r>0&&(i*=256);)o+=this[e+--r]*i;return o>=(i*=128)&&(o-=Math.pow(2,8*t)),o},l.prototype.readInt8=function(e,t){return t||k(e,1,this.length),128&this[e]?-1*(255-this[e]+1):this[e]},l.prototype.readInt16LE=function(e,t){t||k(e,2,this.length);var n=this[e]|this[e+1]<<8;return 32768&n?4294901760|n:n},l.prototype.readInt16BE=function(e,t){t||k(e,2,this.length);var n=this[e+1]|this[e]<<8;return 32768&n?4294901760|n:n},l.prototype.readInt32LE=function(e,t){return t||k(e,4,this.length),this[e]|this[e+1]<<8|this[e+2]<<16|this[e+3]<<24},l.prototype.readInt32BE=function(e,t){return t||k(e,4,this.length),this[e]<<24|this[e+1]<<16|this[e+2]<<8|this[e+3]},l.prototype.readFloatLE=function(e,t){return t||k(e,4,this.length),i.read(this,e,!0,23,4)},l.prototype.readFloatBE=function(e,t){return t||k(e,4,this.length),i.read(this,e,!1,23,4)},l.prototype.readDoubleLE=function(e,t){return t||k(e,8,this.length),i.read(this,e,!0,52,8)},l.prototype.readDoubleBE=function(e,t){return t||k(e,8,this.length),i.read(this,e,!1,52,8)},l.prototype.writeUIntLE=function(e,t,n,r){(e=+e,t|=0,n|=0,r)||L(this,e,t,n,Math.pow(2,8*n)-1,0);var i=1,o=0;for(this[t]=255&e;++o<n&&(i*=256);)this[t+o]=e/i&255;return t+n},l.prototype.writeUIntBE=function(e,t,n,r){(e=+e,t|=0,n|=0,r)||L(this,e,t,n,Math.pow(2,8*n)-1,0);var i=n-1,o=1;for(this[t+i]=255&e;--i>=0&&(o*=256);)this[t+i]=e/o&255;return t+n},l.prototype.writeUInt8=function(e,t,n){return e=+e,t|=0,n||L(this,e,t,1,255,0),l.TYPED_ARRAY_SUPPORT||(e=Math.floor(e)),this[t]=255&e,t+1},l.prototype.writeUInt16LE=function(e,t,n){return e=+e,t|=0,n||L(this,e,t,2,65535,0),l.TYPED_ARRAY_SUPPORT?(this[t]=255&e,this[t+1]=e>>>8):D(this,e,t,!0),t+2},l.prototype.writeUInt16BE=function(e,t,n){return e=+e,t|=0,n||L(this,e,t,2,65535,0),l.TYPED_ARRAY_SUPPORT?(this[t]=e>>>8,this[t+1]=255&e):D(this,e,t,!1),t+2},l.prototype.writeUInt32LE=function(e,t,n){return e=+e,t|=0,n||L(this,e,t,4,4294967295,0),l.TYPED_ARRAY_SUPPORT?(this[t+3]=e>>>24,this[t+2]=e>>>16,this[t+1]=e>>>8,this[t]=255&e):P(this,e,t,!0),t+4},l.prototype.writeUInt32BE=function(e,t,n){return e=+e,t|=0,n||L(this,e,t,4,4294967295,0),l.TYPED_ARRAY_SUPPORT?(this[t]=e>>>24,this[t+1]=e>>>16,this[t+2]=e>>>8,this[t+3]=255&e):P(this,e,t,!1),t+4},l.prototype.writeIntLE=function(e,t,n,r){if(e=+e,t|=0,!r){var i=Math.pow(2,8*n-1);L(this,e,t,n,i-1,-i)}var o=0,a=1,s=0;for(this[t]=255&e;++o<n&&(a*=256);)e<0&&0===s&&0!==this[t+o-1]&&(s=1),this[t+o]=(e/a>>0)-s&255;return t+n},l.prototype.writeIntBE=function(e,t,n,r){if(e=+e,t|=0,!r){var i=Math.pow(2,8*n-1);L(this,e,t,n,i-1,-i)}var o=n-1,a=1,s=0;for(this[t+o]=255&e;--o>=0&&(a*=256);)e<0&&0===s&&0!==this[t+o+1]&&(s=1),this[t+o]=(e/a>>0)-s&255;return t+n},l.prototype.writeInt8=function(e,t,n){return e=+e,t|=0,n||L(this,e,t,1,127,-128),l.TYPED_ARRAY_SUPPORT||(e=Math.floor(e)),e<0&&(e=255+e+1),this[t]=255&e,t+1},l.prototype.writeInt16LE=function(e,t,n){return e=+e,t|=0,n||L(this,e,t,2,32767,-32768),l.TYPED_ARRAY_SUPPORT?(this[t]=255&e,this[t+1]=e>>>8):D(this,e,t,!0),t+2},l.prototype.writeInt16BE=function(e,t,n){return e=+e,t|=0,n||L(this,e,t,2,32767,-32768),l.TYPED_ARRAY_SUPPORT?(this[t]=e>>>8,this[t+1]=255&e):D(this,e,t,!1),t+2},l.prototype.writeInt32LE=function(e,t,n){return e=+e,t|=0,n||L(this,e,t,4,2147483647,-2147483648),l.TYPED_ARRAY_SUPPORT?(this[t]=255&e,this[t+1]=e>>>8,this[t+2]=e>>>16,this[t+3]=e>>>24):P(this,e,t,!0),t+4},l.prototype.writeInt32BE=function(e,t,n){return e=+e,t|=0,n||L(this,e,t,4,2147483647,-2147483648),e<0&&(e=4294967295+e+1),l.TYPED_ARRAY_SUPPORT?(this[t]=e>>>24,this[t+1]=e>>>16,this[t+2]=e>>>8,this[t+3]=255&e):P(this,e,t,!1),t+4},l.prototype.writeFloatLE=function(e,t,n){return j(this,e,t,!0,n)},l.prototype.writeFloatBE=function(e,t,n){return j(this,e,t,!1,n)},l.prototype.writeDoubleLE=function(e,t,n){return R(this,e,t,!0,n)},l.prototype.writeDoubleBE=function(e,t,n){return R(this,e,t,!1,n)},l.prototype.copy=function(e,t,n,r){if(n||(n=0),r||0===r||(r=this.length),t>=e.length&&(t=e.length),t||(t=0),r>0&&r<n&&(r=n),r===n)return 0;if(0===e.length||0===this.length)return 0;if(t<0)throw new RangeError("targetStart out of bounds");if(n<0||n>=this.length)throw new RangeError("sourceStart out of bounds");if(r<0)throw new RangeError("sourceEnd out of bounds");r>this.length&&(r=this.length),e.length-t<r-n&&(r=e.length-t+n);var i,o=r-n;if(this===e&&n<t&&t<r)for(i=o-1;i>=0;--i)e[i+t]=this[i+n];else if(o<1e3||!l.TYPED_ARRAY_SUPPORT)for(i=0;i<o;++i)e[i+t]=this[i+n];else Uint8Array.prototype.set.call(e,this.subarray(n,n+o),t);return o},l.prototype.fill=function(e,t,n,r){if("string"==typeof e){if("string"==typeof t?(r=t,t=0,n=this.length):"string"==typeof n&&(r=n,n=this.length),1===e.length){var i=e.charCodeAt(0);i<256&&(e=i)}if(void 0!==r&&"string"!=typeof r)throw new TypeError("encoding must be a string");if("string"==typeof r&&!l.isEncoding(r))throw new TypeError("Unknown encoding: "+r)}else"number"==typeof e&&(e&=255);if(t<0||this.length<t||this.length<n)throw new RangeError("Out of range index");if(n<=t)return this;var o;if(t>>>=0,n=void 0===n?this.length:n>>>0,e||(e=0),"number"==typeof e)for(o=t;o<n;++o)this[o]=e;else{var a=l.isBuffer(e)?e:B(new l(e,r).toString()),s=a.length;for(o=0;o<n-t;++o)this[o+t]=a[o%s]}return this};var z=/[^+\/0-9A-Za-z-_]/g;function F(e){return e<16?"0"+e.toString(16):e.toString(16)}function B(e,t){var n;t=t||1/0;for(var r=e.length,i=null,o=[],a=0;a<r;++a){if((n=e.charCodeAt(a))>55295&&n<57344){if(!i){if(n>56319){(t-=3)>-1&&o.push(239,191,189);continue}if(a+1===r){(t-=3)>-1&&o.push(239,191,189);continue}i=n;continue}if(n<56320){(t-=3)>-1&&o.push(239,191,189),i=n;continue}n=65536+(i-55296<<10|n-56320)}else i&&(t-=3)>-1&&o.push(239,191,189);if(i=null,n<128){if((t-=1)<0)break;o.push(n)}else if(n<2048){if((t-=2)<0)break;o.push(n>>6|192,63&n|128)}else if(n<65536){if((t-=3)<0)break;o.push(n>>12|224,n>>6&63|128,63&n|128)}else{if(!(n<1114112))throw new Error("Invalid code point");if((t-=4)<0)break;o.push(n>>18|240,n>>12&63|128,n>>6&63|128,63&n|128)}}return o}function H(e){return r.toByteArray(function(e){if((e=function(e){return e.trim?e.trim():e.replace(/^\s+|\s+$/g,"")}(e).replace(z,"")).length<2)return"";for(;e.length%4!=0;)e+="=";return e}(e))}function U(e,t,n,r){for(var i=0;i<r&&!(i+n>=t.length||i>=e.length);++i)t[i+n]=e[i];return i}}).call(this,n(61))},function(e,t,n){"use strict";var r=n(1036),i={body:"",args:[],thisVars:[],localVars:[]};function o(e){if(!e)return i;for(var t=0;t<e.args.length;++t){var n=e.args[t];e.args[t]=0===t?{name:n,lvalue:!0,rvalue:!!e.rvalue,count:e.count||1}:{name:n,lvalue:!1,rvalue:!0,count:1}}return e.thisVars||(e.thisVars=[]),e.localVars||(e.localVars=[]),e}function a(e){for(var t=[],n=0;n<e.args.length;++n)t.push("a"+n);return new Function("P",["return function ",e.funcName,"_ndarrayops(",t.join(","),") {P(",t.join(","),");return a0}"].join(""))(function(e){return r({args:e.args,pre:o(e.pre),body:o(e.body),post:o(e.proc),funcName:e.funcName})}(e))}var s={add:"+",sub:"-",mul:"*",div:"/",mod:"%",band:"&",bor:"|",bxor:"^",lshift:"<<",rshift:">>",rrshift:">>>"};!function(){for(var e in s){var n=s[e];t[e]=a({args:["array","array","array"],body:{args:["a","b","c"],body:"a=b"+n+"c"},funcName:e}),t[e+"eq"]=a({args:["array","array"],body:{args:["a","b"],body:"a"+n+"=b"},rvalue:!0,funcName:e+"eq"}),t[e+"s"]=a({args:["array","array","scalar"],body:{args:["a","b","s"],body:"a=b"+n+"s"},funcName:e+"s"}),t[e+"seq"]=a({args:["array","scalar"],body:{args:["a","s"],body:"a"+n+"=s"},rvalue:!0,funcName:e+"seq"})}}();var l={not:"!",bnot:"~",neg:"-",recip:"1.0/"};!function(){for(var e in l){var n=l[e];t[e]=a({args:["array","array"],body:{args:["a","b"],body:"a="+n+"b"},funcName:e}),t[e+"eq"]=a({args:["array"],body:{args:["a"],body:"a="+n+"a"},rvalue:!0,count:2,funcName:e+"eq"})}}();var u={and:"&&",or:"||",eq:"===",neq:"!==",lt:"<",gt:">",leq:"<=",geq:">="};!function(){for(var e in u){var n=u[e];t[e]=a({args:["array","array","array"],body:{args:["a","b","c"],body:"a=b"+n+"c"},funcName:e}),t[e+"s"]=a({args:["array","array","scalar"],body:{args:["a","b","s"],body:"a=b"+n+"s"},funcName:e+"s"}),t[e+"eq"]=a({args:["array","array"],body:{args:["a","b"],body:"a=a"+n+"b"},rvalue:!0,count:2,funcName:e+"eq"}),t[e+"seq"]=a({args:["array","scalar"],body:{args:["a","s"],body:"a=a"+n+"s"},rvalue:!0,count:2,funcName:e+"seq"})}}();var c=["abs","acos","asin","atan","ceil","cos","exp","floor","log","round","sin","sqrt","tan"];!function(){for(var e=0;e<c.length;++e){var n=c[e];t[n]=a({args:["array","array"],pre:{args:[],body:"this_f=Math."+n,thisVars:["this_f"]},body:{args:["a","b"],body:"a=this_f(b)",thisVars:["this_f"]},funcName:n}),t[n+"eq"]=a({args:["array"],pre:{args:[],body:"this_f=Math."+n,thisVars:["this_f"]},body:{args:["a"],body:"a=this_f(a)",thisVars:["this_f"]},rvalue:!0,count:2,funcName:n+"eq"})}}();var d=["max","min","atan2","pow"];!function(){for(var e=0;e<d.length;++e){var n=d[e];t[n]=a({args:["array","array","array"],pre:{args:[],body:"this_f=Math."+n,thisVars:["this_f"]},body:{args:["a","b","c"],body:"a=this_f(b,c)",thisVars:["this_f"]},funcName:n}),t[n+"s"]=a({args:["array","array","scalar"],pre:{args:[],body:"this_f=Math."+n,thisVars:["this_f"]},body:{args:["a","b","c"],body:"a=this_f(b,c)",thisVars:["this_f"]},funcName:n+"s"}),t[n+"eq"]=a({args:["array","array"],pre:{args:[],body:"this_f=Math."+n,thisVars:["this_f"]},body:{args:["a","b"],body:"a=this_f(a,b)",thisVars:["this_f"]},rvalue:!0,count:2,funcName:n+"eq"}),t[n+"seq"]=a({args:["array","scalar"],pre:{args:[],body:"this_f=Math."+n,thisVars:["this_f"]},body:{args:["a","b"],body:"a=this_f(a,b)",thisVars:["this_f"]},rvalue:!0,count:2,funcName:n+"seq"})}}();var h=["atan2","pow"];!function(){for(var e=0;e<h.length;++e){var n=h[e];t[n+"op"]=a({args:["array","array","array"],pre:{args:[],body:"this_f=Math."+n,thisVars:["this_f"]},body:{args:["a","b","c"],body:"a=this_f(c,b)",thisVars:["this_f"]},funcName:n+"op"}),t[n+"ops"]=a({args:["array","array","scalar"],pre:{args:[],body:"this_f=Math."+n,thisVars:["this_f"]},body:{args:["a","b","c"],body:"a=this_f(c,b)",thisVars:["this_f"]},funcName:n+"ops"}),t[n+"opeq"]=a({args:["array","array"],pre:{args:[],body:"this_f=Math."+n,thisVars:["this_f"]},body:{args:["a","b"],body:"a=this_f(b,a)",thisVars:["this_f"]},rvalue:!0,count:2,funcName:n+"opeq"}),t[n+"opseq"]=a({args:["array","scalar"],pre:{args:[],body:"this_f=Math."+n,thisVars:["this_f"]},body:{args:["a","b"],body:"a=this_f(b,a)",thisVars:["this_f"]},rvalue:!0,count:2,funcName:n+"opseq"})}}(),t.any=r({args:["array"],pre:i,body:{args:[{name:"a",lvalue:!1,rvalue:!0,count:1}],body:"if(a){return true}",localVars:[],thisVars:[]},post:{args:[],localVars:[],thisVars:[],body:"return false"},funcName:"any"}),t.all=r({args:["array"],pre:i,body:{args:[{name:"x",lvalue:!1,rvalue:!0,count:1}],body:"if(!x){return false}",localVars:[],thisVars:[]},post:{args:[],localVars:[],thisVars:[],body:"return true"},funcName:"all"}),t.sum=r({args:["array"],pre:{args:[],localVars:[],thisVars:["this_s"],body:"this_s=0"},body:{args:[{name:"a",lvalue:!1,rvalue:!0,count:1}],body:"this_s+=a",localVars:[],thisVars:["this_s"]},post:{args:[],localVars:[],thisVars:["this_s"],body:"return this_s"},funcName:"sum"}),t.prod=r({args:["array"],pre:{args:[],localVars:[],thisVars:["this_s"],body:"this_s=1"},body:{args:[{name:"a",lvalue:!1,rvalue:!0,count:1}],body:"this_s*=a",localVars:[],thisVars:["this_s"]},post:{args:[],localVars:[],thisVars:["this_s"],body:"return this_s"},funcName:"prod"}),t.norm2squared=r({args:["array"],pre:{args:[],localVars:[],thisVars:["this_s"],body:"this_s=0"},body:{args:[{name:"a",lvalue:!1,rvalue:!0,count:2}],body:"this_s+=a*a",localVars:[],thisVars:["this_s"]},post:{args:[],localVars:[],thisVars:["this_s"],body:"return this_s"},funcName:"norm2squared"}),t.norm2=r({args:["array"],pre:{args:[],localVars:[],thisVars:["this_s"],body:"this_s=0"},body:{args:[{name:"a",lvalue:!1,rvalue:!0,count:2}],body:"this_s+=a*a",localVars:[],thisVars:["this_s"]},post:{args:[],localVars:[],thisVars:["this_s"],body:"return Math.sqrt(this_s)"},funcName:"norm2"}),t.norminf=r({args:["array"],pre:{args:[],localVars:[],thisVars:["this_s"],body:"this_s=0"},body:{args:[{name:"a",lvalue:!1,rvalue:!0,count:4}],body:"if(-a>this_s){this_s=-a}else if(a>this_s){this_s=a}",localVars:[],thisVars:["this_s"]},post:{args:[],localVars:[],thisVars:["this_s"],body:"return this_s"},funcName:"norminf"}),t.norm1=r({args:["array"],pre:{args:[],localVars:[],thisVars:["this_s"],body:"this_s=0"},body:{args:[{name:"a",lvalue:!1,rvalue:!0,count:3}],body:"this_s+=a<0?-a:a",localVars:[],thisVars:["this_s"]},post:{args:[],localVars:[],thisVars:["this_s"],body:"return this_s"},funcName:"norm1"}),t.sup=r({args:["array"],pre:{body:"this_h=-Infinity",args:[],thisVars:["this_h"],localVars:[]},body:{body:"if(_inline_1_arg0_>this_h)this_h=_inline_1_arg0_",args:[{name:"_inline_1_arg0_",lvalue:!1,rvalue:!0,count:2}],thisVars:["this_h"],localVars:[]},post:{body:"return this_h",args:[],thisVars:["this_h"],localVars:[]}}),t.inf=r({args:["array"],pre:{body:"this_h=Infinity",args:[],thisVars:["this_h"],localVars:[]},body:{body:"if(_inline_1_arg0_<this_h)this_h=_inline_1_arg0_",args:[{name:"_inline_1_arg0_",lvalue:!1,rvalue:!0,count:2}],thisVars:["this_h"],localVars:[]},post:{body:"return this_h",args:[],thisVars:["this_h"],localVars:[]}}),t.argmin=r({args:["index","array","shape"],pre:{body:"{this_v=Infinity;this_i=_inline_0_arg2_.slice(0)}",args:[{name:"_inline_0_arg0_",lvalue:!1,rvalue:!1,count:0},{name:"_inline_0_arg1_",lvalue:!1,rvalue:!1,count:0},{name:"_inline_0_arg2_",lvalue:!1,rvalue:!0,count:1}],thisVars:["this_i","this_v"],localVars:[]},body:{body:"{if(_inline_1_arg1_<this_v){this_v=_inline_1_arg1_;for(var _inline_1_k=0;_inline_1_k<_inline_1_arg0_.length;++_inline_1_k){this_i[_inline_1_k]=_inline_1_arg0_[_inline_1_k]}}}",args:[{name:"_inline_1_arg0_",lvalue:!1,rvalue:!0,count:2},{name:"_inline_1_arg1_",lvalue:!1,rvalue:!0,count:2}],thisVars:["this_i","this_v"],localVars:["_inline_1_k"]},post:{body:"{return this_i}",args:[],thisVars:["this_i"],localVars:[]}}),t.argmax=r({args:["index","array","shape"],pre:{body:"{this_v=-Infinity;this_i=_inline_0_arg2_.slice(0)}",args:[{name:"_inline_0_arg0_",lvalue:!1,rvalue:!1,count:0},{name:"_inline_0_arg1_",lvalue:!1,rvalue:!1,count:0},{name:"_inline_0_arg2_",lvalue:!1,rvalue:!0,count:1}],thisVars:["this_i","this_v"],localVars:[]},body:{body:"{if(_inline_1_arg1_>this_v){this_v=_inline_1_arg1_;for(var _inline_1_k=0;_inline_1_k<_inline_1_arg0_.length;++_inline_1_k){this_i[_inline_1_k]=_inline_1_arg0_[_inline_1_k]}}}",args:[{name:"_inline_1_arg0_",lvalue:!1,rvalue:!0,count:2},{name:"_inline_1_arg1_",lvalue:!1,rvalue:!0,count:2}],thisVars:["this_i","this_v"],localVars:["_inline_1_k"]},post:{body:"{return this_i}",args:[],thisVars:["this_i"],localVars:[]}}),t.random=a({args:["array"],pre:{args:[],body:"this_f=Math.random",thisVars:["this_f"]},body:{args:["a"],body:"a=this_f()",thisVars:["this_f"]},funcName:"random"}),t.assign=a({args:["array","array"],body:{args:["a","b"],body:"a=b"},funcName:"assign"}),t.assigns=a({args:["array","scalar"],body:{args:["a","b"],body:"a=b"},funcName:"assigns"}),t.equals=r({args:["array","array"],pre:i,body:{args:[{name:"x",lvalue:!1,rvalue:!0,count:1},{name:"y",lvalue:!1,rvalue:!0,count:1}],body:"if(x!==y){return false}",localVars:[],thisVars:[]},post:{args:[],localVars:[],thisVars:[],body:"return true"},funcName:"equals"})},function(e,t,n){"use strict";var r=n(1792);function i(){this.argTypes=[],this.shimArgs=[],this.arrayArgs=[],this.arrayBlockIndices=[],this.scalarArgs=[],this.offsetArgs=[],this.offsetArgIndex=[],this.indexArgs=[],this.shapeArgs=[],this.funcName="",this.pre=null,this.body=null,this.post=null,this.debug=!1}e.exports=function(e){var t=new i;t.pre=e.pre,t.body=e.body,t.post=e.post;var n=e.args.slice(0);t.argTypes=n;for(var o=0;o<n.length;++o){var a=n[o];if("array"===a||"object"==typeof a&&a.blockIndices){if(t.argTypes[o]="array",t.arrayArgs.push(o),t.arrayBlockIndices.push(a.blockIndices?a.blockIndices:0),t.shimArgs.push("array"+o),o<t.pre.args.length&&t.pre.args[o].count>0)throw new Error("cwise: pre() block may not reference array args");if(o<t.post.args.length&&t.post.args[o].count>0)throw new Error("cwise: post() block may not reference array args")}else if("scalar"===a)t.scalarArgs.push(o),t.shimArgs.push("scalar"+o);else if("index"===a){if(t.indexArgs.push(o),o<t.pre.args.length&&t.pre.args[o].count>0)throw new Error("cwise: pre() block may not reference array index");if(o<t.body.args.length&&t.body.args[o].lvalue)throw new Error("cwise: body() block may not write to array index");if(o<t.post.args.length&&t.post.args[o].count>0)throw new Error("cwise: post() block may not reference array index")}else if("shape"===a){if(t.shapeArgs.push(o),o<t.pre.args.length&&t.pre.args[o].lvalue)throw new Error("cwise: pre() block may not write to array shape");if(o<t.body.args.length&&t.body.args[o].lvalue)throw new Error("cwise: body() block may not write to array shape");if(o<t.post.args.length&&t.post.args[o].lvalue)throw new Error("cwise: post() block may not write to array shape")}else{if("object"!=typeof a||!a.offset)throw new Error("cwise: Unknown argument type "+n[o]);t.argTypes[o]="offset",t.offsetArgs.push({array:a.array,offset:a.offset}),t.offsetArgIndex.push(o)}}if(t.arrayArgs.length<=0)throw new Error("cwise: No array arguments specified");if(t.pre.args.length>n.length)throw new Error("cwise: Too many arguments in pre() block");if(t.body.args.length>n.length)throw new Error("cwise: Too many arguments in body() block");if(t.post.args.length>n.length)throw new Error("cwise: Too many arguments in post() block");return t.debug=!!e.printCode||!!e.debug,t.funcName=e.funcName||"cwise",t.blockSize=e.blockSize||64,r(t)}},function(e,t,n){"use strict";e.exports=function(e,t,n){t?t.bind():e.bindBuffer(e.ELEMENT_ARRAY_BUFFER,null);var r=0|e.getParameter(e.MAX_VERTEX_ATTRIBS);if(n){if(n.length>r)throw new Error("gl-vao: Too many vertex attributes");for(var i=0;i<n.length;++i){var o=n[i];if(o.buffer){var a=o.buffer,s=o.size||4,l=o.type||e.FLOAT,u=!!o.normalized,c=o.stride||0,d=o.offset||0;a.bind(),e.enableVertexAttribArray(i),e.vertexAttribPointer(i,s,l,u,c,d)}else{if("number"==typeof o)e.vertexAttrib1f(i,o);else if(1===o.length)e.vertexAttrib1f(i,o[0]);else if(2===o.length)e.vertexAttrib2f(i,o[0],o[1]);else if(3===o.length)e.vertexAttrib3f(i,o[0],o[1],o[2]);else{if(4!==o.length)throw new Error("gl-vao: Invalid vertex attribute");e.vertexAttrib4f(i,o[0],o[1],o[2],o[3])}e.disableVertexAttribArray(i)}}for(;i<r;++i)e.disableVertexAttribArray(i)}else{e.bindBuffer(e.ARRAY_BUFFER,null);for(i=0;i<r;++i)e.disableVertexAttribArray(i)}}},function(e,t,n){"use strict";e.exports=function(e,t){for(var n={},r=0;r<e.length;++r)for(var i=e[r].name.split("."),o=n,a=0;a<i.length;++a){var s=i[a].split("[");if(s.length>1){s[0]in o||(o[s[0]]=[]),o=o[s[0]];for(var l=1;l<s.length;++l){var u=parseInt(s[l]);l<s.length-1||a<i.length-1?(u in o||(l<s.length-1?o[u]=[]:o[u]={}),o=o[u]):o[u]=t?r:e[r].type}}else a<i.length-1?(s[0]in o||(o[s[0]]={}),o=o[s[0]]):o[s[0]]=t?r:e[r].type}return n}},function(e,t){e.exports=["precision","highp","mediump","lowp","attribute","const","uniform","varying","break","continue","do","for","while","if","else","in","out","inout","float","int","uint","void","bool","true","false","discard","return","mat2","mat3","mat4","vec2","vec3","vec4","ivec2","ivec3","ivec4","bvec2","bvec3","bvec4","sampler1D","sampler2D","sampler3D","samplerCube","sampler1DShadow","sampler2DShadow","struct","asm","class","union","enum","typedef","template","this","packed","goto","switch","default","inline","noinline","volatile","public","static","extern","external","interface","long","short","double","half","fixed","unsigned","input","output","hvec2","hvec3","hvec4","dvec2","dvec3","dvec4","fvec2","fvec3","fvec4","sampler2DRect","sampler3DRect","sampler2DRectShadow","sizeof","cast","namespace","using"]},function(e,t){e.exports=["abs","acos","all","any","asin","atan","ceil","clamp","cos","cross","dFdx","dFdy","degrees","distance","dot","equal","exp","exp2","faceforward","floor","fract","gl_BackColor","gl_BackLightModelProduct","gl_BackLightProduct","gl_BackMaterial","gl_BackSecondaryColor","gl_ClipPlane","gl_ClipVertex","gl_Color","gl_DepthRange","gl_DepthRangeParameters","gl_EyePlaneQ","gl_EyePlaneR","gl_EyePlaneS","gl_EyePlaneT","gl_Fog","gl_FogCoord","gl_FogFragCoord","gl_FogParameters","gl_FragColor","gl_FragCoord","gl_FragData","gl_FragDepth","gl_FragDepthEXT","gl_FrontColor","gl_FrontFacing","gl_FrontLightModelProduct","gl_FrontLightProduct","gl_FrontMaterial","gl_FrontSecondaryColor","gl_LightModel","gl_LightModelParameters","gl_LightModelProducts","gl_LightProducts","gl_LightSource","gl_LightSourceParameters","gl_MaterialParameters","gl_MaxClipPlanes","gl_MaxCombinedTextureImageUnits","gl_MaxDrawBuffers","gl_MaxFragmentUniformComponents","gl_MaxLights","gl_MaxTextureCoords","gl_MaxTextureImageUnits","gl_MaxTextureUnits","gl_MaxVaryingFloats","gl_MaxVertexAttribs","gl_MaxVertexTextureImageUnits","gl_MaxVertexUniformComponents","gl_ModelViewMatrix","gl_ModelViewMatrixInverse","gl_ModelViewMatrixInverseTranspose","gl_ModelViewMatrixTranspose","gl_ModelViewProjectionMatrix","gl_ModelViewProjectionMatrixInverse","gl_ModelViewProjectionMatrixInverseTranspose","gl_ModelViewProjectionMatrixTranspose","gl_MultiTexCoord0","gl_MultiTexCoord1","gl_MultiTexCoord2","gl_MultiTexCoord3","gl_MultiTexCoord4","gl_MultiTexCoord5","gl_MultiTexCoord6","gl_MultiTexCoord7","gl_Normal","gl_NormalMatrix","gl_NormalScale","gl_ObjectPlaneQ","gl_ObjectPlaneR","gl_ObjectPlaneS","gl_ObjectPlaneT","gl_Point","gl_PointCoord","gl_PointParameters","gl_PointSize","gl_Position","gl_ProjectionMatrix","gl_ProjectionMatrixInverse","gl_ProjectionMatrixInverseTranspose","gl_ProjectionMatrixTranspose","gl_SecondaryColor","gl_TexCoord","gl_TextureEnvColor","gl_TextureMatrix","gl_TextureMatrixInverse","gl_TextureMatrixInverseTranspose","gl_TextureMatrixTranspose","gl_Vertex","greaterThan","greaterThanEqual","inversesqrt","length","lessThan","lessThanEqual","log","log2","matrixCompMult","max","min","mix","mod","normalize","not","notEqual","pow","radians","reflect","refract","sign","sin","smoothstep","sqrt","step","tan","texture2D","texture2DLod","texture2DProj","texture2DProjLod","textureCube","textureCubeLod","texture2DLodEXT","texture2DProjLodEXT","textureCubeLodEXT","texture2DGradEXT","texture2DProjGradEXT","textureCubeGradEXT"]},function(e,t,n){"use strict";e.exports=function(e,t){"object"==typeof t&&null!==t||(t={});return r(e,t.canvas||i,t.context||o,t)};var r=n(1823),i=null,o=null;"undefined"!=typeof document&&((i=document.createElement("canvas")).width=8192,i.height=1024,o=i.getContext("2d"))},function(e,t,n){"use strict";e.exports=function(e,t,n){var r=e+t,i=r-e,o=t-i,a=e-(r-i);if(n)return n[0]=a+o,n[1]=r,n;return[a+o,r]}},function(e,t,n){"use strict";e.exports=function(e,t){var n=0|e.length,r=0|t.length;if(1===n&&1===r)return function(e,t){var n=e+t,r=n-e,i=e-(n-r)+(t-r);if(i)return[i,n];return[n]}(e[0],-t[0]);var i,o,a=new Array(n+r),s=0,l=0,u=0,c=Math.abs,d=e[l],h=c(d),f=-t[u],p=c(f);h<p?(o=d,(l+=1)<n&&(d=e[l],h=c(d))):(o=f,(u+=1)<r&&(f=-t[u],p=c(f)));l<n&&h<p||u>=r?(i=d,(l+=1)<n&&(d=e[l],h=c(d))):(i=f,(u+=1)<r&&(f=-t[u],p=c(f)));var g,v,m=i+o,y=m-i,b=o-y,C=b,A=m;for(;l<n&&u<r;)h<p?(i=d,(l+=1)<n&&(d=e[l],h=c(d))):(i=f,(u+=1)<r&&(f=-t[u],p=c(f))),(b=(o=C)-(y=(m=i+o)-i))&&(a[s++]=b),C=A-((g=A+m)-(v=g-A))+(m-v),A=g;for(;l<n;)(b=(o=C)-(y=(m=(i=d)+o)-i))&&(a[s++]=b),C=A-((g=A+m)-(v=g-A))+(m-v),A=g,(l+=1)<n&&(d=e[l]);for(;u<r;)(b=(o=C)-(y=(m=(i=f)+o)-i))&&(a[s++]=b),C=A-((g=A+m)-(v=g-A))+(m-v),A=g,(u+=1)<r&&(f=-t[u]);C&&(a[s++]=C);A&&(a[s++]=A);s||(a[s++]=0);return a.length=s,a}},function(e,t,n){"use strict";function r(e){this.roots=new Array(e),this.ranks=new Array(e);for(var t=0;t<e;++t)this.roots[t]=t,this.ranks[t]=0}e.exports=r;var i=r.prototype;Object.defineProperty(i,"length",{get:function(){return this.roots.length}}),i.makeSet=function(){var e=this.roots.length;return this.roots.push(e),this.ranks.push(0),e},i.find=function(e){for(var t=e,n=this.roots;n[e]!==e;)e=n[e];for(;n[t]!==e;){var r=n[t];n[t]=e,t=r}return e},i.link=function(e,t){var n=this.find(e),r=this.find(t);if(n!==r){var i=this.ranks,o=this.roots,a=i[n],s=i[r];a<s?o[n]=r:s<a?o[r]=n:(o[r]=n,++i[n])}}},function(e,t,n){"use strict";e.exports={init:function(e){var t=i.nextPow2(e);a.length<t&&(r.free(a),a=r.mallocInt32(t));s.length<t&&(r.free(s),s=r.mallocInt32(t));l.length<t&&(r.free(l),l=r.mallocInt32(t));u.length<t&&(r.free(u),u=r.mallocInt32(t));c.length<t&&(r.free(c),c=r.mallocInt32(t));d.length<t&&(r.free(d),d=r.mallocInt32(t));var n=8*t;h.length<n&&(r.free(h),h=r.mallocDouble(n))},sweepBipartite:function(e,t,n,r,i,c,d,g,v,m){for(var y=0,b=2*e,C=e-1,A=b-1,_=n;_<r;++_){var I=c[_],x=b*_;h[y++]=i[x+C],h[y++]=-(I+1),h[y++]=i[x+A],h[y++]=I}for(_=d;_<g;++_){I=m[_]+(1<<28);var w=b*_;h[y++]=v[w+C],h[y++]=-I,h[y++]=v[w+A],h[y++]=I}var M=y>>>1;o(h,M);var S=0,T=0;for(_=0;_<M;++_){var E=0|h[2*_+1];if(E>=1<<28)f(l,u,T--,E=E-(1<<28)|0);else if(E>=0)f(a,s,S--,E);else if(E<=-(1<<28)){E=-E-(1<<28)|0;for(var O=0;O<S;++O){if(void 0!==(k=t(a[O],E)))return k}p(l,u,T++,E)}else{E=-E-1|0;for(O=0;O<T;++O){var k;if(void 0!==(k=t(E,l[O])))return k}p(a,s,S++,E)}}},sweepComplete:function(e,t,n,r,i,g,v,m,y,b){for(var C=0,A=2*e,_=e-1,I=A-1,x=n;x<r;++x){var w=g[x]+1<<1,M=A*x;h[C++]=i[M+_],h[C++]=-w,h[C++]=i[M+I],h[C++]=w}for(x=v;x<m;++x){w=b[x]+1<<1;var S=A*x;h[C++]=y[S+_],h[C++]=1|-w,h[C++]=y[S+I],h[C++]=1|w}var T=C>>>1;o(h,T);var E=0,O=0,k=0;for(x=0;x<T;++x){var L=0|h[2*x+1],D=1&L;if(x<T-1&&L>>1==h[2*x+3]>>1&&(D=2,x+=1),L<0){for(var P=-(L>>1)-1,N=0;N<k;++N){if(void 0!==(j=t(c[N],P)))return j}if(0!==D)for(N=0;N<E;++N){if(void 0!==(j=t(a[N],P)))return j}if(1!==D)for(N=0;N<O;++N){var j;if(void 0!==(j=t(l[N],P)))return j}0===D?p(a,s,E++,P):1===D?p(l,u,O++,P):2===D&&p(c,d,k++,P)}else{P=(L>>1)-1;0===D?f(a,s,E--,P):1===D?f(l,u,O--,P):2===D&&f(c,d,k--,P)}}},scanBipartite:function(e,t,n,r,i,l,u,c,d,g,v,m){var y=0,b=2*e,C=t,A=t+e,_=1,I=1;r?I=1<<28:_=1<<28;for(var x=i;x<l;++x){var w=x+_,M=b*x;h[y++]=u[M+C],h[y++]=-w,h[y++]=u[M+A],h[y++]=w}for(x=d;x<g;++x){w=x+I;var S=b*x;h[y++]=v[S+C],h[y++]=-w}var T=y>>>1;o(h,T);var E=0;for(x=0;x<T;++x){var O=0|h[2*x+1];if(O<0){var k=!1;if((w=-O)>=1<<28?(k=!r,w-=1<<28):(k=!!r,w-=1),k)p(a,s,E++,w);else{var L=m[w],D=b*w,P=v[D+t+1],N=v[D+t+1+e];e:for(var j=0;j<E;++j){var R=a[j],z=b*R;if(!(N<u[z+t+1]||u[z+t+1+e]<P)){for(var F=t+2;F<e;++F)if(v[D+F+e]<u[z+F]||u[z+F+e]<v[D+F])continue e;var B,H=c[R];if(void 0!==(B=r?n(L,H):n(H,L)))return B}}}}else f(a,s,E--,O-_)}},scanComplete:function(e,t,n,r,i,s,l,u,c,d,f){for(var p=0,g=2*e,v=t,m=t+e,y=r;y<i;++y){var b=y+(1<<28),C=g*y;h[p++]=s[C+v],h[p++]=-b,h[p++]=s[C+m],h[p++]=b}for(y=u;y<c;++y){b=y+1;var A=g*y;h[p++]=d[A+v],h[p++]=-b}var _=p>>>1;o(h,_);var I=0;for(y=0;y<_;++y){var x=0|h[2*y+1];if(x<0){if((b=-x)>=1<<28)a[I++]=b-(1<<28);else{var w=f[b-=1],M=g*b,S=d[M+t+1],T=d[M+t+1+e];e:for(var E=0;E<I;++E){var O=a[E],k=l[O];if(k===w)break;var L=g*O;if(!(T<s[L+t+1]||s[L+t+1+e]<S)){for(var D=t+2;D<e;++D)if(d[M+D+e]<s[L+D]||s[L+D+e]<d[M+D])continue e;var P=n(k,w);if(void 0!==P)return P}}}}else{for(b=x-(1<<28),E=I-1;E>=0;--E)if(a[E]===b){for(D=E+1;D<I;++D)a[D-1]=a[D];break}--I}}}};var r=n(112),i=n(215),o=n(1839),a=r.mallocInt32(1024),s=r.mallocInt32(1024),l=r.mallocInt32(1024),u=r.mallocInt32(1024),c=r.mallocInt32(1024),d=r.mallocInt32(1024),h=r.mallocDouble(8192);function f(e,t,n,r){var i=t[r],o=e[n-1];e[i]=o,t[o]=i}function p(e,t,n,r){e[n]=r,t[r]=n}},function(e,t,n){"use strict";e.exports=function(e,t){var n="abcdef".split("").concat(t),r=[];e.indexOf("lo")>=0&&r.push("lo=e[k+n]");e.indexOf("hi")>=0&&r.push("hi=e[k+o]");return n.push("for(var j=2*a,k=j*c,l=k,m=c,n=b,o=a+b,p=c;d>p;++p,k+=j){var _;if($)if(m===p)m+=1,l+=j;else{for(var s=0;j>s;++s){var t=e[k+s];e[k+s]=e[l],e[l++]=t}var u=f[p];f[p]=f[m],f[m++]=u}}return m".replace("_",r.join()).replace("$",e)),Function.apply(void 0,n)}},function(e,t,n){"use strict";n(397);e.exports=function(e){return e&&"object"==typeof e&&Boolean(e.words)}},function(e,t,n){"use strict";var r=n(397),i=n(571);e.exports=function(e){var t=i.exponent(e);return t<52?new r(e):new r(e*Math.pow(2,52-t)).ushln(t-52)}},function(e,t,n){"use strict";var r=n(303);e.exports=function(e,t){return r(e[0].mul(t[1]),e[1].mul(t[0]))}},function(e,t,n){"use strict";var r=n(303);e.exports=function(e,t){return r(e[0].mul(t[0]),e[1].mul(t[1]))}},function(e,t,n){"use strict";var r=n(303);e.exports=function(e,t){return r(e[0].mul(t[1]).sub(e[1].mul(t[0])),e[1].mul(t[1]))}},function(e,t,n){"use strict";e.exports=function(e,t){var n=e.length;if("number"!=typeof t){t=0;for(var i=0;i<n;++i){var o=e[i];t=Math.max(t,o[0],o[1])}t=1+(0|t)}t|=0;var a=new Array(t);for(i=0;i<t;++i)a[i]=[];for(i=0;i<n;++i){o=e[i];a[o[0]].push(o[1]),a[o[1]].push(o[0])}for(var s=0;s<t;++s)r(a[s],(function(e,t){return e-t}));return a};var r=n(394)},function(e,t,n){"use strict";var r=n(218),i=n(180),o=n(181),a=n(569),s=n(1882),l=n(255),u=n(254),c=n(216),d=n(1883),h=n(1886),f=n(112),p=n(1897),g=n(1898),v=p.meshShader,m=p.wireShader,y=p.pointShader,b=p.pickShader,C=p.pointPickShader,A=p.contourShader,_=[1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1];function I(e,t,n,r,i,o,a,s,l,u,c,d,h,f,p,g,v,m,y,b,C,A,I,x,w,M,S){this.gl=e,this.pixelRatio=1,this.cells=[],this.positions=[],this.intensity=[],this.texture=t,this.dirty=!0,this.triShader=n,this.lineShader=r,this.pointShader=i,this.pickShader=o,this.pointPickShader=a,this.contourShader=s,this.trianglePositions=l,this.triangleColors=c,this.triangleNormals=h,this.triangleUVs=d,this.triangleIds=u,this.triangleVAO=f,this.triangleCount=0,this.lineWidth=1,this.edgePositions=p,this.edgeColors=v,this.edgeUVs=m,this.edgeIds=g,this.edgeVAO=y,this.edgeCount=0,this.pointPositions=b,this.pointColors=A,this.pointUVs=I,this.pointSizes=x,this.pointIds=C,this.pointVAO=w,this.pointCount=0,this.contourLineWidth=1,this.contourPositions=M,this.contourVAO=S,this.contourCount=0,this.contourColor=[0,0,0],this.contourEnable=!0,this.pickVertex=!0,this.pickId=1,this.bounds=[[1/0,1/0,1/0],[-1/0,-1/0,-1/0]],this.clipBounds=[[-1/0,-1/0,-1/0],[1/0,1/0,1/0]],this.lightPosition=[1e5,1e5,0],this.ambientLight=.8,this.diffuseLight=.8,this.specularLight=2,this.roughness=.5,this.fresnel=1.5,this.opacity=1,this.hasAlpha=!1,this.opacityscale=!1,this._model=_,this._view=_,this._projection=_,this._resolution=[1,1]}var x=I.prototype;function w(e,t){if(!t)return 1;if(!t.length)return 1;for(var n=0;n<t.length;++n){if(t.length<2)return 1;if(t[n][0]===e)return t[n][1];if(t[n][0]>e&&n>0){var r=(t[n][0]-e)/(t[n][0]-t[n-1][0]);return t[n][1]*(1-r)+r*t[n-1][1]}}return 1}function M(e){var t=r(e,v.vertex,v.fragment);return t.attributes.position.location=0,t.attributes.color.location=2,t.attributes.uv.location=3,t.attributes.normal.location=4,t}function S(e){var t=r(e,m.vertex,m.fragment);return t.attributes.position.location=0,t.attributes.color.location=2,t.attributes.uv.location=3,t}function T(e){var t=r(e,y.vertex,y.fragment);return t.attributes.position.location=0,t.attributes.color.location=2,t.attributes.uv.location=3,t.attributes.pointSize.location=4,t}function E(e){var t=r(e,b.vertex,b.fragment);return t.attributes.position.location=0,t.attributes.id.location=1,t}function O(e){var t=r(e,C.vertex,C.fragment);return t.attributes.position.location=0,t.attributes.id.location=1,t.attributes.pointSize.location=4,t}function k(e){var t=r(e,A.vertex,A.fragment);return t.attributes.position.location=0,t}x.isOpaque=function(){return!this.hasAlpha},x.isTransparent=function(){return this.hasAlpha},x.pickSlots=1,x.setPickBase=function(e){this.pickId=e},x.highlight=function(e){if(e&&this.contourEnable){for(var t=h(this.cells,this.intensity,e.intensity),n=t.cells,r=t.vertexIds,i=t.vertexWeights,o=n.length,a=f.mallocFloat32(6*o),s=0,l=0;l<o;++l)for(var u=n[l],c=0;c<2;++c){var d=u[0];2===u.length&&(d=u[c]);for(var p=r[d][0],g=r[d][1],v=i[d],m=1-v,y=this.positions[p],b=this.positions[g],C=0;C<3;++C)a[s++]=v*y[C]+m*b[C]}this.contourCount=s/3|0,this.contourPositions.update(a.subarray(0,s)),f.free(a)}else this.contourCount=0},x.update=function(e){e=e||{};var t=this.gl;this.dirty=!0,"contourEnable"in e&&(this.contourEnable=e.contourEnable),"contourColor"in e&&(this.contourColor=e.contourColor),"lineWidth"in e&&(this.lineWidth=e.lineWidth),"lightPosition"in e&&(this.lightPosition=e.lightPosition),this.hasAlpha=!1,"opacity"in e&&(this.opacity=e.opacity,this.opacity<1&&(this.hasAlpha=!0)),"opacityscale"in e&&(this.opacityscale=e.opacityscale,this.hasAlpha=!0),"ambient"in e&&(this.ambientLight=e.ambient),"diffuse"in e&&(this.diffuseLight=e.diffuse),"specular"in e&&(this.specularLight=e.specular),"roughness"in e&&(this.roughness=e.roughness),"fresnel"in e&&(this.fresnel=e.fresnel),e.texture?(this.texture.dispose(),this.texture=a(t,e.texture)):e.colormap&&(this.texture.shape=[256,256],this.texture.minFilter=t.LINEAR_MIPMAP_LINEAR,this.texture.magFilter=t.LINEAR,this.texture.setPixels(function(e,t){for(var n=d({colormap:e,nshades:256,format:"rgba"}),r=new Uint8Array(1024),i=0;i<256;++i){for(var o=n[i],a=0;a<3;++a)r[4*i+a]=o[a];r[4*i+3]=t?255*w(i/255,t):255*o[3]}return c(r,[256,256,4],[4,0,1])}(e.colormap,this.opacityscale)),this.texture.generateMipmap());var n=e.cells,r=e.positions;if(r&&n){var i=[],o=[],l=[],u=[],h=[],f=[],p=[],g=[],v=[],m=[],y=[],b=[],C=[],A=[];this.cells=n,this.positions=r;var _=e.vertexNormals,I=e.cellNormals,x=void 0===e.vertexNormalsEpsilon?1e-6:e.vertexNormalsEpsilon,M=void 0===e.faceNormalsEpsilon?1e-6:e.faceNormalsEpsilon;e.useFacetNormals&&!I&&(I=s.faceNormals(n,r,M)),I||_||(_=s.vertexNormals(n,r,x));var S=e.vertexColors,T=e.cellColors,E=e.meshColor||[1,1,1,1],O=e.vertexUVs,k=e.vertexIntensity,L=e.cellUVs,D=e.cellIntensity,P=1/0,N=-1/0;if(!O&&!L)if(k)if(e.vertexIntensityBounds)P=+e.vertexIntensityBounds[0],N=+e.vertexIntensityBounds[1];else for(var j=0;j<k.length;++j){var R=k[j];P=Math.min(P,R),N=Math.max(N,R)}else if(D)if(e.cellIntensityBounds)P=+e.cellIntensityBounds[0],N=+e.cellIntensityBounds[1];else for(j=0;j<D.length;++j){R=D[j];P=Math.min(P,R),N=Math.max(N,R)}else for(j=0;j<r.length;++j){R=r[j][2];P=Math.min(P,R),N=Math.max(N,R)}this.intensity=k||(D||function(e){for(var t=e.length,n=new Array(t),r=0;r<t;++r)n[r]=e[r][2];return n}(r)),this.pickVertex=!(D||T);var z=e.pointSizes,F=e.pointSize||1;this.bounds=[[1/0,1/0,1/0],[-1/0,-1/0,-1/0]];for(j=0;j<r.length;++j)for(var B=r[j],H=0;H<3;++H)!isNaN(B[H])&&isFinite(B[H])&&(this.bounds[0][H]=Math.min(this.bounds[0][H],B[H]),this.bounds[1][H]=Math.max(this.bounds[1][H],B[H]));var U=0,V=0,Y=0;e:for(j=0;j<n.length;++j){var W=n[j];switch(W.length){case 1:for(B=r[X=W[0]],H=0;H<3;++H)if(isNaN(B[H])||!isFinite(B[H]))continue e;m.push(B[0],B[1],B[2]),q=S?S[X]:T?T[j]:E,this.opacityscale&&k?o.push(q[0],q[1],q[2],this.opacity*w((k[X]-P)/(N-P),this.opacityscale)):3===q.length?y.push(q[0],q[1],q[2],this.opacity):(y.push(q[0],q[1],q[2],q[3]*this.opacity),q[3]<1&&(this.hasAlpha=!0)),Z=O?O[X]:k?[(k[X]-P)/(N-P),0]:L?L[j]:D?[(D[j]-P)/(N-P),0]:[(B[2]-P)/(N-P),0],b.push(Z[0],Z[1]),z?C.push(z[X]):C.push(F),A.push(j),Y+=1;break;case 2:for(H=0;H<2;++H){B=r[X=W[H]];for(var G=0;G<3;++G)if(isNaN(B[G])||!isFinite(B[G]))continue e}for(H=0;H<2;++H){B=r[X=W[H]];f.push(B[0],B[1],B[2]),q=S?S[X]:T?T[j]:E,this.opacityscale&&k?o.push(q[0],q[1],q[2],this.opacity*w((k[X]-P)/(N-P),this.opacityscale)):3===q.length?p.push(q[0],q[1],q[2],this.opacity):(p.push(q[0],q[1],q[2],q[3]*this.opacity),q[3]<1&&(this.hasAlpha=!0)),Z=O?O[X]:k?[(k[X]-P)/(N-P),0]:L?L[j]:D?[(D[j]-P)/(N-P),0]:[(B[2]-P)/(N-P),0],g.push(Z[0],Z[1]),v.push(j)}V+=1;break;case 3:for(H=0;H<3;++H)for(B=r[X=W[H]],G=0;G<3;++G)if(isNaN(B[G])||!isFinite(B[G]))continue e;for(H=0;H<3;++H){var X,q,Z,Q;B=r[X=W[2-H]];i.push(B[0],B[1],B[2]),(q=S?S[X]:T?T[j]:E)?this.opacityscale&&k?o.push(q[0],q[1],q[2],this.opacity*w((k[X]-P)/(N-P),this.opacityscale)):3===q.length?o.push(q[0],q[1],q[2],this.opacity):(o.push(q[0],q[1],q[2],q[3]*this.opacity),q[3]<1&&(this.hasAlpha=!0)):o.push(.5,.5,.5,1),Z=O?O[X]:k?[(k[X]-P)/(N-P),0]:L?L[j]:D?[(D[j]-P)/(N-P),0]:[(B[2]-P)/(N-P),0],u.push(Z[0],Z[1]),Q=_?_[X]:I[j],l.push(Q[0],Q[1],Q[2]),h.push(j)}U+=1}}this.pointCount=Y,this.edgeCount=V,this.triangleCount=U,this.pointPositions.update(m),this.pointColors.update(y),this.pointUVs.update(b),this.pointSizes.update(C),this.pointIds.update(new Uint32Array(A)),this.edgePositions.update(f),this.edgeColors.update(p),this.edgeUVs.update(g),this.edgeIds.update(new Uint32Array(v)),this.trianglePositions.update(i),this.triangleColors.update(o),this.triangleUVs.update(u),this.triangleNormals.update(l),this.triangleIds.update(new Uint32Array(h))}},x.drawTransparent=x.draw=function(e){e=e||{};for(var t=this.gl,n=e.model||_,r=e.view||_,i=e.projection||_,o=[[-1e6,-1e6,-1e6],[1e6,1e6,1e6]],a=0;a<3;++a)o[0][a]=Math.max(o[0][a],this.clipBounds[0][a]),o[1][a]=Math.min(o[1][a],this.clipBounds[1][a]);var s={model:n,view:r,projection:i,inverseModel:_.slice(),clipBounds:o,kambient:this.ambientLight,kdiffuse:this.diffuseLight,kspecular:this.specularLight,roughness:this.roughness,fresnel:this.fresnel,eyePosition:[0,0,0],lightPosition:[0,0,0],contourColor:this.contourColor,texture:0};s.inverseModel=u(s.inverseModel,s.model),t.disable(t.CULL_FACE),this.texture.bind(0);var c=new Array(16);l(c,s.view,s.model),l(c,s.projection,c),u(c,c);for(a=0;a<3;++a)s.eyePosition[a]=c[12+a]/c[15];var d,h=c[15];for(a=0;a<3;++a)h+=this.lightPosition[a]*c[4*a+3];for(a=0;a<3;++a){for(var f=c[12+a],p=0;p<3;++p)f+=c[4*p+a]*this.lightPosition[p];s.lightPosition[a]=f/h}this.triangleCount>0&&((d=this.triShader).bind(),d.uniforms=s,this.triangleVAO.bind(),t.drawArrays(t.TRIANGLES,0,3*this.triangleCount),this.triangleVAO.unbind());this.edgeCount>0&&this.lineWidth>0&&((d=this.lineShader).bind(),d.uniforms=s,this.edgeVAO.bind(),t.lineWidth(this.lineWidth*this.pixelRatio),t.drawArrays(t.LINES,0,2*this.edgeCount),this.edgeVAO.unbind());this.pointCount>0&&((d=this.pointShader).bind(),d.uniforms=s,this.pointVAO.bind(),t.drawArrays(t.POINTS,0,this.pointCount),this.pointVAO.unbind());this.contourEnable&&this.contourCount>0&&this.contourLineWidth>0&&((d=this.contourShader).bind(),d.uniforms=s,this.contourVAO.bind(),t.drawArrays(t.LINES,0,this.contourCount),this.contourVAO.unbind())},x.drawPick=function(e){e=e||{};for(var t=this.gl,n=e.model||_,r=e.view||_,i=e.projection||_,o=[[-1e6,-1e6,-1e6],[1e6,1e6,1e6]],a=0;a<3;++a)o[0][a]=Math.max(o[0][a],this.clipBounds[0][a]),o[1][a]=Math.min(o[1][a],this.clipBounds[1][a]);this._model=[].slice.call(n),this._view=[].slice.call(r),this._projection=[].slice.call(i),this._resolution=[t.drawingBufferWidth,t.drawingBufferHeight];var s,l={model:n,view:r,projection:i,clipBounds:o,pickId:this.pickId/255};((s=this.pickShader).bind(),s.uniforms=l,this.triangleCount>0&&(this.triangleVAO.bind(),t.drawArrays(t.TRIANGLES,0,3*this.triangleCount),this.triangleVAO.unbind()),this.edgeCount>0&&(this.edgeVAO.bind(),t.lineWidth(this.lineWidth*this.pixelRatio),t.drawArrays(t.LINES,0,2*this.edgeCount),this.edgeVAO.unbind()),this.pointCount>0)&&((s=this.pointPickShader).bind(),s.uniforms=l,this.pointVAO.bind(),t.drawArrays(t.POINTS,0,this.pointCount),this.pointVAO.unbind())},x.pick=function(e){if(!e)return null;if(e.id!==this.pickId)return null;for(var t=e.value[0]+256*e.value[1]+65536*e.value[2],n=this.cells[t],r=this.positions,i=new Array(n.length),o=0;o<n.length;++o)i[o]=r[n[o]];var a=e.coord[0],s=e.coord[1];if(!this.pickVertex){var l=this.positions[n[0]],u=this.positions[n[1]],c=this.positions[n[2]],d=[(l[0]+u[0]+c[0])/3,(l[1]+u[1]+c[1])/3,(l[2]+u[2]+c[2])/3];return{_cellCenter:!0,position:[a,s],index:t,cell:n,cellId:t,intensity:this.intensity[t],dataCoordinate:d}}var h=g(i,[a*this.pixelRatio,this._resolution[1]-s*this.pixelRatio],this._model,this._view,this._projection,this._resolution);if(!h)return null;var f=h[2],p=0;for(o=0;o<n.length;++o)p+=f[o]*this.intensity[n[o]];return{position:h[1],index:n[h[0]],cell:n,cellId:t,intensity:p,dataCoordinate:this.positions[n[h[0]]]}},x.dispose=function(){this.texture.dispose(),this.triShader.dispose(),this.lineShader.dispose(),this.pointShader.dispose(),this.pickShader.dispose(),this.pointPickShader.dispose(),this.triangleVAO.dispose(),this.trianglePositions.dispose(),this.triangleColors.dispose(),this.triangleUVs.dispose(),this.triangleNormals.dispose(),this.triangleIds.dispose(),this.edgeVAO.dispose(),this.edgePositions.dispose(),this.edgeColors.dispose(),this.edgeUVs.dispose(),this.edgeIds.dispose(),this.pointVAO.dispose(),this.pointPositions.dispose(),this.pointColors.dispose(),this.pointUVs.dispose(),this.pointSizes.dispose(),this.pointIds.dispose(),this.contourVAO.dispose(),this.contourPositions.dispose(),this.contourShader.dispose()},e.exports=function(e,t){1===arguments.length&&(e=(t=e).gl);var n=e.getExtension("OES_standard_derivatives")||e.getExtension("MOZ_OES_standard_derivatives")||e.getExtension("WEBKIT_OES_standard_derivatives");if(!n)throw new Error("derivatives not supported");var r=M(e),s=S(e),l=T(e),u=E(e),d=O(e),h=k(e),f=a(e,c(new Uint8Array([255,255,255,255]),[1,1,4]));f.generateMipmap(),f.minFilter=e.LINEAR_MIPMAP_LINEAR,f.magFilter=e.LINEAR;var p=i(e),g=i(e),v=i(e),m=i(e),y=i(e),b=o(e,[{buffer:p,type:e.FLOAT,size:3},{buffer:y,type:e.UNSIGNED_BYTE,size:4,normalized:!0},{buffer:g,type:e.FLOAT,size:4},{buffer:v,type:e.FLOAT,size:2},{buffer:m,type:e.FLOAT,size:3}]),C=i(e),A=i(e),_=i(e),x=i(e),w=o(e,[{buffer:C,type:e.FLOAT,size:3},{buffer:x,type:e.UNSIGNED_BYTE,size:4,normalized:!0},{buffer:A,type:e.FLOAT,size:4},{buffer:_,type:e.FLOAT,size:2}]),L=i(e),D=i(e),P=i(e),N=i(e),j=i(e),R=o(e,[{buffer:L,type:e.FLOAT,size:3},{buffer:j,type:e.UNSIGNED_BYTE,size:4,normalized:!0},{buffer:D,type:e.FLOAT,size:4},{buffer:P,type:e.FLOAT,size:2},{buffer:N,type:e.FLOAT,size:1}]),z=i(e),F=o(e,[{buffer:z,type:e.FLOAT,size:3}]),B=new I(e,f,r,s,l,u,d,h,p,y,g,v,m,b,C,x,A,_,w,L,j,D,P,N,R,z,F);return B.update(t),B}},function(e,t,n){"use strict";var r=n(1891),i=n(1892),o=n(1894);e.exports=function(e){var t=e.length;if(0===t)return[];if(1===t)return[[0]];var n=e[0].length;if(0===n)return[];if(1===n)return r(e);if(2===n)return i(e);return o(e,n)}},function(e,t,n){"use strict";e.exports=function(e,t){var n=e.length;if(0===n)throw new Error("Must have at least d+1 points");var i=e[0].length;if(n<=i)throw new Error("Must input at least d+1 points");var a=e.slice(0,i+1),s=r.apply(void 0,a);if(0===s)throw new Error("Input not in general position");for(var l=new Array(i+1),c=0;c<=i;++c)l[c]=c;s<0&&(l[0]=1,l[1]=0);var d=new o(l,new Array(i+1),!1),h=d.adjacent,f=new Array(i+2);for(c=0;c<=i;++c){for(var p=l.slice(),g=0;g<=i;++g)g===c&&(p[g]=-1);var v=p[0];p[0]=p[1],p[1]=v;var m=new o(p,new Array(i+1),!0);h[c]=m,f[c]=m}f[i+1]=d;for(c=0;c<=i;++c){p=h[c].vertices;var y=h[c].adjacent;for(g=0;g<=i;++g){var b=p[g];if(b<0)y[g]=d;else for(var C=0;C<=i;++C)h[C].vertices.indexOf(b)<0&&(y[g]=h[C])}}var A=new u(i,a,f),_=!!t;for(c=i+1;c<n;++c)A.insert(e[c],_);return A.boundary()};var r=n(150),i=n(1895).compareCells;function o(e,t,n){this.vertices=e,this.adjacent=t,this.boundary=n,this.lastVisited=-1}function a(e,t,n){this.vertices=e,this.cell=t,this.index=n}function s(e,t){return i(e.vertices,t.vertices)}o.prototype.flip=function(){var e=this.vertices[0];this.vertices[0]=this.vertices[1],this.vertices[1]=e;var t=this.adjacent[0];this.adjacent[0]=this.adjacent[1],this.adjacent[1]=t};var l=[];function u(e,t,n){this.dimension=e,this.vertices=t,this.simplices=n,this.interior=n.filter((function(e){return!e.boundary})),this.tuple=new Array(e+1);for(var i=0;i<=e;++i)this.tuple[i]=this.vertices[i];var o=l[e];o||(o=l[e]=function(e){for(var t=["function orient(){var tuple=this.tuple;return test("],n=0;n<=e;++n)n>0&&t.push(","),t.push("tuple[",n,"]");t.push(")}return orient");var i=new Function("test",t.join("")),o=r[e+1];return o||(o=r),i(o)}(e)),this.orient=o}var c=u.prototype;c.handleBoundaryDegeneracy=function(e,t){var n=this.dimension,r=this.vertices.length-1,i=this.tuple,o=this.vertices,a=[e];for(e.lastVisited=-r;a.length>0;){(e=a.pop()).vertices;for(var s=e.adjacent,l=0;l<=n;++l){var u=s[l];if(u.boundary&&!(u.lastVisited<=-r)){for(var c=u.vertices,d=0;d<=n;++d){var h=c[d];i[d]=h<0?t:o[h]}var f=this.orient();if(f>0)return u;u.lastVisited=-r,0===f&&a.push(u)}}}return null},c.walk=function(e,t){var n=this.vertices.length-1,r=this.dimension,i=this.vertices,o=this.tuple,a=t?this.interior.length*Math.random()|0:this.interior.length-1,s=this.interior[a];e:for(;!s.boundary;){for(var l=s.vertices,u=s.adjacent,c=0;c<=r;++c)o[c]=i[l[c]];s.lastVisited=n;for(c=0;c<=r;++c){var d=u[c];if(!(d.lastVisited>=n)){var h=o[c];o[c]=e;var f=this.orient();if(o[c]=h,f<0){s=d;continue e}d.boundary?d.lastVisited=-n:d.lastVisited=n}}return}return s},c.addPeaks=function(e,t){var n=this.vertices.length-1,r=this.dimension,i=this.vertices,l=this.tuple,u=this.interior,c=this.simplices,d=[t];t.lastVisited=n,t.vertices[t.vertices.indexOf(-1)]=n,t.boundary=!1,u.push(t);for(var h=[];d.length>0;){var f=(t=d.pop()).vertices,p=t.adjacent,g=f.indexOf(n);if(!(g<0))for(var v=0;v<=r;++v)if(v!==g){var m=p[v];if(m.boundary&&!(m.lastVisited>=n)){var y=m.vertices;if(m.lastVisited!==-n){for(var b=0,C=0;C<=r;++C)y[C]<0?(b=C,l[C]=e):l[C]=i[y[C]];if(this.orient()>0){y[b]=n,m.boundary=!1,u.push(m),d.push(m),m.lastVisited=n;continue}m.lastVisited=-n}var A=m.adjacent,_=f.slice(),I=p.slice(),x=new o(_,I,!0);c.push(x);var w=A.indexOf(t);if(!(w<0)){A[w]=x,I[g]=m,_[v]=-1,I[v]=t,p[v]=x,x.flip();for(C=0;C<=r;++C){var M=_[C];if(!(M<0||M===n)){for(var S=new Array(r-1),T=0,E=0;E<=r;++E){var O=_[E];O<0||E===C||(S[T++]=O)}h.push(new a(S,x,C))}}}}}}h.sort(s);for(v=0;v+1<h.length;v+=2){var k=h[v],L=h[v+1],D=k.index,P=L.index;D<0||P<0||(k.cell.adjacent[k.index]=L.cell,L.cell.adjacent[L.index]=k.cell)}},c.insert=function(e,t){var n=this.vertices;n.push(e);var r=this.walk(e,t);if(r){for(var i=this.dimension,o=this.tuple,a=0;a<=i;++a){var s=r.vertices[a];o[a]=s<0?e:n[s]}var l=this.orient(o);l<0||(0!==l||(r=this.handleBoundaryDegeneracy(r,e)))&&this.addPeaks(e,r)}},c.boundary=function(){for(var e=this.dimension,t=[],n=this.simplices,r=n.length,i=0;i<r;++i){var o=n[i];if(o.boundary){for(var a=new Array(e),s=o.vertices,l=0,u=0,c=0;c<=e;++c)s[c]>=0?a[l++]=s[c]:u=1&c;if(u===(1&e)){var d=a[0];a[0]=a[1],a[1]=d}t.push(a)}}return t}},function(e,t,n){"use strict";var r=n(1900);function i(e){for(var t="robustLinearSolve"+e+"d",n=["function ",t,"(A,b){return ["],i=0;i<e;++i){n.push("det([");for(var o=0;o<e;++o){o>0&&n.push(","),n.push("[");for(var a=0;a<e;++a)a>0&&n.push(","),a===i?n.push("+b[",o,"]"):n.push("+A[",o,"][",a,"]");n.push("]")}n.push("]),")}n.push("det(A)]}return ",t);var s=new Function("det",n.join(""));return s(e<6?r[e]:r)}var o=[function(){return[0]},function(e,t){return[[t[0]],[e[0][0]]]}];!function(){for(;o.length<6;)o.push(i(o.length));for(var t=[],n=["function dispatchLinearSolve(A,b){switch(A.length){"],r=0;r<6;++r)t.push("s"+r),n.push("case ",r,":return s",r,"(A,b);");n.push("}var s=CACHE[A.length];if(!s)s=CACHE[A.length]=g(A.length);return s(A,b)}return dispatchLinearSolve"),t.push("CACHE","g",n.join(""));var a=Function.apply(void 0,t);for(e.exports=a.apply(void 0,o.concat([o,i])),r=0;r<6;++r)e.exports[r]=o[r]}()},function(e,t,n){"use strict";var r=n(1903),i=n(1908),o=n(1909);e.exports=function(e,t){"float"!==t&&t||(t="array"),"uint"===t&&(t="uint8"),"uint_clamped"===t&&(t="uint8_clamped");var n=new(o(t))(4),a="uint8"!==t&&"uint8_clamped"!==t;return e.length&&"string"!=typeof e||((e=r(e))[0]/=255,e[1]/=255,e[2]/=255),function(e){return e instanceof Uint8Array||e instanceof Uint8ClampedArray||!!(Array.isArray(e)&&(e[0]>1||0===e[0])&&(e[1]>1||0===e[1])&&(e[2]>1||0===e[2])&&(!e[3]||e[3]>1))}(e)?(n[0]=e[0],n[1]=e[1],n[2]=e[2],n[3]=null!=e[3]?e[3]:255,a&&(n[0]/=255,n[1]/=255,n[2]/=255,n[3]/=255),n):(a?(n[0]=e[0],n[1]=e[1],n[2]=e[2],n[3]=null!=e[3]?e[3]:1):(n[0]=i(Math.floor(255*e[0]),0,255),n[1]=i(Math.floor(255*e[1]),0,255),n[2]=i(Math.floor(255*e[2]),0,255),n[3]=null==e[3]?255:i(Math.floor(255*e[3]),0,255)),n)}},function(e,t,n){"use strict";var r=n(42),i=n(123),o=n(1057),a=n(195),s=n(133).defaultLine,l=n(249).isArrayOrTypedArray,u=o(s);function c(e,t){var n=e;return n[3]*=t,n}function d(e){if(r(e))return u;var t=o(e);return t.length?t:u}function h(e){return r(e)?e:1}e.exports={formatColor:function(e,t,n){var r,i,s,f,p,g=e.color,v=l(g),m=l(t),y=a.extractOpts(e),b=[];if(r=void 0!==y.colorscale?a.makeColorScaleFuncFromTrace(e):d,i=v?function(e,t){return void 0===e[t]?u:o(r(e[t]))}:d,s=m?function(e,t){return void 0===e[t]?1:h(e[t])}:h,v||m)for(var C=0;C<n;C++)f=i(g,C),p=s(t,C),b[C]=c(f,p);else b=c(o(g),t);return b},parseColorScale:function(e){var t=a.extractOpts(e),n=t.colorscale;return t.reversescale&&(n=a.flipScale(t.colorscale)),n.map((function(e){var t=e[0],n=i(e[1]).toRgb();return{index:t,rgb:[n.r,n.g,n.b,n.a]}}))}}},function(e,t,n){"use strict";e.exports={solid:[[],0],dot:[[.5,1],200],dash:[[.5,1],50],longdash:[[.5,1],10],dashdot:[[.5,.625,.875,1],50],longdashdot:[[.5,.7,.8,1],10]}},function(e,t,n){"use strict";var r=n(193),i=n(161),o=n(177).hovertemplateAttrs,a=n(177).texttemplateAttrs,s=n(160),l=n(1059),u=n(574),c=n(63).extendFlat,d=n(124).overrideAll,h=r.line,f=r.marker,p=f.line,g=c({width:h.width,dash:{valType:"enumerated",values:Object.keys(l),dflt:"solid"}},i("line"));var v=e.exports=d({x:r.x,y:r.y,z:{valType:"data_array"},text:c({},r.text,{}),texttemplate:a({},{}),hovertext:c({},r.hovertext,{}),hovertemplate:o(),mode:c({},r.mode,{dflt:"lines+markers"}),surfaceaxis:{valType:"enumerated",values:[-1,0,1,2],dflt:-1},surfacecolor:{valType:"color"},projection:{x:{show:{valType:"boolean",dflt:!1},opacity:{valType:"number",min:0,max:1,dflt:1},scale:{valType:"number",min:0,max:10,dflt:2/3}},y:{show:{valType:"boolean",dflt:!1},opacity:{valType:"number",min:0,max:1,dflt:1},scale:{valType:"number",min:0,max:10,dflt:2/3}},z:{show:{valType:"boolean",dflt:!1},opacity:{valType:"number",min:0,max:1,dflt:1},scale:{valType:"number",min:0,max:10,dflt:2/3}}},connectgaps:r.connectgaps,line:g,marker:c({symbol:{valType:"enumerated",values:Object.keys(u),dflt:"circle",arrayOk:!0},size:c({},f.size,{dflt:8}),sizeref:f.sizeref,sizemin:f.sizemin,sizemode:f.sizemode,opacity:c({},f.opacity,{arrayOk:!1}),colorbar:f.colorbar,line:c({width:c({},p.width,{arrayOk:!1})},i("marker.line"))},i("marker")),textposition:c({},r.textposition,{dflt:"top center"}),textfont:{color:r.textfont.color,size:r.textfont.size,family:c({},r.textfont.family,{arrayOk:!1})},hoverinfo:c({},s.hoverinfo)},"calc","nested");v.x.editType=v.y.editType=v.z.editType="calc+clearAxisTypes"},function(e,t,n){"use strict";var r=n(124).overrideAll,i=n(250),o=n(1913),a=n(384).getSubplotData,s=n(10),l=n(196);t.name="gl3d",t.attr="scene",t.idRoot="scene",t.idRegex=t.attrRegex=s.counterRegex("scene"),t.attributes=n(1958),t.layoutAttributes=n(1069),t.baseLayoutAttrOverrides=r({hoverlabel:i.hoverlabel},"plot","nested"),t.supplyLayoutDefaults=n(1959),t.plot=function(e){for(var t=e._fullLayout,n=e._fullData,r=t._subplots.gl3d,i=0;i<r.length;i++){var s=r[i],l=a(n,"gl3d",s),u=t[s],c=u.camera,d=u._scene;d||(d=new o({id:s,graphDiv:e,container:e.querySelector(".gl-container"),staticPlot:e._context.staticPlot,plotGlPixelRatio:e._context.plotGlPixelRatio,camera:c},t),u._scene=d),d.viewInitial||(d.viewInitial={up:{x:c.up.x,y:c.up.y,z:c.up.z},eye:{x:c.eye.x,y:c.eye.y,z:c.eye.z},center:{x:c.center.x,y:c.center.y,z:c.center.z}}),d.plot(l,t,e.layout)}},t.clean=function(e,t,n,r){for(var i=r._subplots.gl3d||[],o=0;o<i.length;o++){var a=i[o];!t[a]&&r[a]._scene&&(r[a]._scene.destroy(),r._infolayer&&r._infolayer.selectAll(".annotation-"+a).remove())}},t.toSVG=function(e){for(var t=e._fullLayout,n=t._subplots.gl3d,r=t._size,i=0;i<n.length;i++){var o=t[n[i]],a=o.domain,s=o._scene,u=s.toImage("png");t._glimages.append("svg:image").attr({xmlns:l.svg,"xlink:href":u,x:r.l+r.w*a.x[0],y:r.t+r.h*(1-a.y[1]),width:r.w*(a.x[1]-a.x[0]),height:r.h*(a.y[1]-a.y[0]),preserveAspectRatio:"none"}),s.destroy()}},t.cleanId=function(e){if(e.match(/^scene[0-9]*$/)){var t=e.substr(5);return"1"===t&&(t=""),"scene"+t}},t.updateFx=function(e){for(var t=e._fullLayout,n=t._subplots.gl3d,r=0;r<n.length;r++){t[n[r]]._scene.updateFx(t.dragmode,t.hovermode)}}},function(e,t,n){"use strict";e.exports=function(e,t,n){switch(arguments.length){case 0:return new a([0],[0],0);case 1:return"number"==typeof e?new a(r=l(e),r,0):new a(e,l(e.length),0);case 2:if("number"==typeof t){var r=l(e.length);return new a(e,r,+t)}n=0;case 3:if(e.length!==t.length)throw new Error("state and velocity lengths must match");return new a(e,t,n)}};var r=n(1919),i=n(1920);function o(e,t,n){return Math.min(t,Math.max(e,n))}function a(e,t,n){this.dimension=e.length,this.bounds=[new Array(this.dimension),new Array(this.dimension)];for(var r=0;r<this.dimension;++r)this.bounds[0][r]=-1/0,this.bounds[1][r]=1/0;this._state=e.slice().reverse(),this._velocity=t.slice().reverse(),this._time=[n],this._scratch=[e.slice(),e.slice(),e.slice(),e.slice(),e.slice()]}var s=a.prototype;function l(e){for(var t=new Array(e),n=0;n<e;++n)t[n]=0;return t}s.flush=function(e){var t=i.gt(this._time,e)-1;t<=0||(this._time.splice(0,t),this._state.splice(0,t*this.dimension),this._velocity.splice(0,t*this.dimension))},s.curve=function(e){var t=this._time,n=t.length,a=i.le(t,e),s=this._scratch[0],l=this._state,u=this._velocity,c=this.dimension,d=this.bounds;if(a<0)for(var h=c-1,f=0;f<c;++f,--h)s[f]=l[h];else if(a>=n-1){h=l.length-1;var p=e-t[n-1];for(f=0;f<c;++f,--h)s[f]=l[h]+p*u[h]}else{h=c*(a+1)-1;var g=t[a],v=t[a+1]-g||1,m=this._scratch[1],y=this._scratch[2],b=this._scratch[3],C=this._scratch[4],A=!0;for(f=0;f<c;++f,--h)m[f]=l[h],b[f]=u[h]*v,y[f]=l[h+c],C[f]=u[h+c]*v,A=A&&m[f]===y[f]&&b[f]===C[f]&&0===b[f];if(A)for(f=0;f<c;++f)s[f]=m[f];else r(m,b,y,C,(e-g)/v,s)}var _=d[0],I=d[1];for(f=0;f<c;++f)s[f]=o(_[f],I[f],s[f]);return s},s.dcurve=function(e){var t=this._time,n=t.length,o=i.le(t,e),a=this._scratch[0],s=this._state,l=this._velocity,u=this.dimension;if(o>=n-1)for(var c=s.length-1,d=(t[n-1],0);d<u;++d,--c)a[d]=l[c];else{c=u*(o+1)-1;var h=t[o],f=t[o+1]-h||1,p=this._scratch[1],g=this._scratch[2],v=this._scratch[3],m=this._scratch[4],y=!0;for(d=0;d<u;++d,--c)p[d]=s[c],v[d]=l[c]*f,g[d]=s[c+u],m[d]=l[c+u]*f,y=y&&p[d]===g[d]&&v[d]===m[d]&&0===v[d];if(y)for(d=0;d<u;++d)a[d]=0;else{r.derivative(p,v,g,m,(e-h)/f,a);for(d=0;d<u;++d)a[d]/=f}}return a},s.lastT=function(){var e=this._time;return e[e.length-1]},s.stable=function(){for(var e=this._velocity,t=e.length,n=this.dimension-1;n>=0;--n)if(e[--t])return!1;return!0},s.jump=function(e){var t=this.lastT(),n=this.dimension;if(!(e<t||arguments.length!==n+1)){var r=this._state,i=this._velocity,a=r.length-this.dimension,s=this.bounds,l=s[0],u=s[1];this._time.push(t,e);for(var c=0;c<2;++c)for(var d=0;d<n;++d)r.push(r[a++]),i.push(0);this._time.push(e);for(d=n;d>0;--d)r.push(o(l[d-1],u[d-1],arguments[d])),i.push(0)}},s.push=function(e){var t=this.lastT(),n=this.dimension;if(!(e<t||arguments.length!==n+1)){var r=this._state,i=this._velocity,a=r.length-this.dimension,s=e-t,l=this.bounds,u=l[0],c=l[1],d=s>1e-6?1/s:0;this._time.push(e);for(var h=n;h>0;--h){var f=o(u[h-1],c[h-1],arguments[h]);r.push(f),i.push((f-r[a++])*d)}}},s.set=function(e){var t=this.dimension;if(!(e<this.lastT()||arguments.length!==t+1)){var n=this._state,r=this._velocity,i=this.bounds,a=i[0],s=i[1];this._time.push(e);for(var l=t;l>0;--l)n.push(o(a[l-1],s[l-1],arguments[l])),r.push(0)}},s.move=function(e){var t=this.lastT(),n=this.dimension;if(!(e<=t||arguments.length!==n+1)){var r=this._state,i=this._velocity,a=r.length-this.dimension,s=this.bounds,l=s[0],u=s[1],c=e-t,d=c>1e-6?1/c:0;this._time.push(e);for(var h=n;h>0;--h){var f=arguments[h];r.push(o(l[h-1],u[h-1],r[a++]+f)),i.push(f*d)}}},s.idle=function(e){var t=this.lastT();if(!(e<t)){var n=this.dimension,r=this._state,i=this._velocity,a=r.length-n,s=this.bounds,l=s[0],u=s[1],c=e-t;this._time.push(e);for(var d=n-1;d>=0;--d)r.push(o(l[d],u[d],r[a]+c*i[a])),i.push(0),a+=1}}},function(e,t){e.exports=function(e,t,n){var r=t[0],i=t[1],o=t[2],a=n[0],s=n[1],l=n[2];return e[0]=i*l-o*s,e[1]=o*a-r*l,e[2]=r*s-i*a,e}},function(e,t){e.exports=function(e,t){return e[0]*t[0]+e[1]*t[1]+e[2]*t[2]}},function(e,t,n){"use strict";e.exports=function(e,t){t||(t=e,e=window);var n=0,i=0,o=0,a={shift:!1,alt:!1,control:!1,meta:!1},s=!1;function l(e){var t=!1;return"altKey"in e&&(t=t||e.altKey!==a.alt,a.alt=!!e.altKey),"shiftKey"in e&&(t=t||e.shiftKey!==a.shift,a.shift=!!e.shiftKey),"ctrlKey"in e&&(t=t||e.ctrlKey!==a.control,a.control=!!e.ctrlKey),"metaKey"in e&&(t=t||e.metaKey!==a.meta,a.meta=!!e.metaKey),t}function u(e,s){var u=r.x(s),c=r.y(s);"buttons"in s&&(e=0|s.buttons),(e!==n||u!==i||c!==o||l(s))&&(n=0|e,i=u||0,o=c||0,t&&t(n,i,o,a))}function c(e){u(0,e)}function d(){(n||i||o||a.shift||a.alt||a.meta||a.control)&&(i=o=0,n=0,a.shift=a.alt=a.control=a.meta=!1,t&&t(0,0,0,a))}function h(e){l(e)&&t&&t(n,i,o,a)}function f(e){0===r.buttons(e)?u(0,e):u(n,e)}function p(e){u(n|r.buttons(e),e)}function g(e){u(n&~r.buttons(e),e)}function v(){s||(s=!0,e.addEventListener("mousemove",f),e.addEventListener("mousedown",p),e.addEventListener("mouseup",g),e.addEventListener("mouseleave",c),e.addEventListener("mouseenter",c),e.addEventListener("mouseout",c),e.addEventListener("mouseover",c),e.addEventListener("blur",d),e.addEventListener("keyup",h),e.addEventListener("keydown",h),e.addEventListener("keypress",h),e!==window&&(window.addEventListener("blur",d),window.addEventListener("keyup",h),window.addEventListener("keydown",h),window.addEventListener("keypress",h)))}v();var m={element:e};return Object.defineProperties(m,{enabled:{get:function(){return s},set:function(t){t?v():function(){if(!s)return;s=!1,e.removeEventListener("mousemove",f),e.removeEventListener("mousedown",p),e.removeEventListener("mouseup",g),e.removeEventListener("mouseleave",c),e.removeEventListener("mouseenter",c),e.removeEventListener("mouseout",c),e.removeEventListener("mouseover",c),e.removeEventListener("blur",d),e.removeEventListener("keyup",h),e.removeEventListener("keydown",h),e.removeEventListener("keypress",h),e!==window&&(window.removeEventListener("blur",d),window.removeEventListener("keyup",h),window.removeEventListener("keydown",h),window.removeEventListener("keypress",h))}()},enumerable:!0},buttons:{get:function(){return n},enumerable:!0},x:{get:function(){return i},enumerable:!0},y:{get:function(){return o},enumerable:!0},mods:{get:function(){return a},enumerable:!0}}),m};var r=n(1933)},function(e,t,n){"use strict";e.exports=function(e,t,n,o,f){i(s,t,e),i(s,n,s);for(var y=0,b=0;b<2;++b){c[2]=o[b][2];for(var C=0;C<2;++C){c[1]=o[C][1];for(var A=0;A<2;++A)c[0]=o[A][0],h(l[y],c,s),y+=1}}var _=-1;for(b=0;b<8;++b){for(var I=l[b][3],x=0;x<3;++x)u[b][x]=l[b][x]/I;f&&(u[b][2]*=-1),I<0&&(_<0||u[b][2]<u[_][2])&&(_=b)}if(_<0){_=0;for(var w=0;w<3;++w){for(var M=(w+2)%3,S=(w+1)%3,T=-1,E=-1,O=0;O<2;++O){var k=(D=O<<w)+(O<<M)+(1-O<<S),L=D+(1-O<<M)+(O<<S);a(u[D],u[k],u[L],d)<0||(O?T=1:E=1)}if(T<0||E<0)E>T&&(_|=1<<w);else{for(O=0;O<2;++O){k=(D=O<<w)+(O<<M)+(1-O<<S),L=D+(1-O<<M)+(O<<S);var D,P=p([l[D],l[k],l[L],l[D+(1<<M)+(1<<S)]]);O?T=P:E=P}E>T&&(_|=1<<w)}}}var N=7^_,j=-1;for(b=0;b<8;++b)b!==_&&b!==N&&(j<0||u[j][1]>u[b][1])&&(j=b);var R=-1;for(b=0;b<3;++b){if((F=j^1<<b)!==_&&F!==N)R<0&&(R=F),(S=u[F])[0]<u[R][0]&&(R=F)}var z=-1;for(b=0;b<3;++b){var F;if((F=j^1<<b)!==_&&F!==N&&F!==R)z<0&&(z=F),(S=u[F])[0]>u[z][0]&&(z=F)}var B=g;B[0]=B[1]=B[2]=0,B[r.log2(R^j)]=j&R,B[r.log2(j^z)]=j&z;var H=7^z;H===_||H===N?(H=7^R,B[r.log2(z^H)]=H&z):B[r.log2(R^H)]=H&R;var U=v,V=_;for(w=0;w<3;++w)U[w]=V&1<<w?-1:1;return m};var r=n(215),i=n(255),o=n(1067),a=n(150),s=new Array(16),l=new Array(8),u=new Array(8),c=new Array(3),d=[0,0,0];function h(e,t,n){for(var r=0;r<4;++r){e[r]=n[12+r];for(var i=0;i<3;++i)e[r]+=t[i]*n[4*i+r]}}!function(){for(var e=0;e<8;++e)l[e]=[1,1,1,1],u[e]=[1,1,1]}();var f=[[0,0,1,0,0],[0,0,-1,1,0],[0,-1,0,1,0],[0,1,0,1,0],[-1,0,0,1,0],[1,0,0,1,0]];function p(e){for(var t=0;t<f.length;++t)if((e=o.positive(e,f[t])).length<3)return 0;var n=e[0],r=n[0]/n[3],i=n[1]/n[3],a=0;for(t=1;t+1<e.length;++t){var s=e[t],l=e[t+1],u=s[0]/s[3]-r,c=s[1]/s[3]-i,d=l[0]/l[3]-r,h=l[1]/l[3]-i;a+=Math.abs(u*h-c*d)}return a}var g=[1,1,1],v=[0,0,0],m={cubeEdges:g,axis:v}},function(e,t,n){"use strict";var r=n(1941),i=n(197);function o(e,t){var n=i(r(e,t),[t[t.length-1]]);return n[n.length-1]}function a(e,t,n,r){var i=-t/(r-t);i<0?i=0:i>1&&(i=1);for(var o=1-i,a=e.length,s=new Array(a),l=0;l<a;++l)s[l]=i*e[l]+o*n[l];return s}e.exports=function(e,t){for(var n=[],r=[],i=o(e[e.length-1],t),s=e[e.length-1],l=e[0],u=0;u<e.length;++u,s=l){var c=o(l=e[u],t);if(i<0&&c>0||i>0&&c<0){var d=a(s,c,l,i);n.push(d),r.push(d.slice())}c<0?r.push(l.slice()):c>0?n.push(l.slice()):(n.push(l.slice()),r.push(l.slice())),i=c}return{positive:n,negative:r}},e.exports.positive=function(e,t){for(var n=[],r=o(e[e.length-1],t),i=e[e.length-1],s=e[0],l=0;l<e.length;++l,i=s){var u=o(s=e[l],t);(r<0&&u>0||r>0&&u<0)&&n.push(a(i,u,s,r)),u>=0&&n.push(s.slice()),r=u}return n},e.exports.negative=function(e,t){for(var n=[],r=o(e[e.length-1],t),i=e[e.length-1],s=e[0],l=0;l<e.length;++l,i=s){var u=o(s=e[l],t);(r<0&&u>0||r>0&&u<0)&&n.push(a(i,u,s,r)),u<=0&&n.push(s.slice()),r=u}return n}},function(e,t,n){"use strict";var r=n(569);e.exports=function(e,t,n,r){i||(i=e.FRAMEBUFFER_UNSUPPORTED,o=e.FRAMEBUFFER_INCOMPLETE_ATTACHMENT,a=e.FRAMEBUFFER_INCOMPLETE_DIMENSIONS,s=e.FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT);var u=e.getExtension("WEBGL_draw_buffers");!l&&u&&function(e,t){var n=e.getParameter(t.MAX_COLOR_ATTACHMENTS_WEBGL);l=new Array(n+1);for(var r=0;r<=n;++r){for(var i=new Array(n),o=0;o<r;++o)i[o]=e.COLOR_ATTACHMENT0+o;for(o=r;o<n;++o)i[o]=e.NONE;l[r]=i}}(e,u);Array.isArray(t)&&(r=n,n=0|t[1],t=0|t[0]);if("number"!=typeof t)throw new Error("gl-fbo: Missing shape parameter");var c=e.getParameter(e.MAX_RENDERBUFFER_SIZE);if(t<0||t>c||n<0||n>c)throw new Error("gl-fbo: Parameters are too large for FBO");var d=1;if("color"in(r=r||{})){if((d=Math.max(0|r.color,0))<0)throw new Error("gl-fbo: Must specify a nonnegative number of colors");if(d>1){if(!u)throw new Error("gl-fbo: Multiple draw buffer extension not supported");if(d>e.getParameter(u.MAX_COLOR_ATTACHMENTS_WEBGL))throw new Error("gl-fbo: Context does not support "+d+" draw buffers")}}var h=e.UNSIGNED_BYTE,f=e.getExtension("OES_texture_float");if(r.float&&d>0){if(!f)throw new Error("gl-fbo: Context does not support floating point textures");h=e.FLOAT}else r.preferFloat&&d>0&&f&&(h=e.FLOAT);var g=!0;"depth"in r&&(g=!!r.depth);var v=!1;"stencil"in r&&(v=!!r.stencil);return new p(e,t,n,h,d,g,v,u)};var i,o,a,s,l=null;function u(e){return[e.getParameter(e.FRAMEBUFFER_BINDING),e.getParameter(e.RENDERBUFFER_BINDING),e.getParameter(e.TEXTURE_BINDING_2D)]}function c(e,t){e.bindFramebuffer(e.FRAMEBUFFER,t[0]),e.bindRenderbuffer(e.RENDERBUFFER,t[1]),e.bindTexture(e.TEXTURE_2D,t[2])}function d(e){switch(e){case i:throw new Error("gl-fbo: Framebuffer unsupported");case o:throw new Error("gl-fbo: Framebuffer incomplete attachment");case a:throw new Error("gl-fbo: Framebuffer incomplete dimensions");case s:throw new Error("gl-fbo: Framebuffer incomplete missing attachment");default:throw new Error("gl-fbo: Framebuffer failed for unspecified reason")}}function h(e,t,n,i,o,a){if(!i)return null;var s=r(e,t,n,o,i);return s.magFilter=e.NEAREST,s.minFilter=e.NEAREST,s.mipSamples=1,s.bind(),e.framebufferTexture2D(e.FRAMEBUFFER,a,e.TEXTURE_2D,s.handle,0),s}function f(e,t,n,r,i){var o=e.createRenderbuffer();return e.bindRenderbuffer(e.RENDERBUFFER,o),e.renderbufferStorage(e.RENDERBUFFER,r,t,n),e.framebufferRenderbuffer(e.FRAMEBUFFER,i,e.RENDERBUFFER,o),o}function p(e,t,n,r,i,o,a,s){this.gl=e,this._shape=[0|t,0|n],this._destroyed=!1,this._ext=s,this.color=new Array(i);for(var p=0;p<i;++p)this.color[p]=null;this._color_rb=null,this.depth=null,this._depth_rb=null,this._colorType=r,this._useDepth=o,this._useStencil=a;var g=this,v=[0|t,0|n];Object.defineProperties(v,{0:{get:function(){return g._shape[0]},set:function(e){return g.width=e}},1:{get:function(){return g._shape[1]},set:function(e){return g.height=e}}}),this._shapeVector=v,function(e){var t=u(e.gl),n=e.gl,r=e.handle=n.createFramebuffer(),i=e._shape[0],o=e._shape[1],a=e.color.length,s=e._ext,p=e._useStencil,g=e._useDepth,v=e._colorType;n.bindFramebuffer(n.FRAMEBUFFER,r);for(var m=0;m<a;++m)e.color[m]=h(n,i,o,v,n.RGBA,n.COLOR_ATTACHMENT0+m);0===a?(e._color_rb=f(n,i,o,n.RGBA4,n.COLOR_ATTACHMENT0),s&&s.drawBuffersWEBGL(l[0])):a>1&&s.drawBuffersWEBGL(l[a]);var y=n.getExtension("WEBGL_depth_texture");y?p?e.depth=h(n,i,o,y.UNSIGNED_INT_24_8_WEBGL,n.DEPTH_STENCIL,n.DEPTH_STENCIL_ATTACHMENT):g&&(e.depth=h(n,i,o,n.UNSIGNED_SHORT,n.DEPTH_COMPONENT,n.DEPTH_ATTACHMENT)):g&&p?e._depth_rb=f(n,i,o,n.DEPTH_STENCIL,n.DEPTH_STENCIL_ATTACHMENT):g?e._depth_rb=f(n,i,o,n.DEPTH_COMPONENT16,n.DEPTH_ATTACHMENT):p&&(e._depth_rb=f(n,i,o,n.STENCIL_INDEX,n.STENCIL_ATTACHMENT));var b=n.checkFramebufferStatus(n.FRAMEBUFFER);if(b!==n.FRAMEBUFFER_COMPLETE){e._destroyed=!0,n.bindFramebuffer(n.FRAMEBUFFER,null),n.deleteFramebuffer(e.handle),e.handle=null,e.depth&&(e.depth.dispose(),e.depth=null),e._depth_rb&&(n.deleteRenderbuffer(e._depth_rb),e._depth_rb=null);for(m=0;m<e.color.length;++m)e.color[m].dispose(),e.color[m]=null;e._color_rb&&(n.deleteRenderbuffer(e._color_rb),e._color_rb=null),c(n,t),d(b)}c(n,t)}(this)}var g=p.prototype;function v(e,t,n){if(e._destroyed)throw new Error("gl-fbo: Can't resize destroyed FBO");if(e._shape[0]!==t||e._shape[1]!==n){var r=e.gl,i=r.getParameter(r.MAX_RENDERBUFFER_SIZE);if(t<0||t>i||n<0||n>i)throw new Error("gl-fbo: Can't resize FBO, invalid dimensions");e._shape[0]=t,e._shape[1]=n;for(var o=u(r),a=0;a<e.color.length;++a)e.color[a].shape=e._shape;e._color_rb&&(r.bindRenderbuffer(r.RENDERBUFFER,e._color_rb),r.renderbufferStorage(r.RENDERBUFFER,r.RGBA4,e._shape[0],e._shape[1])),e.depth&&(e.depth.shape=e._shape),e._depth_rb&&(r.bindRenderbuffer(r.RENDERBUFFER,e._depth_rb),e._useDepth&&e._useStencil?r.renderbufferStorage(r.RENDERBUFFER,r.DEPTH_STENCIL,e._shape[0],e._shape[1]):e._useDepth?r.renderbufferStorage(r.RENDERBUFFER,r.DEPTH_COMPONENT16,e._shape[0],e._shape[1]):e._useStencil&&r.renderbufferStorage(r.RENDERBUFFER,r.STENCIL_INDEX,e._shape[0],e._shape[1])),r.bindFramebuffer(r.FRAMEBUFFER,e.handle);var s=r.checkFramebufferStatus(r.FRAMEBUFFER);s!==r.FRAMEBUFFER_COMPLETE&&(e.dispose(),c(r,o),d(s)),c(r,o)}}Object.defineProperties(g,{shape:{get:function(){return this._destroyed?[0,0]:this._shapeVector},set:function(e){if(Array.isArray(e)||(e=[0|e,0|e]),2!==e.length)throw new Error("gl-fbo: Shape vector must be length 2");var t=0|e[0],n=0|e[1];return v(this,t,n),[t,n]},enumerable:!1},width:{get:function(){return this._destroyed?0:this._shape[0]},set:function(e){return v(this,e|=0,this._shape[1]),e},enumerable:!1},height:{get:function(){return this._destroyed?0:this._shape[1]},set:function(e){return e|=0,v(this,this._shape[0],e),e},enumerable:!1}}),g.bind=function(){if(!this._destroyed){var e=this.gl;e.bindFramebuffer(e.FRAMEBUFFER,this.handle),e.viewport(0,0,this._shape[0],this._shape[1])}},g.dispose=function(){if(!this._destroyed){this._destroyed=!0;var e=this.gl;e.deleteFramebuffer(this.handle),this.handle=null,this.depth&&(this.depth.dispose(),this.depth=null),this._depth_rb&&(e.deleteRenderbuffer(this._depth_rb),this._depth_rb=null);for(var t=0;t<this.color.length;++t)this.color[t].dispose(),this.color[t]=null;this._color_rb&&(e.deleteRenderbuffer(this._color_rb),this._color_rb=null)}}},function(e,t,n){"use strict";var r=n(1070),i=n(568).attributes,o=n(63).extendFlat,a=n(10).counterRegex;function s(e,t,n){return{x:{valType:"number",dflt:e,editType:"camera"},y:{valType:"number",dflt:t,editType:"camera"},z:{valType:"number",dflt:n,editType:"camera"},editType:"camera"}}e.exports={_arrayAttrRegexps:[a("scene",".annotations",!0)],bgcolor:{valType:"color",dflt:"rgba(0,0,0,0)",editType:"plot"},camera:{up:o(s(0,0,1),{}),center:o(s(0,0,0),{}),eye:o(s(1.25,1.25,1.25),{}),projection:{type:{valType:"enumerated",values:["perspective","orthographic"],dflt:"perspective",editType:"calc"},editType:"calc"},editType:"camera"},domain:i({name:"scene",editType:"plot"}),aspectmode:{valType:"enumerated",values:["auto","cube","data","manual"],dflt:"auto",editType:"plot",impliedEdits:{"aspectratio.x":void 0,"aspectratio.y":void 0,"aspectratio.z":void 0}},aspectratio:{x:{valType:"number",min:0,editType:"plot",impliedEdits:{"^aspectmode":"manual"}},y:{valType:"number",min:0,editType:"plot",impliedEdits:{"^aspectmode":"manual"}},z:{valType:"number",min:0,editType:"plot",impliedEdits:{"^aspectmode":"manual"}},editType:"plot",impliedEdits:{aspectmode:"manual"}},xaxis:r,yaxis:r,zaxis:r,dragmode:{valType:"enumerated",values:["orbit","turntable","zoom","pan",!1],editType:"plot"},hovermode:{valType:"enumerated",values:["closest",!1],dflt:"closest",editType:"modebar"},uirevision:{valType:"any",editType:"none"},editType:"plot",_deprecated:{cameraposition:{valType:"info_array",editType:"camera"}}}},function(e,t,n){"use strict";var r=n(41),i=n(134),o=n(63).extendFlat,a=n(124).overrideAll;e.exports=a({visible:i.visible,showspikes:{valType:"boolean",dflt:!0},spikesides:{valType:"boolean",dflt:!0},spikethickness:{valType:"number",min:0,dflt:2},spikecolor:{valType:"color",dflt:r.defaultLine},showbackground:{valType:"boolean",dflt:!1},backgroundcolor:{valType:"color",dflt:"rgba(204, 204, 204, 0.5)"},showaxeslabels:{valType:"boolean",dflt:!0},color:i.color,categoryorder:i.categoryorder,categoryarray:i.categoryarray,title:{text:i.title.text,font:i.title.font},type:o({},i.type,{values:["-","linear","log","date","category"]}),autotypenumbers:i.autotypenumbers,autorange:i.autorange,rangemode:i.rangemode,range:o({},i.range,{items:[{valType:"any",editType:"plot",impliedEdits:{"^autorange":!1}},{valType:"any",editType:"plot",impliedEdits:{"^autorange":!1}}],anim:!1}),tickmode:i.tickmode,nticks:i.nticks,tick0:i.tick0,dtick:i.dtick,tickvals:i.tickvals,ticktext:i.ticktext,ticks:i.ticks,mirror:i.mirror,ticklen:i.ticklen,tickwidth:i.tickwidth,tickcolor:i.tickcolor,showticklabels:i.showticklabels,tickfont:i.tickfont,tickangle:i.tickangle,tickprefix:i.tickprefix,showtickprefix:i.showtickprefix,ticksuffix:i.ticksuffix,showticksuffix:i.showticksuffix,showexponent:i.showexponent,exponentformat:i.exponentformat,minexponent:i.minexponent,separatethousands:i.separatethousands,tickformat:i.tickformat,tickformatstops:i.tickformatstops,hoverformat:i.hoverformat,showline:i.showline,linecolor:i.linecolor,linewidth:i.linewidth,showgrid:i.showgrid,gridcolor:o({},i.gridcolor,{dflt:"rgb(204, 204, 204)"}),gridwidth:i.gridwidth,zeroline:i.zeroline,zerolinecolor:i.zerolinecolor,zerolinewidth:i.zerolinewidth,_deprecated:{title:i._deprecated.title,titlefont:i._deprecated.titlefont}},"plot","from-root")},function(e,t,n){"use strict";e.exports=n(1962)},function(e,t,n){"use strict";var r=n(161),i=n(177).hovertemplateAttrs,o=n(1963),a=n(160),s=n(63).extendFlat;e.exports=s({x:{valType:"data_array",editType:"calc+clearAxisTypes"},y:{valType:"data_array",editType:"calc+clearAxisTypes"},z:{valType:"data_array",editType:"calc+clearAxisTypes"},i:{valType:"data_array",editType:"calc"},j:{valType:"data_array",editType:"calc"},k:{valType:"data_array",editType:"calc"},text:{valType:"string",dflt:"",arrayOk:!0,editType:"calc"},hovertext:{valType:"string",dflt:"",arrayOk:!0,editType:"calc"},hovertemplate:i({editType:"calc"}),delaunayaxis:{valType:"enumerated",values:["x","y","z"],dflt:"z",editType:"calc"},alphahull:{valType:"number",dflt:-1,editType:"calc"},intensity:{valType:"data_array",editType:"calc"},intensitymode:{valType:"enumerated",values:["vertex","cell"],dflt:"vertex",editType:"calc"},color:{valType:"color",editType:"calc"},vertexcolor:{valType:"data_array",editType:"calc"},facecolor:{valType:"data_array",editType:"calc"},transforms:void 0},r("",{colorAttr:"`intensity`",showScaleDflt:!0,editTypeOverride:"calc"}),{opacity:o.opacity,flatshading:{valType:"boolean",dflt:!1,editType:"calc"},contour:{show:s({},o.contours.x.show,{}),color:o.contours.x.color,width:o.contours.x.width,editType:"calc"},lightposition:{x:s({},o.lightposition.x,{dflt:1e5}),y:s({},o.lightposition.y,{dflt:1e5}),z:s({},o.lightposition.z,{dflt:0}),editType:"calc"},lighting:s({vertexnormalsepsilon:{valType:"number",min:0,max:1,dflt:1e-12,editType:"calc"},facenormalsepsilon:{valType:"number",min:0,max:1,dflt:1e-6,editType:"calc"},editType:"calc"},o.lighting),hoverinfo:s({},a.hoverinfo,{editType:"calc"}),showlegend:s({},a.showlegend,{dflt:!1})})},function(e,t){e.exports=function(e,t){var i=e.length,o=e.length-t.length;if(o)return o;switch(i){case 0:return 0;case 1:return e[0]-t[0];case 2:return e[0]+e[1]-t[0]-t[1]||n(e[0],e[1])-n(t[0],t[1]);case 3:var a=e[0]+e[1],s=t[0]+t[1];if(o=a+e[2]-(s+t[2]))return o;var l=n(e[0],e[1]),u=n(t[0],t[1]);return n(l,e[2])-n(u,t[2])||n(l+e[2],a)-n(u+t[2],s);case 4:var c=e[0],d=e[1],h=e[2],f=e[3],p=t[0],g=t[1],v=t[2],m=t[3];return c+d+h+f-(p+g+v+m)||n(c,d,h,f)-n(p,g,v,m,p)||n(c+d,c+h,c+f,d+h,d+f,h+f)-n(p+g,p+v,p+m,g+v,g+m,v+m)||n(c+d+h,c+d+f,c+h+f,d+h+f)-n(p+g+v,p+g+m,p+v+m,g+v+m);default:for(var y=e.slice().sort(r),b=t.slice().sort(r),C=0;C<i;++C)if(o=y[C]-b[C])return o;return 0}};var n=Math.min;function r(e,t){return e-t}},function(e,t,n){"use strict";e.exports=function(e){for(var t=1,n=1;n<e.length;++n)for(var r=0;r<n;++r)if(e[n]<e[r])t=-t;else if(e[r]===e[n])return 0;return t}},function(e,t,n){"use strict";e.exports={TEXTPAD:3,eventDataKeys:["value","label"]}},function(e,t,n){"use strict";e.exports={barmode:{valType:"enumerated",values:["stack","group","overlay","relative"],dflt:"group",editType:"calc"},barnorm:{valType:"enumerated",values:["","fraction","percent"],dflt:"",editType:"calc"},bargap:{valType:"number",min:0,max:1,editType:"calc"},bargroupgap:{valType:"number",min:0,max:1,dflt:0,editType:"calc"}}},function(e,t,n){"use strict";var r=n(10),i=n(41),o=n(27),a=n(1008),s=n(1009),l=n(1978),u=n(390).getAxisGroup,c=n(398),d=r.coerceFont;function h(e,t,n,r){var i=t.orientation,o=t[{v:"x",h:"y"}[i]+"axis"],a=u(n,o)+i,s=n._alignmentOpts||{},l=r("alignmentgroup"),c=s[a];c||(c=s[a]={});var d=c[l];d?d.traces.push(t):d=c[l]={traces:[t],alignmentIndex:Object.keys(c).length,offsetGroups:{}};var h=r("offsetgroup"),f=d.offsetGroups,p=f[h];h&&(p||(p=f[h]={offsetIndex:Object.keys(f).length}),t._offsetIndex=p.offsetIndex)}function f(e,t,n,i,o,a){var s=!(!1===(a=a||{}).moduleHasSelected),l=!(!1===a.moduleHasUnselected),u=!(!1===a.moduleHasConstrain),c=!(!1===a.moduleHasCliponaxis),h=!(!1===a.moduleHasTextangle),f=!(!1===a.moduleHasInsideanchor),p=!!a.hasPathbar,g=Array.isArray(o)||"auto"===o,v=g||"inside"===o,m=g||"outside"===o;if(v||m){var y=d(i,"textfont",n.font),b=r.extendFlat({},y),C=!(e.textfont&&e.textfont.color);if(C&&delete b.color,d(i,"insidetextfont",b),p){var A=r.extendFlat({},y);C&&delete A.color,d(i,"pathbar.textfont",A)}m&&d(i,"outsidetextfont",y),s&&i("selected.textfont.color"),l&&i("unselected.textfont.color"),u&&i("constraintext"),c&&i("cliponaxis"),h&&i("textangle"),i("texttemplate")}v&&f&&i("insidetextanchor")}e.exports={supplyDefaults:function(e,t,n,u){function d(n,i){return r.coerce(e,t,c,n,i)}if(a(e,t,u,d)){s(e,t,u,d),d("orientation",t.x&&!t.y?"h":"v"),d("base"),d("offset"),d("width"),d("text"),d("hovertext"),d("hovertemplate");var h=d("textposition");f(e,t,u,d,h,{moduleHasSelected:!0,moduleHasUnselected:!0,moduleHasConstrain:!0,moduleHasCliponaxis:!0,moduleHasTextangle:!0,moduleHasInsideanchor:!0}),l(e,t,d,n,u);var p=(t.marker.line||{}).color,g=o.getComponentMethod("errorbars","supplyDefaults");g(e,t,p||i.defaultLine,{axis:"y"}),g(e,t,p||i.defaultLine,{axis:"x",inherit:"y"}),r.coerceSelectionMarkerOpacity(t,d)}else t.visible=!1},crossTraceDefaults:function(e,t){var n;function i(e){return r.coerce(n._input,n,c,e)}if("group"===t.barmode)for(var o=0;o<e.length;o++)"bar"===(n=e[o]).type&&(n._input,h(0,n,t,i))},handleGroupingDefaults:h,handleText:f}},function(e,t,n){"use strict";var r=n(10);e.exports=function(e,t){for(var n=0;n<e.length;n++)e[n].i=n;r.mergeArray(t.text,e,"tx"),r.mergeArray(t.hovertext,e,"htx");var i=t.marker;if(i){r.mergeArray(i.opacity,e,"mo",!0),r.mergeArray(i.color,e,"mc");var o=i.line;o&&(r.mergeArray(o.color,e,"mlc"),r.mergeArrayCastPositive(o.width,e,"mlw"))}}},function(e,t,n){"use strict";var r=n(39),i=n(10);function o(e){return"_"+e+"Text_minsize"}e.exports={recordMinTextSize:function(e,t,n){if(n.uniformtext.mode){var r=o(e),i=n.uniformtext.minsize,a=t.scale*t.fontSize;t.hide=a<i,n[r]=n[r]||1/0,t.hide||(n[r]=Math.min(n[r],Math.max(a,i)))}},clearMinTextSize:function(e,t){t[o(e)]=void 0},resizeText:function(e,t,n){var o=e._fullLayout,a=o["_"+n+"Text_minsize"];if(a){var s,l="hide"===o.uniformtext.mode;switch(n){case"funnelarea":case"pie":case"sunburst":s="g.slice";break;case"treemap":s="g.slice, g.pathbar";break;default:s="g.points > g.point"}t.selectAll(s).each((function(e){var t=e.transform;t&&(t.scale=l&&t.hide?0:a/t.fontSize,r.select(this).select("text").attr("transform",i.getTextTransform(t)))}))}}}},function(e,t,n){"use strict";var r=n(10).isArrayOrTypedArray;e.exports=function(e,t,n){var i;for(r(e)?e.length>t.length&&(e=e.slice(0,t.length)):e=[],i=0;i<t.length;i++)e[i]=n(t[i]);return e}},function(e,t,n){"use strict";e.exports=function(e,t,n){if(0===e.length)return"";var r,i=[],o=n?3:1;for(r=0;r<e.length;r+=o)i.push(e[r]+","+t[r]),n&&r<e.length-o&&(i.push("C"),i.push([e[r+1]+","+t[r+1],e[r+2]+","+t[r+2]+" "].join(" ")));return i.join(n?"":"L")}},function(e,t,n){"use strict";var r=n(42),i=n(10),o=n(72).BADNUM;e.exports=function(e,t,n,a){var s,l,u,c,d,h;function f(e){if(r(e))return+e}if(t&&t.transpose){for(s=0,d=0;d<e.length;d++)s=Math.max(s,e[d].length);if(0===s)return!1;u=function(e){return e.length},c=function(e,t,n){return(e[n]||[])[t]}}else s=e.length,u=function(e,t){return e[t].length},c=function(e,t,n){return(e[t]||[])[n]};var p=function(e,t,n){return t===o||n===o?o:c(e,t,n)};function g(e){if(t&&"carpet"!==t.type&&"contourcarpet"!==t.type&&e&&"category"===e.type&&t["_"+e._id.charAt(0)].length){var n=e._id.charAt(0),r={},a=t["_"+n+"CategoryMap"]||t[n];for(d=0;d<a.length;d++)r[a[d]]=d;return function(t){var n=r[e._categories[t]];return n+1?n:o}}return i.identity}var v=g(n),m=g(a);a&&"category"===a.type&&(s=a._categories.length);var y=new Array(s);for(d=0;d<s;d++)for(l=n&&"category"===n.type?n._categories.length:u(e,d),y[d]=new Array(l),h=0;h<l;h++)y[d][h]=f(p(e,m(d),v(h)));return y}},function(e,t,n){"use strict";var r=n(10),i=n(72).BADNUM,o=n(557);e.exports=function(e,t,n,a,s,l){var u=e._length,c=t.makeCalcdata(e,a),d=n.makeCalcdata(e,s);c=o(e,t,a,c),d=o(e,n,s,d);var h,f,p,g,v=e.text,m=void 0!==v&&r.isArray1D(v),y=e.hovertext,b=void 0!==y&&r.isArray1D(y),C=r.distinctVals(c),A=C.vals,_=r.distinctVals(d),I=_.vals,x=[],w=I.length,M=A.length;for(h=0;h<l.length;h++)x[h]=r.init2dArray(w,M);m&&(p=r.init2dArray(w,M)),b&&(g=r.init2dArray(w,M));var S=r.init2dArray(w,M);for(h=0;h<u;h++)if(c[h]!==i&&d[h]!==i){var T=r.findBin(c[h]+C.minDiff/2,A),E=r.findBin(d[h]+_.minDiff/2,I);for(f=0;f<l.length;f++){var O=e[l[f]];x[f][E][T]=O[h],S[E][T]=h}m&&(p[E][T]=v[h]),b&&(g[E][T]=y[h])}for(e["_"+a]=A,e["_"+s]=I,f=0;f<l.length;f++)e["_"+l[f]]=x[f];m&&(e._text=p),b&&(e._hovertext=g),t&&"category"===t.type&&(e["_"+a+"CategoryMap"]=A.map((function(e){return t._categories[e]}))),n&&"category"===n.type&&(e["_"+s+"CategoryMap"]=I.map((function(e){return n._categories[e]}))),e._after2before=S}},function(e,t,n){"use strict";var r=n(1085),i=n(2012),o=n(161),a=n(63).extendFlat,s=i.contours;e.exports=a({carpet:{valType:"string",editType:"calc"},z:r.z,a:r.x,a0:r.x0,da:r.dx,b:r.y,b0:r.y0,db:r.dy,text:r.text,hovertext:r.hovertext,transpose:r.transpose,atype:r.xtype,btype:r.ytype,fillcolor:i.fillcolor,autocontour:i.autocontour,ncontours:i.ncontours,contours:{type:s.type,start:s.start,end:s.end,size:s.size,coloring:{valType:"enumerated",values:["fill","lines","none"],dflt:"fill",editType:"calc"},showlines:s.showlines,showlabels:s.showlabels,labelfont:s.labelfont,labelformat:s.labelformat,operation:s.operation,value:s.value,editType:"calc",impliedEdits:{autocontour:!1}},line:{color:i.line.color,width:i.line.width,dash:i.line.dash,smoothing:i.line.smoothing,editType:"plot"},transforms:void 0},o("",{cLetter:"z",autoColorDflt:!1}))},function(e,t,n){"use strict";var r=n(193),i=n(160),o=n(177).hovertemplateAttrs,a=n(161),s=(n(194).FORMAT_LINK,n(63).extendFlat);e.exports=s({z:{valType:"data_array",editType:"calc"},x:s({},r.x,{impliedEdits:{xtype:"array"}}),x0:s({},r.x0,{impliedEdits:{xtype:"scaled"}}),dx:s({},r.dx,{impliedEdits:{xtype:"scaled"}}),y:s({},r.y,{impliedEdits:{ytype:"array"}}),y0:s({},r.y0,{impliedEdits:{ytype:"scaled"}}),dy:s({},r.dy,{impliedEdits:{ytype:"scaled"}}),xperiod:s({},r.xperiod,{impliedEdits:{xtype:"scaled"}}),yperiod:s({},r.yperiod,{impliedEdits:{ytype:"scaled"}}),xperiod0:s({},r.xperiod0,{impliedEdits:{xtype:"scaled"}}),yperiod0:s({},r.yperiod0,{impliedEdits:{ytype:"scaled"}}),xperiodalignment:s({},r.xperiodalignment,{impliedEdits:{xtype:"scaled"}}),yperiodalignment:s({},r.yperiodalignment,{impliedEdits:{ytype:"scaled"}}),text:{valType:"data_array",editType:"calc"},hovertext:{valType:"data_array",editType:"calc"},transpose:{valType:"boolean",dflt:!1,editType:"calc"},xtype:{valType:"enumerated",values:["array","scaled"],editType:"calc+clearAxisTypes"},ytype:{valType:"enumerated",values:["array","scaled"],editType:"calc+clearAxisTypes"},zsmooth:{valType:"enumerated",values:["fast","best",!1],dflt:!1,editType:"calc"},hoverongaps:{valType:"boolean",dflt:!0,editType:"none"},connectgaps:{valType:"boolean",editType:"calc"},xgap:{valType:"number",dflt:0,min:0,editType:"plot"},ygap:{valType:"number",dflt:0,min:0,editType:"plot"},zhoverformat:{valType:"string",dflt:"",editType:"none"},hovertemplate:o(),showlegend:s({},i.showlegend,{dflt:!1})},{transforms:void 0},a("",{cLetter:"z",autoColorDflt:!1}))},function(e,t,n){"use strict";var r=n(10),i=n(2013),o=n(1084),a=n(2014),s=n(2015),l=n(2016);e.exports=function(e,t,n,u){function c(n,i){return r.coerce(e,t,o,n,i)}if(c("carpet"),e.a&&e.b){if(!i(e,t,c,u,"a","b"))return void(t.visible=!1);c("text"),"constraint"===c("contours.type")?a(e,t,c,u,n,{hasHover:!1}):(s(e,t,c,(function(n){return r.coerce2(e,t,o,n)})),l(e,t,c,u,{hasHover:!1}))}else t._defaultColor=n,t._length=null}},function(e,t,n){"use strict";var r=n(10);e.exports=function(e,t,n,i){if(i||(i={}),e("contours.showlabels")){var o=t.font;r.coerceFont(e,"contours.labelfont",{family:o.family,size:o.size,color:n}),e("contours.labelformat")}!1!==i.hasHover&&e("zhoverformat")}},function(e,t,n){"use strict";var r=n(39),i=n(195),o=n(581);e.exports=function(e){var t=e.contours,n=t.start,a=o(t),s=t.size||1,l=Math.floor((a-n)/s)+1,u="lines"===t.coloring?0:1,c=i.extractOpts(e);isFinite(s)||(s=1,l=1);var d,h,f=c.reversescale?i.flipScale(c.colorscale):c.colorscale,p=f.length,g=new Array(p),v=new Array(p);if("heatmap"===t.coloring){var m=c.min,y=c.max;for(h=0;h<p;h++)d=f[h],g[h]=d[0]*(y-m)+m,v[h]=d[1];var b=r.extent([m,y,t.start,t.start+s*(l-1)]),C=b[m<y?0:1],A=b[m<y?1:0];C!==m&&(g.splice(0,0,C),v.splice(0,0,v[0])),A!==y&&(g.push(A),v.push(v[v.length-1]))}else for(h=0;h<p;h++)d=f[h],g[h]=(d[0]*(l+u-1)-u/2)*s+n,v[h]=d[1];return i.makeColorScaleFunc({domain:g,range:v},{noNumericCheck:!0})}},function(e,t,n){"use strict";e.exports=function(e,t){for(var n,r=e._fullData.length,i=0;i<r;i++){var o=e._fullData[i];if(o.index!==t.index&&("carpet"===o.type&&(n||(n=o),o.carpet===t.carpet)))return o}return n}},function(e,t,n){"use strict";var r=n(399);function i(e,t){var n=(t[0][0]>e?0:1)+(t[0][1]>e?0:2)+(t[1][1]>e?0:4)+(t[1][0]>e?0:8);return 5===n||10===n?e>(t[0][0]+t[0][1]+t[1][0]+t[1][1])/4?5===n?713:1114:5===n?104:208:15===n?0:n}e.exports=function(e){var t,n,o,a,s,l,u,c,d,h=e[0].z,f=h.length,p=h[0].length,g=2===f||2===p;for(n=0;n<f-1;n++)for(a=[],0===n&&(a=a.concat(r.BOTTOMSTART)),n===f-2&&(a=a.concat(r.TOPSTART)),t=0;t<p-1;t++)for(o=a.slice(),0===t&&(o=o.concat(r.LEFTSTART)),t===p-2&&(o=o.concat(r.RIGHTSTART)),s=t+","+n,l=[[h[n][t],h[n][t+1]],[h[n+1][t],h[n+1][t+1]]],d=0;d<e.length;d++)(u=i((c=e[d]).level,l))&&(c.crossings[s]=u,-1!==o.indexOf(u)&&(c.starts.push([t,n]),g&&-1!==o.indexOf(u,o.indexOf(u)+1)&&c.starts.push([t,n])))}},function(e,t,n){"use strict";var r=n(10),i=n(399);function o(e,t,n,r){return Math.abs(e[0]-t[0])<n&&Math.abs(e[1]-t[1])<r}function a(e,t,n,a,l){var u,c=t.join(","),d=e.crossings[c],h=function(e,t,n){var r=0,o=0;e>20&&t?208===e||1114===e?r=0===n[0]?1:-1:o=0===n[1]?1:-1:-1!==i.BOTTOMSTART.indexOf(e)?o=1:-1!==i.LEFTSTART.indexOf(e)?r=1:-1!==i.TOPSTART.indexOf(e)?o=-1:r=-1;return[r,o]}(d,n,t),f=[s(e,t,[-h[0],-h[1]])],p=e.z.length,g=e.z[0].length,v=t.slice(),m=h.slice();for(u=0;u<1e4;u++){if(d>20?(d=i.CHOOSESADDLE[d][(h[0]||h[1])<0?0:1],e.crossings[c]=i.SADDLEREMAINDER[d]):delete e.crossings[c],!(h=i.NEWDELTA[d])){r.log("Found bad marching index:",d,t,e.level);break}f.push(s(e,t,h)),t[0]+=h[0],t[1]+=h[1],c=t.join(","),o(f[f.length-1],f[f.length-2],a,l)&&f.pop();var y=h[0]&&(t[0]<0||t[0]>g-2)||h[1]&&(t[1]<0||t[1]>p-2);if(t[0]===v[0]&&t[1]===v[1]&&h[0]===m[0]&&h[1]===m[1]||n&&y)break;d=e.crossings[c]}1e4===u&&r.log("Infinite loop in contour?");var b,C,A,_,I,x,w,M,S,T,E,O,k,L,D,P=o(f[0],f[f.length-1],a,l),N=0,j=.2*e.smoothing,R=[],z=0;for(u=1;u<f.length;u++)O=f[u],k=f[u-1],L=void 0,D=void 0,L=O[2]-k[2],D=O[3]-k[3],N+=w=Math.sqrt(L*L+D*D),R.push(w);var F=N/R.length*j;function B(e){return f[e%f.length]}for(u=f.length-2;u>=z;u--)if((b=R[u])<F){for(A=0,C=u-1;C>=z&&b+R[C]<F;C--)b+=R[C];if(P&&u===f.length-2)for(A=0;A<C&&b+R[A]<F;A++)b+=R[A];I=u-C+A+1,x=Math.floor((u+C+A+2)/2),_=P||u!==f.length-2?P||-1!==C?I%2?B(x):[(B(x)[0]+B(x+1)[0])/2,(B(x)[1]+B(x+1)[1])/2]:f[0]:f[f.length-1],f.splice(C+1,u-C+1,_),u=C+1,A&&(z=A),P&&(u===f.length-2?f[A]=f[f.length-1]:0===u&&(f[f.length-1]=f[0]))}for(f.splice(0,z),u=0;u<f.length;u++)f[u].length=2;if(!(f.length<2))if(P)f.pop(),e.paths.push(f);else{n||r.log("Unclosed interior contour?",e.level,v.join(","),f.join("L"));var H=!1;for(M=0;M<e.edgepaths.length;M++)if(T=e.edgepaths[M],!H&&o(T[0],f[f.length-1],a,l)){f.pop(),H=!0;var U=!1;for(S=0;S<e.edgepaths.length;S++)if(o((E=e.edgepaths[S])[E.length-1],f[0],a,l)){U=!0,f.shift(),e.edgepaths.splice(M,1),S===M?e.paths.push(f.concat(E)):(S>M&&S--,e.edgepaths[S]=E.concat(f,T));break}U||(e.edgepaths[M]=f.concat(T))}for(M=0;M<e.edgepaths.length&&!H;M++)o((T=e.edgepaths[M])[T.length-1],f[0],a,l)&&(f.shift(),e.edgepaths[M]=T.concat(f),H=!0);H||e.edgepaths.push(f)}}function s(e,t,n){var r=t[0]+Math.max(n[0],0),i=t[1]+Math.max(n[1],0),o=e.z[i][r],a=e.xaxis,s=e.yaxis;if(n[1]){var l=(e.level-o)/(e.z[i][r+1]-o);return[a.c2p((1-l)*e.x[r]+l*e.x[r+1],!0),s.c2p(e.y[i],!0),r+l,i]}var u=(e.level-o)/(e.z[i+1][r]-o);return[a.c2p(e.x[r],!0),s.c2p((1-u)*e.y[i]+u*e.y[i+1],!0),r,i+u]}e.exports=function(e,t,n){var i,o,s,l;for(t=t||.01,n=n||.01,o=0;o<e.length;o++){for(s=e[o],l=0;l<s.starts.length;l++)a(s,s.starts[l],"edge",t,n);for(i=0;Object.keys(s.crossings).length&&i<1e4;)i++,a(s,Object.keys(s.crossings)[0].split(",").map(Number),void 0,t,n);1e4===i&&r.log("Infinite loop in contour?")}}},function(e,t,n){"use strict";var r=n(10),i=n(2026),o=n(581);e.exports=function(e,t,n){for(var a="constraint"===e.type?i[e._operation](e.value):e,s=a.size,l=[],u=o(a),c=n.trace._carpetTrace,d=c?{xaxis:c.aaxis,yaxis:c.baxis,x:n.a,y:n.b}:{xaxis:t.xaxis,yaxis:t.yaxis,x:n.x,y:n.y},h=a.start;h<u;h+=s)if(l.push(r.extendFlat({level:h,crossings:{},starts:[],edgepaths:[],paths:[],z:n.z,smoothing:n.trace.line.smoothing},d)),l.length>1e3){r.warn("Too many contours, clipping at 1000",e);break}return l}},function(e,t,n){"use strict";var r=n(10);function i(e){return r.extendFlat({},e,{edgepaths:r.extendDeep([],e.edgepaths),paths:r.extendDeep([],e.paths),starts:r.extendDeep([],e.starts)})}e.exports=function(e,t){var n,o,a,s=function(e){return e.reverse()},l=function(e){return e};switch(t){case"=":case"<":return e;case">":for(1!==e.length&&r.warn("Contour data invalid for the specified inequality operation."),o=e[0],n=0;n<o.edgepaths.length;n++)o.edgepaths[n]=s(o.edgepaths[n]);for(n=0;n<o.paths.length;n++)o.paths[n]=s(o.paths[n]);for(n=0;n<o.starts.length;n++)o.starts[n]=s(o.starts[n]);return e;case"][":var u=s;s=l,l=u;case"[]":for(2!==e.length&&r.warn("Contour data invalid for the specified inequality range operation."),o=i(e[0]),a=i(e[1]),n=0;n<o.edgepaths.length;n++)o.edgepaths[n]=s(o.edgepaths[n]);for(n=0;n<o.paths.length;n++)o.paths[n]=s(o.paths[n]);for(n=0;n<o.starts.length;n++)o.starts[n]=s(o.starts[n]);for(;a.edgepaths.length;)o.edgepaths.push(l(a.edgepaths.shift()));for(;a.paths.length;)o.paths.push(l(a.paths.shift()));for(;a.starts.length;)o.starts.push(l(a.starts.shift()));return[o]}}},function(e,t,n){"use strict";e.exports=function(e,t){var n,r=e[0],i=r.z;switch(t.type){case"levels":var o=Math.min(i[0][0],i[0][1]);for(n=0;n<e.length;n++){var a=e[n];a.prefixBoundary=!a.edgepaths.length&&(o>a.level||a.starts.length&&o===a.level)}break;case"constraint":if(r.prefixBoundary=!1,r.edgepaths.length)return;var s=r.x.length,l=r.y.length,u=-1/0,c=1/0;for(n=0;n<l;n++)c=Math.min(c,i[n][0]),c=Math.min(c,i[n][s-1]),u=Math.max(u,i[n][0]),u=Math.max(u,i[n][s-1]);for(n=1;n<s-1;n++)c=Math.min(c,i[0][n]),c=Math.min(c,i[l-1][n]),u=Math.max(u,i[0][n]),u=Math.max(u,i[l-1][n]);var d,h,f=t.value;switch(t._operation){case">":f>u&&(r.prefixBoundary=!0);break;case"<":(f<c||r.starts.length&&f===c)&&(r.prefixBoundary=!0);break;case"[]":d=Math.min(f[0],f[1]),((h=Math.max(f[0],f[1]))<c||d>u||r.starts.length&&h===c)&&(r.prefixBoundary=!0);break;case"][":d=Math.min(f[0],f[1]),h=Math.max(f[0],f[1]),d<c&&h>u&&(r.prefixBoundary=!0)}}}},function(e,t,n){!function(e){"use strict";function t(e){return new RegExp("^(("+e.join(")|(")+"))\\b")}var n,r=t(["and","or","not","is"]),i=["as","assert","break","class","continue","def","del","elif","else","except","finally","for","from","global","if","import","lambda","pass","raise","return","try","while","with","yield","in"],o=["abs","all","any","bin","bool","bytearray","callable","chr","classmethod","compile","complex","delattr","dict","dir","divmod","enumerate","eval","filter","float","format","frozenset","getattr","globals","hasattr","hash","help","hex","id","input","int","isinstance","issubclass","iter","len","list","locals","map","max","memoryview","min","next","object","oct","open","ord","pow","property","range","repr","reversed","round","set","setattr","slice","sorted","staticmethod","str","sum","super","tuple","type","vars","zip","__import__","NotImplemented","Ellipsis","__debug__"];function a(e){return e.scopes[e.scopes.length-1]}e.registerHelper("hintWords","python",i.concat(o)),e.defineMode("python",(function(n,s){for(var l=s.delimiters||s.singleDelimiters||/^[\(\)\[\]\{\}@,:`=;\.\\]/,u=[s.singleOperators,s.doubleOperators,s.doubleDelimiters,s.tripleDelimiters,s.operators||/^([-+*/%\/&|^]=?|[<>=]+|\/\/=?|\*\*=?|!=|[~!@]|\.\.\.)/],c=0;c<u.length;c++)u[c]||u.splice(c--,1);var d=s.hangingIndent||n.indentUnit,h=i,f=o;null!=s.extra_keywords&&(h=h.concat(s.extra_keywords)),null!=s.extra_builtins&&(f=f.concat(s.extra_builtins));var p=!(s.version&&Number(s.version)<3);if(p){var g=s.identifiers||/^[_A-Za-z\u00A1-\uFFFF][_A-Za-z0-9\u00A1-\uFFFF]*/;h=h.concat(["nonlocal","False","True","None","async","await"]),f=f.concat(["ascii","bytes","exec","print"]);var v=new RegExp("^(([rbuf]|(br)|(fr))?('{3}|\"{3}|['\"]))","i")}else g=s.identifiers||/^[_A-Za-z][_A-Za-z0-9]*/,h=h.concat(["exec","print"]),f=f.concat(["apply","basestring","buffer","cmp","coerce","execfile","file","intern","long","raw_input","reduce","reload","unichr","unicode","xrange","False","True","None"]),v=new RegExp("^(([rubf]|(ur)|(br))?('{3}|\"{3}|['\"]))","i");var m=t(h),y=t(f);function b(e,t){var n=e.sol()&&"\\"!=t.lastToken;if(n&&(t.indent=e.indentation()),n&&"py"==a(t).type){var r=a(t).offset;if(e.eatSpace()){var i=e.indentation();return i>r?A(t):i<r&&_(e,t)&&"#"!=e.peek()&&(t.errorToken=!0),null}var o=C(e,t);return r>0&&_(e,t)&&(o+=" error"),o}return C(e,t)}function C(e,t,n){if(e.eatSpace())return null;if(!n&&e.match(/^#.*/))return"comment";if(e.match(/^[0-9\.]/,!1)){var i=!1;if(e.match(/^[\d_]*\.\d+(e[\+\-]?\d+)?/i)&&(i=!0),e.match(/^[\d_]+\.\d*/)&&(i=!0),e.match(/^\.\d+/)&&(i=!0),i)return e.eat(/J/i),"number";var o=!1;if(e.match(/^0x[0-9a-f_]+/i)&&(o=!0),e.match(/^0b[01_]+/i)&&(o=!0),e.match(/^0o[0-7_]+/i)&&(o=!0),e.match(/^[1-9][\d_]*(e[\+\-]?[\d_]+)?/)&&(e.eat(/J/i),o=!0),e.match(/^0(?![\dx])/i)&&(o=!0),o)return e.eat(/L/i),"number"}if(e.match(v))return-1!==e.current().toLowerCase().indexOf("f")?(t.tokenize=function(e,t){for(;"rubf".indexOf(e.charAt(0).toLowerCase())>=0;)e=e.substr(1);var n=1==e.length;function r(e){return function(t,n){var o=C(t,n,!0);return"punctuation"==o&&("{"==t.current()?n.tokenize=r(e+1):"}"==t.current()&&(n.tokenize=e>1?r(e-1):i)),o}}function i(i,o){for(;!i.eol();)if(i.eatWhile(/[^'"\{\}\\]/),i.eat("\\")){if(i.next(),n&&i.eol())return"string"}else{if(i.match(e))return o.tokenize=t,"string";if(i.match("{{"))return"string";if(i.match("{",!1))return o.tokenize=r(0),i.current()?"string":o.tokenize(i,o);if(i.match("}}"))return"string";if(i.match("}"))return"error";i.eat(/['"]/)}if(n){if(s.singleLineStringErrors)return"error";o.tokenize=t}return"string"}return i.isString=!0,i}(e.current(),t.tokenize),t.tokenize(e,t)):(t.tokenize=function(e,t){for(;"rubf".indexOf(e.charAt(0).toLowerCase())>=0;)e=e.substr(1);var n=1==e.length;function r(r,i){for(;!r.eol();)if(r.eatWhile(/[^'"\\]/),r.eat("\\")){if(r.next(),n&&r.eol())return"string"}else{if(r.match(e))return i.tokenize=t,"string";r.eat(/['"]/)}if(n){if(s.singleLineStringErrors)return"error";i.tokenize=t}return"string"}return r.isString=!0,r}(e.current(),t.tokenize),t.tokenize(e,t));for(var a=0;a<u.length;a++)if(e.match(u[a]))return"operator";return e.match(l)?"punctuation":"."==t.lastToken&&e.match(g)?"property":e.match(m)||e.match(r)?"keyword":e.match(y)?"builtin":e.match(/^(self|cls)\b/)?"variable-2":e.match(g)?"def"==t.lastToken||"class"==t.lastToken?"def":"variable":(e.next(),n?null:"error")}function A(e){for(;"py"!=a(e).type;)e.scopes.pop();e.scopes.push({offset:a(e).offset+n.indentUnit,type:"py",align:null})}function _(e,t){for(var n=e.indentation();t.scopes.length>1&&a(t).offset>n;){if("py"!=a(t).type)return!0;t.scopes.pop()}return a(t).offset!=n}function I(e,t){e.sol()&&(t.beginningOfLine=!0);var n=t.tokenize(e,t),r=e.current();if(t.beginningOfLine&&"@"==r)return e.match(g,!1)?"meta":p?"operator":"error";if(/\S/.test(r)&&(t.beginningOfLine=!1),"variable"!=n&&"builtin"!=n||"meta"!=t.lastToken||(n="meta"),"pass"!=r&&"return"!=r||(t.dedent+=1),"lambda"==r&&(t.lambda=!0),":"!=r||t.lambda||"py"!=a(t).type||A(t),1==r.length&&!/string|comment/.test(n)){var i="[({".indexOf(r);if(-1!=i&&function(e,t,n){var r=e.match(/^([\s\[\{\(]|#.*)*$/,!1)?null:e.column()+1;t.scopes.push({offset:t.indent+d,type:n,align:r})}(e,t,"])}".slice(i,i+1)),-1!=(i="])}".indexOf(r))){if(a(t).type!=r)return"error";t.indent=t.scopes.pop().offset-d}}return t.dedent>0&&e.eol()&&"py"==a(t).type&&(t.scopes.length>1&&t.scopes.pop(),t.dedent-=1),n}return{startState:function(e){return{tokenize:b,scopes:[{offset:e||0,type:"py",align:null}],indent:e||0,lastToken:null,lambda:!1,dedent:0}},token:function(e,t){var n=t.errorToken;n&&(t.errorToken=!1);var r=I(e,t);return r&&"comment"!=r&&(t.lastToken="keyword"==r||"punctuation"==r?e.current():r),"punctuation"==r&&(r=null),e.eol()&&t.lambda&&(t.lambda=!1),n?r+" error":r},indent:function(t,n){if(t.tokenize!=b)return t.tokenize.isString?e.Pass:0;var r=a(t),i=r.type==n.charAt(0);return null!=r.align?r.align-(i?1:0):r.offset-(i?d:0)},electricInput:/^\s*[\}\]\)]$/,closeBrackets:{triples:"'\""},lineComment:"#",fold:"indent"}})),e.defineMIME("text/x-python","python"),e.defineMIME("text/x-cython",{name:"python",extra_keywords:(n="by cdef cimport cpdef ctypedef enum except extern gil include nogil property public readonly struct union DEF IF ELIF ELSE",n.split(" "))})}(n(122))},function(e,t,n){"use strict";e.exports=function(e,t){return"string"!=typeof e?e:(/^['"].*['"]$/.test(e)&&(e=e.slice(1,-1)),/["'() \t\n]/.test(e)||t?'"'+e.replace(/"/g,'\\"').replace(/\n/g,"\\n")+'"':e)}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.FactoryBank=void 0;const r=n(306),i=n(67);class o extends r.BaseObserver{constructor(){super(),this.factories={}}getFactories(){return i.values(this.factories)}clearFactories(){for(let e in this.factories)this.deregisterFactory(e)}getFactory(e){if(!this.factories[e])throw new Error(`Cannot find factory with type [${e}]`);return this.factories[e]}registerFactory(e){e.setFactoryBank(this),this.factories[e.getType()]=e,this.fireEvent({factory:e},"factoryAdded")}deregisterFactory(e){const t=this.factories[e];t.setFactoryBank(null),delete this.factories[e],this.fireEvent({factory:t},"factoryRemoved")}}t.FactoryBank=o},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.Matrix=void 0;t.Matrix=class{constructor(e){this.matrix=e}mmul(e){return this.matrix=this.matrix.map((t,n)=>e.asArray()[0].map((r,i)=>t.reduce((t,r,o)=>t+this.matrix[n][o]*e.asArray()[o][i],0))),this}asArray(){return this.matrix}get(e,t){return this.asArray()[e][t]}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.Rectangle=void 0;const r=n(402),i=n(591);class o extends i.Polygon{constructor(e=0,t=0,n=0,i=0){e instanceof r.Point&&t instanceof r.Point&&n instanceof r.Point&&i instanceof r.Point?super([e,t,n,i]):e instanceof r.Point?super([e,new r.Point(e.x+t,e.y),new r.Point(e.x+t,e.y+n),new r.Point(e.x,e.y+n)]):super(o.pointsFromBounds(e,t,n,i))}static pointsFromBounds(e,t,n,i){return[new r.Point(e,t),new r.Point(e+n,t),new r.Point(e+n,t+i),new r.Point(e,t+i)]}updateDimensions(e,t,n,r){this.points=o.pointsFromBounds(e,t,n,r)}setPoints(e){if(4!==e.length)throw"Rectangles must always have 4 points";super.setPoints(e)}containsPoint(e){const t=this.getTopLeft(),n=this.getBottomRight();return e.x>=t.x&&e.x<=n.x&&e.y>=t.y&&e.y<=n.y}getWidth(){return Math.sqrt(Math.pow(this.getTopLeft().x-this.getTopRight().x,2)+Math.pow(this.getTopLeft().y-this.getTopRight().y,2))}getHeight(){return Math.sqrt(Math.pow(this.getBottomLeft().x-this.getTopLeft().x,2)+Math.pow(this.getBottomLeft().y-this.getTopLeft().y,2))}getTopMiddle(){return r.Point.middlePoint(this.getTopLeft(),this.getTopRight())}getBottomMiddle(){return r.Point.middlePoint(this.getBottomLeft(),this.getBottomRight())}getLeftMiddle(){return r.Point.middlePoint(this.getBottomLeft(),this.getTopLeft())}getRightMiddle(){return r.Point.middlePoint(this.getBottomRight(),this.getTopRight())}getTopLeft(){return this.points[0]}getTopRight(){return this.points[1]}getBottomRight(){return this.points[2]}getBottomLeft(){return this.points[3]}}t.Rectangle=o},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.ActionEventBus=void 0;const r=n(182),i=n(67);t.ActionEventBus=class{constructor(e){this.actions={},this.engine=e,this.keys={}}getKeys(){return i.keys(this.keys)}registerAction(e){return e.setEngine(this.engine),this.actions[e.id]=e,()=>{this.deregisterAction(e)}}deregisterAction(e){e.setEngine(null),delete this.actions[e.id]}getActionsForType(e){return i.filter(this.actions,t=>t.options.type===e)}getModelForEvent(e){return e.model?e.model:this.engine.getMouseElement(e.event)}getActionsForEvent(e){const{event:t}=e;return"mousedown"===t.type?this.getActionsForType(r.InputType.MOUSE_DOWN):"mouseup"===t.type?this.getActionsForType(r.InputType.MOUSE_UP):"keydown"===t.type?(this.keys[t.key.toLowerCase()]=!0,this.getActionsForType(r.InputType.KEY_DOWN)):"keyup"===t.type?(delete this.keys[t.key.toLowerCase()],this.getActionsForType(r.InputType.KEY_UP)):"mousemove"===t.type?this.getActionsForType(r.InputType.MOUSE_MOVE):"wheel"===t.type?this.getActionsForType(r.InputType.MOUSE_WHEEL):[]}fireAction(e){const t=this.getActionsForEvent(e);for(let n of t)n.options.fire(e)}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.ZoomCanvasAction=void 0;const r=n(182);class i extends r.Action{constructor(e={}){super({type:r.InputType.MOUSE_WHEEL,fire:t=>{const{event:n}=t;for(let e of this.engine.getModel().getLayers())e.allowRepaint(!1);const r=this.engine.getModel();n.stopPropagation();const i=this.engine.getModel().getZoomLevel()/100;let o=e.inverseZoom?-n.deltaY:n.deltaY;n.ctrlKey&&o%1!=0?o/=3:o/=60,r.getZoomLevel()+o>10&&r.setZoomLevel(r.getZoomLevel()+o);const a=r.getZoomLevel()/100,s=n.currentTarget.getBoundingClientRect(),l=s.width,u=s.height,c=l*a-l*i,d=u*a-u*i,h=n.clientX-s.left,f=n.clientY-s.top,p=(h-r.getOffsetX())/i/l,g=(f-r.getOffsetY())/i/u;r.setOffset(r.getOffsetX()-c*p,r.getOffsetY()-d*g),this.engine.repaintCanvas();for(let e of this.engine.getModel().getLayers())e.allowRepaint(!0)}})}}t.ZoomCanvasAction=i},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.DeleteItemsAction=void 0;const r=n(182),i=n(67);class o extends r.Action{constructor(e={}){const t=e.keyCodes||[46,8],n=Object.assign({ctrlKey:!1,shiftKey:!1,altKey:!1,metaKey:!1},e.modifiers);super({type:r.InputType.KEY_DOWN,fire:e=>{const{keyCode:r,ctrlKey:o,shiftKey:a,altKey:s,metaKey:l}=e.event;-1!==t.indexOf(r)&&i.isEqual({ctrlKey:o,shiftKey:a,altKey:s,metaKey:l},n)&&(i.forEach(this.engine.getModel().getSelectedEntities(),e=>{e.isLocked()||e.remove()}),this.engine.repaintCanvas())}})}}t.DeleteItemsAction=o},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.StateMachine=void 0;const r=n(67),i=n(306);class o extends i.BaseObserver{constructor(e){super(),this.engine=e,this.stateStack=[]}getCurrentState(){return this.currentState}pushState(e){this.stateStack.push(e),this.setState(e)}popState(){this.stateStack.pop(),this.setState(r.last(this.stateStack))}setState(e){e.setEngine(this.engine),this.currentState&&this.currentState.deactivated(e);const t=this.currentState;this.currentState=e,this.currentState&&(this.currentState.activated(t),this.fireEvent({newState:e},"stateChanged"))}}t.StateMachine=o},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.AbstractFactory=void 0;t.AbstractFactory=class{constructor(e){this.type=e}setDiagramEngine(e){this.engine=e}setFactoryBank(e){this.bank=e}getType(){return this.type}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.AbstractModelFactory=void 0;const r=n(1104);class i extends r.AbstractFactory{}t.AbstractModelFactory=i},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.AbstractReactFactory=void 0;const r=n(1105);class i extends r.AbstractModelFactory{}t.AbstractReactFactory=i},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.BasePositionModel=void 0;const r=n(594),i=n(151);class o extends r.BaseModel{constructor(e){super(e),this.position=e.position||new i.Point(0,0)}setPosition(e,t){this.position="object"==typeof e?e:new i.Point(e,t),this.fireEvent({},"positionChanged")}getBoundingBox(){return new i.Rectangle(this.position,0,0)}deserialize(e){super.deserialize(e),this.position=new i.Point(e.data.x,e.data.y)}serialize(){return Object.assign(Object.assign({},super.serialize()),{x:this.position.x,y:this.position.y})}getPosition(){return this.position}getX(){return this.position.x}getY(){return this.position.y}}t.BasePositionModel=o},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.TransformLayerWidget=void 0;const r=n(0),i=n(137),o=n(333);var a;!function(e){const t=o.css`
		top: 0;
		left: 0;
		right: 0;
		bottom: 0;
		position: absolute;
		pointer-events: none;
		transform-origin: 0 0;
		width: 100%;
		height: 100%;
		overflow: visible;
	`;e.DivLayer=i.default.div`
		${t}
	`,e.SvgLayer=i.default.svg`
		${t}
	`}(a||(a={}));class s extends r.Component{constructor(e){super(e),this.state={}}getTransform(){const e=this.props.layer.getParent();return`\n\t\t\ttranslate(\n\t\t\t\t${e.getOffsetX()}px,\n\t\t\t\t${e.getOffsetY()}px)\n\t\t\tscale(\n\t\t\t\t${e.getZoomLevel()/100}\n\t\t\t)\n  \t`}getTransformStyle(){return this.props.layer.getOptions().transformed?{transform:this.getTransform()}:{}}render(){return this.props.layer.getOptions().isSvg?r.createElement(a.SvgLayer,{style:this.getTransformStyle()},this.props.children):r.createElement(a.DivLayer,{style:this.getTransformStyle()},this.props.children)}}t.TransformLayerWidget=s},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.SmartLayerWidget=void 0;const r=n(0);class i extends r.Component{shouldComponentUpdate(){return this.props.layer.isRepaintEnabled()}render(){return this.props.engine.getFactoryForLayer(this.props.layer).generateReactWidget({model:this.props.layer})}}t.SmartLayerWidget=i},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.LayerModel=void 0;const r=n(594),i=n(67);class o extends r.BaseModel{constructor(e={}){super(e),this.models={},this.repaintEnabled=!0}deserialize(e){super.deserialize(e),this.options.isSvg=!!e.data.isSvg,this.options.transformed=!!e.data.transformed,i.forEach(e.data.models,t=>{const n=this.getChildModelFactoryBank(e.engine).getFactory(t.type).generateModel({initialConfig:t});n.deserialize(Object.assign(Object.assign({},e),{data:t})),this.addModel(n)})}serialize(){return Object.assign(Object.assign({},super.serialize()),{isSvg:this.options.isSvg,transformed:this.options.transformed,models:i.mapValues(this.models,e=>e.serialize())})}isRepaintEnabled(){return this.repaintEnabled}allowRepaint(e=!0){this.repaintEnabled=e}remove(){this.parent&&this.parent.removeLayer(this),super.remove()}addModel(e){e.setParent(this),this.models[e.getID()]=e}getSelectionEntities(){return i.flatMap(this.models,e=>e.getSelectionEntities())}getModels(){return this.models}getModel(e){return this.models[e]}removeModel(e){const t="string"==typeof e?e:e.getID();return!!this.models[t]&&(delete this.models[t],!0)}}t.LayerModel=o},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.SelectionBoxWidget=void 0;const r=n(0),i=n(137);var o;!function(e){e.Container=i.default.div`
		position: absolute;
		background-color: rgba(0, 192, 255, 0.2);
		border: solid 2px rgb(0, 192, 255);
	`}(o||(o={}));class a extends r.Component{render(){const{rect:e}=this.props;return r.createElement(o.Container,{style:{top:e.top,left:e.left,width:e.width,height:e.height}})}}t.SelectionBoxWidget=a},function(e,t,n){"use strict";var r=this&&this.__awaiter||function(e,t,n,r){return new(n||(n=Promise))((function(i,o){function a(e){try{l(r.next(e))}catch(e){o(e)}}function s(e){try{l(r.throw(e))}catch(e){o(e)}}function l(e){var t;e.done?i(e.value):(t=e.value,t instanceof n?t:new n((function(e){e(t)}))).then(a,s)}l((r=r.apply(e,t||[])).next())}))};Object.defineProperty(t,"__esModule",{value:!0}),t.DragCanvasState=void 0;const i=n(403);class o extends i.AbstractDisplacementState{constructor(e={}){super({name:"drag-canvas"}),this.config=Object.assign({allowDrag:!0},e)}activated(e){const t=Object.create(null,{activated:{get:()=>super.activated}});return r(this,void 0,void 0,(function*(){t.activated.call(this,e),this.engine.getModel().clearSelection(),yield this.engine.repaintCanvas(!0);for(let e of this.engine.getModel().getLayers())e.allowRepaint(!1);this.initialCanvasX=this.engine.getModel().getOffsetX(),this.initialCanvasY=this.engine.getModel().getOffsetY()}))}deactivated(e){super.deactivated(e);for(let e of this.engine.getModel().getLayers())e.allowRepaint(!0)}fireMouseMoved(e){this.config.allowDrag&&(this.engine.getModel().setOffset(this.initialCanvasX+e.displacementX,this.initialCanvasY+e.displacementY),this.engine.repaintCanvas())}}t.DragCanvasState=o},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.SelectingState=void 0;const r=n(404),i=n(182),o=n(1114);class a extends r.State{constructor(){super({name:"selecting"}),this.keys=["shift"],this.registerAction(new i.Action({type:i.InputType.MOUSE_DOWN,fire:e=>{const t=this.engine.getActionEventBus().getModelForEvent(e);t?(t.setSelected(!0),this.engine.repaintCanvas()):this.transitionWithEvent(new o.SelectionBoxState,e)}}))}}t.SelectingState=a},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.SelectionBoxState=void 0;const r=n(403),i=n(595),o=n(151);class a extends r.AbstractDisplacementState{constructor(){super({name:"selection-box"})}activated(e){super.activated(e),this.layer=new i.SelectionLayerModel,this.engine.getModel().addLayer(this.layer)}deactivated(e){super.deactivated(e),this.layer.remove(),this.engine.repaintCanvas()}getBoxDimensions(e){const t=this.engine.getRelativePoint(e.event.clientX,e.event.clientY);return{left:t.x>this.initialXRelative?this.initialXRelative:t.x,top:t.y>this.initialYRelative?this.initialYRelative:t.y,width:Math.abs(t.x-this.initialXRelative),height:Math.abs(t.y-this.initialYRelative),right:t.x<this.initialXRelative?this.initialXRelative:t.x,bottom:t.y<this.initialYRelative?this.initialYRelative:t.y}}fireMouseMoved(e){this.layer.setBox(this.getBoxDimensions(e));const t=this.engine.getRelativeMousePoint({clientX:this.initialX,clientY:this.initialY});e.virtualDisplacementX<0&&(t.x-=Math.abs(e.virtualDisplacementX)),e.virtualDisplacementY<0&&(t.y-=Math.abs(e.virtualDisplacementY));const n=new o.Rectangle(t,Math.abs(e.virtualDisplacementX),Math.abs(e.virtualDisplacementY));for(let e of this.engine.getModel().getSelectionEntities())if(e.getBoundingBox){const t=e.getBoundingBox();n.containsPoint(t.getTopLeft())&&n.containsPoint(t.getBottomRight())?e.setSelected(!0):e.setSelected(!1)}this.engine.repaintCanvas()}}t.SelectionBoxState=a},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.MoveItemsState=void 0;const r=n(403),i=n(182),o=n(1107);class a extends r.AbstractDisplacementState{constructor(){super({name:"move-items"}),this.registerAction(new i.Action({type:i.InputType.MOUSE_DOWN,fire:e=>{const t=this.engine.getActionEventBus().getModelForEvent(e);t&&(t.isSelected()||this.engine.getModel().clearSelection(),t.setSelected(!0),this.engine.repaintCanvas())}}))}activated(e){super.activated(e),this.initialPositions={}}fireMouseMoved(e){const t=this.engine.getModel().getSelectedEntities(),n=this.engine.getModel();for(let r of t)if(r instanceof o.BasePositionModel){if(r.isLocked())continue;this.initialPositions[r.getID()]||(this.initialPositions[r.getID()]={point:r.getPosition(),item:r});const t=this.initialPositions[r.getID()].point;r.setPosition(n.getGridPosition(t.x+e.virtualDisplacementX),n.getGridPosition(t.y+e.virtualDisplacementY))}this.engine.repaintCanvas()}}t.MoveItemsState=a},function(e,t,n){"use strict";var r=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),i=this&&this.__exportStar||function(e,t){for(var n in e)"default"===n||Object.prototype.hasOwnProperty.call(t,n)||r(t,e,n)};Object.defineProperty(t,"__esModule",{value:!0});const o=n(152),a=n(198),s=n(1130),l=n(69);i(n(152),t),i(n(198),t),i(n(1130),t),t.default=(e={})=>{const t=new o.DiagramEngine(e);return t.getLayerFactories().registerFactory(new o.NodeLayerFactory),t.getLayerFactories().registerFactory(new o.LinkLayerFactory),t.getLayerFactories().registerFactory(new l.SelectionBoxLayerFactory),t.getLabelFactories().registerFactory(new a.DefaultLabelFactory),t.getNodeFactories().registerFactory(new a.DefaultNodeFactory),t.getLinkFactories().registerFactory(new a.DefaultLinkFactory),t.getLinkFactories().registerFactory(new s.PathFindingLinkFactory),t.getPortFactories().registerFactory(new a.DefaultPortFactory),t.getStateMachine().pushState(new o.DefaultDiagramState),t}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.LinkWidget=void 0;const r=n(0),i=n(67),o=n(2052),a=n(69);class s extends r.Component{constructor(e){super(e),this.state={sourcePort:null,targetPort:null}}componentWillUnmount(){this.sourceListener&&this.sourceListener.deregister(),this.targetListener&&this.targetListener.deregister()}static getDerivedStateFromProps(e,t){return{sourcePort:e.link.getSourcePort(),targetPort:e.link.getTargetPort()}}installTarget(){this.targetListener&&this.targetListener.deregister(),this.props.link.getTargetPort()&&(this.targetListener=this.props.link.getTargetPort().registerListener({reportInitialPosition:e=>{this.forceUpdate()}}))}installSource(){this.sourceListener&&this.sourceListener.deregister(),this.props.link.getSourcePort()&&(this.sourceListener=this.props.link.getSourcePort().registerListener({reportInitialPosition:e=>{this.forceUpdate()}}))}componentDidUpdate(e,t,n){t.sourcePort!==this.state.sourcePort&&this.installSource(),t.targetPort!==this.state.targetPort&&this.installTarget()}static generateLinePath(e,t){return`M${e.getX()},${e.getY()} L ${t.getX()},${t.getY()}`}componentDidMount(){this.props.link.getSourcePort()&&this.installSource(),this.props.link.getTargetPort()&&this.installTarget()}render(){const{link:e}=this.props;return e.getSourcePort()&&!e.getSourcePort().reportedPosition||e.getTargetPort()&&!e.getTargetPort().reportedPosition?null:r.createElement(a.PeformanceWidget,{model:this.props.link,serialized:this.props.link.serialize()},()=>r.createElement("g",{"data-linkid":this.props.link.getID()},this.props.diagramEngine.generateWidgetForLink(e),i.map(this.props.link.getLabels(),(e,t)=>r.createElement(o.LabelWidget,{key:e.getID(),engine:this.props.diagramEngine,label:e,index:t}))))}}t.LinkWidget=s},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.LinkLayerWidget=void 0;const r=n(0),i=n(137),o=n(67),a=n(1117);var s;!function(e){e.Container=i.default.div``}(s||(s={}));class l extends r.Component{render(){return r.createElement(r.Fragment,null,o.map(this.props.layer.getLinks(),e=>r.createElement(a.LinkWidget,{key:e.getID(),link:e,diagramEngine:this.props.engine})))}}t.LinkLayerWidget=l},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.NodeLayerWidget=void 0;const r=n(0),i=n(137),o=n(67),a=n(1120);var s;!function(e){e.Container=i.default.div``}(s||(s={}));class l extends r.Component{render(){return r.createElement(r.Fragment,null,o.map(this.props.layer.getNodes(),e=>r.createElement(a.NodeWidget,{key:e.getID(),diagramEngine:this.props.engine,node:e})))}}t.NodeLayerWidget=l},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.NodeWidget=void 0;const r=n(0),i=n(67),o=n(69),a=n(137),s=n(327);var l;!function(e){e.Node=a.default.div`
		position: absolute;
		-webkit-touch-callout: none; /* iOS Safari */
		-webkit-user-select: none; /* Chrome/Safari/Opera */
		user-select: none;
		cursor: move;
		pointer-events: all;
	`}(l||(l={}));class u extends r.Component{constructor(e){super(e),this.ref=r.createRef()}componentWillUnmount(){this.ob.disconnect(),this.ob=null,this.listener.deregister(),this.listener=null}componentDidUpdate(e,t,n){this.listener&&this.props.node!==e.node&&(this.listener.deregister(),this.installSelectionListener())}installSelectionListener(){this.listener=this.props.node.registerListener({selectionChanged:e=>{this.forceUpdate()}})}componentDidMount(){this.ob=new s.default(e=>{const t=e[0].contentRect;this.props.node.updateDimensions({width:t.width,height:t.height}),i.forEach(this.props.node.getPorts(),e=>{e.updateCoords(this.props.diagramEngine.getPortCoords(e))})}),this.ob.observe(this.ref.current),this.installSelectionListener()}render(){return r.createElement(o.PeformanceWidget,{model:this.props.node,serialized:this.props.node.serialize()},()=>r.createElement(l.Node,{className:"node",ref:this.ref,"data-nodeid":this.props.node.getID(),style:{top:this.props.node.getY(),left:this.props.node.getX()}},this.props.diagramEngine.generateWidgetForNode(this.props.node)))}}t.NodeWidget=u},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.DragNewLinkState=void 0;const r=n(69),i=n(406);class o extends r.AbstractDisplacementState{constructor(e={}){super({name:"drag-new-link"}),this.config=Object.assign({allowLooseLinks:!0,allowLinksFromLockedPorts:!1},e),this.registerAction(new r.Action({type:r.InputType.MOUSE_DOWN,fire:e=>{this.port=this.engine.getMouseElement(e.event),this.config.allowLinksFromLockedPorts||!this.port.isLocked()?(this.link=this.port.createLinkModel(),this.link?(this.link.setSelected(!0),this.link.setSourcePort(this.port),this.engine.getModel().addLink(this.link),this.port.reportPosition()):this.eject()):this.eject()}})),this.registerAction(new r.Action({type:r.InputType.MOUSE_UP,fire:e=>{const t=this.engine.getMouseElement(e.event);if(t instanceof i.PortModel)return this.port.canLinkToPort(t)?(this.link.setTargetPort(t),t.reportPosition(),void this.engine.repaintCanvas()):(this.link.remove(),void this.engine.repaintCanvas());this.config.allowLooseLinks||(this.link.remove(),this.engine.repaintCanvas())}}))}fireMouseMoved(e){const t=this.port.getPosition(),n=this.engine.getModel().getZoomLevel()/100,r=this.engine.getModel().getOffsetX()/n,i=this.engine.getModel().getOffsetY()/n,o=this.initialXRelative/n,a=this.initialYRelative/n,s=t.x-r+(o-t.x)+e.virtualDisplacementX,l=t.y-i+(a-t.y)+e.virtualDisplacementY;this.link.getLastPoint().setPosition(s,l),this.engine.repaintCanvas()}}t.DragNewLinkState=o},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.DragDiagramItemsState=void 0;const r=n(69),i=n(67),o=n(597),a=n(406);class s extends r.MoveItemsState{constructor(){super(),this.registerAction(new r.Action({type:r.InputType.MOUSE_UP,fire:e=>{const t=this.engine.getMouseElement(e.event);t instanceof a.PortModel&&i.forEach(this.initialPositions,e=>{if(e.item instanceof o.PointModel){const n=e.item.getParent();if(n.getLastPoint()!==e.item)return;n.getSourcePort().canLinkToPort(t)&&(n.setTargetPort(t),t.reportPosition(),this.engine.repaintCanvas())}})}}))}}t.DragDiagramItemsState=s},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.DefaultLabelWidget=void 0;const r=n(0),i=n(137);var o;!function(e){e.Label=i.default.div`
		background: rgba(0, 0, 0, 0.8);
		border-radius: 5px;
		color: white;
		font-size: 12px;
		padding: 4px 8px;
		font-family: sans-serif;
		user-select: none;
	`}(o||(o={}));class a extends r.Component{render(){return r.createElement(o.Label,null,this.props.model.getOptions().label)}}t.DefaultLabelWidget=a},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.DefaultLinkWidget=void 0;const r=n(0),i=n(152),o=n(1125),a=n(1126);class s extends r.Component{constructor(e){super(e),this.refPaths=[],this.state={selected:!1}}componentDidUpdate(){this.props.link.setRenderedPaths(this.refPaths.map(e=>e.current))}componentDidMount(){this.props.link.setRenderedPaths(this.refPaths.map(e=>e.current))}componentWillUnmount(){this.props.link.setRenderedPaths([])}addPointToLink(e,t){if(!e.shiftKey&&!this.props.link.isLocked()&&this.props.link.getPoints().length-1<=this.props.diagramEngine.getMaxNumberPointsPerLink()){const n=new i.PointModel({link:this.props.link,position:this.props.diagramEngine.getRelativeMousePoint(e)});this.props.link.addPoint(n,t),e.persist(),e.stopPropagation(),this.forceUpdate(()=>{this.props.diagramEngine.getActionEventBus().fireAction({event:e,model:n})})}}generatePoint(e){return r.createElement(o.DefaultLinkPointWidget,{key:e.getID(),point:e,colorSelected:this.props.link.getOptions().selectedColor,color:this.props.link.getOptions().color})}generateLink(e,t,n){const i=r.createRef();return this.refPaths.push(i),r.createElement(a.DefaultLinkSegmentWidget,{key:"link-"+n,path:e,selected:this.state.selected,diagramEngine:this.props.diagramEngine,factory:this.props.diagramEngine.getFactoryForLink(this.props.link),link:this.props.link,forwardRef:i,onSelection:e=>{this.setState({selected:e})},extras:t})}render(){var e=this.props.link.getPoints(),t=[];if(this.refPaths=[],2===e.length)t.push(this.generateLink(this.props.link.getSVGPath(),{onMouseDown:e=>{this.addPointToLink(e,1)}},"0")),null==this.props.link.getTargetPort()&&t.push(this.generatePoint(e[1]));else{for(let n=0;n<e.length-1;n++)t.push(this.generateLink(i.LinkWidget.generateLinePath(e[n],e[n+1]),{"data-linkid":this.props.link.getID(),"data-point":n,onMouseDown:e=>{this.addPointToLink(e,n+1)}},n));for(let n=1;n<e.length-1;n++)t.push(this.generatePoint(e[n]));null==this.props.link.getTargetPort()&&t.push(this.generatePoint(e[e.length-1]))}return r.createElement("g",{"data-default-link-test":this.props.link.getOptions().testName},t)}}t.DefaultLinkWidget=s},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.DefaultLinkPointWidget=void 0;const r=n(0),i=n(137);var o;!function(e){e.PointTop=i.default.circle`
		pointer-events: all;
	`}(o||(o={}));class a extends r.Component{constructor(e){super(e),this.state={selected:!1}}render(){const{point:e}=this.props;return r.createElement("g",null,r.createElement("circle",{cx:e.getPosition().x,cy:e.getPosition().y,r:5,fill:this.state.selected||this.props.point.isSelected()?this.props.colorSelected:this.props.color}),r.createElement(o.PointTop,{className:"point",onMouseLeave:()=>{this.setState({selected:!1})},onMouseEnter:()=>{this.setState({selected:!0})},"data-id":e.getID(),"data-linkid":e.getLink().getID(),cx:e.getPosition().x,cy:e.getPosition().y,r:15,opacity:0}))}}t.DefaultLinkPointWidget=a},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.DefaultLinkSegmentWidget=void 0;const r=n(0);class i extends r.Component{render(){const e=r.cloneElement(this.props.factory.generateLinkSegment(this.props.link,this.props.selected||this.props.link.isSelected(),this.props.path),{ref:this.props.forwardRef}),t=r.cloneElement(e,Object.assign(Object.assign({strokeLinecap:"round",onMouseLeave:()=>{this.props.onSelection(!1)},onMouseEnter:()=>{this.props.onSelection(!0)}},this.props.extras),{ref:null,"data-linkid":this.props.link.getID(),strokeOpacity:this.props.selected?.1:0,strokeWidth:20,fill:"none",onContextMenu:()=>{this.props.link.isLocked()||(event.preventDefault(),this.props.link.remove())}}));return r.createElement("g",null,e,t)}}t.DefaultLinkSegmentWidget=i},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.DefaultNodeModel=void 0;const r=n(67),i=n(152),o=n(602);class a extends i.NodeModel{constructor(e={},t){"string"==typeof e&&(e={name:e,color:t}),super(Object.assign({type:"default",name:"Untitled",color:"rgb(0,192,255)"},e)),this.portsOut=[],this.portsIn=[]}doClone(e,t){t.portsIn=[],t.portsOut=[],super.doClone(e,t)}removePort(e){super.removePort(e),e.getOptions().in?this.portsIn.splice(this.portsIn.indexOf(e),1):this.portsOut.splice(this.portsOut.indexOf(e),1)}addPort(e){return super.addPort(e),e.getOptions().in?-1===this.portsIn.indexOf(e)&&this.portsIn.push(e):-1===this.portsOut.indexOf(e)&&this.portsOut.push(e),e}addInPort(e,t=!0){const n=new o.DefaultPortModel({in:!0,name:e,label:e,alignment:i.PortModelAlignment.LEFT});return t||this.portsIn.splice(0,0,n),this.addPort(n)}addOutPort(e,t=!0){const n=new o.DefaultPortModel({in:!1,name:e,label:e,alignment:i.PortModelAlignment.RIGHT});return t||this.portsOut.splice(0,0,n),this.addPort(n)}deserialize(e){super.deserialize(e),this.options.name=e.data.name,this.options.color=e.data.color,this.portsIn=r.map(e.data.portsInOrder,e=>this.getPortFromID(e)),this.portsOut=r.map(e.data.portsOutOrder,e=>this.getPortFromID(e))}serialize(){return Object.assign(Object.assign({},super.serialize()),{name:this.options.name,color:this.options.color,portsInOrder:r.map(this.portsIn,e=>e.getID()),portsOutOrder:r.map(this.portsOut,e=>e.getID())})}getInPorts(){return this.portsIn}getOutPorts(){return this.portsOut}}t.DefaultNodeModel=a},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.DefaultNodeWidget=void 0;const r=n(0),i=n(67),o=n(1129),a=n(137);var s;!function(e){e.Node=a.default.div`
		background-color: ${e=>e.background};
		border-radius: 5px;
		font-family: sans-serif;
		color: white;
		border: solid 2px black;
		overflow: visible;
		font-size: 11px;
		border: solid 2px ${e=>e.selected?"rgb(0,192,255)":"black"};
	`,e.Title=a.default.div`
		background: rgba(0, 0, 0, 0.3);
		display: flex;
		white-space: nowrap;
		justify-items: center;
	`,e.TitleName=a.default.div`
		flex-grow: 1;
		padding: 5px 5px;
	`,e.Ports=a.default.div`
		display: flex;
		background-image: linear-gradient(rgba(0, 0, 0, 0.1), rgba(0, 0, 0, 0.2));
	`,e.PortsContainer=a.default.div`
		flex-grow: 1;
		display: flex;
		flex-direction: column;

		&:first-of-type {
			margin-right: 10px;
		}

		&:only-child {
			margin-right: 0px;
		}
	`}(s||(s={}));class l extends r.Component{constructor(){super(...arguments),this.generatePort=e=>r.createElement(o.DefaultPortLabel,{engine:this.props.engine,port:e,key:e.getID()})}render(){return r.createElement(s.Node,{"data-default-node-name":this.props.node.getOptions().name,selected:this.props.node.isSelected(),background:this.props.node.getOptions().color},r.createElement(s.Title,null,r.createElement(s.TitleName,null,this.props.node.getOptions().name)),r.createElement(s.Ports,null,r.createElement(s.PortsContainer,null,i.map(this.props.node.getInPorts(),this.generatePort)),r.createElement(s.PortsContainer,null,i.map(this.props.node.getOutPorts(),this.generatePort))))}}t.DefaultNodeWidget=l},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.DefaultPortLabel=void 0;const r=n(0),i=n(152),o=n(137);var a;!function(e){e.PortLabel=o.default.div`
		display: flex;
		margin-top: 1px;
		align-items: center;
	`,e.Label=o.default.div`
		padding: 0 5px;
		flex-grow: 1;
	`,e.Port=o.default.div`
		width: 15px;
		height: 15px;
		background: rgba(255, 255, 255, 0.1);

		&:hover {
			background: rgb(192, 255, 0);
		}
	`}(a||(a={}));class s extends r.Component{render(){const e=r.createElement(i.PortWidget,{engine:this.props.engine,port:this.props.port},r.createElement(a.Port,null)),t=r.createElement(a.Label,null,this.props.port.getOptions().label);return r.createElement(a.PortLabel,null,this.props.port.getOptions().in?e:t,this.props.port.getOptions().in?t:e)}}t.DefaultPortLabel=s},function(e,t,n){"use strict";var r=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),i=this&&this.__exportStar||function(e,t){for(var n in e)"default"===n||Object.prototype.hasOwnProperty.call(t,n)||r(t,e,n)};Object.defineProperty(t,"__esModule",{value:!0}),i(n(1131),t),i(n(1132),t),i(n(1133),t),i(n(1135),t),i(n(1136),t),i(n(1137),t),i(n(1134),t),i(n(2080),t)},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.PathFindingLinkFactory=void 0;const r=n(0),i=n(1132),o=n(1133),a=n(67),s=n(2078),l=n(198),u=n(69);class c extends l.DefaultLinkFactory{constructor(){super(c.NAME),this.ROUTING_SCALING_FACTOR=5,this.canvasMatrix=[],this.routingMatrix=[],this.hAdjustmentFactor=0,this.vAdjustmentFactor=0,this.calculateMatrixDimensions=()=>{const e=a.values(this.engine.getModel().getNodes()).map(e=>({x:e.getX(),width:e.width,y:e.getY(),height:e.height})),t=a.values(this.engine.getModel().getLinks()),n=a.flatMap(t.map(e=>[e.getSourcePort(),e.getTargetPort()])).filter(e=>null!==e).map(e=>({x:e.getX(),width:e.width,y:e.getY(),height:e.height})),r=a.flatMap(t.map(e=>e.getPoints())).map(e=>({x:e.getX(),width:0,y:e.getY(),height:0})),i=(e,t)=>a.reduce(t,(t,n)=>t+a.get(e,n,0),0),o=this.engine.getCanvas(),s=a.concat(e,n,r),l=Math.floor(Math.min(a.get(a.minBy(s,"x"),"x",0),0)/this.ROUTING_SCALING_FACTOR)*this.ROUTING_SCALING_FACTOR,u=a.maxBy(s,e=>i(e,["x","width"])),c=Math.max(i(u,["x","width"]),o.offsetWidth),d=a.minBy(s,"y"),h=Math.floor(Math.min(a.get(d,"y",0),0)/this.ROUTING_SCALING_FACTOR)*this.ROUTING_SCALING_FACTOR,f=a.maxBy(s,e=>i(e,["y","height"])),p=Math.max(i(f,["y","height"]),o.offsetHeight);return{width:Math.ceil(Math.abs(l)+c),hAdjustmentFactor:Math.abs(l)/this.ROUTING_SCALING_FACTOR+1,height:Math.ceil(Math.abs(h)+p),vAdjustmentFactor:Math.abs(h)/this.ROUTING_SCALING_FACTOR+1}},this.markNodes=e=>{a.values(this.engine.getModel().getNodes()).forEach(t=>{const n=Math.floor(t.getX()/this.ROUTING_SCALING_FACTOR),r=Math.ceil((t.getX()+t.width)/this.ROUTING_SCALING_FACTOR),i=Math.floor(t.getY()/this.ROUTING_SCALING_FACTOR),o=Math.ceil((t.getY()+t.height)/this.ROUTING_SCALING_FACTOR);for(let t=n-1;t<=r+1;t++)for(let n=i-1;n<o+1;n++)this.markMatrixPoint(e,this.translateRoutingX(t),this.translateRoutingY(n))})},this.markPorts=e=>{a.flatMap(a.values(this.engine.getModel().getLinks()).map(e=>[].concat(e.getSourcePort(),e.getTargetPort()))).filter(e=>null!==e).forEach(t=>{const n=Math.floor(t.x/this.ROUTING_SCALING_FACTOR),r=Math.ceil((t.x+t.width)/this.ROUTING_SCALING_FACTOR),i=Math.floor(t.y/this.ROUTING_SCALING_FACTOR),o=Math.ceil((t.y+t.height)/this.ROUTING_SCALING_FACTOR);for(let t=n-1;t<=r+1;t++)for(let n=i-1;n<o+1;n++)this.markMatrixPoint(e,this.translateRoutingX(t),this.translateRoutingY(n))})},this.markMatrixPoint=(e,t,n)=>{void 0!==e[n]&&void 0!==e[n][t]&&(e[n][t]=1)}}setDiagramEngine(e){super.setDiagramEngine(e),e.getStateMachine().registerListener({stateChanged:t=>{if(t.newState instanceof u.AbstractDisplacementState){const t=e.getActionEventBus().registerAction(new u.Action({type:u.InputType.MOUSE_UP,fire:()=>{this.calculateRoutingMatrix(),e.repaintCanvas(),t()}}))}}}),this.listener=e.registerListener({canvasReady:()=>{a.defer(()=>{this.calculateRoutingMatrix(),e.repaintCanvas()})}})}setFactoryBank(e){super.setFactoryBank(e),!e&&this.listener&&this.listener.deregister()}generateReactWidget(e){return r.createElement(o.PathFindingLinkWidget,{diagramEngine:this.engine,link:e.model,factory:this})}generateModel(e){return new i.PathFindingLinkModel}getCanvasMatrix(){return 0===this.canvasMatrix.length&&this.calculateCanvasMatrix(),this.canvasMatrix}calculateCanvasMatrix(){const{width:e,hAdjustmentFactor:t,height:n,vAdjustmentFactor:r}=this.calculateMatrixDimensions();this.hAdjustmentFactor=t,this.vAdjustmentFactor=r;const i=Math.ceil(e/this.ROUTING_SCALING_FACTOR),o=Math.ceil(n/this.ROUTING_SCALING_FACTOR);this.canvasMatrix=a.range(0,o).map(()=>new Array(i).fill(0))}getRoutingMatrix(){return 0===this.routingMatrix.length&&this.calculateRoutingMatrix(),this.routingMatrix}calculateRoutingMatrix(){const e=a.cloneDeep(this.getCanvasMatrix());this.markNodes(e),this.markPorts(e),this.routingMatrix=e}translateRoutingX(e,t=!1){return e+this.hAdjustmentFactor*(t?-1:1)}translateRoutingY(e,t=!1){return e+this.vAdjustmentFactor*(t?-1:1)}generateDynamicPath(e){let t=s();return t=t.moveto(e[0][0]*this.ROUTING_SCALING_FACTOR,e[0][1]*this.ROUTING_SCALING_FACTOR),e.slice(1).forEach(e=>{t=t.lineto(e[0]*this.ROUTING_SCALING_FACTOR,e[1]*this.ROUTING_SCALING_FACTOR)}),t.print()}}t.PathFindingLinkFactory=c,c.NAME="pathfinding"},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.PathFindingLinkModel=void 0;const r=n(1131),i=n(198);class o extends i.DefaultLinkModel{constructor(e={}){super(Object.assign({type:r.PathFindingLinkFactory.NAME},e))}performanceTune(){return!1}}t.PathFindingLinkModel=o},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.PathFindingLinkWidget=void 0;const r=n(0),i=n(67),o=n(1134),a=n(198);class s extends r.Component{constructor(e){super(e),this.refPaths=[],this.state={selected:!1},this.pathFinding=new o.default(this.props.factory)}componentDidUpdate(){this.props.link.setRenderedPaths(this.refPaths.map(e=>e.current))}componentDidMount(){this.props.link.setRenderedPaths(this.refPaths.map(e=>e.current))}componentWillUnmount(){this.props.link.setRenderedPaths([])}generateLink(e,t){const n=r.createRef();return this.refPaths.push(n),r.createElement(a.DefaultLinkSegmentWidget,{key:"link-"+t,path:e,selected:this.state.selected,diagramEngine:this.props.diagramEngine,factory:this.props.diagramEngine.getFactoryForLink(this.props.link),link:this.props.link,forwardRef:n,onSelection:e=>{this.setState({selected:e})},extras:{}})}render(){this.refPaths=[];var e=this.props.link.getPoints(),t=[];const n=this.pathFinding.calculateDirectPath(i.first(e),i.last(e)),o=this.props.factory.getRoutingMatrix(),a=this.pathFinding.calculateLinkStartEndCoords(o,n);if(a){const{start:e,end:n,pathToStart:r,pathToEnd:i}=a,s=this.pathFinding.calculateDynamicPath(o,e,n,r,i);t.push(this.generateLink(this.props.factory.generateDynamicPath(s),"0"))}return r.createElement(r.Fragment,null,t)}}t.PathFindingLinkWidget=s},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});const r=n(2062),i=new r.JumpPointFinder({heuristic:r.Heuristic.manhattan,diagonalMovement:r.DiagonalMovement.Never});t.default=class{constructor(e){this.instance=i,this.factory=e}calculateDirectPath(e,t){const n=this.factory.getCanvasMatrix(),o=new r.Grid(n);return i.findPath(this.factory.translateRoutingX(Math.floor(e.getX()/this.factory.ROUTING_SCALING_FACTOR)),this.factory.translateRoutingY(Math.floor(e.getY()/this.factory.ROUTING_SCALING_FACTOR)),this.factory.translateRoutingX(Math.floor(t.getX()/this.factory.ROUTING_SCALING_FACTOR)),this.factory.translateRoutingY(Math.floor(t.getY()/this.factory.ROUTING_SCALING_FACTOR)),o)}calculateLinkStartEndCoords(e,t){const n=t.findIndex(t=>!!e[t[1]]&&0===e[t[1]][t[0]]),r=t.length-1-t.slice().reverse().findIndex(t=>!!e[t[1]]&&0===e[t[1]][t[0]]);if(-1===n||-1===r)return;const i=t.slice(0,n),o=t.slice(r);return{start:{x:t[n][0],y:t[n][1]},end:{x:t[r][0],y:t[r][1]},pathToStart:i,pathToEnd:o}}calculateDynamicPath(e,t,n,o,a){const s=new r.Grid(e),l=i.findPath(t.x,t.y,n.x,n.y,s),u=o.concat(l,a).map(e=>[this.factory.translateRoutingX(e[0],!0),this.factory.translateRoutingY(e[1],!0)]);return r.Util.compressPath(u)}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.RightAngleLinkWidget=void 0;const r=n(0),i=n(152),o=n(198),a=n(151);class s extends r.Component{constructor(e){super(e),this.handleMove=function(e){this.draggingEvent(e,this.dragging_index)}.bind(this),this.handleUp=function(e){this.setState({canDrag:!1,selected:!1}),window.removeEventListener("mousemove",this.handleMove),window.removeEventListener("mouseup",this.handleUp)}.bind(this),this.refPaths=[],this.state={selected:!1,canDrag:!1},this.dragging_index=0}componentDidUpdate(){this.props.link.setRenderedPaths(this.refPaths.map(e=>e.current))}componentDidMount(){this.props.link.setRenderedPaths(this.refPaths.map(e=>e.current))}componentWillUnmount(){this.props.link.setRenderedPaths([])}generateLink(e,t,n){const i=r.createRef();return this.refPaths.push(i),r.createElement(o.DefaultLinkSegmentWidget,{key:"link-"+n,path:e,selected:this.state.selected,diagramEngine:this.props.diagramEngine,factory:this.props.diagramEngine.getFactoryForLink(this.props.link),link:this.props.link,forwardRef:i,onSelection:e=>{this.setState({selected:e})},extras:t})}calculatePositions(e,t,n,r){if(0===n){let t=new i.PointModel({link:this.props.link,position:new a.Point(e[n].getX(),e[n].getY())});return this.props.link.addPoint(t,n),void this.dragging_index++}if(n===e.length-2){let t=new i.PointModel({link:this.props.link,position:new a.Point(e[n+1].getX(),e[n+1].getY())});return void this.props.link.addPoint(t,n+1)}if(n-2>0){let i={[n-2]:e[n-2].getPosition(),[n+1]:e[n+1].getPosition(),[n-1]:e[n-1].getPosition()};if(Math.abs(i[n-1][r]-i[n+1][r])<5)return i[n-2][r]=this.props.diagramEngine.getRelativeMousePoint(t)[r],i[n+1][r]=this.props.diagramEngine.getRelativeMousePoint(t)[r],e[n-2].setPosition(i[n-2]),e[n+1].setPosition(i[n+1]),e[n-1].remove(),e[n-1].remove(),this.dragging_index--,void this.dragging_index--}if(n+2<e.length-2){let i={[n+3]:e[n+3].getPosition(),[n+2]:e[n+2].getPosition(),[n+1]:e[n+1].getPosition(),[n]:e[n].getPosition()};if(Math.abs(i[n+1][r]-i[n+2][r])<5)return i[n][r]=this.props.diagramEngine.getRelativeMousePoint(t)[r],i[n+3][r]=this.props.diagramEngine.getRelativeMousePoint(t)[r],e[n].setPosition(i[n]),e[n+3].setPosition(i[n+3]),e[n+1].remove(),void e[n+1].remove()}let o={[n]:e[n].getPosition(),[n+1]:e[n+1].getPosition()};o[n][r]=this.props.diagramEngine.getRelativeMousePoint(t)[r],o[n+1][r]=this.props.diagramEngine.getRelativeMousePoint(t)[r],e[n].setPosition(o[n]),e[n+1].setPosition(o[n+1])}draggingEvent(e,t){let n=this.props.link.getPoints(),r=Math.abs(n[t].getX()-n[t+1].getX()),i=Math.abs(n[t].getY()-n[t+1].getY());0===r?this.calculatePositions(n,e,t,"x"):0===i&&this.calculatePositions(n,e,t,"y"),this.props.link.setFirstAndLastPathsDirection()}render(){let e=this.props.link.getPoints(),t=[],n=e[0],o=e[e.length-1],s=!1;n.getX()>o.getX()&&(n=e[e.length-1],o=e[0],s=!0);let l=Math.abs(e[0].getY()-e[e.length-1].getY());if(null===this.props.link.getTargetPort()&&2===e.length)[...Array(2)].forEach(e=>{this.props.link.addPoint(new i.PointModel({link:this.props.link,position:new a.Point(n.getX(),o.getY())}),1)}),this.props.link.setManuallyFirstAndLastPathsDirection(!0,!0);else if(null===this.props.link.getTargetPort()&&null!==this.props.link.getSourcePort())e[1].setPosition(o.getX()+(n.getX()-o.getX())/2,s?o.getY():n.getY()),e[2].setPosition(o.getX()+(n.getX()-o.getX())/2,s?n.getY():o.getY());else if(!this.state.canDrag&&e.length>2)for(let t=1;t<e.length;t+=e.length-2)t-1==0?this.props.link.getFirstPathXdirection()?e[t].setPosition(e[t].getX(),e[t-1].getY()):e[t].setPosition(e[t-1].getX(),e[t].getY()):this.props.link.getLastPathXdirection()?e[t-1].setPosition(e[t-1].getX(),e[t].getY()):e[t-1].setPosition(e[t].getX(),e[t-1].getY());2!==e.length||0===l||this.state.canDrag||this.props.link.addPoint(new i.PointModel({link:this.props.link,position:new a.Point(n.getX(),o.getY())}));for(let n=0;n<e.length-1;n++)t.push(this.generateLink(i.LinkWidget.generateLinePath(e[n],e[n+1]),{"data-linkid":this.props.link.getID(),"data-point":n,onMouseDown:e=>{0===e.button&&(this.setState({canDrag:!0}),this.dragging_index=n,window.addEventListener("mousemove",this.handleMove),window.addEventListener("mouseup",this.handleUp))},onMouseEnter:e=>{this.setState({selected:!0}),this.props.link.lastHoverIndexOfPath=n}},n));return this.refPaths=[],r.createElement("g",{"data-default-link-test":this.props.link.getOptions().testName},t)}}t.RightAngleLinkWidget=s,s.defaultProps={color:"red",width:3,link:null,smooth:!1,diagramEngine:null,factory:null}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.RightAngleLinkFactory=void 0;const r=n(0),i=n(1135),o=n(198),a=n(1137);class s extends o.DefaultLinkFactory{constructor(){super(s.NAME)}generateModel(e){return new a.RightAngleLinkModel}generateReactWidget(e){return r.createElement(i.RightAngleLinkWidget,{diagramEngine:this.engine,link:e.model,factory:this})}}t.RightAngleLinkFactory=s,s.NAME="rightAngle"},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.RightAngleLinkModel=void 0;const r=n(198),i=n(1136);class o extends r.DefaultLinkModel{constructor(e={}){super(Object.assign({type:i.RightAngleLinkFactory.NAME},e)),this.lastHoverIndexOfPath=0,this._lastPathXdirection=!1,this._firstPathXdirection=!1}setFirstAndLastPathsDirection(){let e=this.getPoints();for(let t=1;t<e.length;t+=e.length-2){let n=Math.abs(e[t].getX()-e[t-1].getX()),r=Math.abs(e[t].getY()-e[t-1].getY());t-1==0?this._firstPathXdirection=n>r:this._lastPathXdirection=n>r}}addPoint(e,t=1){return super.addPoint(e,t),this.setFirstAndLastPathsDirection(),e}deserialize(e){super.deserialize(e),this.setFirstAndLastPathsDirection()}setManuallyFirstAndLastPathsDirection(e,t){this._firstPathXdirection=e,this._lastPathXdirection=t}getLastPathXdirection(){return this._lastPathXdirection}getFirstPathXdirection(){return this._firstPathXdirection}setWidth(e){this.options.width=e,this.fireEvent({width:e},"widthChanged")}setColor(e){this.options.color=e,this.fireEvent({color:e},"colorChanged")}}t.RightAngleLinkModel=o},function(e,t,n){var r=n(356),i=n(607),o=n(409),a=n(2085),s=n(2086),l=n(1140),u=n(1141),c=n(2089),d=n(2090),h=n(856),f=n(2091),p=n(238),g=n(2092),v=n(2093),m=n(1144),y=n(81),b=n(237),C=n(2097),A=n(130),_=n(2099),I=n(189),x={};x["[object Arguments]"]=x["[object Array]"]=x["[object ArrayBuffer]"]=x["[object DataView]"]=x["[object Boolean]"]=x["[object Date]"]=x["[object Float32Array]"]=x["[object Float64Array]"]=x["[object Int8Array]"]=x["[object Int16Array]"]=x["[object Int32Array]"]=x["[object Map]"]=x["[object Number]"]=x["[object Object]"]=x["[object RegExp]"]=x["[object Set]"]=x["[object String]"]=x["[object Symbol]"]=x["[object Uint8Array]"]=x["[object Uint8ClampedArray]"]=x["[object Uint16Array]"]=x["[object Uint32Array]"]=!0,x["[object Error]"]=x["[object Function]"]=x["[object WeakMap]"]=!1,e.exports=function e(t,n,w,M,S,T){var E,O=1&n,k=2&n,L=4&n;if(w&&(E=S?w(t,M,S,T):w(t)),void 0!==E)return E;if(!A(t))return t;var D=y(t);if(D){if(E=g(t),!O)return u(t,E)}else{var P=p(t),N="[object Function]"==P||"[object GeneratorFunction]"==P;if(b(t))return l(t,O);if("[object Object]"==P||"[object Arguments]"==P||N&&!S){if(E=k||N?{}:m(t),!O)return k?d(t,s(E,t)):c(t,a(E,t))}else{if(!x[P])return S?t:{};E=v(t,P,O)}}T||(T=new r);var j=T.get(t);if(j)return j;T.set(t,E),_(t)?t.forEach((function(r){E.add(e(r,n,w,r,t,T))})):C(t)&&t.forEach((function(r,i){E.set(i,e(r,n,w,i,t,T))}));var R=L?k?f:h:k?keysIn:I,z=D?void 0:R(t);return i(z||t,(function(r,i){z&&(r=t[i=r]),o(E,i,e(r,n,w,i,t,T))})),E}},function(e,t,n){var r=n(207),i=function(){try{var e=r(Object,"defineProperty");return e({},"",{}),e}catch(e){}}();e.exports=i},function(e,t,n){(function(e){var r=n(146),i=t&&!t.nodeType&&t,o=i&&"object"==typeof e&&e&&!e.nodeType&&e,a=o&&o.exports===i?r.Buffer:void 0,s=a?a.allocUnsafe:void 0;e.exports=function(e,t){if(t)return e.slice();var n=e.length,r=s?s(n):new e.constructor(n);return e.copy(r),r}}).call(this,n(188)(e))},function(e,t){e.exports=function(e,t){var n=-1,r=e.length;for(t||(t=Array(r));++n<r;)t[n]=e[n];return t}},function(e,t,n){var r=n(495),i=n(364),o=n(496),a=n(859),s=Object.getOwnPropertySymbols?function(e){for(var t=[];e;)r(t,o(e)),e=i(e);return t}:a;e.exports=s},function(e,t,n){var r=n(608);e.exports=function(e,t){var n=t?r(e.buffer):e.buffer;return new e.constructor(n,e.byteOffset,e.length)}},function(e,t,n){var r=n(1145),i=n(364),o=n(363);e.exports=function(e){return"function"!=typeof e.constructor||o(e)?{}:r(i(e))}},function(e,t,n){var r=n(130),i=Object.create,o=function(){function e(){}return function(t){if(!r(t))return{};if(i)return i(t);e.prototype=t;var n=new e;return e.prototype=void 0,n}}();e.exports=o},function(e,t,n){e.exports=n(1147)},function(e,t,n){var r=n(607),i=n(411),o=n(1148),a=n(81);e.exports=function(e,t){return(a(e)?r:i)(e,o(t))}},function(e,t,n){var r=n(220);e.exports=function(e){return"function"==typeof e?e:r}},function(e,t,n){var r=n(858),i=n(2103),o=n(183),a=n(81);e.exports=function(e,t){return(a(e)?r:i)(e,o(t,3))}},function(e,t,n){var r=n(130);e.exports=function(e){return e==e&&!r(e)}},function(e,t){e.exports=function(e,t){return function(n){return null!=n&&(n[e]===t&&(void 0!==t||e in Object(n)))}}},function(e,t,n){var r=n(2112);e.exports=function(e){return null==e?"":r(e)}},function(e,t,n){var r=n(2113),i=n(1154);e.exports=function(e,t){return null!=e&&i(e,t,r)}},function(e,t,n){var r=n(412),i=n(282),o=n(81),a=n(361),s=n(497),l=n(309);e.exports=function(e,t,n){for(var u=-1,c=(t=r(t,e)).length,d=!1;++u<c;){var h=l(t[u]);if(!(d=null!=e&&n(e,h)))break;e=e[h]}return d||++u!=c?d:!!(c=null==e?0:e.length)&&s(c)&&a(h,c)&&(o(e)||i(e))}},function(e,t){e.exports=function(e){return function(t){return null==t?void 0:t[e]}}},function(e,t,n){var r=n(2116),i=n(1154);e.exports=function(e,t){return null!=e&&i(e,t,r)}},function(e,t){e.exports=function(e){return void 0===e}},function(e,t,n){var r=n(413),i=n(183),o=n(1159),a=n(81);e.exports=function(e,t){return(a(e)?r:o)(e,i(t,3))}},function(e,t,n){var r=n(411),i=n(174);e.exports=function(e,t){var n=-1,o=i(e)?Array(e.length):[];return r(e,(function(e,r,i){o[++n]=t(e,r,i)})),o}},function(e,t,n){var r=n(2118),i=n(411),o=n(183),a=n(2119),s=n(81);e.exports=function(e,t,n){var l=s(e)?r:a,u=arguments.length<3;return l(e,o(t,4),n,u,i)}},function(e,t,n){var r=n(2129),i=Math.max;e.exports=function(e,t,n){return t=i(void 0===t?e.length-1:t,0),function(){for(var o=arguments,a=-1,s=i(o.length-t,0),l=Array(s);++a<s;)l[a]=o[t+a];a=-1;for(var u=Array(t+1);++a<t;)u[a]=o[a];return u[t]=n(l),r(e,this,u)}}},function(e,t,n){var r=n(2130),i=n(2131)(r);e.exports=i},function(e,t){e.exports=function(e,t,n,r){for(var i=e.length,o=n+(r?1:-1);r?o--:++o<i;)if(t(e[o],o,e))return o;return-1}},function(e,t,n){var r=n(174),i=n(158);e.exports=function(e){return i(e)&&r(e)}},function(e,t,n){var r=n(2140),i=n(189);e.exports=function(e){return null==e?[]:r(e,i(e))}},function(e,t,n){var r=n(135),i=n(1167);e.exports=function(e,t,n,r){return function(e,t,n,r){var o,a,s={},l=new i,u=function(e){var t=e.v!==o?e.v:e.w,r=s[t],i=n(e),u=a.distance+i;if(i<0)throw new Error("dijkstra does not allow negative edge weights. Bad edge: "+e+" Weight: "+i);u<r.distance&&(r.distance=u,r.predecessor=o,l.decrease(t,u))};e.nodes().forEach((function(e){var n=e===t?0:Number.POSITIVE_INFINITY;s[e]={distance:n},l.add(e,n)}));for(;l.size()>0&&(o=l.removeMin(),(a=s[o]).distance!==Number.POSITIVE_INFINITY);)r(o).forEach(u);return s}(e,String(t),n||o,r||function(t){return e.outEdges(t)})};var o=r.constant(1)},function(e,t,n){var r=n(135);function i(){this._arr=[],this._keyIndices={}}e.exports=i,i.prototype.size=function(){return this._arr.length},i.prototype.keys=function(){return this._arr.map((function(e){return e.key}))},i.prototype.has=function(e){return r.has(this._keyIndices,e)},i.prototype.priority=function(e){var t=this._keyIndices[e];if(void 0!==t)return this._arr[t].priority},i.prototype.min=function(){if(0===this.size())throw new Error("Queue underflow");return this._arr[0].key},i.prototype.add=function(e,t){var n=this._keyIndices;if(e=String(e),!r.has(n,e)){var i=this._arr,o=i.length;return n[e]=o,i.push({key:e,priority:t}),this._decrease(o),!0}return!1},i.prototype.removeMin=function(){this._swap(0,this._arr.length-1);var e=this._arr.pop();return delete this._keyIndices[e.key],this._heapify(0),e.key},i.prototype.decrease=function(e,t){var n=this._keyIndices[e];if(t>this._arr[n].priority)throw new Error("New priority is greater than current priority. Key: "+e+" Old: "+this._arr[n].priority+" New: "+t);this._arr[n].priority=t,this._decrease(n)},i.prototype._heapify=function(e){var t=this._arr,n=2*e,r=n+1,i=e;n<t.length&&(i=t[n].priority<t[i].priority?n:i,r<t.length&&(i=t[r].priority<t[i].priority?r:i),i!==e&&(this._swap(e,i),this._heapify(i)))},i.prototype._decrease=function(e){for(var t,n=this._arr,r=n[e].priority;0!==e&&!(n[t=e>>1].priority<r);)this._swap(e,t),e=t},i.prototype._swap=function(e,t){var n=this._arr,r=this._keyIndices,i=n[e],o=n[t];n[e]=o,n[t]=i,r[o.key]=e,r[i.key]=t}},function(e,t,n){var r=n(135);e.exports=function(e){var t=0,n=[],i={},o=[];return e.nodes().forEach((function(a){r.has(i,a)||function a(s){var l=i[s]={onStack:!0,lowlink:t,index:t++};if(n.push(s),e.successors(s).forEach((function(e){r.has(i,e)?i[e].onStack&&(l.lowlink=Math.min(l.lowlink,i[e].index)):(a(e),l.lowlink=Math.min(l.lowlink,i[e].lowlink))})),l.lowlink===l.index){var u,c=[];do{u=n.pop(),i[u].onStack=!1,c.push(u)}while(s!==u);o.push(c)}}(a)})),o}},function(e,t,n){var r=n(135);function i(e){var t={},n={},i=[];if(r.each(e.sinks(),(function a(s){if(r.has(n,s))throw new o;r.has(t,s)||(n[s]=!0,t[s]=!0,r.each(e.predecessors(s),a),delete n[s],i.push(s))})),r.size(t)!==e.nodeCount())throw new o;return i}function o(){}e.exports=i,i.CycleException=o,o.prototype=new Error},function(e,t,n){var r=n(135);e.exports=function(e,t,n){r.isArray(t)||(t=[t]);var i=(e.isDirected()?e.successors:e.neighbors).bind(e),o=[],a={};return r.each(t,(function(t){if(!e.hasNode(t))throw new Error("Graph does not have node: "+t);!function e(t,n,i,o,a,s){r.has(o,n)||(o[n]=!0,i||s.push(n),r.each(a(n),(function(n){e(t,n,i,o,a,s)})),i&&s.push(n))}(e,t,"post"===n,a,i,o)})),o}},function(e,t,n){var r=n(2159);e.exports=function(e){return e?(e=r(e))===1/0||e===-1/0?17976931348623157e292*(e<0?-1:1):e==e?e:0:0===e?e:0}},function(e,t,n){var r=n(614);e.exports=function(e){return(null==e?0:e.length)?r(e,1):[]}},function(e,t,n){var r=n(410),i=n(235);e.exports=function(e,t,n){(void 0!==n&&!i(e[t],n)||void 0===n&&!(t in e))&&r(e,t,n)}},function(e,t){e.exports=function(e,t){if(("constructor"!==t||"function"!=typeof e[t])&&"__proto__"!=t)return e[t]}},function(e,t){e.exports=function(e,t){return e<t}},function(e,t,n){"use strict";var r=n(70),i=n(153).Graph,o=n(416).slack;function a(e,t){return r.forEach(e.nodes(),(function n(i){r.forEach(t.nodeEdges(i),(function(r){var a=r.v,s=i===a?r.w:a;e.hasNode(s)||o(t,r)||(e.setNode(s,{}),e.setEdge(i,s,{}),n(s))}))})),e.nodeCount()}function s(e,t){return r.minBy(t.edges(),(function(n){if(e.hasNode(n.v)!==e.hasNode(n.w))return o(t,n)}))}function l(e,t,n){r.forEach(e.nodes(),(function(e){t.node(e).rank+=n}))}e.exports=function(e){var t,n,r=new i({directed:!1}),u=e.nodes()[0],c=e.nodeCount();r.setNode(u,{});for(;a(r,e)<c;)t=s(r,e),n=r.hasNode(t.v)?o(e,t):-o(e,t),l(r,e,n);return r}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.setByString=t.byString=void 0;t.byString=function(e,t){if(t){for(var n=(t=(t=t.replace(/\[(\w+)\]/g,".$1")).replace(/^\./,"")).split("."),r=0,i=n.length;r<i;++r){var o=n[r];if(!e||!(o in e))return;e=e[o]}return e}};t.setByString=function(e,t,n){for(var r=e,i=(t=(t=t.replace(/\[(\w+)\]/g,".$1")).replace(/^\./,"")).split("."),o=i.length,a=0;a<o-1;a++){var s=i[a];r[s]||(r[s]={}),r=r[s]}r[i[o-1]]=n}},function(e,t,n){"use strict";n.r(t);var r=n(645),i=n.n(r),o=n(461),a=n.n(o),s=n(1306),l=n.n(s),u=n(1290),c=n.n(u),d=n(1305),h=n.n(d),f=n(646),p=n.n(f),g=n(651),v=n.n(g),m=n(649),y=n.n(m),b=n(637),C=n.n(b),A=n(1291),_=n.n(A),I=n(1294),x=n.n(I),w=n(1300),M=n.n(w),S=n(464),T=n.n(S),E=n(276),O=n.n(E),k=n(1303),L=n.n(k),D=n(1296),P=n.n(D),N=n(1298),j=n.n(N),R=n(1297),z=n.n(R),F=n(1299),B=n.n(F),H=n(636),U=n.n(H),V=n(1302),Y=n.n(V),W=n(1292),G=n.n(W),X=n(1293),q=n.n(X),Z=n(1304),Q=n.n(Z),K=n(1295),J=n.n(K),$=n(1301),ee=n.n($),te=n(460),ne=n.n(te),re=n(647),ie=n.n(re),oe=n(648),ae=n.n(oe),se=n(650),le=n.n(se),ue=n(465),ce=n.n(ue),de=function(){function e(e){var t=(void 0===e?{}:e).locale;this.yearFormat="yyyy",this.yearMonthFormat="MMMM yyyy",this.dateTime12hFormat="MMMM do hh:mm aaaa",this.dateTime24hFormat="MMMM do HH:mm",this.time12hFormat="hh:mm a",this.time24hFormat="HH:mm",this.dateFormat="MMMM do",this.locale=t}return e.prototype.addDays=function(e,t){return i()(e,t)},e.prototype.isValid=function(e){return U()(this.date(e))},e.prototype.getDiff=function(e,t){return c()(e,this.date(t))},e.prototype.isAfter=function(e,t){return T()(e,t)},e.prototype.isBefore=function(e,t){return O()(e,t)},e.prototype.startOfDay=function(e){return ne()(e)},e.prototype.endOfDay=function(e){return p()(e)},e.prototype.getHours=function(e){return _()(e)},e.prototype.setHours=function(e,t){return G()(e,t)},e.prototype.setMinutes=function(e,t){return q()(e,t)},e.prototype.getSeconds=function(e){return x()(e)},e.prototype.setSeconds=function(e,t){return J()(e,t)},e.prototype.isSameDay=function(e,t){return P()(e,t)},e.prototype.isSameMonth=function(e,t){return z()(e,t)},e.prototype.isSameYear=function(e,t){return j()(e,t)},e.prototype.isSameHour=function(e,t){return B()(e,t)},e.prototype.startOfMonth=function(e){return ie()(e)},e.prototype.endOfMonth=function(e){return ae()(e)},e.prototype.getYear=function(e){return M()(e)},e.prototype.setYear=function(e,t){return ee()(e,t)},e.prototype.date=function(e){return void 0===e?new Date:null===e?null:new Date(e)},e.prototype.parse=function(e,t){return""===e?null:Y()(e,t,new Date,{locale:this.locale})},e.prototype.format=function(e,t){return C()(e,t,{locale:this.locale})},e.prototype.isEqual=function(e,t){return null===e&&null===t||L()(e,t)},e.prototype.isNull=function(e){return null===e},e.prototype.isAfterDay=function(e,t){return T()(e,p()(t))},e.prototype.isBeforeDay=function(e,t){return O()(e,ne()(t))},e.prototype.isBeforeYear=function(e,t){return O()(e,ce()(t))},e.prototype.isAfterYear=function(e,t){return T()(e,y()(t))},e.prototype.formatNumber=function(e){return e},e.prototype.getMinutes=function(e){return e.getMinutes()},e.prototype.getMonth=function(e){return e.getMonth()},e.prototype.setMonth=function(e,t){return Q()(e,t)},e.prototype.getMeridiemText=function(e){return"am"===e?"AM":"PM"},e.prototype.getNextMonth=function(e){return a()(e,1)},e.prototype.getPreviousMonth=function(e){return a()(e,-1)},e.prototype.getMonthArray=function(e){for(var t=[ce()(e)];t.length<12;){var n=t[t.length-1];t.push(this.getNextMonth(n))}return t},e.prototype.mergeDateAndTime=function(e,t){return this.setMinutes(this.setHours(e,this.getHours(t)),this.getMinutes(t))},e.prototype.getWeekdays=function(){var e=this,t=new Date;return h()({start:le()(t,{locale:this.locale}),end:v()(t,{locale:this.locale})}).map((function(t){return e.format(t,"EEEEEE")}))},e.prototype.getWeekArray=function(e){for(var t=le()(ie()(e),{locale:this.locale}),n=v()(ae()(e),{locale:this.locale}),r=0,o=t,a=[];O()(o,n);){var s=Math.floor(r/7);a[s]=a[s]||[],a[s].push(o),o=i()(o,1),r+=1}return a},e.prototype.getYearRange=function(e,t){for(var n=ce()(e),r=y()(t),i=[],o=n;O()(o,r);)i.push(o),o=l()(o,1);return i},e.prototype.getCalendarHeaderText=function(e){return this.format(e,this.yearMonthFormat)},e.prototype.getYearText=function(e){return this.format(e,"yyyy")},e.prototype.getDatePickerHeaderText=function(e){return this.format(e,"EEE, MMM d")},e.prototype.getDateTimePickerHeaderText=function(e){return this.format(e,"MMM d")},e.prototype.getMonthText=function(e){return this.format(e,"MMMM")},e.prototype.getDayText=function(e){return this.format(e,"d")},e.prototype.getHourText=function(e,t){return this.format(e,t?"hh":"HH")},e.prototype.getMinuteText=function(e){return this.format(e,"mm")},e.prototype.getSecondText=function(e){return this.format(e,"ss")},e}();t.default=de},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var r=l(n(2251)),i=l(n(2252)),o=l(n(2254)),a=l(n(2255)),s=l(n(2257));function l(e){return e&&e.__esModule?e:{default:e}}var u={code:"en-US",formatDistance:r.default,formatLong:i.default,formatRelative:o.default,localize:a.default,match:s.default,options:{weekStartsOn:0,firstWeekContainsDate:1}};t.default=u,e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){(0,o.default)(2,arguments);var n=(0,r.default)(t);return(0,i.default)(e,-n)};var r=a(n(77)),i=a(n(2260)),o=a(n(40));function a(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";function r(e,t){switch(e){case"P":return t.date({width:"short"});case"PP":return t.date({width:"medium"});case"PPP":return t.date({width:"long"});case"PPPP":default:return t.date({width:"full"})}}function i(e,t){switch(e){case"p":return t.time({width:"short"});case"pp":return t.time({width:"medium"});case"ppp":return t.time({width:"long"});case"pppp":default:return t.time({width:"full"})}}Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o={p:i,P:function(e,t){var n,o=e.match(/(P+)(p+)?/),a=o[1],s=o[2];if(!s)return r(e,t);switch(a){case"P":n=t.dateTime({width:"short"});break;case"PP":n=t.dateTime({width:"medium"});break;case"PPP":n=t.dateTime({width:"long"});break;case"PPPP":default:n=t.dateTime({width:"full"})}return n.replace("{{date}}",r(a,t)).replace("{{time}}",i(s,t))}};t.default=o,e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){var t=new Date(e.getTime()),n=Math.ceil(t.getTimezoneOffset());t.setSeconds(0,0);var i=n>0?(6e4+r(t))%6e4:r(t);return 6e4*n+i};function r(e){return e.getTime()%6e4}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.isProtectedDayOfYearToken=function(e){return-1!==r.indexOf(e)},t.isProtectedWeekYearToken=function(e){return-1!==i.indexOf(e)},t.throwProtectedError=function(e,t,n){if("YYYY"===e)throw new RangeError("Use `yyyy` instead of `YYYY` (in `".concat(t,"`) for formatting years to the input `").concat(n,"`; see: https://git.io/fxCyr"));if("YY"===e)throw new RangeError("Use `yy` instead of `YY` (in `".concat(t,"`) for formatting years to the input `").concat(n,"`; see: https://git.io/fxCyr"));if("D"===e)throw new RangeError("Use `d` instead of `D` (in `".concat(t,"`) for formatting days of the month to the input `").concat(n,"`; see: https://git.io/fxCyr"));if("DD"===e)throw new RangeError("Use `dd` instead of `DD` (in `".concat(t,"`) for formatting days of the month to the input `").concat(n,"`; see: https://git.io/fxCyr"))};var r=["D","DD"],i=["YY","YYYY"]},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){(0,a.default)(1,arguments);var t=(0,r.default)(e),n=(0,i.default)(t).getTime()-(0,o.default)(t).getTime();return Math.round(n/6048e5)+1};var r=s(n(47)),i=s(n(425)),o=s(n(2266)),a=s(n(40));function s(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){(0,o.default)(1,arguments);var t=(0,r.default)(e),n=t.getUTCFullYear(),a=new Date(0);a.setUTCFullYear(n+1,0,4),a.setUTCHours(0,0,0,0);var s=(0,i.default)(a),l=new Date(0);l.setUTCFullYear(n,0,4),l.setUTCHours(0,0,0,0);var u=(0,i.default)(l);return t.getTime()>=s.getTime()?n+1:t.getTime()>=u.getTime()?n:n-1};var r=a(n(47)),i=a(n(425)),o=a(n(40));function a(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){(0,a.default)(1,arguments);var n=(0,r.default)(e),s=(0,i.default)(n,t).getTime()-(0,o.default)(n,t).getTime();return Math.round(s/6048e5)+1};var r=s(n(47)),i=s(n(424)),o=s(n(2268)),a=s(n(40));function s(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){var n=e<0?"-":"",r=Math.abs(e).toString();for(;r.length<t;)r="0"+r;return n+r},e.exports=t.default},function(e,t,n){var r,i,o=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),a=this&&this.__setModuleDefault||(Object.create?function(e,t){Object.defineProperty(e,"default",{enumerable:!0,value:t})}:function(e,t){e.default=t}),s=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)"default"!==n&&Object.hasOwnProperty.call(e,n)&&o(t,e,n);return a(t,e),t},l=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};r=[n,t,n(262),n(201),n(17),n(2335),n(0),n(78),n(118),n(2336),n(118),n(620),n(620),n(1189),n(1190)],void 0===(i=function(e,t,n,r,i,o,a,u,c,d,h,f,p,g){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.BaseWidgetView=t.createInitialStore=t.WidgetWrapper=t.BaseWidgetModel=t.getEnhancers=void 0,o=l(o),a=s(a),d=l(d),t.getEnhancers=()=>{let e=c.applyMiddleware(d.default);return window.__REDUX_DEVTOOLS_EXTENSION__,e};class v extends n.BoxModel{defaults(){return Object.assign(Object.assign({},super.defaults()),{_model_name:this.model_name,_model_module:v.model_module,_model_module_version:v.model_module_version,_view_name:this.view_name,_view_module:v.view_module,_view_module_version:v.view_module_version,title:"",anchor:"split-right",system_data:{key:"None"},geo_data:{},computed_data:{},recorder_data:{},driver_data:{},progress_geo_update:{},update_signal:0,notification_msg:{update:0,msg:"",log:""},initial_store:{},chart_template:{}})}initialize(e,t){super.initialize(e,t),this.widget_manager.display_model(void 0,this,{})}}t.BaseWidgetModel=v,v.model_module=g.MODULE_NAME,v.model_module_version=g.MODULE_VERSION,v.view_module=g.MODULE_NAME,v.view_module_version=g.MODULE_VERSION;class m extends r.ReactWidget{constructor(e,t){super(),this.onResize=e=>{window.dispatchEvent(new Event("resize"))},this._store=e,this._component=t}render(){return a.createElement(u.Provider,{store:this._store},this._component)}}t.WidgetWrapper=m,t.createInitialStore=(e,t)=>{const{recorderData:n,driverData:r}=t;return Object.assign(Object.assign({},e),{systemArch:Object.assign(Object.assign({},e.systemArch),{systemGraph:t.systemGraph,systemTree:Object.assign(Object.assign({},e.systemArch.systemTree),{nodeData:t.systemTree}),systemPBS:t.systemPBS}),dashboardState:Object.assign(Object.assign({},e.dashboardState),{variableData:t.variableData,portMetaData:t.portMetaData,computedResult:t.computedResult,recorderData:n,driverData:r})})};class y extends n.VBoxView{constructor(){super(...arguments),this.class_name="cosapp-widget-viewer",this.store=h.createStore(f.rootReducer,this.getStore(),t.getEnhancers())}getStore(){const e=Object.assign({},p.initialState),n=this.model.get("system_data"),r=t.createInitialStore(e,n),i=this.model.get("initial_store");for(const e in i)i.hasOwnProperty(e)&&r.hasOwnProperty(e)&&(r[e]=i[e]);return r}_handleKernelStatusChanged(e,t){"restarting"!==t&&"dead"!==t||(e.statusChanged.disconnect(this._handleKernelStatusChanged,this),this.remove())}_initialize(e){const t=e.tracker.currentWidget;if(t){const e=t.sessionContext.session;e&&e.statusChanged.connect(this._handleKernelStatusChanged,this)}}_render(e,t){if(e.shell){const n=this.pWidget,r=new m(this.store,a.createElement(o.default,{send_msg:this.send.bind(this),model:this.model,CompClass:t}));n.addWidget(r),n.addClass(this.class_name),n.addClass("cosapp-geometry-viewer"),n.title.label=this.model.get("title"),n.title.closable=!0,e.shell._rightHandler.sideBar.tabCloseRequested.connect((e,t)=>{t.title.owner.dispose()}),n.id=i.UUID.uuid4();const s=this.model.get("anchor");"right"===s?(e.shell.add(n,"right"),e.shell.expandRight()):e.shell.add(n,"main",{mode:s})}}}t.BaseWidgetView=y}.apply(t,r))||(e.exports=i)},function(e,t,n){var r;void 0===(r=function(e,t){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.MODULE_NAME=t.MODULE_VERSION=void 0;const r=n(2337);t.MODULE_VERSION=r.version,t.MODULE_NAME=r.name}.apply(t,[n,t]))||(e.exports=r)},function(e,t,n){var r=n(2338);"string"==typeof r&&(r=[[e.i,r,""]]);var i={insert:"head",singleton:!1};n(147)(r,i);r.locals&&(e.exports=r.locals)},function(e,t,n){"use strict";n.r(t);var r={};n.r(r),n.d(r,"FILE",(function(){return Zt})),n.d(r,"URL",(function(){return Qt})),n.d(r,"TEXT",(function(){return Kt}));var i=n(33),o=n.n(i),a=n(34),s=n.n(a),l=n(43),u=n.n(l),c=n(44),d=n.n(c),h=n(9),f=n.n(h),p=n(45),g=n.n(p),v=n(8),m=n.n(v),y=n(0),b=n.n(y),C=n(13),A=n(50),_=n.n(A),I=n(4);function x(e){var t=e.cellCount,n=e.cellSize,r=e.computeMetadataCallback,i=e.computeMetadataCallbackProps,o=e.nextCellsCount,a=e.nextCellSize,s=e.nextScrollToIndex,l=e.scrollToIndex,u=e.updateScrollOffsetForScrollToIndex;t===o&&("number"!=typeof n&&"number"!=typeof a||n===a)||(r(i),l>=0&&l===s&&u())}var w=n(98),M=n.n(w),S=n(3),T=n.n(S),E=function(){function e(t){var n=t.cellCount,r=t.cellSizeGetter,i=t.estimatedCellSize;o()(this,e),m()(this,"_cellSizeAndPositionData",{}),m()(this,"_lastMeasuredIndex",-1),m()(this,"_lastBatchedIndex",-1),m()(this,"_cellCount",void 0),m()(this,"_cellSizeGetter",void 0),m()(this,"_estimatedCellSize",void 0),this._cellSizeGetter=r,this._cellCount=n,this._estimatedCellSize=i}return s()(e,[{key:"areOffsetsAdjusted",value:function(){return!1}},{key:"configure",value:function(e){var t=e.cellCount,n=e.estimatedCellSize,r=e.cellSizeGetter;this._cellCount=t,this._estimatedCellSize=n,this._cellSizeGetter=r}},{key:"getCellCount",value:function(){return this._cellCount}},{key:"getEstimatedCellSize",value:function(){return this._estimatedCellSize}},{key:"getLastMeasuredIndex",value:function(){return this._lastMeasuredIndex}},{key:"getOffsetAdjustment",value:function(){return 0}},{key:"getSizeAndPositionOfCell",value:function(e){if(e<0||e>=this._cellCount)throw Error("Requested index ".concat(e," is outside of range 0..").concat(this._cellCount));if(e>this._lastMeasuredIndex)for(var t=this.getSizeAndPositionOfLastMeasuredCell(),n=t.offset+t.size,r=this._lastMeasuredIndex+1;r<=e;r++){var i=this._cellSizeGetter({index:r});if(void 0===i||isNaN(i))throw Error("Invalid size returned for cell ".concat(r," of value ").concat(i));null===i?(this._cellSizeAndPositionData[r]={offset:n,size:0},this._lastBatchedIndex=e):(this._cellSizeAndPositionData[r]={offset:n,size:i},n+=i,this._lastMeasuredIndex=e)}return this._cellSizeAndPositionData[e]}},{key:"getSizeAndPositionOfLastMeasuredCell",value:function(){return this._lastMeasuredIndex>=0?this._cellSizeAndPositionData[this._lastMeasuredIndex]:{offset:0,size:0}}},{key:"getTotalSize",value:function(){var e=this.getSizeAndPositionOfLastMeasuredCell();return e.offset+e.size+(this._cellCount-this._lastMeasuredIndex-1)*this._estimatedCellSize}},{key:"getUpdatedOffsetForIndex",value:function(e){var t=e.align,n=void 0===t?"auto":t,r=e.containerSize,i=e.currentOffset,o=e.targetIndex;if(r<=0)return 0;var a,s=this.getSizeAndPositionOfCell(o),l=s.offset,u=l-r+s.size;switch(n){case"start":a=l;break;case"end":a=u;break;case"center":a=l-(r-s.size)/2;break;default:a=Math.max(u,Math.min(l,i))}var c=this.getTotalSize();return Math.max(0,Math.min(c-r,a))}},{key:"getVisibleCellRange",value:function(e){var t=e.containerSize,n=e.offset;if(0===this.getTotalSize())return{};var r=n+t,i=this._findNearestCell(n),o=this.getSizeAndPositionOfCell(i);n=o.offset+o.size;for(var a=i;n<r&&a<this._cellCount-1;)a++,n+=this.getSizeAndPositionOfCell(a).size;return{start:i,stop:a}}},{key:"resetCell",value:function(e){this._lastMeasuredIndex=Math.min(this._lastMeasuredIndex,e-1)}},{key:"_binarySearch",value:function(e,t,n){for(;t<=e;){var r=t+Math.floor((e-t)/2),i=this.getSizeAndPositionOfCell(r).offset;if(i===n)return r;i<n?t=r+1:i>n&&(e=r-1)}return t>0?t-1:0}},{key:"_exponentialSearch",value:function(e,t){for(var n=1;e<this._cellCount&&this.getSizeAndPositionOfCell(e).offset<t;)e+=n,n*=2;return this._binarySearch(Math.min(e,this._cellCount-1),Math.floor(e/2),t)}},{key:"_findNearestCell",value:function(e){if(isNaN(e))throw Error("Invalid offset ".concat(e," specified"));e=Math.max(0,e);var t=this.getSizeAndPositionOfLastMeasuredCell(),n=Math.max(0,this._lastMeasuredIndex);return t.offset>=e?this._binarySearch(n,0,e):this._exponentialSearch(n,e)}}]),e}(),O=function(){return"undefined"!=typeof window&&window.chrome?16777100:15e5},k=function(){function e(t){var n=t.maxScrollSize,r=void 0===n?O():n,i=M()(t,["maxScrollSize"]);o()(this,e),m()(this,"_cellSizeAndPositionManager",void 0),m()(this,"_maxScrollSize",void 0),this._cellSizeAndPositionManager=new E(i),this._maxScrollSize=r}return s()(e,[{key:"areOffsetsAdjusted",value:function(){return this._cellSizeAndPositionManager.getTotalSize()>this._maxScrollSize}},{key:"configure",value:function(e){this._cellSizeAndPositionManager.configure(e)}},{key:"getCellCount",value:function(){return this._cellSizeAndPositionManager.getCellCount()}},{key:"getEstimatedCellSize",value:function(){return this._cellSizeAndPositionManager.getEstimatedCellSize()}},{key:"getLastMeasuredIndex",value:function(){return this._cellSizeAndPositionManager.getLastMeasuredIndex()}},{key:"getOffsetAdjustment",value:function(e){var t=e.containerSize,n=e.offset,r=this._cellSizeAndPositionManager.getTotalSize(),i=this.getTotalSize(),o=this._getOffsetPercentage({containerSize:t,offset:n,totalSize:i});return Math.round(o*(i-r))}},{key:"getSizeAndPositionOfCell",value:function(e){return this._cellSizeAndPositionManager.getSizeAndPositionOfCell(e)}},{key:"getSizeAndPositionOfLastMeasuredCell",value:function(){return this._cellSizeAndPositionManager.getSizeAndPositionOfLastMeasuredCell()}},{key:"getTotalSize",value:function(){return Math.min(this._maxScrollSize,this._cellSizeAndPositionManager.getTotalSize())}},{key:"getUpdatedOffsetForIndex",value:function(e){var t=e.align,n=void 0===t?"auto":t,r=e.containerSize,i=e.currentOffset,o=e.targetIndex;i=this._safeOffsetToOffset({containerSize:r,offset:i});var a=this._cellSizeAndPositionManager.getUpdatedOffsetForIndex({align:n,containerSize:r,currentOffset:i,targetIndex:o});return this._offsetToSafeOffset({containerSize:r,offset:a})}},{key:"getVisibleCellRange",value:function(e){var t=e.containerSize,n=e.offset;return n=this._safeOffsetToOffset({containerSize:t,offset:n}),this._cellSizeAndPositionManager.getVisibleCellRange({containerSize:t,offset:n})}},{key:"resetCell",value:function(e){this._cellSizeAndPositionManager.resetCell(e)}},{key:"_getOffsetPercentage",value:function(e){var t=e.containerSize,n=e.offset,r=e.totalSize;return r<=t?0:n/(r-t)}},{key:"_offsetToSafeOffset",value:function(e){var t=e.containerSize,n=e.offset,r=this._cellSizeAndPositionManager.getTotalSize(),i=this.getTotalSize();if(r===i)return n;var o=this._getOffsetPercentage({containerSize:t,offset:n,totalSize:r});return Math.round(o*(i-t))}},{key:"_safeOffsetToOffset",value:function(e){var t=e.containerSize,n=e.offset,r=this._cellSizeAndPositionManager.getTotalSize(),i=this.getTotalSize();if(r===i)return n;var o=this._getOffsetPercentage({containerSize:t,offset:n,totalSize:i});return Math.round(o*(r-t))}}]),e}();function L(){var e=!(arguments.length>0&&void 0!==arguments[0])||arguments[0],t={};return function(n){var r=n.callback,i=n.indices,o=Object.keys(i),a=!e||o.every((function(e){var t=i[e];return Array.isArray(t)?t.length>0:t>=0})),s=o.length!==Object.keys(t).length||o.some((function(e){var n=t[e],r=i[e];return Array.isArray(r)?n.join(",")!==r.join(","):n!==r}));t=i,a&&s&&r(i)}}function D(e){var t=e.cellSize,n=e.cellSizeAndPositionManager,r=e.previousCellsCount,i=e.previousCellSize,o=e.previousScrollToAlignment,a=e.previousScrollToIndex,s=e.previousSize,l=e.scrollOffset,u=e.scrollToAlignment,c=e.scrollToIndex,d=e.size,h=e.sizeJustIncreasedFromZero,f=e.updateScrollIndexCallback,p=n.getCellCount(),g=c>=0&&c<p;g&&(d!==s||h||!i||"number"==typeof t&&t!==i||u!==o||c!==a)?f(c):!g&&p>0&&(d<s||p<r)&&l>n.getTotalSize()-d&&f(p-1)}var P,N,j=!("undefined"==typeof window||!window.document||!window.document.createElement);function R(e){if((!P&&0!==P||e)&&j){var t=document.createElement("div");t.style.position="absolute",t.style.top="-9999px",t.style.width="50px",t.style.height="50px",t.style.overflow="scroll",document.body.appendChild(t),P=t.offsetWidth-t.clientWidth,document.body.removeChild(t)}return P}var z,F,B=(N="undefined"!=typeof window?window:"undefined"!=typeof self?self:{}).requestAnimationFrame||N.webkitRequestAnimationFrame||N.mozRequestAnimationFrame||N.oRequestAnimationFrame||N.msRequestAnimationFrame||function(e){return N.setTimeout(e,1e3/60)},H=N.cancelAnimationFrame||N.webkitCancelAnimationFrame||N.mozCancelAnimationFrame||N.oCancelAnimationFrame||N.msCancelAnimationFrame||function(e){N.clearTimeout(e)},U=B,V=H,Y=function(e){return V(e.id)},W=function(e,t){var n;Promise.resolve().then((function(){n=Date.now()}));var r={id:U((function i(){Date.now()-n>=t?e.call():r.id=U(i)}))};return r};function G(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),n.push.apply(n,r)}return n}function X(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?G(n,!0).forEach((function(t){m()(e,t,n[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):G(n).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))}))}return e}var q="observed",Z="requested",Q=(F=z=function(e){function t(e){var n;o()(this,t),n=u()(this,d()(t).call(this,e)),m()(f()(n),"_onGridRenderedMemoizer",L()),m()(f()(n),"_onScrollMemoizer",L(!1)),m()(f()(n),"_deferredInvalidateColumnIndex",null),m()(f()(n),"_deferredInvalidateRowIndex",null),m()(f()(n),"_recomputeScrollLeftFlag",!1),m()(f()(n),"_recomputeScrollTopFlag",!1),m()(f()(n),"_horizontalScrollBarSize",0),m()(f()(n),"_verticalScrollBarSize",0),m()(f()(n),"_scrollbarPresenceChanged",!1),m()(f()(n),"_scrollingContainer",void 0),m()(f()(n),"_childrenToDisplay",void 0),m()(f()(n),"_columnStartIndex",void 0),m()(f()(n),"_columnStopIndex",void 0),m()(f()(n),"_rowStartIndex",void 0),m()(f()(n),"_rowStopIndex",void 0),m()(f()(n),"_renderedColumnStartIndex",0),m()(f()(n),"_renderedColumnStopIndex",0),m()(f()(n),"_renderedRowStartIndex",0),m()(f()(n),"_renderedRowStopIndex",0),m()(f()(n),"_initialScrollTop",void 0),m()(f()(n),"_initialScrollLeft",void 0),m()(f()(n),"_disablePointerEventsTimeoutId",void 0),m()(f()(n),"_styleCache",{}),m()(f()(n),"_cellCache",{}),m()(f()(n),"_debounceScrollEndedCallback",(function(){n._disablePointerEventsTimeoutId=null,n.setState({isScrolling:!1,needToResetStyleCache:!1})})),m()(f()(n),"_invokeOnGridRenderedHelper",(function(){var e=n.props.onSectionRendered;n._onGridRenderedMemoizer({callback:e,indices:{columnOverscanStartIndex:n._columnStartIndex,columnOverscanStopIndex:n._columnStopIndex,columnStartIndex:n._renderedColumnStartIndex,columnStopIndex:n._renderedColumnStopIndex,rowOverscanStartIndex:n._rowStartIndex,rowOverscanStopIndex:n._rowStopIndex,rowStartIndex:n._renderedRowStartIndex,rowStopIndex:n._renderedRowStopIndex}})})),m()(f()(n),"_setScrollingContainerRef",(function(e){n._scrollingContainer=e})),m()(f()(n),"_onScroll",(function(e){e.target===n._scrollingContainer&&n.handleScrollEvent(e.target)}));var r=new k({cellCount:e.columnCount,cellSizeGetter:function(n){return t._wrapSizeGetter(e.columnWidth)(n)},estimatedCellSize:t._getEstimatedColumnSize(e)}),i=new k({cellCount:e.rowCount,cellSizeGetter:function(n){return t._wrapSizeGetter(e.rowHeight)(n)},estimatedCellSize:t._getEstimatedRowSize(e)});return n.state={instanceProps:{columnSizeAndPositionManager:r,rowSizeAndPositionManager:i,prevColumnWidth:e.columnWidth,prevRowHeight:e.rowHeight,prevColumnCount:e.columnCount,prevRowCount:e.rowCount,prevIsScrolling:!0===e.isScrolling,prevScrollToColumn:e.scrollToColumn,prevScrollToRow:e.scrollToRow,scrollbarSize:0,scrollbarSizeMeasured:!1},isScrolling:!1,scrollDirectionHorizontal:1,scrollDirectionVertical:1,scrollLeft:0,scrollTop:0,scrollPositionChangeReason:null,needToResetStyleCache:!1},e.scrollToRow>0&&(n._initialScrollTop=n._getCalculatedScrollTop(e,n.state)),e.scrollToColumn>0&&(n._initialScrollLeft=n._getCalculatedScrollLeft(e,n.state)),n}return g()(t,e),s()(t,[{key:"getOffsetForCell",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},t=e.alignment,n=void 0===t?this.props.scrollToAlignment:t,r=e.columnIndex,i=void 0===r?this.props.scrollToColumn:r,o=e.rowIndex,a=void 0===o?this.props.scrollToRow:o,s=X({},this.props,{scrollToAlignment:n,scrollToColumn:i,scrollToRow:a});return{scrollLeft:this._getCalculatedScrollLeft(s),scrollTop:this._getCalculatedScrollTop(s)}}},{key:"getTotalRowsHeight",value:function(){return this.state.instanceProps.rowSizeAndPositionManager.getTotalSize()}},{key:"getTotalColumnsWidth",value:function(){return this.state.instanceProps.columnSizeAndPositionManager.getTotalSize()}},{key:"handleScrollEvent",value:function(e){var t=e.scrollLeft,n=void 0===t?0:t,r=e.scrollTop,i=void 0===r?0:r;if(!(i<0)){this._debounceScrollEnded();var o=this.props,a=o.autoHeight,s=o.autoWidth,l=o.height,u=o.width,c=this.state.instanceProps,d=c.scrollbarSize,h=c.rowSizeAndPositionManager.getTotalSize(),f=c.columnSizeAndPositionManager.getTotalSize(),p=Math.min(Math.max(0,f-u+d),n),g=Math.min(Math.max(0,h-l+d),i);if(this.state.scrollLeft!==p||this.state.scrollTop!==g){var v={isScrolling:!0,scrollDirectionHorizontal:p!==this.state.scrollLeft?p>this.state.scrollLeft?1:-1:this.state.scrollDirectionHorizontal,scrollDirectionVertical:g!==this.state.scrollTop?g>this.state.scrollTop?1:-1:this.state.scrollDirectionVertical,scrollPositionChangeReason:q};a||(v.scrollTop=g),s||(v.scrollLeft=p),v.needToResetStyleCache=!1,this.setState(v)}this._invokeOnScrollMemoizer({scrollLeft:p,scrollTop:g,totalColumnsWidth:f,totalRowsHeight:h})}}},{key:"invalidateCellSizeAfterRender",value:function(e){var t=e.columnIndex,n=e.rowIndex;this._deferredInvalidateColumnIndex="number"==typeof this._deferredInvalidateColumnIndex?Math.min(this._deferredInvalidateColumnIndex,t):t,this._deferredInvalidateRowIndex="number"==typeof this._deferredInvalidateRowIndex?Math.min(this._deferredInvalidateRowIndex,n):n}},{key:"measureAllCells",value:function(){var e=this.props,t=e.columnCount,n=e.rowCount,r=this.state.instanceProps;r.columnSizeAndPositionManager.getSizeAndPositionOfCell(t-1),r.rowSizeAndPositionManager.getSizeAndPositionOfCell(n-1)}},{key:"recomputeGridSize",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},t=e.columnIndex,n=void 0===t?0:t,r=e.rowIndex,i=void 0===r?0:r,o=this.props,a=o.scrollToColumn,s=o.scrollToRow,l=this.state.instanceProps;l.columnSizeAndPositionManager.resetCell(n),l.rowSizeAndPositionManager.resetCell(i),this._recomputeScrollLeftFlag=a>=0&&(1===this.state.scrollDirectionHorizontal?n<=a:n>=a),this._recomputeScrollTopFlag=s>=0&&(1===this.state.scrollDirectionVertical?i<=s:i>=s),this._styleCache={},this._cellCache={},this.forceUpdate()}},{key:"scrollToCell",value:function(e){var t=e.columnIndex,n=e.rowIndex,r=this.props.columnCount,i=this.props;r>1&&void 0!==t&&this._updateScrollLeftForScrollToColumn(X({},i,{scrollToColumn:t})),void 0!==n&&this._updateScrollTopForScrollToRow(X({},i,{scrollToRow:n}))}},{key:"componentDidMount",value:function(){var e=this.props,n=e.getScrollbarSize,r=e.height,i=e.scrollLeft,o=e.scrollToColumn,a=e.scrollTop,s=e.scrollToRow,l=e.width,u=this.state.instanceProps;if(this._initialScrollTop=0,this._initialScrollLeft=0,this._handleInvalidatedGridSize(),u.scrollbarSizeMeasured||this.setState((function(e){var t=X({},e,{needToResetStyleCache:!1});return t.instanceProps.scrollbarSize=n(),t.instanceProps.scrollbarSizeMeasured=!0,t})),"number"==typeof i&&i>=0||"number"==typeof a&&a>=0){var c=t._getScrollToPositionStateUpdate({prevState:this.state,scrollLeft:i,scrollTop:a});c&&(c.needToResetStyleCache=!1,this.setState(c))}this._scrollingContainer&&(this._scrollingContainer.scrollLeft!==this.state.scrollLeft&&(this._scrollingContainer.scrollLeft=this.state.scrollLeft),this._scrollingContainer.scrollTop!==this.state.scrollTop&&(this._scrollingContainer.scrollTop=this.state.scrollTop));var d=r>0&&l>0;o>=0&&d&&this._updateScrollLeftForScrollToColumn(),s>=0&&d&&this._updateScrollTopForScrollToRow(),this._invokeOnGridRenderedHelper(),this._invokeOnScrollMemoizer({scrollLeft:i||0,scrollTop:a||0,totalColumnsWidth:u.columnSizeAndPositionManager.getTotalSize(),totalRowsHeight:u.rowSizeAndPositionManager.getTotalSize()}),this._maybeCallOnScrollbarPresenceChange()}},{key:"componentDidUpdate",value:function(e,t){var n=this,r=this.props,i=r.autoHeight,o=r.autoWidth,a=r.columnCount,s=r.height,l=r.rowCount,u=r.scrollToAlignment,c=r.scrollToColumn,d=r.scrollToRow,h=r.width,f=this.state,p=f.scrollLeft,g=f.scrollPositionChangeReason,v=f.scrollTop,m=f.instanceProps;this._handleInvalidatedGridSize();var y=a>0&&0===e.columnCount||l>0&&0===e.rowCount;g===Z&&(!o&&p>=0&&(p!==this._scrollingContainer.scrollLeft||y)&&(this._scrollingContainer.scrollLeft=p),!i&&v>=0&&(v!==this._scrollingContainer.scrollTop||y)&&(this._scrollingContainer.scrollTop=v));var b=(0===e.width||0===e.height)&&s>0&&h>0;if(this._recomputeScrollLeftFlag?(this._recomputeScrollLeftFlag=!1,this._updateScrollLeftForScrollToColumn(this.props)):D({cellSizeAndPositionManager:m.columnSizeAndPositionManager,previousCellsCount:e.columnCount,previousCellSize:e.columnWidth,previousScrollToAlignment:e.scrollToAlignment,previousScrollToIndex:e.scrollToColumn,previousSize:e.width,scrollOffset:p,scrollToAlignment:u,scrollToIndex:c,size:h,sizeJustIncreasedFromZero:b,updateScrollIndexCallback:function(){return n._updateScrollLeftForScrollToColumn(n.props)}}),this._recomputeScrollTopFlag?(this._recomputeScrollTopFlag=!1,this._updateScrollTopForScrollToRow(this.props)):D({cellSizeAndPositionManager:m.rowSizeAndPositionManager,previousCellsCount:e.rowCount,previousCellSize:e.rowHeight,previousScrollToAlignment:e.scrollToAlignment,previousScrollToIndex:e.scrollToRow,previousSize:e.height,scrollOffset:v,scrollToAlignment:u,scrollToIndex:d,size:s,sizeJustIncreasedFromZero:b,updateScrollIndexCallback:function(){return n._updateScrollTopForScrollToRow(n.props)}}),this._invokeOnGridRenderedHelper(),p!==t.scrollLeft||v!==t.scrollTop){var C=m.rowSizeAndPositionManager.getTotalSize(),A=m.columnSizeAndPositionManager.getTotalSize();this._invokeOnScrollMemoizer({scrollLeft:p,scrollTop:v,totalColumnsWidth:A,totalRowsHeight:C})}this._maybeCallOnScrollbarPresenceChange()}},{key:"componentWillUnmount",value:function(){this._disablePointerEventsTimeoutId&&Y(this._disablePointerEventsTimeoutId)}},{key:"render",value:function(){var e=this.props,t=e.autoContainerWidth,n=e.autoHeight,r=e.autoWidth,i=e.className,o=e.containerProps,a=e.containerRole,s=e.containerStyle,l=e.height,u=e.id,c=e.noContentRenderer,d=e.role,h=e.style,f=e.tabIndex,p=e.width,g=this.state,v=g.instanceProps,m=g.needToResetStyleCache,b=this._isScrolling(),C={boxSizing:"border-box",direction:"ltr",height:n?"auto":l,position:"relative",width:r?"auto":p,WebkitOverflowScrolling:"touch",willChange:"transform"};m&&(this._styleCache={}),this.state.isScrolling||this._resetStyleCache(),this._calculateChildrenToRender(this.props,this.state);var A=v.columnSizeAndPositionManager.getTotalSize(),x=v.rowSizeAndPositionManager.getTotalSize(),w=x>l?v.scrollbarSize:0,M=A>p?v.scrollbarSize:0;M===this._horizontalScrollBarSize&&w===this._verticalScrollBarSize||(this._horizontalScrollBarSize=M,this._verticalScrollBarSize=w,this._scrollbarPresenceChanged=!0),C.overflowX=A+w<=p?"hidden":"auto",C.overflowY=x+M<=l?"hidden":"auto";var S=this._childrenToDisplay,T=0===S.length&&l>0&&p>0;return y.createElement("div",_()({ref:this._setScrollingContainerRef},o,{"aria-label":this.props["aria-label"],"aria-readonly":this.props["aria-readonly"],className:Object(I.a)("ReactVirtualized__Grid",i),id:u,onScroll:this._onScroll,role:d,style:X({},C,{},h),tabIndex:f}),S.length>0&&y.createElement("div",{className:"ReactVirtualized__Grid__innerScrollContainer",role:a,style:X({width:t?"auto":A,height:x,maxWidth:A,maxHeight:x,overflow:"hidden",pointerEvents:b?"none":"",position:"relative"},s)},S),T&&c())}},{key:"_calculateChildrenToRender",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:this.props,t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:this.state,n=e.cellRenderer,r=e.cellRangeRenderer,i=e.columnCount,o=e.deferredMeasurementCache,a=e.height,s=e.overscanColumnCount,l=e.overscanIndicesGetter,u=e.overscanRowCount,c=e.rowCount,d=e.width,h=e.isScrollingOptOut,f=t.scrollDirectionHorizontal,p=t.scrollDirectionVertical,g=t.instanceProps,v=this._initialScrollTop>0?this._initialScrollTop:t.scrollTop,m=this._initialScrollLeft>0?this._initialScrollLeft:t.scrollLeft,y=this._isScrolling(e,t);if(this._childrenToDisplay=[],a>0&&d>0){var b=g.columnSizeAndPositionManager.getVisibleCellRange({containerSize:d,offset:m}),C=g.rowSizeAndPositionManager.getVisibleCellRange({containerSize:a,offset:v}),A=g.columnSizeAndPositionManager.getOffsetAdjustment({containerSize:d,offset:m}),_=g.rowSizeAndPositionManager.getOffsetAdjustment({containerSize:a,offset:v});this._renderedColumnStartIndex=b.start,this._renderedColumnStopIndex=b.stop,this._renderedRowStartIndex=C.start,this._renderedRowStopIndex=C.stop;var I=l({direction:"horizontal",cellCount:i,overscanCellsCount:s,scrollDirection:f,startIndex:"number"==typeof b.start?b.start:0,stopIndex:"number"==typeof b.stop?b.stop:-1}),x=l({direction:"vertical",cellCount:c,overscanCellsCount:u,scrollDirection:p,startIndex:"number"==typeof C.start?C.start:0,stopIndex:"number"==typeof C.stop?C.stop:-1}),w=I.overscanStartIndex,M=I.overscanStopIndex,S=x.overscanStartIndex,T=x.overscanStopIndex;if(o){if(!o.hasFixedHeight())for(var E=S;E<=T;E++)if(!o.has(E,0)){w=0,M=i-1;break}if(!o.hasFixedWidth())for(var O=w;O<=M;O++)if(!o.has(0,O)){S=0,T=c-1;break}}this._childrenToDisplay=r({cellCache:this._cellCache,cellRenderer:n,columnSizeAndPositionManager:g.columnSizeAndPositionManager,columnStartIndex:w,columnStopIndex:M,deferredMeasurementCache:o,horizontalOffsetAdjustment:A,isScrolling:y,isScrollingOptOut:h,parent:this,rowSizeAndPositionManager:g.rowSizeAndPositionManager,rowStartIndex:S,rowStopIndex:T,scrollLeft:m,scrollTop:v,styleCache:this._styleCache,verticalOffsetAdjustment:_,visibleColumnIndices:b,visibleRowIndices:C}),this._columnStartIndex=w,this._columnStopIndex=M,this._rowStartIndex=S,this._rowStopIndex=T}}},{key:"_debounceScrollEnded",value:function(){var e=this.props.scrollingResetTimeInterval;this._disablePointerEventsTimeoutId&&Y(this._disablePointerEventsTimeoutId),this._disablePointerEventsTimeoutId=W(this._debounceScrollEndedCallback,e)}},{key:"_handleInvalidatedGridSize",value:function(){if("number"==typeof this._deferredInvalidateColumnIndex&&"number"==typeof this._deferredInvalidateRowIndex){var e=this._deferredInvalidateColumnIndex,t=this._deferredInvalidateRowIndex;this._deferredInvalidateColumnIndex=null,this._deferredInvalidateRowIndex=null,this.recomputeGridSize({columnIndex:e,rowIndex:t})}}},{key:"_invokeOnScrollMemoizer",value:function(e){var t=this,n=e.scrollLeft,r=e.scrollTop,i=e.totalColumnsWidth,o=e.totalRowsHeight;this._onScrollMemoizer({callback:function(e){var n=e.scrollLeft,r=e.scrollTop,a=t.props,s=a.height;(0,a.onScroll)({clientHeight:s,clientWidth:a.width,scrollHeight:o,scrollLeft:n,scrollTop:r,scrollWidth:i})},indices:{scrollLeft:n,scrollTop:r}})}},{key:"_isScrolling",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:this.props,t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:this.state;return Object.hasOwnProperty.call(e,"isScrolling")?Boolean(e.isScrolling):Boolean(t.isScrolling)}},{key:"_maybeCallOnScrollbarPresenceChange",value:function(){if(this._scrollbarPresenceChanged){var e=this.props.onScrollbarPresenceChange;this._scrollbarPresenceChanged=!1,e({horizontal:this._horizontalScrollBarSize>0,size:this.state.instanceProps.scrollbarSize,vertical:this._verticalScrollBarSize>0})}}},{key:"scrollToPosition",value:function(e){var n=e.scrollLeft,r=e.scrollTop,i=t._getScrollToPositionStateUpdate({prevState:this.state,scrollLeft:n,scrollTop:r});i&&(i.needToResetStyleCache=!1,this.setState(i))}},{key:"_getCalculatedScrollLeft",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:this.props,n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:this.state;return t._getCalculatedScrollLeft(e,n)}},{key:"_updateScrollLeftForScrollToColumn",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:this.props,n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:this.state,r=t._getScrollLeftForScrollToColumnStateUpdate(e,n);r&&(r.needToResetStyleCache=!1,this.setState(r))}},{key:"_getCalculatedScrollTop",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:this.props,n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:this.state;return t._getCalculatedScrollTop(e,n)}},{key:"_resetStyleCache",value:function(){var e=this._styleCache,t=this._cellCache,n=this.props.isScrollingOptOut;this._cellCache={},this._styleCache={};for(var r=this._rowStartIndex;r<=this._rowStopIndex;r++)for(var i=this._columnStartIndex;i<=this._columnStopIndex;i++){var o="".concat(r,"-").concat(i);this._styleCache[o]=e[o],n&&(this._cellCache[o]=t[o])}}},{key:"_updateScrollTopForScrollToRow",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:this.props,n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:this.state,r=t._getScrollTopForScrollToRowStateUpdate(e,n);r&&(r.needToResetStyleCache=!1,this.setState(r))}}],[{key:"getDerivedStateFromProps",value:function(e,n){var r={};0===e.columnCount&&0!==n.scrollLeft||0===e.rowCount&&0!==n.scrollTop?(r.scrollLeft=0,r.scrollTop=0):(e.scrollLeft!==n.scrollLeft&&e.scrollToColumn<0||e.scrollTop!==n.scrollTop&&e.scrollToRow<0)&&Object.assign(r,t._getScrollToPositionStateUpdate({prevState:n,scrollLeft:e.scrollLeft,scrollTop:e.scrollTop}));var i,o,a=n.instanceProps;return r.needToResetStyleCache=!1,e.columnWidth===a.prevColumnWidth&&e.rowHeight===a.prevRowHeight||(r.needToResetStyleCache=!0),a.columnSizeAndPositionManager.configure({cellCount:e.columnCount,estimatedCellSize:t._getEstimatedColumnSize(e),cellSizeGetter:t._wrapSizeGetter(e.columnWidth)}),a.rowSizeAndPositionManager.configure({cellCount:e.rowCount,estimatedCellSize:t._getEstimatedRowSize(e),cellSizeGetter:t._wrapSizeGetter(e.rowHeight)}),0!==a.prevColumnCount&&0!==a.prevRowCount||(a.prevColumnCount=0,a.prevRowCount=0),e.autoHeight&&!1===e.isScrolling&&!0===a.prevIsScrolling&&Object.assign(r,{isScrolling:!1}),x({cellCount:a.prevColumnCount,cellSize:"number"==typeof a.prevColumnWidth?a.prevColumnWidth:null,computeMetadataCallback:function(){return a.columnSizeAndPositionManager.resetCell(0)},computeMetadataCallbackProps:e,nextCellsCount:e.columnCount,nextCellSize:"number"==typeof e.columnWidth?e.columnWidth:null,nextScrollToIndex:e.scrollToColumn,scrollToIndex:a.prevScrollToColumn,updateScrollOffsetForScrollToIndex:function(){i=t._getScrollLeftForScrollToColumnStateUpdate(e,n)}}),x({cellCount:a.prevRowCount,cellSize:"number"==typeof a.prevRowHeight?a.prevRowHeight:null,computeMetadataCallback:function(){return a.rowSizeAndPositionManager.resetCell(0)},computeMetadataCallbackProps:e,nextCellsCount:e.rowCount,nextCellSize:"number"==typeof e.rowHeight?e.rowHeight:null,nextScrollToIndex:e.scrollToRow,scrollToIndex:a.prevScrollToRow,updateScrollOffsetForScrollToIndex:function(){o=t._getScrollTopForScrollToRowStateUpdate(e,n)}}),a.prevColumnCount=e.columnCount,a.prevColumnWidth=e.columnWidth,a.prevIsScrolling=!0===e.isScrolling,a.prevRowCount=e.rowCount,a.prevRowHeight=e.rowHeight,a.prevScrollToColumn=e.scrollToColumn,a.prevScrollToRow=e.scrollToRow,a.scrollbarSize=e.getScrollbarSize(),void 0===a.scrollbarSize?(a.scrollbarSizeMeasured=!1,a.scrollbarSize=0):a.scrollbarSizeMeasured=!0,r.instanceProps=a,X({},r,{},i,{},o)}},{key:"_getEstimatedColumnSize",value:function(e){return"number"==typeof e.columnWidth?e.columnWidth:e.estimatedColumnSize}},{key:"_getEstimatedRowSize",value:function(e){return"number"==typeof e.rowHeight?e.rowHeight:e.estimatedRowSize}},{key:"_getScrollToPositionStateUpdate",value:function(e){var t=e.prevState,n=e.scrollLeft,r=e.scrollTop,i={scrollPositionChangeReason:Z};return"number"==typeof n&&n>=0&&(i.scrollDirectionHorizontal=n>t.scrollLeft?1:-1,i.scrollLeft=n),"number"==typeof r&&r>=0&&(i.scrollDirectionVertical=r>t.scrollTop?1:-1,i.scrollTop=r),"number"==typeof n&&n>=0&&n!==t.scrollLeft||"number"==typeof r&&r>=0&&r!==t.scrollTop?i:{}}},{key:"_wrapSizeGetter",value:function(e){return"function"==typeof e?e:function(){return e}}},{key:"_getCalculatedScrollLeft",value:function(e,t){var n=e.columnCount,r=e.height,i=e.scrollToAlignment,o=e.scrollToColumn,a=e.width,s=t.scrollLeft,l=t.instanceProps;if(n>0){var u=n-1,c=o<0?u:Math.min(u,o),d=l.rowSizeAndPositionManager.getTotalSize(),h=l.scrollbarSizeMeasured&&d>r?l.scrollbarSize:0;return l.columnSizeAndPositionManager.getUpdatedOffsetForIndex({align:i,containerSize:a-h,currentOffset:s,targetIndex:c})}return 0}},{key:"_getScrollLeftForScrollToColumnStateUpdate",value:function(e,n){var r=n.scrollLeft,i=t._getCalculatedScrollLeft(e,n);return"number"==typeof i&&i>=0&&r!==i?t._getScrollToPositionStateUpdate({prevState:n,scrollLeft:i,scrollTop:-1}):{}}},{key:"_getCalculatedScrollTop",value:function(e,t){var n=e.height,r=e.rowCount,i=e.scrollToAlignment,o=e.scrollToRow,a=e.width,s=t.scrollTop,l=t.instanceProps;if(r>0){var u=r-1,c=o<0?u:Math.min(u,o),d=l.columnSizeAndPositionManager.getTotalSize(),h=l.scrollbarSizeMeasured&&d>a?l.scrollbarSize:0;return l.rowSizeAndPositionManager.getUpdatedOffsetForIndex({align:i,containerSize:n-h,currentOffset:s,targetIndex:c})}return 0}},{key:"_getScrollTopForScrollToRowStateUpdate",value:function(e,n){var r=n.scrollTop,i=t._getCalculatedScrollTop(e,n);return"number"==typeof i&&i>=0&&r!==i?t._getScrollToPositionStateUpdate({prevState:n,scrollLeft:-1,scrollTop:i}):{}}}]),t}(y.PureComponent),m()(z,"propTypes",null),F);m()(Q,"defaultProps",{"aria-label":"grid","aria-readonly":!0,autoContainerWidth:!1,autoHeight:!1,autoWidth:!1,cellRangeRenderer:function(e){for(var t=e.cellCache,n=e.cellRenderer,r=e.columnSizeAndPositionManager,i=e.columnStartIndex,o=e.columnStopIndex,a=e.deferredMeasurementCache,s=e.horizontalOffsetAdjustment,l=e.isScrolling,u=e.isScrollingOptOut,c=e.parent,d=e.rowSizeAndPositionManager,h=e.rowStartIndex,f=e.rowStopIndex,p=e.styleCache,g=e.verticalOffsetAdjustment,v=e.visibleColumnIndices,m=e.visibleRowIndices,y=[],b=r.areOffsetsAdjusted()||d.areOffsetsAdjusted(),C=!l&&!b,A=h;A<=f;A++)for(var _=d.getSizeAndPositionOfCell(A),I=i;I<=o;I++){var x=r.getSizeAndPositionOfCell(I),w=I>=v.start&&I<=v.stop&&A>=m.start&&A<=m.stop,M="".concat(A,"-").concat(I),S=void 0;C&&p[M]?S=p[M]:a&&!a.has(A,I)?S={height:"auto",left:0,position:"absolute",top:0,width:"auto"}:(S={height:_.size,left:x.offset+s,position:"absolute",top:_.offset+g,width:x.size},p[M]=S);var T={columnIndex:I,isScrolling:l,isVisible:w,key:M,parent:c,rowIndex:A,style:S},E=void 0;!u&&!l||s||g?E=n(T):(t[M]||(t[M]=n(T)),E=t[M]),null!=E&&!1!==E&&y.push(E)}return y},containerRole:"rowgroup",containerStyle:{},estimatedColumnSize:100,estimatedRowSize:30,getScrollbarSize:R,noContentRenderer:function(){return null},onScroll:function(){},onScrollbarPresenceChange:function(){},onSectionRendered:function(){},overscanColumnCount:0,overscanIndicesGetter:function(e){var t=e.cellCount,n=e.overscanCellsCount,r=e.scrollDirection,i=e.startIndex,o=e.stopIndex;return 1===r?{overscanStartIndex:Math.max(0,i),overscanStopIndex:Math.min(t-1,o+n)}:{overscanStartIndex:Math.max(0,i-n),overscanStopIndex:Math.min(t-1,o)}},overscanRowCount:10,role:"grid",scrollingResetTimeInterval:150,scrollToAlignment:"auto",scrollToColumn:-1,scrollToRow:-1,style:{},tabIndex:0,isScrollingOptOut:!1}),Object(C.polyfill)(Q);var K=Q;function J(e){var t=e.cellCount,n=e.overscanCellsCount,r=e.scrollDirection,i=e.startIndex,o=e.stopIndex;return n=Math.max(1,n),1===r?{overscanStartIndex:Math.max(0,i-1),overscanStopIndex:Math.min(t-1,o+n)}:{overscanStartIndex:Math.max(0,i-n),overscanStopIndex:Math.min(t-1,o+1)}}var $,ee;function te(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),n.push.apply(n,r)}return n}var ne=(ee=$=function(e){function t(){var e,n;o()(this,t);for(var r=arguments.length,i=new Array(r),a=0;a<r;a++)i[a]=arguments[a];return n=u()(this,(e=d()(t)).call.apply(e,[this].concat(i))),m()(f()(n),"state",{scrollToColumn:0,scrollToRow:0,instanceProps:{prevScrollToColumn:0,prevScrollToRow:0}}),m()(f()(n),"_columnStartIndex",0),m()(f()(n),"_columnStopIndex",0),m()(f()(n),"_rowStartIndex",0),m()(f()(n),"_rowStopIndex",0),m()(f()(n),"_onKeyDown",(function(e){var t=n.props,r=t.columnCount,i=t.disabled,o=t.mode,a=t.rowCount;if(!i){var s=n._getScrollState(),l=s.scrollToColumn,u=s.scrollToRow,c=n._getScrollState(),d=c.scrollToColumn,h=c.scrollToRow;switch(e.key){case"ArrowDown":h="cells"===o?Math.min(h+1,a-1):Math.min(n._rowStopIndex+1,a-1);break;case"ArrowLeft":d="cells"===o?Math.max(d-1,0):Math.max(n._columnStartIndex-1,0);break;case"ArrowRight":d="cells"===o?Math.min(d+1,r-1):Math.min(n._columnStopIndex+1,r-1);break;case"ArrowUp":h="cells"===o?Math.max(h-1,0):Math.max(n._rowStartIndex-1,0)}d===l&&h===u||(e.preventDefault(),n._updateScrollState({scrollToColumn:d,scrollToRow:h}))}})),m()(f()(n),"_onSectionRendered",(function(e){var t=e.columnStartIndex,r=e.columnStopIndex,i=e.rowStartIndex,o=e.rowStopIndex;n._columnStartIndex=t,n._columnStopIndex=r,n._rowStartIndex=i,n._rowStopIndex=o})),n}return g()(t,e),s()(t,[{key:"setScrollIndexes",value:function(e){var t=e.scrollToColumn,n=e.scrollToRow;this.setState({scrollToRow:n,scrollToColumn:t})}},{key:"render",value:function(){var e=this.props,t=e.className,n=e.children,r=this._getScrollState(),i=r.scrollToColumn,o=r.scrollToRow;return y.createElement("div",{className:t,onKeyDown:this._onKeyDown},n({onSectionRendered:this._onSectionRendered,scrollToColumn:i,scrollToRow:o}))}},{key:"_getScrollState",value:function(){return this.props.isControlled?this.props:this.state}},{key:"_updateScrollState",value:function(e){var t=e.scrollToColumn,n=e.scrollToRow,r=this.props,i=r.isControlled,o=r.onScrollToChange;"function"==typeof o&&o({scrollToColumn:t,scrollToRow:n}),i||this.setState({scrollToColumn:t,scrollToRow:n})}}],[{key:"getDerivedStateFromProps",value:function(e,t){return e.isControlled?{}:e.scrollToColumn!==t.instanceProps.prevScrollToColumn||e.scrollToRow!==t.instanceProps.prevScrollToRow?function(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?te(n,!0).forEach((function(t){m()(e,t,n[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):te(n).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))}))}return e}({},t,{scrollToColumn:e.scrollToColumn,scrollToRow:e.scrollToRow,instanceProps:{prevScrollToColumn:e.scrollToColumn,prevScrollToRow:e.scrollToRow}}):{}}}]),t}(y.PureComponent),m()($,"propTypes",null),ee);m()(ne,"defaultProps",{disabled:!1,isControlled:!1,mode:"edges",scrollToColumn:0,scrollToRow:0}),Object(C.polyfill)(ne);var re,ie,oe=n(458);function ae(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),n.push.apply(n,r)}return n}function se(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?ae(n,!0).forEach((function(t){m()(e,t,n[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):ae(n).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))}))}return e}var le=(ie=re=function(e){function t(){var e,n;o()(this,t);for(var r=arguments.length,i=new Array(r),a=0;a<r;a++)i[a]=arguments[a];return n=u()(this,(e=d()(t)).call.apply(e,[this].concat(i))),m()(f()(n),"state",{height:n.props.defaultHeight||0,width:n.props.defaultWidth||0}),m()(f()(n),"_parentNode",void 0),m()(f()(n),"_autoSizer",void 0),m()(f()(n),"_window",void 0),m()(f()(n),"_detectElementResize",void 0),m()(f()(n),"_onResize",(function(){var e=n.props,t=e.disableHeight,r=e.disableWidth,i=e.onResize;if(n._parentNode){var o=n._parentNode.offsetHeight||0,a=n._parentNode.offsetWidth||0,s=(n._window||window).getComputedStyle(n._parentNode)||{},l=parseInt(s.paddingLeft,10)||0,u=parseInt(s.paddingRight,10)||0,c=parseInt(s.paddingTop,10)||0,d=parseInt(s.paddingBottom,10)||0,h=o-c-d,f=a-l-u;(!t&&n.state.height!==h||!r&&n.state.width!==f)&&(n.setState({height:o-c-d,width:a-l-u}),i({height:o,width:a}))}})),m()(f()(n),"_setRef",(function(e){n._autoSizer=e})),n}return g()(t,e),s()(t,[{key:"componentDidMount",value:function(){var e=this.props.nonce;this._autoSizer&&this._autoSizer.parentNode&&this._autoSizer.parentNode.ownerDocument&&this._autoSizer.parentNode.ownerDocument.defaultView&&this._autoSizer.parentNode instanceof this._autoSizer.parentNode.ownerDocument.defaultView.HTMLElement&&(this._parentNode=this._autoSizer.parentNode,this._window=this._autoSizer.parentNode.ownerDocument.defaultView,this._detectElementResize=Object(oe.a)(e,this._window),this._detectElementResize.addResizeListener(this._parentNode,this._onResize),this._onResize())}},{key:"componentWillUnmount",value:function(){this._detectElementResize&&this._parentNode&&this._detectElementResize.removeResizeListener(this._parentNode,this._onResize)}},{key:"render",value:function(){var e=this.props,t=e.children,n=e.className,r=e.disableHeight,i=e.disableWidth,o=e.style,a=this.state,s=a.height,l=a.width,u={overflow:"visible"},c={};return r||(u.height=0,c.height=s),i||(u.width=0,c.width=l),y.createElement("div",{className:n,ref:this._setRef,style:se({},u,{},o)},t(c))}}]),t}(y.Component),m()(re,"propTypes",null),ie);m()(le,"defaultProps",{onResize:function(){},disableHeight:!1,disableWidth:!1,style:{}});var ue,ce,de=n(22),he=(ce=ue=function(e){function t(){var e,n;o()(this,t);for(var r=arguments.length,i=new Array(r),a=0;a<r;a++)i[a]=arguments[a];return n=u()(this,(e=d()(t)).call.apply(e,[this].concat(i))),m()(f()(n),"_child",void 0),m()(f()(n),"_measure",(function(){var e=n.props,t=e.cache,r=e.columnIndex,i=void 0===r?0:r,o=e.parent,a=e.rowIndex,s=void 0===a?n.props.index||0:a,l=n._getCellMeasurements(),u=l.height,c=l.width;u===t.getHeight(s,i)&&c===t.getWidth(s,i)||(t.set(s,i,c,u),o&&"function"==typeof o.recomputeGridSize&&o.recomputeGridSize({columnIndex:i,rowIndex:s}))})),m()(f()(n),"_registerChild",(function(e){!e||e instanceof Element||console.warn("CellMeasurer registerChild expects to be passed Element or null"),n._child=e,e&&n._maybeMeasureCell()})),n}return g()(t,e),s()(t,[{key:"componentDidMount",value:function(){this._maybeMeasureCell()}},{key:"componentDidUpdate",value:function(){this._maybeMeasureCell()}},{key:"render",value:function(){var e=this.props.children;return"function"==typeof e?e({measure:this._measure,registerChild:this._registerChild}):e}},{key:"_getCellMeasurements",value:function(){var e=this.props.cache,t=this._child||Object(de.findDOMNode)(this);if(t&&t.ownerDocument&&t.ownerDocument.defaultView&&t instanceof t.ownerDocument.defaultView.HTMLElement){var n=t.style.width,r=t.style.height;e.hasFixedWidth()||(t.style.width="auto"),e.hasFixedHeight()||(t.style.height="auto");var i=Math.ceil(t.offsetHeight),o=Math.ceil(t.offsetWidth);return n&&(t.style.width=n),r&&(t.style.height=r),{height:i,width:o}}return{height:0,width:0}}},{key:"_maybeMeasureCell",value:function(){var e=this.props,t=e.cache,n=e.columnIndex,r=void 0===n?0:n,i=e.parent,o=e.rowIndex,a=void 0===o?this.props.index||0:o;if(!t.has(a,r)){var s=this._getCellMeasurements(),l=s.height,u=s.width;t.set(a,r,u,l),i&&"function"==typeof i.invalidateCellSizeAfterRender&&i.invalidateCellSizeAfterRender({columnIndex:r,rowIndex:a})}}}]),t}(y.PureComponent),m()(ue,"propTypes",null),ce);m()(he,"__internalCellMeasurerFlag",!1);function fe(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),n.push.apply(n,r)}return n}function pe(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?fe(n,!0).forEach((function(t){m()(e,t,n[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):fe(n).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))}))}return e}var ge="observed",ve="requested",me=function(e){function t(){var e,n;o()(this,t);for(var r=arguments.length,i=new Array(r),a=0;a<r;a++)i[a]=arguments[a];return n=u()(this,(e=d()(t)).call.apply(e,[this].concat(i))),m()(f()(n),"state",{isScrolling:!1,scrollLeft:0,scrollTop:0}),m()(f()(n),"_calculateSizeAndPositionDataOnNextUpdate",!1),m()(f()(n),"_onSectionRenderedMemoizer",L()),m()(f()(n),"_onScrollMemoizer",L(!1)),m()(f()(n),"_invokeOnSectionRenderedHelper",(function(){var e=n.props,t=e.cellLayoutManager,r=e.onSectionRendered;n._onSectionRenderedMemoizer({callback:r,indices:{indices:t.getLastRenderedIndices()}})})),m()(f()(n),"_setScrollingContainerRef",(function(e){n._scrollingContainer=e})),m()(f()(n),"_updateScrollPositionForScrollToCell",(function(){var e=n.props,t=e.cellLayoutManager,r=e.height,i=e.scrollToAlignment,o=e.scrollToCell,a=e.width,s=n.state,l=s.scrollLeft,u=s.scrollTop;if(o>=0){var c=t.getScrollPositionForCell({align:i,cellIndex:o,height:r,scrollLeft:l,scrollTop:u,width:a});c.scrollLeft===l&&c.scrollTop===u||n._setScrollPosition(c)}})),m()(f()(n),"_onScroll",(function(e){if(e.target===n._scrollingContainer){n._enablePointerEventsAfterDelay();var t=n.props,r=t.cellLayoutManager,i=t.height,o=t.isScrollingChange,a=t.width,s=n._scrollbarSize,l=r.getTotalSize(),u=l.height,c=l.width,d=Math.max(0,Math.min(c-a+s,e.target.scrollLeft)),h=Math.max(0,Math.min(u-i+s,e.target.scrollTop));if(n.state.scrollLeft!==d||n.state.scrollTop!==h){var f=e.cancelable?ge:ve;n.state.isScrolling||o(!0),n.setState({isScrolling:!0,scrollLeft:d,scrollPositionChangeReason:f,scrollTop:h})}n._invokeOnScrollMemoizer({scrollLeft:d,scrollTop:h,totalWidth:c,totalHeight:u})}})),n._scrollbarSize=R(),void 0===n._scrollbarSize?(n._scrollbarSizeMeasured=!1,n._scrollbarSize=0):n._scrollbarSizeMeasured=!0,n}return g()(t,e),s()(t,[{key:"recomputeCellSizesAndPositions",value:function(){this._calculateSizeAndPositionDataOnNextUpdate=!0,this.forceUpdate()}},{key:"componentDidMount",value:function(){var e=this.props,t=e.cellLayoutManager,n=e.scrollLeft,r=e.scrollToCell,i=e.scrollTop;this._scrollbarSizeMeasured||(this._scrollbarSize=R(),this._scrollbarSizeMeasured=!0,this.setState({})),r>=0?this._updateScrollPositionForScrollToCell():(n>=0||i>=0)&&this._setScrollPosition({scrollLeft:n,scrollTop:i}),this._invokeOnSectionRenderedHelper();var o=t.getTotalSize(),a=o.height,s=o.width;this._invokeOnScrollMemoizer({scrollLeft:n||0,scrollTop:i||0,totalHeight:a,totalWidth:s})}},{key:"componentDidUpdate",value:function(e,t){var n=this.props,r=n.height,i=n.scrollToAlignment,o=n.scrollToCell,a=n.width,s=this.state,l=s.scrollLeft,u=s.scrollPositionChangeReason,c=s.scrollTop;u===ve&&(l>=0&&l!==t.scrollLeft&&l!==this._scrollingContainer.scrollLeft&&(this._scrollingContainer.scrollLeft=l),c>=0&&c!==t.scrollTop&&c!==this._scrollingContainer.scrollTop&&(this._scrollingContainer.scrollTop=c)),r===e.height&&i===e.scrollToAlignment&&o===e.scrollToCell&&a===e.width||this._updateScrollPositionForScrollToCell(),this._invokeOnSectionRenderedHelper()}},{key:"componentWillUnmount",value:function(){this._disablePointerEventsTimeoutId&&clearTimeout(this._disablePointerEventsTimeoutId)}},{key:"render",value:function(){var e=this.props,t=e.autoHeight,n=e.cellCount,r=e.cellLayoutManager,i=e.className,o=e.height,a=e.horizontalOverscanSize,s=e.id,l=e.noContentRenderer,u=e.style,c=e.verticalOverscanSize,d=e.width,h=this.state,f=h.isScrolling,p=h.scrollLeft,g=h.scrollTop;(this._lastRenderedCellCount!==n||this._lastRenderedCellLayoutManager!==r||this._calculateSizeAndPositionDataOnNextUpdate)&&(this._lastRenderedCellCount=n,this._lastRenderedCellLayoutManager=r,this._calculateSizeAndPositionDataOnNextUpdate=!1,r.calculateSizeAndPositionData());var v=r.getTotalSize(),m=v.height,b=v.width,C=Math.max(0,p-a),A=Math.max(0,g-c),_=Math.min(b,p+d+a),x=Math.min(m,g+o+c),w=o>0&&d>0?r.cellRenderers({height:x-A,isScrolling:f,width:_-C,x:C,y:A}):[],M={boxSizing:"border-box",direction:"ltr",height:t?"auto":o,position:"relative",WebkitOverflowScrolling:"touch",width:d,willChange:"transform"},S=m>o?this._scrollbarSize:0,T=b>d?this._scrollbarSize:0;return M.overflowX=b+S<=d?"hidden":"auto",M.overflowY=m+T<=o?"hidden":"auto",y.createElement("div",{ref:this._setScrollingContainerRef,"aria-label":this.props["aria-label"],className:Object(I.a)("ReactVirtualized__Collection",i),id:s,onScroll:this._onScroll,role:"grid",style:pe({},M,{},u),tabIndex:0},n>0&&y.createElement("div",{className:"ReactVirtualized__Collection__innerScrollContainer",style:{height:m,maxHeight:m,maxWidth:b,overflow:"hidden",pointerEvents:f?"none":"",width:b}},w),0===n&&l())}},{key:"_enablePointerEventsAfterDelay",value:function(){var e=this;this._disablePointerEventsTimeoutId&&clearTimeout(this._disablePointerEventsTimeoutId),this._disablePointerEventsTimeoutId=setTimeout((function(){(0,e.props.isScrollingChange)(!1),e._disablePointerEventsTimeoutId=null,e.setState({isScrolling:!1})}),150)}},{key:"_invokeOnScrollMemoizer",value:function(e){var t=this,n=e.scrollLeft,r=e.scrollTop,i=e.totalHeight,o=e.totalWidth;this._onScrollMemoizer({callback:function(e){var n=e.scrollLeft,r=e.scrollTop,a=t.props,s=a.height;(0,a.onScroll)({clientHeight:s,clientWidth:a.width,scrollHeight:i,scrollLeft:n,scrollTop:r,scrollWidth:o})},indices:{scrollLeft:n,scrollTop:r}})}},{key:"_setScrollPosition",value:function(e){var t=e.scrollLeft,n=e.scrollTop,r={scrollPositionChangeReason:ve};t>=0&&(r.scrollLeft=t),n>=0&&(r.scrollTop=n),(t>=0&&t!==this.state.scrollLeft||n>=0&&n!==this.state.scrollTop)&&this.setState(r)}}],[{key:"getDerivedStateFromProps",value:function(e,t){return 0!==e.cellCount||0===t.scrollLeft&&0===t.scrollTop?e.scrollLeft!==t.scrollLeft||e.scrollTop!==t.scrollTop?{scrollLeft:null!=e.scrollLeft?e.scrollLeft:t.scrollLeft,scrollTop:null!=e.scrollTop?e.scrollTop:t.scrollTop,scrollPositionChangeReason:ve}:null:{scrollLeft:0,scrollTop:0,scrollPositionChangeReason:ve}}}]),t}(y.PureComponent);m()(me,"defaultProps",{"aria-label":"grid",horizontalOverscanSize:0,noContentRenderer:function(){return null},onScroll:function(){return null},onSectionRendered:function(){return null},scrollToAlignment:"auto",scrollToCell:-1,style:{},verticalOverscanSize:0}),me.propTypes={},Object(C.polyfill)(me);var ye=me,be=function(){function e(t){var n=t.height,r=t.width,i=t.x,a=t.y;o()(this,e),this.height=n,this.width=r,this.x=i,this.y=a,this._indexMap={},this._indices=[]}return s()(e,[{key:"addCellIndex",value:function(e){var t=e.index;this._indexMap[t]||(this._indexMap[t]=!0,this._indices.push(t))}},{key:"getCellIndices",value:function(){return this._indices}},{key:"toString",value:function(){return"".concat(this.x,",").concat(this.y," ").concat(this.width,"x").concat(this.height)}}]),e}(),Ce=function(){function e(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:100;o()(this,e),this._sectionSize=t,this._cellMetadata=[],this._sections={}}return s()(e,[{key:"getCellIndices",value:function(e){var t=e.height,n=e.width,r=e.x,i=e.y,o={};return this.getSections({height:t,width:n,x:r,y:i}).forEach((function(e){return e.getCellIndices().forEach((function(e){o[e]=e}))})),Object.keys(o).map((function(e){return o[e]}))}},{key:"getCellMetadata",value:function(e){var t=e.index;return this._cellMetadata[t]}},{key:"getSections",value:function(e){for(var t=e.height,n=e.width,r=e.x,i=e.y,o=Math.floor(r/this._sectionSize),a=Math.floor((r+n-1)/this._sectionSize),s=Math.floor(i/this._sectionSize),l=Math.floor((i+t-1)/this._sectionSize),u=[],c=o;c<=a;c++)for(var d=s;d<=l;d++){var h="".concat(c,".").concat(d);this._sections[h]||(this._sections[h]=new be({height:this._sectionSize,width:this._sectionSize,x:c*this._sectionSize,y:d*this._sectionSize})),u.push(this._sections[h])}return u}},{key:"getTotalSectionCount",value:function(){return Object.keys(this._sections).length}},{key:"toString",value:function(){var e=this;return Object.keys(this._sections).map((function(t){return e._sections[t].toString()}))}},{key:"registerCell",value:function(e){var t=e.cellMetadatum,n=e.index;this._cellMetadata[n]=t,this.getSections(t).forEach((function(e){return e.addCellIndex({index:n})}))}}]),e}();function Ae(e){var t=e.align,n=void 0===t?"auto":t,r=e.cellOffset,i=e.cellSize,o=e.containerSize,a=e.currentOffset,s=r,l=s-o+i;switch(n){case"start":return s;case"end":return l;case"center":return s-(o-i)/2;default:return Math.max(l,Math.min(s,a))}}var _e=function(e){function t(e,n){var r;return o()(this,t),(r=u()(this,d()(t).call(this,e,n)))._cellMetadata=[],r._lastRenderedCellIndices=[],r._cellCache=[],r._isScrollingChange=r._isScrollingChange.bind(f()(r)),r._setCollectionViewRef=r._setCollectionViewRef.bind(f()(r)),r}return g()(t,e),s()(t,[{key:"forceUpdate",value:function(){void 0!==this._collectionView&&this._collectionView.forceUpdate()}},{key:"recomputeCellSizesAndPositions",value:function(){this._cellCache=[],this._collectionView.recomputeCellSizesAndPositions()}},{key:"render",value:function(){var e=_()({},this.props);return y.createElement(ye,_()({cellLayoutManager:this,isScrollingChange:this._isScrollingChange,ref:this._setCollectionViewRef},e))}},{key:"calculateSizeAndPositionData",value:function(){var e=this.props,t=function(e){for(var t=e.cellCount,n=e.cellSizeAndPositionGetter,r=e.sectionSize,i=[],o=new Ce(r),a=0,s=0,l=0;l<t;l++){var u=n({index:l});if(null==u.height||isNaN(u.height)||null==u.width||isNaN(u.width)||null==u.x||isNaN(u.x)||null==u.y||isNaN(u.y))throw Error("Invalid metadata returned for cell ".concat(l,":\n        x:").concat(u.x,", y:").concat(u.y,", width:").concat(u.width,", height:").concat(u.height));a=Math.max(a,u.y+u.height),s=Math.max(s,u.x+u.width),i[l]=u,o.registerCell({cellMetadatum:u,index:l})}return{cellMetadata:i,height:a,sectionManager:o,width:s}}({cellCount:e.cellCount,cellSizeAndPositionGetter:e.cellSizeAndPositionGetter,sectionSize:e.sectionSize});this._cellMetadata=t.cellMetadata,this._sectionManager=t.sectionManager,this._height=t.height,this._width=t.width}},{key:"getLastRenderedIndices",value:function(){return this._lastRenderedCellIndices}},{key:"getScrollPositionForCell",value:function(e){var t=e.align,n=e.cellIndex,r=e.height,i=e.scrollLeft,o=e.scrollTop,a=e.width,s=this.props.cellCount;if(n>=0&&n<s){var l=this._cellMetadata[n];i=Ae({align:t,cellOffset:l.x,cellSize:l.width,containerSize:a,currentOffset:i,targetIndex:n}),o=Ae({align:t,cellOffset:l.y,cellSize:l.height,containerSize:r,currentOffset:o,targetIndex:n})}return{scrollLeft:i,scrollTop:o}}},{key:"getTotalSize",value:function(){return{height:this._height,width:this._width}}},{key:"cellRenderers",value:function(e){var t=this,n=e.height,r=e.isScrolling,i=e.width,o=e.x,a=e.y,s=this.props,l=s.cellGroupRenderer,u=s.cellRenderer;return this._lastRenderedCellIndices=this._sectionManager.getCellIndices({height:n,width:i,x:o,y:a}),l({cellCache:this._cellCache,cellRenderer:u,cellSizeAndPositionGetter:function(e){var n=e.index;return t._sectionManager.getCellMetadata({index:n})},indices:this._lastRenderedCellIndices,isScrolling:r})}},{key:"_isScrollingChange",value:function(e){e||(this._cellCache=[])}},{key:"_setCollectionViewRef",value:function(e){this._collectionView=e}}]),t}(y.PureComponent);m()(_e,"defaultProps",{"aria-label":"grid",cellGroupRenderer:function(e){var t=e.cellCache,n=e.cellRenderer,r=e.cellSizeAndPositionGetter,i=e.indices,o=e.isScrolling;return i.map((function(e){var i=r({index:e}),a={index:e,isScrolling:o,key:e,style:{height:i.height,left:i.x,position:"absolute",top:i.y,width:i.width}};return o?(e in t||(t[e]=n(a)),t[e]):n(a)})).filter((function(e){return!!e}))}}),_e.propTypes={};var Ie=function(e){function t(e,n){var r;return o()(this,t),(r=u()(this,d()(t).call(this,e,n)))._registerChild=r._registerChild.bind(f()(r)),r}return g()(t,e),s()(t,[{key:"componentDidUpdate",value:function(e){var t=this.props,n=t.columnMaxWidth,r=t.columnMinWidth,i=t.columnCount,o=t.width;n===e.columnMaxWidth&&r===e.columnMinWidth&&i===e.columnCount&&o===e.width||this._registeredChild&&this._registeredChild.recomputeGridSize()}},{key:"render",value:function(){var e=this.props,t=e.children,n=e.columnMaxWidth,r=e.columnMinWidth,i=e.columnCount,o=e.width,a=r||1,s=n?Math.min(n,o):o,l=o/i;return l=Math.max(a,l),l=Math.min(s,l),l=Math.floor(l),t({adjustedWidth:Math.min(o,l*i),columnWidth:l,getColumnWidth:function(){return l},registerChild:this._registerChild})}},{key:"_registerChild",value:function(e){if(e&&"function"!=typeof e.recomputeGridSize)throw Error("Unexpected child type registered; only Grid/MultiGrid children are supported.");this._registeredChild=e,this._registeredChild&&this._registeredChild.recomputeGridSize()}}]),t}(y.PureComponent);Ie.propTypes={};var xe=n(223),we=n.n(xe),Me=function(e){function t(e,n){var r;return o()(this,t),(r=u()(this,d()(t).call(this,e,n)))._loadMoreRowsMemoizer=L(),r._onRowsRendered=r._onRowsRendered.bind(f()(r)),r._registerChild=r._registerChild.bind(f()(r)),r}return g()(t,e),s()(t,[{key:"resetLoadMoreRowsCache",value:function(e){this._loadMoreRowsMemoizer=L(),e&&this._doStuff(this._lastRenderedStartIndex,this._lastRenderedStopIndex)}},{key:"render",value:function(){return(0,this.props.children)({onRowsRendered:this._onRowsRendered,registerChild:this._registerChild})}},{key:"_loadUnloadedRanges",value:function(e){var t=this,n=this.props.loadMoreRows;e.forEach((function(e){var r=n(e);r&&r.then((function(){var n,r,i,o,a;n={lastRenderedStartIndex:t._lastRenderedStartIndex,lastRenderedStopIndex:t._lastRenderedStopIndex,startIndex:e.startIndex,stopIndex:e.stopIndex},r=n.lastRenderedStartIndex,i=n.lastRenderedStopIndex,o=n.startIndex,a=n.stopIndex,o>i||a<r||t._registeredChild&&function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:0,n="function"==typeof e.recomputeGridSize?e.recomputeGridSize:e.recomputeRowHeights;n?n.call(e,t):e.forceUpdate()}(t._registeredChild,t._lastRenderedStartIndex)}))}))}},{key:"_onRowsRendered",value:function(e){var t=e.startIndex,n=e.stopIndex;this._lastRenderedStartIndex=t,this._lastRenderedStopIndex=n,this._doStuff(t,n)}},{key:"_doStuff",value:function(e,t){var n,r=this,i=this.props,o=i.isRowLoaded,a=i.minimumBatchSize,s=i.rowCount,l=i.threshold,u=function(e){for(var t=e.isRowLoaded,n=e.minimumBatchSize,r=e.rowCount,i=e.startIndex,o=e.stopIndex,a=[],s=null,l=null,u=i;u<=o;u++){t({index:u})?null!==l&&(a.push({startIndex:s,stopIndex:l}),s=l=null):(l=u,null===s&&(s=u))}if(null!==l){for(var c=Math.min(Math.max(l,s+n-1),r-1),d=l+1;d<=c&&!t({index:d});d++)l=d;a.push({startIndex:s,stopIndex:l})}if(a.length)for(var h=a[0];h.stopIndex-h.startIndex+1<n&&h.startIndex>0;){var f=h.startIndex-1;if(t({index:f}))break;h.startIndex=f}return a}({isRowLoaded:o,minimumBatchSize:a,rowCount:s,startIndex:Math.max(0,e-l),stopIndex:Math.min(s-1,t+l)}),c=(n=[]).concat.apply(n,we()(u.map((function(e){return[e.startIndex,e.stopIndex]}))));this._loadMoreRowsMemoizer({callback:function(){r._loadUnloadedRanges(u)},indices:{squashedUnloadedRanges:c}})}},{key:"_registerChild",value:function(e){this._registeredChild=e}}]),t}(y.PureComponent);m()(Me,"defaultProps",{minimumBatchSize:10,rowCount:0,threshold:15}),Me.propTypes={};var Se,Te,Ee=(Te=Se=function(e){function t(){var e,n;o()(this,t);for(var r=arguments.length,i=new Array(r),a=0;a<r;a++)i[a]=arguments[a];return n=u()(this,(e=d()(t)).call.apply(e,[this].concat(i))),m()(f()(n),"Grid",void 0),m()(f()(n),"_cellRenderer",(function(e){var t=e.parent,r=e.rowIndex,i=e.style,o=e.isScrolling,a=e.isVisible,s=e.key,l=n.props.rowRenderer,u=Object.getOwnPropertyDescriptor(i,"width");return u&&u.writable&&(i.width="100%"),l({index:r,style:i,isScrolling:o,isVisible:a,key:s,parent:t})})),m()(f()(n),"_setRef",(function(e){n.Grid=e})),m()(f()(n),"_onScroll",(function(e){var t=e.clientHeight,r=e.scrollHeight,i=e.scrollTop;(0,n.props.onScroll)({clientHeight:t,scrollHeight:r,scrollTop:i})})),m()(f()(n),"_onSectionRendered",(function(e){var t=e.rowOverscanStartIndex,r=e.rowOverscanStopIndex,i=e.rowStartIndex,o=e.rowStopIndex;(0,n.props.onRowsRendered)({overscanStartIndex:t,overscanStopIndex:r,startIndex:i,stopIndex:o})})),n}return g()(t,e),s()(t,[{key:"forceUpdateGrid",value:function(){this.Grid&&this.Grid.forceUpdate()}},{key:"getOffsetForRow",value:function(e){var t=e.alignment,n=e.index;return this.Grid?this.Grid.getOffsetForCell({alignment:t,rowIndex:n,columnIndex:0}).scrollTop:0}},{key:"invalidateCellSizeAfterRender",value:function(e){var t=e.columnIndex,n=e.rowIndex;this.Grid&&this.Grid.invalidateCellSizeAfterRender({rowIndex:n,columnIndex:t})}},{key:"measureAllRows",value:function(){this.Grid&&this.Grid.measureAllCells()}},{key:"recomputeGridSize",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},t=e.columnIndex,n=void 0===t?0:t,r=e.rowIndex,i=void 0===r?0:r;this.Grid&&this.Grid.recomputeGridSize({rowIndex:i,columnIndex:n})}},{key:"recomputeRowHeights",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:0;this.Grid&&this.Grid.recomputeGridSize({rowIndex:e,columnIndex:0})}},{key:"scrollToPosition",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:0;this.Grid&&this.Grid.scrollToPosition({scrollTop:e})}},{key:"scrollToRow",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:0;this.Grid&&this.Grid.scrollToCell({columnIndex:0,rowIndex:e})}},{key:"render",value:function(){var e=this.props,t=e.className,n=e.noRowsRenderer,r=e.scrollToIndex,i=e.width,o=Object(I.a)("ReactVirtualized__List",t);return y.createElement(K,_()({},this.props,{autoContainerWidth:!0,cellRenderer:this._cellRenderer,className:o,columnWidth:i,columnCount:1,noContentRenderer:n,onScroll:this._onScroll,onSectionRendered:this._onSectionRendered,ref:this._setRef,scrollToRow:r}))}}]),t}(y.PureComponent),m()(Se,"propTypes",null),Te);m()(Ee,"defaultProps",{autoHeight:!1,estimatedRowSize:30,onScroll:function(){},noRowsRenderer:function(){return null},onRowsRendered:function(){},overscanIndicesGetter:J,overscanRowCount:10,scrollToAlignment:"auto",scrollToIndex:-1,style:{}});var Oe=n(634),ke=n.n(Oe);var Le={ge:function(e,t,n,r,i){return"function"==typeof n?function(e,t,n,r,i){for(var o=n+1;t<=n;){var a=t+n>>>1;i(e[a],r)>=0?(o=a,n=a-1):t=a+1}return o}(e,void 0===r?0:0|r,void 0===i?e.length-1:0|i,t,n):function(e,t,n,r){for(var i=n+1;t<=n;){var o=t+n>>>1;e[o]>=r?(i=o,n=o-1):t=o+1}return i}(e,void 0===n?0:0|n,void 0===r?e.length-1:0|r,t)},gt:function(e,t,n,r,i){return"function"==typeof n?function(e,t,n,r,i){for(var o=n+1;t<=n;){var a=t+n>>>1;i(e[a],r)>0?(o=a,n=a-1):t=a+1}return o}(e,void 0===r?0:0|r,void 0===i?e.length-1:0|i,t,n):function(e,t,n,r){for(var i=n+1;t<=n;){var o=t+n>>>1;e[o]>r?(i=o,n=o-1):t=o+1}return i}(e,void 0===n?0:0|n,void 0===r?e.length-1:0|r,t)},lt:function(e,t,n,r,i){return"function"==typeof n?function(e,t,n,r,i){for(var o=t-1;t<=n;){var a=t+n>>>1;i(e[a],r)<0?(o=a,t=a+1):n=a-1}return o}(e,void 0===r?0:0|r,void 0===i?e.length-1:0|i,t,n):function(e,t,n,r){for(var i=t-1;t<=n;){var o=t+n>>>1;e[o]<r?(i=o,t=o+1):n=o-1}return i}(e,void 0===n?0:0|n,void 0===r?e.length-1:0|r,t)},le:function(e,t,n,r,i){return"function"==typeof n?function(e,t,n,r,i){for(var o=t-1;t<=n;){var a=t+n>>>1;i(e[a],r)<=0?(o=a,t=a+1):n=a-1}return o}(e,void 0===r?0:0|r,void 0===i?e.length-1:0|i,t,n):function(e,t,n,r){for(var i=t-1;t<=n;){var o=t+n>>>1;e[o]<=r?(i=o,t=o+1):n=o-1}return i}(e,void 0===n?0:0|n,void 0===r?e.length-1:0|r,t)},eq:function(e,t,n,r,i){return"function"==typeof n?function(e,t,n,r,i){for(;t<=n;){var o=t+n>>>1,a=i(e[o],r);if(0===a)return o;a<=0?t=o+1:n=o-1}return-1}(e,void 0===r?0:0|r,void 0===i?e.length-1:0|i,t,n):function(e,t,n,r){for(;t<=n;){var i=t+n>>>1,o=e[i];if(o===r)return i;o<=r?t=i+1:n=i-1}return-1}(e,void 0===n?0:0|n,void 0===r?e.length-1:0|r,t)}};function De(e,t,n,r,i){this.mid=e,this.left=t,this.right=n,this.leftPoints=r,this.rightPoints=i,this.count=(t?t.count:0)+(n?n.count:0)+r.length}var Pe=De.prototype;function Ne(e,t){e.mid=t.mid,e.left=t.left,e.right=t.right,e.leftPoints=t.leftPoints,e.rightPoints=t.rightPoints,e.count=t.count}function je(e,t){var n=We(t);e.mid=n.mid,e.left=n.left,e.right=n.right,e.leftPoints=n.leftPoints,e.rightPoints=n.rightPoints,e.count=n.count}function Re(e,t){var n=e.intervals([]);n.push(t),je(e,n)}function ze(e,t){var n=e.intervals([]),r=n.indexOf(t);return r<0?0:(n.splice(r,1),je(e,n),1)}function Fe(e,t,n){for(var r=0;r<e.length&&e[r][0]<=t;++r){var i=n(e[r]);if(i)return i}}function Be(e,t,n){for(var r=e.length-1;r>=0&&e[r][1]>=t;--r){var i=n(e[r]);if(i)return i}}function He(e,t){for(var n=0;n<e.length;++n){var r=t(e[n]);if(r)return r}}function Ue(e,t){return e-t}function Ve(e,t){var n=e[0]-t[0];return n||e[1]-t[1]}function Ye(e,t){var n=e[1]-t[1];return n||e[0]-t[0]}function We(e){if(0===e.length)return null;for(var t=[],n=0;n<e.length;++n)t.push(e[n][0],e[n][1]);t.sort(Ue);var r=t[t.length>>1],i=[],o=[],a=[];for(n=0;n<e.length;++n){var s=e[n];s[1]<r?i.push(s):r<s[0]?o.push(s):a.push(s)}var l=a,u=a.slice();return l.sort(Ve),u.sort(Ye),new De(r,We(i),We(o),l,u)}function Ge(e){this.root=e}Pe.intervals=function(e){return e.push.apply(e,this.leftPoints),this.left&&this.left.intervals(e),this.right&&this.right.intervals(e),e},Pe.insert=function(e){var t=this.count-this.leftPoints.length;if(this.count+=1,e[1]<this.mid)this.left?4*(this.left.count+1)>3*(t+1)?Re(this,e):this.left.insert(e):this.left=We([e]);else if(e[0]>this.mid)this.right?4*(this.right.count+1)>3*(t+1)?Re(this,e):this.right.insert(e):this.right=We([e]);else{var n=Le.ge(this.leftPoints,e,Ve),r=Le.ge(this.rightPoints,e,Ye);this.leftPoints.splice(n,0,e),this.rightPoints.splice(r,0,e)}},Pe.remove=function(e){var t=this.count-this.leftPoints;if(e[1]<this.mid)return this.left?4*(this.right?this.right.count:0)>3*(t-1)?ze(this,e):2===(o=this.left.remove(e))?(this.left=null,this.count-=1,1):(1===o&&(this.count-=1),o):0;if(e[0]>this.mid)return this.right?4*(this.left?this.left.count:0)>3*(t-1)?ze(this,e):2===(o=this.right.remove(e))?(this.right=null,this.count-=1,1):(1===o&&(this.count-=1),o):0;if(1===this.count)return this.leftPoints[0]===e?2:0;if(1===this.leftPoints.length&&this.leftPoints[0]===e){if(this.left&&this.right){for(var n=this,r=this.left;r.right;)n=r,r=r.right;if(n===this)r.right=this.right;else{var i=this.left,o=this.right;n.count-=r.count,n.right=r.left,r.left=i,r.right=o}Ne(this,r),this.count=(this.left?this.left.count:0)+(this.right?this.right.count:0)+this.leftPoints.length}else this.left?Ne(this,this.left):Ne(this,this.right);return 1}for(i=Le.ge(this.leftPoints,e,Ve);i<this.leftPoints.length&&this.leftPoints[i][0]===e[0];++i)if(this.leftPoints[i]===e){this.count-=1,this.leftPoints.splice(i,1);for(o=Le.ge(this.rightPoints,e,Ye);o<this.rightPoints.length&&this.rightPoints[o][1]===e[1];++o)if(this.rightPoints[o]===e)return this.rightPoints.splice(o,1),1}return 0},Pe.queryPoint=function(e,t){if(e<this.mid){if(this.left)if(n=this.left.queryPoint(e,t))return n;return Fe(this.leftPoints,e,t)}if(e>this.mid){var n;if(this.right)if(n=this.right.queryPoint(e,t))return n;return Be(this.rightPoints,e,t)}return He(this.leftPoints,t)},Pe.queryInterval=function(e,t,n){var r;if(e<this.mid&&this.left&&(r=this.left.queryInterval(e,t,n)))return r;if(t>this.mid&&this.right&&(r=this.right.queryInterval(e,t,n)))return r;return t<this.mid?Fe(this.leftPoints,t,n):e>this.mid?Be(this.rightPoints,e,n):He(this.leftPoints,n)};var Xe=Ge.prototype;Xe.insert=function(e){this.root?this.root.insert(e):this.root=new De(e[0],null,null,[e],[e])},Xe.remove=function(e){if(this.root){var t=this.root.remove(e);return 2===t&&(this.root=null),0!==t}return!1},Xe.queryPoint=function(e,t){if(this.root)return this.root.queryPoint(e,t)},Xe.queryInterval=function(e,t,n){if(e<=t&&this.root)return this.root.queryInterval(e,t,n)},Object.defineProperty(Xe,"count",{get:function(){return this.root?this.root.count:0}}),Object.defineProperty(Xe,"intervals",{get:function(){return this.root?this.root.intervals([]):[]}});var qe,Ze,Qe=function(){function e(){var t;o()(this,e),m()(this,"_columnSizeMap",{}),m()(this,"_intervalTree",t&&0!==t.length?new Ge(We(t)):new Ge(null)),m()(this,"_leftMap",{})}return s()(e,[{key:"estimateTotalHeight",value:function(e,t,n){var r=e-this.count;return this.tallestColumnSize+Math.ceil(r/t)*n}},{key:"range",value:function(e,t,n){var r=this;this._intervalTree.queryInterval(e,e+t,(function(e){var t=ke()(e,3),i=t[0],o=(t[1],t[2]);return n(o,r._leftMap[o],i)}))}},{key:"setPosition",value:function(e,t,n,r){this._intervalTree.insert([n,n+r,e]),this._leftMap[e]=t;var i=this._columnSizeMap,o=i[t];i[t]=void 0===o?n+r:Math.max(o,n+r)}},{key:"count",get:function(){return this._intervalTree.count}},{key:"shortestColumnSize",get:function(){var e=this._columnSizeMap,t=0;for(var n in e){var r=e[n];t=0===t?r:Math.min(t,r)}return t}},{key:"tallestColumnSize",get:function(){var e=this._columnSizeMap,t=0;for(var n in e){var r=e[n];t=Math.max(t,r)}return t}}]),e}();function Ke(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),n.push.apply(n,r)}return n}function Je(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?Ke(n,!0).forEach((function(t){m()(e,t,n[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):Ke(n).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))}))}return e}var $e=(Ze=qe=function(e){function t(){var e,n;o()(this,t);for(var r=arguments.length,i=new Array(r),a=0;a<r;a++)i[a]=arguments[a];return n=u()(this,(e=d()(t)).call.apply(e,[this].concat(i))),m()(f()(n),"state",{isScrolling:!1,scrollTop:0}),m()(f()(n),"_debounceResetIsScrollingId",void 0),m()(f()(n),"_invalidateOnUpdateStartIndex",null),m()(f()(n),"_invalidateOnUpdateStopIndex",null),m()(f()(n),"_positionCache",new Qe),m()(f()(n),"_startIndex",null),m()(f()(n),"_startIndexMemoized",null),m()(f()(n),"_stopIndex",null),m()(f()(n),"_stopIndexMemoized",null),m()(f()(n),"_debounceResetIsScrollingCallback",(function(){n.setState({isScrolling:!1})})),m()(f()(n),"_setScrollingContainerRef",(function(e){n._scrollingContainer=e})),m()(f()(n),"_onScroll",(function(e){var t=n.props.height,r=e.currentTarget.scrollTop,i=Math.min(Math.max(0,n._getEstimatedTotalHeight()-t),r);r===i&&(n._debounceResetIsScrolling(),n.state.scrollTop!==i&&n.setState({isScrolling:!0,scrollTop:i}))})),n}return g()(t,e),s()(t,[{key:"clearCellPositions",value:function(){this._positionCache=new Qe,this.forceUpdate()}},{key:"invalidateCellSizeAfterRender",value:function(e){var t=e.rowIndex;null===this._invalidateOnUpdateStartIndex?(this._invalidateOnUpdateStartIndex=t,this._invalidateOnUpdateStopIndex=t):(this._invalidateOnUpdateStartIndex=Math.min(this._invalidateOnUpdateStartIndex,t),this._invalidateOnUpdateStopIndex=Math.max(this._invalidateOnUpdateStopIndex,t))}},{key:"recomputeCellPositions",value:function(){var e=this._positionCache.count-1;this._positionCache=new Qe,this._populatePositionCache(0,e),this.forceUpdate()}},{key:"componentDidMount",value:function(){this._checkInvalidateOnUpdate(),this._invokeOnScrollCallback(),this._invokeOnCellsRenderedCallback()}},{key:"componentDidUpdate",value:function(e,t){this._checkInvalidateOnUpdate(),this._invokeOnScrollCallback(),this._invokeOnCellsRenderedCallback(),this.props.scrollTop!==e.scrollTop&&this._debounceResetIsScrolling()}},{key:"componentWillUnmount",value:function(){this._debounceResetIsScrollingId&&Y(this._debounceResetIsScrollingId)}},{key:"render",value:function(){var e,t=this,n=this.props,r=n.autoHeight,i=n.cellCount,o=n.cellMeasurerCache,a=n.cellRenderer,s=n.className,l=n.height,u=n.id,c=n.keyMapper,d=n.overscanByPixels,h=n.role,f=n.style,p=n.tabIndex,g=n.width,v=n.rowDirection,b=this.state,C=b.isScrolling,A=b.scrollTop,_=[],x=this._getEstimatedTotalHeight(),w=this._positionCache.shortestColumnSize,M=this._positionCache.count,S=0;if(this._positionCache.range(Math.max(0,A-d),l+2*d,(function(n,r,i){var s;void 0===e?(S=n,e=n):(S=Math.min(S,n),e=Math.max(e,n)),_.push(a({index:n,isScrolling:C,key:c(n),parent:t,style:(s={height:o.getHeight(n)},m()(s,"ltr"===v?"left":"right",r),m()(s,"position","absolute"),m()(s,"top",i),m()(s,"width",o.getWidth(n)),s)}))})),w<A+l+d&&M<i)for(var T=Math.min(i-M,Math.ceil((A+l+d-w)/o.defaultHeight*g/o.defaultWidth)),E=M;E<M+T;E++)e=E,_.push(a({index:E,isScrolling:C,key:c(E),parent:this,style:{width:o.getWidth(E)}}));return this._startIndex=S,this._stopIndex=e,y.createElement("div",{ref:this._setScrollingContainerRef,"aria-label":this.props["aria-label"],className:Object(I.a)("ReactVirtualized__Masonry",s),id:u,onScroll:this._onScroll,role:h,style:Je({boxSizing:"border-box",direction:"ltr",height:r?"auto":l,overflowX:"hidden",overflowY:x<l?"hidden":"auto",position:"relative",width:g,WebkitOverflowScrolling:"touch",willChange:"transform"},f),tabIndex:p},y.createElement("div",{className:"ReactVirtualized__Masonry__innerScrollContainer",style:{width:"100%",height:x,maxWidth:"100%",maxHeight:x,overflow:"hidden",pointerEvents:C?"none":"",position:"relative"}},_))}},{key:"_checkInvalidateOnUpdate",value:function(){if("number"==typeof this._invalidateOnUpdateStartIndex){var e=this._invalidateOnUpdateStartIndex,t=this._invalidateOnUpdateStopIndex;this._invalidateOnUpdateStartIndex=null,this._invalidateOnUpdateStopIndex=null,this._populatePositionCache(e,t),this.forceUpdate()}}},{key:"_debounceResetIsScrolling",value:function(){var e=this.props.scrollingResetTimeInterval;this._debounceResetIsScrollingId&&Y(this._debounceResetIsScrollingId),this._debounceResetIsScrollingId=W(this._debounceResetIsScrollingCallback,e)}},{key:"_getEstimatedTotalHeight",value:function(){var e=this.props,t=e.cellCount,n=e.cellMeasurerCache,r=e.width,i=Math.max(1,Math.floor(r/n.defaultWidth));return this._positionCache.estimateTotalHeight(t,i,n.defaultHeight)}},{key:"_invokeOnScrollCallback",value:function(){var e=this.props,t=e.height,n=e.onScroll,r=this.state.scrollTop;this._onScrollMemoized!==r&&(n({clientHeight:t,scrollHeight:this._getEstimatedTotalHeight(),scrollTop:r}),this._onScrollMemoized=r)}},{key:"_invokeOnCellsRenderedCallback",value:function(){this._startIndexMemoized===this._startIndex&&this._stopIndexMemoized===this._stopIndex||((0,this.props.onCellsRendered)({startIndex:this._startIndex,stopIndex:this._stopIndex}),this._startIndexMemoized=this._startIndex,this._stopIndexMemoized=this._stopIndex)}},{key:"_populatePositionCache",value:function(e,t){for(var n=this.props,r=n.cellMeasurerCache,i=n.cellPositioner,o=e;o<=t;o++){var a=i(o),s=a.left,l=a.top;this._positionCache.setPosition(o,s,l,r.getHeight(o))}}}],[{key:"getDerivedStateFromProps",value:function(e,t){return void 0!==e.scrollTop&&t.scrollTop!==e.scrollTop?{isScrolling:!0,scrollTop:e.scrollTop}:null}}]),t}(y.PureComponent),m()(qe,"propTypes",null),Ze);function et(){}m()($e,"defaultProps",{autoHeight:!1,keyMapper:function(e){return e},onCellsRendered:et,onScroll:et,overscanByPixels:20,role:"grid",scrollingResetTimeInterval:150,style:{},tabIndex:0,rowDirection:"ltr"});Object(C.polyfill)($e);var tt=function(){function e(){var t=this,n=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};o()(this,e),m()(this,"_cellMeasurerCache",void 0),m()(this,"_columnIndexOffset",void 0),m()(this,"_rowIndexOffset",void 0),m()(this,"columnWidth",(function(e){var n=e.index;t._cellMeasurerCache.columnWidth({index:n+t._columnIndexOffset})})),m()(this,"rowHeight",(function(e){var n=e.index;t._cellMeasurerCache.rowHeight({index:n+t._rowIndexOffset})}));var r=n.cellMeasurerCache,i=n.columnIndexOffset,a=void 0===i?0:i,s=n.rowIndexOffset,l=void 0===s?0:s;this._cellMeasurerCache=r,this._columnIndexOffset=a,this._rowIndexOffset=l}return s()(e,[{key:"clear",value:function(e,t){this._cellMeasurerCache.clear(e+this._rowIndexOffset,t+this._columnIndexOffset)}},{key:"clearAll",value:function(){this._cellMeasurerCache.clearAll()}},{key:"hasFixedHeight",value:function(){return this._cellMeasurerCache.hasFixedHeight()}},{key:"hasFixedWidth",value:function(){return this._cellMeasurerCache.hasFixedWidth()}},{key:"getHeight",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:0;return this._cellMeasurerCache.getHeight(e+this._rowIndexOffset,t+this._columnIndexOffset)}},{key:"getWidth",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:0;return this._cellMeasurerCache.getWidth(e+this._rowIndexOffset,t+this._columnIndexOffset)}},{key:"has",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:0;return this._cellMeasurerCache.has(e+this._rowIndexOffset,t+this._columnIndexOffset)}},{key:"set",value:function(e,t,n,r){this._cellMeasurerCache.set(e+this._rowIndexOffset,t+this._columnIndexOffset,n,r)}},{key:"defaultHeight",get:function(){return this._cellMeasurerCache.defaultHeight}},{key:"defaultWidth",get:function(){return this._cellMeasurerCache.defaultWidth}}]),e}();function nt(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),n.push.apply(n,r)}return n}function rt(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?nt(n,!0).forEach((function(t){m()(e,t,n[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):nt(n).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))}))}return e}var it=function(e){function t(e,n){var r;o()(this,t),r=u()(this,d()(t).call(this,e,n)),m()(f()(r),"state",{scrollLeft:0,scrollTop:0,scrollbarSize:0,showHorizontalScrollbar:!1,showVerticalScrollbar:!1}),m()(f()(r),"_deferredInvalidateColumnIndex",null),m()(f()(r),"_deferredInvalidateRowIndex",null),m()(f()(r),"_bottomLeftGridRef",(function(e){r._bottomLeftGrid=e})),m()(f()(r),"_bottomRightGridRef",(function(e){r._bottomRightGrid=e})),m()(f()(r),"_cellRendererBottomLeftGrid",(function(e){var t=e.rowIndex,n=M()(e,["rowIndex"]),i=r.props,o=i.cellRenderer,a=i.fixedRowCount;return t===i.rowCount-a?y.createElement("div",{key:n.key,style:rt({},n.style,{height:20})}):o(rt({},n,{parent:f()(r),rowIndex:t+a}))})),m()(f()(r),"_cellRendererBottomRightGrid",(function(e){var t=e.columnIndex,n=e.rowIndex,i=M()(e,["columnIndex","rowIndex"]),o=r.props,a=o.cellRenderer,s=o.fixedColumnCount,l=o.fixedRowCount;return a(rt({},i,{columnIndex:t+s,parent:f()(r),rowIndex:n+l}))})),m()(f()(r),"_cellRendererTopRightGrid",(function(e){var t=e.columnIndex,n=M()(e,["columnIndex"]),i=r.props,o=i.cellRenderer,a=i.columnCount,s=i.fixedColumnCount;return t===a-s?y.createElement("div",{key:n.key,style:rt({},n.style,{width:20})}):o(rt({},n,{columnIndex:t+s,parent:f()(r)}))})),m()(f()(r),"_columnWidthRightGrid",(function(e){var t=e.index,n=r.props,i=n.columnCount,o=n.fixedColumnCount,a=n.columnWidth,s=r.state,l=s.scrollbarSize;return s.showHorizontalScrollbar&&t===i-o?l:"function"==typeof a?a({index:t+o}):a})),m()(f()(r),"_onScroll",(function(e){var t=e.scrollLeft,n=e.scrollTop;r.setState({scrollLeft:t,scrollTop:n});var i=r.props.onScroll;i&&i(e)})),m()(f()(r),"_onScrollbarPresenceChange",(function(e){var t=e.horizontal,n=e.size,i=e.vertical,o=r.state,a=o.showHorizontalScrollbar,s=o.showVerticalScrollbar;if(t!==a||i!==s){r.setState({scrollbarSize:n,showHorizontalScrollbar:t,showVerticalScrollbar:i});var l=r.props.onScrollbarPresenceChange;"function"==typeof l&&l({horizontal:t,size:n,vertical:i})}})),m()(f()(r),"_onScrollLeft",(function(e){var t=e.scrollLeft;r._onScroll({scrollLeft:t,scrollTop:r.state.scrollTop})})),m()(f()(r),"_onScrollTop",(function(e){var t=e.scrollTop;r._onScroll({scrollTop:t,scrollLeft:r.state.scrollLeft})})),m()(f()(r),"_rowHeightBottomGrid",(function(e){var t=e.index,n=r.props,i=n.fixedRowCount,o=n.rowCount,a=n.rowHeight,s=r.state,l=s.scrollbarSize;return s.showVerticalScrollbar&&t===o-i?l:"function"==typeof a?a({index:t+i}):a})),m()(f()(r),"_topLeftGridRef",(function(e){r._topLeftGrid=e})),m()(f()(r),"_topRightGridRef",(function(e){r._topRightGrid=e}));var i=e.deferredMeasurementCache,a=e.fixedColumnCount,s=e.fixedRowCount;return r._maybeCalculateCachedStyles(!0),i&&(r._deferredMeasurementCacheBottomLeftGrid=s>0?new tt({cellMeasurerCache:i,columnIndexOffset:0,rowIndexOffset:s}):i,r._deferredMeasurementCacheBottomRightGrid=a>0||s>0?new tt({cellMeasurerCache:i,columnIndexOffset:a,rowIndexOffset:s}):i,r._deferredMeasurementCacheTopRightGrid=a>0?new tt({cellMeasurerCache:i,columnIndexOffset:a,rowIndexOffset:0}):i),r}return g()(t,e),s()(t,[{key:"forceUpdateGrids",value:function(){this._bottomLeftGrid&&this._bottomLeftGrid.forceUpdate(),this._bottomRightGrid&&this._bottomRightGrid.forceUpdate(),this._topLeftGrid&&this._topLeftGrid.forceUpdate(),this._topRightGrid&&this._topRightGrid.forceUpdate()}},{key:"invalidateCellSizeAfterRender",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},t=e.columnIndex,n=void 0===t?0:t,r=e.rowIndex,i=void 0===r?0:r;this._deferredInvalidateColumnIndex="number"==typeof this._deferredInvalidateColumnIndex?Math.min(this._deferredInvalidateColumnIndex,n):n,this._deferredInvalidateRowIndex="number"==typeof this._deferredInvalidateRowIndex?Math.min(this._deferredInvalidateRowIndex,i):i}},{key:"measureAllCells",value:function(){this._bottomLeftGrid&&this._bottomLeftGrid.measureAllCells(),this._bottomRightGrid&&this._bottomRightGrid.measureAllCells(),this._topLeftGrid&&this._topLeftGrid.measureAllCells(),this._topRightGrid&&this._topRightGrid.measureAllCells()}},{key:"recomputeGridSize",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},t=e.columnIndex,n=void 0===t?0:t,r=e.rowIndex,i=void 0===r?0:r,o=this.props,a=o.fixedColumnCount,s=o.fixedRowCount,l=Math.max(0,n-a),u=Math.max(0,i-s);this._bottomLeftGrid&&this._bottomLeftGrid.recomputeGridSize({columnIndex:n,rowIndex:u}),this._bottomRightGrid&&this._bottomRightGrid.recomputeGridSize({columnIndex:l,rowIndex:u}),this._topLeftGrid&&this._topLeftGrid.recomputeGridSize({columnIndex:n,rowIndex:i}),this._topRightGrid&&this._topRightGrid.recomputeGridSize({columnIndex:l,rowIndex:i}),this._leftGridWidth=null,this._topGridHeight=null,this._maybeCalculateCachedStyles(!0)}},{key:"componentDidMount",value:function(){var e=this.props,t=e.scrollLeft,n=e.scrollTop;if(t>0||n>0){var r={};t>0&&(r.scrollLeft=t),n>0&&(r.scrollTop=n),this.setState(r)}this._handleInvalidatedGridSize()}},{key:"componentDidUpdate",value:function(){this._handleInvalidatedGridSize()}},{key:"render",value:function(){var e=this.props,t=e.onScroll,n=e.onSectionRendered,r=(e.onScrollbarPresenceChange,e.scrollLeft,e.scrollToColumn),i=(e.scrollTop,e.scrollToRow),o=M()(e,["onScroll","onSectionRendered","onScrollbarPresenceChange","scrollLeft","scrollToColumn","scrollTop","scrollToRow"]);if(this._prepareForRender(),0===this.props.width||0===this.props.height)return null;var a=this.state,s=a.scrollLeft,l=a.scrollTop;return y.createElement("div",{style:this._containerOuterStyle},y.createElement("div",{style:this._containerTopStyle},this._renderTopLeftGrid(o),this._renderTopRightGrid(rt({},o,{onScroll:t,scrollLeft:s}))),y.createElement("div",{style:this._containerBottomStyle},this._renderBottomLeftGrid(rt({},o,{onScroll:t,scrollTop:l})),this._renderBottomRightGrid(rt({},o,{onScroll:t,onSectionRendered:n,scrollLeft:s,scrollToColumn:r,scrollToRow:i,scrollTop:l}))))}},{key:"_getBottomGridHeight",value:function(e){return e.height-this._getTopGridHeight(e)}},{key:"_getLeftGridWidth",value:function(e){var t=e.fixedColumnCount,n=e.columnWidth;if(null==this._leftGridWidth)if("function"==typeof n){for(var r=0,i=0;i<t;i++)r+=n({index:i});this._leftGridWidth=r}else this._leftGridWidth=n*t;return this._leftGridWidth}},{key:"_getRightGridWidth",value:function(e){return e.width-this._getLeftGridWidth(e)}},{key:"_getTopGridHeight",value:function(e){var t=e.fixedRowCount,n=e.rowHeight;if(null==this._topGridHeight)if("function"==typeof n){for(var r=0,i=0;i<t;i++)r+=n({index:i});this._topGridHeight=r}else this._topGridHeight=n*t;return this._topGridHeight}},{key:"_handleInvalidatedGridSize",value:function(){if("number"==typeof this._deferredInvalidateColumnIndex){var e=this._deferredInvalidateColumnIndex,t=this._deferredInvalidateRowIndex;this._deferredInvalidateColumnIndex=null,this._deferredInvalidateRowIndex=null,this.recomputeGridSize({columnIndex:e,rowIndex:t}),this.forceUpdate()}}},{key:"_maybeCalculateCachedStyles",value:function(e){var t=this.props,n=t.columnWidth,r=t.enableFixedColumnScroll,i=t.enableFixedRowScroll,o=t.height,a=t.fixedColumnCount,s=t.fixedRowCount,l=t.rowHeight,u=t.style,c=t.styleBottomLeftGrid,d=t.styleBottomRightGrid,h=t.styleTopLeftGrid,f=t.styleTopRightGrid,p=t.width,g=e||o!==this._lastRenderedHeight||p!==this._lastRenderedWidth,v=e||n!==this._lastRenderedColumnWidth||a!==this._lastRenderedFixedColumnCount,m=e||s!==this._lastRenderedFixedRowCount||l!==this._lastRenderedRowHeight;(e||g||u!==this._lastRenderedStyle)&&(this._containerOuterStyle=rt({height:o,overflow:"visible",width:p},u)),(e||g||m)&&(this._containerTopStyle={height:this._getTopGridHeight(this.props),position:"relative",width:p},this._containerBottomStyle={height:o-this._getTopGridHeight(this.props),overflow:"visible",position:"relative",width:p}),(e||c!==this._lastRenderedStyleBottomLeftGrid)&&(this._bottomLeftGridStyle=rt({left:0,overflowX:"hidden",overflowY:r?"auto":"hidden",position:"absolute"},c)),(e||v||d!==this._lastRenderedStyleBottomRightGrid)&&(this._bottomRightGridStyle=rt({left:this._getLeftGridWidth(this.props),position:"absolute"},d)),(e||h!==this._lastRenderedStyleTopLeftGrid)&&(this._topLeftGridStyle=rt({left:0,overflowX:"hidden",overflowY:"hidden",position:"absolute",top:0},h)),(e||v||f!==this._lastRenderedStyleTopRightGrid)&&(this._topRightGridStyle=rt({left:this._getLeftGridWidth(this.props),overflowX:i?"auto":"hidden",overflowY:"hidden",position:"absolute",top:0},f)),this._lastRenderedColumnWidth=n,this._lastRenderedFixedColumnCount=a,this._lastRenderedFixedRowCount=s,this._lastRenderedHeight=o,this._lastRenderedRowHeight=l,this._lastRenderedStyle=u,this._lastRenderedStyleBottomLeftGrid=c,this._lastRenderedStyleBottomRightGrid=d,this._lastRenderedStyleTopLeftGrid=h,this._lastRenderedStyleTopRightGrid=f,this._lastRenderedWidth=p}},{key:"_prepareForRender",value:function(){this._lastRenderedColumnWidth===this.props.columnWidth&&this._lastRenderedFixedColumnCount===this.props.fixedColumnCount||(this._leftGridWidth=null),this._lastRenderedFixedRowCount===this.props.fixedRowCount&&this._lastRenderedRowHeight===this.props.rowHeight||(this._topGridHeight=null),this._maybeCalculateCachedStyles(),this._lastRenderedColumnWidth=this.props.columnWidth,this._lastRenderedFixedColumnCount=this.props.fixedColumnCount,this._lastRenderedFixedRowCount=this.props.fixedRowCount,this._lastRenderedRowHeight=this.props.rowHeight}},{key:"_renderBottomLeftGrid",value:function(e){var t=e.enableFixedColumnScroll,n=e.fixedColumnCount,r=e.fixedRowCount,i=e.rowCount,o=e.hideBottomLeftGridScrollbar,a=this.state.showVerticalScrollbar;if(!n)return null;var s=a?1:0,l=this._getBottomGridHeight(e),u=this._getLeftGridWidth(e),c=this.state.showVerticalScrollbar?this.state.scrollbarSize:0,d=o?u+c:u,h=y.createElement(K,_()({},e,{cellRenderer:this._cellRendererBottomLeftGrid,className:this.props.classNameBottomLeftGrid,columnCount:n,deferredMeasurementCache:this._deferredMeasurementCacheBottomLeftGrid,height:l,onScroll:t?this._onScrollTop:void 0,ref:this._bottomLeftGridRef,rowCount:Math.max(0,i-r)+s,rowHeight:this._rowHeightBottomGrid,style:this._bottomLeftGridStyle,tabIndex:null,width:d}));return o?y.createElement("div",{className:"BottomLeftGrid_ScrollWrapper",style:rt({},this._bottomLeftGridStyle,{height:l,width:u,overflowY:"hidden"})},h):h}},{key:"_renderBottomRightGrid",value:function(e){var t=e.columnCount,n=e.fixedColumnCount,r=e.fixedRowCount,i=e.rowCount,o=e.scrollToColumn,a=e.scrollToRow;return y.createElement(K,_()({},e,{cellRenderer:this._cellRendererBottomRightGrid,className:this.props.classNameBottomRightGrid,columnCount:Math.max(0,t-n),columnWidth:this._columnWidthRightGrid,deferredMeasurementCache:this._deferredMeasurementCacheBottomRightGrid,height:this._getBottomGridHeight(e),onScroll:this._onScroll,onScrollbarPresenceChange:this._onScrollbarPresenceChange,ref:this._bottomRightGridRef,rowCount:Math.max(0,i-r),rowHeight:this._rowHeightBottomGrid,scrollToColumn:o-n,scrollToRow:a-r,style:this._bottomRightGridStyle,width:this._getRightGridWidth(e)}))}},{key:"_renderTopLeftGrid",value:function(e){var t=e.fixedColumnCount,n=e.fixedRowCount;return t&&n?y.createElement(K,_()({},e,{className:this.props.classNameTopLeftGrid,columnCount:t,height:this._getTopGridHeight(e),ref:this._topLeftGridRef,rowCount:n,style:this._topLeftGridStyle,tabIndex:null,width:this._getLeftGridWidth(e)})):null}},{key:"_renderTopRightGrid",value:function(e){var t=e.columnCount,n=e.enableFixedRowScroll,r=e.fixedColumnCount,i=e.fixedRowCount,o=e.scrollLeft,a=e.hideTopRightGridScrollbar,s=this.state,l=s.showHorizontalScrollbar,u=s.scrollbarSize;if(!i)return null;var c=l?1:0,d=this._getTopGridHeight(e),h=this._getRightGridWidth(e),f=l?u:0,p=d,g=this._topRightGridStyle;a&&(p=d+f,g=rt({},this._topRightGridStyle,{left:0}));var v=y.createElement(K,_()({},e,{cellRenderer:this._cellRendererTopRightGrid,className:this.props.classNameTopRightGrid,columnCount:Math.max(0,t-r)+c,columnWidth:this._columnWidthRightGrid,deferredMeasurementCache:this._deferredMeasurementCacheTopRightGrid,height:p,onScroll:n?this._onScrollLeft:void 0,ref:this._topRightGridRef,rowCount:i,scrollLeft:o,style:g,tabIndex:null,width:h}));return a?y.createElement("div",{className:"TopRightGrid_ScrollWrapper",style:rt({},this._topRightGridStyle,{height:d,width:h,overflowX:"hidden"})},v):v}}],[{key:"getDerivedStateFromProps",value:function(e,t){return e.scrollLeft!==t.scrollLeft||e.scrollTop!==t.scrollTop?{scrollLeft:null!=e.scrollLeft&&e.scrollLeft>=0?e.scrollLeft:t.scrollLeft,scrollTop:null!=e.scrollTop&&e.scrollTop>=0?e.scrollTop:t.scrollTop}:null}}]),t}(y.PureComponent);m()(it,"defaultProps",{classNameBottomLeftGrid:"",classNameBottomRightGrid:"",classNameTopLeftGrid:"",classNameTopRightGrid:"",enableFixedColumnScroll:!1,enableFixedRowScroll:!1,fixedColumnCount:0,fixedRowCount:0,scrollToColumn:-1,scrollToRow:-1,style:{},styleBottomLeftGrid:{},styleBottomRightGrid:{},styleTopLeftGrid:{},styleTopRightGrid:{},hideTopRightGridScrollbar:!1,hideBottomLeftGridScrollbar:!1}),it.propTypes={},Object(C.polyfill)(it);var ot=function(e){function t(e,n){var r;return o()(this,t),(r=u()(this,d()(t).call(this,e,n))).state={clientHeight:0,clientWidth:0,scrollHeight:0,scrollLeft:0,scrollTop:0,scrollWidth:0},r._onScroll=r._onScroll.bind(f()(r)),r}return g()(t,e),s()(t,[{key:"render",value:function(){var e=this.props.children,t=this.state,n=t.clientHeight,r=t.clientWidth,i=t.scrollHeight,o=t.scrollLeft,a=t.scrollTop,s=t.scrollWidth;return e({clientHeight:n,clientWidth:r,onScroll:this._onScroll,scrollHeight:i,scrollLeft:o,scrollTop:a,scrollWidth:s})}},{key:"_onScroll",value:function(e){var t=e.clientHeight,n=e.clientWidth,r=e.scrollHeight,i=e.scrollLeft,o=e.scrollTop,a=e.scrollWidth;this.setState({clientHeight:t,clientWidth:n,scrollHeight:r,scrollLeft:i,scrollTop:o,scrollWidth:a})}}]),t}(y.PureComponent);ot.propTypes={};function at(e){var t=e.className,n=e.columns,r=e.style;return y.createElement("div",{className:t,role:"row",style:r},n)}at.propTypes=null;var st={ASC:"ASC",DESC:"DESC"};function lt(e){var t=e.sortDirection,n=Object(I.a)("ReactVirtualized__Table__sortableHeaderIcon",{"ReactVirtualized__Table__sortableHeaderIcon--ASC":t===st.ASC,"ReactVirtualized__Table__sortableHeaderIcon--DESC":t===st.DESC});return y.createElement("svg",{className:n,width:18,height:18,viewBox:"0 0 24 24"},t===st.ASC?y.createElement("path",{d:"M7 14l5-5 5 5z"}):y.createElement("path",{d:"M7 10l5 5 5-5z"}),y.createElement("path",{d:"M0 0h24v24H0z",fill:"none"}))}function ut(e){var t=e.dataKey,n=e.label,r=e.sortBy,i=e.sortDirection,o=r===t,a=[y.createElement("span",{className:"ReactVirtualized__Table__headerTruncatedText",key:"label",title:"string"==typeof n?n:null},n)];return o&&a.push(y.createElement(lt,{key:"SortIndicator",sortDirection:i})),a}function ct(e){var t=e.className,n=e.columns,r=e.index,i=e.key,o=e.onRowClick,a=e.onRowDoubleClick,s=e.onRowMouseOut,l=e.onRowMouseOver,u=e.onRowRightClick,c=e.rowData,d=e.style,h={"aria-rowindex":r+1};return(o||a||s||l||u)&&(h["aria-label"]="row",h.tabIndex=0,o&&(h.onClick=function(e){return o({event:e,index:r,rowData:c})}),a&&(h.onDoubleClick=function(e){return a({event:e,index:r,rowData:c})}),s&&(h.onMouseOut=function(e){return s({event:e,index:r,rowData:c})}),l&&(h.onMouseOver=function(e){return l({event:e,index:r,rowData:c})}),u&&(h.onContextMenu=function(e){return u({event:e,index:r,rowData:c})})),y.createElement("div",_()({},h,{className:t,key:i,role:"row",style:d}),n)}lt.propTypes={},ut.propTypes=null,ct.propTypes=null;var dt=function(e){function t(){return o()(this,t),u()(this,d()(t).apply(this,arguments))}return g()(t,e),t}(y.Component);function ht(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),n.push.apply(n,r)}return n}function ft(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?ht(n,!0).forEach((function(t){m()(e,t,n[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):ht(n).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))}))}return e}m()(dt,"defaultProps",{cellDataGetter:function(e){var t=e.dataKey,n=e.rowData;return"function"==typeof n.get?n.get(t):n[t]},cellRenderer:function(e){var t=e.cellData;return null==t?"":String(t)},defaultSortDirection:st.ASC,flexGrow:0,flexShrink:1,headerRenderer:ut,style:{}}),dt.propTypes={};var pt=function(e){function t(e){var n;return o()(this,t),(n=u()(this,d()(t).call(this,e))).state={scrollbarWidth:0},n._createColumn=n._createColumn.bind(f()(n)),n._createRow=n._createRow.bind(f()(n)),n._onScroll=n._onScroll.bind(f()(n)),n._onSectionRendered=n._onSectionRendered.bind(f()(n)),n._setRef=n._setRef.bind(f()(n)),n}return g()(t,e),s()(t,[{key:"forceUpdateGrid",value:function(){this.Grid&&this.Grid.forceUpdate()}},{key:"getOffsetForRow",value:function(e){var t=e.alignment,n=e.index;return this.Grid?this.Grid.getOffsetForCell({alignment:t,rowIndex:n}).scrollTop:0}},{key:"invalidateCellSizeAfterRender",value:function(e){var t=e.columnIndex,n=e.rowIndex;this.Grid&&this.Grid.invalidateCellSizeAfterRender({rowIndex:n,columnIndex:t})}},{key:"measureAllRows",value:function(){this.Grid&&this.Grid.measureAllCells()}},{key:"recomputeGridSize",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},t=e.columnIndex,n=void 0===t?0:t,r=e.rowIndex,i=void 0===r?0:r;this.Grid&&this.Grid.recomputeGridSize({rowIndex:i,columnIndex:n})}},{key:"recomputeRowHeights",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:0;this.Grid&&this.Grid.recomputeGridSize({rowIndex:e})}},{key:"scrollToPosition",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:0;this.Grid&&this.Grid.scrollToPosition({scrollTop:e})}},{key:"scrollToRow",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:0;this.Grid&&this.Grid.scrollToCell({columnIndex:0,rowIndex:e})}},{key:"getScrollbarWidth",value:function(){if(this.Grid){var e=Object(de.findDOMNode)(this.Grid),t=e.clientWidth||0;return(e.offsetWidth||0)-t}return 0}},{key:"componentDidMount",value:function(){this._setScrollbarWidth()}},{key:"componentDidUpdate",value:function(){this._setScrollbarWidth()}},{key:"render",value:function(){var e=this,t=this.props,n=t.children,r=t.className,i=t.disableHeader,o=t.gridClassName,a=t.gridStyle,s=t.headerHeight,l=t.headerRowRenderer,u=t.height,c=t.id,d=t.noRowsRenderer,h=t.rowClassName,f=t.rowStyle,p=t.scrollToIndex,g=t.style,v=t.width,m=this.state.scrollbarWidth,b=i?u:u-s,C="function"==typeof h?h({index:-1}):h,A="function"==typeof f?f({index:-1}):f;return this._cachedColumnStyles=[],y.Children.toArray(n).forEach((function(t,n){var r=e._getFlexStyleForColumn(t,t.props.style);e._cachedColumnStyles[n]=ft({overflow:"hidden"},r)})),y.createElement("div",{"aria-label":this.props["aria-label"],"aria-labelledby":this.props["aria-labelledby"],"aria-colcount":y.Children.toArray(n).length,"aria-rowcount":this.props.rowCount,className:Object(I.a)("ReactVirtualized__Table",r),id:c,role:"grid",style:g},!i&&l({className:Object(I.a)("ReactVirtualized__Table__headerRow",C),columns:this._getHeaderColumns(),style:ft({height:s,overflow:"hidden",paddingRight:m,width:v},A)}),y.createElement(K,_()({},this.props,{"aria-readonly":null,autoContainerWidth:!0,className:Object(I.a)("ReactVirtualized__Table__Grid",o),cellRenderer:this._createRow,columnWidth:v,columnCount:1,height:b,id:void 0,noContentRenderer:d,onScroll:this._onScroll,onSectionRendered:this._onSectionRendered,ref:this._setRef,role:"rowgroup",scrollbarWidth:m,scrollToRow:p,style:ft({},a,{overflowX:"hidden"})})))}},{key:"_createColumn",value:function(e){var t=e.column,n=e.columnIndex,r=e.isScrolling,i=e.parent,o=e.rowData,a=e.rowIndex,s=this.props.onColumnClick,l=t.props,u=l.cellDataGetter,c=l.cellRenderer,d=l.className,h=l.columnData,f=l.dataKey,p=l.id,g=c({cellData:u({columnData:h,dataKey:f,rowData:o}),columnData:h,columnIndex:n,dataKey:f,isScrolling:r,parent:i,rowData:o,rowIndex:a}),v=this._cachedColumnStyles[n],m="string"==typeof g?g:null;return y.createElement("div",{"aria-colindex":n+1,"aria-describedby":p,className:Object(I.a)("ReactVirtualized__Table__rowColumn",d),key:"Row"+a+"-Col"+n,onClick:function(e){s&&s({columnData:h,dataKey:f,event:e})},role:"gridcell",style:v,title:m},g)}},{key:"_createHeader",value:function(e){var t,n,r,i,o,a=e.column,s=e.index,l=this.props,u=l.headerClassName,c=l.headerStyle,d=l.onHeaderClick,h=l.sort,f=l.sortBy,p=l.sortDirection,g=a.props,v=g.columnData,m=g.dataKey,b=g.defaultSortDirection,C=g.disableSort,A=g.headerRenderer,_=g.id,x=g.label,w=!C&&h,M=Object(I.a)("ReactVirtualized__Table__headerColumn",u,a.props.headerClassName,{ReactVirtualized__Table__sortableHeaderColumn:w}),S=this._getFlexStyleForColumn(a,ft({},c,{},a.props.headerStyle)),T=A({columnData:v,dataKey:m,disableSort:C,label:x,sortBy:f,sortDirection:p});if(w||d){var E=f!==m?b:p===st.DESC?st.ASC:st.DESC,O=function(e){w&&h({defaultSortDirection:b,event:e,sortBy:m,sortDirection:E}),d&&d({columnData:v,dataKey:m,event:e})};o=a.props["aria-label"]||x||m,i="none",r=0,t=O,n=function(e){"Enter"!==e.key&&" "!==e.key||O(e)}}return f===m&&(i=p===st.ASC?"ascending":"descending"),y.createElement("div",{"aria-label":o,"aria-sort":i,className:M,id:_,key:"Header-Col"+s,onClick:t,onKeyDown:n,role:"columnheader",style:S,tabIndex:r},T)}},{key:"_createRow",value:function(e){var t=this,n=e.rowIndex,r=e.isScrolling,i=e.key,o=e.parent,a=e.style,s=this.props,l=s.children,u=s.onRowClick,c=s.onRowDoubleClick,d=s.onRowRightClick,h=s.onRowMouseOver,f=s.onRowMouseOut,p=s.rowClassName,g=s.rowGetter,v=s.rowRenderer,m=s.rowStyle,b=this.state.scrollbarWidth,C="function"==typeof p?p({index:n}):p,A="function"==typeof m?m({index:n}):m,_=g({index:n}),x=y.Children.toArray(l).map((function(e,i){return t._createColumn({column:e,columnIndex:i,isScrolling:r,parent:o,rowData:_,rowIndex:n,scrollbarWidth:b})})),w=Object(I.a)("ReactVirtualized__Table__row",C),M=ft({},a,{height:this._getRowHeight(n),overflow:"hidden",paddingRight:b},A);return v({className:w,columns:x,index:n,isScrolling:r,key:i,onRowClick:u,onRowDoubleClick:c,onRowRightClick:d,onRowMouseOver:h,onRowMouseOut:f,rowData:_,style:M})}},{key:"_getFlexStyleForColumn",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n="".concat(e.props.flexGrow," ").concat(e.props.flexShrink," ").concat(e.props.width,"px"),r=ft({},t,{flex:n,msFlex:n,WebkitFlex:n});return e.props.maxWidth&&(r.maxWidth=e.props.maxWidth),e.props.minWidth&&(r.minWidth=e.props.minWidth),r}},{key:"_getHeaderColumns",value:function(){var e=this,t=this.props,n=t.children;return(t.disableHeader?[]:y.Children.toArray(n)).map((function(t,n){return e._createHeader({column:t,index:n})}))}},{key:"_getRowHeight",value:function(e){var t=this.props.rowHeight;return"function"==typeof t?t({index:e}):t}},{key:"_onScroll",value:function(e){var t=e.clientHeight,n=e.scrollHeight,r=e.scrollTop;(0,this.props.onScroll)({clientHeight:t,scrollHeight:n,scrollTop:r})}},{key:"_onSectionRendered",value:function(e){var t=e.rowOverscanStartIndex,n=e.rowOverscanStopIndex,r=e.rowStartIndex,i=e.rowStopIndex;(0,this.props.onRowsRendered)({overscanStartIndex:t,overscanStopIndex:n,startIndex:r,stopIndex:i})}},{key:"_setRef",value:function(e){this.Grid=e}},{key:"_setScrollbarWidth",value:function(){var e=this.getScrollbarWidth();this.setState({scrollbarWidth:e})}}]),t}(y.PureComponent);m()(pt,"defaultProps",{disableHeader:!1,estimatedRowSize:30,headerHeight:0,headerStyle:{},noRowsRenderer:function(){return null},onRowsRendered:function(){return null},onScroll:function(){return null},overscanIndicesGetter:J,overscanRowCount:10,rowRenderer:ct,headerRowRenderer:at,rowStyle:{},scrollToAlignment:"auto",scrollToIndex:-1,style:{}}),pt.propTypes={};var gt=[],vt=null,mt=null;function yt(){mt&&(mt=null,document.body&&null!=vt&&(document.body.style.pointerEvents=vt),vt=null)}function bt(){yt(),gt.forEach((function(e){return e.__resetIsScrolling()}))}function Ct(e){e.currentTarget===window&&null==vt&&document.body&&(vt=document.body.style.pointerEvents,document.body.style.pointerEvents="none"),function(){mt&&Y(mt);var e=0;gt.forEach((function(t){e=Math.max(e,t.props.scrollingResetTimeInterval)})),mt=W(bt,e)}(),gt.forEach((function(t){t.props.scrollElement===e.currentTarget&&t.__handleWindowScrollEvent()}))}function At(e,t){gt.some((function(e){return e.props.scrollElement===t}))||t.addEventListener("scroll",Ct),gt.push(e)}function _t(e,t){(gt=gt.filter((function(t){return t!==e}))).length||(t.removeEventListener("scroll",Ct),mt&&(Y(mt),yt()))}var It,xt,wt=function(e){return e===window},Mt=function(e){return e.getBoundingClientRect()};function St(e,t){if(e){if(wt(e)){var n=window,r=n.innerHeight,i=n.innerWidth;return{height:"number"==typeof r?r:0,width:"number"==typeof i?i:0}}return Mt(e)}return{height:t.serverHeight,width:t.serverWidth}}function Tt(e,t){if(wt(t)&&document.documentElement){var n=document.documentElement,r=Mt(e),i=Mt(n);return{top:r.top-i.top,left:r.left-i.left}}var o=Et(t),a=Mt(e),s=Mt(t);return{top:a.top+o.top-s.top,left:a.left+o.left-s.left}}function Et(e){return wt(e)&&document.documentElement?{top:"scrollY"in window?window.scrollY:document.documentElement.scrollTop,left:"scrollX"in window?window.scrollX:document.documentElement.scrollLeft}:{top:e.scrollTop,left:e.scrollLeft}}function Ot(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),n.push.apply(n,r)}return n}function kt(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?Ot(n,!0).forEach((function(t){m()(e,t,n[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):Ot(n).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))}))}return e}var Lt=function(){return"undefined"!=typeof window?window:void 0},Dt=(xt=It=function(e){function t(){var e,n;o()(this,t);for(var r=arguments.length,i=new Array(r),a=0;a<r;a++)i[a]=arguments[a];return n=u()(this,(e=d()(t)).call.apply(e,[this].concat(i))),m()(f()(n),"_window",Lt()),m()(f()(n),"_isMounted",!1),m()(f()(n),"_positionFromTop",0),m()(f()(n),"_positionFromLeft",0),m()(f()(n),"_detectElementResize",void 0),m()(f()(n),"_child",void 0),m()(f()(n),"state",kt({},St(n.props.scrollElement,n.props),{isScrolling:!1,scrollLeft:0,scrollTop:0})),m()(f()(n),"_registerChild",(function(e){!e||e instanceof Element||console.warn("WindowScroller registerChild expects to be passed Element or null"),n._child=e,n.updatePosition()})),m()(f()(n),"_onChildScroll",(function(e){var t=e.scrollTop;if(n.state.scrollTop!==t){var r=n.props.scrollElement;r&&("function"==typeof r.scrollTo?r.scrollTo(0,t+n._positionFromTop):r.scrollTop=t+n._positionFromTop)}})),m()(f()(n),"_registerResizeListener",(function(e){e===window?window.addEventListener("resize",n._onResize,!1):n._detectElementResize.addResizeListener(e,n._onResize)})),m()(f()(n),"_unregisterResizeListener",(function(e){e===window?window.removeEventListener("resize",n._onResize,!1):e&&n._detectElementResize.removeResizeListener(e,n._onResize)})),m()(f()(n),"_onResize",(function(){n.updatePosition()})),m()(f()(n),"__handleWindowScrollEvent",(function(){if(n._isMounted){var e=n.props.onScroll,t=n.props.scrollElement;if(t){var r=Et(t),i=Math.max(0,r.left-n._positionFromLeft),o=Math.max(0,r.top-n._positionFromTop);n.setState({isScrolling:!0,scrollLeft:i,scrollTop:o}),e({scrollLeft:i,scrollTop:o})}}})),m()(f()(n),"__resetIsScrolling",(function(){n.setState({isScrolling:!1})})),n}return g()(t,e),s()(t,[{key:"updatePosition",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:this.props.scrollElement,t=this.props.onResize,n=this.state,r=n.height,i=n.width,o=this._child||de.findDOMNode(this);if(o instanceof Element&&e){var a=Tt(o,e);this._positionFromTop=a.top,this._positionFromLeft=a.left}var s=St(e,this.props);r===s.height&&i===s.width||(this.setState({height:s.height,width:s.width}),t({height:s.height,width:s.width}))}},{key:"componentDidMount",value:function(){var e=this.props.scrollElement;this._detectElementResize=Object(oe.a)(),this.updatePosition(e),e&&(At(this,e),this._registerResizeListener(e)),this._isMounted=!0}},{key:"componentDidUpdate",value:function(e,t){var n=this.props.scrollElement,r=e.scrollElement;r!==n&&null!=r&&null!=n&&(this.updatePosition(n),_t(this,r),At(this,n),this._unregisterResizeListener(r),this._registerResizeListener(n))}},{key:"componentWillUnmount",value:function(){var e=this.props.scrollElement;e&&(_t(this,e),this._unregisterResizeListener(e)),this._isMounted=!1}},{key:"render",value:function(){var e=this.props.children,t=this.state,n=t.isScrolling,r=t.scrollTop,i=t.scrollLeft,o=t.height,a=t.width;return e({onChildScroll:this._onChildScroll,registerChild:this._registerChild,height:o,isScrolling:n,scrollLeft:i,scrollTop:r,width:a})}}]),t}(y.PureComponent),m()(It,"propTypes",null),xt);m()(Dt,"defaultProps",{onResize:function(){},onScroll:function(){},scrollingResetTimeInterval:150,scrollElement:Lt(),serverHeight:0,serverWidth:0});var Pt=n(644),Nt=n.n(Pt),jt=n(340),Rt=n.n(jt);function zt(e){var t=null;return function(){return null==t&&(t=e()),t}}function Ft(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}var Bt=function(){function e(t){!function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,e),this.entered=[],this.isNodeInDocument=t}var t,n,r;return t=e,(n=[{key:"enter",value:function(e){var t=this,n=this.entered.length;return this.entered=function(e,t){var n=new Set,r=function(e){return n.add(e)};e.forEach(r),t.forEach(r);var i=[];return n.forEach((function(e){return i.push(e)})),i}(this.entered.filter((function(n){return t.isNodeInDocument(n)&&(!n.contains||n.contains(e))})),[e]),0===n&&this.entered.length>0}},{key:"leave",value:function(e){var t,n,r=this.entered.length;return this.entered=(t=this.entered.filter(this.isNodeInDocument),n=e,t.filter((function(e){return e!==n}))),r>0&&0===this.entered.length}},{key:"reset",value:function(){this.entered=[]}}])&&Ft(t.prototype,n),r&&Ft(t,r),e}(),Ht=zt((function(){return/firefox/i.test(navigator.userAgent)})),Ut=zt((function(){return Boolean(window.safari)}));function Vt(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}var Yt=function(){function e(t,n){!function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,e);for(var r=t.length,i=[],o=0;o<r;o++)i.push(o);i.sort((function(e,n){return t[e]<t[n]?-1:1}));for(var a,s,l=[],u=[],c=[],d=0;d<r-1;d++)a=t[d+1]-t[d],s=n[d+1]-n[d],u.push(a),l.push(s),c.push(s/a);for(var h=[c[0]],f=0;f<u.length-1;f++){var p=c[f],g=c[f+1];if(p*g<=0)h.push(0);else{a=u[f];var v=u[f+1],m=a+v;h.push(3*m/((m+v)/p+(m+a)/g))}}h.push(c[c.length-1]);for(var y,b=[],C=[],A=0;A<h.length-1;A++){y=c[A];var _=h[A],I=1/u[A],x=_+h[A+1]-y-y;b.push((y-_-x)*I),C.push(x*I*I)}this.xs=t,this.ys=n,this.c1s=h,this.c2s=b,this.c3s=C}var t,n,r;return t=e,(n=[{key:"interpolate",value:function(e){var t=this.xs,n=this.ys,r=this.c1s,i=this.c2s,o=this.c3s,a=t.length-1;if(e===t[a])return n[a];for(var s,l=0,u=o.length-1;l<=u;){var c=t[s=Math.floor(.5*(l+u))];if(c<e)l=s+1;else{if(!(c>e))return n[s];u=s-1}}var d=e-t[a=Math.max(0,u)],h=d*d;return n[a]+r[a]*d+i[a]*h+o[a]*d*h}}])&&Vt(t.prototype,n),r&&Vt(t,r),e}();function Wt(e){var t=1===e.nodeType?e:e.parentElement;if(!t)return null;var n=t.getBoundingClientRect(),r=n.top;return{x:n.left,y:r}}function Gt(e){return{x:e.clientX,y:e.clientY}}function Xt(e,t,n,r,i){var o,a,s="IMG"===(o=t).nodeName&&(Ht()||!document.documentElement.contains(o)),l=Wt(s?e:t),u={x:n.x-l.x,y:n.y-l.y},c=e.offsetWidth,d=e.offsetHeight,h=r.anchorX,f=r.anchorY,p=function(e,t,n,r){var i=e?t.width:n,o=e?t.height:r;return Ut()&&e&&(o/=window.devicePixelRatio,i/=window.devicePixelRatio),{dragPreviewWidth:i,dragPreviewHeight:o}}(s,t,c,d),g=p.dragPreviewWidth,v=p.dragPreviewHeight,m=i.offsetX,y=i.offsetY,b=0===y||y;return{x:0===m||m?m:new Yt([0,.5,1],[u.x,u.x/c*g,u.x+g-c]).interpolate(h),y:b?y:(a=new Yt([0,.5,1],[u.y,u.y/d*v,u.y+v-d]).interpolate(f),Ut()&&s&&(a+=(window.devicePixelRatio-1)*v),a)}}var qt,Zt="__NATIVE_FILE__",Qt="__NATIVE_URL__",Kt="__NATIVE_TEXT__";function Jt(e,t,n){var r=t.reduce((function(t,n){return t||e.getData(n)}),"");return null!=r?r:n}function $t(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}var en=($t(qt={},Zt,{exposeProperties:{files:function(e){return Array.prototype.slice.call(e.files)},items:function(e){return e.items}},matchesTypes:["Files"]}),$t(qt,Qt,{exposeProperties:{urls:function(e,t){return Jt(e,t,"").split("\n")}},matchesTypes:["Url","text/uri-list"]}),$t(qt,Kt,{exposeProperties:{text:function(e,t){return Jt(e,t,"")}},matchesTypes:["Text","text/plain"]}),qt);function tn(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}var nn=function(){function e(t){!function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,e),this.config=t,this.item={},this.initializeExposedProperties()}var t,n,r;return t=e,(n=[{key:"initializeExposedProperties",value:function(){var e=this;Object.keys(this.config.exposeProperties).forEach((function(t){Object.defineProperty(e.item,t,{configurable:!0,enumerable:!0,get:function(){return console.warn("Browser doesn't allow reading \"".concat(t,'" until the drop event.')),null}})}))}},{key:"loadDataTransfer",value:function(e){var t=this;if(e){var n={};Object.keys(this.config.exposeProperties).forEach((function(r){n[r]={value:t.config.exposeProperties[r](e,t.config.matchesTypes),configurable:!0,enumerable:!0}})),Object.defineProperties(this.item,n)}}},{key:"canDrag",value:function(){return!0}},{key:"beginDrag",value:function(){return this.item}},{key:"isDragging",value:function(e,t){return t===e.getSourceId()}},{key:"endDrag",value:function(){}}])&&tn(t.prototype,n),r&&tn(t,r),e}();function rn(e){if(!e)return null;var t=Array.prototype.slice.call(e.types||[]);return Object.keys(en).filter((function(e){return en[e].matchesTypes.some((function(e){return t.indexOf(e)>-1}))}))[0]||null}function on(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}var an=function(){function e(t){!function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,e),this.globalContext=t}var t,n,r;return t=e,(n=[{key:"window",get:function(){return this.globalContext?this.globalContext:"undefined"!=typeof window?window:void 0}},{key:"document",get:function(){if(this.window)return this.window.document}}])&&on(t.prototype,n),r&&on(t,r),e}();function sn(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),n.push.apply(n,r)}return n}function ln(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?sn(Object(n),!0).forEach((function(t){un(e,t,n[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):sn(Object(n)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))}))}return e}function un(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function cn(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}var dn=function(){function e(t,n){var r=this;!function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,e),this.sourcePreviewNodes=new Map,this.sourcePreviewNodeOptions=new Map,this.sourceNodes=new Map,this.sourceNodeOptions=new Map,this.dragStartSourceIds=null,this.dropTargetIds=[],this.dragEnterTargetIds=[],this.currentNativeSource=null,this.currentNativeHandle=null,this.currentDragSourceNode=null,this.altKeyPressed=!1,this.mouseMoveTimeoutTimer=null,this.asyncEndDragFrameId=null,this.dragOverTargetIds=null,this.getSourceClientOffset=function(e){return Wt(r.sourceNodes.get(e))},this.endDragNativeItem=function(){r.isDraggingNativeItem()&&(r.actions.endDrag(),r.registry.removeSource(r.currentNativeHandle),r.currentNativeHandle=null,r.currentNativeSource=null)},this.isNodeInDocument=function(e){return r.document&&r.document.body&&document.body.contains(e)},this.endDragIfSourceWasRemovedFromDOM=function(){var e=r.currentDragSourceNode;r.isNodeInDocument(e)||r.clearCurrentDragSourceNode()&&r.actions.endDrag()},this.handleTopDragStartCapture=function(){r.clearCurrentDragSourceNode(),r.dragStartSourceIds=[]},this.handleTopDragStart=function(e){if(!e.defaultPrevented){var t=r.dragStartSourceIds;r.dragStartSourceIds=null;var n=Gt(e);r.monitor.isDragging()&&r.actions.endDrag(),r.actions.beginDrag(t||[],{publishSource:!1,getSourceClientOffset:r.getSourceClientOffset,clientOffset:n});var i=e.dataTransfer,o=rn(i);if(r.monitor.isDragging()){if(i&&"function"==typeof i.setDragImage){var a=r.monitor.getSourceId(),s=r.sourceNodes.get(a),l=r.sourcePreviewNodes.get(a)||s;if(l){var u=r.getCurrentSourcePreviewNodeOptions(),c=Xt(s,l,n,{anchorX:u.anchorX,anchorY:u.anchorY},{offsetX:u.offsetX,offsetY:u.offsetY});i.setDragImage(l,c.x,c.y)}}try{i.setData("application/json",{})}catch(e){}r.setCurrentDragSourceNode(e.target),r.getCurrentSourcePreviewNodeOptions().captureDraggingState?r.actions.publishDragSource():setTimeout((function(){return r.actions.publishDragSource()}),0)}else if(o)r.beginDragNativeItem(o);else{if(i&&!i.types&&(e.target&&!e.target.hasAttribute||!e.target.hasAttribute("draggable")))return;e.preventDefault()}}},this.handleTopDragEndCapture=function(){r.clearCurrentDragSourceNode()&&r.actions.endDrag()},this.handleTopDragEnterCapture=function(e){if(r.dragEnterTargetIds=[],r.enterLeaveCounter.enter(e.target)&&!r.monitor.isDragging()){var t=e.dataTransfer,n=rn(t);n&&r.beginDragNativeItem(n,t)}},this.handleTopDragEnter=function(e){var t=r.dragEnterTargetIds;(r.dragEnterTargetIds=[],r.monitor.isDragging())&&(r.altKeyPressed=e.altKey,Ht()||r.actions.hover(t,{clientOffset:Gt(e)}),t.some((function(e){return r.monitor.canDropOnTarget(e)}))&&(e.preventDefault(),e.dataTransfer&&(e.dataTransfer.dropEffect=r.getCurrentDropEffect())))},this.handleTopDragOverCapture=function(){r.dragOverTargetIds=[]},this.handleTopDragOver=function(e){var t=r.dragOverTargetIds;if(r.dragOverTargetIds=[],!r.monitor.isDragging())return e.preventDefault(),void(e.dataTransfer&&(e.dataTransfer.dropEffect="none"));r.altKeyPressed=e.altKey,r.actions.hover(t||[],{clientOffset:Gt(e)}),(t||[]).some((function(e){return r.monitor.canDropOnTarget(e)}))?(e.preventDefault(),e.dataTransfer&&(e.dataTransfer.dropEffect=r.getCurrentDropEffect())):r.isDraggingNativeItem()?e.preventDefault():(e.preventDefault(),e.dataTransfer&&(e.dataTransfer.dropEffect="none"))},this.handleTopDragLeaveCapture=function(e){r.isDraggingNativeItem()&&e.preventDefault(),r.enterLeaveCounter.leave(e.target)&&r.isDraggingNativeItem()&&r.endDragNativeItem()},this.handleTopDropCapture=function(e){r.dropTargetIds=[],e.preventDefault(),r.isDraggingNativeItem()&&r.currentNativeSource.loadDataTransfer(e.dataTransfer),r.enterLeaveCounter.reset()},this.handleTopDrop=function(e){var t=r.dropTargetIds;r.dropTargetIds=[],r.actions.hover(t,{clientOffset:Gt(e)}),r.actions.drop({dropEffect:r.getCurrentDropEffect()}),r.isDraggingNativeItem()?r.endDragNativeItem():r.endDragIfSourceWasRemovedFromDOM()},this.handleSelectStart=function(e){var t=e.target;"function"==typeof t.dragDrop&&("INPUT"===t.tagName||"SELECT"===t.tagName||"TEXTAREA"===t.tagName||t.isContentEditable||(e.preventDefault(),t.dragDrop()))},this.options=new an(n),this.actions=t.getActions(),this.monitor=t.getMonitor(),this.registry=t.getRegistry(),this.enterLeaveCounter=new Bt(this.isNodeInDocument)}var t,n,i;return t=e,(n=[{key:"setup",value:function(){if(void 0!==this.window){if(this.window.__isReactDndBackendSetUp)throw new Error("Cannot have two HTML5 backends at the same time.");this.window.__isReactDndBackendSetUp=!0,this.addEventListeners(this.window)}}},{key:"teardown",value:function(){void 0!==this.window&&(this.window.__isReactDndBackendSetUp=!1,this.removeEventListeners(this.window),this.clearCurrentDragSourceNode(),this.asyncEndDragFrameId&&this.window.cancelAnimationFrame(this.asyncEndDragFrameId))}},{key:"connectDragPreview",value:function(e,t,n){var r=this;return this.sourcePreviewNodeOptions.set(e,n),this.sourcePreviewNodes.set(e,t),function(){r.sourcePreviewNodes.delete(e),r.sourcePreviewNodeOptions.delete(e)}}},{key:"connectDragSource",value:function(e,t,n){var r=this;this.sourceNodes.set(e,t),this.sourceNodeOptions.set(e,n);var i=function(t){return r.handleDragStart(t,e)},o=function(e){return r.handleSelectStart(e)};return t.setAttribute("draggable","true"),t.addEventListener("dragstart",i),t.addEventListener("selectstart",o),function(){r.sourceNodes.delete(e),r.sourceNodeOptions.delete(e),t.removeEventListener("dragstart",i),t.removeEventListener("selectstart",o),t.setAttribute("draggable","false")}}},{key:"connectDropTarget",value:function(e,t){var n=this,r=function(t){return n.handleDragEnter(t,e)},i=function(t){return n.handleDragOver(t,e)},o=function(t){return n.handleDrop(t,e)};return t.addEventListener("dragenter",r),t.addEventListener("dragover",i),t.addEventListener("drop",o),function(){t.removeEventListener("dragenter",r),t.removeEventListener("dragover",i),t.removeEventListener("drop",o)}}},{key:"addEventListeners",value:function(e){e.addEventListener&&(e.addEventListener("dragstart",this.handleTopDragStart),e.addEventListener("dragstart",this.handleTopDragStartCapture,!0),e.addEventListener("dragend",this.handleTopDragEndCapture,!0),e.addEventListener("dragenter",this.handleTopDragEnter),e.addEventListener("dragenter",this.handleTopDragEnterCapture,!0),e.addEventListener("dragleave",this.handleTopDragLeaveCapture,!0),e.addEventListener("dragover",this.handleTopDragOver),e.addEventListener("dragover",this.handleTopDragOverCapture,!0),e.addEventListener("drop",this.handleTopDrop),e.addEventListener("drop",this.handleTopDropCapture,!0))}},{key:"removeEventListeners",value:function(e){e.removeEventListener&&(e.removeEventListener("dragstart",this.handleTopDragStart),e.removeEventListener("dragstart",this.handleTopDragStartCapture,!0),e.removeEventListener("dragend",this.handleTopDragEndCapture,!0),e.removeEventListener("dragenter",this.handleTopDragEnter),e.removeEventListener("dragenter",this.handleTopDragEnterCapture,!0),e.removeEventListener("dragleave",this.handleTopDragLeaveCapture,!0),e.removeEventListener("dragover",this.handleTopDragOver),e.removeEventListener("dragover",this.handleTopDragOverCapture,!0),e.removeEventListener("drop",this.handleTopDrop),e.removeEventListener("drop",this.handleTopDropCapture,!0))}},{key:"getCurrentSourceNodeOptions",value:function(){var e=this.monitor.getSourceId(),t=this.sourceNodeOptions.get(e);return ln({dropEffect:this.altKeyPressed?"copy":"move"},t||{})}},{key:"getCurrentDropEffect",value:function(){return this.isDraggingNativeItem()?"copy":this.getCurrentSourceNodeOptions().dropEffect}},{key:"getCurrentSourcePreviewNodeOptions",value:function(){var e=this.monitor.getSourceId();return ln({anchorX:.5,anchorY:.5,captureDraggingState:!1},this.sourcePreviewNodeOptions.get(e)||{})}},{key:"isDraggingNativeItem",value:function(){var e=this.monitor.getItemType();return Object.keys(r).some((function(t){return r[t]===e}))}},{key:"beginDragNativeItem",value:function(e,t){this.clearCurrentDragSourceNode(),this.currentNativeSource=function(e,t){var n=new nn(en[e]);return n.loadDataTransfer(t),n}(e,t),this.currentNativeHandle=this.registry.addSource(e,this.currentNativeSource),this.actions.beginDrag([this.currentNativeHandle])}},{key:"setCurrentDragSourceNode",value:function(e){var t=this;this.clearCurrentDragSourceNode(),this.currentDragSourceNode=e,this.mouseMoveTimeoutTimer=setTimeout((function(){return t.window&&t.window.addEventListener("mousemove",t.endDragIfSourceWasRemovedFromDOM,!0)}),1e3)}},{key:"clearCurrentDragSourceNode",value:function(){return!!this.currentDragSourceNode&&(this.currentDragSourceNode=null,this.window&&(this.window.clearTimeout(this.mouseMoveTimeoutTimer||void 0),this.window.removeEventListener("mousemove",this.endDragIfSourceWasRemovedFromDOM,!0)),this.mouseMoveTimeoutTimer=null,!0)}},{key:"handleDragStart",value:function(e,t){e.defaultPrevented||(this.dragStartSourceIds||(this.dragStartSourceIds=[]),this.dragStartSourceIds.unshift(t))}},{key:"handleDragEnter",value:function(e,t){this.dragEnterTargetIds.unshift(t)}},{key:"handleDragOver",value:function(e,t){null===this.dragOverTargetIds&&(this.dragOverTargetIds=[]),this.dragOverTargetIds.unshift(t)}},{key:"handleDrop",value:function(e,t){this.dropTargetIds.unshift(t)}},{key:"window",get:function(){return this.options.window}},{key:"document",get:function(){return this.options.document}}])&&cn(t.prototype,n),i&&cn(t,i),e}(),hn=function(e,t){return new dn(e,t)},fn=n(19),pn=n.n(fn);function gn(e){return(gn="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function vn(e){return"function"==typeof e}function mn(){}function yn(e){if(!function(e){return"object"===gn(e)&&null!==e}(e))return!1;if(null===Object.getPrototypeOf(e))return!0;for(var t=e;null!==Object.getPrototypeOf(t);)t=Object.getPrototypeOf(t);return Object.getPrototypeOf(e)===t}function bn(e){var t=e.current;return null==t?null:t.decoratedRef?t.decoratedRef.current:t}var Cn=n(117),An=n.n(Cn),_n=n(91),In=n.n(_n),xn=n(652);function wn(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function Mn(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}function Sn(e,t,n){return t&&Mn(e.prototype,t),n&&Mn(e,n),e}var Tn=function(){function e(t){wn(this,e),this.isDisposed=!1,this.action=vn(t)?t:mn}return Sn(e,[{key:"dispose",value:function(){this.isDisposed||(this.action(),this.isDisposed=!0)}}],[{key:"isDisposable",value:function(e){return e&&vn(e.dispose)}},{key:"_fixup",value:function(t){return e.isDisposable(t)?t:e.empty}},{key:"create",value:function(t){return new e(t)}}]),e}();Tn.empty={dispose:mn};var En=function(){function e(){wn(this,e),this.isDisposed=!1;for(var t=arguments.length,n=new Array(t),r=0;r<t;r++)n[r]=arguments[r];this.disposables=n}return Sn(e,[{key:"add",value:function(e){this.isDisposed?e.dispose():this.disposables.push(e)}},{key:"remove",value:function(e){var t=!1;if(!this.isDisposed){var n=this.disposables.indexOf(e);-1!==n&&(t=!0,this.disposables.splice(n,1),e.dispose())}return t}},{key:"clear",value:function(){if(!this.isDisposed){for(var e=this.disposables.length,t=new Array(e),n=0;n<e;n++)t[n]=this.disposables[n];this.disposables=[];for(var r=0;r<e;r++)t[r].dispose()}}},{key:"dispose",value:function(){if(!this.isDisposed){this.isDisposed=!0;for(var e=this.disposables.length,t=new Array(e),n=0;n<e;n++)t[n]=this.disposables[n];this.disposables=[];for(var r=0;r<e;r++)t[r].dispose()}}}]),e}(),On=function(){function e(){wn(this,e),this.isDisposed=!1}return Sn(e,[{key:"getDisposable",value:function(){return this.current}},{key:"setDisposable",value:function(e){var t=this.isDisposed;if(!t){var n=this.current;this.current=e,n&&n.dispose()}t&&e&&e.dispose()}},{key:"dispose",value:function(){if(!this.isDisposed){this.isDisposed=!0;var e=this.current;this.current=void 0,e&&e.dispose()}}}]),e}();function kn(e){return(kn="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function Ln(e,t){return function(e){if(Array.isArray(e))return e}(e)||function(e,t){if(!(Symbol.iterator in Object(e))&&"[object Arguments]"!==Object.prototype.toString.call(e))return;var n=[],r=!0,i=!1,o=void 0;try{for(var a,s=e[Symbol.iterator]();!(r=(a=s.next()).done)&&(n.push(a.value),!t||n.length!==t);r=!0);}catch(e){i=!0,o=e}finally{try{r||null==s.return||s.return()}finally{if(i)throw o}}return n}(e,t)||function(){throw new TypeError("Invalid attempt to destructure non-iterable instance")}()}function Dn(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}function Pn(e,t){return!t||"object"!==kn(t)&&"function"!=typeof t?function(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}(e):t}function Nn(e){return(Nn=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}function jn(e,t){return(jn=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function Rn(e){var t=e.DecoratedComponent,n=e.createHandler,r=e.createMonitor,i=e.createConnector,o=e.registerHandler,a=e.containerDisplayName,s=e.getType,l=e.collect,u=e.options.arePropsEqual,c=void 0===u?An.a:u,d=t,h=t.displayName||t.name||"Component",f=function(e){function t(e){var n;return function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,t),(n=Pn(this,Nn(t).call(this,e))).decoratedRef=y.createRef(),n.handleChange=function(){var e=n.getCurrentState();An()(e,n.state)||n.setState(e)},n.disposable=new On,n.receiveProps(e),n.dispose(),n}var a,u,f;return function(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&jn(e,t)}(t,e),a=t,(u=[{key:"getHandlerId",value:function(){return this.handlerId}},{key:"getDecoratedComponentInstance",value:function(){return pn()(this.decoratedRef.current,"In order to access an instance of the decorated component, it must either be a class component or use React.forwardRef()"),this.decoratedRef.current}},{key:"shouldComponentUpdate",value:function(e,t){return!c(e,this.props)||!An()(t,this.state)}},{key:"componentDidMount",value:function(){this.disposable=new On,this.currentType=void 0,this.receiveProps(this.props),this.handleChange()}},{key:"componentDidUpdate",value:function(e){c(this.props,e)||(this.receiveProps(this.props),this.handleChange())}},{key:"componentWillUnmount",value:function(){this.dispose()}},{key:"receiveProps",value:function(e){this.handler&&(this.handler.receiveProps(e),this.receiveType(s(e)))}},{key:"receiveType",value:function(e){if(this.handlerMonitor&&this.manager&&this.handlerConnector&&e!==this.currentType){this.currentType=e;var t=Ln(o(e,this.handler,this.manager),2),n=t[0],r=t[1];this.handlerId=n,this.handlerMonitor.receiveHandlerId(n),this.handlerConnector.receiveHandlerId(n);var i=this.manager.getMonitor().subscribeToStateChange(this.handleChange,{handlerIds:[n]});this.disposable.setDisposable(new En(new Tn(i),new Tn(r)))}}},{key:"dispose",value:function(){this.disposable.dispose(),this.handlerConnector&&this.handlerConnector.receiveHandlerId(null)}},{key:"getCurrentState",value:function(){return this.handlerConnector?l(this.handlerConnector.hooks,this.handlerMonitor,this.props):{}}},{key:"render",value:function(){var e=this;return y.createElement(xn.a.Consumer,null,(function(t){var n,r,i=t.dragDropManager;return e.receiveDragDropManager(i),"undefined"!=typeof requestAnimationFrame&&requestAnimationFrame((function(){return e.handlerConnector.reconnect()})),y.createElement(d,Object.assign({},e.props,e.getCurrentState(),{ref:(n=d,(r=n)&&r.prototype&&"function"==typeof r.prototype.render||function(e){return e&&e.$$typeof&&"Symbol(react.forward_ref)"===e.$$typeof.toString()}(n)?e.decoratedRef:null)}))}))}},{key:"receiveDragDropManager",value:function(e){void 0===this.manager&&(pn()(void 0!==e,"Could not find the drag and drop manager in the context of %s. Make sure to render a DndProvider component in your top-level component. Read more: http://react-dnd.github.io/react-dnd/docs/troubleshooting#could-not-find-the-drag-and-drop-manager-in-the-context",h,h),void 0!==e&&(this.manager=e,this.handlerMonitor=r(e),this.handlerConnector=i(e.getBackend()),this.handler=n(this.handlerMonitor,this.decoratedRef)))}}])&&Dn(a.prototype,u),f&&Dn(a,f),t}(y.Component);return f.DecoratedComponent=t,f.displayName="".concat(a,"(").concat(h,")"),In()(f,t)}function zn(e,t,n){var r=n.getRegistry(),i=r.addTarget(e,t);return[i,function(){return r.removeTarget(i)}]}function Fn(e,t,n){var r=n.getRegistry(),i=r.addSource(e,t);return[i,function(){return r.removeSource(i)}]}function Bn(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}var Hn=!1,Un=!1,Vn=function(){function e(t){!function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,e),this.sourceId=null,this.internalMonitor=t.getMonitor()}var t,n,r;return t=e,(n=[{key:"receiveHandlerId",value:function(e){this.sourceId=e}},{key:"getHandlerId",value:function(){return this.sourceId}},{key:"canDrag",value:function(){pn()(!Hn,"You may not call monitor.canDrag() inside your canDrag() implementation. Read more: http://react-dnd.github.io/react-dnd/docs/api/drag-source-monitor");try{return Hn=!0,this.internalMonitor.canDragSource(this.sourceId)}finally{Hn=!1}}},{key:"isDragging",value:function(){if(!this.sourceId)return!1;pn()(!Un,"You may not call monitor.isDragging() inside your isDragging() implementation. Read more: http://react-dnd.github.io/react-dnd/docs/api/drag-source-monitor");try{return Un=!0,this.internalMonitor.isDraggingSource(this.sourceId)}finally{Un=!1}}},{key:"subscribeToStateChange",value:function(e,t){return this.internalMonitor.subscribeToStateChange(e,t)}},{key:"isDraggingSource",value:function(e){return this.internalMonitor.isDraggingSource(e)}},{key:"isOverTarget",value:function(e,t){return this.internalMonitor.isOverTarget(e,t)}},{key:"getTargetIds",value:function(){return this.internalMonitor.getTargetIds()}},{key:"isSourcePublic",value:function(){return this.internalMonitor.isSourcePublic()}},{key:"getSourceId",value:function(){return this.internalMonitor.getSourceId()}},{key:"subscribeToOffsetChange",value:function(e){return this.internalMonitor.subscribeToOffsetChange(e)}},{key:"canDragSource",value:function(e){return this.internalMonitor.canDragSource(e)}},{key:"canDropOnTarget",value:function(e){return this.internalMonitor.canDropOnTarget(e)}},{key:"getItemType",value:function(){return this.internalMonitor.getItemType()}},{key:"getItem",value:function(){return this.internalMonitor.getItem()}},{key:"getDropResult",value:function(){return this.internalMonitor.getDropResult()}},{key:"didDrop",value:function(){return this.internalMonitor.didDrop()}},{key:"getInitialClientOffset",value:function(){return this.internalMonitor.getInitialClientOffset()}},{key:"getInitialSourceClientOffset",value:function(){return this.internalMonitor.getInitialSourceClientOffset()}},{key:"getSourceClientOffset",value:function(){return this.internalMonitor.getSourceClientOffset()}},{key:"getClientOffset",value:function(){return this.internalMonitor.getClientOffset()}},{key:"getDifferenceFromInitialOffset",value:function(){return this.internalMonitor.getDifferenceFromInitialOffset()}}])&&Bn(t.prototype,n),r&&Bn(t,r),e}();function Yn(e,t){"function"==typeof e?e(t):e.current=t}function Wn(e,t){var n=e.ref;return pn()("string"!=typeof n,"Cannot connect React DnD to an element with an existing string ref. Please convert it to use a callback ref instead, or wrap it into a <span> or <div>. Read more: https://facebook.github.io/react/docs/more-about-refs.html#the-ref-callback-attribute"),n?Object(y.cloneElement)(e,{ref:function(e){Yn(n,e),Yn(t,e)}}):Object(y.cloneElement)(e,{ref:t})}function Gn(e){if("string"!=typeof e.type){var t=e.type.displayName||e.type.name||"the component";throw new Error("Only native element nodes can now be passed to React DnD connectors."+"You can either wrap ".concat(t," into a <div>, or turn it into a ")+"drag source or a drop target itself.")}}function Xn(e){var t={};return Object.keys(e).forEach((function(n){var r=e[n];if(n.endsWith("Ref"))t[n]=e[n];else{var i=function(e){return function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:null,n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:null;if(!Object(y.isValidElement)(t)){var r=t;return e(r,n),r}var i=t;Gn(i);var o=n?function(t){return e(t,n)}:e;return Wn(i,o)}}(r);t[n]=function(){return i}}})),t}function qn(e){return(qn="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function Zn(e){return null!==e&&"object"===qn(e)&&e.hasOwnProperty("current")}function Qn(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}var Kn=function(){function e(t){var n=this;!function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,e),this.hooks=Xn({dragSource:function(e,t){n.clearDragSource(),n.dragSourceOptions=t||null,Zn(e)?n.dragSourceRef=e:n.dragSourceNode=e,n.reconnectDragSource()},dragPreview:function(e,t){n.clearDragPreview(),n.dragPreviewOptions=t||null,Zn(e)?n.dragPreviewRef=e:n.dragPreviewNode=e,n.reconnectDragPreview()}}),this.handlerId=null,this.dragSourceRef=null,this.dragSourceOptionsInternal=null,this.dragPreviewRef=null,this.dragPreviewOptionsInternal=null,this.lastConnectedHandlerId=null,this.lastConnectedDragSource=null,this.lastConnectedDragSourceOptions=null,this.lastConnectedDragPreview=null,this.lastConnectedDragPreviewOptions=null,this.backend=t}var t,n,r;return t=e,(n=[{key:"receiveHandlerId",value:function(e){this.handlerId!==e&&(this.handlerId=e,this.reconnect())}},{key:"reconnect",value:function(){this.reconnectDragSource(),this.reconnectDragPreview()}},{key:"reconnectDragSource",value:function(){var e=this.dragSource,t=this.didHandlerIdChange()||this.didConnectedDragSourceChange()||this.didDragSourceOptionsChange();t&&this.disconnectDragSource(),this.handlerId&&(e?t&&(this.lastConnectedHandlerId=this.handlerId,this.lastConnectedDragSource=e,this.lastConnectedDragSourceOptions=this.dragSourceOptions,this.dragSourceUnsubscribe=this.backend.connectDragSource(this.handlerId,e,this.dragSourceOptions)):this.lastConnectedDragSource=e)}},{key:"reconnectDragPreview",value:function(){var e=this.dragPreview,t=this.didHandlerIdChange()||this.didConnectedDragPreviewChange()||this.didDragPreviewOptionsChange();this.handlerId?this.dragPreview&&t&&(this.lastConnectedHandlerId=this.handlerId,this.lastConnectedDragPreview=e,this.lastConnectedDragPreviewOptions=this.dragPreviewOptions,this.disconnectDragPreview(),this.dragPreviewUnsubscribe=this.backend.connectDragPreview(this.handlerId,e,this.dragPreviewOptions)):this.disconnectDragPreview()}},{key:"didHandlerIdChange",value:function(){return this.lastConnectedHandlerId!==this.handlerId}},{key:"didConnectedDragSourceChange",value:function(){return this.lastConnectedDragSource!==this.dragSource}},{key:"didConnectedDragPreviewChange",value:function(){return this.lastConnectedDragPreview!==this.dragPreview}},{key:"didDragSourceOptionsChange",value:function(){return!An()(this.lastConnectedDragSourceOptions,this.dragSourceOptions)}},{key:"didDragPreviewOptionsChange",value:function(){return!An()(this.lastConnectedDragPreviewOptions,this.dragPreviewOptions)}},{key:"disconnectDragSource",value:function(){this.dragSourceUnsubscribe&&(this.dragSourceUnsubscribe(),this.dragSourceUnsubscribe=void 0)}},{key:"disconnectDragPreview",value:function(){this.dragPreviewUnsubscribe&&(this.dragPreviewUnsubscribe(),this.dragPreviewUnsubscribe=void 0,this.dragPreviewNode=null,this.dragPreviewRef=null)}},{key:"clearDragSource",value:function(){this.dragSourceNode=null,this.dragSourceRef=null}},{key:"clearDragPreview",value:function(){this.dragPreviewNode=null,this.dragPreviewRef=null}},{key:"connectTarget",get:function(){return this.dragSource}},{key:"dragSourceOptions",get:function(){return this.dragSourceOptionsInternal},set:function(e){this.dragSourceOptionsInternal=e}},{key:"dragPreviewOptions",get:function(){return this.dragPreviewOptionsInternal},set:function(e){this.dragPreviewOptionsInternal=e}},{key:"dragSource",get:function(){return this.dragSourceNode||this.dragSourceRef&&this.dragSourceRef.current}},{key:"dragPreview",get:function(){return this.dragPreviewNode||this.dragPreviewRef&&this.dragPreviewRef.current}}])&&Qn(t.prototype,n),r&&Qn(t,r),e}();function Jn(e){return(Jn="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function $n(e,t){return"string"==typeof e||"symbol"===Jn(e)||!!t&&Array.isArray(e)&&e.every((function(e){return $n(e,!1)}))}function er(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}var tr=["canDrag","beginDrag","isDragging","endDrag"],nr=["beginDrag"],rr=function(){function e(t,n,r){var i=this;!function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,e),this.props=null,this.beginDrag=function(){if(i.props)return i.spec.beginDrag(i.props,i.monitor,i.ref.current)},this.spec=t,this.monitor=n,this.ref=r}var t,n,r;return t=e,(n=[{key:"receiveProps",value:function(e){this.props=e}},{key:"canDrag",value:function(){return!!this.props&&(!this.spec.canDrag||this.spec.canDrag(this.props,this.monitor))}},{key:"isDragging",value:function(e,t){return!!this.props&&(this.spec.isDragging?this.spec.isDragging(this.props,this.monitor):t===e.getSourceId())}},{key:"endDrag",value:function(){this.props&&this.spec.endDrag&&this.spec.endDrag(this.props,this.monitor,bn(this.ref))}}])&&er(t.prototype,n),r&&er(t,r),e}();function ir(e){return Object.keys(e).forEach((function(t){pn()(tr.indexOf(t)>-1,'Expected the drag source specification to only have some of the following keys: %s. Instead received a specification with an unexpected "%s" key. Read more: http://react-dnd.github.io/react-dnd/docs/api/drag-source',tr.join(", "),t),pn()("function"==typeof e[t],"Expected %s in the drag source specification to be a function. Instead received a specification with %s: %s. Read more: http://react-dnd.github.io/react-dnd/docs/api/drag-source",t,t,e[t])})),nr.forEach((function(t){pn()("function"==typeof e[t],"Expected %s in the drag source specification to be a function. Instead received a specification with %s: %s. Read more: http://react-dnd.github.io/react-dnd/docs/api/drag-source",t,t,e[t])})),function(t,n){return new rr(e,t,n)}}function or(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}var ar=function(){function e(t){var n=this;!function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,e),this.hooks=Xn({dropTarget:function(e,t){n.clearDropTarget(),n.dropTargetOptions=t,Zn(e)?n.dropTargetRef=e:n.dropTargetNode=e,n.reconnect()}}),this.handlerId=null,this.dropTargetRef=null,this.dropTargetOptionsInternal=null,this.lastConnectedHandlerId=null,this.lastConnectedDropTarget=null,this.lastConnectedDropTargetOptions=null,this.backend=t}var t,n,r;return t=e,(n=[{key:"reconnect",value:function(){var e=this.didHandlerIdChange()||this.didDropTargetChange()||this.didOptionsChange();e&&this.disconnectDropTarget();var t=this.dropTarget;this.handlerId&&(t?e&&(this.lastConnectedHandlerId=this.handlerId,this.lastConnectedDropTarget=t,this.lastConnectedDropTargetOptions=this.dropTargetOptions,this.unsubscribeDropTarget=this.backend.connectDropTarget(this.handlerId,t,this.dropTargetOptions)):this.lastConnectedDropTarget=t)}},{key:"receiveHandlerId",value:function(e){e!==this.handlerId&&(this.handlerId=e,this.reconnect())}},{key:"didHandlerIdChange",value:function(){return this.lastConnectedHandlerId!==this.handlerId}},{key:"didDropTargetChange",value:function(){return this.lastConnectedDropTarget!==this.dropTarget}},{key:"didOptionsChange",value:function(){return!An()(this.lastConnectedDropTargetOptions,this.dropTargetOptions)}},{key:"disconnectDropTarget",value:function(){this.unsubscribeDropTarget&&(this.unsubscribeDropTarget(),this.unsubscribeDropTarget=void 0)}},{key:"clearDropTarget",value:function(){this.dropTargetRef=null,this.dropTargetNode=null}},{key:"connectTarget",get:function(){return this.dropTarget}},{key:"dropTargetOptions",get:function(){return this.dropTargetOptionsInternal},set:function(e){this.dropTargetOptionsInternal=e}},{key:"dropTarget",get:function(){return this.dropTargetNode||this.dropTargetRef&&this.dropTargetRef.current}}])&&or(t.prototype,n),r&&or(t,r),e}();function sr(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}var lr=!1,ur=function(){function e(t){!function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,e),this.targetId=null,this.internalMonitor=t.getMonitor()}var t,n,r;return t=e,(n=[{key:"receiveHandlerId",value:function(e){this.targetId=e}},{key:"getHandlerId",value:function(){return this.targetId}},{key:"subscribeToStateChange",value:function(e,t){return this.internalMonitor.subscribeToStateChange(e,t)}},{key:"canDrop",value:function(){if(!this.targetId)return!1;pn()(!lr,"You may not call monitor.canDrop() inside your canDrop() implementation. Read more: http://react-dnd.github.io/react-dnd/docs/api/drop-target-monitor");try{return lr=!0,this.internalMonitor.canDropOnTarget(this.targetId)}finally{lr=!1}}},{key:"isOver",value:function(e){return!!this.targetId&&this.internalMonitor.isOverTarget(this.targetId,e)}},{key:"getItemType",value:function(){return this.internalMonitor.getItemType()}},{key:"getItem",value:function(){return this.internalMonitor.getItem()}},{key:"getDropResult",value:function(){return this.internalMonitor.getDropResult()}},{key:"didDrop",value:function(){return this.internalMonitor.didDrop()}},{key:"getInitialClientOffset",value:function(){return this.internalMonitor.getInitialClientOffset()}},{key:"getInitialSourceClientOffset",value:function(){return this.internalMonitor.getInitialSourceClientOffset()}},{key:"getSourceClientOffset",value:function(){return this.internalMonitor.getSourceClientOffset()}},{key:"getClientOffset",value:function(){return this.internalMonitor.getClientOffset()}},{key:"getDifferenceFromInitialOffset",value:function(){return this.internalMonitor.getDifferenceFromInitialOffset()}}])&&sr(t.prototype,n),r&&sr(t,r),e}();function cr(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}var dr=["canDrop","hover","drop"],hr=function(){function e(t,n,r){!function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,e),this.props=null,this.spec=t,this.monitor=n,this.ref=r}var t,n,r;return t=e,(n=[{key:"receiveProps",value:function(e){this.props=e}},{key:"receiveMonitor",value:function(e){this.monitor=e}},{key:"canDrop",value:function(){return!this.spec.canDrop||this.spec.canDrop(this.props,this.monitor)}},{key:"hover",value:function(){this.spec.hover&&this.spec.hover(this.props,this.monitor,bn(this.ref))}},{key:"drop",value:function(){if(this.spec.drop)return this.spec.drop(this.props,this.monitor,this.ref.current)}}])&&cr(t.prototype,n),r&&cr(t,r),e}();function fr(e){return Object.keys(e).forEach((function(t){pn()(dr.indexOf(t)>-1,'Expected the drop target specification to only have some of the following keys: %s. Instead received a specification with an unexpected "%s" key. Read more: http://react-dnd.github.io/react-dnd/docs/api/drop-target',dr.join(", "),t),pn()("function"==typeof e[t],"Expected %s in the drop target specification to be a function. Instead received a specification with %s: %s. Read more: http://react-dnd.github.io/react-dnd/docs/api/drop-target",t,t,e[t])})),function(t,n){return new hr(e,t,n)}}function pr(e,t,n){var r=arguments.length>3&&void 0!==arguments[3]?arguments[3]:{},i=e;"function"!=typeof e&&(pn()($n(e,!0),'Expected "type" provided as the first argument to DropTarget to be a string, an array of strings, or a function that returns either given the current props. Instead, received %s. Read more: http://react-dnd.github.io/react-dnd/docs/api/drop-target',e),i=function(){return e}),pn()(yn(t),'Expected "spec" provided as the second argument to DropTarget to be a plain object. Instead, received %s. Read more: http://react-dnd.github.io/react-dnd/docs/api/drop-target',t);var o=fr(t);return pn()("function"==typeof n,'Expected "collect" provided as the third argument to DropTarget to be a function that returns a plain object of props to inject. Instead, received %s. Read more: http://react-dnd.github.io/react-dnd/docs/api/drop-target',n),pn()(yn(r),'Expected "options" provided as the fourth argument to DropTarget to be a plain object when specified. Instead, received %s. Read more: http://react-dnd.github.io/react-dnd/docs/api/drop-target',n),function(e){return Rn({containerDisplayName:"DropTarget",createHandler:o,registerHandler:zn,createMonitor:function(e){return new ur(e)},createConnector:function(e){return new ar(e)},DecoratedComponent:e,getType:i,collect:n,options:r})}}var gr=n(2380);function vr(e){return(vr="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function mr(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function yr(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}function br(e,t,n){return t&&yr(e.prototype,t),n&&yr(e,n),e}function Cr(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function Ar(){return(Ar=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e}).apply(this,arguments)}function _r(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{},r=Object.keys(n);"function"==typeof Object.getOwnPropertySymbols&&(r=r.concat(Object.getOwnPropertySymbols(n).filter((function(e){return Object.getOwnPropertyDescriptor(n,e).enumerable})))),r.forEach((function(t){Cr(e,t,n[t])}))}return e}function Ir(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&wr(e,t)}function xr(e){return(xr=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}function wr(e,t){return(wr=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function Mr(e,t){if(null==e)return{};var n,r,i=function(e,t){if(null==e)return{};var n,r,i={},o=Object.keys(e);for(r=0;r<o.length;r++)n=o[r],t.indexOf(n)>=0||(i[n]=e[n]);return i}(e,t);if(Object.getOwnPropertySymbols){var o=Object.getOwnPropertySymbols(e);for(r=0;r<o.length;r++)n=o[r],t.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(e,n)&&(i[n]=e[n])}return i}function Sr(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function Tr(e,t){return!t||"object"!=typeof t&&"function"!=typeof t?Sr(e):t}function Er(e){return function(e){if(Array.isArray(e)){for(var t=0,n=new Array(e.length);t<e.length;t++)n[t]=e[t];return n}}(e)||function(e){if(Symbol.iterator in Object(e)||"[object Arguments]"===Object.prototype.toString.call(e))return Array.from(e)}(e)||function(){throw new TypeError("Invalid attempt to spread non-iterable instance")}()}function Or(){var e=this.constructor.getDerivedStateFromProps(this.props,this.state);null!=e&&this.setState(e)}function kr(e){this.setState(function(t){var n=this.constructor.getDerivedStateFromProps(e,t);return null!=n?n:null}.bind(this))}function Lr(e,t){try{var n=this.props,r=this.state;this.props=e,this.state=t,this.__reactInternalSnapshotFlag=!0,this.__reactInternalSnapshot=this.getSnapshotBeforeUpdate(n,r)}finally{this.props=n,this.state=r}}function Dr(){for(var e=arguments.length,t=new Array(e),n=0;n<e;n++)t[n]=arguments[n];return t.filter(Boolean).join(" ")}n.d(t,"SortableTreeWithoutDndContext",(function(){return fi})),n.d(t,"defaultGetNodeKey",(function(){return ai})),n.d(t,"defaultSearchMethod",(function(){return li})),n.d(t,"getDescendantCount",(function(){return jr})),n.d(t,"getVisibleNodeCount",(function(){return Rr})),n.d(t,"getVisibleNodeInfoAtIndex",(function(){return zr})),n.d(t,"walk",(function(){return Fr})),n.d(t,"map",(function(){return Br})),n.d(t,"toggleExpandedForAll",(function(){return Hr})),n.d(t,"changeNodeAtPath",(function(){return Ur})),n.d(t,"removeNodeAtPath",(function(){return Vr})),n.d(t,"removeNode",(function(){return Yr})),n.d(t,"getNodeAtPath",(function(){return Wr})),n.d(t,"addNodeUnderParent",(function(){return Gr})),n.d(t,"insertNode",(function(){return Xr})),n.d(t,"getFlatDataFromTree",(function(){return qr})),n.d(t,"getTreeFromFlatData",(function(){return Zr})),n.d(t,"isDescendant",(function(){return Qr})),n.d(t,"getDepth",(function(){return Kr})),n.d(t,"find",(function(){return Jr})),Or.__suppressDeprecationWarning=!0,kr.__suppressDeprecationWarning=!0,Lr.__suppressDeprecationWarning=!0;var Pr=function(e){function t(){return mr(this,t),Tr(this,xr(t).apply(this,arguments))}return Ir(t,e),br(t,[{key:"render",value:function(){var e,t=this.props,n=t.children,r=t.listIndex,i=t.swapFrom,o=t.swapLength,a=t.swapDepth,s=t.scaffoldBlockPxWidth,l=t.lowerSiblingCounts,u=t.connectDropTarget,c=t.isOver,d=t.draggedNode,h=t.canDrop,f=t.treeIndex,p=(t.treeId,t.getPrevRow,t.node,t.path,t.rowDirection),g=Mr(t,["children","listIndex","swapFrom","swapLength","swapDepth","scaffoldBlockPxWidth","lowerSiblingCounts","connectDropTarget","isOver","draggedNode","canDrop","treeIndex","treeId","getPrevRow","node","path","rowDirection"]),v="rtl"===p?"rst__rtl":null,m=l.length,C=[];return l.forEach((function(e,t){var n="";if(e>0?n=0===r?"rst__lineHalfHorizontalRight rst__lineHalfVerticalBottom":t===m-1?"rst__lineHalfHorizontalRight rst__lineFullVertical":"rst__lineFullVertical":0===r?n="rst__lineHalfHorizontalRight":t===m-1&&(n="rst__lineHalfVerticalTop rst__lineHalfHorizontalRight"),C.push(b.a.createElement("div",{key:"pre_".concat(1+t),style:{width:s},className:Dr("rst__lineBlock",n,v)})),f!==r&&t===a){var l,u="";u=r===i+o-1?"rst__highlightBottomLeftCorner":f===i?"rst__highlightTopLeftCorner":"rst__highlightLineVertical",l="rtl"===p?{width:s,right:s*t}:{width:s,left:s*t},C.push(b.a.createElement("div",{key:t,style:l,className:Dr("rst__absoluteLineBlock",u,v)}))}})),e="rtl"===p?{right:s*m}:{left:s*m},u(b.a.createElement("div",Ar({},g,{className:Dr("rst__node",v)}),C,b.a.createElement("div",{className:"rst__nodeContent",style:e},y.Children.map(n,(function(e){return Object(y.cloneElement)(e,{isOver:c,canDrop:h,draggedNode:d})})))))}}]),t}(y.Component);function Nr(e){var t=e.targetIndex,n=e.node,r=e.currentIndex,i=e.getNodeKey,o=e.path,a=void 0===o?[]:o,s=e.lowerSiblingCounts,l=void 0===s?[]:s,u=e.ignoreCollapsed,c=void 0===u||u,d=e.isPseudoRoot,h=void 0!==d&&d?[]:Er(a).concat([i({node:n,treeIndex:r})]);if(r===t)return{node:n,lowerSiblingCounts:l,path:h};if(!n.children||c&&!0!==n.expanded)return{nextIndex:r+1};for(var f=r+1,p=n.children.length,g=0;g<p;g+=1){var v=Nr({ignoreCollapsed:c,getNodeKey:i,targetIndex:t,node:n.children[g],currentIndex:f,lowerSiblingCounts:Er(l).concat([p-g-1]),path:h});if(v.node)return v;f=v.nextIndex}return{nextIndex:f}}function jr(e){var t=e.node,n=e.ignoreCollapsed;return Nr({getNodeKey:function(){},ignoreCollapsed:void 0===n||n,node:t,currentIndex:0,targetIndex:-1}).nextIndex-1}function Rr(e){var t=e.treeData;return t.reduce((function(e,t){return e+function e(t){return t.children&&!0===t.expanded&&"function"!=typeof t.children?1+t.children.reduce((function(t,n){return t+e(n)}),0):1}(t)}),0)}function zr(e){var t=e.treeData,n=e.index,r=e.getNodeKey;if(!t||t.length<1)return null;var i=Nr({targetIndex:n,getNodeKey:r,node:{children:t,expanded:!0},currentIndex:-1,path:[],lowerSiblingCounts:[],isPseudoRoot:!0});return i.node?i:null}function Fr(e){var t=e.treeData,n=e.getNodeKey,r=e.callback,i=e.ignoreCollapsed,o=void 0===i||i;!t||t.length<1||function e(t){var n=t.callback,r=t.getNodeKey,i=t.ignoreCollapsed,o=t.isPseudoRoot,a=void 0!==o&&o,s=t.node,l=t.parentNode,u=void 0===l?null:l,c=t.currentIndex,d=t.path,h=void 0===d?[]:d,f=t.lowerSiblingCounts,p=void 0===f?[]:f,g=a?[]:Er(h).concat([r({node:s,treeIndex:c})]);if(!a&&!1===n(a?null:{node:s,parentNode:u,path:g,lowerSiblingCounts:p,treeIndex:c}))return!1;if(!s.children||!0!==s.expanded&&i&&!a)return c;var v=c,m=s.children.length;if("function"!=typeof s.children)for(var y=0;y<m;y+=1)if(!1===(v=e({callback:n,getNodeKey:r,ignoreCollapsed:i,node:s.children[y],parentNode:a?null:s,currentIndex:v+1,lowerSiblingCounts:Er(p).concat([m-y-1]),path:g})))return!1;return v}({callback:r,getNodeKey:n,ignoreCollapsed:o,isPseudoRoot:!0,node:{children:t},currentIndex:-1,path:[],lowerSiblingCounts:[]})}function Br(e){var t=e.treeData,n=e.getNodeKey,r=e.callback,i=e.ignoreCollapsed,o=void 0===i||i;return!t||t.length<1?[]:function e(t){var n=t.callback,r=t.getNodeKey,i=t.ignoreCollapsed,o=t.isPseudoRoot,a=void 0!==o&&o,s=t.node,l=t.parentNode,u=void 0===l?null:l,c=t.currentIndex,d=t.path,h=void 0===d?[]:d,f=t.lowerSiblingCounts,p=void 0===f?[]:f,g=_r({},s),v=a?[]:Er(h).concat([r({node:g,treeIndex:c})]),m={node:g,parentNode:u,path:v,lowerSiblingCounts:p,treeIndex:c};if(!g.children||!0!==g.expanded&&i&&!a)return{treeIndex:c,node:n(m)};var y=c,b=g.children.length;return"function"!=typeof g.children&&(g.children=g.children.map((function(t,o){var s=e({callback:n,getNodeKey:r,ignoreCollapsed:i,node:t,parentNode:a?null:g,currentIndex:y+1,lowerSiblingCounts:Er(p).concat([b-o-1]),path:v});return y=s.treeIndex,s.node}))),{node:n(m),treeIndex:y}}({callback:r,getNodeKey:n,ignoreCollapsed:o,isPseudoRoot:!0,node:{children:t},currentIndex:-1,path:[],lowerSiblingCounts:[]}).node.children}function Hr(e){var t=e.treeData,n=e.expanded,r=void 0===n||n;return Br({treeData:t,callback:function(e){return _r({},e.node,{expanded:r})},getNodeKey:function(e){return e.treeIndex},ignoreCollapsed:!1})}function Ur(e){var t=e.treeData,n=e.path,r=e.newNode,i=e.getNodeKey,o=e.ignoreCollapsed,a=void 0===o||o,s=function e(t){var o=t.isPseudoRoot,s=void 0!==o&&o,l=t.node,u=t.currentTreeIndex,c=t.pathIndex;if(!s&&i({node:l,treeIndex:u})!==n[c])return"RESULT_MISS";if(c>=n.length-1)return"function"==typeof r?r({node:l,treeIndex:u}):r;if(!l.children)throw new Error("Path referenced children of node with no children.");for(var d=u+1,h=0;h<l.children.length;h+=1){var f=e({node:l.children[h],currentTreeIndex:d,pathIndex:c+1});if("RESULT_MISS"!==f)return _r({},l,f?{children:Er(l.children.slice(0,h)).concat([f],Er(l.children.slice(h+1)))}:{children:Er(l.children.slice(0,h)).concat(Er(l.children.slice(h+1)))});d+=1+jr({node:l.children[h],ignoreCollapsed:a})}return"RESULT_MISS"}({node:{children:t},currentTreeIndex:-1,pathIndex:-1,isPseudoRoot:!0});if("RESULT_MISS"===s)throw new Error("No node found at the given path.");return s.children}function Vr(e){var t=e.treeData,n=e.path,r=e.getNodeKey,i=e.ignoreCollapsed;return Ur({treeData:t,path:n,getNodeKey:r,ignoreCollapsed:void 0===i||i,newNode:null})}function Yr(e){var t=e.treeData,n=e.path,r=e.getNodeKey,i=e.ignoreCollapsed,o=null,a=null;return{treeData:Ur({treeData:t,path:n,getNodeKey:r,ignoreCollapsed:void 0===i||i,newNode:function(e){var t=e.node,n=e.treeIndex;return o=t,a=n,null}}),node:o,treeIndex:a}}function Wr(e){var t=e.treeData,n=e.path,r=e.getNodeKey,i=e.ignoreCollapsed,o=void 0===i||i,a=null;try{Ur({treeData:t,path:n,getNodeKey:r,ignoreCollapsed:o,newNode:function(e){var t=e.node,n=e.treeIndex;return a={node:t,treeIndex:n},t}})}catch(e){}return a}function Gr(e){var t=e.treeData,n=e.newNode,r=e.parentKey,i=void 0===r?null:r,o=e.getNodeKey,a=e.ignoreCollapsed,s=void 0===a||a,l=e.expandParent,u=void 0!==l&&l,c=e.addAsFirstChild,d=void 0!==c&&c;if(null===i)return d?{treeData:[n].concat(Er(t||[])),treeIndex:0}:{treeData:Er(t||[]).concat([n]),treeIndex:(t||[]).length};var h=null,f=!1,p=Br({treeData:t,getNodeKey:o,ignoreCollapsed:s,callback:function(e){var t=e.node,r=e.treeIndex,o=e.path,a=o?o[o.length-1]:null;if(f||a!==i)return t;f=!0;var l=_r({},t);if(u&&(l.expanded=!0),!l.children)return h=r+1,_r({},l,{children:[n]});if("function"==typeof l.children)throw new Error("Cannot add to children defined by a function");for(var c=r+1,p=0;p<l.children.length;p+=1)c+=1+jr({node:l.children[p],ignoreCollapsed:s});return h=c,_r({},l,{children:d?[n].concat(Er(l.children)):Er(l.children).concat([n])})}});if(!f)throw new Error("No node found with the given key.");return{treeData:p,treeIndex:h}}function Xr(e){var t=e.treeData,n=e.depth,r=e.minimumTreeIndex,i=e.newNode,o=e.getNodeKey,a=void 0===o?function(){}:o,s=e.ignoreCollapsed,l=void 0===s||s,u=e.expandParent,c=void 0!==u&&u;if(!t&&0===n)return{treeData:[i],treeIndex:0,path:[a({node:i,treeIndex:0})],parentNode:null};var d=function e(t){var n=t.targetDepth,r=t.minimumTreeIndex,i=t.newNode,o=t.ignoreCollapsed,a=t.expandParent,s=t.isPseudoRoot,l=void 0!==s&&s,u=t.isLastChild,c=t.node,d=t.currentIndex,h=t.currentDepth,f=t.getNodeKey,p=t.path,g=void 0===p?[]:p,v=function(e){return l?[]:Er(g).concat([f({node:e,treeIndex:d})])};if(d>=r-1||u&&(!c.children||!c.children.length)){if("function"==typeof c.children)throw new Error("Cannot add to children defined by a function");var m=_r({},c,a?{expanded:!0}:{},{children:c.children?[i].concat(Er(c.children)):[i]});return{node:m,nextIndex:d+2,insertedTreeIndex:d+1,parentPath:v(m),parentNode:l?null:m}}if(h>=n-1){if(!c.children||"function"==typeof c.children||!0!==c.expanded&&o&&!l)return{node:c,nextIndex:d+1};for(var y=d+1,b=null,C=null,A=0;A<c.children.length;A+=1){if(y>=r){b=y,C=A;break}y+=1+jr({node:c.children[A],ignoreCollapsed:o})}if(null===C){if(y<r&&!u)return{node:c,nextIndex:y};b=y,C=c.children.length}var _=_r({},c,{children:Er(c.children.slice(0,C)).concat([i],Er(c.children.slice(C)))});return{node:_,nextIndex:y,insertedTreeIndex:b,parentPath:v(_),parentNode:l?null:_}}if(!c.children||"function"==typeof c.children||!0!==c.expanded&&o&&!l)return{node:c,nextIndex:d+1};var I=null,x=null,w=null,M=d+1,S=c.children;"function"!=typeof S&&(S=S.map((function(t,s){if(null!==I)return t;var l=e({targetDepth:n,minimumTreeIndex:r,newNode:i,ignoreCollapsed:o,expandParent:a,isLastChild:u&&s===S.length-1,node:t,currentIndex:M,currentDepth:h+1,getNodeKey:f,path:[]});return"insertedTreeIndex"in l&&(I=l.insertedTreeIndex,w=l.parentNode,x=l.parentPath),M=l.nextIndex,l.node})));var T=_r({},c,{children:S}),E={node:T,nextIndex:M};return null!==I&&(E.insertedTreeIndex=I,E.parentPath=Er(v(T)).concat(Er(x)),E.parentNode=w),E}({targetDepth:n,minimumTreeIndex:r,newNode:i,ignoreCollapsed:l,expandParent:c,getNodeKey:a,isPseudoRoot:!0,isLastChild:!0,node:{children:t},currentIndex:-1,currentDepth:-1});if(!("insertedTreeIndex"in d))throw new Error("No suitable position found to insert.");var h=d.insertedTreeIndex;return{treeData:d.node.children,treeIndex:h,path:Er(d.parentPath).concat([a({node:i,treeIndex:h})]),parentNode:d.parentNode}}function qr(e){var t=e.treeData,n=e.getNodeKey,r=e.ignoreCollapsed,i=void 0===r||r;if(!t||t.length<1)return[];var o=[];return Fr({treeData:t,getNodeKey:n,ignoreCollapsed:i,callback:function(e){o.push(e)}}),o}function Zr(e){var t=e.flatData,n=e.getKey,r=void 0===n?function(e){return e.id}:n,i=e.getParentKey,o=void 0===i?function(e){return e.parentId}:i,a=e.rootKey,s=void 0===a?"0":a;if(!t)return[];var l={};if(t.forEach((function(e){var t=o(e);t in l?l[t].push(e):l[t]=[e]})),!(s in l))return[];return l[s].map((function(e){return function e(t){var n=r(t);return n in l?_r({},t,{children:l[n].map((function(t){return e(t)}))}):_r({},t)}(e)}))}function Qr(e,t){return!!e.children&&"function"!=typeof e.children&&e.children.some((function(e){return e===t||Qr(e,t)}))}function Kr(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:0;return e.children?"function"==typeof e.children?t+1:e.children.reduce((function(e,n){return Math.max(e,Kr(n,t+1))}),t):t}function Jr(e){var t=e.getNodeKey,n=e.treeData,r=e.searchQuery,i=e.searchMethod,o=e.searchFocusOffset,a=e.expandAllMatchPaths,s=void 0!==a&&a,l=e.expandFocusMatchPaths,u=void 0===l||l,c=0,d=function e(n){var a=n.isPseudoRoot,l=void 0!==a&&a,d=n.node,h=n.currentIndex,f=n.path,p=[],g=!1,v=!1,m=l?[]:Er(void 0===f?[]:f).concat([t({node:d,treeIndex:h})]),y=l?null:{path:m,treeIndex:h},b=d.children&&"function"!=typeof d.children&&d.children.length>0;!l&&i(_r({},y,{node:d,searchQuery:r}))&&(c===o&&(v=!0),c+=1,g=!0);var C=h,A=_r({},d);return b&&(A.children=A.children.map((function(t){var n=e({node:t,currentIndex:C+1,path:m});return n.node.expanded?C=n.treeIndex:C+=1,(n.matches.length>0||n.hasFocusMatch)&&(p=Er(p).concat(Er(n.matches)),n.hasFocusMatch&&(v=!0),(s&&n.matches.length>0||(s||u)&&n.hasFocusMatch)&&(A.expanded=!0)),n.node}))),l||A.expanded||(p=p.map((function(e){return _r({},e,{treeIndex:null})}))),g&&(p=[_r({},y,{node:A})].concat(Er(p))),{node:p.length>0?A:d,matches:p,hasFocusMatch:v,treeIndex:C}}({node:{children:n},isPseudoRoot:!0,currentIndex:-1});return{matches:d.matches,treeData:d.node.children}}Pr.defaultProps={swapFrom:null,swapDepth:null,swapLength:null,canDrop:!1,draggedNode:null,rowDirection:"ltr"},Pr.propTypes={treeIndex:T.a.number.isRequired,treeId:T.a.string.isRequired,swapFrom:T.a.number,swapDepth:T.a.number,swapLength:T.a.number,scaffoldBlockPxWidth:T.a.number.isRequired,lowerSiblingCounts:T.a.arrayOf(T.a.number).isRequired,listIndex:T.a.number.isRequired,children:T.a.node.isRequired,connectDropTarget:T.a.func.isRequired,isOver:T.a.bool.isRequired,canDrop:T.a.bool,draggedNode:T.a.shape({}),getPrevRow:T.a.func.isRequired,node:T.a.shape({}).isRequired,path:T.a.arrayOf(T.a.oneOfType([T.a.string,T.a.number])).isRequired,rowDirection:T.a.string};var $r=function(e){function t(){return mr(this,t),Tr(this,xr(t).apply(this,arguments))}return Ir(t,e),br(t,[{key:"render",value:function(){var e,t=this.props,n=t.scaffoldBlockPxWidth,r=t.toggleChildrenVisibility,i=t.connectDragPreview,o=t.connectDragSource,a=t.isDragging,s=t.canDrop,l=t.canDrag,u=t.node,c=t.title,d=t.subtitle,h=t.draggedNode,f=t.path,p=t.treeIndex,g=t.isSearchMatch,v=t.isSearchFocus,m=t.buttons,y=t.className,C=t.style,A=t.didDrop,_=(t.treeId,t.isOver,t.parentNode,t.rowDirection),I=Mr(t,["scaffoldBlockPxWidth","toggleChildrenVisibility","connectDragPreview","connectDragSource","isDragging","canDrop","canDrag","node","title","subtitle","draggedNode","path","treeIndex","isSearchMatch","isSearchFocus","buttons","className","style","didDrop","treeId","isOver","parentNode","rowDirection"]),x=c||u.title,w=d||u.subtitle,M="rtl"===_?"rst__rtl":null;l&&(e="function"==typeof u.children&&u.expanded?b.a.createElement("div",{className:"rst__loadingHandle"},b.a.createElement("div",{className:"rst__loadingCircle"},Er(new Array(12)).map((function(e,t){return b.a.createElement("div",{key:t,className:Dr("rst__loadingCirclePoint",M)})})))):o(b.a.createElement("div",{className:"rst__moveHandle"}),{dropEffect:"copy"}));var S=h&&Qr(h,u),T=!A&&a,E={left:-.5*n};return"rtl"===_&&(E={right:-.5*n}),b.a.createElement("div",Ar({style:{height:"100%"}},I),r&&u.children&&(u.children.length>0||"function"==typeof u.children)&&b.a.createElement("div",null,b.a.createElement("button",{type:"button","aria-label":u.expanded?"Collapse":"Expand",className:Dr(u.expanded?"rst__collapseButton":"rst__expandButton",M),style:E,onClick:function(){return r({node:u,path:f,treeIndex:p})}}),u.expanded&&!a&&b.a.createElement("div",{style:{width:n},className:Dr("rst__lineChildren",M)})),b.a.createElement("div",{className:Dr("rst__rowWrapper",M)},i(b.a.createElement("div",{className:Dr("rst__row",T&&"rst__rowLandingPad",T&&!s&&"rst__rowCancelPad",g&&"rst__rowSearchMatch",v&&"rst__rowSearchFocus",M,y),style:_r({opacity:S?.5:1},C)},e,b.a.createElement("div",{className:Dr("rst__rowContents",!l&&"rst__rowContentsDragDisabled",M)},b.a.createElement("div",{className:Dr("rst__rowLabel",M)},b.a.createElement("span",{className:Dr("rst__rowTitle",u.subtitle&&"rst__rowTitleWithSubtitle")},"function"==typeof x?x({node:u,path:f,treeIndex:p}):x),w&&b.a.createElement("span",{className:"rst__rowSubtitle"},"function"==typeof w?w({node:u,path:f,treeIndex:p}):w)),b.a.createElement("div",{className:"rst__rowToolbar"},m.map((function(e,t){return b.a.createElement("div",{key:t,className:"rst__toolbarButton"},e)}))))))))}}]),t}(y.Component);$r.defaultProps={isSearchMatch:!1,isSearchFocus:!1,canDrag:!1,toggleChildrenVisibility:null,buttons:[],className:"",style:{},parentNode:null,draggedNode:null,canDrop:!1,title:null,subtitle:null,rowDirection:"ltr"},$r.propTypes={node:T.a.shape({}).isRequired,title:T.a.oneOfType([T.a.func,T.a.node]),subtitle:T.a.oneOfType([T.a.func,T.a.node]),path:T.a.arrayOf(T.a.oneOfType([T.a.string,T.a.number])).isRequired,treeIndex:T.a.number.isRequired,treeId:T.a.string.isRequired,isSearchMatch:T.a.bool,isSearchFocus:T.a.bool,canDrag:T.a.bool,scaffoldBlockPxWidth:T.a.number.isRequired,toggleChildrenVisibility:T.a.func,buttons:T.a.arrayOf(T.a.node),className:T.a.string,style:T.a.shape({}),connectDragPreview:T.a.func.isRequired,connectDragSource:T.a.func.isRequired,parentNode:T.a.shape({}),isDragging:T.a.bool.isRequired,didDrop:T.a.bool.isRequired,draggedNode:T.a.shape({}),isOver:T.a.bool.isRequired,canDrop:T.a.bool,rowDirection:T.a.string};var ei=function(e){function t(){return mr(this,t),Tr(this,xr(t).apply(this,arguments))}return Ir(t,e),br(t,[{key:"render",value:function(){var e=this.props,t=e.children,n=e.connectDropTarget,r=(e.treeId,e.drop,Mr(e,["children","connectDropTarget","treeId","drop"]));return n(b.a.createElement("div",null,y.Children.map(t,(function(e){return Object(y.cloneElement)(e,_r({},r))}))))}}]),t}(y.Component);ei.defaultProps={canDrop:!1,draggedNode:null},ei.propTypes={children:T.a.node.isRequired,connectDropTarget:T.a.func.isRequired,isOver:T.a.bool.isRequired,canDrop:T.a.bool,draggedNode:T.a.shape({}),treeId:T.a.string.isRequired,drop:T.a.func.isRequired};var ti=function(e){var t=e.isOver,n=e.canDrop;return b.a.createElement("div",{className:Dr("rst__placeholder",n&&"rst__placeholderLandingPad",n&&!t&&"rst__placeholderCancelPad")})};ti.defaultProps={isOver:!1,canDrop:!1},ti.propTypes={isOver:T.a.bool,canDrop:T.a.bool};var ni=function(e){var t=[],n=[],r=null;return function(i){var o=Object.keys(i).sort(),a=o.map((function(e){return i[e]}));return(a.length!==t.length||a.some((function(e,n){return e!==t[n]}))||o.some((function(e,t){return e!==n[t]})))&&(t=a,n=o,r=e(i)),r}},ri=ni(Xr),ii=ni(qr),oi=ni(jr);function ai(e){return e.treeIndex}function si(e,t,n,r,i){return"function"==typeof n[e]?String(n[e]({node:n,path:r,treeIndex:i})).indexOf(t)>-1:"object"===vr(n[e])?function e(t){return"string"==typeof t?t:null===t||"object"!==vr(t)||!t.props||!t.props.children||"string"!=typeof t.props.children&&"object"!==vr(t.props.children)?"":"string"==typeof t.props.children?t.props.children:t.props.children.map((function(t){return e(t)})).join("")}(n[e]).indexOf(t)>-1:n[e]&&String(n[e]).indexOf(t)>-1}function li(e){var t=e.node,n=e.path,r=e.treeIndex,i=e.searchQuery;return si("title",i,t,n,r)||si("subtitle",i,t,n,r)}var ui=function(){function e(t){mr(this,e),this.treeRef=t}return br(e,[{key:"getTargetDepth",value:function(e,t,n){var r,i=0,o=e.getPrevRow();if(o){var a=o.path;!this.treeRef.canNodeHaveChildren(o.node)&&(a=a.slice(0,a.length-1)),i=Math.min(a.length,e.path.length)}var s=(t.getItem().path||[]).length;if(t.getItem().treeId!==this.treeId)if(s=0,n){var l=Object(de.findDOMNode)(n).getBoundingClientRect(),u=t.getSourceClientOffset().x-l.left;r=Math.round(u/e.scaffoldBlockPxWidth)}else r=e.path.length;else{var c="rtl"===e.rowDirection?-1:1;r=Math.round(c*t.getDifferenceFromInitialOffset().x/e.scaffoldBlockPxWidth)}var d=Math.min(i,Math.max(0,s+r-1));if(void 0!==this.maxDepth&&null!==this.maxDepth){var h=Kr(t.getItem().node);d=Math.max(0,Math.min(d,this.maxDepth-h-1))}return d}},{key:"canDrop",value:function(e,t){if(!t.isOver())return!1;var n=e.getPrevRow(),r=n?n.path:[],i=n?n.node:{},o=this.getTargetDepth(e,t,null);if(o>=r.length&&"function"==typeof i.children)return!1;if("function"==typeof this.customCanDrop){var a=t.getItem().node,s=ri({treeData:this.treeData,newNode:a,depth:o,getNodeKey:this.getNodeKey,minimumTreeIndex:e.listIndex,expandParent:!0});return this.customCanDrop({node:a,prevPath:t.getItem().path,prevParent:t.getItem().parentNode,prevTreeIndex:t.getItem().treeIndex,nextPath:s.path,nextParent:s.parentNode,nextTreeIndex:s.treeIndex})}return!0}},{key:"wrapSource",value:function(e){var t=this,n={beginDrag:function(e){return t.startDrag(e),{node:e.node,parentNode:e.parentNode,path:e.path,treeIndex:e.treeIndex,treeId:e.treeId}},endDrag:function(e,n){t.endDrag(n.getDropResult())},isDragging:function(e,t){var n=t.getItem().node;return e.node===n}};return function(e,t,n){var r=arguments.length>3&&void 0!==arguments[3]?arguments[3]:{},i=e;"function"!=typeof e&&(pn()($n(e),'Expected "type" provided as the first argument to DragSource to be a string, or a function that returns a string given the current props. Instead, received %s. Read more: http://react-dnd.github.io/react-dnd/docs/api/drag-source',e),i=function(){return e}),pn()(yn(t),'Expected "spec" provided as the second argument to DragSource to be a plain object. Instead, received %s. Read more: http://react-dnd.github.io/react-dnd/docs/api/drag-source',t);var o=ir(t);return pn()("function"==typeof n,'Expected "collect" provided as the third argument to DragSource to be a function that returns a plain object of props to inject. Instead, received %s. Read more: http://react-dnd.github.io/react-dnd/docs/api/drag-source',n),pn()(yn(r),'Expected "options" provided as the fourth argument to DragSource to be a plain object when specified. Instead, received %s. Read more: http://react-dnd.github.io/react-dnd/docs/api/drag-source',n),function(e){return Rn({containerDisplayName:"DragSource",createHandler:o,registerHandler:Fn,createConnector:function(e){return new Kn(e)},createMonitor:function(e){return new Vn(e)},DecoratedComponent:e,getType:i,collect:n,options:r})}}(this.dndType,n,(function(e,t){return{connectDragSource:e.dragSource(),connectDragPreview:e.dragPreview(),isDragging:t.isDragging(),didDrop:t.didDrop()}}))(e)}},{key:"wrapTarget",value:function(e){var t=this,n={drop:function(e,n,r){var i={node:n.getItem().node,path:n.getItem().path,treeIndex:n.getItem().treeIndex,treeId:t.treeId,minimumTreeIndex:e.treeIndex,depth:t.getTargetDepth(e,n,r)};return t.drop(i),i},hover:function(e,n,r){var i=t.getTargetDepth(e,n,r),o=n.getItem().node;(e.node!==o||i!==e.path.length-1)&&(cancelAnimationFrame(t.rafId),t.rafId=requestAnimationFrame((function(){t.dragHover({node:o,path:n.getItem().path,minimumTreeIndex:e.listIndex,depth:i})})))},canDrop:this.canDrop.bind(this)};return pr(this.dndType,n,(function(e,t){var n=t.getItem();return{connectDropTarget:e.dropTarget(),isOver:t.isOver(),canDrop:t.canDrop(),draggedNode:n?n.node:null}}))(e)}},{key:"wrapPlaceholder",value:function(e){var t=this,n={drop:function(e,n){var r=n.getItem(),i={node:r.node,path:r.path,treeIndex:r.treeIndex,treeId:t.treeId,minimumTreeIndex:0,depth:0};return t.drop(i),i}};return pr(this.dndType,n,(function(e,t){var n=t.getItem();return{connectDropTarget:e.dropTarget(),isOver:t.isOver(),canDrop:t.canDrop(),draggedNode:n?n.node:null}}))(e)}},{key:"startDrag",get:function(){return this.treeRef.startDrag}},{key:"dragHover",get:function(){return this.treeRef.dragHover}},{key:"endDrag",get:function(){return this.treeRef.endDrag}},{key:"drop",get:function(){return this.treeRef.drop}},{key:"treeId",get:function(){return this.treeRef.treeId}},{key:"dndType",get:function(){return this.treeRef.dndType}},{key:"treeData",get:function(){return this.treeRef.state.draggingTreeData||this.treeRef.props.treeData}},{key:"getNodeKey",get:function(){return this.treeRef.props.getNodeKey}},{key:"customCanDrop",get:function(){return this.treeRef.props.canDrop}},{key:"maxDepth",get:function(){return this.treeRef.props.maxDepth}}]),e}(),ci=1,di=function(e){var t=_r({},e,{style:_r({},e.theme.style,e.style),innerStyle:_r({},e.theme.innerStyle,e.innerStyle),reactVirtualizedListProps:_r({},e.theme.reactVirtualizedListProps,e.reactVirtualizedListProps)}),n={nodeContentRenderer:$r,placeholderRenderer:ti,rowHeight:62,scaffoldBlockPxWidth:44,slideRegionSize:100,treeNodeRenderer:Pr};return Object.keys(n).forEach((function(r){null===e[r]&&(t[r]=void 0!==e.theme[r]?e.theme[r]:n[r])})),t},hi=function(e){function t(e){var n;mr(this,t),n=Tr(this,xr(t).call(this,e));var r=di(e),i=r.dndType,o=r.nodeContentRenderer,a=r.treeNodeRenderer,s=r.isVirtualized,l=r.slideRegionSize;return n.dndManager=new ui(Sr(Sr(n))),n.treeId="rst__".concat(ci),ci+=1,n.dndType=i||n.treeId,n.nodeContentRenderer=n.dndManager.wrapSource(o),n.treePlaceholderRenderer=n.dndManager.wrapPlaceholder(ei),n.treeNodeRenderer=n.dndManager.wrapTarget(a),s&&(n.scrollZoneVirtualList=(jt.createScrollingComponent||Rt.a)(Ee),n.vStrength=Object(jt.createVerticalStrength)(l),n.hStrength=Object(jt.createHorizontalStrength)(l)),n.state={draggingTreeData:null,draggedNode:null,draggedMinimumTreeIndex:null,draggedDepth:null,searchMatches:[],searchFocusTreeIndex:null,dragging:!1,instanceProps:{treeData:[],ignoreOneTreeUpdate:!1,searchQuery:null,searchFocusOffset:null}},n.toggleChildrenVisibility=n.toggleChildrenVisibility.bind(Sr(Sr(n))),n.moveNode=n.moveNode.bind(Sr(Sr(n))),n.startDrag=n.startDrag.bind(Sr(Sr(n))),n.dragHover=n.dragHover.bind(Sr(Sr(n))),n.endDrag=n.endDrag.bind(Sr(Sr(n))),n.drop=n.drop.bind(Sr(Sr(n))),n.handleDndMonitorChange=n.handleDndMonitorChange.bind(Sr(Sr(n))),n}return Ir(t,e),br(t,[{key:"componentDidMount",value:function(){t.loadLazyChildren(this.props,this.state);var e=t.search(this.props,this.state,!0,!0,!1);this.setState(e),this.clearMonitorSubscription=this.props.dragDropManager.getMonitor().subscribeToStateChange(this.handleDndMonitorChange)}},{key:"componentDidUpdate",value:function(e,t){this.state.dragging!==t.dragging&&this.props.onDragStateChanged&&this.props.onDragStateChanged({isDragging:this.state.dragging,draggedNode:this.state.draggedNode})}},{key:"componentWillUnmount",value:function(){this.clearMonitorSubscription()}},{key:"getRows",value:function(e){return ii({ignoreCollapsed:!0,getNodeKey:this.props.getNodeKey,treeData:e})}},{key:"handleDndMonitorChange",value:function(){!this.props.dragDropManager.getMonitor().isDragging()&&this.state.draggingTreeData&&this.endDrag()}},{key:"toggleChildrenVisibility",value:function(e){var t=e.node,n=e.path,r=Ur({treeData:this.state.instanceProps.treeData,path:n,newNode:function(e){var t=e.node;return _r({},t,{expanded:!t.expanded})},getNodeKey:this.props.getNodeKey});this.props.onChange(r),this.props.onVisibilityToggle({treeData:r,node:t,expanded:!t.expanded,path:n})}},{key:"moveNode",value:function(e){var t=e.node,n=e.path,r=e.treeIndex,i=e.depth,o=e.minimumTreeIndex,a=Xr({treeData:this.state.draggingTreeData,newNode:t,depth:i,minimumTreeIndex:o,expandParent:!0,getNodeKey:this.props.getNodeKey}),s=a.treeData,l=a.treeIndex,u=a.path,c=a.parentNode;this.props.onChange(s),this.props.onMoveNode({treeData:s,node:t,treeIndex:l,path:u,nextPath:u,nextTreeIndex:l,prevPath:n,prevTreeIndex:r,nextParentNode:c})}},{key:"startDrag",value:function(e){var t=this,n=e.path;this.setState((function(e){var r=Yr({treeData:e.instanceProps.treeData,path:n,getNodeKey:t.props.getNodeKey}),i=r.treeData,o=r.node,a=r.treeIndex;return{draggingTreeData:i,draggedNode:o,draggedDepth:n.length-1,draggedMinimumTreeIndex:a,dragging:!0}}))}},{key:"dragHover",value:function(e){var t=this,n=e.node,r=e.depth,i=e.minimumTreeIndex;this.state.draggedDepth===r&&this.state.draggedMinimumTreeIndex===i||this.setState((function(e){var o=e.draggingTreeData,a=e.instanceProps,s=o||a.treeData,l=ri({treeData:s,newNode:n,depth:r,minimumTreeIndex:i,expandParent:!0,getNodeKey:t.props.getNodeKey}),u=t.getRows(l.treeData)[l.treeIndex].path;return{draggedNode:n,draggedDepth:r,draggedMinimumTreeIndex:i,draggingTreeData:Ur({treeData:s,path:u.slice(0,-1),newNode:function(e){return _r({},e.node,{expanded:!0})},getNodeKey:t.props.getNodeKey}),searchFocusTreeIndex:null,dragging:!0}}))}},{key:"endDrag",value:function(e){var t=this,n=this.state.instanceProps;if(e){if(e.treeId!==this.treeId){var r=e.node,i=e.path,o=e.treeIndex,a=this.props.shouldCopyOnOutsideDrop;"function"==typeof a&&(a=a({node:r,prevTreeIndex:o,prevPath:i}));var s=this.state.draggingTreeData||n.treeData;a&&(s=Ur({treeData:n.treeData,path:i,newNode:function(e){return _r({},e.node)},getNodeKey:this.props.getNodeKey})),this.props.onChange(s),this.props.onMoveNode({treeData:s,node:r,treeIndex:null,path:null,nextPath:null,nextTreeIndex:null,prevPath:i,prevTreeIndex:o})}}else t.setState({draggingTreeData:null,draggedNode:null,draggedMinimumTreeIndex:null,draggedDepth:null,dragging:!1})}},{key:"drop",value:function(e){this.moveNode(e)}},{key:"canNodeHaveChildren",value:function(e){var t=this.props.canNodeHaveChildren;return!t||t(e)}},{key:"renderRow",value:function(e,t){var n=t.listIndex,r=t.style,i=t.getPrevRow,o=t.matchKeys,a=t.swapFrom,s=t.swapDepth,l=t.swapLength,u=e.node,c=e.parentNode,d=e.path,h=e.lowerSiblingCounts,f=e.treeIndex,p=di(this.props),g=p.canDrag,v=p.generateNodeProps,m=p.scaffoldBlockPxWidth,y=p.searchFocusOffset,C=p.rowDirection,A=this.treeNodeRenderer,_=this.nodeContentRenderer,I=d[d.length-1],x=I in o,w=x&&o[I]===y,M={node:u,parentNode:c,path:d,lowerSiblingCounts:h,treeIndex:f,isSearchMatch:x,isSearchFocus:w},S=v?v(M):{},T="function"!=typeof g?g:g(M),E={treeIndex:f,scaffoldBlockPxWidth:m,node:u,path:d,treeId:this.treeId,rowDirection:C};return b.a.createElement(A,Ar({style:r,key:I,listIndex:n,getPrevRow:i,lowerSiblingCounts:h,swapFrom:a,swapLength:l,swapDepth:s},E),b.a.createElement(_,Ar({parentNode:c,isSearchMatch:x,isSearchFocus:w,canDrag:T,toggleChildrenVisibility:this.toggleChildrenVisibility},E,S)))}},{key:"render",value:function(){var e,t=this,n=di(this.props),r=n.dragDropManager,i=n.style,o=n.className,a=n.innerStyle,s=n.rowHeight,l=n.isVirtualized,u=n.placeholderRenderer,c=n.reactVirtualizedListProps,d=n.getNodeKey,h=n.rowDirection,f=this.state,p=f.searchMatches,g=f.searchFocusTreeIndex,v=f.draggedNode,m=f.draggedDepth,y=f.draggedMinimumTreeIndex,C=f.instanceProps,A=this.state.draggingTreeData||C.treeData,_="rtl"===h?"rst__rtl":null,I=null,x=null;if(v&&null!==y){var w=ri({treeData:A,newNode:v,depth:m,minimumTreeIndex:y,expandParent:!0,getNodeKey:d}),M=y;I=w.treeIndex,x=1+oi({node:v}),e=function(e,t,n){var r=arguments.length>3&&void 0!==arguments[3]?arguments[3]:1,i=Er(e.slice(0,t)).concat(Er(e.slice(t+r)));return Er(i.slice(0,n)).concat(Er(e.slice(t,t+r)),Er(i.slice(n)))}(this.getRows(w.treeData),I,M,x)}else e=this.getRows(A);var S={};p.forEach((function(e,t){var n=e.path;S[n[n.length-1]]=t}));var T,E=null!==g?{scrollToIndex:g}:{},O=i;if(e.length<1){var k=this.treePlaceholderRenderer,L=u;T=b.a.createElement(k,{treeId:this.treeId,drop:this.drop},b.a.createElement(L,null))}else if(l){O=_r({height:"100%"},O);var D=this.scrollZoneVirtualList;T=b.a.createElement(le,null,(function(n){var i=n.height,o=n.width;return b.a.createElement(D,Ar({},E,{dragDropManager:r,verticalStrength:t.vStrength,horizontalStrength:t.hStrength,speed:30,scrollToAlignment:"start",className:"rst__virtualScrollOverride",width:o,onScroll:function(e){var n=e.scrollTop;t.scrollTop=n},height:i,style:a,rowCount:e.length,estimatedRowSize:"function"!=typeof s?s:void 0,rowHeight:"function"!=typeof s?s:function(t){var n=t.index;return s({index:n,treeIndex:n,node:e[n].node,path:e[n].path})},rowRenderer:function(n){var r=n.index,i=n.style;return t.renderRow(e[r],{listIndex:r,style:i,getPrevRow:function(){return e[r-1]||null},matchKeys:S,swapFrom:I,swapDepth:m,swapLength:x})}},c))}))}else T=e.map((function(n,r){return t.renderRow(n,{listIndex:r,style:{height:"function"!=typeof s?s:s({index:r,treeIndex:r,node:n.node,path:n.path})},getPrevRow:function(){return e[r-1]||null},matchKeys:S,swapFrom:I,swapDepth:m,swapLength:x})}));return b.a.createElement("div",{className:Dr("rst__tree",o,_),style:O},T)}}],[{key:"getDerivedStateFromProps",value:function(e,n){var r=n.instanceProps,i={},o=Nt()(r.treeData,e.treeData);return r.treeData=e.treeData,o?Nt()(r.searchQuery,e.searchQuery)?r.searchFocusOffset!==e.searchFocusOffset&&Object.assign(i,t.search(e,n,!0,!0,!0)):Object.assign(i,t.search(e,n,!0,!0,!1)):(r.ignoreOneTreeUpdate?r.ignoreOneTreeUpdate=!1:(i.searchFocusTreeIndex=null,t.loadLazyChildren(e,n),Object.assign(i,t.search(e,n,!1,!1,!1))),i.draggingTreeData=null,i.draggedNode=null,i.draggedMinimumTreeIndex=null,i.draggedDepth=null,i.dragging=!1),r.searchQuery=e.searchQuery,r.searchFocusOffset=e.searchFocusOffset,i.instanceProps=r,i}},{key:"search",value:function(e,t,n,r,i){var o=e.onChange,a=e.getNodeKey,s=e.searchFinishCallback,l=e.searchQuery,u=e.searchMethod,c=e.searchFocusOffset,d=e.onlyExpandSearchedNodes,h=t.instanceProps;if(!l&&!u)return s&&s([]),{searchMatches:[]};var f={},p=Jr({getNodeKey:a,treeData:d?Hr({treeData:h.treeData,expanded:!1}):h.treeData,searchQuery:l,searchMethod:u||li,searchFocusOffset:c,expandAllMatchPaths:r&&!i,expandFocusMatchPaths:!!r}),g=p.treeData,v=p.matches;r&&(f.ignoreOneTreeUpdate=!0,o(g)),s&&s(v);var m=null;return n&&null!==c&&c<v.length&&(m=v[c].treeIndex),f.searchMatches=v,f.searchFocusTreeIndex=m,f}},{key:"loadLazyChildren",value:function(e,t){var n=t.instanceProps;Fr({treeData:n.treeData,getNodeKey:e.getNodeKey,callback:function(t){var r=t.node,i=t.path,o=t.lowerSiblingCounts,a=t.treeIndex;r.children&&"function"==typeof r.children&&(r.expanded||e.loadCollapsedLazyChildren)&&r.children({node:r,path:i,lowerSiblingCounts:o,treeIndex:a,done:function(t){return e.onChange(Ur({treeData:n.treeData,path:i,newNode:function(e){var n=e.node;return n===r?_r({},n,{children:t}):n},getNodeKey:e.getNodeKey}))}})}})}}]),t}(y.Component);hi.propTypes={dragDropManager:T.a.shape({getMonitor:T.a.func}).isRequired,treeData:T.a.arrayOf(T.a.object).isRequired,style:T.a.shape({}),className:T.a.string,innerStyle:T.a.shape({}),rowHeight:T.a.oneOfType([T.a.number,T.a.func]),slideRegionSize:T.a.number,reactVirtualizedListProps:T.a.shape({}),scaffoldBlockPxWidth:T.a.number,maxDepth:T.a.number,searchMethod:T.a.func,searchQuery:T.a.any,searchFocusOffset:T.a.number,searchFinishCallback:T.a.func,generateNodeProps:T.a.func,isVirtualized:T.a.bool,treeNodeRenderer:T.a.func,nodeContentRenderer:T.a.func,placeholderRenderer:T.a.func,theme:T.a.shape({style:T.a.shape({}),innerStyle:T.a.shape({}),reactVirtualizedListProps:T.a.shape({}),scaffoldBlockPxWidth:T.a.number,slideRegionSize:T.a.number,rowHeight:T.a.oneOfType([T.a.number,T.a.func]),treeNodeRenderer:T.a.func,nodeContentRenderer:T.a.func,placeholderRenderer:T.a.func}),getNodeKey:T.a.func,onChange:T.a.func.isRequired,onMoveNode:T.a.func,canDrag:T.a.oneOfType([T.a.func,T.a.bool]),canDrop:T.a.func,canNodeHaveChildren:T.a.func,shouldCopyOnOutsideDrop:T.a.oneOfType([T.a.func,T.a.bool]),onVisibilityToggle:T.a.func,dndType:T.a.string,onDragStateChanged:T.a.func,onlyExpandSearchedNodes:T.a.bool,rowDirection:T.a.string},hi.defaultProps={canDrag:!0,canDrop:null,canNodeHaveChildren:function(){return!0},className:"",dndType:null,generateNodeProps:null,getNodeKey:ai,innerStyle:{},isVirtualized:!0,maxDepth:null,treeNodeRenderer:null,nodeContentRenderer:null,onMoveNode:function(){},onVisibilityToggle:function(){},placeholderRenderer:null,reactVirtualizedListProps:{},rowHeight:null,scaffoldBlockPxWidth:null,searchFinishCallback:null,searchFocusOffset:null,searchMethod:null,searchQuery:null,shouldCopyOnOutsideDrop:!1,slideRegionSize:null,style:{},theme:{},onDragStateChanged:function(){},onlyExpandSearchedNodes:!1,rowDirection:"ltr"},function(e){var t=e.prototype;if(!t||!t.isReactComponent)throw new Error("Can only polyfill class components");if("function"!=typeof e.getDerivedStateFromProps&&"function"!=typeof t.getSnapshotBeforeUpdate)return e;var n=null,r=null,i=null;if("function"==typeof t.componentWillMount?n="componentWillMount":"function"==typeof t.UNSAFE_componentWillMount&&(n="UNSAFE_componentWillMount"),"function"==typeof t.componentWillReceiveProps?r="componentWillReceiveProps":"function"==typeof t.UNSAFE_componentWillReceiveProps&&(r="UNSAFE_componentWillReceiveProps"),"function"==typeof t.componentWillUpdate?i="componentWillUpdate":"function"==typeof t.UNSAFE_componentWillUpdate&&(i="UNSAFE_componentWillUpdate"),null!==n||null!==r||null!==i){var o=e.displayName||e.name,a="function"==typeof e.getDerivedStateFromProps?"getDerivedStateFromProps()":"getSnapshotBeforeUpdate()";throw Error("Unsafe legacy lifecycles will not be called for components using new component APIs.\n\n"+o+" uses "+a+" but also contains the following legacy lifecycles:"+(null!==n?"\n  "+n:"")+(null!==r?"\n  "+r:"")+(null!==i?"\n  "+i:"")+"\n\nThe above lifecycles should be removed. Learn more about this warning here:\nhttps://fb.me/react-async-component-lifecycle-hooks")}if("function"==typeof e.getDerivedStateFromProps&&(t.componentWillMount=Or,t.componentWillReceiveProps=kr),"function"==typeof t.getSnapshotBeforeUpdate){if("function"!=typeof t.componentDidUpdate)throw new Error("Cannot polyfill getSnapshotBeforeUpdate() for components that do not define componentDidUpdate() on the prototype");t.componentWillUpdate=Lr;var s=t.componentDidUpdate;t.componentDidUpdate=function(e,t,n){var r=this.__reactInternalSnapshotFlag?this.__reactInternalSnapshot:n;s.call(this,e,t,r)}}}(hi);var fi=function(e){return b.a.createElement(xn.a.Consumer,null,(function(t){var n=t.dragDropManager;return void 0===n?null:b.a.createElement(hi,Ar({},e,{dragDropManager:n}))}))};t.default=function(e){return b.a.createElement(gr.a,{backend:hn},b.a.createElement(fi,e))}},function(e,t,n){"use strict";n.r(t);var r=n(21),i=n(0),o=n.n(i),a=n(3),s=Object(i.createContext)(null),l=function(e){var t=e.utils,n=e.children,r=e.locale,o=e.libInstance,a=Object(i.useMemo)((function(){return new t({locale:r,instance:o})}),[t,o,r]);return Object(i.createElement)(s.Provider,{value:a,children:n})};function u(){var e=Object(i.useContext)(s);return function(e){if(!e)throw new Error("Can not find utils in context. You either a) forgot to wrap your component tree in MuiPickersUtilsProvider; or b) mixed named and direct file imports.  Recommendation: use named imports from the module index.")}(e),e}var c=n(4),d=n(1),h=n(5),f=n(187),p=n(903),g=n(26),v=n(902),m=n(7),y=n(907),b=n(928),C=n(910),A=n(911),_=n(909),I=n(633);function x(e,t){return Array.isArray(t)?t.every((function(t){return-1!==e.indexOf(t)})):-1!==e.indexOf(t)}var w=Object(p.a)((function(e){return{staticWrapperRoot:{overflow:"hidden",minWidth:310,display:"flex",flexDirection:"column",backgroundColor:e.palette.background.paper}}}),{name:"MuiPickersStaticWrapper"}),M=function(e){var t=e.children,n=w();return Object(i.createElement)("div",{className:n.staticWrapperRoot,children:t})},S=function(e){var t=e.children,n=e.classes,r=e.onAccept,o=e.onDismiss,a=e.onClear,s=e.onSetToday,l=e.okLabel,u=e.cancelLabel,f=e.clearLabel,p=e.todayLabel,g=e.clearable,v=e.showTodayButton,m=(e.showTabs,e.wider),b=Object(h.a)(e,["children","classes","onAccept","onDismiss","onClear","onSetToday","okLabel","cancelLabel","clearLabel","todayLabel","clearable","showTodayButton","showTabs","wider"]);return Object(i.createElement)(_.a,Object(d.a)({role:"dialog",onClose:o,classes:{paper:Object(c.a)(n.dialogRoot,m&&n.dialogRootWider)}},b),Object(i.createElement)(A.a,{children:t,className:n.dialog}),Object(i.createElement)(C.a,{classes:{root:Object(c.a)((g||v)&&n.withAdditionalAction)}},g&&Object(i.createElement)(y.a,{color:"primary",onClick:a},f),v&&Object(i.createElement)(y.a,{color:"primary",onClick:s},p),u&&Object(i.createElement)(y.a,{color:"primary",onClick:o},u),l&&Object(i.createElement)(y.a,{color:"primary",onClick:r},l)))};S.displayName="ModalDialog";var T=Object(v.a)({dialogRoot:{minWidth:310},dialogRootWider:{minWidth:325},dialog:{"&:first-child":{padding:0}},withAdditionalAction:{justifyContent:"flex-start","& > *:first-child":{marginRight:"auto"}}}),E=Object(m.a)(T,{name:"MuiPickersModal"})(S),O="undefined"==typeof window?i.useEffect:i.useLayoutEffect;function k(e,t){var n=t[e.key];n&&(n(),e.preventDefault())}function L(e,t){var n=Object(i.useRef)(t);n.current=t,O((function(){if(e){var t=function(e){k(e,n.current)};return window.addEventListener("keydown",t),function(){window.removeEventListener("keydown",t)}}}),[e])}var D=function(e){var t=e.open,n=e.children,r=e.okLabel,o=e.cancelLabel,a=e.clearLabel,s=e.todayLabel,l=e.showTodayButton,u=e.clearable,c=e.DialogProps,f=e.showTabs,p=e.wider,g=e.InputComponent,v=e.DateInputProps,m=e.onClear,y=e.onAccept,b=e.onDismiss,C=e.onSetToday,A=Object(h.a)(e,["open","children","okLabel","cancelLabel","clearLabel","todayLabel","showTodayButton","clearable","DialogProps","showTabs","wider","InputComponent","DateInputProps","onClear","onAccept","onDismiss","onSetToday"]);return L(t,{Enter:y}),Object(i.createElement)(i.Fragment,null,Object(i.createElement)(g,Object(d.a)({},A,v)),Object(i.createElement)(E,Object(d.a)({wider:p,showTabs:f,open:t,onClear:m,onAccept:y,onDismiss:b,onSetToday:C,clearLabel:a,todayLabel:s,okLabel:r,cancelLabel:o,clearable:u,showTodayButton:l,children:n},c)))};D.defaultProps={okLabel:"OK",cancelLabel:"Cancel",clearLabel:"Clear",todayLabel:"Today",clearable:!1,showTodayButton:!1};var P=function(e){var t=e.open,n=(e.wider,e.children),r=e.PopoverProps,o=(e.onClear,e.onDismiss),a=(e.onSetToday,e.onAccept),s=(e.showTabs,e.DateInputProps),l=e.InputComponent,u=Object(h.a)(e,["open","wider","children","PopoverProps","onClear","onDismiss","onSetToday","onAccept","showTabs","DateInputProps","InputComponent"]),c=Object(i.useRef)();return L(t,{Enter:a}),Object(i.createElement)(i.Fragment,null,Object(i.createElement)(l,Object(d.a)({},u,s,{inputRef:c})),Object(i.createElement)(I.a,Object(d.a)({open:t,onClose:o,anchorEl:c.current,anchorOrigin:{vertical:"bottom",horizontal:"center"},transformOrigin:{vertical:"top",horizontal:"center"},children:n},r)))};var N=Object(i.createContext)(null),j=function(e){var t=e.variant,n=Object(h.a)(e,["variant"]),r=function(e){switch(e){case"inline":return P;case"static":return M;default:return D}}(t);return Object(i.createElement)(N.Provider,{value:t||"dialog"},Object(i.createElement)(r,n))},R=n(930),z=n(452),F=n(918),B=n(101),H=function(e){function t(t){var n;return(n=e.call(this,t)||this)._state=null,n._del=!1,n._handleChange=function(e){var t=n.state.value,r=e.target.value,i=e.target,o=r.length>t.length,a=n._del,s=t===n.props.format(r);n.setState({value:r,local:!0},(function(){var e=i.selectionStart,l=n.props.refuse||/[^\d]+/g,u=r.substr(0,e).replace(l,"");if(n._state={input:i,before:u,op:o,di:a&&s,del:a},n.props.replace&&n.props.replace(t)&&o&&!s){for(var c=-1,d=0;d!==u.length;++d)c=Math.max(c,r.toLowerCase().indexOf(u[d].toLowerCase(),c+1));var h=r.substr(c+1).replace(l,"")[0];c=r.indexOf(h,c+1),r=""+r.substr(0,c)+r.substr(c+1)}var f=n.props.format(r);t===f?n.setState({value:r}):n.props.onChange(f)}))},n._hKD=function(e){"Delete"===e.code&&(n._del=!0)},n._hKU=function(e){"Delete"===e.code&&(n._del=!1)},n.state={value:t.value,local:!0},n}Object(B.a)(t,e),t.getDerivedStateFromProps=function(e,t){return{value:t.local?t.value:e.value,local:!1}};var n=t.prototype;return n.render=function(){var e=this._handleChange,t=this.state.value;return(0,this.props.children)({value:t,onChange:e})},n.componentWillUnmount=function(){document.removeEventListener("keydown",this._hKD),document.removeEventListener("keyup",this._hKU)},n.componentDidMount=function(){document.addEventListener("keydown",this._hKD),document.addEventListener("keyup",this._hKU)},n.componentDidUpdate=function(){var e=this._state;if(e){for(var t=this.state.value,n=-1,r=0;r!==e.before.length;++r)n=Math.max(n,t.toLowerCase().indexOf(e.before[r].toLowerCase(),n+1));if(this.props.replace&&(e.op||e.del&&!e.di))for(;t[n+1]&&(this.props.refuse||/[^\d]+/).test(t[n+1]);)n+=1;e.input.selectionStart=e.input.selectionEnd=n+1+(e.di?1:0)}this._state=null},t}(i.Component),U=n(1309),V=n(49),Y=n(52),W=n(99),G=n(80),X=n(139),q=n(127),Z=n(138),Q=Object(p.a)((function(e){return{day:{width:36,height:36,fontSize:e.typography.caption.fontSize,margin:"0 2px",color:e.palette.text.primary,fontWeight:e.typography.fontWeightMedium,padding:0},hidden:{opacity:0,pointerEvents:"none"},current:{color:e.palette.primary.main,fontWeight:600},daySelected:{color:e.palette.primary.contrastText,backgroundColor:e.palette.primary.main,fontWeight:e.typography.fontWeightMedium,"&:hover":{backgroundColor:e.palette.primary.main}},dayDisabled:{pointerEvents:"none",color:e.palette.text.hint}}}),{name:"MuiPickersDay"}),K=function(e){var t=e.children,n=e.disabled,r=e.hidden,o=e.current,a=e.selected,s=Object(h.a)(e,["children","disabled","hidden","current","selected"]),l=Q(),u=Object(c.a)(l.day,r&&l.hidden,o&&l.current,a&&l.daySelected,n&&l.dayDisabled);return Object(i.createElement)(z.a,Object(d.a)({className:u,tabIndex:r||n?-1:0},s),Object(i.createElement)(f.a,{variant:"body2",color:"inherit"},t))};K.displayName="Day",K.defaultProps={disabled:!1,hidden:!1,current:!1,selected:!1};var J=K,$=n(2382),ee=n(92);function te(e,t){return e.replace(new RegExp("(^|\\s)"+t+"(?:\\s|$)","g"),"$1").replace(/\s+/g," ").replace(/^\s*|\s*$/g,"")}var ne=n(341),re=function(e,t){return e&&t&&t.split(" ").forEach((function(t){return r=t,void((n=e).classList?n.classList.remove(r):"string"==typeof n.className?n.className=te(n.className,r):n.setAttribute("class",te(n.className&&n.className.baseVal||"",r)));var n,r}))},ie=function(e){function t(){for(var t,n=arguments.length,r=new Array(n),i=0;i<n;i++)r[i]=arguments[i];return(t=e.call.apply(e,[this].concat(r))||this).appliedClasses={appear:{},enter:{},exit:{}},t.onEnter=function(e,n){var r=t.resolveArguments(e,n),i=r[0],o=r[1];t.removeClasses(i,"exit"),t.addClass(i,o?"appear":"enter","base"),t.props.onEnter&&t.props.onEnter(e,n)},t.onEntering=function(e,n){var r=t.resolveArguments(e,n),i=r[0],o=r[1]?"appear":"enter";t.addClass(i,o,"active"),t.props.onEntering&&t.props.onEntering(e,n)},t.onEntered=function(e,n){var r=t.resolveArguments(e,n),i=r[0],o=r[1]?"appear":"enter";t.removeClasses(i,o),t.addClass(i,o,"done"),t.props.onEntered&&t.props.onEntered(e,n)},t.onExit=function(e){var n=t.resolveArguments(e)[0];t.removeClasses(n,"appear"),t.removeClasses(n,"enter"),t.addClass(n,"exit","base"),t.props.onExit&&t.props.onExit(e)},t.onExiting=function(e){var n=t.resolveArguments(e)[0];t.addClass(n,"exit","active"),t.props.onExiting&&t.props.onExiting(e)},t.onExited=function(e){var n=t.resolveArguments(e)[0];t.removeClasses(n,"exit"),t.addClass(n,"exit","done"),t.props.onExited&&t.props.onExited(e)},t.resolveArguments=function(e,n){return t.props.nodeRef?[t.props.nodeRef.current,e]:[e,n]},t.getClassNames=function(e){var n=t.props.classNames,r="string"==typeof n,i=r?""+(r&&n?n+"-":"")+e:n[e];return{baseClassName:i,activeClassName:r?i+"-active":n[e+"Active"],doneClassName:r?i+"-done":n[e+"Done"]}},t}Object(B.a)(t,e);var n=t.prototype;return n.addClass=function(e,t,n){var r=this.getClassNames(t)[n+"ClassName"],i=this.getClassNames("enter").doneClassName;"appear"===t&&"done"===n&&i&&(r+=" "+i),"active"===n&&e&&e.scrollTop,r&&(this.appliedClasses[t][n]=r,function(e,t){e&&t&&t.split(" ").forEach((function(t){return r=t,void((n=e).classList?n.classList.add(r):function(e,t){return e.classList?!!t&&e.classList.contains(t):-1!==(" "+(e.className.baseVal||e.className)+" ").indexOf(" "+t+" ")}(n,r)||("string"==typeof n.className?n.className=n.className+" "+r:n.setAttribute("class",(n.className&&n.className.baseVal||"")+" "+r)));var n,r}))}(e,r))},n.removeClasses=function(e,t){var n=this.appliedClasses[t],r=n.base,i=n.active,o=n.done;this.appliedClasses[t]={},r&&re(e,r),i&&re(e,i),o&&re(e,o)},n.render=function(){var e=this.props,t=(e.classNames,Object(ee.a)(e,["classNames"]));return o.a.createElement(ne.a,Object(d.a)({},t,{onEnter:this.onEnter,onEntered:this.onEntered,onEntering:this.onEntering,onExit:this.onExit,onExiting:this.onExiting,onExited:this.onExited}))},t}(o.a.Component);ie.defaultProps={classNames:""},ie.propTypes={};var oe=ie,ae=n(908),se=function(e){return 1===e.length&&"year"===e[0]},le=function(e){return 2===e.length&&x(e,"month")&&x(e,"year")},ue=function(e){var t=e.children,n=e.value,r=e.disabled,o=e.onSelect,a=e.dayInCurrentMonth,s=Object(h.a)(e,["children","value","disabled","onSelect","dayInCurrentMonth"]),l=Object(i.useCallback)((function(){return o(n)}),[o,n]);return Object(i.createElement)("div",Object(d.a)({role:"presentation",onClick:a&&!r?l:void 0,onKeyPress:a&&!r?l:void 0},s),t)},ce=Object(p.a)((function(e){var t=e.transitions.create("transform",{duration:350,easing:"cubic-bezier(0.35, 0.8, 0.4, 1)"});return{transitionContainer:{display:"block",position:"relative","& > *":{position:"absolute",top:0,right:0,left:0}},"slideEnter-left":{willChange:"transform",transform:"translate(100%)"},"slideEnter-right":{willChange:"transform",transform:"translate(-100%)"},slideEnterActive:{transform:"translate(0%)",transition:t},slideExit:{transform:"translate(0%)"},"slideExitActiveLeft-left":{willChange:"transform",transform:"translate(-200%)",transition:t},"slideExitActiveLeft-right":{willChange:"transform",transform:"translate(200%)",transition:t}}}),{name:"MuiPickersSlideTransition"}),de=function(e){var t=e.children,n=e.transKey,r=e.slideDirection,o=e.className,a=void 0===o?null:o,s=ce(),l={exit:s.slideExit,enterActive:s.slideEnterActive,enter:s["slideEnter-"+r],exitActive:s["slideExitActiveLeft-"+r]};return Object(i.createElement)($.a,{className:Object(c.a)(s.transitionContainer,a),childFactory:function(e){return Object(i.cloneElement)(e,{classNames:l})}},Object(i.createElement)(oe,{mountOnEnter:!0,unmountOnExit:!0,key:n+r,timeout:350,classNames:l,children:t}))},he=Object(p.a)((function(e){return{switchHeader:{display:"flex",justifyContent:"space-between",alignItems:"center",marginTop:e.spacing(.5),marginBottom:e.spacing(1)},transitionContainer:{width:"100%",overflow:"hidden",height:23},iconButton:{zIndex:1,backgroundColor:e.palette.background.paper},daysHeader:{display:"flex",justifyContent:"center",alignItems:"center",maxHeight:16},dayLabel:{width:36,margin:"0 2px",textAlign:"center",color:e.palette.text.hint}}}),{name:"MuiPickersCalendarHeader"}),fe=function(e){var t=e.currentMonth,n=e.onMonthChange,r=e.leftArrowIcon,o=e.rightArrowIcon,a=e.leftArrowButtonProps,s=e.rightArrowButtonProps,l=e.disablePrevMonth,c=e.disableNextMonth,h=e.slideDirection,p=u(),g=he(),v="rtl"===Object(Y.a)().direction;return Object(i.createElement)("div",null,Object(i.createElement)("div",{className:g.switchHeader},Object(i.createElement)(z.a,Object(d.a)({},a,{disabled:l,onClick:function(){return n(p.getPreviousMonth(t),"right")},className:g.iconButton}),v?o:r),Object(i.createElement)(de,{slideDirection:h,transKey:t.toString(),className:g.transitionContainer},Object(i.createElement)(f.a,{align:"center",variant:"body1"},p.getCalendarHeaderText(t))),Object(i.createElement)(z.a,Object(d.a)({},s,{disabled:c,onClick:function(){return n(p.getNextMonth(t),"left")},className:g.iconButton}),v?r:o)),Object(i.createElement)("div",{className:g.daysHeader},p.getWeekdays().map((function(e,t){return Object(i.createElement)(f.a,{key:t,variant:"caption",className:g.dayLabel},e)}))))};fe.displayName="CalendarHeader",fe.defaultProps={leftArrowIcon:Object(i.createElement)((function(e){return o.a.createElement(U.a,e,o.a.createElement("path",{d:"M15.41 16.59L10.83 12l4.58-4.59L14 6l-6 6 6 6 1.41-1.41z"}),o.a.createElement("path",{fill:"none",d:"M0 0h24v24H0V0z"}))}),null),rightArrowIcon:Object(i.createElement)((function(e){return o.a.createElement(U.a,e,o.a.createElement("path",{d:"M8.59 16.59L13.17 12 8.59 7.41 10 6l6 6-6 6-1.41-1.41z"}),o.a.createElement("path",{fill:"none",d:"M0 0h24v24H0V0z"}))}),null),disablePrevMonth:!1,disableNextMonth:!1};var pe=function(e){var t=e.onKeyDown;return Object(i.useEffect)((function(){return window.addEventListener("keydown",t),function(){window.removeEventListener("keydown",t)}}),[t]),null},ge=function(e){function t(){var e,n;Object(W.a)(this,t);for(var r=arguments.length,o=new Array(r),a=0;a<r;a++)o[a]=arguments[a];return(n=Object(X.a)(this,(e=Object(q.a)(t)).call.apply(e,[this].concat(o)))).state={slideDirection:"left",currentMonth:n.props.utils.startOfMonth(n.props.date),loadingQueue:0},n.pushToLoadingQueue=function(){var e=n.state.loadingQueue+1;n.setState({loadingQueue:e})},n.popFromLoadingQueue=function(){var e=n.state.loadingQueue;e=e<=0?0:e-1,n.setState({loadingQueue:e})},n.handleChangeMonth=function(e,t){if(n.setState({currentMonth:e,slideDirection:t}),n.props.onMonthChange){var r=n.props.onMonthChange(e);r&&(n.pushToLoadingQueue(),r.then((function(){n.popFromLoadingQueue()})))}},n.validateMinMaxDate=function(e){var t=n.props,r=t.minDate,i=t.maxDate,o=t.utils,a=t.disableFuture,s=t.disablePast,l=o.date();return Boolean(a&&o.isAfterDay(e,l)||s&&o.isBeforeDay(e,l)||r&&o.isBeforeDay(e,o.date(r))||i&&o.isAfterDay(e,o.date(i)))},n.shouldDisablePrevMonth=function(){var e=n.props,t=e.utils,r=e.disablePast,i=e.minDate,o=t.date(),a=t.startOfMonth(r&&t.isAfter(o,t.date(i))?o:t.date(i));return!t.isBefore(a,n.state.currentMonth)},n.shouldDisableNextMonth=function(){var e=n.props,t=e.utils,r=e.disableFuture,i=e.maxDate,o=t.date(),a=t.startOfMonth(r&&t.isBefore(o,t.date(i))?o:t.date(i));return!t.isAfter(a,n.state.currentMonth)},n.shouldDisableDate=function(e){var t=n.props.shouldDisableDate;return n.validateMinMaxDate(e)||Boolean(t&&t(e))},n.handleDaySelect=function(e){var t=!(arguments.length>1&&void 0!==arguments[1])||arguments[1],r=n.props,i=r.date,o=r.utils;n.props.onChange(o.mergeDateAndTime(e,i),t)},n.moveToDay=function(e){var t=n.props.utils;e&&!n.shouldDisableDate(e)&&(t.getMonth(e)!==t.getMonth(n.state.currentMonth)&&n.handleChangeMonth(t.startOfMonth(e),"left"),n.handleDaySelect(e,!1))},n.handleKeyDown=function(e){var t=n.props,r=t.theme,i=t.date,o=t.utils;k(e,{ArrowUp:function(){return n.moveToDay(o.addDays(i,-7))},ArrowDown:function(){return n.moveToDay(o.addDays(i,7))},ArrowLeft:function(){return n.moveToDay(o.addDays(i,"ltr"===r.direction?-1:1))},ArrowRight:function(){return n.moveToDay(o.addDays(i,"ltr"===r.direction?1:-1))}})},n.renderWeeks=function(){var e=n.props,t=e.utils,r=e.classes;return t.getWeekArray(n.state.currentMonth).map((function(e){return Object(i.createElement)("div",{key:"week-".concat(e[0].toString()),className:r.week},n.renderDays(e))}))},n.renderDays=function(e){var t=n.props,r=t.date,o=t.renderDay,a=t.utils,s=a.date(),l=a.startOfDay(r),u=a.getMonth(n.state.currentMonth);return e.map((function(e){var t=n.shouldDisableDate(e),r=a.getMonth(e)===u,c=Object(i.createElement)(J,{disabled:t,current:a.isSameDay(e,s),hidden:!r,selected:a.isSameDay(l,e)},a.getDayText(e));return o&&(c=o(e,l,r,c)),Object(i.createElement)(ue,{value:e,key:e.toString(),disabled:t,dayInCurrentMonth:r,onSelect:n.handleDaySelect},c)}))},n}return Object(Z.a)(t,e),Object(G.a)(t,[{key:"componentDidMount",value:function(){var e=this.props,t=e.date,n=e.minDate,r=e.maxDate,i=e.utils,o=e.disablePast,a=e.disableFuture;if(this.shouldDisableDate(t)){var s=function(e){var t=e.date,n=e.utils,r=e.minDate,i=e.maxDate,o=e.disableFuture,a=e.disablePast,s=e.shouldDisableDate,l=n.startOfDay(n.date());a&&n.isBefore(r,l)&&(r=l),o&&n.isAfter(i,l)&&(i=l);var u=t,c=t;for(n.isBefore(t,r)&&(u=n.date(r),c=null),n.isAfter(t,i)&&(c&&(c=n.date(i)),u=null);u||c;){if(u&&n.isAfter(u,i)&&(u=null),c&&n.isBefore(c,r)&&(c=null),u){if(!s(u))return u;u=n.addDays(u,1)}if(c){if(!s(c))return c;c=n.addDays(c,-1)}}return n.date()}({date:t,utils:i,minDate:i.date(n),maxDate:i.date(r),disablePast:Boolean(o),disableFuture:Boolean(a),shouldDisableDate:this.shouldDisableDate});this.handleDaySelect(s,!1)}}},{key:"render",value:function(){var e=this.state,t=e.currentMonth,n=e.slideDirection,r=this.props,o=r.classes,a=r.allowKeyboardControl,s=r.leftArrowButtonProps,l=r.leftArrowIcon,u=r.rightArrowButtonProps,c=r.rightArrowIcon,d=r.loadingIndicator,h=d||Object(i.createElement)(ae.a,null);return Object(i.createElement)(i.Fragment,null,a&&"static"!==this.context&&Object(i.createElement)(pe,{onKeyDown:this.handleKeyDown}),Object(i.createElement)(fe,{currentMonth:t,slideDirection:n,onMonthChange:this.handleChangeMonth,leftArrowIcon:l,leftArrowButtonProps:s,rightArrowIcon:c,rightArrowButtonProps:u,disablePrevMonth:this.shouldDisablePrevMonth(),disableNextMonth:this.shouldDisableNextMonth()}),Object(i.createElement)(de,{slideDirection:n,transKey:t.toString(),className:o.transitionContainer},Object(i.createElement)(i.Fragment,null,this.state.loadingQueue>0&&Object(i.createElement)("div",{className:o.progressContainer},h)||Object(i.createElement)("div",null,this.renderWeeks()))))}}],[{key:"getDerivedStateFromProps",value:function(e,t){var n=e.utils,r=e.date;if(!n.isEqual(r,t.lastDate)){var i=n.getMonth(r),o=t.lastDate||r,a=n.getMonth(o);return{lastDate:r,currentMonth:e.utils.startOfMonth(r),slideDirection:i===a?t.slideDirection:n.isAfterDay(r,o)?"left":"right"}}return null}}]),t}(i.Component);ge.contextType=N,ge.defaultProps={minDate:new Date("1900-01-01"),maxDate:new Date("2100-01-01"),disablePast:!1,disableFuture:!1,allowKeyboardControl:!0};var ve,me,ye,be=Object(m.a)((function(e){return{transitionContainer:{minHeight:216,marginTop:e.spacing(1.5)},progressContainer:{width:"100%",height:"100%",display:"flex",justifyContent:"center",alignItems:"center"},week:{display:"flex",justifyContent:"center"}}}),{name:"MuiPickersCalendar",withTheme:!0})(((me=function(e){var t=u();return Object(i.createElement)(ve,Object(d.a)({utils:t},e))}).displayName="WithUtils(".concat((ve=ge).displayName||ve.name,")"),me));!function(e){e.HOURS="hours",e.MINUTES="minutes",e.SECONDS="seconds"}(ye||(ye={}));var Ce=ye,Ae=function(e){function t(){var e,n;Object(W.a)(this,t);for(var r=arguments.length,i=new Array(r),o=0;o<r;o++)i[o]=arguments[o];return(n=Object(X.a)(this,(e=Object(q.a)(t)).call.apply(e,[this].concat(i)))).state={toAnimateTransform:!1,previousType:void 0},n.getAngleStyle=function(){var e=n.props,t=e.value,r=e.isInner,i=e.type,o=360/(i===Ce.HOURS?12:60)*t;return i===Ce.HOURS&&t>12&&(o-=360),{height:r?"26%":"40%",transform:"rotateZ(".concat(o,"deg)")}},n}return Object(Z.a)(t,e),Object(G.a)(t,[{key:"render",value:function(){var e=this.props,t=e.classes,n=e.hasSelected;return Object(i.createElement)("div",{style:this.getAngleStyle(),className:Object(c.a)(t.pointer,this.state.toAnimateTransform&&t.animateTransform)},Object(i.createElement)("div",{className:Object(c.a)(t.thumb,n&&t.noPoint)}))}}]),t}(i.Component);Ae.getDerivedStateFromProps=function(e,t){return e.type!==t.previousType?{toAnimateTransform:!0,previousType:e.type}:{toAnimateTransform:!1,previousType:e.type}};var _e=Object(m.a)((function(e){return Object(v.a)({pointer:{width:2,backgroundColor:e.palette.primary.main,position:"absolute",left:"calc(50% - 1px)",bottom:"50%",transformOrigin:"center bottom 0px"},animateTransform:{transition:e.transitions.create(["transform","height"])},thumb:{width:4,height:4,backgroundColor:e.palette.primary.contrastText,borderRadius:"100%",position:"absolute",top:-21,left:-15,border:"14px solid ".concat(e.palette.primary.main),boxSizing:"content-box"},noPoint:{backgroundColor:e.palette.primary.main}})}),{name:"MuiPickersClockPointer"})(Ae),Ie={x:130,y:130},xe=Ie.x-Ie.x,we=0-Ie.y,Me=function(e,t,n){var r=t-Ie.x,i=n-Ie.y,o=Math.atan2(xe,we)-Math.atan2(r,i),a=57.29577951308232*o;a=Math.round(a/e)*e,a%=360;var s=Math.floor(a/e)||0,l=Math.pow(r,2)+Math.pow(i,2);return{value:s,distance:Math.sqrt(l)}},Se=function(e,t,n){var r=Me(30,e,t),i=r.value,o=r.distance;return i=i||12,n?i%=12:o<90&&(i+=12,i%=24),i},Te=function(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:1,r=6*n,i=Me(r,e,t),o=i.value;return o=o*n%60},Ee=function(e,t){return t.getHours(e)>=12?"pm":"am"},Oe=function(e,t,n,r){if(n&&(r.getHours(e)>=12?"pm":"am")!==t){var i="am"===t?r.getHours(e)-12:r.getHours(e)+12;return r.setHours(e,i)}return e},ke=function(e){function t(){var e,n;Object(W.a)(this,t);for(var r=arguments.length,i=new Array(r),o=0;o<r;o++)i[o]=arguments[o];return(n=Object(X.a)(this,(e=Object(q.a)(t)).call.apply(e,[this].concat(i)))).isMoving=!1,n.handleTouchMove=function(e){n.isMoving=!0,n.setTime(e)},n.handleTouchEnd=function(e){n.isMoving&&(n.setTime(e,!0),n.isMoving=!1)},n.handleMove=function(e){e.preventDefault(),e.stopPropagation(),(void 0===e.buttons?1===e.nativeEvent.which:1===e.buttons)&&n.setTime(e.nativeEvent,!1)},n.handleMouseUp=function(e){n.isMoving&&(n.isMoving=!1),n.setTime(e.nativeEvent,!0)},n.hasSelected=function(){var e=n.props,t=e.type,r=e.value;return t===Ce.HOURS||r%5==0},n}return Object(Z.a)(t,e),Object(G.a)(t,[{key:"setTime",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]&&arguments[1],n=e.offsetX,r=e.offsetY;if(void 0===n){var i=e.target.getBoundingClientRect();n=e.changedTouches[0].clientX-i.left,r=e.changedTouches[0].clientY-i.top}var o=this.props.type===Ce.SECONDS||this.props.type===Ce.MINUTES?Te(n,r,this.props.minutesStep):Se(n,r,Boolean(this.props.ampm));this.props.onChange(o,t)}},{key:"render",value:function(){var e=this.props,t=e.classes,n=e.value,r=e.children,o=e.type,a=!e.ampm&&o===Ce.HOURS&&(n<1||n>12);return Object(i.createElement)("div",{className:t.container},Object(i.createElement)("div",{className:t.clock},Object(i.createElement)("div",{role:"menu",tabIndex:-1,className:t.squareMask,onTouchMove:this.handleTouchMove,onTouchEnd:this.handleTouchEnd,onMouseUp:this.handleMouseUp,onMouseMove:this.handleMove}),Object(i.createElement)("div",{className:t.pin}),Object(i.createElement)(_e,{type:o,value:n,isInner:a,hasSelected:this.hasSelected()}),r))}}]),t}(i.Component);ke.defaultProps={ampm:!1,minutesStep:1};var Le=Object(m.a)((function(e){return Object(v.a)({container:{display:"flex",justifyContent:"center",alignItems:"flex-end",margin:"".concat(e.spacing(2),"px 0 ").concat(e.spacing(1),"px")},clock:{backgroundColor:"rgba(0,0,0,.07)",borderRadius:"50%",height:260,width:260,position:"relative",pointerEvents:"none"},squareMask:{width:"100%",height:"100%",position:"absolute",pointerEvents:"auto",outline:"none",touchActions:"none",userSelect:"none","&:active":{cursor:"move"}},pin:{width:6,height:6,borderRadius:"50%",backgroundColor:e.palette.primary.main,position:"absolute",top:"50%",left:"50%",transform:"translate(-50%, -50%)"}})}),{name:"MuiPickersClock"})(ke),De={0:[0,40],1:[55,19.6],2:[94.4,59.5],3:[109,114],4:[94.4,168.5],5:[54.5,208.4],6:[0,223],7:[-54.5,208.4],8:[-94.4,168.5],9:[-109,114],10:[-94.4,59.5],11:[-54.5,19.6],12:[0,5],13:[36.9,49.9],14:[64,77],15:[74,114],16:[64,151],17:[37,178],18:[0,188],19:[-37,178],20:[-64,151],21:[-74,114],22:[-64,77],23:[-37,50]},Pe=Object(p.a)((function(e){var t=e.spacing(4);return{clockNumber:{width:t,height:32,userSelect:"none",position:"absolute",left:"calc((100% - ".concat("number"==typeof t?"".concat(t,"px"):t,") / 2)"),display:"inline-flex",justifyContent:"center",alignItems:"center",borderRadius:"50%",color:"light"===e.palette.type?e.palette.text.primary:e.palette.text.hint},clockNumberSelected:{color:e.palette.primary.contrastText}}}),{name:"MuiPickersClockNumber"}),Ne=function(e){var t=e.selected,n=e.label,r=e.index,o=e.isInner,a=Pe(),s=Object(c.a)(a.clockNumber,t&&a.clockNumberSelected),l=Object(i.useMemo)((function(){var e=De[r];return{transform:"translate(".concat(e[0],"px, ").concat(e[1],"px")}}),[r]);return Object(i.createElement)(f.a,{component:"span",className:s,variant:o?"body2":"body1",style:l,children:n})},je=function(e){for(var t=e.ampm,n=e.utils,r=e.date,o=n.getHours(r),a=[],s=t?12:23,l=function(e){return t?12===e?12===o||0===o:o===e||o-12===e:o===e},u=t?1:0;u<=s;u+=1){var c=u.toString();0===u&&(c="00");var h={index:u,label:n.formatNumber(c),selected:l(u),isInner:!t&&(0===u||u>12)};a.push(Object(i.createElement)(Ne,Object(d.a)({key:u},h)))}return a},Re=function(e){var t=e.value,n=e.utils.formatNumber;return[Object(i.createElement)(Ne,{label:n("00"),selected:0===t,index:12,key:12}),Object(i.createElement)(Ne,{label:n("05"),selected:5===t,index:1,key:1}),Object(i.createElement)(Ne,{label:n("10"),selected:10===t,index:2,key:2}),Object(i.createElement)(Ne,{label:n("15"),selected:15===t,index:3,key:3}),Object(i.createElement)(Ne,{label:n("20"),selected:20===t,index:4,key:4}),Object(i.createElement)(Ne,{label:n("25"),selected:25===t,index:5,key:5}),Object(i.createElement)(Ne,{label:n("30"),selected:30===t,index:6,key:6}),Object(i.createElement)(Ne,{label:n("35"),selected:35===t,index:7,key:7}),Object(i.createElement)(Ne,{label:n("40"),selected:40===t,index:8,key:8}),Object(i.createElement)(Ne,{label:n("45"),selected:45===t,index:9,key:9}),Object(i.createElement)(Ne,{label:n("50"),selected:50===t,index:10,key:10}),Object(i.createElement)(Ne,{label:n("55"),selected:55===t,index:11,key:11})]},ze=function(e){var t=e.type,n=e.onHourChange,r=e.onMinutesChange,o=e.onSecondsChange,a=e.ampm,s=e.date,l=e.minutesStep,c=u(),h=Object(i.useMemo)((function(){switch(t){case Ce.HOURS:return{value:c.getHours(s),children:je({date:s,utils:c,ampm:Boolean(a)}),onChange:function(e,t){var r=Ee(s,c),i=Oe(c.setHours(s,e),r,Boolean(a),c);n(i,t)}};case Ce.MINUTES:var e=c.getMinutes(s);return{value:e,children:Re({value:e,utils:c}),onChange:function(e,t){var n=c.setMinutes(s,e);r(n,t)}};case Ce.SECONDS:var i=c.getSeconds(s);return{value:i,children:Re({value:i,utils:c}),onChange:function(e,t){var n=c.setSeconds(s,e);o(n,t)}};default:throw new Error("You must provide the type for TimePickerView")}}),[a,s,n,r,o,t,c]);return Object(i.createElement)(Le,Object(d.a)({type:t,ampm:a,minutesStep:l},h))};ze.displayName="TimePickerView",ze.defaultProps={ampm:!0,minutesStep:1};var Fe=Object(i.memo)(ze);function Be(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),n.push.apply(n,r)}return n}Object(a.oneOfType)([a.object,a.string,a.number,Object(a.instanceOf)(Date)]),Object(a.oneOf)(["year","month","day"]);var He={ampm:!0,invalidDateMessage:"Invalid Time Format"},Ue={minDate:new Date("1900-01-01"),maxDate:new Date("2100-01-01"),invalidDateMessage:"Invalid Date Format",minDateMessage:"Date should not be before minimal date",maxDateMessage:"Date should not be after maximal date",allowKeyboardControl:!0},Ve=function(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?Be(n,!0).forEach((function(t){Object(r.a)(e,t,n[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):Be(n).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))}))}return e}({},He,{},Ue,{showTabs:!0});var Ye=Object(p.a)((function(e){return{root:{height:40,display:"flex",alignItems:"center",justifyContent:"center",cursor:"pointer",outline:"none","&:focus":{color:e.palette.primary.main,fontWeight:e.typography.fontWeightMedium}},yearSelected:{margin:"10px 0",fontWeight:e.typography.fontWeightMedium},yearDisabled:{pointerEvents:"none",color:e.palette.text.hint}}}),{name:"MuiPickersYear"}),We=function(e){var t=e.onSelect,n=e.forwardedRef,r=e.value,o=e.selected,a=e.disabled,s=e.children,l=Object(h.a)(e,["onSelect","forwardedRef","value","selected","disabled","children"]),u=Ye(),p=Object(i.useCallback)((function(){return t(r)}),[t,r]);return Object(i.createElement)(f.a,Object(d.a)({role:"button",component:"div",tabIndex:a?-1:0,onClick:p,onKeyPress:p,color:o?"primary":void 0,variant:o?"h5":"subtitle1",children:s,ref:n,className:Object(c.a)(u.root,o&&u.yearSelected,a&&u.yearDisabled)},l))};We.displayName="Year";var Ge=Object(i.forwardRef)((function(e,t){return Object(i.createElement)(We,Object(d.a)({},e,{forwardedRef:t}))})),Xe=Object(p.a)({container:{height:300,overflowY:"auto"}},{name:"MuiPickersYearSelection"}),qe=function(e){var t=e.date,n=e.onChange,r=e.onYearChange,o=e.minDate,a=e.maxDate,s=e.disablePast,l=e.disableFuture,c=e.animateYearScrolling,d=u(),h=Xe(),f=Object(i.useContext)(N),p=Object(i.useRef)(null);Object(i.useEffect)((function(){if(p.current&&p.current.scrollIntoView)try{p.current.scrollIntoView({block:"static"===f?"nearest":"center",behavior:c?"smooth":"auto"})}catch(e){p.current.scrollIntoView()}}),[]);var g=d.getYear(t),v=Object(i.useCallback)((function(e){var i=d.setYear(t,e);r&&r(i),n(i,!0)}),[t,n,r,d]);return Object(i.createElement)("div",{className:h.container},d.getYearRange(o,a).map((function(e){var t=d.getYear(e),n=t===g;return Object(i.createElement)(Ge,{key:d.getYearText(e),selected:n,value:t,onSelect:v,ref:n?p:void 0,disabled:Boolean(s&&d.isBeforeYear(e,d.date())||l&&d.isAfterYear(e,d.date()))},d.getYearText(e))})))},Ze=Object(p.a)((function(e){return{root:{flex:"1 0 33.33%",display:"flex",alignItems:"center",justifyContent:"center",cursor:"pointer",outline:"none",height:75,transition:e.transitions.create("font-size",{duration:"100ms"}),"&:focus":{color:e.palette.primary.main,fontWeight:e.typography.fontWeightMedium}},monthSelected:{color:e.palette.primary.main,fontWeight:e.typography.fontWeightMedium},monthDisabled:{pointerEvents:"none",color:e.palette.text.hint}}}),{name:"MuiPickersMonth"}),Qe=function(e){var t=e.selected,n=e.onSelect,r=e.disabled,o=e.value,a=e.children,s=Object(h.a)(e,["selected","onSelect","disabled","value","children"]),l=Ze(),u=Object(i.useCallback)((function(){n(o)}),[n,o]);return Object(i.createElement)(f.a,Object(d.a)({role:"button",component:"div",className:Object(c.a)(l.root,t&&l.monthSelected,r&&l.monthDisabled),tabIndex:r?-1:0,onClick:u,onKeyPress:u,color:t?"primary":void 0,variant:t?"h5":"subtitle1",children:a},s))};Qe.displayName="Month";var Ke=Object(p.a)({container:{width:310,display:"flex",flexWrap:"wrap",alignContent:"stretch"}},{name:"MuiPickersMonthSelection"}),Je=function(e){var t=e.disablePast,n=e.disableFuture,r=e.minDate,o=e.maxDate,a=e.date,s=e.onMonthChange,l=e.onChange,c=u(),d=Ke(),h=c.getMonth(a),f=function(e){var i=c.date(),a=c.date(r),s=c.date(o),l=c.startOfMonth(t&&c.isAfter(i,a)?i:a),u=c.startOfMonth(n&&c.isBefore(i,s)?i:s),d=c.isBefore(e,l),h=c.isAfter(e,u);return d||h},p=Object(i.useCallback)((function(e){var t=c.setMonth(a,e);l(t,!0),s&&s(t)}),[a,l,s,c]);return Object(i.createElement)("div",{className:d.container},c.getMonthArray(a).map((function(e){var t=c.getMonth(e),n=c.format(e,"MMM");return Object(i.createElement)(Qe,{key:n,value:t,selected:t===h,onSelect:p,disabled:f(e)},n)})))},$e=function(){return"undefined"==typeof window?"portrait":window.screen&&window.screen.orientation&&window.screen.orientation.angle?90===Math.abs(window.screen.orientation.angle)?"landscape":"portrait":window.orientation&&90===Math.abs(Number(window.orientation))?"landscape":"portrait"};function et(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),n.push.apply(n,r)}return n}var tt={year:qe,month:Je,date:be,hours:ze,minutes:ze,seconds:ze},nt=Object(p.a)({container:{display:"flex",flexDirection:"column"},containerLandscape:{flexDirection:"row"},pickerView:{overflowX:"hidden",minHeight:305,minWidth:310,maxWidth:325,display:"flex",flexDirection:"column",justifyContent:"center"},pickerViewLandscape:{padding:"0 8px"}},{name:"MuiPickersBasePicker"}),rt=function(e){var t=e.date,n=e.views,r=e.disableToolbar,o=e.onChange,a=e.openTo,s=e.minDate,l=e.maxDate,f=e.ToolbarComponent,p=e.orientation,g=Object(h.a)(e,["date","views","disableToolbar","onChange","openTo","minDate","maxDate","ToolbarComponent","orientation"]),v=u(),m=nt(),y=function(e){var t=Object(i.useState)($e()),n=Object(V.a)(t,2),r=n[0],o=n[1],a=Object(i.useCallback)((function(){return o($e())}),[]);return O((function(){return window.addEventListener("orientationchange",a),function(){return window.removeEventListener("orientationchange",a)}}),[a]),"landscape"===(e||r)}(p),b=function(e,t,n){var r=Object(i.useState)(t&&x(e,t)?t:e[0]),o=Object(V.a)(r,2),a=o[0],s=o[1];return{handleChangeAndOpenNext:Object(i.useCallback)((function(t,r){var i=e[e.indexOf(a)+1];if(r&&i)return n(t,!1),void s(i);n(t,Boolean(r))}),[n,a,e]),openView:a,setOpenView:s}}(n,a,o),C=b.openView,A=b.setOpenView,_=b.handleChangeAndOpenNext,I=Object(i.useMemo)((function(){return v.date(s)}),[s,v]),w=Object(i.useMemo)((function(){return v.date(l)}),[l,v]);return Object(i.createElement)("div",{className:Object(c.a)(m.container,y&&m.containerLandscape)},!r&&Object(i.createElement)(f,Object(d.a)({},g,{views:n,isLandscape:y,date:t,onChange:o,setOpenView:A,openView:C})),Object(i.createElement)("div",{className:Object(c.a)(m.pickerView,y&&m.pickerViewLandscape)},"year"===C&&Object(i.createElement)(qe,Object(d.a)({},g,{date:t,onChange:_,minDate:I,maxDate:w})),"month"===C&&Object(i.createElement)(Je,Object(d.a)({},g,{date:t,onChange:_,minDate:I,maxDate:w})),"date"===C&&Object(i.createElement)(be,Object(d.a)({},g,{date:t,onChange:_,minDate:I,maxDate:w})),("hours"===C||"minutes"===C||"seconds"===C)&&Object(i.createElement)(ze,Object(d.a)({},g,{date:t,type:C,onHourChange:_,onMinutesChange:_,onSecondsChange:_}))))};rt.defaultProps=function(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?et(n,!0).forEach((function(t){Object(r.a)(e,t,n[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):et(n).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))}))}return e}({},Ue,{views:Object.keys(tt)});var it=Object(p.a)((function(e){var t="light"===e.palette.type?e.palette.primary.contrastText:e.palette.getContrastText(e.palette.background.default);return{toolbarTxt:{color:Object(g.d)(t,.54)},toolbarBtnSelected:{color:t}}}),{name:"MuiPickersToolbarText"}),ot=function(e){var t=e.selected,n=e.label,r=e.className,o=void 0===r?null:r,a=Object(h.a)(e,["selected","label","className"]),s=it();return Object(i.createElement)(f.a,Object(d.a)({children:n,className:Object(c.a)(s.toolbarTxt,o,t&&s.toolbarBtnSelected)},a))},at=function(e){var t=e.classes,n=e.className,r=void 0===n?null:n,o=e.label,a=e.selected,s=e.variant,l=e.align,u=e.typographyClassName,f=Object(h.a)(e,["classes","className","label","selected","variant","align","typographyClassName"]);return Object(i.createElement)(y.a,Object(d.a)({variant:"text",className:Object(c.a)(t.toolbarBtn,r)},f),Object(i.createElement)(ot,{align:l,className:u,variant:s,label:o,selected:a}))};at.defaultProps={className:""};var st=Object(v.a)({toolbarBtn:{padding:0,minWidth:"16px",textTransform:"none"}}),lt=Object(m.a)(st,{name:"MuiPickersToolbarButton"})(at),ut=Object(p.a)((function(e){return{toolbar:{display:"flex",flexDirection:"row",alignItems:"center",justifyContent:"center",height:100,backgroundColor:"light"===e.palette.type?e.palette.primary.main:e.palette.background.default},toolbarLandscape:{height:"auto",maxWidth:150,padding:8,justifyContent:"flex-start"}}}),{name:"MuiPickersToolbar"}),ct=function(e){var t=e.children,n=e.isLandscape,r=e.className,o=void 0===r?null:r,a=Object(h.a)(e,["children","isLandscape","className"]),s=ut();return Object(i.createElement)(b.a,Object(d.a)({className:Object(c.a)(s.toolbar,o,n&&s.toolbarLandscape)},a),t)};function dt(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),n.push.apply(n,r)}return n}var ht=function(e){var t=e.inputValue,n=e.inputVariant,o=e.validationError,a=e.InputProps,s=e.openPicker,l=e.TextFieldComponent,u=void 0===l?R.a:l,c=Object(h.a)(e,["inputValue","inputVariant","validationError","InputProps","openPicker","TextFieldComponent"]),f=Object(i.useMemo)((function(){return function(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?dt(n,!0).forEach((function(t){Object(r.a)(e,t,n[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):dt(n).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))}))}return e}({},a,{readOnly:!0})}),[a]);return Object(i.createElement)(u,Object(d.a)({error:Boolean(o),helperText:o},c,{onClick:s,value:t,variant:n,InputProps:f,onKeyDown:function(e){32===e.keyCode&&(e.stopPropagation(),s())}}))};ht.displayName="PureDateInput";var ft=function(e,t,n,r,i){var o=i.invalidLabel,a=i.emptyLabel,s=i.labelFunc,l=n.date(e);return s?s(r?null:l,o):r?a||"":n.isValid(l)?n.format(l,t):o},pt=function(e,t,n){return t?n:e.endOfDay(n)},gt=function(e,t,n){return t?n:e.startOfDay(n)},vt=function(e,t,n){var r=n.maxDate,i=n.minDate,o=n.disablePast,a=n.disableFuture,s=n.maxDateMessage,l=n.minDateMessage,u=n.invalidDateMessage,c=n.strictCompareDates,d=t.date(e);return null===e?"":t.isValid(e)?r&&t.isAfter(d,pt(t,!!c,t.date(r)))||a&&t.isAfter(d,pt(t,!!c,t.date()))?s:i&&t.isBefore(d,gt(t,!!c,t.date(i)))||o&&t.isBefore(d,gt(t,!!c,t.date()))?l:"":u};function mt(e){var t=!(arguments.length>1&&void 0!==arguments[1])||arguments[1],n=arguments.length>2?arguments[2]:void 0;return e||(t?n["12h"]:n["24h"])}function yt(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),n.push.apply(n,r)}return n}function bt(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?yt(n,!0).forEach((function(t){Object(r.a)(e,t,n[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):yt(n).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))}))}return e}var Ct=function(e){var t=e.inputValue,n=e.inputVariant,o=e.validationError,a=e.KeyboardButtonProps,s=e.InputAdornmentProps,l=e.openPicker,u=e.onChange,c=e.InputProps,f=e.mask,p=e.maskChar,g=void 0===p?"_":p,v=e.refuse,m=void 0===v?/[^\d]+/gi:v,y=e.format,b=e.keyboardIcon,C=e.disabled,A=e.rifmFormatter,_=e.TextFieldComponent,I=void 0===_?R.a:_,x=Object(h.a)(e,["inputValue","inputVariant","validationError","KeyboardButtonProps","InputAdornmentProps","openPicker","onChange","InputProps","mask","maskChar","refuse","format","keyboardIcon","disabled","rifmFormatter","TextFieldComponent"]),w=f||function(e,t){return e.replace(/[a-z]/gi,t)}(y,g),M=Object(i.useMemo)((function(){return function(e,t,n){return function(r){var i="",o=r.replace(n,"");if(""===o)return o;for(var a=0,s=0;a<e.length;){var l=e[a];if(l===t&&s<o.length)i+=o[s],s+=1;else i+=l;a+=1}return i}}(w,g,m)}),[w,g,m]),S=s&&s.position?s.position:"end";return Object(i.createElement)(H,{key:w,value:t,onChange:function(e){u(""===e||e===w?null:e)},refuse:m,format:A||M},(function(e){var t=e.onChange,u=e.value;return Object(i.createElement)(I,Object(d.a)({disabled:C,error:Boolean(o),helperText:o},x,{value:u,onChange:t,variant:n,InputProps:bt({},c,Object(r.a)({},"".concat(S,"Adornment"),Object(i.createElement)(F.a,Object(d.a)({position:S},s),Object(i.createElement)(z.a,Object(d.a)({disabled:C},a,{onClick:l}),b))))}))}))};Ct.defaultProps={keyboardIcon:Object(i.createElement)((function(e){return o.a.createElement(U.a,e,o.a.createElement("path",{d:"M17 12h-5v5h5v-5zM16 1v2H8V1H6v2H5c-1.11 0-1.99.9-1.99 2L3 19c0 1.1.89 2 2 2h14c1.1 0 2-.9 2-2V5c0-1.1-.9-2-2-2h-1V1h-2zm3 18H5V8h14v11z"}),o.a.createElement("path",{fill:"none",d:"M0 0h24v24H0z"}))}),null)};function At(e,t){return{date:function(e,t){var n=t.value,r=t.initialFocusedDate,o=Object(i.useRef)(e.date()),a=e.date(n||r||o.current);return a&&e.isValid(a)?a:o.current}(u(),e),format:e.format||t.getDefaultFormat()}}function _t(e,t){var n=e.autoOk,r=e.disabled,o=e.readOnly,a=e.onAccept,s=e.onChange,l=e.onError,c=e.value,d=e.variant,h=u(),f=function(e){var t=e.open,n=e.onOpen,r=e.onClose,o=null;if(null==t){var a=Object(i.useState)(!1),s=Object(V.a)(a,2);t=s[0],o=s[1]}return{isOpen:t,setIsOpen:Object(i.useCallback)((function(e){return o&&o(e),e?n&&n():r&&r()}),[n,r,o])}}(e),p=f.isOpen,g=f.setIsOpen,v=At(e,t),m=v.date,y=v.format,b=Object(i.useState)(m),C=Object(V.a)(b,2),A=C[0],_=C[1];Object(i.useEffect)((function(){p||h.isEqual(A,m)||_(m)}),[m,p,A,h]);var I=Object(i.useCallback)((function(e){s(e),a&&a(e),g(!1)}),[a,s,g]),x=Object(i.useMemo)((function(){return{format:y,open:p,onClear:function(){return I(null)},onAccept:function(){return I(A)},onSetToday:function(){return _(h.date())},onDismiss:function(){g(!1)}}}),[I,y,p,A,g,h]),w=Object(i.useMemo)((function(){return{date:A,onChange:function(e){var t=!(arguments.length>1&&void 0!==arguments[1])||arguments[1];_(e),t&&n?I(e):"inline"!==d&&"static"!==d||(s(e),a&&a(e))}}}),[I,n,a,s,A,d]),M=vt(c,h,e);Object(i.useEffect)((function(){l&&l(M,c)}),[l,M,c]);var S=ft(m,y,h,null===c,e),T={pickerProps:w,inputProps:Object(i.useMemo)((function(){return{inputValue:S,validationError:M,openPicker:function(){return!o&&!r&&g(!0)}}}),[r,S,o,g,M]),wrapperProps:x};return Object(i.useDebugValue)(T),T}function It(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),n.push.apply(n,r)}return n}function xt(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?It(n,!0).forEach((function(t){Object(r.a)(e,t,n[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):It(n).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))}))}return e}function wt(e,t){var n=e.format,r=void 0===n?t.getDefaultFormat():n,o=e.inputValue,a=e.onChange,s=e.value,l=u(),c=ft(s,r,l,null===s,e),d=Object(i.useState)(c),h=Object(V.a)(d,2),f=h[0],p=h[1],g=o?function(e,t,n){try{return t.parse(e,n)}catch(e){return null}}(o,l,r):s;Object(i.useEffect)((function(){(null===s||l.isValid(s))&&p(c)}),[c,p,l,s]);var v=_t(xt({},e,{value:g,onChange:Object(i.useCallback)((function(e){a(e,null===e?null:l.format(e,r))}),[r,a,l])}),t),m=v.inputProps,y=v.wrapperProps,b=v.pickerProps;return{inputProps:Object(i.useMemo)((function(){return xt({},m,{format:y.format,inputValue:o||f,onChange:function(e){p(e||"");var t=null===e?null:l.parse(e,y.format);a(t,e)}})}),[m,f,o,a,l,y.format]),wrapperProps:y,pickerProps:b}}function Mt(e){var t=e.Input,n=e.useState,r=e.useOptions,o=e.getCustomProps,a=e.DefaultToolbarComponent;return function(e){var s=e.allowKeyboardControl,l=e.ampm,u=e.animateYearScrolling,c=(e.autoOk,e.dateRangeIcon),f=e.disableFuture,p=e.disablePast,g=e.disableToolbar,v=(e.emptyLabel,e.format,e.forwardedRef,e.hideTabs),m=(e.initialFocusedDate,e.invalidDateMessage,e.invalidLabel,e.labelFunc,e.leftArrowButtonProps),y=e.leftArrowIcon,b=e.loadingIndicator,C=e.maxDate,A=(e.maxDateMessage,e.minDate),_=(e.minDateMessage,e.minutesStep),I=(e.onAccept,e.onChange,e.onClose,e.onMonthChange),x=(e.onOpen,e.onYearChange),w=e.openTo,M=e.orientation,S=e.renderDay,T=e.rightArrowButtonProps,E=e.rightArrowIcon,O=e.shouldDisableDate,k=e.strictCompareDates,L=e.timeIcon,D=e.ToolbarComponent,P=void 0===D?a:D,N=(e.value,e.variant),R=e.views,z=Object(h.a)(e,["allowKeyboardControl","ampm","animateYearScrolling","autoOk","dateRangeIcon","disableFuture","disablePast","disableToolbar","emptyLabel","format","forwardedRef","hideTabs","initialFocusedDate","invalidDateMessage","invalidLabel","labelFunc","leftArrowButtonProps","leftArrowIcon","loadingIndicator","maxDate","maxDateMessage","minDate","minDateMessage","minutesStep","onAccept","onChange","onClose","onMonthChange","onOpen","onYearChange","openTo","orientation","renderDay","rightArrowButtonProps","rightArrowIcon","shouldDisableDate","strictCompareDates","timeIcon","ToolbarComponent","value","variant","views"]),F=o?o(e):{},B=r(e),H=n(e,B),U=H.pickerProps,V=H.inputProps,Y=H.wrapperProps;return Object(i.createElement)(j,Object(d.a)({variant:N,InputComponent:t,DateInputProps:V},F,Y,z),Object(i.createElement)(rt,Object(d.a)({},U,{allowKeyboardControl:s,ampm:l,animateYearScrolling:u,dateRangeIcon:c,disableFuture:f,disablePast:p,disableToolbar:g,hideTabs:v,leftArrowButtonProps:m,leftArrowIcon:y,loadingIndicator:b,maxDate:C,minDate:A,minutesStep:_,onMonthChange:I,onYearChange:x,openTo:w,orientation:M,renderDay:S,rightArrowButtonProps:T,rightArrowIcon:E,shouldDisableDate:O,strictCompareDates:k,timeIcon:L,ToolbarComponent:P,views:R})))}}var St=Object(p.a)({toolbar:{flexDirection:"column",alignItems:"flex-start"},toolbarLandscape:{padding:16},dateLandscape:{marginRight:16}},{name:"MuiPickersDatePickerRoot"}),Tt=function(e){var t=e.date,n=e.views,r=e.setOpenView,o=e.isLandscape,a=e.openView,s=u(),l=St(),d=Object(i.useMemo)((function(){return se(n)}),[n]),h=Object(i.useMemo)((function(){return le(n)}),[n]);return Object(i.createElement)(ct,{isLandscape:o,className:Object(c.a)(!d&&l.toolbar,o&&l.toolbarLandscape)},Object(i.createElement)(lt,{variant:d?"h3":"subtitle1",onClick:function(){return r("year")},selected:"year"===a,label:s.getYearText(t)}),!d&&!h&&Object(i.createElement)(lt,{variant:"h4",selected:"date"===a,onClick:function(){return r("date")},align:o?"left":"center",label:s.getDatePickerHeaderText(t),className:Object(c.a)(o&&l.dateLandscape)}),h&&Object(i.createElement)(lt,{variant:"h4",onClick:function(){return r("month")},selected:"month"===a,label:s.getMonthText(t)}))};function Et(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),n.push.apply(n,r)}return n}var Ot=function(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?Et(n,!0).forEach((function(t){Object(r.a)(e,t,n[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):Et(n).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))}))}return e}({},Ue,{openTo:"date",views:["year","date"]});function kt(e){var t=u();return{getDefaultFormat:function(){return function(e,t){return se(e)?t.yearFormat:le(e)?t.yearMonthFormat:t.dateFormat}(e.views,t)}}}var Lt=Mt({useOptions:kt,Input:ht,useState:_t,DefaultToolbarComponent:Tt}),Dt=Mt({useOptions:kt,Input:Ct,useState:wt,DefaultToolbarComponent:Tt});Lt.defaultProps=Ot,Dt.defaultProps=Ot;var Pt=Object(p.a)({toolbarLandscape:{flexWrap:"wrap"},toolbarAmpmLeftPadding:{paddingLeft:50},separator:{margin:"0 4px 0 2px",cursor:"default"},hourMinuteLabel:{display:"flex",justifyContent:"flex-end",alignItems:"flex-end"},hourMinuteLabelAmpmLandscape:{marginTop:"auto"},hourMinuteLabelReverse:{flexDirection:"row-reverse"},ampmSelection:{marginLeft:20,marginRight:-20,display:"flex",flexDirection:"column"},ampmLandscape:{margin:"4px 0 auto",flexDirection:"row",justifyContent:"space-around",flexBasis:"100%"},ampmSelectionWithSeconds:{marginLeft:15,marginRight:10},ampmLabel:{fontSize:18}},{name:"MuiPickersTimePickerToolbar"});function Nt(e,t,n){var r=u();return{meridiemMode:Ee(e,r),handleMeridiemChange:Object(i.useCallback)((function(i){var o=Oe(e,i,Boolean(t),r);n(o,!1)}),[t,e,n,r])}}var jt=function(e){var t=e.date,n=e.views,r=e.ampm,o=e.openView,a=e.onChange,s=e.isLandscape,l=e.setOpenView,d=u(),h=Object(Y.a)(),f=Pt(),p=Nt(t,r,a),g=p.meridiemMode,v=p.handleMeridiemChange,m=s?"h3":"h2";return Object(i.createElement)(ct,{isLandscape:s,className:Object(c.a)(s?f.toolbarLandscape:r&&f.toolbarAmpmLeftPadding)},Object(i.createElement)("div",{className:Object(c.a)(f.hourMinuteLabel,r&&s&&f.hourMinuteLabelAmpmLandscape,{rtl:f.hourMinuteLabelReverse}[h.direction])},x(n,"hours")&&Object(i.createElement)(lt,{variant:m,onClick:function(){return l(Ce.HOURS)},selected:o===Ce.HOURS,label:d.getHourText(t,Boolean(r))}),x(n,["hours","minutes"])&&Object(i.createElement)(ot,{label:":",variant:m,selected:!1,className:f.separator}),x(n,"minutes")&&Object(i.createElement)(lt,{variant:m,onClick:function(){return l(Ce.MINUTES)},selected:o===Ce.MINUTES,label:d.getMinuteText(t)}),x(n,["minutes","seconds"])&&Object(i.createElement)(ot,{variant:"h2",label:":",selected:!1,className:f.separator}),x(n,"seconds")&&Object(i.createElement)(lt,{variant:"h2",onClick:function(){return l(Ce.SECONDS)},selected:o===Ce.SECONDS,label:d.getSecondText(t)})),r&&Object(i.createElement)("div",{className:Object(c.a)(f.ampmSelection,s&&f.ampmLandscape,x(n,"seconds")&&f.ampmSelectionWithSeconds)},Object(i.createElement)(lt,{disableRipple:!0,variant:"subtitle1",selected:"am"===g,typographyClassName:f.ampmLabel,label:d.getMeridiemText("am"),onClick:function(){return v("am")}}),Object(i.createElement)(lt,{disableRipple:!0,variant:"subtitle1",selected:"pm"===g,typographyClassName:f.ampmLabel,label:d.getMeridiemText("pm"),onClick:function(){return v("pm")}})))};function Rt(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),n.push.apply(n,r)}return n}var zt=function(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?Rt(n,!0).forEach((function(t){Object(r.a)(e,t,n[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):Rt(n).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))}))}return e}({},He,{openTo:"hours",views:["hours","minutes"]});function Ft(e){var t=u();return{getDefaultFormat:function(){return mt(e.format,e.ampm,{"12h":t.time12hFormat,"24h":t.time24hFormat})}}}var Bt=Mt({useOptions:Ft,Input:ht,useState:_t,DefaultToolbarComponent:jt}),Ht=Mt({useOptions:Ft,Input:Ct,useState:wt,DefaultToolbarComponent:jt,getCustomProps:function(e){return{refuse:e.ampm?/[^\dap]+/gi:/[^\d]+/gi}}});Bt.defaultProps=zt,Ht.defaultProps=zt;var Ut=n(916),Vt=n(2378),Yt=n(2381),Wt=n(336),Gt=function(e){return"date"===e||"year"===e?"date":"time"},Xt=Object(p.a)((function(e){var t="light"===e.palette.type?e.palette.primary.main:e.palette.background.default;return{tabs:{color:e.palette.getContrastText(t),backgroundColor:t}}}),{name:"MuiPickerDTTabs"}),qt=function(e){var t=e.view,n=e.onChange,r=e.dateRangeIcon,o=e.timeIcon,a=Xt(),s="light"===Object(Y.a)().palette.type?"secondary":"primary";return Object(i.createElement)(Wt.a,null,Object(i.createElement)(Yt.a,{variant:"fullWidth",value:Gt(t),onChange:function(e,r){r!==Gt(t)&&n("date"===r?"date":"hours")},className:a.tabs,indicatorColor:s},Object(i.createElement)(Vt.a,{value:"date",icon:Object(i.createElement)(i.Fragment,null,r)}),Object(i.createElement)(Vt.a,{value:"time",icon:Object(i.createElement)(i.Fragment,null,o)})))};qt.defaultProps={dateRangeIcon:Object(i.createElement)((function(e){return o.a.createElement(U.a,e,o.a.createElement("path",{d:"M9 11H7v2h2v-2zm4 0h-2v2h2v-2zm4 0h-2v2h2v-2zm2-7h-1V2h-2v2H8V2H6v2H5c-1.11 0-1.99.9-1.99 2L3 20c0 1.1.89 2 2 2h14c1.1 0 2-.9 2-2V6c0-1.1-.9-2-2-2zm0 16H5V9h14v11z"}),o.a.createElement("path",{fill:"none",d:"M0 0h24v24H0z"}))}),null),timeIcon:Object(i.createElement)((function(e){return o.a.createElement(U.a,e,o.a.createElement("path",{d:"M11.99 2C6.47 2 2 6.48 2 12s4.47 10 9.99 10C17.52 22 22 17.52 22 12S17.52 2 11.99 2zM12 20c-4.42 0-8-3.58-8-8s3.58-8 8-8 8 3.58 8 8-3.58 8-8 8z"}),o.a.createElement("path",{fill:"none",d:"M0 0h24v24H0z"}),o.a.createElement("path",{d:"M12.5 7H11v6l5.25 3.15.75-1.23-4.5-2.67z"}))}),null)};var Zt=Object(p.a)((function(e){return{toolbar:{paddingLeft:16,paddingRight:16,justifyContent:"space-around"},separator:{margin:"0 4px 0 2px",cursor:"default"}}}),{name:"MuiPickerDTToolbar"}),Qt=function(e){var t=e.date,n=e.openView,r=e.setOpenView,o=e.ampm,a=e.hideTabs,s=e.dateRangeIcon,l=e.timeIcon,c=e.onChange,d=u(),h=Zt(),f=!a&&"undefined"!=typeof window&&window.innerHeight>667,p=Nt(t,o,c),g=p.meridiemMode,v=p.handleMeridiemChange,m="rtl"===Object(Y.a)().direction;return Object(i.createElement)(i.Fragment,null,Object(i.createElement)(ct,{isLandscape:!1,className:h.toolbar},Object(i.createElement)(Ut.a,{container:!0,justify:"center",wrap:"nowrap"},Object(i.createElement)(Ut.a,{item:!0,container:!0,xs:5,justify:"flex-start",direction:"column"},Object(i.createElement)("div",null,Object(i.createElement)(lt,{variant:"subtitle1",onClick:function(){return r("year")},selected:"year"===n,label:d.getYearText(t)})),Object(i.createElement)("div",null,Object(i.createElement)(lt,{variant:"h4",onClick:function(){return r("date")},selected:"date"===n,label:d.getDateTimePickerHeaderText(t)}))),Object(i.createElement)(Ut.a,{item:!0,container:!0,xs:6,justify:"center",alignItems:"flex-end",direction:m?"row-reverse":"row"},Object(i.createElement)(lt,{variant:"h3",onClick:function(){return r("hours")},selected:"hours"===n,label:d.getHourText(t,o)}),Object(i.createElement)(ot,{variant:"h3",label:":",className:h.separator}),Object(i.createElement)(lt,{variant:"h3",onClick:function(){return r("minutes")},selected:"minutes"===n,label:d.getMinuteText(t)})),o&&Object(i.createElement)(Ut.a,{item:!0,container:!0,xs:1,direction:"column",justify:"flex-end"},Object(i.createElement)(lt,{variant:"subtitle1",selected:"am"===g,label:d.getMeridiemText("am"),onClick:function(){return v("am")}}),Object(i.createElement)(lt,{variant:"subtitle1",selected:"pm"===g,label:d.getMeridiemText("pm"),onClick:function(){return v("pm")}})))),f&&Object(i.createElement)(qt,{dateRangeIcon:s,timeIcon:l,view:n,onChange:r}))};function Kt(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),n.push.apply(n,r)}return n}var Jt=function(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?Kt(n,!0).forEach((function(t){Object(r.a)(e,t,n[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):Kt(n).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))}))}return e}({},Ve,{wider:!0,orientation:"portrait",openTo:"date",views:["year","date","hours","minutes"]});function $t(e){var t=u();if("portrait"!==e.orientation)throw new Error("We are not supporting custom orientation for DateTimePicker yet :(");return{getDefaultFormat:function(){return mt(e.format,e.ampm,{"12h":t.dateTime12hFormat,"24h":t.dateTime24hFormat})}}}var en=Mt({useOptions:$t,Input:ht,useState:_t,DefaultToolbarComponent:Qt}),tn=Mt({useOptions:$t,Input:Ct,useState:wt,DefaultToolbarComponent:Qt,getCustomProps:function(e){return{refuse:e.ampm?/[^\dap]+/gi:/[^\d]+/gi}}});function nn(e){var t=e.value,n=e.autoOk,r=void 0===n||n,i=e.onChange,o=e.defaultFormat,a=_t({value:t,onChange:i,autoOk:r},{getDefaultFormat:function(){return o||"MM/dd/yyyy"}});return{pickerProps:a.pickerProps,wrapperProps:a.wrapperProps,inputProps:a.inputProps}}en.defaultProps=Jt,tn.defaultProps=Jt,n.d(t,"useStaticState",(function(){return nn})),n.d(t,"MuiPickersContext",(function(){return s})),n.d(t,"MuiPickersUtilsProvider",(function(){return l})),n.d(t,"useUtils",(function(){return u})),n.d(t,"makePickerWithState",(function(){return Mt})),n.d(t,"useKeyboardPickerState",(function(){return wt})),n.d(t,"usePickerState",(function(){return _t})),n.d(t,"validate",(function(){return vt})),n.d(t,"Calendar",(function(){return be})),n.d(t,"DatePicker",(function(){return Lt})),n.d(t,"KeyboardDatePicker",(function(){return Dt})),n.d(t,"Picker",(function(){return rt})),n.d(t,"Day",(function(){return J})),n.d(t,"Clock",(function(){return Le})),n.d(t,"ClockView",(function(){return ze})),n.d(t,"TimePickerView",(function(){return Fe})),n.d(t,"KeyboardTimePicker",(function(){return Ht})),n.d(t,"TimePicker",(function(){return Bt})),n.d(t,"DateTimePicker",(function(){return en})),n.d(t,"KeyboardDateTimePicker",(function(){return tn}))},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(4)),s=n(274),l=n(57),u=Object(l.a)(o.createElement("path",{d:"M19 5v14H5V5h14m0-2H5c-1.1 0-2 .9-2 2v14c0 1.1.9 2 2 2h14c1.1 0 2-.9 2-2V5c0-1.1-.9-2-2-2z"}),"CheckBoxOutlineBlank"),c=Object(l.a)(o.createElement("path",{d:"M19 3H5c-1.11 0-2 .9-2 2v14c0 1.1.89 2 2 2h14c1.11 0 2-.9 2-2V5c0-1.1-.89-2-2-2zm-9 14l-5-5 1.41-1.41L10 14.17l7.59-7.59L19 8l-9 9z"}),"CheckBox"),d=n(26),h=Object(l.a)(o.createElement("path",{d:"M19 3H5c-1.1 0-2 .9-2 2v14c0 1.1.9 2 2 2h14c1.1 0 2-.9 2-2V5c0-1.1-.9-2-2-2zm-2 10H7v-2h10v2z"}),"IndeterminateCheckBox"),f=n(15),p=n(7),g=o.createElement(c,null),v=o.createElement(u,null),m=o.createElement(h,null),y=o.forwardRef((function(e,t){var n=e.checkedIcon,l=void 0===n?g:n,u=e.classes,c=e.color,d=void 0===c?"secondary":c,h=e.icon,p=void 0===h?v:h,y=e.indeterminate,b=void 0!==y&&y,C=e.indeterminateIcon,A=void 0===C?m:C,_=e.inputProps,I=e.size,x=void 0===I?"medium":I,w=Object(i.a)(e,["checkedIcon","classes","color","icon","indeterminate","indeterminateIcon","inputProps","size"]),M=b?A:p,S=b?A:l;return o.createElement(s.a,Object(r.a)({type:"checkbox",classes:{root:Object(a.a)(u.root,u["color".concat(Object(f.a)(d))],b&&u.indeterminate),checked:u.checked,disabled:u.disabled},color:d,inputProps:Object(r.a)({"data-indeterminate":b},_),icon:o.cloneElement(M,{fontSize:void 0===M.props.fontSize&&"small"===x?x:M.props.fontSize}),checkedIcon:o.cloneElement(S,{fontSize:void 0===S.props.fontSize&&"small"===x?x:S.props.fontSize}),ref:t},w))}));t.a=Object(p.a)((function(e){return{root:{color:e.palette.text.secondary},checked:{},disabled:{},indeterminate:{},colorPrimary:{"&$checked":{color:e.palette.primary.main,"&:hover":{backgroundColor:Object(d.d)(e.palette.primary.main,e.palette.action.hoverOpacity),"@media (hover: none)":{backgroundColor:"transparent"}}},"&$disabled":{color:e.palette.action.disabled}},colorSecondary:{"&$checked":{color:e.palette.secondary.main,"&:hover":{backgroundColor:Object(d.d)(e.palette.secondary.main,e.palette.action.hoverOpacity),"@media (hover: none)":{backgroundColor:"transparent"}}},"&$disabled":{color:e.palette.action.disabled}}}}),{name:"MuiCheckbox"})(y)},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=n(3),s=n.n(a),l=n(369),u=n(52);function c(e){var t=e.children,n=e.only,r=e.width,i=Object(u.a)(),o=!0;if(n)if(Array.isArray(n))for(var a=0;a<n.length;a+=1){if(r===n[a]){o=!1;break}}else n&&r===n&&(o=!1);if(o)for(var s=0;s<i.breakpoints.keys.length;s+=1){var c=i.breakpoints.keys[s],d=e["".concat(c,"Up")],h=e["".concat(c,"Down")];if(d&&Object(l.c)(c,r)||h&&Object(l.b)(c,r)){o=!1;break}}return o?t:null}c.propTypes={children:s.a.node,className:s.a.string,implementation:s.a.oneOf(["js","css"]),initialWidth:s.a.oneOf(["xs","sm","md","lg","xl"]),lgDown:s.a.bool,lgUp:s.a.bool,mdDown:s.a.bool,mdUp:s.a.bool,only:s.a.oneOfType([s.a.oneOf(["xs","sm","md","lg","xl"]),s.a.arrayOf(s.a.oneOf(["xs","sm","md","lg","xl"]))]),smDown:s.a.bool,smUp:s.a.bool,width:s.a.string.isRequired,xlDown:s.a.bool,xlUp:s.a.bool,xsDown:s.a.bool,xsUp:s.a.bool};var d=Object(l.a)()(c),h=n(21),f=n(15),p=n(7);var g=Object(p.a)((function(e){var t={display:"none"};return e.breakpoints.keys.reduce((function(n,r){return n["only".concat(Object(f.a)(r))]=Object(h.a)({},e.breakpoints.only(r),t),n["".concat(r,"Up")]=Object(h.a)({},e.breakpoints.up(r),t),n["".concat(r,"Down")]=Object(h.a)({},e.breakpoints.down(r),t),n}),{})}),{name:"PrivateHiddenCss"})((function(e){var t=e.children,n=e.classes,r=e.className,a=e.only,s=(Object(i.a)(e,["children","classes","className","only"]),Object(u.a)()),l=[];r&&l.push(r);for(var c=0;c<s.breakpoints.keys.length;c+=1){var d=s.breakpoints.keys[c],h=e["".concat(d,"Up")],p=e["".concat(d,"Down")];h&&l.push(n["".concat(d,"Up")]),p&&l.push(n["".concat(d,"Down")])}return a&&(Array.isArray(a)?a:[a]).forEach((function(e){l.push(n["only".concat(Object(f.a)(e))])})),o.createElement("div",{className:l.join(" ")},t)}));t.a=function(e){var t=e.implementation,n=void 0===t?"js":t,a=e.lgDown,s=void 0!==a&&a,l=e.lgUp,u=void 0!==l&&l,c=e.mdDown,h=void 0!==c&&c,f=e.mdUp,p=void 0!==f&&f,v=e.smDown,m=void 0!==v&&v,y=e.smUp,b=void 0!==y&&y,C=e.xlDown,A=void 0!==C&&C,_=e.xlUp,I=void 0!==_&&_,x=e.xsDown,w=void 0!==x&&x,M=e.xsUp,S=void 0!==M&&M,T=Object(i.a)(e,["implementation","lgDown","lgUp","mdDown","mdUp","smDown","smUp","xlDown","xlUp","xsDown","xsUp"]);return"js"===n?o.createElement(d,Object(r.a)({lgDown:s,lgUp:u,mdDown:h,mdUp:p,smDown:m,smUp:b,xlDown:A,xlUp:I,xsDown:w,xsUp:S},T)):o.createElement(g,Object(r.a)({lgDown:s,lgUp:u,mdDown:h,mdUp:p,smDown:m,smUp:b,xlDown:A,xlUp:I,xsDown:w,xsUp:S},T))}},function(e,t,n){"use strict";n.r(t);var r={};n.r(r),n.d(r,"ADD",(function(){return o})),n.d(r,"ADD_COLUMN_LEFT",(function(){return a})),n.d(r,"ADD_COLUMN_RIGHT",(function(){return s})),n.d(r,"ADD_ROW_BOTTOM",(function(){return l})),n.d(r,"ADD_ROW_TOP",(function(){return u})),n.d(r,"ADD_TO_ARTIFACT",(function(){return c})),n.d(r,"ADD_TO_FOLDER",(function(){return d})),n.d(r,"AIRPLANE",(function(){return h})),n.d(r,"ALIGN_CENTER",(function(){return f})),n.d(r,"ALIGN_JUSTIFY",(function(){return p})),n.d(r,"ALIGN_LEFT",(function(){return g})),n.d(r,"ALIGN_RIGHT",(function(){return v})),n.d(r,"ALIGNMENT_BOTTOM",(function(){return m})),n.d(r,"ALIGNMENT_HORIZONTAL_CENTER",(function(){return y})),n.d(r,"ALIGNMENT_LEFT",(function(){return b})),n.d(r,"ALIGNMENT_RIGHT",(function(){return C})),n.d(r,"ALIGNMENT_TOP",(function(){return A})),n.d(r,"ALIGNMENT_VERTICAL_CENTER",(function(){return _})),n.d(r,"ANNOTATION",(function(){return I})),n.d(r,"APPLICATION",(function(){return x})),n.d(r,"APPLICATIONS",(function(){return w})),n.d(r,"ARCHIVE",(function(){return M})),n.d(r,"ARROW_BOTTOM_LEFT",(function(){return S})),n.d(r,"ARROW_BOTTOM_RIGHT",(function(){return T})),n.d(r,"ARROW_DOWN",(function(){return E})),n.d(r,"ARROW_LEFT",(function(){return O})),n.d(r,"ARROW_RIGHT",(function(){return k})),n.d(r,"ARROW_TOP_LEFT",(function(){return L})),n.d(r,"ARROW_TOP_RIGHT",(function(){return D})),n.d(r,"ARROW_UP",(function(){return P})),n.d(r,"ARROWS_HORIZONTAL",(function(){return N})),n.d(r,"ARROWS_VERTICAL",(function(){return j})),n.d(r,"ASTERISK",(function(){return R})),n.d(r,"AUTOMATIC_UPDATES",(function(){return z})),n.d(r,"BADGE",(function(){return F})),n.d(r,"BAN_CIRCLE",(function(){return B})),n.d(r,"BANK_ACCOUNT",(function(){return H})),n.d(r,"BARCODE",(function(){return U})),n.d(r,"BLANK",(function(){return V})),n.d(r,"BLOCKED_PERSON",(function(){return Y})),n.d(r,"BOLD",(function(){return W})),n.d(r,"BOOK",(function(){return G})),n.d(r,"BOOKMARK",(function(){return X})),n.d(r,"BOX",(function(){return q})),n.d(r,"BRIEFCASE",(function(){return Z})),n.d(r,"BRING_DATA",(function(){return Q})),n.d(r,"BUILD",(function(){return K})),n.d(r,"CALCULATOR",(function(){return J})),n.d(r,"CALENDAR",(function(){return $})),n.d(r,"CAMERA",(function(){return ee})),n.d(r,"CARET_DOWN",(function(){return te})),n.d(r,"CARET_LEFT",(function(){return ne})),n.d(r,"CARET_RIGHT",(function(){return re})),n.d(r,"CARET_UP",(function(){return ie})),n.d(r,"CELL_TOWER",(function(){return oe})),n.d(r,"CHANGES",(function(){return ae})),n.d(r,"CHART",(function(){return se})),n.d(r,"CHAT",(function(){return le})),n.d(r,"CHEVRON_BACKWARD",(function(){return ue})),n.d(r,"CHEVRON_DOWN",(function(){return ce})),n.d(r,"CHEVRON_FORWARD",(function(){return de})),n.d(r,"CHEVRON_LEFT",(function(){return he})),n.d(r,"CHEVRON_RIGHT",(function(){return fe})),n.d(r,"CHEVRON_UP",(function(){return pe})),n.d(r,"CIRCLE",(function(){return ge})),n.d(r,"CIRCLE_ARROW_DOWN",(function(){return ve})),n.d(r,"CIRCLE_ARROW_LEFT",(function(){return me})),n.d(r,"CIRCLE_ARROW_RIGHT",(function(){return ye})),n.d(r,"CIRCLE_ARROW_UP",(function(){return be})),n.d(r,"CITATION",(function(){return Ce})),n.d(r,"CLEAN",(function(){return Ae})),n.d(r,"CLIPBOARD",(function(){return _e})),n.d(r,"CLOUD",(function(){return Ie})),n.d(r,"CLOUD_DOWNLOAD",(function(){return xe})),n.d(r,"CLOUD_UPLOAD",(function(){return we})),n.d(r,"CODE",(function(){return Me})),n.d(r,"CODE_BLOCK",(function(){return Se})),n.d(r,"COG",(function(){return Te})),n.d(r,"COLLAPSE_ALL",(function(){return Ee})),n.d(r,"COLUMN_LAYOUT",(function(){return Oe})),n.d(r,"COMMENT",(function(){return ke})),n.d(r,"COMPARISON",(function(){return Le})),n.d(r,"COMPASS",(function(){return De})),n.d(r,"COMPRESSED",(function(){return Pe})),n.d(r,"CONFIRM",(function(){return Ne})),n.d(r,"CONSOLE",(function(){return je})),n.d(r,"CONTRAST",(function(){return Re})),n.d(r,"CONTROL",(function(){return ze})),n.d(r,"CREDIT_CARD",(function(){return Fe})),n.d(r,"CROSS",(function(){return Be})),n.d(r,"CROWN",(function(){return He})),n.d(r,"CUBE",(function(){return Ue})),n.d(r,"CUBE_ADD",(function(){return Ve})),n.d(r,"CUBE_REMOVE",(function(){return Ye})),n.d(r,"CURVED_RANGE_CHART",(function(){return We})),n.d(r,"CUT",(function(){return Ge})),n.d(r,"DASHBOARD",(function(){return Xe})),n.d(r,"DATA_LINEAGE",(function(){return qe})),n.d(r,"DATABASE",(function(){return Ze})),n.d(r,"DELETE",(function(){return Qe})),n.d(r,"DELTA",(function(){return Ke})),n.d(r,"DERIVE_COLUMN",(function(){return Je})),n.d(r,"DESKTOP",(function(){return $e})),n.d(r,"DIAGNOSIS",(function(){return et})),n.d(r,"DIAGRAM_TREE",(function(){return tt})),n.d(r,"DIRECTION_LEFT",(function(){return nt})),n.d(r,"DIRECTION_RIGHT",(function(){return rt})),n.d(r,"DISABLE",(function(){return it})),n.d(r,"DOCUMENT",(function(){return ot})),n.d(r,"DOCUMENT_OPEN",(function(){return at})),n.d(r,"DOCUMENT_SHARE",(function(){return st})),n.d(r,"DOLLAR",(function(){return lt})),n.d(r,"DOT",(function(){return ut})),n.d(r,"DOUBLE_CARET_HORIZONTAL",(function(){return ct})),n.d(r,"DOUBLE_CARET_VERTICAL",(function(){return dt})),n.d(r,"DOUBLE_CHEVRON_DOWN",(function(){return ht})),n.d(r,"DOUBLE_CHEVRON_LEFT",(function(){return ft})),n.d(r,"DOUBLE_CHEVRON_RIGHT",(function(){return pt})),n.d(r,"DOUBLE_CHEVRON_UP",(function(){return gt})),n.d(r,"DOUGHNUT_CHART",(function(){return vt})),n.d(r,"DOWNLOAD",(function(){return mt})),n.d(r,"DRAG_HANDLE_HORIZONTAL",(function(){return yt})),n.d(r,"DRAG_HANDLE_VERTICAL",(function(){return bt})),n.d(r,"DRAW",(function(){return Ct})),n.d(r,"DRIVE_TIME",(function(){return At})),n.d(r,"DUPLICATE",(function(){return _t})),n.d(r,"EDIT",(function(){return It})),n.d(r,"EJECT",(function(){return xt})),n.d(r,"ENDORSED",(function(){return wt})),n.d(r,"ENVELOPE",(function(){return Mt})),n.d(r,"EQUALS",(function(){return St})),n.d(r,"ERASER",(function(){return Tt})),n.d(r,"ERROR",(function(){return Et})),n.d(r,"EURO",(function(){return Ot})),n.d(r,"EXCHANGE",(function(){return kt})),n.d(r,"EXCLUDE_ROW",(function(){return Lt})),n.d(r,"EXPAND_ALL",(function(){return Dt})),n.d(r,"EXPORT",(function(){return Pt})),n.d(r,"EYE_OFF",(function(){return Nt})),n.d(r,"EYE_ON",(function(){return jt})),n.d(r,"EYE_OPEN",(function(){return Rt})),n.d(r,"FAST_BACKWARD",(function(){return zt})),n.d(r,"FAST_FORWARD",(function(){return Ft})),n.d(r,"FEED",(function(){return Bt})),n.d(r,"FEED_SUBSCRIBED",(function(){return Ht})),n.d(r,"FILM",(function(){return Ut})),n.d(r,"FILTER",(function(){return Vt})),n.d(r,"FILTER_KEEP",(function(){return Yt})),n.d(r,"FILTER_LIST",(function(){return Wt})),n.d(r,"FILTER_OPEN",(function(){return Gt})),n.d(r,"FILTER_REMOVE",(function(){return Xt})),n.d(r,"FLAG",(function(){return qt})),n.d(r,"FLAME",(function(){return Zt})),n.d(r,"FLASH",(function(){return Qt})),n.d(r,"FLOPPY_DISK",(function(){return Kt})),n.d(r,"FLOW_BRANCH",(function(){return Jt})),n.d(r,"FLOW_END",(function(){return $t})),n.d(r,"FLOW_LINEAR",(function(){return en})),n.d(r,"FLOW_REVIEW",(function(){return tn})),n.d(r,"FLOW_REVIEW_BRANCH",(function(){return nn})),n.d(r,"FLOWS",(function(){return rn})),n.d(r,"FOLDER_CLOSE",(function(){return on})),n.d(r,"FOLDER_NEW",(function(){return an})),n.d(r,"FOLDER_OPEN",(function(){return sn})),n.d(r,"FOLDER_SHARED",(function(){return ln})),n.d(r,"FOLDER_SHARED_OPEN",(function(){return un})),n.d(r,"FOLLOWER",(function(){return cn})),n.d(r,"FOLLOWING",(function(){return dn})),n.d(r,"FONT",(function(){return hn})),n.d(r,"FORK",(function(){return fn})),n.d(r,"FORM",(function(){return pn})),n.d(r,"FULL_CIRCLE",(function(){return gn})),n.d(r,"FULL_STACKED_CHART",(function(){return vn})),n.d(r,"FULLSCREEN",(function(){return mn})),n.d(r,"FUNCTION",(function(){return yn})),n.d(r,"GANTT_CHART",(function(){return bn})),n.d(r,"GEOLOCATION",(function(){return Cn})),n.d(r,"GEOSEARCH",(function(){return An})),n.d(r,"GIT_BRANCH",(function(){return _n})),n.d(r,"GIT_COMMIT",(function(){return In})),n.d(r,"GIT_MERGE",(function(){return xn})),n.d(r,"GIT_NEW_BRANCH",(function(){return wn})),n.d(r,"GIT_PULL",(function(){return Mn})),n.d(r,"GIT_PUSH",(function(){return Sn})),n.d(r,"GIT_REPO",(function(){return Tn})),n.d(r,"GLASS",(function(){return En})),n.d(r,"GLOBE",(function(){return On})),n.d(r,"GLOBE_NETWORK",(function(){return kn})),n.d(r,"GRAPH",(function(){return Ln})),n.d(r,"GRAPH_REMOVE",(function(){return Dn})),n.d(r,"GREATER_THAN",(function(){return Pn})),n.d(r,"GREATER_THAN_OR_EQUAL_TO",(function(){return Nn})),n.d(r,"GRID",(function(){return jn})),n.d(r,"GRID_VIEW",(function(){return Rn})),n.d(r,"GROUP_OBJECTS",(function(){return zn})),n.d(r,"GROUPED_BAR_CHART",(function(){return Fn})),n.d(r,"HAND",(function(){return Bn})),n.d(r,"HAND_DOWN",(function(){return Hn})),n.d(r,"HAND_LEFT",(function(){return Un})),n.d(r,"HAND_RIGHT",(function(){return Vn})),n.d(r,"HAND_UP",(function(){return Yn})),n.d(r,"HEADER",(function(){return Wn})),n.d(r,"HEADER_ONE",(function(){return Gn})),n.d(r,"HEADER_TWO",(function(){return Xn})),n.d(r,"HEADSET",(function(){return qn})),n.d(r,"HEART",(function(){return Zn})),n.d(r,"HEART_BROKEN",(function(){return Qn})),n.d(r,"HEAT_GRID",(function(){return Kn})),n.d(r,"HEATMAP",(function(){return Jn})),n.d(r,"HELP",(function(){return $n})),n.d(r,"HELPER_MANAGEMENT",(function(){return er})),n.d(r,"HIGHLIGHT",(function(){return tr})),n.d(r,"HISTORY",(function(){return nr})),n.d(r,"HOME",(function(){return rr})),n.d(r,"HORIZONTAL_BAR_CHART",(function(){return ir})),n.d(r,"HORIZONTAL_BAR_CHART_ASC",(function(){return or})),n.d(r,"HORIZONTAL_BAR_CHART_DESC",(function(){return ar})),n.d(r,"HORIZONTAL_DISTRIBUTION",(function(){return sr})),n.d(r,"ID_NUMBER",(function(){return lr})),n.d(r,"IMAGE_ROTATE_LEFT",(function(){return ur})),n.d(r,"IMAGE_ROTATE_RIGHT",(function(){return cr})),n.d(r,"IMPORT",(function(){return dr})),n.d(r,"INBOX",(function(){return hr})),n.d(r,"INBOX_FILTERED",(function(){return fr})),n.d(r,"INBOX_GEO",(function(){return pr})),n.d(r,"INBOX_SEARCH",(function(){return gr})),n.d(r,"INBOX_UPDATE",(function(){return vr})),n.d(r,"INFO_SIGN",(function(){return mr})),n.d(r,"INHERITANCE",(function(){return yr})),n.d(r,"INNER_JOIN",(function(){return br})),n.d(r,"INSERT",(function(){return Cr})),n.d(r,"INTERSECTION",(function(){return Ar})),n.d(r,"IP_ADDRESS",(function(){return _r})),n.d(r,"ISSUE",(function(){return Ir})),n.d(r,"ISSUE_CLOSED",(function(){return xr})),n.d(r,"ISSUE_NEW",(function(){return wr})),n.d(r,"ITALIC",(function(){return Mr})),n.d(r,"JOIN_TABLE",(function(){return Sr})),n.d(r,"KEY",(function(){return Tr})),n.d(r,"KEY_BACKSPACE",(function(){return Er})),n.d(r,"KEY_COMMAND",(function(){return Or})),n.d(r,"KEY_CONTROL",(function(){return kr})),n.d(r,"KEY_DELETE",(function(){return Lr})),n.d(r,"KEY_ENTER",(function(){return Dr})),n.d(r,"KEY_ESCAPE",(function(){return Pr})),n.d(r,"KEY_OPTION",(function(){return Nr})),n.d(r,"KEY_SHIFT",(function(){return jr})),n.d(r,"KEY_TAB",(function(){return Rr})),n.d(r,"KNOWN_VEHICLE",(function(){return zr})),n.d(r,"LAB_TEST",(function(){return Fr})),n.d(r,"LABEL",(function(){return Br})),n.d(r,"LAYER",(function(){return Hr})),n.d(r,"LAYERS",(function(){return Ur})),n.d(r,"LAYOUT",(function(){return Vr})),n.d(r,"LAYOUT_AUTO",(function(){return Yr})),n.d(r,"LAYOUT_BALLOON",(function(){return Wr})),n.d(r,"LAYOUT_CIRCLE",(function(){return Gr})),n.d(r,"LAYOUT_GRID",(function(){return Xr})),n.d(r,"LAYOUT_GROUP_BY",(function(){return qr})),n.d(r,"LAYOUT_HIERARCHY",(function(){return Zr})),n.d(r,"LAYOUT_LINEAR",(function(){return Qr})),n.d(r,"LAYOUT_SKEW_GRID",(function(){return Kr})),n.d(r,"LAYOUT_SORTED_CLUSTERS",(function(){return Jr})),n.d(r,"LEARNING",(function(){return $r})),n.d(r,"LEFT_JOIN",(function(){return ei})),n.d(r,"LESS_THAN",(function(){return ti})),n.d(r,"LESS_THAN_OR_EQUAL_TO",(function(){return ni})),n.d(r,"LIFESAVER",(function(){return ri})),n.d(r,"LIGHTBULB",(function(){return ii})),n.d(r,"LINK",(function(){return oi})),n.d(r,"LIST",(function(){return ai})),n.d(r,"LIST_COLUMNS",(function(){return si})),n.d(r,"LIST_DETAIL_VIEW",(function(){return li})),n.d(r,"LOCATE",(function(){return ui})),n.d(r,"LOCK",(function(){return ci})),n.d(r,"LOG_IN",(function(){return di})),n.d(r,"LOG_OUT",(function(){return hi})),n.d(r,"MANUAL",(function(){return fi})),n.d(r,"MANUALLY_ENTERED_DATA",(function(){return pi})),n.d(r,"MAP",(function(){return gi})),n.d(r,"MAP_CREATE",(function(){return vi})),n.d(r,"MAP_MARKER",(function(){return mi})),n.d(r,"MAXIMIZE",(function(){return yi})),n.d(r,"MEDIA",(function(){return bi})),n.d(r,"MENU",(function(){return Ci})),n.d(r,"MENU_CLOSED",(function(){return Ai})),n.d(r,"MENU_OPEN",(function(){return _i})),n.d(r,"MERGE_COLUMNS",(function(){return Ii})),n.d(r,"MERGE_LINKS",(function(){return xi})),n.d(r,"MINIMIZE",(function(){return wi})),n.d(r,"MINUS",(function(){return Mi})),n.d(r,"MOBILE_PHONE",(function(){return Si})),n.d(r,"MOBILE_VIDEO",(function(){return Ti})),n.d(r,"MOON",(function(){return Ei})),n.d(r,"MORE",(function(){return Oi})),n.d(r,"MOUNTAIN",(function(){return ki})),n.d(r,"MOVE",(function(){return Li})),n.d(r,"MUGSHOT",(function(){return Di})),n.d(r,"MULTI_SELECT",(function(){return Pi})),n.d(r,"MUSIC",(function(){return Ni})),n.d(r,"NEW_DRAWING",(function(){return ji})),n.d(r,"NEW_GRID_ITEM",(function(){return Ri})),n.d(r,"NEW_LAYER",(function(){return zi})),n.d(r,"NEW_LAYERS",(function(){return Fi})),n.d(r,"NEW_LINK",(function(){return Bi})),n.d(r,"NEW_OBJECT",(function(){return Hi})),n.d(r,"NEW_PERSON",(function(){return Ui})),n.d(r,"NEW_PRESCRIPTION",(function(){return Vi})),n.d(r,"NEW_TEXT_BOX",(function(){return Yi})),n.d(r,"NINJA",(function(){return Wi})),n.d(r,"NOT_EQUAL_TO",(function(){return Gi})),n.d(r,"NOTIFICATIONS",(function(){return Xi})),n.d(r,"NOTIFICATIONS_UPDATED",(function(){return qi})),n.d(r,"NUMBERED_LIST",(function(){return Zi})),n.d(r,"NUMERICAL",(function(){return Qi})),n.d(r,"OFFICE",(function(){return Ki})),n.d(r,"OFFLINE",(function(){return Ji})),n.d(r,"OIL_FIELD",(function(){return $i})),n.d(r,"ONE_COLUMN",(function(){return eo})),n.d(r,"OUTDATED",(function(){return to})),n.d(r,"PAGE_LAYOUT",(function(){return no})),n.d(r,"PANEL_STATS",(function(){return ro})),n.d(r,"PANEL_TABLE",(function(){return io})),n.d(r,"PAPERCLIP",(function(){return oo})),n.d(r,"PARAGRAPH",(function(){return ao})),n.d(r,"PATH",(function(){return so})),n.d(r,"PATH_SEARCH",(function(){return lo})),n.d(r,"PAUSE",(function(){return uo})),n.d(r,"PEOPLE",(function(){return co})),n.d(r,"PERCENTAGE",(function(){return ho})),n.d(r,"PERSON",(function(){return fo})),n.d(r,"PHONE",(function(){return po})),n.d(r,"PIE_CHART",(function(){return go})),n.d(r,"PIN",(function(){return vo})),n.d(r,"PIVOT",(function(){return mo})),n.d(r,"PIVOT_TABLE",(function(){return yo})),n.d(r,"PLAY",(function(){return bo})),n.d(r,"PLUS",(function(){return Co})),n.d(r,"POLYGON_FILTER",(function(){return Ao})),n.d(r,"POWER",(function(){return _o})),n.d(r,"PREDICTIVE_ANALYSIS",(function(){return Io})),n.d(r,"PRESCRIPTION",(function(){return xo})),n.d(r,"PRESENTATION",(function(){return wo})),n.d(r,"PRINT",(function(){return Mo})),n.d(r,"PROJECTS",(function(){return So})),n.d(r,"PROPERTIES",(function(){return To})),n.d(r,"PROPERTY",(function(){return Eo})),n.d(r,"PUBLISH_FUNCTION",(function(){return Oo})),n.d(r,"PULSE",(function(){return ko})),n.d(r,"RANDOM",(function(){return Lo})),n.d(r,"RECORD",(function(){return Do})),n.d(r,"REDO",(function(){return Po})),n.d(r,"REFRESH",(function(){return No})),n.d(r,"REGRESSION_CHART",(function(){return jo})),n.d(r,"REMOVE",(function(){return Ro})),n.d(r,"REMOVE_COLUMN",(function(){return zo})),n.d(r,"REMOVE_COLUMN_LEFT",(function(){return Fo})),n.d(r,"REMOVE_COLUMN_RIGHT",(function(){return Bo})),n.d(r,"REMOVE_ROW_BOTTOM",(function(){return Ho})),n.d(r,"REMOVE_ROW_TOP",(function(){return Uo})),n.d(r,"REPEAT",(function(){return Vo})),n.d(r,"RESET",(function(){return Yo})),n.d(r,"RESOLVE",(function(){return Wo})),n.d(r,"RIG",(function(){return Go})),n.d(r,"RIGHT_JOIN",(function(){return Xo})),n.d(r,"RING",(function(){return qo})),n.d(r,"ROTATE_DOCUMENT",(function(){return Zo})),n.d(r,"ROTATE_PAGE",(function(){return Qo})),n.d(r,"SATELLITE",(function(){return Ko})),n.d(r,"SAVED",(function(){return Jo})),n.d(r,"SCATTER_PLOT",(function(){return $o})),n.d(r,"SEARCH",(function(){return ea})),n.d(r,"SEARCH_AROUND",(function(){return ta})),n.d(r,"SEARCH_TEMPLATE",(function(){return na})),n.d(r,"SEARCH_TEXT",(function(){return ra})),n.d(r,"SEGMENTED_CONTROL",(function(){return ia})),n.d(r,"SELECT",(function(){return oa})),n.d(r,"SELECTION",(function(){return aa})),n.d(r,"SEND_TO",(function(){return sa})),n.d(r,"SEND_TO_GRAPH",(function(){return la})),n.d(r,"SEND_TO_MAP",(function(){return ua})),n.d(r,"SERIES_ADD",(function(){return ca})),n.d(r,"SERIES_CONFIGURATION",(function(){return da})),n.d(r,"SERIES_DERIVED",(function(){return ha})),n.d(r,"SERIES_FILTERED",(function(){return fa})),n.d(r,"SERIES_SEARCH",(function(){return pa})),n.d(r,"SETTINGS",(function(){return ga})),n.d(r,"SHARE",(function(){return va})),n.d(r,"SHIELD",(function(){return ma})),n.d(r,"SHOP",(function(){return ya})),n.d(r,"SHOPPING_CART",(function(){return ba})),n.d(r,"SIGNAL_SEARCH",(function(){return Ca})),n.d(r,"SIM_CARD",(function(){return Aa})),n.d(r,"SLASH",(function(){return _a})),n.d(r,"SMALL_CROSS",(function(){return Ia})),n.d(r,"SMALL_MINUS",(function(){return xa})),n.d(r,"SMALL_PLUS",(function(){return wa})),n.d(r,"SMALL_TICK",(function(){return Ma})),n.d(r,"SNOWFLAKE",(function(){return Sa})),n.d(r,"SOCIAL_MEDIA",(function(){return Ta})),n.d(r,"SORT",(function(){return Ea})),n.d(r,"SORT_ALPHABETICAL",(function(){return Oa})),n.d(r,"SORT_ALPHABETICAL_DESC",(function(){return ka})),n.d(r,"SORT_ASC",(function(){return La})),n.d(r,"SORT_DESC",(function(){return Da})),n.d(r,"SORT_NUMERICAL",(function(){return Pa})),n.d(r,"SORT_NUMERICAL_DESC",(function(){return Na})),n.d(r,"SPLIT_COLUMNS",(function(){return ja})),n.d(r,"SQUARE",(function(){return Ra})),n.d(r,"STACKED_CHART",(function(){return za})),n.d(r,"STAR",(function(){return Fa})),n.d(r,"STAR_EMPTY",(function(){return Ba})),n.d(r,"STEP_BACKWARD",(function(){return Ha})),n.d(r,"STEP_CHART",(function(){return Ua})),n.d(r,"STEP_FORWARD",(function(){return Va})),n.d(r,"STOP",(function(){return Ya})),n.d(r,"STOPWATCH",(function(){return Wa})),n.d(r,"STRIKETHROUGH",(function(){return Ga})),n.d(r,"STYLE",(function(){return Xa})),n.d(r,"SWAP_HORIZONTAL",(function(){return qa})),n.d(r,"SWAP_VERTICAL",(function(){return Za})),n.d(r,"SYMBOL_CIRCLE",(function(){return Qa})),n.d(r,"SYMBOL_CROSS",(function(){return Ka})),n.d(r,"SYMBOL_DIAMOND",(function(){return Ja})),n.d(r,"SYMBOL_SQUARE",(function(){return $a})),n.d(r,"SYMBOL_TRIANGLE_DOWN",(function(){return es})),n.d(r,"SYMBOL_TRIANGLE_UP",(function(){return ts})),n.d(r,"TAG",(function(){return ns})),n.d(r,"TAKE_ACTION",(function(){return rs})),n.d(r,"TAXI",(function(){return is})),n.d(r,"TEXT_HIGHLIGHT",(function(){return os})),n.d(r,"TH",(function(){return as})),n.d(r,"TH_DERIVED",(function(){return ss})),n.d(r,"TH_DISCONNECT",(function(){return ls})),n.d(r,"TH_FILTERED",(function(){return us})),n.d(r,"TH_LIST",(function(){return cs})),n.d(r,"THUMBS_DOWN",(function(){return ds})),n.d(r,"THUMBS_UP",(function(){return hs})),n.d(r,"TICK",(function(){return fs})),n.d(r,"TICK_CIRCLE",(function(){return ps})),n.d(r,"TIME",(function(){return gs})),n.d(r,"TIMELINE_AREA_CHART",(function(){return vs})),n.d(r,"TIMELINE_BAR_CHART",(function(){return ms})),n.d(r,"TIMELINE_EVENTS",(function(){return ys})),n.d(r,"TIMELINE_LINE_CHART",(function(){return bs})),n.d(r,"TINT",(function(){return Cs})),n.d(r,"TORCH",(function(){return As})),n.d(r,"TRACTOR",(function(){return _s})),n.d(r,"TRAIN",(function(){return Is})),n.d(r,"TRANSLATE",(function(){return xs})),n.d(r,"TRASH",(function(){return ws})),n.d(r,"TREE",(function(){return Ms})),n.d(r,"TRENDING_DOWN",(function(){return Ss})),n.d(r,"TRENDING_UP",(function(){return Ts})),n.d(r,"TRUCK",(function(){return Es})),n.d(r,"TWO_COLUMNS",(function(){return Os})),n.d(r,"UNARCHIVE",(function(){return ks})),n.d(r,"UNDERLINE",(function(){return Ls})),n.d(r,"UNDO",(function(){return Ds})),n.d(r,"UNGROUP_OBJECTS",(function(){return Ps})),n.d(r,"UNKNOWN_VEHICLE",(function(){return Ns})),n.d(r,"UNLOCK",(function(){return js})),n.d(r,"UNPIN",(function(){return Rs})),n.d(r,"UNRESOLVE",(function(){return zs})),n.d(r,"UPDATED",(function(){return Fs})),n.d(r,"UPLOAD",(function(){return Bs})),n.d(r,"USER",(function(){return Hs})),n.d(r,"VARIABLE",(function(){return Us})),n.d(r,"VERTICAL_BAR_CHART_ASC",(function(){return Vs})),n.d(r,"VERTICAL_BAR_CHART_DESC",(function(){return Ys})),n.d(r,"VERTICAL_DISTRIBUTION",(function(){return Ws})),n.d(r,"VIDEO",(function(){return Gs})),n.d(r,"VOLUME_DOWN",(function(){return Xs})),n.d(r,"VOLUME_OFF",(function(){return qs})),n.d(r,"VOLUME_UP",(function(){return Zs})),n.d(r,"WALK",(function(){return Qs})),n.d(r,"WARNING_SIGN",(function(){return Ks})),n.d(r,"WATERFALL_CHART",(function(){return Js})),n.d(r,"WIDGET",(function(){return $s})),n.d(r,"WIDGET_BUTTON",(function(){return el})),n.d(r,"WIDGET_FOOTER",(function(){return tl})),n.d(r,"WIDGET_HEADER",(function(){return nl})),n.d(r,"WRENCH",(function(){return rl})),n.d(r,"ZOOM_IN",(function(){return il})),n.d(r,"ZOOM_OUT",(function(){return ol})),n.d(r,"ZOOM_TO_FIT",(function(){return al}));var i={};n.r(i),n.d(i,"ADD",(function(){return sl})),n.d(i,"ADD_COLUMN_LEFT",(function(){return ll})),n.d(i,"ADD_COLUMN_RIGHT",(function(){return ul})),n.d(i,"ADD_ROW_BOTTOM",(function(){return cl})),n.d(i,"ADD_ROW_TOP",(function(){return dl})),n.d(i,"ADD_TO_ARTIFACT",(function(){return hl})),n.d(i,"ADD_TO_FOLDER",(function(){return fl})),n.d(i,"AIRPLANE",(function(){return pl})),n.d(i,"ALIGN_CENTER",(function(){return gl})),n.d(i,"ALIGN_JUSTIFY",(function(){return vl})),n.d(i,"ALIGN_LEFT",(function(){return ml})),n.d(i,"ALIGN_RIGHT",(function(){return yl})),n.d(i,"ALIGNMENT_BOTTOM",(function(){return bl})),n.d(i,"ALIGNMENT_HORIZONTAL_CENTER",(function(){return Cl})),n.d(i,"ALIGNMENT_LEFT",(function(){return Al})),n.d(i,"ALIGNMENT_RIGHT",(function(){return _l})),n.d(i,"ALIGNMENT_TOP",(function(){return Il})),n.d(i,"ALIGNMENT_VERTICAL_CENTER",(function(){return xl})),n.d(i,"ANNOTATION",(function(){return wl})),n.d(i,"APP_HEADER",(function(){return Ml})),n.d(i,"APPLICATION",(function(){return Sl})),n.d(i,"APPLICATIONS",(function(){return Tl})),n.d(i,"ARCHIVE",(function(){return El})),n.d(i,"ARRAY",(function(){return Ol})),n.d(i,"ARRAY_BOOLEAN",(function(){return kl})),n.d(i,"ARRAY_DATE",(function(){return Ll})),n.d(i,"ARRAY_NUMERIC",(function(){return Dl})),n.d(i,"ARRAY_STRING",(function(){return Pl})),n.d(i,"ARRAY_TIMESTAMP",(function(){return Nl})),n.d(i,"ARROW_BOTTOM_LEFT",(function(){return jl})),n.d(i,"ARROW_BOTTOM_RIGHT",(function(){return Rl})),n.d(i,"ARROW_DOWN",(function(){return zl})),n.d(i,"ARROW_LEFT",(function(){return Fl})),n.d(i,"ARROW_RIGHT",(function(){return Bl})),n.d(i,"ARROW_TOP_LEFT",(function(){return Hl})),n.d(i,"ARROW_TOP_RIGHT",(function(){return Ul})),n.d(i,"ARROW_UP",(function(){return Vl})),n.d(i,"ARROWS_HORIZONTAL",(function(){return Yl})),n.d(i,"ARROWS_VERTICAL",(function(){return Wl})),n.d(i,"ASTERISK",(function(){return Gl})),n.d(i,"AUTOMATIC_UPDATES",(function(){return Xl})),n.d(i,"BACKLINK",(function(){return ql})),n.d(i,"BADGE",(function(){return Zl})),n.d(i,"BAN_CIRCLE",(function(){return Ql})),n.d(i,"BANK_ACCOUNT",(function(){return Kl})),n.d(i,"BARCODE",(function(){return Jl})),n.d(i,"BLANK",(function(){return $l})),n.d(i,"BLOCKED_PERSON",(function(){return eu})),n.d(i,"BOLD",(function(){return tu})),n.d(i,"BOOK",(function(){return nu})),n.d(i,"BOOKMARK",(function(){return ru})),n.d(i,"BOX",(function(){return iu})),n.d(i,"BRIEFCASE",(function(){return ou})),n.d(i,"BRING_DATA",(function(){return au})),n.d(i,"BUILD",(function(){return su})),n.d(i,"CALCULATOR",(function(){return lu})),n.d(i,"CALENDAR",(function(){return uu})),n.d(i,"CAMERA",(function(){return cu})),n.d(i,"CARET_DOWN",(function(){return du})),n.d(i,"CARET_LEFT",(function(){return hu})),n.d(i,"CARET_RIGHT",(function(){return fu})),n.d(i,"CARET_UP",(function(){return pu})),n.d(i,"CELL_TOWER",(function(){return gu})),n.d(i,"CHANGES",(function(){return vu})),n.d(i,"CHART",(function(){return mu})),n.d(i,"CHAT",(function(){return yu})),n.d(i,"CHEVRON_BACKWARD",(function(){return bu})),n.d(i,"CHEVRON_DOWN",(function(){return Cu})),n.d(i,"CHEVRON_FORWARD",(function(){return Au})),n.d(i,"CHEVRON_LEFT",(function(){return _u})),n.d(i,"CHEVRON_RIGHT",(function(){return Iu})),n.d(i,"CHEVRON_UP",(function(){return xu})),n.d(i,"CIRCLE",(function(){return wu})),n.d(i,"CIRCLE_ARROW_DOWN",(function(){return Mu})),n.d(i,"CIRCLE_ARROW_LEFT",(function(){return Su})),n.d(i,"CIRCLE_ARROW_RIGHT",(function(){return Tu})),n.d(i,"CIRCLE_ARROW_UP",(function(){return Eu})),n.d(i,"CITATION",(function(){return Ou})),n.d(i,"CLEAN",(function(){return ku})),n.d(i,"CLIPBOARD",(function(){return Lu})),n.d(i,"CLOUD",(function(){return Du})),n.d(i,"CLOUD_DOWNLOAD",(function(){return Pu})),n.d(i,"CLOUD_UPLOAD",(function(){return Nu})),n.d(i,"CODE",(function(){return ju})),n.d(i,"CODE_BLOCK",(function(){return Ru})),n.d(i,"COG",(function(){return zu})),n.d(i,"COLLAPSE_ALL",(function(){return Fu})),n.d(i,"COLUMN_LAYOUT",(function(){return Bu})),n.d(i,"COMMENT",(function(){return Hu})),n.d(i,"COMPARISON",(function(){return Uu})),n.d(i,"COMPASS",(function(){return Vu})),n.d(i,"COMPRESSED",(function(){return Yu})),n.d(i,"CONFIRM",(function(){return Wu})),n.d(i,"CONSOLE",(function(){return Gu})),n.d(i,"CONTRAST",(function(){return Xu})),n.d(i,"CONTROL",(function(){return qu})),n.d(i,"CREDIT_CARD",(function(){return Zu})),n.d(i,"CROSS",(function(){return Qu})),n.d(i,"CROWN",(function(){return Ku})),n.d(i,"CUBE",(function(){return Ju})),n.d(i,"CUBE_ADD",(function(){return $u})),n.d(i,"CUBE_REMOVE",(function(){return ec})),n.d(i,"CURVED_RANGE_CHART",(function(){return tc})),n.d(i,"CUT",(function(){return nc})),n.d(i,"CYCLE",(function(){return rc})),n.d(i,"DASHBOARD",(function(){return ic})),n.d(i,"DATA_CONNECTION",(function(){return oc})),n.d(i,"DATA_LINEAGE",(function(){return ac})),n.d(i,"DATABASE",(function(){return sc})),n.d(i,"DELETE",(function(){return lc})),n.d(i,"DELTA",(function(){return uc})),n.d(i,"DERIVE_COLUMN",(function(){return cc})),n.d(i,"DESKTOP",(function(){return dc})),n.d(i,"DIAGNOSIS",(function(){return hc})),n.d(i,"DIAGRAM_TREE",(function(){return fc})),n.d(i,"DIRECTION_LEFT",(function(){return pc})),n.d(i,"DIRECTION_RIGHT",(function(){return gc})),n.d(i,"DISABLE",(function(){return vc})),n.d(i,"DOCUMENT",(function(){return mc})),n.d(i,"DOCUMENT_OPEN",(function(){return yc})),n.d(i,"DOCUMENT_SHARE",(function(){return bc})),n.d(i,"DOLLAR",(function(){return Cc})),n.d(i,"DOT",(function(){return Ac})),n.d(i,"DOUBLE_CARET_HORIZONTAL",(function(){return _c})),n.d(i,"DOUBLE_CARET_VERTICAL",(function(){return Ic})),n.d(i,"DOUBLE_CHEVRON_DOWN",(function(){return xc})),n.d(i,"DOUBLE_CHEVRON_LEFT",(function(){return wc})),n.d(i,"DOUBLE_CHEVRON_RIGHT",(function(){return Mc})),n.d(i,"DOUBLE_CHEVRON_UP",(function(){return Sc})),n.d(i,"DOUGHNUT_CHART",(function(){return Tc})),n.d(i,"DOWNLOAD",(function(){return Ec})),n.d(i,"DRAG_HANDLE_HORIZONTAL",(function(){return Oc})),n.d(i,"DRAG_HANDLE_VERTICAL",(function(){return kc})),n.d(i,"DRAW",(function(){return Lc})),n.d(i,"DRAWER_LEFT",(function(){return Dc})),n.d(i,"DRAWER_LEFT_FILLED",(function(){return Pc})),n.d(i,"DRAWER_RIGHT",(function(){return Nc})),n.d(i,"DRAWER_RIGHT_FILLED",(function(){return jc})),n.d(i,"DRIVE_TIME",(function(){return Rc})),n.d(i,"DUPLICATE",(function(){return zc})),n.d(i,"EDIT",(function(){return Fc})),n.d(i,"EJECT",(function(){return Bc})),n.d(i,"EMOJI",(function(){return Hc})),n.d(i,"ENDORSED",(function(){return Uc})),n.d(i,"ENVELOPE",(function(){return Vc})),n.d(i,"EQUALS",(function(){return Yc})),n.d(i,"ERASER",(function(){return Wc})),n.d(i,"ERROR",(function(){return Gc})),n.d(i,"EURO",(function(){return Xc})),n.d(i,"EXCHANGE",(function(){return qc})),n.d(i,"EXCLUDE_ROW",(function(){return Zc})),n.d(i,"EXPAND_ALL",(function(){return Qc})),n.d(i,"EXPORT",(function(){return Kc})),n.d(i,"EYE_OFF",(function(){return Jc})),n.d(i,"EYE_ON",(function(){return $c})),n.d(i,"EYE_OPEN",(function(){return ed})),n.d(i,"FAST_BACKWARD",(function(){return td})),n.d(i,"FAST_FORWARD",(function(){return nd})),n.d(i,"FEED",(function(){return rd})),n.d(i,"FEED_SUBSCRIBED",(function(){return id})),n.d(i,"FILM",(function(){return od})),n.d(i,"FILTER",(function(){return ad})),n.d(i,"FILTER_KEEP",(function(){return sd})),n.d(i,"FILTER_LIST",(function(){return ld})),n.d(i,"FILTER_OPEN",(function(){return ud})),n.d(i,"FILTER_REMOVE",(function(){return cd})),n.d(i,"FLAG",(function(){return dd})),n.d(i,"FLAME",(function(){return hd})),n.d(i,"FLASH",(function(){return fd})),n.d(i,"FLOPPY_DISK",(function(){return pd})),n.d(i,"FLOW_BRANCH",(function(){return gd})),n.d(i,"FLOW_END",(function(){return vd})),n.d(i,"FLOW_LINEAR",(function(){return md})),n.d(i,"FLOW_REVIEW",(function(){return yd})),n.d(i,"FLOW_REVIEW_BRANCH",(function(){return bd})),n.d(i,"FLOWS",(function(){return Cd})),n.d(i,"FOLDER_CLOSE",(function(){return Ad})),n.d(i,"FOLDER_NEW",(function(){return _d})),n.d(i,"FOLDER_OPEN",(function(){return Id})),n.d(i,"FOLDER_SHARED",(function(){return xd})),n.d(i,"FOLDER_SHARED_OPEN",(function(){return wd})),n.d(i,"FOLLOWER",(function(){return Md})),n.d(i,"FOLLOWING",(function(){return Sd})),n.d(i,"FONT",(function(){return Td})),n.d(i,"FORK",(function(){return Ed})),n.d(i,"FORM",(function(){return Od})),n.d(i,"FULL_CIRCLE",(function(){return kd})),n.d(i,"FULL_STACKED_CHART",(function(){return Ld})),n.d(i,"FULLSCREEN",(function(){return Dd})),n.d(i,"FUNCTION",(function(){return Pd})),n.d(i,"GANTT_CHART",(function(){return Nd})),n.d(i,"GEOFENCE",(function(){return jd})),n.d(i,"GEOLOCATION",(function(){return Rd})),n.d(i,"GEOSEARCH",(function(){return zd})),n.d(i,"GIT_BRANCH",(function(){return Fd})),n.d(i,"GIT_COMMIT",(function(){return Bd})),n.d(i,"GIT_MERGE",(function(){return Hd})),n.d(i,"GIT_NEW_BRANCH",(function(){return Ud})),n.d(i,"GIT_PULL",(function(){return Vd})),n.d(i,"GIT_PUSH",(function(){return Yd})),n.d(i,"GIT_REPO",(function(){return Wd})),n.d(i,"GLASS",(function(){return Gd})),n.d(i,"GLOBE",(function(){return Xd})),n.d(i,"GLOBE_NETWORK",(function(){return qd})),n.d(i,"GRAPH",(function(){return Zd})),n.d(i,"GRAPH_REMOVE",(function(){return Qd})),n.d(i,"GREATER_THAN",(function(){return Kd})),n.d(i,"GREATER_THAN_OR_EQUAL_TO",(function(){return Jd})),n.d(i,"GRID",(function(){return $d})),n.d(i,"GRID_VIEW",(function(){return eh})),n.d(i,"GROUP_OBJECTS",(function(){return th})),n.d(i,"GROUPED_BAR_CHART",(function(){return nh})),n.d(i,"HAND",(function(){return rh})),n.d(i,"HAND_DOWN",(function(){return ih})),n.d(i,"HAND_LEFT",(function(){return oh})),n.d(i,"HAND_RIGHT",(function(){return ah})),n.d(i,"HAND_UP",(function(){return sh})),n.d(i,"HAT",(function(){return lh})),n.d(i,"HEADER",(function(){return uh})),n.d(i,"HEADER_ONE",(function(){return ch})),n.d(i,"HEADER_TWO",(function(){return dh})),n.d(i,"HEADSET",(function(){return hh})),n.d(i,"HEART",(function(){return fh})),n.d(i,"HEART_BROKEN",(function(){return ph})),n.d(i,"HEAT_GRID",(function(){return gh})),n.d(i,"HEATMAP",(function(){return vh})),n.d(i,"HELP",(function(){return mh})),n.d(i,"HELPER_MANAGEMENT",(function(){return yh})),n.d(i,"HIGH_PRIORITY",(function(){return bh})),n.d(i,"HIGHLIGHT",(function(){return Ch})),n.d(i,"HISTORY",(function(){return Ah})),n.d(i,"HOME",(function(){return _h})),n.d(i,"HORIZONTAL_BAR_CHART",(function(){return Ih})),n.d(i,"HORIZONTAL_BAR_CHART_ASC",(function(){return xh})),n.d(i,"HORIZONTAL_BAR_CHART_DESC",(function(){return wh})),n.d(i,"HORIZONTAL_DISTRIBUTION",(function(){return Mh})),n.d(i,"ID_NUMBER",(function(){return Sh})),n.d(i,"IMAGE_ROTATE_LEFT",(function(){return Th})),n.d(i,"IMAGE_ROTATE_RIGHT",(function(){return Eh})),n.d(i,"IMPORT",(function(){return Oh})),n.d(i,"INBOX",(function(){return kh})),n.d(i,"INBOX_FILTERED",(function(){return Lh})),n.d(i,"INBOX_GEO",(function(){return Dh})),n.d(i,"INBOX_SEARCH",(function(){return Ph})),n.d(i,"INBOX_UPDATE",(function(){return Nh})),n.d(i,"INFO_SIGN",(function(){return jh})),n.d(i,"INHERITANCE",(function(){return Rh})),n.d(i,"INHERITED_GROUP",(function(){return zh})),n.d(i,"INNER_JOIN",(function(){return Fh})),n.d(i,"INSERT",(function(){return Bh})),n.d(i,"INTERSECTION",(function(){return Hh})),n.d(i,"IP_ADDRESS",(function(){return Uh})),n.d(i,"ISSUE",(function(){return Vh})),n.d(i,"ISSUE_CLOSED",(function(){return Yh})),n.d(i,"ISSUE_NEW",(function(){return Wh})),n.d(i,"ITALIC",(function(){return Gh})),n.d(i,"JOIN_TABLE",(function(){return Xh})),n.d(i,"KEY",(function(){return qh})),n.d(i,"KEY_BACKSPACE",(function(){return Zh})),n.d(i,"KEY_COMMAND",(function(){return Qh})),n.d(i,"KEY_CONTROL",(function(){return Kh})),n.d(i,"KEY_DELETE",(function(){return Jh})),n.d(i,"KEY_ENTER",(function(){return $h})),n.d(i,"KEY_ESCAPE",(function(){return ef})),n.d(i,"KEY_OPTION",(function(){return tf})),n.d(i,"KEY_SHIFT",(function(){return nf})),n.d(i,"KEY_TAB",(function(){return rf})),n.d(i,"KNOWN_VEHICLE",(function(){return of})),n.d(i,"LAB_TEST",(function(){return af})),n.d(i,"LABEL",(function(){return sf})),n.d(i,"LAYER",(function(){return lf})),n.d(i,"LAYER_OUTLINE",(function(){return uf})),n.d(i,"LAYERS",(function(){return cf})),n.d(i,"LAYOUT",(function(){return df})),n.d(i,"LAYOUT_AUTO",(function(){return hf})),n.d(i,"LAYOUT_BALLOON",(function(){return ff})),n.d(i,"LAYOUT_CIRCLE",(function(){return pf})),n.d(i,"LAYOUT_GRID",(function(){return gf})),n.d(i,"LAYOUT_GROUP_BY",(function(){return vf})),n.d(i,"LAYOUT_HIERARCHY",(function(){return mf})),n.d(i,"LAYOUT_LINEAR",(function(){return yf})),n.d(i,"LAYOUT_SKEW_GRID",(function(){return bf})),n.d(i,"LAYOUT_SORTED_CLUSTERS",(function(){return Cf})),n.d(i,"LEARNING",(function(){return Af})),n.d(i,"LEFT_JOIN",(function(){return _f})),n.d(i,"LESS_THAN",(function(){return If})),n.d(i,"LESS_THAN_OR_EQUAL_TO",(function(){return xf})),n.d(i,"LIFESAVER",(function(){return wf})),n.d(i,"LIGHTBULB",(function(){return Mf})),n.d(i,"LINK",(function(){return Sf})),n.d(i,"LIST",(function(){return Tf})),n.d(i,"LIST_COLUMNS",(function(){return Ef})),n.d(i,"LIST_DETAIL_VIEW",(function(){return Of})),n.d(i,"LOCATE",(function(){return kf})),n.d(i,"LOCK",(function(){return Lf})),n.d(i,"LOG_IN",(function(){return Df})),n.d(i,"LOG_OUT",(function(){return Pf})),n.d(i,"MANUAL",(function(){return Nf})),n.d(i,"MANUALLY_ENTERED_DATA",(function(){return jf})),n.d(i,"MAP",(function(){return Rf})),n.d(i,"MAP_CREATE",(function(){return zf})),n.d(i,"MAP_MARKER",(function(){return Ff})),n.d(i,"MAXIMIZE",(function(){return Bf})),n.d(i,"MEDIA",(function(){return Hf})),n.d(i,"MENU",(function(){return Uf})),n.d(i,"MENU_CLOSED",(function(){return Vf})),n.d(i,"MENU_OPEN",(function(){return Yf})),n.d(i,"MERGE_COLUMNS",(function(){return Wf})),n.d(i,"MERGE_LINKS",(function(){return Gf})),n.d(i,"MINIMIZE",(function(){return Xf})),n.d(i,"MINUS",(function(){return qf})),n.d(i,"MOBILE_PHONE",(function(){return Zf})),n.d(i,"MOBILE_VIDEO",(function(){return Qf})),n.d(i,"MODAL",(function(){return Kf})),n.d(i,"MODAL_FILLED",(function(){return Jf})),n.d(i,"MOON",(function(){return $f})),n.d(i,"MORE",(function(){return ep})),n.d(i,"MOUNTAIN",(function(){return tp})),n.d(i,"MOVE",(function(){return np})),n.d(i,"MUGSHOT",(function(){return rp})),n.d(i,"MULTI_SELECT",(function(){return ip})),n.d(i,"MUSIC",(function(){return op})),n.d(i,"NEW_DRAWING",(function(){return ap})),n.d(i,"NEW_GRID_ITEM",(function(){return sp})),n.d(i,"NEW_LAYER",(function(){return lp})),n.d(i,"NEW_LAYERS",(function(){return up})),n.d(i,"NEW_LINK",(function(){return cp})),n.d(i,"NEW_OBJECT",(function(){return dp})),n.d(i,"NEW_PERSON",(function(){return hp})),n.d(i,"NEW_PRESCRIPTION",(function(){return fp})),n.d(i,"NEW_TEXT_BOX",(function(){return pp})),n.d(i,"NINJA",(function(){return gp})),n.d(i,"NOT_EQUAL_TO",(function(){return vp})),n.d(i,"NOTIFICATIONS",(function(){return mp})),n.d(i,"NOTIFICATIONS_SNOOZE",(function(){return yp})),n.d(i,"NOTIFICATIONS_UPDATED",(function(){return bp})),n.d(i,"NUMBERED_LIST",(function(){return Cp})),n.d(i,"NUMERICAL",(function(){return Ap})),n.d(i,"OFFICE",(function(){return _p})),n.d(i,"OFFLINE",(function(){return Ip})),n.d(i,"OIL_FIELD",(function(){return xp})),n.d(i,"ONE_COLUMN",(function(){return wp})),n.d(i,"OUTDATED",(function(){return Mp})),n.d(i,"PAGE_LAYOUT",(function(){return Sp})),n.d(i,"PANEL_STATS",(function(){return Tp})),n.d(i,"PANEL_TABLE",(function(){return Ep})),n.d(i,"PAPERCLIP",(function(){return Op})),n.d(i,"PARAGRAPH",(function(){return kp})),n.d(i,"PATH",(function(){return Lp})),n.d(i,"PATH_SEARCH",(function(){return Dp})),n.d(i,"PAUSE",(function(){return Pp})),n.d(i,"PEOPLE",(function(){return Np})),n.d(i,"PERCENTAGE",(function(){return jp})),n.d(i,"PERSON",(function(){return Rp})),n.d(i,"PHONE",(function(){return zp})),n.d(i,"PIE_CHART",(function(){return Fp})),n.d(i,"PIN",(function(){return Bp})),n.d(i,"PIVOT",(function(){return Hp})),n.d(i,"PIVOT_TABLE",(function(){return Up})),n.d(i,"PLAY",(function(){return Vp})),n.d(i,"PLUS",(function(){return Yp})),n.d(i,"POLYGON_FILTER",(function(){return Wp})),n.d(i,"POWER",(function(){return Gp})),n.d(i,"PREDICTIVE_ANALYSIS",(function(){return Xp})),n.d(i,"PRESCRIPTION",(function(){return qp})),n.d(i,"PRESENTATION",(function(){return Zp})),n.d(i,"PRINT",(function(){return Qp})),n.d(i,"PROJECTS",(function(){return Kp})),n.d(i,"PROPERTIES",(function(){return Jp})),n.d(i,"PROPERTY",(function(){return $p})),n.d(i,"PUBLISH_FUNCTION",(function(){return eg})),n.d(i,"PULSE",(function(){return tg})),n.d(i,"RANDOM",(function(){return ng})),n.d(i,"RECORD",(function(){return rg})),n.d(i,"REDO",(function(){return ig})),n.d(i,"REFRESH",(function(){return og})),n.d(i,"REGRESSION_CHART",(function(){return ag})),n.d(i,"REMOVE",(function(){return sg})),n.d(i,"REMOVE_COLUMN",(function(){return lg})),n.d(i,"REMOVE_COLUMN_LEFT",(function(){return ug})),n.d(i,"REMOVE_COLUMN_RIGHT",(function(){return cg})),n.d(i,"REMOVE_ROW_BOTTOM",(function(){return dg})),n.d(i,"REMOVE_ROW_TOP",(function(){return hg})),n.d(i,"REPEAT",(function(){return fg})),n.d(i,"RESET",(function(){return pg})),n.d(i,"RESOLVE",(function(){return gg})),n.d(i,"RIG",(function(){return vg})),n.d(i,"RIGHT_JOIN",(function(){return mg})),n.d(i,"RING",(function(){return yg})),n.d(i,"ROTATE_DOCUMENT",(function(){return bg})),n.d(i,"ROTATE_PAGE",(function(){return Cg})),n.d(i,"ROUTE",(function(){return Ag})),n.d(i,"SATELLITE",(function(){return _g})),n.d(i,"SAVED",(function(){return Ig})),n.d(i,"SCATTER_PLOT",(function(){return xg})),n.d(i,"SEARCH",(function(){return wg})),n.d(i,"SEARCH_AROUND",(function(){return Mg})),n.d(i,"SEARCH_TEMPLATE",(function(){return Sg})),n.d(i,"SEARCH_TEXT",(function(){return Tg})),n.d(i,"SEGMENTED_CONTROL",(function(){return Eg})),n.d(i,"SELECT",(function(){return Og})),n.d(i,"SELECTION",(function(){return kg})),n.d(i,"SEND_MESSAGE",(function(){return Lg})),n.d(i,"SEND_TO",(function(){return Dg})),n.d(i,"SEND_TO_GRAPH",(function(){return Pg})),n.d(i,"SEND_TO_MAP",(function(){return Ng})),n.d(i,"SERIES_ADD",(function(){return jg})),n.d(i,"SERIES_CONFIGURATION",(function(){return Rg})),n.d(i,"SERIES_DERIVED",(function(){return zg})),n.d(i,"SERIES_FILTERED",(function(){return Fg})),n.d(i,"SERIES_SEARCH",(function(){return Bg})),n.d(i,"SETTINGS",(function(){return Hg})),n.d(i,"SHARE",(function(){return Ug})),n.d(i,"SHIELD",(function(){return Vg})),n.d(i,"SHOP",(function(){return Yg})),n.d(i,"SHOPPING_CART",(function(){return Wg})),n.d(i,"SIGNAL_SEARCH",(function(){return Gg})),n.d(i,"SIM_CARD",(function(){return Xg})),n.d(i,"SLASH",(function(){return qg})),n.d(i,"SMALL_CROSS",(function(){return Zg})),n.d(i,"SMALL_MINUS",(function(){return Qg})),n.d(i,"SMALL_PLUS",(function(){return Kg})),n.d(i,"SMALL_TICK",(function(){return Jg})),n.d(i,"SNOWFLAKE",(function(){return $g})),n.d(i,"SOCIAL_MEDIA",(function(){return ev})),n.d(i,"SORT",(function(){return tv})),n.d(i,"SORT_ALPHABETICAL",(function(){return nv})),n.d(i,"SORT_ALPHABETICAL_DESC",(function(){return rv})),n.d(i,"SORT_ASC",(function(){return iv})),n.d(i,"SORT_DESC",(function(){return ov})),n.d(i,"SORT_NUMERICAL",(function(){return av})),n.d(i,"SORT_NUMERICAL_DESC",(function(){return sv})),n.d(i,"SPLIT_COLUMNS",(function(){return lv})),n.d(i,"SQUARE",(function(){return uv})),n.d(i,"STACKED_CHART",(function(){return cv})),n.d(i,"STAR",(function(){return dv})),n.d(i,"STAR_EMPTY",(function(){return hv})),n.d(i,"STEP_BACKWARD",(function(){return fv})),n.d(i,"STEP_CHART",(function(){return pv})),n.d(i,"STEP_FORWARD",(function(){return gv})),n.d(i,"STOP",(function(){return vv})),n.d(i,"STOPWATCH",(function(){return mv})),n.d(i,"STRIKETHROUGH",(function(){return yv})),n.d(i,"STYLE",(function(){return bv})),n.d(i,"SWAP_HORIZONTAL",(function(){return Cv})),n.d(i,"SWAP_VERTICAL",(function(){return Av})),n.d(i,"SWITCH",(function(){return _v})),n.d(i,"SYMBOL_CIRCLE",(function(){return Iv})),n.d(i,"SYMBOL_CROSS",(function(){return xv})),n.d(i,"SYMBOL_DIAMOND",(function(){return wv})),n.d(i,"SYMBOL_SQUARE",(function(){return Mv})),n.d(i,"SYMBOL_TRIANGLE_DOWN",(function(){return Sv})),n.d(i,"SYMBOL_TRIANGLE_UP",(function(){return Tv})),n.d(i,"TAG",(function(){return Ev})),n.d(i,"TAKE_ACTION",(function(){return Ov})),n.d(i,"TAXI",(function(){return kv})),n.d(i,"TEXT_HIGHLIGHT",(function(){return Lv})),n.d(i,"TH",(function(){return Dv})),n.d(i,"TH_DERIVED",(function(){return Pv})),n.d(i,"TH_DISCONNECT",(function(){return Nv})),n.d(i,"TH_FILTERED",(function(){return jv})),n.d(i,"TH_LIST",(function(){return Rv})),n.d(i,"THUMBS_DOWN",(function(){return zv})),n.d(i,"THUMBS_UP",(function(){return Fv})),n.d(i,"TICK",(function(){return Bv})),n.d(i,"TICK_CIRCLE",(function(){return Hv})),n.d(i,"TIME",(function(){return Uv})),n.d(i,"TIMELINE_AREA_CHART",(function(){return Vv})),n.d(i,"TIMELINE_BAR_CHART",(function(){return Yv})),n.d(i,"TIMELINE_EVENTS",(function(){return Wv})),n.d(i,"TIMELINE_LINE_CHART",(function(){return Gv})),n.d(i,"TINT",(function(){return Xv})),n.d(i,"TORCH",(function(){return qv})),n.d(i,"TRACTOR",(function(){return Zv})),n.d(i,"TRAIN",(function(){return Qv})),n.d(i,"TRANSLATE",(function(){return Kv})),n.d(i,"TRASH",(function(){return Jv})),n.d(i,"TREE",(function(){return $v})),n.d(i,"TRENDING_DOWN",(function(){return em})),n.d(i,"TRENDING_UP",(function(){return tm})),n.d(i,"TRUCK",(function(){return nm})),n.d(i,"TWO_COLUMNS",(function(){return rm})),n.d(i,"UNARCHIVE",(function(){return im})),n.d(i,"UNDERLINE",(function(){return om})),n.d(i,"UNDO",(function(){return am})),n.d(i,"UNGROUP_OBJECTS",(function(){return sm})),n.d(i,"UNKNOWN_VEHICLE",(function(){return lm})),n.d(i,"UNLOCK",(function(){return um})),n.d(i,"UNPIN",(function(){return cm})),n.d(i,"UNRESOLVE",(function(){return dm})),n.d(i,"UPDATED",(function(){return hm})),n.d(i,"UPLOAD",(function(){return fm})),n.d(i,"USER",(function(){return pm})),n.d(i,"VARIABLE",(function(){return gm})),n.d(i,"VERTICAL_BAR_CHART_ASC",(function(){return vm})),n.d(i,"VERTICAL_BAR_CHART_DESC",(function(){return mm})),n.d(i,"VERTICAL_DISTRIBUTION",(function(){return ym})),n.d(i,"VIDEO",(function(){return bm})),n.d(i,"VIRUS",(function(){return Cm})),n.d(i,"VOLUME_DOWN",(function(){return Am})),n.d(i,"VOLUME_OFF",(function(){return _m})),n.d(i,"VOLUME_UP",(function(){return Im})),n.d(i,"WALK",(function(){return xm})),n.d(i,"WARNING_SIGN",(function(){return wm})),n.d(i,"WATERFALL_CHART",(function(){return Mm})),n.d(i,"WIDGET",(function(){return Sm})),n.d(i,"WIDGET_BUTTON",(function(){return Tm})),n.d(i,"WIDGET_FOOTER",(function(){return Em})),n.d(i,"WIDGET_HEADER",(function(){return Om})),n.d(i,"WRENCH",(function(){return km})),n.d(i,"ZOOM_IN",(function(){return Lm})),n.d(i,"ZOOM_OUT",(function(){return Dm})),n.d(i,"ZOOM_TO_FIT",(function(){return Pm}));var o="",a="",s="",l="",u="",c="",d="",h="",f="",p="",g="",v="",m="",y="",b="",C="",A="",_="",I="",x="",w="",M="",S="↙",T="↘",E="↓ ",O="←",k="→",L="↖",D="↗",P="↑ ",N="↔ ",j="↕ ",R="*",z="",F="",B="",H="",U="",V="",Y="",W="",G="",X="",q="",Z="",Q="",K="",J="",$="",ee="",te="⌄",ne="〈",re="〉",ie="⌃",oe="",ae="",se="",le="",ue="",ce="",de="",he="",fe="",pe="",ge="",ve="",me="",ye="",be="",Ce="",Ae="",_e="",Ie="☁",xe="",we="",Me="",Se="",Te="",Ee="",Oe="",ke="",Le="",De="",Pe="",Ne="",je="",Re="",ze="",Fe="",Be="✗",He="",Ue="",Ve="",Ye="",We="",Ge="",Xe="",qe="",Ze="",Qe="",Ke="Δ",Je="",$e="",et="",tt="",nt="",rt="",it="",ot="",at="",st="",lt="$",ut="•",ct="",dt="",ht="",ft="",pt="",gt="",vt="",mt="",yt="",bt="",Ct="",At="",_t="",It="✎",xt="⏏",wt="",Mt="✉",St="",Tt="",Et="",Ot="€",kt="",Lt="",Dt="",Pt="",Nt="",jt="",Rt="",zt="",Ft="",Bt="",Ht="",Ut="",Vt="",Yt="",Wt="",Gt="",Xt="",qt="⚑",Zt="",Qt="",Kt="",Jt="",$t="",en="",tn="",nn="",rn="",on="",an="",sn="",ln="",un="",cn="",dn="",hn="",fn="",pn="",gn="",vn="",mn="",yn="",bn="",Cn="",An="",_n="",In="",xn="",wn="",Mn="",Sn="",Tn="",En="",On="",kn="",Ln="",Dn="",Pn="",Nn="",jn="",Rn="",zn="",Fn="",Bn="",Hn="",Un="",Vn="",Yn="",Wn="",Gn="",Xn="",qn="",Zn="♥",Qn="",Kn="",Jn="",$n="?",er="",tr="",nr="",rr="⌂",ir="",or="",ar="",sr="",lr="",ur="",cr="",dr="",hr="",fr="",pr="",gr="",vr="",mr="ℹ",yr="",br="",Cr="",Ar="",_r="",Ir="",xr="",wr="",Mr="",Sr="",Tr="",Er="",Or="",kr="",Lr="",Dr="",Pr="",Nr="",jr="",Rr="",zr="",Fr="",Br="",Hr="",Ur="",Vr="",Yr="",Wr="",Gr="",Xr="",qr="",Zr="",Qr="",Kr="",Jr="",$r="",ei="",ti="",ni="",ri="",ii="",oi="",ai="☰",si="",li="",ui="",ci="",di="",hi="",fi="",pi="",gi="",vi="",mi="",yi="",bi="",Ci="",Ai="",_i="",Ii="",xi="",wi="",Mi="−",Si="",Ti="",Ei="",Oi="",ki="",Li="",Di="",Pi="",Ni="",ji="",Ri="",zi="",Fi="",Bi="",Hi="",Ui="",Vi="",Yi="",Wi="",Gi="",Xi="",qi="",Zi="",Qi="",Ki="",Ji="",$i="",eo="",to="",no="",ro="",io="",oo="",ao="",so="",lo="",uo="",co="",ho="",fo="",po="☎",go="",vo="",mo="",yo="",bo="",Co="+",Ao="",_o="",Io="",xo="",wo="",Mo="⎙",So="",To="",Eo="",Oo="",ko="",Lo="",Do="",Po="",No="",jo="",Ro="",zo="",Fo="",Bo="",Ho="",Uo="",Vo="",Yo="",Wo="",Go="",Xo="",qo="",Zo="",Qo="",Ko="",Jo="",$o="",ea="",ta="",na="",ra="",ia="",oa="",aa="⦿",sa="",la="",ua="",ca="",da="",ha="",fa="",pa="",ga="",va="",ma="",ya="",ba="",Ca="",Aa="",_a="",Ia="",xa="",wa="",Ma="",Sa="",Ta="",Ea="",Oa="",ka="",La="",Da="",Pa="",Na="",ja="",Ra="",za="",Fa="★",Ba="☆",Ha="",Ua="",Va="",Ya="",Wa="",Ga="",Xa="",qa="",Za="",Qa="",Ka="",Ja="",$a="",es="",ts="",ns="",rs="",is="",os="",as="",ss="",ls="",us="",cs="",ds="",hs="",fs="✓",ps="",gs="⏲",vs="",ms="",ys="",bs="",Cs="",As="",_s="",Is="",xs="",ws="",Ms="",Ss="",Ts="",Es="",Os="",ks="",Ls="⎁",Ds="⎌",Ps="",Ns="",js="",Rs="",zs="",Fs="",Bs="",Hs="",Us="",Vs="",Ys="",Ws="",Gs="",Xs="",qs="",Zs="",Qs="",Ks="",Js="",$s="",el="",tl="",nl="",rl="",il="",ol="",al="",sl="add",ll="add-column-left",ul="add-column-right",cl="add-row-bottom",dl="add-row-top",hl="add-to-artifact",fl="add-to-folder",pl="airplane",gl="align-center",vl="align-justify",ml="align-left",yl="align-right",bl="alignment-bottom",Cl="alignment-horizontal-center",Al="alignment-left",_l="alignment-right",Il="alignment-top",xl="alignment-vertical-center",wl="annotation",Ml="app-header",Sl="application",Tl="applications",El="archive",Ol="array",kl="array-boolean",Ll="array-date",Dl="array-numeric",Pl="array-string",Nl="array-timestamp",jl="arrow-bottom-left",Rl="arrow-bottom-right",zl="arrow-down",Fl="arrow-left",Bl="arrow-right",Hl="arrow-top-left",Ul="arrow-top-right",Vl="arrow-up",Yl="arrows-horizontal",Wl="arrows-vertical",Gl="asterisk",Xl="automatic-updates",ql="backlink",Zl="badge",Ql="ban-circle",Kl="bank-account",Jl="barcode",$l="blank",eu="blocked-person",tu="bold",nu="book",ru="bookmark",iu="box",ou="briefcase",au="bring-data",su="build",lu="calculator",uu="calendar",cu="camera",du="caret-down",hu="caret-left",fu="caret-right",pu="caret-up",gu="cell-tower",vu="changes",mu="chart",yu="chat",bu="chevron-backward",Cu="chevron-down",Au="chevron-forward",_u="chevron-left",Iu="chevron-right",xu="chevron-up",wu="circle",Mu="circle-arrow-down",Su="circle-arrow-left",Tu="circle-arrow-right",Eu="circle-arrow-up",Ou="citation",ku="clean",Lu="clipboard",Du="cloud",Pu="cloud-download",Nu="cloud-upload",ju="code",Ru="code-block",zu="cog",Fu="collapse-all",Bu="column-layout",Hu="comment",Uu="comparison",Vu="compass",Yu="compressed",Wu="confirm",Gu="console",Xu="contrast",qu="control",Zu="credit-card",Qu="cross",Ku="crown",Ju="cube",$u="cube-add",ec="cube-remove",tc="curved-range-chart",nc="cut",rc="cycle",ic="dashboard",oc="data-connection",ac="data-lineage",sc="database",lc="delete",uc="delta",cc="derive-column",dc="desktop",hc="diagnosis",fc="diagram-tree",pc="direction-left",gc="direction-right",vc="disable",mc="document",yc="document-open",bc="document-share",Cc="dollar",Ac="dot",_c="double-caret-horizontal",Ic="double-caret-vertical",xc="double-chevron-down",wc="double-chevron-left",Mc="double-chevron-right",Sc="double-chevron-up",Tc="doughnut-chart",Ec="download",Oc="drag-handle-horizontal",kc="drag-handle-vertical",Lc="draw",Dc="drawer-left",Pc="drawer-left-filled",Nc="drawer-right",jc="drawer-right-filled",Rc="drive-time",zc="duplicate",Fc="edit",Bc="eject",Hc="emoji",Uc="endorsed",Vc="envelope",Yc="equals",Wc="eraser",Gc="error",Xc="euro",qc="exchange",Zc="exclude-row",Qc="expand-all",Kc="export",Jc="eye-off",$c="eye-on",ed="eye-open",td="fast-backward",nd="fast-forward",rd="feed",id="feed-subscribed",od="film",ad="filter",sd="filter-keep",ld="filter-list",ud="filter-open",cd="filter-remove",dd="flag",hd="flame",fd="flash",pd="floppy-disk",gd="flow-branch",vd="flow-end",md="flow-linear",yd="flow-review",bd="flow-review-branch",Cd="flows",Ad="folder-close",_d="folder-new",Id="folder-open",xd="folder-shared",wd="folder-shared-open",Md="follower",Sd="following",Td="font",Ed="fork",Od="form",kd="full-circle",Ld="full-stacked-chart",Dd="fullscreen",Pd="function",Nd="gantt-chart",jd="geofence",Rd="geolocation",zd="geosearch",Fd="git-branch",Bd="git-commit",Hd="git-merge",Ud="git-new-branch",Vd="git-pull",Yd="git-push",Wd="git-repo",Gd="glass",Xd="globe",qd="globe-network",Zd="graph",Qd="graph-remove",Kd="greater-than",Jd="greater-than-or-equal-to",$d="grid",eh="grid-view",th="group-objects",nh="grouped-bar-chart",rh="hand",ih="hand-down",oh="hand-left",ah="hand-right",sh="hand-up",lh="hat",uh="header",ch="header-one",dh="header-two",hh="headset",fh="heart",ph="heart-broken",gh="heat-grid",vh="heatmap",mh="help",yh="helper-management",bh="high-priority",Ch="highlight",Ah="history",_h="home",Ih="horizontal-bar-chart",xh="horizontal-bar-chart-asc",wh="horizontal-bar-chart-desc",Mh="horizontal-distribution",Sh="id-number",Th="image-rotate-left",Eh="image-rotate-right",Oh="import",kh="inbox",Lh="inbox-filtered",Dh="inbox-geo",Ph="inbox-search",Nh="inbox-update",jh="info-sign",Rh="inheritance",zh="inherited-group",Fh="inner-join",Bh="insert",Hh="intersection",Uh="ip-address",Vh="issue",Yh="issue-closed",Wh="issue-new",Gh="italic",Xh="join-table",qh="key",Zh="key-backspace",Qh="key-command",Kh="key-control",Jh="key-delete",$h="key-enter",ef="key-escape",tf="key-option",nf="key-shift",rf="key-tab",of="known-vehicle",af="lab-test",sf="label",lf="layer",uf="layer-outline",cf="layers",df="layout",hf="layout-auto",ff="layout-balloon",pf="layout-circle",gf="layout-grid",vf="layout-group-by",mf="layout-hierarchy",yf="layout-linear",bf="layout-skew-grid",Cf="layout-sorted-clusters",Af="learning",_f="left-join",If="less-than",xf="less-than-or-equal-to",wf="lifesaver",Mf="lightbulb",Sf="link",Tf="list",Ef="list-columns",Of="list-detail-view",kf="locate",Lf="lock",Df="log-in",Pf="log-out",Nf="manual",jf="manually-entered-data",Rf="map",zf="map-create",Ff="map-marker",Bf="maximize",Hf="media",Uf="menu",Vf="menu-closed",Yf="menu-open",Wf="merge-columns",Gf="merge-links",Xf="minimize",qf="minus",Zf="mobile-phone",Qf="mobile-video",Kf="modal",Jf="modal-filled",$f="moon",ep="more",tp="mountain",np="move",rp="mugshot",ip="multi-select",op="music",ap="new-drawing",sp="new-grid-item",lp="new-layer",up="new-layers",cp="new-link",dp="new-object",hp="new-person",fp="new-prescription",pp="new-text-box",gp="ninja",vp="not-equal-to",mp="notifications",yp="notifications-snooze",bp="notifications-updated",Cp="numbered-list",Ap="numerical",_p="office",Ip="offline",xp="oil-field",wp="one-column",Mp="outdated",Sp="page-layout",Tp="panel-stats",Ep="panel-table",Op="paperclip",kp="paragraph",Lp="path",Dp="path-search",Pp="pause",Np="people",jp="percentage",Rp="person",zp="phone",Fp="pie-chart",Bp="pin",Hp="pivot",Up="pivot-table",Vp="play",Yp="plus",Wp="polygon-filter",Gp="power",Xp="predictive-analysis",qp="prescription",Zp="presentation",Qp="print",Kp="projects",Jp="properties",$p="property",eg="publish-function",tg="pulse",ng="random",rg="record",ig="redo",og="refresh",ag="regression-chart",sg="remove",lg="remove-column",ug="remove-column-left",cg="remove-column-right",dg="remove-row-bottom",hg="remove-row-top",fg="repeat",pg="reset",gg="resolve",vg="rig",mg="right-join",yg="ring",bg="rotate-document",Cg="rotate-page",Ag="route",_g="satellite",Ig="saved",xg="scatter-plot",wg="search",Mg="search-around",Sg="search-template",Tg="search-text",Eg="segmented-control",Og="select",kg="selection",Lg="send-message",Dg="send-to",Pg="send-to-graph",Ng="send-to-map",jg="series-add",Rg="series-configuration",zg="series-derived",Fg="series-filtered",Bg="series-search",Hg="settings",Ug="share",Vg="shield",Yg="shop",Wg="shopping-cart",Gg="signal-search",Xg="sim-card",qg="slash",Zg="small-cross",Qg="small-minus",Kg="small-plus",Jg="small-tick",$g="snowflake",ev="social-media",tv="sort",nv="sort-alphabetical",rv="sort-alphabetical-desc",iv="sort-asc",ov="sort-desc",av="sort-numerical",sv="sort-numerical-desc",lv="split-columns",uv="square",cv="stacked-chart",dv="star",hv="star-empty",fv="step-backward",pv="step-chart",gv="step-forward",vv="stop",mv="stopwatch",yv="strikethrough",bv="style",Cv="swap-horizontal",Av="swap-vertical",_v="switch",Iv="symbol-circle",xv="symbol-cross",wv="symbol-diamond",Mv="symbol-square",Sv="symbol-triangle-down",Tv="symbol-triangle-up",Ev="tag",Ov="take-action",kv="taxi",Lv="text-highlight",Dv="th",Pv="th-derived",Nv="th-disconnect",jv="th-filtered",Rv="th-list",zv="thumbs-down",Fv="thumbs-up",Bv="tick",Hv="tick-circle",Uv="time",Vv="timeline-area-chart",Yv="timeline-bar-chart",Wv="timeline-events",Gv="timeline-line-chart",Xv="tint",qv="torch",Zv="tractor",Qv="train",Kv="translate",Jv="trash",$v="tree",em="trending-down",tm="trending-up",nm="truck",rm="two-columns",im="unarchive",om="underline",am="undo",sm="ungroup-objects",lm="unknown-vehicle",um="unlock",cm="unpin",dm="unresolve",hm="updated",fm="upload",pm="user",gm="variable",vm="vertical-bar-chart-asc",mm="vertical-bar-chart-desc",ym="vertical-distribution",bm="video",Cm="virus",Am="volume-down",_m="volume-off",Im="volume-up",xm="walk",wm="warning-sign",Mm="waterfall-chart",Sm="widget",Tm="widget-button",Em="widget-footer",Om="widget-header",km="wrench",Lm="zoom-in",Dm="zoom-out",Pm="zoom-to-fit",Nm=n(505);n.d(t,"IconContents",(function(){return r})),n.d(t,"IconNames",(function(){return i})),n.d(t,"IconSvgPaths16",(function(){return Nm.a})),n.d(t,"IconSvgPaths20",(function(){return Nm.b}))},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(4)),s=n(57),l=Object(s.a)(o.createElement("path",{d:"M12 2C6.47 2 2 6.47 2 12s4.47 10 10 10 10-4.47 10-10S17.53 2 12 2zm5 13.59L15.59 17 12 13.41 8.41 17 7 15.59 10.59 12 7 8.41 8.41 7 12 10.59 15.59 7 17 8.41 13.41 12 17 15.59z"}),"Cancel"),u=n(7),c=n(26),d=n(37),h=n(15),f=n(231);function p(e){return"Backspace"===e.key||"Delete"===e.key}var g=o.forwardRef((function(e,t){var n=e.avatar,s=e.classes,u=e.className,c=e.clickable,g=e.color,v=void 0===g?"default":g,m=e.component,y=e.deleteIcon,b=e.disabled,C=void 0!==b&&b,A=e.icon,_=e.label,I=e.onClick,x=e.onDelete,w=e.onKeyDown,M=e.onKeyUp,S=e.size,T=void 0===S?"medium":S,E=e.variant,O=void 0===E?"default":E,k=Object(i.a)(e,["avatar","classes","className","clickable","color","component","deleteIcon","disabled","icon","label","onClick","onDelete","onKeyDown","onKeyUp","size","variant"]),L=o.useRef(null),D=Object(d.a)(L,t),P=function(e){e.stopPropagation(),x&&x(e)},N=!(!1===c||!I)||c,j="small"===T,R=m||(N?f.a:"div"),z=R===f.a?{component:"div"}:{},F=null;if(x){var B=Object(a.a)("default"!==v&&("default"===O?s["deleteIconColor".concat(Object(h.a)(v))]:s["deleteIconOutlinedColor".concat(Object(h.a)(v))]),j&&s.deleteIconSmall);F=y&&o.isValidElement(y)?o.cloneElement(y,{className:Object(a.a)(y.props.className,s.deleteIcon,B),onClick:P}):o.createElement(l,{className:Object(a.a)(s.deleteIcon,B),onClick:P})}var H=null;n&&o.isValidElement(n)&&(H=o.cloneElement(n,{className:Object(a.a)(s.avatar,n.props.className,j&&s.avatarSmall,"default"!==v&&s["avatarColor".concat(Object(h.a)(v))])}));var U=null;return A&&o.isValidElement(A)&&(U=o.cloneElement(A,{className:Object(a.a)(s.icon,A.props.className,j&&s.iconSmall,"default"!==v&&s["iconColor".concat(Object(h.a)(v))])})),o.createElement(R,Object(r.a)({role:N||x?"button":void 0,className:Object(a.a)(s.root,u,"default"!==v&&[s["color".concat(Object(h.a)(v))],N&&s["clickableColor".concat(Object(h.a)(v))],x&&s["deletableColor".concat(Object(h.a)(v))]],"default"!==O&&[s.outlined,{primary:s.outlinedPrimary,secondary:s.outlinedSecondary}[v]],C&&s.disabled,j&&s.sizeSmall,N&&s.clickable,x&&s.deletable),"aria-disabled":!!C||void 0,tabIndex:N||x?0:void 0,onClick:I,onKeyDown:function(e){e.currentTarget===e.target&&p(e)&&e.preventDefault(),w&&w(e)},onKeyUp:function(e){e.currentTarget===e.target&&(x&&p(e)?x(e):"Escape"===e.key&&L.current&&L.current.blur()),M&&M(e)},ref:D},z,k),H||U,o.createElement("span",{className:Object(a.a)(s.label,j&&s.labelSmall)},_),F)}));t.a=Object(u.a)((function(e){var t="light"===e.palette.type?e.palette.grey[300]:e.palette.grey[700],n=Object(c.d)(e.palette.text.primary,.26);return{root:{fontFamily:e.typography.fontFamily,fontSize:e.typography.pxToRem(13),display:"inline-flex",alignItems:"center",justifyContent:"center",height:32,color:e.palette.getContrastText(t),backgroundColor:t,borderRadius:16,whiteSpace:"nowrap",transition:e.transitions.create(["background-color","box-shadow"]),cursor:"default",outline:0,textDecoration:"none",border:"none",padding:0,verticalAlign:"middle",boxSizing:"border-box","&$disabled":{opacity:.5,pointerEvents:"none"},"& $avatar":{marginLeft:5,marginRight:-6,width:24,height:24,color:"light"===e.palette.type?e.palette.grey[700]:e.palette.grey[300],fontSize:e.typography.pxToRem(12)},"& $avatarColorPrimary":{color:e.palette.primary.contrastText,backgroundColor:e.palette.primary.dark},"& $avatarColorSecondary":{color:e.palette.secondary.contrastText,backgroundColor:e.palette.secondary.dark},"& $avatarSmall":{marginLeft:4,marginRight:-4,width:18,height:18,fontSize:e.typography.pxToRem(10)}},sizeSmall:{height:24},colorPrimary:{backgroundColor:e.palette.primary.main,color:e.palette.primary.contrastText},colorSecondary:{backgroundColor:e.palette.secondary.main,color:e.palette.secondary.contrastText},disabled:{},clickable:{userSelect:"none",WebkitTapHighlightColor:"transparent",cursor:"pointer","&:hover, &:focus":{backgroundColor:Object(c.c)(t,.08)},"&:active":{boxShadow:e.shadows[1]}},clickableColorPrimary:{"&:hover, &:focus":{backgroundColor:Object(c.c)(e.palette.primary.main,.08)}},clickableColorSecondary:{"&:hover, &:focus":{backgroundColor:Object(c.c)(e.palette.secondary.main,.08)}},deletable:{"&:focus":{backgroundColor:Object(c.c)(t,.08)}},deletableColorPrimary:{"&:focus":{backgroundColor:Object(c.c)(e.palette.primary.main,.2)}},deletableColorSecondary:{"&:focus":{backgroundColor:Object(c.c)(e.palette.secondary.main,.2)}},outlined:{backgroundColor:"transparent",border:"1px solid ".concat("light"===e.palette.type?"rgba(0, 0, 0, 0.23)":"rgba(255, 255, 255, 0.23)"),"$clickable&:hover, $clickable&:focus, $deletable&:focus":{backgroundColor:Object(c.d)(e.palette.text.primary,e.palette.action.hoverOpacity)},"& $avatar":{marginLeft:4},"& $avatarSmall":{marginLeft:2},"& $icon":{marginLeft:4},"& $iconSmall":{marginLeft:2},"& $deleteIcon":{marginRight:5},"& $deleteIconSmall":{marginRight:3}},outlinedPrimary:{color:e.palette.primary.main,border:"1px solid ".concat(e.palette.primary.main),"$clickable&:hover, $clickable&:focus, $deletable&:focus":{backgroundColor:Object(c.d)(e.palette.primary.main,e.palette.action.hoverOpacity)}},outlinedSecondary:{color:e.palette.secondary.main,border:"1px solid ".concat(e.palette.secondary.main),"$clickable&:hover, $clickable&:focus, $deletable&:focus":{backgroundColor:Object(c.d)(e.palette.secondary.main,e.palette.action.hoverOpacity)}},avatar:{},avatarSmall:{},avatarColorPrimary:{},avatarColorSecondary:{},icon:{color:"light"===e.palette.type?e.palette.grey[700]:e.palette.grey[300],marginLeft:5,marginRight:-6},iconSmall:{width:18,height:18,marginLeft:4,marginRight:-4},iconColorPrimary:{color:"inherit"},iconColorSecondary:{color:"inherit"},label:{overflow:"hidden",textOverflow:"ellipsis",paddingLeft:12,paddingRight:12,whiteSpace:"nowrap"},labelSmall:{paddingLeft:8,paddingRight:8},deleteIcon:{WebkitTapHighlightColor:"transparent",color:n,height:22,width:22,cursor:"pointer",margin:"0 5px 0 -6px","&:hover":{color:Object(c.d)(n,.4)}},deleteIconSmall:{height:16,width:16,marginRight:4,marginLeft:-4},deleteIconColorPrimary:{color:Object(c.d)(e.palette.primary.contrastText,.7),"&:hover, &:active":{color:e.palette.primary.contrastText}},deleteIconColorSecondary:{color:Object(c.d)(e.palette.secondary.contrastText,.7),"&:hover, &:active":{color:e.palette.secondary.contrastText}},deleteIconOutlinedColorPrimary:{color:Object(c.d)(e.palette.primary.main,.7),"&:hover, &:active":{color:e.palette.primary.main}},deleteIconOutlinedColorSecondary:{color:Object(c.d)(e.palette.secondary.main,.7),"&:hover, &:active":{color:e.palette.secondary.main}}}}),{name:"MuiChip"})(g)},function(e,t,n){"use strict";var r=n(74),i=n(49),o=n(5),a=n(1),s=n(0),l=(n(3),n(4)),u=n(7),c=n(52),d=n(26),h=n(166),f=n(59),p=n(53),g=n(37),v=n(15),m=n(100);var y=Object(u.a)((function(e){return{thumb:{"&$open":{"& $offset":{transform:"scale(1) translateY(-10px)"}}},open:{},offset:Object(a.a)({zIndex:1},e.typography.body2,{fontSize:e.typography.pxToRem(12),lineHeight:1.2,transition:e.transitions.create(["transform"],{duration:e.transitions.duration.shortest}),top:-34,transformOrigin:"bottom center",transform:"scale(0)",position:"absolute"}),circle:{display:"flex",alignItems:"center",justifyContent:"center",width:32,height:32,borderRadius:"50% 50% 50% 0",backgroundColor:"currentColor",transform:"rotate(-45deg)"},label:{color:e.palette.primary.contrastText,transform:"rotate(45deg)"}}}),{name:"PrivateValueLabel"})((function(e){var t=e.children,n=e.classes,r=e.className,i=e.open,o=e.value,a=e.valueLabelDisplay;return"off"===a?t:s.cloneElement(t,{className:Object(l.a)(t.props.className,(i||"on"===a)&&n.open,n.thumb)},s.createElement("span",{className:Object(l.a)(n.offset,r)},s.createElement("span",{className:n.circle},s.createElement("span",{className:n.label},o))))}));function b(e,t){return e-t}function C(e,t,n){return Math.min(Math.max(t,e),n)}function A(e,t){return e.reduce((function(e,n,r){var i=Math.abs(t-n);return null===e||i<e.distance||i===e.distance?{distance:i,index:r}:e}),null).index}function _(e,t){if(void 0!==t.current&&e.changedTouches){for(var n=0;n<e.changedTouches.length;n+=1){var r=e.changedTouches[n];if(r.identifier===t.current)return{x:r.clientX,y:r.clientY}}return!1}return{x:e.clientX,y:e.clientY}}function I(e,t,n){return 100*(e-t)/(n-t)}function x(e,t,n){var r=Math.round((e-n)/t)*t+n;return Number(r.toFixed(function(e){if(Math.abs(e)<1){var t=e.toExponential().split("e-"),n=t[0].split(".")[1];return(n?n.length:0)+parseInt(t[1],10)}var r=e.toString().split(".")[1];return r?r.length:0}(t)))}function w(e){var t=e.values,n=e.source,r=e.newValue,i=e.index;if(t[i]===r)return n;var o=t.slice();return o[i]=r,o}function M(e){var t=e.sliderRef,n=e.activeIndex,r=e.setActive;t.current.contains(document.activeElement)&&Number(document.activeElement.getAttribute("data-index"))===n||t.current.querySelector('[role="slider"][data-index="'.concat(n,'"]')).focus(),r&&r(n)}var S={horizontal:{offset:function(e){return{left:"".concat(e,"%")}},leap:function(e){return{width:"".concat(e,"%")}}},"horizontal-reverse":{offset:function(e){return{right:"".concat(e,"%")}},leap:function(e){return{width:"".concat(e,"%")}}},vertical:{offset:function(e){return{bottom:"".concat(e,"%")}},leap:function(e){return{height:"".concat(e,"%")}}}},T=function(e){return e},E=s.forwardRef((function(e,t){var n=e["aria-label"],u=e["aria-labelledby"],d=e["aria-valuetext"],E=e.classes,O=e.className,k=e.color,L=void 0===k?"primary":k,D=e.component,P=void 0===D?"span":D,N=e.defaultValue,j=e.disabled,R=void 0!==j&&j,z=e.getAriaLabel,F=e.getAriaValueText,B=e.marks,H=void 0!==B&&B,U=e.max,V=void 0===U?100:U,Y=e.min,W=void 0===Y?0:Y,G=e.name,X=e.onChange,q=e.onChangeCommitted,Z=e.onMouseDown,Q=e.orientation,K=void 0===Q?"horizontal":Q,J=e.scale,$=void 0===J?T:J,ee=e.step,te=void 0===ee?1:ee,ne=e.ThumbComponent,re=void 0===ne?"span":ne,ie=e.track,oe=void 0===ie?"normal":ie,ae=e.value,se=e.ValueLabelComponent,le=void 0===se?y:se,ue=e.valueLabelDisplay,ce=void 0===ue?"off":ue,de=e.valueLabelFormat,he=void 0===de?T:de,fe=Object(o.a)(e,["aria-label","aria-labelledby","aria-valuetext","classes","className","color","component","defaultValue","disabled","getAriaLabel","getAriaValueText","marks","max","min","name","onChange","onChangeCommitted","onMouseDown","orientation","scale","step","ThumbComponent","track","value","ValueLabelComponent","valueLabelDisplay","valueLabelFormat"]),pe=Object(c.a)(),ge=s.useRef(),ve=s.useState(-1),me=ve[0],ye=ve[1],be=s.useState(-1),Ce=be[0],Ae=be[1],_e=Object(m.a)({controlled:ae,default:N,name:"Slider"}),Ie=Object(i.a)(_e,2),xe=Ie[0],we=Ie[1],Me=Array.isArray(xe),Se=Me?xe.slice().sort(b):[xe];Se=Se.map((function(e){return C(e,W,V)}));var Te=!0===H&&null!==te?Object(r.a)(Array(Math.floor((V-W)/te)+1)).map((function(e,t){return{value:W+te*t}})):H||[],Ee=Object(h.a)(),Oe=Ee.isFocusVisible,ke=Ee.onBlurVisible,Le=Ee.ref,De=s.useState(-1),Pe=De[0],Ne=De[1],je=s.useRef(),Re=Object(g.a)(Le,je),ze=Object(g.a)(t,Re),Fe=Object(p.a)((function(e){var t=Number(e.currentTarget.getAttribute("data-index"));Oe(e)&&Ne(t),Ae(t)})),Be=Object(p.a)((function(){-1!==Pe&&(Ne(-1),ke()),Ae(-1)})),He=Object(p.a)((function(e){var t=Number(e.currentTarget.getAttribute("data-index"));Ae(t)})),Ue=Object(p.a)((function(){Ae(-1)})),Ve="rtl"===pe.direction,Ye=Object(p.a)((function(e){var t,n=Number(e.currentTarget.getAttribute("data-index")),r=Se[n],i=(V-W)/10,o=Te.map((function(e){return e.value})),a=o.indexOf(r),s=Ve?"ArrowLeft":"ArrowRight",l=Ve?"ArrowRight":"ArrowLeft";switch(e.key){case"Home":t=W;break;case"End":t=V;break;case"PageUp":te&&(t=r+i);break;case"PageDown":te&&(t=r-i);break;case s:case"ArrowUp":t=te?r+te:o[a+1]||o[o.length-1];break;case l:case"ArrowDown":t=te?r-te:o[a-1]||o[0];break;default:return}if(e.preventDefault(),te&&(t=x(t,te,W)),t=C(t,W,V),Me){var u=t;t=w({values:Se,source:xe,newValue:t,index:n}).sort(b),M({sliderRef:je,activeIndex:t.indexOf(u)})}we(t),Ne(n),X&&X(e,t),q&&q(e,t)})),We=s.useRef(),Ge=K;Ve&&"vertical"!==K&&(Ge+="-reverse");var Xe=function(e){var t,n,r=e.finger,i=e.move,o=void 0!==i&&i,a=e.values,s=e.source,l=je.current.getBoundingClientRect(),u=l.width,c=l.height,d=l.bottom,h=l.left;if(t=0===Ge.indexOf("vertical")?(d-r.y)/c:(r.x-h)/u,-1!==Ge.indexOf("-reverse")&&(t=1-t),n=function(e,t,n){return(n-t)*e+t}(t,W,V),te)n=x(n,te,W);else{var f=Te.map((function(e){return e.value}));n=f[A(f,n)]}n=C(n,W,V);var p=0;if(Me){var g=n;p=(n=w({values:a,source:s,newValue:n,index:p=o?We.current:A(a,n)}).sort(b)).indexOf(g),We.current=p}return{newValue:n,activeIndex:p}},qe=Object(p.a)((function(e){var t=_(e,ge);if(t){var n=Xe({finger:t,move:!0,values:Se,source:xe}),r=n.newValue,i=n.activeIndex;M({sliderRef:je,activeIndex:i,setActive:ye}),we(r),X&&X(e,r)}})),Ze=Object(p.a)((function(e){var t=_(e,ge);if(t){var n=Xe({finger:t,values:Se,source:xe}).newValue;ye(-1),"touchend"===e.type&&Ae(-1),q&&q(e,n),ge.current=void 0;var r=Object(f.a)(je.current);r.removeEventListener("mousemove",qe),r.removeEventListener("mouseup",Ze),r.removeEventListener("touchmove",qe),r.removeEventListener("touchend",Ze)}})),Qe=Object(p.a)((function(e){e.preventDefault();var t=e.changedTouches[0];null!=t&&(ge.current=t.identifier);var n=_(e,ge),r=Xe({finger:n,values:Se,source:xe}),i=r.newValue,o=r.activeIndex;M({sliderRef:je,activeIndex:o,setActive:ye}),we(i),X&&X(e,i);var a=Object(f.a)(je.current);a.addEventListener("touchmove",qe),a.addEventListener("touchend",Ze)}));s.useEffect((function(){var e=je.current;e.addEventListener("touchstart",Qe);var t=Object(f.a)(e);return function(){e.removeEventListener("touchstart",Qe),t.removeEventListener("mousemove",qe),t.removeEventListener("mouseup",Ze),t.removeEventListener("touchmove",qe),t.removeEventListener("touchend",Ze)}}),[Ze,qe,Qe]);var Ke=Object(p.a)((function(e){Z&&Z(e),e.preventDefault();var t=_(e,ge),n=Xe({finger:t,values:Se,source:xe}),r=n.newValue,i=n.activeIndex;M({sliderRef:je,activeIndex:i,setActive:ye}),we(r),X&&X(e,r);var o=Object(f.a)(je.current);o.addEventListener("mousemove",qe),o.addEventListener("mouseup",Ze)})),Je=I(Me?Se[0]:W,W,V),$e=I(Se[Se.length-1],W,V)-Je,et=Object(a.a)({},S[Ge].offset(Je),S[Ge].leap($e));return s.createElement(P,Object(a.a)({ref:ze,className:Object(l.a)(E.root,E["color".concat(Object(v.a)(L))],O,R&&E.disabled,Te.length>0&&Te.some((function(e){return e.label}))&&E.marked,!1===oe&&E.trackFalse,"vertical"===K&&E.vertical,"inverted"===oe&&E.trackInverted),onMouseDown:Ke},fe),s.createElement("span",{className:E.rail}),s.createElement("span",{className:E.track,style:et}),s.createElement("input",{value:Se.join(","),name:G,type:"hidden"}),Te.map((function(e,t){var n,r=I(e.value,W,V),i=S[Ge].offset(r);return n=!1===oe?-1!==Se.indexOf(e.value):"normal"===oe&&(Me?e.value>=Se[0]&&e.value<=Se[Se.length-1]:e.value<=Se[0])||"inverted"===oe&&(Me?e.value<=Se[0]||e.value>=Se[Se.length-1]:e.value>=Se[0]),s.createElement(s.Fragment,{key:e.value},s.createElement("span",{style:i,"data-index":t,className:Object(l.a)(E.mark,n&&E.markActive)}),null!=e.label?s.createElement("span",{"aria-hidden":!0,"data-index":t,style:i,className:Object(l.a)(E.markLabel,n&&E.markLabelActive)},e.label):null)})),Se.map((function(e,t){var r=I(e,W,V),i=S[Ge].offset(r);return s.createElement(le,{key:t,valueLabelFormat:he,valueLabelDisplay:ce,className:E.valueLabel,value:"function"==typeof he?he($(e),t):he,index:t,open:Ce===t||me===t||"on"===ce,disabled:R},s.createElement(re,{className:Object(l.a)(E.thumb,E["thumbColor".concat(Object(v.a)(L))],me===t&&E.active,R&&E.disabled,Pe===t&&E.focusVisible),tabIndex:R?null:0,role:"slider",style:i,"data-index":t,"aria-label":z?z(t):n,"aria-labelledby":u,"aria-orientation":K,"aria-valuemax":$(V),"aria-valuemin":$(W),"aria-valuenow":$(e),"aria-valuetext":F?F($(e),t):d,onKeyDown:Ye,onFocus:Fe,onBlur:Be,onMouseOver:He,onMouseLeave:Ue}))})))}));t.a=Object(u.a)((function(e){return{root:{height:2,width:"100%",boxSizing:"content-box",padding:"13px 0",display:"inline-block",position:"relative",cursor:"pointer",touchAction:"none",color:e.palette.primary.main,WebkitTapHighlightColor:"transparent","&$disabled":{pointerEvents:"none",cursor:"default",color:e.palette.grey[400]},"&$vertical":{width:2,height:"100%",padding:"0 13px"},"@media (pointer: coarse)":{padding:"20px 0","&$vertical":{padding:"0 20px"}},"@media print":{colorAdjust:"exact"}},colorPrimary:{},colorSecondary:{color:e.palette.secondary.main},marked:{marginBottom:20,"&$vertical":{marginBottom:"auto",marginRight:20}},vertical:{},disabled:{},rail:{display:"block",position:"absolute",width:"100%",height:2,borderRadius:1,backgroundColor:"currentColor",opacity:.38,"$vertical &":{height:"100%",width:2}},track:{display:"block",position:"absolute",height:2,borderRadius:1,backgroundColor:"currentColor","$vertical &":{width:2}},trackFalse:{"& $track":{display:"none"}},trackInverted:{"& $track":{backgroundColor:"light"===e.palette.type?Object(d.i)(e.palette.primary.main,.62):Object(d.a)(e.palette.primary.main,.5)},"& $rail":{opacity:1}},thumb:{position:"absolute",width:12,height:12,marginLeft:-6,marginTop:-5,boxSizing:"border-box",borderRadius:"50%",outline:0,backgroundColor:"currentColor",display:"flex",alignItems:"center",justifyContent:"center",transition:e.transitions.create(["box-shadow"],{duration:e.transitions.duration.shortest}),"&::after":{position:"absolute",content:'""',borderRadius:"50%",left:-15,top:-15,right:-15,bottom:-15},"&$focusVisible,&:hover":{boxShadow:"0px 0px 0px 8px ".concat(Object(d.d)(e.palette.primary.main,.16)),"@media (hover: none)":{boxShadow:"none"}},"&$active":{boxShadow:"0px 0px 0px 14px ".concat(Object(d.d)(e.palette.primary.main,.16))},"&$disabled":{width:8,height:8,marginLeft:-4,marginTop:-3,"&:hover":{boxShadow:"none"}},"$vertical &":{marginLeft:-5,marginBottom:-6},"$vertical &$disabled":{marginLeft:-3,marginBottom:-4}},thumbColorPrimary:{},thumbColorSecondary:{"&$focusVisible,&:hover":{boxShadow:"0px 0px 0px 8px ".concat(Object(d.d)(e.palette.secondary.main,.16))},"&$active":{boxShadow:"0px 0px 0px 14px ".concat(Object(d.d)(e.palette.secondary.main,.16))}},active:{},focusVisible:{},valueLabel:{left:"calc(-50% - 4px)"},mark:{position:"absolute",width:2,height:2,borderRadius:1,backgroundColor:"currentColor"},markActive:{backgroundColor:e.palette.background.paper,opacity:.8},markLabel:Object(a.a)({},e.typography.body2,{color:e.palette.text.secondary,position:"absolute",top:26,transform:"translateX(-50%)",whiteSpace:"nowrap","$vertical &":{top:"auto",left:26,transform:"translateY(50%)"},"@media (pointer: coarse)":{top:40,"$vertical &":{left:31}}}),markLabelActive:{color:e.palette.text.primary}}}),{name:"MuiSlider"})(E)},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(4)),s=n(7),l=n(1311),u=n(921),c=n(639),d=n(925),h=n(928),f=n(187),p=n(338),g=n(337),v=n(52),m=n(452),y=o.createElement(g.a,null),b=o.createElement(p.a,null),C=o.createElement(p.a,null),A=o.createElement(g.a,null),_=o.forwardRef((function(e,t){var n=e.backIconButtonProps,a=e.count,s=e.nextIconButtonProps,l=e.onChangePage,u=e.page,c=e.rowsPerPage,d=Object(i.a)(e,["backIconButtonProps","count","nextIconButtonProps","onChangePage","page","rowsPerPage"]),h=Object(v.a)();return o.createElement("div",Object(r.a)({ref:t},d),o.createElement(m.a,Object(r.a)({onClick:function(e){l(e,u-1)},disabled:0===u,color:"inherit"},n),"rtl"===h.direction?y:b),o.createElement(m.a,Object(r.a)({onClick:function(e){l(e,u+1)},disabled:-1!==a&&u>=Math.ceil(a/c)-1,color:"inherit"},s),"rtl"===h.direction?C:A))})),I=n(186),x=function(e){var t=e.from,n=e.to,r=e.count;return"".concat(t,"-").concat(n," of ").concat(-1!==r?r:"more than ".concat(n))},w=[10,25,50,100],M=o.forwardRef((function(e,t){var n,s=e.ActionsComponent,p=void 0===s?_:s,g=e.backIconButtonProps,v=e.backIconButtonText,m=void 0===v?"Previous page":v,y=e.classes,b=e.className,C=e.colSpan,A=e.component,M=void 0===A?d.a:A,S=e.count,T=e.labelDisplayedRows,E=void 0===T?x:T,O=e.labelRowsPerPage,k=void 0===O?"Rows per page:":O,L=e.nextIconButtonProps,D=e.nextIconButtonText,P=void 0===D?"Next page":D,N=e.onChangePage,j=e.onChangeRowsPerPage,R=e.page,z=e.rowsPerPage,F=e.rowsPerPageOptions,B=void 0===F?w:F,H=e.SelectProps,U=void 0===H?{}:H,V=Object(i.a)(e,["ActionsComponent","backIconButtonProps","backIconButtonText","classes","className","colSpan","component","count","labelDisplayedRows","labelRowsPerPage","nextIconButtonProps","nextIconButtonText","onChangePage","onChangeRowsPerPage","page","rowsPerPage","rowsPerPageOptions","SelectProps"]);M!==d.a&&"td"!==M||(n=C||1e3);var Y=Object(I.a)(),W=Object(I.a)(),G=U.native?"option":u.a;return o.createElement(M,Object(r.a)({className:Object(a.a)(y.root,b),colSpan:n,ref:t},V),o.createElement(h.a,{className:y.toolbar},o.createElement("div",{className:y.spacer}),B.length>1&&o.createElement(f.a,{color:"inherit",variant:"body2",className:y.caption,id:W},k),B.length>1&&o.createElement(c.a,Object(r.a)({classes:{select:y.select,icon:y.selectIcon},input:o.createElement(l.a,{className:Object(a.a)(y.input,y.selectRoot)}),value:z,onChange:j,id:Y,labelId:W},U),B.map((function(e){return o.createElement(G,{className:y.menuItem,key:e.value?e.value:e,value:e.value?e.value:e},e.label?e.label:e)}))),o.createElement(f.a,{color:"inherit",variant:"body2",className:y.caption},E({from:0===S?0:R*z+1,to:-1!==S?Math.min(S,(R+1)*z):(R+1)*z,count:-1===S?-1:S,page:R})),o.createElement(p,{className:y.actions,backIconButtonProps:Object(r.a)({title:m,"aria-label":m},g),count:S,nextIconButtonProps:Object(r.a)({title:P,"aria-label":P},L),onChangePage:N,page:R,rowsPerPage:z})))}));t.a=Object(s.a)((function(e){return{root:{color:e.palette.text.primary,fontSize:e.typography.pxToRem(14),overflow:"auto","&:last-child":{padding:0}},toolbar:{minHeight:52,paddingRight:2},spacer:{flex:"1 1 100%"},caption:{flexShrink:0},selectRoot:{marginRight:32,marginLeft:8},select:{paddingLeft:8,paddingRight:24,textAlign:"right",textAlignLast:"right"},selectIcon:{},input:{color:"inherit",fontSize:"inherit",flexShrink:0},menuItem:{},actions:{flexShrink:0,marginLeft:20}}}),{name:"MuiTablePagination"})(M)},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(4)),s=n(57),l=Object(s.a)(o.createElement("path",{d:"M20 12l-1.41-1.41L13 16.17V4h-2v12.17l-5.58-5.59L4 12l8 8 8-8z"}),"ArrowDownward"),u=n(7),c=n(231),d=n(15),h=o.forwardRef((function(e,t){var n=e.active,s=void 0!==n&&n,u=e.children,h=e.classes,f=e.className,p=e.direction,g=void 0===p?"asc":p,v=e.hideSortIcon,m=void 0!==v&&v,y=e.IconComponent,b=void 0===y?l:y,C=Object(i.a)(e,["active","children","classes","className","direction","hideSortIcon","IconComponent"]);return o.createElement(c.a,Object(r.a)({className:Object(a.a)(h.root,f,s&&h.active),component:"span",disableRipple:!0,ref:t},C),u,m&&!s?null:o.createElement(b,{className:Object(a.a)(h.icon,h["iconDirection".concat(Object(d.a)(g))])}))}));t.a=Object(u.a)((function(e){return{root:{cursor:"pointer",display:"inline-flex",justifyContent:"flex-start",flexDirection:"inherit",alignItems:"center","&:focus":{color:e.palette.text.secondary},"&:hover":{color:e.palette.text.secondary,"& $icon":{opacity:.5}},"&$active":{color:e.palette.text.primary,"&& $icon":{opacity:1,color:e.palette.text.secondary}}},active:{},icon:{fontSize:18,marginRight:4,marginLeft:4,opacity:0,transition:e.transitions.create(["opacity","transform"],{duration:e.transitions.duration.shorter}),userSelect:"none"},iconDirectionDesc:{transform:"rotate(0deg)"},iconDirectionAsc:{transform:"rotate(180deg)"}}}),{name:"MuiTableSortLabel"})(h)},function(e,t,n){var r=n(491);e.exports=function(e,t){return r(e,t)}},function(e,t,n){"use strict";n.d(t,"a",(function(){return s}));var r,i=n(6),o=function(){function e(e){this._root=new r.LeafNode,this.cmp=e}return Object.defineProperty(e.prototype,"isEmpty",{get:function(){return 0===this._root.size},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"size",{get:function(){return this._root.size},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"first",{get:function(){var e=r.firstLeaf(this._root);return e.size>0?e.items[0]:void 0},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"last",{get:function(){var e=r.lastLeaf(this._root);return e.size>0?e.items[e.size-1]:void 0},enumerable:!0,configurable:!0}),e.prototype.iter=function(){return r.iterItems(this._root)},e.prototype.retro=function(){return r.retroItems(this._root)},e.prototype.slice=function(e,t){return r.sliceItems(this._root,e,t)},e.prototype.retroSlice=function(e,t){return r.retroSliceItems(this._root,e,t)},e.prototype.at=function(e){return r.itemAt(this._root,e)},e.prototype.has=function(e,t){return r.hasItem(this._root,e,t)},e.prototype.indexOf=function(e,t){return r.indexOf(this._root,e,t)},e.prototype.get=function(e,t){return r.getItem(this._root,e,t)},e.prototype.assign=function(e){this.clear(),this.update(e)},e.prototype.insert=function(e){var t=r.insertItem(this._root,e,this.cmp);return this._root=r.maybeSplitRoot(this._root),t},e.prototype.update=function(e){var t=this;Object(i.each)(e,(function(e){t.insert(e)}))},e.prototype.delete=function(e,t){var n=r.deleteItem(this._root,e,t);return this._root=r.maybeExtractRoot(this._root),n},e.prototype.remove=function(e){var t=r.removeItem(this._root,e);return this._root=r.maybeExtractRoot(this._root),t},e.prototype.clear=function(){r.clear(this._root),this._root=new r.LeafNode},e}();!function(e){e.from=function(t,n){var r=new e(n);return r.assign(t),r}}(o||(o={})),function(e){var t=function(){function e(){this.items=[],this.sizes=[],this.children=[]}return Object.defineProperty(e.prototype,"type",{get:function(){return 0},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"size",{get:function(){return this.sizes[this.sizes.length-1]},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"width",{get:function(){return this.children.length},enumerable:!0,configurable:!0}),e}();e.BranchNode=t;var n=function(){function e(){this.next=null,this.prev=null,this.items=[]}return Object.defineProperty(e.prototype,"type",{get:function(){return 1},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"size",{get:function(){return this.items.length},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"width",{get:function(){return this.items.length},enumerable:!0,configurable:!0}),e}();function r(e){for(;0===e.type;)e=e.children[0];return e}function o(e){for(;0===e.type;)e=e.children[e.children.length-1];return e}function a(e){0===e.type?(Object(i.each)(e.children,a),e.children.length=0,e.sizes.length=0,e.items.length=0):(e.items.length=0,e.next=null,e.prev=null)}e.LeafNode=n,e.firstLeaf=r,e.lastLeaf=o,e.iterItems=function(e){var t=r(e);return new u(t,0,-1)},e.retroItems=function(e){var t=o(e);return new c(t,t.size-1,-1)},e.sliceItems=function(e,t,n){t=void 0===t?0:t<0?Math.max(0,t+e.size):Math.min(t,e.size),n=void 0===n?e.size:n<0?Math.max(0,n+e.size):Math.min(n,e.size);var r=Math.max(0,n-t);if(0===r)return Object(i.empty)();for(;0===e.type;){var o=d(e.sizes,t);o>0&&(t-=e.sizes[o-1]),e=e.children[o]}return new u(e,t,r)},e.retroSliceItems=function(e,t,n){t=void 0===t?e.size-1:t<0?Math.max(-1,t+e.size):Math.min(t,e.size-1),n=void 0===n?-1:n<0?Math.max(-1,n+e.size):Math.min(n,e.size-1);var r=Math.max(0,t-n);if(0===r)return Object(i.empty)();for(;0===e.type;){var o=d(e.sizes,t);o>0&&(t-=e.sizes[o-1]),e=e.children[o]}return new c(e,t,r)},e.itemAt=function(e,t){if(t<0&&(t+=e.size),!(t<0||t>=e.size)){for(;0===e.type;){var n=d(e.sizes,t);n>0&&(t-=e.sizes[n-1]),e=e.children[n]}return e.items[t]}},e.hasItem=function(e,t,n){for(;0===e.type;){var r=h(e.items,t,n);e=e.children[r]}return f(e.items,t,n)>=0},e.indexOf=function(e,t,n){for(var r=0;0===e.type;){var i=h(e.items,t,n);i>0&&(r+=e.sizes[i-1]),e=e.children[i]}var o=f(e.items,t,n);return o>=0?r+o:-r+o},e.getItem=function(e,t,n){for(;0===e.type;){var r=h(e.items,t,n);e=e.children[r]}var i=f(e.items,t,n);return i>=0?e.items[i]:void 0},e.insertItem=function e(t,n,r){if(1===t.type){var o,a=f(t.items,n,r);return a>=0?(o=t.items[a],t.items[a]=n):(o=void 0,i.ArrayExt.insert(t.items,-a-1,n)),o}var l=h(t.items,n,r),u=t.children[l],c=u.size,d=e(u,n,r),v=u.size;if(t.items[l]=u.items[0],c===v)return d;if(u.width>s){var m=g(u);i.ArrayExt.insert(t.children,l+1,m),i.ArrayExt.insert(t.items,l+1,m.items[0])}return p(t,l),d},e.deleteItem=function e(t,n,r){if(1===t.type){var o=f(t.items,n,r);if(o<0)return;return i.ArrayExt.removeAt(t.items,o)}var a=h(t.items,n,r),s=t.children[a],u=s.size,c=e(s,n,r);return u===s.size||(t.items[a]=s.items[0],s.width<l&&(a=v(t,a)),p(t,a)),c},e.removeItem=function e(t,n){if(n<0&&(n+=t.size),!(n<0||n>=t.size)){if(1===t.type)return i.ArrayExt.removeAt(t.items,n);var r=d(t.sizes,n);r>0&&(n-=t.sizes[r]);var o=t.children[r],a=e(o,n);return t.items[r]=o.items[0],o.width<l&&(r=v(t,r)),p(t,r),a}},e.clear=a,e.maybeSplitRoot=function(e){if(e.width<=s)return e;var n=new t,r=g(e);return n.sizes[0]=e.size,n.sizes[1]=e.size+r.size,n.children[0]=e,n.children[1]=r,n.items[0]=e.items[0],n.items[1]=r.items[0],n},e.maybeExtractRoot=function(e){if(1===e.type)return e;if(e.children.length>1)return e;var t=e.children.pop();return a(e),t};var s=32,l=s>>1,u=function(){function e(e,t,n){this._node=e,this._index=t,this._count=n}return e.prototype.iter=function(){return this},e.prototype.clone=function(){return new e(this._node,this._index,this._count)},e.prototype.next=function(){if(null!==this._node&&0!==this._count)return this._index>=this._node.size?(this._node=this._node.next,this._index=0,this.next()):(this._count>0&&this._count--,this._node.items[this._index++])},e}(),c=function(){function e(e,t,n){this._node=e,this._index=t,this._count=n}return e.prototype.iter=function(){return this},e.prototype.clone=function(){return new e(this._node,this._index,this._count)},e.prototype.next=function(){if(null!==this._node&&0!==this._count)return this._index>=this._node.size&&(this._index=this._node.size-1),this._index<0?(this._node=this._node.prev,this._index=this._node?this._node.size-1:-1,this.next()):(this._count>0&&this._count--,this._node.items[this._index--])},e}();function d(e,t){for(var n=e.length,r=0;r<n;++r)if(e[r]>t)return r;return n-1}function h(e,t,n){for(var r=e.length,i=1;i<r;++i)if(n(e[i],t)>0)return i-1;return r-1}function f(e,t,n){for(var r=e.length,i=0;i<r;++i){var o=n(e[i],t);if(0===o)return i;if(o>0)return-i-1}return-r-1}function p(e,t){for(var n=e.sizes,r=e.children,i=t>0?n[t-1]:0,o=r.length;t<o;++t)i=n[t]=i+r[t].size;n.length=r.length}function g(e){if(1===e.type){for(var r=new n,i=e.items,o=r.items,a=l,s=i.length;a<s;++a)o.push(i[a]);return i.length=l,e.next&&(e.next.prev=r),r.next=e.next,r.prev=e,e.next=r,r}var u=new t,c=e.children,d=u.children;for(a=l,s=c.length;a<s;++a)d.push(c[a]);c.length=l;var h=e.items,f=u.items;for(a=l,s=h.length;a<s;++a)f.push(h[a]);return h.length=l,p(e,l),p(u,0),u}function v(e,t){var n,r,o,s,u,c,d=e.children[t],h=0===t?e.children[t+1]:e.children[t-1],f=0===t,g=1===d.type,v=h.width>l;if(g&&v&&f){var m=h;return(y=d).items.push(m.items.shift()),e.items[t+1]=m.items[0],t}if(g&&v&&!f){m=h;return(y=d).items.unshift(m.items.pop()),e.items[t]=y.items[0],t-1}if(g&&!v&&f){var y=d;return(n=(m=h).items).unshift.apply(n,y.items),i.ArrayExt.removeAt(e.children,t),i.ArrayExt.removeAt(e.items,t+1),y.prev&&(y.prev.next=m),m.prev=y.prev,a(y),t}if(g&&!v&&!f){y=d;return(r=(m=h).items).push.apply(r,y.items),i.ArrayExt.removeAt(e.children,t),i.ArrayExt.removeAt(e.items,t),y.next&&(y.next.prev=m),m.next=y.next,a(y),t-1}if(!g&&v&&f){m=h;return(y=d).children.push(m.children.shift()),y.items.push(m.items.shift()),e.items[t+1]=m.items[0],p(y,y.width-1),p(m,0),t}if(!g&&v&&!f){m=h;return(y=d).children.unshift(m.children.pop()),y.items.unshift(m.items.pop()),e.items[t]=y.items[0],p(y,0),p(m,m.width-1),t-1}if(!g&&!v&&f){y=d;return(o=(m=h).children).unshift.apply(o,y.children),(s=m.items).unshift.apply(s,y.items),i.ArrayExt.removeAt(e.children,t),i.ArrayExt.removeAt(e.items,t+1),p(m,0),y.children.length=0,a(y),t}if(!g&&!v&&!f){y=d;return(u=(m=h).children).push.apply(u,y.children),(c=m.items).push.apply(c,y.items),i.ArrayExt.removeAt(e.children,t),i.ArrayExt.removeAt(e.items,t),p(m,0),y.children.length=0,a(y),t-1}throw"unreachable"}}(r||(r={}));var a,s=function(){function e(){this._first=null,this._last=null,this._size=0}return Object.defineProperty(e.prototype,"isEmpty",{get:function(){return 0===this._size},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"size",{get:function(){return this._size},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"length",{get:function(){return this._size},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"first",{get:function(){return this._first?this._first.value:void 0},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"last",{get:function(){return this._last?this._last.value:void 0},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"firstNode",{get:function(){return this._first},enumerable:!0,configurable:!0}),Object.defineProperty(e.prototype,"lastNode",{get:function(){return this._last},enumerable:!0,configurable:!0}),e.prototype.iter=function(){return new e.ForwardValueIterator(this._first)},e.prototype.retro=function(){return new e.RetroValueIterator(this._last)},e.prototype.nodes=function(){return new e.ForwardNodeIterator(this._first)},e.prototype.retroNodes=function(){return new e.RetroNodeIterator(this._last)},e.prototype.assign=function(e){var t=this;this.clear(),Object(i.each)(e,(function(e){t.addLast(e)}))},e.prototype.push=function(e){this.addLast(e)},e.prototype.pop=function(){return this.removeLast()},e.prototype.shift=function(e){this.addFirst(e)},e.prototype.unshift=function(){return this.removeFirst()},e.prototype.addFirst=function(e){var t=new a.LinkedListNode(this,e);return this._first?(t.next=this._first,this._first.prev=t,this._first=t):(this._first=t,this._last=t),this._size++,t},e.prototype.addLast=function(e){var t=new a.LinkedListNode(this,e);return this._last?(t.prev=this._last,this._last.next=t,this._last=t):(this._first=t,this._last=t),this._size++,t},e.prototype.insertBefore=function(e,t){if(!t||t===this._first)return this.addFirst(e);if(!(t instanceof a.LinkedListNode)||t.list!==this)throw new Error("Reference node is not owned by the list.");var n=new a.LinkedListNode(this,e),r=t,i=r.prev;return n.next=r,n.prev=i,r.prev=n,i.next=n,this._size++,n},e.prototype.insertAfter=function(e,t){if(!t||t===this._last)return this.addLast(e);if(!(t instanceof a.LinkedListNode)||t.list!==this)throw new Error("Reference node is not owned by the list.");var n=new a.LinkedListNode(this,e),r=t,i=r.next;return n.next=i,n.prev=r,r.next=n,i.prev=n,this._size++,n},e.prototype.removeFirst=function(){var e=this._first;if(e)return e===this._last?(this._first=null,this._last=null):(this._first=e.next,this._first.prev=null),e.list=null,e.next=null,e.prev=null,this._size--,e.value},e.prototype.removeLast=function(){var e=this._last;if(e)return e===this._first?(this._first=null,this._last=null):(this._last=e.prev,this._last.next=null),e.list=null,e.next=null,e.prev=null,this._size--,e.value},e.prototype.removeNode=function(e){if(!(e instanceof a.LinkedListNode)||e.list!==this)throw new Error("Node is not owned by the list.");var t=e;t===this._first&&t===this._last?(this._first=null,this._last=null):t===this._first?(this._first=t.next,this._first.prev=null):t===this._last?(this._last=t.prev,this._last.next=null):(t.next.prev=t.prev,t.prev.next=t.next),t.list=null,t.next=null,t.prev=null,this._size--},e.prototype.clear=function(){for(var e=this._first;e;){var t=e.next;e.list=null,e.prev=null,e.next=null,e=t}this._first=null,this._last=null,this._size=0},e}();!function(e){e.from=function(t){var n=new e;return n.assign(t),n};var t=function(){function e(e){this._node=e}return e.prototype.iter=function(){return this},e.prototype.clone=function(){return new e(this._node)},e.prototype.next=function(){if(this._node){var e=this._node;return this._node=e.next,e.value}},e}();e.ForwardValueIterator=t;var n=function(){function e(e){this._node=e}return e.prototype.iter=function(){return this},e.prototype.clone=function(){return new e(this._node)},e.prototype.next=function(){if(this._node){var e=this._node;return this._node=e.prev,e.value}},e}();e.RetroValueIterator=n;var r=function(){function e(e){this._node=e}return e.prototype.iter=function(){return this},e.prototype.clone=function(){return new e(this._node)},e.prototype.next=function(){if(this._node){var e=this._node;return this._node=e.next,e}},e}();e.ForwardNodeIterator=r;var i=function(){function e(e){this._node=e}return e.prototype.iter=function(){return this},e.prototype.clone=function(){return new e(this._node)},e.prototype.next=function(){if(this._node){var e=this._node;return this._node=e.prev,e}},e}();e.RetroNodeIterator=i}(s||(s={})),function(e){var t=function(e,t){this.list=null,this.next=null,this.prev=null,this.list=e,this.value=t};e.LinkedListNode=t}(a||(a={}))},function(e,t,n){"use strict";n.d(t,"a",(function(){return a}));var r,i=n(6),o=n(16);class a{constructor(e={}){this._array=[],this._isDisposed=!1,this._changed=new o.Signal(this),void 0!==e.values&&Object(i.each)(e.values,e=>{this._array.push(e)}),this._itemCmp=e.itemCmp||r.itemCmp}get type(){return"List"}get changed(){return this._changed}get length(){return this._array.length}get isDisposed(){return this._isDisposed}dispose(){this._isDisposed||(this._isDisposed=!0,o.Signal.clearData(this),this.clear())}iter(){return new i.ArrayIterator(this._array)}get(e){return this._array[e]}set(e,t){const n=this._array[e];if(void 0===t)throw new Error("Cannot set an undefined item");(0,this._itemCmp)(n,t)||(this._array[e]=t,this._changed.emit({type:"set",oldIndex:e,newIndex:e,oldValues:[n],newValues:[t]}))}push(e){const t=this._array.push(e);return this._changed.emit({type:"add",oldIndex:-1,newIndex:this.length-1,oldValues:[],newValues:[e]}),t}insert(e,t){i.ArrayExt.insert(this._array,e,t),this._changed.emit({type:"add",oldIndex:-1,newIndex:e,oldValues:[],newValues:[t]})}removeValue(e){const t=this._itemCmp,n=i.ArrayExt.findFirstIndex(this._array,n=>t(n,e));return this.remove(n),n}remove(e){const t=i.ArrayExt.removeAt(this._array,e);if(void 0!==t)return this._changed.emit({type:"remove",oldIndex:e,newIndex:-1,newValues:[],oldValues:[t]}),t}clear(){const e=this._array.slice();this._array.length=0,this._changed.emit({type:"remove",oldIndex:0,newIndex:0,newValues:[],oldValues:e})}move(e,t){if(this.length<=1||e===t)return;const n=[this._array[e]];i.ArrayExt.move(this._array,e,t),this._changed.emit({type:"move",oldIndex:e,newIndex:t,oldValues:n,newValues:n})}pushAll(e){const t=this.length;return Object(i.each)(e,e=>{this._array.push(e)}),this._changed.emit({type:"add",oldIndex:-1,newIndex:t,oldValues:[],newValues:Object(i.toArray)(e)}),this.length}insertAll(e,t){const n=e;Object(i.each)(t,t=>{i.ArrayExt.insert(this._array,e++,t)}),this._changed.emit({type:"add",oldIndex:-1,newIndex:n,oldValues:[],newValues:Object(i.toArray)(t)})}removeRange(e,t){const n=this._array.slice(e,t);for(let n=e;n<t;n++)i.ArrayExt.removeAt(this._array,e);return this._changed.emit({type:"remove",oldIndex:e,newIndex:-1,oldValues:n,newValues:[]}),this.length}}!function(e){e.itemCmp=function(e,t){return e===t}}(r||(r={}))},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.AnchorButton=t.Button=void 0;var r=n(2),i=r.__importStar(n(0)),o=n(209),a=n(502),s=n(1454),l=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.buttonRef=null,t.handleRef=a.refHandler(t,"buttonRef",t.props.elementRef),t}return r.__extends(t,e),t.prototype.render=function(){return i.createElement("button",r.__assign({type:"button",ref:this.handleRef},o.removeNonHTMLProps(this.props),this.getCommonButtonProps()),this.renderChildren())},t.displayName=o.DISPLAYNAME_PREFIX+".Button",t}(s.AbstractButton);t.Button=l;var u=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.buttonRef=null,t.handleRef=a.refHandler(t,"buttonRef",t.props.elementRef),t}return r.__extends(t,e),t.prototype.render=function(){var e=this.props,t=e.href,n=e.tabIndex,a=void 0===n?0:n,s=this.getCommonButtonProps();return i.createElement("a",r.__assign({role:"button",ref:this.handleRef},o.removeNonHTMLProps(this.props),s,{href:s.disabled?void 0:t,tabIndex:s.disabled?-1:a}),this.renderChildren())},t.displayName=o.DISPLAYNAME_PREFIX+".AnchorButton",t}(s.AbstractButton);t.AnchorButton=u},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIyNCIgaGVpZ2h0PSIyNCIgdmlld0JveD0iMCAwIDI0IDI0Ij4KICAgIDxwYXRoCiAgICAgICAgY2xhc3M9ImpwLWljb24wIgogICAgICAgIGZpbGw9IiMwMDAiCiAgICAgICAgZD0iTTI0IDIwLjE4OGwtOC4zMTUtOC4yMDkgOC4yLTguMjgyLTMuNjk3LTMuNjk3LTguMjEyIDguMzE4LTguMzEtOC4yMDMtMy42NjYgMy42NjYgOC4zMjEgOC4yNC04LjIwNiA4LjMxMyAzLjY2NiAzLjY2NiA4LjIzNy04LjMxOCA4LjI4NSA4LjIwM3oiCiAgICAvPgo8L3N2Zz4K"},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIyNCIgaGVpZ2h0PSIyNCIgdmlld0JveD0iMCAwIDI0IDI0Ij4KICAgIDxwYXRoCiAgICAgICAgZmlsbD0iIzAwMCIKICAgICAgICBmaWxsLW9wYWNpdHk9IjAuMCIKICAgICAgICBkPSJNMjQgMjAuMTg4bC04LjMxNS04LjIwOSA4LjItOC4yODItMy42OTctMy42OTctOC4yMTIgOC4zMTgtOC4zMS04LjIwMy0zLjY2NiAzLjY2NiA4LjMyMSA4LjI0LTguMjA2IDguMzEzIDMuNjY2IDMuNjY2IDguMjM3LTguMzE4IDguMjg1IDguMjAzeiIKICAgIC8+Cjwvc3ZnPgo="},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.Collapse=t.AnimationStates=void 0;var r,i=n(2),o=i.__importDefault(n(11)),a=i.__importStar(n(0)),s=n(13),l=n(239),u=n(209);!function(e){e[e.OPEN_START=0]="OPEN_START",e[e.OPENING=1]="OPENING",e[e.OPEN=2]="OPEN",e[e.CLOSING_START=3]="CLOSING_START",e[e.CLOSING=4]="CLOSING",e[e.CLOSED=5]="CLOSED"}(r=t.AnimationStates||(t.AnimationStates={}));var c=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.state={animationState:t.props.isOpen?r.OPEN:r.CLOSED,height:void 0,heightWhenOpen:void 0},t.contents=null,t.contentsRefHandler=function(e){if(t.contents=e,null!=t.contents){var n=t.contents.clientHeight;t.setState({animationState:t.props.isOpen?r.OPEN:r.CLOSED,height:0===n?void 0:n+"px",heightWhenOpen:0===n?void 0:n})}},t}return i.__extends(t,e),t.getDerivedStateFromProps=function(e,t){var n=e.isOpen,i=t.animationState;if(n)switch(i){case r.OPEN:case r.OPENING:break;default:return{animationState:r.OPEN_START}}else switch(i){case r.CLOSED:case r.CLOSING:break;default:return{animationState:r.CLOSING_START,height:t.heightWhenOpen+"px"}}return null},t.prototype.render=function(){var e=this.state.animationState!==r.CLOSED,t=e||this.props.keepChildrenMounted,n=e&&this.state.animationState!==r.CLOSING,i="auto"===this.state.height,s={height:e?this.state.height:void 0,overflowY:i?"visible":void 0,transition:i?"none":void 0},u={transform:n?"translateY(0)":"translateY(-"+this.state.heightWhenOpen+"px)",transition:i?"none":void 0};return a.createElement(this.props.component,{className:o.default(l.Classes.COLLAPSE,this.props.className),style:s},a.createElement("div",{className:l.Classes.COLLAPSE_BODY,ref:this.contentsRefHandler,style:u,"aria-hidden":!e&&this.props.keepChildrenMounted},t?this.props.children:null))},t.prototype.componentDidMount=function(){this.forceUpdate(),this.props.isOpen?this.setState({animationState:r.OPEN,height:"auto"}):this.setState({animationState:r.CLOSED,height:"0px"})},t.prototype.componentDidUpdate=function(){var e=this;if(null!=this.contents){var t=this.props.transitionDuration,n=this.state.animationState;if(n===r.OPEN_START){var i=this.contents.clientHeight;this.setState({animationState:r.OPENING,height:i+"px",heightWhenOpen:i}),this.setTimeout((function(){return e.onDelayedStateChange()}),t)}else if(n===r.CLOSING_START){var o=this.contents.clientHeight;this.setTimeout((function(){return e.setState({animationState:r.CLOSING,height:"0px",heightWhenOpen:o})})),this.setTimeout((function(){return e.onDelayedStateChange()}),t)}}},t.prototype.onDelayedStateChange=function(){switch(this.state.animationState){case r.OPENING:this.setState({animationState:r.OPEN,height:"auto"});break;case r.CLOSING:this.setState({animationState:r.CLOSED})}},t.displayName=u.DISPLAYNAME_PREFIX+".Collapse",t.defaultProps={component:"div",isOpen:!1,keepChildrenMounted:!1,transitionDuration:200},t=i.__decorate([s.polyfill],t)}(l.AbstractPureComponent2);t.Collapse=c},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.Select=void 0;var r=n(506),i=r.__importDefault(n(11)),o=r.__importStar(n(0)),a=n(431),s=n(890),l=n(1503),u=function(e){function t(){var t,n=e.apply(this,arguments)||this;return n.state={isOpen:!1},n.TypedQueryList=l.QueryList.ofType(),n.inputElement=null,n.queryList=null,n.handleInputRef=a.refHandler(n,"inputElement",null===(t=n.props.inputProps)||void 0===t?void 0:t.inputRef),n.handleQueryListRef=function(e){return n.queryList=e},n.renderQueryList=function(e){var t=n.props,l=t.filterable,u=void 0===l||l,c=t.disabled,d=void 0!==c&&c,h=t.inputProps,f=void 0===h?{}:h,p=t.popoverProps,g=void 0===p?{}:p,v=o.createElement(a.InputGroup,r.__assign({leftIcon:"search",placeholder:"Filter...",rightElement:n.maybeRenderClearButton(e.query)},f,{inputRef:n.handleInputRef,onChange:e.handleQueryChange,value:e.query})),m=e.handleKeyDown,y=e.handleKeyUp;return o.createElement(a.Popover,r.__assign({autoFocus:!1,enforceFocus:!1,isOpen:n.state.isOpen,disabled:d,position:a.Position.BOTTOM_LEFT},g,{className:i.default(e.className,g.className),onInteraction:n.handlePopoverInteraction,popoverClassName:i.default(s.Classes.SELECT_POPOVER,g.popoverClassName),onOpening:n.handlePopoverOpening,onOpened:n.handlePopoverOpened,onClosing:n.handlePopoverClosing}),o.createElement("div",{onKeyDown:n.state.isOpen?m:n.handleTargetKeyDown,onKeyUp:n.state.isOpen?y:void 0},n.props.children),o.createElement("div",{onKeyDown:m,onKeyUp:y},u?v:void 0,e.itemList))},n.handleTargetKeyDown=function(e){e.which!==a.Keys.ARROW_UP&&e.which!==a.Keys.ARROW_DOWN||(e.preventDefault(),n.setState({isOpen:!0}))},n.handleItemSelect=function(e,t){var r,i;n.setState({isOpen:!1}),null===(i=(r=n.props).onItemSelect)||void 0===i||i.call(r,e,t)},n.handlePopoverInteraction=function(e){var t,r;n.setState({isOpen:e}),null===(r=null===(t=n.props.popoverProps)||void 0===t?void 0:t.onInteraction)||void 0===r||r.call(t,e)},n.handlePopoverOpening=function(e){var t,r;n.previousFocusedElement=document.activeElement,n.props.resetOnClose&&n.resetQuery(),null===(r=null===(t=n.props.popoverProps)||void 0===t?void 0:t.onOpening)||void 0===r||r.call(t,e)},n.handlePopoverOpened=function(e){var t,r;null!=n.queryList&&n.queryList.scrollActiveItemIntoView(),n.requestAnimationFrame((function(){var e,t=n.props.inputProps;!1!==(void 0===t?{}:t).autoFocus&&(null===(e=a.getRef(n.inputElement))||void 0===e||e.focus())})),null===(r=null===(t=n.props.popoverProps)||void 0===t?void 0:t.onOpened)||void 0===r||r.call(t,e)},n.handlePopoverClosing=function(e){var t,r;n.requestAnimationFrame((function(){void 0!==n.previousFocusedElement&&(n.previousFocusedElement.focus(),n.previousFocusedElement=void 0)})),null===(r=null===(t=n.props.popoverProps)||void 0===t?void 0:t.onClosing)||void 0===r||r.call(t,e)},n.resetQuery=function(){return n.queryList&&n.queryList.setQuery("",!0)},n}return r.__extends(t,e),t.ofType=function(){return t},t.prototype.render=function(){var e=this.props,t=(e.filterable,e.inputProps,e.popoverProps,r.__rest(e,["filterable","inputProps","popoverProps"]));return o.createElement(this.TypedQueryList,r.__assign({},t,{onItemSelect:this.handleItemSelect,ref:this.handleQueryListRef,renderer:this.renderQueryList}))},t.prototype.componentDidUpdate=function(e,t){this.state.isOpen&&!t.isOpen&&null!=this.queryList&&this.queryList.scrollActiveItemIntoView()},t.prototype.maybeRenderClearButton=function(e){return e.length>0?o.createElement(a.Button,{icon:"cross",minimal:!0,onClick:this.resetQuery}):void 0},t.displayName=a.DISPLAYNAME_PREFIX+".Select",t}(a.AbstractPureComponent2);t.Select=u},function(e,t,n){var r=n(881),i=n(1487),o=n(1491),a=n(1493),s=n(1494),l=n(1496),u=Date.prototype.getTime;function c(e,t,n){var f=n||{};return!!(f.strict?o(e,t):e===t)||(!e||!t||"object"!=typeof e&&"object"!=typeof t?f.strict?o(e,t):e==t:function(e,t,n){var o,f;if(typeof e!=typeof t)return!1;if(d(e)||d(t))return!1;if(e.prototype!==t.prototype)return!1;if(i(e)!==i(t))return!1;var p=a(e),g=a(t);if(p!==g)return!1;if(p||g)return e.source===t.source&&s(e)===s(t);if(l(e)&&l(t))return u.call(e)===u.call(t);var v=h(e),m=h(t);if(v!==m)return!1;if(v||m){if(e.length!==t.length)return!1;for(o=0;o<e.length;o++)if(e[o]!==t[o])return!1;return!0}if(typeof e!=typeof t)return!1;try{var y=r(e),b=r(t)}catch(e){return!1}if(y.length!==b.length)return!1;for(y.sort(),b.sort(),o=y.length-1;o>=0;o--)if(y[o]!=b[o])return!1;for(o=y.length-1;o>=0;o--)if(f=y[o],!c(e[f],t[f],n))return!1;return!0}(e,t,f))}function d(e){return null==e}function h(e){return!(!e||"object"!=typeof e||"number"!=typeof e.length)&&("function"==typeof e.copy&&"function"==typeof e.slice&&!(e.length>0&&"number"!=typeof e[0]))}e.exports=c},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.Checkbox=t.Radio=t.Switch=void 0;var r=n(2),i=r.__importDefault(n(11)),o=r.__importStar(n(0)),a=n(13),s=n(239),l=n(209),u=function(e){var t,n=e.alignIndicator,a=e.children,l=e.className,u=e.indicatorChildren,c=e.inline,d=e.inputRef,h=e.label,f=e.labelElement,p=e.large,g=e.style,v=e.type,m=e.typeClassName,y=e.tagName,b=void 0===y?"label":y,C=r.__rest(e,["alignIndicator","children","className","indicatorChildren","inline","inputRef","label","labelElement","large","style","type","typeClassName","tagName"]),A=i.default(s.Classes.CONTROL,m,((t={})[s.Classes.DISABLED]=C.disabled,t[s.Classes.INLINE]=c,t[s.Classes.LARGE]=p,t),s.Classes.alignmentClass(n),l);return o.createElement(b,{className:A,style:g},o.createElement("input",r.__assign({},C,{ref:d,type:v})),o.createElement("span",{className:s.Classes.CONTROL_INDICATOR},u),h,f,a)},c=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return r.__extends(t,e),t.prototype.render=function(){var e=this.props,t=e.innerLabelChecked,n=e.innerLabel,i=r.__rest(e,["innerLabelChecked","innerLabel"]),a=n||t?[o.createElement("div",{key:"checked",className:s.Classes.CONTROL_INDICATOR_CHILD},o.createElement("div",{className:s.Classes.SWITCH_INNER_TEXT},t||n)),o.createElement("div",{key:"unchecked",className:s.Classes.CONTROL_INDICATOR_CHILD},o.createElement("div",{className:s.Classes.SWITCH_INNER_TEXT},n))]:null;return o.createElement(u,r.__assign({},i,{type:"checkbox",typeClassName:s.Classes.SWITCH,indicatorChildren:a}))},t.displayName=l.DISPLAYNAME_PREFIX+".Switch",t=r.__decorate([a.polyfill],t)}(s.AbstractPureComponent2);t.Switch=c;var d=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return r.__extends(t,e),t.prototype.render=function(){return o.createElement(u,r.__assign({},this.props,{type:"radio",typeClassName:s.Classes.RADIO}))},t.displayName=l.DISPLAYNAME_PREFIX+".Radio",t=r.__decorate([a.polyfill],t)}(s.AbstractPureComponent2);t.Radio=d;var h=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.state={indeterminate:t.props.indeterminate||t.props.defaultIndeterminate||!1},t.input=null,t.handleInputRef=s.refHandler(t,"input",t.props.inputRef),t.handleChange=function(e){var n,r,i=e.target.indeterminate;null==t.props.indeterminate&&t.setState({indeterminate:i}),null===(r=(n=t.props).onChange)||void 0===r||r.call(n,e)},t}return r.__extends(t,e),t.getDerivedStateFromProps=function(e){var t=e.indeterminate;return null!=t?{indeterminate:t}:null},t.prototype.render=function(){var e=this.props,t=(e.defaultIndeterminate,e.indeterminate,r.__rest(e,["defaultIndeterminate","indeterminate"]));return o.createElement(u,r.__assign({},t,{inputRef:this.handleInputRef,onChange:this.handleChange,type:"checkbox",typeClassName:s.Classes.CHECKBOX}))},t.prototype.componentDidMount=function(){this.updateIndeterminate()},t.prototype.componentDidUpdate=function(){this.updateIndeterminate()},t.prototype.updateIndeterminate=function(){null!=this.input&&(this.input.indeterminate=this.state.indeterminate)},t.displayName=l.DISPLAYNAME_PREFIX+".Checkbox",t=r.__decorate([a.polyfill],t)}(s.AbstractPureComponent2);t.Checkbox=h},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDI0IDI0Ij4KICA8ZyBjbGFzcz0ianAtaWNvbjMiIGZpbGw9IiM2MTYxNjEiPgogICAgPHBhdGggZD0iTTE5IDEzaC02djZoLTJ2LTZINXYtMmg2VjVoMnY2aDZ2MnoiLz4KICA8L2c+Cjwvc3ZnPgo="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB2aWV3Qm94PSIwIDAgMjQgMjQiIHdpZHRoPSIxNiIgeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj4KICA8ZyBjbGFzcz0ianAtaWNvbjMiIGZpbGw9IiM2MTYxNjEiPgogICAgPHBhdGggZD0iTTIwIDhoLTIuODFjLS40NS0uNzgtMS4wNy0xLjQ1LTEuODItMS45NkwxNyA0LjQxIDE1LjU5IDNsLTIuMTcgMi4xN0MxMi45NiA1LjA2IDEyLjQ5IDUgMTIgNWMtLjQ5IDAtLjk2LjA2LTEuNDEuMTdMOC40MSAzIDcgNC40MWwxLjYyIDEuNjNDNy44OCA2LjU1IDcuMjYgNy4yMiA2LjgxIDhINHYyaDIuMDljLS4wNS4zMy0uMDkuNjYtLjA5IDF2MUg0djJoMnYxYzAgLjM0LjA0LjY3LjA5IDFINHYyaDIuODFjMS4wNCAxLjc5IDIuOTcgMyA1LjE5IDNzNC4xNS0xLjIxIDUuMTktM0gyMHYtMmgtMi4wOWMuMDUtLjMzLjA5LS42Ni4wOS0xdi0xaDJ2LTJoLTJ2LTFjMC0uMzQtLjA0LS42Ny0uMDktMUgyMFY4em0tNiA4aC00di0yaDR2MnptMC00aC00di0yaDR2MnoiLz4KICA8L2c+Cjwvc3ZnPgo="},function(e,t){e.exports="data:image/svg+xml;base64,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"},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDE4IDE4Ij4KICA8ZyBjbGFzcz0ianAtaWNvbjMiIGZpbGw9IiM2MTYxNjEiIHNoYXBlLXJlbmRlcmluZz0iZ2VvbWV0cmljUHJlY2lzaW9uIj4KICAgIDxwYXRoIGQ9Ik01LjIsNS45TDksOS43bDMuOC0zLjhsMS4yLDEuMmwtNC45LDVsLTQuOS01TDUuMiw1Ljl6Ii8+CiAgPC9nPgo8L3N2Zz4K"},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDIwIDIwIj4KCTxnIGNsYXNzPSJqcC1pY29uMyIgZmlsbD0iIzYxNjE2MSIgc2hhcGUtcmVuZGVyaW5nPSJnZW9tZXRyaWNQcmVjaXNpb24iPgoJCTxwb2x5Z29uIGNsYXNzPSJzdDEiIHBvaW50cz0iOS45LDEzLjYgMy42LDcuNCA0LjQsNi42IDkuOSwxMi4yIDE1LjQsNi43IDE2LjEsNy40ICIvPgoJPC9nPgo8L3N2Zz4K"},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDE4IDE4Ij4KICA8ZyBjbGFzcz0ianAtaWNvbjMiIGZpbGw9IiM2MTYxNjEiIHNoYXBlLXJlbmRlcmluZz0iZ2VvbWV0cmljUHJlY2lzaW9uIj4KICAgIDxwYXRoIGQ9Ik01LjIsNy41TDksMTEuMmwzLjgtMy44SDUuMnoiLz4KICA8L2c+Cjwvc3ZnPgo="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDE4IDE4Ij4KCTxnIGNsYXNzPSJqcC1pY29uMyIgZmlsbD0iIzYxNjE2MSIgc2hhcGUtcmVuZGVyaW5nPSJnZW9tZXRyaWNQcmVjaXNpb24iPgoJCTxwYXRoIGQ9Ik0xMC44LDEyLjhMNy4xLDlsMy44LTMuOGwwLDcuNkgxMC44eiIvPgogIDwvZz4KPC9zdmc+Cg=="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDE4IDE4Ij4KICA8ZyBjbGFzcz0ianAtaWNvbjMiIGZpbGw9IiM2MTYxNjEiIHNoYXBlLXJlbmRlcmluZz0iZ2VvbWV0cmljUHJlY2lzaW9uIj4KICAgIDxwYXRoIGQ9Ik03LjIsNS4yTDEwLjksOWwtMy44LDMuOFY1LjJINy4yeiIvPgogIDwvZz4KPC9zdmc+Cg=="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDIwIDIwIj4KCTxnIGNsYXNzPSJqcC1pY29uMyIgZmlsbD0iIzYxNjE2MSIgc2hhcGUtcmVuZGVyaW5nPSJnZW9tZXRyaWNQcmVjaXNpb24iPgoJCTxwb2x5Z29uIGNsYXNzPSJzdDEiIHBvaW50cz0iMTUuNCwxMy4zIDkuOSw3LjcgNC40LDEzLjIgMy42LDEyLjUgOS45LDYuMyAxNi4xLDEyLjYgIi8+Cgk8L2c+Cjwvc3ZnPgo="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDE4IDE4Ij4KCTxnIGNsYXNzPSJqcC1pY29uMyIgZmlsbD0iIzYxNjE2MSIgc2hhcGUtcmVuZGVyaW5nPSJnZW9tZXRyaWNQcmVjaXNpb24iPgoJCTxwYXRoIGQ9Ik01LjIsMTAuNUw5LDYuOGwzLjgsMy44SDUuMnoiLz4KICA8L2c+Cjwvc3ZnPgo="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDIwIDIwIj4KICA8ZyBjbGFzcz0ianAtaWNvbjIiIGZpbGw9IiM0MTQxNDEiPgogICAgPHJlY3QgeD0iMiIgeT0iMiIgd2lkdGg9IjE2IiBoZWlnaHQ9IjE2Ii8+CiAgPC9nPgogIDxnIGNsYXNzPSJqcC1pY29uLWFjY2VudDIiIGZpbGw9IiNGRkYiPgogICAgPHBhdGggZD0iTTcuNiw4aDAuOWwzLjUsOGgtMS4xTDEwLDE0SDZsLTAuOSwySDRMNy42LDh6IE04LDkuMUw2LjQsMTNoMy4yTDgsOS4xeiIvPgogICAgPHBhdGggZD0iTTE2LjYsOS44Yy0wLjIsMC4xLTAuNCwwLjEtMC43LDAuMWMtMC4yLDAtMC40LTAuMS0wLjYtMC4yYy0wLjEtMC4xLTAuMi0wLjQtMC4yLTAuNyBjLTAuMywwLjMtMC42LDAuNS0wLjksMC43Yy0wLjMsMC4xLTAuNywwLjItMS4xLDAuMmMtMC4zLDAtMC41LDAtMC43LTAuMWMtMC4yLTAuMS0wLjQtMC4yLTAuNi0wLjNjLTAuMi0wLjEtMC4zLTAuMy0wLjQtMC41IGMtMC4xLTAuMi0wLjEtMC40LTAuMS0wLjdjMC0wLjMsMC4xLTAuNiwwLjItMC44YzAuMS0wLjIsMC4zLTAuNCwwLjQtMC41QzEyLDcsMTIuMiw2LjksMTIuNSw2LjhjMC4yLTAuMSwwLjUtMC4xLDAuNy0wLjIgYzAuMy0wLjEsMC41LTAuMSwwLjctMC4xYzAuMiwwLDAuNC0wLjEsMC42LTAuMWMwLjIsMCwwLjMtMC4xLDAuNC0wLjJjMC4xLTAuMSwwLjItMC4yLDAuMi0wLjRjMC0xLTEuMS0xLTEuMy0xIGMtMC40LDAtMS40LDAtMS40LDEuMmgtMC45YzAtMC40LDAuMS0wLjcsMC4yLTFjMC4xLTAuMiwwLjMtMC40LDAuNS0wLjZjMC4yLTAuMiwwLjUtMC4zLDAuOC0wLjNDMTMuMyw0LDEzLjYsNCwxMy45LDQgYzAuMywwLDAuNSwwLDAuOCwwLjFjMC4zLDAsMC41LDAuMSwwLjcsMC4yYzAuMiwwLjEsMC40LDAuMywwLjUsMC41QzE2LDUsMTYsNS4yLDE2LDUuNnYyLjljMCwwLjIsMCwwLjQsMCwwLjUgYzAsMC4xLDAuMSwwLjIsMC4zLDAuMmMwLjEsMCwwLjIsMCwwLjMsMFY5Ljh6IE0xNS4yLDYuOWMtMS4yLDAuNi0zLjEsMC4yLTMuMSwxLjRjMCwxLjQsMy4xLDEsMy4xLTAuNVY2Ljl6Ii8+CiAgPC9nPgo8L3N2Zz4K"},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDI0IDI0Ij4KICA8ZyBjbGFzcz0ianAtaWNvbjMiIGZpbGw9IiM2MTYxNjEiPgogICAgPHBhdGggZD0iTTkgMTYuMTdMNC44MyAxMmwtMS40MiAxLjQxTDkgMTkgMjEgN2wtMS40MS0xLjQxeiIvPgogIDwvZz4KPC9zdmc+Cg=="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDI0IDI0Ij4KICA8ZyBjbGFzcz0ianAtaWNvbjMiIGZpbGw9IiM2MTYxNjEiPgogICAgPHBhdGggZD0iTTEyIDJDNi40NyAyIDIgNi40NyAyIDEyczQuNDcgMTAgMTAgMTAgMTAtNC40NyAxMC0xMFMxNy41MyAyIDEyIDJ6bTAgMThjLTQuNDEgMC04LTMuNTktOC04czMuNTktOCA4LTggOCAzLjU5IDggOC0zLjU5IDgtOCA4eiIvPgogIDwvZz4KPC9zdmc+Cg=="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB2aWV3Qm94PSIwIDAgMTggMTgiIHdpZHRoPSIxNiIgeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj4KICA8ZyBjbGFzcz0ianAtaWNvbjMiIGZpbGw9IiM2MTYxNjEiPgogICAgPGNpcmNsZSBjeD0iOSIgY3k9IjkiIHI9IjgiLz4KICA8L2c+Cjwvc3ZnPgo="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDI0IDI0Ij4KICA8bWFzayBpZD0iZG9udXRIb2xlIj4KICAgIDxyZWN0IHdpZHRoPSIyNCIgaGVpZ2h0PSIyNCIgZmlsbD0id2hpdGUiIC8+CiAgICA8Y2lyY2xlIGN4PSIxMiIgY3k9IjEyIiByPSI4IiBmaWxsPSJibGFjayIvPgogIDwvbWFzaz4KCiAgPGcgY2xhc3M9ImpwLWljb24zIiBmaWxsPSIjNjE2MTYxIj4KICAgIDxyZWN0IGhlaWdodD0iMTgiIHdpZHRoPSIyIiB4PSIxMSIgeT0iMyIgdHJhbnNmb3JtPSJyb3RhdGUoMzE1LCAxMiwgMTIpIi8+CiAgICA8Y2lyY2xlIGN4PSIxMiIgY3k9IjEyIiByPSIxMCIgbWFzaz0idXJsKCNkb251dEhvbGUpIi8+CiAgPC9nPgo8L3N2Zz4K"},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB2aWV3Qm94PSIwIDAgMjQgMjQiIHdpZHRoPSIxNiIgeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj4KICA8ZyBjbGFzcz0ianAtaWNvbi1ub25lIGpwLWljb24tc2VsZWN0YWJsZS1pbnZlcnNlIGpwLWljb24zLWhvdmVyIiBmaWxsPSJub25lIj4KICAgIDxjaXJjbGUgY3g9IjEyIiBjeT0iMTIiIHI9IjExIi8+CiAgPC9nPgoKICA8ZyBjbGFzcz0ianAtaWNvbjMganAtaWNvbi1zZWxlY3RhYmxlIGpwLWljb24tYWNjZW50Mi1ob3ZlciIgZmlsbD0iIzYxNjE2MSI+CiAgICA8cGF0aCBkPSJNMTkgNi40MUwxNy41OSA1IDEyIDEwLjU5IDYuNDEgNSA1IDYuNDEgMTAuNTkgMTIgNSAxNy41OSA2LjQxIDE5IDEyIDEzLjQxIDE3LjU5IDE5IDE5IDE3LjU5IDEzLjQxIDEyeiIvPgogIDwvZz4KCiAgPGcgY2xhc3M9ImpwLWljb24tbm9uZSBqcC1pY29uLWJ1c3kiIGZpbGw9Im5vbmUiPgogICAgPGNpcmNsZSBjeD0iMTIiIGN5PSIxMiIgcj0iNyIvPgogIDwvZz4KPC9zdmc+Cg=="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB3aWR0aD0iMjIiIGhlaWdodD0iMjIiIHZpZXdCb3g9IjAgMCAyOCAyOCIgeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj4KCTxnIGNsYXNzPSJqcC1pY29uMyIgZmlsbD0iIzYxNjE2MSI+CgkJPHBhdGggZD0iTTExLjQgMTguNkw2LjggMTRMMTEuNCA5LjRMMTAgOEw0IDE0TDEwIDIwTDExLjQgMTguNlpNMTYuNiAxOC42TDIxLjIgMTRMMTYuNiA5LjRMMTggOEwyNCAxNEwxOCAyMEwxNi42IDE4LjZWMTguNloiLz4KCTwvZz4KPC9zdmc+Cg=="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDIwMCAyMDAiPgogIDxnIGNsYXNzPSJqcC1pY29uLWJyYW5kMSBqcC1pY29uLXNlbGVjdGFibGUiIGZpbGw9IiMwMjg4RDEiPgogICAgPHBhdGggZD0iTTIwIDE5LjhoMTYwdjE1OS45SDIweiIvPgogIDwvZz4KICA8ZyBjbGFzcz0ianAtaWNvbi1zZWxlY3RhYmxlLWludmVyc2UiIGZpbGw9IiNmZmYiPgogICAgPHBhdGggZD0iTTEwNSAxMjcuM2g0MHYxMi44aC00MHpNNTEuMSA3N0w3NCA5OS45bC0yMy4zIDIzLjMgMTAuNSAxMC41IDIzLjMtMjMuM0w5NSA5OS45IDg0LjUgODkuNCA2MS42IDY2LjV6Ii8+CiAgPC9nPgo8L3N2Zz4K"},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB2aWV3Qm94PSIwIDAgMTggMTgiIHdpZHRoPSIxNiIgeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj4KICA8ZyBjbGFzcz0ianAtaWNvbjMiIGZpbGw9IiM2MTYxNjEiPgogICAgPHBhdGggZD0iTTExLjksMUgzLjJDMi40LDEsMS43LDEuNywxLjcsMi41djEwLjJoMS41VjIuNWg4LjdWMXogTTE0LjEsMy45aC04Yy0wLjgsMC0xLjUsMC43LTEuNSwxLjV2MTAuMmMwLDAuOCwwLjcsMS41LDEuNSwxLjVoOCBjMC44LDAsMS41LTAuNywxLjUtMS41VjUuNEMxNS41LDQuNiwxNC45LDMuOSwxNC4xLDMuOXogTTE0LjEsMTUuNWgtOFY1LjRoOFYxNS41eiIvPgogIDwvZz4KPC9zdmc+Cg=="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB2aWV3Qm94PSIwIDAgMjQgMjQiIHdpZHRoPSIxNiIgeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj4KICA8ZyBjbGFzcz0ianAtaWNvbjMiIGZpbGw9IiM2MTYxNjEiPgogICAgPHBhdGggZD0iTTkuNjQgNy42NGMuMjMtLjUuMzYtMS4wNS4zNi0xLjY0IDAtMi4yMS0xLjc5LTQtNC00UzIgMy43OSAyIDZzMS43OSA0IDQgNGMuNTkgMCAxLjE0LS4xMyAxLjY0LS4zNkwxMCAxMmwtMi4zNiAyLjM2QzcuMTQgMTQuMTMgNi41OSAxNCA2IDE0Yy0yLjIxIDAtNCAxLjc5LTQgNHMxLjc5IDQgNCA0IDQtMS43OSA0LTRjMC0uNTktLjEzLTEuMTQtLjM2LTEuNjRMMTIgMTRsNyA3aDN2LTFMOS42NCA3LjY0ek02IDhjLTEuMSAwLTItLjg5LTItMnMuOS0yIDItMiAyIC44OSAyIDItLjkgMi0yIDJ6bTAgMTJjLTEuMSAwLTItLjg5LTItMnMuOS0yIDItMiAyIC44OSAyIDItLjkgMi0yIDJ6bTYtNy41Yy0uMjggMC0uNS0uMjItLjUtLjVzLjIyLS41LjUtLjUuNS4yMi41LjUtLjIyLjUtLjUuNXpNMTkgM2wtNiA2IDIgMiA3LTdWM3oiLz4KICA8L2c+Cjwvc3ZnPgo="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB2aWV3Qm94PSIwIDAgMjQgMjQiIHdpZHRoPSIxNiIgeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj4KICA8ZyBjbGFzcz0ianAtaWNvbjMiIGZpbGw9IiM2MTYxNjEiPgogICAgPHBhdGggZD0iTTE5IDloLTRWM0g5djZINWw3IDcgNy03ek01IDE4djJoMTR2LTJINXoiLz4KICA8L2c+Cjwvc3ZnPgo="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB2aWV3Qm94PSIwIDAgMjQgMjQiIHdpZHRoPSIxNiIgeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj4KICA8ZyBjbGFzcz0ianAtaWNvbjMiIGZpbGw9IiM2MTYxNjEiPgogICAgPHBhdGggZD0iTTMgMTcuMjVWMjFoMy43NUwxNy44MSA5Ljk0bC0zLjc1LTMuNzVMMyAxNy4yNXpNMjAuNzEgNy4wNGMuMzktLjM5LjM5LTEuMDIgMC0xLjQxbC0yLjM0LTIuMzRjLS4zOS0uMzktMS4wMi0uMzktMS40MSAwbC0xLjgzIDEuODMgMy43NSAzLjc1IDEuODMtMS44M3oiLz4KICA8L2c+Cjwvc3ZnPgo="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB2aWV3Qm94PSIwIDAgMjQgMjQiIHdpZHRoPSIxNiIgeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj4KICA8ZyBjbGFzcz0ianAtaWNvbjMiIGZpbGw9IiM2MTYxNjEiPgogICAgPGNpcmNsZSBjeD0iNSIgY3k9IjEyIiByPSIyIi8+CiAgICA8Y2lyY2xlIGN4PSIxMiIgY3k9IjEyIiByPSIyIi8+CiAgICA8Y2lyY2xlIGN4PSIxOSIgY3k9IjEyIiByPSIyIi8+CiAgPC9nPgo8L3N2Zz4K"},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDI0IDI0Ij4KICA8ZyBjbGFzcz0ianAtaWNvbjMiIGZpbGw9IiM2MTYxNjEiPgogICAgPHBhdGggZD0iTTIwLjUgMTFIMTlWN2MwLTEuMS0uOS0yLTItMmgtNFYzLjVDMTMgMi4xMiAxMS44OCAxIDEwLjUgMVM4IDIuMTIgOCAzLjVWNUg0Yy0xLjEgMC0xLjk5LjktMS45OSAydjMuOEgzLjVjMS40OSAwIDIuNyAxLjIxIDIuNyAyLjdzLTEuMjEgMi43LTIuNyAyLjdIMlYyMGMwIDEuMS45IDIgMiAyaDMuOHYtMS41YzAtMS40OSAxLjIxLTIuNyAyLjctMi43IDEuNDkgMCAyLjcgMS4yMSAyLjcgMi43VjIySDE3YzEuMSAwIDItLjkgMi0ydi00aDEuNWMxLjM4IDAgMi41LTEuMTIgMi41LTIuNVMyMS44OCAxMSAyMC41IDExeiIvPgogIDwvZz4KPC9zdmc+Cg=="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIyNCIgaGVpZ2h0PSIyNCIgdmlld0JveD0iMCAwIDI0IDI0Ij4KICAgIDxnIGNsYXNzPSJqcC1pY29uMyIgZmlsbD0iIzYxNjE2MSI+CiAgICAgICAgPHBhdGggZD0iTTQgMThsOC41LTZMNCA2djEyem05LTEydjEybDguNS02TDEzIDZ6Ii8+CiAgICA8L2c+Cjwvc3ZnPgo="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDIyIDIyIj4KICA8cGF0aCBjbGFzcz0ianAtaWNvbjMganAtaWNvbi1zZWxlY3RhYmxlIiBmaWxsPSIjNjE2MTYxIiBkPSJNMTkuMyA4LjJsLTUuNS01LjVjLS4zLS4zLS43LS41LTEuMi0uNUgzLjljLS44LjEtMS42LjktMS42IDEuOHYxNC4xYzAgLjkuNyAxLjYgMS42IDEuNmgxNC4yYy45IDAgMS42LS43IDEuNi0xLjZWOS40Yy4xLS41LS4xLS45LS40LTEuMnptLTUuOC0zLjNsMy40IDMuNmgtMy40VjQuOXptMy45IDEyLjdINC43Yy0uMSAwLS4yIDAtLjItLjJWNC43YzAtLjIuMS0uMy4yLS4zaDcuMnY0LjRzMCAuOC4zIDEuMWMuMy4zIDEuMS4zIDEuMS4zaDQuM3Y3LjJzLS4xLjItLjIuMnoiLz4KPC9zdmc+Cg=="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDI0IDI0Ij4KICA8ZyBjbGFzcz0ianAtaWNvbjMiIGZpbGw9IiM2MTYxNjEiPgogICAgPHBhdGggZD0iTTkgMTZoNnYtNmg0bC03LTctNyA3aDR6bS00IDJoMTR2Mkg1eiIvPgogIDwvZz4KPC9zdmc+Cg=="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDI0IDI0Ij4KICA8ZyBjbGFzcz0ianAtaWNvbjMiIGZpbGw9IiM2MTYxNjEiPgogICAgPHBhdGggZD0iTTEwIDE4aDR2LTJoLTR2MnpNMyA2djJoMThWNkgzem0zIDdoMTJ2LTJINnYyeiIvPgogIDwvZz4KPC9zdmc+Cg=="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDI0IDI0Ij4KICA8cGF0aCBjbGFzcz0ianAtaWNvbjMganAtaWNvbi1zZWxlY3RhYmxlIiBmaWxsPSIjNjE2MTYxIiBkPSJNMTAgNEg0Yy0xLjEgMC0xLjk5LjktMS45OSAyTDIgMThjMCAxLjEuOSAyIDIgMmgxNmMxLjEgMCAyLS45IDItMlY4YzAtMS4xLS45LTItMi0yaC04bC0yLTJ6Ii8+Cjwvc3ZnPgo="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDUxMiA1MTIiPgogIDxwYXRoIGNsYXNzPSJqcC1pY29uMCBqcC1pY29uLXNlbGVjdGFibGUiIGZpbGw9IiMwMDAiIGQ9Ik0xMDguNCAwaDIzdjIyLjhoMjEuMlYwaDIzdjY5aC0yM1Y0NmgtMjF2MjNoLTIzLjJNMjA2IDIzaC0yMC4zVjBoNjMuN3YyM0gyMjl2NDZoLTIzbTUzLjUtNjloMjQuMWwxNC44IDI0LjNMMzEzLjIgMGgyNC4xdjY5aC0yM1YzNC44bC0xNi4xIDI0LjgtMTYuMS0yNC44VjY5aC0yMi42bTg5LjItNjloMjN2NDYuMmgzMi42VjY5aC01NS42Ii8+CiAgPHBhdGggY2xhc3M9ImpwLWljb24tc2VsZWN0YWJsZSIgZmlsbD0iI2U0NGQyNiIgZD0iTTEwNy42IDQ3MWwtMzMtMzcwLjRoMzYyLjhsLTMzIDM3MC4yTDI1NS43IDUxMiIvPgogIDxwYXRoIGNsYXNzPSJqcC1pY29uLXNlbGVjdGFibGUiIGZpbGw9IiNmMTY1MjkiIGQ9Ik0yNTYgNDgwLjVWMTMxaDE0OC4zTDM3NiA0NDciLz4KICA8cGF0aCBjbGFzcz0ianAtaWNvbi1zZWxlY3RhYmxlLWludmVyc2UiIGZpbGw9IiNlYmViZWIiIGQ9Ik0xNDIgMTc2LjNoMTE0djQ1LjRoLTY0LjJsNC4yIDQ2LjVoNjB2NDUuM0gxNTQuNG0yIDIyLjhIMjAybDMuMiAzNi4zIDUwLjggMTMuNnY0Ny40bC05My4yLTI2Ii8+CiAgPHBhdGggY2xhc3M9ImpwLWljb24tc2VsZWN0YWJsZS1pbnZlcnNlIiBmaWxsPSIjZmZmIiBkPSJNMzY5LjYgMTc2LjNIMjU1Ljh2NDUuNGgxMDkuNm0tNC4xIDQ2LjVIMjU1Ljh2NDUuNGg1NmwtNS4zIDU5LTUwLjcgMTMuNnY0Ny4ybDkzLTI1LjgiLz4KPC9zdmc+Cg=="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDIyIDIyIj4KICA8cGF0aCBjbGFzcz0ianAtaWNvbi1icmFuZDQganAtaWNvbi1zZWxlY3RhYmxlLWludmVyc2UiIGZpbGw9IiNGRkYiIGQ9Ik0yLjIgMi4yaDE3LjV2MTcuNUgyLjJ6Ii8+CiAgPHBhdGggY2xhc3M9ImpwLWljb24tYnJhbmQwIGpwLWljb24tc2VsZWN0YWJsZSIgZmlsbD0iIzNGNTFCNSIgZD0iTTIuMiAyLjJ2MTcuNWgxNy41bC4xLTE3LjVIMi4yem0xMi4xIDIuMmMxLjIgMCAyLjIgMSAyLjIgMi4ycy0xIDIuMi0yLjIgMi4yLTIuMi0xLTIuMi0yLjIgMS0yLjIgMi4yLTIuMnpNNC40IDE3LjZsMy4zLTguOCAzLjMgNi42IDIuMi0zLjIgNC40IDUuNEg0LjR6Ii8+Cjwvc3ZnPgo="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDI0IDI0Ij4KICA8cGF0aCBjbGFzcz0ianAtaWNvbjMganAtaWNvbi1zZWxlY3RhYmxlIiBmaWxsPSIjNjE2MTYxIiBkPSJNMjAgNEg0Yy0xLjEgMC0xLjk5LjktMS45OSAyTDIgMThjMCAxLjEuOSAyIDIgMmgxNmMxLjEgMCAyLS45IDItMlY2YzAtMS4xLS45LTItMi0yem0tNSAxNEg0di00aDExdjR6bTAtNUg0VjloMTF2NHptNSA1aC00VjloNHY5eiIvPgo8L3N2Zz4K"},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDIyIDIyIj4KICA8ZyBjbGFzcz0ianAtaWNvbi13YXJuMSBqcC1pY29uLXNlbGVjdGFibGUiIGZpbGw9IiNGOUE4MjUiPgogICAgPHBhdGggZD0iTTIwLjIgMTEuOGMtMS42IDAtMS43LjUtMS43IDEgMCAuNC4xLjkuMSAxLjMuMS41LjEuOS4xIDEuMyAwIDEuNy0xLjQgMi4zLTMuNSAyLjNoLS45di0xLjloLjVjMS4xIDAgMS40IDAgMS40LS44IDAtLjMgMC0uNi0uMS0xIDAtLjQtLjEtLjgtLjEtMS4yIDAtMS4zIDAtMS44IDEuMy0yLTEuMy0uMi0xLjMtLjctMS4zLTIgMC0uNC4xLS44LjEtMS4yLjEtLjQuMS0uNy4xLTEgMC0uOC0uNC0uNy0xLjQtLjhoLS41VjQuMWguOWMyLjIgMCAzLjUuNyAzLjUgMi4zIDAgLjQtLjEuOS0uMSAxLjMtLjEuNS0uMS45LS4xIDEuMyAwIC41LjIgMSAxLjcgMXYxLjh6TTEuOCAxMC4xYzEuNiAwIDEuNy0uNSAxLjctMSAwLS40LS4xLS45LS4xLTEuMy0uMS0uNS0uMS0uOS0uMS0xLjMgMC0xLjYgMS40LTIuMyAzLjUtMi4zaC45djEuOWgtLjVjLTEgMC0xLjQgMC0xLjQuOCAwIC4zIDAgLjYuMSAxIDAgLjIuMS42LjEgMSAwIDEuMyAwIDEuOC0xLjMgMkM2IDExLjIgNiAxMS43IDYgMTNjMCAuNC0uMS44LS4xIDEuMi0uMS4zLS4xLjctLjEgMSAwIC44LjMuOCAxLjQuOGguNXYxLjloLS45Yy0yLjEgMC0zLjUtLjYtMy41LTIuMyAwLS40LjEtLjkuMS0xLjMuMS0uNS4xLS45LjEtMS4zIDAtLjUtLjItMS0xLjctMXYtMS45eiIvPgogICAgPGNpcmNsZSBjeD0iMTEiIGN5PSIxMy44IiByPSIyLjEiLz4KICAgIDxjaXJjbGUgY3g9IjExIiBjeT0iOC4yIiByPSIyLjEiLz4KICA8L2c+Cjwvc3ZnPgo="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB3aWR0aD0iMTUyIiBoZWlnaHQ9IjE2NSIgdmlld0JveD0iMCAwIDE1MiAxNjUiIHZlcnNpb249IjEuMSIgeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj4KICA8ZyBjbGFzcz0ianAtaWNvbi13YXJuMCIgZmlsbD0iI0YzNzcyNiI+CiAgICA8cGF0aCB0cmFuc2Zvcm09InRyYW5zbGF0ZSgwLjA3ODk0NywgMTEwLjU4MjkyNykiIGQ9Ik03NS45NDIyODQyLDI5LjU4MDQ1NjEgQzQzLjMwMjM5NDcsMjkuNTgwNDU2MSAxNC43OTY3ODMyLDE3LjY1MzQ2MzQgMCwwIEM1LjUxMDgzMjExLDE1Ljg0MDY4MjkgMTUuNzgxNTM4OSwyOS41NjY3NzMyIDI5LjM5MDQ5NDcsMzkuMjc4NDE3MSBDNDIuOTk5Nyw0OC45ODk4NTM3IDU5LjI3MzcsNTQuMjA2NzgwNSA3NS45NjA1Nzg5LDU0LjIwNjc4MDUgQzkyLjY0NzQ1NzksNTQuMjA2NzgwNSAxMDguOTIxNDU4LDQ4Ljk4OTg1MzcgMTIyLjUzMDY2MywzOS4yNzg0MTcxIEMxMzYuMTM5NDUzLDI5LjU2Njc3MzIgMTQ2LjQxMDI4NCwxNS44NDA2ODI5IDE1MS45MjExNTgsMCBDMTM3LjA4Nzg2OCwxNy42NTM0NjM0IDEwOC41ODI1ODksMjkuNTgwNDU2MSA3NS45NDIyODQyLDI5LjU4MDQ1NjEgTDc1Ljk0MjI4NDIsMjkuNTgwNDU2MSBaIiAvPgogICAgPHBhdGggdHJhbnNmb3JtPSJ0cmFuc2xhdGUoMC4wMzczNjgsIDAuNzA0ODc4KSIgZD0iTTc1Ljk3ODQ1NzksMjQuNjI2NDA3MyBDMTA4LjYxODc2MywyNC42MjY0MDczIDEzNy4xMjQ0NTgsMzYuNTUzNDQxNSAxNTEuOTIxMTU4LDU0LjIwNjc4MDUgQzE0Ni40MTAyODQsMzguMzY2MjIyIDEzNi4xMzk0NTMsMjQuNjQwMTMxNyAxMjIuNTMwNjYzLDE0LjkyODQ4NzggQzEwOC45MjE0NTgsNS4yMTY4NDM5IDkyLjY0NzQ1NzksMCA3NS45NjA1Nzg5LDAgQzU5LjI3MzcsMCA0Mi45OTk3LDUuMjE2ODQzOSAyOS4zOTA0OTQ3LDE0LjkyODQ4NzggQzE1Ljc4MTUzODksMjQuNjQwMTMxNyA1LjUxMDgzMjExLDM4LjM2NjIyMiAwLDU0LjIwNjc4MDUgQzE0LjgzMzA4MTYsMzYuNTg5OTI5MyA0My4zMzg1Njg0LDI0LjYyNjQwNzMgNzUuOTc4NDU3OSwyNC42MjY0MDczIEw3NS45Nzg0NTc5LDI0LjYyNjQwNzMgWiIgLz4KICA8L2c+Cjwvc3ZnPgo="},function(e,t){e.exports="data:image/svg+xml;base64,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"},function(e,t){e.exports="data:image/svg+xml;base64, "},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDI0IDI0Ij4KICAgIDxwYXRoIGNsYXNzPSJqcC1pY29uMiIgZmlsbD0iIzYxNjE2MSIgZD0iTTE1IDlIOXY2aDZWOXptLTIgNGgtMnYtMmgydjJ6bTgtMlY5aC0yVjdjMC0xLjEtLjktMi0yLTJoLTJWM2gtMnYyaC0yVjNIOXYySDdjLTEuMSAwLTIgLjktMiAydjJIM3YyaDJ2MkgzdjJoMnYyYzAgMS4xLjkgMiAyIDJoMnYyaDJ2LTJoMnYyaDJ2LTJoMmMxLjEgMCAyLS45IDItMnYtMmgydi0yaC0ydi0yaDJ6bS00IDZIN1Y3aDEwdjEweiIvPgo8L3N2Zz4K"},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDI0IDI0Ij4KICA8cGF0aCBjbGFzcz0ianAtaWNvbjMganAtaWNvbi1zZWxlY3RhYmxlIiBmaWxsPSIjNjE2MTYxIiBkPSJNMjAgNUg0Yy0xLjEgMC0xLjk5LjktMS45OSAyTDIgMTdjMCAxLjEuOSAyIDIgMmgxNmMxLjEgMCAyLS45IDItMlY3YzAtMS4xLS45LTItMi0yem0tOSAzaDJ2MmgtMlY4em0wIDNoMnYyaC0ydi0yek04IDhoMnYySDhWOHptMCAzaDJ2Mkg4di0yem0tMSAySDV2LTJoMnYyem0wLTNINVY4aDJ2MnptOSA3SDh2LTJoOHYyem0wLTRoLTJ2LTJoMnYyem0wLTNoLTJWOGgydjJ6bTMgM2gtMnYtMmgydjJ6bTAtM2gtMlY4aDJ2MnoiLz4KPC9zdmc+Cg=="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDI0IDI0Ij4KICA8cGF0aCBjbGFzcz0ianAtaWNvbjMganAtaWNvbi1zZWxlY3RhYmxlIiBmaWxsPSIjNjE2MTYxIiBkPSJNMTkgMTlINVY1aDdWM0g1YTIgMiAwIDAwLTIgMnYxNGEyIDIgMCAwMDIgMmgxNGMxLjEgMCAyLS45IDItMnYtN2gtMnY3ek0xNCAzdjJoMy41OWwtOS44MyA5LjgzIDEuNDEgMS40MUwxOSA2LjQxVjEwaDJWM2gtN3oiLz4KPC9zdmc+Cg=="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDI0IDI0Ij4KICAgIDxwYXRoIGZpbGw9IndoaXRlIiBkPSJNNS44OCA0LjEyTDEzLjc2IDEybC03Ljg4IDcuODhMOCAyMmwxMC0xMEw4IDJ6Ii8+Cjwvc3ZnPgo="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB2aWV3Qm94PSIwIDAgMjQgMjQiIHdpZHRoPSIxNiIgeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj4KICA8ZyBjbGFzcz0ianAtaWNvbjMiIGZpbGw9IiM2MTYxNjEiPgogICAgPHBhdGggZD0iTTMuOSAxMmMwLTEuNzEgMS4zOS0zLjEgMy4xLTMuMWg0VjdIN2MtMi43NiAwLTUgMi4yNC01IDVzMi4yNCA1IDUgNWg0di0xLjlIN2MtMS43MSAwLTMuMS0xLjM5LTMuMS0zLjF6TTggMTNoOHYtMkg4djJ6bTktNmgtNHYxLjloNGMxLjcxIDAgMy4xIDEuMzkgMy4xIDMuMXMtMS4zOSAzLjEtMy4xIDMuMWgtNFYxN2g0YzIuNzYgMCA1LTIuMjQgNS01cy0yLjI0LTUtNS01eiIvPgogIDwvZz4KPC9zdmc+Cg=="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDI0IDI0Ij4KICAgIDxwYXRoIGNsYXNzPSJqcC1pY29uMiBqcC1pY29uLXNlbGVjdGFibGUiIGZpbGw9IiM2MTYxNjEiIGQ9Ik0xOSA1djE0SDVWNWgxNG0xLjEtMkgzLjljLS41IDAtLjkuNC0uOS45djE2LjJjMCAuNC40LjkuOS45aDE2LjJjLjQgMCAuOS0uNS45LS45VjMuOWMwLS41LS41LS45LS45LS45ek0xMSA3aDZ2MmgtNlY3em0wIDRoNnYyaC02di0yem0wIDRoNnYyaC02ek03IDdoMnYySDd6bTAgNGgydjJIN3ptMCA0aDJ2Mkg3eiIvPgo8L3N2Zz4="},function(e,t){e.exports="data:image/svg+xml;base64,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"},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDIyIDIyIj4KICA8cGF0aCBjbGFzcz0ianAtaWNvbi1jb250cmFzdDAganAtaWNvbi1zZWxlY3RhYmxlIiBmaWxsPSIjN0IxRkEyIiBkPSJNNSAxNC45aDEybC02LjEgNnptOS40LTYuOGMwLTEuMy0uMS0yLjktLjEtNC41LS40IDEuNC0uOSAyLjktMS4zIDQuM2wtMS4zIDQuM2gtMkw4LjUgNy45Yy0uNC0xLjMtLjctMi45LTEtNC4zLS4xIDEuNi0uMSAzLjItLjIgNC42TDcgMTIuNEg0LjhsLjctMTFoMy4zTDEwIDVjLjQgMS4yLjcgMi43IDEgMy45LjMtMS4yLjctMi42IDEtMy45bDEuMi0zLjdoMy4zbC42IDExaC0yLjRsLS4zLTQuMnoiLz4KPC9zdmc+Cg=="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDI0IDI0Ij4KICA8ZyBjbGFzcz0ianAtaWNvbjMiIGZpbGw9IiM2MTYxNjEiPgogICAgPHBhdGggZD0iTTIwIDZoLThsLTItMkg0Yy0xLjExIDAtMS45OS44OS0xLjk5IDJMMiAxOGMwIDEuMTEuODkgMiAyIDJoMTZjMS4xMSAwIDItLjg5IDItMlY4YzAtMS4xMS0uODktMi0yLTJ6bS0xIDhoLTN2M2gtMnYtM2gtM3YtMmgzVjloMnYzaDN2MnoiLz4KICA8L2c+Cjwvc3ZnPgo="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyBmaWxsPSJub25lIiB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDI1IDI1Ij4KICAgIDxwYXRoIGNsYXNzPSJqcC1pY29uMiIgc3Ryb2tlPSIjMzMzMzMzIiBzdHJva2Utd2lkdGg9IjIiIHRyYW5zZm9ybT0idHJhbnNsYXRlKDMgMykiIGQ9Ik0xLjg2MDk0IDExLjQ0MDlDMC44MjY0NDggOC43NzAyNyAwLjg2Mzc3OSA2LjA1NzY0IDEuMjQ5MDcgNC4xOTkzMkMyLjQ4MjA2IDMuOTMzNDcgNC4wODA2OCAzLjQwMzQ3IDUuNjAxMDIgMi44NDQ5QzcuMjM1NDkgMi4yNDQ0IDguODU2NjYgMS41ODE1IDkuOTg3NiAxLjA5NTM5QzExLjA1OTcgMS41ODM0MSAxMi42MDk0IDIuMjQ0NCAxNC4yMTggMi44NDMzOUMxNS43NTAzIDMuNDEzOTQgMTcuMzk5NSAzLjk1MjU4IDE4Ljc1MzkgNC4yMTM4NUMxOS4xMzY0IDYuMDcxNzcgMTkuMTcwOSA4Ljc3NzIyIDE4LjEzOSAxMS40NDA5QzE3LjAzMDMgMTQuMzAzMiAxNC42NjY4IDE3LjE4NDQgOS45OTk5OSAxOC45MzU0QzUuMzMzMTkgMTcuMTg0NCAyLjk2OTY4IDE0LjMwMzIgMS44NjA5NCAxMS40NDA5WiIvPgogICAgPHBhdGggY2xhc3M9ImpwLWljb24yIiBzdHJva2U9IiMzMzMzMzMiIHN0cm9rZS13aWR0aD0iMiIgdHJhbnNmb3JtPSJ0cmFuc2xhdGUoOS4zMTU5MiA5LjMyMDMxKSIgZD0iTTcuMzY4NDIgMEwwIDcuMzY0NzkiLz4KICAgIDxwYXRoIGNsYXNzPSJqcC1pY29uMiIgc3Ryb2tlPSIjMzMzMzMzIiBzdHJva2Utd2lkdGg9IjIiIHRyYW5zZm9ybT0idHJhbnNsYXRlKDkuMzE1OTIgMTYuNjgzNikgc2NhbGUoMSAtMSkiIGQ9Ik03LjM2ODQyIDBMMCA3LjM2NDc5Ii8+Cjwvc3ZnPgo="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDIyIDIyIj4KICA8ZyBjbGFzcz0ianAtaWNvbi13YXJuMCBqcC1pY29uLXNlbGVjdGFibGUiIGZpbGw9IiNFRjZDMDAiPgogICAgPHBhdGggZD0iTTE4LjcgMy4zdjE1LjRIMy4zVjMuM2gxNS40bTEuNS0xLjVIMS44djE4LjNoMTguM2wuMS0xOC4zeiIvPgogICAgPHBhdGggZD0iTTE2LjUgMTYuNWwtNS40LTQuMy01LjYgNC4zdi0xMWgxMXoiLz4KICA8L2c+Cjwvc3ZnPgo="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB3aWR0aD0iMjIiIGhlaWdodD0iMjIiIHZpZXdCb3g9IjAgMCAyOCAyOCIgeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj4KCTxnIGNsYXNzPSJqcC1pY29uMyIgZmlsbD0iIzYxNjE2MSI+CgkJPHBhdGggZD0iTTQgMTlINlYxOS41SDVWMjAuNUg2VjIxSDRWMjJIN1YxOEg0VjE5Wk01IDEwSDZWNkg0VjdINVYxMFpNNCAxM0g1LjhMNCAxNS4xVjE2SDdWMTVINS4yTDcgMTIuOVYxMkg0VjEzWk05IDdWOUgyM1Y3SDlaTTkgMjFIMjNWMTlIOVYyMVpNOSAxNUgyM1YxM0g5VjE1WiIvPgoJPC9nPgo8L3N2Zz4K"},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHZpZXdCb3g9IjAgMCAyNCAyNCIgd2lkdGg9IjE2Ij4KICA8ZyBjbGFzcz0ianAtaWNvbjMiIGZpbGw9IiM2MTYxNjEiPgogICAgPHBhdGggZD0iTTEyIDIuMDJjLTUuNTEgMC05Ljk4IDQuNDctOS45OCA5Ljk4czQuNDcgOS45OCA5Ljk4IDkuOTggOS45OC00LjQ3IDkuOTgtOS45OFMxNy41MSAyLjAyIDEyIDIuMDJ6TTExLjQ4IDIwdi02LjI2SDhMMTMgNHY2LjI2aDMuMzVMMTEuNDggMjB6Ii8+CiAgPC9nPgo8L3N2Zz4K"},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDI0IDI0Ij4KICA8ZyBjbGFzcz0ianAtaWNvbjMiIGZpbGw9IiM2MTYxNjEiPgogICAgPHBhdGggZD0iTTE4IDEzVjIwSDRWNkg5LjAyQzkuMDcgNS4yOSA5LjI0IDQuNjIgOS41IDRINEMyLjkgNCAyIDQuOSAyIDZWMjBDMiAyMS4xIDIuOSAyMiA0IDIySDE4QzE5LjEgMjIgMjAgMjEuMSAyMCAyMFYxNUwxOCAxM1pNMTkuMyA4Ljg5QzE5Ljc0IDguMTkgMjAgNy4zOCAyMCA2LjVDMjAgNC4wMSAxNy45OSAyIDE1LjUgMkMxMy4wMSAyIDExIDQuMDEgMTEgNi41QzExIDguOTkgMTMuMDEgMTEgMTUuNDkgMTFDMTYuMzcgMTEgMTcuMTkgMTAuNzQgMTcuODggMTAuM0wyMSAxMy40MkwyMi40MiAxMkwxOS4zIDguODlaTTE1LjUgOUMxNC4xMiA5IDEzIDcuODggMTMgNi41QzEzIDUuMTIgMTQuMTIgNCAxNS41IDRDMTYuODggNCAxOCA1LjEyIDE4IDYuNUMxOCA3Ljg4IDE2Ljg4IDkgMTUuNSA5WiIvPgogICAgPHBhdGggZmlsbC1ydWxlPSJldmVub2RkIiBjbGlwLXJ1bGU9ImV2ZW5vZGQiIGQ9Ik00IDZIOS4wMTg5NEM5LjAwNjM5IDYuMTY1MDIgOSA2LjMzMTc2IDkgNi41QzkgOC44MTU3NyAxMC4yMTEgMTAuODQ4NyAxMi4wMzQzIDEySDlWMTRIMTZWMTIuOTgxMUMxNi41NzAzIDEyLjkzNzcgMTcuMTIgMTIuODIwNyAxNy42Mzk2IDEyLjYzOTZMMTggMTNWMjBINFY2Wk04IDhINlYxMEg4VjhaTTYgMTJIOFYxNEg2VjEyWk04IDE2SDZWMThIOFYxNlpNOSAxNkgxNlYxOEg5VjE2WiIvPgogIDwvZz4KPC9zdmc+Cg=="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyBoZWlnaHQ9IjI0IiB2aWV3Qm94PSIwIDAgMjQgMjQiIHdpZHRoPSIyNCIgeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj4KICAgIDxnIGNsYXNzPSJqcC1pY29uMyIgZmlsbD0iIzYxNjE2MSI+CiAgICAgICAgPHBhdGggZD0iTTE5IDJoLTQuMThDMTQuNC44NCAxMy4zIDAgMTIgMGMtMS4zIDAtMi40Ljg0LTIuODIgMkg1Yy0xLjEgMC0yIC45LTIgMnYxNmMwIDEuMS45IDIgMiAyaDE0YzEuMSAwIDItLjkgMi0yVjRjMC0xLjEtLjktMi0yLTJ6bS03IDBjLjU1IDAgMSAuNDUgMSAxcy0uNDUgMS0xIDEtMS0uNDUtMS0xIC40NS0xIDEtMXptNyAxOEg1VjRoMnYzaDEwVjRoMnYxNnoiLz4KICAgIDwvZz4KPC9zdmc+Cg=="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZwogICB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHZpZXdCb3g9IjAgMCAyMiAyMiIgd2lkdGg9IjE2Ij4KICAgIDxwYXRoIHRyYW5zZm9ybT0icm90YXRlKDQ1KSIgY2xhc3M9ImpwLWljb24tc2VsZWN0YWJsZSIgZmlsbD0iI0ZGMkEyQSIKICAgICAgIGQ9Im0gMjIuMzQ0MzY5LC0zLjAxNjM2NDIgaCA1LjYzODYwNCB2IDEuNTc5MjQzMyBoIC0zLjU0OTIyNyB2IDEuNTA4NjkyOTkgaCAzLjMzNzU3NiBWIDEuNjUwODE1NCBoIC0zLjMzNzU3NiB2IDMuNDM1MjYxMyBoIC0yLjA4OTM3NyB6IG0gLTcuMTM2NDQ0LDEuNTc5MjQzMyB2IDQuOTQzOTU0MyBoIDAuNzQ4OTIgcSAxLjI4MDc2MSwwIDEuOTUzNzAzLC0wLjYzNDk1MzUgMC42NzgzNjksLTAuNjM0OTUzNSAwLjY3ODM2OSwtMS44NDUxNjQxIDAsLTEuMjA0NzgzNTUgLTAuNjcyOTQyLC0xLjgzNDMxMDExIC0wLjY3Mjk0MiwtMC42Mjk1MjY1OSAtMS45NTkxMywtMC42Mjk1MjY1OSB6IG0gLTIuMDg5Mzc3LC0xLjU3OTI0MzMgaCAyLjIwMzM0MyBxIDEuODQ1MTY0LDAgMi43NDYwMzksMC4yNjU5MjA3IDAuOTA2MzAxLDAuMjYwNDkzNyAxLjU1MjEwOCwwLjg5MDAyMDMgMC41Njk4MywwLjU0ODEyMjMgMC44NDY2MDUsMS4yNjQ0ODAwNiAwLjI3Njc3NCwwLjcxNjM1NzgxIDAuMjc2Nzc0LDEuNjIyNjU4OTQgMCwwLjkxNzE1NTEgLTAuMjc2Nzc0LDEuNjM4OTM5OSAtMC4yNzY3NzUsMC43MTYzNTc4IC0wLjg0NjYwNSwxLjI2NDQ4IC0wLjY1MTIzNCwwLjYyOTUyNjYgLTEuNTYyOTYyLDAuODk1NDQ3MyAtMC45MTE3MjgsMC4yNjA0OTM3IC0yLjczNTE4NSwwLjI2MDQ5MzcgaCAtMi4yMDMzNDMgeiBtIC04LjE0NTg1NjUsMCBoIDMuNDY3ODIzIHEgMS41NDY2ODE2LDAgMi4zNzE1Nzg1LDAuNjg5MjIzIDAuODMwMzI0LDAuNjgzNzk2MSAwLjgzMDMyNCwxLjk1MzcwMzE0IDAsMS4yNzUzMzM5NyAtMC44MzAzMjQsMS45NjQ1NTcwNiBRIDkuOTg3MTk2MSwyLjI3NDkxNSA4LjQ0MDUxNDUsMi4yNzQ5MTUgSCA3LjA2MjA2ODQgViA1LjA4NjA3NjcgSCA0Ljk3MjY5MTUgWiBtIDIuMDg5Mzc2OSwxLjUxNDExOTkgdiAyLjI2MzAzOTQzIGggMS4xNTU5NDEgcSAwLjYwNzgxODgsMCAwLjkzODg2MjksLTAuMjkzMDU1NDcgMC4zMzEwNDQxLC0wLjI5ODQ4MjQxIDAuMzMxMDQ0MSwtMC44NDExNzc3MiAwLC0wLjU0MjY5NTMxIC0wLjMzMTA0NDEsLTAuODM1NzUwNzQgLTAuMzMxMDQ0MSwtMC4yOTMwNTU1IC0wLjkzODg2MjksLTAuMjkzMDU1NSB6IgovPgo8L3N2Zz4K"},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDIyIDIyIj4KICA8ZyBjbGFzcz0ianAtaWNvbi1icmFuZDAganAtaWNvbi1zZWxlY3RhYmxlIiBmaWxsPSIjMEQ0N0ExIj4KICAgIDxwYXRoIGQ9Ik0xMS4xIDYuOVY1LjhINi45YzAtLjUgMC0xLjMuMi0xLjYuNC0uNy44LTEuMSAxLjctMS40IDEuNy0uMyAyLjUtLjMgMy45LS4xIDEgLjEgMS45LjkgMS45IDEuOXY0LjJjMCAuNS0uOSAxLjYtMiAxLjZIOC44Yy0xLjUgMC0yLjQgMS40LTIuNCAyLjh2Mi4ySDQuN0MzLjUgMTUuMSAzIDE0IDMgMTMuMVY5Yy0uMS0xIC42LTIgMS44LTIgMS41LS4xIDYuMy0uMSA2LjMtLjF6Ii8+CiAgICA8cGF0aCBkPSJNMTAuOSAxNS4xdjEuMWg0LjJjMCAuNSAwIDEuMy0uMiAxLjYtLjQuNy0uOCAxLjEtMS43IDEuNC0xLjcuMy0yLjUuMy0zLjkuMS0xLS4xLTEuOS0uOS0xLjktMS45di00LjJjMC0uNS45LTEuNiAyLTEuNmgzLjhjMS41IDAgMi40LTEuNCAyLjQtMi44VjYuNmgxLjdDMTguNSA2LjkgMTkgOCAxOSA4LjlWMTNjMCAxLS43IDIuMS0xLjkgMi4xaC02LjJ6Ii8+CiAgPC9nPgo8L3N2Zz4K"},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDIyIDIyIj4KICA8cGF0aCBjbGFzcz0ianAtaWNvbi1jb250cmFzdDMganAtaWNvbi1zZWxlY3RhYmxlIiBmaWxsPSIjMjE5NkYzIiBkPSJNNC40IDIuNWMxLjItLjEgMi45LS4zIDQuOS0uMyAyLjUgMCA0LjEuNCA1LjIgMS4zIDEgLjcgMS41IDEuOSAxLjUgMy41IDAgMi0xLjQgMy41LTIuOSA0LjEgMS4yLjQgMS43IDEuNiAyLjIgMyAuNiAxLjkgMSAzLjkgMS4zIDQuNmgtMy44Yy0uMy0uNC0uOC0xLjctMS4yLTMuN3MtMS4yLTIuNi0yLjYtMi42aC0uOXY2LjRINC40VjIuNXptMy43IDYuOWgxLjRjMS45IDAgMi45LS45IDIuOS0yLjNzLTEtMi4zLTIuOC0yLjNjLS43IDAtMS4zIDAtMS42LjJ2NC41aC4xdi0uMXoiLz4KPC9zdmc+Cg=="},function(e,t){e.exports="data:image/svg+xml;base64,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"},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIGhlaWdodD0iMjQiIHZpZXdCb3g9IjAgMCAyNCAyNCIgd2lkdGg9IjE2Ij4KICA8ZyBjbGFzcz0ianAtaWNvbjMiIGZpbGw9IiM2MTYxNjEiPgogICAgICA8cGF0aCBkPSJNMCAwaDI0djI0SDB6IiBmaWxsPSJub25lIi8+PHBhdGggZD0iTTE4LjQgMTAuNkMxNi41NSA4Ljk5IDE0LjE1IDggMTEuNSA4Yy00LjY1IDAtOC41OCAzLjAzLTkuOTYgNy4yMkwzLjkgMTZjMS4wNS0zLjE5IDQuMDUtNS41IDcuNi01LjUgMS45NSAwIDMuNzMuNzIgNS4xMiAxLjg4TDEzIDE2aDlWN2wtMy42IDMuNnoiLz4KICA8L2c+Cjwvc3ZnPgo="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDIwIDIwIj4KICA8ZyBjbGFzcz0ianAtaWNvbjIiIGZpbGw9IiM0MTQxNDEiPgogICAgPHJlY3QgeD0iMiIgeT0iMiIgd2lkdGg9IjE2IiBoZWlnaHQ9IjE2Ii8+CiAgPC9nPgoKICA8ZyBjbGFzcz0ianAtaWNvbi1hY2NlbnQyIiBmaWxsPSIjRkZGIj4KICAgIDxjaXJjbGUgY2xhc3M9InN0MiIgY3g9IjUuNSIgY3k9IjE0LjUiIHI9IjEuNSIvPgogICAgPHJlY3QgeD0iMTIiIHk9IjQiIGNsYXNzPSJzdDIiIHdpZHRoPSIxIiBoZWlnaHQ9IjgiLz4KICAgIDxyZWN0IHg9IjguNSIgeT0iNy41IiB0cmFuc2Zvcm09Im1hdHJpeCgwLjg2NiAtMC41IDAuNSAwLjg2NiAtMi4zMjU1IDcuMzIxOSkiIGNsYXNzPSJzdDIiIHdpZHRoPSI4IiBoZWlnaHQ9IjEiLz4KICAgIDxyZWN0IHg9IjEyIiB5PSI0IiB0cmFuc2Zvcm09Im1hdHJpeCgwLjUgLTAuODY2IDAuODY2IDAuNSAtMC42Nzc5IDE0LjgyNTIpIiBjbGFzcz0ic3QyIiB3aWR0aD0iMSIgaGVpZ2h0PSI4Ii8+CiAgPC9nPgo8L3N2Zz4K"},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyBoZWlnaHQ9IjI0IiB2aWV3Qm94PSIwIDAgMjQgMjQiIHdpZHRoPSIyNCIgeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj4KICAgIDxnIGNsYXNzPSJqcC1pY29uMyIgZmlsbD0iIzYxNjE2MSI+CiAgICAgICAgPHBhdGggZD0iTTggNXYxNGwxMS03eiIvPgogICAgPC9nPgo8L3N2Zz4K"},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDUxMiA1MTIiPgogIDxnIGNsYXNzPSJqcC1pY29uMyIgZmlsbD0iIzYxNjE2MSI+CiAgICA8cGF0aCBkPSJNMjU2IDhDMTE5IDggOCAxMTkgOCAyNTZzMTExIDI0OCAyNDggMjQ4IDI0OC0xMTEgMjQ4LTI0OFMzOTMgOCAyNTYgOHptOTYgMzI4YzAgOC44LTcuMiAxNi0xNiAxNkgxNzZjLTguOCAwLTE2LTcuMi0xNi0xNlYxNzZjMC04LjggNy4yLTE2IDE2LTE2aDE2MGM4LjggMCAxNiA3LjIgMTYgMTZ2MTYweiIvPgogIDwvZz4KPC9zdmc+Cg=="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyBoZWlnaHQ9IjI0IiB2aWV3Qm94PSIwIDAgMjQgMjQiIHdpZHRoPSIyNCIgeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj4KICAgIDxnIGNsYXNzPSJqcC1pY29uMyIgZmlsbD0iIzYxNjE2MSI+CiAgICAgICAgPHBhdGggZD0iTTE3IDNINWMtMS4xMSAwLTIgLjktMiAydjE0YzAgMS4xLjg5IDIgMiAyaDE0YzEuMSAwIDItLjkgMi0yVjdsLTQtNHptLTUgMTZjLTEuNjYgMC0zLTEuMzQtMy0zczEuMzQtMyAzLTMgMyAxLjM0IDMgMy0xLjM0IDMtMyAzem0zLTEwSDVWNWgxMHY0eiIvPgogICAgPC9nPgo8L3N2Zz4K"},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB2aWV3Qm94PSIwIDAgMTggMTgiIHdpZHRoPSIxNiIgeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj4KICA8ZyBjbGFzcz0ianAtaWNvbjMiIGZpbGw9IiM2MTYxNjEiPgogICAgPHBhdGggZD0iTTEyLjEsMTAuOWgtMC43bC0wLjItMC4yYzAuOC0wLjksMS4zLTIuMiwxLjMtMy41YzAtMy0yLjQtNS40LTUuNC01LjRTMS44LDQuMiwxLjgsNy4xczIuNCw1LjQsNS40LDUuNCBjMS4zLDAsMi41LTAuNSwzLjUtMS4zbDAuMiwwLjJ2MC43bDQuMSw0LjFsMS4yLTEuMkwxMi4xLDEwLjl6IE03LjEsMTAuOWMtMi4xLDAtMy43LTEuNy0zLjctMy43czEuNy0zLjcsMy43LTMuN3MzLjcsMS43LDMuNywzLjcgUzkuMiwxMC45LDcuMSwxMC45eiIvPgogIDwvZz4KPC9zdmc+Cg=="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDI0IDI0Ij4KICA8cGF0aCBjbGFzcz0ianAtaWNvbjMganAtaWNvbi1zZWxlY3RhYmxlIiBmaWxsPSIjNjE2MTYxIiBkPSJNMTkuNDMgMTIuOThjLjA0LS4zMi4wNy0uNjQuMDctLjk4cy0uMDMtLjY2LS4wNy0uOThsMi4xMS0xLjY1Yy4xOS0uMTUuMjQtLjQyLjEyLS42NGwtMi0zLjQ2Yy0uMTItLjIyLS4zOS0uMy0uNjEtLjIybC0yLjQ5IDFjLS41Mi0uNC0xLjA4LS43My0xLjY5LS45OGwtLjM4LTIuNjVBLjQ4OC40ODggMCAwMDE0IDJoLTRjLS4yNSAwLS40Ni4xOC0uNDkuNDJsLS4zOCAyLjY1Yy0uNjEuMjUtMS4xNy41OS0xLjY5Ljk4bC0yLjQ5LTFjLS4yMy0uMDktLjQ5IDAtLjYxLjIybC0yIDMuNDZjLS4xMy4yMi0uMDcuNDkuMTIuNjRsMi4xMSAxLjY1Yy0uMDQuMzItLjA3LjY1LS4wNy45OHMuMDMuNjYuMDcuOThsLTIuMTEgMS42NWMtLjE5LjE1LS4yNC40Mi0uMTIuNjRsMiAzLjQ2Yy4xMi4yMi4zOS4zLjYxLjIybDIuNDktMWMuNTIuNCAxLjA4LjczIDEuNjkuOThsLjM4IDIuNjVjLjAzLjI0LjI0LjQyLjQ5LjQyaDRjLjI1IDAgLjQ2LS4xOC40OS0uNDJsLjM4LTIuNjVjLjYxLS4yNSAxLjE3LS41OSAxLjY5LS45OGwyLjQ5IDFjLjIzLjA5LjQ5IDAgLjYxLS4yMmwyLTMuNDZjLjEyLS4yMi4wNy0uNDktLjEyLS42NGwtMi4xMS0xLjY1ek0xMiAxNS41Yy0xLjkzIDAtMy41LTEuNTctMy41LTMuNXMxLjU3LTMuNSAzLjUtMy41IDMuNSAxLjU3IDMuNSAzLjUtMS41NyAzLjUtMy41IDMuNXoiLz4KPC9zdmc+Cg=="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDIyIDIyIj4KICA8cGF0aCBjbGFzcz0ianAtaWNvbi1jb250cmFzdDEganAtaWNvbi1zZWxlY3RhYmxlIiBmaWxsPSIjNENBRjUwIiBkPSJNMi4yIDIuMnYxNy42aDE3LjZWMi4ySDIuMnptMTUuNCA3LjdoLTUuNVY0LjRoNS41djUuNXpNOS45IDQuNHY1LjVINC40VjQuNGg1LjV6bS01LjUgNy43aDUuNXY1LjVINC40di01LjV6bTcuNyA1LjV2LTUuNWg1LjV2NS41aC01LjV6Ii8+Cjwvc3ZnPgo="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyBoZWlnaHQ9IjI0IiB2aWV3Qm94PSIwIDAgMjQgMjQiIHdpZHRoPSIyNCIgeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj4KICAgIDxnIGNsYXNzPSJqcC1pY29uMyIgZmlsbD0iIzYxNjE2MSI+CiAgICAgICAgPHBhdGggZD0iTTAgMGgyNHYyNEgweiIgZmlsbD0ibm9uZSIvPgogICAgICAgIDxwYXRoIGQ9Ik02IDZoMTJ2MTJINnoiLz4KICAgIDwvZz4KPC9zdmc+Cg=="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDI0IDI0Ij4KICA8ZyBjbGFzcz0ianAtaWNvbjMiIGZpbGw9IiM2MTYxNjEiPgogICAgPHBhdGggZD0iTTIxIDNIM2MtMS4xIDAtMiAuOS0yIDJ2MTRjMCAxLjEuOSAyIDIgMmgxOGMxLjEgMCAyLS45IDItMlY1YzAtMS4xLS45LTItMi0yem0wIDE2SDNWNWgxMHY0aDh2MTB6Ii8+CiAgPC9nPgo8L3N2Zz4K"},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyBoZWlnaHQ9IjI0IiB2aWV3Qm94PSIwIDAgMjQgMjQiIHdpZHRoPSIyNCIgeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj4KICAgIDxnIGNsYXNzPSJqcC1pY29uMyIgZmlsbD0iIzYxNjE2MSI+CiAgICAgICAgPHBhdGggZD0iTTAgMGgyNHYyNEgweiIgZmlsbD0ibm9uZSIvPgogICAgICAgIDxwYXRoIGQ9Ik0yMSw4SDNWNGgxOFY4eiBNMjEsMTBIM3Y0aDE4VjEweiBNMjEsMTZIM3Y0aDE4VjE2eiIvPgogICAgPC9nPgo8L3N2Zz4="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB3aWR0aD0iMjgiIGhlaWdodD0iMjgiIHZpZXdCb3g9IjAgMCA0MyAyOCIgeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj4KCTxnIGNsYXNzPSJqcC1pY29uMyIgZmlsbD0iIzYxNjE2MSI+CgkJPHBhdGggZD0iTTI4LjgzMzIgMTIuMzM0TDMyLjk5OTggMTYuNTAwN0wzNy4xNjY1IDEyLjMzNEgyOC44MzMyWiIvPgoJCTxwYXRoIGQ9Ik0xNi4yMDk1IDIxLjYxMDRDMTUuNjg3MyAyMi4xMjk5IDE0Ljg0NDMgMjIuMTI5OSAxNC4zMjQ4IDIxLjYxMDRMNi45ODI5IDE0LjcyNDVDNi41NzI0IDE0LjMzOTQgNi4wODMxMyAxMy42MDk4IDYuMDQ3ODYgMTMuMDQ4MkM1Ljk1MzQ3IDExLjUyODggNi4wMjAwMiA4LjYxOTQ0IDYuMDY2MjEgNy4wNzY5NUM2LjA4MjgxIDYuNTE0NzcgNi41NTU0OCA2LjA0MzQ3IDcuMTE4MDQgNi4wMzA1NUM5LjA4ODYzIDUuOTg0NzMgMTMuMjYzOCA1LjkzNTc5IDEzLjY1MTggNi4zMjQyNUwyMS43MzY5IDEzLjYzOUMyMi4yNTYgMTQuMTU4NSAyMS43ODUxIDE1LjQ3MjQgMjEuMjYyIDE1Ljk5NDZMMTYuMjA5NSAyMS42MTA0Wk05Ljc3NTg1IDguMjY1QzkuMzM1NTEgNy44MjU2NiA4LjYyMzUxIDcuODI1NjYgOC4xODI4IDguMjY1QzcuNzQzNDYgOC43MDU3MSA3Ljc0MzQ2IDkuNDE3MzMgOC4xODI4IDkuODU2NjdDOC42MjM4MiAxMC4yOTY0IDkuMzM1ODIgMTAuMjk2NCA5Ljc3NTg1IDkuODU2NjdDMTAuMjE1NiA5LjQxNzMzIDEwLjIxNTYgOC43MDUzMyA5Ljc3NTg1IDguMjY1WiIvPgoJPC9nPgo8L3N2Zz4K"},function(e,t){e.exports="data:image/svg+xml;base64,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"},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDI0IDI0Ij4KICA8cGF0aCBjbGFzcz0ianAtaWNvbjMganAtaWNvbi1zZWxlY3RhYmxlIiBmaWxsPSIjNjE2MTYxIiBkPSJNMTUgMTVIM3YyaDEydi0yem0wLThIM3YyaDEyVjd6TTMgMTNoMTh2LTJIM3Yyem0wIDhoMTh2LTJIM3Yyek0zIDN2MmgxOFYzSDN6Ii8+Cjwvc3ZnPgo="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHhtbG5zOnhsaW5rPSJodHRwOi8vd3d3LnczLm9yZy8xOTk5L3hsaW5rIiB2ZXJzaW9uPSIxLjEiIHdpZHRoPSIyNCIgaGVpZ2h0PSIyNCIgdmlld0JveD0iMCAwIDI0IDI0IiBjbGFzcz0ianAtaWNvbjMiIGZpbGw9IiM2MTYxNjEiPgoJPHBhdGggZD0iTTcsNUgyMVY3SDdWNU03LDEzVjExSDIxVjEzSDdNNCw0LjVBMS41LDEuNSAwIDAsMSA1LjUsNkExLjUsMS41IDAgMCwxIDQsNy41QTEuNSwxLjUgMCAwLDEgMi41LDZBMS41LDEuNSAwIDAsMSA0LDQuNU00LDEwLjVBMS41LDEuNSAwIDAsMSA1LjUsMTJBMS41LDEuNSAwIDAsMSA0LDEzLjVBMS41LDEuNSAwIDAsMSAyLjUsMTJBMS41LDEuNSAwIDAsMSA0LDEwLjVNNywxOVYxN0gyMVYxOUg3TTQsMTYuNUExLjUsMS41IDAgMCwxIDUuNSwxOEExLjUsMS41IDAgMCwxIDQsMTkuNUExLjUsMS41IDAgMCwxIDIuNSwxOEExLjUsMS41IDAgMCwxIDQsMTYuNVoiIC8+Cjwvc3ZnPgo="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyBoZWlnaHQ9IjI0IiB2aWV3Qm94PSIwIDAgMjQgMjQiIHdpZHRoPSIyNCIgeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj4KICAgIDxnIGNsYXNzPSJqcC1pY29uMyIgZmlsbD0iIzYxNjE2MSI+CiAgICAgICAgPHBhdGggZD0iTTAgMGgyNHYyNEgweiIgZmlsbD0ibm9uZSIvPgogICAgICAgIDxwYXRoIGQ9Ik0yMiAxMVYzaC03djNIOVYzSDJ2OGg3VjhoMnYxMGg0djNoN3YtOGgtN3YzaC0yVjhoMnYzeiIvPgogICAgPC9nPgo8L3N2Zz4="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyBmaWxsPSJub25lIiB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDI0IDI1Ij4KICAgIDxwYXRoIGNsYXNzPSJqcC1pY29uMiIgc3Ryb2tlPSIjMzMzMzMzIiBzdHJva2Utd2lkdGg9IjIiIHRyYW5zZm9ybT0idHJhbnNsYXRlKDIgMykiIGQ9Ik0xLjg2MDk0IDExLjQ0MDlDMC44MjY0NDggOC43NzAyNyAwLjg2Mzc3OSA2LjA1NzY0IDEuMjQ5MDcgNC4xOTkzMkMyLjQ4MjA2IDMuOTMzNDcgNC4wODA2OCAzLjQwMzQ3IDUuNjAxMDIgMi44NDQ5QzcuMjM1NDkgMi4yNDQ0IDguODU2NjYgMS41ODE1IDkuOTg3NiAxLjA5NTM5QzExLjA1OTcgMS41ODM0MSAxMi42MDk0IDIuMjQ0NCAxNC4yMTggMi44NDMzOUMxNS43NTAzIDMuNDEzOTQgMTcuMzk5NSAzLjk1MjU4IDE4Ljc1MzkgNC4yMTM4NUMxOS4xMzY0IDYuMDcxNzcgMTkuMTcwOSA4Ljc3NzIyIDE4LjEzOSAxMS40NDA5QzE3LjAzMDMgMTQuMzAzMiAxNC42NjY4IDE3LjE4NDQgOS45OTk5OSAxOC45MzU0QzUuMzMzMiAxNy4xODQ0IDIuOTY5NjggMTQuMzAzMiAxLjg2MDk0IDExLjQ0MDlaIi8+CiAgICA8cGF0aCBjbGFzcz0ianAtaWNvbjIiIGZpbGw9IiMzMzMzMzMiIHN0cm9rZT0iIzMzMzMzMyIgdHJhbnNmb3JtPSJ0cmFuc2xhdGUoOCA5Ljg2NzE5KSIgZD0iTTIuODYwMTUgNC44NjUzNUwwLjcyNjU0OSAyLjk5OTU5TDAgMy42MzA0NUwyLjg2MDE1IDYuMTMxNTdMOCAwLjYzMDg3Mkw3LjI3ODU3IDBMMi44NjAxNSA0Ljg2NTM1WiIvPgo8L3N2Zz4K"},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB2aWV3Qm94PSIwIDAgMjQgMjQiIHdpZHRoPSIxNiIgeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj4KICA8ZyBjbGFzcz0ianAtaWNvbjMiIGZpbGw9IiM2MTYxNjEiPgogICAgPHBhdGggZD0iTTEyLjUgOGMtMi42NSAwLTUuMDUuOTktNi45IDIuNkwyIDd2OWg5bC0zLjYyLTMuNjJjMS4zOS0xLjE2IDMuMTYtMS44OCA1LjEyLTEuODggMy41NCAwIDYuNTUgMi4zMSA3LjYgNS41bDIuMzctLjc4QzIxLjA4IDExLjAzIDE3LjE1IDggMTIuNSA4eiIvPgogIDwvZz4KPC9zdmc+Cg=="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDIyIDIyIj4KICA8ZyBjbGFzcz0ianAtaWNvbjEganAtaWNvbi1zZWxlY3RhYmxlIiBmaWxsPSIjMjEyMTIxIj4KICAgIDxwYXRoIGQ9Ik0xMC42IDUuNGwyLjItMy4ySDIuMnY3LjNsNC02LjZ6Ii8+CiAgICA8cGF0aCBkPSJNMTUuOCAyLjJsLTQuNCA2LjZMNyA2LjNsLTQuOCA4djUuNWgxNy42VjIuMmgtNHptLTcgMTUuNEg1LjV2LTQuNGgzLjN2NC40em00LjQgMEg5LjhWOS44aDMuNHY3Ljh6bTQuNCAwaC0zLjRWNi41aDMuNHYxMS4xeiIvPgogIDwvZz4KPC9zdmc+Cg=="},function(e,t){e.exports="data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgdmlld0JveD0iMCAwIDIyIDIyIj4KICA8ZyBjbGFzcz0ianAtaWNvbi1jb250cmFzdDIganAtaWNvbi1zZWxlY3RhYmxlIiBmaWxsPSIjRDgxQjYwIj4KICAgIDxwYXRoIGQ9Ik03LjIgMTguNnYtNS40TDMgNS42aDMuM2wxLjQgMy4xYy4zLjkuNiAxLjYgMSAyLjUuMy0uOC42LTEuNiAxLTIuNWwxLjQtMy4xaDMuNGwtNC40IDcuNnY1LjVsLTIuOS0uMXoiLz4KICAgIDxjaXJjbGUgY2xhc3M9InN0MCIgY3g9IjE3LjYiIGN5PSIxNi41IiByPSIyLjEiLz4KICAgIDxjaXJjbGUgY2xhc3M9InN0MCIgY3g9IjE3LjYiIGN5PSIxMSIgcj0iMi4xIi8+CiAgPC9nPgo8L3N2Zz4K"},function(e,t,n){(function(t){var n=/[&<>"'`]/g,r=RegExp(n.source),i="object"==typeof t&&t&&t.Object===Object&&t,o="object"==typeof self&&self&&self.Object===Object&&self,a=i||o||Function("return this")();var s,l=(s={"&":"&amp;","<":"&lt;",">":"&gt;",'"':"&quot;","'":"&#39;","`":"&#96;"},function(e){return null==s?void 0:s[e]}),u=Object.prototype.toString,c=a.Symbol,d=c?c.prototype:void 0,h=d?d.toString:void 0;function f(e){if("string"==typeof e)return e;if(function(e){return"symbol"==typeof e||function(e){return!!e&&"object"==typeof e}(e)&&"[object Symbol]"==u.call(e)}(e))return h?h.call(e):"";var t=e+"";return"0"==t&&1/e==-1/0?"-0":t}e.exports=function(e){var t;return(e=null==(t=e)?"":f(t))&&r.test(e)?e.replace(n,l):e}}).call(this,n(61))},function(e,t,n){"use strict";n.d(t,"b",(function(){return h}));var r=n(1),i=n(5),o=n(0),a=(n(3),n(456)),s=n(7),l=n(142),u=n(106),c=n(455),d=n(454),h=function(e){return{root:{},select:{"-moz-appearance":"none","-webkit-appearance":"none",userSelect:"none",borderRadius:0,minWidth:16,cursor:"pointer","&:focus":{backgroundColor:"light"===e.palette.type?"rgba(0, 0, 0, 0.05)":"rgba(255, 255, 255, 0.05)",borderRadius:0},"&::-ms-expand":{display:"none"},"&$disabled":{cursor:"default"},"&[multiple]":{height:"auto"},"&:not([multiple]) option, &:not([multiple]) optgroup":{backgroundColor:e.palette.background.paper},"&&":{paddingRight:24}},filled:{"&&":{paddingRight:32}},outlined:{borderRadius:e.shape.borderRadius,"&&":{paddingRight:32}},selectMenu:{height:"auto",minHeight:"1.1876em",textOverflow:"ellipsis",whiteSpace:"nowrap",overflow:"hidden"},disabled:{},icon:{position:"absolute",right:0,top:"calc(50% - 12px)",pointerEvents:"none",color:e.palette.action.active,"&$disabled":{color:e.palette.action.disabled}},iconOpen:{transform:"rotate(180deg)"},iconFilled:{right:7},iconOutlined:{right:7},nativeInput:{bottom:0,left:0,position:"absolute",opacity:0,pointerEvents:"none",width:"100%"}}},f=o.createElement(d.a,null),p=o.forwardRef((function(e,t){var n=e.children,s=e.classes,d=e.IconComponent,h=void 0===d?c.a:d,p=e.input,g=void 0===p?f:p,v=e.inputProps,m=(e.variant,Object(i.a)(e,["children","classes","IconComponent","input","inputProps","variant"])),y=Object(u.a)(),b=Object(l.a)({props:e,muiFormControl:y,states:["variant"]});return o.cloneElement(g,Object(r.a)({inputComponent:a.a,inputProps:Object(r.a)({children:n,classes:s,IconComponent:h,variant:b.variant,type:void 0},v,g?g.props.inputProps:{}),ref:t},m))}));p.muiName="Select",t.a=Object(s.a)(h,{name:"MuiNativeSelect"})(p)},function(e,t,n){"use strict";(function(e){
/**!
 * @fileOverview Kickass library to create and place poppers near their reference elements.
 * @version 1.16.1-lts
 * @license
 * Copyright (c) 2016 Federico Zivolo and contributors
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
var n="undefined"!=typeof window&&"undefined"!=typeof document&&"undefined"!=typeof navigator,r=function(){for(var e=["Edge","Trident","Firefox"],t=0;t<e.length;t+=1)if(n&&navigator.userAgent.indexOf(e[t])>=0)return 1;return 0}();var i=n&&window.Promise?function(e){var t=!1;return function(){t||(t=!0,window.Promise.resolve().then((function(){t=!1,e()})))}}:function(e){var t=!1;return function(){t||(t=!0,setTimeout((function(){t=!1,e()}),r))}};function o(e){return e&&"[object Function]"==={}.toString.call(e)}function a(e,t){if(1!==e.nodeType)return[];var n=e.ownerDocument.defaultView.getComputedStyle(e,null);return t?n[t]:n}function s(e){return"HTML"===e.nodeName?e:e.parentNode||e.host}function l(e){if(!e)return document.body;switch(e.nodeName){case"HTML":case"BODY":return e.ownerDocument.body;case"#document":return e.body}var t=a(e),n=t.overflow,r=t.overflowX,i=t.overflowY;return/(auto|scroll|overlay)/.test(n+i+r)?e:l(s(e))}function u(e){return e&&e.referenceNode?e.referenceNode:e}var c=n&&!(!window.MSInputMethodContext||!document.documentMode),d=n&&/MSIE 10/.test(navigator.userAgent);function h(e){return 11===e?c:10===e?d:c||d}function f(e){if(!e)return document.documentElement;for(var t=h(10)?document.body:null,n=e.offsetParent||null;n===t&&e.nextElementSibling;)n=(e=e.nextElementSibling).offsetParent;var r=n&&n.nodeName;return r&&"BODY"!==r&&"HTML"!==r?-1!==["TH","TD","TABLE"].indexOf(n.nodeName)&&"static"===a(n,"position")?f(n):n:e?e.ownerDocument.documentElement:document.documentElement}function p(e){return null!==e.parentNode?p(e.parentNode):e}function g(e,t){if(!(e&&e.nodeType&&t&&t.nodeType))return document.documentElement;var n=e.compareDocumentPosition(t)&Node.DOCUMENT_POSITION_FOLLOWING,r=n?e:t,i=n?t:e,o=document.createRange();o.setStart(r,0),o.setEnd(i,0);var a,s,l=o.commonAncestorContainer;if(e!==l&&t!==l||r.contains(i))return"BODY"===(s=(a=l).nodeName)||"HTML"!==s&&f(a.firstElementChild)!==a?f(l):l;var u=p(e);return u.host?g(u.host,t):g(e,p(t).host)}function v(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:"top",n="top"===t?"scrollTop":"scrollLeft",r=e.nodeName;if("BODY"===r||"HTML"===r){var i=e.ownerDocument.documentElement,o=e.ownerDocument.scrollingElement||i;return o[n]}return e[n]}function m(e,t){var n=arguments.length>2&&void 0!==arguments[2]&&arguments[2],r=v(t,"top"),i=v(t,"left"),o=n?-1:1;return e.top+=r*o,e.bottom+=r*o,e.left+=i*o,e.right+=i*o,e}function y(e,t){var n="x"===t?"Left":"Top",r="Left"===n?"Right":"Bottom";return parseFloat(e["border"+n+"Width"])+parseFloat(e["border"+r+"Width"])}function b(e,t,n,r){return Math.max(t["offset"+e],t["scroll"+e],n["client"+e],n["offset"+e],n["scroll"+e],h(10)?parseInt(n["offset"+e])+parseInt(r["margin"+("Height"===e?"Top":"Left")])+parseInt(r["margin"+("Height"===e?"Bottom":"Right")]):0)}function C(e){var t=e.body,n=e.documentElement,r=h(10)&&getComputedStyle(n);return{height:b("Height",t,n,r),width:b("Width",t,n,r)}}var A=function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")},_=function(){function e(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}return function(t,n,r){return n&&e(t.prototype,n),r&&e(t,r),t}}(),I=function(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e},x=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e};function w(e){return x({},e,{right:e.left+e.width,bottom:e.top+e.height})}function M(e){var t={};try{if(h(10)){t=e.getBoundingClientRect();var n=v(e,"top"),r=v(e,"left");t.top+=n,t.left+=r,t.bottom+=n,t.right+=r}else t=e.getBoundingClientRect()}catch(e){}var i={left:t.left,top:t.top,width:t.right-t.left,height:t.bottom-t.top},o="HTML"===e.nodeName?C(e.ownerDocument):{},s=o.width||e.clientWidth||i.width,l=o.height||e.clientHeight||i.height,u=e.offsetWidth-s,c=e.offsetHeight-l;if(u||c){var d=a(e);u-=y(d,"x"),c-=y(d,"y"),i.width-=u,i.height-=c}return w(i)}function S(e,t){var n=arguments.length>2&&void 0!==arguments[2]&&arguments[2],r=h(10),i="HTML"===t.nodeName,o=M(e),s=M(t),u=l(e),c=a(t),d=parseFloat(c.borderTopWidth),f=parseFloat(c.borderLeftWidth);n&&i&&(s.top=Math.max(s.top,0),s.left=Math.max(s.left,0));var p=w({top:o.top-s.top-d,left:o.left-s.left-f,width:o.width,height:o.height});if(p.marginTop=0,p.marginLeft=0,!r&&i){var g=parseFloat(c.marginTop),v=parseFloat(c.marginLeft);p.top-=d-g,p.bottom-=d-g,p.left-=f-v,p.right-=f-v,p.marginTop=g,p.marginLeft=v}return(r&&!n?t.contains(u):t===u&&"BODY"!==u.nodeName)&&(p=m(p,t)),p}function T(e){var t=arguments.length>1&&void 0!==arguments[1]&&arguments[1],n=e.ownerDocument.documentElement,r=S(e,n),i=Math.max(n.clientWidth,window.innerWidth||0),o=Math.max(n.clientHeight,window.innerHeight||0),a=t?0:v(n),s=t?0:v(n,"left"),l={top:a-r.top+r.marginTop,left:s-r.left+r.marginLeft,width:i,height:o};return w(l)}function E(e){var t=e.nodeName;if("BODY"===t||"HTML"===t)return!1;if("fixed"===a(e,"position"))return!0;var n=s(e);return!!n&&E(n)}function O(e){if(!e||!e.parentElement||h())return document.documentElement;for(var t=e.parentElement;t&&"none"===a(t,"transform");)t=t.parentElement;return t||document.documentElement}function k(e,t,n,r){var i=arguments.length>4&&void 0!==arguments[4]&&arguments[4],o={top:0,left:0},a=i?O(e):g(e,u(t));if("viewport"===r)o=T(a,i);else{var c=void 0;"scrollParent"===r?"BODY"===(c=l(s(t))).nodeName&&(c=e.ownerDocument.documentElement):c="window"===r?e.ownerDocument.documentElement:r;var d=S(c,a,i);if("HTML"!==c.nodeName||E(a))o=d;else{var h=C(e.ownerDocument),f=h.height,p=h.width;o.top+=d.top-d.marginTop,o.bottom=f+d.top,o.left+=d.left-d.marginLeft,o.right=p+d.left}}var v="number"==typeof(n=n||0);return o.left+=v?n:n.left||0,o.top+=v?n:n.top||0,o.right-=v?n:n.right||0,o.bottom-=v?n:n.bottom||0,o}function L(e){return e.width*e.height}function D(e,t,n,r,i){var o=arguments.length>5&&void 0!==arguments[5]?arguments[5]:0;if(-1===e.indexOf("auto"))return e;var a=k(n,r,o,i),s={top:{width:a.width,height:t.top-a.top},right:{width:a.right-t.right,height:a.height},bottom:{width:a.width,height:a.bottom-t.bottom},left:{width:t.left-a.left,height:a.height}},l=Object.keys(s).map((function(e){return x({key:e},s[e],{area:L(s[e])})})).sort((function(e,t){return t.area-e.area})),u=l.filter((function(e){var t=e.width,r=e.height;return t>=n.clientWidth&&r>=n.clientHeight})),c=u.length>0?u[0].key:l[0].key,d=e.split("-")[1];return c+(d?"-"+d:"")}function P(e,t,n){var r=arguments.length>3&&void 0!==arguments[3]?arguments[3]:null,i=r?O(t):g(t,u(n));return S(n,i,r)}function N(e){var t=e.ownerDocument.defaultView.getComputedStyle(e),n=parseFloat(t.marginTop||0)+parseFloat(t.marginBottom||0),r=parseFloat(t.marginLeft||0)+parseFloat(t.marginRight||0);return{width:e.offsetWidth+r,height:e.offsetHeight+n}}function j(e){var t={left:"right",right:"left",bottom:"top",top:"bottom"};return e.replace(/left|right|bottom|top/g,(function(e){return t[e]}))}function R(e,t,n){n=n.split("-")[0];var r=N(e),i={width:r.width,height:r.height},o=-1!==["right","left"].indexOf(n),a=o?"top":"left",s=o?"left":"top",l=o?"height":"width",u=o?"width":"height";return i[a]=t[a]+t[l]/2-r[l]/2,i[s]=n===s?t[s]-r[u]:t[j(s)],i}function z(e,t){return Array.prototype.find?e.find(t):e.filter(t)[0]}function F(e,t,n){return(void 0===n?e:e.slice(0,function(e,t,n){if(Array.prototype.findIndex)return e.findIndex((function(e){return e[t]===n}));var r=z(e,(function(e){return e[t]===n}));return e.indexOf(r)}(e,"name",n))).forEach((function(e){e.function&&console.warn("`modifier.function` is deprecated, use `modifier.fn`!");var n=e.function||e.fn;e.enabled&&o(n)&&(t.offsets.popper=w(t.offsets.popper),t.offsets.reference=w(t.offsets.reference),t=n(t,e))})),t}function B(){if(!this.state.isDestroyed){var e={instance:this,styles:{},arrowStyles:{},attributes:{},flipped:!1,offsets:{}};e.offsets.reference=P(this.state,this.popper,this.reference,this.options.positionFixed),e.placement=D(this.options.placement,e.offsets.reference,this.popper,this.reference,this.options.modifiers.flip.boundariesElement,this.options.modifiers.flip.padding),e.originalPlacement=e.placement,e.positionFixed=this.options.positionFixed,e.offsets.popper=R(this.popper,e.offsets.reference,e.placement),e.offsets.popper.position=this.options.positionFixed?"fixed":"absolute",e=F(this.modifiers,e),this.state.isCreated?this.options.onUpdate(e):(this.state.isCreated=!0,this.options.onCreate(e))}}function H(e,t){return e.some((function(e){var n=e.name;return e.enabled&&n===t}))}function U(e){for(var t=[!1,"ms","Webkit","Moz","O"],n=e.charAt(0).toUpperCase()+e.slice(1),r=0;r<t.length;r++){var i=t[r],o=i?""+i+n:e;if(void 0!==document.body.style[o])return o}return null}function V(){return this.state.isDestroyed=!0,H(this.modifiers,"applyStyle")&&(this.popper.removeAttribute("x-placement"),this.popper.style.position="",this.popper.style.top="",this.popper.style.left="",this.popper.style.right="",this.popper.style.bottom="",this.popper.style.willChange="",this.popper.style[U("transform")]=""),this.disableEventListeners(),this.options.removeOnDestroy&&this.popper.parentNode.removeChild(this.popper),this}function Y(e){var t=e.ownerDocument;return t?t.defaultView:window}function W(e,t,n,r){n.updateBound=r,Y(e).addEventListener("resize",n.updateBound,{passive:!0});var i=l(e);return function e(t,n,r,i){var o="BODY"===t.nodeName,a=o?t.ownerDocument.defaultView:t;a.addEventListener(n,r,{passive:!0}),o||e(l(a.parentNode),n,r,i),i.push(a)}(i,"scroll",n.updateBound,n.scrollParents),n.scrollElement=i,n.eventsEnabled=!0,n}function G(){this.state.eventsEnabled||(this.state=W(this.reference,this.options,this.state,this.scheduleUpdate))}function X(){var e,t;this.state.eventsEnabled&&(cancelAnimationFrame(this.scheduleUpdate),this.state=(e=this.reference,t=this.state,Y(e).removeEventListener("resize",t.updateBound),t.scrollParents.forEach((function(e){e.removeEventListener("scroll",t.updateBound)})),t.updateBound=null,t.scrollParents=[],t.scrollElement=null,t.eventsEnabled=!1,t))}function q(e){return""!==e&&!isNaN(parseFloat(e))&&isFinite(e)}function Z(e,t){Object.keys(t).forEach((function(n){var r="";-1!==["width","height","top","right","bottom","left"].indexOf(n)&&q(t[n])&&(r="px"),e.style[n]=t[n]+r}))}var Q=n&&/Firefox/i.test(navigator.userAgent);function K(e,t,n){var r=z(e,(function(e){return e.name===t})),i=!!r&&e.some((function(e){return e.name===n&&e.enabled&&e.order<r.order}));if(!i){var o="`"+t+"`",a="`"+n+"`";console.warn(a+" modifier is required by "+o+" modifier in order to work, be sure to include it before "+o+"!")}return i}var J=["auto-start","auto","auto-end","top-start","top","top-end","right-start","right","right-end","bottom-end","bottom","bottom-start","left-end","left","left-start"],$=J.slice(3);function ee(e){var t=arguments.length>1&&void 0!==arguments[1]&&arguments[1],n=$.indexOf(e),r=$.slice(n+1).concat($.slice(0,n));return t?r.reverse():r}var te="flip",ne="clockwise",re="counterclockwise";function ie(e,t,n,r){var i=[0,0],o=-1!==["right","left"].indexOf(r),a=e.split(/(\+|\-)/).map((function(e){return e.trim()})),s=a.indexOf(z(a,(function(e){return-1!==e.search(/,|\s/)})));a[s]&&-1===a[s].indexOf(",")&&console.warn("Offsets separated by white space(s) are deprecated, use a comma (,) instead.");var l=/\s*,\s*|\s+/,u=-1!==s?[a.slice(0,s).concat([a[s].split(l)[0]]),[a[s].split(l)[1]].concat(a.slice(s+1))]:[a];return(u=u.map((function(e,r){var i=(1===r?!o:o)?"height":"width",a=!1;return e.reduce((function(e,t){return""===e[e.length-1]&&-1!==["+","-"].indexOf(t)?(e[e.length-1]=t,a=!0,e):a?(e[e.length-1]+=t,a=!1,e):e.concat(t)}),[]).map((function(e){return function(e,t,n,r){var i=e.match(/((?:\-|\+)?\d*\.?\d*)(.*)/),o=+i[1],a=i[2];if(!o)return e;if(0===a.indexOf("%")){var s=void 0;switch(a){case"%p":s=n;break;case"%":case"%r":default:s=r}return w(s)[t]/100*o}if("vh"===a||"vw"===a){return("vh"===a?Math.max(document.documentElement.clientHeight,window.innerHeight||0):Math.max(document.documentElement.clientWidth,window.innerWidth||0))/100*o}return o}(e,i,t,n)}))}))).forEach((function(e,t){e.forEach((function(n,r){q(n)&&(i[t]+=n*("-"===e[r-1]?-1:1))}))})),i}var oe={placement:"bottom",positionFixed:!1,eventsEnabled:!0,removeOnDestroy:!1,onCreate:function(){},onUpdate:function(){},modifiers:{shift:{order:100,enabled:!0,fn:function(e){var t=e.placement,n=t.split("-")[0],r=t.split("-")[1];if(r){var i=e.offsets,o=i.reference,a=i.popper,s=-1!==["bottom","top"].indexOf(n),l=s?"left":"top",u=s?"width":"height",c={start:I({},l,o[l]),end:I({},l,o[l]+o[u]-a[u])};e.offsets.popper=x({},a,c[r])}return e}},offset:{order:200,enabled:!0,fn:function(e,t){var n=t.offset,r=e.placement,i=e.offsets,o=i.popper,a=i.reference,s=r.split("-")[0],l=void 0;return l=q(+n)?[+n,0]:ie(n,o,a,s),"left"===s?(o.top+=l[0],o.left-=l[1]):"right"===s?(o.top+=l[0],o.left+=l[1]):"top"===s?(o.left+=l[0],o.top-=l[1]):"bottom"===s&&(o.left+=l[0],o.top+=l[1]),e.popper=o,e},offset:0},preventOverflow:{order:300,enabled:!0,fn:function(e,t){var n=t.boundariesElement||f(e.instance.popper);e.instance.reference===n&&(n=f(n));var r=U("transform"),i=e.instance.popper.style,o=i.top,a=i.left,s=i[r];i.top="",i.left="",i[r]="";var l=k(e.instance.popper,e.instance.reference,t.padding,n,e.positionFixed);i.top=o,i.left=a,i[r]=s,t.boundaries=l;var u=t.priority,c=e.offsets.popper,d={primary:function(e){var n=c[e];return c[e]<l[e]&&!t.escapeWithReference&&(n=Math.max(c[e],l[e])),I({},e,n)},secondary:function(e){var n="right"===e?"left":"top",r=c[n];return c[e]>l[e]&&!t.escapeWithReference&&(r=Math.min(c[n],l[e]-("right"===e?c.width:c.height))),I({},n,r)}};return u.forEach((function(e){var t=-1!==["left","top"].indexOf(e)?"primary":"secondary";c=x({},c,d[t](e))})),e.offsets.popper=c,e},priority:["left","right","top","bottom"],padding:5,boundariesElement:"scrollParent"},keepTogether:{order:400,enabled:!0,fn:function(e){var t=e.offsets,n=t.popper,r=t.reference,i=e.placement.split("-")[0],o=Math.floor,a=-1!==["top","bottom"].indexOf(i),s=a?"right":"bottom",l=a?"left":"top",u=a?"width":"height";return n[s]<o(r[l])&&(e.offsets.popper[l]=o(r[l])-n[u]),n[l]>o(r[s])&&(e.offsets.popper[l]=o(r[s])),e}},arrow:{order:500,enabled:!0,fn:function(e,t){var n;if(!K(e.instance.modifiers,"arrow","keepTogether"))return e;var r=t.element;if("string"==typeof r){if(!(r=e.instance.popper.querySelector(r)))return e}else if(!e.instance.popper.contains(r))return console.warn("WARNING: `arrow.element` must be child of its popper element!"),e;var i=e.placement.split("-")[0],o=e.offsets,s=o.popper,l=o.reference,u=-1!==["left","right"].indexOf(i),c=u?"height":"width",d=u?"Top":"Left",h=d.toLowerCase(),f=u?"left":"top",p=u?"bottom":"right",g=N(r)[c];l[p]-g<s[h]&&(e.offsets.popper[h]-=s[h]-(l[p]-g)),l[h]+g>s[p]&&(e.offsets.popper[h]+=l[h]+g-s[p]),e.offsets.popper=w(e.offsets.popper);var v=l[h]+l[c]/2-g/2,m=a(e.instance.popper),y=parseFloat(m["margin"+d]),b=parseFloat(m["border"+d+"Width"]),C=v-e.offsets.popper[h]-y-b;return C=Math.max(Math.min(s[c]-g,C),0),e.arrowElement=r,e.offsets.arrow=(I(n={},h,Math.round(C)),I(n,f,""),n),e},element:"[x-arrow]"},flip:{order:600,enabled:!0,fn:function(e,t){if(H(e.instance.modifiers,"inner"))return e;if(e.flipped&&e.placement===e.originalPlacement)return e;var n=k(e.instance.popper,e.instance.reference,t.padding,t.boundariesElement,e.positionFixed),r=e.placement.split("-")[0],i=j(r),o=e.placement.split("-")[1]||"",a=[];switch(t.behavior){case te:a=[r,i];break;case ne:a=ee(r);break;case re:a=ee(r,!0);break;default:a=t.behavior}return a.forEach((function(s,l){if(r!==s||a.length===l+1)return e;r=e.placement.split("-")[0],i=j(r);var u=e.offsets.popper,c=e.offsets.reference,d=Math.floor,h="left"===r&&d(u.right)>d(c.left)||"right"===r&&d(u.left)<d(c.right)||"top"===r&&d(u.bottom)>d(c.top)||"bottom"===r&&d(u.top)<d(c.bottom),f=d(u.left)<d(n.left),p=d(u.right)>d(n.right),g=d(u.top)<d(n.top),v=d(u.bottom)>d(n.bottom),m="left"===r&&f||"right"===r&&p||"top"===r&&g||"bottom"===r&&v,y=-1!==["top","bottom"].indexOf(r),b=!!t.flipVariations&&(y&&"start"===o&&f||y&&"end"===o&&p||!y&&"start"===o&&g||!y&&"end"===o&&v),C=!!t.flipVariationsByContent&&(y&&"start"===o&&p||y&&"end"===o&&f||!y&&"start"===o&&v||!y&&"end"===o&&g),A=b||C;(h||m||A)&&(e.flipped=!0,(h||m)&&(r=a[l+1]),A&&(o=function(e){return"end"===e?"start":"start"===e?"end":e}(o)),e.placement=r+(o?"-"+o:""),e.offsets.popper=x({},e.offsets.popper,R(e.instance.popper,e.offsets.reference,e.placement)),e=F(e.instance.modifiers,e,"flip"))})),e},behavior:"flip",padding:5,boundariesElement:"viewport",flipVariations:!1,flipVariationsByContent:!1},inner:{order:700,enabled:!1,fn:function(e){var t=e.placement,n=t.split("-")[0],r=e.offsets,i=r.popper,o=r.reference,a=-1!==["left","right"].indexOf(n),s=-1===["top","left"].indexOf(n);return i[a?"left":"top"]=o[n]-(s?i[a?"width":"height"]:0),e.placement=j(t),e.offsets.popper=w(i),e}},hide:{order:800,enabled:!0,fn:function(e){if(!K(e.instance.modifiers,"hide","preventOverflow"))return e;var t=e.offsets.reference,n=z(e.instance.modifiers,(function(e){return"preventOverflow"===e.name})).boundaries;if(t.bottom<n.top||t.left>n.right||t.top>n.bottom||t.right<n.left){if(!0===e.hide)return e;e.hide=!0,e.attributes["x-out-of-boundaries"]=""}else{if(!1===e.hide)return e;e.hide=!1,e.attributes["x-out-of-boundaries"]=!1}return e}},computeStyle:{order:850,enabled:!0,fn:function(e,t){var n=t.x,r=t.y,i=e.offsets.popper,o=z(e.instance.modifiers,(function(e){return"applyStyle"===e.name})).gpuAcceleration;void 0!==o&&console.warn("WARNING: `gpuAcceleration` option moved to `computeStyle` modifier and will not be supported in future versions of Popper.js!");var a=void 0!==o?o:t.gpuAcceleration,s=f(e.instance.popper),l=M(s),u={position:i.position},c=function(e,t){var n=e.offsets,r=n.popper,i=n.reference,o=Math.round,a=Math.floor,s=function(e){return e},l=o(i.width),u=o(r.width),c=-1!==["left","right"].indexOf(e.placement),d=-1!==e.placement.indexOf("-"),h=t?c||d||l%2==u%2?o:a:s,f=t?o:s;return{left:h(l%2==1&&u%2==1&&!d&&t?r.left-1:r.left),top:f(r.top),bottom:f(r.bottom),right:h(r.right)}}(e,window.devicePixelRatio<2||!Q),d="bottom"===n?"top":"bottom",h="right"===r?"left":"right",p=U("transform"),g=void 0,v=void 0;if(v="bottom"===d?"HTML"===s.nodeName?-s.clientHeight+c.bottom:-l.height+c.bottom:c.top,g="right"===h?"HTML"===s.nodeName?-s.clientWidth+c.right:-l.width+c.right:c.left,a&&p)u[p]="translate3d("+g+"px, "+v+"px, 0)",u[d]=0,u[h]=0,u.willChange="transform";else{var m="bottom"===d?-1:1,y="right"===h?-1:1;u[d]=v*m,u[h]=g*y,u.willChange=d+", "+h}var b={"x-placement":e.placement};return e.attributes=x({},b,e.attributes),e.styles=x({},u,e.styles),e.arrowStyles=x({},e.offsets.arrow,e.arrowStyles),e},gpuAcceleration:!0,x:"bottom",y:"right"},applyStyle:{order:900,enabled:!0,fn:function(e){var t,n;return Z(e.instance.popper,e.styles),t=e.instance.popper,n=e.attributes,Object.keys(n).forEach((function(e){!1!==n[e]?t.setAttribute(e,n[e]):t.removeAttribute(e)})),e.arrowElement&&Object.keys(e.arrowStyles).length&&Z(e.arrowElement,e.arrowStyles),e},onLoad:function(e,t,n,r,i){var o=P(i,t,e,n.positionFixed),a=D(n.placement,o,t,e,n.modifiers.flip.boundariesElement,n.modifiers.flip.padding);return t.setAttribute("x-placement",a),Z(t,{position:n.positionFixed?"fixed":"absolute"}),n},gpuAcceleration:void 0}}},ae=function(){function e(t,n){var r=this,a=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};A(this,e),this.scheduleUpdate=function(){return requestAnimationFrame(r.update)},this.update=i(this.update.bind(this)),this.options=x({},e.Defaults,a),this.state={isDestroyed:!1,isCreated:!1,scrollParents:[]},this.reference=t&&t.jquery?t[0]:t,this.popper=n&&n.jquery?n[0]:n,this.options.modifiers={},Object.keys(x({},e.Defaults.modifiers,a.modifiers)).forEach((function(t){r.options.modifiers[t]=x({},e.Defaults.modifiers[t]||{},a.modifiers?a.modifiers[t]:{})})),this.modifiers=Object.keys(this.options.modifiers).map((function(e){return x({name:e},r.options.modifiers[e])})).sort((function(e,t){return e.order-t.order})),this.modifiers.forEach((function(e){e.enabled&&o(e.onLoad)&&e.onLoad(r.reference,r.popper,r.options,e,r.state)})),this.update();var s=this.options.eventsEnabled;s&&this.enableEventListeners(),this.state.eventsEnabled=s}return _(e,[{key:"update",value:function(){return B.call(this)}},{key:"destroy",value:function(){return V.call(this)}},{key:"enableEventListeners",value:function(){return G.call(this)}},{key:"disableEventListeners",value:function(){return X.call(this)}}]),e}();ae.Utils=("undefined"!=typeof window?window:e).PopperUtils,ae.placements=J,ae.Defaults=oe,t.a=ae}).call(this,n(61))},function(e,t,n){"use strict";e.exports=n(1577)},function(e,t,n){"use strict";function r(e){var t,n=e.Symbol;return"function"==typeof n?n.observable?t=n.observable:(t=n("observable"),n.observable=t):t="@@observable",t}n.d(t,"a",(function(){return r}))},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){(0,i.default)(2,arguments);var n=(0,r.default)(e),o=(0,r.default)(t);return n.getTime()-o.getTime()};var r=o(n(47)),i=o(n(40));function o(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){(0,i.default)(1,arguments);var t=(0,r.default)(e),n=t.getHours();return n};var r=o(n(47)),i=o(n(40));function o(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){(0,o.default)(2,arguments);var n=(0,i.default)(e),a=(0,r.default)(t);return n.setHours(a),n};var r=a(n(77)),i=a(n(47)),o=a(n(40));function a(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){(0,o.default)(2,arguments);var n=(0,i.default)(e),a=(0,r.default)(t);return n.setMinutes(a),n};var r=a(n(77)),i=a(n(47)),o=a(n(40));function a(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){(0,i.default)(1,arguments);var t=(0,r.default)(e),n=t.getSeconds();return n};var r=o(n(47)),i=o(n(40));function o(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){(0,o.default)(2,arguments);var n=(0,i.default)(e),a=(0,r.default)(t);return n.setSeconds(a),n};var r=a(n(77)),i=a(n(47)),o=a(n(40));function a(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){(0,i.default)(2,arguments);var n=(0,r.default)(e),o=(0,r.default)(t);return n.getTime()===o.getTime()};var r=o(n(460)),i=o(n(40));function o(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){(0,i.default)(2,arguments);var n=(0,r.default)(e),o=(0,r.default)(t);return n.getFullYear()===o.getFullYear()&&n.getMonth()===o.getMonth()};var r=o(n(47)),i=o(n(40));function o(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){(0,i.default)(2,arguments);var n=(0,r.default)(e),o=(0,r.default)(t);return n.getFullYear()===o.getFullYear()};var r=o(n(47)),i=o(n(40));function o(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){(0,i.default)(2,arguments);var n=(0,r.default)(e),o=(0,r.default)(t);return n.getTime()===o.getTime()};var r=o(n(2250)),i=o(n(40));function o(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){(0,i.default)(1,arguments);var t=(0,r.default)(e),n=t.getFullYear();return n};var r=o(n(47)),i=o(n(40));function o(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){(0,o.default)(2,arguments);var n=(0,i.default)(e),a=(0,r.default)(t);if(isNaN(n))return new Date(NaN);return n.setFullYear(a),n};var r=a(n(77)),i=a(n(47)),o=a(n(40));function a(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t,n,f){(0,h.default)(3,arguments);var v=String(e),m=String(t),_=f||{},I=_.locale||r.default;if(!I.match)throw new RangeError("locale must contain match property");var x=I.options&&I.options.firstWeekContainsDate,w=null==x?1:(0,c.default)(x),M=null==_.firstWeekContainsDate?w:(0,c.default)(_.firstWeekContainsDate);if(!(M>=1&&M<=7))throw new RangeError("firstWeekContainsDate must be between 1 and 7 inclusively");var S=I.options&&I.options.weekStartsOn,T=null==S?0:(0,c.default)(S),E=null==_.weekStartsOn?T:(0,c.default)(_.weekStartsOn);if(!(E>=0&&E<=6))throw new RangeError("weekStartsOn must be between 0 and 6 inclusively");if(""===m)return""===v?(0,o.default)(n):new Date(NaN);var O,k={firstWeekContainsDate:M,weekStartsOn:E,locale:I},L=[{priority:10,subPriority:-1,set:C,index:0}],D=m.match(g).map((function(e){var t=e[0];return"p"===t||"P"===t?(0,s.default[t])(e,I.formatLong,k):e})).join("").match(p),P=[];for(O=0;O<D.length;O++){var N=D[O];!_.useAdditionalWeekYearTokens&&(0,u.isProtectedWeekYearToken)(N)&&(0,u.throwProtectedError)(N,m,e),!_.useAdditionalDayOfYearTokens&&(0,u.isProtectedDayOfYearToken)(N)&&(0,u.throwProtectedError)(N,m,e);var j=N[0],R=d.default[j];if(R){var z=R.incompatibleTokens;if(Array.isArray(z)){for(var F=void 0,B=0;B<P.length;B++){var H=P[B].token;if(-1!==z.indexOf(H)||H===j){F=P[B];break}}if(F)throw new RangeError("The format string mustn't contain `".concat(F.fullToken,"` and `").concat(N,"` at the same time"))}else if("*"===R.incompatibleTokens&&P.length)throw new RangeError("The format string mustn't contain `".concat(N,"` and any other token at the same time"));P.push({token:j,fullToken:N});var U=R.parse(v,N,I.match,k);if(!U)return new Date(NaN);L.push({priority:R.priority,subPriority:R.subPriority||0,set:R.set,validate:R.validate,value:U.value,index:L.length}),v=U.rest}else{if(j.match(b))throw new RangeError("Format string contains an unescaped latin alphabet character `"+j+"`");if("''"===N?N="'":"'"===j&&(N=A(N)),0!==v.indexOf(N))return new Date(NaN);v=v.slice(N.length)}}if(v.length>0&&y.test(v))return new Date(NaN);var V=L.map((function(e){return e.priority})).sort((function(e,t){return t-e})).filter((function(e,t,n){return n.indexOf(e)===t})).map((function(e){return L.filter((function(t){return t.priority===e})).sort((function(e,t){return t.subPriority-e.subPriority}))})).map((function(e){return e[0]})),Y=(0,o.default)(n);if(isNaN(Y))return new Date(NaN);var W=(0,i.default)(Y,(0,l.default)(Y)),G={};for(O=0;O<V.length;O++){var X=V[O];if(X.validate&&!X.validate(W,X.value,k))return new Date(NaN);var q=X.set(W,G,X.value,k);q[0]?(W=q[0],(0,a.default)(G,q[1])):W=q}return W};var r=f(n(1179)),i=f(n(1180)),o=f(n(47)),a=f(n(2261)),s=f(n(1181)),l=f(n(1182)),u=n(1183),c=f(n(77)),d=f(n(2262)),h=f(n(40));function f(e){return e&&e.__esModule?e:{default:e}}var p=/[yYQqMLwIdDecihHKkms]o|(\w)\1*|''|'(''|[^'])+('|$)|./g,g=/P+p+|P+|p+|''|'(''|[^'])+('|$)|./g,v=/^'([^]*?)'?$/,m=/''/g,y=/\S/,b=/[a-zA-Z]/;function C(e,t){if(t.timestampIsSet)return e;var n=new Date(0);return n.setFullYear(e.getUTCFullYear(),e.getUTCMonth(),e.getUTCDate()),n.setHours(e.getUTCHours(),e.getUTCMinutes(),e.getUTCSeconds(),e.getUTCMilliseconds()),n}function A(e){return e.match(v)[1].replace(m,"'")}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){(0,i.default)(2,arguments);var n=(0,r.default)(e),o=(0,r.default)(t);return n.getTime()===o.getTime()};var r=o(n(47)),i=o(n(40));function o(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){(0,a.default)(2,arguments);var n=(0,i.default)(e),s=(0,r.default)(t),l=n.getFullYear(),u=n.getDate(),c=new Date(0);c.setFullYear(l,s,15),c.setHours(0,0,0,0);var d=(0,o.default)(c);return n.setMonth(s,Math.min(u,d)),n};var r=s(n(77)),i=s(n(47)),o=s(n(2272)),a=s(n(40));function s(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){(0,i.default)(1,arguments);var n=e||{},o=(0,r.default)(n.start),a=(0,r.default)(n.end),s=a.getTime();if(!(o.getTime()<=s))throw new RangeError("Invalid interval");var l=[],u=o;u.setHours(0,0,0,0);var c=t&&"step"in t?Number(t.step):1;if(c<1||isNaN(c))throw new RangeError("`options.step` must be a number greater than 1");for(;u.getTime()<=s;)l.push((0,r.default)(u)),u.setDate(u.getDate()+c),u.setHours(0,0,0,0);return l};var r=o(n(47)),i=o(n(40));function o(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){(0,o.default)(2,arguments);var n=(0,r.default)(t);return(0,i.default)(e,12*n)};var r=a(n(77)),i=a(n(461)),o=a(n(40));function a(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";n.d(t,"a",(function(){return qt})),n.d(t,"b",(function(){return Zt})),n.d(t,"c",(function(){return Qt})),n.d(t,"d",(function(){return Kt})),n.d(t,"e",(function(){return Jt})),n.d(t,"f",(function(){return $t})),n.d(t,"g",(function(){return en})),n.d(t,"h",(function(){return tn})),n.d(t,"i",(function(){return nn})),n.d(t,"j",(function(){return rn})),n.d(t,"k",(function(){return on})),n.d(t,"l",(function(){return an}));var r=n(23),i=n(1210),o=n.n(i),a=n(1211),s=n.n(a),l=n(1212),u=n.n(l),c=n(1213),d=n.n(c),h=n(1214),f=n.n(h),p=n(1215),g=n.n(p),v=n(1216),m=n.n(v),y=n(1217),b=n.n(y),C=n(1218),A=n.n(C),_=n(1219),I=n.n(_),x=n(1220),w=n.n(x),M=n(1221),S=n.n(M),T=n(1222),E=n.n(T),O=n(1223),k=n.n(O),L=n(1224),D=n.n(L),P=n(1225),N=n.n(P),j=n(1226),R=n.n(j),z=n(1227),F=n.n(z),B=n(1228),H=n.n(B),U=n(1229),V=n.n(U),Y=n(1230),W=n.n(Y),G=n(1231),X=n.n(G),q=n(1232),Z=n.n(q),Q=n(1233),K=n.n(Q),J=n(1234),$=n.n(J),ee=n(1235),te=n.n(ee),ne=n(1236),re=n.n(ne),ie=n(1237),oe=n.n(ie),ae=n(1238),se=n.n(ae),le=n(1239),ue=n.n(le),ce=n(1240),de=n.n(ce),he=n(1241),fe=n.n(he),pe=n(1242),ge=n.n(pe),ve=n(1243),me=n.n(ve),ye=n(1244),be=n.n(ye),Ce=n(1245),Ae=n.n(Ce),_e=n(1246),Ie=n.n(_e),xe=n(1247),we=n.n(xe),Me=n(1248),Se=n.n(Me),Te=n(1249),Ee=n.n(Te),Oe=n(1250),ke=n.n(Oe),Le=n(1251),De=n.n(Le),Pe=n(1252),Ne=n.n(Pe),je=n(1253),Re=n.n(je),ze=n(1254),Fe=n.n(ze),Be=n(1255),He=n.n(Be),Ue=n(1256),Ve=n.n(Ue),Ye=n(1257),We=n.n(Ye),Ge=n(1258),Xe=n.n(Ge),qe=n(1259),Ze=n.n(qe),Qe=n(1260),Ke=n.n(Qe),Je=n(1261),$e=n.n(Je),et=n(1262),tt=n.n(et),nt=n(1263),rt=n.n(nt),it=n(1264),ot=n.n(it),at=n(1265),st=n.n(at),lt=n(441),ut=n.n(lt),ct=n(1266),dt=n.n(ct),ht=n(1267),ft=n.n(ht),pt=n(1268),gt=n.n(pt),vt=n(1269),mt=n.n(vt),yt=n(1270),bt=n.n(yt),Ct=n(1271),At=n.n(Ct),_t=n(1272),It=n.n(_t),xt=n(1273),wt=n.n(xt),Mt=n(1274),St=n.n(Mt),Tt=n(1275),Et=n.n(Tt),Ot=n(1276),kt=n.n(Ot),Lt=n(1277),Dt=n.n(Lt),Pt=n(1278),Nt=n.n(Pt),jt=n(1279),Rt=n.n(jt),zt=n(1280),Ft=n.n(zt),Bt=n(1281),Ht=n.n(Bt),Ut=n(1282),Vt=n.n(Ut),Yt=n(1283),Wt=n.n(Yt),Gt=n(1284),Xt=n.n(Gt);new r.a({name:"ui-components:add",svgstr:o.a}),new r.a({name:"ui-components:bug",svgstr:s.a}),new r.a({name:"ui-components:build",svgstr:u.a});const qt=new r.a({name:"ui-components:caret-down-empty",svgstr:d.a}),Zt=(new r.a({name:"ui-components:caret-down-empty-thin",svgstr:f.a}),new r.a({name:"ui-components:caret-down",svgstr:g.a})),Qt=(new r.a({name:"ui-components:caret-left",svgstr:m.a}),new r.a({name:"ui-components:caret-right",svgstr:b.a}),new r.a({name:"ui-components:caret-up-empty-thin",svgstr:A.a}),new r.a({name:"ui-components:caret-up",svgstr:I.a})),Kt=(new r.a({name:"ui-components:case-sensitive",svgstr:w.a}),new r.a({name:"ui-components:check",svgstr:S.a}),new r.a({name:"ui-components:circle-empty",svgstr:E.a})),Jt=new r.a({name:"ui-components:circle",svgstr:k.a}),$t=(new r.a({name:"ui-components:clear",svgstr:D.a}),new r.a({name:"ui-components:close",svgstr:N.a})),en=(new r.a({name:"ui-components:code",svgstr:R.a}),new r.a({name:"ui-components:console",svgstr:F.a}),new r.a({name:"ui-components:copy",svgstr:H.a}),new r.a({name:"ui-components:cut",svgstr:V.a}),new r.a({name:"ui-components:download",svgstr:W.a}),new r.a({name:"ui-components:edit",svgstr:X.a}),new r.a({name:"ui-components:ellipses",svgstr:Z.a}),new r.a({name:"ui-components:extension",svgstr:K.a}),new r.a({name:"ui-components:fast-forward",svgstr:$.a}),new r.a({name:"ui-components:file",svgstr:te.a}),new r.a({name:"ui-components:file-upload",svgstr:re.a}),new r.a({name:"ui-components:filter-list",svgstr:oe.a}),new r.a({name:"ui-components:folder",svgstr:se.a}),new r.a({name:"ui-components:html5",svgstr:ue.a}),new r.a({name:"ui-components:image",svgstr:de.a}),new r.a({name:"ui-components:inspector",svgstr:fe.a}),new r.a({name:"ui-components:json",svgstr:ge.a}),new r.a({name:"ui-components:jupyter-favicon",svgstr:me.a}),new r.a({name:"ui-components:jupyter",svgstr:be.a}),new r.a({name:"ui-components:jupyterlab-wordmark",svgstr:Ae.a}),new r.a({name:"ui-components:kernel",svgstr:Ie.a})),tn=(new r.a({name:"ui-components:keyboard",svgstr:we.a}),new r.a({name:"ui-components:launcher",svgstr:Se.a}),new r.a({name:"ui-components:line-form",svgstr:Ee.a})),nn=(new r.a({name:"ui-components:link",svgstr:ke.a}),new r.a({name:"ui-components:list",svgstr:De.a}),new r.a({name:"ui-components:listings-info",svgstr:Ne.a}),new r.a({name:"ui-components:markdown",svgstr:Re.a}),new r.a({name:"ui-components:new-folder",svgstr:Fe.a}),new r.a({name:"ui-components:not-trusted",svgstr:He.a}),new r.a({name:"ui-components:notebook",svgstr:Ve.a}),new r.a({name:"ui-components:numbering",svgstr:We.a}),new r.a({name:"ui-components:offline-bolt",svgstr:Xe.a})),rn=(new r.a({name:"ui-components:palette",svgstr:Ze.a}),new r.a({name:"ui-components:paste",svgstr:Ke.a}),new r.a({name:"ui-components:pdf",svgstr:$e.a}),new r.a({name:"ui-components:python",svgstr:tt.a}),new r.a({name:"ui-components:r-kernel",svgstr:rt.a}),new r.a({name:"ui-components:react",svgstr:ot.a}),new r.a({name:"ui-components:redo",svgstr:st.a}),new r.a({name:"ui-components:refresh",svgstr:ut.a})),on=(new r.a({name:"ui-components:regex",svgstr:dt.a}),new r.a({name:"ui-components:run",svgstr:ft.a}),new r.a({name:"ui-components:running",svgstr:gt.a}),new r.a({name:"ui-components:save",svgstr:mt.a}),new r.a({name:"ui-components:search",svgstr:bt.a}),new r.a({name:"ui-components:settings",svgstr:At.a}),new r.a({name:"ui-components:spreadsheet",svgstr:It.a}),new r.a({name:"ui-components:stop",svgstr:wt.a})),an=(new r.a({name:"ui-components:tab",svgstr:St.a}),new r.a({name:"ui-components:table-rows",svgstr:Et.a}),new r.a({name:"ui-components:tag",svgstr:kt.a}),new r.a({name:"ui-components:terminal",svgstr:Dt.a}));new r.a({name:"ui-components:text-editor",svgstr:Nt.a}),new r.a({name:"ui-components:toc",svgstr:Rt.a}),new r.a({name:"ui-components:tree-view",svgstr:Ft.a}),new r.a({name:"ui-components:trusted",svgstr:Ht.a}),new r.a({name:"ui-components:undo",svgstr:Vt.a}),new r.a({name:"ui-components:vega",svgstr:Wt.a}),new r.a({name:"ui-components:yaml",svgstr:Xt.a})},function(e,t,n){"use strict";n.d(t,"a",(function(){return o}));var r=n(449),i=["checked","disabled","error","focused","focusVisible","required","expanded","selected"];function o(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},t=e.disableGlobal,n=void 0!==t&&t,o=e.productionPrefix,a=void 0===o?"jss":o,s=e.seed,l=void 0===s?"":s,u=""===l?"":"".concat(l,"-"),c=0,d=function(){return c+=1};return function(e,t){var o=t.options.name;if(o&&0===o.indexOf("Mui")&&!t.options.link&&!n){if(-1!==i.indexOf(e.key))return"Mui-".concat(e.key);var s="".concat(u).concat(o,"-").concat(e.key);return t.options.theme[r.a]&&""===l?"".concat(s,"-").concat(d()):s}return"".concat(u).concat(a).concat(d())}}},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(4)),s=n(7),l=n(15),u=o.forwardRef((function(e,t){var n=e.children,s=e.classes,u=e.className,c=e.color,d=void 0===c?"inherit":c,h=e.component,f=void 0===h?"svg":h,p=e.fontSize,g=void 0===p?"default":p,v=e.htmlColor,m=e.titleAccess,y=e.viewBox,b=void 0===y?"0 0 24 24":y,C=Object(i.a)(e,["children","classes","className","color","component","fontSize","htmlColor","titleAccess","viewBox"]);return o.createElement(f,Object(r.a)({className:Object(a.a)(s.root,u,"inherit"!==d&&s["color".concat(Object(l.a)(d))],"default"!==g&&s["fontSize".concat(Object(l.a)(g))]),focusable:"false",viewBox:b,color:v,"aria-hidden":!m||void 0,role:m?"img":void 0,ref:t},C),n,m?o.createElement("title",null,m):null)}));u.muiName="SvgIcon",t.a=Object(s.a)((function(e){return{root:{userSelect:"none",width:"1em",height:"1em",display:"inline-block",fill:"currentColor",flexShrink:0,fontSize:e.typography.pxToRem(24),transition:e.transitions.create("fill",{duration:e.transitions.duration.shorter})},colorPrimary:{color:e.palette.primary.main},colorSecondary:{color:e.palette.secondary.main},colorAction:{color:e.palette.action.active},colorError:{color:e.palette.error.main},colorDisabled:{color:e.palette.action.disabled},fontSizeInherit:{fontSize:"inherit"},fontSizeSmall:{fontSize:e.typography.pxToRem(20)},fontSizeLarge:{fontSize:e.typography.pxToRem(35)}}}),{name:"MuiSvgIcon"})(u)},function(e,t,n){"use strict";var r=n(1),i=n(49),o=n(5),a=n(0),s=(n(3),n(341)),l=n(65),u=n(52),c=n(79),d=n(37),h={entering:{opacity:1},entered:{opacity:1}},f={enter:l.b.enteringScreen,exit:l.b.leavingScreen},p=a.forwardRef((function(e,t){var n=e.children,l=e.disableStrictModeCompat,p=void 0!==l&&l,g=e.in,v=e.onEnter,m=e.onEntered,y=e.onEntering,b=e.onExit,C=e.onExited,A=e.onExiting,_=e.style,I=e.TransitionComponent,x=void 0===I?s.a:I,w=e.timeout,M=void 0===w?f:w,S=Object(o.a)(e,["children","disableStrictModeCompat","in","onEnter","onEntered","onEntering","onExit","onExited","onExiting","style","TransitionComponent","timeout"]),T=Object(u.a)(),E=T.unstable_strictMode&&!p,O=a.useRef(null),k=Object(d.a)(n.ref,t),L=Object(d.a)(E?O:void 0,k),D=function(e){return function(t,n){if(e){var r=E?[O.current,t]:[t,n],o=Object(i.a)(r,2),a=o[0],s=o[1];void 0===s?e(a):e(a,s)}}},P=D(y),N=D((function(e,t){Object(c.b)(e);var n=Object(c.a)({style:_,timeout:M},{mode:"enter"});e.style.webkitTransition=T.transitions.create("opacity",n),e.style.transition=T.transitions.create("opacity",n),v&&v(e,t)})),j=D(m),R=D(A),z=D((function(e){var t=Object(c.a)({style:_,timeout:M},{mode:"exit"});e.style.webkitTransition=T.transitions.create("opacity",t),e.style.transition=T.transitions.create("opacity",t),b&&b(e)})),F=D(C);return a.createElement(x,Object(r.a)({appear:!0,in:g,nodeRef:E?O:void 0,onEnter:N,onEntered:j,onEntering:P,onExit:z,onExited:F,onExiting:R,timeout:M},S),(function(e,t){return a.cloneElement(n,Object(r.a)({style:Object(r.a)({opacity:0,visibility:"exited"!==e||g?void 0:"hidden"},h[e],_,n.props.style),ref:L},t))}))}));t.a=p},function(e,t,n){"use strict";var r=n(5),i=n(1),o=n(334),a=n(0),s=(n(3),n(4)),l=n(142),u=n(169),c=n(7),d=n(15),h=n(37),f=n(2356),p=n(227),g="undefined"==typeof window?a.useEffect:a.useLayoutEffect,v=a.forwardRef((function(e,t){var n=e["aria-describedby"],c=e.autoComplete,v=e.autoFocus,m=e.classes,y=e.className,b=(e.color,e.defaultValue),C=e.disabled,A=e.endAdornment,_=(e.error,e.fullWidth),I=void 0!==_&&_,x=e.id,w=e.inputComponent,M=void 0===w?"input":w,S=e.inputProps,T=void 0===S?{}:S,E=e.inputRef,O=(e.margin,e.multiline),k=void 0!==O&&O,L=e.name,D=e.onBlur,P=e.onChange,N=e.onClick,j=e.onFocus,R=e.onKeyDown,z=e.onKeyUp,F=e.placeholder,B=e.readOnly,H=e.renderSuffix,U=e.rows,V=e.rowsMax,Y=e.rowsMin,W=e.startAdornment,G=e.type,X=void 0===G?"text":G,q=e.value,Z=Object(r.a)(e,["aria-describedby","autoComplete","autoFocus","classes","className","color","defaultValue","disabled","endAdornment","error","fullWidth","id","inputComponent","inputProps","inputRef","margin","multiline","name","onBlur","onChange","onClick","onFocus","onKeyDown","onKeyUp","placeholder","readOnly","renderSuffix","rows","rowsMax","rowsMin","startAdornment","type","value"]),Q=null!=T.value?T.value:q,K=a.useRef(null!=Q).current,J=a.useRef(),$=a.useCallback((function(e){0}),[]),ee=Object(h.a)(T.ref,$),te=Object(h.a)(E,ee),ne=Object(h.a)(J,te),re=a.useState(!1),ie=re[0],oe=re[1],ae=Object(u.b)();var se=Object(l.a)({props:e,muiFormControl:ae,states:["color","disabled","error","hiddenLabel","margin","required","filled"]});se.focused=ae?ae.focused:ie,a.useEffect((function(){!ae&&C&&ie&&(oe(!1),D&&D())}),[ae,C,ie,D]);var le=ae&&ae.onFilled,ue=ae&&ae.onEmpty,ce=a.useCallback((function(e){Object(p.b)(e)?le&&le():ue&&ue()}),[le,ue]);g((function(){K&&ce({value:Q})}),[Q,ce,K]);a.useEffect((function(){ce(J.current)}),[]);var de=M,he=Object(i.a)({},T,{ref:ne});"string"!=typeof de?he=Object(i.a)({inputRef:ne,type:X},he,{ref:null}):k?!U||V||Y?(he=Object(i.a)({rows:U,rowsMax:V},he),de=f.a):de="textarea":he=Object(i.a)({type:X},he);return a.useEffect((function(){ae&&ae.setAdornedStart(Boolean(W))}),[ae,W]),a.createElement("div",Object(i.a)({className:Object(s.a)(m.root,m["color".concat(Object(d.a)(se.color||"primary"))],y,se.disabled&&m.disabled,se.error&&m.error,I&&m.fullWidth,se.focused&&m.focused,ae&&m.formControl,k&&m.multiline,W&&m.adornedStart,A&&m.adornedEnd,"dense"===se.margin&&m.marginDense),onClick:function(e){J.current&&e.currentTarget===e.target&&J.current.focus(),N&&N(e)},ref:t},Z),W,a.createElement(u.a.Provider,{value:null},a.createElement(de,Object(i.a)({"aria-invalid":se.error,"aria-describedby":n,autoComplete:c,autoFocus:v,defaultValue:b,disabled:se.disabled,id:x,onAnimationStart:function(e){ce("mui-auto-fill-cancel"===e.animationName?J.current:{value:"x"})},name:L,placeholder:F,readOnly:B,required:se.required,rows:U,value:Q,onKeyDown:R,onKeyUp:z},he,{className:Object(s.a)(m.input,T.className,se.disabled&&m.disabled,k&&m.inputMultiline,se.hiddenLabel&&m.inputHiddenLabel,W&&m.inputAdornedStart,A&&m.inputAdornedEnd,"search"===X&&m.inputTypeSearch,"dense"===se.margin&&m.inputMarginDense),onBlur:function(e){D&&D(e),T.onBlur&&T.onBlur(e),ae&&ae.onBlur?ae.onBlur(e):oe(!1)},onChange:function(e){if(!K){var t=e.target||J.current;if(null==t)throw new Error(Object(o.a)(1));ce({value:t.value})}for(var n=arguments.length,r=new Array(n>1?n-1:0),i=1;i<n;i++)r[i-1]=arguments[i];T.onChange&&T.onChange.apply(T,[e].concat(r)),P&&P.apply(void 0,[e].concat(r))},onFocus:function(e){se.disabled?e.stopPropagation():(j&&j(e),T.onFocus&&T.onFocus(e),ae&&ae.onFocus?ae.onFocus(e):oe(!0))}}))),A,H?H(Object(i.a)({},se,{startAdornment:W})):null)}));t.a=Object(c.a)((function(e){var t="light"===e.palette.type,n={color:"currentColor",opacity:t?.42:.5,transition:e.transitions.create("opacity",{duration:e.transitions.duration.shorter})},r={opacity:"0 !important"},o={opacity:t?.42:.5};return{"@global":{"@keyframes mui-auto-fill":{},"@keyframes mui-auto-fill-cancel":{}},root:Object(i.a)({},e.typography.body1,{color:e.palette.text.primary,lineHeight:"1.1876em",boxSizing:"border-box",position:"relative",cursor:"text",display:"inline-flex",alignItems:"center","&$disabled":{color:e.palette.text.disabled,cursor:"default"}}),formControl:{},focused:{},disabled:{},adornedStart:{},adornedEnd:{},error:{},marginDense:{},multiline:{padding:"".concat(6,"px 0 ").concat(7,"px"),"&$marginDense":{paddingTop:3}},colorSecondary:{},fullWidth:{width:"100%"},input:{font:"inherit",letterSpacing:"inherit",color:"currentColor",padding:"".concat(6,"px 0 ").concat(7,"px"),border:0,boxSizing:"content-box",background:"none",height:"1.1876em",margin:0,WebkitTapHighlightColor:"transparent",display:"block",minWidth:0,width:"100%",animationName:"mui-auto-fill-cancel",animationDuration:"10ms","&::-webkit-input-placeholder":n,"&::-moz-placeholder":n,"&:-ms-input-placeholder":n,"&::-ms-input-placeholder":n,"&:focus":{outline:0},"&:invalid":{boxShadow:"none"},"&::-webkit-search-decoration":{"-webkit-appearance":"none"},"label[data-shrink=false] + $formControl &":{"&::-webkit-input-placeholder":r,"&::-moz-placeholder":r,"&:-ms-input-placeholder":r,"&::-ms-input-placeholder":r,"&:focus::-webkit-input-placeholder":o,"&:focus::-moz-placeholder":o,"&:focus:-ms-input-placeholder":o,"&:focus::-ms-input-placeholder":o},"&$disabled":{opacity:1},"&:-webkit-autofill":{animationDuration:"5000s",animationName:"mui-auto-fill"}},inputMarginDense:{paddingTop:3},inputMultiline:{height:"auto",resize:"none",padding:0},inputTypeSearch:{"-moz-appearance":"textfield","-webkit-appearance":"textfield"},inputAdornedStart:{},inputAdornedEnd:{},inputHiddenLabel:{}}}),{name:"MuiInputBase"})(v)},function(e,t,n){"use strict";var r=n(1),i=n(49),o=n(5),a=n(0),s=(n(3),n(341)),l=n(52),u=n(79),c=n(37);function d(e){return"scale(".concat(e,", ").concat(Math.pow(e,2),")")}var h={entering:{opacity:1,transform:d(1)},entered:{opacity:1,transform:"none"}},f=a.forwardRef((function(e,t){var n=e.children,f=e.disableStrictModeCompat,p=void 0!==f&&f,g=e.in,v=e.onEnter,m=e.onEntered,y=e.onEntering,b=e.onExit,C=e.onExited,A=e.onExiting,_=e.style,I=e.timeout,x=void 0===I?"auto":I,w=e.TransitionComponent,M=void 0===w?s.a:w,S=Object(o.a)(e,["children","disableStrictModeCompat","in","onEnter","onEntered","onEntering","onExit","onExited","onExiting","style","timeout","TransitionComponent"]),T=a.useRef(),E=a.useRef(),O=Object(l.a)(),k=O.unstable_strictMode&&!p,L=a.useRef(null),D=Object(c.a)(n.ref,t),P=Object(c.a)(k?L:void 0,D),N=function(e){return function(t,n){if(e){var r=k?[L.current,t]:[t,n],o=Object(i.a)(r,2),a=o[0],s=o[1];void 0===s?e(a):e(a,s)}}},j=N(y),R=N((function(e,t){Object(u.b)(e);var n,r=Object(u.a)({style:_,timeout:x},{mode:"enter"}),i=r.duration,o=r.delay;"auto"===x?(n=O.transitions.getAutoHeightDuration(e.clientHeight),E.current=n):n=i,e.style.transition=[O.transitions.create("opacity",{duration:n,delay:o}),O.transitions.create("transform",{duration:.666*n,delay:o})].join(","),v&&v(e,t)})),z=N(m),F=N(A),B=N((function(e){var t,n=Object(u.a)({style:_,timeout:x},{mode:"exit"}),r=n.duration,i=n.delay;"auto"===x?(t=O.transitions.getAutoHeightDuration(e.clientHeight),E.current=t):t=r,e.style.transition=[O.transitions.create("opacity",{duration:t,delay:i}),O.transitions.create("transform",{duration:.666*t,delay:i||.333*t})].join(","),e.style.opacity="0",e.style.transform=d(.75),b&&b(e)})),H=N(C);return a.useEffect((function(){return function(){clearTimeout(T.current)}}),[]),a.createElement(M,Object(r.a)({appear:!0,in:g,nodeRef:k?L:void 0,onEnter:R,onEntered:z,onEntering:j,onExit:B,onExited:H,onExiting:F,addEndListener:function(e,t){var n=k?e:t;"auto"===x&&(T.current=setTimeout(n,E.current||0))},timeout:"auto"===x?null:x},S),(function(e,t){return a.cloneElement(n,Object(r.a)({style:Object(r.a)({opacity:0,transform:d(.75),visibility:"exited"!==e||g?void 0:"hidden"},h[e],_,n.props.style),ref:P},t))}))}));f.muiSupportAuto=!0,t.a=f},function(e,t,n){"use strict";var r=n(89),i=Date.now(),o="fnValues"+i,a="fnStyle"+ ++i,s=function(){return{onCreateRule:function(e,t,n){if("function"!=typeof t)return null;var i=Object(r.d)(e,{},n);return i[a]=t,i},onProcessStyle:function(e,t){if(o in t||a in t)return e;var n={};for(var r in e){var i=e[r];"function"==typeof i&&(delete e[r],n[r]=i)}return t[o]=n,e},onUpdate:function(e,t,n,r){var i=t,s=i[a];s&&(i.style=s(e)||{});var l=i[o];if(l)for(var u in l)i.prop(u,l[u](e),r)}}},l=n(1),u="@global",c=function(){function e(e,t,n){for(var i in this.type="global",this.at=u,this.rules=void 0,this.options=void 0,this.key=void 0,this.isProcessed=!1,this.key=e,this.options=n,this.rules=new r.a(Object(l.a)({},n,{parent:this})),t)this.rules.add(i,t[i]);this.rules.process()}var t=e.prototype;return t.getRule=function(e){return this.rules.get(e)},t.addRule=function(e,t,n){var r=this.rules.add(e,t,n);return r&&this.options.jss.plugins.onProcessRule(r),r},t.indexOf=function(e){return this.rules.indexOf(e)},t.toString=function(){return this.rules.toString()},e}(),d=function(){function e(e,t,n){this.type="global",this.at=u,this.options=void 0,this.rule=void 0,this.isProcessed=!1,this.key=void 0,this.key=e,this.options=n;var r=e.substr("@global ".length);this.rule=n.jss.createRule(r,t,Object(l.a)({},n,{parent:this}))}return e.prototype.toString=function(e){return this.rule?this.rule.toString(e):""},e}(),h=/\s*,\s*/g;function f(e,t){for(var n=e.split(h),r="",i=0;i<n.length;i++)r+=t+" "+n[i].trim(),n[i+1]&&(r+=", ");return r}var p=function(){return{onCreateRule:function(e,t,n){if(!e)return null;if(e===u)return new c(e,t,n);if("@"===e[0]&&"@global "===e.substr(0,"@global ".length))return new d(e,t,n);var r=n.parent;return r&&("global"===r.type||r.options.parent&&"global"===r.options.parent.type)&&(n.scoped=!1),!1===n.scoped&&(n.selector=e),null},onProcessRule:function(e,t){"style"===e.type&&t&&(function(e,t){var n=e.options,r=e.style,i=r?r[u]:null;if(i){for(var o in i)t.addRule(o,i[o],Object(l.a)({},n,{selector:f(o,e.selector)}));delete r[u]}}(e,t),function(e,t){var n=e.options,r=e.style;for(var i in r)if("@"===i[0]&&i.substr(0,u.length)===u){var o=f(i.substr(u.length),e.selector);t.addRule(o,r[i],Object(l.a)({},n,{selector:o})),delete r[i]}}(e,t))}}},g=/\s*,\s*/g,v=/&/g,m=/\$([\w-]+)/g;var y=function(){function e(e,t){return function(n,r){var i=e.getRule(r)||t&&t.getRule(r);return i?(i=i).selector:r}}function t(e,t){for(var n=t.split(g),r=e.split(g),i="",o=0;o<n.length;o++)for(var a=n[o],s=0;s<r.length;s++){var l=r[s];i&&(i+=", "),i+=-1!==l.indexOf("&")?l.replace(v,a):a+" "+l}return i}function n(e,t,n){if(n)return Object(l.a)({},n,{index:n.index+1});var r=e.options.nestingLevel;r=void 0===r?1:r+1;var i=Object(l.a)({},e.options,{nestingLevel:r,index:t.indexOf(e)+1});return delete i.name,i}return{onProcessStyle:function(r,i,o){if("style"!==i.type)return r;var a,s,u=i,c=u.options.parent;for(var d in r){var h=-1!==d.indexOf("&"),f="@"===d[0];if(h||f){if(a=n(u,c,a),h){var p=t(d,u.selector);s||(s=e(c,o)),p=p.replace(m,s),c.addRule(p,r[d],Object(l.a)({},a,{selector:p}))}else f&&c.addRule(d,{},a).addRule(u.key,r[d],{selector:u.selector});delete r[d]}}return r}}},b=/[A-Z]/g,C=/^ms-/,A={};function _(e){return"-"+e.toLowerCase()}var I=function(e){if(A.hasOwnProperty(e))return A[e];var t=e.replace(b,_);return A[e]=C.test(t)?"-"+t:t};function x(e){var t={};for(var n in e){t[0===n.indexOf("--")?n:I(n)]=e[n]}return e.fallbacks&&(Array.isArray(e.fallbacks)?t.fallbacks=e.fallbacks.map(x):t.fallbacks=x(e.fallbacks)),t}var w=function(){return{onProcessStyle:function(e){if(Array.isArray(e)){for(var t=0;t<e.length;t++)e[t]=x(e[t]);return e}return x(e)},onChangeValue:function(e,t,n){if(0===t.indexOf("--"))return e;var r=I(t);return t===r?e:(n.prop(r,e),null)}}},M=r.f&&CSS?CSS.px:"px",S=r.f&&CSS?CSS.ms:"ms",T=r.f&&CSS?CSS.percent:"%";function E(e){var t=/(-[a-z])/g,n=function(e){return e[1].toUpperCase()},r={};for(var i in e)r[i]=e[i],r[i.replace(t,n)]=e[i];return r}var O=E({"animation-delay":S,"animation-duration":S,"background-position":M,"background-position-x":M,"background-position-y":M,"background-size":M,border:M,"border-bottom":M,"border-bottom-left-radius":M,"border-bottom-right-radius":M,"border-bottom-width":M,"border-left":M,"border-left-width":M,"border-radius":M,"border-right":M,"border-right-width":M,"border-top":M,"border-top-left-radius":M,"border-top-right-radius":M,"border-top-width":M,"border-width":M,"border-block":M,"border-block-end":M,"border-block-end-width":M,"border-block-start":M,"border-block-start-width":M,"border-block-width":M,"border-inline":M,"border-inline-end":M,"border-inline-end-width":M,"border-inline-start":M,"border-inline-start-width":M,"border-inline-width":M,"border-start-start-radius":M,"border-start-end-radius":M,"border-end-start-radius":M,"border-end-end-radius":M,margin:M,"margin-bottom":M,"margin-left":M,"margin-right":M,"margin-top":M,"margin-block":M,"margin-block-end":M,"margin-block-start":M,"margin-inline":M,"margin-inline-end":M,"margin-inline-start":M,padding:M,"padding-bottom":M,"padding-left":M,"padding-right":M,"padding-top":M,"padding-block":M,"padding-block-end":M,"padding-block-start":M,"padding-inline":M,"padding-inline-end":M,"padding-inline-start":M,"mask-position-x":M,"mask-position-y":M,"mask-size":M,height:M,width:M,"min-height":M,"max-height":M,"min-width":M,"max-width":M,bottom:M,left:M,top:M,right:M,inset:M,"inset-block":M,"inset-block-end":M,"inset-block-start":M,"inset-inline":M,"inset-inline-end":M,"inset-inline-start":M,"box-shadow":M,"text-shadow":M,"column-gap":M,"column-rule":M,"column-rule-width":M,"column-width":M,"font-size":M,"font-size-delta":M,"letter-spacing":M,"text-decoration-thickness":M,"text-indent":M,"text-stroke":M,"text-stroke-width":M,"word-spacing":M,motion:M,"motion-offset":M,outline:M,"outline-offset":M,"outline-width":M,perspective:M,"perspective-origin-x":T,"perspective-origin-y":T,"transform-origin":T,"transform-origin-x":T,"transform-origin-y":T,"transform-origin-z":T,"transition-delay":S,"transition-duration":S,"vertical-align":M,"flex-basis":M,"shape-margin":M,size:M,gap:M,grid:M,"grid-gap":M,"row-gap":M,"grid-row-gap":M,"grid-column-gap":M,"grid-template-rows":M,"grid-template-columns":M,"grid-auto-rows":M,"grid-auto-columns":M,"box-shadow-x":M,"box-shadow-y":M,"box-shadow-blur":M,"box-shadow-spread":M,"font-line-height":M,"text-shadow-x":M,"text-shadow-y":M,"text-shadow-blur":M});function k(e,t,n){if(null==t)return t;if(Array.isArray(t))for(var r=0;r<t.length;r++)t[r]=k(e,t[r],n);else if("object"==typeof t)if("fallbacks"===e)for(var i in t)t[i]=k(i,t[i],n);else for(var o in t)t[o]=k(e+"-"+o,t[o],n);else if("number"==typeof t&&!Number.isNaN(t)){var a=n[e]||O[e];return!a||0===t&&a===M?t.toString():"function"==typeof a?a(t).toString():""+t+a}return t}var L=function(e){void 0===e&&(e={});var t=E(e);return{onProcessStyle:function(e,n){if("style"!==n.type)return e;for(var r in e)e[r]=k(r,e[r],t);return e},onChangeValue:function(e,n){return k(n,e,t)}}},D=n(229),P=n(74),N="",j="",R="",z="",F=D.a&&"ontouchstart"in document.documentElement;if(D.a){var B={Moz:"-moz-",ms:"-ms-",O:"-o-",Webkit:"-webkit-"},H=document.createElement("p").style;for(var U in B)if(U+"Transform"in H){N=U,j=B[U];break}"Webkit"===N&&"msHyphens"in H&&(N="ms",j=B.ms,z="edge"),"Webkit"===N&&"-apple-trailing-word"in H&&(R="apple")}var V=N,Y=j,W=R,G=z,X=F;var q={noPrefill:["appearance"],supportedProperty:function(e){return"appearance"===e&&("ms"===V?"-webkit-"+e:Y+e)}},Z={noPrefill:["color-adjust"],supportedProperty:function(e){return"color-adjust"===e&&("Webkit"===V?Y+"print-"+e:e)}},Q=/[-\s]+(.)?/g;function K(e,t){return t?t.toUpperCase():""}function J(e){return e.replace(Q,K)}function $(e){return J("-"+e)}var ee,te={noPrefill:["mask"],supportedProperty:function(e,t){if(!/^mask/.test(e))return!1;if("Webkit"===V){if(J("mask-image")in t)return e;if(V+$("mask-image")in t)return Y+e}return e}},ne={noPrefill:["text-orientation"],supportedProperty:function(e){return"text-orientation"===e&&("apple"!==W||X?e:Y+e)}},re={noPrefill:["transform"],supportedProperty:function(e,t,n){return"transform"===e&&(n.transform?e:Y+e)}},ie={noPrefill:["transition"],supportedProperty:function(e,t,n){return"transition"===e&&(n.transition?e:Y+e)}},oe={noPrefill:["writing-mode"],supportedProperty:function(e){return"writing-mode"===e&&("Webkit"===V||"ms"===V&&"edge"!==G?Y+e:e)}},ae={noPrefill:["user-select"],supportedProperty:function(e){return"user-select"===e&&("Moz"===V||"ms"===V||"apple"===W?Y+e:e)}},se={supportedProperty:function(e,t){return!!/^break-/.test(e)&&("Webkit"===V?"WebkitColumn"+$(e)in t&&Y+"column-"+e:"Moz"===V&&("page"+$(e)in t&&"page-"+e))}},le={supportedProperty:function(e,t){if(!/^(border|margin|padding)-inline/.test(e))return!1;if("Moz"===V)return e;var n=e.replace("-inline","");return V+$(n)in t&&Y+n}},ue={supportedProperty:function(e,t){return J(e)in t&&e}},ce={supportedProperty:function(e,t){var n=$(e);return"-"===e[0]||"-"===e[0]&&"-"===e[1]?e:V+n in t?Y+e:"Webkit"!==V&&"Webkit"+n in t&&"-webkit-"+e}},de={supportedProperty:function(e){return"scroll-snap"===e.substring(0,11)&&("ms"===V?""+Y+e:e)}},he={supportedProperty:function(e){return"overscroll-behavior"===e&&("ms"===V?Y+"scroll-chaining":e)}},fe={"flex-grow":"flex-positive","flex-shrink":"flex-negative","flex-basis":"flex-preferred-size","justify-content":"flex-pack",order:"flex-order","align-items":"flex-align","align-content":"flex-line-pack"},pe={supportedProperty:function(e,t){var n=fe[e];return!!n&&(V+$(n)in t&&Y+n)}},ge={flex:"box-flex","flex-grow":"box-flex","flex-direction":["box-orient","box-direction"],order:"box-ordinal-group","align-items":"box-align","flex-flow":["box-orient","box-direction"],"justify-content":"box-pack"},ve=Object.keys(ge),me=function(e){return Y+e},ye=[q,Z,te,ne,re,ie,oe,ae,se,le,ue,ce,de,he,pe,{supportedProperty:function(e,t,n){var r=n.multiple;if(ve.indexOf(e)>-1){var i=ge[e];if(!Array.isArray(i))return V+$(i)in t&&Y+i;if(!r)return!1;for(var o=0;o<i.length;o++)if(!(V+$(i[0])in t))return!1;return i.map(me)}return!1}}],be=ye.filter((function(e){return e.supportedProperty})).map((function(e){return e.supportedProperty})),Ce=ye.filter((function(e){return e.noPrefill})).reduce((function(e,t){return e.push.apply(e,Object(P.a)(t.noPrefill)),e}),[]),Ae={};if(D.a){ee=document.createElement("p");var _e=window.getComputedStyle(document.documentElement,"");for(var Ie in _e)isNaN(Ie)||(Ae[_e[Ie]]=_e[Ie]);Ce.forEach((function(e){return delete Ae[e]}))}function xe(e,t){if(void 0===t&&(t={}),!ee)return e;if(null!=Ae[e])return Ae[e];"transition"!==e&&"transform"!==e||(t[e]=e in ee.style);for(var n=0;n<be.length&&(Ae[e]=be[n](e,ee.style,t),!Ae[e]);n++);try{ee.style[e]=""}catch(e){return!1}return Ae[e]}var we,Me={},Se={transition:1,"transition-property":1,"-webkit-transition":1,"-webkit-transition-property":1},Te=/(^\s*[\w-]+)|, (\s*[\w-]+)(?![^()]*\))/g;function Ee(e,t,n){if("var"===t)return"var";if("all"===t)return"all";if("all"===n)return", all";var r=t?xe(t):", "+xe(n);return r||(t||n)}function Oe(e,t){var n=t;if(!we||"content"===e)return t;if("string"!=typeof n||!isNaN(parseInt(n,10)))return n;var r=e+n;if(null!=Me[r])return Me[r];try{we.style[e]=n}catch(e){return Me[r]=!1,!1}if(Se[e])n=n.replace(Te,Ee);else if(""===we.style[e]&&("-ms-flex"===(n=Y+n)&&(we.style[e]="-ms-flexbox"),we.style[e]=n,""===we.style[e]))return Me[r]=!1,!1;return we.style[e]="",Me[r]=n,Me[r]}D.a&&(we=document.createElement("p"));var ke=function(){function e(t){for(var n in t){var i=t[n];if("fallbacks"===n&&Array.isArray(i))t[n]=i.map(e);else{var o=!1,a=xe(n);a&&a!==n&&(o=!0);var s=!1,l=Oe(a,Object(r.g)(i));l&&l!==i&&(s=!0),(o||s)&&(o&&delete t[n],t[a||n]=l||i)}}return t}return{onProcessRule:function(e){if("keyframes"===e.type){var t=e;t.at="-"===(n=t.at)[1]||"ms"===V?n:"@"+Y+"keyframes"+n.substr(10)}var n},onProcessStyle:function(t,n){return"style"!==n.type?t:e(t)},onChangeValue:function(e,t){return Oe(t,Object(r.g)(e))||e}}};var Le=function(){var e=function(e,t){return e.length===t.length?e>t?1:-1:e.length-t.length};return{onProcessStyle:function(t,n){if("style"!==n.type)return t;for(var r={},i=Object.keys(t).sort(e),o=0;o<i.length;o++)r[i[o]]=t[i[o]];return r}}};function De(){return{plugins:[s(),p(),y(),w(),L(),"undefined"==typeof window?null:ke(),Le()]}}n.d(t,"a",(function(){return De}))},function(e,t,n){!function(e){"use strict";e.defineMode("javascript",(function(t,n){var r,i,o=t.indentUnit,a=n.statementIndent,s=n.jsonld,l=n.json||s,u=n.typescript,c=n.wordCharacters||/[\w$\xa1-\uffff]/,d=function(){function e(e){return{type:e,style:"keyword"}}var t=e("keyword a"),n=e("keyword b"),r=e("keyword c"),i=e("keyword d"),o=e("operator"),a={type:"atom",style:"atom"};return{if:e("if"),while:t,with:t,else:n,do:n,try:n,finally:n,return:i,break:i,continue:i,new:e("new"),delete:r,void:r,throw:r,debugger:e("debugger"),var:e("var"),const:e("var"),let:e("var"),function:e("function"),catch:e("catch"),for:e("for"),switch:e("switch"),case:e("case"),default:e("default"),in:o,typeof:o,instanceof:o,true:a,false:a,null:a,undefined:a,NaN:a,Infinity:a,this:e("this"),class:e("class"),super:e("atom"),yield:r,export:e("export"),import:e("import"),extends:r,await:r}}(),h=/[+\-*&%=<>!?|~^@]/,f=/^@(context|id|value|language|type|container|list|set|reverse|index|base|vocab|graph)"/;function p(e,t,n){return r=e,i=n,t}function g(e,t){var n,r=e.next();if('"'==r||"'"==r)return t.tokenize=(n=r,function(e,t){var r,i=!1;if(s&&"@"==e.peek()&&e.match(f))return t.tokenize=g,p("jsonld-keyword","meta");for(;null!=(r=e.next())&&(r!=n||i);)i=!i&&"\\"==r;return i||(t.tokenize=g),p("string","string")}),t.tokenize(e,t);if("."==r&&e.match(/^\d[\d_]*(?:[eE][+\-]?[\d_]+)?/))return p("number","number");if("."==r&&e.match(".."))return p("spread","meta");if(/[\[\]{}\(\),;\:\.]/.test(r))return p(r);if("="==r&&e.eat(">"))return p("=>","operator");if("0"==r&&e.match(/^(?:x[\dA-Fa-f_]+|o[0-7_]+|b[01_]+)n?/))return p("number","number");if(/\d/.test(r))return e.match(/^[\d_]*(?:n|(?:\.[\d_]*)?(?:[eE][+\-]?[\d_]+)?)?/),p("number","number");if("/"==r)return e.eat("*")?(t.tokenize=v,v(e,t)):e.eat("/")?(e.skipToEnd(),p("comment","comment")):Xe(e,t,1)?(function(e){for(var t,n=!1,r=!1;null!=(t=e.next());){if(!n){if("/"==t&&!r)return;"["==t?r=!0:r&&"]"==t&&(r=!1)}n=!n&&"\\"==t}}(e),e.match(/^\b(([gimyus])(?![gimyus]*\2))+\b/),p("regexp","string-2")):(e.eat("="),p("operator","operator",e.current()));if("`"==r)return t.tokenize=m,m(e,t);if("#"==r&&"!"==e.peek())return e.skipToEnd(),p("meta","meta");if("#"==r&&e.eatWhile(c))return p("variable","property");if("<"==r&&e.match("!--")||"-"==r&&e.match("->")&&!/\S/.test(e.string.slice(0,e.start)))return e.skipToEnd(),p("comment","comment");if(h.test(r))return">"==r&&t.lexical&&">"==t.lexical.type||(e.eat("=")?"!"!=r&&"="!=r||e.eat("="):/[<>*+\-]/.test(r)&&(e.eat(r),">"==r&&e.eat(r))),"?"==r&&e.eat(".")?p("."):p("operator","operator",e.current());if(c.test(r)){e.eatWhile(c);var i=e.current();if("."!=t.lastType){if(d.propertyIsEnumerable(i)){var o=d[i];return p(o.type,o.style,i)}if("async"==i&&e.match(/^(\s|\/\*.*?\*\/)*[\[\(\w]/,!1))return p("async","keyword",i)}return p("variable","variable",i)}}function v(e,t){for(var n,r=!1;n=e.next();){if("/"==n&&r){t.tokenize=g;break}r="*"==n}return p("comment","comment")}function m(e,t){for(var n,r=!1;null!=(n=e.next());){if(!r&&("`"==n||"$"==n&&e.eat("{"))){t.tokenize=g;break}r=!r&&"\\"==n}return p("quasi","string-2",e.current())}function y(e,t){t.fatArrowAt&&(t.fatArrowAt=null);var n=e.string.indexOf("=>",e.start);if(!(n<0)){if(u){var r=/:\s*(?:\w+(?:<[^>]*>|\[\])?|\{[^}]*\})\s*$/.exec(e.string.slice(e.start,n));r&&(n=r.index)}for(var i=0,o=!1,a=n-1;a>=0;--a){var s=e.string.charAt(a),l="([{}])".indexOf(s);if(l>=0&&l<3){if(!i){++a;break}if(0==--i){"("==s&&(o=!0);break}}else if(l>=3&&l<6)++i;else if(c.test(s))o=!0;else if(/["'\/`]/.test(s))for(;;--a){if(0==a)return;if(e.string.charAt(a-1)==s&&"\\"!=e.string.charAt(a-2)){a--;break}}else if(o&&!i){++a;break}}o&&!i&&(t.fatArrowAt=a)}}var b={atom:!0,number:!0,variable:!0,string:!0,regexp:!0,this:!0,"jsonld-keyword":!0};function C(e,t,n,r,i,o){this.indented=e,this.column=t,this.type=n,this.prev=i,this.info=o,null!=r&&(this.align=r)}function A(e,t){for(var n=e.localVars;n;n=n.next)if(n.name==t)return!0;for(var r=e.context;r;r=r.prev)for(n=r.vars;n;n=n.next)if(n.name==t)return!0}var _={state:null,column:null,marked:null,cc:null};function I(){for(var e=arguments.length-1;e>=0;e--)_.cc.push(arguments[e])}function x(){return I.apply(null,arguments),!0}function w(e,t){for(var n=t;n;n=n.next)if(n.name==e)return!0;return!1}function M(e){var t=_.state;if(_.marked="def",t.context)if("var"==t.lexical.info&&t.context&&t.context.block){var r=function e(t,n){if(n){if(n.block){var r=e(t,n.prev);return r?r==n.prev?n:new T(r,n.vars,!0):null}return w(t,n.vars)?n:new T(n.prev,new E(t,n.vars),!1)}return null}(e,t.context);if(null!=r)return void(t.context=r)}else if(!w(e,t.localVars))return void(t.localVars=new E(e,t.localVars));n.globalVars&&!w(e,t.globalVars)&&(t.globalVars=new E(e,t.globalVars))}function S(e){return"public"==e||"private"==e||"protected"==e||"abstract"==e||"readonly"==e}function T(e,t,n){this.prev=e,this.vars=t,this.block=n}function E(e,t){this.name=e,this.next=t}var O=new E("this",new E("arguments",null));function k(){_.state.context=new T(_.state.context,_.state.localVars,!1),_.state.localVars=O}function L(){_.state.context=new T(_.state.context,_.state.localVars,!0),_.state.localVars=null}function D(){_.state.localVars=_.state.context.vars,_.state.context=_.state.context.prev}function P(e,t){var n=function(){var n=_.state,r=n.indented;if("stat"==n.lexical.type)r=n.lexical.indented;else for(var i=n.lexical;i&&")"==i.type&&i.align;i=i.prev)r=i.indented;n.lexical=new C(r,_.stream.column(),e,null,n.lexical,t)};return n.lex=!0,n}function N(){var e=_.state;e.lexical.prev&&(")"==e.lexical.type&&(e.indented=e.lexical.indented),e.lexical=e.lexical.prev)}function j(e){return function t(n){return n==e?x():";"==e||"}"==n||")"==n||"]"==n?I():x(t)}}function R(e,t){return"var"==e?x(P("vardef",t),ye,j(";"),N):"keyword a"==e?x(P("form"),H,R,N):"keyword b"==e?x(P("form"),R,N):"keyword d"==e?_.stream.match(/^\s*$/,!1)?x():x(P("stat"),V,j(";"),N):"debugger"==e?x(j(";")):"{"==e?x(P("}"),L,oe,N,D):";"==e?x():"if"==e?("else"==_.state.lexical.info&&_.state.cc[_.state.cc.length-1]==N&&_.state.cc.pop()(),x(P("form"),H,R,N,xe)):"function"==e?x(Te):"for"==e?x(P("form"),we,R,N):"class"==e||u&&"interface"==t?(_.marked="keyword",x(P("form","class"==e?e:t),De,N)):"variable"==e?u&&"declare"==t?(_.marked="keyword",x(R)):u&&("module"==t||"enum"==t||"type"==t)&&_.stream.match(/^\s*\w/,!1)?(_.marked="keyword","enum"==t?x(We):"type"==t?x(Oe,j("operator"),ce,j(";")):x(P("form"),be,j("{"),P("}"),oe,N,N)):u&&"namespace"==t?(_.marked="keyword",x(P("form"),F,R,N)):u&&"abstract"==t?(_.marked="keyword",x(R)):x(P("stat"),J):"switch"==e?x(P("form"),H,j("{"),P("}","switch"),L,oe,N,N,D):"case"==e?x(F,j(":")):"default"==e?x(j(":")):"catch"==e?x(P("form"),k,z,R,N,D):"export"==e?x(P("stat"),Re,N):"import"==e?x(P("stat"),Fe,N):"async"==e?x(R):"@"==t?x(F,R):I(P("stat"),F,j(";"),N)}function z(e){if("("==e)return x(ke,j(")"))}function F(e,t){return U(e,t,!1)}function B(e,t){return U(e,t,!0)}function H(e){return"("!=e?I():x(P(")"),V,j(")"),N)}function U(e,t,n){if(_.state.fatArrowAt==_.stream.start){var r=n?Z:q;if("("==e)return x(k,P(")"),re(ke,")"),N,j("=>"),r,D);if("variable"==e)return I(k,be,j("=>"),r,D)}var i=n?W:Y;return b.hasOwnProperty(e)?x(i):"function"==e?x(Te,i):"class"==e||u&&"interface"==t?(_.marked="keyword",x(P("form"),Le,N)):"keyword c"==e||"async"==e?x(n?B:F):"("==e?x(P(")"),V,j(")"),N,i):"operator"==e||"spread"==e?x(n?B:F):"["==e?x(P("]"),Ye,N,i):"{"==e?ie(ee,"}",null,i):"quasi"==e?I(G,i):"new"==e?x(function(e){return function(t){return"."==t?x(e?K:Q):"variable"==t&&u?x(ge,e?W:Y):I(e?B:F)}}(n)):"import"==e?x(F):x()}function V(e){return e.match(/[;\}\)\],]/)?I():I(F)}function Y(e,t){return","==e?x(V):W(e,t,!1)}function W(e,t,n){var r=0==n?Y:W,i=0==n?F:B;return"=>"==e?x(k,n?Z:q,D):"operator"==e?/\+\+|--/.test(t)||u&&"!"==t?x(r):u&&"<"==t&&_.stream.match(/^([^<>]|<[^<>]*>)*>\s*\(/,!1)?x(P(">"),re(ce,">"),N,r):"?"==t?x(F,j(":"),i):x(i):"quasi"==e?I(G,r):";"!=e?"("==e?ie(B,")","call",r):"."==e?x($,r):"["==e?x(P("]"),V,j("]"),N,r):u&&"as"==t?(_.marked="keyword",x(ce,r)):"regexp"==e?(_.state.lastType=_.marked="operator",_.stream.backUp(_.stream.pos-_.stream.start-1),x(i)):void 0:void 0}function G(e,t){return"quasi"!=e?I():"${"!=t.slice(t.length-2)?x(G):x(F,X)}function X(e){if("}"==e)return _.marked="string-2",_.state.tokenize=m,x(G)}function q(e){return y(_.stream,_.state),I("{"==e?R:F)}function Z(e){return y(_.stream,_.state),I("{"==e?R:B)}function Q(e,t){if("target"==t)return _.marked="keyword",x(Y)}function K(e,t){if("target"==t)return _.marked="keyword",x(W)}function J(e){return":"==e?x(N,R):I(Y,j(";"),N)}function $(e){if("variable"==e)return _.marked="property",x()}function ee(e,t){return"async"==e?(_.marked="property",x(ee)):"variable"==e||"keyword"==_.style?(_.marked="property","get"==t||"set"==t?x(te):(u&&_.state.fatArrowAt==_.stream.start&&(n=_.stream.match(/^\s*:\s*/,!1))&&(_.state.fatArrowAt=_.stream.pos+n[0].length),x(ne))):"number"==e||"string"==e?(_.marked=s?"property":_.style+" property",x(ne)):"jsonld-keyword"==e?x(ne):u&&S(t)?(_.marked="keyword",x(ee)):"["==e?x(F,ae,j("]"),ne):"spread"==e?x(B,ne):"*"==t?(_.marked="keyword",x(ee)):":"==e?I(ne):void 0;var n}function te(e){return"variable"!=e?I(ne):(_.marked="property",x(Te))}function ne(e){return":"==e?x(B):"("==e?I(Te):void 0}function re(e,t,n){function r(i,o){if(n?n.indexOf(i)>-1:","==i){var a=_.state.lexical;return"call"==a.info&&(a.pos=(a.pos||0)+1),x((function(n,r){return n==t||r==t?I():I(e)}),r)}return i==t||o==t?x():n&&n.indexOf(";")>-1?I(e):x(j(t))}return function(n,i){return n==t||i==t?x():I(e,r)}}function ie(e,t,n){for(var r=3;r<arguments.length;r++)_.cc.push(arguments[r]);return x(P(t,n),re(e,t),N)}function oe(e){return"}"==e?x():I(R,oe)}function ae(e,t){if(u){if(":"==e)return x(ce);if("?"==t)return x(ae)}}function se(e,t){if(u&&(":"==e||"in"==t))return x(ce)}function le(e){if(u&&":"==e)return _.stream.match(/^\s*\w+\s+is\b/,!1)?x(F,ue,ce):x(ce)}function ue(e,t){if("is"==t)return _.marked="keyword",x()}function ce(e,t){return"keyof"==t||"typeof"==t||"infer"==t?(_.marked="keyword",x("typeof"==t?B:ce)):"variable"==e||"void"==t?(_.marked="type",x(pe)):"|"==t||"&"==t?x(ce):"string"==e||"number"==e||"atom"==e?x(pe):"["==e?x(P("]"),re(ce,"]",","),N,pe):"{"==e?x(P("}"),re(he,"}",",;"),N,pe):"("==e?x(re(fe,")"),de,pe):"<"==e?x(re(ce,">"),ce):void 0}function de(e){if("=>"==e)return x(ce)}function he(e,t){return"variable"==e||"keyword"==_.style?(_.marked="property",x(he)):"?"==t||"number"==e||"string"==e?x(he):":"==e?x(ce):"["==e?x(j("variable"),se,j("]"),he):"("==e?I(Ee,he):void 0}function fe(e,t){return"variable"==e&&_.stream.match(/^\s*[?:]/,!1)||"?"==t?x(fe):":"==e?x(ce):"spread"==e?x(fe):I(ce)}function pe(e,t){return"<"==t?x(P(">"),re(ce,">"),N,pe):"|"==t||"."==e||"&"==t?x(ce):"["==e?x(ce,j("]"),pe):"extends"==t||"implements"==t?(_.marked="keyword",x(ce)):"?"==t?x(ce,j(":"),ce):void 0}function ge(e,t){if("<"==t)return x(P(">"),re(ce,">"),N,pe)}function ve(){return I(ce,me)}function me(e,t){if("="==t)return x(ce)}function ye(e,t){return"enum"==t?(_.marked="keyword",x(We)):I(be,ae,_e,Ie)}function be(e,t){return u&&S(t)?(_.marked="keyword",x(be)):"variable"==e?(M(t),x()):"spread"==e?x(be):"["==e?ie(Ae,"]"):"{"==e?ie(Ce,"}"):void 0}function Ce(e,t){return"variable"!=e||_.stream.match(/^\s*:/,!1)?("variable"==e&&(_.marked="property"),"spread"==e?x(be):"}"==e?I():"["==e?x(F,j("]"),j(":"),Ce):x(j(":"),be,_e)):(M(t),x(_e))}function Ae(){return I(be,_e)}function _e(e,t){if("="==t)return x(B)}function Ie(e){if(","==e)return x(ye)}function xe(e,t){if("keyword b"==e&&"else"==t)return x(P("form","else"),R,N)}function we(e,t){return"await"==t?x(we):"("==e?x(P(")"),Me,N):void 0}function Me(e){return"var"==e?x(ye,Se):"variable"==e?x(Se):I(Se)}function Se(e,t){return")"==e?x():";"==e?x(Se):"in"==t||"of"==t?(_.marked="keyword",x(F,Se)):I(F,Se)}function Te(e,t){return"*"==t?(_.marked="keyword",x(Te)):"variable"==e?(M(t),x(Te)):"("==e?x(k,P(")"),re(ke,")"),N,le,R,D):u&&"<"==t?x(P(">"),re(ve,">"),N,Te):void 0}function Ee(e,t){return"*"==t?(_.marked="keyword",x(Ee)):"variable"==e?(M(t),x(Ee)):"("==e?x(k,P(")"),re(ke,")"),N,le,D):u&&"<"==t?x(P(">"),re(ve,">"),N,Ee):void 0}function Oe(e,t){return"keyword"==e||"variable"==e?(_.marked="type",x(Oe)):"<"==t?x(P(">"),re(ve,">"),N):void 0}function ke(e,t){return"@"==t&&x(F,ke),"spread"==e?x(ke):u&&S(t)?(_.marked="keyword",x(ke)):u&&"this"==e?x(ae,_e):I(be,ae,_e)}function Le(e,t){return"variable"==e?De(e,t):Pe(e,t)}function De(e,t){if("variable"==e)return M(t),x(Pe)}function Pe(e,t){return"<"==t?x(P(">"),re(ve,">"),N,Pe):"extends"==t||"implements"==t||u&&","==e?("implements"==t&&(_.marked="keyword"),x(u?ce:F,Pe)):"{"==e?x(P("}"),Ne,N):void 0}function Ne(e,t){return"async"==e||"variable"==e&&("static"==t||"get"==t||"set"==t||u&&S(t))&&_.stream.match(/^\s+[\w$\xa1-\uffff]/,!1)?(_.marked="keyword",x(Ne)):"variable"==e||"keyword"==_.style?(_.marked="property",x(je,Ne)):"number"==e||"string"==e?x(je,Ne):"["==e?x(F,ae,j("]"),je,Ne):"*"==t?(_.marked="keyword",x(Ne)):u&&"("==e?I(Ee,Ne):";"==e||","==e?x(Ne):"}"==e?x():"@"==t?x(F,Ne):void 0}function je(e,t){if("?"==t)return x(je);if(":"==e)return x(ce,_e);if("="==t)return x(B);var n=_.state.lexical.prev;return I(n&&"interface"==n.info?Ee:Te)}function Re(e,t){return"*"==t?(_.marked="keyword",x(Ve,j(";"))):"default"==t?(_.marked="keyword",x(F,j(";"))):"{"==e?x(re(ze,"}"),Ve,j(";")):I(R)}function ze(e,t){return"as"==t?(_.marked="keyword",x(j("variable"))):"variable"==e?I(B,ze):void 0}function Fe(e){return"string"==e?x():"("==e?I(F):I(Be,He,Ve)}function Be(e,t){return"{"==e?ie(Be,"}"):("variable"==e&&M(t),"*"==t&&(_.marked="keyword"),x(Ue))}function He(e){if(","==e)return x(Be,He)}function Ue(e,t){if("as"==t)return _.marked="keyword",x(Be)}function Ve(e,t){if("from"==t)return _.marked="keyword",x(F)}function Ye(e){return"]"==e?x():I(re(B,"]"))}function We(){return I(P("form"),be,j("{"),P("}"),re(Ge,"}"),N,N)}function Ge(){return I(be,_e)}function Xe(e,t,n){return t.tokenize==g&&/^(?:operator|sof|keyword [bcd]|case|new|export|default|spread|[\[{}\(,;:]|=>)$/.test(t.lastType)||"quasi"==t.lastType&&/\{\s*$/.test(e.string.slice(0,e.pos-(n||0)))}return D.lex=!0,N.lex=!0,{startState:function(e){var t={tokenize:g,lastType:"sof",cc:[],lexical:new C((e||0)-o,0,"block",!1),localVars:n.localVars,context:n.localVars&&new T(null,null,!1),indented:e||0};return n.globalVars&&"object"==typeof n.globalVars&&(t.globalVars=n.globalVars),t},token:function(e,t){if(e.sol()&&(t.lexical.hasOwnProperty("align")||(t.lexical.align=!1),t.indented=e.indentation(),y(e,t)),t.tokenize!=v&&e.eatSpace())return null;var n=t.tokenize(e,t);return"comment"==r?n:(t.lastType="operator"!=r||"++"!=i&&"--"!=i?r:"incdec",function(e,t,n,r,i){var o=e.cc;for(_.state=e,_.stream=i,_.marked=null,_.cc=o,_.style=t,e.lexical.hasOwnProperty("align")||(e.lexical.align=!0);;)if((o.length?o.pop():l?F:R)(n,r)){for(;o.length&&o[o.length-1].lex;)o.pop()();return _.marked?_.marked:"variable"==n&&A(e,r)?"variable-2":t}}(t,n,r,i,e))},indent:function(t,r){if(t.tokenize==v)return e.Pass;if(t.tokenize!=g)return 0;var i,s=r&&r.charAt(0),l=t.lexical;if(!/^\s*else\b/.test(r))for(var u=t.cc.length-1;u>=0;--u){var c=t.cc[u];if(c==N)l=l.prev;else if(c!=xe)break}for(;("stat"==l.type||"form"==l.type)&&("}"==s||(i=t.cc[t.cc.length-1])&&(i==Y||i==W)&&!/^[,\.=+\-*:?[\(]/.test(r));)l=l.prev;a&&")"==l.type&&"stat"==l.prev.type&&(l=l.prev);var d=l.type,f=s==d;return"vardef"==d?l.indented+("operator"==t.lastType||","==t.lastType?l.info.length+1:0):"form"==d&&"{"==s?l.indented:"form"==d?l.indented+o:"stat"==d?l.indented+(function(e,t){return"operator"==e.lastType||","==e.lastType||h.test(t.charAt(0))||/[,.]/.test(t.charAt(0))}(t,r)?a||o:0):"switch"!=l.info||f||0==n.doubleIndentSwitch?l.align?l.column+(f?0:1):l.indented+(f?0:o):l.indented+(/^(?:case|default)\b/.test(r)?o:2*o)},electricInput:/^\s*(?:case .*?:|default:|\{|\})$/,blockCommentStart:l?null:"/*",blockCommentEnd:l?null:"*/",blockCommentContinue:l?null:" * ",lineComment:l?null:"//",fold:"brace",closeBrackets:"()[]{}''\"\"``",helperType:l?"json":"javascript",jsonldMode:s,jsonMode:l,expressionAllowed:Xe,skipExpression:function(e){var t=e.cc[e.cc.length-1];t!=F&&t!=B||e.cc.pop()}}})),e.registerHelper("wordChars","javascript",/[\w$]/),e.defineMIME("text/javascript","javascript"),e.defineMIME("text/ecmascript","javascript"),e.defineMIME("application/javascript","javascript"),e.defineMIME("application/x-javascript","javascript"),e.defineMIME("application/ecmascript","javascript"),e.defineMIME("application/json",{name:"javascript",json:!0}),e.defineMIME("application/x-json",{name:"javascript",json:!0}),e.defineMIME("application/ld+json",{name:"javascript",jsonld:!0}),e.defineMIME("text/typescript",{name:"javascript",typescript:!0}),e.defineMIME("application/typescript",{name:"javascript",typescript:!0})}(n(24))},function(e,t,n){!function(e){"use strict";function t(e,t){if(!e.hasOwnProperty(t))throw new Error("Undefined state "+t+" in simple mode")}function n(e,t){if(!e)return/(?:)/;var n="";return e instanceof RegExp?(e.ignoreCase&&(n="i"),e=e.source):e=String(e),new RegExp((!1===t?"":"^")+"(?:"+e+")",n)}function r(e,r){(e.next||e.push)&&t(r,e.next||e.push),this.regex=n(e.regex),this.token=function(e){if(!e)return null;if(e.apply)return e;if("string"==typeof e)return e.replace(/\./g," ");for(var t=[],n=0;n<e.length;n++)t.push(e[n]&&e[n].replace(/\./g," "));return t}(e.token),this.data=e}function i(e,t){return function(n,r){if(r.pending){var i=r.pending.shift();return 0==r.pending.length&&(r.pending=null),n.pos+=i.text.length,i.token}if(r.local){if(r.local.end&&n.match(r.local.end)){var o=r.local.endToken||null;return r.local=r.localState=null,o}var s;return o=r.local.mode.token(n,r.localState),r.local.endScan&&(s=r.local.endScan.exec(n.current()))&&(n.pos=n.start+s.index),o}for(var l=e[r.state],u=0;u<l.length;u++){var c=l[u],d=(!c.data.sol||n.sol())&&n.match(c.regex);if(d){c.data.next?r.state=c.data.next:c.data.push?((r.stack||(r.stack=[])).push(r.state),r.state=c.data.push):c.data.pop&&r.stack&&r.stack.length&&(r.state=r.stack.pop()),c.data.mode&&a(t,r,c.data.mode,c.token),c.data.indent&&r.indent.push(n.indentation()+t.indentUnit),c.data.dedent&&r.indent.pop();var h=c.token;if(h&&h.apply&&(h=h(d)),d.length>2&&c.token&&"string"!=typeof c.token){r.pending=[];for(var f=2;f<d.length;f++)d[f]&&r.pending.push({text:d[f],token:c.token[f-1]});return n.backUp(d[0].length-(d[1]?d[1].length:0)),h[0]}return h&&h.join?h[0]:h}}return n.next(),null}}function o(e,t){if(e===t)return!0;if(!e||"object"!=typeof e||!t||"object"!=typeof t)return!1;var n=0;for(var r in e)if(e.hasOwnProperty(r)){if(!t.hasOwnProperty(r)||!o(e[r],t[r]))return!1;n++}for(var r in t)t.hasOwnProperty(r)&&n--;return 0==n}function a(t,r,i,a){var s;if(i.persistent)for(var l=r.persistentStates;l&&!s;l=l.next)(i.spec?o(i.spec,l.spec):i.mode==l.mode)&&(s=l);var u=s?s.mode:i.mode||e.getMode(t,i.spec),c=s?s.state:e.startState(u);i.persistent&&!s&&(r.persistentStates={mode:u,spec:i.spec,state:c,next:r.persistentStates}),r.localState=c,r.local={mode:u,end:i.end&&n(i.end),endScan:i.end&&!1!==i.forceEnd&&n(i.end,!1),endToken:a&&a.join?a[a.length-1]:a}}function s(t,n){return function(r,i,o){if(r.local&&r.local.mode.indent)return r.local.mode.indent(r.localState,i,o);if(null==r.indent||r.local||n.dontIndentStates&&function(e,t){for(var n=0;n<t.length;n++)if(t[n]===e)return!0}(r.state,n.dontIndentStates)>-1)return e.Pass;var a=r.indent.length-1,s=t[r.state];e:for(;;){for(var l=0;l<s.length;l++){var u=s[l];if(u.data.dedent&&!1!==u.data.dedentIfLineStart){var c=u.regex.exec(i);if(c&&c[0]){a--,(u.next||u.push)&&(s=t[u.next||u.push]),i=i.slice(c[0].length);continue e}}}break}return a<0?0:r.indent[a]}}e.defineSimpleMode=function(t,n){e.defineMode(t,(function(t){return e.simpleMode(t,n)}))},e.simpleMode=function(n,o){t(o,"start");var a={},l=o.meta||{},u=!1;for(var c in o)if(c!=l&&o.hasOwnProperty(c))for(var d=a[c]=[],h=o[c],f=0;f<h.length;f++){var p=h[f];d.push(new r(p,o)),(p.indent||p.dedent)&&(u=!0)}var g={startState:function(){return{state:"start",pending:null,local:null,localState:null,indent:u?[]:null}},copyState:function(t){var n={state:t.state,pending:t.pending,local:t.local,localState:null,indent:t.indent&&t.indent.slice(0)};t.localState&&(n.localState=e.copyState(t.local.mode,t.localState)),t.stack&&(n.stack=t.stack.slice(0));for(var r=t.persistentStates;r;r=r.next)n.persistentStates={mode:r.mode,spec:r.spec,state:r.state==t.localState?n.localState:e.copyState(r.mode,r.state),next:n.persistentStates};return n},token:i(a,n),innerMode:function(e){return e.local&&{mode:e.local.mode,state:e.localState}},indent:s(a,l)};if(l)for(var v in l)l.hasOwnProperty(v)&&(g[v]=l[v]);return g}}(n(24))},function(e,t,n){!function(e){"use strict";function t(e){for(var t={},n=0;n<e.length;++n)t[e[n].toLowerCase()]=!0;return t}e.defineMode("css",(function(t,n){var r=n.inline;n.propertyKeywords||(n=e.resolveMode("text/css"));var i,o,a=t.indentUnit,s=n.tokenHooks,l=n.documentTypes||{},u=n.mediaTypes||{},c=n.mediaFeatures||{},d=n.mediaValueKeywords||{},h=n.propertyKeywords||{},f=n.nonStandardPropertyKeywords||{},p=n.fontProperties||{},g=n.counterDescriptors||{},v=n.colorKeywords||{},m=n.valueKeywords||{},y=n.allowNested,b=n.lineComment,C=!0===n.supportsAtComponent;function A(e,t){return i=t,e}function _(e,t){var n=e.next();if(s[n]){var r=s[n](e,t);if(!1!==r)return r}return"@"==n?(e.eatWhile(/[\w\\\-]/),A("def",e.current())):"="==n||("~"==n||"|"==n)&&e.eat("=")?A(null,"compare"):'"'==n||"'"==n?(t.tokenize=I(n),t.tokenize(e,t)):"#"==n?(e.eatWhile(/[\w\\\-]/),A("atom","hash")):"!"==n?(e.match(/^\s*\w*/),A("keyword","important")):/\d/.test(n)||"."==n&&e.eat(/\d/)?(e.eatWhile(/[\w.%]/),A("number","unit")):"-"!==n?/[,+>*\/]/.test(n)?A(null,"select-op"):"."==n&&e.match(/^-?[_a-z][_a-z0-9-]*/i)?A("qualifier","qualifier"):/[:;{}\[\]\(\)]/.test(n)?A(null,n):e.match(/[\w-.]+(?=\()/)?(/^(url(-prefix)?|domain|regexp)$/.test(e.current().toLowerCase())&&(t.tokenize=x),A("variable callee","variable")):/[\w\\\-]/.test(n)?(e.eatWhile(/[\w\\\-]/),A("property","word")):A(null,null):/[\d.]/.test(e.peek())?(e.eatWhile(/[\w.%]/),A("number","unit")):e.match(/^-[\w\\\-]*/)?(e.eatWhile(/[\w\\\-]/),e.match(/^\s*:/,!1)?A("variable-2","variable-definition"):A("variable-2","variable")):e.match(/^\w+-/)?A("meta","meta"):void 0}function I(e){return function(t,n){for(var r,i=!1;null!=(r=t.next());){if(r==e&&!i){")"==e&&t.backUp(1);break}i=!i&&"\\"==r}return(r==e||!i&&")"!=e)&&(n.tokenize=null),A("string","string")}}function x(e,t){return e.next(),e.match(/\s*[\"\')]/,!1)?t.tokenize=null:t.tokenize=I(")"),A(null,"(")}function w(e,t,n){this.type=e,this.indent=t,this.prev=n}function M(e,t,n,r){return e.context=new w(n,t.indentation()+(!1===r?0:a),e.context),n}function S(e){return e.context.prev&&(e.context=e.context.prev),e.context.type}function T(e,t,n){return k[n.context.type](e,t,n)}function E(e,t,n,r){for(var i=r||1;i>0;i--)n.context=n.context.prev;return T(e,t,n)}function O(e){var t=e.current().toLowerCase();o=m.hasOwnProperty(t)?"atom":v.hasOwnProperty(t)?"keyword":"variable"}var k={top:function(e,t,n){if("{"==e)return M(n,t,"block");if("}"==e&&n.context.prev)return S(n);if(C&&/@component/i.test(e))return M(n,t,"atComponentBlock");if(/^@(-moz-)?document$/i.test(e))return M(n,t,"documentTypes");if(/^@(media|supports|(-moz-)?document|import)$/i.test(e))return M(n,t,"atBlock");if(/^@(font-face|counter-style)/i.test(e))return n.stateArg=e,"restricted_atBlock_before";if(/^@(-(moz|ms|o|webkit)-)?keyframes$/i.test(e))return"keyframes";if(e&&"@"==e.charAt(0))return M(n,t,"at");if("hash"==e)o="builtin";else if("word"==e)o="tag";else{if("variable-definition"==e)return"maybeprop";if("interpolation"==e)return M(n,t,"interpolation");if(":"==e)return"pseudo";if(y&&"("==e)return M(n,t,"parens")}return n.context.type},block:function(e,t,n){if("word"==e){var r=t.current().toLowerCase();return h.hasOwnProperty(r)?(o="property","maybeprop"):f.hasOwnProperty(r)?(o="string-2","maybeprop"):y?(o=t.match(/^\s*:(?:\s|$)/,!1)?"property":"tag","block"):(o+=" error","maybeprop")}return"meta"==e?"block":y||"hash"!=e&&"qualifier"!=e?k.top(e,t,n):(o="error","block")},maybeprop:function(e,t,n){return":"==e?M(n,t,"prop"):T(e,t,n)},prop:function(e,t,n){if(";"==e)return S(n);if("{"==e&&y)return M(n,t,"propBlock");if("}"==e||"{"==e)return E(e,t,n);if("("==e)return M(n,t,"parens");if("hash"!=e||/^#([0-9a-fA-f]{3,4}|[0-9a-fA-f]{6}|[0-9a-fA-f]{8})$/.test(t.current())){if("word"==e)O(t);else if("interpolation"==e)return M(n,t,"interpolation")}else o+=" error";return"prop"},propBlock:function(e,t,n){return"}"==e?S(n):"word"==e?(o="property","maybeprop"):n.context.type},parens:function(e,t,n){return"{"==e||"}"==e?E(e,t,n):")"==e?S(n):"("==e?M(n,t,"parens"):"interpolation"==e?M(n,t,"interpolation"):("word"==e&&O(t),"parens")},pseudo:function(e,t,n){return"meta"==e?"pseudo":"word"==e?(o="variable-3",n.context.type):T(e,t,n)},documentTypes:function(e,t,n){return"word"==e&&l.hasOwnProperty(t.current())?(o="tag",n.context.type):k.atBlock(e,t,n)},atBlock:function(e,t,n){if("("==e)return M(n,t,"atBlock_parens");if("}"==e||";"==e)return E(e,t,n);if("{"==e)return S(n)&&M(n,t,y?"block":"top");if("interpolation"==e)return M(n,t,"interpolation");if("word"==e){var r=t.current().toLowerCase();o="only"==r||"not"==r||"and"==r||"or"==r?"keyword":u.hasOwnProperty(r)?"attribute":c.hasOwnProperty(r)?"property":d.hasOwnProperty(r)?"keyword":h.hasOwnProperty(r)?"property":f.hasOwnProperty(r)?"string-2":m.hasOwnProperty(r)?"atom":v.hasOwnProperty(r)?"keyword":"error"}return n.context.type},atComponentBlock:function(e,t,n){return"}"==e?E(e,t,n):"{"==e?S(n)&&M(n,t,y?"block":"top",!1):("word"==e&&(o="error"),n.context.type)},atBlock_parens:function(e,t,n){return")"==e?S(n):"{"==e||"}"==e?E(e,t,n,2):k.atBlock(e,t,n)},restricted_atBlock_before:function(e,t,n){return"{"==e?M(n,t,"restricted_atBlock"):"word"==e&&"@counter-style"==n.stateArg?(o="variable","restricted_atBlock_before"):T(e,t,n)},restricted_atBlock:function(e,t,n){return"}"==e?(n.stateArg=null,S(n)):"word"==e?(o="@font-face"==n.stateArg&&!p.hasOwnProperty(t.current().toLowerCase())||"@counter-style"==n.stateArg&&!g.hasOwnProperty(t.current().toLowerCase())?"error":"property","maybeprop"):"restricted_atBlock"},keyframes:function(e,t,n){return"word"==e?(o="variable","keyframes"):"{"==e?M(n,t,"top"):T(e,t,n)},at:function(e,t,n){return";"==e?S(n):"{"==e||"}"==e?E(e,t,n):("word"==e?o="tag":"hash"==e&&(o="builtin"),"at")},interpolation:function(e,t,n){return"}"==e?S(n):"{"==e||";"==e?E(e,t,n):("word"==e?o="variable":"variable"!=e&&"("!=e&&")"!=e&&(o="error"),"interpolation")}};return{startState:function(e){return{tokenize:null,state:r?"block":"top",stateArg:null,context:new w(r?"block":"top",e||0,null)}},token:function(e,t){if(!t.tokenize&&e.eatSpace())return null;var n=(t.tokenize||_)(e,t);return n&&"object"==typeof n&&(i=n[1],n=n[0]),o=n,"comment"!=i&&(t.state=k[t.state](i,e,t)),o},indent:function(e,t){var n=e.context,r=t&&t.charAt(0),i=n.indent;return"prop"!=n.type||"}"!=r&&")"!=r||(n=n.prev),n.prev&&("}"!=r||"block"!=n.type&&"top"!=n.type&&"interpolation"!=n.type&&"restricted_atBlock"!=n.type?(")"!=r||"parens"!=n.type&&"atBlock_parens"!=n.type)&&("{"!=r||"at"!=n.type&&"atBlock"!=n.type)||(i=Math.max(0,n.indent-a)):i=(n=n.prev).indent),i},electricChars:"}",blockCommentStart:"/*",blockCommentEnd:"*/",blockCommentContinue:" * ",lineComment:b,fold:"brace"}}));var n=["domain","regexp","url","url-prefix"],r=t(n),i=["all","aural","braille","handheld","print","projection","screen","tty","tv","embossed"],o=t(i),a=["width","min-width","max-width","height","min-height","max-height","device-width","min-device-width","max-device-width","device-height","min-device-height","max-device-height","aspect-ratio","min-aspect-ratio","max-aspect-ratio","device-aspect-ratio","min-device-aspect-ratio","max-device-aspect-ratio","color","min-color","max-color","color-index","min-color-index","max-color-index","monochrome","min-monochrome","max-monochrome","resolution","min-resolution","max-resolution","scan","grid","orientation","device-pixel-ratio","min-device-pixel-ratio","max-device-pixel-ratio","pointer","any-pointer","hover","any-hover","prefers-color-scheme"],s=t(a),l=["landscape","portrait","none","coarse","fine","on-demand","hover","interlace","progressive","dark","light"],u=t(l),c=["align-content","align-items","align-self","alignment-adjust","alignment-baseline","all","anchor-point","animation","animation-delay","animation-direction","animation-duration","animation-fill-mode","animation-iteration-count","animation-name","animation-play-state","animation-timing-function","appearance","azimuth","backdrop-filter","backface-visibility","background","background-attachment","background-blend-mode","background-clip","background-color","background-image","background-origin","background-position","background-position-x","background-position-y","background-repeat","background-size","baseline-shift","binding","bleed","block-size","bookmark-label","bookmark-level","bookmark-state","bookmark-target","border","border-bottom","border-bottom-color","border-bottom-left-radius","border-bottom-right-radius","border-bottom-style","border-bottom-width","border-collapse","border-color","border-image","border-image-outset","border-image-repeat","border-image-slice","border-image-source","border-image-width","border-left","border-left-color","border-left-style","border-left-width","border-radius","border-right","border-right-color","border-right-style","border-right-width","border-spacing","border-style","border-top","border-top-color","border-top-left-radius","border-top-right-radius","border-top-style","border-top-width","border-width","bottom","box-decoration-break","box-shadow","box-sizing","break-after","break-before","break-inside","caption-side","caret-color","clear","clip","color","color-profile","column-count","column-fill","column-gap","column-rule","column-rule-color","column-rule-style","column-rule-width","column-span","column-width","columns","contain","content","counter-increment","counter-reset","crop","cue","cue-after","cue-before","cursor","direction","display","dominant-baseline","drop-initial-after-adjust","drop-initial-after-align","drop-initial-before-adjust","drop-initial-before-align","drop-initial-size","drop-initial-value","elevation","empty-cells","fit","fit-position","flex","flex-basis","flex-direction","flex-flow","flex-grow","flex-shrink","flex-wrap","float","float-offset","flow-from","flow-into","font","font-family","font-feature-settings","font-kerning","font-language-override","font-optical-sizing","font-size","font-size-adjust","font-stretch","font-style","font-synthesis","font-variant","font-variant-alternates","font-variant-caps","font-variant-east-asian","font-variant-ligatures","font-variant-numeric","font-variant-position","font-variation-settings","font-weight","gap","grid","grid-area","grid-auto-columns","grid-auto-flow","grid-auto-rows","grid-column","grid-column-end","grid-column-gap","grid-column-start","grid-gap","grid-row","grid-row-end","grid-row-gap","grid-row-start","grid-template","grid-template-areas","grid-template-columns","grid-template-rows","hanging-punctuation","height","hyphens","icon","image-orientation","image-rendering","image-resolution","inline-box-align","inset","inset-block","inset-block-end","inset-block-start","inset-inline","inset-inline-end","inset-inline-start","isolation","justify-content","justify-items","justify-self","left","letter-spacing","line-break","line-height","line-height-step","line-stacking","line-stacking-ruby","line-stacking-shift","line-stacking-strategy","list-style","list-style-image","list-style-position","list-style-type","margin","margin-bottom","margin-left","margin-right","margin-top","marks","marquee-direction","marquee-loop","marquee-play-count","marquee-speed","marquee-style","mask-clip","mask-composite","mask-image","mask-mode","mask-origin","mask-position","mask-repeat","mask-size","mask-type","max-block-size","max-height","max-inline-size","max-width","min-block-size","min-height","min-inline-size","min-width","mix-blend-mode","move-to","nav-down","nav-index","nav-left","nav-right","nav-up","object-fit","object-position","offset","offset-anchor","offset-distance","offset-path","offset-position","offset-rotate","opacity","order","orphans","outline","outline-color","outline-offset","outline-style","outline-width","overflow","overflow-style","overflow-wrap","overflow-x","overflow-y","padding","padding-bottom","padding-left","padding-right","padding-top","page","page-break-after","page-break-before","page-break-inside","page-policy","pause","pause-after","pause-before","perspective","perspective-origin","pitch","pitch-range","place-content","place-items","place-self","play-during","position","presentation-level","punctuation-trim","quotes","region-break-after","region-break-before","region-break-inside","region-fragment","rendering-intent","resize","rest","rest-after","rest-before","richness","right","rotate","rotation","rotation-point","row-gap","ruby-align","ruby-overhang","ruby-position","ruby-span","scale","scroll-behavior","scroll-margin","scroll-margin-block","scroll-margin-block-end","scroll-margin-block-start","scroll-margin-bottom","scroll-margin-inline","scroll-margin-inline-end","scroll-margin-inline-start","scroll-margin-left","scroll-margin-right","scroll-margin-top","scroll-padding","scroll-padding-block","scroll-padding-block-end","scroll-padding-block-start","scroll-padding-bottom","scroll-padding-inline","scroll-padding-inline-end","scroll-padding-inline-start","scroll-padding-left","scroll-padding-right","scroll-padding-top","scroll-snap-align","scroll-snap-type","shape-image-threshold","shape-inside","shape-margin","shape-outside","size","speak","speak-as","speak-header","speak-numeral","speak-punctuation","speech-rate","stress","string-set","tab-size","table-layout","target","target-name","target-new","target-position","text-align","text-align-last","text-combine-upright","text-decoration","text-decoration-color","text-decoration-line","text-decoration-skip","text-decoration-skip-ink","text-decoration-style","text-emphasis","text-emphasis-color","text-emphasis-position","text-emphasis-style","text-height","text-indent","text-justify","text-orientation","text-outline","text-overflow","text-rendering","text-shadow","text-size-adjust","text-space-collapse","text-transform","text-underline-position","text-wrap","top","touch-action","transform","transform-origin","transform-style","transition","transition-delay","transition-duration","transition-property","transition-timing-function","translate","unicode-bidi","user-select","vertical-align","visibility","voice-balance","voice-duration","voice-family","voice-pitch","voice-range","voice-rate","voice-stress","voice-volume","volume","white-space","widows","width","will-change","word-break","word-spacing","word-wrap","writing-mode","z-index","clip-path","clip-rule","mask","enable-background","filter","flood-color","flood-opacity","lighting-color","stop-color","stop-opacity","pointer-events","color-interpolation","color-interpolation-filters","color-rendering","fill","fill-opacity","fill-rule","image-rendering","marker","marker-end","marker-mid","marker-start","paint-order","shape-rendering","stroke","stroke-dasharray","stroke-dashoffset","stroke-linecap","stroke-linejoin","stroke-miterlimit","stroke-opacity","stroke-width","text-rendering","baseline-shift","dominant-baseline","glyph-orientation-horizontal","glyph-orientation-vertical","text-anchor","writing-mode"],d=t(c),h=["border-block","border-block-color","border-block-end","border-block-end-color","border-block-end-style","border-block-end-width","border-block-start","border-block-start-color","border-block-start-style","border-block-start-width","border-block-style","border-block-width","border-inline","border-inline-color","border-inline-end","border-inline-end-color","border-inline-end-style","border-inline-end-width","border-inline-start","border-inline-start-color","border-inline-start-style","border-inline-start-width","border-inline-style","border-inline-width","margin-block","margin-block-end","margin-block-start","margin-inline","margin-inline-end","margin-inline-start","padding-block","padding-block-end","padding-block-start","padding-inline","padding-inline-end","padding-inline-start","scroll-snap-stop","scrollbar-3d-light-color","scrollbar-arrow-color","scrollbar-base-color","scrollbar-dark-shadow-color","scrollbar-face-color","scrollbar-highlight-color","scrollbar-shadow-color","scrollbar-track-color","searchfield-cancel-button","searchfield-decoration","searchfield-results-button","searchfield-results-decoration","shape-inside","zoom"],f=t(h),p=t(["font-display","font-family","src","unicode-range","font-variant","font-feature-settings","font-stretch","font-weight","font-style"]),g=t(["additive-symbols","fallback","negative","pad","prefix","range","speak-as","suffix","symbols","system"]),v=["aliceblue","antiquewhite","aqua","aquamarine","azure","beige","bisque","black","blanchedalmond","blue","blueviolet","brown","burlywood","cadetblue","chartreuse","chocolate","coral","cornflowerblue","cornsilk","crimson","cyan","darkblue","darkcyan","darkgoldenrod","darkgray","darkgreen","darkkhaki","darkmagenta","darkolivegreen","darkorange","darkorchid","darkred","darksalmon","darkseagreen","darkslateblue","darkslategray","darkturquoise","darkviolet","deeppink","deepskyblue","dimgray","dodgerblue","firebrick","floralwhite","forestgreen","fuchsia","gainsboro","ghostwhite","gold","goldenrod","gray","grey","green","greenyellow","honeydew","hotpink","indianred","indigo","ivory","khaki","lavender","lavenderblush","lawngreen","lemonchiffon","lightblue","lightcoral","lightcyan","lightgoldenrodyellow","lightgray","lightgreen","lightpink","lightsalmon","lightseagreen","lightskyblue","lightslategray","lightsteelblue","lightyellow","lime","limegreen","linen","magenta","maroon","mediumaquamarine","mediumblue","mediumorchid","mediumpurple","mediumseagreen","mediumslateblue","mediumspringgreen","mediumturquoise","mediumvioletred","midnightblue","mintcream","mistyrose","moccasin","navajowhite","navy","oldlace","olive","olivedrab","orange","orangered","orchid","palegoldenrod","palegreen","paleturquoise","palevioletred","papayawhip","peachpuff","peru","pink","plum","powderblue","purple","rebeccapurple","red","rosybrown","royalblue","saddlebrown","salmon","sandybrown","seagreen","seashell","sienna","silver","skyblue","slateblue","slategray","snow","springgreen","steelblue","tan","teal","thistle","tomato","turquoise","violet","wheat","white","whitesmoke","yellow","yellowgreen"],m=t(v),y=["above","absolute","activeborder","additive","activecaption","afar","after-white-space","ahead","alias","all","all-scroll","alphabetic","alternate","always","amharic","amharic-abegede","antialiased","appworkspace","arabic-indic","armenian","asterisks","attr","auto","auto-flow","avoid","avoid-column","avoid-page","avoid-region","axis-pan","background","backwards","baseline","below","bidi-override","binary","bengali","blink","block","block-axis","bold","bolder","border","border-box","both","bottom","break","break-all","break-word","bullets","button","button-bevel","buttonface","buttonhighlight","buttonshadow","buttontext","calc","cambodian","capitalize","caps-lock-indicator","caption","captiontext","caret","cell","center","checkbox","circle","cjk-decimal","cjk-earthly-branch","cjk-heavenly-stem","cjk-ideographic","clear","clip","close-quote","col-resize","collapse","color","color-burn","color-dodge","column","column-reverse","compact","condensed","contain","content","contents","content-box","context-menu","continuous","copy","counter","counters","cover","crop","cross","crosshair","currentcolor","cursive","cyclic","darken","dashed","decimal","decimal-leading-zero","default","default-button","dense","destination-atop","destination-in","destination-out","destination-over","devanagari","difference","disc","discard","disclosure-closed","disclosure-open","document","dot-dash","dot-dot-dash","dotted","double","down","e-resize","ease","ease-in","ease-in-out","ease-out","element","ellipse","ellipsis","embed","end","ethiopic","ethiopic-abegede","ethiopic-abegede-am-et","ethiopic-abegede-gez","ethiopic-abegede-ti-er","ethiopic-abegede-ti-et","ethiopic-halehame-aa-er","ethiopic-halehame-aa-et","ethiopic-halehame-am-et","ethiopic-halehame-gez","ethiopic-halehame-om-et","ethiopic-halehame-sid-et","ethiopic-halehame-so-et","ethiopic-halehame-ti-er","ethiopic-halehame-ti-et","ethiopic-halehame-tig","ethiopic-numeric","ew-resize","exclusion","expanded","extends","extra-condensed","extra-expanded","fantasy","fast","fill","fill-box","fixed","flat","flex","flex-end","flex-start","footnotes","forwards","from","geometricPrecision","georgian","graytext","grid","groove","gujarati","gurmukhi","hand","hangul","hangul-consonant","hard-light","hebrew","help","hidden","hide","higher","highlight","highlighttext","hiragana","hiragana-iroha","horizontal","hsl","hsla","hue","icon","ignore","inactiveborder","inactivecaption","inactivecaptiontext","infinite","infobackground","infotext","inherit","initial","inline","inline-axis","inline-block","inline-flex","inline-grid","inline-table","inset","inside","intrinsic","invert","italic","japanese-formal","japanese-informal","justify","kannada","katakana","katakana-iroha","keep-all","khmer","korean-hangul-formal","korean-hanja-formal","korean-hanja-informal","landscape","lao","large","larger","left","level","lighter","lighten","line-through","linear","linear-gradient","lines","list-item","listbox","listitem","local","logical","loud","lower","lower-alpha","lower-armenian","lower-greek","lower-hexadecimal","lower-latin","lower-norwegian","lower-roman","lowercase","ltr","luminosity","malayalam","manipulation","match","matrix","matrix3d","media-controls-background","media-current-time-display","media-fullscreen-button","media-mute-button","media-play-button","media-return-to-realtime-button","media-rewind-button","media-seek-back-button","media-seek-forward-button","media-slider","media-sliderthumb","media-time-remaining-display","media-volume-slider","media-volume-slider-container","media-volume-sliderthumb","medium","menu","menulist","menulist-button","menulist-text","menulist-textfield","menutext","message-box","middle","min-intrinsic","mix","mongolian","monospace","move","multiple","multiple_mask_images","multiply","myanmar","n-resize","narrower","ne-resize","nesw-resize","no-close-quote","no-drop","no-open-quote","no-repeat","none","normal","not-allowed","nowrap","ns-resize","numbers","numeric","nw-resize","nwse-resize","oblique","octal","opacity","open-quote","optimizeLegibility","optimizeSpeed","oriya","oromo","outset","outside","outside-shape","overlay","overline","padding","padding-box","painted","page","paused","persian","perspective","pinch-zoom","plus-darker","plus-lighter","pointer","polygon","portrait","pre","pre-line","pre-wrap","preserve-3d","progress","push-button","radial-gradient","radio","read-only","read-write","read-write-plaintext-only","rectangle","region","relative","repeat","repeating-linear-gradient","repeating-radial-gradient","repeat-x","repeat-y","reset","reverse","rgb","rgba","ridge","right","rotate","rotate3d","rotateX","rotateY","rotateZ","round","row","row-resize","row-reverse","rtl","run-in","running","s-resize","sans-serif","saturation","scale","scale3d","scaleX","scaleY","scaleZ","screen","scroll","scrollbar","scroll-position","se-resize","searchfield","searchfield-cancel-button","searchfield-decoration","searchfield-results-button","searchfield-results-decoration","self-start","self-end","semi-condensed","semi-expanded","separate","serif","show","sidama","simp-chinese-formal","simp-chinese-informal","single","skew","skewX","skewY","skip-white-space","slide","slider-horizontal","slider-vertical","sliderthumb-horizontal","sliderthumb-vertical","slow","small","small-caps","small-caption","smaller","soft-light","solid","somali","source-atop","source-in","source-out","source-over","space","space-around","space-between","space-evenly","spell-out","square","square-button","start","static","status-bar","stretch","stroke","stroke-box","sub","subpixel-antialiased","svg_masks","super","sw-resize","symbolic","symbols","system-ui","table","table-caption","table-cell","table-column","table-column-group","table-footer-group","table-header-group","table-row","table-row-group","tamil","telugu","text","text-bottom","text-top","textarea","textfield","thai","thick","thin","threeddarkshadow","threedface","threedhighlight","threedlightshadow","threedshadow","tibetan","tigre","tigrinya-er","tigrinya-er-abegede","tigrinya-et","tigrinya-et-abegede","to","top","trad-chinese-formal","trad-chinese-informal","transform","translate","translate3d","translateX","translateY","translateZ","transparent","ultra-condensed","ultra-expanded","underline","unidirectional-pan","unset","up","upper-alpha","upper-armenian","upper-greek","upper-hexadecimal","upper-latin","upper-norwegian","upper-roman","uppercase","urdu","url","var","vertical","vertical-text","view-box","visible","visibleFill","visiblePainted","visibleStroke","visual","w-resize","wait","wave","wider","window","windowframe","windowtext","words","wrap","wrap-reverse","x-large","x-small","xor","xx-large","xx-small"],b=t(y),C=n.concat(i).concat(a).concat(l).concat(c).concat(h).concat(v).concat(y);function A(e,t){for(var n,r=!1;null!=(n=e.next());){if(r&&"/"==n){t.tokenize=null;break}r="*"==n}return["comment","comment"]}e.registerHelper("hintWords","css",C),e.defineMIME("text/css",{documentTypes:r,mediaTypes:o,mediaFeatures:s,mediaValueKeywords:u,propertyKeywords:d,nonStandardPropertyKeywords:f,fontProperties:p,counterDescriptors:g,colorKeywords:m,valueKeywords:b,tokenHooks:{"/":function(e,t){return!!e.eat("*")&&(t.tokenize=A,A(e,t))}},name:"css"}),e.defineMIME("text/x-scss",{mediaTypes:o,mediaFeatures:s,mediaValueKeywords:u,propertyKeywords:d,nonStandardPropertyKeywords:f,colorKeywords:m,valueKeywords:b,fontProperties:p,allowNested:!0,lineComment:"//",tokenHooks:{"/":function(e,t){return e.eat("/")?(e.skipToEnd(),["comment","comment"]):e.eat("*")?(t.tokenize=A,A(e,t)):["operator","operator"]},":":function(e){return!!e.match(/\s*\{/,!1)&&[null,null]},$:function(e){return e.match(/^[\w-]+/),e.match(/^\s*:/,!1)?["variable-2","variable-definition"]:["variable-2","variable"]},"#":function(e){return!!e.eat("{")&&[null,"interpolation"]}},name:"css",helperType:"scss"}),e.defineMIME("text/x-less",{mediaTypes:o,mediaFeatures:s,mediaValueKeywords:u,propertyKeywords:d,nonStandardPropertyKeywords:f,colorKeywords:m,valueKeywords:b,fontProperties:p,allowNested:!0,lineComment:"//",tokenHooks:{"/":function(e,t){return e.eat("/")?(e.skipToEnd(),["comment","comment"]):e.eat("*")?(t.tokenize=A,A(e,t)):["operator","operator"]},"@":function(e){return e.eat("{")?[null,"interpolation"]:!e.match(/^(charset|document|font-face|import|(-(moz|ms|o|webkit)-)?keyframes|media|namespace|page|supports)\b/i,!1)&&(e.eatWhile(/[\w\\\-]/),e.match(/^\s*:/,!1)?["variable-2","variable-definition"]:["variable-2","variable"])},"&":function(){return["atom","atom"]}},name:"css",helperType:"less"}),e.defineMIME("text/x-gss",{documentTypes:r,mediaTypes:o,mediaFeatures:s,propertyKeywords:d,nonStandardPropertyKeywords:f,fontProperties:p,counterDescriptors:g,colorKeywords:m,valueKeywords:b,supportsAtComponent:!0,tokenHooks:{"/":function(e,t){return!!e.eat("*")&&(t.tokenize=A,A(e,t))}},name:"css",helperType:"gss"})}(n(24))},function(e,t,n){!function(e){"use strict";var t={autoSelfClosers:{area:!0,base:!0,br:!0,col:!0,command:!0,embed:!0,frame:!0,hr:!0,img:!0,input:!0,keygen:!0,link:!0,meta:!0,param:!0,source:!0,track:!0,wbr:!0,menuitem:!0},implicitlyClosed:{dd:!0,li:!0,optgroup:!0,option:!0,p:!0,rp:!0,rt:!0,tbody:!0,td:!0,tfoot:!0,th:!0,tr:!0},contextGrabbers:{dd:{dd:!0,dt:!0},dt:{dd:!0,dt:!0},li:{li:!0},option:{option:!0,optgroup:!0},optgroup:{optgroup:!0},p:{address:!0,article:!0,aside:!0,blockquote:!0,dir:!0,div:!0,dl:!0,fieldset:!0,footer:!0,form:!0,h1:!0,h2:!0,h3:!0,h4:!0,h5:!0,h6:!0,header:!0,hgroup:!0,hr:!0,menu:!0,nav:!0,ol:!0,p:!0,pre:!0,section:!0,table:!0,ul:!0},rp:{rp:!0,rt:!0},rt:{rp:!0,rt:!0},tbody:{tbody:!0,tfoot:!0},td:{td:!0,th:!0},tfoot:{tbody:!0},th:{td:!0,th:!0},thead:{tbody:!0,tfoot:!0},tr:{tr:!0}},doNotIndent:{pre:!0},allowUnquoted:!0,allowMissing:!0,caseFold:!0},n={autoSelfClosers:{},implicitlyClosed:{},contextGrabbers:{},doNotIndent:{},allowUnquoted:!1,allowMissing:!1,allowMissingTagName:!1,caseFold:!1};e.defineMode("xml",(function(r,i){var o,a,s=r.indentUnit,l={},u=i.htmlMode?t:n;for(var c in u)l[c]=u[c];for(var c in i)l[c]=i[c];function d(e,t){function n(n){return t.tokenize=n,n(e,t)}var r=e.next();return"<"==r?e.eat("!")?e.eat("[")?e.match("CDATA[")?n(f("atom","]]>")):null:e.match("--")?n(f("comment","--\x3e")):e.match("DOCTYPE",!0,!0)?(e.eatWhile(/[\w\._\-]/),n(function e(t){return function(n,r){for(var i;null!=(i=n.next());){if("<"==i)return r.tokenize=e(t+1),r.tokenize(n,r);if(">"==i){if(1==t){r.tokenize=d;break}return r.tokenize=e(t-1),r.tokenize(n,r)}}return"meta"}}(1))):null:e.eat("?")?(e.eatWhile(/[\w\._\-]/),t.tokenize=f("meta","?>"),"meta"):(o=e.eat("/")?"closeTag":"openTag",t.tokenize=h,"tag bracket"):"&"==r?(e.eat("#")?e.eat("x")?e.eatWhile(/[a-fA-F\d]/)&&e.eat(";"):e.eatWhile(/[\d]/)&&e.eat(";"):e.eatWhile(/[\w\.\-:]/)&&e.eat(";"))?"atom":"error":(e.eatWhile(/[^&<]/),null)}function h(e,t){var n,r,i=e.next();if(">"==i||"/"==i&&e.eat(">"))return t.tokenize=d,o=">"==i?"endTag":"selfcloseTag","tag bracket";if("="==i)return o="equals",null;if("<"==i){t.tokenize=d,t.state=m,t.tagName=t.tagStart=null;var a=t.tokenize(e,t);return a?a+" tag error":"tag error"}return/[\'\"]/.test(i)?(t.tokenize=(n=i,(r=function(e,t){for(;!e.eol();)if(e.next()==n){t.tokenize=h;break}return"string"}).isInAttribute=!0,r),t.stringStartCol=e.column(),t.tokenize(e,t)):(e.match(/^[^\s\u00a0=<>\"\']*[^\s\u00a0=<>\"\'\/]/),"word")}function f(e,t){return function(n,r){for(;!n.eol();){if(n.match(t)){r.tokenize=d;break}n.next()}return e}}function p(e,t,n){this.prev=e.context,this.tagName=t,this.indent=e.indented,this.startOfLine=n,(l.doNotIndent.hasOwnProperty(t)||e.context&&e.context.noIndent)&&(this.noIndent=!0)}function g(e){e.context&&(e.context=e.context.prev)}function v(e,t){for(var n;;){if(!e.context)return;if(n=e.context.tagName,!l.contextGrabbers.hasOwnProperty(n)||!l.contextGrabbers[n].hasOwnProperty(t))return;g(e)}}function m(e,t,n){return"openTag"==e?(n.tagStart=t.column(),y):"closeTag"==e?b:m}function y(e,t,n){return"word"==e?(n.tagName=t.current(),a="tag",_):l.allowMissingTagName&&"endTag"==e?(a="tag bracket",_(e,0,n)):(a="error",y)}function b(e,t,n){if("word"==e){var r=t.current();return n.context&&n.context.tagName!=r&&l.implicitlyClosed.hasOwnProperty(n.context.tagName)&&g(n),n.context&&n.context.tagName==r||!1===l.matchClosing?(a="tag",C):(a="tag error",A)}return l.allowMissingTagName&&"endTag"==e?(a="tag bracket",C(e,0,n)):(a="error",A)}function C(e,t,n){return"endTag"!=e?(a="error",C):(g(n),m)}function A(e,t,n){return a="error",C(e,0,n)}function _(e,t,n){if("word"==e)return a="attribute",I;if("endTag"==e||"selfcloseTag"==e){var r=n.tagName,i=n.tagStart;return n.tagName=n.tagStart=null,"selfcloseTag"==e||l.autoSelfClosers.hasOwnProperty(r)?v(n,r):(v(n,r),n.context=new p(n,r,i==n.indented)),m}return a="error",_}function I(e,t,n){return"equals"==e?x:(l.allowMissing||(a="error"),_(e,0,n))}function x(e,t,n){return"string"==e?w:"word"==e&&l.allowUnquoted?(a="string",_):(a="error",_(e,0,n))}function w(e,t,n){return"string"==e?w:_(e,0,n)}return d.isInText=!0,{startState:function(e){var t={tokenize:d,state:m,indented:e||0,tagName:null,tagStart:null,context:null};return null!=e&&(t.baseIndent=e),t},token:function(e,t){if(!t.tagName&&e.sol()&&(t.indented=e.indentation()),e.eatSpace())return null;o=null;var n=t.tokenize(e,t);return(n||o)&&"comment"!=n&&(a=null,t.state=t.state(o||n,e,t),a&&(n="error"==a?n+" error":a)),n},indent:function(t,n,r){var i=t.context;if(t.tokenize.isInAttribute)return t.tagStart==t.indented?t.stringStartCol+1:t.indented+s;if(i&&i.noIndent)return e.Pass;if(t.tokenize!=h&&t.tokenize!=d)return r?r.match(/^(\s*)/)[0].length:0;if(t.tagName)return!1!==l.multilineTagIndentPastTag?t.tagStart+t.tagName.length+2:t.tagStart+s*(l.multilineTagIndentFactor||1);if(l.alignCDATA&&/<!\[CDATA\[/.test(n))return 0;var o=n&&/^<(\/)?([\w_:\.-]*)/.exec(n);if(o&&o[1])for(;i;){if(i.tagName==o[2]){i=i.prev;break}if(!l.implicitlyClosed.hasOwnProperty(i.tagName))break;i=i.prev}else if(o)for(;i;){var a=l.contextGrabbers[i.tagName];if(!a||!a.hasOwnProperty(o[2]))break;i=i.prev}for(;i&&i.prev&&!i.startOfLine;)i=i.prev;return i?i.indent+s:t.baseIndent||0},electricInput:/<\/[\s\w:]+>$/,blockCommentStart:"\x3c!--",blockCommentEnd:"--\x3e",configuration:l.htmlMode?"html":"xml",helperType:l.htmlMode?"html":"xml",skipAttribute:function(e){e.state==x&&(e.state=_)},xmlCurrentTag:function(e){return e.tagName?{name:e.tagName,close:"closeTag"==e.type}:null},xmlCurrentContext:function(e){for(var t=[],n=e.context;n;n=n.prev)n.tagName&&t.push(n.tagName);return t.reverse()}}})),e.defineMIME("text/xml","xml"),e.defineMIME("application/xml","xml"),e.mimeModes.hasOwnProperty("text/html")||e.defineMIME("text/html",{name:"xml",htmlMode:!0})}(n(24))},function(e,t,n){!function(e){"use strict";e.overlayMode=function(t,n,r){return{startState:function(){return{base:e.startState(t),overlay:e.startState(n),basePos:0,baseCur:null,overlayPos:0,overlayCur:null,streamSeen:null}},copyState:function(r){return{base:e.copyState(t,r.base),overlay:e.copyState(n,r.overlay),basePos:r.basePos,baseCur:null,overlayPos:r.overlayPos,overlayCur:null}},token:function(e,i){return(e!=i.streamSeen||Math.min(i.basePos,i.overlayPos)<e.start)&&(i.streamSeen=e,i.basePos=i.overlayPos=e.start),e.start==i.basePos&&(i.baseCur=t.token(e,i.base),i.basePos=e.pos),e.start==i.overlayPos&&(e.pos=e.start,i.overlayCur=n.token(e,i.overlay),i.overlayPos=e.pos),e.pos=Math.min(i.basePos,i.overlayPos),null==i.overlayCur?i.baseCur:null!=i.baseCur&&i.overlay.combineTokens||r&&null==i.overlay.combineTokens?i.baseCur+" "+i.overlayCur:i.overlayCur},indent:t.indent&&function(e,n,r){return t.indent(e.base,n,r)},electricChars:t.electricChars,innerMode:function(e){return{state:e.base,mode:t}},blankLine:function(e){var i,o;return t.blankLine&&(i=t.blankLine(e.base)),n.blankLine&&(o=n.blankLine(e.overlay)),null==o?i:r&&null!=i?i+" "+o:o}}}}(n(24))},function(e,t,n){!function(e){"use strict";e.multiplexingMode=function(t){var n=Array.prototype.slice.call(arguments,1);function r(e,t,n,r){if("string"==typeof t){var i=e.indexOf(t,n);return r&&i>-1?i+t.length:i}var o=t.exec(n?e.slice(n):e);return o?o.index+n+(r?o[0].length:0):-1}return{startState:function(){return{outer:e.startState(t),innerActive:null,inner:null}},copyState:function(n){return{outer:e.copyState(t,n.outer),innerActive:n.innerActive,inner:n.innerActive&&e.copyState(n.innerActive.mode,n.inner)}},token:function(i,o){if(o.innerActive){var a=o.innerActive;if(u=i.string,!a.close&&i.sol())return o.innerActive=o.inner=null,this.token(i,o);if((d=a.close?r(u,a.close,i.pos,a.parseDelimiters):-1)==i.pos&&!a.parseDelimiters)return i.match(a.close),o.innerActive=o.inner=null,a.delimStyle&&a.delimStyle+" "+a.delimStyle+"-close";d>-1&&(i.string=u.slice(0,d));var s=a.mode.token(i,o.inner);return d>-1&&(i.string=u),d==i.pos&&a.parseDelimiters&&(o.innerActive=o.inner=null),a.innerStyle&&(s=s?s+" "+a.innerStyle:a.innerStyle),s}for(var l=1/0,u=i.string,c=0;c<n.length;++c){var d,h=n[c];if((d=r(u,h.open,i.pos))==i.pos){h.parseDelimiters||i.match(h.open),o.innerActive=h;var f=0;if(t.indent){var p=t.indent(o.outer,"","");p!==e.Pass&&(f=p)}return o.inner=e.startState(h.mode,f),h.delimStyle&&h.delimStyle+" "+h.delimStyle+"-open"}-1!=d&&d<l&&(l=d)}l!=1/0&&(i.string=u.slice(0,l));var g=t.token(i,o.outer);return l!=1/0&&(i.string=u),g},indent:function(n,r,i){var o=n.innerActive?n.innerActive.mode:t;return o.indent?o.indent(n.innerActive?n.inner:n.outer,r,i):e.Pass},blankLine:function(r){var i=r.innerActive?r.innerActive.mode:t;if(i.blankLine&&i.blankLine(r.innerActive?r.inner:r.outer),r.innerActive)"\n"===r.innerActive.close&&(r.innerActive=r.inner=null);else for(var o=0;o<n.length;++o){var a=n[o];"\n"===a.open&&(r.innerActive=a,r.inner=e.startState(a.mode,i.indent?i.indent(r.outer,"",""):0))}},electricChars:t.electricChars,innerMode:function(e){return e.inner?{state:e.inner,mode:e.innerActive.mode}:{state:e.outer,mode:t}}}}}(n(24))},function(e,t,n){!function(e){"use strict";function t(e,t,n,r,i,o){this.indented=e,this.column=t,this.type=n,this.info=r,this.align=i,this.prev=o}function n(e,n,r,i){var o=e.indented;return e.context&&"statement"==e.context.type&&"statement"!=r&&(o=e.context.indented),e.context=new t(o,n,r,i,null,e.context)}function r(e){var t=e.context.type;return")"!=t&&"]"!=t&&"}"!=t||(e.indented=e.context.indented),e.context=e.context.prev}function i(e,t,n){return"variable"==t.prevToken||"type"==t.prevToken||!!/\S(?:[^- ]>|[*\]])\s*$|\*$/.test(e.string.slice(0,n))||!(!t.typeAtEndOfLine||e.column()!=e.indentation())||void 0}function o(e){for(;;){if(!e||"top"==e.type)return!0;if("}"==e.type&&"namespace"!=e.prev.info)return!1;e=e.prev}}function a(e){for(var t={},n=e.split(" "),r=0;r<n.length;++r)t[n[r]]=!0;return t}function s(e,t){return"function"==typeof e?e(t):e.propertyIsEnumerable(t)}e.defineMode("clike",(function(a,l){var u,c,d=a.indentUnit,h=l.statementIndentUnit||d,f=l.dontAlignCalls,p=l.keywords||{},g=l.types||{},v=l.builtin||{},m=l.blockKeywords||{},y=l.defKeywords||{},b=l.atoms||{},C=l.hooks||{},A=l.multiLineStrings,_=!1!==l.indentStatements,I=!1!==l.indentSwitch,x=l.namespaceSeparator,w=l.isPunctuationChar||/[\[\]{}\(\),;\:\.]/,M=l.numberStart||/[\d\.]/,S=l.number||/^(?:0x[a-f\d]+|0b[01]+|(?:\d+\.?\d*|\.\d+)(?:e[-+]?\d+)?)(u|ll?|l|f)?/i,T=l.isOperatorChar||/[+\-*&%=<>!?|\/]/,E=l.isIdentifierChar||/[\w\$_\xa1-\uffff]/,O=l.isReservedIdentifier||!1;function k(e,t){var n,r=e.next();if(C[r]){var i=C[r](e,t);if(!1!==i)return i}if('"'==r||"'"==r)return t.tokenize=(n=r,function(e,t){for(var r,i=!1,o=!1;null!=(r=e.next());){if(r==n&&!i){o=!0;break}i=!i&&"\\"==r}return(o||!i&&!A)&&(t.tokenize=null),"string"}),t.tokenize(e,t);if(w.test(r))return u=r,null;if(M.test(r)){if(e.backUp(1),e.match(S))return"number";e.next()}if("/"==r){if(e.eat("*"))return t.tokenize=L,L(e,t);if(e.eat("/"))return e.skipToEnd(),"comment"}if(T.test(r)){for(;!e.match(/^\/[\/*]/,!1)&&e.eat(T););return"operator"}if(e.eatWhile(E),x)for(;e.match(x);)e.eatWhile(E);var o=e.current();return s(p,o)?(s(m,o)&&(u="newstatement"),s(y,o)&&(c=!0),"keyword"):s(g,o)?"type":s(v,o)||O&&O(o)?(s(m,o)&&(u="newstatement"),"builtin"):s(b,o)?"atom":"variable"}function L(e,t){for(var n,r=!1;n=e.next();){if("/"==n&&r){t.tokenize=null;break}r="*"==n}return"comment"}function D(e,t){l.typeFirstDefinitions&&e.eol()&&o(t.context)&&(t.typeAtEndOfLine=i(e,t,e.pos))}return{startState:function(e){return{tokenize:null,context:new t((e||0)-d,0,"top",null,!1),indented:0,startOfLine:!0,prevToken:null}},token:function(e,t){var a=t.context;if(e.sol()&&(null==a.align&&(a.align=!1),t.indented=e.indentation(),t.startOfLine=!0),e.eatSpace())return D(e,t),null;u=c=null;var s=(t.tokenize||k)(e,t);if("comment"==s||"meta"==s)return s;if(null==a.align&&(a.align=!0),";"==u||":"==u||","==u&&e.match(/^\s*(?:\/\/.*)?$/,!1))for(;"statement"==t.context.type;)r(t);else if("{"==u)n(t,e.column(),"}");else if("["==u)n(t,e.column(),"]");else if("("==u)n(t,e.column(),")");else if("}"==u){for(;"statement"==a.type;)a=r(t);for("}"==a.type&&(a=r(t));"statement"==a.type;)a=r(t)}else u==a.type?r(t):_&&(("}"==a.type||"top"==a.type)&&";"!=u||"statement"==a.type&&"newstatement"==u)&&n(t,e.column(),"statement",e.current());if("variable"==s&&("def"==t.prevToken||l.typeFirstDefinitions&&i(e,t,e.start)&&o(t.context)&&e.match(/^\s*\(/,!1))&&(s="def"),C.token){var d=C.token(e,t,s);void 0!==d&&(s=d)}return"def"==s&&!1===l.styleDefs&&(s="variable"),t.startOfLine=!1,t.prevToken=c?"def":s||u,D(e,t),s},indent:function(t,n){if(t.tokenize!=k&&null!=t.tokenize||t.typeAtEndOfLine)return e.Pass;var r=t.context,i=n&&n.charAt(0),o=i==r.type;if("statement"==r.type&&"}"==i&&(r=r.prev),l.dontIndentStatements)for(;"statement"==r.type&&l.dontIndentStatements.test(r.info);)r=r.prev;if(C.indent){var a=C.indent(t,r,n,d);if("number"==typeof a)return a}var s=r.prev&&"switch"==r.prev.info;if(l.allmanIndentation&&/[{(]/.test(i)){for(;"top"!=r.type&&"}"!=r.type;)r=r.prev;return r.indented}return"statement"==r.type?r.indented+("{"==i?0:h):!r.align||f&&")"==r.type?")"!=r.type||o?r.indented+(o?0:d)+(o||!s||/^(?:case|default)\b/.test(n)?0:d):r.indented+h:r.column+(o?0:1)},electricInput:I?/^\s*(?:case .*?:|default:|\{\}?|\})$/:/^\s*[{}]$/,blockCommentStart:"/*",blockCommentEnd:"*/",blockCommentContinue:" * ",lineComment:"//",fold:"brace"}}));var l="auto if break case register continue return default do sizeof static else struct switch extern typedef union for goto while enum const volatile inline restrict asm fortran",u="alignas alignof and and_eq audit axiom bitand bitor catch class compl concept constexpr const_cast decltype delete dynamic_cast explicit export final friend import module mutable namespace new noexcept not not_eq operator or or_eq override private protected public reinterpret_cast requires static_assert static_cast template this thread_local throw try typeid typename using virtual xor xor_eq",c="bycopy byref in inout oneway out self super atomic nonatomic retain copy readwrite readonly strong weak assign typeof nullable nonnull null_resettable _cmd @interface @implementation @end @protocol @encode @property @synthesize @dynamic @class @public @package @private @protected @required @optional @try @catch @finally @import @selector @encode @defs @synchronized @autoreleasepool @compatibility_alias @available",d="FOUNDATION_EXPORT FOUNDATION_EXTERN NS_INLINE NS_FORMAT_FUNCTION  NS_RETURNS_RETAINEDNS_ERROR_ENUM NS_RETURNS_NOT_RETAINED NS_RETURNS_INNER_POINTER NS_DESIGNATED_INITIALIZER NS_ENUM NS_OPTIONS NS_REQUIRES_NIL_TERMINATION NS_ASSUME_NONNULL_BEGIN NS_ASSUME_NONNULL_END NS_SWIFT_NAME NS_REFINED_FOR_SWIFT",h=a("int long char short double float unsigned signed void bool"),f=a("SEL instancetype id Class Protocol BOOL");function p(e){return s(h,e)||/.+_t$/.test(e)}function g(e){return p(e)||s(f,e)}var v="case do else for if switch while struct enum union";function m(e,t){if(!t.startOfLine)return!1;for(var n,r=null;n=e.peek();){if("\\"==n&&e.match(/^.$/)){r=m;break}if("/"==n&&e.match(/^\/[\/\*]/,!1))break;e.next()}return t.tokenize=r,"meta"}function y(e,t){return"type"==t.prevToken&&"type"}function b(e){return!(!e||e.length<2||"_"!=e[0]||"_"!=e[1]&&e[1]===e[1].toLowerCase())}function C(e){return e.eatWhile(/[\w\.']/),"number"}function A(e,t){if(e.backUp(1),e.match(/(R|u8R|uR|UR|LR)/)){var n=e.match(/"([^\s\\()]{0,16})\(/);return!!n&&(t.cpp11RawStringDelim=n[1],t.tokenize=x,x(e,t))}return e.match(/(u8|u|U|L)/)?!!e.match(/["']/,!1)&&"string":(e.next(),!1)}function _(e){var t=/(\w+)::~?(\w+)$/.exec(e);return t&&t[1]==t[2]}function I(e,t){for(var n;null!=(n=e.next());)if('"'==n&&!e.eat('"')){t.tokenize=null;break}return"string"}function x(e,t){var n=t.cpp11RawStringDelim.replace(/[^\w\s]/g,"\\$&");return e.match(new RegExp(".*?\\)"+n+'"'))?t.tokenize=null:e.skipToEnd(),"string"}function w(t,n){"string"==typeof t&&(t=[t]);var r=[];function i(e){if(e)for(var t in e)e.hasOwnProperty(t)&&r.push(t)}i(n.keywords),i(n.types),i(n.builtin),i(n.atoms),r.length&&(n.helperType=t[0],e.registerHelper("hintWords",t[0],r));for(var o=0;o<t.length;++o)e.defineMIME(t[o],n)}function M(e,t){for(var n=!1;!e.eol();){if(!n&&e.match('"""')){t.tokenize=null;break}n="\\"==e.next()&&!n}return"string"}function S(e){return function(t,n){for(var r;r=t.next();){if("*"==r&&t.eat("/")){if(1==e){n.tokenize=null;break}return n.tokenize=S(e-1),n.tokenize(t,n)}if("/"==r&&t.eat("*"))return n.tokenize=S(e+1),n.tokenize(t,n)}return"comment"}}w(["text/x-csrc","text/x-c","text/x-chdr"],{name:"clike",keywords:a(l),types:p,blockKeywords:a(v),defKeywords:a("struct enum union"),typeFirstDefinitions:!0,atoms:a("NULL true false"),isReservedIdentifier:b,hooks:{"#":m,"*":y},modeProps:{fold:["brace","include"]}}),w(["text/x-c++src","text/x-c++hdr"],{name:"clike",keywords:a(l+" "+u),types:p,blockKeywords:a(v+" class try catch"),defKeywords:a("struct enum union class namespace"),typeFirstDefinitions:!0,atoms:a("true false NULL nullptr"),dontIndentStatements:/^template$/,isIdentifierChar:/[\w\$_~\xa1-\uffff]/,isReservedIdentifier:b,hooks:{"#":m,"*":y,u:A,U:A,L:A,R:A,0:C,1:C,2:C,3:C,4:C,5:C,6:C,7:C,8:C,9:C,token:function(e,t,n){if("variable"==n&&"("==e.peek()&&(";"==t.prevToken||null==t.prevToken||"}"==t.prevToken)&&_(e.current()))return"def"}},namespaceSeparator:"::",modeProps:{fold:["brace","include"]}}),w("text/x-java",{name:"clike",keywords:a("abstract assert break case catch class const continue default do else enum extends final finally for goto if implements import instanceof interface native new package private protected public return static strictfp super switch synchronized this throw throws transient try volatile while @interface"),types:a("byte short int long float double boolean char void Boolean Byte Character Double Float Integer Long Number Object Short String StringBuffer StringBuilder Void"),blockKeywords:a("catch class do else finally for if switch try while"),defKeywords:a("class interface enum @interface"),typeFirstDefinitions:!0,atoms:a("true false null"),number:/^(?:0x[a-f\d_]+|0b[01_]+|(?:[\d_]+\.?\d*|\.\d+)(?:e[-+]?[\d_]+)?)(u|ll?|l|f)?/i,hooks:{"@":function(e){return!e.match("interface",!1)&&(e.eatWhile(/[\w\$_]/),"meta")}},modeProps:{fold:["brace","import"]}}),w("text/x-csharp",{name:"clike",keywords:a("abstract as async await base break case catch checked class const continue default delegate do else enum event explicit extern finally fixed for foreach goto if implicit in interface internal is lock namespace new operator out override params private protected public readonly ref return sealed sizeof stackalloc static struct switch this throw try typeof unchecked unsafe using virtual void volatile while add alias ascending descending dynamic from get global group into join let orderby partial remove select set value var yield"),types:a("Action Boolean Byte Char DateTime DateTimeOffset Decimal Double Func Guid Int16 Int32 Int64 Object SByte Single String Task TimeSpan UInt16 UInt32 UInt64 bool byte char decimal double short int long object sbyte float string ushort uint ulong"),blockKeywords:a("catch class do else finally for foreach if struct switch try while"),defKeywords:a("class interface namespace struct var"),typeFirstDefinitions:!0,atoms:a("true false null"),hooks:{"@":function(e,t){return e.eat('"')?(t.tokenize=I,I(e,t)):(e.eatWhile(/[\w\$_]/),"meta")}}}),w("text/x-scala",{name:"clike",keywords:a("abstract case catch class def do else extends final finally for forSome if implicit import lazy match new null object override package private protected return sealed super this throw trait try type val var while with yield _ assert assume require print println printf readLine readBoolean readByte readShort readChar readInt readLong readFloat readDouble"),types:a("AnyVal App Application Array BufferedIterator BigDecimal BigInt Char Console Either Enumeration Equiv Error Exception Fractional Function IndexedSeq Int Integral Iterable Iterator List Map Numeric Nil NotNull Option Ordered Ordering PartialFunction PartialOrdering Product Proxy Range Responder Seq Serializable Set Specializable Stream StringBuilder StringContext Symbol Throwable Traversable TraversableOnce Tuple Unit Vector Boolean Byte Character CharSequence Class ClassLoader Cloneable Comparable Compiler Double Exception Float Integer Long Math Number Object Package Pair Process Runtime Runnable SecurityManager Short StackTraceElement StrictMath String StringBuffer System Thread ThreadGroup ThreadLocal Throwable Triple Void"),multiLineStrings:!0,blockKeywords:a("catch class enum do else finally for forSome if match switch try while"),defKeywords:a("class enum def object package trait type val var"),atoms:a("true false null"),indentStatements:!1,indentSwitch:!1,isOperatorChar:/[+\-*&%=<>!?|\/#:@]/,hooks:{"@":function(e){return e.eatWhile(/[\w\$_]/),"meta"},'"':function(e,t){return!!e.match('""')&&(t.tokenize=M,t.tokenize(e,t))},"'":function(e){return e.eatWhile(/[\w\$_\xa1-\uffff]/),"atom"},"=":function(e,n){var r=n.context;return!("}"!=r.type||!r.align||!e.eat(">"))&&(n.context=new t(r.indented,r.column,r.type,r.info,null,r.prev),"operator")},"/":function(e,t){return!!e.eat("*")&&(t.tokenize=S(1),t.tokenize(e,t))}},modeProps:{closeBrackets:{pairs:'()[]{}""',triples:'"'}}}),w("text/x-kotlin",{name:"clike",keywords:a("package as typealias class interface this super val operator var fun for is in This throw return annotation break continue object if else while do try when !in !is as? file import where by get set abstract enum open inner override private public internal protected catch finally out final vararg reified dynamic companion constructor init sealed field property receiver param sparam lateinit data inline noinline tailrec external annotation crossinline const operator infix suspend actual expect setparam"),types:a("Boolean Byte Character CharSequence Class ClassLoader Cloneable Comparable Compiler Double Exception Float Integer Long Math Number Object Package Pair Process Runtime Runnable SecurityManager Short StackTraceElement StrictMath String StringBuffer System Thread ThreadGroup ThreadLocal Throwable Triple Void Annotation Any BooleanArray ByteArray Char CharArray DeprecationLevel DoubleArray Enum FloatArray Function Int IntArray Lazy LazyThreadSafetyMode LongArray Nothing ShortArray Unit"),intendSwitch:!1,indentStatements:!1,multiLineStrings:!0,number:/^(?:0x[a-f\d_]+|0b[01_]+|(?:[\d_]+(\.\d+)?|\.\d+)(?:e[-+]?[\d_]+)?)(u|ll?|l|f)?/i,blockKeywords:a("catch class do else finally for if where try while enum"),defKeywords:a("class val var object interface fun"),atoms:a("true false null this"),hooks:{"@":function(e){return e.eatWhile(/[\w\$_]/),"meta"},"*":function(e,t){return"."==t.prevToken?"variable":"operator"},'"':function(e,t){var n;return t.tokenize=(n=e.match('""'),function(e,t){for(var r,i=!1,o=!1;!e.eol();){if(!n&&!i&&e.match('"')){o=!0;break}if(n&&e.match('"""')){o=!0;break}r=e.next(),!i&&"$"==r&&e.match("{")&&e.skipTo("}"),i=!i&&"\\"==r&&!n}return!o&&n||(t.tokenize=null),"string"}),t.tokenize(e,t)},"/":function(e,t){return!!e.eat("*")&&(t.tokenize=S(1),t.tokenize(e,t))},indent:function(e,t,n,r){var i=n&&n.charAt(0);return"}"!=e.prevToken&&")"!=e.prevToken||""!=n?"operator"==e.prevToken&&"}"!=n&&"}"!=e.context.type||"variable"==e.prevToken&&"."==i||("}"==e.prevToken||")"==e.prevToken)&&"."==i?2*r+t.indented:t.align&&"}"==t.type?t.indented+(e.context.type==(n||"").charAt(0)?0:r):void 0:e.indented}},modeProps:{closeBrackets:{triples:'"'}}}),w(["x-shader/x-vertex","x-shader/x-fragment"],{name:"clike",keywords:a("sampler1D sampler2D sampler3D samplerCube sampler1DShadow sampler2DShadow const attribute uniform varying break continue discard return for while do if else struct in out inout"),types:a("float int bool void vec2 vec3 vec4 ivec2 ivec3 ivec4 bvec2 bvec3 bvec4 mat2 mat3 mat4"),blockKeywords:a("for while do if else struct"),builtin:a("radians degrees sin cos tan asin acos atan pow exp log exp2 sqrt inversesqrt abs sign floor ceil fract mod min max clamp mix step smoothstep length distance dot cross normalize ftransform faceforward reflect refract matrixCompMult lessThan lessThanEqual greaterThan greaterThanEqual equal notEqual any all not texture1D texture1DProj texture1DLod texture1DProjLod texture2D texture2DProj texture2DLod texture2DProjLod texture3D texture3DProj texture3DLod texture3DProjLod textureCube textureCubeLod shadow1D shadow2D shadow1DProj shadow2DProj shadow1DLod shadow2DLod shadow1DProjLod shadow2DProjLod dFdx dFdy fwidth noise1 noise2 noise3 noise4"),atoms:a("true false gl_FragColor gl_SecondaryColor gl_Normal gl_Vertex gl_MultiTexCoord0 gl_MultiTexCoord1 gl_MultiTexCoord2 gl_MultiTexCoord3 gl_MultiTexCoord4 gl_MultiTexCoord5 gl_MultiTexCoord6 gl_MultiTexCoord7 gl_FogCoord gl_PointCoord gl_Position gl_PointSize gl_ClipVertex gl_FrontColor gl_BackColor gl_FrontSecondaryColor gl_BackSecondaryColor gl_TexCoord gl_FogFragCoord gl_FragCoord gl_FrontFacing gl_FragData gl_FragDepth gl_ModelViewMatrix gl_ProjectionMatrix gl_ModelViewProjectionMatrix gl_TextureMatrix gl_NormalMatrix gl_ModelViewMatrixInverse gl_ProjectionMatrixInverse gl_ModelViewProjectionMatrixInverse gl_TexureMatrixTranspose gl_ModelViewMatrixInverseTranspose gl_ProjectionMatrixInverseTranspose gl_ModelViewProjectionMatrixInverseTranspose gl_TextureMatrixInverseTranspose gl_NormalScale gl_DepthRange gl_ClipPlane gl_Point gl_FrontMaterial gl_BackMaterial gl_LightSource gl_LightModel gl_FrontLightModelProduct gl_BackLightModelProduct gl_TextureColor gl_EyePlaneS gl_EyePlaneT gl_EyePlaneR gl_EyePlaneQ gl_FogParameters gl_MaxLights gl_MaxClipPlanes gl_MaxTextureUnits gl_MaxTextureCoords gl_MaxVertexAttribs gl_MaxVertexUniformComponents gl_MaxVaryingFloats gl_MaxVertexTextureImageUnits gl_MaxTextureImageUnits gl_MaxFragmentUniformComponents gl_MaxCombineTextureImageUnits gl_MaxDrawBuffers"),indentSwitch:!1,hooks:{"#":m},modeProps:{fold:["brace","include"]}}),w("text/x-nesc",{name:"clike",keywords:a(l+" as atomic async call command component components configuration event generic implementation includes interface module new norace nx_struct nx_union post provides signal task uses abstract extends"),types:p,blockKeywords:a(v),atoms:a("null true false"),hooks:{"#":m},modeProps:{fold:["brace","include"]}}),w("text/x-objectivec",{name:"clike",keywords:a(l+" "+c),types:g,builtin:a(d),blockKeywords:a(v+" @synthesize @try @catch @finally @autoreleasepool @synchronized"),defKeywords:a("struct enum union @interface @implementation @protocol @class"),dontIndentStatements:/^@.*$/,typeFirstDefinitions:!0,atoms:a("YES NO NULL Nil nil true false nullptr"),isReservedIdentifier:b,hooks:{"#":m,"*":y},modeProps:{fold:["brace","include"]}}),w("text/x-objectivec++",{name:"clike",keywords:a(l+" "+c+" "+u),types:g,builtin:a(d),blockKeywords:a(v+" @synthesize @try @catch @finally @autoreleasepool @synchronized class try catch"),defKeywords:a("struct enum union @interface @implementation @protocol @class class namespace"),dontIndentStatements:/^@.*$|^template$/,typeFirstDefinitions:!0,atoms:a("YES NO NULL Nil nil true false nullptr"),isReservedIdentifier:b,hooks:{"#":m,"*":y,u:A,U:A,L:A,R:A,0:C,1:C,2:C,3:C,4:C,5:C,6:C,7:C,8:C,9:C,token:function(e,t,n){if("variable"==n&&"("==e.peek()&&(";"==t.prevToken||null==t.prevToken||"}"==t.prevToken)&&_(e.current()))return"def"}},namespaceSeparator:"::",modeProps:{fold:["brace","include"]}}),w("text/x-squirrel",{name:"clike",keywords:a("base break clone continue const default delete enum extends function in class foreach local resume return this throw typeof yield constructor instanceof static"),types:p,blockKeywords:a("case catch class else for foreach if switch try while"),defKeywords:a("function local class"),typeFirstDefinitions:!0,atoms:a("true false null"),hooks:{"#":m},modeProps:{fold:["brace","include"]}});var T=null;w("text/x-ceylon",{name:"clike",keywords:a("abstracts alias assembly assert assign break case catch class continue dynamic else exists extends finally for function given if import in interface is let module new nonempty object of out outer package return satisfies super switch then this throw try value void while"),types:function(e){var t=e.charAt(0);return t===t.toUpperCase()&&t!==t.toLowerCase()},blockKeywords:a("case catch class dynamic else finally for function if interface module new object switch try while"),defKeywords:a("class dynamic function interface module object package value"),builtin:a("abstract actual aliased annotation by default deprecated doc final formal late license native optional sealed see serializable shared suppressWarnings tagged throws variable"),isPunctuationChar:/[\[\]{}\(\),;\:\.`]/,isOperatorChar:/[+\-*&%=<>!?|^~:\/]/,numberStart:/[\d#$]/,number:/^(?:#[\da-fA-F_]+|\$[01_]+|[\d_]+[kMGTPmunpf]?|[\d_]+\.[\d_]+(?:[eE][-+]?\d+|[kMGTPmunpf]|)|)/i,multiLineStrings:!0,typeFirstDefinitions:!0,atoms:a("true false null larger smaller equal empty finished"),indentSwitch:!1,styleDefs:!1,hooks:{"@":function(e){return e.eatWhile(/[\w\$_]/),"meta"},'"':function(e,t){return t.tokenize=function e(t){return function(n,r){for(var i,o=!1,a=!1;!n.eol();){if(!o&&n.match('"')&&("single"==t||n.match('""'))){a=!0;break}if(!o&&n.match("``")){T=e(t),a=!0;break}i=n.next(),o="single"==t&&!o&&"\\"==i}return a&&(r.tokenize=null),"string"}}(e.match('""')?"triple":"single"),t.tokenize(e,t)},"`":function(e,t){return!(!T||!e.match("`"))&&(t.tokenize=T,T=null,t.tokenize(e,t))},"'":function(e){return e.eatWhile(/[\w\$_\xa1-\uffff]/),"atom"},token:function(e,t,n){if(("variable"==n||"type"==n)&&"."==t.prevToken)return"variable-2"}},modeProps:{fold:["brace","import"],closeBrackets:{triples:'"'}}})}(n(24))},function(e,t,n){!function(e){"use strict";function t(e){return new RegExp("^(("+e.join(")|(")+"))\\b")}var n,r=t(["and","or","not","is"]),i=["as","assert","break","class","continue","def","del","elif","else","except","finally","for","from","global","if","import","lambda","pass","raise","return","try","while","with","yield","in"],o=["abs","all","any","bin","bool","bytearray","callable","chr","classmethod","compile","complex","delattr","dict","dir","divmod","enumerate","eval","filter","float","format","frozenset","getattr","globals","hasattr","hash","help","hex","id","input","int","isinstance","issubclass","iter","len","list","locals","map","max","memoryview","min","next","object","oct","open","ord","pow","property","range","repr","reversed","round","set","setattr","slice","sorted","staticmethod","str","sum","super","tuple","type","vars","zip","__import__","NotImplemented","Ellipsis","__debug__"];function a(e){return e.scopes[e.scopes.length-1]}e.registerHelper("hintWords","python",i.concat(o)),e.defineMode("python",(function(n,s){for(var l=s.delimiters||s.singleDelimiters||/^[\(\)\[\]\{\}@,:`=;\.\\]/,u=[s.singleOperators,s.doubleOperators,s.doubleDelimiters,s.tripleDelimiters,s.operators||/^([-+*/%\/&|^]=?|[<>=]+|\/\/=?|\*\*=?|!=|[~!@]|\.\.\.)/],c=0;c<u.length;c++)u[c]||u.splice(c--,1);var d=s.hangingIndent||n.indentUnit,h=i,f=o;null!=s.extra_keywords&&(h=h.concat(s.extra_keywords)),null!=s.extra_builtins&&(f=f.concat(s.extra_builtins));var p=!(s.version&&Number(s.version)<3);if(p){var g=s.identifiers||/^[_A-Za-z\u00A1-\uFFFF][_A-Za-z0-9\u00A1-\uFFFF]*/;h=h.concat(["nonlocal","False","True","None","async","await"]),f=f.concat(["ascii","bytes","exec","print"]);var v=new RegExp("^(([rbuf]|(br)|(fr))?('{3}|\"{3}|['\"]))","i")}else g=s.identifiers||/^[_A-Za-z][_A-Za-z0-9]*/,h=h.concat(["exec","print"]),f=f.concat(["apply","basestring","buffer","cmp","coerce","execfile","file","intern","long","raw_input","reduce","reload","unichr","unicode","xrange","False","True","None"]),v=new RegExp("^(([rubf]|(ur)|(br))?('{3}|\"{3}|['\"]))","i");var m=t(h),y=t(f);function b(e,t){var n=e.sol()&&"\\"!=t.lastToken;if(n&&(t.indent=e.indentation()),n&&"py"==a(t).type){var r=a(t).offset;if(e.eatSpace()){var i=e.indentation();return i>r?A(t):i<r&&_(e,t)&&"#"!=e.peek()&&(t.errorToken=!0),null}var o=C(e,t);return r>0&&_(e,t)&&(o+=" error"),o}return C(e,t)}function C(e,t,n){if(e.eatSpace())return null;if(!n&&e.match(/^#.*/))return"comment";if(e.match(/^[0-9\.]/,!1)){var i=!1;if(e.match(/^[\d_]*\.\d+(e[\+\-]?\d+)?/i)&&(i=!0),e.match(/^[\d_]+\.\d*/)&&(i=!0),e.match(/^\.\d+/)&&(i=!0),i)return e.eat(/J/i),"number";var o=!1;if(e.match(/^0x[0-9a-f_]+/i)&&(o=!0),e.match(/^0b[01_]+/i)&&(o=!0),e.match(/^0o[0-7_]+/i)&&(o=!0),e.match(/^[1-9][\d_]*(e[\+\-]?[\d_]+)?/)&&(e.eat(/J/i),o=!0),e.match(/^0(?![\dx])/i)&&(o=!0),o)return e.eat(/L/i),"number"}if(e.match(v))return-1!==e.current().toLowerCase().indexOf("f")?(t.tokenize=function(e,t){for(;"rubf".indexOf(e.charAt(0).toLowerCase())>=0;)e=e.substr(1);var n=1==e.length;function r(e){return function(t,n){var o=C(t,n,!0);return"punctuation"==o&&("{"==t.current()?n.tokenize=r(e+1):"}"==t.current()&&(n.tokenize=e>1?r(e-1):i)),o}}function i(i,o){for(;!i.eol();)if(i.eatWhile(/[^'"\{\}\\]/),i.eat("\\")){if(i.next(),n&&i.eol())return"string"}else{if(i.match(e))return o.tokenize=t,"string";if(i.match("{{"))return"string";if(i.match("{",!1))return o.tokenize=r(0),i.current()?"string":o.tokenize(i,o);if(i.match("}}"))return"string";if(i.match("}"))return"error";i.eat(/['"]/)}if(n){if(s.singleLineStringErrors)return"error";o.tokenize=t}return"string"}return i.isString=!0,i}(e.current(),t.tokenize),t.tokenize(e,t)):(t.tokenize=function(e,t){for(;"rubf".indexOf(e.charAt(0).toLowerCase())>=0;)e=e.substr(1);var n=1==e.length;function r(r,i){for(;!r.eol();)if(r.eatWhile(/[^'"\\]/),r.eat("\\")){if(r.next(),n&&r.eol())return"string"}else{if(r.match(e))return i.tokenize=t,"string";r.eat(/['"]/)}if(n){if(s.singleLineStringErrors)return"error";i.tokenize=t}return"string"}return r.isString=!0,r}(e.current(),t.tokenize),t.tokenize(e,t));for(var a=0;a<u.length;a++)if(e.match(u[a]))return"operator";return e.match(l)?"punctuation":"."==t.lastToken&&e.match(g)?"property":e.match(m)||e.match(r)?"keyword":e.match(y)?"builtin":e.match(/^(self|cls)\b/)?"variable-2":e.match(g)?"def"==t.lastToken||"class"==t.lastToken?"def":"variable":(e.next(),n?null:"error")}function A(e){for(;"py"!=a(e).type;)e.scopes.pop();e.scopes.push({offset:a(e).offset+n.indentUnit,type:"py",align:null})}function _(e,t){for(var n=e.indentation();t.scopes.length>1&&a(t).offset>n;){if("py"!=a(t).type)return!0;t.scopes.pop()}return a(t).offset!=n}function I(e,t){e.sol()&&(t.beginningOfLine=!0);var n=t.tokenize(e,t),r=e.current();if(t.beginningOfLine&&"@"==r)return e.match(g,!1)?"meta":p?"operator":"error";if(/\S/.test(r)&&(t.beginningOfLine=!1),"variable"!=n&&"builtin"!=n||"meta"!=t.lastToken||(n="meta"),"pass"!=r&&"return"!=r||(t.dedent+=1),"lambda"==r&&(t.lambda=!0),":"!=r||t.lambda||"py"!=a(t).type||A(t),1==r.length&&!/string|comment/.test(n)){var i="[({".indexOf(r);if(-1!=i&&function(e,t,n){var r=e.match(/^([\s\[\{\(]|#.*)*$/,!1)?null:e.column()+1;t.scopes.push({offset:t.indent+d,type:n,align:r})}(e,t,"])}".slice(i,i+1)),-1!=(i="])}".indexOf(r))){if(a(t).type!=r)return"error";t.indent=t.scopes.pop().offset-d}}return t.dedent>0&&e.eol()&&"py"==a(t).type&&(t.scopes.length>1&&t.scopes.pop(),t.dedent-=1),n}return{startState:function(e){return{tokenize:b,scopes:[{offset:e||0,type:"py",align:null}],indent:e||0,lastToken:null,lambda:!1,dedent:0}},token:function(e,t){var n=t.errorToken;n&&(t.errorToken=!1);var r=I(e,t);return r&&"comment"!=r&&(t.lastToken="keyword"==r||"punctuation"==r?e.current():r),"punctuation"==r&&(r=null),e.eol()&&t.lambda&&(t.lambda=!1),n?r+" error":r},indent:function(t,n){if(t.tokenize!=b)return t.tokenize.isString?e.Pass:0;var r=a(t),i=r.type==n.charAt(0);return null!=r.align?r.align-(i?1:0):r.offset-(i?d:0)},electricInput:/^\s*[\}\]\)]$/,closeBrackets:{triples:"'\""},lineComment:"#",fold:"indent"}})),e.defineMIME("text/x-python","python"),e.defineMIME("text/x-cython",{name:"python",extra_keywords:(n="by cdef cimport cpdef ctypedef enum except extern gil include nogil property public readonly struct union DEF IF ELIF ELSE",n.split(" "))})}(n(24))},function(e,t,n){!function(e){"use strict";e.defineMode("stex",(function(e,t){function n(e,t){e.cmdState.push(t)}function r(e){return e.cmdState.length>0?e.cmdState[e.cmdState.length-1]:null}function i(e,t,n){return function(){this.name=e,this.bracketNo=0,this.style=t,this.styles=n,this.argument=null,this.styleIdentifier=function(){return this.styles[this.bracketNo-1]||null},this.openBracket=function(){return this.bracketNo++,"bracket"},this.closeBracket=function(){}}}var o={};function a(e,t){e.f=t}function s(e,t){var i;if(e.match(/^\\[a-zA-Z@]+/)){var s=e.current().slice(1);return n(t,i=new(i=o.hasOwnProperty(s)?o[s]:o.DEFAULT)),a(t,u),i.style}if(e.match(/^\\[$&%#{}_]/))return"tag";if(e.match(/^\\[,;!\/\\]/))return"tag";if(e.match("\\["))return a(t,(function(e,t){return l(e,t,"\\]")})),"keyword";if(e.match("\\("))return a(t,(function(e,t){return l(e,t,"\\)")})),"keyword";if(e.match("$$"))return a(t,(function(e,t){return l(e,t,"$$")})),"keyword";if(e.match("$"))return a(t,(function(e,t){return l(e,t,"$")})),"keyword";var c=e.next();return"%"==c?(e.skipToEnd(),"comment"):"}"==c||"]"==c?(i=r(t))?(i.closeBracket(c),a(t,u),"bracket"):"error":"{"==c||"["==c?(n(t,i=new(i=o.DEFAULT)),"bracket"):/\d/.test(c)?(e.eatWhile(/[\w.%]/),"atom"):(e.eatWhile(/[\w\-_]/),"begin"==(i=function(e){for(var t=e.cmdState,n=t.length-1;n>=0;n--){var r=t[n];if("DEFAULT"!=r.name)return r}return{styleIdentifier:function(){return null}}}(t)).name&&(i.argument=e.current()),i.styleIdentifier())}function l(e,t,n){if(e.eatSpace())return null;if(n&&e.match(n))return a(t,s),"keyword";if(e.match(/^\\[a-zA-Z@]+/))return"tag";if(e.match(/^[a-zA-Z]+/))return"variable-2";if(e.match(/^\\[$&%#{}_]/))return"tag";if(e.match(/^\\[,;!\/]/))return"tag";if(e.match(/^[\^_&]/))return"tag";if(e.match(/^[+\-<>|=,\/@!*:;'"`~#?]/))return null;if(e.match(/^(\d+\.\d*|\d*\.\d+|\d+)/))return"number";var r=e.next();return"{"==r||"}"==r||"["==r||"]"==r||"("==r||")"==r?"bracket":"%"==r?(e.skipToEnd(),"comment"):"error"}function u(e,t){var n=e.peek();return"{"==n||"["==n?(r(t).openBracket(n),e.eat(n),a(t,s),"bracket"):/[ \t\r]/.test(n)?(e.eat(n),null):(a(t,s),function(e){var t=e.cmdState.pop();t&&t.closeBracket()}(t),s(e,t))}return o.importmodule=i("importmodule","tag",["string","builtin"]),o.documentclass=i("documentclass","tag",["","atom"]),o.usepackage=i("usepackage","tag",["atom"]),o.begin=i("begin","tag",["atom"]),o.end=i("end","tag",["atom"]),o.label=i("label","tag",["atom"]),o.ref=i("ref","tag",["atom"]),o.eqref=i("eqref","tag",["atom"]),o.cite=i("cite","tag",["atom"]),o.bibitem=i("bibitem","tag",["atom"]),o.Bibitem=i("Bibitem","tag",["atom"]),o.RBibitem=i("RBibitem","tag",["atom"]),o.DEFAULT=function(){this.name="DEFAULT",this.style="tag",this.styleIdentifier=this.openBracket=this.closeBracket=function(){}},{startState:function(){return{cmdState:[],f:t.inMathMode?function(e,t){return l(e,t)}:s}},copyState:function(e){return{cmdState:e.cmdState.slice(),f:e.f}},token:function(e,t){return t.f(e,t)},blankLine:function(e){e.f=s,e.cmdState.length=0},lineComment:"%"}})),e.defineMIME("text/x-stex","stex"),e.defineMIME("text/x-latex","stex")}(n(24))},function(e,t,n){!function(e){"use strict";var t=/^((?:(?:aaas?|about|acap|adiumxtra|af[ps]|aim|apt|attachment|aw|beshare|bitcoin|bolo|callto|cap|chrome(?:-extension)?|cid|coap|com-eventbrite-attendee|content|crid|cvs|data|dav|dict|dlna-(?:playcontainer|playsingle)|dns|doi|dtn|dvb|ed2k|facetime|feed|file|finger|fish|ftp|geo|gg|git|gizmoproject|go|gopher|gtalk|h323|hcp|https?|iax|icap|icon|im|imap|info|ipn|ipp|irc[6s]?|iris(?:\.beep|\.lwz|\.xpc|\.xpcs)?|itms|jar|javascript|jms|keyparc|lastfm|ldaps?|magnet|mailto|maps|market|message|mid|mms|ms-help|msnim|msrps?|mtqp|mumble|mupdate|mvn|news|nfs|nih?|nntp|notes|oid|opaquelocktoken|palm|paparazzi|platform|pop|pres|proxy|psyc|query|res(?:ource)?|rmi|rsync|rtmp|rtsp|secondlife|service|session|sftp|sgn|shttp|sieve|sips?|skype|sm[bs]|snmp|soap\.beeps?|soldat|spotify|ssh|steam|svn|tag|teamspeak|tel(?:net)?|tftp|things|thismessage|tip|tn3270|tv|udp|unreal|urn|ut2004|vemmi|ventrilo|view-source|webcal|wss?|wtai|wyciwyg|xcon(?:-userid)?|xfire|xmlrpc\.beeps?|xmpp|xri|ymsgr|z39\.50[rs]?):(?:\/{1,3}|[a-z0-9%])|www\d{0,3}[.]|[a-z0-9.\-]+[.][a-z]{2,4}\/)(?:[^\s()<>]|\([^\s()<>]*\))+(?:\([^\s()<>]*\)|[^\s`*!()\[\]{};:'".,<>?«»“”‘’]))/i;e.defineMode("gfm",(function(n,r){var i=0,o={startState:function(){return{code:!1,codeBlock:!1,ateSpace:!1}},copyState:function(e){return{code:e.code,codeBlock:e.codeBlock,ateSpace:e.ateSpace}},token:function(e,n){if(n.combineTokens=null,n.codeBlock)return e.match(/^```+/)?(n.codeBlock=!1,null):(e.skipToEnd(),null);if(e.sol()&&(n.code=!1),e.sol()&&e.match(/^```+/))return e.skipToEnd(),n.codeBlock=!0,null;if("`"===e.peek()){e.next();var o=e.pos;e.eatWhile("`");var a=1+e.pos-o;return n.code?a===i&&(n.code=!1):(i=a,n.code=!0),null}if(n.code)return e.next(),null;if(e.eatSpace())return n.ateSpace=!0,null;if((e.sol()||n.ateSpace)&&(n.ateSpace=!1,!1!==r.gitHubSpice)){if(e.match(/^(?:[a-zA-Z0-9\-_]+\/)?(?:[a-zA-Z0-9\-_]+@)?(?=.{0,6}\d)(?:[a-f0-9]{7,40}\b)/))return n.combineTokens=!0,"link";if(e.match(/^(?:[a-zA-Z0-9\-_]+\/)?(?:[a-zA-Z0-9\-_]+)?#[0-9]+\b/))return n.combineTokens=!0,"link"}return e.match(t)&&"]("!=e.string.slice(e.start-2,e.start)&&(0==e.start||/\W/.test(e.string.charAt(e.start-1)))?(n.combineTokens=!0,"link"):(e.next(),null)},blankLine:function(e){return e.code=!1,null}},a={taskLists:!0,strikethrough:!0,emoji:!0};for(var s in r)a[s]=r[s];return a.name="markdown",e.overlayMode(e.getMode(n,a),o)}),"markdown"),e.defineMIME("text/x-gfm","gfm")}(n(24),n(655),n(1318))},function(e,t,n){!function(e){"use strict";e.defineMode("julia",(function(t,n){function r(e,t){return void 0===t&&(t="\\b"),new RegExp("^(("+e.join(")|(")+"))"+t)}var i=n.operators||r(["[<>]:","[<>=]=","<<=?",">>>?=?","=>","->","\\/\\/","[\\\\%*+\\-<>!=\\/^|&\\u00F7\\u22BB]=?","\\?","\\$","~",":","\\u00D7","\\u2208","\\u2209","\\u220B","\\u220C","\\u2218","\\u221A","\\u221B","\\u2229","\\u222A","\\u2260","\\u2264","\\u2265","\\u2286","\\u2288","\\u228A","\\u22C5","\\b(in|isa)\\b(?!.?\\()"],""),o=n.delimiters||/^[;,()[\]{}]/,a=n.identifiers||/^[_A-Za-z\u00A1-\u2217\u2219-\uFFFF][\w\u00A1-\u2217\u2219-\uFFFF]*!*/,s=r(["\\\\[0-7]{1,3}","\\\\x[A-Fa-f0-9]{1,2}","\\\\[abefnrtv0%?'\"\\\\]","([^\\u0027\\u005C\\uD800-\\uDFFF]|[\\uD800-\\uDFFF][\\uDC00-\\uDFFF])"],"'"),l=["if","else","elseif","while","for","begin","let","end","do","try","catch","finally","return","break","continue","global","local","const","export","import","importall","using","function","where","macro","module","baremodule","struct","type","mutable","immutable","quote","typealias","abstract","primitive","bitstype"],u=["true","false","nothing","NaN","Inf"];e.registerHelper("hintWords","julia",l.concat(u));var c=r(["begin","function","type","struct","immutable","let","macro","for","while","quote","if","else","elseif","try","finally","catch","do"]),d=r(["end","else","elseif","catch","finally"]),h=r(l),f=r(u),p=/^@[_A-Za-z][\w]*/,g=/^:[_A-Za-z\u00A1-\uFFFF][\w\u00A1-\uFFFF]*!*/,v=/^(`|([_A-Za-z\u00A1-\uFFFF]*"("")?))/;function m(e){return e.nestedArrays>0}function y(e,t){return void 0===t&&(t=0),e.scopes.length<=t?null:e.scopes[e.scopes.length-(t+1)]}function b(e,t){if(e.match(/^#=/,!1))return t.tokenize=_,t.tokenize(e,t);var n=t.leavingExpr;if(e.sol()&&(n=!1),t.leavingExpr=!1,n&&e.match(/^'+/))return"operator";if(e.match(/\.{4,}/))return"error";if(e.match(/\.{1,3}/))return"operator";if(e.eatSpace())return null;var r,s,l=e.peek();if("#"===l)return e.skipToEnd(),"comment";if("["===l&&(t.scopes.push("["),t.nestedArrays++),"("===l&&(t.scopes.push("("),t.nestedGenerators++),m(t)&&"]"===l){for(;t.scopes.length&&"["!==y(t);)t.scopes.pop();t.scopes.pop(),t.nestedArrays--,t.leavingExpr=!0}if(function(e){return e.nestedGenerators>0}(t)&&")"===l){for(;t.scopes.length&&"("!==y(t);)t.scopes.pop();t.scopes.pop(),t.nestedGenerators--,t.leavingExpr=!0}if(m(t)){if("end"==t.lastToken&&e.match(/^:/))return"operator";if(e.match(/^end/))return"number"}if((r=e.match(c,!1))&&t.scopes.push(r[0]),e.match(d,!1)&&t.scopes.pop(),e.match(/^::(?![:\$])/))return t.tokenize=A,t.tokenize(e,t);if(!n&&e.match(g)||e.match(/:([<>]:|<<=?|>>>?=?|->|\/\/|\.{2,3}|[\.\\%*+\-<>!\/^|&]=?|[~\?\$])/))return"builtin";if(e.match(i))return"operator";if(e.match(/^\.?\d/,!1)){var u=RegExp(/^im\b/),x=!1;if(e.match(/^0x\.[0-9a-f_]+p[\+\-]?[_\d]+/i)&&(x=!0),e.match(/^0x[0-9a-f_]+/i)&&(x=!0),e.match(/^0b[01_]+/i)&&(x=!0),e.match(/^0o[0-7_]+/i)&&(x=!0),e.match(/^(?:(?:\d[_\d]*)?\.(?!\.)(?:\d[_\d]*)?|\d[_\d]*\.(?!\.)(?:\d[_\d]*))?([Eef][\+\-]?[_\d]+)?/i)&&(x=!0),e.match(/^\d[_\d]*(e[\+\-]?\d+)?/i)&&(x=!0),x)return e.match(u),t.leavingExpr=!0,"number"}if(e.match(/^'/))return t.tokenize=I,t.tokenize(e,t);if(e.match(v))return t.tokenize=('"""'===(s=e.current()).substr(-3)?s='"""':'"'===s.substr(-1)&&(s='"'),function(e,t){if(e.eat("\\"))e.next();else{if(e.match(s))return t.tokenize=b,t.leavingExpr=!0,"string";e.eat(/[`"]/)}return e.eatWhile(/[^\\`"]/),"string"}),t.tokenize(e,t);if(e.match(p))return"meta";if(e.match(o))return null;if(e.match(h))return"keyword";if(e.match(f))return"builtin";var w=t.isDefinition||"function"==t.lastToken||"macro"==t.lastToken||"type"==t.lastToken||"struct"==t.lastToken||"immutable"==t.lastToken;return e.match(a)?w?"."===e.peek()?(t.isDefinition=!0,"variable"):(t.isDefinition=!1,"def"):e.match(/^({[^}]*})*\(/,!1)?(t.tokenize=C,t.tokenize(e,t)):(t.leavingExpr=!0,"variable"):(e.next(),"error")}function C(e,t){var n=e.match(/^(\(\s*)/);if(n&&(t.firstParenPos<0&&(t.firstParenPos=t.scopes.length),t.scopes.push("("),t.charsAdvanced+=n[1].length),"("==y(t)&&e.match(/^\)/)&&(t.scopes.pop(),t.charsAdvanced+=1,t.scopes.length<=t.firstParenPos)){var r=e.match(/^(\s*where\s+[^\s=]+)*\s*?=(?!=)/,!1);return e.backUp(t.charsAdvanced),t.firstParenPos=-1,t.charsAdvanced=0,t.tokenize=b,r?"def":"builtin"}if(e.match(/^$/g,!1)){for(e.backUp(t.charsAdvanced);t.scopes.length>t.firstParenPos;)t.scopes.pop();return t.firstParenPos=-1,t.charsAdvanced=0,t.tokenize=b,"builtin"}return t.charsAdvanced+=e.match(/^([^()]*)/)[1].length,t.tokenize(e,t)}function A(e,t){return e.match(/.*?(?=,|;|{|}|\(|\)|=|$|\s)/),e.match(/^{/)?t.nestedParameters++:e.match(/^}/)&&t.nestedParameters>0&&t.nestedParameters--,t.nestedParameters>0?e.match(/.*?(?={|})/)||e.next():0==t.nestedParameters&&(t.tokenize=b),"builtin"}function _(e,t){return e.match(/^#=/)&&t.nestedComments++,e.match(/.*?(?=(#=|=#))/)||e.skipToEnd(),e.match(/^=#/)&&(t.nestedComments--,0==t.nestedComments&&(t.tokenize=b)),"comment"}function I(e,t){var n,r=!1;if(e.match(s))r=!0;else if(n=e.match(/\\u([a-f0-9]{1,4})(?=')/i))((i=parseInt(n[1],16))<=55295||i>=57344)&&(r=!0,e.next());else if(n=e.match(/\\U([A-Fa-f0-9]{5,8})(?=')/)){var i;(i=parseInt(n[1],16))<=1114111&&(r=!0,e.next())}return r?(t.leavingExpr=!0,t.tokenize=b,"string"):(e.match(/^[^']+(?=')/)||e.skipToEnd(),e.match(/^'/)&&(t.tokenize=b),"error")}return{startState:function(){return{tokenize:b,scopes:[],lastToken:null,leavingExpr:!1,isDefinition:!1,nestedArrays:0,nestedComments:0,nestedGenerators:0,nestedParameters:0,charsAdvanced:0,firstParenPos:-1}},token:function(e,t){var n=t.tokenize(e,t),r=e.current();return r&&n&&(t.lastToken=r),n},indent:function(e,n){var r=0;return("]"===n||")"===n||/^end\b/.test(n)||/^else/.test(n)||/^catch\b/.test(n)||/^elseif\b/.test(n)||/^finally/.test(n))&&(r=-1),(e.scopes.length+r)*t.indentUnit},electricInput:/\b(end|else|catch|finally)\b/,blockCommentStart:"#=",blockCommentEnd:"=#",lineComment:"#",closeBrackets:'()[]{}""',fold:"indent"}})),e.defineMIME("text/x-julia","julia")}(n(24))},function(e,t,n){!function(e){"use strict";e.registerHelper("wordChars","r",/[\w.]/),e.defineMode("r",(function(t){function n(e){for(var t={},n=0;n<e.length;++n)t[e[n]]=!0;return t}var r=["NULL","NA","Inf","NaN","NA_integer_","NA_real_","NA_complex_","NA_character_","TRUE","FALSE"],i=["list","quote","bquote","eval","return","call","parse","deparse"],o=["if","else","repeat","while","function","for","in","next","break"];e.registerHelper("hintWords","r",r.concat(i,o));var a,s=n(r),l=n(i),u=n(o),c=n(["if","else","repeat","while","function","for"]),d=/[+\-*\/^<>=!&|~$:]/;function h(e,t){a=null;var n,r=e.next();if("#"==r)return e.skipToEnd(),"comment";if("0"==r&&e.eat("x"))return e.eatWhile(/[\da-f]/i),"number";if("."==r&&e.eat(/\d/))return e.match(/\d*(?:e[+\-]?\d+)?/),"number";if(/\d/.test(r))return e.match(/\d*(?:\.\d+)?(?:e[+\-]\d+)?L?/),"number";if("'"==r||'"'==r)return t.tokenize=(n=r,function(e,t){if(e.eat("\\")){var r=e.next();return"x"==r?e.match(/^[a-f0-9]{2}/i):("u"==r||"U"==r)&&e.eat("{")&&e.skipTo("}")?e.next():"u"==r?e.match(/^[a-f0-9]{4}/i):"U"==r?e.match(/^[a-f0-9]{8}/i):/[0-7]/.test(r)&&e.match(/^[0-7]{1,2}/),"string-2"}for(var i;null!=(i=e.next());){if(i==n){t.tokenize=h;break}if("\\"==i){e.backUp(1);break}}return"string"}),"string";if("`"==r)return e.match(/[^`]+`/),"variable-3";if("."==r&&e.match(/.[.\d]+/))return"keyword";if(/[\w\.]/.test(r)&&"_"!=r){e.eatWhile(/[\w\.]/);var i=e.current();return s.propertyIsEnumerable(i)?"atom":u.propertyIsEnumerable(i)?(c.propertyIsEnumerable(i)&&!e.match(/\s*if(\s+|$)/,!1)&&(a="block"),"keyword"):l.propertyIsEnumerable(i)?"builtin":"variable"}return"%"==r?(e.skipTo("%")&&e.next(),"operator variable-2"):"<"==r&&e.eat("-")||"<"==r&&e.match("<-")||"-"==r&&e.match(/>>?/)?"operator arrow":"="==r&&t.ctx.argList?"arg-is":d.test(r)?"$"==r?"operator dollar":(e.eatWhile(d),"operator"):/[\(\){}\[\];]/.test(r)?(a=r,";"==r?"semi":null):null}function f(e,t,n){e.ctx={type:t,indent:e.indent,flags:0,column:n.column(),prev:e.ctx}}function p(e,t){var n=e.ctx;e.ctx={type:n.type,indent:n.indent,flags:n.flags|t,column:n.column,prev:n.prev}}function g(e){e.indent=e.ctx.indent,e.ctx=e.ctx.prev}return{startState:function(){return{tokenize:h,ctx:{type:"top",indent:-t.indentUnit,flags:2},indent:0,afterIdent:!1}},token:function(e,t){if(e.sol()&&(0==(3&t.ctx.flags)&&(t.ctx.flags|=2),4&t.ctx.flags&&g(t),t.indent=e.indentation()),e.eatSpace())return null;var n=t.tokenize(e,t);return"comment"!=n&&0==(2&t.ctx.flags)&&p(t,1),";"!=a&&"{"!=a&&"}"!=a||"block"!=t.ctx.type||g(t),"{"==a?f(t,"}",e):"("==a?(f(t,")",e),t.afterIdent&&(t.ctx.argList=!0)):"["==a?f(t,"]",e):"block"==a?f(t,"block",e):a==t.ctx.type?g(t):"block"==t.ctx.type&&"comment"!=n&&p(t,4),t.afterIdent="variable"==n||"keyword"==n,n},indent:function(e,n){if(e.tokenize!=h)return 0;var r=n&&n.charAt(0),i=e.ctx,o=r==i.type;return 4&i.flags&&(i=i.prev),"block"==i.type?i.indent+("{"==r?0:t.indentUnit):1&i.flags?i.column+(o?0:1):i.indent+(o?0:t.indentUnit)},lineComment:"#"}})),e.defineMIME("text/x-rsrc","r")}(n(24))},function(e,t,n){!function(e){"use strict";e.defineMode("shell",(function(){var t={};function n(e,n){for(var r=0;r<n.length;r++)t[n[r]]=e}var r=["true","false"],i=["if","then","do","else","elif","while","until","for","in","esac","fi","fin","fil","done","exit","set","unset","export","function"],o=["ab","awk","bash","beep","cat","cc","cd","chown","chmod","chroot","clear","cp","curl","cut","diff","echo","find","gawk","gcc","get","git","grep","hg","kill","killall","ln","ls","make","mkdir","openssl","mv","nc","nl","node","npm","ping","ps","restart","rm","rmdir","sed","service","sh","shopt","shred","source","sort","sleep","ssh","start","stop","su","sudo","svn","tee","telnet","top","touch","vi","vim","wall","wc","wget","who","write","yes","zsh"];function a(e,n){if(e.eatSpace())return null;var r=e.sol(),i=e.next();if("\\"===i)return e.next(),null;if("'"===i||'"'===i||"`"===i)return n.tokens.unshift(s(i,"`"===i?"quote":"string")),c(e,n);if("#"===i)return r&&e.eat("!")?(e.skipToEnd(),"meta"):(e.skipToEnd(),"comment");if("$"===i)return n.tokens.unshift(u),c(e,n);if("+"===i||"="===i)return"operator";if("-"===i)return e.eat("-"),e.eatWhile(/\w/),"attribute";if(/\d/.test(i)&&(e.eatWhile(/\d/),e.eol()||!/\w/.test(e.peek())))return"number";e.eatWhile(/[\w-]/);var o=e.current();return"="===e.peek()&&/\w+/.test(o)?"def":t.hasOwnProperty(o)?t[o]:null}function s(e,t){var n="("==e?")":"{"==e?"}":e;return function(r,i){for(var o,a=!1;null!=(o=r.next());){if(o===n&&!a){i.tokens.shift();break}if("$"===o&&!a&&"'"!==e&&r.peek()!=n){a=!0,r.backUp(1),i.tokens.unshift(u);break}if(!a&&e!==n&&o===e)return i.tokens.unshift(s(e,t)),c(r,i);if(!a&&/['"]/.test(o)&&!/['"]/.test(e)){i.tokens.unshift(l(o,"string")),r.backUp(1);break}a=!a&&"\\"===o}return t}}function l(e,t){return function(n,r){return r.tokens[0]=s(e,t),n.next(),c(n,r)}}e.registerHelper("hintWords","shell",r.concat(i,o)),n("atom",r),n("keyword",i),n("builtin",o);var u=function(e,t){t.tokens.length>1&&e.eat("$");var n=e.next();return/['"({]/.test(n)?(t.tokens[0]=s(n,"("==n?"quote":"{"==n?"def":"string"),c(e,t)):(/\d/.test(n)||e.eatWhile(/\w/),t.tokens.shift(),"def")};function c(e,t){return(t.tokens[0]||a)(e,t)}return{startState:function(){return{tokens:[]}},token:function(e,t){return c(e,t)},closeBrackets:"()[]{}''\"\"``",lineComment:"#",fold:"brace"}})),e.defineMIME("text/x-sh","shell"),e.defineMIME("application/x-sh","shell")}(n(24))},function(e,t,n){!function(e){"use strict";function t(e){for(var t;null!=(t=e.next());)if("`"==t&&!e.eat("`"))return"variable-2";return e.backUp(e.current().length-1),e.eatWhile(/\w/)?"variable-2":null}function n(e){return e.eat("@")&&(e.match(/^session\./),e.match(/^local\./),e.match(/^global\./)),e.eat("'")?(e.match(/^.*'/),"variable-2"):e.eat('"')?(e.match(/^.*"/),"variable-2"):e.eat("`")?(e.match(/^.*`/),"variable-2"):e.match(/^[0-9a-zA-Z$\.\_]+/)?"variable-2":null}function r(e){return e.eat("N")?"atom":e.match(/^[a-zA-Z.#!?]/)?"variable-2":null}e.defineMode("sql",(function(t,n){var r=n.client||{},s=n.atoms||{false:!0,true:!0,null:!0},l=n.builtin||o(a),u=n.keywords||o(i),c=n.operatorChars||/^[*+\-%<>!=&|~^\/]/,d=n.support||{},h=n.hooks||{},f=n.dateSQL||{date:!0,time:!0,timestamp:!0},p=!1!==n.backslashStringEscapes,g=n.brackets||/^[\{}\(\)\[\]]/,v=n.punctuation||/^[;.,:]/;function m(e,t){var n=e.next();if(h[n]){var i=h[n](e,t);if(!1!==i)return i}if(d.hexNumber&&("0"==n&&e.match(/^[xX][0-9a-fA-F]+/)||("x"==n||"X"==n)&&e.match(/^'[0-9a-fA-F]+'/)))return"number";if(d.binaryNumber&&(("b"==n||"B"==n)&&e.match(/^'[01]+'/)||"0"==n&&e.match(/^b[01]+/)))return"number";if(n.charCodeAt(0)>47&&n.charCodeAt(0)<58)return e.match(/^[0-9]*(\.[0-9]+)?([eE][-+]?[0-9]+)?/),d.decimallessFloat&&e.match(/^\.(?!\.)/),"number";if("?"==n&&(e.eatSpace()||e.eol()||e.eat(";")))return"variable-3";if("'"==n||'"'==n&&d.doubleQuote)return t.tokenize=y(n),t.tokenize(e,t);if((d.nCharCast&&("n"==n||"N"==n)||d.charsetCast&&"_"==n&&e.match(/[a-z][a-z0-9]*/i))&&("'"==e.peek()||'"'==e.peek()))return"keyword";if(d.escapeConstant&&("e"==n||"E"==n)&&("'"==e.peek()||'"'==e.peek()&&d.doubleQuote))return t.tokenize=function(e,t){return(t.tokenize=y(e.next(),!0))(e,t)},"keyword";if(d.commentSlashSlash&&"/"==n&&e.eat("/"))return e.skipToEnd(),"comment";if(d.commentHash&&"#"==n||"-"==n&&e.eat("-")&&(!d.commentSpaceRequired||e.eat(" ")))return e.skipToEnd(),"comment";if("/"==n&&e.eat("*"))return t.tokenize=function e(t){return function(n,r){var i=n.match(/^.*?(\/\*|\*\/)/);return i?"/*"==i[1]?r.tokenize=e(t+1):r.tokenize=t>1?e(t-1):m:n.skipToEnd(),"comment"}}(1),t.tokenize(e,t);if("."!=n){if(c.test(n))return e.eatWhile(c),"operator";if(g.test(n))return"bracket";if(v.test(n))return e.eatWhile(v),"punctuation";if("{"==n&&(e.match(/^( )*(d|D|t|T|ts|TS)( )*'[^']*'( )*}/)||e.match(/^( )*(d|D|t|T|ts|TS)( )*"[^"]*"( )*}/)))return"number";e.eatWhile(/^[_\w\d]/);var o=e.current().toLowerCase();return f.hasOwnProperty(o)&&(e.match(/^( )+'[^']*'/)||e.match(/^( )+"[^"]*"/))?"number":s.hasOwnProperty(o)?"atom":l.hasOwnProperty(o)?"builtin":u.hasOwnProperty(o)?"keyword":r.hasOwnProperty(o)?"string-2":null}return d.zerolessFloat&&e.match(/^(?:\d+(?:e[+-]?\d+)?)/i)?"number":e.match(/^\.+/)?null:d.ODBCdotTable&&e.match(/^[\w\d_$#]+/)?"variable-2":void 0}function y(e,t){return function(n,r){for(var i,o=!1;null!=(i=n.next());){if(i==e&&!o){r.tokenize=m;break}o=(p||t)&&!o&&"\\"==i}return"string"}}function b(e,t,n){t.context={prev:t.context,indent:e.indentation(),col:e.column(),type:n}}return{startState:function(){return{tokenize:m,context:null}},token:function(e,t){if(e.sol()&&t.context&&null==t.context.align&&(t.context.align=!1),t.tokenize==m&&e.eatSpace())return null;var n=t.tokenize(e,t);if("comment"==n)return n;t.context&&null==t.context.align&&(t.context.align=!0);var r=e.current();return"("==r?b(e,t,")"):"["==r?b(e,t,"]"):t.context&&t.context.type==r&&function(e){e.indent=e.context.indent,e.context=e.context.prev}(t),n},indent:function(n,r){var i=n.context;if(!i)return e.Pass;var o=r.charAt(0)==i.type;return i.align?i.col+(o?0:1):i.indent+(o?0:t.indentUnit)},blockCommentStart:"/*",blockCommentEnd:"*/",lineComment:d.commentSlashSlash?"//":d.commentHash?"#":"--",closeBrackets:"()[]{}''\"\"``"}}));var i="alter and as asc between by count create delete desc distinct drop from group having in insert into is join like not on or order select set table union update values where limit ";function o(e){for(var t={},n=e.split(" "),r=0;r<n.length;++r)t[n[r]]=!0;return t}var a="bool boolean bit blob enum long longblob longtext medium mediumblob mediumint mediumtext time timestamp tinyblob tinyint tinytext text bigint int int1 int2 int3 int4 int8 integer float float4 float8 double char varbinary varchar varcharacter precision real date datetime year unsigned signed decimal numeric";e.defineMIME("text/x-sql",{name:"sql",keywords:o(i+"begin"),builtin:o(a),atoms:o("false true null unknown"),dateSQL:o("date time timestamp"),support:o("ODBCdotTable doubleQuote binaryNumber hexNumber")}),e.defineMIME("text/x-mssql",{name:"sql",client:o("$partition binary_checksum checksum connectionproperty context_info current_request_id error_line error_message error_number error_procedure error_severity error_state formatmessage get_filestream_transaction_context getansinull host_id host_name isnull isnumeric min_active_rowversion newid newsequentialid rowcount_big xact_state object_id"),keywords:o(i+"begin trigger proc view index for add constraint key primary foreign collate clustered nonclustered declare exec go if use index holdlock nolock nowait paglock readcommitted readcommittedlock readpast readuncommitted repeatableread rowlock serializable snapshot tablock tablockx updlock with"),builtin:o("bigint numeric bit smallint decimal smallmoney int tinyint money float real char varchar text nchar nvarchar ntext binary varbinary image cursor timestamp hierarchyid uniqueidentifier sql_variant xml table "),atoms:o("is not null like and or in left right between inner outer join all any some cross unpivot pivot exists"),operatorChars:/^[*+\-%<>!=^\&|\/]/,brackets:/^[\{}\(\)]/,punctuation:/^[;.,:/]/,backslashStringEscapes:!1,dateSQL:o("date datetimeoffset datetime2 smalldatetime datetime time"),hooks:{"@":n}}),e.defineMIME("text/x-mysql",{name:"sql",client:o("charset clear connect edit ego exit go help nopager notee nowarning pager print prompt quit rehash source status system tee"),keywords:o(i+"accessible action add after algorithm all analyze asensitive at authors auto_increment autocommit avg avg_row_length before binary binlog both btree cache call cascade cascaded case catalog_name chain change changed character check checkpoint checksum class_origin client_statistics close coalesce code collate collation collations column columns comment commit committed completion concurrent condition connection consistent constraint contains continue contributors convert cross current current_date current_time current_timestamp current_user cursor data database databases day_hour day_microsecond day_minute day_second deallocate dec declare default delay_key_write delayed delimiter des_key_file describe deterministic dev_pop dev_samp deviance diagnostics directory disable discard distinctrow div dual dumpfile each elseif enable enclosed end ends engine engines enum errors escape escaped even event events every execute exists exit explain extended fast fetch field fields first flush for force foreign found_rows full fulltext function general get global grant grants group group_concat handler hash help high_priority hosts hour_microsecond hour_minute hour_second if ignore ignore_server_ids import index index_statistics infile inner innodb inout insensitive insert_method install interval invoker isolation iterate key keys kill language last leading leave left level limit linear lines list load local localtime localtimestamp lock logs low_priority master master_heartbeat_period master_ssl_verify_server_cert masters match max max_rows maxvalue message_text middleint migrate min min_rows minute_microsecond minute_second mod mode modifies modify mutex mysql_errno natural next no no_write_to_binlog offline offset one online open optimize option optionally out outer outfile pack_keys parser partition partitions password phase plugin plugins prepare preserve prev primary privileges procedure processlist profile profiles purge query quick range read read_write reads real rebuild recover references regexp relaylog release remove rename reorganize repair repeatable replace require resignal restrict resume return returns revoke right rlike rollback rollup row row_format rtree savepoint schedule schema schema_name schemas second_microsecond security sensitive separator serializable server session share show signal slave slow smallint snapshot soname spatial specific sql sql_big_result sql_buffer_result sql_cache sql_calc_found_rows sql_no_cache sql_small_result sqlexception sqlstate sqlwarning ssl start starting starts status std stddev stddev_pop stddev_samp storage straight_join subclass_origin sum suspend table_name table_statistics tables tablespace temporary terminated to trailing transaction trigger triggers truncate uncommitted undo uninstall unique unlock upgrade usage use use_frm user user_resources user_statistics using utc_date utc_time utc_timestamp value variables varying view views warnings when while with work write xa xor year_month zerofill begin do then else loop repeat"),builtin:o("bool boolean bit blob decimal double float long longblob longtext medium mediumblob mediumint mediumtext time timestamp tinyblob tinyint tinytext text bigint int int1 int2 int3 int4 int8 integer float float4 float8 double char varbinary varchar varcharacter precision date datetime year unsigned signed numeric"),atoms:o("false true null unknown"),operatorChars:/^[*+\-%<>!=&|^]/,dateSQL:o("date time timestamp"),support:o("ODBCdotTable decimallessFloat zerolessFloat binaryNumber hexNumber doubleQuote nCharCast charsetCast commentHash commentSpaceRequired"),hooks:{"@":n,"`":t,"\\":r}}),e.defineMIME("text/x-mariadb",{name:"sql",client:o("charset clear connect edit ego exit go help nopager notee nowarning pager print prompt quit rehash source status system tee"),keywords:o(i+"accessible action add after algorithm all always analyze asensitive at authors auto_increment autocommit avg avg_row_length before binary binlog both btree cache call cascade cascaded case catalog_name chain change changed character check checkpoint checksum class_origin client_statistics close coalesce code collate collation collations column columns comment commit committed completion concurrent condition connection consistent constraint contains continue contributors convert cross current current_date current_time current_timestamp current_user cursor data database databases day_hour day_microsecond day_minute day_second deallocate dec declare default delay_key_write delayed delimiter des_key_file describe deterministic dev_pop dev_samp deviance diagnostics directory disable discard distinctrow div dual dumpfile each elseif enable enclosed end ends engine engines enum errors escape escaped even event events every execute exists exit explain extended fast fetch field fields first flush for force foreign found_rows full fulltext function general generated get global grant grants group groupby_concat handler hard hash help high_priority hosts hour_microsecond hour_minute hour_second if ignore ignore_server_ids import index index_statistics infile inner innodb inout insensitive insert_method install interval invoker isolation iterate key keys kill language last leading leave left level limit linear lines list load local localtime localtimestamp lock logs low_priority master master_heartbeat_period master_ssl_verify_server_cert masters match max max_rows maxvalue message_text middleint migrate min min_rows minute_microsecond minute_second mod mode modifies modify mutex mysql_errno natural next no no_write_to_binlog offline offset one online open optimize option optionally out outer outfile pack_keys parser partition partitions password persistent phase plugin plugins prepare preserve prev primary privileges procedure processlist profile profiles purge query quick range read read_write reads real rebuild recover references regexp relaylog release remove rename reorganize repair repeatable replace require resignal restrict resume return returns revoke right rlike rollback rollup row row_format rtree savepoint schedule schema schema_name schemas second_microsecond security sensitive separator serializable server session share show shutdown signal slave slow smallint snapshot soft soname spatial specific sql sql_big_result sql_buffer_result sql_cache sql_calc_found_rows sql_no_cache sql_small_result sqlexception sqlstate sqlwarning ssl start starting starts status std stddev stddev_pop stddev_samp storage straight_join subclass_origin sum suspend table_name table_statistics tables tablespace temporary terminated to trailing transaction trigger triggers truncate uncommitted undo uninstall unique unlock upgrade usage use use_frm user user_resources user_statistics using utc_date utc_time utc_timestamp value variables varying view views virtual warnings when while with work write xa xor year_month zerofill begin do then else loop repeat"),builtin:o("bool boolean bit blob decimal double float long longblob longtext medium mediumblob mediumint mediumtext time timestamp tinyblob tinyint tinytext text bigint int int1 int2 int3 int4 int8 integer float float4 float8 double char varbinary varchar varcharacter precision date datetime year unsigned signed numeric"),atoms:o("false true null unknown"),operatorChars:/^[*+\-%<>!=&|^]/,dateSQL:o("date time timestamp"),support:o("ODBCdotTable decimallessFloat zerolessFloat binaryNumber hexNumber doubleQuote nCharCast charsetCast commentHash commentSpaceRequired"),hooks:{"@":n,"`":t,"\\":r}}),e.defineMIME("text/x-sqlite",{name:"sql",client:o("auth backup bail binary changes check clone databases dbinfo dump echo eqp exit explain fullschema headers help import imposter indexes iotrace limit lint load log mode nullvalue once open output print prompt quit read restore save scanstats schema separator session shell show stats system tables testcase timeout timer trace vfsinfo vfslist vfsname width"),keywords:o(i+"abort action add after all analyze attach autoincrement before begin cascade case cast check collate column commit conflict constraint cross current_date current_time current_timestamp database default deferrable deferred detach each else end escape except exclusive exists explain fail for foreign full glob if ignore immediate index indexed initially inner instead intersect isnull key left limit match natural no notnull null of offset outer plan pragma primary query raise recursive references regexp reindex release rename replace restrict right rollback row savepoint temp temporary then to transaction trigger unique using vacuum view virtual when with without"),builtin:o("bool boolean bit blob decimal double float long longblob longtext medium mediumblob mediumint mediumtext time timestamp tinyblob tinyint tinytext text clob bigint int int2 int8 integer float double char varchar date datetime year unsigned signed numeric real"),atoms:o("null current_date current_time current_timestamp"),operatorChars:/^[*+\-%<>!=&|/~]/,dateSQL:o("date time timestamp datetime"),support:o("decimallessFloat zerolessFloat"),identifierQuote:'"',hooks:{"@":n,":":n,"?":n,$:n,'"':function(e){for(var t;null!=(t=e.next());)if('"'==t&&!e.eat('"'))return"variable-2";return e.backUp(e.current().length-1),e.eatWhile(/\w/)?"variable-2":null},"`":t}}),e.defineMIME("text/x-cassandra",{name:"sql",client:{},keywords:o("add all allow alter and any apply as asc authorize batch begin by clustering columnfamily compact consistency count create custom delete desc distinct drop each_quorum exists filtering from grant if in index insert into key keyspace keyspaces level limit local_one local_quorum modify nan norecursive nosuperuser not of on one order password permission permissions primary quorum rename revoke schema select set storage superuser table three to token truncate ttl two type unlogged update use user users using values where with writetime"),builtin:o("ascii bigint blob boolean counter decimal double float frozen inet int list map static text timestamp timeuuid tuple uuid varchar varint"),atoms:o("false true infinity NaN"),operatorChars:/^[<>=]/,dateSQL:{},support:o("commentSlashSlash decimallessFloat"),hooks:{}}),e.defineMIME("text/x-plsql",{name:"sql",client:o("appinfo arraysize autocommit autoprint autorecovery autotrace blockterminator break btitle cmdsep colsep compatibility compute concat copycommit copytypecheck define describe echo editfile embedded escape exec execute feedback flagger flush heading headsep instance linesize lno loboffset logsource long longchunksize markup native newpage numformat numwidth pagesize pause pno recsep recsepchar release repfooter repheader serveroutput shiftinout show showmode size spool sqlblanklines sqlcase sqlcode sqlcontinue sqlnumber sqlpluscompatibility sqlprefix sqlprompt sqlterminator suffix tab term termout time timing trimout trimspool ttitle underline verify version wrap"),keywords:o("abort accept access add all alter and any array arraylen as asc assert assign at attributes audit authorization avg base_table begin between binary_integer body boolean by case cast char char_base check close cluster clusters colauth column comment commit compress connect connected constant constraint crash create current currval cursor data_base database date dba deallocate debugoff debugon decimal declare default definition delay delete desc digits dispose distinct do drop else elseif elsif enable end entry escape exception exception_init exchange exclusive exists exit external fast fetch file for force form from function generic goto grant group having identified if immediate in increment index indexes indicator initial initrans insert interface intersect into is key level library like limited local lock log logging long loop master maxextents maxtrans member minextents minus mislabel mode modify multiset new next no noaudit nocompress nologging noparallel not nowait number_base object of off offline on online only open option or order out package parallel partition pctfree pctincrease pctused pls_integer positive positiven pragma primary prior private privileges procedure public raise range raw read rebuild record ref references refresh release rename replace resource restrict return returning returns reverse revoke rollback row rowid rowlabel rownum rows run savepoint schema segment select separate session set share snapshot some space split sql start statement storage subtype successful synonym tabauth table tables tablespace task terminate then to trigger truncate type union unique unlimited unrecoverable unusable update use using validate value values variable view views when whenever where while with work"),builtin:o("abs acos add_months ascii asin atan atan2 average bfile bfilename bigserial bit blob ceil character chartorowid chr clob concat convert cos cosh count dec decode deref dual dump dup_val_on_index empty error exp false float floor found glb greatest hextoraw initcap instr instrb int integer isopen last_day least length lengthb ln lower lpad ltrim lub make_ref max min mlslabel mod months_between natural naturaln nchar nclob new_time next_day nextval nls_charset_decl_len nls_charset_id nls_charset_name nls_initcap nls_lower nls_sort nls_upper nlssort no_data_found notfound null number numeric nvarchar2 nvl others power rawtohex real reftohex round rowcount rowidtochar rowtype rpad rtrim serial sign signtype sin sinh smallint soundex sqlcode sqlerrm sqrt stddev string substr substrb sum sysdate tan tanh to_char text to_date to_label to_multi_byte to_number to_single_byte translate true trunc uid unlogged upper user userenv varchar varchar2 variance varying vsize xml"),operatorChars:/^[*\/+\-%<>!=~]/,dateSQL:o("date time timestamp"),support:o("doubleQuote nCharCast zerolessFloat binaryNumber hexNumber")}),e.defineMIME("text/x-hive",{name:"sql",keywords:o("select alter $elem$ $key$ $value$ add after all analyze and archive as asc before between binary both bucket buckets by cascade case cast change cluster clustered clusterstatus collection column columns comment compute concatenate continue create cross cursor data database databases dbproperties deferred delete delimited desc describe directory disable distinct distribute drop else enable end escaped exclusive exists explain export extended external fetch fields fileformat first format formatted from full function functions grant group having hold_ddltime idxproperties if import in index indexes inpath inputdriver inputformat insert intersect into is items join keys lateral left like limit lines load local location lock locks mapjoin materialized minus msck no_drop nocompress not of offline on option or order out outer outputdriver outputformat overwrite partition partitioned partitions percent plus preserve procedure purge range rcfile read readonly reads rebuild recordreader recordwriter recover reduce regexp rename repair replace restrict revoke right rlike row schema schemas semi sequencefile serde serdeproperties set shared show show_database sort sorted ssl statistics stored streamtable table tables tablesample tblproperties temporary terminated textfile then tmp to touch transform trigger unarchive undo union uniquejoin unlock update use using utc utc_tmestamp view when where while with admin authorization char compact compactions conf cube current current_date current_timestamp day decimal defined dependency directories elem_type exchange file following for grouping hour ignore inner interval jar less logical macro minute month more none noscan over owner partialscan preceding pretty principals protection reload rewrite role roles rollup rows second server sets skewed transactions truncate unbounded unset uri user values window year"),builtin:o("bool boolean long timestamp tinyint smallint bigint int float double date datetime unsigned string array struct map uniontype key_type utctimestamp value_type varchar"),atoms:o("false true null unknown"),operatorChars:/^[*+\-%<>!=]/,dateSQL:o("date timestamp"),support:o("ODBCdotTable doubleQuote binaryNumber hexNumber")}),e.defineMIME("text/x-pgsql",{name:"sql",client:o("source"),keywords:o(i+"a abort abs absent absolute access according action ada add admin after aggregate alias all allocate also alter always analyse analyze and any are array array_agg array_max_cardinality as asc asensitive assert assertion assignment asymmetric at atomic attach attribute attributes authorization avg backward base64 before begin begin_frame begin_partition bernoulli between bigint binary bit bit_length blob blocked bom boolean both breadth by c cache call called cardinality cascade cascaded case cast catalog catalog_name ceil ceiling chain char char_length character character_length character_set_catalog character_set_name character_set_schema characteristics characters check checkpoint class class_origin clob close cluster coalesce cobol collate collation collation_catalog collation_name collation_schema collect column column_name columns command_function command_function_code comment comments commit committed concurrently condition condition_number configuration conflict connect connection connection_name constant constraint constraint_catalog constraint_name constraint_schema constraints constructor contains content continue control conversion convert copy corr corresponding cost count covar_pop covar_samp create cross csv cube cume_dist current current_catalog current_date current_default_transform_group current_path current_role current_row current_schema current_time current_timestamp current_transform_group_for_type current_user cursor cursor_name cycle data database datalink datatype date datetime_interval_code datetime_interval_precision day db deallocate debug dec decimal declare default defaults deferrable deferred defined definer degree delete delimiter delimiters dense_rank depends depth deref derived desc describe descriptor detach detail deterministic diagnostics dictionary disable discard disconnect dispatch distinct dlnewcopy dlpreviouscopy dlurlcomplete dlurlcompleteonly dlurlcompletewrite dlurlpath dlurlpathonly dlurlpathwrite dlurlscheme dlurlserver dlvalue do document domain double drop dump dynamic dynamic_function dynamic_function_code each element else elseif elsif empty enable encoding encrypted end end_frame end_partition endexec enforced enum equals errcode error escape event every except exception exclude excluding exclusive exec execute exists exit exp explain expression extension external extract false family fetch file filter final first first_value flag float floor following for force foreach foreign fortran forward found frame_row free freeze from fs full function functions fusion g general generated get global go goto grant granted greatest group grouping groups handler having header hex hierarchy hint hold hour id identity if ignore ilike immediate immediately immutable implementation implicit import in include including increment indent index indexes indicator info inherit inherits initially inline inner inout input insensitive insert instance instantiable instead int integer integrity intersect intersection interval into invoker is isnull isolation join k key key_member key_type label lag language large last last_value lateral lead leading leakproof least left length level library like like_regex limit link listen ln load local localtime localtimestamp location locator lock locked log logged loop lower m map mapping match matched materialized max max_cardinality maxvalue member merge message message_length message_octet_length message_text method min minute minvalue mod mode modifies module month more move multiset mumps name names namespace national natural nchar nclob nesting new next nfc nfd nfkc nfkd nil no none normalize normalized not nothing notice notify notnull nowait nth_value ntile null nullable nullif nulls number numeric object occurrences_regex octet_length octets of off offset oids old on only open operator option options or order ordering ordinality others out outer output over overlaps overlay overriding owned owner p pad parallel parameter parameter_mode parameter_name parameter_ordinal_position parameter_specific_catalog parameter_specific_name parameter_specific_schema parser partial partition pascal passing passthrough password path percent percent_rank percentile_cont percentile_disc perform period permission pg_context pg_datatype_name pg_exception_context pg_exception_detail pg_exception_hint placing plans pli policy portion position position_regex power precedes preceding precision prepare prepared preserve primary print_strict_params prior privileges procedural procedure procedures program public publication query quote raise range rank read reads real reassign recheck recovery recursive ref references referencing refresh regr_avgx regr_avgy regr_count regr_intercept regr_r2 regr_slope regr_sxx regr_sxy regr_syy reindex relative release rename repeatable replace replica requiring reset respect restart restore restrict result result_oid return returned_cardinality returned_length returned_octet_length returned_sqlstate returning returns reverse revoke right role rollback rollup routine routine_catalog routine_name routine_schema routines row row_count row_number rows rowtype rule savepoint scale schema schema_name schemas scope scope_catalog scope_name scope_schema scroll search second section security select selective self sensitive sequence sequences serializable server server_name session session_user set setof sets share show similar simple size skip slice smallint snapshot some source space specific specific_name specifictype sql sqlcode sqlerror sqlexception sqlstate sqlwarning sqrt stable stacked standalone start state statement static statistics stddev_pop stddev_samp stdin stdout storage strict strip structure style subclass_origin submultiset subscription substring substring_regex succeeds sum symmetric sysid system system_time system_user t table table_name tables tablesample tablespace temp template temporary text then ties time timestamp timezone_hour timezone_minute to token top_level_count trailing transaction transaction_active transactions_committed transactions_rolled_back transform transforms translate translate_regex translation treat trigger trigger_catalog trigger_name trigger_schema trim trim_array true truncate trusted type types uescape unbounded uncommitted under unencrypted union unique unknown unlink unlisten unlogged unnamed unnest until untyped update upper uri usage use_column use_variable user user_defined_type_catalog user_defined_type_code user_defined_type_name user_defined_type_schema using vacuum valid validate validator value value_of values var_pop var_samp varbinary varchar variable_conflict variadic varying verbose version versioning view views volatile warning when whenever where while whitespace width_bucket window with within without work wrapper write xml xmlagg xmlattributes xmlbinary xmlcast xmlcomment xmlconcat xmldeclaration xmldocument xmlelement xmlexists xmlforest xmliterate xmlnamespaces xmlparse xmlpi xmlquery xmlroot xmlschema xmlserialize xmltable xmltext xmlvalidate year yes zone"),builtin:o("bigint int8 bigserial serial8 bit varying varbit boolean bool box bytea character char varchar cidr circle date double precision float8 inet integer int int4 interval json jsonb line lseg macaddr macaddr8 money numeric decimal path pg_lsn point polygon real float4 smallint int2 smallserial serial2 serial serial4 text time without zone with timetz timestamp timestamptz tsquery tsvector txid_snapshot uuid xml"),atoms:o("false true null unknown"),operatorChars:/^[*\/+\-%<>!=&|^\/#@?~]/,backslashStringEscapes:!1,dateSQL:o("date time timestamp"),support:o("ODBCdotTable decimallessFloat zerolessFloat binaryNumber hexNumber nCharCast charsetCast escapeConstant")}),e.defineMIME("text/x-gql",{name:"sql",keywords:o("ancestor and asc by contains desc descendant distinct from group has in is limit offset on order select superset where"),atoms:o("false true"),builtin:o("blob datetime first key __key__ string integer double boolean null"),operatorChars:/^[*+\-%<>!=]/}),e.defineMIME("text/x-gpsql",{name:"sql",client:o("source"),keywords:o("abort absolute access action active add admin after aggregate all also alter always analyse analyze and any array as asc assertion assignment asymmetric at authorization backward before begin between bigint binary bit boolean both by cache called cascade cascaded case cast chain char character characteristics check checkpoint class close cluster coalesce codegen collate column comment commit committed concurrency concurrently configuration connection constraint constraints contains content continue conversion copy cost cpu_rate_limit create createdb createexttable createrole createuser cross csv cube current current_catalog current_date current_role current_schema current_time current_timestamp current_user cursor cycle data database day deallocate dec decimal declare decode default defaults deferrable deferred definer delete delimiter delimiters deny desc dictionary disable discard distinct distributed do document domain double drop dxl each else enable encoding encrypted end enum errors escape every except exchange exclude excluding exclusive execute exists explain extension external extract false family fetch fields filespace fill filter first float following for force foreign format forward freeze from full function global grant granted greatest group group_id grouping handler hash having header hold host hour identity if ignore ilike immediate immutable implicit in including inclusive increment index indexes inherit inherits initially inline inner inout input insensitive insert instead int integer intersect interval into invoker is isnull isolation join key language large last leading least left level like limit list listen load local localtime localtimestamp location lock log login mapping master match maxvalue median merge minute minvalue missing mode modifies modify month move name names national natural nchar new newline next no nocreatedb nocreateexttable nocreaterole nocreateuser noinherit nologin none noovercommit nosuperuser not nothing notify notnull nowait null nullif nulls numeric object of off offset oids old on only operator option options or order ordered others out outer over overcommit overlaps overlay owned owner parser partial partition partitions passing password percent percentile_cont percentile_disc placing plans position preceding precision prepare prepared preserve primary prior privileges procedural procedure protocol queue quote randomly range read readable reads real reassign recheck recursive ref references reindex reject relative release rename repeatable replace replica reset resource restart restrict returning returns revoke right role rollback rollup rootpartition row rows rule savepoint scatter schema scroll search second security segment select sequence serializable session session_user set setof sets share show similar simple smallint some split sql stable standalone start statement statistics stdin stdout storage strict strip subpartition subpartitions substring superuser symmetric sysid system table tablespace temp template temporary text then threshold ties time timestamp to trailing transaction treat trigger trim true truncate trusted type unbounded uncommitted unencrypted union unique unknown unlisten until update user using vacuum valid validation validator value values varchar variadic varying verbose version view volatile web when where whitespace window with within without work writable write xml xmlattributes xmlconcat xmlelement xmlexists xmlforest xmlparse xmlpi xmlroot xmlserialize year yes zone"),builtin:o("bigint int8 bigserial serial8 bit varying varbit boolean bool box bytea character char varchar cidr circle date double precision float float8 inet integer int int4 interval json jsonb line lseg macaddr macaddr8 money numeric decimal path pg_lsn point polygon real float4 smallint int2 smallserial serial2 serial serial4 text time without zone with timetz timestamp timestamptz tsquery tsvector txid_snapshot uuid xml"),atoms:o("false true null unknown"),operatorChars:/^[*+\-%<>!=&|^\/#@?~]/,dateSQL:o("date time timestamp"),support:o("ODBCdotTable decimallessFloat zerolessFloat binaryNumber hexNumber nCharCast charsetCast")}),e.defineMIME("text/x-sparksql",{name:"sql",keywords:o("add after all alter analyze and anti archive array as asc at between bucket buckets by cache cascade case cast change clear cluster clustered codegen collection column columns comment commit compact compactions compute concatenate cost create cross cube current current_date current_timestamp database databases datata dbproperties defined delete delimited deny desc describe dfs directories distinct distribute drop else end escaped except exchange exists explain export extended external false fields fileformat first following for format formatted from full function functions global grant group grouping having if ignore import in index indexes inner inpath inputformat insert intersect interval into is items join keys last lateral lazy left like limit lines list load local location lock locks logical macro map minus msck natural no not null nulls of on optimize option options or order out outer outputformat over overwrite partition partitioned partitions percent preceding principals purge range recordreader recordwriter recover reduce refresh regexp rename repair replace reset restrict revoke right rlike role roles rollback rollup row rows schema schemas select semi separated serde serdeproperties set sets show skewed sort sorted start statistics stored stratify struct table tables tablesample tblproperties temp temporary terminated then to touch transaction transactions transform true truncate unarchive unbounded uncache union unlock unset use using values view when where window with"),builtin:o("tinyint smallint int bigint boolean float double string binary timestamp decimal array map struct uniontype delimited serde sequencefile textfile rcfile inputformat outputformat"),atoms:o("false true null"),operatorChars:/^[*\/+\-%<>!=~&|^]/,dateSQL:o("date time timestamp"),support:o("ODBCdotTable doubleQuote zerolessFloat")}),e.defineMIME("text/x-esper",{name:"sql",client:o("source"),keywords:o("alter and as asc between by count create delete desc distinct drop from group having in insert into is join like not on or order select set table union update values where limit after all and as at asc avedev avg between by case cast coalesce count create current_timestamp day days delete define desc distinct else end escape events every exists false first from full group having hour hours in inner insert instanceof into irstream is istream join last lastweekday left limit like max match_recognize matches median measures metadatasql min minute minutes msec millisecond milliseconds not null offset on or order outer output partition pattern prev prior regexp retain-union retain-intersection right rstream sec second seconds select set some snapshot sql stddev sum then true unidirectional until update variable weekday when where window"),builtin:{},atoms:o("false true null"),operatorChars:/^[*+\-%<>!=&|^\/#@?~]/,dateSQL:o("time"),support:o("decimallessFloat zerolessFloat binaryNumber hexNumber")})}(n(24))},,,,,,,,,,,,function(e,t,n){var r,i,o=this&&this.__awaiter||function(e,t,n,r){return new(n||(n=Promise))((function(i,o){function a(e){try{l(r.next(e))}catch(e){o(e)}}function s(e){try{l(r.throw(e))}catch(e){o(e)}}function l(e){var t;e.done?i(e.value):(t=e.value,t instanceof n?t:new n((function(e){e(t)}))).then(a,s)}l((r=r.apply(e,t||[])).next())}))},a=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};r=[n,t,n(84),n(849),n(1381),n(35),n(262)],void 0===(i=function(e,t,n,r,i,s,l){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),r=a(r),t.base=s,t.controls=l;class u{constructor(e,t,n){this._baseUrl=e,this._wsUrl=t,this._cosappModule=n,this._statusElement=document.getElementById("cosapp_loading_status"),this._kernelModel={name:window.sessionStorage.getItem("adso_kernel_name"),id:window.sessionStorage.getItem("adso_kernel_id")}}getCode(e){return o(this,void 0,void 0,(function*(){const t=this._baseUrl+"cosapp/code",r=n.ServerConnection.makeSettings(),i={module:e},o={body:JSON.stringify(i),method:"POST"};let a;try{a=yield n.ServerConnection.makeRequest(t,o,r)}catch(e){return void console.log("error",e)}return yield a.json()}))}checkKernelAlive(e){return o(this,void 0,void 0,(function*(){const t=`${this._baseUrl}api/kernels/${e}`,r=n.ServerConnection.makeSettings(),i={};let o;try{o=yield n.ServerConnection.makeRequest(t,i,r)}catch(e){return!1}const a=yield o.json();return!("message"in a)||!a.message.includes("Kernel does not exist")}))}connectKernel(){return o(this,void 0,void 0,(function*(){const e=n.ServerConnection.makeSettings({baseUrl:this._baseUrl,wsUrl:this._wsUrl});this._kernelManager=new n.KernelManager({serverSettings:e});let t=!1;this._kernelModel.id&&this._kernelModel.name&&(this._statusElement.innerText="Checking existing kernel",t=yield this.checkKernelAlive(this._kernelModel.id)),t?(this._statusElement.innerText="Connecting to existing kernel",this._kernel=yield this._kernelManager.connectTo({model:this._kernelModel})):(this._statusElement.innerText="Starting new kernel",this._kernel=yield this._kernelManager.startNew(),window.sessionStorage.setItem("adso_kernel_name",this._kernel.model.name),window.sessionStorage.setItem("adso_kernel_id",this._kernel.model.id)),this._kernel.connectionStatusChanged.connect((e,t)=>{const n=document.getElementById("app_kernel_status_on"),r=document.getElementById("app_kernel_status_off");"disconnected"===t?(n.style.display="none",r.style.display=""):"connected"===t&&(n.style.display="",r.style.display="none")}),this._statusElement.innerText="Getting bootstrap code";const r=yield this.getCode(this._cosappModule),{title:a,code:s}=r,l=document.getElementById("app_title");l&&(l.innerText=a.replace(/(\r\n|\n|\r)/gm,"")),this._widgetManager=new i.WidgetManager(this._kernel),this._statusElement.innerText="Executing bootstrap code";this._kernel.requestExecute({code:s}).onIOPub=e=>o(this,void 0,void 0,(function*(){if(console.log("msg",e.content),(n.KernelMessage.isStreamMsg(e)||n.KernelMessage.isExecuteInputMsg(e))&&console.log("result",e.content),n.KernelMessage.isDisplayDataMsg(e)){const t=e.content.data["application/vnd.jupyter.widget-view+json"];if(this._statusElement.innerText="Done, starting interface",void 0!==t&&2===t.version_major){const n=this._widgetManager.get_model(t.model_id);void 0!==n&&n.then(t=>{this._widgetManager.display_model(e,t)})}}else if(n.KernelMessage.isErrorMsg(e)){let t=e.content.ename+"\n";t+=e.content.evalue+"\n",console.log("error",e.content),this._statusElement.style.width="90%",this._statusElement.style.color="red",this._statusElement.innerText=t}}))}))}shutdownKernel(){return o(this,void 0,void 0,(function*(){console.log("do nothing")}))}}!function(){const{BASEURL:e,COSAPP_MODULE:t}=r.default();let n;if(console.log("BASEURL",e,"COSAPP_MODULE",t),window.location.protocol.startsWith("https://")){n="wss://"+e.replace("https://","")}else{n="ws://"+e.replace("http://","")}const i=new u(e,n,t);window.addEventListener("beforeunload",e=>{i.shutdownKernel()}),document.readyState,i.connectKernel()}()}.apply(t,r))||(e.exports=i)},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.ConfigWithDefaults=t.ConfigSection=void 0;const r=n(48),i=n(84);!function(e){e.create=function(e){const t=new o(e);return t.load().then(()=>t)}}(t.ConfigSection||(t.ConfigSection={}));class o{constructor(e){var t;this._url="unknown";const n=this.serverSettings=null!==(t=e.serverSettings)&&void 0!==t?t:i.ServerConnection.makeSettings();this._url=r.URLExt.join(n.baseUrl,"api/config",encodeURIComponent(e.name))}get data(){return this._data}async load(){const e=await i.ServerConnection.makeRequest(this._url,{},this.serverSettings);if(200!==e.status){throw await i.ServerConnection.ResponseError.create(e)}this._data=await e.json()}async update(e){this._data=Object.assign(Object.assign({},this._data),e);const t={method:"PATCH",body:JSON.stringify(e)},n=await i.ServerConnection.makeRequest(this._url,t,this.serverSettings);if(200!==n.status){throw await i.ServerConnection.ResponseError.create(n)}return this._data=await n.json(),this._data}}t.ConfigWithDefaults=class{constructor(e){var t,n;this._className="",this._section=e.section,this._defaults=null!==(t=e.defaults)&&void 0!==t?t:{},this._className=null!==(n=e.className)&&void 0!==n?n:""}get(e){const t=this._classData();return e in t?t[e]:this._defaults[e]}set(e,t){const n={};if(n[e]=t,this._className){const e={};return e[this._className]=n,this._section.update(e)}return this._section.update(n)}_classData(){const e=this._section.data;return this._className&&this._className in e?e[this._className]:e}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.ActivityMonitor=void 0;const r=n(16);t.ActivityMonitor=class{constructor(e){this._timer=-1,this._timeout=-1,this._isDisposed=!1,this._activityStopped=new r.Signal(this),e.signal.connect(this._onSignalFired,this),this._timeout=e.timeout||1e3}get activityStopped(){return this._activityStopped}get timeout(){return this._timeout}set timeout(e){this._timeout=e}get isDisposed(){return this._isDisposed}dispose(){this._isDisposed||(this._isDisposed=!0,r.Signal.clearData(this))}_onSignalFired(e,t){clearTimeout(this._timer),this._sender=e,this._args=t,this._timer=setTimeout(()=>{this._activityStopped.emit({sender:this._sender,args:this._args})},this._timeout)}}},function(e,t,n){(function(e,t){!function(e,n){"use strict";if(!e.setImmediate){var r,i,o,a,s,l=1,u={},c=!1,d=e.document,h=Object.getPrototypeOf&&Object.getPrototypeOf(e);h=h&&h.setTimeout?h:e,"[object process]"==={}.toString.call(e.process)?r=function(e){t.nextTick((function(){p(e)}))}:!function(){if(e.postMessage&&!e.importScripts){var t=!0,n=e.onmessage;return e.onmessage=function(){t=!1},e.postMessage("","*"),e.onmessage=n,t}}()?e.MessageChannel?((o=new MessageChannel).port1.onmessage=function(e){p(e.data)},r=function(e){o.port2.postMessage(e)}):d&&"onreadystatechange"in d.createElement("script")?(i=d.documentElement,r=function(e){var t=d.createElement("script");t.onreadystatechange=function(){p(e),t.onreadystatechange=null,i.removeChild(t),t=null},i.appendChild(t)}):r=function(e){setTimeout(p,0,e)}:(a="setImmediate$"+Math.random()+"$",s=function(t){t.source===e&&"string"==typeof t.data&&0===t.data.indexOf(a)&&p(+t.data.slice(a.length))},e.addEventListener?e.addEventListener("message",s,!1):e.attachEvent("onmessage",s),r=function(t){e.postMessage(a+t,"*")}),h.setImmediate=function(e){"function"!=typeof e&&(e=new Function(""+e));for(var t=new Array(arguments.length-1),n=0;n<t.length;n++)t[n]=arguments[n+1];var i={callback:e,args:t};return u[l]=i,r(l),l++},h.clearImmediate=f}function f(e){delete u[e]}function p(e){if(c)setTimeout(p,0,e);else{var t=u[e];if(t){c=!0;try{!function(e){var t=e.callback,n=e.args;switch(n.length){case 0:t();break;case 1:t(n[0]);break;case 2:t(n[0],n[1]);break;case 3:t(n[0],n[1],n[2]);break;default:t.apply(void 0,n)}}(t)}finally{f(e),c=!1}}}}}("undefined"==typeof self?void 0===e?this:e:self)}).call(this,n(61),n(145))},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0})},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.MarkdownCodeBlocks=void 0,function(e){e.CODE_BLOCK_MARKER="```";const t=[".markdown",".mdown",".mkdn",".md",".mkd",".mdwn",".mdtxt",".mdtext",".text",".txt",".Rmd"];class n{constructor(e){this.startLine=e,this.code="",this.endLine=-1}}e.MarkdownCodeBlock=n,e.isMarkdown=function(e){return t.indexOf(e)>-1},e.findMarkdownCodeBlocks=function(t){if(!t||""===t)return[];const r=t.split("\n"),i=[];let o=null;for(let t=0;t<r.length;t++){const a=r[t],s=0===a.indexOf(e.CODE_BLOCK_MARKER),l=null!=o;if(s||l)if(l)o&&(s?(o.endLine=t-1,i.push(o),o=null):o.code+=a+"\n");else{o=new n(t);const r=a.indexOf(e.CODE_BLOCK_MARKER),s=a.lastIndexOf(e.CODE_BLOCK_MARKER);r!==s&&(o.code=a.substring(r+e.CODE_BLOCK_MARKER.length,s),o.endLine=t,i.push(o),o=null)}}return i}}(t.MarkdownCodeBlocks||(t.MarkdownCodeBlocks={}))},function(module,exports,__webpack_require__){"use strict";(function(process){var __importDefault=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};Object.defineProperty(exports,"__esModule",{value:!0}),exports.PageConfig=void 0;const coreutils_1=__webpack_require__(17),minimist_1=__importDefault(__webpack_require__(1347)),url_1=__webpack_require__(705);var PageConfig;!function(PageConfig){function getOption(name){if(configData)return configData[name]||getBodyData(name);configData=Object.create(null);let found=!1;if("undefined"!=typeof document&&document){const e=document.getElementById("jupyter-config-data");e&&(configData=JSON.parse(e.textContent||""),found=!0)}if(!found&&void 0!==process)try{const cli=minimist_1.default(process.argv.slice(2)),path=__webpack_require__(486);let fullPath="";"jupyter-config-data"in cli?fullPath=path.resolve(cli["jupyter-config-data"]):"JUPYTER_CONFIG_DATA"in process.env&&(fullPath=path.resolve(process.env.JUPYTER_CONFIG_DATA)),fullPath&&(configData=eval("require")(fullPath))}catch(e){console.error(e)}if(coreutils_1.JSONExt.isObject(configData))for(const e in configData)"string"!=typeof configData[e]&&(configData[e]=JSON.stringify(configData[e]));else configData=Object.create(null);return configData[name]||getBodyData(name)}function setOption(e,t){const n=getOption(e);return configData[e]=t,n}function getBaseUrl(){return url_1.URLExt.normalize(getOption("baseUrl")||"/")}function getTreeUrl(){return url_1.URLExt.join(getBaseUrl(),getOption("treeUrl"))}function getShareUrl(){return url_1.URLExt.normalize(getOption("shareUrl")||getBaseUrl())}function getTreeShareUrl(){return url_1.URLExt.normalize(url_1.URLExt.join(getShareUrl(),getOption("treeUrl")))}function getUrl(e){var t,n,r;let i=getOption("baseUrl")||"/";const o=null!==(t=e.mode)&&void 0!==t?t:getOption("mode"),a=null!==(n=e.workspace)&&void 0!==n?n:getOption("workspace"),s="multiple-document"===o?"lab":"doc";i=url_1.URLExt.join(i,s),a!==PageConfig.defaultWorkspace&&(i=url_1.URLExt.join(i,"workspaces",encodeURIComponent(getOption("workspace"))));const l=null!==(r=e.treePath)&&void 0!==r?r:getOption("treePath");return l&&(i=url_1.URLExt.join(i,"tree",url_1.URLExt.encodeParts(l))),i}function getWsUrl(e){let t=getOption("wsUrl");if(!t){if(0!==(e=e?url_1.URLExt.normalize(e):getBaseUrl()).indexOf("http"))return"";t="ws"+e.slice(4)}return url_1.URLExt.normalize(t)}function getNBConvertURL({path:e,format:t,download:n}){const r=url_1.URLExt.encodeParts(e),i=url_1.URLExt.join(getBaseUrl(),"nbconvert",t,r);return n?i+"?download=true":i}function getToken(){return getOption("token")||getBodyData("jupyterApiToken")}function getNotebookVersion(){const e=getOption("notebookVersion");return""===e?[0,0,0]:JSON.parse(e)}PageConfig.getOption=getOption,PageConfig.setOption=setOption,PageConfig.getBaseUrl=getBaseUrl,PageConfig.getTreeUrl=getTreeUrl,PageConfig.getShareUrl=getShareUrl,PageConfig.getTreeShareUrl=getTreeShareUrl,PageConfig.getUrl=getUrl,PageConfig.defaultWorkspace="default",PageConfig.getWsUrl=getWsUrl,PageConfig.getNBConvertURL=getNBConvertURL,PageConfig.getToken=getToken,PageConfig.getNotebookVersion=getNotebookVersion;let configData=null,Extension;function getBodyData(e){if("undefined"==typeof document||!document.body)return"";const t=document.body.dataset[e];return void 0===t?"":decodeURIComponent(t)}!function(e){function t(e){try{const t=getOption(e);if(t)return JSON.parse(t)}catch(t){console.warn(`Unable to parse ${e}.`,t)}return[]}e.deferred=t("deferredExtensions"),e.disabled=t("disabledExtensions"),e.isDeferred=function(t){const n=t.indexOf(":");let r="";return-1!==n&&(r=t.slice(0,n)),e.deferred.some(e=>e===t||r&&e===r)},e.isDisabled=function(t){const n=t.indexOf(":");let r="";return-1!==n&&(r=t.slice(0,n)),e.disabled.some(e=>e===t||r&&e===r)}}(Extension=PageConfig.Extension||(PageConfig.Extension={}))}(PageConfig=exports.PageConfig||(exports.PageConfig={}))}).call(this,__webpack_require__(145))},function(e,t){},function(e,t){function n(e){return"number"==typeof e||(!!/^0x[0-9a-f]+$/i.test(e)||/^[-+]?(?:\d+(?:\.\d*)?|\.\d+)(e[-+]?\d+)?$/.test(e))}e.exports=function(e,t){t||(t={});var r={bools:{},strings:{},unknownFn:null};"function"==typeof t.unknown&&(r.unknownFn=t.unknown),"boolean"==typeof t.boolean&&t.boolean?r.allBools=!0:[].concat(t.boolean).filter(Boolean).forEach((function(e){r.bools[e]=!0}));var i={};Object.keys(t.alias||{}).forEach((function(e){i[e]=[].concat(t.alias[e]),i[e].forEach((function(t){i[t]=[e].concat(i[e].filter((function(e){return t!==e})))}))})),[].concat(t.string).filter(Boolean).forEach((function(e){r.strings[e]=!0,i[e]&&(r.strings[i[e]]=!0)}));var o=t.default||{},a={_:[]};Object.keys(r.bools).forEach((function(e){l(e,void 0!==o[e]&&o[e])}));var s=[];function l(e,t,o){if(!o||!r.unknownFn||function(e,t){return r.allBools&&/^--[^=]+$/.test(t)||r.strings[e]||r.bools[e]||i[e]}(e,o)||!1!==r.unknownFn(o)){var s=!r.strings[e]&&n(t)?Number(t):t;u(a,e.split("."),s),(i[e]||[]).forEach((function(e){u(a,e.split("."),s)}))}}function u(e,t,n){for(var i=e,o=0;o<t.length-1;o++){if("__proto__"===(a=t[o]))return;void 0===i[a]&&(i[a]={}),i[a]!==Object.prototype&&i[a]!==Number.prototype&&i[a]!==String.prototype||(i[a]={}),i[a]===Array.prototype&&(i[a]=[]),i=i[a]}var a;"__proto__"!==(a=t[t.length-1])&&(i!==Object.prototype&&i!==Number.prototype&&i!==String.prototype||(i={}),i===Array.prototype&&(i=[]),void 0===i[a]||r.bools[a]||"boolean"==typeof i[a]?i[a]=n:Array.isArray(i[a])?i[a].push(n):i[a]=[i[a],n])}function c(e){return i[e].some((function(e){return r.bools[e]}))}-1!==e.indexOf("--")&&(s=e.slice(e.indexOf("--")+1),e=e.slice(0,e.indexOf("--")));for(var d=0;d<e.length;d++){var h=e[d];if(/^--.+=/.test(h)){var f=h.match(/^--([^=]+)=([\s\S]*)$/),p=f[1],g=f[2];r.bools[p]&&(g="false"!==g),l(p,g,h)}else if(/^--no-.+/.test(h)){l(p=h.match(/^--no-(.+)/)[1],!1,h)}else if(/^--.+/.test(h)){p=h.match(/^--(.+)/)[1];void 0===(b=e[d+1])||/^-/.test(b)||r.bools[p]||r.allBools||i[p]&&c(p)?/^(true|false)$/.test(b)?(l(p,"true"===b,h),d++):l(p,!r.strings[p]||"",h):(l(p,b,h),d++)}else if(/^-[^-]+/.test(h)){for(var v=h.slice(1,-1).split(""),m=!1,y=0;y<v.length;y++){var b;if("-"!==(b=h.slice(y+2))){if(/[A-Za-z]/.test(v[y])&&/=/.test(b)){l(v[y],b.split("=")[1],h),m=!0;break}if(/[A-Za-z]/.test(v[y])&&/-?\d+(\.\d*)?(e-?\d+)?$/.test(b)){l(v[y],b,h),m=!0;break}if(v[y+1]&&v[y+1].match(/\W/)){l(v[y],h.slice(y+2),h),m=!0;break}l(v[y],!r.strings[v[y]]||"",h)}else l(v[y],b,h)}p=h.slice(-1)[0];m||"-"===p||(!e[d+1]||/^(-|--)[^-]/.test(e[d+1])||r.bools[p]||i[p]&&c(p)?e[d+1]&&/^(true|false)$/.test(e[d+1])?(l(p,"true"===e[d+1],h),d++):l(p,!r.strings[p]||"",h):(l(p,e[d+1],h),d++))}else if(r.unknownFn&&!1===r.unknownFn(h)||a._.push(r.strings._||!n(h)?h:Number(h)),t.stopEarly){a._.push.apply(a._,e.slice(d+1));break}}return Object.keys(o).forEach((function(e){var t,n,r;t=a,n=e.split("."),r=t,n.slice(0,-1).forEach((function(e){r=r[e]||{}})),n[n.length-1]in r||(u(a,e.split("."),o[e]),(i[e]||[]).forEach((function(t){u(a,t.split("."),o[e])})))})),t["--"]?(a["--"]=new Array,s.forEach((function(e){a["--"].push(e)}))):s.forEach((function(e){a._.push(e)})),a}},function(e,t,n){"use strict";(function(t){var r=n(1349),i=n(1350),o=/^[A-Za-z][A-Za-z0-9+-.]*:\/\//,a=/^([a-z][a-z0-9.+-]*:)?(\/\/)?([\S\s]*)/i,s=new RegExp("^[\\x09\\x0A\\x0B\\x0C\\x0D\\x20\\xA0\\u1680\\u180E\\u2000\\u2001\\u2002\\u2003\\u2004\\u2005\\u2006\\u2007\\u2008\\u2009\\u200A\\u202F\\u205F\\u3000\\u2028\\u2029\\uFEFF]+");function l(e){return(e||"").toString().replace(s,"")}var u=[["#","hash"],["?","query"],function(e){return e.replace("\\","/")},["/","pathname"],["@","auth",1],[NaN,"host",void 0,1,1],[/:(\d+)$/,"port",void 0,1],[NaN,"hostname",void 0,1,1]],c={hash:1,query:1};function d(e){var n,r=("undefined"!=typeof window?window:void 0!==t?t:"undefined"!=typeof self?self:{}).location||{},i={},a=typeof(e=e||r);if("blob:"===e.protocol)i=new f(unescape(e.pathname),{});else if("string"===a)for(n in i=new f(e,{}),c)delete i[n];else if("object"===a){for(n in e)n in c||(i[n]=e[n]);void 0===i.slashes&&(i.slashes=o.test(e.href))}return i}function h(e){e=l(e);var t=a.exec(e);return{protocol:t[1]?t[1].toLowerCase():"",slashes:!!t[2],rest:t[3]}}function f(e,t,n){if(e=l(e),!(this instanceof f))return new f(e,t,n);var o,a,s,c,p,g,v=u.slice(),m=typeof t,y=this,b=0;for("object"!==m&&"string"!==m&&(n=t,t=null),n&&"function"!=typeof n&&(n=i.parse),t=d(t),o=!(a=h(e||"")).protocol&&!a.slashes,y.slashes=a.slashes||o&&t.slashes,y.protocol=a.protocol||t.protocol||"",e=a.rest,a.slashes||(v[3]=[/(.*)/,"pathname"]);b<v.length;b++)"function"!=typeof(c=v[b])?(s=c[0],g=c[1],s!=s?y[g]=e:"string"==typeof s?~(p=e.indexOf(s))&&("number"==typeof c[2]?(y[g]=e.slice(0,p),e=e.slice(p+c[2])):(y[g]=e.slice(p),e=e.slice(0,p))):(p=s.exec(e))&&(y[g]=p[1],e=e.slice(0,p.index)),y[g]=y[g]||o&&c[3]&&t[g]||"",c[4]&&(y[g]=y[g].toLowerCase())):e=c(e);n&&(y.query=n(y.query)),o&&t.slashes&&"/"!==y.pathname.charAt(0)&&(""!==y.pathname||""!==t.pathname)&&(y.pathname=function(e,t){if(""===e)return t;for(var n=(t||"/").split("/").slice(0,-1).concat(e.split("/")),r=n.length,i=n[r-1],o=!1,a=0;r--;)"."===n[r]?n.splice(r,1):".."===n[r]?(n.splice(r,1),a++):a&&(0===r&&(o=!0),n.splice(r,1),a--);return o&&n.unshift(""),"."!==i&&".."!==i||n.push(""),n.join("/")}(y.pathname,t.pathname)),r(y.port,y.protocol)||(y.host=y.hostname,y.port=""),y.username=y.password="",y.auth&&(c=y.auth.split(":"),y.username=c[0]||"",y.password=c[1]||""),y.origin=y.protocol&&y.host&&"file:"!==y.protocol?y.protocol+"//"+y.host:"null",y.href=y.toString()}f.prototype={set:function(e,t,n){var o=this;switch(e){case"query":"string"==typeof t&&t.length&&(t=(n||i.parse)(t)),o[e]=t;break;case"port":o[e]=t,r(t,o.protocol)?t&&(o.host=o.hostname+":"+t):(o.host=o.hostname,o[e]="");break;case"hostname":o[e]=t,o.port&&(t+=":"+o.port),o.host=t;break;case"host":o[e]=t,/:\d+$/.test(t)?(t=t.split(":"),o.port=t.pop(),o.hostname=t.join(":")):(o.hostname=t,o.port="");break;case"protocol":o.protocol=t.toLowerCase(),o.slashes=!n;break;case"pathname":case"hash":if(t){var a="pathname"===e?"/":"#";o[e]=t.charAt(0)!==a?a+t:t}else o[e]=t;break;default:o[e]=t}for(var s=0;s<u.length;s++){var l=u[s];l[4]&&(o[l[1]]=o[l[1]].toLowerCase())}return o.origin=o.protocol&&o.host&&"file:"!==o.protocol?o.protocol+"//"+o.host:"null",o.href=o.toString(),o},toString:function(e){e&&"function"==typeof e||(e=i.stringify);var t,n=this,r=n.protocol;r&&":"!==r.charAt(r.length-1)&&(r+=":");var o=r+(n.slashes?"//":"");return n.username&&(o+=n.username,n.password&&(o+=":"+n.password),o+="@"),o+=n.host+n.pathname,(t="object"==typeof n.query?e(n.query):n.query)&&(o+="?"!==t.charAt(0)?"?"+t:t),n.hash&&(o+=n.hash),o}},f.extractProtocol=h,f.location=d,f.trimLeft=l,f.qs=i,e.exports=f}).call(this,n(61))},function(e,t,n){"use strict";e.exports=function(e,t){if(t=t.split(":")[0],!(e=+e))return!1;switch(t){case"http":case"ws":return 80!==e;case"https":case"wss":return 443!==e;case"ftp":return 21!==e;case"gopher":return 70!==e;case"file":return!1}return 0!==e}},function(e,t,n){"use strict";var r=Object.prototype.hasOwnProperty;function i(e){try{return decodeURIComponent(e.replace(/\+/g," "))}catch(e){return null}}function o(e){try{return encodeURIComponent(e)}catch(e){return null}}t.stringify=function(e,t){t=t||"";var n,i,a=[];for(i in"string"!=typeof t&&(t="?"),e)if(r.call(e,i)){if((n=e[i])||null!=n&&!isNaN(n)||(n=""),i=o(i),n=o(n),null===i||null===n)continue;a.push(i+"="+n)}return a.length?t+a.join("&"):""},t.parse=function(e){for(var t,n=/([^=?#&]+)=?([^&]*)/g,r={};t=n.exec(e);){var o=i(t[1]),a=i(t[2]);null===o||null===a||o in r||(r[o]=a)}return r}},function(e,t,n){"use strict";var r=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),i=this&&this.__setModuleDefault||(Object.create?function(e,t){Object.defineProperty(e,"default",{enumerable:!0,value:t})}:function(e,t){e.default=t}),o=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)"default"!==n&&Object.prototype.hasOwnProperty.call(e,n)&&r(t,e,n);return i(t,e),t};Object.defineProperty(t,"__esModule",{value:!0}),t.PathExt=void 0;const a=o(n(486));!function(e){function t(e){return 0===e.indexOf("/")&&(e=e.slice(1)),e}e.join=function(...e){const n=a.join(...e);return"."===n?"":t(n)},e.basename=function(e,t){return a.basename(e,t)},e.dirname=function(e){const n=t(a.dirname(e));return"."===n?"":n},e.extname=function(e){return a.extname(e)},e.normalize=function(e){return""===e?"":t(a.normalize(e))},e.resolve=function(...e){return t(a.resolve(...e))},e.relative=function(e,n){return t(a.relative(e,n))},e.normalizeExtension=function(e){return e.length>0&&0!==e.indexOf(".")&&(e="."+e),e},e.removeSlash=t}(t.PathExt||(t.PathExt={}))},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.Text=void 0,function(e){const t="𝐚".length>1;e.jsIndexToCharIndex=function(e,n){if(t)return e;let r=e;for(let t=0;t+1<n.length&&t<e;t++){const e=n.charCodeAt(t);if(e>=55296&&e<=56319){const e=n.charCodeAt(t+1);e>=56320&&e<=57343&&(r--,t++)}}return r},e.charIndexToJsIndex=function(e,n){if(t)return e;let r=e;for(let e=0;e+1<n.length&&e<r;e++){const t=n.charCodeAt(e);if(t>=55296&&t<=56319){const t=n.charCodeAt(e+1);t>=56320&&t<=57343&&(r++,e++)}}return r},e.camelCase=function(e,t=!1){return e.replace(/^(\w)|[\s-_:]+(\w)/g,(function(e,n,r){return r?r.toUpperCase():t?n.toUpperCase():n.toLowerCase()}))},e.titleCase=function(e){return(e||"").toLowerCase().split(" ").map(e=>e.charAt(0).toUpperCase()+e.slice(1)).join(" ")}}(t.Text||(t.Text={}))},function(e,t,n){"use strict";var r=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};Object.defineProperty(t,"__esModule",{value:!0}),t.Time=void 0;const i=r(n(12));!function(e){e.formatHuman=function(e){i.default.locale(document.documentElement.lang);let t=i.default(e).fromNow();return t="a few seconds ago"===t?"seconds ago":t,t},e.format=function(e,t="YYYY-MM-DD HH:mm"){return i.default(e).format(t)}}(t.Time||(t.Time={}))},function(e,t,n){var r={"./af":706,"./af.js":706,"./ar":707,"./ar-dz":708,"./ar-dz.js":708,"./ar-kw":709,"./ar-kw.js":709,"./ar-ly":710,"./ar-ly.js":710,"./ar-ma":711,"./ar-ma.js":711,"./ar-sa":712,"./ar-sa.js":712,"./ar-tn":713,"./ar-tn.js":713,"./ar.js":707,"./az":714,"./az.js":714,"./be":715,"./be.js":715,"./bg":716,"./bg.js":716,"./bm":717,"./bm.js":717,"./bn":718,"./bn-bd":719,"./bn-bd.js":719,"./bn.js":718,"./bo":720,"./bo.js":720,"./br":721,"./br.js":721,"./bs":722,"./bs.js":722,"./ca":723,"./ca.js":723,"./cs":724,"./cs.js":724,"./cv":725,"./cv.js":725,"./cy":726,"./cy.js":726,"./da":727,"./da.js":727,"./de":728,"./de-at":729,"./de-at.js":729,"./de-ch":730,"./de-ch.js":730,"./de.js":728,"./dv":731,"./dv.js":731,"./el":732,"./el.js":732,"./en-au":733,"./en-au.js":733,"./en-ca":734,"./en-ca.js":734,"./en-gb":735,"./en-gb.js":735,"./en-ie":736,"./en-ie.js":736,"./en-il":737,"./en-il.js":737,"./en-in":738,"./en-in.js":738,"./en-nz":739,"./en-nz.js":739,"./en-sg":740,"./en-sg.js":740,"./eo":741,"./eo.js":741,"./es":742,"./es-do":743,"./es-do.js":743,"./es-mx":744,"./es-mx.js":744,"./es-us":745,"./es-us.js":745,"./es.js":742,"./et":746,"./et.js":746,"./eu":747,"./eu.js":747,"./fa":748,"./fa.js":748,"./fi":749,"./fi.js":749,"./fil":750,"./fil.js":750,"./fo":751,"./fo.js":751,"./fr":752,"./fr-ca":753,"./fr-ca.js":753,"./fr-ch":754,"./fr-ch.js":754,"./fr.js":752,"./fy":755,"./fy.js":755,"./ga":756,"./ga.js":756,"./gd":757,"./gd.js":757,"./gl":758,"./gl.js":758,"./gom-deva":759,"./gom-deva.js":759,"./gom-latn":760,"./gom-latn.js":760,"./gu":761,"./gu.js":761,"./he":762,"./he.js":762,"./hi":763,"./hi.js":763,"./hr":764,"./hr.js":764,"./hu":765,"./hu.js":765,"./hy-am":766,"./hy-am.js":766,"./id":767,"./id.js":767,"./is":768,"./is.js":768,"./it":769,"./it-ch":770,"./it-ch.js":770,"./it.js":769,"./ja":771,"./ja.js":771,"./jv":772,"./jv.js":772,"./ka":773,"./ka.js":773,"./kk":774,"./kk.js":774,"./km":775,"./km.js":775,"./kn":776,"./kn.js":776,"./ko":777,"./ko.js":777,"./ku":778,"./ku.js":778,"./ky":779,"./ky.js":779,"./lb":780,"./lb.js":780,"./lo":781,"./lo.js":781,"./lt":782,"./lt.js":782,"./lv":783,"./lv.js":783,"./me":784,"./me.js":784,"./mi":785,"./mi.js":785,"./mk":786,"./mk.js":786,"./ml":787,"./ml.js":787,"./mn":788,"./mn.js":788,"./mr":789,"./mr.js":789,"./ms":790,"./ms-my":791,"./ms-my.js":791,"./ms.js":790,"./mt":792,"./mt.js":792,"./my":793,"./my.js":793,"./nb":794,"./nb.js":794,"./ne":795,"./ne.js":795,"./nl":796,"./nl-be":797,"./nl-be.js":797,"./nl.js":796,"./nn":798,"./nn.js":798,"./oc-lnc":799,"./oc-lnc.js":799,"./pa-in":800,"./pa-in.js":800,"./pl":801,"./pl.js":801,"./pt":802,"./pt-br":803,"./pt-br.js":803,"./pt.js":802,"./ro":804,"./ro.js":804,"./ru":805,"./ru.js":805,"./sd":806,"./sd.js":806,"./se":807,"./se.js":807,"./si":808,"./si.js":808,"./sk":809,"./sk.js":809,"./sl":810,"./sl.js":810,"./sq":811,"./sq.js":811,"./sr":812,"./sr-cyrl":813,"./sr-cyrl.js":813,"./sr.js":812,"./ss":814,"./ss.js":814,"./sv":815,"./sv.js":815,"./sw":816,"./sw.js":816,"./ta":817,"./ta.js":817,"./te":818,"./te.js":818,"./tet":819,"./tet.js":819,"./tg":820,"./tg.js":820,"./th":821,"./th.js":821,"./tk":822,"./tk.js":822,"./tl-ph":823,"./tl-ph.js":823,"./tlh":824,"./tlh.js":824,"./tr":825,"./tr.js":825,"./tzl":826,"./tzl.js":826,"./tzm":827,"./tzm-latn":828,"./tzm-latn.js":828,"./tzm.js":827,"./ug-cn":829,"./ug-cn.js":829,"./uk":830,"./uk.js":830,"./ur":831,"./ur.js":831,"./uz":832,"./uz-latn":833,"./uz-latn.js":833,"./uz.js":832,"./vi":834,"./vi.js":834,"./x-pseudo":835,"./x-pseudo.js":835,"./yo":836,"./yo.js":836,"./zh-cn":837,"./zh-cn.js":837,"./zh-hk":838,"./zh-hk.js":838,"./zh-mo":839,"./zh-mo.js":839,"./zh-tw":840,"./zh-tw.js":840};function i(e){var t=o(e);return n(t)}function o(e){if(!n.o(r,e)){var t=new Error("Cannot find module '"+e+"'");throw t.code="MODULE_NOT_FOUND",t}return r[e]}i.keys=function(){return Object.keys(r)},i.resolve=o,e.exports=i,i.id=1354},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.validateCheckpointModel=t.validateContentsModel=void 0;const r=n(352);t.validateContentsModel=function(e){r.validateProperty(e,"name","string"),r.validateProperty(e,"path","string"),r.validateProperty(e,"type","string"),r.validateProperty(e,"created","string"),r.validateProperty(e,"last_modified","string"),r.validateProperty(e,"mimetype","object"),r.validateProperty(e,"content","object"),r.validateProperty(e,"format","object")},t.validateCheckpointModel=function(e){r.validateProperty(e,"id","string"),r.validateProperty(e,"last_modified","string")}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0})},function(e,t){},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=WebSocket},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.KernelManager=void 0;const r=n(6),i=n(261),o=n(16),a=n(84),s=n(354),l=n(487),u=n(1360);class c extends s.BaseManager{constructor(e={}){var t;super(e),this._isReady=!1,this._kernelConnections=new Set,this._models=new Map,this._runningChanged=new o.Signal(this),this._connectionFailure=new o.Signal(this),this._pollModels=new i.Poll({auto:!1,factory:()=>this.requestRunning(),frequency:{interval:1e4,backoff:!0,max:3e5},name:"@jupyterlab/services:KernelManager#models",standby:null!==(t=e.standby)&&void 0!==t?t:"when-hidden"}),this._ready=(async()=>{await this._pollModels.start(),await this._pollModels.tick,this._isReady=!0})()}get isReady(){return this._isReady}get ready(){return this._ready}get runningChanged(){return this._runningChanged}get connectionFailure(){return this._connectionFailure}dispose(){this.isDisposed||(this._models.clear(),this._kernelConnections.forEach(e=>e.dispose()),this._pollModels.dispose(),super.dispose())}connectTo(e){var t;const{id:n}=e.model;let r=null===(t=e.handleComms)||void 0===t||t;if(void 0===e.handleComms)for(const e of this._kernelConnections)if(e.id===n&&e.handleComms){r=!1;break}const i=new u.KernelConnection(Object.assign(Object.assign({handleComms:r},e),{serverSettings:this.serverSettings}));return this._onStarted(i),this._models.has(n)||this.refreshRunning().catch(()=>{}),i}running(){return r.iter([...this._models.values()])}async refreshRunning(){await this._pollModels.refresh(),await this._pollModels.tick}async startNew(e={},t={}){const n=await l.startNew(e,this.serverSettings);return this.connectTo(Object.assign(Object.assign({},t),{model:n}))}async shutdown(e){await l.shutdownKernel(e,this.serverSettings),await this.refreshRunning()}async shutdownAll(){await this.refreshRunning(),await Promise.all([...this._models.keys()].map(e=>l.shutdownKernel(e,this.serverSettings))),await this.refreshRunning()}async findById(e){return this._models.has(e)||await this.refreshRunning(),this._models.get(e)}async requestRunning(){var e;let t;try{t=await l.listRunning(this.serverSettings)}catch(t){throw(t instanceof a.ServerConnection.NetworkError||503===(null===(e=t.response)||void 0===e?void 0:e.status))&&this._connectionFailure.emit(t),t}this.isDisposed||this._models.size===t.length&&r.every(t,e=>{const t=this._models.get(e.id);return!!t&&t.name===e.name})||(this._models=new Map(t.map(e=>[e.id,e])),this._kernelConnections.forEach(e=>{this._models.has(e.id)||e.handleShutdown()}),this._runningChanged.emit(t))}_onStarted(e){this._kernelConnections.add(e),e.statusChanged.connect(this._onStatusChanged,this),e.disposed.connect(this._onDisposed,this)}_onDisposed(e){this._kernelConnections.delete(e),this.refreshRunning().catch(()=>{})}_onStatusChanged(e,t){"dead"===t&&this.refreshRunning().catch(()=>{})}}t.KernelManager=c},function(e,t,n){"use strict";var r=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),i=this&&this.__setModuleDefault||(Object.create?function(e,t){Object.defineProperty(e,"default",{enumerable:!0,value:t})}:function(e,t){e.default=t}),o=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)"default"!==n&&Object.prototype.hasOwnProperty.call(e,n)&&r(t,e,n);return i(t,e),t};Object.defineProperty(t,"__esModule",{value:!0}),t.KernelConnection=void 0;const a=n(48),s=n(17),l=n(16),u=n(84),c=n(1361),d=o(n(353)),h=n(1362),f=o(n(1363)),p=o(n(488)),g=n(489),v=o(n(487));class m{constructor(e){var t,n,r,i;this._createSocket=()=>{this._errorIfDisposed(),this._clearSocket(),this._updateConnectionStatus("connecting");const e=this.serverSettings,t=a.URLExt.join(e.wsUrl,v.KERNEL_SERVICE_URL,encodeURIComponent(this._id)),n=t.replace(/^((?:\w+:)?\/\/)(?:[^@\/]+@)/,"$1");console.debug("Starting WebSocket: "+n);let r=a.URLExt.join(t,"channels?session_id="+encodeURIComponent(this._clientId));const i=e.token;e.appendToken&&""!==i&&(r=r+"&token="+encodeURIComponent(i)),this._ws=new e.WebSocket(r),this._ws.binaryType="arraybuffer",this._ws.onmessage=this._onWSMessage,this._ws.onopen=this._onWSOpen,this._ws.onclose=this._onWSClose,this._ws.onerror=this._onWSClose},this._onWSOpen=e=>{this._updateConnectionStatus("connected")},this._onWSMessage=e=>{let t;try{t=f.deserialize(e.data),p.validateMessage(t)}catch(e){throw e.message="Kernel message validation error: "+e.message,e}this._kernelSession=t.header.session,this._msgChain=this._msgChain.then(()=>this._handleMessage(t)).catch(e=>{e.message.startsWith("Canceled future for ")&&console.error(e)}),this._anyMessage.emit({msg:t,direction:"recv"})},this._onWSClose=e=>{this.isDisposed||this._reconnect()},this._id="",this._name="",this._status="unknown",this._connectionStatus="connecting",this._kernelSession="",this._isDisposed=!1,this._ws=null,this._username="",this._reconnectLimit=7,this._reconnectAttempt=0,this._reconnectTimeout=null,this._futures=new Map,this._comms=new Map,this._targetRegistry=Object.create(null),this._info=new s.PromiseDelegate,this._pendingMessages=[],this._statusChanged=new l.Signal(this),this._connectionStatusChanged=new l.Signal(this),this._disposed=new l.Signal(this),this._iopubMessage=new l.Signal(this),this._anyMessage=new l.Signal(this),this._unhandledMessage=new l.Signal(this),this._displayIdToParentIds=new Map,this._msgIdToDisplayIds=new Map,this._msgChain=Promise.resolve(),this._noOp=()=>{},this._name=e.model.name,this._id=e.model.id,this.serverSettings=null!==(t=e.serverSettings)&&void 0!==t?t:u.ServerConnection.makeSettings(),this._clientId=null!==(n=e.clientId)&&void 0!==n?n:s.UUID.uuid4(),this._username=null!==(r=e.username)&&void 0!==r?r:"",this.handleComms=null===(i=e.handleComms)||void 0===i||i,this._createSocket(),this.requestKernelInfo()}get disposed(){return this._disposed}get statusChanged(){return this._statusChanged}get connectionStatusChanged(){return this._connectionStatusChanged}get iopubMessage(){return this._iopubMessage}get unhandledMessage(){return this._unhandledMessage}get model(){return{id:this.id,name:this.name}}get anyMessage(){return this._anyMessage}get id(){return this._id}get name(){return this._name}get username(){return this._username}get clientId(){return this._clientId}get status(){return this._status}get connectionStatus(){return this._connectionStatus}get isDisposed(){return this._isDisposed}get info(){return this._info.promise}get spec(){return this._specPromise||(this._specPromise=g.KernelSpecAPI.getSpecs(this.serverSettings).then(e=>e.kernelspecs[this._name])),this._specPromise}clone(e={}){return new m(Object.assign({model:this.model,username:this.username,serverSettings:this.serverSettings,handleComms:!1},e))}dispose(){this.isDisposed||(this._isDisposed=!0,this._disposed.emit(),this._updateConnectionStatus("disconnected"),this._clearKernelState(),this._pendingMessages=[],this._clearSocket(),l.Signal.clearData(this))}sendShellMessage(e,t=!1,n=!0){return this._sendKernelShellControl(h.KernelShellFutureHandler,e,t,n)}sendControlMessage(e,t=!1,n=!0){return this._sendKernelShellControl(h.KernelControlFutureHandler,e,t,n)}_sendKernelShellControl(e,t,n=!1,r=!0){this._sendMessage(t),this._anyMessage.emit({msg:t,direction:"send"});const i=new e(()=>{const e=t.header.msg_id;this._futures.delete(e);const n=this._msgIdToDisplayIds.get(e);n&&(n.forEach(t=>{const n=this._displayIdToParentIds.get(t);if(n){const r=n.indexOf(e);if(-1===r)return;1===n.length?this._displayIdToParentIds.delete(t):(n.splice(r,1),this._displayIdToParentIds.set(t,n))}}),this._msgIdToDisplayIds.delete(e))},t,n,r,this);return this._futures.set(t.header.msg_id,i),i}_sendMessage(e,t=!0){if("dead"===this.status)throw new Error("Kernel is dead");if("_RESTARTING_"===this._kernelSession&&d.isInfoRequestMsg(e)){if("connected"===this.connectionStatus)return void this._ws.send(f.serialize(e));throw new Error("Could not send message: status is not connected")}if(t&&this._pendingMessages.length>0)this._pendingMessages.push(e);else if("connected"===this.connectionStatus&&"_RESTARTING_"!==this._kernelSession)this._ws.send(f.serialize(e));else{if(!t)throw new Error("Could not send message");this._pendingMessages.push(e)}}async interrupt(){if("dead"===this.status)throw new Error("Kernel is dead");return v.interruptKernel(this.id,this.serverSettings)}async restart(){if("dead"===this.status)throw new Error("Kernel is dead");this._updateStatus("restarting"),this._clearKernelState(),this._kernelSession="_RESTARTING_",await v.restartKernel(this.id,this.serverSettings),await this.reconnect()}reconnect(){this._errorIfDisposed();const e=new s.PromiseDelegate,t=(n,r)=>{"connected"===r?(e.resolve(),this.connectionStatusChanged.disconnect(t,this)):"disconnected"===r&&(e.reject(new Error("Kernel connection disconnected")),this.connectionStatusChanged.disconnect(t,this))};return this.connectionStatusChanged.connect(t,this),this._reconnectAttempt=0,this._reconnect(),e.promise}async shutdown(){"dead"!==this.status&&await v.shutdownKernel(this.id,this.serverSettings),this.handleShutdown()}handleShutdown(){this._updateStatus("dead"),this.dispose()}async requestKernelInfo(){const e=d.createMessage({msgType:"kernel_info_request",channel:"shell",username:this._username,session:this._clientId,content:{}});let t;try{t=await y.handleShellMessage(this,e)}catch(e){if(this.isDisposed)return;throw e}if(this._errorIfDisposed(),t)return void 0===t.content.status&&(t.content.status="ok"),"ok"!==t.content.status?(this._info.reject("Kernel info reply errored"),t):(this._info.resolve(t.content),this._kernelSession=t.header.session,t)}requestComplete(e){const t=d.createMessage({msgType:"complete_request",channel:"shell",username:this._username,session:this._clientId,content:e});return y.handleShellMessage(this,t)}requestInspect(e){const t=d.createMessage({msgType:"inspect_request",channel:"shell",username:this._username,session:this._clientId,content:e});return y.handleShellMessage(this,t)}requestHistory(e){const t=d.createMessage({msgType:"history_request",channel:"shell",username:this._username,session:this._clientId,content:e});return y.handleShellMessage(this,t)}requestExecute(e,t=!0,n){const r=d.createMessage({msgType:"execute_request",channel:"shell",username:this._username,session:this._clientId,content:Object.assign(Object.assign({},{silent:!1,store_history:!0,user_expressions:{},allow_stdin:!0,stop_on_error:!1}),e),metadata:n});return this.sendShellMessage(r,!0,t)}requestDebug(e,t=!0){const n=d.createMessage({msgType:"debug_request",channel:"control",username:this._username,session:this._clientId,content:e});return this.sendControlMessage(n,!0,t)}requestIsComplete(e){const t=d.createMessage({msgType:"is_complete_request",channel:"shell",username:this._username,session:this._clientId,content:e});return y.handleShellMessage(this,t)}requestCommInfo(e){const t=d.createMessage({msgType:"comm_info_request",channel:"shell",username:this._username,session:this._clientId,content:e});return y.handleShellMessage(this,t)}sendInputReply(e){const t=d.createMessage({msgType:"input_reply",channel:"stdin",username:this._username,session:this._clientId,content:e});this._sendMessage(t),this._anyMessage.emit({msg:t,direction:"send"})}createComm(e,t=s.UUID.uuid4()){if(!this.handleComms)throw new Error("Comms are disabled on this kernel connection");if(this._comms.has(t))throw new Error("Comm is already created");const n=new c.CommHandler(e,t,this,()=>{this._unregisterComm(t)});return this._comms.set(t,n),n}hasComm(e){return this._comms.has(e)}registerCommTarget(e,t){this.handleComms&&(this._targetRegistry[e]=t)}removeCommTarget(e,t){this.handleComms&&(this.isDisposed||this._targetRegistry[e]!==t||delete this._targetRegistry[e])}registerMessageHook(e,t){var n;const r=null===(n=this._futures)||void 0===n?void 0:n.get(e);r&&r.registerMessageHook(t)}removeMessageHook(e,t){var n;const r=null===(n=this._futures)||void 0===n?void 0:n.get(e);r&&r.removeMessageHook(t)}async _handleDisplayId(e,t){var n,r;const i=t.parent_header.msg_id;let o=this._displayIdToParentIds.get(e);if(o){const e={header:s.JSONExt.deepCopy(t.header),parent_header:s.JSONExt.deepCopy(t.parent_header),metadata:s.JSONExt.deepCopy(t.metadata),content:s.JSONExt.deepCopy(t.content),channel:t.channel,buffers:t.buffers?t.buffers.slice():[]};e.header.msg_type="update_display_data",await Promise.all(o.map(async t=>{const n=this._futures&&this._futures.get(t);n&&await n.handleMsg(e)}))}if("update_display_data"===t.header.msg_type)return!0;o=null!==(n=this._displayIdToParentIds.get(e))&&void 0!==n?n:[],-1===o.indexOf(i)&&o.push(i),this._displayIdToParentIds.set(e,o);const a=null!==(r=this._msgIdToDisplayIds.get(i))&&void 0!==r?r:[];return-1===a.indexOf(i)&&a.push(i),this._msgIdToDisplayIds.set(i,a),!1}_clearSocket(){null!==this._ws&&(this._ws.onopen=this._noOp,this._ws.onclose=this._noOp,this._ws.onerror=this._noOp,this._ws.onmessage=this._noOp,this._ws.close(),this._ws=null)}_updateStatus(e){this._status!==e&&"dead"!==this._status&&(this._status=e,y.logKernelStatus(this),this._statusChanged.emit(e),"dead"===e&&this.dispose())}_sendPending(){for(;"connected"===this.connectionStatus&&"_RESTARTING_"!==this._kernelSession&&this._pendingMessages.length>0;)this._sendMessage(this._pendingMessages[0],!1),this._pendingMessages.shift()}_clearKernelState(){this._kernelSession="",this._pendingMessages=[],this._futures.forEach(e=>{e.dispose()}),this._comms.forEach(e=>{e.dispose()}),this._msgChain=Promise.resolve(),this._futures=new Map,this._comms=new Map,this._displayIdToParentIds.clear(),this._msgIdToDisplayIds.clear()}_assertCurrentMessage(e){if(this._errorIfDisposed(),e.header.session!==this._kernelSession)throw new Error("Canceling handling of old message: "+e.header.msg_type)}async _handleCommOpen(e){this._assertCurrentMessage(e);const t=e.content,n=new c.CommHandler(t.target_name,t.comm_id,this,()=>{this._unregisterComm(t.comm_id)});this._comms.set(t.comm_id,n);try{const r=await y.loadObject(t.target_name,t.target_module,this._targetRegistry);await r(n,e)}catch(e){throw n.close(),console.error("Exception opening new comm"),e}}async _handleCommClose(e){this._assertCurrentMessage(e);const t=e.content,n=this._comms.get(t.comm_id);if(!n)return void console.error("Comm not found for comm id "+t.comm_id);this._unregisterComm(n.commId);const r=n.onClose;r&&await r(e),n.dispose()}async _handleCommMsg(e){this._assertCurrentMessage(e);const t=e.content,n=this._comms.get(t.comm_id);if(!n)return;const r=n.onMsg;r&&await r(e)}_unregisterComm(e){this._comms.delete(e)}_updateConnectionStatus(e){if(this._connectionStatus!==e){if(this._connectionStatus=e,"connecting"!==e&&(this._reconnectAttempt=0,clearTimeout(this._reconnectTimeout)),"dead"!==this.status)if("connected"===e){let e="_RESTARTING_"===this._kernelSession,t=this.requestKernelInfo(),n=!1,r=()=>{n||(n=!0,e&&"_RESTARTING_"===this._kernelSession&&(this._kernelSession=""),clearTimeout(i),this._pendingMessages.length>0&&this._sendPending())};t.then(r);let i=setTimeout(r,3e3)}else this._updateStatus("unknown");this._connectionStatusChanged.emit(e)}}async _handleMessage(e){var t,n;let r=!1;if(e.parent_header&&"iopub"===e.channel&&(d.isDisplayDataMsg(e)||d.isUpdateDisplayDataMsg(e)||d.isExecuteResultMsg(e))){const n=(null!==(t=e.content.transient)&&void 0!==t?t:{}).display_id;n&&(r=await this._handleDisplayId(n,e),this._assertCurrentMessage(e))}if(!r&&e.parent_header){const t=e.parent_header,r=null===(n=this._futures)||void 0===n?void 0:n.get(t.msg_id);if(r)await r.handleMsg(e),this._assertCurrentMessage(e);else{const n=t.session===this.clientId;"iopub"!==e.channel&&n&&this._unhandledMessage.emit(e)}}if("iopub"===e.channel){switch(e.header.msg_type){case"status":{const t=e.content.execution_state;"restarting"===t&&Promise.resolve().then(async()=>{this._updateStatus("autorestarting"),this._clearKernelState(),await this.reconnect()}),this._updateStatus(t);break}case"comm_open":this.handleComms&&await this._handleCommOpen(e);break;case"comm_msg":this.handleComms&&await this._handleCommMsg(e);break;case"comm_close":this.handleComms&&await this._handleCommClose(e)}this.isDisposed||(this._assertCurrentMessage(e),this._iopubMessage.emit(e))}}_reconnect(){if(this._errorIfDisposed(),clearTimeout(this._reconnectTimeout),this._reconnectAttempt<this._reconnectLimit){this._updateConnectionStatus("connecting");const e=y.getRandomIntInclusive(0,1e3*(Math.pow(2,this._reconnectAttempt)-1));console.warn(`Connection lost, reconnecting in ${Math.floor(e/1e3)} seconds.`),this._reconnectTimeout=setTimeout(this._createSocket,e),this._reconnectAttempt+=1}else this._updateConnectionStatus("disconnected");this._clearSocket()}_errorIfDisposed(){if(this.isDisposed)throw new Error("Kernel connection is disposed")}}var y;t.KernelConnection=m,function(e){e.logKernelStatus=function(e){switch(e.status){case"idle":case"busy":case"unknown":return;default:console.debug(`Kernel: ${e.status} (${e.id})`)}},e.handleShellMessage=async function(e,t){return e.sendShellMessage(t,!0).done},e.loadObject=function(e,t,n){return new Promise((r,i)=>{if(t){if("undefined"==typeof requirejs)throw new Error("requirejs not found");requirejs([t],n=>{if(void 0===n[e]){i(new Error(`Object '${e}' not found in module '${t}'`))}else r(n[e])},i)}else(null==n?void 0:n[e])?r(n[e]):i(new Error(`Object '${e}' not found in registry`))})},e.getRandomIntInclusive=function(e,t){return e=Math.ceil(e),t=Math.floor(t),Math.floor(Math.random()*(t-e+1))+e}}(y||(y={}))},function(e,t,n){"use strict";var r=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),i=this&&this.__setModuleDefault||(Object.create?function(e,t){Object.defineProperty(e,"default",{enumerable:!0,value:t})}:function(e,t){e.default=t}),o=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)"default"!==n&&Object.prototype.hasOwnProperty.call(e,n)&&r(t,e,n);return i(t,e),t};Object.defineProperty(t,"__esModule",{value:!0}),t.CommHandler=void 0;const a=n(86),s=o(n(353));class l extends a.DisposableDelegate{constructor(e,t,n,r){super(r),this._target="",this._id="",this._id=t,this._target=e,this._kernel=n}get commId(){return this._id}get targetName(){return this._target}get onClose(){return this._onClose}set onClose(e){this._onClose=e}get onMsg(){return this._onMsg}set onMsg(e){this._onMsg=e}open(e,t,n=[]){if(this.isDisposed||this._kernel.isDisposed)throw new Error("Cannot open");const r=s.createMessage({msgType:"comm_open",channel:"shell",username:this._kernel.username,session:this._kernel.clientId,content:{comm_id:this._id,target_name:this._target,data:null!=e?e:{}},metadata:t,buffers:n});return this._kernel.sendShellMessage(r,!1,!0)}send(e,t,n=[],r=!0){if(this.isDisposed||this._kernel.isDisposed)throw new Error("Cannot send");const i=s.createMessage({msgType:"comm_msg",channel:"shell",username:this._kernel.username,session:this._kernel.clientId,content:{comm_id:this._id,data:e},metadata:t,buffers:n});return this._kernel.sendShellMessage(i,!1,r)}close(e,t,n=[]){if(this.isDisposed||this._kernel.isDisposed)throw new Error("Cannot close");const r=s.createMessage({msgType:"comm_close",channel:"shell",username:this._kernel.username,session:this._kernel.clientId,content:{comm_id:this._id,data:null!=e?e:{}},metadata:t,buffers:n}),i=this._kernel.sendShellMessage(r,!1,!0),o=this._onClose;if(o){o(s.createMessage({msgType:"comm_close",channel:"iopub",username:this._kernel.username,session:this._kernel.clientId,content:{comm_id:this._id,data:null!=e?e:{}},metadata:t,buffers:n}))}return this.dispose(),i}}t.CommHandler=l},function(e,t,n){"use strict";(function(e){var r=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),i=this&&this.__setModuleDefault||(Object.create?function(e,t){Object.defineProperty(e,"default",{enumerable:!0,value:t})}:function(e,t){e.default=t}),o=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)"default"!==n&&Object.prototype.hasOwnProperty.call(e,n)&&r(t,e,n);return i(t,e),t};Object.defineProperty(t,"__esModule",{value:!0}),t.KernelShellFutureHandler=t.KernelControlFutureHandler=t.KernelFutureHandler=void 0;const a=n(17),s=n(86),l=o(n(353));class u extends s.DisposableDelegate{constructor(e,t,n,r,i){super(e),this._status=0,this._stdin=c.noOp,this._iopub=c.noOp,this._reply=c.noOp,this._done=new a.PromiseDelegate,this._hooks=new c.HookList,this._disposeOnDone=!0,this._msg=t,n||this._setFlag(c.KernelFutureFlag.GotReply),this._disposeOnDone=r,this._kernel=i}get msg(){return this._msg}get done(){return this._done.promise}get onReply(){return this._reply}set onReply(e){this._reply=e}get onIOPub(){return this._iopub}set onIOPub(e){this._iopub=e}get onStdin(){return this._stdin}set onStdin(e){this._stdin=e}registerMessageHook(e){if(this.isDisposed)throw new Error("Kernel future is disposed");this._hooks.add(e)}removeMessageHook(e){this.isDisposed||this._hooks.remove(e)}sendInputReply(e){this._kernel.sendInputReply(e)}dispose(){this._stdin=c.noOp,this._iopub=c.noOp,this._reply=c.noOp,this._hooks=null,this._testFlag(c.KernelFutureFlag.IsDone)||(this._done.promise.catch(()=>{}),this._done.reject(new Error(`Canceled future for ${this.msg.header.msg_type} message before replies were done`))),super.dispose()}async handleMsg(e){switch(e.channel){case"control":case"shell":e.channel===this.msg.channel&&e.parent_header.msg_id===this.msg.header.msg_id&&await this._handleReply(e);break;case"stdin":await this._handleStdin(e);break;case"iopub":await this._handleIOPub(e)}}async _handleReply(e){const t=this._reply;t&&await t(e),this._replyMsg=e,this._setFlag(c.KernelFutureFlag.GotReply),this._testFlag(c.KernelFutureFlag.GotIdle)&&this._handleDone()}async _handleStdin(e){const t=this._stdin;t&&await t(e)}async _handleIOPub(e){const t=await this._hooks.process(e),n=this._iopub;t&&n&&await n(e),l.isStatusMsg(e)&&"idle"===e.content.execution_state&&(this._setFlag(c.KernelFutureFlag.GotIdle),this._testFlag(c.KernelFutureFlag.GotReply)&&this._handleDone())}_handleDone(){this._testFlag(c.KernelFutureFlag.IsDone)||(this._setFlag(c.KernelFutureFlag.IsDone),this._done.resolve(this._replyMsg),this._disposeOnDone&&this.dispose())}_testFlag(e){return 0!=(this._status&e)}_setFlag(e){this._status|=e}}t.KernelFutureHandler=u;t.KernelControlFutureHandler=class extends u{};var c;t.KernelShellFutureHandler=class extends u{},function(t){t.noOp=()=>{};const n="function"==typeof requestAnimationFrame?requestAnimationFrame:e;let r;t.HookList=class{constructor(){this._hooks=[]}add(e){this.remove(e),this._hooks.push(e)}remove(e){const t=this._hooks.indexOf(e);t>=0&&(this._hooks[t]=null,this._scheduleCompact())}async process(e){await this._processing;const t=new a.PromiseDelegate;let n;this._processing=t.promise;for(let r=this._hooks.length-1;r>=0;r--){const i=this._hooks[r];if(null!==i){try{n=await i(e)}catch(e){n=!0,console.error(e)}if(!1===n)return t.resolve(void 0),!1}}return t.resolve(void 0),!0}_scheduleCompact(){this._compactScheduled||(this._compactScheduled=!0,n(()=>{this._processing=this._processing.then(()=>{this._compactScheduled=!1,this._compact()})}))}_compact(){let e=0;for(let t=0,n=this._hooks.length;t<n;t++){const n=this._hooks[t];null===this._hooks[t]?e++:this._hooks[t-e]=n}this._hooks.length-=e}},function(e){e[e.GotReply=1]="GotReply",e[e.GotIdle=2]="GotIdle",e[e.IsDone=4]="IsDone",e[e.DisposeOnDone=8]="DisposeOnDone"}(r=t.KernelFutureFlag||(t.KernelFutureFlag={}))}(c||(c={}))}).call(this,n(234).setImmediate)},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.serialize=t.deserialize=void 0,t.deserialize=function(e){let t;return t="string"==typeof e?JSON.parse(e):function(e){const t=new DataView(e),n=t.getUint32(0),r=[];if(n<2)throw new Error("Invalid incoming Kernel Message");for(let e=1;e<=n;e++)r.push(t.getUint32(4*e));const i=new Uint8Array(e.slice(r[0],r[1])),o=JSON.parse(new TextDecoder("utf8").decode(i));o.buffers=[];for(let t=1;t<n;t++){const n=r[t],i=r[t+1]||e.byteLength;o.buffers.push(new DataView(e.slice(n,i)))}return o}(e),t},t.serialize=function(e){var t;let n;return n=(null===(t=e.buffers)||void 0===t?void 0:t.length)?function(e){const t=[],n=[],r=new TextEncoder;let i=[];void 0!==e.buffers&&(i=e.buffers,delete e.buffers);const o=r.encode(JSON.stringify(e));n.push(o.buffer);for(let e=0;e<i.length;e++){const t=i[e];n.push(ArrayBuffer.isView(t)?t.buffer:t)}const a=n.length;t.push(4*(a+1));for(let e=0;e+1<n.length;e++)t.push(t[t.length-1]+n[e].byteLength);const s=new Uint8Array(t[t.length-1]+n[n.length-1].byteLength),l=new DataView(s.buffer);l.setUint32(0,a);for(let e=0;e<t.length;e++)l.setUint32(4*(e+1),t[e]);for(let e=0;e<n.length;e++)s.set(new Uint8Array(n[e]),t[e]);return s.buffer}(e):JSON.stringify(e),n}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0})},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.validateSpecModels=t.validateSpecModel=void 0;const r=n(352);function i(e){const t=e.spec;if(!t)throw new Error("Invalid kernel spec");r.validateProperty(e,"name","string"),r.validateProperty(e,"resources","object"),r.validateProperty(t,"language","string"),r.validateProperty(t,"display_name","string"),r.validateProperty(t,"argv","array");let n=null;t.hasOwnProperty("metadata")&&(r.validateProperty(t,"metadata","object"),n=t.metadata);let i=null;return t.hasOwnProperty("env")&&(r.validateProperty(t,"env","object"),i=t.env),{name:e.name,resources:e.resources,language:t.language,display_name:t.display_name,argv:t.argv,metadata:n,env:i}}t.validateSpecModel=i,t.validateSpecModels=function(e){if(!e.hasOwnProperty("kernelspecs"))throw new Error("No kernelspecs found");let t=Object.keys(e.kernelspecs);const n=Object.create(null);let r=e.default;for(let r=0;r<t.length;r++){const o=e.kernelspecs[t[r]];try{n[t[r]]=i(o)}catch(e){console.warn("Removing errant kernel spec: "+t[r])}}if(t=Object.keys(n),!t.length)throw new Error("No valid kernelspecs found");return r&&"string"==typeof r&&r in n||(r=t[0],console.warn(`Default kernel not found, using '${t[0]}'`)),{default:r,kernelspecs:n}}},function(e,t,n){"use strict";var r=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),i=this&&this.__setModuleDefault||(Object.create?function(e,t){Object.defineProperty(e,"default",{enumerable:!0,value:t})}:function(e,t){e.default=t}),o=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)"default"!==n&&Object.prototype.hasOwnProperty.call(e,n)&&r(t,e,n);return i(t,e),t};Object.defineProperty(t,"__esModule",{value:!0}),t.KernelSpecManager=void 0;const a=n(17),s=n(261),l=n(16),u=o(n(843)),c=n(354);class d extends c.BaseManager{constructor(e={}){var t;super(e),this._isReady=!1,this._connectionFailure=new l.Signal(this),this._specs=null,this._specsChanged=new l.Signal(this),this._ready=Promise.all([this.requestSpecs()]).then(e=>{}).catch(e=>{}).then(()=>{this.isDisposed||(this._isReady=!0)}),this._pollSpecs=new s.Poll({auto:!1,factory:()=>this.requestSpecs(),frequency:{interval:61e3,backoff:!0,max:3e5},name:"@jupyterlab/services:KernelSpecManager#specs",standby:null!==(t=e.standby)&&void 0!==t?t:"when-hidden"}),this.ready.then(()=>{this._pollSpecs.start()})}get isReady(){return this._isReady}get ready(){return this._ready}get specs(){return this._specs}get specsChanged(){return this._specsChanged}get connectionFailure(){return this._connectionFailure}dispose(){this._pollSpecs.dispose(),super.dispose()}async refreshSpecs(){await this._pollSpecs.refresh(),await this._pollSpecs.tick}async requestSpecs(){const e=await u.getSpecs(this.serverSettings);this.isDisposed||a.JSONExt.deepEqual(e,this._specs)||(this._specs=e,this._specsChanged.emit(e))}}t.KernelSpecManager=d},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.ServiceManager=void 0;const r=n(16),i=n(1368),o=n(844),a=n(841),s=n(489),l=n(845),u=n(846),c=n(847),d=n(129),h=n(848),f=n(842);t.ServiceManager=class{constructor(e={}){var t,n;this._isDisposed=!1,this._connectionFailure=new r.Signal(this),this._isReady=!1;const p=e.defaultDrive,g=null!==(t=e.serverSettings)&&void 0!==t?t:d.ServerConnection.makeSettings(),v={defaultDrive:p,serverSettings:g,standby:null!==(n=e.standby)&&void 0!==n?n:"when-hidden"},m=new f.KernelManager(v);this.serverSettings=g,this.contents=new a.ContentsManager(v),this.sessions=new l.SessionManager(Object.assign(Object.assign({},v),{kernelManager:m})),this.settings=new u.SettingManager(v),this.terminals=new c.TerminalManager(v),this.builder=new i.BuildManager(v),this.workspaces=new h.WorkspaceManager(v),this.nbconvert=new o.NbConvertManager(v),this.kernelspecs=new s.KernelSpecManager(v),this.kernelspecs.connectionFailure.connect(this._onConnectionFailure,this),this.sessions.connectionFailure.connect(this._onConnectionFailure,this),this.terminals.connectionFailure.connect(this._onConnectionFailure,this);const y=[this.sessions.ready,this.kernelspecs.ready];this.terminals.isAvailable()&&y.push(this.terminals.ready),this._readyPromise=Promise.all(y).then(()=>{this._isReady=!0})}get connectionFailure(){return this._connectionFailure}get isDisposed(){return this._isDisposed}dispose(){this.isDisposed||(this._isDisposed=!0,r.Signal.clearData(this),this.contents.dispose(),this.sessions.dispose(),this.terminals.dispose())}get isReady(){return this._isReady}get ready(){return this._readyPromise}_onConnectionFailure(e,t){this._connectionFailure.emit(t)}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.BuildManager=void 0;const r=n(48),i=n(129);t.BuildManager=class{constructor(e={}){var t;this._url="",this.serverSettings=null!==(t=e.serverSettings)&&void 0!==t?t:i.ServerConnection.makeSettings();const{baseUrl:n,appUrl:o}=this.serverSettings;this._url=r.URLExt.join(n,o,"api/build")}get isAvailable(){return"true"===r.PageConfig.getOption("buildAvailable").toLowerCase()}get shouldCheck(){return"true"===r.PageConfig.getOption("buildCheck").toLowerCase()}getStatus(){const{_url:e,serverSettings:t}=this;return i.ServerConnection.makeRequest(e,{},t).then(e=>{if(200!==e.status)throw new i.ServerConnection.ResponseError(e);return e.json()}).then(e=>{if("string"!=typeof e.status)throw new Error("Invalid data");if("string"!=typeof e.message)throw new Error("Invalid data");return e})}build(){const{_url:e,serverSettings:t}=this;return i.ServerConnection.makeRequest(e,{method:"POST"},t).then(e=>{if(400===e.status)throw new i.ServerConnection.ResponseError(e,"Build aborted");if(200!==e.status){const t=`Build failed with ${e.status}, please run 'jupyter lab build' on the server for full output`;throw new i.ServerConnection.ResponseError(e,t)}})}cancel(){const{_url:e,serverSettings:t}=this;return i.ServerConnection.makeRequest(e,{method:"DELETE"},t).then(e=>{if(204!==e.status)throw new i.ServerConnection.ResponseError(e)})}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0})},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.validateModels=t.updateLegacySessionModel=t.validateModel=void 0;const r=n(488),i=n(352);function o(e){i.validateProperty(e,"id","string"),i.validateProperty(e,"type","string"),i.validateProperty(e,"name","string"),i.validateProperty(e,"path","string"),i.validateProperty(e,"kernel","object"),r.validateModel(e.kernel)}t.validateModel=o,t.updateLegacySessionModel=function(e){void 0===e.path&&void 0!==e.notebook&&(e.path=e.notebook.path,e.type="notebook",e.name="")},t.validateModels=function(e){if(!Array.isArray(e))throw new Error("Invalid session list");e.forEach(e=>o(e))}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.SessionManager=void 0;const r=n(6),i=n(261),o=n(16),a=n(129),s=n(354),l=n(1372),u=n(490);class c extends s.BaseManager{constructor(e){var t;super(e),this._isReady=!1,this._sessionConnections=new Set,this._models=new Map,this._runningChanged=new o.Signal(this),this._connectionFailure=new o.Signal(this),this._connectToKernel=e=>this._kernelManager.connectTo(e),this._kernelManager=e.kernelManager,this._pollModels=new i.Poll({auto:!1,factory:()=>this.requestRunning(),frequency:{interval:1e4,backoff:!0,max:3e5},name:"@jupyterlab/services:SessionManager#models",standby:null!==(t=e.standby)&&void 0!==t?t:"when-hidden"}),this._ready=(async()=>{await this._pollModels.start(),await this._pollModels.tick,await this._kernelManager.ready,this._isReady=!0})()}get isReady(){return this._isReady}get ready(){return this._ready}get runningChanged(){return this._runningChanged}get connectionFailure(){return this._connectionFailure}dispose(){this.isDisposed||(this._models.clear(),this._sessionConnections.forEach(e=>e.dispose()),this._pollModels.dispose(),super.dispose())}connectTo(e){const t=new l.SessionConnection(Object.assign(Object.assign({},e),{connectToKernel:this._connectToKernel,serverSettings:this.serverSettings}));return this._onStarted(t),this._models.has(e.model.id)||this.refreshRunning().catch(()=>{}),t}running(){return r.iter([...this._models.values()])}async refreshRunning(){await this._pollModels.refresh(),await this._pollModels.tick}async startNew(e,t={}){const n=await u.startSession(e,this.serverSettings);return await this.refreshRunning(),this.connectTo(Object.assign(Object.assign({},t),{model:n}))}async shutdown(e){await u.shutdownSession(e,this.serverSettings),await this.refreshRunning()}async shutdownAll(){await this.refreshRunning(),await Promise.all([...this._models.keys()].map(e=>u.shutdownSession(e,this.serverSettings))),await this.refreshRunning()}async stopIfNeeded(e){try{const t=(await u.listRunning(this.serverSettings)).filter(t=>t.path===e);if(1===t.length){const e=t[0].id;await this.shutdown(e)}}catch(e){}}async findById(e){return this._models.has(e)||await this.refreshRunning(),this._models.get(e)}async findByPath(e){for(const t of this._models.values())if(t.path===e)return t;await this.refreshRunning();for(const t of this._models.values())if(t.path===e)return t}async requestRunning(){var e;let t;try{t=await u.listRunning(this.serverSettings)}catch(t){throw(t instanceof a.ServerConnection.NetworkError||503===(null===(e=t.response)||void 0===e?void 0:e.status))&&this._connectionFailure.emit(t),t}this.isDisposed||this._models.size===t.length&&r.every(t,e=>{var t,n,r,i;const o=this._models.get(e.id);return!!o&&((null===(t=o.kernel)||void 0===t?void 0:t.id)===(null===(n=e.kernel)||void 0===n?void 0:n.id)&&(null===(r=o.kernel)||void 0===r?void 0:r.name)===(null===(i=e.kernel)||void 0===i?void 0:i.name)&&o.name===e.name&&o.path===e.path&&o.type===e.type)})||(this._models=new Map(t.map(e=>[e.id,e])),this._sessionConnections.forEach(e=>{this._models.has(e.id)?e.update(this._models.get(e.id)):e.dispose()}),this._runningChanged.emit(t))}_onStarted(e){this._sessionConnections.add(e),e.disposed.connect(this._onDisposed,this),e.propertyChanged.connect(this._onChanged,this),e.kernelChanged.connect(this._onChanged,this)}_onDisposed(e){this._sessionConnections.delete(e),this.refreshRunning().catch(()=>{})}_onChanged(){this.refreshRunning().catch(()=>{})}}t.SessionManager=c},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.SessionConnection=void 0;const r=n(16),i=n(84),o=n(490),a=n(17);t.SessionConnection=class{constructor(e){var t,n,o,s;this._id="",this._path="",this._name="",this._type="",this._kernel=null,this._isDisposed=!1,this._disposed=new r.Signal(this),this._kernelChanged=new r.Signal(this),this._statusChanged=new r.Signal(this),this._connectionStatusChanged=new r.Signal(this),this._iopubMessage=new r.Signal(this),this._unhandledMessage=new r.Signal(this),this._anyMessage=new r.Signal(this),this._propertyChanged=new r.Signal(this),this._id=e.model.id,this._name=e.model.name,this._path=e.model.path,this._type=e.model.type,this._username=null!==(t=e.username)&&void 0!==t?t:"",this._clientId=null!==(n=e.clientId)&&void 0!==n?n:a.UUID.uuid4(),this._connectToKernel=e.connectToKernel,this._kernelConnectionOptions=null!==(o=e.kernelConnectionOptions)&&void 0!==o?o:{},this.serverSettings=null!==(s=e.serverSettings)&&void 0!==s?s:i.ServerConnection.makeSettings(),this.setupKernel(e.model.kernel)}get disposed(){return this._disposed}get kernelChanged(){return this._kernelChanged}get statusChanged(){return this._statusChanged}get connectionStatusChanged(){return this._connectionStatusChanged}get iopubMessage(){return this._iopubMessage}get unhandledMessage(){return this._unhandledMessage}get anyMessage(){return this._anyMessage}get propertyChanged(){return this._propertyChanged}get id(){return this._id}get kernel(){return this._kernel}get path(){return this._path}get type(){return this._type}get name(){return this._name}get model(){return{id:this.id,kernel:this.kernel&&{id:this.kernel.id,name:this.kernel.name},path:this._path,type:this._type,name:this._name}}get isDisposed(){return this._isDisposed}update(e){const t=this.model;if(this._path=e.path,this._name=e.name,this._type=e.type,null===this._kernel&&null!==e.kernel||null!==this._kernel&&null===e.kernel||null!==this._kernel&&null!==e.kernel&&this._kernel.id!==e.kernel.id){null!==this._kernel&&this._kernel.dispose();const t=this._kernel||null;this.setupKernel(e.kernel);const n=this._kernel||null;this._kernelChanged.emit({name:"kernel",oldValue:t,newValue:n})}this._handleModelChange(t)}dispose(){if(!this.isDisposed){if(this._isDisposed=!0,this._disposed.emit(),this._kernel){this._kernel.dispose();const e=this._kernel;this._kernel=null;const t=this._kernel;this._kernelChanged.emit({name:"kernel",oldValue:e,newValue:t})}r.Signal.clearData(this)}}async setPath(e){if(this.isDisposed)throw new Error("Session is disposed");await this._patch({path:e})}async setName(e){if(this.isDisposed)throw new Error("Session is disposed");await this._patch({name:e})}async setType(e){if(this.isDisposed)throw new Error("Session is disposed");await this._patch({type:e})}async changeKernel(e){if(this.isDisposed)throw new Error("Session is disposed");return await this._patch({kernel:e}),this.kernel}async shutdown(){if(this.isDisposed)throw new Error("Session is disposed");await o.shutdownSession(this.id,this.serverSettings),this.dispose()}setupKernel(e){if(null===e)return void(this._kernel=null);const t=this._connectToKernel(Object.assign(Object.assign({},this._kernelConnectionOptions),{model:e,username:this._username,clientId:this._clientId,serverSettings:this.serverSettings}));this._kernel=t,t.statusChanged.connect(this.onKernelStatus,this),t.connectionStatusChanged.connect(this.onKernelConnectionStatus,this),t.unhandledMessage.connect(this.onUnhandledMessage,this),t.iopubMessage.connect(this.onIOPubMessage,this),t.anyMessage.connect(this.onAnyMessage,this)}onKernelStatus(e,t){this._statusChanged.emit(t)}onKernelConnectionStatus(e,t){this._connectionStatusChanged.emit(t)}onIOPubMessage(e,t){this._iopubMessage.emit(t)}onUnhandledMessage(e,t){this._unhandledMessage.emit(t)}onAnyMessage(e,t){this._anyMessage.emit(t)}async _patch(e){const t=await o.updateSession(Object.assign(Object.assign({},e),{id:this._id}),this.serverSettings);return this.update(t),t}_handleModelChange(e){e.name!==this._name&&this._propertyChanged.emit("name"),e.type!==this._type&&this._propertyChanged.emit("type"),e.path!==this._path&&this._propertyChanged.emit("path")}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.DataConnector=void 0;t.DataConnector=class{async list(e){throw new Error("DataConnector#list method has not been implemented.")}async remove(e){throw new Error("DataConnector#remove method has not been implemented.")}async save(e,t){throw new Error("DataConnector#save method has not been implemented.")}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0})},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.RestorablePool=void 0;const r=n(17),i=n(102),o=n(16);var a;t.RestorablePool=class{constructor(e){this._added=new o.Signal(this),this._current=null,this._currentChanged=new o.Signal(this),this._hasRestored=!1,this._isDisposed=!1,this._objects=new Set,this._restore=null,this._restored=new r.PromiseDelegate,this._updated=new o.Signal(this),this.namespace=e.namespace}get added(){return this._added}get current(){return this._current}set current(e){this._current!==e&&null!==e&&this._objects.has(e)&&(this._current=e,this._currentChanged.emit(this._current))}get currentChanged(){return this._currentChanged}get isDisposed(){return this._isDisposed}get restored(){return this._restored.promise}get size(){return this._objects.size}get updated(){return this._updated}async add(e){var t,n;if(e.isDisposed){const t="A disposed object cannot be added.";throw console.warn(t,e),new Error(t)}if(this._objects.has(e)){const t="This object already exists in the pool.";throw console.warn(t,e),new Error(t)}if(this._objects.add(e),e.disposed.connect(this._onInstanceDisposed,this),!a.injectedProperty.get(e)){if(this._restore){const{connector:r}=this._restore,i=this._restore.name(e);if(i){const o=`${this.namespace}:${i}`,s=null===(n=(t=this._restore).args)||void 0===n?void 0:n.call(t,e);a.nameProperty.set(e,o),await r.save(o,{data:s})}}this._added.emit(e)}}dispose(){this.isDisposed||(this._current=null,this._isDisposed=!0,this._objects.clear(),o.Signal.clearData(this))}find(e){const t=this._objects.values();for(const n of t)if(e(n))return n}forEach(e){this._objects.forEach(e)}filter(e){const t=[];return this.forEach(n=>{e(n)&&t.push(n)}),t}inject(e){return a.injectedProperty.set(e,!0),this.add(e)}has(e){return this._objects.has(e)}async restore(e){if(this._hasRestored)throw new Error("This pool has already been restored.");this._hasRestored=!0;const{command:t,connector:n,registry:r,when:i}=e,o=this.namespace,a=i?[n.list(o)].concat(i):[n.list(o)];this._restore=e;const[s]=await Promise.all(a),l=await Promise.all(s.ids.map(async(e,i)=>{const o=s.values[i],a=o&&o.data;return void 0===a?n.remove(e):r.execute(t,a).catch(()=>n.remove(e))}));return this._restored.resolve(),l}async save(e){var t,n;const r=a.injectedProperty.get(e);if(!this._restore||!this.has(e)||r)return;const{connector:i}=this._restore,o=this._restore.name(e),s=a.nameProperty.get(e),l=o?`${this.namespace}:${o}`:"";if(s&&s!==l&&await i.remove(s),a.nameProperty.set(e,l),l){const r=null===(n=(t=this._restore).args)||void 0===n?void 0:n.call(t,e);await i.save(l,{data:r})}s!==l&&this._updated.emit(e)}_onInstanceDisposed(e){if(this._objects.delete(e),e===this._current&&(this._current=null,this._currentChanged.emit(this._current)),a.injectedProperty.get(e))return;if(!this._restore)return;const{connector:t}=this._restore,n=a.nameProperty.get(e);n&&t.remove(n)}},function(e){e.injectedProperty=new i.AttachedProperty({name:"injected",create:()=>!1}),e.nameProperty=new i.AttachedProperty({name:"name",create:()=>""})}(a||(a={}))},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.StateDB=void 0;const r=n(16);class i{constructor(e={}){this._changed=new r.Signal(this);const{connector:t,transform:n}=e;this._connector=t||new i.Connector,this._ready=n?n.then(e=>{const{contents:t,type:n}=e;switch(n){case"cancel":return;case"clear":return this._clear();case"merge":return this._merge(t||{});case"overwrite":return this._overwrite(t||{});default:return}}):Promise.resolve(void 0)}get changed(){return this._changed}async clear(){await this._ready,await this._clear()}async fetch(e){return await this._ready,this._fetch(e)}async list(e){return await this._ready,this._list(e)}async remove(e){await this._ready,await this._remove(e),this._changed.emit({id:e,type:"remove"})}async save(e,t){await this._ready,await this._save(e,t),this._changed.emit({id:e,type:"save"})}async toJSON(){await this._ready;const{ids:e,values:t}=await this._list();return t.reduce((t,n,r)=>(t[e[r]]=n,t),{})}async _clear(){await Promise.all((await this._list()).ids.map(e=>this._remove(e)))}async _fetch(e){const t=await this._connector.fetch(e);if(t)return JSON.parse(t).v}async _list(e=""){const{ids:t,values:n}=await this._connector.list(e);return{ids:t,values:n.map(e=>JSON.parse(e).v)}}async _merge(e){await Promise.all(Object.keys(e).map(t=>e[t]&&this._save(t,e[t])))}async _overwrite(e){await this._clear(),await this._merge(e)}async _remove(e){return this._connector.remove(e)}async _save(e,t){return this._connector.save(e,JSON.stringify({v:t}))}}t.StateDB=i,function(e){e.Connector=class{constructor(){this._storage={}}async fetch(e){return this._storage[e]}async list(e=""){return Object.keys(this._storage).reduce((t,n)=>(""!==e&&e!==n.split(":")[0]||(t.ids.push(n),t.values.push(this._storage[n])),t),{ids:[],values:[]})}async remove(e){delete this._storage[e]}async save(e,t){this._storage[e]=t}}}(i=t.StateDB||(t.StateDB={}))},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.IStateDB=void 0;const r=n(17);t.IStateDB=new r.Token("@jupyterlab/coreutils:IStateDB")},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.isAvailable=void 0;const r=n(355);Object.defineProperty(t,"isAvailable",{enumerable:!0,get:function(){return r.isAvailable}})},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.TerminalManager=void 0;const r=n(6),i=n(261),o=n(16),a=n(84),s=n(354),l=n(355),u=n(1380);class c extends s.BaseManager{constructor(e={}){var t;if(super(e),this._isReady=!1,this._names=[],this._terminalConnections=new Set,this._runningChanged=new o.Signal(this),this._connectionFailure=new o.Signal(this),!this.isAvailable())return this._ready=Promise.reject("Terminals unavailable"),void this._ready.catch(e=>{});this._pollModels=new i.Poll({auto:!1,factory:()=>this.requestRunning(),frequency:{interval:1e4,backoff:!0,max:3e5},name:"@jupyterlab/services:TerminalManager#models",standby:null!==(t=e.standby)&&void 0!==t?t:"when-hidden"}),this._ready=(async()=>{await this._pollModels.start(),await this._pollModels.tick,this._isReady=!0})()}get isReady(){return this._isReady}get ready(){return this._ready}get runningChanged(){return this._runningChanged}get connectionFailure(){return this._connectionFailure}dispose(){this.isDisposed||(this._names.length=0,this._terminalConnections.forEach(e=>e.dispose()),this._pollModels.dispose(),super.dispose())}isAvailable(){return l.isAvailable()}connectTo(e){const t=new u.TerminalConnection(Object.assign(Object.assign({},e),{serverSettings:this.serverSettings}));return this._onStarted(t),this._names.includes(e.model.name)||this.refreshRunning().catch(()=>{}),t}running(){return r.iter(this._models)}async refreshRunning(){await this._pollModels.refresh(),await this._pollModels.tick}async startNew(){const e=await l.startNew(this.serverSettings);return await this.refreshRunning(),this.connectTo({model:e})}async shutdown(e){await l.shutdownTerminal(e,this.serverSettings),await this.refreshRunning()}async shutdownAll(){await this.refreshRunning(),await Promise.all(this._names.map(e=>l.shutdownTerminal(e,this.serverSettings))),await this.refreshRunning()}async requestRunning(){var e;let t;try{t=await l.listRunning(this.serverSettings)}catch(t){throw(t instanceof a.ServerConnection.NetworkError||503===(null===(e=t.response)||void 0===e?void 0:e.status))&&this._connectionFailure.emit(t),t}if(this.isDisposed)return;const n=t.map(({name:e})=>e).sort();n!==this._names&&(this._names=n,this._terminalConnections.forEach(e=>{n.includes(e.name)||e.dispose()}),this._runningChanged.emit(this._models))}_onStarted(e){this._terminalConnections.add(e),e.disposed.connect(this._onDisposed,this)}_onDisposed(e){this._terminalConnections.delete(e),this.refreshRunning().catch(()=>{})}get _models(){return this._names.map(e=>({name:e}))}}t.TerminalManager=c},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.TerminalConnection=void 0;const r=n(48),i=n(17),o=n(16),a=n(84),s=n(355);class l{constructor(e){var t;this._createSocket=()=>{this._errorIfDisposed(),this._clearSocket(),this._updateConnectionStatus("connecting");const e=this._name,t=this.serverSettings;let n=r.URLExt.join(t.wsUrl,"terminals","websocket",encodeURIComponent(e));const i=t.token;t.appendToken&&""!==i&&(n=n+"?token="+encodeURIComponent(i)),this._ws=new t.WebSocket(n),this._ws.onmessage=this._onWSMessage,this._ws.onclose=this._onWSClose,this._ws.onerror=this._onWSClose},this._onWSMessage=e=>{if(this._isDisposed)return;const t=JSON.parse(e.data);"disconnect"===t[0]&&this.dispose(),"connecting"!==this._connectionStatus?this._messageReceived.emit({type:t[0],content:t.slice(1)}):"setup"===t[0]&&this._updateConnectionStatus("connected")},this._onWSClose=e=>{console.warn("Terminal websocket closed: "+e.code),this.isDisposed||this._reconnect()},this._connectionStatus="connecting",this._connectionStatusChanged=new o.Signal(this),this._isDisposed=!1,this._disposed=new o.Signal(this),this._messageReceived=new o.Signal(this),this._reconnectTimeout=null,this._ws=null,this._noOp=()=>{},this._reconnectLimit=7,this._reconnectAttempt=0,this._pendingMessages=[],this._name=e.model.name,this.serverSettings=null!==(t=e.serverSettings)&&void 0!==t?t:a.ServerConnection.makeSettings(),this._createSocket()}get disposed(){return this._disposed}get messageReceived(){return this._messageReceived}get name(){return this._name}get model(){return{name:this._name}}get isDisposed(){return this._isDisposed}dispose(){this._isDisposed||(this._isDisposed=!0,this._disposed.emit(),this._updateConnectionStatus("disconnected"),this._clearSocket(),o.Signal.clearData(this))}send(e){this._sendMessage(e)}_sendMessage(e,t=!0){if(!this._isDisposed&&e.content)if("connected"===this.connectionStatus&&this._ws){const t=[e.type,...e.content];this._ws.send(JSON.stringify(t))}else{if(!t)throw new Error("Could not send message: "+JSON.stringify(e));this._pendingMessages.push(e)}}_sendPending(){for(;"connected"===this.connectionStatus&&this._pendingMessages.length>0;)this._sendMessage(this._pendingMessages[0],!1),this._pendingMessages.shift()}reconnect(){this._errorIfDisposed();const e=new i.PromiseDelegate,t=(n,r)=>{"connected"===r?(e.resolve(),this.connectionStatusChanged.disconnect(t,this)):"disconnected"===r&&(e.reject(new Error("Terminal connection disconnected")),this.connectionStatusChanged.disconnect(t,this))};return this.connectionStatusChanged.connect(t,this),this._reconnectAttempt=0,this._reconnect(),e.promise}_reconnect(){if(this._errorIfDisposed(),clearTimeout(this._reconnectTimeout),this._reconnectAttempt<this._reconnectLimit){this._updateConnectionStatus("connecting");const e=u.getRandomIntInclusive(0,1e3*(Math.pow(2,this._reconnectAttempt)-1));console.error(`Connection lost, reconnecting in ${Math.floor(e/1e3)} seconds.`),this._reconnectTimeout=setTimeout(this._createSocket,e),this._reconnectAttempt+=1}else this._updateConnectionStatus("disconnected");this._clearSocket()}_clearSocket(){null!==this._ws&&(this._ws.onopen=this._noOp,this._ws.onclose=this._noOp,this._ws.onerror=this._noOp,this._ws.onmessage=this._noOp,this._ws.close(),this._ws=null)}async shutdown(){await s.shutdownTerminal(this.name,this.serverSettings),this.dispose()}clone(){return new l(this)}_updateConnectionStatus(e){this._connectionStatus!==e&&(this._connectionStatus=e,"connecting"!==e&&(this._reconnectAttempt=0,clearTimeout(this._reconnectTimeout)),"connected"===e&&this._sendPending(),this._connectionStatusChanged.emit(e))}_errorIfDisposed(){if(this.isDisposed)throw new Error("Terminal connection is disposed")}get connectionStatusChanged(){return this._connectionStatusChanged}get connectionStatus(){return this._connectionStatus}}var u;t.TerminalConnection=l,function(e){e.getTermUrl=function(e,t){return r.URLExt.join(e,s.TERMINAL_SERVICE_URL,encodeURIComponent(t))},e.getRandomIntInclusive=function(e,t){return e=Math.ceil(e),t=Math.floor(t),Math.floor(Math.random()*(t-e+1))+e}}(u||(u={}))},function(e,t,n){var r,i,o=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),a=this&&this.__setModuleDefault||(Object.create?function(e,t){Object.defineProperty(e,"default",{enumerable:!0,value:t})}:function(e,t){e.default=t}),s=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)"default"!==n&&Object.hasOwnProperty.call(e,n)&&o(t,e,n);return a(t,e),t},l=this&&this.__awaiter||function(e,t,n,r){return new(n||(n=Promise))((function(i,o){function a(e){try{l(r.next(e))}catch(e){o(e)}}function s(e){try{l(r.throw(e))}catch(e){o(e)}}function l(e){var t;e.done?i(e.value):(t=e.value,t instanceof n?t:new n((function(e){e(t)}))).then(a,s)}l((r=r.apply(e,t||[])).next())}))};r=[n,t,n(35),n(2346),n(1561),n(2339),n(2342),n(2345)],void 0===(i=function(e,t,n,r,i,o,a,u){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.WidgetManager=void 0,n=s(n);class c extends r.HTMLManager{constructor(e){super(),this.removeSplash=()=>{const e=document.getElementById("top_bar"),t=document.getElementById("cosapp_splash_screen");e.style.visibility="visible",t.classList.add("splash-fade"),t.addEventListener("transitionend",e=>{"transitionend"===e.type&&(t.style.display="none")},!1)},this.kernel=e,e.registerCommTarget(this.comm_target_name,(e,t)=>l(this,void 0,void 0,(function*(){const r=new n.shims.services.Comm(e);yield this.handle_comm_open(r,t)})))}loadClass(e,t,n){return"SysExplorerModel"===e?Promise.resolve(i.SysExplorerModel):"SysExplorerView"===e?Promise.resolve(o.SysExplorerAppView):"IpyWidgetRenderModel"===e?Promise.resolve(a.IpyWidgetRenderModel):"IpyWidgetRenderView"===e?Promise.resolve(a.IpyWidgetRenderView):"@jupyter-widgets/base"===t||"@jupyter-widgets/controls"===t?super.loadClass(e,t,n):u.remoteLoader(t,n).then(r=>r[e]?r[e]:Promise.reject(`Can not import ${t}@${n}`))}display_view(e,t,n){return t instanceof o.SysExplorerAppView?Promise.resolve(t).then(e=>(this.removeSplash(),e.on("remove",()=>{console.log("view removed",e)}),e)):Promise.resolve(t).then(e=>(e.on("remove",()=>{console.log("view removed",e)}),e))}_create_comm(e,t,r,i){return l(this,void 0,void 0,(function*(){const o=this.kernel.createComm(e,t);return(r||i)&&o.open(r,i),Promise.resolve(new n.shims.services.Comm(o))}))}_get_comm_info(){return this.kernel.requestCommInfo({target_name:this.comm_target_name}).then(e=>e.content.comms)}}t.WidgetManager=c}.apply(t,r))||(e.exports=i)},function(e,t,n){var r=n(356),i=n(852),o=n(1414),a=n(1416),s=n(238),l=n(81),u=n(237),c=n(283),d="[object Object]",h=Object.prototype.hasOwnProperty;e.exports=function(e,t,n,f,p,g){var v=l(e),m=l(t),y=v?"[object Array]":s(e),b=m?"[object Array]":s(t),C=(y="[object Arguments]"==y?d:y)==d,A=(b="[object Arguments]"==b?d:b)==d,_=y==b;if(_&&u(e)){if(!u(t))return!1;v=!0,C=!1}if(_&&!C)return g||(g=new r),v||c(e)?i(e,t,n,f,p,g):o(e,t,y,n,f,p,g);if(!(1&n)){var I=C&&h.call(e,"__wrapped__"),x=A&&h.call(t,"__wrapped__");if(I||x){var w=I?e.value():e,M=x?t.value():t;return g||(g=new r),p(w,M,n,f,g)}}return!!_&&(g||(g=new r),a(e,t,n,f,p,g))}},function(e,t){e.exports=function(){this.__data__=[],this.size=0}},function(e,t,n){var r=n(358),i=Array.prototype.splice;e.exports=function(e){var t=this.__data__,n=r(t,e);return!(n<0)&&(n==t.length-1?t.pop():i.call(t,n,1),--this.size,!0)}},function(e,t,n){var r=n(358);e.exports=function(e){var t=this.__data__,n=r(t,e);return n<0?void 0:t[n][1]}},function(e,t,n){var r=n(358);e.exports=function(e){return r(this.__data__,e)>-1}},function(e,t,n){var r=n(358);e.exports=function(e,t){var n=this.__data__,i=r(n,e);return i<0?(++this.size,n.push([e,t])):n[i][1]=t,this}},function(e,t,n){var r=n(357);e.exports=function(){this.__data__=new r,this.size=0}},function(e,t){e.exports=function(e){var t=this.__data__,n=t.delete(e);return this.size=t.size,n}},function(e,t){e.exports=function(e){return this.__data__.get(e)}},function(e,t){e.exports=function(e){return this.__data__.has(e)}},function(e,t,n){var r=n(357),i=n(492),o=n(493);e.exports=function(e,t){var n=this.__data__;if(n instanceof r){var a=n.__data__;if(!i||a.length<199)return a.push([e,t]),this.size=++n.size,this;n=this.__data__=new o(a)}return n.set(e,t),this.size=n.size,this}},function(e,t,n){var r=n(281),i=n(1396),o=n(130),a=n(851),s=/^\[object .+?Constructor\]$/,l=Function.prototype,u=Object.prototype,c=l.toString,d=u.hasOwnProperty,h=RegExp("^"+c.call(d).replace(/[\\^$.*+?()[\]{}|]/g,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$");e.exports=function(e){return!(!o(e)||i(e))&&(r(e)?h:s).test(a(e))}},function(e,t,n){var r=n(236),i=Object.prototype,o=i.hasOwnProperty,a=i.toString,s=r?r.toStringTag:void 0;e.exports=function(e){var t=o.call(e,s),n=e[s];try{e[s]=void 0;var r=!0}catch(e){}var i=a.call(e);return r&&(t?e[s]=n:delete e[s]),i}},function(e,t){var n=Object.prototype.toString;e.exports=function(e){return n.call(e)}},function(e,t,n){var r,i=n(1397),o=(r=/[^.]+$/.exec(i&&i.keys&&i.keys.IE_PROTO||""))?"Symbol(src)_1."+r:"";e.exports=function(e){return!!o&&o in e}},function(e,t,n){var r=n(146)["__core-js_shared__"];e.exports=r},function(e,t){e.exports=function(e,t){return null==e?void 0:e[t]}},function(e,t,n){var r=n(1400),i=n(357),o=n(492);e.exports=function(){this.size=0,this.__data__={hash:new r,map:new(o||i),string:new r}}},function(e,t,n){var r=n(1401),i=n(1402),o=n(1403),a=n(1404),s=n(1405);function l(e){var t=-1,n=null==e?0:e.length;for(this.clear();++t<n;){var r=e[t];this.set(r[0],r[1])}}l.prototype.clear=r,l.prototype.delete=i,l.prototype.get=o,l.prototype.has=a,l.prototype.set=s,e.exports=l},function(e,t,n){var r=n(359);e.exports=function(){this.__data__=r?r(null):{},this.size=0}},function(e,t){e.exports=function(e){var t=this.has(e)&&delete this.__data__[e];return this.size-=t?1:0,t}},function(e,t,n){var r=n(359),i=Object.prototype.hasOwnProperty;e.exports=function(e){var t=this.__data__;if(r){var n=t[e];return"__lodash_hash_undefined__"===n?void 0:n}return i.call(t,e)?t[e]:void 0}},function(e,t,n){var r=n(359),i=Object.prototype.hasOwnProperty;e.exports=function(e){var t=this.__data__;return r?void 0!==t[e]:i.call(t,e)}},function(e,t,n){var r=n(359);e.exports=function(e,t){var n=this.__data__;return this.size+=this.has(e)?0:1,n[e]=r&&void 0===t?"__lodash_hash_undefined__":t,this}},function(e,t,n){var r=n(360);e.exports=function(e){var t=r(this,e).delete(e);return this.size-=t?1:0,t}},function(e,t){e.exports=function(e){var t=typeof e;return"string"==t||"number"==t||"symbol"==t||"boolean"==t?"__proto__"!==e:null===e}},function(e,t,n){var r=n(360);e.exports=function(e){return r(this,e).get(e)}},function(e,t,n){var r=n(360);e.exports=function(e){return r(this,e).has(e)}},function(e,t,n){var r=n(360);e.exports=function(e,t){var n=r(this,e),i=n.size;return n.set(e,t),this.size+=n.size==i?0:1,this}},function(e,t){e.exports=function(e){return this.__data__.set(e,"__lodash_hash_undefined__"),this}},function(e,t){e.exports=function(e){return this.__data__.has(e)}},function(e,t){e.exports=function(e,t){for(var n=-1,r=null==e?0:e.length;++n<r;)if(t(e[n],n,e))return!0;return!1}},function(e,t,n){var r=n(236),i=n(855),o=n(235),a=n(852),s=n(1415),l=n(494),u=r?r.prototype:void 0,c=u?u.valueOf:void 0;e.exports=function(e,t,n,r,u,d,h){switch(n){case"[object DataView]":if(e.byteLength!=t.byteLength||e.byteOffset!=t.byteOffset)return!1;e=e.buffer,t=t.buffer;case"[object ArrayBuffer]":return!(e.byteLength!=t.byteLength||!d(new i(e),new i(t)));case"[object Boolean]":case"[object Date]":case"[object Number]":return o(+e,+t);case"[object Error]":return e.name==t.name&&e.message==t.message;case"[object RegExp]":case"[object String]":return e==t+"";case"[object Map]":var f=s;case"[object Set]":var p=1&r;if(f||(f=l),e.size!=t.size&&!p)return!1;var g=h.get(e);if(g)return g==t;r|=2,h.set(e,t);var v=a(f(e),f(t),r,u,d,h);return h.delete(e),v;case"[object Symbol]":if(c)return c.call(e)==c.call(t)}return!1}},function(e,t){e.exports=function(e){var t=-1,n=Array(e.size);return e.forEach((function(e,r){n[++t]=[r,e]})),n}},function(e,t,n){var r=n(856),i=Object.prototype.hasOwnProperty;e.exports=function(e,t,n,o,a,s){var l=1&n,u=r(e),c=u.length;if(c!=r(t).length&&!l)return!1;for(var d=c;d--;){var h=u[d];if(!(l?h in t:i.call(t,h)))return!1}var f=s.get(e);if(f&&s.get(t))return f==t;var p=!0;s.set(e,t),s.set(t,e);for(var g=l;++d<c;){var v=e[h=u[d]],m=t[h];if(o)var y=l?o(m,v,h,t,e,s):o(v,m,h,e,t,s);if(!(void 0===y?v===m||a(v,m,n,o,s):y)){p=!1;break}g||(g="constructor"==h)}if(p&&!g){var b=e.constructor,C=t.constructor;b==C||!("constructor"in e)||!("constructor"in t)||"function"==typeof b&&b instanceof b&&"function"==typeof C&&C instanceof C||(p=!1)}return s.delete(e),s.delete(t),p}},function(e,t){e.exports=function(e,t){for(var n=-1,r=Array(e);++n<e;)r[n]=t(n);return r}},function(e,t,n){var r=n(208),i=n(158);e.exports=function(e){return i(e)&&"[object Arguments]"==r(e)}},function(e,t){e.exports=function(){return!1}},function(e,t,n){var r=n(208),i=n(497),o=n(158),a={};a["[object Float32Array]"]=a["[object Float64Array]"]=a["[object Int8Array]"]=a["[object Int16Array]"]=a["[object Int32Array]"]=a["[object Uint8Array]"]=a["[object Uint8ClampedArray]"]=a["[object Uint16Array]"]=a["[object Uint32Array]"]=!0,a["[object Arguments]"]=a["[object Array]"]=a["[object ArrayBuffer]"]=a["[object Boolean]"]=a["[object DataView]"]=a["[object Date]"]=a["[object Error]"]=a["[object Function]"]=a["[object Map]"]=a["[object Number]"]=a["[object Object]"]=a["[object RegExp]"]=a["[object Set]"]=a["[object String]"]=a["[object WeakMap]"]=!1,e.exports=function(e){return o(e)&&i(e.length)&&!!a[r(e)]}},function(e,t,n){var r=n(861)(Object.keys,Object);e.exports=r},function(e,t,n){var r=n(207)(n(146),"DataView");e.exports=r},function(e,t,n){var r=n(207)(n(146),"Promise");e.exports=r},function(e,t,n){var r=n(207)(n(146),"WeakMap");e.exports=r},function(e,t,n){var r,i,o;
/*!
 * jQuery UI Slider 1.12.1
 * http://jqueryui.com
 *
 * Copyright jQuery Foundation and other contributors
 * Released under the MIT license.
 * http://jquery.org/license
 */i=[n(71),n(1426),n(1428),n(284),n(863)],void 0===(o="function"==typeof(r=function(e){return e.widget("ui.slider",e.ui.mouse,{version:"1.12.1",widgetEventPrefix:"slide",options:{animate:!1,classes:{"ui-slider":"ui-corner-all","ui-slider-handle":"ui-corner-all","ui-slider-range":"ui-corner-all ui-widget-header"},distance:0,max:100,min:0,orientation:"horizontal",range:!1,step:1,value:0,values:null,change:null,slide:null,start:null,stop:null},numPages:5,_create:function(){this._keySliding=!1,this._mouseSliding=!1,this._animateOff=!0,this._handleIndex=null,this._detectOrientation(),this._mouseInit(),this._calculateNewMax(),this._addClass("ui-slider ui-slider-"+this.orientation,"ui-widget ui-widget-content"),this._refresh(),this._animateOff=!1},_refresh:function(){this._createRange(),this._createHandles(),this._setupEvents(),this._refreshValue()},_createHandles:function(){var t,n,r=this.options,i=this.element.find(".ui-slider-handle"),o=[];for(n=r.values&&r.values.length||1,i.length>n&&(i.slice(n).remove(),i=i.slice(0,n)),t=i.length;t<n;t++)o.push("<span tabindex='0'></span>");this.handles=i.add(e(o.join("")).appendTo(this.element)),this._addClass(this.handles,"ui-slider-handle","ui-state-default"),this.handle=this.handles.eq(0),this.handles.each((function(t){e(this).data("ui-slider-handle-index",t).attr("tabIndex",0)}))},_createRange:function(){var t=this.options;t.range?(!0===t.range&&(t.values?t.values.length&&2!==t.values.length?t.values=[t.values[0],t.values[0]]:e.isArray(t.values)&&(t.values=t.values.slice(0)):t.values=[this._valueMin(),this._valueMin()]),this.range&&this.range.length?(this._removeClass(this.range,"ui-slider-range-min ui-slider-range-max"),this.range.css({left:"",bottom:""})):(this.range=e("<div>").appendTo(this.element),this._addClass(this.range,"ui-slider-range")),"min"!==t.range&&"max"!==t.range||this._addClass(this.range,"ui-slider-range-"+t.range)):(this.range&&this.range.remove(),this.range=null)},_setupEvents:function(){this._off(this.handles),this._on(this.handles,this._handleEvents),this._hoverable(this.handles),this._focusable(this.handles)},_destroy:function(){this.handles.remove(),this.range&&this.range.remove(),this._mouseDestroy()},_mouseCapture:function(t){var n,r,i,o,a,s,l,u=this,c=this.options;return!c.disabled&&(this.elementSize={width:this.element.outerWidth(),height:this.element.outerHeight()},this.elementOffset=this.element.offset(),n={x:t.pageX,y:t.pageY},r=this._normValueFromMouse(n),i=this._valueMax()-this._valueMin()+1,this.handles.each((function(t){var n=Math.abs(r-u.values(t));(i>n||i===n&&(t===u._lastChangedValue||u.values(t)===c.min))&&(i=n,o=e(this),a=t)})),!1!==this._start(t,a)&&(this._mouseSliding=!0,this._handleIndex=a,this._addClass(o,null,"ui-state-active"),o.trigger("focus"),s=o.offset(),l=!e(t.target).parents().addBack().is(".ui-slider-handle"),this._clickOffset=l?{left:0,top:0}:{left:t.pageX-s.left-o.width()/2,top:t.pageY-s.top-o.height()/2-(parseInt(o.css("borderTopWidth"),10)||0)-(parseInt(o.css("borderBottomWidth"),10)||0)+(parseInt(o.css("marginTop"),10)||0)},this.handles.hasClass("ui-state-hover")||this._slide(t,a,r),this._animateOff=!0,!0))},_mouseStart:function(){return!0},_mouseDrag:function(e){var t={x:e.pageX,y:e.pageY},n=this._normValueFromMouse(t);return this._slide(e,this._handleIndex,n),!1},_mouseStop:function(e){return this._removeClass(this.handles,null,"ui-state-active"),this._mouseSliding=!1,this._stop(e,this._handleIndex),this._change(e,this._handleIndex),this._handleIndex=null,this._clickOffset=null,this._animateOff=!1,!1},_detectOrientation:function(){this.orientation="vertical"===this.options.orientation?"vertical":"horizontal"},_normValueFromMouse:function(e){var t,n,r,i,o;return"horizontal"===this.orientation?(t=this.elementSize.width,n=e.x-this.elementOffset.left-(this._clickOffset?this._clickOffset.left:0)):(t=this.elementSize.height,n=e.y-this.elementOffset.top-(this._clickOffset?this._clickOffset.top:0)),(r=n/t)>1&&(r=1),r<0&&(r=0),"vertical"===this.orientation&&(r=1-r),i=this._valueMax()-this._valueMin(),o=this._valueMin()+r*i,this._trimAlignValue(o)},_uiHash:function(e,t,n){var r={handle:this.handles[e],handleIndex:e,value:void 0!==t?t:this.value()};return this._hasMultipleValues()&&(r.value=void 0!==t?t:this.values(e),r.values=n||this.values()),r},_hasMultipleValues:function(){return this.options.values&&this.options.values.length},_start:function(e,t){return this._trigger("start",e,this._uiHash(t))},_slide:function(e,t,n){var r,i=this.value(),o=this.values();this._hasMultipleValues()&&(r=this.values(t?0:1),i=this.values(t),2===this.options.values.length&&!0===this.options.range&&(n=0===t?Math.min(r,n):Math.max(r,n)),o[t]=n),n!==i&&!1!==this._trigger("slide",e,this._uiHash(t,n,o))&&(this._hasMultipleValues()?this.values(t,n):this.value(n))},_stop:function(e,t){this._trigger("stop",e,this._uiHash(t))},_change:function(e,t){this._keySliding||this._mouseSliding||(this._lastChangedValue=t,this._trigger("change",e,this._uiHash(t)))},value:function(e){return arguments.length?(this.options.value=this._trimAlignValue(e),this._refreshValue(),void this._change(null,0)):this._value()},values:function(t,n){var r,i,o;if(arguments.length>1)return this.options.values[t]=this._trimAlignValue(n),this._refreshValue(),void this._change(null,t);if(!arguments.length)return this._values();if(!e.isArray(arguments[0]))return this._hasMultipleValues()?this._values(t):this.value();for(r=this.options.values,i=arguments[0],o=0;o<r.length;o+=1)r[o]=this._trimAlignValue(i[o]),this._change(null,o);this._refreshValue()},_setOption:function(t,n){var r,i=0;switch("range"===t&&!0===this.options.range&&("min"===n?(this.options.value=this._values(0),this.options.values=null):"max"===n&&(this.options.value=this._values(this.options.values.length-1),this.options.values=null)),e.isArray(this.options.values)&&(i=this.options.values.length),this._super(t,n),t){case"orientation":this._detectOrientation(),this._removeClass("ui-slider-horizontal ui-slider-vertical")._addClass("ui-slider-"+this.orientation),this._refreshValue(),this.options.range&&this._refreshRange(n),this.handles.css("horizontal"===n?"bottom":"left","");break;case"value":this._animateOff=!0,this._refreshValue(),this._change(null,0),this._animateOff=!1;break;case"values":for(this._animateOff=!0,this._refreshValue(),r=i-1;r>=0;r--)this._change(null,r);this._animateOff=!1;break;case"step":case"min":case"max":this._animateOff=!0,this._calculateNewMax(),this._refreshValue(),this._animateOff=!1;break;case"range":this._animateOff=!0,this._refresh(),this._animateOff=!1}},_setOptionDisabled:function(e){this._super(e),this._toggleClass(null,"ui-state-disabled",!!e)},_value:function(){var e=this.options.value;return e=this._trimAlignValue(e)},_values:function(e){var t,n,r;if(arguments.length)return t=this.options.values[e],t=this._trimAlignValue(t);if(this._hasMultipleValues()){for(n=this.options.values.slice(),r=0;r<n.length;r+=1)n[r]=this._trimAlignValue(n[r]);return n}return[]},_trimAlignValue:function(e){if(e<=this._valueMin())return this._valueMin();if(e>=this._valueMax())return this._valueMax();var t=this.options.step>0?this.options.step:1,n=(e-this._valueMin())%t,r=e-n;return 2*Math.abs(n)>=t&&(r+=n>0?t:-t),parseFloat(r.toFixed(5))},_calculateNewMax:function(){var e=this.options.max,t=this._valueMin(),n=this.options.step;(e=Math.round((e-t)/n)*n+t)>this.options.max&&(e-=n),this.max=parseFloat(e.toFixed(this._precision()))},_precision:function(){var e=this._precisionOf(this.options.step);return null!==this.options.min&&(e=Math.max(e,this._precisionOf(this.options.min))),e},_precisionOf:function(e){var t=e.toString(),n=t.indexOf(".");return-1===n?0:t.length-n-1},_valueMin:function(){return this.options.min},_valueMax:function(){return this.max},_refreshRange:function(e){"vertical"===e&&this.range.css({width:"",left:""}),"horizontal"===e&&this.range.css({height:"",bottom:""})},_refreshValue:function(){var t,n,r,i,o,a=this.options.range,s=this.options,l=this,u=!this._animateOff&&s.animate,c={};this._hasMultipleValues()?this.handles.each((function(r){n=(l.values(r)-l._valueMin())/(l._valueMax()-l._valueMin())*100,c["horizontal"===l.orientation?"left":"bottom"]=n+"%",e(this).stop(1,1)[u?"animate":"css"](c,s.animate),!0===l.options.range&&("horizontal"===l.orientation?(0===r&&l.range.stop(1,1)[u?"animate":"css"]({left:n+"%"},s.animate),1===r&&l.range[u?"animate":"css"]({width:n-t+"%"},{queue:!1,duration:s.animate})):(0===r&&l.range.stop(1,1)[u?"animate":"css"]({bottom:n+"%"},s.animate),1===r&&l.range[u?"animate":"css"]({height:n-t+"%"},{queue:!1,duration:s.animate}))),t=n})):(r=this.value(),i=this._valueMin(),o=this._valueMax(),n=o!==i?(r-i)/(o-i)*100:0,c["horizontal"===this.orientation?"left":"bottom"]=n+"%",this.handle.stop(1,1)[u?"animate":"css"](c,s.animate),"min"===a&&"horizontal"===this.orientation&&this.range.stop(1,1)[u?"animate":"css"]({width:n+"%"},s.animate),"max"===a&&"horizontal"===this.orientation&&this.range.stop(1,1)[u?"animate":"css"]({width:100-n+"%"},s.animate),"min"===a&&"vertical"===this.orientation&&this.range.stop(1,1)[u?"animate":"css"]({height:n+"%"},s.animate),"max"===a&&"vertical"===this.orientation&&this.range.stop(1,1)[u?"animate":"css"]({height:100-n+"%"},s.animate))},_handleEvents:{keydown:function(t){var n,r,i,o=e(t.target).data("ui-slider-handle-index");switch(t.keyCode){case e.ui.keyCode.HOME:case e.ui.keyCode.END:case e.ui.keyCode.PAGE_UP:case e.ui.keyCode.PAGE_DOWN:case e.ui.keyCode.UP:case e.ui.keyCode.RIGHT:case e.ui.keyCode.DOWN:case e.ui.keyCode.LEFT:if(t.preventDefault(),!this._keySliding&&(this._keySliding=!0,this._addClass(e(t.target),null,"ui-state-active"),!1===this._start(t,o)))return}switch(i=this.options.step,n=r=this._hasMultipleValues()?this.values(o):this.value(),t.keyCode){case e.ui.keyCode.HOME:r=this._valueMin();break;case e.ui.keyCode.END:r=this._valueMax();break;case e.ui.keyCode.PAGE_UP:r=this._trimAlignValue(n+(this._valueMax()-this._valueMin())/this.numPages);break;case e.ui.keyCode.PAGE_DOWN:r=this._trimAlignValue(n-(this._valueMax()-this._valueMin())/this.numPages);break;case e.ui.keyCode.UP:case e.ui.keyCode.RIGHT:if(n===this._valueMax())return;r=this._trimAlignValue(n+i);break;case e.ui.keyCode.DOWN:case e.ui.keyCode.LEFT:if(n===this._valueMin())return;r=this._trimAlignValue(n-i)}this._slide(t,o,r)},keyup:function(t){var n=e(t.target).data("ui-slider-handle-index");this._keySliding&&(this._keySliding=!1,this._stop(t,n),this._change(t,n),this._removeClass(e(t.target),null,"ui-state-active"))}}})})?r.apply(t,i):r)||(e.exports=o)},function(e,t,n){var r,i,o;
/*!
 * jQuery UI Mouse 1.12.1
 * http://jqueryui.com
 *
 * Copyright jQuery Foundation and other contributors
 * Released under the MIT license.
 * http://jquery.org/license
 */i=[n(71),n(1427),n(284),n(863)],void 0===(o="function"==typeof(r=function(e){var t=!1;return e(document).on("mouseup",(function(){t=!1})),e.widget("ui.mouse",{version:"1.12.1",options:{cancel:"input, textarea, button, select, option",distance:1,delay:0},_mouseInit:function(){var t=this;this.element.on("mousedown."+this.widgetName,(function(e){return t._mouseDown(e)})).on("click."+this.widgetName,(function(n){if(!0===e.data(n.target,t.widgetName+".preventClickEvent"))return e.removeData(n.target,t.widgetName+".preventClickEvent"),n.stopImmediatePropagation(),!1})),this.started=!1},_mouseDestroy:function(){this.element.off("."+this.widgetName),this._mouseMoveDelegate&&this.document.off("mousemove."+this.widgetName,this._mouseMoveDelegate).off("mouseup."+this.widgetName,this._mouseUpDelegate)},_mouseDown:function(n){if(!t){this._mouseMoved=!1,this._mouseStarted&&this._mouseUp(n),this._mouseDownEvent=n;var r=this,i=1===n.which,o=!("string"!=typeof this.options.cancel||!n.target.nodeName)&&e(n.target).closest(this.options.cancel).length;return!(i&&!o&&this._mouseCapture(n)&&(this.mouseDelayMet=!this.options.delay,this.mouseDelayMet||(this._mouseDelayTimer=setTimeout((function(){r.mouseDelayMet=!0}),this.options.delay)),this._mouseDistanceMet(n)&&this._mouseDelayMet(n)&&(this._mouseStarted=!1!==this._mouseStart(n),!this._mouseStarted)?(n.preventDefault(),0):(!0===e.data(n.target,this.widgetName+".preventClickEvent")&&e.removeData(n.target,this.widgetName+".preventClickEvent"),this._mouseMoveDelegate=function(e){return r._mouseMove(e)},this._mouseUpDelegate=function(e){return r._mouseUp(e)},this.document.on("mousemove."+this.widgetName,this._mouseMoveDelegate).on("mouseup."+this.widgetName,this._mouseUpDelegate),n.preventDefault(),t=!0,0)))}},_mouseMove:function(t){if(this._mouseMoved){if(e.ui.ie&&(!document.documentMode||document.documentMode<9)&&!t.button)return this._mouseUp(t);if(!t.which)if(t.originalEvent.altKey||t.originalEvent.ctrlKey||t.originalEvent.metaKey||t.originalEvent.shiftKey)this.ignoreMissingWhich=!0;else if(!this.ignoreMissingWhich)return this._mouseUp(t)}return(t.which||t.button)&&(this._mouseMoved=!0),this._mouseStarted?(this._mouseDrag(t),t.preventDefault()):(this._mouseDistanceMet(t)&&this._mouseDelayMet(t)&&(this._mouseStarted=!1!==this._mouseStart(this._mouseDownEvent,t),this._mouseStarted?this._mouseDrag(t):this._mouseUp(t)),!this._mouseStarted)},_mouseUp:function(n){this.document.off("mousemove."+this.widgetName,this._mouseMoveDelegate).off("mouseup."+this.widgetName,this._mouseUpDelegate),this._mouseStarted&&(this._mouseStarted=!1,n.target===this._mouseDownEvent.target&&e.data(n.target,this.widgetName+".preventClickEvent",!0),this._mouseStop(n)),this._mouseDelayTimer&&(clearTimeout(this._mouseDelayTimer),delete this._mouseDelayTimer),this.ignoreMissingWhich=!1,t=!1,n.preventDefault()},_mouseDistanceMet:function(e){return Math.max(Math.abs(this._mouseDownEvent.pageX-e.pageX),Math.abs(this._mouseDownEvent.pageY-e.pageY))>=this.options.distance},_mouseDelayMet:function(){return this.mouseDelayMet},_mouseStart:function(){},_mouseDrag:function(){},_mouseStop:function(){},_mouseCapture:function(){return!0}})})?r.apply(t,i):r)||(e.exports=o)},function(e,t,n){var r,i,o;i=[n(71),n(284)],void 0===(o="function"==typeof(r=function(e){return e.ui.ie=!!/msie [\w.]+/.exec(navigator.userAgent.toLowerCase())})?r.apply(t,i):r)||(e.exports=o)},function(e,t,n){var r,i,o;
/*!
 * jQuery UI Keycode 1.12.1
 * http://jqueryui.com
 *
 * Copyright jQuery Foundation and other contributors
 * Released under the MIT license.
 * http://jquery.org/license
 */i=[n(71),n(284)],void 0===(o="function"==typeof(r=function(e){return e.ui.keyCode={BACKSPACE:8,COMMA:188,DELETE:46,DOWN:40,END:35,ENTER:13,ESCAPE:27,HOME:36,LEFT:37,PAGE_DOWN:34,PAGE_UP:33,PERIOD:190,RIGHT:39,SPACE:32,TAB:9,UP:38}})?r.apply(t,i):r)||(e.exports=o)},function(e){e.exports=JSON.parse('{"name":"@jupyter-widgets/controls","version":"3.0.0","description":"Jupyter interactive widgets","repository":{"type":"git","url":"https://github.com/jupyter-widgets/ipywidgets.git"},"license":"BSD-3-Clause","author":"Project Jupyter","main":"lib/index.js","typings":"lib/index.d.ts","files":["lib/**/*.d.ts","lib/**/*.js","css/*.css","dist/"],"scripts":{"build":"npm run build:src && npm run build:css","build:css":"postcss --use postcss-import --use postcss-cssnext -o css/widgets.built.css css/widgets.css","build:src":"tsc","build:test":"tsc --project test && webpack --config test/webpack.conf.js","clean":"npm run clean:src","clean:src":"rimraf lib && rimraf tsconfig.tsbuildinfo","lint":"tslint --project tslint.json --format stylish","prepublish":"npm run clean && npm run build","test":"npm run test:unit","test:coverage":"npm run build:test && webpack --config test/webpack-cov.conf.js && karma start test/karma-cov.conf.js","test:unit":"npm run test:unit:firefox && npm run test:unit:chrome","test:unit:chrome":"npm run test:unit:default -- --browsers=Chrome","test:unit:default":"npm run build:test && karma start test/karma.conf.js --log-level debug","test:unit:firefox":"npm run test:unit:default -- --browsers=Firefox","test:unit:ie":"npm run test:unit:default -- --browsers=IE"},"dependencies":{"@jupyter-widgets/base":"^4.0.0","@lumino/algorithm":"^1.1.0","@lumino/domutils":"^1.1.0","@lumino/messaging":"^1.2.1","@lumino/signaling":"^1.2.0","@lumino/widgets":"^1.3.0","d3-format":"^1.3.0","jquery":"^3.1.1","jquery-ui":"^1.12.1","underscore":"^1.8.3"},"devDependencies":{"@jupyterlab/services":"^6.0.0","@types/d3-format":"^1.3.1","@types/expect.js":"^0.3.29","@types/mathjax":"^0.0.35","@types/mocha":"^5.2.7","@types/node":"^12.0.10","chai":"^4.0.0","css-loader":"^3.4.0","expect.js":"^0.3.1","file-loader":"^5.0.2","istanbul-instrumenter-loader":"^3.0.1","karma":"^4.1.0","karma-chrome-launcher":"^2.2.0","karma-coverage":"^1.1.2","karma-firefox-launcher":"^1.1.0","karma-ie-launcher":"^1.0.0","karma-mocha":"^1.3.0","karma-mocha-reporter":"^2.2.5","karma-webpack":"^4.0.2","mocha":"^6.1.4","npm-run-all":"^4.1.5","postcss-cli":"^6.1.2","postcss-cssnext":"^3.1.0","postcss-import":"^12.0.1","postcss-loader":"^3.0.0","rimraf":"^2.6.1","sinon":"^7.3.2","sinon-chai":"^3.3.0","style-loader":"^1.1.2","tslint":"^5.20.1","typescript":"~3.7.4","url-loader":"^3.0.0","webpack":"^4.41.5"},"gitHead":"8ec6746db708fc084102cabe34f706c204098009"}')},function(e,t,n){var r=n(1431);"string"==typeof r&&(r=[[e.i,r,""]]);var i={insert:"head",singleton:!1};n(147)(r,i);r.locals&&(e.exports=r.locals)},function(e,t,n){(e.exports=n(68)(!1)).push([e.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n\n.jupyter-widgets.widget-output .jp-OutputArea-prompt {\n    display: none;\n}\n",""])},function(e,t,n){!function(e){"use strict";e.runMode=function(t,n,r,i){var o=e.getMode(e.defaults,n),a=i&&i.tabSize||e.defaults.tabSize;if(r.appendChild){var s=/MSIE \d/.test(navigator.userAgent)&&(null==document.documentMode||document.documentMode<9),l=r,u=0;l.innerHTML="",r=function(e,t){if("\n"==e)return l.appendChild(document.createTextNode(s?"\r":e)),void(u=0);for(var n="",r=0;;){var i=e.indexOf("\t",r);if(-1==i){n+=e.slice(r),u+=e.length-r;break}u+=i-r,n+=e.slice(r,i);var o=a-u%a;u+=o;for(var c=0;c<o;++c)n+=" ";r=i+1}if(t){var d=l.appendChild(document.createElement("span"));d.className="cm-"+t.replace(/ +/g," cm-"),d.appendChild(document.createTextNode(n))}else l.appendChild(document.createTextNode(n))}}for(var c=e.splitLines(t),d=i&&i.state||e.startState(o),h=0,f=c.length;h<f;++h){h&&r("\n");var p=new e.StringStream(c[h],null,{lookAhead:function(e){return c[h+e]},baseToken:function(){}});for(!p.string&&o.blankLine&&o.blankLine(d);!p.eol();){var g=o.token(p,d);r(p.current(),g,h,p.start,d),p.start=p.pos}}}}(n(24))},function(e,t,n){!function(e){"use strict";var t={},n=/[^\s\u00a0]/,r=e.Pos,i=e.cmpPos;function o(e){var t=e.search(n);return-1==t?0:t}function a(e,t){var n=e.getMode();return!1!==n.useInnerComments&&n.innerMode?e.getModeAt(t):n}e.commands.toggleComment=function(e){e.toggleComment()},e.defineExtension("toggleComment",(function(e){e||(e=t);for(var n=1/0,i=this.listSelections(),o=null,a=i.length-1;a>=0;a--){var s=i[a].from(),l=i[a].to();s.line>=n||(l.line>=n&&(l=r(n,0)),n=s.line,null==o?this.uncomment(s,l,e)?o="un":(this.lineComment(s,l,e),o="line"):"un"==o?this.uncomment(s,l,e):this.lineComment(s,l,e))}})),e.defineExtension("lineComment",(function(e,i,s){s||(s=t);var l=this,u=a(l,e),c=l.getLine(e.line);if(null!=c&&(d=e,h=c,!/\bstring\b/.test(l.getTokenTypeAt(r(d.line,0)))||/^[\'\"\`]/.test(h))){var d,h,f=s.lineComment||u.lineComment;if(f){var p=Math.min(0!=i.ch||i.line==e.line?i.line+1:i.line,l.lastLine()+1),g=null==s.padding?" ":s.padding,v=s.commentBlankLines||e.line==i.line;l.operation((function(){if(s.indent){for(var t=null,i=e.line;i<p;++i){var a=(u=l.getLine(i)).slice(0,o(u));(null==t||t.length>a.length)&&(t=a)}for(i=e.line;i<p;++i){var u=l.getLine(i),c=t.length;(v||n.test(u))&&(u.slice(0,c)!=t&&(c=o(u)),l.replaceRange(t+f+g,r(i,0),r(i,c)))}}else for(i=e.line;i<p;++i)(v||n.test(l.getLine(i)))&&l.replaceRange(f+g,r(i,0))}))}else(s.blockCommentStart||u.blockCommentStart)&&(s.fullLines=!0,l.blockComment(e,i,s))}})),e.defineExtension("blockComment",(function(e,o,s){s||(s=t);var l=this,u=a(l,e),c=s.blockCommentStart||u.blockCommentStart,d=s.blockCommentEnd||u.blockCommentEnd;if(c&&d){if(!/\bcomment\b/.test(l.getTokenTypeAt(r(e.line,0)))){var h=Math.min(o.line,l.lastLine());h!=e.line&&0==o.ch&&n.test(l.getLine(h))&&--h;var f=null==s.padding?" ":s.padding;e.line>h||l.operation((function(){if(0!=s.fullLines){var t=n.test(l.getLine(h));l.replaceRange(f+d,r(h)),l.replaceRange(c+f,r(e.line,0));var a=s.blockCommentLead||u.blockCommentLead;if(null!=a)for(var p=e.line+1;p<=h;++p)(p!=h||t)&&l.replaceRange(a+f,r(p,0))}else{var g=0==i(l.getCursor("to"),o),v=!l.somethingSelected();l.replaceRange(d,o),g&&l.setSelection(v?o:l.getCursor("from"),o),l.replaceRange(c,e)}}))}}else(s.lineComment||u.lineComment)&&0!=s.fullLines&&l.lineComment(e,o,s)})),e.defineExtension("uncomment",(function(e,i,o){o||(o=t);var s,l=this,u=a(l,e),c=Math.min(0!=i.ch||i.line==e.line?i.line:i.line-1,l.lastLine()),d=Math.min(e.line,c),h=o.lineComment||u.lineComment,f=[],p=null==o.padding?" ":o.padding;e:if(h){for(var g=d;g<=c;++g){var v=l.getLine(g),m=v.indexOf(h);if(m>-1&&!/comment/.test(l.getTokenTypeAt(r(g,m+1)))&&(m=-1),-1==m&&n.test(v))break e;if(m>-1&&n.test(v.slice(0,m)))break e;f.push(v)}if(l.operation((function(){for(var e=d;e<=c;++e){var t=f[e-d],n=t.indexOf(h),i=n+h.length;n<0||(t.slice(i,i+p.length)==p&&(i+=p.length),s=!0,l.replaceRange("",r(e,n),r(e,i)))}})),s)return!0}var y=o.blockCommentStart||u.blockCommentStart,b=o.blockCommentEnd||u.blockCommentEnd;if(!y||!b)return!1;var C=o.blockCommentLead||u.blockCommentLead,A=l.getLine(d),_=A.indexOf(y);if(-1==_)return!1;var I=c==d?A:l.getLine(c),x=I.indexOf(b,c==d?_+y.length:0),w=r(d,_+1),M=r(c,x+1);if(-1==x||!/comment/.test(l.getTokenTypeAt(w))||!/comment/.test(l.getTokenTypeAt(M))||l.getRange(w,M,"\n").indexOf(b)>-1)return!1;var S=A.lastIndexOf(y,e.ch),T=-1==S?-1:A.slice(0,e.ch).indexOf(b,S+y.length);if(-1!=S&&-1!=T&&T+b.length!=e.ch)return!1;T=I.indexOf(b,i.ch);var E=I.slice(i.ch).lastIndexOf(y,T-i.ch);return S=-1==T||-1==E?-1:i.ch+E,(-1==T||-1==S||S==i.ch)&&(l.operation((function(){l.replaceRange("",r(c,x-(p&&I.slice(x-p.length,x)==p?p.length:0)),r(c,x+b.length));var e=_+y.length;if(p&&A.slice(e,e+p.length)==p&&(e+=p.length),l.replaceRange("",r(d,_),r(d,e)),C)for(var t=d+1;t<=c;++t){var i=l.getLine(t),o=i.indexOf(C);if(-1!=o&&!n.test(i.slice(0,o))){var a=o+C.length;p&&i.slice(a,a+p.length)==p&&(a+=p.length),l.replaceRange("",r(t,o),r(t,a))}}})),!0)}))}(n(24))},function(e,t,n){!function(e){"use strict";function t(t){t.state.rulerDiv.textContent="";var n=t.getOption("rulers"),r=t.defaultCharWidth(),i=t.charCoords(e.Pos(t.firstLine(),0),"div").left;t.state.rulerDiv.style.minHeight=t.display.scroller.offsetHeight+30+"px";for(var o=0;o<n.length;o++){var a=document.createElement("div");a.className="CodeMirror-ruler";var s,l=n[o];"number"==typeof l?s=l:(s=l.column,l.className&&(a.className+=" "+l.className),l.color&&(a.style.borderColor=l.color),l.lineStyle&&(a.style.borderLeftStyle=l.lineStyle),l.width&&(a.style.borderLeftWidth=l.width)),a.style.left=i+s*r+"px",t.state.rulerDiv.appendChild(a)}}e.defineOption("rulers",!1,(function(e,n){e.state.rulerDiv&&(e.state.rulerDiv.parentElement.removeChild(e.state.rulerDiv),e.state.rulerDiv=null,e.off("refresh",t)),n&&n.length&&(e.state.rulerDiv=e.display.lineSpace.parentElement.insertBefore(document.createElement("div"),e.display.lineSpace),e.state.rulerDiv.className="CodeMirror-rulers",t(e),e.on("refresh",t))}))}(n(24))},function(e,t,n){!function(e){var t={pairs:"()[]{}''\"\"",closeBefore:")]}'\":;>",triples:"",explode:"[]{}"},n=e.Pos;function r(e,n){return"pairs"==n&&"string"==typeof e?e:"object"==typeof e&&null!=e[n]?e[n]:t[n]}e.defineOption("autoCloseBrackets",!1,(function(t,n,a){a&&a!=e.Init&&(t.removeKeyMap(i),t.state.closeBrackets=null),n&&(o(r(n,"pairs")),t.state.closeBrackets=n,t.addKeyMap(i))}));var i={Backspace:function(t){var i=s(t);if(!i||t.getOption("disableInput"))return e.Pass;for(var o=r(i,"pairs"),a=t.listSelections(),u=0;u<a.length;u++){if(!a[u].empty())return e.Pass;var c=l(t,a[u].head);if(!c||o.indexOf(c)%2!=0)return e.Pass}for(u=a.length-1;u>=0;u--){var d=a[u].head;t.replaceRange("",n(d.line,d.ch-1),n(d.line,d.ch+1),"+delete")}},Enter:function(t){var n=s(t),i=n&&r(n,"explode");if(!i||t.getOption("disableInput"))return e.Pass;for(var o=t.listSelections(),a=0;a<o.length;a++){if(!o[a].empty())return e.Pass;var u=l(t,o[a].head);if(!u||i.indexOf(u)%2!=0)return e.Pass}t.operation((function(){var e=t.lineSeparator()||"\n";t.replaceSelection(e+e,null),t.execCommand("goCharLeft"),o=t.listSelections();for(var n=0;n<o.length;n++){var r=o[n].head.line;t.indentLine(r,null,!0),t.indentLine(r+1,null,!0)}}))}};function o(e){for(var t=0;t<e.length;t++){var n=e.charAt(t),r="'"+n+"'";i[r]||(i[r]=a(n))}}function a(t){return function(i){return function(t,i){var o=s(t);if(!o||t.getOption("disableInput"))return e.Pass;var a=r(o,"pairs"),l=a.indexOf(i);if(-1==l)return e.Pass;for(var c,d=r(o,"closeBefore"),h=r(o,"triples"),f=a.charAt(l+1)==i,p=t.listSelections(),g=l%2==0,v=0;v<p.length;v++){var m,y=p[v],b=y.head,C=t.getRange(b,n(b.line,b.ch+1));if(g&&!y.empty())m="surround";else if(!f&&g||C!=i)if(f&&b.ch>1&&h.indexOf(i)>=0&&t.getRange(n(b.line,b.ch-2),b)==i+i){if(b.ch>2&&/\bstring/.test(t.getTokenTypeAt(n(b.line,b.ch-2))))return e.Pass;m="addFour"}else if(f){var A=0==b.ch?" ":t.getRange(n(b.line,b.ch-1),b);if(e.isWordChar(C)||A==i||e.isWordChar(A))return e.Pass;m="both"}else{if(!g||!(0===C.length||/\s/.test(C)||d.indexOf(C)>-1))return e.Pass;m="both"}else m=f&&u(t,b)?"both":h.indexOf(i)>=0&&t.getRange(b,n(b.line,b.ch+3))==i+i+i?"skipThree":"skip";if(c){if(c!=m)return e.Pass}else c=m}var _=l%2?a.charAt(l-1):i,I=l%2?i:a.charAt(l+1);t.operation((function(){if("skip"==c)t.execCommand("goCharRight");else if("skipThree"==c)for(var r=0;r<3;r++)t.execCommand("goCharRight");else if("surround"==c){var i=t.getSelections();for(r=0;r<i.length;r++)i[r]=_+i[r]+I;for(t.replaceSelections(i,"around"),i=t.listSelections().slice(),r=0;r<i.length;r++)i[r]=(o=i[r],a=void 0,a=e.cmpPos(o.anchor,o.head)>0,{anchor:new n(o.anchor.line,o.anchor.ch+(a?-1:1)),head:new n(o.head.line,o.head.ch+(a?1:-1))});t.setSelections(i)}else"both"==c?(t.replaceSelection(_+I,null),t.triggerElectric(_+I),t.execCommand("goCharLeft")):"addFour"==c&&(t.replaceSelection(_+_+_+_,"before"),t.execCommand("goCharRight"));var o,a}))}(i,t)}}function s(e){var t=e.state.closeBrackets;return!t||t.override?t:e.getModeAt(e.getCursor()).closeBrackets||t}function l(e,t){var r=e.getRange(n(t.line,t.ch-1),n(t.line,t.ch+1));return 2==r.length?r:null}function u(e,t){var r=e.getTokenAt(n(t.line,t.ch+1));return/\bstring/.test(r.type)&&r.start==t.ch&&(0==t.ch||!/\bstring/.test(e.getTokenTypeAt(t)))}o(t.pairs+"`")}(n(24))},function(e,t,n){!function(e){"use strict";e.defineOption("foldGutter",!1,(function(t,r,i){var o;i&&i!=e.Init&&(t.clearGutter(t.state.foldGutter.options.gutter),t.state.foldGutter=null,t.off("gutterClick",l),t.off("changes",u),t.off("viewportChange",c),t.off("fold",d),t.off("unfold",d),t.off("swapDoc",u)),r&&(t.state.foldGutter=new n((!0===(o=r)&&(o={}),null==o.gutter&&(o.gutter="CodeMirror-foldgutter"),null==o.indicatorOpen&&(o.indicatorOpen="CodeMirror-foldgutter-open"),null==o.indicatorFolded&&(o.indicatorFolded="CodeMirror-foldgutter-folded"),o)),s(t),t.on("gutterClick",l),t.on("changes",u),t.on("viewportChange",c),t.on("fold",d),t.on("unfold",d),t.on("swapDoc",u))}));var t=e.Pos;function n(e){this.options=e,this.from=this.to=0}function r(e,n){for(var r=e.findMarks(t(n,0),t(n+1,0)),i=0;i<r.length;++i)if(r[i].__isFold){var o=r[i].find(-1);if(o&&o.line===n)return r[i]}}function i(e){if("string"==typeof e){var t=document.createElement("div");return t.className=e+" CodeMirror-guttermarker-subtle",t}return e.cloneNode(!0)}function o(e,n,o){var s=e.state.foldGutter.options,l=n-1,u=e.foldOption(s,"minFoldSize"),c=e.foldOption(s,"rangeFinder"),d="string"==typeof s.indicatorFolded&&a(s.indicatorFolded),h="string"==typeof s.indicatorOpen&&a(s.indicatorOpen);e.eachLine(n,o,(function(n){++l;var o=null,a=n.gutterMarkers;if(a&&(a=a[s.gutter]),r(e,l)){if(d&&a&&d.test(a.className))return;o=i(s.indicatorFolded)}else{var f=t(l,0),p=c&&c(e,f);if(p&&p.to.line-p.from.line>=u){if(h&&a&&h.test(a.className))return;o=i(s.indicatorOpen)}}(o||a)&&e.setGutterMarker(n,s.gutter,o)}))}function a(e){return new RegExp("(^|\\s)"+e+"(?:$|\\s)\\s*")}function s(e){var t=e.getViewport(),n=e.state.foldGutter;n&&(e.operation((function(){o(e,t.from,t.to)})),n.from=t.from,n.to=t.to)}function l(e,n,i){var o=e.state.foldGutter;if(o){var a=o.options;if(i==a.gutter){var s=r(e,n);s?s.clear():e.foldCode(t(n,0),a)}}}function u(e){var t=e.state.foldGutter;if(t){var n=t.options;t.from=t.to=0,clearTimeout(t.changeUpdate),t.changeUpdate=setTimeout((function(){s(e)}),n.foldOnChangeTimeSpan||600)}}function c(e){var t=e.state.foldGutter;if(t){var n=t.options;clearTimeout(t.changeUpdate),t.changeUpdate=setTimeout((function(){var n=e.getViewport();t.from==t.to||n.from-t.to>20||t.from-n.to>20?s(e):e.operation((function(){n.from<t.from&&(o(e,n.from,t.from),t.from=n.from),n.to>t.to&&(o(e,t.to,n.to),t.to=n.to)}))}),n.updateViewportTimeSpan||400)}}function d(e,t){var n=e.state.foldGutter;if(n){var r=t.line;r>=n.from&&r<n.to&&o(e,r,r+1)}}}(n(24),n(868))},function(e,t,n){!function(e){"use strict";e.registerHelper("fold","brace",(function(t,n){var r,i=n.line,o=t.getLine(i);function a(a){for(var s=n.ch,l=0;;){var u=s<=0?-1:o.lastIndexOf(a,s-1);if(-1!=u){if(1==l&&u<n.ch)break;if(r=t.getTokenTypeAt(e.Pos(i,u+1)),!/^(comment|string)/.test(r))return u+1;s=u-1}else{if(1==l)break;l=1,s=o.length}}}var s="{",l="}",u=a("{");if(null==u&&(s="[",l="]",u=a("[")),null!=u){var c,d,h=1,f=t.lastLine();e:for(var p=i;p<=f;++p)for(var g=t.getLine(p),v=p==i?u:0;;){var m=g.indexOf(s,v),y=g.indexOf(l,v);if(m<0&&(m=g.length),y<0&&(y=g.length),(v=Math.min(m,y))==g.length)break;if(t.getTokenTypeAt(e.Pos(p,v+1))==r)if(v==m)++h;else if(!--h){c=p,d=v;break e}++v}if(null!=c&&i!=c)return{from:e.Pos(i,u),to:e.Pos(c,d)}}})),e.registerHelper("fold","import",(function(t,n){function r(n){if(n<t.firstLine()||n>t.lastLine())return null;var r=t.getTokenAt(e.Pos(n,1));if(/\S/.test(r.string)||(r=t.getTokenAt(e.Pos(n,r.end+1))),"keyword"!=r.type||"import"!=r.string)return null;for(var i=n,o=Math.min(t.lastLine(),n+10);i<=o;++i){var a=t.getLine(i).indexOf(";");if(-1!=a)return{startCh:r.end,end:e.Pos(i,a)}}}var i,o=n.line,a=r(o);if(!a||r(o-1)||(i=r(o-2))&&i.end.line==o-1)return null;for(var s=a.end;;){var l=r(s.line+1);if(null==l)break;s=l.end}return{from:t.clipPos(e.Pos(o,a.startCh+1)),to:s}})),e.registerHelper("fold","include",(function(t,n){function r(n){if(n<t.firstLine()||n>t.lastLine())return null;var r=t.getTokenAt(e.Pos(n,1));return/\S/.test(r.string)||(r=t.getTokenAt(e.Pos(n,r.end+1))),"meta"==r.type&&"#include"==r.string.slice(0,8)?r.start+8:void 0}var i=n.line,o=r(i);if(null==o||null!=r(i-1))return null;for(var a=i;null!=r(a+1);)++a;return{from:e.Pos(i,o+1),to:t.clipPos(e.Pos(a))}}))}(n(24))},function(e,t,n){!function(e){"use strict";function t(t,n){var r=t.getLine(n),i=r.search(/\S/);return-1==i||/\bcomment\b/.test(t.getTokenTypeAt(e.Pos(n,i+1)))?-1:e.countColumn(r,null,t.getOption("tabSize"))}e.registerHelper("fold","indent",(function(n,r){var i=t(n,r.line);if(!(i<0)){for(var o=null,a=r.line+1,s=n.lastLine();a<=s;++a){var l=t(n,a);if(-1==l);else{if(!(l>i))break;o=a}}return o?{from:e.Pos(r.line,n.getLine(r.line).length),to:e.Pos(o,n.getLine(o).length)}:void 0}}))}(n(24))},function(e,t,n){!function(e){"use strict";e.registerHelper("fold","markdown",(function(t,n){function r(n){var r=t.getTokenTypeAt(e.Pos(n,0));return r&&/\bheader\b/.test(r)}function i(e,t,n){var i=t&&t.match(/^#+/);return i&&r(e)?i[0].length:(i=n&&n.match(/^[=\-]+\s*$/))&&r(e+1)?"="==n[0]?1:2:100}var o=t.getLine(n.line),a=t.getLine(n.line+1),s=i(n.line,o,a);if(100!==s){for(var l=t.lastLine(),u=n.line,c=t.getLine(u+2);u<l&&!(i(u+1,a,c)<=s);)++u,a=c,c=t.getLine(u+2);return{from:e.Pos(n.line,o.length),to:e.Pos(u,t.getLine(u).length)}}}))}(n(24))},function(e,t,n){!function(e){"use strict";var t=e.Pos;function n(e,t){return e.line-t.line||e.ch-t.ch}var r="A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD",i=new RegExp("<(/?)(["+r+"]["+r+"-:.0-9\\u00B7\\u0300-\\u036F\\u203F-\\u2040]*)","g");function o(e,t,n,r){this.line=t,this.ch=n,this.cm=e,this.text=e.getLine(t),this.min=r?Math.max(r.from,e.firstLine()):e.firstLine(),this.max=r?Math.min(r.to-1,e.lastLine()):e.lastLine()}function a(e,n){var r=e.cm.getTokenTypeAt(t(e.line,n));return r&&/\btag\b/.test(r)}function s(e){if(!(e.line>=e.max))return e.ch=0,e.text=e.cm.getLine(++e.line),!0}function l(e){if(!(e.line<=e.min))return e.text=e.cm.getLine(--e.line),e.ch=e.text.length,!0}function u(e){for(;;){var t=e.text.indexOf(">",e.ch);if(-1==t){if(s(e))continue;return}if(a(e,t+1)){var n=e.text.lastIndexOf("/",t),r=n>-1&&!/\S/.test(e.text.slice(n+1,t));return e.ch=t+1,r?"selfClose":"regular"}e.ch=t+1}}function c(e){for(;;){var t=e.ch?e.text.lastIndexOf("<",e.ch-1):-1;if(-1==t){if(l(e))continue;return}if(a(e,t+1)){i.lastIndex=t,e.ch=t;var n=i.exec(e.text);if(n&&n.index==t)return n}else e.ch=t}}function d(e){for(;;){i.lastIndex=e.ch;var t=i.exec(e.text);if(!t){if(s(e))continue;return}if(a(e,t.index+1))return e.ch=t.index+t[0].length,t;e.ch=t.index+1}}function h(e){for(;;){var t=e.ch?e.text.lastIndexOf(">",e.ch-1):-1;if(-1==t){if(l(e))continue;return}if(a(e,t+1)){var n=e.text.lastIndexOf("/",t),r=n>-1&&!/\S/.test(e.text.slice(n+1,t));return e.ch=t+1,r?"selfClose":"regular"}e.ch=t}}function f(e,n){for(var r=[];;){var i,o=d(e),a=e.line,s=e.ch-(o?o[0].length:0);if(!o||!(i=u(e)))return;if("selfClose"!=i)if(o[1]){for(var l=r.length-1;l>=0;--l)if(r[l]==o[2]){r.length=l;break}if(l<0&&(!n||n==o[2]))return{tag:o[2],from:t(a,s),to:t(e.line,e.ch)}}else r.push(o[2])}}function p(e,n){for(var r=[];;){var i=h(e);if(!i)return;if("selfClose"!=i){var o=e.line,a=e.ch,s=c(e);if(!s)return;if(s[1])r.push(s[2]);else{for(var l=r.length-1;l>=0;--l)if(r[l]==s[2]){r.length=l;break}if(l<0&&(!n||n==s[2]))return{tag:s[2],from:t(e.line,e.ch),to:t(o,a)}}}else c(e)}}e.registerHelper("fold","xml",(function(e,r){for(var i=new o(e,r.line,0);;){var a=d(i);if(!a||i.line!=r.line)return;var s=u(i);if(!s)return;if(!a[1]&&"selfClose"!=s){var l=t(i.line,i.ch),c=f(i,a[2]);return c&&n(c.from,l)>0?{from:l,to:c.from}:null}}})),e.findMatchingTag=function(e,r,i){var a=new o(e,r.line,r.ch,i);if(-1!=a.text.indexOf(">")||-1!=a.text.indexOf("<")){var s=u(a),l=s&&t(a.line,a.ch),d=s&&c(a);if(s&&d&&!(n(a,r)>0)){var h={from:t(a.line,a.ch),to:l,tag:d[2]};return"selfClose"==s?{open:h,close:null,at:"open"}:d[1]?{open:p(a,d[2]),close:h,at:"close"}:{open:h,close:f(a=new o(e,l.line,l.ch,i),d[2]),at:"open"}}}},e.findEnclosingTag=function(e,t,n,r){for(var i=new o(e,t.line,t.ch,n);;){var a=p(i,r);if(!a)break;var s=f(new o(e,t.line,t.ch,n),a.tag);if(s)return{open:a,close:s}}},e.scanForClosingTag=function(e,t,n,r){return f(new o(e,t.line,t.ch,r?{from:0,to:r}:null),n)}}(n(24))},function(e,t,n){!function(e){"use strict";e.registerGlobalHelper("fold","comment",(function(e){return e.blockCommentStart&&e.blockCommentEnd}),(function(t,n){var r=t.getModeAt(n),i=r.blockCommentStart,o=r.blockCommentEnd;if(i&&o){for(var a,s=n.line,l=t.getLine(s),u=n.ch,c=0;;){var d=u<=0?-1:l.lastIndexOf(i,u-1);if(-1!=d){if(1==c&&d<n.ch)return;if(/comment/.test(t.getTokenTypeAt(e.Pos(s,d+1)))&&(0==d||l.slice(d-o.length,d)==o||!/comment/.test(t.getTokenTypeAt(e.Pos(s,d))))){a=d+i.length;break}u=d-1}else{if(1==c)return;c=1,u=l.length}}var h,f,p=1,g=t.lastLine();e:for(var v=s;v<=g;++v)for(var m=t.getLine(v),y=v==s?a:0;;){var b=m.indexOf(i,y),C=m.indexOf(o,y);if(b<0&&(b=m.length),C<0&&(C=m.length),(y=Math.min(b,C))==m.length)break;if(y==b)++p;else if(!--p){h=v,f=y;break e}++y}if(null!=h&&(s!=h||f!=a))return{from:e.Pos(s,a),to:e.Pos(h,f)}}}))}(n(24))},function(e,t,n){!function(e){"use strict";function t(t,r){e.changeEnd(r).line==t.lastLine()&&n(t)}function n(e){var t="";e.lineCount()>1&&(t=e.display.scroller.clientHeight-30-e.getLineHandle(e.lastLine()).height+"px"),e.state.scrollPastEndPadding!=t&&(e.state.scrollPastEndPadding=t,e.display.lineSpace.parentNode.style.paddingBottom=t,e.off("refresh",n),e.setSize(),e.on("refresh",n))}e.defineOption("scrollPastEnd",!1,(function(r,i,o){o&&o!=e.Init&&(r.off("change",t),r.off("refresh",n),r.display.lineSpace.parentNode.style.paddingBottom="",r.state.scrollPastEndPadding=null),i&&(r.on("change",t),r.on("refresh",n),n(r))}))}(n(24))},function(e,t,n){!function(e){"use strict";function t(){this.posFrom=this.posTo=this.lastQuery=this.query=null,this.overlay=null}function n(e){return e.state.search||(e.state.search=new t)}function r(e){return"string"==typeof e&&e==e.toLowerCase()}function i(e,t,n){return e.getSearchCursor(t,n,{caseFold:r(t),multiline:!0})}function o(e,t,n,r,i){e.openDialog?e.openDialog(t,i,{value:r,selectValueOnOpen:!0}):i(prompt(n,r))}function a(e){return e.replace(/\\([nrt\\])/g,(function(e,t){return"n"==t?"\n":"r"==t?"\r":"t"==t?"\t":"\\"==t?"\\":e}))}function s(e){var t=e.match(/^\/(.*)\/([a-z]*)$/);if(t)try{e=new RegExp(t[1],-1==t[2].indexOf("i")?"":"i")}catch(e){}else e=a(e);return("string"==typeof e?""==e:e.test(""))&&(e=/x^/),e}function l(e,t,n){t.queryText=n,t.query=s(n),e.removeOverlay(t.overlay,r(t.query)),t.overlay=function(e,t){return"string"==typeof e?e=new RegExp(e.replace(/[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|]/g,"\\$&"),t?"gi":"g"):e.global||(e=new RegExp(e.source,e.ignoreCase?"gi":"g")),{token:function(t){e.lastIndex=t.pos;var n=e.exec(t.string);if(n&&n.index==t.pos)return t.pos+=n[0].length||1,"searching";n?t.pos=n.index:t.skipToEnd()}}}(t.query,r(t.query)),e.addOverlay(t.overlay),e.showMatchesOnScrollbar&&(t.annotate&&(t.annotate.clear(),t.annotate=null),t.annotate=e.showMatchesOnScrollbar(t.query,r(t.query)))}function u(t,r,i,a){var s=n(t);if(s.query)return c(t,r);var u=t.getSelection()||s.lastQuery;if(u instanceof RegExp&&"x^"==u.source&&(u=null),i&&t.openDialog){var f=null,p=function(n,r){e.e_stop(r),n&&(n!=s.queryText&&(l(t,s,n),s.posFrom=s.posTo=t.getCursor()),f&&(f.style.opacity=1),c(t,r.shiftKey,(function(e,n){var r;n.line<3&&document.querySelector&&(r=t.display.wrapper.querySelector(".CodeMirror-dialog"))&&r.getBoundingClientRect().bottom-4>t.cursorCoords(n,"window").top&&((f=r).style.opacity=.4)})))};!function(e,t,n,r,i){e.openDialog(t,r,{value:n,selectValueOnOpen:!0,closeOnEnter:!1,onClose:function(){d(e)},onKeyDown:i})}(t,h(t),u,p,(function(r,i){var o=e.keyName(r),a=t.getOption("extraKeys"),s=a&&a[o]||e.keyMap[t.getOption("keyMap")][o];"findNext"==s||"findPrev"==s||"findPersistentNext"==s||"findPersistentPrev"==s?(e.e_stop(r),l(t,n(t),i),t.execCommand(s)):"find"!=s&&"findPersistent"!=s||(e.e_stop(r),p(i,r))})),a&&u&&(l(t,s,u),c(t,r))}else o(t,h(t),"Search for:",u,(function(e){e&&!s.query&&t.operation((function(){l(t,s,e),s.posFrom=s.posTo=t.getCursor(),c(t,r)}))}))}function c(t,r,o){t.operation((function(){var a=n(t),s=i(t,a.query,r?a.posFrom:a.posTo);(s.find(r)||(s=i(t,a.query,r?e.Pos(t.lastLine()):e.Pos(t.firstLine(),0))).find(r))&&(t.setSelection(s.from(),s.to()),t.scrollIntoView({from:s.from(),to:s.to()},20),a.posFrom=s.from(),a.posTo=s.to(),o&&o(s.from(),s.to()))}))}function d(e){e.operation((function(){var t=n(e);t.lastQuery=t.query,t.query&&(t.query=t.queryText=null,e.removeOverlay(t.overlay),t.annotate&&(t.annotate.clear(),t.annotate=null))}))}function h(e){return'<span class="CodeMirror-search-label">'+e.phrase("Search:")+'</span> <input type="text" style="width: 10em" class="CodeMirror-search-field"/> <span style="color: #888" class="CodeMirror-search-hint">'+e.phrase("(Use /re/ syntax for regexp search)")+"</span>"}function f(e,t,n){e.operation((function(){for(var r=i(e,t);r.findNext();)if("string"!=typeof t){var o=e.getRange(r.from(),r.to()).match(t);r.replace(n.replace(/\$(\d)/g,(function(e,t){return o[t]})))}else r.replace(n)}))}function p(e,t){if(!e.getOption("readOnly")){var r=e.getSelection()||n(e).lastQuery,l='<span class="CodeMirror-search-label">'+(t?e.phrase("Replace all:"):e.phrase("Replace:"))+"</span>";o(e,l+function(e){return' <input type="text" style="width: 10em" class="CodeMirror-search-field"/> <span style="color: #888" class="CodeMirror-search-hint">'+e.phrase("(Use /re/ syntax for regexp search)")+"</span>"}(e),l,r,(function(n){n&&(n=s(n),o(e,function(e){return'<span class="CodeMirror-search-label">'+e.phrase("With:")+'</span> <input type="text" style="width: 10em" class="CodeMirror-search-field"/>'}(e),e.phrase("Replace with:"),"",(function(r){if(r=a(r),t)f(e,n,r);else{d(e);var o=i(e,n,e.getCursor("from")),s=function(){var t,a=o.from();!(t=o.findNext())&&(o=i(e,n),!(t=o.findNext())||a&&o.from().line==a.line&&o.from().ch==a.ch)||(e.setSelection(o.from(),o.to()),e.scrollIntoView({from:o.from(),to:o.to()}),function(e,t,n,r){e.openConfirm?e.openConfirm(t,r):confirm(n)&&r[0]()}(e,function(e){return'<span class="CodeMirror-search-label">'+e.phrase("Replace?")+"</span> <button>"+e.phrase("Yes")+"</button> <button>"+e.phrase("No")+"</button> <button>"+e.phrase("All")+"</button> <button>"+e.phrase("Stop")+"</button> "}(e),e.phrase("Replace?"),[function(){l(t)},s,function(){f(e,n,r)}]))},l=function(e){o.replace("string"==typeof n?r:r.replace(/\$(\d)/g,(function(t,n){return e[n]}))),s()};s()}})))}))}}e.commands.find=function(e){d(e),u(e)},e.commands.findPersistent=function(e){d(e),u(e,!1,!0)},e.commands.findPersistentNext=function(e){u(e,!1,!0,!0)},e.commands.findPersistentPrev=function(e){u(e,!0,!0,!0)},e.commands.findNext=u,e.commands.findPrev=function(e){u(e,!0)},e.commands.clearSearch=d,e.commands.replace=p,e.commands.replaceAll=function(e){p(e,!0)}}(n(24),n(501),n(869))},function(e,t,n){!function(e){"use strict";function t(e,t){var n=Number(t);return/^[-+]/.test(t)?e.getCursor().line+n:n-1}e.commands.jumpToLine=function(e){var n=e.getCursor();!function(e,t,n,r,i){e.openDialog?e.openDialog(t,i,{value:r,selectValueOnOpen:!0}):i(prompt(n,r))}(e,function(e){return e.phrase("Jump to line:")+' <input type="text" style="width: 10em" class="CodeMirror-search-field"/> <span style="color: #888" class="CodeMirror-search-hint">'+e.phrase("(Use line:column or scroll% syntax)")+"</span>"}(e),e.phrase("Jump to line:"),n.line+1+":"+n.ch,(function(r){var i;if(r)if(i=/^\s*([\+\-]?\d+)\s*\:\s*(\d+)\s*$/.exec(r))e.setCursor(t(e,i[1]),Number(i[2]));else if(i=/^\s*([\+\-]?\d+(\.\d+)?)\%\s*/.exec(r)){var o=Math.round(e.lineCount()*Number(i[1])/100);/^[-+]/.test(i[1])&&(o=n.line+o+1),e.setCursor(o-1,n.ch)}else(i=/^\s*\:?\s*([\+\-]?\d+)\s*/.exec(r))&&e.setCursor(t(e,i[1]),n.ch)}))},e.keyMap.default["Alt-G"]="jumpToLine"}(n(24),n(869))},function(e,t,n){!function(e){"use strict";var t="CodeMirror-activeline-background";function n(e){for(var n=0;n<e.state.activeLines.length;n++)e.removeLineClass(e.state.activeLines[n],"wrap","CodeMirror-activeline"),e.removeLineClass(e.state.activeLines[n],"background",t),e.removeLineClass(e.state.activeLines[n],"gutter","CodeMirror-activeline-gutter")}function r(e,r){for(var i=[],o=0;o<r.length;o++){var a=r[o],s=e.getOption("styleActiveLine");if("object"==typeof s&&s.nonEmpty?a.anchor.line==a.head.line:a.empty()){var l=e.getLineHandleVisualStart(a.head.line);i[i.length-1]!=l&&i.push(l)}}(function(e,t){if(e.length!=t.length)return!1;for(var n=0;n<e.length;n++)if(e[n]!=t[n])return!1;return!0})(e.state.activeLines,i)||e.operation((function(){n(e);for(var r=0;r<i.length;r++)e.addLineClass(i[r],"wrap","CodeMirror-activeline"),e.addLineClass(i[r],"background",t),e.addLineClass(i[r],"gutter","CodeMirror-activeline-gutter");e.state.activeLines=i}))}function i(e,t){r(e,t.ranges)}e.defineOption("styleActiveLine",!1,(function(t,o,a){var s=a!=e.Init&&a;o!=s&&(s&&(t.off("beforeSelectionChange",i),n(t),delete t.state.activeLines),o&&(t.state.activeLines=[],r(t,t.listSelections()),t.on("beforeSelectionChange",i)))}))}(n(24))},function(e,t,n){!function(e){"use strict";function t(e){e.state.markedSelection&&e.operation((function(){!function(e){if(!e.somethingSelected())return a(e);if(e.listSelections().length>1)return s(e);var t=e.getCursor("start"),n=e.getCursor("end"),r=e.state.markedSelection;if(!r.length)return o(e,t,n);var l=r[0].find(),u=r[r.length-1].find();if(!l||!u||n.line-t.line<=8||i(t,u.to)>=0||i(n,l.from)<=0)return s(e);for(;i(t,l.from)>0;)r.shift().clear(),l=r[0].find();for(i(t,l.from)<0&&(l.to.line-t.line<8?(r.shift().clear(),o(e,t,l.to,0)):o(e,t,l.from,0));i(n,u.to)<0;)r.pop().clear(),u=r[r.length-1].find();i(n,u.to)>0&&(n.line-u.from.line<8?(r.pop().clear(),o(e,u.from,n)):o(e,u.to,n))}(e)}))}function n(e){e.state.markedSelection&&e.state.markedSelection.length&&e.operation((function(){a(e)}))}e.defineOption("styleSelectedText",!1,(function(r,i,o){var l=o&&o!=e.Init;i&&!l?(r.state.markedSelection=[],r.state.markedSelectionStyle="string"==typeof i?i:"CodeMirror-selectedtext",s(r),r.on("cursorActivity",t),r.on("change",n)):!i&&l&&(r.off("cursorActivity",t),r.off("change",n),a(r),r.state.markedSelection=r.state.markedSelectionStyle=null)}));var r=e.Pos,i=e.cmpPos;function o(e,t,n,o){if(0!=i(t,n))for(var a=e.state.markedSelection,s=e.state.markedSelectionStyle,l=t.line;;){var u=l==t.line?t:r(l,0),c=l+8,d=c>=n.line,h=d?n:r(c,0),f=e.markText(u,h,{className:s});if(null==o?a.push(f):a.splice(o++,0,f),d)break;l=c}}function a(e){for(var t=e.state.markedSelection,n=0;n<t.length;++n)t[n].clear();t.length=0}function s(e){a(e);for(var t=e.listSelections(),n=0;n<t.length;n++)o(e,t[n].from(),t[n].to())}}(n(24))},function(e,t,n){!function(e){"use strict";function t(e){e.state.selectionPointer.rects=null,n(e)}function n(e){e.state.selectionPointer.willUpdate||(e.state.selectionPointer.willUpdate=!0,setTimeout((function(){!function(e){var t=e.state.selectionPointer;if(t){if(null==t.rects&&null!=t.mouseX&&(t.rects=[],e.somethingSelected()))for(var n=e.display.selectionDiv.firstChild;n;n=n.nextSibling)t.rects.push(n.getBoundingClientRect());var r=!1;if(null!=t.mouseX)for(var i=0;i<t.rects.length;i++){var o=t.rects[i];o.left<=t.mouseX&&o.right>=t.mouseX&&o.top<=t.mouseY&&o.bottom>=t.mouseY&&(r=!0)}var a=r?t.value:"";e.display.lineDiv.style.cursor!=a&&(e.display.lineDiv.style.cursor=a)}}(e),e.state.selectionPointer.willUpdate=!1}),50))}e.defineOption("selectionPointer",!1,(function(r,i){var o=r.state.selectionPointer;o&&(e.off(r.getWrapperElement(),"mousemove",o.mousemove),e.off(r.getWrapperElement(),"mouseout",o.mouseout),e.off(window,"scroll",o.windowScroll),r.off("cursorActivity",t),r.off("scroll",t),r.state.selectionPointer=null,r.display.lineDiv.style.cursor=""),i&&(o=r.state.selectionPointer={value:"string"==typeof i?i:"default",mousemove:function(e){!function(e,t){var r=e.state.selectionPointer;(null==t.buttons?t.which:t.buttons)?r.mouseX=r.mouseY=null:(r.mouseX=t.clientX,r.mouseY=t.clientY),n(e)}(r,e)},mouseout:function(e){!function(e,t){if(!e.getWrapperElement().contains(t.relatedTarget)){var r=e.state.selectionPointer;r.mouseX=r.mouseY=null,n(e)}}(r,e)},windowScroll:function(){t(r)},rects:null,mouseX:null,mouseY:null,willUpdate:!1},e.on(r.getWrapperElement(),"mousemove",o.mousemove),e.on(r.getWrapperElement(),"mouseout",o.mouseout),e.on(window,"scroll",o.windowScroll),r.on("cursorActivity",t),r.on("scroll",t))}))}(n(24))},function(e,t,n){!function(e){"use strict";var t=e.Pos;function n(e,t){return e.line==t.line&&e.ch==t.ch}var r=[];function i(e){r.push(e),r.length>50&&r.shift()}function o(e){return r[r.length-(e?Math.min(e,1):1)]||""}var a=null;function s(e,t,o,s,l){null==l&&(l=e.getRange(t,o)),"grow"==s&&a&&a.cm==e&&n(t,a.pos)&&e.isClean(a.gen)?function(e){if(!r.length)return i(e);r[r.length-1]+=e}(l):!1!==s&&i(l),e.replaceRange("",t,o,"+delete"),a="grow"==s?{cm:e,pos:t,gen:e.changeGeneration()}:null}function l(e,t,n){return e.findPosH(t,n,"char",!0)}function u(e,t,n){return e.findPosH(t,n,"word",!0)}function c(e,t,n){return e.findPosV(t,n,"line",e.doc.sel.goalColumn)}function d(e,t,n){return e.findPosV(t,n,"page",e.doc.sel.goalColumn)}function h(e,n,r){for(var i=n.line,o=e.getLine(i),a=/\S/.test(r<0?o.slice(0,n.ch):o.slice(n.ch)),s=e.firstLine(),l=e.lastLine();;){if((i+=r)<s||i>l)return e.clipPos(t(i-r,r<0?0:null));if(o=e.getLine(i),/\S/.test(o))a=!0;else if(a)return t(i,0)}}function f(e,n,r){for(var i=n.line,o=n.ch,a=e.getLine(n.line),s=!1;;){var l=a.charAt(o+(r<0?-1:0));if(l){if(s&&/[!?.]/.test(l))return t(i,o+(r>0?1:0));s||(s=/\w/.test(l)),o+=r}else{if(i==(r<0?e.firstLine():e.lastLine()))return t(i,o);if(a=e.getLine(i+r),!/\S/.test(a))return t(i,o);i+=r,o=r<0?a.length:0}}}function p(e,r,i){var o;if(e.findMatchingBracket&&(o=e.findMatchingBracket(r,{strict:!0}))&&o.match&&(o.forward?1:-1)==i)return i>0?t(o.to.line,o.to.ch+1):o.to;for(var a=!0;;a=!1){var s=e.getTokenAt(r),l=t(r.line,i<0?s.start:s.end);if(!(a&&i>0&&s.end==r.ch)&&/\w/.test(s.string))return l;var u=e.findPosH(l,i,"char");if(n(l,u))return r;r=u}}function g(e,t){var n=e.state.emacsPrefix;return n?(x(e),"-"==n?-1:Number(n)):t?null:1}function v(e){var t="string"==typeof e?function(t){t.execCommand(e)}:e;return function(e){var n=g(e);t(e);for(var r=1;r<n;++r)t(e)}}function m(e,t,r,i){var o=g(e);o<0&&(i=-i,o=-o);for(var a=0;a<o;++a){var s=r(e,t,i);if(n(s,t))break;t=s}return t}function y(e,t){var n=function(n){n.extendSelection(m(n,n.getCursor(),e,t))};return n.motion=!0,n}function b(e,t,n,r){for(var i,o=e.listSelections(),a=o.length;a--;)s(e,i=o[a].head,m(e,i,t,n),r)}function C(e,t){if(e.somethingSelected()){for(var n,r=e.listSelections(),i=r.length;i--;)s(e,(n=r[i]).anchor,n.head,t);return!0}}function A(e,t){e.state.emacsPrefix?"-"!=t&&(e.state.emacsPrefix+=t):(e.state.emacsPrefix=t,e.on("keyHandled",I),e.on("inputRead",w))}var _={"Alt-G":!0,"Ctrl-X":!0,"Ctrl-Q":!0,"Ctrl-U":!0};function I(e,t){e.state.emacsPrefixMap||_.hasOwnProperty(t)||x(e)}function x(e){e.state.emacsPrefix=null,e.off("keyHandled",I),e.off("inputRead",w)}function w(e,t){var n=g(e);if(n>1&&"+input"==t.origin){for(var r=t.text.join("\n"),i="",o=1;o<n;++o)i+=r;e.replaceSelection(i)}}function M(e,t){("string"!=typeof t||!/^\d$/.test(t)&&"Ctrl-U"!=t)&&(e.removeKeyMap(k),e.state.emacsPrefixMap=!1,e.off("keyHandled",M),e.off("inputRead",M))}function S(e){e.setCursor(e.getCursor()),e.setExtending(!e.getExtending()),e.on("change",(function(){e.setExtending(!1)}))}function T(e){e.setExtending(!1),e.setCursor(e.getCursor())}function E(e,t){var n=e.getCursor(),r=e.findPosH(n,1,"word");e.replaceRange(t(e.getRange(n,r)),n,r),e.setCursor(r)}e.emacs={kill:s,killRegion:C,repeated:v};var O=e.keyMap.emacs=e.normalizeKeyMap({"Ctrl-W":function(e){s(e,e.getCursor("start"),e.getCursor("end"),!0)},"Ctrl-K":v((function(e){var n=e.getCursor(),r=e.clipPos(t(n.line)),i=e.getRange(n,r);/\S/.test(i)||(i+="\n",r=t(n.line+1,0)),s(e,n,r,"grow",i)})),"Alt-W":function(e){i(e.getSelection()),T(e)},"Ctrl-Y":function(e){var t=e.getCursor();e.replaceRange(o(g(e)),t,t,"paste"),e.setSelection(t,e.getCursor())},"Alt-Y":function(e){e.replaceSelection((r.length>1&&r.pop(),o()),"around","paste")},"Ctrl-Space":S,"Ctrl-Shift-2":S,"Ctrl-F":y(l,1),"Ctrl-B":y(l,-1),Right:y(l,1),Left:y(l,-1),"Ctrl-D":function(e){b(e,l,1,!1)},Delete:function(e){C(e,!1)||b(e,l,1,!1)},"Ctrl-H":function(e){b(e,l,-1,!1)},Backspace:function(e){C(e,!1)||b(e,l,-1,!1)},"Alt-F":y(u,1),"Alt-B":y(u,-1),"Alt-Right":y(u,1),"Alt-Left":y(u,-1),"Alt-D":function(e){b(e,u,1,"grow")},"Alt-Backspace":function(e){b(e,u,-1,"grow")},"Ctrl-N":y(c,1),"Ctrl-P":y(c,-1),Down:y(c,1),Up:y(c,-1),"Ctrl-A":"goLineStart","Ctrl-E":"goLineEnd",End:"goLineEnd",Home:"goLineStart","Alt-V":y(d,-1),"Ctrl-V":y(d,1),PageUp:y(d,-1),PageDown:y(d,1),"Ctrl-Up":y(h,-1),"Ctrl-Down":y(h,1),"Alt-A":y(f,-1),"Alt-E":y(f,1),"Alt-K":function(e){b(e,f,1,"grow")},"Ctrl-Alt-K":function(e){b(e,p,1,"grow")},"Ctrl-Alt-Backspace":function(e){b(e,p,-1,"grow")},"Ctrl-Alt-F":y(p,1),"Ctrl-Alt-B":y(p,-1),"Shift-Ctrl-Alt-2":function(e){var t=e.getCursor();e.setSelection(m(e,t,p,1),t)},"Ctrl-Alt-T":function(e){var t=p(e,e.getCursor(),-1),n=p(e,t,1),r=p(e,n,1),i=p(e,r,-1);e.replaceRange(e.getRange(i,r)+e.getRange(n,i)+e.getRange(t,n),t,r)},"Ctrl-Alt-U":v((function(e){for(var n=e.getCursor(),r=n.line,i=n.ch,o=[];r>=e.firstLine();){for(var a=e.getLine(r),s=null==i?a.length:i;s>0;)if(")"==(i=a.charAt(--s)))o.push("(");else if("]"==i)o.push("[");else if("}"==i)o.push("{");else if(/[\(\{\[]/.test(i)&&(!o.length||o.pop()!=i))return e.extendSelection(t(r,s));--r,i=null}})),"Alt-Space":function(e){for(var n=e.getCursor(),r=n.ch,i=n.ch,o=e.getLine(n.line);r&&/\s/.test(o.charAt(r-1));)--r;for(;i<o.length&&/\s/.test(o.charAt(i));)++i;e.replaceRange(" ",t(n.line,r),t(n.line,i))},"Ctrl-O":v((function(e){e.replaceSelection("\n","start")})),"Ctrl-T":v((function(e){e.execCommand("transposeChars")})),"Alt-C":v((function(e){E(e,(function(e){var t=e.search(/\w/);return-1==t?e:e.slice(0,t)+e.charAt(t).toUpperCase()+e.slice(t+1).toLowerCase()}))})),"Alt-U":v((function(e){E(e,(function(e){return e.toUpperCase()}))})),"Alt-L":v((function(e){E(e,(function(e){return e.toLowerCase()}))})),"Alt-;":"toggleComment","Ctrl-/":v("undo"),"Shift-Ctrl--":v("undo"),"Ctrl-Z":v("undo"),"Cmd-Z":v("undo"),"Shift-Ctrl-Z":"redo","Shift-Alt-,":"goDocStart","Shift-Alt-.":"goDocEnd","Ctrl-S":"findPersistentNext","Ctrl-R":"findPersistentPrev","Ctrl-G":function(e){e.execCommand("clearSearch"),T(e)},"Shift-Alt-5":"replace","Alt-/":"autocomplete",Enter:"newlineAndIndent","Ctrl-J":v((function(e){e.replaceSelection("\n","end")})),Tab:"indentAuto","Alt-G G":function(e){var t=g(e,!0);if(null!=t&&t>0)return e.setCursor(t-1);!function(e,t,n){e.openDialog?e.openDialog(t+': <input type="text" style="width: 10em"/>',n,{bottom:!0}):n(prompt(t,""))}(e,"Goto line",(function(t){var n;t&&!isNaN(n=Number(t))&&n==(0|n)&&n>0&&e.setCursor(n-1)}))},"Ctrl-X Tab":function(e){e.indentSelection(g(e,!0)||e.getOption("indentUnit"))},"Ctrl-X Ctrl-X":function(e){e.setSelection(e.getCursor("head"),e.getCursor("anchor"))},"Ctrl-X Ctrl-S":"save","Ctrl-X Ctrl-W":"save","Ctrl-X S":"saveAll","Ctrl-X F":"open","Ctrl-X U":v("undo"),"Ctrl-X K":"close","Ctrl-X Delete":function(e){s(e,e.getCursor(),f(e,e.getCursor(),1),"grow")},"Ctrl-X H":"selectAll","Ctrl-Q Tab":v("insertTab"),"Ctrl-U":function(e){e.state.emacsPrefixMap=!0,e.addKeyMap(k),e.on("keyHandled",M),e.on("inputRead",M)},fallthrough:"default"}),k={"Ctrl-G":x};function L(e){k[e]=function(t){A(t,e)},O["Ctrl-"+e]=function(t){A(t,e)},_["Ctrl-"+e]=!0}for(var D=0;D<10;++D)L(String(D));L("-")}(n(24))},function(e,t,n){!function(e){"use strict";var t=e.commands,n=e.Pos;function r(t,r){t.extendSelectionsBy((function(i){return t.display.shift||t.doc.extend||i.empty()?function(t,r,i){if(i<0&&0==r.ch)return t.clipPos(n(r.line-1));var o=t.getLine(r.line);if(i>0&&r.ch>=o.length)return t.clipPos(n(r.line+1,0));for(var a,s="start",l=r.ch,u=l,c=i<0?0:o.length,d=0;u!=c;u+=i,d++){var h=o.charAt(i<0?u-1:u),f="_"!=h&&e.isWordChar(h)?"w":"o";if("w"==f&&h.toUpperCase()==h&&(f="W"),"start"==s)"o"!=f?(s="in",a=f):l=u+i;else if("in"==s&&a!=f){if("w"==a&&"W"==f&&i<0&&u--,"W"==a&&"w"==f&&i>0){if(u==l+1){a="w";continue}u--}break}}return n(r.line,u)}(t.doc,i.head,r):r<0?i.from():i.to()}))}function i(t,r){if(t.isReadOnly())return e.Pass;t.operation((function(){for(var e=t.listSelections().length,i=[],o=-1,a=0;a<e;a++){var s=t.listSelections()[a].head;if(!(s.line<=o)){var l=n(s.line+(r?0:1),0);t.replaceRange("\n",l,null,"+insertLine"),t.indentLine(l.line,null,!0),i.push({head:l,anchor:l}),o=s.line+1}}t.setSelections(i)})),t.execCommand("indentAuto")}function o(t,r){for(var i=r.ch,o=i,a=t.getLine(r.line);i&&e.isWordChar(a.charAt(i-1));)--i;for(;o<a.length&&e.isWordChar(a.charAt(o));)++o;return{from:n(r.line,i),to:n(r.line,o),word:a.slice(i,o)}}function a(e,t){for(var n=e.listSelections(),r=[],i=0;i<n.length;i++){var o=n[i],a=e.findPosV(o.anchor,t,"line",o.anchor.goalColumn),s=e.findPosV(o.head,t,"line",o.head.goalColumn);a.goalColumn=null!=o.anchor.goalColumn?o.anchor.goalColumn:e.cursorCoords(o.anchor,"div").left,s.goalColumn=null!=o.head.goalColumn?o.head.goalColumn:e.cursorCoords(o.head,"div").left;var l={anchor:a,head:s};r.push(o),r.push(l)}e.setSelections(r)}function s(t){for(var r=t.listSelections(),i=[],o=0;o<r.length;o++){var a=r[o],s=a.head,l=t.scanForBracket(s,-1);if(!l)return!1;for(;;){var u=t.scanForBracket(s,1);if(!u)return!1;if(u.ch=="(){}[]".charAt("(){}[]".indexOf(l.ch)+1)){var c=n(l.pos.line,l.pos.ch+1);if(0!=e.cmpPos(c,a.from())||0!=e.cmpPos(u.pos,a.to())){i.push({anchor:c,head:u.pos});break}if(!(l=t.scanForBracket(l.pos,-1)))return!1}s=n(u.pos.line,u.pos.ch+1)}}return t.setSelections(i),!0}function l(e){return e?/\bpunctuation\b/.test(e)?e:void 0:null}function u(t,r){if(t.isReadOnly())return e.Pass;for(var i,o=t.listSelections(),a=[],s=0;s<o.length;s++){var l=o[s];if(!l.empty()){for(var u=l.from().line,c=l.to().line;s<o.length-1&&o[s+1].from().line==c;)c=o[++s].to().line;o[s].to().ch||c--,a.push(u,c)}}a.length?i=!0:a.push(t.firstLine(),t.lastLine()),t.operation((function(){for(var e=[],o=0;o<a.length;o+=2){var s=a[o],l=a[o+1],u=n(s,0),c=n(l),d=t.getRange(u,c,!1);r?d.sort():d.sort((function(e,t){var n=e.toUpperCase(),r=t.toUpperCase();return n!=r&&(e=n,t=r),e<t?-1:e==t?0:1})),t.replaceRange(d,u,c),i&&e.push({anchor:u,head:n(l+1,0)})}i&&t.setSelections(e,0)}))}function c(t,n){t.operation((function(){for(var r=t.listSelections(),i=[],a=[],s=0;s<r.length;s++)(u=r[s]).empty()?(i.push(s),a.push("")):a.push(n(t.getRange(u.from(),u.to())));var l;for(t.replaceSelections(a,"around","case"),s=i.length-1;s>=0;s--){var u=r[i[s]];if(!(l&&e.cmpPos(u.head,l)>0)){var c=o(t,u.head);l=c.from,t.replaceRange(n(c.word),c.from,c.to)}}}))}function d(t){var n=t.getCursor("from"),r=t.getCursor("to");if(0==e.cmpPos(n,r)){var i=o(t,n);if(!i.word)return;n=i.from,r=i.to}return{from:n,to:r,query:t.getRange(n,r),word:i}}function h(e,t){var r=d(e);if(r){var i=r.query,o=e.getSearchCursor(i,t?r.to:r.from);(t?o.findNext():o.findPrevious())?e.setSelection(o.from(),o.to()):(o=e.getSearchCursor(i,t?n(e.firstLine(),0):e.clipPos(n(e.lastLine()))),(t?o.findNext():o.findPrevious())?e.setSelection(o.from(),o.to()):r.word&&e.setSelection(r.from,r.to))}}t.goSubwordLeft=function(e){r(e,-1)},t.goSubwordRight=function(e){r(e,1)},t.scrollLineUp=function(e){var t=e.getScrollInfo();if(!e.somethingSelected()){var n=e.lineAtHeight(t.top+t.clientHeight,"local");e.getCursor().line>=n&&e.execCommand("goLineUp")}e.scrollTo(null,t.top-e.defaultTextHeight())},t.scrollLineDown=function(e){var t=e.getScrollInfo();if(!e.somethingSelected()){var n=e.lineAtHeight(t.top,"local")+1;e.getCursor().line<=n&&e.execCommand("goLineDown")}e.scrollTo(null,t.top+e.defaultTextHeight())},t.splitSelectionByLine=function(e){for(var t=e.listSelections(),r=[],i=0;i<t.length;i++)for(var o=t[i].from(),a=t[i].to(),s=o.line;s<=a.line;++s)a.line>o.line&&s==a.line&&0==a.ch||r.push({anchor:s==o.line?o:n(s,0),head:s==a.line?a:n(s)});e.setSelections(r,0)},t.singleSelectionTop=function(e){var t=e.listSelections()[0];e.setSelection(t.anchor,t.head,{scroll:!1})},t.selectLine=function(e){for(var t=e.listSelections(),r=[],i=0;i<t.length;i++){var o=t[i];r.push({anchor:n(o.from().line,0),head:n(o.to().line+1,0)})}e.setSelections(r)},t.insertLineAfter=function(e){return i(e,!1)},t.insertLineBefore=function(e){return i(e,!0)},t.selectNextOccurrence=function(t){var r=t.getCursor("from"),i=t.getCursor("to"),a=t.state.sublimeFindFullWord==t.doc.sel;if(0==e.cmpPos(r,i)){var s=o(t,r);if(!s.word)return;t.setSelection(s.from,s.to),a=!0}else{var l=t.getRange(r,i),u=a?new RegExp("\\b"+l+"\\b"):l,c=t.getSearchCursor(u,i),d=c.findNext();if(d||(d=(c=t.getSearchCursor(u,n(t.firstLine(),0))).findNext()),!d||function(t,n,r){for(var i=0;i<t.length;i++)if(0==e.cmpPos(t[i].from(),n)&&0==e.cmpPos(t[i].to(),r))return!0;return!1}(t.listSelections(),c.from(),c.to()))return;t.addSelection(c.from(),c.to())}a&&(t.state.sublimeFindFullWord=t.doc.sel)},t.skipAndSelectNextOccurrence=function(n){var r=n.getCursor("anchor"),i=n.getCursor("head");t.selectNextOccurrence(n),0!=e.cmpPos(r,i)&&n.doc.setSelections(n.doc.listSelections().filter((function(e){return e.anchor!=r||e.head!=i})))},t.addCursorToPrevLine=function(e){a(e,-1)},t.addCursorToNextLine=function(e){a(e,1)},t.selectScope=function(e){s(e)||e.execCommand("selectAll")},t.selectBetweenBrackets=function(t){if(!s(t))return e.Pass},t.goToBracket=function(t){t.extendSelectionsBy((function(r){var i=t.scanForBracket(r.head,1,l(t.getTokenTypeAt(r.head)));if(i&&0!=e.cmpPos(i.pos,r.head))return i.pos;var o=t.scanForBracket(r.head,-1,l(t.getTokenTypeAt(n(r.head.line,r.head.ch+1))));return o&&n(o.pos.line,o.pos.ch+1)||r.head}))},t.swapLineUp=function(t){if(t.isReadOnly())return e.Pass;for(var r=t.listSelections(),i=[],o=t.firstLine()-1,a=[],s=0;s<r.length;s++){var l=r[s],u=l.from().line-1,c=l.to().line;a.push({anchor:n(l.anchor.line-1,l.anchor.ch),head:n(l.head.line-1,l.head.ch)}),0!=l.to().ch||l.empty()||--c,u>o?i.push(u,c):i.length&&(i[i.length-1]=c),o=c}t.operation((function(){for(var e=0;e<i.length;e+=2){var r=i[e],o=i[e+1],s=t.getLine(r);t.replaceRange("",n(r,0),n(r+1,0),"+swapLine"),o>t.lastLine()?t.replaceRange("\n"+s,n(t.lastLine()),null,"+swapLine"):t.replaceRange(s+"\n",n(o,0),null,"+swapLine")}t.setSelections(a),t.scrollIntoView()}))},t.swapLineDown=function(t){if(t.isReadOnly())return e.Pass;for(var r=t.listSelections(),i=[],o=t.lastLine()+1,a=r.length-1;a>=0;a--){var s=r[a],l=s.to().line+1,u=s.from().line;0!=s.to().ch||s.empty()||l--,l<o?i.push(l,u):i.length&&(i[i.length-1]=u),o=u}t.operation((function(){for(var e=i.length-2;e>=0;e-=2){var r=i[e],o=i[e+1],a=t.getLine(r);r==t.lastLine()?t.replaceRange("",n(r-1),n(r),"+swapLine"):t.replaceRange("",n(r,0),n(r+1,0),"+swapLine"),t.replaceRange(a+"\n",n(o,0),null,"+swapLine")}t.scrollIntoView()}))},t.toggleCommentIndented=function(e){e.toggleComment({indent:!0})},t.joinLines=function(e){for(var t=e.listSelections(),r=[],i=0;i<t.length;i++){for(var o=t[i],a=o.from(),s=a.line,l=o.to().line;i<t.length-1&&t[i+1].from().line==l;)l=t[++i].to().line;r.push({start:s,end:l,anchor:!o.empty()&&a})}e.operation((function(){for(var t=0,i=[],o=0;o<r.length;o++){for(var a,s=r[o],l=s.anchor&&n(s.anchor.line-t,s.anchor.ch),u=s.start;u<=s.end;u++){var c=u-t;u==s.end&&(a=n(c,e.getLine(c).length+1)),c<e.lastLine()&&(e.replaceRange(" ",n(c),n(c+1,/^\s*/.exec(e.getLine(c+1))[0].length)),++t)}i.push({anchor:l||a,head:a})}e.setSelections(i,0)}))},t.duplicateLine=function(e){e.operation((function(){for(var t=e.listSelections().length,r=0;r<t;r++){var i=e.listSelections()[r];i.empty()?e.replaceRange(e.getLine(i.head.line)+"\n",n(i.head.line,0)):e.replaceRange(e.getRange(i.from(),i.to()),i.from())}e.scrollIntoView()}))},t.sortLines=function(e){u(e,!0)},t.sortLinesInsensitive=function(e){u(e,!1)},t.nextBookmark=function(e){var t=e.state.sublimeBookmarks;if(t)for(;t.length;){var n=t.shift(),r=n.find();if(r)return t.push(n),e.setSelection(r.from,r.to)}},t.prevBookmark=function(e){var t=e.state.sublimeBookmarks;if(t)for(;t.length;){t.unshift(t.pop());var n=t[t.length-1].find();if(n)return e.setSelection(n.from,n.to);t.pop()}},t.toggleBookmark=function(e){for(var t=e.listSelections(),n=e.state.sublimeBookmarks||(e.state.sublimeBookmarks=[]),r=0;r<t.length;r++){for(var i=t[r].from(),o=t[r].to(),a=t[r].empty()?e.findMarksAt(i):e.findMarks(i,o),s=0;s<a.length;s++)if(a[s].sublimeBookmark){a[s].clear();for(var l=0;l<n.length;l++)n[l]==a[s]&&n.splice(l--,1);break}s==a.length&&n.push(e.markText(i,o,{sublimeBookmark:!0,clearWhenEmpty:!1}))}},t.clearBookmarks=function(e){var t=e.state.sublimeBookmarks;if(t)for(var n=0;n<t.length;n++)t[n].clear();t.length=0},t.selectBookmarks=function(e){var t=e.state.sublimeBookmarks,n=[];if(t)for(var r=0;r<t.length;r++){var i=t[r].find();i?n.push({anchor:i.from,head:i.to}):t.splice(r--,0)}n.length&&e.setSelections(n,0)},t.smartBackspace=function(t){if(t.somethingSelected())return e.Pass;t.operation((function(){for(var r=t.listSelections(),i=t.getOption("indentUnit"),o=r.length-1;o>=0;o--){var a=r[o].head,s=t.getRange({line:a.line,ch:0},a),l=e.countColumn(s,null,t.getOption("tabSize")),u=t.findPosH(a,-1,"char",!1);if(s&&!/\S/.test(s)&&l%i==0){var c=new n(a.line,e.findColumn(s,l-i,i));c.ch!=a.ch&&(u=c)}t.replaceRange("",u,a,"+delete")}}))},t.delLineRight=function(e){e.operation((function(){for(var t=e.listSelections(),r=t.length-1;r>=0;r--)e.replaceRange("",t[r].anchor,n(t[r].to().line),"+delete");e.scrollIntoView()}))},t.upcaseAtCursor=function(e){c(e,(function(e){return e.toUpperCase()}))},t.downcaseAtCursor=function(e){c(e,(function(e){return e.toLowerCase()}))},t.setSublimeMark=function(e){e.state.sublimeMark&&e.state.sublimeMark.clear(),e.state.sublimeMark=e.setBookmark(e.getCursor())},t.selectToSublimeMark=function(e){var t=e.state.sublimeMark&&e.state.sublimeMark.find();t&&e.setSelection(e.getCursor(),t)},t.deleteToSublimeMark=function(t){var n=t.state.sublimeMark&&t.state.sublimeMark.find();if(n){var r=t.getCursor(),i=n;if(e.cmpPos(r,i)>0){var o=i;i=r,r=o}t.state.sublimeKilled=t.getRange(r,i),t.replaceRange("",r,i)}},t.swapWithSublimeMark=function(e){var t=e.state.sublimeMark&&e.state.sublimeMark.find();t&&(e.state.sublimeMark.clear(),e.state.sublimeMark=e.setBookmark(e.getCursor()),e.setCursor(t))},t.sublimeYank=function(e){null!=e.state.sublimeKilled&&e.replaceSelection(e.state.sublimeKilled,null,"paste")},t.showInCenter=function(e){var t=e.cursorCoords(null,"local");e.scrollTo(null,(t.top+t.bottom)/2-e.getScrollInfo().clientHeight/2)},t.findUnder=function(e){h(e,!0)},t.findUnderPrevious=function(e){h(e,!1)},t.findAllUnder=function(e){var t=d(e);if(t){for(var n=e.getSearchCursor(t.query),r=[],i=-1;n.findNext();)r.push({anchor:n.from(),head:n.to()}),n.from().line<=t.from.line&&n.from().ch<=t.from.ch&&i++;e.setSelections(r,i)}};var f=e.keyMap;f.macSublime={"Cmd-Left":"goLineStartSmart","Shift-Tab":"indentLess","Shift-Ctrl-K":"deleteLine","Alt-Q":"wrapLines","Ctrl-Left":"goSubwordLeft","Ctrl-Right":"goSubwordRight","Ctrl-Alt-Up":"scrollLineUp","Ctrl-Alt-Down":"scrollLineDown","Cmd-L":"selectLine","Shift-Cmd-L":"splitSelectionByLine",Esc:"singleSelectionTop","Cmd-Enter":"insertLineAfter","Shift-Cmd-Enter":"insertLineBefore","Cmd-D":"selectNextOccurrence","Shift-Cmd-Space":"selectScope","Shift-Cmd-M":"selectBetweenBrackets","Cmd-M":"goToBracket","Cmd-Ctrl-Up":"swapLineUp","Cmd-Ctrl-Down":"swapLineDown","Cmd-/":"toggleCommentIndented","Cmd-J":"joinLines","Shift-Cmd-D":"duplicateLine",F5:"sortLines","Cmd-F5":"sortLinesInsensitive",F2:"nextBookmark","Shift-F2":"prevBookmark","Cmd-F2":"toggleBookmark","Shift-Cmd-F2":"clearBookmarks","Alt-F2":"selectBookmarks",Backspace:"smartBackspace","Cmd-K Cmd-D":"skipAndSelectNextOccurrence","Cmd-K Cmd-K":"delLineRight","Cmd-K Cmd-U":"upcaseAtCursor","Cmd-K Cmd-L":"downcaseAtCursor","Cmd-K Cmd-Space":"setSublimeMark","Cmd-K Cmd-A":"selectToSublimeMark","Cmd-K Cmd-W":"deleteToSublimeMark","Cmd-K Cmd-X":"swapWithSublimeMark","Cmd-K Cmd-Y":"sublimeYank","Cmd-K Cmd-C":"showInCenter","Cmd-K Cmd-G":"clearBookmarks","Cmd-K Cmd-Backspace":"delLineLeft","Cmd-K Cmd-1":"foldAll","Cmd-K Cmd-0":"unfoldAll","Cmd-K Cmd-J":"unfoldAll","Ctrl-Shift-Up":"addCursorToPrevLine","Ctrl-Shift-Down":"addCursorToNextLine","Cmd-F3":"findUnder","Shift-Cmd-F3":"findUnderPrevious","Alt-F3":"findAllUnder","Shift-Cmd-[":"fold","Shift-Cmd-]":"unfold","Cmd-I":"findIncremental","Shift-Cmd-I":"findIncrementalReverse","Cmd-H":"replace",F3:"findNext","Shift-F3":"findPrev",fallthrough:"macDefault"},e.normalizeKeyMap(f.macSublime),f.pcSublime={"Shift-Tab":"indentLess","Shift-Ctrl-K":"deleteLine","Alt-Q":"wrapLines","Ctrl-T":"transposeChars","Alt-Left":"goSubwordLeft","Alt-Right":"goSubwordRight","Ctrl-Up":"scrollLineUp","Ctrl-Down":"scrollLineDown","Ctrl-L":"selectLine","Shift-Ctrl-L":"splitSelectionByLine",Esc:"singleSelectionTop","Ctrl-Enter":"insertLineAfter","Shift-Ctrl-Enter":"insertLineBefore","Ctrl-D":"selectNextOccurrence","Shift-Ctrl-Space":"selectScope","Shift-Ctrl-M":"selectBetweenBrackets","Ctrl-M":"goToBracket","Shift-Ctrl-Up":"swapLineUp","Shift-Ctrl-Down":"swapLineDown","Ctrl-/":"toggleCommentIndented","Ctrl-J":"joinLines","Shift-Ctrl-D":"duplicateLine",F9:"sortLines","Ctrl-F9":"sortLinesInsensitive",F2:"nextBookmark","Shift-F2":"prevBookmark","Ctrl-F2":"toggleBookmark","Shift-Ctrl-F2":"clearBookmarks","Alt-F2":"selectBookmarks",Backspace:"smartBackspace","Ctrl-K Ctrl-D":"skipAndSelectNextOccurrence","Ctrl-K Ctrl-K":"delLineRight","Ctrl-K Ctrl-U":"upcaseAtCursor","Ctrl-K Ctrl-L":"downcaseAtCursor","Ctrl-K Ctrl-Space":"setSublimeMark","Ctrl-K Ctrl-A":"selectToSublimeMark","Ctrl-K Ctrl-W":"deleteToSublimeMark","Ctrl-K Ctrl-X":"swapWithSublimeMark","Ctrl-K Ctrl-Y":"sublimeYank","Ctrl-K Ctrl-C":"showInCenter","Ctrl-K Ctrl-G":"clearBookmarks","Ctrl-K Ctrl-Backspace":"delLineLeft","Ctrl-K Ctrl-1":"foldAll","Ctrl-K Ctrl-0":"unfoldAll","Ctrl-K Ctrl-J":"unfoldAll","Ctrl-Alt-Up":"addCursorToPrevLine","Ctrl-Alt-Down":"addCursorToNextLine","Ctrl-F3":"findUnder","Shift-Ctrl-F3":"findUnderPrevious","Alt-F3":"findAllUnder","Shift-Ctrl-[":"fold","Shift-Ctrl-]":"unfold","Ctrl-I":"findIncremental","Shift-Ctrl-I":"findIncrementalReverse","Ctrl-H":"replace",F3:"findNext","Shift-F3":"findPrev",fallthrough:"pcDefault"},e.normalizeKeyMap(f.pcSublime);var p=f.default==f.macDefault;f.sublime=p?f.macSublime:f.pcSublime}(n(24),n(501),n(867))},function(e,t,n){"use strict";
/** @license React v17.0.1
 * react.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var r=n(365),i=60103,o=60106;t.Fragment=60107,t.StrictMode=60108,t.Profiler=60114;var a=60109,s=60110,l=60112;t.Suspense=60113;var u=60115,c=60116;if("function"==typeof Symbol&&Symbol.for){var d=Symbol.for;i=d("react.element"),o=d("react.portal"),t.Fragment=d("react.fragment"),t.StrictMode=d("react.strict_mode"),t.Profiler=d("react.profiler"),a=d("react.provider"),s=d("react.context"),l=d("react.forward_ref"),t.Suspense=d("react.suspense"),u=d("react.memo"),c=d("react.lazy")}var h="function"==typeof Symbol&&Symbol.iterator;function f(e){for(var t="https://reactjs.org/docs/error-decoder.html?invariant="+e,n=1;n<arguments.length;n++)t+="&args[]="+encodeURIComponent(arguments[n]);return"Minified React error #"+e+"; visit "+t+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}var p={isMounted:function(){return!1},enqueueForceUpdate:function(){},enqueueReplaceState:function(){},enqueueSetState:function(){}},g={};function v(e,t,n){this.props=e,this.context=t,this.refs=g,this.updater=n||p}function m(){}function y(e,t,n){this.props=e,this.context=t,this.refs=g,this.updater=n||p}v.prototype.isReactComponent={},v.prototype.setState=function(e,t){if("object"!=typeof e&&"function"!=typeof e&&null!=e)throw Error(f(85));this.updater.enqueueSetState(this,e,t,"setState")},v.prototype.forceUpdate=function(e){this.updater.enqueueForceUpdate(this,e,"forceUpdate")},m.prototype=v.prototype;var b=y.prototype=new m;b.constructor=y,r(b,v.prototype),b.isPureReactComponent=!0;var C={current:null},A=Object.prototype.hasOwnProperty,_={key:!0,ref:!0,__self:!0,__source:!0};function I(e,t,n){var r,o={},a=null,s=null;if(null!=t)for(r in void 0!==t.ref&&(s=t.ref),void 0!==t.key&&(a=""+t.key),t)A.call(t,r)&&!_.hasOwnProperty(r)&&(o[r]=t[r]);var l=arguments.length-2;if(1===l)o.children=n;else if(1<l){for(var u=Array(l),c=0;c<l;c++)u[c]=arguments[c+2];o.children=u}if(e&&e.defaultProps)for(r in l=e.defaultProps)void 0===o[r]&&(o[r]=l[r]);return{$$typeof:i,type:e,key:a,ref:s,props:o,_owner:C.current}}function x(e){return"object"==typeof e&&null!==e&&e.$$typeof===i}var w=/\/+/g;function M(e,t){return"object"==typeof e&&null!==e&&null!=e.key?function(e){var t={"=":"=0",":":"=2"};return"$"+e.replace(/[=:]/g,(function(e){return t[e]}))}(""+e.key):t.toString(36)}function S(e,t,n,r,a){var s=typeof e;"undefined"!==s&&"boolean"!==s||(e=null);var l=!1;if(null===e)l=!0;else switch(s){case"string":case"number":l=!0;break;case"object":switch(e.$$typeof){case i:case o:l=!0}}if(l)return a=a(l=e),e=""===r?"."+M(l,0):r,Array.isArray(a)?(n="",null!=e&&(n=e.replace(w,"$&/")+"/"),S(a,t,n,"",(function(e){return e}))):null!=a&&(x(a)&&(a=function(e,t){return{$$typeof:i,type:e.type,key:t,ref:e.ref,props:e.props,_owner:e._owner}}(a,n+(!a.key||l&&l.key===a.key?"":(""+a.key).replace(w,"$&/")+"/")+e)),t.push(a)),1;if(l=0,r=""===r?".":r+":",Array.isArray(e))for(var u=0;u<e.length;u++){var c=r+M(s=e[u],u);l+=S(s,t,n,c,a)}else if("function"==typeof(c=function(e){return null===e||"object"!=typeof e?null:"function"==typeof(e=h&&e[h]||e["@@iterator"])?e:null}(e)))for(e=c.call(e),u=0;!(s=e.next()).done;)l+=S(s=s.value,t,n,c=r+M(s,u++),a);else if("object"===s)throw t=""+e,Error(f(31,"[object Object]"===t?"object with keys {"+Object.keys(e).join(", ")+"}":t));return l}function T(e,t,n){if(null==e)return e;var r=[],i=0;return S(e,r,"","",(function(e){return t.call(n,e,i++)})),r}function E(e){if(-1===e._status){var t=e._result;t=t(),e._status=0,e._result=t,t.then((function(t){0===e._status&&(t=t.default,e._status=1,e._result=t)}),(function(t){0===e._status&&(e._status=2,e._result=t)}))}if(1===e._status)return e._result;throw e._result}var O={current:null};function k(){var e=O.current;if(null===e)throw Error(f(321));return e}var L={ReactCurrentDispatcher:O,ReactCurrentBatchConfig:{transition:0},ReactCurrentOwner:C,IsSomeRendererActing:{current:!1},assign:r};t.Children={map:T,forEach:function(e,t,n){T(e,(function(){t.apply(this,arguments)}),n)},count:function(e){var t=0;return T(e,(function(){t++})),t},toArray:function(e){return T(e,(function(e){return e}))||[]},only:function(e){if(!x(e))throw Error(f(143));return e}},t.Component=v,t.PureComponent=y,t.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED=L,t.cloneElement=function(e,t,n){if(null==e)throw Error(f(267,e));var o=r({},e.props),a=e.key,s=e.ref,l=e._owner;if(null!=t){if(void 0!==t.ref&&(s=t.ref,l=C.current),void 0!==t.key&&(a=""+t.key),e.type&&e.type.defaultProps)var u=e.type.defaultProps;for(c in t)A.call(t,c)&&!_.hasOwnProperty(c)&&(o[c]=void 0===t[c]&&void 0!==u?u[c]:t[c])}var c=arguments.length-2;if(1===c)o.children=n;else if(1<c){u=Array(c);for(var d=0;d<c;d++)u[d]=arguments[d+2];o.children=u}return{$$typeof:i,type:e.type,key:a,ref:s,props:o,_owner:l}},t.createContext=function(e,t){return void 0===t&&(t=null),(e={$$typeof:s,_calculateChangedBits:t,_currentValue:e,_currentValue2:e,_threadCount:0,Provider:null,Consumer:null}).Provider={$$typeof:a,_context:e},e.Consumer=e},t.createElement=I,t.createFactory=function(e){var t=I.bind(null,e);return t.type=e,t},t.createRef=function(){return{current:null}},t.forwardRef=function(e){return{$$typeof:l,render:e}},t.isValidElement=x,t.lazy=function(e){return{$$typeof:c,_payload:{_status:-1,_result:e},_init:E}},t.memo=function(e,t){return{$$typeof:u,type:e,compare:void 0===t?null:t}},t.useCallback=function(e,t){return k().useCallback(e,t)},t.useContext=function(e,t){return k().useContext(e,t)},t.useDebugValue=function(){},t.useEffect=function(e,t){return k().useEffect(e,t)},t.useImperativeHandle=function(e,t,n){return k().useImperativeHandle(e,t,n)},t.useLayoutEffect=function(e,t){return k().useLayoutEffect(e,t)},t.useMemo=function(e,t){return k().useMemo(e,t)},t.useReducer=function(e,t,n){return k().useReducer(e,t,n)},t.useRef=function(e){return k().useRef(e)},t.useState=function(e){return k().useState(e)},t.version="17.0.1"},function(e,t,n){"use strict";
/** @license React v17.0.1
 * react-dom.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var r=n(0),i=n(365),o=n(1452);function a(e){for(var t="https://reactjs.org/docs/error-decoder.html?invariant="+e,n=1;n<arguments.length;n++)t+="&args[]="+encodeURIComponent(arguments[n]);return"Minified React error #"+e+"; visit "+t+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}if(!r)throw Error(a(227));var s=new Set,l={};function u(e,t){c(e,t),c(e+"Capture",t)}function c(e,t){for(l[e]=t,e=0;e<t.length;e++)s.add(t[e])}var d=!("undefined"==typeof window||void 0===window.document||void 0===window.document.createElement),h=/^[:A-Z_a-z\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u02FF\u0370-\u037D\u037F-\u1FFF\u200C-\u200D\u2070-\u218F\u2C00-\u2FEF\u3001-\uD7FF\uF900-\uFDCF\uFDF0-\uFFFD][:A-Z_a-z\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u02FF\u0370-\u037D\u037F-\u1FFF\u200C-\u200D\u2070-\u218F\u2C00-\u2FEF\u3001-\uD7FF\uF900-\uFDCF\uFDF0-\uFFFD\-.0-9\u00B7\u0300-\u036F\u203F-\u2040]*$/,f=Object.prototype.hasOwnProperty,p={},g={};function v(e,t,n,r,i,o,a){this.acceptsBooleans=2===t||3===t||4===t,this.attributeName=r,this.attributeNamespace=i,this.mustUseProperty=n,this.propertyName=e,this.type=t,this.sanitizeURL=o,this.removeEmptyString=a}var m={};"children dangerouslySetInnerHTML defaultValue defaultChecked innerHTML suppressContentEditableWarning suppressHydrationWarning style".split(" ").forEach((function(e){m[e]=new v(e,0,!1,e,null,!1,!1)})),[["acceptCharset","accept-charset"],["className","class"],["htmlFor","for"],["httpEquiv","http-equiv"]].forEach((function(e){var t=e[0];m[t]=new v(t,1,!1,e[1],null,!1,!1)})),["contentEditable","draggable","spellCheck","value"].forEach((function(e){m[e]=new v(e,2,!1,e.toLowerCase(),null,!1,!1)})),["autoReverse","externalResourcesRequired","focusable","preserveAlpha"].forEach((function(e){m[e]=new v(e,2,!1,e,null,!1,!1)})),"allowFullScreen async autoFocus autoPlay controls default defer disabled disablePictureInPicture disableRemotePlayback formNoValidate hidden loop noModule noValidate open playsInline readOnly required reversed scoped seamless itemScope".split(" ").forEach((function(e){m[e]=new v(e,3,!1,e.toLowerCase(),null,!1,!1)})),["checked","multiple","muted","selected"].forEach((function(e){m[e]=new v(e,3,!0,e,null,!1,!1)})),["capture","download"].forEach((function(e){m[e]=new v(e,4,!1,e,null,!1,!1)})),["cols","rows","size","span"].forEach((function(e){m[e]=new v(e,6,!1,e,null,!1,!1)})),["rowSpan","start"].forEach((function(e){m[e]=new v(e,5,!1,e.toLowerCase(),null,!1,!1)}));var y=/[\-:]([a-z])/g;function b(e){return e[1].toUpperCase()}function C(e,t,n,r){var i=m.hasOwnProperty(t)?m[t]:null;(null!==i?0===i.type:!r&&(2<t.length&&("o"===t[0]||"O"===t[0])&&("n"===t[1]||"N"===t[1])))||(function(e,t,n,r){if(null==t||function(e,t,n,r){if(null!==n&&0===n.type)return!1;switch(typeof t){case"function":case"symbol":return!0;case"boolean":return!r&&(null!==n?!n.acceptsBooleans:"data-"!==(e=e.toLowerCase().slice(0,5))&&"aria-"!==e);default:return!1}}(e,t,n,r))return!0;if(r)return!1;if(null!==n)switch(n.type){case 3:return!t;case 4:return!1===t;case 5:return isNaN(t);case 6:return isNaN(t)||1>t}return!1}(t,n,i,r)&&(n=null),r||null===i?function(e){return!!f.call(g,e)||!f.call(p,e)&&(h.test(e)?g[e]=!0:(p[e]=!0,!1))}(t)&&(null===n?e.removeAttribute(t):e.setAttribute(t,""+n)):i.mustUseProperty?e[i.propertyName]=null===n?3!==i.type&&"":n:(t=i.attributeName,r=i.attributeNamespace,null===n?e.removeAttribute(t):(n=3===(i=i.type)||4===i&&!0===n?"":""+n,r?e.setAttributeNS(r,t,n):e.setAttribute(t,n))))}"accent-height alignment-baseline arabic-form baseline-shift cap-height clip-path clip-rule color-interpolation color-interpolation-filters color-profile color-rendering dominant-baseline enable-background fill-opacity fill-rule flood-color flood-opacity font-family font-size font-size-adjust font-stretch font-style font-variant font-weight glyph-name glyph-orientation-horizontal glyph-orientation-vertical horiz-adv-x horiz-origin-x image-rendering letter-spacing lighting-color marker-end marker-mid marker-start overline-position overline-thickness paint-order panose-1 pointer-events rendering-intent shape-rendering stop-color stop-opacity strikethrough-position strikethrough-thickness stroke-dasharray stroke-dashoffset stroke-linecap stroke-linejoin stroke-miterlimit stroke-opacity stroke-width text-anchor text-decoration text-rendering underline-position underline-thickness unicode-bidi unicode-range units-per-em v-alphabetic v-hanging v-ideographic v-mathematical vector-effect vert-adv-y vert-origin-x vert-origin-y word-spacing writing-mode xmlns:xlink x-height".split(" ").forEach((function(e){var t=e.replace(y,b);m[t]=new v(t,1,!1,e,null,!1,!1)})),"xlink:actuate xlink:arcrole xlink:role xlink:show xlink:title xlink:type".split(" ").forEach((function(e){var t=e.replace(y,b);m[t]=new v(t,1,!1,e,"http://www.w3.org/1999/xlink",!1,!1)})),["xml:base","xml:lang","xml:space"].forEach((function(e){var t=e.replace(y,b);m[t]=new v(t,1,!1,e,"http://www.w3.org/XML/1998/namespace",!1,!1)})),["tabIndex","crossOrigin"].forEach((function(e){m[e]=new v(e,1,!1,e.toLowerCase(),null,!1,!1)})),m.xlinkHref=new v("xlinkHref",1,!1,"xlink:href","http://www.w3.org/1999/xlink",!0,!1),["src","href","action","formAction"].forEach((function(e){m[e]=new v(e,1,!1,e.toLowerCase(),null,!0,!0)}));var A=r.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED,_=60103,I=60106,x=60107,w=60108,M=60114,S=60109,T=60110,E=60112,O=60113,k=60120,L=60115,D=60116,P=60121,N=60128,j=60129,R=60130,z=60131;if("function"==typeof Symbol&&Symbol.for){var F=Symbol.for;_=F("react.element"),I=F("react.portal"),x=F("react.fragment"),w=F("react.strict_mode"),M=F("react.profiler"),S=F("react.provider"),T=F("react.context"),E=F("react.forward_ref"),O=F("react.suspense"),k=F("react.suspense_list"),L=F("react.memo"),D=F("react.lazy"),P=F("react.block"),F("react.scope"),N=F("react.opaque.id"),j=F("react.debug_trace_mode"),R=F("react.offscreen"),z=F("react.legacy_hidden")}var B,H="function"==typeof Symbol&&Symbol.iterator;function U(e){return null===e||"object"!=typeof e?null:"function"==typeof(e=H&&e[H]||e["@@iterator"])?e:null}function V(e){if(void 0===B)try{throw Error()}catch(e){var t=e.stack.trim().match(/\n( *(at )?)/);B=t&&t[1]||""}return"\n"+B+e}var Y=!1;function W(e,t){if(!e||Y)return"";Y=!0;var n=Error.prepareStackTrace;Error.prepareStackTrace=void 0;try{if(t)if(t=function(){throw Error()},Object.defineProperty(t.prototype,"props",{set:function(){throw Error()}}),"object"==typeof Reflect&&Reflect.construct){try{Reflect.construct(t,[])}catch(e){var r=e}Reflect.construct(e,[],t)}else{try{t.call()}catch(e){r=e}e.call(t.prototype)}else{try{throw Error()}catch(e){r=e}e()}}catch(e){if(e&&r&&"string"==typeof e.stack){for(var i=e.stack.split("\n"),o=r.stack.split("\n"),a=i.length-1,s=o.length-1;1<=a&&0<=s&&i[a]!==o[s];)s--;for(;1<=a&&0<=s;a--,s--)if(i[a]!==o[s]){if(1!==a||1!==s)do{if(a--,0>--s||i[a]!==o[s])return"\n"+i[a].replace(" at new "," at ")}while(1<=a&&0<=s);break}}}finally{Y=!1,Error.prepareStackTrace=n}return(e=e?e.displayName||e.name:"")?V(e):""}function G(e){switch(e.tag){case 5:return V(e.type);case 16:return V("Lazy");case 13:return V("Suspense");case 19:return V("SuspenseList");case 0:case 2:case 15:return e=W(e.type,!1);case 11:return e=W(e.type.render,!1);case 22:return e=W(e.type._render,!1);case 1:return e=W(e.type,!0);default:return""}}function X(e){if(null==e)return null;if("function"==typeof e)return e.displayName||e.name||null;if("string"==typeof e)return e;switch(e){case x:return"Fragment";case I:return"Portal";case M:return"Profiler";case w:return"StrictMode";case O:return"Suspense";case k:return"SuspenseList"}if("object"==typeof e)switch(e.$$typeof){case T:return(e.displayName||"Context")+".Consumer";case S:return(e._context.displayName||"Context")+".Provider";case E:var t=e.render;return t=t.displayName||t.name||"",e.displayName||(""!==t?"ForwardRef("+t+")":"ForwardRef");case L:return X(e.type);case P:return X(e._render);case D:t=e._payload,e=e._init;try{return X(e(t))}catch(e){}}return null}function q(e){switch(typeof e){case"boolean":case"number":case"object":case"string":case"undefined":return e;default:return""}}function Z(e){var t=e.type;return(e=e.nodeName)&&"input"===e.toLowerCase()&&("checkbox"===t||"radio"===t)}function Q(e){e._valueTracker||(e._valueTracker=function(e){var t=Z(e)?"checked":"value",n=Object.getOwnPropertyDescriptor(e.constructor.prototype,t),r=""+e[t];if(!e.hasOwnProperty(t)&&void 0!==n&&"function"==typeof n.get&&"function"==typeof n.set){var i=n.get,o=n.set;return Object.defineProperty(e,t,{configurable:!0,get:function(){return i.call(this)},set:function(e){r=""+e,o.call(this,e)}}),Object.defineProperty(e,t,{enumerable:n.enumerable}),{getValue:function(){return r},setValue:function(e){r=""+e},stopTracking:function(){e._valueTracker=null,delete e[t]}}}}(e))}function K(e){if(!e)return!1;var t=e._valueTracker;if(!t)return!0;var n=t.getValue(),r="";return e&&(r=Z(e)?e.checked?"true":"false":e.value),(e=r)!==n&&(t.setValue(e),!0)}function J(e){if(void 0===(e=e||("undefined"!=typeof document?document:void 0)))return null;try{return e.activeElement||e.body}catch(t){return e.body}}function $(e,t){var n=t.checked;return i({},t,{defaultChecked:void 0,defaultValue:void 0,value:void 0,checked:null!=n?n:e._wrapperState.initialChecked})}function ee(e,t){var n=null==t.defaultValue?"":t.defaultValue,r=null!=t.checked?t.checked:t.defaultChecked;n=q(null!=t.value?t.value:n),e._wrapperState={initialChecked:r,initialValue:n,controlled:"checkbox"===t.type||"radio"===t.type?null!=t.checked:null!=t.value}}function te(e,t){null!=(t=t.checked)&&C(e,"checked",t,!1)}function ne(e,t){te(e,t);var n=q(t.value),r=t.type;if(null!=n)"number"===r?(0===n&&""===e.value||e.value!=n)&&(e.value=""+n):e.value!==""+n&&(e.value=""+n);else if("submit"===r||"reset"===r)return void e.removeAttribute("value");t.hasOwnProperty("value")?ie(e,t.type,n):t.hasOwnProperty("defaultValue")&&ie(e,t.type,q(t.defaultValue)),null==t.checked&&null!=t.defaultChecked&&(e.defaultChecked=!!t.defaultChecked)}function re(e,t,n){if(t.hasOwnProperty("value")||t.hasOwnProperty("defaultValue")){var r=t.type;if(!("submit"!==r&&"reset"!==r||void 0!==t.value&&null!==t.value))return;t=""+e._wrapperState.initialValue,n||t===e.value||(e.value=t),e.defaultValue=t}""!==(n=e.name)&&(e.name=""),e.defaultChecked=!!e._wrapperState.initialChecked,""!==n&&(e.name=n)}function ie(e,t,n){"number"===t&&J(e.ownerDocument)===e||(null==n?e.defaultValue=""+e._wrapperState.initialValue:e.defaultValue!==""+n&&(e.defaultValue=""+n))}function oe(e,t){return e=i({children:void 0},t),(t=function(e){var t="";return r.Children.forEach(e,(function(e){null!=e&&(t+=e)})),t}(t.children))&&(e.children=t),e}function ae(e,t,n,r){if(e=e.options,t){t={};for(var i=0;i<n.length;i++)t["$"+n[i]]=!0;for(n=0;n<e.length;n++)i=t.hasOwnProperty("$"+e[n].value),e[n].selected!==i&&(e[n].selected=i),i&&r&&(e[n].defaultSelected=!0)}else{for(n=""+q(n),t=null,i=0;i<e.length;i++){if(e[i].value===n)return e[i].selected=!0,void(r&&(e[i].defaultSelected=!0));null!==t||e[i].disabled||(t=e[i])}null!==t&&(t.selected=!0)}}function se(e,t){if(null!=t.dangerouslySetInnerHTML)throw Error(a(91));return i({},t,{value:void 0,defaultValue:void 0,children:""+e._wrapperState.initialValue})}function le(e,t){var n=t.value;if(null==n){if(n=t.children,t=t.defaultValue,null!=n){if(null!=t)throw Error(a(92));if(Array.isArray(n)){if(!(1>=n.length))throw Error(a(93));n=n[0]}t=n}null==t&&(t=""),n=t}e._wrapperState={initialValue:q(n)}}function ue(e,t){var n=q(t.value),r=q(t.defaultValue);null!=n&&((n=""+n)!==e.value&&(e.value=n),null==t.defaultValue&&e.defaultValue!==n&&(e.defaultValue=n)),null!=r&&(e.defaultValue=""+r)}function ce(e){var t=e.textContent;t===e._wrapperState.initialValue&&""!==t&&null!==t&&(e.value=t)}var de="http://www.w3.org/1999/xhtml",he="http://www.w3.org/2000/svg";function fe(e){switch(e){case"svg":return"http://www.w3.org/2000/svg";case"math":return"http://www.w3.org/1998/Math/MathML";default:return"http://www.w3.org/1999/xhtml"}}function pe(e,t){return null==e||"http://www.w3.org/1999/xhtml"===e?fe(t):"http://www.w3.org/2000/svg"===e&&"foreignObject"===t?"http://www.w3.org/1999/xhtml":e}var ge,ve=function(e){return"undefined"!=typeof MSApp&&MSApp.execUnsafeLocalFunction?function(t,n,r,i){MSApp.execUnsafeLocalFunction((function(){return e(t,n)}))}:e}((function(e,t){if(e.namespaceURI!==he||"innerHTML"in e)e.innerHTML=t;else{for((ge=ge||document.createElement("div")).innerHTML="<svg>"+t.valueOf().toString()+"</svg>",t=ge.firstChild;e.firstChild;)e.removeChild(e.firstChild);for(;t.firstChild;)e.appendChild(t.firstChild)}}));function me(e,t){if(t){var n=e.firstChild;if(n&&n===e.lastChild&&3===n.nodeType)return void(n.nodeValue=t)}e.textContent=t}var ye={animationIterationCount:!0,borderImageOutset:!0,borderImageSlice:!0,borderImageWidth:!0,boxFlex:!0,boxFlexGroup:!0,boxOrdinalGroup:!0,columnCount:!0,columns:!0,flex:!0,flexGrow:!0,flexPositive:!0,flexShrink:!0,flexNegative:!0,flexOrder:!0,gridArea:!0,gridRow:!0,gridRowEnd:!0,gridRowSpan:!0,gridRowStart:!0,gridColumn:!0,gridColumnEnd:!0,gridColumnSpan:!0,gridColumnStart:!0,fontWeight:!0,lineClamp:!0,lineHeight:!0,opacity:!0,order:!0,orphans:!0,tabSize:!0,widows:!0,zIndex:!0,zoom:!0,fillOpacity:!0,floodOpacity:!0,stopOpacity:!0,strokeDasharray:!0,strokeDashoffset:!0,strokeMiterlimit:!0,strokeOpacity:!0,strokeWidth:!0},be=["Webkit","ms","Moz","O"];function Ce(e,t,n){return null==t||"boolean"==typeof t||""===t?"":n||"number"!=typeof t||0===t||ye.hasOwnProperty(e)&&ye[e]?(""+t).trim():t+"px"}function Ae(e,t){for(var n in e=e.style,t)if(t.hasOwnProperty(n)){var r=0===n.indexOf("--"),i=Ce(n,t[n],r);"float"===n&&(n="cssFloat"),r?e.setProperty(n,i):e[n]=i}}Object.keys(ye).forEach((function(e){be.forEach((function(t){t=t+e.charAt(0).toUpperCase()+e.substring(1),ye[t]=ye[e]}))}));var _e=i({menuitem:!0},{area:!0,base:!0,br:!0,col:!0,embed:!0,hr:!0,img:!0,input:!0,keygen:!0,link:!0,meta:!0,param:!0,source:!0,track:!0,wbr:!0});function Ie(e,t){if(t){if(_e[e]&&(null!=t.children||null!=t.dangerouslySetInnerHTML))throw Error(a(137,e));if(null!=t.dangerouslySetInnerHTML){if(null!=t.children)throw Error(a(60));if("object"!=typeof t.dangerouslySetInnerHTML||!("__html"in t.dangerouslySetInnerHTML))throw Error(a(61))}if(null!=t.style&&"object"!=typeof t.style)throw Error(a(62))}}function xe(e,t){if(-1===e.indexOf("-"))return"string"==typeof t.is;switch(e){case"annotation-xml":case"color-profile":case"font-face":case"font-face-src":case"font-face-uri":case"font-face-format":case"font-face-name":case"missing-glyph":return!1;default:return!0}}function we(e){return(e=e.target||e.srcElement||window).correspondingUseElement&&(e=e.correspondingUseElement),3===e.nodeType?e.parentNode:e}var Me=null,Se=null,Te=null;function Ee(e){if(e=Jr(e)){if("function"!=typeof Me)throw Error(a(280));var t=e.stateNode;t&&(t=ei(t),Me(e.stateNode,e.type,t))}}function Oe(e){Se?Te?Te.push(e):Te=[e]:Se=e}function ke(){if(Se){var e=Se,t=Te;if(Te=Se=null,Ee(e),t)for(e=0;e<t.length;e++)Ee(t[e])}}function Le(e,t){return e(t)}function De(e,t,n,r,i){return e(t,n,r,i)}function Pe(){}var Ne=Le,je=!1,Re=!1;function ze(){null===Se&&null===Te||(Pe(),ke())}function Fe(e,t){var n=e.stateNode;if(null===n)return null;var r=ei(n);if(null===r)return null;n=r[t];e:switch(t){case"onClick":case"onClickCapture":case"onDoubleClick":case"onDoubleClickCapture":case"onMouseDown":case"onMouseDownCapture":case"onMouseMove":case"onMouseMoveCapture":case"onMouseUp":case"onMouseUpCapture":case"onMouseEnter":(r=!r.disabled)||(r=!("button"===(e=e.type)||"input"===e||"select"===e||"textarea"===e)),e=!r;break e;default:e=!1}if(e)return null;if(n&&"function"!=typeof n)throw Error(a(231,t,typeof n));return n}var Be=!1;if(d)try{var He={};Object.defineProperty(He,"passive",{get:function(){Be=!0}}),window.addEventListener("test",He,He),window.removeEventListener("test",He,He)}catch(e){Be=!1}function Ue(e,t,n,r,i,o,a,s,l){var u=Array.prototype.slice.call(arguments,3);try{t.apply(n,u)}catch(e){this.onError(e)}}var Ve=!1,Ye=null,We=!1,Ge=null,Xe={onError:function(e){Ve=!0,Ye=e}};function qe(e,t,n,r,i,o,a,s,l){Ve=!1,Ye=null,Ue.apply(Xe,arguments)}function Ze(e){var t=e,n=e;if(e.alternate)for(;t.return;)t=t.return;else{e=t;do{0!=(1026&(t=e).flags)&&(n=t.return),e=t.return}while(e)}return 3===t.tag?n:null}function Qe(e){if(13===e.tag){var t=e.memoizedState;if(null===t&&(null!==(e=e.alternate)&&(t=e.memoizedState)),null!==t)return t.dehydrated}return null}function Ke(e){if(Ze(e)!==e)throw Error(a(188))}function Je(e){if(!(e=function(e){var t=e.alternate;if(!t){if(null===(t=Ze(e)))throw Error(a(188));return t!==e?null:e}for(var n=e,r=t;;){var i=n.return;if(null===i)break;var o=i.alternate;if(null===o){if(null!==(r=i.return)){n=r;continue}break}if(i.child===o.child){for(o=i.child;o;){if(o===n)return Ke(i),e;if(o===r)return Ke(i),t;o=o.sibling}throw Error(a(188))}if(n.return!==r.return)n=i,r=o;else{for(var s=!1,l=i.child;l;){if(l===n){s=!0,n=i,r=o;break}if(l===r){s=!0,r=i,n=o;break}l=l.sibling}if(!s){for(l=o.child;l;){if(l===n){s=!0,n=o,r=i;break}if(l===r){s=!0,r=o,n=i;break}l=l.sibling}if(!s)throw Error(a(189))}}if(n.alternate!==r)throw Error(a(190))}if(3!==n.tag)throw Error(a(188));return n.stateNode.current===n?e:t}(e)))return null;for(var t=e;;){if(5===t.tag||6===t.tag)return t;if(t.child)t.child.return=t,t=t.child;else{if(t===e)break;for(;!t.sibling;){if(!t.return||t.return===e)return null;t=t.return}t.sibling.return=t.return,t=t.sibling}}return null}function $e(e,t){for(var n=e.alternate;null!==t;){if(t===e||t===n)return!0;t=t.return}return!1}var et,tt,nt,rt,it=!1,ot=[],at=null,st=null,lt=null,ut=new Map,ct=new Map,dt=[],ht="mousedown mouseup touchcancel touchend touchstart auxclick dblclick pointercancel pointerdown pointerup dragend dragstart drop compositionend compositionstart keydown keypress keyup input textInput copy cut paste click change contextmenu reset submit".split(" ");function ft(e,t,n,r,i){return{blockedOn:e,domEventName:t,eventSystemFlags:16|n,nativeEvent:i,targetContainers:[r]}}function pt(e,t){switch(e){case"focusin":case"focusout":at=null;break;case"dragenter":case"dragleave":st=null;break;case"mouseover":case"mouseout":lt=null;break;case"pointerover":case"pointerout":ut.delete(t.pointerId);break;case"gotpointercapture":case"lostpointercapture":ct.delete(t.pointerId)}}function gt(e,t,n,r,i,o){return null===e||e.nativeEvent!==o?(e=ft(t,n,r,i,o),null!==t&&(null!==(t=Jr(t))&&tt(t)),e):(e.eventSystemFlags|=r,t=e.targetContainers,null!==i&&-1===t.indexOf(i)&&t.push(i),e)}function vt(e){var t=Kr(e.target);if(null!==t){var n=Ze(t);if(null!==n)if(13===(t=n.tag)){if(null!==(t=Qe(n)))return e.blockedOn=t,void rt(e.lanePriority,(function(){o.unstable_runWithPriority(e.priority,(function(){nt(n)}))}))}else if(3===t&&n.stateNode.hydrate)return void(e.blockedOn=3===n.tag?n.stateNode.containerInfo:null)}e.blockedOn=null}function mt(e){if(null!==e.blockedOn)return!1;for(var t=e.targetContainers;0<t.length;){var n=Jt(e.domEventName,e.eventSystemFlags,t[0],e.nativeEvent);if(null!==n)return null!==(t=Jr(n))&&tt(t),e.blockedOn=n,!1;t.shift()}return!0}function yt(e,t,n){mt(e)&&n.delete(t)}function bt(){for(it=!1;0<ot.length;){var e=ot[0];if(null!==e.blockedOn){null!==(e=Jr(e.blockedOn))&&et(e);break}for(var t=e.targetContainers;0<t.length;){var n=Jt(e.domEventName,e.eventSystemFlags,t[0],e.nativeEvent);if(null!==n){e.blockedOn=n;break}t.shift()}null===e.blockedOn&&ot.shift()}null!==at&&mt(at)&&(at=null),null!==st&&mt(st)&&(st=null),null!==lt&&mt(lt)&&(lt=null),ut.forEach(yt),ct.forEach(yt)}function Ct(e,t){e.blockedOn===t&&(e.blockedOn=null,it||(it=!0,o.unstable_scheduleCallback(o.unstable_NormalPriority,bt)))}function At(e){function t(t){return Ct(t,e)}if(0<ot.length){Ct(ot[0],e);for(var n=1;n<ot.length;n++){var r=ot[n];r.blockedOn===e&&(r.blockedOn=null)}}for(null!==at&&Ct(at,e),null!==st&&Ct(st,e),null!==lt&&Ct(lt,e),ut.forEach(t),ct.forEach(t),n=0;n<dt.length;n++)(r=dt[n]).blockedOn===e&&(r.blockedOn=null);for(;0<dt.length&&null===(n=dt[0]).blockedOn;)vt(n),null===n.blockedOn&&dt.shift()}function _t(e,t){var n={};return n[e.toLowerCase()]=t.toLowerCase(),n["Webkit"+e]="webkit"+t,n["Moz"+e]="moz"+t,n}var It={animationend:_t("Animation","AnimationEnd"),animationiteration:_t("Animation","AnimationIteration"),animationstart:_t("Animation","AnimationStart"),transitionend:_t("Transition","TransitionEnd")},xt={},wt={};function Mt(e){if(xt[e])return xt[e];if(!It[e])return e;var t,n=It[e];for(t in n)if(n.hasOwnProperty(t)&&t in wt)return xt[e]=n[t];return e}d&&(wt=document.createElement("div").style,"AnimationEvent"in window||(delete It.animationend.animation,delete It.animationiteration.animation,delete It.animationstart.animation),"TransitionEvent"in window||delete It.transitionend.transition);var St=Mt("animationend"),Tt=Mt("animationiteration"),Et=Mt("animationstart"),Ot=Mt("transitionend"),kt=new Map,Lt=new Map,Dt=["abort","abort",St,"animationEnd",Tt,"animationIteration",Et,"animationStart","canplay","canPlay","canplaythrough","canPlayThrough","durationchange","durationChange","emptied","emptied","encrypted","encrypted","ended","ended","error","error","gotpointercapture","gotPointerCapture","load","load","loadeddata","loadedData","loadedmetadata","loadedMetadata","loadstart","loadStart","lostpointercapture","lostPointerCapture","playing","playing","progress","progress","seeking","seeking","stalled","stalled","suspend","suspend","timeupdate","timeUpdate",Ot,"transitionEnd","waiting","waiting"];function Pt(e,t){for(var n=0;n<e.length;n+=2){var r=e[n],i=e[n+1];i="on"+(i[0].toUpperCase()+i.slice(1)),Lt.set(r,t),kt.set(r,i),u(i,[r])}}(0,o.unstable_now)();var Nt=8;function jt(e){if(0!=(1&e))return Nt=15,1;if(0!=(2&e))return Nt=14,2;if(0!=(4&e))return Nt=13,4;var t=24&e;return 0!==t?(Nt=12,t):0!=(32&e)?(Nt=11,32):0!==(t=192&e)?(Nt=10,t):0!=(256&e)?(Nt=9,256):0!==(t=3584&e)?(Nt=8,t):0!=(4096&e)?(Nt=7,4096):0!==(t=4186112&e)?(Nt=6,t):0!==(t=62914560&e)?(Nt=5,t):67108864&e?(Nt=4,67108864):0!=(134217728&e)?(Nt=3,134217728):0!==(t=805306368&e)?(Nt=2,t):0!=(1073741824&e)?(Nt=1,1073741824):(Nt=8,e)}function Rt(e,t){var n=e.pendingLanes;if(0===n)return Nt=0;var r=0,i=0,o=e.expiredLanes,a=e.suspendedLanes,s=e.pingedLanes;if(0!==o)r=o,i=Nt=15;else if(0!==(o=134217727&n)){var l=o&~a;0!==l?(r=jt(l),i=Nt):0!==(s&=o)&&(r=jt(s),i=Nt)}else 0!==(o=n&~a)?(r=jt(o),i=Nt):0!==s&&(r=jt(s),i=Nt);if(0===r)return 0;if(r=n&((0>(r=31-Vt(r))?0:1<<r)<<1)-1,0!==t&&t!==r&&0==(t&a)){if(jt(t),i<=Nt)return t;Nt=i}if(0!==(t=e.entangledLanes))for(e=e.entanglements,t&=r;0<t;)i=1<<(n=31-Vt(t)),r|=e[n],t&=~i;return r}function zt(e){return 0!==(e=-1073741825&e.pendingLanes)?e:1073741824&e?1073741824:0}function Ft(e,t){switch(e){case 15:return 1;case 14:return 2;case 12:return 0===(e=Bt(24&~t))?Ft(10,t):e;case 10:return 0===(e=Bt(192&~t))?Ft(8,t):e;case 8:return 0===(e=Bt(3584&~t))&&(0===(e=Bt(4186112&~t))&&(e=512)),e;case 2:return 0===(t=Bt(805306368&~t))&&(t=268435456),t}throw Error(a(358,e))}function Bt(e){return e&-e}function Ht(e){for(var t=[],n=0;31>n;n++)t.push(e);return t}function Ut(e,t,n){e.pendingLanes|=t;var r=t-1;e.suspendedLanes&=r,e.pingedLanes&=r,(e=e.eventTimes)[t=31-Vt(t)]=n}var Vt=Math.clz32?Math.clz32:function(e){return 0===e?32:31-(Yt(e)/Wt|0)|0},Yt=Math.log,Wt=Math.LN2;var Gt=o.unstable_UserBlockingPriority,Xt=o.unstable_runWithPriority,qt=!0;function Zt(e,t,n,r){je||Pe();var i=Kt,o=je;je=!0;try{De(i,e,t,n,r)}finally{(je=o)||ze()}}function Qt(e,t,n,r){Xt(Gt,Kt.bind(null,e,t,n,r))}function Kt(e,t,n,r){var i;if(qt)if((i=0==(4&t))&&0<ot.length&&-1<ht.indexOf(e))e=ft(null,e,t,n,r),ot.push(e);else{var o=Jt(e,t,n,r);if(null===o)i&&pt(e,r);else{if(i){if(-1<ht.indexOf(e))return e=ft(o,e,t,n,r),void ot.push(e);if(function(e,t,n,r,i){switch(t){case"focusin":return at=gt(at,e,t,n,r,i),!0;case"dragenter":return st=gt(st,e,t,n,r,i),!0;case"mouseover":return lt=gt(lt,e,t,n,r,i),!0;case"pointerover":var o=i.pointerId;return ut.set(o,gt(ut.get(o)||null,e,t,n,r,i)),!0;case"gotpointercapture":return o=i.pointerId,ct.set(o,gt(ct.get(o)||null,e,t,n,r,i)),!0}return!1}(o,e,t,n,r))return;pt(e,r)}Or(e,t,r,null,n)}}}function Jt(e,t,n,r){var i=we(r);if(null!==(i=Kr(i))){var o=Ze(i);if(null===o)i=null;else{var a=o.tag;if(13===a){if(null!==(i=Qe(o)))return i;i=null}else if(3===a){if(o.stateNode.hydrate)return 3===o.tag?o.stateNode.containerInfo:null;i=null}else o!==i&&(i=null)}}return Or(e,t,r,i,n),null}var $t=null,en=null,tn=null;function nn(){if(tn)return tn;var e,t,n=en,r=n.length,i="value"in $t?$t.value:$t.textContent,o=i.length;for(e=0;e<r&&n[e]===i[e];e++);var a=r-e;for(t=1;t<=a&&n[r-t]===i[o-t];t++);return tn=i.slice(e,1<t?1-t:void 0)}function rn(e){var t=e.keyCode;return"charCode"in e?0===(e=e.charCode)&&13===t&&(e=13):e=t,10===e&&(e=13),32<=e||13===e?e:0}function on(){return!0}function an(){return!1}function sn(e){function t(t,n,r,i,o){for(var a in this._reactName=t,this._targetInst=r,this.type=n,this.nativeEvent=i,this.target=o,this.currentTarget=null,e)e.hasOwnProperty(a)&&(t=e[a],this[a]=t?t(i):i[a]);return this.isDefaultPrevented=(null!=i.defaultPrevented?i.defaultPrevented:!1===i.returnValue)?on:an,this.isPropagationStopped=an,this}return i(t.prototype,{preventDefault:function(){this.defaultPrevented=!0;var e=this.nativeEvent;e&&(e.preventDefault?e.preventDefault():"unknown"!=typeof e.returnValue&&(e.returnValue=!1),this.isDefaultPrevented=on)},stopPropagation:function(){var e=this.nativeEvent;e&&(e.stopPropagation?e.stopPropagation():"unknown"!=typeof e.cancelBubble&&(e.cancelBubble=!0),this.isPropagationStopped=on)},persist:function(){},isPersistent:on}),t}var ln,un,cn,dn={eventPhase:0,bubbles:0,cancelable:0,timeStamp:function(e){return e.timeStamp||Date.now()},defaultPrevented:0,isTrusted:0},hn=sn(dn),fn=i({},dn,{view:0,detail:0}),pn=sn(fn),gn=i({},fn,{screenX:0,screenY:0,clientX:0,clientY:0,pageX:0,pageY:0,ctrlKey:0,shiftKey:0,altKey:0,metaKey:0,getModifierState:Mn,button:0,buttons:0,relatedTarget:function(e){return void 0===e.relatedTarget?e.fromElement===e.srcElement?e.toElement:e.fromElement:e.relatedTarget},movementX:function(e){return"movementX"in e?e.movementX:(e!==cn&&(cn&&"mousemove"===e.type?(ln=e.screenX-cn.screenX,un=e.screenY-cn.screenY):un=ln=0,cn=e),ln)},movementY:function(e){return"movementY"in e?e.movementY:un}}),vn=sn(gn),mn=sn(i({},gn,{dataTransfer:0})),yn=sn(i({},fn,{relatedTarget:0})),bn=sn(i({},dn,{animationName:0,elapsedTime:0,pseudoElement:0})),Cn=sn(i({},dn,{clipboardData:function(e){return"clipboardData"in e?e.clipboardData:window.clipboardData}})),An=sn(i({},dn,{data:0})),_n={Esc:"Escape",Spacebar:" ",Left:"ArrowLeft",Up:"ArrowUp",Right:"ArrowRight",Down:"ArrowDown",Del:"Delete",Win:"OS",Menu:"ContextMenu",Apps:"ContextMenu",Scroll:"ScrollLock",MozPrintableKey:"Unidentified"},In={8:"Backspace",9:"Tab",12:"Clear",13:"Enter",16:"Shift",17:"Control",18:"Alt",19:"Pause",20:"CapsLock",27:"Escape",32:" ",33:"PageUp",34:"PageDown",35:"End",36:"Home",37:"ArrowLeft",38:"ArrowUp",39:"ArrowRight",40:"ArrowDown",45:"Insert",46:"Delete",112:"F1",113:"F2",114:"F3",115:"F4",116:"F5",117:"F6",118:"F7",119:"F8",120:"F9",121:"F10",122:"F11",123:"F12",144:"NumLock",145:"ScrollLock",224:"Meta"},xn={Alt:"altKey",Control:"ctrlKey",Meta:"metaKey",Shift:"shiftKey"};function wn(e){var t=this.nativeEvent;return t.getModifierState?t.getModifierState(e):!!(e=xn[e])&&!!t[e]}function Mn(){return wn}var Sn=sn(i({},fn,{key:function(e){if(e.key){var t=_n[e.key]||e.key;if("Unidentified"!==t)return t}return"keypress"===e.type?13===(e=rn(e))?"Enter":String.fromCharCode(e):"keydown"===e.type||"keyup"===e.type?In[e.keyCode]||"Unidentified":""},code:0,location:0,ctrlKey:0,shiftKey:0,altKey:0,metaKey:0,repeat:0,locale:0,getModifierState:Mn,charCode:function(e){return"keypress"===e.type?rn(e):0},keyCode:function(e){return"keydown"===e.type||"keyup"===e.type?e.keyCode:0},which:function(e){return"keypress"===e.type?rn(e):"keydown"===e.type||"keyup"===e.type?e.keyCode:0}})),Tn=sn(i({},gn,{pointerId:0,width:0,height:0,pressure:0,tangentialPressure:0,tiltX:0,tiltY:0,twist:0,pointerType:0,isPrimary:0})),En=sn(i({},fn,{touches:0,targetTouches:0,changedTouches:0,altKey:0,metaKey:0,ctrlKey:0,shiftKey:0,getModifierState:Mn})),On=sn(i({},dn,{propertyName:0,elapsedTime:0,pseudoElement:0})),kn=sn(i({},gn,{deltaX:function(e){return"deltaX"in e?e.deltaX:"wheelDeltaX"in e?-e.wheelDeltaX:0},deltaY:function(e){return"deltaY"in e?e.deltaY:"wheelDeltaY"in e?-e.wheelDeltaY:"wheelDelta"in e?-e.wheelDelta:0},deltaZ:0,deltaMode:0})),Ln=[9,13,27,32],Dn=d&&"CompositionEvent"in window,Pn=null;d&&"documentMode"in document&&(Pn=document.documentMode);var Nn=d&&"TextEvent"in window&&!Pn,jn=d&&(!Dn||Pn&&8<Pn&&11>=Pn),Rn=String.fromCharCode(32),zn=!1;function Fn(e,t){switch(e){case"keyup":return-1!==Ln.indexOf(t.keyCode);case"keydown":return 229!==t.keyCode;case"keypress":case"mousedown":case"focusout":return!0;default:return!1}}function Bn(e){return"object"==typeof(e=e.detail)&&"data"in e?e.data:null}var Hn=!1;var Un={color:!0,date:!0,datetime:!0,"datetime-local":!0,email:!0,month:!0,number:!0,password:!0,range:!0,search:!0,tel:!0,text:!0,time:!0,url:!0,week:!0};function Vn(e){var t=e&&e.nodeName&&e.nodeName.toLowerCase();return"input"===t?!!Un[e.type]:"textarea"===t}function Yn(e,t,n,r){Oe(r),0<(t=Lr(t,"onChange")).length&&(n=new hn("onChange","change",null,n,r),e.push({event:n,listeners:t}))}var Wn=null,Gn=null;function Xn(e){xr(e,0)}function qn(e){if(K($r(e)))return e}function Zn(e,t){if("change"===e)return t}var Qn=!1;if(d){var Kn;if(d){var Jn="oninput"in document;if(!Jn){var $n=document.createElement("div");$n.setAttribute("oninput","return;"),Jn="function"==typeof $n.oninput}Kn=Jn}else Kn=!1;Qn=Kn&&(!document.documentMode||9<document.documentMode)}function er(){Wn&&(Wn.detachEvent("onpropertychange",tr),Gn=Wn=null)}function tr(e){if("value"===e.propertyName&&qn(Gn)){var t=[];if(Yn(t,Gn,e,we(e)),e=Xn,je)e(t);else{je=!0;try{Le(e,t)}finally{je=!1,ze()}}}}function nr(e,t,n){"focusin"===e?(er(),Gn=n,(Wn=t).attachEvent("onpropertychange",tr)):"focusout"===e&&er()}function rr(e){if("selectionchange"===e||"keyup"===e||"keydown"===e)return qn(Gn)}function ir(e,t){if("click"===e)return qn(t)}function or(e,t){if("input"===e||"change"===e)return qn(t)}var ar="function"==typeof Object.is?Object.is:function(e,t){return e===t&&(0!==e||1/e==1/t)||e!=e&&t!=t},sr=Object.prototype.hasOwnProperty;function lr(e,t){if(ar(e,t))return!0;if("object"!=typeof e||null===e||"object"!=typeof t||null===t)return!1;var n=Object.keys(e),r=Object.keys(t);if(n.length!==r.length)return!1;for(r=0;r<n.length;r++)if(!sr.call(t,n[r])||!ar(e[n[r]],t[n[r]]))return!1;return!0}function ur(e){for(;e&&e.firstChild;)e=e.firstChild;return e}function cr(e,t){var n,r=ur(e);for(e=0;r;){if(3===r.nodeType){if(n=e+r.textContent.length,e<=t&&n>=t)return{node:r,offset:t-e};e=n}e:{for(;r;){if(r.nextSibling){r=r.nextSibling;break e}r=r.parentNode}r=void 0}r=ur(r)}}function dr(){for(var e=window,t=J();t instanceof e.HTMLIFrameElement;){try{var n="string"==typeof t.contentWindow.location.href}catch(e){n=!1}if(!n)break;t=J((e=t.contentWindow).document)}return t}function hr(e){var t=e&&e.nodeName&&e.nodeName.toLowerCase();return t&&("input"===t&&("text"===e.type||"search"===e.type||"tel"===e.type||"url"===e.type||"password"===e.type)||"textarea"===t||"true"===e.contentEditable)}var fr=d&&"documentMode"in document&&11>=document.documentMode,pr=null,gr=null,vr=null,mr=!1;function yr(e,t,n){var r=n.window===n?n.document:9===n.nodeType?n:n.ownerDocument;mr||null==pr||pr!==J(r)||("selectionStart"in(r=pr)&&hr(r)?r={start:r.selectionStart,end:r.selectionEnd}:r={anchorNode:(r=(r.ownerDocument&&r.ownerDocument.defaultView||window).getSelection()).anchorNode,anchorOffset:r.anchorOffset,focusNode:r.focusNode,focusOffset:r.focusOffset},vr&&lr(vr,r)||(vr=r,0<(r=Lr(gr,"onSelect")).length&&(t=new hn("onSelect","select",null,t,n),e.push({event:t,listeners:r}),t.target=pr)))}Pt("cancel cancel click click close close contextmenu contextMenu copy copy cut cut auxclick auxClick dblclick doubleClick dragend dragEnd dragstart dragStart drop drop focusin focus focusout blur input input invalid invalid keydown keyDown keypress keyPress keyup keyUp mousedown mouseDown mouseup mouseUp paste paste pause pause play play pointercancel pointerCancel pointerdown pointerDown pointerup pointerUp ratechange rateChange reset reset seeked seeked submit submit touchcancel touchCancel touchend touchEnd touchstart touchStart volumechange volumeChange".split(" "),0),Pt("drag drag dragenter dragEnter dragexit dragExit dragleave dragLeave dragover dragOver mousemove mouseMove mouseout mouseOut mouseover mouseOver pointermove pointerMove pointerout pointerOut pointerover pointerOver scroll scroll toggle toggle touchmove touchMove wheel wheel".split(" "),1),Pt(Dt,2);for(var br="change selectionchange textInput compositionstart compositionend compositionupdate".split(" "),Cr=0;Cr<br.length;Cr++)Lt.set(br[Cr],0);c("onMouseEnter",["mouseout","mouseover"]),c("onMouseLeave",["mouseout","mouseover"]),c("onPointerEnter",["pointerout","pointerover"]),c("onPointerLeave",["pointerout","pointerover"]),u("onChange","change click focusin focusout input keydown keyup selectionchange".split(" ")),u("onSelect","focusout contextmenu dragend focusin keydown keyup mousedown mouseup selectionchange".split(" ")),u("onBeforeInput",["compositionend","keypress","textInput","paste"]),u("onCompositionEnd","compositionend focusout keydown keypress keyup mousedown".split(" ")),u("onCompositionStart","compositionstart focusout keydown keypress keyup mousedown".split(" ")),u("onCompositionUpdate","compositionupdate focusout keydown keypress keyup mousedown".split(" "));var Ar="abort canplay canplaythrough durationchange emptied encrypted ended error loadeddata loadedmetadata loadstart pause play playing progress ratechange seeked seeking stalled suspend timeupdate volumechange waiting".split(" "),_r=new Set("cancel close invalid load scroll toggle".split(" ").concat(Ar));function Ir(e,t,n){var r=e.type||"unknown-event";e.currentTarget=n,function(e,t,n,r,i,o,s,l,u){if(qe.apply(this,arguments),Ve){if(!Ve)throw Error(a(198));var c=Ye;Ve=!1,Ye=null,We||(We=!0,Ge=c)}}(r,t,void 0,e),e.currentTarget=null}function xr(e,t){t=0!=(4&t);for(var n=0;n<e.length;n++){var r=e[n],i=r.event;r=r.listeners;e:{var o=void 0;if(t)for(var a=r.length-1;0<=a;a--){var s=r[a],l=s.instance,u=s.currentTarget;if(s=s.listener,l!==o&&i.isPropagationStopped())break e;Ir(i,s,u),o=l}else for(a=0;a<r.length;a++){if(l=(s=r[a]).instance,u=s.currentTarget,s=s.listener,l!==o&&i.isPropagationStopped())break e;Ir(i,s,u),o=l}}}if(We)throw e=Ge,We=!1,Ge=null,e}function wr(e,t){var n=ti(t),r=e+"__bubble";n.has(r)||(Er(t,e,2,!1),n.add(r))}var Mr="_reactListening"+Math.random().toString(36).slice(2);function Sr(e){e[Mr]||(e[Mr]=!0,s.forEach((function(t){_r.has(t)||Tr(t,!1,e,null),Tr(t,!0,e,null)})))}function Tr(e,t,n,r){var i=4<arguments.length&&void 0!==arguments[4]?arguments[4]:0,o=n;if("selectionchange"===e&&9!==n.nodeType&&(o=n.ownerDocument),null!==r&&!t&&_r.has(e)){if("scroll"!==e)return;i|=2,o=r}var a=ti(o),s=e+"__"+(t?"capture":"bubble");a.has(s)||(t&&(i|=4),Er(o,e,i,t),a.add(s))}function Er(e,t,n,r){var i=Lt.get(t);switch(void 0===i?2:i){case 0:i=Zt;break;case 1:i=Qt;break;default:i=Kt}n=i.bind(null,t,n,e),i=void 0,!Be||"touchstart"!==t&&"touchmove"!==t&&"wheel"!==t||(i=!0),r?void 0!==i?e.addEventListener(t,n,{capture:!0,passive:i}):e.addEventListener(t,n,!0):void 0!==i?e.addEventListener(t,n,{passive:i}):e.addEventListener(t,n,!1)}function Or(e,t,n,r,i){var o=r;if(0==(1&t)&&0==(2&t)&&null!==r)e:for(;;){if(null===r)return;var a=r.tag;if(3===a||4===a){var s=r.stateNode.containerInfo;if(s===i||8===s.nodeType&&s.parentNode===i)break;if(4===a)for(a=r.return;null!==a;){var l=a.tag;if((3===l||4===l)&&((l=a.stateNode.containerInfo)===i||8===l.nodeType&&l.parentNode===i))return;a=a.return}for(;null!==s;){if(null===(a=Kr(s)))return;if(5===(l=a.tag)||6===l){r=o=a;continue e}s=s.parentNode}}r=r.return}!function(e,t,n){if(Re)return e(t,n);Re=!0;try{Ne(e,t,n)}finally{Re=!1,ze()}}((function(){var r=o,i=we(n),a=[];e:{var s=kt.get(e);if(void 0!==s){var l=hn,u=e;switch(e){case"keypress":if(0===rn(n))break e;case"keydown":case"keyup":l=Sn;break;case"focusin":u="focus",l=yn;break;case"focusout":u="blur",l=yn;break;case"beforeblur":case"afterblur":l=yn;break;case"click":if(2===n.button)break e;case"auxclick":case"dblclick":case"mousedown":case"mousemove":case"mouseup":case"mouseout":case"mouseover":case"contextmenu":l=vn;break;case"drag":case"dragend":case"dragenter":case"dragexit":case"dragleave":case"dragover":case"dragstart":case"drop":l=mn;break;case"touchcancel":case"touchend":case"touchmove":case"touchstart":l=En;break;case St:case Tt:case Et:l=bn;break;case Ot:l=On;break;case"scroll":l=pn;break;case"wheel":l=kn;break;case"copy":case"cut":case"paste":l=Cn;break;case"gotpointercapture":case"lostpointercapture":case"pointercancel":case"pointerdown":case"pointermove":case"pointerout":case"pointerover":case"pointerup":l=Tn}var c=0!=(4&t),d=!c&&"scroll"===e,h=c?null!==s?s+"Capture":null:s;c=[];for(var f,p=r;null!==p;){var g=(f=p).stateNode;if(5===f.tag&&null!==g&&(f=g,null!==h&&(null!=(g=Fe(p,h))&&c.push(kr(p,g,f)))),d)break;p=p.return}0<c.length&&(s=new l(s,u,null,n,i),a.push({event:s,listeners:c}))}}if(0==(7&t)){if(l="mouseout"===e||"pointerout"===e,(!(s="mouseover"===e||"pointerover"===e)||0!=(16&t)||!(u=n.relatedTarget||n.fromElement)||!Kr(u)&&!u[Zr])&&(l||s)&&(s=i.window===i?i:(s=i.ownerDocument)?s.defaultView||s.parentWindow:window,l?(l=r,null!==(u=(u=n.relatedTarget||n.toElement)?Kr(u):null)&&(u!==(d=Ze(u))||5!==u.tag&&6!==u.tag)&&(u=null)):(l=null,u=r),l!==u)){if(c=vn,g="onMouseLeave",h="onMouseEnter",p="mouse","pointerout"!==e&&"pointerover"!==e||(c=Tn,g="onPointerLeave",h="onPointerEnter",p="pointer"),d=null==l?s:$r(l),f=null==u?s:$r(u),(s=new c(g,p+"leave",l,n,i)).target=d,s.relatedTarget=f,g=null,Kr(i)===r&&((c=new c(h,p+"enter",u,n,i)).target=f,c.relatedTarget=d,g=c),d=g,l&&u)e:{for(h=u,p=0,f=c=l;f;f=Dr(f))p++;for(f=0,g=h;g;g=Dr(g))f++;for(;0<p-f;)c=Dr(c),p--;for(;0<f-p;)h=Dr(h),f--;for(;p--;){if(c===h||null!==h&&c===h.alternate)break e;c=Dr(c),h=Dr(h)}c=null}else c=null;null!==l&&Pr(a,s,l,c,!1),null!==u&&null!==d&&Pr(a,d,u,c,!0)}if("select"===(l=(s=r?$r(r):window).nodeName&&s.nodeName.toLowerCase())||"input"===l&&"file"===s.type)var v=Zn;else if(Vn(s))if(Qn)v=or;else{v=rr;var m=nr}else(l=s.nodeName)&&"input"===l.toLowerCase()&&("checkbox"===s.type||"radio"===s.type)&&(v=ir);switch(v&&(v=v(e,r))?Yn(a,v,n,i):(m&&m(e,s,r),"focusout"===e&&(m=s._wrapperState)&&m.controlled&&"number"===s.type&&ie(s,"number",s.value)),m=r?$r(r):window,e){case"focusin":(Vn(m)||"true"===m.contentEditable)&&(pr=m,gr=r,vr=null);break;case"focusout":vr=gr=pr=null;break;case"mousedown":mr=!0;break;case"contextmenu":case"mouseup":case"dragend":mr=!1,yr(a,n,i);break;case"selectionchange":if(fr)break;case"keydown":case"keyup":yr(a,n,i)}var y;if(Dn)e:{switch(e){case"compositionstart":var b="onCompositionStart";break e;case"compositionend":b="onCompositionEnd";break e;case"compositionupdate":b="onCompositionUpdate";break e}b=void 0}else Hn?Fn(e,n)&&(b="onCompositionEnd"):"keydown"===e&&229===n.keyCode&&(b="onCompositionStart");b&&(jn&&"ko"!==n.locale&&(Hn||"onCompositionStart"!==b?"onCompositionEnd"===b&&Hn&&(y=nn()):(en="value"in($t=i)?$t.value:$t.textContent,Hn=!0)),0<(m=Lr(r,b)).length&&(b=new An(b,e,null,n,i),a.push({event:b,listeners:m}),y?b.data=y:null!==(y=Bn(n))&&(b.data=y))),(y=Nn?function(e,t){switch(e){case"compositionend":return Bn(t);case"keypress":return 32!==t.which?null:(zn=!0,Rn);case"textInput":return(e=t.data)===Rn&&zn?null:e;default:return null}}(e,n):function(e,t){if(Hn)return"compositionend"===e||!Dn&&Fn(e,t)?(e=nn(),tn=en=$t=null,Hn=!1,e):null;switch(e){case"paste":return null;case"keypress":if(!(t.ctrlKey||t.altKey||t.metaKey)||t.ctrlKey&&t.altKey){if(t.char&&1<t.char.length)return t.char;if(t.which)return String.fromCharCode(t.which)}return null;case"compositionend":return jn&&"ko"!==t.locale?null:t.data;default:return null}}(e,n))&&(0<(r=Lr(r,"onBeforeInput")).length&&(i=new An("onBeforeInput","beforeinput",null,n,i),a.push({event:i,listeners:r}),i.data=y))}xr(a,t)}))}function kr(e,t,n){return{instance:e,listener:t,currentTarget:n}}function Lr(e,t){for(var n=t+"Capture",r=[];null!==e;){var i=e,o=i.stateNode;5===i.tag&&null!==o&&(i=o,null!=(o=Fe(e,n))&&r.unshift(kr(e,o,i)),null!=(o=Fe(e,t))&&r.push(kr(e,o,i))),e=e.return}return r}function Dr(e){if(null===e)return null;do{e=e.return}while(e&&5!==e.tag);return e||null}function Pr(e,t,n,r,i){for(var o=t._reactName,a=[];null!==n&&n!==r;){var s=n,l=s.alternate,u=s.stateNode;if(null!==l&&l===r)break;5===s.tag&&null!==u&&(s=u,i?null!=(l=Fe(n,o))&&a.unshift(kr(n,l,s)):i||null!=(l=Fe(n,o))&&a.push(kr(n,l,s))),n=n.return}0!==a.length&&e.push({event:t,listeners:a})}function Nr(){}var jr=null,Rr=null;function zr(e,t){switch(e){case"button":case"input":case"select":case"textarea":return!!t.autoFocus}return!1}function Fr(e,t){return"textarea"===e||"option"===e||"noscript"===e||"string"==typeof t.children||"number"==typeof t.children||"object"==typeof t.dangerouslySetInnerHTML&&null!==t.dangerouslySetInnerHTML&&null!=t.dangerouslySetInnerHTML.__html}var Br="function"==typeof setTimeout?setTimeout:void 0,Hr="function"==typeof clearTimeout?clearTimeout:void 0;function Ur(e){1===e.nodeType?e.textContent="":9===e.nodeType&&(null!=(e=e.body)&&(e.textContent=""))}function Vr(e){for(;null!=e;e=e.nextSibling){var t=e.nodeType;if(1===t||3===t)break}return e}function Yr(e){e=e.previousSibling;for(var t=0;e;){if(8===e.nodeType){var n=e.data;if("$"===n||"$!"===n||"$?"===n){if(0===t)return e;t--}else"/$"===n&&t++}e=e.previousSibling}return null}var Wr=0;var Gr=Math.random().toString(36).slice(2),Xr="__reactFiber$"+Gr,qr="__reactProps$"+Gr,Zr="__reactContainer$"+Gr,Qr="__reactEvents$"+Gr;function Kr(e){var t=e[Xr];if(t)return t;for(var n=e.parentNode;n;){if(t=n[Zr]||n[Xr]){if(n=t.alternate,null!==t.child||null!==n&&null!==n.child)for(e=Yr(e);null!==e;){if(n=e[Xr])return n;e=Yr(e)}return t}n=(e=n).parentNode}return null}function Jr(e){return!(e=e[Xr]||e[Zr])||5!==e.tag&&6!==e.tag&&13!==e.tag&&3!==e.tag?null:e}function $r(e){if(5===e.tag||6===e.tag)return e.stateNode;throw Error(a(33))}function ei(e){return e[qr]||null}function ti(e){var t=e[Qr];return void 0===t&&(t=e[Qr]=new Set),t}var ni=[],ri=-1;function ii(e){return{current:e}}function oi(e){0>ri||(e.current=ni[ri],ni[ri]=null,ri--)}function ai(e,t){ri++,ni[ri]=e.current,e.current=t}var si={},li=ii(si),ui=ii(!1),ci=si;function di(e,t){var n=e.type.contextTypes;if(!n)return si;var r=e.stateNode;if(r&&r.__reactInternalMemoizedUnmaskedChildContext===t)return r.__reactInternalMemoizedMaskedChildContext;var i,o={};for(i in n)o[i]=t[i];return r&&((e=e.stateNode).__reactInternalMemoizedUnmaskedChildContext=t,e.__reactInternalMemoizedMaskedChildContext=o),o}function hi(e){return null!=(e=e.childContextTypes)}function fi(){oi(ui),oi(li)}function pi(e,t,n){if(li.current!==si)throw Error(a(168));ai(li,t),ai(ui,n)}function gi(e,t,n){var r=e.stateNode;if(e=t.childContextTypes,"function"!=typeof r.getChildContext)return n;for(var o in r=r.getChildContext())if(!(o in e))throw Error(a(108,X(t)||"Unknown",o));return i({},n,r)}function vi(e){return e=(e=e.stateNode)&&e.__reactInternalMemoizedMergedChildContext||si,ci=li.current,ai(li,e),ai(ui,ui.current),!0}function mi(e,t,n){var r=e.stateNode;if(!r)throw Error(a(169));n?(e=gi(e,t,ci),r.__reactInternalMemoizedMergedChildContext=e,oi(ui),oi(li),ai(li,e)):oi(ui),ai(ui,n)}var yi=null,bi=null,Ci=o.unstable_runWithPriority,Ai=o.unstable_scheduleCallback,_i=o.unstable_cancelCallback,Ii=o.unstable_shouldYield,xi=o.unstable_requestPaint,wi=o.unstable_now,Mi=o.unstable_getCurrentPriorityLevel,Si=o.unstable_ImmediatePriority,Ti=o.unstable_UserBlockingPriority,Ei=o.unstable_NormalPriority,Oi=o.unstable_LowPriority,ki=o.unstable_IdlePriority,Li={},Di=void 0!==xi?xi:function(){},Pi=null,Ni=null,ji=!1,Ri=wi(),zi=1e4>Ri?wi:function(){return wi()-Ri};function Fi(){switch(Mi()){case Si:return 99;case Ti:return 98;case Ei:return 97;case Oi:return 96;case ki:return 95;default:throw Error(a(332))}}function Bi(e){switch(e){case 99:return Si;case 98:return Ti;case 97:return Ei;case 96:return Oi;case 95:return ki;default:throw Error(a(332))}}function Hi(e,t){return e=Bi(e),Ci(e,t)}function Ui(e,t,n){return e=Bi(e),Ai(e,t,n)}function Vi(){if(null!==Ni){var e=Ni;Ni=null,_i(e)}Yi()}function Yi(){if(!ji&&null!==Pi){ji=!0;var e=0;try{var t=Pi;Hi(99,(function(){for(;e<t.length;e++){var n=t[e];do{n=n(!0)}while(null!==n)}})),Pi=null}catch(t){throw null!==Pi&&(Pi=Pi.slice(e+1)),Ai(Si,Vi),t}finally{ji=!1}}}var Wi=A.ReactCurrentBatchConfig;function Gi(e,t){if(e&&e.defaultProps){for(var n in t=i({},t),e=e.defaultProps)void 0===t[n]&&(t[n]=e[n]);return t}return t}var Xi=ii(null),qi=null,Zi=null,Qi=null;function Ki(){Qi=Zi=qi=null}function Ji(e){var t=Xi.current;oi(Xi),e.type._context._currentValue=t}function $i(e,t){for(;null!==e;){var n=e.alternate;if((e.childLanes&t)===t){if(null===n||(n.childLanes&t)===t)break;n.childLanes|=t}else e.childLanes|=t,null!==n&&(n.childLanes|=t);e=e.return}}function eo(e,t){qi=e,Qi=Zi=null,null!==(e=e.dependencies)&&null!==e.firstContext&&(0!=(e.lanes&t)&&(La=!0),e.firstContext=null)}function to(e,t){if(Qi!==e&&!1!==t&&0!==t)if("number"==typeof t&&1073741823!==t||(Qi=e,t=1073741823),t={context:e,observedBits:t,next:null},null===Zi){if(null===qi)throw Error(a(308));Zi=t,qi.dependencies={lanes:0,firstContext:t,responders:null}}else Zi=Zi.next=t;return e._currentValue}var no=!1;function ro(e){e.updateQueue={baseState:e.memoizedState,firstBaseUpdate:null,lastBaseUpdate:null,shared:{pending:null},effects:null}}function io(e,t){e=e.updateQueue,t.updateQueue===e&&(t.updateQueue={baseState:e.baseState,firstBaseUpdate:e.firstBaseUpdate,lastBaseUpdate:e.lastBaseUpdate,shared:e.shared,effects:e.effects})}function oo(e,t){return{eventTime:e,lane:t,tag:0,payload:null,callback:null,next:null}}function ao(e,t){if(null!==(e=e.updateQueue)){var n=(e=e.shared).pending;null===n?t.next=t:(t.next=n.next,n.next=t),e.pending=t}}function so(e,t){var n=e.updateQueue,r=e.alternate;if(null!==r&&n===(r=r.updateQueue)){var i=null,o=null;if(null!==(n=n.firstBaseUpdate)){do{var a={eventTime:n.eventTime,lane:n.lane,tag:n.tag,payload:n.payload,callback:n.callback,next:null};null===o?i=o=a:o=o.next=a,n=n.next}while(null!==n);null===o?i=o=t:o=o.next=t}else i=o=t;return n={baseState:r.baseState,firstBaseUpdate:i,lastBaseUpdate:o,shared:r.shared,effects:r.effects},void(e.updateQueue=n)}null===(e=n.lastBaseUpdate)?n.firstBaseUpdate=t:e.next=t,n.lastBaseUpdate=t}function lo(e,t,n,r){var o=e.updateQueue;no=!1;var a=o.firstBaseUpdate,s=o.lastBaseUpdate,l=o.shared.pending;if(null!==l){o.shared.pending=null;var u=l,c=u.next;u.next=null,null===s?a=c:s.next=c,s=u;var d=e.alternate;if(null!==d){var h=(d=d.updateQueue).lastBaseUpdate;h!==s&&(null===h?d.firstBaseUpdate=c:h.next=c,d.lastBaseUpdate=u)}}if(null!==a){for(h=o.baseState,s=0,d=c=u=null;;){l=a.lane;var f=a.eventTime;if((r&l)===l){null!==d&&(d=d.next={eventTime:f,lane:0,tag:a.tag,payload:a.payload,callback:a.callback,next:null});e:{var p=e,g=a;switch(l=t,f=n,g.tag){case 1:if("function"==typeof(p=g.payload)){h=p.call(f,h,l);break e}h=p;break e;case 3:p.flags=-4097&p.flags|64;case 0:if(null==(l="function"==typeof(p=g.payload)?p.call(f,h,l):p))break e;h=i({},h,l);break e;case 2:no=!0}}null!==a.callback&&(e.flags|=32,null===(l=o.effects)?o.effects=[a]:l.push(a))}else f={eventTime:f,lane:l,tag:a.tag,payload:a.payload,callback:a.callback,next:null},null===d?(c=d=f,u=h):d=d.next=f,s|=l;if(null===(a=a.next)){if(null===(l=o.shared.pending))break;a=l.next,l.next=null,o.lastBaseUpdate=l,o.shared.pending=null}}null===d&&(u=h),o.baseState=u,o.firstBaseUpdate=c,o.lastBaseUpdate=d,Ps|=s,e.lanes=s,e.memoizedState=h}}function uo(e,t,n){if(e=t.effects,t.effects=null,null!==e)for(t=0;t<e.length;t++){var r=e[t],i=r.callback;if(null!==i){if(r.callback=null,r=n,"function"!=typeof i)throw Error(a(191,i));i.call(r)}}}var co=(new r.Component).refs;function ho(e,t,n,r){n=null==(n=n(r,t=e.memoizedState))?t:i({},t,n),e.memoizedState=n,0===e.lanes&&(e.updateQueue.baseState=n)}var fo={isMounted:function(e){return!!(e=e._reactInternals)&&Ze(e)===e},enqueueSetState:function(e,t,n){e=e._reactInternals;var r=ol(),i=al(e),o=oo(r,i);o.payload=t,null!=n&&(o.callback=n),ao(e,o),sl(e,i,r)},enqueueReplaceState:function(e,t,n){e=e._reactInternals;var r=ol(),i=al(e),o=oo(r,i);o.tag=1,o.payload=t,null!=n&&(o.callback=n),ao(e,o),sl(e,i,r)},enqueueForceUpdate:function(e,t){e=e._reactInternals;var n=ol(),r=al(e),i=oo(n,r);i.tag=2,null!=t&&(i.callback=t),ao(e,i),sl(e,r,n)}};function po(e,t,n,r,i,o,a){return"function"==typeof(e=e.stateNode).shouldComponentUpdate?e.shouldComponentUpdate(r,o,a):!t.prototype||!t.prototype.isPureReactComponent||(!lr(n,r)||!lr(i,o))}function go(e,t,n){var r=!1,i=si,o=t.contextType;return"object"==typeof o&&null!==o?o=to(o):(i=hi(t)?ci:li.current,o=(r=null!=(r=t.contextTypes))?di(e,i):si),t=new t(n,o),e.memoizedState=null!==t.state&&void 0!==t.state?t.state:null,t.updater=fo,e.stateNode=t,t._reactInternals=e,r&&((e=e.stateNode).__reactInternalMemoizedUnmaskedChildContext=i,e.__reactInternalMemoizedMaskedChildContext=o),t}function vo(e,t,n,r){e=t.state,"function"==typeof t.componentWillReceiveProps&&t.componentWillReceiveProps(n,r),"function"==typeof t.UNSAFE_componentWillReceiveProps&&t.UNSAFE_componentWillReceiveProps(n,r),t.state!==e&&fo.enqueueReplaceState(t,t.state,null)}function mo(e,t,n,r){var i=e.stateNode;i.props=n,i.state=e.memoizedState,i.refs=co,ro(e);var o=t.contextType;"object"==typeof o&&null!==o?i.context=to(o):(o=hi(t)?ci:li.current,i.context=di(e,o)),lo(e,n,i,r),i.state=e.memoizedState,"function"==typeof(o=t.getDerivedStateFromProps)&&(ho(e,t,o,n),i.state=e.memoizedState),"function"==typeof t.getDerivedStateFromProps||"function"==typeof i.getSnapshotBeforeUpdate||"function"!=typeof i.UNSAFE_componentWillMount&&"function"!=typeof i.componentWillMount||(t=i.state,"function"==typeof i.componentWillMount&&i.componentWillMount(),"function"==typeof i.UNSAFE_componentWillMount&&i.UNSAFE_componentWillMount(),t!==i.state&&fo.enqueueReplaceState(i,i.state,null),lo(e,n,i,r),i.state=e.memoizedState),"function"==typeof i.componentDidMount&&(e.flags|=4)}var yo=Array.isArray;function bo(e,t,n){if(null!==(e=n.ref)&&"function"!=typeof e&&"object"!=typeof e){if(n._owner){if(n=n._owner){if(1!==n.tag)throw Error(a(309));var r=n.stateNode}if(!r)throw Error(a(147,e));var i=""+e;return null!==t&&null!==t.ref&&"function"==typeof t.ref&&t.ref._stringRef===i?t.ref:((t=function(e){var t=r.refs;t===co&&(t=r.refs={}),null===e?delete t[i]:t[i]=e})._stringRef=i,t)}if("string"!=typeof e)throw Error(a(284));if(!n._owner)throw Error(a(290,e))}return e}function Co(e,t){if("textarea"!==e.type)throw Error(a(31,"[object Object]"===Object.prototype.toString.call(t)?"object with keys {"+Object.keys(t).join(", ")+"}":t))}function Ao(e){function t(t,n){if(e){var r=t.lastEffect;null!==r?(r.nextEffect=n,t.lastEffect=n):t.firstEffect=t.lastEffect=n,n.nextEffect=null,n.flags=8}}function n(n,r){if(!e)return null;for(;null!==r;)t(n,r),r=r.sibling;return null}function r(e,t){for(e=new Map;null!==t;)null!==t.key?e.set(t.key,t):e.set(t.index,t),t=t.sibling;return e}function i(e,t){return(e=Fl(e,t)).index=0,e.sibling=null,e}function o(t,n,r){return t.index=r,e?null!==(r=t.alternate)?(r=r.index)<n?(t.flags=2,n):r:(t.flags=2,n):n}function s(t){return e&&null===t.alternate&&(t.flags=2),t}function l(e,t,n,r){return null===t||6!==t.tag?((t=Vl(n,e.mode,r)).return=e,t):((t=i(t,n)).return=e,t)}function u(e,t,n,r){return null!==t&&t.elementType===n.type?((r=i(t,n.props)).ref=bo(e,t,n),r.return=e,r):((r=Bl(n.type,n.key,n.props,null,e.mode,r)).ref=bo(e,t,n),r.return=e,r)}function c(e,t,n,r){return null===t||4!==t.tag||t.stateNode.containerInfo!==n.containerInfo||t.stateNode.implementation!==n.implementation?((t=Yl(n,e.mode,r)).return=e,t):((t=i(t,n.children||[])).return=e,t)}function d(e,t,n,r,o){return null===t||7!==t.tag?((t=Hl(n,e.mode,r,o)).return=e,t):((t=i(t,n)).return=e,t)}function h(e,t,n){if("string"==typeof t||"number"==typeof t)return(t=Vl(""+t,e.mode,n)).return=e,t;if("object"==typeof t&&null!==t){switch(t.$$typeof){case _:return(n=Bl(t.type,t.key,t.props,null,e.mode,n)).ref=bo(e,null,t),n.return=e,n;case I:return(t=Yl(t,e.mode,n)).return=e,t}if(yo(t)||U(t))return(t=Hl(t,e.mode,n,null)).return=e,t;Co(e,t)}return null}function f(e,t,n,r){var i=null!==t?t.key:null;if("string"==typeof n||"number"==typeof n)return null!==i?null:l(e,t,""+n,r);if("object"==typeof n&&null!==n){switch(n.$$typeof){case _:return n.key===i?n.type===x?d(e,t,n.props.children,r,i):u(e,t,n,r):null;case I:return n.key===i?c(e,t,n,r):null}if(yo(n)||U(n))return null!==i?null:d(e,t,n,r,null);Co(e,n)}return null}function p(e,t,n,r,i){if("string"==typeof r||"number"==typeof r)return l(t,e=e.get(n)||null,""+r,i);if("object"==typeof r&&null!==r){switch(r.$$typeof){case _:return e=e.get(null===r.key?n:r.key)||null,r.type===x?d(t,e,r.props.children,i,r.key):u(t,e,r,i);case I:return c(t,e=e.get(null===r.key?n:r.key)||null,r,i)}if(yo(r)||U(r))return d(t,e=e.get(n)||null,r,i,null);Co(t,r)}return null}function g(i,a,s,l){for(var u=null,c=null,d=a,g=a=0,v=null;null!==d&&g<s.length;g++){d.index>g?(v=d,d=null):v=d.sibling;var m=f(i,d,s[g],l);if(null===m){null===d&&(d=v);break}e&&d&&null===m.alternate&&t(i,d),a=o(m,a,g),null===c?u=m:c.sibling=m,c=m,d=v}if(g===s.length)return n(i,d),u;if(null===d){for(;g<s.length;g++)null!==(d=h(i,s[g],l))&&(a=o(d,a,g),null===c?u=d:c.sibling=d,c=d);return u}for(d=r(i,d);g<s.length;g++)null!==(v=p(d,i,g,s[g],l))&&(e&&null!==v.alternate&&d.delete(null===v.key?g:v.key),a=o(v,a,g),null===c?u=v:c.sibling=v,c=v);return e&&d.forEach((function(e){return t(i,e)})),u}function v(i,s,l,u){var c=U(l);if("function"!=typeof c)throw Error(a(150));if(null==(l=c.call(l)))throw Error(a(151));for(var d=c=null,g=s,v=s=0,m=null,y=l.next();null!==g&&!y.done;v++,y=l.next()){g.index>v?(m=g,g=null):m=g.sibling;var b=f(i,g,y.value,u);if(null===b){null===g&&(g=m);break}e&&g&&null===b.alternate&&t(i,g),s=o(b,s,v),null===d?c=b:d.sibling=b,d=b,g=m}if(y.done)return n(i,g),c;if(null===g){for(;!y.done;v++,y=l.next())null!==(y=h(i,y.value,u))&&(s=o(y,s,v),null===d?c=y:d.sibling=y,d=y);return c}for(g=r(i,g);!y.done;v++,y=l.next())null!==(y=p(g,i,v,y.value,u))&&(e&&null!==y.alternate&&g.delete(null===y.key?v:y.key),s=o(y,s,v),null===d?c=y:d.sibling=y,d=y);return e&&g.forEach((function(e){return t(i,e)})),c}return function(e,r,o,l){var u="object"==typeof o&&null!==o&&o.type===x&&null===o.key;u&&(o=o.props.children);var c="object"==typeof o&&null!==o;if(c)switch(o.$$typeof){case _:e:{for(c=o.key,u=r;null!==u;){if(u.key===c){switch(u.tag){case 7:if(o.type===x){n(e,u.sibling),(r=i(u,o.props.children)).return=e,e=r;break e}break;default:if(u.elementType===o.type){n(e,u.sibling),(r=i(u,o.props)).ref=bo(e,u,o),r.return=e,e=r;break e}}n(e,u);break}t(e,u),u=u.sibling}o.type===x?((r=Hl(o.props.children,e.mode,l,o.key)).return=e,e=r):((l=Bl(o.type,o.key,o.props,null,e.mode,l)).ref=bo(e,r,o),l.return=e,e=l)}return s(e);case I:e:{for(u=o.key;null!==r;){if(r.key===u){if(4===r.tag&&r.stateNode.containerInfo===o.containerInfo&&r.stateNode.implementation===o.implementation){n(e,r.sibling),(r=i(r,o.children||[])).return=e,e=r;break e}n(e,r);break}t(e,r),r=r.sibling}(r=Yl(o,e.mode,l)).return=e,e=r}return s(e)}if("string"==typeof o||"number"==typeof o)return o=""+o,null!==r&&6===r.tag?(n(e,r.sibling),(r=i(r,o)).return=e,e=r):(n(e,r),(r=Vl(o,e.mode,l)).return=e,e=r),s(e);if(yo(o))return g(e,r,o,l);if(U(o))return v(e,r,o,l);if(c&&Co(e,o),void 0===o&&!u)switch(e.tag){case 1:case 22:case 0:case 11:case 15:throw Error(a(152,X(e.type)||"Component"))}return n(e,r)}}var _o=Ao(!0),Io=Ao(!1),xo={},wo=ii(xo),Mo=ii(xo),So=ii(xo);function To(e){if(e===xo)throw Error(a(174));return e}function Eo(e,t){switch(ai(So,t),ai(Mo,e),ai(wo,xo),e=t.nodeType){case 9:case 11:t=(t=t.documentElement)?t.namespaceURI:pe(null,"");break;default:t=pe(t=(e=8===e?t.parentNode:t).namespaceURI||null,e=e.tagName)}oi(wo),ai(wo,t)}function Oo(){oi(wo),oi(Mo),oi(So)}function ko(e){To(So.current);var t=To(wo.current),n=pe(t,e.type);t!==n&&(ai(Mo,e),ai(wo,n))}function Lo(e){Mo.current===e&&(oi(wo),oi(Mo))}var Do=ii(0);function Po(e){for(var t=e;null!==t;){if(13===t.tag){var n=t.memoizedState;if(null!==n&&(null===(n=n.dehydrated)||"$?"===n.data||"$!"===n.data))return t}else if(19===t.tag&&void 0!==t.memoizedProps.revealOrder){if(0!=(64&t.flags))return t}else if(null!==t.child){t.child.return=t,t=t.child;continue}if(t===e)break;for(;null===t.sibling;){if(null===t.return||t.return===e)return null;t=t.return}t.sibling.return=t.return,t=t.sibling}return null}var No=null,jo=null,Ro=!1;function zo(e,t){var n=Rl(5,null,null,0);n.elementType="DELETED",n.type="DELETED",n.stateNode=t,n.return=e,n.flags=8,null!==e.lastEffect?(e.lastEffect.nextEffect=n,e.lastEffect=n):e.firstEffect=e.lastEffect=n}function Fo(e,t){switch(e.tag){case 5:var n=e.type;return null!==(t=1!==t.nodeType||n.toLowerCase()!==t.nodeName.toLowerCase()?null:t)&&(e.stateNode=t,!0);case 6:return null!==(t=""===e.pendingProps||3!==t.nodeType?null:t)&&(e.stateNode=t,!0);case 13:default:return!1}}function Bo(e){if(Ro){var t=jo;if(t){var n=t;if(!Fo(e,t)){if(!(t=Vr(n.nextSibling))||!Fo(e,t))return e.flags=-1025&e.flags|2,Ro=!1,void(No=e);zo(No,n)}No=e,jo=Vr(t.firstChild)}else e.flags=-1025&e.flags|2,Ro=!1,No=e}}function Ho(e){for(e=e.return;null!==e&&5!==e.tag&&3!==e.tag&&13!==e.tag;)e=e.return;No=e}function Uo(e){if(e!==No)return!1;if(!Ro)return Ho(e),Ro=!0,!1;var t=e.type;if(5!==e.tag||"head"!==t&&"body"!==t&&!Fr(t,e.memoizedProps))for(t=jo;t;)zo(e,t),t=Vr(t.nextSibling);if(Ho(e),13===e.tag){if(!(e=null!==(e=e.memoizedState)?e.dehydrated:null))throw Error(a(317));e:{for(e=e.nextSibling,t=0;e;){if(8===e.nodeType){var n=e.data;if("/$"===n){if(0===t){jo=Vr(e.nextSibling);break e}t--}else"$"!==n&&"$!"!==n&&"$?"!==n||t++}e=e.nextSibling}jo=null}}else jo=No?Vr(e.stateNode.nextSibling):null;return!0}function Vo(){jo=No=null,Ro=!1}var Yo=[];function Wo(){for(var e=0;e<Yo.length;e++)Yo[e]._workInProgressVersionPrimary=null;Yo.length=0}var Go=A.ReactCurrentDispatcher,Xo=A.ReactCurrentBatchConfig,qo=0,Zo=null,Qo=null,Ko=null,Jo=!1,$o=!1;function ea(){throw Error(a(321))}function ta(e,t){if(null===t)return!1;for(var n=0;n<t.length&&n<e.length;n++)if(!ar(e[n],t[n]))return!1;return!0}function na(e,t,n,r,i,o){if(qo=o,Zo=t,t.memoizedState=null,t.updateQueue=null,t.lanes=0,Go.current=null===e||null===e.memoizedState?Ta:Ea,e=n(r,i),$o){o=0;do{if($o=!1,!(25>o))throw Error(a(301));o+=1,Ko=Qo=null,t.updateQueue=null,Go.current=Oa,e=n(r,i)}while($o)}if(Go.current=Sa,t=null!==Qo&&null!==Qo.next,qo=0,Ko=Qo=Zo=null,Jo=!1,t)throw Error(a(300));return e}function ra(){var e={memoizedState:null,baseState:null,baseQueue:null,queue:null,next:null};return null===Ko?Zo.memoizedState=Ko=e:Ko=Ko.next=e,Ko}function ia(){if(null===Qo){var e=Zo.alternate;e=null!==e?e.memoizedState:null}else e=Qo.next;var t=null===Ko?Zo.memoizedState:Ko.next;if(null!==t)Ko=t,Qo=e;else{if(null===e)throw Error(a(310));e={memoizedState:(Qo=e).memoizedState,baseState:Qo.baseState,baseQueue:Qo.baseQueue,queue:Qo.queue,next:null},null===Ko?Zo.memoizedState=Ko=e:Ko=Ko.next=e}return Ko}function oa(e,t){return"function"==typeof t?t(e):t}function aa(e){var t=ia(),n=t.queue;if(null===n)throw Error(a(311));n.lastRenderedReducer=e;var r=Qo,i=r.baseQueue,o=n.pending;if(null!==o){if(null!==i){var s=i.next;i.next=o.next,o.next=s}r.baseQueue=i=o,n.pending=null}if(null!==i){i=i.next,r=r.baseState;var l=s=o=null,u=i;do{var c=u.lane;if((qo&c)===c)null!==l&&(l=l.next={lane:0,action:u.action,eagerReducer:u.eagerReducer,eagerState:u.eagerState,next:null}),r=u.eagerReducer===e?u.eagerState:e(r,u.action);else{var d={lane:c,action:u.action,eagerReducer:u.eagerReducer,eagerState:u.eagerState,next:null};null===l?(s=l=d,o=r):l=l.next=d,Zo.lanes|=c,Ps|=c}u=u.next}while(null!==u&&u!==i);null===l?o=r:l.next=s,ar(r,t.memoizedState)||(La=!0),t.memoizedState=r,t.baseState=o,t.baseQueue=l,n.lastRenderedState=r}return[t.memoizedState,n.dispatch]}function sa(e){var t=ia(),n=t.queue;if(null===n)throw Error(a(311));n.lastRenderedReducer=e;var r=n.dispatch,i=n.pending,o=t.memoizedState;if(null!==i){n.pending=null;var s=i=i.next;do{o=e(o,s.action),s=s.next}while(s!==i);ar(o,t.memoizedState)||(La=!0),t.memoizedState=o,null===t.baseQueue&&(t.baseState=o),n.lastRenderedState=o}return[o,r]}function la(e,t,n){var r=t._getVersion;r=r(t._source);var i=t._workInProgressVersionPrimary;if(null!==i?e=i===r:(e=e.mutableReadLanes,(e=(qo&e)===e)&&(t._workInProgressVersionPrimary=r,Yo.push(t))),e)return n(t._source);throw Yo.push(t),Error(a(350))}function ua(e,t,n,r){var i=Ms;if(null===i)throw Error(a(349));var o=t._getVersion,s=o(t._source),l=Go.current,u=l.useState((function(){return la(i,t,n)})),c=u[1],d=u[0];u=Ko;var h=e.memoizedState,f=h.refs,p=f.getSnapshot,g=h.source;h=h.subscribe;var v=Zo;return e.memoizedState={refs:f,source:t,subscribe:r},l.useEffect((function(){f.getSnapshot=n,f.setSnapshot=c;var e=o(t._source);if(!ar(s,e)){e=n(t._source),ar(d,e)||(c(e),e=al(v),i.mutableReadLanes|=e&i.pendingLanes),e=i.mutableReadLanes,i.entangledLanes|=e;for(var r=i.entanglements,a=e;0<a;){var l=31-Vt(a),u=1<<l;r[l]|=e,a&=~u}}}),[n,t,r]),l.useEffect((function(){return r(t._source,(function(){var e=f.getSnapshot,n=f.setSnapshot;try{n(e(t._source));var r=al(v);i.mutableReadLanes|=r&i.pendingLanes}catch(e){n((function(){throw e}))}}))}),[t,r]),ar(p,n)&&ar(g,t)&&ar(h,r)||((e={pending:null,dispatch:null,lastRenderedReducer:oa,lastRenderedState:d}).dispatch=c=Ma.bind(null,Zo,e),u.queue=e,u.baseQueue=null,d=la(i,t,n),u.memoizedState=u.baseState=d),d}function ca(e,t,n){return ua(ia(),e,t,n)}function da(e){var t=ra();return"function"==typeof e&&(e=e()),t.memoizedState=t.baseState=e,e=(e=t.queue={pending:null,dispatch:null,lastRenderedReducer:oa,lastRenderedState:e}).dispatch=Ma.bind(null,Zo,e),[t.memoizedState,e]}function ha(e,t,n,r){return e={tag:e,create:t,destroy:n,deps:r,next:null},null===(t=Zo.updateQueue)?(t={lastEffect:null},Zo.updateQueue=t,t.lastEffect=e.next=e):null===(n=t.lastEffect)?t.lastEffect=e.next=e:(r=n.next,n.next=e,e.next=r,t.lastEffect=e),e}function fa(e){return e={current:e},ra().memoizedState=e}function pa(){return ia().memoizedState}function ga(e,t,n,r){var i=ra();Zo.flags|=e,i.memoizedState=ha(1|t,n,void 0,void 0===r?null:r)}function va(e,t,n,r){var i=ia();r=void 0===r?null:r;var o=void 0;if(null!==Qo){var a=Qo.memoizedState;if(o=a.destroy,null!==r&&ta(r,a.deps))return void ha(t,n,o,r)}Zo.flags|=e,i.memoizedState=ha(1|t,n,o,r)}function ma(e,t){return ga(516,4,e,t)}function ya(e,t){return va(516,4,e,t)}function ba(e,t){return va(4,2,e,t)}function Ca(e,t){return"function"==typeof t?(e=e(),t(e),function(){t(null)}):null!=t?(e=e(),t.current=e,function(){t.current=null}):void 0}function Aa(e,t,n){return n=null!=n?n.concat([e]):null,va(4,2,Ca.bind(null,t,e),n)}function _a(){}function Ia(e,t){var n=ia();t=void 0===t?null:t;var r=n.memoizedState;return null!==r&&null!==t&&ta(t,r[1])?r[0]:(n.memoizedState=[e,t],e)}function xa(e,t){var n=ia();t=void 0===t?null:t;var r=n.memoizedState;return null!==r&&null!==t&&ta(t,r[1])?r[0]:(e=e(),n.memoizedState=[e,t],e)}function wa(e,t){var n=Fi();Hi(98>n?98:n,(function(){e(!0)})),Hi(97<n?97:n,(function(){var n=Xo.transition;Xo.transition=1;try{e(!1),t()}finally{Xo.transition=n}}))}function Ma(e,t,n){var r=ol(),i=al(e),o={lane:i,action:n,eagerReducer:null,eagerState:null,next:null},a=t.pending;if(null===a?o.next=o:(o.next=a.next,a.next=o),t.pending=o,a=e.alternate,e===Zo||null!==a&&a===Zo)$o=Jo=!0;else{if(0===e.lanes&&(null===a||0===a.lanes)&&null!==(a=t.lastRenderedReducer))try{var s=t.lastRenderedState,l=a(s,n);if(o.eagerReducer=a,o.eagerState=l,ar(l,s))return}catch(e){}sl(e,i,r)}}var Sa={readContext:to,useCallback:ea,useContext:ea,useEffect:ea,useImperativeHandle:ea,useLayoutEffect:ea,useMemo:ea,useReducer:ea,useRef:ea,useState:ea,useDebugValue:ea,useDeferredValue:ea,useTransition:ea,useMutableSource:ea,useOpaqueIdentifier:ea,unstable_isNewReconciler:!1},Ta={readContext:to,useCallback:function(e,t){return ra().memoizedState=[e,void 0===t?null:t],e},useContext:to,useEffect:ma,useImperativeHandle:function(e,t,n){return n=null!=n?n.concat([e]):null,ga(4,2,Ca.bind(null,t,e),n)},useLayoutEffect:function(e,t){return ga(4,2,e,t)},useMemo:function(e,t){var n=ra();return t=void 0===t?null:t,e=e(),n.memoizedState=[e,t],e},useReducer:function(e,t,n){var r=ra();return t=void 0!==n?n(t):t,r.memoizedState=r.baseState=t,e=(e=r.queue={pending:null,dispatch:null,lastRenderedReducer:e,lastRenderedState:t}).dispatch=Ma.bind(null,Zo,e),[r.memoizedState,e]},useRef:fa,useState:da,useDebugValue:_a,useDeferredValue:function(e){var t=da(e),n=t[0],r=t[1];return ma((function(){var t=Xo.transition;Xo.transition=1;try{r(e)}finally{Xo.transition=t}}),[e]),n},useTransition:function(){var e=da(!1),t=e[0];return fa(e=wa.bind(null,e[1])),[e,t]},useMutableSource:function(e,t,n){var r=ra();return r.memoizedState={refs:{getSnapshot:t,setSnapshot:null},source:e,subscribe:n},ua(r,e,t,n)},useOpaqueIdentifier:function(){if(Ro){var e=!1,t=function(e){return{$$typeof:N,toString:e,valueOf:e}}((function(){throw e||(e=!0,n("r:"+(Wr++).toString(36))),Error(a(355))})),n=da(t)[1];return 0==(2&Zo.mode)&&(Zo.flags|=516,ha(5,(function(){n("r:"+(Wr++).toString(36))}),void 0,null)),t}return da(t="r:"+(Wr++).toString(36)),t},unstable_isNewReconciler:!1},Ea={readContext:to,useCallback:Ia,useContext:to,useEffect:ya,useImperativeHandle:Aa,useLayoutEffect:ba,useMemo:xa,useReducer:aa,useRef:pa,useState:function(){return aa(oa)},useDebugValue:_a,useDeferredValue:function(e){var t=aa(oa),n=t[0],r=t[1];return ya((function(){var t=Xo.transition;Xo.transition=1;try{r(e)}finally{Xo.transition=t}}),[e]),n},useTransition:function(){var e=aa(oa)[0];return[pa().current,e]},useMutableSource:ca,useOpaqueIdentifier:function(){return aa(oa)[0]},unstable_isNewReconciler:!1},Oa={readContext:to,useCallback:Ia,useContext:to,useEffect:ya,useImperativeHandle:Aa,useLayoutEffect:ba,useMemo:xa,useReducer:sa,useRef:pa,useState:function(){return sa(oa)},useDebugValue:_a,useDeferredValue:function(e){var t=sa(oa),n=t[0],r=t[1];return ya((function(){var t=Xo.transition;Xo.transition=1;try{r(e)}finally{Xo.transition=t}}),[e]),n},useTransition:function(){var e=sa(oa)[0];return[pa().current,e]},useMutableSource:ca,useOpaqueIdentifier:function(){return sa(oa)[0]},unstable_isNewReconciler:!1},ka=A.ReactCurrentOwner,La=!1;function Da(e,t,n,r){t.child=null===e?Io(t,null,n,r):_o(t,e.child,n,r)}function Pa(e,t,n,r,i){n=n.render;var o=t.ref;return eo(t,i),r=na(e,t,n,r,o,i),null===e||La?(t.flags|=1,Da(e,t,r,i),t.child):(t.updateQueue=e.updateQueue,t.flags&=-517,e.lanes&=~i,es(e,t,i))}function Na(e,t,n,r,i,o){if(null===e){var a=n.type;return"function"!=typeof a||zl(a)||void 0!==a.defaultProps||null!==n.compare||void 0!==n.defaultProps?((e=Bl(n.type,null,r,t,t.mode,o)).ref=t.ref,e.return=t,t.child=e):(t.tag=15,t.type=a,ja(e,t,a,r,i,o))}return a=e.child,0==(i&o)&&(i=a.memoizedProps,(n=null!==(n=n.compare)?n:lr)(i,r)&&e.ref===t.ref)?es(e,t,o):(t.flags|=1,(e=Fl(a,r)).ref=t.ref,e.return=t,t.child=e)}function ja(e,t,n,r,i,o){if(null!==e&&lr(e.memoizedProps,r)&&e.ref===t.ref){if(La=!1,0==(o&i))return t.lanes=e.lanes,es(e,t,o);0!=(16384&e.flags)&&(La=!0)}return Fa(e,t,n,r,o)}function Ra(e,t,n){var r=t.pendingProps,i=r.children,o=null!==e?e.memoizedState:null;if("hidden"===r.mode||"unstable-defer-without-hiding"===r.mode)if(0==(4&t.mode))t.memoizedState={baseLanes:0},gl(t,n);else{if(0==(1073741824&n))return e=null!==o?o.baseLanes|n:n,t.lanes=t.childLanes=1073741824,t.memoizedState={baseLanes:e},gl(t,e),null;t.memoizedState={baseLanes:0},gl(t,null!==o?o.baseLanes:n)}else null!==o?(r=o.baseLanes|n,t.memoizedState=null):r=n,gl(t,r);return Da(e,t,i,n),t.child}function za(e,t){var n=t.ref;(null===e&&null!==n||null!==e&&e.ref!==n)&&(t.flags|=128)}function Fa(e,t,n,r,i){var o=hi(n)?ci:li.current;return o=di(t,o),eo(t,i),n=na(e,t,n,r,o,i),null===e||La?(t.flags|=1,Da(e,t,n,i),t.child):(t.updateQueue=e.updateQueue,t.flags&=-517,e.lanes&=~i,es(e,t,i))}function Ba(e,t,n,r,i){if(hi(n)){var o=!0;vi(t)}else o=!1;if(eo(t,i),null===t.stateNode)null!==e&&(e.alternate=null,t.alternate=null,t.flags|=2),go(t,n,r),mo(t,n,r,i),r=!0;else if(null===e){var a=t.stateNode,s=t.memoizedProps;a.props=s;var l=a.context,u=n.contextType;"object"==typeof u&&null!==u?u=to(u):u=di(t,u=hi(n)?ci:li.current);var c=n.getDerivedStateFromProps,d="function"==typeof c||"function"==typeof a.getSnapshotBeforeUpdate;d||"function"!=typeof a.UNSAFE_componentWillReceiveProps&&"function"!=typeof a.componentWillReceiveProps||(s!==r||l!==u)&&vo(t,a,r,u),no=!1;var h=t.memoizedState;a.state=h,lo(t,r,a,i),l=t.memoizedState,s!==r||h!==l||ui.current||no?("function"==typeof c&&(ho(t,n,c,r),l=t.memoizedState),(s=no||po(t,n,s,r,h,l,u))?(d||"function"!=typeof a.UNSAFE_componentWillMount&&"function"!=typeof a.componentWillMount||("function"==typeof a.componentWillMount&&a.componentWillMount(),"function"==typeof a.UNSAFE_componentWillMount&&a.UNSAFE_componentWillMount()),"function"==typeof a.componentDidMount&&(t.flags|=4)):("function"==typeof a.componentDidMount&&(t.flags|=4),t.memoizedProps=r,t.memoizedState=l),a.props=r,a.state=l,a.context=u,r=s):("function"==typeof a.componentDidMount&&(t.flags|=4),r=!1)}else{a=t.stateNode,io(e,t),s=t.memoizedProps,u=t.type===t.elementType?s:Gi(t.type,s),a.props=u,d=t.pendingProps,h=a.context,"object"==typeof(l=n.contextType)&&null!==l?l=to(l):l=di(t,l=hi(n)?ci:li.current);var f=n.getDerivedStateFromProps;(c="function"==typeof f||"function"==typeof a.getSnapshotBeforeUpdate)||"function"!=typeof a.UNSAFE_componentWillReceiveProps&&"function"!=typeof a.componentWillReceiveProps||(s!==d||h!==l)&&vo(t,a,r,l),no=!1,h=t.memoizedState,a.state=h,lo(t,r,a,i);var p=t.memoizedState;s!==d||h!==p||ui.current||no?("function"==typeof f&&(ho(t,n,f,r),p=t.memoizedState),(u=no||po(t,n,u,r,h,p,l))?(c||"function"!=typeof a.UNSAFE_componentWillUpdate&&"function"!=typeof a.componentWillUpdate||("function"==typeof a.componentWillUpdate&&a.componentWillUpdate(r,p,l),"function"==typeof a.UNSAFE_componentWillUpdate&&a.UNSAFE_componentWillUpdate(r,p,l)),"function"==typeof a.componentDidUpdate&&(t.flags|=4),"function"==typeof a.getSnapshotBeforeUpdate&&(t.flags|=256)):("function"!=typeof a.componentDidUpdate||s===e.memoizedProps&&h===e.memoizedState||(t.flags|=4),"function"!=typeof a.getSnapshotBeforeUpdate||s===e.memoizedProps&&h===e.memoizedState||(t.flags|=256),t.memoizedProps=r,t.memoizedState=p),a.props=r,a.state=p,a.context=l,r=u):("function"!=typeof a.componentDidUpdate||s===e.memoizedProps&&h===e.memoizedState||(t.flags|=4),"function"!=typeof a.getSnapshotBeforeUpdate||s===e.memoizedProps&&h===e.memoizedState||(t.flags|=256),r=!1)}return Ha(e,t,n,r,o,i)}function Ha(e,t,n,r,i,o){za(e,t);var a=0!=(64&t.flags);if(!r&&!a)return i&&mi(t,n,!1),es(e,t,o);r=t.stateNode,ka.current=t;var s=a&&"function"!=typeof n.getDerivedStateFromError?null:r.render();return t.flags|=1,null!==e&&a?(t.child=_o(t,e.child,null,o),t.child=_o(t,null,s,o)):Da(e,t,s,o),t.memoizedState=r.state,i&&mi(t,n,!0),t.child}function Ua(e){var t=e.stateNode;t.pendingContext?pi(0,t.pendingContext,t.pendingContext!==t.context):t.context&&pi(0,t.context,!1),Eo(e,t.containerInfo)}var Va,Ya,Wa,Ga={dehydrated:null,retryLane:0};function Xa(e,t,n){var r,i=t.pendingProps,o=Do.current,a=!1;return(r=0!=(64&t.flags))||(r=(null===e||null!==e.memoizedState)&&0!=(2&o)),r?(a=!0,t.flags&=-65):null!==e&&null===e.memoizedState||void 0===i.fallback||!0===i.unstable_avoidThisFallback||(o|=1),ai(Do,1&o),null===e?(void 0!==i.fallback&&Bo(t),e=i.children,o=i.fallback,a?(e=qa(t,e,o,n),t.child.memoizedState={baseLanes:n},t.memoizedState=Ga,e):"number"==typeof i.unstable_expectedLoadTime?(e=qa(t,e,o,n),t.child.memoizedState={baseLanes:n},t.memoizedState=Ga,t.lanes=33554432,e):((n=Ul({mode:"visible",children:e},t.mode,n,null)).return=t,t.child=n)):(e.memoizedState,a?(i=Qa(e,t,i.children,i.fallback,n),a=t.child,o=e.child.memoizedState,a.memoizedState=null===o?{baseLanes:n}:{baseLanes:o.baseLanes|n},a.childLanes=e.childLanes&~n,t.memoizedState=Ga,i):(n=Za(e,t,i.children,n),t.memoizedState=null,n))}function qa(e,t,n,r){var i=e.mode,o=e.child;return t={mode:"hidden",children:t},0==(2&i)&&null!==o?(o.childLanes=0,o.pendingProps=t):o=Ul(t,i,0,null),n=Hl(n,i,r,null),o.return=e,n.return=e,o.sibling=n,e.child=o,n}function Za(e,t,n,r){var i=e.child;return e=i.sibling,n=Fl(i,{mode:"visible",children:n}),0==(2&t.mode)&&(n.lanes=r),n.return=t,n.sibling=null,null!==e&&(e.nextEffect=null,e.flags=8,t.firstEffect=t.lastEffect=e),t.child=n}function Qa(e,t,n,r,i){var o=t.mode,a=e.child;e=a.sibling;var s={mode:"hidden",children:n};return 0==(2&o)&&t.child!==a?((n=t.child).childLanes=0,n.pendingProps=s,null!==(a=n.lastEffect)?(t.firstEffect=n.firstEffect,t.lastEffect=a,a.nextEffect=null):t.firstEffect=t.lastEffect=null):n=Fl(a,s),null!==e?r=Fl(e,r):(r=Hl(r,o,i,null)).flags|=2,r.return=t,n.return=t,n.sibling=r,t.child=n,r}function Ka(e,t){e.lanes|=t;var n=e.alternate;null!==n&&(n.lanes|=t),$i(e.return,t)}function Ja(e,t,n,r,i,o){var a=e.memoizedState;null===a?e.memoizedState={isBackwards:t,rendering:null,renderingStartTime:0,last:r,tail:n,tailMode:i,lastEffect:o}:(a.isBackwards=t,a.rendering=null,a.renderingStartTime=0,a.last=r,a.tail=n,a.tailMode=i,a.lastEffect=o)}function $a(e,t,n){var r=t.pendingProps,i=r.revealOrder,o=r.tail;if(Da(e,t,r.children,n),0!=(2&(r=Do.current)))r=1&r|2,t.flags|=64;else{if(null!==e&&0!=(64&e.flags))e:for(e=t.child;null!==e;){if(13===e.tag)null!==e.memoizedState&&Ka(e,n);else if(19===e.tag)Ka(e,n);else if(null!==e.child){e.child.return=e,e=e.child;continue}if(e===t)break e;for(;null===e.sibling;){if(null===e.return||e.return===t)break e;e=e.return}e.sibling.return=e.return,e=e.sibling}r&=1}if(ai(Do,r),0==(2&t.mode))t.memoizedState=null;else switch(i){case"forwards":for(n=t.child,i=null;null!==n;)null!==(e=n.alternate)&&null===Po(e)&&(i=n),n=n.sibling;null===(n=i)?(i=t.child,t.child=null):(i=n.sibling,n.sibling=null),Ja(t,!1,i,n,o,t.lastEffect);break;case"backwards":for(n=null,i=t.child,t.child=null;null!==i;){if(null!==(e=i.alternate)&&null===Po(e)){t.child=i;break}e=i.sibling,i.sibling=n,n=i,i=e}Ja(t,!0,n,null,o,t.lastEffect);break;case"together":Ja(t,!1,null,null,void 0,t.lastEffect);break;default:t.memoizedState=null}return t.child}function es(e,t,n){if(null!==e&&(t.dependencies=e.dependencies),Ps|=t.lanes,0!=(n&t.childLanes)){if(null!==e&&t.child!==e.child)throw Error(a(153));if(null!==t.child){for(n=Fl(e=t.child,e.pendingProps),t.child=n,n.return=t;null!==e.sibling;)e=e.sibling,(n=n.sibling=Fl(e,e.pendingProps)).return=t;n.sibling=null}return t.child}return null}function ts(e,t){if(!Ro)switch(e.tailMode){case"hidden":t=e.tail;for(var n=null;null!==t;)null!==t.alternate&&(n=t),t=t.sibling;null===n?e.tail=null:n.sibling=null;break;case"collapsed":n=e.tail;for(var r=null;null!==n;)null!==n.alternate&&(r=n),n=n.sibling;null===r?t||null===e.tail?e.tail=null:e.tail.sibling=null:r.sibling=null}}function ns(e,t,n){var r=t.pendingProps;switch(t.tag){case 2:case 16:case 15:case 0:case 11:case 7:case 8:case 12:case 9:case 14:return null;case 1:return hi(t.type)&&fi(),null;case 3:return Oo(),oi(ui),oi(li),Wo(),(r=t.stateNode).pendingContext&&(r.context=r.pendingContext,r.pendingContext=null),null!==e&&null!==e.child||(Uo(t)?t.flags|=4:r.hydrate||(t.flags|=256)),null;case 5:Lo(t);var o=To(So.current);if(n=t.type,null!==e&&null!=t.stateNode)Ya(e,t,n,r),e.ref!==t.ref&&(t.flags|=128);else{if(!r){if(null===t.stateNode)throw Error(a(166));return null}if(e=To(wo.current),Uo(t)){r=t.stateNode,n=t.type;var s=t.memoizedProps;switch(r[Xr]=t,r[qr]=s,n){case"dialog":wr("cancel",r),wr("close",r);break;case"iframe":case"object":case"embed":wr("load",r);break;case"video":case"audio":for(e=0;e<Ar.length;e++)wr(Ar[e],r);break;case"source":wr("error",r);break;case"img":case"image":case"link":wr("error",r),wr("load",r);break;case"details":wr("toggle",r);break;case"input":ee(r,s),wr("invalid",r);break;case"select":r._wrapperState={wasMultiple:!!s.multiple},wr("invalid",r);break;case"textarea":le(r,s),wr("invalid",r)}for(var u in Ie(n,s),e=null,s)s.hasOwnProperty(u)&&(o=s[u],"children"===u?"string"==typeof o?r.textContent!==o&&(e=["children",o]):"number"==typeof o&&r.textContent!==""+o&&(e=["children",""+o]):l.hasOwnProperty(u)&&null!=o&&"onScroll"===u&&wr("scroll",r));switch(n){case"input":Q(r),re(r,s,!0);break;case"textarea":Q(r),ce(r);break;case"select":case"option":break;default:"function"==typeof s.onClick&&(r.onclick=Nr)}r=e,t.updateQueue=r,null!==r&&(t.flags|=4)}else{switch(u=9===o.nodeType?o:o.ownerDocument,e===de&&(e=fe(n)),e===de?"script"===n?((e=u.createElement("div")).innerHTML="<script><\/script>",e=e.removeChild(e.firstChild)):"string"==typeof r.is?e=u.createElement(n,{is:r.is}):(e=u.createElement(n),"select"===n&&(u=e,r.multiple?u.multiple=!0:r.size&&(u.size=r.size))):e=u.createElementNS(e,n),e[Xr]=t,e[qr]=r,Va(e,t),t.stateNode=e,u=xe(n,r),n){case"dialog":wr("cancel",e),wr("close",e),o=r;break;case"iframe":case"object":case"embed":wr("load",e),o=r;break;case"video":case"audio":for(o=0;o<Ar.length;o++)wr(Ar[o],e);o=r;break;case"source":wr("error",e),o=r;break;case"img":case"image":case"link":wr("error",e),wr("load",e),o=r;break;case"details":wr("toggle",e),o=r;break;case"input":ee(e,r),o=$(e,r),wr("invalid",e);break;case"option":o=oe(e,r);break;case"select":e._wrapperState={wasMultiple:!!r.multiple},o=i({},r,{value:void 0}),wr("invalid",e);break;case"textarea":le(e,r),o=se(e,r),wr("invalid",e);break;default:o=r}Ie(n,o);var c=o;for(s in c)if(c.hasOwnProperty(s)){var d=c[s];"style"===s?Ae(e,d):"dangerouslySetInnerHTML"===s?null!=(d=d?d.__html:void 0)&&ve(e,d):"children"===s?"string"==typeof d?("textarea"!==n||""!==d)&&me(e,d):"number"==typeof d&&me(e,""+d):"suppressContentEditableWarning"!==s&&"suppressHydrationWarning"!==s&&"autoFocus"!==s&&(l.hasOwnProperty(s)?null!=d&&"onScroll"===s&&wr("scroll",e):null!=d&&C(e,s,d,u))}switch(n){case"input":Q(e),re(e,r,!1);break;case"textarea":Q(e),ce(e);break;case"option":null!=r.value&&e.setAttribute("value",""+q(r.value));break;case"select":e.multiple=!!r.multiple,null!=(s=r.value)?ae(e,!!r.multiple,s,!1):null!=r.defaultValue&&ae(e,!!r.multiple,r.defaultValue,!0);break;default:"function"==typeof o.onClick&&(e.onclick=Nr)}zr(n,r)&&(t.flags|=4)}null!==t.ref&&(t.flags|=128)}return null;case 6:if(e&&null!=t.stateNode)Wa(0,t,e.memoizedProps,r);else{if("string"!=typeof r&&null===t.stateNode)throw Error(a(166));n=To(So.current),To(wo.current),Uo(t)?(r=t.stateNode,n=t.memoizedProps,r[Xr]=t,r.nodeValue!==n&&(t.flags|=4)):((r=(9===n.nodeType?n:n.ownerDocument).createTextNode(r))[Xr]=t,t.stateNode=r)}return null;case 13:return oi(Do),r=t.memoizedState,0!=(64&t.flags)?(t.lanes=n,t):(r=null!==r,n=!1,null===e?void 0!==t.memoizedProps.fallback&&Uo(t):n=null!==e.memoizedState,r&&!n&&0!=(2&t.mode)&&(null===e&&!0!==t.memoizedProps.unstable_avoidThisFallback||0!=(1&Do.current)?0===ks&&(ks=3):(0!==ks&&3!==ks||(ks=4),null===Ms||0==(134217727&Ps)&&0==(134217727&Ns)||dl(Ms,Ts))),(r||n)&&(t.flags|=4),null);case 4:return Oo(),null===e&&Sr(t.stateNode.containerInfo),null;case 10:return Ji(t),null;case 17:return hi(t.type)&&fi(),null;case 19:if(oi(Do),null===(r=t.memoizedState))return null;if(s=0!=(64&t.flags),null===(u=r.rendering))if(s)ts(r,!1);else{if(0!==ks||null!==e&&0!=(64&e.flags))for(e=t.child;null!==e;){if(null!==(u=Po(e))){for(t.flags|=64,ts(r,!1),null!==(s=u.updateQueue)&&(t.updateQueue=s,t.flags|=4),null===r.lastEffect&&(t.firstEffect=null),t.lastEffect=r.lastEffect,r=n,n=t.child;null!==n;)e=r,(s=n).flags&=2,s.nextEffect=null,s.firstEffect=null,s.lastEffect=null,null===(u=s.alternate)?(s.childLanes=0,s.lanes=e,s.child=null,s.memoizedProps=null,s.memoizedState=null,s.updateQueue=null,s.dependencies=null,s.stateNode=null):(s.childLanes=u.childLanes,s.lanes=u.lanes,s.child=u.child,s.memoizedProps=u.memoizedProps,s.memoizedState=u.memoizedState,s.updateQueue=u.updateQueue,s.type=u.type,e=u.dependencies,s.dependencies=null===e?null:{lanes:e.lanes,firstContext:e.firstContext}),n=n.sibling;return ai(Do,1&Do.current|2),t.child}e=e.sibling}null!==r.tail&&zi()>Fs&&(t.flags|=64,s=!0,ts(r,!1),t.lanes=33554432)}else{if(!s)if(null!==(e=Po(u))){if(t.flags|=64,s=!0,null!==(n=e.updateQueue)&&(t.updateQueue=n,t.flags|=4),ts(r,!0),null===r.tail&&"hidden"===r.tailMode&&!u.alternate&&!Ro)return null!==(t=t.lastEffect=r.lastEffect)&&(t.nextEffect=null),null}else 2*zi()-r.renderingStartTime>Fs&&1073741824!==n&&(t.flags|=64,s=!0,ts(r,!1),t.lanes=33554432);r.isBackwards?(u.sibling=t.child,t.child=u):(null!==(n=r.last)?n.sibling=u:t.child=u,r.last=u)}return null!==r.tail?(n=r.tail,r.rendering=n,r.tail=n.sibling,r.lastEffect=t.lastEffect,r.renderingStartTime=zi(),n.sibling=null,t=Do.current,ai(Do,s?1&t|2:1&t),n):null;case 23:case 24:return vl(),null!==e&&null!==e.memoizedState!=(null!==t.memoizedState)&&"unstable-defer-without-hiding"!==r.mode&&(t.flags|=4),null}throw Error(a(156,t.tag))}function rs(e){switch(e.tag){case 1:hi(e.type)&&fi();var t=e.flags;return 4096&t?(e.flags=-4097&t|64,e):null;case 3:if(Oo(),oi(ui),oi(li),Wo(),0!=(64&(t=e.flags)))throw Error(a(285));return e.flags=-4097&t|64,e;case 5:return Lo(e),null;case 13:return oi(Do),4096&(t=e.flags)?(e.flags=-4097&t|64,e):null;case 19:return oi(Do),null;case 4:return Oo(),null;case 10:return Ji(e),null;case 23:case 24:return vl(),null;default:return null}}function is(e,t){try{var n="",r=t;do{n+=G(r),r=r.return}while(r);var i=n}catch(e){i="\nError generating stack: "+e.message+"\n"+e.stack}return{value:e,source:t,stack:i}}function os(e,t){try{console.error(t.value)}catch(e){setTimeout((function(){throw e}))}}Va=function(e,t){for(var n=t.child;null!==n;){if(5===n.tag||6===n.tag)e.appendChild(n.stateNode);else if(4!==n.tag&&null!==n.child){n.child.return=n,n=n.child;continue}if(n===t)break;for(;null===n.sibling;){if(null===n.return||n.return===t)return;n=n.return}n.sibling.return=n.return,n=n.sibling}},Ya=function(e,t,n,r){var o=e.memoizedProps;if(o!==r){e=t.stateNode,To(wo.current);var a,s=null;switch(n){case"input":o=$(e,o),r=$(e,r),s=[];break;case"option":o=oe(e,o),r=oe(e,r),s=[];break;case"select":o=i({},o,{value:void 0}),r=i({},r,{value:void 0}),s=[];break;case"textarea":o=se(e,o),r=se(e,r),s=[];break;default:"function"!=typeof o.onClick&&"function"==typeof r.onClick&&(e.onclick=Nr)}for(d in Ie(n,r),n=null,o)if(!r.hasOwnProperty(d)&&o.hasOwnProperty(d)&&null!=o[d])if("style"===d){var u=o[d];for(a in u)u.hasOwnProperty(a)&&(n||(n={}),n[a]="")}else"dangerouslySetInnerHTML"!==d&&"children"!==d&&"suppressContentEditableWarning"!==d&&"suppressHydrationWarning"!==d&&"autoFocus"!==d&&(l.hasOwnProperty(d)?s||(s=[]):(s=s||[]).push(d,null));for(d in r){var c=r[d];if(u=null!=o?o[d]:void 0,r.hasOwnProperty(d)&&c!==u&&(null!=c||null!=u))if("style"===d)if(u){for(a in u)!u.hasOwnProperty(a)||c&&c.hasOwnProperty(a)||(n||(n={}),n[a]="");for(a in c)c.hasOwnProperty(a)&&u[a]!==c[a]&&(n||(n={}),n[a]=c[a])}else n||(s||(s=[]),s.push(d,n)),n=c;else"dangerouslySetInnerHTML"===d?(c=c?c.__html:void 0,u=u?u.__html:void 0,null!=c&&u!==c&&(s=s||[]).push(d,c)):"children"===d?"string"!=typeof c&&"number"!=typeof c||(s=s||[]).push(d,""+c):"suppressContentEditableWarning"!==d&&"suppressHydrationWarning"!==d&&(l.hasOwnProperty(d)?(null!=c&&"onScroll"===d&&wr("scroll",e),s||u===c||(s=[])):"object"==typeof c&&null!==c&&c.$$typeof===N?c.toString():(s=s||[]).push(d,c))}n&&(s=s||[]).push("style",n);var d=s;(t.updateQueue=d)&&(t.flags|=4)}},Wa=function(e,t,n,r){n!==r&&(t.flags|=4)};var as="function"==typeof WeakMap?WeakMap:Map;function ss(e,t,n){(n=oo(-1,n)).tag=3,n.payload={element:null};var r=t.value;return n.callback=function(){Vs||(Vs=!0,Ys=r),os(0,t)},n}function ls(e,t,n){(n=oo(-1,n)).tag=3;var r=e.type.getDerivedStateFromError;if("function"==typeof r){var i=t.value;n.payload=function(){return os(0,t),r(i)}}var o=e.stateNode;return null!==o&&"function"==typeof o.componentDidCatch&&(n.callback=function(){"function"!=typeof r&&(null===Ws?Ws=new Set([this]):Ws.add(this),os(0,t));var e=t.stack;this.componentDidCatch(t.value,{componentStack:null!==e?e:""})}),n}var us="function"==typeof WeakSet?WeakSet:Set;function cs(e){var t=e.ref;if(null!==t)if("function"==typeof t)try{t(null)}catch(t){Dl(e,t)}else t.current=null}function ds(e,t){switch(t.tag){case 0:case 11:case 15:case 22:return;case 1:if(256&t.flags&&null!==e){var n=e.memoizedProps,r=e.memoizedState;t=(e=t.stateNode).getSnapshotBeforeUpdate(t.elementType===t.type?n:Gi(t.type,n),r),e.__reactInternalSnapshotBeforeUpdate=t}return;case 3:return void(256&t.flags&&Ur(t.stateNode.containerInfo));case 5:case 6:case 4:case 17:return}throw Error(a(163))}function hs(e,t,n){switch(n.tag){case 0:case 11:case 15:case 22:if(null!==(t=null!==(t=n.updateQueue)?t.lastEffect:null)){e=t=t.next;do{if(3==(3&e.tag)){var r=e.create;e.destroy=r()}e=e.next}while(e!==t)}if(null!==(t=null!==(t=n.updateQueue)?t.lastEffect:null)){e=t=t.next;do{var i=e;r=i.next,0!=(4&(i=i.tag))&&0!=(1&i)&&(Ol(n,e),El(n,e)),e=r}while(e!==t)}return;case 1:return e=n.stateNode,4&n.flags&&(null===t?e.componentDidMount():(r=n.elementType===n.type?t.memoizedProps:Gi(n.type,t.memoizedProps),e.componentDidUpdate(r,t.memoizedState,e.__reactInternalSnapshotBeforeUpdate))),void(null!==(t=n.updateQueue)&&uo(n,t,e));case 3:if(null!==(t=n.updateQueue)){if(e=null,null!==n.child)switch(n.child.tag){case 5:e=n.child.stateNode;break;case 1:e=n.child.stateNode}uo(n,t,e)}return;case 5:return e=n.stateNode,void(null===t&&4&n.flags&&zr(n.type,n.memoizedProps)&&e.focus());case 6:case 4:case 12:return;case 13:return void(null===n.memoizedState&&(n=n.alternate,null!==n&&(n=n.memoizedState,null!==n&&(n=n.dehydrated,null!==n&&At(n)))));case 19:case 17:case 20:case 21:case 23:case 24:return}throw Error(a(163))}function fs(e,t){for(var n=e;;){if(5===n.tag){var r=n.stateNode;if(t)"function"==typeof(r=r.style).setProperty?r.setProperty("display","none","important"):r.display="none";else{r=n.stateNode;var i=n.memoizedProps.style;i=null!=i&&i.hasOwnProperty("display")?i.display:null,r.style.display=Ce("display",i)}}else if(6===n.tag)n.stateNode.nodeValue=t?"":n.memoizedProps;else if((23!==n.tag&&24!==n.tag||null===n.memoizedState||n===e)&&null!==n.child){n.child.return=n,n=n.child;continue}if(n===e)break;for(;null===n.sibling;){if(null===n.return||n.return===e)return;n=n.return}n.sibling.return=n.return,n=n.sibling}}function ps(e,t){if(bi&&"function"==typeof bi.onCommitFiberUnmount)try{bi.onCommitFiberUnmount(yi,t)}catch(e){}switch(t.tag){case 0:case 11:case 14:case 15:case 22:if(null!==(e=t.updateQueue)&&null!==(e=e.lastEffect)){var n=e=e.next;do{var r=n,i=r.destroy;if(r=r.tag,void 0!==i)if(0!=(4&r))Ol(t,n);else{r=t;try{i()}catch(e){Dl(r,e)}}n=n.next}while(n!==e)}break;case 1:if(cs(t),"function"==typeof(e=t.stateNode).componentWillUnmount)try{e.props=t.memoizedProps,e.state=t.memoizedState,e.componentWillUnmount()}catch(e){Dl(t,e)}break;case 5:cs(t);break;case 4:ys(e,t)}}function gs(e){e.alternate=null,e.child=null,e.dependencies=null,e.firstEffect=null,e.lastEffect=null,e.memoizedProps=null,e.memoizedState=null,e.pendingProps=null,e.return=null,e.updateQueue=null}function vs(e){return 5===e.tag||3===e.tag||4===e.tag}function ms(e){e:{for(var t=e.return;null!==t;){if(vs(t))break e;t=t.return}throw Error(a(160))}var n=t;switch(t=n.stateNode,n.tag){case 5:var r=!1;break;case 3:case 4:t=t.containerInfo,r=!0;break;default:throw Error(a(161))}16&n.flags&&(me(t,""),n.flags&=-17);e:t:for(n=e;;){for(;null===n.sibling;){if(null===n.return||vs(n.return)){n=null;break e}n=n.return}for(n.sibling.return=n.return,n=n.sibling;5!==n.tag&&6!==n.tag&&18!==n.tag;){if(2&n.flags)continue t;if(null===n.child||4===n.tag)continue t;n.child.return=n,n=n.child}if(!(2&n.flags)){n=n.stateNode;break e}}r?function e(t,n,r){var i=t.tag,o=5===i||6===i;if(o)t=o?t.stateNode:t.stateNode.instance,n?8===r.nodeType?r.parentNode.insertBefore(t,n):r.insertBefore(t,n):(8===r.nodeType?(n=r.parentNode).insertBefore(t,r):(n=r).appendChild(t),null!==(r=r._reactRootContainer)&&void 0!==r||null!==n.onclick||(n.onclick=Nr));else if(4!==i&&null!==(t=t.child))for(e(t,n,r),t=t.sibling;null!==t;)e(t,n,r),t=t.sibling}(e,n,t):function e(t,n,r){var i=t.tag,o=5===i||6===i;if(o)t=o?t.stateNode:t.stateNode.instance,n?r.insertBefore(t,n):r.appendChild(t);else if(4!==i&&null!==(t=t.child))for(e(t,n,r),t=t.sibling;null!==t;)e(t,n,r),t=t.sibling}(e,n,t)}function ys(e,t){for(var n,r,i=t,o=!1;;){if(!o){o=i.return;e:for(;;){if(null===o)throw Error(a(160));switch(n=o.stateNode,o.tag){case 5:r=!1;break e;case 3:case 4:n=n.containerInfo,r=!0;break e}o=o.return}o=!0}if(5===i.tag||6===i.tag){e:for(var s=e,l=i,u=l;;)if(ps(s,u),null!==u.child&&4!==u.tag)u.child.return=u,u=u.child;else{if(u===l)break e;for(;null===u.sibling;){if(null===u.return||u.return===l)break e;u=u.return}u.sibling.return=u.return,u=u.sibling}r?(s=n,l=i.stateNode,8===s.nodeType?s.parentNode.removeChild(l):s.removeChild(l)):n.removeChild(i.stateNode)}else if(4===i.tag){if(null!==i.child){n=i.stateNode.containerInfo,r=!0,i.child.return=i,i=i.child;continue}}else if(ps(e,i),null!==i.child){i.child.return=i,i=i.child;continue}if(i===t)break;for(;null===i.sibling;){if(null===i.return||i.return===t)return;4===(i=i.return).tag&&(o=!1)}i.sibling.return=i.return,i=i.sibling}}function bs(e,t){switch(t.tag){case 0:case 11:case 14:case 15:case 22:var n=t.updateQueue;if(null!==(n=null!==n?n.lastEffect:null)){var r=n=n.next;do{3==(3&r.tag)&&(e=r.destroy,r.destroy=void 0,void 0!==e&&e()),r=r.next}while(r!==n)}return;case 1:return;case 5:if(null!=(n=t.stateNode)){r=t.memoizedProps;var i=null!==e?e.memoizedProps:r;e=t.type;var o=t.updateQueue;if(t.updateQueue=null,null!==o){for(n[qr]=r,"input"===e&&"radio"===r.type&&null!=r.name&&te(n,r),xe(e,i),t=xe(e,r),i=0;i<o.length;i+=2){var s=o[i],l=o[i+1];"style"===s?Ae(n,l):"dangerouslySetInnerHTML"===s?ve(n,l):"children"===s?me(n,l):C(n,s,l,t)}switch(e){case"input":ne(n,r);break;case"textarea":ue(n,r);break;case"select":e=n._wrapperState.wasMultiple,n._wrapperState.wasMultiple=!!r.multiple,null!=(o=r.value)?ae(n,!!r.multiple,o,!1):e!==!!r.multiple&&(null!=r.defaultValue?ae(n,!!r.multiple,r.defaultValue,!0):ae(n,!!r.multiple,r.multiple?[]:"",!1))}}}return;case 6:if(null===t.stateNode)throw Error(a(162));return void(t.stateNode.nodeValue=t.memoizedProps);case 3:return void((n=t.stateNode).hydrate&&(n.hydrate=!1,At(n.containerInfo)));case 12:return;case 13:return null!==t.memoizedState&&(zs=zi(),fs(t.child,!0)),void Cs(t);case 19:return void Cs(t);case 17:return;case 23:case 24:return void fs(t,null!==t.memoizedState)}throw Error(a(163))}function Cs(e){var t=e.updateQueue;if(null!==t){e.updateQueue=null;var n=e.stateNode;null===n&&(n=e.stateNode=new us),t.forEach((function(t){var r=Nl.bind(null,e,t);n.has(t)||(n.add(t),t.then(r,r))}))}}function As(e,t){return null!==e&&(null===(e=e.memoizedState)||null!==e.dehydrated)&&(null!==(t=t.memoizedState)&&null===t.dehydrated)}var _s=Math.ceil,Is=A.ReactCurrentDispatcher,xs=A.ReactCurrentOwner,ws=0,Ms=null,Ss=null,Ts=0,Es=0,Os=ii(0),ks=0,Ls=null,Ds=0,Ps=0,Ns=0,js=0,Rs=null,zs=0,Fs=1/0;function Bs(){Fs=zi()+500}var Hs,Us=null,Vs=!1,Ys=null,Ws=null,Gs=!1,Xs=null,qs=90,Zs=[],Qs=[],Ks=null,Js=0,$s=null,el=-1,tl=0,nl=0,rl=null,il=!1;function ol(){return 0!=(48&ws)?zi():-1!==el?el:el=zi()}function al(e){if(0==(2&(e=e.mode)))return 1;if(0==(4&e))return 99===Fi()?1:2;if(0===tl&&(tl=Ds),0!==Wi.transition){0!==nl&&(nl=null!==Rs?Rs.pendingLanes:0),e=tl;var t=4186112&~nl;return 0===(t&=-t)&&(0===(t=(e=4186112&~e)&-e)&&(t=8192)),t}return e=Fi(),0!=(4&ws)&&98===e?e=Ft(12,tl):e=Ft(e=function(e){switch(e){case 99:return 15;case 98:return 10;case 97:case 96:return 8;case 95:return 2;default:return 0}}(e),tl),e}function sl(e,t,n){if(50<Js)throw Js=0,$s=null,Error(a(185));if(null===(e=ll(e,t)))return null;Ut(e,t,n),e===Ms&&(Ns|=t,4===ks&&dl(e,Ts));var r=Fi();1===t?0!=(8&ws)&&0==(48&ws)?hl(e):(ul(e,n),0===ws&&(Bs(),Vi())):(0==(4&ws)||98!==r&&99!==r||(null===Ks?Ks=new Set([e]):Ks.add(e)),ul(e,n)),Rs=e}function ll(e,t){e.lanes|=t;var n=e.alternate;for(null!==n&&(n.lanes|=t),n=e,e=e.return;null!==e;)e.childLanes|=t,null!==(n=e.alternate)&&(n.childLanes|=t),n=e,e=e.return;return 3===n.tag?n.stateNode:null}function ul(e,t){for(var n=e.callbackNode,r=e.suspendedLanes,i=e.pingedLanes,o=e.expirationTimes,s=e.pendingLanes;0<s;){var l=31-Vt(s),u=1<<l,c=o[l];if(-1===c){if(0==(u&r)||0!=(u&i)){c=t,jt(u);var d=Nt;o[l]=10<=d?c+250:6<=d?c+5e3:-1}}else c<=t&&(e.expiredLanes|=u);s&=~u}if(r=Rt(e,e===Ms?Ts:0),t=Nt,0===r)null!==n&&(n!==Li&&_i(n),e.callbackNode=null,e.callbackPriority=0);else{if(null!==n){if(e.callbackPriority===t)return;n!==Li&&_i(n)}15===t?(n=hl.bind(null,e),null===Pi?(Pi=[n],Ni=Ai(Si,Yi)):Pi.push(n),n=Li):14===t?n=Ui(99,hl.bind(null,e)):n=Ui(n=function(e){switch(e){case 15:case 14:return 99;case 13:case 12:case 11:case 10:return 98;case 9:case 8:case 7:case 6:case 4:case 5:return 97;case 3:case 2:case 1:return 95;case 0:return 90;default:throw Error(a(358,e))}}(t),cl.bind(null,e)),e.callbackPriority=t,e.callbackNode=n}}function cl(e){if(el=-1,nl=tl=0,0!=(48&ws))throw Error(a(327));var t=e.callbackNode;if(Tl()&&e.callbackNode!==t)return null;var n=Rt(e,e===Ms?Ts:0);if(0===n)return null;var r=n,i=ws;ws|=16;var o=bl();for(Ms===e&&Ts===r||(Bs(),ml(e,r));;)try{_l();break}catch(t){yl(e,t)}if(Ki(),Is.current=o,ws=i,null!==Ss?r=0:(Ms=null,Ts=0,r=ks),0!=(Ds&Ns))ml(e,0);else if(0!==r){if(2===r&&(ws|=64,e.hydrate&&(e.hydrate=!1,Ur(e.containerInfo)),0!==(n=zt(e))&&(r=Cl(e,n))),1===r)throw t=Ls,ml(e,0),dl(e,n),ul(e,zi()),t;switch(e.finishedWork=e.current.alternate,e.finishedLanes=n,r){case 0:case 1:throw Error(a(345));case 2:wl(e);break;case 3:if(dl(e,n),(62914560&n)===n&&10<(r=zs+500-zi())){if(0!==Rt(e,0))break;if(((i=e.suspendedLanes)&n)!==n){ol(),e.pingedLanes|=e.suspendedLanes&i;break}e.timeoutHandle=Br(wl.bind(null,e),r);break}wl(e);break;case 4:if(dl(e,n),(4186112&n)===n)break;for(r=e.eventTimes,i=-1;0<n;){var s=31-Vt(n);o=1<<s,(s=r[s])>i&&(i=s),n&=~o}if(n=i,10<(n=(120>(n=zi()-n)?120:480>n?480:1080>n?1080:1920>n?1920:3e3>n?3e3:4320>n?4320:1960*_s(n/1960))-n)){e.timeoutHandle=Br(wl.bind(null,e),n);break}wl(e);break;case 5:wl(e);break;default:throw Error(a(329))}}return ul(e,zi()),e.callbackNode===t?cl.bind(null,e):null}function dl(e,t){for(t&=~js,t&=~Ns,e.suspendedLanes|=t,e.pingedLanes&=~t,e=e.expirationTimes;0<t;){var n=31-Vt(t),r=1<<n;e[n]=-1,t&=~r}}function hl(e){if(0!=(48&ws))throw Error(a(327));if(Tl(),e===Ms&&0!=(e.expiredLanes&Ts)){var t=Ts,n=Cl(e,t);0!=(Ds&Ns)&&(n=Cl(e,t=Rt(e,t)))}else n=Cl(e,t=Rt(e,0));if(0!==e.tag&&2===n&&(ws|=64,e.hydrate&&(e.hydrate=!1,Ur(e.containerInfo)),0!==(t=zt(e))&&(n=Cl(e,t))),1===n)throw n=Ls,ml(e,0),dl(e,t),ul(e,zi()),n;return e.finishedWork=e.current.alternate,e.finishedLanes=t,wl(e),ul(e,zi()),null}function fl(e,t){var n=ws;ws|=1;try{return e(t)}finally{0===(ws=n)&&(Bs(),Vi())}}function pl(e,t){var n=ws;ws&=-2,ws|=8;try{return e(t)}finally{0===(ws=n)&&(Bs(),Vi())}}function gl(e,t){ai(Os,Es),Es|=t,Ds|=t}function vl(){Es=Os.current,oi(Os)}function ml(e,t){e.finishedWork=null,e.finishedLanes=0;var n=e.timeoutHandle;if(-1!==n&&(e.timeoutHandle=-1,Hr(n)),null!==Ss)for(n=Ss.return;null!==n;){var r=n;switch(r.tag){case 1:null!=(r=r.type.childContextTypes)&&fi();break;case 3:Oo(),oi(ui),oi(li),Wo();break;case 5:Lo(r);break;case 4:Oo();break;case 13:case 19:oi(Do);break;case 10:Ji(r);break;case 23:case 24:vl()}n=n.return}Ms=e,Ss=Fl(e.current,null),Ts=Es=Ds=t,ks=0,Ls=null,js=Ns=Ps=0}function yl(e,t){for(;;){var n=Ss;try{if(Ki(),Go.current=Sa,Jo){for(var r=Zo.memoizedState;null!==r;){var i=r.queue;null!==i&&(i.pending=null),r=r.next}Jo=!1}if(qo=0,Ko=Qo=Zo=null,$o=!1,xs.current=null,null===n||null===n.return){ks=1,Ls=t,Ss=null;break}e:{var o=e,a=n.return,s=n,l=t;if(t=Ts,s.flags|=2048,s.firstEffect=s.lastEffect=null,null!==l&&"object"==typeof l&&"function"==typeof l.then){var u=l;if(0==(2&s.mode)){var c=s.alternate;c?(s.updateQueue=c.updateQueue,s.memoizedState=c.memoizedState,s.lanes=c.lanes):(s.updateQueue=null,s.memoizedState=null)}var d=0!=(1&Do.current),h=a;do{var f;if(f=13===h.tag){var p=h.memoizedState;if(null!==p)f=null!==p.dehydrated;else{var g=h.memoizedProps;f=void 0!==g.fallback&&(!0!==g.unstable_avoidThisFallback||!d)}}if(f){var v=h.updateQueue;if(null===v){var m=new Set;m.add(u),h.updateQueue=m}else v.add(u);if(0==(2&h.mode)){if(h.flags|=64,s.flags|=16384,s.flags&=-2981,1===s.tag)if(null===s.alternate)s.tag=17;else{var y=oo(-1,1);y.tag=2,ao(s,y)}s.lanes|=1;break e}l=void 0,s=t;var b=o.pingCache;if(null===b?(b=o.pingCache=new as,l=new Set,b.set(u,l)):void 0===(l=b.get(u))&&(l=new Set,b.set(u,l)),!l.has(s)){l.add(s);var C=Pl.bind(null,o,u,s);u.then(C,C)}h.flags|=4096,h.lanes=t;break e}h=h.return}while(null!==h);l=Error((X(s.type)||"A React component")+" suspended while rendering, but no fallback UI was specified.\n\nAdd a <Suspense fallback=...> component higher in the tree to provide a loading indicator or placeholder to display.")}5!==ks&&(ks=2),l=is(l,s),h=a;do{switch(h.tag){case 3:o=l,h.flags|=4096,t&=-t,h.lanes|=t,so(h,ss(0,o,t));break e;case 1:o=l;var A=h.type,_=h.stateNode;if(0==(64&h.flags)&&("function"==typeof A.getDerivedStateFromError||null!==_&&"function"==typeof _.componentDidCatch&&(null===Ws||!Ws.has(_)))){h.flags|=4096,t&=-t,h.lanes|=t,so(h,ls(h,o,t));break e}}h=h.return}while(null!==h)}xl(n)}catch(e){t=e,Ss===n&&null!==n&&(Ss=n=n.return);continue}break}}function bl(){var e=Is.current;return Is.current=Sa,null===e?Sa:e}function Cl(e,t){var n=ws;ws|=16;var r=bl();for(Ms===e&&Ts===t||ml(e,t);;)try{Al();break}catch(t){yl(e,t)}if(Ki(),ws=n,Is.current=r,null!==Ss)throw Error(a(261));return Ms=null,Ts=0,ks}function Al(){for(;null!==Ss;)Il(Ss)}function _l(){for(;null!==Ss&&!Ii();)Il(Ss)}function Il(e){var t=Hs(e.alternate,e,Es);e.memoizedProps=e.pendingProps,null===t?xl(e):Ss=t,xs.current=null}function xl(e){var t=e;do{var n=t.alternate;if(e=t.return,0==(2048&t.flags)){if(null!==(n=ns(n,t,Es)))return void(Ss=n);if(24!==(n=t).tag&&23!==n.tag||null===n.memoizedState||0!=(1073741824&Es)||0==(4&n.mode)){for(var r=0,i=n.child;null!==i;)r|=i.lanes|i.childLanes,i=i.sibling;n.childLanes=r}null!==e&&0==(2048&e.flags)&&(null===e.firstEffect&&(e.firstEffect=t.firstEffect),null!==t.lastEffect&&(null!==e.lastEffect&&(e.lastEffect.nextEffect=t.firstEffect),e.lastEffect=t.lastEffect),1<t.flags&&(null!==e.lastEffect?e.lastEffect.nextEffect=t:e.firstEffect=t,e.lastEffect=t))}else{if(null!==(n=rs(t)))return n.flags&=2047,void(Ss=n);null!==e&&(e.firstEffect=e.lastEffect=null,e.flags|=2048)}if(null!==(t=t.sibling))return void(Ss=t);Ss=t=e}while(null!==t);0===ks&&(ks=5)}function wl(e){var t=Fi();return Hi(99,Ml.bind(null,e,t)),null}function Ml(e,t){do{Tl()}while(null!==Xs);if(0!=(48&ws))throw Error(a(327));var n=e.finishedWork;if(null===n)return null;if(e.finishedWork=null,e.finishedLanes=0,n===e.current)throw Error(a(177));e.callbackNode=null;var r=n.lanes|n.childLanes,i=r,o=e.pendingLanes&~i;e.pendingLanes=i,e.suspendedLanes=0,e.pingedLanes=0,e.expiredLanes&=i,e.mutableReadLanes&=i,e.entangledLanes&=i,i=e.entanglements;for(var s=e.eventTimes,l=e.expirationTimes;0<o;){var u=31-Vt(o),c=1<<u;i[u]=0,s[u]=-1,l[u]=-1,o&=~c}if(null!==Ks&&0==(24&r)&&Ks.has(e)&&Ks.delete(e),e===Ms&&(Ss=Ms=null,Ts=0),1<n.flags?null!==n.lastEffect?(n.lastEffect.nextEffect=n,r=n.firstEffect):r=n:r=n.firstEffect,null!==r){if(i=ws,ws|=32,xs.current=null,jr=qt,hr(s=dr())){if("selectionStart"in s)l={start:s.selectionStart,end:s.selectionEnd};else e:if(l=(l=s.ownerDocument)&&l.defaultView||window,(c=l.getSelection&&l.getSelection())&&0!==c.rangeCount){l=c.anchorNode,o=c.anchorOffset,u=c.focusNode,c=c.focusOffset;try{l.nodeType,u.nodeType}catch(e){l=null;break e}var d=0,h=-1,f=-1,p=0,g=0,v=s,m=null;t:for(;;){for(var y;v!==l||0!==o&&3!==v.nodeType||(h=d+o),v!==u||0!==c&&3!==v.nodeType||(f=d+c),3===v.nodeType&&(d+=v.nodeValue.length),null!==(y=v.firstChild);)m=v,v=y;for(;;){if(v===s)break t;if(m===l&&++p===o&&(h=d),m===u&&++g===c&&(f=d),null!==(y=v.nextSibling))break;m=(v=m).parentNode}v=y}l=-1===h||-1===f?null:{start:h,end:f}}else l=null;l=l||{start:0,end:0}}else l=null;Rr={focusedElem:s,selectionRange:l},qt=!1,rl=null,il=!1,Us=r;do{try{Sl()}catch(e){if(null===Us)throw Error(a(330));Dl(Us,e),Us=Us.nextEffect}}while(null!==Us);rl=null,Us=r;do{try{for(s=e;null!==Us;){var b=Us.flags;if(16&b&&me(Us.stateNode,""),128&b){var C=Us.alternate;if(null!==C){var A=C.ref;null!==A&&("function"==typeof A?A(null):A.current=null)}}switch(1038&b){case 2:ms(Us),Us.flags&=-3;break;case 6:ms(Us),Us.flags&=-3,bs(Us.alternate,Us);break;case 1024:Us.flags&=-1025;break;case 1028:Us.flags&=-1025,bs(Us.alternate,Us);break;case 4:bs(Us.alternate,Us);break;case 8:ys(s,l=Us);var _=l.alternate;gs(l),null!==_&&gs(_)}Us=Us.nextEffect}}catch(e){if(null===Us)throw Error(a(330));Dl(Us,e),Us=Us.nextEffect}}while(null!==Us);if(A=Rr,C=dr(),b=A.focusedElem,s=A.selectionRange,C!==b&&b&&b.ownerDocument&&function e(t,n){return!(!t||!n)&&(t===n||(!t||3!==t.nodeType)&&(n&&3===n.nodeType?e(t,n.parentNode):"contains"in t?t.contains(n):!!t.compareDocumentPosition&&!!(16&t.compareDocumentPosition(n))))}(b.ownerDocument.documentElement,b)){null!==s&&hr(b)&&(C=s.start,void 0===(A=s.end)&&(A=C),"selectionStart"in b?(b.selectionStart=C,b.selectionEnd=Math.min(A,b.value.length)):(A=(C=b.ownerDocument||document)&&C.defaultView||window).getSelection&&(A=A.getSelection(),l=b.textContent.length,_=Math.min(s.start,l),s=void 0===s.end?_:Math.min(s.end,l),!A.extend&&_>s&&(l=s,s=_,_=l),l=cr(b,_),o=cr(b,s),l&&o&&(1!==A.rangeCount||A.anchorNode!==l.node||A.anchorOffset!==l.offset||A.focusNode!==o.node||A.focusOffset!==o.offset)&&((C=C.createRange()).setStart(l.node,l.offset),A.removeAllRanges(),_>s?(A.addRange(C),A.extend(o.node,o.offset)):(C.setEnd(o.node,o.offset),A.addRange(C))))),C=[];for(A=b;A=A.parentNode;)1===A.nodeType&&C.push({element:A,left:A.scrollLeft,top:A.scrollTop});for("function"==typeof b.focus&&b.focus(),b=0;b<C.length;b++)(A=C[b]).element.scrollLeft=A.left,A.element.scrollTop=A.top}qt=!!jr,Rr=jr=null,e.current=n,Us=r;do{try{for(b=e;null!==Us;){var I=Us.flags;if(36&I&&hs(b,Us.alternate,Us),128&I){C=void 0;var x=Us.ref;if(null!==x){var w=Us.stateNode;switch(Us.tag){case 5:C=w;break;default:C=w}"function"==typeof x?x(C):x.current=C}}Us=Us.nextEffect}}catch(e){if(null===Us)throw Error(a(330));Dl(Us,e),Us=Us.nextEffect}}while(null!==Us);Us=null,Di(),ws=i}else e.current=n;if(Gs)Gs=!1,Xs=e,qs=t;else for(Us=r;null!==Us;)t=Us.nextEffect,Us.nextEffect=null,8&Us.flags&&((I=Us).sibling=null,I.stateNode=null),Us=t;if(0===(r=e.pendingLanes)&&(Ws=null),1===r?e===$s?Js++:(Js=0,$s=e):Js=0,n=n.stateNode,bi&&"function"==typeof bi.onCommitFiberRoot)try{bi.onCommitFiberRoot(yi,n,void 0,64==(64&n.current.flags))}catch(e){}if(ul(e,zi()),Vs)throw Vs=!1,e=Ys,Ys=null,e;return 0!=(8&ws)||Vi(),null}function Sl(){for(;null!==Us;){var e=Us.alternate;il||null===rl||(0!=(8&Us.flags)?$e(Us,rl)&&(il=!0):13===Us.tag&&As(e,Us)&&$e(Us,rl)&&(il=!0));var t=Us.flags;0!=(256&t)&&ds(e,Us),0==(512&t)||Gs||(Gs=!0,Ui(97,(function(){return Tl(),null}))),Us=Us.nextEffect}}function Tl(){if(90!==qs){var e=97<qs?97:qs;return qs=90,Hi(e,kl)}return!1}function El(e,t){Zs.push(t,e),Gs||(Gs=!0,Ui(97,(function(){return Tl(),null})))}function Ol(e,t){Qs.push(t,e),Gs||(Gs=!0,Ui(97,(function(){return Tl(),null})))}function kl(){if(null===Xs)return!1;var e=Xs;if(Xs=null,0!=(48&ws))throw Error(a(331));var t=ws;ws|=32;var n=Qs;Qs=[];for(var r=0;r<n.length;r+=2){var i=n[r],o=n[r+1],s=i.destroy;if(i.destroy=void 0,"function"==typeof s)try{s()}catch(e){if(null===o)throw Error(a(330));Dl(o,e)}}for(n=Zs,Zs=[],r=0;r<n.length;r+=2){i=n[r],o=n[r+1];try{var l=i.create;i.destroy=l()}catch(e){if(null===o)throw Error(a(330));Dl(o,e)}}for(l=e.current.firstEffect;null!==l;)e=l.nextEffect,l.nextEffect=null,8&l.flags&&(l.sibling=null,l.stateNode=null),l=e;return ws=t,Vi(),!0}function Ll(e,t,n){ao(e,t=ss(0,t=is(n,t),1)),t=ol(),null!==(e=ll(e,1))&&(Ut(e,1,t),ul(e,t))}function Dl(e,t){if(3===e.tag)Ll(e,e,t);else for(var n=e.return;null!==n;){if(3===n.tag){Ll(n,e,t);break}if(1===n.tag){var r=n.stateNode;if("function"==typeof n.type.getDerivedStateFromError||"function"==typeof r.componentDidCatch&&(null===Ws||!Ws.has(r))){var i=ls(n,e=is(t,e),1);if(ao(n,i),i=ol(),null!==(n=ll(n,1)))Ut(n,1,i),ul(n,i);else if("function"==typeof r.componentDidCatch&&(null===Ws||!Ws.has(r)))try{r.componentDidCatch(t,e)}catch(e){}break}}n=n.return}}function Pl(e,t,n){var r=e.pingCache;null!==r&&r.delete(t),t=ol(),e.pingedLanes|=e.suspendedLanes&n,Ms===e&&(Ts&n)===n&&(4===ks||3===ks&&(62914560&Ts)===Ts&&500>zi()-zs?ml(e,0):js|=n),ul(e,t)}function Nl(e,t){var n=e.stateNode;null!==n&&n.delete(t),0===(t=0)&&(0==(2&(t=e.mode))?t=1:0==(4&t)?t=99===Fi()?1:2:(0===tl&&(tl=Ds),0===(t=Bt(62914560&~tl))&&(t=4194304))),n=ol(),null!==(e=ll(e,t))&&(Ut(e,t,n),ul(e,n))}function jl(e,t,n,r){this.tag=e,this.key=n,this.sibling=this.child=this.return=this.stateNode=this.type=this.elementType=null,this.index=0,this.ref=null,this.pendingProps=t,this.dependencies=this.memoizedState=this.updateQueue=this.memoizedProps=null,this.mode=r,this.flags=0,this.lastEffect=this.firstEffect=this.nextEffect=null,this.childLanes=this.lanes=0,this.alternate=null}function Rl(e,t,n,r){return new jl(e,t,n,r)}function zl(e){return!(!(e=e.prototype)||!e.isReactComponent)}function Fl(e,t){var n=e.alternate;return null===n?((n=Rl(e.tag,t,e.key,e.mode)).elementType=e.elementType,n.type=e.type,n.stateNode=e.stateNode,n.alternate=e,e.alternate=n):(n.pendingProps=t,n.type=e.type,n.flags=0,n.nextEffect=null,n.firstEffect=null,n.lastEffect=null),n.childLanes=e.childLanes,n.lanes=e.lanes,n.child=e.child,n.memoizedProps=e.memoizedProps,n.memoizedState=e.memoizedState,n.updateQueue=e.updateQueue,t=e.dependencies,n.dependencies=null===t?null:{lanes:t.lanes,firstContext:t.firstContext},n.sibling=e.sibling,n.index=e.index,n.ref=e.ref,n}function Bl(e,t,n,r,i,o){var s=2;if(r=e,"function"==typeof e)zl(e)&&(s=1);else if("string"==typeof e)s=5;else e:switch(e){case x:return Hl(n.children,i,o,t);case j:s=8,i|=16;break;case w:s=8,i|=1;break;case M:return(e=Rl(12,n,t,8|i)).elementType=M,e.type=M,e.lanes=o,e;case O:return(e=Rl(13,n,t,i)).type=O,e.elementType=O,e.lanes=o,e;case k:return(e=Rl(19,n,t,i)).elementType=k,e.lanes=o,e;case R:return Ul(n,i,o,t);case z:return(e=Rl(24,n,t,i)).elementType=z,e.lanes=o,e;default:if("object"==typeof e&&null!==e)switch(e.$$typeof){case S:s=10;break e;case T:s=9;break e;case E:s=11;break e;case L:s=14;break e;case D:s=16,r=null;break e;case P:s=22;break e}throw Error(a(130,null==e?e:typeof e,""))}return(t=Rl(s,n,t,i)).elementType=e,t.type=r,t.lanes=o,t}function Hl(e,t,n,r){return(e=Rl(7,e,r,t)).lanes=n,e}function Ul(e,t,n,r){return(e=Rl(23,e,r,t)).elementType=R,e.lanes=n,e}function Vl(e,t,n){return(e=Rl(6,e,null,t)).lanes=n,e}function Yl(e,t,n){return(t=Rl(4,null!==e.children?e.children:[],e.key,t)).lanes=n,t.stateNode={containerInfo:e.containerInfo,pendingChildren:null,implementation:e.implementation},t}function Wl(e,t,n){this.tag=t,this.containerInfo=e,this.finishedWork=this.pingCache=this.current=this.pendingChildren=null,this.timeoutHandle=-1,this.pendingContext=this.context=null,this.hydrate=n,this.callbackNode=null,this.callbackPriority=0,this.eventTimes=Ht(0),this.expirationTimes=Ht(-1),this.entangledLanes=this.finishedLanes=this.mutableReadLanes=this.expiredLanes=this.pingedLanes=this.suspendedLanes=this.pendingLanes=0,this.entanglements=Ht(0),this.mutableSourceEagerHydrationData=null}function Gl(e,t,n){var r=3<arguments.length&&void 0!==arguments[3]?arguments[3]:null;return{$$typeof:I,key:null==r?null:""+r,children:e,containerInfo:t,implementation:n}}function Xl(e,t,n,r){var i=t.current,o=ol(),s=al(i);e:if(n){t:{if(Ze(n=n._reactInternals)!==n||1!==n.tag)throw Error(a(170));var l=n;do{switch(l.tag){case 3:l=l.stateNode.context;break t;case 1:if(hi(l.type)){l=l.stateNode.__reactInternalMemoizedMergedChildContext;break t}}l=l.return}while(null!==l);throw Error(a(171))}if(1===n.tag){var u=n.type;if(hi(u)){n=gi(n,u,l);break e}}n=l}else n=si;return null===t.context?t.context=n:t.pendingContext=n,(t=oo(o,s)).payload={element:e},null!==(r=void 0===r?null:r)&&(t.callback=r),ao(i,t),sl(i,s,o),s}function ql(e){if(!(e=e.current).child)return null;switch(e.child.tag){case 5:default:return e.child.stateNode}}function Zl(e,t){if(null!==(e=e.memoizedState)&&null!==e.dehydrated){var n=e.retryLane;e.retryLane=0!==n&&n<t?n:t}}function Ql(e,t){Zl(e,t),(e=e.alternate)&&Zl(e,t)}function Kl(e,t,n){var r=null!=n&&null!=n.hydrationOptions&&n.hydrationOptions.mutableSources||null;if(n=new Wl(e,t,null!=n&&!0===n.hydrate),t=Rl(3,null,null,2===t?7:1===t?3:0),n.current=t,t.stateNode=n,ro(t),e[Zr]=n.current,Sr(8===e.nodeType?e.parentNode:e),r)for(e=0;e<r.length;e++){var i=(t=r[e])._getVersion;i=i(t._source),null==n.mutableSourceEagerHydrationData?n.mutableSourceEagerHydrationData=[t,i]:n.mutableSourceEagerHydrationData.push(t,i)}this._internalRoot=n}function Jl(e){return!(!e||1!==e.nodeType&&9!==e.nodeType&&11!==e.nodeType&&(8!==e.nodeType||" react-mount-point-unstable "!==e.nodeValue))}function $l(e,t,n,r,i){var o=n._reactRootContainer;if(o){var a=o._internalRoot;if("function"==typeof i){var s=i;i=function(){var e=ql(a);s.call(e)}}Xl(t,a,e,i)}else{if(o=n._reactRootContainer=function(e,t){if(t||(t=!(!(t=e?9===e.nodeType?e.documentElement:e.firstChild:null)||1!==t.nodeType||!t.hasAttribute("data-reactroot"))),!t)for(var n;n=e.lastChild;)e.removeChild(n);return new Kl(e,0,t?{hydrate:!0}:void 0)}(n,r),a=o._internalRoot,"function"==typeof i){var l=i;i=function(){var e=ql(a);l.call(e)}}pl((function(){Xl(t,a,e,i)}))}return ql(a)}function eu(e,t){var n=2<arguments.length&&void 0!==arguments[2]?arguments[2]:null;if(!Jl(t))throw Error(a(200));return Gl(e,t,null,n)}Hs=function(e,t,n){var r=t.lanes;if(null!==e)if(e.memoizedProps!==t.pendingProps||ui.current)La=!0;else{if(0==(n&r)){switch(La=!1,t.tag){case 3:Ua(t),Vo();break;case 5:ko(t);break;case 1:hi(t.type)&&vi(t);break;case 4:Eo(t,t.stateNode.containerInfo);break;case 10:r=t.memoizedProps.value;var i=t.type._context;ai(Xi,i._currentValue),i._currentValue=r;break;case 13:if(null!==t.memoizedState)return 0!=(n&t.child.childLanes)?Xa(e,t,n):(ai(Do,1&Do.current),null!==(t=es(e,t,n))?t.sibling:null);ai(Do,1&Do.current);break;case 19:if(r=0!=(n&t.childLanes),0!=(64&e.flags)){if(r)return $a(e,t,n);t.flags|=64}if(null!==(i=t.memoizedState)&&(i.rendering=null,i.tail=null,i.lastEffect=null),ai(Do,Do.current),r)break;return null;case 23:case 24:return t.lanes=0,Ra(e,t,n)}return es(e,t,n)}La=0!=(16384&e.flags)}else La=!1;switch(t.lanes=0,t.tag){case 2:if(r=t.type,null!==e&&(e.alternate=null,t.alternate=null,t.flags|=2),e=t.pendingProps,i=di(t,li.current),eo(t,n),i=na(null,t,r,e,i,n),t.flags|=1,"object"==typeof i&&null!==i&&"function"==typeof i.render&&void 0===i.$$typeof){if(t.tag=1,t.memoizedState=null,t.updateQueue=null,hi(r)){var o=!0;vi(t)}else o=!1;t.memoizedState=null!==i.state&&void 0!==i.state?i.state:null,ro(t);var s=r.getDerivedStateFromProps;"function"==typeof s&&ho(t,r,s,e),i.updater=fo,t.stateNode=i,i._reactInternals=t,mo(t,r,e,n),t=Ha(null,t,r,!0,o,n)}else t.tag=0,Da(null,t,i,n),t=t.child;return t;case 16:i=t.elementType;e:{switch(null!==e&&(e.alternate=null,t.alternate=null,t.flags|=2),e=t.pendingProps,i=(o=i._init)(i._payload),t.type=i,o=t.tag=function(e){if("function"==typeof e)return zl(e)?1:0;if(null!=e){if((e=e.$$typeof)===E)return 11;if(e===L)return 14}return 2}(i),e=Gi(i,e),o){case 0:t=Fa(null,t,i,e,n);break e;case 1:t=Ba(null,t,i,e,n);break e;case 11:t=Pa(null,t,i,e,n);break e;case 14:t=Na(null,t,i,Gi(i.type,e),r,n);break e}throw Error(a(306,i,""))}return t;case 0:return r=t.type,i=t.pendingProps,Fa(e,t,r,i=t.elementType===r?i:Gi(r,i),n);case 1:return r=t.type,i=t.pendingProps,Ba(e,t,r,i=t.elementType===r?i:Gi(r,i),n);case 3:if(Ua(t),r=t.updateQueue,null===e||null===r)throw Error(a(282));if(r=t.pendingProps,i=null!==(i=t.memoizedState)?i.element:null,io(e,t),lo(t,r,null,n),(r=t.memoizedState.element)===i)Vo(),t=es(e,t,n);else{if((o=(i=t.stateNode).hydrate)&&(jo=Vr(t.stateNode.containerInfo.firstChild),No=t,o=Ro=!0),o){if(null!=(e=i.mutableSourceEagerHydrationData))for(i=0;i<e.length;i+=2)(o=e[i])._workInProgressVersionPrimary=e[i+1],Yo.push(o);for(n=Io(t,null,r,n),t.child=n;n;)n.flags=-3&n.flags|1024,n=n.sibling}else Da(e,t,r,n),Vo();t=t.child}return t;case 5:return ko(t),null===e&&Bo(t),r=t.type,i=t.pendingProps,o=null!==e?e.memoizedProps:null,s=i.children,Fr(r,i)?s=null:null!==o&&Fr(r,o)&&(t.flags|=16),za(e,t),Da(e,t,s,n),t.child;case 6:return null===e&&Bo(t),null;case 13:return Xa(e,t,n);case 4:return Eo(t,t.stateNode.containerInfo),r=t.pendingProps,null===e?t.child=_o(t,null,r,n):Da(e,t,r,n),t.child;case 11:return r=t.type,i=t.pendingProps,Pa(e,t,r,i=t.elementType===r?i:Gi(r,i),n);case 7:return Da(e,t,t.pendingProps,n),t.child;case 8:case 12:return Da(e,t,t.pendingProps.children,n),t.child;case 10:e:{r=t.type._context,i=t.pendingProps,s=t.memoizedProps,o=i.value;var l=t.type._context;if(ai(Xi,l._currentValue),l._currentValue=o,null!==s)if(l=s.value,0===(o=ar(l,o)?0:0|("function"==typeof r._calculateChangedBits?r._calculateChangedBits(l,o):1073741823))){if(s.children===i.children&&!ui.current){t=es(e,t,n);break e}}else for(null!==(l=t.child)&&(l.return=t);null!==l;){var u=l.dependencies;if(null!==u){s=l.child;for(var c=u.firstContext;null!==c;){if(c.context===r&&0!=(c.observedBits&o)){1===l.tag&&((c=oo(-1,n&-n)).tag=2,ao(l,c)),l.lanes|=n,null!==(c=l.alternate)&&(c.lanes|=n),$i(l.return,n),u.lanes|=n;break}c=c.next}}else s=10===l.tag&&l.type===t.type?null:l.child;if(null!==s)s.return=l;else for(s=l;null!==s;){if(s===t){s=null;break}if(null!==(l=s.sibling)){l.return=s.return,s=l;break}s=s.return}l=s}Da(e,t,i.children,n),t=t.child}return t;case 9:return i=t.type,r=(o=t.pendingProps).children,eo(t,n),r=r(i=to(i,o.unstable_observedBits)),t.flags|=1,Da(e,t,r,n),t.child;case 14:return o=Gi(i=t.type,t.pendingProps),Na(e,t,i,o=Gi(i.type,o),r,n);case 15:return ja(e,t,t.type,t.pendingProps,r,n);case 17:return r=t.type,i=t.pendingProps,i=t.elementType===r?i:Gi(r,i),null!==e&&(e.alternate=null,t.alternate=null,t.flags|=2),t.tag=1,hi(r)?(e=!0,vi(t)):e=!1,eo(t,n),go(t,r,i),mo(t,r,i,n),Ha(null,t,r,!0,e,n);case 19:return $a(e,t,n);case 23:case 24:return Ra(e,t,n)}throw Error(a(156,t.tag))},Kl.prototype.render=function(e){Xl(e,this._internalRoot,null,null)},Kl.prototype.unmount=function(){var e=this._internalRoot,t=e.containerInfo;Xl(null,e,null,(function(){t[Zr]=null}))},et=function(e){13===e.tag&&(sl(e,4,ol()),Ql(e,4))},tt=function(e){13===e.tag&&(sl(e,67108864,ol()),Ql(e,67108864))},nt=function(e){if(13===e.tag){var t=ol(),n=al(e);sl(e,n,t),Ql(e,n)}},rt=function(e,t){return t()},Me=function(e,t,n){switch(t){case"input":if(ne(e,n),t=n.name,"radio"===n.type&&null!=t){for(n=e;n.parentNode;)n=n.parentNode;for(n=n.querySelectorAll("input[name="+JSON.stringify(""+t)+'][type="radio"]'),t=0;t<n.length;t++){var r=n[t];if(r!==e&&r.form===e.form){var i=ei(r);if(!i)throw Error(a(90));K(r),ne(r,i)}}}break;case"textarea":ue(e,n);break;case"select":null!=(t=n.value)&&ae(e,!!n.multiple,t,!1)}},Le=fl,De=function(e,t,n,r,i){var o=ws;ws|=4;try{return Hi(98,e.bind(null,t,n,r,i))}finally{0===(ws=o)&&(Bs(),Vi())}},Pe=function(){0==(49&ws)&&(function(){if(null!==Ks){var e=Ks;Ks=null,e.forEach((function(e){e.expiredLanes|=24&e.pendingLanes,ul(e,zi())}))}Vi()}(),Tl())},Ne=function(e,t){var n=ws;ws|=2;try{return e(t)}finally{0===(ws=n)&&(Bs(),Vi())}};var tu={Events:[Jr,$r,ei,Oe,ke,Tl,{current:!1}]},nu={findFiberByHostInstance:Kr,bundleType:0,version:"17.0.1",rendererPackageName:"react-dom"},ru={bundleType:nu.bundleType,version:nu.version,rendererPackageName:nu.rendererPackageName,rendererConfig:nu.rendererConfig,overrideHookState:null,overrideHookStateDeletePath:null,overrideHookStateRenamePath:null,overrideProps:null,overridePropsDeletePath:null,overridePropsRenamePath:null,setSuspenseHandler:null,scheduleUpdate:null,currentDispatcherRef:A.ReactCurrentDispatcher,findHostInstanceByFiber:function(e){return null===(e=Je(e))?null:e.stateNode},findFiberByHostInstance:nu.findFiberByHostInstance||function(){return null},findHostInstancesForRefresh:null,scheduleRefresh:null,scheduleRoot:null,setRefreshHandler:null,getCurrentFiber:null};if("undefined"!=typeof __REACT_DEVTOOLS_GLOBAL_HOOK__){var iu=__REACT_DEVTOOLS_GLOBAL_HOOK__;if(!iu.isDisabled&&iu.supportsFiber)try{yi=iu.inject(ru),bi=iu}catch(e){}}t.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED=tu,t.createPortal=eu,t.findDOMNode=function(e){if(null==e)return null;if(1===e.nodeType)return e;var t=e._reactInternals;if(void 0===t){if("function"==typeof e.render)throw Error(a(188));throw Error(a(268,Object.keys(e)))}return e=null===(e=Je(t))?null:e.stateNode},t.flushSync=function(e,t){var n=ws;if(0!=(48&n))return e(t);ws|=1;try{if(e)return Hi(99,e.bind(null,t))}finally{ws=n,Vi()}},t.hydrate=function(e,t,n){if(!Jl(t))throw Error(a(200));return $l(null,e,t,!0,n)},t.render=function(e,t,n){if(!Jl(t))throw Error(a(200));return $l(null,e,t,!1,n)},t.unmountComponentAtNode=function(e){if(!Jl(e))throw Error(a(40));return!!e._reactRootContainer&&(pl((function(){$l(null,null,e,!1,(function(){e._reactRootContainer=null,e[Zr]=null}))})),!0)},t.unstable_batchedUpdates=fl,t.unstable_createPortal=function(e,t){return eu(e,t,2<arguments.length&&void 0!==arguments[2]?arguments[2]:null)},t.unstable_renderSubtreeIntoContainer=function(e,t,n,r){if(!Jl(n))throw Error(a(200));if(null==e||void 0===e._reactInternals)throw Error(a(38));return $l(e,t,n,!1,r)},t.version="17.0.1"},function(e,t,n){"use strict";e.exports=n(1453)},function(e,t,n){"use strict";
/** @license React v0.20.1
 * scheduler.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var r,i,o,a;if("object"==typeof performance&&"function"==typeof performance.now){var s=performance;t.unstable_now=function(){return s.now()}}else{var l=Date,u=l.now();t.unstable_now=function(){return l.now()-u}}if("undefined"==typeof window||"function"!=typeof MessageChannel){var c=null,d=null,h=function(){if(null!==c)try{var e=t.unstable_now();c(!0,e),c=null}catch(e){throw setTimeout(h,0),e}};r=function(e){null!==c?setTimeout(r,0,e):(c=e,setTimeout(h,0))},i=function(e,t){d=setTimeout(e,t)},o=function(){clearTimeout(d)},t.unstable_shouldYield=function(){return!1},a=t.unstable_forceFrameRate=function(){}}else{var f=window.setTimeout,p=window.clearTimeout;if("undefined"!=typeof console){var g=window.cancelAnimationFrame;"function"!=typeof window.requestAnimationFrame&&console.error("This browser doesn't support requestAnimationFrame. Make sure that you load a polyfill in older browsers. https://reactjs.org/link/react-polyfills"),"function"!=typeof g&&console.error("This browser doesn't support cancelAnimationFrame. Make sure that you load a polyfill in older browsers. https://reactjs.org/link/react-polyfills")}var v=!1,m=null,y=-1,b=5,C=0;t.unstable_shouldYield=function(){return t.unstable_now()>=C},a=function(){},t.unstable_forceFrameRate=function(e){0>e||125<e?console.error("forceFrameRate takes a positive int between 0 and 125, forcing frame rates higher than 125 fps is not supported"):b=0<e?Math.floor(1e3/e):5};var A=new MessageChannel,_=A.port2;A.port1.onmessage=function(){if(null!==m){var e=t.unstable_now();C=e+b;try{m(!0,e)?_.postMessage(null):(v=!1,m=null)}catch(e){throw _.postMessage(null),e}}else v=!1},r=function(e){m=e,v||(v=!0,_.postMessage(null))},i=function(e,n){y=f((function(){e(t.unstable_now())}),n)},o=function(){p(y),y=-1}}function I(e,t){var n=e.length;e.push(t);e:for(;;){var r=n-1>>>1,i=e[r];if(!(void 0!==i&&0<M(i,t)))break e;e[r]=t,e[n]=i,n=r}}function x(e){return void 0===(e=e[0])?null:e}function w(e){var t=e[0];if(void 0!==t){var n=e.pop();if(n!==t){e[0]=n;e:for(var r=0,i=e.length;r<i;){var o=2*(r+1)-1,a=e[o],s=o+1,l=e[s];if(void 0!==a&&0>M(a,n))void 0!==l&&0>M(l,a)?(e[r]=l,e[s]=n,r=s):(e[r]=a,e[o]=n,r=o);else{if(!(void 0!==l&&0>M(l,n)))break e;e[r]=l,e[s]=n,r=s}}}return t}return null}function M(e,t){var n=e.sortIndex-t.sortIndex;return 0!==n?n:e.id-t.id}var S=[],T=[],E=1,O=null,k=3,L=!1,D=!1,P=!1;function N(e){for(var t=x(T);null!==t;){if(null===t.callback)w(T);else{if(!(t.startTime<=e))break;w(T),t.sortIndex=t.expirationTime,I(S,t)}t=x(T)}}function j(e){if(P=!1,N(e),!D)if(null!==x(S))D=!0,r(R);else{var t=x(T);null!==t&&i(j,t.startTime-e)}}function R(e,n){D=!1,P&&(P=!1,o()),L=!0;var r=k;try{for(N(n),O=x(S);null!==O&&(!(O.expirationTime>n)||e&&!t.unstable_shouldYield());){var a=O.callback;if("function"==typeof a){O.callback=null,k=O.priorityLevel;var s=a(O.expirationTime<=n);n=t.unstable_now(),"function"==typeof s?O.callback=s:O===x(S)&&w(S),N(n)}else w(S);O=x(S)}if(null!==O)var l=!0;else{var u=x(T);null!==u&&i(j,u.startTime-n),l=!1}return l}finally{O=null,k=r,L=!1}}var z=a;t.unstable_IdlePriority=5,t.unstable_ImmediatePriority=1,t.unstable_LowPriority=4,t.unstable_NormalPriority=3,t.unstable_Profiling=null,t.unstable_UserBlockingPriority=2,t.unstable_cancelCallback=function(e){e.callback=null},t.unstable_continueExecution=function(){D||L||(D=!0,r(R))},t.unstable_getCurrentPriorityLevel=function(){return k},t.unstable_getFirstCallbackNode=function(){return x(S)},t.unstable_next=function(e){switch(k){case 1:case 2:case 3:var t=3;break;default:t=k}var n=k;k=t;try{return e()}finally{k=n}},t.unstable_pauseExecution=function(){},t.unstable_requestPaint=z,t.unstable_runWithPriority=function(e,t){switch(e){case 1:case 2:case 3:case 4:case 5:break;default:e=3}var n=k;k=e;try{return t()}finally{k=n}},t.unstable_scheduleCallback=function(e,n,a){var s=t.unstable_now();switch("object"==typeof a&&null!==a?a="number"==typeof(a=a.delay)&&0<a?s+a:s:a=s,e){case 1:var l=-1;break;case 2:l=250;break;case 5:l=1073741823;break;case 4:l=1e4;break;default:l=5e3}return e={id:E++,callback:n,priorityLevel:e,startTime:a,expirationTime:l=a+l,sortIndex:-1},a>s?(e.sortIndex=a,I(T,e),null===x(S)&&e===x(T)&&(P?o():P=!0,i(j,a-s))):(e.sortIndex=l,I(S,e),D||L||(D=!0,r(R))),e},t.unstable_wrapCallback=function(e){var t=k;return function(){var n=k;k=t;try{return e.apply(this,arguments)}finally{k=n}}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.AbstractButton=void 0;var r=n(2),i=r.__importDefault(n(11)),o=r.__importStar(n(0)),a=n(239),s=n(873),l=n(1469),u=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.state={isActive:!1},t.handleKeyDown=function(e){var n,r;a.Keys.isKeyboardClick(e.which)&&(e.preventDefault(),e.which!==t.currentKeyDown&&t.setState({isActive:!0})),t.currentKeyDown=e.which,null===(r=(n=t.props).onKeyDown)||void 0===r||r.call(n,e)},t.handleKeyUp=function(e){var n,r,i;a.Keys.isKeyboardClick(e.which)&&(t.setState({isActive:!1}),null===(n=a.getRef(t.buttonRef))||void 0===n||n.click()),t.currentKeyDown=void 0,null===(i=(r=t.props).onKeyUp)||void 0===i||i.call(r,e)},t.handleBlur=function(e){var n,r;t.state.isActive&&t.setState({isActive:!1}),null===(r=(n=t.props).onBlur)||void 0===r||r.call(n,e)},t}return r.__extends(t,e),t.prototype.getCommonButtonProps=function(){var e,t=this.props,n=t.active,r=t.alignText,o=t.fill,s=t.large,l=t.loading,u=t.outlined,c=t.minimal,d=t.small,h=t.tabIndex,f=this.props.disabled||l;return{className:i.default(a.Classes.BUTTON,((e={})[a.Classes.ACTIVE]=!f&&(n||this.state.isActive),e[a.Classes.DISABLED]=f,e[a.Classes.FILL]=o,e[a.Classes.LARGE]=s,e[a.Classes.LOADING]=l,e[a.Classes.MINIMAL]=c,e[a.Classes.OUTLINED]=u,e[a.Classes.SMALL]=d,e),a.Classes.alignmentClass(r),a.Classes.intentClass(this.props.intent),this.props.className),disabled:f,onBlur:this.handleBlur,onClick:f?void 0:this.props.onClick,onKeyDown:this.handleKeyDown,onKeyUp:this.handleKeyUp,tabIndex:f?-1:h}},t.prototype.renderChildren=function(){var e=this.props,t=e.children,n=e.icon,r=e.loading,i=e.rightIcon,u=e.text;return[r&&o.createElement(l.Spinner,{key:"loading",className:a.Classes.BUTTON_SPINNER,size:s.Icon.SIZE_LARGE}),o.createElement(s.Icon,{key:"leftIcon",icon:n}),(!a.Utils.isReactNodeEmpty(u)||!a.Utils.isReactNodeEmpty(t))&&o.createElement("span",{key:"text",className:a.Classes.BUTTON_TEXT},u,t),o.createElement(s.Icon,{key:"rightIcon",icon:i})]},t}(a.AbstractPureComponent2);t.AbstractButton=u},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.AbstractComponent=void 0;var r=n(2),i=r.__importStar(n(0)),o=n(240),a=function(e){function t(t,n){var r=e.call(this,t,n)||this;return r.timeoutIds=[],r.clearTimeouts=function(){if(r.timeoutIds.length>0){for(var e=0,t=r.timeoutIds;e<t.length;e++){var n=t[e];window.clearTimeout(n)}r.timeoutIds=[]}},o.isNodeEnv("production")||r.validateProps(r.props),r}return r.__extends(t,e),t.prototype.componentWillReceiveProps=function(e){o.isNodeEnv("production")||this.validateProps(e)},t.prototype.componentWillUnmount=function(){this.clearTimeouts()},t.prototype.setTimeout=function(e,t){var n=window.setTimeout(e,t);return this.timeoutIds.push(n),function(){return window.clearTimeout(n)}},t.prototype.validateProps=function(e){},t}(i.Component);t.AbstractComponent=a},function(e,t,n){"use strict";function r(e,t,n){return void 0===n&&(n=function(e,t){return e===t}),null==e&&null==t||null!=e&&null!=t&&e.length===t.length&&e.every((function(e,r){return n(e,t[r])}))}function i(e,t,n){if(e===t)return!0;if(null==e&&null==t)return!0;if(null==e||null==t)return!1;if(Array.isArray(e)||Array.isArray(t))return r(e,t,i);if(s(e)||s(t))return e===t;if(null!=n)return a(e,t,n);if(e.constructor!==t.constructor)return!1;var o=Object.keys(e),l=Object.keys(t);return null!=o&&null!=l&&(0===o.length&&0===l.length||r(o,l)&&a(e,t,o))}function o(e,t,n){return function(e,t,n){if(function(e){return null!=e&&null!=e.include}(n))return n.include;if(function(e){return null!=e&&null!=e.exclude}(n)){var r=Object.keys(e),i=Object.keys(t),o=l(r.concat(i));return n.exclude.forEach((function(e){return delete o[e]})),Object.keys(o)}return[]}(e,t,n).every((function(n){return e.hasOwnProperty(n)===t.hasOwnProperty(n)&&e[n]===t[n]}))}function a(e,t,n){return n.every((function(n){return e.hasOwnProperty(n)===t.hasOwnProperty(n)&&i(e[n],t[n])}))}function s(e){return"number"==typeof e||"string"==typeof e||"boolean"==typeof e}function l(e){return e.reduce((function(e,t){return e[t]=!0,e}),{})}Object.defineProperty(t,"__esModule",{value:!0}),t.getDeepUnequalKeyValues=t.deepCompareKeys=t.shallowCompareKeys=t.arraysEqual=void 0,t.arraysEqual=r,t.shallowCompareKeys=function(e,t,n){if(null==e&&null==t)return!0;if(null==e||null==t)return!1;if(Array.isArray(e)||Array.isArray(t))return!1;if(null!=n)return o(e,t,n);var r=Object.keys(e),i=Object.keys(t);return o(e,t,{include:r})&&o(e,t,{include:i})},t.deepCompareKeys=i,t.getDeepUnequalKeyValues=function(e,t,n){return void 0===e&&(e={}),void 0===t&&(t={}),function(e,t,n,r){return n.filter((function(n){return!r(e,t,n)})).map((function(n){return{key:n,valueA:e[n],valueB:t[n]}}))}(e,t,null==n?function(e,t){var n=Object.keys(e),r=Object.keys(t),i=l(n.concat(r));return Object.keys(i)}(e,t):n,(function(e,t,n){return i(e,t,[n])}))}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.throttle=t.throttleReactEventCallback=t.throttleEvent=t.elementIsOrContains=void 0;var r=n(503);function i(e,t,n){var i=!1;return function(){for(var o=[],a=0;a<arguments.length;a++)o[a]=arguments[a];r.isFunction(t)&&t.apply(void 0,o),i||(i=!0,r.isFunction(n)&&n.apply(void 0,o),requestAnimationFrame((function(){e.apply(void 0,o),i=!1})))}}t.elementIsOrContains=function(e,t){return e===t||e.contains(t)},t.throttleEvent=function(e,t,n){var r=i((function(t){e.dispatchEvent(new CustomEvent(n,t))}));return e.addEventListener(t,r),r},t.throttleReactEventCallback=function(e,t){return void 0===t&&(t={}),i(e,(function(e){t.preventDefault&&e.preventDefault()}),(function(e){return e.persist()}))},t.throttle=function(e){return i(e)}},function(e,t,n){"use strict";(function(e){Object.defineProperty(t,"__esModule",{value:!0}),t.countDecimalPlaces=t.clamp=t.approxEqual=t.arrayLengthCompare=t.isNodeEnv=void 0;var r=n(504);t.isNodeEnv=function(t){return void 0!==e&&e.env&&"production"===t},t.arrayLengthCompare=function(e,t){return void 0===e&&(e=[]),void 0===t&&(t=[]),e.length-t.length},t.approxEqual=function(e,t,n){return void 0===n&&(n=1e-5),Math.abs(e-t)<=n},t.clamp=function(e,t,n){if(null==e)return e;if(n<t)throw new Error(r.CLAMP_MIN_MAX);return Math.min(Math.max(e,t),n)},t.countDecimalPlaces=function(e){if(!isFinite(e))return 0;for(var t=1,n=0;Math.round(e*t)/t!==e;)t*=10,n++;return n}}).call(this,n(145))},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.createReactRef=t.isElementOfType=t.getDisplayName=t.isReactElement=t.ensureElement=t.isReactChildrenElementOrElements=t.isReactNodeEmpty=void 0;var r=n(2).__importStar(n(0));function i(e,t){return void 0===t&&(t=!1),null==e||""===e||!1===e||!t&&Array.isArray(e)&&(0===e.length||e.every((function(e){return i(e,!0)})))}function o(e){return"object"==typeof e&&void 0!==e.type&&void 0!==e.props}t.isReactNodeEmpty=i,t.isReactChildrenElementOrElements=function(e){return!i(e,!0)&&!0!==e},t.ensureElement=function(e,t){return void 0===t&&(t="span"),null==e||"boolean"==typeof e?void 0:"string"==typeof e?e.trim().length>0?r.createElement(t,{},e):void 0:"number"==typeof e||"symbol"==typeof e.type||Array.isArray(e)?r.createElement(t,{},e):o(e)?e:void 0},t.isReactElement=o,t.getDisplayName=function(e){return e.displayName||e.name||"Unknown"},t.isElementOfType=function(e,t){return null!=e&&null!=e.type&&null!=e.type.displayName&&e.type.displayName===t.displayName},t.createReactRef=function(){return void 0!==r.createRef?r.createRef():{current:null}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.safeInvokeMember=void 0;var r=n(503);t.safeInvokeMember=function(e,t){for(var n=[],i=2;i<arguments.length;i++)n[i-2]=arguments[i];if(null!=e){var o=e[t];if(r.isFunction(o))return o.apply(void 0,n)}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.AbstractComponent2=void 0;var r=n(2),i=r.__importStar(n(0)),o=n(240),a=function(e){function t(t,n){var r=e.call(this,t,n)||this;return r.timeoutIds=[],r.requestIds=[],r.clearTimeouts=function(){if(r.timeoutIds.length>0){for(var e=0,t=r.timeoutIds;e<t.length;e++){var n=t[e];window.clearTimeout(n)}r.timeoutIds=[]}},r.cancelAnimationFrames=function(){if(r.requestIds.length>0){for(var e=0,t=r.requestIds;e<t.length;e++){var n=t[e];window.cancelAnimationFrame(n)}r.requestIds=[]}},o.isNodeEnv("production")||r.validateProps(r.props),r}return r.__extends(t,e),t.prototype.componentDidUpdate=function(e,t,n){o.isNodeEnv("production")||this.validateProps(this.props)},t.prototype.componentWillUnmount=function(){this.clearTimeouts(),this.cancelAnimationFrames()},t.prototype.requestAnimationFrame=function(e){var t=window.requestAnimationFrame(e);return this.requestIds.push(t),function(){return window.cancelAnimationFrame(t)}},t.prototype.setTimeout=function(e,t){var n=window.setTimeout(e,t);return this.timeoutIds.push(n),function(){return window.clearTimeout(n)}},t.prototype.validateProps=function(e){},t}(i.Component);t.AbstractComponent2=a},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.AbstractPureComponent=void 0;var r=n(2),i=r.__importStar(n(0)),o=n(240),a=function(e){function t(t,n){var r=e.call(this,t,n)||this;return r.timeoutIds=[],r.clearTimeouts=function(){if(r.timeoutIds.length>0){for(var e=0,t=r.timeoutIds;e<t.length;e++){var n=t[e];window.clearTimeout(n)}r.timeoutIds=[]}},o.isNodeEnv("production")||r.validateProps(r.props),r}return r.__extends(t,e),t.prototype.componentWillReceiveProps=function(e){o.isNodeEnv("production")||this.validateProps(e)},t.prototype.componentWillUnmount=function(){this.clearTimeouts()},t.prototype.setTimeout=function(e,t){var n=window.setTimeout(e,t);return this.timeoutIds.push(n),function(){return window.clearTimeout(n)}},t.prototype.validateProps=function(e){},t}(i.PureComponent);t.AbstractPureComponent=a},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.AbstractPureComponent2=void 0;var r=n(2),i=r.__importStar(n(0)),o=n(240),a=function(e){function t(t,n){var r=e.call(this,t,n)||this;return r.timeoutIds=[],r.requestIds=[],r.clearTimeouts=function(){if(r.timeoutIds.length>0){for(var e=0,t=r.timeoutIds;e<t.length;e++){var n=t[e];window.clearTimeout(n)}r.timeoutIds=[]}},r.cancelAnimationFrames=function(){if(r.requestIds.length>0){for(var e=0,t=r.requestIds;e<t.length;e++){var n=t[e];window.cancelAnimationFrame(n)}r.requestIds=[]}},o.isNodeEnv("production")||r.validateProps(r.props),r}return r.__extends(t,e),t.prototype.componentDidUpdate=function(e,t,n){o.isNodeEnv("production")||this.validateProps(this.props)},t.prototype.componentWillUnmount=function(){this.clearTimeouts(),this.cancelAnimationFrames()},t.prototype.requestAnimationFrame=function(e){var t=window.requestAnimationFrame(e);return this.requestIds.push(t),function(){return window.cancelAnimationFrame(t)}},t.prototype.setTimeout=function(e,t){var n=window.setTimeout(e,t);return this.timeoutIds.push(n),function(){return window.clearTimeout(n)}},t.prototype.validateProps=function(e){},t}(i.PureComponent);t.AbstractPureComponent2=a},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.Boundary=void 0,t.Boundary={START:"start",END:"end"}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.Colors=void 0,t.Colors={BLACK:"#10161A",BLUE1:"#0E5A8A",BLUE2:"#106BA3",BLUE3:"#137CBD",BLUE4:"#2B95D6",BLUE5:"#48AFF0",COBALT1:"#1F4B99",COBALT2:"#2458B3",COBALT3:"#2965CC",COBALT4:"#4580E6",COBALT5:"#669EFF",DARK_GRAY1:"#182026",DARK_GRAY2:"#202B33",DARK_GRAY3:"#293742",DARK_GRAY4:"#30404D",DARK_GRAY5:"#394B59",FOREST1:"#1D7324",FOREST2:"#238C2C",FOREST3:"#29A634",FOREST4:"#43BF4D",FOREST5:"#62D96B",GOLD1:"#A67908",GOLD2:"#BF8C0A",GOLD3:"#D99E0B",GOLD4:"#F2B824",GOLD5:"#FFC940",GRAY1:"#5C7080",GRAY2:"#738694",GRAY3:"#8A9BA8",GRAY4:"#A7B6C2",GRAY5:"#BFCCD6",GREEN1:"#0A6640",GREEN2:"#0D8050",GREEN3:"#0F9960",GREEN4:"#15B371",GREEN5:"#3DCC91",INDIGO1:"#5642A6",INDIGO2:"#634DBF",INDIGO3:"#7157D9",INDIGO4:"#9179F2",INDIGO5:"#AD99FF",LIGHT_GRAY1:"#CED9E0",LIGHT_GRAY2:"#D8E1E8",LIGHT_GRAY3:"#E1E8ED",LIGHT_GRAY4:"#EBF1F5",LIGHT_GRAY5:"#F5F8FA",LIME1:"#728C23",LIME2:"#87A629",LIME3:"#9BBF30",LIME4:"#B6D94C",LIME5:"#D1F26D",ORANGE1:"#A66321",ORANGE2:"#BF7326",ORANGE3:"#D9822B",ORANGE4:"#F29D49",ORANGE5:"#FFB366",RED1:"#A82A2A",RED2:"#C23030",RED3:"#DB3737",RED4:"#F55656",RED5:"#FF7373",ROSE1:"#A82255",ROSE2:"#C22762",ROSE3:"#DB2C6F",ROSE4:"#F5498B",ROSE5:"#FF66A1",SEPIA1:"#63411E",SEPIA2:"#7D5125",SEPIA3:"#96622D",SEPIA4:"#B07B46",SEPIA5:"#C99765",TURQUOISE1:"#008075",TURQUOISE2:"#00998C",TURQUOISE3:"#00B3A4",TURQUOISE4:"#14CCBD",TURQUOISE5:"#2EE6D6",VERMILION1:"#9E2B0E",VERMILION2:"#B83211",VERMILION3:"#D13913",VERMILION4:"#EB532D",VERMILION5:"#FF6E4A",VIOLET1:"#5C255C",VIOLET2:"#752F75",VIOLET3:"#8F398F",VIOLET4:"#A854A8",VIOLET5:"#C274C2",WHITE:"#FFFFFF"}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0})},function(e,t,n){"use strict";(function(e){Object.defineProperty(t,"__esModule",{value:!0}),t.ALERT_CONTENTS=t.ALERT_BODY=t.ALERT=t.RTL=t.LIST_UNSTYLED=t.LIST=t.HEADING=t.CODE_BLOCK=t.CODE=t.BLOCKQUOTE=t.TEXT_OVERFLOW_ELLIPSIS=t.TEXT_DISABLED=t.TEXT_MUTED=t.TEXT_SMALL=t.TEXT_LARGE=t.MONOSPACE_TEXT=t.RUNNING_TEXT=t.UI_TEXT=t.FOCUS_DISABLED=t.INTENT_DANGER=t.INTENT_WARNING=t.INTENT_SUCCESS=t.INTENT_PRIMARY=t.ELEVATION_4=t.ELEVATION_3=t.ELEVATION_2=t.ELEVATION_1=t.ELEVATION_0=t.POSITION_RIGHT=t.POSITION_LEFT=t.POSITION_BOTTOM=t.POSITION_TOP=t.VERTICAL=t.SMALL=t.ROUND=t.MULTILINE=t.OUTLINED=t.MINIMAL=t.LOADING=t.LARGE=t.INTERACTIVE=t.INLINE=t.FIXED_TOP=t.FIXED=t.FILL=t.DISABLED=t.DARK=t.ALIGN_RIGHT=t.ALIGN_LEFT=t.ACTIVE=void 0,t.INPUT_LEFT_CONTAINER=t.INPUT_GROUP=t.INPUT_GHOST=t.INPUT=t.HTML_TABLE_STRIPED=t.HTML_TABLE_CONDENSED=t.HTML_TABLE_BORDERED=t.HTML_TABLE=t.SELECT=t.HTML_SELECT=t.FLEX_EXPANDER=t.EDITABLE_TEXT_PLACEHOLDER=t.EDITABLE_TEXT_INPUT=t.EDITABLE_TEXT_EDITING=t.EDITABLE_TEXT_CONTENT=t.EDITABLE_TEXT=t.DRAWER_HEADER=t.DRAWER_FOOTER=t.DRAWER_BODY=t.DRAWER=t.DIVIDER=t.DIALOG_STEP_ICON=t.DIALOG_STEP_TITLE=t.DIALOG_STEP_CONTAINER=t.DIALOG_STEP=t.DIALOG_HEADER=t.DIALOG_FOOTER_ACTIONS=t.DIALOG_FOOTER=t.DIALOG_CLOSE_BUTTON=t.DIALOG_BODY=t.DIALOG_CONTAINER=t.DIALOG=t.CONTROL_GROUP=t.CONTEXT_MENU_POPOVER_TARGET=t.CONTEXT_MENU=t.COLLAPSIBLE_LIST=t.COLLAPSE_BODY=t.COLLAPSE=t.CARD=t.CALLOUT_ICON=t.CALLOUT=t.BUTTON_TEXT=t.BUTTON_SPINNER=t.BUTTON_GROUP=t.BUTTON=t.BREADCRUMBS_COLLAPSED=t.BREADCRUMBS=t.BREADCRUMB_CURRENT=t.BREADCRUMB=t.ALERT_FOOTER=void 0,t.OVERLAY_SCROLL_CONTAINER=t.OVERLAY_OPEN=t.OVERLAY_INLINE=t.OVERLAY_CONTENT=t.OVERLAY_CONTAINER=t.OVERLAY_BACKDROP=t.OVERLAY=t.OVERFLOW_LIST_SPACER=t.OVERFLOW_LIST=t.NUMERIC_INPUT=t.NON_IDEAL_STATE_VISUAL=t.NON_IDEAL_STATE=t.NAVBAR_DIVIDER=t.NAVBAR_HEADING=t.NAVBAR_GROUP=t.NAVBAR=t.MULTISTEP_DIALOG_FOOTER=t.MULTISTEP_DIALOG_RIGHT_PANEL=t.MULTISTEP_DIALOG_LEFT_PANEL=t.MULTISTEP_DIALOG_PANELS=t.MULTISTEP_DIALOG=t.MENU_HEADER=t.MENU_DIVIDER=t.MENU_SUBMENU=t.MENU_ITEM_LABEL=t.MENU_ITEM=t.MENU=t.FORM_HELPER_TEXT=t.FORM_CONTENT=t.FORM_GROUP=t.LABEL=t.HOTKEY_DIALOG=t.HOTKEY_COLUMN=t.HOTKEY_LABEL=t.HOTKEY=t.MODIFIER_KEY=t.KEY_COMBO=t.KEY=t.FILE_UPLOAD_INPUT_CUSTOM_TEXT=t.FILE_UPLOAD_INPUT=t.FILE_INPUT_HAS_SELECTION=t.FILE_INPUT=t.SWITCH_INNER_TEXT=t.SWITCH=t.RADIO=t.CHECKBOX=t.CONTROL_INDICATOR_CHILD=t.CONTROL_INDICATOR=t.CONTROL=t.INPUT_ACTION=void 0,t.TOOLTIP=t.TOAST_MESSAGE=t.TOAST_CONTAINER=t.TOAST=t.TAG_INPUT_VALUES=t.TAG_INPUT_ICON=t.TAG_INPUT=t.TAG_REMOVE=t.TAG=t.TABS=t.TAB_PANEL=t.TAB_LIST=t.TAB_INDICATOR_WRAPPER=t.TAB_INDICATOR=t.TAB=t.SPINNER_TRACK=t.SPINNER_NO_SPIN=t.SPINNER_HEAD=t.SPINNER_ANIMATION=t.SPINNER=t.END=t.START=t.SLIDER_PROGRESS=t.SLIDER_TRACK=t.SLIDER_LABEL=t.SLIDER_HANDLE=t.SLIDER_AXIS=t.SLIDER=t.SKELETON=t.PORTAL=t.PROGRESS_NO_ANIMATION=t.PROGRESS_NO_STRIPES=t.PROGRESS_METER=t.PROGRESS_BAR=t.TRANSITION_CONTAINER=t.POPOVER_WRAPPER=t.POPOVER_TARGET=t.POPOVER_OPEN=t.POPOVER_DISMISS_OVERRIDE=t.POPOVER_DISMISS=t.POPOVER_CONTENT_SIZING=t.POPOVER_CONTENT=t.POPOVER_CAPTURING_DISMISS=t.POPOVER_BACKDROP=t.POPOVER_ARROW=t.POPOVER=t.PANEL_STACK_VIEW=t.PANEL_STACK_HEADER_BACK=t.PANEL_STACK_HEADER=t.PANEL_STACK=void 0,t.positionClass=t.intentClass=t.iconClass=t.elevationClass=t.alignmentClass=t.getClassNamespace=t.ICON_LARGE=t.ICON_STANDARD=t.ICON=t.TREE_ROOT=t.TREE_NODE_SELECTED=t.TREE_NODE_SECONDARY_LABEL=t.TREE_NODE_LIST=t.TREE_NODE_LABEL=t.TREE_NODE_ICON=t.TREE_NODE_EXPANDED=t.TREE_NODE_CONTENT=t.TREE_NODE_CARET_OPEN=t.TREE_NODE_CARET_NONE=t.TREE_NODE_CARET_CLOSED=t.TREE_NODE_CARET=t.TREE_NODE=t.TREE=t.TOOLTIP_INDICATOR=void 0;var r=n(870),i=n(871),o=n(366),a=n(872),s=e.env.BLUEPRINT_NAMESPACE||e.env.REACT_APP_BLUEPRINT_NAMESPACE||"bp3";function l(e){if(void 0!==e)return s+"-elevation-"+e}function u(e){if(null!=e&&e!==o.Intent.NONE)return s+"-intent-"+e.toLowerCase()}function c(e){if(void 0!==e)return s+"-position-"+e}t.ACTIVE=s+"-active",t.ALIGN_LEFT=s+"-align-left",t.ALIGN_RIGHT=s+"-align-right",t.DARK=s+"-dark",t.DISABLED=s+"-disabled",t.FILL=s+"-fill",t.FIXED=s+"-fixed",t.FIXED_TOP=s+"-fixed-top",t.INLINE=s+"-inline",t.INTERACTIVE=s+"-interactive",t.LARGE=s+"-large",t.LOADING=s+"-loading",t.MINIMAL=s+"-minimal",t.OUTLINED=s+"-outlined",t.MULTILINE=s+"-multiline",t.ROUND=s+"-round",t.SMALL=s+"-small",t.VERTICAL=s+"-vertical",t.POSITION_TOP=c(a.Position.TOP),t.POSITION_BOTTOM=c(a.Position.BOTTOM),t.POSITION_LEFT=c(a.Position.LEFT),t.POSITION_RIGHT=c(a.Position.RIGHT),t.ELEVATION_0=l(i.Elevation.ZERO),t.ELEVATION_1=l(i.Elevation.ONE),t.ELEVATION_2=l(i.Elevation.TWO),t.ELEVATION_3=l(i.Elevation.THREE),t.ELEVATION_4=l(i.Elevation.FOUR),t.INTENT_PRIMARY=u(o.Intent.PRIMARY),t.INTENT_SUCCESS=u(o.Intent.SUCCESS),t.INTENT_WARNING=u(o.Intent.WARNING),t.INTENT_DANGER=u(o.Intent.DANGER),t.FOCUS_DISABLED=s+"-focus-disabled",t.UI_TEXT=s+"-ui-text",t.RUNNING_TEXT=s+"-running-text",t.MONOSPACE_TEXT=s+"-monospace-text",t.TEXT_LARGE=s+"-text-large",t.TEXT_SMALL=s+"-text-small",t.TEXT_MUTED=s+"-text-muted",t.TEXT_DISABLED=s+"-text-disabled",t.TEXT_OVERFLOW_ELLIPSIS=s+"-text-overflow-ellipsis",t.BLOCKQUOTE=s+"-blockquote",t.CODE=s+"-code",t.CODE_BLOCK=s+"-code-block",t.HEADING=s+"-heading",t.LIST=s+"-list",t.LIST_UNSTYLED=s+"-list-unstyled",t.RTL=s+"-rtl",t.ALERT=s+"-alert",t.ALERT_BODY=t.ALERT+"-body",t.ALERT_CONTENTS=t.ALERT+"-contents",t.ALERT_FOOTER=t.ALERT+"-footer",t.BREADCRUMB=s+"-breadcrumb",t.BREADCRUMB_CURRENT=t.BREADCRUMB+"-current",t.BREADCRUMBS=t.BREADCRUMB+"s",t.BREADCRUMBS_COLLAPSED=t.BREADCRUMB+"s-collapsed",t.BUTTON=s+"-button",t.BUTTON_GROUP=t.BUTTON+"-group",t.BUTTON_SPINNER=t.BUTTON+"-spinner",t.BUTTON_TEXT=t.BUTTON+"-text",t.CALLOUT=s+"-callout",t.CALLOUT_ICON=t.CALLOUT+"-icon",t.CARD=s+"-card",t.COLLAPSE=s+"-collapse",t.COLLAPSE_BODY=t.COLLAPSE+"-body",t.COLLAPSIBLE_LIST=s+"-collapse-list",t.CONTEXT_MENU=s+"-context-menu",t.CONTEXT_MENU_POPOVER_TARGET=t.CONTEXT_MENU+"-popover-target",t.CONTROL_GROUP=s+"-control-group",t.DIALOG=s+"-dialog",t.DIALOG_CONTAINER=t.DIALOG+"-container",t.DIALOG_BODY=t.DIALOG+"-body",t.DIALOG_CLOSE_BUTTON=t.DIALOG+"-close-button",t.DIALOG_FOOTER=t.DIALOG+"-footer",t.DIALOG_FOOTER_ACTIONS=t.DIALOG+"-footer-actions",t.DIALOG_HEADER=t.DIALOG+"-header",t.DIALOG_STEP=s+"-dialog-step",t.DIALOG_STEP_CONTAINER=t.DIALOG_STEP+"-container",t.DIALOG_STEP_TITLE=t.DIALOG_STEP+"-title",t.DIALOG_STEP_ICON=t.DIALOG_STEP+"-icon",t.DIVIDER=s+"-divider",t.DRAWER=s+"-drawer",t.DRAWER_BODY=t.DRAWER+"-body",t.DRAWER_FOOTER=t.DRAWER+"-footer",t.DRAWER_HEADER=t.DRAWER+"-header",t.EDITABLE_TEXT=s+"-editable-text",t.EDITABLE_TEXT_CONTENT=t.EDITABLE_TEXT+"-content",t.EDITABLE_TEXT_EDITING=t.EDITABLE_TEXT+"-editing",t.EDITABLE_TEXT_INPUT=t.EDITABLE_TEXT+"-input",t.EDITABLE_TEXT_PLACEHOLDER=t.EDITABLE_TEXT+"-placeholder",t.FLEX_EXPANDER=s+"-flex-expander",t.HTML_SELECT=s+"-html-select",t.SELECT=s+"-select",t.HTML_TABLE=s+"-html-table",t.HTML_TABLE_BORDERED=t.HTML_TABLE+"-bordered",t.HTML_TABLE_CONDENSED=t.HTML_TABLE+"-condensed",t.HTML_TABLE_STRIPED=t.HTML_TABLE+"-striped",t.INPUT=s+"-input",t.INPUT_GHOST=t.INPUT+"-ghost",t.INPUT_GROUP=t.INPUT+"-group",t.INPUT_LEFT_CONTAINER=t.INPUT+"-left-container",t.INPUT_ACTION=t.INPUT+"-action",t.CONTROL=s+"-control",t.CONTROL_INDICATOR=t.CONTROL+"-indicator",t.CONTROL_INDICATOR_CHILD=t.CONTROL_INDICATOR+"-child",t.CHECKBOX=s+"-checkbox",t.RADIO=s+"-radio",t.SWITCH=s+"-switch",t.SWITCH_INNER_TEXT=t.SWITCH+"-inner-text",t.FILE_INPUT=s+"-file-input",t.FILE_INPUT_HAS_SELECTION=s+"-file-input-has-selection",t.FILE_UPLOAD_INPUT=s+"-file-upload-input",t.FILE_UPLOAD_INPUT_CUSTOM_TEXT=s+"-file-upload-input-custom-text",t.KEY=s+"-key",t.KEY_COMBO=t.KEY+"-combo",t.MODIFIER_KEY=s+"-modifier-key",t.HOTKEY=s+"-hotkey",t.HOTKEY_LABEL=t.HOTKEY+"-label",t.HOTKEY_COLUMN=t.HOTKEY+"-column",t.HOTKEY_DIALOG=t.HOTKEY+"-dialog",t.LABEL=s+"-label",t.FORM_GROUP=s+"-form-group",t.FORM_CONTENT=s+"-form-content",t.FORM_HELPER_TEXT=s+"-form-helper-text",t.MENU=s+"-menu",t.MENU_ITEM=t.MENU+"-item",t.MENU_ITEM_LABEL=t.MENU_ITEM+"-label",t.MENU_SUBMENU=s+"-submenu",t.MENU_DIVIDER=t.MENU+"-divider",t.MENU_HEADER=t.MENU+"-header",t.MULTISTEP_DIALOG=s+"-multistep-dialog",t.MULTISTEP_DIALOG_PANELS=t.MULTISTEP_DIALOG+"-panels",t.MULTISTEP_DIALOG_LEFT_PANEL=t.MULTISTEP_DIALOG+"-left-panel",t.MULTISTEP_DIALOG_RIGHT_PANEL=t.MULTISTEP_DIALOG+"-right-panel",t.MULTISTEP_DIALOG_FOOTER=t.MULTISTEP_DIALOG+"-footer",t.NAVBAR=s+"-navbar",t.NAVBAR_GROUP=t.NAVBAR+"-group",t.NAVBAR_HEADING=t.NAVBAR+"-heading",t.NAVBAR_DIVIDER=t.NAVBAR+"-divider",t.NON_IDEAL_STATE=s+"-non-ideal-state",t.NON_IDEAL_STATE_VISUAL=t.NON_IDEAL_STATE+"-visual",t.NUMERIC_INPUT=s+"-numeric-input",t.OVERFLOW_LIST=s+"-overflow-list",t.OVERFLOW_LIST_SPACER=t.OVERFLOW_LIST+"-spacer",t.OVERLAY=s+"-overlay",t.OVERLAY_BACKDROP=t.OVERLAY+"-backdrop",t.OVERLAY_CONTAINER=t.OVERLAY+"-container",t.OVERLAY_CONTENT=t.OVERLAY+"-content",t.OVERLAY_INLINE=t.OVERLAY+"-inline",t.OVERLAY_OPEN=t.OVERLAY+"-open",t.OVERLAY_SCROLL_CONTAINER=t.OVERLAY+"-scroll-container",t.PANEL_STACK=s+"-panel-stack",t.PANEL_STACK_HEADER=t.PANEL_STACK+"-header",t.PANEL_STACK_HEADER_BACK=t.PANEL_STACK+"-header-back",t.PANEL_STACK_VIEW=t.PANEL_STACK+"-view",t.POPOVER=s+"-popover",t.POPOVER_ARROW=t.POPOVER+"-arrow",t.POPOVER_BACKDROP=t.POPOVER+"-backdrop",t.POPOVER_CAPTURING_DISMISS=t.POPOVER+"-capturing-dismiss",t.POPOVER_CONTENT=t.POPOVER+"-content",t.POPOVER_CONTENT_SIZING=t.POPOVER_CONTENT+"-sizing",t.POPOVER_DISMISS=t.POPOVER+"-dismiss",t.POPOVER_DISMISS_OVERRIDE=t.POPOVER_DISMISS+"-override",t.POPOVER_OPEN=t.POPOVER+"-open",t.POPOVER_TARGET=t.POPOVER+"-target",t.POPOVER_WRAPPER=t.POPOVER+"-wrapper",t.TRANSITION_CONTAINER=s+"-transition-container",t.PROGRESS_BAR=s+"-progress-bar",t.PROGRESS_METER=s+"-progress-meter",t.PROGRESS_NO_STRIPES=s+"-no-stripes",t.PROGRESS_NO_ANIMATION=s+"-no-animation",t.PORTAL=s+"-portal",t.SKELETON=s+"-skeleton",t.SLIDER=s+"-slider",t.SLIDER_AXIS=t.SLIDER+"-axis",t.SLIDER_HANDLE=t.SLIDER+"-handle",t.SLIDER_LABEL=t.SLIDER+"-label",t.SLIDER_TRACK=t.SLIDER+"-track",t.SLIDER_PROGRESS=t.SLIDER+"-progress",t.START=s+"-start",t.END=s+"-end",t.SPINNER=s+"-spinner",t.SPINNER_ANIMATION=t.SPINNER+"-animation",t.SPINNER_HEAD=t.SPINNER+"-head",t.SPINNER_NO_SPIN=s+"-no-spin",t.SPINNER_TRACK=t.SPINNER+"-track",t.TAB=s+"-tab",t.TAB_INDICATOR=t.TAB+"-indicator",t.TAB_INDICATOR_WRAPPER=t.TAB_INDICATOR+"-wrapper",t.TAB_LIST=t.TAB+"-list",t.TAB_PANEL=t.TAB+"-panel",t.TABS=t.TAB+"s",t.TAG=s+"-tag",t.TAG_REMOVE=t.TAG+"-remove",t.TAG_INPUT=s+"-tag-input",t.TAG_INPUT_ICON=t.TAG_INPUT+"-icon",t.TAG_INPUT_VALUES=t.TAG_INPUT+"-values",t.TOAST=s+"-toast",t.TOAST_CONTAINER=t.TOAST+"-container",t.TOAST_MESSAGE=t.TOAST+"-message",t.TOOLTIP=s+"-tooltip",t.TOOLTIP_INDICATOR=t.TOOLTIP+"-indicator",t.TREE=s+"-tree",t.TREE_NODE=s+"-tree-node",t.TREE_NODE_CARET=t.TREE_NODE+"-caret",t.TREE_NODE_CARET_CLOSED=t.TREE_NODE_CARET+"-closed",t.TREE_NODE_CARET_NONE=t.TREE_NODE_CARET+"-none",t.TREE_NODE_CARET_OPEN=t.TREE_NODE_CARET+"-open",t.TREE_NODE_CONTENT=t.TREE_NODE+"-content",t.TREE_NODE_EXPANDED=t.TREE_NODE+"-expanded",t.TREE_NODE_ICON=t.TREE_NODE+"-icon",t.TREE_NODE_LABEL=t.TREE_NODE+"-label",t.TREE_NODE_LIST=t.TREE_NODE+"-list",t.TREE_NODE_SECONDARY_LABEL=t.TREE_NODE+"-secondary-label",t.TREE_NODE_SELECTED=t.TREE_NODE+"-selected",t.TREE_ROOT=s+"-tree-root",t.ICON=s+"-icon",t.ICON_STANDARD=t.ICON+"-standard",t.ICON_LARGE=t.ICON+"-large",t.getClassNamespace=function(){return s},t.alignmentClass=function(e){switch(e){case r.Alignment.LEFT:return t.ALIGN_LEFT;case r.Alignment.RIGHT:return t.ALIGN_RIGHT;default:return}},t.elevationClass=l,t.iconClass=function(e){if(null!=e)return 0===e.indexOf(s+"-icon-")?e:s+"-icon-"+e},t.intentClass=u,t.positionClass=c}).call(this,n(145))},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.isKeyboardClick=t.DELETE=t.ARROW_DOWN=t.ARROW_RIGHT=t.ARROW_UP=t.ARROW_LEFT=t.SPACE=t.ESCAPE=t.SHIFT=t.ENTER=t.TAB=t.BACKSPACE=void 0,t.BACKSPACE=8,t.TAB=9,t.ENTER=13,t.SHIFT=16,t.ESCAPE=27,t.SPACE=32,t.ARROW_LEFT=37,t.ARROW_UP=38,t.ARROW_RIGHT=39,t.ARROW_DOWN=40,t.DELETE=46,t.isKeyboardClick=function(e){return e===t.ENTER||e===t.SPACE}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.Spinner=void 0;var r=n(2),i=r.__importDefault(n(11)),o=r.__importStar(n(0)),a=n(13),s=n(239),l=n(504),u=n(209),c=n(240),d=45,h="M 50,50 m 0,-45 a 45,45 0 1 1 0,90 a 45,45 0 1 1 0,-90",f=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}var n;return r.__extends(t,e),n=t,t.prototype.componentDidUpdate=function(e){e.value!==this.props.value&&this.forceUpdate()},t.prototype.render=function(){var e,t=this.props,r=t.className,a=t.intent,l=t.value,u=t.tagName,d=void 0===u?"div":u,f=this.getSize(),p=i.default(s.Classes.SPINNER,s.Classes.intentClass(a),((e={})[s.Classes.SPINNER_NO_SPIN]=null!=l,e),r),g=Math.min(16,4*n.SIZE_LARGE/f),v=280-280*(null==l?.25:c.clamp(l,0,1));return o.createElement(d,{className:p},o.createElement(d,{className:s.Classes.SPINNER_ANIMATION},o.createElement("svg",{width:f,height:f,strokeWidth:g.toFixed(2),viewBox:this.getViewBox(g)},o.createElement("path",{className:s.Classes.SPINNER_TRACK,d:h}),o.createElement("path",{className:s.Classes.SPINNER_HEAD,d:h,pathLength:280,strokeDasharray:"280 280",strokeDashoffset:v}))))},t.prototype.validateProps=function(e){var t=e.className,n=void 0===t?"":t;null!=e.size&&(n.indexOf(s.Classes.SMALL)>=0||n.indexOf(s.Classes.LARGE)>=0)&&console.warn(l.SPINNER_WARN_CLASSES_SIZE)},t.prototype.getSize=function(){var e=this.props,t=e.className,r=void 0===t?"":t,i=e.size;return null==i?r.indexOf(s.Classes.SMALL)>=0?n.SIZE_SMALL:r.indexOf(s.Classes.LARGE)>=0?n.SIZE_LARGE:n.SIZE_STANDARD:Math.max(10,i)},t.prototype.getViewBox=function(e){var t=d+e/2,n=(50-t).toFixed(2),r=(2*t).toFixed(2);return n+" "+n+" "+r+" "+r},t.displayName=u.DISPLAYNAME_PREFIX+".Spinner",t.SIZE_SMALL=20,t.SIZE_STANDARD=50,t.SIZE_LARGE=100,t=n=r.__decorate([a.polyfill],t)}(s.AbstractPureComponent2);t.Spinner=f},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.AsyncControllableInput=void 0;var r=n(2),i=r.__importStar(n(0)),o=n(13),a=n(209),s=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.state={hasPendingUpdate:!1,isComposing:!1,nextValue:t.props.value,value:t.props.value},t.handleCompositionStart=function(e){var n,r;t.setState({isComposing:!0,nextValue:t.state.value}),null===(r=(n=t.props).onCompositionStart)||void 0===r||r.call(n,e)},t.handleCompositionEnd=function(e){var n,r;t.setState({isComposing:!1}),null===(r=(n=t.props).onCompositionEnd)||void 0===r||r.call(n,e)},t.handleChange=function(e){var n,r,i=e.target.value;t.setState({nextValue:i}),null===(r=(n=t.props).onChange)||void 0===r||r.call(n,e)},t}return r.__extends(t,e),t.getDerivedStateFromProps=function(e,t){return t.isComposing||void 0===e.value?null:t.nextValue!==t.value?e.value===t.nextValue?t.hasPendingUpdate?{value:e.value,hasPendingUpdate:!1}:{value:t.nextValue}:e.value===t.value?{hasPendingUpdate:!0}:{value:e.value,nextValue:e.value,hasPendingUpdate:!1}:{value:e.value,nextValue:e.value,hasPendingUpdate:!1}},t.prototype.render=function(){var e=this.state,t=e.isComposing,n=e.hasPendingUpdate,o=e.value,a=e.nextValue,s=this.props,l=s.inputRef,u=r.__rest(s,["inputRef"]);return i.createElement("input",r.__assign({},u,{ref:l,value:t||n?a:o,onCompositionStart:this.handleCompositionStart,onCompositionEnd:this.handleCompositionEnd,onChange:this.handleChange}))},t.displayName=a.DISPLAYNAME_PREFIX+".AsyncControllableInput",t=r.__decorate([o.polyfill],t)}(i.PureComponent);t.AsyncControllableInput=s},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(1472),i=n(1473),o=n(874),a=function(){return r.create()},s=function(){function e(e){var t=this,n=e.autoGenerateTag;this.cssRaw=function(e){e&&(t._raw+=e||"",t._pendingRawChange=!0,t._styleUpdated())},this.cssRule=function(e){for(var n=[],r=1;r<arguments.length;r++)n[r-1]=arguments[r];var a=i.convertToStyles(o.extend.apply(void 0,n));t._freeStyle.registerRule(e,a),t._styleUpdated()},this.forceRenderStyles=function(){var e=t._getTag();e&&(e.textContent=t.getStyles())},this.fontFace=function(){for(var e=[],n=0;n<arguments.length;n++)e[n]=arguments[n];for(var r=t._freeStyle,i=0,o=e;i<o.length;i++){var a=o[i];r.registerRule("@font-face",a)}t._styleUpdated()},this.getStyles=function(){return(t._raw||"")+t._freeStyle.getStyles()},this.keyframes=function(e){var n=i.convertToKeyframes(e),r=t._freeStyle.registerKeyframes(n);return t._styleUpdated(),r},this.reinit=function(){var e=a();t._freeStyle=e,t._lastFreeStyleChangeId=e.changeId,t._raw="",t._pendingRawChange=!1;var n=t._getTag();n&&(n.textContent="")},this.setStylesTarget=function(e){t._tag&&(t._tag.textContent=""),t._tag=e,t.forceRenderStyles()},this.stylesheet=function(e){for(var n={},r=0,i=Object.getOwnPropertyNames(e);r<i.length;r++){var o=i[r],a=e[o];a&&(a.$debugName=o,n[o]=t.style(a))}return n};var r=a();this._autoGenerateTag=n,this._freeStyle=r,this._lastFreeStyleChangeId=r.changeId,this._pending=0,this._pendingRawChange=!1,this._raw="",this._tag=void 0,this.style=this.style.bind(this)}return e.prototype._afterAllSync=function(e){var t=this;this._pending++;var n=this._pending;o.raf((function(){n===t._pending&&e()}))},e.prototype._getTag=function(){if(this._tag)return this._tag;if(this._autoGenerateTag){var e="undefined"==typeof window?{textContent:""}:document.createElement("style");return"undefined"!=typeof document&&document.head.appendChild(e),this._tag=e,e}},e.prototype._styleUpdated=function(){var e=this,t=this._freeStyle.changeId,n=this._lastFreeStyleChangeId;(this._pendingRawChange||t!==n)&&(this._lastFreeStyleChangeId=t,this._pendingRawChange=!1,this._afterAllSync((function(){return e.forceRenderStyles()})))},e.prototype.style=function(){var e=this._freeStyle.registerStyle(i.convertToStyles(o.extend.apply(void 0,arguments)));return this._styleUpdated(),e},e}();t.TypeStyle=s},function(e,t,n){"use strict";n.r(t),n.d(t,"Cache",(function(){return p})),n.d(t,"Selector",(function(){return g})),n.d(t,"Style",(function(){return v})),n.d(t,"Rule",(function(){return m})),n.d(t,"FreeStyle",(function(){return b})),n.d(t,"create",(function(){return C}));let r=0;const i=Object.create(null),o=["animation-iteration-count","border-image-outset","border-image-slice","border-image-width","box-flex","box-flex-group","box-ordinal-group","column-count","columns","counter-increment","counter-reset","flex","flex-grow","flex-positive","flex-shrink","flex-negative","flex-order","font-weight","grid-area","grid-column","grid-column-end","grid-column-span","grid-column-start","grid-row","grid-row-end","grid-row-span","grid-row-start","line-clamp","line-height","opacity","order","orphans","tab-size","widows","z-index","zoom","fill-opacity","flood-opacity","stop-opacity","stroke-dasharray","stroke-dashoffset","stroke-miterlimit","stroke-opacity","stroke-width"];for(const e of o)for(const t of["-webkit-","-ms-","-moz-","-o-",""])i[t+e]=!0;function a(e,t){return t&&"number"==typeof t&&!i[e]?`${e}:${t}px`:`${e}:${t}`}function s(e){return e.sort((e,t)=>e[0]>t[0]?1:-1)}function l(e){return e.map(([e,t])=>Array.isArray(t)?t.map(t=>a(e,t)).join(";"):a(e,t)).join(";")}function u(e,t){return-1===e.indexOf("&")?`${t} ${e}`:e.replace(/&/g,t)}function c(e,t,n,r,i){const{style:o,nested:a,isUnique:d}=function(e,t){const n=[],r=[];for(const t of Object.keys(e)){const o=t.trim(),a=e[t];36!==o.charCodeAt(0)&&null!=a&&("object"!=typeof a||Array.isArray(a)?n.push([(i=o,i.replace(/[A-Z]/g,e=>"-"+e.toLowerCase()).replace(/^ms-/,"-ms-")),a]):r.push([o,a]))}var i;return{style:l(s(n)),nested:t?r:s(r),isUnique:!!e.$unique}}(t,""!==e);let h=o;if(64===e.charCodeAt(0)){const t={selector:e,styles:[],rules:[],style:i?"":o};n.push(t),o&&i&&t.styles.push({selector:i,style:o,isUnique:d});for(const[e,n]of a)h+=e+c(e,n,t.rules,t.styles,i)}else{const t=i?u(e,i):e;o&&r.push({selector:t,style:o,isUnique:d});for(const[e,i]of a)h+=e+c(e,i,n,r,t)}return h}function d(e,t,n,i,o,a){for(const{selector:n,style:s,isUnique:l}of i){const i=a?u(n,o):n,c=l?"u\0"+(++r).toString(36):`s\0${t}\0${s}`,d=new v(s,c);d.add(new g(i,`k\0${t}\0${i}`)),e.add(d)}for(const{selector:r,style:i,rules:s,styles:l}of n){const n=new m(r,i,`r\0${t}\0${r}\0${i}`);d(n,t,s,l,o,a),e.add(n)}}function h(e){let t="";for(let n=0;n<e.length;n++)t+=e[n];return t}const f={add:()=>{},change:()=>{},remove:()=>{}};class p{constructor(e=f){this.changes=e,this.sheet=[],this.changeId=0,this._keys=[],this._children=Object.create(null),this._counters=Object.create(null)}add(e){const t=this._counters[e.id]||0,n=this._children[e.id]||e.clone();if(this._counters[e.id]=t+1,0===t)this._children[n.id]=n,this._keys.push(n.id),this.sheet.push(n.getStyles()),this.changeId++,this.changes.add(n,this._keys.length-1);else if(n instanceof p&&e instanceof p){const t=this._keys.indexOf(e.id),r=n.changeId;n.merge(e),n.changeId!==r&&(this.sheet.splice(t,1,n.getStyles()),this.changeId++,this.changes.change(n,t,t))}}remove(e){const t=this._counters[e.id];if(t){this._counters[e.id]=t-1;const n=this._children[e.id],r=this._keys.indexOf(n.id);if(1===t)delete this._counters[e.id],delete this._children[e.id],this._keys.splice(r,1),this.sheet.splice(r,1),this.changeId++,this.changes.remove(n,r);else if(n instanceof p&&e instanceof p){const t=n.changeId;n.unmerge(e),n.changeId!==t&&(this.sheet.splice(r,1,n.getStyles()),this.changeId++,this.changes.change(n,r,r))}}}values(){return this._keys.map(e=>this._children[e])}merge(e){for(const t of e.values())this.add(t);return this}unmerge(e){for(const t of e.values())this.remove(t);return this}clone(){return(new p).merge(this)}}class g{constructor(e,t){this.selector=e,this.id=t}getStyles(){return this.selector}clone(){return this}}class v extends p{constructor(e,t){super(),this.style=e,this.id=t}getStyles(){return`${this.sheet.join(",")}{${this.style}}`}clone(){return new v(this.style,this.id).merge(this)}}class m extends p{constructor(e,t,n){super(),this.rule=e,this.style=t,this.id=n}getStyles(){return`${this.rule}{${this.style}${h(this.sheet)}}`}clone(){return new m(this.rule,this.style,this.id).merge(this)}}function y(e,t){const n="f"+function(e){let t=5381,n=e.length;for(;n--;)t=33*t^e.charCodeAt(n);return(t>>>0).toString(36)}(e);return n}class b extends p{constructor(e,t){super(t),this.id=e}registerStyle(e){const t=[],n=[],r=c("&",e,t,n),i=y(r);return d(this,r,t,n,"."+i,!0),i}registerKeyframes(e){return this.registerHashRule("@keyframes",e)}registerHashRule(e,t){const n=[],r=[],i=c("",t,n,r),o=y(i),a=new m(`${e} ${o}`,"",`h\0${i}\0${e}`);return d(a,i,n,r,"",!1),this.add(a),o}registerRule(e,t){const n=[],r=[];d(this,c(e,t,n,r),n,r,"",!1)}registerCss(e){return this.registerRule("",e)}getStyles(){return h(this.sheet)}clone(){return new b(this.id,this.changes).merge(this)}}function C(e){return new b("f"+(++r).toString(36),e)}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.convertToStyles=function e(t){var n={};for(var r in t){var i=t[r];if("$nest"===r){var o=i;for(var a in o){var s=o[a];n[a]=e(s)}}else"$debugName"===r?n.$displayName=i:n[r]=i}return n},t.convertToKeyframes=function(e){var t={};for(var n in e)"$debugName"!==n&&(t[n]=e[n]);return e.$debugName&&(t.$displayName=e.$debugName),t}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0})},function(e,t){
/*!
Copyright (C) 2013-2015 by Andrea Giammarchi - @WebReflection

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/
!function(e){"use strict";function t(){return d.createDocumentFragment()}function n(e){return d.createElement(e)}function r(e,t){if(!e)throw new Error("Failed to construct "+t+": 1 argument required, but only 0 present.")}function i(e){if(1===e.length)return o(e[0]);for(var n=t(),r=L.call(e),i=0;i<e.length;i++)n.appendChild(o(r[i]));return n}function o(e){return"object"==typeof e?e:d.createTextNode(e)}for(var a,s,l,u,c,d=e.document,h=Object.prototype.hasOwnProperty,f=Object.defineProperty||function(e,t,n){return h.call(n,"value")?e[t]=n.value:(h.call(n,"get")&&e.__defineGetter__(t,n.get),h.call(n,"set")&&e.__defineSetter__(t,n.set)),e},p=[].indexOf||function(e){for(var t=this.length;t--&&this[t]!==e;);return t},g=function(e){var t=void 0===e.className,n=t?e.getAttribute("class")||"":e.className,r=t||"object"==typeof n,i=(r?t?n:n.baseVal:n).replace(m,"");i.length&&k.push.apply(this,i.split(y)),this._isSVG=r,this._=e},v={get:function(){return new g(this)},set:function(){}},m=/^\s+|\s+$/g,y=/\s+/,b=function(e,t){return this.contains(e)?t||this.remove(e):(void 0===t||t)&&(t=!0,this.add(e)),!!t},C=e.DocumentFragment&&DocumentFragment.prototype,A=e.Node,_=(A||Element).prototype,I=e.CharacterData||A,x=I&&I.prototype,w=e.DocumentType,M=w&&w.prototype,S=(e.Element||A||e.HTMLElement).prototype,T=e.HTMLSelectElement||n("select").constructor,E=T.prototype.remove,O=e.SVGElement,k=["matches",S.matchesSelector||S.webkitMatchesSelector||S.khtmlMatchesSelector||S.mozMatchesSelector||S.msMatchesSelector||S.oMatchesSelector||function(e){var t=this.parentNode;return!!t&&-1<p.call(t.querySelectorAll(e),this)},"closest",function(e){for(var t,n=this;(t=n&&n.matches)&&!n.matches(e);)n=n.parentNode;return t?n:null},"prepend",function(){var e=this.firstChild,t=i(arguments);e?this.insertBefore(t,e):this.appendChild(t)},"append",function(){this.appendChild(i(arguments))},"before",function(){var e=this.parentNode;e&&e.insertBefore(i(arguments),this)},"after",function(){var e=this.parentNode,t=this.nextSibling,n=i(arguments);e&&(t?e.insertBefore(n,t):e.appendChild(n))},"toggleAttribute",function(e,t){var n=this.hasAttribute(e);return 1<arguments.length?n&&!t?this.removeAttribute(e):t&&!n&&this.setAttribute(e,""):n?this.removeAttribute(e):this.setAttribute(e,""),this.hasAttribute(e)},"replace",function(){this.replaceWith.apply(this,arguments)},"replaceWith",function(){var e=this.parentNode;e&&e.replaceChild(i(arguments),this)},"remove",function(){var e=this.parentNode;e&&e.removeChild(this)}],L=k.slice,D=k.length;D;D-=2)if((s=k[D-2])in S||(S[s]=k[D-1]),"remove"!==s||E._dom4||((T.prototype[s]=function(){return 0<arguments.length?E.apply(this,arguments):S.remove.call(this)})._dom4=!0),/^(?:before|after|replace|replaceWith|remove)$/.test(s)&&(I&&!(s in x)&&(x[s]=k[D-1]),w&&!(s in M)&&(M[s]=k[D-1])),/^(?:append|prepend)$/.test(s))if(C)s in C||(C[s]=k[D-1]);else try{t().constructor.prototype[s]=k[D-1]}catch(e){}var P;n("a").matches("a")||(S[s]=(P=S[s],function(e){return P.call(this.parentNode?this:t().appendChild(this),e)})),g.prototype={length:0,add:function(){for(var e,t=0;t<arguments.length;t++)e=arguments[t],this.contains(e)||k.push.call(this,s);this._isSVG?this._.setAttribute("class",""+this):this._.className=""+this},contains:function(e){return function(t){return-1<(D=e.call(this,s=function(e){if(!e)throw"SyntaxError";if(y.test(e))throw"InvalidCharacterError";return e}(t)))}}([].indexOf||function(e){for(D=this.length;D--&&this[D]!==e;);return D}),item:function(e){return this[e]||null},remove:function(){for(var e,t=0;t<arguments.length;t++)e=arguments[t],this.contains(e)&&k.splice.call(this,D,1);this._isSVG?this._.setAttribute("class",""+this):this._.className=""+this},toggle:b,toString:function(){return k.join.call(this," ")}},O&&!("classList"in O.prototype)&&f(O.prototype,"classList",v),"classList"in d.documentElement?((u=n("div").classList).add("a","b","a"),"a b"!=u&&("add"in(l=u.constructor.prototype)||(l=e.TemporaryTokenList.prototype),c=function(e){return function(){for(var t=0;t<arguments.length;)e.call(this,arguments[t++])}},l.add=c(l.add),l.remove=c(l.remove),l.toggle=b)):f(S,"classList",v),"contains"in _||f(_,"contains",{value:function(e){for(;e&&e!==this;)e=e.parentNode;return this===e}}),"head"in d||f(d,"head",{get:function(){return a||(a=d.getElementsByTagName("head")[0])}}),function(){for(var t,n=e.requestAnimationFrame,r=e.cancelAnimationFrame,i=["o","ms","moz","webkit"],o=i.length;!r&&o--;)n=n||e[i[o]+"RequestAnimationFrame"],r=e[i[o]+"CancelAnimationFrame"]||e[i[o]+"CancelRequestAnimationFrame"];r||(n?(t=n,n=function(e){var n=!0;return t((function(){n&&e.apply(this,arguments)})),function(){n=!1}},r=function(e){e()}):(n=function(e){return setTimeout(e,15,15)},r=function(e){clearTimeout(e)})),e.requestAnimationFrame=n,e.cancelAnimationFrame=r}();try{new e.CustomEvent("?")}catch(t){e.CustomEvent=function(e,t){function n(e,t,n,r){this.initEvent(e,t,n),this.detail=r}return function(r,i){var o=d.createEvent(e);if("string"!=typeof r)throw new Error("An event name must be provided");return"Event"==e&&(o.initCustomEvent=n),null==i&&(i=t),o.initCustomEvent(r,i.bubbles,i.cancelable,i.detail),o}}(e.CustomEvent?"CustomEvent":"Event",{bubbles:!1,cancelable:!1,detail:null})}try{new Event("_")}catch(t){t=function(e){function t(e,t){r(arguments.length,"Event");var n=d.createEvent("Event");return t||(t={}),n.initEvent(e,!!t.bubbles,!!t.cancelable),n}return t.prototype=e.prototype,t}(e.Event||function(){}),f(e,"Event",{value:t}),Event!==t&&(Event=t)}try{new KeyboardEvent("_",{})}catch(t){t=function(t){var n,i=0,o={char:"",key:"",location:0,ctrlKey:!1,shiftKey:!1,altKey:!1,metaKey:!1,altGraphKey:!1,repeat:!1,locale:navigator.language,detail:0,bubbles:!1,cancelable:!1,keyCode:0,charCode:0,which:0};try{var a=d.createEvent("KeyboardEvent");a.initKeyboardEvent("keyup",!1,!1,e,"+",3,!0,!1,!0,!1,!1),i="+"==(a.keyIdentifier||a.key)&&3==(a.keyLocation||a.location)&&(a.ctrlKey?a.altKey?1:3:a.shiftKey?2:4)||9}catch(e){}function s(e){for(var t=[],n=["ctrlKey","Control","shiftKey","Shift","altKey","Alt","metaKey","Meta","altGraphKey","AltGraph"],r=0;r<n.length;r+=2)e[n[r]]&&t.push(n[r+1]);return t.join(" ")}function l(e,t){for(var n in t)t.hasOwnProperty(n)&&!t.hasOwnProperty.call(e,n)&&(e[n]=t[n]);return e}function u(e,t,n){try{t[e]=n[e]}catch(e){}}function c(t,a){r(arguments.length,"KeyboardEvent"),a=l(a||{},o);var c,h=d.createEvent(n),f=a.ctrlKey,p=a.shiftKey,g=a.altKey,v=a.metaKey,m=a.altGraphKey,y=i>3?s(a):null,b=String(a.key),C=String(a.char),A=a.location,_=a.keyCode||(a.keyCode=b)&&b.charCodeAt(0)||0,I=a.charCode||(a.charCode=C)&&C.charCodeAt(0)||0,x=a.bubbles,w=a.cancelable,M=a.repeat,S=a.locale,T=a.view||e;if(a.which||(a.which=a.keyCode),"initKeyEvent"in h)h.initKeyEvent(t,x,w,T,f,g,p,v,_,I);else if(0<i&&"initKeyboardEvent"in h){switch(c=[t,x,w,T],i){case 1:c.push(b,A,f,p,g,v,m);break;case 2:c.push(f,g,p,v,_,I);break;case 3:c.push(b,A,f,g,p,v,m);break;case 4:c.push(b,A,y,M,S);break;default:c.push(char,b,A,y,M,S)}h.initKeyboardEvent.apply(h,c)}else h.initEvent(t,x,w);for(b in h)o.hasOwnProperty(b)&&h[b]!==a[b]&&u(b,h,a);return h}return n=0<i?"KeyboardEvent":"Event",c.prototype=t.prototype,c}(e.KeyboardEvent||function(){}),f(e,"KeyboardEvent",{value:t}),KeyboardEvent!==t&&(KeyboardEvent=t)}try{new MouseEvent("_",{})}catch(t){t=function(t){function n(t,n){r(arguments.length,"MouseEvent");var i=d.createEvent("MouseEvent");return n||(n={}),i.initMouseEvent(t,!!n.bubbles,!!n.cancelable,n.view||e,n.detail||1,n.screenX||0,n.screenY||0,n.clientX||0,n.clientY||0,!!n.ctrlKey,!!n.altKey,!!n.shiftKey,!!n.metaKey,n.button||0,n.relatedTarget||null),i}return n.prototype=t.prototype,n}(e.MouseEvent||function(){}),f(e,"MouseEvent",{value:t}),MouseEvent!==t&&(MouseEvent=t)}d.querySelectorAll("*").forEach||function(){function e(e){var t=e.querySelectorAll;e.querySelectorAll=function(e){var n=t.call(this,e);return n.forEach=Array.prototype.forEach,n}}e(d),e(Element.prototype)}();try{d.querySelector(":scope *")}catch(e){!function(){var e="data-scope-"+(1e9*Math.random()>>>0),t=Element.prototype,n=t.querySelector,r=t.querySelectorAll;function i(t,n,r){if(t.type!=d.ELEMENT_NODE)return n.call(t,r);t.setAttribute(e,null);var i=n.call(t,String(r).replace(/(^|,\s*)(:scope([ >]|$))/g,(function(t,n,r,i){return n+"["+e+"]"+(i||" ")})));return t.removeAttribute(e),i}t.querySelector=function(e){return i(this,n,e)},t.querySelectorAll=function(e){return i(this,r,e)}}()}}(window),function(e){"use strict";var t=e.WeakMap||function(){var e,t=0,n=!1,r=!1;function i(t,i,o){r=o,n=!1,e=void 0,t.dispatchEvent(i)}function o(e){this.value=e}function s(){t++,this.__ce__=new a("@DOMMap:"+t+Math.random())}return o.prototype.handleEvent=function(t){n=!0,r?t.currentTarget.removeEventListener(t.type,this,!1):e=this.value},s.prototype={constructor:s,delete:function(e){return i(e,this.__ce__,!0),n},get:function(t){i(t,this.__ce__,!1);var n=e;return e=void 0,n},has:function(e){return i(e,this.__ce__,!1),n},set:function(e,t){return i(e,this.__ce__,!0),e.addEventListener(this.__ce__.type,new o(t),!1),this}},s}();function n(){}function r(e,t,n){function i(e){i.once&&(e.currentTarget.removeEventListener(e.type,t,i),i.removed=!0),i.passive&&(e.preventDefault=r.preventDefault),"function"==typeof i.callback?i.callback.call(this,e):i.callback&&i.callback.handleEvent(e),i.passive&&delete e.preventDefault}return i.type=e,i.callback=t,i.capture=!!n.capture,i.passive=!!n.passive,i.once=!!n.once,i.removed=!1,i}n.prototype=(Object.create||Object)(null),r.preventDefault=function(){};var i,o,a=e.CustomEvent,s=e.dispatchEvent,l=e.addEventListener,u=e.removeEventListener,c=0,d=function(){c++},h=[].indexOf||function(e){for(var t=this.length;t--&&this[t]!==e;);return t},f=function(e){return"".concat(e.capture?"1":"0",e.passive?"1":"0",e.once?"1":"0")};try{l("_",d,{once:!0}),s(new a("_")),s(new a("_")),u("_",d,{once:!0})}catch(e){}1!==c&&(o=new t,i=function(e){if(e){var t=e.prototype;t.addEventListener=function(e){return function(t,i,a){if(a&&"boolean"!=typeof a){var s,l,u,c=o.get(this),d=f(a);c||o.set(this,c=new n),t in c||(c[t]={handler:[],wrap:[]}),l=c[t],(s=h.call(l.handler,i))<0?(s=l.handler.push(i)-1,l.wrap[s]=u=new n):u=l.wrap[s],d in u||(u[d]=r(t,i,a),e.call(this,t,u[d],u[d].capture))}else e.call(this,t,i,a)}}(t.addEventListener),t.removeEventListener=function(e){return function(t,n,r){if(r&&"boolean"!=typeof r){var i,a,s,l,u=o.get(this);if(u&&t in u&&(s=u[t],-1<(a=h.call(s.handler,n))&&(i=f(r))in(l=s.wrap[a]))){for(i in e.call(this,t,l[i],l[i].capture),delete l[i],l)return;s.handler.splice(a,1),s.wrap.splice(a,1),0===s.handler.length&&delete u[t]}}else e.call(this,t,n,r)}}(t.removeEventListener)}},e.EventTarget?i(EventTarget):(i(e.Text),i(e.Element||e.HTMLElement),i(e.HTMLDocument),i(e.Window||{prototype:e}),i(e.XMLHttpRequest)))}(self)},function(e,t,n){"use strict";t.__esModule=!0;var r=n(0),i=(a(r),a(n(3))),o=a(n(1479));a(n(625));function a(e){return e&&e.__esModule?e:{default:e}}function s(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function l(e,t){if(!e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return!t||"object"!=typeof t&&"function"!=typeof t?e:t}function u(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function, not "+typeof t);e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,enumerable:!1,writable:!0,configurable:!0}}),t&&(Object.setPrototypeOf?Object.setPrototypeOf(e,t):e.__proto__=t)}function c(e){var t=[];return{on:function(e){t.push(e)},off:function(e){t=t.filter((function(t){return t!==e}))},get:function(){return e},set:function(n,r){e=n,t.forEach((function(t){return t(e,r)}))}}}t.default=function(e,t){var n,a,d="__create-react-context-"+(0,o.default)()+"__",h=function(e){function n(){var t,r;s(this,n);for(var i=arguments.length,o=Array(i),a=0;a<i;a++)o[a]=arguments[a];return t=r=l(this,e.call.apply(e,[this].concat(o))),r.emitter=c(r.props.value),l(r,t)}return u(n,e),n.prototype.getChildContext=function(){var e;return(e={})[d]=this.emitter,e},n.prototype.componentWillReceiveProps=function(e){if(this.props.value!==e.value){var n=this.props.value,r=e.value,i=void 0;((o=n)===(a=r)?0!==o||1/o==1/a:o!=o&&a!=a)?i=0:(i="function"==typeof t?t(n,r):1073741823,0!==(i|=0)&&this.emitter.set(e.value,i))}var o,a},n.prototype.render=function(){return this.props.children},n}(r.Component);h.childContextTypes=((n={})[d]=i.default.object.isRequired,n);var f=function(t){function n(){var e,r;s(this,n);for(var i=arguments.length,o=Array(i),a=0;a<i;a++)o[a]=arguments[a];return e=r=l(this,t.call.apply(t,[this].concat(o))),r.state={value:r.getValue()},r.onUpdate=function(e,t){0!=((0|r.observedBits)&t)&&r.setState({value:r.getValue()})},l(r,e)}return u(n,t),n.prototype.componentWillReceiveProps=function(e){var t=e.observedBits;this.observedBits=null==t?1073741823:t},n.prototype.componentDidMount=function(){this.context[d]&&this.context[d].on(this.onUpdate);var e=this.props.observedBits;this.observedBits=null==e?1073741823:e},n.prototype.componentWillUnmount=function(){this.context[d]&&this.context[d].off(this.onUpdate)},n.prototype.getValue=function(){return this.context[d]?this.context[d].get():e},n.prototype.render=function(){return(e=this.props.children,Array.isArray(e)?e[0]:e)(this.state.value);var e},n}(r.Component);return f.contextTypes=((a={})[d]=i.default.object,a),{Provider:h,Consumer:f}},e.exports=t.default},function(e,t,n){"use strict";var r=n(1478);function i(){}function o(){}o.resetWarningCache=i,e.exports=function(){function e(e,t,n,i,o,a){if(a!==r){var s=new Error("Calling PropTypes validators directly is not supported by the `prop-types` package. Use PropTypes.checkPropTypes() to call them. Read more at http://fb.me/use-check-prop-types");throw s.name="Invariant Violation",s}}function t(){return e}e.isRequired=e;var n={array:e,bool:e,func:e,number:e,object:e,string:e,symbol:e,any:e,arrayOf:t,element:e,elementType:e,instanceOf:t,node:e,objectOf:t,oneOf:t,oneOfType:t,shape:t,exact:t,checkPropTypes:o,resetWarningCache:i};return n.PropTypes=n,n}},function(e,t,n){"use strict";e.exports="SECRET_DO_NOT_PASS_THIS_OR_YOU_WILL_BE_FIRED"},function(e,t,n){"use strict";(function(t){var n="__global_unique_id__";e.exports=function(){return t[n]=(t[n]||0)+1}}).call(this,n(61))},function(e,t,n){"use strict";t.__esModule=!0,t.default=void 0;!function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)if(Object.prototype.hasOwnProperty.call(e,n)){var r=Object.defineProperty&&Object.getOwnPropertyDescriptor?Object.getOwnPropertyDescriptor(e,n):{};r.get||r.set?Object.defineProperty(t,n,r):t[n]=e[n]}t.default=e}(n(3));var r=s(n(1481)),i=s(n(1483)),o=s(n(0)),a=s(n(878));n(879);function s(e){return e&&e.__esModule?e:{default:e}}function l(){return(l=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e}).apply(this,arguments)}var u=function(e,t){return e&&t&&t.split(" ").forEach((function(t){return(0,r.default)(e,t)}))},c=function(e,t){return e&&t&&t.split(" ").forEach((function(t){return(0,i.default)(e,t)}))},d=function(e){var t,n;function r(){for(var t,n=arguments.length,r=new Array(n),i=0;i<n;i++)r[i]=arguments[i];return(t=e.call.apply(e,[this].concat(r))||this).onEnter=function(e,n){var r=t.getClassNames(n?"appear":"enter").className;t.removeClasses(e,"exit"),u(e,r),t.props.onEnter&&t.props.onEnter(e,n)},t.onEntering=function(e,n){var r=t.getClassNames(n?"appear":"enter").activeClassName;t.reflowAndAddClass(e,r),t.props.onEntering&&t.props.onEntering(e,n)},t.onEntered=function(e,n){var r=t.getClassNames("appear").doneClassName,i=t.getClassNames("enter").doneClassName,o=n?r+" "+i:i;t.removeClasses(e,n?"appear":"enter"),u(e,o),t.props.onEntered&&t.props.onEntered(e,n)},t.onExit=function(e){var n=t.getClassNames("exit").className;t.removeClasses(e,"appear"),t.removeClasses(e,"enter"),u(e,n),t.props.onExit&&t.props.onExit(e)},t.onExiting=function(e){var n=t.getClassNames("exit").activeClassName;t.reflowAndAddClass(e,n),t.props.onExiting&&t.props.onExiting(e)},t.onExited=function(e){var n=t.getClassNames("exit").doneClassName;t.removeClasses(e,"exit"),u(e,n),t.props.onExited&&t.props.onExited(e)},t.getClassNames=function(e){var n=t.props.classNames,r="string"==typeof n,i=r?(r&&n?n+"-":"")+e:n[e];return{className:i,activeClassName:r?i+"-active":n[e+"Active"],doneClassName:r?i+"-done":n[e+"Done"]}},t}n=e,(t=r).prototype=Object.create(n.prototype),t.prototype.constructor=t,t.__proto__=n;var i=r.prototype;return i.removeClasses=function(e,t){var n=this.getClassNames(t),r=n.className,i=n.activeClassName,o=n.doneClassName;r&&c(e,r),i&&c(e,i),o&&c(e,o)},i.reflowAndAddClass=function(e,t){t&&(e&&e.scrollTop,u(e,t))},i.render=function(){var e=l({},this.props);return delete e.classNames,o.default.createElement(a.default,l({},e,{onEnter:this.onEnter,onEntered:this.onEntered,onEntering:this.onEntering,onExit:this.onExit,onExiting:this.onExiting,onExited:this.onExited}))},r}(o.default.Component);d.defaultProps={classNames:""},d.propTypes={};var h=d;t.default=h,e.exports=t.default},function(e,t,n){"use strict";var r=n(36);t.__esModule=!0,t.default=function(e,t){e.classList?e.classList.add(t):(0,i.default)(e,t)||("string"==typeof e.className?e.className=e.className+" "+t:e.setAttribute("class",(e.className&&e.className.baseVal||"")+" "+t))};var i=r(n(1482));e.exports=t.default},function(e,t,n){"use strict";t.__esModule=!0,t.default=function(e,t){return e.classList?!!t&&e.classList.contains(t):-1!==(" "+(e.className.baseVal||e.className)+" ").indexOf(" "+t+" ")},e.exports=t.default},function(e,t,n){"use strict";function r(e,t){return e.replace(new RegExp("(^|\\s)"+t+"(?:\\s|$)","g"),"$1").replace(/\s+/g," ").replace(/^\s*|\s*$/g,"")}e.exports=function(e,t){e.classList?e.classList.remove(t):"string"==typeof e.className?e.className=r(e.className,t):e.setAttribute("class",r(e.className&&e.className.baseVal||"",t))}},function(e,t,n){"use strict";t.__esModule=!0,t.default=void 0;a(n(3));var r=a(n(0)),i=n(22),o=a(n(880));function a(e){return e&&e.__esModule?e:{default:e}}var s=function(e){var t,n;function a(){for(var t,n=arguments.length,r=new Array(n),i=0;i<n;i++)r[i]=arguments[i];return(t=e.call.apply(e,[this].concat(r))||this).handleEnter=function(){for(var e=arguments.length,n=new Array(e),r=0;r<e;r++)n[r]=arguments[r];return t.handleLifecycle("onEnter",0,n)},t.handleEntering=function(){for(var e=arguments.length,n=new Array(e),r=0;r<e;r++)n[r]=arguments[r];return t.handleLifecycle("onEntering",0,n)},t.handleEntered=function(){for(var e=arguments.length,n=new Array(e),r=0;r<e;r++)n[r]=arguments[r];return t.handleLifecycle("onEntered",0,n)},t.handleExit=function(){for(var e=arguments.length,n=new Array(e),r=0;r<e;r++)n[r]=arguments[r];return t.handleLifecycle("onExit",1,n)},t.handleExiting=function(){for(var e=arguments.length,n=new Array(e),r=0;r<e;r++)n[r]=arguments[r];return t.handleLifecycle("onExiting",1,n)},t.handleExited=function(){for(var e=arguments.length,n=new Array(e),r=0;r<e;r++)n[r]=arguments[r];return t.handleLifecycle("onExited",1,n)},t}n=e,(t=a).prototype=Object.create(n.prototype),t.prototype.constructor=t,t.__proto__=n;var s=a.prototype;return s.handleLifecycle=function(e,t,n){var o,a=this.props.children,s=r.default.Children.toArray(a)[t];s.props[e]&&(o=s.props)[e].apply(o,n),this.props[e]&&this.props[e]((0,i.findDOMNode)(this))},s.render=function(){var e=this.props,t=e.children,n=e.in,i=function(e,t){if(null==e)return{};var n,r,i={},o=Object.keys(e);for(r=0;r<o.length;r++)n=o[r],t.indexOf(n)>=0||(i[n]=e[n]);return i}(e,["children","in"]),a=r.default.Children.toArray(t),s=a[0],l=a[1];return delete i.onEnter,delete i.onEntering,delete i.onEntered,delete i.onExit,delete i.onExiting,delete i.onExited,r.default.createElement(o.default,i,n?r.default.cloneElement(s,{key:"first",onEnter:this.handleEnter,onEntering:this.handleEntering,onEntered:this.handleEntered}):r.default.cloneElement(l,{key:"second",onEnter:this.handleExit,onEntering:this.handleExiting,onEntered:this.handleExited}))},a}(r.default.Component);s.propTypes={};var l=s;t.default=l,e.exports=t.default},function(e,t,n){"use strict";t.__esModule=!0,t.getChildMapping=i,t.mergeChildMappings=o,t.getInitialChildMapping=function(e,t){return i(e.children,(function(n){return(0,r.cloneElement)(n,{onExited:t.bind(null,n),in:!0,appear:a(n,"appear",e),enter:a(n,"enter",e),exit:a(n,"exit",e)})}))},t.getNextChildMapping=function(e,t,n){var s=i(e.children),l=o(t,s);return Object.keys(l).forEach((function(i){var o=l[i];if((0,r.isValidElement)(o)){var u=i in t,c=i in s,d=t[i],h=(0,r.isValidElement)(d)&&!d.props.in;!c||u&&!h?c||!u||h?c&&u&&(0,r.isValidElement)(d)&&(l[i]=(0,r.cloneElement)(o,{onExited:n.bind(null,o),in:d.props.in,exit:a(o,"exit",e),enter:a(o,"enter",e)})):l[i]=(0,r.cloneElement)(o,{in:!1}):l[i]=(0,r.cloneElement)(o,{onExited:n.bind(null,o),in:!0,exit:a(o,"exit",e),enter:a(o,"enter",e)})}})),l};var r=n(0);function i(e,t){var n=Object.create(null);return e&&r.Children.map(e,(function(e){return e})).forEach((function(e){n[e.key]=function(e){return t&&(0,r.isValidElement)(e)?t(e):e}(e)})),n}function o(e,t){function n(n){return n in t?t[n]:e[n]}e=e||{},t=t||{};var r,i=Object.create(null),o=[];for(var a in e)a in t?o.length&&(i[a]=o,o=[]):o.push(a);var s={};for(var l in t){if(i[l])for(r=0;r<i[l].length;r++){var u=i[l][r];s[i[l][r]]=n(u)}s[l]=n(l)}for(r=0;r<o.length;r++)s[o[r]]=n(o[r]);return s}function a(e,t,n){return null!=n[t]?n[t]:e.props[t]}},function(e,t,n){"use strict";var r;if(!Object.keys){var i=Object.prototype.hasOwnProperty,o=Object.prototype.toString,a=n(882),s=Object.prototype.propertyIsEnumerable,l=!s.call({toString:null},"toString"),u=s.call((function(){}),"prototype"),c=["toString","toLocaleString","valueOf","hasOwnProperty","isPrototypeOf","propertyIsEnumerable","constructor"],d=function(e){var t=e.constructor;return t&&t.prototype===e},h={$applicationCache:!0,$console:!0,$external:!0,$frame:!0,$frameElement:!0,$frames:!0,$innerHeight:!0,$innerWidth:!0,$onmozfullscreenchange:!0,$onmozfullscreenerror:!0,$outerHeight:!0,$outerWidth:!0,$pageXOffset:!0,$pageYOffset:!0,$parent:!0,$scrollLeft:!0,$scrollTop:!0,$scrollX:!0,$scrollY:!0,$self:!0,$webkitIndexedDB:!0,$webkitStorageInfo:!0,$window:!0},f=function(){if("undefined"==typeof window)return!1;for(var e in window)try{if(!h["$"+e]&&i.call(window,e)&&null!==window[e]&&"object"==typeof window[e])try{d(window[e])}catch(e){return!0}}catch(e){return!0}return!1}();r=function(e){var t=null!==e&&"object"==typeof e,n="[object Function]"===o.call(e),r=a(e),s=t&&"[object String]"===o.call(e),h=[];if(!t&&!n&&!r)throw new TypeError("Object.keys called on a non-object");var p=u&&n;if(s&&e.length>0&&!i.call(e,0))for(var g=0;g<e.length;++g)h.push(String(g));if(r&&e.length>0)for(var v=0;v<e.length;++v)h.push(String(v));else for(var m in e)p&&"prototype"===m||!i.call(e,m)||h.push(String(m));if(l)for(var y=function(e){if("undefined"==typeof window||!f)return d(e);try{return d(e)}catch(e){return!1}}(e),b=0;b<c.length;++b)y&&"constructor"===c[b]||!i.call(e,c[b])||h.push(c[b]);return h}}e.exports=r},function(e,t,n){"use strict";var r="function"==typeof Symbol&&"symbol"==typeof Symbol.toStringTag,i=n(883)("Object.prototype.toString"),o=function(e){return!(r&&e&&"object"==typeof e&&Symbol.toStringTag in e)&&"[object Arguments]"===i(e)},a=function(e){return!!o(e)||null!==e&&"object"==typeof e&&"number"==typeof e.length&&e.length>=0&&"[object Array]"!==i(e)&&"[object Function]"===i(e.callee)},s=function(){return o(arguments)}();o.isLegacyArguments=a,e.exports=s?o:a},function(e,t,n){"use strict";e.exports=function(){if("function"!=typeof Symbol||"function"!=typeof Object.getOwnPropertySymbols)return!1;if("symbol"==typeof Symbol.iterator)return!0;var e={},t=Symbol("test"),n=Object(t);if("string"==typeof t)return!1;if("[object Symbol]"!==Object.prototype.toString.call(t))return!1;if("[object Symbol]"!==Object.prototype.toString.call(n))return!1;for(t in e[t]=42,e)return!1;if("function"==typeof Object.keys&&0!==Object.keys(e).length)return!1;if("function"==typeof Object.getOwnPropertyNames&&0!==Object.getOwnPropertyNames(e).length)return!1;var r=Object.getOwnPropertySymbols(e);if(1!==r.length||r[0]!==t)return!1;if(!Object.prototype.propertyIsEnumerable.call(e,t))return!1;if("function"==typeof Object.getOwnPropertyDescriptor){var i=Object.getOwnPropertyDescriptor(e,t);if(42!==i.value||!0!==i.enumerable)return!1}return!0}},function(e,t,n){"use strict";var r="Function.prototype.bind called on incompatible ",i=Array.prototype.slice,o=Object.prototype.toString;e.exports=function(e){var t=this;if("function"!=typeof t||"[object Function]"!==o.call(t))throw new TypeError(r+t);for(var n,a=i.call(arguments,1),s=function(){if(this instanceof n){var r=t.apply(this,a.concat(i.call(arguments)));return Object(r)===r?r:this}return t.apply(e,a.concat(i.call(arguments)))},l=Math.max(0,t.length-a.length),u=[],c=0;c<l;c++)u.push("$"+c);if(n=Function("binder","return function ("+u.join(",")+"){ return binder.apply(this,arguments); }")(s),t.prototype){var d=function(){};d.prototype=t.prototype,n.prototype=new d,d.prototype=null}return n}},function(e,t,n){"use strict";var r=n(507);e.exports=r.call(Function.call,Object.prototype.hasOwnProperty)},function(e,t,n){"use strict";var r=n(285),i=n(508),o=n(886),a=n(887),s=n(1492),l=i(a(),Object);r(l,{getPolyfill:a,implementation:o,shim:s}),e.exports=l},function(e,t,n){"use strict";var r=n(887),i=n(285);e.exports=function(){var e=r();return i(Object,{is:e},{is:function(){return Object.is!==e}}),e}},function(e,t,n){"use strict";var r,i,o,a,s=n(883),l=n(885)()&&"symbol"==typeof Symbol.toStringTag;if(l){r=s("Object.prototype.hasOwnProperty"),i=s("RegExp.prototype.exec"),o={};var u=function(){throw o};a={toString:u,valueOf:u},"symbol"==typeof Symbol.toPrimitive&&(a[Symbol.toPrimitive]=u)}var c=s("Object.prototype.toString"),d=Object.getOwnPropertyDescriptor;e.exports=l?function(e){if(!e||"object"!=typeof e)return!1;var t=d(e,"lastIndex");if(!(t&&r(t,"value")))return!1;try{i(e,a)}catch(e){return e===o}}:function(e){return!(!e||"object"!=typeof e&&"function"!=typeof e)&&"[object RegExp]"===c(e)}},function(e,t,n){"use strict";var r=n(285),i=n(508),o=n(888),a=n(889),s=n(1495),l=i(o);r(l,{getPolyfill:a,implementation:o,shim:s}),e.exports=l},function(e,t,n){"use strict";var r=n(285).supportsDescriptors,i=n(889),o=Object.getOwnPropertyDescriptor,a=Object.defineProperty,s=TypeError,l=Object.getPrototypeOf,u=/a/;e.exports=function(){if(!r||!l)throw new s("RegExp.prototype.flags requires a true ES5 environment that supports property descriptors");var e=i(),t=l(u),n=o(t,"flags");return n&&n.get===e||a(t,"flags",{configurable:!0,enumerable:!1,get:e}),e}},function(e,t,n){"use strict";var r=Date.prototype.getDay,i=Object.prototype.toString,o="function"==typeof Symbol&&"symbol"==typeof Symbol.toStringTag;e.exports=function(e){return"object"==typeof e&&null!==e&&(o?function(e){try{return r.call(e),!0}catch(e){return!1}}(e):"[object Date]"===i.call(e))}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.SELECT_POPOVER=t.SELECT=t.OMNIBAR_OVERLAY=t.OMNIBAR=t.MULTISELECT_TAG_INPUT_INPUT=t.MULTISELECT_POPOVER=t.MULTISELECT=void 0;var r=n(431).Classes.getClassNamespace();t.MULTISELECT=r+"-multi-select",t.MULTISELECT_POPOVER=t.MULTISELECT+"-popover",t.MULTISELECT_TAG_INPUT_INPUT=t.MULTISELECT+"-tag-input-input",t.OMNIBAR=r+"-omnibar",t.OMNIBAR_OVERLAY=t.OMNIBAR+"-overlay",t.SELECT=r+"-select",t.SELECT_POPOVER=t.SELECT+"-popover"},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.renderFilteredItems=void 0,t.renderFilteredItems=function(e,t,n){if(0===e.query.length&&void 0!==n)return n;var r=e.filteredItems.map(e.renderItem).filter((function(e){return null!=e}));return r.length>0?r:t}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0})},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.executeItemsEqual=void 0;var r=n(431);t.executeItemsEqual=function(e,t,n){return void 0===e||null==t||null==n?t===n:r.Utils.isFunction(e)?e(t,n):t[e]===n[e]}},function(e,t,n){"use strict";function r(e){if(null==e)return!1;var t=Object.keys(e);return 1===t.length&&"__blueprintCreateNewItemBrand"===t[0]&&"blueprint-create-new-item"===e.__blueprintCreateNewItemBrand}Object.defineProperty(t,"__esModule",{value:!0}),t.getActiveItem=t.isCreateNewItem=t.getCreateNewItem=void 0,t.getCreateNewItem=function(){return{__blueprintCreateNewItemBrand:"blueprint-create-new-item"}},t.isCreateNewItem=r,t.getActiveItem=function(e){return null==e||r(e)?null:e}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0})},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.getFirstEnabledItem=t.QueryList=void 0;var r=n(506),i=r.__importStar(n(0)),o=n(431),a=n(890),s=function(e){function t(t,n){var r,s,l=e.call(this,t,n)||this;l.refHandlers={itemsParent:function(e){return l.itemsParentRef=e}},l.shouldCheckActiveItemInViewport=!1,l.expectedNextActiveItem=null,l.isEnterKeyPressed=!1,l.renderItemList=function(e){var t=l.props,n=t.initialContent,r=t.noResults,s=e.renderCreateItem(),u=null!=s?null:r,c=a.renderFilteredItems(e,u,n);if(null==c&&null==s)return null;var d=l.isCreateItemFirst();return i.createElement(o.Menu,{ulRef:e.itemsParentRef},d&&s,c,!d&&s)},l.renderItem=function(e,t){if(!0!==l.props.disabled){var n=l.state,r=n.activeItem,i=n.query,o=l.state.filteredItems.indexOf(e)>=0,s={active:a.executeItemsEqual(l.props.itemsEqual,a.getActiveItem(r),e),disabled:d(e,t,l.props.itemDisabled),matchesPredicate:o};return l.props.itemRenderer(e,{handleClick:function(t){return l.handleItemSelect(e,t)},index:t,modifiers:s,query:i})}return null},l.renderCreateItemMenuItem=function(){if(l.isCreateItemRendered()){var e=l.state,t=e.activeItem,n=e.query.trim(),r=a.isCreateNewItem(t);return l.props.createNewItemRenderer(n,r,(function(e){l.handleItemCreate(n,e)}))}return null},l.handleItemCreate=function(e,t){var n,r,i,o,a=null===(r=(n=l.props).createNewItemFromQuery)||void 0===r?void 0:r.call(n,e);null!=a&&(null===(o=(i=l.props).onItemSelect)||void 0===o||o.call(i,a,t),l.maybeResetQuery())},l.handleItemSelect=function(e,t){var n,r;l.setActiveItem(e),null===(r=(n=l.props).onItemSelect)||void 0===r||r.call(n,e,t),l.maybeResetQuery()},l.handlePaste=function(e){for(var t,n=l.props,r=n.createNewItemFromQuery,i=n.onItemsPaste,o=[],a=[],s=0,c=e;s<c.length;s++){var d=c[s],h=u(d,l.props);if(void 0!==h)t=h,a.push(h);else if(l.canCreateItems()){var f=null==r?void 0:r(d);void 0!==f&&a.push(f)}else o.push(d)}l.setQuery(o.join(", "),!1),void 0!==t&&l.setActiveItem(t),null==i||i(a)},l.handleKeyDown=function(e){var t,n,r=e.keyCode;if(r===o.Keys.ARROW_UP||r===o.Keys.ARROW_DOWN){e.preventDefault();var i=l.getNextActiveItem(r===o.Keys.ARROW_UP?-1:1);null!=i&&l.setActiveItem(i)}else r===o.Keys.ENTER&&(l.isEnterKeyPressed=!0);null===(n=(t=l.props).onKeyDown)||void 0===n||n.call(t,e)},l.handleKeyUp=function(e){var t=l.props.onKeyUp,n=l.state.activeItem;e.keyCode===o.Keys.ENTER&&l.isEnterKeyPressed&&(e.preventDefault(),null==n||a.isCreateNewItem(n)?l.handleItemCreate(l.state.query,e):l.handleItemSelect(n,e),l.isEnterKeyPressed=!1),null==t||t(e)},l.handleInputQueryChange=function(e){var t,n,r=null==e?"":e.target.value;l.setQuery(r),null===(n=(t=l.props).onQueryChange)||void 0===n||n.call(t,r,e)};var f=t.query,p=void 0===f?"":f,g=null===(r=t.createNewItemFromQuery)||void 0===r?void 0:r.call(t,p),v=c(p,t);return l.state={activeItem:void 0!==t.activeItem?t.activeItem:null!==(s=t.initialActiveItem)&&void 0!==s?s:h(v,t.itemDisabled),createNewItem:g,filteredItems:v,query:p},l}return r.__extends(t,e),t.ofType=function(){return t},t.prototype.render=function(){var e=this.props,t=e.className,n=e.items,i=e.renderer,o=e.itemListRenderer,a=void 0===o?this.renderItemList:o,s=this.state,l=(s.createNewItem,r.__rest(s,["createNewItem"]));return i(r.__assign(r.__assign({},l),{className:t,handleItemSelect:this.handleItemSelect,handleKeyDown:this.handleKeyDown,handleKeyUp:this.handleKeyUp,handlePaste:this.handlePaste,handleQueryChange:this.handleInputQueryChange,itemList:a(r.__assign(r.__assign({},l),{items:n,itemsParentRef:this.refHandlers.itemsParent,renderCreateItem:this.renderCreateItemMenuItem,renderItem:this.renderItem}))}))},t.prototype.componentDidUpdate=function(e){var t=this;void 0!==this.props.activeItem&&this.props.activeItem!==this.state.activeItem&&(this.shouldCheckActiveItemInViewport=!0,this.setState({activeItem:this.props.activeItem})),null!=this.props.query&&this.props.query!==e.query?this.setQuery(this.props.query,this.props.resetOnQuery,this.props):o.Utils.shallowCompareKeys(this.props,e,{include:["items","itemListPredicate","itemPredicate"]})||this.setQuery(this.state.query),this.shouldCheckActiveItemInViewport&&(this.requestAnimationFrame((function(){return t.scrollActiveItemIntoView()})),this.shouldCheckActiveItemInViewport=!1)},t.prototype.scrollActiveItemIntoView=function(){var e=!1!==this.props.scrollToActiveItem,t=!a.executeItemsEqual(this.props.itemsEqual,a.getActiveItem(this.expectedNextActiveItem),a.getActiveItem(this.props.activeItem));if(this.expectedNextActiveItem=null,e||!t){var n=this.getActiveElement();if(null!=this.itemsParentRef&&null!=n){var r=n.offsetTop,i=n.offsetHeight,o=this.itemsParentRef,s=o.offsetTop,l=o.scrollTop,u=o.clientHeight,c=this.getItemsParentPadding(),d=c.paddingTop,h=r+i+c.paddingBottom-s,f=r-d-s;h>=l+u?this.itemsParentRef.scrollTop=h+i-u:f<=l&&(this.itemsParentRef.scrollTop=f-i)}}},t.prototype.setQuery=function(e,t,n){var r;void 0===t&&(t=this.props.resetOnQuery),void 0===n&&(n=this.props);var i=n.createNewItemFromQuery;this.shouldCheckActiveItemInViewport=!0,e!==this.state.query&&(null===(r=n.onQueryChange)||void 0===r||r.call(n,e));var o=e.trim(),s=c(o,n),l=null!=i&&""!==o?i(o):void 0;this.setState({createNewItem:l,filteredItems:s,query:e});var u=this.getActiveIndex(s);(t||u<0||d(a.getActiveItem(this.state.activeItem),u,n.itemDisabled))&&(this.isCreateItemRendered()&&this.isCreateItemFirst()?this.setActiveItem(a.getCreateNewItem()):this.setActiveItem(h(s,n.itemDisabled)))},t.prototype.setActiveItem=function(e){var t,n,r,i;this.expectedNextActiveItem=e,void 0===this.props.activeItem&&(this.shouldCheckActiveItemInViewport=!0,this.setState({activeItem:e})),a.isCreateNewItem(e)?null===(n=(t=this.props).onActiveItemChange)||void 0===n||n.call(t,null,!0):null===(i=(r=this.props).onActiveItemChange)||void 0===i||i.call(r,e,!1)},t.prototype.getActiveElement=function(){var e=this.state.activeItem;if(null!=this.itemsParentRef){if(a.isCreateNewItem(e)){var t=this.isCreateItemFirst()?0:this.state.filteredItems.length;return this.itemsParentRef.children.item(t)}var n=this.getActiveIndex();return this.itemsParentRef.children.item(n)}},t.prototype.getActiveIndex=function(e){void 0===e&&(e=this.state.filteredItems);var t=this.state.activeItem;if(null==t||a.isCreateNewItem(t))return-1;for(var n=0;n<e.length;++n)if(a.executeItemsEqual(this.props.itemsEqual,e[n],t))return n;return-1},t.prototype.getItemsParentPadding=function(){var e=getComputedStyle(this.itemsParentRef),t=e.paddingTop;return{paddingBottom:l(e.paddingBottom),paddingTop:l(t)}},t.prototype.getNextActiveItem=function(e,t){if((void 0===t&&(t=this.getActiveIndex()),this.isCreateItemRendered())&&(0===t&&-1===e||t===this.state.filteredItems.length-1&&1===e))return a.getCreateNewItem();return h(this.state.filteredItems,this.props.itemDisabled,e,t)},t.prototype.isCreateItemRendered=function(){return this.canCreateItems()&&""!==this.state.query&&!this.wouldCreatedItemMatchSomeExistingItem()},t.prototype.isCreateItemFirst=function(){return"first"===this.props.createNewItemPosition},t.prototype.canCreateItems=function(){return null!=this.props.createNewItemFromQuery&&null!=this.props.createNewItemRenderer},t.prototype.wouldCreatedItemMatchSomeExistingItem=function(){var e=this;return this.state.filteredItems.some((function(t){return a.executeItemsEqual(e.props.itemsEqual,t,e.state.createNewItem)}))},t.prototype.maybeResetQuery=function(){this.props.resetOnSelect&&this.setQuery("",!0)},t.displayName=o.DISPLAYNAME_PREFIX+".QueryList",t.defaultProps={disabled:!1,resetOnQuery:!0},t}(o.AbstractComponent2);function l(e){return null==e?0:parseInt(e.slice(0,-2),10)}function u(e,t){var n=t.items,r=t.itemPredicate;if(o.Utils.isFunction(r))for(var i=0;i<n.length;i++){var a=n[i];if(r(e,a,i,!0))return a}}function c(e,t){var n=t.items,r=t.itemPredicate,i=t.itemListPredicate;return o.Utils.isFunction(i)?i(e,n):o.Utils.isFunction(r)?n.filter((function(t,n){return r(e,t,n)})):n}function d(e,t,n){return null!=n&&null!=e&&(o.Utils.isFunction(n)?n(e,t):!!e[n])}function h(e,t,n,r){if(void 0===n&&(n=1),void 0===r&&(r=e.length-1),0===e.length)return null;var i,o,a,s=r,l=e.length-1;do{if(a=l,!d(e[s=(i=s+n)<(o=0)?a:i>a?o:i],s,t))return e[s]}while(s!==r&&-1!==r);return null}t.QueryList=s,t.getFirstEnabledItem=h},function(e,t){e.exports=__WEBPACK_EXTERNAL_MODULE__1504__},function(e,t,n){var r=n(1506);"string"==typeof r&&(r=[[e.i,r,""]]);var i={insert:"head",singleton:!1};n(147)(r,i);r.locals&&(e.exports=r.locals)},function(e,t,n){(t=e.exports=n(68)(!1)).i(n(1507),""),t.i(n(1508),""),t.i(n(1509),""),t.i(n(1510),""),t.i(n(1511),""),t.i(n(1512),""),t.i(n(1513),""),t.i(n(1514),""),t.i(n(1515),""),t.push([e.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Copyright (c) 2014-2017, PhosphorJS Contributors\n|\n| Distributed under the terms of the BSD 3-Clause License.\n|\n| The full license is in the file LICENSE, distributed with this software.\n|----------------------------------------------------------------------------*/\n",""])},function(e,t,n){(e.exports=n(68)(!1)).push([e.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Copyright (c) 2014-2017, PhosphorJS Contributors\n|\n| Distributed under the terms of the BSD 3-Clause License.\n|\n| The full license is in the file LICENSE, distributed with this software.\n|----------------------------------------------------------------------------*/\n\n\n/* <DEPRECATED> */ .p-Widget, /* </DEPRECATED> */\n.lm-Widget {\n  box-sizing: border-box;\n  position: relative;\n  overflow: hidden;\n  cursor: default;\n}\n\n\n/* <DEPRECATED> */ .p-Widget.p-mod-hidden, /* </DEPRECATED> */\n.lm-Widget.lm-mod-hidden {\n  display: none !important;\n}\n",""])},function(e,t,n){(e.exports=n(68)(!1)).push([e.i,'/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Copyright (c) 2014-2017, PhosphorJS Contributors\n|\n| Distributed under the terms of the BSD 3-Clause License.\n|\n| The full license is in the file LICENSE, distributed with this software.\n|----------------------------------------------------------------------------*/\n\n\n/* <DEPRECATED> */ .p-CommandPalette, /* </DEPRECATED> */\n.lm-CommandPalette {\n  display: flex;\n  flex-direction: column;\n  -webkit-user-select: none;\n  -moz-user-select: none;\n  -ms-user-select: none;\n  user-select: none;\n}\n\n\n/* <DEPRECATED> */ .p-CommandPalette-search, /* </DEPRECATED> */\n.lm-CommandPalette-search {\n  flex: 0 0 auto;\n}\n\n\n/* <DEPRECATED> */ .p-CommandPalette-content, /* </DEPRECATED> */\n.lm-CommandPalette-content {\n  flex: 1 1 auto;\n  margin: 0;\n  padding: 0;\n  min-height: 0;\n  overflow: auto;\n  list-style-type: none;\n}\n\n\n/* <DEPRECATED> */ .p-CommandPalette-header, /* </DEPRECATED> */\n.lm-CommandPalette-header {\n  overflow: hidden;\n  white-space: nowrap;\n  text-overflow: ellipsis;\n}\n\n\n/* <DEPRECATED> */ .p-CommandPalette-item, /* </DEPRECATED> */\n.lm-CommandPalette-item {\n  display: flex;\n  flex-direction: row;\n}\n\n\n/* <DEPRECATED> */ .p-CommandPalette-itemIcon, /* </DEPRECATED> */\n.lm-CommandPalette-itemIcon {\n  flex: 0 0 auto;\n}\n\n\n/* <DEPRECATED> */ .p-CommandPalette-itemContent, /* </DEPRECATED> */\n.lm-CommandPalette-itemContent {\n  flex: 1 1 auto;\n  overflow: hidden;\n}\n\n\n/* <DEPRECATED> */ .p-CommandPalette-itemShortcut, /* </DEPRECATED> */\n.lm-CommandPalette-itemShortcut {\n  flex: 0 0 auto;\n}\n\n\n/* <DEPRECATED> */ .p-CommandPalette-itemLabel, /* </DEPRECATED> */\n.lm-CommandPalette-itemLabel {\n  overflow: hidden;\n  white-space: nowrap;\n  text-overflow: ellipsis;\n}\n\n.lm-close-icon {\n\tborder:1px solid transparent;\n  background-color: transparent;\n  position: absolute;\n\tz-index:1;\n\tright:3%;\n\ttop: 0;\n\tbottom: 0;\n\tmargin: auto;\n\tpadding: 7px 0;\n\tdisplay: none;\n\tvertical-align: middle;\n  outline: 0;\n  cursor: pointer;\n}\n.lm-close-icon:after {\n\tcontent: "X";\n\tdisplay: block;\n\twidth: 15px;\n\theight: 15px;\n\ttext-align: center;\n\tcolor:#000;\n\tfont-weight: normal;\n\tfont-size: 12px;\n\tcursor: pointer;\n}\n',""])},function(e,t,n){(e.exports=n(68)(!1)).push([e.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Copyright (c) 2014-2017, PhosphorJS Contributors\n|\n| Distributed under the terms of the BSD 3-Clause License.\n|\n| The full license is in the file LICENSE, distributed with this software.\n|----------------------------------------------------------------------------*/\n\n\n/* <DEPRECATED> */ .p-DockPanel, /* </DEPRECATED> */\n.lm-DockPanel {\n  z-index: 0;\n}\n\n\n/* <DEPRECATED> */ .p-DockPanel-widget, /* </DEPRECATED> */\n.lm-DockPanel-widget {\n  z-index: 0;\n}\n\n\n/* <DEPRECATED> */ .p-DockPanel-tabBar, /* </DEPRECATED> */\n.lm-DockPanel-tabBar {\n  z-index: 1;\n}\n\n\n/* <DEPRECATED> */ .p-DockPanel-handle, /* </DEPRECATED> */\n.lm-DockPanel-handle {\n  z-index: 2;\n}\n\n\n/* <DEPRECATED> */ .p-DockPanel-handle.p-mod-hidden, /* </DEPRECATED> */\n.lm-DockPanel-handle.lm-mod-hidden {\n  display: none !important;\n}\n\n\n/* <DEPRECATED> */ .p-DockPanel-handle:after, /* </DEPRECATED> */\n.lm-DockPanel-handle:after {\n  position: absolute;\n  top: 0;\n  left: 0;\n  width: 100%;\n  height: 100%;\n  content: '';\n}\n\n\n/* <DEPRECATED> */\n.p-DockPanel-handle[data-orientation='horizontal'],\n/* </DEPRECATED> */\n.lm-DockPanel-handle[data-orientation='horizontal'] {\n  cursor: ew-resize;\n}\n\n\n/* <DEPRECATED> */\n.p-DockPanel-handle[data-orientation='vertical'],\n/* </DEPRECATED> */\n.lm-DockPanel-handle[data-orientation='vertical'] {\n  cursor: ns-resize;\n}\n\n\n/* <DEPRECATED> */\n.p-DockPanel-handle[data-orientation='horizontal']:after,\n/* </DEPRECATED> */\n.lm-DockPanel-handle[data-orientation='horizontal']:after {\n  left: 50%;\n  min-width: 8px;\n  transform: translateX(-50%);\n}\n\n\n/* <DEPRECATED> */\n.p-DockPanel-handle[data-orientation='vertical']:after,\n/* </DEPRECATED> */\n.lm-DockPanel-handle[data-orientation='vertical']:after {\n  top: 50%;\n  min-height: 8px;\n  transform: translateY(-50%);\n}\n\n\n/* <DEPRECATED> */ .p-DockPanel-overlay, /* </DEPRECATED> */\n.lm-DockPanel-overlay {\n  z-index: 3;\n  box-sizing: border-box;\n  pointer-events: none;\n}\n\n\n/* <DEPRECATED> */ .p-DockPanel-overlay.p-mod-hidden, /* </DEPRECATED> */\n.lm-DockPanel-overlay.lm-mod-hidden {\n  display: none !important;\n}\n",""])},function(e,t,n){(e.exports=n(68)(!1)).push([e.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Copyright (c) 2014-2017, PhosphorJS Contributors\n|\n| Distributed under the terms of the BSD 3-Clause License.\n|\n| The full license is in the file LICENSE, distributed with this software.\n|----------------------------------------------------------------------------*/\n\n\n/* <DEPRECATED> */ .p-Menu, /* </DEPRECATED> */\n.lm-Menu {\n  z-index: 10000;\n  position: absolute;\n  white-space: nowrap;\n  overflow-x: hidden;\n  overflow-y: auto;\n  outline: none;\n  -webkit-user-select: none;\n  -moz-user-select: none;\n  -ms-user-select: none;\n  user-select: none;\n}\n\n\n/* <DEPRECATED> */ .p-Menu-content, /* </DEPRECATED> */\n.lm-Menu-content {\n  margin: 0;\n  padding: 0;\n  display: table;\n  list-style-type: none;\n}\n\n\n/* <DEPRECATED> */ .p-Menu-item, /* </DEPRECATED> */\n.lm-Menu-item {\n  display: table-row;\n}\n\n\n/* <DEPRECATED> */\n.p-Menu-item.p-mod-hidden,\n.p-Menu-item.p-mod-collapsed,\n/* </DEPRECATED> */\n.lm-Menu-item.lm-mod-hidden,\n.lm-Menu-item.lm-mod-collapsed {\n  display: none !important;\n}\n\n\n/* <DEPRECATED> */\n.p-Menu-itemIcon,\n.p-Menu-itemSubmenuIcon,\n/* </DEPRECATED> */\n.lm-Menu-itemIcon,\n.lm-Menu-itemSubmenuIcon {\n  display: table-cell;\n  text-align: center;\n}\n\n\n/* <DEPRECATED> */ .p-Menu-itemLabel, /* </DEPRECATED> */\n.lm-Menu-itemLabel {\n  display: table-cell;\n  text-align: left;\n}\n\n\n/* <DEPRECATED> */ .p-Menu-itemShortcut, /* </DEPRECATED> */\n.lm-Menu-itemShortcut {\n  display: table-cell;\n  text-align: right;\n}\n",""])},function(e,t,n){(e.exports=n(68)(!1)).push([e.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Copyright (c) 2014-2017, PhosphorJS Contributors\n|\n| Distributed under the terms of the BSD 3-Clause License.\n|\n| The full license is in the file LICENSE, distributed with this software.\n|----------------------------------------------------------------------------*/\n\n\n/* <DEPRECATED> */ .p-MenuBar, /* </DEPRECATED> */\n.lm-MenuBar {\n  outline: none;\n  -webkit-user-select: none;\n  -moz-user-select: none;\n  -ms-user-select: none;\n  user-select: none;\n}\n\n\n/* <DEPRECATED> */ .p-MenuBar-content, /* </DEPRECATED> */\n.lm-MenuBar-content {\n  margin: 0;\n  padding: 0;\n  display: flex;\n  flex-direction: row;\n  list-style-type: none;\n}\n\n\n/* <DEPRECATED> */ .p--MenuBar-item, /* </DEPRECATED> */\n.lm-MenuBar-item {\n  box-sizing: border-box;\n}\n\n\n/* <DEPRECATED> */\n.p-MenuBar-itemIcon,\n.p-MenuBar-itemLabel,\n/* </DEPRECATED> */\n.lm-MenuBar-itemIcon,\n.lm-MenuBar-itemLabel {\n  display: inline-block;\n}\n",""])},function(e,t,n){(e.exports=n(68)(!1)).push([e.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Copyright (c) 2014-2017, PhosphorJS Contributors\n|\n| Distributed under the terms of the BSD 3-Clause License.\n|\n| The full license is in the file LICENSE, distributed with this software.\n|----------------------------------------------------------------------------*/\n\n\n/* <DEPRECATED> */ .p-ScrollBar, /* </DEPRECATED> */\n.lm-ScrollBar {\n  display: flex;\n  -webkit-user-select: none;\n  -moz-user-select: none;\n  -ms-user-select: none;\n  user-select: none;\n}\n\n\n/* <DEPRECATED> */\n.p-ScrollBar[data-orientation='horizontal'],\n/* </DEPRECATED> */\n.lm-ScrollBar[data-orientation='horizontal'] {\n  flex-direction: row;\n}\n\n\n/* <DEPRECATED> */\n.p-ScrollBar[data-orientation='vertical'],\n/* </DEPRECATED> */\n.lm-ScrollBar[data-orientation='vertical'] {\n  flex-direction: column;\n}\n\n\n/* <DEPRECATED> */ .p-ScrollBar-button, /* </DEPRECATED> */\n.lm-ScrollBar-button {\n  box-sizing: border-box;\n  flex: 0 0 auto;\n}\n\n\n/* <DEPRECATED> */ .p-ScrollBar-track, /* </DEPRECATED> */\n.lm-ScrollBar-track {\n  box-sizing: border-box;\n  position: relative;\n  overflow: hidden;\n  flex: 1 1 auto;\n}\n\n\n/* <DEPRECATED> */ .p-ScrollBar-thumb, /* </DEPRECATED> */\n.lm-ScrollBar-thumb {\n  box-sizing: border-box;\n  position: absolute;\n}\n",""])},function(e,t,n){(e.exports=n(68)(!1)).push([e.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Copyright (c) 2014-2017, PhosphorJS Contributors\n|\n| Distributed under the terms of the BSD 3-Clause License.\n|\n| The full license is in the file LICENSE, distributed with this software.\n|----------------------------------------------------------------------------*/\n\n\n/* <DEPRECATED> */ .p-SplitPanel-child, /* </DEPRECATED> */\n.lm-SplitPanel-child {\n  z-index: 0;\n}\n\n\n/* <DEPRECATED> */ .p-SplitPanel-handle, /* </DEPRECATED> */\n.lm-SplitPanel-handle {\n  z-index: 1;\n}\n\n\n/* <DEPRECATED> */ .p-SplitPanel-handle.p-mod-hidden, /* </DEPRECATED> */\n.lm-SplitPanel-handle.lm-mod-hidden {\n  display: none !important;\n}\n\n\n/* <DEPRECATED> */ .p-SplitPanel-handle:after, /* </DEPRECATED> */\n.lm-SplitPanel-handle:after {\n  position: absolute;\n  top: 0;\n  left: 0;\n  width: 100%;\n  height: 100%;\n  content: '';\n}\n\n\n/* <DEPRECATED> */\n.p-SplitPanel[data-orientation='horizontal'] > .p-SplitPanel-handle,\n/* </DEPRECATED> */\n.lm-SplitPanel[data-orientation='horizontal'] > .lm-SplitPanel-handle {\n  cursor: ew-resize;\n}\n\n\n/* <DEPRECATED> */\n.p-SplitPanel[data-orientation='vertical'] > .p-SplitPanel-handle,\n/* </DEPRECATED> */\n.lm-SplitPanel[data-orientation='vertical'] > .lm-SplitPanel-handle {\n  cursor: ns-resize;\n}\n\n\n/* <DEPRECATED> */\n.p-SplitPanel[data-orientation='horizontal'] > .p-SplitPanel-handle:after,\n/* </DEPRECATED> */\n.lm-SplitPanel[data-orientation='horizontal'] > .lm-SplitPanel-handle:after {\n  left: 50%;\n  min-width: 8px;\n  transform: translateX(-50%);\n}\n\n\n/* <DEPRECATED> */\n.p-SplitPanel[data-orientation='vertical'] > .p-SplitPanel-handle:after,\n/* </DEPRECATED> */\n.lm-SplitPanel[data-orientation='vertical'] > .lm-SplitPanel-handle:after {\n  top: 50%;\n  min-height: 8px;\n  transform: translateY(-50%);\n}\n",""])},function(e,t,n){(e.exports=n(68)(!1)).push([e.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Copyright (c) 2014-2017, PhosphorJS Contributors\n|\n| Distributed under the terms of the BSD 3-Clause License.\n|\n| The full license is in the file LICENSE, distributed with this software.\n|----------------------------------------------------------------------------*/\n\n\n/* <DEPRECATED> */ .p-TabBar, /* </DEPRECATED> */\n.lm-TabBar {\n  display: flex;\n  -webkit-user-select: none;\n  -moz-user-select: none;\n  -ms-user-select: none;\n  user-select: none;\n}\n\n\n/* <DEPRECATED> */ .p-TabBar[data-orientation='horizontal'], /* </DEPRECATED> */\n.lm-TabBar[data-orientation='horizontal'] {\n  flex-direction: row;\n}\n\n\n/* <DEPRECATED> */ .p-TabBar[data-orientation='vertical'], /* </DEPRECATED> */\n.lm-TabBar[data-orientation='vertical'] {\n  flex-direction: column;\n}\n\n\n/* <DEPRECATED> */ .p-TabBar-content, /* </DEPRECATED> */\n.lm-TabBar-content {\n  margin: 0;\n  padding: 0;\n  display: flex;\n  flex: 1 1 auto;\n  list-style-type: none;\n}\n\n\n/* <DEPRECATED> */\n.p-TabBar[data-orientation='horizontal'] > .p-TabBar-content,\n/* </DEPRECATED> */\n.lm-TabBar[data-orientation='horizontal'] > .lm-TabBar-content {\n  flex-direction: row;\n}\n\n\n/* <DEPRECATED> */\n.p-TabBar[data-orientation='vertical'] > .p-TabBar-content,\n/* </DEPRECATED> */\n.lm-TabBar[data-orientation='vertical'] > .lm-TabBar-content {\n  flex-direction: column;\n}\n\n\n/* <DEPRECATED> */ .p-TabBar-tab, /* </DEPRECATED> */\n.lm-TabBar-tab {\n  display: flex;\n  flex-direction: row;\n  box-sizing: border-box;\n  overflow: hidden;\n}\n\n\n/* <DEPRECATED> */\n.p-TabBar-tabIcon,\n.p-TabBar-tabCloseIcon,\n/* </DEPRECATED> */\n.lm-TabBar-tabIcon,\n.lm-TabBar-tabCloseIcon {\n  flex: 0 0 auto;\n}\n\n\n/* <DEPRECATED> */ .p-TabBar-tabLabel, /* </DEPRECATED> */\n.lm-TabBar-tabLabel {\n  flex: 1 1 auto;\n  overflow: hidden;\n  white-space: nowrap;\n}\n\n\n.lm-TabBar-tabInput {\n  user-select: all;\n  width: 100%;\n  box-sizing : border-box;\n}\n\n\n/* <DEPRECATED> */ .p-TabBar-tab.p-mod-hidden, /* </DEPRECATED> */\n.lm-TabBar-tab.lm-mod-hidden {\n  display: none !important;\n}\n\n\n/* <DEPRECATED> */ .p-TabBar.p-mod-dragging .p-TabBar-tab, /* </DEPRECATED> */\n.lm-TabBar.lm-mod-dragging .lm-TabBar-tab {\n  position: relative;\n}\n\n\n/* <DEPRECATED> */\n.p-TabBar.p-mod-dragging[data-orientation='horizontal'] .p-TabBar-tab,\n/* </DEPRECATED> */\n.lm-TabBar.lm-mod-dragging[data-orientation='horizontal'] .lm-TabBar-tab {\n  left: 0;\n  transition: left 150ms ease;\n}\n\n\n/* <DEPRECATED> */\n.p-TabBar.p-mod-dragging[data-orientation='vertical'] .p-TabBar-tab,\n/* </DEPRECATED> */\n.lm-TabBar.lm-mod-dragging[data-orientation='vertical'] .lm-TabBar-tab {\n  top: 0;\n  transition: top 150ms ease;\n}\n\n\n/* <DEPRECATED> */\n.p-TabBar.p-mod-dragging .p-TabBar-tab.p-mod-dragging,\n/* </DEPRECATED> */\n.lm-TabBar.lm-mod-dragging .lm-TabBar-tab.lm-mod-dragging {\n  transition: none;\n}\n",""])},function(e,t,n){(e.exports=n(68)(!1)).push([e.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Copyright (c) 2014-2017, PhosphorJS Contributors\n|\n| Distributed under the terms of the BSD 3-Clause License.\n|\n| The full license is in the file LICENSE, distributed with this software.\n|----------------------------------------------------------------------------*/\n\n\n/* <DEPRECATED> */ .p-TabPanel-tabBar, /* </DEPRECATED> */\n.lm-TabPanel-tabBar {\n  z-index: 1;\n}\n\n\n/* <DEPRECATED> */ .p-TabPanel-stackedPanel, /* </DEPRECATED> */\n.lm-TabPanel-stackedPanel {\n  z-index: 0;\n}\n",""])},function(e,t,n){var r=n(1517);"string"==typeof r&&(r=[[e.i,r,""]]);var i={insert:"head",singleton:!1};n(147)(r,i);r.locals&&(e.exports=r.locals)},function(e,t,n){(t=e.exports=n(68)(!1)).i(n(1518),""),t.i(n(1520),""),t.push([e.i,"/* Copyright (c) Jupyter Development Team.\n * Distributed under the terms of the Modified BSD License.\n */\n",""])},function(e,t,n){(t=e.exports=n(68)(!1)).i(n(1519),""),t.push([e.i,"/*-----------------------------------------------------------------------------\n| Copyright (c) Jupyter Development Team.\n| Distributed under the terms of the Modified BSD License.\n|----------------------------------------------------------------------------*/\n\n/*\nThis file is copied from the JupyterLab project to define default styling for\nwhen the widget styling is compiled down to eliminate CSS variables. We make one\nchange - we comment out the font import below.\n*/\n\n/*\nThe following CSS variables define the main, public API for styling JupyterLab.\nThese variables should be used by all plugins wherever possible. In other\nwords, plugins should not define custom colors, sizes, etc unless absolutely\nnecessary. This enables users to change the visual theme of JupyterLab\nby changing these variables.\n\nMany variables appear in an ordered sequence (0,1,2,3). These sequences\nare designed to work well together, so for example, `--jp-border-color1` should\nbe used with `--jp-layout-color1`. The numbers have the following meanings:\n\n* 0: super-primary, reserved for special emphasis\n* 1: primary, most important under normal situations\n* 2: secondary, next most important under normal situations\n* 3: tertiary, next most important under normal situations\n\nThroughout JupyterLab, we are mostly following principles from Google's\nMaterial Design when selecting colors. We are not, however, following\nall of MD as it is not optimized for dense, information rich UIs.\n*/\n\n\n/*\n * Optional monospace font for input/output prompt.\n */\n /* Commented out in ipywidgets since we don't need it. */\n/* @import url('https://fonts.googleapis.com/css?family=Roboto+Mono'); */\n\n/*\n * Added for compabitility with output area\n */\n:root {\n  --jp-icon-search: none;\n  --jp-ui-select-caret: none;\n}\n\n\n:root {\n\n  /* Borders\n\n  The following variables, specify the visual styling of borders in JupyterLab.\n   */\n\n  --jp-border-width: 1px;\n  --jp-border-color0: var(--md-grey-700);\n  --jp-border-color1: var(--md-grey-500);\n  --jp-border-color2: var(--md-grey-300);\n  --jp-border-color3: var(--md-grey-100);\n\n  /* UI Fonts\n\n  The UI font CSS variables are used for the typography all of the JupyterLab\n  user interface elements that are not directly user generated content.\n  */\n\n  --jp-ui-font-scale-factor: 1.2;\n  --jp-ui-font-size0: calc(var(--jp-ui-font-size1)/var(--jp-ui-font-scale-factor));\n  --jp-ui-font-size1: 13px; /* Base font size */\n  --jp-ui-font-size2: calc(var(--jp-ui-font-size1)*var(--jp-ui-font-scale-factor));\n  --jp-ui-font-size3: calc(var(--jp-ui-font-size2)*var(--jp-ui-font-scale-factor));\n  --jp-ui-icon-font-size: 14px; /* Ensures px perfect FontAwesome icons */\n  --jp-ui-font-family: \"Helvetica Neue\", Helvetica, Arial, sans-serif;\n\n  /* Use these font colors against the corresponding main layout colors.\n     In a light theme, these go from dark to light.\n  */\n\n  --jp-ui-font-color0: rgba(0,0,0,1.0);\n  --jp-ui-font-color1: rgba(0,0,0,0.8);\n  --jp-ui-font-color2: rgba(0,0,0,0.5);\n  --jp-ui-font-color3: rgba(0,0,0,0.3);\n\n  /* Use these against the brand/accent/warn/error colors.\n     These will typically go from light to darker, in both a dark and light theme\n   */\n\n  --jp-ui-inverse-font-color0: rgba(255, 255, 255, 1);\n  --jp-ui-inverse-font-color1: rgba(255, 255, 255, 1);\n  --jp-ui-inverse-font-color2: rgba(255, 255, 255, 0.7);\n  --jp-ui-inverse-font-color3: rgba(255, 255, 255, 0.5);\n\n  /* For backwards compatibility, we still define these below until ipywidgets 8.0.\n  See https://github.com/jupyter-widgets/ipywidgets/pull/2801 */\n  --jp-inverse-ui-font-color0: rgba(255,255,255,1);\n  --jp-inverse-ui-font-color1: rgba(255,255,255,1.0);\n  --jp-inverse-ui-font-color2: rgba(255,255,255,0.7);\n  --jp-inverse-ui-font-color3: rgba(255,255,255,0.5);\n\n  /* Content Fonts\n\n  Content font variables are used for typography of user generated content.\n  */\n\n  --jp-content-font-size: 13px;\n  --jp-content-line-height: 1.5;\n  --jp-content-font-color0: black;\n  --jp-content-font-color1: black;\n  --jp-content-font-color2: var(--md-grey-700);\n  --jp-content-font-color3: var(--md-grey-500);\n\n  --jp-ui-font-scale-factor: 1.2;\n  --jp-ui-font-size0: calc(var(--jp-ui-font-size1)/var(--jp-ui-font-scale-factor));\n  --jp-ui-font-size1: 13px; /* Base font size */\n  --jp-ui-font-size2: calc(var(--jp-ui-font-size1)*var(--jp-ui-font-scale-factor));\n  --jp-ui-font-size3: calc(var(--jp-ui-font-size2)*var(--jp-ui-font-scale-factor));\n\n  --jp-code-font-size: 13px;\n  --jp-code-line-height: 1.307;\n  --jp-code-padding: 5px;\n  --jp-code-font-family: monospace;\n\n\n  /* Layout\n\n  The following are the main layout colors use in JupyterLab. In a light\n  theme these would go from light to dark.\n  */\n\n  --jp-layout-color0: white;\n  --jp-layout-color1: white;\n  --jp-layout-color2: var(--md-grey-200);\n  --jp-layout-color3: var(--md-grey-400);\n\n  /* Brand/accent */\n\n  --jp-brand-color0: var(--md-blue-700);\n  --jp-brand-color1: var(--md-blue-500);\n  --jp-brand-color2: var(--md-blue-300);\n  --jp-brand-color3: var(--md-blue-100);\n\n  --jp-accent-color0: var(--md-green-700);\n  --jp-accent-color1: var(--md-green-500);\n  --jp-accent-color2: var(--md-green-300);\n  --jp-accent-color3: var(--md-green-100);\n\n  /* State colors (warn, error, success, info) */\n\n  --jp-warn-color0: var(--md-orange-700);\n  --jp-warn-color1: var(--md-orange-500);\n  --jp-warn-color2: var(--md-orange-300);\n  --jp-warn-color3: var(--md-orange-100);\n\n  --jp-error-color0: var(--md-red-700);\n  --jp-error-color1: var(--md-red-500);\n  --jp-error-color2: var(--md-red-300);\n  --jp-error-color3: var(--md-red-100);\n\n  --jp-success-color0: var(--md-green-700);\n  --jp-success-color1: var(--md-green-500);\n  --jp-success-color2: var(--md-green-300);\n  --jp-success-color3: var(--md-green-100);\n\n  --jp-info-color0: var(--md-cyan-700);\n  --jp-info-color1: var(--md-cyan-500);\n  --jp-info-color2: var(--md-cyan-300);\n  --jp-info-color3: var(--md-cyan-100);\n\n  /* Cell specific styles */\n\n  --jp-cell-padding: 5px;\n  --jp-cell-editor-background: #f7f7f7;\n  --jp-cell-editor-border-color: #cfcfcf;\n  --jp-cell-editor-background-edit: var(--jp-ui-layout-color1);\n  --jp-cell-editor-border-color-edit: var(--jp-brand-color1);\n  --jp-cell-prompt-width: 100px;\n  --jp-cell-prompt-font-family: 'Roboto Mono', monospace;\n  --jp-cell-prompt-letter-spacing: 0px;\n  --jp-cell-prompt-opacity: 1.0;\n  --jp-cell-prompt-opacity-not-active: 0.4;\n  --jp-cell-prompt-font-color-not-active: var(--md-grey-700);\n  /* A custom blend of MD grey and blue 600\n   * See https://meyerweb.com/eric/tools/color-blend/#546E7A:1E88E5:5:hex */\n  --jp-cell-inprompt-font-color: #307FC1;\n  /* A custom blend of MD grey and orange 600\n   * https://meyerweb.com/eric/tools/color-blend/#546E7A:F4511E:5:hex */\n  --jp-cell-outprompt-font-color: #BF5B3D;\n\n  /* Notebook specific styles */\n\n  --jp-notebook-padding: 10px;\n  --jp-notebook-scroll-padding: 100px;\n\n  /* Console specific styles */\n\n  --jp-console-background: var(--md-grey-100);\n\n  /* Toolbar specific styles */\n\n  --jp-toolbar-border-color: var(--md-grey-400);\n  --jp-toolbar-micro-height: 8px;\n  --jp-toolbar-background: var(--jp-layout-color0);\n  --jp-toolbar-box-shadow: 0px 0px 2px 0px rgba(0,0,0,0.24);\n  --jp-toolbar-header-margin: 4px 4px 0px 4px;\n  --jp-toolbar-active-background: var(--md-grey-300);\n}\n",""])},function(e,t,n){(e.exports=n(68)(!1)).push([e.i,'/**\n * The material design colors are adapted from google-material-color v1.2.6\n * https://github.com/danlevan/google-material-color\n * https://github.com/danlevan/google-material-color/blob/f67ca5f4028b2f1b34862f64b0ca67323f91b088/dist/palette.var.css\n *\n * The license for the material design color CSS variables is as follows (see\n * https://github.com/danlevan/google-material-color/blob/f67ca5f4028b2f1b34862f64b0ca67323f91b088/LICENSE)\n *\n * The MIT License (MIT)\n *\n * Copyright (c) 2014 Dan Le Van\n *\n * Permission is hereby granted, free of charge, to any person obtaining a copy\n * of this software and associated documentation files (the "Software"), to deal\n * in the Software without restriction, including without limitation the rights\n * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell\n * copies of the Software, and to permit persons to whom the Software is\n * furnished to do so, subject to the following conditions:\n *\n * The above copyright notice and this permission notice shall be included in\n * all copies or substantial portions of the Software.\n *\n * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\n * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\n * SOFTWARE.\n */\n:root {\n  --md-red-50: #FFEBEE;\n  --md-red-100: #FFCDD2;\n  --md-red-200: #EF9A9A;\n  --md-red-300: #E57373;\n  --md-red-400: #EF5350;\n  --md-red-500: #F44336;\n  --md-red-600: #E53935;\n  --md-red-700: #D32F2F;\n  --md-red-800: #C62828;\n  --md-red-900: #B71C1C;\n  --md-red-A100: #FF8A80;\n  --md-red-A200: #FF5252;\n  --md-red-A400: #FF1744;\n  --md-red-A700: #D50000;\n\n  --md-pink-50: #FCE4EC;\n  --md-pink-100: #F8BBD0;\n  --md-pink-200: #F48FB1;\n  --md-pink-300: #F06292;\n  --md-pink-400: #EC407A;\n  --md-pink-500: #E91E63;\n  --md-pink-600: #D81B60;\n  --md-pink-700: #C2185B;\n  --md-pink-800: #AD1457;\n  --md-pink-900: #880E4F;\n  --md-pink-A100: #FF80AB;\n  --md-pink-A200: #FF4081;\n  --md-pink-A400: #F50057;\n  --md-pink-A700: #C51162;\n\n  --md-purple-50: #F3E5F5;\n  --md-purple-100: #E1BEE7;\n  --md-purple-200: #CE93D8;\n  --md-purple-300: #BA68C8;\n  --md-purple-400: #AB47BC;\n  --md-purple-500: #9C27B0;\n  --md-purple-600: #8E24AA;\n  --md-purple-700: #7B1FA2;\n  --md-purple-800: #6A1B9A;\n  --md-purple-900: #4A148C;\n  --md-purple-A100: #EA80FC;\n  --md-purple-A200: #E040FB;\n  --md-purple-A400: #D500F9;\n  --md-purple-A700: #AA00FF;\n\n  --md-deep-purple-50: #EDE7F6;\n  --md-deep-purple-100: #D1C4E9;\n  --md-deep-purple-200: #B39DDB;\n  --md-deep-purple-300: #9575CD;\n  --md-deep-purple-400: #7E57C2;\n  --md-deep-purple-500: #673AB7;\n  --md-deep-purple-600: #5E35B1;\n  --md-deep-purple-700: #512DA8;\n  --md-deep-purple-800: #4527A0;\n  --md-deep-purple-900: #311B92;\n  --md-deep-purple-A100: #B388FF;\n  --md-deep-purple-A200: #7C4DFF;\n  --md-deep-purple-A400: #651FFF;\n  --md-deep-purple-A700: #6200EA;\n\n  --md-indigo-50: #E8EAF6;\n  --md-indigo-100: #C5CAE9;\n  --md-indigo-200: #9FA8DA;\n  --md-indigo-300: #7986CB;\n  --md-indigo-400: #5C6BC0;\n  --md-indigo-500: #3F51B5;\n  --md-indigo-600: #3949AB;\n  --md-indigo-700: #303F9F;\n  --md-indigo-800: #283593;\n  --md-indigo-900: #1A237E;\n  --md-indigo-A100: #8C9EFF;\n  --md-indigo-A200: #536DFE;\n  --md-indigo-A400: #3D5AFE;\n  --md-indigo-A700: #304FFE;\n\n  --md-blue-50: #E3F2FD;\n  --md-blue-100: #BBDEFB;\n  --md-blue-200: #90CAF9;\n  --md-blue-300: #64B5F6;\n  --md-blue-400: #42A5F5;\n  --md-blue-500: #2196F3;\n  --md-blue-600: #1E88E5;\n  --md-blue-700: #1976D2;\n  --md-blue-800: #1565C0;\n  --md-blue-900: #0D47A1;\n  --md-blue-A100: #82B1FF;\n  --md-blue-A200: #448AFF;\n  --md-blue-A400: #2979FF;\n  --md-blue-A700: #2962FF;\n\n  --md-light-blue-50: #E1F5FE;\n  --md-light-blue-100: #B3E5FC;\n  --md-light-blue-200: #81D4FA;\n  --md-light-blue-300: #4FC3F7;\n  --md-light-blue-400: #29B6F6;\n  --md-light-blue-500: #03A9F4;\n  --md-light-blue-600: #039BE5;\n  --md-light-blue-700: #0288D1;\n  --md-light-blue-800: #0277BD;\n  --md-light-blue-900: #01579B;\n  --md-light-blue-A100: #80D8FF;\n  --md-light-blue-A200: #40C4FF;\n  --md-light-blue-A400: #00B0FF;\n  --md-light-blue-A700: #0091EA;\n\n  --md-cyan-50: #E0F7FA;\n  --md-cyan-100: #B2EBF2;\n  --md-cyan-200: #80DEEA;\n  --md-cyan-300: #4DD0E1;\n  --md-cyan-400: #26C6DA;\n  --md-cyan-500: #00BCD4;\n  --md-cyan-600: #00ACC1;\n  --md-cyan-700: #0097A7;\n  --md-cyan-800: #00838F;\n  --md-cyan-900: #006064;\n  --md-cyan-A100: #84FFFF;\n  --md-cyan-A200: #18FFFF;\n  --md-cyan-A400: #00E5FF;\n  --md-cyan-A700: #00B8D4;\n\n  --md-teal-50: #E0F2F1;\n  --md-teal-100: #B2DFDB;\n  --md-teal-200: #80CBC4;\n  --md-teal-300: #4DB6AC;\n  --md-teal-400: #26A69A;\n  --md-teal-500: #009688;\n  --md-teal-600: #00897B;\n  --md-teal-700: #00796B;\n  --md-teal-800: #00695C;\n  --md-teal-900: #004D40;\n  --md-teal-A100: #A7FFEB;\n  --md-teal-A200: #64FFDA;\n  --md-teal-A400: #1DE9B6;\n  --md-teal-A700: #00BFA5;\n\n  --md-green-50: #E8F5E9;\n  --md-green-100: #C8E6C9;\n  --md-green-200: #A5D6A7;\n  --md-green-300: #81C784;\n  --md-green-400: #66BB6A;\n  --md-green-500: #4CAF50;\n  --md-green-600: #43A047;\n  --md-green-700: #388E3C;\n  --md-green-800: #2E7D32;\n  --md-green-900: #1B5E20;\n  --md-green-A100: #B9F6CA;\n  --md-green-A200: #69F0AE;\n  --md-green-A400: #00E676;\n  --md-green-A700: #00C853;\n\n  --md-light-green-50: #F1F8E9;\n  --md-light-green-100: #DCEDC8;\n  --md-light-green-200: #C5E1A5;\n  --md-light-green-300: #AED581;\n  --md-light-green-400: #9CCC65;\n  --md-light-green-500: #8BC34A;\n  --md-light-green-600: #7CB342;\n  --md-light-green-700: #689F38;\n  --md-light-green-800: #558B2F;\n  --md-light-green-900: #33691E;\n  --md-light-green-A100: #CCFF90;\n  --md-light-green-A200: #B2FF59;\n  --md-light-green-A400: #76FF03;\n  --md-light-green-A700: #64DD17;\n\n  --md-lime-50: #F9FBE7;\n  --md-lime-100: #F0F4C3;\n  --md-lime-200: #E6EE9C;\n  --md-lime-300: #DCE775;\n  --md-lime-400: #D4E157;\n  --md-lime-500: #CDDC39;\n  --md-lime-600: #C0CA33;\n  --md-lime-700: #AFB42B;\n  --md-lime-800: #9E9D24;\n  --md-lime-900: #827717;\n  --md-lime-A100: #F4FF81;\n  --md-lime-A200: #EEFF41;\n  --md-lime-A400: #C6FF00;\n  --md-lime-A700: #AEEA00;\n\n  --md-yellow-50: #FFFDE7;\n  --md-yellow-100: #FFF9C4;\n  --md-yellow-200: #FFF59D;\n  --md-yellow-300: #FFF176;\n  --md-yellow-400: #FFEE58;\n  --md-yellow-500: #FFEB3B;\n  --md-yellow-600: #FDD835;\n  --md-yellow-700: #FBC02D;\n  --md-yellow-800: #F9A825;\n  --md-yellow-900: #F57F17;\n  --md-yellow-A100: #FFFF8D;\n  --md-yellow-A200: #FFFF00;\n  --md-yellow-A400: #FFEA00;\n  --md-yellow-A700: #FFD600;\n\n  --md-amber-50: #FFF8E1;\n  --md-amber-100: #FFECB3;\n  --md-amber-200: #FFE082;\n  --md-amber-300: #FFD54F;\n  --md-amber-400: #FFCA28;\n  --md-amber-500: #FFC107;\n  --md-amber-600: #FFB300;\n  --md-amber-700: #FFA000;\n  --md-amber-800: #FF8F00;\n  --md-amber-900: #FF6F00;\n  --md-amber-A100: #FFE57F;\n  --md-amber-A200: #FFD740;\n  --md-amber-A400: #FFC400;\n  --md-amber-A700: #FFAB00;\n\n  --md-orange-50: #FFF3E0;\n  --md-orange-100: #FFE0B2;\n  --md-orange-200: #FFCC80;\n  --md-orange-300: #FFB74D;\n  --md-orange-400: #FFA726;\n  --md-orange-500: #FF9800;\n  --md-orange-600: #FB8C00;\n  --md-orange-700: #F57C00;\n  --md-orange-800: #EF6C00;\n  --md-orange-900: #E65100;\n  --md-orange-A100: #FFD180;\n  --md-orange-A200: #FFAB40;\n  --md-orange-A400: #FF9100;\n  --md-orange-A700: #FF6D00;\n\n  --md-deep-orange-50: #FBE9E7;\n  --md-deep-orange-100: #FFCCBC;\n  --md-deep-orange-200: #FFAB91;\n  --md-deep-orange-300: #FF8A65;\n  --md-deep-orange-400: #FF7043;\n  --md-deep-orange-500: #FF5722;\n  --md-deep-orange-600: #F4511E;\n  --md-deep-orange-700: #E64A19;\n  --md-deep-orange-800: #D84315;\n  --md-deep-orange-900: #BF360C;\n  --md-deep-orange-A100: #FF9E80;\n  --md-deep-orange-A200: #FF6E40;\n  --md-deep-orange-A400: #FF3D00;\n  --md-deep-orange-A700: #DD2C00;\n\n  --md-brown-50: #EFEBE9;\n  --md-brown-100: #D7CCC8;\n  --md-brown-200: #BCAAA4;\n  --md-brown-300: #A1887F;\n  --md-brown-400: #8D6E63;\n  --md-brown-500: #795548;\n  --md-brown-600: #6D4C41;\n  --md-brown-700: #5D4037;\n  --md-brown-800: #4E342E;\n  --md-brown-900: #3E2723;\n\n  --md-grey-50: #FAFAFA;\n  --md-grey-100: #F5F5F5;\n  --md-grey-200: #EEEEEE;\n  --md-grey-300: #E0E0E0;\n  --md-grey-400: #BDBDBD;\n  --md-grey-500: #9E9E9E;\n  --md-grey-600: #757575;\n  --md-grey-700: #616161;\n  --md-grey-800: #424242;\n  --md-grey-900: #212121;\n\n  --md-blue-grey-50: #ECEFF1;\n  --md-blue-grey-100: #CFD8DC;\n  --md-blue-grey-200: #B0BEC5;\n  --md-blue-grey-300: #90A4AE;\n  --md-blue-grey-400: #78909C;\n  --md-blue-grey-500: #607D8B;\n  --md-blue-grey-600: #546E7A;\n  --md-blue-grey-700: #455A64;\n  --md-blue-grey-800: #37474F;\n  --md-blue-grey-900: #263238;\n}',""])},function(e,t,n){(t=e.exports=n(68)(!1)).i(n(1521),""),t.push([e.i,'/* Copyright (c) Jupyter Development Team.\n * Distributed under the terms of the Modified BSD License.\n */\n\n/*\n * We assume that the CSS variables in\n * https://github.com/jupyterlab/jupyterlab/blob/master/src/default-theme/variables.css\n * have been defined.\n */\n\n:root {\n    --jp-widgets-color: var(--jp-content-font-color1);\n    --jp-widgets-label-color: var(--jp-widgets-color);\n    --jp-widgets-readout-color: var(--jp-widgets-color);\n    --jp-widgets-font-size: var(--jp-ui-font-size1);\n    --jp-widgets-margin: 2px;\n    --jp-widgets-inline-height: 28px;\n    --jp-widgets-inline-width: 300px;\n    --jp-widgets-inline-width-short: calc(var(--jp-widgets-inline-width) / 2 - var(--jp-widgets-margin));\n    --jp-widgets-inline-width-tiny: calc(var(--jp-widgets-inline-width-short) / 2 - var(--jp-widgets-margin));\n    --jp-widgets-inline-margin: 4px; /* margin between inline elements */\n    --jp-widgets-inline-label-width: 80px;\n    --jp-widgets-border-width: var(--jp-border-width);\n    --jp-widgets-vertical-height: 200px;\n    --jp-widgets-horizontal-tab-height: 24px;\n    --jp-widgets-horizontal-tab-width: 144px;\n    --jp-widgets-horizontal-tab-top-border: 2px;\n    --jp-widgets-progress-thickness: 20px;\n    --jp-widgets-container-padding: 15px;\n    --jp-widgets-input-padding: 4px;\n    --jp-widgets-radio-item-height-adjustment: 8px;\n    --jp-widgets-radio-item-height: calc(var(--jp-widgets-inline-height) - var(--jp-widgets-radio-item-height-adjustment));\n    --jp-widgets-slider-track-thickness: 4px;\n    --jp-widgets-slider-border-width: var(--jp-widgets-border-width);\n    --jp-widgets-slider-handle-size: 16px;\n    --jp-widgets-slider-handle-border-color: var(--jp-border-color1);\n    --jp-widgets-slider-handle-background-color: var(--jp-layout-color1);\n    --jp-widgets-slider-active-handle-color: var(--jp-brand-color1);\n    --jp-widgets-menu-item-height: 24px;\n    --jp-widgets-dropdown-arrow: url("data:image/svg+xml;base64,PD94bWwgdmVyc2lvbj0iMS4wIiBlbmNvZGluZz0idXRmLTgiPz4KPCEtLSBHZW5lcmF0b3I6IEFkb2JlIElsbHVzdHJhdG9yIDE5LjIuMSwgU1ZHIEV4cG9ydCBQbHVnLUluIC4gU1ZHIFZlcnNpb246IDYuMDAgQnVpbGQgMCkgIC0tPgo8c3ZnIHZlcnNpb249IjEuMSIgaWQ9IkxheWVyXzEiIHhtbG5zPSJodHRwOi8vd3d3LnczLm9yZy8yMDAwL3N2ZyIgeG1sbnM6eGxpbms9Imh0dHA6Ly93d3cudzMub3JnLzE5OTkveGxpbmsiIHg9IjBweCIgeT0iMHB4IgoJIHZpZXdCb3g9IjAgMCAxOCAxOCIgc3R5bGU9ImVuYWJsZS1iYWNrZ3JvdW5kOm5ldyAwIDAgMTggMTg7IiB4bWw6c3BhY2U9InByZXNlcnZlIj4KPHN0eWxlIHR5cGU9InRleHQvY3NzIj4KCS5zdDB7ZmlsbDpub25lO30KPC9zdHlsZT4KPHBhdGggZD0iTTUuMiw1LjlMOSw5LjdsMy44LTMuOGwxLjIsMS4ybC00LjksNWwtNC45LTVMNS4yLDUuOXoiLz4KPHBhdGggY2xhc3M9InN0MCIgZD0iTTAtMC42aDE4djE4SDBWLTAuNnoiLz4KPC9zdmc+Cg");\n    --jp-widgets-input-color: var(--jp-ui-font-color1);\n    --jp-widgets-input-background-color: var(--jp-layout-color1);\n    --jp-widgets-input-border-color: var(--jp-border-color1);\n    --jp-widgets-input-focus-border-color: var(--jp-brand-color2);\n    --jp-widgets-input-border-width: var(--jp-widgets-border-width);\n    --jp-widgets-disabled-opacity: 0.6;\n\n    /* From Material Design Lite */\n    --md-shadow-key-umbra-opacity: 0.2;\n    --md-shadow-key-penumbra-opacity: 0.14;\n    --md-shadow-ambient-shadow-opacity: 0.12;\n}\n\n.jupyter-widgets {\n    margin: var(--jp-widgets-margin);\n    box-sizing: border-box;\n    color: var(--jp-widgets-color);\n    overflow: visible;\n}\n\n.jupyter-widgets.jupyter-widgets-disconnected::before {\n    line-height: var(--jp-widgets-inline-height);\n    height: var(--jp-widgets-inline-height);\n}\n\n.jp-Output-result > .jupyter-widgets {\n    margin-left: 0;\n    margin-right: 0;\n}\n\n/* vbox and hbox */\n\n.widget-inline-hbox {\n    /* Horizontal widgets */\n    box-sizing: border-box;\n    display: flex;\n    flex-direction: row;\n    align-items: baseline;\n}\n\n.widget-inline-vbox {\n    /* Vertical Widgets */\n    box-sizing: border-box;\n    display: flex;\n    flex-direction: column;\n    align-items: center;\n}\n\n.widget-box {\n    box-sizing: border-box;\n    display: flex;\n    margin: 0;\n    overflow: auto;\n}\n\n.widget-gridbox {\n    box-sizing: border-box;\n    display: grid;\n    margin: 0;\n    overflow: auto;\n}\n\n.widget-hbox {\n    flex-direction: row;\n}\n\n.widget-vbox {\n    flex-direction: column;\n}\n\n/* General Button Styling */\n\n.jupyter-button {\n    padding-left: 10px;\n    padding-right: 10px;\n    padding-top: 0px;\n    padding-bottom: 0px;\n    display: inline-block;\n    white-space: nowrap;\n    overflow: hidden;\n    text-overflow: ellipsis;\n    text-align: center;\n    font-size: var(--jp-widgets-font-size);\n    cursor: pointer;\n\n    height: var(--jp-widgets-inline-height);\n    border: 0px solid;\n    line-height: var(--jp-widgets-inline-height);\n    box-shadow: none;\n\n    color: var(--jp-ui-font-color1);\n    background-color: var(--jp-layout-color2);\n    border-color: var(--jp-border-color2);\n    border: none;\n    user-select: none;\n}\n\n.jupyter-button i.fa {\n    margin-right: var(--jp-widgets-inline-margin);\n    pointer-events: none;\n}\n\n.jupyter-button:empty:before {\n    content: "\\200b"; /* zero-width space */\n}\n\n.jupyter-widgets.jupyter-button:disabled {\n    opacity: var(--jp-widgets-disabled-opacity);\n}\n\n.jupyter-button i.fa.center {\n    margin-right: 0;\n}\n\n.jupyter-button:hover:enabled, .jupyter-button:focus:enabled {\n    /* MD Lite 2dp shadow */\n    box-shadow: 0 2px 2px 0 rgba(0, 0, 0, var(--md-shadow-key-penumbra-opacity)),\n                0 3px 1px -2px rgba(0, 0, 0, var(--md-shadow-key-umbra-opacity)),\n                0 1px 5px 0 rgba(0, 0, 0, var(--md-shadow-ambient-shadow-opacity));\n}\n\n.jupyter-button:active, .jupyter-button.mod-active {\n    /* MD Lite 4dp shadow */\n    box-shadow: 0 4px 5px 0 rgba(0, 0, 0, var(--md-shadow-key-penumbra-opacity)),\n                0 1px 10px 0 rgba(0, 0, 0, var(--md-shadow-ambient-shadow-opacity)),\n                0 2px 4px -1px rgba(0, 0, 0, var(--md-shadow-key-umbra-opacity));\n    color: var(--jp-ui-font-color1);\n    background-color: var(--jp-layout-color3);\n}\n\n.jupyter-button:focus:enabled {\n    outline: 1px solid var(--jp-widgets-input-focus-border-color);\n}\n\n/* Button "Primary" Styling */\n\n.jupyter-button.mod-primary {\n    color: var(--jp-ui-inverse-font-color1, var(--jp-inverse-ui-font-color1));\n    background-color: var(--jp-brand-color1);\n}\n\n.jupyter-button.mod-primary.mod-active {\n    color: var(--jp-ui-inverse-font-color0, var(--jp-inverse-ui-font-color0));\n    background-color: var(--jp-brand-color0);\n}\n\n.jupyter-button.mod-primary:active {\n    color: var(--jp-ui-inverse-font-color0, var(--jp-inverse-ui-font-color0));\n    background-color: var(--jp-brand-color0);\n}\n\n/* Button "Success" Styling */\n\n.jupyter-button.mod-success {\n    color: var(--jp-ui-inverse-font-color1, var(--jp-inverse-ui-font-color1));\n    background-color: var(--jp-success-color1);\n}\n\n.jupyter-button.mod-success.mod-active {\n    color: var(--jp-ui-inverse-font-color0, var(--jp-inverse-ui-font-color0));\n    background-color: var(--jp-success-color0);\n}\n\n.jupyter-button.mod-success:active {\n    color: var(--jp-ui-inverse-font-color0, var(--jp-inverse-ui-font-color0));\n    background-color: var(--jp-success-color0);\n}\n\n /* Button "Info" Styling */\n\n.jupyter-button.mod-info {\n    color: var(--jp-ui-inverse-font-color1, var(--jp-inverse-ui-font-color1));\n    background-color: var(--jp-info-color1);\n}\n\n.jupyter-button.mod-info.mod-active {\n    color: var(--jp-ui-inverse-font-color0, var(--jp-inverse-ui-font-color0));\n    background-color: var(--jp-info-color0);\n}\n\n.jupyter-button.mod-info:active {\n    color: var(--jp-ui-inverse-font-color0, var(--jp-inverse-ui-font-color0));\n    background-color: var(--jp-info-color0);\n}\n\n/* Button "Warning" Styling */\n\n.jupyter-button.mod-warning {\n    color: var(--jp-ui-inverse-font-color1, var(--jp-inverse-ui-font-color1));\n    background-color: var(--jp-warn-color1);\n}\n\n.jupyter-button.mod-warning.mod-active {\n    color: var(--jp-ui-inverse-font-color0, var(--jp-inverse-ui-font-color0));\n    background-color: var(--jp-warn-color0);\n}\n\n.jupyter-button.mod-warning:active {\n    color: var(--jp-ui-inverse-font-color0, var(--jp-inverse-ui-font-color0));\n    background-color: var(--jp-warn-color0);\n}\n\n/* Button "Danger" Styling */\n\n.jupyter-button.mod-danger {\n    color: var(--jp-ui-inverse-font-color1, var(--jp-inverse-ui-font-color1));\n    background-color: var(--jp-error-color1);\n}\n\n.jupyter-button.mod-danger.mod-active {\n    color: var(--jp-ui-inverse-font-color0, var(--jp-inverse-ui-font-color0));\n    background-color: var(--jp-error-color0);\n}\n\n.jupyter-button.mod-danger:active {\n    color: var(--jp-ui-inverse-font-color0, var(--jp-inverse-ui-font-color0));\n    background-color: var(--jp-error-color0);\n}\n\n/* Widget Button, Widget Toggle Button, Widget Upload */\n\n.widget-button, .widget-toggle-button, .widget-upload {\n    width: var(--jp-widgets-inline-width-short);\n}\n\n/* Widget Label Styling */\n\n/* Override Bootstrap label css */\n.jupyter-widgets label {\n    margin-bottom: initial;\n}\n\n.widget-label-basic {\n    /* Basic Label */\n    color: var(--jp-widgets-label-color);\n    font-size: var(--jp-widgets-font-size);\n    overflow: hidden;\n    text-overflow: ellipsis;\n    white-space: nowrap;\n    line-height: var(--jp-widgets-inline-height);\n}\n\n.widget-label {\n    /* Label */\n    color: var(--jp-widgets-label-color);\n    font-size: var(--jp-widgets-font-size);\n    overflow: hidden;\n    text-overflow: ellipsis;\n    white-space: nowrap;\n    line-height: var(--jp-widgets-inline-height);\n}\n\n.widget-inline-hbox .widget-label {\n    /* Horizontal Widget Label */\n    color: var(--jp-widgets-label-color);\n    text-align: right;\n    margin-right: calc( var(--jp-widgets-inline-margin) * 2 );\n    width: var(--jp-widgets-inline-label-width);\n    flex-shrink: 0;\n}\n\n.widget-inline-vbox .widget-label {\n    /* Vertical Widget Label */\n    color: var(--jp-widgets-label-color);\n    text-align: center;\n    line-height: var(--jp-widgets-inline-height);\n}\n\n/* Widget Readout Styling */\n\n.widget-readout {\n    color: var(--jp-widgets-readout-color);\n    font-size: var(--jp-widgets-font-size);\n    height: var(--jp-widgets-inline-height);\n    line-height: var(--jp-widgets-inline-height);\n    overflow: hidden;\n    white-space: nowrap;\n    text-align: center;\n}\n\n.widget-readout.overflow {\n    /* Overflowing Readout */\n\n    /* From Material Design Lite\n        shadow-key-umbra-opacity: 0.2;\n        shadow-key-penumbra-opacity: 0.14;\n        shadow-ambient-shadow-opacity: 0.12;\n     */\n    -webkit-box-shadow: 0 2px 2px 0 rgba(0, 0, 0, 0.2),\n                        0 3px 1px -2px rgba(0, 0, 0, 0.14),\n                        0 1px 5px 0 rgba(0, 0, 0, 0.12);\n\n    -moz-box-shadow: 0 2px 2px 0 rgba(0, 0, 0, 0.2),\n                     0 3px 1px -2px rgba(0, 0, 0, 0.14),\n                     0 1px 5px 0 rgba(0, 0, 0, 0.12);\n\n    box-shadow: 0 2px 2px 0 rgba(0, 0, 0, 0.2),\n                0 3px 1px -2px rgba(0, 0, 0, 0.14),\n                0 1px 5px 0 rgba(0, 0, 0, 0.12);\n}\n\n.widget-inline-hbox .widget-readout {\n    /* Horizontal Readout */\n    text-align: center;\n    max-width: var(--jp-widgets-inline-width-short);\n    min-width: var(--jp-widgets-inline-width-tiny);\n    margin-left: var(--jp-widgets-inline-margin);\n}\n\n.widget-inline-vbox .widget-readout {\n    /* Vertical Readout */\n    margin-top: var(--jp-widgets-inline-margin);\n    /* as wide as the widget */\n    width: inherit;\n}\n\n/* Widget Checkbox Styling */\n\n.widget-checkbox {\n    width: var(--jp-widgets-inline-width);\n    height: var(--jp-widgets-inline-height);\n    line-height: var(--jp-widgets-inline-height);\n}\n\n.widget-checkbox input[type="checkbox"] {\n    margin: 0px calc( var(--jp-widgets-inline-margin) * 2 ) 0px 0px;\n    line-height: var(--jp-widgets-inline-height);\n    font-size: large;\n    flex-grow: 1;\n    flex-shrink: 0;\n    align-self: center;\n}\n\n/* Widget Valid Styling */\n\n.widget-valid {\n    height: var(--jp-widgets-inline-height);\n    line-height: var(--jp-widgets-inline-height);\n    width: var(--jp-widgets-inline-width-short);\n    font-size: var(--jp-widgets-font-size);\n}\n\n.widget-valid i:before {\n    line-height: var(--jp-widgets-inline-height);\n    margin-right: var(--jp-widgets-inline-margin);\n    margin-left: var(--jp-widgets-inline-margin);\n}\n\n.widget-valid.mod-valid i:before {\n    color: green;\n}\n\n.widget-valid.mod-invalid i:before {\n    color: red;\n}\n\n.widget-valid.mod-valid .widget-valid-readout {\n    display: none;\n}\n\n/* Widget Text and TextArea Stying */\n\n.widget-textarea, .widget-text {\n    width: var(--jp-widgets-inline-width);\n}\n\n.widget-text input[type="text"], .widget-text input[type="number"], .widget-text input[type="password"] {\n    height: var(--jp-widgets-inline-height);\n    line-height: var(--jp-widgets-inline-height);\n}\n\n.widget-text input[type="text"]:disabled, .widget-text input[type="number"]:disabled, .widget-text input[type="password"]:disabled, .widget-textarea textarea:disabled {\n    opacity: var(--jp-widgets-disabled-opacity);\n}\n\n.widget-text input[type="text"], .widget-text input[type="number"], .widget-text input[type="password"], .widget-textarea textarea {\n    box-sizing: border-box;\n    border: var(--jp-widgets-input-border-width) solid var(--jp-widgets-input-border-color);\n    background-color: var(--jp-widgets-input-background-color);\n    color: var(--jp-widgets-input-color);\n    font-size: var(--jp-widgets-font-size);\n    flex-grow: 1;\n    min-width: 0; /* This makes it possible for the flexbox to shrink this input */\n    flex-shrink: 1;\n    outline: none !important;\n}\n    \n.widget-text input[type="text"], .widget-text input[type="password"], .widget-textarea textarea {\n    padding: var(--jp-widgets-input-padding) calc( var(--jp-widgets-input-padding) *  2);\n}\n\n.widget-text input[type="number"] {\n    padding: var(--jp-widgets-input-padding) 0 var(--jp-widgets-input-padding) calc(var(--jp-widgets-input-padding) *  2);\n}\n\n.widget-textarea textarea {\n    height: inherit;\n    width: inherit;\n}\n\n.widget-text input:focus, .widget-textarea textarea:focus {\n    border-color: var(--jp-widgets-input-focus-border-color);\n}\n\n/* Widget Slider */\n\n.widget-slider .ui-slider {\n    /* Slider Track */\n    border: var(--jp-widgets-slider-border-width) solid var(--jp-layout-color3);\n    background: var(--jp-layout-color3);\n    box-sizing: border-box;\n    position: relative;\n    border-radius: 0px;\n}\n\n.widget-slider .ui-slider .ui-slider-handle {\n    /* Slider Handle */\n    outline: none !important; /* focused slider handles are colored - see below */\n    position: absolute;\n    background-color: var(--jp-widgets-slider-handle-background-color);\n    border: var(--jp-widgets-slider-border-width) solid var(--jp-widgets-slider-handle-border-color);\n    box-sizing: border-box;\n    z-index: 1;\n    background-image: none; /* Override jquery-ui */\n}\n\n/* Override jquery-ui */\n.widget-slider .ui-slider .ui-slider-handle:hover, .widget-slider .ui-slider .ui-slider-handle:focus {\n    background-color: var(--jp-widgets-slider-active-handle-color);\n    border: var(--jp-widgets-slider-border-width) solid var(--jp-widgets-slider-active-handle-color);\n}\n\n.widget-slider .ui-slider .ui-slider-handle:active {\n    background-color: var(--jp-widgets-slider-active-handle-color);\n    border-color: var(--jp-widgets-slider-active-handle-color);\n    z-index: 2;\n    transform: scale(1.2);\n}\n\n.widget-slider  .ui-slider .ui-slider-range {\n    /* Interval between the two specified value of a double slider */\n    position: absolute;\n    background: var(--jp-widgets-slider-active-handle-color);\n    z-index: 0;\n}\n\n/* Shapes of Slider Handles */\n\n.widget-hslider .ui-slider .ui-slider-handle {\n    width: var(--jp-widgets-slider-handle-size);\n    height: var(--jp-widgets-slider-handle-size);\n    margin-top: calc((var(--jp-widgets-slider-track-thickness) - var(--jp-widgets-slider-handle-size)) / 2 - var(--jp-widgets-slider-border-width));\n    margin-left: calc(var(--jp-widgets-slider-handle-size) / -2 + var(--jp-widgets-slider-border-width));\n    border-radius: 50%;\n    top: 0;\n}\n\n.widget-vslider .ui-slider .ui-slider-handle {\n    width: var(--jp-widgets-slider-handle-size);\n    height: var(--jp-widgets-slider-handle-size);\n    margin-bottom: calc(var(--jp-widgets-slider-handle-size) / -2 + var(--jp-widgets-slider-border-width));\n    margin-left: calc((var(--jp-widgets-slider-track-thickness) - var(--jp-widgets-slider-handle-size)) / 2 - var(--jp-widgets-slider-border-width));\n    border-radius: 50%;\n    left: 0;\n}\n\n.widget-hslider .ui-slider .ui-slider-range {\n    height: calc( var(--jp-widgets-slider-track-thickness) * 2 );\n    margin-top: calc((var(--jp-widgets-slider-track-thickness) - var(--jp-widgets-slider-track-thickness) * 2 ) / 2 - var(--jp-widgets-slider-border-width));\n}\n\n.widget-vslider .ui-slider .ui-slider-range {\n    width: calc( var(--jp-widgets-slider-track-thickness) * 2 );\n    margin-left: calc((var(--jp-widgets-slider-track-thickness) - var(--jp-widgets-slider-track-thickness) * 2 ) / 2 - var(--jp-widgets-slider-border-width));\n}\n\n/* Horizontal Slider */\n\n.widget-hslider {\n    width: var(--jp-widgets-inline-width);\n    height: var(--jp-widgets-inline-height);\n    line-height: var(--jp-widgets-inline-height);\n\n    /* Override the align-items baseline. This way, the description and readout\n    still seem to align their baseline properly, and we don\'t have to have\n    align-self: stretch in the .slider-container. */\n    align-items: center;\n}\n\n.widgets-slider .slider-container {\n    overflow: visible;\n}\n\n.widget-hslider .slider-container {\n    height: var(--jp-widgets-inline-height);\n    margin-left: calc(var(--jp-widgets-slider-handle-size) / 2 - 2 * var(--jp-widgets-slider-border-width));\n    margin-right: calc(var(--jp-widgets-slider-handle-size) / 2 - 2 * var(--jp-widgets-slider-border-width));\n    flex: 1 1 var(--jp-widgets-inline-width-short);\n}\n\n.widget-hslider .ui-slider {\n    /* Inner, invisible slide div */\n    height: var(--jp-widgets-slider-track-thickness);\n    margin-top: calc((var(--jp-widgets-inline-height) - var(--jp-widgets-slider-track-thickness)) / 2);\n    width: 100%;\n}\n\n/* Vertical Slider */\n\n.widget-vbox .widget-label {\n    height: var(--jp-widgets-inline-height);\n    line-height: var(--jp-widgets-inline-height);\n}\n\n.widget-vslider {\n    /* Vertical Slider */\n    height: var(--jp-widgets-vertical-height);\n    width: var(--jp-widgets-inline-width-tiny);\n}\n\n.widget-vslider .slider-container {\n    flex: 1 1 var(--jp-widgets-inline-width-short);\n    margin-left: auto;\n    margin-right: auto;\n    margin-bottom: calc(var(--jp-widgets-slider-handle-size) / 2 - 2 * var(--jp-widgets-slider-border-width));\n    margin-top: calc(var(--jp-widgets-slider-handle-size) / 2 - 2 * var(--jp-widgets-slider-border-width));\n    display: flex;\n    flex-direction: column;\n}\n\n.widget-vslider .ui-slider-vertical {\n    /* Inner, invisible slide div */\n    width: var(--jp-widgets-slider-track-thickness);\n    flex-grow: 1;\n    margin-left: auto;\n    margin-right: auto;\n}\n\n/* Widget Progress Styling */\n\n.progress-bar {\n    -webkit-transition: none;\n    -moz-transition: none;\n    -ms-transition: none;\n    -o-transition: none;\n    transition: none;\n}\n\n.progress-bar {\n    height: var(--jp-widgets-inline-height);\n}\n\n.progress-bar {\n    background-color: var(--jp-brand-color1);\n}\n\n.progress-bar-success {\n    background-color: var(--jp-success-color1);\n}\n\n.progress-bar-info {\n    background-color: var(--jp-info-color1);\n}\n\n.progress-bar-warning {\n    background-color: var(--jp-warn-color1);\n}\n\n.progress-bar-danger {\n    background-color: var(--jp-error-color1);\n}\n\n.progress {\n    background-color: var(--jp-layout-color2);\n    border: none;\n    box-shadow: none;\n}\n\n/* Horisontal Progress */\n\n.widget-hprogress {\n    /* Progress Bar */\n    height: var(--jp-widgets-inline-height);\n    line-height: var(--jp-widgets-inline-height);\n    width: var(--jp-widgets-inline-width);\n    align-items: center;\n\n}\n\n.widget-hprogress .progress {\n    flex-grow: 1;\n    margin-top: var(--jp-widgets-input-padding);\n    margin-bottom: var(--jp-widgets-input-padding);\n    align-self: stretch;\n    /* Override bootstrap style */\n    height: initial;\n}\n\n/* Vertical Progress */\n\n.widget-vprogress {\n    height: var(--jp-widgets-vertical-height);\n    width: var(--jp-widgets-inline-width-tiny);\n}\n\n.widget-vprogress .progress {\n    flex-grow: 1;\n    width: var(--jp-widgets-progress-thickness);\n    margin-left: auto;\n    margin-right: auto;\n    margin-bottom: 0;\n}\n\n/* Select Widget Styling */\n\n.widget-dropdown {\n    height: var(--jp-widgets-inline-height);\n    width: var(--jp-widgets-inline-width);\n    line-height: var(--jp-widgets-inline-height);\n}\n\n.widget-dropdown > select {\n    padding-right: 20px;\n    border: var(--jp-widgets-input-border-width) solid var(--jp-widgets-input-border-color);\n    border-radius: 0;\n    height: inherit;\n    flex: 1 1 var(--jp-widgets-inline-width-short);\n    min-width: 0; /* This makes it possible for the flexbox to shrink this input */\n    box-sizing: border-box;\n    outline: none !important;\n    box-shadow: none;\n    background-color: var(--jp-widgets-input-background-color);\n    color: var(--jp-widgets-input-color);\n    font-size: var(--jp-widgets-font-size);\n    vertical-align: top;\n    padding-left: calc( var(--jp-widgets-input-padding) * 2);\n\tappearance: none;\n\t-webkit-appearance: none;\n\t-moz-appearance: none;\n    background-repeat: no-repeat;\n\tbackground-size: 20px;\n\tbackground-position: right center;\n    background-image: var(--jp-widgets-dropdown-arrow);\n}\n.widget-dropdown > select:focus {\n    border-color: var(--jp-widgets-input-focus-border-color);\n}\n\n.widget-dropdown > select:disabled {\n    opacity: var(--jp-widgets-disabled-opacity);\n}\n\n/* To disable the dotted border in Firefox around select controls.\n   See http://stackoverflow.com/a/18853002 */\n.widget-dropdown > select:-moz-focusring {\n    color: transparent;\n    text-shadow: 0 0 0 #000;\n}\n\n/* Select and SelectMultiple */\n\n.widget-select {\n    width: var(--jp-widgets-inline-width);\n    line-height: var(--jp-widgets-inline-height);\n\n    /* Because Firefox defines the baseline of a select as the bottom of the\n    control, we align the entire control to the top and add padding to the\n    select to get an approximate first line baseline alignment. */\n    align-items: flex-start;\n}\n\n.widget-select > select {\n    border: var(--jp-widgets-input-border-width) solid var(--jp-widgets-input-border-color);\n    background-color: var(--jp-widgets-input-background-color);\n    color: var(--jp-widgets-input-color);\n    font-size: var(--jp-widgets-font-size);\n    flex: 1 1 var(--jp-widgets-inline-width-short);\n    outline: none !important;\n    overflow: auto;\n    height: inherit;\n\n    /* Because Firefox defines the baseline of a select as the bottom of the\n    control, we align the entire control to the top and add padding to the\n    select to get an approximate first line baseline alignment. */\n    padding-top: 5px;\n}\n\n.widget-select > select:focus {\n    border-color: var(--jp-widgets-input-focus-border-color);\n}\n\n.wiget-select > select > option {\n    padding-left: var(--jp-widgets-input-padding);\n    line-height: var(--jp-widgets-inline-height);\n    /* line-height doesn\'t work on some browsers for select options */\n    padding-top: calc(var(--jp-widgets-inline-height)-var(--jp-widgets-font-size)/2);\n    padding-bottom: calc(var(--jp-widgets-inline-height)-var(--jp-widgets-font-size)/2);\n}\n\n\n\n/* Toggle Buttons Styling */\n\n.widget-toggle-buttons {\n    line-height: var(--jp-widgets-inline-height);\n}\n\n.widget-toggle-buttons .widget-toggle-button {\n    margin-left: var(--jp-widgets-margin);\n    margin-right: var(--jp-widgets-margin);\n}\n\n.widget-toggle-buttons .jupyter-button:disabled {\n    opacity: var(--jp-widgets-disabled-opacity);\n}\n\n/* Radio Buttons Styling */\n\n.widget-radio {\n    width: var(--jp-widgets-inline-width);\n    line-height: var(--jp-widgets-inline-height);\n}\n\n.widget-radio-box {\n    display: flex;\n    flex-direction: column;\n    align-items: stretch;\n    box-sizing: border-box;\n    flex-grow: 1;\n    margin-bottom: var(--jp-widgets-radio-item-height-adjustment);\n}\n\n.widget-radio-box label {\n    height: var(--jp-widgets-radio-item-height);\n    line-height: var(--jp-widgets-radio-item-height);\n    font-size: var(--jp-widgets-font-size);\n}\n\n.widget-radio-box input {\n    height: var(--jp-widgets-radio-item-height);\n    line-height: var(--jp-widgets-radio-item-height);\n    margin: 0 calc( var(--jp-widgets-input-padding) * 2 ) 0 1px;\n    float: left;\n}\n\n/* Color Picker Styling */\n\n.widget-colorpicker {\n    width: var(--jp-widgets-inline-width);\n    height: var(--jp-widgets-inline-height);\n    line-height: var(--jp-widgets-inline-height);\n}\n\n.widget-colorpicker > .widget-colorpicker-input {\n    flex-grow: 1;\n    flex-shrink: 1;\n    min-width: var(--jp-widgets-inline-width-tiny);\n}\n\n.widget-colorpicker input[type="color"] {\n    width: var(--jp-widgets-inline-height);\n    height: var(--jp-widgets-inline-height);\n    padding: 0 2px; /* make the color square actually square on Chrome on OS X */\n    background: var(--jp-widgets-input-background-color);\n    color: var(--jp-widgets-input-color);\n    border: var(--jp-widgets-input-border-width) solid var(--jp-widgets-input-border-color);\n    border-left: none;\n    flex-grow: 0;\n    flex-shrink: 0;\n    box-sizing: border-box;\n    align-self: stretch;\n    outline: none !important;\n}\n\n.widget-colorpicker.concise input[type="color"] {\n    border-left: var(--jp-widgets-input-border-width) solid var(--jp-widgets-input-border-color);\n}\n\n.widget-colorpicker input[type="color"]:focus, .widget-colorpicker input[type="text"]:focus {\n    border-color: var(--jp-widgets-input-focus-border-color);\n}\n\n.widget-colorpicker input[type="text"] {\n    flex-grow: 1;\n    outline: none !important;\n    height: var(--jp-widgets-inline-height);\n    line-height: var(--jp-widgets-inline-height);\n    background: var(--jp-widgets-input-background-color);\n    color: var(--jp-widgets-input-color);\n    border: var(--jp-widgets-input-border-width) solid var(--jp-widgets-input-border-color);\n    font-size: var(--jp-widgets-font-size);\n    padding: var(--jp-widgets-input-padding) calc( var(--jp-widgets-input-padding) *  2 );\n    min-width: 0; /* This makes it possible for the flexbox to shrink this input */\n    flex-shrink: 1;\n    box-sizing: border-box;\n}\n\n.widget-colorpicker input[type="text"]:disabled {\n    opacity: var(--jp-widgets-disabled-opacity);\n}\n\n/* Date Picker Styling */\n\n.widget-datepicker {\n    width: var(--jp-widgets-inline-width);\n    height: var(--jp-widgets-inline-height);\n    line-height: var(--jp-widgets-inline-height);\n}\n\n.widget-datepicker input[type="date"] {\n    flex-grow: 1;\n    flex-shrink: 1;\n    min-width: 0; /* This makes it possible for the flexbox to shrink this input */\n    outline: none !important;\n    height: var(--jp-widgets-inline-height);\n    border: var(--jp-widgets-input-border-width) solid var(--jp-widgets-input-border-color);\n    background-color: var(--jp-widgets-input-background-color);\n    color: var(--jp-widgets-input-color);\n    font-size: var(--jp-widgets-font-size);\n    padding: var(--jp-widgets-input-padding) calc( var(--jp-widgets-input-padding) *  2 );\n    box-sizing: border-box;\n}\n\n.widget-datepicker input[type="date"]:focus {\n    border-color: var(--jp-widgets-input-focus-border-color);\n}\n\n.widget-datepicker input[type="date"]:invalid {\n    border-color: var(--jp-warn-color1);\n}\n\n.widget-datepicker input[type="date"]:disabled {\n    opacity: var(--jp-widgets-disabled-opacity);\n}\n\n/* Play Widget */\n\n.widget-play {\n    width: var(--jp-widgets-inline-width-short);\n    display: flex;\n    align-items: stretch;\n}\n\n.widget-play .jupyter-button {\n    flex-grow: 1;\n    height: auto;\n}\n\n.widget-play .jupyter-button:disabled {\n    opacity: var(--jp-widgets-disabled-opacity);\n}\n\n/* Tab Widget */\n\n.jupyter-widgets.widget-tab {\n    display: flex;\n    flex-direction: column;\n}\n\n.jupyter-widgets.widget-tab > .p-TabBar {\n    /* Necessary so that a tab can be shifted down to overlay the border of the box below. */\n    overflow-x: visible;\n    overflow-y: visible;\n}\n\n.jupyter-widgets.widget-tab > .p-TabBar > .p-TabBar-content {\n    /* Make sure that the tab grows from bottom up */\n    align-items: flex-end;\n    min-width: 0;\n    min-height: 0;\n}\n\n.jupyter-widgets.widget-tab > .widget-tab-contents {\n    width: 100%;\n    box-sizing: border-box;\n    margin: 0;\n    background: var(--jp-layout-color1);\n    color: var(--jp-ui-font-color1);\n    border: var(--jp-border-width) solid var(--jp-border-color1);\n    padding: var(--jp-widgets-container-padding);\n    flex-grow: 1;\n    overflow: auto;\n}\n\n.jupyter-widgets.widget-tab > .p-TabBar {\n    font: var(--jp-widgets-font-size) Helvetica, Arial, sans-serif;\n    min-height: calc(var(--jp-widgets-horizontal-tab-height) + var(--jp-border-width));\n}\n\n.jupyter-widgets.widget-tab > .p-TabBar .p-TabBar-tab {\n    flex: 0 1 var(--jp-widgets-horizontal-tab-width);\n    min-width: 35px;\n    min-height: calc(var(--jp-widgets-horizontal-tab-height) + var(--jp-border-width));\n    line-height: var(--jp-widgets-horizontal-tab-height);\n    margin-left: calc(-1 * var(--jp-border-width));\n    padding: 0px 10px;\n    background: var(--jp-layout-color2);\n    color: var(--jp-ui-font-color2);\n    border: var(--jp-border-width) solid var(--jp-border-color1);\n    border-bottom: none;\n    position: relative;\n}\n\n.jupyter-widgets.widget-tab > .p-TabBar .p-TabBar-tab.p-mod-current {\n    color: var(--jp-ui-font-color0);\n    /* We want the background to match the tab content background */\n    background: var(--jp-layout-color1);\n    min-height: calc(var(--jp-widgets-horizontal-tab-height) + 2 * var(--jp-border-width));\n    transform: translateY(var(--jp-border-width));\n    overflow: visible;\n}\n\n.jupyter-widgets.widget-tab > .p-TabBar .p-TabBar-tab.p-mod-current:before {\n    position: absolute;\n    top: calc(-1 * var(--jp-border-width));\n    left: calc(-1 * var(--jp-border-width));\n    content: \'\';\n    height: var(--jp-widgets-horizontal-tab-top-border);\n    width: calc(100% + 2 * var(--jp-border-width));\n    background: var(--jp-brand-color1);\n}\n\n.jupyter-widgets.widget-tab > .p-TabBar .p-TabBar-tab:first-child {\n    margin-left: 0;\n}\n\n.jupyter-widgets.widget-tab > .p-TabBar .p-TabBar-tab:hover:not(.p-mod-current) {\n    background: var(--jp-layout-color1);\n    color: var(--jp-ui-font-color1);\n}\n\n.jupyter-widgets.widget-tab > .p-TabBar .p-mod-closable > .p-TabBar-tabCloseIcon {\n    margin-left: 4px;\n}\n\n/* This font-awesome strategy may not work across FA4 and FA5, but we don\'t\nactually support closable tabs, so it really doesn\'t matter */\n.jupyter-widgets.widget-tab > .p-TabBar .p-mod-closable > .p-TabBar-tabCloseIcon:before {\n    font-family: FontAwesome;\n    content: \'\\f00d\'; /* close */\n}\n\n.jupyter-widgets.widget-tab > .p-TabBar .p-TabBar-tabIcon,\n.jupyter-widgets.widget-tab > .p-TabBar .p-TabBar-tabLabel,\n.jupyter-widgets.widget-tab > .p-TabBar .p-TabBar-tabCloseIcon {\n    line-height: var(--jp-widgets-horizontal-tab-height);\n}\n\n/* Accordion Widget */\n\n.p-Collapse {\n    display: flex;\n    flex-direction: column;\n    align-items: stretch;\n}\n\n.p-Collapse-header {\n    padding: var(--jp-widgets-input-padding);\n    cursor: pointer;\n    color: var(--jp-ui-font-color2);\n    background-color: var(--jp-layout-color2);\n    border: var(--jp-widgets-border-width) solid var(--jp-border-color1);\n    padding: calc(var(--jp-widgets-container-padding) * 2 / 3) var(--jp-widgets-container-padding);\n    font-weight: bold;\n}\n\n.p-Collapse-header:hover {\n    background-color: var(--jp-layout-color1);\n    color: var(--jp-ui-font-color1);\n}\n\n.p-Collapse-open > .p-Collapse-header {\n    background-color: var(--jp-layout-color1);\n    color: var(--jp-ui-font-color0);\n    cursor: default;\n    border-bottom: none;\n}\n\n.p-Collapse-contents {\n    padding: var(--jp-widgets-container-padding);\n    background-color: var(--jp-layout-color1);\n    color: var(--jp-ui-font-color1);\n    border-left: var(--jp-widgets-border-width) solid var(--jp-border-color1);\n    border-right: var(--jp-widgets-border-width) solid var(--jp-border-color1);\n    border-bottom: var(--jp-widgets-border-width) solid var(--jp-border-color1);\n    overflow: auto;\n}\n\n.p-Accordion {\n    display: flex;\n    flex-direction: column;\n    align-items: stretch;\n}\n\n.p-Accordion .p-Collapse {\n    margin-bottom: 0;\n}\n\n.p-Accordion .p-Collapse + .p-Collapse {\n    margin-top: 4px;\n}\n\n\n\n/* HTML widget */\n\n.widget-html, .widget-htmlmath {\n    font-size: var(--jp-widgets-font-size);\n}\n\n.widget-html > .widget-html-content, .widget-htmlmath > .widget-html-content {\n    /* Fill out the area in the HTML widget */\n    align-self: stretch;\n    flex-grow: 1;\n    flex-shrink: 1;\n    /* Makes sure the baseline is still aligned with other elements */\n    line-height: var(--jp-widgets-inline-height);\n    /* Make it possible to have absolutely-positioned elements in the html */\n    position: relative;\n}\n\n\n/* Image widget  */\n\n.widget-image {\n    max-width: 100%;\n    height: auto;\n}\n',""])},function(e,t,n){(e.exports=n(68)(!1)).push([e.i,"/* This file has code derived from PhosphorJS CSS files, as noted below. The license for this PhosphorJS code is:\n\nCopyright (c) 2014-2017, PhosphorJS Contributors\nAll rights reserved.\n\nRedistribution and use in source and binary forms, with or without\nmodification, are permitted provided that the following conditions are met:\n\n* Redistributions of source code must retain the above copyright notice, this\n  list of conditions and the following disclaimer.\n\n* Redistributions in binary form must reproduce the above copyright notice,\n  this list of conditions and the following disclaimer in the documentation\n  and/or other materials provided with the distribution.\n\n* Neither the name of the copyright holder nor the names of its\n  contributors may be used to endorse or promote products derived from\n  this software without specific prior written permission.\n\nTHIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS \"AS IS\"\nAND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE\nIMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE\nDISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE\nFOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL\nDAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR\nSERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER\nCAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,\nOR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE\nOF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.\n\n*/\n\n/*\n * The following section is derived from https://github.com/phosphorjs/phosphor/blob/23b9d075ebc5b73ab148b6ebfc20af97f85714c4/packages/widgets/style/tabbar.css \n * We've scoped the rules so that they are consistent with exactly our code.\n */\n\n.jupyter-widgets.widget-tab > .p-TabBar {\n  display: flex;\n  -webkit-user-select: none;\n  -moz-user-select: none;\n  -ms-user-select: none;\n  user-select: none;\n}\n\n\n.jupyter-widgets.widget-tab > .p-TabBar[data-orientation='horizontal'] {\n  flex-direction: row;\n}\n\n\n.jupyter-widgets.widget-tab > .p-TabBar[data-orientation='vertical'] {\n  flex-direction: column;\n}\n\n\n.jupyter-widgets.widget-tab > .p-TabBar > .p-TabBar-content {\n  margin: 0;\n  padding: 0;\n  display: flex;\n  flex: 1 1 auto;\n  list-style-type: none;\n}\n\n\n.jupyter-widgets.widget-tab > .p-TabBar[data-orientation='horizontal'] > .p-TabBar-content {\n  flex-direction: row;\n}\n\n\n.jupyter-widgets.widget-tab > .p-TabBar[data-orientation='vertical'] > .p-TabBar-content {\n  flex-direction: column;\n}\n\n\n.jupyter-widgets.widget-tab > .p-TabBar .p-TabBar-tab {\n  display: flex;\n  flex-direction: row;\n  box-sizing: border-box;\n  overflow: hidden;\n}\n\n\n.jupyter-widgets.widget-tab > .p-TabBar .p-TabBar-tabIcon,\n.jupyter-widgets.widget-tab > .p-TabBar .p-TabBar-tabCloseIcon {\n  flex: 0 0 auto;\n}\n\n\n.jupyter-widgets.widget-tab > .p-TabBar .p-TabBar-tabLabel {\n  flex: 1 1 auto;\n  overflow: hidden;\n  white-space: nowrap;\n}\n\n\n.jupyter-widgets.widget-tab > .p-TabBar .p-TabBar-tab.p-mod-hidden {\n  display: none !important;\n}\n\n\n.jupyter-widgets.widget-tab > .p-TabBar.p-mod-dragging .p-TabBar-tab {\n  position: relative;\n}\n\n\n.jupyter-widgets.widget-tab > .p-TabBar.p-mod-dragging[data-orientation='horizontal'] .p-TabBar-tab {\n  left: 0;\n  transition: left 150ms ease;\n}\n\n\n.jupyter-widgets.widget-tab > .p-TabBar.p-mod-dragging[data-orientation='vertical'] .p-TabBar-tab {\n  top: 0;\n  transition: top 150ms ease;\n}\n\n\n.jupyter-widgets.widget-tab > .p-TabBar.p-mod-dragging .p-TabBar-tab.p-mod-dragging {\n  transition: none;\n}\n\n/* End tabbar.css */\n",""])},function(e,t,n){"use strict";var r=n(1523),i=n(509),o=n(1527),a=n(892),s=n(893),l=n(1528),u=n(1529),c=n(1550),d=n(241);e.exports=m,m.prototype.validate=function(e,t){var n;if("string"==typeof e){if(!(n=this.getSchema(e)))throw new Error('no schema with key or ref "'+e+'"')}else{var r=this._addSchema(e);n=r.validate||this._compile(r)}var i=n(t);!0!==n.$async&&(this.errors=n.errors);return i},m.prototype.compile=function(e,t){var n=this._addSchema(e,void 0,t);return n.validate||this._compile(n)},m.prototype.addSchema=function(e,t,n,r){if(Array.isArray(e)){for(var o=0;o<e.length;o++)this.addSchema(e[o],void 0,n,r);return this}var a=this._getId(e);if(void 0!==a&&"string"!=typeof a)throw new Error("schema id must be string");return I(this,t=i.normalizeId(t||a)),this._schemas[t]=this._addSchema(e,n,r,!0),this},m.prototype.addMetaSchema=function(e,t,n){return this.addSchema(e,t,n,!0),this},m.prototype.validateSchema=function(e,t){var n=e.$schema;if(void 0!==n&&"string"!=typeof n)throw new Error("$schema must be a string");if(!(n=n||this._opts.defaultMeta||function(e){var t=e._opts.meta;return e._opts.defaultMeta="object"==typeof t?e._getId(t)||t:e.getSchema(p)?p:void 0,e._opts.defaultMeta}(this)))return this.logger.warn("meta-schema not available"),this.errors=null,!0;var r=this.validate(n,e);if(!r&&t){var i="schema is invalid: "+this.errorsText();if("log"!=this._opts.validateSchema)throw new Error(i);this.logger.error(i)}return r},m.prototype.getSchema=function(e){var t=y(this,e);switch(typeof t){case"object":return t.validate||this._compile(t);case"string":return this.getSchema(t);case"undefined":return function(e,t){var n=i.schema.call(e,{schema:{}},t);if(n){var o=n.schema,s=n.root,l=n.baseId,u=r.call(e,o,s,void 0,l);return e._fragments[t]=new a({ref:t,fragment:!0,schema:o,root:s,baseId:l,validate:u}),u}}(this,e)}},m.prototype.removeSchema=function(e){if(e instanceof RegExp)return b(this,this._schemas,e),b(this,this._refs,e),this;switch(typeof e){case"undefined":return b(this,this._schemas),b(this,this._refs),this._cache.clear(),this;case"string":var t=y(this,e);return t&&this._cache.del(t.cacheKey),delete this._schemas[e],delete this._refs[e],this;case"object":var n=this._opts.serialize,r=n?n(e):e;this._cache.del(r);var o=this._getId(e);o&&(o=i.normalizeId(o),delete this._schemas[o],delete this._refs[o])}return this},m.prototype.addFormat=function(e,t){"string"==typeof t&&(t=new RegExp(t));return this._formats[e]=t,this},m.prototype.errorsText=function(e,t){if(!(e=e||this.errors))return"No errors";for(var n=void 0===(t=t||{}).separator?", ":t.separator,r=void 0===t.dataVar?"data":t.dataVar,i="",o=0;o<e.length;o++){var a=e[o];a&&(i+=r+a.dataPath+" "+a.message+n)}return i.slice(0,-n.length)},m.prototype._addSchema=function(e,t,n,r){if("object"!=typeof e&&"boolean"!=typeof e)throw new Error("schema should be object or boolean");var o=this._opts.serialize,s=o?o(e):e,l=this._cache.get(s);if(l)return l;r=r||!1!==this._opts.addUsedSchema;var u=i.normalizeId(this._getId(e));u&&r&&I(this,u);var c,d=!1!==this._opts.validateSchema&&!t;d&&!(c=u&&u==i.normalizeId(e.$schema))&&this.validateSchema(e,!0);var h=i.ids.call(this,e),f=new a({id:u,schema:e,localRefs:h,cacheKey:s,meta:n});"#"!=u[0]&&r&&(this._refs[u]=f);this._cache.put(s,f),d&&c&&this.validateSchema(e,!0);return f},m.prototype._compile=function(e,t){if(e.compiling)return e.validate=o,o.schema=e.schema,o.errors=null,o.root=t||o,!0===e.schema.$async&&(o.$async=!0),o;var n,i;e.compiling=!0,e.meta&&(n=this._opts,this._opts=this._metaOpts);try{i=r.call(this,e.schema,t,e.localRefs)}catch(t){throw delete e.validate,t}finally{e.compiling=!1,e.meta&&(this._opts=n)}return e.validate=i,e.refs=i.refs,e.refVal=i.refVal,e.root=i.root,i;function o(){var t=e.validate,n=t.apply(this,arguments);return o.errors=t.errors,n}},m.prototype.compileAsync=n(1551);var h=n(1552);m.prototype.addKeyword=h.add,m.prototype.getKeyword=h.get,m.prototype.removeKeyword=h.remove,m.prototype.validateKeyword=h.validate;var f=n(511);m.ValidationError=f.Validation,m.MissingRefError=f.MissingRef,m.$dataMetaSchema=c;var p="http://json-schema.org/draft-07/schema",g=["removeAdditional","useDefaults","coerceTypes","strictDefaults"],v=["/properties"];function m(e){if(!(this instanceof m))return new m(e);e=this._opts=d.copy(e)||{},function(e){var t=e._opts.logger;if(!1===t)e.logger={log:x,warn:x,error:x};else{if(void 0===t&&(t=console),!("object"==typeof t&&t.log&&t.warn&&t.error))throw new Error("logger must implement log, warn and error methods");e.logger=t}}(this),this._schemas={},this._refs={},this._fragments={},this._formats=l(e.format),this._cache=e.cache||new o,this._loadingSchemas={},this._compilations=[],this.RULES=u(),this._getId=function(e){switch(e.schemaId){case"auto":return _;case"id":return C;default:return A}}(e),e.loopRequired=e.loopRequired||1/0,"property"==e.errorDataPath&&(e._errorDataPathProperty=!0),void 0===e.serialize&&(e.serialize=s),this._metaOpts=function(e){for(var t=d.copy(e._opts),n=0;n<g.length;n++)delete t[g[n]];return t}(this),e.formats&&function(e){for(var t in e._opts.formats){var n=e._opts.formats[t];e.addFormat(t,n)}}(this),e.keywords&&function(e){for(var t in e._opts.keywords){var n=e._opts.keywords[t];e.addKeyword(t,n)}}(this),function(e){var t;e._opts.$data&&(t=n(1555),e.addMetaSchema(t,t.$id,!0));if(!1===e._opts.meta)return;var r=n(899);e._opts.$data&&(r=c(r,v));e.addMetaSchema(r,p,!0),e._refs["http://json-schema.org/schema"]=p}(this),"object"==typeof e.meta&&this.addMetaSchema(e.meta),e.nullable&&this.addKeyword("nullable",{metaSchema:{type:"boolean"}}),function(e){var t=e._opts.schemas;if(!t)return;if(Array.isArray(t))e.addSchema(t);else for(var n in t)e.addSchema(t[n],n)}(this)}function y(e,t){return t=i.normalizeId(t),e._schemas[t]||e._refs[t]||e._fragments[t]}function b(e,t,n){for(var r in t){var i=t[r];i.meta||n&&!n.test(r)||(e._cache.del(i.cacheKey),delete t[r])}}function C(e){return e.$id&&this.logger.warn("schema $id ignored",e.$id),e.id}function A(e){return e.id&&this.logger.warn("schema id ignored",e.id),e.$id}function _(e){if(e.$id&&e.id&&e.$id!=e.id)throw new Error("schema $id is different from id");return e.$id||e.id}function I(e,t){if(e._schemas[t]||e._refs[t])throw new Error('schema with key or id "'+t+'" already exists')}function x(){}},function(e,t,n){"use strict";var r=n(509),i=n(241),o=n(511),a=n(893),s=n(894),l=i.ucs2length,u=n(510),c=o.Validation;function d(e,t,n){var r=f.call(this,e,t,n);return r>=0?{index:r,compiling:!0}:(r=this._compilations.length,this._compilations[r]={schema:e,root:t,baseId:n},{index:r,compiling:!1})}function h(e,t,n){var r=f.call(this,e,t,n);r>=0&&this._compilations.splice(r,1)}function f(e,t,n){for(var r=0;r<this._compilations.length;r++){var i=this._compilations[r];if(i.schema==e&&i.root==t&&i.baseId==n)return r}return-1}function p(e,t){return"var pattern"+e+" = new RegExp("+i.toQuotedString(t[e])+");"}function g(e){return"var default"+e+" = defaults["+e+"];"}function v(e,t){return void 0===t[e]?"":"var refVal"+e+" = refVal["+e+"];"}function m(e){return"var customRule"+e+" = customRules["+e+"];"}function y(e,t){if(!e.length)return"";for(var n="",r=0;r<e.length;r++)n+=t(r,e);return n}e.exports=function e(t,n,f,b){var C=this,A=this._opts,_=[void 0],I={},x=[],w={},M=[],S={},T=[];n=n||{schema:t,refVal:_,refs:I};var E=d.call(this,t,n,b),O=this._compilations[E.index];if(E.compiling)return O.callValidate=function e(){var t=O.validate,n=t.apply(this,arguments);return e.errors=t.errors,n};var k=this._formats,L=this.RULES;try{var D=N(t,n,f,b);O.validate=D;var P=O.callValidate;return P&&(P.schema=D.schema,P.errors=null,P.refs=D.refs,P.refVal=D.refVal,P.root=D.root,P.$async=D.$async,A.sourceCode&&(P.source=D.source)),D}finally{h.call(this,t,n,b)}function N(t,a,d,h){var f=!a||a&&a.schema==t;if(a.schema!=n.schema)return e.call(C,t,a,d,h);var b,w=!0===t.$async,S=s({isTop:!0,schema:t,isRoot:f,baseId:h,root:a,schemaPath:"",errSchemaPath:"#",errorPath:'""',MissingRefError:o.MissingRef,RULES:L,validate:s,util:i,resolve:r,resolveRef:j,usePattern:F,useDefault:B,useCustomRule:H,opts:A,formats:k,logger:C.logger,self:C});S=y(_,v)+y(x,p)+y(M,g)+y(T,m)+S,A.processCode&&(S=A.processCode(S,t));try{b=new Function("self","RULES","formats","root","refVal","defaults","customRules","equal","ucs2length","ValidationError",S)(C,L,k,n,_,M,T,u,l,c),_[0]=b}catch(e){throw C.logger.error("Error compiling schema, function code:",S),e}return b.schema=t,b.errors=null,b.refs=I,b.refVal=_,b.root=f?b:a,w&&(b.$async=!0),!0===A.sourceCode&&(b.source={code:S,patterns:x,defaults:M}),b}function j(t,i,o){i=r.url(t,i);var a,s,l=I[i];if(void 0!==l)return z(a=_[l],s="refVal["+l+"]");if(!o&&n.refs){var u=n.refs[i];if(void 0!==u)return z(a=n.refVal[u],s=R(i,a))}s=R(i);var c=r.call(C,N,n,i);if(void 0===c){var d=f&&f[i];d&&(c=r.inlineRef(d,A.inlineRefs)?d:e.call(C,d,n,f,t))}if(void 0!==c)return function(e,t){var n=I[e];_[n]=t}(i,c),z(c,s);!function(e){delete I[e]}(i)}function R(e,t){var n=_.length;return _[n]=t,I[e]=n,"refVal"+n}function z(e,t){return"object"==typeof e||"boolean"==typeof e?{code:t,schema:e,inline:!0}:{code:t,$async:e&&!!e.$async}}function F(e){var t=w[e];return void 0===t&&(t=w[e]=x.length,x[t]=e),"pattern"+t}function B(e){switch(typeof e){case"boolean":case"number":return""+e;case"string":return i.toQuotedString(e);case"object":if(null===e)return"null";var t=a(e),n=S[t];return void 0===n&&(n=S[t]=M.length,M[n]=e),"default"+n}}function H(e,t,n,r){if(!1!==C._opts.validateSchema){var i=e.definition.dependencies;if(i&&!i.every((function(e){return Object.prototype.hasOwnProperty.call(n,e)})))throw new Error("parent schema must have all required keywords: "+i.join(","));var o=e.definition.validateSchema;if(o)if(!o(t)){var a="keyword schema is invalid: "+C.errorsText(o.errors);if("log"!=C._opts.validateSchema)throw new Error(a);C.logger.error(a)}}var s,l=e.definition.compile,u=e.definition.inline,c=e.definition.macro;if(l)s=l.call(C,t,n,r);else if(c)s=c.call(C,t,n,r),!1!==A.validateSchema&&C.validateSchema(s,!0);else if(u)s=u.call(C,r,e.keyword,t,n);else if(!(s=e.definition.validate))return;if(void 0===s)throw new Error('custom keyword "'+e.keyword+'"failed to compile');var d=T.length;return T[d]=s,{code:"customRule"+d,validate:s}}}},function(e,t,n){
/** @license URI.js v4.4.1 (c) 2011 Gary Court. License: http://github.com/garycourt/uri-js */
!function(e){"use strict";function t(){for(var e=arguments.length,t=Array(e),n=0;n<e;n++)t[n]=arguments[n];if(t.length>1){t[0]=t[0].slice(0,-1);for(var r=t.length-1,i=1;i<r;++i)t[i]=t[i].slice(1,-1);return t[r]=t[r].slice(1),t.join("")}return t[0]}function n(e){return"(?:"+e+")"}function r(e){return void 0===e?"undefined":null===e?"null":Object.prototype.toString.call(e).split(" ").pop().split("]").shift().toLowerCase()}function i(e){return e.toUpperCase()}function o(e){var r=t("[0-9]","[A-Fa-f]"),i=n(n("%[EFef]"+r+"%"+r+r+"%"+r+r)+"|"+n("%[89A-Fa-f]"+r+"%"+r+r)+"|"+n("%"+r+r)),o="[\\!\\$\\&\\'\\(\\)\\*\\+\\,\\;\\=]",a=t("[\\:\\/\\?\\#\\[\\]\\@]",o),s=e?"[\\uE000-\\uF8FF]":"[]",l=t("[A-Za-z]","[0-9]","[\\-\\.\\_\\~]",e?"[\\xA0-\\u200D\\u2010-\\u2029\\u202F-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFEF]":"[]"),u=n("[A-Za-z]"+t("[A-Za-z]","[0-9]","[\\+\\-\\.]")+"*"),c=n(n(i+"|"+t(l,o,"[\\:]"))+"*"),d=(n(n("25[0-5]")+"|"+n("2[0-4][0-9]")+"|"+n("1[0-9][0-9]")+"|"+n("[1-9][0-9]")+"|[0-9]"),n(n("25[0-5]")+"|"+n("2[0-4][0-9]")+"|"+n("1[0-9][0-9]")+"|"+n("0?[1-9][0-9]")+"|0?0?[0-9]")),h=n(d+"\\."+d+"\\."+d+"\\."+d),f=n(r+"{1,4}"),p=n(n(f+"\\:"+f)+"|"+h),g=n(n(f+"\\:")+"{6}"+p),v=n("\\:\\:"+n(f+"\\:")+"{5}"+p),m=n(n(f)+"?\\:\\:"+n(f+"\\:")+"{4}"+p),y=n(n(n(f+"\\:")+"{0,1}"+f)+"?\\:\\:"+n(f+"\\:")+"{3}"+p),b=n(n(n(f+"\\:")+"{0,2}"+f)+"?\\:\\:"+n(f+"\\:")+"{2}"+p),C=n(n(n(f+"\\:")+"{0,3}"+f)+"?\\:\\:"+f+"\\:"+p),A=n(n(n(f+"\\:")+"{0,4}"+f)+"?\\:\\:"+p),_=n(n(n(f+"\\:")+"{0,5}"+f)+"?\\:\\:"+f),I=n(n(n(f+"\\:")+"{0,6}"+f)+"?\\:\\:"),x=n([g,v,m,y,b,C,A,_,I].join("|")),w=n(n(l+"|"+i)+"+"),M=(n(x+"\\%25"+w),n(x+n("\\%25|\\%(?!"+r+"{2})")+w)),S=n("[vV]"+r+"+\\."+t(l,o,"[\\:]")+"+"),T=n("\\["+n(M+"|"+x+"|"+S)+"\\]"),E=n(n(i+"|"+t(l,o))+"*"),O=n(T+"|"+h+"(?!"+E+")|"+E),k=n("[0-9]*"),L=n(n(c+"@")+"?"+O+n("\\:"+k)+"?"),D=n(i+"|"+t(l,o,"[\\:\\@]")),P=n(D+"*"),N=n(D+"+"),j=n(n(i+"|"+t(l,o,"[\\@]"))+"+"),R=n(n("\\/"+P)+"*"),z=n("\\/"+n(N+R)+"?"),F=n(j+R),B=n(N+R),H="(?!"+D+")",U=(n(R+"|"+z+"|"+F+"|"+B+"|"+H),n(n(D+"|"+t("[\\/\\?]",s))+"*")),V=n(n(D+"|[\\/\\?]")+"*"),Y=n(n("\\/\\/"+L+R)+"|"+z+"|"+B+"|"+H),W=n(u+"\\:"+Y+n("\\?"+U)+"?"+n("\\#"+V)+"?"),G=n(n("\\/\\/"+L+R)+"|"+z+"|"+F+"|"+H),X=n(G+n("\\?"+U)+"?"+n("\\#"+V)+"?");return n(W+"|"+X),n(u+"\\:"+Y+n("\\?"+U)+"?"),n(n("\\/\\/("+n("("+c+")@")+"?("+O+")"+n("\\:("+k+")")+"?)")+"?("+R+"|"+z+"|"+B+"|"+H+")"),n("\\?("+U+")"),n("\\#("+V+")"),n(n("\\/\\/("+n("("+c+")@")+"?("+O+")"+n("\\:("+k+")")+"?)")+"?("+R+"|"+z+"|"+F+"|"+H+")"),n("\\?("+U+")"),n("\\#("+V+")"),n(n("\\/\\/("+n("("+c+")@")+"?("+O+")"+n("\\:("+k+")")+"?)")+"?("+R+"|"+z+"|"+B+"|"+H+")"),n("\\?("+U+")"),n("\\#("+V+")"),n("("+c+")@"),n("\\:("+k+")"),{NOT_SCHEME:new RegExp(t("[^]","[A-Za-z]","[0-9]","[\\+\\-\\.]"),"g"),NOT_USERINFO:new RegExp(t("[^\\%\\:]",l,o),"g"),NOT_HOST:new RegExp(t("[^\\%\\[\\]\\:]",l,o),"g"),NOT_PATH:new RegExp(t("[^\\%\\/\\:\\@]",l,o),"g"),NOT_PATH_NOSCHEME:new RegExp(t("[^\\%\\/\\@]",l,o),"g"),NOT_QUERY:new RegExp(t("[^\\%]",l,o,"[\\:\\@\\/\\?]",s),"g"),NOT_FRAGMENT:new RegExp(t("[^\\%]",l,o,"[\\:\\@\\/\\?]"),"g"),ESCAPE:new RegExp(t("[^]",l,o),"g"),UNRESERVED:new RegExp(l,"g"),OTHER_CHARS:new RegExp(t("[^\\%]",l,a),"g"),PCT_ENCODED:new RegExp(i,"g"),IPV4ADDRESS:new RegExp("^("+h+")$"),IPV6ADDRESS:new RegExp("^\\[?("+x+")"+n(n("\\%25|\\%(?!"+r+"{2})")+"("+w+")")+"?\\]?$")}}var a=o(!1),s=o(!0),l=function(e,t){if(Array.isArray(e))return e;if(Symbol.iterator in Object(e))return function(e,t){var n=[],r=!0,i=!1,o=void 0;try{for(var a,s=e[Symbol.iterator]();!(r=(a=s.next()).done)&&(n.push(a.value),!t||n.length!==t);r=!0);}catch(e){i=!0,o=e}finally{try{!r&&s.return&&s.return()}finally{if(i)throw o}}return n}(e,t);throw new TypeError("Invalid attempt to destructure non-iterable instance")},u=2147483647,c=/^xn--/,d=/[^\0-\x7E]/,h=/[\x2E\u3002\uFF0E\uFF61]/g,f={overflow:"Overflow: input needs wider integers to process","not-basic":"Illegal input >= 0x80 (not a basic code point)","invalid-input":"Invalid input"},p=Math.floor,g=String.fromCharCode;function v(e){throw new RangeError(f[e])}function m(e,t){var n=e.split("@"),r="";n.length>1&&(r=n[0]+"@",e=n[1]);var i=function(e,t){for(var n=[],r=e.length;r--;)n[r]=t(e[r]);return n}((e=e.replace(h,".")).split("."),t).join(".");return r+i}function y(e){for(var t=[],n=0,r=e.length;n<r;){var i=e.charCodeAt(n++);if(i>=55296&&i<=56319&&n<r){var o=e.charCodeAt(n++);56320==(64512&o)?t.push(((1023&i)<<10)+(1023&o)+65536):(t.push(i),n--)}else t.push(i)}return t}var b=function(e,t){return e+22+75*(e<26)-((0!=t)<<5)},C=function(e,t,n){var r=0;for(e=n?p(e/700):e>>1,e+=p(e/t);e>455;r+=36)e=p(e/35);return p(r+36*e/(e+38))},A=function(e){var t,n=[],r=e.length,i=0,o=128,a=72,s=e.lastIndexOf("-");s<0&&(s=0);for(var l=0;l<s;++l)e.charCodeAt(l)>=128&&v("not-basic"),n.push(e.charCodeAt(l));for(var c=s>0?s+1:0;c<r;){for(var d=i,h=1,f=36;;f+=36){c>=r&&v("invalid-input");var g=(t=e.charCodeAt(c++))-48<10?t-22:t-65<26?t-65:t-97<26?t-97:36;(g>=36||g>p((u-i)/h))&&v("overflow"),i+=g*h;var m=f<=a?1:f>=a+26?26:f-a;if(g<m)break;var y=36-m;h>p(u/y)&&v("overflow"),h*=y}var b=n.length+1;a=C(i-d,b,0==d),p(i/b)>u-o&&v("overflow"),o+=p(i/b),i%=b,n.splice(i++,0,o)}return String.fromCodePoint.apply(String,n)},_=function(e){var t=[],n=(e=y(e)).length,r=128,i=0,o=72,a=!0,s=!1,l=void 0;try{for(var c,d=e[Symbol.iterator]();!(a=(c=d.next()).done);a=!0){var h=c.value;h<128&&t.push(g(h))}}catch(e){s=!0,l=e}finally{try{!a&&d.return&&d.return()}finally{if(s)throw l}}var f=t.length,m=f;for(f&&t.push("-");m<n;){var A=u,_=!0,I=!1,x=void 0;try{for(var w,M=e[Symbol.iterator]();!(_=(w=M.next()).done);_=!0){var S=w.value;S>=r&&S<A&&(A=S)}}catch(e){I=!0,x=e}finally{try{!_&&M.return&&M.return()}finally{if(I)throw x}}var T=m+1;A-r>p((u-i)/T)&&v("overflow"),i+=(A-r)*T,r=A;var E=!0,O=!1,k=void 0;try{for(var L,D=e[Symbol.iterator]();!(E=(L=D.next()).done);E=!0){var P=L.value;if(P<r&&++i>u&&v("overflow"),P==r){for(var N=i,j=36;;j+=36){var R=j<=o?1:j>=o+26?26:j-o;if(N<R)break;var z=N-R,F=36-R;t.push(g(b(R+z%F,0))),N=p(z/F)}t.push(g(b(N,0))),o=C(i,T,m==f),i=0,++m}}}catch(e){O=!0,k=e}finally{try{!E&&D.return&&D.return()}finally{if(O)throw k}}++i,++r}return t.join("")},I=function(e){return m(e,(function(e){return d.test(e)?"xn--"+_(e):e}))},x=function(e){return m(e,(function(e){return c.test(e)?A(e.slice(4).toLowerCase()):e}))},w={};function M(e){var t=e.charCodeAt(0);return t<16?"%0"+t.toString(16).toUpperCase():t<128?"%"+t.toString(16).toUpperCase():t<2048?"%"+(t>>6|192).toString(16).toUpperCase()+"%"+(63&t|128).toString(16).toUpperCase():"%"+(t>>12|224).toString(16).toUpperCase()+"%"+(t>>6&63|128).toString(16).toUpperCase()+"%"+(63&t|128).toString(16).toUpperCase()}function S(e){for(var t="",n=0,r=e.length;n<r;){var i=parseInt(e.substr(n+1,2),16);if(i<128)t+=String.fromCharCode(i),n+=3;else if(i>=194&&i<224){if(r-n>=6){var o=parseInt(e.substr(n+4,2),16);t+=String.fromCharCode((31&i)<<6|63&o)}else t+=e.substr(n,6);n+=6}else if(i>=224){if(r-n>=9){var a=parseInt(e.substr(n+4,2),16),s=parseInt(e.substr(n+7,2),16);t+=String.fromCharCode((15&i)<<12|(63&a)<<6|63&s)}else t+=e.substr(n,9);n+=9}else t+=e.substr(n,3),n+=3}return t}function T(e,t){function n(e){var n=S(e);return n.match(t.UNRESERVED)?n:e}return e.scheme&&(e.scheme=String(e.scheme).replace(t.PCT_ENCODED,n).toLowerCase().replace(t.NOT_SCHEME,"")),void 0!==e.userinfo&&(e.userinfo=String(e.userinfo).replace(t.PCT_ENCODED,n).replace(t.NOT_USERINFO,M).replace(t.PCT_ENCODED,i)),void 0!==e.host&&(e.host=String(e.host).replace(t.PCT_ENCODED,n).toLowerCase().replace(t.NOT_HOST,M).replace(t.PCT_ENCODED,i)),void 0!==e.path&&(e.path=String(e.path).replace(t.PCT_ENCODED,n).replace(e.scheme?t.NOT_PATH:t.NOT_PATH_NOSCHEME,M).replace(t.PCT_ENCODED,i)),void 0!==e.query&&(e.query=String(e.query).replace(t.PCT_ENCODED,n).replace(t.NOT_QUERY,M).replace(t.PCT_ENCODED,i)),void 0!==e.fragment&&(e.fragment=String(e.fragment).replace(t.PCT_ENCODED,n).replace(t.NOT_FRAGMENT,M).replace(t.PCT_ENCODED,i)),e}function E(e){return e.replace(/^0*(.*)/,"$1")||"0"}function O(e,t){var n=e.match(t.IPV4ADDRESS)||[],r=l(n,2)[1];return r?r.split(".").map(E).join("."):e}function k(e,t){var n=e.match(t.IPV6ADDRESS)||[],r=l(n,3),i=r[1],o=r[2];if(i){for(var a=i.toLowerCase().split("::").reverse(),s=l(a,2),u=s[0],c=s[1],d=c?c.split(":").map(E):[],h=u.split(":").map(E),f=t.IPV4ADDRESS.test(h[h.length-1]),p=f?7:8,g=h.length-p,v=Array(p),m=0;m<p;++m)v[m]=d[m]||h[g+m]||"";f&&(v[p-1]=O(v[p-1],t));var y=v.reduce((function(e,t,n){if(!t||"0"===t){var r=e[e.length-1];r&&r.index+r.length===n?r.length++:e.push({index:n,length:1})}return e}),[]).sort((function(e,t){return t.length-e.length}))[0],b=void 0;if(y&&y.length>1){var C=v.slice(0,y.index),A=v.slice(y.index+y.length);b=C.join(":")+"::"+A.join(":")}else b=v.join(":");return o&&(b+="%"+o),b}return e}var L=/^(?:([^:\/?#]+):)?(?:\/\/((?:([^\/?#@]*)@)?(\[[^\/?#\]]+\]|[^\/?#:]*)(?:\:(\d*))?))?([^?#]*)(?:\?([^#]*))?(?:#((?:.|\n|\r)*))?/i,D=void 0==="".match(/(){0}/)[1];function P(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n={},r=!1!==t.iri?s:a;"suffix"===t.reference&&(e=(t.scheme?t.scheme+":":"")+"//"+e);var i=e.match(L);if(i){D?(n.scheme=i[1],n.userinfo=i[3],n.host=i[4],n.port=parseInt(i[5],10),n.path=i[6]||"",n.query=i[7],n.fragment=i[8],isNaN(n.port)&&(n.port=i[5])):(n.scheme=i[1]||void 0,n.userinfo=-1!==e.indexOf("@")?i[3]:void 0,n.host=-1!==e.indexOf("//")?i[4]:void 0,n.port=parseInt(i[5],10),n.path=i[6]||"",n.query=-1!==e.indexOf("?")?i[7]:void 0,n.fragment=-1!==e.indexOf("#")?i[8]:void 0,isNaN(n.port)&&(n.port=e.match(/\/\/(?:.|\n)*\:(?:\/|\?|\#|$)/)?i[4]:void 0)),n.host&&(n.host=k(O(n.host,r),r)),void 0!==n.scheme||void 0!==n.userinfo||void 0!==n.host||void 0!==n.port||n.path||void 0!==n.query?void 0===n.scheme?n.reference="relative":void 0===n.fragment?n.reference="absolute":n.reference="uri":n.reference="same-document",t.reference&&"suffix"!==t.reference&&t.reference!==n.reference&&(n.error=n.error||"URI is not a "+t.reference+" reference.");var o=w[(t.scheme||n.scheme||"").toLowerCase()];if(t.unicodeSupport||o&&o.unicodeSupport)T(n,r);else{if(n.host&&(t.domainHost||o&&o.domainHost))try{n.host=I(n.host.replace(r.PCT_ENCODED,S).toLowerCase())}catch(e){n.error=n.error||"Host's domain name can not be converted to ASCII via punycode: "+e}T(n,a)}o&&o.parse&&o.parse(n,t)}else n.error=n.error||"URI can not be parsed.";return n}function N(e,t){var n=!1!==t.iri?s:a,r=[];return void 0!==e.userinfo&&(r.push(e.userinfo),r.push("@")),void 0!==e.host&&r.push(k(O(String(e.host),n),n).replace(n.IPV6ADDRESS,(function(e,t,n){return"["+t+(n?"%25"+n:"")+"]"}))),"number"!=typeof e.port&&"string"!=typeof e.port||(r.push(":"),r.push(String(e.port))),r.length?r.join(""):void 0}var j=/^\.\.?\//,R=/^\/\.(\/|$)/,z=/^\/\.\.(\/|$)/,F=/^\/?(?:.|\n)*?(?=\/|$)/;function B(e){for(var t=[];e.length;)if(e.match(j))e=e.replace(j,"");else if(e.match(R))e=e.replace(R,"/");else if(e.match(z))e=e.replace(z,"/"),t.pop();else if("."===e||".."===e)e="";else{var n=e.match(F);if(!n)throw new Error("Unexpected dot segment condition");var r=n[0];e=e.slice(r.length),t.push(r)}return t.join("")}function H(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=t.iri?s:a,r=[],i=w[(t.scheme||e.scheme||"").toLowerCase()];if(i&&i.serialize&&i.serialize(e,t),e.host)if(n.IPV6ADDRESS.test(e.host));else if(t.domainHost||i&&i.domainHost)try{e.host=t.iri?x(e.host):I(e.host.replace(n.PCT_ENCODED,S).toLowerCase())}catch(n){e.error=e.error||"Host's domain name can not be converted to "+(t.iri?"Unicode":"ASCII")+" via punycode: "+n}T(e,n),"suffix"!==t.reference&&e.scheme&&(r.push(e.scheme),r.push(":"));var o=N(e,t);if(void 0!==o&&("suffix"!==t.reference&&r.push("//"),r.push(o),e.path&&"/"!==e.path.charAt(0)&&r.push("/")),void 0!==e.path){var l=e.path;t.absolutePath||i&&i.absolutePath||(l=B(l)),void 0===o&&(l=l.replace(/^\/\//,"/%2F")),r.push(l)}return void 0!==e.query&&(r.push("?"),r.push(e.query)),void 0!==e.fragment&&(r.push("#"),r.push(e.fragment)),r.join("")}function U(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},r=arguments[3],i={};return r||(e=P(H(e,n),n),t=P(H(t,n),n)),!(n=n||{}).tolerant&&t.scheme?(i.scheme=t.scheme,i.userinfo=t.userinfo,i.host=t.host,i.port=t.port,i.path=B(t.path||""),i.query=t.query):(void 0!==t.userinfo||void 0!==t.host||void 0!==t.port?(i.userinfo=t.userinfo,i.host=t.host,i.port=t.port,i.path=B(t.path||""),i.query=t.query):(t.path?("/"===t.path.charAt(0)?i.path=B(t.path):(void 0===e.userinfo&&void 0===e.host&&void 0===e.port||e.path?e.path?i.path=e.path.slice(0,e.path.lastIndexOf("/")+1)+t.path:i.path=t.path:i.path="/"+t.path,i.path=B(i.path)),i.query=t.query):(i.path=e.path,void 0!==t.query?i.query=t.query:i.query=e.query),i.userinfo=e.userinfo,i.host=e.host,i.port=e.port),i.scheme=e.scheme),i.fragment=t.fragment,i}function V(e,t){return e&&e.toString().replace(t&&t.iri?s.PCT_ENCODED:a.PCT_ENCODED,S)}var Y={scheme:"http",domainHost:!0,parse:function(e,t){return e.host||(e.error=e.error||"HTTP URIs must have a host."),e},serialize:function(e,t){var n="https"===String(e.scheme).toLowerCase();return e.port!==(n?443:80)&&""!==e.port||(e.port=void 0),e.path||(e.path="/"),e}},W={scheme:"https",domainHost:Y.domainHost,parse:Y.parse,serialize:Y.serialize};function G(e){return"boolean"==typeof e.secure?e.secure:"wss"===String(e.scheme).toLowerCase()}var X={scheme:"ws",domainHost:!0,parse:function(e,t){var n=e;return n.secure=G(n),n.resourceName=(n.path||"/")+(n.query?"?"+n.query:""),n.path=void 0,n.query=void 0,n},serialize:function(e,t){if(e.port!==(G(e)?443:80)&&""!==e.port||(e.port=void 0),"boolean"==typeof e.secure&&(e.scheme=e.secure?"wss":"ws",e.secure=void 0),e.resourceName){var n=e.resourceName.split("?"),r=l(n,2),i=r[0],o=r[1];e.path=i&&"/"!==i?i:void 0,e.query=o,e.resourceName=void 0}return e.fragment=void 0,e}},q={scheme:"wss",domainHost:X.domainHost,parse:X.parse,serialize:X.serialize},Z={},Q="[A-Za-z0-9\\-\\.\\_\\~\\xA0-\\u200D\\u2010-\\u2029\\u202F-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFEF]",K="[0-9A-Fa-f]",J=n(n("%[EFef]"+K+"%"+K+K+"%"+K+K)+"|"+n("%[89A-Fa-f]"+K+"%"+K+K)+"|"+n("%"+K+K)),$=t("[\\!\\$\\%\\'\\(\\)\\*\\+\\,\\-\\.0-9\\<\\>A-Z\\x5E-\\x7E]",'[\\"\\\\]'),ee=new RegExp(Q,"g"),te=new RegExp(J,"g"),ne=new RegExp(t("[^]","[A-Za-z0-9\\!\\$\\%\\'\\*\\+\\-\\^\\_\\`\\{\\|\\}\\~]","[\\.]",'[\\"]',$),"g"),re=new RegExp(t("[^]",Q,"[\\!\\$\\'\\(\\)\\*\\+\\,\\;\\:\\@]"),"g"),ie=re;function oe(e){var t=S(e);return t.match(ee)?t:e}var ae={scheme:"mailto",parse:function(e,t){var n=e,r=n.to=n.path?n.path.split(","):[];if(n.path=void 0,n.query){for(var i=!1,o={},a=n.query.split("&"),s=0,l=a.length;s<l;++s){var u=a[s].split("=");switch(u[0]){case"to":for(var c=u[1].split(","),d=0,h=c.length;d<h;++d)r.push(c[d]);break;case"subject":n.subject=V(u[1],t);break;case"body":n.body=V(u[1],t);break;default:i=!0,o[V(u[0],t)]=V(u[1],t)}}i&&(n.headers=o)}n.query=void 0;for(var f=0,p=r.length;f<p;++f){var g=r[f].split("@");if(g[0]=V(g[0]),t.unicodeSupport)g[1]=V(g[1],t).toLowerCase();else try{g[1]=I(V(g[1],t).toLowerCase())}catch(e){n.error=n.error||"Email address's domain name can not be converted to ASCII via punycode: "+e}r[f]=g.join("@")}return n},serialize:function(e,t){var n,r=e,o=null!=(n=e.to)?n instanceof Array?n:"number"!=typeof n.length||n.split||n.setInterval||n.call?[n]:Array.prototype.slice.call(n):[];if(o){for(var a=0,s=o.length;a<s;++a){var l=String(o[a]),u=l.lastIndexOf("@"),c=l.slice(0,u).replace(te,oe).replace(te,i).replace(ne,M),d=l.slice(u+1);try{d=t.iri?x(d):I(V(d,t).toLowerCase())}catch(e){r.error=r.error||"Email address's domain name can not be converted to "+(t.iri?"Unicode":"ASCII")+" via punycode: "+e}o[a]=c+"@"+d}r.path=o.join(",")}var h=e.headers=e.headers||{};e.subject&&(h.subject=e.subject),e.body&&(h.body=e.body);var f=[];for(var p in h)h[p]!==Z[p]&&f.push(p.replace(te,oe).replace(te,i).replace(re,M)+"="+h[p].replace(te,oe).replace(te,i).replace(ie,M));return f.length&&(r.query=f.join("&")),r}},se=/^([^\:]+)\:(.*)/,le={scheme:"urn",parse:function(e,t){var n=e.path&&e.path.match(se),r=e;if(n){var i=t.scheme||r.scheme||"urn",o=n[1].toLowerCase(),a=n[2],s=i+":"+(t.nid||o),l=w[s];r.nid=o,r.nss=a,r.path=void 0,l&&(r=l.parse(r,t))}else r.error=r.error||"URN can not be parsed.";return r},serialize:function(e,t){var n=t.scheme||e.scheme||"urn",r=e.nid,i=n+":"+(t.nid||r),o=w[i];o&&(e=o.serialize(e,t));var a=e,s=e.nss;return a.path=(r||t.nid)+":"+s,a}},ue=/^[0-9A-Fa-f]{8}(?:\-[0-9A-Fa-f]{4}){3}\-[0-9A-Fa-f]{12}$/,ce={scheme:"urn:uuid",parse:function(e,t){var n=e;return n.uuid=n.nss,n.nss=void 0,t.tolerant||n.uuid&&n.uuid.match(ue)||(n.error=n.error||"UUID is not valid."),n},serialize:function(e,t){var n=e;return n.nss=(e.uuid||"").toLowerCase(),n}};w[Y.scheme]=Y,w[W.scheme]=W,w[X.scheme]=X,w[q.scheme]=q,w[ae.scheme]=ae,w[le.scheme]=le,w[ce.scheme]=ce,e.SCHEMES=w,e.pctEncChar=M,e.pctDecChars=S,e.parse=P,e.removeDotSegments=B,e.serialize=H,e.resolveComponents=U,e.resolve=function(e,t,n){var r=function(e,t){var n=e;if(t)for(var r in t)n[r]=t[r];return n}({scheme:"null"},n);return H(U(P(e,r),P(t,r),r,!0),r)},e.normalize=function(e,t){return"string"==typeof e?e=H(P(e,t),t):"object"===r(e)&&(e=P(H(e,t),t)),e},e.equal=function(e,t,n){return"string"==typeof e?e=H(P(e,n),n):"object"===r(e)&&(e=H(e,n)),"string"==typeof t?t=H(P(t,n),n):"object"===r(t)&&(t=H(t,n)),e===t},e.escapeComponent=function(e,t){return e&&e.toString().replace(t&&t.iri?s.ESCAPE:a.ESCAPE,M)},e.unescapeComponent=V,Object.defineProperty(e,"__esModule",{value:!0})}(t)},function(e,t,n){"use strict";e.exports=function(e){for(var t,n=0,r=e.length,i=0;i<r;)n++,(t=e.charCodeAt(i++))>=55296&&t<=56319&&i<r&&56320==(64512&(t=e.charCodeAt(i)))&&i++;return n}},function(e,t,n){"use strict";var r=e.exports=function(e,t,n){"function"==typeof t&&(n=t,t={}),function e(t,n,i,o,a,s,l,u,c,d){if(o&&"object"==typeof o&&!Array.isArray(o)){for(var h in n(o,a,s,l,u,c,d),o){var f=o[h];if(Array.isArray(f)){if(h in r.arrayKeywords)for(var p=0;p<f.length;p++)e(t,n,i,f[p],a+"/"+h+"/"+p,s,a,h,o,p)}else if(h in r.propsKeywords){if(f&&"object"==typeof f)for(var g in f)e(t,n,i,f[g],a+"/"+h+"/"+g.replace(/~/g,"~0").replace(/\//g,"~1"),s,a,h,o,g)}else(h in r.keywords||t.allKeys&&!(h in r.skipKeywords))&&e(t,n,i,f,a+"/"+h,s,a,h,o)}i(o,a,s,l,u,c,d)}}(t,"function"==typeof(n=t.cb||n)?n:n.pre||function(){},n.post||function(){},e,"",e)};r.keywords={additionalItems:!0,items:!0,contains:!0,additionalProperties:!0,propertyNames:!0,not:!0},r.arrayKeywords={items:!0,allOf:!0,anyOf:!0,oneOf:!0},r.propsKeywords={definitions:!0,properties:!0,patternProperties:!0,dependencies:!0},r.skipKeywords={default:!0,enum:!0,const:!0,required:!0,maximum:!0,minimum:!0,exclusiveMaximum:!0,exclusiveMinimum:!0,multipleOf:!0,maxLength:!0,minLength:!0,pattern:!0,format:!0,maxItems:!0,minItems:!0,uniqueItems:!0,maxProperties:!0,minProperties:!0}},function(e,t,n){"use strict";var r=e.exports=function(){this._cache={}};r.prototype.put=function(e,t){this._cache[e]=t},r.prototype.get=function(e){return this._cache[e]},r.prototype.del=function(e){delete this._cache[e]},r.prototype.clear=function(){this._cache={}}},function(e,t,n){"use strict";var r=n(241),i=/^(\d\d\d\d)-(\d\d)-(\d\d)$/,o=[0,31,28,31,30,31,30,31,31,30,31,30,31],a=/^(\d\d):(\d\d):(\d\d)(\.\d+)?(z|[+-]\d\d(?::?\d\d)?)?$/i,s=/^(?=.{1,253}\.?$)[a-z0-9](?:[a-z0-9-]{0,61}[a-z0-9])?(?:\.[a-z0-9](?:[-0-9a-z]{0,61}[0-9a-z])?)*\.?$/i,l=/^(?:[a-z][a-z0-9+\-.]*:)(?:\/?\/(?:(?:[a-z0-9\-._~!$&'()*+,;=:]|%[0-9a-f]{2})*@)?(?:\[(?:(?:(?:(?:[0-9a-f]{1,4}:){6}|::(?:[0-9a-f]{1,4}:){5}|(?:[0-9a-f]{1,4})?::(?:[0-9a-f]{1,4}:){4}|(?:(?:[0-9a-f]{1,4}:){0,1}[0-9a-f]{1,4})?::(?:[0-9a-f]{1,4}:){3}|(?:(?:[0-9a-f]{1,4}:){0,2}[0-9a-f]{1,4})?::(?:[0-9a-f]{1,4}:){2}|(?:(?:[0-9a-f]{1,4}:){0,3}[0-9a-f]{1,4})?::[0-9a-f]{1,4}:|(?:(?:[0-9a-f]{1,4}:){0,4}[0-9a-f]{1,4})?::)(?:[0-9a-f]{1,4}:[0-9a-f]{1,4}|(?:(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(?:25[0-5]|2[0-4]\d|[01]?\d\d?))|(?:(?:[0-9a-f]{1,4}:){0,5}[0-9a-f]{1,4})?::[0-9a-f]{1,4}|(?:(?:[0-9a-f]{1,4}:){0,6}[0-9a-f]{1,4})?::)|[Vv][0-9a-f]+\.[a-z0-9\-._~!$&'()*+,;=:]+)\]|(?:(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(?:25[0-5]|2[0-4]\d|[01]?\d\d?)|(?:[a-z0-9\-._~!$&'()*+,;=]|%[0-9a-f]{2})*)(?::\d*)?(?:\/(?:[a-z0-9\-._~!$&'()*+,;=:@]|%[0-9a-f]{2})*)*|\/(?:(?:[a-z0-9\-._~!$&'()*+,;=:@]|%[0-9a-f]{2})+(?:\/(?:[a-z0-9\-._~!$&'()*+,;=:@]|%[0-9a-f]{2})*)*)?|(?:[a-z0-9\-._~!$&'()*+,;=:@]|%[0-9a-f]{2})+(?:\/(?:[a-z0-9\-._~!$&'()*+,;=:@]|%[0-9a-f]{2})*)*)(?:\?(?:[a-z0-9\-._~!$&'()*+,;=:@/?]|%[0-9a-f]{2})*)?(?:#(?:[a-z0-9\-._~!$&'()*+,;=:@/?]|%[0-9a-f]{2})*)?$/i,u=/^(?:(?:[^\x00-\x20"'<>%\\^`{|}]|%[0-9a-f]{2})|\{[+#./;?&=,!@|]?(?:[a-z0-9_]|%[0-9a-f]{2})+(?::[1-9][0-9]{0,3}|\*)?(?:,(?:[a-z0-9_]|%[0-9a-f]{2})+(?::[1-9][0-9]{0,3}|\*)?)*\})*$/i,c=/^(?:(?:http[s\u017F]?|ftp):\/\/)(?:(?:[\0-\x08\x0E-\x1F!-\x9F\xA1-\u167F\u1681-\u1FFF\u200B-\u2027\u202A-\u202E\u2030-\u205E\u2060-\u2FFF\u3001-\uD7FF\uE000-\uFEFE\uFF00-\uFFFF]|[\uD800-\uDBFF][\uDC00-\uDFFF]|[\uD800-\uDBFF](?![\uDC00-\uDFFF])|(?:[^\uD800-\uDBFF]|^)[\uDC00-\uDFFF])+(?::(?:[\0-\x08\x0E-\x1F!-\x9F\xA1-\u167F\u1681-\u1FFF\u200B-\u2027\u202A-\u202E\u2030-\u205E\u2060-\u2FFF\u3001-\uD7FF\uE000-\uFEFE\uFF00-\uFFFF]|[\uD800-\uDBFF][\uDC00-\uDFFF]|[\uD800-\uDBFF](?![\uDC00-\uDFFF])|(?:[^\uD800-\uDBFF]|^)[\uDC00-\uDFFF])*)?@)?(?:(?!10(?:\.[0-9]{1,3}){3})(?!127(?:\.[0-9]{1,3}){3})(?!169\.254(?:\.[0-9]{1,3}){2})(?!192\.168(?:\.[0-9]{1,3}){2})(?!172\.(?:1[6-9]|2[0-9]|3[01])(?:\.[0-9]{1,3}){2})(?:[1-9][0-9]?|1[0-9][0-9]|2[01][0-9]|22[0-3])(?:\.(?:1?[0-9]{1,2}|2[0-4][0-9]|25[0-5])){2}(?:\.(?:[1-9][0-9]?|1[0-9][0-9]|2[0-4][0-9]|25[0-4]))|(?:(?:(?:[0-9a-z\xA1-\uD7FF\uE000-\uFFFF]|[\uD800-\uDBFF](?![\uDC00-\uDFFF])|(?:[^\uD800-\uDBFF]|^)[\uDC00-\uDFFF])+-)*(?:[0-9a-z\xA1-\uD7FF\uE000-\uFFFF]|[\uD800-\uDBFF](?![\uDC00-\uDFFF])|(?:[^\uD800-\uDBFF]|^)[\uDC00-\uDFFF])+)(?:\.(?:(?:[0-9a-z\xA1-\uD7FF\uE000-\uFFFF]|[\uD800-\uDBFF](?![\uDC00-\uDFFF])|(?:[^\uD800-\uDBFF]|^)[\uDC00-\uDFFF])+-)*(?:[0-9a-z\xA1-\uD7FF\uE000-\uFFFF]|[\uD800-\uDBFF](?![\uDC00-\uDFFF])|(?:[^\uD800-\uDBFF]|^)[\uDC00-\uDFFF])+)*(?:\.(?:(?:[a-z\xA1-\uD7FF\uE000-\uFFFF]|[\uD800-\uDBFF](?![\uDC00-\uDFFF])|(?:[^\uD800-\uDBFF]|^)[\uDC00-\uDFFF]){2,})))(?::[0-9]{2,5})?(?:\/(?:[\0-\x08\x0E-\x1F!-\x9F\xA1-\u167F\u1681-\u1FFF\u200B-\u2027\u202A-\u202E\u2030-\u205E\u2060-\u2FFF\u3001-\uD7FF\uE000-\uFEFE\uFF00-\uFFFF]|[\uD800-\uDBFF][\uDC00-\uDFFF]|[\uD800-\uDBFF](?![\uDC00-\uDFFF])|(?:[^\uD800-\uDBFF]|^)[\uDC00-\uDFFF])*)?$/i,d=/^(?:urn:uuid:)?[0-9a-f]{8}-(?:[0-9a-f]{4}-){3}[0-9a-f]{12}$/i,h=/^(?:\/(?:[^~/]|~0|~1)*)*$/,f=/^#(?:\/(?:[a-z0-9_\-.!$&'()*+,;:=@]|%[0-9a-f]{2}|~0|~1)*)*$/i,p=/^(?:0|[1-9][0-9]*)(?:#|(?:\/(?:[^~/]|~0|~1)*)*)$/;function g(e){return e="full"==e?"full":"fast",r.copy(g[e])}function v(e){var t=e.match(i);if(!t)return!1;var n=+t[1],r=+t[2],a=+t[3];return r>=1&&r<=12&&a>=1&&a<=(2==r&&function(e){return e%4==0&&(e%100!=0||e%400==0)}(n)?29:o[r])}function m(e,t){var n=e.match(a);if(!n)return!1;var r=n[1],i=n[2],o=n[3],s=n[5];return(r<=23&&i<=59&&o<=59||23==r&&59==i&&60==o)&&(!t||s)}e.exports=g,g.fast={date:/^\d\d\d\d-[0-1]\d-[0-3]\d$/,time:/^(?:[0-2]\d:[0-5]\d:[0-5]\d|23:59:60)(?:\.\d+)?(?:z|[+-]\d\d(?::?\d\d)?)?$/i,"date-time":/^\d\d\d\d-[0-1]\d-[0-3]\d[t\s](?:[0-2]\d:[0-5]\d:[0-5]\d|23:59:60)(?:\.\d+)?(?:z|[+-]\d\d(?::?\d\d)?)$/i,uri:/^(?:[a-z][a-z0-9+\-.]*:)(?:\/?\/)?[^\s]*$/i,"uri-reference":/^(?:(?:[a-z][a-z0-9+\-.]*:)?\/?\/)?(?:[^\\\s#][^\s#]*)?(?:#[^\\\s]*)?$/i,"uri-template":u,url:c,email:/^[a-z0-9.!#$%&'*+/=?^_`{|}~-]+@[a-z0-9](?:[a-z0-9-]{0,61}[a-z0-9])?(?:\.[a-z0-9](?:[a-z0-9-]{0,61}[a-z0-9])?)*$/i,hostname:s,ipv4:/^(?:(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(?:25[0-5]|2[0-4]\d|[01]?\d\d?)$/,ipv6:/^\s*(?:(?:(?:[0-9a-f]{1,4}:){7}(?:[0-9a-f]{1,4}|:))|(?:(?:[0-9a-f]{1,4}:){6}(?::[0-9a-f]{1,4}|(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(?:(?:[0-9a-f]{1,4}:){5}(?:(?:(?::[0-9a-f]{1,4}){1,2})|:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(?:(?:[0-9a-f]{1,4}:){4}(?:(?:(?::[0-9a-f]{1,4}){1,3})|(?:(?::[0-9a-f]{1,4})?:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(?:(?:[0-9a-f]{1,4}:){3}(?:(?:(?::[0-9a-f]{1,4}){1,4})|(?:(?::[0-9a-f]{1,4}){0,2}:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(?:(?:[0-9a-f]{1,4}:){2}(?:(?:(?::[0-9a-f]{1,4}){1,5})|(?:(?::[0-9a-f]{1,4}){0,3}:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(?:(?:[0-9a-f]{1,4}:){1}(?:(?:(?::[0-9a-f]{1,4}){1,6})|(?:(?::[0-9a-f]{1,4}){0,4}:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(?::(?:(?:(?::[0-9a-f]{1,4}){1,7})|(?:(?::[0-9a-f]{1,4}){0,5}:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:)))(?:%.+)?\s*$/i,regex:A,uuid:d,"json-pointer":h,"json-pointer-uri-fragment":f,"relative-json-pointer":p},g.full={date:v,time:m,"date-time":function(e){var t=e.split(y);return 2==t.length&&v(t[0])&&m(t[1],!0)},uri:function(e){return b.test(e)&&l.test(e)},"uri-reference":/^(?:[a-z][a-z0-9+\-.]*:)?(?:\/?\/(?:(?:[a-z0-9\-._~!$&'()*+,;=:]|%[0-9a-f]{2})*@)?(?:\[(?:(?:(?:(?:[0-9a-f]{1,4}:){6}|::(?:[0-9a-f]{1,4}:){5}|(?:[0-9a-f]{1,4})?::(?:[0-9a-f]{1,4}:){4}|(?:(?:[0-9a-f]{1,4}:){0,1}[0-9a-f]{1,4})?::(?:[0-9a-f]{1,4}:){3}|(?:(?:[0-9a-f]{1,4}:){0,2}[0-9a-f]{1,4})?::(?:[0-9a-f]{1,4}:){2}|(?:(?:[0-9a-f]{1,4}:){0,3}[0-9a-f]{1,4})?::[0-9a-f]{1,4}:|(?:(?:[0-9a-f]{1,4}:){0,4}[0-9a-f]{1,4})?::)(?:[0-9a-f]{1,4}:[0-9a-f]{1,4}|(?:(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(?:25[0-5]|2[0-4]\d|[01]?\d\d?))|(?:(?:[0-9a-f]{1,4}:){0,5}[0-9a-f]{1,4})?::[0-9a-f]{1,4}|(?:(?:[0-9a-f]{1,4}:){0,6}[0-9a-f]{1,4})?::)|[Vv][0-9a-f]+\.[a-z0-9\-._~!$&'()*+,;=:]+)\]|(?:(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(?:25[0-5]|2[0-4]\d|[01]?\d\d?)|(?:[a-z0-9\-._~!$&'"()*+,;=]|%[0-9a-f]{2})*)(?::\d*)?(?:\/(?:[a-z0-9\-._~!$&'"()*+,;=:@]|%[0-9a-f]{2})*)*|\/(?:(?:[a-z0-9\-._~!$&'"()*+,;=:@]|%[0-9a-f]{2})+(?:\/(?:[a-z0-9\-._~!$&'"()*+,;=:@]|%[0-9a-f]{2})*)*)?|(?:[a-z0-9\-._~!$&'"()*+,;=:@]|%[0-9a-f]{2})+(?:\/(?:[a-z0-9\-._~!$&'"()*+,;=:@]|%[0-9a-f]{2})*)*)?(?:\?(?:[a-z0-9\-._~!$&'"()*+,;=:@/?]|%[0-9a-f]{2})*)?(?:#(?:[a-z0-9\-._~!$&'"()*+,;=:@/?]|%[0-9a-f]{2})*)?$/i,"uri-template":u,url:c,email:/^[a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*@(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?$/i,hostname:s,ipv4:/^(?:(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(?:25[0-5]|2[0-4]\d|[01]?\d\d?)$/,ipv6:/^\s*(?:(?:(?:[0-9a-f]{1,4}:){7}(?:[0-9a-f]{1,4}|:))|(?:(?:[0-9a-f]{1,4}:){6}(?::[0-9a-f]{1,4}|(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(?:(?:[0-9a-f]{1,4}:){5}(?:(?:(?::[0-9a-f]{1,4}){1,2})|:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(?:(?:[0-9a-f]{1,4}:){4}(?:(?:(?::[0-9a-f]{1,4}){1,3})|(?:(?::[0-9a-f]{1,4})?:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(?:(?:[0-9a-f]{1,4}:){3}(?:(?:(?::[0-9a-f]{1,4}){1,4})|(?:(?::[0-9a-f]{1,4}){0,2}:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(?:(?:[0-9a-f]{1,4}:){2}(?:(?:(?::[0-9a-f]{1,4}){1,5})|(?:(?::[0-9a-f]{1,4}){0,3}:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(?:(?:[0-9a-f]{1,4}:){1}(?:(?:(?::[0-9a-f]{1,4}){1,6})|(?:(?::[0-9a-f]{1,4}){0,4}:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(?::(?:(?:(?::[0-9a-f]{1,4}){1,7})|(?:(?::[0-9a-f]{1,4}){0,5}:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:)))(?:%.+)?\s*$/i,regex:A,uuid:d,"json-pointer":h,"json-pointer-uri-fragment":f,"relative-json-pointer":p};var y=/t|\s/i;var b=/\/|:/;var C=/[^\\]\\Z/;function A(e){if(C.test(e))return!1;try{return new RegExp(e),!0}catch(e){return!1}}},function(e,t,n){"use strict";var r=n(1530),i=n(241).toHash;e.exports=function(){var e=[{type:"number",rules:[{maximum:["exclusiveMaximum"]},{minimum:["exclusiveMinimum"]},"multipleOf","format"]},{type:"string",rules:["maxLength","minLength","pattern","format"]},{type:"array",rules:["maxItems","minItems","items","contains","uniqueItems"]},{type:"object",rules:["maxProperties","minProperties","required","dependencies","propertyNames",{properties:["additionalProperties","patternProperties"]}]},{rules:["$ref","const","enum","not","anyOf","oneOf","allOf","if"]}],t=["type","$comment"];return e.all=i(t),e.types=i(["number","integer","string","array","object","boolean","null"]),e.forEach((function(n){n.rules=n.rules.map((function(n){var i;if("object"==typeof n){var o=Object.keys(n)[0];i=n[o],n=o,i.forEach((function(n){t.push(n),e.all[n]=!0}))}return t.push(n),e.all[n]={keyword:n,code:r[n],implements:i}})),e.all.$comment={keyword:"$comment",code:r.$comment},n.type&&(e.types[n.type]=n)})),e.keywords=i(t.concat(["$schema","$id","id","$data","$async","title","description","default","definitions","examples","readOnly","writeOnly","contentMediaType","contentEncoding","additionalItems","then","else"])),e.custom={},e}},function(e,t,n){"use strict";e.exports={$ref:n(1531),allOf:n(1532),anyOf:n(1533),$comment:n(1534),const:n(1535),contains:n(1536),dependencies:n(1537),enum:n(1538),format:n(1539),if:n(1540),items:n(1541),maximum:n(895),minimum:n(895),maxItems:n(896),minItems:n(896),maxLength:n(897),minLength:n(897),maxProperties:n(898),minProperties:n(898),multipleOf:n(1542),not:n(1543),oneOf:n(1544),pattern:n(1545),properties:n(1546),propertyNames:n(1547),required:n(1548),uniqueItems:n(1549),validate:n(894)}},function(e,t,n){"use strict";e.exports=function(e,t,n){var r,i,o=" ",a=e.level,s=e.dataLevel,l=e.schema[t],u=e.errSchemaPath+"/"+t,c=!e.opts.allErrors,d="data"+(s||""),h="valid"+a;if("#"==l||"#/"==l)e.isRoot?(r=e.async,i="validate"):(r=!0===e.root.schema.$async,i="root.refVal[0]");else{var f=e.resolveRef(e.baseId,l,e.isRoot);if(void 0===f){var p=e.MissingRefError.message(e.baseId,l);if("fail"==e.opts.missingRefs){e.logger.error(p),(y=y||[]).push(o),o="",!1!==e.createErrors?(o+=" { keyword: '$ref' , dataPath: (dataPath || '') + "+e.errorPath+" , schemaPath: "+e.util.toQuotedString(u)+" , params: { ref: '"+e.util.escapeQuotes(l)+"' } ",!1!==e.opts.messages&&(o+=" , message: 'can\\'t resolve reference "+e.util.escapeQuotes(l)+"' "),e.opts.verbose&&(o+=" , schema: "+e.util.toQuotedString(l)+" , parentSchema: validate.schema"+e.schemaPath+" , data: "+d+" "),o+=" } "):o+=" {} ";var g=o;o=y.pop(),!e.compositeRule&&c?e.async?o+=" throw new ValidationError(["+g+"]); ":o+=" validate.errors = ["+g+"]; return false; ":o+=" var err = "+g+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",c&&(o+=" if (false) { ")}else{if("ignore"!=e.opts.missingRefs)throw new e.MissingRefError(e.baseId,l,p);e.logger.warn(p),c&&(o+=" if (true) { ")}}else if(f.inline){var v=e.util.copy(e);v.level++;var m="valid"+v.level;v.schema=f.schema,v.schemaPath="",v.errSchemaPath=l,o+=" "+e.validate(v).replace(/validate\.schema/g,f.code)+" ",c&&(o+=" if ("+m+") { ")}else r=!0===f.$async||e.async&&!1!==f.$async,i=f.code}if(i){var y;(y=y||[]).push(o),o="",e.opts.passContext?o+=" "+i+".call(this, ":o+=" "+i+"( ",o+=" "+d+", (dataPath || '')",'""'!=e.errorPath&&(o+=" + "+e.errorPath);var b=o+=" , "+(s?"data"+(s-1||""):"parentData")+" , "+(s?e.dataPathArr[s]:"parentDataProperty")+", rootData)  ";if(o=y.pop(),r){if(!e.async)throw new Error("async schema referenced by sync schema");c&&(o+=" var "+h+"; "),o+=" try { await "+b+"; ",c&&(o+=" "+h+" = true; "),o+=" } catch (e) { if (!(e instanceof ValidationError)) throw e; if (vErrors === null) vErrors = e.errors; else vErrors = vErrors.concat(e.errors); errors = vErrors.length; ",c&&(o+=" "+h+" = false; "),o+=" } ",c&&(o+=" if ("+h+") { ")}else o+=" if (!"+b+") { if (vErrors === null) vErrors = "+i+".errors; else vErrors = vErrors.concat("+i+".errors); errors = vErrors.length; } ",c&&(o+=" else { ")}return o}},function(e,t,n){"use strict";e.exports=function(e,t,n){var r=" ",i=e.schema[t],o=e.schemaPath+e.util.getProperty(t),a=e.errSchemaPath+"/"+t,s=!e.opts.allErrors,l=e.util.copy(e),u="";l.level++;var c="valid"+l.level,d=l.baseId,h=!0,f=i;if(f)for(var p,g=-1,v=f.length-1;g<v;)p=f[g+=1],(e.opts.strictKeywords?"object"==typeof p&&Object.keys(p).length>0||!1===p:e.util.schemaHasRules(p,e.RULES.all))&&(h=!1,l.schema=p,l.schemaPath=o+"["+g+"]",l.errSchemaPath=a+"/"+g,r+="  "+e.validate(l)+" ",l.baseId=d,s&&(r+=" if ("+c+") { ",u+="}"));return s&&(r+=h?" if (true) { ":" "+u.slice(0,-1)+" "),r}},function(e,t,n){"use strict";e.exports=function(e,t,n){var r=" ",i=e.level,o=e.dataLevel,a=e.schema[t],s=e.schemaPath+e.util.getProperty(t),l=e.errSchemaPath+"/"+t,u=!e.opts.allErrors,c="data"+(o||""),d="valid"+i,h="errs__"+i,f=e.util.copy(e),p="";f.level++;var g="valid"+f.level;if(a.every((function(t){return e.opts.strictKeywords?"object"==typeof t&&Object.keys(t).length>0||!1===t:e.util.schemaHasRules(t,e.RULES.all)}))){var v=f.baseId;r+=" var "+h+" = errors; var "+d+" = false;  ";var m=e.compositeRule;e.compositeRule=f.compositeRule=!0;var y=a;if(y)for(var b,C=-1,A=y.length-1;C<A;)b=y[C+=1],f.schema=b,f.schemaPath=s+"["+C+"]",f.errSchemaPath=l+"/"+C,r+="  "+e.validate(f)+" ",f.baseId=v,r+=" "+d+" = "+d+" || "+g+"; if (!"+d+") { ",p+="}";e.compositeRule=f.compositeRule=m,r+=" "+p+" if (!"+d+") {   var err =   ",!1!==e.createErrors?(r+=" { keyword: 'anyOf' , dataPath: (dataPath || '') + "+e.errorPath+" , schemaPath: "+e.util.toQuotedString(l)+" , params: {} ",!1!==e.opts.messages&&(r+=" , message: 'should match some schema in anyOf' "),e.opts.verbose&&(r+=" , schema: validate.schema"+s+" , parentSchema: validate.schema"+e.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ",r+=";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",!e.compositeRule&&u&&(e.async?r+=" throw new ValidationError(vErrors); ":r+=" validate.errors = vErrors; return false; "),r+=" } else {  errors = "+h+"; if (vErrors !== null) { if ("+h+") vErrors.length = "+h+"; else vErrors = null; } ",e.opts.allErrors&&(r+=" } ")}else u&&(r+=" if (true) { ");return r}},function(e,t,n){"use strict";e.exports=function(e,t,n){var r=" ",i=e.schema[t],o=e.errSchemaPath+"/"+t,a=(e.opts.allErrors,e.util.toQuotedString(i));return!0===e.opts.$comment?r+=" console.log("+a+");":"function"==typeof e.opts.$comment&&(r+=" self._opts.$comment("+a+", "+e.util.toQuotedString(o)+", validate.root.schema);"),r}},function(e,t,n){"use strict";e.exports=function(e,t,n){var r=" ",i=e.level,o=e.dataLevel,a=e.schema[t],s=e.schemaPath+e.util.getProperty(t),l=e.errSchemaPath+"/"+t,u=!e.opts.allErrors,c="data"+(o||""),d="valid"+i,h=e.opts.$data&&a&&a.$data;h&&(r+=" var schema"+i+" = "+e.util.getData(a.$data,o,e.dataPathArr)+"; "),h||(r+=" var schema"+i+" = validate.schema"+s+";"),r+="var "+d+" = equal("+c+", schema"+i+"); if (!"+d+") {   ";var f=f||[];f.push(r),r="",!1!==e.createErrors?(r+=" { keyword: 'const' , dataPath: (dataPath || '') + "+e.errorPath+" , schemaPath: "+e.util.toQuotedString(l)+" , params: { allowedValue: schema"+i+" } ",!1!==e.opts.messages&&(r+=" , message: 'should be equal to constant' "),e.opts.verbose&&(r+=" , schema: validate.schema"+s+" , parentSchema: validate.schema"+e.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ";var p=r;return r=f.pop(),!e.compositeRule&&u?e.async?r+=" throw new ValidationError(["+p+"]); ":r+=" validate.errors = ["+p+"]; return false; ":r+=" var err = "+p+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",r+=" }",u&&(r+=" else { "),r}},function(e,t,n){"use strict";e.exports=function(e,t,n){var r=" ",i=e.level,o=e.dataLevel,a=e.schema[t],s=e.schemaPath+e.util.getProperty(t),l=e.errSchemaPath+"/"+t,u=!e.opts.allErrors,c="data"+(o||""),d="valid"+i,h="errs__"+i,f=e.util.copy(e);f.level++;var p="valid"+f.level,g="i"+i,v=f.dataLevel=e.dataLevel+1,m="data"+v,y=e.baseId,b=e.opts.strictKeywords?"object"==typeof a&&Object.keys(a).length>0||!1===a:e.util.schemaHasRules(a,e.RULES.all);if(r+="var "+h+" = errors;var "+d+";",b){var C=e.compositeRule;e.compositeRule=f.compositeRule=!0,f.schema=a,f.schemaPath=s,f.errSchemaPath=l,r+=" var "+p+" = false; for (var "+g+" = 0; "+g+" < "+c+".length; "+g+"++) { ",f.errorPath=e.util.getPathExpr(e.errorPath,g,e.opts.jsonPointers,!0);var A=c+"["+g+"]";f.dataPathArr[v]=g;var _=e.validate(f);f.baseId=y,e.util.varOccurences(_,m)<2?r+=" "+e.util.varReplace(_,m,A)+" ":r+=" var "+m+" = "+A+"; "+_+" ",r+=" if ("+p+") break; }  ",e.compositeRule=f.compositeRule=C,r+="  if (!"+p+") {"}else r+=" if ("+c+".length == 0) {";var I=I||[];I.push(r),r="",!1!==e.createErrors?(r+=" { keyword: 'contains' , dataPath: (dataPath || '') + "+e.errorPath+" , schemaPath: "+e.util.toQuotedString(l)+" , params: {} ",!1!==e.opts.messages&&(r+=" , message: 'should contain a valid item' "),e.opts.verbose&&(r+=" , schema: validate.schema"+s+" , parentSchema: validate.schema"+e.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ";var x=r;return r=I.pop(),!e.compositeRule&&u?e.async?r+=" throw new ValidationError(["+x+"]); ":r+=" validate.errors = ["+x+"]; return false; ":r+=" var err = "+x+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",r+=" } else { ",b&&(r+="  errors = "+h+"; if (vErrors !== null) { if ("+h+") vErrors.length = "+h+"; else vErrors = null; } "),e.opts.allErrors&&(r+=" } "),r}},function(e,t,n){"use strict";e.exports=function(e,t,n){var r=" ",i=e.level,o=e.dataLevel,a=e.schema[t],s=e.schemaPath+e.util.getProperty(t),l=e.errSchemaPath+"/"+t,u=!e.opts.allErrors,c="data"+(o||""),d="errs__"+i,h=e.util.copy(e),f="";h.level++;var p="valid"+h.level,g={},v={},m=e.opts.ownProperties;for(A in a)if("__proto__"!=A){var y=a[A],b=Array.isArray(y)?v:g;b[A]=y}r+="var "+d+" = errors;";var C=e.errorPath;for(var A in r+="var missing"+i+";",v)if((b=v[A]).length){if(r+=" if ( "+c+e.util.getProperty(A)+" !== undefined ",m&&(r+=" && Object.prototype.hasOwnProperty.call("+c+", '"+e.util.escapeQuotes(A)+"') "),u){r+=" && ( ";var _=b;if(_)for(var I=-1,x=_.length-1;I<x;){O=_[I+=1],I&&(r+=" || "),r+=" ( ( "+(P=c+(D=e.util.getProperty(O)))+" === undefined ",m&&(r+=" || ! Object.prototype.hasOwnProperty.call("+c+", '"+e.util.escapeQuotes(O)+"') "),r+=") && (missing"+i+" = "+e.util.toQuotedString(e.opts.jsonPointers?O:D)+") ) "}r+=")) {  ";var w="missing"+i,M="' + "+w+" + '";e.opts._errorDataPathProperty&&(e.errorPath=e.opts.jsonPointers?e.util.getPathExpr(C,w,!0):C+" + "+w);var S=S||[];S.push(r),r="",!1!==e.createErrors?(r+=" { keyword: 'dependencies' , dataPath: (dataPath || '') + "+e.errorPath+" , schemaPath: "+e.util.toQuotedString(l)+" , params: { property: '"+e.util.escapeQuotes(A)+"', missingProperty: '"+M+"', depsCount: "+b.length+", deps: '"+e.util.escapeQuotes(1==b.length?b[0]:b.join(", "))+"' } ",!1!==e.opts.messages&&(r+=" , message: 'should have ",1==b.length?r+="property "+e.util.escapeQuotes(b[0]):r+="properties "+e.util.escapeQuotes(b.join(", ")),r+=" when property "+e.util.escapeQuotes(A)+" is present' "),e.opts.verbose&&(r+=" , schema: validate.schema"+s+" , parentSchema: validate.schema"+e.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ";var T=r;r=S.pop(),!e.compositeRule&&u?e.async?r+=" throw new ValidationError(["+T+"]); ":r+=" validate.errors = ["+T+"]; return false; ":r+=" var err = "+T+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; "}else{r+=" ) { ";var E=b;if(E)for(var O,k=-1,L=E.length-1;k<L;){O=E[k+=1];var D=e.util.getProperty(O),P=(M=e.util.escapeQuotes(O),c+D);e.opts._errorDataPathProperty&&(e.errorPath=e.util.getPath(C,O,e.opts.jsonPointers)),r+=" if ( "+P+" === undefined ",m&&(r+=" || ! Object.prototype.hasOwnProperty.call("+c+", '"+e.util.escapeQuotes(O)+"') "),r+=") {  var err =   ",!1!==e.createErrors?(r+=" { keyword: 'dependencies' , dataPath: (dataPath || '') + "+e.errorPath+" , schemaPath: "+e.util.toQuotedString(l)+" , params: { property: '"+e.util.escapeQuotes(A)+"', missingProperty: '"+M+"', depsCount: "+b.length+", deps: '"+e.util.escapeQuotes(1==b.length?b[0]:b.join(", "))+"' } ",!1!==e.opts.messages&&(r+=" , message: 'should have ",1==b.length?r+="property "+e.util.escapeQuotes(b[0]):r+="properties "+e.util.escapeQuotes(b.join(", ")),r+=" when property "+e.util.escapeQuotes(A)+" is present' "),e.opts.verbose&&(r+=" , schema: validate.schema"+s+" , parentSchema: validate.schema"+e.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ",r+=";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; } "}}r+=" }   ",u&&(f+="}",r+=" else { ")}e.errorPath=C;var N=h.baseId;for(var A in g){y=g[A];(e.opts.strictKeywords?"object"==typeof y&&Object.keys(y).length>0||!1===y:e.util.schemaHasRules(y,e.RULES.all))&&(r+=" "+p+" = true; if ( "+c+e.util.getProperty(A)+" !== undefined ",m&&(r+=" && Object.prototype.hasOwnProperty.call("+c+", '"+e.util.escapeQuotes(A)+"') "),r+=") { ",h.schema=y,h.schemaPath=s+e.util.getProperty(A),h.errSchemaPath=l+"/"+e.util.escapeFragment(A),r+="  "+e.validate(h)+" ",h.baseId=N,r+=" }  ",u&&(r+=" if ("+p+") { ",f+="}"))}return u&&(r+="   "+f+" if ("+d+" == errors) {"),r}},function(e,t,n){"use strict";e.exports=function(e,t,n){var r=" ",i=e.level,o=e.dataLevel,a=e.schema[t],s=e.schemaPath+e.util.getProperty(t),l=e.errSchemaPath+"/"+t,u=!e.opts.allErrors,c="data"+(o||""),d="valid"+i,h=e.opts.$data&&a&&a.$data;h&&(r+=" var schema"+i+" = "+e.util.getData(a.$data,o,e.dataPathArr)+"; ");var f="i"+i,p="schema"+i;h||(r+=" var "+p+" = validate.schema"+s+";"),r+="var "+d+";",h&&(r+=" if (schema"+i+" === undefined) "+d+" = true; else if (!Array.isArray(schema"+i+")) "+d+" = false; else {"),r+=d+" = false;for (var "+f+"=0; "+f+"<"+p+".length; "+f+"++) if (equal("+c+", "+p+"["+f+"])) { "+d+" = true; break; }",h&&(r+="  }  "),r+=" if (!"+d+") {   ";var g=g||[];g.push(r),r="",!1!==e.createErrors?(r+=" { keyword: 'enum' , dataPath: (dataPath || '') + "+e.errorPath+" , schemaPath: "+e.util.toQuotedString(l)+" , params: { allowedValues: schema"+i+" } ",!1!==e.opts.messages&&(r+=" , message: 'should be equal to one of the allowed values' "),e.opts.verbose&&(r+=" , schema: validate.schema"+s+" , parentSchema: validate.schema"+e.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ";var v=r;return r=g.pop(),!e.compositeRule&&u?e.async?r+=" throw new ValidationError(["+v+"]); ":r+=" validate.errors = ["+v+"]; return false; ":r+=" var err = "+v+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",r+=" }",u&&(r+=" else { "),r}},function(e,t,n){"use strict";e.exports=function(e,t,n){var r=" ",i=e.level,o=e.dataLevel,a=e.schema[t],s=e.schemaPath+e.util.getProperty(t),l=e.errSchemaPath+"/"+t,u=!e.opts.allErrors,c="data"+(o||"");if(!1===e.opts.format)return u&&(r+=" if (true) { "),r;var d,h=e.opts.$data&&a&&a.$data;h?(r+=" var schema"+i+" = "+e.util.getData(a.$data,o,e.dataPathArr)+"; ",d="schema"+i):d=a;var f=e.opts.unknownFormats,p=Array.isArray(f);if(h){r+=" var "+(g="format"+i)+" = formats["+d+"]; var "+(v="isObject"+i)+" = typeof "+g+" == 'object' && !("+g+" instanceof RegExp) && "+g+".validate; var "+(m="formatType"+i)+" = "+v+" && "+g+".type || 'string'; if ("+v+") { ",e.async&&(r+=" var async"+i+" = "+g+".async; "),r+=" "+g+" = "+g+".validate; } if (  ",h&&(r+=" ("+d+" !== undefined && typeof "+d+" != 'string') || "),r+=" (","ignore"!=f&&(r+=" ("+d+" && !"+g+" ",p&&(r+=" && self._opts.unknownFormats.indexOf("+d+") == -1 "),r+=") || "),r+=" ("+g+" && "+m+" == '"+n+"' && !(typeof "+g+" == 'function' ? ",e.async?r+=" (async"+i+" ? await "+g+"("+c+") : "+g+"("+c+")) ":r+=" "+g+"("+c+") ",r+=" : "+g+".test("+c+"))))) {"}else{var g;if(!(g=e.formats[a])){if("ignore"==f)return e.logger.warn('unknown format "'+a+'" ignored in schema at path "'+e.errSchemaPath+'"'),u&&(r+=" if (true) { "),r;if(p&&f.indexOf(a)>=0)return u&&(r+=" if (true) { "),r;throw new Error('unknown format "'+a+'" is used in schema at path "'+e.errSchemaPath+'"')}var v,m=(v="object"==typeof g&&!(g instanceof RegExp)&&g.validate)&&g.type||"string";if(v){var y=!0===g.async;g=g.validate}if(m!=n)return u&&(r+=" if (true) { "),r;if(y){if(!e.async)throw new Error("async format in sync schema");r+=" if (!(await "+(b="formats"+e.util.getProperty(a)+".validate")+"("+c+"))) { "}else{r+=" if (! ";var b="formats"+e.util.getProperty(a);v&&(b+=".validate"),r+="function"==typeof g?" "+b+"("+c+") ":" "+b+".test("+c+") ",r+=") { "}}var C=C||[];C.push(r),r="",!1!==e.createErrors?(r+=" { keyword: 'format' , dataPath: (dataPath || '') + "+e.errorPath+" , schemaPath: "+e.util.toQuotedString(l)+" , params: { format:  ",r+=h?""+d:""+e.util.toQuotedString(a),r+="  } ",!1!==e.opts.messages&&(r+=" , message: 'should match format \"",r+=h?"' + "+d+" + '":""+e.util.escapeQuotes(a),r+="\"' "),e.opts.verbose&&(r+=" , schema:  ",r+=h?"validate.schema"+s:""+e.util.toQuotedString(a),r+="         , parentSchema: validate.schema"+e.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ";var A=r;return r=C.pop(),!e.compositeRule&&u?e.async?r+=" throw new ValidationError(["+A+"]); ":r+=" validate.errors = ["+A+"]; return false; ":r+=" var err = "+A+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",r+=" } ",u&&(r+=" else { "),r}},function(e,t,n){"use strict";e.exports=function(e,t,n){var r=" ",i=e.level,o=e.dataLevel,a=e.schema[t],s=e.schemaPath+e.util.getProperty(t),l=e.errSchemaPath+"/"+t,u=!e.opts.allErrors,c="data"+(o||""),d="valid"+i,h="errs__"+i,f=e.util.copy(e);f.level++;var p="valid"+f.level,g=e.schema.then,v=e.schema.else,m=void 0!==g&&(e.opts.strictKeywords?"object"==typeof g&&Object.keys(g).length>0||!1===g:e.util.schemaHasRules(g,e.RULES.all)),y=void 0!==v&&(e.opts.strictKeywords?"object"==typeof v&&Object.keys(v).length>0||!1===v:e.util.schemaHasRules(v,e.RULES.all)),b=f.baseId;if(m||y){var C;f.createErrors=!1,f.schema=a,f.schemaPath=s,f.errSchemaPath=l,r+=" var "+h+" = errors; var "+d+" = true;  ";var A=e.compositeRule;e.compositeRule=f.compositeRule=!0,r+="  "+e.validate(f)+" ",f.baseId=b,f.createErrors=!0,r+="  errors = "+h+"; if (vErrors !== null) { if ("+h+") vErrors.length = "+h+"; else vErrors = null; }  ",e.compositeRule=f.compositeRule=A,m?(r+=" if ("+p+") {  ",f.schema=e.schema.then,f.schemaPath=e.schemaPath+".then",f.errSchemaPath=e.errSchemaPath+"/then",r+="  "+e.validate(f)+" ",f.baseId=b,r+=" "+d+" = "+p+"; ",m&&y?r+=" var "+(C="ifClause"+i)+" = 'then'; ":C="'then'",r+=" } ",y&&(r+=" else { ")):r+=" if (!"+p+") { ",y&&(f.schema=e.schema.else,f.schemaPath=e.schemaPath+".else",f.errSchemaPath=e.errSchemaPath+"/else",r+="  "+e.validate(f)+" ",f.baseId=b,r+=" "+d+" = "+p+"; ",m&&y?r+=" var "+(C="ifClause"+i)+" = 'else'; ":C="'else'",r+=" } "),r+=" if (!"+d+") {   var err =   ",!1!==e.createErrors?(r+=" { keyword: 'if' , dataPath: (dataPath || '') + "+e.errorPath+" , schemaPath: "+e.util.toQuotedString(l)+" , params: { failingKeyword: "+C+" } ",!1!==e.opts.messages&&(r+=" , message: 'should match \"' + "+C+" + '\" schema' "),e.opts.verbose&&(r+=" , schema: validate.schema"+s+" , parentSchema: validate.schema"+e.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ",r+=";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",!e.compositeRule&&u&&(e.async?r+=" throw new ValidationError(vErrors); ":r+=" validate.errors = vErrors; return false; "),r+=" }   ",u&&(r+=" else { ")}else u&&(r+=" if (true) { ");return r}},function(e,t,n){"use strict";e.exports=function(e,t,n){var r=" ",i=e.level,o=e.dataLevel,a=e.schema[t],s=e.schemaPath+e.util.getProperty(t),l=e.errSchemaPath+"/"+t,u=!e.opts.allErrors,c="data"+(o||""),d="valid"+i,h="errs__"+i,f=e.util.copy(e),p="";f.level++;var g="valid"+f.level,v="i"+i,m=f.dataLevel=e.dataLevel+1,y="data"+m,b=e.baseId;if(r+="var "+h+" = errors;var "+d+";",Array.isArray(a)){var C=e.schema.additionalItems;if(!1===C){r+=" "+d+" = "+c+".length <= "+a.length+"; ";var A=l;l=e.errSchemaPath+"/additionalItems",r+="  if (!"+d+") {   ";var _=_||[];_.push(r),r="",!1!==e.createErrors?(r+=" { keyword: 'additionalItems' , dataPath: (dataPath || '') + "+e.errorPath+" , schemaPath: "+e.util.toQuotedString(l)+" , params: { limit: "+a.length+" } ",!1!==e.opts.messages&&(r+=" , message: 'should NOT have more than "+a.length+" items' "),e.opts.verbose&&(r+=" , schema: false , parentSchema: validate.schema"+e.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ";var I=r;r=_.pop(),!e.compositeRule&&u?e.async?r+=" throw new ValidationError(["+I+"]); ":r+=" validate.errors = ["+I+"]; return false; ":r+=" var err = "+I+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",r+=" } ",l=A,u&&(p+="}",r+=" else { ")}var x=a;if(x)for(var w,M=-1,S=x.length-1;M<S;)if(w=x[M+=1],e.opts.strictKeywords?"object"==typeof w&&Object.keys(w).length>0||!1===w:e.util.schemaHasRules(w,e.RULES.all)){r+=" "+g+" = true; if ("+c+".length > "+M+") { ";var T=c+"["+M+"]";f.schema=w,f.schemaPath=s+"["+M+"]",f.errSchemaPath=l+"/"+M,f.errorPath=e.util.getPathExpr(e.errorPath,M,e.opts.jsonPointers,!0),f.dataPathArr[m]=M;var E=e.validate(f);f.baseId=b,e.util.varOccurences(E,y)<2?r+=" "+e.util.varReplace(E,y,T)+" ":r+=" var "+y+" = "+T+"; "+E+" ",r+=" }  ",u&&(r+=" if ("+g+") { ",p+="}")}if("object"==typeof C&&(e.opts.strictKeywords?"object"==typeof C&&Object.keys(C).length>0||!1===C:e.util.schemaHasRules(C,e.RULES.all))){f.schema=C,f.schemaPath=e.schemaPath+".additionalItems",f.errSchemaPath=e.errSchemaPath+"/additionalItems",r+=" "+g+" = true; if ("+c+".length > "+a.length+") {  for (var "+v+" = "+a.length+"; "+v+" < "+c+".length; "+v+"++) { ",f.errorPath=e.util.getPathExpr(e.errorPath,v,e.opts.jsonPointers,!0);T=c+"["+v+"]";f.dataPathArr[m]=v;E=e.validate(f);f.baseId=b,e.util.varOccurences(E,y)<2?r+=" "+e.util.varReplace(E,y,T)+" ":r+=" var "+y+" = "+T+"; "+E+" ",u&&(r+=" if (!"+g+") break; "),r+=" } }  ",u&&(r+=" if ("+g+") { ",p+="}")}}else if(e.opts.strictKeywords?"object"==typeof a&&Object.keys(a).length>0||!1===a:e.util.schemaHasRules(a,e.RULES.all)){f.schema=a,f.schemaPath=s,f.errSchemaPath=l,r+="  for (var "+v+" = 0; "+v+" < "+c+".length; "+v+"++) { ",f.errorPath=e.util.getPathExpr(e.errorPath,v,e.opts.jsonPointers,!0);T=c+"["+v+"]";f.dataPathArr[m]=v;E=e.validate(f);f.baseId=b,e.util.varOccurences(E,y)<2?r+=" "+e.util.varReplace(E,y,T)+" ":r+=" var "+y+" = "+T+"; "+E+" ",u&&(r+=" if (!"+g+") break; "),r+=" }"}return u&&(r+=" "+p+" if ("+h+" == errors) {"),r}},function(e,t,n){"use strict";e.exports=function(e,t,n){var r,i=" ",o=e.level,a=e.dataLevel,s=e.schema[t],l=e.schemaPath+e.util.getProperty(t),u=e.errSchemaPath+"/"+t,c=!e.opts.allErrors,d="data"+(a||""),h=e.opts.$data&&s&&s.$data;if(h?(i+=" var schema"+o+" = "+e.util.getData(s.$data,a,e.dataPathArr)+"; ",r="schema"+o):r=s,!h&&"number"!=typeof s)throw new Error(t+" must be number");i+="var division"+o+";if (",h&&(i+=" "+r+" !== undefined && ( typeof "+r+" != 'number' || "),i+=" (division"+o+" = "+d+" / "+r+", ",e.opts.multipleOfPrecision?i+=" Math.abs(Math.round(division"+o+") - division"+o+") > 1e-"+e.opts.multipleOfPrecision+" ":i+=" division"+o+" !== parseInt(division"+o+") ",i+=" ) ",h&&(i+="  )  "),i+=" ) {   ";var f=f||[];f.push(i),i="",!1!==e.createErrors?(i+=" { keyword: 'multipleOf' , dataPath: (dataPath || '') + "+e.errorPath+" , schemaPath: "+e.util.toQuotedString(u)+" , params: { multipleOf: "+r+" } ",!1!==e.opts.messages&&(i+=" , message: 'should be multiple of ",i+=h?"' + "+r:r+"'"),e.opts.verbose&&(i+=" , schema:  ",i+=h?"validate.schema"+l:""+s,i+="         , parentSchema: validate.schema"+e.schemaPath+" , data: "+d+" "),i+=" } "):i+=" {} ";var p=i;return i=f.pop(),!e.compositeRule&&c?e.async?i+=" throw new ValidationError(["+p+"]); ":i+=" validate.errors = ["+p+"]; return false; ":i+=" var err = "+p+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",i+="} ",c&&(i+=" else { "),i}},function(e,t,n){"use strict";e.exports=function(e,t,n){var r=" ",i=e.level,o=e.dataLevel,a=e.schema[t],s=e.schemaPath+e.util.getProperty(t),l=e.errSchemaPath+"/"+t,u=!e.opts.allErrors,c="data"+(o||""),d="errs__"+i,h=e.util.copy(e);h.level++;var f="valid"+h.level;if(e.opts.strictKeywords?"object"==typeof a&&Object.keys(a).length>0||!1===a:e.util.schemaHasRules(a,e.RULES.all)){h.schema=a,h.schemaPath=s,h.errSchemaPath=l,r+=" var "+d+" = errors;  ";var p,g=e.compositeRule;e.compositeRule=h.compositeRule=!0,h.createErrors=!1,h.opts.allErrors&&(p=h.opts.allErrors,h.opts.allErrors=!1),r+=" "+e.validate(h)+" ",h.createErrors=!0,p&&(h.opts.allErrors=p),e.compositeRule=h.compositeRule=g,r+=" if ("+f+") {   ";var v=v||[];v.push(r),r="",!1!==e.createErrors?(r+=" { keyword: 'not' , dataPath: (dataPath || '') + "+e.errorPath+" , schemaPath: "+e.util.toQuotedString(l)+" , params: {} ",!1!==e.opts.messages&&(r+=" , message: 'should NOT be valid' "),e.opts.verbose&&(r+=" , schema: validate.schema"+s+" , parentSchema: validate.schema"+e.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ";var m=r;r=v.pop(),!e.compositeRule&&u?e.async?r+=" throw new ValidationError(["+m+"]); ":r+=" validate.errors = ["+m+"]; return false; ":r+=" var err = "+m+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",r+=" } else {  errors = "+d+"; if (vErrors !== null) { if ("+d+") vErrors.length = "+d+"; else vErrors = null; } ",e.opts.allErrors&&(r+=" } ")}else r+="  var err =   ",!1!==e.createErrors?(r+=" { keyword: 'not' , dataPath: (dataPath || '') + "+e.errorPath+" , schemaPath: "+e.util.toQuotedString(l)+" , params: {} ",!1!==e.opts.messages&&(r+=" , message: 'should NOT be valid' "),e.opts.verbose&&(r+=" , schema: validate.schema"+s+" , parentSchema: validate.schema"+e.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ",r+=";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",u&&(r+=" if (false) { ");return r}},function(e,t,n){"use strict";e.exports=function(e,t,n){var r=" ",i=e.level,o=e.dataLevel,a=e.schema[t],s=e.schemaPath+e.util.getProperty(t),l=e.errSchemaPath+"/"+t,u=!e.opts.allErrors,c="data"+(o||""),d="valid"+i,h="errs__"+i,f=e.util.copy(e),p="";f.level++;var g="valid"+f.level,v=f.baseId,m="prevValid"+i,y="passingSchemas"+i;r+="var "+h+" = errors , "+m+" = false , "+d+" = false , "+y+" = null; ";var b=e.compositeRule;e.compositeRule=f.compositeRule=!0;var C=a;if(C)for(var A,_=-1,I=C.length-1;_<I;)A=C[_+=1],(e.opts.strictKeywords?"object"==typeof A&&Object.keys(A).length>0||!1===A:e.util.schemaHasRules(A,e.RULES.all))?(f.schema=A,f.schemaPath=s+"["+_+"]",f.errSchemaPath=l+"/"+_,r+="  "+e.validate(f)+" ",f.baseId=v):r+=" var "+g+" = true; ",_&&(r+=" if ("+g+" && "+m+") { "+d+" = false; "+y+" = ["+y+", "+_+"]; } else { ",p+="}"),r+=" if ("+g+") { "+d+" = "+m+" = true; "+y+" = "+_+"; }";return e.compositeRule=f.compositeRule=b,r+=p+"if (!"+d+") {   var err =   ",!1!==e.createErrors?(r+=" { keyword: 'oneOf' , dataPath: (dataPath || '') + "+e.errorPath+" , schemaPath: "+e.util.toQuotedString(l)+" , params: { passingSchemas: "+y+" } ",!1!==e.opts.messages&&(r+=" , message: 'should match exactly one schema in oneOf' "),e.opts.verbose&&(r+=" , schema: validate.schema"+s+" , parentSchema: validate.schema"+e.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ",r+=";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",!e.compositeRule&&u&&(e.async?r+=" throw new ValidationError(vErrors); ":r+=" validate.errors = vErrors; return false; "),r+="} else {  errors = "+h+"; if (vErrors !== null) { if ("+h+") vErrors.length = "+h+"; else vErrors = null; }",e.opts.allErrors&&(r+=" } "),r}},function(e,t,n){"use strict";e.exports=function(e,t,n){var r,i=" ",o=e.level,a=e.dataLevel,s=e.schema[t],l=e.schemaPath+e.util.getProperty(t),u=e.errSchemaPath+"/"+t,c=!e.opts.allErrors,d="data"+(a||""),h=e.opts.$data&&s&&s.$data;h?(i+=" var schema"+o+" = "+e.util.getData(s.$data,a,e.dataPathArr)+"; ",r="schema"+o):r=s,i+="if ( ",h&&(i+=" ("+r+" !== undefined && typeof "+r+" != 'string') || "),i+=" !"+(h?"(new RegExp("+r+"))":e.usePattern(s))+".test("+d+") ) {   ";var f=f||[];f.push(i),i="",!1!==e.createErrors?(i+=" { keyword: 'pattern' , dataPath: (dataPath || '') + "+e.errorPath+" , schemaPath: "+e.util.toQuotedString(u)+" , params: { pattern:  ",i+=h?""+r:""+e.util.toQuotedString(s),i+="  } ",!1!==e.opts.messages&&(i+=" , message: 'should match pattern \"",i+=h?"' + "+r+" + '":""+e.util.escapeQuotes(s),i+="\"' "),e.opts.verbose&&(i+=" , schema:  ",i+=h?"validate.schema"+l:""+e.util.toQuotedString(s),i+="         , parentSchema: validate.schema"+e.schemaPath+" , data: "+d+" "),i+=" } "):i+=" {} ";var p=i;return i=f.pop(),!e.compositeRule&&c?e.async?i+=" throw new ValidationError(["+p+"]); ":i+=" validate.errors = ["+p+"]; return false; ":i+=" var err = "+p+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",i+="} ",c&&(i+=" else { "),i}},function(e,t,n){"use strict";e.exports=function(e,t,n){var r=" ",i=e.level,o=e.dataLevel,a=e.schema[t],s=e.schemaPath+e.util.getProperty(t),l=e.errSchemaPath+"/"+t,u=!e.opts.allErrors,c="data"+(o||""),d="errs__"+i,h=e.util.copy(e),f="";h.level++;var p="valid"+h.level,g="key"+i,v="idx"+i,m=h.dataLevel=e.dataLevel+1,y="data"+m,b="dataProperties"+i,C=Object.keys(a||{}).filter(D),A=e.schema.patternProperties||{},_=Object.keys(A).filter(D),I=e.schema.additionalProperties,x=C.length||_.length,w=!1===I,M="object"==typeof I&&Object.keys(I).length,S=e.opts.removeAdditional,T=w||M||S,E=e.opts.ownProperties,O=e.baseId,k=e.schema.required;if(k&&(!e.opts.$data||!k.$data)&&k.length<e.opts.loopRequired)var L=e.util.toHash(k);function D(e){return"__proto__"!==e}if(r+="var "+d+" = errors;var "+p+" = true;",E&&(r+=" var "+b+" = undefined;"),T){if(r+=E?" "+b+" = "+b+" || Object.keys("+c+"); for (var "+v+"=0; "+v+"<"+b+".length; "+v+"++) { var "+g+" = "+b+"["+v+"]; ":" for (var "+g+" in "+c+") { ",x){if(r+=" var isAdditional"+i+" = !(false ",C.length)if(C.length>8)r+=" || validate.schema"+s+".hasOwnProperty("+g+") ";else{var P=C;if(P)for(var N=-1,j=P.length-1;N<j;)Z=P[N+=1],r+=" || "+g+" == "+e.util.toQuotedString(Z)+" "}if(_.length){var R=_;if(R)for(var z=-1,F=R.length-1;z<F;)oe=R[z+=1],r+=" || "+e.usePattern(oe)+".test("+g+") "}r+=" ); if (isAdditional"+i+") { "}if("all"==S)r+=" delete "+c+"["+g+"]; ";else{var B=e.errorPath,H="' + "+g+" + '";if(e.opts._errorDataPathProperty&&(e.errorPath=e.util.getPathExpr(e.errorPath,g,e.opts.jsonPointers)),w)if(S)r+=" delete "+c+"["+g+"]; ";else{r+=" "+p+" = false; ";var U=l;l=e.errSchemaPath+"/additionalProperties",(ne=ne||[]).push(r),r="",!1!==e.createErrors?(r+=" { keyword: 'additionalProperties' , dataPath: (dataPath || '') + "+e.errorPath+" , schemaPath: "+e.util.toQuotedString(l)+" , params: { additionalProperty: '"+H+"' } ",!1!==e.opts.messages&&(r+=" , message: '",e.opts._errorDataPathProperty?r+="is an invalid additional property":r+="should NOT have additional properties",r+="' "),e.opts.verbose&&(r+=" , schema: false , parentSchema: validate.schema"+e.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ";var V=r;r=ne.pop(),!e.compositeRule&&u?e.async?r+=" throw new ValidationError(["+V+"]); ":r+=" validate.errors = ["+V+"]; return false; ":r+=" var err = "+V+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",l=U,u&&(r+=" break; ")}else if(M)if("failing"==S){r+=" var "+d+" = errors;  ";var Y=e.compositeRule;e.compositeRule=h.compositeRule=!0,h.schema=I,h.schemaPath=e.schemaPath+".additionalProperties",h.errSchemaPath=e.errSchemaPath+"/additionalProperties",h.errorPath=e.opts._errorDataPathProperty?e.errorPath:e.util.getPathExpr(e.errorPath,g,e.opts.jsonPointers);var W=c+"["+g+"]";h.dataPathArr[m]=g;var G=e.validate(h);h.baseId=O,e.util.varOccurences(G,y)<2?r+=" "+e.util.varReplace(G,y,W)+" ":r+=" var "+y+" = "+W+"; "+G+" ",r+=" if (!"+p+") { errors = "+d+"; if (validate.errors !== null) { if (errors) validate.errors.length = errors; else validate.errors = null; } delete "+c+"["+g+"]; }  ",e.compositeRule=h.compositeRule=Y}else{h.schema=I,h.schemaPath=e.schemaPath+".additionalProperties",h.errSchemaPath=e.errSchemaPath+"/additionalProperties",h.errorPath=e.opts._errorDataPathProperty?e.errorPath:e.util.getPathExpr(e.errorPath,g,e.opts.jsonPointers);W=c+"["+g+"]";h.dataPathArr[m]=g;G=e.validate(h);h.baseId=O,e.util.varOccurences(G,y)<2?r+=" "+e.util.varReplace(G,y,W)+" ":r+=" var "+y+" = "+W+"; "+G+" ",u&&(r+=" if (!"+p+") break; ")}e.errorPath=B}x&&(r+=" } "),r+=" }  ",u&&(r+=" if ("+p+") { ",f+="}")}var X=e.opts.useDefaults&&!e.compositeRule;if(C.length){var q=C;if(q)for(var Z,Q=-1,K=q.length-1;Q<K;){var J=a[Z=q[Q+=1]];if(e.opts.strictKeywords?"object"==typeof J&&Object.keys(J).length>0||!1===J:e.util.schemaHasRules(J,e.RULES.all)){var $=e.util.getProperty(Z),ee=(W=c+$,X&&void 0!==J.default);h.schema=J,h.schemaPath=s+$,h.errSchemaPath=l+"/"+e.util.escapeFragment(Z),h.errorPath=e.util.getPath(e.errorPath,Z,e.opts.jsonPointers),h.dataPathArr[m]=e.util.toQuotedString(Z);G=e.validate(h);if(h.baseId=O,e.util.varOccurences(G,y)<2){G=e.util.varReplace(G,y,W);var te=W}else{te=y;r+=" var "+y+" = "+W+"; "}if(ee)r+=" "+G+" ";else{if(L&&L[Z]){r+=" if ( "+te+" === undefined ",E&&(r+=" || ! Object.prototype.hasOwnProperty.call("+c+", '"+e.util.escapeQuotes(Z)+"') "),r+=") { "+p+" = false; ";B=e.errorPath,U=l;var ne,re=e.util.escapeQuotes(Z);e.opts._errorDataPathProperty&&(e.errorPath=e.util.getPath(B,Z,e.opts.jsonPointers)),l=e.errSchemaPath+"/required",(ne=ne||[]).push(r),r="",!1!==e.createErrors?(r+=" { keyword: 'required' , dataPath: (dataPath || '') + "+e.errorPath+" , schemaPath: "+e.util.toQuotedString(l)+" , params: { missingProperty: '"+re+"' } ",!1!==e.opts.messages&&(r+=" , message: '",e.opts._errorDataPathProperty?r+="is a required property":r+="should have required property \\'"+re+"\\'",r+="' "),e.opts.verbose&&(r+=" , schema: validate.schema"+s+" , parentSchema: validate.schema"+e.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ";V=r;r=ne.pop(),!e.compositeRule&&u?e.async?r+=" throw new ValidationError(["+V+"]); ":r+=" validate.errors = ["+V+"]; return false; ":r+=" var err = "+V+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",l=U,e.errorPath=B,r+=" } else { "}else u?(r+=" if ( "+te+" === undefined ",E&&(r+=" || ! Object.prototype.hasOwnProperty.call("+c+", '"+e.util.escapeQuotes(Z)+"') "),r+=") { "+p+" = true; } else { "):(r+=" if ("+te+" !== undefined ",E&&(r+=" &&   Object.prototype.hasOwnProperty.call("+c+", '"+e.util.escapeQuotes(Z)+"') "),r+=" ) { ");r+=" "+G+" } "}}u&&(r+=" if ("+p+") { ",f+="}")}}if(_.length){var ie=_;if(ie)for(var oe,ae=-1,se=ie.length-1;ae<se;){J=A[oe=ie[ae+=1]];if(e.opts.strictKeywords?"object"==typeof J&&Object.keys(J).length>0||!1===J:e.util.schemaHasRules(J,e.RULES.all)){h.schema=J,h.schemaPath=e.schemaPath+".patternProperties"+e.util.getProperty(oe),h.errSchemaPath=e.errSchemaPath+"/patternProperties/"+e.util.escapeFragment(oe),r+=E?" "+b+" = "+b+" || Object.keys("+c+"); for (var "+v+"=0; "+v+"<"+b+".length; "+v+"++) { var "+g+" = "+b+"["+v+"]; ":" for (var "+g+" in "+c+") { ",r+=" if ("+e.usePattern(oe)+".test("+g+")) { ",h.errorPath=e.util.getPathExpr(e.errorPath,g,e.opts.jsonPointers);W=c+"["+g+"]";h.dataPathArr[m]=g;G=e.validate(h);h.baseId=O,e.util.varOccurences(G,y)<2?r+=" "+e.util.varReplace(G,y,W)+" ":r+=" var "+y+" = "+W+"; "+G+" ",u&&(r+=" if (!"+p+") break; "),r+=" } ",u&&(r+=" else "+p+" = true; "),r+=" }  ",u&&(r+=" if ("+p+") { ",f+="}")}}}return u&&(r+=" "+f+" if ("+d+" == errors) {"),r}},function(e,t,n){"use strict";e.exports=function(e,t,n){var r=" ",i=e.level,o=e.dataLevel,a=e.schema[t],s=e.schemaPath+e.util.getProperty(t),l=e.errSchemaPath+"/"+t,u=!e.opts.allErrors,c="data"+(o||""),d="errs__"+i,h=e.util.copy(e);h.level++;var f="valid"+h.level;if(r+="var "+d+" = errors;",e.opts.strictKeywords?"object"==typeof a&&Object.keys(a).length>0||!1===a:e.util.schemaHasRules(a,e.RULES.all)){h.schema=a,h.schemaPath=s,h.errSchemaPath=l;var p="key"+i,g="idx"+i,v="i"+i,m="' + "+p+" + '",y="data"+(h.dataLevel=e.dataLevel+1),b="dataProperties"+i,C=e.opts.ownProperties,A=e.baseId;C&&(r+=" var "+b+" = undefined; "),r+=C?" "+b+" = "+b+" || Object.keys("+c+"); for (var "+g+"=0; "+g+"<"+b+".length; "+g+"++) { var "+p+" = "+b+"["+g+"]; ":" for (var "+p+" in "+c+") { ",r+=" var startErrs"+i+" = errors; ";var _=p,I=e.compositeRule;e.compositeRule=h.compositeRule=!0;var x=e.validate(h);h.baseId=A,e.util.varOccurences(x,y)<2?r+=" "+e.util.varReplace(x,y,_)+" ":r+=" var "+y+" = "+_+"; "+x+" ",e.compositeRule=h.compositeRule=I,r+=" if (!"+f+") { for (var "+v+"=startErrs"+i+"; "+v+"<errors; "+v+"++) { vErrors["+v+"].propertyName = "+p+"; }   var err =   ",!1!==e.createErrors?(r+=" { keyword: 'propertyNames' , dataPath: (dataPath || '') + "+e.errorPath+" , schemaPath: "+e.util.toQuotedString(l)+" , params: { propertyName: '"+m+"' } ",!1!==e.opts.messages&&(r+=" , message: 'property name \\'"+m+"\\' is invalid' "),e.opts.verbose&&(r+=" , schema: validate.schema"+s+" , parentSchema: validate.schema"+e.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ",r+=";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",!e.compositeRule&&u&&(e.async?r+=" throw new ValidationError(vErrors); ":r+=" validate.errors = vErrors; return false; "),u&&(r+=" break; "),r+=" } }"}return u&&(r+="  if ("+d+" == errors) {"),r}},function(e,t,n){"use strict";e.exports=function(e,t,n){var r=" ",i=e.level,o=e.dataLevel,a=e.schema[t],s=e.schemaPath+e.util.getProperty(t),l=e.errSchemaPath+"/"+t,u=!e.opts.allErrors,c="data"+(o||""),d="valid"+i,h=e.opts.$data&&a&&a.$data;h&&(r+=" var schema"+i+" = "+e.util.getData(a.$data,o,e.dataPathArr)+"; ");var f="schema"+i;if(!h)if(a.length<e.opts.loopRequired&&e.schema.properties&&Object.keys(e.schema.properties).length){var p=[],g=a;if(g)for(var v,m=-1,y=g.length-1;m<y;){v=g[m+=1];var b=e.schema.properties[v];b&&(e.opts.strictKeywords?"object"==typeof b&&Object.keys(b).length>0||!1===b:e.util.schemaHasRules(b,e.RULES.all))||(p[p.length]=v)}}else p=a;if(h||p.length){var C=e.errorPath,A=h||p.length>=e.opts.loopRequired,_=e.opts.ownProperties;if(u)if(r+=" var missing"+i+"; ",A){h||(r+=" var "+f+" = validate.schema"+s+"; ");var I="' + "+(E="schema"+i+"["+(M="i"+i)+"]")+" + '";e.opts._errorDataPathProperty&&(e.errorPath=e.util.getPathExpr(C,E,e.opts.jsonPointers)),r+=" var "+d+" = true; ",h&&(r+=" if (schema"+i+" === undefined) "+d+" = true; else if (!Array.isArray(schema"+i+")) "+d+" = false; else {"),r+=" for (var "+M+" = 0; "+M+" < "+f+".length; "+M+"++) { "+d+" = "+c+"["+f+"["+M+"]] !== undefined ",_&&(r+=" &&   Object.prototype.hasOwnProperty.call("+c+", "+f+"["+M+"]) "),r+="; if (!"+d+") break; } ",h&&(r+="  }  "),r+="  if (!"+d+") {   ",(T=T||[]).push(r),r="",!1!==e.createErrors?(r+=" { keyword: 'required' , dataPath: (dataPath || '') + "+e.errorPath+" , schemaPath: "+e.util.toQuotedString(l)+" , params: { missingProperty: '"+I+"' } ",!1!==e.opts.messages&&(r+=" , message: '",e.opts._errorDataPathProperty?r+="is a required property":r+="should have required property \\'"+I+"\\'",r+="' "),e.opts.verbose&&(r+=" , schema: validate.schema"+s+" , parentSchema: validate.schema"+e.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ";var x=r;r=T.pop(),!e.compositeRule&&u?e.async?r+=" throw new ValidationError(["+x+"]); ":r+=" validate.errors = ["+x+"]; return false; ":r+=" var err = "+x+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",r+=" } else { "}else{r+=" if ( ";var w=p;if(w)for(var M=-1,S=w.length-1;M<S;){k=w[M+=1],M&&(r+=" || "),r+=" ( ( "+(N=c+(P=e.util.getProperty(k)))+" === undefined ",_&&(r+=" || ! Object.prototype.hasOwnProperty.call("+c+", '"+e.util.escapeQuotes(k)+"') "),r+=") && (missing"+i+" = "+e.util.toQuotedString(e.opts.jsonPointers?k:P)+") ) "}r+=") {  ";var T;I="' + "+(E="missing"+i)+" + '";e.opts._errorDataPathProperty&&(e.errorPath=e.opts.jsonPointers?e.util.getPathExpr(C,E,!0):C+" + "+E),(T=T||[]).push(r),r="",!1!==e.createErrors?(r+=" { keyword: 'required' , dataPath: (dataPath || '') + "+e.errorPath+" , schemaPath: "+e.util.toQuotedString(l)+" , params: { missingProperty: '"+I+"' } ",!1!==e.opts.messages&&(r+=" , message: '",e.opts._errorDataPathProperty?r+="is a required property":r+="should have required property \\'"+I+"\\'",r+="' "),e.opts.verbose&&(r+=" , schema: validate.schema"+s+" , parentSchema: validate.schema"+e.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ";x=r;r=T.pop(),!e.compositeRule&&u?e.async?r+=" throw new ValidationError(["+x+"]); ":r+=" validate.errors = ["+x+"]; return false; ":r+=" var err = "+x+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",r+=" } else { "}else if(A){h||(r+=" var "+f+" = validate.schema"+s+"; ");var E;I="' + "+(E="schema"+i+"["+(M="i"+i)+"]")+" + '";e.opts._errorDataPathProperty&&(e.errorPath=e.util.getPathExpr(C,E,e.opts.jsonPointers)),h&&(r+=" if ("+f+" && !Array.isArray("+f+")) {  var err =   ",!1!==e.createErrors?(r+=" { keyword: 'required' , dataPath: (dataPath || '') + "+e.errorPath+" , schemaPath: "+e.util.toQuotedString(l)+" , params: { missingProperty: '"+I+"' } ",!1!==e.opts.messages&&(r+=" , message: '",e.opts._errorDataPathProperty?r+="is a required property":r+="should have required property \\'"+I+"\\'",r+="' "),e.opts.verbose&&(r+=" , schema: validate.schema"+s+" , parentSchema: validate.schema"+e.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ",r+=";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; } else if ("+f+" !== undefined) { "),r+=" for (var "+M+" = 0; "+M+" < "+f+".length; "+M+"++) { if ("+c+"["+f+"["+M+"]] === undefined ",_&&(r+=" || ! Object.prototype.hasOwnProperty.call("+c+", "+f+"["+M+"]) "),r+=") {  var err =   ",!1!==e.createErrors?(r+=" { keyword: 'required' , dataPath: (dataPath || '') + "+e.errorPath+" , schemaPath: "+e.util.toQuotedString(l)+" , params: { missingProperty: '"+I+"' } ",!1!==e.opts.messages&&(r+=" , message: '",e.opts._errorDataPathProperty?r+="is a required property":r+="should have required property \\'"+I+"\\'",r+="' "),e.opts.verbose&&(r+=" , schema: validate.schema"+s+" , parentSchema: validate.schema"+e.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ",r+=";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; } } ",h&&(r+="  }  ")}else{var O=p;if(O)for(var k,L=-1,D=O.length-1;L<D;){k=O[L+=1];var P=e.util.getProperty(k),N=(I=e.util.escapeQuotes(k),c+P);e.opts._errorDataPathProperty&&(e.errorPath=e.util.getPath(C,k,e.opts.jsonPointers)),r+=" if ( "+N+" === undefined ",_&&(r+=" || ! Object.prototype.hasOwnProperty.call("+c+", '"+e.util.escapeQuotes(k)+"') "),r+=") {  var err =   ",!1!==e.createErrors?(r+=" { keyword: 'required' , dataPath: (dataPath || '') + "+e.errorPath+" , schemaPath: "+e.util.toQuotedString(l)+" , params: { missingProperty: '"+I+"' } ",!1!==e.opts.messages&&(r+=" , message: '",e.opts._errorDataPathProperty?r+="is a required property":r+="should have required property \\'"+I+"\\'",r+="' "),e.opts.verbose&&(r+=" , schema: validate.schema"+s+" , parentSchema: validate.schema"+e.schemaPath+" , data: "+c+" "),r+=" } "):r+=" {} ",r+=";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; } "}}e.errorPath=C}else u&&(r+=" if (true) {");return r}},function(e,t,n){"use strict";e.exports=function(e,t,n){var r,i=" ",o=e.level,a=e.dataLevel,s=e.schema[t],l=e.schemaPath+e.util.getProperty(t),u=e.errSchemaPath+"/"+t,c=!e.opts.allErrors,d="data"+(a||""),h="valid"+o,f=e.opts.$data&&s&&s.$data;if(f?(i+=" var schema"+o+" = "+e.util.getData(s.$data,a,e.dataPathArr)+"; ",r="schema"+o):r=s,(s||f)&&!1!==e.opts.uniqueItems){f&&(i+=" var "+h+"; if ("+r+" === false || "+r+" === undefined) "+h+" = true; else if (typeof "+r+" != 'boolean') "+h+" = false; else { "),i+=" var i = "+d+".length , "+h+" = true , j; if (i > 1) { ";var p=e.schema.items&&e.schema.items.type,g=Array.isArray(p);if(!p||"object"==p||"array"==p||g&&(p.indexOf("object")>=0||p.indexOf("array")>=0))i+=" outer: for (;i--;) { for (j = i; j--;) { if (equal("+d+"[i], "+d+"[j])) { "+h+" = false; break outer; } } } ";else{i+=" var itemIndices = {}, item; for (;i--;) { var item = "+d+"[i]; ";var v="checkDataType"+(g?"s":"");i+=" if ("+e.util[v](p,"item",e.opts.strictNumbers,!0)+") continue; ",g&&(i+=" if (typeof item == 'string') item = '\"' + item; "),i+=" if (typeof itemIndices[item] == 'number') { "+h+" = false; j = itemIndices[item]; break; } itemIndices[item] = i; } "}i+=" } ",f&&(i+="  }  "),i+=" if (!"+h+") {   ";var m=m||[];m.push(i),i="",!1!==e.createErrors?(i+=" { keyword: 'uniqueItems' , dataPath: (dataPath || '') + "+e.errorPath+" , schemaPath: "+e.util.toQuotedString(u)+" , params: { i: i, j: j } ",!1!==e.opts.messages&&(i+=" , message: 'should NOT have duplicate items (items ## ' + j + ' and ' + i + ' are identical)' "),e.opts.verbose&&(i+=" , schema:  ",i+=f?"validate.schema"+l:""+s,i+="         , parentSchema: validate.schema"+e.schemaPath+" , data: "+d+" "),i+=" } "):i+=" {} ";var y=i;i=m.pop(),!e.compositeRule&&c?e.async?i+=" throw new ValidationError(["+y+"]); ":i+=" validate.errors = ["+y+"]; return false; ":i+=" var err = "+y+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",i+=" } ",c&&(i+=" else { ")}else c&&(i+=" if (true) { ");return i}},function(e,t,n){"use strict";var r=["multipleOf","maximum","exclusiveMaximum","minimum","exclusiveMinimum","maxLength","minLength","pattern","additionalItems","maxItems","minItems","uniqueItems","maxProperties","minProperties","required","additionalProperties","enum","format","const"];e.exports=function(e,t){for(var n=0;n<t.length;n++){e=JSON.parse(JSON.stringify(e));var i,o=t[n].split("/"),a=e;for(i=1;i<o.length;i++)a=a[o[i]];for(i=0;i<r.length;i++){var s=r[i],l=a[s];l&&(a[s]={anyOf:[l,{$ref:"https://raw.githubusercontent.com/ajv-validator/ajv/master/lib/refs/data.json#"}]})}}return e}},function(e,t,n){"use strict";var r=n(511).MissingRef;e.exports=function e(t,n,i){var o=this;if("function"!=typeof this._opts.loadSchema)throw new Error("options.loadSchema should be a function");"function"==typeof n&&(i=n,n=void 0);var a=s(t).then((function(){var e=o._addSchema(t,void 0,n);return e.validate||function e(t){try{return o._compile(t)}catch(e){if(e instanceof r)return i(e);throw e}function i(r){var i=r.missingSchema;if(u(i))throw new Error("Schema "+i+" is loaded but "+r.missingRef+" cannot be resolved");var a=o._loadingSchemas[i];return a||(a=o._loadingSchemas[i]=o._opts.loadSchema(i)).then(l,l),a.then((function(e){if(!u(i))return s(e).then((function(){u(i)||o.addSchema(e,i,void 0,n)}))})).then((function(){return e(t)}));function l(){delete o._loadingSchemas[i]}function u(e){return o._refs[e]||o._schemas[e]}}}(e)}));i&&a.then((function(e){i(null,e)}),i);return a;function s(t){var n=t.$schema;return n&&!o.getSchema(n)?e.call(o,{$ref:n},!0):Promise.resolve()}}},function(e,t,n){"use strict";var r=/^[a-z_$][a-z0-9_$-]*$/i,i=n(1553),o=n(1554);e.exports={add:function(e,t){var n=this.RULES;if(n.keywords[e])throw new Error("Keyword "+e+" is already defined");if(!r.test(e))throw new Error("Keyword "+e+" is not a valid identifier");if(t){this.validateKeyword(t,!0);var o=t.type;if(Array.isArray(o))for(var a=0;a<o.length;a++)l(e,o[a],t);else l(e,o,t);var s=t.metaSchema;s&&(t.$data&&this._opts.$data&&(s={anyOf:[s,{$ref:"https://raw.githubusercontent.com/ajv-validator/ajv/master/lib/refs/data.json#"}]}),t.validateSchema=this.compile(s,!0))}function l(e,t,r){for(var o,a=0;a<n.length;a++){var s=n[a];if(s.type==t){o=s;break}}o||(o={type:t,rules:[]},n.push(o));var l={keyword:e,definition:r,custom:!0,code:i,implements:r.implements};o.rules.push(l),n.custom[e]=l}return n.keywords[e]=n.all[e]=!0,this},get:function(e){var t=this.RULES.custom[e];return t?t.definition:this.RULES.keywords[e]||!1},remove:function(e){var t=this.RULES;delete t.keywords[e],delete t.all[e],delete t.custom[e];for(var n=0;n<t.length;n++)for(var r=t[n].rules,i=0;i<r.length;i++)if(r[i].keyword==e){r.splice(i,1);break}return this},validate:function e(t,n){e.errors=null;var r=this._validateKeyword=this._validateKeyword||this.compile(o,!0);if(r(t))return!0;if(e.errors=r.errors,n)throw new Error("custom keyword definition is invalid: "+this.errorsText(r.errors));return!1}}},function(e,t,n){"use strict";e.exports=function(e,t,n){var r,i,o=" ",a=e.level,s=e.dataLevel,l=e.schema[t],u=e.schemaPath+e.util.getProperty(t),c=e.errSchemaPath+"/"+t,d=!e.opts.allErrors,h="data"+(s||""),f="valid"+a,p="errs__"+a,g=e.opts.$data&&l&&l.$data;g?(o+=" var schema"+a+" = "+e.util.getData(l.$data,s,e.dataPathArr)+"; ",i="schema"+a):i=l;var v,m,y,b,C,A="definition"+a,_=this.definition,I="";if(g&&_.$data){C="keywordValidate"+a;var x=_.validateSchema;o+=" var "+A+" = RULES.custom['"+t+"'].definition; var "+C+" = "+A+".validate;"}else{if(!(b=e.useCustomRule(this,l,e.schema,e)))return;i="validate.schema"+u,C=b.code,v=_.compile,m=_.inline,y=_.macro}var w=C+".errors",M="i"+a,S="ruleErr"+a,T=_.async;if(T&&!e.async)throw new Error("async keyword in sync schema");if(m||y||(o+=w+" = null;"),o+="var "+p+" = errors;var "+f+";",g&&_.$data&&(I+="}",o+=" if ("+i+" === undefined) { "+f+" = true; } else { ",x&&(I+="}",o+=" "+f+" = "+A+".validateSchema("+i+"); if ("+f+") { ")),m)_.statements?o+=" "+b.validate+" ":o+=" "+f+" = "+b.validate+"; ";else if(y){var E=e.util.copy(e);I="";E.level++;var O="valid"+E.level;E.schema=b.validate,E.schemaPath="";var k=e.compositeRule;e.compositeRule=E.compositeRule=!0;var L=e.validate(E).replace(/validate\.schema/g,C);e.compositeRule=E.compositeRule=k,o+=" "+L}else{(j=j||[]).push(o),o="",o+="  "+C+".call( ",e.opts.passContext?o+="this":o+="self",v||!1===_.schema?o+=" , "+h+" ":o+=" , "+i+" , "+h+" , validate.schema"+e.schemaPath+" ",o+=" , (dataPath || '')",'""'!=e.errorPath&&(o+=" + "+e.errorPath);var D=s?"data"+(s-1||""):"parentData",P=s?e.dataPathArr[s]:"parentDataProperty",N=o+=" , "+D+" , "+P+" , rootData )  ";o=j.pop(),!1===_.errors?(o+=" "+f+" = ",T&&(o+="await "),o+=N+"; "):o+=T?" var "+(w="customErrors"+a)+" = null; try { "+f+" = await "+N+"; } catch (e) { "+f+" = false; if (e instanceof ValidationError) "+w+" = e.errors; else throw e; } ":" "+w+" = null; "+f+" = "+N+"; "}if(_.modifying&&(o+=" if ("+D+") "+h+" = "+D+"["+P+"];"),o+=""+I,_.valid)d&&(o+=" if (true) { ");else{var j;o+=" if ( ",void 0===_.valid?(o+=" !",o+=y?""+O:""+f):o+=" "+!_.valid+" ",o+=") { ",r=this.keyword,(j=j||[]).push(o),o="",(j=j||[]).push(o),o="",!1!==e.createErrors?(o+=" { keyword: '"+(r||"custom")+"' , dataPath: (dataPath || '') + "+e.errorPath+" , schemaPath: "+e.util.toQuotedString(c)+" , params: { keyword: '"+this.keyword+"' } ",!1!==e.opts.messages&&(o+=" , message: 'should pass \""+this.keyword+"\" keyword validation' "),e.opts.verbose&&(o+=" , schema: validate.schema"+u+" , parentSchema: validate.schema"+e.schemaPath+" , data: "+h+" "),o+=" } "):o+=" {} ";var R=o;o=j.pop(),!e.compositeRule&&d?e.async?o+=" throw new ValidationError(["+R+"]); ":o+=" validate.errors = ["+R+"]; return false; ":o+=" var err = "+R+";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ";var z=o;o=j.pop(),m?_.errors?"full"!=_.errors&&(o+="  for (var "+M+"="+p+"; "+M+"<errors; "+M+"++) { var "+S+" = vErrors["+M+"]; if ("+S+".dataPath === undefined) "+S+".dataPath = (dataPath || '') + "+e.errorPath+"; if ("+S+".schemaPath === undefined) { "+S+'.schemaPath = "'+c+'"; } ',e.opts.verbose&&(o+=" "+S+".schema = "+i+"; "+S+".data = "+h+"; "),o+=" } "):!1===_.errors?o+=" "+z+" ":(o+=" if ("+p+" == errors) { "+z+" } else {  for (var "+M+"="+p+"; "+M+"<errors; "+M+"++) { var "+S+" = vErrors["+M+"]; if ("+S+".dataPath === undefined) "+S+".dataPath = (dataPath || '') + "+e.errorPath+"; if ("+S+".schemaPath === undefined) { "+S+'.schemaPath = "'+c+'"; } ',e.opts.verbose&&(o+=" "+S+".schema = "+i+"; "+S+".data = "+h+"; "),o+=" } } "):y?(o+="   var err =   ",!1!==e.createErrors?(o+=" { keyword: '"+(r||"custom")+"' , dataPath: (dataPath || '') + "+e.errorPath+" , schemaPath: "+e.util.toQuotedString(c)+" , params: { keyword: '"+this.keyword+"' } ",!1!==e.opts.messages&&(o+=" , message: 'should pass \""+this.keyword+"\" keyword validation' "),e.opts.verbose&&(o+=" , schema: validate.schema"+u+" , parentSchema: validate.schema"+e.schemaPath+" , data: "+h+" "),o+=" } "):o+=" {} ",o+=";  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ",!e.compositeRule&&d&&(e.async?o+=" throw new ValidationError(vErrors); ":o+=" validate.errors = vErrors; return false; ")):!1===_.errors?o+=" "+z+" ":(o+=" if (Array.isArray("+w+")) { if (vErrors === null) vErrors = "+w+"; else vErrors = vErrors.concat("+w+"); errors = vErrors.length;  for (var "+M+"="+p+"; "+M+"<errors; "+M+"++) { var "+S+" = vErrors["+M+"]; if ("+S+".dataPath === undefined) "+S+".dataPath = (dataPath || '') + "+e.errorPath+";  "+S+'.schemaPath = "'+c+'";  ',e.opts.verbose&&(o+=" "+S+".schema = "+i+"; "+S+".data = "+h+"; "),o+=" } } else { "+z+" } "),o+=" } ",d&&(o+=" else { ")}return o}},function(e,t,n){"use strict";var r=n(899);e.exports={$id:"https://github.com/ajv-validator/ajv/blob/master/lib/definition_schema.js",definitions:{simpleTypes:r.definitions.simpleTypes},type:"object",dependencies:{schema:["validate"],$data:["validate"],statements:["inline"],valid:{not:{required:["macro"]}}},properties:{type:r.properties.type,schema:{type:"boolean"},statements:{type:"boolean"},dependencies:{type:"array",items:{type:"string"}},metaSchema:{type:"object"},modifying:{type:"boolean"},valid:{type:"boolean"},$data:{type:"boolean"},async:{type:"boolean"},errors:{anyOf:[{type:"boolean"},{const:"full"}]}}}},function(e){e.exports=JSON.parse('{"$schema":"http://json-schema.org/draft-07/schema#","$id":"https://raw.githubusercontent.com/ajv-validator/ajv/master/lib/refs/data.json#","description":"Meta-schema for $data reference (JSON Schema extension proposal)","type":"object","required":["$data"],"properties":{"$data":{"type":"string","anyOf":[{"format":"relative-json-pointer"},{"format":"json-pointer"}]}},"additionalProperties":false}')},function(e){e.exports=JSON.parse('{"$schema":"http://json-schema.org/draft-04/schema#","description":"Jupyter Interactive Widget State JSON schema.","type":"object","properties":{"version_major":{"description":"Format version (major)","type":"number","minimum":1,"maximum":1},"version_minor":{"description":"Format version (minor)","type":"number"},"state":{"description":"Model State for All Widget Models","type":"object","additionalProperties":{"type":"object","properties":{"model_name":{"description":"Name of the JavaScript class holding the model implementation","type":"string"},"model_module":{"description":"Name of the JavaScript module holding the model implementation","type":"string"},"model_module_version":{"description":"Semver range for the JavaScript module holding the model implementation","type":"string"},"state":{"description":"Serialized state of the model","type":"object","additional_properties":true}},"required":["model_name","model_module","state"],"additionalProperties":false}}},"required":["version_major","version_minor","state"],"additionalProperties":false}')},function(e){e.exports=JSON.parse('{"$schema":"http://json-schema.org/draft-04/schema#","description":"Jupyter Interactive Widget View JSON schema.","type":"object","properties":{"version_major":{"description":"Format version (major)","type":"number","minimum":1,"maximum":1},"version_minor":{"description":"Format version (minor)","type":"number"},"model_id":{"description":"Unique identifier of the widget model to be displayed","type":"string"}},"required":["model_id"],"additionalProperties":false}')},function(e){e.exports=JSON.parse('{"$schema":"http://json-schema.org/draft-07/schema#","description":"Jupyter Interactive Widget State JSON schema.","type":"object","properties":{"version_major":{"description":"Format version (major)","type":"number","minimum":2,"maximum":2},"version_minor":{"description":"Format version (minor)","type":"number","minimum":0,"maximum":0},"state":{"description":"Model State for All Widget Models - keys are model ids, values are model state","type":"object","additionalProperties":{"type":"object","properties":{"model_name":{"description":"Name of the JavaScript class holding the model implementation","type":"string"},"model_module":{"description":"Name of the JavaScript module holding the model implementation","type":"string"},"model_module_version":{"description":"Semver range for the JavaScript module holding the model implementation","type":"string"},"state":{"description":"Serialized state of the model","type":"object"},"buffers":{"description":"Binary buffers in the state","type":"array","items":{"type":"object","properties":{"path":{"description":"A path for a binary buffer value.","type":"array","items":{"description":"An object key or array index","type":["string","number"]}},"data":{"description":"A binary buffer encoded as specified in the \'encoding\' property","type":"string"},"encoding":{"description":"The encoding of the buffer data","type":"string","oneOf":[{"const":"hex","description":"Base 16 encoding, as specified in RFC 4648, section 8 (https://tools.ietf.org/html/rfc4648#section-8)"},{"const":"base64","description":"Base 64 encoding, as specified in RFC 4648, section 4 (https://tools.ietf.org/html/rfc4648#section-4)"}]}},"required":["path","data","encoding"]}}},"required":["model_name","model_module","state"]}}},"required":["version_major","version_minor","state"]}')},function(e){e.exports=JSON.parse('{"$schema":"http://json-schema.org/draft-07/schema#","description":"Jupyter Interactive Widget View JSON schema.","type":"object","properties":{"version_major":{"description":"Format version (major)","type":"number","minimum":2,"maximum":2},"version_minor":{"description":"Format version (minor)","type":"number"},"model_id":{"description":"Unique identifier of the widget model to be displayed","type":"string"}},"required":["model_id"]}')},function(e){e.exports=JSON.parse('{"name":"@jupyter-widgets/html-manager","version":"0.20.0","description":"Standalone package for rendering Jupyter widgets outside notebooks","homepage":"https://github.com/jupyter-widgets/ipywidgets#readme","bugs":{"url":"https://github.com/jupyter-widgets/ipywidgets/issues"},"repository":{"type":"git","url":"https://github.com/jupyter-widgets/ipywidgets.git"},"license":"BSD-3-Clause","author":"Jupyter Project","main":"lib/index.js","typings":"lib/index.d.ts","files":["lib/**/*.d.ts","lib/**/*.js","dist/","css/*.css"],"scripts":{"build":"npm run build:src && webpack && npm run build:embed-amd","build:embed-amd":"node scripts/concat-amd-build.js && rimraf dist/amd","build:src":"tsc","build:test":"tsc --project test && webpack --config test/webpack.conf.js","clean":"rimraf lib && rimraf tsconfig.tsbuildinfo && rimraf dist","lint":"tslint --project tslint.json --format stylish","prepublish":"npm run clean && npm run build","test":"npm run test:unit","test:unit":"npm run test:unit:firefox && npm run test:unit:chrome","test:unit:chrome":"npm run test:unit:default -- --browsers=Chrome","test:unit:default":"npm run build:test && karma start test/karma.conf.js --log-level debug --browsers=Firefox","test:unit:firefox":"npm run test:unit:default -- --browsers=Firefox"},"dependencies":{"@jupyter-widgets/base":"^4.0.0","@jupyter-widgets/controls":"^3.0.0","@jupyter-widgets/output":"^4.0.0","@jupyter-widgets/schema":"^0.4.0","@jupyterlab/outputarea":"^3.0.0","@jupyterlab/rendermime":"^3.0.0","@jupyterlab/rendermime-interfaces":"^3.0.0","@lumino/widgets":"^1.6.0","ajv":"^6.10.0","font-awesome":"^4.7.0","jquery":"^3.1.1"},"devDependencies":{"@types/mocha":"^5.2.7","@types/requirejs":"^2.1.31","chai":"^4.0.0","css-loader":"^3.4.0","file-loader":"^5.0.2","karma":"^4.1.0","karma-chrome-launcher":"^2.2.0","karma-firefox-launcher":"^1.1.0","karma-ie-launcher":"^1.0.0","karma-mocha":"^1.3.0","karma-mocha-reporter":"^2.2.5","mocha":"^6.1.4","rimraf":"^2.6.1","style-loader":"^1.1.2","tslint":"^5.20.1","typescript":"~3.7.4","url-loader":"^3.0.0","webpack":"^4.41.5","webpack-cli":"^3.3.4"},"gitHead":"8ec6746db708fc084102cabe34f706c204098009"}')},function(e,t,n){var r,i,o=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),a=this&&this.__setModuleDefault||(Object.create?function(e,t){Object.defineProperty(e,"default",{enumerable:!0,value:t})}:function(e,t){e.default=t}),s=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)"default"!==n&&Object.hasOwnProperty.call(e,n)&&o(t,e,n);return a(t,e),t},l=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};r=[n,t,n(901),n(1188),n(17),n(0)],void 0===(i=function(e,t,n,r,i,o){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.SysExplorerView=t.SysExplorerModel=void 0,n=l(n),o=s(o);class a extends r.BaseWidgetModel{constructor(){super(...arguments),this.model_name="SysExplorerModel",this.view_name="SysExplorerView"}}t.SysExplorerModel=a;class u extends r.BaseWidgetView{initialize(e){super.initialize(e),this._initialize(u)}render(){if(super.render(),u.shell){const e=this.pWidget,t=new r.WidgetWrapper(this.store,o.createElement(n.default,{send_msg:this.send.bind(this),model:this.model}));e.addWidget(t),e.addClass(this.class_name),e.addClass("cosapp-geometry-viewer"),e.title.label=this.model.get("title"),e.title.closable=!0,u.shell._rightHandler.sideBar.tabCloseRequested.connect((e,t)=>{t.title.owner.dispose()}),e.id=i.UUID.uuid4();const a=this.model.get("anchor");"right"===a?(u.shell.add(e,"right"),u.shell.expandRight()):u.shell.add(e,"main",{mode:a})}}}t.SysExplorerView=u}.apply(t,r))||(e.exports=i)},function(e,t,n){"use strict";e.exports=n(1563)},function(e,t,n){"use strict";
/** @license React v16.13.1
 * react-is.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var r="function"==typeof Symbol&&Symbol.for,i=r?Symbol.for("react.element"):60103,o=r?Symbol.for("react.portal"):60106,a=r?Symbol.for("react.fragment"):60107,s=r?Symbol.for("react.strict_mode"):60108,l=r?Symbol.for("react.profiler"):60114,u=r?Symbol.for("react.provider"):60109,c=r?Symbol.for("react.context"):60110,d=r?Symbol.for("react.async_mode"):60111,h=r?Symbol.for("react.concurrent_mode"):60111,f=r?Symbol.for("react.forward_ref"):60112,p=r?Symbol.for("react.suspense"):60113,g=r?Symbol.for("react.suspense_list"):60120,v=r?Symbol.for("react.memo"):60115,m=r?Symbol.for("react.lazy"):60116,y=r?Symbol.for("react.block"):60121,b=r?Symbol.for("react.fundamental"):60117,C=r?Symbol.for("react.responder"):60118,A=r?Symbol.for("react.scope"):60119;function _(e){if("object"==typeof e&&null!==e){var t=e.$$typeof;switch(t){case i:switch(e=e.type){case d:case h:case a:case l:case s:case p:return e;default:switch(e=e&&e.$$typeof){case c:case f:case m:case v:case u:return e;default:return t}}case o:return t}}}function I(e){return _(e)===h}t.AsyncMode=d,t.ConcurrentMode=h,t.ContextConsumer=c,t.ContextProvider=u,t.Element=i,t.ForwardRef=f,t.Fragment=a,t.Lazy=m,t.Memo=v,t.Portal=o,t.Profiler=l,t.StrictMode=s,t.Suspense=p,t.isAsyncMode=function(e){return I(e)||_(e)===d},t.isConcurrentMode=I,t.isContextConsumer=function(e){return _(e)===c},t.isContextProvider=function(e){return _(e)===u},t.isElement=function(e){return"object"==typeof e&&null!==e&&e.$$typeof===i},t.isForwardRef=function(e){return _(e)===f},t.isFragment=function(e){return _(e)===a},t.isLazy=function(e){return _(e)===m},t.isMemo=function(e){return _(e)===v},t.isPortal=function(e){return _(e)===o},t.isProfiler=function(e){return _(e)===l},t.isStrictMode=function(e){return _(e)===s},t.isSuspense=function(e){return _(e)===p},t.isValidElementType=function(e){return"string"==typeof e||"function"==typeof e||e===a||e===h||e===l||e===s||e===p||e===g||"object"==typeof e&&null!==e&&(e.$$typeof===m||e.$$typeof===v||e.$$typeof===u||e.$$typeof===c||e.$$typeof===f||e.$$typeof===b||e.$$typeof===C||e.$$typeof===A||e.$$typeof===y)},t.typeOf=_},function(e,t,n){"use strict";
/** @license React v17.0.1
 * react-is.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var r=60103,i=60106,o=60107,a=60108,s=60114,l=60109,u=60110,c=60112,d=60113,h=60120,f=60115,p=60116,g=60121,v=60122,m=60117,y=60129,b=60131;if("function"==typeof Symbol&&Symbol.for){var C=Symbol.for;r=C("react.element"),i=C("react.portal"),o=C("react.fragment"),a=C("react.strict_mode"),s=C("react.profiler"),l=C("react.provider"),u=C("react.context"),c=C("react.forward_ref"),d=C("react.suspense"),h=C("react.suspense_list"),f=C("react.memo"),p=C("react.lazy"),g=C("react.block"),v=C("react.server.block"),m=C("react.fundamental"),y=C("react.debug_trace_mode"),b=C("react.legacy_hidden")}function A(e){if("object"==typeof e&&null!==e){var t=e.$$typeof;switch(t){case r:switch(e=e.type){case o:case s:case a:case d:case h:return e;default:switch(e=e&&e.$$typeof){case u:case c:case p:case f:case l:return e;default:return t}}case i:return t}}}var _=l,I=r,x=c,w=o,M=p,S=f,T=i,E=s,O=a,k=d;t.ContextConsumer=u,t.ContextProvider=_,t.Element=I,t.ForwardRef=x,t.Fragment=w,t.Lazy=M,t.Memo=S,t.Portal=T,t.Profiler=E,t.StrictMode=O,t.Suspense=k,t.isAsyncMode=function(){return!1},t.isConcurrentMode=function(){return!1},t.isContextConsumer=function(e){return A(e)===u},t.isContextProvider=function(e){return A(e)===l},t.isElement=function(e){return"object"==typeof e&&null!==e&&e.$$typeof===r},t.isForwardRef=function(e){return A(e)===c},t.isFragment=function(e){return A(e)===o},t.isLazy=function(e){return A(e)===p},t.isMemo=function(e){return A(e)===f},t.isPortal=function(e){return A(e)===i},t.isProfiler=function(e){return A(e)===s},t.isStrictMode=function(e){return A(e)===a},t.isSuspense=function(e){return A(e)===d},t.isValidElementType=function(e){return"string"==typeof e||"function"==typeof e||e===o||e===s||e===y||e===a||e===d||e===h||e===b||"object"==typeof e&&null!==e&&(e.$$typeof===p||e.$$typeof===f||e.$$typeof===l||e.$$typeof===u||e.$$typeof===c||e.$$typeof===m||e.$$typeof===g||e[0]===v)},t.typeOf=A},function(e,t,n){"use strict";var r=n(36),i=n(46);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(0)),a=(0,r(n(66)).default)(o.createElement("path",{d:"M4 6H2v14c0 1.1.9 2 2 2h14v-2H4V6zm16-4H8c-1.1 0-2 .9-2 2v12c0 1.1.9 2 2 2h12c1.1 0 2-.9 2-2V4c0-1.1-.9-2-2-2zm0 14H8V4h12v12zm-7-1h2v-4h4V9h-4V5h-2v4H9v2h4z"}),"AddToPhotosOutlined");t.default=a},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(289),i=n(244),o=n(190),a=n(111),s=n(131),l=n(159),u=n(243),c=n(935),d=n(516),h=n(290),f=n(191),p=function(){function e(){this._borderRects={inner:s.default.empty(),outer:s.default.empty()},this._attributes={},this._idMap={},this._nextId=0,this._borders=new c.default(this),this._pointerFine=!0}return e.fromJson=function(t){var n=new e;return e._attributeDefinitions.fromJson(t.global,n._attributes),t.borders&&(n._borders=c.default._fromJson(t.borders,n)),n._root=d.default._fromJson(t.layout,n),n._tidy(),n},e._createAttributeDefinitions=function(){var e=new i.default;return e.add("splitterSize",-1).setType(r.default.INT),e.add("enableEdgeDock",!0).setType(r.default.BOOLEAN),e.add("marginInsets",{top:0,right:0,bottom:0,left:0}).setType(r.default.JSON),e.add("tabEnableClose",!0).setType(r.default.BOOLEAN),e.add("tabCloseType",1).setType(r.default.INT),e.add("tabEnableFloat",!1).setType(r.default.BOOLEAN),e.add("tabEnableDrag",!0).setType(r.default.BOOLEAN),e.add("tabEnableRename",!0).setType(r.default.BOOLEAN),e.add("tabClassName",void 0).setType(r.default.STRING),e.add("tabIcon",void 0).setType(r.default.STRING),e.add("tabEnableRenderOnDemand",!0).setType(r.default.BOOLEAN),e.add("tabDragSpeed",.3).setType(r.default.NUMBER),e.add("tabSetEnableDeleteWhenEmpty",!0).setType(r.default.BOOLEAN),e.add("tabSetEnableDrop",!0).setType(r.default.BOOLEAN),e.add("tabSetEnableDrag",!0).setType(r.default.BOOLEAN),e.add("tabSetEnableDivide",!0).setType(r.default.BOOLEAN),e.add("tabSetEnableMaximize",!0).setType(r.default.BOOLEAN),e.add("tabSetAutoSelectTab",!0).setType(r.default.BOOLEAN),e.add("tabSetClassNameTabStrip",void 0).setType(r.default.STRING),e.add("tabSetClassNameHeader",void 0).setType(r.default.STRING),e.add("tabSetEnableTabStrip",!0).setType(r.default.BOOLEAN),e.add("tabSetHeaderHeight",0).setType(r.default.INT).setFrom(0),e.add("tabSetTabStripHeight",0).setType(r.default.INT).setFrom(0),e.add("tabSetMarginInsets",{top:0,right:0,bottom:0,left:0}).setType(r.default.JSON),e.add("tabSetBorderInsets",{top:0,right:0,bottom:0,left:0}).setType(r.default.JSON),e.add("tabSetTabLocation","top").setType(r.default.STRING),e.add("tabSetMinWidth",0).setType(r.default.NUMBER),e.add("tabSetMinHeight",0).setType(r.default.NUMBER),e.add("borderSize",200).setType(r.default.INT).setFrom(0),e.add("borderMinSize",0).setType(r.default.INT).setFrom(0),e.add("borderBarSize",0).setType(r.default.INT).setFrom(0),e.add("borderEnableDrop",!0).setType(r.default.BOOLEAN),e.add("borderAutoSelectTabWhenOpen",!0).setType(r.default.BOOLEAN),e.add("borderAutoSelectTabWhenClosed",!1).setType(r.default.BOOLEAN),e.add("borderClassName",void 0).setType(r.default.STRING),e},e.prototype._setChangeListener=function(e){this._changeListener=e},e.prototype.getActiveTabset=function(){return this._activeTabSet&&this.getNodeById(this._activeTabSet.getId())?this._activeTabSet:void 0},e.prototype._setActiveTabset=function(e){this._activeTabSet=e},e.prototype.getMaximizedTabset=function(){return this._maximizedTabSet},e.prototype._setMaximizedTabset=function(e){this._maximizedTabSet=e},e.prototype.getRoot=function(){return this._root},e.prototype.getBorderSet=function(){return this._borders},e.prototype._getOuterInnerRects=function(){return this._borderRects},e.prototype._getPointerFine=function(){return this._pointerFine},e.prototype._setPointerFine=function(e){this._pointerFine=e},e.prototype.visitNodes=function(e){this._borders._forEachNode(e),this._root._forEachNode(e,0)},e.prototype.getNodeById=function(e){return this._idMap[e]},e.prototype.doAction=function(e){switch(e.type){case l.default.ADD_NODE:var t=new h.default(this,e.data.json,!0);((n=this._idMap[e.data.toNode])instanceof f.default||n instanceof u.default||n instanceof d.default)&&n.drop(t,o.default.getByName(e.data.location),e.data.index,e.data.select);break;case l.default.MOVE_NODE:var n,r=this._idMap[e.data.fromNode];if(r instanceof h.default||r instanceof f.default)((n=this._idMap[e.data.toNode])instanceof f.default||n instanceof u.default||n instanceof d.default)&&n.drop(r,o.default.getByName(e.data.location),e.data.index,e.data.select);break;case l.default.DELETE_TAB:(v=this._idMap[e.data.node])instanceof h.default&&(delete this._idMap[e.data.node],v._delete());break;case l.default.FLOAT_TAB:(v=this._idMap[e.data.node])instanceof h.default&&v._setFloating(!0);break;case l.default.UNFLOAT_TAB:(v=this._idMap[e.data.node])instanceof h.default&&v._setFloating(!1);break;case l.default.RENAME_TAB:(v=this._idMap[e.data.node])instanceof h.default&&v._setName(e.data.text);break;case l.default.SELECT_TAB:var i=this._idMap[e.data.tabNode];if(i instanceof h.default){var a=i.getParent(),s=a.getChildren().indexOf(i);a instanceof u.default?a.getSelected()===s?a._setSelected(-1):a._setSelected(s):a instanceof f.default&&(a.getSelected()!==s&&a._setSelected(s),this._activeTabSet=a)}break;case l.default.SET_ACTIVE_TABSET:var c=this._idMap[e.data.tabsetNode];c instanceof f.default&&(this._activeTabSet=c);break;case l.default.ADJUST_SPLIT:var p=this._idMap[e.data.node1],g=this._idMap[e.data.node2];(p instanceof f.default||p instanceof d.default)&&(g instanceof f.default||g instanceof d.default)&&(this._adjustSplitSide(p,e.data.weight1,e.data.pixelWidth1),this._adjustSplitSide(g,e.data.weight2,e.data.pixelWidth2));break;case l.default.ADJUST_BORDER_SPLIT:(v=this._idMap[e.data.node])instanceof u.default&&v._setSize(e.data.pos);break;case l.default.MAXIMIZE_TOGGLE:(v=this._idMap[e.data.node])instanceof f.default&&(v===this._maximizedTabSet?this._maximizedTabSet=void 0:(this._maximizedTabSet=v,this._activeTabSet=v));break;case l.default.UPDATE_MODEL_ATTRIBUTES:this._updateAttrs(e.data.json);break;case l.default.UPDATE_NODE_ATTRIBUTES:var v;(v=this._idMap[e.data.node])._updateAttrs(e.data.json)}this._updateIdMap(),void 0!==this._changeListener&&this._changeListener()},e.prototype._updateIdMap=function(){var e=this;this._idMap={},this.visitNodes((function(t){return e._idMap[t.getId()]=t}))},e.prototype._adjustSplitSide=function(e,t,n){e._setWeight(t),null!=e.getWidth()&&e.getOrientation()===a.default.VERT?e._updateAttrs({width:n}):null!=e.getHeight()&&e.getOrientation()===a.default.HORZ&&e._updateAttrs({height:n})},e.prototype.toJson=function(){var t={global:{},layout:{}};return e._attributeDefinitions.toJson(t.global,this._attributes),this.visitNodes((function(e){e._fireEvent("save",void 0)})),t.borders=this._borders._toJson(),t.layout=this._root._toJson(),t},e.prototype.getSplitterSize=function(){var e=this._attributes.splitterSize;return-1===e&&(e=this._pointerFine?8:12),e},e.prototype.isEnableEdgeDock=function(){return this._attributes.enableEdgeDock},e.prototype._addNode=function(e){var t=e.getId();if(void 0!==this._idMap[t])throw new Error("Error: each node must have a unique id, duplicate id:"+e.getId());"splitter"!==e.getType()&&(this._idMap[t]=e)},e.prototype._layout=function(e,t){var n;return this._borderRects=this._borders._layoutBorder({outer:e,inner:e},t),e=this._borderRects.inner.removeInsets(this._getAttribute("marginInsets")),null===(n=this._root)||void 0===n||n.calcMinSize(),this._root._layout(e,t),e},e.prototype._findDropTargetNode=function(e,t,n){var r=this._root._findDropTargetNode(e,t,n);return void 0===r&&(r=this._borders._findDropTargetNode(e,t,n)),r},e.prototype._tidy=function(){this._root._tidy()},e.prototype._updateAttrs=function(t){e._attributeDefinitions.update(t,this._attributes)},e.prototype._nextUniqueId=function(){for(this._nextId++;void 0!==this._idMap["#"+this._nextId];)this._nextId++;return"#"+this._nextId},e.prototype._getAttribute=function(e){return this._attributes[e]},e.prototype.setOnAllowDrop=function(e){this._onAllowDrop=e},e.prototype._getOnAllowDrop=function(){return this._onAllowDrop},e.prototype.toString=function(){return JSON.stringify(this.toJson())},e._attributeDefinitions=e._createAttributeDefinitions(),e}();t.default=p},function(e,t,n){"use strict";var r,i=this&&this.__extends||(r=function(e,t){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(e[n]=t[n])})(e,t)},function(e,t){function n(){this.constructor=e}r(e,t),e.prototype=null===t?Object.create(t):(n.prototype=t.prototype,new n)});Object.defineProperty(t,"__esModule",{value:!0}),t.Layout=void 0;var o=n(0),a=n(190),s=n(515),l=n(159),u=n(371),c=n(290),d=n(191),h=n(131),f=n(148),p=n(1568),g=n(1570),v=n(1571),m=n(1572),y=n(1574),b=n(1575),C=n(1576),A="undefined"!=typeof window&&(window.document.documentMode||/Edge\//.test(window.navigator.userAgent)),_="undefined"!=typeof window&&window.matchMedia&&window.matchMedia("(hover: hover) and (pointer: fine)").matches&&!A,I=function(e){function t(t){var n=e.call(this,t)||this;return n.firstMove=!1,n.dragDivText="",n.onModelChange=function(){n.forceUpdate(),n.props.onModelChange&&n.props.onModelChange(n.props.model)},n.updateRect=function(){var e=n.getDomRect(),t=new h.default(0,0,e.width,e.height);t.equals(n.state.rect)||0===t.width||0===t.height||n.setState({rect:t})},n.updateLayoutMetrics=function(){if(n.findHeaderBarSizeRef.current){var e=n.findHeaderBarSizeRef.current.getBoundingClientRect().height;e!==n.state.calculatedHeaderBarSize&&n.setState({calculatedHeaderBarSize:e})}if(n.findTabBarSizeRef.current){var t=n.findTabBarSizeRef.current.getBoundingClientRect().height;t!==n.state.calculatedTabBarSize&&n.setState({calculatedTabBarSize:t})}if(n.findBorderBarSizeRef.current){var r=n.findBorderBarSizeRef.current.getBoundingClientRect().height;r!==n.state.calculatedBorderBarSize&&n.setState({calculatedBorderBarSize:r})}},n.getClassName=function(e){return void 0===n.props.classNameMapper?e:n.props.classNameMapper(e)},n.onCloseWindow=function(e){n.doAction(l.default.unFloatTab(e));try{n.props.model.getNodeById(e)._setWindow(void 0)}catch(e){}},n.onSetWindow=function(e,t){n.props.model.getNodeById(e)._setWindow(t)},n.onCancelAdd=function(){n.selfRef.current.removeChild(n.dragDiv),n.dragDiv=void 0,null!=n.fnNewNodeDropped&&(n.fnNewNodeDropped(),n.fnNewNodeDropped=void 0),s.default.instance.hideGlass(),n.newTabJson=void 0},n.onCancelDrag=function(e){if(e){var t=n.selfRef.current;try{t.removeChild(n.outlineDiv)}catch(e){}try{t.removeChild(n.dragDiv)}catch(e){}n.dragDiv=void 0,n.hideEdges(t),null!=n.fnNewNodeDropped&&(n.fnNewNodeDropped(),n.fnNewNodeDropped=void 0),s.default.instance.hideGlass(),n.newTabJson=void 0}},n.onDragDivMouseDown=function(e){e.preventDefault(),n.dragStart(e,n.dragDivText,c.default._fromJson(n.newTabJson,n.props.model,!1),!0,void 0,void 0)},n.dragStart=function(e,t,r,i,o,a){void 0===n.props.model.getMaximizedTabset()&&i?(n.dragNode=r,n.dragDivText=t,s.default.instance.startDrag(e,n.onDragStart,n.onDragMove,n.onDragEnd,n.onCancelDrag,o,a,n.currentDocument)):s.default.instance.startDrag(e,void 0,void 0,void 0,void 0,o,a,n.currentDocument)},n.onDragStart=function(){n.dropInfo=void 0;var e=n.selfRef.current;return n.outlineDiv=n.currentDocument.createElement("div"),n.outlineDiv.className=n.getClassName(f.CLASSES.FLEXLAYOUT__OUTLINE_RECT),e.appendChild(n.outlineDiv),null==n.dragDiv&&(n.dragDiv=n.currentDocument.createElement("div"),n.dragDiv.className=n.getClassName(f.CLASSES.FLEXLAYOUT__DRAG_RECT),n.dragDiv.innerHTML=n.dragDivText,e.appendChild(n.dragDiv)),n.showEdges(e),void 0!==n.dragNode&&n.dragNode instanceof c.default&&void 0!==n.dragNode.getTabRect()&&n.dragNode.getTabRect().positionElement(n.outlineDiv),n.firstMove=!0,!0},n.onDragMove=function(e){if(!1===n.firstMove){var t=n.props.model._getAttribute("tabDragSpeed");n.outlineDiv.style.transition="top "+t+"s, left "+t+"s, width "+t+"s, height "+t+"s"}n.firstMove=!1;var r=n.selfRef.current.getBoundingClientRect(),i={x:e.clientX-r.left,y:e.clientY-r.top};n.dragDiv.style.left=i.x-n.dragDiv.getBoundingClientRect().width/2+"px",n.dragDiv.style.top=i.y+5+"px";var o=n.props.model._findDropTargetNode(n.dragNode,i.x,i.y);o&&(n.dropInfo=o,n.outlineDiv.className=n.getClassName(o.className),o.rect.positionElement(n.outlineDiv))},n.onDragEnd=function(){var e=n.selfRef.current;e.removeChild(n.outlineDiv),e.removeChild(n.dragDiv),n.dragDiv=void 0,n.hideEdges(e),s.default.instance.hideGlass(),n.dropInfo&&(void 0!==n.newTabJson?(n.doAction(l.default.addNode(n.newTabJson,n.dropInfo.node.getId(),n.dropInfo.location,n.dropInfo.index)),null!=n.fnNewNodeDropped&&(n.fnNewNodeDropped(),n.fnNewNodeDropped=void 0),n.newTabJson=void 0):void 0!==n.dragNode&&n.doAction(l.default.moveNode(n.dragNode.getId(),n.dropInfo.node.getId(),n.dropInfo.location,n.dropInfo.index)))},n.props.model._setChangeListener(n.onModelChange),n.tabIds=[],n.selfRef=o.createRef(),n.findHeaderBarSizeRef=o.createRef(),n.findTabBarSizeRef=o.createRef(),n.findBorderBarSizeRef=o.createRef(),n.supportsPopout=void 0!==t.supportsPopout?t.supportsPopout:_,n.popoutURL=t.popoutURL?t.popoutURL:"popout.html",n.icons=t.closeIcon?Object.assign({close:t.closeIcon},t.icons):t.icons,n.firstRender=!0,n.state={rect:new h.default(0,0,0,0),calculatedHeaderBarSize:25,calculatedTabBarSize:26,calculatedBorderBarSize:30},n}return i(t,e),t.prototype.styleFont=function(e){return this.props.font&&(this.props.font.size&&(e.fontSize=this.props.font.size),this.props.font.family&&(e.fontFamily=this.props.font.family),this.props.font.style&&(e.fontStyle=this.props.font.style),this.props.font.weight&&(e.fontWeight=this.props.font.weight)),e},t.prototype.doAction=function(e){if(void 0!==this.props.onAction){var t=this.props.onAction(e);void 0!==t&&this.props.model.doAction(t)}else this.props.model.doAction(e)},t.prototype.componentDidMount=function(){this.updateRect(),this.updateLayoutMetrics(),this.currentDocument=this.selfRef.current.ownerDocument,this.currentWindow=this.currentDocument.defaultView,this.currentWindow.addEventListener("resize",this.updateRect)},t.prototype.componentDidUpdate=function(){this.updateRect(),this.updateLayoutMetrics(),this.props.model!==this.previousModel&&(void 0!==this.previousModel&&this.previousModel._setChangeListener(void 0),this.props.model._setChangeListener(this.onModelChange),this.previousModel=this.props.model)},t.prototype.getCurrentDocument=function(){return this.currentDocument},t.prototype.getDomRect=function(){return this.selfRef.current.getBoundingClientRect()},t.prototype.getRootDiv=function(){return this.selfRef.current},t.prototype.isSupportsPopout=function(){return this.supportsPopout},t.prototype.getPopoutURL=function(){return this.popoutURL},t.prototype.componentWillUnmount=function(){this.currentWindow.removeEventListener("resize",this.updateRect)},t.prototype.render=function(){var e=this;if(this.firstRender)return this.firstRender=!1,o.createElement("div",{ref:this.selfRef,className:this.getClassName(f.CLASSES.FLEXLAYOUT__LAYOUT)},this.metricsElements());this.props.model._setPointerFine(window&&window.matchMedia&&window.matchMedia("(pointer: fine)").matches);var t=[],n=[],r=[],i={},a=[],s={headerBarSize:this.state.calculatedHeaderBarSize,tabBarSize:this.state.calculatedTabBarSize,borderBarSize:this.state.calculatedBorderBarSize};this.centerRect=this.props.model._layout(this.state.rect,s),this.renderBorder(this.props.model.getBorderSet(),t,i,r,a),this.renderChildren(this.props.model.getRoot(),n,i,r,a);var l=[],u={};return this.tabIds.forEach((function(e){i[e]&&(l.push(e),u[e]=e)})),this.tabIds=l,Object.keys(i).forEach((function(t){u[t]||e.tabIds.push(t)})),o.createElement("div",{ref:this.selfRef,className:this.getClassName(f.CLASSES.FLEXLAYOUT__LAYOUT)},n,this.tabIds.map((function(e){return i[e]})),t,a,r,this.metricsElements())},t.prototype.metricsElements=function(){var e=this.styleFont({visibility:"hidden"});return o.createElement(o.Fragment,null,o.createElement("div",{key:"findHeaderBarSize",ref:this.findHeaderBarSizeRef,style:e,className:this.getClassName(f.CLASSES.FLEXLAYOUT__TABSET_HEADER_SIZER)},"FindHeaderBarSize"),o.createElement("div",{key:"findTabBarSize",ref:this.findTabBarSizeRef,style:e,className:this.getClassName(f.CLASSES.FLEXLAYOUT__TABSET_SIZER)},"FindTabBarSize"),o.createElement("div",{key:"findBorderBarSize",ref:this.findBorderBarSizeRef,style:e,className:this.getClassName(f.CLASSES.FLEXLAYOUT__BORDER_SIZER)},"FindBorderBarSize"))},t.prototype.renderBorder=function(e,t,n,r,i){for(var a=0,s=e.getBorders();a<s.length;a++){var l=s[a];if(l.isShowing()){t.push(o.createElement(p.BorderTabSet,{key:"border_"+l.getLocation().getName(),border:l,layout:this,iconFactory:this.props.iconFactory,titleFactory:this.props.titleFactory,icons:this.icons}));for(var d=0,h=0,f=l._getDrawChildren();h<f.length;h++){var m=f[h];if(m instanceof u.default)i.push(o.createElement(g.Splitter,{key:m.getId(),layout:this,node:m}));else if(m instanceof c.default)if(this.supportsPopout&&m.isFloating()){var A=this._getScreenRect(m);r.push(o.createElement(y.FloatingWindow,{key:m.getId(),url:this.popoutURL,rect:A,title:m.getName(),id:m.getId(),onSetWindow:this.onSetWindow,onCloseWindow:this.onCloseWindow},o.createElement(b.FloatingWindowTab,{layout:this,node:m,factory:this.props.factory}))),n[m.getId()]=o.createElement(C.TabFloating,{key:m.getId(),layout:this,node:m,selected:d===l.getSelected()})}else n[m.getId()]=o.createElement(v.Tab,{key:m.getId(),layout:this,node:m,selected:d===l.getSelected(),factory:this.props.factory});d++}}}},t.prototype.renderChildren=function(e,t,n,r,i){for(var a=0,s=e._getDrawChildren();a<s.length;a++){var l=s[a];if(l instanceof u.default)i.push(o.createElement(g.Splitter,{key:l.getId(),layout:this,node:l}));else if(l instanceof d.default)t.push(o.createElement(m.TabSet,{key:l.getId(),layout:this,node:l,iconFactory:this.props.iconFactory,titleFactory:this.props.titleFactory,icons:this.icons})),this.renderChildren(l,t,n,r,i);else if(l instanceof c.default){var h=l.getParent().getChildren()[l.getParent().getSelected()];if(void 0===h&&console.warn("undefined selectedTab should not happen"),this.supportsPopout&&l.isFloating()){var f=this._getScreenRect(l);r.push(o.createElement(y.FloatingWindow,{key:l.getId(),url:this.popoutURL,rect:f,title:l.getName(),id:l.getId(),onSetWindow:this.onSetWindow,onCloseWindow:this.onCloseWindow},o.createElement(b.FloatingWindowTab,{layout:this,node:l,factory:this.props.factory}))),n[l.getId()]=o.createElement(C.TabFloating,{key:l.getId(),layout:this,node:l,selected:l===h})}else n[l.getId()]=o.createElement(v.Tab,{key:l.getId(),layout:this,node:l,selected:l===h,factory:this.props.factory})}else this.renderChildren(l,t,n,r,i)}},t.prototype._getScreenRect=function(e){var t=e.getRect().clone(),n=this.selfRef.current.getBoundingClientRect(),r=Math.min(80,this.currentWindow.outerHeight-this.currentWindow.innerHeight),i=Math.min(80,this.currentWindow.outerWidth-this.currentWindow.innerWidth);return t.x=t.x+n.x+this.currentWindow.screenX+i,t.y=t.y+n.y+this.currentWindow.screenY+r,t},t.prototype.addTabToTabSet=function(e,t){void 0!==this.props.model.getNodeById(e)&&this.doAction(l.default.addNode(t,e,a.default.CENTER,-1))},t.prototype.addTabToActiveTabSet=function(e){var t=this.props.model.getActiveTabset();void 0!==t&&this.doAction(l.default.addNode(e,t.getId(),a.default.CENTER,-1))},t.prototype.addTabWithDragAndDrop=function(e,t,n){this.fnNewNodeDropped=n,this.newTabJson=t,this.dragStart(void 0,e,c.default._fromJson(t,this.props.model,!1),!0,void 0,void 0)},t.prototype.addTabWithDragAndDropIndirect=function(e,t,n){this.fnNewNodeDropped=n,this.newTabJson=t,s.default.instance.addGlass(this.onCancelAdd),this.dragDivText=e,this.dragDiv=this.currentDocument.createElement("div"),this.dragDiv.className=this.getClassName("flexlayout__drag_rect"),this.dragDiv.innerHTML=this.dragDivText,this.dragDiv.addEventListener("mousedown",this.onDragDivMouseDown),this.dragDiv.addEventListener("touchstart",this.onDragDivMouseDown);var r=new h.default(10,10,150,50);r.centerInRect(this.state.rect),this.dragDiv.style.left=r.x+"px",this.dragDiv.style.top=r.y+"px",this.selfRef.current.appendChild(this.dragDiv)},t.prototype.showEdges=function(e){if(this.props.model.isEnableEdgeDock()){var t=e.getBoundingClientRect(),n=this.centerRect;this.edgeTopDiv=this.currentDocument.createElement("div"),this.edgeTopDiv.className=this.getClassName(f.CLASSES.FLEXLAYOUT__EDGE_RECT),this.edgeTopDiv.style.top=n.y+"px",this.edgeTopDiv.style.left=n.x+(n.width-100)/2+"px",this.edgeTopDiv.style.width="100px",this.edgeTopDiv.style.height="10px",this.edgeTopDiv.style.borderBottomLeftRadius="50px",this.edgeTopDiv.style.borderBottomRightRadius="50px",this.edgeLeftDiv=this.currentDocument.createElement("div"),this.edgeLeftDiv.className=this.getClassName(f.CLASSES.FLEXLAYOUT__EDGE_RECT),this.edgeLeftDiv.style.top=n.y+(n.height-100)/2+"px",this.edgeLeftDiv.style.left=n.x+"px",this.edgeLeftDiv.style.width="10px",this.edgeLeftDiv.style.height="100px",this.edgeLeftDiv.style.borderTopRightRadius="50px",this.edgeLeftDiv.style.borderBottomRightRadius="50px",this.edgeBottomDiv=this.currentDocument.createElement("div"),this.edgeBottomDiv.className=this.getClassName(f.CLASSES.FLEXLAYOUT__EDGE_RECT),this.edgeBottomDiv.style.bottom=t.height-n.getBottom()+"px",this.edgeBottomDiv.style.left=n.x+(n.width-100)/2+"px",this.edgeBottomDiv.style.width="100px",this.edgeBottomDiv.style.height="10px",this.edgeBottomDiv.style.borderTopLeftRadius="50px",this.edgeBottomDiv.style.borderTopRightRadius="50px",this.edgeRightDiv=this.currentDocument.createElement("div"),this.edgeRightDiv.className=this.getClassName(f.CLASSES.FLEXLAYOUT__EDGE_RECT),this.edgeRightDiv.style.top=n.y+(n.height-100)/2+"px",this.edgeRightDiv.style.right=t.width-n.getRight()+"px",this.edgeRightDiv.style.width="10px",this.edgeRightDiv.style.height="100px",this.edgeRightDiv.style.borderTopLeftRadius="50px",this.edgeRightDiv.style.borderBottomLeftRadius="50px",e.appendChild(this.edgeTopDiv),e.appendChild(this.edgeLeftDiv),e.appendChild(this.edgeBottomDiv),e.appendChild(this.edgeRightDiv)}},t.prototype.hideEdges=function(e){if(this.props.model.isEnableEdgeDock())try{e.removeChild(this.edgeTopDiv),e.removeChild(this.edgeLeftDiv),e.removeChild(this.edgeBottomDiv),e.removeChild(this.edgeRightDiv)}catch(e){}},t.prototype.maximize=function(e){this.doAction(l.default.maximizeToggle(e.getId()))},t.prototype.customizeTab=function(e,t){this.props.onRenderTab&&this.props.onRenderTab(e,t)},t.prototype.customizeTabSet=function(e,t){this.props.onRenderTabSet&&this.props.onRenderTabSet(e,t)},t.prototype.i18nName=function(e,t){var n;return this.props.i18nMapper&&(n=this.props.i18nMapper(e,t)),void 0===n&&(n=e+(void 0===t?"":t)),n},t}(o.Component);t.Layout=I,t.default=I},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.BorderTabSet=void 0;var r=n(0),i=n(190),o=n(1569),a=n(937),s=n(159),l=n(210),u=n(938),c=n(111),d=n(148);t.BorderTabSet=function(e){for(var t=e.border,n=e.layout,h=e.iconFactory,f=e.titleFactory,p=e.icons,g=r.useRef(null),v=r.useRef(null),m=u.useTabOverflow(t,c.default.flip(t.getOrientation()),g),y=m.selfRef,b=m.position,C=m.userControlledLeft,A=m.hiddenTabs,_=m.onMouseWheel,I=function(e){e.stopPropagation()},x=function(e){n.doAction(s.default.selectTab(e.node.getId())),C.current=!1},w=n.getClassName,M=t.getTabHeaderRect().styleWithPosition({}),S=[],T=function(e){var i=t.getSelected()===e,a=t.getChildren()[e];S.push(r.createElement(o.BorderButton,{layout:n,border:t.getLocation().getName(),node:a,key:a.getId(),selected:i,iconFactory:h,titleFactory:f,icons:p}))},E=0;E<t.getChildren().length;E++)T(E);var O=w(d.CLASSES.FLEXLAYOUT__BORDER)+" "+w(d.CLASSES.FLEXLAYOUT__BORDER_+t.getLocation().getName());void 0!==t.getClassName()&&(O+=" "+t.getClassName());var k,L=[],D={headerContent:{},buttons:L};if(n.customizeTabSet(t,D),L=D.buttons,A.length>0){var P=n.i18nName(l.I18nLabel.Overflow_Menu_Tooltip);L.push(r.createElement("button",{key:"overflowbutton",ref:v,className:w("flexlayout__border_toolbar_button_overflow")+" "+w("flexlayout__border_toolbar_button_overflow_"+t.getLocation().getName()),title:P,onClick:function(){var e=v.current;a.showPopup(n.getRootDiv(),e,A,x,n.getClassName)},onMouseDown:I,onTouchStart:I},null==p?void 0:p.more,A.length))}var N=t.getSelected();if(-1!==N){var j=t.getChildren()[N];if(void 0!==j&&n.isSupportsPopout()&&j.isEnableFloat()&&!j.isFloating()){var R=n.i18nName(l.I18nLabel.Float_Tab);L.push(r.createElement("button",{key:"float",title:R,className:w(d.CLASSES.FLEXLAYOUT__BORDER_TOOLBAR_BUTTON)+" "+w(d.CLASSES.FLEXLAYOUT__BORDER_TOOLBAR_BUTTON_FLOAT),onClick:function(){var e=t.getChildren()[t.getSelected()];void 0!==e&&n.doAction(s.default.floatTab(e.getId()))},onMouseDown:I,onTouchStart:I}))}}k=r.createElement("div",{key:"toolbar",ref:g,className:w(d.CLASSES.FLEXLAYOUT__BORDER_TOOLBAR)+" "+w(d.CLASSES.FLEXLAYOUT__BORDER_TOOLBAR_+t.getLocation().getName())},L),M=n.styleFont(M);var z={},F=t.getBorderBarSize()-1;return z=t.getLocation()===i.default.LEFT?{right:F,height:F,top:b}:t.getLocation()===i.default.RIGHT?{left:F,height:F,top:b}:{height:F,left:b},r.createElement("div",{ref:y,style:M,className:O,onWheel:_},r.createElement("div",{style:{height:F},className:w(d.CLASSES.FLEXLAYOUT__BORDER_INNER)+" "+w(d.CLASSES.FLEXLAYOUT__BORDER_INNER_+t.getLocation().getName())},r.createElement("div",{style:z,className:w(d.CLASSES.FLEXLAYOUT__BORDER_INNER_TAB_CONTAINER)+" "+w(d.CLASSES.FLEXLAYOUT__BORDER_INNER_TAB_CONTAINER_+t.getLocation().getName())},S)),k)}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.BorderButton=void 0;var r=n(0),i=n(514),o=n(159),a=n(131),s=n(936),l=n(148);t.BorderButton=function(e){var t=e.layout,n=e.node,u=e.selected,c=e.border,d=e.iconFactory,h=e.titleFactory,f=e.icons,p=r.useRef(null),g=function(r){var o=t.i18nName(i.I18nLabel.Move_Tab,n.getName());e.layout.dragStart(r,o,n,n.isEnableDrag(),v,(function(e){}))},v=function(){t.doAction(o.default.selectTab(n.getId()))},m=function(e){e.stopPropagation()};r.useLayoutEffect((function(){y()}));var y=function(){var e=t.getDomRect(),r=p.current.getBoundingClientRect();n._setTabRect(new a.default(r.left-e.left,r.top-e.top,r.width,r.height))},b=t.getClassName,C=b(l.CLASSES.FLEXLAYOUT__BORDER_BUTTON)+" "+b(l.CLASSES.FLEXLAYOUT__BORDER_BUTTON_+c);C+=u?" "+b(l.CLASSES.FLEXLAYOUT__BORDER_BUTTON__SELECTED):" "+b(l.CLASSES.FLEXLAYOUT__BORDER_BUTTON__UNSELECTED),void 0!==n.getClassName()&&(C+=" "+n.getClassName());var A=d?d(n):void 0,_=(h?h(n):void 0)||n.getName();void 0===typeof A&&void 0!==typeof n.getIcon()&&(A=r.createElement("img",{src:n.getIcon(),alt:"leadingContent"}));var I=[],x={leading:A,content:_,name:n.getName(),buttons:I};t.customizeTab(n,x),n._setRenderedName(x.name);var w=r.createElement("div",{className:b(l.CLASSES.FLEXLAYOUT__BORDER_BUTTON_CONTENT)},x.content),M=r.createElement("div",{className:b(l.CLASSES.FLEXLAYOUT__BORDER_BUTTON_LEADING)},x.leading);if(n.isEnableClose()){var S=t.i18nName(i.I18nLabel.Close_Tab);I.push(r.createElement("div",{key:"close",title:S,className:b(l.CLASSES.FLEXLAYOUT__BORDER_BUTTON_TRAILING),onMouseDown:m,onClick:function(e){var r;r=n.getCloseType(),u||r===s.ICloseType.Always||r===s.ICloseType.Visible&&window.matchMedia&&window.matchMedia("(hover: hover) and (pointer: fine)").matches?t.doAction(o.default.deleteTab(n.getId())):v()},onTouchStart:m},null==f?void 0:f.close))}return r.createElement("div",{ref:p,style:{},className:C,onMouseDown:g,onTouchStart:g},M,w,I)}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.Splitter=void 0;var r=n(0),i=n(515),o=n(159),a=n(243),s=n(111),l=n(148);t.Splitter=function(e){var t=e.layout,n=e.node,u=r.useRef([]),c=r.useRef(void 0),d=n.getParent(),h=function(e){i.default.instance.startDrag(e,p,g,v,f,void 0,void 0,t.getCurrentDocument()),u.current=d._getSplitterBounds(n,!0);var r=t.getRootDiv();c.current=t.getCurrentDocument().createElement("div"),c.current.style.position="absolute",c.current.className=t.getClassName(l.CLASSES.FLEXLAYOUT__SPLITTER_DRAG),c.current.style.cursor=n.getOrientation()===s.default.HORZ?"ns-resize":"ew-resize",n.getRect().positionElement(c.current),r.appendChild(c.current)},f=function(e){t.getRootDiv().removeChild(c.current)},p=function(){return!0},g=function(e){var r=t.getDomRect(),i=e.clientX-r.left,o=e.clientY-r.top;c&&(n.getOrientation()===s.default.HORZ?c.current.style.top=m(o-4)+"px":c.current.style.left=m(i-4)+"px")},v=function(){var e=0;if(c&&(e=n.getOrientation()===s.default.HORZ?c.current.offsetTop:c.current.offsetLeft),d instanceof a.default){var r=d._calculateSplit(n,e);t.doAction(o.default.adjustBorderSplit(n.getParent().getId(),r))}else{var i=d._calculateSplit(n,e);void 0!==i&&t.doAction(o.default.adjustSplit(i))}t.getRootDiv().removeChild(c.current)},m=function(e){var t=u.current,n=e;return e<t[0]&&(n=t[0]),e>t[1]&&(n=t[1]),n},y=t.getClassName,b=n._styleWithPosition({cursor:n.getOrientation()===s.default.HORZ?"ns-resize":"ew-resize"}),C=y(l.CLASSES.FLEXLAYOUT__SPLITTER)+" "+y(l.CLASSES.FLEXLAYOUT__SPLITTER_+n.getOrientation().getName());return d instanceof a.default?C+=" "+y(l.CLASSES.FLEXLAYOUT__SPLITTER_BORDER):void 0!==n.getModel().getMaximizedTabset()&&(b.display="none"),r.createElement("div",{style:b,onTouchStart:h,onMouseDown:h,className:C})}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.Tab=void 0;var r=n(0),i=n(0),o=n(159),a=n(191),s=n(148),l=n(939),u=n(210),c=n(514);t.Tab=function(e){var t=e.layout,n=e.selected,d=e.node,h=e.factory,f=r.useState(!e.node.isEnableRenderOnDemand()||e.selected),p=f[0],g=f[1];r.useLayoutEffect((function(){!p&&n&&g(!0)}));var v,m=function(){var e=d.getParent();e.getType()===a.default.TYPE&&(e.isActive()||t.doAction(o.default.setActiveTabset(e.getId())))},y=t.getClassName,b=d.getParent(),C=d._styleWithPosition({display:n?"block":"none"});b instanceof a.default&&(void 0===d.getModel().getMaximizedTabset()||b.isMaximized()||(C.display="none")),p&&(v=h(d));var A=y(s.CLASSES.FLEXLAYOUT__TAB);return b instanceof c.BorderNode&&(A+=" "+y(s.CLASSES.FLEXLAYOUT__TAB_BORDER),A+=" "+y(s.CLASSES.FLEXLAYOUT__TAB_BORDER_+b.getLocation().getName())),r.createElement("div",{className:A,onMouseDown:m,onTouchStart:m,style:C},r.createElement(l.ErrorBoundary,{message:e.layout.i18nName(u.I18nLabel.Error_rendering_component)},r.createElement(i.Fragment,null,v)))}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.TabSet=void 0;var r=n(0),i=n(210),o=n(159),a=n(937),s=n(1573),l=n(938),u=n(111),c=n(148);t.TabSet=function(e){var t=e.node,n=e.layout,d=e.iconFactory,h=e.titleFactory,f=e.icons,p=r.useRef(null),g=r.useRef(null),v=l.useTabOverflow(t,u.default.HORZ,p),m=v.selfRef,y=v.position,b=v.userControlledLeft,C=v.hiddenTabs,A=v.onMouseWheel,_=function(e){n.doAction(o.default.selectTab(e.node.getId())),b.current=!1},I=function(e){var r=t.getName();r=void 0===r?"":": "+r,n.doAction(o.default.setActiveTabset(t.getId()));var a=n.i18nName(i.I18nLabel.Move_Tabset,r);n.dragStart(e,a,t,t.isEnableDrag(),(function(e){}),M)},x=function(e){e.stopPropagation()},w=function(){return!!t.isEnableMaximize()&&(t.getModel().getMaximizedTabset()===t||(t.getParent()!==t.getModel().getRoot()||1!==t.getModel().getRoot().getChildren().length))},M=function(e){w()&&n.maximize(t)},S=n.getClassName,T=t.getSelectedNode(),E=t._styleWithPosition();void 0===t.getModel().getMaximizedTabset()||t.isMaximized()||(E.display="none");var O=[];if(t.isEnableTabStrip())for(var k=0;k<t.getChildren().length;k++){var L=t.getChildren()[k],D=t.getSelected()===k;O.push(r.createElement(s.TabButton,{layout:n,node:L,key:L.getId(),selected:D,show:!0,height:t.getTabStripHeight(),iconFactory:d,titleFactory:h,icons:f}))}var P=[],N={headerContent:t.getName(),buttons:P};n.customizeTabSet(t,N);var j,R=N.headerContent;if(P=N.buttons,C.length>0){var z=n.i18nName(i.I18nLabel.Overflow_Menu_Tooltip);P.push(r.createElement("button",{key:"overflowbutton",ref:g,className:S(c.CLASSES.FLEXLAYOUT__TAB_BUTTON_OVERFLOW),title:z,onClick:function(){var e=g.current;a.showPopup(n.getRootDiv(),e,C,_,n.getClassName)},onMouseDown:x,onTouchStart:x},null==f?void 0:f.more,C.length))}if(void 0!==T&&n.isSupportsPopout()&&T.isEnableFloat()&&!T.isFloating()){var F=n.i18nName(i.I18nLabel.Float_Tab);P.push(r.createElement("button",{key:"float",title:F,className:S(c.CLASSES.FLEXLAYOUT__TAB_TOOLBAR_BUTTON)+" "+S(c.CLASSES.FLEXLAYOUT__TAB_TOOLBAR_BUTTON_FLOAT),onClick:function(){void 0!==T&&n.doAction(o.default.floatTab(T.getId()))},onMouseDown:x,onTouchStart:x},null==f?void 0:f.popout))}if(w()){var B=n.i18nName(i.I18nLabel.Restore),H=n.i18nName(i.I18nLabel.Maximize);P.push(r.createElement("button",{key:"max",title:t.isMaximized()?B:H,className:S(c.CLASSES.FLEXLAYOUT__TAB_TOOLBAR_BUTTON)+" "+S(c.CLASSES.FLEXLAYOUT__TAB_TOOLBAR_BUTTON_+(t.isMaximized()?"max":"min")),onClick:function(){w()&&n.maximize(t)},onMouseDown:x,onTouchStart:x},t.isMaximized()?null==f?void 0:f.restore:null==f?void 0:f.maximize))}j=r.createElement("div",{key:"toolbar",ref:p,className:S(c.CLASSES.FLEXLAYOUT__TAB_TOOLBAR),onMouseDown:x},P);var U,V,Y=void 0!==t.getName(),W=S(c.CLASSES.FLEXLAYOUT__TABSET_TABBAR_OUTER);if(void 0!==t.getClassNameTabStrip()&&(W+=" "+t.getClassNameTabStrip()),W+=" "+c.CLASSES.FLEXLAYOUT__TABSET_TABBAR_OUTER_+t.getTabLocation(),t.isActive()&&!Y&&(W+=" "+S(c.CLASSES.FLEXLAYOUT__TABSET_SELECTED)),t.isMaximized()&&!Y&&(W+=" "+S(c.CLASSES.FLEXLAYOUT__TABSET_MAXIMIZED)),Y){var G=S(c.CLASSES.FLEXLAYOUT__TABSET_HEADER);t.isActive()&&(G+=" "+S(c.CLASSES.FLEXLAYOUT__TABSET_SELECTED)),t.isMaximized()&&(G+=" "+S(c.CLASSES.FLEXLAYOUT__TABSET_MAXIMIZED)),void 0!==t.getClassNameHeader()&&(G+=" "+t.getClassNameHeader()),U=r.createElement("div",{className:G,style:{height:t.getHeaderHeight()+"px"},onMouseDown:I,onTouchStart:I},r.createElement("div",{className:S(c.CLASSES.FLEXLAYOUT__TABSET_HEADER_CONTENT)},R),j);var X={height:t.getTabStripHeight()+"px"};"top"===t.getTabLocation()?X.top=t.getHeaderHeight()+"px":X.bottom="0px",V=r.createElement("div",{className:W,style:X,onMouseDown:I,onTouchStart:I},r.createElement("div",{className:S(c.CLASSES.FLEXLAYOUT__TABSET_TABBAR_INNER)+" "+S(c.CLASSES.FLEXLAYOUT__TABSET_TABBAR_INNER_+t.getTabLocation())},r.createElement("div",{style:{left:y},className:S(c.CLASSES.FLEXLAYOUT__TABSET_TABBAR_INNER_TAB_CONTAINER)+" "+S(c.CLASSES.FLEXLAYOUT__TABSET_TABBAR_INNER_TAB_CONTAINER_+t.getTabLocation())},O)))}else{X={height:t.getTabStripHeight()+"px"};"top"===t.getTabLocation()?X.top="0px":X.bottom="0px",V=r.createElement("div",{className:W,style:X,onMouseDown:I,onTouchStart:I},r.createElement("div",{className:S(c.CLASSES.FLEXLAYOUT__TABSET_TABBAR_INNER)+" "+S(c.CLASSES.FLEXLAYOUT__TABSET_TABBAR_INNER_+t.getTabLocation())},r.createElement("div",{style:{left:y},className:S(c.CLASSES.FLEXLAYOUT__TABSET_TABBAR_INNER_TAB_CONTAINER)+" "+S(c.CLASSES.FLEXLAYOUT__TABSET_TABBAR_INNER_TAB_CONTAINER_+t.getTabLocation())},O)),j)}return E=n.styleFont(E),r.createElement("div",{ref:m,style:E,className:S(c.CLASSES.FLEXLAYOUT__TABSET),onWheel:A},U,V)}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.TabButton=void 0;var r=n(0),i=n(210),o=n(159),a=n(131),s=n(936),l=n(148);t.TabButton=function(e){var t=e.layout,n=e.node,u=e.show,c=e.selected,d=e.iconFactory,h=e.titleFactory,f=e.icons,p=r.useRef(null),g=r.useRef(null),v=r.useRef(0),m=r.useState(!1),y=m[0],b=m[1],C=function(e){var r=t.i18nName(i.I18nLabel.Move_Tab,n.getName());t.dragStart(e,r,n,n.isEnableDrag(),A,_)},A=function(){t.doAction(o.default.selectTab(n.getId()))},_=function(e){if(n.isEnableRename())b(!0),t.getCurrentDocument().body.addEventListener("mousedown",I),t.getCurrentDocument().body.addEventListener("touchstart",I);else{var r=n.getParent();r.isEnableMaximize()&&t.maximize(r)}},I=function(e){e.target!==g.current&&(b(!1),t.getCurrentDocument().body.removeEventListener("mousedown",I),t.getCurrentDocument().body.removeEventListener("touchstart",I))},x=function(e){e.stopPropagation()};r.useLayoutEffect((function(){w(),y&&g.current.select()}));var w=function(){var e=t.getDomRect(),r=p.current.getBoundingClientRect();n._setTabRect(new a.default(r.left-e.left,r.top-e.top,r.width,r.height)),v.current=g.current.getBoundingClientRect().width},M=function(e){e.stopPropagation()},S=t.getClassName,T=n.getParent(),E=l.CLASSES.FLEXLAYOUT__TAB_BUTTON,O=S(E);O+=" "+S(E+"_"+T.getTabLocation()),O+=c?" "+S(E+"--selected"):" "+S(E+"--unselected"),void 0!==n.getClassName()&&(O+=" "+n.getClassName());var k=d?d(n):void 0,L=(h?h(n):void 0)||n.getName();void 0===typeof k&&void 0!==typeof n.getIcon()&&(k=r.createElement("img",{src:n.getIcon(),alt:"leadingContent"}));var D=[],P={leading:k,content:L,name:n.getName(),buttons:D};t.customizeTab(n,P),n._setRenderedName(P.name);var N=r.createElement("div",{ref:g,className:S(l.CLASSES.FLEXLAYOUT__TAB_BUTTON_CONTENT)},P.content),j=r.createElement("div",{className:S(l.CLASSES.FLEXLAYOUT__TAB_BUTTON_LEADING)},P.leading);if(y){var R={width:v+"px"};N=r.createElement("input",{style:R,ref:g,className:S(l.CLASSES.FLEXLAYOUT__TAB_BUTTON_TEXTBOX),type:"text",autoFocus:!0,defaultValue:n.getName(),onKeyDown:function(e){27===e.keyCode?b(!1):13===e.keyCode&&(b(!1),t.doAction(o.default.renameTab(n.getId(),e.target.value)))},onMouseDown:M,onTouchStart:M})}if(n.isEnableClose()){var z=t.i18nName(i.I18nLabel.Close_Tab);D.push(r.createElement("div",{key:"close",title:z,className:S(l.CLASSES.FLEXLAYOUT__TAB_BUTTON_TRAILING),onMouseDown:x,onClick:function(e){var r;r=n.getCloseType(),c||r===s.ICloseType.Always||r===s.ICloseType.Visible&&window.matchMedia&&window.matchMedia("(hover: hover) and (pointer: fine)").matches?t.doAction(o.default.deleteTab(n.getId())):A()},onTouchStart:x},null==f?void 0:f.close))}return r.createElement("div",{ref:p,style:{visibility:u?"visible":"hidden"},className:O,onMouseDown:C,onTouchStart:C},j,N,D)}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.FloatingWindow=void 0;var r=n(0),i=n(22),o=n(148);t.FloatingWindow=function(e){var t=e.title,n=e.id,a=e.url,s=e.rect,l=e.onCloseWindow,u=e.onSetWindow,c=e.children,d=r.useRef(null),h=r.useState(void 0),f=h[0],p=h[1];return r.useLayoutEffect((function(){var e=s;return d.current=window.open(a,n,"left="+e.x+",top="+e.y+",width="+e.width+",height="+e.height),null!==d.current?(u(n,d.current),window.addEventListener("beforeunload",(function(){d.current&&(d.current.close(),d.current=null)})),d.current.addEventListener("load",(function(){var e=d.current.document;e.title=t;var r,i,a,s=e.createElement("div");s.className=o.CLASSES.FLEXLAYOUT__FLOATING_WINDOW_CONTENT,e.body.appendChild(s),(r=e,i=r.head,a=[],Array.from(window.document.styleSheets).forEach((function(e){if(e.href){var t=r.createElement("link");t.type=e.type,t.rel="stylesheet",t.href=e.href,i.appendChild(t),a.push(new Promise((function(e,n){t.onload=function(){return e(!0)}})))}else try{var n=e.cssRules,o=r.createElement("style");Array.from(n).forEach((function(e){o.appendChild(r.createTextNode(e.cssText))})),i.appendChild(o)}catch(e){}})),Promise.all(a)).then((function(){p(s)})),d.current.addEventListener("beforeunload",(function(){l(n)}))}))):(console.warn("Unable to open window "+a),l(n)),function(){setTimeout((function(){d.current&&(d.current.close(),d.current=null)}),0)}}),[]),void 0!==f?i.createPortal(c,f):null}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.FloatingWindowTab=void 0;var r=n(0),i=n(939),o=n(210),a=n(0),s=n(148);t.FloatingWindowTab=function(e){var t=e.layout,n=e.node,l=e.factory,u=t.getClassName,c=l(n);return r.createElement("div",{className:u(s.CLASSES.FLEXLAYOUT__FLOATING_WINDOW_TAB)},r.createElement(i.ErrorBoundary,{message:e.layout.i18nName(o.I18nLabel.Error_rendering_component)},r.createElement(a.Fragment,null,c)))}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.TabFloating=void 0;var r=n(0),i=n(159),o=n(191),a=n(148),s=n(210);t.TabFloating=function(e){var t=e.layout,n=e.selected,l=e.node,u=function(){var e=l.getParent();e.getType()===o.default.TYPE&&(e.isActive()||t.doAction(i.default.setActiveTabset(e.getId())))},c=t.getClassName,d=l._styleWithPosition({display:n?"flex":"none"}),h=t.i18nName(s.I18nLabel.Floating_Window_Message),f=t.i18nName(s.I18nLabel.Floating_Window_Show_Window),p=t.i18nName(s.I18nLabel.Floating_Window_Dock_Window);return r.createElement("div",{className:c(a.CLASSES.FLEXLAYOUT__TAB_FLOATING),onMouseDown:u,onTouchStart:u,style:d},r.createElement("div",{className:c(a.CLASSES.FLEXLAYOUT__TAB_FLOATING_INNER)},r.createElement("div",null,h),r.createElement("div",null,r.createElement("a",{href:"#",onClick:function(e){e.preventDefault(),l.getWindow()&&l.getWindow().focus()}},f)),r.createElement("div",null,r.createElement("a",{href:"#",onClick:function(e){e.preventDefault(),t.doAction(i.default.unFloatTab(l.getId()))}},p))))}},function(e,t,n){"use strict";
/** @license React v16.13.1
 * react-is.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var r="function"==typeof Symbol&&Symbol.for,i=r?Symbol.for("react.element"):60103,o=r?Symbol.for("react.portal"):60106,a=r?Symbol.for("react.fragment"):60107,s=r?Symbol.for("react.strict_mode"):60108,l=r?Symbol.for("react.profiler"):60114,u=r?Symbol.for("react.provider"):60109,c=r?Symbol.for("react.context"):60110,d=r?Symbol.for("react.async_mode"):60111,h=r?Symbol.for("react.concurrent_mode"):60111,f=r?Symbol.for("react.forward_ref"):60112,p=r?Symbol.for("react.suspense"):60113,g=r?Symbol.for("react.suspense_list"):60120,v=r?Symbol.for("react.memo"):60115,m=r?Symbol.for("react.lazy"):60116,y=r?Symbol.for("react.block"):60121,b=r?Symbol.for("react.fundamental"):60117,C=r?Symbol.for("react.responder"):60118,A=r?Symbol.for("react.scope"):60119;function _(e){if("object"==typeof e&&null!==e){var t=e.$$typeof;switch(t){case i:switch(e=e.type){case d:case h:case a:case l:case s:case p:return e;default:switch(e=e&&e.$$typeof){case c:case f:case m:case v:case u:return e;default:return t}}case o:return t}}}function I(e){return _(e)===h}t.AsyncMode=d,t.ConcurrentMode=h,t.ContextConsumer=c,t.ContextProvider=u,t.Element=i,t.ForwardRef=f,t.Fragment=a,t.Lazy=m,t.Memo=v,t.Portal=o,t.Profiler=l,t.StrictMode=s,t.Suspense=p,t.isAsyncMode=function(e){return I(e)||_(e)===d},t.isConcurrentMode=I,t.isContextConsumer=function(e){return _(e)===c},t.isContextProvider=function(e){return _(e)===u},t.isElement=function(e){return"object"==typeof e&&null!==e&&e.$$typeof===i},t.isForwardRef=function(e){return _(e)===f},t.isFragment=function(e){return _(e)===a},t.isLazy=function(e){return _(e)===m},t.isMemo=function(e){return _(e)===v},t.isPortal=function(e){return _(e)===o},t.isProfiler=function(e){return _(e)===l},t.isStrictMode=function(e){return _(e)===s},t.isSuspense=function(e){return _(e)===p},t.isValidElementType=function(e){return"string"==typeof e||"function"==typeof e||e===a||e===h||e===l||e===s||e===p||e===g||"object"==typeof e&&null!==e&&(e.$$typeof===m||e.$$typeof===v||e.$$typeof===u||e.$$typeof===c||e.$$typeof===f||e.$$typeof===b||e.$$typeof===C||e.$$typeof===A||e.$$typeof===y)},t.typeOf=_},function(e,t){e.exports=function(e){if(!e.webpackPolyfill){var t=Object.create(e);t.children||(t.children=[]),Object.defineProperty(t,"loaded",{enumerable:!0,get:function(){return t.l}}),Object.defineProperty(t,"id",{enumerable:!0,get:function(){return t.i}}),Object.defineProperty(t,"exports",{enumerable:!0}),t.webpackPolyfill=1}return t}},function(e,t,n){var r,i,o=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),a=this&&this.__setModuleDefault||(Object.create?function(e,t){Object.defineProperty(e,"default",{enumerable:!0,value:t})}:function(e,t){e.default=t}),s=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)"default"!==n&&Object.hasOwnProperty.call(e,n)&&o(t,e,n);return a(t,e),t},l=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};r=[n,t,n(0),n(90),n(78),n(176),n(75),n(961),n(519)],void 0===(i=function(e,t,n,r,i,o,a,u,c){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.PBSElement=void 0,n=s(n),o=s(o),a=l(a),u=l(u);const d=function(e,t,n){return[["M",e-n,t-n],["a",n,n,0,1,0,2*n,0],["a",n,n,0,1,0,2*-n,0],["M",e+2-n,t-n],["L",e+n-2,t-n]]},h=function(e,t,n){return[["M",e-n,t-n],["a",n,n,0,1,0,2*n,0],["a",n,n,0,1,0,2*-n,0],["M",e+2-n,t-n],["L",e+n-2,t-n],["M",e,t-2*n+2],["L",e,t-2]]};class f extends n.Component{constructor(e){super(e),this.handleResize=()=>{if(this._divRef.current){const e=this._divRef.current.clientWidth,t=this._divRef.current.clientHeight;this.graph&&(this.graph.changeSize(e,t),this.graph.fitView(),this.graph.refresh())}},this.createGraph=()=>{let e={};this.props.pbsData.nodeData.length>0&&(e=JSON.parse(JSON.stringify(this.props.pbsData.nodeData[0])));const t=this._divRef.current.clientWidth,n=this._divRef.current.clientHeight;this.graph=new u.default.TreeGraph({container:this._divRef.current,width:t,height:n,modes:{default:[{type:"collapse-expand",onChange:function(e,t){const n=e.get("model"),r=e.get("group").find(e=>"collapse-icon"===e.get("name"));return t?r.attr("symbol",h):r.attr("symbol",d),n.collapsed=t,!0}},"drag-canvas","zoom-canvas","drag-node"]},defaultNode:{type:"tree-node",anchorPoints:[[.5,1],[.5,0]]},defaultEdge:{type:"flow-line",style:{stroke:"#91d5ff",endArrow:!0}},fitView:!0,layout:{type:"compactBox",direction:"TB",getId:function(e){return e.id},getHeight:function(){return 16},getWidth:function(){return 16},getVGap:function(){return 40},getHGap:function(){return 40}}}),this.graph.data(e),this.graph.on("afterlayout",this.saveNodePosition),this.graph.render(),this.graph.layout(!0),this.updated=!0},this.saveNodePosition=()=>{const e=this.graph.getNodes();let t=!1;this.graph.get("layout").radial&&(t=!0);const n={__$graph_style$__:{visible:t,position:[0,0]}};for(const e in this.props.positionData)"__$graph_style$__"!==e&&(n[e]=Object.assign(Object.assign({},this.props.positionData[e]),{position:null}));e.forEach(e=>{const t=e.get("model");n[t.id]?n[t.id].position=[t.x,t.y]:n[t.id]={visible:!0,position:[t.x,t.y]}}),this.props.saveGraphPosition(n)},this.expandGraph=()=>{this.graph.getNodes().forEach(e=>{const t=e.get("model"),n=e.get("group").find(e=>"collapse-icon"===e.get("name"));n&&n.attr("symbol",d),t.collapsed=!1}),this.graph.layout(!0)},this.resetGraph=()=>{this.graph.getNodes().forEach(e=>{const t=e.get("model");if(2===t.id.split(".").length){const n=e.get("group").find(e=>"collapse-icon"===e.get("name"));n&&n.attr("symbol",h),t.collapsed=!0}}),this.graph.layout(!0)},this.radialLayout=()=>{this.graph.updateLayout({direction:"RL",getId:function(e){return e.id},getHeight:()=>26,getWidth:()=>26,getVGap:()=>20,getHGap:()=>20,radial:!0}),this.graph.fitView(),this.graph.refresh()},this.flatLayout=()=>{this.graph.updateLayout({type:"compactBox",direction:"TB",getId:function(e){return e.id},getHeight:function(){return 16},getWidth:function(){return 16},getVGap:function(){return 40},getHGap:function(){return 40}}),this.graph.fitView(),this.graph.refresh()},this._divRef=n.default.createRef(),this.updated=!1,window.onresize=c.debounce(this.handleResize,500)}componentDidMount(){u.default.registerNode("tree-node",{getAnchorPoints:function(){return[[.5,1],[.5,0]]},drawShape:function(e,t){const n=t.addShape("rect",{attrs:{fill:"rgb(0, 134, 178)",stroke:"black"},name:"rect-shape"}),r=e.id.split("."),i=r[r.length-1],o=t.addShape("text",{attrs:{text:i,x:0,y:0,textAlign:"left",textBaseline:"middle",fill:"#fff",fontFamily:"sans-serif",fontSize:11,fontWeight:400},name:"rect-shape"}).getBBox(),a=e.children&&e.children.length>0;let s;return a&&t.addShape("marker",{attrs:{x:o.width/2,y:o.height/2+12,r:5,symbol:e.collapsed?h:d,stroke:"#fff",lineWidth:2},name:"collapse-icon"}),s=o.width<50?(50-o.width)/2:5,n.attr({x:o.minX-s,y:o.minY-6,width:o.width+2*s,height:o.height+(a?22:12)}),n}},"single-node"),u.default.registerEdge("flow-line",{draw(e,t){const n=e.startPoint,r=e.endPoint,{style:i}=e;return t.addShape("path",{attrs:{stroke:i.stroke,endArrow:i.endArrow,path:[["M",n.x,n.y],["L",n.x,(n.y+r.y)/2],["L",r.x,(n.y+r.y)/2],["L",r.x,r.y]]}})}}),setTimeout(()=>{this.createGraph()},500)}componentDidUpdate(e,t){if(e.systemPBSUpdated!==this.props.systemPBSUpdated){const e=JSON.parse(JSON.stringify(this.props.pbsData.nodeData[0]));u.default.Util.traverseTreeUp(e,e=>{if(e.children){const t=[];e.children.forEach((e,n)=>{this.props.positionData[e.id].visible||t.push(n)});for(let n=t.length-1;n>=0;n--)e.children.splice(t[n],1)}return!0}),this.graph.changeData(e)}}render(){return n.default.createElement("div",{className:"cosapp-widget-box"},n.default.createElement("div",{ref:this._divRef,style:{height:"calc(100% - 30px)"},className:"pbsView"}),n.default.createElement("div",{style:{height:"30px",display:"flex",background:"#e0e0e0"}},n.default.createElement(a.default,{onClick:this.flatLayout},"Flat layout"),n.default.createElement(a.default,{onClick:this.radialLayout},"Radial layout")))}}t.PBSElement=f,t.default=r.withStyles(e=>({textColor:{color:"rgb(250, 250, 250)",background:"#525354",margin:"0px 5px"}}))(i.connect(e=>(e=>({pbsData:e.systemArch.systemTree,positionData:e.systemArch.systemPBS,systemPBSUpdated:e.systemArch.systemPBSUpdated}))(e),e=>({saveGraphPosition:t=>e(o.archSaveGraphPosition(t))}))(f))}.apply(t,r))||(e.exports=i)},function(e,t,n){var r=n(941);e.exports=function(e){if(Array.isArray(e))return r(e)}},function(e,t){e.exports=function(e){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e))return Array.from(e)}},function(e,t){e.exports=function(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}},function(e,t){e.exports=function(e){if(Array.isArray(e))return e}},function(e,t){e.exports=function(e,t){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(e)){var n=[],r=!0,i=!1,o=void 0;try{for(var a,s=e[Symbol.iterator]();!(r=(a=s.next()).done)&&(n.push(a.value),!t||n.length!==t);r=!0);}catch(e){i=!0,o=e}finally{try{r||null==s.return||s.return()}finally{if(i)throw o}}return n}}},function(e,t){e.exports=function(){throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}},function(e,t,n){"use strict";
/** @license React v16.14.0
 * react.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var r=n(365),i="function"==typeof Symbol&&Symbol.for,o=i?Symbol.for("react.element"):60103,a=i?Symbol.for("react.portal"):60106,s=i?Symbol.for("react.fragment"):60107,l=i?Symbol.for("react.strict_mode"):60108,u=i?Symbol.for("react.profiler"):60114,c=i?Symbol.for("react.provider"):60109,d=i?Symbol.for("react.context"):60110,h=i?Symbol.for("react.forward_ref"):60112,f=i?Symbol.for("react.suspense"):60113,p=i?Symbol.for("react.memo"):60115,g=i?Symbol.for("react.lazy"):60116,v="function"==typeof Symbol&&Symbol.iterator;function m(e){for(var t="https://reactjs.org/docs/error-decoder.html?invariant="+e,n=1;n<arguments.length;n++)t+="&args[]="+encodeURIComponent(arguments[n]);return"Minified React error #"+e+"; visit "+t+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}var y={isMounted:function(){return!1},enqueueForceUpdate:function(){},enqueueReplaceState:function(){},enqueueSetState:function(){}},b={};function C(e,t,n){this.props=e,this.context=t,this.refs=b,this.updater=n||y}function A(){}function _(e,t,n){this.props=e,this.context=t,this.refs=b,this.updater=n||y}C.prototype.isReactComponent={},C.prototype.setState=function(e,t){if("object"!=typeof e&&"function"!=typeof e&&null!=e)throw Error(m(85));this.updater.enqueueSetState(this,e,t,"setState")},C.prototype.forceUpdate=function(e){this.updater.enqueueForceUpdate(this,e,"forceUpdate")},A.prototype=C.prototype;var I=_.prototype=new A;I.constructor=_,r(I,C.prototype),I.isPureReactComponent=!0;var x={current:null},w=Object.prototype.hasOwnProperty,M={key:!0,ref:!0,__self:!0,__source:!0};function S(e,t,n){var r,i={},a=null,s=null;if(null!=t)for(r in void 0!==t.ref&&(s=t.ref),void 0!==t.key&&(a=""+t.key),t)w.call(t,r)&&!M.hasOwnProperty(r)&&(i[r]=t[r]);var l=arguments.length-2;if(1===l)i.children=n;else if(1<l){for(var u=Array(l),c=0;c<l;c++)u[c]=arguments[c+2];i.children=u}if(e&&e.defaultProps)for(r in l=e.defaultProps)void 0===i[r]&&(i[r]=l[r]);return{$$typeof:o,type:e,key:a,ref:s,props:i,_owner:x.current}}function T(e){return"object"==typeof e&&null!==e&&e.$$typeof===o}var E=/\/+/g,O=[];function k(e,t,n,r){if(O.length){var i=O.pop();return i.result=e,i.keyPrefix=t,i.func=n,i.context=r,i.count=0,i}return{result:e,keyPrefix:t,func:n,context:r,count:0}}function L(e){e.result=null,e.keyPrefix=null,e.func=null,e.context=null,e.count=0,10>O.length&&O.push(e)}function D(e,t,n){return null==e?0:function e(t,n,r,i){var s=typeof t;"undefined"!==s&&"boolean"!==s||(t=null);var l=!1;if(null===t)l=!0;else switch(s){case"string":case"number":l=!0;break;case"object":switch(t.$$typeof){case o:case a:l=!0}}if(l)return r(i,t,""===n?"."+P(t,0):n),1;if(l=0,n=""===n?".":n+":",Array.isArray(t))for(var u=0;u<t.length;u++){var c=n+P(s=t[u],u);l+=e(s,c,r,i)}else if(null===t||"object"!=typeof t?c=null:c="function"==typeof(c=v&&t[v]||t["@@iterator"])?c:null,"function"==typeof c)for(t=c.call(t),u=0;!(s=t.next()).done;)l+=e(s=s.value,c=n+P(s,u++),r,i);else if("object"===s)throw r=""+t,Error(m(31,"[object Object]"===r?"object with keys {"+Object.keys(t).join(", ")+"}":r,""));return l}(e,"",t,n)}function P(e,t){return"object"==typeof e&&null!==e&&null!=e.key?function(e){var t={"=":"=0",":":"=2"};return"$"+(""+e).replace(/[=:]/g,(function(e){return t[e]}))}(e.key):t.toString(36)}function N(e,t){e.func.call(e.context,t,e.count++)}function j(e,t,n){var r=e.result,i=e.keyPrefix;e=e.func.call(e.context,t,e.count++),Array.isArray(e)?R(e,r,n,(function(e){return e})):null!=e&&(T(e)&&(e=function(e,t){return{$$typeof:o,type:e.type,key:t,ref:e.ref,props:e.props,_owner:e._owner}}(e,i+(!e.key||t&&t.key===e.key?"":(""+e.key).replace(E,"$&/")+"/")+n)),r.push(e))}function R(e,t,n,r,i){var o="";null!=n&&(o=(""+n).replace(E,"$&/")+"/"),D(e,j,t=k(t,o,r,i)),L(t)}var z={current:null};function F(){var e=z.current;if(null===e)throw Error(m(321));return e}var B={ReactCurrentDispatcher:z,ReactCurrentBatchConfig:{suspense:null},ReactCurrentOwner:x,IsSomeRendererActing:{current:!1},assign:r};t.Children={map:function(e,t,n){if(null==e)return e;var r=[];return R(e,r,null,t,n),r},forEach:function(e,t,n){if(null==e)return e;D(e,N,t=k(null,null,t,n)),L(t)},count:function(e){return D(e,(function(){return null}),null)},toArray:function(e){var t=[];return R(e,t,null,(function(e){return e})),t},only:function(e){if(!T(e))throw Error(m(143));return e}},t.Component=C,t.Fragment=s,t.Profiler=u,t.PureComponent=_,t.StrictMode=l,t.Suspense=f,t.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED=B,t.cloneElement=function(e,t,n){if(null==e)throw Error(m(267,e));var i=r({},e.props),a=e.key,s=e.ref,l=e._owner;if(null!=t){if(void 0!==t.ref&&(s=t.ref,l=x.current),void 0!==t.key&&(a=""+t.key),e.type&&e.type.defaultProps)var u=e.type.defaultProps;for(c in t)w.call(t,c)&&!M.hasOwnProperty(c)&&(i[c]=void 0===t[c]&&void 0!==u?u[c]:t[c])}var c=arguments.length-2;if(1===c)i.children=n;else if(1<c){u=Array(c);for(var d=0;d<c;d++)u[d]=arguments[d+2];i.children=u}return{$$typeof:o,type:e.type,key:a,ref:s,props:i,_owner:l}},t.createContext=function(e,t){return void 0===t&&(t=null),(e={$$typeof:d,_calculateChangedBits:t,_currentValue:e,_currentValue2:e,_threadCount:0,Provider:null,Consumer:null}).Provider={$$typeof:c,_context:e},e.Consumer=e},t.createElement=S,t.createFactory=function(e){var t=S.bind(null,e);return t.type=e,t},t.createRef=function(){return{current:null}},t.forwardRef=function(e){return{$$typeof:h,render:e}},t.isValidElement=T,t.lazy=function(e){return{$$typeof:g,_ctor:e,_status:-1,_result:null}},t.memo=function(e,t){return{$$typeof:p,type:e,compare:void 0===t?null:t}},t.useCallback=function(e,t){return F().useCallback(e,t)},t.useContext=function(e,t){return F().useContext(e,t)},t.useDebugValue=function(){},t.useEffect=function(e,t){return F().useEffect(e,t)},t.useImperativeHandle=function(e,t,n){return F().useImperativeHandle(e,t,n)},t.useLayoutEffect=function(e,t){return F().useLayoutEffect(e,t)},t.useMemo=function(e,t){return F().useMemo(e,t)},t.useReducer=function(e,t,n){return F().useReducer(e,t,n)},t.useRef=function(e){return F().useRef(e)},t.useState=function(e){return F().useState(e)},t.version="16.14.0"},function(e,t,n){"use strict";!function e(){if("undefined"!=typeof __REACT_DEVTOOLS_GLOBAL_HOOK__&&"function"==typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE){0;try{__REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE(e)}catch(e){console.error(e)}}}(),e.exports=n(1588)},function(e,t,n){"use strict";
/** @license React v16.14.0
 * react-dom.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var r=n(943),i=n(365),o=n(1589);function a(e){for(var t="https://reactjs.org/docs/error-decoder.html?invariant="+e,n=1;n<arguments.length;n++)t+="&args[]="+encodeURIComponent(arguments[n]);return"Minified React error #"+e+"; visit "+t+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}if(!r)throw Error(a(227));function s(e,t,n,r,i,o,a,s,l){var u=Array.prototype.slice.call(arguments,3);try{t.apply(n,u)}catch(e){this.onError(e)}}var l=!1,u=null,c=!1,d=null,h={onError:function(e){l=!0,u=e}};function f(e,t,n,r,i,o,a,c,d){l=!1,u=null,s.apply(h,arguments)}var p=null,g=null,v=null;function m(e,t,n){var r=e.type||"unknown-event";e.currentTarget=v(n),function(e,t,n,r,i,o,s,h,p){if(f.apply(this,arguments),l){if(!l)throw Error(a(198));var g=u;l=!1,u=null,c||(c=!0,d=g)}}(r,t,void 0,e),e.currentTarget=null}var y=null,b={};function C(){if(y)for(var e in b){var t=b[e],n=y.indexOf(e);if(!(-1<n))throw Error(a(96,e));if(!_[n]){if(!t.extractEvents)throw Error(a(97,e));for(var r in _[n]=t,n=t.eventTypes){var i=void 0,o=n[r],s=t,l=r;if(I.hasOwnProperty(l))throw Error(a(99,l));I[l]=o;var u=o.phasedRegistrationNames;if(u){for(i in u)u.hasOwnProperty(i)&&A(u[i],s,l);i=!0}else o.registrationName?(A(o.registrationName,s,l),i=!0):i=!1;if(!i)throw Error(a(98,r,e))}}}}function A(e,t,n){if(x[e])throw Error(a(100,e));x[e]=t,w[e]=t.eventTypes[n].dependencies}var _=[],I={},x={},w={};function M(e){var t,n=!1;for(t in e)if(e.hasOwnProperty(t)){var r=e[t];if(!b.hasOwnProperty(t)||b[t]!==r){if(b[t])throw Error(a(102,t));b[t]=r,n=!0}}n&&C()}var S=!("undefined"==typeof window||void 0===window.document||void 0===window.document.createElement),T=null,E=null,O=null;function k(e){if(e=g(e)){if("function"!=typeof T)throw Error(a(280));var t=e.stateNode;t&&(t=p(t),T(e.stateNode,e.type,t))}}function L(e){E?O?O.push(e):O=[e]:E=e}function D(){if(E){var e=E,t=O;if(O=E=null,k(e),t)for(e=0;e<t.length;e++)k(t[e])}}function P(e,t){return e(t)}function N(e,t,n,r,i){return e(t,n,r,i)}function j(){}var R=P,z=!1,F=!1;function B(){null===E&&null===O||(j(),D())}function H(e,t,n){if(F)return e(t,n);F=!0;try{return R(e,t,n)}finally{F=!1,B()}}var U=/^[:A-Z_a-z\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u02FF\u0370-\u037D\u037F-\u1FFF\u200C-\u200D\u2070-\u218F\u2C00-\u2FEF\u3001-\uD7FF\uF900-\uFDCF\uFDF0-\uFFFD][:A-Z_a-z\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u02FF\u0370-\u037D\u037F-\u1FFF\u200C-\u200D\u2070-\u218F\u2C00-\u2FEF\u3001-\uD7FF\uF900-\uFDCF\uFDF0-\uFFFD\-.0-9\u00B7\u0300-\u036F\u203F-\u2040]*$/,V=Object.prototype.hasOwnProperty,Y={},W={};function G(e,t,n,r,i,o){this.acceptsBooleans=2===t||3===t||4===t,this.attributeName=r,this.attributeNamespace=i,this.mustUseProperty=n,this.propertyName=e,this.type=t,this.sanitizeURL=o}var X={};"children dangerouslySetInnerHTML defaultValue defaultChecked innerHTML suppressContentEditableWarning suppressHydrationWarning style".split(" ").forEach((function(e){X[e]=new G(e,0,!1,e,null,!1)})),[["acceptCharset","accept-charset"],["className","class"],["htmlFor","for"],["httpEquiv","http-equiv"]].forEach((function(e){var t=e[0];X[t]=new G(t,1,!1,e[1],null,!1)})),["contentEditable","draggable","spellCheck","value"].forEach((function(e){X[e]=new G(e,2,!1,e.toLowerCase(),null,!1)})),["autoReverse","externalResourcesRequired","focusable","preserveAlpha"].forEach((function(e){X[e]=new G(e,2,!1,e,null,!1)})),"allowFullScreen async autoFocus autoPlay controls default defer disabled disablePictureInPicture formNoValidate hidden loop noModule noValidate open playsInline readOnly required reversed scoped seamless itemScope".split(" ").forEach((function(e){X[e]=new G(e,3,!1,e.toLowerCase(),null,!1)})),["checked","multiple","muted","selected"].forEach((function(e){X[e]=new G(e,3,!0,e,null,!1)})),["capture","download"].forEach((function(e){X[e]=new G(e,4,!1,e,null,!1)})),["cols","rows","size","span"].forEach((function(e){X[e]=new G(e,6,!1,e,null,!1)})),["rowSpan","start"].forEach((function(e){X[e]=new G(e,5,!1,e.toLowerCase(),null,!1)}));var q=/[\-:]([a-z])/g;function Z(e){return e[1].toUpperCase()}"accent-height alignment-baseline arabic-form baseline-shift cap-height clip-path clip-rule color-interpolation color-interpolation-filters color-profile color-rendering dominant-baseline enable-background fill-opacity fill-rule flood-color flood-opacity font-family font-size font-size-adjust font-stretch font-style font-variant font-weight glyph-name glyph-orientation-horizontal glyph-orientation-vertical horiz-adv-x horiz-origin-x image-rendering letter-spacing lighting-color marker-end marker-mid marker-start overline-position overline-thickness paint-order panose-1 pointer-events rendering-intent shape-rendering stop-color stop-opacity strikethrough-position strikethrough-thickness stroke-dasharray stroke-dashoffset stroke-linecap stroke-linejoin stroke-miterlimit stroke-opacity stroke-width text-anchor text-decoration text-rendering underline-position underline-thickness unicode-bidi unicode-range units-per-em v-alphabetic v-hanging v-ideographic v-mathematical vector-effect vert-adv-y vert-origin-x vert-origin-y word-spacing writing-mode xmlns:xlink x-height".split(" ").forEach((function(e){var t=e.replace(q,Z);X[t]=new G(t,1,!1,e,null,!1)})),"xlink:actuate xlink:arcrole xlink:role xlink:show xlink:title xlink:type".split(" ").forEach((function(e){var t=e.replace(q,Z);X[t]=new G(t,1,!1,e,"http://www.w3.org/1999/xlink",!1)})),["xml:base","xml:lang","xml:space"].forEach((function(e){var t=e.replace(q,Z);X[t]=new G(t,1,!1,e,"http://www.w3.org/XML/1998/namespace",!1)})),["tabIndex","crossOrigin"].forEach((function(e){X[e]=new G(e,1,!1,e.toLowerCase(),null,!1)})),X.xlinkHref=new G("xlinkHref",1,!1,"xlink:href","http://www.w3.org/1999/xlink",!0),["src","href","action","formAction"].forEach((function(e){X[e]=new G(e,1,!1,e.toLowerCase(),null,!0)}));var Q=r.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED;function K(e,t,n,r){var i=X.hasOwnProperty(t)?X[t]:null;(null!==i?0===i.type:!r&&(2<t.length&&("o"===t[0]||"O"===t[0])&&("n"===t[1]||"N"===t[1])))||(function(e,t,n,r){if(null==t||function(e,t,n,r){if(null!==n&&0===n.type)return!1;switch(typeof t){case"function":case"symbol":return!0;case"boolean":return!r&&(null!==n?!n.acceptsBooleans:"data-"!==(e=e.toLowerCase().slice(0,5))&&"aria-"!==e);default:return!1}}(e,t,n,r))return!0;if(r)return!1;if(null!==n)switch(n.type){case 3:return!t;case 4:return!1===t;case 5:return isNaN(t);case 6:return isNaN(t)||1>t}return!1}(t,n,i,r)&&(n=null),r||null===i?function(e){return!!V.call(W,e)||!V.call(Y,e)&&(U.test(e)?W[e]=!0:(Y[e]=!0,!1))}(t)&&(null===n?e.removeAttribute(t):e.setAttribute(t,""+n)):i.mustUseProperty?e[i.propertyName]=null===n?3!==i.type&&"":n:(t=i.attributeName,r=i.attributeNamespace,null===n?e.removeAttribute(t):(n=3===(i=i.type)||4===i&&!0===n?"":""+n,r?e.setAttributeNS(r,t,n):e.setAttribute(t,n))))}Q.hasOwnProperty("ReactCurrentDispatcher")||(Q.ReactCurrentDispatcher={current:null}),Q.hasOwnProperty("ReactCurrentBatchConfig")||(Q.ReactCurrentBatchConfig={suspense:null});var J=/^(.*)[\\\/]/,$="function"==typeof Symbol&&Symbol.for,ee=$?Symbol.for("react.element"):60103,te=$?Symbol.for("react.portal"):60106,ne=$?Symbol.for("react.fragment"):60107,re=$?Symbol.for("react.strict_mode"):60108,ie=$?Symbol.for("react.profiler"):60114,oe=$?Symbol.for("react.provider"):60109,ae=$?Symbol.for("react.context"):60110,se=$?Symbol.for("react.concurrent_mode"):60111,le=$?Symbol.for("react.forward_ref"):60112,ue=$?Symbol.for("react.suspense"):60113,ce=$?Symbol.for("react.suspense_list"):60120,de=$?Symbol.for("react.memo"):60115,he=$?Symbol.for("react.lazy"):60116,fe=$?Symbol.for("react.block"):60121,pe="function"==typeof Symbol&&Symbol.iterator;function ge(e){return null===e||"object"!=typeof e?null:"function"==typeof(e=pe&&e[pe]||e["@@iterator"])?e:null}function ve(e){if(null==e)return null;if("function"==typeof e)return e.displayName||e.name||null;if("string"==typeof e)return e;switch(e){case ne:return"Fragment";case te:return"Portal";case ie:return"Profiler";case re:return"StrictMode";case ue:return"Suspense";case ce:return"SuspenseList"}if("object"==typeof e)switch(e.$$typeof){case ae:return"Context.Consumer";case oe:return"Context.Provider";case le:var t=e.render;return t=t.displayName||t.name||"",e.displayName||(""!==t?"ForwardRef("+t+")":"ForwardRef");case de:return ve(e.type);case fe:return ve(e.render);case he:if(e=1===e._status?e._result:null)return ve(e)}return null}function me(e){var t="";do{e:switch(e.tag){case 3:case 4:case 6:case 7:case 10:case 9:var n="";break e;default:var r=e._debugOwner,i=e._debugSource,o=ve(e.type);n=null,r&&(n=ve(r.type)),r=o,o="",i?o=" (at "+i.fileName.replace(J,"")+":"+i.lineNumber+")":n&&(o=" (created by "+n+")"),n="\n    in "+(r||"Unknown")+o}t+=n,e=e.return}while(e);return t}function ye(e){switch(typeof e){case"boolean":case"number":case"object":case"string":case"undefined":return e;default:return""}}function be(e){var t=e.type;return(e=e.nodeName)&&"input"===e.toLowerCase()&&("checkbox"===t||"radio"===t)}function Ce(e){e._valueTracker||(e._valueTracker=function(e){var t=be(e)?"checked":"value",n=Object.getOwnPropertyDescriptor(e.constructor.prototype,t),r=""+e[t];if(!e.hasOwnProperty(t)&&void 0!==n&&"function"==typeof n.get&&"function"==typeof n.set){var i=n.get,o=n.set;return Object.defineProperty(e,t,{configurable:!0,get:function(){return i.call(this)},set:function(e){r=""+e,o.call(this,e)}}),Object.defineProperty(e,t,{enumerable:n.enumerable}),{getValue:function(){return r},setValue:function(e){r=""+e},stopTracking:function(){e._valueTracker=null,delete e[t]}}}}(e))}function Ae(e){if(!e)return!1;var t=e._valueTracker;if(!t)return!0;var n=t.getValue(),r="";return e&&(r=be(e)?e.checked?"true":"false":e.value),(e=r)!==n&&(t.setValue(e),!0)}function _e(e,t){var n=t.checked;return i({},t,{defaultChecked:void 0,defaultValue:void 0,value:void 0,checked:null!=n?n:e._wrapperState.initialChecked})}function Ie(e,t){var n=null==t.defaultValue?"":t.defaultValue,r=null!=t.checked?t.checked:t.defaultChecked;n=ye(null!=t.value?t.value:n),e._wrapperState={initialChecked:r,initialValue:n,controlled:"checkbox"===t.type||"radio"===t.type?null!=t.checked:null!=t.value}}function xe(e,t){null!=(t=t.checked)&&K(e,"checked",t,!1)}function we(e,t){xe(e,t);var n=ye(t.value),r=t.type;if(null!=n)"number"===r?(0===n&&""===e.value||e.value!=n)&&(e.value=""+n):e.value!==""+n&&(e.value=""+n);else if("submit"===r||"reset"===r)return void e.removeAttribute("value");t.hasOwnProperty("value")?Se(e,t.type,n):t.hasOwnProperty("defaultValue")&&Se(e,t.type,ye(t.defaultValue)),null==t.checked&&null!=t.defaultChecked&&(e.defaultChecked=!!t.defaultChecked)}function Me(e,t,n){if(t.hasOwnProperty("value")||t.hasOwnProperty("defaultValue")){var r=t.type;if(!("submit"!==r&&"reset"!==r||void 0!==t.value&&null!==t.value))return;t=""+e._wrapperState.initialValue,n||t===e.value||(e.value=t),e.defaultValue=t}""!==(n=e.name)&&(e.name=""),e.defaultChecked=!!e._wrapperState.initialChecked,""!==n&&(e.name=n)}function Se(e,t,n){"number"===t&&e.ownerDocument.activeElement===e||(null==n?e.defaultValue=""+e._wrapperState.initialValue:e.defaultValue!==""+n&&(e.defaultValue=""+n))}function Te(e,t){return e=i({children:void 0},t),(t=function(e){var t="";return r.Children.forEach(e,(function(e){null!=e&&(t+=e)})),t}(t.children))&&(e.children=t),e}function Ee(e,t,n,r){if(e=e.options,t){t={};for(var i=0;i<n.length;i++)t["$"+n[i]]=!0;for(n=0;n<e.length;n++)i=t.hasOwnProperty("$"+e[n].value),e[n].selected!==i&&(e[n].selected=i),i&&r&&(e[n].defaultSelected=!0)}else{for(n=""+ye(n),t=null,i=0;i<e.length;i++){if(e[i].value===n)return e[i].selected=!0,void(r&&(e[i].defaultSelected=!0));null!==t||e[i].disabled||(t=e[i])}null!==t&&(t.selected=!0)}}function Oe(e,t){if(null!=t.dangerouslySetInnerHTML)throw Error(a(91));return i({},t,{value:void 0,defaultValue:void 0,children:""+e._wrapperState.initialValue})}function ke(e,t){var n=t.value;if(null==n){if(n=t.children,t=t.defaultValue,null!=n){if(null!=t)throw Error(a(92));if(Array.isArray(n)){if(!(1>=n.length))throw Error(a(93));n=n[0]}t=n}null==t&&(t=""),n=t}e._wrapperState={initialValue:ye(n)}}function Le(e,t){var n=ye(t.value),r=ye(t.defaultValue);null!=n&&((n=""+n)!==e.value&&(e.value=n),null==t.defaultValue&&e.defaultValue!==n&&(e.defaultValue=n)),null!=r&&(e.defaultValue=""+r)}function De(e){var t=e.textContent;t===e._wrapperState.initialValue&&""!==t&&null!==t&&(e.value=t)}var Pe="http://www.w3.org/1999/xhtml",Ne="http://www.w3.org/2000/svg";function je(e){switch(e){case"svg":return"http://www.w3.org/2000/svg";case"math":return"http://www.w3.org/1998/Math/MathML";default:return"http://www.w3.org/1999/xhtml"}}function Re(e,t){return null==e||"http://www.w3.org/1999/xhtml"===e?je(t):"http://www.w3.org/2000/svg"===e&&"foreignObject"===t?"http://www.w3.org/1999/xhtml":e}var ze,Fe=function(e){return"undefined"!=typeof MSApp&&MSApp.execUnsafeLocalFunction?function(t,n,r,i){MSApp.execUnsafeLocalFunction((function(){return e(t,n)}))}:e}((function(e,t){if(e.namespaceURI!==Ne||"innerHTML"in e)e.innerHTML=t;else{for((ze=ze||document.createElement("div")).innerHTML="<svg>"+t.valueOf().toString()+"</svg>",t=ze.firstChild;e.firstChild;)e.removeChild(e.firstChild);for(;t.firstChild;)e.appendChild(t.firstChild)}}));function Be(e,t){if(t){var n=e.firstChild;if(n&&n===e.lastChild&&3===n.nodeType)return void(n.nodeValue=t)}e.textContent=t}function He(e,t){var n={};return n[e.toLowerCase()]=t.toLowerCase(),n["Webkit"+e]="webkit"+t,n["Moz"+e]="moz"+t,n}var Ue={animationend:He("Animation","AnimationEnd"),animationiteration:He("Animation","AnimationIteration"),animationstart:He("Animation","AnimationStart"),transitionend:He("Transition","TransitionEnd")},Ve={},Ye={};function We(e){if(Ve[e])return Ve[e];if(!Ue[e])return e;var t,n=Ue[e];for(t in n)if(n.hasOwnProperty(t)&&t in Ye)return Ve[e]=n[t];return e}S&&(Ye=document.createElement("div").style,"AnimationEvent"in window||(delete Ue.animationend.animation,delete Ue.animationiteration.animation,delete Ue.animationstart.animation),"TransitionEvent"in window||delete Ue.transitionend.transition);var Ge=We("animationend"),Xe=We("animationiteration"),qe=We("animationstart"),Ze=We("transitionend"),Qe="abort canplay canplaythrough durationchange emptied encrypted ended error loadeddata loadedmetadata loadstart pause play playing progress ratechange seeked seeking stalled suspend timeupdate volumechange waiting".split(" "),Ke=new("function"==typeof WeakMap?WeakMap:Map);function Je(e){var t=Ke.get(e);return void 0===t&&(t=new Map,Ke.set(e,t)),t}function $e(e){var t=e,n=e;if(e.alternate)for(;t.return;)t=t.return;else{e=t;do{0!=(1026&(t=e).effectTag)&&(n=t.return),e=t.return}while(e)}return 3===t.tag?n:null}function et(e){if(13===e.tag){var t=e.memoizedState;if(null===t&&(null!==(e=e.alternate)&&(t=e.memoizedState)),null!==t)return t.dehydrated}return null}function tt(e){if($e(e)!==e)throw Error(a(188))}function nt(e){if(!(e=function(e){var t=e.alternate;if(!t){if(null===(t=$e(e)))throw Error(a(188));return t!==e?null:e}for(var n=e,r=t;;){var i=n.return;if(null===i)break;var o=i.alternate;if(null===o){if(null!==(r=i.return)){n=r;continue}break}if(i.child===o.child){for(o=i.child;o;){if(o===n)return tt(i),e;if(o===r)return tt(i),t;o=o.sibling}throw Error(a(188))}if(n.return!==r.return)n=i,r=o;else{for(var s=!1,l=i.child;l;){if(l===n){s=!0,n=i,r=o;break}if(l===r){s=!0,r=i,n=o;break}l=l.sibling}if(!s){for(l=o.child;l;){if(l===n){s=!0,n=o,r=i;break}if(l===r){s=!0,r=o,n=i;break}l=l.sibling}if(!s)throw Error(a(189))}}if(n.alternate!==r)throw Error(a(190))}if(3!==n.tag)throw Error(a(188));return n.stateNode.current===n?e:t}(e)))return null;for(var t=e;;){if(5===t.tag||6===t.tag)return t;if(t.child)t.child.return=t,t=t.child;else{if(t===e)break;for(;!t.sibling;){if(!t.return||t.return===e)return null;t=t.return}t.sibling.return=t.return,t=t.sibling}}return null}function rt(e,t){if(null==t)throw Error(a(30));return null==e?t:Array.isArray(e)?Array.isArray(t)?(e.push.apply(e,t),e):(e.push(t),e):Array.isArray(t)?[e].concat(t):[e,t]}function it(e,t,n){Array.isArray(e)?e.forEach(t,n):e&&t.call(n,e)}var ot=null;function at(e){if(e){var t=e._dispatchListeners,n=e._dispatchInstances;if(Array.isArray(t))for(var r=0;r<t.length&&!e.isPropagationStopped();r++)m(e,t[r],n[r]);else t&&m(e,t,n);e._dispatchListeners=null,e._dispatchInstances=null,e.isPersistent()||e.constructor.release(e)}}function st(e){if(null!==e&&(ot=rt(ot,e)),e=ot,ot=null,e){if(it(e,at),ot)throw Error(a(95));if(c)throw e=d,c=!1,d=null,e}}function lt(e){return(e=e.target||e.srcElement||window).correspondingUseElement&&(e=e.correspondingUseElement),3===e.nodeType?e.parentNode:e}function ut(e){if(!S)return!1;var t=(e="on"+e)in document;return t||((t=document.createElement("div")).setAttribute(e,"return;"),t="function"==typeof t[e]),t}var ct=[];function dt(e){e.topLevelType=null,e.nativeEvent=null,e.targetInst=null,e.ancestors.length=0,10>ct.length&&ct.push(e)}function ht(e,t,n,r){if(ct.length){var i=ct.pop();return i.topLevelType=e,i.eventSystemFlags=r,i.nativeEvent=t,i.targetInst=n,i}return{topLevelType:e,eventSystemFlags:r,nativeEvent:t,targetInst:n,ancestors:[]}}function ft(e){var t=e.targetInst,n=t;do{if(!n){e.ancestors.push(n);break}var r=n;if(3===r.tag)r=r.stateNode.containerInfo;else{for(;r.return;)r=r.return;r=3!==r.tag?null:r.stateNode.containerInfo}if(!r)break;5!==(t=n.tag)&&6!==t||e.ancestors.push(n),n=Sn(r)}while(n);for(n=0;n<e.ancestors.length;n++){t=e.ancestors[n];var i=lt(e.nativeEvent);r=e.topLevelType;var o=e.nativeEvent,a=e.eventSystemFlags;0===n&&(a|=64);for(var s=null,l=0;l<_.length;l++){var u=_[l];u&&(u=u.extractEvents(r,t,o,i,a))&&(s=rt(s,u))}st(s)}}function pt(e,t,n){if(!n.has(e)){switch(e){case"scroll":qt(t,"scroll",!0);break;case"focus":case"blur":qt(t,"focus",!0),qt(t,"blur",!0),n.set("blur",null),n.set("focus",null);break;case"cancel":case"close":ut(e)&&qt(t,e,!0);break;case"invalid":case"submit":case"reset":break;default:-1===Qe.indexOf(e)&&Xt(e,t)}n.set(e,null)}}var gt,vt,mt,yt=!1,bt=[],Ct=null,At=null,_t=null,It=new Map,xt=new Map,wt=[],Mt="mousedown mouseup touchcancel touchend touchstart auxclick dblclick pointercancel pointerdown pointerup dragend dragstart drop compositionend compositionstart keydown keypress keyup input textInput close cancel copy cut paste click change contextmenu reset submit".split(" "),St="focus blur dragenter dragleave mouseover mouseout pointerover pointerout gotpointercapture lostpointercapture".split(" ");function Tt(e,t,n,r,i){return{blockedOn:e,topLevelType:t,eventSystemFlags:32|n,nativeEvent:i,container:r}}function Et(e,t){switch(e){case"focus":case"blur":Ct=null;break;case"dragenter":case"dragleave":At=null;break;case"mouseover":case"mouseout":_t=null;break;case"pointerover":case"pointerout":It.delete(t.pointerId);break;case"gotpointercapture":case"lostpointercapture":xt.delete(t.pointerId)}}function Ot(e,t,n,r,i,o){return null===e||e.nativeEvent!==o?(e=Tt(t,n,r,i,o),null!==t&&(null!==(t=Tn(t))&&vt(t)),e):(e.eventSystemFlags|=r,e)}function kt(e){var t=Sn(e.target);if(null!==t){var n=$e(t);if(null!==n)if(13===(t=n.tag)){if(null!==(t=et(n)))return e.blockedOn=t,void o.unstable_runWithPriority(e.priority,(function(){mt(n)}))}else if(3===t&&n.stateNode.hydrate)return void(e.blockedOn=3===n.tag?n.stateNode.containerInfo:null)}e.blockedOn=null}function Lt(e){if(null!==e.blockedOn)return!1;var t=Jt(e.topLevelType,e.eventSystemFlags,e.container,e.nativeEvent);if(null!==t){var n=Tn(t);return null!==n&&vt(n),e.blockedOn=t,!1}return!0}function Dt(e,t,n){Lt(e)&&n.delete(t)}function Pt(){for(yt=!1;0<bt.length;){var e=bt[0];if(null!==e.blockedOn){null!==(e=Tn(e.blockedOn))&&gt(e);break}var t=Jt(e.topLevelType,e.eventSystemFlags,e.container,e.nativeEvent);null!==t?e.blockedOn=t:bt.shift()}null!==Ct&&Lt(Ct)&&(Ct=null),null!==At&&Lt(At)&&(At=null),null!==_t&&Lt(_t)&&(_t=null),It.forEach(Dt),xt.forEach(Dt)}function Nt(e,t){e.blockedOn===t&&(e.blockedOn=null,yt||(yt=!0,o.unstable_scheduleCallback(o.unstable_NormalPriority,Pt)))}function jt(e){function t(t){return Nt(t,e)}if(0<bt.length){Nt(bt[0],e);for(var n=1;n<bt.length;n++){var r=bt[n];r.blockedOn===e&&(r.blockedOn=null)}}for(null!==Ct&&Nt(Ct,e),null!==At&&Nt(At,e),null!==_t&&Nt(_t,e),It.forEach(t),xt.forEach(t),n=0;n<wt.length;n++)(r=wt[n]).blockedOn===e&&(r.blockedOn=null);for(;0<wt.length&&null===(n=wt[0]).blockedOn;)kt(n),null===n.blockedOn&&wt.shift()}var Rt={},zt=new Map,Ft=new Map,Bt=["abort","abort",Ge,"animationEnd",Xe,"animationIteration",qe,"animationStart","canplay","canPlay","canplaythrough","canPlayThrough","durationchange","durationChange","emptied","emptied","encrypted","encrypted","ended","ended","error","error","gotpointercapture","gotPointerCapture","load","load","loadeddata","loadedData","loadedmetadata","loadedMetadata","loadstart","loadStart","lostpointercapture","lostPointerCapture","playing","playing","progress","progress","seeking","seeking","stalled","stalled","suspend","suspend","timeupdate","timeUpdate",Ze,"transitionEnd","waiting","waiting"];function Ht(e,t){for(var n=0;n<e.length;n+=2){var r=e[n],i=e[n+1],o="on"+(i[0].toUpperCase()+i.slice(1));o={phasedRegistrationNames:{bubbled:o,captured:o+"Capture"},dependencies:[r],eventPriority:t},Ft.set(r,t),zt.set(r,o),Rt[i]=o}}Ht("blur blur cancel cancel click click close close contextmenu contextMenu copy copy cut cut auxclick auxClick dblclick doubleClick dragend dragEnd dragstart dragStart drop drop focus focus input input invalid invalid keydown keyDown keypress keyPress keyup keyUp mousedown mouseDown mouseup mouseUp paste paste pause pause play play pointercancel pointerCancel pointerdown pointerDown pointerup pointerUp ratechange rateChange reset reset seeked seeked submit submit touchcancel touchCancel touchend touchEnd touchstart touchStart volumechange volumeChange".split(" "),0),Ht("drag drag dragenter dragEnter dragexit dragExit dragleave dragLeave dragover dragOver mousemove mouseMove mouseout mouseOut mouseover mouseOver pointermove pointerMove pointerout pointerOut pointerover pointerOver scroll scroll toggle toggle touchmove touchMove wheel wheel".split(" "),1),Ht(Bt,2);for(var Ut="change selectionchange textInput compositionstart compositionend compositionupdate".split(" "),Vt=0;Vt<Ut.length;Vt++)Ft.set(Ut[Vt],0);var Yt=o.unstable_UserBlockingPriority,Wt=o.unstable_runWithPriority,Gt=!0;function Xt(e,t){qt(t,e,!1)}function qt(e,t,n){var r=Ft.get(t);switch(void 0===r?2:r){case 0:r=Zt.bind(null,t,1,e);break;case 1:r=Qt.bind(null,t,1,e);break;default:r=Kt.bind(null,t,1,e)}n?e.addEventListener(t,r,!0):e.addEventListener(t,r,!1)}function Zt(e,t,n,r){z||j();var i=Kt,o=z;z=!0;try{N(i,e,t,n,r)}finally{(z=o)||B()}}function Qt(e,t,n,r){Wt(Yt,Kt.bind(null,e,t,n,r))}function Kt(e,t,n,r){if(Gt)if(0<bt.length&&-1<Mt.indexOf(e))e=Tt(null,e,t,n,r),bt.push(e);else{var i=Jt(e,t,n,r);if(null===i)Et(e,r);else if(-1<Mt.indexOf(e))e=Tt(i,e,t,n,r),bt.push(e);else if(!function(e,t,n,r,i){switch(t){case"focus":return Ct=Ot(Ct,e,t,n,r,i),!0;case"dragenter":return At=Ot(At,e,t,n,r,i),!0;case"mouseover":return _t=Ot(_t,e,t,n,r,i),!0;case"pointerover":var o=i.pointerId;return It.set(o,Ot(It.get(o)||null,e,t,n,r,i)),!0;case"gotpointercapture":return o=i.pointerId,xt.set(o,Ot(xt.get(o)||null,e,t,n,r,i)),!0}return!1}(i,e,t,n,r)){Et(e,r),e=ht(e,r,null,t);try{H(ft,e)}finally{dt(e)}}}}function Jt(e,t,n,r){if(null!==(n=Sn(n=lt(r)))){var i=$e(n);if(null===i)n=null;else{var o=i.tag;if(13===o){if(null!==(n=et(i)))return n;n=null}else if(3===o){if(i.stateNode.hydrate)return 3===i.tag?i.stateNode.containerInfo:null;n=null}else i!==n&&(n=null)}}e=ht(e,r,n,t);try{H(ft,e)}finally{dt(e)}return null}var $t={animationIterationCount:!0,borderImageOutset:!0,borderImageSlice:!0,borderImageWidth:!0,boxFlex:!0,boxFlexGroup:!0,boxOrdinalGroup:!0,columnCount:!0,columns:!0,flex:!0,flexGrow:!0,flexPositive:!0,flexShrink:!0,flexNegative:!0,flexOrder:!0,gridArea:!0,gridRow:!0,gridRowEnd:!0,gridRowSpan:!0,gridRowStart:!0,gridColumn:!0,gridColumnEnd:!0,gridColumnSpan:!0,gridColumnStart:!0,fontWeight:!0,lineClamp:!0,lineHeight:!0,opacity:!0,order:!0,orphans:!0,tabSize:!0,widows:!0,zIndex:!0,zoom:!0,fillOpacity:!0,floodOpacity:!0,stopOpacity:!0,strokeDasharray:!0,strokeDashoffset:!0,strokeMiterlimit:!0,strokeOpacity:!0,strokeWidth:!0},en=["Webkit","ms","Moz","O"];function tn(e,t,n){return null==t||"boolean"==typeof t||""===t?"":n||"number"!=typeof t||0===t||$t.hasOwnProperty(e)&&$t[e]?(""+t).trim():t+"px"}function nn(e,t){for(var n in e=e.style,t)if(t.hasOwnProperty(n)){var r=0===n.indexOf("--"),i=tn(n,t[n],r);"float"===n&&(n="cssFloat"),r?e.setProperty(n,i):e[n]=i}}Object.keys($t).forEach((function(e){en.forEach((function(t){t=t+e.charAt(0).toUpperCase()+e.substring(1),$t[t]=$t[e]}))}));var rn=i({menuitem:!0},{area:!0,base:!0,br:!0,col:!0,embed:!0,hr:!0,img:!0,input:!0,keygen:!0,link:!0,meta:!0,param:!0,source:!0,track:!0,wbr:!0});function on(e,t){if(t){if(rn[e]&&(null!=t.children||null!=t.dangerouslySetInnerHTML))throw Error(a(137,e,""));if(null!=t.dangerouslySetInnerHTML){if(null!=t.children)throw Error(a(60));if("object"!=typeof t.dangerouslySetInnerHTML||!("__html"in t.dangerouslySetInnerHTML))throw Error(a(61))}if(null!=t.style&&"object"!=typeof t.style)throw Error(a(62,""))}}function an(e,t){if(-1===e.indexOf("-"))return"string"==typeof t.is;switch(e){case"annotation-xml":case"color-profile":case"font-face":case"font-face-src":case"font-face-uri":case"font-face-format":case"font-face-name":case"missing-glyph":return!1;default:return!0}}var sn=Pe;function ln(e,t){var n=Je(e=9===e.nodeType||11===e.nodeType?e:e.ownerDocument);t=w[t];for(var r=0;r<t.length;r++)pt(t[r],e,n)}function un(){}function cn(e){if(void 0===(e=e||("undefined"!=typeof document?document:void 0)))return null;try{return e.activeElement||e.body}catch(t){return e.body}}function dn(e){for(;e&&e.firstChild;)e=e.firstChild;return e}function hn(e,t){var n,r=dn(e);for(e=0;r;){if(3===r.nodeType){if(n=e+r.textContent.length,e<=t&&n>=t)return{node:r,offset:t-e};e=n}e:{for(;r;){if(r.nextSibling){r=r.nextSibling;break e}r=r.parentNode}r=void 0}r=dn(r)}}function fn(){for(var e=window,t=cn();t instanceof e.HTMLIFrameElement;){try{var n="string"==typeof t.contentWindow.location.href}catch(e){n=!1}if(!n)break;t=cn((e=t.contentWindow).document)}return t}function pn(e){var t=e&&e.nodeName&&e.nodeName.toLowerCase();return t&&("input"===t&&("text"===e.type||"search"===e.type||"tel"===e.type||"url"===e.type||"password"===e.type)||"textarea"===t||"true"===e.contentEditable)}var gn=null,vn=null;function mn(e,t){switch(e){case"button":case"input":case"select":case"textarea":return!!t.autoFocus}return!1}function yn(e,t){return"textarea"===e||"option"===e||"noscript"===e||"string"==typeof t.children||"number"==typeof t.children||"object"==typeof t.dangerouslySetInnerHTML&&null!==t.dangerouslySetInnerHTML&&null!=t.dangerouslySetInnerHTML.__html}var bn="function"==typeof setTimeout?setTimeout:void 0,Cn="function"==typeof clearTimeout?clearTimeout:void 0;function An(e){for(;null!=e;e=e.nextSibling){var t=e.nodeType;if(1===t||3===t)break}return e}function _n(e){e=e.previousSibling;for(var t=0;e;){if(8===e.nodeType){var n=e.data;if("$"===n||"$!"===n||"$?"===n){if(0===t)return e;t--}else"/$"===n&&t++}e=e.previousSibling}return null}var In=Math.random().toString(36).slice(2),xn="__reactInternalInstance$"+In,wn="__reactEventHandlers$"+In,Mn="__reactContainere$"+In;function Sn(e){var t=e[xn];if(t)return t;for(var n=e.parentNode;n;){if(t=n[Mn]||n[xn]){if(n=t.alternate,null!==t.child||null!==n&&null!==n.child)for(e=_n(e);null!==e;){if(n=e[xn])return n;e=_n(e)}return t}n=(e=n).parentNode}return null}function Tn(e){return!(e=e[xn]||e[Mn])||5!==e.tag&&6!==e.tag&&13!==e.tag&&3!==e.tag?null:e}function En(e){if(5===e.tag||6===e.tag)return e.stateNode;throw Error(a(33))}function On(e){return e[wn]||null}function kn(e){do{e=e.return}while(e&&5!==e.tag);return e||null}function Ln(e,t){var n=e.stateNode;if(!n)return null;var r=p(n);if(!r)return null;n=r[t];e:switch(t){case"onClick":case"onClickCapture":case"onDoubleClick":case"onDoubleClickCapture":case"onMouseDown":case"onMouseDownCapture":case"onMouseMove":case"onMouseMoveCapture":case"onMouseUp":case"onMouseUpCapture":case"onMouseEnter":(r=!r.disabled)||(r=!("button"===(e=e.type)||"input"===e||"select"===e||"textarea"===e)),e=!r;break e;default:e=!1}if(e)return null;if(n&&"function"!=typeof n)throw Error(a(231,t,typeof n));return n}function Dn(e,t,n){(t=Ln(e,n.dispatchConfig.phasedRegistrationNames[t]))&&(n._dispatchListeners=rt(n._dispatchListeners,t),n._dispatchInstances=rt(n._dispatchInstances,e))}function Pn(e){if(e&&e.dispatchConfig.phasedRegistrationNames){for(var t=e._targetInst,n=[];t;)n.push(t),t=kn(t);for(t=n.length;0<t--;)Dn(n[t],"captured",e);for(t=0;t<n.length;t++)Dn(n[t],"bubbled",e)}}function Nn(e,t,n){e&&n&&n.dispatchConfig.registrationName&&(t=Ln(e,n.dispatchConfig.registrationName))&&(n._dispatchListeners=rt(n._dispatchListeners,t),n._dispatchInstances=rt(n._dispatchInstances,e))}function jn(e){e&&e.dispatchConfig.registrationName&&Nn(e._targetInst,null,e)}function Rn(e){it(e,Pn)}var zn=null,Fn=null,Bn=null;function Hn(){if(Bn)return Bn;var e,t,n=Fn,r=n.length,i="value"in zn?zn.value:zn.textContent,o=i.length;for(e=0;e<r&&n[e]===i[e];e++);var a=r-e;for(t=1;t<=a&&n[r-t]===i[o-t];t++);return Bn=i.slice(e,1<t?1-t:void 0)}function Un(){return!0}function Vn(){return!1}function Yn(e,t,n,r){for(var i in this.dispatchConfig=e,this._targetInst=t,this.nativeEvent=n,e=this.constructor.Interface)e.hasOwnProperty(i)&&((t=e[i])?this[i]=t(n):"target"===i?this.target=r:this[i]=n[i]);return this.isDefaultPrevented=(null!=n.defaultPrevented?n.defaultPrevented:!1===n.returnValue)?Un:Vn,this.isPropagationStopped=Vn,this}function Wn(e,t,n,r){if(this.eventPool.length){var i=this.eventPool.pop();return this.call(i,e,t,n,r),i}return new this(e,t,n,r)}function Gn(e){if(!(e instanceof this))throw Error(a(279));e.destructor(),10>this.eventPool.length&&this.eventPool.push(e)}function Xn(e){e.eventPool=[],e.getPooled=Wn,e.release=Gn}i(Yn.prototype,{preventDefault:function(){this.defaultPrevented=!0;var e=this.nativeEvent;e&&(e.preventDefault?e.preventDefault():"unknown"!=typeof e.returnValue&&(e.returnValue=!1),this.isDefaultPrevented=Un)},stopPropagation:function(){var e=this.nativeEvent;e&&(e.stopPropagation?e.stopPropagation():"unknown"!=typeof e.cancelBubble&&(e.cancelBubble=!0),this.isPropagationStopped=Un)},persist:function(){this.isPersistent=Un},isPersistent:Vn,destructor:function(){var e,t=this.constructor.Interface;for(e in t)this[e]=null;this.nativeEvent=this._targetInst=this.dispatchConfig=null,this.isPropagationStopped=this.isDefaultPrevented=Vn,this._dispatchInstances=this._dispatchListeners=null}}),Yn.Interface={type:null,target:null,currentTarget:function(){return null},eventPhase:null,bubbles:null,cancelable:null,timeStamp:function(e){return e.timeStamp||Date.now()},defaultPrevented:null,isTrusted:null},Yn.extend=function(e){function t(){}function n(){return r.apply(this,arguments)}var r=this;t.prototype=r.prototype;var o=new t;return i(o,n.prototype),n.prototype=o,n.prototype.constructor=n,n.Interface=i({},r.Interface,e),n.extend=r.extend,Xn(n),n},Xn(Yn);var qn=Yn.extend({data:null}),Zn=Yn.extend({data:null}),Qn=[9,13,27,32],Kn=S&&"CompositionEvent"in window,Jn=null;S&&"documentMode"in document&&(Jn=document.documentMode);var $n=S&&"TextEvent"in window&&!Jn,er=S&&(!Kn||Jn&&8<Jn&&11>=Jn),tr=String.fromCharCode(32),nr={beforeInput:{phasedRegistrationNames:{bubbled:"onBeforeInput",captured:"onBeforeInputCapture"},dependencies:["compositionend","keypress","textInput","paste"]},compositionEnd:{phasedRegistrationNames:{bubbled:"onCompositionEnd",captured:"onCompositionEndCapture"},dependencies:"blur compositionend keydown keypress keyup mousedown".split(" ")},compositionStart:{phasedRegistrationNames:{bubbled:"onCompositionStart",captured:"onCompositionStartCapture"},dependencies:"blur compositionstart keydown keypress keyup mousedown".split(" ")},compositionUpdate:{phasedRegistrationNames:{bubbled:"onCompositionUpdate",captured:"onCompositionUpdateCapture"},dependencies:"blur compositionupdate keydown keypress keyup mousedown".split(" ")}},rr=!1;function ir(e,t){switch(e){case"keyup":return-1!==Qn.indexOf(t.keyCode);case"keydown":return 229!==t.keyCode;case"keypress":case"mousedown":case"blur":return!0;default:return!1}}function or(e){return"object"==typeof(e=e.detail)&&"data"in e?e.data:null}var ar=!1;var sr={eventTypes:nr,extractEvents:function(e,t,n,r){var i;if(Kn)e:{switch(e){case"compositionstart":var o=nr.compositionStart;break e;case"compositionend":o=nr.compositionEnd;break e;case"compositionupdate":o=nr.compositionUpdate;break e}o=void 0}else ar?ir(e,n)&&(o=nr.compositionEnd):"keydown"===e&&229===n.keyCode&&(o=nr.compositionStart);return o?(er&&"ko"!==n.locale&&(ar||o!==nr.compositionStart?o===nr.compositionEnd&&ar&&(i=Hn()):(Fn="value"in(zn=r)?zn.value:zn.textContent,ar=!0)),o=qn.getPooled(o,t,n,r),i?o.data=i:null!==(i=or(n))&&(o.data=i),Rn(o),i=o):i=null,(e=$n?function(e,t){switch(e){case"compositionend":return or(t);case"keypress":return 32!==t.which?null:(rr=!0,tr);case"textInput":return(e=t.data)===tr&&rr?null:e;default:return null}}(e,n):function(e,t){if(ar)return"compositionend"===e||!Kn&&ir(e,t)?(e=Hn(),Bn=Fn=zn=null,ar=!1,e):null;switch(e){case"paste":return null;case"keypress":if(!(t.ctrlKey||t.altKey||t.metaKey)||t.ctrlKey&&t.altKey){if(t.char&&1<t.char.length)return t.char;if(t.which)return String.fromCharCode(t.which)}return null;case"compositionend":return er&&"ko"!==t.locale?null:t.data;default:return null}}(e,n))?((t=Zn.getPooled(nr.beforeInput,t,n,r)).data=e,Rn(t)):t=null,null===i?t:null===t?i:[i,t]}},lr={color:!0,date:!0,datetime:!0,"datetime-local":!0,email:!0,month:!0,number:!0,password:!0,range:!0,search:!0,tel:!0,text:!0,time:!0,url:!0,week:!0};function ur(e){var t=e&&e.nodeName&&e.nodeName.toLowerCase();return"input"===t?!!lr[e.type]:"textarea"===t}var cr={change:{phasedRegistrationNames:{bubbled:"onChange",captured:"onChangeCapture"},dependencies:"blur change click focus input keydown keyup selectionchange".split(" ")}};function dr(e,t,n){return(e=Yn.getPooled(cr.change,e,t,n)).type="change",L(n),Rn(e),e}var hr=null,fr=null;function pr(e){st(e)}function gr(e){if(Ae(En(e)))return e}function vr(e,t){if("change"===e)return t}var mr=!1;function yr(){hr&&(hr.detachEvent("onpropertychange",br),fr=hr=null)}function br(e){if("value"===e.propertyName&&gr(fr))if(e=dr(fr,e,lt(e)),z)st(e);else{z=!0;try{P(pr,e)}finally{z=!1,B()}}}function Cr(e,t,n){"focus"===e?(yr(),fr=n,(hr=t).attachEvent("onpropertychange",br)):"blur"===e&&yr()}function Ar(e){if("selectionchange"===e||"keyup"===e||"keydown"===e)return gr(fr)}function _r(e,t){if("click"===e)return gr(t)}function Ir(e,t){if("input"===e||"change"===e)return gr(t)}S&&(mr=ut("input")&&(!document.documentMode||9<document.documentMode));var xr={eventTypes:cr,_isInputEventSupported:mr,extractEvents:function(e,t,n,r){var i=t?En(t):window,o=i.nodeName&&i.nodeName.toLowerCase();if("select"===o||"input"===o&&"file"===i.type)var a=vr;else if(ur(i))if(mr)a=Ir;else{a=Ar;var s=Cr}else(o=i.nodeName)&&"input"===o.toLowerCase()&&("checkbox"===i.type||"radio"===i.type)&&(a=_r);if(a&&(a=a(e,t)))return dr(a,n,r);s&&s(e,i,t),"blur"===e&&(e=i._wrapperState)&&e.controlled&&"number"===i.type&&Se(i,"number",i.value)}},wr=Yn.extend({view:null,detail:null}),Mr={Alt:"altKey",Control:"ctrlKey",Meta:"metaKey",Shift:"shiftKey"};function Sr(e){var t=this.nativeEvent;return t.getModifierState?t.getModifierState(e):!!(e=Mr[e])&&!!t[e]}function Tr(){return Sr}var Er=0,Or=0,kr=!1,Lr=!1,Dr=wr.extend({screenX:null,screenY:null,clientX:null,clientY:null,pageX:null,pageY:null,ctrlKey:null,shiftKey:null,altKey:null,metaKey:null,getModifierState:Tr,button:null,buttons:null,relatedTarget:function(e){return e.relatedTarget||(e.fromElement===e.srcElement?e.toElement:e.fromElement)},movementX:function(e){if("movementX"in e)return e.movementX;var t=Er;return Er=e.screenX,kr?"mousemove"===e.type?e.screenX-t:0:(kr=!0,0)},movementY:function(e){if("movementY"in e)return e.movementY;var t=Or;return Or=e.screenY,Lr?"mousemove"===e.type?e.screenY-t:0:(Lr=!0,0)}}),Pr=Dr.extend({pointerId:null,width:null,height:null,pressure:null,tangentialPressure:null,tiltX:null,tiltY:null,twist:null,pointerType:null,isPrimary:null}),Nr={mouseEnter:{registrationName:"onMouseEnter",dependencies:["mouseout","mouseover"]},mouseLeave:{registrationName:"onMouseLeave",dependencies:["mouseout","mouseover"]},pointerEnter:{registrationName:"onPointerEnter",dependencies:["pointerout","pointerover"]},pointerLeave:{registrationName:"onPointerLeave",dependencies:["pointerout","pointerover"]}},jr={eventTypes:Nr,extractEvents:function(e,t,n,r,i){var o="mouseover"===e||"pointerover"===e,a="mouseout"===e||"pointerout"===e;if(o&&0==(32&i)&&(n.relatedTarget||n.fromElement)||!a&&!o)return null;(o=r.window===r?r:(o=r.ownerDocument)?o.defaultView||o.parentWindow:window,a)?(a=t,null!==(t=(t=n.relatedTarget||n.toElement)?Sn(t):null)&&(t!==$e(t)||5!==t.tag&&6!==t.tag)&&(t=null)):a=null;if(a===t)return null;if("mouseout"===e||"mouseover"===e)var s=Dr,l=Nr.mouseLeave,u=Nr.mouseEnter,c="mouse";else"pointerout"!==e&&"pointerover"!==e||(s=Pr,l=Nr.pointerLeave,u=Nr.pointerEnter,c="pointer");if(e=null==a?o:En(a),o=null==t?o:En(t),(l=s.getPooled(l,a,n,r)).type=c+"leave",l.target=e,l.relatedTarget=o,(n=s.getPooled(u,t,n,r)).type=c+"enter",n.target=o,n.relatedTarget=e,c=t,(r=a)&&c)e:{for(u=c,a=0,e=s=r;e;e=kn(e))a++;for(e=0,t=u;t;t=kn(t))e++;for(;0<a-e;)s=kn(s),a--;for(;0<e-a;)u=kn(u),e--;for(;a--;){if(s===u||s===u.alternate)break e;s=kn(s),u=kn(u)}s=null}else s=null;for(u=s,s=[];r&&r!==u&&(null===(a=r.alternate)||a!==u);)s.push(r),r=kn(r);for(r=[];c&&c!==u&&(null===(a=c.alternate)||a!==u);)r.push(c),c=kn(c);for(c=0;c<s.length;c++)Nn(s[c],"bubbled",l);for(c=r.length;0<c--;)Nn(r[c],"captured",n);return 0==(64&i)?[l]:[l,n]}};var Rr="function"==typeof Object.is?Object.is:function(e,t){return e===t&&(0!==e||1/e==1/t)||e!=e&&t!=t},zr=Object.prototype.hasOwnProperty;function Fr(e,t){if(Rr(e,t))return!0;if("object"!=typeof e||null===e||"object"!=typeof t||null===t)return!1;var n=Object.keys(e),r=Object.keys(t);if(n.length!==r.length)return!1;for(r=0;r<n.length;r++)if(!zr.call(t,n[r])||!Rr(e[n[r]],t[n[r]]))return!1;return!0}var Br=S&&"documentMode"in document&&11>=document.documentMode,Hr={select:{phasedRegistrationNames:{bubbled:"onSelect",captured:"onSelectCapture"},dependencies:"blur contextmenu dragend focus keydown keyup mousedown mouseup selectionchange".split(" ")}},Ur=null,Vr=null,Yr=null,Wr=!1;function Gr(e,t){var n=t.window===t?t.document:9===t.nodeType?t:t.ownerDocument;return Wr||null==Ur||Ur!==cn(n)?null:("selectionStart"in(n=Ur)&&pn(n)?n={start:n.selectionStart,end:n.selectionEnd}:n={anchorNode:(n=(n.ownerDocument&&n.ownerDocument.defaultView||window).getSelection()).anchorNode,anchorOffset:n.anchorOffset,focusNode:n.focusNode,focusOffset:n.focusOffset},Yr&&Fr(Yr,n)?null:(Yr=n,(e=Yn.getPooled(Hr.select,Vr,e,t)).type="select",e.target=Ur,Rn(e),e))}var Xr={eventTypes:Hr,extractEvents:function(e,t,n,r,i,o){if(!(o=!(i=o||(r.window===r?r.document:9===r.nodeType?r:r.ownerDocument)))){e:{i=Je(i),o=w.onSelect;for(var a=0;a<o.length;a++)if(!i.has(o[a])){i=!1;break e}i=!0}o=!i}if(o)return null;switch(i=t?En(t):window,e){case"focus":(ur(i)||"true"===i.contentEditable)&&(Ur=i,Vr=t,Yr=null);break;case"blur":Yr=Vr=Ur=null;break;case"mousedown":Wr=!0;break;case"contextmenu":case"mouseup":case"dragend":return Wr=!1,Gr(n,r);case"selectionchange":if(Br)break;case"keydown":case"keyup":return Gr(n,r)}return null}},qr=Yn.extend({animationName:null,elapsedTime:null,pseudoElement:null}),Zr=Yn.extend({clipboardData:function(e){return"clipboardData"in e?e.clipboardData:window.clipboardData}}),Qr=wr.extend({relatedTarget:null});function Kr(e){var t=e.keyCode;return"charCode"in e?0===(e=e.charCode)&&13===t&&(e=13):e=t,10===e&&(e=13),32<=e||13===e?e:0}var Jr={Esc:"Escape",Spacebar:" ",Left:"ArrowLeft",Up:"ArrowUp",Right:"ArrowRight",Down:"ArrowDown",Del:"Delete",Win:"OS",Menu:"ContextMenu",Apps:"ContextMenu",Scroll:"ScrollLock",MozPrintableKey:"Unidentified"},$r={8:"Backspace",9:"Tab",12:"Clear",13:"Enter",16:"Shift",17:"Control",18:"Alt",19:"Pause",20:"CapsLock",27:"Escape",32:" ",33:"PageUp",34:"PageDown",35:"End",36:"Home",37:"ArrowLeft",38:"ArrowUp",39:"ArrowRight",40:"ArrowDown",45:"Insert",46:"Delete",112:"F1",113:"F2",114:"F3",115:"F4",116:"F5",117:"F6",118:"F7",119:"F8",120:"F9",121:"F10",122:"F11",123:"F12",144:"NumLock",145:"ScrollLock",224:"Meta"},ei=wr.extend({key:function(e){if(e.key){var t=Jr[e.key]||e.key;if("Unidentified"!==t)return t}return"keypress"===e.type?13===(e=Kr(e))?"Enter":String.fromCharCode(e):"keydown"===e.type||"keyup"===e.type?$r[e.keyCode]||"Unidentified":""},location:null,ctrlKey:null,shiftKey:null,altKey:null,metaKey:null,repeat:null,locale:null,getModifierState:Tr,charCode:function(e){return"keypress"===e.type?Kr(e):0},keyCode:function(e){return"keydown"===e.type||"keyup"===e.type?e.keyCode:0},which:function(e){return"keypress"===e.type?Kr(e):"keydown"===e.type||"keyup"===e.type?e.keyCode:0}}),ti=Dr.extend({dataTransfer:null}),ni=wr.extend({touches:null,targetTouches:null,changedTouches:null,altKey:null,metaKey:null,ctrlKey:null,shiftKey:null,getModifierState:Tr}),ri=Yn.extend({propertyName:null,elapsedTime:null,pseudoElement:null}),ii=Dr.extend({deltaX:function(e){return"deltaX"in e?e.deltaX:"wheelDeltaX"in e?-e.wheelDeltaX:0},deltaY:function(e){return"deltaY"in e?e.deltaY:"wheelDeltaY"in e?-e.wheelDeltaY:"wheelDelta"in e?-e.wheelDelta:0},deltaZ:null,deltaMode:null}),oi={eventTypes:Rt,extractEvents:function(e,t,n,r){var i=zt.get(e);if(!i)return null;switch(e){case"keypress":if(0===Kr(n))return null;case"keydown":case"keyup":e=ei;break;case"blur":case"focus":e=Qr;break;case"click":if(2===n.button)return null;case"auxclick":case"dblclick":case"mousedown":case"mousemove":case"mouseup":case"mouseout":case"mouseover":case"contextmenu":e=Dr;break;case"drag":case"dragend":case"dragenter":case"dragexit":case"dragleave":case"dragover":case"dragstart":case"drop":e=ti;break;case"touchcancel":case"touchend":case"touchmove":case"touchstart":e=ni;break;case Ge:case Xe:case qe:e=qr;break;case Ze:e=ri;break;case"scroll":e=wr;break;case"wheel":e=ii;break;case"copy":case"cut":case"paste":e=Zr;break;case"gotpointercapture":case"lostpointercapture":case"pointercancel":case"pointerdown":case"pointermove":case"pointerout":case"pointerover":case"pointerup":e=Pr;break;default:e=Yn}return Rn(t=e.getPooled(i,t,n,r)),t}};if(y)throw Error(a(101));y=Array.prototype.slice.call("ResponderEventPlugin SimpleEventPlugin EnterLeaveEventPlugin ChangeEventPlugin SelectEventPlugin BeforeInputEventPlugin".split(" ")),C(),p=On,g=Tn,v=En,M({SimpleEventPlugin:oi,EnterLeaveEventPlugin:jr,ChangeEventPlugin:xr,SelectEventPlugin:Xr,BeforeInputEventPlugin:sr});var ai=[],si=-1;function li(e){0>si||(e.current=ai[si],ai[si]=null,si--)}function ui(e,t){si++,ai[si]=e.current,e.current=t}var ci={},di={current:ci},hi={current:!1},fi=ci;function pi(e,t){var n=e.type.contextTypes;if(!n)return ci;var r=e.stateNode;if(r&&r.__reactInternalMemoizedUnmaskedChildContext===t)return r.__reactInternalMemoizedMaskedChildContext;var i,o={};for(i in n)o[i]=t[i];return r&&((e=e.stateNode).__reactInternalMemoizedUnmaskedChildContext=t,e.__reactInternalMemoizedMaskedChildContext=o),o}function gi(e){return null!=(e=e.childContextTypes)}function vi(){li(hi),li(di)}function mi(e,t,n){if(di.current!==ci)throw Error(a(168));ui(di,t),ui(hi,n)}function yi(e,t,n){var r=e.stateNode;if(e=t.childContextTypes,"function"!=typeof r.getChildContext)return n;for(var o in r=r.getChildContext())if(!(o in e))throw Error(a(108,ve(t)||"Unknown",o));return i({},n,{},r)}function bi(e){return e=(e=e.stateNode)&&e.__reactInternalMemoizedMergedChildContext||ci,fi=di.current,ui(di,e),ui(hi,hi.current),!0}function Ci(e,t,n){var r=e.stateNode;if(!r)throw Error(a(169));n?(e=yi(e,t,fi),r.__reactInternalMemoizedMergedChildContext=e,li(hi),li(di),ui(di,e)):li(hi),ui(hi,n)}var Ai=o.unstable_runWithPriority,_i=o.unstable_scheduleCallback,Ii=o.unstable_cancelCallback,xi=o.unstable_requestPaint,wi=o.unstable_now,Mi=o.unstable_getCurrentPriorityLevel,Si=o.unstable_ImmediatePriority,Ti=o.unstable_UserBlockingPriority,Ei=o.unstable_NormalPriority,Oi=o.unstable_LowPriority,ki=o.unstable_IdlePriority,Li={},Di=o.unstable_shouldYield,Pi=void 0!==xi?xi:function(){},Ni=null,ji=null,Ri=!1,zi=wi(),Fi=1e4>zi?wi:function(){return wi()-zi};function Bi(){switch(Mi()){case Si:return 99;case Ti:return 98;case Ei:return 97;case Oi:return 96;case ki:return 95;default:throw Error(a(332))}}function Hi(e){switch(e){case 99:return Si;case 98:return Ti;case 97:return Ei;case 96:return Oi;case 95:return ki;default:throw Error(a(332))}}function Ui(e,t){return e=Hi(e),Ai(e,t)}function Vi(e,t,n){return e=Hi(e),_i(e,t,n)}function Yi(e){return null===Ni?(Ni=[e],ji=_i(Si,Gi)):Ni.push(e),Li}function Wi(){if(null!==ji){var e=ji;ji=null,Ii(e)}Gi()}function Gi(){if(!Ri&&null!==Ni){Ri=!0;var e=0;try{var t=Ni;Ui(99,(function(){for(;e<t.length;e++){var n=t[e];do{n=n(!0)}while(null!==n)}})),Ni=null}catch(t){throw null!==Ni&&(Ni=Ni.slice(e+1)),_i(Si,Wi),t}finally{Ri=!1}}}function Xi(e,t,n){return 1073741821-(1+((1073741821-e+t/10)/(n/=10)|0))*n}function qi(e,t){if(e&&e.defaultProps)for(var n in t=i({},t),e=e.defaultProps)void 0===t[n]&&(t[n]=e[n]);return t}var Zi={current:null},Qi=null,Ki=null,Ji=null;function $i(){Ji=Ki=Qi=null}function eo(e){var t=Zi.current;li(Zi),e.type._context._currentValue=t}function to(e,t){for(;null!==e;){var n=e.alternate;if(e.childExpirationTime<t)e.childExpirationTime=t,null!==n&&n.childExpirationTime<t&&(n.childExpirationTime=t);else{if(!(null!==n&&n.childExpirationTime<t))break;n.childExpirationTime=t}e=e.return}}function no(e,t){Qi=e,Ji=Ki=null,null!==(e=e.dependencies)&&null!==e.firstContext&&(e.expirationTime>=t&&(Oa=!0),e.firstContext=null)}function ro(e,t){if(Ji!==e&&!1!==t&&0!==t)if("number"==typeof t&&1073741823!==t||(Ji=e,t=1073741823),t={context:e,observedBits:t,next:null},null===Ki){if(null===Qi)throw Error(a(308));Ki=t,Qi.dependencies={expirationTime:0,firstContext:t,responders:null}}else Ki=Ki.next=t;return e._currentValue}var io=!1;function oo(e){e.updateQueue={baseState:e.memoizedState,baseQueue:null,shared:{pending:null},effects:null}}function ao(e,t){e=e.updateQueue,t.updateQueue===e&&(t.updateQueue={baseState:e.baseState,baseQueue:e.baseQueue,shared:e.shared,effects:e.effects})}function so(e,t){return(e={expirationTime:e,suspenseConfig:t,tag:0,payload:null,callback:null,next:null}).next=e}function lo(e,t){if(null!==(e=e.updateQueue)){var n=(e=e.shared).pending;null===n?t.next=t:(t.next=n.next,n.next=t),e.pending=t}}function uo(e,t){var n=e.alternate;null!==n&&ao(n,e),null===(n=(e=e.updateQueue).baseQueue)?(e.baseQueue=t.next=t,t.next=t):(t.next=n.next,n.next=t)}function co(e,t,n,r){var o=e.updateQueue;io=!1;var a=o.baseQueue,s=o.shared.pending;if(null!==s){if(null!==a){var l=a.next;a.next=s.next,s.next=l}a=s,o.shared.pending=null,null!==(l=e.alternate)&&(null!==(l=l.updateQueue)&&(l.baseQueue=s))}if(null!==a){l=a.next;var u=o.baseState,c=0,d=null,h=null,f=null;if(null!==l)for(var p=l;;){if((s=p.expirationTime)<r){var g={expirationTime:p.expirationTime,suspenseConfig:p.suspenseConfig,tag:p.tag,payload:p.payload,callback:p.callback,next:null};null===f?(h=f=g,d=u):f=f.next=g,s>c&&(c=s)}else{null!==f&&(f=f.next={expirationTime:1073741823,suspenseConfig:p.suspenseConfig,tag:p.tag,payload:p.payload,callback:p.callback,next:null}),ol(s,p.suspenseConfig);e:{var v=e,m=p;switch(s=t,g=n,m.tag){case 1:if("function"==typeof(v=m.payload)){u=v.call(g,u,s);break e}u=v;break e;case 3:v.effectTag=-4097&v.effectTag|64;case 0:if(null==(s="function"==typeof(v=m.payload)?v.call(g,u,s):v))break e;u=i({},u,s);break e;case 2:io=!0}}null!==p.callback&&(e.effectTag|=32,null===(s=o.effects)?o.effects=[p]:s.push(p))}if(null===(p=p.next)||p===l){if(null===(s=o.shared.pending))break;p=a.next=s.next,s.next=l,o.baseQueue=a=s,o.shared.pending=null}}null===f?d=u:f.next=h,o.baseState=d,o.baseQueue=f,al(c),e.expirationTime=c,e.memoizedState=u}}function ho(e,t,n){if(e=t.effects,t.effects=null,null!==e)for(t=0;t<e.length;t++){var r=e[t],i=r.callback;if(null!==i){if(r.callback=null,r=i,i=n,"function"!=typeof r)throw Error(a(191,r));r.call(i)}}}var fo=Q.ReactCurrentBatchConfig,po=(new r.Component).refs;function go(e,t,n,r){n=null==(n=n(r,t=e.memoizedState))?t:i({},t,n),e.memoizedState=n,0===e.expirationTime&&(e.updateQueue.baseState=n)}var vo={isMounted:function(e){return!!(e=e._reactInternalFiber)&&$e(e)===e},enqueueSetState:function(e,t,n){e=e._reactInternalFiber;var r=Gs(),i=fo.suspense;(i=so(r=Xs(r,e,i),i)).payload=t,null!=n&&(i.callback=n),lo(e,i),qs(e,r)},enqueueReplaceState:function(e,t,n){e=e._reactInternalFiber;var r=Gs(),i=fo.suspense;(i=so(r=Xs(r,e,i),i)).tag=1,i.payload=t,null!=n&&(i.callback=n),lo(e,i),qs(e,r)},enqueueForceUpdate:function(e,t){e=e._reactInternalFiber;var n=Gs(),r=fo.suspense;(r=so(n=Xs(n,e,r),r)).tag=2,null!=t&&(r.callback=t),lo(e,r),qs(e,n)}};function mo(e,t,n,r,i,o,a){return"function"==typeof(e=e.stateNode).shouldComponentUpdate?e.shouldComponentUpdate(r,o,a):!t.prototype||!t.prototype.isPureReactComponent||(!Fr(n,r)||!Fr(i,o))}function yo(e,t,n){var r=!1,i=ci,o=t.contextType;return"object"==typeof o&&null!==o?o=ro(o):(i=gi(t)?fi:di.current,o=(r=null!=(r=t.contextTypes))?pi(e,i):ci),t=new t(n,o),e.memoizedState=null!==t.state&&void 0!==t.state?t.state:null,t.updater=vo,e.stateNode=t,t._reactInternalFiber=e,r&&((e=e.stateNode).__reactInternalMemoizedUnmaskedChildContext=i,e.__reactInternalMemoizedMaskedChildContext=o),t}function bo(e,t,n,r){e=t.state,"function"==typeof t.componentWillReceiveProps&&t.componentWillReceiveProps(n,r),"function"==typeof t.UNSAFE_componentWillReceiveProps&&t.UNSAFE_componentWillReceiveProps(n,r),t.state!==e&&vo.enqueueReplaceState(t,t.state,null)}function Co(e,t,n,r){var i=e.stateNode;i.props=n,i.state=e.memoizedState,i.refs=po,oo(e);var o=t.contextType;"object"==typeof o&&null!==o?i.context=ro(o):(o=gi(t)?fi:di.current,i.context=pi(e,o)),co(e,n,i,r),i.state=e.memoizedState,"function"==typeof(o=t.getDerivedStateFromProps)&&(go(e,t,o,n),i.state=e.memoizedState),"function"==typeof t.getDerivedStateFromProps||"function"==typeof i.getSnapshotBeforeUpdate||"function"!=typeof i.UNSAFE_componentWillMount&&"function"!=typeof i.componentWillMount||(t=i.state,"function"==typeof i.componentWillMount&&i.componentWillMount(),"function"==typeof i.UNSAFE_componentWillMount&&i.UNSAFE_componentWillMount(),t!==i.state&&vo.enqueueReplaceState(i,i.state,null),co(e,n,i,r),i.state=e.memoizedState),"function"==typeof i.componentDidMount&&(e.effectTag|=4)}var Ao=Array.isArray;function _o(e,t,n){if(null!==(e=n.ref)&&"function"!=typeof e&&"object"!=typeof e){if(n._owner){if(n=n._owner){if(1!==n.tag)throw Error(a(309));var r=n.stateNode}if(!r)throw Error(a(147,e));var i=""+e;return null!==t&&null!==t.ref&&"function"==typeof t.ref&&t.ref._stringRef===i?t.ref:((t=function(e){var t=r.refs;t===po&&(t=r.refs={}),null===e?delete t[i]:t[i]=e})._stringRef=i,t)}if("string"!=typeof e)throw Error(a(284));if(!n._owner)throw Error(a(290,e))}return e}function Io(e,t){if("textarea"!==e.type)throw Error(a(31,"[object Object]"===Object.prototype.toString.call(t)?"object with keys {"+Object.keys(t).join(", ")+"}":t,""))}function xo(e){function t(t,n){if(e){var r=t.lastEffect;null!==r?(r.nextEffect=n,t.lastEffect=n):t.firstEffect=t.lastEffect=n,n.nextEffect=null,n.effectTag=8}}function n(n,r){if(!e)return null;for(;null!==r;)t(n,r),r=r.sibling;return null}function r(e,t){for(e=new Map;null!==t;)null!==t.key?e.set(t.key,t):e.set(t.index,t),t=t.sibling;return e}function i(e,t){return(e=Ml(e,t)).index=0,e.sibling=null,e}function o(t,n,r){return t.index=r,e?null!==(r=t.alternate)?(r=r.index)<n?(t.effectTag=2,n):r:(t.effectTag=2,n):n}function s(t){return e&&null===t.alternate&&(t.effectTag=2),t}function l(e,t,n,r){return null===t||6!==t.tag?((t=El(n,e.mode,r)).return=e,t):((t=i(t,n)).return=e,t)}function u(e,t,n,r){return null!==t&&t.elementType===n.type?((r=i(t,n.props)).ref=_o(e,t,n),r.return=e,r):((r=Sl(n.type,n.key,n.props,null,e.mode,r)).ref=_o(e,t,n),r.return=e,r)}function c(e,t,n,r){return null===t||4!==t.tag||t.stateNode.containerInfo!==n.containerInfo||t.stateNode.implementation!==n.implementation?((t=Ol(n,e.mode,r)).return=e,t):((t=i(t,n.children||[])).return=e,t)}function d(e,t,n,r,o){return null===t||7!==t.tag?((t=Tl(n,e.mode,r,o)).return=e,t):((t=i(t,n)).return=e,t)}function h(e,t,n){if("string"==typeof t||"number"==typeof t)return(t=El(""+t,e.mode,n)).return=e,t;if("object"==typeof t&&null!==t){switch(t.$$typeof){case ee:return(n=Sl(t.type,t.key,t.props,null,e.mode,n)).ref=_o(e,null,t),n.return=e,n;case te:return(t=Ol(t,e.mode,n)).return=e,t}if(Ao(t)||ge(t))return(t=Tl(t,e.mode,n,null)).return=e,t;Io(e,t)}return null}function f(e,t,n,r){var i=null!==t?t.key:null;if("string"==typeof n||"number"==typeof n)return null!==i?null:l(e,t,""+n,r);if("object"==typeof n&&null!==n){switch(n.$$typeof){case ee:return n.key===i?n.type===ne?d(e,t,n.props.children,r,i):u(e,t,n,r):null;case te:return n.key===i?c(e,t,n,r):null}if(Ao(n)||ge(n))return null!==i?null:d(e,t,n,r,null);Io(e,n)}return null}function p(e,t,n,r,i){if("string"==typeof r||"number"==typeof r)return l(t,e=e.get(n)||null,""+r,i);if("object"==typeof r&&null!==r){switch(r.$$typeof){case ee:return e=e.get(null===r.key?n:r.key)||null,r.type===ne?d(t,e,r.props.children,i,r.key):u(t,e,r,i);case te:return c(t,e=e.get(null===r.key?n:r.key)||null,r,i)}if(Ao(r)||ge(r))return d(t,e=e.get(n)||null,r,i,null);Io(t,r)}return null}function g(i,a,s,l){for(var u=null,c=null,d=a,g=a=0,v=null;null!==d&&g<s.length;g++){d.index>g?(v=d,d=null):v=d.sibling;var m=f(i,d,s[g],l);if(null===m){null===d&&(d=v);break}e&&d&&null===m.alternate&&t(i,d),a=o(m,a,g),null===c?u=m:c.sibling=m,c=m,d=v}if(g===s.length)return n(i,d),u;if(null===d){for(;g<s.length;g++)null!==(d=h(i,s[g],l))&&(a=o(d,a,g),null===c?u=d:c.sibling=d,c=d);return u}for(d=r(i,d);g<s.length;g++)null!==(v=p(d,i,g,s[g],l))&&(e&&null!==v.alternate&&d.delete(null===v.key?g:v.key),a=o(v,a,g),null===c?u=v:c.sibling=v,c=v);return e&&d.forEach((function(e){return t(i,e)})),u}function v(i,s,l,u){var c=ge(l);if("function"!=typeof c)throw Error(a(150));if(null==(l=c.call(l)))throw Error(a(151));for(var d=c=null,g=s,v=s=0,m=null,y=l.next();null!==g&&!y.done;v++,y=l.next()){g.index>v?(m=g,g=null):m=g.sibling;var b=f(i,g,y.value,u);if(null===b){null===g&&(g=m);break}e&&g&&null===b.alternate&&t(i,g),s=o(b,s,v),null===d?c=b:d.sibling=b,d=b,g=m}if(y.done)return n(i,g),c;if(null===g){for(;!y.done;v++,y=l.next())null!==(y=h(i,y.value,u))&&(s=o(y,s,v),null===d?c=y:d.sibling=y,d=y);return c}for(g=r(i,g);!y.done;v++,y=l.next())null!==(y=p(g,i,v,y.value,u))&&(e&&null!==y.alternate&&g.delete(null===y.key?v:y.key),s=o(y,s,v),null===d?c=y:d.sibling=y,d=y);return e&&g.forEach((function(e){return t(i,e)})),c}return function(e,r,o,l){var u="object"==typeof o&&null!==o&&o.type===ne&&null===o.key;u&&(o=o.props.children);var c="object"==typeof o&&null!==o;if(c)switch(o.$$typeof){case ee:e:{for(c=o.key,u=r;null!==u;){if(u.key===c){switch(u.tag){case 7:if(o.type===ne){n(e,u.sibling),(r=i(u,o.props.children)).return=e,e=r;break e}break;default:if(u.elementType===o.type){n(e,u.sibling),(r=i(u,o.props)).ref=_o(e,u,o),r.return=e,e=r;break e}}n(e,u);break}t(e,u),u=u.sibling}o.type===ne?((r=Tl(o.props.children,e.mode,l,o.key)).return=e,e=r):((l=Sl(o.type,o.key,o.props,null,e.mode,l)).ref=_o(e,r,o),l.return=e,e=l)}return s(e);case te:e:{for(u=o.key;null!==r;){if(r.key===u){if(4===r.tag&&r.stateNode.containerInfo===o.containerInfo&&r.stateNode.implementation===o.implementation){n(e,r.sibling),(r=i(r,o.children||[])).return=e,e=r;break e}n(e,r);break}t(e,r),r=r.sibling}(r=Ol(o,e.mode,l)).return=e,e=r}return s(e)}if("string"==typeof o||"number"==typeof o)return o=""+o,null!==r&&6===r.tag?(n(e,r.sibling),(r=i(r,o)).return=e,e=r):(n(e,r),(r=El(o,e.mode,l)).return=e,e=r),s(e);if(Ao(o))return g(e,r,o,l);if(ge(o))return v(e,r,o,l);if(c&&Io(e,o),void 0===o&&!u)switch(e.tag){case 1:case 0:throw e=e.type,Error(a(152,e.displayName||e.name||"Component"))}return n(e,r)}}var wo=xo(!0),Mo=xo(!1),So={},To={current:So},Eo={current:So},Oo={current:So};function ko(e){if(e===So)throw Error(a(174));return e}function Lo(e,t){switch(ui(Oo,t),ui(Eo,e),ui(To,So),e=t.nodeType){case 9:case 11:t=(t=t.documentElement)?t.namespaceURI:Re(null,"");break;default:t=Re(t=(e=8===e?t.parentNode:t).namespaceURI||null,e=e.tagName)}li(To),ui(To,t)}function Do(){li(To),li(Eo),li(Oo)}function Po(e){ko(Oo.current);var t=ko(To.current),n=Re(t,e.type);t!==n&&(ui(Eo,e),ui(To,n))}function No(e){Eo.current===e&&(li(To),li(Eo))}var jo={current:0};function Ro(e){for(var t=e;null!==t;){if(13===t.tag){var n=t.memoizedState;if(null!==n&&(null===(n=n.dehydrated)||"$?"===n.data||"$!"===n.data))return t}else if(19===t.tag&&void 0!==t.memoizedProps.revealOrder){if(0!=(64&t.effectTag))return t}else if(null!==t.child){t.child.return=t,t=t.child;continue}if(t===e)break;for(;null===t.sibling;){if(null===t.return||t.return===e)return null;t=t.return}t.sibling.return=t.return,t=t.sibling}return null}function zo(e,t){return{responder:e,props:t}}var Fo=Q.ReactCurrentDispatcher,Bo=Q.ReactCurrentBatchConfig,Ho=0,Uo=null,Vo=null,Yo=null,Wo=!1;function Go(){throw Error(a(321))}function Xo(e,t){if(null===t)return!1;for(var n=0;n<t.length&&n<e.length;n++)if(!Rr(e[n],t[n]))return!1;return!0}function qo(e,t,n,r,i,o){if(Ho=o,Uo=t,t.memoizedState=null,t.updateQueue=null,t.expirationTime=0,Fo.current=null===e||null===e.memoizedState?ma:ya,e=n(r,i),t.expirationTime===Ho){o=0;do{if(t.expirationTime=0,!(25>o))throw Error(a(301));o+=1,Yo=Vo=null,t.updateQueue=null,Fo.current=ba,e=n(r,i)}while(t.expirationTime===Ho)}if(Fo.current=va,t=null!==Vo&&null!==Vo.next,Ho=0,Yo=Vo=Uo=null,Wo=!1,t)throw Error(a(300));return e}function Zo(){var e={memoizedState:null,baseState:null,baseQueue:null,queue:null,next:null};return null===Yo?Uo.memoizedState=Yo=e:Yo=Yo.next=e,Yo}function Qo(){if(null===Vo){var e=Uo.alternate;e=null!==e?e.memoizedState:null}else e=Vo.next;var t=null===Yo?Uo.memoizedState:Yo.next;if(null!==t)Yo=t,Vo=e;else{if(null===e)throw Error(a(310));e={memoizedState:(Vo=e).memoizedState,baseState:Vo.baseState,baseQueue:Vo.baseQueue,queue:Vo.queue,next:null},null===Yo?Uo.memoizedState=Yo=e:Yo=Yo.next=e}return Yo}function Ko(e,t){return"function"==typeof t?t(e):t}function Jo(e){var t=Qo(),n=t.queue;if(null===n)throw Error(a(311));n.lastRenderedReducer=e;var r=Vo,i=r.baseQueue,o=n.pending;if(null!==o){if(null!==i){var s=i.next;i.next=o.next,o.next=s}r.baseQueue=i=o,n.pending=null}if(null!==i){i=i.next,r=r.baseState;var l=s=o=null,u=i;do{var c=u.expirationTime;if(c<Ho){var d={expirationTime:u.expirationTime,suspenseConfig:u.suspenseConfig,action:u.action,eagerReducer:u.eagerReducer,eagerState:u.eagerState,next:null};null===l?(s=l=d,o=r):l=l.next=d,c>Uo.expirationTime&&(Uo.expirationTime=c,al(c))}else null!==l&&(l=l.next={expirationTime:1073741823,suspenseConfig:u.suspenseConfig,action:u.action,eagerReducer:u.eagerReducer,eagerState:u.eagerState,next:null}),ol(c,u.suspenseConfig),r=u.eagerReducer===e?u.eagerState:e(r,u.action);u=u.next}while(null!==u&&u!==i);null===l?o=r:l.next=s,Rr(r,t.memoizedState)||(Oa=!0),t.memoizedState=r,t.baseState=o,t.baseQueue=l,n.lastRenderedState=r}return[t.memoizedState,n.dispatch]}function $o(e){var t=Qo(),n=t.queue;if(null===n)throw Error(a(311));n.lastRenderedReducer=e;var r=n.dispatch,i=n.pending,o=t.memoizedState;if(null!==i){n.pending=null;var s=i=i.next;do{o=e(o,s.action),s=s.next}while(s!==i);Rr(o,t.memoizedState)||(Oa=!0),t.memoizedState=o,null===t.baseQueue&&(t.baseState=o),n.lastRenderedState=o}return[o,r]}function ea(e){var t=Zo();return"function"==typeof e&&(e=e()),t.memoizedState=t.baseState=e,e=(e=t.queue={pending:null,dispatch:null,lastRenderedReducer:Ko,lastRenderedState:e}).dispatch=ga.bind(null,Uo,e),[t.memoizedState,e]}function ta(e,t,n,r){return e={tag:e,create:t,destroy:n,deps:r,next:null},null===(t=Uo.updateQueue)?(t={lastEffect:null},Uo.updateQueue=t,t.lastEffect=e.next=e):null===(n=t.lastEffect)?t.lastEffect=e.next=e:(r=n.next,n.next=e,e.next=r,t.lastEffect=e),e}function na(){return Qo().memoizedState}function ra(e,t,n,r){var i=Zo();Uo.effectTag|=e,i.memoizedState=ta(1|t,n,void 0,void 0===r?null:r)}function ia(e,t,n,r){var i=Qo();r=void 0===r?null:r;var o=void 0;if(null!==Vo){var a=Vo.memoizedState;if(o=a.destroy,null!==r&&Xo(r,a.deps))return void ta(t,n,o,r)}Uo.effectTag|=e,i.memoizedState=ta(1|t,n,o,r)}function oa(e,t){return ra(516,4,e,t)}function aa(e,t){return ia(516,4,e,t)}function sa(e,t){return ia(4,2,e,t)}function la(e,t){return"function"==typeof t?(e=e(),t(e),function(){t(null)}):null!=t?(e=e(),t.current=e,function(){t.current=null}):void 0}function ua(e,t,n){return n=null!=n?n.concat([e]):null,ia(4,2,la.bind(null,t,e),n)}function ca(){}function da(e,t){return Zo().memoizedState=[e,void 0===t?null:t],e}function ha(e,t){var n=Qo();t=void 0===t?null:t;var r=n.memoizedState;return null!==r&&null!==t&&Xo(t,r[1])?r[0]:(n.memoizedState=[e,t],e)}function fa(e,t){var n=Qo();t=void 0===t?null:t;var r=n.memoizedState;return null!==r&&null!==t&&Xo(t,r[1])?r[0]:(e=e(),n.memoizedState=[e,t],e)}function pa(e,t,n){var r=Bi();Ui(98>r?98:r,(function(){e(!0)})),Ui(97<r?97:r,(function(){var r=Bo.suspense;Bo.suspense=void 0===t?null:t;try{e(!1),n()}finally{Bo.suspense=r}}))}function ga(e,t,n){var r=Gs(),i=fo.suspense;i={expirationTime:r=Xs(r,e,i),suspenseConfig:i,action:n,eagerReducer:null,eagerState:null,next:null};var o=t.pending;if(null===o?i.next=i:(i.next=o.next,o.next=i),t.pending=i,o=e.alternate,e===Uo||null!==o&&o===Uo)Wo=!0,i.expirationTime=Ho,Uo.expirationTime=Ho;else{if(0===e.expirationTime&&(null===o||0===o.expirationTime)&&null!==(o=t.lastRenderedReducer))try{var a=t.lastRenderedState,s=o(a,n);if(i.eagerReducer=o,i.eagerState=s,Rr(s,a))return}catch(e){}qs(e,r)}}var va={readContext:ro,useCallback:Go,useContext:Go,useEffect:Go,useImperativeHandle:Go,useLayoutEffect:Go,useMemo:Go,useReducer:Go,useRef:Go,useState:Go,useDebugValue:Go,useResponder:Go,useDeferredValue:Go,useTransition:Go},ma={readContext:ro,useCallback:da,useContext:ro,useEffect:oa,useImperativeHandle:function(e,t,n){return n=null!=n?n.concat([e]):null,ra(4,2,la.bind(null,t,e),n)},useLayoutEffect:function(e,t){return ra(4,2,e,t)},useMemo:function(e,t){var n=Zo();return t=void 0===t?null:t,e=e(),n.memoizedState=[e,t],e},useReducer:function(e,t,n){var r=Zo();return t=void 0!==n?n(t):t,r.memoizedState=r.baseState=t,e=(e=r.queue={pending:null,dispatch:null,lastRenderedReducer:e,lastRenderedState:t}).dispatch=ga.bind(null,Uo,e),[r.memoizedState,e]},useRef:function(e){return e={current:e},Zo().memoizedState=e},useState:ea,useDebugValue:ca,useResponder:zo,useDeferredValue:function(e,t){var n=ea(e),r=n[0],i=n[1];return oa((function(){var n=Bo.suspense;Bo.suspense=void 0===t?null:t;try{i(e)}finally{Bo.suspense=n}}),[e,t]),r},useTransition:function(e){var t=ea(!1),n=t[0];return t=t[1],[da(pa.bind(null,t,e),[t,e]),n]}},ya={readContext:ro,useCallback:ha,useContext:ro,useEffect:aa,useImperativeHandle:ua,useLayoutEffect:sa,useMemo:fa,useReducer:Jo,useRef:na,useState:function(){return Jo(Ko)},useDebugValue:ca,useResponder:zo,useDeferredValue:function(e,t){var n=Jo(Ko),r=n[0],i=n[1];return aa((function(){var n=Bo.suspense;Bo.suspense=void 0===t?null:t;try{i(e)}finally{Bo.suspense=n}}),[e,t]),r},useTransition:function(e){var t=Jo(Ko),n=t[0];return t=t[1],[ha(pa.bind(null,t,e),[t,e]),n]}},ba={readContext:ro,useCallback:ha,useContext:ro,useEffect:aa,useImperativeHandle:ua,useLayoutEffect:sa,useMemo:fa,useReducer:$o,useRef:na,useState:function(){return $o(Ko)},useDebugValue:ca,useResponder:zo,useDeferredValue:function(e,t){var n=$o(Ko),r=n[0],i=n[1];return aa((function(){var n=Bo.suspense;Bo.suspense=void 0===t?null:t;try{i(e)}finally{Bo.suspense=n}}),[e,t]),r},useTransition:function(e){var t=$o(Ko),n=t[0];return t=t[1],[ha(pa.bind(null,t,e),[t,e]),n]}},Ca=null,Aa=null,_a=!1;function Ia(e,t){var n=xl(5,null,null,0);n.elementType="DELETED",n.type="DELETED",n.stateNode=t,n.return=e,n.effectTag=8,null!==e.lastEffect?(e.lastEffect.nextEffect=n,e.lastEffect=n):e.firstEffect=e.lastEffect=n}function xa(e,t){switch(e.tag){case 5:var n=e.type;return null!==(t=1!==t.nodeType||n.toLowerCase()!==t.nodeName.toLowerCase()?null:t)&&(e.stateNode=t,!0);case 6:return null!==(t=""===e.pendingProps||3!==t.nodeType?null:t)&&(e.stateNode=t,!0);case 13:default:return!1}}function wa(e){if(_a){var t=Aa;if(t){var n=t;if(!xa(e,t)){if(!(t=An(n.nextSibling))||!xa(e,t))return e.effectTag=-1025&e.effectTag|2,_a=!1,void(Ca=e);Ia(Ca,n)}Ca=e,Aa=An(t.firstChild)}else e.effectTag=-1025&e.effectTag|2,_a=!1,Ca=e}}function Ma(e){for(e=e.return;null!==e&&5!==e.tag&&3!==e.tag&&13!==e.tag;)e=e.return;Ca=e}function Sa(e){if(e!==Ca)return!1;if(!_a)return Ma(e),_a=!0,!1;var t=e.type;if(5!==e.tag||"head"!==t&&"body"!==t&&!yn(t,e.memoizedProps))for(t=Aa;t;)Ia(e,t),t=An(t.nextSibling);if(Ma(e),13===e.tag){if(!(e=null!==(e=e.memoizedState)?e.dehydrated:null))throw Error(a(317));e:{for(e=e.nextSibling,t=0;e;){if(8===e.nodeType){var n=e.data;if("/$"===n){if(0===t){Aa=An(e.nextSibling);break e}t--}else"$"!==n&&"$!"!==n&&"$?"!==n||t++}e=e.nextSibling}Aa=null}}else Aa=Ca?An(e.stateNode.nextSibling):null;return!0}function Ta(){Aa=Ca=null,_a=!1}var Ea=Q.ReactCurrentOwner,Oa=!1;function ka(e,t,n,r){t.child=null===e?Mo(t,null,n,r):wo(t,e.child,n,r)}function La(e,t,n,r,i){n=n.render;var o=t.ref;return no(t,i),r=qo(e,t,n,r,o,i),null===e||Oa?(t.effectTag|=1,ka(e,t,r,i),t.child):(t.updateQueue=e.updateQueue,t.effectTag&=-517,e.expirationTime<=i&&(e.expirationTime=0),qa(e,t,i))}function Da(e,t,n,r,i,o){if(null===e){var a=n.type;return"function"!=typeof a||wl(a)||void 0!==a.defaultProps||null!==n.compare||void 0!==n.defaultProps?((e=Sl(n.type,null,r,null,t.mode,o)).ref=t.ref,e.return=t,t.child=e):(t.tag=15,t.type=a,Pa(e,t,a,r,i,o))}return a=e.child,i<o&&(i=a.memoizedProps,(n=null!==(n=n.compare)?n:Fr)(i,r)&&e.ref===t.ref)?qa(e,t,o):(t.effectTag|=1,(e=Ml(a,r)).ref=t.ref,e.return=t,t.child=e)}function Pa(e,t,n,r,i,o){return null!==e&&Fr(e.memoizedProps,r)&&e.ref===t.ref&&(Oa=!1,i<o)?(t.expirationTime=e.expirationTime,qa(e,t,o)):ja(e,t,n,r,o)}function Na(e,t){var n=t.ref;(null===e&&null!==n||null!==e&&e.ref!==n)&&(t.effectTag|=128)}function ja(e,t,n,r,i){var o=gi(n)?fi:di.current;return o=pi(t,o),no(t,i),n=qo(e,t,n,r,o,i),null===e||Oa?(t.effectTag|=1,ka(e,t,n,i),t.child):(t.updateQueue=e.updateQueue,t.effectTag&=-517,e.expirationTime<=i&&(e.expirationTime=0),qa(e,t,i))}function Ra(e,t,n,r,i){if(gi(n)){var o=!0;bi(t)}else o=!1;if(no(t,i),null===t.stateNode)null!==e&&(e.alternate=null,t.alternate=null,t.effectTag|=2),yo(t,n,r),Co(t,n,r,i),r=!0;else if(null===e){var a=t.stateNode,s=t.memoizedProps;a.props=s;var l=a.context,u=n.contextType;"object"==typeof u&&null!==u?u=ro(u):u=pi(t,u=gi(n)?fi:di.current);var c=n.getDerivedStateFromProps,d="function"==typeof c||"function"==typeof a.getSnapshotBeforeUpdate;d||"function"!=typeof a.UNSAFE_componentWillReceiveProps&&"function"!=typeof a.componentWillReceiveProps||(s!==r||l!==u)&&bo(t,a,r,u),io=!1;var h=t.memoizedState;a.state=h,co(t,r,a,i),l=t.memoizedState,s!==r||h!==l||hi.current||io?("function"==typeof c&&(go(t,n,c,r),l=t.memoizedState),(s=io||mo(t,n,s,r,h,l,u))?(d||"function"!=typeof a.UNSAFE_componentWillMount&&"function"!=typeof a.componentWillMount||("function"==typeof a.componentWillMount&&a.componentWillMount(),"function"==typeof a.UNSAFE_componentWillMount&&a.UNSAFE_componentWillMount()),"function"==typeof a.componentDidMount&&(t.effectTag|=4)):("function"==typeof a.componentDidMount&&(t.effectTag|=4),t.memoizedProps=r,t.memoizedState=l),a.props=r,a.state=l,a.context=u,r=s):("function"==typeof a.componentDidMount&&(t.effectTag|=4),r=!1)}else a=t.stateNode,ao(e,t),s=t.memoizedProps,a.props=t.type===t.elementType?s:qi(t.type,s),l=a.context,"object"==typeof(u=n.contextType)&&null!==u?u=ro(u):u=pi(t,u=gi(n)?fi:di.current),(d="function"==typeof(c=n.getDerivedStateFromProps)||"function"==typeof a.getSnapshotBeforeUpdate)||"function"!=typeof a.UNSAFE_componentWillReceiveProps&&"function"!=typeof a.componentWillReceiveProps||(s!==r||l!==u)&&bo(t,a,r,u),io=!1,l=t.memoizedState,a.state=l,co(t,r,a,i),h=t.memoizedState,s!==r||l!==h||hi.current||io?("function"==typeof c&&(go(t,n,c,r),h=t.memoizedState),(c=io||mo(t,n,s,r,l,h,u))?(d||"function"!=typeof a.UNSAFE_componentWillUpdate&&"function"!=typeof a.componentWillUpdate||("function"==typeof a.componentWillUpdate&&a.componentWillUpdate(r,h,u),"function"==typeof a.UNSAFE_componentWillUpdate&&a.UNSAFE_componentWillUpdate(r,h,u)),"function"==typeof a.componentDidUpdate&&(t.effectTag|=4),"function"==typeof a.getSnapshotBeforeUpdate&&(t.effectTag|=256)):("function"!=typeof a.componentDidUpdate||s===e.memoizedProps&&l===e.memoizedState||(t.effectTag|=4),"function"!=typeof a.getSnapshotBeforeUpdate||s===e.memoizedProps&&l===e.memoizedState||(t.effectTag|=256),t.memoizedProps=r,t.memoizedState=h),a.props=r,a.state=h,a.context=u,r=c):("function"!=typeof a.componentDidUpdate||s===e.memoizedProps&&l===e.memoizedState||(t.effectTag|=4),"function"!=typeof a.getSnapshotBeforeUpdate||s===e.memoizedProps&&l===e.memoizedState||(t.effectTag|=256),r=!1);return za(e,t,n,r,o,i)}function za(e,t,n,r,i,o){Na(e,t);var a=0!=(64&t.effectTag);if(!r&&!a)return i&&Ci(t,n,!1),qa(e,t,o);r=t.stateNode,Ea.current=t;var s=a&&"function"!=typeof n.getDerivedStateFromError?null:r.render();return t.effectTag|=1,null!==e&&a?(t.child=wo(t,e.child,null,o),t.child=wo(t,null,s,o)):ka(e,t,s,o),t.memoizedState=r.state,i&&Ci(t,n,!0),t.child}function Fa(e){var t=e.stateNode;t.pendingContext?mi(0,t.pendingContext,t.pendingContext!==t.context):t.context&&mi(0,t.context,!1),Lo(e,t.containerInfo)}var Ba,Ha,Ua,Va={dehydrated:null,retryTime:0};function Ya(e,t,n){var r,i=t.mode,o=t.pendingProps,a=jo.current,s=!1;if((r=0!=(64&t.effectTag))||(r=0!=(2&a)&&(null===e||null!==e.memoizedState)),r?(s=!0,t.effectTag&=-65):null!==e&&null===e.memoizedState||void 0===o.fallback||!0===o.unstable_avoidThisFallback||(a|=1),ui(jo,1&a),null===e){if(void 0!==o.fallback&&wa(t),s){if(s=o.fallback,(o=Tl(null,i,0,null)).return=t,0==(2&t.mode))for(e=null!==t.memoizedState?t.child.child:t.child,o.child=e;null!==e;)e.return=o,e=e.sibling;return(n=Tl(s,i,n,null)).return=t,o.sibling=n,t.memoizedState=Va,t.child=o,n}return i=o.children,t.memoizedState=null,t.child=Mo(t,null,i,n)}if(null!==e.memoizedState){if(i=(e=e.child).sibling,s){if(o=o.fallback,(n=Ml(e,e.pendingProps)).return=t,0==(2&t.mode)&&(s=null!==t.memoizedState?t.child.child:t.child)!==e.child)for(n.child=s;null!==s;)s.return=n,s=s.sibling;return(i=Ml(i,o)).return=t,n.sibling=i,n.childExpirationTime=0,t.memoizedState=Va,t.child=n,i}return n=wo(t,e.child,o.children,n),t.memoizedState=null,t.child=n}if(e=e.child,s){if(s=o.fallback,(o=Tl(null,i,0,null)).return=t,o.child=e,null!==e&&(e.return=o),0==(2&t.mode))for(e=null!==t.memoizedState?t.child.child:t.child,o.child=e;null!==e;)e.return=o,e=e.sibling;return(n=Tl(s,i,n,null)).return=t,o.sibling=n,n.effectTag|=2,o.childExpirationTime=0,t.memoizedState=Va,t.child=o,n}return t.memoizedState=null,t.child=wo(t,e,o.children,n)}function Wa(e,t){e.expirationTime<t&&(e.expirationTime=t);var n=e.alternate;null!==n&&n.expirationTime<t&&(n.expirationTime=t),to(e.return,t)}function Ga(e,t,n,r,i,o){var a=e.memoizedState;null===a?e.memoizedState={isBackwards:t,rendering:null,renderingStartTime:0,last:r,tail:n,tailExpiration:0,tailMode:i,lastEffect:o}:(a.isBackwards=t,a.rendering=null,a.renderingStartTime=0,a.last=r,a.tail=n,a.tailExpiration=0,a.tailMode=i,a.lastEffect=o)}function Xa(e,t,n){var r=t.pendingProps,i=r.revealOrder,o=r.tail;if(ka(e,t,r.children,n),0!=(2&(r=jo.current)))r=1&r|2,t.effectTag|=64;else{if(null!==e&&0!=(64&e.effectTag))e:for(e=t.child;null!==e;){if(13===e.tag)null!==e.memoizedState&&Wa(e,n);else if(19===e.tag)Wa(e,n);else if(null!==e.child){e.child.return=e,e=e.child;continue}if(e===t)break e;for(;null===e.sibling;){if(null===e.return||e.return===t)break e;e=e.return}e.sibling.return=e.return,e=e.sibling}r&=1}if(ui(jo,r),0==(2&t.mode))t.memoizedState=null;else switch(i){case"forwards":for(n=t.child,i=null;null!==n;)null!==(e=n.alternate)&&null===Ro(e)&&(i=n),n=n.sibling;null===(n=i)?(i=t.child,t.child=null):(i=n.sibling,n.sibling=null),Ga(t,!1,i,n,o,t.lastEffect);break;case"backwards":for(n=null,i=t.child,t.child=null;null!==i;){if(null!==(e=i.alternate)&&null===Ro(e)){t.child=i;break}e=i.sibling,i.sibling=n,n=i,i=e}Ga(t,!0,n,null,o,t.lastEffect);break;case"together":Ga(t,!1,null,null,void 0,t.lastEffect);break;default:t.memoizedState=null}return t.child}function qa(e,t,n){null!==e&&(t.dependencies=e.dependencies);var r=t.expirationTime;if(0!==r&&al(r),t.childExpirationTime<n)return null;if(null!==e&&t.child!==e.child)throw Error(a(153));if(null!==t.child){for(n=Ml(e=t.child,e.pendingProps),t.child=n,n.return=t;null!==e.sibling;)e=e.sibling,(n=n.sibling=Ml(e,e.pendingProps)).return=t;n.sibling=null}return t.child}function Za(e,t){switch(e.tailMode){case"hidden":t=e.tail;for(var n=null;null!==t;)null!==t.alternate&&(n=t),t=t.sibling;null===n?e.tail=null:n.sibling=null;break;case"collapsed":n=e.tail;for(var r=null;null!==n;)null!==n.alternate&&(r=n),n=n.sibling;null===r?t||null===e.tail?e.tail=null:e.tail.sibling=null:r.sibling=null}}function Qa(e,t,n){var r=t.pendingProps;switch(t.tag){case 2:case 16:case 15:case 0:case 11:case 7:case 8:case 12:case 9:case 14:return null;case 1:return gi(t.type)&&vi(),null;case 3:return Do(),li(hi),li(di),(n=t.stateNode).pendingContext&&(n.context=n.pendingContext,n.pendingContext=null),null!==e&&null!==e.child||!Sa(t)||(t.effectTag|=4),null;case 5:No(t),n=ko(Oo.current);var o=t.type;if(null!==e&&null!=t.stateNode)Ha(e,t,o,r,n),e.ref!==t.ref&&(t.effectTag|=128);else{if(!r){if(null===t.stateNode)throw Error(a(166));return null}if(e=ko(To.current),Sa(t)){r=t.stateNode,o=t.type;var s=t.memoizedProps;switch(r[xn]=t,r[wn]=s,o){case"iframe":case"object":case"embed":Xt("load",r);break;case"video":case"audio":for(e=0;e<Qe.length;e++)Xt(Qe[e],r);break;case"source":Xt("error",r);break;case"img":case"image":case"link":Xt("error",r),Xt("load",r);break;case"form":Xt("reset",r),Xt("submit",r);break;case"details":Xt("toggle",r);break;case"input":Ie(r,s),Xt("invalid",r),ln(n,"onChange");break;case"select":r._wrapperState={wasMultiple:!!s.multiple},Xt("invalid",r),ln(n,"onChange");break;case"textarea":ke(r,s),Xt("invalid",r),ln(n,"onChange")}for(var l in on(o,s),e=null,s)if(s.hasOwnProperty(l)){var u=s[l];"children"===l?"string"==typeof u?r.textContent!==u&&(e=["children",u]):"number"==typeof u&&r.textContent!==""+u&&(e=["children",""+u]):x.hasOwnProperty(l)&&null!=u&&ln(n,l)}switch(o){case"input":Ce(r),Me(r,s,!0);break;case"textarea":Ce(r),De(r);break;case"select":case"option":break;default:"function"==typeof s.onClick&&(r.onclick=un)}n=e,t.updateQueue=n,null!==n&&(t.effectTag|=4)}else{switch(l=9===n.nodeType?n:n.ownerDocument,e===sn&&(e=je(o)),e===sn?"script"===o?((e=l.createElement("div")).innerHTML="<script><\/script>",e=e.removeChild(e.firstChild)):"string"==typeof r.is?e=l.createElement(o,{is:r.is}):(e=l.createElement(o),"select"===o&&(l=e,r.multiple?l.multiple=!0:r.size&&(l.size=r.size))):e=l.createElementNS(e,o),e[xn]=t,e[wn]=r,Ba(e,t),t.stateNode=e,l=an(o,r),o){case"iframe":case"object":case"embed":Xt("load",e),u=r;break;case"video":case"audio":for(u=0;u<Qe.length;u++)Xt(Qe[u],e);u=r;break;case"source":Xt("error",e),u=r;break;case"img":case"image":case"link":Xt("error",e),Xt("load",e),u=r;break;case"form":Xt("reset",e),Xt("submit",e),u=r;break;case"details":Xt("toggle",e),u=r;break;case"input":Ie(e,r),u=_e(e,r),Xt("invalid",e),ln(n,"onChange");break;case"option":u=Te(e,r);break;case"select":e._wrapperState={wasMultiple:!!r.multiple},u=i({},r,{value:void 0}),Xt("invalid",e),ln(n,"onChange");break;case"textarea":ke(e,r),u=Oe(e,r),Xt("invalid",e),ln(n,"onChange");break;default:u=r}on(o,u);var c=u;for(s in c)if(c.hasOwnProperty(s)){var d=c[s];"style"===s?nn(e,d):"dangerouslySetInnerHTML"===s?null!=(d=d?d.__html:void 0)&&Fe(e,d):"children"===s?"string"==typeof d?("textarea"!==o||""!==d)&&Be(e,d):"number"==typeof d&&Be(e,""+d):"suppressContentEditableWarning"!==s&&"suppressHydrationWarning"!==s&&"autoFocus"!==s&&(x.hasOwnProperty(s)?null!=d&&ln(n,s):null!=d&&K(e,s,d,l))}switch(o){case"input":Ce(e),Me(e,r,!1);break;case"textarea":Ce(e),De(e);break;case"option":null!=r.value&&e.setAttribute("value",""+ye(r.value));break;case"select":e.multiple=!!r.multiple,null!=(n=r.value)?Ee(e,!!r.multiple,n,!1):null!=r.defaultValue&&Ee(e,!!r.multiple,r.defaultValue,!0);break;default:"function"==typeof u.onClick&&(e.onclick=un)}mn(o,r)&&(t.effectTag|=4)}null!==t.ref&&(t.effectTag|=128)}return null;case 6:if(e&&null!=t.stateNode)Ua(0,t,e.memoizedProps,r);else{if("string"!=typeof r&&null===t.stateNode)throw Error(a(166));n=ko(Oo.current),ko(To.current),Sa(t)?(n=t.stateNode,r=t.memoizedProps,n[xn]=t,n.nodeValue!==r&&(t.effectTag|=4)):((n=(9===n.nodeType?n:n.ownerDocument).createTextNode(r))[xn]=t,t.stateNode=n)}return null;case 13:return li(jo),r=t.memoizedState,0!=(64&t.effectTag)?(t.expirationTime=n,t):(n=null!==r,r=!1,null===e?void 0!==t.memoizedProps.fallback&&Sa(t):(r=null!==(o=e.memoizedState),n||null===o||null!==(o=e.child.sibling)&&(null!==(s=t.firstEffect)?(t.firstEffect=o,o.nextEffect=s):(t.firstEffect=t.lastEffect=o,o.nextEffect=null),o.effectTag=8)),n&&!r&&0!=(2&t.mode)&&(null===e&&!0!==t.memoizedProps.unstable_avoidThisFallback||0!=(1&jo.current)?Ss===Cs&&(Ss=As):(Ss!==Cs&&Ss!==As||(Ss=_s),0!==Ls&&null!==xs&&(Dl(xs,Ms),Pl(xs,Ls)))),(n||r)&&(t.effectTag|=4),null);case 4:return Do(),null;case 10:return eo(t),null;case 17:return gi(t.type)&&vi(),null;case 19:if(li(jo),null===(r=t.memoizedState))return null;if(o=0!=(64&t.effectTag),null===(s=r.rendering)){if(o)Za(r,!1);else if(Ss!==Cs||null!==e&&0!=(64&e.effectTag))for(s=t.child;null!==s;){if(null!==(e=Ro(s))){for(t.effectTag|=64,Za(r,!1),null!==(o=e.updateQueue)&&(t.updateQueue=o,t.effectTag|=4),null===r.lastEffect&&(t.firstEffect=null),t.lastEffect=r.lastEffect,r=t.child;null!==r;)s=n,(o=r).effectTag&=2,o.nextEffect=null,o.firstEffect=null,o.lastEffect=null,null===(e=o.alternate)?(o.childExpirationTime=0,o.expirationTime=s,o.child=null,o.memoizedProps=null,o.memoizedState=null,o.updateQueue=null,o.dependencies=null):(o.childExpirationTime=e.childExpirationTime,o.expirationTime=e.expirationTime,o.child=e.child,o.memoizedProps=e.memoizedProps,o.memoizedState=e.memoizedState,o.updateQueue=e.updateQueue,s=e.dependencies,o.dependencies=null===s?null:{expirationTime:s.expirationTime,firstContext:s.firstContext,responders:s.responders}),r=r.sibling;return ui(jo,1&jo.current|2),t.child}s=s.sibling}}else{if(!o)if(null!==(e=Ro(s))){if(t.effectTag|=64,o=!0,null!==(n=e.updateQueue)&&(t.updateQueue=n,t.effectTag|=4),Za(r,!0),null===r.tail&&"hidden"===r.tailMode&&!s.alternate)return null!==(t=t.lastEffect=r.lastEffect)&&(t.nextEffect=null),null}else 2*Fi()-r.renderingStartTime>r.tailExpiration&&1<n&&(t.effectTag|=64,o=!0,Za(r,!1),t.expirationTime=t.childExpirationTime=n-1);r.isBackwards?(s.sibling=t.child,t.child=s):(null!==(n=r.last)?n.sibling=s:t.child=s,r.last=s)}return null!==r.tail?(0===r.tailExpiration&&(r.tailExpiration=Fi()+500),n=r.tail,r.rendering=n,r.tail=n.sibling,r.lastEffect=t.lastEffect,r.renderingStartTime=Fi(),n.sibling=null,t=jo.current,ui(jo,o?1&t|2:1&t),n):null}throw Error(a(156,t.tag))}function Ka(e){switch(e.tag){case 1:gi(e.type)&&vi();var t=e.effectTag;return 4096&t?(e.effectTag=-4097&t|64,e):null;case 3:if(Do(),li(hi),li(di),0!=(64&(t=e.effectTag)))throw Error(a(285));return e.effectTag=-4097&t|64,e;case 5:return No(e),null;case 13:return li(jo),4096&(t=e.effectTag)?(e.effectTag=-4097&t|64,e):null;case 19:return li(jo),null;case 4:return Do(),null;case 10:return eo(e),null;default:return null}}function Ja(e,t){return{value:e,source:t,stack:me(t)}}Ba=function(e,t){for(var n=t.child;null!==n;){if(5===n.tag||6===n.tag)e.appendChild(n.stateNode);else if(4!==n.tag&&null!==n.child){n.child.return=n,n=n.child;continue}if(n===t)break;for(;null===n.sibling;){if(null===n.return||n.return===t)return;n=n.return}n.sibling.return=n.return,n=n.sibling}},Ha=function(e,t,n,r,o){var a=e.memoizedProps;if(a!==r){var s,l,u=t.stateNode;switch(ko(To.current),e=null,n){case"input":a=_e(u,a),r=_e(u,r),e=[];break;case"option":a=Te(u,a),r=Te(u,r),e=[];break;case"select":a=i({},a,{value:void 0}),r=i({},r,{value:void 0}),e=[];break;case"textarea":a=Oe(u,a),r=Oe(u,r),e=[];break;default:"function"!=typeof a.onClick&&"function"==typeof r.onClick&&(u.onclick=un)}for(s in on(n,r),n=null,a)if(!r.hasOwnProperty(s)&&a.hasOwnProperty(s)&&null!=a[s])if("style"===s)for(l in u=a[s])u.hasOwnProperty(l)&&(n||(n={}),n[l]="");else"dangerouslySetInnerHTML"!==s&&"children"!==s&&"suppressContentEditableWarning"!==s&&"suppressHydrationWarning"!==s&&"autoFocus"!==s&&(x.hasOwnProperty(s)?e||(e=[]):(e=e||[]).push(s,null));for(s in r){var c=r[s];if(u=null!=a?a[s]:void 0,r.hasOwnProperty(s)&&c!==u&&(null!=c||null!=u))if("style"===s)if(u){for(l in u)!u.hasOwnProperty(l)||c&&c.hasOwnProperty(l)||(n||(n={}),n[l]="");for(l in c)c.hasOwnProperty(l)&&u[l]!==c[l]&&(n||(n={}),n[l]=c[l])}else n||(e||(e=[]),e.push(s,n)),n=c;else"dangerouslySetInnerHTML"===s?(c=c?c.__html:void 0,u=u?u.__html:void 0,null!=c&&u!==c&&(e=e||[]).push(s,c)):"children"===s?u===c||"string"!=typeof c&&"number"!=typeof c||(e=e||[]).push(s,""+c):"suppressContentEditableWarning"!==s&&"suppressHydrationWarning"!==s&&(x.hasOwnProperty(s)?(null!=c&&ln(o,s),e||u===c||(e=[])):(e=e||[]).push(s,c))}n&&(e=e||[]).push("style",n),o=e,(t.updateQueue=o)&&(t.effectTag|=4)}},Ua=function(e,t,n,r){n!==r&&(t.effectTag|=4)};var $a="function"==typeof WeakSet?WeakSet:Set;function es(e,t){var n=t.source,r=t.stack;null===r&&null!==n&&(r=me(n)),null!==n&&ve(n.type),t=t.value,null!==e&&1===e.tag&&ve(e.type);try{console.error(t)}catch(e){setTimeout((function(){throw e}))}}function ts(e){var t=e.ref;if(null!==t)if("function"==typeof t)try{t(null)}catch(t){yl(e,t)}else t.current=null}function ns(e,t){switch(t.tag){case 0:case 11:case 15:case 22:return;case 1:if(256&t.effectTag&&null!==e){var n=e.memoizedProps,r=e.memoizedState;t=(e=t.stateNode).getSnapshotBeforeUpdate(t.elementType===t.type?n:qi(t.type,n),r),e.__reactInternalSnapshotBeforeUpdate=t}return;case 3:case 5:case 6:case 4:case 17:return}throw Error(a(163))}function rs(e,t){if(null!==(t=null!==(t=t.updateQueue)?t.lastEffect:null)){var n=t=t.next;do{if((n.tag&e)===e){var r=n.destroy;n.destroy=void 0,void 0!==r&&r()}n=n.next}while(n!==t)}}function is(e,t){if(null!==(t=null!==(t=t.updateQueue)?t.lastEffect:null)){var n=t=t.next;do{if((n.tag&e)===e){var r=n.create;n.destroy=r()}n=n.next}while(n!==t)}}function os(e,t,n){switch(n.tag){case 0:case 11:case 15:case 22:return void is(3,n);case 1:if(e=n.stateNode,4&n.effectTag)if(null===t)e.componentDidMount();else{var r=n.elementType===n.type?t.memoizedProps:qi(n.type,t.memoizedProps);e.componentDidUpdate(r,t.memoizedState,e.__reactInternalSnapshotBeforeUpdate)}return void(null!==(t=n.updateQueue)&&ho(n,t,e));case 3:if(null!==(t=n.updateQueue)){if(e=null,null!==n.child)switch(n.child.tag){case 5:e=n.child.stateNode;break;case 1:e=n.child.stateNode}ho(n,t,e)}return;case 5:return e=n.stateNode,void(null===t&&4&n.effectTag&&mn(n.type,n.memoizedProps)&&e.focus());case 6:case 4:case 12:return;case 13:return void(null===n.memoizedState&&(n=n.alternate,null!==n&&(n=n.memoizedState,null!==n&&(n=n.dehydrated,null!==n&&jt(n)))));case 19:case 17:case 20:case 21:return}throw Error(a(163))}function as(e,t,n){switch("function"==typeof _l&&_l(t),t.tag){case 0:case 11:case 14:case 15:case 22:if(null!==(e=t.updateQueue)&&null!==(e=e.lastEffect)){var r=e.next;Ui(97<n?97:n,(function(){var e=r;do{var n=e.destroy;if(void 0!==n){var i=t;try{n()}catch(e){yl(i,e)}}e=e.next}while(e!==r)}))}break;case 1:ts(t),"function"==typeof(n=t.stateNode).componentWillUnmount&&function(e,t){try{t.props=e.memoizedProps,t.state=e.memoizedState,t.componentWillUnmount()}catch(t){yl(e,t)}}(t,n);break;case 5:ts(t);break;case 4:cs(e,t,n)}}function ss(e){var t=e.alternate;e.return=null,e.child=null,e.memoizedState=null,e.updateQueue=null,e.dependencies=null,e.alternate=null,e.firstEffect=null,e.lastEffect=null,e.pendingProps=null,e.memoizedProps=null,e.stateNode=null,null!==t&&ss(t)}function ls(e){return 5===e.tag||3===e.tag||4===e.tag}function us(e){e:{for(var t=e.return;null!==t;){if(ls(t)){var n=t;break e}t=t.return}throw Error(a(160))}switch(t=n.stateNode,n.tag){case 5:var r=!1;break;case 3:case 4:t=t.containerInfo,r=!0;break;default:throw Error(a(161))}16&n.effectTag&&(Be(t,""),n.effectTag&=-17);e:t:for(n=e;;){for(;null===n.sibling;){if(null===n.return||ls(n.return)){n=null;break e}n=n.return}for(n.sibling.return=n.return,n=n.sibling;5!==n.tag&&6!==n.tag&&18!==n.tag;){if(2&n.effectTag)continue t;if(null===n.child||4===n.tag)continue t;n.child.return=n,n=n.child}if(!(2&n.effectTag)){n=n.stateNode;break e}}r?function e(t,n,r){var i=t.tag,o=5===i||6===i;if(o)t=o?t.stateNode:t.stateNode.instance,n?8===r.nodeType?r.parentNode.insertBefore(t,n):r.insertBefore(t,n):(8===r.nodeType?(n=r.parentNode).insertBefore(t,r):(n=r).appendChild(t),null!==(r=r._reactRootContainer)&&void 0!==r||null!==n.onclick||(n.onclick=un));else if(4!==i&&null!==(t=t.child))for(e(t,n,r),t=t.sibling;null!==t;)e(t,n,r),t=t.sibling}(e,n,t):function e(t,n,r){var i=t.tag,o=5===i||6===i;if(o)t=o?t.stateNode:t.stateNode.instance,n?r.insertBefore(t,n):r.appendChild(t);else if(4!==i&&null!==(t=t.child))for(e(t,n,r),t=t.sibling;null!==t;)e(t,n,r),t=t.sibling}(e,n,t)}function cs(e,t,n){for(var r,i,o=t,s=!1;;){if(!s){s=o.return;e:for(;;){if(null===s)throw Error(a(160));switch(r=s.stateNode,s.tag){case 5:i=!1;break e;case 3:case 4:r=r.containerInfo,i=!0;break e}s=s.return}s=!0}if(5===o.tag||6===o.tag){e:for(var l=e,u=o,c=n,d=u;;)if(as(l,d,c),null!==d.child&&4!==d.tag)d.child.return=d,d=d.child;else{if(d===u)break e;for(;null===d.sibling;){if(null===d.return||d.return===u)break e;d=d.return}d.sibling.return=d.return,d=d.sibling}i?(l=r,u=o.stateNode,8===l.nodeType?l.parentNode.removeChild(u):l.removeChild(u)):r.removeChild(o.stateNode)}else if(4===o.tag){if(null!==o.child){r=o.stateNode.containerInfo,i=!0,o.child.return=o,o=o.child;continue}}else if(as(e,o,n),null!==o.child){o.child.return=o,o=o.child;continue}if(o===t)break;for(;null===o.sibling;){if(null===o.return||o.return===t)return;4===(o=o.return).tag&&(s=!1)}o.sibling.return=o.return,o=o.sibling}}function ds(e,t){switch(t.tag){case 0:case 11:case 14:case 15:case 22:return void rs(3,t);case 1:return;case 5:var n=t.stateNode;if(null!=n){var r=t.memoizedProps,i=null!==e?e.memoizedProps:r;e=t.type;var o=t.updateQueue;if(t.updateQueue=null,null!==o){for(n[wn]=r,"input"===e&&"radio"===r.type&&null!=r.name&&xe(n,r),an(e,i),t=an(e,r),i=0;i<o.length;i+=2){var s=o[i],l=o[i+1];"style"===s?nn(n,l):"dangerouslySetInnerHTML"===s?Fe(n,l):"children"===s?Be(n,l):K(n,s,l,t)}switch(e){case"input":we(n,r);break;case"textarea":Le(n,r);break;case"select":t=n._wrapperState.wasMultiple,n._wrapperState.wasMultiple=!!r.multiple,null!=(e=r.value)?Ee(n,!!r.multiple,e,!1):t!==!!r.multiple&&(null!=r.defaultValue?Ee(n,!!r.multiple,r.defaultValue,!0):Ee(n,!!r.multiple,r.multiple?[]:"",!1))}}}return;case 6:if(null===t.stateNode)throw Error(a(162));return void(t.stateNode.nodeValue=t.memoizedProps);case 3:return void((t=t.stateNode).hydrate&&(t.hydrate=!1,jt(t.containerInfo)));case 12:return;case 13:if(n=t,null===t.memoizedState?r=!1:(r=!0,n=t.child,Ps=Fi()),null!==n)e:for(e=n;;){if(5===e.tag)o=e.stateNode,r?"function"==typeof(o=o.style).setProperty?o.setProperty("display","none","important"):o.display="none":(o=e.stateNode,i=null!=(i=e.memoizedProps.style)&&i.hasOwnProperty("display")?i.display:null,o.style.display=tn("display",i));else if(6===e.tag)e.stateNode.nodeValue=r?"":e.memoizedProps;else{if(13===e.tag&&null!==e.memoizedState&&null===e.memoizedState.dehydrated){(o=e.child.sibling).return=e,e=o;continue}if(null!==e.child){e.child.return=e,e=e.child;continue}}if(e===n)break;for(;null===e.sibling;){if(null===e.return||e.return===n)break e;e=e.return}e.sibling.return=e.return,e=e.sibling}return void hs(t);case 19:return void hs(t);case 17:return}throw Error(a(163))}function hs(e){var t=e.updateQueue;if(null!==t){e.updateQueue=null;var n=e.stateNode;null===n&&(n=e.stateNode=new $a),t.forEach((function(t){var r=Cl.bind(null,e,t);n.has(t)||(n.add(t),t.then(r,r))}))}}var fs="function"==typeof WeakMap?WeakMap:Map;function ps(e,t,n){(n=so(n,null)).tag=3,n.payload={element:null};var r=t.value;return n.callback=function(){js||(js=!0,Rs=r),es(e,t)},n}function gs(e,t,n){(n=so(n,null)).tag=3;var r=e.type.getDerivedStateFromError;if("function"==typeof r){var i=t.value;n.payload=function(){return es(e,t),r(i)}}var o=e.stateNode;return null!==o&&"function"==typeof o.componentDidCatch&&(n.callback=function(){"function"!=typeof r&&(null===zs?zs=new Set([this]):zs.add(this),es(e,t));var n=t.stack;this.componentDidCatch(t.value,{componentStack:null!==n?n:""})}),n}var vs,ms=Math.ceil,ys=Q.ReactCurrentDispatcher,bs=Q.ReactCurrentOwner,Cs=0,As=3,_s=4,Is=0,xs=null,ws=null,Ms=0,Ss=Cs,Ts=null,Es=1073741823,Os=1073741823,ks=null,Ls=0,Ds=!1,Ps=0,Ns=null,js=!1,Rs=null,zs=null,Fs=!1,Bs=null,Hs=90,Us=null,Vs=0,Ys=null,Ws=0;function Gs(){return 0!=(48&Is)?1073741821-(Fi()/10|0):0!==Ws?Ws:Ws=1073741821-(Fi()/10|0)}function Xs(e,t,n){if(0==(2&(t=t.mode)))return 1073741823;var r=Bi();if(0==(4&t))return 99===r?1073741823:1073741822;if(0!=(16&Is))return Ms;if(null!==n)e=Xi(e,0|n.timeoutMs||5e3,250);else switch(r){case 99:e=1073741823;break;case 98:e=Xi(e,150,100);break;case 97:case 96:e=Xi(e,5e3,250);break;case 95:e=2;break;default:throw Error(a(326))}return null!==xs&&e===Ms&&--e,e}function qs(e,t){if(50<Vs)throw Vs=0,Ys=null,Error(a(185));if(null!==(e=Zs(e,t))){var n=Bi();1073741823===t?0!=(8&Is)&&0==(48&Is)?$s(e):(Ks(e),0===Is&&Wi()):Ks(e),0==(4&Is)||98!==n&&99!==n||(null===Us?Us=new Map([[e,t]]):(void 0===(n=Us.get(e))||n>t)&&Us.set(e,t))}}function Zs(e,t){e.expirationTime<t&&(e.expirationTime=t);var n=e.alternate;null!==n&&n.expirationTime<t&&(n.expirationTime=t);var r=e.return,i=null;if(null===r&&3===e.tag)i=e.stateNode;else for(;null!==r;){if(n=r.alternate,r.childExpirationTime<t&&(r.childExpirationTime=t),null!==n&&n.childExpirationTime<t&&(n.childExpirationTime=t),null===r.return&&3===r.tag){i=r.stateNode;break}r=r.return}return null!==i&&(xs===i&&(al(t),Ss===_s&&Dl(i,Ms)),Pl(i,t)),i}function Qs(e){var t=e.lastExpiredTime;if(0!==t)return t;if(!Ll(e,t=e.firstPendingTime))return t;var n=e.lastPingedTime;return 2>=(e=n>(e=e.nextKnownPendingLevel)?n:e)&&t!==e?0:e}function Ks(e){if(0!==e.lastExpiredTime)e.callbackExpirationTime=1073741823,e.callbackPriority=99,e.callbackNode=Yi($s.bind(null,e));else{var t=Qs(e),n=e.callbackNode;if(0===t)null!==n&&(e.callbackNode=null,e.callbackExpirationTime=0,e.callbackPriority=90);else{var r=Gs();if(1073741823===t?r=99:1===t||2===t?r=95:r=0>=(r=10*(1073741821-t)-10*(1073741821-r))?99:250>=r?98:5250>=r?97:95,null!==n){var i=e.callbackPriority;if(e.callbackExpirationTime===t&&i>=r)return;n!==Li&&Ii(n)}e.callbackExpirationTime=t,e.callbackPriority=r,t=1073741823===t?Yi($s.bind(null,e)):Vi(r,Js.bind(null,e),{timeout:10*(1073741821-t)-Fi()}),e.callbackNode=t}}}function Js(e,t){if(Ws=0,t)return Nl(e,t=Gs()),Ks(e),null;var n=Qs(e);if(0!==n){if(t=e.callbackNode,0!=(48&Is))throw Error(a(327));if(gl(),e===xs&&n===Ms||nl(e,n),null!==ws){var r=Is;Is|=16;for(var i=il();;)try{ll();break}catch(t){rl(e,t)}if($i(),Is=r,ys.current=i,1===Ss)throw t=Ts,nl(e,n),Dl(e,n),Ks(e),t;if(null===ws)switch(i=e.finishedWork=e.current.alternate,e.finishedExpirationTime=n,r=Ss,xs=null,r){case Cs:case 1:throw Error(a(345));case 2:Nl(e,2<n?2:n);break;case As:if(Dl(e,n),n===(r=e.lastSuspendedTime)&&(e.nextKnownPendingLevel=dl(i)),1073741823===Es&&10<(i=Ps+500-Fi())){if(Ds){var o=e.lastPingedTime;if(0===o||o>=n){e.lastPingedTime=n,nl(e,n);break}}if(0!==(o=Qs(e))&&o!==n)break;if(0!==r&&r!==n){e.lastPingedTime=r;break}e.timeoutHandle=bn(hl.bind(null,e),i);break}hl(e);break;case _s:if(Dl(e,n),n===(r=e.lastSuspendedTime)&&(e.nextKnownPendingLevel=dl(i)),Ds&&(0===(i=e.lastPingedTime)||i>=n)){e.lastPingedTime=n,nl(e,n);break}if(0!==(i=Qs(e))&&i!==n)break;if(0!==r&&r!==n){e.lastPingedTime=r;break}if(1073741823!==Os?r=10*(1073741821-Os)-Fi():1073741823===Es?r=0:(r=10*(1073741821-Es)-5e3,0>(r=(i=Fi())-r)&&(r=0),(n=10*(1073741821-n)-i)<(r=(120>r?120:480>r?480:1080>r?1080:1920>r?1920:3e3>r?3e3:4320>r?4320:1960*ms(r/1960))-r)&&(r=n)),10<r){e.timeoutHandle=bn(hl.bind(null,e),r);break}hl(e);break;case 5:if(1073741823!==Es&&null!==ks){o=Es;var s=ks;if(0>=(r=0|s.busyMinDurationMs)?r=0:(i=0|s.busyDelayMs,r=(o=Fi()-(10*(1073741821-o)-(0|s.timeoutMs||5e3)))<=i?0:i+r-o),10<r){Dl(e,n),e.timeoutHandle=bn(hl.bind(null,e),r);break}}hl(e);break;default:throw Error(a(329))}if(Ks(e),e.callbackNode===t)return Js.bind(null,e)}}return null}function $s(e){var t=e.lastExpiredTime;if(t=0!==t?t:1073741823,0!=(48&Is))throw Error(a(327));if(gl(),e===xs&&t===Ms||nl(e,t),null!==ws){var n=Is;Is|=16;for(var r=il();;)try{sl();break}catch(t){rl(e,t)}if($i(),Is=n,ys.current=r,1===Ss)throw n=Ts,nl(e,t),Dl(e,t),Ks(e),n;if(null!==ws)throw Error(a(261));e.finishedWork=e.current.alternate,e.finishedExpirationTime=t,xs=null,hl(e),Ks(e)}return null}function el(e,t){var n=Is;Is|=1;try{return e(t)}finally{0===(Is=n)&&Wi()}}function tl(e,t){var n=Is;Is&=-2,Is|=8;try{return e(t)}finally{0===(Is=n)&&Wi()}}function nl(e,t){e.finishedWork=null,e.finishedExpirationTime=0;var n=e.timeoutHandle;if(-1!==n&&(e.timeoutHandle=-1,Cn(n)),null!==ws)for(n=ws.return;null!==n;){var r=n;switch(r.tag){case 1:null!=(r=r.type.childContextTypes)&&vi();break;case 3:Do(),li(hi),li(di);break;case 5:No(r);break;case 4:Do();break;case 13:case 19:li(jo);break;case 10:eo(r)}n=n.return}xs=e,ws=Ml(e.current,null),Ms=t,Ss=Cs,Ts=null,Os=Es=1073741823,ks=null,Ls=0,Ds=!1}function rl(e,t){for(;;){try{if($i(),Fo.current=va,Wo)for(var n=Uo.memoizedState;null!==n;){var r=n.queue;null!==r&&(r.pending=null),n=n.next}if(Ho=0,Yo=Vo=Uo=null,Wo=!1,null===ws||null===ws.return)return Ss=1,Ts=t,ws=null;e:{var i=e,o=ws.return,a=ws,s=t;if(t=Ms,a.effectTag|=2048,a.firstEffect=a.lastEffect=null,null!==s&&"object"==typeof s&&"function"==typeof s.then){var l=s;if(0==(2&a.mode)){var u=a.alternate;u?(a.updateQueue=u.updateQueue,a.memoizedState=u.memoizedState,a.expirationTime=u.expirationTime):(a.updateQueue=null,a.memoizedState=null)}var c=0!=(1&jo.current),d=o;do{var h;if(h=13===d.tag){var f=d.memoizedState;if(null!==f)h=null!==f.dehydrated;else{var p=d.memoizedProps;h=void 0!==p.fallback&&(!0!==p.unstable_avoidThisFallback||!c)}}if(h){var g=d.updateQueue;if(null===g){var v=new Set;v.add(l),d.updateQueue=v}else g.add(l);if(0==(2&d.mode)){if(d.effectTag|=64,a.effectTag&=-2981,1===a.tag)if(null===a.alternate)a.tag=17;else{var m=so(1073741823,null);m.tag=2,lo(a,m)}a.expirationTime=1073741823;break e}s=void 0,a=t;var y=i.pingCache;if(null===y?(y=i.pingCache=new fs,s=new Set,y.set(l,s)):void 0===(s=y.get(l))&&(s=new Set,y.set(l,s)),!s.has(a)){s.add(a);var b=bl.bind(null,i,l,a);l.then(b,b)}d.effectTag|=4096,d.expirationTime=t;break e}d=d.return}while(null!==d);s=Error((ve(a.type)||"A React component")+" suspended while rendering, but no fallback UI was specified.\n\nAdd a <Suspense fallback=...> component higher in the tree to provide a loading indicator or placeholder to display."+me(a))}5!==Ss&&(Ss=2),s=Ja(s,a),d=o;do{switch(d.tag){case 3:l=s,d.effectTag|=4096,d.expirationTime=t,uo(d,ps(d,l,t));break e;case 1:l=s;var C=d.type,A=d.stateNode;if(0==(64&d.effectTag)&&("function"==typeof C.getDerivedStateFromError||null!==A&&"function"==typeof A.componentDidCatch&&(null===zs||!zs.has(A)))){d.effectTag|=4096,d.expirationTime=t,uo(d,gs(d,l,t));break e}}d=d.return}while(null!==d)}ws=cl(ws)}catch(e){t=e;continue}break}}function il(){var e=ys.current;return ys.current=va,null===e?va:e}function ol(e,t){e<Es&&2<e&&(Es=e),null!==t&&e<Os&&2<e&&(Os=e,ks=t)}function al(e){e>Ls&&(Ls=e)}function sl(){for(;null!==ws;)ws=ul(ws)}function ll(){for(;null!==ws&&!Di();)ws=ul(ws)}function ul(e){var t=vs(e.alternate,e,Ms);return e.memoizedProps=e.pendingProps,null===t&&(t=cl(e)),bs.current=null,t}function cl(e){ws=e;do{var t=ws.alternate;if(e=ws.return,0==(2048&ws.effectTag)){if(t=Qa(t,ws,Ms),1===Ms||1!==ws.childExpirationTime){for(var n=0,r=ws.child;null!==r;){var i=r.expirationTime,o=r.childExpirationTime;i>n&&(n=i),o>n&&(n=o),r=r.sibling}ws.childExpirationTime=n}if(null!==t)return t;null!==e&&0==(2048&e.effectTag)&&(null===e.firstEffect&&(e.firstEffect=ws.firstEffect),null!==ws.lastEffect&&(null!==e.lastEffect&&(e.lastEffect.nextEffect=ws.firstEffect),e.lastEffect=ws.lastEffect),1<ws.effectTag&&(null!==e.lastEffect?e.lastEffect.nextEffect=ws:e.firstEffect=ws,e.lastEffect=ws))}else{if(null!==(t=Ka(ws)))return t.effectTag&=2047,t;null!==e&&(e.firstEffect=e.lastEffect=null,e.effectTag|=2048)}if(null!==(t=ws.sibling))return t;ws=e}while(null!==ws);return Ss===Cs&&(Ss=5),null}function dl(e){var t=e.expirationTime;return t>(e=e.childExpirationTime)?t:e}function hl(e){var t=Bi();return Ui(99,fl.bind(null,e,t)),null}function fl(e,t){do{gl()}while(null!==Bs);if(0!=(48&Is))throw Error(a(327));var n=e.finishedWork,r=e.finishedExpirationTime;if(null===n)return null;if(e.finishedWork=null,e.finishedExpirationTime=0,n===e.current)throw Error(a(177));e.callbackNode=null,e.callbackExpirationTime=0,e.callbackPriority=90,e.nextKnownPendingLevel=0;var i=dl(n);if(e.firstPendingTime=i,r<=e.lastSuspendedTime?e.firstSuspendedTime=e.lastSuspendedTime=e.nextKnownPendingLevel=0:r<=e.firstSuspendedTime&&(e.firstSuspendedTime=r-1),r<=e.lastPingedTime&&(e.lastPingedTime=0),r<=e.lastExpiredTime&&(e.lastExpiredTime=0),e===xs&&(ws=xs=null,Ms=0),1<n.effectTag?null!==n.lastEffect?(n.lastEffect.nextEffect=n,i=n.firstEffect):i=n:i=n.firstEffect,null!==i){var o=Is;Is|=32,bs.current=null,gn=Gt;var s=fn();if(pn(s)){if("selectionStart"in s)var l={start:s.selectionStart,end:s.selectionEnd};else e:{var u=(l=(l=s.ownerDocument)&&l.defaultView||window).getSelection&&l.getSelection();if(u&&0!==u.rangeCount){l=u.anchorNode;var c=u.anchorOffset,d=u.focusNode;u=u.focusOffset;try{l.nodeType,d.nodeType}catch(e){l=null;break e}var h=0,f=-1,p=-1,g=0,v=0,m=s,y=null;t:for(;;){for(var b;m!==l||0!==c&&3!==m.nodeType||(f=h+c),m!==d||0!==u&&3!==m.nodeType||(p=h+u),3===m.nodeType&&(h+=m.nodeValue.length),null!==(b=m.firstChild);)y=m,m=b;for(;;){if(m===s)break t;if(y===l&&++g===c&&(f=h),y===d&&++v===u&&(p=h),null!==(b=m.nextSibling))break;y=(m=y).parentNode}m=b}l=-1===f||-1===p?null:{start:f,end:p}}else l=null}l=l||{start:0,end:0}}else l=null;vn={activeElementDetached:null,focusedElem:s,selectionRange:l},Gt=!1,Ns=i;do{try{pl()}catch(e){if(null===Ns)throw Error(a(330));yl(Ns,e),Ns=Ns.nextEffect}}while(null!==Ns);Ns=i;do{try{for(s=e,l=t;null!==Ns;){var C=Ns.effectTag;if(16&C&&Be(Ns.stateNode,""),128&C){var A=Ns.alternate;if(null!==A){var _=A.ref;null!==_&&("function"==typeof _?_(null):_.current=null)}}switch(1038&C){case 2:us(Ns),Ns.effectTag&=-3;break;case 6:us(Ns),Ns.effectTag&=-3,ds(Ns.alternate,Ns);break;case 1024:Ns.effectTag&=-1025;break;case 1028:Ns.effectTag&=-1025,ds(Ns.alternate,Ns);break;case 4:ds(Ns.alternate,Ns);break;case 8:cs(s,c=Ns,l),ss(c)}Ns=Ns.nextEffect}}catch(e){if(null===Ns)throw Error(a(330));yl(Ns,e),Ns=Ns.nextEffect}}while(null!==Ns);if(_=vn,A=fn(),C=_.focusedElem,l=_.selectionRange,A!==C&&C&&C.ownerDocument&&function e(t,n){return!(!t||!n)&&(t===n||(!t||3!==t.nodeType)&&(n&&3===n.nodeType?e(t,n.parentNode):"contains"in t?t.contains(n):!!t.compareDocumentPosition&&!!(16&t.compareDocumentPosition(n))))}(C.ownerDocument.documentElement,C)){null!==l&&pn(C)&&(A=l.start,void 0===(_=l.end)&&(_=A),"selectionStart"in C?(C.selectionStart=A,C.selectionEnd=Math.min(_,C.value.length)):(_=(A=C.ownerDocument||document)&&A.defaultView||window).getSelection&&(_=_.getSelection(),c=C.textContent.length,s=Math.min(l.start,c),l=void 0===l.end?s:Math.min(l.end,c),!_.extend&&s>l&&(c=l,l=s,s=c),c=hn(C,s),d=hn(C,l),c&&d&&(1!==_.rangeCount||_.anchorNode!==c.node||_.anchorOffset!==c.offset||_.focusNode!==d.node||_.focusOffset!==d.offset)&&((A=A.createRange()).setStart(c.node,c.offset),_.removeAllRanges(),s>l?(_.addRange(A),_.extend(d.node,d.offset)):(A.setEnd(d.node,d.offset),_.addRange(A))))),A=[];for(_=C;_=_.parentNode;)1===_.nodeType&&A.push({element:_,left:_.scrollLeft,top:_.scrollTop});for("function"==typeof C.focus&&C.focus(),C=0;C<A.length;C++)(_=A[C]).element.scrollLeft=_.left,_.element.scrollTop=_.top}Gt=!!gn,vn=gn=null,e.current=n,Ns=i;do{try{for(C=e;null!==Ns;){var I=Ns.effectTag;if(36&I&&os(C,Ns.alternate,Ns),128&I){A=void 0;var x=Ns.ref;if(null!==x){var w=Ns.stateNode;switch(Ns.tag){case 5:A=w;break;default:A=w}"function"==typeof x?x(A):x.current=A}}Ns=Ns.nextEffect}}catch(e){if(null===Ns)throw Error(a(330));yl(Ns,e),Ns=Ns.nextEffect}}while(null!==Ns);Ns=null,Pi(),Is=o}else e.current=n;if(Fs)Fs=!1,Bs=e,Hs=t;else for(Ns=i;null!==Ns;)t=Ns.nextEffect,Ns.nextEffect=null,Ns=t;if(0===(t=e.firstPendingTime)&&(zs=null),1073741823===t?e===Ys?Vs++:(Vs=0,Ys=e):Vs=0,"function"==typeof Al&&Al(n.stateNode,r),Ks(e),js)throw js=!1,e=Rs,Rs=null,e;return 0!=(8&Is)||Wi(),null}function pl(){for(;null!==Ns;){var e=Ns.effectTag;0!=(256&e)&&ns(Ns.alternate,Ns),0==(512&e)||Fs||(Fs=!0,Vi(97,(function(){return gl(),null}))),Ns=Ns.nextEffect}}function gl(){if(90!==Hs){var e=97<Hs?97:Hs;return Hs=90,Ui(e,vl)}}function vl(){if(null===Bs)return!1;var e=Bs;if(Bs=null,0!=(48&Is))throw Error(a(331));var t=Is;for(Is|=32,e=e.current.firstEffect;null!==e;){try{var n=e;if(0!=(512&n.effectTag))switch(n.tag){case 0:case 11:case 15:case 22:rs(5,n),is(5,n)}}catch(t){if(null===e)throw Error(a(330));yl(e,t)}n=e.nextEffect,e.nextEffect=null,e=n}return Is=t,Wi(),!0}function ml(e,t,n){lo(e,t=ps(e,t=Ja(n,t),1073741823)),null!==(e=Zs(e,1073741823))&&Ks(e)}function yl(e,t){if(3===e.tag)ml(e,e,t);else for(var n=e.return;null!==n;){if(3===n.tag){ml(n,e,t);break}if(1===n.tag){var r=n.stateNode;if("function"==typeof n.type.getDerivedStateFromError||"function"==typeof r.componentDidCatch&&(null===zs||!zs.has(r))){lo(n,e=gs(n,e=Ja(t,e),1073741823)),null!==(n=Zs(n,1073741823))&&Ks(n);break}}n=n.return}}function bl(e,t,n){var r=e.pingCache;null!==r&&r.delete(t),xs===e&&Ms===n?Ss===_s||Ss===As&&1073741823===Es&&Fi()-Ps<500?nl(e,Ms):Ds=!0:Ll(e,n)&&(0!==(t=e.lastPingedTime)&&t<n||(e.lastPingedTime=n,Ks(e)))}function Cl(e,t){var n=e.stateNode;null!==n&&n.delete(t),0===(t=0)&&(t=Xs(t=Gs(),e,null)),null!==(e=Zs(e,t))&&Ks(e)}vs=function(e,t,n){var r=t.expirationTime;if(null!==e){var i=t.pendingProps;if(e.memoizedProps!==i||hi.current)Oa=!0;else{if(r<n){switch(Oa=!1,t.tag){case 3:Fa(t),Ta();break;case 5:if(Po(t),4&t.mode&&1!==n&&i.hidden)return t.expirationTime=t.childExpirationTime=1,null;break;case 1:gi(t.type)&&bi(t);break;case 4:Lo(t,t.stateNode.containerInfo);break;case 10:r=t.memoizedProps.value,i=t.type._context,ui(Zi,i._currentValue),i._currentValue=r;break;case 13:if(null!==t.memoizedState)return 0!==(r=t.child.childExpirationTime)&&r>=n?Ya(e,t,n):(ui(jo,1&jo.current),null!==(t=qa(e,t,n))?t.sibling:null);ui(jo,1&jo.current);break;case 19:if(r=t.childExpirationTime>=n,0!=(64&e.effectTag)){if(r)return Xa(e,t,n);t.effectTag|=64}if(null!==(i=t.memoizedState)&&(i.rendering=null,i.tail=null),ui(jo,jo.current),!r)return null}return qa(e,t,n)}Oa=!1}}else Oa=!1;switch(t.expirationTime=0,t.tag){case 2:if(r=t.type,null!==e&&(e.alternate=null,t.alternate=null,t.effectTag|=2),e=t.pendingProps,i=pi(t,di.current),no(t,n),i=qo(null,t,r,e,i,n),t.effectTag|=1,"object"==typeof i&&null!==i&&"function"==typeof i.render&&void 0===i.$$typeof){if(t.tag=1,t.memoizedState=null,t.updateQueue=null,gi(r)){var o=!0;bi(t)}else o=!1;t.memoizedState=null!==i.state&&void 0!==i.state?i.state:null,oo(t);var s=r.getDerivedStateFromProps;"function"==typeof s&&go(t,r,s,e),i.updater=vo,t.stateNode=i,i._reactInternalFiber=t,Co(t,r,e,n),t=za(null,t,r,!0,o,n)}else t.tag=0,ka(null,t,i,n),t=t.child;return t;case 16:e:{if(i=t.elementType,null!==e&&(e.alternate=null,t.alternate=null,t.effectTag|=2),e=t.pendingProps,function(e){if(-1===e._status){e._status=0;var t=e._ctor;t=t(),e._result=t,t.then((function(t){0===e._status&&(t=t.default,e._status=1,e._result=t)}),(function(t){0===e._status&&(e._status=2,e._result=t)}))}}(i),1!==i._status)throw i._result;switch(i=i._result,t.type=i,o=t.tag=function(e){if("function"==typeof e)return wl(e)?1:0;if(null!=e){if((e=e.$$typeof)===le)return 11;if(e===de)return 14}return 2}(i),e=qi(i,e),o){case 0:t=ja(null,t,i,e,n);break e;case 1:t=Ra(null,t,i,e,n);break e;case 11:t=La(null,t,i,e,n);break e;case 14:t=Da(null,t,i,qi(i.type,e),r,n);break e}throw Error(a(306,i,""))}return t;case 0:return r=t.type,i=t.pendingProps,ja(e,t,r,i=t.elementType===r?i:qi(r,i),n);case 1:return r=t.type,i=t.pendingProps,Ra(e,t,r,i=t.elementType===r?i:qi(r,i),n);case 3:if(Fa(t),r=t.updateQueue,null===e||null===r)throw Error(a(282));if(r=t.pendingProps,i=null!==(i=t.memoizedState)?i.element:null,ao(e,t),co(t,r,null,n),(r=t.memoizedState.element)===i)Ta(),t=qa(e,t,n);else{if((i=t.stateNode.hydrate)&&(Aa=An(t.stateNode.containerInfo.firstChild),Ca=t,i=_a=!0),i)for(n=Mo(t,null,r,n),t.child=n;n;)n.effectTag=-3&n.effectTag|1024,n=n.sibling;else ka(e,t,r,n),Ta();t=t.child}return t;case 5:return Po(t),null===e&&wa(t),r=t.type,i=t.pendingProps,o=null!==e?e.memoizedProps:null,s=i.children,yn(r,i)?s=null:null!==o&&yn(r,o)&&(t.effectTag|=16),Na(e,t),4&t.mode&&1!==n&&i.hidden?(t.expirationTime=t.childExpirationTime=1,t=null):(ka(e,t,s,n),t=t.child),t;case 6:return null===e&&wa(t),null;case 13:return Ya(e,t,n);case 4:return Lo(t,t.stateNode.containerInfo),r=t.pendingProps,null===e?t.child=wo(t,null,r,n):ka(e,t,r,n),t.child;case 11:return r=t.type,i=t.pendingProps,La(e,t,r,i=t.elementType===r?i:qi(r,i),n);case 7:return ka(e,t,t.pendingProps,n),t.child;case 8:case 12:return ka(e,t,t.pendingProps.children,n),t.child;case 10:e:{r=t.type._context,i=t.pendingProps,s=t.memoizedProps,o=i.value;var l=t.type._context;if(ui(Zi,l._currentValue),l._currentValue=o,null!==s)if(l=s.value,0===(o=Rr(l,o)?0:0|("function"==typeof r._calculateChangedBits?r._calculateChangedBits(l,o):1073741823))){if(s.children===i.children&&!hi.current){t=qa(e,t,n);break e}}else for(null!==(l=t.child)&&(l.return=t);null!==l;){var u=l.dependencies;if(null!==u){s=l.child;for(var c=u.firstContext;null!==c;){if(c.context===r&&0!=(c.observedBits&o)){1===l.tag&&((c=so(n,null)).tag=2,lo(l,c)),l.expirationTime<n&&(l.expirationTime=n),null!==(c=l.alternate)&&c.expirationTime<n&&(c.expirationTime=n),to(l.return,n),u.expirationTime<n&&(u.expirationTime=n);break}c=c.next}}else s=10===l.tag&&l.type===t.type?null:l.child;if(null!==s)s.return=l;else for(s=l;null!==s;){if(s===t){s=null;break}if(null!==(l=s.sibling)){l.return=s.return,s=l;break}s=s.return}l=s}ka(e,t,i.children,n),t=t.child}return t;case 9:return i=t.type,r=(o=t.pendingProps).children,no(t,n),r=r(i=ro(i,o.unstable_observedBits)),t.effectTag|=1,ka(e,t,r,n),t.child;case 14:return o=qi(i=t.type,t.pendingProps),Da(e,t,i,o=qi(i.type,o),r,n);case 15:return Pa(e,t,t.type,t.pendingProps,r,n);case 17:return r=t.type,i=t.pendingProps,i=t.elementType===r?i:qi(r,i),null!==e&&(e.alternate=null,t.alternate=null,t.effectTag|=2),t.tag=1,gi(r)?(e=!0,bi(t)):e=!1,no(t,n),yo(t,r,i),Co(t,r,i,n),za(null,t,r,!0,e,n);case 19:return Xa(e,t,n)}throw Error(a(156,t.tag))};var Al=null,_l=null;function Il(e,t,n,r){this.tag=e,this.key=n,this.sibling=this.child=this.return=this.stateNode=this.type=this.elementType=null,this.index=0,this.ref=null,this.pendingProps=t,this.dependencies=this.memoizedState=this.updateQueue=this.memoizedProps=null,this.mode=r,this.effectTag=0,this.lastEffect=this.firstEffect=this.nextEffect=null,this.childExpirationTime=this.expirationTime=0,this.alternate=null}function xl(e,t,n,r){return new Il(e,t,n,r)}function wl(e){return!(!(e=e.prototype)||!e.isReactComponent)}function Ml(e,t){var n=e.alternate;return null===n?((n=xl(e.tag,t,e.key,e.mode)).elementType=e.elementType,n.type=e.type,n.stateNode=e.stateNode,n.alternate=e,e.alternate=n):(n.pendingProps=t,n.effectTag=0,n.nextEffect=null,n.firstEffect=null,n.lastEffect=null),n.childExpirationTime=e.childExpirationTime,n.expirationTime=e.expirationTime,n.child=e.child,n.memoizedProps=e.memoizedProps,n.memoizedState=e.memoizedState,n.updateQueue=e.updateQueue,t=e.dependencies,n.dependencies=null===t?null:{expirationTime:t.expirationTime,firstContext:t.firstContext,responders:t.responders},n.sibling=e.sibling,n.index=e.index,n.ref=e.ref,n}function Sl(e,t,n,r,i,o){var s=2;if(r=e,"function"==typeof e)wl(e)&&(s=1);else if("string"==typeof e)s=5;else e:switch(e){case ne:return Tl(n.children,i,o,t);case se:s=8,i|=7;break;case re:s=8,i|=1;break;case ie:return(e=xl(12,n,t,8|i)).elementType=ie,e.type=ie,e.expirationTime=o,e;case ue:return(e=xl(13,n,t,i)).type=ue,e.elementType=ue,e.expirationTime=o,e;case ce:return(e=xl(19,n,t,i)).elementType=ce,e.expirationTime=o,e;default:if("object"==typeof e&&null!==e)switch(e.$$typeof){case oe:s=10;break e;case ae:s=9;break e;case le:s=11;break e;case de:s=14;break e;case he:s=16,r=null;break e;case fe:s=22;break e}throw Error(a(130,null==e?e:typeof e,""))}return(t=xl(s,n,t,i)).elementType=e,t.type=r,t.expirationTime=o,t}function Tl(e,t,n,r){return(e=xl(7,e,r,t)).expirationTime=n,e}function El(e,t,n){return(e=xl(6,e,null,t)).expirationTime=n,e}function Ol(e,t,n){return(t=xl(4,null!==e.children?e.children:[],e.key,t)).expirationTime=n,t.stateNode={containerInfo:e.containerInfo,pendingChildren:null,implementation:e.implementation},t}function kl(e,t,n){this.tag=t,this.current=null,this.containerInfo=e,this.pingCache=this.pendingChildren=null,this.finishedExpirationTime=0,this.finishedWork=null,this.timeoutHandle=-1,this.pendingContext=this.context=null,this.hydrate=n,this.callbackNode=null,this.callbackPriority=90,this.lastExpiredTime=this.lastPingedTime=this.nextKnownPendingLevel=this.lastSuspendedTime=this.firstSuspendedTime=this.firstPendingTime=0}function Ll(e,t){var n=e.firstSuspendedTime;return e=e.lastSuspendedTime,0!==n&&n>=t&&e<=t}function Dl(e,t){var n=e.firstSuspendedTime,r=e.lastSuspendedTime;n<t&&(e.firstSuspendedTime=t),(r>t||0===n)&&(e.lastSuspendedTime=t),t<=e.lastPingedTime&&(e.lastPingedTime=0),t<=e.lastExpiredTime&&(e.lastExpiredTime=0)}function Pl(e,t){t>e.firstPendingTime&&(e.firstPendingTime=t);var n=e.firstSuspendedTime;0!==n&&(t>=n?e.firstSuspendedTime=e.lastSuspendedTime=e.nextKnownPendingLevel=0:t>=e.lastSuspendedTime&&(e.lastSuspendedTime=t+1),t>e.nextKnownPendingLevel&&(e.nextKnownPendingLevel=t))}function Nl(e,t){var n=e.lastExpiredTime;(0===n||n>t)&&(e.lastExpiredTime=t)}function jl(e,t,n,r){var i=t.current,o=Gs(),s=fo.suspense;o=Xs(o,i,s);e:if(n){t:{if($e(n=n._reactInternalFiber)!==n||1!==n.tag)throw Error(a(170));var l=n;do{switch(l.tag){case 3:l=l.stateNode.context;break t;case 1:if(gi(l.type)){l=l.stateNode.__reactInternalMemoizedMergedChildContext;break t}}l=l.return}while(null!==l);throw Error(a(171))}if(1===n.tag){var u=n.type;if(gi(u)){n=yi(n,u,l);break e}}n=l}else n=ci;return null===t.context?t.context=n:t.pendingContext=n,(t=so(o,s)).payload={element:e},null!==(r=void 0===r?null:r)&&(t.callback=r),lo(i,t),qs(i,o),o}function Rl(e){if(!(e=e.current).child)return null;switch(e.child.tag){case 5:default:return e.child.stateNode}}function zl(e,t){null!==(e=e.memoizedState)&&null!==e.dehydrated&&e.retryTime<t&&(e.retryTime=t)}function Fl(e,t){zl(e,t),(e=e.alternate)&&zl(e,t)}function Bl(e,t,n){var r=new kl(e,t,n=null!=n&&!0===n.hydrate),i=xl(3,null,null,2===t?7:1===t?3:0);r.current=i,i.stateNode=r,oo(i),e[Mn]=r.current,n&&0!==t&&function(e,t){var n=Je(t);Mt.forEach((function(e){pt(e,t,n)})),St.forEach((function(e){pt(e,t,n)}))}(0,9===e.nodeType?e:e.ownerDocument),this._internalRoot=r}function Hl(e){return!(!e||1!==e.nodeType&&9!==e.nodeType&&11!==e.nodeType&&(8!==e.nodeType||" react-mount-point-unstable "!==e.nodeValue))}function Ul(e,t,n,r,i){var o=n._reactRootContainer;if(o){var a=o._internalRoot;if("function"==typeof i){var s=i;i=function(){var e=Rl(a);s.call(e)}}jl(t,a,e,i)}else{if(o=n._reactRootContainer=function(e,t){if(t||(t=!(!(t=e?9===e.nodeType?e.documentElement:e.firstChild:null)||1!==t.nodeType||!t.hasAttribute("data-reactroot"))),!t)for(var n;n=e.lastChild;)e.removeChild(n);return new Bl(e,0,t?{hydrate:!0}:void 0)}(n,r),a=o._internalRoot,"function"==typeof i){var l=i;i=function(){var e=Rl(a);l.call(e)}}tl((function(){jl(t,a,e,i)}))}return Rl(a)}function Vl(e,t,n){var r=3<arguments.length&&void 0!==arguments[3]?arguments[3]:null;return{$$typeof:te,key:null==r?null:""+r,children:e,containerInfo:t,implementation:n}}function Yl(e,t){var n=2<arguments.length&&void 0!==arguments[2]?arguments[2]:null;if(!Hl(t))throw Error(a(200));return Vl(e,t,null,n)}Bl.prototype.render=function(e){jl(e,this._internalRoot,null,null)},Bl.prototype.unmount=function(){var e=this._internalRoot,t=e.containerInfo;jl(null,e,null,(function(){t[Mn]=null}))},gt=function(e){if(13===e.tag){var t=Xi(Gs(),150,100);qs(e,t),Fl(e,t)}},vt=function(e){13===e.tag&&(qs(e,3),Fl(e,3))},mt=function(e){if(13===e.tag){var t=Gs();qs(e,t=Xs(t,e,null)),Fl(e,t)}},T=function(e,t,n){switch(t){case"input":if(we(e,n),t=n.name,"radio"===n.type&&null!=t){for(n=e;n.parentNode;)n=n.parentNode;for(n=n.querySelectorAll("input[name="+JSON.stringify(""+t)+'][type="radio"]'),t=0;t<n.length;t++){var r=n[t];if(r!==e&&r.form===e.form){var i=On(r);if(!i)throw Error(a(90));Ae(r),we(r,i)}}}break;case"textarea":Le(e,n);break;case"select":null!=(t=n.value)&&Ee(e,!!n.multiple,t,!1)}},P=el,N=function(e,t,n,r,i){var o=Is;Is|=4;try{return Ui(98,e.bind(null,t,n,r,i))}finally{0===(Is=o)&&Wi()}},j=function(){0==(49&Is)&&(function(){if(null!==Us){var e=Us;Us=null,e.forEach((function(e,t){Nl(t,e),Ks(t)})),Wi()}}(),gl())},R=function(e,t){var n=Is;Is|=2;try{return e(t)}finally{0===(Is=n)&&Wi()}};var Wl,Gl,Xl={Events:[Tn,En,On,M,I,Rn,function(e){it(e,jn)},L,D,Kt,st,gl,{current:!1}]};Gl=(Wl={findFiberByHostInstance:Sn,bundleType:0,version:"16.14.0",rendererPackageName:"react-dom"}).findFiberByHostInstance,function(e){if("undefined"==typeof __REACT_DEVTOOLS_GLOBAL_HOOK__)return!1;var t=__REACT_DEVTOOLS_GLOBAL_HOOK__;if(t.isDisabled||!t.supportsFiber)return!0;try{var n=t.inject(e);Al=function(e){try{t.onCommitFiberRoot(n,e,void 0,64==(64&e.current.effectTag))}catch(e){}},_l=function(e){try{t.onCommitFiberUnmount(n,e)}catch(e){}}}catch(e){}}(i({},Wl,{overrideHookState:null,overrideProps:null,setSuspenseHandler:null,scheduleUpdate:null,currentDispatcherRef:Q.ReactCurrentDispatcher,findHostInstanceByFiber:function(e){return null===(e=nt(e))?null:e.stateNode},findFiberByHostInstance:function(e){return Gl?Gl(e):null},findHostInstancesForRefresh:null,scheduleRefresh:null,scheduleRoot:null,setRefreshHandler:null,getCurrentFiber:null})),t.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED=Xl,t.createPortal=Yl,t.findDOMNode=function(e){if(null==e)return null;if(1===e.nodeType)return e;var t=e._reactInternalFiber;if(void 0===t){if("function"==typeof e.render)throw Error(a(188));throw Error(a(268,Object.keys(e)))}return e=null===(e=nt(t))?null:e.stateNode},t.flushSync=function(e,t){if(0!=(48&Is))throw Error(a(187));var n=Is;Is|=1;try{return Ui(99,e.bind(null,t))}finally{Is=n,Wi()}},t.hydrate=function(e,t,n){if(!Hl(t))throw Error(a(200));return Ul(null,e,t,!0,n)},t.render=function(e,t,n){if(!Hl(t))throw Error(a(200));return Ul(null,e,t,!1,n)},t.unmountComponentAtNode=function(e){if(!Hl(e))throw Error(a(40));return!!e._reactRootContainer&&(tl((function(){Ul(null,null,e,!1,(function(){e._reactRootContainer=null,e[Mn]=null}))})),!0)},t.unstable_batchedUpdates=el,t.unstable_createPortal=function(e,t){return Yl(e,t,2<arguments.length&&void 0!==arguments[2]?arguments[2]:null)},t.unstable_renderSubtreeIntoContainer=function(e,t,n,r){if(!Hl(n))throw Error(a(200));if(null==e||void 0===e._reactInternalFiber)throw Error(a(38));return Ul(e,t,n,!1,r)},t.version="16.14.0"},function(e,t,n){"use strict";e.exports=n(1590)},function(e,t,n){"use strict";
/** @license React v0.19.1
 * scheduler.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var r,i,o,a,s;if("undefined"==typeof window||"function"!=typeof MessageChannel){var l=null,u=null,c=function(){if(null!==l)try{var e=t.unstable_now();l(!0,e),l=null}catch(e){throw setTimeout(c,0),e}},d=Date.now();t.unstable_now=function(){return Date.now()-d},r=function(e){null!==l?setTimeout(r,0,e):(l=e,setTimeout(c,0))},i=function(e,t){u=setTimeout(e,t)},o=function(){clearTimeout(u)},a=function(){return!1},s=t.unstable_forceFrameRate=function(){}}else{var h=window.performance,f=window.Date,p=window.setTimeout,g=window.clearTimeout;if("undefined"!=typeof console){var v=window.cancelAnimationFrame;"function"!=typeof window.requestAnimationFrame&&console.error("This browser doesn't support requestAnimationFrame. Make sure that you load a polyfill in older browsers. https://fb.me/react-polyfills"),"function"!=typeof v&&console.error("This browser doesn't support cancelAnimationFrame. Make sure that you load a polyfill in older browsers. https://fb.me/react-polyfills")}if("object"==typeof h&&"function"==typeof h.now)t.unstable_now=function(){return h.now()};else{var m=f.now();t.unstable_now=function(){return f.now()-m}}var y=!1,b=null,C=-1,A=5,_=0;a=function(){return t.unstable_now()>=_},s=function(){},t.unstable_forceFrameRate=function(e){0>e||125<e?console.error("forceFrameRate takes a positive int between 0 and 125, forcing framerates higher than 125 fps is not unsupported"):A=0<e?Math.floor(1e3/e):5};var I=new MessageChannel,x=I.port2;I.port1.onmessage=function(){if(null!==b){var e=t.unstable_now();_=e+A;try{b(!0,e)?x.postMessage(null):(y=!1,b=null)}catch(e){throw x.postMessage(null),e}}else y=!1},r=function(e){b=e,y||(y=!0,x.postMessage(null))},i=function(e,n){C=p((function(){e(t.unstable_now())}),n)},o=function(){g(C),C=-1}}function w(e,t){var n=e.length;e.push(t);e:for(;;){var r=n-1>>>1,i=e[r];if(!(void 0!==i&&0<T(i,t)))break e;e[r]=t,e[n]=i,n=r}}function M(e){return void 0===(e=e[0])?null:e}function S(e){var t=e[0];if(void 0!==t){var n=e.pop();if(n!==t){e[0]=n;e:for(var r=0,i=e.length;r<i;){var o=2*(r+1)-1,a=e[o],s=o+1,l=e[s];if(void 0!==a&&0>T(a,n))void 0!==l&&0>T(l,a)?(e[r]=l,e[s]=n,r=s):(e[r]=a,e[o]=n,r=o);else{if(!(void 0!==l&&0>T(l,n)))break e;e[r]=l,e[s]=n,r=s}}}return t}return null}function T(e,t){var n=e.sortIndex-t.sortIndex;return 0!==n?n:e.id-t.id}var E=[],O=[],k=1,L=null,D=3,P=!1,N=!1,j=!1;function R(e){for(var t=M(O);null!==t;){if(null===t.callback)S(O);else{if(!(t.startTime<=e))break;S(O),t.sortIndex=t.expirationTime,w(E,t)}t=M(O)}}function z(e){if(j=!1,R(e),!N)if(null!==M(E))N=!0,r(F);else{var t=M(O);null!==t&&i(z,t.startTime-e)}}function F(e,n){N=!1,j&&(j=!1,o()),P=!0;var r=D;try{for(R(n),L=M(E);null!==L&&(!(L.expirationTime>n)||e&&!a());){var s=L.callback;if(null!==s){L.callback=null,D=L.priorityLevel;var l=s(L.expirationTime<=n);n=t.unstable_now(),"function"==typeof l?L.callback=l:L===M(E)&&S(E),R(n)}else S(E);L=M(E)}if(null!==L)var u=!0;else{var c=M(O);null!==c&&i(z,c.startTime-n),u=!1}return u}finally{L=null,D=r,P=!1}}function B(e){switch(e){case 1:return-1;case 2:return 250;case 5:return 1073741823;case 4:return 1e4;default:return 5e3}}var H=s;t.unstable_IdlePriority=5,t.unstable_ImmediatePriority=1,t.unstable_LowPriority=4,t.unstable_NormalPriority=3,t.unstable_Profiling=null,t.unstable_UserBlockingPriority=2,t.unstable_cancelCallback=function(e){e.callback=null},t.unstable_continueExecution=function(){N||P||(N=!0,r(F))},t.unstable_getCurrentPriorityLevel=function(){return D},t.unstable_getFirstCallbackNode=function(){return M(E)},t.unstable_next=function(e){switch(D){case 1:case 2:case 3:var t=3;break;default:t=D}var n=D;D=t;try{return e()}finally{D=n}},t.unstable_pauseExecution=function(){},t.unstable_requestPaint=H,t.unstable_runWithPriority=function(e,t){switch(e){case 1:case 2:case 3:case 4:case 5:break;default:e=3}var n=D;D=e;try{return t()}finally{D=n}},t.unstable_scheduleCallback=function(e,n,a){var s=t.unstable_now();if("object"==typeof a&&null!==a){var l=a.delay;l="number"==typeof l&&0<l?s+l:s,a="number"==typeof a.timeout?a.timeout:B(e)}else a=B(e),l=s;return e={id:k++,callback:n,priorityLevel:e,startTime:l,expirationTime:a=l+a,sortIndex:-1},l>s?(e.sortIndex=l,w(O,e),null===M(E)&&e===M(O)&&(j?o():j=!0,i(z,l-s))):(e.sortIndex=a,w(E,e),N||P||(N=!0,r(F))),e},t.unstable_shouldYield=function(){var e=t.unstable_now();R(e);var n=M(E);return n!==L&&null!==L&&null!==n&&null!==n.callback&&n.startTime<=e&&n.expirationTime<L.expirationTime||a()},t.unstable_wrapCallback=function(e){var t=D;return function(){var n=D;D=t;try{return e.apply(this,arguments)}finally{D=n}}}},function(e,t,n){(function(t){var n=/^\s+|\s+$/g,r=/^[-+]0x[0-9a-f]+$/i,i=/^0b[01]+$/i,o=/^0o[0-7]+$/i,a=parseInt,s="object"==typeof t&&t&&t.Object===Object&&t,l="object"==typeof self&&self&&self.Object===Object&&self,u=s||l||Function("return this")(),c=Object.prototype.toString,d=Math.max,h=Math.min,f=function(){return u.Date.now()};function p(e,t,n){var r,i,o,a,s,l,u=0,c=!1,p=!1,m=!0;if("function"!=typeof e)throw new TypeError("Expected a function");function y(t){var n=r,o=i;return r=i=void 0,u=t,a=e.apply(o,n)}function b(e){return u=e,s=setTimeout(A,t),c?y(e):a}function C(e){var n=e-l;return void 0===l||n>=t||n<0||p&&e-u>=o}function A(){var e=f();if(C(e))return _(e);s=setTimeout(A,function(e){var n=t-(e-l);return p?h(n,o-(e-u)):n}(e))}function _(e){return s=void 0,m&&r?y(e):(r=i=void 0,a)}function I(){var e=f(),n=C(e);if(r=arguments,i=this,l=e,n){if(void 0===s)return b(l);if(p)return s=setTimeout(A,t),y(l)}return void 0===s&&(s=setTimeout(A,t)),a}return t=v(t)||0,g(n)&&(c=!!n.leading,o=(p="maxWait"in n)?d(v(n.maxWait)||0,t):o,m="trailing"in n?!!n.trailing:m),I.cancel=function(){void 0!==s&&clearTimeout(s),u=0,r=l=i=s=void 0},I.flush=function(){return void 0===s?a:_(f())},I}function g(e){var t=typeof e;return!!e&&("object"==t||"function"==t)}function v(e){if("number"==typeof e)return e;if(function(e){return"symbol"==typeof e||function(e){return!!e&&"object"==typeof e}(e)&&"[object Symbol]"==c.call(e)}(e))return NaN;if(g(e)){var t="function"==typeof e.valueOf?e.valueOf():e;e=g(t)?t+"":t}if("string"!=typeof e)return 0===e?e:+e;e=e.replace(n,"");var s=i.test(e);return s||o.test(e)?a(e.slice(2),s?2:8):r.test(e)?NaN:+e}e.exports=function(e,t,n){var r=!0,i=!0;if("function"!=typeof e)throw new TypeError("Expected a function");return g(n)&&(r="leading"in n?!!n.leading:r,i="trailing"in n?!!n.trailing:i),p(e,t,{leading:r,maxWait:t,trailing:i})}}).call(this,n(61))},function(e,t,n){(function(t){for(var r=n(1593),i="undefined"==typeof window?t:window,o=["moz","webkit"],a="AnimationFrame",s=i["request"+a],l=i["cancel"+a]||i["cancelRequest"+a],u=0;!s&&u<o.length;u++)s=i[o[u]+"Request"+a],l=i[o[u]+"Cancel"+a]||i[o[u]+"CancelRequest"+a];if(!s||!l){var c=0,d=0,h=[];s=function(e){if(0===h.length){var t=r(),n=Math.max(0,1e3/60-(t-c));c=n+t,setTimeout((function(){var e=h.slice(0);h.length=0;for(var t=0;t<e.length;t++)if(!e[t].cancelled)try{e[t].callback(c)}catch(e){setTimeout((function(){throw e}),0)}}),Math.round(n))}return h.push({handle:++d,callback:e,cancelled:!1}),d},l=function(e){for(var t=0;t<h.length;t++)h[t].handle===e&&(h[t].cancelled=!0)}}e.exports=function(e){return s.call(i,e)},e.exports.cancel=function(){l.apply(i,arguments)},e.exports.polyfill=function(e){e||(e=i),e.requestAnimationFrame=s,e.cancelAnimationFrame=l}}).call(this,n(61))},function(e,t,n){(function(t){(function(){var n,r,i,o,a,s;"undefined"!=typeof performance&&null!==performance&&performance.now?e.exports=function(){return performance.now()}:null!=t&&t.hrtime?(e.exports=function(){return(n()-a)/1e6},r=t.hrtime,o=(n=function(){var e;return 1e9*(e=r())[0]+e[1]})(),s=1e9*t.uptime(),a=o-s):Date.now?(e.exports=function(){return Date.now()-i},i=Date.now()):(e.exports=function(){return(new Date).getTime()-i},i=(new Date).getTime())}).call(this)}).call(this,n(145))},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){return e.displayName||e.name||("string"==typeof e&&e.length>0?e:"Unknown")}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(372);t.DragDropContext=r.DragDropContext,t.DragDropContextProvider=r.DragDropContextProvider,t.DragDropContextConsumer=r.Consumer;var i=n(1617);t.DragLayer=i.default;var o=n(1618);t.DragSource=o.default;var a=n(1626);t.DropTarget=a.default;var s=n(1628);t.DragPreviewImage=s.default;var l=n(1629);t.useDrag=l.useDrag,t.useDragLayer=l.useDragLayer,t.useDrop=l.useDrop},function(e,t,n){"use strict";function r(e){for(var n in e)t.hasOwnProperty(n)||(t[n]=e[n])}Object.defineProperty(t,"__esModule",{value:!0}),r(n(944)),r(n(1597))},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(1598);t.createDragDropManager=function(e,t,n){return new r.default(e,t,n)}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(118),i=n(1599),o=n(373),a=n(1611),s=n(1613);var l=function(){function e(e,t,n){var o=this;void 0===t&&(t={}),void 0===n&&(n=!1),this.context=t,this.isSetUp=!1,this.handleRefCountChange=function(){var e=o.store.getState().refCount>0;e&&!o.isSetUp?(o.backend.setup(),o.isSetUp=!0):!e&&o.isSetUp&&(o.backend.teardown(),o.isSetUp=!1)};var l=function(e){var t="undefined"!=typeof window&&window.__REDUX_DEVTOOLS_EXTENSION__;return r.createStore(i.default,e&&t&&t({name:"dnd-core",instanceId:"dnd-core"}))}(n);this.store=l,this.monitor=new a.default(l,new s.default(l)),this.backend=e(this),l.subscribe(this.handleRefCountChange)}return e.prototype.getContext=function(){return this.context},e.prototype.getMonitor=function(){return this.monitor},e.prototype.getBackend=function(){return this.backend},e.prototype.getRegistry=function(){return this.monitor.registry},e.prototype.getActions=function(){var e=this,t=this.store.dispatch;var n=o.default(this);return Object.keys(n).reduce((function(r,i){var o,a=n[i];return r[i]=(o=a,function(){for(var n=[],r=0;r<arguments.length;r++)n[r]=arguments[r];var i=o.apply(e,n);void 0!==i&&t(i)}),r}),{})},e.prototype.dispatch=function(e){this.store.dispatch(e)},e}();t.default=l},function(e,t,n){"use strict";var r=this&&this.__assign||function(){return(r=Object.assign||function(e){for(var t,n=1,r=arguments.length;n<r;n++)for(var i in t=arguments[n])Object.prototype.hasOwnProperty.call(t,i)&&(e[i]=t[i]);return e}).apply(this,arguments)};Object.defineProperty(t,"__esModule",{value:!0});var i=n(1600),o=n(1607),a=n(1608),s=n(1609),l=n(1610),u=n(246);t.default=function(e,t){return void 0===e&&(e={}),{dirtyHandlerIds:s.default(e.dirtyHandlerIds,{type:t.type,payload:r({},t.payload,{prevTargetIds:u.get(e,"dragOperation.targetIds",[])})}),dragOffset:i.default(e.dragOffset,t),refCount:a.default(e.refCount,t),dragOperation:o.default(e.dragOperation,t),stateId:l.default(e.stateId)}}},function(e,t,n){"use strict";var r=this&&this.__assign||function(){return(r=Object.assign||function(e){for(var t,n=1,r=arguments.length;n<r;n++)for(var i in t=arguments[n])Object.prototype.hasOwnProperty.call(t,i)&&(e[i]=t[i]);return e}).apply(this,arguments)};Object.defineProperty(t,"__esModule",{value:!0});var i=n(373),o=n(946),a={initialSourceClientOffset:null,initialClientOffset:null,clientOffset:null};t.default=function(e,t){void 0===e&&(e=a);var n=t.payload;switch(t.type){case i.INIT_COORDS:case i.BEGIN_DRAG:return{initialSourceClientOffset:n.sourceClientOffset,initialClientOffset:n.clientOffset,clientOffset:n.clientOffset};case i.HOVER:return o.areCoordsEqual(e.clientOffset,n.clientOffset)?e:r({},e,{clientOffset:n.clientOffset});case i.END_DRAG:case i.DROP:return a;default:return e}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(1602),i=n(246),o=n(19),a=n(211),s={type:a.INIT_COORDS,payload:{clientOffset:null,sourceClientOffset:null}};t.default=function(e){return function(t,n){void 0===t&&(t=[]),void 0===n&&(n={publishSource:!0});var l=n.publishSource,u=void 0===l||l,c=n.clientOffset,d=n.getSourceClientOffset,h=e.getMonitor(),f=e.getRegistry();e.dispatch(r.setClientOffset(c)),function(e,t,n){o(!t.isDragging(),"Cannot call beginDrag while dragging.");for(var r=0,i=e;r<i.length;r++){var a=i[r];o(n.getSource(a),"Expected sourceIds to be registered.")}}(t,h,f);var p=function(e,t){for(var n=null,r=e.length-1;r>=0;r--)if(t.canDragSource(e[r])){n=e[r];break}return n}(t,h);if(null!==p){var g=null;c&&(!function(e){o("function"==typeof e,"When clientOffset is provided, getSourceClientOffset must be a function.")}(d),g=d(p)),e.dispatch(r.setClientOffset(c,g));var v=f.getSource(p).beginDrag(h,p);!function(e){o(i.isObject(e),"Item must be an object.")}(v),f.pinSource(p);var m=f.getSourceType(p);return{type:a.BEGIN_DRAG,payload:{itemType:m,item:v,sourceId:p,clientOffset:c||null,sourceClientOffset:g||null,isSourcePublic:!!u}}}e.dispatch(s)}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(211);t.setClientOffset=function(e,t){return{type:r.INIT_COORDS,payload:{sourceClientOffset:t||null,clientOffset:e||null}}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(211);t.default=function(e){return function(){if(e.getMonitor().isDragging())return{type:r.PUBLISH_DRAG_SOURCE}}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(945),i=n(211),o=n(19);t.default=function(e){return function(t,n){var a=(void 0===n?{}:n).clientOffset;!function(e){o(Array.isArray(e),"Expected targetIds to be an array.")}(t);var s=t.slice(0),l=e.getMonitor(),u=e.getRegistry();return function(e,t,n){o(t.isDragging(),"Cannot call hover while not dragging."),o(!t.didDrop(),"Cannot call hover after drop.");for(var r=0;r<e.length;r++){var i=e[r];o(e.lastIndexOf(i)===r,"Expected targetIds to be unique in the passed array.");var a=n.getTarget(i);o(a,"Expected targetIds to be registered.")}}(s,l,u),function(e,t,n){for(var i=e.length-1;i>=0;i--){var o=e[i],a=t.getTargetType(o);r.default(a,n)||e.splice(i,1)}}(s,u,l.getItemType()),function(e,t,n){for(var r=0,i=e;r<i.length;r++){var o=i[r];n.getTarget(o).hover(t,o)}}(s,l,u),{type:i.HOVER,payload:{targetIds:s,clientOffset:a||null}}}}},function(e,t,n){"use strict";var r=this&&this.__assign||function(){return(r=Object.assign||function(e){for(var t,n=1,r=arguments.length;n<r;n++)for(var i in t=arguments[n])Object.prototype.hasOwnProperty.call(t,i)&&(e[i]=t[i]);return e}).apply(this,arguments)};Object.defineProperty(t,"__esModule",{value:!0});var i=n(211),o=n(246),a=n(19);t.default=function(e){return function(t){void 0===t&&(t={});var n=e.getMonitor(),s=e.getRegistry();!function(e){a(e.isDragging(),"Cannot call drop while not dragging."),a(!e.didDrop(),"Cannot call drop twice during one drag operation.")}(n),function(e){var t=e.getTargetIds().filter(e.canDropOnTarget,e);return t.reverse(),t}(n).forEach((function(l,u){var c=function(e,t,n,r){var i=n.getTarget(e),s=i?i.drop(r,e):void 0;(function(e){a(void 0===e||o.isObject(e),"Drop result must either be an object or undefined.")})(s),void 0===s&&(s=0===t?{}:r.getDropResult());return s}(l,u,s,n),d={type:i.DROP,payload:{dropResult:r({},t,c)}};e.dispatch(d)}))}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(211),i=n(19);t.default=function(e){return function(){var t=e.getMonitor(),n=e.getRegistry();!function(e){i(e.isDragging(),"Cannot call endDrag while not dragging.")}(t);var o=t.getSourceId();return n.getSource(o,!0).endDrag(t,o),n.unpinSource(),{type:r.END_DRAG}}}},function(e,t,n){"use strict";var r=this&&this.__assign||function(){return(r=Object.assign||function(e){for(var t,n=1,r=arguments.length;n<r;n++)for(var i in t=arguments[n])Object.prototype.hasOwnProperty.call(t,i)&&(e[i]=t[i]);return e}).apply(this,arguments)};Object.defineProperty(t,"__esModule",{value:!0});var i=n(373),o=n(374),a=n(246),s={itemType:null,item:null,sourceId:null,targetIds:[],dropResult:null,didDrop:!1,isSourcePublic:null};t.default=function(e,t){void 0===e&&(e=s);var n=t.payload;switch(t.type){case i.BEGIN_DRAG:return r({},e,{itemType:n.itemType,item:n.item,sourceId:n.sourceId,isSourcePublic:n.isSourcePublic,dropResult:null,didDrop:!1});case i.PUBLISH_DRAG_SOURCE:return r({},e,{isSourcePublic:!0});case i.HOVER:return r({},e,{targetIds:n.targetIds});case o.REMOVE_TARGET:return-1===e.targetIds.indexOf(n.targetId)?e:r({},e,{targetIds:a.without(e.targetIds,n.targetId)});case i.DROP:return r({},e,{dropResult:n.dropResult,didDrop:!0,targetIds:[]});case i.END_DRAG:return r({},e,{itemType:null,item:null,sourceId:null,dropResult:null,didDrop:!1,isSourcePublic:null,targetIds:[]});default:return e}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(374);t.default=function(e,t){switch(void 0===e&&(e=0),t.type){case r.ADD_SOURCE:case r.ADD_TARGET:return e+1;case r.REMOVE_SOURCE:case r.REMOVE_TARGET:return e-1;default:return e}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(373),i=n(374),o=n(946),a=n(947),s=n(246);t.default=function(e,t){switch(void 0===e&&(e=a.NONE),t.type){case r.HOVER:break;case i.ADD_SOURCE:case i.ADD_TARGET:case i.REMOVE_TARGET:case i.REMOVE_SOURCE:return a.NONE;case r.BEGIN_DRAG:case r.PUBLISH_DRAG_SOURCE:case r.END_DRAG:case r.DROP:default:return a.ALL}var n=t.payload,l=n.targetIds,u=void 0===l?[]:l,c=n.prevTargetIds,d=void 0===c?[]:c,h=s.xor(u,d);if(!(h.length>0||!o.areArraysEqual(u,d)))return a.NONE;var f=d[d.length-1],p=u[u.length-1];return f!==p&&(f&&h.push(f),p&&h.push(p)),h}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){return void 0===e&&(e=0),e+1}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(945),i=n(1612),o=n(947),a=n(19),s=function(){function e(e,t){this.store=e,this.registry=t}return e.prototype.subscribeToStateChange=function(e,t){var n=this;void 0===t&&(t={handlerIds:void 0});var r=t.handlerIds;a("function"==typeof e,"listener must be a function."),a(void 0===r||Array.isArray(r),"handlerIds, when specified, must be an array of strings.");var i=this.store.getState().stateId;return this.store.subscribe((function(){var t=n.store.getState(),a=t.stateId;try{a===i||a===i+1&&!o.areDirty(t.dirtyHandlerIds,r)||e()}finally{i=a}}))},e.prototype.subscribeToOffsetChange=function(e){var t=this;a("function"==typeof e,"listener must be a function.");var n=this.store.getState().dragOffset;return this.store.subscribe((function(){var r=t.store.getState().dragOffset;r!==n&&(n=r,e())}))},e.prototype.canDragSource=function(e){if(!e)return!1;var t=this.registry.getSource(e);return a(t,"Expected to find a valid source."),!this.isDragging()&&t.canDrag(this,e)},e.prototype.canDropOnTarget=function(e){if(!e)return!1;var t=this.registry.getTarget(e);if(a(t,"Expected to find a valid target."),!this.isDragging()||this.didDrop())return!1;var n=this.registry.getTargetType(e),i=this.getItemType();return r.default(n,i)&&t.canDrop(this,e)},e.prototype.isDragging=function(){return Boolean(this.getItemType())},e.prototype.isDraggingSource=function(e){if(!e)return!1;var t=this.registry.getSource(e,!0);return a(t,"Expected to find a valid source."),!(!this.isDragging()||!this.isSourcePublic())&&(this.registry.getSourceType(e)===this.getItemType()&&t.isDragging(this,e))},e.prototype.isOverTarget=function(e,t){if(void 0===t&&(t={shallow:!1}),!e)return!1;var n=t.shallow;if(!this.isDragging())return!1;var i=this.registry.getTargetType(e),o=this.getItemType();if(o&&!r.default(i,o))return!1;var a=this.getTargetIds();if(!a.length)return!1;var s=a.indexOf(e);return n?s===a.length-1:s>-1},e.prototype.getItemType=function(){return this.store.getState().dragOperation.itemType},e.prototype.getItem=function(){return this.store.getState().dragOperation.item},e.prototype.getSourceId=function(){return this.store.getState().dragOperation.sourceId},e.prototype.getTargetIds=function(){return this.store.getState().dragOperation.targetIds},e.prototype.getDropResult=function(){return this.store.getState().dragOperation.dropResult},e.prototype.didDrop=function(){return this.store.getState().dragOperation.didDrop},e.prototype.isSourcePublic=function(){return this.store.getState().dragOperation.isSourcePublic},e.prototype.getInitialClientOffset=function(){return this.store.getState().dragOffset.initialClientOffset},e.prototype.getInitialSourceClientOffset=function(){return this.store.getState().dragOffset.initialSourceClientOffset},e.prototype.getClientOffset=function(){return this.store.getState().dragOffset.clientOffset},e.prototype.getSourceClientOffset=function(){return i.getSourceClientOffset(this.store.getState().dragOffset)},e.prototype.getDifferenceFromInitialOffset=function(){return i.getDifferenceFromInitialOffset(this.store.getState().dragOffset)},e}();t.default=s},function(e,t,n){"use strict";function r(e,t){return{x:e.x+t.x,y:e.y+t.y}}function i(e,t){return{x:e.x-t.x,y:e.y-t.y}}Object.defineProperty(t,"__esModule",{value:!0}),t.add=r,t.subtract=i,t.getSourceClientOffset=function(e){var t=e.clientOffset,n=e.initialClientOffset,o=e.initialSourceClientOffset;return t&&n&&o?i(r(t,o),n):null},t.getDifferenceFromInitialOffset=function(e){var t=e.clientOffset,n=e.initialClientOffset;return t&&n?i(t,n):null}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(374),i=n(1614),o=n(944),a=n(1615),s=n(19),l=n(635);function u(e){switch(e[0]){case"S":return o.HandlerRole.SOURCE;case"T":return o.HandlerRole.TARGET;default:s(!1,"Cannot parse handler ID: "+e)}}function c(e,t){var n=e.entries(),r=!1;do{var i=n.next(),o=i.done;if(i.value[1]===t)return!0;r=o}while(!r);return!1}var d=function(){function e(e){this.store=e,this.types=new Map,this.dragSources=new Map,this.dropTargets=new Map,this.pinnedSourceId=null,this.pinnedSource=null}return e.prototype.addSource=function(e,t){a.validateType(e),a.validateSourceContract(t);var n=this.addHandler(o.HandlerRole.SOURCE,e,t);return this.store.dispatch(r.addSource(n)),n},e.prototype.addTarget=function(e,t){a.validateType(e,!0),a.validateTargetContract(t);var n=this.addHandler(o.HandlerRole.TARGET,e,t);return this.store.dispatch(r.addTarget(n)),n},e.prototype.containsHandler=function(e){return c(this.dragSources,e)||c(this.dropTargets,e)},e.prototype.getSource=function(e,t){return void 0===t&&(t=!1),s(this.isSourceId(e),"Expected a valid source ID."),t&&e===this.pinnedSourceId?this.pinnedSource:this.dragSources.get(e)},e.prototype.getTarget=function(e){return s(this.isTargetId(e),"Expected a valid target ID."),this.dropTargets.get(e)},e.prototype.getSourceType=function(e){return s(this.isSourceId(e),"Expected a valid source ID."),this.types.get(e)},e.prototype.getTargetType=function(e){return s(this.isTargetId(e),"Expected a valid target ID."),this.types.get(e)},e.prototype.isSourceId=function(e){return u(e)===o.HandlerRole.SOURCE},e.prototype.isTargetId=function(e){return u(e)===o.HandlerRole.TARGET},e.prototype.removeSource=function(e){var t=this;s(this.getSource(e),"Expected an existing source."),this.store.dispatch(r.removeSource(e)),l((function(){t.dragSources.delete(e),t.types.delete(e)}))},e.prototype.removeTarget=function(e){s(this.getTarget(e),"Expected an existing target."),this.store.dispatch(r.removeTarget(e)),this.dropTargets.delete(e),this.types.delete(e)},e.prototype.pinSource=function(e){var t=this.getSource(e);s(t,"Expected an existing source."),this.pinnedSourceId=e,this.pinnedSource=t},e.prototype.unpinSource=function(){s(this.pinnedSource,"No source is pinned at the time."),this.pinnedSourceId=null,this.pinnedSource=null},e.prototype.addHandler=function(e,t,n){var r=function(e){var t=i.default().toString();switch(e){case o.HandlerRole.SOURCE:return"S"+t;case o.HandlerRole.TARGET:return"T"+t;default:throw new Error("Unknown Handler Role: "+e)}}(e);return this.types.set(r,t),e===o.HandlerRole.SOURCE?this.dragSources.set(r,n):e===o.HandlerRole.TARGET&&this.dropTargets.set(r,n),r},e}();t.default=d},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=0;t.default=function(){return r++}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(19);t.validateSourceContract=function(e){r("function"==typeof e.canDrag,"Expected canDrag to be a function."),r("function"==typeof e.beginDrag,"Expected beginDrag to be a function."),r("function"==typeof e.endDrag,"Expected endDrag to be a function.")},t.validateTargetContract=function(e){r("function"==typeof e.canDrop,"Expected canDrop to be a function."),r("function"==typeof e.hover,"Expected hover to be a function."),r("function"==typeof e.drop,"Expected beginDrag to be a function.")},t.validateType=function e(t,n){n&&Array.isArray(t)?t.forEach((function(t){return e(t,!1)})):r("string"==typeof t||"symbol"==typeof t,n?"Type can only be a string, a symbol, or an array of either.":"Type can only be a string or a symbol.")}},function(e,t,n){"use strict";(function(t){function n(e){i.length||(r(),!0),i[i.length]=e}e.exports=n;var r,i=[],o=0;function a(){for(;o<i.length;){var e=o;if(o+=1,i[e].call(),o>1024){for(var t=0,n=i.length-o;t<n;t++)i[t]=i[t+o];i.length-=o,o=0}}i.length=0,o=0,!1}var s,l,u,c=void 0!==t?t:self,d=c.MutationObserver||c.WebKitMutationObserver;function h(e){return function(){var t=setTimeout(r,0),n=setInterval(r,50);function r(){clearTimeout(t),clearInterval(n),e()}}}"function"==typeof d?(s=1,l=new d(a),u=document.createTextNode(""),l.observe(u,{characterData:!0}),r=function(){s=-s,u.data=s}):r=h(a),n.requestFlush=r,n.makeRequestCallFromTimer=h}).call(this,n(61))},function(e,t,n){"use strict";var r,i=this&&this.__extends||(r=function(e,t){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(e,t)},function(e,t){function n(){this.constructor=e}r(e,t),e.prototype=null===t?Object.create(t):(n.prototype=t.prototype,new n)}),o=this&&this.__assign||function(){return(o=Object.assign||function(e){for(var t,n=1,r=arguments.length;n<r;n++)for(var i in t=arguments[n])Object.prototype.hasOwnProperty.call(t,i)&&(e[i]=t[i]);return e}).apply(this,arguments)};Object.defineProperty(t,"__esModule",{value:!0});var a=n(0),s=n(375),l=n(372),u=n(517),c=n(212),d=n(91),h=n(19),f=n(117);t.default=function(e,t){return void 0===t&&(t={}),s.default("DragLayer","collect[, options]",e,t),h("function"==typeof e,'Expected "collect" provided as the first argument to DragLayer to be a function that collects props to inject into the component. ',"Instead, received %s. Read more: http://react-dnd.github.io/react-dnd/docs/api/drag-layer",e),h(c.isPlainObject(t),'Expected "options" provided as the second argument to DragLayer to be a plain object when specified. Instead, received %s. Read more: http://react-dnd.github.io/react-dnd/docs/api/drag-layer',t),function(n){var r=n,s=t.arePropsEqual,c=void 0===s?f:s,p=r.displayName||r.name||"Component",g=function(t){function s(){var e=null!==t&&t.apply(this,arguments)||this;return e.isCurrentlyMounted=!1,e.ref=a.createRef(),e.handleChange=function(){if(e.isCurrentlyMounted){var t=e.getCurrentState();f(t,e.state)||e.setState(t)}},e}return i(s,t),s.prototype.getDecoratedComponentInstance=function(){return h(this.ref.current,"In order to access an instance of the decorated component, it must either be a class component or use React.forwardRef()"),this.ref.current},s.prototype.shouldComponentUpdate=function(e,t){return!c(e,this.props)||!f(t,this.state)},s.prototype.componentDidMount=function(){this.isCurrentlyMounted=!0,this.handleChange()},s.prototype.componentWillUnmount=function(){this.isCurrentlyMounted=!1,this.unsubscribeFromOffsetChange&&(this.unsubscribeFromOffsetChange(),this.unsubscribeFromOffsetChange=void 0),this.unsubscribeFromStateChange&&(this.unsubscribeFromStateChange(),this.unsubscribeFromStateChange=void 0)},s.prototype.render=function(){var e=this;return a.createElement(l.Consumer,null,(function(t){var n=t.dragDropManager;return void 0===n?null:(e.receiveDragDropManager(n),e.isCurrentlyMounted?a.createElement(r,o({},e.props,e.state,{ref:u.isRefable(r)?e.ref:null})):null)}))},s.prototype.receiveDragDropManager=function(e){if(void 0===this.manager){this.manager=e,h("object"==typeof e,"Could not find the drag and drop manager in the context of %s. Make sure to wrap the top-level component of your app with DragDropContext. Read more: http://react-dnd.github.io/react-dnd/docs/troubleshooting#could-not-find-the-drag-and-drop-manager-in-the-context",p,p);var t=this.manager.getMonitor();this.unsubscribeFromOffsetChange=t.subscribeToOffsetChange(this.handleChange),this.unsubscribeFromStateChange=t.subscribeToStateChange(this.handleChange)}},s.prototype.getCurrentState=function(){if(!this.manager)return{};var t=this.manager.getMonitor();return e(t,this.props)},s.displayName="DragLayer("+p+")",s.DecoratedComponent=n,s}(a.Component);return d(g,n)}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(375),i=n(948),o=n(949),a=n(1623),s=n(951),l=n(952),u=n(955),c=n(212),d=n(19);t.default=function(e,t,n,h){void 0===h&&(h={}),r.default("DragSource","type, spec, collect[, options]",e,t,n,h);var f=e;"function"!=typeof e&&(d(u.default(e),'Expected "type" provided as the first argument to DragSource to be a string, or a function that returns a string given the current props. Instead, received %s. Read more: http://react-dnd.github.io/react-dnd/docs/api/drag-source',e),f=function(){return e}),d(c.isPlainObject(t),'Expected "spec" provided as the second argument to DragSource to be a plain object. Instead, received %s. Read more: http://react-dnd.github.io/react-dnd/docs/api/drag-source',t);var p=a.default(t);return d("function"==typeof n,'Expected "collect" provided as the third argument to DragSource to be a function that returns a plain object of props to inject. Instead, received %s. Read more: http://react-dnd.github.io/react-dnd/docs/api/drag-source',n),d(c.isPlainObject(h),'Expected "options" provided as the fourth argument to DragSource to be a plain object when specified. Instead, received %s. Read more: http://react-dnd.github.io/react-dnd/docs/api/drag-source',n),function(e){return i.default({containerDisplayName:"DragSource",createHandler:p,registerHandler:o.default,createConnector:function(e){return new l.default(e)},createMonitor:function(e){return new s.default(e)},DecoratedComponent:e,getType:f,collect:n,options:h})}}},function(e,t,n){"use strict";function r(e){for(var n in e)t.hasOwnProperty(n)||(t[n]=e[n])}Object.defineProperty(t,"__esModule",{value:!0}),r(n(1620)),r(n(1621)),r(n(1622))},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(212),i=function(){function e(e){this.isDisposed=!1,this.action=r.isFunction(e)?e:r.noop}return e.isDisposable=function(e){return e&&r.isFunction(e.dispose)},e._fixup=function(t){return e.isDisposable(t)?t:e.empty},e.create=function(t){return new e(t)},e.prototype.dispose=function(){this.isDisposed||(this.action(),this.isDisposed=!0)},e.empty={dispose:r.noop},e}();t.Disposable=i},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=function(){function e(){this.isDisposed=!1}return e.prototype.getDisposable=function(){return this.current},e.prototype.setDisposable=function(e){var t=this.isDisposed;if(!t){var n=this.current;this.current=e,n&&n.dispose()}t&&e&&e.dispose()},e.prototype.dispose=function(){if(!this.isDisposed){this.isDisposed=!0;var e=this.current;this.current=void 0,e&&e.dispose()}},e}();t.SerialDisposable=r},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=function(){function e(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];this.isDisposed=!1,this.disposables=e}return e.prototype.add=function(e){this.isDisposed?e.dispose():this.disposables.push(e)},e.prototype.remove=function(e){var t=!1;if(!this.isDisposed){var n=this.disposables.indexOf(e);-1!==n&&(t=!0,this.disposables.splice(n,1),e.dispose())}return t},e.prototype.clear=function(){if(!this.isDisposed){for(var e=this.disposables.length,t=new Array(e),n=0;n<e;n++)t[n]=this.disposables[n];this.disposables=[];for(n=0;n<e;n++)t[n].dispose()}},e.prototype.dispose=function(){if(!this.isDisposed){this.isDisposed=!0;for(var e=this.disposables.length,t=new Array(e),n=0;n<e;n++)t[n]=this.disposables[n];this.disposables=[];for(n=0;n<e;n++)t[n].dispose()}},e}();t.CompositeDisposable=r},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(950),i=(n(212),n(19)),o=["canDrag","beginDrag","isDragging","endDrag"],a=["beginDrag"],s=function(){function e(e,t,n){var r=this;this.spec=e,this.monitor=t,this.ref=n,this.props=null,this.beginDrag=function(){if(r.props)return r.spec.beginDrag(r.props,r.monitor,r.ref.current)}}return e.prototype.receiveProps=function(e){this.props=e},e.prototype.canDrag=function(){return!!this.props&&(!this.spec.canDrag||this.spec.canDrag(this.props,this.monitor))},e.prototype.isDragging=function(e,t){return!!this.props&&(this.spec.isDragging?this.spec.isDragging(this.props,this.monitor):t===e.getSourceId())},e.prototype.endDrag=function(){this.props&&this.spec.endDrag&&this.spec.endDrag(this.props,this.monitor,r.getDecoratedComponent(this.ref))},e}();t.default=function(e){return Object.keys(e).forEach((function(t){i(o.indexOf(t)>-1,'Expected the drag source specification to only have some of the following keys: %s. Instead received a specification with an unexpected "%s" key. Read more: http://react-dnd.github.io/react-dnd/docs/api/drag-source',o.join(", "),t),i("function"==typeof e[t],"Expected %s in the drag source specification to be a function. Instead received a specification with %s: %s. Read more: http://react-dnd.github.io/react-dnd/docs/api/drag-source",t,t,e[t])})),a.forEach((function(t){i("function"==typeof e[t],"Expected %s in the drag source specification to be a function. Instead received a specification with %s: %s. Read more: http://react-dnd.github.io/react-dnd/docs/api/drag-source",t,t,e[t])})),function(t,n){return new s(e,t,n)}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(0),i=n(19);function o(e,t){"function"==typeof e?e(t):e.current=t}t.default=function(e,t){var n=e.ref;return i("string"!=typeof n,"Cannot connect React DnD to an element with an existing string ref. Please convert it to use a callback ref instead, or wrap it into a <span> or <div>. Read more: https://facebook.github.io/react/docs/more-about-refs.html#the-ref-callback-attribute"),n?r.cloneElement(e,{ref:function(e){o(t,e),n&&o(n,e)}}):r.cloneElement(e,{ref:t})}},function(e,t,n){"use strict";String.prototype.endsWith||(String.prototype.endsWith=function(e,t){return(void 0===t||t>this.length)&&(t=this.length),this.substring(t-e.length,t)===e})},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(375),i=n(948),o=n(956),a=n(1627),s=n(955),l=n(957),u=n(958),c=n(212),d=n(19);t.default=function(e,t,n,h){void 0===h&&(h={}),r.default("DropTarget","type, spec, collect[, options]",e,t,n,h);var f=e;"function"!=typeof e&&(d(s.default(e,!0),'Expected "type" provided as the first argument to DropTarget to be a string, an array of strings, or a function that returns either given the current props. Instead, received %s. Read more: http://react-dnd.github.io/react-dnd/docs/api/drop-target',e),f=function(){return e}),d(c.isPlainObject(t),'Expected "spec" provided as the second argument to DropTarget to be a plain object. Instead, received %s. Read more: http://react-dnd.github.io/react-dnd/docs/api/drop-target',t);var p=a.default(t);return d("function"==typeof n,'Expected "collect" provided as the third argument to DropTarget to be a function that returns a plain object of props to inject. Instead, received %s. Read more: http://react-dnd.github.io/react-dnd/docs/api/drop-target',n),d(c.isPlainObject(h),'Expected "options" provided as the fourth argument to DropTarget to be a plain object when specified. Instead, received %s. Read more: http://react-dnd.github.io/react-dnd/docs/api/drop-target',n),function(e){return i.default({containerDisplayName:"DropTarget",createHandler:p,registerHandler:o.default,createMonitor:function(e){return new l.default(e)},createConnector:function(e){return new u.default(e)},DecoratedComponent:e,getType:f,collect:n,options:h})}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(950),i=(n(212),n(19)),o=["canDrop","hover","drop"],a=function(){function e(e,t,n){this.spec=e,this.monitor=t,this.ref=n,this.props=null}return e.prototype.receiveProps=function(e){this.props=e},e.prototype.receiveMonitor=function(e){this.monitor=e},e.prototype.canDrop=function(){return!this.spec.canDrop||this.spec.canDrop(this.props,this.monitor)},e.prototype.hover=function(){this.spec.hover&&this.spec.hover(this.props,this.monitor,r.getDecoratedComponent(this.ref))},e.prototype.drop=function(){if(this.spec.drop)return this.spec.drop(this.props,this.monitor,this.ref.current)},e}();t.default=function(e){return Object.keys(e).forEach((function(t){i(o.indexOf(t)>-1,'Expected the drop target specification to only have some of the following keys: %s. Instead received a specification with an unexpected "%s" key. Read more: http://react-dnd.github.io/react-dnd/docs/api/drop-target',o.join(", "),t),i("function"==typeof e[t],"Expected %s in the drop target specification to be a function. Instead received a specification with %s: %s. Read more: http://react-dnd.github.io/react-dnd/docs/api/drop-target",t,t,e[t])})),function(t,n){return new a(e,t,n)}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(0).memo((function(e){var t=e.connect,n=e.src;if("undefined"!=typeof Image){var r=new Image;r.src=n,r.onload=function(){return t(r)}}return null}));r.displayName="DragPreviewImage",t.default=r},function(e,t,n){"use strict";function r(e){for(var n in e)t.hasOwnProperty(n)||(t[n]=e[n])}Object.defineProperty(t,"__esModule",{value:!0}),r(n(1630)),r(n(1632)),r(n(1634))},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(959),i=n(1631),o=n(0),a=n(19);t.useDrag=function(e){var t=o.useRef(e);t.current=e,a(null!=e.item,"item must be defined"),a(null!=e.item.type,"item type must be defined");var n=i.useDragSourceMonitor(),s=n[0],l=n[1];i.useDragHandler(t,s,l);var u=r.useMonitorOutput(s,t.current.collect||function(){return{}},(function(){return l.reconnect()})),c=o.useMemo((function(){return l.hooks.dragSource()}),[l]),d=o.useMemo((function(){return l.hooks.dragPreview()}),[l]);return o.useEffect((function(){l.dragSourceOptions=t.current.options||null,l.reconnect()}),[l]),o.useEffect((function(){l.dragPreviewOptions=t.current.previewOptions||null,l.reconnect()}),[l]),[u,c,d]}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(0),i=n(949),o=n(518),a=n(951),s=n(952),l=n(19);t.useDragSourceMonitor=function(){var e=o.useDragDropManager();return[r.useMemo((function(){return new a.default(e)}),[e]),r.useMemo((function(){return new s.default(e.getBackend())}),[e])]},t.useDragHandler=function(e,t,n){var a=o.useDragDropManager(),s=r.useMemo((function(){return{beginDrag:function(){var n=e.current,r=n.begin,i=n.item;if(r){var o=r(t);return l(null==o||"object"==typeof o,"dragSpec.begin() must either return an object, undefined, or null"),o||i||{}}return i||{}},canDrag:function(){return"boolean"==typeof e.current.canDrag?e.current.canDrag:"function"!=typeof e.current.canDrag||e.current.canDrag(t)},isDragging:function(n,r){var i=e.current.isDragging;return i?i(t):r===n.getSourceId()},endDrag:function(){var r=e.current.end;r&&r(t.getItem(),t),n.reconnect()}}}),[]);r.useEffect((function(){var r=i.default(e.current.item.type,s,a),o=r[0],l=r[1];return t.receiveHandlerId(o),n.receiveHandlerId(o),l}),[])}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(959),i=n(1633),o=n(0),a=n(19);t.useDrop=function(e){var t=o.useRef(e);t.current=e,a(null!=e.accept,"accept must be defined");var n=i.useDropTargetMonitor(),s=n[0],l=n[1];i.useDropHandler(t,s,l);var u=r.useMonitorOutput(s,t.current.collect||function(){return{}},(function(){return l.reconnect()})),c=o.useMemo((function(){return l.hooks.dropTarget()}),[l]);return o.useEffect((function(){l.dropTargetOptions=e.options||null,l.reconnect()}),[e.options]),[u,c]}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(0),i=n(956),o=n(518),a=n(958),s=n(957);t.useDropTargetMonitor=function(){var e=o.useDragDropManager();return[r.useMemo((function(){return new s.default(e)}),[e]),r.useMemo((function(){return new a.default(e.getBackend())}),[e])]},t.useDropHandler=function(e,t,n){var a=o.useDragDropManager(),s=r.useMemo((function(){return{canDrop:function(){var n=e.current.canDrop;return!n||n(t.getItem(),t)},hover:function(){var n=e.current.hover;n&&n(t.getItem(),t)},drop:function(){var n=e.current.drop;if(n)return n(t.getItem(),t)}}}),[t]);r.useEffect((function(){var r=i.default(e.current.accept,s,a),o=r[0],l=r[1];return t.receiveHandlerId(o),n.receiveHandlerId(o),l}),[t,n])}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(0),i=n(518),o=n(960);t.useDragLayer=function(e){var t=i.useDragDropManager().getMonitor(),n=o.useCollector(t,e),a=n[0],s=n[1];return r.useEffect((function(){return t.subscribeToOffsetChange(s)})),r.useEffect((function(){return t.subscribeToStateChange(s)})),a}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.noop=function(){},t.intBetween=function(e,t,n){return Math.floor(Math.min(t,Math.max(e,n)))},t.getCoords=function(e){if("touchmove"===e.type)return{x:e.changedTouches[0].clientX,y:e.changedTouches[0].clientY};return{x:e.clientX,y:e.clientY}}},function(e,t,n){var r,i,o=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),a=this&&this.__setModuleDefault||(Object.create?function(e,t){Object.defineProperty(e,"default",{enumerable:!0,value:t})}:function(e,t){e.default=t}),s=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)"default"!==n&&Object.hasOwnProperty.call(e,n)&&o(t,e,n);return a(t,e),t},l=this&&this.__rest||function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(null!=e&&"function"==typeof Object.getOwnPropertySymbols){var i=0;for(r=Object.getOwnPropertySymbols(e);i<r.length;i++)t.indexOf(r[i])<0&&Object.prototype.propertyIsEnumerable.call(e,r[i])&&(n[r[i]]=e[r[i]])}return n},u=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};r=[n,t,n(90),n(75),n(108),n(109),n(110),n(175),n(103),n(247),n(291),n(192),n(120),n(76),n(132),n(376),n(88),n(1637),n(377),n(378),n(1638),n(432),n(0),n(520),n(379),n(78),n(248),n(1642),n(582),n(583),n(584),n(585),n(400),n(586),n(587),n(588),n(589),n(1095),n(590),n(305)],void 0===(i=function(e,t,n,r,i,o,a,c,d,h,f,p,g,v,m,y,b,C,A,_,I,x,w,M,S,T,E,O){"use strict";function k(e){return w.default.createElement(S.default,{handle:".draggable-dialog-title",cancel:'[class*="MuiDialogContent-root"]'},w.default.createElement(v.default,Object.assign({},e)))}Object.defineProperty(t,"__esModule",{value:!0}),t.ChartElement=void 0,r=u(r),i=u(i),o=u(o),a=u(a),c=u(c),d=u(d),h=u(h),f=u(f),p=u(p),g=u(g),v=u(v),m=u(m),y=u(y),b=u(b),C=u(C),A=u(A),_=u(_),I=u(I),x=u(x),w=s(w),S=u(S),O=u(O);class L extends w.Component{constructor(e){super(e),this.prepareStateData=(e,t,n)=>{const r=[],i={},o={};Object.keys(e).forEach(t=>{const n=t.split("."),a=n.length,s=`${n[a-2]}.${n[a-1]}`,l=n.slice(0,a-2).join("."),u=e[t][1],c=["All"];if(Array.isArray(u))for(let e=0;e<u.length;e++)c.push(""+e);i[l]?(i[l].push(s),o[l][s]=c):(r.push(l),i[l]=[s],o[l]={[s]:c})});const a=Object.keys(t),s={},l={},u={};a.forEach(e=>{s[e]=[],u[e]={};for(const[n,r]of Object.entries(t[e]))if(!["Section","Status","Error code","Reference"].includes(n))if(s[e].push(n),r.length>0)if(Array.isArray(r[0])){const t=["All"];for(let e=0;e<r[0].length;e++)t.push(""+e);u[e][n]=t}else u[e][n]=["All"];else u[e][n]=["All"];const n=[...t[e].Reference];isNaN(n[0])?l[e]=n:l[e]=n.map(e=>""+e),l[e].unshift("All")});const c=Object.keys(n),d={};return c.forEach(e=>{d[e]=Object.keys(n[e])}),{listSystem:r,dictSystemVariable:i,dictSystemVariableLength:o,listRecorder:a,dictRecorderVariable:s,dictRecorderRef:l,dictRecorderVariableLength:u,listDriverWithTrace:c,dictDriverVariable:d}},this.closeDialog=()=>{this.setState(e=>Object.assign(Object.assign({},e),{openSetting:!1}))},this.openSetting=()=>{this.setState(e=>Object.assign(Object.assign({},e),{dialogType:"new",openSetting:!0}))},this.openAdvancedEdit=()=>{const e={},t=["x","y","z","a","b","name"];if(this.state.traceData.forEach(n=>{e[n.name]={};for(const r in n)t.includes(r)||(e[n.name][r]=JSON.parse(JSON.stringify(n[r])))}),Object.keys(e).length>0){const t=Object.keys(e)[0];this.setState(n=>Object.assign(Object.assign({},n),{dialogType:"advance",openSetting:!0,advancedTraceDataConfig:e,advanceTraceNameHistory:t,currentAdvancedTraceDataConfig:JSON.stringify(e[t],null,2)}))}},this.openEditing=()=>{const e=Object.keys(this.state.traceConfig)[0];if(!e)return;const t=this.state.traceConfig[e],{name:n}=t,r=l(t,["name"]);"Variables"===r.dataSource?this.setState(t=>Object.assign(Object.assign(Object.assign({},t),{listSelector:t.listSystem,dictSelector:t.dictSystemVariable,traceNameHistory:e,traceName:e,dialogType:"edit",openSetting:!0}),r)):"Recorders"===r.dataSource?this.setState(t=>Object.assign(Object.assign(Object.assign({},t),{listSelector:t.listRecorder,dictSelector:t.dictRecorderVariable,traceNameHistory:e,traceName:e,dialogType:"edit",openSetting:!0}),r)):"Drivers data"===r.dataSource&&this.setState(t=>Object.assign(Object.assign(Object.assign({},t),{listSelector:t.listDriverWithTrace,dictSelector:t.dictDriverVariable,traceNameHistory:e,traceName:e,dialogType:"edit",openSetting:!0}),r))},this.openLayoutSetting=()=>{this.setState(e=>{if(e.layoutSettingDialog)return Object.assign(Object.assign({},e),{layoutSettingDialog:!e.layoutSettingDialog});{const t={title:e.plotLayout.title.text,xAxis:e.plotLayout.xaxis.title.text,yAxis:e.plotLayout.yaxis.title.text,zAxis:e.plotLayout.zaxis.title.text,xAxisMin:"",xAxisMax:"",yAxisMax:"",yAxisMin:"",zAxisMax:"",zAxisMin:""};return["x","y","z"].forEach(n=>{const r=e.plotLayout[n+"axis"];("normal fixed"===e[n+"AxisScale"]||"log fixed"===e[n+"AxisScale"]&&r.range)&&(t[n+"AxisMin"]=r.range[0],t[n+"AxisMax"]=r.range[1])}),Object.assign(Object.assign({},e),{plotLayoutConfig:t,layoutSettingDialog:!e.layoutSettingDialog})}})},this.updatePlotLayout=()=>{this.setState(e=>Object.assign(Object.assign({},e),{layoutSettingDialog:!1,plotLayout:Object.assign(Object.assign({},e.plotLayout),{title:Object.assign(Object.assign({},e.plotLayout.title),{text:e.plotLayoutConfig.title}),xaxis:Object.assign(Object.assign({},e.plotLayout.xaxis),{title:Object.assign(Object.assign({},e.plotLayout.xaxis.title),{text:e.plotLayoutConfig.xAxis}),type:e.xAxisScale.replace(" fixed",""),range:e.xAxisScale.includes("fixed")?[e.plotLayoutConfig.xAxisMin,e.plotLayoutConfig.xAxisMax]:["",""],autorange:!e.xAxisScale.includes("fixed")}),yaxis:Object.assign(Object.assign({},e.plotLayout.yaxis),{title:Object.assign(Object.assign({},e.plotLayout.yaxis.title),{text:e.plotLayoutConfig.yAxis}),type:e.yAxisScale.replace(" fixed",""),range:e.yAxisScale.includes("fixed")?[e.plotLayoutConfig.yAxisMin,e.plotLayoutConfig.yAxisMax]:["",""],autorange:!e.yAxisScale.includes("fixed")}),zaxis:Object.assign(Object.assign({},e.plotLayout.zaxis),{title:Object.assign(Object.assign({},e.plotLayout.zaxis.title),{text:e.plotLayoutConfig.zAxis}),type:e.zAxisScale.replace(" fixed",""),range:e.zAxisScale.includes("fixed")?[e.plotLayoutConfig.zAxisMin,e.plotLayoutConfig.zAxisMax]:["",""],autorange:!e.zAxisScale.includes("fixed")}),legend:Object.assign(Object.assign({},e.plotLayout.legend),{orientation:e.legendPosition})})}))},this.handleTraceNameChange=e=>{this.setState(Object.assign(Object.assign({},this.state),{traceName:e.target.value}))},this.handlePlotTypeChange=e=>{const t=e.target.value;"contour"===t||"3D scatter"===t?this.setState(Object.assign(Object.assign({},this.state),{plotType:t,xAxisConfig:!0,yAxisConfig:!0,zAxisConfig:!0})):this.setState(Object.assign(Object.assign({},this.state),{plotType:t}))},this.handleDatasourceChange=e=>{if("Variables"===e.target.value){const e=this.state.listSystem[0],t=e,n=e,r=this.state.dictSystemVariable[e][0],i=this.state.dictSystemVariableLength[e][r][0],o=r,a=i,s=r,l=i;this.setState(u=>Object.assign(Object.assign({},u),{dataSource:"Variables",listSelector:u.listSystem,dictSelector:u.dictSystemVariable,systemNameX:e,systemNameY:t,systemNameZ:n,xData:r,yData:o,zData:s,xDataIndex:i,yDataIndex:a,zDataIndex:l}))}else if("Recorders"===e.target.value){const e=this.state.listRecorder[0],t=this.state.dictRecorderRef[e][0],n=this.state.dictRecorderVariable[e][0],r=this.state.dictRecorderVariableLength[e][n][0],i=e,o=t,a=n,s=r,l=e,u=t,c=n,d=r;this.setState(h=>Object.assign(Object.assign({},h),{dataSource:"Recorders",listSelector:h.listRecorder,dictSelector:h.dictRecorderVariable,systemNameX:e,recorderRefX:t,xData:n,xDataIndex:r,systemNameY:i,recorderRefY:o,yData:a,yDataIndex:s,systemNameZ:l,recorderRefZ:u,zData:c,zDataIndex:d}))}else if("Drivers data"===e.target.value){const e=this.state.listDriverWithTrace[0],t=this.state.dictDriverVariable[e][0],n=e,r=t,i=e,o=t;this.setState(a=>Object.assign(Object.assign({},a),{dataSource:"Drivers data",listSelector:a.listDriverWithTrace,dictSelector:a.dictDriverVariable,systemNameX:e,xData:t,systemNameY:n,yData:r,systemNameZ:i,zData:o}))}},this.handleAxisConfig=e=>()=>{const t=e+"AxisConfig",n=this.state.plotType;"contour"===n||"3d scatter"===n?"z"===e?this.setState(e=>Object.assign(Object.assign({},e),{[t]:!e[t]})):this.setState(e=>Object.assign(Object.assign({},e),{[t]:!0})):this.setState(e=>Object.assign(Object.assign({},e),{[t]:!e[t]}))},this.handleSystemSelectChange=e=>(t,n)=>{let r,i,o;r=""===n?Object.keys(this.state.dictSelector)[0]:n,i=this.state.dictSelector[r].length>0?this.state.dictSelector[r][0]:"None","Variables"===this.state.dataSource?(o=this.state.dictSystemVariableLength[r][i][0],this.setState(t=>Object.assign(Object.assign({},t),{["systemName"+e.toUpperCase()]:n,[e+"Data"]:i,[e+"DataIndex"]:o}))):"Recorders"===this.state.dataSource?(o=this.state.dictRecorderVariableLength[r][i][0],this.setState(t=>Object.assign(Object.assign({},t),{["systemName"+e.toUpperCase()]:n,[e+"Data"]:i,[e+"DataIndex"]:o}))):"Drivers data"===this.state.dataSource&&this.setState(t=>Object.assign(Object.assign({},t),{["systemName"+e.toUpperCase()]:n,[e+"Data"]:i}))},this.handleRecorderRefChange=e=>(t,n)=>{const r="recorderRef"+e.toUpperCase();this.setState(Object.assign(Object.assign({},this.state),{[r]:n}))},this.handleDataChange=e=>(t,n)=>{const r=this.state["systemName"+e.toUpperCase()];let i;"Variables"===this.state.dataSource?(i=this.state.dictSystemVariableLength[r][n][0],this.setState(Object.assign(Object.assign({},this.state),{[e+"Data"]:n,[e+"DataIndex"]:i}))):"Recorders"===this.state.dataSource?(i=this.state.dictRecorderVariableLength[r][n][0],this.setState(Object.assign(Object.assign({},this.state),{[e+"Data"]:n,[e+"DataIndex"]:i}))):"Drivers data"===this.state.dataSource&&this.setState(Object.assign(Object.assign({},this.state),{[e+"Data"]:n}))},this.handleDataIndexChange=e=>(t,n)=>{const r=e+"DataIndex";this.setState(Object.assign(Object.assign({},this.state),{[r]:n}))},this.handleAdvanceTraceSelectorChange=(e,t)=>{const n=JSON.stringify(this.state.advancedTraceDataConfig[t],null,2);this.setState(e=>Object.assign(Object.assign({},e),{advanceTraceNameHistory:t,currentAdvancedTraceDataConfig:n}))},this.handleAdvanceTraceConfigChange=(e,t,n)=>{this.setState(e=>Object.assign(Object.assign({},e),{currentAdvancedTraceDataConfig:n}))},this.handleTraceSelectorChange=(e,t)=>{const n=this.state.traceConfig[t],{name:r}=n,i=l(n,["name"]);"Variables"===i.dataSource?this.setState(e=>Object.assign(Object.assign(Object.assign({},e),{listSelector:e.listSystem,dictSelector:e.dictSystemVariable,traceNameHistory:t,traceName:t}),i)):"Recorders"===i.dataSource?this.setState(e=>Object.assign(Object.assign(Object.assign({},e),{listSelector:e.listRecorder,dictSelector:e.dictRecorderVariable,traceNameHistory:t,traceName:t}),i)):"Drivers data"===i.dataSource&&this.setState(e=>Object.assign(Object.assign(Object.assign({},e),{listSelector:e.listDriverWithTrace,dictSelector:e.dictDriverVariable,traceNameHistory:t,traceName:t}),i))},this.handlePlotLayoutChange=e=>t=>{t.persist();const{value:n}=t.target;this.setState(t=>Object.assign(Object.assign({},t),{plotLayoutConfig:Object.assign(Object.assign({},t.plotLayoutConfig),{[e]:n})}))},this.handleAxisChange=e=>t=>{const n=t.target.value;this.setState(t=>Object.assign(Object.assign({},t),{[e+"AxisScale"]:n}))},this.handleLegendPosition=e=>{const t=e.target.value;this.setState(e=>Object.assign(Object.assign({},e),{legendPosition:t}))},this.appendTrace=()=>{let e;e="X-axis"===this.state.plotLayout.xaxis.title.text&&"Y-axis"===this.state.plotLayout.yaxis.title.text?{title:{text:"New chart"},xaxis:{title:{text:this.state.xData},exponentformat:"power",autorange:!0,type:"normal"},yaxis:{title:{text:this.state.yData},exponentformat:"power",autorange:!0,type:"normal"},zaxis:{title:{text:this.state.zData},exponentformat:"power",autorange:!0,type:"normal"},showlegend:!0,legend:{orientation:this.state.legendPosition}}:this.state.plotLayout,this.setState(t=>{let n=t.traceName;""===n&&(n=`${t.xData}-${t.yData}:${t.plotType}`);const r={name:n,plotType:t.plotType,dataSource:t.dataSource,xAxisConfig:t.xAxisConfig,systemNameX:t.systemNameX,recorderRefX:t.recorderRefX,xData:t.xData,xDataIndex:t.xDataIndex,yAxisConfig:t.yAxisConfig,systemNameY:t.systemNameY,recorderRefY:t.recorderRefY,yData:t.yData,yDataIndex:t.yDataIndex,zAxisConfig:t.zAxisConfig,systemNameZ:t.systemNameZ,recorderRefZ:t.recorderRefZ,zData:t.zData,zDataIndex:t.zDataIndex},i=Object.assign(Object.assign({},t.traceConfig),{[n]:r}),o=this.plotDataFactory(i,t.dictRecorderRef,t.advancedTraceDataConfig);return Object.assign(Object.assign({},t),{plotLayout:e,traceConfig:i,traceData:o})})},this.newTrace=()=>{let e;e="X-axis"===this.state.plotLayout.xaxis.title.text&&"Y-axis"===this.state.plotLayout.yaxis.title.text&&"New chart"===this.state.plotLayout.title.text?{title:{text:"New chart"},xaxis:{title:{text:this.state.xData},exponentformat:"power",autorange:!0,type:"normal"},yaxis:{title:{text:this.state.yData},exponentformat:"power",autorange:!0,type:"normal"},zaxis:{title:{text:this.state.zData},exponentformat:"power",autorange:!0,type:"normal"},showlegend:!0,legend:{orientation:this.state.legendPosition}}:this.state.plotLayout,this.setState(t=>{let n=t.traceName;""===n&&(n=`${t.xData}-${t.yData}:${t.plotType}`);const r={[n]:{name:n,plotType:t.plotType,dataSource:t.dataSource,xAxisConfig:t.xAxisConfig,systemNameX:t.systemNameX,recorderRefX:t.recorderRefX,xData:t.xData,xDataIndex:t.xDataIndex,yAxisConfig:t.yAxisConfig,systemNameY:t.systemNameY,recorderRefY:t.recorderRefY,yData:t.yData,yDataIndex:t.yDataIndex,zAxisConfig:t.zAxisConfig,systemNameZ:t.systemNameZ,recorderRefZ:t.recorderRefZ,zData:t.zData,zDataIndex:t.zDataIndex}},i=this.plotDataFactory(r,t.dictRecorderRef,this.state.advancedTraceDataConfig);return Object.assign(Object.assign({},t),{plotLayout:e,traceConfig:r,traceData:i})})},this.plotDataFactory=(e,t,n={},r="interface")=>{const i={};for(const[t,n]of Object.entries(e))if("Recorders"===n.dataSource){const e=this.state.dictRecorderVariableLength[n.systemNameX][n.xData].slice(1),r=this.state.dictRecorderVariableLength[n.systemNameY][n.yData].slice(1);if(n.xAxisConfig&&n.yAxisConfig)if("All"===n.recorderRefX&&"All"===n.recorderRefY)if("All"===n.xDataIndex)if("All"===n.yDataIndex)for(let o=0;o<Math.min(e.length,r.length);o++){const e=`${t}[${o}]`,r=Object.assign({},n);r.name=e,r.xDataIndex=o+"",r.yDataIndex=o+"",i[e]=r}else for(let r=0;r<e.length;r++){const e=`${t}[${r}]`,o=Object.assign({},n);o.name=e,o.xDataIndex=r+"",i[e]=o}else if("All"===n.yDataIndex)for(let e=0;e<r.length;e++){const r=`${t}[${e}]`,o=Object.assign({},n);o.yDataIndex=e+"",i[r]=o}else i[t]=n;else i[t]=n;else if(n.xAxisConfig&&!n.yAxisConfig)if("All"===n.recorderRefX)if("All"===n.xDataIndex)for(let r=0;r<e.length;r++){const e=`${t}[${r}]`,o=Object.assign({},n);o.name=e,o.xDataIndex=r+"",i[e]=o}else i[t]=n;else i[t]=n;else if(!n.xAxisConfig&&n.yAxisConfig)if("All"===n.recorderRefY)if("All"===n.yDataIndex)for(let e=0;e<r.length;e++){const r=`${t}[${e}]`,o=Object.assign({},n);o.name=r,o.yDataIndex=e+"",i[r]=o}else i[t]=n;else i[t]=n;else i[t]=n}else i[t]=n;const o=[],a=[1/0,-1/0];for(const e in i){const n=i[e],r={name:n.name};let s,l,u,c=null,d=!1;if("lines"===n.plotType?(r.type="scatter",r.mode="lines"):"bar"===n.plotType?r.type="bar":"scatter"===n.plotType?(r.type="scatter",r.mode="markers"):"contour"===n.plotType?(r.type="carpet",r.aaxis={showticklabels:"none",showgrid:!1},r.baxis={showticklabels:"none",showgrid:!1},r.carpet=Math.random().toString(36).substr(2,16)+Math.random().toString(36).substr(2,16),d=!0):"3d scatter"===n.plotType&&(r.type="scatter3d",r.mode="markers",d=!0),"Variables"===n.dataSource){const e=`${n.systemNameX}.${n.xData}`,t=`${n.systemNameY}.${n.yData}`;if(s="All"===n.xDataIndex?this.props.computedResult[e][1]:this.props.computedResult[e][1][parseInt(n.xDataIndex)],l="All"===n.yDataIndex?this.props.computedResult[t][1]:this.props.computedResult[t][1][parseInt(n.yDataIndex)],d){const e=`${n.systemNameZ}.${n.zData}`;u="All"===n.zDataIndex?this.props.computedResult[e][1]:this.props.computedResult[e][1][parseInt(n.zDataIndex)]}}else if("Recorders"===n.dataSource){const e=this.props.recorderData[n.systemNameX][n.xData];s=L.recorderDataSelector(t,n.recorderRefX,n.xDataIndex,n.systemNameX,e);const r=this.props.recorderData[n.systemNameY][n.yData];if(l=L.recorderDataSelector(t,n.recorderRefY,n.yDataIndex,n.systemNameY,r),d){const e=this.props.recorderData[n.systemNameZ][n.zData];u=L.recorderDataSelector(t,n.recorderRefZ,n.zDataIndex,n.systemNameZ,e)}}else"Drivers data"===n.dataSource&&(s=this.props.driverData[n.systemNameX][n.xData],l=this.props.driverData[n.systemNameY][n.yData]);if(n.xAxisConfig&&n.yAxisConfig){if(r.x=Array.isArray(s)?s:[s],r.y=Array.isArray(l)?l:[l],"contour"===n.plotType){const{a:e,b:t,x:i,y:o,z:d}=E.carpet_data_factory(s,l,u);r.a=e,r.b=t,r.x=i,r.y=o,r.z=d,a[0]=Math.min(a[0],Math.min(...d)),a[1]=Math.max(a[1],Math.max(...d)),n.zAxisConfig?c={name:n.name+"_contour_carpet",type:"contourcarpet",a:e,b:t,z:d,colorscale:"RdBu",contours:{showlabels:!0},carpet:r.carpet,colorbar:{exponentformat:"power"}}:(r.aaxis={showticklabels:"none",showgrid:!1},r.baxis={showticklabels:"none",showgrid:!1})}else"3d scatter"===n.plotType&&(r.z=Array.isArray(u)?u:[u]);"Recorders"===n.dataSource&&"All"===n.recorderRefX&&"All"===n.recorderRefY&&(r.text=t[n.systemNameX].slice(1).map(e=>"Ref. = "+e))}else n.xAxisConfig&&!n.yAxisConfig?(r.x=Array.isArray(s)?s:[s],"Recorders"===n.dataSource&&"All"===n.recorderRefX&&(r.y=t[n.systemNameX].slice(1))):!n.xAxisConfig&&n.yAxisConfig&&(r.y=Array.isArray(l)?l:[l],"Recorders"===n.dataSource&&"All"===n.recorderRefY&&(r.x=t[n.systemNameY].slice(1)));o.push(r),c&&o.push(c)}return o.forEach(e=>{if("contourcarpet"===e.type&&a[0]!==1/0&&a[1]!==-1/0){const t=a[0],n=a[1],r=(n-t)/10;e.autocontour=!1,e.contours={showlabels:!0,start:t,end:n,size:r}}const t=n[e.name];if(t)for(const n in t)"advanced"===r?e[n]=JSON.parse(JSON.stringify(t[n])):"interface"===r&&(n in e||(e[n]=JSON.parse(JSON.stringify(t[n]))))}),o},this.removeAdvancedSetting=()=>{const e=this.state.advanceTraceNameHistory,t=this.state.advancedTraceDataConfig,n=e,r=(t[n],l(t,["symbol"==typeof n?n:n+""])),i=this.plotDataFactory(this.state.traceConfig,this.state.dictRecorderRef,r,"advanced");if(0===Object.keys(r).length)this.setState(e=>Object.assign(Object.assign({},e),{advancedTraceDataConfig:r,traceData:i}),this.closeDialog);else{const e=Object.keys(r)[0],t=JSON.stringify(r[e],null,2);this.setState(n=>Object.assign(Object.assign({},n),{advancedTraceDataConfig:r,traceData:i,advanceTraceNameHistory:e,currentAdvancedTraceDataConfig:t}))}},this.saveAdvancedSetting=()=>{const e=this.state.advanceTraceNameHistory;try{const t=JSON.parse(this.state.currentAdvancedTraceDataConfig),n=Object.assign(Object.assign({},this.state.advancedTraceDataConfig),{[e]:t}),r=this.plotDataFactory(this.state.traceConfig,this.state.dictRecorderRef,n,"advanced");this.setState(e=>Object.assign(Object.assign({},e),{advancedTraceDataConfig:n,traceData:r}))}catch(e){alert(e)}},this.saveSetting=()=>{const e=this.state.traceNameHistory,t=this.state.traceName;this.setState(n=>{const r={name:t,plotType:n.plotType,dataSource:n.dataSource,xAxisConfig:n.xAxisConfig,systemNameX:n.systemNameX,recorderRefX:n.recorderRefX,xData:n.xData,xDataIndex:n.xDataIndex,yAxisConfig:n.yAxisConfig,systemNameY:n.systemNameY,recorderRefY:n.recorderRefY,yData:n.yData,yDataIndex:n.yDataIndex,zAxisConfig:n.zAxisConfig,systemNameZ:n.systemNameZ,recorderRefZ:n.recorderRefZ,zData:n.zData,zDataIndex:n.zDataIndex},i=n.traceConfig,o=e,a=(i[o],l(i,["symbol"==typeof o?o:o+""])),s=Object.assign(Object.assign({},a),{[t]:r}),u=this.plotDataFactory(s,n.dictRecorderRef,this.state.advancedTraceDataConfig);return Object.assign(Object.assign({},n),{traceNameHistory:t,traceConfig:s,traceData:u})})},this.removeTrace=()=>{const e=this.state.traceNameHistory;this.setState(t=>{const n=t.traceConfig,r=e,i=(n[r],l(n,["symbol"==typeof r?r:r+""])),o=[];for(let n=0;n<t.traceData.length;n++){const r=t.traceData[n];r.name!==e&&o.push(r)}const a=Object.keys(i)[0];return Object.assign(Object.assign({},t),{traceNameHistory:a,traceConfig:i,traceData:o})})},this.layoutAxisConfigFactory=e=>w.default.createElement(h.default,{style:{flexDirection:"row"}},w.default.createElement(d.default,{className:this.props.classes.formControlTiny,style:{margin:"8px 8px 8px 8px"}},w.default.createElement(b.default,{label:e.toUpperCase()+"-Axis title",className:this.props.classes.textField,value:this.state.plotLayoutConfig[e+"Axis"],onChange:this.handlePlotLayoutChange(e+"Axis"),margin:"normal"})),w.default.createElement(d.default,{className:this.props.classes.formControlTiny,style:{paddingTop:"16px"}},w.default.createElement("div",{style:{width:"90%",margin:"8px 8px 8px 8px",position:"relative"}},w.default.createElement(p.default,{id:e+"-axis-scale-id"},e.toUpperCase()+"-Axis scale"),w.default.createElement(m.default,{labelId:e+"-axis-scale-id",style:{width:"100%"},value:this.state[e+"AxisScale"],onChange:this.handleAxisChange(""+e)},w.default.createElement(g.default,{value:"normal"},"Linear Auto"),w.default.createElement(g.default,{value:"normal fixed"},"Linear fixed"),w.default.createElement(g.default,{value:"log"},"Log Auto"),w.default.createElement(g.default,{value:"log fixed"},"Log fixed")))),w.default.createElement(d.default,{className:this.props.classes.formControlTiny,style:{margin:"8px 8px 8px 8px"}},w.default.createElement(b.default,{label:e.toUpperCase()+"-Axis min",className:this.props.classes.textField,value:this.state.plotLayoutConfig[e+"AxisMin"],onChange:this.handlePlotLayoutChange(e+"AxisMin"),margin:"normal"})),w.default.createElement(d.default,{className:this.props.classes.formControlTiny,style:{margin:"8px 8px 8px 8px"}},w.default.createElement(b.default,{label:e.toUpperCase()+"-Axis max",className:this.props.classes.textField,value:this.state.plotLayoutConfig[e+"AxisMax"],onChange:this.handlePlotLayoutChange(e+"AxisMax"),margin:"normal"}))),this.initialState=e.initialState;const{listSystem:t,dictSystemVariable:n,dictSystemVariableLength:r,listRecorder:i,dictRecorderVariable:o,dictRecorderRef:a,dictRecorderVariableLength:s,listDriverWithTrace:u,dictDriverVariable:c}=Object.assign({},this.prepareStateData(e.computedResult,e.recorderData,e.driverData));this.state={renderSignal:-1,openSetting:!1,layoutSettingDialog:!1,plotLayoutConfig:{title:"",xAxis:"",yAxis:"",zAxis:"",xAxisMin:"",xAxisMax:"",yAxisMax:"",yAxisMin:"",zAxisMax:"",zAxisMin:""},legendPosition:"h",xAxisScale:"normal",yAxisScale:"normal",zAxisScale:"normal",dialogType:"new",traceNameHistory:"",advanceTraceNameHistory:"",advancedTraceDataConfig:{},currentAdvancedTraceDataConfig:"{}",traceName:"",dataSource:"Variables",xAxisConfig:!0,systemNameX:t[0],recorderRefX:"All",xData:n[t[0]][0],xDataIndex:"All",yAxisConfig:!0,systemNameY:t[0],recorderRefY:"All",yData:n[t[0]][0],yDataIndex:"All",zAxisConfig:!0,systemNameZ:t[0],recorderRefZ:"All",zData:n[t[0]][0],zDataIndex:"All",listSystem:t,dictSystemVariable:n,dictSystemVariableLength:r,listRecorder:i,dictRecorderVariable:o,dictRecorderVariableLength:s,dictRecorderRef:a,listDriverWithTrace:u,dictDriverVariable:c,listSelector:t,dictSelector:n,plotType:"scatter",traceConfig:{},traceData:[],plotLayout:{title:{text:"New chart"},xaxis:{title:{text:"X-axis"},exponentformat:"power",autorange:!0,type:"normal"},yaxis:{title:{text:"Y-axis"},exponentformat:"power",autorange:!0,type:"normal"},zaxis:{title:{text:"Y-axis"},exponentformat:"power",autorange:!0,type:"normal"}}}}componentDidMount(){this.initialState&&this.setState(e=>{const t=this.plotDataFactory(this.initialState.traceConfig,e.dictRecorderRef,this.initialState.advancedTraceDataConfig,"advanced");return Object.assign(Object.assign({},e),{traceData:t,plotLayout:this.initialState.plotLayout,traceConfig:this.initialState.traceConfig,advancedTraceDataConfig:this.initialState.advancedTraceDataConfig})})}componentDidUpdate(e,t){if(e.computedResult!==this.props.computedResult){const{listSystem:e,dictSystemVariable:t,dictSystemVariableLength:n,listRecorder:r,dictRecorderVariable:i,dictRecorderRef:o,dictRecorderVariableLength:a,listDriverWithTrace:s,dictDriverVariable:l}=Object.assign({},this.prepareStateData(this.props.computedResult,this.props.recorderData,this.props.driverData));this.setState(e=>{const t=this.plotDataFactory(e.traceConfig,o,e.advancedTraceDataConfig,"advanced");return Object.assign(Object.assign({},e),{traceData:t,dictSystemVariableLength:n,dictRecorderRef:o,dictRecorderVariableLength:a,listDriverWithTrace:s,dictDriverVariable:l})})}}render(){let e,t;this.dialogInput={traceSelector:w.default.createElement(d.default,{className:this.props.classes.formControl},w.default.createElement(x.default,{disableClearable:!0,value:this.state.traceNameHistory,onChange:this.handleTraceSelectorChange,options:Object.keys(this.state.traceConfig),renderInput:e=>w.default.createElement(b.default,Object.assign({},e,{variant:"standard",label:"Select trace",placeholder:"",fullWidth:!0}))})),traceName:w.default.createElement(d.default,{className:this.props.classes.formControl},w.default.createElement(b.default,{value:this.state.traceName,onChange:this.handleTraceNameChange,label:"Trace name"})),dataSource:w.default.createElement(d.default,{className:this.props.classes.formControlShort},w.default.createElement(p.default,{id:"data-source-id"},"Select data source"),w.default.createElement(m.default,{labelId:"data-source-id",value:this.state.dataSource,onChange:this.handleDatasourceChange},w.default.createElement(g.default,{value:"Variables"},"Variables"),w.default.createElement(g.default,{disabled:0===this.state.listRecorder.length,value:"Recorders"},"Recorders"),w.default.createElement(g.default,{disabled:0===this.state.listDriverWithTrace.length,value:"Drivers data"},"Drivers data"))),plotType:w.default.createElement(d.default,{className:this.props.classes.formControlShort},w.default.createElement(p.default,{id:"plot-type-select-id"},"Select plot type"),w.default.createElement(m.default,{labelId:"plot-type-select-id",value:this.state.plotType,onChange:this.handlePlotTypeChange},w.default.createElement(g.default,{value:"scatter"},"Scatter charts"),w.default.createElement(g.default,{value:"lines"},"Line charts"),w.default.createElement(g.default,{value:"bar"},"Bar charts"),w.default.createElement(g.default,{value:"contour"},"Contour charts"),w.default.createElement(g.default,{value:"3d scatter"},"3D Scatter charts"))),systemAndRefX:D({classes:this.props.classes,axisConfig:this.state.xAxisConfig,handleaxisConfig:this.handleAxisConfig("x"),systemName:this.state.systemNameX,handleSystemSelectChange:this.handleSystemSelectChange("x"),listSelector:this.state.listSelector,dictSelector:this.state.dictSelector,dataSource:this.state.dataSource,recorderRef:this.state.recorderRefX,handlerecorderRefChange:this.handleRecorderRefChange("x"),dictRecorderRef:this.state.dictRecorderRef,axisData:this.state.xData,handleDataChange:this.handleDataChange("x"),dataIndex:this.state.xDataIndex,handleDataIndexChange:this.handleDataIndexChange("x"),dictRecorderVariableLength:this.state.dictRecorderVariableLength,dictSystemVariableLength:this.state.dictSystemVariableLength},"X"),systemAndRefY:D({classes:this.props.classes,axisConfig:this.state.yAxisConfig,handleaxisConfig:this.handleAxisConfig("y"),systemName:this.state.systemNameY,handleSystemSelectChange:this.handleSystemSelectChange("y"),listSelector:this.state.listSelector,dictSelector:this.state.dictSelector,dataSource:this.state.dataSource,recorderRef:this.state.recorderRefY,handlerecorderRefChange:this.handleRecorderRefChange("y"),dictRecorderRef:this.state.dictRecorderRef,axisData:this.state.yData,handleDataChange:this.handleDataChange("y"),dataIndex:this.state.yDataIndex,handleDataIndexChange:this.handleDataIndexChange("y"),dictRecorderVariableLength:this.state.dictRecorderVariableLength,dictSystemVariableLength:this.state.dictSystemVariableLength},"Y"),systemAndRefZ:D({classes:this.props.classes,axisConfig:this.state.zAxisConfig,handleaxisConfig:this.handleAxisConfig("z"),systemName:this.state.systemNameZ,handleSystemSelectChange:this.handleSystemSelectChange("z"),listSelector:this.state.listSelector,dictSelector:this.state.dictSelector,dataSource:this.state.dataSource,recorderRef:this.state.recorderRefZ,handlerecorderRefChange:this.handleRecorderRefChange("z"),dictRecorderRef:this.state.dictRecorderRef,axisData:this.state.zData,handleDataChange:this.handleDataChange("z"),dataIndex:this.state.zDataIndex,handleDataIndexChange:this.handleDataIndexChange("z"),dictRecorderVariableLength:this.state.dictRecorderVariableLength,dictSystemVariableLength:this.state.dictSystemVariableLength},"Z")},"new"===this.state.dialogType?(e=w.default.createElement(o.default,null,w.default.createElement(r.default,{autoFocus:!0,onClick:this.closeDialog,color:"primary"},"Close"),w.default.createElement(r.default,{autoFocus:!0,onClick:this.appendTrace,color:"primary"},"Append"),w.default.createElement(r.default,{autoFocus:!0,onClick:this.newTrace,color:"primary"},"Plot")),t=w.default.createElement(a.default,{style:{paddingTop:"0px"}},this.dialogInput.traceName,w.default.createElement(d.default,{className:this.props.classes.formControl},w.default.createElement(h.default,{style:{flexDirection:"row"}},this.dialogInput.plotType,this.dialogInput.dataSource)),this.dialogInput.systemAndRefX,this.dialogInput.systemAndRefY,["contour","3d scatter"].includes(this.state.plotType)&&"Drivers data"!==this.state.dataSource?this.dialogInput.systemAndRefZ:w.default.createElement("div",null))):"edit"===this.state.dialogType?(t=w.default.createElement(a.default,null,this.dialogInput.traceSelector,this.dialogInput.traceName,w.default.createElement(d.default,{className:this.props.classes.formControl},w.default.createElement(h.default,{style:{flexDirection:"row"}},this.dialogInput.plotType,this.dialogInput.dataSource)),this.dialogInput.systemAndRefX,this.dialogInput.systemAndRefY,["contour","3d scatter"].includes(this.state.plotType)?this.dialogInput.systemAndRefZ:w.default.createElement("div",null)),e=w.default.createElement(o.default,null,w.default.createElement(r.default,{autoFocus:!0,onClick:this.closeDialog,color:"primary"},"Close"),w.default.createElement(r.default,{autoFocus:!0,onClick:this.removeTrace,color:"primary"},"Remove"),w.default.createElement(r.default,{autoFocus:!0,onClick:this.saveSetting,color:"primary"},"Save"))):"advance"===this.state.dialogType&&(t=w.default.createElement(a.default,null,w.default.createElement(d.default,{className:this.props.classes.formControl},w.default.createElement(x.default,{disableClearable:!0,value:this.state.advanceTraceNameHistory,onChange:this.handleAdvanceTraceSelectorChange,options:Object.keys(this.state.advancedTraceDataConfig),renderInput:e=>w.default.createElement(b.default,Object.assign({},e,{variant:"standard",label:"Select trace",placeholder:"",fullWidth:!0}))})),w.default.createElement(d.default,{className:this.props.classes.formControl},w.default.createElement(M.Controlled,{value:this.state.currentAdvancedTraceDataConfig,options:{mode:"javascript",lineNumbers:!0,lineWrapping:!0,smartIndent:!0,indentWithTabs:!1,tabSize:4,indentUnit:4,foldGutter:!0,gutters:["CodeMirror-linenumbers","CodeMirror-foldgutter"],autoCloseTags:!0,matchBrackets:!0,autoCloseBrackets:!0},onBeforeChange:this.handleAdvanceTraceConfigChange,onChange:(e,t,n)=>{}}))),e=w.default.createElement(o.default,null,w.default.createElement(r.default,{autoFocus:!0,onClick:this.closeDialog,color:"primary"},"Close"),w.default.createElement(r.default,{autoFocus:!0,onClick:this.removeAdvancedSetting,color:"secondary"},"Remove"),w.default.createElement(r.default,{autoFocus:!0,onClick:this.saveAdvancedSetting,color:"primary"},"Save")));const n=this.layoutAxisConfigFactory("x"),s=this.layoutAxisConfigFactory("y"),l=this.layoutAxisConfigFactory("z");return w.default.createElement("div",{className:"cosapp-widget-box"},w.default.createElement(i.default,{open:this.state.openSetting,"aria-labelledby":"draggable-dialog-title",fullWidth:!0,maxWidth:"md",PaperComponent:k},w.default.createElement(c.default,{style:{cursor:"move",paddingTop:"8px",paddingBottom:"0px"},className:"draggable-dialog-title"},"Plot configuration"),t,e),w.default.createElement(i.default,{open:this.state.layoutSettingDialog,"aria-labelledby":"draggable-dialog-title",fullWidth:!0,maxWidth:"md",PaperComponent:k},w.default.createElement(c.default,{style:{cursor:"move"},className:"draggable-dialog-title"},"Layout configuration"),w.default.createElement(a.default,null,w.default.createElement(b.default,{label:"Plot title",className:this.props.classes.textField,value:this.state.plotLayoutConfig.title,onChange:this.handlePlotLayoutChange("title"),margin:"normal"}),n,s,l,w.default.createElement(h.default,{style:{flexDirection:"row"}},w.default.createElement(d.default,{className:this.props.classes.formControlShort},w.default.createElement("div",{style:{width:"90%",margin:"8px 8px 8px 8px",position:"relative"}},w.default.createElement(p.default,{id:"legend-position-id"},"Select legend position"),w.default.createElement(m.default,{labelId:"legend-position-id",style:{width:"100%"},value:this.state.legendPosition,onChange:this.handleLegendPosition},w.default.createElement(g.default,{value:"v"},"Vertical"),w.default.createElement(g.default,{value:"h"},"Horizontal")))))),w.default.createElement(o.default,null,w.default.createElement(r.default,{autoFocus:!0,onClick:this.openLayoutSetting,color:"primary"},"Close"),w.default.createElement(r.default,{autoFocus:!0,onClick:this.updatePlotLayout,color:"primary"},"Save"))),w.default.createElement(O.default,{style:{width:"100%",height:"calc(100% - 35px)"},data:this.state.traceData,layout:this.state.plotLayout,config:{responsive:!0}}),w.default.createElement("div",{className:this.props.classes.bottomBarDiv},w.default.createElement(r.default,{onClick:this.openSetting,style:{color:"rgb(50,50,50)"}},w.default.createElement(C.default,null)),w.default.createElement(r.default,{onClick:this.openEditing,style:{color:"rgb(50,50,50)"}},w.default.createElement(A.default,null)),w.default.createElement(r.default,{onClick:this.openAdvancedEdit,style:{color:"rgb(50,50,50)"}},w.default.createElement(I.default,null)),w.default.createElement(r.default,{onClick:this.openLayoutSetting,style:{color:"rgb(50,50,50)"}}," ",w.default.createElement(_.default,null))))}}function D(e,t){let n;return"Recorders"===e.dataSource||"Variables"===e.dataSource?n=w.default.createElement(x.default,{disabled:!e.axisConfig,disableClearable:!0,value:e.dataIndex,onChange:e.handleDataIndexChange,options:"Recorders"===e.dataSource?e.dictRecorderVariableLength[e.systemName][e.axisData]:e.dictSystemVariableLength[e.systemName][e.axisData],renderInput:e=>w.default.createElement(b.default,Object.assign({},e,{variant:"standard",label:"Select variable index",placeholder:"",fullWidth:!0}))}):"Drivers data"===e.dataSource&&(n=w.default.createElement(x.default,{disabled:!0,disableClearable:!0,value:"None",options:["None"],renderInput:e=>w.default.createElement(b.default,Object.assign({},e,{variant:"standard",label:"Select variable index",placeholder:"",fullWidth:!0}))})),w.default.createElement(d.default,{className:e.classes.formControl},w.default.createElement(f.default,{component:"legend"},w.default.createElement("span",{style:{color:e.axisConfig?"#3f51b5":"rgba(0, 0, 0, 0.54)"}},t+"-Axis configuration"),w.default.createElement("span",{style:{marginLeft:"5%"}}),w.default.createElement(y.default,{checked:e.axisConfig,onChange:e.handleaxisConfig,color:"primary",size:"small"})),w.default.createElement(h.default,{style:{flexDirection:"row"}},w.default.createElement(x.default,{disabled:!e.axisConfig,className:e.classes.formControlShort,value:e.systemName,onChange:e.handleSystemSelectChange,options:e.listSelector,getOptionLabel:e=>e.replace("chart_viewer.",""),defaultValue:e.listSelector[0],disableClearable:!0,renderInput:t=>w.default.createElement(b.default,Object.assign({},t,{variant:"standard",label:"Recorders"!==e.dataSource?"Select system":"Select recorder",placeholder:"",fullWidth:!0}))}),w.default.createElement(x.default,{disabled:!e.axisConfig||"Recorders"!==e.dataSource,disableClearable:!0,className:e.classes.formControlShort,value:"Recorders"===e.dataSource?e.recorderRef:"None",onChange:e.handlerecorderRefChange,options:"Recorders"===e.dataSource?e.dictRecorderRef[e.systemName]:["None"],defaultValue:"All",renderInput:e=>w.default.createElement(b.default,Object.assign({},e,{variant:"standard",label:"Select reference",placeholder:"",fullWidth:!0}))}),w.default.createElement(d.default,{className:e.classes.formControlShort},w.default.createElement(x.default,{disabled:!e.axisConfig,disableClearable:!0,value:e.axisData,onChange:e.handleDataChange,options:e.dictSelector[e.systemName],renderInput:e=>w.default.createElement(b.default,Object.assign({},e,{variant:"standard",label:"Select variable",placeholder:"",fullWidth:!0}))})),w.default.createElement(d.default,{className:e.classes.formControlShort},n)))}t.ChartElement=L,L.recorderDataSelector=(e,t,n,r,i)=>{let o;if("All"===t)"All"===n?o=Object.assign({},i):(o=[],i.forEach(e=>{o.push(e[parseInt(n)])}));else{const a=e[r].indexOf(t)-1;o="All"===n?i[a]:i[a][parseInt(n)]}return o},t.default=T.connect(e=>(e=>({computedResult:e.dashboardState.computedResult,recorderData:e.dashboardState.recorderData,driverData:e.dashboardState.driverData}))(e),e=>({}),null,{forwardRef:!0})(n.withStyles(e=>({textSizeSmall:{},toolbarHeigt:{minHeight:36,background:"rgb(50, 50, 50)"},viewSelector:{minWidth:120,color:"rgb(250, 250, 250)"},bgSelector:{minWidth:60,color:"rgb(250, 250, 250)"},textColor:{color:"rgb(250, 250, 250)"},backGround:{color:"rgb(50, 50, 50)"},formControl:{padding:e.spacing(1),width:"90%"},formControlShort:{padding:e.spacing(1),width:"calc(50% - 16px)"},formControlTiny:{width:"22%"},selectEmpty:{marginTop:e.spacing(2)},textField:{marginLeft:e.spacing(1),marginRight:e.spacing(1),width:"90%"},bottomBarDiv:{width:"100%",height:"35px",background:"#e0e0e0",display:"flex",flexDirection:"row-reverse"}}))(w.forwardRef((e,t)=>w.default.createElement(L,Object.assign({},e,{ref:t})))))}.apply(t,r))||(e.exports=i)},function(e,t,n){"use strict";var r=n(36),i=n(46);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(0)),a=(0,r(n(66)).default)(o.createElement("path",{d:"M12 2C6.48 2 2 6.48 2 12s4.48 10 10 10 10-4.48 10-10S17.52 2 12 2zm5 11h-4v4h-2v-4H7v-2h4V7h2v4h4v2z"}),"AddCircle");t.default=a},function(e,t,n){"use strict";var r=n(36),i=n(46);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(0)),a=(0,r(n(66)).default)(o.createElement("path",{d:"M2 20h20v-4H2v4zm2-3h2v2H4v-2zM2 4v4h20V4H2zm4 3H4V5h2v2zm-4 7h20v-4H2v4zm2-3h2v2H4v-2z"}),"Storage");t.default=a},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),Object.defineProperty(t,"DraggableCore",{enumerable:!0,get:function(){return c.default}}),t.default=void 0;var r=function(e){if(e&&e.__esModule)return e;if(null===e||"object"!==p(e)&&"function"!=typeof e)return{default:e};var t=f();if(t&&t.has(e))return t.get(e);var n={},r=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var i in e)if(Object.prototype.hasOwnProperty.call(e,i)){var o=r?Object.getOwnPropertyDescriptor(e,i):null;o&&(o.get||o.set)?Object.defineProperty(n,i,o):n[i]=e[i]}n.default=e,t&&t.set(e,n);return n}(n(0)),i=h(n(3)),o=h(n(22)),a=h(n(11)),s=n(521),l=n(962),u=n(380),c=h(n(1641)),d=h(n(963));function h(e){return e&&e.__esModule?e:{default:e}}function f(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return f=function(){return e},e}function p(e){return(p="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function g(){return(g=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e}).apply(this,arguments)}function v(e,t){if(null==e)return{};var n,r,i=function(e,t){if(null==e)return{};var n,r,i={},o=Object.keys(e);for(r=0;r<o.length;r++)n=o[r],t.indexOf(n)>=0||(i[n]=e[n]);return i}(e,t);if(Object.getOwnPropertySymbols){var o=Object.getOwnPropertySymbols(e);for(r=0;r<o.length;r++)n=o[r],t.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(e,n)&&(i[n]=e[n])}return i}function m(e,t){return function(e){if(Array.isArray(e))return e}(e)||function(e,t){if("undefined"==typeof Symbol||!(Symbol.iterator in Object(e)))return;var n=[],r=!0,i=!1,o=void 0;try{for(var a,s=e[Symbol.iterator]();!(r=(a=s.next()).done)&&(n.push(a.value),!t||n.length!==t);r=!0);}catch(e){i=!0,o=e}finally{try{r||null==s.return||s.return()}finally{if(i)throw o}}return n}(e,t)||function(e,t){if(!e)return;if("string"==typeof e)return y(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);"Object"===n&&e.constructor&&(n=e.constructor.name);if("Map"===n||"Set"===n)return Array.from(e);if("Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n))return y(e,t)}(e,t)||function(){throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}()}function y(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,r=new Array(t);n<t;n++)r[n]=e[n];return r}function b(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),n.push.apply(n,r)}return n}function C(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?b(Object(n),!0).forEach((function(t){T(e,t,n[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):b(Object(n)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))}))}return e}function A(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}function _(e,t,n){return t&&A(e.prototype,t),n&&A(e,n),e}function I(e,t){return(I=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function x(e){var t=function(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],(function(){}))),!0}catch(e){return!1}}();return function(){var n,r=S(e);if(t){var i=S(this).constructor;n=Reflect.construct(r,arguments,i)}else n=r.apply(this,arguments);return w(this,n)}}function w(e,t){return!t||"object"!==p(t)&&"function"!=typeof t?M(e):t}function M(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function S(e){return(S=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}function T(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}var E=function(e){!function(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&I(e,t)}(n,e);var t=x(n);function n(e){var r;return function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,n),T(M(r=t.call(this,e)),"onDragStart",(function(e,t){if((0,d.default)("Draggable: onDragStart: %j",t),!1===r.props.onStart(e,(0,l.createDraggableData)(M(r),t)))return!1;r.setState({dragging:!0,dragged:!0})})),T(M(r),"onDrag",(function(e,t){if(!r.state.dragging)return!1;(0,d.default)("Draggable: onDrag: %j",t);var n=(0,l.createDraggableData)(M(r),t),i={x:n.x,y:n.y};if(r.props.bounds){var o=i.x,a=i.y;i.x+=r.state.slackX,i.y+=r.state.slackY;var s=m((0,l.getBoundPosition)(M(r),i.x,i.y),2),u=s[0],c=s[1];i.x=u,i.y=c,i.slackX=r.state.slackX+(o-i.x),i.slackY=r.state.slackY+(a-i.y),n.x=i.x,n.y=i.y,n.deltaX=i.x-r.state.x,n.deltaY=i.y-r.state.y}if(!1===r.props.onDrag(e,n))return!1;r.setState(i)})),T(M(r),"onDragStop",(function(e,t){if(!r.state.dragging)return!1;if(!1===r.props.onStop(e,(0,l.createDraggableData)(M(r),t)))return!1;(0,d.default)("Draggable: onDragStop: %j",t);var n={dragging:!1,slackX:0,slackY:0};if(Boolean(r.props.position)){var i=r.props.position,o=i.x,a=i.y;n.x=o,n.y=a}r.setState(n)})),r.state={dragging:!1,dragged:!1,x:e.position?e.position.x:e.defaultPosition.x,y:e.position?e.position.y:e.defaultPosition.y,prevPropsPosition:C({},e.position),slackX:0,slackY:0,isElementSVG:!1},!e.position||e.onDrag||e.onStop||console.warn("A `position` was applied to this <Draggable>, without drag handlers. This will make this component effectively undraggable. Please attach `onDrag` or `onStop` handlers so you can adjust the `position` of this element."),r}return _(n,null,[{key:"getDerivedStateFromProps",value:function(e,t){var n=e.position,r=t.prevPropsPosition;return!n||r&&n.x===r.x&&n.y===r.y?null:((0,d.default)("Draggable: getDerivedStateFromProps %j",{position:n,prevPropsPosition:r}),{x:n.x,y:n.y,prevPropsPosition:C({},n)})}}]),_(n,[{key:"componentDidMount",value:function(){void 0!==window.SVGElement&&this.findDOMNode()instanceof window.SVGElement&&this.setState({isElementSVG:!0})}},{key:"componentWillUnmount",value:function(){this.setState({dragging:!1})}},{key:"findDOMNode",value:function(){return this.props.nodeRef?this.props.nodeRef.current:o.default.findDOMNode(this)}},{key:"render",value:function(){var e,t=this.props,n=(t.axis,t.bounds,t.children),i=t.defaultPosition,o=t.defaultClassName,u=t.defaultClassNameDragging,d=t.defaultClassNameDragged,h=t.position,f=t.positionOffset,p=(t.scale,v(t,["axis","bounds","children","defaultPosition","defaultClassName","defaultClassNameDragging","defaultClassNameDragged","position","positionOffset","scale"])),m={},y=null,b=!Boolean(h)||this.state.dragging,A=h||i,_={x:(0,l.canDragX)(this)&&b?this.state.x:A.x,y:(0,l.canDragY)(this)&&b?this.state.y:A.y};this.state.isElementSVG?y=(0,s.createSVGTransform)(_,f):m=(0,s.createCSSTransform)(_,f);var I=(0,a.default)(n.props.className||"",o,(T(e={},u,this.state.dragging),T(e,d,this.state.dragged),e));return r.createElement(c.default,g({},p,{onStart:this.onDragStart,onDrag:this.onDrag,onStop:this.onDragStop}),r.cloneElement(r.Children.only(n),{className:I,style:C(C({},n.props.style),m),transform:y}))}}]),n}(r.Component);t.default=E,T(E,"displayName","Draggable"),T(E,"propTypes",C(C({},c.default.propTypes),{},{axis:i.default.oneOf(["both","x","y","none"]),bounds:i.default.oneOfType([i.default.shape({left:i.default.number,right:i.default.number,top:i.default.number,bottom:i.default.number}),i.default.string,i.default.oneOf([!1])]),defaultClassName:i.default.string,defaultClassNameDragging:i.default.string,defaultClassNameDragged:i.default.string,defaultPosition:i.default.shape({x:i.default.number,y:i.default.number}),positionOffset:i.default.shape({x:i.default.oneOfType([i.default.number,i.default.string]),y:i.default.oneOfType([i.default.number,i.default.string])}),position:i.default.shape({x:i.default.number,y:i.default.number}),className:u.dontSetMe,style:u.dontSetMe,transform:u.dontSetMe})),T(E,"defaultProps",C(C({},c.default.defaultProps),{},{axis:"both",bounds:!1,defaultClassName:"react-draggable",defaultClassNameDragging:"react-draggable-dragging",defaultClassNameDragged:"react-draggable-dragged",defaultPosition:{x:0,y:0},position:null,scale:1}))},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.getPrefix=i,t.browserPrefixToKey=o,t.browserPrefixToStyle=function(e,t){return t?"-".concat(t.toLowerCase(),"-").concat(e):e},t.default=void 0;var r=["Moz","Webkit","O","ms"];function i(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:"transform";if("undefined"==typeof window||void 0===window.document)return"";var t=window.document.documentElement.style;if(e in t)return"";for(var n=0;n<r.length;n++)if(o(e,r[n])in t)return r[n];return""}function o(e,t){return t?"".concat(t).concat(function(e){for(var t="",n=!0,r=0;r<e.length;r++)n?(t+=e[r].toUpperCase(),n=!1):"-"===e[r]?n=!0:t+=e[r];return t}(e)):e}var a=i();t.default=a},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var r=function(e){if(e&&e.__esModule)return e;if(null===e||"object"!==h(e)&&"function"!=typeof e)return{default:e};var t=d();if(t&&t.has(e))return t.get(e);var n={},r=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var i in e)if(Object.prototype.hasOwnProperty.call(e,i)){var o=r?Object.getOwnPropertyDescriptor(e,i):null;o&&(o.get||o.set)?Object.defineProperty(n,i,o):n[i]=e[i]}n.default=e,t&&t.set(e,n);return n}(n(0)),i=c(n(3)),o=c(n(22)),a=n(521),s=n(962),l=n(380),u=c(n(963));function c(e){return e&&e.__esModule?e:{default:e}}function d(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return d=function(){return e},e}function h(e){return(h="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function f(e,t){return function(e){if(Array.isArray(e))return e}(e)||function(e,t){if("undefined"==typeof Symbol||!(Symbol.iterator in Object(e)))return;var n=[],r=!0,i=!1,o=void 0;try{for(var a,s=e[Symbol.iterator]();!(r=(a=s.next()).done)&&(n.push(a.value),!t||n.length!==t);r=!0);}catch(e){i=!0,o=e}finally{try{r||null==s.return||s.return()}finally{if(i)throw o}}return n}(e,t)||function(e,t){if(!e)return;if("string"==typeof e)return p(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);"Object"===n&&e.constructor&&(n=e.constructor.name);if("Map"===n||"Set"===n)return Array.from(e);if("Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n))return p(e,t)}(e,t)||function(){throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}()}function p(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,r=new Array(t);n<t;n++)r[n]=e[n];return r}function g(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function v(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}function m(e,t){return(m=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function y(e){var t=function(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],(function(){}))),!0}catch(e){return!1}}();return function(){var n,r=A(e);if(t){var i=A(this).constructor;n=Reflect.construct(r,arguments,i)}else n=r.apply(this,arguments);return b(this,n)}}function b(e,t){return!t||"object"!==h(t)&&"function"!=typeof t?C(e):t}function C(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function A(e){return(A=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}function _(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}var I={start:"touchstart",move:"touchmove",stop:"touchend"},x={start:"mousedown",move:"mousemove",stop:"mouseup"},w=x,M=function(e){!function(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&m(e,t)}(c,e);var t,n,i,l=y(c);function c(){var e;g(this,c);for(var t=arguments.length,n=new Array(t),r=0;r<t;r++)n[r]=arguments[r];return _(C(e=l.call.apply(l,[this].concat(n))),"state",{dragging:!1,lastX:NaN,lastY:NaN,touchIdentifier:null}),_(C(e),"mounted",!1),_(C(e),"handleDragStart",(function(t){if(e.props.onMouseDown(t),!e.props.allowAnyClick&&"number"==typeof t.button&&0!==t.button)return!1;var n=e.findDOMNode();if(!n||!n.ownerDocument||!n.ownerDocument.body)throw new Error("<DraggableCore> not mounted on DragStart!");var r=n.ownerDocument;if(!(e.props.disabled||!(t.target instanceof r.defaultView.Node)||e.props.handle&&!(0,a.matchesSelectorAndParentsTo)(t.target,e.props.handle,n)||e.props.cancel&&(0,a.matchesSelectorAndParentsTo)(t.target,e.props.cancel,n))){"touchstart"===t.type&&t.preventDefault();var i=(0,a.getTouchIdentifier)(t);e.setState({touchIdentifier:i});var o=(0,s.getControlPosition)(t,i,C(e));if(null!=o){var l=o.x,c=o.y,d=(0,s.createCoreData)(C(e),l,c);(0,u.default)("DraggableCore: handleDragStart: %j",d),(0,u.default)("calling",e.props.onStart),!1!==e.props.onStart(t,d)&&!1!==e.mounted&&(e.props.enableUserSelectHack&&(0,a.addUserSelectStyles)(r),e.setState({dragging:!0,lastX:l,lastY:c}),(0,a.addEvent)(r,w.move,e.handleDrag),(0,a.addEvent)(r,w.stop,e.handleDragStop))}}})),_(C(e),"handleDrag",(function(t){var n=(0,s.getControlPosition)(t,e.state.touchIdentifier,C(e));if(null!=n){var r=n.x,i=n.y;if(Array.isArray(e.props.grid)){var o=r-e.state.lastX,a=i-e.state.lastY,l=f((0,s.snapToGrid)(e.props.grid,o,a),2);if(o=l[0],a=l[1],!o&&!a)return;r=e.state.lastX+o,i=e.state.lastY+a}var c=(0,s.createCoreData)(C(e),r,i);if((0,u.default)("DraggableCore: handleDrag: %j",c),!1!==e.props.onDrag(t,c)&&!1!==e.mounted)e.setState({lastX:r,lastY:i});else try{e.handleDragStop(new MouseEvent("mouseup"))}catch(t){var d=document.createEvent("MouseEvents");d.initMouseEvent("mouseup",!0,!0,window,0,0,0,0,0,!1,!1,!1,!1,0,null),e.handleDragStop(d)}}})),_(C(e),"handleDragStop",(function(t){if(e.state.dragging){var n=(0,s.getControlPosition)(t,e.state.touchIdentifier,C(e));if(null!=n){var r=n.x,i=n.y,o=(0,s.createCoreData)(C(e),r,i);if(!1===e.props.onStop(t,o)||!1===e.mounted)return!1;var l=e.findDOMNode();l&&e.props.enableUserSelectHack&&(0,a.removeUserSelectStyles)(l.ownerDocument),(0,u.default)("DraggableCore: handleDragStop: %j",o),e.setState({dragging:!1,lastX:NaN,lastY:NaN}),l&&((0,u.default)("DraggableCore: Removing handlers"),(0,a.removeEvent)(l.ownerDocument,w.move,e.handleDrag),(0,a.removeEvent)(l.ownerDocument,w.stop,e.handleDragStop))}}})),_(C(e),"onMouseDown",(function(t){return w=x,e.handleDragStart(t)})),_(C(e),"onMouseUp",(function(t){return w=x,e.handleDragStop(t)})),_(C(e),"onTouchStart",(function(t){return w=I,e.handleDragStart(t)})),_(C(e),"onTouchEnd",(function(t){return w=I,e.handleDragStop(t)})),e}return t=c,(n=[{key:"componentDidMount",value:function(){this.mounted=!0;var e=this.findDOMNode();e&&(0,a.addEvent)(e,I.start,this.onTouchStart,{passive:!1})}},{key:"componentWillUnmount",value:function(){this.mounted=!1;var e=this.findDOMNode();if(e){var t=e.ownerDocument;(0,a.removeEvent)(t,x.move,this.handleDrag),(0,a.removeEvent)(t,I.move,this.handleDrag),(0,a.removeEvent)(t,x.stop,this.handleDragStop),(0,a.removeEvent)(t,I.stop,this.handleDragStop),(0,a.removeEvent)(e,I.start,this.onTouchStart,{passive:!1}),this.props.enableUserSelectHack&&(0,a.removeUserSelectStyles)(t)}}},{key:"findDOMNode",value:function(){return this.props.nodeRef?this.props.nodeRef.current:o.default.findDOMNode(this)}},{key:"render",value:function(){return r.cloneElement(r.Children.only(this.props.children),{onMouseDown:this.onMouseDown,onMouseUp:this.onMouseUp,onTouchEnd:this.onTouchEnd})}}])&&v(t.prototype,n),i&&v(t,i),c}(r.Component);t.default=M,_(M,"displayName","DraggableCore"),_(M,"propTypes",{allowAnyClick:i.default.bool,disabled:i.default.bool,enableUserSelectHack:i.default.bool,offsetParent:function(e,t){if(e[t]&&1!==e[t].nodeType)throw new Error("Draggable's offsetParent must be a DOM Node.")},grid:i.default.arrayOf(i.default.number),handle:i.default.string,cancel:i.default.string,nodeRef:i.default.object,onStart:i.default.func,onDrag:i.default.func,onStop:i.default.func,onMouseDown:i.default.func,scale:i.default.number,className:l.dontSetMe,style:l.dontSetMe,transform:l.dontSetMe}),_(M,"defaultProps",{allowAnyClick:!1,cancel:null,disabled:!1,enableUserSelectHack:!0,offsetParent:null,handle:null,grid:null,transform:null,onStart:function(){},onDrag:function(){},onStop:function(){},onMouseDown:function(){},scale:1})},function(e,t,n){var r,i,o=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};r=[n,t,n(1643),n(1785)],void 0===(i=function(e,t,r,i){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),r=o(r),i=o(i),r.default.register([n(1786),n(1071),n(1976),n(1987),n(2010),n(1071)]);const a=i.default(r.default);t.default=a}.apply(t,r))||(e.exports=i)},function(e,t,n){"use strict";e.exports=n(1644)},function(e,t,n){"use strict";t.version=n(964).version,n(1645).polyfill(),n(1646),n(1676)();for(var r=n(27),i=t.register=r.register,o=n(1677),a=Object.keys(o),s=0;s<a.length;s++){var l=a[s];"_"!==l.charAt(0)&&(t[l]=o[l]),i({moduleType:"apiMethod",name:l,fn:o[l]})}i(n(1719)),i([n(1737),n(179),n(1738),n(1744),n(1748),n(1752),n(1756),n(1760),n(1763),n(1768),n(1772),n(1773),n(195),n(1778)]),i([n(1781),n(1782)]),window.PlotlyLocales&&Array.isArray(window.PlotlyLocales)&&(i(window.PlotlyLocales),delete window.PlotlyLocales),t.Icons=n(550),t.Plots=n(82),t.Fx=n(179),t.Snapshot=n(1783),t.PlotSchema=n(297),t.Queue=n(982),t.d3=n(39)},function(e,t,n){(function(t,n){
/*!
 * @overview es6-promise - a tiny implementation of Promises/A+.
 * @copyright Copyright (c) 2014 Yehuda Katz, Tom Dale, Stefan Penner and contributors (Conversion to ES6 API by Jake Archibald)
 * @license   Licensed under MIT license
 *            See https://raw.githubusercontent.com/stefanpenner/es6-promise/master/LICENSE
 * @version   v4.2.8+1e68dce6
 */var r;r=function(){"use strict";function e(e){return"function"==typeof e}var r=Array.isArray?Array.isArray:function(e){return"[object Array]"===Object.prototype.toString.call(e)},i=0,o=void 0,a=void 0,s=function(e,t){p[i]=e,p[i+1]=t,2===(i+=2)&&(a?a(g):C())},l="undefined"!=typeof window?window:void 0,u=l||{},c=u.MutationObserver||u.WebKitMutationObserver,d="undefined"==typeof self&&void 0!==t&&"[object process]"==={}.toString.call(t),h="undefined"!=typeof Uint8ClampedArray&&"undefined"!=typeof importScripts&&"undefined"!=typeof MessageChannel;function f(){var e=setTimeout;return function(){return e(g,1)}}var p=new Array(1e3);function g(){for(var e=0;e<i;e+=2)(0,p[e])(p[e+1]),p[e]=void 0,p[e+1]=void 0;i=0}var v,m,y,b,C=void 0;function A(e,t){var n=this,r=new this.constructor(x);void 0===r[I]&&P(r);var i=n._state;if(i){var o=arguments[i-1];s((function(){return L(i,r,o,n._result)}))}else O(n,r,e,t);return r}function _(e){if(e&&"object"==typeof e&&e.constructor===this)return e;var t=new this(x);return M(t,e),t}d?C=function(){return t.nextTick(g)}:c?(m=0,y=new c(g),b=document.createTextNode(""),y.observe(b,{characterData:!0}),C=function(){b.data=m=++m%2}):h?((v=new MessageChannel).port1.onmessage=g,C=function(){return v.port2.postMessage(0)}):C=void 0===l?function(){try{var e=Function("return this")().require("vertx");return void 0!==(o=e.runOnLoop||e.runOnContext)?function(){o(g)}:f()}catch(e){return f()}}():f();var I=Math.random().toString(36).substring(2);function x(){}function w(t,n,r){n.constructor===t.constructor&&r===A&&n.constructor.resolve===_?function(e,t){1===t._state?T(e,t._result):2===t._state?E(e,t._result):O(t,void 0,(function(t){return M(e,t)}),(function(t){return E(e,t)}))}(t,n):void 0===r?T(t,n):e(r)?function(e,t,n){s((function(e){var r=!1,i=function(e,t,n,r){try{e.call(t,n,r)}catch(e){return e}}(n,t,(function(n){r||(r=!0,t!==n?M(e,n):T(e,n))}),(function(t){r||(r=!0,E(e,t))}),e._label);!r&&i&&(r=!0,E(e,i))}),e)}(t,n,r):T(t,n)}function M(e,t){if(e===t)E(e,new TypeError("You cannot resolve a promise with itself"));else if(i=typeof(r=t),null===r||"object"!==i&&"function"!==i)T(e,t);else{var n=void 0;try{n=t.then}catch(t){return void E(e,t)}w(e,t,n)}var r,i}function S(e){e._onerror&&e._onerror(e._result),k(e)}function T(e,t){void 0===e._state&&(e._result=t,e._state=1,0!==e._subscribers.length&&s(k,e))}function E(e,t){void 0===e._state&&(e._state=2,e._result=t,s(S,e))}function O(e,t,n,r){var i=e._subscribers,o=i.length;e._onerror=null,i[o]=t,i[o+1]=n,i[o+2]=r,0===o&&e._state&&s(k,e)}function k(e){var t=e._subscribers,n=e._state;if(0!==t.length){for(var r=void 0,i=void 0,o=e._result,a=0;a<t.length;a+=3)r=t[a],i=t[a+n],r?L(n,r,i,o):i(o);e._subscribers.length=0}}function L(t,n,r,i){var o=e(r),a=void 0,s=void 0,l=!0;if(o){try{a=r(i)}catch(e){l=!1,s=e}if(n===a)return void E(n,new TypeError("A promises callback cannot return that same promise."))}else a=i;void 0!==n._state||(o&&l?M(n,a):!1===l?E(n,s):1===t?T(n,a):2===t&&E(n,a))}var D=0;function P(e){e[I]=D++,e._state=void 0,e._result=void 0,e._subscribers=[]}var N=function(){function e(e,t){this._instanceConstructor=e,this.promise=new e(x),this.promise[I]||P(this.promise),r(t)?(this.length=t.length,this._remaining=t.length,this._result=new Array(this.length),0===this.length?T(this.promise,this._result):(this.length=this.length||0,this._enumerate(t),0===this._remaining&&T(this.promise,this._result))):E(this.promise,new Error("Array Methods must be provided an Array"))}return e.prototype._enumerate=function(e){for(var t=0;void 0===this._state&&t<e.length;t++)this._eachEntry(e[t],t)},e.prototype._eachEntry=function(e,t){var n=this._instanceConstructor,r=n.resolve;if(r===_){var i=void 0,o=void 0,a=!1;try{i=e.then}catch(e){a=!0,o=e}if(i===A&&void 0!==e._state)this._settledAt(e._state,t,e._result);else if("function"!=typeof i)this._remaining--,this._result[t]=e;else if(n===j){var s=new n(x);a?E(s,o):w(s,e,i),this._willSettleAt(s,t)}else this._willSettleAt(new n((function(t){return t(e)})),t)}else this._willSettleAt(r(e),t)},e.prototype._settledAt=function(e,t,n){var r=this.promise;void 0===r._state&&(this._remaining--,2===e?E(r,n):this._result[t]=n),0===this._remaining&&T(r,this._result)},e.prototype._willSettleAt=function(e,t){var n=this;O(e,void 0,(function(e){return n._settledAt(1,t,e)}),(function(e){return n._settledAt(2,t,e)}))},e}(),j=function(){function t(e){this[I]=D++,this._result=this._state=void 0,this._subscribers=[],x!==e&&("function"!=typeof e&&function(){throw new TypeError("You must pass a resolver function as the first argument to the promise constructor")}(),this instanceof t?function(e,t){try{t((function(t){M(e,t)}),(function(t){E(e,t)}))}catch(t){E(e,t)}}(this,e):function(){throw new TypeError("Failed to construct 'Promise': Please use the 'new' operator, this object constructor cannot be called as a function.")}())}return t.prototype.catch=function(e){return this.then(null,e)},t.prototype.finally=function(t){var n=this.constructor;return e(t)?this.then((function(e){return n.resolve(t()).then((function(){return e}))}),(function(e){return n.resolve(t()).then((function(){throw e}))})):this.then(t,t)},t}();return j.prototype.then=A,j.all=function(e){return new N(this,e).promise},j.race=function(e){var t=this;return r(e)?new t((function(n,r){for(var i=e.length,o=0;o<i;o++)t.resolve(e[o]).then(n,r)})):new t((function(e,t){return t(new TypeError("You must pass an array to race."))}))},j.resolve=_,j.reject=function(e){var t=new this(x);return E(t,e),t},j._setScheduler=function(e){a=e},j._setAsap=function(e){s=e},j._asap=s,j.polyfill=function(){var e=void 0;if(void 0!==n)e=n;else if("undefined"!=typeof self)e=self;else try{e=Function("return this")()}catch(e){throw new Error("polyfill failed because global object is unavailable in this environment")}var t=e.Promise;if(t){var r=null;try{r=Object.prototype.toString.call(t.resolve())}catch(e){}if("[object Promise]"===r&&!t.cast)return}e.Promise=j},j.Promise=j,j},e.exports=r()}).call(this,n(145),n(61))},function(e,t,n){"use strict";var r=n(10),i={"X,X div":"direction:ltr;font-family:'Open Sans', verdana, arial, sans-serif;margin:0;padding:0;","X input,X button":"font-family:'Open Sans', verdana, arial, sans-serif;","X input:focus,X button:focus":"outline:none;","X a":"text-decoration:none;","X a:hover":"text-decoration:none;","X .crisp":"shape-rendering:crispEdges;","X .user-select-none":"-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;-o-user-select:none;user-select:none;","X svg":"overflow:hidden;","X svg a":"fill:#447adb;","X svg a:hover":"fill:#3c6dc5;","X .main-svg":"position:absolute;top:0;left:0;pointer-events:none;","X .main-svg .draglayer":"pointer-events:all;","X .cursor-default":"cursor:default;","X .cursor-pointer":"cursor:pointer;","X .cursor-crosshair":"cursor:crosshair;","X .cursor-move":"cursor:move;","X .cursor-col-resize":"cursor:col-resize;","X .cursor-row-resize":"cursor:row-resize;","X .cursor-ns-resize":"cursor:ns-resize;","X .cursor-ew-resize":"cursor:ew-resize;","X .cursor-sw-resize":"cursor:sw-resize;","X .cursor-s-resize":"cursor:s-resize;","X .cursor-se-resize":"cursor:se-resize;","X .cursor-w-resize":"cursor:w-resize;","X .cursor-e-resize":"cursor:e-resize;","X .cursor-nw-resize":"cursor:nw-resize;","X .cursor-n-resize":"cursor:n-resize;","X .cursor-ne-resize":"cursor:ne-resize;","X .cursor-grab":"cursor:-webkit-grab;cursor:grab;","X .modebar":"position:absolute;top:2px;right:2px;","X .ease-bg":"-webkit-transition:background-color 0.3s ease 0s;-moz-transition:background-color 0.3s ease 0s;-ms-transition:background-color 0.3s ease 0s;-o-transition:background-color 0.3s ease 0s;transition:background-color 0.3s ease 0s;","X .modebar--hover>:not(.watermark)":"opacity:0;-webkit-transition:opacity 0.3s ease 0s;-moz-transition:opacity 0.3s ease 0s;-ms-transition:opacity 0.3s ease 0s;-o-transition:opacity 0.3s ease 0s;transition:opacity 0.3s ease 0s;","X:hover .modebar--hover .modebar-group":"opacity:1;","X .modebar-group":"float:left;display:inline-block;box-sizing:border-box;padding-left:8px;position:relative;vertical-align:middle;white-space:nowrap;","X .modebar-btn":"position:relative;font-size:16px;padding:3px 4px;height:22px;cursor:pointer;line-height:normal;box-sizing:border-box;","X .modebar-btn svg":"position:relative;top:2px;","X .modebar.vertical":"display:flex;flex-direction:column;flex-wrap:wrap;align-content:flex-end;max-height:100%;","X .modebar.vertical svg":"top:-1px;","X .modebar.vertical .modebar-group":"display:block;float:none;padding-left:0px;padding-bottom:8px;","X .modebar.vertical .modebar-group .modebar-btn":"display:block;text-align:center;","X [data-title]:before,X [data-title]:after":"position:absolute;-webkit-transform:translate3d(0, 0, 0);-moz-transform:translate3d(0, 0, 0);-ms-transform:translate3d(0, 0, 0);-o-transform:translate3d(0, 0, 0);transform:translate3d(0, 0, 0);display:none;opacity:0;z-index:1001;pointer-events:none;top:110%;right:50%;","X [data-title]:hover:before,X [data-title]:hover:after":"display:block;opacity:1;","X [data-title]:before":"content:'';position:absolute;background:transparent;border:6px solid transparent;z-index:1002;margin-top:-12px;border-bottom-color:#69738a;margin-right:-6px;","X [data-title]:after":"content:attr(data-title);background:#69738a;color:white;padding:8px 10px;font-size:12px;line-height:12px;white-space:nowrap;margin-right:-18px;border-radius:2px;","X .vertical [data-title]:before,X .vertical [data-title]:after":"top:0%;right:200%;","X .vertical [data-title]:before":"border:6px solid transparent;border-left-color:#69738a;margin-top:8px;margin-right:-30px;","X .select-outline":"fill:none;stroke-width:1;shape-rendering:crispEdges;","X .select-outline-1":"stroke:white;","X .select-outline-2":"stroke:black;stroke-dasharray:2px 2px;",Y:"font-family:'Open Sans', verdana, arial, sans-serif;position:fixed;top:50px;right:20px;z-index:10000;font-size:10pt;max-width:180px;","Y p":"margin:0;","Y .notifier-note":"min-width:180px;max-width:250px;border:1px solid #fff;z-index:3000;margin:0;background-color:#8c97af;background-color:rgba(140,151,175,0.9);color:#fff;padding:10px;overflow-wrap:break-word;word-wrap:break-word;-ms-hyphens:auto;-webkit-hyphens:auto;hyphens:auto;","Y .notifier-close":"color:#fff;opacity:0.8;float:right;padding:0 5px;background:none;border:none;font-size:20px;font-weight:bold;line-height:20px;","Y .notifier-close:hover":"color:#444;text-decoration:none;cursor:pointer;"};for(var o in i){var a=o.replace(/^,/," ,").replace(/X/g,".js-plotly-plot .plotly").replace(/Y/g,".plotly-notifier");r.addStyleRule(a,i[o])}},function(e,t,n){"use strict";var r=n(522),i=/^\w*$/;e.exports=function(e,t,n,o){var a,s,l;n=n||"name",o=o||"value";var u={};t&&t.length?(l=r(e,t),s=l.get()):s=e,t=t||"";var c={};if(s)for(a=0;a<s.length;a++)c[s[a][n]]=a;var d=i.test(o),h={set:function(e,t){var i=null===t?4:0;if(!s){if(!l||4===i)return;s=[],l.set(s)}var a=c[e];if(void 0===a){if(4===i)return;i|=3,a=s.length,c[e]=a}else t!==(d?s[a][o]:r(s[a],o).get())&&(i|=2);var f=s[a]=s[a]||{};return f[n]=e,d?f[o]=t:r(f,o).set(t),null!==t&&(i&=-5),u[a]=u[a]|i,h},get:function(e){if(s){var t=c[e];return void 0===t?void 0:d?s[t][o]:r(s[t],o).get()}},rename:function(e,t){var r=c[e];return void 0===r||(u[r]=1|u[r],c[t]=r,delete c[e],s[r][n]=t),h},remove:function(e){var t=c[e];if(void 0===t)return h;var i=s[t];if(Object.keys(i).length>2)return u[t]=2|u[t],h.set(e,null);if(d){for(a=t;a<s.length;a++)u[a]=3|u[a];for(a=t;a<s.length;a++)c[s[a][n]]--;s.splice(t,1),delete c[e]}else r(i,o).set(null),u[t]=6|u[t];return h},constructUpdate:function(){for(var e,i,a={},l=Object.keys(u),c=0;c<l.length;c++)i=l[c],e=t+"["+i+"]",s[i]?(1&u[i]&&(a[e+"."+n]=s[i][n]),2&u[i]&&(a[e+"."+o]=d?4&u[i]?null:s[i][o]:4&u[i]?null:r(s[i],o).get())):a[e]=null;return a}};return h}},function(e,t,n){"use strict";var r=/^(.*)(\.[^\.\[\]]+|\[\d\])$/,i=/^[^\.\[\]]+$/;e.exports=function(e,t){for(;t;){var n=e.match(r);if(n)e=n[1];else{if(!e.match(i))throw new Error("bad relativeAttr call:"+[e,t]);e=""}if("^"!==t.charAt(0))break;t=t.slice(1)}return e&&"["!==t.charAt(0)?e+"."+t:e+t}},function(e,t,n){"use strict";var r=n(249).isArrayOrTypedArray,i=n(292);e.exports=function e(t,n){for(var o in n){var a=n[o],s=t[o];if(s!==a)if("_"===o.charAt(0)||"function"==typeof a){if(o in t)continue;t[o]=a}else if(r(a)&&r(s)&&i(a[0])){if("customdata"===o||"ids"===o)continue;for(var l=Math.min(a.length,s.length),u=0;u<l;u++)s[u]!==a[u]&&i(a[u])&&i(s[u])&&e(s[u],a[u])}else i(a)&&i(s)&&(e(s,a),Object.keys(s).length||delete t[o])}}},function(e,t,n){"use strict";var r=n(42),i=n(123),o=n(160),a=n(251),s=n(525).DESELECTDIM,l=n(522),u=n(294).counter,c=n(293).modHalf,d=n(249).isArrayOrTypedArray;function h(e,n){var r=t.valObjectMeta[n.valType];if(n.arrayOk&&d(e))return!0;if(r.validateFunction)return r.validateFunction(e,n);var i={},o=i,a={set:function(e){o=e}};return r.coerceFunction(e,a,i,n),o!==i}t.valObjectMeta={data_array:{coerceFunction:function(e,t,n){d(e)?t.set(e):void 0!==n&&t.set(n)}},enumerated:{coerceFunction:function(e,t,n,r){r.coerceNumber&&(e=+e),-1===r.values.indexOf(e)?t.set(n):t.set(e)},validateFunction:function(e,t){t.coerceNumber&&(e=+e);for(var n=t.values,r=0;r<n.length;r++){var i=String(n[r]);if("/"===i.charAt(0)&&"/"===i.charAt(i.length-1)){if(new RegExp(i.substr(1,i.length-2)).test(e))return!0}else if(e===n[r])return!0}return!1}},boolean:{coerceFunction:function(e,t,n){!0===e||!1===e?t.set(e):t.set(n)}},number:{coerceFunction:function(e,t,n,i){!r(e)||void 0!==i.min&&e<i.min||void 0!==i.max&&e>i.max?t.set(n):t.set(+e)}},integer:{coerceFunction:function(e,t,n,i){e%1||!r(e)||void 0!==i.min&&e<i.min||void 0!==i.max&&e>i.max?t.set(n):t.set(+e)}},string:{coerceFunction:function(e,t,n,r){if("string"!=typeof e){var i="number"==typeof e;!0!==r.strict&&i?t.set(String(e)):t.set(n)}else r.noBlank&&!e?t.set(n):t.set(e)}},color:{coerceFunction:function(e,t,n){i(e).isValid()?t.set(e):t.set(n)}},colorlist:{coerceFunction:function(e,t,n){Array.isArray(e)&&e.length&&e.every((function(e){return i(e).isValid()}))?t.set(e):t.set(n)}},colorscale:{coerceFunction:function(e,t,n){t.set(a.get(e,n))}},angle:{coerceFunction:function(e,t,n){"auto"===e?t.set("auto"):r(e)?t.set(c(+e,360)):t.set(n)}},subplotid:{coerceFunction:function(e,t,n,r){var i=r.regex||u(n);"string"==typeof e&&i.test(e)?t.set(e):t.set(n)},validateFunction:function(e,t){var n=t.dflt;return e===n||"string"==typeof e&&!!u(n).test(e)}},flaglist:{coerceFunction:function(e,t,n,r){if("string"==typeof e)if(-1===(r.extras||[]).indexOf(e)){for(var i=e.split("+"),o=0;o<i.length;){var a=i[o];-1===r.flags.indexOf(a)||i.indexOf(a)<o?i.splice(o,1):o++}i.length?t.set(i.join("+")):t.set(n)}else t.set(e);else t.set(n)}},any:{coerceFunction:function(e,t,n){void 0===e?t.set(n):t.set(e)}},info_array:{coerceFunction:function(e,n,r,i){function o(e,n,r){var i,o={set:function(e){i=e}};return void 0===r&&(r=n.dflt),t.valObjectMeta[n.valType].coerceFunction(e,o,r,n),i}var a=2===i.dimensions||"1-2"===i.dimensions&&Array.isArray(e)&&Array.isArray(e[0]);if(Array.isArray(e)){var s,l,u,c,d,h,f=i.items,p=[],g=Array.isArray(f),v=g&&a&&Array.isArray(f[0]),m=a&&g&&!v,y=g&&!m?f.length:e.length;if(r=Array.isArray(r)?r:[],a)for(s=0;s<y;s++)for(p[s]=[],u=Array.isArray(e[s])?e[s]:[],d=m?f.length:g?f[s].length:u.length,l=0;l<d;l++)c=m?f[l]:g?f[s][l]:f,void 0!==(h=o(u[l],c,(r[s]||[])[l]))&&(p[s][l]=h);else for(s=0;s<y;s++)void 0!==(h=o(e[s],g?f[s]:f,r[s]))&&(p[s]=h);n.set(p)}else n.set(r)},validateFunction:function(e,t){if(!Array.isArray(e))return!1;var n=t.items,r=Array.isArray(n),i=2===t.dimensions;if(!t.freeLength&&e.length!==n.length)return!1;for(var o=0;o<e.length;o++)if(i){if(!Array.isArray(e[o])||!t.freeLength&&e[o].length!==n[o].length)return!1;for(var a=0;a<e[o].length;a++)if(!h(e[o][a],r?n[o][a]:n))return!1}else if(!h(e[o],r?n[o]:n))return!1;return!0}}},t.coerce=function(e,n,r,i,o){var a=l(r,i).get(),s=l(e,i),u=l(n,i),c=s.get(),f=n._template;if(void 0===c&&f&&(c=l(f,i).get(),f=0),void 0===o&&(o=a.dflt),a.arrayOk&&d(c))return u.set(c),c;var p=t.valObjectMeta[a.valType].coerceFunction;p(c,u,o,a);var g=u.get();return f&&g===o&&!h(c,a)&&(p(c=l(f,i).get(),u,o,a),g=u.get()),g},t.coerce2=function(e,n,r,i,o){var a=l(e,i),s=t.coerce(e,n,r,i,o),u=a.get();return null!=u&&s},t.coerceFont=function(e,t,n){var r={};return n=n||{},r.family=e(t+".family",n.family),r.size=e(t+".size",n.size),r.color=e(t+".color",n.color),r},t.coerceHoverinfo=function(e,n,r){var i,a=n._module.attributes,s=a.hoverinfo?a:o,l=s.hoverinfo;if(1===r._dataLength){var u="all"===l.dflt?l.flags.slice():l.dflt.split("+");u.splice(u.indexOf("name"),1),i=u.join("+")}return t.coerce(e,n,s,"hoverinfo",i)},t.coerceSelectionMarkerOpacity=function(e,t){if(e.marker){var n,r,i=e.marker.opacity;if(void 0!==i)d(i)||e.selected||e.unselected||(n=i,r=s*i),t("selected.marker.opacity",n),t("unselected.marker.opacity",r)}},t.validate=h},function(e,t,n){"use strict";var r,i,o=n(317).timeFormat,a=n(42),s=n(252),l=n(293).mod,u=n(72),c=u.BADNUM,d=u.ONEDAY,h=u.ONEHOUR,f=u.ONEMIN,p=u.ONESEC,g=u.EPOCHJD,v=n(27),m=n(317).utcFormat,y=/^\s*(-?\d\d\d\d|\d\d)(-(\d?\d)(-(\d?\d)([ Tt]([01]?\d|2[0-3])(:([0-5]\d)(:([0-5]\d(\.\d+)?))?(Z|z|[+\-]\d\d(:?\d\d)?)?)?)?)?)?\s*$/m,b=/^\s*(-?\d\d\d\d|\d\d)(-(\d?\di?)(-(\d?\d)([ Tt]([01]?\d|2[0-3])(:([0-5]\d)(:([0-5]\d(\.\d+)?))?(Z|z|[+\-]\d\d(:?\d\d)?)?)?)?)?)?\s*$/m,C=(new Date).getFullYear()-70;function A(e){return e&&v.componentsRegistry.calendars&&"string"==typeof e&&"gregorian"!==e}function _(e,t){return String(e+Math.pow(10,t)).substr(1)}t.dateTick0=function(e,n){var r=function(e,t){return A(e)?t?v.getComponentMethod("calendars","CANONICAL_SUNDAY")[e]:v.getComponentMethod("calendars","CANONICAL_TICK")[e]:t?"2000-01-02":"2000-01-01"}(e,!!n);if(n<2)return r;var i=t.dateTime2ms(r,e);return i+=d*(n-1),t.ms2DateTime(i,0,e)},t.dfltRange=function(e){return A(e)?v.getComponentMethod("calendars","DFLTRANGE")[e]:["2000-01-01","2001-01-01"]},t.isJSDate=function(e){return"object"==typeof e&&null!==e&&"function"==typeof e.getTime},t.dateTime2ms=function(e,n){if(t.isJSDate(e)){var o=e.getTimezoneOffset()*f,a=(e.getUTCMinutes()-e.getMinutes())*f+(e.getUTCSeconds()-e.getSeconds())*p+(e.getUTCMilliseconds()-e.getMilliseconds());if(a){var s=3*f;o=o-s/2+l(a-o+s/2,s)}return(e=Number(e)-o)>=r&&e<=i?e:c}if("string"!=typeof e&&"number"!=typeof e)return c;e=String(e);var u=A(n),m=e.charAt(0);!u||"G"!==m&&"g"!==m||(e=e.substr(1),n="");var _=u&&"chinese"===n.substr(0,7),I=e.match(_?b:y);if(!I)return c;var x=I[1],w=I[3]||"1",M=Number(I[5]||1),S=Number(I[7]||0),T=Number(I[9]||0),E=Number(I[11]||0);if(u){if(2===x.length)return c;var O;x=Number(x);try{var k=v.getComponentMethod("calendars","getCal")(n);if(_){var L="i"===w.charAt(w.length-1);w=parseInt(w,10),O=k.newDate(x,k.toMonthIndex(x,w,L),M)}else O=k.newDate(x,Number(w),M)}catch(e){return c}return O?(O.toJD()-g)*d+S*h+T*f+E*p:c}x=2===x.length?(Number(x)+2e3-C)%100+C:Number(x),w-=1;var D=new Date(Date.UTC(2e3,w,M,S,T));return D.setUTCFullYear(x),D.getUTCMonth()!==w||D.getUTCDate()!==M?c:D.getTime()+E*p},r=t.MIN_MS=t.dateTime2ms("-9999"),i=t.MAX_MS=t.dateTime2ms("9999-12-31 23:59:59.9999"),t.isDateTime=function(e,n){return t.dateTime2ms(e,n)!==c};var I=90*d,x=3*h,w=5*f;function M(e,t,n,r,i){if((t||n||r||i)&&(e+=" "+_(t,2)+":"+_(n,2),(r||i)&&(e+=":"+_(r,2),i))){for(var o=4;i%10==0;)o-=1,i/=10;e+="."+_(i,o)}return e}t.ms2DateTime=function(e,t,n){if("number"!=typeof e||!(e>=r&&e<=i))return c;t||(t=0);var o,a,s,u,y,b,C=Math.floor(10*l(e+.05,1)),_=Math.round(e-C/10);if(A(n)){var S=Math.floor(_/d)+g,T=Math.floor(l(e,d));try{o=v.getComponentMethod("calendars","getCal")(n).fromJD(S).formatDate("yyyy-mm-dd")}catch(e){o=m("G%Y-%m-%d")(new Date(_))}if("-"===o.charAt(0))for(;o.length<11;)o="-0"+o.substr(1);else for(;o.length<10;)o="0"+o;a=t<I?Math.floor(T/h):0,s=t<I?Math.floor(T%h/f):0,u=t<x?Math.floor(T%f/p):0,y=t<w?T%p*10+C:0}else b=new Date(_),o=m("%Y-%m-%d")(b),a=t<I?b.getUTCHours():0,s=t<I?b.getUTCMinutes():0,u=t<x?b.getUTCSeconds():0,y=t<w?10*b.getUTCMilliseconds()+C:0;return M(o,a,s,u,y)},t.ms2DateTimeLocal=function(e){if(!(e>=r+d&&e<=i-d))return c;var t=Math.floor(10*l(e+.05,1)),n=new Date(Math.round(e-t/10));return M(o("%Y-%m-%d")(n),n.getHours(),n.getMinutes(),n.getSeconds(),10*n.getUTCMilliseconds()+t)},t.cleanDate=function(e,n,r){if(e===c)return n;if(t.isJSDate(e)||"number"==typeof e&&isFinite(e)){if(A(r))return s.error("JS Dates and milliseconds are incompatible with world calendars",e),n;if(!(e=t.ms2DateTimeLocal(+e))&&void 0!==n)return n}else if(!t.isDateTime(e,r))return s.error("unrecognized date",e),n;return e};var S=/%\d?f/g;function T(e,t,n,r){e=e.replace(S,(function(e){var n=Math.min(+e.charAt(1)||6,6);return(t/1e3%1+2).toFixed(n).substr(2).replace(/0+$/,"")||"0"}));var i=new Date(Math.floor(t+.05));if(A(r))try{e=v.getComponentMethod("calendars","worldCalFmt")(e,t,r)}catch(e){return"Invalid"}return n(e)(i)}var E=[59,59.9,59.99,59.999,59.9999];t.formatDate=function(e,t,n,r,i,o){if(i=A(i)&&i,!t)if("y"===n)t=o.year;else if("m"===n)t=o.month;else{if("d"!==n)return function(e,t){var n=l(e+.05,d),r=_(Math.floor(n/h),2)+":"+_(l(Math.floor(n/f),60),2);if("M"!==t){a(t)||(t=0);var i=(100+Math.min(l(e/p,60),E[t])).toFixed(t).substr(1);t>0&&(i=i.replace(/0+$/,"").replace(/[\.]$/,"")),r+=":"+i}return r}(e,n)+"\n"+T(o.dayMonthYear,e,r,i);t=o.dayMonth+"\n"+o.year}return T(t,e,r,i)};var O=3*d;t.incrementMonth=function(e,t,n){n=A(n)&&n;var r=l(e,d);if(e=Math.round(e-r),n)try{var i=Math.round(e/d)+g,o=v.getComponentMethod("calendars","getCal")(n),a=o.fromJD(i);return t%12?o.add(a,t,"m"):o.add(a,t/12,"y"),(a.toJD()-g)*d+r}catch(t){s.error("invalid ms "+e+" in calendar "+n)}var u=new Date(e+O);return u.setUTCMonth(u.getUTCMonth()+t)+r-O},t.findExactDates=function(e,t){for(var n,r,i=0,o=0,s=0,l=0,u=A(t)&&v.getComponentMethod("calendars","getCal")(t),c=0;c<e.length;c++)if(r=e[c],a(r)){if(!(r%d))if(u)try{1===(n=u.fromJD(r/d+g)).day()?1===n.month()?i++:o++:s++}catch(e){}else 1===(n=new Date(r)).getUTCDate()?0===n.getUTCMonth()?i++:o++:s++}else l++;s+=o+=i;var h=e.length-l;return{exactYears:i/h,exactMonths:o/h,exactDays:s/h}}},function(e,t){e.exports=function(e,t){return e[0]=t[0],e[1]=t[1],e[2]=t[2],e[3]=t[3],e[4]=t[4],e[5]=t[5],e[6]=t[6],e[7]=t[7],e[8]=t[8],e[9]=t[9],e[10]=t[10],e[11]=t[11],e[12]=t[12],e[13]=t[13],e[14]=t[14],e[15]=t[15],e}},function(e,t){e.exports=function(e,t){var n=t[0],r=t[1],i=t[2],o=t[3],a=t[4],s=t[5],l=t[6],u=t[7],c=t[8],d=t[9],h=t[10],f=t[11],p=t[12],g=t[13],v=t[14],m=t[15];return e[0]=s*(h*m-f*v)-d*(l*m-u*v)+g*(l*f-u*h),e[1]=-(r*(h*m-f*v)-d*(i*m-o*v)+g*(i*f-o*h)),e[2]=r*(l*m-u*v)-s*(i*m-o*v)+g*(i*u-o*l),e[3]=-(r*(l*f-u*h)-s*(i*f-o*h)+d*(i*u-o*l)),e[4]=-(a*(h*m-f*v)-c*(l*m-u*v)+p*(l*f-u*h)),e[5]=n*(h*m-f*v)-c*(i*m-o*v)+p*(i*f-o*h),e[6]=-(n*(l*m-u*v)-a*(i*m-o*v)+p*(i*u-o*l)),e[7]=n*(l*f-u*h)-a*(i*f-o*h)+c*(i*u-o*l),e[8]=a*(d*m-f*g)-c*(s*m-u*g)+p*(s*f-u*d),e[9]=-(n*(d*m-f*g)-c*(r*m-o*g)+p*(r*f-o*d)),e[10]=n*(s*m-u*g)-a*(r*m-o*g)+p*(r*u-o*s),e[11]=-(n*(s*f-u*d)-a*(r*f-o*d)+c*(r*u-o*s)),e[12]=-(a*(d*v-h*g)-c*(s*v-l*g)+p*(s*h-l*d)),e[13]=n*(d*v-h*g)-c*(r*v-i*g)+p*(r*h-i*d),e[14]=-(n*(s*v-l*g)-a*(r*v-i*g)+p*(r*l-i*s)),e[15]=n*(s*h-l*d)-a*(r*h-i*d)+c*(r*l-i*s),e}},function(e,t){e.exports=function(e,t,n){var r,i,o,a=n[0],s=n[1],l=n[2],u=Math.sqrt(a*a+s*s+l*l);if(Math.abs(u)<1e-6)return null;return a*=u=1/u,s*=u,l*=u,r=Math.sin(t),i=Math.cos(t),o=1-i,e[0]=a*a*o+i,e[1]=s*a*o+l*r,e[2]=l*a*o-s*r,e[3]=0,e[4]=a*s*o-l*r,e[5]=s*s*o+i,e[6]=l*s*o+a*r,e[7]=0,e[8]=a*l*o+s*r,e[9]=s*l*o-a*r,e[10]=l*l*o+i,e[11]=0,e[12]=0,e[13]=0,e[14]=0,e[15]=1,e}},function(e,t){e.exports=function(e,t){return e[0]=t[0],e[1]=0,e[2]=0,e[3]=0,e[4]=0,e[5]=t[1],e[6]=0,e[7]=0,e[8]=0,e[9]=0,e[10]=t[2],e[11]=0,e[12]=0,e[13]=0,e[14]=0,e[15]=1,e}},function(e,t){e.exports=function(e,t){return e[0]=1,e[1]=0,e[2]=0,e[3]=0,e[4]=0,e[5]=1,e[6]=0,e[7]=0,e[8]=0,e[9]=0,e[10]=1,e[11]=0,e[12]=t[0],e[13]=t[1],e[14]=t[2],e[15]=1,e}},function(e,t){e.exports=function(e,t){var n=Math.sin(t),r=Math.cos(t);return e[0]=1,e[1]=0,e[2]=0,e[3]=0,e[4]=0,e[5]=r,e[6]=n,e[7]=0,e[8]=0,e[9]=-n,e[10]=r,e[11]=0,e[12]=0,e[13]=0,e[14]=0,e[15]=1,e}},function(e,t){e.exports=function(e,t){var n=Math.sin(t),r=Math.cos(t);return e[0]=r,e[1]=0,e[2]=-n,e[3]=0,e[4]=0,e[5]=1,e[6]=0,e[7]=0,e[8]=n,e[9]=0,e[10]=r,e[11]=0,e[12]=0,e[13]=0,e[14]=0,e[15]=1,e}},function(e,t){e.exports=function(e,t){var n=Math.sin(t),r=Math.cos(t);return e[0]=r,e[1]=n,e[2]=0,e[3]=0,e[4]=-n,e[5]=r,e[6]=0,e[7]=0,e[8]=0,e[9]=0,e[10]=1,e[11]=0,e[12]=0,e[13]=0,e[14]=0,e[15]=1,e}},function(e,t){e.exports=function(e,t,n,r,i,o,a){var s=1/(n-t),l=1/(i-r),u=1/(o-a);return e[0]=2*o*s,e[1]=0,e[2]=0,e[3]=0,e[4]=0,e[5]=2*o*l,e[6]=0,e[7]=0,e[8]=(n+t)*s,e[9]=(i+r)*l,e[10]=(a+o)*u,e[11]=-1,e[12]=0,e[13]=0,e[14]=a*o*2*u,e[15]=0,e}},function(e,t){e.exports=function(e,t,n,r){var i=Math.tan(t.upDegrees*Math.PI/180),o=Math.tan(t.downDegrees*Math.PI/180),a=Math.tan(t.leftDegrees*Math.PI/180),s=Math.tan(t.rightDegrees*Math.PI/180),l=2/(a+s),u=2/(i+o);return e[0]=l,e[1]=0,e[2]=0,e[3]=0,e[4]=0,e[5]=u,e[6]=0,e[7]=0,e[8]=-(a-s)*l*.5,e[9]=(i-o)*u*.5,e[10]=r/(n-r),e[11]=-1,e[12]=0,e[13]=0,e[14]=r*n/(n-r),e[15]=0,e}},function(e,t){e.exports=function(e){return"mat4("+e[0]+", "+e[1]+", "+e[2]+", "+e[3]+", "+e[4]+", "+e[5]+", "+e[6]+", "+e[7]+", "+e[8]+", "+e[9]+", "+e[10]+", "+e[11]+", "+e[12]+", "+e[13]+", "+e[14]+", "+e[15]+")"}},function(e,t,n){"use strict";var r=n(296).dash,i=n(63).extendFlat;e.exports={newshape:{line:{color:{valType:"color",editType:"none"},width:{valType:"number",min:0,dflt:4,editType:"none"},dash:i({},r,{dflt:"solid",editType:"none"}),editType:"none"},fillcolor:{valType:"color",dflt:"rgba(0,0,0,0)",editType:"none"},fillrule:{valType:"enumerated",values:["evenodd","nonzero"],dflt:"evenodd",editType:"none"},opacity:{valType:"number",min:0,max:1,dflt:1,editType:"none"},layer:{valType:"enumerated",values:["below","above"],dflt:"above",editType:"none"},drawdirection:{valType:"enumerated",values:["ortho","horizontal","vertical","diagonal"],dflt:"diagonal",editType:"none"},editType:"none"},activeshape:{fillcolor:{valType:"color",dflt:"rgb(255,0,255)",editType:"none"},opacity:{valType:"number",min:0,max:1,dflt:.5,editType:"none"},editType:"none"}}},function(e,t,n){"use strict";var r=n(42),i=n(249).isArrayOrTypedArray;t.aggNums=function(e,n,o,a){var s,l;if((!a||a>o.length)&&(a=o.length),r(n)||(n=!1),i(o[0])){for(l=new Array(a),s=0;s<a;s++)l[s]=t.aggNums(e,n,o[s]);o=l}for(s=0;s<a;s++)r(n)?r(o[s])&&(n=e(+n,+o[s])):n=o[s];return n},t.len=function(e){return t.aggNums((function(e){return e+1}),0,e)},t.mean=function(e,n){return n||(n=t.len(e)),t.aggNums((function(e,t){return e+t}),0,e)/n},t.midRange=function(e){if(void 0!==e&&0!==e.length)return(t.aggNums(Math.max,null,e)+t.aggNums(Math.min,null,e))/2},t.variance=function(e,n,i){return n||(n=t.len(e)),r(i)||(i=t.mean(e,n)),t.aggNums((function(e,t){return e+Math.pow(t-i,2)}),0,e)/n},t.stdev=function(e,n,r){return Math.sqrt(t.variance(e,n,r))},t.median=function(e){var n=e.slice().sort();return t.interp(n,.5)},t.interp=function(e,t){if(!r(t))throw"n should be a finite number";if((t=t*e.length-.5)<0)return e[0];if(t>e.length-1)return e[e.length-1];var n=t%1;return n*e[Math.ceil(t)]+(1-n)*e[Math.floor(t)]}},function(e,t,n){"use strict";var r=n(293),i=r.mod,o=r.modHalf,a=Math.PI,s=2*a;function l(e){return Math.abs(e[1]-e[0])>s-1e-14}function u(e,t){return o(t-e,s)}function c(e,t){if(l(t))return!0;var n,r;t[0]<t[1]?(n=t[0],r=t[1]):(n=t[1],r=t[0]),(n=i(n,s))>(r=i(r,s))&&(r+=s);var o=i(e,s),a=o+s;return o>=n&&o<=r||a>=n&&a<=r}function d(e,t,n,r,i,o,u){i=i||0,o=o||0;var c,d,h,f,p,g=l([n,r]);function v(e,t){return[e*Math.cos(t)+i,o-e*Math.sin(t)]}g?(c=0,d=a,h=s):n<r?(c=n,h=r):(c=r,h=n),e<t?(f=e,p=t):(f=t,p=e);var m,y=Math.abs(h-c)<=a?0:1;function b(e,t,n){return"A"+[e,e]+" "+[0,y,n]+" "+v(e,t)}return g?m=null===f?"M"+v(p,c)+b(p,d,0)+b(p,h,0)+"Z":"M"+v(f,c)+b(f,d,0)+b(f,h,0)+"ZM"+v(p,c)+b(p,d,1)+b(p,h,1)+"Z":null===f?(m="M"+v(p,c)+b(p,h,0),u&&(m+="L0,0Z")):m="M"+v(f,c)+"L"+v(p,c)+b(p,h,0)+"L"+v(f,h)+b(f,c,1)+"Z",m}e.exports={deg2rad:function(e){return e/180*a},rad2deg:function(e){return e/a*180},angleDelta:u,angleDist:function(e,t){return Math.abs(u(e,t))},isFullCircle:l,isAngleInsideSector:c,isPtInsideSector:function(e,t,n,r){return!!c(t,r)&&(n[0]<n[1]?(i=n[0],o=n[1]):(i=n[1],o=n[0]),e>=i&&e<=o);var i,o},pathArc:function(e,t,n,r,i){return d(null,e,t,n,r,i,0)},pathSector:function(e,t,n,r,i){return d(null,e,t,n,r,i,1)},pathAnnulus:function(e,t,n,r,i,o){return d(e,t,n,r,i,o,1)}}},function(e,t,n){"use strict";t.isLeftAnchor=function(e){return"left"===e.xanchor||"auto"===e.xanchor&&e.x<=1/3},t.isCenterAnchor=function(e){return"center"===e.xanchor||"auto"===e.xanchor&&e.x>1/3&&e.x<2/3},t.isRightAnchor=function(e){return"right"===e.xanchor||"auto"===e.xanchor&&e.x>=2/3},t.isTopAnchor=function(e){return"top"===e.yanchor||"auto"===e.yanchor&&e.y>=2/3},t.isMiddleAnchor=function(e){return"middle"===e.yanchor||"auto"===e.yanchor&&e.y>1/3&&e.y<2/3},t.isBottomAnchor=function(e){return"bottom"===e.yanchor||"auto"===e.yanchor&&e.y<=1/3}},function(e,t,n){"use strict";var r,i,o,a=n(293).mod;function s(e,t,n,r,i,o,a,s){var l=n-e,u=i-e,c=a-i,d=r-t,h=o-t,f=s-o,p=l*f-c*d;if(0===p)return null;var g=(u*f-c*h)/p,v=(u*d-l*h)/p;return v<0||v>1||g<0||g>1?null:{x:e+l*g,y:t+d*g}}function l(e,t,n,r,i){var o=r*e+i*t;if(o<0)return r*r+i*i;if(o>n){var a=r-e,s=i-t;return a*a+s*s}var l=r*t-i*e;return l*l/n}t.segmentsIntersect=s,t.segmentDistance=function(e,t,n,r,i,o,a,u){if(s(e,t,n,r,i,o,a,u))return 0;var c=n-e,d=r-t,h=a-i,f=u-o,p=c*c+d*d,g=h*h+f*f,v=Math.min(l(c,d,p,i-e,o-t),l(c,d,p,a-e,u-t),l(h,f,g,e-i,t-o),l(h,f,g,n-i,r-o));return Math.sqrt(v)},t.getTextLocation=function(e,t,n,s){if(e===i&&s===o||(r={},i=e,o=s),r[n])return r[n];var l=e.getPointAtLength(a(n-s/2,t)),u=e.getPointAtLength(a(n+s/2,t)),c=Math.atan((u.y-l.y)/(u.x-l.x)),d=e.getPointAtLength(a(n,t)),h={x:(4*d.x+l.x+u.x)/6,y:(4*d.y+l.y+u.y)/6,theta:c};return r[n]=h,h},t.clearLocationCache=function(){i=null},t.getVisibleSegment=function(e,t,n){var r,i,o=t.left,a=t.right,s=t.top,l=t.bottom,u=0,c=e.getTotalLength(),d=c;function h(t){var n=e.getPointAtLength(t);0===t?r=n:t===c&&(i=n);var u=n.x<o?o-n.x:n.x>a?n.x-a:0,d=n.y<s?s-n.y:n.y>l?n.y-l:0;return Math.sqrt(u*u+d*d)}for(var f=h(u);f;){if((u+=f+n)>d)return;f=h(u)}for(f=h(d);f;){if(u>(d-=f+n))return;f=h(d)}return{min:u,max:d,len:d-u,total:c,isClosed:0===u&&d===c&&Math.abs(r.x-i.x)<.1&&Math.abs(r.y-i.y)<.1}},t.findPointOnPath=function(e,t,n,r){for(var i,o,a,s=(r=r||{}).pathLength||e.getTotalLength(),l=r.tolerance||.001,u=r.iterationLimit||30,c=e.getPointAtLength(0)[n]>e.getPointAtLength(s)[n]?-1:1,d=0,h=0,f=s;d<u;){if(i=(h+f)/2,a=(o=e.getPointAtLength(i))[n]-t,Math.abs(a)<l)return o;c*a>0?f=i:h=i,d++}return o}},function(e,t,n){"use strict";e.exports=function(e){e._responsiveChartHandler&&(window.removeEventListener("resize",e._responsiveChartHandler),delete e._responsiveChartHandler)}},function(e,t,n){"use strict";var r=n(42),i=n(978);e.exports=function(e){var t;if("string"!=typeof(t=e&&e.hasOwnProperty("userAgent")?e.userAgent:function(){var e;"undefined"!=typeof navigator&&(e=navigator.userAgent);e&&e.headers&&"string"==typeof e.headers["user-agent"]&&(e=e.headers["user-agent"]);return e}()))return!0;var n=i({ua:{headers:{"user-agent":t}},tablet:!0,featureDetect:!1});if(!n)for(var o=t.split(" "),a=1;a<o.length;a++){if(-1!==o[a].indexOf("Safari"))for(var s=a-1;s>-1;s--){var l=o[s];if("Version/"===l.substr(0,8)){var u=l.substr(8).split(".")[0];if(r(u)&&(u=+u),u>=13)return!0}}}return n}},function(e,t,n){"use strict";var r=n(39);e.exports=function(e,t,n){var i=e.selectAll("g."+n.replace(/\s/g,".")).data(t,(function(e){return e[0].trace.uid}));i.exit().remove(),i.enter().append("g").attr("class",n),i.order();var o=e.classed("rangeplot")?"nodeRangePlot3":"node3";return i.each((function(e){e[0][o]=r.select(this)})),i}},function(e,t,n){"use strict";var r=n(27);e.exports=function(e,t){for(var n=e._context.locale,i=0;i<2;i++){for(var o=e._context.locales,a=0;a<2;a++){var s=(o[n]||{}).dictionary;if(s){var l=s[t];if(l)return l}o=r.localeRegistry}var u=n.split("-")[0];if(u===n)break;n=u}return t}},function(e,t,n){"use strict";e.exports=function(e){for(var t={},n=[],r=0,i=0;i<e.length;i++){var o=e[i];1!==t[o]&&(t[o]=1,n[r++]=o)}return n}},function(e,t,n){"use strict";function r(e){return!0===e.visible}function i(e){var t=e[0].trace;return!0===t.visible&&0!==t._length}e.exports=function(e){for(var t,n=(t=e,Array.isArray(t)&&Array.isArray(t[0])&&t[0][0]&&t[0][0].trace?i:r),o=[],a=0;a<e.length;a++){var s=e[a];n(s)&&o.push(s)}return o}},function(e,t,n){"use strict";e.exports=function(e,t){if(!t)return e;var n=1/Math.abs(t),r=n>1?(n*e+n*t)/n:e+t,i=String(r).length;if(i>16){var o=String(t).length;if(i>=String(e).length+o){var a=parseFloat(r).toPrecision(12);-1===a.indexOf("e+")&&(r=+a)}}return r}},function(e,t,n){"use strict";var r=n(42),i=n(72).BADNUM,o=/^['"%,$#\s']+|[, ]|['"%,$#\s']+$/g;e.exports=function(e){return"string"==typeof e&&(e=e.replace(o,"")),r(e)?Number(e):i}},function(e,t,n){"use strict";e.exports=function(){"undefined"!=typeof MathJax&&("local"!==(window.PlotlyConfig||{}).MathJaxConfig&&(MathJax.Hub.Config({messageStyle:"none",skipStartupTypeset:!0,displayAlign:"left",tex2jax:{inlineMath:[["$","$"],["\\(","\\)"]]}}),MathJax.Hub.Configured()))}},function(e,t,n){"use strict";var r=n(979);t.plot=r.plot,t.newPlot=r.newPlot,t.restyle=r.restyle,t.relayout=r.relayout,t.redraw=r.redraw,t.update=r.update,t._guiRestyle=r._guiRestyle,t._guiRelayout=r._guiRelayout,t._guiUpdate=r._guiUpdate,t._storeDirectGUIEdit=r._storeDirectGUIEdit,t.react=r.react,t.extendTraces=r.extendTraces,t.prependTraces=r.prependTraces,t.addTraces=r.addTraces,t.deleteTraces=r.deleteTraces,t.moveTraces=r.moveTraces,t.purge=r.purge,t.addFrames=r.addFrames,t.deleteFrames=r.deleteFrames,t.animate=r.animate,t.setPlotConfig=r.setPlotConfig,t.toImage=n(1006),t.validate=n(1716),t.downloadImage=n(1007);var i=n(1718);t.makeTemplate=i.makeTemplate,t.validateTemplate=i.validateTemplate},function(e,t,n){"use strict";var r=n(193),i=r.marker,o=n(63).extendFlat;["Area traces are deprecated!","Please switch to the *barpolar* trace type."].join(" ");e.exports={r:o({},r.r,{}),t:o({},r.t,{}),marker:{color:o({},i.color,{}),size:o({},i.size,{}),symbol:o({},i.symbol,{}),opacity:o({},i.opacity,{}),editType:"calc"}}},function(e,t,n){"use strict";var r=n(10),i=n(64),o=n(984),a=n(213);e.exports=function(e,t){function n(n,i){return r.coerce(e,t,o,n,i)}n("colorscale.sequential"),n("colorscale.sequentialminus"),n("colorscale.diverging");var s,l,u=t._colorAxes;function c(e,t){return r.coerce(s,l,o.coloraxis,e,t)}for(var d in u){var h=u[d];if(h[0])s=e[d]||{},(l=i.newContainer(t,d,"coloraxis"))._name=d,a(s,l,t,c,{prefix:"",cLetter:"c"});else{for(var f=0;f<h[2].length;f++)h[2][f]();delete t._colorAxes[d]}}}},function(e,t,n){"use strict";var r=n(10),i=n(149).hasColorscale,o=n(149).extractOpts;e.exports=function(e,t){function n(e,t){var n=e["_"+t];void 0!==n&&(e[t]=n)}function a(e,i){var a=i.container?r.nestedProperty(e,i.container).get():e;if(a)if(a.coloraxis)a._colorAx=t[a.coloraxis];else{var s=o(a),l=s.auto;(l||void 0===s.min)&&n(a,i.min),(l||void 0===s.max)&&n(a,i.max),s.autocolorscale&&n(a,"colorscale")}}for(var s=0;s<e.length;s++){var l=e[s],u=l._module.colorbar;if(u)if(Array.isArray(u))for(var c=0;c<u.length;c++)a(l,u[c]);else a(l,u);i(l,"marker.line")&&a(l,{container:"marker.line",min:"cmin",max:"cmax"})}for(var d in t._colorAxes)a(t[d],{min:"cmin",max:"cmax"})}},function(e,t,n){"use strict";var r=n(39);e.exports={circle:{n:0,f:function(e){var t=r.round(e,2);return"M"+t+",0A"+t+","+t+" 0 1,1 0,-"+t+"A"+t+","+t+" 0 0,1 "+t+",0Z"}},square:{n:1,f:function(e){var t=r.round(e,2);return"M"+t+","+t+"H-"+t+"V-"+t+"H"+t+"Z"}},diamond:{n:2,f:function(e){var t=r.round(1.3*e,2);return"M"+t+",0L0,"+t+"L-"+t+",0L0,-"+t+"Z"}},cross:{n:3,f:function(e){var t=r.round(.4*e,2),n=r.round(1.2*e,2);return"M"+n+","+t+"H"+t+"V"+n+"H-"+t+"V"+t+"H-"+n+"V-"+t+"H-"+t+"V-"+n+"H"+t+"V-"+t+"H"+n+"Z"}},x:{n:4,f:function(e){var t=r.round(.8*e/Math.sqrt(2),2),n="l"+t+","+t,i="l"+t+",-"+t,o="l-"+t+",-"+t,a="l-"+t+","+t;return"M0,"+t+n+i+o+i+o+a+o+a+n+a+n+"Z"}},"triangle-up":{n:5,f:function(e){var t=r.round(2*e/Math.sqrt(3),2);return"M-"+t+","+r.round(e/2,2)+"H"+t+"L0,-"+r.round(e,2)+"Z"}},"triangle-down":{n:6,f:function(e){var t=r.round(2*e/Math.sqrt(3),2);return"M-"+t+",-"+r.round(e/2,2)+"H"+t+"L0,"+r.round(e,2)+"Z"}},"triangle-left":{n:7,f:function(e){var t=r.round(2*e/Math.sqrt(3),2);return"M"+r.round(e/2,2)+",-"+t+"V"+t+"L-"+r.round(e,2)+",0Z"}},"triangle-right":{n:8,f:function(e){var t=r.round(2*e/Math.sqrt(3),2);return"M-"+r.round(e/2,2)+",-"+t+"V"+t+"L"+r.round(e,2)+",0Z"}},"triangle-ne":{n:9,f:function(e){var t=r.round(.6*e,2),n=r.round(1.2*e,2);return"M-"+n+",-"+t+"H"+t+"V"+n+"Z"}},"triangle-se":{n:10,f:function(e){var t=r.round(.6*e,2),n=r.round(1.2*e,2);return"M"+t+",-"+n+"V"+t+"H-"+n+"Z"}},"triangle-sw":{n:11,f:function(e){var t=r.round(.6*e,2),n=r.round(1.2*e,2);return"M"+n+","+t+"H-"+t+"V-"+n+"Z"}},"triangle-nw":{n:12,f:function(e){var t=r.round(.6*e,2),n=r.round(1.2*e,2);return"M-"+t+","+n+"V-"+t+"H"+n+"Z"}},pentagon:{n:13,f:function(e){var t=r.round(.951*e,2),n=r.round(.588*e,2),i=r.round(-e,2),o=r.round(-.309*e,2);return"M"+t+","+o+"L"+n+","+r.round(.809*e,2)+"H-"+n+"L-"+t+","+o+"L0,"+i+"Z"}},hexagon:{n:14,f:function(e){var t=r.round(e,2),n=r.round(e/2,2),i=r.round(e*Math.sqrt(3)/2,2);return"M"+i+",-"+n+"V"+n+"L0,"+t+"L-"+i+","+n+"V-"+n+"L0,-"+t+"Z"}},hexagon2:{n:15,f:function(e){var t=r.round(e,2),n=r.round(e/2,2),i=r.round(e*Math.sqrt(3)/2,2);return"M-"+n+","+i+"H"+n+"L"+t+",0L"+n+",-"+i+"H-"+n+"L-"+t+",0Z"}},octagon:{n:16,f:function(e){var t=r.round(.924*e,2),n=r.round(.383*e,2);return"M-"+n+",-"+t+"H"+n+"L"+t+",-"+n+"V"+n+"L"+n+","+t+"H-"+n+"L-"+t+","+n+"V-"+n+"Z"}},star:{n:17,f:function(e){var t=1.4*e,n=r.round(.225*t,2),i=r.round(.951*t,2),o=r.round(.363*t,2),a=r.round(.588*t,2),s=r.round(-t,2),l=r.round(-.309*t,2),u=r.round(.118*t,2),c=r.round(.809*t,2);return"M"+n+","+l+"H"+i+"L"+o+","+u+"L"+a+","+c+"L0,"+r.round(.382*t,2)+"L-"+a+","+c+"L-"+o+","+u+"L-"+i+","+l+"H-"+n+"L0,"+s+"Z"}},hexagram:{n:18,f:function(e){var t=r.round(.66*e,2),n=r.round(.38*e,2),i=r.round(.76*e,2);return"M-"+i+",0l-"+n+",-"+t+"h"+i+"l"+n+",-"+t+"l"+n+","+t+"h"+i+"l-"+n+","+t+"l"+n+","+t+"h-"+i+"l-"+n+","+t+"l-"+n+",-"+t+"h-"+i+"Z"}},"star-triangle-up":{n:19,f:function(e){var t=r.round(e*Math.sqrt(3)*.8,2),n=r.round(.8*e,2),i=r.round(1.6*e,2),o=r.round(4*e,2),a="A "+o+","+o+" 0 0 1 ";return"M-"+t+","+n+a+t+","+n+a+"0,-"+i+a+"-"+t+","+n+"Z"}},"star-triangle-down":{n:20,f:function(e){var t=r.round(e*Math.sqrt(3)*.8,2),n=r.round(.8*e,2),i=r.round(1.6*e,2),o=r.round(4*e,2),a="A "+o+","+o+" 0 0 1 ";return"M"+t+",-"+n+a+"-"+t+",-"+n+a+"0,"+i+a+t+",-"+n+"Z"}},"star-square":{n:21,f:function(e){var t=r.round(1.1*e,2),n=r.round(2*e,2),i="A "+n+","+n+" 0 0 1 ";return"M-"+t+",-"+t+i+"-"+t+","+t+i+t+","+t+i+t+",-"+t+i+"-"+t+",-"+t+"Z"}},"star-diamond":{n:22,f:function(e){var t=r.round(1.4*e,2),n=r.round(1.9*e,2),i="A "+n+","+n+" 0 0 1 ";return"M-"+t+",0"+i+"0,"+t+i+t+",0"+i+"0,-"+t+i+"-"+t+",0Z"}},"diamond-tall":{n:23,f:function(e){var t=r.round(.7*e,2),n=r.round(1.4*e,2);return"M0,"+n+"L"+t+",0L0,-"+n+"L-"+t+",0Z"}},"diamond-wide":{n:24,f:function(e){var t=r.round(1.4*e,2),n=r.round(.7*e,2);return"M0,"+n+"L"+t+",0L0,-"+n+"L-"+t+",0Z"}},hourglass:{n:25,f:function(e){var t=r.round(e,2);return"M"+t+","+t+"H-"+t+"L"+t+",-"+t+"H-"+t+"Z"},noDot:!0},bowtie:{n:26,f:function(e){var t=r.round(e,2);return"M"+t+","+t+"V-"+t+"L-"+t+","+t+"V-"+t+"Z"},noDot:!0},"circle-cross":{n:27,f:function(e){var t=r.round(e,2);return"M0,"+t+"V-"+t+"M"+t+",0H-"+t+"M"+t+",0A"+t+","+t+" 0 1,1 0,-"+t+"A"+t+","+t+" 0 0,1 "+t+",0Z"},needLine:!0,noDot:!0},"circle-x":{n:28,f:function(e){var t=r.round(e,2),n=r.round(e/Math.sqrt(2),2);return"M"+n+","+n+"L-"+n+",-"+n+"M"+n+",-"+n+"L-"+n+","+n+"M"+t+",0A"+t+","+t+" 0 1,1 0,-"+t+"A"+t+","+t+" 0 0,1 "+t+",0Z"},needLine:!0,noDot:!0},"square-cross":{n:29,f:function(e){var t=r.round(e,2);return"M0,"+t+"V-"+t+"M"+t+",0H-"+t+"M"+t+","+t+"H-"+t+"V-"+t+"H"+t+"Z"},needLine:!0,noDot:!0},"square-x":{n:30,f:function(e){var t=r.round(e,2);return"M"+t+","+t+"L-"+t+",-"+t+"M"+t+",-"+t+"L-"+t+","+t+"M"+t+","+t+"H-"+t+"V-"+t+"H"+t+"Z"},needLine:!0,noDot:!0},"diamond-cross":{n:31,f:function(e){var t=r.round(1.3*e,2);return"M"+t+",0L0,"+t+"L-"+t+",0L0,-"+t+"ZM0,-"+t+"V"+t+"M-"+t+",0H"+t},needLine:!0,noDot:!0},"diamond-x":{n:32,f:function(e){var t=r.round(1.3*e,2),n=r.round(.65*e,2);return"M"+t+",0L0,"+t+"L-"+t+",0L0,-"+t+"ZM-"+n+",-"+n+"L"+n+","+n+"M-"+n+","+n+"L"+n+",-"+n},needLine:!0,noDot:!0},"cross-thin":{n:33,f:function(e){var t=r.round(1.4*e,2);return"M0,"+t+"V-"+t+"M"+t+",0H-"+t},needLine:!0,noDot:!0,noFill:!0},"x-thin":{n:34,f:function(e){var t=r.round(e,2);return"M"+t+","+t+"L-"+t+",-"+t+"M"+t+",-"+t+"L-"+t+","+t},needLine:!0,noDot:!0,noFill:!0},asterisk:{n:35,f:function(e){var t=r.round(1.2*e,2),n=r.round(.85*e,2);return"M0,"+t+"V-"+t+"M"+t+",0H-"+t+"M"+n+","+n+"L-"+n+",-"+n+"M"+n+",-"+n+"L-"+n+","+n},needLine:!0,noDot:!0,noFill:!0},hash:{n:36,f:function(e){var t=r.round(e/2,2),n=r.round(e,2);return"M"+t+","+n+"V-"+n+"m-"+n+",0V"+n+"M"+n+","+t+"H-"+n+"m0,-"+n+"H"+n},needLine:!0,noFill:!0},"y-up":{n:37,f:function(e){var t=r.round(1.2*e,2),n=r.round(1.6*e,2),i=r.round(.8*e,2);return"M-"+t+","+i+"L0,0M"+t+","+i+"L0,0M0,-"+n+"L0,0"},needLine:!0,noDot:!0,noFill:!0},"y-down":{n:38,f:function(e){var t=r.round(1.2*e,2),n=r.round(1.6*e,2),i=r.round(.8*e,2);return"M-"+t+",-"+i+"L0,0M"+t+",-"+i+"L0,0M0,"+n+"L0,0"},needLine:!0,noDot:!0,noFill:!0},"y-left":{n:39,f:function(e){var t=r.round(1.2*e,2),n=r.round(1.6*e,2),i=r.round(.8*e,2);return"M"+i+","+t+"L0,0M"+i+",-"+t+"L0,0M-"+n+",0L0,0"},needLine:!0,noDot:!0,noFill:!0},"y-right":{n:40,f:function(e){var t=r.round(1.2*e,2),n=r.round(1.6*e,2),i=r.round(.8*e,2);return"M-"+i+","+t+"L0,0M-"+i+",-"+t+"L0,0M"+n+",0L0,0"},needLine:!0,noDot:!0,noFill:!0},"line-ew":{n:41,f:function(e){var t=r.round(1.4*e,2);return"M"+t+",0H-"+t},needLine:!0,noDot:!0,noFill:!0},"line-ns":{n:42,f:function(e){var t=r.round(1.4*e,2);return"M0,"+t+"V-"+t},needLine:!0,noDot:!0,noFill:!0},"line-ne":{n:43,f:function(e){var t=r.round(e,2);return"M"+t+",-"+t+"L-"+t+","+t},needLine:!0,noDot:!0,noFill:!0},"line-nw":{n:44,f:function(e){var t=r.round(e,2);return"M"+t+","+t+"L-"+t+",-"+t},needLine:!0,noDot:!0,noFill:!0},"arrow-up":{n:45,f:function(e){var t=r.round(e,2);return"M0,0L-"+t+","+r.round(2*e,2)+"H"+t+"Z"},noDot:!0},"arrow-down":{n:46,f:function(e){var t=r.round(e,2);return"M0,0L-"+t+",-"+r.round(2*e,2)+"H"+t+"Z"},noDot:!0},"arrow-left":{n:47,f:function(e){var t=r.round(2*e,2),n=r.round(e,2);return"M0,0L"+t+",-"+n+"V"+n+"Z"},noDot:!0},"arrow-right":{n:48,f:function(e){var t=r.round(2*e,2),n=r.round(e,2);return"M0,0L-"+t+",-"+n+"V"+n+"Z"},noDot:!0},"arrow-bar-up":{n:49,f:function(e){var t=r.round(e,2);return"M-"+t+",0H"+t+"M0,0L-"+t+","+r.round(2*e,2)+"H"+t+"Z"},needLine:!0,noDot:!0},"arrow-bar-down":{n:50,f:function(e){var t=r.round(e,2);return"M-"+t+",0H"+t+"M0,0L-"+t+",-"+r.round(2*e,2)+"H"+t+"Z"},needLine:!0,noDot:!0},"arrow-bar-left":{n:51,f:function(e){var t=r.round(2*e,2),n=r.round(e,2);return"M0,-"+n+"V"+n+"M0,0L"+t+",-"+n+"V"+n+"Z"},needLine:!0,noDot:!0},"arrow-bar-right":{n:52,f:function(e){var t=r.round(2*e,2),n=r.round(e,2);return"M0,-"+n+"V"+n+"M0,0L-"+t+",-"+n+"V"+n+"Z"},needLine:!0,noDot:!0}}},function(e,t,n){"use strict";var r=n(134),i=n(63).extendFlat,o=n(124).overrideAll,a=["Legacy polar charts are deprecated!","Please switch to *polar* subplots."].join(" "),s=i({},r.domain,{});function l(e,t){return i({},t,{showline:{valType:"boolean"},showticklabels:{valType:"boolean"},tickorientation:{valType:"enumerated",values:["horizontal","vertical"]},ticklen:{valType:"number",min:0},tickcolor:{valType:"color"},ticksuffix:{valType:"string"},endpadding:{valType:"number",description:a},visible:{valType:"boolean"}})}e.exports=o({radialaxis:l(0,{range:{valType:"info_array",items:[{valType:"number"},{valType:"number"}]},domain:s,orientation:{valType:"number"}}),angularaxis:l(0,{range:{valType:"info_array",items:[{valType:"number",dflt:0},{valType:"number",dflt:360}]},domain:s}),layout:{direction:{valType:"enumerated",values:["clockwise","counterclockwise"]},orientation:{valType:"angle"}}},"plot","nested")},function(e,t,n){"use strict";var r=n(27),i=n(10);function o(e,t,n){var r,o,a,s=!1;if("data"===t.type)r=e._fullData[null!==t.traces?t.traces[0]:0];else{if("layout"!==t.type)return!1;r=e._fullLayout}return o=i.nestedProperty(r,t.prop).get(),(a=n[t.type]=n[t.type]||{}).hasOwnProperty(t.prop)&&a[t.prop]!==o&&(s=!0),a[t.prop]=o,{changed:s,value:o}}function a(e,t){var n=[],r=t[0],o={};if("string"==typeof r)o[r]=t[1];else{if(!i.isPlainObject(r))return n;o=r}return l(o,(function(e,t,r){n.push({type:"layout",prop:e,value:r})}),"",0),n}function s(e,t){var n,r,o,a,s=[];if(r=t[0],o=t[1],n=t[2],a={},"string"==typeof r)a[r]=o;else{if(!i.isPlainObject(r))return s;a=r,void 0===n&&(n=o)}return void 0===n&&(n=null),l(a,(function(t,r,i){var o,a;if(Array.isArray(i)){a=i.slice();var l=Math.min(a.length,e.data.length);n&&(l=Math.min(l,n.length)),o=[];for(var u=0;u<l;u++)o[u]=n?n[u]:u}else a=i,o=n?n.slice():null;if(null===o)Array.isArray(a)&&(a=a[0]);else if(Array.isArray(o)){if(!Array.isArray(a)){var c=a;a=[];for(var d=0;d<o.length;d++)a[d]=c}a.length=Math.min(o.length,a.length)}s.push({type:"data",prop:t,traces:o,value:a})}),"",0),s}function l(e,t,n,r){Object.keys(e).forEach((function(o){var a=e[o];if("_"!==o[0]){var s=n+(r>0?".":"")+o;i.isPlainObject(a)?l(a,t,s,r+1):t(s,o,a)}}))}t.manageCommandObserver=function(e,n,r,a){var s={},l=!0;n&&n._commandObserver&&(s=n._commandObserver),s.cache||(s.cache={}),s.lookupTable={};var u=t.hasSimpleAPICommandBindings(e,r,s.lookupTable);if(n&&n._commandObserver){if(u)return s;if(n._commandObserver.remove)return n._commandObserver.remove(),n._commandObserver=null,s}if(u){o(e,u,s.cache),s.check=function(){if(l){var t=o(e,u,s.cache);return t.changed&&a&&void 0!==s.lookupTable[t.value]&&(s.disable(),Promise.resolve(a({value:t.value,type:u.type,prop:u.prop,traces:u.traces,index:s.lookupTable[t.value]})).then(s.enable,s.enable)),t.changed}};for(var c=["plotly_relayout","plotly_redraw","plotly_restyle","plotly_update","plotly_animatingframe","plotly_afterplot"],d=0;d<c.length;d++)e._internalOn(c[d],s.check);s.remove=function(){for(var t=0;t<c.length;t++)e._removeInternalListener(c[t],s.check)}}else i.log("Unable to automatically bind plot updates to API command"),s.lookupTable={},s.remove=function(){};return s.disable=function(){l=!1},s.enable=function(){l=!0},n&&(n._commandObserver=s),s},t.hasSimpleAPICommandBindings=function(e,n,r){var i,o,a=n.length;for(i=0;i<a;i++){var s,l=n[i],u=l.method,c=l.args;if(Array.isArray(c)||(c=[]),!u)return!1;var d=t.computeAPICommandBindings(e,u,c);if(1!==d.length)return!1;if(o){if((s=d[0]).type!==o.type)return!1;if(s.prop!==o.prop)return!1;if(Array.isArray(o.traces)){if(!Array.isArray(s.traces))return!1;s.traces.sort();for(var h=0;h<o.traces.length;h++)if(o.traces[h]!==s.traces[h])return!1}else if(s.prop!==o.prop)return!1}else o=d[0],Array.isArray(o.traces)&&o.traces.sort();var f=(s=d[0]).value;if(Array.isArray(f)){if(1!==f.length)return!1;f=f[0]}r&&(r[f]=i)}return o},t.executeAPICommand=function(e,t,n){if("skip"===t)return Promise.resolve();var o=r.apiMethodRegistry[t],a=[e];Array.isArray(n)||(n=[]);for(var s=0;s<n.length;s++)a.push(n[s]);return o.apply(null,a).catch((function(e){return i.warn("API call to Plotly."+t+" rejected.",e),Promise.reject(e)}))},t.computeAPICommandBindings=function(e,t,n){var r;switch(Array.isArray(n)||(n=[]),t){case"restyle":r=s(e,n);break;case"relayout":r=a(e,n);break;case"update":r=s(e,[n[0],n[2]]).concat(a(e,[n[1]]));break;case"animate":r=function(e,t){return Array.isArray(t[0])&&1===t[0].length&&-1!==["string","number"].indexOf(typeof t[0][0])?[{type:"layout",prop:"_currentFrame",value:t[0][0].toString()}]:[]}(0,n);break;default:r=[]}return r}},function(e,t,n){"use strict";(e.exports=n(990)).manager=n(1685)},function(e,t,n){"use strict";var r=n(39),i=n(10),o=n(41),a=n(990),s=n(1686),l=i.extendDeepAll,u=e.exports={};u.framework=function(e){var t,n,i,o,c,d=new s;function h(n,s){return s&&(c=s),r.select(r.select(c).node().parentNode).selectAll(".svg-container>*:not(.chart-root)").remove(),t=t?l(t,n):n,i||(i=a.Axis()),o=a.adapter.plotly().convert(t),i.config(o).render(c),e.data=t.data,e.layout=t.layout,u.fillLayout(e),t}return h.isPolar=!0,h.svg=function(){return i.svg()},h.getConfig=function(){return t},h.getLiveConfig=function(){return a.adapter.plotly().convert(i.getLiveConfig(),!0)},h.getLiveScales=function(){return{t:i.angularScale(),r:i.radialScale()}},h.setUndoPoint=function(){var e,r,i=this,o=a.util.cloneJson(t);e=o,r=n,d.add({undo:function(){r&&i(r)},redo:function(){i(e)}}),n=a.util.cloneJson(o)},h.undo=function(){d.undo()},h.redo=function(){d.redo()},h},u.fillLayout=function(e){var t=r.select(e).selectAll(".plot-container"),n=t.selectAll(".svg-container"),i=e.framework&&e.framework.svg&&e.framework.svg(),a={width:800,height:600,paper_bgcolor:o.background,_container:t,_paperdiv:n,_paper:i};e._fullLayout=l(a,e.layout)}},function(e,t,n){"use strict";e.exports=function(){var e,t=[],n=-1,r=!1;function i(e,t){return e?(r=!0,e[t](),r=!1,this):this}return{add:function(e){return r||(t.splice(n+1,t.length-n),t.push(e),n=t.length-1),this},setCallback:function(t){e=t},undo:function(){var r=t[n];return r?(i(r,"undo"),n-=1,e&&e(r.undo),this):this},redo:function(){var r=t[n+1];return r?(i(r,"redo"),n+=1,e&&e(r.redo),this):this},clear:function(){t=[],n=-1},hasUndo:function(){return-1!==n},hasRedo:function(){return n<t.length-1},getCommands:function(){return t},getPreviousCommand:function(){return t[n-1]},getIndex:function(){return n}}}},function(e,t,n){"use strict";e.exports=function(e,t,n,r,i){var o=(e-n)/(r-n),a=o+t/(r-n),s=(o+a)/2;return"left"===i||"bottom"===i?o:"center"===i||"middle"===i?s:"right"===i||"top"===i?a:o<2/3-s?o:a>4/3-s?a:s}},function(e,t,n){"use strict";var r=n(10),i=[["sw-resize","s-resize","se-resize"],["w-resize","move","e-resize"],["nw-resize","n-resize","ne-resize"]];e.exports=function(e,t,n,o){return e="left"===n?0:"center"===n?1:"right"===n?2:r.constrain(Math.floor(3*e),0,2),t="bottom"===o?0:"middle"===o?1:"top"===o?2:r.constrain(Math.floor(3*t),0,2),i[t][e]}},function(e,t,n){"use strict";var r=n(383),i=n(539),o=n(527).getGraphDiv,a=n(381),s=e.exports={};s.wrapped=function(e,t,n){(e=o(e))._fullLayout&&i.clear(e._fullLayout._uid+a.HOVERID),s.raw(e,t,n)},s.raw=function(e,t){var n=e._fullLayout,i=e._hoverdata;t||(t={}),t.target&&!1===r.triggerHandler(e,"plotly_beforehover",t)||(n._hoverlayer.selectAll("g").remove(),n._hoverlayer.selectAll("line").remove(),n._hoverlayer.selectAll("circle").remove(),e._hoverdata=void 0,t.target&&i&&e.emit("plotly_unhover",{event:t,points:i}))}},function(e,t,n){"use strict";var r=n(214),i="data-savedcursor";e.exports=function(e,t){var n=e.attr(i);if(t){if(!n){for(var o=(e.attr("class")||"").split(" "),a=0;a<o.length;a++){var s=o[a];0===s.indexOf("cursor-")&&e.attr(i,s.substr(7)).classed(s,!1)}e.attr(i)||e.attr(i,"!!")}r(e,t)}else n&&(e.attr(i,null),"!!"===n?r(e):r(e,n))}},function(e,t,n){"use strict";var r=n(10),i=n(27),o=!0;e.exports=function(e,t,n){var a=t._fullLayout;if(!t._dragged&&!t._editing){var s,l=a.legend.itemclick,u=a.legend.itemdoubleclick;if(1===n&&"toggle"===l&&"toggleothers"===u&&o&&t.data&&t._context.showTips?(r.notifier(r._(t,"Double-click on legend to isolate one trace"),"long"),o=!1):o=!1,1===n?s=l:2===n&&(s=u),s){var c,d,h,f,p,g=a.hiddenlabels?a.hiddenlabels.slice():[],v=e.data()[0][0],m=t._fullData,y=v.trace,b=y.legendgroup,C={},A=[],_=[],I=[];if(i.traceIs(y,"pie-like")){var x=v.label,w=g.indexOf(x);"toggle"===s?-1===w?g.push(x):g.splice(w,1):"toggleothers"===s&&(g=[],t.calcdata[0].forEach((function(e){x!==e.label&&g.push(e.label)})),t._fullLayout.hiddenlabels&&t._fullLayout.hiddenlabels.length===g.length&&-1===w&&(g=[])),i.call("_guiRelayout",t,"hiddenlabels",g)}else{var M,S=b&&b.length,T=[];if(S)for(c=0;c<m.length;c++)(M=m[c]).visible&&M.legendgroup===b&&T.push(c);if("toggle"===s){var E;switch(y.visible){case!0:E="legendonly";break;case!1:E=!1;break;case"legendonly":E=!0}if(S)for(c=0;c<m.length;c++)!1!==m[c].visible&&m[c].legendgroup===b&&z(m[c],E);else z(y,E)}else if("toggleothers"===s){var O,k,L,D,P=!0;for(c=0;c<m.length;c++)if(O=m[c]===y,L=!0!==m[c].showlegend,!(O||L||(k=S&&m[c].legendgroup===b)||!0!==m[c].visible||i.traceIs(m[c],"notLegendIsolatable"))){P=!1;break}for(c=0;c<m.length;c++)if(!1!==m[c].visible&&!i.traceIs(m[c],"notLegendIsolatable"))switch(y.visible){case"legendonly":z(m[c],!0);break;case!0:D=!!P||"legendonly",O=m[c]===y,L=!0!==m[c].showlegend&&!m[c].legendgroup,k=O||S&&m[c].legendgroup===b,z(m[c],!(!k&&!L)||D)}}for(c=0;c<_.length;c++)if(h=_[c]){var N=h.constructUpdate(),j=Object.keys(N);for(d=0;d<j.length;d++)f=j[d],(C[f]=C[f]||[])[I[c]]=N[f]}for(p=Object.keys(C),c=0;c<p.length;c++)for(f=p[c],d=0;d<A.length;d++)C[f].hasOwnProperty(d)||(C[f][d]=void 0);i.call("_guiRestyle",t,C,A)}}}function R(e,t,n){var r=A.indexOf(e),i=C[t];return i||(i=C[t]=[]),-1===A.indexOf(e)&&(A.push(e),r=A.length-1),i[r]=n,r}function z(e,t){var n=e._fullInput;if(i.hasTransform(n,"groupby")){var o=_[n.index];if(!o){var a=i.getTransformIndices(n,"groupby"),s=a[a.length-1];o=r.keyedContainer(n,"transforms["+s+"].styles","target","value.visible"),_[n.index]=o}var l=o.get(e._group);void 0===l&&(l=!0),!1!==l&&o.set(e._group,t),I[n.index]=R(n.index,"visible",!1!==n.visible)}else{var u=!1!==n.visible&&t;R(n.index,"visible",u)}}}},function(e,t,n){"use strict";var r=n(27),i=n(546);e.exports=function(e,t){var n,o,a={},s=[],l=!1,u={},c=0,d=0,h=t._main;function f(e,n){if(""!==e&&i.isGrouped(t))-1===s.indexOf(e)?(s.push(e),l=!0,a[e]=[[n]]):a[e].push([n]);else{var r="~~i"+c;s.push(r),a[r]=[[n]],c++}}for(n=0;n<e.length;n++){var p=e[n],g=p[0],v=g.trace,m=v.legendgroup;if(!h||v.visible&&v.showlegend)if(r.traceIs(v,"pie-like"))for(u[m]||(u[m]={}),o=0;o<p.length;o++){var y=p[o].label;u[m][y]||(f(m,{label:y,color:p[o].color,i:p[o].i,trace:v,pts:p[o].pts}),u[m][y]=!0,d=Math.max(d,(y||"").length))}else f(m,g),d=Math.max(d,(v.name||"").length)}if(!s.length)return[];var b,C,A=s.length;if(l&&i.isGrouped(t))for(C=new Array(A),n=0;n<A;n++)b=a[s[n]],C[n]=i.isReversed(t)?b.reverse():b;else{for(C=[new Array(A)],n=0;n<A;n++)b=a[s[n]][0],C[0][i.isReversed(t)?A-n-1:n]=b;A=1}return t._lgroupsLength=A,t._maxNameLength=d,C}},function(e,t,n){"use strict";var r=n(41),i=n(999).castOption;e.exports=function(e,t,n){var o=n.marker.line,a=i(o.color,t.pts)||r.defaultLine,s=i(o.width,t.pts)||0;e.style("stroke-width",s).call(r.fill,t.color).call(r.stroke,a)}},function(e,t,n){"use strict";var r=n(10),i=n(547),o=n(250);e.exports=function(e,t){i(e,t,(function(n,i){return r.coerce(e,t,o,n,i)}))}},function(e,t,n){"use strict";var r=n(10),i=n(524),o=n(547);e.exports=function(e,t,n,a){var s=r.extendFlat({},a.hoverlabel);t.hovertemplate&&(s.namelength=-1),o(e,t,(function(n,o){return r.coerce(e,t,i,n,o)}),s)}},function(e,t,n){"use strict";var r=n(10),i=n(162).isUnifiedHover,o=n(250),a=n(1e3),s=n(547);e.exports=function(e,t,n){function l(n,i){return r.coerce(e,t,o,n,i)}var u=a(e,t,n);u&&(l("hoverdistance"),l("spikedistance",i(u)?-1:void 0)),"select"===l("dragmode")&&l("selectdirection");var c=t._has("mapbox"),d=t._has("geo"),h=t._basePlotModules.length;"zoom"===t.dragmode&&((c||d)&&1===h||c&&d&&2===h)&&(t.dragmode="pan"),s(e,t,l)}},function(e,t,n){"use strict";var r=n(10),i=n(27);function o(e,t,n,i){i=i||r.identity,Array.isArray(e)&&(t[0][n]=i(e))}e.exports=function(e){var t=e.calcdata,n=e._fullLayout;function a(e){return function(t){return r.coerceHoverinfo({hoverinfo:t},{_module:e._module},n)}}for(var s=0;s<t.length;s++){var l=t[s],u=l[0].trace;if(!i.traceIs(u,"pie-like")){var c=i.traceIs(u,"2dMap")?o:r.fillArray;c(u.hoverinfo,l,"hi",a(u)),u.hovertemplate&&c(u.hovertemplate,l,"ht"),u.hoverlabel&&(c(u.hoverlabel.bgcolor,l,"hbg"),c(u.hoverlabel.bordercolor,l,"hbc"),c(u.hoverlabel.font.size,l,"hts"),c(u.hoverlabel.font.color,l,"htc"),c(u.hoverlabel.font.family,l,"htf"),c(u.hoverlabel.namelength,l,"hnl"),c(u.hoverlabel.align,l,"hta"))}}}},function(e,t,n){"use strict";var r=n(27),i=n(993).hover;e.exports=function(e,t,n){var o=r.getComponentMethod("annotations","onClick")(e,e._hoverdata);function a(){e.emit("plotly_click",{points:e._hoverdata,event:t})}void 0!==n&&i(e,t,n,!0),e._hoverdata&&t&&t.target&&(o&&o.then?o.then(a):a(),t.stopImmediatePropagation&&t.stopImmediatePropagation())}},function(e,t,n){"use strict";var r=n(39),i=n(123),o=n(387),a=n(27),s=n(10),l=s.strTranslate,u=n(96),c=n(41),d=n(56),h=n(179),f=n(55),p=n(214),g=n(163),v=n(388),m=v.selectingOrDrawing,y=v.freeMode,b=n(97).FROM_TL,C=n(548),A=n(549).redrawReglTraces,_=n(82),I=n(83).getFromId,x=n(391).prepSelect,w=n(391).clearSelect,M=n(391).selectOnClick,S=n(1004),T=n(95),E=T.MINDRAG,O=T.MINZOOM,k=!0;function L(e,t,n,r){var i=s.ensureSingle(e.draglayer,t,n,(function(t){t.classed("drag",!0).style({fill:"transparent","stroke-width":0}).attr("data-subplot",e.id)}));return i.call(p,r),i.node()}function D(e,t,n,i,o,a,s){var l=L(e,"rect",t,n);return r.select(l).call(d.setRect,i,o,a,s),l}function P(e,t){for(var n=0;n<e.length;n++)if(!e[n].fixedrange)return t;return""}function N(e,t,n,r,i){for(var o=0;o<e.length;o++){var a=e[o];if(!a.fixedrange)if(a.rangebreaks){var s="y"===a._id.charAt(0),l=s?1-t:t,u=s?1-n:n;r[a._name+".range[0]"]=a.l2r(a.p2l(l*a._length)),r[a._name+".range[1]"]=a.l2r(a.p2l(u*a._length))}else{var c=a._rl[0],d=a._rl[1]-c;r[a._name+".range[0]"]=a.l2r(c+d*t),r[a._name+".range[1]"]=a.l2r(c+d*n)}}if(i&&i.length){var h=(t+(1-n))/2;N(i,h,1-h,r,[])}}function j(e,t){for(var n=0;n<e.length;n++){var r=e[n];if(!r.fixedrange)if(r.rangebreaks){var i=r._length,o=(r.p2l(0+t)-r.p2l(0)+(r.p2l(i+t)-r.p2l(i)))/2;r.range=[r.l2r(r._rl[0]-o),r.l2r(r._rl[1]-o)]}else r.range=[r.l2r(r._rl[0]-t/r._m),r.l2r(r._rl[1]-t/r._m)]}}function R(e){return 1-(e>=0?Math.min(e,.9):1/(1/Math.max(e,-.3)+3.222))}function z(e,t,n,r,i){return e.append("path").attr("class","zoombox").style({fill:t>.2?"rgba(0,0,0,0)":"rgba(255,255,255,0)","stroke-width":0}).attr("transform",l(n,r)).attr("d",i+"Z")}function F(e,t,n){return e.append("path").attr("class","zoombox-corners").style({fill:c.background,stroke:c.defaultLine,"stroke-width":1,opacity:0}).attr("transform",l(t,n)).attr("d","M0,0Z")}function B(e,t,n,r,i,o){e.attr("d",r+"M"+n.l+","+n.t+"v"+n.h+"h"+n.w+"v-"+n.h+"h-"+n.w+"Z"),H(e,t,i,o)}function H(e,t,n,r){n||(e.transition().style("fill",r>.2?"rgba(0,0,0,0.4)":"rgba(255,255,255,0.3)").duration(200),t.transition().style("opacity",1).duration(200))}function U(e){r.select(e).selectAll(".zoombox,.js-zoombox-backdrop,.js-zoombox-menu,.zoombox-corners").remove()}function V(e){k&&e.data&&e._context.showTips&&(s.notifier(s._(e,"Double-click to zoom back out"),"long"),k=!1)}function Y(e){var t=Math.floor(Math.min(e.b-e.t,e.r-e.l,O)/2);return"M"+(e.l-3.5)+","+(e.t-.5+t)+"h3v"+-t+"h"+t+"v-3h-"+(t+3)+"ZM"+(e.r+3.5)+","+(e.t-.5+t)+"h-3v"+-t+"h"+-t+"v-3h"+(t+3)+"ZM"+(e.r+3.5)+","+(e.b+.5-t)+"h-3v"+t+"h"+-t+"v3h"+(t+3)+"ZM"+(e.l-3.5)+","+(e.b+.5-t)+"h3v"+t+"h"+t+"v3h-"+(t+3)+"Z"}function W(e,t,n,r,i){for(var o,a,l,u,c=!1,d={},h={},f=(i||{}).xaHash,p=(i||{}).yaHash,g=0;g<t.length;g++){var v=t[g];for(o in n)if(v[o]){for(l in v)i&&(f[l]||p[l])||("x"===l.charAt(0)?n:r)[l]||(d[l]=o);for(a in r)i&&(f[a]||p[a])||!v[a]||(c=!0)}for(a in r)if(v[a])for(u in v)i&&(f[u]||p[u])||("x"===u.charAt(0)?n:r)[u]||(h[u]=a)}c&&(s.extendFlat(d,h),h={});var m={},y=[];for(l in d){var b=I(e,l);y.push(b),m[b._id]=b}var C={},A=[];for(u in h){var _=I(e,u);A.push(_),C[_._id]=_}return{xaHash:m,yaHash:C,xaxes:y,yaxes:A,xLinks:d,yLinks:h,isSubplotConstrained:c}}function G(e,t){if(o){var n=void 0!==e.onwheel?"wheel":"mousewheel";e._onwheel&&e.removeEventListener(n,e._onwheel),e._onwheel=t,e.addEventListener(n,t,{passive:!1})}else void 0!==e.onwheel?e.onwheel=t:void 0!==e.onmousewheel?e.onmousewheel=t:e.isAddedWheelEvent||(e.isAddedWheelEvent=!0,e.addEventListener("wheel",t,{passive:!1}))}function X(e){var t=[];for(var n in e)t.push(e[n]);return t}e.exports={makeDragBox:function(e,t,n,o,l,c,p,v){var k,L,H,q,Z,Q,K,J,$,ee,te,ne,re,ie,oe,ae,se,le,ue,ce,de,he,fe,pe=e._fullLayout._zoomlayer,ge=p+v==="nsew",ve=1===(p+v).length;function me(){if(k=t.xaxis,L=t.yaxis,$=k._length,ee=L._length,K=k._offset,J=L._offset,(H={})[k._id]=k,(q={})[L._id]=L,p&&v)for(var n=t.overlays,r=0;r<n.length;r++){var i=n[r].xaxis;H[i._id]=i;var o=n[r].yaxis;q[o._id]=o}Z=X(H),Q=X(q),re=P(Z,v),ie=P(Q,p),oe=!ie&&!re,ne=W(e,e._fullLayout._axisMatchGroups,H,q);var a=(te=W(e,e._fullLayout._axisConstraintGroups,H,q,ne)).isSubplotConstrained||ne.isSubplotConstrained;ae=v||a,se=p||a;var s=e._fullLayout;le=s._has("scattergl"),ue=s._has("splom"),ce=s._has("svg")}me();var ye=function(e,t,n){if(!e)return"pointer";if("nsew"===e)return n?"":"pan"===t?"move":"crosshair";return e.toLowerCase()+"-resize"}(ie+re,e._fullLayout.dragmode,ge),be=D(t,p+v+"drag",ye,n,o,l,c);if(oe&&!ge)return be.onmousedown=null,be.style.pointerEvents="none",be;var Ce,Ae,_e,Ie,xe,we,Me,Se,Te,Ee,Oe={element:be,gd:e,plotinfo:t};function ke(){Oe.plotinfo.selection=!1,w(e)}function Le(e,n){var i=Oe.gd;if(i._fullLayout._activeShapeIndex>=0)i._fullLayout._deactivateShape(i);else{var o=i._fullLayout.clickmode;if(U(i),2!==e||ve||Ve(),ge)o.indexOf("select")>-1&&M(n,i,Z,Q,t.id,Oe),o.indexOf("event")>-1&&h.click(i,n,t.id);else if(1===e&&ve){var s=p?L:k,l="s"===p||"w"===v?0:1,c=s._name+".range["+l+"]",d=function(e,t){var n,i=e.range[t],o=Math.abs(i-e.range[1-t]);return"date"===e.type?i:"log"===e.type?(n=Math.ceil(Math.max(0,-Math.log(o)/Math.LN10))+3,r.format("."+n+"g")(Math.pow(10,i))):(n=Math.floor(Math.log(Math.abs(i))/Math.LN10)-Math.floor(Math.log(o)/Math.LN10)+4,r.format("."+String(n)+"g")(i))}(s,l),f="left",g="middle";if(s.fixedrange)return;p?(g="n"===p?"top":"bottom","right"===s.side&&(f="right")):"e"===v&&(f="right"),i._context.showAxisRangeEntryBoxes&&r.select(be).call(u.makeEditable,{gd:i,immediate:!0,background:i._fullLayout.paper_bgcolor,text:String(d),fill:s.tickfont?s.tickfont.color:"#444",horizontalAlign:f,verticalAlign:g}).on("edit",(function(e){var t=s.d2r(e);void 0!==t&&a.call("_guiRelayout",i,c,t)}))}}}function De(t,n){if(e._transitioningWithDuration)return!1;var r=Math.max(0,Math.min($,he*t+Ce)),i=Math.max(0,Math.min(ee,fe*n+Ae)),o=Math.abs(r-Ce),a=Math.abs(i-Ae);function s(){Me="",_e.r=_e.l,_e.t=_e.b,Te.attr("d","M0,0Z")}if(_e.l=Math.min(Ce,r),_e.r=Math.max(Ce,r),_e.t=Math.min(Ae,i),_e.b=Math.max(Ae,i),te.isSubplotConstrained)o>O||a>O?(Me="xy",o/$>a/ee?(a=o*ee/$,Ae>i?_e.t=Ae-a:_e.b=Ae+a):(o=a*$/ee,Ce>r?_e.l=Ce-o:_e.r=Ce+o),Te.attr("d",Y(_e))):s();else if(ne.isSubplotConstrained)if(o>O||a>O){Me="xy";var l=Math.min(_e.l/$,(ee-_e.b)/ee),u=Math.max(_e.r/$,(ee-_e.t)/ee);_e.l=l*$,_e.r=u*$,_e.b=(1-l)*ee,_e.t=(1-u)*ee,Te.attr("d",Y(_e))}else s();else!ie||a<Math.min(Math.max(.6*o,E),O)?o<E||!re?s():(_e.t=0,_e.b=ee,Me="x",Te.attr("d",function(e,t){return"M"+(e.l-.5)+","+(t-O-.5)+"h-3v"+(2*O+1)+"h3ZM"+(e.r+.5)+","+(t-O-.5)+"h3v"+(2*O+1)+"h-3Z"}(_e,Ae))):!re||o<Math.min(.6*a,O)?(_e.l=0,_e.r=$,Me="y",Te.attr("d",function(e,t){return"M"+(t-O-.5)+","+(e.t-.5)+"v-3h"+(2*O+1)+"v3ZM"+(t-O-.5)+","+(e.b+.5)+"v3h"+(2*O+1)+"v-3Z"}(_e,Ce))):(Me="xy",Te.attr("d",Y(_e)));_e.w=_e.r-_e.l,_e.h=_e.b-_e.t,Me&&(Ee=!0),e._dragged=Ee,B(Se,Te,_e,xe,we,Ie),Pe(),e.emit("plotly_relayouting",de),we=!0}function Pe(){de={},"xy"!==Me&&"x"!==Me||(N(Z,_e.l/$,_e.r/$,de,te.xaxes),He("x",de)),"xy"!==Me&&"y"!==Me||(N(Q,(ee-_e.b)/ee,(ee-_e.t)/ee,de,te.yaxes),He("y",de))}function Ne(){Pe(),U(e),Ye(),V(e)}Oe.prepFn=function(t,n,r){var o=Oe.dragmode,a=e._fullLayout.dragmode;a!==o&&(Oe.dragmode=a),me(),he=e._fullLayout._invScaleX,fe=e._fullLayout._invScaleY,oe||(ge?t.shiftKey?"pan"===a?a="zoom":m(a)||(a="pan"):t.ctrlKey&&(a="pan"):a="pan"),y(a)?Oe.minDrag=1:Oe.minDrag=void 0,m(a)?(Oe.xaxes=Z,Oe.yaxes=Q,x(t,n,r,Oe,a)):(Oe.clickFn=Le,m(o)&&ke(),oe||("zoom"===a?(Oe.moveFn=De,Oe.doneFn=Ne,Oe.minDrag=1,function(t,n,r){var o=be.getBoundingClientRect();Ce=n-o.left,Ae=r-o.top,e._fullLayout._calcInverseTransform(e);var a=s.apply3DTransform(e._fullLayout._invTransform)(Ce,Ae);Ce=a[0],Ae=a[1],_e={l:Ce,r:Ce,w:0,t:Ae,b:Ae,h:0},Ie=e._hmpixcount?e._hmlumcount/e._hmpixcount:i(e._fullLayout.plot_bgcolor).getLuminance(),we=!1,Me="xy",Ee=!1,Se=z(pe,Ie,K,J,xe="M0,0H"+$+"V"+ee+"H0V0"),Te=F(pe,K,J)}(0,n,r)):"pan"===a&&(Oe.moveFn=Be,Oe.doneFn=Ye))),e._fullLayout._redrag=function(){var t=e._dragdata;if(t&&t.element===be){var n=e._fullLayout.dragmode;m(n)||(me(),We([0,0,$,ee]),Oe.moveFn(t.dx,t.dy))}}},g.init(Oe);var je=[0,0,$,ee],Re=null,ze=T.REDRAWDELAY,Fe=t.mainplot?e._fullLayout._plots[t.mainplot]:t;function Be(t,n){if(t*=he,n*=fe,!e._transitioningWithDuration){if(e._fullLayout._replotting=!0,"ew"===re||"ns"===ie){var r=re?-t:0,i=ie?-n:0;if(ne.isSubplotConstrained){if(re&&ie){var o=(t/$-n/ee)/2;r=-(t=o*$),i=-(n=-o*ee)}ie?r=-i*$/ee:i=-r*ee/$}return re&&(j(Z,t),He("x")),ie&&(j(Q,n),He("y")),We([r,i,$,ee]),Ue(),void e.emit("plotly_relayouting",de)}var a,s,l="w"===re==("n"===ie)?1:-1;if(re&&ie&&(te.isSubplotConstrained||ne.isSubplotConstrained)){var u=(t/$+l*n/ee)/2;t=u*$,n=l*u*ee}if("w"===re?t=f(Z,0,t):"e"===re?t=f(Z,1,-t):re||(t=0),"n"===ie?n=f(Q,1,n):"s"===ie?n=f(Q,0,-n):ie||(n=0),a="w"===re?t:0,s="n"===ie?n:0,te.isSubplotConstrained&&!ne.isSubplotConstrained||ne.isSubplotConstrained&&re&&ie&&l>0){var c;if(ne.isSubplotConstrained||!re&&1===ie.length){for(c=0;c<Z.length;c++)Z[c].range=Z[c]._r.slice(),S(Z[c],1-n/ee);a=(t=n*$/ee)/2}if(ne.isSubplotConstrained||!ie&&1===re.length){for(c=0;c<Q.length;c++)Q[c].range=Q[c]._r.slice(),S(Q[c],1-t/$);s=(n=t*ee/$)/2}}ne.isSubplotConstrained&&ie||He("x"),ne.isSubplotConstrained&&re||He("y");var d=$-t,h=ee-n;!ne.isSubplotConstrained||re&&ie||(re?(s=a?0:t*ee/$,h=d*ee/$):(a=s?0:n*$/ee,d=h*$/ee)),We([a,s,d,h]),Ue(),e.emit("plotly_relayouting",de)}function f(e,t,n){for(var r,i,o=1-t,a=0;a<e.length;a++){var s=e[a];if(!s.fixedrange){r=s,i=s._rl[o]+(s._rl[t]-s._rl[o])/R(n/s._length);var l=s.l2r(i);!1!==l&&void 0!==l&&(s.range[t]=l)}}return r._length*(r._rl[t]-i)/(r._rl[t]-r._rl[o])}}function He(e,t){for(var n=ne.isSubplotConstrained?{x:Q,y:Z}[e]:ne[e+"axes"],r=ne.isSubplotConstrained?{x:Z,y:Q}[e]:[],i=0;i<n.length;i++){var o=n[i],a=o._id,s=ne.xLinks[a]||ne.yLinks[a],l=r[0]||H[s]||q[s];l&&(t?(t[o._name+".range[0]"]=t[l._name+".range[0]"],t[o._name+".range[1]"]=t[l._name+".range[1]"]):o.range=l.range.slice())}}function Ue(){var t,n=[];function r(e){for(t=0;t<e.length;t++)e[t].fixedrange||n.push(e[t]._id)}for(ae&&(r(Z),r(te.xaxes),r(ne.xaxes)),se&&(r(Q),r(te.yaxes),r(ne.yaxes)),de={},t=0;t<n.length;t++){var i=n[t],o=I(e,i);f.drawOne(e,o,{skipTitle:!0}),de[o._name+".range[0]"]=o.range[0],de[o._name+".range[1]"]=o.range[1]}f.redrawComponents(e,n)}function Ve(){if(!e._transitioningWithDuration){var t=e._context.doubleClick,n=[];re&&(n=n.concat(Z)),ie&&(n=n.concat(Q)),ne.xaxes&&(n=n.concat(ne.xaxes)),ne.yaxes&&(n=n.concat(ne.yaxes));var r,i,o,s={};if("reset+autosize"===t)for(t="autosize",i=0;i<n.length;i++)if((r=n[i])._rangeInitial&&(r.range[0]!==r._rangeInitial[0]||r.range[1]!==r._rangeInitial[1])||!r._rangeInitial&&!r.autorange){t="reset";break}if("autosize"===t)for(i=0;i<n.length;i++)(r=n[i]).fixedrange||(s[r._name+".autorange"]=!0);else if("reset"===t)for((re||te.isSubplotConstrained)&&(n=n.concat(te.xaxes)),ie&&!te.isSubplotConstrained&&(n=n.concat(te.yaxes)),te.isSubplotConstrained&&(re?ie||(n=n.concat(Q)):n=n.concat(Z)),i=0;i<n.length;i++)(r=n[i]).fixedrange||(r._rangeInitial?(o=r._rangeInitial,s[r._name+".range[0]"]=o[0],s[r._name+".range[1]"]=o[1]):s[r._name+".autorange"]=!0);e.emit("plotly_doubleclick",null),a.call("_guiRelayout",e,s)}}function Ye(){We([0,0,$,ee]),s.syncOrAsync([_.previousPromises,function(){e._fullLayout._replotting=!1,a.call("_guiRelayout",e,de)}],e)}function We(t){var n,r,i,o,l=e._fullLayout,u=l._plots,c=l._subplots.cartesian;if(ue&&a.subplotsRegistry.splom.drag(e),le)for(n=0;n<c.length;n++)if(i=(r=u[c[n]]).xaxis,o=r.yaxis,r._scene){var h=s.simpleMap(i.range,i.r2l),f=s.simpleMap(o.range,o.r2l);r._scene.update({range:[h[0],f[0],h[1],f[1]]})}if((ue||le)&&(C(e),A(e)),ce){var g=t[2]/k._length,m=t[3]/L._length;for(n=0;n<c.length;n++){i=(r=u[c[n]]).xaxis,o=r.yaxis;var y,b,_,I,x=(ae||ne.isSubplotConstrained)&&!i.fixedrange&&H[i._id],w=(se||ne.isSubplotConstrained)&&!o.fixedrange&&q[o._id];if(x?(y=g,_=v||ne.isSubplotConstrained?t[0]:qe(i,y)):ne.xaHash[i._id]?(y=g,_=t[0]*i._length/k._length):ne.yaHash[i._id]?(y=m,_="ns"===ie?-t[1]*i._length/L._length:qe(i,y,{n:"top",s:"bottom"}[ie])):_=Xe(i,y=Ge(i,g,m)),w?(b=m,I=p||ne.isSubplotConstrained?t[1]:qe(o,b)):ne.yaHash[o._id]?(b=m,I=t[1]*o._length/L._length):ne.xaHash[o._id]?(b=g,I="ew"===re?-t[0]*o._length/k._length:qe(o,b,{e:"right",w:"left"}[re])):I=Xe(o,b=Ge(o,g,m)),y||b){y||(y=1),b||(b=1);var M=i._offset-_/y,S=o._offset-I/b;r.clipRect.call(d.setTranslate,_,I).call(d.setScale,y,b),r.plot.call(d.setTranslate,M,S).call(d.setScale,1/y,1/b),y===r.xScaleFactor&&b===r.yScaleFactor||(d.setPointGroupScale(r.zoomScalePts,y,b),d.setTextPointsScale(r.zoomScaleTxt,y,b)),d.hideOutsideRangePoints(r.clipOnAxisFalseTraces,r),r.xScaleFactor=y,r.yScaleFactor=b}}}}function Ge(e,t,n){return e.fixedrange?0:ae&&te.xaHash[e._id]?t:se&&(te.isSubplotConstrained?te.xaHash:te.yaHash)[e._id]?n:0}function Xe(e,t){return t?(e.range=e._r.slice(),S(e,t),qe(e,t)):0}function qe(e,t,n){return e._length*(1-t)*b[n||e.constraintoward||"middle"]}return p.length*v.length!=1&&G(be,(function(t){if(e._context._scrollZoom.cartesian||e._fullLayout._enablescrollzoom){if(ke(),e._transitioningWithDuration)return t.preventDefault(),void t.stopPropagation();me(),clearTimeout(Re);var n=-t.deltaY;if(isFinite(n)||(n=t.wheelDelta/10),isFinite(n)){var r,i=Math.exp(-Math.min(Math.max(n,-20),20)/200),o=Fe.draglayer.select(".nsewdrag").node().getBoundingClientRect(),a=(t.clientX-o.left)/o.width,l=(o.bottom-t.clientY)/o.height;if(ae){for(v||(a=.5),r=0;r<Z.length;r++)u(Z[r],a,i);He("x"),je[2]*=i,je[0]+=je[2]*a*(1/i-1)}if(se){for(p||(l=.5),r=0;r<Q.length;r++)u(Q[r],l,i);He("y"),je[3]*=i,je[1]+=je[3]*(1-l)*(1/i-1)}We(je),Ue(),e.emit("plotly_relayouting",de),Re=setTimeout((function(){je=[0,0,$,ee],Ye()}),ze),t.preventDefault()}else s.log("Did not find wheel motion attributes: ",t)}function u(e,t,n){if(!e.fixedrange){var r=s.simpleMap(e.range,e.r2l),i=r[0]+(r[1]-r[0])*t;e.range=r.map((function(t){return e.l2r(i+(t-i)*n)}))}}})),be},makeDragger:L,makeRectDragger:D,makeZoombox:z,makeCorners:F,updateZoombox:B,xyCorners:Y,transitionZoombox:H,removeZoombox:U,showDoubleClickNotifier:V,attachWheelEventHandler:G}},function(e,t,n){"use strict";t.manage=n(1701)},function(e,t,n){"use strict";var r=n(83),i=n(125),o=n(27),a=n(162).isUnifiedHover,s=n(1702),l=n(1703);e.exports=function(e){var t=e._fullLayout,n=e._context,c=t._modeBar;if(n.displayModeBar||n.watermark){if(!Array.isArray(n.modeBarButtonsToRemove))throw new Error(["*modeBarButtonsToRemove* configuration options","must be an array."].join(" "));if(!Array.isArray(n.modeBarButtonsToAdd))throw new Error(["*modeBarButtonsToAdd* configuration options","must be an array."].join(" "));var d,h=n.modeBarButtons;d=Array.isArray(h)&&h.length?function(e){for(var t=0;t<e.length;t++)for(var n=e[t],r=0;r<n.length;r++){var i=n[r];if("string"==typeof i){if(void 0===l[i])throw new Error(["*modeBarButtons* configuration options","invalid button name"].join(" "));e[t][r]=l[i]}}return e}(h):!n.displayModeBar&&n.watermark?[]:function(e){var t=e._fullLayout,n=e._fullData,s=e._context,c=s.modeBarButtonsToRemove,d=s.modeBarButtonsToAdd,h=t._has("cartesian"),f=t._has("gl3d"),p=t._has("geo"),g=t._has("pie"),v=t._has("funnelarea"),m=t._has("gl2d"),y=t._has("ternary"),b=t._has("mapbox"),C=t._has("polar"),A=t._has("sankey"),_=function(e){for(var t=r.list({_fullLayout:e},null,!0),n=0;n<t.length;n++)if(!t[n].fixedrange)return!1;return!0}(t),I=a(t.hovermode),x=[];function w(e){if(e.length){for(var t=[],n=0;n<e.length;n++){var r=e[n];-1===c.indexOf(r)&&t.push(l[r])}x.push(t)}}var M=["toImage"];s.showEditInChartStudio?M.push("editInChartStudio"):s.showSendToCloud&&M.push("sendDataToCloud");w(M);var S=[],T=[],E=[],O=[];(h||m||g||v||y)+p+f+b+C>1?(T=["toggleHover"],E=["resetViews"]):p?(S=["zoomInGeo","zoomOutGeo"],T=["hoverClosestGeo"],E=["resetGeo"]):f?(T=["hoverClosest3d"],E=["resetCameraDefault3d","resetCameraLastSave3d"]):b?(S=["zoomInMapbox","zoomOutMapbox"],T=["toggleHover"],E=["resetViewMapbox"]):m?T=["hoverClosestGl2d"]:g?T=["hoverClosestPie"]:A?(T=["hoverClosestCartesian","hoverCompareCartesian"],E=["resetViewSankey"]):T=["toggleHover"];h&&(T=["toggleSpikelines","hoverClosestCartesian","hoverCompareCartesian"]);(function(e){for(var t=0;t<e.length;t++)if(!o.traceIs(e[t],"noHover"))return!1;return!0}(n)||I)&&(T=[]);!h&&!m||_||(S=["zoomIn2d","zoomOut2d","autoScale2d"],"resetViews"!==E[0]&&(E=["resetScale2d"]));f?O=["zoom3d","pan3d","orbitRotation","tableRotation"]:(h||m)&&!_||y?O=["zoom2d","pan2d"]:b||p?O=["pan2d"]:C&&(O=["zoom2d"]);(function(e){for(var t=!1,n=0;n<e.length&&!t;n++){var r=e[n];r._module&&r._module.selectPoints&&(o.traceIs(r,"scatter-like")?(i.hasMarkers(r)||i.hasText(r))&&(t=!0):o.traceIs(r,"box-violin")&&"all"!==r.boxpoints&&"all"!==r.points||(t=!0))}return t})(n)&&O.push("select2d","lasso2d");if(Array.isArray(d)){for(var k=[],L=0;L<d.length;L++){var D=d[L];"string"==typeof D?-1!==u.indexOf(D)&&(t._has("mapbox")||t._has("cartesian"))&&O.push(D):k.push(D)}d=k}return w(O),w(S.concat(E)),w(T),function(e,t){if(t.length)if(Array.isArray(t[0]))for(var n=0;n<t.length;n++)e.push(t[n]);else e.push(t);return e}(x,d)}(e),c?c.update(e,d):t._modeBar=s(e,d)}else c&&(c.destroy(),delete t._modeBar)};var u=["drawline","drawopenpath","drawclosedpath","drawcircle","drawrect","eraseshape"]},function(e,t,n){"use strict";var r=n(39),i=n(42),o=n(10),a=n(550),s=new DOMParser;function l(e){this.container=e.container,this.element=document.createElement("div"),this.update(e.graphInfo,e.buttons),this.container.appendChild(this.element)}var u=l.prototype;u.update=function(e,t){this.graphInfo=e;var n=this.graphInfo._context,r=this.graphInfo._fullLayout,i="modebar-"+r._uid;this.element.setAttribute("id",i),this._uid=i,this.element.className="modebar","hover"===n.displayModeBar&&(this.element.className+=" modebar--hover ease-bg"),"v"===r.modebar.orientation&&(this.element.className+=" vertical",t=t.reverse());var a=r.modebar,s="hover"===n.displayModeBar?".js-plotly-plot .plotly:hover ":"";o.deleteRelatedStyleRule(i),o.addRelatedStyleRule(i,s+"#"+i+" .modebar-group","background-color: "+a.bgcolor),o.addRelatedStyleRule(i,"#"+i+" .modebar-btn .icon path","fill: "+a.color),o.addRelatedStyleRule(i,"#"+i+" .modebar-btn:hover .icon path","fill: "+a.activecolor),o.addRelatedStyleRule(i,"#"+i+" .modebar-btn.active .icon path","fill: "+a.activecolor);var l=!this.hasButtons(t),u=this.hasLogo!==n.displaylogo,c=this.locale!==n.locale;if(this.locale=n.locale,(l||u||c)&&(this.removeAllButtons(),this.updateButtons(t),n.watermark||n.displaylogo)){var d=this.getLogo();n.watermark&&(d.className=d.className+" watermark"),"v"===r.modebar.orientation?this.element.insertBefore(d,this.element.childNodes[0]):this.element.appendChild(d),this.hasLogo=!0}this.updateActiveButton()},u.updateButtons=function(e){var t=this;this.buttons=e,this.buttonElements=[],this.buttonsNames=[],this.buttons.forEach((function(e){var n=t.createGroup();e.forEach((function(e){var r=e.name;if(!r)throw new Error("must provide button 'name' in button config");if(-1!==t.buttonsNames.indexOf(r))throw new Error("button name '"+r+"' is taken");t.buttonsNames.push(r);var i=t.createButton(e);t.buttonElements.push(i),n.appendChild(i)})),t.element.appendChild(n)}))},u.createGroup=function(){var e=document.createElement("div");return e.className="modebar-group",e},u.createButton=function(e){var t=this,n=document.createElement("a");n.setAttribute("rel","tooltip"),n.className="modebar-btn";var i=e.title;void 0===i?i=e.name:"function"==typeof i&&(i=i(this.graphInfo)),(i||0===i)&&n.setAttribute("data-title",i),void 0!==e.attr&&n.setAttribute("data-attr",e.attr);var o=e.val;if(void 0!==o&&("function"==typeof o&&(o=o(this.graphInfo)),n.setAttribute("data-val",o)),"function"!=typeof e.click)throw new Error("must provide button 'click' function in button config");n.addEventListener("click",(function(n){e.click(t.graphInfo,n),t.updateActiveButton(n.currentTarget)})),n.setAttribute("data-toggle",e.toggle||!1),e.toggle&&r.select(n).classed("active",!0);var s=e.icon;return"function"==typeof s?n.appendChild(s()):n.appendChild(this.createIcon(s||a.question)),n.setAttribute("data-gravity",e.gravity||"n"),n},u.createIcon=function(e){var t,n=i(e.height)?Number(e.height):e.ascent-e.descent,r="http://www.w3.org/2000/svg";if(e.path){(t=document.createElementNS(r,"svg")).setAttribute("viewBox",[0,0,e.width,n].join(" ")),t.setAttribute("class","icon");var o=document.createElementNS(r,"path");o.setAttribute("d",e.path),e.transform?o.setAttribute("transform",e.transform):void 0!==e.ascent&&o.setAttribute("transform","matrix(1 0 0 -1 0 "+e.ascent+")"),t.appendChild(o)}e.svg&&(t=s.parseFromString(e.svg,"application/xml").childNodes[0]);return t.setAttribute("height","1em"),t.setAttribute("width","1em"),t},u.updateActiveButton=function(e){var t=this.graphInfo._fullLayout,n=void 0!==e?e.getAttribute("data-attr"):null;this.buttonElements.forEach((function(e){var i=e.getAttribute("data-val")||!0,a=e.getAttribute("data-attr"),s="true"===e.getAttribute("data-toggle"),l=r.select(e);if(s)a===n&&l.classed("active",!l.classed("active"));else{var u=null===a?a:o.nestedProperty(t,a).get();l.classed("active",u===i)}}))},u.hasButtons=function(e){var t=this.buttons;if(!t)return!1;if(e.length!==t.length)return!1;for(var n=0;n<e.length;++n){if(e[n].length!==t[n].length)return!1;for(var r=0;r<e[n].length;r++)if(e[n][r].name!==t[n][r].name)return!1}return!0},u.getLogo=function(){var e=this.createGroup(),t=document.createElement("a");return t.href="https://plotly.com/",t.target="_blank",t.setAttribute("data-title",o._(this.graphInfo,"Produced with Plotly")),t.className="modebar-btn plotlyjsicon modebar-btn--logo",t.appendChild(this.createIcon(a.newplotlylogo)),e.appendChild(t),e},u.removeAllButtons=function(){for(;this.element.firstChild;)this.element.removeChild(this.element.firstChild);this.hasLogo=!1},u.destroy=function(){o.removeElement(this.container.querySelector(".modebar")),o.deleteRelatedStyleRule(this._uid)},e.exports=function(e,t){var n=e._fullLayout,i=new l({graphInfo:e,container:n._modebardiv.node(),buttons:t});return n._privateplot&&r.select(i.element).append("span").classed("badge-private float--left",!0).text("PRIVATE"),i}},function(e,t,n){"use strict";var r=n(27),i=n(82),o=n(83),a=n(550),s=n(1001).eraseActiveShape,l=n(10),u=l._,c=e.exports={};function d(e,t){var n,i,a=t.currentTarget,s=a.getAttribute("data-attr"),l=a.getAttribute("data-val")||!0,u=e._fullLayout,c={},d=o.list(e,null,!0),h=u._cartesianSpikesEnabled;if("zoom"===s){var f,p="in"===l?.5:2,g=(1+p)/2,v=(1-p)/2;for(i=0;i<d.length;i++)if(!(n=d[i]).fixedrange)if(f=n._name,"auto"===l)c[f+".autorange"]=!0;else if("reset"===l){if(void 0===n._rangeInitial)c[f+".autorange"]=!0;else{var m=n._rangeInitial.slice();c[f+".range[0]"]=m[0],c[f+".range[1]"]=m[1]}void 0!==n._showSpikeInitial&&(c[f+".showspikes"]=n._showSpikeInitial,"on"!==h||n._showSpikeInitial||(h="off"))}else{var y=[n.r2l(n.range[0]),n.r2l(n.range[1])],b=[g*y[0]+v*y[1],g*y[1]+v*y[0]];c[f+".range[0]"]=n.l2r(b[0]),c[f+".range[1]"]=n.l2r(b[1])}}else"hovermode"!==s||"x"!==l&&"y"!==l||(l=u._isHoriz?"y":"x",a.setAttribute("data-val",l)),c[s]=l;u._cartesianSpikesEnabled=h,r.call("_guiRelayout",e,c)}function h(e,t){for(var n=t.currentTarget,i=n.getAttribute("data-attr"),o=n.getAttribute("data-val")||!0,a=e._fullLayout._subplots.gl3d||[],s={},l=i.split("."),u=0;u<a.length;u++)s[a[u]+"."+l[1]]=o;var c="pan"===o?o:"zoom";s.dragmode=c,r.call("_guiRelayout",e,s)}function f(e,t){for(var n=t.currentTarget.getAttribute("data-attr"),i="resetLastSave"===n,o="resetDefault"===n,a=e._fullLayout,s=a._subplots.gl3d||[],l={},u=0;u<s.length;u++){var c,d=s[u],h=d+".camera",f=d+".aspectratio",p=d+".aspectmode",g=a[d]._scene;i?(l[h+".up"]=g.viewInitial.up,l[h+".eye"]=g.viewInitial.eye,l[h+".center"]=g.viewInitial.center,c=!0):o&&(l[h+".up"]=null,l[h+".eye"]=null,l[h+".center"]=null,c=!0),c&&(l[f+".x"]=g.viewInitial.aspectratio.x,l[f+".y"]=g.viewInitial.aspectratio.y,l[f+".z"]=g.viewInitial.aspectratio.z,l[p]=g.viewInitial.aspectmode)}r.call("_guiRelayout",e,l)}function p(e,t){var n=t.currentTarget,r=n._previousVal,i=e._fullLayout,o=i._subplots.gl3d||[],a=["xaxis","yaxis","zaxis"],s={},l={};if(r)l=r,n._previousVal=null;else{for(var u=0;u<o.length;u++){var c=o[u],d=i[c],h=c+".hovermode";s[h]=d.hovermode,l[h]=!1;for(var f=0;f<3;f++){var p=a[f],g=c+"."+p+".showspikes";l[g]=!1,s[g]=d[p].showspikes}}n._previousVal=s}return l}function g(e,t){for(var n=t.currentTarget,i=n.getAttribute("data-attr"),o=n.getAttribute("data-val")||!0,a=e._fullLayout,s=a._subplots.geo||[],l=0;l<s.length;l++){var u=s[l],c=a[u];if("zoom"===i){var d=c.projection.scale,h="in"===o?2*d:.5*d;r.call("_guiRelayout",e,u+".projection.scale",h)}}"reset"===i&&b(e,"geo")}function v(e){var t=e._fullLayout;return!t.hovermode&&(t._has("cartesian")?t._isHoriz?"y":"x":"closest")}function m(e){var t=v(e);r.call("_guiRelayout",e,"hovermode",t)}function y(e,t){for(var n=t.currentTarget.getAttribute("data-val"),i=e._fullLayout,o=i._subplots.mapbox||[],a={},s=0;s<o.length;s++){var l=o[s],u=i[l].zoom,c="in"===n?1.05*u:u/1.05;a[l+".zoom"]=c}r.call("_guiRelayout",e,a)}function b(e,t){for(var n=e._fullLayout,i=n._subplots[t]||[],o={},a=0;a<i.length;a++)for(var s=i[a],l=n[s]._subplot.viewInitial,u=Object.keys(l),c=0;c<u.length;c++){var d=u[c];o[s+"."+d]=l[d]}r.call("_guiRelayout",e,o)}c.toImage={name:"toImage",title:function(e){var t=(e._context.toImageButtonOptions||{}).format||"png";return u(e,"png"===t?"Download plot as a png":"Download plot")},icon:a.camera,click:function(e){var t=e._context.toImageButtonOptions,n={format:t.format||"png"};l.notifier(u(e,"Taking snapshot - this may take a few seconds"),"long"),"svg"!==n.format&&l.isIE()&&(l.notifier(u(e,"IE only supports svg.  Changing format to svg."),"long"),n.format="svg"),["filename","width","height","scale"].forEach((function(e){e in t&&(n[e]=t[e])})),r.call("downloadImage",e,n).then((function(t){l.notifier(u(e,"Snapshot succeeded")+" - "+t,"long")})).catch((function(){l.notifier(u(e,"Sorry, there was a problem downloading your snapshot!"),"long")}))}},c.sendDataToCloud={name:"sendDataToCloud",title:function(e){return u(e,"Edit in Chart Studio")},icon:a.disk,click:function(e){i.sendDataToCloud(e)}},c.editInChartStudio={name:"editInChartStudio",title:function(e){return u(e,"Edit in Chart Studio")},icon:a.pencil,click:function(e){i.sendDataToCloud(e)}},c.zoom2d={name:"zoom2d",title:function(e){return u(e,"Zoom")},attr:"dragmode",val:"zoom",icon:a.zoombox,click:d},c.pan2d={name:"pan2d",title:function(e){return u(e,"Pan")},attr:"dragmode",val:"pan",icon:a.pan,click:d},c.select2d={name:"select2d",title:function(e){return u(e,"Box Select")},attr:"dragmode",val:"select",icon:a.selectbox,click:d},c.lasso2d={name:"lasso2d",title:function(e){return u(e,"Lasso Select")},attr:"dragmode",val:"lasso",icon:a.lasso,click:d},c.drawclosedpath={name:"drawclosedpath",title:function(e){return u(e,"Draw closed freeform")},attr:"dragmode",val:"drawclosedpath",icon:a.drawclosedpath,click:d},c.drawopenpath={name:"drawopenpath",title:function(e){return u(e,"Draw open freeform")},attr:"dragmode",val:"drawopenpath",icon:a.drawopenpath,click:d},c.drawline={name:"drawline",title:function(e){return u(e,"Draw line")},attr:"dragmode",val:"drawline",icon:a.drawline,click:d},c.drawrect={name:"drawrect",title:function(e){return u(e,"Draw rectangle")},attr:"dragmode",val:"drawrect",icon:a.drawrect,click:d},c.drawcircle={name:"drawcircle",title:function(e){return u(e,"Draw circle")},attr:"dragmode",val:"drawcircle",icon:a.drawcircle,click:d},c.eraseshape={name:"eraseshape",title:function(e){return u(e,"Erase active shape")},icon:a.eraseshape,click:s},c.zoomIn2d={name:"zoomIn2d",title:function(e){return u(e,"Zoom in")},attr:"zoom",val:"in",icon:a.zoom_plus,click:d},c.zoomOut2d={name:"zoomOut2d",title:function(e){return u(e,"Zoom out")},attr:"zoom",val:"out",icon:a.zoom_minus,click:d},c.autoScale2d={name:"autoScale2d",title:function(e){return u(e,"Autoscale")},attr:"zoom",val:"auto",icon:a.autoscale,click:d},c.resetScale2d={name:"resetScale2d",title:function(e){return u(e,"Reset axes")},attr:"zoom",val:"reset",icon:a.home,click:d},c.hoverClosestCartesian={name:"hoverClosestCartesian",title:function(e){return u(e,"Show closest data on hover")},attr:"hovermode",val:"closest",icon:a.tooltip_basic,gravity:"ne",click:d},c.hoverCompareCartesian={name:"hoverCompareCartesian",title:function(e){return u(e,"Compare data on hover")},attr:"hovermode",val:function(e){return e._fullLayout._isHoriz?"y":"x"},icon:a.tooltip_compare,gravity:"ne",click:d},c.zoom3d={name:"zoom3d",title:function(e){return u(e,"Zoom")},attr:"scene.dragmode",val:"zoom",icon:a.zoombox,click:h},c.pan3d={name:"pan3d",title:function(e){return u(e,"Pan")},attr:"scene.dragmode",val:"pan",icon:a.pan,click:h},c.orbitRotation={name:"orbitRotation",title:function(e){return u(e,"Orbital rotation")},attr:"scene.dragmode",val:"orbit",icon:a["3d_rotate"],click:h},c.tableRotation={name:"tableRotation",title:function(e){return u(e,"Turntable rotation")},attr:"scene.dragmode",val:"turntable",icon:a["z-axis"],click:h},c.resetCameraDefault3d={name:"resetCameraDefault3d",title:function(e){return u(e,"Reset camera to default")},attr:"resetDefault",icon:a.home,click:f},c.resetCameraLastSave3d={name:"resetCameraLastSave3d",title:function(e){return u(e,"Reset camera to last save")},attr:"resetLastSave",icon:a.movie,click:f},c.hoverClosest3d={name:"hoverClosest3d",title:function(e){return u(e,"Toggle show closest data on hover")},attr:"hovermode",val:null,toggle:!0,icon:a.tooltip_basic,gravity:"ne",click:function(e,t){var n=p(e,t);r.call("_guiRelayout",e,n)}},c.zoomInGeo={name:"zoomInGeo",title:function(e){return u(e,"Zoom in")},attr:"zoom",val:"in",icon:a.zoom_plus,click:g},c.zoomOutGeo={name:"zoomOutGeo",title:function(e){return u(e,"Zoom out")},attr:"zoom",val:"out",icon:a.zoom_minus,click:g},c.resetGeo={name:"resetGeo",title:function(e){return u(e,"Reset")},attr:"reset",val:null,icon:a.autoscale,click:g},c.hoverClosestGeo={name:"hoverClosestGeo",title:function(e){return u(e,"Toggle show closest data on hover")},attr:"hovermode",val:null,toggle:!0,icon:a.tooltip_basic,gravity:"ne",click:m},c.hoverClosestGl2d={name:"hoverClosestGl2d",title:function(e){return u(e,"Toggle show closest data on hover")},attr:"hovermode",val:null,toggle:!0,icon:a.tooltip_basic,gravity:"ne",click:m},c.hoverClosestPie={name:"hoverClosestPie",title:function(e){return u(e,"Toggle show closest data on hover")},attr:"hovermode",val:"closest",icon:a.tooltip_basic,gravity:"ne",click:m},c.resetViewSankey={name:"resetSankeyGroup",title:function(e){return u(e,"Reset view")},icon:a.home,click:function(e){for(var t={"node.groups":[],"node.x":[],"node.y":[]},n=0;n<e._fullData.length;n++){var i=e._fullData[n]._viewInitial;t["node.groups"].push(i.node.groups.slice()),t["node.x"].push(i.node.x.slice()),t["node.y"].push(i.node.y.slice())}r.call("restyle",e,t)}},c.toggleHover={name:"toggleHover",title:function(e){return u(e,"Toggle show closest data on hover")},attr:"hovermode",val:null,toggle:!0,icon:a.tooltip_basic,gravity:"ne",click:function(e,t){var n=p(e,t);n.hovermode=v(e),r.call("_guiRelayout",e,n)}},c.resetViews={name:"resetViews",title:function(e){return u(e,"Reset views")},icon:a.home,click:function(e,t){var n=t.currentTarget;n.setAttribute("data-attr","zoom"),n.setAttribute("data-val","reset"),d(e,t),n.setAttribute("data-attr","resetLastSave"),f(e,t),b(e,"geo"),b(e,"mapbox")}},c.toggleSpikelines={name:"toggleSpikelines",title:function(e){return u(e,"Toggle Spike Lines")},icon:a.spikeline,attr:"_cartesianSpikesEnabled",val:"on",click:function(e){var t=e._fullLayout,n=t._cartesianSpikesEnabled;t._cartesianSpikesEnabled="on"===n?"off":"on",r.call("_guiRelayout",e,function(e){for(var t="on"===e._fullLayout._cartesianSpikesEnabled,n=o.list(e,null,!0),r={},i=0;i<n.length;i++){var a=n[i];r[a._name+".showspikes"]=!!t||a._showSpikeInitial}return r}(e))}},c.resetViewMapbox={name:"resetViewMapbox",title:function(e){return u(e,"Reset view")},attr:"reset",icon:a.home,click:function(e){b(e,"mapbox")}},c.zoomInMapbox={name:"zoomInMapbox",title:function(e){return u(e,"Zoom in")},attr:"zoom",val:"in",icon:a.zoom_plus,click:y},c.zoomOutMapbox={name:"zoomOutMapbox",title:function(e){return u(e,"Zoom out")},attr:"zoom",val:"out",icon:a.zoom_minus,click:y}},function(e,t){e.exports=function(e){var t=[];return e.replace(r,(function(e,r,o){var a=r.toLowerCase();for(o=function(e){var t=e.match(i);return t?t.map(Number):[]}(o),"m"==a&&o.length>2&&(t.push([r].concat(o.splice(0,2))),a="l",r="m"==r?"l":"L");;){if(o.length==n[a])return o.unshift(r),t.push(o);if(o.length<n[a])throw new Error("malformed path data");t.push([r].concat(o.splice(0,n[a])))}})),t};var n={a:7,c:6,h:1,l:2,m:2,q:4,s:4,t:2,v:1,z:0},r=/([astvzqmhlc])([^astvzqmhlc]*)/gi;var i=/-?[0-9]*\.?[0-9]+(?:e[-+]?\d+)?/gi},function(e,t,n){
/*
 * @copyright 2016 Sean Connelly (@voidqk), http://syntheti.cc
 * @license MIT
 * @preserve Project Home: https://github.com/voidqk/polybooljs
 */
var r,i=n(1706),o=n(1707),a=n(1708),s=n(1710),l=n(1711),u=n(1712),c=!1,d=o();function h(e,t,n){var i=r.segments(e),o=r.segments(t),a=n(r.combine(i,o));return r.polygon(a)}r={buildLog:function(e){return!0===e?c=i():!1===e&&(c=!1),!1!==c&&c.list},epsilon:function(e){return d.epsilon(e)},segments:function(e){var t=a(!0,d,c);return e.regions.forEach(t.addRegion),{segments:t.calculate(e.inverted),inverted:e.inverted}},combine:function(e,t){return{combined:a(!1,d,c).calculate(e.segments,e.inverted,t.segments,t.inverted),inverted1:e.inverted,inverted2:t.inverted}},selectUnion:function(e){return{segments:l.union(e.combined,c),inverted:e.inverted1||e.inverted2}},selectIntersect:function(e){return{segments:l.intersect(e.combined,c),inverted:e.inverted1&&e.inverted2}},selectDifference:function(e){return{segments:l.difference(e.combined,c),inverted:e.inverted1&&!e.inverted2}},selectDifferenceRev:function(e){return{segments:l.differenceRev(e.combined,c),inverted:!e.inverted1&&e.inverted2}},selectXor:function(e){return{segments:l.xor(e.combined,c),inverted:e.inverted1!==e.inverted2}},polygon:function(e){return{regions:s(e.segments,d,c),inverted:e.inverted}},polygonFromGeoJSON:function(e){return u.toPolygon(r,e)},polygonToGeoJSON:function(e){return u.fromPolygon(r,d,e)},union:function(e,t){return h(e,t,r.selectUnion)},intersect:function(e,t){return h(e,t,r.selectIntersect)},difference:function(e,t){return h(e,t,r.selectDifference)},differenceRev:function(e,t){return h(e,t,r.selectDifferenceRev)},xor:function(e,t){return h(e,t,r.selectXor)}},"object"==typeof window&&(window.PolyBool=r),e.exports=r},function(e,t){e.exports=function(){var e,t=0,n=!1;function r(t,n){return e.list.push({type:t,data:n?JSON.parse(JSON.stringify(n)):void 0}),e}return e={list:[],segmentId:function(){return t++},checkIntersection:function(e,t){return r("check",{seg1:e,seg2:t})},segmentChop:function(e,t){return r("div_seg",{seg:e,pt:t}),r("chop",{seg:e,pt:t})},statusRemove:function(e){return r("pop_seg",{seg:e})},segmentUpdate:function(e){return r("seg_update",{seg:e})},segmentNew:function(e,t){return r("new_seg",{seg:e,primary:t})},segmentRemove:function(e){return r("rem_seg",{seg:e})},tempStatus:function(e,t,n){return r("temp_status",{seg:e,above:t,below:n})},rewind:function(e){return r("rewind",{seg:e})},status:function(e,t,n){return r("status",{seg:e,above:t,below:n})},vert:function(t){return t===n?e:(n=t,r("vert",{x:t}))},log:function(e){return"string"!=typeof e&&(e=JSON.stringify(e,!1,"  ")),r("log",{txt:e})},reset:function(){return r("reset")},selected:function(e){return r("selected",{segs:e})},chainStart:function(e){return r("chain_start",{seg:e})},chainRemoveHead:function(e,t){return r("chain_rem_head",{index:e,pt:t})},chainRemoveTail:function(e,t){return r("chain_rem_tail",{index:e,pt:t})},chainNew:function(e,t){return r("chain_new",{pt1:e,pt2:t})},chainMatch:function(e){return r("chain_match",{index:e})},chainClose:function(e){return r("chain_close",{index:e})},chainAddHead:function(e,t){return r("chain_add_head",{index:e,pt:t})},chainAddTail:function(e,t){return r("chain_add_tail",{index:e,pt:t})},chainConnect:function(e,t){return r("chain_con",{index1:e,index2:t})},chainReverse:function(e){return r("chain_rev",{index:e})},chainJoin:function(e,t){return r("chain_join",{index1:e,index2:t})},done:function(){return r("done")}}}},function(e,t){e.exports=function(e){"number"!=typeof e&&(e=1e-10);var t={epsilon:function(t){return"number"==typeof t&&(e=t),e},pointAboveOrOnLine:function(t,n,r){var i=n[0],o=n[1],a=r[0],s=r[1],l=t[0];return(a-i)*(t[1]-o)-(s-o)*(l-i)>=-e},pointBetween:function(t,n,r){var i=t[1]-n[1],o=r[0]-n[0],a=t[0]-n[0],s=r[1]-n[1],l=a*o+i*s;return!(l<e)&&!(l-(o*o+s*s)>-e)},pointsSameX:function(t,n){return Math.abs(t[0]-n[0])<e},pointsSameY:function(t,n){return Math.abs(t[1]-n[1])<e},pointsSame:function(e,n){return t.pointsSameX(e,n)&&t.pointsSameY(e,n)},pointsCompare:function(e,n){return t.pointsSameX(e,n)?t.pointsSameY(e,n)?0:e[1]<n[1]?-1:1:e[0]<n[0]?-1:1},pointsCollinear:function(t,n,r){var i=t[0]-n[0],o=t[1]-n[1],a=n[0]-r[0],s=n[1]-r[1];return Math.abs(i*s-a*o)<e},linesIntersect:function(t,n,r,i){var o=n[0]-t[0],a=n[1]-t[1],s=i[0]-r[0],l=i[1]-r[1],u=o*l-a*s;if(Math.abs(u)<e)return!1;var c=t[0]-r[0],d=t[1]-r[1],h=(s*d-l*c)/u,f=(o*d-a*c)/u,p={alongA:0,alongB:0,pt:[t[0]+h*o,t[1]+h*a]};return p.alongA=h<=-e?-2:h<e?-1:h-1<=-e?0:h-1<e?1:2,p.alongB=f<=-e?-2:f<e?-1:f-1<=-e?0:f-1<e?1:2,p},pointInsideRegion:function(t,n){for(var r=t[0],i=t[1],o=n[n.length-1][0],a=n[n.length-1][1],s=!1,l=0;l<n.length;l++){var u=n[l][0],c=n[l][1];c-i>e!=a-i>e&&(o-u)*(i-c)/(a-c)+u-r>e&&(s=!s),o=u,a=c}return s}};return t}},function(e,t,n){var r=n(1709);e.exports=function(e,t,n){function i(e,t,r){return{id:n?n.segmentId():-1,start:e,end:t,myFill:{above:r.myFill.above,below:r.myFill.below},otherFill:null}}var o=r.create();function a(e,n){o.insertBefore(e,(function(r){return function(e,n,r,i,o,a){var s=t.pointsCompare(n,o);return 0!==s?s:t.pointsSame(r,a)?0:e!==i?e?1:-1:t.pointAboveOrOnLine(r,i?o:a,i?a:o)?1:-1}(e.isStart,e.pt,n,r.isStart,r.pt,r.other.pt)<0}))}function s(e,t){var n=function(e,t){var n=r.node({isStart:!0,pt:e.start,seg:e,primary:t,other:null,status:null});return a(n,e.end),n}(e,t);return function(e,t,n){var i=r.node({isStart:!1,pt:t.end,seg:t,primary:n,other:e,status:null});e.other=i,a(i,e.pt)}(n,e,t),n}function l(e,t){var r=i(t,e.seg.end,e.seg);return function(e,t){n&&n.segmentChop(e.seg,t),e.other.remove(),e.seg.end=t,e.other.pt=t,a(e.other,e.pt)}(e,t),s(r,e.primary)}function u(i,a){var s=r.create();function u(e){return s.findTransition((function(n){var r,i,o,a,s,l;return(r=e,i=n.ev,o=r.seg.start,a=r.seg.end,s=i.seg.start,l=i.seg.end,t.pointsCollinear(o,s,l)?t.pointsCollinear(a,s,l)||t.pointAboveOrOnLine(a,s,l)?1:-1:t.pointAboveOrOnLine(o,s,l)?1:-1)>0}))}function c(e,r){var i=e.seg,o=r.seg,a=i.start,s=i.end,u=o.start,c=o.end;n&&n.checkIntersection(i,o);var d=t.linesIntersect(a,s,u,c);if(!1===d){if(!t.pointsCollinear(a,s,u))return!1;if(t.pointsSame(a,c)||t.pointsSame(s,u))return!1;var h=t.pointsSame(a,u),f=t.pointsSame(s,c);if(h&&f)return r;var p=!h&&t.pointBetween(a,u,c),g=!f&&t.pointBetween(s,u,c);if(h)return g?l(r,s):l(e,c),r;p&&(f||(g?l(r,s):l(e,c)),l(r,a))}else 0===d.alongA&&(-1===d.alongB?l(e,u):0===d.alongB?l(e,d.pt):1===d.alongB&&l(e,c)),0===d.alongB&&(-1===d.alongA?l(r,a):0===d.alongA?l(r,d.pt):1===d.alongA&&l(r,s));return!1}for(var d=[];!o.isEmpty();){var h=o.getHead();if(n&&n.vert(h.pt[0]),h.isStart){n&&n.segmentNew(h.seg,h.primary);var f=u(h),p=f.before?f.before.ev:null,g=f.after?f.after.ev:null;function v(){if(p){var e=c(h,p);if(e)return e}return!!g&&c(h,g)}n&&n.tempStatus(h.seg,!!p&&p.seg,!!g&&g.seg);var m,y=v();if(y){var b;if(e)(b=null===h.seg.myFill.below||h.seg.myFill.above!==h.seg.myFill.below)&&(y.seg.myFill.above=!y.seg.myFill.above);else y.seg.otherFill=h.seg.myFill;n&&n.segmentUpdate(y.seg),h.other.remove(),h.remove()}if(o.getHead()!==h){n&&n.rewind(h.seg);continue}if(e)b=null===h.seg.myFill.below||h.seg.myFill.above!==h.seg.myFill.below,h.seg.myFill.below=g?g.seg.myFill.above:i,h.seg.myFill.above=b?!h.seg.myFill.below:h.seg.myFill.below;else if(null===h.seg.otherFill)m=g?h.primary===g.primary?g.seg.otherFill.above:g.seg.myFill.above:h.primary?a:i,h.seg.otherFill={above:m,below:m};n&&n.status(h.seg,!!p&&p.seg,!!g&&g.seg),h.other.status=f.insert(r.node({ev:h}))}else{var C=h.status;if(null===C)throw new Error("PolyBool: Zero-length segment detected; your epsilon is probably too small or too large");if(s.exists(C.prev)&&s.exists(C.next)&&c(C.prev.ev,C.next.ev),n&&n.statusRemove(C.ev.seg),C.remove(),!h.primary){var A=h.seg.myFill;h.seg.myFill=h.seg.otherFill,h.seg.otherFill=A}d.push(h.seg)}o.getHead().remove()}return n&&n.done(),d}return e?{addRegion:function(e){for(var r,i,o,a=e[e.length-1],l=0;l<e.length;l++){r=a,a=e[l];var u=t.pointsCompare(r,a);0!==u&&s((i=u<0?r:a,o=u<0?a:r,{id:n?n.segmentId():-1,start:i,end:o,myFill:{above:null,below:null},otherFill:null}),!0)}},calculate:function(e){return u(e,!1)}}:{calculate:function(e,t,n,r){return e.forEach((function(e){s(i(e.start,e.end,e),!0)})),n.forEach((function(e){s(i(e.start,e.end,e),!1)})),u(t,r)}}}},function(e,t){e.exports={create:function(){var e={root:{root:!0,next:null},exists:function(t){return null!==t&&t!==e.root},isEmpty:function(){return null===e.root.next},getHead:function(){return e.root.next},insertBefore:function(t,n){for(var r=e.root,i=e.root.next;null!==i;){if(n(i))return t.prev=i.prev,t.next=i,i.prev.next=t,void(i.prev=t);r=i,i=i.next}r.next=t,t.prev=r,t.next=null},findTransition:function(t){for(var n=e.root,r=e.root.next;null!==r&&!t(r);)n=r,r=r.next;return{before:n===e.root?null:n,after:r,insert:function(e){return e.prev=n,e.next=r,n.next=e,null!==r&&(r.prev=e),e}}}};return e},node:function(e){return e.prev=null,e.next=null,e.remove=function(){e.prev.next=e.next,e.next&&(e.next.prev=e.prev),e.prev=null,e.next=null},e}}},function(e,t){e.exports=function(e,t,n){var r=[],i=[];return e.forEach((function(e){var o=e.start,a=e.end;if(t.pointsSame(o,a))console.warn("PolyBool: Warning: Zero-length segment detected; your epsilon is probably too small or too large");else{n&&n.chainStart(e);for(var s={index:0,matches_head:!1,matches_pt1:!1},l={index:0,matches_head:!1,matches_pt1:!1},u=s,c=0;c<r.length;c++){var d=(v=r[c])[0],h=(v[1],v[v.length-1]);v[v.length-2];if(t.pointsSame(d,o)){if(x(c,!0,!0))break}else if(t.pointsSame(d,a)){if(x(c,!0,!1))break}else if(t.pointsSame(h,o)){if(x(c,!1,!0))break}else if(t.pointsSame(h,a)&&x(c,!1,!1))break}if(u===s)return r.push([o,a]),void(n&&n.chainNew(o,a));if(u===l){n&&n.chainMatch(s.index);var f=s.index,p=s.matches_pt1?a:o,g=s.matches_head,v=r[f],m=g?v[0]:v[v.length-1],y=g?v[1]:v[v.length-2],b=g?v[v.length-1]:v[0],C=g?v[v.length-2]:v[1];return t.pointsCollinear(y,m,p)&&(g?(n&&n.chainRemoveHead(s.index,p),v.shift()):(n&&n.chainRemoveTail(s.index,p),v.pop()),m=y),t.pointsSame(b,p)?(r.splice(f,1),t.pointsCollinear(C,b,m)&&(g?(n&&n.chainRemoveTail(s.index,m),v.pop()):(n&&n.chainRemoveHead(s.index,m),v.shift())),n&&n.chainClose(s.index),void i.push(v)):void(g?(n&&n.chainAddHead(s.index,p),v.unshift(p)):(n&&n.chainAddTail(s.index,p),v.push(p)))}var A=s.index,_=l.index;n&&n.chainConnect(A,_);var I=r[A].length<r[_].length;s.matches_head?l.matches_head?I?(w(A),M(A,_)):(w(_),M(_,A)):M(_,A):l.matches_head?M(A,_):I?(w(A),M(_,A)):(w(_),M(A,_))}function x(e,t,n){return u.index=e,u.matches_head=t,u.matches_pt1=n,u===s?(u=l,!1):(u=null,!0)}function w(e){n&&n.chainReverse(e),r[e].reverse()}function M(e,i){var o=r[e],a=r[i],s=o[o.length-1],l=o[o.length-2],u=a[0],c=a[1];t.pointsCollinear(l,s,u)&&(n&&n.chainRemoveTail(e,s),o.pop(),s=l),t.pointsCollinear(s,u,c)&&(n&&n.chainRemoveHead(i,u),a.shift()),n&&n.chainJoin(e,i),r[e]=o.concat(a),r.splice(i,1)}})),i}},function(e,t){function n(e,t,n){var r=[];return e.forEach((function(e){var i=(e.myFill.above?8:0)+(e.myFill.below?4:0)+(e.otherFill&&e.otherFill.above?2:0)+(e.otherFill&&e.otherFill.below?1:0);0!==t[i]&&r.push({id:n?n.segmentId():-1,start:e.start,end:e.end,myFill:{above:1===t[i],below:2===t[i]},otherFill:null})})),n&&n.selected(r),r}var r={union:function(e,t){return n(e,[0,2,1,0,2,2,0,0,1,0,1,0,0,0,0,0],t)},intersect:function(e,t){return n(e,[0,0,0,0,0,2,0,2,0,0,1,1,0,2,1,0],t)},difference:function(e,t){return n(e,[0,0,0,0,2,0,2,0,1,1,0,0,0,1,2,0],t)},differenceRev:function(e,t){return n(e,[0,2,1,0,0,0,1,1,0,2,0,2,0,0,0,0],t)},xor:function(e,t){return n(e,[0,2,1,0,2,0,0,1,1,0,0,2,0,1,2,0],t)}};e.exports=r},function(e,t){var n={toPolygon:function(e,t){function n(t){if(t.length<=0)return e.segments({inverted:!1,regions:[]});function n(t){var n=t.slice(0,t.length-1);return e.segments({inverted:!1,regions:[n]})}for(var r=n(t[0]),i=1;i<t.length;i++)r=e.selectDifference(e.combine(r,n(t[i])));return r}if("Polygon"===t.type)return e.polygon(n(t.coordinates));if("MultiPolygon"===t.type){for(var r=e.segments({inverted:!1,regions:[]}),i=0;i<t.coordinates.length;i++)r=e.selectUnion(e.combine(r,n(t.coordinates[i])));return e.polygon(r)}throw new Error("PolyBool: Cannot convert GeoJSON object to PolyBool polygon")},fromPolygon:function(e,t,n){function r(e,n){return t.pointInsideRegion([.5*(e[0][0]+e[1][0]),.5*(e[0][1]+e[1][1])],n)}function i(e){return{region:e,children:[]}}n=e.polygon(e.segments(n));var o=i(null);function a(e,t){for(var n=0;n<e.children.length;n++){if(r(t,(s=e.children[n]).region))return void a(s,t)}var o=i(t);for(n=0;n<e.children.length;n++){var s;r((s=e.children[n]).region,t)&&(o.children.push(s),e.children.splice(n,1),n--)}e.children.push(o)}for(var s=0;s<n.regions.length;s++){var l=n.regions[s];l.length<3||a(o,l)}function u(e,t){for(var n=0,r=e[e.length-1][0],i=e[e.length-1][1],o=[],a=0;a<e.length;a++){var s=e[a][0],l=e[a][1];o.push([s,l]),n+=l*r-s*i,r=s,i=l}return n<0!==t&&o.reverse(),o.push([o[0][0],o[0][1]]),o}var c=[];function d(e){var t=[u(e.region,!1)];c.push(t);for(var n=0;n<e.children.length;n++)t.push(h(e.children[n]))}function h(e){for(var t=0;t<e.children.length;t++)d(e.children[t]);return u(e.region,!0)}for(s=0;s<o.children.length;s++)d(o.children[s]);return c.length<=0?{type:"Polygon",coordinates:[]}:1==c.length?{type:"Polygon",coordinates:c[0]}:{type:"MultiPolygon",coordinates:c}}};e.exports=n},function(e,t,n){"use strict";var r=n(292),i=n(526),o=n(252),a=n(538).sorterAsc,s=n(27);t.containerArrayMatch=n(1714);var l=t.isAddVal=function(e){return"add"===e||r(e)},u=t.isRemoveVal=function(e){return null===e||"remove"===e};t.applyContainerArrayChanges=function(e,t,n,r,c){var d=t.astr,h=s.getComponentMethod(d,"supplyLayoutDefaults"),f=s.getComponentMethod(d,"draw"),p=s.getComponentMethod(d,"drawOne"),g=r.replot||r.recalc||h===i||f===i,v=e.layout,m=e._fullLayout;if(n[""]){Object.keys(n).length>1&&o.warn("Full array edits are incompatible with other edits",d);var y=n[""][""];if(u(y))t.set(null);else{if(!Array.isArray(y))return o.warn("Unrecognized full array edit value",d,y),!0;t.set(y)}return!g&&(h(v,m),f(e),!0)}var b,C,A,_,I,x,w,M,S=Object.keys(n).map(Number).sort(a),T=t.get(),E=T||[],O=c(m,d).get(),k=[],L=-1,D=E.length;for(b=0;b<S.length;b++)if(_=n[A=S[b]],I=Object.keys(_),x=_[""],w=l(x),A<0||A>E.length-(w?0:1))o.warn("index out of range",d,A);else if(void 0!==x)I.length>1&&o.warn("Insertion & removal are incompatible with edits to the same index.",d,A),u(x)?k.push(A):w?("add"===x&&(x={}),E.splice(A,0,x),O&&O.splice(A,0,{})):o.warn("Unrecognized full object edit value",d,A,x),-1===L&&(L=A);else for(C=0;C<I.length;C++)M=d+"["+A+"].",c(E[A],I[C],M).set(_[I[C]]);for(b=k.length-1;b>=0;b--)E.splice(k[b],1),O&&O.splice(k[b],1);if(E.length?T||t.set(E):t.set(null),g)return!1;if(h(v,m),p!==i){var P;if(-1===L)P=S;else{for(D=Math.max(E.length,D),P=[],b=0;b<S.length&&!((A=S[b])>=L);b++)P.push(A);for(b=L;b<D;b++)P.push(b)}for(b=0;b<P.length;b++)p(e,P[b])}else f(e);return!0}},function(e,t,n){"use strict";var r=n(27);e.exports=function(e){for(var t,n,i=r.layoutArrayContainers,o=r.layoutArrayRegexes,a=e.split("[")[0],s=0;s<o.length;s++)if((n=e.match(o[s]))&&0===n.index){t=n[0];break}if(t||(t=i[i.indexOf(a)]),!t)return!1;var l=e.substr(t.length);return l?!!(n=l.match(/^\[(0|[1-9][0-9]*)\](\.(.+))?$/))&&{array:t,index:Number(n[1]),property:n[3]||""}:{array:t,index:"",property:""}}},function(e,t,n){"use strict";var r=n(42),i=n(535),o=n(27),a=n(10),s=n(82),l=n(83),u=n(41),c=l.cleanId,d=l.getFromTrace,h=o.traceIs;function f(e,t){var n=e[t],r=t.charAt(0);n&&"paper"!==n&&(e[t]=c(n,r,!0))}function p(e){function t(t,n){var r=e[t],i=e.title&&e.title[n];r&&!i&&(e.title||(e.title={}),e.title[n]=e[t],delete e[t])}e&&("string"!=typeof e.title&&"number"!=typeof e.title||(e.title={text:e.title}),t("titlefont","font"),t("titleposition","position"),t("titleside","side"),t("titleoffset","offset"))}function g(e){if(!a.isPlainObject(e))return!1;var t=e.name;return delete e.name,delete e.showlegend,("string"==typeof t||"number"==typeof t)&&String(t)}function v(e,t,n,r){if(n&&!r)return e;if(r&&!n)return t;if(!e.trim())return t;if(!t.trim())return e;var i,o=Math.min(e.length,t.length);for(i=0;i<o&&e.charAt(i)===t.charAt(i);i++);return e.substr(0,i).trim()}function m(e){var t="middle",n="center";return"string"==typeof e&&(-1!==e.indexOf("top")?t="top":-1!==e.indexOf("bottom")&&(t="bottom"),-1!==e.indexOf("left")?n="left":-1!==e.indexOf("right")&&(n="right")),t+" "+n}function y(e,t){return t in e&&"object"==typeof e[t]&&0===Object.keys(e[t]).length}t.clearPromiseQueue=function(e){Array.isArray(e._promises)&&e._promises.length>0&&a.log("Clearing previous rejected promises from queue."),e._promises=[]},t.cleanLayout=function(e){var n,r;e||(e={}),e.xaxis1&&(e.xaxis||(e.xaxis=e.xaxis1),delete e.xaxis1),e.yaxis1&&(e.yaxis||(e.yaxis=e.yaxis1),delete e.yaxis1),e.scene1&&(e.scene||(e.scene=e.scene1),delete e.scene1);var o=(s.subplotsRegistry.cartesian||{}).attrRegex,l=(s.subplotsRegistry.polar||{}).attrRegex,d=(s.subplotsRegistry.ternary||{}).attrRegex,h=(s.subplotsRegistry.gl3d||{}).attrRegex,g=Object.keys(e);for(n=0;n<g.length;n++){var v=g[n];if(o&&o.test(v)){var m=e[v];m.anchor&&"free"!==m.anchor&&(m.anchor=c(m.anchor)),m.overlaying&&(m.overlaying=c(m.overlaying)),m.type||(m.isdate?m.type="date":m.islog?m.type="log":!1===m.isdate&&!1===m.islog&&(m.type="linear")),"withzero"!==m.autorange&&"tozero"!==m.autorange||(m.autorange=!0,m.rangemode="tozero"),delete m.islog,delete m.isdate,delete m.categories,y(m,"domain")&&delete m.domain,void 0!==m.autotick&&(void 0===m.tickmode&&(m.tickmode=m.autotick?"auto":"linear"),delete m.autotick),p(m)}else if(l&&l.test(v)){p(e[v].radialaxis)}else if(d&&d.test(v)){var b=e[v];p(b.aaxis),p(b.baxis),p(b.caxis)}else if(h&&h.test(v)){var C=e[v],A=C.cameraposition;if(Array.isArray(A)&&4===A[0].length){var _=A[0],I=A[1],x=A[2],w=i([],_),M=[];for(r=0;r<3;++r)M[r]=I[r]+x*w[2+4*r];C.camera={eye:{x:M[0],y:M[1],z:M[2]},center:{x:I[0],y:I[1],z:I[2]},up:{x:0,y:0,z:1}},delete C.cameraposition}p(C.xaxis),p(C.yaxis),p(C.zaxis)}}var S=Array.isArray(e.annotations)?e.annotations.length:0;for(n=0;n<S;n++){var T=e.annotations[n];a.isPlainObject(T)&&(T.ref&&("paper"===T.ref?(T.xref="paper",T.yref="paper"):"data"===T.ref&&(T.xref="x",T.yref="y"),delete T.ref),f(T,"xref"),f(T,"yref"))}var E=Array.isArray(e.shapes)?e.shapes.length:0;for(n=0;n<E;n++){var O=e.shapes[n];a.isPlainObject(O)&&(f(O,"xref"),f(O,"yref"))}var k=Array.isArray(e.images)?e.images.length:0;for(n=0;n<k;n++){var L=e.images[n];a.isPlainObject(L)&&(f(L,"xref"),f(L,"yref"))}var D=e.legend;return D&&(D.x>3?(D.x=1.02,D.xanchor="left"):D.x<-2&&(D.x=-.02,D.xanchor="right"),D.y>3?(D.y=1.02,D.yanchor="bottom"):D.y<-2&&(D.y=-.02,D.yanchor="top")),p(e),"rotate"===e.dragmode&&(e.dragmode="orbit"),u.clean(e),e.template&&e.template.layout&&t.cleanLayout(e.template.layout),e},t.cleanData=function(e){for(var n=0;n<e.length;n++){var r,i=e[n];if("histogramy"===i.type&&"xbins"in i&&!("ybins"in i)&&(i.ybins=i.xbins,delete i.xbins),i.error_y&&"opacity"in i.error_y){var l=u.defaults,d=i.error_y.color||(h(i,"bar")?u.defaultLine:l[n%l.length]);i.error_y.color=u.addOpacity(u.rgb(d),u.opacity(d)*i.error_y.opacity),delete i.error_y.opacity}if("bardir"in i&&("h"!==i.bardir||!h(i,"bar")&&"histogram"!==i.type.substr(0,9)||(i.orientation="h",t.swapXYData(i)),delete i.bardir),"histogramy"===i.type&&t.swapXYData(i),"histogramx"!==i.type&&"histogramy"!==i.type||(i.type="histogram"),"scl"in i&&!("colorscale"in i)&&(i.colorscale=i.scl,delete i.scl),"reversescl"in i&&!("reversescale"in i)&&(i.reversescale=i.reversescl,delete i.reversescl),i.xaxis&&(i.xaxis=c(i.xaxis,"x")),i.yaxis&&(i.yaxis=c(i.yaxis,"y")),h(i,"gl3d")&&i.scene&&(i.scene=s.subplotsRegistry.gl3d.cleanId(i.scene)),!h(i,"pie-like")&&!h(i,"bar-like"))if(Array.isArray(i.textposition))for(r=0;r<i.textposition.length;r++)i.textposition[r]=m(i.textposition[r]);else i.textposition&&(i.textposition=m(i.textposition));var f=o.getModule(i);if(f&&f.colorbar){var b=f.colorbar.container,C=b?i[b]:i;C&&C.colorscale&&("YIGnBu"===C.colorscale&&(C.colorscale="YlGnBu"),"YIOrRd"===C.colorscale&&(C.colorscale="YlOrRd"))}if("surface"===i.type&&a.isPlainObject(i.contours)){var A=["x","y","z"];for(r=0;r<A.length;r++){var _=i.contours[A[r]];a.isPlainObject(_)&&(_.highlightColor&&(_.highlightcolor=_.highlightColor,delete _.highlightColor),_.highlightWidth&&(_.highlightwidth=_.highlightWidth,delete _.highlightWidth))}}if("candlestick"===i.type||"ohlc"===i.type){var I=!1!==(i.increasing||{}).showlegend,x=!1!==(i.decreasing||{}).showlegend,w=g(i.increasing),M=g(i.decreasing);if(!1!==w&&!1!==M){var S=v(w,M,I,x);S&&(i.name=S)}else!w&&!M||i.name||(i.name=w||M)}if(Array.isArray(i.transforms)){var T=i.transforms;for(r=0;r<T.length;r++){var E=T[r];if(a.isPlainObject(E))switch(E.type){case"filter":E.filtersrc&&(E.target=E.filtersrc,delete E.filtersrc),E.calendar&&(E.valuecalendar||(E.valuecalendar=E.calendar),delete E.calendar);break;case"groupby":if(E.styles=E.styles||E.style,E.styles&&!Array.isArray(E.styles)){var O=E.styles,k=Object.keys(O);E.styles=[];for(var L=0;L<k.length;L++)E.styles.push({target:k[L],value:O[k[L]]})}}}}y(i,"line")&&delete i.line,"marker"in i&&(y(i.marker,"line")&&delete i.marker.line,y(i,"marker")&&delete i.marker),u.clean(i),i.autobinx&&(delete i.autobinx,delete i.xbins),i.autobiny&&(delete i.autobiny,delete i.ybins),p(i),i.colorbar&&p(i.colorbar),i.marker&&i.marker.colorbar&&p(i.marker.colorbar),i.line&&i.line.colorbar&&p(i.line.colorbar),i.aaxis&&p(i.aaxis),i.baxis&&p(i.baxis)}},t.swapXYData=function(e){var t;if(a.swapAttrs(e,["?","?0","d?","?bins","nbins?","autobin?","?src","error_?"]),Array.isArray(e.z)&&Array.isArray(e.z[0])&&(e.transpose?delete e.transpose:e.transpose=!0),e.error_x&&e.error_y){var n=e.error_y,r="copy_ystyle"in n?n.copy_ystyle:!(n.color||n.thickness||n.width);a.swapAttrs(e,["error_?.copy_ystyle"]),r&&a.swapAttrs(e,["error_?.color","error_?.thickness","error_?.width"])}if("string"==typeof e.hoverinfo){var i=e.hoverinfo.split("+");for(t=0;t<i.length;t++)"x"===i[t]?i[t]="y":"y"===i[t]&&(i[t]="x");e.hoverinfo=i.join("+")}},t.coerceTraceIndices=function(e,t){if(r(t))return[t];if(!Array.isArray(t)||!t.length)return e.data.map((function(e,t){return t}));if(Array.isArray(t)){for(var n=[],i=0;i<t.length;i++)a.isIndex(t[i],e.data.length)?n.push(t[i]):a.warn("trace index (",t[i],") is not a number or is out of bounds");return n}return t},t.manageArrayContainers=function(e,t,n){var i=e.obj,o=e.parts,s=o.length,l=o[s-1],u=r(l);if(u&&null===t){var c=o.slice(0,s-1).join(".");a.nestedProperty(i,c).get().splice(l,1)}else u&&void 0===e.get()?(void 0===e.get()&&(n[e.astr]=null),e.set(t)):e.set(t)};var b=/(\.[^\[\]\.]+|\[[^\[\]\.]+\])$/;function C(e){var t=e.search(b);if(t>0)return e.substr(0,t)}t.hasParent=function(e,t){for(var n=C(t);n;){if(n in e)return!0;n=C(n)}return!1};var A=["x","y","z"];t.clearAxisTypes=function(e,t,n){for(var r=0;r<t.length;r++)for(var i=e._fullData[r],o=0;o<3;o++){var s=d(e,i,A[o]);if(s&&"log"!==s.type){var l=s._name,u=s._id.substr(1);if("scene"===u.substr(0,5)){if(void 0!==n[u])continue;l=u+"."+l}var c=l+".type";void 0===n[l]&&void 0===n[c]&&a.nestedProperty(e.layout,c).set(null)}}}},function(e,t,n){"use strict";var r=n(10),i=n(82),o=n(297),a=n(253).dfltConfig,s=r.isPlainObject,l=Array.isArray,u=r.isArrayOrTypedArray;function c(e,t,n,i,o,a){a=a||[];for(var d=Object.keys(e),h=0;h<d.length;h++){var f=d[h];if("transforms"!==f){var m=a.slice();m.push(f);var y=e[f],b=t[f],C=v(n,f),A=(C||{}).valType,_="info_array"===A,I="colorscale"===A,x=(C||{}).items;if(g(n,f))if(s(y)&&s(b)&&"any"!==A)c(y,b,C,i,o,m);else if(_&&l(y)){y.length>b.length&&i.push(p("unused",o,m.concat(b.length)));var w,M,S,T,E,O=b.length,k=Array.isArray(x);if(k&&(O=Math.min(O,x.length)),2===C.dimensions)for(M=0;M<O;M++)if(l(y[M])){y[M].length>b[M].length&&i.push(p("unused",o,m.concat(M,b[M].length)));var L=b[M].length;for(w=0;w<(k?Math.min(L,x[M].length):L);w++)S=k?x[M][w]:x,T=y[M][w],E=b[M][w],r.validate(T,S)?E!==T&&E!==+T&&i.push(p("dynamic",o,m.concat(M,w),T,E)):i.push(p("value",o,m.concat(M,w),T))}else i.push(p("array",o,m.concat(M),y[M]));else for(M=0;M<O;M++)S=k?x[M]:x,T=y[M],E=b[M],r.validate(T,S)?E!==T&&E!==+T&&i.push(p("dynamic",o,m.concat(M),T,E)):i.push(p("value",o,m.concat(M),T))}else if(C.items&&!_&&l(y)){var D,P,N=x[Object.keys(x)[0]],j=[];for(D=0;D<b.length;D++){var R=b[D]._index||D;if((P=m.slice()).push(R),s(y[R])&&s(b[D])){j.push(R);var z=y[R],F=b[D];s(z)&&!1!==z.visible&&!1===F.visible?i.push(p("invisible",o,P)):c(z,F,N,i,o,P)}}for(D=0;D<y.length;D++)(P=m.slice()).push(D),s(y[D])?-1===j.indexOf(D)&&i.push(p("unused",o,P)):i.push(p("object",o,P,y[D]))}else!s(y)&&s(b)?i.push(p("object",o,m,y)):u(y)||!u(b)||_||I?f in t?r.validate(y,C)?"enumerated"===C.valType&&(C.coerceNumber&&y!==+b||y!==b)&&i.push(p("dynamic",o,m,y,b)):i.push(p("value",o,m,y)):i.push(p("unused",o,m,y)):i.push(p("array",o,m,y));else i.push(p("schema",o,m))}}return i}function d(e,t){for(var n=e.layout.layoutAttributes,i=0;i<t.length;i++){var o=t[i],a=e.traces[o.type],s=a.layoutAttributes;s&&(o.subplot?r.extendFlat(n[a.attributes.subplot.dflt],s):r.extendFlat(n,s))}return n}e.exports=function(e,t){void 0===e&&(e=[]),void 0===t&&(t={});var n,u,h=o.get(),f=[],g={_context:r.extendFlat({},a)};l(e)?(g.data=r.extendDeep([],e),n=e):(g.data=[],n=[],f.push(p("array","data"))),s(t)?(g.layout=r.extendDeep({},t),u=t):(g.layout={},u={},arguments.length>1&&f.push(p("object","layout"))),i.supplyDefaults(g);for(var v=g._fullData,m=n.length,y=0;y<m;y++){var b=n[y],C=["data",y];if(s(b)){var A=v[y],_=A.type,I=h.traces[_].attributes;I.type={valType:"enumerated",values:[_]},!1===A.visible&&!1!==b.visible&&f.push(p("invisible",C)),c(b,A,I,f,C);var x=b.transforms,w=A.transforms;if(x){l(x)||f.push(p("array",C,["transforms"])),C.push("transforms");for(var M=0;M<x.length;M++){var S=["transforms",M],T=x[M].type;if(s(x[M])){var E=h.transforms[T]?h.transforms[T].attributes:{};E.type={valType:"enumerated",values:Object.keys(h.transforms)},c(x[M],w[M],E,f,C,S)}else f.push(p("object",C,S))}}}else f.push(p("object",C))}var O=g._fullLayout,k=d(h,v);return c(u,O,k,f,"layout"),0===f.length?void 0:f};var h={object:function(e,t){return("layout"===e&&""===t?"The layout argument":"data"===e[0]&&""===t?"Trace "+e[1]+" in the data argument":f(e)+"key "+t)+" must be linked to an object container"},array:function(e,t){return("data"===e?"The data argument":f(e)+"key "+t)+" must be linked to an array container"},schema:function(e,t){return f(e)+"key "+t+" is not part of the schema"},unused:function(e,t,n){var r=s(n)?"container":"key";return f(e)+r+" "+t+" did not get coerced"},dynamic:function(e,t,n,r){return[f(e)+"key",t,"(set to '"+n+"')","got reset to","'"+r+"'","during defaults."].join(" ")},invisible:function(e,t){return(t?f(e)+"item "+t:"Trace "+e[1])+" got defaulted to be not visible"},value:function(e,t,n){return[f(e)+"key "+t,"is set to an invalid value ("+n+")"].join(" ")}};function f(e){return l(e)?"In data trace "+e[1]+", ":"In "+e+", "}function p(e,t,n,i,o){var a,s;n=n||"",l(t)?(a=t[0],s=t[1]):(a=t,s=null);var u=function(e){if(!l(e))return String(e);for(var t="",n=0;n<e.length;n++){var r=e[n];"number"==typeof r?t=t.substr(0,t.length-1)+"["+r+"]":t+=r,n<e.length-1&&(t+=".")}return t}(n),c=h[e](t,u,i,o);return r.log(c),{code:e,container:a,trace:s,path:n,astr:u,msg:c}}function g(e,t){var n=y(t),r=n.keyMinusId,i=n.id;return!!(r in e&&e[r]._isSubplotObj&&i)||t in e}function v(e,t){return t in e?e[t]:e[y(t).keyMinusId]}var m=r.counterRegex("([a-z]+)");function y(e){var t=e.match(m);return{keyMinusId:t&&t[1],id:t&&t[2]}}},function(e,t,n){"use strict";var r=n(10),i=n(256);e.exports=function(e,t,n){var o=document.createElement("a"),a="download"in o;return new Promise((function(s,l){var u,c;if(r.isIE9orBelow()&&l(new Error("IE < 10 unsupported")),r.isSafari()){var d="svg"===n?",":";base64,";return i.octetStream(d+encodeURIComponent(e)),s(t)}return r.isIE()?(u=i.createBlob(e,"svg"),window.navigator.msSaveBlob(u,t),u=null,s(t)):a?(u=i.createBlob(e,n),c=i.createObjectURL(u),o.href=c,o.download=t,document.body.appendChild(o),o.click(),document.body.removeChild(o),i.revokeObjectURL(c),u=null,s(t)):void l(new Error("download error"))}))}},function(e,t,n){"use strict";var r=n(10),i=r.isPlainObject,o=n(297),a=n(82),s=n(160),l=n(64),u=n(253).dfltConfig;function c(e,t){e=r.extendDeep({},e);var n,o,a=Object.keys(e).sort();function s(t,n,r){if(i(n)&&i(t))c(t,n);else if(Array.isArray(n)&&Array.isArray(t)){var a=l.arrayTemplater({_template:e},r);for(o=0;o<n.length;o++){var s=n[o],u=a.newItem(s)._template;u&&c(u,s)}var d=a.defaultItems();for(o=0;o<d.length;o++)n.push(d[o]._template);for(o=0;o<n.length;o++)delete n[o].templateitemname}}for(n=0;n<a.length;n++){var u=a[n],h=e[u];if(u in t?s(h,t[u],u):t[u]=h,d(u)===u)for(var f in t){var p=d(f);f===p||p!==u||f in e||s(h,t[f],u)}}}function d(e){return e.replace(/[0-9]+$/,"")}function h(e,t,n,o,a){var s=a&&n(a);for(var u in e){var c=e[u],f=g(e,u,o),p=g(e,u,a),v=n(p);if(!v){var m=d(u);m!==u&&(v=n(p=g(e,m,a)))}if((!s||s!==v)&&!(!v||v._noTemplating||"data_array"===v.valType||v.arrayOk&&Array.isArray(c)))if(!v.valType&&i(c))h(c,t,n,f,p);else if(v._isLinkedToArray&&Array.isArray(c))for(var y=!1,b=0,C={},A=0;A<c.length;A++){var _=c[A];if(i(_)){var I=_.name;if(I)C[I]||(h(_,t,n,g(c,b,f),g(c,b,p)),b++,C[I]=1);else if(!y){var x=g(e,l.arrayDefaultKey(u),o),w=g(c,b,f);h(_,t,n,w,g(c,b,p));var M=r.nestedProperty(t,w);r.nestedProperty(t,x).set(M.get()),M.set(null),y=!0}}}else{r.nestedProperty(t,f).set(c)}}}function f(e,t){return o.getLayoutValObject(e,r.nestedProperty({},t).parts)}function p(e,t){return o.getTraceValObject(e,r.nestedProperty({},t).parts)}function g(e,t,n){return n?Array.isArray(e)?n+"["+t+"]":n+"."+t:t}function v(e){for(var t=0;t<e.length;t++)if(i(e[t]))return!0}function m(e){var t;switch(e.code){case"data":t="The template has no key data.";break;case"layout":t="The template has no key layout.";break;case"missing":t=e.path?"There are no templates for item "+e.path+" with name "+e.templateitemname:"There are no templates for trace "+e.index+", of type "+e.traceType+".";break;case"unused":t=e.path?"The template item at "+e.path+" was not used in constructing the plot.":e.dataCount?"Some of the templates of type "+e.traceType+" were not used. The template has "+e.templateCount+" traces, the data only has "+e.dataCount+" of this type.":"The template has "+e.templateCount+" traces of type "+e.traceType+" but there are none in the data.";break;case"reused":t="Some of the templates of type "+e.traceType+" were used more than once. The template has "+e.templateCount+" traces, the data has "+e.dataCount+" of this type."}return e.msg=t,e}t.makeTemplate=function(e){e=r.isPlainObject(e)?e:r.getGraphDiv(e),e=r.extendDeep({_context:u},{data:e.data,layout:e.layout}),a.supplyDefaults(e);var t=e.data||[],n=e.layout||{};n._basePlotModules=e._fullLayout._basePlotModules,n._modules=e._fullLayout._modules;var o={data:{},layout:{}};t.forEach((function(e){var t={};h(e,t,p.bind(null,e));var n=r.coerce(e,{},s,"type"),i=o.data[n];i||(i=o.data[n]=[]),i.push(t)})),h(n,o.layout,f.bind(null,n)),delete o.layout.template;var l=n.template;if(i(l)){var d,g,v,m,y,b,C=l.layout;i(C)&&c(C,o.layout);var A=l.data;if(i(A)){for(g in o.data)if(v=A[g],Array.isArray(v)){for(b=(y=o.data[g]).length,m=v.length,d=0;d<b;d++)c(v[d%m],y[d]);for(d=b;d<m;d++)y.push(r.extendDeep({},v[d]))}for(g in A)g in o.data||(o.data[g]=r.extendDeep([],A[g]))}}return o},t.validateTemplate=function(e,t){var n=r.extendDeep({},{_context:u,data:e.data,layout:e.layout}),o=n.layout||{};i(t)||(t=o.template||{});var s=t.layout,l=t.data,c=[];n.layout=o,n.layout.template=t,a.supplyDefaults(n);var h=n._fullLayout,f=n._fullData,p={};if(i(s)?(!function e(t,n){for(var r in t)if("_"!==r.charAt(0)&&i(t[r])){var o,a=d(r),s=[];for(o=0;o<n.length;o++)s.push(g(t,r,n[o])),a!==r&&s.push(g(t,a,n[o]));for(o=0;o<s.length;o++)p[s[o]]=1;e(t[r],s)}}(h,["layout"]),function e(t,n){for(var r in t)if(-1===r.indexOf("defaults")&&i(t[r])){var o=g(t,r,n);p[o]?e(t[r],o):c.push({code:"unused",path:o})}}(s,"layout")):c.push({code:"layout"}),i(l)){for(var y,b={},C=0;C<f.length;C++){var A=f[C];b[y=A.type]=(b[y]||0)+1,A._fullInput._template||c.push({code:"missing",index:A._fullInput.index,traceType:y})}for(y in l){var _=l[y].length,I=b[y]||0;_>I?c.push({code:"unused",traceType:y,templateCount:_,dataCount:I}):I>_&&c.push({code:"reused",traceType:y,templateCount:_,dataCount:I})}}else c.push({code:"data"});if(function e(t,n){for(var r in t)if("_"!==r.charAt(0)){var o=t[r],a=g(t,r,n);i(o)?(Array.isArray(t)&&!1===o._template&&o.templateitemname&&c.push({code:"missing",path:a,templateitemname:o.templateitemname}),e(o,a)):Array.isArray(o)&&v(o)&&e(o,a)}}({data:f,layout:h},""),c.length)return c.map(m)}},function(e,t,n){"use strict";var r=n(125);e.exports={hasLines:r.hasLines,hasMarkers:r.hasMarkers,hasText:r.hasText,isBubble:r.isBubble,attributes:n(193),supplyDefaults:n(1720),crossTraceDefaults:n(1724),calc:n(1013).calc,crossTraceCalc:n(1725),arraysToCalcdata:n(558),plot:n(1726),colorbar:n(1016),formatLabels:n(1729),style:n(1017).style,styleOnSelect:n(1017).styleOnSelect,hoverPoints:n(1730),selectPoints:n(1732),animatable:!0,moduleType:"trace",name:"scatter",basePlotModule:n(301),categories:["cartesian","svg","symbols","errorBarsOK","showLegend","scatter-like","zoomScale"],meta:{}}},function(e,t,n){"use strict";var r=n(10),i=n(27),o=n(193),a=n(544),s=n(125),l=n(1008),u=n(1009),c=n(1721),d=n(1010),h=n(1011),f=n(1722),p=n(1012),g=n(1723);e.exports=function(e,t,n,v){function m(n,i){return r.coerce(e,t,o,n,i)}var y=l(e,t,v,m);if(y||(t.visible=!1),t.visible){u(e,t,v,m);var b=c(e,t,v,m),C=!b&&y<a.PTS_LINESONLY?"lines+markers":"lines";m("text"),m("hovertext"),m("mode",C),s.hasLines(t)&&(h(e,t,n,v,m),f(e,t,m),m("connectgaps"),m("line.simplify")),s.hasMarkers(t)&&d(e,t,n,v,m,{gradient:!0}),s.hasText(t)&&(m("texttemplate"),p(e,t,v,m));var A=[];(s.hasMarkers(t)||s.hasText(t))&&(m("cliponaxis"),m("marker.maxdisplayed"),A.push("points")),m("fill",b?b.fillDflt:"none"),"none"!==t.fill&&(g(e,t,n,m),s.hasLines(t)||f(e,t,m));var _=(t.line||{}).color,I=(t.marker||{}).color;"tonext"!==t.fill&&"toself"!==t.fill||A.push("fills"),m("hoveron",A.join("+")||"points"),"fills"!==t.hoveron&&m("hovertemplate");var x=i.getComponentMethod("errorbars","supplyDefaults");x(e,t,_||I||n,{axis:"y"}),x(e,t,_||I||n,{axis:"x",inherit:"y"}),r.coerceSelectionMarkerOpacity(t,m)}}},function(e,t,n){"use strict";var r=["orientation","groupnorm","stackgaps"];e.exports=function(e,t,n,i){var o=n._scatterStackOpts,a=i("stackgroup");if(a){var s=t.xaxis+t.yaxis,l=o[s];l||(l=o[s]={});var u=l[a],c=!1;u?u.traces.push(t):(u=l[a]={traceIndices:[],traces:[t]},c=!0);for(var d={orientation:t.x&&!t.y?"h":"v"},h=0;h<r.length;h++){var f=r[h],p=f+"Found";if(!u[p]){var g=void 0!==e[f],v="orientation"===f;if((g||c)&&(u[f]=i(f,d[f]),v&&(u.fillDflt="h"===u[f]?"tonextx":"tonexty"),g&&(u[p]=!0,!c&&(delete u.traces[0][f],v))))for(var m=0;m<u.traces.length-1;m++){var y=u.traces[m];y._input.fill!==y.fill&&(y.fill=u.fillDflt)}}}return u}}},function(e,t,n){"use strict";e.exports=function(e,t,n){"spline"===n("line.shape")&&n("line.smoothing")}},function(e,t,n){"use strict";var r=n(41),i=n(10).isArrayOrTypedArray;e.exports=function(e,t,n,o){var a=!1;if(t.marker){var s=t.marker.color,l=(t.marker.line||{}).color;s&&!i(s)?a=s:l&&!i(l)&&(a=l)}o("fillcolor",r.addOpacity((t.line||{}).color||a||n,.5))}},function(e,t,n){"use strict";e.exports=function(e){for(var t=0;t<e.length;t++){var n=e[t];if("scatter"===n.type){var r=n.fill;if("none"!==r&&"toself"!==r&&(n.opacity=void 0,"tonexty"===r||"tonextx"===r))for(var i=t-1;i>=0;i--){var o=e[i];if("scatter"===o.type&&o.xaxis===n.xaxis&&o.yaxis===n.yaxis){o.opacity=void 0;break}}}}}},function(e,t,n){"use strict";var r=n(1013);function i(e,t,n,r,i,o,a){i[r]=!0;var s={i:null,gap:!0,s:0};if(s[a]=n,e.splice(t,0,s),t&&n===e[t-1][a]){var l=e[t-1];s.s=l.s,s.i=l.i,s.gap=l.gap}else o&&(s.s=function(e,t,n,r){var i=e[t-1],o=e[t+1];return o?i?i.s+(o.s-i.s)*(n-i[r])/(o[r]-i[r]):o.s:i.s}(e,t,n,a));t||(e[0].t=e[1].t,e[0].trace=e[1].trace,delete e[1].t,delete e[1].trace)}e.exports=function(e,t){var n=t.xaxis,o=t.yaxis,a=n._id+o._id,s=e._fullLayout._scatterStackOpts[a];if(s){var l,u,c,d,h,f,p,g,v,m,y,b,C,A,_,I=e.calcdata;for(var x in s){var w=(m=s[x]).traceIndices;if(w.length){for(y="interpolate"===m.stackgaps,b=m.groupnorm,"v"===m.orientation?(C="x",A="y"):(C="y",A="x"),_=new Array(w.length),l=0;l<_.length;l++)_[l]=!1;f=I[w[0]];var M=new Array(f.length);for(l=0;l<f.length;l++)M[l]=f[l][C];for(l=1;l<w.length;l++){for(h=I[w[l]],u=c=0;u<h.length;u++){for(p=h[u][C];p>M[c]&&c<M.length;c++)i(h,u,M[c],l,_,y,C),u++;if(p!==M[c]){for(d=0;d<l;d++)i(I[w[d]],c,p,d,_,y,C);M.splice(c,0,p)}c++}for(;c<M.length;c++)i(h,u,M[c],l,_,y,C),u++}var S=M.length;for(u=0;u<f.length;u++){for(g=f[u][A]=f[u].s,l=1;l<w.length;l++)(h=I[w[l]])[0].trace._rawLength=h[0].trace._length,h[0].trace._length=S,g+=h[u].s,h[u][A]=g;if(b)for(v=("fraction"===b?g:g/100)||1,l=0;l<w.length;l++){var T=I[w[l]][u];T[A]/=v,T.sNorm=T.s/v}}for(l=0;l<w.length;l++){var E=(h=I[w[l]])[0].trace,O=r.calcMarkerSize(E,E._rawLength),k=Array.isArray(O);if(O&&_[l]||k){var L=O;for(O=new Array(S),u=0;u<S;u++)O[u]=h[u].gap?0:k?L[h[u].i]:L}var D=new Array(S),P=new Array(S);for(u=0;u<S;u++)D[u]=h[u].x,P[u]=h[u].y;r.calcAxisExpansion(e,E,n,o,D,P,O),h[0].t.orientation=m.orientation}}}}}},function(e,t,n){"use strict";var r=n(39),i=n(27),o=n(10),a=o.ensureSingle,s=o.identity,l=n(56),u=n(125),c=n(1727),d=n(1728),h=n(1005).tester;function f(e,t,n,d,f,p,g){var v;!function(e,t,n,i,a){var s=n.xaxis,l=n.yaxis,c=r.extent(o.simpleMap(s.range,s.r2c)),d=r.extent(o.simpleMap(l.range,l.r2c)),h=i[0].trace;if(!u.hasMarkers(h))return;var f=h.marker.maxdisplayed;if(0===f)return;var p=i.filter((function(e){return e.x>=c[0]&&e.x<=c[1]&&e.y>=d[0]&&e.y<=d[1]})),g=Math.ceil(p.length/f),v=0;a.forEach((function(e,n){var r=e[0].trace;u.hasMarkers(r)&&r.marker.maxdisplayed>0&&n<t&&v++}));var m=Math.round(v*g/3+Math.floor(v/3)*g/7.1);i.forEach((function(e){delete e.vis})),p.forEach((function(e,t){0===Math.round((t+m)%g)&&(e.vis=!0)}))}(0,t,n,d,f);var m=!!g&&g.duration>0;function y(e){return m?e.transition():e}var b=n.xaxis,C=n.yaxis,A=d[0].trace,_=A.line,I=r.select(p),x=a(I,"g","errorbars"),w=a(I,"g","lines"),M=a(I,"g","points"),S=a(I,"g","text");if(i.getComponentMethod("errorbars","plot")(e,x,n,g),!0===A.visible){var T,E;y(I).style("opacity",A.opacity);var O=A.fill.charAt(A.fill.length-1);"x"!==O&&"y"!==O&&(O=""),d[0][n.isRangePlot?"nodeRangePlot3":"node3"]=I;var k,L,D="",P=[],N=A._prevtrace;N&&(D=N._prevRevpath||"",E=N._nextFill,P=N._polygons);var j,R,z,F,B,H,U,V="",Y="",W=[],G=o.noop;if(T=A._ownFill,u.hasLines(A)||"none"!==A.fill){for(E&&E.datum(d),-1!==["hv","vh","hvh","vhv"].indexOf(_.shape)?(j=l.steps(_.shape),R=l.steps(_.shape.split("").reverse().join(""))):j=R="spline"===_.shape?function(e){var t=e[e.length-1];return e.length>1&&e[0][0]===t[0]&&e[0][1]===t[1]?l.smoothclosed(e.slice(1),_.smoothing):l.smoothopen(e,_.smoothing)}:function(e){return"M"+e.join("L")},z=function(e){return R(e.reverse())},W=c(d,{xaxis:b,yaxis:C,connectGaps:A.connectgaps,baseTolerance:Math.max(_.width||1,3)/4,shape:_.shape,simplify:_.simplify,fill:A.fill}),U=A._polygons=new Array(W.length),v=0;v<W.length;v++)A._polygons[v]=h(W[v]);W.length&&(F=W[0][0],H=(B=W[W.length-1])[B.length-1]),G=function(e){return function(t){if(k=j(t),L=z(t),V?O?(V+="L"+k.substr(1),Y=L+"L"+Y.substr(1)):(V+="Z"+k,Y=L+"Z"+Y):(V=k,Y=L),u.hasLines(A)&&t.length>1){var n=r.select(this);if(n.datum(d),e)y(n.style("opacity",0).attr("d",k).call(l.lineGroupStyle)).style("opacity",1);else{var i=y(n);i.attr("d",k),l.singleLineStyle(d,i)}}}}}var X=w.selectAll(".js-line").data(W);y(X.exit()).style("opacity",0).remove(),X.each(G(!1)),X.enter().append("path").classed("js-line",!0).style("vector-effect","non-scaling-stroke").call(l.lineGroupStyle).each(G(!0)),l.setClipUrl(X,n.layerClipId,e),W.length?(T?(T.datum(d),F&&H&&(O?("y"===O?F[1]=H[1]=C.c2p(0,!0):"x"===O&&(F[0]=H[0]=b.c2p(0,!0)),y(T).attr("d","M"+H+"L"+F+"L"+V.substr(1)).call(l.singleFillStyle)):y(T).attr("d",V+"Z").call(l.singleFillStyle))):E&&("tonext"===A.fill.substr(0,6)&&V&&D?("tonext"===A.fill?y(E).attr("d",V+"Z"+D+"Z").call(l.singleFillStyle):y(E).attr("d",V+"L"+D.substr(1)+"Z").call(l.singleFillStyle),A._polygons=A._polygons.concat(P)):(Z(E),A._polygons=null)),A._prevRevpath=Y,A._prevPolygons=U):(T?Z(T):E&&Z(E),A._polygons=A._prevRevpath=A._prevPolygons=null),M.datum(d),S.datum(d),function(t,i,o){var a,c=o[0].trace,d=u.hasMarkers(c),h=u.hasText(c),f=ee(c),p=te,g=te;if(d||h){var v=s,A=c.stackgroup,_=A&&"infer zero"===e._fullLayout._scatterStackOpts[b._id+C._id][A].stackgaps;c.marker.maxdisplayed||c._needsCull?v=_?K:Q:A&&!_&&(v=J),d&&(p=v),h&&(g=v)}var I,x=(a=t.selectAll("path.point").data(p,f)).enter().append("path").classed("point",!0);m&&x.call(l.pointStyle,c,e).call(l.translatePoints,b,C).style("opacity",0).transition().style("opacity",1),a.order(),d&&(I=l.makePointStyleFns(c)),a.each((function(t){var i=r.select(this),o=y(i);l.translatePoint(t,o,b,C)?(l.singlePointStyle(t,o,c,I,e),n.layerClipId&&l.hideOutsideRangePoint(t,o,b,C,c.xcalendar,c.ycalendar),c.customdata&&i.classed("plotly-customdata",null!==t.data&&void 0!==t.data)):o.remove()})),m?a.exit().transition().style("opacity",0).remove():a.exit().remove(),(a=i.selectAll("g").data(g,f)).enter().append("g").classed("textpoint",!0).append("text"),a.order(),a.each((function(e){var t=r.select(this),i=y(t.select("text"));l.translatePoint(e,i,b,C)?n.layerClipId&&l.hideOutsideRangePoint(e,t,b,C,c.xcalendar,c.ycalendar):t.remove()})),a.selectAll("text").call(l.textPointStyle,c,e).each((function(e){var t=b.c2p(e.x),n=C.c2p(e.y);r.select(this).selectAll("tspan.line").each((function(){y(r.select(this)).attr({x:t,y:n})}))})),a.exit().remove()}(M,S,d);var q=!1===A.cliponaxis?null:n.layerClipId;l.setClipUrl(M,q,e),l.setClipUrl(S,q,e)}function Z(e){y(e).attr("d","M0,0Z")}function Q(e){return e.filter((function(e){return!e.gap&&e.vis}))}function K(e){return e.filter((function(e){return e.vis}))}function J(e){return e.filter((function(e){return!e.gap}))}function $(e){return e.id}function ee(e){if(e.ids)return $}function te(){return!1}}e.exports=function(e,t,n,i,o,u){var c,h,p=!o,g=!!o&&o.duration>0,v=d(e,t,n);((c=i.selectAll("g.trace").data(v,(function(e){return e[0].trace.uid}))).enter().append("g").attr("class",(function(e){return"trace scatter trace"+e[0].trace.uid})).style("stroke-miterlimit",2),c.order(),function(e,t,n){t.each((function(t){var i=a(r.select(this),"g","fills");l.setClipUrl(i,n.layerClipId,e);var o=t[0].trace,u=[];o._ownfill&&u.push("_ownFill"),o._nexttrace&&u.push("_nextFill");var c=i.selectAll("g").data(u,s);c.enter().append("g"),c.exit().each((function(e){o[e]=null})).remove(),c.order().each((function(e){o[e]=a(r.select(this),"path","js-fill")}))}))}(e,c,t),g)?(u&&(h=u()),r.transition().duration(o.duration).ease(o.easing).each("end",(function(){h&&h()})).each("interrupt",(function(){h&&h()})).each((function(){i.selectAll("g.trace").each((function(n,r){f(e,r,t,n,v,this,o)}))}))):c.each((function(n,r){f(e,r,t,n,v,this,o)}));p&&c.exit().remove(),i.selectAll("path:not([d])").remove()}},function(e,t,n){"use strict";var r=n(72),i=r.BADNUM,o=r.LOG_CLIP,a=o+.5,s=o-.5,l=n(10),u=l.segmentsIntersect,c=l.constrain,d=n(544);e.exports=function(e,t){var n,r,o,h,f,p,g,v,m,y,b,C,A,_,I,x,w,M,S=t.xaxis,T=t.yaxis,E="log"===S.type,O="log"===T.type,k=S._length,L=T._length,D=t.connectGaps,P=t.baseTolerance,N=t.shape,j="linear"===N,R=t.fill&&"none"!==t.fill,z=[],F=d.minTolerance,B=e.length,H=new Array(B),U=0;function V(n){var r=e[n];if(!r)return!1;var o=t.linearized?S.l2p(r.x):S.c2p(r.x),l=t.linearized?T.l2p(r.y):T.c2p(r.y);if(o===i){if(E&&(o=S.c2p(r.x,!0)),o===i)return!1;O&&l===i&&(o*=Math.abs(S._m*L*(S._m>0?a:s)/(T._m*k*(T._m>0?a:s)))),o*=1e3}if(l===i){if(O&&(l=T.c2p(r.y,!0)),l===i)return!1;l*=1e3}return[o,l]}function Y(e,t,n,r){var i=n-e,o=r-t,a=.5-e,s=.5-t,l=i*i+o*o,u=i*a+o*s;if(u>0&&u<l){var c=a*o-s*i;if(c*c<l)return!0}}function W(e,t){var n=e[0]/k,r=e[1]/L,i=Math.max(0,-n,n-1,-r,r-1);return i&&void 0!==w&&Y(n,r,w,M)&&(i=0),i&&t&&Y(n,r,t[0]/k,t[1]/L)&&(i=0),(1+d.toleranceGrowth*i)*P}function G(e,t){var n=e[0]-t[0],r=e[1]-t[1];return Math.sqrt(n*n+r*r)}var X,q,Z,Q,K,J,$,ee=d.maxScreensAway,te=-k*ee,ne=k*(1+ee),re=-L*ee,ie=L*(1+ee),oe=[[te,re,ne,re],[ne,re,ne,ie],[ne,ie,te,ie],[te,ie,te,re]];function ae(e){if(e[0]<te||e[0]>ne||e[1]<re||e[1]>ie)return[c(e[0],te,ne),c(e[1],re,ie)]}function se(e,t){return e[0]===t[0]&&(e[0]===te||e[0]===ne)||(e[1]===t[1]&&(e[1]===re||e[1]===ie)||void 0)}function le(e,t,n){return function(r,i){var o=ae(r),a=ae(i),s=[];if(o&&a&&se(o,a))return s;o&&s.push(o),a&&s.push(a);var u=2*l.constrain((r[e]+i[e])/2,t,n)-((o||r)[e]+(a||i)[e]);u&&((o&&a?u>0==o[e]>a[e]?o:a:o||a)[e]+=u);return s}}function ue(e){var t=e[0],n=e[1],r=t===H[U-1][0],i=n===H[U-1][1];if(!r||!i)if(U>1){var o=t===H[U-2][0],a=n===H[U-2][1];r&&(t===te||t===ne)&&o?a?U--:H[U-1]=e:i&&(n===re||n===ie)&&a?o?U--:H[U-1]=e:H[U++]=e}else H[U++]=e}function ce(e){H[U-1][0]!==e[0]&&H[U-1][1]!==e[1]&&ue([Z,Q]),ue(e),K=null,Z=Q=0}function de(e){if(w=e[0]/k,M=e[1]/L,X=e[0]<te?te:e[0]>ne?ne:0,q=e[1]<re?re:e[1]>ie?ie:0,X||q){if(U)if(K){var t=$(K,e);t.length>1&&(ce(t[0]),H[U++]=t[1])}else J=$(H[U-1],e)[0],H[U++]=J;else H[U++]=[X||e[0],q||e[1]];var n=H[U-1];X&&q&&(n[0]!==X||n[1]!==q)?(K&&(Z!==X&&Q!==q?ue(Z&&Q?(r=K,o=(i=e)[0]-r[0],a=(i[1]-r[1])/o,(r[1]*i[0]-i[1]*r[0])/o>0?[a>0?te:ne,ie]:[a>0?ne:te,re]):[Z||X,Q||q]):Z&&Q&&ue([Z,Q])),ue([X,q])):Z-X&&Q-q&&ue([X||Z,q||Q]),K=e,Z=X,Q=q}else K&&ce($(K,e)[0]),H[U++]=e;var r,i,o,a}for("linear"===N||"spline"===N?$=function(e,t){for(var n=[],r=0,i=0;i<4;i++){var o=oe[i],a=u(e[0],e[1],t[0],t[1],o[0],o[1],o[2],o[3]);a&&(!r||Math.abs(a.x-n[0][0])>1||Math.abs(a.y-n[0][1])>1)&&(a=[a.x,a.y],r&&G(a,e)<G(n[0],e)?n.unshift(a):n.push(a),r++)}return n}:"hv"===N||"vh"===N?$=function(e,t){var n=[],r=ae(e),i=ae(t);return r&&i&&se(r,i)||(r&&n.push(r),i&&n.push(i)),n}:"hvh"===N?$=le(0,te,ne):"vhv"===N&&($=le(1,re,ie)),n=0;n<B;n++)if(r=V(n)){for(U=0,K=null,de(r),n++;n<B;n++){if(!(h=V(n))){if(D)continue;break}if(j&&t.simplify){var he=V(n+1);if(y=G(h,r),R&&(0===U||U===B-1)||!(y<W(h,he)*F)){for(v=[(h[0]-r[0])/y,(h[1]-r[1])/y],f=r,b=y,C=_=I=0,g=!1,o=h,n++;n<e.length;n++){if(p=he,he=V(n+1),!p){if(D)continue;break}if(x=(m=[p[0]-r[0],p[1]-r[1]])[0]*v[1]-m[1]*v[0],_=Math.min(_,x),(I=Math.max(I,x))-_>W(p,he))break;o=p,(A=m[0]*v[0]+m[1]*v[1])>b?(b=A,h=p,g=!1):A<C&&(C=A,f=p,g=!0)}if(g?(de(h),o!==f&&de(f)):(f!==r&&de(f),o!==h&&de(h)),de(o),n>=e.length||!p)break;de(p),r=p}}else de(h)}K&&ue([Z||K[0],Q||K[1]]),z.push(H.slice(0,U))}return z}},function(e,t,n){"use strict";var r={tonextx:1,tonexty:1,tonext:1};e.exports=function(e,t,n){var i,o,a,s,l,u={},c=!1,d=-1,h=0,f=-1;for(o=0;o<n.length;o++)(a=(i=n[o][0].trace).stackgroup||"")?a in u?l=u[a]:(l=u[a]=h,h++):i.fill in r&&f>=0?l=f:(l=f=h,h++),l<d&&(c=!0),i._groupIndex=d=l;var p=n.slice();c&&p.sort((function(e,t){var n=e[0].trace,r=t[0].trace;return n._groupIndex-r._groupIndex||n.index-r.index}));var g={};for(o=0;o<p.length;o++)a=(i=p[o][0].trace).stackgroup||"",!0===i.visible?(i._nexttrace=null,i.fill in r&&(s=g[a],i._prevtrace=s||null,s&&(s._nexttrace=i)),i._ownfill=i.fill&&("tozero"===i.fill.substr(0,6)||"toself"===i.fill||"to"===i.fill.substr(0,2)&&!i._prevtrace),g[a]=i):i._prevtrace=i._nexttrace=i._ownfill=null;return p}},function(e,t,n){"use strict";var r=n(55);e.exports=function(e,t,n){var i={},o={_fullLayout:n},a=r.getFromTrace(o,t,"x"),s=r.getFromTrace(o,t,"y");return i.xLabel=r.tickText(a,e.x,!0).text,i.yLabel=r.tickText(s,e.y,!0).text,i}},function(e,t,n){"use strict";var r=n(10),i=n(179),o=n(27),a=n(1731),s=n(41),l=r.fillText;e.exports=function(e,t,n,u){var c=e.cd,d=c[0].trace,h=e.xa,f=e.ya,p=h.c2p(t),g=f.c2p(n),v=[p,g],m=d.hoveron||"",y=-1!==d.mode.indexOf("markers")?3:.5;if(-1!==m.indexOf("points")){var b=function(e){var t=Math.max(y,e.mrc||0),n=h.c2p(e.x)-p,r=f.c2p(e.y)-g;return Math.max(Math.sqrt(n*n+r*r)-t,1-y/t)},C=i.getDistanceFunction(u,(function(e){var t=Math.max(3,e.mrc||0),n=1-1/t,r=Math.abs(h.c2p(e.x)-p);return r<t?n*r/t:r-t+n}),(function(e){var t=Math.max(3,e.mrc||0),n=1-1/t,r=Math.abs(f.c2p(e.y)-g);return r<t?n*r/t:r-t+n}),b);if(i.getClosest(c,C,e),!1!==e.index){var A=c[e.index],_=h.c2p(A.x,!0),I=f.c2p(A.y,!0),x=A.mrc||1;e.index=A.i;var w=c[0].t.orientation,M=w&&(A.sNorm||A.s),S="h"===w?M:void 0!==A.orig_x?A.orig_x:A.x,T="v"===w?M:void 0!==A.orig_y?A.orig_y:A.y;return r.extendFlat(e,{color:a(d,A),x0:_-x,x1:_+x,xLabelVal:S,y0:I-x,y1:I+x,yLabelVal:T,spikeDistance:b(A),hovertemplate:d.hovertemplate}),l(A,d,e),o.getComponentMethod("errorbars","hoverInfo")(A,d,e),[e]}}if(-1!==m.indexOf("fills")&&d._polygons){var E,O,k,L,D,P,N,j,R,z=d._polygons,F=[],B=!1,H=1/0,U=-1/0,V=1/0,Y=-1/0;for(E=0;E<z.length;E++)(k=z[E]).contains(v)&&(B=!B,F.push(k),V=Math.min(V,k.ymin),Y=Math.max(Y,k.ymax));if(B){var W=((V=Math.max(V,0))+(Y=Math.min(Y,f._length)))/2;for(E=0;E<F.length;E++)for(L=F[E].pts,O=1;O<L.length;O++)(j=L[O-1][1])>W!=(R=L[O][1])>=W&&(P=L[O-1][0],N=L[O][0],R-j&&(D=P+(N-P)*(W-j)/(R-j),H=Math.min(H,D),U=Math.max(U,D)));H=Math.max(H,0),U=Math.min(U,h._length);var G=s.defaultLine;return s.opacity(d.fillcolor)?G=d.fillcolor:s.opacity((d.line||{}).color)&&(G=d.line.color),r.extendFlat(e,{distance:e.maxHoverDistance,x0:H,x1:U,y0:W,y1:W,color:G,hovertemplate:!1}),delete e.index,d.text&&!Array.isArray(d.text)?e.text=String(d.text):e.text=d.name,[e]}}}},function(e,t,n){"use strict";var r=n(41),i=n(125);e.exports=function(e,t){var n,o;if("lines"===e.mode)return(n=e.line.color)&&r.opacity(n)?n:e.fillcolor;if("none"===e.mode)return e.fill?e.fillcolor:"";var a=t.mcc||(e.marker||{}).color,s=t.mlcc||((e.marker||{}).line||{}).color;return(o=a&&r.opacity(a)?a:s&&r.opacity(s)&&(t.mlw||((e.marker||{}).line||{}).width)?s:"")?r.opacity(o)<.3?r.addOpacity(o,.3):o:(n=(e.line||{}).color)&&r.opacity(n)&&i.hasLines(e)&&e.line.width?n:e.fillcolor}},function(e,t,n){"use strict";var r=n(125);e.exports=function(e,t){var n,i,o,a,s=e.cd,l=e.xaxis,u=e.yaxis,c=[],d=s[0].trace;if(!r.hasMarkers(d)&&!r.hasText(d))return[];if(!1===t)for(n=0;n<s.length;n++)s[n].selected=0;else for(n=0;n<s.length;n++)i=s[n],o=l.c2p(i.x),a=u.c2p(i.y),null!==i.i&&t.contains([o,a],!1,n,e)?(c.push({pointNumber:i.i,x:l.c2d(i.x),y:u.c2d(i.y)}),i.selected=1):i.selected=0;return c}},function(e,t,n){"use strict";e.exports={xaxis:{valType:"subplotid",dflt:"x",editType:"calc+clearAxisTypes"},yaxis:{valType:"subplotid",dflt:"y",editType:"calc+clearAxisTypes"}}},function(e,t,n){"use strict";var r=n(10),i=n(41),o=n(162).isUnifiedHover,a=n(1e3),s=n(64),l=n(295),u=n(134),c=n(1018),d=n(559),h=n(390),f=n(1020),p=n(83),g=p.id2name,v=p.name2id,m=n(95).AX_ID_PATTERN,y=n(27),b=y.traceIs,C=y.getComponentMethod;function A(e,t,n){Array.isArray(e[t])?e[t].push(n):e[t]=[n]}e.exports=function(e,t,n){var y,_,I=t.autotypenumbers,x={},w={},M={},S={},T={},E={},O={},k={},L={},D={};for(y=0;y<n.length;y++){var P=n[y];if(b(P,"cartesian")||b(P,"gl2d")){var N,j;if(P.xaxis)N=g(P.xaxis),A(x,N,P);else if(P.xaxes)for(_=0;_<P.xaxes.length;_++)A(x,g(P.xaxes[_]),P);if(P.yaxis)j=g(P.yaxis),A(x,j,P);else if(P.yaxes)for(_=0;_<P.yaxes.length;_++)A(x,g(P.yaxes[_]),P);if("funnel"===P.type?"h"===P.orientation?(N&&(w[N]=!0),j&&(O[j]=!0)):j&&(M[j]=!0):"image"===P.type?(j&&(k[j]=!0),N&&(k[N]=!0)):(j&&(T[j]=!0,E[j]=!0),b(P,"carpet")&&("carpet"!==P.type||P._cheater)||N&&(S[N]=!0)),"carpet"===P.type&&P._cheater&&N&&(w[N]=!0),b(P,"2dMap")&&(L[N]=!0,L[j]=!0),b(P,"oriented"))D["h"===P.orientation?j:N]=!0}}var R=t._subplots,z=R.xaxis,F=R.yaxis,B=r.simpleMap(z,g),H=r.simpleMap(F,g),U=B.concat(H),V=i.background;z.length&&F.length&&(V=r.coerce(e,t,l,"plot_bgcolor"));var Y,W,G,X,q,Z=i.combine(V,t.paper_bgcolor);function Q(){var e=x[Y]||[];q._traceIndices=e.map((function(e){return e._expandedIndex})),q._annIndices=[],q._shapeIndices=[],q._imgIndices=[],q._subplotsWith=[],q._counterAxes=[],q._name=q._attr=Y,q._id=W}function K(e,t){return r.coerce(X,q,u,e,t)}function J(e,t){return r.coerce2(X,q,u,e,t)}function $(e){return"x"===e?F:z}function ee(t,n){for(var r="x"===t?B:H,i=[],o=0;o<r.length;o++){var a=r[o];a===n||(e[a]||{}).overlaying||i.push(v(a))}return i}var te={x:$("x"),y:$("y")},ne=te.x.concat(te.y),re={},ie=[];function oe(){var e=X.matches;m.test(e)&&-1===ne.indexOf(e)&&(re[e]=X.type,ie=Object.keys(re))}var ae=a(e,t,n),se=o(ae);for(y=0;y<U.length;y++){Y=U[y],W=v(Y),G=Y.charAt(0),r.isPlainObject(e[Y])||(e[Y]={}),X=e[Y],q=s.newContainer(t,Y,G+"axis"),Q();var le="x"===G&&!S[Y]&&w[Y]||"y"===G&&!T[Y]&&M[Y],ue="y"===G&&(!E[Y]&&O[Y]||k[Y]),ce={letter:G,font:t.font,outerTicks:L[Y],showGrid:!D[Y],data:x[Y]||[],bgColor:Z,calendar:t.calendar,automargin:!0,visibleDflt:le,reverseDflt:ue,autotypenumbersDflt:I,splomStash:((t._splomAxes||{})[G]||{})[W]};K("uirevision",t.uirevision),c(X,q,K,ce),d(X,q,K,ce,t);var de=se&&G===ae.charAt(0),he=J("spikecolor",se?q.color:void 0),fe=J("spikethickness",se?1.5:void 0),pe=J("spikedash",se?"dot":void 0),ge=J("spikemode",se?"across":void 0),ve=J("spikesnap",se?"hovered data":void 0);K("showspikes",!!(de||he||fe||pe||ge||ve))||(delete q.spikecolor,delete q.spikethickness,delete q.spikedash,delete q.spikemode,delete q.spikesnap),f(X,q,K,{letter:G,counterAxes:te[G],overlayableAxes:ee(G,Y),grid:t.grid}),K("title.standoff"),oe(),q._input=X}for(y=0;y<ie.length;){W=ie[y++],G=(Y=g(W)).charAt(0),r.isPlainObject(e[Y])||(e[Y]={}),X=e[Y],q=s.newContainer(t,Y,G+"axis"),Q();var me={letter:G,font:t.font,outerTicks:L[Y],showGrid:!D[Y],data:[],bgColor:Z,calendar:t.calendar,automargin:!0,visibleDflt:!1,reverseDflt:!1,autotypenumbersDflt:I,splomStash:((t._splomAxes||{})[G]||{})[W]};K("uirevision",t.uirevision),q.type=re[W]||"linear",d(X,q,K,me,t),f(X,q,K,{letter:G,counterAxes:te[G],overlayableAxes:ee(G,Y),grid:t.grid}),K("fixedrange"),oe(),q._input=X}var ye=C("rangeslider","handleDefaults"),be=C("rangeselector","handleDefaults");for(y=0;y<B.length;y++)Y=B[y],X=e[Y],q=t[Y],ye(e,t,Y),"date"===q.type&&be(X,q,t,H,q.calendar),K("fixedrange");for(y=0;y<H.length;y++){Y=H[y],X=e[Y],q=t[Y];var Ce=t[g(q.anchor)];K("fixedrange",C("rangeslider","isVisible")(Ce))}h.handleDefaults(e,t,{axIds:ne.concat(ie).sort(p.idSort),axHasImage:k})}},function(e,t,n){"use strict";var r=n(123).mix,i=n(133).lightFraction,o=n(10);e.exports=function(e,t,n,a){var s=(a=a||{}).dfltColor;function l(n,r){return o.coerce2(e,t,a.attributes,n,r)}var u=l("linecolor",s),c=l("linewidth");n("showline",a.showLine||!!u||!!c)||(delete t.linecolor,delete t.linewidth);var d=l("gridcolor",r(s,a.bgColor,a.blend||i).toRgbString()),h=l("gridwidth");if(n("showgrid",a.showGrid||!!d||!!h)||(delete t.gridcolor,delete t.gridwidth),!a.noZeroLine){var f=l("zerolinecolor",s),p=l("zerolinewidth");n("zeroline",a.showGrid||!!f||!!p)||(delete t.zerolinecolor,delete t.zerolinewidth)}}},function(e,t,n){"use strict";var r=n(39),i=n(27),o=n(10),a=n(56),s=n(55);e.exports=function(e,t,n,l){var u=e._fullLayout;if(0!==t.length){var c,d,h,f;l&&(c=l());var p=r.ease(n.easing);return e._transitionData._interruptCallbacks.push((function(){return window.cancelAnimationFrame(f),f=null,function(){for(var n={},r=0;r<t.length;r++){var o=t[r],a=o.plotinfo.xaxis,s=o.plotinfo.yaxis;o.xr0&&(n[a._name+".range"]=o.xr0.slice()),o.yr0&&(n[s._name+".range"]=o.yr0.slice())}return i.call("relayout",e,n).then((function(){for(var e=0;e<t.length;e++)g(t[e].plotinfo)}))}()})),d=Date.now(),f=window.requestAnimationFrame((function r(){h=Date.now();for(var o=Math.min(1,(h-d)/n.duration),a=p(o),s=0;s<t.length;s++)v(t[s],a);h-d>n.duration?(!function(){for(var n={},r=0;r<t.length;r++){var o=t[r],a=o.plotinfo.xaxis,s=o.plotinfo.yaxis;o.xr1&&(n[a._name+".range"]=o.xr1.slice()),o.yr1&&(n[s._name+".range"]=o.yr1.slice())}c&&c(),i.call("relayout",e,n).then((function(){for(var e=0;e<t.length;e++)g(t[e].plotinfo)}))}(),f=window.cancelAnimationFrame(r)):f=window.requestAnimationFrame(r)})),Promise.resolve()}function g(e){var t=e.xaxis,n=e.yaxis;u._defs.select("#"+e.clipId+"> rect").call(a.setTranslate,0,0).call(a.setScale,1,1),e.plot.call(a.setTranslate,t._offset,n._offset).call(a.setScale,1,1);var r=e.plot.selectAll(".scatterlayer .trace");r.selectAll(".point").call(a.setPointGroupScale,1,1),r.selectAll(".textpoint").call(a.setTextPointsScale,1,1),r.call(a.hideOutsideRangePoints,e)}function v(t,n){var r=t.plotinfo,i=r.xaxis,l=r.yaxis,u=i._length,c=l._length,d=!!t.xr1,h=!!t.yr1,f=[];if(d){var p=o.simpleMap(t.xr0,i.r2l),g=o.simpleMap(t.xr1,i.r2l),v=p[1]-p[0],m=g[1]-g[0];f[0]=(p[0]*(1-n)+n*g[0]-p[0])/(p[1]-p[0])*u,f[2]=u*(1-n+n*m/v),i.range[0]=i.l2r(p[0]*(1-n)+n*g[0]),i.range[1]=i.l2r(p[1]*(1-n)+n*g[1])}else f[0]=0,f[2]=u;if(h){var y=o.simpleMap(t.yr0,l.r2l),b=o.simpleMap(t.yr1,l.r2l),C=y[1]-y[0],A=b[1]-b[0];f[1]=(y[1]*(1-n)+n*b[1]-y[1])/(y[0]-y[1])*c,f[3]=c*(1-n+n*A/C),l.range[0]=i.l2r(y[0]*(1-n)+n*b[0]),l.range[1]=l.l2r(y[1]*(1-n)+n*b[1])}else f[1]=0,f[3]=c;s.drawOne(e,i,{skipTitle:!0}),s.drawOne(e,l,{skipTitle:!0}),s.redrawComponents(e,[i._id,l._id]);var _=d?u/f[2]:1,I=h?c/f[3]:1,x=d?f[0]:0,w=h?f[1]:0,M=d?f[0]/f[2]*u:0,S=h?f[1]/f[3]*c:0,T=i._offset-M,E=l._offset-S;r.clipRect.call(a.setTranslate,x,w).call(a.setScale,1/_,1/I),r.plot.call(a.setTranslate,T,E).call(a.setScale,_,I),a.setPointGroupScale(r.zoomScalePts,1/_,1/I),a.setTextPointsScale(r.zoomScaleTxt,1/_,1/I)}s.redrawComponents(e)}},function(e,t,n){"use strict";e.exports={moduleType:"component",name:"legend",layoutAttributes:n(995),supplyLayoutDefaults:n(994),draw:n(996),style:n(998)}},function(e,t,n){"use strict";var r=n(560),i=n(1740);e.exports={moduleType:"component",name:"annotations",layoutAttributes:n(392),supplyLayoutDefaults:n(1741),includeBasePlot:n(562)("annotations"),calcAutorange:n(1742),draw:r.draw,drawOne:r.drawOne,drawRaw:r.drawRaw,hasClickToShow:i.hasClickToShow,onClick:i.onClick,convertCoords:n(1743)}},function(e,t,n){"use strict";var r=n(39),i=n(41),o=n(1021),a=n(10),s=a.strScale,l=a.strRotate,u=a.strTranslate;e.exports=function(e,t,n){var a,c,d,h,f=e.node(),p=o[n.arrowhead||0],g=o[n.startarrowhead||0],v=(n.arrowwidth||1)*(n.arrowsize||1),m=(n.arrowwidth||1)*(n.startarrowsize||1),y=t.indexOf("start")>=0,b=t.indexOf("end")>=0,C=p.backoff*v+n.standoff,A=g.backoff*m+n.startstandoff;if("line"===f.nodeName){a={x:+e.attr("x1"),y:+e.attr("y1")},c={x:+e.attr("x2"),y:+e.attr("y2")};var _=a.x-c.x,I=a.y-c.y;if(h=(d=Math.atan2(I,_))+Math.PI,C&&A&&C+A>Math.sqrt(_*_+I*I))return void P();if(C){if(C*C>_*_+I*I)return void P();var x=C*Math.cos(d),w=C*Math.sin(d);c.x+=x,c.y+=w,e.attr({x2:c.x,y2:c.y})}if(A){if(A*A>_*_+I*I)return void P();var M=A*Math.cos(d),S=A*Math.sin(d);a.x-=M,a.y-=S,e.attr({x1:a.x,y1:a.y})}}else if("path"===f.nodeName){var T=f.getTotalLength(),E="";if(T<C+A)return void P();var O=f.getPointAtLength(0),k=f.getPointAtLength(.1);d=Math.atan2(O.y-k.y,O.x-k.x),a=f.getPointAtLength(Math.min(A,T)),E="0px,"+A+"px,";var L=f.getPointAtLength(T),D=f.getPointAtLength(T-.1);h=Math.atan2(L.y-D.y,L.x-D.x),c=f.getPointAtLength(Math.max(0,T-C)),E+=T-(E?A+C:C)+"px,"+T+"px",e.style("stroke-dasharray",E)}function P(){e.style("stroke-dasharray","0px,100px")}function N(t,o,a,c){t.path&&(t.noRotate&&(a=0),r.select(f.parentNode).append("path").attr({class:e.attr("class"),d:t.path,transform:u(o.x,o.y)+l(180*a/Math.PI)+s(c)}).style({fill:i.rgb(n.arrowcolor),"stroke-width":0}))}y&&N(g,a,d,m),b&&N(p,c,h,v)}},function(e,t,n){"use strict";var r=n(10),i=n(27),o=n(64).arrayEditor;function a(e,t){var n,r,i,o,a,l,u,c=e._fullLayout.annotations,d=[],h=[],f=[],p=(t||[]).length;for(n=0;n<c.length;n++)if(o=(i=c[n]).clicktoshow){for(r=0;r<p;r++)if(l=(a=t[r]).xaxis,u=a.yaxis,l._id===i.xref&&u._id===i.yref&&l.d2r(a.x)===s(i._xclick,l)&&u.d2r(a.y)===s(i._yclick,u)){(i.visible?"onout"===o?h:f:d).push(n);break}r===p&&i.visible&&"onout"===o&&h.push(n)}return{on:d,off:h,explicitOff:f}}function s(e,t){return"log"===t.type?t.l2r(e):t.d2r(e)}e.exports={hasClickToShow:function(e,t){var n=a(e,t);return n.on.length>0||n.explicitOff.length>0},onClick:function(e,t){var n,s,l=a(e,t),u=l.on,c=l.off.concat(l.explicitOff),d={},h=e._fullLayout.annotations;if(!u.length&&!c.length)return;for(n=0;n<u.length;n++)(s=o(e.layout,"annotations",h[u[n]])).modifyItem("visible",!0),r.extendFlat(d,s.getUpdateObj());for(n=0;n<c.length;n++)(s=o(e.layout,"annotations",h[c[n]])).modifyItem("visible",!1),r.extendFlat(d,s.getUpdateObj());return i.call("update",e,{},d)}}},function(e,t,n){"use strict";var r=n(10),i=n(55),o=n(178),a=n(1022),s=n(392);function l(e,t,n){function o(n,i){return r.coerce(e,t,s,n,i)}var l=o("visible"),u=o("clicktoshow");if(l||u){a(e,t,n,o);for(var c=t.showarrow,d=["x","y"],h=[-10,-30],f={_fullLayout:n},p=0;p<2;p++){var g=d[p],v=i.coerceRef(e,t,f,g,"","paper");if("paper"!==v)i.getFromId(f,v)._annIndices.push(t._index);if(i.coercePosition(t,f,o,v,g,.5),c){var m="a"+g,y=i.coerceRef(e,t,f,m,"pixel",["pixel","paper"]);"pixel"!==y&&y!==v&&(y=t[m]="pixel");var b="pixel"===y?h[p]:.4;i.coercePosition(t,f,o,y,m,b)}o(g+"anchor"),o(g+"shift")}if(r.noneOrAll(e,t,["x","y"]),c&&r.noneOrAll(e,t,["ax","ay"]),u){var C=o("xclick"),A=o("yclick");t._xclick=void 0===C?t.x:i.cleanPosition(C,f,t.xref),t._yclick=void 0===A?t.y:i.cleanPosition(A,f,t.yref)}}}e.exports=function(e,t){o(e,t,{name:"annotations",handleItemDefaults:l})}},function(e,t,n){"use strict";var r=n(10),i=n(55),o=n(560).draw;function a(e){var t=e._fullLayout;r.filterVisible(t.annotations).forEach((function(t){var n=i.getFromId(e,t.xref),r=i.getFromId(e,t.yref),o=i.getRefType(t.xref),a=i.getRefType(t.yref);t._extremes={},"range"===o&&s(t,n),"range"===a&&s(t,r)}))}function s(e,t){var n,r=t._id,o=r.charAt(0),a=e[o],s=e["a"+o],l=e[o+"ref"],u=e["a"+o+"ref"],c=e["_"+o+"padplus"],d=e["_"+o+"padminus"],h={x:1,y:-1}[o]*e[o+"shift"],f=3*e.arrowsize*e.arrowwidth||0,p=f+h,g=f-h,v=3*e.startarrowsize*e.arrowwidth||0,m=v+h,y=v-h;if(u===l){var b=i.findExtremes(t,[t.r2c(a)],{ppadplus:p,ppadminus:g}),C=i.findExtremes(t,[t.r2c(s)],{ppadplus:Math.max(c,m),ppadminus:Math.max(d,y)});n={min:[b.min[0],C.min[0]],max:[b.max[0],C.max[0]]}}else m=s?m+s:m,y=s?y-s:y,n=i.findExtremes(t,[t.r2c(a)],{ppadplus:Math.max(c,p,m),ppadminus:Math.max(d,g,y)});e._extremes[r]=n}e.exports=function(e){var t=e._fullLayout;if(r.filterVisible(t.annotations).length&&e._fullData.length)return r.syncOrAsync([o,a],e)}},function(e,t,n){"use strict";var r=n(42),i=n(523);e.exports=function(e,t,n,o){t=t||{};var a="log"===n&&"linear"===t.type,s="linear"===n&&"log"===t.type;if(a||s)for(var l,u,c=e._fullLayout.annotations,d=t._id.charAt(0),h=0;h<c.length;h++)l=c[h],u="annotations["+h+"].",l[d+"ref"]===t._id&&f(d),l["a"+d+"ref"]===t._id&&f("a"+d);function f(e){var n=l[e],s=null;s=a?i(n,t.range):Math.pow(10,n),r(s)||(s=null),o(u+e,s)}}},function(e,t,n){"use strict";var r=n(27),i=n(10);e.exports={moduleType:"component",name:"annotations3d",schema:{subplots:{scene:{annotations:n(563)}}},layoutAttributes:n(563),handleDefaults:n(1745),includeBasePlot:function(e,t){var n=r.subplotsRegistry.gl3d;if(!n)return;for(var o=n.attrRegex,a=Object.keys(e),s=0;s<a.length;s++){var l=a[s];o.test(l)&&(e[l].annotations||[]).length&&(i.pushUnique(t._basePlotModules,n),i.pushUnique(t._subplots.gl3d,l))}},convert:n(1746),draw:n(1747)}},function(e,t,n){"use strict";var r=n(10),i=n(55),o=n(178),a=n(1022),s=n(563);function l(e,t,n,o){function l(n,i){return r.coerce(e,t,s,n,i)}function u(e){var r=e+"axis",o={_fullLayout:{}};return o._fullLayout[r]=n[r],i.coercePosition(t,o,l,e,e,.5)}l("visible")&&(a(e,t,o.fullLayout,l),u("x"),u("y"),u("z"),r.noneOrAll(e,t,["x","y","z"]),t.xref="x",t.yref="y",t.zref="z",l("xanchor"),l("yanchor"),l("xshift"),l("yshift"),t.showarrow&&(t.axref="pixel",t.ayref="pixel",l("ax",-10),l("ay",-30),r.noneOrAll(e,t,["ax","ay"])))}e.exports=function(e,t,n){o(e,t,{name:"annotations",handleItemDefaults:l,fullLayout:n.fullLayout})}},function(e,t,n){"use strict";var r=n(10),i=n(55);function o(e,t){var n=t.fullSceneLayout.domain,o=t.fullLayout._size,a={pdata:null,type:"linear",autorange:!1,range:[-1/0,1/0]};e._xa={},r.extendFlat(e._xa,a),i.setConvert(e._xa),e._xa._offset=o.l+n.x[0]*o.w,e._xa.l2p=function(){return.5*(1+e._pdata[0]/e._pdata[3])*o.w*(n.x[1]-n.x[0])},e._ya={},r.extendFlat(e._ya,a),i.setConvert(e._ya),e._ya._offset=o.t+(1-n.y[1])*o.h,e._ya.l2p=function(){return.5*(1-e._pdata[1]/e._pdata[3])*o.h*(n.y[1]-n.y[0])}}e.exports=function(e){for(var t=e.fullSceneLayout.annotations,n=0;n<t.length;n++)o(t[n],e);e.fullLayout._infolayer.selectAll(".annotation-"+e.id).remove()}},function(e,t,n){"use strict";var r=n(560).drawRaw,i=n(1023),o=["x","y","z"];e.exports=function(e){for(var t=e.fullSceneLayout,n=e.dataScale,a=t.annotations,s=0;s<a.length;s++){for(var l=a[s],u=!1,c=0;c<3;c++){var d=o[c],h=l[d],f=t[d+"axis"].r2fraction(h);if(f<0||f>1){u=!0;break}}u?e.fullLayout._infolayer.select(".annotation-"+e.id+'[data-index="'+s+'"]').remove():(l._pdata=i(e.glplot.cameraParams,[t.xaxis.r2l(l.x)*n[0],t.yaxis.r2l(l.y)*n[1],t.zaxis.r2l(l.z)*n[2]]),r(e.graphDiv,l,s,e.id,l._xa,l._ya))}}},function(e,t,n){"use strict";var r=n(1001);e.exports={moduleType:"component",name:"shapes",layoutAttributes:n(1024),supplyLayoutDefaults:n(1749),supplyDrawNewShapeDefaults:n(1750),includeBasePlot:n(562)("shapes"),calcAutorange:n(1751),draw:r.draw,drawOne:r.drawOne}},function(e,t,n){"use strict";var r=n(10),i=n(55),o=n(178),a=n(1024),s=n(554);function l(e,t,n){function o(n,i){return r.coerce(e,t,a,n,i)}if(o("visible")){var l=o("path"),u=o("type",l?"path":"rect");"path"!==t.type&&delete t.path,o("editable"),o("layer"),o("opacity"),o("fillcolor"),o("fillrule"),o("line.width")&&(o("line.color"),o("line.dash"));for(var c=o("xsizemode"),d=o("ysizemode"),h=["x","y"],f=0;f<2;f++){var p,g,v,m=h[f],y=m+"anchor",b="x"===m?c:d,C={_fullLayout:n},A=i.coerceRef(e,t,C,m,void 0,"paper");if("range"===i.getRefType(A)?((p=i.getFromId(C,A))._shapeIndices.push(t._index),v=s.rangeToShapePosition(p),g=s.shapePositionToRange(p)):g=v=r.identity,"path"!==u){var _=m+"0",I=m+"1",x=e[_],w=e[I];e[_]=g(e[_],!0),e[I]=g(e[I],!0),"pixel"===b?(o(_,0),o(I,10)):(i.coercePosition(t,C,o,A,_,.25),i.coercePosition(t,C,o,A,I,.75)),t[_]=v(t[_]),t[I]=v(t[I]),e[_]=x,e[I]=w}if("pixel"===b){var M=e[y];e[y]=g(e[y],!0),i.coercePosition(t,C,o,A,y,.25),t[y]=v(t[y]),e[y]=M}}"path"===u?o("path"):r.noneOrAll(e,t,["x0","x1","y0","y1"])}}e.exports=function(e,t){o(e,t,{name:"shapes",handleItemDefaults:l})}},function(e,t,n){"use strict";var r=n(41);e.exports=function(e,t,n){if(n("newshape.drawdirection"),n("newshape.layer"),n("newshape.fillcolor"),n("newshape.fillrule"),n("newshape.opacity"),n("newshape.line.width")){var i=(e||{}).plot_bgcolor||"#FFF";n("newshape.line.color",r.contrast(i)),n("newshape.line.dash")}n("activeshape.fillcolor"),n("activeshape.opacity")}},function(e,t,n){"use strict";var r=n(10),i=n(55),o=n(553),a=n(554);function s(e){return u(e.line.width,e.xsizemode,e.x0,e.x1,e.path,!1)}function l(e){return u(e.line.width,e.ysizemode,e.y0,e.y1,e.path,!0)}function u(e,t,n,i,s,l){var u=e/2,c=l;if("pixel"===t){var d=s?a.extractPathCoords(s,l?o.paramIsY:o.paramIsX):[n,i],h=r.aggNums(Math.max,null,d),f=r.aggNums(Math.min,null,d),p=f<0?Math.abs(f)+u:u,g=h>0?h+u:u;return{ppad:u,ppadplus:c?p:g,ppadminus:c?g:p}}return{ppad:u}}function c(e,t,n,r,i){var s="category"===e.type||"multicategory"===e.type?e.r2c:e.d2c;if(void 0!==t)return[s(t),s(n)];if(r){var l,u,c,d,h=1/0,f=-1/0,p=r.match(o.segmentRE);for("date"===e.type&&(s=a.decodeDate(s)),l=0;l<p.length;l++)void 0!==(u=i[p[l].charAt(0)].drawn)&&(!(c=p[l].substr(1).match(o.paramRE))||c.length<u||((d=s(c[u]))<h&&(h=d),d>f&&(f=d)));return f>=h?[h,f]:void 0}}e.exports=function(e){var t=e._fullLayout,n=r.filterVisible(t.shapes);if(n.length&&e._fullData.length)for(var a=0;a<n.length;a++){var u,d,h=n[a];h._extremes={};var f=i.getRefType(h.xref),p=i.getRefType(h.yref);if("paper"!==h.xref&&"domain"!==f){var g="pixel"===h.xsizemode?h.xanchor:h.x0,v="pixel"===h.xsizemode?h.xanchor:h.x1;(d=c(u=i.getFromId(e,h.xref),g,v,h.path,o.paramIsX))&&(h._extremes[u._id]=i.findExtremes(u,d,s(h)))}if("paper"!==h.yref&&"domain"!==p){var m="pixel"===h.ysizemode?h.yanchor:h.y0,y="pixel"===h.ysizemode?h.yanchor:h.y1;(d=c(u=i.getFromId(e,h.yref),m,y,h.path,o.paramIsY))&&(h._extremes[u._id]=i.findExtremes(u,d,l(h)))}}}},function(e,t,n){"use strict";e.exports={moduleType:"component",name:"images",layoutAttributes:n(1025),supplyLayoutDefaults:n(1753),includeBasePlot:n(562)("images"),draw:n(1754),convertCoords:n(1755)}},function(e,t,n){"use strict";var r=n(10),i=n(55),o=n(178),a=n(1025);function s(e,t,n){function o(n,i){return r.coerce(e,t,a,n,i)}var s=o("source");if(!o("visible",!!s))return t;o("layer"),o("xanchor"),o("yanchor"),o("sizex"),o("sizey"),o("sizing"),o("opacity");for(var l={_fullLayout:n},u=["x","y"],c=0;c<2;c++){var d=u[c],h=i.coerceRef(e,t,l,d,"paper",void 0);if("paper"!==h)i.getFromId(l,h)._imgIndices.push(t._index);i.coercePosition(t,l,o,h,d,0)}return t}e.exports=function(e,t){o(e,t,{name:"images",handleItemDefaults:s})}},function(e,t,n){"use strict";var r=n(39),i=n(56),o=n(55),a=n(83),s=n(196);e.exports=function(e){var t,n,l=e._fullLayout,u=[],c={},d=[];for(n=0;n<l.images.length;n++){var h=l.images[n];if(h.visible)if("below"===h.layer&&"paper"!==h.xref&&"paper"!==h.yref){t=a.ref2id(h.xref)+a.ref2id(h.yref);var f=l._plots[t];if(!f){d.push(h);continue}f.mainplot&&(t=f.mainplot.id),c[t]||(c[t]=[]),c[t].push(h)}else"above"===h.layer?u.push(h):d.push(h)}var p={left:{sizing:"xMin",offset:0},center:{sizing:"xMid",offset:-.5},right:{sizing:"xMax",offset:-1}},g={top:{sizing:"YMin",offset:0},middle:{sizing:"YMid",offset:-.5},bottom:{sizing:"YMax",offset:-1}};function v(t){var n=r.select(this);if(this._imgSrc!==t.source)if(n.attr("xmlns",s.svg),t.source&&"data:"===t.source.slice(0,5))n.attr("xlink:href",t.source),this._imgSrc=t.source;else{var i=new Promise(function(e){var r=new Image;function i(){n.remove(),e()}this.img=r,r.setAttribute("crossOrigin","anonymous"),r.onerror=i,r.onload=function(){var t=document.createElement("canvas");t.width=this.width,t.height=this.height,t.getContext("2d").drawImage(this,0,0);var r=t.toDataURL("image/png");n.attr("xlink:href",r),e()},n.on("error",i),r.src=t.source,this._imgSrc=t.source}.bind(this));e._promises.push(i)}}function m(t){var n,a,s=r.select(this),u=o.getFromId(e,t.xref),c=o.getFromId(e,t.yref),d="domain"===o.getRefType(t.xref),h="domain"===o.getRefType(t.yref),f=l._size;n=void 0!==u?"string"==typeof t.xref&&d?u._length*t.sizex:Math.abs(u.l2p(t.sizex)-u.l2p(0)):t.sizex*f.w,a=void 0!==c?"string"==typeof t.yref&&h?c._length*t.sizey:Math.abs(c.l2p(t.sizey)-c.l2p(0)):t.sizey*f.h;var v,m,y=n*p[t.xanchor].offset,b=a*g[t.yanchor].offset,C=p[t.xanchor].sizing+g[t.yanchor].sizing;switch(v=void 0!==u?"string"==typeof t.xref&&d?u._length*t.x+u._offset:u.r2p(t.x)+u._offset:t.x*f.w+f.l,v+=y,m=void 0!==c?"string"==typeof t.yref&&h?c._length*(1-t.y)+c._offset:c.r2p(t.y)+c._offset:f.h-t.y*f.h+f.t,m+=b,t.sizing){case"fill":C+=" slice";break;case"stretch":C="none"}s.attr({x:v,y:m,width:n,height:a,preserveAspectRatio:C,opacity:t.opacity});var A=(u&&"domain"!==o.getRefType(t.xref)?u._id:"")+(c&&"domain"!==o.getRefType(t.yref)?c._id:"");i.setClipUrl(s,A?"clip"+l._uid+A:null,e)}var y=l._imageLowerLayer.selectAll("image").data(d),b=l._imageUpperLayer.selectAll("image").data(u);y.enter().append("image"),b.enter().append("image"),y.exit().remove(),b.exit().remove(),y.each((function(e){v.bind(this)(e),m.bind(this)(e)})),b.each((function(e){v.bind(this)(e),m.bind(this)(e)}));var C=Object.keys(l._plots);for(n=0;n<C.length;n++){t=C[n];var A=l._plots[t];if(A.imagelayer){var _=A.imagelayer.selectAll("image").data(c[t]||[]);_.enter().append("image"),_.exit().remove(),_.each((function(e){v.bind(this)(e),m.bind(this)(e)}))}}}},function(e,t,n){"use strict";var r=n(42),i=n(523);e.exports=function(e,t,n,o){t=t||{};var a="log"===n&&"linear"===t.type,s="linear"===n&&"log"===t.type;if(a||s)for(var l,u,c=e._fullLayout.images,d=t._id.charAt(0),h=0;h<c.length;h++)if(u="images["+h+"].",(l=c[h])[d+"ref"]===t._id){var f=l[d],p=l["size"+d],g=null,v=null;if(a){g=i(f,t.range);var m=p/Math.pow(10,g)/2;v=2*Math.log(m+Math.sqrt(1+m*m))/Math.LN10}else v=(g=Math.pow(10,f))*(Math.pow(10,p/2)-Math.pow(10,-p/2));r(g)?r(v)||(v=null):(g=null,v=null),o(u+d,g),o(u+"size"+d,v)}}},function(e,t,n){"use strict";var r=n(564);e.exports={moduleType:"component",name:r.name,layoutAttributes:n(1026),supplyLayoutDefaults:n(1757),draw:n(1758)}},function(e,t,n){"use strict";var r=n(10),i=n(178),o=n(1026),a=n(564).name,s=o.buttons;function l(e,t,n){function a(n,i){return r.coerce(e,t,o,n,i)}a("visible",i(e,t,{name:"buttons",handleItemDefaults:u}).length>0)&&(a("active"),a("direction"),a("type"),a("showactive"),a("x"),a("y"),r.noneOrAll(e,t,["x","y"]),a("xanchor"),a("yanchor"),a("pad.t"),a("pad.r"),a("pad.b"),a("pad.l"),r.coerceFont(a,"font",n.font),a("bgcolor",n.paper_bgcolor),a("bordercolor"),a("borderwidth"))}function u(e,t){function n(n,i){return r.coerce(e,t,s,n,i)}n("visible","skip"===e.method||Array.isArray(e.args))&&(n("method"),n("args"),n("args2"),n("label"),n("execute"))}e.exports=function(e,t){i(e,t,{name:a,handleItemDefaults:l})}},function(e,t,n){"use strict";var r=n(39),i=n(82),o=n(41),a=n(56),s=n(10),l=n(96),u=n(64).arrayEditor,c=n(97).LINE_SPACING,d=n(564),h=n(1759);function f(e){return e._index}function p(e,t){return+e.attr(d.menuIndexAttrName)===t._index}function g(e,t,n,r,i,o,a,s){t.active=a,u(e.layout,d.name,t).applyUpdate("active",a),"buttons"===t.type?m(e,r,null,null,t):"dropdown"===t.type&&(i.attr(d.menuIndexAttrName,"-1"),v(e,r,i,o,t),s||m(e,r,i,o,t))}function v(e,t,n,r,i){var o=s.ensureSingle(t,"g",d.headerClassName,(function(e){e.style("pointer-events","all")})),l=i._dims,u=i.active,c=i.buttons[u]||d.blankHeaderOpts,h={y:i.pad.t,yPad:0,x:i.pad.l,xPad:0,index:0},f={width:l.headerWidth,height:l.headerHeight};o.call(y,i,c,e).call(M,i,h,f),s.ensureSingle(t,"text",d.headerArrowClassName,(function(e){e.attr("text-anchor","end").call(a.font,i.font).text(d.arrowSymbol[i.direction])})).attr({x:l.headerWidth-d.arrowOffsetX+i.pad.l,y:l.headerHeight/2+d.textOffsetY+i.pad.t}),o.on("click",(function(){n.call(S,String(p(n,i)?-1:i._index)),m(e,t,n,r,i)})),o.on("mouseover",(function(){o.call(_)})),o.on("mouseout",(function(){o.call(I,i)})),a.setTranslate(t,l.lx,l.ly)}function m(e,t,n,o,a){n||(n=t).attr("pointer-events","all");var l=function(e){return-1==+e.attr(d.menuIndexAttrName)}(n)&&"buttons"!==a.type?[]:a.buttons,u="dropdown"===a.type?d.dropdownButtonClassName:d.buttonClassName,c=n.selectAll("g."+u).data(s.filterVisible(l)),h=c.enter().append("g").classed(u,!0),f=c.exit();"dropdown"===a.type?(h.attr("opacity","0").transition().attr("opacity","1"),f.transition().attr("opacity","0").remove()):f.remove();var p=0,v=0,m=a._dims,b=-1!==["up","down"].indexOf(a.direction);"dropdown"===a.type&&(b?v=m.headerHeight+d.gapButtonHeader:p=m.headerWidth+d.gapButtonHeader),"dropdown"===a.type&&"up"===a.direction&&(v=-d.gapButtonHeader+d.gapButton-m.openHeight),"dropdown"===a.type&&"left"===a.direction&&(p=-d.gapButtonHeader+d.gapButton-m.openWidth);var C={x:m.lx+p+a.pad.l,y:m.ly+v+a.pad.t,yPad:d.gapButton,xPad:d.gapButton,index:0},x={l:C.x+a.borderwidth,t:C.y+a.borderwidth};c.each((function(s,l){var u=r.select(this);u.call(y,a,s,e).call(M,a,C),u.on("click",(function(){r.event.defaultPrevented||(s.execute&&(s.args2&&a.active===l?(g(e,a,0,t,n,o,-1),i.executeAPICommand(e,s.method,s.args2)):(g(e,a,0,t,n,o,l),i.executeAPICommand(e,s.method,s.args))),e.emit("plotly_buttonclicked",{menu:a,button:s,active:a.active}))})),u.on("mouseover",(function(){u.call(_)})),u.on("mouseout",(function(){u.call(I,a),c.call(A,a)}))})),c.call(A,a),b?(x.w=Math.max(m.openWidth,m.headerWidth),x.h=C.y-x.t):(x.w=C.x-x.l,x.h=Math.max(m.openHeight,m.headerHeight)),x.direction=a.direction,o&&(c.size()?function(e,t,n,r,i,o){var a,s,l,u=i.direction,c="up"===u||"down"===u,h=i._dims,f=i.active;if(c)for(s=0,l=0;l<f;l++)s+=h.heights[l]+d.gapButton;else for(a=0,l=0;l<f;l++)a+=h.widths[l]+d.gapButton;r.enable(o,a,s),r.hbar&&r.hbar.attr("opacity","0").transition().attr("opacity","1");r.vbar&&r.vbar.attr("opacity","0").transition().attr("opacity","1")}(0,0,0,o,a,x):function(e){var t=!!e.hbar,n=!!e.vbar;t&&e.hbar.transition().attr("opacity","0").each("end",(function(){t=!1,n||e.disable()}));n&&e.vbar.transition().attr("opacity","0").each("end",(function(){n=!1,t||e.disable()}))}(o))}function y(e,t,n,r){e.call(b,t).call(C,t,n,r)}function b(e,t){s.ensureSingle(e,"rect",d.itemRectClassName,(function(e){e.attr({rx:d.rx,ry:d.ry,"shape-rendering":"crispEdges"})})).call(o.stroke,t.bordercolor).call(o.fill,t.bgcolor).style("stroke-width",t.borderwidth+"px")}function C(e,t,n,r){var i=s.ensureSingle(e,"text",d.itemTextClassName,(function(e){e.attr({"text-anchor":"start","data-notex":1})})),o=n.label,u=r._fullLayout._meta;u&&(o=s.templateString(o,u)),i.call(a.font,t.font).text(o).call(l.convertToTspans,r)}function A(e,t){var n=t.active;e.each((function(e,i){var a=r.select(this);i===n&&t.showactive&&a.select("rect."+d.itemRectClassName).call(o.fill,d.activeColor)}))}function _(e){e.select("rect."+d.itemRectClassName).call(o.fill,d.hoverColor)}function I(e,t){e.select("rect."+d.itemRectClassName).call(o.fill,t.bgcolor)}function x(e,t){var n=t._dims={width1:0,height1:0,heights:[],widths:[],totalWidth:0,totalHeight:0,openWidth:0,openHeight:0,lx:0,ly:0},o=a.tester.selectAll("g."+d.dropdownButtonClassName).data(s.filterVisible(t.buttons));o.enter().append("g").classed(d.dropdownButtonClassName,!0);var u=-1!==["up","down"].indexOf(t.direction);o.each((function(i,o){var s=r.select(this);s.call(y,t,i,e);var h=s.select("."+d.itemTextClassName),f=h.node()&&a.bBox(h.node()).width,p=Math.max(f+d.textPadX,d.minWidth),g=t.font.size*c,v=l.lineCount(h),m=Math.max(g*v,d.minHeight)+d.textOffsetY;m=Math.ceil(m),p=Math.ceil(p),n.widths[o]=p,n.heights[o]=m,n.height1=Math.max(n.height1,m),n.width1=Math.max(n.width1,p),u?(n.totalWidth=Math.max(n.totalWidth,p),n.openWidth=n.totalWidth,n.totalHeight+=m+d.gapButton,n.openHeight+=m+d.gapButton):(n.totalWidth+=p+d.gapButton,n.openWidth+=p+d.gapButton,n.totalHeight=Math.max(n.totalHeight,m),n.openHeight=n.totalHeight)})),u?n.totalHeight-=d.gapButton:n.totalWidth-=d.gapButton,n.headerWidth=n.width1+d.arrowPadX,n.headerHeight=n.height1,"dropdown"===t.type&&(u?(n.width1+=d.arrowPadX,n.totalHeight=n.height1):n.totalWidth=n.width1,n.totalWidth+=d.arrowPadX),o.remove();var h=n.totalWidth+t.pad.l+t.pad.r,f=n.totalHeight+t.pad.t+t.pad.b,p=e._fullLayout._size;n.lx=p.l+p.w*t.x,n.ly=p.t+p.h*(1-t.y);var g="left";s.isRightAnchor(t)&&(n.lx-=h,g="right"),s.isCenterAnchor(t)&&(n.lx-=h/2,g="center");var v="top";s.isBottomAnchor(t)&&(n.ly-=f,v="bottom"),s.isMiddleAnchor(t)&&(n.ly-=f/2,v="middle"),n.totalWidth=Math.ceil(n.totalWidth),n.totalHeight=Math.ceil(n.totalHeight),n.lx=Math.round(n.lx),n.ly=Math.round(n.ly),i.autoMargin(e,w(t),{x:t.x,y:t.y,l:h*({right:1,center:.5}[g]||0),r:h*({left:1,center:.5}[g]||0),b:f*({top:1,middle:.5}[v]||0),t:f*({bottom:1,middle:.5}[v]||0)})}function w(e){return d.autoMarginIdRoot+e._index}function M(e,t,n,r){r=r||{};var i=e.select("."+d.itemRectClassName),o=e.select("."+d.itemTextClassName),s=t.borderwidth,u=n.index,h=t._dims;a.setTranslate(e,s+n.x,s+n.y);var f=-1!==["up","down"].indexOf(t.direction),p=r.height||(f?h.heights[u]:h.height1);i.attr({x:0,y:0,width:r.width||(f?h.width1:h.widths[u]),height:p});var g=t.font.size*c,v=(l.lineCount(o)-1)*g/2;l.positionText(o,d.textOffsetX,p/2-v+d.textOffsetY),f?n.y+=h.heights[u]+n.yPad:n.x+=h.widths[u]+n.xPad,n.index++}function S(e,t){e.attr(d.menuIndexAttrName,t||"-1").selectAll("g."+d.dropdownButtonClassName).remove()}e.exports=function(e){var t=e._fullLayout,n=s.filterVisible(t[d.name]);function o(t){i.autoMargin(e,w(t))}var a=t._menulayer.selectAll("g."+d.containerClassName).data(n.length>0?[0]:[]);if(a.enter().append("g").classed(d.containerClassName,!0).style("cursor","pointer"),a.exit().each((function(){r.select(this).selectAll("g."+d.headerGroupClassName).each(o)})).remove(),0!==n.length){var l=a.selectAll("g."+d.headerGroupClassName).data(n,f);l.enter().append("g").classed(d.headerGroupClassName,!0);for(var u=s.ensureSingle(a,"g",d.dropdownButtonGroupClassName,(function(e){e.style("pointer-events","all")})),c=0;c<n.length;c++){var y=n[c];x(e,y)}var b="updatemenus"+t._uid,C=new h(e,u,b);l.enter().size()&&(u.node().parentNode.appendChild(u.node()),u.call(S)),l.exit().each((function(e){u.call(S),o(e)})).remove(),l.each((function(t){var n=r.select(this),o="dropdown"===t.type?u:null;i.manageCommandObserver(e,t,t.buttons,(function(r){g(e,t,t.buttons[r.index],n,o,C,r.index,!0)})),"dropdown"===t.type?(v(e,n,u,C,t),p(u,t)&&m(e,n,u,C,t)):m(e,n,null,null,t)}))}}},function(e,t,n){"use strict";e.exports=s;var r=n(39),i=n(41),o=n(56),a=n(10);function s(e,t,n){this.gd=e,this.container=t,this.id=n,this.position=null,this.translateX=null,this.translateY=null,this.hbar=null,this.vbar=null,this.bg=this.container.selectAll("rect.scrollbox-bg").data([0]),this.bg.exit().on(".drag",null).on("wheel",null).remove(),this.bg.enter().append("rect").classed("scrollbox-bg",!0).style("pointer-events","all").attr({opacity:0,x:0,y:0,width:0,height:0})}s.barWidth=2,s.barLength=20,s.barRadius=2,s.barPad=1,s.barColor="#808BA4",s.prototype.enable=function(e,t,n){var a=this.gd._fullLayout,l=a.width,u=a.height;this.position=e;var c,d,h,f,p=this.position.l,g=this.position.w,v=this.position.t,m=this.position.h,y=this.position.direction,b="down"===y,C="left"===y,A="up"===y,_=g,I=m;b||C||"right"===y||A||(this.position.direction="down",b=!0),b||A?(d=(c=p)+_,b?(h=v,I=(f=Math.min(h+I,u))-h):I=(f=v+I)-(h=Math.max(f-I,0))):(f=(h=v)+I,C?_=(d=p+_)-(c=Math.max(d-_,0)):(c=p,_=(d=Math.min(c+_,l))-c)),this._box={l:c,t:h,w:_,h:I};var x=g>_,w=s.barLength+2*s.barPad,M=s.barWidth+2*s.barPad,S=p,T=v+m;T+M>u&&(T=u-M);var E=this.container.selectAll("rect.scrollbar-horizontal").data(x?[0]:[]);E.exit().on(".drag",null).remove(),E.enter().append("rect").classed("scrollbar-horizontal",!0).call(i.fill,s.barColor),x?(this.hbar=E.attr({rx:s.barRadius,ry:s.barRadius,x:S,y:T,width:w,height:M}),this._hbarXMin=S+w/2,this._hbarTranslateMax=_-w):(delete this.hbar,delete this._hbarXMin,delete this._hbarTranslateMax);var O=m>I,k=s.barWidth+2*s.barPad,L=s.barLength+2*s.barPad,D=p+g,P=v;D+k>l&&(D=l-k);var N=this.container.selectAll("rect.scrollbar-vertical").data(O?[0]:[]);N.exit().on(".drag",null).remove(),N.enter().append("rect").classed("scrollbar-vertical",!0).call(i.fill,s.barColor),O?(this.vbar=N.attr({rx:s.barRadius,ry:s.barRadius,x:D,y:P,width:k,height:L}),this._vbarYMin=P+L/2,this._vbarTranslateMax=I-L):(delete this.vbar,delete this._vbarYMin,delete this._vbarTranslateMax);var j=this.id,R=c-.5,z=O?d+k+.5:d+.5,F=h-.5,B=x?f+M+.5:f+.5,H=a._topdefs.selectAll("#"+j).data(x||O?[0]:[]);if(H.exit().remove(),H.enter().append("clipPath").attr("id",j).append("rect"),x||O?(this._clipRect=H.select("rect").attr({x:Math.floor(R),y:Math.floor(F),width:Math.ceil(z)-Math.floor(R),height:Math.ceil(B)-Math.floor(F)}),this.container.call(o.setClipUrl,j,this.gd),this.bg.attr({x:p,y:v,width:g,height:m})):(this.bg.attr({width:0,height:0}),this.container.on("wheel",null).on(".drag",null).call(o.setClipUrl,null),delete this._clipRect),x||O){var U=r.behavior.drag().on("dragstart",(function(){r.event.sourceEvent.preventDefault()})).on("drag",this._onBoxDrag.bind(this));this.container.on("wheel",null).on("wheel",this._onBoxWheel.bind(this)).on(".drag",null).call(U);var V=r.behavior.drag().on("dragstart",(function(){r.event.sourceEvent.preventDefault(),r.event.sourceEvent.stopPropagation()})).on("drag",this._onBarDrag.bind(this));x&&this.hbar.on(".drag",null).call(V),O&&this.vbar.on(".drag",null).call(V)}this.setTranslate(t,n)},s.prototype.disable=function(){(this.hbar||this.vbar)&&(this.bg.attr({width:0,height:0}),this.container.on("wheel",null).on(".drag",null).call(o.setClipUrl,null),delete this._clipRect),this.hbar&&(this.hbar.on(".drag",null),this.hbar.remove(),delete this.hbar,delete this._hbarXMin,delete this._hbarTranslateMax),this.vbar&&(this.vbar.on(".drag",null),this.vbar.remove(),delete this.vbar,delete this._vbarYMin,delete this._vbarTranslateMax)},s.prototype._onBoxDrag=function(){var e=this.translateX,t=this.translateY;this.hbar&&(e-=r.event.dx),this.vbar&&(t-=r.event.dy),this.setTranslate(e,t)},s.prototype._onBoxWheel=function(){var e=this.translateX,t=this.translateY;this.hbar&&(e+=r.event.deltaY),this.vbar&&(t+=r.event.deltaY),this.setTranslate(e,t)},s.prototype._onBarDrag=function(){var e=this.translateX,t=this.translateY;if(this.hbar){var n=e+this._hbarXMin,i=n+this._hbarTranslateMax;e=(a.constrain(r.event.x,n,i)-n)/(i-n)*(this.position.w-this._box.w)}if(this.vbar){var o=t+this._vbarYMin,s=o+this._vbarTranslateMax;t=(a.constrain(r.event.y,o,s)-o)/(s-o)*(this.position.h-this._box.h)}this.setTranslate(e,t)},s.prototype.setTranslate=function(e,t){var n=this.position.w-this._box.w,r=this.position.h-this._box.h;if(e=a.constrain(e||0,0,n),t=a.constrain(t||0,0,r),this.translateX=e,this.translateY=t,this.container.call(o.setTranslate,this._box.l-this.position.l-e,this._box.t-this.position.t-t),this._clipRect&&this._clipRect.attr({x:Math.floor(this.position.l+e-.5),y:Math.floor(this.position.t+t-.5)}),this.hbar){var i=e/n;this.hbar.call(o.setTranslate,e+i*this._hbarTranslateMax,t)}if(this.vbar){var s=t/r;this.vbar.call(o.setTranslate,e,t+s*this._vbarTranslateMax)}}},function(e,t,n){"use strict";var r=n(393);e.exports={moduleType:"component",name:r.name,layoutAttributes:n(1027),supplyLayoutDefaults:n(1761),draw:n(1762)}},function(e,t,n){"use strict";var r=n(10),i=n(178),o=n(1027),a=n(393).name,s=o.steps;function l(e,t,n){function a(n,i){return r.coerce(e,t,o,n,i)}for(var s=i(e,t,{name:"steps",handleItemDefaults:u}),l=0,c=0;c<s.length;c++)s[c].visible&&l++;if(l<2?t.visible=!1:a("visible")){t._stepCount=l;var d=t._visibleSteps=r.filterVisible(s);(s[a("active")]||{}).visible||(t.active=d[0]._index),a("x"),a("y"),r.noneOrAll(e,t,["x","y"]),a("xanchor"),a("yanchor"),a("len"),a("lenmode"),a("pad.t"),a("pad.r"),a("pad.b"),a("pad.l"),r.coerceFont(a,"font",n.font),a("currentvalue.visible")&&(a("currentvalue.xanchor"),a("currentvalue.prefix"),a("currentvalue.suffix"),a("currentvalue.offset"),r.coerceFont(a,"currentvalue.font",t.font)),a("transition.duration"),a("transition.easing"),a("bgcolor"),a("activebgcolor"),a("bordercolor"),a("borderwidth"),a("ticklen"),a("tickwidth"),a("tickcolor"),a("minorticklen")}}function u(e,t){function n(n,i){return r.coerce(e,t,s,n,i)}if("skip"===e.method||Array.isArray(e.args)?n("visible"):t.visible=!1){n("method"),n("args");var i=n("label","step-"+t._index);n("value",i),n("execute")}}e.exports=function(e,t){i(e,t,{name:a,handleItemDefaults:l})}},function(e,t,n){"use strict";var r=n(39),i=n(82),o=n(41),a=n(56),s=n(10),l=s.strTranslate,u=n(96),c=n(64).arrayEditor,d=n(393),h=n(97),f=h.LINE_SPACING,p=h.FROM_TL,g=h.FROM_BR;function v(e){return d.autoMarginIdRoot+e._index}function m(e){return e._index}function y(e,t){var n=a.tester.selectAll("g."+d.labelGroupClass).data(t._visibleSteps);n.enter().append("g").classed(d.labelGroupClass,!0);var o=0,l=0;n.each((function(e){var n=A(r.select(this),{step:e},t).node();if(n){var i=a.bBox(n);l=Math.max(l,i.height),o=Math.max(o,i.width)}})),n.remove();var c=t._dims={};c.inputAreaWidth=Math.max(d.railWidth,d.gripHeight);var h=e._fullLayout._size;c.lx=h.l+h.w*t.x,c.ly=h.t+h.h*(1-t.y),"fraction"===t.lenmode?c.outerLength=Math.round(h.w*t.len):c.outerLength=t.len,c.inputAreaStart=0,c.inputAreaLength=Math.round(c.outerLength-t.pad.l-t.pad.r);var f=(c.inputAreaLength-2*d.stepInset)/(t._stepCount-1),m=o+d.labelPadding;if(c.labelStride=Math.max(1,Math.ceil(m/f)),c.labelHeight=l,c.currentValueMaxWidth=0,c.currentValueHeight=0,c.currentValueTotalHeight=0,c.currentValueMaxLines=1,t.currentvalue.visible){var y=a.tester.append("g");n.each((function(e){var n=b(y,t,e.label),r=n.node()&&a.bBox(n.node())||{width:0,height:0},i=u.lineCount(n);c.currentValueMaxWidth=Math.max(c.currentValueMaxWidth,Math.ceil(r.width)),c.currentValueHeight=Math.max(c.currentValueHeight,Math.ceil(r.height)),c.currentValueMaxLines=Math.max(c.currentValueMaxLines,i)})),c.currentValueTotalHeight=c.currentValueHeight+t.currentvalue.offset,y.remove()}c.height=c.currentValueTotalHeight+d.tickOffset+t.ticklen+d.labelOffset+c.labelHeight+t.pad.t+t.pad.b;var C="left";s.isRightAnchor(t)&&(c.lx-=c.outerLength,C="right"),s.isCenterAnchor(t)&&(c.lx-=c.outerLength/2,C="center");var _="top";s.isBottomAnchor(t)&&(c.ly-=c.height,_="bottom"),s.isMiddleAnchor(t)&&(c.ly-=c.height/2,_="middle"),c.outerLength=Math.ceil(c.outerLength),c.height=Math.ceil(c.height),c.lx=Math.round(c.lx),c.ly=Math.round(c.ly);var I={y:t.y,b:c.height*g[_],t:c.height*p[_]};"fraction"===t.lenmode?(I.l=0,I.xl=t.x-t.len*p[C],I.r=0,I.xr=t.x+t.len*g[C]):(I.x=t.x,I.l=c.outerLength*p[C],I.r=c.outerLength*g[C]),i.autoMargin(e,v(t),I)}function b(e,t,n){if(t.currentvalue.visible){var r,i,o=t._dims;switch(t.currentvalue.xanchor){case"right":r=o.inputAreaLength-d.currentValueInset-o.currentValueMaxWidth,i="left";break;case"center":r=.5*o.inputAreaLength,i="middle";break;default:r=d.currentValueInset,i="left"}var l=s.ensureSingle(e,"text",d.labelClass,(function(e){e.attr({"text-anchor":i,"data-notex":1})})),c=t.currentvalue.prefix?t.currentvalue.prefix:"";if("string"==typeof n)c+=n;else{var h=t.steps[t.active].label,p=t._gd._fullLayout._meta;p&&(h=s.templateString(h,p)),c+=h}t.currentvalue.suffix&&(c+=t.currentvalue.suffix),l.call(a.font,t.currentvalue.font).text(c).call(u.convertToTspans,t._gd);var g=u.lineCount(l),v=(o.currentValueMaxLines+1-g)*t.currentvalue.font.size*f;return u.positionText(l,r,v),l}}function C(e,t,n){s.ensureSingle(e,"rect",d.gripRectClass,(function(r){r.call(w,t,e,n).style("pointer-events","all")})).attr({width:d.gripWidth,height:d.gripHeight,rx:d.gripRadius,ry:d.gripRadius}).call(o.stroke,n.bordercolor).call(o.fill,n.bgcolor).style("stroke-width",n.borderwidth+"px")}function A(e,t,n){var r=s.ensureSingle(e,"text",d.labelClass,(function(e){e.attr({"text-anchor":"middle","data-notex":1})})),i=t.step.label,o=n._gd._fullLayout._meta;return o&&(i=s.templateString(i,o)),r.call(a.font,n.font).text(i).call(u.convertToTspans,n._gd),r}function _(e,t){var n=s.ensureSingle(e,"g",d.labelsClass),i=t._dims,o=n.selectAll("g."+d.labelGroupClass).data(i.labelSteps);o.enter().append("g").classed(d.labelGroupClass,!0),o.exit().remove(),o.each((function(e){var n=r.select(this);n.call(A,e,t),a.setTranslate(n,T(t,e.fraction),d.tickOffset+t.ticklen+t.font.size*f+d.labelOffset+i.currentValueTotalHeight)}))}function I(e,t,n,r,i){var o=Math.round(r*(n._stepCount-1)),a=n._visibleSteps[o]._index;a!==n.active&&x(e,t,n,a,!0,i)}function x(e,t,n,r,o,a){var s=n.active;n.active=r,c(e.layout,d.name,n).applyUpdate("active",r);var l=n.steps[n.active];t.call(S,n,a),t.call(b,n),e.emit("plotly_sliderchange",{slider:n,step:n.steps[n.active],interaction:o,previousActive:s}),l&&l.method&&o&&(t._nextMethod?(t._nextMethod.step=l,t._nextMethod.doCallback=o,t._nextMethod.doTransition=a):(t._nextMethod={step:l,doCallback:o,doTransition:a},t._nextMethodRaf=window.requestAnimationFrame((function(){var n=t._nextMethod.step;n.method&&(n.execute&&i.executeAPICommand(e,n.method,n.args),t._nextMethod=null,t._nextMethodRaf=null)}))))}function w(e,t,n){var i=n.node(),a=r.select(t);function s(){return n.data()[0]}e.on("mousedown",(function(){var e=s();t.emit("plotly_sliderstart",{slider:e});var l=n.select("."+d.gripRectClass);r.event.stopPropagation(),r.event.preventDefault(),l.call(o.fill,e.activebgcolor);var u=E(e,r.mouse(i)[0]);I(t,n,e,u,!0),e._dragging=!0,a.on("mousemove",(function(){var e=s(),o=E(e,r.mouse(i)[0]);I(t,n,e,o,!1)})),a.on("mouseup",(function(){var e=s();e._dragging=!1,l.call(o.fill,e.bgcolor),a.on("mouseup",null),a.on("mousemove",null),t.emit("plotly_sliderend",{slider:e,step:e.steps[e.active]})}))}))}function M(e,t){var n=e.selectAll("rect."+d.tickRectClass).data(t._visibleSteps),i=t._dims;n.enter().append("rect").classed(d.tickRectClass,!0),n.exit().remove(),n.attr({width:t.tickwidth+"px","shape-rendering":"crispEdges"}),n.each((function(e,n){var s=n%i.labelStride==0,l=r.select(this);l.attr({height:s?t.ticklen:t.minorticklen}).call(o.fill,t.tickcolor),a.setTranslate(l,T(t,n/(t._stepCount-1))-.5*t.tickwidth,(s?d.tickOffset:d.minorTickOffset)+i.currentValueTotalHeight)}))}function S(e,t,n){for(var r=e.select("rect."+d.gripRectClass),i=0,o=0;o<t._stepCount;o++)if(t._visibleSteps[o]._index===t.active){i=o;break}var a=T(t,i/(t._stepCount-1));if(!t._invokingCommand){var s=r;n&&t.transition.duration>0&&(s=s.transition().duration(t.transition.duration).ease(t.transition.easing)),s.attr("transform",l(a-.5*d.gripWidth,t._dims.currentValueTotalHeight))}}function T(e,t){var n=e._dims;return n.inputAreaStart+d.stepInset+(n.inputAreaLength-2*d.stepInset)*Math.min(1,Math.max(0,t))}function E(e,t){var n=e._dims;return Math.min(1,Math.max(0,(t-d.stepInset-n.inputAreaStart)/(n.inputAreaLength-2*d.stepInset-2*n.inputAreaStart)))}function O(e,t,n){var r=n._dims,i=s.ensureSingle(e,"rect",d.railTouchRectClass,(function(r){r.call(w,t,e,n).style("pointer-events","all")}));i.attr({width:r.inputAreaLength,height:Math.max(r.inputAreaWidth,d.tickOffset+n.ticklen+r.labelHeight)}).call(o.fill,n.bgcolor).attr("opacity",0),a.setTranslate(i,0,r.currentValueTotalHeight)}function k(e,t){var n=t._dims,r=n.inputAreaLength-2*d.railInset,i=s.ensureSingle(e,"rect",d.railRectClass);i.attr({width:r,height:d.railWidth,rx:d.railRadius,ry:d.railRadius,"shape-rendering":"crispEdges"}).call(o.stroke,t.bordercolor).call(o.fill,t.bgcolor).style("stroke-width",t.borderwidth+"px"),a.setTranslate(i,d.railInset,.5*(n.inputAreaWidth-d.railWidth)+n.currentValueTotalHeight)}e.exports=function(e){var t=e._fullLayout,n=function(e,t){for(var n=e[d.name],r=[],i=0;i<n.length;i++){var o=n[i];o.visible&&(o._gd=t,r.push(o))}return r}(t,e),o=t._infolayer.selectAll("g."+d.containerClassName).data(n.length>0?[0]:[]);function s(t){t._commandObserver&&(t._commandObserver.remove(),delete t._commandObserver),i.autoMargin(e,v(t))}if(o.enter().append("g").classed(d.containerClassName,!0).style("cursor","ew-resize"),o.exit().each((function(){r.select(this).selectAll("g."+d.groupClassName).each(s)})).remove(),0!==n.length){var l=o.selectAll("g."+d.groupClassName).data(n,m);l.enter().append("g").classed(d.groupClassName,!0),l.exit().each(s).remove();for(var u=0;u<n.length;u++){var c=n[u];y(e,c)}l.each((function(t){var n=r.select(this);!function(e){var t=e._dims;t.labelSteps=[];for(var n=e._stepCount,r=0;r<n;r+=t.labelStride)t.labelSteps.push({fraction:r/(n-1),step:e._visibleSteps[r]})}(t),i.manageCommandObserver(e,t,t._visibleSteps,(function(t){var r=n.data()[0];r.active!==t.index&&(r._dragging||x(e,n,r,t.index,!1,!0))})),function(e,t,n){(n.steps[n.active]||{}).visible||(n.active=n._visibleSteps[0]._index);t.call(b,n).call(k,n).call(_,n).call(M,n).call(O,e,n).call(C,e,n);var r=n._dims;a.setTranslate(t,r.lx+n.pad.l,r.ly+n.pad.t),t.call(S,n,!1),t.call(b,n)}(e,r.select(this),t)}))}}},function(e,t,n){"use strict";var r=n(10),i=n(565),o=n(1028),a=n(1764);e.exports={moduleType:"component",name:"rangeslider",schema:{subplots:{xaxis:{rangeslider:r.extendFlat({},i,{yaxis:o})}}},layoutAttributes:n(565),handleDefaults:n(1765),calcAutorange:n(1766),draw:n(1767),isVisible:a.isVisible,makeData:a.makeData,autoMarginOpts:a.autoMarginOpts}},function(e,t,n){"use strict";var r=n(83),i=n(96),o=n(566),a=n(97).LINE_SPACING,s=o.name;function l(e){var t=e&&e[s];return t&&t.visible}t.isVisible=l,t.makeData=function(e){var t=r.list({_fullLayout:e},"x",!0),n=e.margin,i=[];if(!e._has("gl2d"))for(var o=0;o<t.length;o++){var a=t[o];if(l(a)){i.push(a);var u=a[s];u._id=s+a._id,u._height=(e.height-n.b-n.t)*u.thickness,u._offsetShift=Math.floor(u.borderwidth/2)}}e._rangeSliderData=i},t.autoMarginOpts=function(e,t){var n=e._fullLayout,r=t[s],l=t._id.charAt(0),u=0,c=0;"bottom"===t.side&&(u=t._depth,t.title.text!==n._dfltTitle[l]&&(c=1.5*t.title.font.size+10+r._offsetShift,c+=(t.title.text.match(i.BR_TAG_ALL)||[]).length*t.title.font.size*a));return{x:0,y:t._counterDomainMin,l:0,r:0,t:0,b:r._height+u+Math.max(n.margin.b,c),pad:o.extraPad+2*r._offsetShift}}},function(e,t,n){"use strict";var r=n(10),i=n(64),o=n(83),a=n(565),s=n(1028);e.exports=function(e,t,n){var l=e[n],u=t[n];if(l.rangeslider||t._requestRangeslider[u._id]){r.isPlainObject(l.rangeslider)||(l.rangeslider={});var c,d,h=l.rangeslider,f=i.newContainer(u,"rangeslider");if(A("visible")){A("bgcolor",t.plot_bgcolor),A("bordercolor"),A("borderwidth"),A("thickness"),A("autorange",!u.isValidRange(h.range)),A("range");var p=t._subplots;if(p)for(var g=p.cartesian.filter((function(e){return e.substr(0,e.indexOf("y"))===o.name2id(n)})).map((function(e){return e.substr(e.indexOf("y"),e.length)})),v=r.simpleMap(g,o.id2name),m=0;m<v.length;m++){var y=v[m];c=h[y]||{},d=i.newContainer(f,y,"yaxis");var b,C=t[y];c.range&&C.isValidRange(c.range)&&(b="fixed"),"match"!==_("rangemode",b)&&_("range",C.range.slice())}f._input=h}}function A(e,t){return r.coerce(h,f,a,e,t)}function _(e,t){return r.coerce(c,d,s,e,t)}}},function(e,t,n){"use strict";var r=n(83).list,i=n(386).getAutoRange,o=n(566);e.exports=function(e){for(var t=r(e,"x",!0),n=0;n<t.length;n++){var a=t[n],s=a[o.name];s&&s.visible&&s.autorange&&(s._input.autorange=!0,s._input.range=s.range=i(e,a))}}},function(e,t,n){"use strict";var r=n(39),i=n(27),o=n(82),a=n(10),s=a.strTranslate,l=n(56),u=n(41),c=n(385),d=n(301),h=n(83),f=n(163),p=n(214),g=n(566);function v(e,t,n,r){var i=a.ensureSingle(e,"rect",g.bgClassName,(function(e){e.attr({x:0,y:0,"shape-rendering":"crispEdges"})})),o=r.borderwidth%2==0?r.borderwidth:r.borderwidth-1,u=-r._offsetShift,c=l.crispRound(t,r.borderwidth);i.attr({width:r._width+o,height:r._height+o,transform:s(u,u),fill:r.bgcolor,stroke:r.bordercolor,"stroke-width":c})}function m(e,t,n,r){var i=t._fullLayout;a.ensureSingleById(i._topdefs,"clipPath",r._clipId,(function(e){e.append("rect").attr({x:0,y:0})})).select("rect").attr({width:r._width,height:r._height})}function y(e,t,n,i){var s,u=t.calcdata,c=e.selectAll("g."+g.rangePlotClassName).data(n._subplotsWith,a.identity);c.enter().append("g").attr("class",(function(e){return g.rangePlotClassName+" "+e})).call(l.setClipUrl,i._clipId,t),c.order(),c.exit().remove(),c.each((function(e,a){var l=r.select(this),c=0===a,f=h.getFromId(t,e,"y"),p=f._name,g=i[p],v={data:[],layout:{xaxis:{type:n.type,domain:[0,1],range:i.range.slice(),calendar:n.calendar},width:i._width,height:i._height,margin:{t:0,b:0,l:0,r:0}},_context:t._context};n.rangebreaks&&(v.layout.xaxis.rangebreaks=n.rangebreaks),v.layout[p]={type:f.type,domain:[0,1],range:"match"!==g.rangemode?g.range.slice():f.range.slice(),calendar:f.calendar},f.rangebreaks&&(v.layout[p].rangebreaks=f.rangebreaks),o.supplyDefaults(v);var m=v._fullLayout.xaxis,y=v._fullLayout[p];m.clearCalc(),m.setScale(),y.clearCalc(),y.setScale();var b={id:e,plotgroup:l,xaxis:m,yaxis:y,isRangePlot:!0};c?s=b:(b.mainplot="xy",b.mainplotinfo=s),d.rangePlot(t,b,function(e,t){for(var n=[],r=0;r<e.length;r++){var i=e[r],o=i[0].trace;o.xaxis+o.yaxis===t&&n.push(i)}return n}(u,e))}))}function b(e,t,n,r,i){(a.ensureSingle(e,"rect",g.maskMinClassName,(function(e){e.attr({x:0,y:0,"shape-rendering":"crispEdges"})})).attr("height",r._height).call(u.fill,g.maskColor),a.ensureSingle(e,"rect",g.maskMaxClassName,(function(e){e.attr({y:0,"shape-rendering":"crispEdges"})})).attr("height",r._height).call(u.fill,g.maskColor),"match"!==i.rangemode)&&(a.ensureSingle(e,"rect",g.maskMinOppAxisClassName,(function(e){e.attr({y:0,"shape-rendering":"crispEdges"})})).attr("width",r._width).call(u.fill,g.maskOppAxisColor),a.ensureSingle(e,"rect",g.maskMaxOppAxisClassName,(function(e){e.attr({y:0,"shape-rendering":"crispEdges"})})).attr("width",r._width).style("border-top",g.maskOppBorder).call(u.fill,g.maskOppAxisColor))}function C(e,t,n,r){t._context.staticPlot||a.ensureSingle(e,"rect",g.slideBoxClassName,(function(e){e.attr({y:0,cursor:g.slideBoxCursor,"shape-rendering":"crispEdges"})})).attr({height:r._height,fill:g.slideBoxFill})}function A(e,t,n,r){var i=a.ensureSingle(e,"g",g.grabberMinClassName),o=a.ensureSingle(e,"g",g.grabberMaxClassName),s={x:0,width:g.handleWidth,rx:g.handleRadius,fill:u.background,stroke:u.defaultLine,"stroke-width":g.handleStrokeWidth,"shape-rendering":"crispEdges"},l={y:Math.round(r._height/4),height:Math.round(r._height/2)};a.ensureSingle(i,"rect",g.handleMinClassName,(function(e){e.attr(s)})).attr(l),a.ensureSingle(o,"rect",g.handleMaxClassName,(function(e){e.attr(s)})).attr(l);var c={width:g.grabAreaWidth,x:0,y:0,fill:g.grabAreaFill,cursor:t._context.staticPlot?void 0:g.grabAreaCursor};a.ensureSingle(i,"rect",g.grabAreaMinClassName,(function(e){e.attr(c)})).attr("height",r._height),a.ensureSingle(o,"rect",g.grabAreaMaxClassName,(function(e){e.attr(c)})).attr("height",r._height)}e.exports=function(e){for(var t=e._fullLayout,n=t._rangeSliderData,o=0;o<n.length;o++){var l=n[o][g.name];l._clipId=l._id+"-"+t._uid}var u=t._infolayer.selectAll("g."+g.containerClassName).data(n,(function(e){return e._name}));u.exit().each((function(e){var n=e[g.name];t._topdefs.select("#"+n._clipId).remove()})).remove(),0!==n.length&&(u.enter().append("g").classed(g.containerClassName,!0).attr("pointer-events","all"),u.each((function(n){var o=r.select(this),l=n[g.name],u=t[h.id2name(n.anchor)],d=l[h.id2name(n.anchor)];if(l.range){var _,I=a.simpleMap(l.range,n.r2l),x=a.simpleMap(n.range,n.r2l);_=x[0]<x[1]?[Math.min(I[0],x[0]),Math.max(I[1],x[1])]:[Math.max(I[0],x[0]),Math.min(I[1],x[1])],l.range=l._input.range=a.simpleMap(_,n.l2r)}n.cleanRange("rangeslider.range");var w=t._size,M=n.domain;l._width=w.w*(M[1]-M[0]);var S=Math.round(w.l+w.w*M[0]),T=Math.round(w.t+w.h*(1-n._counterDomainMin)+("bottom"===n.side?n._depth:0)+l._offsetShift+g.extraPad);o.attr("transform",s(S,T)),l._rl=a.simpleMap(l.range,n.r2l);var E=l._rl[0],O=l._rl[1],k=O-E;if(l.p2d=function(e){return e/l._width*k+E},l.d2p=function(e){return(e-E)/k*l._width},n.rangebreaks){var L=n.locateBreaks(E,O);if(L.length){var D,P,N=0;for(D=0;D<L.length;D++)N+=(P=L[D]).max-P.min;var j=l._width/(O-E-N),R=[-j*E];for(D=0;D<L.length;D++)P=L[D],R.push(R[R.length-1]-j*(P.max-P.min));for(l.d2p=function(e){for(var t=R[0],n=0;n<L.length;n++){var r=L[n];if(e>=r.max)t=R[n+1];else if(e<r.min)break}return t+j*e},D=0;D<L.length;D++)(P=L[D]).pmin=l.d2p(P.min),P.pmax=l.d2p(P.max);l.p2d=function(e){for(var t=R[0],n=0;n<L.length;n++){var r=L[n];if(e>=r.pmax)t=R[n+1];else if(e<r.pmin)break}return(e-t)/j}}}if("match"!==d.rangemode){var z=u.r2l(d.range[0]),F=u.r2l(d.range[1])-z;l.d2pOppAxis=function(e){return(e-z)/F*l._height}}o.call(v,e,n,l).call(m,e,n,l).call(y,e,n,l).call(b,e,n,l,d).call(C,e,n,l).call(A,e,n,l),function(e,t,n,o){if(t._context.staticPlot)return;var s=e.select("rect."+g.slideBoxClassName).node(),l=e.select("rect."+g.grabAreaMinClassName).node(),u=e.select("rect."+g.grabAreaMaxClassName).node();function c(){var c=r.event,d=c.target,h=c.clientX||c.touches[0].clientX,g=h-e.node().getBoundingClientRect().left,v=o.d2p(n._rl[0]),m=o.d2p(n._rl[1]),y=f.coverSlip();function b(e){var c,f,b,C=+(e.clientX||e.touches[0].clientX)-h;switch(d){case s:b="ew-resize",c=v+C,f=m+C;break;case l:b="col-resize",c=v+C,f=m;break;case u:b="col-resize",c=v,f=m+C;break;default:b="ew-resize",c=g,f=g+C}if(f<c){var A=f;f=c,c=A}o._pixelMin=c,o._pixelMax=f,p(r.select(y),b),function(e,t,n,r){function o(e){return n.l2r(a.constrain(e,r._rl[0],r._rl[1]))}var s=o(r.p2d(r._pixelMin)),l=o(r.p2d(r._pixelMax));window.requestAnimationFrame((function(){i.call("_guiRelayout",t,n._name+".range",[s,l])}))}(0,t,n,o)}function C(){y.removeEventListener("mousemove",b),y.removeEventListener("mouseup",C),this.removeEventListener("touchmove",b),this.removeEventListener("touchend",C),a.removeElement(y)}this.addEventListener("touchmove",b),this.addEventListener("touchend",C),y.addEventListener("mousemove",b),y.addEventListener("mouseup",C)}e.on("mousedown",c),e.on("touchstart",c)}(o,e,n,l),function(e,t,n,r,i,o){var l=g.handleWidth/2;function u(e){return a.constrain(e,0,r._width)}function c(e){return a.constrain(e,0,r._height)}function d(e){return a.constrain(e,-l,r._width+l)}var h=u(r.d2p(n._rl[0])),f=u(r.d2p(n._rl[1]));if(e.select("rect."+g.slideBoxClassName).attr("x",h).attr("width",f-h),e.select("rect."+g.maskMinClassName).attr("width",h),e.select("rect."+g.maskMaxClassName).attr("x",f).attr("width",r._width-f),"match"!==o.rangemode){var p=r._height-c(r.d2pOppAxis(i._rl[1])),v=r._height-c(r.d2pOppAxis(i._rl[0]));e.select("rect."+g.maskMinOppAxisClassName).attr("x",h).attr("height",p).attr("width",f-h),e.select("rect."+g.maskMaxOppAxisClassName).attr("x",h).attr("y",v).attr("height",r._height-v).attr("width",f-h),e.select("rect."+g.slideBoxClassName).attr("y",p).attr("height",v-p)}var m=Math.round(d(h-l))-.5,y=Math.round(d(f-l))+.5;e.select("g."+g.grabberMinClassName).attr("transform",s(m,.5)),e.select("g."+g.grabberMaxClassName).attr("transform",s(y,.5))}(o,0,n,l,u,d),"bottom"===n.side&&c.draw(e,n._id+"title",{propContainer:n,propName:n._name+".title",placeholder:t._dfltTitle.x,attributes:{x:n._offset+n._length/2,y:T+l._height+l._offsetShift+10+1.5*n.title.font.size,"text-anchor":"middle"}})})))}},function(e,t,n){"use strict";e.exports={moduleType:"component",name:"rangeselector",schema:{subplots:{xaxis:{rangeselector:n(567)}}},layoutAttributes:n(567),handleDefaults:n(1769),draw:n(1770)}},function(e,t,n){"use strict";var r=n(10),i=n(41),o=n(64),a=n(178),s=n(567),l=n(1029);function u(e,t,n,i){var o=i.calendar;function a(n,i){return r.coerce(e,t,s.buttons,n,i)}if(a("visible")){var l=a("step");"all"!==l&&(!o||"gregorian"===o||"month"!==l&&"year"!==l?a("stepmode"):t.stepmode="backward",a("count")),a("label")}}e.exports=function(e,t,n,c,d){var h=e.rangeselector||{},f=o.newContainer(t,"rangeselector");function p(e,t){return r.coerce(h,f,s,e,t)}if(p("visible",a(h,f,{name:"buttons",handleItemDefaults:u,calendar:d}).length>0)){var g=function(e,t,n){for(var r=n.filter((function(n){return t[n].anchor===e._id})),i=0,o=0;o<r.length;o++){var a=t[r[o]].domain;a&&(i=Math.max(a[1],i))}return[e.domain[0],i+l.yPad]}(t,n,c);p("x",g[0]),p("y",g[1]),r.noneOrAll(e,t,["x","y"]),p("xanchor"),p("yanchor"),r.coerceFont(p,"font",n.font);var v=p("bgcolor");p("activecolor",i.contrast(v,l.lightAmount,l.darkAmount)),p("bordercolor"),p("borderwidth")}}},function(e,t,n){"use strict";var r=n(39),i=n(27),o=n(82),a=n(41),s=n(56),l=n(10),u=l.strTranslate,c=n(96),d=n(83),h=n(97),f=h.LINE_SPACING,p=h.FROM_TL,g=h.FROM_BR,v=n(1029),m=n(1771);function y(e){return e._id}function b(e,t,n){var r=l.ensureSingle(e,"rect","selector-rect",(function(e){e.attr("shape-rendering","crispEdges")}));r.attr({rx:v.rx,ry:v.ry}),r.call(a.stroke,t.bordercolor).call(a.fill,function(e,t){return t._isActive||t._isHovered?e.activecolor:e.bgcolor}(t,n)).style("stroke-width",t.borderwidth+"px")}function C(e,t,n,r){l.ensureSingle(e,"text","selector-text",(function(e){e.attr("text-anchor","middle")})).call(s.font,t.font).text(function(e,t){if(e.label)return t?l.templateString(e.label,t):e.label;return"all"===e.step?"all":e.count+e.step.charAt(0)}(n,r._fullLayout._meta)).call((function(e){c.convertToTspans(e,r)}))}e.exports=function(e){var t=e._fullLayout._infolayer.selectAll(".rangeselector").data(function(e){for(var t=d.list(e,"x",!0),n=[],r=0;r<t.length;r++){var i=t[r];i.rangeselector&&i.rangeselector.visible&&n.push(i)}return n}(e),y);t.enter().append("g").classed("rangeselector",!0),t.exit().remove(),t.style({cursor:"pointer","pointer-events":"all"}),t.each((function(t){var n=r.select(this),a=t,d=a.rangeselector,h=n.selectAll("g.button").data(l.filterVisible(d.buttons));h.enter().append("g").classed("button",!0),h.exit().remove(),h.each((function(t){var n=r.select(this),o=m(a,t);t._isActive=function(e,t,n){if("all"===t.step)return!0===e.autorange;var r=Object.keys(n);return e.range[0]===n[r[0]]&&e.range[1]===n[r[1]]}(a,t,o),n.call(b,d,t),n.call(C,d,t,e),n.on("click",(function(){e._dragged||i.call("_guiRelayout",e,o)})),n.on("mouseover",(function(){t._isHovered=!0,n.call(b,d,t)})),n.on("mouseout",(function(){t._isHovered=!1,n.call(b,d,t)}))})),function(e,t,n,i,a){var d=0,h=0,m=n.borderwidth;t.each((function(){var e=r.select(this).select(".selector-text"),t=n.font.size*f,i=Math.max(t*c.lineCount(e),16)+3;h=Math.max(h,i)})),t.each((function(){var e=r.select(this),t=e.select(".selector-rect"),i=e.select(".selector-text"),o=i.node()&&s.bBox(i.node()).width,a=n.font.size*f,l=c.lineCount(i),p=Math.max(o+10,v.minButtonWidth);e.attr("transform",u(m+d,m)),t.attr({x:0,y:0,width:p,height:h}),c.positionText(i,p/2,h/2-(l-1)*a/2+3),d+=p+5}));var y=e._fullLayout._size,b=y.l+y.w*n.x,C=y.t+y.h*(1-n.y),A="left";l.isRightAnchor(n)&&(b-=d,A="right");l.isCenterAnchor(n)&&(b-=d/2,A="center");var _="top";l.isBottomAnchor(n)&&(C-=h,_="bottom");l.isMiddleAnchor(n)&&(C-=h/2,_="middle");d=Math.ceil(d),h=Math.ceil(h),b=Math.round(b),C=Math.round(C),o.autoMargin(e,i+"-range-selector",{x:n.x,y:n.y,l:d*p[A],r:d*g[A],b:h*g[_],t:h*p[_]}),a.attr("transform",u(b,C))}(e,h,d,a._name,n)}))}},function(e,t,n){"use strict";var r=n(39);e.exports=function(e,t){var n=e._name,i={};if("all"===t.step)i[n+".autorange"]=!0;else{var o=function(e,t){var n,i=e.range,o=new Date(e.r2l(i[1])),a=t.step,s=t.count;switch(t.stepmode){case"backward":n=e.l2r(+r.time[a].utc.offset(o,-s));break;case"todate":var l=r.time[a].utc.offset(o,-s);n=e.l2r(+r.time[a].utc.ceil(l))}var u=i[1];return[n,u]}(e,t);i[n+".range[0]"]=o[0],i[n+".range[1]"]=o[1]}return i}},function(e,t,n){"use strict";var r=n(10),i=n(294).counter,o=n(568).attributes,a=n(95).idRegex,s=n(64),l={rows:{valType:"integer",min:1,editType:"plot"},roworder:{valType:"enumerated",values:["top to bottom","bottom to top"],dflt:"top to bottom",editType:"plot"},columns:{valType:"integer",min:1,editType:"plot"},subplots:{valType:"info_array",freeLength:!0,dimensions:2,items:{valType:"enumerated",values:[i("xy").toString(),""],editType:"plot"},editType:"plot"},xaxes:{valType:"info_array",freeLength:!0,items:{valType:"enumerated",values:[a.x.toString(),""],editType:"plot"},editType:"plot"},yaxes:{valType:"info_array",freeLength:!0,items:{valType:"enumerated",values:[a.y.toString(),""],editType:"plot"},editType:"plot"},pattern:{valType:"enumerated",values:["independent","coupled"],dflt:"coupled",editType:"plot"},xgap:{valType:"number",min:0,max:1,editType:"plot"},ygap:{valType:"number",min:0,max:1,editType:"plot"},domain:o({name:"grid",editType:"plot",noGridCell:!0},{}),xside:{valType:"enumerated",values:["bottom","bottom plot","top plot","top"],dflt:"bottom plot",editType:"plot"},yside:{valType:"enumerated",values:["left","left plot","right plot","right"],dflt:"left plot",editType:"plot"},editType:"plot"};function u(e,t,n){var r=t[n+"axes"],i=Object.keys((e._splomAxes||{})[n]||{});return Array.isArray(r)?r:i.length?i:void 0}function c(e,t,n,r,i,o){var a=t(e+"gap",n),s=t("domain."+e);t(e+"side",r);for(var l=new Array(i),u=s[0],c=(s[1]-u)/(i-a),d=c*(1-a),h=0;h<i;h++){var f=u+c*h;l[o?i-1-h:h]=[f,f+d]}return l}function d(e,t,n,r,i){var o,a=new Array(n);function s(e,n){-1!==t.indexOf(n)&&void 0===r[n]?(a[e]=n,r[n]=e):a[e]=""}if(Array.isArray(e))for(o=0;o<n;o++)s(o,e[o]);else for(s(0,i),o=1;o<n;o++)s(o,i+(o+1));return a}e.exports={moduleType:"component",name:"grid",schema:{layout:{grid:l}},layoutAttributes:l,sizeDefaults:function(e,t){var n=e.grid||{},i=u(t,n,"x"),o=u(t,n,"y");if(e.grid||i||o){var a,d,h=Array.isArray(n.subplots)&&Array.isArray(n.subplots[0]),f=Array.isArray(i),p=Array.isArray(o),g=f&&i!==n.xaxes&&p&&o!==n.yaxes;h?(a=n.subplots.length,d=n.subplots[0].length):(p&&(a=o.length),f&&(d=i.length));var v=s.newContainer(t,"grid"),m=x("rows",a),y=x("columns",d);if(m*y>1){if(!h&&!f&&!p)"independent"===x("pattern")&&(h=!0);v._hasSubplotGrid=h;var b,C,A="top to bottom"===x("roworder"),_=h?.2:.1,I=h?.3:.1;g&&t._splomGridDflt&&(b=t._splomGridDflt.xside,C=t._splomGridDflt.yside),v._domains={x:c("x",x,_,b,y),y:c("y",x,I,C,m,A)}}else delete t.grid}function x(e,t){return r.coerce(n,v,l,e,t)}},contentDefaults:function(e,t){var n=t.grid;if(n&&n._domains){var r,i,o,a,s,l,c,h=e.grid||{},f=t._subplots,p=n._hasSubplotGrid,g=n.rows,v=n.columns,m="independent"===n.pattern,y=n._axisMap={};if(p){var b=h.subplots||[];l=n.subplots=new Array(g);var C=1;for(r=0;r<g;r++){var A=l[r]=new Array(v),_=b[r]||[];for(i=0;i<v;i++)if(m?(s=1===C?"xy":"x"+C+"y"+C,C++):s=_[i],A[i]="",-1!==f.cartesian.indexOf(s)){if(c=s.indexOf("y"),o=s.slice(0,c),a=s.slice(c),void 0!==y[o]&&y[o]!==i||void 0!==y[a]&&y[a]!==r)continue;A[i]=s,y[o]=i,y[a]=r}}}else{var I=u(t,h,"x"),x=u(t,h,"y");n.xaxes=d(I,f.xaxis,v,y,"x"),n.yaxes=d(x,f.yaxis,g,y,"y")}var w=n._anchors={},M="top to bottom"===n.roworder;for(var S in y){var T,E,O,k=S.charAt(0),L=n[k+"side"];if(L.length<8)w[S]="free";else if("x"===k){if("t"===L.charAt(0)===M?(T=0,E=1,O=g):(T=g-1,E=-1,O=-1),p){var D=y[S];for(r=T;r!==O;r+=E)if((s=l[r][D])&&(c=s.indexOf("y"),s.slice(0,c)===S)){w[S]=s.slice(c);break}}else for(r=T;r!==O;r+=E)if(a=n.yaxes[r],-1!==f.cartesian.indexOf(S+a)){w[S]=a;break}}else if("l"===L.charAt(0)?(T=0,E=1,O=v):(T=v-1,E=-1,O=-1),p){var P=y[S];for(r=T;r!==O;r+=E)if((s=l[P][r])&&(c=s.indexOf("y"),s.slice(c)===S)){w[S]=s.slice(0,c);break}}else for(r=T;r!==O;r+=E)if(o=n.xaxes[r],-1!==f.cartesian.indexOf(o+S)){w[S]=o;break}}}}}},function(e,t,n){"use strict";var r=n(10),i=n(124).overrideAll,o=n(1030),a={error_x:r.extendFlat({},o),error_y:r.extendFlat({},o)};delete a.error_x.copy_zstyle,delete a.error_y.copy_zstyle,delete a.error_y.copy_ystyle;var s={error_x:r.extendFlat({},o),error_y:r.extendFlat({},o),error_z:r.extendFlat({},o)};delete s.error_x.copy_ystyle,delete s.error_y.copy_ystyle,delete s.error_z.copy_ystyle,delete s.error_z.copy_zstyle,e.exports={moduleType:"component",name:"errorbars",schema:{traces:{scatter:a,bar:a,histogram:a,scatter3d:i(s,"calc","nested"),scattergl:i(a,"calc","nested")}},supplyDefaults:n(1774),calc:n(1775),makeComputeError:n(1031),plot:n(1776),style:n(1777),hoverInfo:function(e,t,n){(t.error_y||{}).visible&&(n.yerr=e.yh-e.y,t.error_y.symmetric||(n.yerrneg=e.y-e.ys));(t.error_x||{}).visible&&(n.xerr=e.xh-e.x,t.error_x.symmetric||(n.xerrneg=e.x-e.xs))}}},function(e,t,n){"use strict";var r=n(42),i=n(27),o=n(10),a=n(64),s=n(1030);e.exports=function(e,t,n,l){var u="error_"+l.axis,c=a.newContainer(t,u),d=e[u]||{};function h(e,t){return o.coerce(d,c,s,e,t)}if(!1!==h("visible",void 0!==d.array||void 0!==d.value||"sqrt"===d.type)){var f=h("type","array"in d?"data":"percent"),p=!0;"sqrt"!==f&&(p=h("symmetric",!(("data"===f?"arrayminus":"valueminus")in d))),"data"===f?(h("array"),h("traceref"),p||(h("arrayminus"),h("tracerefminus"))):"percent"!==f&&"constant"!==f||(h("value"),p||h("valueminus"));var g="copy_"+l.inherit+"style";if(l.inherit)(t["error_"+l.inherit]||{}).visible&&h(g,!(d.color||r(d.thickness)||r(d.width)));l.inherit&&c[g]||(h("color",n),h("thickness"),h("width",i.traceIs(t,"gl3d")?0:4))}}},function(e,t,n){"use strict";var r=n(42),i=n(27),o=n(55),a=n(10),s=n(1031);function l(e,t,n,i){var l=t["error_"+i]||{},u=[];if(l.visible&&-1!==["linear","log"].indexOf(n.type)){for(var c=s(l),d=0;d<e.length;d++){var h=e[d],f=h.i;if(void 0===f)f=d;else if(null===f)continue;var p=h[i];if(r(n.c2l(p))){var g=c(p,f);if(r(g[0])&&r(g[1])){var v=h[i+"s"]=p-g[0],m=h[i+"h"]=p+g[1];u.push(v,m)}}}var y=n._id,b=t._extremes[y],C=o.findExtremes(n,u,a.extendFlat({tozero:b.opts.tozero},{padded:!0}));b.min=b.min.concat(C.min),b.max=b.max.concat(C.max)}}e.exports=function(e){for(var t=e.calcdata,n=0;n<t.length;n++){var r=t[n],a=r[0].trace;if(!0===a.visible&&i.traceIs(a,"errorBarsOK")){var s=o.getFromId(e,a.xaxis),u=o.getFromId(e,a.yaxis);l(r,a,s,"x"),l(r,a,u,"y")}}}},function(e,t,n){"use strict";var r=n(39),i=n(42),o=n(56),a=n(125);e.exports=function(e,t,n,s){var l=n.xaxis,u=n.yaxis,c=s&&s.duration>0;t.each((function(t){var d,h=t[0].trace,f=h.error_x||{},p=h.error_y||{};h.ids&&(d=function(e){return e.id});var g=a.hasMarkers(h)&&h.marker.maxdisplayed>0;p.visible||f.visible||(t=[]);var v=r.select(this).selectAll("g.errorbar").data(t,d);if(v.exit().remove(),t.length){f.visible||v.selectAll("path.xerror").remove(),p.visible||v.selectAll("path.yerror").remove(),v.style("opacity",1);var m=v.enter().append("g").classed("errorbar",!0);c&&m.style("opacity",0).transition().duration(s.duration).style("opacity",1),o.setClipUrl(v,n.layerClipId,e),v.each((function(e){var t=r.select(this),n=function(e,t,n){var r={x:t.c2p(e.x),y:n.c2p(e.y)};void 0!==e.yh&&(r.yh=n.c2p(e.yh),r.ys=n.c2p(e.ys),i(r.ys)||(r.noYS=!0,r.ys=n.c2p(e.ys,!0)));void 0!==e.xh&&(r.xh=t.c2p(e.xh),r.xs=t.c2p(e.xs),i(r.xs)||(r.noXS=!0,r.xs=t.c2p(e.xs,!0)));return r}(e,l,u);if(!g||e.vis){var o,a=t.select("path.yerror");if(p.visible&&i(n.x)&&i(n.yh)&&i(n.ys)){var d=p.width;o="M"+(n.x-d)+","+n.yh+"h"+2*d+"m-"+d+",0V"+n.ys,n.noYS||(o+="m-"+d+",0h"+2*d),!a.size()?a=t.append("path").style("vector-effect","non-scaling-stroke").classed("yerror",!0):c&&(a=a.transition().duration(s.duration).ease(s.easing)),a.attr("d",o)}else a.remove();var h=t.select("path.xerror");if(f.visible&&i(n.y)&&i(n.xh)&&i(n.xs)){var v=(f.copy_ystyle?p:f).width;o="M"+n.xh+","+(n.y-v)+"v"+2*v+"m0,-"+v+"H"+n.xs,n.noXS||(o+="m0,-"+v+"v"+2*v),!h.size()?h=t.append("path").style("vector-effect","non-scaling-stroke").classed("xerror",!0):c&&(h=h.transition().duration(s.duration).ease(s.easing)),h.attr("d",o)}else h.remove()}}))}}))}},function(e,t,n){"use strict";var r=n(39),i=n(41);e.exports=function(e){e.each((function(e){var t=e[0].trace,n=t.error_y||{},o=t.error_x||{},a=r.select(this);a.selectAll("path.yerror").style("stroke-width",n.thickness+"px").call(i.stroke,n.color),o.copy_ystyle&&(o=n),a.selectAll("path.xerror").style("stroke-width",o.thickness+"px").call(i.stroke,o.color)}))}},function(e,t,n){"use strict";e.exports={moduleType:"component",name:"colorbar",attributes:n(541),supplyDefaults:n(986),draw:n(1779).draw,hasColorbar:n(985)}},function(e,t,n){"use strict";var r=n(39),i=n(123),o=n(82),a=n(27),s=n(55),l=n(163),u=n(10),c=u.strTranslate,d=n(63).extendFlat,h=n(214),f=n(56),p=n(41),g=n(385),v=n(96),m=n(149).flipScale,y=n(559),b=n(1020),C=n(134),A=n(97),_=A.LINE_SPACING,I=A.FROM_TL,x=A.FROM_BR,w=n(1780).cn;e.exports={draw:function(e){var t=e._fullLayout._infolayer.selectAll("g."+w.colorbar).data(function(e){var t,n,r,i,o=e._fullLayout,a=e.calcdata,s=[];function l(e){return d(e,{_fillcolor:null,_line:{color:null,width:null,dash:null},_levels:{start:null,end:null,size:null},_filllevels:null,_fillgradient:null,_zrange:null})}function u(){"function"==typeof i.calc?i.calc(e,r,t):(t._fillgradient=n.reversescale?m(n.colorscale):n.colorscale,t._zrange=[n[i.min],n[i.max]])}for(var c=0;c<a.length;c++){var h=a[c],f=(r=h[0].trace)._module.colorbar;if(!0===r.visible&&f)for(var p=Array.isArray(f),g=p?f:[f],v=0;v<g.length;v++){var y=(i=g[v]).container;(n=y?r[y]:r)&&n.showscale&&((t=l(n.colorbar))._id="cb"+r.uid+(p&&y?"-"+y:""),t._traceIndex=r.index,t._propPrefix=(y?y+".":"")+"colorbar.",t._meta=r._meta,u(),s.push(t))}}for(var b in o._colorAxes)if((n=o[b]).showscale){var C=o._colorAxes[b];(t=l(n.colorbar))._id="cb"+b,t._propPrefix=b+".colorbar.",t._meta=o._meta,i={min:"cmin",max:"cmax"},"heatmap"!==C[0]&&(r=C[1],i.calc=r._module.colorbar.calc),u(),s.push(t)}return s}(e),(function(e){return e._id}));t.enter().append("g").attr("class",(function(e){return e._id})).classed(w.colorbar,!0),t.each((function(t){var n=r.select(this);u.ensureSingle(n,"rect",w.cbbg),u.ensureSingle(n,"g",w.cbfills),u.ensureSingle(n,"g",w.cblines),u.ensureSingle(n,"g",w.cbaxis,(function(e){e.classed(w.crisp,!0)})),u.ensureSingle(n,"g",w.cbtitleunshift,(function(e){e.append("g").classed(w.cbtitle,!0)})),u.ensureSingle(n,"rect",w.cboutline);var m=function(e,t,n){var a=n._fullLayout,l=a._size,h=t._fillcolor,m=t._line,A=t.title,M=A.side,S=t._zrange||r.extent(("function"==typeof h?h:m.color).domain()),T="function"==typeof m.color?m.color:function(){return m.color},E="function"==typeof h?h:function(){return h},O=t._levels,k=function(e,t,n){var r,i,o=t._levels,a=[],s=[],l=o.end+o.size/100,u=o.size,c=1.001*n[0]-.001*n[1],d=1.001*n[1]-.001*n[0];for(i=0;i<1e5&&(r=o.start+i*u,!(u>0?r>=l:r<=l));i++)r>c&&r<d&&a.push(r);if(t._fillgradient)s=[0];else if("function"==typeof t._fillcolor){var h=t._filllevels;if(h)for(l=h.end+h.size/100,u=h.size,i=0;i<1e5&&(r=h.start+i*u,!(u>0?r>=l:r<=l));i++)r>n[0]&&r<n[1]&&s.push(r);else(s=a.map((function(e){return e-o.size/2}))).push(s[s.length-1]+o.size)}else t._fillcolor&&"string"==typeof t._fillcolor&&(s=[0]);o.size<0&&(a.reverse(),s.reverse());return{line:a,fill:s}}(0,t,S),L=k.fill,D=k.line,P=Math.round(t.thickness*("fraction"===t.thicknessmode?l.w:1)),N=P/l.w,j=Math.round(t.len*("fraction"===t.lenmode?l.h:1)),R=j/l.h,z=t.xpad/l.w,F=(t.borderwidth+t.outlinewidth)/2,B=t.ypad/l.h,H=Math.round(t.x*l.w+t.xpad),U=t.x-N*({middle:.5,right:1}[t.xanchor]||0),V=t.y+R*(({top:-.5,bottom:.5}[t.yanchor]||0)-.5),Y=Math.round(l.h*(1-V)),W=Y-j;t._lenFrac=R,t._thickFrac=N,t._xLeftFrac=U,t._yBottomFrac=V;var G=t._axis=function(e,t,n){var r=e._fullLayout,i={type:"linear",range:n,tickmode:t.tickmode,nticks:t.nticks,tick0:t.tick0,dtick:t.dtick,tickvals:t.tickvals,ticktext:t.ticktext,ticks:t.ticks,ticklen:t.ticklen,tickwidth:t.tickwidth,tickcolor:t.tickcolor,showticklabels:t.showticklabels,ticklabelposition:t.ticklabelposition,tickfont:t.tickfont,tickangle:t.tickangle,tickformat:t.tickformat,exponentformat:t.exponentformat,minexponent:t.minexponent,separatethousands:t.separatethousands,showexponent:t.showexponent,showtickprefix:t.showtickprefix,tickprefix:t.tickprefix,showticksuffix:t.showticksuffix,ticksuffix:t.ticksuffix,title:t.title,showline:!0,anchor:"free",side:"right",position:1},o={type:"linear",_id:"y"+t._id},a={letter:"y",font:r.font,noHover:!0,noTickson:!0,noTicklabelmode:!0,calendar:r.calendar};function s(e,t){return u.coerce(i,o,C,e,t)}return y(i,o,s,a,r),b(i,o,s,a),o}(n,t,S);G.position=t.x+z+N,-1!==["top","bottom"].indexOf(M)&&(G.title.side=M,G.titlex=t.x+z,G.titley=V+("top"===A.side?R-B:B));if(m.color&&"auto"===t.tickmode){G.tickmode="linear",G.tick0=O.start;var X=O.size,q=u.constrain((Y-W)/50,4,15)+1,Z=(S[1]-S[0])/((t.nticks||q)*X);if(Z>1){var Q=Math.pow(10,Math.floor(Math.log(Z)/Math.LN10));X*=Q*u.roundUp(Z/Q,[2,5,10]),(Math.abs(O.start)/O.size+1e-6)%1<2e-6&&(G.tick0=0)}G.dtick=X}G.domain=[V+B,V+R-B],G.setScale(),e.attr("transform",c(Math.round(l.l),Math.round(l.t)));var K,J=e.select("."+w.cbtitleunshift).attr("transform",c(-Math.round(l.l),-Math.round(l.t))),$=e.select("."+w.cbaxis),ee=0;function te(r,i){var o={propContainer:G,propName:t._propPrefix+"title",traceIndex:t._traceIndex,_meta:t._meta,placeholder:a._dfltTitle.colorbar,containerGroup:e.select("."+w.cbtitle)},s="h"===r.charAt(0)?r.substr(1):"h"+r;e.selectAll("."+s+",."+s+"-math-group").remove(),g.draw(n,r,d(o,i||{}))}return u.syncOrAsync([o.previousPromises,function(){if(-1!==["top","bottom"].indexOf(M)){var e,n=l.l+(t.x+z)*l.w,r=G.title.font.size;e="top"===M?(1-(V+R-B))*l.h+l.t+3+.75*r:(1-(V+B))*l.h+l.t-3-.25*r,te(G._id+"title",{attributes:{x:n,y:e,"text-anchor":"start"}})}},function(){if(-1!==["top","bottom"].indexOf(M)){var o=e.select("."+w.cbtitle),a=o.select("text"),d=[-t.outlinewidth/2,t.outlinewidth/2],h=o.select(".h"+G._id+"title-math-group").node(),p=15.6;if(a.node()&&(p=parseInt(a.node().style.fontSize,10)*_),h?(ee=f.bBox(h).height)>p&&(d[1]-=(ee-p)/2):a.node()&&!a.classed(w.jsPlaceholder)&&(ee=f.bBox(a.node()).height),ee){if(ee+=5,"top"===M)G.domain[1]-=ee/l.h,d[1]*=-1;else{G.domain[0]+=ee/l.h;var g=v.lineCount(a);d[1]+=(1-g)*p}o.attr("transform",c(d[0],d[1])),G.setScale()}}e.selectAll("."+w.cbfills+",."+w.cblines).attr("transform",c(0,Math.round(l.h*(1-G.domain[1])))),$.attr("transform",c(0,Math.round(-l.t)));var y=e.select("."+w.cbfills).selectAll("rect."+w.cbfill).attr("style","").data(L);y.enter().append("rect").classed(w.cbfill,!0).style("stroke","none"),y.exit().remove();var b=S.map(G.c2p).map(Math.round).sort((function(e,t){return e-t}));y.each((function(e,o){var a=[0===o?S[0]:(L[o]+L[o-1])/2,o===L.length-1?S[1]:(L[o]+L[o+1])/2].map(G.c2p).map(Math.round);a[1]=u.constrain(a[1]+(a[1]>a[0])?1:-1,b[0],b[1]);var s=r.select(this).attr({x:H,width:Math.max(P,2),y:r.min(a),height:Math.max(r.max(a)-r.min(a),2)});if(t._fillgradient)f.gradient(s,n,t._id,"vertical",t._fillgradient,"fill");else{var l=E(e).replace("e-","");s.attr("fill",i(l).toHexString())}}));var C=e.select("."+w.cblines).selectAll("path."+w.cbline).data(m.color&&m.width?D:[]);C.enter().append("path").classed(w.cbline,!0),C.exit().remove(),C.each((function(e){r.select(this).attr("d","M"+H+","+(Math.round(G.c2p(e))+m.width/2%1)+"h"+P).call(f.lineGroupStyle,m.width,T(e),m.dash)})),$.selectAll("g."+G._id+"tick,path").remove();var A=H+P+(t.outlinewidth||0)/2-("outside"===t.ticks?1:0),I=s.calcTicks(G),x=s.getTickSigns(G)[2];return s.drawTicks(n,G,{vals:"inside"===G.ticks?s.clipEnds(G,I):I,layer:$,path:s.makeTickPath(G,A,x),transFn:s.makeTransTickFn(G)}),s.drawLabels(n,G,{vals:I,layer:$,transFn:s.makeTransTickLabelFn(G),labelFns:s.makeLabelFns(G,A)})},function(){if(-1===["top","bottom"].indexOf(M)){var e=G.title.font.size,t=G._offset+G._length/2,i=l.l+(G.position||0)*l.w+("right"===G.side?10+e*(G.showticklabels?1:.5):-10-e*(G.showticklabels?.5:0));te("h"+G._id+"title",{avoid:{selection:r.select(n).selectAll("g."+G._id+"tick"),side:M,offsetLeft:l.l,offsetTop:0,maxShift:a.width},attributes:{x:i,y:t,"text-anchor":"middle"},transform:{rotate:"-90",offset:0}})}},o.previousPromises,function(){var r=P+t.outlinewidth/2;if(-1===G.ticklabelposition.indexOf("inside")&&(r+=f.bBox($.node()).width),(K=J.select("text")).node()&&!K.classed(w.jsPlaceholder)){var i,a=J.select(".h"+G._id+"title-math-group").node();i=a&&-1!==["top","bottom"].indexOf(M)?f.bBox(a).width:f.bBox(J.node()).right-H-l.l,r=Math.max(r,i)}var s=2*t.xpad+r+t.borderwidth+t.outlinewidth/2,u=Y-W;e.select("."+w.cbbg).attr({x:H-t.xpad-(t.borderwidth+t.outlinewidth)/2,y:W-F,width:Math.max(s,2),height:Math.max(u+2*F,2)}).call(p.fill,t.bgcolor).call(p.stroke,t.bordercolor).style("stroke-width",t.borderwidth),e.selectAll("."+w.cboutline).attr({x:H,y:W+t.ypad+("top"===M?ee:0),width:Math.max(P,2),height:Math.max(u-2*t.ypad-ee,2)}).call(p.stroke,t.outlinecolor).style({fill:"none","stroke-width":t.outlinewidth});var d=({center:.5,right:1}[t.xanchor]||0)*s;e.attr("transform",c(l.l-d,l.t));var h={},g=I[t.yanchor],v=x[t.yanchor];"pixels"===t.lenmode?(h.y=t.y,h.t=u*g,h.b=u*v):(h.t=h.b=0,h.yt=t.y+t.len*g,h.yb=t.y-t.len*v);var m=I[t.xanchor],y=x[t.xanchor];if("pixels"===t.thicknessmode)h.x=t.x,h.l=s*m,h.r=s*y;else{var b=s-P;h.l=b*m,h.r=b*y,h.xl=t.x-t.thickness*m,h.xr=t.x+t.thickness*y}o.autoMargin(n,t._id,h)}],n)}(n,t,e);m&&m.then&&(e._promises||[]).push(m),e._context.edits.colorbarPosition&&function(e,t,n){var r,i,o,s=n._fullLayout._size;l.init({element:e.node(),gd:n,prepFn:function(){r=e.attr("transform"),h(e)},moveFn:function(n,a){e.attr("transform",r+c(n,a)),i=l.align(t._xLeftFrac+n/s.w,t._thickFrac,0,1,t.xanchor),o=l.align(t._yBottomFrac-a/s.h,t._lenFrac,0,1,t.yanchor);var u=l.getCursor(i,o,t.xanchor,t.yanchor);h(e,u)},doneFn:function(){if(h(e),void 0!==i&&void 0!==o){var r={};r[t._propPrefix+"x"]=i,r[t._propPrefix+"y"]=o,void 0!==t._traceIndex?a.call("_guiRestyle",n,r,t._traceIndex):a.call("_guiRelayout",n,r)}}})}(n,t,e)})),t.exit().each((function(t){o.autoMargin(e,t._id)})).remove(),t.order()}}},function(e,t,n){"use strict";e.exports={cn:{colorbar:"colorbar",cbbg:"cbbg",cbfill:"cbfill",cbfills:"cbfills",cbline:"cbline",cblines:"cblines",cbaxis:"cbaxis",cbtitleunshift:"cbtitleunshift",cbtitle:"cbtitle",cboutline:"cboutline",crisp:"crisp",jsPlaceholder:"js-placeholder"}}},function(e,t,n){"use strict";e.exports={moduleType:"locale",name:"en",dictionary:{"Click to enter Colorscale title":"Click to enter Colourscale title"},format:{days:["Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"],shortDays:["Sun","Mon","Tue","Wed","Thu","Fri","Sat"],months:["January","February","March","April","May","June","July","August","September","October","November","December"],shortMonths:["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"],periods:["AM","PM"],dateTime:"%a %b %e %X %Y",date:"%d/%m/%Y",time:"%H:%M:%S",decimal:".",thousands:",",grouping:[3],currency:["$",""],year:"%Y",month:"%b %Y",dayMonth:"%b %-d",dayMonthYear:"%b %-d, %Y"}}},function(e,t,n){"use strict";e.exports={moduleType:"locale",name:"en-US",dictionary:{"Click to enter Colorscale title":"Click to enter Colorscale title"},format:{date:"%m/%d/%Y"}}},function(e,t,n){"use strict";var r=n(256),i={getDelay:r.getDelay,getRedrawFunc:r.getRedrawFunc,clone:n(1032),toSVG:n(555),svgToImg:n(556),toImage:n(1784),downloadImage:n(1007)};e.exports=i},function(e,t,n){"use strict";var r=n(540).EventEmitter,i=n(27),o=n(10),a=n(256),s=n(1032),l=n(555),u=n(556);e.exports=function(e,t){var n=new r,c=s(e,{format:"png"}),d=c.gd;d.style.position="absolute",d.style.left="-5000px",document.body.appendChild(d);var h=a.getRedrawFunc(d);return i.call("plot",d,c.data,c.layout,c.config).then(h).then((function(){var e=a.getDelay(d._fullLayout);setTimeout((function(){var e=l(d),r=document.createElement("canvas");r.id=o.randstr(),(n=u({format:t.format,width:d._fullLayout.width,height:d._fullLayout.height,canvas:r,emitter:n,svg:e})).clean=function(){d&&document.body.removeChild(d)}}),e)})).catch((function(e){n.emit("error",e)})),n}},function(e,t,n){"use strict";function r(e){return(r="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){var t=function(t){!function(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&u(e,t)}(a,t);var n,r,i=(n=a,r=function(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],(function(){}))),!0}catch(e){return!1}}(),function(){var e,t=h(n);if(r){var i=h(this).constructor;e=Reflect.construct(t,arguments,i)}else e=t.apply(this,arguments);return c(this,e)});function a(e){var t;return function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,a),(t=i.call(this,e)).p=Promise.resolve(),t.resizeHandler=null,t.handlers={},t.syncWindowResize=t.syncWindowResize.bind(d(t)),t.syncEventHandlers=t.syncEventHandlers.bind(d(t)),t.attachUpdateEvents=t.attachUpdateEvents.bind(d(t)),t.getRef=t.getRef.bind(d(t)),t.handleUpdate=t.handleUpdate.bind(d(t)),t.figureCallback=t.figureCallback.bind(d(t)),t.updatePlotly=t.updatePlotly.bind(d(t)),t}return function(e,t,n){t&&l(e.prototype,t);n&&l(e,n)}(a,[{key:"updatePlotly",value:function(t,n,r){var i=this;this.p=this.p.then((function(){if(!i.unmounting){if(!i.el)throw new Error("Missing element reference");return e.react(i.el,{data:i.props.data,layout:i.props.layout,config:i.props.config,frames:i.props.frames})}})).then((function(){i.unmounting||(i.syncWindowResize(t),i.syncEventHandlers(),i.figureCallback(n),r&&i.attachUpdateEvents())})).catch((function(e){i.props.onError&&i.props.onError(e)}))}},{key:"componentDidMount",value:function(){this.unmounting=!1,this.updatePlotly(!0,this.props.onInitialized,!0)}},{key:"componentDidUpdate",value:function(e){this.unmounting=!1;var t=e.frames&&e.frames.length?e.frames.length:0,n=this.props.frames&&this.props.frames.length?this.props.frames.length:0,r=!(e.layout===this.props.layout&&e.data===this.props.data&&e.config===this.props.config&&n===t),i=void 0!==e.revision,o=e.revision!==this.props.revision;(r||i&&(!i||o))&&this.updatePlotly(!1,this.props.onUpdate,!1)}},{key:"componentWillUnmount",value:function(){this.unmounting=!0,this.figureCallback(this.props.onPurge),this.resizeHandler&&g&&(window.removeEventListener("resize",this.resizeHandler),this.resizeHandler=null),this.removeUpdateEvents(),e.purge(this.el)}},{key:"attachUpdateEvents",value:function(){var e=this;this.el&&this.el.removeListener&&p.forEach((function(t){e.el.on(t,e.handleUpdate)}))}},{key:"removeUpdateEvents",value:function(){var e=this;this.el&&this.el.removeListener&&p.forEach((function(t){e.el.removeListener(t,e.handleUpdate)}))}},{key:"handleUpdate",value:function(){this.figureCallback(this.props.onUpdate)}},{key:"figureCallback",value:function(e){if("function"==typeof e){var t=this.el;e({data:t.data,layout:t.layout,frames:this.el._transitionData?this.el._transitionData._frames:null},this.el)}}},{key:"syncWindowResize",value:function(t){var n=this;g&&(this.props.useResizeHandler&&!this.resizeHandler?(this.resizeHandler=function(){return e.Plots.resize(n.el)},window.addEventListener("resize",this.resizeHandler),t&&this.resizeHandler()):!this.props.useResizeHandler&&this.resizeHandler&&(window.removeEventListener("resize",this.resizeHandler),this.resizeHandler=null))}},{key:"getRef",value:function(e){this.el=e,this.props.debug&&g&&(window.gd=this.el)}},{key:"syncEventHandlers",value:function(){var e=this;f.forEach((function(t){var n=e.props["on"+t],r=e.handlers[t],i=Boolean(r);n&&!i?e.addEventHandler(t,n):!n&&i?e.removeEventHandler(t):n&&i&&n!==r&&(e.removeEventHandler(t),e.addEventHandler(t,n))}))}},{key:"addEventHandler",value:function(e,t){this.handlers[e]=t,this.el.on(this.getPlotlyEventName(e),this.handlers[e])}},{key:"removeEventHandler",value:function(e){this.el.removeListener(this.getPlotlyEventName(e),this.handlers[e]),delete this.handlers[e]}},{key:"getPlotlyEventName",value:function(e){return"plotly_"+e.toLowerCase()}},{key:"render",value:function(){return o.default.createElement("div",{id:this.props.divId,style:this.props.style,ref:this.getRef,className:this.props.className})}}]),a}(o.Component);return t.propTypes={data:a.default.arrayOf(a.default.object),config:a.default.object,layout:a.default.object,frames:a.default.arrayOf(a.default.object),revision:a.default.number,onInitialized:a.default.func,onPurge:a.default.func,onError:a.default.func,onUpdate:a.default.func,debug:a.default.bool,style:a.default.object,className:a.default.string,useResizeHandler:a.default.bool,divId:a.default.string},f.forEach((function(e){t.propTypes["on"+e]=a.default.func})),t.defaultProps={debug:!1,useResizeHandler:!1,data:[],style:{position:"relative",display:"inline-block"}},t};var i,o=function(e){if(e&&e.__esModule)return e;if(null===e||"object"!==r(e)&&"function"!=typeof e)return{default:e};var t=s();if(t&&t.has(e))return t.get(e);var n={},i=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var o in e)if(Object.prototype.hasOwnProperty.call(e,o)){var a=i?Object.getOwnPropertyDescriptor(e,o):null;a&&(a.get||a.set)?Object.defineProperty(n,o,a):n[o]=e[o]}n.default=e,t&&t.set(e,n);return n}(n(0)),a=(i=n(3))&&i.__esModule?i:{default:i};function s(){if("function"!=typeof WeakMap)return null;var e=new WeakMap;return s=function(){return e},e}function l(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}function u(e,t){return(u=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function c(e,t){return!t||"object"!==r(t)&&"function"!=typeof t?d(e):t}function d(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function h(e){return(h=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}var f=["AfterExport","AfterPlot","Animated","AnimatingFrame","AnimationInterrupted","AutoSize","BeforeExport","BeforeHover","ButtonClicked","Click","ClickAnnotation","Deselect","DoubleClick","Framework","Hover","LegendClick","LegendDoubleClick","Relayout","Relayouting","Restyle","Redraw","Selected","Selecting","SliderChange","SliderEnd","SliderStart","SunburstClick","Transitioning","TransitionInterrupted","Unhover"],p=["plotly_restyle","plotly_redraw","plotly_relayout","plotly_relayouting","plotly_doubleclick","plotly_animated","plotly_sunburstclick"],g="undefined"!=typeof window},function(e,t,n){"use strict";e.exports=n(1787)},function(e,t,n){"use strict";e.exports={plot:n(1788),attributes:n(1060),markerSymbols:n(574),supplyDefaults:n(1911),colorbar:[{container:"marker",min:"cmin",max:"cmax"},{container:"line",min:"cmin",max:"cmax"}],calc:n(1912),moduleType:"trace",name:"scatter3d",basePlotModule:n(1061),categories:["gl3d","symbols","showLegend","scatter-like"],meta:{}}},function(e,t,n){"use strict";var r=n(1789),i=n(1820),o=n(1880),a=n(1053),s=n(573),l=n(10),u=n(304),c=n(1058).formatColor,d=n(989),h=n(1059),f=n(574),p=n(55),g=n(162).appendArrayPointValue,v=n(1910);function m(e,t){this.scene=e,this.uid=t,this.linePlot=null,this.scatterPlot=null,this.errorBars=null,this.textMarkers=null,this.delaunayMesh=null,this.color=null,this.mode="",this.dataPoints=[],this.axesBounds=[[-1/0,-1/0,-1/0],[1/0,1/0,1/0]],this.textLabels=null,this.data=null}var y=m.prototype;function b(e){return null==e?0:e.indexOf("left")>-1?-1:e.indexOf("right")>-1?1:0}function C(e){return null==e?0:e.indexOf("top")>-1?-1:e.indexOf("bottom")>-1?1:0}function A(e,t){return t(4*e)}function _(e){return f[e]}function I(e,t,n,r,i){var o=null;if(l.isArrayOrTypedArray(e)){o=[];for(var a=0;a<t;a++)void 0===e[a]?o[a]=r:o[a]=n(e[a],i)}else o=n(e,l.identity);return o}function x(e,t){var n,r,i,o,a,s,h=[],f=e.fullSceneLayout,m=e.dataScale,y=f.xaxis,x=f.yaxis,w=f.zaxis,M=t.marker,S=t.line,T=t.x||[],E=t.y||[],O=t.z||[],k=T.length,L=t.xcalendar,D=t.ycalendar,P=t.zcalendar;for(a=0;a<k;a++)n=y.d2l(T[a],0,L)*m[0],r=x.d2l(E[a],0,D)*m[1],i=w.d2l(O[a],0,P)*m[2],h[a]=[n,r,i];if(Array.isArray(t.text))s=t.text;else if(void 0!==t.text)for(s=new Array(k),a=0;a<k;a++)s[a]=t.text;function N(e,t){var n=f[e];return p.tickText(n,n.d2l(t),!0).text}var j=t.texttemplate;if(j){var R=e.fullLayout._d3locale,z=Array.isArray(j),F=z?Math.min(j.length,k):k,B=z?function(e){return j[e]}:function(){return j};for(s=new Array(F),a=0;a<F;a++){var H={x:T[a],y:E[a],z:O[a]},U={xLabel:N("xaxis",T[a]),yLabel:N("yaxis",E[a]),zLabel:N("zaxis",O[a])},V={};g(V,t,a);var Y=t._meta||{};s[a]=l.texttemplateString(B(a),U,R,V,H,Y)}}if(o={position:h,mode:t.mode,text:s},"line"in t&&(o.lineColor=c(S,1,k),o.lineWidth=S.width,o.lineDashes=S.dash),"marker"in t){var W=d(t);o.scatterColor=c(M,1,k),o.scatterSize=I(M.size,k,A,20,W),o.scatterMarker=I(M.symbol,k,_,"●"),o.scatterLineWidth=M.line.width,o.scatterLineColor=c(M.line,1,k),o.scatterAngle=0}"textposition"in t&&(o.textOffset=function(e){var t=[0,0];if(Array.isArray(e))for(var n=0;n<e.length;n++)t[n]=[0,0],e[n]&&(t[n][0]=b(e[n]),t[n][1]=C(e[n]));else t[0]=b(e),t[1]=C(e);return t}(t.textposition),o.textColor=c(t.textfont,1,k),o.textSize=I(t.textfont.size,k,l.identity,12),o.textFont=t.textfont.family,o.textAngle=0);var G=["x","y","z"];for(o.project=[!1,!1,!1],o.projectScale=[1,1,1],o.projectOpacity=[1,1,1],a=0;a<3;++a){var X=t.projection[G[a]];(o.project[a]=X.show)&&(o.projectOpacity[a]=X.opacity,o.projectScale[a]=X.scale)}o.errorBounds=v(t,m,f);var q=function(e){for(var t=[0,0,0],n=[[0,0,0],[0,0,0],[0,0,0]],r=[1,1,1],i=0;i<3;i++){var o=e[i];o&&!1!==o.copy_zstyle&&!1!==e[2].visible&&(o=e[2]),o&&o.visible&&(t[i]=o.width/2,n[i]=u(o.color),r[i]=o.thickness)}return{capSize:t,color:n,lineWidth:r}}([t.error_x,t.error_y,t.error_z]);return o.errorColor=q.color,o.errorLineWidth=q.lineWidth,o.errorCapSize=q.capSize,o.delaunayAxis=t.surfaceaxis,o.delaunayColor=u(t.surfacecolor),o}function w(e){if(l.isArrayOrTypedArray(e)){var t=e[0];return l.isArrayOrTypedArray(t)&&(e=t),"rgb("+e.slice(0,3).map((function(e){return Math.round(255*e)}))+")"}return null}function M(e){return l.isArrayOrTypedArray(e)?4===e.length&&"number"==typeof e[0]?w(e):e.map(w):null}y.handlePick=function(e){if(e.object&&(e.object===this.linePlot||e.object===this.delaunayMesh||e.object===this.textMarkers||e.object===this.scatterPlot)){var t=e.index=e.data.index;return e.object.highlight&&e.object.highlight(null),this.scatterPlot&&(e.object=this.scatterPlot,this.scatterPlot.highlight(e.data)),e.textLabel="",this.textLabels&&(Array.isArray(this.textLabels)?(this.textLabels[t]||0===this.textLabels[t])&&(e.textLabel=this.textLabels[t]):e.textLabel=this.textLabels),e.traceCoordinate=[this.data.x[t],this.data.y[t],this.data.z[t]],!0}},y.update=function(e){var t,n,l,u,c=this.scene.glplot.gl,d=h.solid;this.data=e;var f=x(this.scene,e);"mode"in f&&(this.mode=f.mode),"lineDashes"in f&&f.lineDashes in h&&(d=h[f.lineDashes]),this.color=M(f.scatterColor)||M(f.lineColor),this.dataPoints=f.position,t={gl:this.scene.glplot.gl,position:f.position,color:f.lineColor,lineWidth:f.lineWidth||1,dashes:d[0],dashScale:d[1],opacity:e.opacity,connectGaps:e.connectgaps},-1!==this.mode.indexOf("lines")?this.linePlot?this.linePlot.update(t):(this.linePlot=r(t),this.linePlot._trace=this,this.scene.glplot.add(this.linePlot)):this.linePlot&&(this.scene.glplot.remove(this.linePlot),this.linePlot.dispose(),this.linePlot=null);var p=e.opacity;if(e.marker&&e.marker.opacity&&(p*=e.marker.opacity),n={gl:this.scene.glplot.gl,position:f.position,color:f.scatterColor,size:f.scatterSize,glyph:f.scatterMarker,opacity:p,orthographic:!0,lineWidth:f.scatterLineWidth,lineColor:f.scatterLineColor,project:f.project,projectScale:f.projectScale,projectOpacity:f.projectOpacity},-1!==this.mode.indexOf("markers")?this.scatterPlot?this.scatterPlot.update(n):(this.scatterPlot=i(n),this.scatterPlot._trace=this,this.scatterPlot.highlightScale=1,this.scene.glplot.add(this.scatterPlot)):this.scatterPlot&&(this.scene.glplot.remove(this.scatterPlot),this.scatterPlot.dispose(),this.scatterPlot=null),u={gl:this.scene.glplot.gl,position:f.position,glyph:f.text,color:f.textColor,size:f.textSize,angle:f.textAngle,alignment:f.textOffset,font:f.textFont,orthographic:!0,lineWidth:0,project:!1,opacity:e.opacity},this.textLabels=e.hovertext||e.text,-1!==this.mode.indexOf("text")?this.textMarkers?this.textMarkers.update(u):(this.textMarkers=i(u),this.textMarkers._trace=this,this.textMarkers.highlightScale=1,this.scene.glplot.add(this.textMarkers)):this.textMarkers&&(this.scene.glplot.remove(this.textMarkers),this.textMarkers.dispose(),this.textMarkers=null),l={gl:this.scene.glplot.gl,position:f.position,color:f.errorColor,error:f.errorBounds,lineWidth:f.errorLineWidth,capSize:f.errorCapSize,opacity:e.opacity},this.errorBars?f.errorBounds?this.errorBars.update(l):(this.scene.glplot.remove(this.errorBars),this.errorBars.dispose(),this.errorBars=null):f.errorBounds&&(this.errorBars=o(l),this.errorBars._trace=this,this.scene.glplot.add(this.errorBars)),f.delaunayAxis>=0){var g=function(e,t,n){var r,i=(n+1)%3,o=(n+2)%3,a=[],l=[];for(r=0;r<e.length;++r){var u=e[r];!isNaN(u[i])&&isFinite(u[i])&&!isNaN(u[o])&&isFinite(u[o])&&(a.push([u[i],u[o]]),l.push(r))}var c=s(a);for(r=0;r<c.length;++r)for(var d=c[r],h=0;h<d.length;++h)d[h]=l[d[h]];return{positions:e,cells:c,meshColor:t}}(f.position,f.delaunayColor,f.delaunayAxis);g.opacity=e.opacity,this.delaunayMesh?this.delaunayMesh.update(g):(g.gl=c,this.delaunayMesh=a(g),this.delaunayMesh._trace=this,this.scene.glplot.add(this.delaunayMesh))}else this.delaunayMesh&&(this.scene.glplot.remove(this.delaunayMesh),this.delaunayMesh.dispose(),this.delaunayMesh=null)},y.dispose=function(){this.linePlot&&(this.scene.glplot.remove(this.linePlot),this.linePlot.dispose()),this.scatterPlot&&(this.scene.glplot.remove(this.scatterPlot),this.scatterPlot.dispose()),this.errorBars&&(this.scene.glplot.remove(this.errorBars),this.errorBars.dispose()),this.textMarkers&&(this.scene.glplot.remove(this.textMarkers),this.textMarkers.dispose()),this.delaunayMesh&&(this.scene.glplot.remove(this.delaunayMesh),this.delaunayMesh.dispose())},e.exports=function(e,t){var n=new m(e,t.uid);return n.update(t),n}},function(e,t,n){"use strict";e.exports=function(e){var t=e.gl||e.scene&&e.scene.gl,n=d(t);n.attributes.position.location=0,n.attributes.nextPosition.location=1,n.attributes.arcLength.location=2,n.attributes.lineWidth.location=3,n.attributes.color.location=4;var a=h(t);a.attributes.position.location=0,a.attributes.nextPosition.location=1,a.attributes.arcLength.location=2,a.attributes.lineWidth.location=3,a.attributes.color.location=4;for(var s=r(t),l=i(t,[{buffer:s,size:3,offset:0,stride:48},{buffer:s,size:3,offset:12,stride:48},{buffer:s,size:1,offset:24,stride:48},{buffer:s,size:1,offset:28,stride:48},{buffer:s,size:4,offset:32,stride:48}]),c=u(new Array(1024),[256,1,4]),f=0;f<1024;++f)c.data[f]=255;var p=o(t,c);p.wrap=t.REPEAT;var g=new m(t,n,a,s,l,p);return g.update(e),g};var r=n(180),i=n(181),o=n(569),a=new Uint8Array(4),s=new Float32Array(a.buffer);var l=n(302),u=n(216),c=n(1798),d=c.createShader,h=c.createPickShader,f=[1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1];function p(e,t){for(var n=0,r=0;r<3;++r){var i=e[r]-t[r];n+=i*i}return Math.sqrt(n)}function g(e){for(var t=[[-1e6,-1e6,-1e6],[1e6,1e6,1e6]],n=0;n<3;++n)t[0][n]=Math.max(e[0][n],t[0][n]),t[1][n]=Math.min(e[1][n],t[1][n]);return t}function v(e,t,n,r){this.arcLength=e,this.position=t,this.index=n,this.dataCoordinate=r}function m(e,t,n,r,i,o){this.gl=e,this.shader=t,this.pickShader=n,this.buffer=r,this.vao=i,this.clipBounds=[[-1/0,-1/0,-1/0],[1/0,1/0,1/0]],this.points=[],this.arcLength=[],this.vertexCount=0,this.bounds=[[0,0,0],[0,0,0]],this.pickId=0,this.lineWidth=1,this.texture=o,this.dashScale=1,this.opacity=1,this.hasAlpha=!1,this.dirty=!0,this.pixelRatio=1}var y=m.prototype;y.isTransparent=function(){return this.hasAlpha},y.isOpaque=function(){return!this.hasAlpha},y.pickSlots=1,y.setPickBase=function(e){this.pickId=e},y.drawTransparent=y.draw=function(e){if(this.vertexCount){var t=this.gl,n=this.shader,r=this.vao;n.bind(),n.uniforms={model:e.model||f,view:e.view||f,projection:e.projection||f,clipBounds:g(this.clipBounds),dashTexture:this.texture.bind(),dashScale:this.dashScale/this.arcLength[this.arcLength.length-1],opacity:this.opacity,screenShape:[t.drawingBufferWidth,t.drawingBufferHeight],pixelRatio:this.pixelRatio},r.bind(),r.draw(t.TRIANGLE_STRIP,this.vertexCount),r.unbind()}},y.drawPick=function(e){if(this.vertexCount){var t=this.gl,n=this.pickShader,r=this.vao;n.bind(),n.uniforms={model:e.model||f,view:e.view||f,projection:e.projection||f,pickId:this.pickId,clipBounds:g(this.clipBounds),screenShape:[t.drawingBufferWidth,t.drawingBufferHeight],pixelRatio:this.pixelRatio},r.bind(),r.draw(t.TRIANGLE_STRIP,this.vertexCount),r.unbind()}},y.update=function(e){var t,n;this.dirty=!0;var r=!!e.connectGaps;"dashScale"in e&&(this.dashScale=e.dashScale),this.hasAlpha=!1,"opacity"in e&&(this.opacity=+e.opacity,this.opacity<1&&(this.hasAlpha=!0));var i=[],o=[],a=[],s=0,c=0,d=[[1/0,1/0,1/0],[-1/0,-1/0,-1/0]],h=e.position||e.positions;if(h){var f=e.color||e.colors||[0,0,0,1],g=e.lineWidth||1,v=!1;e:for(t=1;t<h.length;++t){var m,y,b,C=h[t-1],A=h[t];for(o.push(s),a.push(C.slice()),n=0;n<3;++n){if(isNaN(C[n])||isNaN(A[n])||!isFinite(C[n])||!isFinite(A[n])){if(!r&&i.length>0){for(var _=0;_<24;++_)i.push(i[i.length-12]);c+=2,v=!0}continue e}d[0][n]=Math.min(d[0][n],C[n],A[n]),d[1][n]=Math.max(d[1][n],C[n],A[n])}Array.isArray(f[0])?(m=f.length>t-1?f[t-1]:f.length>0?f[f.length-1]:[0,0,0,1],y=f.length>t?f[t]:f.length>0?f[f.length-1]:[0,0,0,1]):m=y=f,3===m.length&&(m=[m[0],m[1],m[2],1]),3===y.length&&(y=[y[0],y[1],y[2],1]),!this.hasAlpha&&m[3]<1&&(this.hasAlpha=!0),b=Array.isArray(g)?g.length>t-1?g[t-1]:g.length>0?g[g.length-1]:[0,0,0,1]:g;var I=s;if(s+=p(C,A),v){for(n=0;n<2;++n)i.push(C[0],C[1],C[2],A[0],A[1],A[2],I,b,m[0],m[1],m[2],m[3]);c+=2,v=!1}i.push(C[0],C[1],C[2],A[0],A[1],A[2],I,b,m[0],m[1],m[2],m[3],C[0],C[1],C[2],A[0],A[1],A[2],I,-b,m[0],m[1],m[2],m[3],A[0],A[1],A[2],C[0],C[1],C[2],s,-b,y[0],y[1],y[2],y[3],A[0],A[1],A[2],C[0],C[1],C[2],s,b,y[0],y[1],y[2],y[3]),c+=4}}if(this.buffer.update(i),o.push(s),a.push(h[h.length-1].slice()),this.bounds=d,this.vertexCount=c,this.points=a,this.arcLength=o,"dashes"in e){var x=e.dashes.slice();for(x.unshift(0),t=1;t<x.length;++t)x[t]=x[t-1]+x[t];var w=u(new Array(1024),[256,1,4]);for(t=0;t<256;++t){for(n=0;n<4;++n)w.set(t,0,n,0);1&l.le(x,x[x.length-1]*t/255)?w.set(t,0,0,0):w.set(t,0,0,255)}this.texture.setPixels(w)}},y.dispose=function(){this.shader.dispose(),this.vao.dispose(),this.buffer.dispose()},y.pick=function(e){if(!e)return null;if(e.id!==this.pickId)return null;var t=function(e,t,n,r){return a[0]=r,a[1]=n,a[2]=t,a[3]=e,s[0]}(e.value[0],e.value[1],e.value[2],0),n=l.le(this.arcLength,t);if(n<0)return null;if(n===this.arcLength.length-1)return new v(this.arcLength[this.arcLength.length-1],this.points[this.points.length-1].slice(),n);for(var r=this.points[n],i=this.points[Math.min(n+1,this.points.length-1)],o=(t-this.arcLength[n])/(this.arcLength[n+1]-this.arcLength[n]),u=1-o,c=[0,0,0],d=0;d<3;++d)c[d]=u*r[d]+o*i[d];var h=Math.min(o<.5?n:n+1,this.points.length-1);return new v(t,c,h,this.points[h])}},function(e,t){
/*! ieee754. BSD-3-Clause License. Feross Aboukhadijeh <https://feross.org/opensource> */
t.read=function(e,t,n,r,i){var o,a,s=8*i-r-1,l=(1<<s)-1,u=l>>1,c=-7,d=n?i-1:0,h=n?-1:1,f=e[t+d];for(d+=h,o=f&(1<<-c)-1,f>>=-c,c+=s;c>0;o=256*o+e[t+d],d+=h,c-=8);for(a=o&(1<<-c)-1,o>>=-c,c+=r;c>0;a=256*a+e[t+d],d+=h,c-=8);if(0===o)o=1-u;else{if(o===l)return a?NaN:1/0*(f?-1:1);a+=Math.pow(2,r),o-=u}return(f?-1:1)*a*Math.pow(2,o-r)},t.write=function(e,t,n,r,i,o){var a,s,l,u=8*o-i-1,c=(1<<u)-1,d=c>>1,h=23===i?Math.pow(2,-24)-Math.pow(2,-77):0,f=r?0:o-1,p=r?1:-1,g=t<0||0===t&&1/t<0?1:0;for(t=Math.abs(t),isNaN(t)||t===1/0?(s=isNaN(t)?1:0,a=c):(a=Math.floor(Math.log(t)/Math.LN2),t*(l=Math.pow(2,-a))<1&&(a--,l*=2),(t+=a+d>=1?h/l:h*Math.pow(2,1-d))*l>=2&&(a++,l/=2),a+d>=c?(s=0,a=c):a+d>=1?(s=(t*l-1)*Math.pow(2,i),a+=d):(s=t*Math.pow(2,d-1)*Math.pow(2,i),a=0));i>=8;e[n+f]=255&s,f+=p,s/=256,i-=8);for(a=a<<i|s,u+=i;u>0;e[n+f]=255&a,f+=p,a/=256,u-=8);e[n+f-p]|=128*g}},function(e,t){var n={}.toString;e.exports=Array.isArray||function(e){return"[object Array]"==n.call(e)}},function(e,t,n){"use strict";var r=n(1793);e.exports=function(e){var t=["'use strict'","var CACHED={}"],n=[],i=e.funcName+"_cwise_thunk";t.push(["return function ",i,"(",e.shimArgs.join(","),"){"].join(""));for(var o=[],a=[],s=[["array",e.arrayArgs[0],".shape.slice(",Math.max(0,e.arrayBlockIndices[0]),e.arrayBlockIndices[0]<0?","+e.arrayBlockIndices[0]+")":")"].join("")],l=[],u=[],c=0;c<e.arrayArgs.length;++c){var d=e.arrayArgs[c];n.push(["t",d,"=array",d,".dtype,","r",d,"=array",d,".order"].join("")),o.push("t"+d),o.push("r"+d),a.push("t"+d),a.push("r"+d+".join()"),s.push("array"+d+".data"),s.push("array"+d+".stride"),s.push("array"+d+".offset|0"),c>0&&(l.push("array"+e.arrayArgs[0]+".shape.length===array"+d+".shape.length+"+(Math.abs(e.arrayBlockIndices[0])-Math.abs(e.arrayBlockIndices[c]))),u.push("array"+e.arrayArgs[0]+".shape[shapeIndex+"+Math.max(0,e.arrayBlockIndices[0])+"]===array"+d+".shape[shapeIndex+"+Math.max(0,e.arrayBlockIndices[c])+"]"))}for(e.arrayArgs.length>1&&(t.push("if (!("+l.join(" && ")+")) throw new Error('cwise: Arrays do not all have the same dimensionality!')"),t.push("for(var shapeIndex=array"+e.arrayArgs[0]+".shape.length-"+Math.abs(e.arrayBlockIndices[0])+"; shapeIndex--\x3e0;) {"),t.push("if (!("+u.join(" && ")+")) throw new Error('cwise: Arrays do not all have the same shape!')"),t.push("}")),c=0;c<e.scalarArgs.length;++c)s.push("scalar"+e.scalarArgs[c]);return n.push(["type=[",a.join(","),"].join()"].join("")),n.push("proc=CACHED[type]"),t.push("var "+n.join(",")),t.push(["if(!proc){","CACHED[type]=proc=compile([",o.join(","),"])}","return proc(",s.join(","),")}"].join("")),e.debug&&console.log("-----Generated thunk:\n"+t.join("\n")+"\n----------"),new Function("compile",t.join("\n"))(r.bind(void 0,e))}},function(e,t,n){"use strict";var r=n(394);function i(e,t,n){var r,i,o=e.length,a=t.arrayArgs.length,s=t.indexArgs.length>0,l=[],u=[],c=0,d=0;for(r=0;r<o;++r)u.push(["i",r,"=0"].join(""));for(i=0;i<a;++i)for(r=0;r<o;++r)d=c,c=e[r],0===r?u.push(["d",i,"s",r,"=t",i,"p",c].join("")):u.push(["d",i,"s",r,"=(t",i,"p",c,"-s",d,"*t",i,"p",d,")"].join(""));for(u.length>0&&l.push("var "+u.join(",")),r=o-1;r>=0;--r)c=e[r],l.push(["for(i",r,"=0;i",r,"<s",c,";++i",r,"){"].join(""));for(l.push(n),r=0;r<o;++r){for(d=c,c=e[r],i=0;i<a;++i)l.push(["p",i,"+=d",i,"s",r].join(""));s&&(r>0&&l.push(["index[",d,"]-=s",d].join("")),l.push(["++index[",c,"]"].join(""))),l.push("}")}return l.join("\n")}function o(e,t,n){for(var r=e.body,i=[],o=[],a=0;a<e.args.length;++a){var s=e.args[a];if(!(s.count<=0)){var l=new RegExp(s.name,"g"),u="",c=t.arrayArgs.indexOf(a);switch(t.argTypes[a]){case"offset":var d=t.offsetArgIndex.indexOf(a);c=t.offsetArgs[d].array,u="+q"+d;case"array":u="p"+c+u;var h="l"+a,f="a"+c;if(0===t.arrayBlockIndices[c])1===s.count?"generic"===n[c]?s.lvalue?(i.push(["var ",h,"=",f,".get(",u,")"].join("")),r=r.replace(l,h),o.push([f,".set(",u,",",h,")"].join(""))):r=r.replace(l,[f,".get(",u,")"].join("")):r=r.replace(l,[f,"[",u,"]"].join("")):"generic"===n[c]?(i.push(["var ",h,"=",f,".get(",u,")"].join("")),r=r.replace(l,h),s.lvalue&&o.push([f,".set(",u,",",h,")"].join(""))):(i.push(["var ",h,"=",f,"[",u,"]"].join("")),r=r.replace(l,h),s.lvalue&&o.push([f,"[",u,"]=",h].join("")));else{for(var p=[s.name],g=[u],v=0;v<Math.abs(t.arrayBlockIndices[c]);v++)p.push("\\s*\\[([^\\]]+)\\]"),g.push("$"+(v+1)+"*t"+c+"b"+v);if(l=new RegExp(p.join(""),"g"),u=g.join("+"),"generic"===n[c])throw new Error("cwise: Generic arrays not supported in combination with blocks!");r=r.replace(l,[f,"[",u,"]"].join(""))}break;case"scalar":r=r.replace(l,"Y"+t.scalarArgs.indexOf(a));break;case"index":r=r.replace(l,"index");break;case"shape":r=r.replace(l,"shape")}}}return[i.join("\n"),r,o.join("\n")].join("\n").trim()}function a(e){for(var t=new Array(e.length),n=!0,r=0;r<e.length;++r){var i=e[r],o=i.match(/\d+/);o=o?o[0]:"",0===i.charAt(0)?t[r]="u"+i.charAt(1)+o:t[r]=i.charAt(0)+o,r>0&&(n=n&&t[r]===t[r-1])}return n?t[0]:t.join("")}e.exports=function(e,t){for(var n=t[1].length-Math.abs(e.arrayBlockIndices[0])|0,s=new Array(e.arrayArgs.length),l=new Array(e.arrayArgs.length),u=0;u<e.arrayArgs.length;++u)l[u]=t[2*u],s[u]=t[2*u+1];var c=[],d=[],h=[],f=[],p=[];for(u=0;u<e.arrayArgs.length;++u){e.arrayBlockIndices[u]<0?(h.push(0),f.push(n),c.push(n),d.push(n+e.arrayBlockIndices[u])):(h.push(e.arrayBlockIndices[u]),f.push(e.arrayBlockIndices[u]+n),c.push(0),d.push(e.arrayBlockIndices[u]));for(var g=[],v=0;v<s[u].length;v++)h[u]<=s[u][v]&&s[u][v]<f[u]&&g.push(s[u][v]-h[u]);p.push(g)}var m=["SS"],y=["'use strict'"],b=[];for(v=0;v<n;++v)b.push(["s",v,"=SS[",v,"]"].join(""));for(u=0;u<e.arrayArgs.length;++u){m.push("a"+u),m.push("t"+u),m.push("p"+u);for(v=0;v<n;++v)b.push(["t",u,"p",v,"=t",u,"[",h[u]+v,"]"].join(""));for(v=0;v<Math.abs(e.arrayBlockIndices[u]);++v)b.push(["t",u,"b",v,"=t",u,"[",c[u]+v,"]"].join(""))}for(u=0;u<e.scalarArgs.length;++u)m.push("Y"+u);if(e.shapeArgs.length>0&&b.push("shape=SS.slice(0)"),e.indexArgs.length>0){var C=new Array(n);for(u=0;u<n;++u)C[u]="0";b.push(["index=[",C.join(","),"]"].join(""))}for(u=0;u<e.offsetArgs.length;++u){var A=e.offsetArgs[u],_=[];for(v=0;v<A.offset.length;++v)0!==A.offset[v]&&(1===A.offset[v]?_.push(["t",A.array,"p",v].join("")):_.push([A.offset[v],"*t",A.array,"p",v].join("")));0===_.length?b.push("q"+u+"=0"):b.push(["q",u,"=",_.join("+")].join(""))}var I=r([].concat(e.pre.thisVars).concat(e.body.thisVars).concat(e.post.thisVars));for((b=b.concat(I)).length>0&&y.push("var "+b.join(",")),u=0;u<e.arrayArgs.length;++u)y.push("p"+u+"|=0");e.pre.body.length>3&&y.push(o(e.pre,e,l));var x=o(e.body,e,l),w=function(e){for(var t=0,n=e[0].length;t<n;){for(var r=1;r<e.length;++r)if(e[r][t]!==e[0][t])return t;++t}return t}(p);w<n?y.push(function(e,t,n,r){for(var o=t.length,a=n.arrayArgs.length,s=n.blockSize,l=n.indexArgs.length>0,u=[],c=0;c<a;++c)u.push(["var offset",c,"=p",c].join(""));for(c=e;c<o;++c)u.push(["for(var j"+c+"=SS[",t[c],"]|0;j",c,">0;){"].join("")),u.push(["if(j",c,"<",s,"){"].join("")),u.push(["s",t[c],"=j",c].join("")),u.push(["j",c,"=0"].join("")),u.push(["}else{s",t[c],"=",s].join("")),u.push(["j",c,"-=",s,"}"].join("")),l&&u.push(["index[",t[c],"]=j",c].join(""));for(c=0;c<a;++c){for(var d=["offset"+c],h=e;h<o;++h)d.push(["j",h,"*t",c,"p",t[h]].join(""));u.push(["p",c,"=(",d.join("+"),")"].join(""))}for(u.push(i(t,n,r)),c=e;c<o;++c)u.push("}");return u.join("\n")}(w,p[0],e,x)):y.push(i(p[0],e,x)),e.post.body.length>3&&y.push(o(e.post,e,l)),e.debug&&console.log("-----Generated cwise routine for ",t,":\n"+y.join("\n")+"\n----------");var M=[e.funcName||"unnamed","_cwise_loop_",s[0].join("s"),"m",w,a(l)].join("");return new Function(["function ",M,"(",m.join(","),"){",y.join("\n"),"} return ",M].join(""))()}},function(e,t,n){"use strict";e.exports=function(e){for(var t=new Array(e),n=0;n<e;++n)t[n]=n;return t}},function(e,t){function n(e){return!!e.constructor&&"function"==typeof e.constructor.isBuffer&&e.constructor.isBuffer(e)}
/*!
 * Determine if an object is a Buffer
 *
 * @author   Feross Aboukhadijeh <https://feross.org>
 * @license  MIT
 */
e.exports=function(e){return null!=e&&(n(e)||function(e){return"function"==typeof e.readFloatLE&&"function"==typeof e.slice&&n(e.slice(0,0))}(e)||!!e._isBuffer)}},function(e,t,n){"use strict";var r=n(1037);function i(e,t,n,r,i,o){this.location=e,this.dimension=t,this.a=n,this.b=r,this.c=i,this.d=o}function o(e,t,n){this.gl=e,this._ext=t,this.handle=n,this._attribs=[],this._useElements=!1,this._elementsType=e.UNSIGNED_SHORT}i.prototype.bind=function(e){switch(this.dimension){case 1:e.vertexAttrib1f(this.location,this.a);break;case 2:e.vertexAttrib2f(this.location,this.a,this.b);break;case 3:e.vertexAttrib3f(this.location,this.a,this.b,this.c);break;case 4:e.vertexAttrib4f(this.location,this.a,this.b,this.c,this.d)}},o.prototype.bind=function(){this._ext.bindVertexArrayOES(this.handle);for(var e=0;e<this._attribs.length;++e)this._attribs[e].bind(this.gl)},o.prototype.unbind=function(){this._ext.bindVertexArrayOES(null)},o.prototype.dispose=function(){this._ext.deleteVertexArrayOES(this.handle)},o.prototype.update=function(e,t,n){if(this.bind(),r(this.gl,t,e),this.unbind(),this._attribs.length=0,e)for(var o=0;o<e.length;++o){var a=e[o];"number"==typeof a?this._attribs.push(new i(o,1,a)):Array.isArray(a)&&this._attribs.push(new i(o,a.length,a[0],a[1],a[2],a[3]))}this._useElements=!!t,this._elementsType=n||this.gl.UNSIGNED_SHORT},o.prototype.draw=function(e,t,n){n=n||0;var r=this.gl;this._useElements?r.drawElements(e,t,this._elementsType,n):r.drawArrays(e,n,t)},e.exports=function(e,t){return new o(e,t,t.createVertexArrayOES())}},function(e,t,n){"use strict";var r=n(1037);function i(e){this.gl=e,this._elements=null,this._attributes=null,this._elementsType=e.UNSIGNED_SHORT}i.prototype.bind=function(){r(this.gl,this._elements,this._attributes)},i.prototype.update=function(e,t,n){this._elements=t,this._attributes=e,this._elementsType=n||this.gl.UNSIGNED_SHORT},i.prototype.dispose=function(){},i.prototype.unbind=function(){},i.prototype.draw=function(e,t,n){n=n||0;var r=this.gl;this._elements?r.drawElements(e,t,this._elementsType,n):r.drawArrays(e,n,t)},e.exports=function(e){return new i(e)}},function(e,t,n){var r=n(217),i=n(218),o=r(["precision highp float;\n#define GLSLIFY 1\n\nattribute vec3 position, nextPosition;\nattribute float arcLength, lineWidth;\nattribute vec4 color;\n\nuniform vec2 screenShape;\nuniform float pixelRatio;\nuniform mat4 model, view, projection;\n\nvarying vec4 fragColor;\nvarying vec3 worldPosition;\nvarying float pixelArcLength;\n\nvec4 project(vec3 p) {\n  return projection * view * model * vec4(p, 1.0);\n}\n\nvoid main() {\n  vec4 startPoint = project(position);\n  vec4 endPoint   = project(nextPosition);\n\n  vec2 A = startPoint.xy / startPoint.w;\n  vec2 B =   endPoint.xy /   endPoint.w;\n\n  float clipAngle = atan(\n    (B.y - A.y) * screenShape.y,\n    (B.x - A.x) * screenShape.x\n  );\n\n  vec2 offset = 0.5 * pixelRatio * lineWidth * vec2(\n    sin(clipAngle),\n    -cos(clipAngle)\n  ) / screenShape;\n\n  gl_Position = vec4(startPoint.xy + startPoint.w * offset, startPoint.zw);\n\n  worldPosition = position;\n  pixelArcLength = arcLength;\n  fragColor = color;\n}\n"]),a=r(["precision highp float;\n#define GLSLIFY 1\n\nbool outOfRange(float a, float b, float p) {\n  return ((p > max(a, b)) || \n          (p < min(a, b)));\n}\n\nbool outOfRange(vec2 a, vec2 b, vec2 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y));\n}\n\nbool outOfRange(vec3 a, vec3 b, vec3 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y) ||\n          outOfRange(a.z, b.z, p.z));\n}\n\nbool outOfRange(vec4 a, vec4 b, vec4 p) {\n  return outOfRange(a.xyz, b.xyz, p.xyz);\n}\n\nuniform vec3      clipBounds[2];\nuniform sampler2D dashTexture;\nuniform float     dashScale;\nuniform float     opacity;\n\nvarying vec3    worldPosition;\nvarying float   pixelArcLength;\nvarying vec4    fragColor;\n\nvoid main() {\n  if (\n    outOfRange(clipBounds[0], clipBounds[1], worldPosition) ||\n    fragColor.a * opacity == 0.\n  ) discard;\n\n  float dashWeight = texture2D(dashTexture, vec2(dashScale * pixelArcLength, 0)).r;\n  if(dashWeight < 0.5) {\n    discard;\n  }\n  gl_FragColor = fragColor * opacity;\n}\n"]),s=r(["precision highp float;\n#define GLSLIFY 1\n\n#define FLOAT_MAX  1.70141184e38\n#define FLOAT_MIN  1.17549435e-38\n\n// https://github.com/mikolalysenko/glsl-read-float/blob/master/index.glsl\nvec4 packFloat(float v) {\n  float av = abs(v);\n\n  //Handle special cases\n  if(av < FLOAT_MIN) {\n    return vec4(0.0, 0.0, 0.0, 0.0);\n  } else if(v > FLOAT_MAX) {\n    return vec4(127.0, 128.0, 0.0, 0.0) / 255.0;\n  } else if(v < -FLOAT_MAX) {\n    return vec4(255.0, 128.0, 0.0, 0.0) / 255.0;\n  }\n\n  vec4 c = vec4(0,0,0,0);\n\n  //Compute exponent and mantissa\n  float e = floor(log2(av));\n  float m = av * pow(2.0, -e) - 1.0;\n\n  //Unpack mantissa\n  c[1] = floor(128.0 * m);\n  m -= c[1] / 128.0;\n  c[2] = floor(32768.0 * m);\n  m -= c[2] / 32768.0;\n  c[3] = floor(8388608.0 * m);\n\n  //Unpack exponent\n  float ebias = e + 127.0;\n  c[0] = floor(ebias / 2.0);\n  ebias -= c[0] * 2.0;\n  c[1] += floor(ebias) * 128.0;\n\n  //Unpack sign bit\n  c[0] += 128.0 * step(0.0, -v);\n\n  //Scale back to range\n  return c / 255.0;\n}\n\nbool outOfRange(float a, float b, float p) {\n  return ((p > max(a, b)) || \n          (p < min(a, b)));\n}\n\nbool outOfRange(vec2 a, vec2 b, vec2 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y));\n}\n\nbool outOfRange(vec3 a, vec3 b, vec3 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y) ||\n          outOfRange(a.z, b.z, p.z));\n}\n\nbool outOfRange(vec4 a, vec4 b, vec4 p) {\n  return outOfRange(a.xyz, b.xyz, p.xyz);\n}\n\nuniform float pickId;\nuniform vec3 clipBounds[2];\n\nvarying vec3 worldPosition;\nvarying float pixelArcLength;\nvarying vec4 fragColor;\n\nvoid main() {\n  if (outOfRange(clipBounds[0], clipBounds[1], worldPosition)) discard;\n\n  gl_FragColor = vec4(pickId/255.0, packFloat(pixelArcLength).xyz);\n}"]),l=[{name:"position",type:"vec3"},{name:"nextPosition",type:"vec3"},{name:"arcLength",type:"float"},{name:"lineWidth",type:"float"},{name:"color",type:"vec4"}];t.createShader=function(e){return i(e,o,a,null,l)},t.createPickShader=function(e){return i(e,o,s,null,l)}},function(e,t,n){"use strict";var r=n(1038),i=n(395);function o(e){return new Function("y","return function(){return y}")(e)}function a(e,t){for(var n=new Array(e),r=0;r<e;++r)n[r]=t;return n}e.exports=function(e,t,n,s){function l(e,t,n){switch(n){case"bool":case"int":case"sampler2D":case"samplerCube":return"gl.uniform1i(locations["+t+"],obj"+e+")";case"float":return"gl.uniform1f(locations["+t+"],obj"+e+")";default:var r=n.indexOf("vec");if(!(0<=r&&r<=1&&n.length===4+r)){if(0===n.indexOf("mat")&&4===n.length){var o;if((o=n.charCodeAt(n.length-1)-48)<2||o>4)throw new i("","Invalid uniform dimension type for matrix "+name+": "+n);return"gl.uniformMatrix"+o+"fv(locations["+t+"],false,obj"+e+")"}throw new i("","Unknown uniform data type for "+name+": "+n)}if((o=n.charCodeAt(n.length-1)-48)<2||o>4)throw new i("","Invalid data type");switch(n.charAt(0)){case"b":case"i":return"gl.uniform"+o+"iv(locations["+t+"],obj"+e+")";case"v":return"gl.uniform"+o+"fv(locations["+t+"],obj"+e+")";default:throw new i("","Unrecognized data type for vector "+name+": "+n)}}}function u(t){for(var r=["return function updateProperty(obj){"],i=function e(t,n){if("object"!=typeof n)return[[t,n]];var r=[];for(var i in n){var o=n[i],a=t;parseInt(i)+""===i?a+="["+i+"]":a+="."+i,"object"==typeof o?r.push.apply(r,e(a,o)):r.push([a,o])}return r}("",t),o=0;o<i.length;++o){var a=i[o],u=a[0],c=a[1];s[c]&&r.push(l(u,c,n[c].type))}return r.push("return obj}"),new Function("gl","locations",r.join("\n"))(e,s)}function c(r,l,c){if("object"==typeof c){var h=d(c);Object.defineProperty(r,l,{get:o(h),set:u(c),enumerable:!0,configurable:!1})}else s[c]?Object.defineProperty(r,l,{get:(f=c,new Function("gl","wrapper","locations","return function(){return gl.getUniform(wrapper.program,locations["+f+"])}")(e,t,s)),set:u(c),enumerable:!0,configurable:!1}):r[l]=function(e){switch(e){case"bool":return!1;case"int":case"sampler2D":case"samplerCube":case"float":return 0;default:var t=e.indexOf("vec");if(0<=t&&t<=1&&e.length===4+t){if((n=e.charCodeAt(e.length-1)-48)<2||n>4)throw new i("","Invalid data type");return"b"===e.charAt(0)?a(n,!1):a(n,0)}if(0===e.indexOf("mat")&&4===e.length){var n;if((n=e.charCodeAt(e.length-1)-48)<2||n>4)throw new i("","Invalid uniform dimension type for matrix "+name+": "+e);return a(n*n,0)}throw new i("","Unknown uniform data type for "+name+": "+e)}}(n[c].type);var f}function d(e){var t;if(Array.isArray(e)){t=new Array(e.length);for(var n=0;n<e.length;++n)c(t,n,e[n])}else for(var r in t={},e)c(t,r,e[r]);return t}var h=r(n,!0);return{get:o(d(h)),set:u(h),enumerable:!0,configurable:!0}}},function(e,t,n){"use strict";e.exports=function(e,t,n,i){for(var o={},l=0,u=n.length;l<u;++l){var c=n[l],d=c.name,h=c.type,f=c.locations;switch(h){case"bool":case"int":case"float":a(e,t,f[0],i,1,o,d);break;default:if(h.indexOf("vec")>=0){if((p=h.charCodeAt(h.length-1)-48)<2||p>4)throw new r("","Invalid data type for attribute "+d+": "+h);a(e,t,f[0],i,p,o,d)}else{if(!(h.indexOf("mat")>=0))throw new r("","Unknown data type for attribute "+d+": "+h);var p;if((p=h.charCodeAt(h.length-1)-48)<2||p>4)throw new r("","Invalid data type for attribute "+d+": "+h);s(e,t,f,i,p,o,d)}}}return o};var r=n(395);function i(e,t,n,r,i,o){this._gl=e,this._wrapper=t,this._index=n,this._locations=r,this._dimension=i,this._constFunc=o}var o=i.prototype;function a(e,t,n,r,o,a,s){for(var l=["gl","v"],u=[],c=0;c<o;++c)l.push("x"+c),u.push("x"+c);l.push("if(x0.length===void 0){return gl.vertexAttrib"+o+"f(v,"+u.join()+")}else{return gl.vertexAttrib"+o+"fv(v,x0)}");var d=Function.apply(null,l),h=new i(e,t,n,r,o,d);Object.defineProperty(a,s,{set:function(t){return e.disableVertexAttribArray(r[n]),d(e,r[n],t),t},get:function(){return h},enumerable:!0})}function s(e,t,n,r,i,o,s){for(var l=new Array(i),u=new Array(i),c=0;c<i;++c)a(e,t,n[c],r,i,l,c),u[c]=l[c];Object.defineProperty(l,"location",{set:function(e){if(Array.isArray(e))for(var t=0;t<i;++t)u[t].location=e[t];else for(t=0;t<i;++t)u[t].location=e+t;return e},get:function(){for(var e=new Array(i),t=0;t<i;++t)e[t]=r[n[t]];return e},enumerable:!0}),l.pointer=function(t,o,a,s){t=t||e.FLOAT,o=!!o,a=a||i*i,s=s||0;for(var l=0;l<i;++l){var u=r[n[l]];e.vertexAttribPointer(u,i,t,o,a,s+l*i),e.enableVertexAttribArray(u)}};var d=new Array(i),h=e["vertexAttrib"+i+"fv"];Object.defineProperty(o,s,{set:function(t){for(var o=0;o<i;++o){var a=r[n[o]];if(e.disableVertexAttribArray(a),Array.isArray(t[0]))h.call(e,a,t[o]);else{for(var s=0;s<i;++s)d[s]=t[i*o+s];h.call(e,a,d)}}return t},get:function(){return l},enumerable:!0})}o.pointer=function(e,t,n,r){var i=this._gl,o=this._locations[this._index];i.vertexAttribPointer(o,this._dimension,e||i.FLOAT,!!t,n||0,r||0),i.enableVertexAttribArray(o)},o.set=function(e,t,n,r){return this._constFunc(this._locations[this._index],e,t,n,r)},Object.defineProperty(o,"location",{get:function(){return this._locations[this._index]},set:function(e){return e!==this._locations[this._index]&&(this._locations[this._index]=0|e,this._wrapper.program=null),0|e}})},function(e,t,n){"use strict";t.shader=function(e,t,n){return c(e).getShaderReference(t,n)},t.program=function(e,t,n,r,i){return c(e).getProgram(t,n,r,i)};var r=n(395),i=n(1802),o=new("undefined"==typeof WeakMap?n(1816):WeakMap),a=0;function s(e,t,n,r,i,o,a){this.id=e,this.src=t,this.type=n,this.shader=r,this.count=o,this.programs=[],this.cache=a}function l(e){this.gl=e,this.shaders=[{},{}],this.programs={}}s.prototype.dispose=function(){if(0==--this.count){for(var e=this.cache,t=e.gl,n=this.programs,r=0,i=n.length;r<i;++r){var o=e.programs[n[r]];o&&(delete e.programs[r],t.deleteProgram(o))}t.deleteShader(this.shader),delete e.shaders[this.type===t.FRAGMENT_SHADER|0][this.src]}};var u=l.prototype;function c(e){var t=o.get(e);return t||(t=new l(e),o.set(e,t)),t}u.getShaderReference=function(e,t){var n=this.gl,o=this.shaders[e===n.FRAGMENT_SHADER|0],l=o[t];if(l&&n.isShader(l.shader))l.count+=1;else{var u=function(e,t,n){var o=e.createShader(t);if(e.shaderSource(o,n),e.compileShader(o),!e.getShaderParameter(o,e.COMPILE_STATUS)){var a=e.getShaderInfoLog(o);try{var s=i(a,n,t)}catch(e){throw console.warn("Failed to format compiler error: "+e),new r(a,"Error compiling shader:\n"+a)}throw new r(a,s.short,s.long)}return o}(n,e,t);l=o[t]=new s(a++,t,e,u,[],1,this)}return l},u.getProgram=function(e,t,n,i){var o=[e.id,t.id,n.join(":"),i.join(":")].join("@"),a=this.programs[o];return a&&this.gl.isProgram(a)||(this.programs[o]=a=function(e,t,n,i,o){var a=e.createProgram();e.attachShader(a,t),e.attachShader(a,n);for(var s=0;s<i.length;++s)e.bindAttribLocation(a,o[s],i[s]);if(e.linkProgram(a),!e.getProgramParameter(a,e.LINK_STATUS)){var l=e.getProgramInfoLog(a);throw new r(l,"Error linking program: "+l)}return a}(this.gl,e.shader,t.shader,n,i),e.programs.push(o),t.programs.push(o)),a}},function(e,t,n){var r=n(1803).sprintf,i=n(1804),o=n(1806),a=n(1813);e.exports=function(e,t,n){"use strict";var s=o(t)||"of unknown name (see npm glsl-shader-name)",l="unknown type";void 0!==n&&(l=n===i.FRAGMENT_SHADER?"fragment":"vertex");for(var u=r("Error compiling %s shader %s:\n",l,s),c=r("%s%s",u,e),d=e.split("\n"),h={},f=0;f<d.length;f++){var p=d[f];if(""!==p&&"\0"!==p){var g=parseInt(p.split(":")[2]);if(isNaN(g))throw new Error(r("Could not parse error: %s",p));h[g]=p}}var v=a(t).split("\n");for(f=0;f<v.length;f++)if(h[f+3]||h[f+2]||h[f+1]){var m=v[f];if(u+=m+"\n",h[f+1]){var y=h[f+1];y=y.substr(y.split(":",3).join(":").length+1).trim(),u+=r("^^^ %s\n\n",y)}}return{long:u.trim(),short:c.trim()}}},function(e,t,n){var r;!function(){"use strict";var i={not_string:/[^s]/,not_bool:/[^t]/,not_type:/[^T]/,not_primitive:/[^v]/,number:/[diefg]/,numeric_arg:/[bcdiefguxX]/,json:/[j]/,not_json:/[^j]/,text:/^[^\x25]+/,modulo:/^\x25{2}/,placeholder:/^\x25(?:([1-9]\d*)\$|\(([^)]+)\))?(\+)?(0|'[^$])?(-)?(\d+)?(?:\.(\d+))?([b-gijostTuvxX])/,key:/^([a-z_][a-z_\d]*)/i,key_access:/^\.([a-z_][a-z_\d]*)/i,index_access:/^\[(\d+)\]/,sign:/^[+-]/};function o(e){return s(u(e),arguments)}function a(e,t){return o.apply(null,[e].concat(t||[]))}function s(e,t){var n,r,a,s,l,u,c,d,h,f=1,p=e.length,g="";for(r=0;r<p;r++)if("string"==typeof e[r])g+=e[r];else if("object"==typeof e[r]){if((s=e[r]).keys)for(n=t[f],a=0;a<s.keys.length;a++){if(null==n)throw new Error(o('[sprintf] Cannot access property "%s" of undefined value "%s"',s.keys[a],s.keys[a-1]));n=n[s.keys[a]]}else n=s.param_no?t[s.param_no]:t[f++];if(i.not_type.test(s.type)&&i.not_primitive.test(s.type)&&n instanceof Function&&(n=n()),i.numeric_arg.test(s.type)&&"number"!=typeof n&&isNaN(n))throw new TypeError(o("[sprintf] expecting number but found %T",n));switch(i.number.test(s.type)&&(d=n>=0),s.type){case"b":n=parseInt(n,10).toString(2);break;case"c":n=String.fromCharCode(parseInt(n,10));break;case"d":case"i":n=parseInt(n,10);break;case"j":n=JSON.stringify(n,null,s.width?parseInt(s.width):0);break;case"e":n=s.precision?parseFloat(n).toExponential(s.precision):parseFloat(n).toExponential();break;case"f":n=s.precision?parseFloat(n).toFixed(s.precision):parseFloat(n);break;case"g":n=s.precision?String(Number(n.toPrecision(s.precision))):parseFloat(n);break;case"o":n=(parseInt(n,10)>>>0).toString(8);break;case"s":n=String(n),n=s.precision?n.substring(0,s.precision):n;break;case"t":n=String(!!n),n=s.precision?n.substring(0,s.precision):n;break;case"T":n=Object.prototype.toString.call(n).slice(8,-1).toLowerCase(),n=s.precision?n.substring(0,s.precision):n;break;case"u":n=parseInt(n,10)>>>0;break;case"v":n=n.valueOf(),n=s.precision?n.substring(0,s.precision):n;break;case"x":n=(parseInt(n,10)>>>0).toString(16);break;case"X":n=(parseInt(n,10)>>>0).toString(16).toUpperCase()}i.json.test(s.type)?g+=n:(!i.number.test(s.type)||d&&!s.sign?h="":(h=d?"+":"-",n=n.toString().replace(i.sign,"")),u=s.pad_char?"0"===s.pad_char?"0":s.pad_char.charAt(1):" ",c=s.width-(h+n).length,l=s.width&&c>0?u.repeat(c):"",g+=s.align?h+n+l:"0"===u?h+l+n:l+h+n)}return g}var l=Object.create(null);function u(e){if(l[e])return l[e];for(var t,n=e,r=[],o=0;n;){if(null!==(t=i.text.exec(n)))r.push(t[0]);else if(null!==(t=i.modulo.exec(n)))r.push("%");else{if(null===(t=i.placeholder.exec(n)))throw new SyntaxError("[sprintf] unexpected placeholder");if(t[2]){o|=1;var a=[],s=t[2],u=[];if(null===(u=i.key.exec(s)))throw new SyntaxError("[sprintf] failed to parse named argument key");for(a.push(u[1]);""!==(s=s.substring(u[0].length));)if(null!==(u=i.key_access.exec(s)))a.push(u[1]);else{if(null===(u=i.index_access.exec(s)))throw new SyntaxError("[sprintf] failed to parse named argument key");a.push(u[1])}t[2]=a}else o|=2;if(3===o)throw new Error("[sprintf] mixing positional and named placeholders is not (yet) supported");r.push({placeholder:t[0],param_no:t[1],keys:t[2],sign:t[3],pad_char:t[4],align:t[5],width:t[6],precision:t[7],type:t[8]})}n=n.substring(t[0].length)}return l[e]=r}t.sprintf=o,t.vsprintf=a,"undefined"!=typeof window&&(window.sprintf=o,window.vsprintf=a,void 0===(r=function(){return{sprintf:o,vsprintf:a}}.call(t,n,t,e))||(e.exports=r))}()},function(e,t,n){var r=n(1805);e.exports=function(e){return r[e]}},function(e,t){e.exports={0:"NONE",1:"ONE",2:"LINE_LOOP",3:"LINE_STRIP",4:"TRIANGLES",5:"TRIANGLE_STRIP",6:"TRIANGLE_FAN",256:"DEPTH_BUFFER_BIT",512:"NEVER",513:"LESS",514:"EQUAL",515:"LEQUAL",516:"GREATER",517:"NOTEQUAL",518:"GEQUAL",519:"ALWAYS",768:"SRC_COLOR",769:"ONE_MINUS_SRC_COLOR",770:"SRC_ALPHA",771:"ONE_MINUS_SRC_ALPHA",772:"DST_ALPHA",773:"ONE_MINUS_DST_ALPHA",774:"DST_COLOR",775:"ONE_MINUS_DST_COLOR",776:"SRC_ALPHA_SATURATE",1024:"STENCIL_BUFFER_BIT",1028:"FRONT",1029:"BACK",1032:"FRONT_AND_BACK",1280:"INVALID_ENUM",1281:"INVALID_VALUE",1282:"INVALID_OPERATION",1285:"OUT_OF_MEMORY",1286:"INVALID_FRAMEBUFFER_OPERATION",2304:"CW",2305:"CCW",2849:"LINE_WIDTH",2884:"CULL_FACE",2885:"CULL_FACE_MODE",2886:"FRONT_FACE",2928:"DEPTH_RANGE",2929:"DEPTH_TEST",2930:"DEPTH_WRITEMASK",2931:"DEPTH_CLEAR_VALUE",2932:"DEPTH_FUNC",2960:"STENCIL_TEST",2961:"STENCIL_CLEAR_VALUE",2962:"STENCIL_FUNC",2963:"STENCIL_VALUE_MASK",2964:"STENCIL_FAIL",2965:"STENCIL_PASS_DEPTH_FAIL",2966:"STENCIL_PASS_DEPTH_PASS",2967:"STENCIL_REF",2968:"STENCIL_WRITEMASK",2978:"VIEWPORT",3024:"DITHER",3042:"BLEND",3088:"SCISSOR_BOX",3089:"SCISSOR_TEST",3106:"COLOR_CLEAR_VALUE",3107:"COLOR_WRITEMASK",3317:"UNPACK_ALIGNMENT",3333:"PACK_ALIGNMENT",3379:"MAX_TEXTURE_SIZE",3386:"MAX_VIEWPORT_DIMS",3408:"SUBPIXEL_BITS",3410:"RED_BITS",3411:"GREEN_BITS",3412:"BLUE_BITS",3413:"ALPHA_BITS",3414:"DEPTH_BITS",3415:"STENCIL_BITS",3553:"TEXTURE_2D",4352:"DONT_CARE",4353:"FASTEST",4354:"NICEST",5120:"BYTE",5121:"UNSIGNED_BYTE",5122:"SHORT",5123:"UNSIGNED_SHORT",5124:"INT",5125:"UNSIGNED_INT",5126:"FLOAT",5386:"INVERT",5890:"TEXTURE",6401:"STENCIL_INDEX",6402:"DEPTH_COMPONENT",6406:"ALPHA",6407:"RGB",6408:"RGBA",6409:"LUMINANCE",6410:"LUMINANCE_ALPHA",7680:"KEEP",7681:"REPLACE",7682:"INCR",7683:"DECR",7936:"VENDOR",7937:"RENDERER",7938:"VERSION",9728:"NEAREST",9729:"LINEAR",9984:"NEAREST_MIPMAP_NEAREST",9985:"LINEAR_MIPMAP_NEAREST",9986:"NEAREST_MIPMAP_LINEAR",9987:"LINEAR_MIPMAP_LINEAR",10240:"TEXTURE_MAG_FILTER",10241:"TEXTURE_MIN_FILTER",10242:"TEXTURE_WRAP_S",10243:"TEXTURE_WRAP_T",10497:"REPEAT",10752:"POLYGON_OFFSET_UNITS",16384:"COLOR_BUFFER_BIT",32769:"CONSTANT_COLOR",32770:"ONE_MINUS_CONSTANT_COLOR",32771:"CONSTANT_ALPHA",32772:"ONE_MINUS_CONSTANT_ALPHA",32773:"BLEND_COLOR",32774:"FUNC_ADD",32777:"BLEND_EQUATION_RGB",32778:"FUNC_SUBTRACT",32779:"FUNC_REVERSE_SUBTRACT",32819:"UNSIGNED_SHORT_4_4_4_4",32820:"UNSIGNED_SHORT_5_5_5_1",32823:"POLYGON_OFFSET_FILL",32824:"POLYGON_OFFSET_FACTOR",32854:"RGBA4",32855:"RGB5_A1",32873:"TEXTURE_BINDING_2D",32926:"SAMPLE_ALPHA_TO_COVERAGE",32928:"SAMPLE_COVERAGE",32936:"SAMPLE_BUFFERS",32937:"SAMPLES",32938:"SAMPLE_COVERAGE_VALUE",32939:"SAMPLE_COVERAGE_INVERT",32968:"BLEND_DST_RGB",32969:"BLEND_SRC_RGB",32970:"BLEND_DST_ALPHA",32971:"BLEND_SRC_ALPHA",33071:"CLAMP_TO_EDGE",33170:"GENERATE_MIPMAP_HINT",33189:"DEPTH_COMPONENT16",33306:"DEPTH_STENCIL_ATTACHMENT",33635:"UNSIGNED_SHORT_5_6_5",33648:"MIRRORED_REPEAT",33901:"ALIASED_POINT_SIZE_RANGE",33902:"ALIASED_LINE_WIDTH_RANGE",33984:"TEXTURE0",33985:"TEXTURE1",33986:"TEXTURE2",33987:"TEXTURE3",33988:"TEXTURE4",33989:"TEXTURE5",33990:"TEXTURE6",33991:"TEXTURE7",33992:"TEXTURE8",33993:"TEXTURE9",33994:"TEXTURE10",33995:"TEXTURE11",33996:"TEXTURE12",33997:"TEXTURE13",33998:"TEXTURE14",33999:"TEXTURE15",34e3:"TEXTURE16",34001:"TEXTURE17",34002:"TEXTURE18",34003:"TEXTURE19",34004:"TEXTURE20",34005:"TEXTURE21",34006:"TEXTURE22",34007:"TEXTURE23",34008:"TEXTURE24",34009:"TEXTURE25",34010:"TEXTURE26",34011:"TEXTURE27",34012:"TEXTURE28",34013:"TEXTURE29",34014:"TEXTURE30",34015:"TEXTURE31",34016:"ACTIVE_TEXTURE",34024:"MAX_RENDERBUFFER_SIZE",34041:"DEPTH_STENCIL",34055:"INCR_WRAP",34056:"DECR_WRAP",34067:"TEXTURE_CUBE_MAP",34068:"TEXTURE_BINDING_CUBE_MAP",34069:"TEXTURE_CUBE_MAP_POSITIVE_X",34070:"TEXTURE_CUBE_MAP_NEGATIVE_X",34071:"TEXTURE_CUBE_MAP_POSITIVE_Y",34072:"TEXTURE_CUBE_MAP_NEGATIVE_Y",34073:"TEXTURE_CUBE_MAP_POSITIVE_Z",34074:"TEXTURE_CUBE_MAP_NEGATIVE_Z",34076:"MAX_CUBE_MAP_TEXTURE_SIZE",34338:"VERTEX_ATTRIB_ARRAY_ENABLED",34339:"VERTEX_ATTRIB_ARRAY_SIZE",34340:"VERTEX_ATTRIB_ARRAY_STRIDE",34341:"VERTEX_ATTRIB_ARRAY_TYPE",34342:"CURRENT_VERTEX_ATTRIB",34373:"VERTEX_ATTRIB_ARRAY_POINTER",34466:"NUM_COMPRESSED_TEXTURE_FORMATS",34467:"COMPRESSED_TEXTURE_FORMATS",34660:"BUFFER_SIZE",34661:"BUFFER_USAGE",34816:"STENCIL_BACK_FUNC",34817:"STENCIL_BACK_FAIL",34818:"STENCIL_BACK_PASS_DEPTH_FAIL",34819:"STENCIL_BACK_PASS_DEPTH_PASS",34877:"BLEND_EQUATION_ALPHA",34921:"MAX_VERTEX_ATTRIBS",34922:"VERTEX_ATTRIB_ARRAY_NORMALIZED",34930:"MAX_TEXTURE_IMAGE_UNITS",34962:"ARRAY_BUFFER",34963:"ELEMENT_ARRAY_BUFFER",34964:"ARRAY_BUFFER_BINDING",34965:"ELEMENT_ARRAY_BUFFER_BINDING",34975:"VERTEX_ATTRIB_ARRAY_BUFFER_BINDING",35040:"STREAM_DRAW",35044:"STATIC_DRAW",35048:"DYNAMIC_DRAW",35632:"FRAGMENT_SHADER",35633:"VERTEX_SHADER",35660:"MAX_VERTEX_TEXTURE_IMAGE_UNITS",35661:"MAX_COMBINED_TEXTURE_IMAGE_UNITS",35663:"SHADER_TYPE",35664:"FLOAT_VEC2",35665:"FLOAT_VEC3",35666:"FLOAT_VEC4",35667:"INT_VEC2",35668:"INT_VEC3",35669:"INT_VEC4",35670:"BOOL",35671:"BOOL_VEC2",35672:"BOOL_VEC3",35673:"BOOL_VEC4",35674:"FLOAT_MAT2",35675:"FLOAT_MAT3",35676:"FLOAT_MAT4",35678:"SAMPLER_2D",35680:"SAMPLER_CUBE",35712:"DELETE_STATUS",35713:"COMPILE_STATUS",35714:"LINK_STATUS",35715:"VALIDATE_STATUS",35716:"INFO_LOG_LENGTH",35717:"ATTACHED_SHADERS",35718:"ACTIVE_UNIFORMS",35719:"ACTIVE_UNIFORM_MAX_LENGTH",35720:"SHADER_SOURCE_LENGTH",35721:"ACTIVE_ATTRIBUTES",35722:"ACTIVE_ATTRIBUTE_MAX_LENGTH",35724:"SHADING_LANGUAGE_VERSION",35725:"CURRENT_PROGRAM",36003:"STENCIL_BACK_REF",36004:"STENCIL_BACK_VALUE_MASK",36005:"STENCIL_BACK_WRITEMASK",36006:"FRAMEBUFFER_BINDING",36007:"RENDERBUFFER_BINDING",36048:"FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE",36049:"FRAMEBUFFER_ATTACHMENT_OBJECT_NAME",36050:"FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL",36051:"FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE",36053:"FRAMEBUFFER_COMPLETE",36054:"FRAMEBUFFER_INCOMPLETE_ATTACHMENT",36055:"FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT",36057:"FRAMEBUFFER_INCOMPLETE_DIMENSIONS",36061:"FRAMEBUFFER_UNSUPPORTED",36064:"COLOR_ATTACHMENT0",36096:"DEPTH_ATTACHMENT",36128:"STENCIL_ATTACHMENT",36160:"FRAMEBUFFER",36161:"RENDERBUFFER",36162:"RENDERBUFFER_WIDTH",36163:"RENDERBUFFER_HEIGHT",36164:"RENDERBUFFER_INTERNAL_FORMAT",36168:"STENCIL_INDEX8",36176:"RENDERBUFFER_RED_SIZE",36177:"RENDERBUFFER_GREEN_SIZE",36178:"RENDERBUFFER_BLUE_SIZE",36179:"RENDERBUFFER_ALPHA_SIZE",36180:"RENDERBUFFER_DEPTH_SIZE",36181:"RENDERBUFFER_STENCIL_SIZE",36194:"RGB565",36336:"LOW_FLOAT",36337:"MEDIUM_FLOAT",36338:"HIGH_FLOAT",36339:"LOW_INT",36340:"MEDIUM_INT",36341:"HIGH_INT",36346:"SHADER_COMPILER",36347:"MAX_VERTEX_UNIFORM_VECTORS",36348:"MAX_VARYING_VECTORS",36349:"MAX_FRAGMENT_UNIFORM_VECTORS",37440:"UNPACK_FLIP_Y_WEBGL",37441:"UNPACK_PREMULTIPLY_ALPHA_WEBGL",37442:"CONTEXT_LOST_WEBGL",37443:"UNPACK_COLORSPACE_CONVERSION_WEBGL",37444:"BROWSER_DEFAULT_WEBGL"}},function(e,t,n){var r=n(1807),i=n(1812);e.exports=function(e){for(var t=Array.isArray(e)?e:r(e),n=0;n<t.length;n++){var o=t[n];if("preprocessor"===o.type){var a=o.data.match(/\#define\s+SHADER_NAME(_B64)?\s+(.+)$/);if(a&&a[2]){var s=a[1],l=a[2];return(s?i(l):l).trim()}}}}},function(e,t,n){var r=n(1808);e.exports=function(e,t){var n=r(t),i=[];return i=(i=i.concat(n(e))).concat(n(null))}},function(e,t,n){e.exports=function(e){var t,n,u,c=0,d=0,h=999,f=[],p=[],g=1,v=0,m=0,y=!1,b=!1,C="",A=o,_=r;"300 es"===(e=e||{}).version&&(A=s,_=a);var I={},x={};for(c=0;c<A.length;c++)I[A[c]]=!0;for(c=0;c<_.length;c++)x[_[c]]=!0;return function(e){return p=[],null!==e?function(e){c=0,e.toString&&(e=e.toString());var n;C+=e.replace(/\r\n/g,"\n"),u=C.length;for(;t=C[c],c<u;){switch(n=c,h){case 0:c=E();break;case 1:case 2:c=T();break;case 3:c=O();break;case 4:c=D();break;case 11:c=L();break;case 5:c=P();break;case 9999:c=N();break;case 9:c=S();break;case 999:c=M()}if(n!==c)switch(C[n]){case"\n":v=0,++g;break;default:++v}}return d+=c,C=C.slice(c),p}(e):function(e){f.length&&w(f.join(""));return h=10,w("(eof)"),p}()};function w(e){e.length&&p.push({type:l[h],data:e,position:m,line:g,column:v})}function M(){return f=f.length?[]:f,"/"===n&&"*"===t?(m=d+c-1,h=0,n=t,c+1):"/"===n&&"/"===t?(m=d+c-1,h=1,n=t,c+1):"#"===t?(h=2,m=d+c,c):/\s/.test(t)?(h=9,m=d+c,c):(y=/\d/.test(t),b=/[^\w_]/.test(t),m=d+c,h=y?4:b?3:9999,c)}function S(){return/[^\s]/g.test(t)?(w(f.join("")),h=999,c):(f.push(t),n=t,c+1)}function T(){return"\r"!==t&&"\n"!==t||"\\"===n?(f.push(t),n=t,c+1):(w(f.join("")),h=999,c)}function E(){return"/"===t&&"*"===n?(f.push(t),w(f.join("")),h=999,c+1):(f.push(t),n=t,c+1)}function O(){if("."===n&&/\d/.test(t))return h=5,c;if("/"===n&&"*"===t)return h=0,c;if("/"===n&&"/"===t)return h=1,c;if("."===t&&f.length){for(;k(f););return h=5,c}if(";"===t||")"===t||"("===t){if(f.length)for(;k(f););return w(t),h=999,c+1}var e=2===f.length&&"="!==t;if(/[\w_\d\s]/.test(t)||e){for(;k(f););return h=999,c}return f.push(t),n=t,c+1}function k(e){for(var t,n,r=0;;){if(t=i.indexOf(e.slice(0,e.length+r).join("")),n=i[t],-1===t){if(r--+e.length>0)continue;n=e.slice(0,1).join("")}return w(n),m+=n.length,(f=f.slice(n.length)).length}}function L(){return/[^a-fA-F0-9]/.test(t)?(w(f.join("")),h=999,c):(f.push(t),n=t,c+1)}function D(){return"."===t||/[eE]/.test(t)?(f.push(t),h=5,n=t,c+1):"x"===t&&1===f.length&&"0"===f[0]?(h=11,f.push(t),n=t,c+1):/[^\d]/.test(t)?(w(f.join("")),h=999,c):(f.push(t),n=t,c+1)}function P(){return"f"===t&&(f.push(t),n=t,c+=1),/[eE]/.test(t)?(f.push(t),n=t,c+1):("-"!==t&&"+"!==t||!/[eE]/.test(n))&&/[^\d]/.test(t)?(w(f.join("")),h=999,c):(f.push(t),n=t,c+1)}function N(){if(/[^\d\w_]/.test(t)){var e=f.join("");return h=x[e]?8:I[e]?7:6,w(f.join("")),h=999,c}return f.push(t),n=t,c+1}};var r=n(1039),i=n(1809),o=n(1040),a=n(1810),s=n(1811),l=["block-comment","line-comment","preprocessor","operator","integer","float","ident","builtin","keyword","whitespace","eof","integer"]},function(e,t){e.exports=["<<=",">>=","++","--","<<",">>","<=",">=","==","!=","&&","||","+=","-=","*=","/=","%=","&=","^^","^=","|=","(",")","[","]",".","!","~","*","/","%","+","-","<",">","&","^","|","?",":","=",",",";","{","}"]},function(e,t,n){var r=n(1039);e.exports=r.slice().concat(["layout","centroid","smooth","case","mat2x2","mat2x3","mat2x4","mat3x2","mat3x3","mat3x4","mat4x2","mat4x3","mat4x4","uvec2","uvec3","uvec4","samplerCubeShadow","sampler2DArray","sampler2DArrayShadow","isampler2D","isampler3D","isamplerCube","isampler2DArray","usampler2D","usampler3D","usamplerCube","usampler2DArray","coherent","restrict","readonly","writeonly","resource","atomic_uint","noperspective","patch","sample","subroutine","common","partition","active","filter","image1D","image2D","image3D","imageCube","iimage1D","iimage2D","iimage3D","iimageCube","uimage1D","uimage2D","uimage3D","uimageCube","image1DArray","image2DArray","iimage1DArray","iimage2DArray","uimage1DArray","uimage2DArray","image1DShadow","image2DShadow","image1DArrayShadow","image2DArrayShadow","imageBuffer","iimageBuffer","uimageBuffer","sampler1DArray","sampler1DArrayShadow","isampler1D","isampler1DArray","usampler1D","usampler1DArray","isampler2DRect","usampler2DRect","samplerBuffer","isamplerBuffer","usamplerBuffer","sampler2DMS","isampler2DMS","usampler2DMS","sampler2DMSArray","isampler2DMSArray","usampler2DMSArray"])},function(e,t,n){var r=n(1040);r=r.slice().filter((function(e){return!/^(gl\_|texture)/.test(e)})),e.exports=r.concat(["gl_VertexID","gl_InstanceID","gl_Position","gl_PointSize","gl_FragCoord","gl_FrontFacing","gl_FragDepth","gl_PointCoord","gl_MaxVertexAttribs","gl_MaxVertexUniformVectors","gl_MaxVertexOutputVectors","gl_MaxFragmentInputVectors","gl_MaxVertexTextureImageUnits","gl_MaxCombinedTextureImageUnits","gl_MaxTextureImageUnits","gl_MaxFragmentUniformVectors","gl_MaxDrawBuffers","gl_MinProgramTexelOffset","gl_MaxProgramTexelOffset","gl_DepthRangeParameters","gl_DepthRange","trunc","round","roundEven","isnan","isinf","floatBitsToInt","floatBitsToUint","intBitsToFloat","uintBitsToFloat","packSnorm2x16","unpackSnorm2x16","packUnorm2x16","unpackUnorm2x16","packHalf2x16","unpackHalf2x16","outerProduct","transpose","determinant","inverse","texture","textureSize","textureProj","textureLod","textureOffset","texelFetch","texelFetchOffset","textureProjOffset","textureLodOffset","textureProjLod","textureProjLodOffset","textureGrad","textureGradOffset","textureProjGrad","textureProjGradOffset"])},function(e,t){e.exports=function(e){return atob(e)}},function(e,t,n){var r=n(1814);e.exports=function(e,t,n){t="number"==typeof t?t:1,n=n||": ";var i=e.split(/\r?\n/),o=String(i.length+t-1).length;return i.map((function(e,i){var a=i+t,s=String(a).length;return r(a,o-s)+n+e})).join("\n")}},function(e,t,n){"use strict";
/*!
 * pad-left <https://github.com/jonschlinkert/pad-left>
 *
 * Copyright (c) 2014-2015, Jon Schlinkert.
 * Licensed under the MIT license.
 */var r=n(1815);e.exports=function(e,t,n){return r(n=void 0!==n?n+"":" ",t)+e}},function(e,t,n){"use strict";
/*!
 * repeat-string <https://github.com/jonschlinkert/repeat-string>
 *
 * Copyright (c) 2014-2015, Jon Schlinkert.
 * Licensed under the MIT License.
 */var r,i="";e.exports=function(e,t){if("string"!=typeof e)throw new TypeError("expected a string");if(1===t)return e;if(2===t)return e+e;var n=e.length*t;if(r!==e||void 0===r)r=e,i="";else if(i.length>=n)return i.substr(0,n);for(;n>i.length&&t>1;)1&t&&(i+=e),t>>=1,e+=e;return i=(i+=e).substr(0,n)}},function(e,t,n){var r=n(1817);e.exports=function(){var e=r();return{get:function(t,n){var r=e(t);return r.hasOwnProperty("value")?r.value:n},set:function(t,n){return e(t).value=n,this},has:function(t){return"value"in e(t)},delete:function(t){return delete e(t).value}}}},function(e,t,n){var r=n(1818);e.exports=function(){var e={};return function(t){if(("object"!=typeof t||null===t)&&"function"!=typeof t)throw new Error("Weakmap-shim: Key must be object");var n=t.valueOf(e);return n&&n.identity===e?n:r(t,e)}}},function(e,t){e.exports=function(e,t){var n={identity:t},r=e.valueOf;return Object.defineProperty(e,"valueOf",{value:function(e){return e!==t?r.apply(this,arguments):n},writable:!0}),n}},function(e,t,n){"use strict";t.uniforms=function(e,t){for(var n=e.getProgramParameter(t,e.ACTIVE_UNIFORMS),r=[],i=0;i<n;++i){var a=e.getActiveUniform(t,i);if(a){var s=o(e,a.type);if(a.size>1)for(var l=0;l<a.size;++l)r.push({name:a.name.replace("[0]","["+l+"]"),type:s});else r.push({name:a.name,type:s})}}return r},t.attributes=function(e,t){for(var n=e.getProgramParameter(t,e.ACTIVE_ATTRIBUTES),r=[],i=0;i<n;++i){var a=e.getActiveAttrib(t,i);a&&r.push({name:a.name,type:o(e,a.type)})}return r};var r={FLOAT:"float",FLOAT_VEC2:"vec2",FLOAT_VEC3:"vec3",FLOAT_VEC4:"vec4",INT:"int",INT_VEC2:"ivec2",INT_VEC3:"ivec3",INT_VEC4:"ivec4",BOOL:"bool",BOOL_VEC2:"bvec2",BOOL_VEC3:"bvec3",BOOL_VEC4:"bvec4",FLOAT_MAT2:"mat2",FLOAT_MAT3:"mat3",FLOAT_MAT4:"mat4",SAMPLER_2D:"sampler2D",SAMPLER_CUBE:"samplerCube"},i=null;function o(e,t){if(!i){var n=Object.keys(r);i={};for(var o=0;o<n.length;++o){var a=n[o];i[e[a]]=r[a]}}return i[t]}},function(e,t,n){"use strict";var r=n(965),i=n(180),o=n(181),a=n(112),s=n(255),l=n(1821),u=n(1822),c=n(1879),d=[1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1];function h(e,t){var n=e[0],r=e[1],i=e[2],o=e[3];return e[0]=t[0]*n+t[4]*r+t[8]*i+t[12]*o,e[1]=t[1]*n+t[5]*r+t[9]*i+t[13]*o,e[2]=t[2]*n+t[6]*r+t[10]*i+t[14]*o,e[3]=t[3]*n+t[7]*r+t[11]*i+t[15]*o,e}function f(e,t,n,r){return h(r,r),h(r,r),h(r,r)}function p(e,t){this.index=e,this.dataCoordinate=this.position=t}function g(e){return!0===e||e>1?1:e}function v(e,t,n,r,i,o,a,s,l,u,c,d){this.gl=e,this.pixelRatio=1,this.shader=t,this.orthoShader=n,this.projectShader=r,this.pointBuffer=i,this.colorBuffer=o,this.glyphBuffer=a,this.idBuffer=s,this.vao=l,this.vertexCount=0,this.lineVertexCount=0,this.opacity=1,this.hasAlpha=!1,this.lineWidth=0,this.projectScale=[2/3,2/3,2/3],this.projectOpacity=[1,1,1],this.projectHasAlpha=!1,this.pickId=0,this.pickPerspectiveShader=u,this.pickOrthoShader=c,this.pickProjectShader=d,this.points=[],this._selectResult=new p(0,[0,0,0]),this.useOrtho=!0,this.bounds=[[1/0,1/0,1/0],[-1/0,-1/0,-1/0]],this.axesProject=[!0,!0,!0],this.axesBounds=[[-1/0,-1/0,-1/0],[1/0,1/0,1/0]],this.highlightId=[1,1,1,1],this.highlightScale=2,this.clipBounds=[[-1/0,-1/0,-1/0],[1/0,1/0,1/0]],this.dirty=!0}e.exports=function(e){var t=e.gl,n=l.createPerspective(t),r=l.createOrtho(t),a=l.createProject(t),s=l.createPickPerspective(t),u=l.createPickOrtho(t),c=l.createPickProject(t),d=i(t),h=i(t),f=i(t),p=i(t),g=o(t,[{buffer:d,size:3,type:t.FLOAT},{buffer:h,size:4,type:t.FLOAT},{buffer:f,size:2,type:t.FLOAT},{buffer:p,size:4,type:t.UNSIGNED_BYTE,normalized:!0}]),m=new v(t,n,r,a,d,h,f,p,g,s,u,c);return m.update(e),m};var m=v.prototype;m.pickSlots=1,m.setPickBase=function(e){this.pickId=e},m.isTransparent=function(){if(this.hasAlpha)return!0;for(var e=0;e<3;++e)if(this.axesProject[e]&&this.projectHasAlpha)return!0;return!1},m.isOpaque=function(){if(!this.hasAlpha)return!0;for(var e=0;e<3;++e)if(this.axesProject[e]&&!this.projectHasAlpha)return!0;return!1};var y=[0,0],b=[0,0,0],C=[0,0,0],A=[0,0,0,1],_=[0,0,0,1],I=d.slice(),x=[0,0,0],w=[[0,0,0],[0,0,0]];function M(e){return e[0]=e[1]=e[2]=0,e}function S(e,t){return e[0]=t[0],e[1]=t[1],e[2]=t[2],e[3]=1,e}function T(e,t,n,r){return e[0]=t[0],e[1]=t[1],e[2]=t[2],e[n]=r,e}function E(e,t,n,r){var i,o=t.axesProject,a=t.gl,l=e.uniforms,u=n.model||d,c=n.view||d,h=n.projection||d,p=t.axesBounds,g=function(e){for(var t=w,n=0;n<2;++n)for(var r=0;r<3;++r)t[n][r]=Math.max(Math.min(e[n][r],1e8),-1e8);return t}(t.clipBounds);i=t.axes&&t.axes.lastCubeProps?t.axes.lastCubeProps.axis:[1,1,1],y[0]=2/a.drawingBufferWidth,y[1]=2/a.drawingBufferHeight,e.bind(),l.view=c,l.projection=h,l.screenSize=y,l.highlightId=t.highlightId,l.highlightScale=t.highlightScale,l.clipBounds=g,l.pickGroup=t.pickId/255,l.pixelRatio=r;for(var v=0;v<3;++v)if(o[v]){l.scale=t.projectScale[v],l.opacity=t.projectOpacity[v];for(var m=I,E=0;E<16;++E)m[E]=0;for(E=0;E<4;++E)m[5*E]=1;m[5*v]=0,i[v]<0?m[12+v]=p[0][v]:m[12+v]=p[1][v],s(m,u,m),l.model=m;var O=(v+1)%3,k=(v+2)%3,L=M(b),D=M(C);L[O]=1,D[k]=1;var P=f(0,0,0,S(A,L)),N=f(0,0,0,S(_,D));if(Math.abs(P[1])>Math.abs(N[1])){var j=P;P=N,N=j,j=L,L=D,D=j;var R=O;O=k,k=R}P[0]<0&&(L[O]=-1),N[1]>0&&(D[k]=-1);var z=0,F=0;for(E=0;E<4;++E)z+=Math.pow(u[4*O+E],2),F+=Math.pow(u[4*k+E],2);L[O]/=Math.sqrt(z),D[k]/=Math.sqrt(F),l.axes[0]=L,l.axes[1]=D,l.fragClipBounds[0]=T(x,g[0],v,-1e8),l.fragClipBounds[1]=T(x,g[1],v,1e8),t.vao.bind(),t.vao.draw(a.TRIANGLES,t.vertexCount),t.lineWidth>0&&(a.lineWidth(t.lineWidth*r),t.vao.draw(a.LINES,t.lineVertexCount,t.vertexCount)),t.vao.unbind()}}var O=[[-1e8,-1e8,-1e8],[1e8,1e8,1e8]];function k(e,t,n,r,i,o,a){var s=n.gl;if((o===n.projectHasAlpha||a)&&E(t,n,r,i),o===n.hasAlpha||a){e.bind();var l=e.uniforms;l.model=r.model||d,l.view=r.view||d,l.projection=r.projection||d,y[0]=2/s.drawingBufferWidth,y[1]=2/s.drawingBufferHeight,l.screenSize=y,l.highlightId=n.highlightId,l.highlightScale=n.highlightScale,l.fragClipBounds=O,l.clipBounds=n.axes.bounds,l.opacity=n.opacity,l.pickGroup=n.pickId/255,l.pixelRatio=i,n.vao.bind(),n.vao.draw(s.TRIANGLES,n.vertexCount),n.lineWidth>0&&(s.lineWidth(n.lineWidth*i),n.vao.draw(s.LINES,n.lineVertexCount,n.vertexCount)),n.vao.unbind()}}function L(e,t,n,i){var o;o=Array.isArray(e)?t<e.length?e[t]:void 0:e,o=c(o);var a=!0;r(o)&&(o="▼",a=!1);var s=u(o,n,i);return{mesh:s[0],lines:s[1],bounds:s[2],visible:a}}m.draw=function(e){k(this.useOrtho?this.orthoShader:this.shader,this.projectShader,this,e,this.pixelRatio,!1,!1)},m.drawTransparent=function(e){k(this.useOrtho?this.orthoShader:this.shader,this.projectShader,this,e,this.pixelRatio,!0,!1)},m.drawPick=function(e){k(this.useOrtho?this.pickOrthoShader:this.pickPerspectiveShader,this.pickProjectShader,this,e,1,!0,!0)},m.pick=function(e){if(!e)return null;if(e.id!==this.pickId)return null;var t=e.value[2]+(e.value[1]<<8)+(e.value[0]<<16);if(t>=this.pointCount||t<0)return null;var n=this.points[t],r=this._selectResult;r.index=t;for(var i=0;i<3;++i)r.position[i]=r.dataCoordinate[i]=n[i];return r},m.highlight=function(e){if(e){var t=e.index,n=255&t,r=t>>8&255,i=t>>16&255;this.highlightId=[n/255,r/255,i/255,0]}else this.highlightId=[1,1,1,1]},m.update=function(e){if("perspective"in(e=e||{})&&(this.useOrtho=!e.perspective),"orthographic"in e&&(this.useOrtho=!!e.orthographic),"lineWidth"in e&&(this.lineWidth=e.lineWidth),"project"in e)if(Array.isArray(e.project))this.axesProject=e.project;else{var t=!!e.project;this.axesProject=[t,t,t]}if("projectScale"in e)if(Array.isArray(e.projectScale))this.projectScale=e.projectScale.slice();else{var n=+e.projectScale;this.projectScale=[n,n,n]}if(this.projectHasAlpha=!1,"projectOpacity"in e){if(Array.isArray(e.projectOpacity))this.projectOpacity=e.projectOpacity.slice();else{n=+e.projectOpacity;this.projectOpacity=[n,n,n]}for(var r=0;r<3;++r)this.projectOpacity[r]=g(this.projectOpacity[r]),this.projectOpacity[r]<1&&(this.projectHasAlpha=!0)}this.hasAlpha=!1,"opacity"in e&&(this.opacity=g(e.opacity),this.opacity<1&&(this.hasAlpha=!0)),this.dirty=!0;var i,o,s=e.position,l=e.font||"normal",u=e.alignment||[0,0];if(2===u.length)i=u[0],o=u[1];else{i=[],o=[];for(r=0;r<u.length;++r)i[r]=u[r][0],o[r]=u[r][1]}var c=[1/0,1/0,1/0],d=[-1/0,-1/0,-1/0],h=e.glyph,f=e.color,p=e.size,v=e.angle,m=e.lineColor,y=-1,b=0,C=0,A=0;if(s.length){A=s.length;e:for(r=0;r<A;++r){for(var _=s[r],I=0;I<3;++I)if(isNaN(_[I])||!isFinite(_[I]))continue e;var x=(F=L(h,r,l,this.pixelRatio)).mesh,w=F.lines,M=F.bounds;b+=3*x.cells.length,C+=2*w.edges.length}}var S=b+C,T=a.mallocFloat(3*S),E=a.mallocFloat(4*S),O=a.mallocFloat(2*S),k=a.mallocUint32(S);if(S>0){var D=0,P=b,N=[0,0,0,1],j=[0,0,0,1],R=Array.isArray(f)&&Array.isArray(f[0]),z=Array.isArray(m)&&Array.isArray(m[0]);e:for(r=0;r<A;++r){y+=1;for(_=s[r],I=0;I<3;++I){if(isNaN(_[I])||!isFinite(_[I]))continue e;d[I]=Math.max(d[I],_[I]),c[I]=Math.min(c[I],_[I])}x=(F=L(h,r,l,this.pixelRatio)).mesh,w=F.lines,M=F.bounds;var F,B=F.visible;if(B)if(Array.isArray(f)){if(3===(H=R?r<f.length?f[r]:[0,0,0,0]:f).length){for(I=0;I<3;++I)N[I]=H[I];N[3]=1}else if(4===H.length){for(I=0;I<4;++I)N[I]=H[I];!this.hasAlpha&&H[3]<1&&(this.hasAlpha=!0)}}else N[0]=N[1]=N[2]=0,N[3]=1;else N=[1,1,1,0];if(B)if(Array.isArray(m)){var H;if(3===(H=z?r<m.length?m[r]:[0,0,0,0]:m).length){for(I=0;I<3;++I)j[I]=H[I];j[I]=1}else if(4===H.length){for(I=0;I<4;++I)j[I]=H[I];!this.hasAlpha&&H[3]<1&&(this.hasAlpha=!0)}}else j[0]=j[1]=j[2]=0,j[3]=1;else j=[1,1,1,0];var U=.5;B?Array.isArray(p)?U=r<p.length?+p[r]:12:p?U=+p:this.useOrtho&&(U=12):U=0;var V=0;Array.isArray(v)?V=r<v.length?+v[r]:0:v&&(V=+v);var Y=Math.cos(V),W=Math.sin(V);for(_=s[r],I=0;I<3;++I)d[I]=Math.max(d[I],_[I]),c[I]=Math.min(c[I],_[I]);var G=i,X=o;G=0;Array.isArray(i)?G=r<i.length?i[r]:0:i&&(G=i);X=0;Array.isArray(o)?X=r<o.length?o[r]:0:o&&(X=o);var q=[G*=G>0?1-M[0][0]:G<0?1+M[1][0]:1,X*=X>0?1-M[0][1]:X<0?1+M[1][1]:1],Z=x.cells||[],Q=x.positions||[];for(I=0;I<Z.length;++I)for(var K=Z[I],J=0;J<3;++J){for(var $=0;$<3;++$)T[3*D+$]=_[$];for($=0;$<4;++$)E[4*D+$]=N[$];k[D]=y;var ee=Q[K[J]];O[2*D]=U*(Y*ee[0]-W*ee[1]+q[0]),O[2*D+1]=U*(W*ee[0]+Y*ee[1]+q[1]),D+=1}for(Z=w.edges,Q=w.positions,I=0;I<Z.length;++I)for(K=Z[I],J=0;J<2;++J){for($=0;$<3;++$)T[3*P+$]=_[$];for($=0;$<4;++$)E[4*P+$]=j[$];k[P]=y;ee=Q[K[J]];O[2*P]=U*(Y*ee[0]-W*ee[1]+q[0]),O[2*P+1]=U*(W*ee[0]+Y*ee[1]+q[1]),P+=1}}}this.bounds=[c,d],this.points=s,this.pointCount=s.length,this.vertexCount=b,this.lineVertexCount=C,this.pointBuffer.update(T),this.colorBuffer.update(E),this.glyphBuffer.update(O),this.idBuffer.update(k),a.free(T),a.free(E),a.free(O),a.free(k)},m.dispose=function(){this.shader.dispose(),this.orthoShader.dispose(),this.pickPerspectiveShader.dispose(),this.pickOrthoShader.dispose(),this.vao.dispose(),this.pointBuffer.dispose(),this.colorBuffer.dispose(),this.glyphBuffer.dispose(),this.idBuffer.dispose()}},function(e,t,n){var r=n(218),i=n(217),o=i(["precision highp float;\n#define GLSLIFY 1\n\nbool outOfRange(float a, float b, float p) {\n  return ((p > max(a, b)) || \n          (p < min(a, b)));\n}\n\nbool outOfRange(vec2 a, vec2 b, vec2 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y));\n}\n\nbool outOfRange(vec3 a, vec3 b, vec3 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y) ||\n          outOfRange(a.z, b.z, p.z));\n}\n\nbool outOfRange(vec4 a, vec4 b, vec4 p) {\n  return outOfRange(a.xyz, b.xyz, p.xyz);\n}\n\nattribute vec3 position;\nattribute vec4 color;\nattribute vec2 glyph;\nattribute vec4 id;\n\nuniform vec4 highlightId;\nuniform float highlightScale;\nuniform mat4 model, view, projection;\nuniform vec3 clipBounds[2];\n\nvarying vec4 interpColor;\nvarying vec4 pickId;\nvarying vec3 dataCoordinate;\n\nvoid main() {\n  if (outOfRange(clipBounds[0], clipBounds[1], position)) {\n\n    gl_Position = vec4(0,0,0,0);\n  } else {\n    float scale = 1.0;\n    if(distance(highlightId, id) < 0.0001) {\n      scale = highlightScale;\n    }\n\n    vec4 worldPosition = model * vec4(position, 1);\n    vec4 viewPosition = view * worldPosition;\n    viewPosition = viewPosition / viewPosition.w;\n    vec4 clipPosition = projection * (viewPosition + scale * vec4(glyph.x, -glyph.y, 0, 0));\n\n    gl_Position = clipPosition;\n    interpColor = color;\n    pickId = id;\n    dataCoordinate = position;\n  }\n}"]),a=i(["precision highp float;\n#define GLSLIFY 1\n\nbool outOfRange(float a, float b, float p) {\n  return ((p > max(a, b)) || \n          (p < min(a, b)));\n}\n\nbool outOfRange(vec2 a, vec2 b, vec2 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y));\n}\n\nbool outOfRange(vec3 a, vec3 b, vec3 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y) ||\n          outOfRange(a.z, b.z, p.z));\n}\n\nbool outOfRange(vec4 a, vec4 b, vec4 p) {\n  return outOfRange(a.xyz, b.xyz, p.xyz);\n}\n\nattribute vec3 position;\nattribute vec4 color;\nattribute vec2 glyph;\nattribute vec4 id;\n\nuniform mat4 model, view, projection;\nuniform vec2 screenSize;\nuniform vec3 clipBounds[2];\nuniform float highlightScale, pixelRatio;\nuniform vec4 highlightId;\n\nvarying vec4 interpColor;\nvarying vec4 pickId;\nvarying vec3 dataCoordinate;\n\nvoid main() {\n  if (outOfRange(clipBounds[0], clipBounds[1], position)) {\n\n    gl_Position = vec4(0,0,0,0);\n  } else {\n    float scale = pixelRatio;\n    if(distance(highlightId.bgr, id.bgr) < 0.001) {\n      scale *= highlightScale;\n    }\n\n    vec4 worldPosition = model * vec4(position, 1.0);\n    vec4 viewPosition = view * worldPosition;\n    vec4 clipPosition = projection * viewPosition;\n    clipPosition /= clipPosition.w;\n\n    gl_Position = clipPosition + vec4(screenSize * scale * vec2(glyph.x, -glyph.y), 0.0, 0.0);\n    interpColor = color;\n    pickId = id;\n    dataCoordinate = position;\n  }\n}"]),s=i(["precision highp float;\n#define GLSLIFY 1\n\nbool outOfRange(float a, float b, float p) {\n  return ((p > max(a, b)) || \n          (p < min(a, b)));\n}\n\nbool outOfRange(vec2 a, vec2 b, vec2 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y));\n}\n\nbool outOfRange(vec3 a, vec3 b, vec3 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y) ||\n          outOfRange(a.z, b.z, p.z));\n}\n\nbool outOfRange(vec4 a, vec4 b, vec4 p) {\n  return outOfRange(a.xyz, b.xyz, p.xyz);\n}\n\nattribute vec3 position;\nattribute vec4 color;\nattribute vec2 glyph;\nattribute vec4 id;\n\nuniform float highlightScale;\nuniform vec4 highlightId;\nuniform vec3 axes[2];\nuniform mat4 model, view, projection;\nuniform vec2 screenSize;\nuniform vec3 clipBounds[2];\nuniform float scale, pixelRatio;\n\nvarying vec4 interpColor;\nvarying vec4 pickId;\nvarying vec3 dataCoordinate;\n\nvoid main() {\n  if (outOfRange(clipBounds[0], clipBounds[1], position)) {\n\n    gl_Position = vec4(0,0,0,0);\n  } else {\n    float lscale = pixelRatio * scale;\n    if(distance(highlightId, id) < 0.0001) {\n      lscale *= highlightScale;\n    }\n\n    vec4 clipCenter   = projection * view * model * vec4(position, 1);\n    vec3 dataPosition = position + 0.5*lscale*(axes[0] * glyph.x + axes[1] * glyph.y) * clipCenter.w * screenSize.y;\n    vec4 clipPosition = projection * view * model * vec4(dataPosition, 1);\n\n    gl_Position = clipPosition;\n    interpColor = color;\n    pickId = id;\n    dataCoordinate = dataPosition;\n  }\n}\n"]),l=i(["precision highp float;\n#define GLSLIFY 1\n\nbool outOfRange(float a, float b, float p) {\n  return ((p > max(a, b)) || \n          (p < min(a, b)));\n}\n\nbool outOfRange(vec2 a, vec2 b, vec2 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y));\n}\n\nbool outOfRange(vec3 a, vec3 b, vec3 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y) ||\n          outOfRange(a.z, b.z, p.z));\n}\n\nbool outOfRange(vec4 a, vec4 b, vec4 p) {\n  return outOfRange(a.xyz, b.xyz, p.xyz);\n}\n\nuniform vec3 fragClipBounds[2];\nuniform float opacity;\n\nvarying vec4 interpColor;\nvarying vec3 dataCoordinate;\n\nvoid main() {\n  if (\n    outOfRange(fragClipBounds[0], fragClipBounds[1], dataCoordinate) ||\n    interpColor.a * opacity == 0.\n  ) discard;\n  gl_FragColor = interpColor * opacity;\n}\n"]),u=i(["precision highp float;\n#define GLSLIFY 1\n\nbool outOfRange(float a, float b, float p) {\n  return ((p > max(a, b)) || \n          (p < min(a, b)));\n}\n\nbool outOfRange(vec2 a, vec2 b, vec2 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y));\n}\n\nbool outOfRange(vec3 a, vec3 b, vec3 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y) ||\n          outOfRange(a.z, b.z, p.z));\n}\n\nbool outOfRange(vec4 a, vec4 b, vec4 p) {\n  return outOfRange(a.xyz, b.xyz, p.xyz);\n}\n\nuniform vec3 fragClipBounds[2];\nuniform float pickGroup;\n\nvarying vec4 pickId;\nvarying vec3 dataCoordinate;\n\nvoid main() {\n  if (outOfRange(fragClipBounds[0], fragClipBounds[1], dataCoordinate)) discard;\n\n  gl_FragColor = vec4(pickGroup, pickId.bgr);\n}"]),c=[{name:"position",type:"vec3"},{name:"color",type:"vec4"},{name:"glyph",type:"vec2"},{name:"id",type:"vec4"}],d={vertex:o,fragment:l,attributes:c},h={vertex:a,fragment:l,attributes:c},f={vertex:s,fragment:l,attributes:c},p={vertex:o,fragment:u,attributes:c},g={vertex:a,fragment:u,attributes:c},v={vertex:s,fragment:u,attributes:c};function m(e,t){var n=r(e,t),i=n.attributes;return i.position.location=0,i.color.location=1,i.glyph.location=2,i.id.location=3,n}t.createPerspective=function(e){return m(e,d)},t.createOrtho=function(e){return m(e,h)},t.createProject=function(e){return m(e,f)},t.createPickPerspective=function(e){return m(e,p)},t.createPickOrtho=function(e){return m(e,g)},t.createPickProject=function(e){return m(e,v)}},function(e,t,n){"use strict";var r=n(1041);e.exports=function(e,t,n){var o=i[t];o||(o=i[t]={});if(e in o)return o[e];var a={textAlign:"center",textBaseline:"middle",lineHeight:1,font:t,lineSpacing:1.25,styletags:{breaklines:!0,bolds:!0,italics:!0,subscripts:!0,superscripts:!0},triangles:!0},s=r(e,a);a.triangles=!1;var l,u,c=r(e,a);if(n&&1!==n){for(l=0;l<s.positions.length;++l)for(u=0;u<s.positions[l].length;++u)s.positions[l][u]/=n;for(l=0;l<c.positions.length;++l)for(u=0;u<c.positions[l].length;++u)c.positions[l][u]/=n}var d=[[1/0,1/0],[-1/0,-1/0]],h=c.positions.length;for(l=0;l<h;++l){var f=c.positions[l];for(u=0;u<2;++u)d[0][u]=Math.min(d[0][u],f[u]),d[1][u]=Math.max(d[1][u],f[u])}return o[e]=[s,c,d]};var i={}},function(e,t,n){e.exports=function(e,t,n,r){var o=64,a=1.25,s={breaklines:!1,bolds:!1,italics:!1,subscripts:!1,superscripts:!1};r&&(r.size&&r.size>0&&(o=r.size),r.lineSpacing&&r.lineSpacing>0&&(a=r.lineSpacing),r.styletags&&r.styletags.breaklines&&(s.breaklines=!!r.styletags.breaklines),r.styletags&&r.styletags.bolds&&(s.bolds=!!r.styletags.bolds),r.styletags&&r.styletags.italics&&(s.italics=!!r.styletags.italics),r.styletags&&r.styletags.subscripts&&(s.subscripts=!!r.styletags.subscripts),r.styletags&&r.styletags.superscripts&&(s.superscripts=!!r.styletags.superscripts));return n.font=[r.fontStyle,r.fontVariant,r.fontWeight,o+"px",r.font].filter((function(e){return e})).join(" "),n.textAlign="start",n.textBaseline="alphabetic",n.direction="ltr",h(function(e,t,n,r,o,a){n=n.replace(/\n/g,""),n=!0===a.breaklines?n.replace(/\<br\>/g,"\n"):n.replace(/\<br\>/g," ");var s="",l=[];for(f=0;f<n.length;++f)l[f]=s;!0===a.bolds&&(l=u("b","b|",n,l));!0===a.italics&&(l=u("i","i|",n,l));!0===a.superscripts&&(l=u("sup","+1",n,l));!0===a.subscripts&&(l=u("sub","-1",n,l));var c=[],d="";for(f=0;f<n.length;++f)null!==l[f]&&(d+=n[f],c.push(l[f]));var h,f,p,g,v,m=d.split("\n"),y=m.length,b=Math.round(o*r),C=r,A=2*r,_=0,I=y*b+A;e.height<I&&(e.height=I);t.fillStyle="#000",t.fillRect(0,0,e.width,e.height),t.fillStyle="#fff";var x=0,w="";function M(){if(""!==w){var e=t.measureText(w).width;t.fillText(w,C+p,A+g),p+=e}}function S(){return Math.round(v)+"px "}function T(e,n){var r=""+t.font;if(!0===a.subscripts){var i=e.indexOf("-"),o=n.indexOf("-"),s=i>-1?parseInt(e[1+i]):0,l=o>-1?parseInt(n[1+o]):0;s!==l&&(r=r.replace(S(),"?px "),v*=Math.pow(.75,l-s),r=r.replace("?px ",S())),g+=.25*b*(l-s)}if(!0===a.superscripts){var u=e.indexOf("+"),c=n.indexOf("+"),d=u>-1?parseInt(e[1+u]):0,h=c>-1?parseInt(n[1+c]):0;d!==h&&(r=r.replace(S(),"?px "),v*=Math.pow(.75,h-d),r=r.replace("?px ",S())),g-=.25*b*(h-d)}if(!0===a.bolds){var f=e.indexOf("b|")>-1,p=n.indexOf("b|")>-1;!f&&p&&(r=m?r.replace("italic ","italic bold "):"bold "+r),f&&!p&&(r=r.replace("bold ",""))}if(!0===a.italics){var m=e.indexOf("i|")>-1,y=n.indexOf("i|")>-1;!m&&y&&(r="italic "+r),m&&!y&&(r=r.replace("italic ",""))}t.font=r}for(h=0;h<y;++h){var E=m[h]+"\n";for(p=0,g=h*b,v=r,w="",f=0;f<E.length;++f){var O=f+x<c.length?c[f+x]:c[c.length-1];s===O?w+=E[f]:(M(),w=E[f],void 0!==O&&(T(s,O),s=O))}M(),x+=E.length;var k=0|Math.round(p+2*C);_<k&&(_=k)}var L=_,D=A+b*y;return i(t.getImageData(0,0,L,D).data,[D,L,4]).pick(-1,-1,0).transpose(1,0)}(t,n,e,o,a,s),r,o)},e.exports.processPixels=h;var r=n(1824),i=n(216),o=n(1833),a=n(1837),s=n(1859),l=n(1865);function u(e,t,n,r){for(var i="<"+e+">",o="</"+e+">",a=i.length,s=o.length,l="+"===t[0]||"-"===t[0],u=0,c=-s;u>-1&&-1!==(u=n.indexOf(i,u))&&-1!==(c=n.indexOf(o,u+a))&&!(c<=u);){for(var d=u;d<c+s;++d)if(d<u+a||d>=c)r[d]=null,n=n.substr(0,d)+" "+n.substr(d+1);else if(null!==r[d]){var h=r[d].indexOf(t[0]);-1===h?r[d]+=t:l&&(r[d]=r[d].substr(0,h+1)+(1+parseInt(r[d][h+1]))+r[d].substr(h+2))}var f=u+a,p=n.substr(f,c-f).indexOf(i);u=-1!==p?p:c+s}return r}function c(e,t){var n=r(e,128);return t?o(n.cells,n.positions,.25):{edges:n.cells,positions:n.positions}}function d(e,t,n,r){var i=c(e,r),o=function(e,t,n){for(var r=t.textAlign||"start",i=t.textBaseline||"alphabetic",o=[1<<30,1<<30],a=[0,0],s=e.length,l=0;l<s;++l)for(var u=e[l],c=0;c<2;++c)o[c]=0|Math.min(o[c],u[c]),a[c]=0|Math.max(a[c],u[c]);var d=0;switch(r){case"center":d=-.5*(o[0]+a[0]);break;case"right":case"end":d=-a[0];break;case"left":case"start":d=-o[0];break;default:throw new Error("vectorize-text: Unrecognized textAlign: '"+r+"'")}var h=0;switch(i){case"hanging":case"top":h=-o[1];break;case"middle":h=-.5*(o[1]+a[1]);break;case"alphabetic":case"ideographic":h=-3*n;break;case"bottom":h=-a[1];break;default:throw new Error("vectorize-text: Unrecoginized textBaseline: '"+i+"'")}var f=1/n;return"lineHeight"in t?f*=+t.lineHeight:"width"in t?f=t.width/(a[0]-o[0]):"height"in t&&(f=t.height/(a[1]-o[1])),e.map((function(e){return[f*(e[0]+d),f*(e[1]+h)]}))}(i.positions,t,n),u=i.edges,d="ccw"===t.orientation;if(a(o,u),t.polygons||t.polygon||t.polyline){for(var h=l(u,o),f=new Array(h.length),p=0;p<h.length;++p){for(var g=h[p],v=new Array(g.length),m=0;m<g.length;++m){for(var y=g[m],b=new Array(y.length),C=0;C<y.length;++C)b[C]=o[y[C]].slice();d&&b.reverse(),v[m]=b}f[p]=v}return f}return t.triangles||t.triangulate||t.triangle?{cells:s(o,u,{delaunay:!1,exterior:!1,interior:!0}),positions:o}:{edges:u,positions:o}}function h(e,t,n){try{return d(e,t,n,!0)}catch(e){}try{return d(e,t,n,!1)}catch(e){}return t.polygons||t.polyline||t.polygon?[]:t.triangles||t.triangulate||t.triangle?{cells:[],positions:[]}:{edges:[],positions:[]}}},function(e,t,n){"use strict";e.exports=function(e,t){if(e.dimension<=0)return{positions:[],cells:[]};if(1===e.dimension)return function(e,t){for(var n=o(e,t),r=n.length,i=new Array(r),a=new Array(r),s=0;s<r;++s)i[s]=[n[s]],a[s]=[s];return{positions:i,cells:a}}(e,t);var n=e.order.join()+"-"+e.dtype,s=a[n];t=+t||0;s||(s=a[n]=function(e,t){var n=e.length,o=["'use strict';"],a="surfaceNets"+e.join("_")+"d"+t;o.push("var contour=genContour({","order:[",e.join(),"],","scalarArguments: 3,","phase:function phaseFunc(p,a,b,c) { return (p > c)|0 },"),"generic"===t&&o.push("getters:[0],");for(var s=[],l=[],u=0;u<n;++u)s.push("d"+u),l.push("d"+u);for(u=0;u<1<<n;++u)s.push("v"+u),l.push("v"+u);for(u=0;u<1<<n;++u)s.push("p"+u),l.push("p"+u);s.push("a","b","c"),l.push("a","c"),o.push("vertex:function vertexFunc(",s.join(),"){");var c=[];for(u=0;u<1<<n;++u)c.push("(p"+u+"<<"+u+")");o.push("var m=(",c.join("+"),")|0;if(m===0||m===",(1<<(1<<n))-1,"){return}");var d=[],h=[];1<<(1<<n)<=128?(o.push("switch(m){"),h=o):o.push("switch(m>>>7){");for(u=0;u<1<<(1<<n);++u){if(1<<(1<<n)>128&&u%128==0){d.length>0&&h.push("}}");var f="vExtra"+d.length;o.push("case ",u>>>7,":",f,"(m&0x7f,",l.join(),");break;"),h=["function ",f,"(m,",l.join(),"){switch(m){"],d.push(h)}h.push("case ",127&u,":");for(var p=new Array(n),g=new Array(n),v=new Array(n),m=new Array(n),y=0,b=0;b<n;++b)p[b]=[],g[b]=[],v[b]=0,m[b]=0;for(b=0;b<1<<n;++b)for(var C=0;C<n;++C){var A=b^1<<C;if(!(A>b)&&!(u&1<<A)!=!(u&1<<b)){var _=1;u&1<<A?g[C].push("v"+A+"-v"+b):(g[C].push("v"+b+"-v"+A),_=-_),_<0?(p[C].push("-v"+b+"-v"+A),v[C]+=2):(p[C].push("v"+b+"+v"+A),v[C]-=2),y+=1;for(var I=0;I<n;++I)I!==C&&(A&1<<I?m[I]+=1:m[I]-=1)}}var x=[];for(C=0;C<n;++C)if(0===p[C].length)x.push("d"+C+"-0.5");else{var w="";v[C]<0?w=v[C]+"*c":v[C]>0&&(w="+"+v[C]+"*c");var M=p[C].length/y*.5,S=.5+m[C]/y*.5;x.push("d"+C+"-"+S+"-"+M+"*("+p[C].join("+")+w+")/("+g[C].join("+")+")")}h.push("a.push([",x.join(),"]);","break;")}o.push("}},"),d.length>0&&h.push("}}");var T=[];for(u=0;u<1<<n-1;++u)T.push("v"+u);T.push("c0","c1","p0","p1","a","b","c"),o.push("cell:function cellFunc(",T.join(),"){");var E=i(n-1);o.push("if(p0){b.push(",E.map((function(e){return"["+e.map((function(e){return"v"+e}))+"]"})).join(),")}else{b.push(",E.map((function(e){var t=e.slice();return t.reverse(),"["+t.map((function(e){return"v"+e}))+"]"})).join(),")}}});function ",a,"(array,level){var verts=[],cells=[];contour(array,verts,cells,level);return {positions:verts,cells:cells};} return ",a,";");for(u=0;u<d.length;++u)o.push(d[u].join(""));return new Function("genContour",o.join(""))(r)}(e.order,e.dtype));return s(e,t)};var r=n(1825),i=n(1826),o=n(1831);var a={}},function(e,t,n){"use strict";var r=n(112);function i(e){return"a"+e}function o(e){return"d"+e}function a(e,t){return"c"+e+"_"+t}function s(e){return"s"+e}function l(e,t){return"t"+e+"_"+t}function u(e){return"o"+e}function c(e){return"x"+e}function d(e){return"p"+e}function h(e,t){return"d"+e+"_"+t}function f(e){return"i"+e}function p(e,t){return"u"+e+"_"+t}function g(e){return"b"+e}function v(e){return"y"+e}function m(e){return"e"+e}function y(e){return"v"+e}e.exports=function(e){function t(e){throw new Error("ndarray-extract-contour: "+e)}"object"!=typeof e&&t("Must specify arguments");var n=e.order;Array.isArray(n)||t("Must specify order");var C=e.arrayArguments||1;C<1&&t("Must have at least one array argument");var A=e.scalarArguments||0;A<0&&t("Scalar arg count must be > 0");"function"!=typeof e.vertex&&t("Must specify vertex creation function");"function"!=typeof e.cell&&t("Must specify cell creation function");"function"!=typeof e.phase&&t("Must specify phase function");for(var _=e.getters||[],I=new Array(C),x=0;x<C;++x)_.indexOf(x)>=0?I[x]=!0:I[x]=!1;return function(e,t,n,C,A,_){var I=_.length,x=A.length;if(x<2)throw new Error("ndarray-extract-contour: Dimension must be at least 2");for(var w="extractContour"+A.join("_"),M=[],S=[],T=[],E=0;E<I;++E)T.push(i(E));for(E=0;E<C;++E)T.push(c(E));for(E=0;E<x;++E)S.push(s(E)+"="+i(0)+".shape["+E+"]|0");for(E=0;E<I;++E){S.push(o(E)+"="+i(E)+".data",u(E)+"="+i(E)+".offset|0");for(var O=0;O<x;++O)S.push(l(E,O)+"="+i(E)+".stride["+O+"]|0")}for(E=0;E<I;++E){S.push(d(E)+"="+u(E)),S.push(a(E,0));for(O=1;O<1<<x;++O){for(var k=[],L=0;L<x;++L)O&1<<L&&k.push("-"+l(E,L));S.push(h(E,O)+"=("+k.join("")+")|0"),S.push(a(E,O)+"=0")}}for(E=0;E<I;++E)for(O=0;O<x;++O){var D=[l(E,A[O])];O>0&&D.push(l(E,A[O-1])+"*"+s(A[O-1])),S.push(p(E,A[O])+"=("+D.join("-")+")|0")}for(E=0;E<x;++E)S.push(f(E)+"=0");S.push("N=0");var P=["2"];for(E=x-2;E>=0;--E)P.push(s(A[E]));S.push("Q=("+P.join("*")+")|0","P=mallocUint32(Q)","V=mallocUint32(Q)","X=0"),S.push(g(0)+"=0");for(O=1;O<1<<x;++O){var N=[],j=[];for(L=0;L<x;++L)O&1<<L&&(0===j.length?N.push("1"):N.unshift(j.join("*"))),j.push(s(A[L]));var R="";N[0].indexOf(s(A[x-2]))<0&&(R="-");var z=b(x,O,A);S.push(m(z)+"=(-"+N.join("-")+")|0",v(z)+"=("+R+N.join("-")+")|0",g(z)+"=0")}function F(e,t){M.push("for(",f(A[e]),"=",t,";",f(A[e]),"<",s(A[e]),";","++",f(A[e]),"){")}function B(e){for(var t=0;t<I;++t)M.push(d(t),"+=",p(t,A[e]),";");M.push("}")}function H(){for(var e=1;e<1<<x;++e)M.push("T","=",m(e),";",m(e),"=",v(e),";",v(e),"=","T",";")}S.push(y(0)+"=0","T=0"),function e(t,n){if(t<0)!function(e){for(var t=0;t<I;++t)_[t]?M.push(a(t,0),"=",o(t),".get(",d(t),");"):M.push(a(t,0),"=",o(t),"[",d(t),"];");var n=[];for(t=0;t<I;++t)n.push(a(t,0));for(t=0;t<C;++t)n.push(c(t));M.push(g(0),"=","P","[","X","]=phase(",n.join(),");");for(var r=1;r<1<<x;++r)M.push(g(r),"=","P","[","X","+",m(r),"];");var i=[];for(r=1;r<1<<x;++r)i.push("("+g(0)+"!=="+g(r)+")");M.push("if(",i.join("||"),"){");var s=[];for(t=0;t<x;++t)s.push(f(t));for(t=0;t<I;++t){s.push(a(t,0));for(r=1;r<1<<x;++r)_[t]?M.push(a(t,r),"=",o(t),".get(",d(t),"+",h(t,r),");"):M.push(a(t,r),"=",o(t),"[",d(t),"+",h(t,r),"];"),s.push(a(t,r))}for(t=0;t<1<<x;++t)s.push(g(t));for(t=0;t<C;++t)s.push(c(t));M.push("vertex(",s.join(),");",y(0),"=","V","[","X","]=","N","++;");var l=(1<<x)-1,u=g(l);for(r=0;r<x;++r)if(0==(e&~(1<<r))){for(var p=l^1<<r,v=g(p),b=[],A=p;A>0;A=A-1&p)b.push("V[X+"+m(A)+"]");b.push(y(0));for(A=0;A<I;++A)1&r?b.push(a(A,l),a(A,p)):b.push(a(A,p),a(A,l));1&r?b.push(u,v):b.push(v,u);for(A=0;A<C;++A)b.push(c(A));M.push("if(",u,"!==",v,"){","face(",b.join(),")}")}M.push("}","X","+=1;")}(n);else{!function(e){for(var t=e-1;t>=0;--t)F(t,0);var n=[];for(t=0;t<I;++t)_[t]?n.push(o(t)+".get("+d(t)+")"):n.push(o(t)+"["+d(t)+"]");for(t=0;t<C;++t)n.push(c(t));for(M.push("P","[","X","++]=phase(",n.join(),");"),t=0;t<e;++t)B(t);for(var r=0;r<I;++r)M.push(d(r),"+=",p(r,A[e]),";")}(t),M.push("if(",s(A[t]),">0){",f(A[t]),"=1;"),e(t-1,n|1<<A[t]);for(var r=0;r<I;++r)M.push(d(r),"+=",p(r,A[t]),";");t===x-1&&(M.push("X","=0;"),H()),F(t,2),e(t-1,n),t===x-1&&(M.push("if(",f(A[x-1]),"&1){","X","=0;}"),H()),B(t),M.push("}")}}(x-1,0),M.push("freeUint32(","V",");freeUint32(","P",");");var U=["'use strict';","function ",w,"(",T.join(),"){","var ",S.join(),";",M.join(""),"}","return ",w].join("");return new Function("vertex","face","phase","mallocUint32","freeUint32",U)(e,t,n,r.mallocUint32,r.freeUint32)}(e.vertex,e.cell,e.phase,A,n,I)};function b(e,t,n){for(var r=0,i=0;i<e;++i)t&1<<i&&(r|=1<<n[i]);return r}},function(e,t,n){"use strict";e.exports=function(e){if(e<0)return[];if(0===e)return[[0]];for(var t=0|Math.round(o(e+1)),n=[],a=0;a<t;++a){for(var s=r.unrank(e,a),l=[0],u=0,c=0;c<s.length;++c)u+=1<<s[c],l.push(u);i(s)<1&&(l[0]=u,l[e]=0),n.push(l)}return n};var r=n(1827),i=n(1829),o=n(1830)},function(e,t,n){"use strict";var r=n(112),i=n(1828);t.rank=function(e){var t=e.length;switch(t){case 0:case 1:return 0;case 2:return e[1]}var n,o,a,s=r.mallocUint32(t),l=r.mallocUint32(t),u=0;for(i(e,l),a=0;a<t;++a)s[a]=e[a];for(a=t-1;a>0;--a)o=l[a],n=s[a],s[a]=s[o],s[o]=n,l[a]=l[n],l[n]=o,u=(u+n)*a;return r.freeUint32(l),r.freeUint32(s),u},t.unrank=function(e,t,n){switch(e){case 0:return n||[];case 1:return n?(n[0]=0,n):[0];case 2:return n?(t?(n[0]=0,n[1]=1):(n[0]=1,n[1]=0),n):t?[0,1]:[1,0]}var r,i,o,a=1;for((n=n||new Array(e))[0]=0,o=1;o<e;++o)n[o]=o,a=a*o|0;for(o=e-1;o>0;--o)t=t-(r=t/a|0)*a|0,a=a/o|0,i=0|n[o],n[o]=0|n[r],n[r]=0|i;return n}},function(e,t,n){"use strict";e.exports=function(e,t){t=t||new Array(e.length);for(var n=0;n<e.length;++n)t[e[n]]=n;return t}},function(e,t,n){"use strict";e.exports=function(e){var t=e.length;if(t<32){for(var n=1,i=0;i<t;++i)for(var o=0;o<i;++o)if(e[i]<e[o])n=-n;else if(e[i]===e[o])return 0;return n}var a=r.mallocUint8(t);for(i=0;i<t;++i)a[i]=0;for(n=1,i=0;i<t;++i)if(!a[i]){var s=1;a[i]=1;for(o=e[i];o!==i;o=e[o]){if(a[o])return r.freeUint8(a),0;s+=1,a[o]=1}1&s||(n=-n)}return r.freeUint8(a),n};var r=n(112)},function(e,t){var n=[.9999999999998099,676.5203681218851,-1259.1392167224028,771.3234287776531,-176.6150291621406,12.507343278686905,-.13857109526572012,9984369578019572e-21,1.5056327351493116e-7],r=[.9999999999999971,57.15623566586292,-59.59796035547549,14.136097974741746,-.4919138160976202,3399464998481189e-20,4652362892704858e-20,-9837447530487956e-20,.0001580887032249125,-.00021026444172410488,.00021743961811521265,-.0001643181065367639,8441822398385275e-20,-26190838401581408e-21,36899182659531625e-22];function i(e){if(e<0)return Number("0/0");for(var t=r[0],n=r.length-1;n>0;--n)t+=r[n]/(e+n);var i=e+607/128+.5;return.5*Math.log(2*Math.PI)+(e+.5)*Math.log(i)-i+Math.log(t)-Math.log(e)}e.exports=function e(t){if(t<.5)return Math.PI/(Math.sin(Math.PI*t)*e(1-t));if(t>100)return Math.exp(i(t));t-=1;for(var r=n[0],o=1;o<9;o++)r+=n[o]/(t+o);var a=t+7+.5;return Math.sqrt(2*Math.PI)*Math.pow(a,t+.5)*Math.exp(-a)*r},e.exports.log=i},function(e,t,n){"use strict";e.exports=function(e,t){var n=[];return t=+t||0,r(e.hi(e.shape[0]-1),n,t),n};var r=n(1832)},function(e,t,n){e.exports=n(1036)({args:["array",{offset:[1],array:0},"scalar","scalar","index"],pre:{body:"{}",args:[],thisVars:[],localVars:[]},post:{body:"{}",args:[],thisVars:[],localVars:[]},body:{body:"{\n        var _inline_1_da = _inline_1_arg0_ - _inline_1_arg3_\n        var _inline_1_db = _inline_1_arg1_ - _inline_1_arg3_\n        if((_inline_1_da >= 0) !== (_inline_1_db >= 0)) {\n          _inline_1_arg2_.push(_inline_1_arg4_[0] + 0.5 + 0.5 * (_inline_1_da + _inline_1_db) / (_inline_1_da - _inline_1_db))\n        }\n      }",args:[{name:"_inline_1_arg0_",lvalue:!1,rvalue:!0,count:1},{name:"_inline_1_arg1_",lvalue:!1,rvalue:!0,count:1},{name:"_inline_1_arg2_",lvalue:!1,rvalue:!0,count:1},{name:"_inline_1_arg3_",lvalue:!1,rvalue:!0,count:2},{name:"_inline_1_arg4_",lvalue:!1,rvalue:!0,count:1}],thisVars:[],localVars:["_inline_1_da","_inline_1_db"]},funcName:"zeroCrossings"})},function(e,t,n){"use strict";e.exports=function(e,t,n){for(var o=t.length,a=e.length,s=new Array(o),l=new Array(o),u=new Array(o),c=new Array(o),d=0;d<o;++d)s[d]=l[d]=-1,u[d]=1/0,c[d]=!1;for(d=0;d<a;++d){var h=e[d];if(2!==h.length)throw new Error("Input must be a graph");var f=h[1],p=h[0];-1!==l[p]?l[p]=-2:l[p]=f,-1!==s[f]?s[f]=-2:s[f]=p}function g(e){if(c[e])return 1/0;var n,i,o,a,u,d=s[e],h=l[e];return d<0||h<0?1/0:(n=t[e],i=t[d],o=t[h],a=Math.abs(r(n,i,o)),u=Math.sqrt(Math.pow(i[0]-o[0],2)+Math.pow(i[1]-o[1],2)),a/u)}function v(e,t){var n=x[e],r=x[t];x[e]=r,x[t]=n,w[n]=t,w[r]=e}function m(e){return u[x[e]]}function y(e){return 1&e?e-1>>1:(e>>1)-1}function b(e){for(var t=m(e);;){var n=t,r=2*e+1,i=2*(e+1),o=e;if(r<M){var a=m(r);a<n&&(o=r,n=a)}if(i<M)m(i)<n&&(o=i);if(o===e)return e;v(e,o),e=o}}function C(e){for(var t=m(e);e>0;){var n=y(e);if(n>=0)if(t<m(n)){v(e,n),e=n;continue}return e}}function A(){if(M>0){var e=x[0];return v(0,M-1),M-=1,b(0),e}return-1}function _(e,t){var n=x[e];return u[n]===t?e:(u[n]=-1/0,C(e),A(),u[n]=t,C((M+=1)-1))}function I(e){if(!c[e]){c[e]=!0;var t=s[e],n=l[e];s[n]>=0&&(s[n]=t),l[t]>=0&&(l[t]=n),w[t]>=0&&_(w[t],g(t)),w[n]>=0&&_(w[n],g(n))}}var x=[],w=new Array(o);for(d=0;d<o;++d){(u[d]=g(d))<1/0?(w[d]=x.length,x.push(d)):w[d]=-1}var M=x.length;for(d=M>>1;d>=0;--d)b(d);for(;;){var S=A();if(S<0||u[S]>n)break;I(S)}var T=[];for(d=0;d<o;++d)c[d]||(w[d]=T.length,T.push(t[d].slice()));T.length;function E(e,t){if(e[t]<0)return t;var n=t,r=t;do{var i=e[r];if(!c[r]||i<0||i===r)break;if(i=e[r=i],!c[r]||i<0||i===r)break;r=i,n=e[n]}while(n!==r);for(var o=t;o!==r;o=e[o])e[o]=r;return r}var O=[];return e.forEach((function(e){var t=E(s,e[0]),n=E(l,e[1]);if(t>=0&&n>=0&&t!==n){var r=w[t],i=w[n];r!==i&&O.push([r,i])}})),i.unique(i.normalize(O)),{positions:T,edges:O}};var r=n(150),i=n(1834)},function(e,t,n){"use strict";var r=n(1835),i=n(1836);function o(e,t){var n=e.length,r=e.length-t.length,i=Math.min;if(r)return r;switch(n){case 0:return 0;case 1:return e[0]-t[0];case 2:return(s=e[0]+e[1]-t[0]-t[1])||i(e[0],e[1])-i(t[0],t[1]);case 3:var o=e[0]+e[1],a=t[0]+t[1];if(s=o+e[2]-(a+t[2]))return s;var s,l=i(e[0],e[1]),u=i(t[0],t[1]);return(s=i(l,e[2])-i(u,t[2]))||i(l+e[2],o)-i(u+t[2],a);default:var c=e.slice(0);c.sort();var d=t.slice(0);d.sort();for(var h=0;h<n;++h)if(r=c[h]-d[h])return r;return 0}}function a(e,t){return o(e[0],t[0])}function s(e,t){if(t){for(var n=e.length,r=new Array(n),i=0;i<n;++i)r[i]=[e[i],t[i]];r.sort(a);for(i=0;i<n;++i)e[i]=r[i][0],t[i]=r[i][1];return e}return e.sort(o),e}function l(e){if(0===e.length)return[];for(var t=1,n=e.length,r=1;r<n;++r){var i=e[r];if(o(i,e[r-1])){if(r===t){t++;continue}e[t++]=i}}return e.length=t,e}function u(e,t){for(var n=0,r=e.length-1,i=-1;n<=r;){var a=n+r>>1,s=o(e[a],t);s<=0?(0===s&&(i=a),n=a+1):s>0&&(r=a-1)}return i}function c(e,t){for(var n=new Array(e.length),i=0,a=n.length;i<a;++i)n[i]=[];for(var s=[],l=(i=0,t.length);i<l;++i)for(var c=t[i],d=c.length,h=1,f=1<<d;h<f;++h){s.length=r.popCount(h);for(var p=0,g=0;g<d;++g)h&1<<g&&(s[p++]=c[g]);var v=u(e,s);if(!(v<0))for(;n[v++].push(i),!(v>=e.length||0!==o(e[v],s)););}return n}function d(e,t){if(t<0)return[];for(var n=[],i=(1<<t+1)-1,o=0;o<e.length;++o)for(var a=e[o],l=i;l<1<<a.length;l=r.nextCombination(l)){for(var u=new Array(t+1),c=0,d=0;d<a.length;++d)l&1<<d&&(u[c++]=a[d]);n.push(u)}return s(n)}t.dimension=function(e){for(var t=0,n=Math.max,r=0,i=e.length;r<i;++r)t=n(t,e[r].length);return t-1},t.countVertices=function(e){for(var t=-1,n=Math.max,r=0,i=e.length;r<i;++r)for(var o=e[r],a=0,s=o.length;a<s;++a)t=n(t,o[a]);return t+1},t.cloneCells=function(e){for(var t=new Array(e.length),n=0,r=e.length;n<r;++n)t[n]=e[n].slice(0);return t},t.compareCells=o,t.normalize=s,t.unique=l,t.findCell=u,t.incidence=c,t.dual=function(e,t){if(!t)return c(l(d(e,0)),e);for(var n=new Array(t),r=0;r<t;++r)n[r]=[];r=0;for(var i=e.length;r<i;++r)for(var o=e[r],a=0,s=o.length;a<s;++a)n[o[a]].push(r);return n},t.explode=function(e){for(var t=[],n=0,r=e.length;n<r;++n)for(var i=e[n],o=0|i.length,a=1,l=1<<o;a<l;++a){for(var u=[],c=0;c<o;++c)a>>>c&1&&u.push(i[c]);t.push(u)}return s(t)},t.skeleton=d,t.boundary=function(e){for(var t=[],n=0,r=e.length;n<r;++n)for(var i=e[n],o=0,a=i.length;o<a;++o){for(var l=new Array(i.length-1),u=0,c=0;u<a;++u)u!==o&&(l[c++]=i[u]);t.push(l)}return s(t)},t.connectedComponents=function(e,t){return t?function(e,t){for(var n=new i(t),r=0;r<e.length;++r)for(var o=e[r],a=0;a<o.length;++a)for(var s=a+1;s<o.length;++s)n.link(o[a],o[s]);var l=[],u=n.ranks;for(r=0;r<u.length;++r)u[r]=-1;for(r=0;r<e.length;++r){var c=n.find(e[r][0]);u[c]<0?(u[c]=l.length,l.push([e[r].slice(0)])):l[u[c]].push(e[r].slice(0))}return l}(e,t):function(e){for(var t=l(s(d(e,0))),n=new i(t.length),r=0;r<e.length;++r)for(var o=e[r],a=0;a<o.length;++a)for(var c=u(t,[o[a]]),h=a+1;h<o.length;++h)n.link(c,u(t,[o[h]]));var f=[],p=n.ranks;for(r=0;r<p.length;++r)p[r]=-1;for(r=0;r<e.length;++r){var g=n.find(u(t,[e[r][0]]));p[g]<0?(p[g]=f.length,f.push([e[r].slice(0)])):f[p[g]].push(e[r].slice(0))}return f}(e)}},function(e,t,n){"use strict";function r(e){var t=32;return(e&=-e)&&t--,65535&e&&(t-=16),16711935&e&&(t-=8),252645135&e&&(t-=4),858993459&e&&(t-=2),1431655765&e&&(t-=1),t}t.INT_BITS=32,t.INT_MAX=2147483647,t.INT_MIN=-1<<31,t.sign=function(e){return(e>0)-(e<0)},t.abs=function(e){var t=e>>31;return(e^t)-t},t.min=function(e,t){return t^(e^t)&-(e<t)},t.max=function(e,t){return e^(e^t)&-(e<t)},t.isPow2=function(e){return!(e&e-1||!e)},t.log2=function(e){var t,n;return t=(e>65535)<<4,t|=n=((e>>>=t)>255)<<3,t|=n=((e>>>=n)>15)<<2,(t|=n=((e>>>=n)>3)<<1)|(e>>>=n)>>1},t.log10=function(e){return e>=1e9?9:e>=1e8?8:e>=1e7?7:e>=1e6?6:e>=1e5?5:e>=1e4?4:e>=1e3?3:e>=100?2:e>=10?1:0},t.popCount=function(e){return 16843009*((e=(858993459&(e-=e>>>1&1431655765))+(e>>>2&858993459))+(e>>>4)&252645135)>>>24},t.countTrailingZeros=r,t.nextPow2=function(e){return e+=0===e,--e,e|=e>>>1,e|=e>>>2,e|=e>>>4,e|=e>>>8,(e|=e>>>16)+1},t.prevPow2=function(e){return e|=e>>>1,e|=e>>>2,e|=e>>>4,e|=e>>>8,(e|=e>>>16)-(e>>>1)},t.parity=function(e){return e^=e>>>16,e^=e>>>8,e^=e>>>4,27030>>>(e&=15)&1};var i=new Array(256);!function(e){for(var t=0;t<256;++t){var n=t,r=t,i=7;for(n>>>=1;n;n>>>=1)r<<=1,r|=1&n,--i;e[t]=r<<i&255}}(i),t.reverse=function(e){return i[255&e]<<24|i[e>>>8&255]<<16|i[e>>>16&255]<<8|i[e>>>24&255]},t.interleave2=function(e,t){return(e=1431655765&((e=858993459&((e=252645135&((e=16711935&((e&=65535)|e<<8))|e<<4))|e<<2))|e<<1))|(t=1431655765&((t=858993459&((t=252645135&((t=16711935&((t&=65535)|t<<8))|t<<4))|t<<2))|t<<1))<<1},t.deinterleave2=function(e,t){return(e=65535&((e=16711935&((e=252645135&((e=858993459&((e=e>>>t&1431655765)|e>>>1))|e>>>2))|e>>>4))|e>>>16))<<16>>16},t.interleave3=function(e,t,n){return e=1227133513&((e=3272356035&((e=251719695&((e=4278190335&((e&=1023)|e<<16))|e<<8))|e<<4))|e<<2),(e|=(t=1227133513&((t=3272356035&((t=251719695&((t=4278190335&((t&=1023)|t<<16))|t<<8))|t<<4))|t<<2))<<1)|(n=1227133513&((n=3272356035&((n=251719695&((n=4278190335&((n&=1023)|n<<16))|n<<8))|n<<4))|n<<2))<<2},t.deinterleave3=function(e,t){return(e=1023&((e=4278190335&((e=251719695&((e=3272356035&((e=e>>>t&1227133513)|e>>>2))|e>>>4))|e>>>8))|e>>>16))<<22>>22},t.nextCombination=function(e){var t=e|e-1;return t+1|(~t&-~t)-1>>>r(e)+1}},function(e,t,n){"use strict";function r(e){this.roots=new Array(e),this.ranks=new Array(e);for(var t=0;t<e;++t)this.roots[t]=t,this.ranks[t]=0}e.exports=r,r.prototype.length=function(){return this.roots.length},r.prototype.makeSet=function(){var e=this.roots.length;return this.roots.push(e),this.ranks.push(0),e},r.prototype.find=function(e){for(var t=this.roots;t[e]!==e;){var n=t[e];t[e]=t[n],e=n}return e},r.prototype.link=function(e,t){var n=this.find(e),r=this.find(t);if(n!==r){var i=this.ranks,o=this.roots,a=i[n],s=i[r];a<s?o[n]=r:s<a?o[r]=n:(o[r]=n,++i[n])}}},function(e,t,n){"use strict";e.exports=function(e,t,n){var r;if(n){r=t;for(var i=new Array(t.length),o=0;o<t.length;++o){var a=t[o];i[o]=[a[0],a[1],n[o]]}t=i}var s=function(e,t,n){var r=p(e,[],f(e));return m(t,r,n),!!r}(e,t,!!n);for(;y(e,t,!!n);)s=!0;if(n&&s){r.length=0,n.length=0;for(o=0;o<t.length;++o){a=t[o];r.push([a[0],a[1]]),n.push(a[2])}}return s};var r=n(1044),i=n(1838),o=n(1843),a=n(570),s=n(1847),l=n(1848),u=n(1851),c=n(1852),d=n(1853);function h(e){var t=l(e);return[c(t,-1/0),c(t,1/0)]}function f(e){for(var t=new Array(e.length),n=0;n<e.length;++n){var r=e[n];t[n]=[c(r[0],-1/0),c(r[1],-1/0),c(r[0],1/0),c(r[1],1/0)]}return t}function p(e,t,n){for(var o=t.length,a=new r(o),s=[],l=0;l<t.length;++l){var u=t[l],d=h(u[0]),f=h(u[1]);s.push([c(d[0],-1/0),c(f[0],-1/0),c(d[1],1/0),c(f[1],1/0)])}i(s,(function(e,t){a.link(e,t)}));var p=!0,g=new Array(o);for(l=0;l<o;++l){(m=a.find(l))!==l&&(p=!1,e[m]=[Math.min(e[l][0],e[m][0]),Math.min(e[l][1],e[m][1])])}if(p)return null;var v=0;for(l=0;l<o;++l){var m;(m=a.find(l))===l?(g[l]=v,e[v++]=e[l]):g[l]=-1}e.length=v;for(l=0;l<o;++l)g[l]<0&&(g[l]=g[a.find(l)]);return g}function g(e,t){return e[0]-t[0]||e[1]-t[1]}function v(e,t){var n=e[0]-t[0]||e[1]-t[1];return n||(e[2]<t[2]?-1:e[2]>t[2]?1:0)}function m(e,t,n){if(0!==e.length){if(t)for(var r=0;r<e.length;++r){var i=t[(a=e[r])[0]],o=t[a[1]];a[0]=Math.min(i,o),a[1]=Math.max(i,o)}else for(r=0;r<e.length;++r){var a;i=(a=e[r])[0],o=a[1];a[0]=Math.min(i,o),a[1]=Math.max(i,o)}n?e.sort(v):e.sort(g);var s=1;for(r=1;r<e.length;++r){var l=e[r-1],u=e[r];(u[0]!==l[0]||u[1]!==l[1]||n&&u[2]!==l[2])&&(e[s++]=u)}e.length=s}}function y(e,t,n){var r=function(e,t){for(var n=new Array(t.length),r=0;r<t.length;++r){var i=t[r],o=e[i[0]],a=e[i[1]];n[r]=[c(Math.min(o[0],a[0]),-1/0),c(Math.min(o[1],a[1]),-1/0),c(Math.max(o[0],a[0]),1/0),c(Math.max(o[1],a[1]),1/0)]}return n}(e,t),h=function(e,t,n){var r=[];return i(n,(function(n,i){var a=t[n],s=t[i];if(a[0]!==s[0]&&a[0]!==s[1]&&a[1]!==s[0]&&a[1]!==s[1]){var l=e[a[0]],u=e[a[1]],c=e[s[0]],d=e[s[1]];o(l,u,c,d)&&r.push([n,i])}})),r}(e,t,r),g=f(e),v=function(e,t,n,r){var a=[];return i(n,r,(function(n,r){var i=t[n];if(i[0]!==r&&i[1]!==r){var s=e[r],l=e[i[0]],u=e[i[1]];o(l,u,s,s)&&a.push([n,r])}})),a}(e,t,r,g),y=p(e,function(e,t,n,r,i){var o,c,h=e.map((function(e){return[a(e[0]),a(e[1])]}));for(o=0;o<n.length;++o){var f=n[o];c=f[0];var p=f[1],g=t[c],v=t[p],m=d(u(e[g[0]]),u(e[g[1]]),u(e[v[0]]),u(e[v[1]]));if(m){var y=e.length;e.push([l(m[0]),l(m[1])]),h.push(m),r.push([c,y],[p,y])}}for(r.sort((function(e,t){if(e[0]!==t[0])return e[0]-t[0];var n=h[e[1]],r=h[t[1]];return s(n[0],r[0])||s(n[1],r[1])})),o=r.length-1;o>=0;--o){var b=t[c=(S=r[o])[0]],C=b[0],A=b[1],_=e[C],I=e[A];if((_[0]-I[0]||_[1]-I[1])<0){var x=C;C=A,A=x}b[0]=C;var w,M=b[1]=S[1];for(i&&(w=b[2]);o>0&&r[o-1][0]===c;){var S,T=(S=r[--o])[1];i?t.push([M,T,w]):t.push([M,T]),M=T}i?t.push([M,A,w]):t.push([M,A])}return h}(e,t,h,v,n));return m(t,y,n),!!y||(h.length>0||v.length>0)}},function(e,t,n){"use strict";e.exports=function(e,t,n){switch(arguments.length){case 1:return d(e);case 2:return"function"==typeof t?u(e,e,t,!0):h(e,t);case 3:return u(e,t,n,!1);default:throw new Error("box-intersect: Invalid arguments")}};var r,i=n(112),o=n(1045),a=n(1840);function s(e,t){for(var n=0;n<e;++n)if(!(t[n]<=t[n+e]))return!0;return!1}function l(e,t,n,r){for(var i=0,o=0,a=0,l=e.length;a<l;++a){var u=e[a];if(!s(t,u)){for(var c=0;c<2*t;++c)n[i++]=u[c];r[o++]=a}}return o}function u(e,t,n,r){var s=e.length,u=t.length;if(!(s<=0||u<=0)){var c=e[0].length>>>1;if(!(c<=0)){var d,h=i.mallocDouble(2*c*s),f=i.mallocInt32(s);if((s=l(e,c,h,f))>0){if(1===c&&r)o.init(s),d=o.sweepComplete(c,n,0,s,h,f,0,s,h,f);else{var p=i.mallocDouble(2*c*u),g=i.mallocInt32(u);(u=l(t,c,p,g))>0&&(o.init(s+u),d=1===c?o.sweepBipartite(c,n,0,s,h,f,0,u,p,g):a(c,n,r,s,h,f,u,p,g),i.free(p),i.free(g))}i.free(h),i.free(f)}return d}}}function c(e,t){r.push([e,t])}function d(e){return r=[],u(e,e,c,!0),r}function h(e,t){return r=[],u(e,t,c,!1),r}},function(e,t,n){"use strict";e.exports=function(e,t){t<=128?r(0,t-1,e):function e(t,n,c){var d=(n-t+1)/6|0,h=t+d,f=n-d,p=t+n>>1,g=p-d,v=p+d,m=h,y=g,b=p,C=v,A=f,_=t+1,I=n-1,x=0;l(m,y,c)&&(x=m,m=y,y=x);l(C,A,c)&&(x=C,C=A,A=x);l(m,b,c)&&(x=m,m=b,b=x);l(y,b,c)&&(x=y,y=b,b=x);l(m,C,c)&&(x=m,m=C,C=x);l(b,C,c)&&(x=b,b=C,C=x);l(y,A,c)&&(x=y,y=A,A=x);l(y,b,c)&&(x=y,y=b,b=x);l(C,A,c)&&(x=C,C=A,A=x);for(var w=c[2*y],M=c[2*y+1],S=c[2*C],T=c[2*C+1],E=2*m,O=2*b,k=2*A,L=2*h,D=2*p,P=2*f,N=0;N<2;++N){var j=c[E+N],R=c[O+N],z=c[k+N];c[L+N]=j,c[D+N]=R,c[P+N]=z}o(g,t,c),o(v,n,c);for(var F=_;F<=I;++F)if(u(F,w,M,c))F!==_&&i(F,_,c),++_;else if(!u(F,S,T,c))for(;;){if(u(I,S,T,c)){u(I,w,M,c)?(a(F,_,I,c),++_,--I):(i(F,I,c),--I);break}if(--I<F)break}s(t,_-1,w,M,c),s(n,I+1,S,T,c),_-2-t<=32?r(t,_-2,c):e(t,_-2,c);n-(I+2)<=32?r(I+2,n,c):e(I+2,n,c);I-_<=32?r(_,I,c):e(_,I,c)}(0,t-1,e)};function r(e,t,n){for(var r=2*(e+1),i=e+1;i<=t;++i){for(var o=n[r++],a=n[r++],s=i,l=r-2;s-- >e;){var u=n[l-2],c=n[l-1];if(u<o)break;if(u===o&&c<a)break;n[l]=u,n[l+1]=c,l-=2}n[l]=o,n[l+1]=a}}function i(e,t,n){t*=2;var r=n[e*=2],i=n[e+1];n[e]=n[t],n[e+1]=n[t+1],n[t]=r,n[t+1]=i}function o(e,t,n){t*=2,n[e*=2]=n[t],n[e+1]=n[t+1]}function a(e,t,n,r){t*=2,n*=2;var i=r[e*=2],o=r[e+1];r[e]=r[t],r[e+1]=r[t+1],r[t]=r[n],r[t+1]=r[n+1],r[n]=i,r[n+1]=o}function s(e,t,n,r,i){t*=2,i[e*=2]=i[t],i[t]=n,i[e+1]=i[t+1],i[t+1]=r}function l(e,t,n){t*=2;var r=n[e*=2],i=n[t];return!(r<i)&&(r!==i||n[e+1]>n[t+1])}function u(e,t,n,r){var i=r[e*=2];return i<t||i===t&&r[e+1]<n}},function(e,t,n){"use strict";e.exports=function(e,t,n,o,c,_,I,x,w){!function(e,t){var n=8*i.log2(t+1)*(e+1)|0,o=i.nextPow2(6*n);m.length<o&&(r.free(m),m=r.mallocInt32(o));var a=i.nextPow2(2*n);y.length<a&&(r.free(y),y=r.mallocDouble(a))}(e,o+I);var M,S=0,T=2*e;b(S++,0,0,o,0,I,n?16:0,-1/0,1/0),n||b(S++,0,0,I,0,o,1,-1/0,1/0);for(;S>0;){var E=6*(S-=1),O=m[E],k=m[E+1],L=m[E+2],D=m[E+3],P=m[E+4],N=m[E+5],j=2*S,R=y[j],z=y[j+1],F=1&N,B=!!(16&N),H=c,U=_,V=x,Y=w;if(F&&(H=x,U=w,V=c,Y=_),!(2&N&&(L=f(e,O,k,L,H,U,z),k>=L)||4&N&&(k=p(e,O,k,L,H,U,R))>=L)){var W=L-k,G=P-D;if(B){if(e*W*(W+G)<1<<22){if(void 0!==(M=l.scanComplete(e,O,t,k,L,H,U,D,P,V,Y)))return M;continue}}else{if(e*Math.min(W,G)<128){if(void 0!==(M=a(e,O,t,F,k,L,H,U,D,P,V,Y)))return M;continue}if(e*W*G<1<<22){if(void 0!==(M=l.scanBipartite(e,O,t,F,k,L,H,U,D,P,V,Y)))return M;continue}}var X=d(e,O,k,L,H,U,R,z);if(k<X)if(e*(X-k)<128){if(void 0!==(M=s(e,O+1,t,k,X,H,U,D,P,V,Y)))return M}else if(O===e-2){if(void 0!==(M=F?l.sweepBipartite(e,t,D,P,V,Y,k,X,H,U):l.sweepBipartite(e,t,k,X,H,U,D,P,V,Y)))return M}else b(S++,O+1,k,X,D,P,F,-1/0,1/0),b(S++,O+1,D,P,k,X,1^F,-1/0,1/0);if(X<L){var q=u(e,O,D,P,V,Y),Z=V[T*q+O],Q=h(e,O,q,P,V,Y,Z);if(Q<P&&b(S++,O,X,L,Q,P,(4|F)+(B?16:0),Z,z),D<q&&b(S++,O,X,L,D,q,(2|F)+(B?16:0),R,Z),q+1===Q){if(void 0!==(M=B?A(e,O,t,X,L,H,U,q,V,Y[q]):C(e,O,t,F,X,L,H,U,q,V,Y[q])))return M}else if(q<Q){var K;if(B){if(K=g(e,O,X,L,H,U,Z),X<K){var J=h(e,O,X,K,H,U,Z);if(O===e-2){if(X<J&&void 0!==(M=l.sweepComplete(e,t,X,J,H,U,q,Q,V,Y)))return M;if(J<K&&void 0!==(M=l.sweepBipartite(e,t,J,K,H,U,q,Q,V,Y)))return M}else X<J&&b(S++,O+1,X,J,q,Q,16,-1/0,1/0),J<K&&(b(S++,O+1,J,K,q,Q,0,-1/0,1/0),b(S++,O+1,q,Q,J,K,1,-1/0,1/0))}}else K=F?v(e,O,X,L,H,U,Z):g(e,O,X,L,H,U,Z),X<K&&(O===e-2?M=F?l.sweepBipartite(e,t,q,Q,V,Y,X,K,H,U):l.sweepBipartite(e,t,X,K,H,U,q,Q,V,Y):(b(S++,O+1,X,K,q,Q,F,-1/0,1/0),b(S++,O+1,q,Q,X,K,1^F,-1/0,1/0)))}}}}};var r=n(112),i=n(215),o=n(1841),a=o.partial,s=o.full,l=n(1045),u=n(1842),c=n(1046),d=c("!(lo>=p0)&&!(p1>=hi)",["p0","p1"]),h=c("lo===p0",["p0"]),f=c("lo<p0",["p0"]),p=c("hi<=p0",["p0"]),g=c("lo<=p0&&p0<=hi",["p0"]),v=c("lo<p0&&p0<=hi",["p0"]),m=r.mallocInt32(1024),y=r.mallocDouble(1024);function b(e,t,n,r,i,o,a,s,l){var u=6*e;m[u]=t,m[u+1]=n,m[u+2]=r,m[u+3]=i,m[u+4]=o,m[u+5]=a;var c=2*e;y[c]=s,y[c+1]=l}function C(e,t,n,r,i,o,a,s,l,u,c){var d=2*e,h=l*d,f=u[h+t];e:for(var p=i,g=i*d;p<o;++p,g+=d){var v=a[g+t],m=a[g+t+e];if(!(f<v||m<f)&&(!r||f!==v)){for(var y,b=s[p],C=t+1;C<e;++C){v=a[g+C],m=a[g+C+e];var A=u[h+C],_=u[h+C+e];if(m<A||_<v)continue e}if(void 0!==(y=r?n(c,b):n(b,c)))return y}}}function A(e,t,n,r,i,o,a,s,l,u){var c=2*e,d=s*c,h=l[d+t];e:for(var f=r,p=r*c;f<i;++f,p+=c){var g=a[f];if(g!==u){var v=o[p+t],m=o[p+t+e];if(!(h<v||m<h)){for(var y=t+1;y<e;++y){v=o[p+y],m=o[p+y+e];var b=l[d+y],C=l[d+y+e];if(m<b||C<v)continue e}var A=n(g,u);if(void 0!==A)return A}}}}},function(e,t,n){"use strict";var r=["d","ax","vv","rs","re","rb","ri","bs","be","bb","bi"];function i(e){var t="bruteForce"+(e?"Full":"Partial"),n=[],i=r.slice();e||i.splice(3,0,"fp");var o=["function "+t+"("+i.join()+"){"];function a(t,i){var a=function(e,t,n){var i="bruteForce"+(e?"Red":"Blue")+(t?"Flip":"")+(n?"Full":""),o=["function ",i,"(",r.join(),"){","var ","es","=2*","d",";"],a="for(var i=rs,rp=es*rs;i<re;++i,rp+=es){var x0=rb[ax+rp],x1=rb[ax+rp+d],xi=ri[i];",s="for(var j=bs,bp=es*bs;j<be;++j,bp+=es){var y0=bb[ax+bp],"+(n?"y1=bb[ax+bp+d],":"")+"yi=bi[j];";return e?o.push(a,"Q",":",s):o.push(s,"Q",":",a),n?o.push("if(y1<x0||x1<y0)continue;"):t?o.push("if(y0<=x0||x1<y0)continue;"):o.push("if(y0<x0||x1<y0)continue;"),o.push("for(var k=ax+1;k<d;++k){var r0=rb[k+rp],r1=rb[k+d+rp],b0=bb[k+bp],b1=bb[k+d+bp];if(r1<b0||b1<r0)continue Q;}var rv=vv("),t?o.push("yi,xi"):o.push("xi,yi"),o.push(");if(rv!==void 0)return rv;}}}"),{name:i,code:o.join("")}}(t,i,e);n.push(a.code),o.push("return "+a.name+"("+r.join()+");")}o.push("if(re-rs>be-bs){"),e?(a(!0,!1),o.push("}else{"),a(!1,!1)):(o.push("if(fp){"),a(!0,!0),o.push("}else{"),a(!0,!1),o.push("}}else{if(fp){"),a(!1,!0),o.push("}else{"),a(!1,!1),o.push("}")),o.push("}}return "+t);var s=n.join("")+o.join("");return new Function(s)()}t.partial=i(!1),t.full=i(!0)},function(e,t,n){"use strict";e.exports=function(e,t,n,o,a,s){if(o<=n+1)return n;var l=n,u=o,c=o+n>>>1,d=2*e,h=c,f=a[d*c+t];for(;l<u;){if(u-l<8){i(e,t,l,u,a,s),f=a[d*c+t];break}var p=u-l,g=Math.random()*p+l|0,v=a[d*g+t],m=Math.random()*p+l|0,y=a[d*m+t],b=Math.random()*p+l|0,C=a[d*b+t];v<=y?C>=y?(h=m,f=y):v>=C?(h=g,f=v):(h=b,f=C):y>=C?(h=m,f=y):C>=v?(h=g,f=v):(h=b,f=C);for(var A=d*(u-1),_=d*h,I=0;I<d;++I,++A,++_){var x=a[A];a[A]=a[_],a[_]=x}var w=s[u-1];s[u-1]=s[h],s[h]=w,h=r(e,t,l,u-1,a,s,f);for(A=d*(u-1),_=d*h,I=0;I<d;++I,++A,++_){x=a[A];a[A]=a[_],a[_]=x}w=s[u-1];if(s[u-1]=s[h],s[h]=w,c<h){for(u=h-1;l<u&&a[d*(u-1)+t]===f;)u-=1;u+=1}else{if(!(h<c))break;for(l=h+1;l<u&&a[d*l+t]===f;)l+=1}}return r(e,t,n,c,a,s,a[d*c+t])};var r=n(1046)("lo<p0",["p0"]);function i(e,t,n,r,i,o){for(var a=2*e,s=a*(n+1)+t,l=n+1;l<r;++l,s+=a)for(var u=i[s],c=l,d=a*(l-1);c>n&&i[d+t]>u;--c,d-=a){for(var h=d,f=d+a,p=0;p<a;++p,++h,++f){var g=i[h];i[h]=i[f],i[f]=g}var v=o[c];o[c]=o[c-1],o[c-1]=v}}},function(e,t,n){"use strict";e.exports=function(e,t,n,i){var o=r(e,n,i),a=r(t,n,i);if(o>0&&a>0||o<0&&a<0)return!1;var s=r(n,e,t),l=r(i,e,t);if(s>0&&l>0||s<0&&l<0)return!1;if(0===o&&0===a&&0===s&&0===l)return function(e,t,n,r){for(var i=0;i<2;++i){var o=e[i],a=t[i],s=Math.min(o,a),l=Math.max(o,a),u=n[i],c=r[i],d=Math.min(u,c);if(Math.max(u,c)<s||l<d)return!1}return!0}(e,t,n,i);return!0};var r=n(150)[3]},function(e,t,n){"use strict";var r=n(1047);e.exports=function(e){return Array.isArray(e)&&2===e.length&&r(e[0])&&r(e[1])}},function(e,t){},function(e,t,n){"use strict";var r=n(397);e.exports=function(e){return new r(e)}},function(e,t,n){"use strict";e.exports=function(e,t){return e[0].mul(t[1]).cmp(t[0].mul(e[1]))}},function(e,t,n){"use strict";var r=n(1849),i=n(1850);e.exports=function(e){var t=e[0],n=e[1];if(0===t.cmpn(0))return 0;var o=t.abs().divmod(n.abs()),a=o.div,s=r(a),l=o.mod,u=t.negative!==n.negative?-1:1;if(0===l.cmpn(0))return u*s;if(s){var c=i(s)+4,d=r(l.ushln(c).divRound(n));return u*(s+d*Math.pow(2,-c))}var h=n.bitLength()-l.bitLength()+53;d=r(l.ushln(h).divRound(n));return h<1023?u*d*Math.pow(2,-h):(d*=Math.pow(2,-1023),u*d*Math.pow(2,1023-h))}},function(e,t,n){"use strict";var r=n(572);e.exports=function(e){var t=e.length,n=e.words,i=0;if(1===t)i=n[0];else if(2===t)i=n[0]+67108864*n[1];else for(var o=0;o<t;o++){var a=n[o];i+=a*Math.pow(67108864,o)}return r(e)*i}},function(e,t,n){"use strict";var r=n(571),i=n(215).countTrailingZeros;e.exports=function(e){var t=i(r.lo(e));if(t<32)return t;var n=i(r.hi(e));if(n>20)return 52;return n+32}},function(e,t,n){"use strict";e.exports=function(e){for(var t=new Array(e.length),n=0;n<e.length;++n)t[n]=r(e[n]);return t};var r=n(570)},function(e,t,n){"use strict";var r=n(571),i=Math.pow(2,-1074);e.exports=function(e,t){if(isNaN(e)||isNaN(t))return NaN;if(e===t)return e;if(0===e)return t<0?-i:i;var n=r.hi(e),o=r.lo(e);t>e==e>0?o===-1>>>0?(n+=1,o=0):o+=1:0===o?(o=-1>>>0,n-=1):o-=1;return r.pack(o,n)}},function(e,t,n){"use strict";e.exports=function(e,t,n,r){var o=s(t,e),d=s(r,n),h=c(o,d);if(0===a(h))return null;var f=s(e,n),p=c(d,f),g=i(p,h),v=u(o,g);return l(e,v)};var r=n(1050),i=n(1049),o=n(1051),a=n(1854),s=n(1855),l=n(1856),u=n(1858);function c(e,t){return o(r(e[0],t[1]),r(e[1],t[0]))}},function(e,t,n){"use strict";var r=n(572);e.exports=function(e){return r(e[0])*r(e[1])}},function(e,t,n){"use strict";var r=n(1051);e.exports=function(e,t){for(var n=e.length,i=new Array(n),o=0;o<n;++o)i[o]=r(e[o],t[o]);return i}},function(e,t,n){"use strict";var r=n(1857);e.exports=function(e,t){for(var n=e.length,i=new Array(n),o=0;o<n;++o)i[o]=r(e[o],t[o]);return i}},function(e,t,n){"use strict";var r=n(303);e.exports=function(e,t){return r(e[0].mul(t[1]).add(t[0].mul(e[1])),e[1].mul(t[1]))}},function(e,t,n){"use strict";var r=n(570),i=n(1050);e.exports=function(e,t){for(var n=r(t),o=e.length,a=new Array(o),s=0;s<o;++s)a[s]=i(e[s],n);return a}},function(e,t,n){"use strict";var r=n(1860),i=n(1861),o=n(1862),a=n(1864);function s(e){return[Math.min(e[0],e[1]),Math.max(e[0],e[1])]}function l(e,t){return e[0]-t[0]||e[1]-t[1]}function u(e,t,n){return t in e?e[t]:n}e.exports=function(e,t,n){Array.isArray(t)?(n=n||{},t=t||[]):(n=t||{},t=[]);var c=!!u(n,"delaunay",!0),d=!!u(n,"interior",!0),h=!!u(n,"exterior",!0),f=!!u(n,"infinity",!1);if(!d&&!h||0===e.length)return[];var p=r(e,t);if(c||d!==h||f){for(var g=i(e.length,function(e){return e.map(s).sort(l)}(t)),v=0;v<p.length;++v){var m=p[v];g.addTriangle(m[0],m[1],m[2])}return c&&o(e,g),h?d?f?a(g,0,f):g.cells():a(g,1,f):a(g,-1)}return p}},function(e,t,n){"use strict";var r=n(302),i=n(150)[3];function o(e,t,n,r,i){this.a=e,this.b=t,this.idx=n,this.lowerIds=r,this.upperIds=i}function a(e,t,n,r){this.a=e,this.b=t,this.type=n,this.idx=r}function s(e,t){var n=e.a[0]-t.a[0]||e.a[1]-t.a[1]||e.type-t.type;return n||(0!==e.type&&(n=i(e.a,e.b,t.b))?n:e.idx-t.idx)}function l(e,t){return i(e.a,e.b,t)}function u(e,t,n,o,a){for(var s=r.lt(t,o,l),u=r.gt(t,o,l),c=s;c<u;++c){for(var d=t[c],h=d.lowerIds,f=h.length;f>1&&i(n[h[f-2]],n[h[f-1]],o)>0;)e.push([h[f-1],h[f-2],a]),f-=1;h.length=f,h.push(a);var p=d.upperIds;for(f=p.length;f>1&&i(n[p[f-2]],n[p[f-1]],o)<0;)e.push([p[f-2],p[f-1],a]),f-=1;p.length=f,p.push(a)}}function c(e,t){var n;return(n=e.a[0]<t.a[0]?i(e.a,e.b,t.a):i(t.b,t.a,e.a))?n:(n=t.b[0]<e.b[0]?i(e.a,e.b,t.b):i(t.b,t.a,e.b))||e.idx-t.idx}function d(e,t,n){var i=r.le(e,n,c),a=e[i],s=a.upperIds,l=s[s.length-1];a.upperIds=[l],e.splice(i+1,0,new o(n.a,n.b,n.idx,[l],s))}function h(e,t,n){var i=n.a;n.a=n.b,n.b=i;var o=r.eq(e,n,c),a=e[o];e[o-1].upperIds=a.upperIds,e.splice(o,1)}e.exports=function(e,t){for(var n=e.length,r=t.length,i=[],l=0;l<n;++l)i.push(new a(e[l],null,0,l));for(l=0;l<r;++l){var c=t[l],f=e[c[0]],p=e[c[1]];f[0]<p[0]?i.push(new a(f,p,2,l),new a(p,f,1,l)):f[0]>p[0]&&i.push(new a(p,f,2,l),new a(f,p,1,l))}i.sort(s);for(var g=i[0].a[0]-(1+Math.abs(i[0].a[0]))*Math.pow(2,-52),v=[new o([g,1],[g,0],-1,[],[],[],[])],m=[],y=(l=0,i.length);l<y;++l){var b=i[l],C=b.type;0===C?u(m,v,e,b.a,b.idx):2===C?d(v,e,b):h(v,e,b)}return m}},function(e,t,n){"use strict";var r=n(302);function i(e,t){this.stars=e,this.edges=t}e.exports=function(e,t){for(var n=new Array(e),r=0;r<e;++r)n[r]=[];return new i(n,t)};var o=i.prototype;function a(e,t,n){for(var r=1,i=e.length;r<i;r+=2)if(e[r-1]===t&&e[r]===n)return e[r-1]=e[i-2],e[r]=e[i-1],void(e.length=i-2)}o.isConstraint=function(){var e=[0,0];function t(e,t){return e[0]-t[0]||e[1]-t[1]}return function(n,i){return e[0]=Math.min(n,i),e[1]=Math.max(n,i),r.eq(this.edges,e,t)>=0}}(),o.removeTriangle=function(e,t,n){var r=this.stars;a(r[e],t,n),a(r[t],n,e),a(r[n],e,t)},o.addTriangle=function(e,t,n){var r=this.stars;r[e].push(t,n),r[t].push(n,e),r[n].push(e,t)},o.opposite=function(e,t){for(var n=this.stars[t],r=1,i=n.length;r<i;r+=2)if(n[r]===e)return n[r-1];return-1},o.flip=function(e,t){var n=this.opposite(e,t),r=this.opposite(t,e);this.removeTriangle(e,t,n),this.removeTriangle(t,e,r),this.addTriangle(e,r,n),this.addTriangle(t,n,r)},o.edges=function(){for(var e=this.stars,t=[],n=0,r=e.length;n<r;++n)for(var i=e[n],o=0,a=i.length;o<a;o+=2)t.push([i[o],i[o+1]]);return t},o.cells=function(){for(var e=this.stars,t=[],n=0,r=e.length;n<r;++n)for(var i=e[n],o=0,a=i.length;o<a;o+=2){var s=i[o],l=i[o+1];n<Math.min(s,l)&&t.push([n,s,l])}return t}},function(e,t,n){"use strict";var r=n(1863)[4];n(302);function i(e,t,n,i,o,a){var s=t.opposite(i,o);if(!(s<0)){if(o<i){var l=i;i=o,o=l,l=a,a=s,s=l}t.isConstraint(i,o)||r(e[i],e[o],e[a],e[s])<0&&n.push(i,o)}}e.exports=function(e,t){for(var n=[],o=e.length,a=t.stars,s=0;s<o;++s)for(var l=a[s],u=1;u<l.length;u+=2){if(!((f=l[u])<s)&&!t.isConstraint(s,f)){for(var c=l[u-1],d=-1,h=1;h<l.length;h+=2)if(l[h-1]===f){d=l[h];break}d<0||r(e[s],e[f],e[c],e[d])<0&&n.push(s,f)}}for(;n.length>0;){for(var f=n.pop(),p=(s=n.pop(),c=-1,d=-1,l=a[s],1);p<l.length;p+=2){var g=l[p-1],v=l[p];g===f?d=v:v===f&&(c=g)}c<0||d<0||(r(e[s],e[f],e[c],e[d])>=0||(t.flip(s,f),i(e,t,n,c,s,d),i(e,t,n,s,d,c),i(e,t,n,d,f,c),i(e,t,n,f,c,d)))}}},function(e,t,n){"use strict";var r=n(257),i=n(197),o=n(1043),a=n(396);function s(e,t){for(var n=new Array(e.length-1),r=1;r<e.length;++r)for(var i=n[r-1]=new Array(e.length-1),o=0,a=0;o<e.length;++o)o!==t&&(i[a++]=e[r][o]);return n}function l(e){if(1===e.length)return e[0];if(2===e.length)return["sum(",e[0],",",e[1],")"].join("");var t=e.length>>1;return["sum(",l(e.slice(0,t)),",",l(e.slice(t)),")"].join("")}function u(e,t){if("m"===e.charAt(0)){if("w"===t.charAt(0)){var n=e.split("[");return["w",t.substr(1),"m",n[0].substr(1)].join("")}return["prod(",e,",",t,")"].join("")}return u(t,e)}function c(e){if(2===e.length)return[["diff(",u(e[0][0],e[1][1]),",",u(e[1][0],e[0][1]),")"].join("")];for(var t=[],n=0;n<e.length;++n)t.push(["scale(",l(c(s(e,n))),",",(r=n,!0&r?"-":""),e[0][n],")"].join(""));return t;var r}function d(e,t){for(var n=[],r=0;r<t-2;++r)n.push(["prod(m",e,"[",r,"],m",e,"[",r,"])"].join(""));return l(n)}function h(e){for(var t=[],n=[],u=function(e){for(var t=new Array(e),n=0;n<e;++n){t[n]=new Array(e);for(var r=0;r<e;++r)t[n][r]=["m",r,"[",e-n-2,"]"].join("")}return t}(e),h=0;h<e;++h)u[0][h]="1",u[e-1][h]="w"+h;for(h=0;h<e;++h)0==(1&h)?t.push.apply(t,c(s(u,h))):n.push.apply(n,c(s(u,h)));var f=l(t),p=l(n),g="exactInSphere"+e,v=[];for(h=0;h<e;++h)v.push("m"+h);var m=["function ",g,"(",v.join(),"){"];for(h=0;h<e;++h){m.push("var w",h,"=",d(h,e),";");for(var y=0;y<e;++y)y!==h&&m.push("var w",h,"m",y,"=scale(w",h,",m",y,"[0]);")}return m.push("var p=",f,",n=",p,",d=diff(p,n);return d[d.length-1];}return ",g),new Function("sum","diff","prod","scale",m.join(""))(i,o,r,a)}var f=[function(){return 0},function(){return 0},function(){return 0}];function p(e){var t=f[e.length];return t||(t=f[e.length]=h(e.length)),t.apply(void 0,e)}!function(){for(;f.length<=6;)f.push(h(f.length));for(var t=[],n=["slow"],r=0;r<=6;++r)t.push("a"+r),n.push("o"+r);var i=["function testInSphere(",t.join(),"){switch(arguments.length){case 0:case 1:return 0;"];for(r=2;r<=6;++r)i.push("case ",r,":return o",r,"(",t.slice(0,r).join(),");");i.push("}var s=new Array(arguments.length);for(var i=0;i<arguments.length;++i){s[i]=arguments[i]};return slow(s);}return testInSphere"),n.push(i.join(""));var o=Function.apply(void 0,n);for(e.exports=o.apply(void 0,[p].concat(f)),r=0;r<=6;++r)e.exports[r]=f[r]}()},function(e,t,n){"use strict";var r,i=n(302);function o(e,t,n,r,i,o,a){this.cells=e,this.neighbor=t,this.flags=r,this.constraint=n,this.active=i,this.next=o,this.boundary=a}function a(e,t){return e[0]-t[0]||e[1]-t[1]||e[2]-t[2]}e.exports=function(e,t,n){var r=function(e,t){for(var n=e.cells(),r=n.length,i=0;i<r;++i){var s=(m=n[i])[0],l=m[1],u=m[2];l<u?l<s&&(m[0]=l,m[1]=u,m[2]=s):u<s&&(m[0]=u,m[1]=s,m[2]=l)}n.sort(a);var c=new Array(r);for(i=0;i<c.length;++i)c[i]=0;var d=[],h=[],f=new Array(3*r),p=new Array(3*r),g=null;t&&(g=[]);var v=new o(n,f,p,c,d,h,g);for(i=0;i<r;++i)for(var m=n[i],y=0;y<3;++y){s=m[y],l=m[(y+1)%3];var b=f[3*i+y]=v.locate(l,s,e.opposite(l,s)),C=p[3*i+y]=e.isConstraint(s,l);b<0&&(C?h.push(i):(d.push(i),c[i]=1),t&&g.push([l,s,-1]))}return v}(e,n);if(0===t)return n?r.cells.concat(r.boundary):r.cells;var i=1,s=r.active,l=r.next,u=r.flags,c=r.cells,d=r.constraint,h=r.neighbor;for(;s.length>0||l.length>0;){for(;s.length>0;){var f=s.pop();if(u[f]!==-i){u[f]=i;c[f];for(var p=0;p<3;++p){var g=h[3*f+p];g>=0&&0===u[g]&&(d[3*f+p]?l.push(g):(s.push(g),u[g]=i))}}}var v=l;l=s,s=v,l.length=0,i=-i}var m=function(e,t,n){for(var r=0,i=0;i<e.length;++i)t[i]===n&&(e[r++]=e[i]);return e.length=r,e}(c,u,t);if(n)return m.concat(r.boundary);return m},o.prototype.locate=(r=[0,0,0],function(e,t,n){var o=e,s=t,l=n;return t<n?t<e&&(o=t,s=n,l=e):n<e&&(o=n,s=e,l=t),o<0?-1:(r[0]=o,r[1]=s,r[2]=l,i.eq(this.cells,r,a))})},function(e,t,n){"use strict";e.exports=function(e,t){var n=u(e,t);e=n[0];for(var d=(t=n[1]).length,h=(e.length,r(e,t.length)),f=0;f<d;++f)if(h[f].length%2==1)throw new Error("planar-graph-to-polyline: graph must be manifold");var p=i(e,t);var g=(p=p.filter((function(e){for(var n=e.length,r=[0],i=0;i<n;++i){var o=t[e[i]],l=t[e[(i+1)%n]],u=a(-o[0],o[1]),c=a(-o[0],l[1]),d=a(l[0],o[1]),h=a(l[0],l[1]);r=s(r,s(s(u,c),s(d,h)))}return r[r.length-1]>0}))).length,v=new Array(g),m=new Array(g);for(f=0;f<g;++f){v[f]=f;var y=new Array(g),b=p[f].map((function(e){return t[e]})),C=o([b]),A=0;e:for(var _=0;_<g;++_)if(y[_]=0,f!==_){for(var I=(V=p[_]).length,x=0;x<I;++x){var w=C(t[V[x]]);if(0!==w){w<0&&(y[_]=1,A+=1);continue e}}y[_]=1,A+=1}m[f]=[A,f,y]}m.sort((function(e,t){return t[0]-e[0]}));for(f=0;f<g;++f){var M=(y=m[f])[1],S=y[2];for(_=0;_<g;++_)S[_]&&(v[_]=M)}var T=function(e){for(var t=new Array(e),n=0;n<e;++n)t[n]=[];return t}(g);for(f=0;f<g;++f)T[f].push(v[f]),T[v[f]].push(f);var E={},O=c(d,!1);for(f=0;f<g;++f)for(I=(V=p[f]).length,_=0;_<I;++_){var k=V[_],L=V[(_+1)%I],D=Math.min(k,L)+":"+Math.max(k,L);if(D in E){var P=E[D];T[P].push(f),T[f].push(P),O[k]=O[L]=!0}else E[D]=f}function N(e){for(var t=e.length,n=0;n<t;++n)if(!O[e[n]])return!1;return!0}var j=[],R=c(g,-1);for(f=0;f<g;++f)v[f]!==f||N(p[f])?R[f]=-1:(j.push(f),R[f]=0);n=[];for(;j.length>0;){var z=j.pop(),F=T[z];l(F,(function(e,t){return e-t}));var B,H=F.length,U=R[z];if(0===U){var V=p[z];B=[V]}for(f=0;f<H;++f){var Y=F[f];if(!(R[Y]>=0))if(R[Y]=1^U,j.push(Y),0===U)N(V=p[Y])||(V.reverse(),B.push(V))}0===U&&n.push(B)}return n};var r=n(1052),i=n(1866),o=n(1870),a=n(257),s=n(197),l=n(394),u=n(1878);function c(e,t){for(var n=new Array(e),r=0;r<e;++r)n[r]=t;return n}},function(e,t,n){"use strict";e.exports=function(e,t){for(var n=0|t.length,i=e.length,o=[new Array(n),new Array(n)],a=0;a<n;++a)o[0][a]=[],o[1][a]=[];for(a=0;a<i;++a){var s=e[a];o[0][s[0]].push(s),o[1][s[1]].push(s)}var l=[];for(a=0;a<n;++a)o[0][a].length+o[1][a].length===0&&l.push([a]);function u(e,t){var n=o[t][e[t]];n.splice(n.indexOf(e),1)}function c(e,n,i){for(var a,s,l,c=0;c<2;++c)if(o[c][n].length>0){a=o[c][n][0],l=c;break}s=a[1^l];for(var d=0;d<2;++d)for(var h=o[d][n],f=0;f<h.length;++f){var p=h[f],g=p[1^d];r(t[e],t[n],t[s],t[g])>0&&(a=p,s=g,l=d)}return i||a&&u(a,l),s}function d(e,n){var i=o[n][e][0],a=[e];u(i,n);for(var s=i[1^n];;){for(;s!==e;)a.push(s),s=c(a[a.length-2],s,!1);if(o[0][e].length+o[1][e].length===0)break;var l=a[a.length-1],d=e,h=a[1],f=c(l,d,!0);if(r(t[l],t[d],t[h],t[f])<0)break;a.push(e),s=c(l,d)}return a}function h(e,t){return t[1]===t[t.length-1]}for(a=0;a<n;++a)for(var f=0;f<2;++f){for(var p=[];o[f][a].length>0;){o[0][a].length;var g=d(a,f);h(0,g)?p.push.apply(p,g):(p.length>0&&l.push(p),p=g)}p.length>0&&l.push(p)}return l};var r=n(1867)},function(e,t,n){"use strict";e.exports=function(e,t,n,o){var a=r(t,n,o);if(0===a){var s=i(r(e,t,n)),u=i(r(e,t,o));if(s===u){if(0===s){var c=l(e,t,n),d=l(e,t,o);return c===d?0:c?1:-1}return 0}return 0===u?s>0||l(e,t,o)?-1:1:0===s?u>0||l(e,t,n)?1:-1:i(u-s)}var h=r(e,t,n);return h>0?a>0&&r(e,t,o)>0?1:-1:h<0?a>0||r(e,t,o)>0?1:-1:r(e,t,o)>0||l(e,t,n)?1:-1};var r=n(150),i=n(1868),o=n(1042),a=n(1869),s=n(197);function l(e,t,n){var r=o(e[0],-t[0]),i=o(e[1],-t[1]),l=o(n[0],-t[0]),u=o(n[1],-t[1]),c=s(a(r,l),a(i,u));return c[c.length-1]>=0}},function(e,t,n){"use strict";e.exports=function(e){return e<0?-1:e>0?1:0}},function(e,t,n){"use strict";var r=n(197),i=n(396);e.exports=function(e,t){if(1===e.length)return i(t,e[0]);if(1===t.length)return i(e,t[0]);if(0===e.length||0===t.length)return[0];var n=[0];if(e.length<t.length)for(var o=0;o<e.length;++o)n=r(n,i(t,e[o]));else for(o=0;o<t.length;++o)n=r(n,i(e,t[o]));return n}},function(e,t,n){e.exports=function(e){for(var t=e.length,n=[],o=[],s=0;s<t;++s)for(var c=e[s],d=c.length,h=d-1,f=0;f<d;h=f++){var p=c[h],g=c[f];p[0]===g[0]?o.push([p,g]):n.push([p,g])}if(0===n.length)return 0===o.length?u:(v=l(o),function(e){return v(e[0],e[1])?0:1});var v;var m=i(n),y=function(e,t){return function(n){var i=a.le(t,n[0]);if(i<0)return 1;var o=e[i];if(!o){if(!(i>0&&t[i]===n[0]))return 1;o=e[i-1]}for(var s=1;o;){var l=o.key,u=r(n,l[0],l[1]);if(l[0][0]<l[1][0])if(u<0)o=o.left;else{if(!(u>0))return 0;s=-1,o=o.right}else if(u>0)o=o.left;else{if(!(u<0))return 0;s=1,o=o.right}}return s}}(m.slabs,m.coordinates);return 0===o.length?y:function(e,t){return function(n){return e(n[0],n[1])?0:t(n)}}(l(o),y)};var r=n(150)[3],i=n(1871),o=n(1875),a=n(1877);function s(){return!0}function l(e){for(var t={},n=0;n<e.length;++n){var r=e[n],i=r[0][0],a=r[0][1],l=r[1][1],u=[Math.min(a,l),Math.max(a,l)];i in t?t[i].push(u):t[i]=[u]}var c={},d=Object.keys(t);for(n=0;n<d.length;++n){var h=t[d[n]];c[d[n]]=o(h)}return function(e){return function(t,n){var r=e[t];return!!r&&!!r.queryPoint(n,s)}}(c)}function u(e){return 1}},function(e,t,n){"use strict";e.exports=function(e){for(var t=e.length,n=2*t,r=new Array(n),o=0;o<t;++o){var l=e[o],u=l[0][0]<l[1][0];r[2*o]=new d(l[0][0],l,u,o),r[2*o+1]=new d(l[1][0],l,!u,o)}r.sort((function(e,t){var n=e.x-t.x;return n||((n=e.create-t.create)||Math.min(e.segment[0][1],e.segment[1][1])-Math.min(t.segment[0][1],t.segment[1][1]))}));var h=i(a),f=[],p=[],g=[];for(o=0;o<n;){for(var v=r[o].x,m=[];o<n;){var y=r[o];if(y.x!==v)break;o+=1,y.segment[0][0]===y.x&&y.segment[1][0]===y.x?y.create&&(y.segment[0][1]<y.segment[1][1]?(m.push(new c(y.segment[0][1],y.index,!0,!0)),m.push(new c(y.segment[1][1],y.index,!1,!1))):(m.push(new c(y.segment[1][1],y.index,!0,!1)),m.push(new c(y.segment[0][1],y.index,!1,!0)))):h=y.create?h.insert(y.segment,y.index):h.remove(y.segment)}f.push(h.root),p.push(v),g.push(m)}return new s(f,p,g)};var r=n(1872),i=n(1873),o=n(150),a=n(1874);function s(e,t,n){this.slabs=e,this.coordinates=t,this.horizontal=n}function l(e,t){return e.y-t}function u(e,t){for(var n=null;e;){var r,i,a=e.key;a[0][0]<a[1][0]?(r=a[0],i=a[1]):(r=a[1],i=a[0]);var s=o(r,i,t);if(s<0)e=e.left;else if(s>0)if(t[0]!==a[1][0])n=e,e=e.right;else{if(l=u(e.right,t))return l;e=e.left}else{if(t[0]!==a[1][0])return e;var l;if(l=u(e.right,t))return l;e=e.left}}return n}function c(e,t,n,r){this.y=e,this.index=t,this.start=n,this.closed=r}function d(e,t,n,r){this.x=e,this.segment=t,this.create=n,this.index=r}s.prototype.castUp=function(e){var t=r.le(this.coordinates,e[0]);if(t<0)return-1;this.slabs[t];var n=u(this.slabs[t],e),i=-1;if(n&&(i=n.value),this.coordinates[t]===e[0]){var s=null;if(n&&(s=n.key),t>0){var c=u(this.slabs[t-1],e);c&&(s?a(c.key,s)>0&&(s=c.key,i=c.value):(i=c.value,s=c.key))}var d=this.horizontal[t];if(d.length>0){var h=r.ge(d,e[1],l);if(h<d.length){var f=d[h];if(e[1]===f.y){if(f.closed)return f.index;for(;h<d.length-1&&d[h+1].y===e[1];)if((f=d[h+=1]).closed)return f.index;if(f.y===e[1]&&!f.start){if((h+=1)>=d.length)return i;f=d[h]}}if(f.start)if(s){var p=o(s[0],s[1],[e[0],f.y]);s[0][0]>s[1][0]&&(p=-p),p>0&&(i=f.index)}else i=f.index;else f.y!==e[1]&&(i=f.index)}}}return i}},function(e,t,n){"use strict";function r(e,t,n,r,i,o){var a=["function ",e,"(a,l,h,",r.join(","),"){",o?"":"var i=",n?"l-1":"h+1",";while(l<=h){var m=(l+h)>>>1,x=a",i?".get(m)":"[m]"];return o?t.indexOf("c")<0?a.push(";if(x===y){return m}else if(x<=y){"):a.push(";var p=c(x,y);if(p===0){return m}else if(p<=0){"):a.push(";if(",t,"){i=m;"),n?a.push("l=m+1}else{h=m-1}"):a.push("h=m-1}else{l=m+1}"),a.push("}"),o?a.push("return -1};"):a.push("return i};"),a.join("")}function i(e,t,n,i){return new Function([r("A","x"+e+"y",t,["y"],!1,i),r("B","x"+e+"y",t,["y"],!0,i),r("P","c(x,y)"+e+"0",t,["y","c"],!1,i),r("Q","c(x,y)"+e+"0",t,["y","c"],!0,i),"function dispatchBsearch",n,"(a,y,c,l,h){if(a.shape){if(typeof(c)==='function'){return Q(a,(l===undefined)?0:l|0,(h===undefined)?a.shape[0]-1:h|0,y,c)}else{return B(a,(c===undefined)?0:c|0,(l===undefined)?a.shape[0]-1:l|0,y)}}else{if(typeof(c)==='function'){return P(a,(l===undefined)?0:l|0,(h===undefined)?a.length-1:h|0,y,c)}else{return A(a,(c===undefined)?0:c|0,(l===undefined)?a.length-1:l|0,y)}}}return dispatchBsearch",n].join(""))()}e.exports={ge:i(">=",!1,"GE"),gt:i(">",!1,"GT"),lt:i("<",!0,"LT"),le:i("<=",!0,"LE"),eq:i("-",!0,"EQ",!0)}},function(e,t,n){"use strict";e.exports=function(e){return new s(e||g,null)};function r(e,t,n,r,i,o){this._color=e,this.key=t,this.value=n,this.left=r,this.right=i,this._count=o}function i(e){return new r(e._color,e.key,e.value,e.left,e.right,e._count)}function o(e,t){return new r(e,t.key,t.value,t.left,t.right,t._count)}function a(e){e._count=1+(e.left?e.left._count:0)+(e.right?e.right._count:0)}function s(e,t){this._compare=e,this.root=t}var l=s.prototype;function u(e,t){var n;if(t.left&&(n=u(e,t.left)))return n;return(n=e(t.key,t.value))||(t.right?u(e,t.right):void 0)}function c(e,t,n,r){if(t(e,r.key)<=0){var i;if(r.left)if(i=c(e,t,n,r.left))return i;if(i=n(r.key,r.value))return i}if(r.right)return c(e,t,n,r.right)}function d(e,t,n,r,i){var o,a=n(e,i.key),s=n(t,i.key);if(a<=0){if(i.left&&(o=d(e,t,n,r,i.left)))return o;if(s>0&&(o=r(i.key,i.value)))return o}if(s>0&&i.right)return d(e,t,n,r,i.right)}function h(e,t){this.tree=e,this._stack=t}Object.defineProperty(l,"keys",{get:function(){var e=[];return this.forEach((function(t,n){e.push(t)})),e}}),Object.defineProperty(l,"values",{get:function(){var e=[];return this.forEach((function(t,n){e.push(n)})),e}}),Object.defineProperty(l,"length",{get:function(){return this.root?this.root._count:0}}),l.insert=function(e,t){for(var n=this._compare,i=this.root,l=[],u=[];i;){var c=n(e,i.key);l.push(i),u.push(c),i=c<=0?i.left:i.right}l.push(new r(0,e,t,null,null,1));for(var d=l.length-2;d>=0;--d){i=l[d];u[d]<=0?l[d]=new r(i._color,i.key,i.value,l[d+1],i.right,i._count+1):l[d]=new r(i._color,i.key,i.value,i.left,l[d+1],i._count+1)}for(d=l.length-1;d>1;--d){var h=l[d-1];i=l[d];if(1===h._color||1===i._color)break;var f=l[d-2];if(f.left===h)if(h.left===i){if(!(p=f.right)||0!==p._color){if(f._color=0,f.left=h.right,h._color=1,h.right=f,l[d-2]=h,l[d-1]=i,a(f),a(h),d>=3)(g=l[d-3]).left===f?g.left=h:g.right=h;break}h._color=1,f.right=o(1,p),f._color=0,d-=1}else{if(!(p=f.right)||0!==p._color){if(h.right=i.left,f._color=0,f.left=i.right,i._color=1,i.left=h,i.right=f,l[d-2]=i,l[d-1]=h,a(f),a(h),a(i),d>=3)(g=l[d-3]).left===f?g.left=i:g.right=i;break}h._color=1,f.right=o(1,p),f._color=0,d-=1}else if(h.right===i){if(!(p=f.left)||0!==p._color){if(f._color=0,f.right=h.left,h._color=1,h.left=f,l[d-2]=h,l[d-1]=i,a(f),a(h),d>=3)(g=l[d-3]).right===f?g.right=h:g.left=h;break}h._color=1,f.left=o(1,p),f._color=0,d-=1}else{var p;if(!(p=f.left)||0!==p._color){var g;if(h.left=i.right,f._color=0,f.right=i.left,i._color=1,i.right=h,i.left=f,l[d-2]=i,l[d-1]=h,a(f),a(h),a(i),d>=3)(g=l[d-3]).right===f?g.right=i:g.left=i;break}h._color=1,f.left=o(1,p),f._color=0,d-=1}}return l[0]._color=1,new s(n,l[0])},l.forEach=function(e,t,n){if(this.root)switch(arguments.length){case 1:return u(e,this.root);case 2:return c(t,this._compare,e,this.root);case 3:if(this._compare(t,n)>=0)return;return d(t,n,this._compare,e,this.root)}},Object.defineProperty(l,"begin",{get:function(){for(var e=[],t=this.root;t;)e.push(t),t=t.left;return new h(this,e)}}),Object.defineProperty(l,"end",{get:function(){for(var e=[],t=this.root;t;)e.push(t),t=t.right;return new h(this,e)}}),l.at=function(e){if(e<0)return new h(this,[]);for(var t=this.root,n=[];;){if(n.push(t),t.left){if(e<t.left._count){t=t.left;continue}e-=t.left._count}if(!e)return new h(this,n);if(e-=1,!t.right)break;if(e>=t.right._count)break;t=t.right}return new h(this,[])},l.ge=function(e){for(var t=this._compare,n=this.root,r=[],i=0;n;){var o=t(e,n.key);r.push(n),o<=0&&(i=r.length),n=o<=0?n.left:n.right}return r.length=i,new h(this,r)},l.gt=function(e){for(var t=this._compare,n=this.root,r=[],i=0;n;){var o=t(e,n.key);r.push(n),o<0&&(i=r.length),n=o<0?n.left:n.right}return r.length=i,new h(this,r)},l.lt=function(e){for(var t=this._compare,n=this.root,r=[],i=0;n;){var o=t(e,n.key);r.push(n),o>0&&(i=r.length),n=o<=0?n.left:n.right}return r.length=i,new h(this,r)},l.le=function(e){for(var t=this._compare,n=this.root,r=[],i=0;n;){var o=t(e,n.key);r.push(n),o>=0&&(i=r.length),n=o<0?n.left:n.right}return r.length=i,new h(this,r)},l.find=function(e){for(var t=this._compare,n=this.root,r=[];n;){var i=t(e,n.key);if(r.push(n),0===i)return new h(this,r);n=i<=0?n.left:n.right}return new h(this,[])},l.remove=function(e){var t=this.find(e);return t?t.remove():this},l.get=function(e){for(var t=this._compare,n=this.root;n;){var r=t(e,n.key);if(0===r)return n.value;n=r<=0?n.left:n.right}};var f=h.prototype;function p(e,t){e.key=t.key,e.value=t.value,e.left=t.left,e.right=t.right,e._color=t._color,e._count=t._count}function g(e,t){return e<t?-1:e>t?1:0}Object.defineProperty(f,"valid",{get:function(){return this._stack.length>0}}),Object.defineProperty(f,"node",{get:function(){return this._stack.length>0?this._stack[this._stack.length-1]:null},enumerable:!0}),f.clone=function(){return new h(this.tree,this._stack.slice())},f.remove=function(){var e=this._stack;if(0===e.length)return this.tree;var t=new Array(e.length),n=e[e.length-1];t[t.length-1]=new r(n._color,n.key,n.value,n.left,n.right,n._count);for(var l=e.length-2;l>=0;--l){(n=e[l]).left===e[l+1]?t[l]=new r(n._color,n.key,n.value,t[l+1],n.right,n._count):t[l]=new r(n._color,n.key,n.value,n.left,t[l+1],n._count)}if((n=t[t.length-1]).left&&n.right){var u=t.length;for(n=n.left;n.right;)t.push(n),n=n.right;var c=t[u-1];t.push(new r(n._color,c.key,c.value,n.left,n.right,n._count)),t[u-1].key=n.key,t[u-1].value=n.value;for(l=t.length-2;l>=u;--l)n=t[l],t[l]=new r(n._color,n.key,n.value,n.left,t[l+1],n._count);t[u-1].left=t[u]}if(0===(n=t[t.length-1])._color){var d=t[t.length-2];d.left===n?d.left=null:d.right===n&&(d.right=null),t.pop();for(l=0;l<t.length;++l)t[l]._count--;return new s(this.tree._compare,t[0])}if(n.left||n.right){n.left?p(n,n.left):n.right&&p(n,n.right),n._color=1;for(l=0;l<t.length-1;++l)t[l]._count--;return new s(this.tree._compare,t[0])}if(1===t.length)return new s(this.tree._compare,null);for(l=0;l<t.length;++l)t[l]._count--;var h=t[t.length-2];return function(e){for(var t,n,r,s,l=e.length-1;l>=0;--l){if(t=e[l],0===l)return void(t._color=1);if((n=e[l-1]).left===t){if((r=n.right).right&&0===r.right._color){if(s=(r=n.right=i(r)).right=i(r.right),n.right=r.left,r.left=n,r.right=s,r._color=n._color,t._color=1,n._color=1,s._color=1,a(n),a(r),l>1)(u=e[l-2]).left===n?u.left=r:u.right=r;return void(e[l-1]=r)}if(r.left&&0===r.left._color){if(s=(r=n.right=i(r)).left=i(r.left),n.right=s.left,r.left=s.right,s.left=n,s.right=r,s._color=n._color,n._color=1,r._color=1,t._color=1,a(n),a(r),a(s),l>1)(u=e[l-2]).left===n?u.left=s:u.right=s;return void(e[l-1]=s)}if(1===r._color){if(0===n._color)return n._color=1,void(n.right=o(0,r));n.right=o(0,r);continue}r=i(r),n.right=r.left,r.left=n,r._color=n._color,n._color=0,a(n),a(r),l>1&&((u=e[l-2]).left===n?u.left=r:u.right=r),e[l-1]=r,e[l]=n,l+1<e.length?e[l+1]=t:e.push(t),l+=2}else{if((r=n.left).left&&0===r.left._color){if(s=(r=n.left=i(r)).left=i(r.left),n.left=r.right,r.right=n,r.left=s,r._color=n._color,t._color=1,n._color=1,s._color=1,a(n),a(r),l>1)(u=e[l-2]).right===n?u.right=r:u.left=r;return void(e[l-1]=r)}if(r.right&&0===r.right._color){if(s=(r=n.left=i(r)).right=i(r.right),n.left=s.right,r.right=s.left,s.right=n,s.left=r,s._color=n._color,n._color=1,r._color=1,t._color=1,a(n),a(r),a(s),l>1)(u=e[l-2]).right===n?u.right=s:u.left=s;return void(e[l-1]=s)}if(1===r._color){if(0===n._color)return n._color=1,void(n.left=o(0,r));n.left=o(0,r);continue}var u;r=i(r),n.left=r.right,r.right=n,r._color=n._color,n._color=0,a(n),a(r),l>1&&((u=e[l-2]).right===n?u.right=r:u.left=r),e[l-1]=r,e[l]=n,l+1<e.length?e[l+1]=t:e.push(t),l+=2}}}(t),h.left===n?h.left=null:h.right=null,new s(this.tree._compare,t[0])},Object.defineProperty(f,"key",{get:function(){if(this._stack.length>0)return this._stack[this._stack.length-1].key},enumerable:!0}),Object.defineProperty(f,"value",{get:function(){if(this._stack.length>0)return this._stack[this._stack.length-1].value},enumerable:!0}),Object.defineProperty(f,"index",{get:function(){var e=0,t=this._stack;if(0===t.length){var n=this.tree.root;return n?n._count:0}t[t.length-1].left&&(e=t[t.length-1].left._count);for(var r=t.length-2;r>=0;--r)t[r+1]===t[r].right&&(++e,t[r].left&&(e+=t[r].left._count));return e},enumerable:!0}),f.next=function(){var e=this._stack;if(0!==e.length){var t=e[e.length-1];if(t.right)for(t=t.right;t;)e.push(t),t=t.left;else for(e.pop();e.length>0&&e[e.length-1].right===t;)t=e[e.length-1],e.pop()}},Object.defineProperty(f,"hasNext",{get:function(){var e=this._stack;if(0===e.length)return!1;if(e[e.length-1].right)return!0;for(var t=e.length-1;t>0;--t)if(e[t-1].left===e[t])return!0;return!1}}),f.update=function(e){var t=this._stack;if(0===t.length)throw new Error("Can't update empty node!");var n=new Array(t.length),i=t[t.length-1];n[n.length-1]=new r(i._color,i.key,e,i.left,i.right,i._count);for(var o=t.length-2;o>=0;--o)(i=t[o]).left===t[o+1]?n[o]=new r(i._color,i.key,i.value,n[o+1],i.right,i._count):n[o]=new r(i._color,i.key,i.value,i.left,n[o+1],i._count);return new s(this.tree._compare,n[0])},f.prev=function(){var e=this._stack;if(0!==e.length){var t=e[e.length-1];if(t.left)for(t=t.left;t;)e.push(t),t=t.right;else for(e.pop();e.length>0&&e[e.length-1].left===t;)t=e[e.length-1],e.pop()}},Object.defineProperty(f,"hasPrev",{get:function(){var e=this._stack;if(0===e.length)return!1;if(e[e.length-1].left)return!0;for(var t=e.length-1;t>0;--t)if(e[t-1].right===e[t])return!0;return!1}})},function(e,t,n){"use strict";e.exports=function(e,t){var n,o,a,s;if(t[0][0]<t[1][0])n=t[0],o=t[1];else{if(!(t[0][0]>t[1][0]))return i(t,e);n=t[1],o=t[0]}if(e[0][0]<e[1][0])a=e[0],s=e[1];else{if(!(e[0][0]>e[1][0]))return-i(e,t);a=e[1],s=e[0]}var l=r(n,o,s),u=r(n,o,a);if(l<0){if(u<=0)return l}else if(l>0){if(u>=0)return l}else if(u)return u;if(l=r(s,a,o),u=r(s,a,n),l<0){if(u<=0)return l}else if(l>0){if(u>=0)return l}else if(u)return u;return o[0]-s[0]};var r=n(150);function i(e,t){var n,i,o,a;if(t[0][0]<t[1][0])n=t[0],i=t[1];else{if(!(t[0][0]>t[1][0])){var s=Math.min(e[0][1],e[1][1]),l=Math.max(e[0][1],e[1][1]),u=Math.min(t[0][1],t[1][1]),c=Math.max(t[0][1],t[1][1]);return l<u?l-u:s>c?s-c:l-c}n=t[1],i=t[0]}e[0][1]<e[1][1]?(o=e[0],a=e[1]):(o=e[1],a=e[0]);var d=r(i,n,o);return d||((d=r(i,n,a))||a-i)}},function(e,t,n){"use strict";var r=n(1876);function i(e,t,n,r,i){this.mid=e,this.left=t,this.right=n,this.leftPoints=r,this.rightPoints=i,this.count=(t?t.count:0)+(n?n.count:0)+r.length}e.exports=function(e){if(!e||0===e.length)return new m(null);return new m(v(e))};var o=i.prototype;function a(e,t){e.mid=t.mid,e.left=t.left,e.right=t.right,e.leftPoints=t.leftPoints,e.rightPoints=t.rightPoints,e.count=t.count}function s(e,t){var n=v(t);e.mid=n.mid,e.left=n.left,e.right=n.right,e.leftPoints=n.leftPoints,e.rightPoints=n.rightPoints,e.count=n.count}function l(e,t){var n=e.intervals([]);n.push(t),s(e,n)}function u(e,t){var n=e.intervals([]),r=n.indexOf(t);return r<0?0:(n.splice(r,1),s(e,n),1)}function c(e,t,n){for(var r=0;r<e.length&&e[r][0]<=t;++r){var i=n(e[r]);if(i)return i}}function d(e,t,n){for(var r=e.length-1;r>=0&&e[r][1]>=t;--r){var i=n(e[r]);if(i)return i}}function h(e,t){for(var n=0;n<e.length;++n){var r=t(e[n]);if(r)return r}}function f(e,t){return e-t}function p(e,t){var n=e[0]-t[0];return n||e[1]-t[1]}function g(e,t){var n=e[1]-t[1];return n||e[0]-t[0]}function v(e){if(0===e.length)return null;for(var t=[],n=0;n<e.length;++n)t.push(e[n][0],e[n][1]);t.sort(f);var r=t[t.length>>1],o=[],a=[],s=[];for(n=0;n<e.length;++n){var l=e[n];l[1]<r?o.push(l):r<l[0]?a.push(l):s.push(l)}var u=s,c=s.slice();return u.sort(p),c.sort(g),new i(r,v(o),v(a),u,c)}function m(e){this.root=e}o.intervals=function(e){return e.push.apply(e,this.leftPoints),this.left&&this.left.intervals(e),this.right&&this.right.intervals(e),e},o.insert=function(e){var t=this.count-this.leftPoints.length;if(this.count+=1,e[1]<this.mid)this.left?4*(this.left.count+1)>3*(t+1)?l(this,e):this.left.insert(e):this.left=v([e]);else if(e[0]>this.mid)this.right?4*(this.right.count+1)>3*(t+1)?l(this,e):this.right.insert(e):this.right=v([e]);else{var n=r.ge(this.leftPoints,e,p),i=r.ge(this.rightPoints,e,g);this.leftPoints.splice(n,0,e),this.rightPoints.splice(i,0,e)}},o.remove=function(e){var t=this.count-this.leftPoints;if(e[1]<this.mid)return this.left?4*(this.right?this.right.count:0)>3*(t-1)?u(this,e):2===(s=this.left.remove(e))?(this.left=null,this.count-=1,1):(1===s&&(this.count-=1),s):0;if(e[0]>this.mid)return this.right?4*(this.left?this.left.count:0)>3*(t-1)?u(this,e):2===(s=this.right.remove(e))?(this.right=null,this.count-=1,1):(1===s&&(this.count-=1),s):0;if(1===this.count)return this.leftPoints[0]===e?2:0;if(1===this.leftPoints.length&&this.leftPoints[0]===e){if(this.left&&this.right){for(var n=this,i=this.left;i.right;)n=i,i=i.right;if(n===this)i.right=this.right;else{var o=this.left,s=this.right;n.count-=i.count,n.right=i.left,i.left=o,i.right=s}a(this,i),this.count=(this.left?this.left.count:0)+(this.right?this.right.count:0)+this.leftPoints.length}else this.left?a(this,this.left):a(this,this.right);return 1}for(o=r.ge(this.leftPoints,e,p);o<this.leftPoints.length&&this.leftPoints[o][0]===e[0];++o)if(this.leftPoints[o]===e){this.count-=1,this.leftPoints.splice(o,1);for(s=r.ge(this.rightPoints,e,g);s<this.rightPoints.length&&this.rightPoints[s][1]===e[1];++s)if(this.rightPoints[s]===e)return this.rightPoints.splice(s,1),1}return 0},o.queryPoint=function(e,t){if(e<this.mid){if(this.left)if(n=this.left.queryPoint(e,t))return n;return c(this.leftPoints,e,t)}if(e>this.mid){var n;if(this.right)if(n=this.right.queryPoint(e,t))return n;return d(this.rightPoints,e,t)}return h(this.leftPoints,t)},o.queryInterval=function(e,t,n){var r;if(e<this.mid&&this.left&&(r=this.left.queryInterval(e,t,n)))return r;if(t>this.mid&&this.right&&(r=this.right.queryInterval(e,t,n)))return r;return t<this.mid?c(this.leftPoints,t,n):e>this.mid?d(this.rightPoints,e,n):h(this.leftPoints,n)};var y=m.prototype;y.insert=function(e){this.root?this.root.insert(e):this.root=new i(e[0],null,null,[e],[e])},y.remove=function(e){if(this.root){var t=this.root.remove(e);return 2===t&&(this.root=null),0!==t}return!1},y.queryPoint=function(e,t){if(this.root)return this.root.queryPoint(e,t)},y.queryInterval=function(e,t,n){if(e<=t&&this.root)return this.root.queryInterval(e,t,n)},Object.defineProperty(y,"count",{get:function(){return this.root?this.root.count:0}}),Object.defineProperty(y,"intervals",{get:function(){return this.root?this.root.intervals([]):[]}})},function(e,t,n){"use strict";function r(e,t,n,r,i,o){var a=["function ",e,"(a,l,h,",r.join(","),"){",o?"":"var i=",n?"l-1":"h+1",";while(l<=h){var m=(l+h)>>>1,x=a",i?".get(m)":"[m]"];return o?t.indexOf("c")<0?a.push(";if(x===y){return m}else if(x<=y){"):a.push(";var p=c(x,y);if(p===0){return m}else if(p<=0){"):a.push(";if(",t,"){i=m;"),n?a.push("l=m+1}else{h=m-1}"):a.push("h=m-1}else{l=m+1}"),a.push("}"),o?a.push("return -1};"):a.push("return i};"),a.join("")}function i(e,t,n,i){return new Function([r("A","x"+e+"y",t,["y"],!1,i),r("B","x"+e+"y",t,["y"],!0,i),r("P","c(x,y)"+e+"0",t,["y","c"],!1,i),r("Q","c(x,y)"+e+"0",t,["y","c"],!0,i),"function dispatchBsearch",n,"(a,y,c,l,h){if(a.shape){if(typeof(c)==='function'){return Q(a,(l===undefined)?0:l|0,(h===undefined)?a.shape[0]-1:h|0,y,c)}else{return B(a,(c===undefined)?0:c|0,(l===undefined)?a.shape[0]-1:l|0,y)}}else{if(typeof(c)==='function'){return P(a,(l===undefined)?0:l|0,(h===undefined)?a.length-1:h|0,y,c)}else{return A(a,(c===undefined)?0:c|0,(l===undefined)?a.length-1:l|0,y)}}}return dispatchBsearch",n].join(""))()}e.exports={ge:i(">=",!1,"GE"),gt:i(">",!1,"GT"),lt:i("<",!0,"LT"),le:i("<=",!0,"LE"),eq:i("-",!0,"EQ",!0)}},function(e,t,n){"use strict";function r(e,t,n,r,i,o){var a=["function ",e,"(a,l,h,",r.join(","),"){",o?"":"var i=",n?"l-1":"h+1",";while(l<=h){var m=(l+h)>>>1,x=a",i?".get(m)":"[m]"];return o?t.indexOf("c")<0?a.push(";if(x===y){return m}else if(x<=y){"):a.push(";var p=c(x,y);if(p===0){return m}else if(p<=0){"):a.push(";if(",t,"){i=m;"),n?a.push("l=m+1}else{h=m-1}"):a.push("h=m-1}else{l=m+1}"),a.push("}"),o?a.push("return -1};"):a.push("return i};"),a.join("")}function i(e,t,n,i){return new Function([r("A","x"+e+"y",t,["y"],!1,i),r("B","x"+e+"y",t,["y"],!0,i),r("P","c(x,y)"+e+"0",t,["y","c"],!1,i),r("Q","c(x,y)"+e+"0",t,["y","c"],!0,i),"function dispatchBsearch",n,"(a,y,c,l,h){if(a.shape){if(typeof(c)==='function'){return Q(a,(l===undefined)?0:l|0,(h===undefined)?a.shape[0]-1:h|0,y,c)}else{return B(a,(c===undefined)?0:c|0,(l===undefined)?a.shape[0]-1:l|0,y)}}else{if(typeof(c)==='function'){return P(a,(l===undefined)?0:l|0,(h===undefined)?a.length-1:h|0,y,c)}else{return A(a,(c===undefined)?0:c|0,(l===undefined)?a.length-1:l|0,y)}}}return dispatchBsearch",n].join(""))()}e.exports={ge:i(">=",!1,"GE"),gt:i(">",!1,"GT"),lt:i("<",!0,"LT"),le:i("<=",!0,"LE"),eq:i("-",!0,"EQ",!0)}},function(e,t,n){"use strict";e.exports=function(e,t){for(var n=r(e,t.length),i=new Array(t.length),o=new Array(t.length),a=[],s=0;s<t.length;++s){var l=n[s].length;o[s]=l,i[s]=!0,l<=1&&a.push(s)}for(;a.length>0;){var u=a.pop();i[u]=!1;var c=n[u];for(s=0;s<c.length;++s){var d=c[s];0==--o[d]&&a.push(d)}}var h=new Array(t.length),f=[];for(s=0;s<t.length;++s)if(i[s]){u=f.length;h[s]=u,f.push(t[s])}else h[s]=-1;var p=[];for(s=0;s<e.length;++s){var g=e[s];i[g[0]]&&i[g[1]]&&p.push([h[g[0]],h[g[1]]])}return[p,f]};var r=n(1052)},function(e,t,n){"use strict";e.exports=function(e){return e||0===e?e.toString():""}},function(e,t,n){"use strict";e.exports=function(e){var t=e.gl,n=r(t),a=i(t,[{buffer:n,type:t.FLOAT,size:3,offset:0,stride:40},{buffer:n,type:t.FLOAT,size:4,offset:12,stride:40},{buffer:n,type:t.FLOAT,size:3,offset:28,stride:40}]),l=o(t);l.attributes.position.location=0,l.attributes.color.location=1,l.attributes.offset.location=2;var u=new s(t,n,a,l);return u.update(e),u};var r=n(180),i=n(181),o=n(1881),a=[1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1];function s(e,t,n,r){this.gl=e,this.shader=r,this.buffer=t,this.vao=n,this.pixelRatio=1,this.bounds=[[1/0,1/0,1/0],[-1/0,-1/0,-1/0]],this.clipBounds=[[-1/0,-1/0,-1/0],[1/0,1/0,1/0]],this.lineWidth=[1,1,1],this.capSize=[10,10,10],this.lineCount=[0,0,0],this.lineOffset=[0,0,0],this.opacity=1,this.hasAlpha=!1}var l=s.prototype;function u(e,t){for(var n=0;n<3;++n)e[0][n]=Math.min(e[0][n],t[n]),e[1][n]=Math.max(e[1][n],t[n])}l.isOpaque=function(){return!this.hasAlpha},l.isTransparent=function(){return this.hasAlpha},l.drawTransparent=l.draw=function(e){var t=this.gl,n=this.shader.uniforms;this.shader.bind();var r=n.view=e.view||a,i=n.projection=e.projection||a;n.model=e.model||a,n.clipBounds=this.clipBounds,n.opacity=this.opacity;var o=r[12],s=r[13],l=r[14],u=r[15],c=(e._ortho||!1?2:1)*this.pixelRatio*(i[3]*o+i[7]*s+i[11]*l+i[15]*u)/t.drawingBufferHeight;this.vao.bind();for(var d=0;d<3;++d)t.lineWidth(this.lineWidth[d]*this.pixelRatio),n.capSize=this.capSize[d]*c,this.lineCount[d]&&t.drawArrays(t.LINES,this.lineOffset[d],this.lineCount[d]);this.vao.unbind()};var c=function(){for(var e=new Array(3),t=0;t<3;++t){for(var n=[],r=1;r<=2;++r)for(var i=-1;i<=1;i+=2){var o=[0,0,0];o[(r+t)%3]=i,n.push(o)}e[t]=n}return e}();function d(e,t,n,r){for(var i=c[r],o=0;o<i.length;++o){var a=i[o];e.push(t[0],t[1],t[2],n[0],n[1],n[2],n[3],a[0],a[1],a[2])}return i.length}l.update=function(e){"lineWidth"in(e=e||{})&&(this.lineWidth=e.lineWidth,Array.isArray(this.lineWidth)||(this.lineWidth=[this.lineWidth,this.lineWidth,this.lineWidth])),"capSize"in e&&(this.capSize=e.capSize,Array.isArray(this.capSize)||(this.capSize=[this.capSize,this.capSize,this.capSize])),this.hasAlpha=!1,"opacity"in e&&(this.opacity=+e.opacity,this.opacity<1&&(this.hasAlpha=!0));var t=e.color||[[0,0,0],[0,0,0],[0,0,0]],n=e.position,r=e.error;if(Array.isArray(t[0])||(t=[t,t,t]),n&&r){var i=[],o=n.length,a=0;this.bounds=[[1/0,1/0,1/0],[-1/0,-1/0,-1/0]],this.lineCount=[0,0,0];for(var s=0;s<3;++s){this.lineOffset[s]=a;e:for(var l=0;l<o;++l){for(var c=n[l],h=0;h<3;++h)if(isNaN(c[h])||!isFinite(c[h]))continue e;var f=r[l],p=t[s];if(Array.isArray(p[0])&&(p=t[l]),3===p.length?p=[p[0],p[1],p[2],1]:4===p.length&&(p=[p[0],p[1],p[2],p[3]],!this.hasAlpha&&p[3]<1&&(this.hasAlpha=!0)),!isNaN(f[0][s])&&!isNaN(f[1][s])){var g;if(f[0][s]<0)(g=c.slice())[s]+=f[0][s],i.push(c[0],c[1],c[2],p[0],p[1],p[2],p[3],0,0,0,g[0],g[1],g[2],p[0],p[1],p[2],p[3],0,0,0),u(this.bounds,g),a+=2+d(i,g,p,s);if(f[1][s]>0)(g=c.slice())[s]+=f[1][s],i.push(c[0],c[1],c[2],p[0],p[1],p[2],p[3],0,0,0,g[0],g[1],g[2],p[0],p[1],p[2],p[3],0,0,0),u(this.bounds,g),a+=2+d(i,g,p,s)}}this.lineCount[s]=a-this.lineOffset[s]}this.buffer.update(i)}},l.dispose=function(){this.shader.dispose(),this.buffer.dispose(),this.vao.dispose()}},function(e,t,n){"use strict";var r=n(217),i=n(218),o=r(["precision highp float;\n#define GLSLIFY 1\n\nattribute vec3 position, offset;\nattribute vec4 color;\nuniform mat4 model, view, projection;\nuniform float capSize;\nvarying vec4 fragColor;\nvarying vec3 fragPosition;\n\nvoid main() {\n  vec4 worldPosition  = model * vec4(position, 1.0);\n  worldPosition       = (worldPosition / worldPosition.w) + vec4(capSize * offset, 0.0);\n  gl_Position         = projection * view * worldPosition;\n  fragColor           = color;\n  fragPosition        = position;\n}"]),a=r(["precision highp float;\n#define GLSLIFY 1\n\nbool outOfRange(float a, float b, float p) {\n  return ((p > max(a, b)) || \n          (p < min(a, b)));\n}\n\nbool outOfRange(vec2 a, vec2 b, vec2 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y));\n}\n\nbool outOfRange(vec3 a, vec3 b, vec3 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y) ||\n          outOfRange(a.z, b.z, p.z));\n}\n\nbool outOfRange(vec4 a, vec4 b, vec4 p) {\n  return outOfRange(a.xyz, b.xyz, p.xyz);\n}\n\nuniform vec3 clipBounds[2];\nuniform float opacity;\nvarying vec3 fragPosition;\nvarying vec4 fragColor;\n\nvoid main() {\n  if (\n    outOfRange(clipBounds[0], clipBounds[1], fragPosition) ||\n    fragColor.a * opacity == 0.\n  ) discard;\n\n  gl_FragColor = opacity * fragColor;\n}"]);e.exports=function(e){return i(e,o,a,null,[{name:"position",type:"vec3"},{name:"color",type:"vec4"},{name:"offset",type:"vec3"}])}},function(e,t){t.vertexNormals=function(e,t,n){for(var r=t.length,i=new Array(r),o=void 0===n?1e-6:n,a=0;a<r;++a)i[a]=[0,0,0];for(a=0;a<e.length;++a)for(var s=e[a],l=0,u=s[s.length-1],c=s[0],d=0;d<s.length;++d){l=u,u=c,c=s[(d+1)%s.length];for(var h=t[l],f=t[u],p=t[c],g=new Array(3),v=0,m=new Array(3),y=0,b=0;b<3;++b)g[b]=h[b]-f[b],v+=g[b]*g[b],m[b]=p[b]-f[b],y+=m[b]*m[b];if(v*y>o){var C=i[u],A=1/Math.sqrt(v*y);for(b=0;b<3;++b){var _=(b+1)%3,I=(b+2)%3;C[b]+=A*(m[_]*g[I]-m[I]*g[_])}}}for(a=0;a<r;++a){C=i[a];var x=0;for(b=0;b<3;++b)x+=C[b]*C[b];if(x>o)for(A=1/Math.sqrt(x),b=0;b<3;++b)C[b]*=A;else for(b=0;b<3;++b)C[b]=0}return i},t.faceNormals=function(e,t,n){for(var r=e.length,i=new Array(r),o=void 0===n?1e-6:n,a=0;a<r;++a){for(var s=e[a],l=new Array(3),u=0;u<3;++u)l[u]=t[s[u]];var c=new Array(3),d=new Array(3);for(u=0;u<3;++u)c[u]=l[1][u]-l[0][u],d[u]=l[2][u]-l[0][u];var h=new Array(3),f=0;for(u=0;u<3;++u){var p=(u+1)%3,g=(u+2)%3;h[u]=c[p]*d[g]-c[g]*d[p],f+=h[u]*h[u]}f=f>o?1/Math.sqrt(f):0;for(u=0;u<3;++u)h[u]*=f;i[a]=h}return i}},function(e,t,n){"use strict";var r=n(1884),i=n(1885);function o(e){return[e[0]/255,e[1]/255,e[2]/255,e[3]]}function a(e){for(var t,n="#",r=0;r<3;++r)n+=("00"+(t=(t=e[r]).toString(16))).substr(t.length);return n}function s(e){return"rgba("+e.join(",")+")"}e.exports=function(e){var t,n,l,u,c,d,h,f,p,g;e||(e={});f=(e.nshades||72)-1,h=e.format||"hex",(d=e.colormap)||(d="jet");if("string"==typeof d){if(d=d.toLowerCase(),!r[d])throw Error(d+" not a supported colorscale");c=r[d]}else{if(!Array.isArray(d))throw Error("unsupported colormap option",d);c=d.slice()}if(c.length>f+1)throw new Error(d+" map requires nshades to be at least size "+c.length);p=Array.isArray(e.alpha)?2!==e.alpha.length?[1,1]:e.alpha.slice():"number"==typeof e.alpha?[e.alpha,e.alpha]:[1,1];t=c.map((function(e){return Math.round(e.index*f)})),p[0]=Math.min(Math.max(p[0],0),1),p[1]=Math.min(Math.max(p[1],0),1);var v=c.map((function(e,t){var n=c[t].index,r=c[t].rgb.slice();return 4===r.length&&r[3]>=0&&r[3]<=1||(r[3]=p[0]+(p[1]-p[0])*n),r})),m=[];for(g=0;g<t.length-1;++g){u=t[g+1]-t[g],n=v[g],l=v[g+1];for(var y=0;y<u;y++){var b=y/u;m.push([Math.round(i(n[0],l[0],b)),Math.round(i(n[1],l[1],b)),Math.round(i(n[2],l[2],b)),i(n[3],l[3],b)])}}m.push(c[c.length-1].rgb.concat(p[1])),"hex"===h?m=m.map(a):"rgbaString"===h?m=m.map(s):"float"===h&&(m=m.map(o));return m}},function(e,t){e.exports={jet:[{index:0,rgb:[0,0,131]},{index:.125,rgb:[0,60,170]},{index:.375,rgb:[5,255,255]},{index:.625,rgb:[255,255,0]},{index:.875,rgb:[250,0,0]},{index:1,rgb:[128,0,0]}],hsv:[{index:0,rgb:[255,0,0]},{index:.169,rgb:[253,255,2]},{index:.173,rgb:[247,255,2]},{index:.337,rgb:[0,252,4]},{index:.341,rgb:[0,252,10]},{index:.506,rgb:[1,249,255]},{index:.671,rgb:[2,0,253]},{index:.675,rgb:[8,0,253]},{index:.839,rgb:[255,0,251]},{index:.843,rgb:[255,0,245]},{index:1,rgb:[255,0,6]}],hot:[{index:0,rgb:[0,0,0]},{index:.3,rgb:[230,0,0]},{index:.6,rgb:[255,210,0]},{index:1,rgb:[255,255,255]}],spring:[{index:0,rgb:[255,0,255]},{index:1,rgb:[255,255,0]}],summer:[{index:0,rgb:[0,128,102]},{index:1,rgb:[255,255,102]}],autumn:[{index:0,rgb:[255,0,0]},{index:1,rgb:[255,255,0]}],winter:[{index:0,rgb:[0,0,255]},{index:1,rgb:[0,255,128]}],bone:[{index:0,rgb:[0,0,0]},{index:.376,rgb:[84,84,116]},{index:.753,rgb:[169,200,200]},{index:1,rgb:[255,255,255]}],copper:[{index:0,rgb:[0,0,0]},{index:.804,rgb:[255,160,102]},{index:1,rgb:[255,199,127]}],greys:[{index:0,rgb:[0,0,0]},{index:1,rgb:[255,255,255]}],yignbu:[{index:0,rgb:[8,29,88]},{index:.125,rgb:[37,52,148]},{index:.25,rgb:[34,94,168]},{index:.375,rgb:[29,145,192]},{index:.5,rgb:[65,182,196]},{index:.625,rgb:[127,205,187]},{index:.75,rgb:[199,233,180]},{index:.875,rgb:[237,248,217]},{index:1,rgb:[255,255,217]}],greens:[{index:0,rgb:[0,68,27]},{index:.125,rgb:[0,109,44]},{index:.25,rgb:[35,139,69]},{index:.375,rgb:[65,171,93]},{index:.5,rgb:[116,196,118]},{index:.625,rgb:[161,217,155]},{index:.75,rgb:[199,233,192]},{index:.875,rgb:[229,245,224]},{index:1,rgb:[247,252,245]}],yiorrd:[{index:0,rgb:[128,0,38]},{index:.125,rgb:[189,0,38]},{index:.25,rgb:[227,26,28]},{index:.375,rgb:[252,78,42]},{index:.5,rgb:[253,141,60]},{index:.625,rgb:[254,178,76]},{index:.75,rgb:[254,217,118]},{index:.875,rgb:[255,237,160]},{index:1,rgb:[255,255,204]}],bluered:[{index:0,rgb:[0,0,255]},{index:1,rgb:[255,0,0]}],rdbu:[{index:0,rgb:[5,10,172]},{index:.35,rgb:[106,137,247]},{index:.5,rgb:[190,190,190]},{index:.6,rgb:[220,170,132]},{index:.7,rgb:[230,145,90]},{index:1,rgb:[178,10,28]}],picnic:[{index:0,rgb:[0,0,255]},{index:.1,rgb:[51,153,255]},{index:.2,rgb:[102,204,255]},{index:.3,rgb:[153,204,255]},{index:.4,rgb:[204,204,255]},{index:.5,rgb:[255,255,255]},{index:.6,rgb:[255,204,255]},{index:.7,rgb:[255,153,255]},{index:.8,rgb:[255,102,204]},{index:.9,rgb:[255,102,102]},{index:1,rgb:[255,0,0]}],rainbow:[{index:0,rgb:[150,0,90]},{index:.125,rgb:[0,0,200]},{index:.25,rgb:[0,25,255]},{index:.375,rgb:[0,152,255]},{index:.5,rgb:[44,255,150]},{index:.625,rgb:[151,255,0]},{index:.75,rgb:[255,234,0]},{index:.875,rgb:[255,111,0]},{index:1,rgb:[255,0,0]}],portland:[{index:0,rgb:[12,51,131]},{index:.25,rgb:[10,136,186]},{index:.5,rgb:[242,211,56]},{index:.75,rgb:[242,143,56]},{index:1,rgb:[217,30,30]}],blackbody:[{index:0,rgb:[0,0,0]},{index:.2,rgb:[230,0,0]},{index:.4,rgb:[230,210,0]},{index:.7,rgb:[255,255,255]},{index:1,rgb:[160,200,255]}],earth:[{index:0,rgb:[0,0,130]},{index:.1,rgb:[0,180,180]},{index:.2,rgb:[40,210,40]},{index:.4,rgb:[230,230,50]},{index:.6,rgb:[120,70,20]},{index:1,rgb:[255,255,255]}],electric:[{index:0,rgb:[0,0,0]},{index:.15,rgb:[30,0,100]},{index:.4,rgb:[120,0,100]},{index:.6,rgb:[160,90,0]},{index:.8,rgb:[230,200,0]},{index:1,rgb:[255,250,220]}],alpha:[{index:0,rgb:[255,255,255,0]},{index:1,rgb:[255,255,255,1]}],viridis:[{index:0,rgb:[68,1,84]},{index:.13,rgb:[71,44,122]},{index:.25,rgb:[59,81,139]},{index:.38,rgb:[44,113,142]},{index:.5,rgb:[33,144,141]},{index:.63,rgb:[39,173,129]},{index:.75,rgb:[92,200,99]},{index:.88,rgb:[170,220,50]},{index:1,rgb:[253,231,37]}],inferno:[{index:0,rgb:[0,0,4]},{index:.13,rgb:[31,12,72]},{index:.25,rgb:[85,15,109]},{index:.38,rgb:[136,34,106]},{index:.5,rgb:[186,54,85]},{index:.63,rgb:[227,89,51]},{index:.75,rgb:[249,140,10]},{index:.88,rgb:[249,201,50]},{index:1,rgb:[252,255,164]}],magma:[{index:0,rgb:[0,0,4]},{index:.13,rgb:[28,16,68]},{index:.25,rgb:[79,18,123]},{index:.38,rgb:[129,37,129]},{index:.5,rgb:[181,54,122]},{index:.63,rgb:[229,80,100]},{index:.75,rgb:[251,135,97]},{index:.88,rgb:[254,194,135]},{index:1,rgb:[252,253,191]}],plasma:[{index:0,rgb:[13,8,135]},{index:.13,rgb:[75,3,161]},{index:.25,rgb:[125,3,168]},{index:.38,rgb:[168,34,150]},{index:.5,rgb:[203,70,121]},{index:.63,rgb:[229,107,93]},{index:.75,rgb:[248,148,65]},{index:.88,rgb:[253,195,40]},{index:1,rgb:[240,249,33]}],warm:[{index:0,rgb:[125,0,179]},{index:.13,rgb:[172,0,187]},{index:.25,rgb:[219,0,170]},{index:.38,rgb:[255,0,130]},{index:.5,rgb:[255,63,74]},{index:.63,rgb:[255,123,0]},{index:.75,rgb:[234,176,0]},{index:.88,rgb:[190,228,0]},{index:1,rgb:[147,255,0]}],cool:[{index:0,rgb:[125,0,179]},{index:.13,rgb:[116,0,218]},{index:.25,rgb:[98,74,237]},{index:.38,rgb:[68,146,231]},{index:.5,rgb:[0,204,197]},{index:.63,rgb:[0,247,146]},{index:.75,rgb:[0,255,88]},{index:.88,rgb:[40,255,8]},{index:1,rgb:[147,255,0]}],"rainbow-soft":[{index:0,rgb:[125,0,179]},{index:.1,rgb:[199,0,180]},{index:.2,rgb:[255,0,121]},{index:.3,rgb:[255,108,0]},{index:.4,rgb:[222,194,0]},{index:.5,rgb:[150,255,0]},{index:.6,rgb:[0,255,55]},{index:.7,rgb:[0,246,150]},{index:.8,rgb:[50,167,222]},{index:.9,rgb:[103,51,235]},{index:1,rgb:[124,0,186]}],bathymetry:[{index:0,rgb:[40,26,44]},{index:.13,rgb:[59,49,90]},{index:.25,rgb:[64,76,139]},{index:.38,rgb:[63,110,151]},{index:.5,rgb:[72,142,158]},{index:.63,rgb:[85,174,163]},{index:.75,rgb:[120,206,163]},{index:.88,rgb:[187,230,172]},{index:1,rgb:[253,254,204]}],cdom:[{index:0,rgb:[47,15,62]},{index:.13,rgb:[87,23,86]},{index:.25,rgb:[130,28,99]},{index:.38,rgb:[171,41,96]},{index:.5,rgb:[206,67,86]},{index:.63,rgb:[230,106,84]},{index:.75,rgb:[242,149,103]},{index:.88,rgb:[249,193,135]},{index:1,rgb:[254,237,176]}],chlorophyll:[{index:0,rgb:[18,36,20]},{index:.13,rgb:[25,63,41]},{index:.25,rgb:[24,91,59]},{index:.38,rgb:[13,119,72]},{index:.5,rgb:[18,148,80]},{index:.63,rgb:[80,173,89]},{index:.75,rgb:[132,196,122]},{index:.88,rgb:[175,221,162]},{index:1,rgb:[215,249,208]}],density:[{index:0,rgb:[54,14,36]},{index:.13,rgb:[89,23,80]},{index:.25,rgb:[110,45,132]},{index:.38,rgb:[120,77,178]},{index:.5,rgb:[120,113,213]},{index:.63,rgb:[115,151,228]},{index:.75,rgb:[134,185,227]},{index:.88,rgb:[177,214,227]},{index:1,rgb:[230,241,241]}],"freesurface-blue":[{index:0,rgb:[30,4,110]},{index:.13,rgb:[47,14,176]},{index:.25,rgb:[41,45,236]},{index:.38,rgb:[25,99,212]},{index:.5,rgb:[68,131,200]},{index:.63,rgb:[114,156,197]},{index:.75,rgb:[157,181,203]},{index:.88,rgb:[200,208,216]},{index:1,rgb:[241,237,236]}],"freesurface-red":[{index:0,rgb:[60,9,18]},{index:.13,rgb:[100,17,27]},{index:.25,rgb:[142,20,29]},{index:.38,rgb:[177,43,27]},{index:.5,rgb:[192,87,63]},{index:.63,rgb:[205,125,105]},{index:.75,rgb:[216,162,148]},{index:.88,rgb:[227,199,193]},{index:1,rgb:[241,237,236]}],oxygen:[{index:0,rgb:[64,5,5]},{index:.13,rgb:[106,6,15]},{index:.25,rgb:[144,26,7]},{index:.38,rgb:[168,64,3]},{index:.5,rgb:[188,100,4]},{index:.63,rgb:[206,136,11]},{index:.75,rgb:[220,174,25]},{index:.88,rgb:[231,215,44]},{index:1,rgb:[248,254,105]}],par:[{index:0,rgb:[51,20,24]},{index:.13,rgb:[90,32,35]},{index:.25,rgb:[129,44,34]},{index:.38,rgb:[159,68,25]},{index:.5,rgb:[182,99,19]},{index:.63,rgb:[199,134,22]},{index:.75,rgb:[212,171,35]},{index:.88,rgb:[221,210,54]},{index:1,rgb:[225,253,75]}],phase:[{index:0,rgb:[145,105,18]},{index:.13,rgb:[184,71,38]},{index:.25,rgb:[186,58,115]},{index:.38,rgb:[160,71,185]},{index:.5,rgb:[110,97,218]},{index:.63,rgb:[50,123,164]},{index:.75,rgb:[31,131,110]},{index:.88,rgb:[77,129,34]},{index:1,rgb:[145,105,18]}],salinity:[{index:0,rgb:[42,24,108]},{index:.13,rgb:[33,50,162]},{index:.25,rgb:[15,90,145]},{index:.38,rgb:[40,118,137]},{index:.5,rgb:[59,146,135]},{index:.63,rgb:[79,175,126]},{index:.75,rgb:[120,203,104]},{index:.88,rgb:[193,221,100]},{index:1,rgb:[253,239,154]}],temperature:[{index:0,rgb:[4,35,51]},{index:.13,rgb:[23,51,122]},{index:.25,rgb:[85,59,157]},{index:.38,rgb:[129,79,143]},{index:.5,rgb:[175,95,130]},{index:.63,rgb:[222,112,101]},{index:.75,rgb:[249,146,66]},{index:.88,rgb:[249,196,65]},{index:1,rgb:[232,250,91]}],turbidity:[{index:0,rgb:[34,31,27]},{index:.13,rgb:[65,50,41]},{index:.25,rgb:[98,69,52]},{index:.38,rgb:[131,89,57]},{index:.5,rgb:[161,112,59]},{index:.63,rgb:[185,140,66]},{index:.75,rgb:[202,174,88]},{index:.88,rgb:[216,209,126]},{index:1,rgb:[233,246,171]}],"velocity-blue":[{index:0,rgb:[17,32,64]},{index:.13,rgb:[35,52,116]},{index:.25,rgb:[29,81,156]},{index:.38,rgb:[31,113,162]},{index:.5,rgb:[50,144,169]},{index:.63,rgb:[87,173,176]},{index:.75,rgb:[149,196,189]},{index:.88,rgb:[203,221,211]},{index:1,rgb:[254,251,230]}],"velocity-green":[{index:0,rgb:[23,35,19]},{index:.13,rgb:[24,64,38]},{index:.25,rgb:[11,95,45]},{index:.38,rgb:[39,123,35]},{index:.5,rgb:[95,146,12]},{index:.63,rgb:[152,165,18]},{index:.75,rgb:[201,186,69]},{index:.88,rgb:[233,216,137]},{index:1,rgb:[255,253,205]}],cubehelix:[{index:0,rgb:[0,0,0]},{index:.07,rgb:[22,5,59]},{index:.13,rgb:[60,4,105]},{index:.2,rgb:[109,1,135]},{index:.27,rgb:[161,0,147]},{index:.33,rgb:[210,2,142]},{index:.4,rgb:[251,11,123]},{index:.47,rgb:[255,29,97]},{index:.53,rgb:[255,54,69]},{index:.6,rgb:[255,85,46]},{index:.67,rgb:[255,120,34]},{index:.73,rgb:[255,157,37]},{index:.8,rgb:[241,191,57]},{index:.87,rgb:[224,220,93]},{index:.93,rgb:[218,241,142]},{index:1,rgb:[227,253,198]}]}},function(e,t){e.exports=function(e,t,n){return e*(1-n)+t*n}},function(e,t,n){"use strict";e.exports=function(e,t,n,s){n=n||0,void 0===s&&(s=function(e){for(var t=e.length,n=0,r=0;r<t;++r)n=0|Math.max(n,e[r].length);return n-1}(e));if(0===e.length||s<1)return{cells:[],vertexIds:[],vertexWeights:[]};var l=function(e,t){for(var n=e.length,r=i.mallocUint8(n),o=0;o<n;++o)r[o]=e[o]<t|0;return r}(t,+n),u=function(e,t){for(var n=e.length,a=t*(t+1)/2*n|0,s=i.mallocUint32(2*a),l=0,u=0;u<n;++u)for(var c=e[u],d=(t=c.length,0);d<t;++d)for(var h=0;h<d;++h){var f=c[h],p=c[d];s[l++]=0|Math.min(f,p),s[l++]=0|Math.max(f,p)}o(r(s,[l/2|0,2]));var g=2;for(u=2;u<l;u+=2)s[u-2]===s[u]&&s[u-1]===s[u+1]||(s[g++]=s[u],s[g++]=s[u+1]);return r(s,[g/2|0,2])}(e,s),c=function(e,t,n,o){for(var a=e.data,s=e.shape[0],l=i.mallocDouble(s),u=0,c=0;c<s;++c){var d=a[2*c],h=a[2*c+1];if(n[d]!==n[h]){var f=t[d],p=t[h];a[2*u]=d,a[2*u+1]=h,l[u++]=(p-o)/(p-f)}}return e.shape[0]=u,r(l,[u])}(u,t,l,+n),d=function(e,t){var n=i.mallocInt32(2*t),r=e.shape[0],o=e.data;n[0]=0;for(var a=0,s=0;s<r;++s){var l=o[2*s];if(l!==a){for(n[2*a+1]=s;++a<l;)n[2*a]=s,n[2*a+1]=s;n[2*a]=s}}n[2*a+1]=r;for(;++a<t;)n[2*a]=n[2*a+1]=r;return n}(u,0|t.length),h=a(s)(e,u.data,d,l),f=function(e){for(var t=0|e.shape[0],n=e.data,r=new Array(t),i=0;i<t;++i)r[i]=[n[2*i],n[2*i+1]];return r}(u),p=[].slice.call(c.data,0,c.shape[0]);return i.free(l),i.free(u.data),i.free(c.data),i.free(d),{cells:h,vertexIds:f,vertexWeights:p}};var r=n(216),i=n(112),o=n(1887),a=n(1889)},function(e,t,n){"use strict";var r=n(1888),i={};e.exports=function(e){var t=e.order,n=e.dtype,o=[t,n].join(":"),a=i[o];return a||(i[o]=a=r(t,n)),a(e),e}},function(e,t,n){"use strict";var r=n(112),i=32;function o(e){switch(e){case"uint8":return[r.mallocUint8,r.freeUint8];case"uint16":return[r.mallocUint16,r.freeUint16];case"uint32":return[r.mallocUint32,r.freeUint32];case"int8":return[r.mallocInt8,r.freeInt8];case"int16":return[r.mallocInt16,r.freeInt16];case"int32":return[r.mallocInt32,r.freeInt32];case"float32":return[r.mallocFloat,r.freeFloat];case"float64":return[r.mallocDouble,r.freeDouble];default:return null}}function a(e){for(var t=[],n=0;n<e;++n)t.push("s"+n);for(n=0;n<e;++n)t.push("n"+n);for(n=1;n<e;++n)t.push("d"+n);for(n=1;n<e;++n)t.push("e"+n);for(n=1;n<e;++n)t.push("f"+n);return t}e.exports=function(e,t){var n=["'use strict'"],r=["ndarraySortWrapper",e.join("d"),t].join("");n.push(["function ",r,"(",["array"].join(","),"){"].join(""));for(var s=["data=array.data,offset=array.offset|0,shape=array.shape,stride=array.stride"],l=0;l<e.length;++l)s.push(["s",l,"=stride[",l,"]|0,n",l,"=shape[",l,"]|0"].join(""));var u=new Array(e.length),c=[];for(l=0;l<e.length;++l){0!==(f=e[l])&&(0===c.length?u[f]="1":u[f]=c.join("*"),c.push("n"+f))}var d=-1,h=-1;for(l=0;l<e.length;++l){var f,p=e[l];0!==p&&(d>0?s.push(["d",p,"=s",p,"-d",d,"*n",d].join("")):s.push(["d",p,"=s",p].join("")),d=p),0!==(f=e.length-1-l)&&(h>0?s.push(["e",f,"=s",f,"-e",h,"*n",h,",f",f,"=",u[f],"-f",h,"*n",h].join("")):s.push(["e",f,"=s",f,",f",f,"=",u[f]].join("")),h=f)}n.push("var "+s.join(","));var g=["0","n0-1","data","offset"].concat(a(e.length));n.push(["if(n0<=",i,"){","insertionSort(",g.join(","),")}else{","quickSort(",g.join(","),")}"].join("")),n.push("}return "+r);var v=new Function("insertionSort","quickSort",n.join("\n")),m=function(e,t){var n=["'use strict'"],r=["ndarrayInsertionSort",e.join("d"),t].join(""),i=["left","right","data","offset"].concat(a(e.length)),s=o(t),l=["i,j,cptr,ptr=left*s0+offset"];if(e.length>1){for(var u=[],c=1;c<e.length;++c)l.push("i"+c),u.push("n"+c);s?l.push("scratch=malloc("+u.join("*")+")"):l.push("scratch=new Array("+u.join("*")+")"),l.push("dptr","sptr","a","b")}else l.push("scratch");function d(e){return"generic"===t?["data.get(",e,")"].join(""):["data[",e,"]"].join("")}function h(e,n){return"generic"===t?["data.set(",e,",",n,")"].join(""):["data[",e,"]=",n].join("")}if(n.push(["function ",r,"(",i.join(","),"){var ",l.join(",")].join(""),"for(i=left+1;i<=right;++i){","j=i;ptr+=s0","cptr=ptr"),e.length>1){n.push("dptr=0;sptr=ptr");for(c=e.length-1;c>=0;--c){0!==(f=e[c])&&n.push(["for(i",f,"=0;i",f,"<n",f,";++i",f,"){"].join(""))}n.push("scratch[dptr++]=",d("sptr"));for(c=0;c<e.length;++c){0!==(f=e[c])&&n.push("sptr+=d"+f,"}")}n.push("__g:while(j--\x3eleft){","dptr=0","sptr=cptr-s0");for(c=1;c<e.length;++c)1===c&&n.push("__l:"),n.push(["for(i",c,"=0;i",c,"<n",c,";++i",c,"){"].join(""));n.push(["a=",d("sptr"),"\nb=scratch[dptr]\nif(a<b){break __g}\nif(a>b){break __l}"].join(""));for(c=e.length-1;c>=1;--c)n.push("sptr+=e"+c,"dptr+=f"+c,"}");n.push("dptr=cptr;sptr=cptr-s0");for(c=e.length-1;c>=0;--c){0!==(f=e[c])&&n.push(["for(i",f,"=0;i",f,"<n",f,";++i",f,"){"].join(""))}n.push(h("dptr",d("sptr")));for(c=0;c<e.length;++c){0!==(f=e[c])&&n.push(["dptr+=d",f,";sptr+=d",f].join(""),"}")}n.push("cptr-=s0\n}"),n.push("dptr=cptr;sptr=0");for(c=e.length-1;c>=0;--c){0!==(f=e[c])&&n.push(["for(i",f,"=0;i",f,"<n",f,";++i",f,"){"].join(""))}n.push(h("dptr","scratch[sptr++]"));for(c=0;c<e.length;++c){var f;0!==(f=e[c])&&n.push("dptr+=d"+f,"}")}}else n.push("scratch="+d("ptr"),"while((j--\x3eleft)&&("+d("cptr-s0")+">scratch)){",h("cptr",d("cptr-s0")),"cptr-=s0","}",h("cptr","scratch"));return n.push("}"),e.length>1&&s&&n.push("free(scratch)"),n.push("} return "+r),s?new Function("malloc","free",n.join("\n"))(s[0],s[1]):new Function(n.join("\n"))()}(e,t),y=function(e,t,n){var r=["'use strict'"],s=["ndarrayQuickSort",e.join("d"),t].join(""),l=["left","right","data","offset"].concat(a(e.length)),u=o(t),c=0;r.push(["function ",s,"(",l.join(","),"){"].join(""));var d=["sixth=((right-left+1)/6)|0","index1=left+sixth","index5=right-sixth","index3=(left+right)>>1","index2=index3-sixth","index4=index3+sixth","el1=index1","el2=index2","el3=index3","el4=index4","el5=index5","less=left+1","great=right-1","pivots_are_equal=true","tmp","tmp0","x","y","z","k","ptr0","ptr1","ptr2","comp_pivot1=0","comp_pivot2=0","comp=0"];if(e.length>1){for(var h=[],f=1;f<e.length;++f)h.push("n"+f),d.push("i"+f);for(f=0;f<8;++f)d.push("b_ptr"+f);d.push("ptr3","ptr4","ptr5","ptr6","ptr7","pivot_ptr","ptr_shift","elementSize="+h.join("*")),u?d.push("pivot1=malloc(elementSize)","pivot2=malloc(elementSize)"):d.push("pivot1=new Array(elementSize),pivot2=new Array(elementSize)")}else d.push("pivot1","pivot2");function p(e){return["(offset+",e,"*s0)"].join("")}function g(e){return"generic"===t?["data.get(",e,")"].join(""):["data[",e,"]"].join("")}function v(e,n){return"generic"===t?["data.set(",e,",",n,")"].join(""):["data[",e,"]=",n].join("")}function m(t,n,i){if(1===t.length)r.push("ptr0="+p(t[0]));else for(var o=0;o<t.length;++o)r.push(["b_ptr",o,"=s0*",t[o]].join(""));n&&r.push("pivot_ptr=0"),r.push("ptr_shift=offset");for(o=e.length-1;o>=0;--o){0!==(a=e[o])&&r.push(["for(i",a,"=0;i",a,"<n",a,";++i",a,"){"].join(""))}if(t.length>1)for(o=0;o<t.length;++o)r.push(["ptr",o,"=b_ptr",o,"+ptr_shift"].join(""));r.push(i),n&&r.push("++pivot_ptr");for(o=0;o<e.length;++o){var a;0!==(a=e[o])&&(t.length>1?r.push("ptr_shift+=d"+a):r.push("ptr0+=d"+a),r.push("}"))}}function y(t,n,i,o){if(1===n.length)r.push("ptr0="+p(n[0]));else{for(var a=0;a<n.length;++a)r.push(["b_ptr",a,"=s0*",n[a]].join(""));r.push("ptr_shift=offset")}i&&r.push("pivot_ptr=0"),t&&r.push(t+":");for(a=1;a<e.length;++a)r.push(["for(i",a,"=0;i",a,"<n",a,";++i",a,"){"].join(""));if(n.length>1)for(a=0;a<n.length;++a)r.push(["ptr",a,"=b_ptr",a,"+ptr_shift"].join(""));r.push(o);for(a=e.length-1;a>=1;--a)i&&r.push("pivot_ptr+=f"+a),n.length>1?r.push("ptr_shift+=e"+a):r.push("ptr0+=e"+a),r.push("}")}function b(){e.length>1&&u&&r.push("free(pivot1)","free(pivot2)")}function C(t,n){var i="el"+t,o="el"+n;if(e.length>1){var a="__l"+ ++c;y(a,[i,o],!1,["comp=",g("ptr0"),"-",g("ptr1"),"\n","if(comp>0){tmp0=",i,";",i,"=",o,";",o,"=tmp0;break ",a,"}\n","if(comp<0){break ",a,"}"].join(""))}else r.push(["if(",g(p(i)),">",g(p(o)),"){tmp0=",i,";",i,"=",o,";",o,"=tmp0}"].join(""))}function A(t,n){e.length>1?m([t,n],!1,v("ptr0",g("ptr1"))):r.push(v(p(t),g(p(n))))}function _(t,n,i){if(e.length>1){var o="__l"+ ++c;y(o,[n],!0,[t,"=",g("ptr0"),"-pivot",i,"[pivot_ptr]\n","if(",t,"!==0){break ",o,"}"].join(""))}else r.push([t,"=",g(p(n)),"-pivot",i].join(""))}function I(t,n){e.length>1?m([t,n],!1,["tmp=",g("ptr0"),"\n",v("ptr0",g("ptr1")),"\n",v("ptr1","tmp")].join("")):r.push(["ptr0=",p(t),"\n","ptr1=",p(n),"\n","tmp=",g("ptr0"),"\n",v("ptr0",g("ptr1")),"\n",v("ptr1","tmp")].join(""))}function x(t,n,i){e.length>1?(m([t,n,i],!1,["tmp=",g("ptr0"),"\n",v("ptr0",g("ptr1")),"\n",v("ptr1",g("ptr2")),"\n",v("ptr2","tmp")].join("")),r.push("++"+n,"--"+i)):r.push(["ptr0=",p(t),"\n","ptr1=",p(n),"\n","ptr2=",p(i),"\n","++",n,"\n","--",i,"\n","tmp=",g("ptr0"),"\n",v("ptr0",g("ptr1")),"\n",v("ptr1",g("ptr2")),"\n",v("ptr2","tmp")].join(""))}function w(e,t){I(e,t),r.push("--"+t)}function M(t,n,i){e.length>1?m([t,n],!0,[v("ptr0",g("ptr1")),"\n",v("ptr1",["pivot",i,"[pivot_ptr]"].join(""))].join("")):r.push(v(p(t),g(p(n))),v(p(n),"pivot"+i))}function S(t,n){r.push(["if((",n,"-",t,")<=",i,"){\n","insertionSort(",t,",",n,",data,offset,",a(e.length).join(","),")\n","}else{\n",s,"(",t,",",n,",data,offset,",a(e.length).join(","),")\n","}"].join(""))}function T(t,n,i){e.length>1?(r.push(["__l",++c,":while(true){"].join("")),m([t],!0,["if(",g("ptr0"),"!==pivot",n,"[pivot_ptr]){break __l",c,"}"].join("")),r.push(i,"}")):r.push(["while(",g(p(t)),"===pivot",n,"){",i,"}"].join(""))}return r.push("var "+d.join(",")),C(1,2),C(4,5),C(1,3),C(2,3),C(1,4),C(3,4),C(2,5),C(2,3),C(4,5),e.length>1?m(["el1","el2","el3","el4","el5","index1","index3","index5"],!0,["pivot1[pivot_ptr]=",g("ptr1"),"\n","pivot2[pivot_ptr]=",g("ptr3"),"\n","pivots_are_equal=pivots_are_equal&&(pivot1[pivot_ptr]===pivot2[pivot_ptr])\n","x=",g("ptr0"),"\n","y=",g("ptr2"),"\n","z=",g("ptr4"),"\n",v("ptr5","x"),"\n",v("ptr6","y"),"\n",v("ptr7","z")].join("")):r.push(["pivot1=",g(p("el2")),"\n","pivot2=",g(p("el4")),"\n","pivots_are_equal=pivot1===pivot2\n","x=",g(p("el1")),"\n","y=",g(p("el3")),"\n","z=",g(p("el5")),"\n",v(p("index1"),"x"),"\n",v(p("index3"),"y"),"\n",v(p("index5"),"z")].join("")),A("index2","left"),A("index4","right"),r.push("if(pivots_are_equal){"),r.push("for(k=less;k<=great;++k){"),_("comp","k",1),r.push("if(comp===0){continue}"),r.push("if(comp<0){"),r.push("if(k!==less){"),I("k","less"),r.push("}"),r.push("++less"),r.push("}else{"),r.push("while(true){"),_("comp","great",1),r.push("if(comp>0){"),r.push("great--"),r.push("}else if(comp<0){"),x("k","less","great"),r.push("break"),r.push("}else{"),w("k","great"),r.push("break"),r.push("}"),r.push("}"),r.push("}"),r.push("}"),r.push("}else{"),r.push("for(k=less;k<=great;++k){"),_("comp_pivot1","k",1),r.push("if(comp_pivot1<0){"),r.push("if(k!==less){"),I("k","less"),r.push("}"),r.push("++less"),r.push("}else{"),_("comp_pivot2","k",2),r.push("if(comp_pivot2>0){"),r.push("while(true){"),_("comp","great",2),r.push("if(comp>0){"),r.push("if(--great<k){break}"),r.push("continue"),r.push("}else{"),_("comp","great",1),r.push("if(comp<0){"),x("k","less","great"),r.push("}else{"),w("k","great"),r.push("}"),r.push("break"),r.push("}"),r.push("}"),r.push("}"),r.push("}"),r.push("}"),r.push("}"),M("left","(less-1)",1),M("right","(great+1)",2),S("left","(less-2)"),S("(great+2)","right"),r.push("if(pivots_are_equal){"),b(),r.push("return"),r.push("}"),r.push("if(less<index1&&great>index5){"),T("less",1,"++less"),T("great",2,"--great"),r.push("for(k=less;k<=great;++k){"),_("comp_pivot1","k",1),r.push("if(comp_pivot1===0){"),r.push("if(k!==less){"),I("k","less"),r.push("}"),r.push("++less"),r.push("}else{"),_("comp_pivot2","k",2),r.push("if(comp_pivot2===0){"),r.push("while(true){"),_("comp","great",2),r.push("if(comp===0){"),r.push("if(--great<k){break}"),r.push("continue"),r.push("}else{"),_("comp","great",1),r.push("if(comp<0){"),x("k","less","great"),r.push("}else{"),w("k","great"),r.push("}"),r.push("break"),r.push("}"),r.push("}"),r.push("}"),r.push("}"),r.push("}"),r.push("}"),b(),S("less","great"),r.push("}return "+s),e.length>1&&u?new Function("insertionSort","malloc","free",r.join("\n"))(n,u[0],u[1]):new Function("insertionSort",r.join("\n"))(n)}(e,t,m);return v(m,y)}},function(e,t,n){"use strict";e.exports=function(e){var t=o[e];t||(t=o[e]=function(e){var t=0,n=new Array(e+1);n[0]=[[]];for(var o=1;o<=e;++o)for(var a=n[o]=i(o),s=0;s<a.length;++s)t=Math.max(t,a[o].length);var l=["function B(C,E,i,j){","var a=Math.min(i,j)|0,b=Math.max(i,j)|0,l=C[2*a],h=C[2*a+1];","while(l<h){","var m=(l+h)>>1,v=E[2*m+1];","if(v===b){return m}","if(b<v){h=m}else{l=m+1}","}","return l;","};","function getContour",e,"d(F,E,C,S){","var n=F.length,R=[];","for(var i=0;i<n;++i){var c=F[i],l=c.length;"];function u(e){if(!(e.length<=0)){l.push("R.push(");for(var t=0;t<e.length;++t){var n=e[t];t>0&&l.push(","),l.push("[");for(var r=0;r<n.length;++r){var i=n[r];r>0&&l.push(","),l.push("B(C,E,c[",i[0],"],c[",i[1],"])")}l.push("]")}l.push(");")}}for(o=e+1;o>1;--o){o<e+1&&l.push("else "),l.push("if(l===",o,"){");var c=[];for(s=0;s<o;++s)c.push("(S[c["+s+"]]<<"+s+")");l.push("var M=",c.join("+"),";if(M===0||M===",(1<<o)-1,"){continue}switch(M){");for(a=n[o-1],s=0;s<a.length;++s)l.push("case ",s,":"),u(a[s]),l.push("break;");l.push("}}")}return l.push("}return R;};return getContour",e,"d"),new Function("pool",l.join(""))(r)}(e));return t};var r=n(112),i=n(1890),o={}},function(e,t,n){"use strict";e.exports=function(e){for(var t=1<<e+1,n=new Array(t),r=0;r<t;++r)n[r]=o(e,r);return n};var r=n(1054);function i(e,t,n){for(var r=new Array(e),i=0;i<e;++i)r[i]=0,i===t&&(r[i]+=.5),i===n&&(r[i]+=.5);return r}function o(e,t){if(0===t||t===(1<<e+1)-1)return[];for(var n=[],o=[],a=0;a<=e;++a)if(t&1<<a){n.push(i(e,a-1,a-1)),o.push(null);for(var s=0;s<=e;++s)~t&1<<s&&(n.push(i(e,a-1,s-1)),o.push([a,s]))}var l=r(n),u=[];e:for(a=0;a<l.length;++a){var c=l[a],d=[];for(s=0;s<c.length;++s){if(!o[c[s]])continue e;d.push(o[c[s]].slice())}u.push(d)}return u}},function(e,t,n){"use strict";e.exports=function(e){for(var t=0,n=0,r=1;r<e.length;++r)e[r][0]<e[t][0]&&(t=r),e[r][0]>e[n][0]&&(n=r);return t<n?[[t],[n]]:t>n?[[n],[t]]:[[t]]}},function(e,t,n){"use strict";e.exports=function(e){var t=r(e),n=t.length;if(n<=2)return[];for(var i=new Array(n),o=t[n-1],a=0;a<n;++a){var s=t[a];i[a]=[o,s],o=s}return i};var r=n(1893)},function(e,t,n){"use strict";e.exports=function(e){var t=e.length;if(t<3){for(var n=new Array(t),i=0;i<t;++i)n[i]=i;return 2===t&&e[0][0]===e[1][0]&&e[0][1]===e[1][1]?[0]:n}var o=new Array(t);for(i=0;i<t;++i)o[i]=i;o.sort((function(t,n){var r=e[t][0]-e[n][0];return r||e[t][1]-e[n][1]}));var a=[o[0],o[1]],s=[o[0],o[1]];for(i=2;i<t;++i){for(var l=o[i],u=e[l],c=a.length;c>1&&r(e[a[c-2]],e[a[c-1]],u)<=0;)c-=1,a.pop();for(a.push(l),c=s.length;c>1&&r(e[s[c-2]],e[s[c-1]],u)>=0;)c-=1,s.pop();s.push(l)}n=new Array(s.length+a.length-2);for(var d=0,h=(i=0,a.length);i<h;++i)n[d++]=a[i];for(var f=s.length-2;f>0;--f)n[d++]=s[f];return n};var r=n(150)[3]},function(e,t,n){"use strict";e.exports=function(e,t){try{return r(e,!0)}catch(a){var n=i(e);if(n.length<=t)return[];var o=function(e,t){for(var n=e.length,r=new Array(n),i=0;i<t.length;++i)r[i]=e[t[i]];var o=t.length;for(i=0;i<n;++i)t.indexOf(i)<0&&(r[o++]=e[i]);return r}(e,n);return function(e,t){for(var n=e.length,r=t.length,i=0;i<n;++i)for(var o=e[i],a=0;a<o.length;++a){var s=o[a];if(s<r)o[a]=t[s];else{s-=r;for(var l=0;l<r;++l)s>=t[l]&&(s+=1);o[a]=s}}return e}(r(o,!0),n)}};var r=n(1055),i=n(1896)},function(e,t,n){"use strict";var r=n(215),i=n(1044);function o(e,t){var n=e.length,r=e.length-t.length,i=Math.min;if(r)return r;switch(n){case 0:return 0;case 1:return e[0]-t[0];case 2:return(s=e[0]+e[1]-t[0]-t[1])||i(e[0],e[1])-i(t[0],t[1]);case 3:var o=e[0]+e[1],a=t[0]+t[1];if(s=o+e[2]-(a+t[2]))return s;var s,l=i(e[0],e[1]),u=i(t[0],t[1]);return(s=i(l,e[2])-i(u,t[2]))||i(l+e[2],o)-i(u+t[2],a);default:var c=e.slice(0);c.sort();var d=t.slice(0);d.sort();for(var h=0;h<n;++h)if(r=c[h]-d[h])return r;return 0}}function a(e,t){return o(e[0],t[0])}function s(e,t){if(t){for(var n=e.length,r=new Array(n),i=0;i<n;++i)r[i]=[e[i],t[i]];r.sort(a);for(i=0;i<n;++i)e[i]=r[i][0],t[i]=r[i][1];return e}return e.sort(o),e}function l(e){if(0===e.length)return[];for(var t=1,n=e.length,r=1;r<n;++r){var i=e[r];if(o(i,e[r-1])){if(r===t){t++;continue}e[t++]=i}}return e.length=t,e}function u(e,t){for(var n=0,r=e.length-1,i=-1;n<=r;){var a=n+r>>1,s=o(e[a],t);s<=0?(0===s&&(i=a),n=a+1):s>0&&(r=a-1)}return i}function c(e,t){for(var n=new Array(e.length),i=0,a=n.length;i<a;++i)n[i]=[];for(var s=[],l=(i=0,t.length);i<l;++i)for(var c=t[i],d=c.length,h=1,f=1<<d;h<f;++h){s.length=r.popCount(h);for(var p=0,g=0;g<d;++g)h&1<<g&&(s[p++]=c[g]);var v=u(e,s);if(!(v<0))for(;n[v++].push(i),!(v>=e.length||0!==o(e[v],s)););}return n}function d(e,t){if(t<0)return[];for(var n=[],i=(1<<t+1)-1,o=0;o<e.length;++o)for(var a=e[o],l=i;l<1<<a.length;l=r.nextCombination(l)){for(var u=new Array(t+1),c=0,d=0;d<a.length;++d)l&1<<d&&(u[c++]=a[d]);n.push(u)}return s(n)}t.dimension=function(e){for(var t=0,n=Math.max,r=0,i=e.length;r<i;++r)t=n(t,e[r].length);return t-1},t.countVertices=function(e){for(var t=-1,n=Math.max,r=0,i=e.length;r<i;++r)for(var o=e[r],a=0,s=o.length;a<s;++a)t=n(t,o[a]);return t+1},t.cloneCells=function(e){for(var t=new Array(e.length),n=0,r=e.length;n<r;++n)t[n]=e[n].slice(0);return t},t.compareCells=o,t.normalize=s,t.unique=l,t.findCell=u,t.incidence=c,t.dual=function(e,t){if(!t)return c(l(d(e,0)),e);for(var n=new Array(t),r=0;r<t;++r)n[r]=[];r=0;for(var i=e.length;r<i;++r)for(var o=e[r],a=0,s=o.length;a<s;++a)n[o[a]].push(r);return n},t.explode=function(e){for(var t=[],n=0,r=e.length;n<r;++n)for(var i=e[n],o=0|i.length,a=1,l=1<<o;a<l;++a){for(var u=[],c=0;c<o;++c)a>>>c&1&&u.push(i[c]);t.push(u)}return s(t)},t.skeleton=d,t.boundary=function(e){for(var t=[],n=0,r=e.length;n<r;++n)for(var i=e[n],o=0,a=i.length;o<a;++o){for(var l=new Array(i.length-1),u=0,c=0;u<a;++u)u!==o&&(l[c++]=i[u]);t.push(l)}return s(t)},t.connectedComponents=function(e,t){return t?function(e,t){for(var n=new i(t),r=0;r<e.length;++r)for(var o=e[r],a=0;a<o.length;++a)for(var s=a+1;s<o.length;++s)n.link(o[a],o[s]);var l=[],u=n.ranks;for(r=0;r<u.length;++r)u[r]=-1;for(r=0;r<e.length;++r){var c=n.find(e[r][0]);u[c]<0?(u[c]=l.length,l.push([e[r].slice(0)])):l[u[c]].push(e[r].slice(0))}return l}(e,t):function(e){for(var t=l(s(d(e,0))),n=new i(t.length),r=0;r<e.length;++r)for(var o=e[r],a=0;a<o.length;++a)for(var c=u(t,[o[a]]),h=a+1;h<o.length;++h)n.link(c,u(t,[o[h]]));var f=[],p=n.ranks;for(r=0;r<p.length;++r)p[r]=-1;for(r=0;r<e.length;++r){var g=n.find(u(t,[e[r][0]]));p[g]<0?(p[g]=f.length,f.push([e[r].slice(0)])):f[p[g]].push(e[r].slice(0))}return f}(e)}},function(e,t,n){"use strict";e.exports=function(e){var t=e.length;if(0===t)return[];if(1===t)return[0];for(var n=e[0].length,r=[e[0]],o=[0],a=1;a<t;++a)if(r.push(e[a]),i(r,n)){if(o.push(a),o.length===n+1)return o}else r.pop();return o};var r=n(150);function i(e,t){for(var n=new Array(t+1),i=0;i<e.length;++i)n[i]=e[i];for(i=0;i<=e.length;++i){for(var o=e.length;o<=t;++o){for(var a=new Array(t),s=0;s<t;++s)a[s]=Math.pow(o+1-i,s);n[o]=a}if(r.apply(void 0,n))return!0}return!1}},function(e,t,n){var r=n(217),i=r(["precision highp float;\n#define GLSLIFY 1\n\nattribute vec3 position, normal;\nattribute vec4 color;\nattribute vec2 uv;\n\nuniform mat4 model\n           , view\n           , projection\n           , inverseModel;\nuniform vec3 eyePosition\n           , lightPosition;\n\nvarying vec3 f_normal\n           , f_lightDirection\n           , f_eyeDirection\n           , f_data;\nvarying vec4 f_color;\nvarying vec2 f_uv;\n\nvec4 project(vec3 p) {\n  return projection * view * model * vec4(p, 1.0);\n}\n\nvoid main() {\n  gl_Position      = project(position);\n\n  //Lighting geometry parameters\n  vec4 cameraCoordinate = view * vec4(position , 1.0);\n  cameraCoordinate.xyz /= cameraCoordinate.w;\n  f_lightDirection = lightPosition - cameraCoordinate.xyz;\n  f_eyeDirection   = eyePosition - cameraCoordinate.xyz;\n  f_normal  = normalize((vec4(normal, 0.0) * inverseModel).xyz);\n\n  f_color          = color;\n  f_data           = position;\n  f_uv             = uv;\n}\n"]),o=r(["#extension GL_OES_standard_derivatives : enable\n\nprecision highp float;\n#define GLSLIFY 1\n\nfloat beckmannDistribution(float x, float roughness) {\n  float NdotH = max(x, 0.0001);\n  float cos2Alpha = NdotH * NdotH;\n  float tan2Alpha = (cos2Alpha - 1.0) / cos2Alpha;\n  float roughness2 = roughness * roughness;\n  float denom = 3.141592653589793 * roughness2 * cos2Alpha * cos2Alpha;\n  return exp(tan2Alpha / roughness2) / denom;\n}\n\nfloat cookTorranceSpecular(\n  vec3 lightDirection,\n  vec3 viewDirection,\n  vec3 surfaceNormal,\n  float roughness,\n  float fresnel) {\n\n  float VdotN = max(dot(viewDirection, surfaceNormal), 0.0);\n  float LdotN = max(dot(lightDirection, surfaceNormal), 0.0);\n\n  //Half angle vector\n  vec3 H = normalize(lightDirection + viewDirection);\n\n  //Geometric term\n  float NdotH = max(dot(surfaceNormal, H), 0.0);\n  float VdotH = max(dot(viewDirection, H), 0.000001);\n  float LdotH = max(dot(lightDirection, H), 0.000001);\n  float G1 = (2.0 * NdotH * VdotN) / VdotH;\n  float G2 = (2.0 * NdotH * LdotN) / LdotH;\n  float G = min(1.0, min(G1, G2));\n  \n  //Distribution term\n  float D = beckmannDistribution(NdotH, roughness);\n\n  //Fresnel term\n  float F = pow(1.0 - VdotN, fresnel);\n\n  //Multiply terms and done\n  return  G * F * D / max(3.14159265 * VdotN, 0.000001);\n}\n\n//#pragma glslify: beckmann = require(glsl-specular-beckmann) // used in gl-surface3d\n\nbool outOfRange(float a, float b, float p) {\n  return ((p > max(a, b)) || \n          (p < min(a, b)));\n}\n\nbool outOfRange(vec2 a, vec2 b, vec2 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y));\n}\n\nbool outOfRange(vec3 a, vec3 b, vec3 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y) ||\n          outOfRange(a.z, b.z, p.z));\n}\n\nbool outOfRange(vec4 a, vec4 b, vec4 p) {\n  return outOfRange(a.xyz, b.xyz, p.xyz);\n}\n\nuniform vec3 clipBounds[2];\nuniform float roughness\n            , fresnel\n            , kambient\n            , kdiffuse\n            , kspecular;\nuniform sampler2D texture;\n\nvarying vec3 f_normal\n           , f_lightDirection\n           , f_eyeDirection\n           , f_data;\nvarying vec4 f_color;\nvarying vec2 f_uv;\n\nvoid main() {\n  if (f_color.a == 0.0 ||\n    outOfRange(clipBounds[0], clipBounds[1], f_data)\n  ) discard;\n\n  vec3 N = normalize(f_normal);\n  vec3 L = normalize(f_lightDirection);\n  vec3 V = normalize(f_eyeDirection);\n\n  if(gl_FrontFacing) {\n    N = -N;\n  }\n\n  float specular = min(1.0, max(0.0, cookTorranceSpecular(L, V, N, roughness, fresnel)));\n  //float specular = max(0.0, beckmann(L, V, N, roughness)); // used in gl-surface3d\n\n  float diffuse  = min(kambient + kdiffuse * max(dot(N, L), 0.0), 1.0);\n\n  vec4 surfaceColor = vec4(f_color.rgb, 1.0) * texture2D(texture, f_uv);\n  vec4 litColor = surfaceColor.a * vec4(diffuse * surfaceColor.rgb + kspecular * vec3(1,1,1) * specular,  1.0);\n\n  gl_FragColor = litColor * f_color.a;\n}\n"]),a=r(["precision highp float;\n#define GLSLIFY 1\n\nattribute vec3 position;\nattribute vec4 color;\nattribute vec2 uv;\n\nuniform mat4 model, view, projection;\n\nvarying vec4 f_color;\nvarying vec3 f_data;\nvarying vec2 f_uv;\n\nvoid main() {\n  gl_Position = projection * view * model * vec4(position, 1.0);\n  f_color = color;\n  f_data  = position;\n  f_uv    = uv;\n}"]),s=r(["precision highp float;\n#define GLSLIFY 1\n\nbool outOfRange(float a, float b, float p) {\n  return ((p > max(a, b)) || \n          (p < min(a, b)));\n}\n\nbool outOfRange(vec2 a, vec2 b, vec2 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y));\n}\n\nbool outOfRange(vec3 a, vec3 b, vec3 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y) ||\n          outOfRange(a.z, b.z, p.z));\n}\n\nbool outOfRange(vec4 a, vec4 b, vec4 p) {\n  return outOfRange(a.xyz, b.xyz, p.xyz);\n}\n\nuniform vec3 clipBounds[2];\nuniform sampler2D texture;\nuniform float opacity;\n\nvarying vec4 f_color;\nvarying vec3 f_data;\nvarying vec2 f_uv;\n\nvoid main() {\n  if (outOfRange(clipBounds[0], clipBounds[1], f_data)) discard;\n\n  gl_FragColor = f_color * texture2D(texture, f_uv) * opacity;\n}"]),l=r(["precision highp float;\n#define GLSLIFY 1\n\nbool outOfRange(float a, float b, float p) {\n  return ((p > max(a, b)) || \n          (p < min(a, b)));\n}\n\nbool outOfRange(vec2 a, vec2 b, vec2 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y));\n}\n\nbool outOfRange(vec3 a, vec3 b, vec3 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y) ||\n          outOfRange(a.z, b.z, p.z));\n}\n\nbool outOfRange(vec4 a, vec4 b, vec4 p) {\n  return outOfRange(a.xyz, b.xyz, p.xyz);\n}\n\nattribute vec3 position;\nattribute vec4 color;\nattribute vec2 uv;\nattribute float pointSize;\n\nuniform mat4 model, view, projection;\nuniform vec3 clipBounds[2];\n\nvarying vec4 f_color;\nvarying vec2 f_uv;\n\nvoid main() {\n  if (outOfRange(clipBounds[0], clipBounds[1], position)) {\n\n    gl_Position = vec4(0.0, 0.0 ,0.0 ,0.0);\n  } else {\n    gl_Position = projection * view * model * vec4(position, 1.0);\n  }\n  gl_PointSize = pointSize;\n  f_color = color;\n  f_uv = uv;\n}"]),u=r(["precision highp float;\n#define GLSLIFY 1\n\nuniform sampler2D texture;\nuniform float opacity;\n\nvarying vec4 f_color;\nvarying vec2 f_uv;\n\nvoid main() {\n  vec2 pointR = gl_PointCoord.xy - vec2(0.5, 0.5);\n  if(dot(pointR, pointR) > 0.25) {\n    discard;\n  }\n  gl_FragColor = f_color * texture2D(texture, f_uv) * opacity;\n}"]),c=r(["precision highp float;\n#define GLSLIFY 1\n\nattribute vec3 position;\nattribute vec4 id;\n\nuniform mat4 model, view, projection;\n\nvarying vec3 f_position;\nvarying vec4 f_id;\n\nvoid main() {\n  gl_Position = projection * view * model * vec4(position, 1.0);\n  f_id        = id;\n  f_position  = position;\n}"]),d=r(["precision highp float;\n#define GLSLIFY 1\n\nbool outOfRange(float a, float b, float p) {\n  return ((p > max(a, b)) || \n          (p < min(a, b)));\n}\n\nbool outOfRange(vec2 a, vec2 b, vec2 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y));\n}\n\nbool outOfRange(vec3 a, vec3 b, vec3 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y) ||\n          outOfRange(a.z, b.z, p.z));\n}\n\nbool outOfRange(vec4 a, vec4 b, vec4 p) {\n  return outOfRange(a.xyz, b.xyz, p.xyz);\n}\n\nuniform vec3  clipBounds[2];\nuniform float pickId;\n\nvarying vec3 f_position;\nvarying vec4 f_id;\n\nvoid main() {\n  if (outOfRange(clipBounds[0], clipBounds[1], f_position)) discard;\n\n  gl_FragColor = vec4(pickId, f_id.xyz);\n}"]),h=r(["precision highp float;\n#define GLSLIFY 1\n\nbool outOfRange(float a, float b, float p) {\n  return ((p > max(a, b)) || \n          (p < min(a, b)));\n}\n\nbool outOfRange(vec2 a, vec2 b, vec2 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y));\n}\n\nbool outOfRange(vec3 a, vec3 b, vec3 p) {\n  return (outOfRange(a.x, b.x, p.x) ||\n          outOfRange(a.y, b.y, p.y) ||\n          outOfRange(a.z, b.z, p.z));\n}\n\nbool outOfRange(vec4 a, vec4 b, vec4 p) {\n  return outOfRange(a.xyz, b.xyz, p.xyz);\n}\n\nattribute vec3  position;\nattribute float pointSize;\nattribute vec4  id;\n\nuniform mat4 model, view, projection;\nuniform vec3 clipBounds[2];\n\nvarying vec3 f_position;\nvarying vec4 f_id;\n\nvoid main() {\n  if (outOfRange(clipBounds[0], clipBounds[1], position)) {\n\n    gl_Position = vec4(0.0, 0.0, 0.0, 0.0);\n  } else {\n    gl_Position  = projection * view * model * vec4(position, 1.0);\n    gl_PointSize = pointSize;\n  }\n  f_id         = id;\n  f_position   = position;\n}"]),f=r(["precision highp float;\n#define GLSLIFY 1\n\nattribute vec3 position;\n\nuniform mat4 model, view, projection;\n\nvoid main() {\n  gl_Position = projection * view * model * vec4(position, 1.0);\n}"]),p=r(["precision highp float;\n#define GLSLIFY 1\n\nuniform vec3 contourColor;\n\nvoid main() {\n  gl_FragColor = vec4(contourColor, 1.0);\n}\n"]);t.meshShader={vertex:i,fragment:o,attributes:[{name:"position",type:"vec3"},{name:"normal",type:"vec3"},{name:"color",type:"vec4"},{name:"uv",type:"vec2"}]},t.wireShader={vertex:a,fragment:s,attributes:[{name:"position",type:"vec3"},{name:"color",type:"vec4"},{name:"uv",type:"vec2"}]},t.pointShader={vertex:l,fragment:u,attributes:[{name:"position",type:"vec3"},{name:"color",type:"vec4"},{name:"uv",type:"vec2"},{name:"pointSize",type:"float"}]},t.pickShader={vertex:c,fragment:d,attributes:[{name:"position",type:"vec3"},{name:"id",type:"vec4"}]},t.pointPickShader={vertex:h,fragment:d,attributes:[{name:"position",type:"vec3"},{name:"pointSize",type:"float"},{name:"id",type:"vec4"}]},t.contourShader={vertex:f,fragment:p,attributes:[{name:"position",type:"vec3"}]}},function(e,t,n){"use strict";var r=n(1899),i=n(1902);function o(e,t){for(var n=[0,0,0,0],r=0;r<4;++r)for(var i=0;i<4;++i)n[i]+=e[4*r+i]*t[r];return n}function a(e,t,n,r,i){for(var a=o(r,o(n,o(t,[e[0],e[1],e[2],1]))),s=0;s<3;++s)a[s]/=a[3];return[.5*i[0]*(1+a[0]),.5*i[1]*(1-a[1])]}function s(e,t){for(var n=[0,0,0],r=0;r<e.length;++r)for(var i=e[r],o=t[r],a=0;a<3;++a)n[a]+=o*i[a];return n}e.exports=function(e,t,n,o,l,u){if(1===e.length)return[0,e[0].slice()];for(var c=new Array(e.length),d=0;d<e.length;++d)c[d]=a(e[d],n,o,l,u);var h=0,f=1/0;for(d=0;d<c.length;++d){for(var p=0,g=0;g<2;++g)p+=Math.pow(c[d][g]-t[g],2);p<f&&(f=p,h=d)}var v=function(e,t){if(2===e.length){for(var n=0,o=0,a=0;a<2;++a)n+=Math.pow(t[a]-e[0][a],2),o+=Math.pow(t[a]-e[1][a],2);return n=Math.sqrt(n),o=Math.sqrt(o),n+o<1e-6?[1,0]:[o/(n+o),n/(o+n)]}if(3===e.length){var s=[0,0];return i(e[0],e[1],e[2],t,s),r(e,s)}return[]}(c,t),m=0;for(d=0;d<3;++d){if(v[d]<-.001||v[d]>1.0001)return null;m+=v[d]}if(Math.abs(m-1)>.001)return null;return[h,s(e,v),v]}},function(e,t,n){"use strict";e.exports=function(e,t){for(var n=t.length,o=new Array(n+1),a=0;a<n;++a){for(var s=new Array(n+1),l=0;l<=n;++l)s[l]=e[l][a];o[a]=s}o[n]=new Array(n+1);for(a=0;a<=n;++a)o[n][a]=1;var u=new Array(n+1);for(a=0;a<n;++a)u[a]=t[a];u[n]=1;var c=r(o,u),d=i(c[n+1]);0===d&&(d=1);var h=new Array(n+1);for(a=0;a<=n;++a)h[a]=i(c[a])/d;return h};var r=n(1056);function i(e){for(var t=0,n=0;n<e.length;++n)t+=e[n];return t}},function(e,t,n){"use strict";var r=n(257),i=n(197),o=n(396),a=n(1901);function s(e,t){for(var n=new Array(e.length-1),r=1;r<e.length;++r)for(var i=n[r-1]=new Array(e.length-1),o=0,a=0;o<e.length;++o)o!==t&&(i[a++]=e[r][o]);return n}function l(e){for(var t=new Array(e),n=0;n<e;++n){t[n]=new Array(e);for(var r=0;r<e;++r)t[n][r]=["m[",n,"][",r,"]"].join("")}return t}function u(e){if(2===e.length)return["sum(prod(",e[0][0],",",e[1][1],"),prod(-",e[0][1],",",e[1][0],"))"].join("");for(var t=[],n=0;n<e.length;++n)t.push(["scale(",u(s(e,n)),",",(r=n,1&r?"-":""),e[0][n],")"].join(""));return function e(t){if(1===t.length)return t[0];if(2===t.length)return["sum(",t[0],",",t[1],")"].join("");var n=t.length>>1;return["sum(",e(t.slice(0,n)),",",e(t.slice(n)),")"].join("")}(t);var r}function c(e){return new Function("sum","scale","prod","compress",["function robustDeterminant",e,"(m){return compress(",u(l(e)),")};return robustDeterminant",e].join(""))(i,o,r,a)}var d=[function(){return[0]},function(e){return[e[0][0]]}];!function(){for(;d.length<6;)d.push(c(d.length));for(var t=[],n=["function robustDeterminant(m){switch(m.length){"],r=0;r<6;++r)t.push("det"+r),n.push("case ",r,":return det",r,"(m);");n.push("}var det=CACHE[m.length];if(!det)det=CACHE[m.length]=gen(m.length);return det(m);}return robustDeterminant"),t.push("CACHE","gen",n.join(""));var i=Function.apply(void 0,t);for(e.exports=i.apply(void 0,d.concat([d,c])),r=0;r<d.length;++r)e.exports[r]=d[r]}()},function(e,t,n){"use strict";e.exports=function(e){for(var t=e.length,n=e[e.length-1],r=t,i=t-2;i>=0;--i){var o=n,a=e[i];(l=a-((n=o+a)-o))&&(e[--r]=n,n=l)}var s=0;for(i=r;i<t;++i){var l;o=e[i];(l=(a=n)-((n=o+a)-o))&&(e[s++]=l)}return e[s++]=n,e.length=s,e}},function(e,t,n){"use strict";var r=new Float64Array(4),i=new Float64Array(4),o=new Float64Array(4);e.exports=function(e,t,n,a,s){r.length<a.length&&(r=new Float64Array(a.length),i=new Float64Array(a.length),o=new Float64Array(a.length));for(var l=0;l<a.length;++l)r[l]=e[l]-a[l],i[l]=t[l]-e[l],o[l]=n[l]-e[l];var u=0,c=0,d=0,h=0,f=0,p=0;for(l=0;l<a.length;++l){var g=i[l],v=o[l],m=r[l];u+=g*g,c+=g*v,d+=v*v,h+=m*g,f+=m*v,p+=m*m}var y,b,C,A,_,I=Math.abs(u*d-c*c),x=c*f-d*h,w=c*h-u*f;if(x+w<=I)if(x<0)w<0&&h<0?(w=0,-h>=u?(x=1,y=u+2*h+p):y=h*(x=-h/u)+p):(x=0,f>=0?(w=0,y=p):-f>=d?(w=1,y=d+2*f+p):y=f*(w=-f/d)+p);else if(w<0)w=0,h>=0?(x=0,y=p):-h>=u?(x=1,y=u+2*h+p):y=h*(x=-h/u)+p;else{var M=1/I;y=(x*=M)*(u*x+c*(w*=M)+2*h)+w*(c*x+d*w+2*f)+p}else x<0?(C=d+f)>(b=c+h)?(A=C-b)>=(_=u-2*c+d)?(x=1,w=0,y=u+2*h+p):y=(x=A/_)*(u*x+c*(w=1-x)+2*h)+w*(c*x+d*w+2*f)+p:(x=0,C<=0?(w=1,y=d+2*f+p):f>=0?(w=0,y=p):y=f*(w=-f/d)+p):w<0?(C=u+h)>(b=c+f)?(A=C-b)>=(_=u-2*c+d)?(w=1,x=0,y=d+2*f+p):y=(x=1-(w=A/_))*(u*x+c*w+2*h)+w*(c*x+d*w+2*f)+p:(w=0,C<=0?(x=1,y=u+2*h+p):h>=0?(x=0,y=p):y=h*(x=-h/u)+p):(A=d+f-c-h)<=0?(x=0,w=1,y=d+2*f+p):A>=(_=u-2*c+d)?(x=1,w=0,y=u+2*h+p):y=(x=A/_)*(u*x+c*(w=1-x)+2*h)+w*(c*x+d*w+2*f)+p;var S=1-x-w;for(l=0;l<a.length;++l)s[l]=S*e[l]+x*t[l]+w*n[l];return y<0?0:y}},function(e,t,n){"use strict";var r=n(1904),i=n(1906);e.exports=function(e){var t;Array.isArray(e)&&e.raw&&(e=String.raw.apply(null,arguments));var n=r(e);return n.space?((t=Array(3))[0]=Math.min(Math.max(n.values[0],0),255),t[1]=Math.min(Math.max(n.values[1],0),255),t[2]=Math.min(Math.max(n.values[2],0),255),"h"===n.space[0]&&(t=i.rgb(t)),t.push(Math.min(Math.max(n.alpha,0),1)),t):[]}},function(e,t,n){"use strict";var r=n(1905);e.exports=function(e){var t,n,o=[],a=1;if("string"==typeof e)if(r[e])o=r[e].slice(),n="rgb";else if("transparent"===e)a=0,n="rgb",o=[0,0,0];else if(/^#[A-Fa-f0-9]+$/.test(e)){var s=(c=e.slice(1)).length;a=1,s<=4?(o=[parseInt(c[0]+c[0],16),parseInt(c[1]+c[1],16),parseInt(c[2]+c[2],16)],4===s&&(a=parseInt(c[3]+c[3],16)/255)):(o=[parseInt(c[0]+c[1],16),parseInt(c[2]+c[3],16),parseInt(c[4]+c[5],16)],8===s&&(a=parseInt(c[6]+c[7],16)/255)),o[0]||(o[0]=0),o[1]||(o[1]=0),o[2]||(o[2]=0),n="rgb"}else if(t=/^((?:rgb|hs[lvb]|hwb|cmyk?|xy[zy]|gray|lab|lchu?v?|[ly]uv|lms)a?)\s*\(([^\)]*)\)/.exec(e)){var l=t[1],u="rgb"===l,c=l.replace(/a$/,"");n=c;s="cmyk"===c?4:"gray"===c?1:3;o=t[2].trim().split(/\s*[,\/]\s*|\s+/).map((function(e,t){if(/%$/.test(e))return t===s?parseFloat(e)/100:"rgb"===c?255*parseFloat(e)/100:parseFloat(e);if("h"===c[t]){if(/deg$/.test(e))return parseFloat(e);if(void 0!==i[e])return i[e]}return parseFloat(e)})),l===c&&o.push(1),a=u||void 0===o[s]?1:o[s],o=o.slice(0,s)}else e.length>10&&/[0-9](?:\s|\/)/.test(e)&&(o=e.match(/([0-9]+)/g).map((function(e){return parseFloat(e)})),n=e.match(/([a-z])/gi).join("").toLowerCase());else isNaN(e)?Array.isArray(e)||e.length?(o=[e[0],e[1],e[2]],n="rgb",a=4===e.length?e[3]:1):e instanceof Object&&(null!=e.r||null!=e.red||null!=e.R?(n="rgb",o=[e.r||e.red||e.R||0,e.g||e.green||e.G||0,e.b||e.blue||e.B||0]):(n="hsl",o=[e.h||e.hue||e.H||0,e.s||e.saturation||e.S||0,e.l||e.lightness||e.L||e.b||e.brightness]),a=e.a||e.alpha||e.opacity||1,null!=e.opacity&&(a/=100)):(n="rgb",o=[e>>>16,(65280&e)>>>8,255&e]);return{space:n,values:o,alpha:a}};var i={red:0,orange:60,yellow:120,green:180,blue:240,purple:300}},function(e,t,n){"use strict";e.exports={aliceblue:[240,248,255],antiquewhite:[250,235,215],aqua:[0,255,255],aquamarine:[127,255,212],azure:[240,255,255],beige:[245,245,220],bisque:[255,228,196],black:[0,0,0],blanchedalmond:[255,235,205],blue:[0,0,255],blueviolet:[138,43,226],brown:[165,42,42],burlywood:[222,184,135],cadetblue:[95,158,160],chartreuse:[127,255,0],chocolate:[210,105,30],coral:[255,127,80],cornflowerblue:[100,149,237],cornsilk:[255,248,220],crimson:[220,20,60],cyan:[0,255,255],darkblue:[0,0,139],darkcyan:[0,139,139],darkgoldenrod:[184,134,11],darkgray:[169,169,169],darkgreen:[0,100,0],darkgrey:[169,169,169],darkkhaki:[189,183,107],darkmagenta:[139,0,139],darkolivegreen:[85,107,47],darkorange:[255,140,0],darkorchid:[153,50,204],darkred:[139,0,0],darksalmon:[233,150,122],darkseagreen:[143,188,143],darkslateblue:[72,61,139],darkslategray:[47,79,79],darkslategrey:[47,79,79],darkturquoise:[0,206,209],darkviolet:[148,0,211],deeppink:[255,20,147],deepskyblue:[0,191,255],dimgray:[105,105,105],dimgrey:[105,105,105],dodgerblue:[30,144,255],firebrick:[178,34,34],floralwhite:[255,250,240],forestgreen:[34,139,34],fuchsia:[255,0,255],gainsboro:[220,220,220],ghostwhite:[248,248,255],gold:[255,215,0],goldenrod:[218,165,32],gray:[128,128,128],green:[0,128,0],greenyellow:[173,255,47],grey:[128,128,128],honeydew:[240,255,240],hotpink:[255,105,180],indianred:[205,92,92],indigo:[75,0,130],ivory:[255,255,240],khaki:[240,230,140],lavender:[230,230,250],lavenderblush:[255,240,245],lawngreen:[124,252,0],lemonchiffon:[255,250,205],lightblue:[173,216,230],lightcoral:[240,128,128],lightcyan:[224,255,255],lightgoldenrodyellow:[250,250,210],lightgray:[211,211,211],lightgreen:[144,238,144],lightgrey:[211,211,211],lightpink:[255,182,193],lightsalmon:[255,160,122],lightseagreen:[32,178,170],lightskyblue:[135,206,250],lightslategray:[119,136,153],lightslategrey:[119,136,153],lightsteelblue:[176,196,222],lightyellow:[255,255,224],lime:[0,255,0],limegreen:[50,205,50],linen:[250,240,230],magenta:[255,0,255],maroon:[128,0,0],mediumaquamarine:[102,205,170],mediumblue:[0,0,205],mediumorchid:[186,85,211],mediumpurple:[147,112,219],mediumseagreen:[60,179,113],mediumslateblue:[123,104,238],mediumspringgreen:[0,250,154],mediumturquoise:[72,209,204],mediumvioletred:[199,21,133],midnightblue:[25,25,112],mintcream:[245,255,250],mistyrose:[255,228,225],moccasin:[255,228,181],navajowhite:[255,222,173],navy:[0,0,128],oldlace:[253,245,230],olive:[128,128,0],olivedrab:[107,142,35],orange:[255,165,0],orangered:[255,69,0],orchid:[218,112,214],palegoldenrod:[238,232,170],palegreen:[152,251,152],paleturquoise:[175,238,238],palevioletred:[219,112,147],papayawhip:[255,239,213],peachpuff:[255,218,185],peru:[205,133,63],pink:[255,192,203],plum:[221,160,221],powderblue:[176,224,230],purple:[128,0,128],rebeccapurple:[102,51,153],red:[255,0,0],rosybrown:[188,143,143],royalblue:[65,105,225],saddlebrown:[139,69,19],salmon:[250,128,114],sandybrown:[244,164,96],seagreen:[46,139,87],seashell:[255,245,238],sienna:[160,82,45],silver:[192,192,192],skyblue:[135,206,235],slateblue:[106,90,205],slategray:[112,128,144],slategrey:[112,128,144],snow:[255,250,250],springgreen:[0,255,127],steelblue:[70,130,180],tan:[210,180,140],teal:[0,128,128],thistle:[216,191,216],tomato:[255,99,71],turquoise:[64,224,208],violet:[238,130,238],wheat:[245,222,179],white:[255,255,255],whitesmoke:[245,245,245],yellow:[255,255,0],yellowgreen:[154,205,50]}},function(e,t,n){"use strict";var r=n(1907);e.exports={name:"hsl",min:[0,0,0],max:[360,100,100],channel:["hue","saturation","lightness"],alias:["HSL"],rgb:function(e){var t,n,r,i,o,a=e[0]/360,s=e[1]/100,l=e[2]/100;if(0===s)return[o=255*l,o,o];t=2*l-(n=l<.5?l*(1+s):l+s-l*s),i=[0,0,0];for(var u=0;u<3;u++)(r=a+1/3*-(u-1))<0?r++:r>1&&r--,o=6*r<1?t+6*(n-t)*r:2*r<1?n:3*r<2?t+(n-t)*(2/3-r)*6:t,i[u]=255*o;return i}},r.hsl=function(e){var t,n,r=e[0]/255,i=e[1]/255,o=e[2]/255,a=Math.min(r,i,o),s=Math.max(r,i,o),l=s-a;return s===a?t=0:r===s?t=(i-o)/l:i===s?t=2+(o-r)/l:o===s&&(t=4+(r-i)/l),(t=Math.min(60*t,360))<0&&(t+=360),n=(a+s)/2,[t,100*(s===a?0:n<=.5?l/(s+a):l/(2-s-a)),100*n]}},function(e,t,n){"use strict";e.exports={name:"rgb",min:[0,0,0],max:[255,255,255],channel:["red","green","blue"],alias:["RGB"]}},function(e,t){e.exports=function(e,t,n){return t<n?e<t?t:e>n?n:e:e<n?n:e>t?t:e}},function(e,t){e.exports=function(e){switch(e){case"int8":return Int8Array;case"int16":return Int16Array;case"int32":return Int32Array;case"uint8":return Uint8Array;case"uint16":return Uint16Array;case"uint32":return Uint32Array;case"float32":return Float32Array;case"float64":return Float64Array;case"array":return Array;case"uint8_clamped":return Uint8ClampedArray}}},function(e,t,n){"use strict";var r=n(27);function i(e,t,n,i){if(!t||!t.visible)return null;for(var o=r.getComponentMethod("errorbars","makeComputeError")(t),a=new Array(e.length),s=0;s<e.length;s++){var l=o(+e[s],s);if("log"===i.type){var u=i.c2l(e[s]),c=e[s]-l[0],d=e[s]+l[1];if(a[s]=[(i.c2l(c,!0)-u)*n,(i.c2l(d,!0)-u)*n],c>0){var h=i.c2l(c);i._lowerLogErrorBound||(i._lowerLogErrorBound=h),i._lowerErrorBound=Math.min(i._lowerLogErrorBound,h)}}else a[s]=[-l[0]*n,l[1]*n]}return a}e.exports=function(e,t,n){var r=[i(e.x,e.error_x,t[0],n.xaxis),i(e.y,e.error_y,t[1],n.yaxis),i(e.z,e.error_z,t[2],n.zaxis)],o=function(e){for(var t=0;t<e.length;t++)if(e[t])return e[t].length;return 0}(r);if(0===o)return null;for(var a=new Array(o),s=0;s<o;s++){for(var l=[[0,0,0],[0,0,0]],u=0;u<3;u++)if(r[u])for(var c=0;c<2;c++)l[c][u]=r[u][s][c];a[s]=l}return a}},function(e,t,n){"use strict";var r=n(27),i=n(10),o=n(125),a=n(1010),s=n(1011),l=n(1012),u=n(1060);e.exports=function(e,t,n,c){function d(n,r){return i.coerce(e,t,u,n,r)}if(function(e,t,n,i){var o=0,a=n("x"),s=n("y"),l=n("z");r.getComponentMethod("calendars","handleTraceDefaults")(e,t,["x","y","z"],i),a&&s&&l&&(o=Math.min(a.length,s.length,l.length),t._length=t._xlength=t._ylength=t._zlength=o);return o}(e,t,d,c)){d("text"),d("hovertext"),d("hovertemplate"),d("mode"),o.hasLines(t)&&(d("connectgaps"),s(e,t,n,c,d)),o.hasMarkers(t)&&a(e,t,n,c,d,{noSelect:!0}),o.hasText(t)&&(d("texttemplate"),l(e,t,c,d,{noSelect:!0}));var h=(t.line||{}).color,f=(t.marker||{}).color;d("surfaceaxis")>=0&&d("surfacecolor",h||f);for(var p=["x","y","z"],g=0;g<3;++g){var v="projection."+p[g];d(v+".show")&&(d(v+".opacity"),d(v+".scale"))}var m=r.getComponentMethod("errorbars","supplyDefaults");m(e,t,h||f||n,{axis:"z"}),m(e,t,h||f||n,{axis:"y",inherit:"z"}),m(e,t,h||f||n,{axis:"x",inherit:"z"})}else t.visible=!1}},function(e,t,n){"use strict";var r=n(558),i=n(1014);e.exports=function(e,t){var n=[{x:!1,y:!1,trace:t,t:{}}];return r(n,t),i(e,t),n}},function(e,t,n){"use strict";var r,i,o=n(1914),a=o.createCamera,s=o.createScene,l=n(1952),u=n(387),c=n(27),d=n(10),h=d.preserveDrawingBuffer(),f=n(55),p=n(179),g=n(304),v=n(1954),m=n(1023),y=n(1955),b=n(1956),C=n(1957);function A(e,t){var n=document.createElement("div"),r=e.container;this.graphDiv=e.graphDiv;var i=document.createElementNS("http://www.w3.org/2000/svg","svg");i.style.position="absolute",i.style.top=i.style.left="0px",i.style.width=i.style.height="100%",i.style["z-index"]=20,i.style["pointer-events"]="none",n.appendChild(i),this.svgContainer=i,n.id=e.id,n.style.position="absolute",n.style.top=n.style.left="0px",n.style.width=n.style.height="100%",r.appendChild(n),this.fullLayout=t,this.id=e.id||"scene",this.fullSceneLayout=t[this.id],this.plotArgs=[[],{},{}],this.axesOptions=y(t,t[this.id]),this.spikeOptions=b(t[this.id]),this.container=n,this.staticMode=!!e.staticPlot,this.pixelRatio=this.pixelRatio||e.plotGlPixelRatio||2,this.dataScale=[1,1,1],this.contourLevels=[[],[],[]],this.convertAnnotations=c.getComponentMethod("annotations3d","convert"),this.drawAnnotations=c.getComponentMethod("annotations3d","draw"),this.initializeGLPlot()}var _=A.prototype;_.prepareOptions=function(){var e={canvas:this.canvas,gl:this.gl,glOptions:{preserveDrawingBuffer:h,premultipliedAlpha:!0,antialias:!0},container:this.container,axes:this.axesOptions,spikes:this.spikeOptions,pickRadius:10,snapToData:!0,autoScale:!0,autoBounds:!1,cameraObject:this.camera,pixelRatio:this.pixelRatio};if(this.staticMode){if(!(i||(r=document.createElement("canvas"),i=l({canvas:r,preserveDrawingBuffer:!0,premultipliedAlpha:!0,antialias:!0}))))throw new Error("error creating static canvas/context for image server");e.gl=i,e.canvas=r}return e};var I=!0;_.tryCreatePlot=function(){var e=this.prepareOptions(),t=!0;try{this.glplot=s(e)}catch(n){if(this.staticMode||!I||h)t=!1;else{d.warn(["webgl setup failed possibly due to","false preserveDrawingBuffer config.","The mobile/tablet device may not be detected by is-mobile module.","Enabling preserveDrawingBuffer in second attempt to create webgl scene..."].join(" "));try{h=e.glOptions.preserveDrawingBuffer=!0,this.glplot=s(e)}catch(n){h=e.glOptions.preserveDrawingBuffer=!1,t=!1}}}return I=!1,t},_.initializeGLCamera=function(){var e=this.fullSceneLayout.camera,t="orthographic"===e.projection.type;this.camera=a(this.container,{center:[e.center.x,e.center.y,e.center.z],eye:[e.eye.x,e.eye.y,e.eye.z],up:[e.up.x,e.up.y,e.up.z],_ortho:t,zoomMin:.01,zoomMax:100,mode:"orbit"})},_.initializeGLPlot=function(){var e=this;if(e.initializeGLCamera(),!e.tryCreatePlot())return v(e);e.traces={},e.make4thDimension();var t=e.graphDiv,n=t.layout,r=function(){var t={};return e.isCameraChanged(n)&&(t[e.id+".camera"]=e.getCamera()),e.isAspectChanged(n)&&(t[e.id+".aspectratio"]=e.glplot.getAspectratio(),"manual"!==n[e.id].aspectmode&&(e.fullSceneLayout.aspectmode=n[e.id].aspectmode=t[e.id+".aspectmode"]="manual")),t},i=function(e){if(!1!==e.fullSceneLayout.dragmode){var t=r();e.saveLayout(n),e.graphDiv.emit("plotly_relayout",t)}};return e.glplot.canvas&&(e.glplot.canvas.addEventListener("mouseup",(function(){i(e)})),e.glplot.canvas.addEventListener("wheel",(function(n){if(t._context._scrollZoom.gl3d){if(e.camera._ortho){var r=n.deltaX>n.deltaY?1.1:1/1.1,o=e.glplot.getAspectratio();e.glplot.setAspectratio({x:r*o.x,y:r*o.y,z:r*o.z})}i(e)}}),!!u&&{passive:!1}),e.glplot.canvas.addEventListener("mousemove",(function(){if(!1!==e.fullSceneLayout.dragmode&&0!==e.camera.mouseListener.buttons){var t=r();e.graphDiv.emit("plotly_relayouting",t)}})),e.staticMode||e.glplot.canvas.addEventListener("webglcontextlost",(function(n){t&&t.emit&&t.emit("plotly_webglcontextlost",{event:n,layer:e.id})}),!1)),e.glplot.oncontextloss=function(){e.recoverContext()},e.glplot.onrender=function(){e.render()},!0},_.render=function(){var e,t=this,n=t.graphDiv,r=t.svgContainer,i=t.container.getBoundingClientRect();n._fullLayout._calcInverseTransform(n);var o=n._fullLayout._invScaleX,a=n._fullLayout._invScaleY,s=i.width*o,l=i.height*a;r.setAttributeNS(null,"viewBox","0 0 "+s+" "+l),r.setAttributeNS(null,"width",s),r.setAttributeNS(null,"height",l),C(t),t.glplot.axes.update(t.axesOptions);for(var u,c=Object.keys(t.traces),h=null,g=t.glplot.selection,v=0;v<c.length;++v)"skip"!==(e=t.traces[c[v]]).data.hoverinfo&&e.handlePick(g)&&(h=e),e.setContourLevels&&e.setContourLevels();function y(e,n){var r=t.fullSceneLayout[e];return f.tickText(r,r.d2l(n),"hover").text}if(null!==h){var b=m(t.glplot.cameraParams,g.dataCoordinate);e=h.data;var A,_=n._fullData[e.index],I=g.index,x={xLabel:y("xaxis",g.traceCoordinate[0]),yLabel:y("yaxis",g.traceCoordinate[1]),zLabel:y("zaxis",g.traceCoordinate[2])},w=p.castHoverinfo(_,t.fullLayout,I),M=(w||"").split("+"),S=w&&"all"===w;_.hovertemplate||S||(-1===M.indexOf("x")&&(x.xLabel=void 0),-1===M.indexOf("y")&&(x.yLabel=void 0),-1===M.indexOf("z")&&(x.zLabel=void 0),-1===M.indexOf("text")&&(g.textLabel=void 0),-1===M.indexOf("name")&&(h.name=void 0));var T=[];"cone"===e.type||"streamtube"===e.type?(x.uLabel=y("xaxis",g.traceCoordinate[3]),(S||-1!==M.indexOf("u"))&&T.push("u: "+x.uLabel),x.vLabel=y("yaxis",g.traceCoordinate[4]),(S||-1!==M.indexOf("v"))&&T.push("v: "+x.vLabel),x.wLabel=y("zaxis",g.traceCoordinate[5]),(S||-1!==M.indexOf("w"))&&T.push("w: "+x.wLabel),x.normLabel=g.traceCoordinate[6].toPrecision(3),(S||-1!==M.indexOf("norm"))&&T.push("norm: "+x.normLabel),"streamtube"===e.type&&(x.divergenceLabel=g.traceCoordinate[7].toPrecision(3),(S||-1!==M.indexOf("divergence"))&&T.push("divergence: "+x.divergenceLabel)),g.textLabel&&T.push(g.textLabel),A=T.join("<br>")):"isosurface"===e.type||"volume"===e.type?(x.valueLabel=f.tickText(t._mockAxis,t._mockAxis.d2l(g.traceCoordinate[3]),"hover").text,T.push("value: "+x.valueLabel),g.textLabel&&T.push(g.textLabel),A=T.join("<br>")):A=g.textLabel;var E={x:g.traceCoordinate[0],y:g.traceCoordinate[1],z:g.traceCoordinate[2],data:_._input,fullData:_,curveNumber:_.index,pointNumber:I};p.appendArrayPointValue(E,_,I),e._module.eventData&&(E=_._module.eventData(E,g,_,{},I));var O={points:[E]};t.fullSceneLayout.hovermode&&p.loneHover({trace:_,x:(.5+.5*b[0]/b[3])*s,y:(.5-.5*b[1]/b[3])*l,xLabel:x.xLabel,yLabel:x.yLabel,zLabel:x.zLabel,text:A,name:h.name,color:p.castHoverOption(_,I,"bgcolor")||h.color,borderColor:p.castHoverOption(_,I,"bordercolor"),fontFamily:p.castHoverOption(_,I,"font.family"),fontSize:p.castHoverOption(_,I,"font.size"),fontColor:p.castHoverOption(_,I,"font.color"),nameLength:p.castHoverOption(_,I,"namelength"),textAlign:p.castHoverOption(_,I,"align"),hovertemplate:d.castOption(_,I,"hovertemplate"),hovertemplateLabels:d.extendFlat({},E,x),eventData:[E]},{container:r,gd:n}),g.buttons&&g.distance<5?n.emit("plotly_click",O):n.emit("plotly_hover",O),u=O}else p.loneUnhover(r),n.emit("plotly_unhover",u);t.drawAnnotations(t)},_.recoverContext=function(){var e=this;e.glplot.dispose();var t=function(){e.glplot.gl.isContextLost()?requestAnimationFrame(t):e.initializeGLPlot()?e.plot.apply(e,e.plotArgs):d.error("Catastrophic and unrecoverable WebGL error. Context lost.")};requestAnimationFrame(t)};var x=["xaxis","yaxis","zaxis"];function w(e,t,n){for(var r=e.fullSceneLayout,i=0;i<3;i++){var o=x[i],a=o.charAt(0),s=r[o],l=t[a],u=t[a+"calendar"],c=t["_"+a+"length"];if(d.isArrayOrTypedArray(l))for(var h,f=0;f<(c||l.length);f++)if(d.isArrayOrTypedArray(l[f]))for(var p=0;p<l[f].length;++p)h=s.d2l(l[f][p],0,u),!isNaN(h)&&isFinite(h)&&(n[0][i]=Math.min(n[0][i],h),n[1][i]=Math.max(n[1][i],h));else h=s.d2l(l[f],0,u),!isNaN(h)&&isFinite(h)&&(n[0][i]=Math.min(n[0][i],h),n[1][i]=Math.max(n[1][i],h));else n[0][i]=Math.min(n[0][i],0),n[1][i]=Math.max(n[1][i],c-1)}}_.plot=function(e,t,n){if(this.plotArgs=[e,t,n],!this.glplot.contextLost){var r,i,o,a,s,l,u=t[this.id],c=n[this.id];this.fullLayout=t,this.fullSceneLayout=u,this.axesOptions.merge(t,u),this.spikeOptions.merge(u),this.setViewport(u),this.updateFx(u.dragmode,u.hovermode),this.camera.enableWheel=this.graphDiv._context._scrollZoom.gl3d,this.glplot.setClearColor(g(u.bgcolor)),this.setConvert(s),e?Array.isArray(e)||(e=[e]):e=[];var d=[[1/0,1/0,1/0],[-1/0,-1/0,-1/0]];for(o=0;o<e.length;++o)!0===(r=e[o]).visible&&0!==r._length&&w(this,r,d);!function(e,t){for(var n=e.fullSceneLayout,r=n.annotations||[],i=0;i<3;i++)for(var o=x[i],a=o.charAt(0),s=n[o],l=0;l<r.length;l++){var u=r[l];if(u.visible){var c=s.r2l(u[a]);!isNaN(c)&&isFinite(c)&&(t[0][i]=Math.min(t[0][i],c),t[1][i]=Math.max(t[1][i],c))}}}(this,d);var h=[1,1,1];for(a=0;a<3;++a)d[1][a]===d[0][a]?h[a]=1:h[a]=1/(d[1][a]-d[0][a]);for(this.dataScale=h,this.convertAnnotations(this),o=0;o<e.length;++o)!0===(r=e[o]).visible&&0!==r._length&&((i=this.traces[r.uid])?i.data.type===r.type?i.update(r):(i.dispose(),i=r._module.plot(this,r),this.traces[r.uid]=i):(i=r._module.plot(this,r),this.traces[r.uid]=i),i.name=r.name);var f=Object.keys(this.traces);e:for(o=0;o<f.length;++o){for(a=0;a<e.length;++a)if(e[a].uid===f[o]&&!0===e[a].visible&&0!==e[a]._length)continue e;(i=this.traces[f[o]]).dispose(),delete this.traces[f[o]]}this.glplot.objects.sort((function(e,t){return e._trace.data.index-t._trace.data.index}));var p,v=[[0,0,0],[0,0,0]],m=[],y={};for(o=0;o<3;++o){if((l=(s=u[x[o]]).type)in y?(y[l].acc*=h[o],y[l].count+=1):y[l]={acc:h[o],count:1},s.autorange){v[0][o]=1/0,v[1][o]=-1/0;var b=this.glplot.objects,C=this.fullSceneLayout.annotations||[],A=s._name.charAt(0);for(a=0;a<b.length;a++){var _=b[a],I=_.bounds,M=_._trace.data._pad||0;"ErrorBars"===_.constructor.name&&s._lowerLogErrorBound?v[0][o]=Math.min(v[0][o],s._lowerLogErrorBound):v[0][o]=Math.min(v[0][o],I[0][o]/h[o]-M),v[1][o]=Math.max(v[1][o],I[1][o]/h[o]+M)}for(a=0;a<C.length;a++){var S=C[a];if(S.visible){var T=s.r2l(S[A]);v[0][o]=Math.min(v[0][o],T),v[1][o]=Math.max(v[1][o],T)}}if("rangemode"in s&&"tozero"===s.rangemode&&(v[0][o]=Math.min(v[0][o],0),v[1][o]=Math.max(v[1][o],0)),v[0][o]>v[1][o])v[0][o]=-1,v[1][o]=1;else{var E=v[1][o]-v[0][o];v[0][o]-=E/32,v[1][o]+=E/32}if("reversed"===s.autorange){var O=v[0][o];v[0][o]=v[1][o],v[1][o]=O}}else{var k=s.range;v[0][o]=s.r2l(k[0]),v[1][o]=s.r2l(k[1])}v[0][o]===v[1][o]&&(v[0][o]-=1,v[1][o]+=1),m[o]=v[1][o]-v[0][o],this.glplot.setBounds(o,{min:v[0][o]*h[o],max:v[1][o]*h[o]})}var L=u.aspectmode;if("cube"===L)p=[1,1,1];else if("manual"===L){var D=u.aspectratio;p=[D.x,D.y,D.z]}else{if("auto"!==L&&"data"!==L)throw new Error("scene.js aspectRatio was not one of the enumerated types");var P=[1,1,1];for(o=0;o<3;++o){var N=y[l=(s=u[x[o]]).type];P[o]=Math.pow(N.acc,1/N.count)/h[o]}p="data"===L||Math.max.apply(null,P)/Math.min.apply(null,P)<=4?P:[1,1,1]}u.aspectratio.x=c.aspectratio.x=p[0],u.aspectratio.y=c.aspectratio.y=p[1],u.aspectratio.z=c.aspectratio.z=p[2],this.glplot.setAspectratio(u.aspectratio),this.viewInitial.aspectratio||(this.viewInitial.aspectratio={x:u.aspectratio.x,y:u.aspectratio.y,z:u.aspectratio.z}),this.viewInitial.aspectmode||(this.viewInitial.aspectmode=u.aspectmode);var j=u.domain||null,R=t._size||null;if(j&&R){var z=this.container.style;z.position="absolute",z.left=R.l+j.x[0]*R.w+"px",z.top=R.t+(1-j.y[1])*R.h+"px",z.width=R.w*(j.x[1]-j.x[0])+"px",z.height=R.h*(j.y[1]-j.y[0])+"px"}this.glplot.redraw()}},_.destroy=function(){this.glplot&&(this.camera.mouseListener.enabled=!1,this.container.removeEventListener("wheel",this.camera.wheelListener),this.camera=null,this.glplot.dispose(),this.container.parentNode.removeChild(this.container),this.glplot=null)},_.getCamera=function(){var e;return this.camera.view.recalcMatrix(this.camera.view.lastT()),{up:{x:(e=this.camera).up[0],y:e.up[1],z:e.up[2]},center:{x:e.center[0],y:e.center[1],z:e.center[2]},eye:{x:e.eye[0],y:e.eye[1],z:e.eye[2]},projection:{type:!0===e._ortho?"orthographic":"perspective"}}},_.setViewport=function(e){var t,n=e.camera;this.camera.lookAt.apply(this,[[(t=n).eye.x,t.eye.y,t.eye.z],[t.center.x,t.center.y,t.center.z],[t.up.x,t.up.y,t.up.z]]),this.glplot.setAspectratio(e.aspectratio),"orthographic"===n.projection.type!==this.camera._ortho&&(this.glplot.redraw(),this.glplot.clearRGBA(),this.glplot.dispose(),this.initializeGLPlot())},_.isCameraChanged=function(e){var t=this.getCamera(),n=d.nestedProperty(e,this.id+".camera").get();function r(e,t,n,r){var i=["up","center","eye"],o=["x","y","z"];return t[i[n]]&&e[i[n]][o[r]]===t[i[n]][o[r]]}var i=!1;if(void 0===n)i=!0;else{for(var o=0;o<3;o++)for(var a=0;a<3;a++)if(!r(t,n,o,a)){i=!0;break}(!n.projection||t.projection&&t.projection.type!==n.projection.type)&&(i=!0)}return i},_.isAspectChanged=function(e){var t=this.glplot.getAspectratio(),n=d.nestedProperty(e,this.id+".aspectratio").get();return void 0===n||n.x!==t.x||n.y!==t.y||n.z!==t.z},_.saveLayout=function(e){var t,n,r,i,o,a,s=this.fullLayout,l=this.isCameraChanged(e),u=this.isAspectChanged(e),h=l||u;if(h){var f={};if(l&&(t=this.getCamera(),r=(n=d.nestedProperty(e,this.id+".camera")).get(),f[this.id+".camera"]=r),u&&(i=this.glplot.getAspectratio(),a=(o=d.nestedProperty(e,this.id+".aspectratio")).get(),f[this.id+".aspectratio"]=a),c.call("_storeDirectGUIEdit",e,s._preGUI,f),l)n.set(t),d.nestedProperty(s,this.id+".camera").set(t);if(u)o.set(i),d.nestedProperty(s,this.id+".aspectratio").set(i),this.glplot.redraw()}return h},_.updateFx=function(e,t){var n=this.camera;if(n)if("orbit"===e)n.mode="orbit",n.keyBindingMode="rotate";else if("turntable"===e){n.up=[0,0,1],n.mode="turntable",n.keyBindingMode="rotate";var r=this.graphDiv,i=r._fullLayout,o=this.fullSceneLayout.camera,a=o.up.x,s=o.up.y,l=o.up.z;if(l/Math.sqrt(a*a+s*s+l*l)<.999){var u=this.id+".camera.up",h={x:0,y:0,z:1},f={};f[u]=h;var p=r.layout;c.call("_storeDirectGUIEdit",p,i._preGUI,f),o.up=h,d.nestedProperty(p,u).set(h)}}else n.keyBindingMode=e;this.fullSceneLayout.hovermode=t},_.toImage=function(e){e||(e="png"),this.staticMode&&this.container.appendChild(r),this.glplot.redraw();var t=this.glplot.gl,n=t.drawingBufferWidth,i=t.drawingBufferHeight;t.bindFramebuffer(t.FRAMEBUFFER,null);var o=new Uint8Array(n*i*4);t.readPixels(0,0,n,i,t.RGBA,t.UNSIGNED_BYTE,o),function(e,t,n){for(var r=0,i=n-1;r<i;++r,--i)for(var o=0;o<t;++o)for(var a=0;a<4;++a){var s=4*(t*r+o)+a,l=4*(t*i+o)+a,u=e[s];e[s]=e[l],e[l]=u}}(o,n,i),function(e,t,n){for(var r=0;r<n;++r)for(var i=0;i<t;++i){var o=4*(t*r+i),a=e[o+3];if(a>0)for(var s=255/a,l=0;l<3;++l)e[o+l]=Math.min(s*e[o+l],255)}}(o,n,i);var a=document.createElement("canvas");a.width=n,a.height=i;var s,l=a.getContext("2d"),u=l.createImageData(n,i);switch(u.data.set(o),l.putImageData(u,0,0),e){case"jpeg":s=a.toDataURL("image/jpeg");break;case"webp":s=a.toDataURL("image/webp");break;default:s=a.toDataURL("image/png")}return this.staticMode&&this.container.removeChild(r),s},_.setConvert=function(){for(var e=0;e<3;e++){var t=this.fullSceneLayout[x[e]];f.setConvert(t,this.fullLayout),t.setScale=d.noop}},_.make4thDimension=function(){var e=this.graphDiv._fullLayout;this._mockAxis={type:"linear",showexponent:"all",exponentformat:"B"},f.setConvert(this._mockAxis,e)},e.exports=A},function(e,t,n){"use strict";var r=n(1915),i=n(1937),o=n(1943),a=n(1946),s=n(1948),l=n(1068),u=n(1949),c=n(1065),d=n(975),h=n(976),f=n(1951),p=n(978)({tablet:!0,featureDetect:!0});function g(){this.mouse=[-1,-1],this.screen=null,this.distance=1/0,this.index=null,this.dataCoordinate=null,this.dataPosition=null,this.object=null,this.data=null}function v(e){var t=Math.round(Math.log(Math.abs(e))/Math.log(10));if(t<0){var n=Math.round(Math.pow(10,-t));return Math.ceil(e*n)/n}if(t>0){n=Math.round(Math.pow(10,t));return Math.ceil(e/n)*n}return Math.ceil(e)}function m(e){return"boolean"!=typeof e||e}e.exports={createScene:function(e){(e=e||{}).camera=e.camera||{};var t=e.canvas;if(!t){if(t=document.createElement("canvas"),e.container)e.container.appendChild(t);else document.body.appendChild(t)}var n=e.gl;n||(e.glOptions&&(p=!!e.glOptions.preserveDrawingBuffer),n=function(e,t){var n=null;try{(n=e.getContext("webgl",t))||(n=e.getContext("experimental-webgl",t))}catch(e){return null}return n}(t,e.glOptions||{premultipliedAlpha:!0,antialias:!0,preserveDrawingBuffer:p}));if(!n)throw new Error("webgl not supported");var y=e.bounds||[[-10,-10,-10],[10,10,10]],b=new g,C=l(n,n.drawingBufferWidth,n.drawingBufferHeight,{preferFloat:!p}),A=f(n),_=e.cameraObject&&!0===e.cameraObject._ortho||e.camera.projection&&"orthographic"===e.camera.projection.type||!1,I={eye:e.camera.eye||[2,0,0],center:e.camera.center||[0,0,0],up:e.camera.up||[0,1,0],zoomMin:e.camera.zoomMax||.1,zoomMax:e.camera.zoomMin||100,mode:e.camera.mode||"turntable",_ortho:_},x=e.axes||{},w=i(n,x);w.enable=!x.disable;var M=e.spikes||{},S=a(n,M),T=[],E=[],O=[],k=[],L=!0,D=!0,P=new Array(16),N=new Array(16),j={view:null,projection:P,model:N,_ortho:!1},R=(D=!0,[n.drawingBufferWidth,n.drawingBufferHeight]),z=e.cameraObject||r(t,I),F={gl:n,contextLost:!1,pixelRatio:e.pixelRatio||1,canvas:t,selection:b,camera:z,axes:w,axesPixels:null,spikes:S,bounds:y,objects:T,shape:R,aspect:e.aspectRatio||[1,1,1],pickRadius:e.pickRadius||10,zNear:e.zNear||.01,zFar:e.zFar||1e3,fovy:e.fovy||Math.PI/4,clearColor:e.clearColor||[0,0,0,0],autoResize:m(e.autoResize),autoBounds:m(e.autoBounds),autoScale:!!e.autoScale,autoCenter:m(e.autoCenter),clipToBounds:m(e.clipToBounds),snapToData:!!e.snapToData,onselect:e.onselect||null,onrender:e.onrender||null,onclick:e.onclick||null,cameraParams:j,oncontextloss:null,mouseListener:null,_stopped:!1,getAspectratio:function(){return{x:this.aspect[0],y:this.aspect[1],z:this.aspect[2]}},setAspectratio:function(e){this.aspect[0]=e.x,this.aspect[1]=e.y,this.aspect[2]=e.z,D=!0},setBounds:function(e,t){this.bounds[0][e]=t.min,this.bounds[1][e]=t.max},setClearColor:function(e){this.clearColor=e},clearRGBA:function(){this.gl.clearColor(this.clearColor[0],this.clearColor[1],this.clearColor[2],this.clearColor[3]),this.gl.clear(this.gl.COLOR_BUFFER_BIT|this.gl.DEPTH_BUFFER_BIT)}},B=[n.drawingBufferWidth/F.pixelRatio|0,n.drawingBufferHeight/F.pixelRatio|0];function H(){if(!F._stopped&&F.autoResize){var e=t.parentNode,n=1,r=1;e&&e!==document.body?(n=e.clientWidth,r=e.clientHeight):(n=window.innerWidth,r=window.innerHeight);var i=0|Math.ceil(n*F.pixelRatio),o=0|Math.ceil(r*F.pixelRatio);if(i!==t.width||o!==t.height){t.width=i,t.height=o;var a=t.style;a.position=a.position||"absolute",a.left="0px",a.top="0px",a.width=n+"px",a.height=r+"px",L=!0}}}F.autoResize&&H();function U(){for(var e=T.length,t=k.length,r=0;r<t;++r)O[r]=0;e:for(r=0;r<e;++r){var i=T[r],o=i.pickSlots;if(o){for(var a=0;a<t;++a)if(O[a]+o<255){E[r]=a,i.setPickBase(O[a]+1),O[a]+=o;continue e}var l=s(n,R);E[r]=t,k.push(l),O.push(o),i.setPickBase(1),t+=1}else E[r]=-1}for(;t>0&&0===O[t-1];)O.pop(),k.pop().dispose()}function V(){if(F.contextLost)return!0;n.isContextLost()&&(F.contextLost=!0,F.mouseListener.enabled=!1,F.selection.object=null,F.oncontextloss&&F.oncontextloss())}window.addEventListener("resize",H),F.update=function(e){F._stopped||(e=e||{},L=!0,D=!0)},F.add=function(e){F._stopped||(e.axes=w,T.push(e),E.push(-1),L=!0,D=!0,U())},F.remove=function(e){if(!F._stopped){var t=T.indexOf(e);t<0||(T.splice(t,1),E.pop(),L=!0,D=!0,U())}},F.dispose=function(){if(!F._stopped&&(F._stopped=!0,window.removeEventListener("resize",H),t.removeEventListener("webglcontextlost",V),F.mouseListener.enabled=!1,!F.contextLost)){w.dispose(),S.dispose();for(var e=0;e<T.length;++e)T[e].dispose();C.dispose();for(e=0;e<k.length;++e)k[e].dispose();A.dispose(),n=null,w=null,S=null,T=[]}},F._mouseRotating=!1,F._prevButtons=0,F.enableMouseListeners=function(){F.mouseListener=c(t,(function(e,t,n){if(!F._stopped){var r=k.length,i=T.length,o=b.object;b.distance=1/0,b.mouse[0]=t,b.mouse[1]=n,b.object=null,b.screen=null,b.dataCoordinate=b.dataPosition=null;var a=!1;if(e&&F._prevButtons)F._mouseRotating=!0;else{F._mouseRotating&&(D=!0),F._mouseRotating=!1;for(var s=0;s<r;++s){var l=k[s].query(t,B[1]-n-1,F.pickRadius);if(l){if(l.distance>b.distance)continue;for(var u=0;u<i;++u){var c=T[u];if(E[u]===s){var d=c.pick(l);d&&(b.buttons=e,b.screen=l.coord,b.distance=l.distance,b.object=c,b.index=d.distance,b.dataPosition=d.position,b.dataCoordinate=d.dataCoordinate,b.data=d,a=!0)}}}}}o&&o!==b.object&&(o.highlight&&o.highlight(null),L=!0),b.object&&(b.object.highlight&&b.object.highlight(b.data),L=!0),(a=a||b.object!==o)&&F.onselect&&F.onselect(b),1&e&&!(1&F._prevButtons)&&F.onclick&&F.onclick(b),F._prevButtons=e}}))},t.addEventListener("webglcontextlost",V);var Y=[[1/0,1/0,1/0],[-1/0,-1/0,-1/0]],W=[Y[0].slice(),Y[1].slice()];function G(){if(!V()){H();var e=F.camera.tick();j.view=F.camera.matrix,L=L||e,D=D||e,w.pixelRatio=F.pixelRatio,S.pixelRatio=F.pixelRatio;var t=T.length,r=Y[0],i=Y[1];r[0]=r[1]=r[2]=1/0,i[0]=i[1]=i[2]=-1/0;for(var a=0;a<t;++a){(O=T[a]).pixelRatio=F.pixelRatio,O.axes=F.axes,L=L||!!O.dirty,D=D||!!O.dirty;var s=O.bounds;if(s)for(var l=s[0],c=s[1],f=0;f<3;++f)r[f]=Math.min(r[f],l[f]),i[f]=Math.max(i[f],c[f])}var p=F.bounds;if(F.autoBounds)for(f=0;f<3;++f){if(i[f]<r[f])r[f]=-1,i[f]=1;else{r[f]===i[f]&&(r[f]-=1,i[f]+=1);var g=.05*(i[f]-r[f]);r[f]=r[f]-g,i[f]=i[f]+g}p[0][f]=r[f],p[1][f]=i[f]}var m=!1;for(f=0;f<3;++f)m=m||W[0][f]!==p[0][f]||W[1][f]!==p[1][f],W[0][f]=p[0][f],W[1][f]=p[1][f];if(D=D||m,L=L||m){if(m){var y=[0,0,0];for(a=0;a<3;++a)y[a]=v((p[1][a]-p[0][a])/10);w.autoTicks?w.update({bounds:p,tickSpacing:y}):w.update({bounds:p})}var I=n.drawingBufferWidth,x=n.drawingBufferHeight;R[0]=I,R[1]=x,B[0]=0|Math.max(I/F.pixelRatio,1),B[1]=0|Math.max(x/F.pixelRatio,1),function(e,t){var n=e.bounds,r=e.cameraParams,i=r.projection,o=r.model,a=e.gl.drawingBufferWidth,s=e.gl.drawingBufferHeight,l=e.zNear,u=e.zFar,c=e.fovy,f=a/s;t?(h(i,-f,f,-1,1,l,u),r._ortho=!0):(d(i,c,f,l,u),r._ortho=!1);for(var p=0;p<16;++p)o[p]=0;o[15]=1;var g=0;for(p=0;p<3;++p)g=Math.max(g,n[1][p]-n[0][p]);for(p=0;p<3;++p)e.autoScale?o[5*p]=e.aspect[p]/(n[1][p]-n[0][p]):o[5*p]=1/g,e.autoCenter&&(o[12+p]=.5*-o[5*p]*(n[0][p]+n[1][p]))}(F,_);for(a=0;a<t;++a){(O=T[a]).axesBounds=p,F.clipToBounds&&(O.clipBounds=p)}b.object&&(F.snapToData?S.position=b.dataCoordinate:S.position=b.dataPosition,S.bounds=p),D&&(D=!1,function(){if(!V()){n.colorMask(!0,!0,!0,!0),n.depthMask(!0),n.disable(n.BLEND),n.enable(n.DEPTH_TEST),n.depthFunc(n.LEQUAL);for(var e=T.length,t=k.length,r=0;r<t;++r){var i=k[r];i.shape=B,i.begin();for(var o=0;o<e;++o)if(E[o]===r){var a=T[o];a.drawPick&&(a.pixelRatio=1,a.drawPick(j))}i.end()}}}()),F.axesPixels=o(F.axes,j,I,x),F.onrender&&F.onrender(),n.bindFramebuffer(n.FRAMEBUFFER,null),n.viewport(0,0,I,x),F.clearRGBA(),n.depthMask(!0),n.colorMask(!0,!0,!0,!0),n.enable(n.DEPTH_TEST),n.depthFunc(n.LEQUAL),n.disable(n.BLEND),n.disable(n.CULL_FACE);var M=!1;w.enable&&(M=M||w.isTransparent(),w.draw(j)),S.axes=w,b.object&&S.draw(j),n.disable(n.CULL_FACE);for(a=0;a<t;++a){(O=T[a]).axes=w,O.pixelRatio=F.pixelRatio,O.isOpaque&&O.isOpaque()&&O.draw(j),O.isTransparent&&O.isTransparent()&&(M=!0)}if(M){C.shape=R,C.bind(),n.clear(n.DEPTH_BUFFER_BIT),n.colorMask(!1,!1,!1,!1),n.depthMask(!0),n.depthFunc(n.LESS),w.enable&&w.isTransparent()&&w.drawTransparent(j);for(a=0;a<t;++a){(O=T[a]).isOpaque&&O.isOpaque()&&O.draw(j)}n.enable(n.BLEND),n.blendEquation(n.FUNC_ADD),n.blendFunc(n.ONE,n.ONE_MINUS_SRC_ALPHA),n.colorMask(!0,!0,!0,!0),n.depthMask(!1),n.clearColor(0,0,0,0),n.clear(n.COLOR_BUFFER_BIT),w.isTransparent()&&w.drawTransparent(j);for(a=0;a<t;++a){var O;(O=T[a]).isTransparent&&O.isTransparent()&&O.drawTransparent(j)}n.bindFramebuffer(n.FRAMEBUFFER,null),n.blendFunc(n.ONE,n.ONE_MINUS_SRC_ALPHA),n.disable(n.DEPTH_TEST),A.bind(),C.color[0].bind(0),A.uniforms.accumBuffer=0,u(n),n.disable(n.BLEND)}L=!1;for(a=0;a<t;++a)T[a].dirty=!1}}}return F.enableMouseListeners(),function e(){if(F._stopped||F.contextLost)return;G(),requestAnimationFrame(e)}(),F.redraw=function(){F._stopped||(L=!0,G())},F},createCamera:r}},function(e,t,n){"use strict";e.exports=function(e,t){e=e||document.body,t=t||{};var n=[.01,1/0];"distanceLimits"in t&&(n[0]=t.distanceLimits[0],n[1]=t.distanceLimits[1]);"zoomMin"in t&&(n[0]=t.zoomMin);"zoomMax"in t&&(n[1]=t.zoomMax);var u=i({center:t.center||[0,0,0],up:t.up||[0,1,0],eye:t.eye||[0,0,10],mode:t.mode||"orbit",distanceLimits:n}),c=[0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0],d=0,h=e.clientWidth,f=e.clientHeight,p={keyBindingMode:"rotate",enableWheel:!0,view:u,element:e,delay:t.delay||16,rotateSpeed:t.rotateSpeed||1,zoomSpeed:t.zoomSpeed||1,translateSpeed:t.translateSpeed||1,flipX:!!t.flipX,flipY:!!t.flipY,modes:u.modes,_ortho:t._ortho||t.projection&&"orthographic"===t.projection.type||!1,tick:function(){var t=r(),n=this.delay,i=t-2*n;u.idle(t-n),u.recalcMatrix(i),u.flush(t-(100+2*n));for(var o=!0,a=u.computedMatrix,s=0;s<16;++s)o=o&&c[s]===a[s],c[s]=a[s];var l=e.clientWidth===h&&e.clientHeight===f;return h=e.clientWidth,f=e.clientHeight,o?!l:(d=Math.exp(u.computedRadius[0]),!0)},lookAt:function(e,t,n){u.lookAt(u.lastT(),e,t,n)},rotate:function(e,t,n){u.rotate(u.lastT(),e,t,n)},pan:function(e,t,n){u.pan(u.lastT(),e,t,n)},translate:function(e,t,n){u.translate(u.lastT(),e,t,n)}};return Object.defineProperties(p,{matrix:{get:function(){return u.computedMatrix},set:function(e){return u.setMatrix(u.lastT(),e),u.computedMatrix},enumerable:!0},mode:{get:function(){return u.getMode()},set:function(e){var t=u.computedUp.slice(),n=u.computedEye.slice(),i=u.computedCenter.slice();if(u.setMode(e),"turntable"===e){var o=r();u._active.lookAt(o,n,i,t),u._active.lookAt(o+500,n,i,[0,0,1]),u._active.flush(o)}return u.getMode()},enumerable:!0},center:{get:function(){return u.computedCenter},set:function(e){return u.lookAt(u.lastT(),null,e),u.computedCenter},enumerable:!0},eye:{get:function(){return u.computedEye},set:function(e){return u.lookAt(u.lastT(),e),u.computedEye},enumerable:!0},up:{get:function(){return u.computedUp},set:function(e){return u.lookAt(u.lastT(),null,null,e),u.computedUp},enumerable:!0},distance:{get:function(){return d},set:function(e){return u.setDistance(u.lastT(),e),e},enumerable:!0},distanceLimits:{get:function(){return u.getDistanceLimits(n)},set:function(e){return u.setDistanceLimits(e),e},enumerable:!0}}),e.addEventListener("contextmenu",(function(e){return e.preventDefault(),!1})),p._lastX=-1,p._lastY=-1,p._lastMods={shift:!1,control:!1,alt:!1,meta:!1},p.enableMouseListeners=function(){function t(t,n,i,o){var a=p.keyBindingMode;if(!1!==a){var s="rotate"===a,l="pan"===a,c="zoom"===a,h=!!o.control,f=!!o.alt,g=!!o.shift,v=!!(1&t),m=!!(2&t),y=!!(4&t),b=1/e.clientHeight,C=b*(n-p._lastX),A=b*(i-p._lastY),_=p.flipX?1:-1,I=p.flipY?1:-1,x=Math.PI*p.rotateSpeed,w=r();if(-1!==p._lastX&&-1!==p._lastY&&((s&&v&&!h&&!f&&!g||v&&!h&&!f&&g)&&u.rotate(w,_*x*C,-I*x*A,0),(l&&v&&!h&&!f&&!g||m||v&&h&&!f&&!g)&&u.pan(w,-p.translateSpeed*C*d,p.translateSpeed*A*d,0),c&&v&&!h&&!f&&!g||y||v&&!h&&f&&!g)){var M=-p.zoomSpeed*A/window.innerHeight*(w-u.lastT())*100;u.pan(w,0,0,d*(Math.exp(M)-1))}return p._lastX=n,p._lastY=i,p._lastMods=o,!0}}p.mouseListener=o(e,t),e.addEventListener("touchstart",(function(n){var r=s(n.changedTouches[0],e);t(0,r[0],r[1],p._lastMods),t(1,r[0],r[1],p._lastMods)}),!!l&&{passive:!0}),e.addEventListener("touchmove",(function(n){var r=s(n.changedTouches[0],e);t(1,r[0],r[1],p._lastMods),n.preventDefault()}),!!l&&{passive:!1}),e.addEventListener("touchend",(function(e){t(0,p._lastX,p._lastY,p._lastMods)}),!!l&&{passive:!0}),p.wheelListener=a(e,(function(e,t){if(!1!==p.keyBindingMode&&p.enableWheel){var n=p.flipX?1:-1,i=p.flipY?1:-1,o=r();if(Math.abs(e)>Math.abs(t))u.rotate(o,0,0,-e*n*Math.PI*p.rotateSpeed/window.innerWidth);else if(!p._ortho){var a=-p.zoomSpeed*i*t/window.innerHeight*(o-u.lastT())/20;u.pan(o,0,0,d*(Math.exp(a)-1))}}}),!0)},p.enableMouseListeners(),p};var r=n(1916),i=n(1917),o=n(1065),a=n(1934),s=n(992),l=n(387)},function(e,t,n){(function(t){e.exports=t.performance&&t.performance.now?function(){return performance.now()}:Date.now||function(){return+new Date}}).call(this,n(61))},function(e,t,n){"use strict";e.exports=function(e){var t=(e=e||{}).eye||[0,0,1],n=e.center||[0,0,0],s=e.up||[0,1,0],l=e.distanceLimits||[0,1/0],u=e.mode||"turntable",c=r(),d=i(),h=o();return c.setDistanceLimits(l[0],l[1]),c.lookAt(0,t,n,s),d.setDistanceLimits(l[0],l[1]),d.lookAt(0,t,n,s),h.setDistanceLimits(l[0],l[1]),h.lookAt(0,t,n,s),new a({turntable:c,orbit:d,matrix:h},u)};var r=n(1918),i=n(1921),o=n(1923);function a(e,t){this._controllerNames=Object.keys(e),this._controllerList=this._controllerNames.map((function(t){return e[t]})),this._mode=t,this._active=e[t],this._active||(this._mode="turntable",this._active=e.turntable),this.modes=this._controllerNames,this.computedMatrix=this._active.computedMatrix,this.computedEye=this._active.computedEye,this.computedUp=this._active.computedUp,this.computedCenter=this._active.computedCenter,this.computedRadius=this._active.computedRadius}var s=a.prototype;[["flush",1],["idle",1],["lookAt",4],["rotate",4],["pan",4],["translate",4],["setMatrix",2],["setDistanceLimits",2],["setDistance",2]].forEach((function(e){for(var t=e[0],n=[],r=0;r<e[1];++r)n.push("a"+r);var i="var cc=this._controllerList;for(var i=0;i<cc.length;++i){cc[i]."+e[0]+"("+n.join()+")}";s[t]=Function.apply(null,n.concat(i))})),s.recalcMatrix=function(e){this._active.recalcMatrix(e)},s.getDistance=function(e){return this._active.getDistance(e)},s.getDistanceLimits=function(e){return this._active.getDistanceLimits(e)},s.lastT=function(){return this._active.lastT()},s.setMode=function(e){if(e!==this._mode){var t=this._controllerNames.indexOf(e);if(!(t<0)){var n=this._active,r=this._controllerList[t],i=Math.max(n.lastT(),r.lastT());n.recalcMatrix(i),r.setMatrix(i,n.computedMatrix),this._active=r,this._mode=e,this.computedMatrix=this._active.computedMatrix,this.computedEye=this._active.computedEye,this.computedUp=this._active.computedUp,this.computedCenter=this._active.computedCenter,this.computedRadius=this._active.computedRadius}}},s.getMode=function(){return this._mode}},function(e,t,n){"use strict";e.exports=function(e){var t=(e=e||{}).center||[0,0,0],n=e.up||[0,1,0],r=e.right||d(n),i=e.radius||1,o=e.theta||0,c=e.phi||0;if(t=[].slice.call(t,0,3),n=[].slice.call(n,0,3),s(n,n),r=[].slice.call(r,0,3),s(r,r),"eye"in e){var f=e.eye,p=[f[0]-t[0],f[1]-t[1],f[2]-t[2]];a(r,p,n),u(r[0],r[1],r[2])<1e-6?r=d(n):s(r,r),i=u(p[0],p[1],p[2]);var g=l(n,p)/i,v=l(r,p)/i;c=Math.acos(g),o=Math.acos(v)}return i=Math.log(i),new h(e.zoomMin,e.zoomMax,t,n,r,i,o,c)};var r=n(1062),i=n(254),o=n(970),a=n(1063),s=n(575),l=n(1064);function u(e,t,n){return Math.sqrt(Math.pow(e,2)+Math.pow(t,2)+Math.pow(n,2))}function c(e){return Math.min(1,Math.max(-1,e))}function d(e){var t=Math.abs(e[0]),n=Math.abs(e[1]),r=Math.abs(e[2]),i=[0,0,0];t>Math.max(n,r)?i[2]=1:n>Math.max(t,r)?i[0]=1:i[1]=1;for(var o=0,a=0,l=0;l<3;++l)o+=e[l]*e[l],a+=i[l]*e[l];for(l=0;l<3;++l)i[l]-=a/o*e[l];return s(i,i),i}function h(e,t,n,i,o,a,s,l){this.center=r(n),this.up=r(i),this.right=r(o),this.radius=r([a]),this.angle=r([s,l]),this.angle.bounds=[[-1/0,-Math.PI/2],[1/0,Math.PI/2]],this.setDistanceLimits(e,t),this.computedCenter=this.center.curve(0),this.computedUp=this.up.curve(0),this.computedRight=this.right.curve(0),this.computedRadius=this.radius.curve(0),this.computedAngle=this.angle.curve(0),this.computedToward=[0,0,0],this.computedEye=[0,0,0],this.computedMatrix=new Array(16);for(var u=0;u<16;++u)this.computedMatrix[u]=.5;this.recalcMatrix(0)}var f=h.prototype;f.setDistanceLimits=function(e,t){e=e>0?Math.log(e):-1/0,t=t>0?Math.log(t):1/0,t=Math.max(t,e),this.radius.bounds[0][0]=e,this.radius.bounds[1][0]=t},f.getDistanceLimits=function(e){var t=this.radius.bounds[0];return e?(e[0]=Math.exp(t[0][0]),e[1]=Math.exp(t[1][0]),e):[Math.exp(t[0][0]),Math.exp(t[1][0])]},f.recalcMatrix=function(e){this.center.curve(e),this.up.curve(e),this.right.curve(e),this.radius.curve(e),this.angle.curve(e);for(var t=this.computedUp,n=this.computedRight,r=0,i=0,o=0;o<3;++o)i+=t[o]*n[o],r+=t[o]*t[o];var l=Math.sqrt(r),c=0;for(o=0;o<3;++o)n[o]-=t[o]*i/r,c+=n[o]*n[o],t[o]/=l;var d=Math.sqrt(c);for(o=0;o<3;++o)n[o]/=d;var h=this.computedToward;a(h,t,n),s(h,h);var f=Math.exp(this.computedRadius[0]),p=this.computedAngle[0],g=this.computedAngle[1],v=Math.cos(p),m=Math.sin(p),y=Math.cos(g),b=Math.sin(g),C=this.computedCenter,A=v*y,_=m*y,I=b,x=-v*b,w=-m*b,M=y,S=this.computedEye,T=this.computedMatrix;for(o=0;o<3;++o){var E=A*n[o]+_*h[o]+I*t[o];T[4*o+1]=x*n[o]+w*h[o]+M*t[o],T[4*o+2]=E,T[4*o+3]=0}var O=T[1],k=T[5],L=T[9],D=T[2],P=T[6],N=T[10],j=k*N-L*P,R=L*D-O*N,z=O*P-k*D,F=u(j,R,z);j/=F,R/=F,z/=F,T[0]=j,T[4]=R,T[8]=z;for(o=0;o<3;++o)S[o]=C[o]+T[2+4*o]*f;for(o=0;o<3;++o){c=0;for(var B=0;B<3;++B)c+=T[o+4*B]*S[B];T[12+o]=-c}T[15]=1},f.getMatrix=function(e,t){this.recalcMatrix(e);var n=this.computedMatrix;if(t){for(var r=0;r<16;++r)t[r]=n[r];return t}return n};var p=[0,0,0];f.rotate=function(e,t,n,r){if(this.angle.move(e,t,n),r){this.recalcMatrix(e);var i=this.computedMatrix;p[0]=i[2],p[1]=i[6],p[2]=i[10];for(var a=this.computedUp,s=this.computedRight,l=this.computedToward,u=0;u<3;++u)i[4*u]=a[u],i[4*u+1]=s[u],i[4*u+2]=l[u];o(i,i,r,p);for(u=0;u<3;++u)a[u]=i[4*u],s[u]=i[4*u+1];this.up.set(e,a[0],a[1],a[2]),this.right.set(e,s[0],s[1],s[2])}},f.pan=function(e,t,n,r){t=t||0,n=n||0,r=r||0,this.recalcMatrix(e);var i=this.computedMatrix,o=(Math.exp(this.computedRadius[0]),i[1]),a=i[5],s=i[9],l=u(o,a,s);o/=l,a/=l,s/=l;var c=i[0],d=i[4],h=i[8],f=c*o+d*a+h*s,p=u(c-=o*f,d-=a*f,h-=s*f),g=(c/=p)*t+o*n,v=(d/=p)*t+a*n,m=(h/=p)*t+s*n;this.center.move(e,g,v,m);var y=Math.exp(this.computedRadius[0]);y=Math.max(1e-4,y+r),this.radius.set(e,Math.log(y))},f.translate=function(e,t,n,r){this.center.move(e,t||0,n||0,r||0)},f.setMatrix=function(e,t,n,r){var o=1;"number"==typeof n&&(o=0|n),(o<0||o>3)&&(o=1);var a=(o+2)%3;t||(this.recalcMatrix(e),t=this.computedMatrix);var s=t[o],l=t[o+4],d=t[o+8];if(r){var h=Math.abs(s),f=Math.abs(l),p=Math.abs(d),g=Math.max(h,f,p);h===g?(s=s<0?-1:1,l=d=0):p===g?(d=d<0?-1:1,s=l=0):(l=l<0?-1:1,s=d=0)}else{var v=u(s,l,d);s/=v,l/=v,d/=v}var m,y,b=t[a],C=t[a+4],A=t[a+8],_=b*s+C*l+A*d,I=u(b-=s*_,C-=l*_,A-=d*_),x=l*(A/=I)-d*(C/=I),w=d*(b/=I)-s*A,M=s*C-l*b,S=u(x,w,M);if(x/=S,w/=S,M/=S,this.center.jump(e,Y,W,G),this.radius.idle(e),this.up.jump(e,s,l,d),this.right.jump(e,b,C,A),2===o){var T=t[1],E=t[5],O=t[9],k=T*b+E*C+O*A,L=T*x+E*w+O*M;m=j<0?-Math.PI/2:Math.PI/2,y=Math.atan2(L,k)}else{var D=t[2],P=t[6],N=t[10],j=D*s+P*l+N*d,R=D*b+P*C+N*A,z=D*x+P*w+N*M;m=Math.asin(c(j)),y=Math.atan2(z,R)}this.angle.jump(e,y,m),this.recalcMatrix(e);var F=t[2],B=t[6],H=t[10],U=this.computedMatrix;i(U,t);var V=U[15],Y=U[12]/V,W=U[13]/V,G=U[14]/V,X=Math.exp(this.computedRadius[0]);this.center.jump(e,Y-F*X,W-B*X,G-H*X)},f.lastT=function(){return Math.max(this.center.lastT(),this.up.lastT(),this.right.lastT(),this.radius.lastT(),this.angle.lastT())},f.idle=function(e){this.center.idle(e),this.up.idle(e),this.right.idle(e),this.radius.idle(e),this.angle.idle(e)},f.flush=function(e){this.center.flush(e),this.up.flush(e),this.right.flush(e),this.radius.flush(e),this.angle.flush(e)},f.setDistance=function(e,t){t>0&&this.radius.set(e,Math.log(t))},f.lookAt=function(e,t,n,r){this.recalcMatrix(e),t=t||this.computedEye,n=n||this.computedCenter;var i=(r=r||this.computedUp)[0],o=r[1],a=r[2],s=u(i,o,a);if(!(s<1e-6)){i/=s,o/=s,a/=s;var l=t[0]-n[0],d=t[1]-n[1],h=t[2]-n[2],f=u(l,d,h);if(!(f<1e-6)){l/=f,d/=f,h/=f;var p=this.computedRight,g=p[0],v=p[1],m=p[2],y=i*g+o*v+a*m,b=u(g-=y*i,v-=y*o,m-=y*a);if(!(b<.01&&(b=u(g=o*h-a*d,v=a*l-i*h,m=i*d-o*l))<1e-6)){g/=b,v/=b,m/=b,this.up.set(e,i,o,a),this.right.set(e,g,v,m),this.center.set(e,n[0],n[1],n[2]),this.radius.set(e,Math.log(f));var C=o*m-a*v,A=a*g-i*m,_=i*v-o*g,I=u(C,A,_),x=i*l+o*d+a*h,w=g*l+v*d+m*h,M=(C/=I)*l+(A/=I)*d+(_/=I)*h,S=Math.asin(c(x)),T=Math.atan2(M,w),E=this.angle._state,O=E[E.length-1],k=E[E.length-2];O%=2*Math.PI;var L=Math.abs(O+2*Math.PI-T),D=Math.abs(O-T),P=Math.abs(O-2*Math.PI-T);L<D&&(O+=2*Math.PI),P<D&&(O-=2*Math.PI),this.angle.jump(this.angle.lastT(),O,k),this.angle.set(e,T,S)}}}}},function(e,t,n){"use strict";e.exports=function(e,t,n,r,i,o){var a=i-1,s=i*i,l=a*a,u=(1+2*i)*l,c=i*l,d=s*(3-2*i),h=s*a;if(e.length){o||(o=new Array(e.length));for(var f=e.length-1;f>=0;--f)o[f]=u*e[f]+c*t[f]+d*n[f]+h*r[f];return o}return u*e+c*t+d*n+h*r},e.exports.derivative=function(e,t,n,r,i,o){var a=6*i*i-6*i,s=3*i*i-4*i+1,l=-6*i*i+6*i,u=3*i*i-2*i;if(e.length){o||(o=new Array(e.length));for(var c=e.length-1;c>=0;--c)o[c]=a*e[c]+s*t[c]+l*n[c]+u*r[c];return o}return a*e+s*t+l*n[c]+u*r}},function(e,t,n){"use strict";function r(e,t,n,r,i,o){var a=["function ",e,"(a,l,h,",r.join(","),"){",o?"":"var i=",n?"l-1":"h+1",";while(l<=h){var m=(l+h)>>>1,x=a",i?".get(m)":"[m]"];return o?t.indexOf("c")<0?a.push(";if(x===y){return m}else if(x<=y){"):a.push(";var p=c(x,y);if(p===0){return m}else if(p<=0){"):a.push(";if(",t,"){i=m;"),n?a.push("l=m+1}else{h=m-1}"):a.push("h=m-1}else{l=m+1}"),a.push("}"),o?a.push("return -1};"):a.push("return i};"),a.join("")}function i(e,t,n,i){return new Function([r("A","x"+e+"y",t,["y"],!1,i),r("B","x"+e+"y",t,["y"],!0,i),r("P","c(x,y)"+e+"0",t,["y","c"],!1,i),r("Q","c(x,y)"+e+"0",t,["y","c"],!0,i),"function dispatchBsearch",n,"(a,y,c,l,h){if(a.shape){if(typeof(c)==='function'){return Q(a,(l===undefined)?0:l|0,(h===undefined)?a.shape[0]-1:h|0,y,c)}else{return B(a,(c===undefined)?0:c|0,(l===undefined)?a.shape[0]-1:l|0,y)}}else{if(typeof(c)==='function'){return P(a,(l===undefined)?0:l|0,(h===undefined)?a.length-1:h|0,y,c)}else{return A(a,(c===undefined)?0:c|0,(l===undefined)?a.length-1:l|0,y)}}}return dispatchBsearch",n].join(""))()}e.exports={ge:i(">=",!1,"GE"),gt:i(">",!1,"GT"),lt:i("<",!0,"LT"),le:i("<=",!0,"LE"),eq:i("-",!0,"EQ",!0)}},function(e,t,n){"use strict";e.exports=function(e){var t=(e=e||{}).center||[0,0,0],n=e.rotation||[0,0,0,1],r=e.radius||1;t=[].slice.call(t,0,3),c(n=[].slice.call(n,0,4),n);var i=new d(n,t,Math.log(r));i.setDistanceLimits(e.zoomMin,e.zoomMax),("eye"in e||"up"in e)&&i.lookAt(0,e.eye,e.center,e.up);return i};var r=n(1062),i=n(536),o=n(535),a=n(254),s=n(1922);function l(e,t,n){return Math.sqrt(Math.pow(e,2)+Math.pow(t,2)+Math.pow(n,2))}function u(e,t,n,r){return Math.sqrt(Math.pow(e,2)+Math.pow(t,2)+Math.pow(n,2)+Math.pow(r,2))}function c(e,t){var n=t[0],r=t[1],i=t[2],o=t[3],a=u(n,r,i,o);a>1e-6?(e[0]=n/a,e[1]=r/a,e[2]=i/a,e[3]=o/a):(e[0]=e[1]=e[2]=0,e[3]=1)}function d(e,t,n){this.radius=r([n]),this.center=r(t),this.rotation=r(e),this.computedRadius=this.radius.curve(0),this.computedCenter=this.center.curve(0),this.computedRotation=this.rotation.curve(0),this.computedUp=[.1,0,0],this.computedEye=[.1,0,0],this.computedMatrix=[.1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0],this.recalcMatrix(0)}var h=d.prototype;h.lastT=function(){return Math.max(this.radius.lastT(),this.center.lastT(),this.rotation.lastT())},h.recalcMatrix=function(e){this.radius.curve(e),this.center.curve(e),this.rotation.curve(e);var t=this.computedRotation;c(t,t);var n=this.computedMatrix;o(n,t);var r=this.computedCenter,i=this.computedEye,a=this.computedUp,s=Math.exp(this.computedRadius[0]);i[0]=r[0]+s*n[2],i[1]=r[1]+s*n[6],i[2]=r[2]+s*n[10],a[0]=n[1],a[1]=n[5],a[2]=n[9];for(var l=0;l<3;++l){for(var u=0,d=0;d<3;++d)u+=n[l+4*d]*i[d];n[12+l]=-u}},h.getMatrix=function(e,t){this.recalcMatrix(e);var n=this.computedMatrix;if(t){for(var r=0;r<16;++r)t[r]=n[r];return t}return n},h.idle=function(e){this.center.idle(e),this.radius.idle(e),this.rotation.idle(e)},h.flush=function(e){this.center.flush(e),this.radius.flush(e),this.rotation.flush(e)},h.pan=function(e,t,n,r){t=t||0,n=n||0,r=r||0,this.recalcMatrix(e);var i=this.computedMatrix,o=i[1],a=i[5],s=i[9],u=l(o,a,s);o/=u,a/=u,s/=u;var c=i[0],d=i[4],h=i[8],f=c*o+d*a+h*s,p=l(c-=o*f,d-=a*f,h-=s*f);c/=p,d/=p,h/=p;var g=i[2],v=i[6],m=i[10],y=g*o+v*a+m*s,b=g*c+v*d+m*h,C=l(g-=y*o+b*c,v-=y*a+b*d,m-=y*s+b*h);g/=C,v/=C,m/=C;var A=c*t+o*n,_=d*t+a*n,I=h*t+s*n;this.center.move(e,A,_,I);var x=Math.exp(this.computedRadius[0]);x=Math.max(1e-4,x+r),this.radius.set(e,Math.log(x))},h.rotate=function(e,t,n,r){this.recalcMatrix(e),t=t||0,n=n||0;var i=this.computedMatrix,o=i[0],a=i[4],s=i[8],c=i[1],d=i[5],h=i[9],f=i[2],p=i[6],g=i[10],v=t*o+n*c,m=t*a+n*d,y=t*s+n*h,b=-(p*y-g*m),C=-(g*v-f*y),A=-(f*m-p*v),_=Math.sqrt(Math.max(0,1-Math.pow(b,2)-Math.pow(C,2)-Math.pow(A,2))),I=u(b,C,A,_);I>1e-6?(b/=I,C/=I,A/=I,_/=I):(b=C=A=0,_=1);var x=this.computedRotation,w=x[0],M=x[1],S=x[2],T=x[3],E=w*_+T*b+M*A-S*C,O=M*_+T*C+S*b-w*A,k=S*_+T*A+w*C-M*b,L=T*_-w*b-M*C-S*A;if(r){b=f,C=p,A=g;var D=Math.sin(r)/l(b,C,A);b*=D,C*=D,A*=D,L=L*(_=Math.cos(t))-(E=E*_+L*b+O*A-k*C)*b-(O=O*_+L*C+k*b-E*A)*C-(k=k*_+L*A+E*C-O*b)*A}var P=u(E,O,k,L);P>1e-6?(E/=P,O/=P,k/=P,L/=P):(E=O=k=0,L=1),this.rotation.set(e,E,O,k,L)},h.lookAt=function(e,t,n,r){this.recalcMatrix(e),n=n||this.computedCenter,t=t||this.computedEye,r=r||this.computedUp;var o=this.computedMatrix;i(o,t,n,r);var a=this.computedRotation;s(a,o[0],o[1],o[2],o[4],o[5],o[6],o[8],o[9],o[10]),c(a,a),this.rotation.set(e,a[0],a[1],a[2],a[3]);for(var l=0,u=0;u<3;++u)l+=Math.pow(n[u]-t[u],2);this.radius.set(e,.5*Math.log(Math.max(l,1e-6))),this.center.set(e,n[0],n[1],n[2])},h.translate=function(e,t,n,r){this.center.move(e,t||0,n||0,r||0)},h.setMatrix=function(e,t){var n=this.computedRotation;s(n,t[0],t[1],t[2],t[4],t[5],t[6],t[8],t[9],t[10]),c(n,n),this.rotation.set(e,n[0],n[1],n[2],n[3]);var r=this.computedMatrix;a(r,t);var i=r[15];if(Math.abs(i)>1e-6){var o=r[12]/i,l=r[13]/i,u=r[14]/i;this.recalcMatrix(e);var d=Math.exp(this.computedRadius[0]);this.center.set(e,o-r[2]*d,l-r[6]*d,u-r[10]*d),this.radius.idle(e)}else this.center.idle(e),this.radius.idle(e)},h.setDistance=function(e,t){t>0&&this.radius.set(e,Math.log(t))},h.setDistanceLimits=function(e,t){e=e>0?Math.log(e):-1/0,t=t>0?Math.log(t):1/0,t=Math.max(t,e),this.radius.bounds[0][0]=e,this.radius.bounds[1][0]=t},h.getDistanceLimits=function(e){var t=this.radius.bounds;return e?(e[0]=Math.exp(t[0][0]),e[1]=Math.exp(t[1][0]),e):[Math.exp(t[0][0]),Math.exp(t[1][0])]},h.toJSON=function(){return this.recalcMatrix(this.lastT()),{center:this.computedCenter.slice(),rotation:this.computedRotation.slice(),distance:Math.log(this.computedRadius[0]),zoomMin:this.radius.bounds[0][0],zoomMax:this.radius.bounds[1][0]}},h.fromJSON=function(e){var t=this.lastT(),n=e.center;n&&this.center.set(t,n[0],n[1],n[2]);var r=e.rotation;r&&this.rotation.set(t,r[0],r[1],r[2],r[3]);var i=e.distance;i&&i>0&&this.radius.set(t,Math.log(i)),this.setDistanceLimits(e.zoomMin,e.zoomMax)}},function(e,t,n){"use strict";e.exports=function(e,t,n,r,i,o,a,s,l,u){var c=t+o+u;if(d>0){var d=Math.sqrt(c+1);e[0]=.5*(a-l)/d,e[1]=.5*(s-r)/d,e[2]=.5*(n-o)/d,e[3]=.5*d}else{var h=Math.max(t,o,u);d=Math.sqrt(2*h-c+1);t>=h?(e[0]=.5*d,e[1]=.5*(i+n)/d,e[2]=.5*(s+r)/d,e[3]=.5*(a-l)/d):o>=h?(e[0]=.5*(n+i)/d,e[1]=.5*d,e[2]=.5*(l+a)/d,e[3]=.5*(s-r)/d):(e[0]=.5*(r+s)/d,e[1]=.5*(a+l)/d,e[2]=.5*d,e[3]=.5*(n-i)/d)}return e}},function(e,t,n){"use strict";var r=n(1924),i=n(1925),o=n(254),a=n(971),s=n(972),l=n(973),u=n(536),c=n(533),d=(n(534),n(575)),h=[0,0,0];function f(e){this._components=e.slice(),this._time=[0],this.prevMatrix=e.slice(),this.nextMatrix=e.slice(),this.computedMatrix=e.slice(),this.computedInverse=e.slice(),this.computedEye=[0,0,0],this.computedUp=[0,0,0],this.computedCenter=[0,0,0],this.computedRadius=[0],this._limits=[-1/0,1/0]}e.exports=function(e){return new f((e=e||{}).matrix||[1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1])};var p=f.prototype;p.recalcMatrix=function(e){var t=this._time,n=r.le(t,e),a=this.computedMatrix;if(!(n<0)){var s=this._components;if(n===t.length-1)for(var l=16*n,u=0;u<16;++u)a[u]=s[l++];else{var c=t[n+1]-t[n],h=(l=16*n,this.prevMatrix),f=!0;for(u=0;u<16;++u)h[u]=s[l++];var p=this.nextMatrix;for(u=0;u<16;++u)p[u]=s[l++],f=f&&h[u]===p[u];if(c<1e-6||f)for(u=0;u<16;++u)a[u]=h[u];else i(a,h,p,(e-t[n])/c)}var g=this.computedUp;g[0]=a[1],g[1]=a[5],g[2]=a[9],d(g,g);var v=this.computedInverse;o(v,a);var m=this.computedEye,y=v[15];m[0]=v[12]/y,m[1]=v[13]/y,m[2]=v[14]/y;var b=this.computedCenter,C=Math.exp(this.computedRadius[0]);for(u=0;u<3;++u)b[u]=m[u]-a[2+4*u]*C}},p.idle=function(e){if(!(e<this.lastT())){for(var t=this._components,n=t.length-16,r=0;r<16;++r)t.push(t[n++]);this._time.push(e)}},p.flush=function(e){var t=r.gt(this._time,e)-2;t<0||(this._time.splice(0,t),this._components.splice(0,16*t))},p.lastT=function(){return this._time[this._time.length-1]},p.lookAt=function(e,t,n,r){this.recalcMatrix(e),t=t||this.computedEye,n=n||h,r=r||this.computedUp,this.setMatrix(e,u(this.computedMatrix,t,n,r));for(var i=0,o=0;o<3;++o)i+=Math.pow(n[o]-t[o],2);i=Math.log(Math.sqrt(i)),this.computedRadius[0]=i},p.rotate=function(e,t,n,r){this.recalcMatrix(e);var i=this.computedInverse;t&&s(i,i,t),n&&a(i,i,n),r&&l(i,i,r),this.setMatrix(e,o(this.computedMatrix,i))};var g=[0,0,0];p.pan=function(e,t,n,r){g[0]=-(t||0),g[1]=-(n||0),g[2]=-(r||0),this.recalcMatrix(e);var i=this.computedInverse;c(i,i,g),this.setMatrix(e,o(i,i))},p.translate=function(e,t,n,r){g[0]=t||0,g[1]=n||0,g[2]=r||0,this.recalcMatrix(e);var i=this.computedMatrix;c(i,i,g),this.setMatrix(e,i)},p.setMatrix=function(e,t){if(!(e<this.lastT())){this._time.push(e);for(var n=0;n<16;++n)this._components.push(t[n])}},p.setDistance=function(e,t){this.computedRadius[0]=t},p.setDistanceLimits=function(e,t){var n=this._limits;n[0]=e,n[1]=t},p.getDistanceLimits=function(e){var t=this._limits;return e?(e[0]=t[0],e[1]=t[1],e):t}},function(e,t,n){"use strict";function r(e,t,n,r,i,o){var a=["function ",e,"(a,l,h,",r.join(","),"){",o?"":"var i=",n?"l-1":"h+1",";while(l<=h){var m=(l+h)>>>1,x=a",i?".get(m)":"[m]"];return o?t.indexOf("c")<0?a.push(";if(x===y){return m}else if(x<=y){"):a.push(";var p=c(x,y);if(p===0){return m}else if(p<=0){"):a.push(";if(",t,"){i=m;"),n?a.push("l=m+1}else{h=m-1}"):a.push("h=m-1}else{l=m+1}"),a.push("}"),o?a.push("return -1};"):a.push("return i};"),a.join("")}function i(e,t,n,i){return new Function([r("A","x"+e+"y",t,["y"],!1,i),r("B","x"+e+"y",t,["y"],!0,i),r("P","c(x,y)"+e+"0",t,["y","c"],!1,i),r("Q","c(x,y)"+e+"0",t,["y","c"],!0,i),"function dispatchBsearch",n,"(a,y,c,l,h){if(a.shape){if(typeof(c)==='function'){return Q(a,(l===undefined)?0:l|0,(h===undefined)?a.shape[0]-1:h|0,y,c)}else{return B(a,(c===undefined)?0:c|0,(l===undefined)?a.shape[0]-1:l|0,y)}}else{if(typeof(c)==='function'){return P(a,(l===undefined)?0:l|0,(h===undefined)?a.length-1:h|0,y,c)}else{return A(a,(c===undefined)?0:c|0,(l===undefined)?a.length-1:l|0,y)}}}return dispatchBsearch",n].join(""))()}e.exports={ge:i(">=",!1,"GE"),gt:i(">",!1,"GT"),lt:i("<",!0,"LT"),le:i("<=",!0,"LE"),eq:i("-",!0,"EQ",!0)}},function(e,t,n){var r=n(1926),i=n(1927),o=n(1928),a=n(532),s=n(1931),l=d(),u=d(),c=d();function d(){return{translate:h(),scale:h(1),skew:h(),perspective:[0,0,0,1],quaternion:[0,0,0,1]}}function h(e){return[e||0,e||0,e||0]}e.exports=function(e,t,n,d){if(0===a(t)||0===a(n))return!1;var h=o(t,l.translate,l.scale,l.skew,l.perspective,l.quaternion),f=o(n,u.translate,u.scale,u.skew,u.perspective,u.quaternion);return!(!h||!f)&&(r(c.translate,l.translate,u.translate,d),r(c.skew,l.skew,u.skew,d),r(c.scale,l.scale,u.scale,d),r(c.perspective,l.perspective,u.perspective,d),s(c.quaternion,l.quaternion,u.quaternion,d),i(e,c.translate,c.scale,c.skew,c.perspective,c.quaternion),!0)}},function(e,t){e.exports=function(e,t,n,r){var i=t[0],o=t[1],a=t[2];return e[0]=i+r*(n[0]-i),e[1]=o+r*(n[1]-o),e[2]=a+r*(n[2]-a),e}},function(e,t,n){var r={identity:n(530),translate:n(533),multiply:n(255),create:n(529),scale:n(534),fromRotationTranslation:n(974)},i=(r.create(),r.create());e.exports=function(e,t,n,o,a,s){return r.identity(e),r.fromRotationTranslation(e,s,t),e[3]=a[0],e[7]=a[1],e[11]=a[2],e[15]=a[3],r.identity(i),0!==o[2]&&(i[9]=o[2],r.multiply(e,e,i)),0!==o[1]&&(i[9]=0,i[8]=o[1],r.multiply(e,e,i)),0!==o[0]&&(i[8]=0,i[4]=o[0],r.multiply(e,e,i)),r.scale(e,e,n),e}},function(e,t,n){var r=n(1929),i=n(529),o=n(969),a=n(532),s=n(254),l=n(531),u={length:n(1930),normalize:n(575),dot:n(1064),cross:n(1063)},c=i(),d=i(),h=[0,0,0,0],f=[[0,0,0],[0,0,0],[0,0,0]],p=[0,0,0];function g(e,t,n,r,i){e[0]=t[0]*r+n[0]*i,e[1]=t[1]*r+n[1]*i,e[2]=t[2]*r+n[2]*i}e.exports=function(e,t,n,i,v,m){if(t||(t=[0,0,0]),n||(n=[0,0,0]),i||(i=[0,0,0]),v||(v=[0,0,0,1]),m||(m=[0,0,0,1]),!r(c,e))return!1;if(o(d,c),d[3]=0,d[7]=0,d[11]=0,d[15]=1,Math.abs(a(d)<1e-8))return!1;var y,b,C,A,_,I,x,w=c[3],M=c[7],S=c[11],T=c[12],E=c[13],O=c[14],k=c[15];if(0!==w||0!==M||0!==S){if(h[0]=w,h[1]=M,h[2]=S,h[3]=k,!s(d,d))return!1;l(d,d),y=v,C=d,A=(b=h)[0],_=b[1],I=b[2],x=b[3],y[0]=C[0]*A+C[4]*_+C[8]*I+C[12]*x,y[1]=C[1]*A+C[5]*_+C[9]*I+C[13]*x,y[2]=C[2]*A+C[6]*_+C[10]*I+C[14]*x,y[3]=C[3]*A+C[7]*_+C[11]*I+C[15]*x}else v[0]=v[1]=v[2]=0,v[3]=1;if(t[0]=T,t[1]=E,t[2]=O,function(e,t){e[0][0]=t[0],e[0][1]=t[1],e[0][2]=t[2],e[1][0]=t[4],e[1][1]=t[5],e[1][2]=t[6],e[2][0]=t[8],e[2][1]=t[9],e[2][2]=t[10]}(f,c),n[0]=u.length(f[0]),u.normalize(f[0],f[0]),i[0]=u.dot(f[0],f[1]),g(f[1],f[1],f[0],1,-i[0]),n[1]=u.length(f[1]),u.normalize(f[1],f[1]),i[0]/=n[1],i[1]=u.dot(f[0],f[2]),g(f[2],f[2],f[0],1,-i[1]),i[2]=u.dot(f[1],f[2]),g(f[2],f[2],f[1],1,-i[2]),n[2]=u.length(f[2]),u.normalize(f[2],f[2]),i[1]/=n[2],i[2]/=n[2],u.cross(p,f[1],f[2]),u.dot(f[0],p)<0)for(var L=0;L<3;L++)n[L]*=-1,f[L][0]*=-1,f[L][1]*=-1,f[L][2]*=-1;return m[0]=.5*Math.sqrt(Math.max(1+f[0][0]-f[1][1]-f[2][2],0)),m[1]=.5*Math.sqrt(Math.max(1-f[0][0]+f[1][1]-f[2][2],0)),m[2]=.5*Math.sqrt(Math.max(1-f[0][0]-f[1][1]+f[2][2],0)),m[3]=.5*Math.sqrt(Math.max(1+f[0][0]+f[1][1]+f[2][2],0)),f[2][1]>f[1][2]&&(m[0]=-m[0]),f[0][2]>f[2][0]&&(m[1]=-m[1]),f[1][0]>f[0][1]&&(m[2]=-m[2]),!0}},function(e,t){e.exports=function(e,t){var n=t[15];if(0===n)return!1;for(var r=1/n,i=0;i<16;i++)e[i]=t[i]*r;return!0}},function(e,t){e.exports=function(e){var t=e[0],n=e[1],r=e[2];return Math.sqrt(t*t+n*n+r*r)}},function(e,t,n){e.exports=n(1932)},function(e,t){e.exports=function(e,t,n,r){var i,o,a,s,l,u=t[0],c=t[1],d=t[2],h=t[3],f=n[0],p=n[1],g=n[2],v=n[3];(o=u*f+c*p+d*g+h*v)<0&&(o=-o,f=-f,p=-p,g=-g,v=-v);1-o>1e-6?(i=Math.acos(o),a=Math.sin(i),s=Math.sin((1-r)*i)/a,l=Math.sin(r*i)/a):(s=1-r,l=r);return e[0]=s*u+l*f,e[1]=s*c+l*p,e[2]=s*d+l*g,e[3]=s*h+l*v,e}},function(e,t,n){"use strict";function r(e){return e.target||e.srcElement||window}t.buttons=function(e){if("object"==typeof e){if("buttons"in e)return e.buttons;if("which"in e){if(2===(t=e.which))return 4;if(3===t)return 2;if(t>0)return 1<<t-1}else if("button"in e){var t;if(1===(t=e.button))return 4;if(2===t)return 2;if(t>=0)return 1<<t}}return 0},t.element=r,t.x=function(e){if("object"==typeof e){if("offsetX"in e)return e.offsetX;var t=r(e).getBoundingClientRect();return e.clientX-t.left}return 0},t.y=function(e){if("object"==typeof e){if("offsetY"in e)return e.offsetY;var t=r(e).getBoundingClientRect();return e.clientY-t.top}return 0}},function(e,t,n){"use strict";var r=n(1935);e.exports=function(e,t,n){"function"==typeof e&&(n=!!t,t=e,e=window);var i=r("ex",e),o=function(e){n&&e.preventDefault();var r=e.deltaX||0,o=e.deltaY||0,a=e.deltaZ||0,s=1;switch(e.deltaMode){case 1:s=i;break;case 2:s=window.innerHeight}if(o*=s,a*=s,(r*=s)||o||a)return t(r,o,a,e)};return e.addEventListener("wheel",o),o}},function(e,t,n){"use strict";var r=n(1936);e.exports=s;var i=a("in",document.body);function o(e,t){var n=r(getComputedStyle(e).getPropertyValue(t));return n[0]*s(n[1],e)}function a(e,t){var n=document.createElement("div");n.style.height="128"+e,t.appendChild(n);var r=o(n,"height")/128;return t.removeChild(n),r}function s(e,t){if(!e)return null;switch(t=t||document.body,e=(e+""||"px").trim().toLowerCase(),t!==window&&t!==document||(t=document.body),e){case"%":return t.clientHeight/100;case"ch":case"ex":return a(e,t);case"em":return o(t,"font-size");case"rem":return o(document.body,"font-size");case"vw":return window.innerWidth/100;case"vh":return window.innerHeight/100;case"vmin":return Math.min(window.innerWidth,window.innerHeight)/100;case"vmax":return Math.max(window.innerWidth,window.innerHeight)/100;case"in":return i;case"cm":return i/2.54;case"mm":return i/25.4;case"pt":return i/72;case"pc":return i/6;case"px":return 1}var n=r(e);if(!isNaN(n[0])&&n[1]){var l=s(n[1],t);return"number"==typeof l?n[0]*l:null}return null}},function(e,t){e.exports=function(e,t){t||(t=[0,""]),e=String(e);var n=parseFloat(e,10);return t[0]=n,t[1]=e.match(/[\d.\-\+]*\s*(.*)/)[1]||"",t}},function(e,t,n){"use strict";e.exports=function(e,t){var n=new c(e);return n.update(t),n};var r=n(1938),i=n(1939),o=n(1940),a=n(1066),s=n(1942),l=new Float32Array([1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1]);function u(e,t){return e[0]=t[0],e[1]=t[1],e[2]=t[2],e}function c(e){this.gl=e,this.pixelRatio=1,this.bounds=[[-10,-10,-10],[10,10,10]],this.ticks=[[],[],[]],this.autoTicks=!0,this.tickSpacing=[1,1,1],this.tickEnable=[!0,!0,!0],this.tickFont=["sans-serif","sans-serif","sans-serif"],this.tickSize=[12,12,12],this.tickAngle=[0,0,0],this.tickAlign=["auto","auto","auto"],this.tickColor=[[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.tickPad=[10,10,10],this.lastCubeProps={cubeEdges:[0,0,0],axis:[0,0,0]},this.labels=["x","y","z"],this.labelEnable=[!0,!0,!0],this.labelFont="sans-serif",this.labelSize=[20,20,20],this.labelAngle=[0,0,0],this.labelAlign=["auto","auto","auto"],this.labelColor=[[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.labelPad=[10,10,10],this.lineEnable=[!0,!0,!0],this.lineMirror=[!1,!1,!1],this.lineWidth=[1,1,1],this.lineColor=[[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.lineTickEnable=[!0,!0,!0],this.lineTickMirror=[!1,!1,!1],this.lineTickLength=[0,0,0],this.lineTickWidth=[1,1,1],this.lineTickColor=[[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.gridEnable=[!0,!0,!0],this.gridWidth=[1,1,1],this.gridColor=[[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.zeroEnable=[!0,!0,!0],this.zeroLineColor=[[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.zeroLineWidth=[2,2,2],this.backgroundEnable=[!1,!1,!1],this.backgroundColor=[[.8,.8,.8,.5],[.8,.8,.8,.5],[.8,.8,.8,.5]],this._firstInit=!0,this._text=null,this._lines=null,this._background=o(e)}var d=c.prototype;function h(){this.primalOffset=[0,0,0],this.primalMinor=[0,0,0],this.mirrorOffset=[0,0,0],this.mirrorMinor=[0,0,0]}d.update=function(e){function t(t,n,r){if(r in e){var i,o=e[r],a=this[r];(t?Array.isArray(o)&&Array.isArray(o[0]):Array.isArray(o))?this[r]=i=[n(o[0]),n(o[1]),n(o[2])]:this[r]=i=[n(o),n(o),n(o)];for(var s=0;s<3;++s)if(i[s]!==a[s])return!0}return!1}e=e||{};var n,o=t.bind(this,!1,Number),a=t.bind(this,!1,Boolean),l=t.bind(this,!1,String),u=t.bind(this,!0,(function(e){if(Array.isArray(e)){if(3===e.length)return[+e[0],+e[1],+e[2],1];if(4===e.length)return[+e[0],+e[1],+e[2],+e[3]]}return[0,0,0,1]})),c=!1,d=!1;if("bounds"in e)for(var h=e.bounds,f=0;f<2;++f)for(var p=0;p<3;++p)h[f][p]!==this.bounds[f][p]&&(d=!0),this.bounds[f][p]=h[f][p];if("ticks"in e){n=e.ticks,c=!0,this.autoTicks=!1;for(f=0;f<3;++f)this.tickSpacing[f]=0}else o("tickSpacing")&&(this.autoTicks=!0,d=!0);if(this._firstInit&&("ticks"in e||"tickSpacing"in e||(this.autoTicks=!0),d=!0,c=!0,this._firstInit=!1),d&&this.autoTicks&&(n=s.create(this.bounds,this.tickSpacing),c=!0),c){for(f=0;f<3;++f)n[f].sort((function(e,t){return e.x-t.x}));s.equal(n,this.ticks)?c=!1:this.ticks=n}a("tickEnable"),l("tickFont")&&(c=!0),o("tickSize"),o("tickAngle"),o("tickPad"),u("tickColor");var g=l("labels");l("labelFont")&&(g=!0),a("labelEnable"),o("labelSize"),o("labelPad"),u("labelColor"),a("lineEnable"),a("lineMirror"),o("lineWidth"),u("lineColor"),a("lineTickEnable"),a("lineTickMirror"),o("lineTickLength"),o("lineTickWidth"),u("lineTickColor"),a("gridEnable"),o("gridWidth"),u("gridColor"),a("zeroEnable"),u("zeroLineColor"),o("zeroLineWidth"),a("backgroundEnable"),u("backgroundColor"),this._text?this._text&&(g||c)&&this._text.update(this.bounds,this.labels,this.labelFont,this.ticks,this.tickFont):this._text=r(this.gl,this.bounds,this.labels,this.labelFont,this.ticks,this.tickFont),this._lines&&c&&(this._lines.dispose(),this._lines=null),this._lines||(this._lines=i(this.gl,this.bounds,this.ticks))};var f=[new h,new h,new h];function p(e,t,n,r,i){for(var o=e.primalOffset,a=e.primalMinor,s=e.mirrorOffset,l=e.mirrorMinor,u=r[t],c=0;c<3;++c)if(t!==c){var d=o,h=s,f=a,p=l;u&1<<c&&(d=s,h=o,f=l,p=a),d[c]=n[0][c],h[c]=n[1][c],i[c]>0?(f[c]=-1,p[c]=0):(f[c]=0,p[c]=1)}}var g=[0,0,0],v={model:l,view:l,projection:l,_ortho:!1};d.isOpaque=function(){return!0},d.isTransparent=function(){return!1},d.drawTransparent=function(e){};var m=[0,0,0],y=[0,0,0],b=[0,0,0];d.draw=function(e){e=e||v;for(var t=this.gl,n=e.model||l,r=e.view||l,i=e.projection||l,o=this.bounds,s=e._ortho||!1,c=a(n,r,i,o,s),d=c.cubeEdges,h=c.axis,C=r[12],A=r[13],_=r[14],I=r[15],x=(s?2:1)*this.pixelRatio*(i[3]*C+i[7]*A+i[11]*_+i[15]*I)/t.drawingBufferHeight,w=0;w<3;++w)this.lastCubeProps.cubeEdges[w]=d[w],this.lastCubeProps.axis[w]=h[w];var M=f;for(w=0;w<3;++w)p(f[w],w,this.bounds,d,h);t=this.gl;var S,T=g;for(w=0;w<3;++w)this.backgroundEnable[w]?T[w]=h[w]:T[w]=0;this._background.draw(n,r,i,o,T,this.backgroundColor),this._lines.bind(n,r,i,this);for(w=0;w<3;++w){var E=[0,0,0];h[w]>0?E[w]=o[1][w]:E[w]=o[0][w];for(var O=0;O<2;++O){var k=(w+1+O)%3,L=(w+1+(1^O))%3;this.gridEnable[k]&&this._lines.drawGrid(k,L,this.bounds,E,this.gridColor[k],this.gridWidth[k]*this.pixelRatio)}for(O=0;O<2;++O){k=(w+1+O)%3,L=(w+1+(1^O))%3;this.zeroEnable[L]&&Math.min(o[0][L],o[1][L])<=0&&Math.max(o[0][L],o[1][L])>=0&&this._lines.drawZero(k,L,this.bounds,E,this.zeroLineColor[L],this.zeroLineWidth[L]*this.pixelRatio)}}for(w=0;w<3;++w){this.lineEnable[w]&&this._lines.drawAxisLine(w,this.bounds,M[w].primalOffset,this.lineColor[w],this.lineWidth[w]*this.pixelRatio),this.lineMirror[w]&&this._lines.drawAxisLine(w,this.bounds,M[w].mirrorOffset,this.lineColor[w],this.lineWidth[w]*this.pixelRatio);var D=u(m,M[w].primalMinor),P=u(y,M[w].mirrorMinor),N=this.lineTickLength;for(O=0;O<3;++O){var j=x/n[5*O];D[O]*=N[O]*j,P[O]*=N[O]*j}this.lineTickEnable[w]&&this._lines.drawAxisTicks(w,M[w].primalOffset,D,this.lineTickColor[w],this.lineTickWidth[w]*this.pixelRatio),this.lineTickMirror[w]&&this._lines.drawAxisTicks(w,M[w].mirrorOffset,P,this.lineTickColor[w],this.lineTickWidth[w]*this.pixelRatio)}this._lines.unbind(),this._text.bind(n,r,i,this.pixelRatio);var R,z;function F(e){(z=[0,0,0])[e]=1}function B(e,t,n){var r=(e+1)%3,i=(e+2)%3,o=t[r],a=t[i],s=n[r],l=n[i];o>0&&l>0||o>0&&l<0||o<0&&l>0||o<0&&l<0?F(r):(a>0&&s>0||a>0&&s<0||a<0&&s>0||a<0&&s<0)&&F(i)}for(w=0;w<3;++w){var H=M[w].primalMinor,U=M[w].mirrorMinor,V=u(b,M[w].primalOffset);for(O=0;O<3;++O)this.lineTickEnable[w]&&(V[O]+=x*H[O]*Math.max(this.lineTickLength[O],0)/n[5*O]);var Y=[0,0,0];if(Y[w]=1,this.tickEnable[w]){-3600===this.tickAngle[w]?(this.tickAngle[w]=0,this.tickAlign[w]="auto"):this.tickAlign[w]=-1,R=1,"auto"===(S=[this.tickAlign[w],.5,R])[0]?S[0]=0:S[0]=parseInt(""+S[0]),z=[0,0,0],B(w,H,U);for(O=0;O<3;++O)V[O]+=x*H[O]*this.tickPad[O]/n[5*O];this._text.drawTicks(w,this.tickSize[w],this.tickAngle[w],V,this.tickColor[w],Y,z,S)}if(this.labelEnable[w]){R=0,z=[0,0,0],this.labels[w].length>4&&(F(w),R=1),"auto"===(S=[this.labelAlign[w],.5,R])[0]?S[0]=0:S[0]=parseInt(""+S[0]);for(O=0;O<3;++O)V[O]+=x*H[O]*this.labelPad[O]/n[5*O];V[w]+=.5*(o[0][w]+o[1][w]),this._text.drawLabel(w,this.labelSize[w],this.labelAngle[w],V,this.labelColor[w],[0,0,0],z,S)}}this._text.unbind()},d.dispose=function(){this._text.dispose(),this._lines.dispose(),this._background.dispose(),this._lines=null,this._text=null,this._background=null,this.gl=null}},function(e,t,n){"use strict";(function(t){e.exports=function(e,t,n,o,s,l){var c=r(e),d=i(e,[{buffer:c,size:3}]),h=a(e);h.attributes.position.location=0;var f=new u(e,h,c,d);return f.update(t,n,o,s,l),f};var r=n(180),i=n(181),o=n(1041),a=n(576).text,s=window||t.global||{},l=s.__TEXT_CACHE||{};s.__TEXT_CACHE={};function u(e,t,n,r){this.gl=e,this.shader=t,this.buffer=n,this.vao=r,this.tickOffset=this.tickCount=this.labelOffset=this.labelCount=null}var c=u.prototype,d=[0,0];c.bind=function(e,t,n,r){this.vao.bind(),this.shader.bind();var i=this.shader.uniforms;i.model=e,i.view=t,i.projection=n,i.pixelScale=r,d[0]=this.gl.drawingBufferWidth,d[1]=this.gl.drawingBufferHeight,this.shader.uniforms.resolution=d},c.unbind=function(){this.vao.unbind()},c.update=function(e,t,n,r,i){var a=[];function s(e,t,n,r,i,s){var u=l[n];u||(u=l[n]={});var c=u[t];c||(c=u[t]=function(e,t){try{return o(e,t)}catch(t){return console.warn('error vectorizing text:"'+e+'" error:',t),{cells:[],positions:[]}}}(t,{triangles:!0,font:n,textAlign:"center",textBaseline:"middle",lineSpacing:i,styletags:s}));for(var d=(r||12)/12,h=c.positions,f=c.cells,p=0,g=f.length;p<g;++p)for(var v=f[p],m=2;m>=0;--m){var y=h[v[m]];a.push(d*y[0],-d*y[1],e)}}for(var u=[0,0,0],c=[0,0,0],d=[0,0,0],h=[0,0,0],f={breaklines:!0,bolds:!0,italics:!0,subscripts:!0,superscripts:!0},p=0;p<3;++p){d[p]=a.length/3|0,s(.5*(e[0][p]+e[1][p]),t[p],n[p],12,1.25,f),h[p]=(a.length/3|0)-d[p],u[p]=a.length/3|0;for(var g=0;g<r[p].length;++g)r[p][g].text&&s(r[p][g].x,r[p][g].text,r[p][g].font||i,r[p][g].fontSize||12,1.25,f);c[p]=(a.length/3|0)-u[p]}this.buffer.update(a),this.tickOffset=u,this.tickCount=c,this.labelOffset=d,this.labelCount=h},c.drawTicks=function(e,t,n,r,i,o,a,s){this.tickCount[e]&&(this.shader.uniforms.axis=o,this.shader.uniforms.color=i,this.shader.uniforms.angle=n,this.shader.uniforms.scale=t,this.shader.uniforms.offset=r,this.shader.uniforms.alignDir=a,this.shader.uniforms.alignOpt=s,this.vao.draw(this.gl.TRIANGLES,this.tickCount[e],this.tickOffset[e]))},c.drawLabel=function(e,t,n,r,i,o,a,s){this.labelCount[e]&&(this.shader.uniforms.axis=o,this.shader.uniforms.color=i,this.shader.uniforms.angle=n,this.shader.uniforms.scale=t,this.shader.uniforms.offset=r,this.shader.uniforms.alignDir=a,this.shader.uniforms.alignOpt=s,this.vao.draw(this.gl.TRIANGLES,this.labelCount[e],this.labelOffset[e]))},c.dispose=function(){this.shader.dispose(),this.vao.dispose(),this.buffer.dispose()}}).call(this,n(145))},function(e,t,n){"use strict";e.exports=function(e,t,n){var a=[],s=[0,0,0],l=[0,0,0],u=[0,0,0],c=[0,0,0];a.push(0,0,1,0,1,1,0,0,-1,0,0,-1,0,1,1,0,1,-1);for(var d=0;d<3;++d){for(var h=a.length/3|0,p=0;p<n[d].length;++p){var g=+n[d][p].x;a.push(g,0,1,g,1,1,g,0,-1,g,0,-1,g,1,1,g,1,-1)}var v=a.length/3|0;s[d]=h,l[d]=v-h;h=a.length/3|0;for(var m=0;m<n[d].length;++m){g=+n[d][m].x;a.push(g,0,1,g,1,1,g,0,-1,g,0,-1,g,1,1,g,1,-1)}v=a.length/3|0;u[d]=h,c[d]=v-h}var y=r(e,new Float32Array(a)),b=i(e,[{buffer:y,type:e.FLOAT,size:3,stride:0,offset:0}]),C=o(e);return C.attributes.position.location=0,new f(e,y,b,C,l,s,c,u)};var r=n(180),i=n(181),o=n(576).line,a=[0,0,0],s=[0,0,0],l=[0,0,0],u=[0,0,0],c=[1,1];function d(e){return e[0]=e[1]=e[2]=0,e}function h(e,t){return e[0]=t[0],e[1]=t[1],e[2]=t[2],e}function f(e,t,n,r,i,o,a,s){this.gl=e,this.vertBuffer=t,this.vao=n,this.shader=r,this.tickCount=i,this.tickOffset=o,this.gridCount=a,this.gridOffset=s}var p=f.prototype;p.bind=function(e,t,n){this.shader.bind(),this.shader.uniforms.model=e,this.shader.uniforms.view=t,this.shader.uniforms.projection=n,c[0]=this.gl.drawingBufferWidth,c[1]=this.gl.drawingBufferHeight,this.shader.uniforms.screenShape=c,this.vao.bind()},p.unbind=function(){this.vao.unbind()},p.drawAxisLine=function(e,t,n,r,i){var o=d(s);this.shader.uniforms.majorAxis=s,o[e]=t[1][e]-t[0][e],this.shader.uniforms.minorAxis=o;var a,c=h(u,n);c[e]+=t[0][e],this.shader.uniforms.offset=c,this.shader.uniforms.lineWidth=i,this.shader.uniforms.color=r,(a=d(l))[(e+2)%3]=1,this.shader.uniforms.screenAxis=a,this.vao.draw(this.gl.TRIANGLES,6),(a=d(l))[(e+1)%3]=1,this.shader.uniforms.screenAxis=a,this.vao.draw(this.gl.TRIANGLES,6)},p.drawAxisTicks=function(e,t,n,r,i){if(this.tickCount[e]){var o=d(a);o[e]=1,this.shader.uniforms.majorAxis=o,this.shader.uniforms.offset=t,this.shader.uniforms.minorAxis=n,this.shader.uniforms.color=r,this.shader.uniforms.lineWidth=i;var s=d(l);s[e]=1,this.shader.uniforms.screenAxis=s,this.vao.draw(this.gl.TRIANGLES,this.tickCount[e],this.tickOffset[e])}},p.drawGrid=function(e,t,n,r,i,o){if(this.gridCount[e]){var c=d(s);c[t]=n[1][t]-n[0][t],this.shader.uniforms.minorAxis=c;var f=h(u,r);f[t]+=n[0][t],this.shader.uniforms.offset=f;var p=d(a);p[e]=1,this.shader.uniforms.majorAxis=p;var g=d(l);g[e]=1,this.shader.uniforms.screenAxis=g,this.shader.uniforms.lineWidth=o,this.shader.uniforms.color=i,this.vao.draw(this.gl.TRIANGLES,this.gridCount[e],this.gridOffset[e])}},p.drawZero=function(e,t,n,r,i,o){var a=d(s);this.shader.uniforms.majorAxis=a,a[e]=n[1][e]-n[0][e],this.shader.uniforms.minorAxis=a;var c=h(u,r);c[e]+=n[0][e],this.shader.uniforms.offset=c;var f=d(l);f[t]=1,this.shader.uniforms.screenAxis=f,this.shader.uniforms.lineWidth=o,this.shader.uniforms.color=i,this.vao.draw(this.gl.TRIANGLES,6)},p.dispose=function(){this.vao.dispose(),this.vertBuffer.dispose(),this.shader.dispose()}},function(e,t,n){"use strict";e.exports=function(e){for(var t=[],n=[],s=0,l=0;l<3;++l)for(var u=(l+1)%3,c=(l+2)%3,d=[0,0,0],h=[0,0,0],f=-1;f<=1;f+=2){n.push(s,s+2,s+1,s+1,s+2,s+3),d[l]=f,h[l]=f;for(var p=-1;p<=1;p+=2){d[u]=p;for(var g=-1;g<=1;g+=2)d[c]=g,t.push(d[0],d[1],d[2],h[0],h[1],h[2]),s+=1}var v=u;u=c,c=v}var m=r(e,new Float32Array(t)),y=r(e,new Uint16Array(n),e.ELEMENT_ARRAY_BUFFER),b=i(e,[{buffer:m,type:e.FLOAT,size:3,offset:0,stride:24},{buffer:m,type:e.FLOAT,size:3,offset:12,stride:24}],y),C=o(e);return C.attributes.position.location=0,C.attributes.normal.location=1,new a(e,m,b,C)};var r=n(180),i=n(181),o=n(576).bg;function a(e,t,n,r){this.gl=e,this.buffer=t,this.vao=n,this.shader=r}var s=a.prototype;s.draw=function(e,t,n,r,i,o){for(var a=!1,s=0;s<3;++s)a=a||i[s];if(a){var l=this.gl;l.enable(l.POLYGON_OFFSET_FILL),l.polygonOffset(1,2),this.shader.bind(),this.shader.uniforms={model:e,view:t,projection:n,bounds:r,enable:i,colors:o},this.vao.bind(),this.vao.draw(this.gl.TRIANGLES,36),this.vao.unbind(),l.disable(l.POLYGON_OFFSET_FILL)}},s.dispose=function(){this.vao.dispose(),this.buffer.dispose(),this.shader.dispose()}},function(e,t,n){"use strict";var r=n(257),i=n(197);e.exports=function(e,t){for(var n=r(e[0],t[0]),o=1;o<e.length;++o)n=i(n,r(e[o],t[o]));return n}},function(e,t,n){"use strict";function r(e,t){var n=e+"",r=n.indexOf("."),i=0;r>=0&&(i=n.length-r-1);var o=Math.pow(10,i),a=Math.round(e*t*o),s=a+"";if(s.indexOf("e")>=0)return s;var l=a/o,u=a%o;a<0?(l=0|-Math.ceil(l),u=0|-u):(l=0|Math.floor(l),u|=0);var c=""+l;if(a<0&&(c="-"+c),i){for(var d=""+u;d.length<i;)d="0"+d;return c+"."+d}return c}t.create=function(e,t){for(var n=[],i=0;i<3;++i){for(var o=[],a=(e[0][i],e[1][i],0);a*t[i]<=e[1][i];++a)o.push({x:a*t[i],text:r(t[i],a)});for(a=-1;a*t[i]>=e[0][i];--a)o.push({x:a*t[i],text:r(t[i],a)});n.push(o)}return n},t.equal=function(e,t){for(var n=0;n<3;++n){if(e[n].length!==t[n].length)return!1;for(var r=0;r<e[n].length;++r){var i=e[n][r],o=t[n][r];if(i.x!==o.x||i.text!==o.text||i.font!==o.font||i.fontColor!==o.fontColor||i.fontSize!==o.fontSize||i.dx!==o.dx||i.dy!==o.dy)return!1}}return!0}},function(e,t,n){"use strict";e.exports=function(e,t,n,l,d){var h=t.model||u,f=t.view||u,m=t.projection||u,y=t._ortho||!1,b=e.bounds,C=(d=d||o(h,f,m,b,y)).axis;a(c,f,h),a(c,m,c);for(var A=g,_=0;_<3;++_)A[_].lo=1/0,A[_].hi=-1/0,A[_].pixelsPerDataUnit=1/0;var I=r(s(c,c));s(c,c);for(var x=0;x<3;++x){var w=(x+1)%3,M=(x+2)%3,S=v;e:for(_=0;_<2;++_){var T=[];if(C[x]<0!=!!_){S[x]=b[_][x];for(var E=0;E<2;++E){S[w]=b[E^_][w];for(var O=0;O<2;++O)S[M]=b[O^E^_][M],T.push(S.slice())}var k=y?5:4;for(E=k;E===k;++E){if(0===T.length)continue e;T=i.positive(T,I[E])}for(E=0;E<T.length;++E){M=T[E];var L=p(v,c,M,n,l);for(O=0;O<3;++O)A[O].lo=Math.min(A[O].lo,M[O]),A[O].hi=Math.max(A[O].hi,M[O]),O!==x&&(A[O].pixelsPerDataUnit=Math.min(A[O].pixelsPerDataUnit,Math.abs(L[O])))}}}}return A};var r=n(1944),i=n(1067),o=n(1066),a=n(255),s=n(531),l=n(1945),u=new Float32Array([1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1]),c=new Float32Array(16);function d(e,t,n){this.lo=e,this.hi=t,this.pixelsPerDataUnit=n}var h=[0,0,0,1],f=[0,0,0,1];function p(e,t,n,r,i){for(var o=0;o<3;++o){for(var a=h,s=f,u=0;u<3;++u)s[u]=a[u]=n[u];s[3]=a[3]=1,s[o]+=1,l(s,s,t),s[3]<0&&(e[o]=1/0),a[o]-=1,l(a,a,t),a[3]<0&&(e[o]=1/0);var c=(a[0]/a[3]-s[0]/s[3])*r,d=(a[1]/a[3]-s[1]/s[3])*i;e[o]=.25*Math.sqrt(c*c+d*d)}return e}var g=[new d(1/0,-1/0,1/0),new d(1/0,-1/0,1/0),new d(1/0,-1/0,1/0)],v=[0,0,0]},function(e,t,n){"use strict";e.exports=function(e,t,n){var r=t||0,i=n||1;return[[e[12]+e[0],e[13]+e[1],e[14]+e[2],e[15]+e[3]],[e[12]-e[0],e[13]-e[1],e[14]-e[2],e[15]-e[3]],[e[12]+e[4],e[13]+e[5],e[14]+e[6],e[15]+e[7]],[e[12]-e[4],e[13]-e[5],e[14]-e[6],e[15]-e[7]],[r*e[12]+e[8],r*e[13]+e[9],r*e[14]+e[10],r*e[15]+e[11]],[i*e[12]-e[8],i*e[13]-e[9],i*e[14]-e[10],i*e[15]-e[11]]]}},function(e,t){e.exports=function(e,t,n){var r=t[0],i=t[1],o=t[2],a=t[3];return e[0]=n[0]*r+n[4]*i+n[8]*o+n[12]*a,e[1]=n[1]*r+n[5]*i+n[9]*o+n[13]*a,e[2]=n[2]*r+n[6]*i+n[10]*o+n[14]*a,e[3]=n[3]*r+n[7]*i+n[11]*o+n[15]*a,e}},function(e,t,n){"use strict";var r=n(180),i=n(181),o=n(1947);e.exports=function(e,t){var n=[];function a(e,t,r,i,o,a){var s=[e,t,r,0,0,0,1];s[i+3]=1,s[i]=o,n.push.apply(n,s),s[6]=-1,n.push.apply(n,s),s[i]=a,n.push.apply(n,s),n.push.apply(n,s),s[6]=1,n.push.apply(n,s),s[i]=o,n.push.apply(n,s)}a(0,0,0,0,0,1),a(0,0,0,1,0,1),a(0,0,0,2,0,1),a(1,0,0,1,-1,1),a(1,0,0,2,-1,1),a(0,1,0,0,-1,1),a(0,1,0,2,-1,1),a(0,0,1,0,-1,1),a(0,0,1,1,-1,1);var l=r(e,n),u=i(e,[{type:e.FLOAT,buffer:l,size:3,offset:0,stride:28},{type:e.FLOAT,buffer:l,size:3,offset:12,stride:28},{type:e.FLOAT,buffer:l,size:1,offset:24,stride:28}]),c=o(e);c.attributes.position.location=0,c.attributes.color.location=1,c.attributes.weight.location=2;var d=new s(e,l,u,c);return d.update(t),d};var a=[1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1];function s(e,t,n,r){this.gl=e,this.buffer=t,this.vao=n,this.shader=r,this.pixelRatio=1,this.bounds=[[-1e3,-1e3,-1e3],[1e3,1e3,1e3]],this.position=[0,0,0],this.lineWidth=[2,2,2],this.colors=[[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.enabled=[!0,!0,!0],this.drawSides=[!0,!0,!0],this.axes=null}var l=s.prototype,u=[0,0,0],c=[0,0,0],d=[0,0];l.isTransparent=function(){return!1},l.drawTransparent=function(e){},l.draw=function(e){var t=this.gl,n=this.vao,r=this.shader;n.bind(),r.bind();var i,o=e.model||a,s=e.view||a,l=e.projection||a;this.axes&&(i=this.axes.lastCubeProps.axis);for(var h=u,f=c,p=0;p<3;++p)i&&i[p]<0?(h[p]=this.bounds[0][p],f[p]=this.bounds[1][p]):(h[p]=this.bounds[1][p],f[p]=this.bounds[0][p]);d[0]=t.drawingBufferWidth,d[1]=t.drawingBufferHeight,r.uniforms.model=o,r.uniforms.view=s,r.uniforms.projection=l,r.uniforms.coordinates=[this.position,h,f],r.uniforms.colors=this.colors,r.uniforms.screenShape=d;for(p=0;p<3;++p)r.uniforms.lineWidth=this.lineWidth[p]*this.pixelRatio,this.enabled[p]&&(n.draw(t.TRIANGLES,6,6*p),this.drawSides[p]&&n.draw(t.TRIANGLES,12,18+12*p));n.unbind()},l.update=function(e){e&&("bounds"in e&&(this.bounds=e.bounds),"position"in e&&(this.position=e.position),"lineWidth"in e&&(this.lineWidth=e.lineWidth),"colors"in e&&(this.colors=e.colors),"enabled"in e&&(this.enabled=e.enabled),"drawSides"in e&&(this.drawSides=e.drawSides))},l.dispose=function(){this.vao.dispose(),this.buffer.dispose(),this.shader.dispose()}},function(e,t,n){"use strict";var r=n(217),i=n(218),o=r(["precision mediump float;\n#define GLSLIFY 1\n\nattribute vec3 position, color;\nattribute float weight;\n\nuniform mat4 model, view, projection;\nuniform vec3 coordinates[3];\nuniform vec4 colors[3];\nuniform vec2 screenShape;\nuniform float lineWidth;\n\nvarying vec4 fragColor;\n\nvoid main() {\n  vec3 vertexPosition = mix(coordinates[0],\n    mix(coordinates[2], coordinates[1], 0.5 * (position + 1.0)), abs(position));\n\n  vec4 clipPos = projection * view * model * vec4(vertexPosition, 1.0);\n  vec2 clipOffset = (projection * view * model * vec4(color, 0.0)).xy;\n  vec2 delta = weight * clipOffset * screenShape;\n  vec2 lineOffset = normalize(vec2(delta.y, -delta.x)) / screenShape;\n\n  gl_Position   = vec4(clipPos.xy + clipPos.w * 0.5 * lineWidth * lineOffset, clipPos.z, clipPos.w);\n  fragColor     = color.x * colors[0] + color.y * colors[1] + color.z * colors[2];\n}\n"]),a=r(["precision mediump float;\n#define GLSLIFY 1\n\nvarying vec4 fragColor;\n\nvoid main() {\n  gl_FragColor = fragColor;\n}"]);e.exports=function(e){return i(e,o,a,null,[{name:"position",type:"vec3"},{name:"color",type:"vec3"},{name:"weight",type:"float"}])}},function(e,t,n){"use strict";e.exports=function(e,t){var n=t[0],o=t[1],a=r(e,n,o,{}),s=i.mallocUint8(n*o*4);return new l(e,a,s)};var r=n(1068),i=n(112),o=n(216),a=n(215).nextPow2;function s(e,t,n,r,i){this.coord=[e,t],this.id=n,this.value=r,this.distance=i}function l(e,t,n){this.gl=e,this.fbo=t,this.buffer=n,this._readTimeout=null;var r=this;this._readCallback=function(){r.gl&&(t.bind(),e.readPixels(0,0,t.shape[0],t.shape[1],e.RGBA,e.UNSIGNED_BYTE,r.buffer),r._readTimeout=null)}}var u=l.prototype;Object.defineProperty(u,"shape",{get:function(){return this.gl?this.fbo.shape.slice():[0,0]},set:function(e){if(this.gl){this.fbo.shape=e;var t=this.fbo.shape[0],n=this.fbo.shape[1];if(n*t*4>this.buffer.length){i.free(this.buffer);for(var r=this.buffer=i.mallocUint8(a(n*t*4)),o=0;o<n*t*4;++o)r[o]=255}return e}}}),u.begin=function(){var e=this.gl;this.shape;e&&(this.fbo.bind(),e.clearColor(1,1,1,1),e.clear(e.COLOR_BUFFER_BIT|e.DEPTH_BUFFER_BIT))},u.end=function(){var e=this.gl;e&&(e.bindFramebuffer(e.FRAMEBUFFER,null),this._readTimeout||clearTimeout(this._readTimeout),this._readTimeout=setTimeout(this._readCallback,1))},u.query=function(e,t,n){if(!this.gl)return null;var r=this.fbo.shape.slice();e|=0,t|=0,"number"!=typeof n&&(n=1);var i=0|Math.min(Math.max(e-n,0),r[0]),a=0|Math.min(Math.max(e+n,0),r[0]),l=0|Math.min(Math.max(t-n,0),r[1]),u=0|Math.min(Math.max(t+n,0),r[1]);if(a<=i||u<=l)return null;var c=[a-i,u-l],d=o(this.buffer,[c[0],c[1],4],[4,4*r[0],1],4*(i+r[0]*l)),h=function(e,t,n){for(var r=1e8,i=-1,o=-1,a=e.shape[0],s=e.shape[1],l=0;l<a;l++)for(var u=0;u<s;u++){var c=e.get(l,u,0),d=e.get(l,u,1),h=e.get(l,u,2),f=e.get(l,u,3);if(c<255||d<255||h<255||f<255){var p=t-l,g=n-u,v=p*p+g*g;v<r&&(r=v,i=l,o=u)}}return[i,o,r]}(d.hi(c[0],c[1],1),n,n),f=h[0],p=h[1];return f<0||Math.pow(this.radius,2)<h[2]?null:new s(f+i|0,p+l|0,d.get(f,p,0),[d.get(f,p,1),d.get(f,p,2),d.get(f,p,3)],Math.sqrt(h[2]))},u.dispose=function(){this.gl&&(this.fbo.dispose(),i.free(this.buffer),this.gl=null,this._readTimeout&&clearTimeout(this._readTimeout))}},function(e,t,n){"use strict";var r="undefined"==typeof WeakMap?n(1950):WeakMap,i=n(180),o=n(181),a=new r;e.exports=function(e){var t=a.get(e),n=t&&(t._triangleBuffer.handle||t._triangleBuffer.buffer);if(!n||!e.isBuffer(n)){var r=i(e,new Float32Array([-1,-1,-1,4,4,-1]));(t=o(e,[{buffer:r,type:e.FLOAT,size:2}]))._triangleBuffer=r,a.set(e,t)}t.bind(),e.drawArrays(e.TRIANGLES,0,3),t.unbind()}},function(e,t){!function(){"use strict";if("undefined"==typeof ses||!ses.ok||ses.ok()){"undefined"!=typeof ses&&(ses.weakMapPermitHostObjects=g);var t=!1;if("function"==typeof WeakMap){var n=WeakMap;if("undefined"!=typeof navigator&&/Firefox/.test(navigator.userAgent));else{var r=new n,i=Object.freeze({});if(r.set(i,1),1===r.get(i))return void(e.exports=WeakMap);t=!0}}Object.prototype.hasOwnProperty;var o=Object.getOwnPropertyNames,a=Object.defineProperty,s=Object.isExtensible,l="weakmap:ident:"+Math.random()+"___";if("undefined"!=typeof crypto&&"function"==typeof crypto.getRandomValues&&"function"==typeof ArrayBuffer&&"function"==typeof Uint8Array){var u=new ArrayBuffer(25),c=new Uint8Array(u);crypto.getRandomValues(c),l="weakmap:rand:"+Array.prototype.map.call(c,(function(e){return(e%36).toString(36)})).join("")+"___"}if(a(Object,"getOwnPropertyNames",{value:function(e){return o(e).filter(v)}}),"getPropertyNames"in Object){var d=Object.getPropertyNames;a(Object,"getPropertyNames",{value:function(e){return d(e).filter(v)}})}!function(){var e=Object.freeze;a(Object,"freeze",{value:function(t){return m(t),e(t)}});var t=Object.seal;a(Object,"seal",{value:function(e){return m(e),t(e)}});var n=Object.preventExtensions;a(Object,"preventExtensions",{value:function(e){return m(e),n(e)}})}();var h=!1,f=0,p=function(){this instanceof p||b();var e=[],t=[],n=f++;return Object.create(p.prototype,{get___:{value:y((function(r,i){var o,a=m(r);return a?n in a?a[n]:i:(o=e.indexOf(r))>=0?t[o]:i}))},has___:{value:y((function(t){var r=m(t);return r?n in r:e.indexOf(t)>=0}))},set___:{value:y((function(r,i){var o,a=m(r);return a?a[n]=i:(o=e.indexOf(r))>=0?t[o]=i:(o=e.length,t[o]=i,e[o]=r),this}))},delete___:{value:y((function(r){var i,o,a=m(r);return a?n in a&&delete a[n]:!((i=e.indexOf(r))<0)&&(o=e.length-1,e[i]=void 0,t[i]=t[o],e[i]=e[o],e.length=o,t.length=o,!0)}))}})};p.prototype=Object.create(Object.prototype,{get:{value:function(e,t){return this.get___(e,t)},writable:!0,configurable:!0},has:{value:function(e){return this.has___(e)},writable:!0,configurable:!0},set:{value:function(e,t){return this.set___(e,t)},writable:!0,configurable:!0},delete:{value:function(e){return this.delete___(e)},writable:!0,configurable:!0}}),"function"==typeof n?function(){function r(){this instanceof p||b();var e,r=new n,i=void 0,o=!1;return e=t?function(e,t){return r.set(e,t),r.has(e)||(i||(i=new p),i.set(e,t)),this}:function(e,t){if(o)try{r.set(e,t)}catch(n){i||(i=new p),i.set___(e,t)}else r.set(e,t);return this},Object.create(p.prototype,{get___:{value:y((function(e,t){return i?r.has(e)?r.get(e):i.get___(e,t):r.get(e,t)}))},has___:{value:y((function(e){return r.has(e)||!!i&&i.has___(e)}))},set___:{value:y(e)},delete___:{value:y((function(e){var t=!!r.delete(e);return i&&i.delete___(e)||t}))},permitHostObjects___:{value:y((function(e){if(e!==g)throw new Error("bogus call to permitHostObjects___");o=!0}))}})}t&&"undefined"!=typeof Proxy&&(Proxy=void 0),r.prototype=p.prototype,e.exports=r,Object.defineProperty(WeakMap.prototype,"constructor",{value:WeakMap,enumerable:!1,configurable:!0,writable:!0})}():("undefined"!=typeof Proxy&&(Proxy=void 0),e.exports=p)}function g(e){e.permitHostObjects___&&e.permitHostObjects___(g)}function v(e){return!("weakmap:"==e.substr(0,"weakmap:".length)&&"___"===e.substr(e.length-3))}function m(e){if(e!==Object(e))throw new TypeError("Not an object: "+e);var t=e[l];if(t&&t.key===e)return t;if(s(e)){t={key:e};try{return a(e,l,{value:t,writable:!1,enumerable:!1,configurable:!1}),t}catch(e){return}}}function y(e){return e.prototype=null,Object.freeze(e)}function b(){h||"undefined"==typeof console||(h=!0,console.warn("WeakMap should be invoked as new WeakMap(), not WeakMap(). This will be an error in the future."))}}()},function(e,t,n){var r=n(217),i=n(218),o=r(["precision mediump float;\n#define GLSLIFY 1\nattribute vec2 position;\nvarying vec2 uv;\nvoid main() {\n  uv = position;\n  gl_Position = vec4(position, 0, 1);\n}"]),a=r(["precision mediump float;\n#define GLSLIFY 1\n\nuniform sampler2D accumBuffer;\nvarying vec2 uv;\n\nvoid main() {\n  vec4 accum = texture2D(accumBuffer, 0.5 * (uv + 1.0));\n  gl_FragColor = min(vec4(1,1,1,1), accum);\n}"]);e.exports=function(e){return i(e,o,a,null,[{name:"position",type:"vec2"}])}},function(e,t,n){var r=n(1953);e.exports=function(e){return r("webgl",e)}},function(e,t){e.exports=function(e,t){if("string"!=typeof e)throw new TypeError("must specify type string");if(t=t||{},"undefined"==typeof document&&!t.canvas)return null;var n=t.canvas||document.createElement("canvas");"number"==typeof t.width&&(n.width=t.width);"number"==typeof t.height&&(n.height=t.height);var r,i=t;try{var o=[e];0===e.indexOf("webgl")&&o.push("experimental-"+e);for(var a=0;a<o.length;a++)if(r=n.getContext(o[a],i))return r}catch(e){r=null}return r||null}},function(e,t,n){"use strict";var r=n(41),i=function(){};e.exports=function(e){for(var t in e)"function"==typeof e[t]&&(e[t]=i);e.destroy=function(){e.container.parentNode.removeChild(e.container)};var n=document.createElement("div");n.className="no-webgl",n.style.cursor="pointer",n.style.fontSize="24px",n.style.color=r.defaults[0],n.style.position="absolute",n.style.left=n.style.top="0px",n.style.width=n.style.height="100%",n.style["background-color"]=r.lightLine,n.style["z-index"]=30;var o=document.createElement("p");return o.textContent="WebGL is not supported by your browser - visit https://get.webgl.org for more info",o.style.position="relative",o.style.top="50%",o.style.left="50%",o.style.height="30%",o.style.width="50%",o.style.margin="-15% 0 0 -25%",n.appendChild(o),e.container.appendChild(n),e.container.style.background="#FFFFFF",e.container.onclick=function(){window.open("https://get.webgl.org")},!1}},function(e,t,n){"use strict";var r=n(304),i=n(10),o=["xaxis","yaxis","zaxis"];function a(){this.bounds=[[-10,-10,-10],[10,10,10]],this.ticks=[[],[],[]],this.tickEnable=[!0,!0,!0],this.tickFont=["sans-serif","sans-serif","sans-serif"],this.tickSize=[12,12,12],this.tickAngle=[0,0,0],this.tickColor=[[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.tickPad=[18,18,18],this.labels=["x","y","z"],this.labelEnable=[!0,!0,!0],this.labelFont=["Open Sans","Open Sans","Open Sans"],this.labelSize=[20,20,20],this.labelColor=[[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.labelPad=[30,30,30],this.lineEnable=[!0,!0,!0],this.lineMirror=[!1,!1,!1],this.lineWidth=[1,1,1],this.lineColor=[[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.lineTickEnable=[!0,!0,!0],this.lineTickMirror=[!1,!1,!1],this.lineTickLength=[10,10,10],this.lineTickWidth=[1,1,1],this.lineTickColor=[[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.gridEnable=[!0,!0,!0],this.gridWidth=[1,1,1],this.gridColor=[[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.zeroEnable=[!0,!0,!0],this.zeroLineColor=[[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.zeroLineWidth=[2,2,2],this.backgroundEnable=[!0,!0,!0],this.backgroundColor=[[.8,.8,.8,.5],[.8,.8,.8,.5],[.8,.8,.8,.5]],this._defaultTickPad=this.tickPad.slice(),this._defaultLabelPad=this.labelPad.slice(),this._defaultLineTickLength=this.lineTickLength.slice()}a.prototype.merge=function(e,t){for(var n=0;n<3;++n){var a=t[o[n]];a.visible?(this.labels[n]=e._meta?i.templateString(a.title.text,e._meta):a.title.text,"font"in a.title&&(a.title.font.color&&(this.labelColor[n]=r(a.title.font.color)),a.title.font.family&&(this.labelFont[n]=a.title.font.family),a.title.font.size&&(this.labelSize[n]=a.title.font.size)),"showline"in a&&(this.lineEnable[n]=a.showline),"linecolor"in a&&(this.lineColor[n]=r(a.linecolor)),"linewidth"in a&&(this.lineWidth[n]=a.linewidth),"showgrid"in a&&(this.gridEnable[n]=a.showgrid),"gridcolor"in a&&(this.gridColor[n]=r(a.gridcolor)),"gridwidth"in a&&(this.gridWidth[n]=a.gridwidth),"log"===a.type?this.zeroEnable[n]=!1:"zeroline"in a&&(this.zeroEnable[n]=a.zeroline),"zerolinecolor"in a&&(this.zeroLineColor[n]=r(a.zerolinecolor)),"zerolinewidth"in a&&(this.zeroLineWidth[n]=a.zerolinewidth),"ticks"in a&&a.ticks?this.lineTickEnable[n]=!0:this.lineTickEnable[n]=!1,"ticklen"in a&&(this.lineTickLength[n]=this._defaultLineTickLength[n]=a.ticklen),"tickcolor"in a&&(this.lineTickColor[n]=r(a.tickcolor)),"tickwidth"in a&&(this.lineTickWidth[n]=a.tickwidth),"tickangle"in a&&(this.tickAngle[n]="auto"===a.tickangle?-3600:Math.PI*-a.tickangle/180),"showticklabels"in a&&(this.tickEnable[n]=a.showticklabels),"tickfont"in a&&(a.tickfont.color&&(this.tickColor[n]=r(a.tickfont.color)),a.tickfont.family&&(this.tickFont[n]=a.tickfont.family),a.tickfont.size&&(this.tickSize[n]=a.tickfont.size)),"mirror"in a?-1!==["ticks","all","allticks"].indexOf(a.mirror)?(this.lineTickMirror[n]=!0,this.lineMirror[n]=!0):!0===a.mirror?(this.lineTickMirror[n]=!1,this.lineMirror[n]=!0):(this.lineTickMirror[n]=!1,this.lineMirror[n]=!1):this.lineMirror[n]=!1,"showbackground"in a&&!1!==a.showbackground?(this.backgroundEnable[n]=!0,this.backgroundColor[n]=r(a.backgroundcolor)):this.backgroundEnable[n]=!1):(this.tickEnable[n]=!1,this.labelEnable[n]=!1,this.lineEnable[n]=!1,this.lineTickEnable[n]=!1,this.gridEnable[n]=!1,this.zeroEnable[n]=!1,this.backgroundEnable[n]=!1)}},e.exports=function(e,t){var n=new a;return n.merge(e,t),n}},function(e,t,n){"use strict";var r=n(304),i=["xaxis","yaxis","zaxis"];function o(){this.enabled=[!0,!0,!0],this.colors=[[0,0,0,1],[0,0,0,1],[0,0,0,1]],this.drawSides=[!0,!0,!0],this.lineWidth=[1,1,1]}o.prototype.merge=function(e){for(var t=0;t<3;++t){var n=e[i[t]];n.visible?(this.enabled[t]=n.showspikes,this.colors[t]=r(n.spikecolor),this.drawSides[t]=n.spikesides,this.lineWidth[t]=n.spikethickness):(this.enabled[t]=!1,this.drawSides[t]=!1)}},e.exports=function(e){var t=new o;return t.merge(e),t}},function(e,t,n){"use strict";e.exports=function(e){for(var t=e.axesOptions,n=e.glplot.axesPixels,s=e.fullSceneLayout,l=[[],[],[]],u=0;u<3;++u){var c=s[o[u]];if(c._length=(n[u].hi-n[u].lo)*n[u].pixelsPerDataUnit/e.dataScale[u],Math.abs(c._length)===1/0||isNaN(c._length))l[u]=[];else{c._input_range=c.range.slice(),c.range[0]=n[u].lo/e.dataScale[u],c.range[1]=n[u].hi/e.dataScale[u],c._m=1/(e.dataScale[u]*n[u].pixelsPerDataUnit),c.range[0]===c.range[1]&&(c.range[0]-=1,c.range[1]+=1);var d=c.tickmode;if("auto"===c.tickmode){c.tickmode="linear";var h=c.nticks||i.constrain(c._length/40,4,9);r.autoTicks(c,Math.abs(c.range[1]-c.range[0])/h)}for(var f=r.calcTicks(c,{msUTC:!0}),p=0;p<f.length;++p)f[p].x=f[p].x*e.dataScale[u],"date"===c.type&&(f[p].text=f[p].text.replace(/\<br\>/g," "));l[u]=f,c.tickmode=d}}t.ticks=l;for(u=0;u<3;++u){a[u]=.5*(e.glplot.bounds[0][u]+e.glplot.bounds[1][u]);for(p=0;p<2;++p)t.bounds[p][u]=e.glplot.bounds[p][u]}e.contourLevels=function(e){for(var t=new Array(3),n=0;n<3;++n){for(var r=e[n],i=new Array(r.length),o=0;o<r.length;++o)i[o]=r[o].x;t[n]=i}return t}(l)};var r=n(55),i=n(10),o=["xaxis","yaxis","zaxis"],a=[0,0,0]},function(e,t,n){"use strict";e.exports={scene:{valType:"subplotid",dflt:"scene",editType:"calc+clearAxisTypes"}}},function(e,t,n){"use strict";var r=n(10),i=n(41),o=n(27),a=n(1960),s=n(1961),l=n(1069),u=n(384).getSubplotData;function c(e,t,n,r){for(var a=n("bgcolor"),l=i.combine(a,r.paper_bgcolor),c=["up","center","eye"],d=0;d<c.length;d++)n("camera."+c[d]+".x"),n("camera."+c[d]+".y"),n("camera."+c[d]+".z");n("camera.projection.type");var h=!!n("aspectratio.x")&&!!n("aspectratio.y")&&!!n("aspectratio.z"),f=n("aspectmode",h?"manual":"auto");h||(e.aspectratio=t.aspectratio={x:1,y:1,z:1},"manual"===f&&(t.aspectmode="auto"),e.aspectmode=t.aspectmode);var p=u(r.fullData,"gl3d",r.id);s(e,t,{font:r.font,scene:r.id,data:p,bgColor:l,calendar:r.calendar,autotypenumbersDflt:r.autotypenumbersDflt,fullLayout:r.fullLayout}),o.getComponentMethod("annotations3d","handleDefaults")(e,t,r);var g=r.getDfltFromLayout("dragmode");if(!1!==g&&!g)if(g="orbit",e.camera&&e.camera.up){var v=e.camera.up.x,m=e.camera.up.y,y=e.camera.up.z;0!==y&&(v&&m&&y?y/Math.sqrt(v*v+m*m+y*y)>.999&&(g="turntable"):g="turntable")}else g="turntable";n("dragmode",g),n("hovermode",r.getDfltFromLayout("hovermode"))}e.exports=function(e,t,n){var i=t._basePlotModules.length>1;a(e,t,n,{type:"gl3d",attributes:l,handleDefaults:c,fullLayout:t,font:t.font,fullData:n,getDfltFromLayout:function(t){if(!i)return r.validate(e[t],l[t])?e[t]:void 0},autotypenumbersDflt:t.autotypenumbers,paper_bgcolor:t.paper_bgcolor,calendar:t.calendar})}},function(e,t,n){"use strict";var r=n(10),i=n(64),o=n(568).defaults;e.exports=function(e,t,n,a){var s,l,u=a.type,c=a.attributes,d=a.handleDefaults,h=a.partition||"x",f=t._subplots[u],p=f.length,g=p&&f[0].replace(/\d+$/,"");function v(e,t){return r.coerce(s,l,c,e,t)}for(var m=0;m<p;m++){var y=f[m];s=e[y]?e[y]:e[y]={},l=i.newContainer(t,y,g),v("uirevision",t.uirevision);var b={};b[h]=[m/p,(m+1)/p],o(l,t,v,b),a.id=y,d(s,l,v,a)}}},function(e,t,n){"use strict";var r=n(123).mix,i=n(10),o=n(64),a=n(1070),s=n(1018),l=n(559),u=["xaxis","yaxis","zaxis"];e.exports=function(e,t,n){var c,d;function h(e,t){return i.coerce(c,d,a,e,t)}for(var f=0;f<u.length;f++){var p=u[f];c=e[p]||{},(d=o.newContainer(t,p))._id=p[0]+n.scene,d._name=p,s(c,d,h,n),l(c,d,h,{font:n.font,letter:p[0],data:n.data,showGrid:!0,noTickson:!0,noTicklabelmode:!0,noTicklabelposition:!0,bgColor:n.bgColor,calendar:n.calendar},n.fullLayout),h("gridcolor",r(d.color,n.bgColor,13600/187).toRgbString()),h("title.text",p[0]),d.setScale=i.noop,h("showspikes")&&(h("spikesides"),h("spikethickness"),h("spikecolor",d.color)),h("showaxeslabels"),h("showbackground")&&h("backgroundcolor")}}},function(e,t,n){"use strict";e.exports={attributes:n(1072),supplyDefaults:n(1964),calc:n(1965),colorbar:{min:"cmin",max:"cmax"},plot:n(1966),moduleType:"trace",name:"mesh3d",basePlotModule:n(1061),categories:["gl3d","showLegend"],meta:{}}},function(e,t,n){"use strict";var r=n(41),i=n(161),o=n(177).hovertemplateAttrs,a=n(160),s=n(63).extendFlat,l=n(124).overrideAll;function u(e){return{show:{valType:"boolean",dflt:!1},start:{valType:"number",dflt:null,editType:"plot"},end:{valType:"number",dflt:null,editType:"plot"},size:{valType:"number",dflt:null,min:0,editType:"plot"},project:{x:{valType:"boolean",dflt:!1},y:{valType:"boolean",dflt:!1},z:{valType:"boolean",dflt:!1}},color:{valType:"color",dflt:r.defaultLine},usecolormap:{valType:"boolean",dflt:!1},width:{valType:"number",min:1,max:16,dflt:2},highlight:{valType:"boolean",dflt:!0},highlightcolor:{valType:"color",dflt:r.defaultLine},highlightwidth:{valType:"number",min:1,max:16,dflt:2}}}var c=e.exports=l(s({z:{valType:"data_array"},x:{valType:"data_array"},y:{valType:"data_array"},text:{valType:"string",dflt:"",arrayOk:!0},hovertext:{valType:"string",dflt:"",arrayOk:!0},hovertemplate:o(),connectgaps:{valType:"boolean",dflt:!1,editType:"calc"},surfacecolor:{valType:"data_array"}},i("",{colorAttr:"z or surfacecolor",showScaleDflt:!0,autoColorDflt:!1,editTypeOverride:"calc"}),{contours:{x:u(),y:u(),z:u()},hidesurface:{valType:"boolean",dflt:!1},lightposition:{x:{valType:"number",min:-1e5,max:1e5,dflt:10},y:{valType:"number",min:-1e5,max:1e5,dflt:1e4},z:{valType:"number",min:-1e5,max:1e5,dflt:0}},lighting:{ambient:{valType:"number",min:0,max:1,dflt:.8},diffuse:{valType:"number",min:0,max:1,dflt:.8},specular:{valType:"number",min:0,max:2,dflt:.05},roughness:{valType:"number",min:0,max:1,dflt:.5},fresnel:{valType:"number",min:0,max:5,dflt:.2}},opacity:{valType:"number",min:0,max:1,dflt:1},opacityscale:{valType:"any",editType:"calc"},_deprecated:{zauto:s({},i.zauto,{}),zmin:s({},i.zmin,{}),zmax:s({},i.zmax,{})},hoverinfo:s({},a.hoverinfo),showlegend:s({},a.showlegend,{dflt:!1})}),"calc","nested");c.x.editType=c.y.editType=c.z.editType="calc+clearAxisTypes",c.transforms=void 0},function(e,t,n){"use strict";var r=n(27),i=n(10),o=n(213),a=n(1072);e.exports=function(e,t,n,s){function l(n,r){return i.coerce(e,t,a,n,r)}function u(e){var t=e.map((function(e){var t=l(e);return t&&i.isArrayOrTypedArray(t)?t:null}));return t.every((function(e){return e&&e.length===t[0].length}))&&t}u(["x","y","z"])?(u(["i","j","k"]),(!t.i||t.j&&t.k)&&(!t.j||t.k&&t.i)&&(!t.k||t.i&&t.j)?(r.getComponentMethod("calendars","handleTraceDefaults")(e,t,["x","y","z"],s),["lighting.ambient","lighting.diffuse","lighting.specular","lighting.roughness","lighting.fresnel","lighting.vertexnormalsepsilon","lighting.facenormalsepsilon","lightposition.x","lightposition.y","lightposition.z","flatshading","alphahull","delaunayaxis","opacity"].forEach((function(e){l(e)})),l("contour.show")&&(l("contour.color"),l("contour.width")),"intensity"in e?(l("intensity"),l("intensitymode"),o(e,t,s,l,{prefix:"",cLetter:"c"})):(t.showscale=!1,"facecolor"in e?l("facecolor"):"vertexcolor"in e?l("vertexcolor"):l("color",n)),l("text"),l("hovertext"),l("hovertemplate"),t._length=null):t.visible=!1):t.visible=!1}},function(e,t,n){"use strict";var r=n(298);e.exports=function(e,t){t.intensity&&r(e,t,{vals:t.intensity,containerStr:"",cLetter:"c"})}},function(e,t,n){"use strict";var r=n(1053),i=n(573),o=n(1967),a=n(1054),s=n(1058).parseColorScale,l=n(304),u=n(195).extractOpts,c=n(1975);function d(e,t,n){this.scene=e,this.uid=n,this.mesh=t,this.name="",this.color="#fff",this.data=null,this.showContour=!1}var h=d.prototype;function f(e){for(var t=[],n=e.length,r=0;r<n;r++)t[r]=l(e[r]);return t}function p(e,t,n,r){for(var i=[],o=t.length,a=0;a<o;a++)i[a]=e.d2l(t[a],0,r)*n;return i}function g(e){for(var t=[],n=e.length,r=0;r<n;r++)t[r]=Math.round(e[r]);return t}function v(e,t){for(var n=e.length,r=0;r<n;r++)if(e[r]<=-.5||e[r]>=t-.5)return!1;return!0}h.handlePick=function(e){if(e.object===this.mesh){var t=e.index=e.data.index;e.data._cellCenter?e.traceCoordinate=e.data.dataCoordinate:e.traceCoordinate=[this.data.x[t],this.data.y[t],this.data.z[t]];var n=this.data.hovertext||this.data.text;return Array.isArray(n)&&void 0!==n[t]?e.textLabel=n[t]:n&&(e.textLabel=n),!0}},h.update=function(e){var t=this.scene,n=t.fullSceneLayout;this.data=e;var r,d=e.x.length,h=c(p(n.xaxis,e.x,t.dataScale[0],e.xcalendar),p(n.yaxis,e.y,t.dataScale[1],e.ycalendar),p(n.zaxis,e.z,t.dataScale[2],e.zcalendar));if(e.i&&e.j&&e.k){if(e.i.length!==e.j.length||e.j.length!==e.k.length||!v(e.i,d)||!v(e.j,d)||!v(e.k,d))return;r=c(g(e.i),g(e.j),g(e.k))}else r=0===e.alphahull?a(h):e.alphahull>0?o(e.alphahull,h):function(e,t){for(var n=["x","y","z"].indexOf(e),r=[],o=t.length,a=0;a<o;a++)r[a]=[t[a][(n+1)%3],t[a][(n+2)%3]];return i(r)}(e.delaunayaxis,h);var m={positions:h,cells:r,lightPosition:[e.lightposition.x,e.lightposition.y,e.lightposition.z],ambient:e.lighting.ambient,diffuse:e.lighting.diffuse,specular:e.lighting.specular,roughness:e.lighting.roughness,fresnel:e.lighting.fresnel,vertexNormalsEpsilon:e.lighting.vertexnormalsepsilon,faceNormalsEpsilon:e.lighting.facenormalsepsilon,opacity:e.opacity,contourEnable:e.contour.show,contourColor:l(e.contour.color).slice(0,3),contourWidth:e.contour.width,useFacetNormals:e.flatshading};if(e.intensity){var y=u(e);this.color="#fff";var b=e.intensitymode;m[b+"Intensity"]=e.intensity,m[b+"IntensityBounds"]=[y.min,y.max],m.colormap=s(e)}else e.vertexcolor?(this.color=e.vertexcolor[0],m.vertexColors=f(e.vertexcolor)):e.facecolor?(this.color=e.facecolor[0],m.cellColors=f(e.facecolor)):(this.color=e.color,m.meshColor=l(e.color));this.mesh.update(m)},h.dispose=function(){this.scene.glplot.remove(this.mesh),this.mesh.dispose()},e.exports=function(e,t){var n=e.glplot.gl,i=r({gl:n}),o=new d(e,i,t.uid);return i._trace=o,o.update(t),e.glplot.add(i),o}},function(e,t,n){e.exports=function(e,t){return i(r(e,t))};var r=n(1968),i=n(1971)},function(e,t,n){"use strict";e.exports=function(e,t){return r(t).filter((function(n){for(var r=new Array(n.length),o=0;o<n.length;++o)r[o]=t[n[o]];return i(r)*e<1}))};var r=n(573),i=n(1969)},function(e,t,n){e.exports=function(e){for(var t=r(e),n=0,i=0;i<e.length;++i)for(var o=e[i],a=0;a<t.length;++a)n+=Math.pow(o[a]-t[a],2);return Math.sqrt(n/e.length)};var r=n(1970)},function(e,t,n){"use strict";var r=n(1033),i=n(1056);function o(e,t){for(var n=0,r=e.length,i=0;i<r;++i)n+=e[i]*t[i];return n}function a(e){var t=e.length;if(0===t)return[];e[0].length;var n=r([e.length+1,e.length+1],1),a=r([e.length+1],1);n[t][t]=0;for(var s=0;s<t;++s){for(var l=0;l<=s;++l)n[l][s]=n[s][l]=2*o(e[s],e[l]);a[s]=o(e[s],e[s])}var u=i(n,a),c=0,d=u[t+1];for(s=0;s<d.length;++s)c+=d[s];var h=new Array(t);for(s=0;s<t;++s){d=u[s];var f=0;for(l=0;l<d.length;++l)f+=d[l];h[s]=f/c}return h}function s(e){if(0===e.length)return[];for(var t=e[0].length,n=r([t]),i=a(e),o=0;o<e.length;++o)for(var s=0;s<t;++s)n[s]+=e[o][s]*i[o];return n}s.barycenetric=a,e.exports=s},function(e,t,n){"use strict";e.exports=function(e){return i(r(e))};var r=n(1972),i=n(1973)},function(e,t,n){"use strict";e.exports=function(e){var t,n,r,i=e.length,o=0;for(t=0;t<i;++t)o+=e[t].length;var a=new Array(o),s=0;for(t=0;t<i;++t){var l=e[t],u=l.length;for(n=0;n<u;++n){var c=a[s++]=new Array(u-1),d=0;for(r=0;r<u;++r)r!==n&&(c[d++]=l[r]);if(1&n){var h=c[1];c[1]=c[0],c[0]=h}}}return a}},function(e,t,n){"use strict";var r=n(1073),i=n(1974),o=n(1074);e.exports=function(e){e.sort(i);for(var t=e.length,n=0,a=0;a<t;++a){var s=e[a],l=o(s);if(0!==l){if(n>0){var u=e[n-1];if(0===r(s,u)&&o(u)!==l){n-=1;continue}}e[n++]=s}}return e.length=n,e}},function(e,t,n){"use strict";var r=n(1073),i=n(1074);e.exports=function(e,t){return r(e,t)||i(e)-i(t)}},function(e,t,n){"use strict";e.exports=function(e,t,n,r){r=r||e.length;for(var i=new Array(r),o=0;o<r;o++)i[o]=[e[o],t[o],n[o]];return i}},function(e,t,n){"use strict";e.exports=n(1977)},function(e,t,n){"use strict";e.exports={attributes:n(398),layoutAttributes:n(1076),supplyDefaults:n(1077).supplyDefaults,crossTraceDefaults:n(1077).crossTraceDefaults,supplyLayoutDefaults:n(1979),calc:n(1980),crossTraceCalc:n(1981).crossTraceCalc,colorbar:n(1016),arraysToCalcdata:n(1078),plot:n(1983).plot,style:n(577).style,styleOnSelect:n(577).styleOnSelect,hoverPoints:n(1984).hoverPoints,eventData:n(1985),selectPoints:n(1986),moduleType:"trace",name:"bar",basePlotModule:n(301),categories:["bar-like","cartesian","svg","bar","oriented","errorBarsOK","showLegend","zoomScale"],animatable:!0,meta:{}}},function(e,t,n){"use strict";var r=n(41),i=n(149).hasColorscale,o=n(213);e.exports=function(e,t,n,a,s){n("marker.color",a),i(e,"marker")&&o(e,t,s,n,{prefix:"marker.",cLetter:"c"}),n("marker.line.color",r.defaultLine),i(e,"marker.line")&&o(e,t,s,n,{prefix:"marker.line.",cLetter:"c"}),n("marker.line.width"),n("marker.opacity"),n("selected.marker.color"),n("unselected.marker.color")}},function(e,t,n){"use strict";var r=n(27),i=n(55),o=n(10),a=n(1076);e.exports=function(e,t,n){function s(n,r){return o.coerce(e,t,a,n,r)}for(var l=!1,u=!1,c=!1,d={},h=s("barmode"),f=0;f<n.length;f++){var p=n[f];if(r.traceIs(p,"bar")&&p.visible){if(l=!0,"group"===h){var g=p.xaxis+p.yaxis;d[g]&&(c=!0),d[g]=!0}if(p.visible&&"histogram"===p.type)"category"!==i.getFromId({_fullLayout:t},p["v"===p.orientation?"xaxis":"yaxis"]).type&&(u=!0)}}l?("overlay"!==h&&s("barnorm"),s("bargap",u&&!c?0:.2),s("bargroupgap")):delete t.barmode}},function(e,t,n){"use strict";var r=n(55),i=n(557),o=n(149).hasColorscale,a=n(298),s=n(1078),l=n(1015);e.exports=function(e,t){var n,u,c,d,h=r.getFromId(e,t.xaxis||"x"),f=r.getFromId(e,t.yaxis||"y"),p={msUTC:!(!t.base&&0!==t.base)};"h"===t.orientation?(n=h.makeCalcdata(t,"x",p),c=f.makeCalcdata(t,"y"),u=i(t,f,"y",c),d=!!t.yperiodalignment):(n=f.makeCalcdata(t,"y",p),c=h.makeCalcdata(t,"x"),u=i(t,h,"x",c),d=!!t.xperiodalignment);for(var g=Math.min(u.length,n.length),v=new Array(g),m=0;m<g;m++)v[m]={p:u[m],s:n[m]},d&&(v[m].orig_p=c[m]),t.ids&&(v[m].id=String(t.ids[m]));return o(t,"marker")&&a(e,t,{vals:t.marker.color,containerStr:"marker",cLetter:"c"}),o(t,"marker.line")&&a(e,t,{vals:t.marker.line.color,containerStr:"marker.line",cLetter:"c"}),s(v,t),l(v,t),v}},function(e,t,n){"use strict";var r=n(42),i=n(10).isArrayOrTypedArray,o=n(72).BADNUM,a=n(27),s=n(55),l=n(390).getAxisGroup,u=n(1982);function c(e,t,n,a,c){if(a.length){var C,A,_,I;switch(function(e,t){var n,o;for(n=0;n<t.length;n++){var a,s=t[n],l=s[0].trace,u="funnel"===l.type?l._base:l.base,c="h"===l.orientation?l.xcalendar:l.ycalendar,d="category"===e.type||"multicategory"===e.type?function(){return null}:e.d2c;if(i(u)){for(o=0;o<Math.min(u.length,s.length);o++)a=d(u[o],0,c),r(a)?(s[o].b=+a,s[o].hasB=1):s[o].b=0;for(;o<s.length;o++)s[o].b=0}else{a=d(u,0,c);var h=r(a);for(a=h?a:0,o=0;o<s.length;o++)s[o].b=a,h&&(s[o].hasB=1)}}}(n,a),c.mode){case"overlay":d(t,n,a,c);break;case"group":for(C=[],A=[],_=0;_<a.length;_++)void 0===(I=a[_])[0].trace.offset?A.push(I):C.push(I);A.length&&function(e,t,n,r,i){var a=new u(r,{sepNegVal:!1,overlapNoMerge:!i.norm});(function(e,t,n,r){for(var i=e._fullLayout,o=n.positions,a=n.distinctPositions,s=n.minDiff,u=n.traces,c=u.length,d=o.length!==a.length,h=s*(1-r.gap),v=l(i,t._id)+u[0][0].trace.orientation,m=i._alignmentOpts[v]||{},y=0;y<c;y++){var b,C,A=u[y],_=A[0].trace,I=m[_.alignmentgroup]||{},x=Object.keys(I.offsetGroups||{}).length,w=(b=x?h/x:d?h/c:h)*(1-(r.groupgap||0));C=x?((2*_._offsetIndex+1-x)*b-w)/2:d?((2*y+1-c)*b-w)/2:-w/2;var M=A[0].t;M.barwidth=w,M.poffset=C,M.bargroupwidth=h,M.bardelta=s}n.binWidth=u[0][0].t.barwidth/100,f(n),p(t,n),g(t,n,d)})(e,t,a,i),function(e){for(var t=e.traces,n=0;n<t.length;n++){var r=t[n];if(void 0===r[0].trace.base)for(var i=new u([r],{sepNegVal:!0,overlapNoMerge:!0}),a=0;a<r.length;a++){var s=r[a];if(s.p!==o){var l=i.put(s.p,s.b+s.s);l&&(s.b=l)}}}}(a),i.norm?(m(a),y(n,a,i)):v(n,a)}(e,t,n,A,c),C.length&&d(t,n,C,c);break;case"stack":case"relative":for(C=[],A=[],_=0;_<a.length;_++)void 0===(I=a[_])[0].trace.base?A.push(I):C.push(I);A.length&&function(e,t,n,r,i){var a=new u(r,{sepNegVal:"relative"===i.mode,overlapNoMerge:!(i.norm||"stack"===i.mode||"relative"===i.mode)});h(t,a,i),function(e,t,n){var r,i,a,l,u,c,d=b(e),h=t.traces;for(l=0;l<h.length;l++)if(r=h[l],"funnel"===(i=r[0].trace).type)for(u=0;u<r.length;u++)(c=r[u]).s!==o&&t.put(c.p,-.5*c.s);for(l=0;l<h.length;l++){r=h[l],i=r[0].trace,a="funnel"===i.type;var f=[];for(u=0;u<r.length;u++)if((c=r[u]).s!==o){var p;p=a?c.s:c.s+c.b;var g=t.put(c.p,p),v=g+p;c.b=g,c[d]=v,n.norm||(f.push(v),c.hasB&&f.push(g))}n.norm||(i._extremes[e._id]=s.findExtremes(e,f,{tozero:!0,padded:!0}))}}(n,a,i);for(var l=0;l<r.length;l++)for(var c=r[l],d=0;d<c.length;d++){var f=c[d];if(f.s!==o)f.b+f.s===a.get(f.p,f.s)&&(f._outmost=!0)}i.norm&&y(n,a,i)}(0,t,n,A,c),C.length&&d(t,n,C,c)}!function(e,t){var n,i,o,a=b(t),s={},l=1/0,u=-1/0;for(n=0;n<e.length;n++)for(o=e[n],i=0;i<o.length;i++){var c=o[i].p;r(c)&&(l=Math.min(l,c),u=Math.max(u,c))}var d=1e4/(u-l),h=s.round=function(e){return String(Math.round(d*(e-l)))};for(n=0;n<e.length;n++){(o=e[n])[0].t.extents=s;var f=o[0].t.poffset,p=Array.isArray(f);for(i=0;i<o.length;i++){var g=o[i],v=g[a]-g.w/2;if(r(v)){var m=g[a]+g.w/2,y=h(g.p);s[y]?s[y]=[Math.min(v,s[y][0]),Math.max(m,s[y][1])]:s[y]=[v,m]}g.p0=g.p+(p?f[i]:f),g.p1=g.p0+g.w,g.s0=g.b,g.s1=g.s0+g.s}}}(a,t)}}function d(e,t,n,r){for(var i=0;i<n.length;i++){var o=n[i],a=new u([o],{unitMinDiff:r.xCat||r.yCat,sepNegVal:!1,overlapNoMerge:!r.norm});h(e,a,r),r.norm?(m(a),y(t,a,r)):v(t,a)}}function h(e,t,n){for(var r=t.minDiff,i=t.traces,o=r*(1-n.gap),a=o*(1-(n.groupgap||0)),s=-a/2,l=0;l<i.length;l++){var u=i[l][0].t;u.barwidth=a,u.poffset=s,u.bargroupwidth=o,u.bardelta=r}t.binWidth=i[0][0].t.barwidth/100,f(t),p(e,t),g(e,t)}function f(e){var t,n,o=e.traces;for(t=0;t<o.length;t++){var a,s=o[t],l=s[0],u=l.trace,c=l.t,d=u._offset||u.offset,h=c.poffset;if(i(d)){for(a=Array.prototype.slice.call(d,0,s.length),n=0;n<a.length;n++)r(a[n])||(a[n]=h);for(n=a.length;n<s.length;n++)a.push(h);c.poffset=a}else void 0!==d&&(c.poffset=d);var f=u._width||u.width,p=c.barwidth;if(i(f)){var g=Array.prototype.slice.call(f,0,s.length);for(n=0;n<g.length;n++)r(g[n])||(g[n]=p);for(n=g.length;n<s.length;n++)g.push(p);if(c.barwidth=g,void 0===d){for(a=[],n=0;n<s.length;n++)a.push(h+(p-g[n])/2);c.poffset=a}}else void 0!==f&&(c.barwidth=f,void 0===d&&(c.poffset=h+(p-f)/2))}}function p(e,t){for(var n=t.traces,r=b(e),i=0;i<n.length;i++)for(var o=n[i],a=o[0].t,s=a.poffset,l=Array.isArray(s),u=a.barwidth,c=Array.isArray(u),d=0;d<o.length;d++){var h=o[d],f=h.w=c?u[d]:u;h[r]=h.p+(l?s[d]:s)+f/2}}function g(e,t,n){var r=t.traces,i=t.minDiff/2;s.minDtick(e,t.minDiff,t.distinctPositions[0],n);for(var o=0;o<r.length;o++){var a,l,u,c,d=r[o],h=d[0],f=h.trace,p=[];for(c=0;c<d.length;c++)l=(a=d[c]).p-i,u=a.p+i,p.push(l,u);if(f.width||f.offset){var g=h.t,v=g.poffset,m=g.barwidth,y=Array.isArray(v),b=Array.isArray(m);for(c=0;c<d.length;c++){a=d[c];var C=y?v[c]:v,A=b?m[c]:m;u=(l=a.p+C)+A,p.push(l,u)}}f._extremes[e._id]=s.findExtremes(e,p,{padded:!1})}}function v(e,t){for(var n=t.traces,r=b(e),i=0;i<n.length;i++){for(var o=n[i],a=o[0].trace,l=[],u=!1,c=0;c<o.length;c++){var d=o[c],h=d.b,f=h+d.s;d[r]=f,l.push(f),d.hasB&&l.push(h),d.hasB&&d.b||(u=!0)}a._extremes[e._id]=s.findExtremes(e,l,{tozero:u,padded:!0})}}function m(e){for(var t=e.traces,n=0;n<t.length;n++)for(var r=t[n],i=0;i<r.length;i++){var a=r[i];a.s!==o&&e.put(a.p,a.b+a.s)}}function y(e,t,n){var i=t.traces,a=b(e),l="fraction"===n.norm?1:100,u=l/1e9,c=e.l2c(e.c2l(0)),d="stack"===n.mode?l:c;function h(t){return r(e.c2l(t))&&(t<c-u||t>d+u||!r(c))}for(var f=0;f<i.length;f++){for(var p=i[f],g=p[0].trace,v=[],m=!1,y=!1,C=0;C<p.length;C++){var A=p[C];if(A.s!==o){var _=Math.abs(l/t.get(A.p,A.s));A.b*=_,A.s*=_;var I=A.b,x=I+A.s;A[a]=x,v.push(x),y=y||h(x),A.hasB&&(v.push(I),y=y||h(I)),A.hasB&&A.b||(m=!0)}}g._extremes[e._id]=s.findExtremes(e,v,{tozero:m,padded:y})}}function b(e){return e._id.charAt(0)}e.exports={crossTraceCalc:function(e,t){for(var n=t.xaxis,r=t.yaxis,i=e._fullLayout,o=e._fullData,s=e.calcdata,l=[],u=[],d=0;d<o.length;d++){var h=o[d];if(!0===h.visible&&a.traceIs(h,"bar")&&h.xaxis===n._id&&h.yaxis===r._id&&("h"===h.orientation?l.push(s[d]):u.push(s[d]),h._computePh))for(var f=e.calcdata[d],p=0;p<f.length;p++)"function"==typeof f[p].ph0&&(f[p].ph0=f[p].ph0()),"function"==typeof f[p].ph1&&(f[p].ph1=f[p].ph1())}var g={xCat:"category"===n.type||"multicategory"===n.type,yCat:"category"===r.type||"multicategory"===r.type,mode:i.barmode,norm:i.barnorm,gap:i.bargap,groupgap:i.bargroupgap};c(e,n,r,u,g),c(e,r,n,l,g)},setGroupPositions:c}},function(e,t,n){"use strict";e.exports=o;var r=n(10).distinctVals,i=n(72).BADNUM;function o(e,t){this.traces=e,this.sepNegVal=t.sepNegVal,this.overlapNoMerge=t.overlapNoMerge;for(var n=1/0,o=[],a=0;a<e.length;a++){for(var s=e[a],l=0;l<s.length;l++){var u=s[l];u.p!==i&&o.push(u.p)}s[0]&&s[0].width1&&(n=Math.min(s[0].width1,n))}this.positions=o;var c=r(o,{unitMinDiff:t.unitMinDiff});this.distinctPositions=c.vals,1===c.vals.length&&n!==1/0?this.minDiff=n:this.minDiff=Math.min(c.minDiff,n),this.binWidth=this.minDiff,this.bins={}}o.prototype.put=function(e,t){var n=this.getLabel(e,t),r=this.bins[n]||0;return this.bins[n]=r+t,r},o.prototype.get=function(e,t){var n=this.getLabel(e,t);return this.bins[n]||0},o.prototype.getLabel=function(e,t){return(t<0&&this.sepNegVal?"v":"^")+(this.overlapNoMerge?e:Math.round(e/this.binWidth))}},function(e,t,n){"use strict";var r=n(39),i=n(42),o=n(10),a=n(96),s=n(41),l=n(56),u=n(27),c=n(55).tickText,d=n(1079),h=d.recordMinTextSize,f=d.clearMinTextSize,p=n(577),g=n(578),v=n(1075),m=n(398),y=m.text,b=m.textposition,C=n(162).appendArrayPointValue,A=v.TEXTPAD;function _(e){return e.id}function I(e){if(e.ids)return _}function x(e,t){return e<t?1:-1}function w(e,t,n,r){var i;return!t.uniformtext.mode&&M(n)?(r&&(i=r()),e.transition().duration(n.duration).ease(n.easing).each("end",(function(){i&&i()})).each("interrupt",(function(){i&&i()}))):e}function M(e){return e&&e.duration>0}function S(e){return"auto"===e?0:e}function T(e,t){var n=Math.PI/180*t,r=Math.abs(Math.sin(n)),i=Math.abs(Math.cos(n));return{x:e.width*i+e.height*r,y:e.width*r+e.height*i}}function E(e,t,n,r,i,o){var a=!!o.isHorizontal,s=!!o.constrained,l=o.angle||0,u=o.anchor||"end",c="end"===u,d="start"===u,h=((o.leftToRight||0)+1)/2,f=1-h,p=i.width,g=i.height,v=Math.abs(t-e),m=Math.abs(r-n),y=v>2*A&&m>2*A?A:0;v-=2*y,m-=2*y;var b=S(l);"auto"!==l||p<=v&&g<=m||!(p>v||g>m)||(p>m||g>v)&&p<g==v<m||(b+=90);var C=T(i,b),_=1;s&&(_=Math.min(1,v/C.x,m/C.y));var I=i.left*f+i.right*h,w=(i.top+i.bottom)/2,M=(e+A)*f+(t-A)*h,E=(n+r)/2,O=0,k=0;if(d||c){var L=(a?C.x:C.y)/2,D=a?x(e,t):x(n,r);a?d?(M=e+D*y,O=-D*L):(M=t-D*y,O=D*L):d?(E=n+D*y,k=-D*L):(E=r-D*y,k=D*L)}return{textX:I,textY:w,targetX:M,targetY:E,anchorX:O,anchorY:k,scale:_,rotate:b}}e.exports={plot:function(e,t,n,d,v,m){var _=t.xaxis,O=t.yaxis,k=e._fullLayout;v||(v={mode:k.barmode,norm:k.barmode,gap:k.bargap,groupgap:k.bargroupgap},f("bar",k));var L=o.makeTraceGroups(d,n,"trace bars").each((function(n){var u=r.select(this),d=n[0].trace,f="waterfall"===d.type,L="funnel"===d.type,D="bar"===d.type||L,P=0;f&&d.connector.visible&&"between"===d.connector.mode&&(P=d.connector.line.width/2);var N="h"===d.orientation,j=M(v),R=o.ensureSingle(u,"g","points"),z=I(d),F=R.selectAll("g.point").data(o.identity,z);F.enter().append("g").classed("point",!0),F.exit().remove(),F.each((function(u,f){var I,M,L=r.select(this),R=function(e,t,n,r){var i=[],o=[],a=r?t:n,s=r?n:t;return i[0]=a.c2p(e.s0,!0),o[0]=s.c2p(e.p0,!0),i[1]=a.c2p(e.s1,!0),o[1]=s.c2p(e.p1,!0),r?[i,o]:[o,i]}(u,_,O,N),z=R[0][0],F=R[0][1],B=R[1][0],H=R[1][1],U=0==(N?F-z:H-B);if(U&&D&&g.getLineWidth(d,u)&&(U=!1),U||(U=!(i(z)&&i(F)&&i(B)&&i(H))),u.isBlank=U,U&&(N?F=z:H=B),P&&!U&&(N?(z-=x(z,F)*P,F+=x(z,F)*P):(B-=x(B,H)*P,H+=x(B,H)*P)),"waterfall"===d.type){if(!U){var V=d[u.dir].marker;I=V.line.width,M=V.color}}else I=g.getLineWidth(d,u),M=u.mc||d.marker.color;function Y(e){var t=r.round(I/2%1,2);return 0===v.gap&&0===v.groupgap?r.round(Math.round(e)-t,2):e}if(!e._context.staticPlot){var W=s.opacity(M)<1||I>.01?Y:function(e,t,n){return n&&e===t?e:Math.abs(e-t)>=2?Y(e):e>t?Math.ceil(e):Math.floor(e)};z=W(z,F,N),F=W(F,z,N),B=W(B,H,!N),H=W(H,B,!N)}var G=w(o.ensureSingle(L,"path"),k,v,m);if(G.style("vector-effect","non-scaling-stroke").attr("d",isNaN((F-z)*(H-B))||U&&e._context.staticPlot?"M0,0Z":"M"+z+","+B+"V"+H+"H"+F+"V"+B+"Z").call(l.setClipUrl,t.layerClipId,e),!k.uniformtext.mode&&j){var X=l.makePointStyleFns(d);l.singlePointStyle(u,G,d,X,e)}!function(e,t,n,r,i,s,u,d,f,v,m){var _,I=t.xaxis,M=t.yaxis,O=e._fullLayout;function k(t,n,r){return o.ensureSingle(t,"text").text(n).attr({class:"bartext bartext-"+_,"text-anchor":"middle","data-notex":1}).call(l.font,r).call(a.convertToTspans,e)}var L=r[0].trace,D="h"===L.orientation,P=function(e,t,n,r,i){var a,s=t[0].trace;a=s.texttemplate?function(e,t,n,r,i){var a=t[0].trace,s=o.castOption(a,n,"texttemplate");if(!s)return"";var l,u,d,h,f="waterfall"===a.type,p="funnel"===a.type;"h"===a.orientation?(l="y",u=i,d="x",h=r):(l="x",u=r,d="y",h=i);function g(e){return c(h,+e,!0).text}var v=t[n],m={};m.label=v.p,m.labelLabel=m[l+"Label"]=(y=v.p,c(u,y,!0).text);var y;var b=o.castOption(a,v.i,"text");(0===b||b)&&(m.text=b);m.value=v.s,m.valueLabel=m[d+"Label"]=g(v.s);var A={};C(A,a,v.i),f&&(m.delta=+v.rawS||v.s,m.deltaLabel=g(m.delta),m.final=v.v,m.finalLabel=g(m.final),m.initial=m.final-m.delta,m.initialLabel=g(m.initial));p&&(m.value=v.s,m.valueLabel=g(m.value),m.percentInitial=v.begR,m.percentInitialLabel=o.formatPercent(v.begR),m.percentPrevious=v.difR,m.percentPreviousLabel=o.formatPercent(v.difR),m.percentTotal=v.sumR,m.percenTotalLabel=o.formatPercent(v.sumR));var _=o.castOption(a,v.i,"customdata");_&&(m.customdata=_);return o.texttemplateString(s,m,e._d3locale,A,m,a._meta||{})}(e,t,n,r,i):s.textinfo?function(e,t,n,r){var i=e[0].trace,a="h"===i.orientation,s="waterfall"===i.type,l="funnel"===i.type;function u(e){return c(a?n:r,+e,!0).text}var d,h=i.textinfo,f=e[t],p=h.split("+"),g=[],v=function(e){return-1!==p.indexOf(e)};v("label")&&g.push((m=e[t].p,c(a?r:n,m,!0).text));var m;v("text")&&(0===(d=o.castOption(i,f.i,"text"))||d)&&g.push(d);if(s){var y=+f.rawS||f.s,b=f.v,C=b-y;v("initial")&&g.push(u(C)),v("delta")&&g.push(u(y)),v("final")&&g.push(u(b))}if(l){v("value")&&g.push(u(f.s));var A=0;v("percent initial")&&A++,v("percent previous")&&A++,v("percent total")&&A++;var _=A>1;v("percent initial")&&(d=o.formatPercent(f.begR),_&&(d+=" of initial"),g.push(d)),v("percent previous")&&(d=o.formatPercent(f.difR),_&&(d+=" of previous"),g.push(d)),v("percent total")&&(d=o.formatPercent(f.sumR),_&&(d+=" of total"),g.push(d))}return g.join("<br>")}(t,n,r,i):g.getValue(s.text,n);return g.coerceString(y,a)}(O,r,i,I,M);_=function(e,t){var n=g.getValue(e.textposition,t);return g.coerceEnumerated(b,n)}(L,i);var N="stack"===v.mode||"relative"===v.mode,j=r[i],R=!N||j._outmost;if(!P||"none"===_||(j.isBlank||s===u||d===f)&&("auto"===_||"inside"===_))return void n.select("text").remove();var z=O.font,F=p.getBarColor(r[i],L),B=p.getInsideTextFont(L,i,z,F),H=p.getOutsideTextFont(L,i,z),U=n.datum();D?"log"===I.type&&U.s0<=0&&(s=I.range[0]<I.range[1]?0:I._length):"log"===M.type&&U.s0<=0&&(d=M.range[0]<M.range[1]?M._length:0);var V,Y,W,G,X,q=Math.abs(u-s)-2*A,Z=Math.abs(f-d)-2*A;"outside"===_&&(R||j.hasB||(_="inside"));if("auto"===_)if(R){_="inside",X=o.ensureUniformFontSize(e,B),V=k(n,P,X),Y=l.bBox(V.node()),W=Y.width,G=Y.height;var Q=W<=q&&G<=Z,K=W<=Z&&G<=q,J=D?q>=W*(Z/G):Z>=G*(q/W);W>0&&G>0&&(Q||K||J)?_="inside":(_="outside",V.remove(),V=null)}else _="inside";if(!V){X=o.ensureUniformFontSize(e,"outside"===_?H:B);var $=(V=k(n,P,X)).attr("transform");if(V.attr("transform",""),Y=l.bBox(V.node()),W=Y.width,G=Y.height,V.attr("transform",$),W<=0||G<=0)return void V.remove()}var ee,te,ne=L.textangle;"outside"===_?(te="both"===L.constraintext||"outside"===L.constraintext,ee=function(e,t,n,r,i,o){var a,s=!!o.isHorizontal,l=!!o.constrained,u=o.angle||0,c=i.width,d=i.height,h=Math.abs(t-e),f=Math.abs(r-n);a=s?f>2*A?A:0:h>2*A?A:0;var p=1;l&&(p=s?Math.min(1,f/d):Math.min(1,h/c));var g=S(u),v=T(i,g),m=(s?v.x:v.y)/2,y=(i.left+i.right)/2,b=(i.top+i.bottom)/2,C=(e+t)/2,_=(n+r)/2,I=0,w=0,M=s?x(t,e):x(n,r);s?(C=t-M*a,I=M*m):(_=r+M*a,w=-M*m);return{textX:y,textY:b,targetX:C,targetY:_,anchorX:I,anchorY:w,scale:p,rotate:g}}(s,u,d,f,Y,{isHorizontal:D,constrained:te,angle:ne})):(te="both"===L.constraintext||"inside"===L.constraintext,ee=E(s,u,d,f,Y,{isHorizontal:D,constrained:te,angle:ne,anchor:L.insidetextanchor}));ee.fontSize=X.size,h(L.type,ee,O),j.transform=ee,w(V,O,v,m).attr("transform",o.getTextTransform(ee))}(e,t,L,n,f,z,F,B,H,v,m),t.layerClipId&&l.hideOutsideRangePoint(u,L.select("text"),_,O,d.xcalendar,d.ycalendar)}));var B=!1===d.cliponaxis;l.setClipUrl(u,B?null:t.layerClipId,e)}));u.getComponentMethod("errorbars","plot")(e,L,t,v)},toMoveInsideBar:E}},function(e,t,n){"use strict";var r=n(179),i=n(27),o=n(41),a=n(10).fillText,s=n(578).getLineWidth,l=n(55).hoverLabelText,u=n(72).BADNUM;function c(e,t,n,i){var o,s,c,d,h,f,p,g=e.cd,v=g[0].trace,m=g[0].t,y="closest"===i,b="waterfall"===v.type,C=e.maxHoverDistance;function A(e){return e[c]-e.w/2}function _(e){return e[c]+e.w/2}var I=y?A:function(e){return Math.min(A(e),e.p-m.bardelta/2)},x=y?_:function(e){return Math.max(_(e),e.p+m.bardelta/2)};function w(e,t){return r.inbox(e-o,t-o,C+Math.min(1,Math.abs(t-e)/p)-1)}function M(e){return w(I(e),x(e))}function S(e){var t=s,n=e.b,i=e[d];if(b){var o=Math.abs(e.rawS)||0;t>0?i+=o:t<0&&(i-=o)}return r.inbox(n-t,i-t,C+(i-t)/(i-n)-1)}"h"===v.orientation?(o=n,s=t,c="y",d="x",h=S,f=M):(o=t,s=n,c="x",d="y",f=S,h=M);var T=e[c+"a"],E=e[d+"a"];p=Math.abs(T.r2c(T.range[1])-T.r2c(T.range[0]));var O=r.getDistanceFunction(i,h,f,(function(e){return(h(e)+f(e))/2}));if(r.getClosest(g,O,e),!1!==e.index&&g[e.index].p!==u){y||(I=function(e){return Math.min(A(e),e.p-m.bargroupwidth/2)},x=function(e){return Math.max(_(e),e.p+m.bargroupwidth/2)});var k=g[e.index],L=v.base?k.b+k.s:k.s;e[d+"0"]=e[d+"1"]=E.c2p(k[d],!0),e[d+"LabelVal"]=L;var D=m.extents[m.extents.round(k.p)];e[c+"0"]=T.c2p(y?I(k):D[0],!0),e[c+"1"]=T.c2p(y?x(k):D[1],!0);var P=void 0!==k.orig_p;return e[c+"LabelVal"]=P?k.orig_p:k.p,e.labelLabel=l(T,e[c+"LabelVal"]),e.valueLabel=l(E,e[d+"LabelVal"]),e.baseLabel=l(E,k.b),e.spikeDistance=(S(k)+function(e){return w(A(e),_(e))}(k))/2-C,e[c+"Spike"]=T.c2p(k.p,!0),a(k,v,e),e.hovertemplate=v.hovertemplate,e}}function d(e,t){var n=t.mcc||e.marker.color,r=t.mlcc||e.marker.line.color,i=s(e,t);return o.opacity(n)?n:o.opacity(r)&&i?r:void 0}e.exports={hoverPoints:function(e,t,n,r){var o=c(e,t,n,r);if(o){var a=o.cd,s=a[0].trace,l=a[o.index];return o.color=d(s,l),i.getComponentMethod("errorbars","hoverInfo")(l,s,o),[o]}},hoverOnBars:c,getTraceColor:d}},function(e,t,n){"use strict";e.exports=function(e,t,n){return e.x="xVal"in t?t.xVal:t.x,e.y="yVal"in t?t.yVal:t.y,t.xa&&(e.xaxis=t.xa),t.ya&&(e.yaxis=t.ya),"h"===n.orientation?(e.label=e.y,e.value=e.x):(e.label=e.x,e.value=e.y),e}},function(e,t,n){"use strict";function r(e,t,n,r,i){var o=t.c2p(r?e.s0:e.p0,!0),a=t.c2p(r?e.s1:e.p1,!0),s=n.c2p(r?e.p0:e.s0,!0),l=n.c2p(r?e.p1:e.s1,!0);return i?[(o+a)/2,(s+l)/2]:r?[a,(s+l)/2]:[(o+a)/2,l]}e.exports=function(e,t){var n,i=e.cd,o=e.xaxis,a=e.yaxis,s=i[0].trace,l="funnel"===s.type,u="h"===s.orientation,c=[];if(!1===t)for(n=0;n<i.length;n++)i[n].selected=0;else for(n=0;n<i.length;n++){var d=i[n],h="ct"in d?d.ct:r(d,o,a,u,l);t.contains(h,!1,n,e)?(c.push({pointNumber:n,x:o.c2d(d.x),y:a.c2d(d.y)}),d.selected=1):d.selected=0}return c}},function(e,t,n){"use strict";e.exports=n(1988)},function(e,t,n){"use strict";e.exports={attributes:n(579),supplyDefaults:n(1990),plot:n(1994),calc:n(1996),animatable:!0,isContainer:!0,moduleType:"trace",name:"carpet",basePlotModule:n(301),categories:["cartesian","svg","carpet","carpetAxis","notLegendIsolatable","noMultiCategory","noHover","noSortingByValue"],meta:{}}},function(e,t,n){"use strict";var r=n(94),i=n(133),o=n(134),a=n(124).overrideAll;n(194).FORMAT_LINK,n(194).TIME_FORMAT_LINK;e.exports={color:{valType:"color",editType:"calc"},smoothing:{valType:"number",dflt:1,min:0,max:1.3,editType:"calc"},title:{text:{valType:"string",dflt:"",editType:"calc"},font:r({editType:"calc"}),offset:{valType:"number",dflt:10,editType:"calc"},editType:"calc"},type:{valType:"enumerated",values:["-","linear","date","category"],dflt:"-",editType:"calc"},autotypenumbers:o.autotypenumbers,autorange:{valType:"enumerated",values:[!0,!1,"reversed"],dflt:!0,editType:"calc"},rangemode:{valType:"enumerated",values:["normal","tozero","nonnegative"],dflt:"normal",editType:"calc"},range:{valType:"info_array",editType:"calc",items:[{valType:"any",editType:"calc"},{valType:"any",editType:"calc"}]},fixedrange:{valType:"boolean",dflt:!1,editType:"calc"},cheatertype:{valType:"enumerated",values:["index","value"],dflt:"value",editType:"calc"},tickmode:{valType:"enumerated",values:["linear","array"],dflt:"array",editType:"calc"},nticks:{valType:"integer",min:0,dflt:0,editType:"calc"},tickvals:{valType:"data_array",editType:"calc"},ticktext:{valType:"data_array",editType:"calc"},showticklabels:{valType:"enumerated",values:["start","end","both","none"],dflt:"start",editType:"calc"},tickfont:r({editType:"calc"}),tickangle:{valType:"angle",dflt:"auto",editType:"calc"},tickprefix:{valType:"string",dflt:"",editType:"calc"},showtickprefix:{valType:"enumerated",values:["all","first","last","none"],dflt:"all",editType:"calc"},ticksuffix:{valType:"string",dflt:"",editType:"calc"},showticksuffix:{valType:"enumerated",values:["all","first","last","none"],dflt:"all",editType:"calc"},showexponent:{valType:"enumerated",values:["all","first","last","none"],dflt:"all",editType:"calc"},exponentformat:{valType:"enumerated",values:["none","e","E","power","SI","B"],dflt:"B",editType:"calc"},minexponent:{valType:"number",dflt:3,min:0,editType:"calc"},separatethousands:{valType:"boolean",dflt:!1,editType:"calc"},tickformat:{valType:"string",dflt:"",editType:"calc"},tickformatstops:a(o.tickformatstops,"calc","from-root"),categoryorder:{valType:"enumerated",values:["trace","category ascending","category descending","array"],dflt:"trace",editType:"calc"},categoryarray:{valType:"data_array",editType:"calc"},labelpadding:{valType:"integer",dflt:10,editType:"calc"},labelprefix:{valType:"string",editType:"calc"},labelsuffix:{valType:"string",dflt:"",editType:"calc"},showline:{valType:"boolean",dflt:!1,editType:"calc"},linecolor:{valType:"color",dflt:i.defaultLine,editType:"calc"},linewidth:{valType:"number",min:0,dflt:1,editType:"calc"},gridcolor:{valType:"color",editType:"calc"},gridwidth:{valType:"number",min:0,dflt:1,editType:"calc"},showgrid:{valType:"boolean",dflt:!0,editType:"calc"},minorgridcount:{valType:"integer",min:0,dflt:0,editType:"calc"},minorgridwidth:{valType:"number",min:0,dflt:1,editType:"calc"},minorgridcolor:{valType:"color",dflt:i.lightLine,editType:"calc"},startline:{valType:"boolean",editType:"calc"},startlinecolor:{valType:"color",editType:"calc"},startlinewidth:{valType:"number",dflt:1,editType:"calc"},endline:{valType:"boolean",editType:"calc"},endlinewidth:{valType:"number",dflt:1,editType:"calc"},endlinecolor:{valType:"color",editType:"calc"},tick0:{valType:"number",min:0,dflt:0,editType:"calc"},dtick:{valType:"number",min:0,dflt:1,editType:"calc"},arraytick0:{valType:"integer",min:0,dflt:0,editType:"calc"},arraydtick:{valType:"integer",min:1,dflt:1,editType:"calc"},_deprecated:{title:{valType:"string",editType:"calc"},titlefont:r({editType:"calc"}),titleoffset:{valType:"number",dflt:10,editType:"calc"}},editType:"calc"}},function(e,t,n){"use strict";var r=n(10),i=n(1991),o=n(1992),a=n(579),s=n(133);e.exports=function(e,t,n,l){function u(n,i){return r.coerce(e,t,a,n,i)}t._clipPathId="clip"+t.uid+"carpet";var c=u("color",s.defaultLine);(r.coerceFont(u,"font"),u("carpet"),o(e,t,l,u,c),t.a&&t.b)?(t.a.length<3&&(t.aaxis.smoothing=0),t.b.length<3&&(t.baxis.smoothing=0),i(e,t,u)||(t.visible=!1),t._cheater&&u("cheaterslope")):t.visible=!1}},function(e,t,n){"use strict";var r=n(10).isArray1D;e.exports=function(e,t,n){var i=n("x"),o=i&&i.length,a=n("y"),s=a&&a.length;if(!o&&!s)return!1;if(t._cheater=!i,o&&!r(i)||s&&!r(a))t._length=null;else{var l=o?i.length:1/0;s&&(l=Math.min(l,a.length)),t.a&&t.a.length&&(l=Math.min(l,t.a.length)),t.b&&t.b.length&&(l=Math.min(l,t.b.length)),t._length=l}return!0}},function(e,t,n){"use strict";var r=n(1993),i=n(64);e.exports=function(e,t,n,o,a){o("a")||(o("da"),o("a0")),o("b")||(o("db"),o("b0")),function(e,t,n,o){["aaxis","baxis"].forEach((function(a){var s=a.charAt(0),l=e[a]||{},u=i.newContainer(t,a),c={tickfont:"x",id:s+"axis",letter:s,font:t.font,name:a,data:e[s],calendar:t.calendar,dfltColor:o,bgColor:n.paper_bgcolor,autotypenumbersDflt:n.autotypenumbers,fullLayout:n};r(l,u,c),u._categories=u._categories||[],e[a]||"-"===l.type||(e[a]={type:l.type})}))}(e,t,n,a)}},function(e,t,n){"use strict";var r=n(579),i=n(41).addOpacity,o=n(27),a=n(10),s=n(542),l=n(543),u=n(1019),c=n(300),d=n(545);e.exports=function(e,t,n){var h=n.letter,f=n.font||{},p=r[h+"axis"];function g(n,r){return a.coerce(e,t,p,n,r)}function v(n,r){return a.coerce2(e,t,p,n,r)}n.name&&(t._name=n.name,t._id=n.name),g("autotypenumbers",n.autotypenumbersDflt);var m=g("type");("-"===m&&(n.data&&function(e,t){if("-"!==e.type)return;var n=e._id.charAt(0),r=e[n+"calendar"];e.type=d(t,r,{autotypenumbers:e.autotypenumbers})}(t,n.data),"-"===t.type?t.type="linear":m=e.type=t.type),g("smoothing"),g("cheatertype"),g("showticklabels"),g("labelprefix",h+" = "),g("labelsuffix"),g("showtickprefix"),g("showticksuffix"),g("separatethousands"),g("tickformat"),g("exponentformat"),g("minexponent"),g("showexponent"),g("categoryorder"),g("tickmode"),g("tickvals"),g("ticktext"),g("tick0"),g("dtick"),"array"===t.tickmode&&(g("arraytick0"),g("arraydtick")),g("labelpadding"),t._hovertitle=h,"date"===m)&&o.getComponentMethod("calendars","handleDefaults")(e,t,"calendar",n.calendar);c(t,n.fullLayout),t.c2p=a.identity;var y=g("color",n.dfltColor),b=y===e.color?y:f.color;g("title.text")&&(a.coerceFont(g,"title.font",{family:f.family,size:Math.round(1.2*f.size),color:b}),g("title.offset")),g("tickangle"),g("autorange",!t.isValidRange(e.range))&&g("rangemode"),g("range"),t.cleanRange(),g("fixedrange"),s(e,t,g,m),l(e,t,g,m,n),u(e,t,g,{data:n.data,dataAttr:h});var C=v("gridcolor",i(y,.3)),A=v("gridwidth"),_=g("showgrid");_||(delete t.gridcolor,delete t.gridwidth);var I=v("startlinecolor",y),x=v("startlinewidth",A);g("startline",t.showgrid||!!I||!!x)||(delete t.startlinecolor,delete t.startlinewidth);var w=v("endlinecolor",y),M=v("endlinewidth",A);return g("endline",t.showgrid||!!w||!!M)||(delete t.endlinecolor,delete t.endlinewidth),_?(g("minorgridcount"),g("minorgridwidth",A),g("minorgridcolor",i(C,.06)),t.minorgridcount||(delete t.minorgridwidth,delete t.minorgridcolor)):(delete t.gridcolor,delete t.gridWidth),"none"===t.showticklabels&&(delete t.tickfont,delete t.tickangle,delete t.showexponent,delete t.exponentformat,delete t.minexponent,delete t.tickformat,delete t.showticksuffix,delete t.showtickprefix),t.showticksuffix||delete t.ticksuffix,t.showtickprefix||delete t.tickprefix,g("tickmode"),t}},function(e,t,n){"use strict";var r=n(39),i=n(56),o=n(1080),a=n(1081),s=n(1995),l=n(96),u=n(10),c=u.strRotate,d=u.strTranslate,h=n(97);function f(e,t,n,i,s,l){var u="const-"+s+"-lines",c=n.selectAll("."+u).data(l);c.enter().append("path").classed(u,!0).style("vector-effect","non-scaling-stroke"),c.each((function(n){var i=n,s=i.x,l=i.y,u=o([],s,e.c2p),c=o([],l,t.c2p),d="M"+a(u,c,i.smoothing);r.select(this).attr("d",d).style("stroke-width",i.width).style("stroke",i.color).style("fill","none")})),c.exit().remove()}function p(e,t,n,o,a,u,h,f){var p=u.selectAll("text."+f).data(h);p.enter().append("text").classed(f,!0);var g=0,v={};return p.each((function(a,u){var h;if("auto"===a.axis.tickangle)h=s(o,t,n,a.xy,a.dxy);else{var f=(a.axis.tickangle+180)*Math.PI/180;h=s(o,t,n,a.xy,[Math.cos(f),Math.sin(f)])}u||(v={angle:h.angle,flip:h.flip});var p=(a.endAnchor?-1:1)*h.flip,m=r.select(this).attr({"text-anchor":p>0?"start":"end","data-notex":1}).call(i.font,a.font).text(a.text).call(l.convertToTspans,e),y=i.bBox(this);m.attr("transform",d(h.p[0],h.p[1])+c(h.angle)+d(a.axis.labelpadding*p,.3*y.height)),g=Math.max(g,y.width+a.axis.labelpadding)})),p.exit().remove(),v.maxExtent=g,v}e.exports=function(e,t,n,i){var l=t.xaxis,c=t.yaxis,d=e._fullLayout._clips;u.makeTraceGroups(i,n,"trace").each((function(t){var n=r.select(this),i=t[0],h=i.trace,g=h.aaxis,v=h.baxis,y=u.ensureSingle(n,"g","minorlayer"),b=u.ensureSingle(n,"g","majorlayer"),C=u.ensureSingle(n,"g","boundarylayer"),A=u.ensureSingle(n,"g","labellayer");n.style("opacity",h.opacity),f(l,c,b,g,"a",g._gridlines),f(l,c,b,v,"b",v._gridlines),f(l,c,y,g,"a",g._minorgridlines),f(l,c,y,v,"b",v._minorgridlines),f(l,c,C,g,"a-boundary",g._boundarylines),f(l,c,C,v,"b-boundary",v._boundarylines);var _=p(e,l,c,h,i,A,g._labels,"a-label"),I=p(e,l,c,h,i,A,v._labels,"b-label");!function(e,t,n,r,i,o,a,l){var c,d,h,f,p=u.aggNums(Math.min,null,n.a),g=u.aggNums(Math.max,null,n.a),v=u.aggNums(Math.min,null,n.b),y=u.aggNums(Math.max,null,n.b);c=.5*(p+g),d=v,h=n.ab2xy(c,d,!0),f=n.dxyda_rough(c,d),void 0===a.angle&&u.extendFlat(a,s(n,i,o,h,n.dxydb_rough(c,d)));m(e,t,n,r,h,f,n.aaxis,i,o,a,"a-title"),c=p,d=.5*(v+y),h=n.ab2xy(c,d,!0),f=n.dxydb_rough(c,d),void 0===l.angle&&u.extendFlat(l,s(n,i,o,h,n.dxyda_rough(c,d)));m(e,t,n,r,h,f,n.baxis,i,o,l,"b-title")}(e,A,h,i,l,c,_,I),function(e,t,n,r,i){var s,l,c,d,h=n.select("#"+e._clipPathId);h.size()||(h=n.append("clipPath").classed("carpetclip",!0));var f=u.ensureSingle(h,"path","carpetboundary"),p=t.clipsegments,g=[];for(d=0;d<p.length;d++)s=p[d],l=o([],s.x,r.c2p),c=o([],s.y,i.c2p),g.push(a(l,c,s.bicubic));var v="M"+g.join("L")+"Z";h.attr("id",e._clipPathId),f.attr("d",v)}(h,i,d,l,c)}))};var g=h.LINE_SPACING,v=(1-h.MID_SHIFT)/g+1;function m(e,t,n,o,a,u,h,f,p,m,y){var b=[];h.title.text&&b.push(h.title.text);var C=t.selectAll("text."+y).data(b),A=m.maxExtent;C.enter().append("text").classed(y,!0),C.each((function(){var t=s(n,f,p,a,u);-1===["start","both"].indexOf(h.showticklabels)&&(A=0);var o=h.title.font.size;A+=o+h.title.offset;var y=(m.angle+(m.flip<0?180:0)-t.angle+450)%360,b=y>90&&y<270,C=r.select(this);C.text(h.title.text).call(l.convertToTspans,e),b&&(A=(-l.lineCount(C)+v)*g*o-A),C.attr("transform",d(t.p[0],t.p[1])+c(t.angle)+d(0,A)).attr("text-anchor","middle").call(i.font,h.title.font)})),C.exit().remove()}},function(e,t,n){"use strict";e.exports=function(e,t,n,r,i,o){var a=i[0]*e.dpdx(t),s=i[1]*e.dpdy(n),l=1,u=1;if(o){var c=Math.sqrt(i[0]*i[0]+i[1]*i[1]),d=Math.sqrt(o[0]*o[0]+o[1]*o[1]),h=(i[0]*o[0]+i[1]*o[1])/c/d;u=Math.max(0,h)}var f=180*Math.atan2(s,a)/Math.PI;return f<-90?(f+=180,l=-l):f>90&&(f-=180,l=-l),{angle:f,flip:l,p:e.c2p(r,t,n),offsetMultplier:u}}},function(e,t,n){"use strict";var r=n(55),i=n(10).isArray1D,o=n(1997),a=n(1998),s=n(1999),l=n(2e3),u=n(2001),c=n(1082),d=n(2002),h=n(1083),f=n(2003);e.exports=function(e,t){var n=r.getFromId(e,t.xaxis),p=r.getFromId(e,t.yaxis),g=t.aaxis,v=t.baxis,m=t.x,y=t.y,b=[];m&&i(m)&&b.push("x"),y&&i(y)&&b.push("y"),b.length&&h(t,g,v,"a","b",b);var C=t._a=t._a||t.a,A=t._b=t._b||t.b;m=t._x||t.x,y=t._y||t.y;var _={};if(t._cheater){var I="index"===g.cheatertype?C.length:C,x="index"===v.cheatertype?A.length:A;m=o(I,x,t.cheaterslope)}t._x=m=c(m),t._y=y=c(y),d(m,C,A),d(y,C,A),f(t),t.setScale();var w=a(m),M=a(y),S=.5*(w[1]-w[0]),T=.5*(w[1]+w[0]),E=.5*(M[1]-M[0]),O=.5*(M[1]+M[0]);return w=[T-1.3*S,T+1.3*S],M=[O-1.3*E,O+1.3*E],t._extremes[n._id]=r.findExtremes(n,w,{padded:!0}),t._extremes[p._id]=r.findExtremes(p,M,{padded:!0}),s(t,"a","b"),s(t,"b","a"),l(t,g),l(t,v),_.clipsegments=u(t._xctrl,t._yctrl,g,v),_.x=m,_.y=y,_.a=C,_.b=A,[_]}},function(e,t,n){"use strict";var r=n(10).isArrayOrTypedArray;e.exports=function(e,t,n){var i,o,a,s,l,u,c=[],d=r(e)?e.length:e,h=r(t)?t.length:t,f=r(e)?e:null,p=r(t)?t:null;f&&(a=(f.length-1)/(f[f.length-1]-f[0])/(d-1)),p&&(s=(p.length-1)/(p[p.length-1]-p[0])/(h-1));var g=1/0,v=-1/0;for(o=0;o<h;o++)for(c[o]=[],l=p?(p[o]-p[0])*s:o/(h-1),i=0;i<d;i++)u=(f?(f[i]-f[0])*a:i/(d-1))-l*n,g=Math.min(u,g),v=Math.max(u,v),c[o][i]=u;var m=1/(v-g),y=-g*m;for(o=0;o<h;o++)for(i=0;i<d;i++)c[o][i]=m*c[o][i]+y;return c}},function(e,t,n){"use strict";var r=n(10).isArrayOrTypedArray;e.exports=function(e){return function e(t,n){if(!r(t)||n>=10)return null;for(var i=1/0,o=-1/0,a=t.length,s=0;s<a;s++){var l=t[s];if(r(l)){var u=e(l,n+1);u&&(i=Math.min(u[0],i),o=Math.max(u[1],o))}else i=Math.min(l,i),o=Math.max(l,o)}return[i,o]}(e,0)}},function(e,t,n){"use strict";var r=n(55),i=n(63).extendFlat;e.exports=function(e,t,n){var o,a,s,l,u,c,d,h,f,p,g,v,m,y,b=e["_"+t],C=e[t+"axis"],A=C._gridlines=[],_=C._minorgridlines=[],I=C._boundarylines=[],x=e["_"+n],w=e[n+"axis"];"array"===C.tickmode&&(C.tickvals=b.slice());var M=e._xctrl,S=e._yctrl,T=M[0].length,E=M.length,O=e._a.length,k=e._b.length;r.prepTicks(C),"array"===C.tickmode&&delete C.tickvals;var L=C.smoothing?3:1;function D(r){var i,o,a,s,l,u,c,d,f,p,g,v,m=[],y=[],b={};if("b"===t)for(o=e.b2j(r),a=Math.floor(Math.max(0,Math.min(k-2,o))),s=o-a,b.length=k,b.crossLength=O,b.xy=function(t){return e.evalxy([],t,o)},b.dxy=function(t,n){return e.dxydi([],t,a,n,s)},i=0;i<O;i++)u=Math.min(O-2,i),c=i-u,d=e.evalxy([],i,o),w.smoothing&&i>0&&(f=e.dxydi([],i-1,a,0,s),m.push(l[0]+f[0]/3),y.push(l[1]+f[1]/3),p=e.dxydi([],i-1,a,1,s),m.push(d[0]-p[0]/3),y.push(d[1]-p[1]/3)),m.push(d[0]),y.push(d[1]),l=d;else for(i=e.a2i(r),u=Math.floor(Math.max(0,Math.min(O-2,i))),c=i-u,b.length=O,b.crossLength=k,b.xy=function(t){return e.evalxy([],i,t)},b.dxy=function(t,n){return e.dxydj([],u,t,c,n)},o=0;o<k;o++)a=Math.min(k-2,o),s=o-a,d=e.evalxy([],i,o),w.smoothing&&o>0&&(g=e.dxydj([],u,o-1,c,0),m.push(l[0]+g[0]/3),y.push(l[1]+g[1]/3),v=e.dxydj([],u,o-1,c,1),m.push(d[0]-v[0]/3),y.push(d[1]-v[1]/3)),m.push(d[0]),y.push(d[1]),l=d;return b.axisLetter=t,b.axis=C,b.crossAxis=w,b.value=r,b.constvar=n,b.index=h,b.x=m,b.y=y,b.smoothing=w.smoothing,b}function P(r){var i,o,a,s,l,u=[],c=[],d={};if(d.length=b.length,d.crossLength=x.length,"b"===t)for(a=Math.max(0,Math.min(k-2,r)),l=Math.min(1,Math.max(0,r-a)),d.xy=function(t){return e.evalxy([],t,r)},d.dxy=function(t,n){return e.dxydi([],t,a,n,l)},i=0;i<T;i++)u[i]=M[r*L][i],c[i]=S[r*L][i];else for(o=Math.max(0,Math.min(O-2,r)),s=Math.min(1,Math.max(0,r-o)),d.xy=function(t){return e.evalxy([],r,t)},d.dxy=function(t,n){return e.dxydj([],o,t,s,n)},i=0;i<E;i++)u[i]=M[i][r*L],c[i]=S[i][r*L];return d.axisLetter=t,d.axis=C,d.crossAxis=w,d.value=b[r],d.constvar=n,d.index=r,d.x=u,d.y=c,d.smoothing=w.smoothing,d}if("array"===C.tickmode){for(l=5e-15,c=(u=[Math.floor((b.length-1-C.arraytick0)/C.arraydtick*(1+l)),Math.ceil(-C.arraytick0/C.arraydtick/(1+l))].sort((function(e,t){return e-t})))[0]-1,d=u[1]+1,h=c;h<d;h++)(a=C.arraytick0+C.arraydtick*h)<0||a>b.length-1||A.push(i(P(a),{color:C.gridcolor,width:C.gridwidth}));for(h=c;h<d;h++)if(s=C.arraytick0+C.arraydtick*h,g=Math.min(s+C.arraydtick,b.length-1),!(s<0||s>b.length-1||g<0||g>b.length-1))for(v=b[s],m=b[g],o=0;o<C.minorgridcount;o++)(y=g-s)<=0||(p=v+(m-v)*(o+1)/(C.minorgridcount+1)*(C.arraydtick/y))<b[0]||p>b[b.length-1]||_.push(i(D(p),{color:C.minorgridcolor,width:C.minorgridwidth}));C.startline&&I.push(i(P(0),{color:C.startlinecolor,width:C.startlinewidth})),C.endline&&I.push(i(P(b.length-1),{color:C.endlinecolor,width:C.endlinewidth}))}else{for(l=5e-15,c=(u=[Math.floor((b[b.length-1]-C.tick0)/C.dtick*(1+l)),Math.ceil((b[0]-C.tick0)/C.dtick/(1+l))].sort((function(e,t){return e-t})))[0],d=u[1],h=c;h<=d;h++)f=C.tick0+C.dtick*h,A.push(i(D(f),{color:C.gridcolor,width:C.gridwidth}));for(h=c-1;h<d+1;h++)for(f=C.tick0+C.dtick*h,o=0;o<C.minorgridcount;o++)(p=f+C.dtick*(o+1)/(C.minorgridcount+1))<b[0]||p>b[b.length-1]||_.push(i(D(p),{color:C.minorgridcolor,width:C.minorgridwidth}));C.startline&&I.push(i(D(b[0]),{color:C.startlinecolor,width:C.startlinewidth})),C.endline&&I.push(i(D(b[b.length-1]),{color:C.endlinecolor,width:C.endlinewidth}))}}},function(e,t,n){"use strict";var r=n(55),i=n(63).extendFlat;e.exports=function(e,t){var n,o,a,s=t._labels=[],l=t._gridlines;for(n=0;n<l.length;n++)a=l[n],-1!==["start","both"].indexOf(t.showticklabels)&&(o=r.tickText(t,a.value),i(o,{prefix:void 0,suffix:void 0,endAnchor:!0,xy:a.xy(0),dxy:a.dxy(0,0),axis:a.axis,length:a.crossAxis.length,font:a.axis.tickfont,isFirst:0===n,isLast:n===l.length-1}),s.push(o)),-1!==["end","both"].indexOf(t.showticklabels)&&(o=r.tickText(t,a.value),i(o,{endAnchor:!1,xy:a.xy(a.crossLength-1),dxy:a.dxy(a.crossLength-2,1),axis:a.axis,length:a.crossAxis.length,font:a.axis.tickfont,isFirst:0===n,isLast:n===l.length-1}),s.push(o))}},function(e,t,n){"use strict";e.exports=function(e,t,n,r){var i,o,a,s=[],l=!!n.smoothing,u=!!r.smoothing,c=e[0].length-1,d=e.length-1;for(i=0,o=[],a=[];i<=c;i++)o[i]=e[0][i],a[i]=t[0][i];for(s.push({x:o,y:a,bicubic:l}),i=0,o=[],a=[];i<=d;i++)o[i]=e[i][c],a[i]=t[i][c];for(s.push({x:o,y:a,bicubic:u}),i=c,o=[],a=[];i>=0;i--)o[c-i]=e[d][i],a[c-i]=t[d][i];for(s.push({x:o,y:a,bicubic:l}),i=d,o=[],a=[];i>=0;i--)o[d-i]=e[i][0],a[d-i]=t[i][0];return s.push({x:o,y:a,bicubic:u}),s}},function(e,t,n){"use strict";var r=n(10);e.exports=function(e,t,n){var i,o,a,s=[],l=[],u=e[0].length,c=e.length;function d(t,n){var r,i=0,o=0;return t>0&&void 0!==(r=e[n][t-1])&&(o++,i+=r),t<u-1&&void 0!==(r=e[n][t+1])&&(o++,i+=r),n>0&&void 0!==(r=e[n-1][t])&&(o++,i+=r),n<c-1&&void 0!==(r=e[n+1][t])&&(o++,i+=r),i/Math.max(1,o)}var h,f,p,g,v,m,y,b,C,A,_,I=0;for(i=0;i<u;i++)for(o=0;o<c;o++)void 0===e[o][i]&&(s.push(i),l.push(o),e[o][i]=d(i,o)),I=Math.max(I,Math.abs(e[o][i]));if(!s.length)return e;var x=0,w=0,M=s.length;do{for(x=0,a=0;a<M;a++){i=s[a],o=l[a];var S,T,E,O,k,L,D=0,P=0;0===i?(E=t[k=Math.min(u-1,2)],O=t[1],S=e[o][k],P+=(T=e[o][1])+(T-S)*(t[0]-O)/(O-E),D++):i===u-1&&(E=t[k=Math.max(0,u-3)],O=t[u-2],S=e[o][k],P+=(T=e[o][u-2])+(T-S)*(t[u-1]-O)/(O-E),D++),(0===i||i===u-1)&&o>0&&o<c-1&&(h=n[o+1]-n[o],P+=((f=n[o]-n[o-1])*e[o+1][i]+h*e[o-1][i])/(f+h),D++),0===o?(E=n[L=Math.min(c-1,2)],O=n[1],S=e[L][i],P+=(T=e[1][i])+(T-S)*(n[0]-O)/(O-E),D++):o===c-1&&(E=n[L=Math.max(0,c-3)],O=n[c-2],S=e[L][i],P+=(T=e[c-2][i])+(T-S)*(n[c-1]-O)/(O-E),D++),(0===o||o===c-1)&&i>0&&i<u-1&&(h=t[i+1]-t[i],P+=((f=t[i]-t[i-1])*e[o][i+1]+h*e[o][i-1])/(f+h),D++),D?P/=D:(p=t[i+1]-t[i],g=t[i]-t[i-1],b=(v=n[o+1]-n[o])*(m=n[o]-n[o-1])*(v+m),P=((y=p*g*(p+g))*(m*e[o+1][i]+v*e[o-1][i])+b*(g*e[o][i+1]+p*e[o][i-1]))/(b*(g+p)+y*(m+v))),x+=(A=(C=P-e[o][i])/I)*A,_=D?0:.85,e[o][i]+=C*(1+_)}x=Math.sqrt(x)}while(w++<100&&x>1e-5);return r.log("Smoother converged to",x,"after",w,"iterations"),e}},function(e,t,n){"use strict";var r=n(2004),i=n(538).findBin,o=n(2005),a=n(2007),s=n(2008),l=n(2009);e.exports=function(e){var t=e._a,n=e._b,u=t.length,c=n.length,d=e.aaxis,h=e.baxis,f=t[0],p=t[u-1],g=n[0],v=n[c-1],m=t[t.length-1]-t[0],y=n[n.length-1]-n[0],b=m*r.RELATIVE_CULL_TOLERANCE,C=y*r.RELATIVE_CULL_TOLERANCE;f-=b,p+=b,g-=C,v+=C,e.isVisible=function(e,t){return e>f&&e<p&&t>g&&t<v},e.isOccluded=function(e,t){return e<f||e>p||t<g||t>v},e.setScale=function(){var t=e._x,n=e._y,r=o(e._xctrl,e._yctrl,t,n,d.smoothing,h.smoothing);e._xctrl=r[0],e._yctrl=r[1],e.evalxy=a([e._xctrl,e._yctrl],u,c,d.smoothing,h.smoothing),e.dxydi=s([e._xctrl,e._yctrl],d.smoothing,h.smoothing),e.dxydj=l([e._xctrl,e._yctrl],d.smoothing,h.smoothing)},e.i2a=function(e){var n=Math.max(0,Math.floor(e[0]),u-2),r=e[0]-n;return(1-r)*t[n]+r*t[n+1]},e.j2b=function(e){var t=Math.max(0,Math.floor(e[1]),u-2),r=e[1]-t;return(1-r)*n[t]+r*n[t+1]},e.ij2ab=function(t){return[e.i2a(t[0]),e.j2b(t[1])]},e.a2i=function(e){var n=Math.max(0,Math.min(i(e,t),u-2)),r=t[n],o=t[n+1];return Math.max(0,Math.min(u-1,n+(e-r)/(o-r)))},e.b2j=function(e){var t=Math.max(0,Math.min(i(e,n),c-2)),r=n[t],o=n[t+1];return Math.max(0,Math.min(c-1,t+(e-r)/(o-r)))},e.ab2ij=function(t){return[e.a2i(t[0]),e.b2j(t[1])]},e.i2c=function(t,n){return e.evalxy([],t,n)},e.ab2xy=function(r,i,o){if(!o&&(r<t[0]||r>t[u-1]|i<n[0]||i>n[c-1]))return[!1,!1];var a=e.a2i(r),s=e.b2j(i),l=e.evalxy([],a,s);if(o){var d,h,f,p,g=0,v=0,m=[];r<t[0]?(d=0,h=0,g=(r-t[0])/(t[1]-t[0])):r>t[u-1]?(d=u-2,h=1,g=(r-t[u-1])/(t[u-1]-t[u-2])):h=a-(d=Math.max(0,Math.min(u-2,Math.floor(a)))),i<n[0]?(f=0,p=0,v=(i-n[0])/(n[1]-n[0])):i>n[c-1]?(f=c-2,p=1,v=(i-n[c-1])/(n[c-1]-n[c-2])):p=s-(f=Math.max(0,Math.min(c-2,Math.floor(s)))),g&&(e.dxydi(m,d,f,h,p),l[0]+=m[0]*g,l[1]+=m[1]*g),v&&(e.dxydj(m,d,f,h,p),l[0]+=m[0]*v,l[1]+=m[1]*v)}return l},e.c2p=function(e,t,n){return[t.c2p(e[0]),n.c2p(e[1])]},e.p2x=function(e,t,n){return[t.p2c(e[0]),n.p2c(e[1])]},e.dadi=function(e){var n=Math.max(0,Math.min(t.length-2,e));return t[n+1]-t[n]},e.dbdj=function(e){var t=Math.max(0,Math.min(n.length-2,e));return n[t+1]-n[t]},e.dxyda=function(t,n,r,i){var o=e.dxydi(null,t,n,r,i),a=e.dadi(t,r);return[o[0]/a,o[1]/a]},e.dxydb=function(t,n,r,i){var o=e.dxydj(null,t,n,r,i),a=e.dbdj(n,i);return[o[0]/a,o[1]/a]},e.dxyda_rough=function(t,n,r){var i=m*(r||.1),o=e.ab2xy(t+i,n,!0),a=e.ab2xy(t-i,n,!0);return[.5*(o[0]-a[0])/i,.5*(o[1]-a[1])/i]},e.dxydb_rough=function(t,n,r){var i=y*(r||.1),o=e.ab2xy(t,n+i,!0),a=e.ab2xy(t,n-i,!0);return[.5*(o[0]-a[0])/i,.5*(o[1]-a[1])/i]},e.dpdx=function(e){return e._m},e.dpdy=function(e){return e._m}}},function(e,t,n){"use strict";e.exports={RELATIVE_CULL_TOLERANCE:1e-6}},function(e,t,n){"use strict";var r=n(2006),i=n(10).ensureArray;function o(e,t,n){var r=-.5*n[0]+1.5*t[0],i=-.5*n[1]+1.5*t[1];return[(2*r+e[0])/3,(2*i+e[1])/3]}e.exports=function(e,t,n,a,s,l){var u,c,d,h,f,p,g,v,m,y,b=n[0].length,C=n.length,A=s?3*b-2:b,_=l?3*C-2:C;for(e=i(e,_),t=i(t,_),d=0;d<_;d++)e[d]=i(e[d],A),t[d]=i(t[d],A);for(c=0,h=0;c<C;c++,h+=l?3:1)for(f=e[h],p=t[h],g=n[c],v=a[c],u=0,d=0;u<b;u++,d+=s?3:1)f[d]=g[u],p[d]=v[u];if(s)for(c=0,h=0;c<C;c++,h+=l?3:1){for(u=1,d=3;u<b-1;u++,d+=3)m=r([n[c][u-1],a[c][u-1]],[n[c][u],a[c][u]],[n[c][u+1],a[c][u+1]],s),e[h][d-1]=m[0][0],t[h][d-1]=m[0][1],e[h][d+1]=m[1][0],t[h][d+1]=m[1][1];y=o([e[h][0],t[h][0]],[e[h][2],t[h][2]],[e[h][3],t[h][3]]),e[h][1]=y[0],t[h][1]=y[1],y=o([e[h][A-1],t[h][A-1]],[e[h][A-3],t[h][A-3]],[e[h][A-4],t[h][A-4]]),e[h][A-2]=y[0],t[h][A-2]=y[1]}if(l)for(d=0;d<A;d++){for(h=3;h<_-3;h+=3)m=r([e[h-3][d],t[h-3][d]],[e[h][d],t[h][d]],[e[h+3][d],t[h+3][d]],l),e[h-1][d]=m[0][0],t[h-1][d]=m[0][1],e[h+1][d]=m[1][0],t[h+1][d]=m[1][1];y=o([e[0][d],t[0][d]],[e[2][d],t[2][d]],[e[3][d],t[3][d]]),e[1][d]=y[0],t[1][d]=y[1],y=o([e[_-1][d],t[_-1][d]],[e[_-3][d],t[_-3][d]],[e[_-4][d],t[_-4][d]]),e[_-2][d]=y[0],t[_-2][d]=y[1]}if(s&&l)for(h=1;h<_;h+=(h+1)%3==0?2:1){for(d=3;d<A-3;d+=3)m=r([e[h][d-3],t[h][d-3]],[e[h][d],t[h][d]],[e[h][d+3],t[h][d+3]],s),e[h][d-1]=.5*(e[h][d-1]+m[0][0]),t[h][d-1]=.5*(t[h][d-1]+m[0][1]),e[h][d+1]=.5*(e[h][d+1]+m[1][0]),t[h][d+1]=.5*(t[h][d+1]+m[1][1]);y=o([e[h][0],t[h][0]],[e[h][2],t[h][2]],[e[h][3],t[h][3]]),e[h][1]=.5*(e[h][1]+y[0]),t[h][1]=.5*(t[h][1]+y[1]),y=o([e[h][A-1],t[h][A-1]],[e[h][A-3],t[h][A-3]],[e[h][A-4],t[h][A-4]]),e[h][A-2]=.5*(e[h][A-2]+y[0]),t[h][A-2]=.5*(t[h][A-2]+y[1])}return[e,t]}},function(e,t,n){"use strict";e.exports=function(e,t,n,r){var i=e[0]-t[0],o=e[1]-t[1],a=n[0]-t[0],s=n[1]-t[1],l=Math.pow(i*i+o*o,.25),u=Math.pow(a*a+s*s,.25),c=(u*u*i-l*l*a)*r,d=(u*u*o-l*l*s)*r,h=u*(l+u)*3,f=l*(l+u)*3;return[[t[0]+(h&&c/h),t[1]+(h&&d/h)],[t[0]-(f&&c/f),t[1]-(f&&d/f)]]}},function(e,t,n){"use strict";e.exports=function(e,t,n,r,i){var o=t-2,a=n-2;return r&&i?function(t,n,r){var i,s,l,u,c,d;t||(t=[]);var h=Math.max(0,Math.min(Math.floor(n),o)),f=Math.max(0,Math.min(Math.floor(r),a)),p=Math.max(0,Math.min(1,n-h)),g=Math.max(0,Math.min(1,r-f));h*=3,f*=3;var v=p*p,m=v*p,y=1-p,b=y*y,C=b*y,A=g*g,_=A*g,I=1-g,x=I*I,w=x*I;for(d=0;d<e.length;d++)i=C*(c=e[d])[f][h]+3*(b*p*c[f][h+1]+y*v*c[f][h+2])+m*c[f][h+3],s=C*c[f+1][h]+3*(b*p*c[f+1][h+1]+y*v*c[f+1][h+2])+m*c[f+1][h+3],l=C*c[f+2][h]+3*(b*p*c[f+2][h+1]+y*v*c[f+2][h+2])+m*c[f+2][h+3],u=C*c[f+3][h]+3*(b*p*c[f+3][h+1]+y*v*c[f+3][h+2])+m*c[f+3][h+3],t[d]=w*i+3*(x*g*s+I*A*l)+_*u;return t}:r?function(t,n,r){t||(t=[]);var i,s,l,u,c,d,h=Math.max(0,Math.min(Math.floor(n),o)),f=Math.max(0,Math.min(Math.floor(r),a)),p=Math.max(0,Math.min(1,n-h)),g=Math.max(0,Math.min(1,r-f));h*=3;var v=p*p,m=v*p,y=1-p,b=y*y,C=b*y,A=1-g;for(c=0;c<e.length;c++)i=A*(d=e[c])[f][h]+g*d[f+1][h],s=A*d[f][h+1]+g*d[f+1][h+1],l=A*d[f][h+2]+g*d[f+1][h+1],u=A*d[f][h+3]+g*d[f+1][h+1],t[c]=C*i+3*(b*p*s+y*v*l)+m*u;return t}:i?function(t,n,r){t||(t=[]);var i,s,l,u,c,d,h=Math.max(0,Math.min(Math.floor(n),o)),f=Math.max(0,Math.min(Math.floor(r),a)),p=Math.max(0,Math.min(1,n-h)),g=Math.max(0,Math.min(1,r-f));f*=3;var v=g*g,m=v*g,y=1-g,b=y*y,C=b*y,A=1-p;for(c=0;c<e.length;c++)i=A*(d=e[c])[f][h]+p*d[f][h+1],s=A*d[f+1][h]+p*d[f+1][h+1],l=A*d[f+2][h]+p*d[f+2][h+1],u=A*d[f+3][h]+p*d[f+3][h+1],t[c]=C*i+3*(b*g*s+y*v*l)+m*u;return t}:function(t,n,r){t||(t=[]);var i,s,l,u,c=Math.max(0,Math.min(Math.floor(n),o)),d=Math.max(0,Math.min(Math.floor(r),a)),h=Math.max(0,Math.min(1,n-c)),f=Math.max(0,Math.min(1,r-d)),p=1-f,g=1-h;for(l=0;l<e.length;l++)i=g*(u=e[l])[d][c]+h*u[d][c+1],s=g*u[d+1][c]+h*u[d+1][c+1],t[l]=p*i+f*s;return t}}},function(e,t,n){"use strict";e.exports=function(e,t,n){return t&&n?function(t,n,r,i,o){var a,s,l,u,c,d;t||(t=[]),n*=3,r*=3;var h=i*i,f=1-i,p=f*f,g=f*i*2,v=-3*p,m=3*(p-g),y=3*(g-h),b=3*h,C=o*o,A=C*o,_=1-o,I=_*_,x=I*_;for(d=0;d<e.length;d++)a=v*(c=e[d])[r][n]+m*c[r][n+1]+y*c[r][n+2]+b*c[r][n+3],s=v*c[r+1][n]+m*c[r+1][n+1]+y*c[r+1][n+2]+b*c[r+1][n+3],l=v*c[r+2][n]+m*c[r+2][n+1]+y*c[r+2][n+2]+b*c[r+2][n+3],u=v*c[r+3][n]+m*c[r+3][n+1]+y*c[r+3][n+2]+b*c[r+3][n+3],t[d]=x*a+3*(I*o*s+_*C*l)+A*u;return t}:t?function(t,n,r,i,o){var a,s,l,u;t||(t=[]),n*=3;var c=i*i,d=1-i,h=d*d,f=d*i*2,p=-3*h,g=3*(h-f),v=3*(f-c),m=3*c,y=1-o;for(l=0;l<e.length;l++)a=p*(u=e[l])[r][n]+g*u[r][n+1]+v*u[r][n+2]+m*u[r][n+3],s=p*u[r+1][n]+g*u[r+1][n+1]+v*u[r+1][n+2]+m*u[r+1][n+3],t[l]=y*a+o*s;return t}:n?function(t,n,r,i,o){var a,s,l,u,c,d;t||(t=[]),r*=3;var h=o*o,f=h*o,p=1-o,g=p*p,v=g*p;for(c=0;c<e.length;c++)a=(d=e[c])[r][n+1]-d[r][n],s=d[r+1][n+1]-d[r+1][n],l=d[r+2][n+1]-d[r+2][n],u=d[r+3][n+1]-d[r+3][n],t[c]=v*a+3*(g*o*s+p*h*l)+f*u;return t}:function(t,n,r,i,o){var a,s,l,u;t||(t=[]);var c=1-o;for(l=0;l<e.length;l++)a=(u=e[l])[r][n+1]-u[r][n],s=u[r+1][n+1]-u[r+1][n],t[l]=c*a+o*s;return t}}},function(e,t,n){"use strict";e.exports=function(e,t,n){return t&&n?function(t,n,r,i,o){var a,s,l,u,c,d;t||(t=[]),n*=3,r*=3;var h=i*i,f=h*i,p=1-i,g=p*p,v=g*p,m=o*o,y=1-o,b=y*y,C=y*o*2,A=-3*b,_=3*(b-C),I=3*(C-m),x=3*m;for(d=0;d<e.length;d++)a=A*(c=e[d])[r][n]+_*c[r+1][n]+I*c[r+2][n]+x*c[r+3][n],s=A*c[r][n+1]+_*c[r+1][n+1]+I*c[r+2][n+1]+x*c[r+3][n+1],l=A*c[r][n+2]+_*c[r+1][n+2]+I*c[r+2][n+2]+x*c[r+3][n+2],u=A*c[r][n+3]+_*c[r+1][n+3]+I*c[r+2][n+3]+x*c[r+3][n+3],t[d]=v*a+3*(g*i*s+p*h*l)+f*u;return t}:t?function(t,n,r,i,o){var a,s,l,u,c,d;t||(t=[]),n*=3;var h=o*o,f=h*o,p=1-o,g=p*p,v=g*p;for(c=0;c<e.length;c++)a=(d=e[c])[r+1][n]-d[r][n],s=d[r+1][n+1]-d[r][n+1],l=d[r+1][n+2]-d[r][n+2],u=d[r+1][n+3]-d[r][n+3],t[c]=v*a+3*(g*o*s+p*h*l)+f*u;return t}:n?function(t,n,r,i,o){var a,s,l,u;t||(t=[]),r*=3;var c=1-i,d=o*o,h=1-o,f=h*h,p=h*o*2,g=-3*f,v=3*(f-p),m=3*(p-d),y=3*d;for(l=0;l<e.length;l++)a=g*(u=e[l])[r][n]+v*u[r+1][n]+m*u[r+2][n]+y*u[r+3][n],s=g*u[r][n+1]+v*u[r+1][n+1]+m*u[r+2][n+1]+y*u[r+3][n+1],t[l]=c*a+i*s;return t}:function(t,n,r,i,o){var a,s,l,u;t||(t=[]);var c=1-i;for(l=0;l<e.length;l++)a=(u=e[l])[r+1][n]-u[r][n],s=u[r+1][n+1]-u[r][n+1],t[l]=c*a+i*s;return t}}},function(e,t,n){"use strict";e.exports=n(2011)},function(e,t,n){"use strict";e.exports={attributes:n(1084),supplyDefaults:n(1086),colorbar:n(2017),calc:n(2018),plot:n(2023),style:n(2028),moduleType:"trace",name:"contourcarpet",basePlotModule:n(301),categories:["cartesian","svg","carpet","contour","symbols","showLegend","hasLines","carpetDependent","noHover","noSortingByValue"],meta:{}}},function(e,t,n){"use strict";var r=n(1085),i=n(193),o=n(161),a=n(296).dash,s=n(94),l=n(63).extendFlat,u=n(580),c=u.COMPARISON_OPS2,d=u.INTERVAL_OPS,h=(n(194).FORMAT_LINK,i.line);e.exports=l({z:r.z,x:r.x,x0:r.x0,dx:r.dx,y:r.y,y0:r.y0,dy:r.dy,xperiod:r.xperiod,yperiod:r.yperiod,xperiod0:i.xperiod0,yperiod0:i.yperiod0,xperiodalignment:r.xperiodalignment,yperiodalignment:r.yperiodalignment,text:r.text,hovertext:r.hovertext,transpose:r.transpose,xtype:r.xtype,ytype:r.ytype,zhoverformat:r.zhoverformat,hovertemplate:r.hovertemplate,hoverongaps:r.hoverongaps,connectgaps:l({},r.connectgaps,{}),fillcolor:{valType:"color",editType:"calc"},autocontour:{valType:"boolean",dflt:!0,editType:"calc",impliedEdits:{"contours.start":void 0,"contours.end":void 0,"contours.size":void 0}},ncontours:{valType:"integer",dflt:15,min:1,editType:"calc"},contours:{type:{valType:"enumerated",values:["levels","constraint"],dflt:"levels",editType:"calc"},start:{valType:"number",dflt:null,editType:"plot",impliedEdits:{"^autocontour":!1}},end:{valType:"number",dflt:null,editType:"plot",impliedEdits:{"^autocontour":!1}},size:{valType:"number",dflt:null,min:0,editType:"plot",impliedEdits:{"^autocontour":!1}},coloring:{valType:"enumerated",values:["fill","heatmap","lines","none"],dflt:"fill",editType:"calc"},showlines:{valType:"boolean",dflt:!0,editType:"plot"},showlabels:{valType:"boolean",dflt:!1,editType:"plot"},labelfont:s({editType:"plot",colorEditType:"style"}),labelformat:{valType:"string",dflt:"",editType:"plot"},operation:{valType:"enumerated",values:[].concat(c).concat(d),dflt:"=",editType:"calc"},value:{valType:"any",dflt:0,editType:"calc"},editType:"calc",impliedEdits:{autocontour:!1}},line:{color:l({},h.color,{editType:"style+colorbars"}),width:{valType:"number",min:0,editType:"style+colorbars"},dash:a,smoothing:l({},h.smoothing,{}),editType:"plot"}},o("",{cLetter:"z",autoColorDflt:!1,editTypeOverride:"calc"}))},function(e,t,n){"use strict";var r=n(42),i=n(10),o=n(27);function a(e,t){var n=t(e);return"scaled"===(n?t(e+"type","array"):"scaled")&&(t(e+"0"),t("d"+e)),n}e.exports=function(e,t,n,s,l,u){var c,d,h=n("z");if(l=l||"x",u=u||"y",void 0===h||!h.length)return 0;if(i.isArray1D(e.z)){c=n(l),d=n(u);var f=i.minRowLength(c),p=i.minRowLength(d);if(0===f||0===p)return 0;t._length=Math.min(f,p,h.length)}else{if(c=a(l,n),d=a(u,n),!function(e){for(var t,n=!0,o=!1,a=!1,s=0;s<e.length;s++){if(t=e[s],!i.isArrayOrTypedArray(t)){n=!1;break}t.length>0&&(o=!0);for(var l=0;l<t.length;l++)if(r(t[l])){a=!0;break}}return n&&o&&a}(h))return 0;n("transpose"),t._length=null}return"heatmapgl"===e.type||"contourgl"===e.type||o.getComponentMethod("calendars","handleTraceDefaults")(e,t,[l,u],s),!0}},function(e,t,n){"use strict";var r=n(42),i=n(1087),o=n(41),a=o.addOpacity,s=o.opacity,l=n(580),u=l.CONSTRAINT_REDUCTION,c=l.COMPARISON_OPS2;e.exports=function(e,t,n,o,l,d){var h,f,p,g=t.contours,v=n("contours.operation");(g._operation=u[v],function(e,t){var n;-1===c.indexOf(t.operation)?(e("contours.value",[0,1]),Array.isArray(t.value)?t.value.length>2?t.value=t.value.slice(2):0===t.length?t.value=[0,1]:t.length<2?(n=parseFloat(t.value[0]),t.value=[n,n+1]):t.value=[parseFloat(t.value[0]),parseFloat(t.value[1])]:r(t.value)&&(n=parseFloat(t.value),t.value=[n,n+1])):(e("contours.value",0),r(t.value)||(Array.isArray(t.value)?t.value=parseFloat(t.value[0]):t.value=0))}(n,g),"="===v?h=g.showlines=!0:(h=n("contours.showlines"),p=n("fillcolor",a((e.line||{}).color||l,.5))),h)&&(f=n("line.color",p&&s(p)?a(t.fillcolor,1):l),n("line.width",2),n("line.dash"));n("line.smoothing"),i(n,o,f,d)}},function(e,t,n){"use strict";e.exports=function(e,t,n,r){var i=r("contours.start"),o=r("contours.end"),a=!1===i||!1===o,s=n("contours.size");!(a?t.autocontour=!0:n("autocontour",!1))&&s||n("ncontours")}},function(e,t,n){"use strict";var r=n(213),i=n(1087);e.exports=function(e,t,n,o,a){var s,l=n("contours.coloring"),u="";"fill"===l&&(s=n("contours.showlines")),!1!==s&&("lines"!==l&&(u=n("line.color","#000")),n("line.width",.5),n("line.dash")),"none"!==l&&(!0!==e.showlegend&&(t.showlegend=!1),t._dfltShowLegend=!1,r(e,t,o,n,{prefix:"",cLetter:"z"})),n("line.smoothing"),i(n,o,u,a)}},function(e,t,n){"use strict";var r=n(195),i=n(1088),o=n(581);e.exports={min:"zmin",max:"zmax",calc:function(e,t,n){var a=t.contours,s=t.line,l=a.size||1,u=a.coloring,c=i(t,{isColorbar:!0});if("heatmap"===u){var d=r.extractOpts(t);n._fillgradient=d.reversescale?r.flipScale(d.colorscale):d.colorscale,n._zrange=[d.min,d.max]}else"fill"===u&&(n._fillcolor=c);n._line={color:"lines"===u?c:s.color,width:!1!==a.showlines?s.width:0,dash:s.dash},n._levels={start:a.start,end:o(a),size:l}}}},function(e,t,n){"use strict";var r=n(298),i=n(10),o=n(1083),a=n(1082),s=n(2019),l=n(2020),u=n(2021),c=n(1086),d=n(1089),h=n(2022);e.exports=function(e,t){var n=t._carpetTrace=d(e,t);if(n&&n.visible&&"legendonly"!==n.visible){if(!t.a||!t.b){var f=e.data[n.index],p=e.data[t.index];p.a||(p.a=f.a),p.b||(p.b=f.b),c(p,t,t._defaultColor,e._fullLayout)}var g=function(e,t){var n,c,d,h,f,p,g,v=t._carpetTrace,m=v.aaxis,y=v.baxis;m._minDtick=0,y._minDtick=0,i.isArray1D(t.z)&&o(t,m,y,"a","b",["z"]);n=t._a=t._a||t.a,h=t._b=t._b||t.b,n=n?m.makeCalcdata(t,"_a"):[],h=h?y.makeCalcdata(t,"_b"):[],c=t.a0||0,d=t.da||1,f=t.b0||0,p=t.db||1,g=t._z=a(t._z||t.z,t.transpose),t._emptypoints=l(g),s(g,t._emptypoints);var b=i.maxRowLength(g),C="scaled"===t.xtype?"":n,A=u(t,C,c,d,b,m),_="scaled"===t.ytype?"":h,I=u(t,_,f,p,g.length,y),x={a:A,b:I,z:g};"levels"===t.contours.type&&"none"!==t.contours.coloring&&r(e,t,{vals:g,containerStr:"",cLetter:"z"});return[x]}(e,t);return h(t,t._z),g}}},function(e,t,n){"use strict";var r=n(10),i=[[-1,0],[1,0],[0,-1],[0,1]];function o(e){return.5-.25*Math.min(1,.5*e)}function a(e,t,n){var r,o,a,s,l,u,c,d,h,f,p,g,v,m=0;for(s=0;s<t.length;s++){for(o=(r=t[s])[0],a=r[1],p=e[o][a],f=0,h=0,l=0;l<4;l++)(c=e[o+(u=i[l])[0]])&&void 0!==(d=c[a+u[1]])&&(0===f?g=v=d:(g=Math.min(g,d),v=Math.max(v,d)),h++,f+=d);if(0===h)throw"iterateInterp2d order is wrong: no defined neighbors";e[o][a]=f/h,void 0===p?h<4&&(m=1):(e[o][a]=(1+n)*e[o][a]-n*p,v>g&&(m=Math.max(m,Math.abs(e[o][a]-p)/(v-g))))}return m}e.exports=function(e,t){var n,i=1;for(a(e,t),n=0;n<t.length&&!(t[n][2]<4);n++);for(t=t.slice(n),n=0;n<100&&i>.01;n++)i=a(e,t,o(i));return i>.01&&r.log("interp2d didn't converge quickly",i),e}},function(e,t,n){"use strict";var r=n(10).maxRowLength;e.exports=function(e){var t,n,i,o,a,s,l,u,c=[],d={},h=[],f=e[0],p=[],g=[0,0,0],v=r(e);for(n=0;n<e.length;n++)for(t=p,p=f,f=e[n+1]||[],i=0;i<v;i++)void 0===p[i]&&((s=(void 0!==p[i-1]?1:0)+(void 0!==p[i+1]?1:0)+(void 0!==t[i]?1:0)+(void 0!==f[i]?1:0))?(0===n&&s++,0===i&&s++,n===e.length-1&&s++,i===p.length-1&&s++,s<4&&(d[[n,i]]=[n,i,s]),c.push([n,i,s])):h.push([n,i]));for(;h.length;){for(l={},u=!1,a=h.length-1;a>=0;a--)(s=((d[[(n=(o=h[a])[0])-1,i=o[1]]]||g)[2]+(d[[n+1,i]]||g)[2]+(d[[n,i-1]]||g)[2]+(d[[n,i+1]]||g)[2])/20)&&(l[o]=[n,i,s],h.splice(a,1),u=!0);if(!u)throw"findEmpties iterated with no new neighbors";for(o in l)d[o]=l[o],c.push(l[o])}return c.sort((function(e,t){return t[2]-e[2]}))}},function(e,t,n){"use strict";var r=n(27),i=n(10).isArrayOrTypedArray;e.exports=function(e,t,n,o,a,s){var l,u,c,d=[],h=r.traceIs(e,"contour"),f=r.traceIs(e,"histogram"),p=r.traceIs(e,"gl2d");if(i(t)&&t.length>1&&!f&&"category"!==s.type){var g=t.length;if(!(g<=a))return h?t.slice(0,a):t.slice(0,a+1);if(h||p)d=t.slice(0,a);else if(1===a)d=[t[0]-.5,t[0]+.5];else{for(d=[1.5*t[0]-.5*t[1]],c=1;c<g;c++)d.push(.5*(t[c-1]+t[c]));d.push(1.5*t[g-1]-.5*t[g-2])}if(g<a){var v=d[d.length-1],m=v-d[d.length-2];for(c=g;c<a;c++)v+=m,d.push(v)}}else{var y=e[s._id.charAt(0)+"calendar"];if(f)l=s.r2c(n,0,y);else if(i(t)&&1===t.length)l=t[0];else if(void 0===n)l=0;else{l=("log"===s.type?s.d2c:s.r2c)(n,0,y)}for(u=o||1,c=h||p?0:-.5;c<a;c++)d.push(l+u*c)}return d}},function(e,t,n){"use strict";var r=n(55),i=n(10);function o(e,t,n){var i={type:"linear",range:[e,t]};return r.autoTicks(i,(t-e)/(n||15)),i}e.exports=function(e,t){var n=e.contours;if(e.autocontour){var a=e.zmin,s=e.zmax;(e.zauto||void 0===a)&&(a=i.aggNums(Math.min,null,t)),(e.zauto||void 0===s)&&(s=i.aggNums(Math.max,null,t));var l=o(a,s,e.ncontours);n.size=l.dtick,n.start=r.tickFirst(l),l.range.reverse(),n.end=r.tickFirst(l),n.start===a&&(n.start+=n.size),n.end===s&&(n.end-=n.size),n.start>n.end&&(n.start=n.end=(n.start+n.end)/2),e._input.contours||(e._input.contours={}),i.extendFlat(e._input.contours,{start:n.start,end:n.end,size:n.size}),e._input.autocontour=!0}else if("constraint"!==n.type){var u,c=n.start,d=n.end,h=e._input.contours;if(c>d&&(n.start=h.start=d,d=n.end=h.end=c,c=n.start),!(n.size>0))u=c===d?1:o(c,d,e.ncontours).dtick,h.size=n.size=u}}},function(e,t,n){"use strict";var r=n(39),i=n(1080),o=n(1081),a=n(56),s=n(10),l=n(1090),u=n(1091),c=n(2024),d=n(399),h=n(1093),f=n(1092),p=n(1094),g=n(1089),v=n(2027);function m(e,t,n){var r=e.getPointAtLength(t),i=e.getPointAtLength(n),o=i.x-r.x,a=i.y-r.y,s=Math.sqrt(o*o+a*a);return[o/s,a/s]}function y(e){var t=Math.sqrt(e[0]*e[0]+e[1]*e[1]);return[e[0]/t,e[1]/t]}function b(e,t){var n=Math.abs(e[0]*t[0]+e[1]*t[1]);return Math.sqrt(1-n*n)/n}e.exports=function(e,t,n,C){var A=t.xaxis,_=t.yaxis;s.makeTraceGroups(C,n,"contour").each((function(n){var C=r.select(this),I=n[0],x=I.trace,w=x._carpetTrace=g(e,x),M=e.calcdata[w.index][0];if(w.visible&&"legendonly"!==w.visible){var S=I.a,T=I.b,E=x.contours,O=f(E,t,I),k="constraint"===E.type,L=E._operation,D=k?"="===L?"lines":"fill":E.coloring,P=[[S[0],T[T.length-1]],[S[S.length-1],T[T.length-1]],[S[S.length-1],T[0]],[S[0],T[0]]];l(O);var N=1e-8*(S[S.length-1]-S[0]),j=1e-8*(T[T.length-1]-T[0]);u(O,N,j);var R,z,F,B,H=O;"constraint"===E.type&&(H=h(O,L)),function(e,t){var n,r,i,o,a,s,l,u,c;for(n=0;n<e.length;n++){for(o=e[n],a=o.pedgepaths=[],s=o.ppaths=[],r=0;r<o.edgepaths.length;r++){for(c=o.edgepaths[r],l=[],i=0;i<c.length;i++)l[i]=t(c[i]);a.push(l)}for(r=0;r<o.paths.length;r++){for(c=o.paths[r],u=[],i=0;i<c.length;i++)u[i]=t(c[i]);s.push(u)}}}(O,Y);var U=[];for(B=M.clipsegments.length-1;B>=0;B--)R=M.clipsegments[B],z=i([],R.x,A.c2p),F=i([],R.y,_.c2p),z.reverse(),F.reverse(),U.push(o(z,F,R.bicubic));var V="M"+U.join("L")+"Z";!function(e,t,n,r,a,l){var u,c,d,h,f=s.ensureSingle(e,"g","contourbg").selectAll("path").data("fill"!==l||a?[]:[0]);f.enter().append("path"),f.exit().remove();var p=[];for(h=0;h<t.length;h++)u=t[h],c=i([],u.x,n.c2p),d=i([],u.y,r.c2p),p.push(o(c,d,u.bicubic));f.attr("d","M"+p.join("L")+"Z").style("stroke","none")}(C,M.clipsegments,A,_,k,D),function(e,t,n,i,o,l,u,c,d,h,f){var g="fill"===h;g&&p(o,e.contours);var m=s.ensureSingle(t,"g","contourfill").selectAll("path").data(g?o:[]);m.enter().append("path"),m.exit().remove(),m.each((function(e){var t=(e.prefixBoundary?f:"")+function(e,t,n,r,i,o,l,u){var c,d,h,f,p,g,m,y="",b=t.edgepaths.map((function(e,t){return t})),C=!0,A=1e-4*Math.abs(n[0][0]-n[2][0]),_=1e-4*Math.abs(n[0][1]-n[2][1]);function I(e){return Math.abs(e[1]-n[0][1])<_}function x(e){return Math.abs(e[1]-n[2][1])<_}function w(e){return Math.abs(e[0]-n[0][0])<A}function M(e){return Math.abs(e[0]-n[2][0])<A}function S(e,t){var n,r,a,s,c="";for(I(e)&&!M(e)||x(e)&&!w(e)?(s=i.aaxis,a=v(i,o,[e[0],t[0]],.5*(e[1]+t[1]))):(s=i.baxis,a=v(i,o,.5*(e[0]+t[0]),[e[1],t[1]])),n=1;n<a.length;n++)for(c+=s.smoothing?"C":"L",r=0;r<a[n].length;r++){var d=a[n][r];c+=[l.c2p(d[0]),u.c2p(d[1])]+" "}return c}c=0,d=null;for(;b.length;){var T=t.edgepaths[c][0];for(d&&(y+=S(d,T)),m=a.smoothopen(t.edgepaths[c].map(r),t.smoothing),y+=C?m:m.replace(/^M/,"L"),b.splice(b.indexOf(c),1),d=t.edgepaths[c][t.edgepaths[c].length-1],p=-1,f=0;f<4;f++){if(!d){s.log("Missing end?",c,t);break}for(I(d)&&!M(d)?h=n[1]:w(d)?h=n[0]:x(d)?h=n[3]:M(d)&&(h=n[2]),g=0;g<t.edgepaths.length;g++){var E=t.edgepaths[g][0];Math.abs(d[0]-h[0])<A?Math.abs(d[0]-E[0])<A&&(E[1]-d[1])*(h[1]-E[1])>=0&&(h=E,p=g):Math.abs(d[1]-h[1])<_?Math.abs(d[1]-E[1])<_&&(E[0]-d[0])*(h[0]-E[0])>=0&&(h=E,p=g):s.log("endpt to newendpt is not vert. or horz.",d,h,E)}if(p>=0)break;y+=S(d,h),d=h}if(p===t.edgepaths.length){s.log("unclosed perimeter path");break}c=p,(C=-1===b.indexOf(c))&&(c=b[0],y+=S(d,h)+"Z",d=null)}for(c=0;c<t.paths.length;c++)y+=a.smoothclosed(t.paths[c].map(r),t.smoothing);return y}(0,e,l,u,c,d,n,i);t?r.select(this).attr("d",t).style("stroke","none"):r.select(this).remove()}))}(x,C,A,_,H,P,Y,w,M,D,V),function(e,t,n,i,o,l,u){var h=s.ensureSingle(e,"g","contourlines"),f=!1!==o.showlines,p=o.showlabels,g=f&&p,v=c.createLines(h,f||p,t),C=c.createLineClip(h,g,n,i.trace.uid),A=e.selectAll("g.contourlabels").data(p?[0]:[]);if(A.exit().remove(),A.enter().append("g").classed("contourlabels",!0),p){var _=l.xaxis,I=l.yaxis,x=_._length,w=I._length,M=[[[0,0],[x,0],[x,w],[0,w]]],S=[];s.clearLocationCache();var T=c.labelFormatter(n,i),E=a.tester.append("text").attr("data-notex",1).call(a.font,o.labelfont),O={left:0,right:x,center:x/2,top:0,bottom:w,middle:w/2},k=Math.sqrt(x*x+w*w),L=d.LABELDISTANCE*k/Math.max(1,t.length/d.LABELINCREASE);v.each((function(e){var t=c.calcTextOpts(e.level,T,E,n);r.select(this).selectAll("path").each((function(n){var r=s.getVisibleSegment(this,O,t.height/2);if(r&&(function(e,t,n,r,i,o){for(var a,s=0;s<n.pedgepaths.length;s++)t===n.pedgepaths[s]&&(a=n.edgepaths[s]);if(!a)return;var l=i.a[0],u=i.a[i.a.length-1],c=i.b[0],d=i.b[i.b.length-1];function h(e,t){var n,r=0;return(Math.abs(e[0]-l)<.1||Math.abs(e[0]-u)<.1)&&(n=y(i.dxydb_rough(e[0],e[1],.1)),r=Math.max(r,o*b(t,n)/2)),(Math.abs(e[1]-c)<.1||Math.abs(e[1]-d)<.1)&&(n=y(i.dxyda_rough(e[0],e[1],.1)),r=Math.max(r,o*b(t,n)/2)),r}var f=m(e,0,1),p=m(e,r.total,r.total-1),g=h(a[0],f),v=r.total-h(a[a.length-1],p);r.min<g&&(r.min=g);r.max>v&&(r.max=v);r.len=r.max-r.min}(this,n,e,r,u,t.height),!(r.len<(t.width+t.height)*d.LABELMIN)))for(var i=Math.min(Math.ceil(r.len/L),d.LABELMAX),o=0;o<i;o++){var a=c.findBestTextLocation(this,r,t,S,O);if(!a)break;c.addLabelData(a,t,S,M)}}))})),E.remove(),c.drawLabels(A,S,n,C,g?M:null)}p&&!f&&v.remove()}(C,O,e,I,E,t,w),a.setClipUrl(C,w._clipPathId,e)}function Y(e){var t=w.ab2xy(e[0],e[1],!0);return[A.c2p(t[0]),_.c2p(t[1])]}}))}},function(e,t,n){"use strict";var r=n(39),i=n(10),o=n(56),a=n(195),s=n(96),l=n(55),u=n(300),c=n(2025),d=n(1090),h=n(1091),f=n(1092),p=n(1093),g=n(1094),v=n(399),m=v.LABELOPTIMIZER;function y(e,t){var n,r,a,s,l,u,c,d="",h=0,f=e.edgepaths.map((function(e,t){return t})),p=!0;function g(e){return Math.abs(e[1]-t[2][1])<.01}function v(e){return Math.abs(e[0]-t[0][0])<.01}function m(e){return Math.abs(e[0]-t[2][0])<.01}for(;f.length;){for(u=o.smoothopen(e.edgepaths[h],e.smoothing),d+=p?u:u.replace(/^M/,"L"),f.splice(f.indexOf(h),1),n=e.edgepaths[h][e.edgepaths[h].length-1],s=-1,a=0;a<4;a++){if(!n){i.log("Missing end?",h,e);break}for(c=n,Math.abs(c[1]-t[0][1])<.01&&!m(n)?r=t[1]:v(n)?r=t[0]:g(n)?r=t[3]:m(n)&&(r=t[2]),l=0;l<e.edgepaths.length;l++){var y=e.edgepaths[l][0];Math.abs(n[0]-r[0])<.01?Math.abs(n[0]-y[0])<.01&&(y[1]-n[1])*(r[1]-y[1])>=0&&(r=y,s=l):Math.abs(n[1]-r[1])<.01?Math.abs(n[1]-y[1])<.01&&(y[0]-n[0])*(r[0]-y[0])>=0&&(r=y,s=l):i.log("endpt to newendpt is not vert. or horz.",n,r,y)}if(n=r,s>=0)break;d+="L"+r}if(s===e.edgepaths.length){i.log("unclosed perimeter path");break}h=s,(p=-1===f.indexOf(h))&&(h=f[0],d+="Z")}for(h=0;h<e.paths.length;h++)d+=o.smoothclosed(e.paths[h],e.smoothing);return d}function b(e,t,n,r){var o=t.width/2,a=t.height/2,s=e.x,l=e.y,u=e.theta,c=Math.cos(u)*o,d=Math.sin(u)*o,h=(s>r.center?r.right-s:s-r.left)/(c+Math.abs(Math.sin(u)*a)),f=(l>r.middle?r.bottom-l:l-r.top)/(Math.abs(d)+Math.cos(u)*a);if(h<1||f<1)return 1/0;var p=m.EDGECOST*(1/(h-1)+1/(f-1));p+=m.ANGLECOST*u*u;for(var g=s-c,v=l-d,y=s+c,b=l+d,C=0;C<n.length;C++){var A=n[C],_=Math.cos(A.theta)*A.width/2,I=Math.sin(A.theta)*A.width/2,x=2*i.segmentDistance(g,v,y,b,A.x-_,A.y-I,A.x+_,A.y+I)/(t.height+A.height),w=A.level===t.level,M=w?m.SAMELEVELDISTANCE:1;if(x<=M)return 1/0;p+=m.NEIGHBORCOST*(w?m.SAMELEVELFACTOR:1)/(x-M)}return p}function C(e){var t,n,r=e.trace._emptypoints,i=[],o=e.z.length,a=e.z[0].length,s=[];for(t=0;t<a;t++)s.push(1);for(t=0;t<o;t++)i.push(s.slice());for(t=0;t<r.length;t++)i[(n=r[t])[0]][n[1]]=0;return e.zmask=i,i}t.plot=function(e,n,a,s){var l=n.xaxis,u=n.yaxis;i.makeTraceGroups(s,a,"contour").each((function(a){var s=r.select(this),m=a[0],b=m.trace,A=m.x,_=m.y,I=b.contours,x=f(I,n,m),w=i.ensureSingle(s,"g","heatmapcoloring"),M=[];"heatmap"===I.coloring&&(M=[a]),c(e,n,M,w),d(x),h(x);var S=l.c2p(A[0],!0),T=l.c2p(A[A.length-1],!0),E=u.c2p(_[0],!0),O=u.c2p(_[_.length-1],!0),k=[[S,O],[T,O],[T,E],[S,E]],L=x;"constraint"===I.type&&(L=p(x,I._operation)),function(e,t,n){var r=i.ensureSingle(e,"g","contourbg").selectAll("path").data("fill"===n.coloring?[0]:[]);r.enter().append("path"),r.exit().remove(),r.attr("d","M"+t.join("L")+"Z").style("stroke","none")}(s,k,I),function(e,t,n,o){var a="fill"===o.coloring||"constraint"===o.type&&"="!==o._operation,s="M"+n.join("L")+"Z";a&&g(t,o);var l=i.ensureSingle(e,"g","contourfill").selectAll("path").data(a?t:[]);l.enter().append("path"),l.exit().remove(),l.each((function(e){var t=(e.prefixBoundary?s:"")+y(e,n);t?r.select(this).attr("d",t).style("stroke","none"):r.select(this).remove()}))}(s,L,k,I),function(e,n,a,s,l){var u=i.ensureSingle(e,"g","contourlines"),c=!1!==l.showlines,d=l.showlabels,h=c&&d,f=t.createLines(u,c||d,n),p=t.createLineClip(u,h,a,s.trace.uid),g=e.selectAll("g.contourlabels").data(d?[0]:[]);if(g.exit().remove(),g.enter().append("g").classed("contourlabels",!0),d){var m=[],y=[];i.clearLocationCache();var b=t.labelFormatter(a,s),C=o.tester.append("text").attr("data-notex",1).call(o.font,l.labelfont),A=n[0].xaxis,_=n[0].yaxis,I=A._length,x=_._length,w=A.range,M=_.range,S=i.aggNums(Math.min,null,s.x),T=i.aggNums(Math.max,null,s.x),E=i.aggNums(Math.min,null,s.y),O=i.aggNums(Math.max,null,s.y),k=Math.max(A.c2p(S,!0),0),L=Math.min(A.c2p(T,!0),I),D=Math.max(_.c2p(O,!0),0),P=Math.min(_.c2p(E,!0),x),N={};w[0]<w[1]?(N.left=k,N.right=L):(N.left=L,N.right=k),M[0]<M[1]?(N.top=D,N.bottom=P):(N.top=P,N.bottom=D),N.middle=(N.top+N.bottom)/2,N.center=(N.left+N.right)/2,m.push([[N.left,N.top],[N.right,N.top],[N.right,N.bottom],[N.left,N.bottom]]);var j=Math.sqrt(I*I+x*x),R=v.LABELDISTANCE*j/Math.max(1,n.length/v.LABELINCREASE);f.each((function(e){var n=t.calcTextOpts(e.level,b,C,a);r.select(this).selectAll("path").each((function(){var e=i.getVisibleSegment(this,N,n.height/2);if(e&&!(e.len<(n.width+n.height)*v.LABELMIN))for(var r=Math.min(Math.ceil(e.len/R),v.LABELMAX),o=0;o<r;o++){var a=t.findBestTextLocation(this,e,n,y,N);if(!a)break;t.addLabelData(a,n,y,m)}}))})),C.remove(),t.drawLabels(g,y,a,p,h?m:null)}d&&!c&&f.remove()}(s,x,e,m,I),function(e,t,n,r,a){var s=r.trace,l=n._fullLayout._clips,u="clip"+s.uid,c=l.selectAll("#"+u).data(s.connectgaps?[]:[0]);if(c.enter().append("clipPath").classed("contourclip",!0).attr("id",u),c.exit().remove(),!1===s.connectgaps){var f={level:.9,crossings:{},starts:[],edgepaths:[],paths:[],xaxis:t.xaxis,yaxis:t.yaxis,x:r.x,y:r.y,z:C(r),smoothing:0};d([f]),h([f]),g([f],{type:"levels"}),i.ensureSingle(c,"path","").attr("d",(f.prefixBoundary?"M"+a.join("L")+"Z":"")+y(f,a))}else u=null;o.setClipUrl(e,u,n)}(s,n,e,m,k)}))},t.createLines=function(e,t,n){var r=n[0].smoothing,i=e.selectAll("g.contourlevel").data(t?n:[]);if(i.exit().remove(),i.enter().append("g").classed("contourlevel",!0),t){var a=i.selectAll("path.openline").data((function(e){return e.pedgepaths||e.edgepaths}));a.exit().remove(),a.enter().append("path").classed("openline",!0),a.attr("d",(function(e){return o.smoothopen(e,r)})).style("stroke-miterlimit",1).style("vector-effect","non-scaling-stroke");var s=i.selectAll("path.closedline").data((function(e){return e.ppaths||e.paths}));s.exit().remove(),s.enter().append("path").classed("closedline",!0),s.attr("d",(function(e){return o.smoothclosed(e,r)})).style("stroke-miterlimit",1).style("vector-effect","non-scaling-stroke")}return i},t.createLineClip=function(e,t,n,r){var i=t?"clipline"+r:null,a=n._fullLayout._clips.selectAll("#"+i).data(t?[0]:[]);return a.exit().remove(),a.enter().append("clipPath").classed("contourlineclip",!0).attr("id",i),o.setClipUrl(e,i,n),a},t.labelFormatter=function(e,t){var n=e._fullLayout,r=t.trace,i=r.contours,o={type:"linear",_id:"ycontour",showexponent:"all",exponentformat:"B"};if(i.labelformat)o.tickformat=i.labelformat,u(o,n);else{var s=a.extractOpts(r);if(s&&s.colorbar&&s.colorbar._axis)o=s.colorbar._axis;else{if("constraint"===i.type){var c=i.value;Array.isArray(c)?o.range=[c[0],c[c.length-1]]:o.range=[c,c]}else o.range=[i.start,i.end],o.nticks=(i.end-i.start)/i.size;o.range[0]===o.range[1]&&(o.range[1]+=o.range[0]||1),o.nticks||(o.nticks=1e3),u(o,n),l.prepTicks(o),o._tmin=null,o._tmax=null}}return function(e){return l.tickText(o,e).text}},t.calcTextOpts=function(e,t,n,r){var i=t(e);n.text(i).call(s.convertToTspans,r);var a=n.node(),l=o.bBox(a,!0);return{text:i,width:l.width,height:l.height,fontSize:+a.style["font-size"].replace("px",""),level:e,dy:(l.top+l.bottom)/2}},t.findBestTextLocation=function(e,t,n,r,o){var a,s,l,u,c,d=n.width;t.isClosed?(s=t.len/m.INITIALSEARCHPOINTS,a=t.min+s/2,l=t.max):(s=(t.len-d)/(m.INITIALSEARCHPOINTS+1),a=t.min+s+d/2,l=t.max-(s+d)/2);for(var h=1/0,f=0;f<m.ITERATIONS;f++){for(var p=a;p<l;p+=s){var g=i.getTextLocation(e,t.total,p,d),v=b(g,n,r,o);v<h&&(h=v,c=g,u=p)}if(h>2*m.MAXCOST)break;f&&(s/=2),l=(a=u-s/2)+1.5*s}if(h<=m.MAXCOST)return c},t.addLabelData=function(e,t,n,r){var i=t.fontSize,o=t.width+i/3,a=Math.max(0,t.height-i/3),s=e.x,l=e.y,u=e.theta,c=Math.sin(u),d=Math.cos(u),h=function(e,t){return[s+e*d-t*c,l+e*c+t*d]},f=[h(-o/2,-a/2),h(-o/2,a/2),h(o/2,a/2),h(o/2,-a/2)];n.push({text:t.text,x:s,y:l,dy:t.dy,theta:u,level:t.level,width:o,height:a}),r.push(f)},t.drawLabels=function(e,t,n,o,a){var l=e.selectAll("text").data(t,(function(e){return e.text+","+e.x+","+e.y+","+e.theta}));if(l.exit().remove(),l.enter().append("text").attr({"data-notex":1,"text-anchor":"middle"}).each((function(e){var t=e.x+Math.sin(e.theta)*e.dy,i=e.y-Math.cos(e.theta)*e.dy;r.select(this).text(e.text).attr({x:t,y:i,transform:"rotate("+180*e.theta/Math.PI+" "+t+" "+i+")"}).call(s.convertToTspans,n)})),a){for(var u="",c=0;c<a.length;c++)u+="M"+a[c].join("L")+"Z";i.ensureSingle(o,"path","").attr("d",u)}}},function(e,t,n){"use strict";var r=n(39),i=n(123),o=n(27),a=n(10),s=n(195).makeColorScaleFuncFromTrace,l=n(196);function u(e,t){var n=t.length-2,r=a.constrain(a.findBin(e,t),0,n),i=t[r],o=t[r+1],s=a.constrain(r+(e-i)/(o-i)-.5,0,n),l=Math.round(s),u=Math.abs(s-l);return s&&s!==n&&u?{bin0:l,frac:u,bin1:Math.round(l+u/(s-l))}:{bin0:l,bin1:l,frac:0}}function c(e,t){var n=t.length-1,r=a.constrain(a.findBin(e,t),0,n),i=t[r],o=(e-i)/(t[r+1]-i)||0;return o<=0?{bin0:r,bin1:r,frac:0}:o<.5?{bin0:r,bin1:r+1,frac:o}:{bin0:r+1,bin1:r,frac:1-o}}function d(e,t,n){e[t]=n[0],e[t+1]=n[1],e[t+2]=n[2],e[t+3]=Math.round(255*n[3])}e.exports=function(e,t,n,h){var f=t.xaxis,p=t.yaxis;a.makeTraceGroups(h,n,"hm").each((function(t){var n,h,g,v,m,y,b=r.select(this),C=t[0],A=C.trace,_=C.z,I=C.x,x=C.y,w=C.xCenter,M=C.yCenter,S=o.traceIs(A,"contour"),T=S?"best":A.zsmooth,E=_.length,O=a.maxRowLength(_),k=!1,L=!1;for(y=0;void 0===n&&y<I.length-1;)n=f.c2p(I[y]),y++;for(y=I.length-1;void 0===h&&y>0;)h=f.c2p(I[y]),y--;for(h<n&&(g=h,h=n,n=g,k=!0),y=0;void 0===v&&y<x.length-1;)v=p.c2p(x[y]),y++;for(y=x.length-1;void 0===m&&y>0;)m=p.c2p(x[y]),y--;if(m<v&&(g=v,v=m,m=g,L=!0),S&&(w=I,M=x,I=C.xfill,x=C.yfill),"fast"!==T){var D="best"===T?0:.5;n=Math.max(-D*f._length,n),h=Math.min((1+D)*f._length,h),v=Math.max(-D*p._length,v),m=Math.min((1+D)*p._length,m)}var P=Math.round(h-n),N=Math.round(m-v);if(P<=0||N<=0){b.selectAll("image").data([]).exit().remove()}else{var j,R;"fast"===T?(j=O,R=E):(j=P,R=N);var z=document.createElement("canvas");z.width=j,z.height=R;var F,B,H=z.getContext("2d"),U=s(A,{noNumericCheck:!0,returnArray:!0});"fast"===T?(F=k?function(e){return O-1-e}:a.identity,B=L?function(e){return E-1-e}:a.identity):(F=function(e){return a.constrain(Math.round(f.c2p(I[e])-n),0,P)},B=function(e){return a.constrain(Math.round(p.c2p(x[e])-v),0,N)});var V,Y,W,G,X,q=B(0),Z=[q,q],Q=k?0:1,K=L?0:1,J=0,$=0,ee=0,te=0;if(T){var ne,re=0;try{ne=new Uint8Array(P*N*4)}catch(e){ne=new Array(P*N*4)}if("best"===T){var ie,oe,ae,se=w||I,le=M||x,ue=new Array(se.length),ce=new Array(le.length),de=new Array(P),he=w?c:u,fe=M?c:u;for(y=0;y<se.length;y++)ue[y]=Math.round(f.c2p(se[y])-n);for(y=0;y<le.length;y++)ce[y]=Math.round(p.c2p(le[y])-v);for(y=0;y<P;y++)de[y]=he(y,ue);for(Y=0;Y<N;Y++)for(oe=_[(ie=fe(Y,ce)).bin0],ae=_[ie.bin1],y=0;y<P;y++,re+=4)d(ne,re,X=xe(oe,ae,de[y],ie))}else for(Y=0;Y<E;Y++)for(G=_[Y],Z=B(Y),y=0;y<P;y++)X=Ie(G[y],1),d(ne,re=4*(Z*P+F(y)),X);var pe=H.createImageData(P,N);try{pe.data.set(ne)}catch(e){var ge=pe.data,ve=ge.length;for(Y=0;Y<ve;Y++)ge[Y]=ne[Y]}H.putImageData(pe,0,0)}else{var me=A.xgap,ye=A.ygap,be=Math.floor(me/2),Ce=Math.floor(ye/2);for(Y=0;Y<E;Y++)if(G=_[Y],Z.reverse(),Z[K]=B(Y+1),Z[0]!==Z[1]&&void 0!==Z[0]&&void 0!==Z[1])for(V=[W=F(0),W],y=0;y<O;y++)V.reverse(),V[Q]=F(y+1),V[0]!==V[1]&&void 0!==V[0]&&void 0!==V[1]&&(X=Ie(G[y],(V[1]-V[0])*(Z[1]-Z[0])),H.fillStyle="rgba("+X.join(",")+")",H.fillRect(V[0]+be,Z[0]+Ce,V[1]-V[0]-me,Z[1]-Z[0]-ye))}$=Math.round($/J),ee=Math.round(ee/J),te=Math.round(te/J);var Ae=i("rgb("+$+","+ee+","+te+")");e._hmpixcount=(e._hmpixcount||0)+J,e._hmlumcount=(e._hmlumcount||0)+J*Ae.getLuminance();var _e=b.selectAll("image").data(t);_e.enter().append("svg:image").attr({xmlns:l.svg,preserveAspectRatio:"none"}),_e.attr({height:N,width:P,x:n,y:v,"xlink:href":z.toDataURL("image/png")})}function Ie(e,t){if(void 0!==e){var n=U(e);return n[0]=Math.round(n[0]),n[1]=Math.round(n[1]),n[2]=Math.round(n[2]),J+=t,$+=n[0]*t,ee+=n[1]*t,te+=n[2]*t,n}return[0,0,0,0]}function xe(e,t,n,r){var i=e[n.bin0];if(void 0===i)return Ie(void 0,1);var o,a=e[n.bin1],s=t[n.bin0],l=t[n.bin1],u=a-i||0,c=s-i||0;return o=void 0===a?void 0===l?0:void 0===s?2*(l-i):2*(2*l-s-i)/3:void 0===l?void 0===s?0:2*(2*i-a-s)/3:void 0===s?2*(2*l-a-i)/3:l+i-a-s,Ie(i+n.frac*u+r.frac*(c+n.frac*o))}}))}},function(e,t,n){"use strict";var r=n(580),i=n(42);function o(e,t){var n,o=Array.isArray(t);function a(e){return i(e)?+e:null}return-1!==r.COMPARISON_OPS2.indexOf(e)?n=a(o?t[0]:t):-1!==r.INTERVAL_OPS.indexOf(e)?n=o?[a(t[0]),a(t[1])]:[a(t),a(t)]:-1!==r.SET_OPS.indexOf(e)&&(n=o?t.map(a):[a(t)]),n}function a(e){return function(t){t=o(e,t);var n=Math.min(t[0],t[1]),r=Math.max(t[0],t[1]);return{start:n,end:r,size:r-n}}}function s(e){return function(t){return{start:t=o(e,t),end:1/0,size:1/0}}}e.exports={"[]":a("[]"),"][":a("]["),">":s(">"),"<":s("<"),"=":s("=")}},function(e,t,n){"use strict";var r=n(10).isArrayOrTypedArray;e.exports=function(e,t,n,i){var o,a,s,l,u,c,d,h,f,p,g,v,m,y=r(n)?"a":"b",b=("a"===y?e.aaxis:e.baxis).smoothing,C="a"===y?e.a2i:e.b2j,A="a"===y?n:i,_="a"===y?i:n,I="a"===y?t.a.length:t.b.length,x="a"===y?t.b.length:t.a.length,w=Math.floor("a"===y?e.b2j(_):e.a2i(_)),M="a"===y?function(t){return e.evalxy([],t,w)}:function(t){return e.evalxy([],w,t)};b&&(s=Math.max(0,Math.min(x-2,w)),l=w-s,a="a"===y?function(t,n){return e.dxydi([],t,s,n,l)}:function(t,n){return e.dxydj([],s,t,l,n)});var S=C(A[0]),T=C(A[1]),E=S<T?1:-1,O=1e-8*(T-S),k=E>0?Math.floor:Math.ceil,L=E>0?Math.ceil:Math.floor,D=E>0?Math.min:Math.max,P=E>0?Math.max:Math.min,N=k(S+O),j=L(T-O),R=[[d=M(S)]];for(o=N;o*E<j*E;o+=E)u=[],g=P(S,o),m=(v=D(T,o+E))-g,c=Math.max(0,Math.min(I-2,Math.floor(.5*(g+v)))),h=M(v),b&&(f=a(c,g-c),p=a(c,v-c),u.push([d[0]+f[0]/3*m,d[1]+f[1]/3*m]),u.push([h[0]-p[0]/3*m,h[1]-p[1]/3*m])),u.push(h),R.push(u),d=h;return R}},function(e,t,n){"use strict";var r=n(39),i=n(56),o=n(2029),a=n(1088);e.exports=function(e){var t=r.select(e).selectAll("g.contour");t.style("opacity",(function(e){return e[0].trace.opacity})),t.each((function(e){var t=r.select(this),n=e[0].trace,o=n.contours,s=n.line,l=o.size||1,u=o.start,c="constraint"===o.type,d=!c&&"lines"===o.coloring,h=!c&&"fill"===o.coloring,f=d||h?a(n):null;t.selectAll("g.contourlevel").each((function(e){r.select(this).selectAll("path").call(i.lineGroupStyle,s.width,d?f(e.level):s.color,s.dash)}));var p=o.labelfont;if(t.selectAll("g.contourlabels text").each((function(e){i.font(r.select(this),{family:p.family,size:p.size,color:p.color||(d?f(e.level):s.color)})})),c)t.selectAll("g.contourfill path").style("fill",n.fillcolor);else if(h){var g;t.selectAll("g.contourfill path").style("fill",(function(e){return void 0===g&&(g=e.level),f(e.level+.5*l)})),void 0===g&&(g=u),t.selectAll("g.contourbg path").style("fill",f(g-.5*l))}})),o(e)}},function(e,t,n){"use strict";var r=n(39);e.exports=function(e){r.select(e).selectAll(".hm image").style("opacity",(function(e){return e.trace.opacity}))}},function(e,t,n){!function(e){"use strict";var t=e.Pos;function n(e,t){return e.line-t.line||e.ch-t.ch}var r="A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD",i=new RegExp("<(/?)(["+r+"]["+r+"-:.0-9\\u00B7\\u0300-\\u036F\\u203F-\\u2040]*)","g");function o(e,t,n,r){this.line=t,this.ch=n,this.cm=e,this.text=e.getLine(t),this.min=r?Math.max(r.from,e.firstLine()):e.firstLine(),this.max=r?Math.min(r.to-1,e.lastLine()):e.lastLine()}function a(e,n){var r=e.cm.getTokenTypeAt(t(e.line,n));return r&&/\btag\b/.test(r)}function s(e){if(!(e.line>=e.max))return e.ch=0,e.text=e.cm.getLine(++e.line),!0}function l(e){if(!(e.line<=e.min))return e.text=e.cm.getLine(--e.line),e.ch=e.text.length,!0}function u(e){for(;;){var t=e.text.indexOf(">",e.ch);if(-1==t){if(s(e))continue;return}if(a(e,t+1)){var n=e.text.lastIndexOf("/",t),r=n>-1&&!/\S/.test(e.text.slice(n+1,t));return e.ch=t+1,r?"selfClose":"regular"}e.ch=t+1}}function c(e){for(;;){var t=e.ch?e.text.lastIndexOf("<",e.ch-1):-1;if(-1==t){if(l(e))continue;return}if(a(e,t+1)){i.lastIndex=t,e.ch=t;var n=i.exec(e.text);if(n&&n.index==t)return n}else e.ch=t}}function d(e){for(;;){i.lastIndex=e.ch;var t=i.exec(e.text);if(!t){if(s(e))continue;return}if(a(e,t.index+1))return e.ch=t.index+t[0].length,t;e.ch=t.index+1}}function h(e){for(;;){var t=e.ch?e.text.lastIndexOf(">",e.ch-1):-1;if(-1==t){if(l(e))continue;return}if(a(e,t+1)){var n=e.text.lastIndexOf("/",t),r=n>-1&&!/\S/.test(e.text.slice(n+1,t));return e.ch=t+1,r?"selfClose":"regular"}e.ch=t}}function f(e,n){for(var r=[];;){var i,o=d(e),a=e.line,s=e.ch-(o?o[0].length:0);if(!o||!(i=u(e)))return;if("selfClose"!=i)if(o[1]){for(var l=r.length-1;l>=0;--l)if(r[l]==o[2]){r.length=l;break}if(l<0&&(!n||n==o[2]))return{tag:o[2],from:t(a,s),to:t(e.line,e.ch)}}else r.push(o[2])}}function p(e,n){for(var r=[];;){var i=h(e);if(!i)return;if("selfClose"!=i){var o=e.line,a=e.ch,s=c(e);if(!s)return;if(s[1])r.push(s[2]);else{for(var l=r.length-1;l>=0;--l)if(r[l]==s[2]){r.length=l;break}if(l<0&&(!n||n==s[2]))return{tag:s[2],from:t(e.line,e.ch),to:t(o,a)}}}else c(e)}}e.registerHelper("fold","xml",(function(e,r){for(var i=new o(e,r.line,0);;){var a=d(i);if(!a||i.line!=r.line)return;var s=u(i);if(!s)return;if(!a[1]&&"selfClose"!=s){var l=t(i.line,i.ch),c=f(i,a[2]);return c&&n(c.from,l)>0?{from:l,to:c.from}:null}}})),e.findMatchingTag=function(e,r,i){var a=new o(e,r.line,r.ch,i);if(-1!=a.text.indexOf(">")||-1!=a.text.indexOf("<")){var s=u(a),l=s&&t(a.line,a.ch),d=s&&c(a);if(s&&d&&!(n(a,r)>0)){var h={from:t(a.line,a.ch),to:l,tag:d[2]};return"selfClose"==s?{open:h,close:null,at:"open"}:d[1]?{open:p(a,d[2]),close:h,at:"close"}:{open:h,close:f(a=new o(e,l.line,l.ch,i),d[2]),at:"open"}}}},e.findEnclosingTag=function(e,t,n,r){for(var i=new o(e,t.line,t.ch,n);;){var a=p(i,r);if(!a)break;var s=f(new o(e,t.line,t.ch,n),a.tag);if(s)return{open:a,close:s}}},e.scanForClosingTag=function(e,t,n,r){return f(new o(e,t.line,t.ch,r?{from:0,to:r}:null),n)}}(n(122))},function(e,t,n){(e.exports=n(68)(!1)).push([e.i,'.CodeMirror-foldmarker {\n  color: blue;\n  text-shadow: #b9f 1px 1px 2px, #b9f -1px -1px 2px, #b9f 1px -1px 2px, #b9f -1px 1px 2px;\n  font-family: arial;\n  line-height: .3;\n  cursor: pointer;\n}\n.CodeMirror-foldgutter {\n  width: .7em;\n}\n.CodeMirror-foldgutter-open,\n.CodeMirror-foldgutter-folded {\n  cursor: pointer;\n}\n.CodeMirror-foldgutter-open:after {\n  content: "\\25BE";\n}\n.CodeMirror-foldgutter-folded:after {\n  content: "\\25B8";\n}\n',""])},function(e,t,n){(e.exports=n(68)(!1)).push([e.i,".CodeMirror-hints {\n  position: absolute;\n  z-index: 10;\n  overflow: hidden;\n  list-style: none;\n\n  margin: 0;\n  padding: 2px;\n\n  -webkit-box-shadow: 2px 3px 5px rgba(0,0,0,.2);\n  -moz-box-shadow: 2px 3px 5px rgba(0,0,0,.2);\n  box-shadow: 2px 3px 5px rgba(0,0,0,.2);\n  border-radius: 3px;\n  border: 1px solid silver;\n\n  background: white;\n  font-size: 90%;\n  font-family: monospace;\n\n  max-height: 20em;\n  overflow-y: auto;\n}\n\n.CodeMirror-hint {\n  margin: 0;\n  padding: 0 4px;\n  border-radius: 2px;\n  white-space: pre;\n  color: black;\n  cursor: pointer;\n}\n\nli.CodeMirror-hint-active {\n  background: #08f;\n  color: white;\n}\n",""])},function(e,t,n){(e.exports=n(68)(!1)).push([e.i,"/*\n  Name:       material\n  Author:     Mattia Astorino (http://github.com/equinusocio)\n  Website:    https://material-theme.site/\n*/\n\n.cm-s-material.CodeMirror {\n  background-color: #263238;\n  color: #EEFFFF;\n}\n\n.cm-s-material .CodeMirror-gutters {\n  background: #263238;\n  color: #546E7A;\n  border: none;\n}\n\n.cm-s-material .CodeMirror-guttermarker,\n.cm-s-material .CodeMirror-guttermarker-subtle,\n.cm-s-material .CodeMirror-linenumber {\n  color: #546E7A;\n}\n\n.cm-s-material .CodeMirror-cursor {\n  border-left: 1px solid #FFCC00;\n}\n\n.cm-s-material div.CodeMirror-selected {\n  background: rgba(128, 203, 196, 0.2);\n}\n\n.cm-s-material.CodeMirror-focused div.CodeMirror-selected {\n  background: rgba(128, 203, 196, 0.2);\n}\n\n.cm-s-material .CodeMirror-line::selection,\n.cm-s-material .CodeMirror-line>span::selection,\n.cm-s-material .CodeMirror-line>span>span::selection {\n  background: rgba(128, 203, 196, 0.2);\n}\n\n.cm-s-material .CodeMirror-line::-moz-selection,\n.cm-s-material .CodeMirror-line>span::-moz-selection,\n.cm-s-material .CodeMirror-line>span>span::-moz-selection {\n  background: rgba(128, 203, 196, 0.2);\n}\n\n.cm-s-material .CodeMirror-activeline-background {\n  background: rgba(0, 0, 0, 0.5);\n}\n\n.cm-s-material .cm-keyword {\n  color: #C792EA;\n}\n\n.cm-s-material .cm-operator {\n  color: #89DDFF;\n}\n\n.cm-s-material .cm-variable-2 {\n  color: #EEFFFF;\n}\n\n.cm-s-material .cm-variable-3,\n.cm-s-material .cm-type {\n  color: #f07178;\n}\n\n.cm-s-material .cm-builtin {\n  color: #FFCB6B;\n}\n\n.cm-s-material .cm-atom {\n  color: #F78C6C;\n}\n\n.cm-s-material .cm-number {\n  color: #FF5370;\n}\n\n.cm-s-material .cm-def {\n  color: #82AAFF;\n}\n\n.cm-s-material .cm-string {\n  color: #C3E88D;\n}\n\n.cm-s-material .cm-string-2 {\n  color: #f07178;\n}\n\n.cm-s-material .cm-comment {\n  color: #546E7A;\n}\n\n.cm-s-material .cm-variable {\n  color: #f07178;\n}\n\n.cm-s-material .cm-tag {\n  color: #FF5370;\n}\n\n.cm-s-material .cm-meta {\n  color: #FFCB6B;\n}\n\n.cm-s-material .cm-attribute {\n  color: #C792EA;\n}\n\n.cm-s-material .cm-property {\n  color: #C792EA;\n}\n\n.cm-s-material .cm-qualifier {\n  color: #DECB6B;\n}\n\n.cm-s-material .cm-variable-3,\n.cm-s-material .cm-type {\n  color: #DECB6B;\n}\n\n\n.cm-s-material .cm-error {\n  color: rgba(255, 255, 255, 1.0);\n  background-color: #FF5370;\n}\n\n.cm-s-material .CodeMirror-matchingbracket {\n  text-decoration: underline;\n  color: white !important;\n}",""])},function(e,t,n){t=e.exports=n(68)(!1);var r=n(1096),i=r(n(2035)),o=r(n(2036)),a=r(n(2037)),s=r(n(2038)),l=r(n(2039));t.push([e.i,".flexlayout__layout {\n  left: 0;\n  top: 0;\n  right: 0;\n  bottom: 0;\n  position: absolute;\n  overflow: hidden;\n}\n.flexlayout__splitter {\n  background-color: #f7f7f7;\n}\n@media (hover: hover) {\n  .flexlayout__splitter:hover {\n    background-color: #e2e2e2;\n  }\n}\n.flexlayout__splitter_border {\n  z-index: 10;\n}\n.flexlayout__splitter_drag {\n  z-index: 1000;\n  background-color: #e2e2e2;\n}\n.flexlayout__outline_rect {\n  position: absolute;\n  cursor: move;\n  box-sizing: border-box;\n  border: 2px solid red;\n  box-shadow: inset 0 0 60px rgba(0, 0, 0, 0.2);\n  border-radius: 5px;\n  z-index: 1000;\n}\n.flexlayout__outline_rect_edge {\n  cursor: move;\n  border: 2px solid green;\n  box-shadow: inset 0 0 60px rgba(0, 0, 0, 0.2);\n  border-radius: 5px;\n  z-index: 1000;\n  box-sizing: border-box;\n}\n.flexlayout__edge_rect {\n  position: absolute;\n  z-index: 1000;\n  box-shadow: inset 0 0 5px rgba(0, 0, 0, 0.2);\n  background-color: gray;\n}\n.flexlayout__drag_rect {\n  position: absolute;\n  cursor: move;\n  color: black;\n  background-color: #f7f7f7;\n  border: 2px solid #e2e2e2;\n  box-shadow: inset 0 0 60px rgba(0, 0, 0, 0.3);\n  border-radius: 5px;\n  z-index: 1000;\n  box-sizing: border-box;\n  opacity: 0.9;\n  text-align: center;\n  display: flex;\n  justify-content: center;\n  flex-direction: column;\n  overflow: hidden;\n  padding: 10px;\n  word-wrap: break-word;\n  font-size: medium;\n  font-family: Roboto, Arial, sans-serif;\n}\n.flexlayout__tabset {\n  overflow: hidden;\n  background-color: #f7f7f7;\n  box-sizing: border-box;\n  font-size: medium;\n  font-family: Roboto, Arial, sans-serif;\n  background-color: white;\n}\n.flexlayout__tabset_header {\n  position: absolute;\n  display: flex;\n  align-items: center;\n  left: 0;\n  right: 0;\n  padding: 3px 3px 3px 5px;\n  box-sizing: border-box;\n  border-bottom: 1px solid #e9e9e9;\n  color: black;\n  background-color: white;\n}\n.flexlayout__tabset_header_content {\n  flex-grow: 1;\n}\n.flexlayout__tabset_tabbar_outer {\n  box-sizing: border-box;\n  background-color: #f7f7f7;\n  position: absolute;\n  left: 0;\n  right: 0;\n  overflow: hidden;\n  display: flex;\n  background-color: white;\n}\n.flexlayout__tabset_tabbar_outer_top {\n  border-bottom: 1px solid #e9e9e9;\n}\n.flexlayout__tabset_tabbar_outer_bottom {\n  border-top: 1px solid #e9e9e9;\n}\n.flexlayout__tabset_tabbar_inner {\n  position: relative;\n  box-sizing: border-box;\n  display: flex;\n  flex-grow: 1;\n  overflow: hidden;\n}\n.flexlayout__tabset_tabbar_inner_tab_container {\n  display: flex;\n  box-sizing: border-box;\n  position: absolute;\n  top: 0;\n  bottom: 0;\n  width: 10000px;\n}\n.flexlayout__tabset_tabbar_inner_tab_container_top {\n  border-top: 2px solid transparent;\n}\n.flexlayout__tabset_tabbar_inner_tab_container_bottom {\n  border-bottom: 2px solid transparent;\n}\n.flexlayout__tabset-selected {\n  background-color: #f7f7f7;\n}\n.flexlayout__tabset-maximized {\n  background-color: #d4d4d4;\n}\n.flexlayout__tab {\n  overflow: auto;\n  position: absolute;\n  box-sizing: border-box;\n  color: black;\n  background-color: white;\n}\n.flexlayout__tab_button {\n  display: inline-flex;\n  align-items: center;\n  box-sizing: border-box;\n  padding: 3px 8px;\n  margin: 0px 2px;\n  cursor: pointer;\n}\n.flexlayout__tab_button--selected {\n  background-color: #e9e9e9;\n  color: black;\n}\n@media (hover: hover) {\n  .flexlayout__tab_button:hover {\n    background-color: #e9e9e9;\n    color: black;\n  }\n}\n.flexlayout__tab_button--unselected {\n  color: gray;\n}\n.flexlayout__tab_button_leading {\n  display: inline-block;\n}\n.flexlayout__tab_button_content {\n  display: inline-block;\n}\n.flexlayout__tab_button_textbox {\n  border: none;\n  color: green;\n  background-color: #e9e9e9;\n}\n.flexlayout__tab_button_textbox:focus {\n  outline: none;\n}\n.flexlayout__tab_button_trailing {\n  display: inline-block;\n  margin-left: 8px;\n  min-width: 8px;\n  min-height: 8px;\n}\n@media (pointer: coarse) {\n  .flexlayout__tab_button_trailing {\n    min-width: 20px;\n    min-height: 20px;\n  }\n}\n@media (hover: hover) {\n  .flexlayout__tab_button:hover .flexlayout__tab_button_trailing {\n    background: transparent url("+i+") no-repeat center;\n  }\n}\n.flexlayout__tab_button--selected .flexlayout__tab_button_trailing {\n  background: transparent url("+i+") no-repeat center;\n}\n.flexlayout__tab_button_overflow {\n  margin-left: 10px;\n  padding-left: 12px;\n  border: none;\n  color: gray;\n  font-size: inherit;\n  background: transparent url("+o+") no-repeat left;\n}\n.flexlayout__tab_toolbar {\n  display: flex;\n  align-items: center;\n}\n.flexlayout__tab_toolbar_button {\n  min-width: 20px;\n  min-height: 20px;\n  border: none;\n  outline: none;\n}\n.flexlayout__tab_toolbar_button-min {\n  background: transparent url("+a+") no-repeat center;\n}\n.flexlayout__tab_toolbar_button-max {\n  background: transparent url("+s+") no-repeat center;\n}\n.flexlayout__tab_toolbar_button-float {\n  background: transparent url("+l+") no-repeat center;\n}\n.flexlayout__tab_floating {\n  overflow: auto;\n  position: absolute;\n  box-sizing: border-box;\n  color: black;\n  background-color: white;\n  display: flex;\n  justify-content: center;\n  align-items: center;\n}\n.flexlayout__tab_floating_inner {\n  overflow: auto;\n  display: flex;\n  flex-direction: column;\n  justify-content: center;\n  align-items: center;\n}\n.flexlayout__tab_floating_inner div {\n  margin-bottom: 5px;\n  text-align: center;\n}\n.flexlayout__tab_floating_inner div a {\n  color: royalblue;\n}\n.flexlayout__border {\n  box-sizing: border-box;\n  overflow: hidden;\n  display: flex;\n  font-size: medium;\n  font-family: Roboto, Arial, sans-serif;\n  background-color: white;\n}\n.flexlayout__border_top {\n  border-bottom: 1px solid #e9e9e9;\n  align-items: center;\n}\n.flexlayout__border_bottom {\n  border-top: 1px solid #e9e9e9;\n  align-items: center;\n}\n.flexlayout__border_left {\n  border-right: 1px solid #e9e9e9;\n  align-content: center;\n  flex-direction: column;\n}\n.flexlayout__border_right {\n  border-left: 1px solid #e9e9e9;\n  align-content: center;\n  flex-direction: column;\n}\n.flexlayout__border_inner {\n  position: relative;\n  box-sizing: border-box;\n  display: flex;\n  overflow: hidden;\n  flex-grow: 1;\n}\n.flexlayout__border_inner_tab_container {\n  white-space: nowrap;\n  display: flex;\n  box-sizing: border-box;\n  position: absolute;\n  top: 0;\n  bottom: 0;\n  width: 10000px;\n}\n.flexlayout__border_inner_tab_container_right {\n  transform-origin: top left;\n  transform: rotate(90deg);\n}\n.flexlayout__border_inner_tab_container_left {\n  flex-direction: row-reverse;\n  transform-origin: top right;\n  transform: rotate(-90deg);\n}\n.flexlayout__border_button {\n  display: flex;\n  align-items: center;\n  cursor: pointer;\n  padding: 3px 8px;\n  margin: 2px;\n  box-sizing: border-box;\n  white-space: nowrap;\n  background-color: #e9e9e9;\n}\n.flexlayout__border_button--selected {\n  background-color: #e9e9e9;\n  color: black;\n}\n@media (hover: hover) {\n  .flexlayout__border_button:hover {\n    background-color: #e9e9e9;\n    color: black;\n  }\n}\n.flexlayout__border_button--unselected {\n  color: gray;\n}\n.flexlayout__border_button_leading {\n  display: inline;\n}\n.flexlayout__border_button_content {\n  display: inline-block;\n}\n.flexlayout__border_button_trailing {\n  display: inline-block;\n  margin-left: 8px;\n  min-width: 8px;\n  min-height: 8px;\n}\n@media (pointer: coarse) {\n  .flexlayout__border_button_trailing {\n    min-width: 20px;\n    min-height: 20px;\n  }\n}\n@media (hover: hover) {\n  .flexlayout__border_button:hover .flexlayout__border_button_trailing {\n    background: transparent url("+i+") no-repeat center;\n  }\n}\n.flexlayout__border_button--selected .flexlayout__border_button_trailing {\n  background: transparent url("+i+") no-repeat center;\n}\n.flexlayout__border_toolbar {\n  display: flex;\n  align-items: center;\n}\n.flexlayout__border_toolbar_left {\n  flex-direction: column;\n}\n.flexlayout__border_toolbar_right {\n  flex-direction: column;\n}\n.flexlayout__border_toolbar_button {\n  min-width: 20px;\n  min-height: 20px;\n  border: none;\n  outline: none;\n}\n.flexlayout__border_toolbar_button-float {\n  background: transparent url("+l+") no-repeat center;\n}\n.flexlayout__border_toolbar_button_overflow {\n  border: none;\n  padding-left: 12px;\n  color: gray;\n  font-size: inherit;\n  background: transparent url("+o+") no-repeat left;\n}\n.flexlayout__border_toolbar_button_overflow_top, .flexlayout__border_toolbar_button_overflow_bottom {\n  margin-left: 10px;\n}\n.flexlayout__border_toolbar_button_overflow_right, .flexlayout__border_toolbar_button_overflow_left {\n  padding-right: 0px;\n  margin-top: 5px;\n}\n.flexlayout__popup_menu {\n  font-size: medium;\n  font-family: Roboto, Arial, sans-serif;\n}\n.flexlayout__popup_menu_item {\n  padding: 2px 10px 2px 10px;\n  white-space: nowrap;\n}\n@media (hover: hover) {\n  .flexlayout__popup_menu_item:hover {\n    background-color: #d4d4d4;\n  }\n}\n.flexlayout__popup_menu_container {\n  box-shadow: inset 0 0 5px rgba(0, 0, 0, 0.15);\n  border: 1px solid #d4d4d4;\n  color: black;\n  background: white;\n  border-radius: 3px;\n  position: absolute;\n  z-index: 1000;\n  max-height: 50%;\n  min-width: 100px;\n  overflow: auto;\n}\n.flexlayout__floating_window _body {\n  height: 100%;\n}\n.flexlayout__floating_window_content {\n  left: 0;\n  top: 0;\n  right: 0;\n  bottom: 0;\n  position: absolute;\n}\n.flexlayout__floating_window_tab {\n  overflow: auto;\n  left: 0;\n  top: 0;\n  right: 0;\n  bottom: 0;\n  position: absolute;\n  box-sizing: border-box;\n  background-color: white;\n  color: black;\n}\n.flexlayout__error_boundary_container {\n  left: 0;\n  top: 0;\n  right: 0;\n  bottom: 0;\n  position: absolute;\n  display: flex;\n  justify-content: center;\n}\n.flexlayout__error_boundary_content {\n  display: flex;\n  align-items: center;\n}\n.flexlayout__tabset_sizer {\n  padding-top: 5px;\n  padding-bottom: 3px;\n  font-size: medium;\n  font-family: Roboto, Arial, sans-serif;\n}\n.flexlayout__tabset_header_sizer {\n  padding-top: 3px;\n  padding-bottom: 3px;\n  font-size: medium;\n  font-family: Roboto, Arial, sans-serif;\n}\n.flexlayout__border_sizer {\n  padding-top: 6px;\n  padding-bottom: 5px;\n  font-size: medium;\n  font-family: Roboto, Arial, sans-serif;\n}\n",""])},function(e,t){e.exports="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAgAAAAICAYAAADED76LAAADZXpUWHRSYXcgcHJvZmlsZSB0eXBlIGV4aWYAAHja7ZdtduMqDIb/s4q7BCQhJJaDsTlndjDLvy82dtJOM5M2/XM/TGyILEsvPMhtwvbzRw9/4eCYc0hqnkvOEUcqqXDFwONx1P1KMe3X4zjv0Vt7uG4wTIJepn+e9g12hj9Pe5txKux6F6hs88by9kadgdhnglPRTCR0JIgzcKgzkPDMnI7vyzGtmIvb/RTa9O/1bqo+p5bEOGsmS7gmjma5YOwck2Hd1iE0t7hrpHUmePc9nK4MTbwJScQ1DYVynBVnxlVk+JAkjBP6irMcSkGLQzSMy7muvz9+pzyc0ie9t6jPET2w35EO+3zPG/KOUL76D+2kN3u4R7pzu8uc85X5jb3qleIiF058va/e+3bMrqaMKec5qXMq+wh+y1it/amMZjg1+ljs0QqaoyQa9tGKTbagNSrEwNgp0UqVOm1736hBYuKNDT1zY6EWYHTAKNzkgItGnU2KrOLYAA3bQWDlSwvtacuerpEj8UoesHsIwQiPvNTCM069j4Iiin6tFXTxqDSoGOSIQiS4gQj1uai6L/DZ3h+Dq4Cg7svsmGCNy4gA/IvSbXPJDlrgqOiPAiZbZwAsERQoxJCAQMwkSpmiMQcjwkI6AFVIZ0m8AAup8gqRnEQy4KAKkBvPGO2urHyY8SIECJUcxMCmSAWslBT7x5JjD1UVTaqa1dS1aM2SR4XlbHm8UauJJVPLZuZWrAYXT66e3dy9eC1cBG9cLajH4qWUWpG0InLF0xUOtS68yJIWXfJiiy9lqY1Dk5aattyseSutrrzKijpe82qrr2WtG23YSlvadMubbb6VrXZstS49de25W/deQq8XtYn1l/YJajSp8U5qONpFDVazMwSN14kOZiDGiQDcBgGSwDyYRaeUeJAbzGJhVIUyROqAs9IgBoJpI9ZOF7sbOQ14jX4LtwAQ/B3kwkD3BLlfuX1Eba37HzrZCY0yHIsaBdWnlR0fvBQ/7gVxj2F46BP/GMPpzhS+FuPUcbsVXtNx64O/MLd7XeFzeR/34TvWZ/ThNR0f4f9CmPhZ/I+mc+8SXtPxFP4ndMT3+OV1XeFzeR/vuvBqjP+r//nt8K+r/lsfvqzjK/j/QdV/N70Qv0XPf6/6peM/FPw8DX8DQCrUTb/DsaEAAAAGYktHRAD/AP8A/6C9p5MAAAAJcEhZcwAAFiUAABYlAUlSJPAAAAAHdElNRQfkBh4NKRbGzZ/RAAAAGXRFWHRDb21tZW50AENyZWF0ZWQgd2l0aCBHSU1QV4EOFwAAADhJREFUGNOFjUEKAEAIArWPSy/fPQUiQXOSkZLd/QBAEmGMrxSZmcKRRG5XPlk4uCe8nLfuKsvMH50+IRBMM7okAAAAAElFTkSuQmCC"},function(e,t){e.exports="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAoAAAAKCAYAAACNMs+9AAABhWlDQ1BJQ0MgcHJvZmlsZQAAKJF9kT1Iw1AUhU9bpVUqDu0g4pChOlkoKuKoVShChVArtOpg8tI/aNKQpLg4Cq4FB38Wqw4uzro6uAqC4A+Ik6OToouUeF9SaBHjhcf7OO+ew3v3Af5mlalmTwJQNcvIpJJCLr8qBF/RBx8iSCAkMVOfE8U0POvrnnqp7uI8y7vvzxpQCiYDfALxLNMNi3iDeHrT0jnvE0dZWVKIz4nHDbog8SPXZZffOJcc9vPMqJHNzBNHiYVSF8tdzMqGSjxFHFNUjfL9OZcVzluc1Wqdte/JXxguaCvLXKc1ghQWsQQRAmTUUUEVFuK0a6SYyNB50sM/7PhFcsnkqoCRYwE1qJAcP/gf/J6tWZyccJPCSaD3xbY/RoHgLtBq2Pb3sW23ToDAM3Cldfy1JjDzSXqjo8WOgMFt4OK6o8l7wOUOMPSkS4bkSAFa/mIReD+jb8oDkVugf82dW/scpw9AlmaVvgEODoGxEmWve7w71D23f3va8/sBPppyklpvd0AAAAAGYktHRAD/AP8A/6C9p5MAAAAJcEhZcwAACxMAAAsTAQCanBgAAAAHdElNRQfkCQMGDhTecLvqAAAAGXRFWHRDb21tZW50AENyZWF0ZWQgd2l0aCBHSU1QV4EOFwAAAEFJREFUGNNjLCoqYiAGMDEQCUhW2M/AwPAfB+5HVljIwMAwAYtBE6ByKFajK4YrYmBgYGBBM6EQBxtDIYYC2gUPAGmIDd2l9p6tAAAAAElFTkSuQmCC"},function(e,t){e.exports="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABUAAAAUCAYAAABiS3YzAAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAABYlAAAWJQFJUiTwAAAAB3RJTUUH3wsOCAciLIHE4wAAAEdJREFUOMvtkksOADAERGl6b5x8eoBqo6E7b+kzJiBqqmEvaGaINIsIhydFRG8185RQVTD7RgC87yTrdPw4VIvWMzMf0DQ7CzmmFh3I1FWCAAAAAElFTkSuQmCC"},function(e,t){e.exports="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAABYlAAAWJQFJUiTwAAAAB3RJTUUH3wsOCAsDzF2bsQAAAB1pVFh0Q29tbWVudAAAAAAAQ3JlYXRlZCB3aXRoIEdJTVBkLmUHAAAAkUlEQVQ4y+2TMQsDIQyFX0L/sZuDoIuI3P8VDaZTSwdbT7mhw30QyBAeL9FHqooZqgoiwhnojOAKjIv5f8HHbCCEMDyyc462HbbWaNRvOQQAay2YmQCAmRFjXBf8XDXn/O6NMbT9KKP1juPQaQpG5b3XWitEBCKCUgq89/pt/lU/Hfber/02KaXlXN5ZvgU3eAL/NmVaTMFRBwAAAABJRU5ErkJggg=="},function(e,t){e.exports="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAJq3pUWHRSYXcgcHJvZmlsZSB0eXBlIGV4aWYAAHjatVhrlvIwrvyvVcwS/JJlL8fPc2YHd/lTchII0NDQfLdpSDCJLalKUjk0/u+/k/6Dv2BTpMCSYo7R4C/kkF3BSTLbX1mf1oT1uX0J+2/2dpwuPzgMeRz99jXFfXxg3OF6t4+3fZ6CcT5NlMf+Q739oewTubQvsI8fC3m7LWD2iansE3m3r7xbWDe3TMxJzi60/fq5/77CgDfpR/DiIkcrAZ/BGZGYcZ6cCYK4dTU0NrNstH1f4O47HZc62OSGt97gM6iFfnsXvCM+nUeY9HecB3zT8d1SoOXICM7zEdfXf68sp8P0HfIbSC9n9sn4CWla/h4/+DuE4uX447jl6zidIV24nVaO8bLyzfgYd1E4UNP3nD3NOTbvSohwOe5OHa6sM1xXNVrrroiX4M0mabD1lfFKSIkGHnWQrOLVbLYOME0bbLfFTjvWsdkGE4MbTnB0rgHQRhhMACO75hXcoC87nfjsu0+AuYEOXgG/2GLXsnkt12zCwt0mAnssJrO45asXvXPRnJpQ1pp0iRXscpppsEKRs5aMxWVAxM49qLwCfLzu/xRXDwR5hTnBwWKqzgD4K9srufwC2uNCxnFLYCt9nwAhggUMY6wHAiZazzZaI86RWItAJgBUYLrzwVXAYpldh5EueB8BDrIAa+MesetSx24bRiEEEOwjeQE22ReAFQKDPxISOFTYc2DmyMKJM5foo2ZYjBK1ohbxEoQlikiSLIWSTyFxiklSSjmV7LJHxeWMfMwp51wKFi2YueDuggtKqa76GirXWKWmmmtpjppvoXGLTVpquZXuuu/I4x679NRzL8MOUGmEwSMOGWnkUSaoNv0Mk2ecMtPMNMsFtR3Wh9cHqNkdNbeQ0gvlghpGRY4prJYTVsyAmAsWgIsiYD05p5iZZENwipxiZrJDVrCDkazgdKuIAcEwrONpL9hdkWNCGf0nuBGAcP8COVLo3kDuEbefUOtlNTq/ENI01KAaj+ybeY7ak59RkJMGwUuwSFAEXQm4ESFFy59dKgrbaMPrRY7NxD/OquGQ5yzIjqKtEEYUl4q23adH+u2Cd4+3E81k3NWTOz+MOXlyOHK4AdeWFxANn/jxw5G+92tzg95B5B1A6HdPqh1TGlvck7tkx4V7H6hBvknqrQ72uUH6GecziNNNKj2BZ2hfVWZLfUqfM7ap6wcukC8iej5NrkiKOkdyComJsH+Qi2iwKfTafF+3WFje0NDWTajJI9gAlSFlTdJc7ChQCx6DOODXvJymlIufw/o4Dbje0QBXLExbWLYAZ1Dtg7GjLttuzcHXZRAmGsskNPLDqGUSptiMujVJxw+TzgbBGnowZzPmzpRnkbkGhr6LzDUw9FtkZg+85rRPjtJQRwfUyKwe5rUcUIIwe7MMs0YCFWqT8VvmYaKOusSWXi1muY1YIOlL9yiqHZW2jpryGD1yFltTiKNYgU4Tspn9yNwhoeBJw5LN1dwXtKiGcSK8s14DywJhDA9m6xqUGfdgUBeey9WNBNUHlOGOHsVgeuw8GCU+GfSDVoL1EGTNY7eBtZPnjmVDhNsQWnGhhlhYAAYhXUbA9sOUlWpPj8C63OYkPakKHxc30ur2h+JWEc22eeM8slsICtNvK8XK8BBU9MMvv+vQSHesMVLlsoL/dAF6UvVk/BYlSPIyo5k9N6U6FQsmoO6YspE+xMYMz0HWnasA9we26lR5XmenJ8u+KHDIP0YGzdThOMCvKwepRsBeXI9Y4rzCW8cTC+h1b3i/NdC3zfrgAc2NBhsLHjgAS04sAAEOHkC0QCGjRlVkQ+8QWrq5DahWvmMFqJ9Q5FU9OIpQnbzOjdmPZO4GdCvBP/T2G/ZD1J45pBSiew7pVAE+3sz+eLzHv1JcVVrxr2DIJYXuI6e1ClmDelEieDlmUOqrMmu62QhIWtTBmodFaSt9cusQeNCFEI9Qzdi0bJrYDN2wYC87UaQsalhdfQejc8tUoVKV81CxxlbtO2WKBiPPVFdu19GPfjGwnQwWePjCkKBiIxsPRSp2dBR/7Evz4GBHQ/MBvxz895KQEvC/6Wz16j7IbOKPCU6fZPirBKdPMvzx2DpA4oIqTwZf3CgDugdVPmMVExAvVHl4PIIU1kc/j3Cf0F7dmNBgUt8aTOXVYMzWYJgFUznX4VHnyR37slGFM3YlWeLo+owLrQ0Bzw2bmqF4ZXlIgd5BpYCeAitw68irBvjRtTpJW0qFQxIsFbM+rZkNN91agnmwkRmhD+9X7XZKmUvtHlV1Q7RRI+4hC5daoW1ypSU3dEJ0PZuEsQ0ZwwEf3Rphypr0sYpv2JH1lCey3LR2qW46MTWVM8gOWI4NSlcBhbAjJposXqlp0TRxKZiJpbEZhEnjEUT6hjtn6tCH3NGKBxSWF6vwqR+wFMrfd9lk5dJ2KseuMnPZABkCaVNYqQCnBRTAXm+APihN4Miw0ioTD4G5ojELKJaRr6WIIdUtkq9CGQmQiLqjRDAz4M51igMeHpvPCKWuwWZtF5sewrZ4l6XLFlQlrQQbI1QL8iFMU5ibMF33AZBCS0KVpU2dHKLdbBI09YMaqiZVnl7EqfUXcSqrTtCuTstSp2eDls4IJ5JeTNJ4nk1avYY21bMZpb/vZqlRS7b3PRlOJi3Zvpu0DNICSovoPxh0GyHk+mKnrn3mp4ZhMbRQHrli34SDRUbHGT1ix9b7LK5iQy8egPqWPVhRxvNWR/fC+6+6m+6F9499NZkLu5AhWRJyMUN2o4fWWgPKgScUQnQCruhDFgXKgSRhYCnPaSpGw6zEDU4zrxc1rkVt/wwSIK7So4QxLXq/7iw7qk0GxYGoeMkw2eojVsRH4CE2lqi5r1UJ/WlL/UOc6PdHA++JZ9IT7bxDdzCLlGjC2vinxr/s+7dw3aLcbFBWbUAd0InQBMyLzUoXbUkgmwHT0K45evANXGUbUQ+ySa637IRy080my799yPLFfoQ+C+n/60OWy0SfKuqfBTV9+djoxcbvjxynTx5/vQKEyj96Wkcf+AGTISoSNK0+xYUo8eichZvqkoC9SK8oR9WpIIIEZtdKh5ThjLLkkq8QxPpUdokmo6U7CKSq1wZSVRX0mdAttUKmsbVHhAeKyGhniANhg1p1aO8ogioans+EGzEXYbJ9pmMebXrbTLh1zYUc/222Tpth39tFm2Hf20XngH1jF50D9o1d9Ajk3+yiRyD/Zhc9I9indtEzgn1qF70m/vt20Wviv28XvSb++3bROwn5jl30TkK+YxfdAQmp2TP9D283m5Cejt72AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAABYlAAAWJQFJUiTwAAAAB3RJTUUH5AYVCSoGIF/zqwAAABl0RVh0Q29tbWVudABDcmVhdGVkIHdpdGggR0lNUFeBDhcAAABHSURBVDjLY2AYBYMOMOKTbGxs/I9Lrr6+HqteJrq6EJ9rKXIhPq+TbCCyYbhcRbSBpBqG10ByDMMZKeQaRtDLpBo2CkYKAACXTBwQEkssDgAAAABJRU5ErkJggg=="},function(e,t,n){var r,i,o=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),a=this&&this.__setModuleDefault||(Object.create?function(e,t){Object.defineProperty(e,"default",{enumerable:!0,value:t})}:function(e,t){e.default=t}),s=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)"default"!==n&&Object.hasOwnProperty.call(e,n)&&o(t,e,n);return a(t,e),t},l=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};r=[n,t,n(961),n(90),n(75),n(69),n(1116),n(0),n(78),n(2212),n(2213)],void 0===(i=function(e,t,n,r,i,o,a,u,c,d,h){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.GraphPanel=void 0,n=l(n),i=l(i),a=s(a),u=s(u);class f extends u.Component{constructor(e){super(e),this.createNodeFromData=(e,t=[])=>{const n={};for(let r=0;r<e.systemList.length;r++){const i=e.systemList[r];if(t.length>0&&!t.includes(i))continue;const o=e.systemGraphData[i].inPort,a=e.systemGraphData[i].outPort,s=f.createNode(i,o,a);if(null!=e.systemGraphData[i].position){const t=e.systemGraphData[i].position[0],n=e.systemGraphData[i].position[1];s.setPosition(t,n)}n[i]=s}return n},this.creatConnectionFromData=(e,t,n,r=[])=>{const i=[];return e.systemList.forEach(o=>{if(r.length>0&&!r.includes(o))return;e.systemGraphData[o].connections.forEach(e=>{const{startSys:r,endSys:a,startPort:s,endPort:l}=f.createLink(o,e),u=t[r],c=t[a];if(u.getPorts().hasOwnProperty(s)&&c.getPorts().hasOwnProperty(l)){const e=u.getPorts()[s],t=c.getPorts()[l],r=e.link(t,n);i.push(r)}})}),i},this.createDefaultPosition=()=>{let e;if(0===Object.keys(this.props.pbsNodePosition).length){let t={};this.props.pbsData.nodeData.length>0&&(t=JSON.parse(JSON.stringify(this.props.pbsData.nodeData[0])));const r=document.createElement("div"),i=this._divRef.current.clientWidth,o=this._divRef.current.clientHeight,a=new n.default.TreeGraph({container:r,width:i,height:o,fitView:!0,layout:{type:"compactBox",direction:"TB",getHeight:function(){return 16},getWidth:function(){return 16},getVGap:function(){return 40},getHGap:function(){return 40}}});a.data(t),a.layout(!0);const s={__$graph_style$__:{visible:!1,position:[0,0]}};a.getNodes().forEach(e=>{const t=e.get("model");s[t.id]?s[t.id].position=[t.x,t.y]:s[t.id]={visible:!0,position:[t.x,t.y]}}),e=s}else e=Object.assign({},this.props.pbsNodePosition);return e},this.autoZoomGraph=e=>{const t=this.state.engine,n=this.state.model,r=n.getNodes().filter(e=>e.isVisible()).map(e=>e),i=t.getBoundingNodesRect(r,e);if(i){const e=t.getCanvas().getBoundingClientRect(),r={x:e.left,y:e.top},o={x:r.x+n.getOffsetX(),y:r.y+n.getOffsetY()},a=t.getCanvas().clientWidth/i.getWidth(),s=t.getCanvas().clientHeight/i.getHeight(),l=a<s?a:s;this.model.setZoomLevel(100*l);const u={x:o.x+i.getTopLeft().x*l,y:o.y+i.getTopLeft().y*l},c=i.getWidth()*l,d=i.getHeight()*l,h=(e.width-c)/2,f=(e.height-d)/2;this.model.setOffset(this.model.getOffsetX()+r.x-u.x+h,this.model.getOffsetY()+r.y-u.y+f),t.repaintCanvas()}},this.syncPosition=()=>{let e;e=0===Object.keys(this.props.pbsNodePosition).length?this._defaultPosition:this.props.pbsNodePosition;const t=this.state.model,n=[],r=t.getNodes(),i=e.__$graph_style$__.visible;let o=3;i&&(o=1),r.forEach(t=>{const r=t.getOptions().name,i=e[r].position;i?t.setPosition(o*i[0],o*i[1]):(n.push(r),t.setVisible(!1))}),this.setState(e=>Object.assign(Object.assign({},e),{model:t}),()=>{this.autoZoomGraph(50)})},this.engine=a.default(),this.engine.getLinkFactories().registerFactory(new h.AdvancedLinkFactory),this.engine.getNodeFactories().registerFactory(new h.CustomNodeFactory),this.model=new a.DiagramModel;const t=this.engine.getStateMachine().getCurrentState();t instanceof a.DefaultDiagramState&&(t.dragNewLink.config.allowLooseLinks=!1),this.engine.setModel(this.model),this._divRef=u.default.createRef(),this.state={engine:this.engine,model:this.model,nodeData:{},filterMenuToggle:!1,filterMode:0}}static connectionDataGenerator(e){const t=e.getSourcePort(),n=t.getName(),r=t.getParent().getOptions().name,i=e.getTargetPort(),o=i.getName(),a=i.getParent().getOptions().name;let s,l;if(a.includes("."))if(r.includes(".")){const e=r.split("."),t=a.split(".");l=e[0];for(let n=1;n<Math.min(e.length,t.length)&&e[n]===t[n];n++)l=l.concat("."+e[n]);const i=a.replace(l,"")+"."+o,u=r.replace(l,"")+"."+n;s=[i.replace(/^./,""),u.replace(/^./,"")]}else l=r,s=[a.replace(l+".","")+"."+o,n];else l=a,s=[o,r.replace(l+".","")+"."+n];return{sysKey:l,connection:s}}componentDidUpdate(e,t){}static createLink(e,t){const n=t[1],r=t[0];let i,o,a,s;if(n.includes(".")){const t=n.split(".");a=t[t.length-1],i=e+"."+t.slice(0,t.length-1).join(".")}else i=e,a=n;if(r.includes(".")){const t=r.split(".");s=t[t.length-1],o=e+"."+t.slice(0,t.length-1).join(".")}else o=e,s=r;return{startSys:i,endSys:o,startPort:a,endPort:s}}componentDidMount(){const e=this.createNodeFromData(this.props.systemData);Object.values(e).forEach(e=>{this.model.addNode(e)});const t=this.engine.getLinkFactories().getFactory(a.PathFindingLinkFactory.NAME);this.creatConnectionFromData(this.props.systemData,e,t).forEach(e=>{this.model.addLink(e)}),this._defaultPosition=this.createDefaultPosition(),this.setState(()=>({engine:this.engine,model:this.model,nodeData:e}),()=>{setTimeout(this.syncPosition,500)}),this.model.setLocked(!0)}static createNode(e,t,n){const r=new h.CustomNodeModel(e,"rgb(0,192,255)");return t.forEach(e=>{const t=new h.CustomPortModel(!0,e,e);r.addPort(t)}),n.forEach(e=>{const t=new h.CustomPortModel(!1,e,e);r.addPort(t)}),r}render(){return u.default.createElement("div",{className:"cosapp-widget-box"},u.default.createElement("div",{ref:this._divRef,style:{height:"calc(100% - 30px)"}},u.default.createElement(d.BaseCanvasWidget,null,u.default.createElement(o.CanvasWidget,{engine:this.state.engine}))),u.default.createElement("div",{style:{height:"30px",display:"flex",background:"#e0e0e0"}},u.default.createElement(i.default,{onClick:this.syncPosition},"Update position")))}}t.GraphPanel=f,t.default=r.withStyles(e=>({textColor:{color:"rgb(250, 250, 250)",background:"#525354",margin:"0px 5px"}}))(c.connect(e=>(e=>({systemData:e.systemArch.systemGraph,pbsNodePosition:e.systemArch.systemPBS,pbsData:e.systemArch.systemTree}))(e),e=>({}))(f))}.apply(t,r))||(e.exports=i)},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.CanvasEngine=void 0;const r=n(67),i=n(1097),o=n(306),a=n(151),s=n(1100),l=n(1101),u=n(1102),c=n(1103);class d extends o.BaseObserver{constructor(e={}){super(),this.model=null,this.eventBus=new s.ActionEventBus(this),this.stateMachine=new c.StateMachine(this),this.layerFactories=new i.FactoryBank,this.registerFactoryBank(this.layerFactories),this.options=Object.assign({registerDefaultDeleteItemsAction:!0,registerDefaultZoomCanvasAction:!0,repaintDebounceMs:0},e),!0===this.options.registerDefaultZoomCanvasAction&&this.eventBus.registerAction(new l.ZoomCanvasAction),!0===this.options.registerDefaultDeleteItemsAction&&this.eventBus.registerAction(new u.DeleteItemsAction)}getStateMachine(){return this.stateMachine}getRelativeMousePoint(e){const t=this.getRelativePoint(e.clientX,e.clientY);return new a.Point((t.x-this.model.getOffsetX())/(this.model.getZoomLevel()/100),(t.y-this.model.getOffsetY())/(this.model.getZoomLevel()/100))}getRelativePoint(e,t){const n=this.canvas.getBoundingClientRect();return new a.Point(e-n.left,t-n.top)}registerFactoryBank(e){e.registerListener({factoryAdded:e=>{e.factory.setDiagramEngine(this)},factoryRemoved:e=>{e.factory.setDiagramEngine(null)}})}getActionEventBus(){return this.eventBus}getLayerFactories(){return this.layerFactories}getFactoryForLayer(e){return"string"==typeof e?this.layerFactories.getFactory(e):this.layerFactories.getFactory(e.getType())}setModel(e){this.model=e,this.canvas&&requestAnimationFrame(()=>{this.repaintCanvas()})}getModel(){return this.model}repaintCanvas(e){const{repaintDebounceMs:t}=this.options,n=()=>{this.iterateListeners(e=>{e.repaintCanvas&&e.repaintCanvas()})};let i=n;if(t>0&&(i=r.debounce(n,t)),e)return new Promise(e=>{const t=this.registerListener({rendered:()=>{e(),t.deregister()}});i()});i()}setCanvas(e){this.canvas!==e&&(this.canvas=e,e&&this.fireEvent({},"canvasReady"))}getCanvas(){return this.canvas}getMouseElement(e){return null}zoomToFit(){const e=this.canvas.clientWidth/this.canvas.scrollWidth,t=this.canvas.clientHeight/this.canvas.scrollHeight,n=e<t?e:t;this.model.setZoomLevel(this.model.getZoomLevel()*n),this.model.setOffset(0,0),this.repaintCanvas()}}t.CanvasEngine=d},function(e,t,n){var r=n(2043);e.exports=function(e,t,n){for(var i=n?e:e.parentNode;i&&i!==document;){if(r(i,t))return i;i=i.parentNode}}},function(e,t){var n=Element.prototype,r=n.matchesSelector||n.webkitMatchesSelector||n.mozMatchesSelector||n.msMatchesSelector||n.oMatchesSelector;e.exports=function(e,t){if(r)return r.call(e,t);for(var n=e.parentNode.querySelectorAll(t),i=0;i<n.length;++i)if(n[i]==e)return!0;return!1}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.BezierCurve=t.BezierCurvepPoints=void 0;const r=n(402),i=n(591);var o;!function(e){e[e.SOURCE=0]="SOURCE",e[e.SOURCE_CONTROL=1]="SOURCE_CONTROL",e[e.TARGET_CONTROL=2]="TARGET_CONTROL",e[e.TARGET=3]="TARGET"}(o=t.BezierCurvepPoints||(t.BezierCurvepPoints={}));class a extends i.Polygon{constructor(){super([new r.Point(0,0),new r.Point(0,0),new r.Point(0,0),new r.Point(0,0)])}getSVGCurve(){return`M${this.getSource().toSVG()} C${this.getSourceControl().toSVG()}, ${this.getTargetControl().toSVG()}, ${this.getTarget().toSVG()}`}setPoints(e){if(4!==e.length)throw new Error("BezierCurve must have extactly 4 points");super.setPoints(e)}getSource(){return this.points[o.SOURCE]}getSourceControl(){return this.points[o.SOURCE_CONTROL]}getTargetControl(){return this.points[o.TARGET_CONTROL]}getTarget(){return this.points[o.TARGET]}setSource(e){this.points[o.SOURCE]=e}setSourceControl(e){this.points[o.SOURCE_CONTROL]=e}setTargetControl(e){this.points[o.TARGET_CONTROL]=e}setTarget(e){this.points[o.TARGET]=e}}t.BezierCurve=a},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0})},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.CanvasWidget=void 0;const r=n(0),i=n(1108),o=n(137),a=n(1109);var s;!function(e){e.Canvas=o.default.div`
		position: relative;
		cursor: move;
		overflow: hidden;
	`}(s||(s={}));class l extends r.Component{constructor(e){super(e),this.ref=r.createRef(),this.state={action:null,diagramEngineListener:null}}componentWillUnmount(){this.props.engine.deregisterListener(this.canvasListener),this.props.engine.setCanvas(null),document.removeEventListener("keyup",this.keyUp),document.removeEventListener("keydown",this.keyDown)}registerCanvas(){this.props.engine.setCanvas(this.ref.current),this.props.engine.iterateListeners(e=>{e.rendered&&e.rendered()})}componentDidUpdate(){this.registerCanvas()}componentDidMount(){this.canvasListener=this.props.engine.registerListener({repaintCanvas:()=>{this.forceUpdate()}}),this.keyDown=e=>{this.props.engine.getActionEventBus().fireAction({event:e})},this.keyUp=e=>{this.props.engine.getActionEventBus().fireAction({event:e})},document.addEventListener("keyup",this.keyUp),document.addEventListener("keydown",this.keyDown),this.registerCanvas()}render(){const e=this.props.engine.getModel();return r.createElement(s.Canvas,{className:this.props.className,ref:this.ref,onWheel:e=>{this.props.engine.getActionEventBus().fireAction({event:e})},onMouseDown:e=>{this.props.engine.getActionEventBus().fireAction({event:e})},onMouseUp:e=>{this.props.engine.getActionEventBus().fireAction({event:e})},onMouseMove:e=>{this.props.engine.getActionEventBus().fireAction({event:e})}},e.getLayers().map(e=>r.createElement(i.TransformLayerWidget,{layer:e,key:e.getID()},r.createElement(a.SmartLayerWidget,{layer:e,engine:this.props.engine,key:e.getID()}))))}}t.CanvasWidget=l},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.SelectionBoxLayerFactory=void 0;const r=n(0),i=n(1106),o=n(595),a=n(1111);class s extends i.AbstractReactFactory{constructor(){super("selection")}generateModel(e){return new o.SelectionLayerModel}generateReactWidget(e){return r.createElement(a.SelectionBoxWidget,{rect:e.model.box})}}t.SelectionBoxLayerFactory=s},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.PeformanceWidget=void 0;const r=n(0),i=n(67);class o extends r.Component{shouldComponentUpdate(e,t,n){return!this.props.model.performanceTune()||(this.props.model!==e.model||!i.isEqual(this.props.serialized,e.serialized))}render(){return this.props.children()}}t.PeformanceWidget=o},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.DefaultState=void 0;const r=n(404),i=n(182),o=n(1112),a=n(1113),s=n(1115);class l extends r.State{constructor(){super({name:"default"}),this.childStates=[new a.SelectingState],this.registerAction(new i.Action({type:i.InputType.MOUSE_DOWN,fire:e=>{this.engine.getActionEventBus().getModelForEvent(e)?this.transitionWithEvent(new s.MoveItemsState,e):this.transitionWithEvent(new o.DragCanvasState,e)}}))}}t.DefaultState=l},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.DiagramModel=void 0;const r=n(67),i=n(596),o=n(405),a=n(69),s=n(598),l=n(599);class u extends a.CanvasModel{constructor(e={}){super(e),this.addLayer(new l.LinkLayerModel),this.addLayer(new s.NodeLayerModel)}deserialize(e){this.layers=[],super.deserialize(e)}addLayer(e){super.addLayer(e),e instanceof s.NodeLayerModel&&(this.activeNodeLayer=e),e instanceof l.LinkLayerModel&&(this.activeLinkLayer=e)}getLinkLayers(){return r.filter(this.layers,e=>e instanceof l.LinkLayerModel)}getNodeLayers(){return r.filter(this.layers,e=>e instanceof s.NodeLayerModel)}getActiveNodeLayer(){if(!this.activeNodeLayer){const e=this.getNodeLayers();0===e.length?this.addLayer(new s.NodeLayerModel):this.activeNodeLayer=e[0]}return this.activeNodeLayer}getActiveLinkLayer(){if(!this.activeLinkLayer){const e=this.getLinkLayers();0===e.length?this.addLayer(new l.LinkLayerModel):this.activeLinkLayer=e[0]}return this.activeLinkLayer}getNode(e){for(const t of this.getNodeLayers()){const n=t.getModel(e);if(n)return n}}getLink(e){for(const t of this.getLinkLayers()){const n=t.getModel(e);if(n)return n}}addAll(...e){return r.forEach(e,e=>{e instanceof i.LinkModel?this.addLink(e):e instanceof o.NodeModel&&this.addNode(e)}),e}addLink(e){return this.getActiveLinkLayer().addModel(e),this.fireEvent({link:e,isCreated:!0},"linksUpdated"),e}addNode(e){return this.getActiveNodeLayer().addModel(e),this.fireEvent({node:e,isCreated:!0},"nodesUpdated"),e}removeLink(e){r.some(this.getLinkLayers(),t=>t.removeModel(e))&&this.fireEvent({link:e,isCreated:!1},"linksUpdated")}removeNode(e){r.some(this.getNodeLayers(),t=>t.removeModel(e))&&this.fireEvent({node:e,isCreated:!1},"nodesUpdated")}getLinks(){return r.flatMap(this.getLinkLayers(),e=>r.values(e.getModels()))}getNodes(){return r.flatMap(this.getNodeLayers(),e=>r.values(e.getModels()))}}t.DiagramModel=u},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.LabelModel=void 0;const r=n(69);class i extends r.BaseModel{constructor(e){super(Object.assign(Object.assign({},e),{offsetX:e.offsetX||0,offsetY:e.offsetY||0}))}deserialize(e){super.deserialize(e),this.options.offsetX=e.data.offsetX,this.options.offsetY=e.data.offsetY}serialize(){return Object.assign(Object.assign({},super.serialize()),{offsetX:this.options.offsetX,offsetY:this.options.offsetY})}}t.LabelModel=i},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.LabelWidget=void 0;const r=n(0),i=n(137);var o;!function(e){e.Label=i.default.div`
		display: inline-block;
		position: absolute;
	`,e.Foreign=i.default.foreignObject`
		pointer-events: none;
		overflow: visible;
	`}(o||(o={}));class a extends r.Component{constructor(e){super(e),this.findPathAndRelativePositionToRenderLabel=e=>{const t=this.props.label.getParent(),n=t.getRenderedPath().map(e=>e.getTotalLength());let r=n.reduce((e,t)=>e+t,0)*(e/(t.getLabels().length+1)),i=0;for(;i<t.getRenderedPath().length;){if(r-n[i]<0)return{path:t.getRenderedPath()[i],position:r};r-=n[i],i++}},this.calculateLabelPosition=()=>{const e=this.findPathAndRelativePositionToRenderLabel(this.props.index+1);if(!e)return;const{path:t,position:n}=e,r=this.ref.current.offsetWidth,i=this.ref.current.offsetHeight,o=t.getPointAtLength(n),a=o.x-r/2+this.props.label.getOptions().offsetX,s=o.y-i/2+this.props.label.getOptions().offsetY;this.ref.current.style.transform=`translate(${a}px, ${s}px)`},this.ref=r.createRef()}componentDidUpdate(){window.requestAnimationFrame(this.calculateLabelPosition)}componentDidMount(){window.requestAnimationFrame(this.calculateLabelPosition)}render(){const e=this.props.engine.getCanvas();return r.createElement(o.Foreign,{key:this.props.label.getID(),width:e.offsetWidth,height:e.offsetHeight},r.createElement(o.Label,{ref:this.ref},this.props.engine.getFactoryForLabel(this.props.label).generateReactWidget({model:this.props.label})))}}t.LabelWidget=a},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.LinkLayerFactory=void 0;const r=n(0),i=n(69),o=n(599),a=n(1118);class s extends i.AbstractReactFactory{constructor(){super("diagram-links")}generateModel(e){return new o.LinkLayerModel}generateReactWidget(e){return r.createElement(a.LinkLayerWidget,{layer:e.model,engine:this.engine})}}t.LinkLayerFactory=s},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.NodeLayerFactory=void 0;const r=n(0),i=n(69),o=n(598),a=n(1119);class s extends i.AbstractReactFactory{constructor(){super("diagram-nodes")}generateModel(e){return new o.NodeLayerModel}generateReactWidget(e){return r.createElement(a.NodeLayerWidget,{layer:e.model,engine:this.engine})}}t.NodeLayerFactory=s},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.PortWidget=void 0;const r=n(0),i=n(67),o=n(69);class a extends r.Component{constructor(e){super(e),this.ref=r.createRef()}report(){this.props.port.updateCoords(this.props.engine.getPortCoords(this.props.port,this.ref.current))}componentWillUnmount(){this.engineListenerHandle&&this.engineListenerHandle.deregister()}componentDidUpdate(e,t,n){this.props.port.reportedPosition||this.report()}componentDidMount(){this.engineListenerHandle=this.props.engine.registerListener({canvasReady:()=>{this.report()}}),this.props.engine.getCanvas()&&this.report()}getExtraProps(){if(o.Toolkit.TESTING){return{"data-links":i.keys(this.props.port.getNode().getPort(this.props.port.getName()).links).join(",")}}return{}}render(){return r.createElement("div",Object.assign({style:this.props.style,ref:this.ref,className:"port "+(this.props.className||""),"data-name":this.props.port.getName(),"data-nodeid":this.props.port.getNode().getID()},this.getExtraProps()),this.props.children)}}t.PortWidget=a},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.DefaultDiagramState=void 0;const r=n(69),i=n(406),o=n(1121),a=n(1122);class s extends r.State{constructor(){super({name:"default-diagrams"}),this.childStates=[new r.SelectingState],this.dragCanvas=new r.DragCanvasState,this.dragNewLink=new o.DragNewLinkState,this.dragItems=new a.DragDiagramItemsState,this.registerAction(new r.Action({type:r.InputType.MOUSE_DOWN,fire:e=>{const t=this.engine.getActionEventBus().getModelForEvent(e);t?t instanceof i.PortModel?this.transitionWithEvent(this.dragNewLink,e):this.transitionWithEvent(this.dragItems,e):this.transitionWithEvent(this.dragCanvas,e)}}))}}t.DefaultDiagramState=s},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.DiagramEngine=void 0;const r=n(405),i=n(151),o=n(69);class a extends o.CanvasEngine{constructor(e={}){super(e),this.maxNumberPointsPerLink=1e3,this.nodeFactories=new o.FactoryBank,this.linkFactories=new o.FactoryBank,this.portFactories=new o.FactoryBank,this.labelFactories=new o.FactoryBank;const t=e=>{e.registerListener({factoryAdded:e=>{e.factory.setDiagramEngine(this)},factoryRemoved:e=>{e.factory.setDiagramEngine(null)}})};t(this.nodeFactories),t(this.linkFactories),t(this.portFactories),t(this.labelFactories)}getMouseElement(e){var t=e.target,n=this.model,r=o.Toolkit.closest(t,".port[data-name]");if(r){var i=o.Toolkit.closest(t,".node[data-nodeid]");return n.getNode(i.getAttribute("data-nodeid")).getPort(r.getAttribute("data-name"))}return(r=o.Toolkit.closest(t,".point[data-id]"))?n.getLink(r.getAttribute("data-linkid")).getPointModel(r.getAttribute("data-id")):(r=o.Toolkit.closest(t,"[data-linkid]"))?n.getLink(r.getAttribute("data-linkid")):(r=o.Toolkit.closest(t,".node[data-nodeid]"))?n.getNode(r.getAttribute("data-nodeid")):null}
//!-------------- FACTORIES ------------
getNodeFactories(){return this.nodeFactories}getLinkFactories(){return this.linkFactories}getLabelFactories(){return this.labelFactories}getPortFactories(){return this.portFactories}getFactoryForNode(e){return"string"==typeof e?this.nodeFactories.getFactory(e):this.nodeFactories.getFactory(e.getType())}getFactoryForLink(e){return"string"==typeof e?this.linkFactories.getFactory(e):this.linkFactories.getFactory(e.getType())}getFactoryForLabel(e){return"string"==typeof e?this.labelFactories.getFactory(e):this.labelFactories.getFactory(e.getType())}getFactoryForPort(e){return"string"==typeof e?this.portFactories.getFactory(e):this.portFactories.getFactory(e.getType())}generateWidgetForLink(e){return this.getFactoryForLink(e).generateReactWidget({model:e})}generateWidgetForNode(e){return this.getFactoryForNode(e).generateReactWidget({model:e})}getNodeElement(e){const t=this.canvas.querySelector(`.node[data-nodeid="${e.getID()}"]`);if(null===t)throw new Error("Cannot find Node element with nodeID: ["+e.getID()+"]");return t}getNodePortElement(e){var t=this.canvas.querySelector(`.port[data-name="${e.getName()}"][data-nodeid="${e.getParent().getID()}"]`);if(null===t)throw new Error("Cannot find Node Port element with nodeID: ["+e.getParent().getID()+"] and name: ["+e.getName()+"]");return t}getPortCenter(e){return this.getPortCoords(e).getOrigin()}getPortCoords(e,t){if(!this.canvas)throw new Error("Canvas needs to be set first");t||(t=this.getNodePortElement(e));const n=t.getBoundingClientRect(),r=this.getRelativeMousePoint({clientX:n.left,clientY:n.top}),o=this.model.getZoomLevel()/100;return new i.Rectangle(r.x,r.y,n.width/o,n.height/o)}getNodeDimensions(e){if(!this.canvas)return{width:0,height:0};const t=this.getNodeElement(e).getBoundingClientRect();return{width:t.width,height:t.height}}getBoundingNodesRect(e,t){if(e){if(0===e.length)return new i.Rectangle(0,0,0,0);let n=i.Polygon.boundingBoxFromPolygons(e.map(e=>e.getBoundingBox()));return t?new i.Rectangle(n.getTopLeft().x-t,n.getTopLeft().y-t,n.getWidth()+2*t,n.getHeight()+2*t):n}}zoomToFitNodes(e){let t,n=this.model.getSelectedEntities().filter(e=>e instanceof r.NodeModel).map(e=>e);if(0==n.length){let n=this.model.getSelectionEntities().filter(e=>e instanceof r.NodeModel).map(e=>e);t=this.getBoundingNodesRect(n,e)}else t=this.getBoundingNodesRect(n,e);if(t){let e=this.canvas.getBoundingClientRect(),n={x:e.left,y:e.top},r={x:n.x+this.getModel().getOffsetX(),y:n.y+this.getModel().getOffsetY()};const i=this.canvas.clientWidth/t.getWidth(),o=this.canvas.clientHeight/t.getHeight(),a=i<o?i:o;this.model.setZoomLevel(100*a);let s={x:r.x+t.getTopLeft().x*a,y:r.y+t.getTopLeft().y*a};this.model.setOffset(this.model.getOffsetX()+n.x-s.x,this.model.getOffsetY()+n.y-s.y),this.repaintCanvas()}}getMaxNumberPointsPerLink(){return this.maxNumberPointsPerLink}setMaxNumberPointsPerLink(e){this.maxNumberPointsPerLink=e}}t.DiagramEngine=a},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.DefaultLabelFactory=void 0;const r=n(0),i=n(600),o=n(1123),a=n(69);class s extends a.AbstractReactFactory{constructor(){super("default")}generateReactWidget(e){return r.createElement(o.DefaultLabelWidget,{model:e.model})}generateModel(e){return new i.DefaultLabelModel}}t.DefaultLabelFactory=s},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.DefaultLinkFactory=void 0;const r=n(0),i=n(601),o=n(1124),a=n(137),s=n(333),l=n(69);var u;!function(e){e.Keyframes=s.keyframes`
		from {
			stroke-dashoffset: 24;
		}
		to {
			stroke-dashoffset: 0;
		}
	`;const t=s.css`
		stroke-dasharray: 10, 2;
		animation: ${e.Keyframes} 1s linear infinite;
	`;e.Path=a.default.path`
		${e=>e.selected&&t};
		fill: none;
		pointer-events: all;
	`}(u||(u={}));class c extends l.AbstractReactFactory{constructor(e="default"){super(e)}generateReactWidget(e){return r.createElement(o.DefaultLinkWidget,{link:e.model,diagramEngine:this.engine})}generateModel(e){return new i.DefaultLinkModel}generateLinkSegment(e,t,n){return r.createElement(u.Path,{selected:t,stroke:t?e.getOptions().selectedColor:e.getOptions().color,strokeWidth:e.getOptions().width,d:n})}}t.DefaultLinkFactory=c},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.DefaultNodeFactory=void 0;const r=n(0),i=n(1127),o=n(1128),a=n(69);class s extends a.AbstractReactFactory{constructor(){super("default")}generateReactWidget(e){return r.createElement(o.DefaultNodeWidget,{engine:this.engine,node:e.model})}generateModel(e){return new i.DefaultNodeModel}}t.DefaultNodeFactory=s},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.DefaultPortFactory=void 0;const r=n(602),i=n(69);class o extends i.AbstractModelFactory{constructor(){super("default")}generateModel(){return new r.DefaultPortModel({name:"unknown"})}}t.DefaultPortFactory=o},function(e,t,n){e.exports=n(2063)},function(e,t,n){e.exports={Heap:n(407),Node:n(603),Grid:n(2065),Util:n(219),DiagonalMovement:n(126),Heuristic:n(307),AStarFinder:n(604),BestFirstFinder:n(2066),BreadthFirstFinder:n(2067),DijkstraFinder:n(2068),BiAStarFinder:n(605),BiBestFirstFinder:n(2069),BiBreadthFirstFinder:n(2070),BiDijkstraFinder:n(2071),IDAStarFinder:n(2072),JumpPointFinder:n(2073)}},function(e,t,n){(function(e){(function(){var t,n,r,i,o,a,s,l,u,c,d,h,f,p,g;r=Math.floor,c=Math.min,n=function(e,t){return e<t?-1:e>t?1:0},u=function(e,t,i,o,a){var s;if(null==i&&(i=0),null==a&&(a=n),i<0)throw new Error("lo must be non-negative");for(null==o&&(o=e.length);i<o;)a(t,e[s=r((i+o)/2)])<0?o=s:i=s+1;return[].splice.apply(e,[i,i-i].concat(t)),t},a=function(e,t,r){return null==r&&(r=n),e.push(t),p(e,0,e.length-1,r)},o=function(e,t){var r,i;return null==t&&(t=n),r=e.pop(),e.length?(i=e[0],e[0]=r,g(e,0,t)):i=r,i},l=function(e,t,r){var i;return null==r&&(r=n),i=e[0],e[0]=t,g(e,0,r),i},s=function(e,t,r){var i;return null==r&&(r=n),e.length&&r(e[0],t)<0&&(t=(i=[e[0],t])[0],e[0]=i[1],g(e,0,r)),t},i=function(e,t){var i,o,a,s,l,u;for(null==t&&(t=n),l=[],o=0,a=(s=function(){u=[];for(var t=0,n=r(e.length/2);0<=n?t<n:t>n;0<=n?t++:t--)u.push(t);return u}.apply(this).reverse()).length;o<a;o++)i=s[o],l.push(g(e,i,t));return l},f=function(e,t,r){var i;if(null==r&&(r=n),-1!==(i=e.indexOf(t)))return p(e,0,i,r),g(e,i,r)},d=function(e,t,r){var o,a,l,u,c;if(null==r&&(r=n),!(a=e.slice(0,t)).length)return a;for(i(a,r),l=0,u=(c=e.slice(t)).length;l<u;l++)o=c[l],s(a,o,r);return a.sort(r).reverse()},h=function(e,t,r){var a,s,l,d,h,f,p,g,v;if(null==r&&(r=n),10*t<=e.length){if(!(l=e.slice(0,t).sort(r)).length)return l;for(s=l[l.length-1],d=0,f=(p=e.slice(t)).length;d<f;d++)r(a=p[d],s)<0&&(u(l,a,0,null,r),l.pop(),s=l[l.length-1]);return l}for(i(e,r),v=[],h=0,g=c(t,e.length);0<=g?h<g:h>g;0<=g?++h:--h)v.push(o(e,r));return v},p=function(e,t,r,i){var o,a,s;for(null==i&&(i=n),o=e[r];r>t&&i(o,a=e[s=r-1>>1])<0;)e[r]=a,r=s;return e[r]=o},g=function(e,t,r){var i,o,a,s,l;for(null==r&&(r=n),o=e.length,l=t,a=e[t],i=2*t+1;i<o;)(s=i+1)<o&&!(r(e[i],e[s])<0)&&(i=s),e[t]=e[i],i=2*(t=i)+1;return e[t]=a,p(e,l,t,r)},t=function(){function e(e){this.cmp=null!=e?e:n,this.nodes=[]}return e.push=a,e.pop=o,e.replace=l,e.pushpop=s,e.heapify=i,e.updateItem=f,e.nlargest=d,e.nsmallest=h,e.prototype.push=function(e){return a(this.nodes,e,this.cmp)},e.prototype.pop=function(){return o(this.nodes,this.cmp)},e.prototype.peek=function(){return this.nodes[0]},e.prototype.contains=function(e){return-1!==this.nodes.indexOf(e)},e.prototype.replace=function(e){return l(this.nodes,e,this.cmp)},e.prototype.pushpop=function(e){return s(this.nodes,e,this.cmp)},e.prototype.heapify=function(){return i(this.nodes,this.cmp)},e.prototype.updateItem=function(e){return f(this.nodes,e,this.cmp)},e.prototype.clear=function(){return this.nodes=[]},e.prototype.empty=function(){return 0===this.nodes.length},e.prototype.size=function(){return this.nodes.length},e.prototype.clone=function(){var t;return(t=new e).nodes=this.nodes.slice(0),t},e.prototype.toArray=function(){return this.nodes.slice(0)},e.prototype.insert=e.prototype.push,e.prototype.top=e.prototype.peek,e.prototype.front=e.prototype.peek,e.prototype.has=e.prototype.contains,e.prototype.copy=e.prototype.clone,e}(),(null!==e?e.exports:void 0)?e.exports=t:window.Heap=t}).call(this)}).call(this,n(188)(e))},function(e,t,n){var r=n(603),i=n(126);function o(e,t,n){var r;"object"!=typeof e?r=e:(t=e.length,r=e[0].length,n=e),this.width=r,this.height=t,this.nodes=this._buildNodes(r,t,n)}o.prototype._buildNodes=function(e,t,n){var i,o,a=new Array(t);for(i=0;i<t;++i)for(a[i]=new Array(e),o=0;o<e;++o)a[i][o]=new r(o,i);if(void 0===n)return a;if(n.length!==t||n[0].length!==e)throw new Error("Matrix size does not fit");for(i=0;i<t;++i)for(o=0;o<e;++o)n[i][o]&&(a[i][o].walkable=!1);return a},o.prototype.getNodeAt=function(e,t){return this.nodes[t][e]},o.prototype.isWalkableAt=function(e,t){return this.isInside(e,t)&&this.nodes[t][e].walkable},o.prototype.isInside=function(e,t){return e>=0&&e<this.width&&t>=0&&t<this.height},o.prototype.setWalkableAt=function(e,t,n){this.nodes[t][e].walkable=n},o.prototype.getNeighbors=function(e,t){var n=e.x,r=e.y,o=[],a=!1,s=!1,l=!1,u=!1,c=!1,d=!1,h=!1,f=!1,p=this.nodes;if(this.isWalkableAt(n,r-1)&&(o.push(p[r-1][n]),a=!0),this.isWalkableAt(n+1,r)&&(o.push(p[r][n+1]),l=!0),this.isWalkableAt(n,r+1)&&(o.push(p[r+1][n]),c=!0),this.isWalkableAt(n-1,r)&&(o.push(p[r][n-1]),h=!0),t===i.Never)return o;if(t===i.OnlyWhenNoObstacles)s=h&&a,u=a&&l,d=l&&c,f=c&&h;else if(t===i.IfAtMostOneObstacle)s=h||a,u=a||l,d=l||c,f=c||h;else{if(t!==i.Always)throw new Error("Incorrect value of diagonalMovement");s=!0,u=!0,d=!0,f=!0}return s&&this.isWalkableAt(n-1,r-1)&&o.push(p[r-1][n-1]),u&&this.isWalkableAt(n+1,r-1)&&o.push(p[r-1][n+1]),d&&this.isWalkableAt(n+1,r+1)&&o.push(p[r+1][n+1]),f&&this.isWalkableAt(n-1,r+1)&&o.push(p[r+1][n-1]),o},o.prototype.clone=function(){var e,t,n=this.width,i=this.height,a=this.nodes,s=new o(n,i),l=new Array(i);for(e=0;e<i;++e)for(l[e]=new Array(n),t=0;t<n;++t)l[e][t]=new r(t,e,a[e][t].walkable);return s.nodes=l,s},e.exports=o},function(e,t,n){var r=n(604);function i(e){r.call(this,e);var t=this.heuristic;this.heuristic=function(e,n){return 1e6*t(e,n)}}i.prototype=new r,i.prototype.constructor=i,e.exports=i},function(e,t,n){var r=n(219),i=n(126);function o(e){e=e||{},this.allowDiagonal=e.allowDiagonal,this.dontCrossCorners=e.dontCrossCorners,this.diagonalMovement=e.diagonalMovement,this.diagonalMovement||(this.allowDiagonal?this.dontCrossCorners?this.diagonalMovement=i.OnlyWhenNoObstacles:this.diagonalMovement=i.IfAtMostOneObstacle:this.diagonalMovement=i.Never)}o.prototype.findPath=function(e,t,n,i,o){var a,s,l,u,c,d=[],h=this.diagonalMovement,f=o.getNodeAt(e,t),p=o.getNodeAt(n,i);for(d.push(f),f.opened=!0;d.length;){if((l=d.shift()).closed=!0,l===p)return r.backtrace(p);for(u=0,c=(a=o.getNeighbors(l,h)).length;u<c;++u)(s=a[u]).closed||s.opened||(d.push(s),s.opened=!0,s.parent=l)}return[]},e.exports=o},function(e,t,n){var r=n(604);function i(e){r.call(this,e),this.heuristic=function(e,t){return 0}}i.prototype=new r,i.prototype.constructor=i,e.exports=i},function(e,t,n){var r=n(605);function i(e){r.call(this,e);var t=this.heuristic;this.heuristic=function(e,n){return 1e6*t(e,n)}}i.prototype=new r,i.prototype.constructor=i,e.exports=i},function(e,t,n){var r=n(219),i=n(126);function o(e){e=e||{},this.allowDiagonal=e.allowDiagonal,this.dontCrossCorners=e.dontCrossCorners,this.diagonalMovement=e.diagonalMovement,this.diagonalMovement||(this.allowDiagonal?this.dontCrossCorners?this.diagonalMovement=i.OnlyWhenNoObstacles:this.diagonalMovement=i.IfAtMostOneObstacle:this.diagonalMovement=i.Never)}o.prototype.findPath=function(e,t,n,i,o){var a,s,l,u,c,d=o.getNodeAt(e,t),h=o.getNodeAt(n,i),f=[],p=[],g=this.diagonalMovement;for(f.push(d),d.opened=!0,d.by=0,p.push(h),h.opened=!0,h.by=1;f.length&&p.length;){for((l=f.shift()).closed=!0,u=0,c=(a=o.getNeighbors(l,g)).length;u<c;++u)if(!(s=a[u]).closed)if(s.opened){if(1===s.by)return r.biBacktrace(l,s)}else f.push(s),s.parent=l,s.opened=!0,s.by=0;for((l=p.shift()).closed=!0,u=0,c=(a=o.getNeighbors(l,g)).length;u<c;++u)if(!(s=a[u]).closed)if(s.opened){if(0===s.by)return r.biBacktrace(s,l)}else p.push(s),s.parent=l,s.opened=!0,s.by=1}return[]},e.exports=o},function(e,t,n){var r=n(605);function i(e){r.call(this,e),this.heuristic=function(e,t){return 0}}i.prototype=new r,i.prototype.constructor=i,e.exports=i},function(e,t,n){n(219);var r=n(307),i=n(603),o=n(126);function a(e){e=e||{},this.allowDiagonal=e.allowDiagonal,this.dontCrossCorners=e.dontCrossCorners,this.diagonalMovement=e.diagonalMovement,this.heuristic=e.heuristic||r.manhattan,this.weight=e.weight||1,this.trackRecursion=e.trackRecursion||!1,this.timeLimit=e.timeLimit||1/0,this.diagonalMovement||(this.allowDiagonal?this.dontCrossCorners?this.diagonalMovement=o.OnlyWhenNoObstacles:this.diagonalMovement=o.IfAtMostOneObstacle:this.diagonalMovement=o.Never),this.diagonalMovement===o.Never?this.heuristic=e.heuristic||r.manhattan:this.heuristic=e.heuristic||r.octile}a.prototype.findPath=function(e,t,n,r,o){var a,s,l,u=(new Date).getTime(),c=function(e,t){return this.heuristic(Math.abs(t.x-e.x),Math.abs(t.y-e.y))}.bind(this),d=function(e,t,n,r,a){if(this.timeLimit>0&&(new Date).getTime()-u>1e3*this.timeLimit)return 1/0;var s,l,h,p,g=t+c(e,f)*this.weight;if(g>n)return g;if(e==f)return r[a]=[e.x,e.y],e;var v,m,y=o.getNeighbors(e,this.diagonalMovement);for(h=0,s=1/0;p=y[h];++h){if(this.trackRecursion&&(p.retainCount=p.retainCount+1||1,!0!==p.tested&&(p.tested=!0)),(l=d(p,t+(m=p,(v=e).x===m.x||v.y===m.y?1:Math.SQRT2),n,r,a+1))instanceof i)return r[a]=[e.x,e.y],l;this.trackRecursion&&0==--p.retainCount&&(p.tested=!1),l<s&&(s=l)}return s}.bind(this),h=o.getNodeAt(e,t),f=o.getNodeAt(n,r),p=c(h,f);for(a=0;;++a){if((l=d(h,0,p,s=[],0))===1/0)return[];if(l instanceof i)return s;p=l}return[]},e.exports=a},function(e,t,n){var r=n(126),i=n(2074),o=n(2075),a=n(2076),s=n(2077);e.exports=function(e){return(e=e||{}).diagonalMovement===r.Never?new i(e):e.diagonalMovement===r.Always?new o(e):e.diagonalMovement===r.OnlyWhenNoObstacles?new a(e):new s(e)}},function(e,t,n){var r=n(408),i=n(126);function o(e){r.call(this,e)}o.prototype=new r,o.prototype.constructor=o,o.prototype._jump=function(e,t,n,r){var i=this.grid,o=e-n,a=t-r;if(!i.isWalkableAt(e,t))return null;if(!0===this.trackJumpRecursion&&(i.getNodeAt(e,t).tested=!0),i.getNodeAt(e,t)===this.endNode)return[e,t];if(0!==o){if(i.isWalkableAt(e,t-1)&&!i.isWalkableAt(e-o,t-1)||i.isWalkableAt(e,t+1)&&!i.isWalkableAt(e-o,t+1))return[e,t]}else{if(0===a)throw new Error("Only horizontal and vertical movements are allowed");if(i.isWalkableAt(e-1,t)&&!i.isWalkableAt(e-1,t-a)||i.isWalkableAt(e+1,t)&&!i.isWalkableAt(e+1,t-a))return[e,t];if(this._jump(e+1,t,e,t)||this._jump(e-1,t,e,t))return[e,t]}return this._jump(e+o,t+a,e,t)},o.prototype._findNeighbors=function(e){var t,n,r,o,a,s,l,u,c=e.parent,d=e.x,h=e.y,f=this.grid,p=[];if(c)t=c.x,n=c.y,r=(d-t)/Math.max(Math.abs(d-t),1),o=(h-n)/Math.max(Math.abs(h-n),1),0!==r?(f.isWalkableAt(d,h-1)&&p.push([d,h-1]),f.isWalkableAt(d,h+1)&&p.push([d,h+1]),f.isWalkableAt(d+r,h)&&p.push([d+r,h])):0!==o&&(f.isWalkableAt(d-1,h)&&p.push([d-1,h]),f.isWalkableAt(d+1,h)&&p.push([d+1,h]),f.isWalkableAt(d,h+o)&&p.push([d,h+o]));else for(l=0,u=(a=f.getNeighbors(e,i.Never)).length;l<u;++l)s=a[l],p.push([s.x,s.y]);return p},e.exports=o},function(e,t,n){var r=n(408),i=n(126);function o(e){r.call(this,e)}o.prototype=new r,o.prototype.constructor=o,o.prototype._jump=function(e,t,n,r){var i=this.grid,o=e-n,a=t-r;if(!i.isWalkableAt(e,t))return null;if(!0===this.trackJumpRecursion&&(i.getNodeAt(e,t).tested=!0),i.getNodeAt(e,t)===this.endNode)return[e,t];if(0!==o&&0!==a){if(i.isWalkableAt(e-o,t+a)&&!i.isWalkableAt(e-o,t)||i.isWalkableAt(e+o,t-a)&&!i.isWalkableAt(e,t-a))return[e,t];if(this._jump(e+o,t,e,t)||this._jump(e,t+a,e,t))return[e,t]}else if(0!==o){if(i.isWalkableAt(e+o,t+1)&&!i.isWalkableAt(e,t+1)||i.isWalkableAt(e+o,t-1)&&!i.isWalkableAt(e,t-1))return[e,t]}else if(i.isWalkableAt(e+1,t+a)&&!i.isWalkableAt(e+1,t)||i.isWalkableAt(e-1,t+a)&&!i.isWalkableAt(e-1,t))return[e,t];return this._jump(e+o,t+a,e,t)},o.prototype._findNeighbors=function(e){var t,n,r,o,a,s,l,u,c=e.parent,d=e.x,h=e.y,f=this.grid,p=[];if(c)t=c.x,n=c.y,r=(d-t)/Math.max(Math.abs(d-t),1),o=(h-n)/Math.max(Math.abs(h-n),1),0!==r&&0!==o?(f.isWalkableAt(d,h+o)&&p.push([d,h+o]),f.isWalkableAt(d+r,h)&&p.push([d+r,h]),f.isWalkableAt(d+r,h+o)&&p.push([d+r,h+o]),f.isWalkableAt(d-r,h)||p.push([d-r,h+o]),f.isWalkableAt(d,h-o)||p.push([d+r,h-o])):0===r?(f.isWalkableAt(d,h+o)&&p.push([d,h+o]),f.isWalkableAt(d+1,h)||p.push([d+1,h+o]),f.isWalkableAt(d-1,h)||p.push([d-1,h+o])):(f.isWalkableAt(d+r,h)&&p.push([d+r,h]),f.isWalkableAt(d,h+1)||p.push([d+r,h+1]),f.isWalkableAt(d,h-1)||p.push([d+r,h-1]));else for(l=0,u=(a=f.getNeighbors(e,i.Always)).length;l<u;++l)s=a[l],p.push([s.x,s.y]);return p},e.exports=o},function(e,t,n){var r=n(408),i=n(126);function o(e){r.call(this,e)}o.prototype=new r,o.prototype.constructor=o,o.prototype._jump=function(e,t,n,r){var i=this.grid,o=e-n,a=t-r;if(!i.isWalkableAt(e,t))return null;if(!0===this.trackJumpRecursion&&(i.getNodeAt(e,t).tested=!0),i.getNodeAt(e,t)===this.endNode)return[e,t];if(0!==o&&0!==a){if(this._jump(e+o,t,e,t)||this._jump(e,t+a,e,t))return[e,t]}else if(0!==o){if(i.isWalkableAt(e,t-1)&&!i.isWalkableAt(e-o,t-1)||i.isWalkableAt(e,t+1)&&!i.isWalkableAt(e-o,t+1))return[e,t]}else if(0!==a&&(i.isWalkableAt(e-1,t)&&!i.isWalkableAt(e-1,t-a)||i.isWalkableAt(e+1,t)&&!i.isWalkableAt(e+1,t-a)))return[e,t];return i.isWalkableAt(e+o,t)&&i.isWalkableAt(e,t+a)?this._jump(e+o,t+a,e,t):null},o.prototype._findNeighbors=function(e){var t,n,r,o,a,s,l,u,c,d=e.parent,h=e.x,f=e.y,p=this.grid,g=[];if(d){if(t=d.x,n=d.y,r=(h-t)/Math.max(Math.abs(h-t),1),o=(f-n)/Math.max(Math.abs(f-n),1),0!==r&&0!==o)p.isWalkableAt(h,f+o)&&g.push([h,f+o]),p.isWalkableAt(h+r,f)&&g.push([h+r,f]),p.isWalkableAt(h,f+o)&&p.isWalkableAt(h+r,f)&&g.push([h+r,f+o]);else if(0!==r){c=p.isWalkableAt(h+r,f);var v=p.isWalkableAt(h,f+1),m=p.isWalkableAt(h,f-1);c&&(g.push([h+r,f]),v&&g.push([h+r,f+1]),m&&g.push([h+r,f-1])),v&&g.push([h,f+1]),m&&g.push([h,f-1])}else if(0!==o){c=p.isWalkableAt(h,f+o);var y=p.isWalkableAt(h+1,f),b=p.isWalkableAt(h-1,f);c&&(g.push([h,f+o]),y&&g.push([h+1,f+o]),b&&g.push([h-1,f+o])),y&&g.push([h+1,f]),b&&g.push([h-1,f])}}else for(l=0,u=(a=p.getNeighbors(e,i.OnlyWhenNoObstacles)).length;l<u;++l)s=a[l],g.push([s.x,s.y]);return g},e.exports=o},function(e,t,n){var r=n(408),i=n(126);function o(e){r.call(this,e)}o.prototype=new r,o.prototype.constructor=o,o.prototype._jump=function(e,t,n,r){var i=this.grid,o=e-n,a=t-r;if(!i.isWalkableAt(e,t))return null;if(!0===this.trackJumpRecursion&&(i.getNodeAt(e,t).tested=!0),i.getNodeAt(e,t)===this.endNode)return[e,t];if(0!==o&&0!==a){if(i.isWalkableAt(e-o,t+a)&&!i.isWalkableAt(e-o,t)||i.isWalkableAt(e+o,t-a)&&!i.isWalkableAt(e,t-a))return[e,t];if(this._jump(e+o,t,e,t)||this._jump(e,t+a,e,t))return[e,t]}else if(0!==o){if(i.isWalkableAt(e+o,t+1)&&!i.isWalkableAt(e,t+1)||i.isWalkableAt(e+o,t-1)&&!i.isWalkableAt(e,t-1))return[e,t]}else if(i.isWalkableAt(e+1,t+a)&&!i.isWalkableAt(e+1,t)||i.isWalkableAt(e-1,t+a)&&!i.isWalkableAt(e-1,t))return[e,t];return i.isWalkableAt(e+o,t)||i.isWalkableAt(e,t+a)?this._jump(e+o,t+a,e,t):null},o.prototype._findNeighbors=function(e){var t,n,r,o,a,s,l,u,c=e.parent,d=e.x,h=e.y,f=this.grid,p=[];if(c)t=c.x,n=c.y,r=(d-t)/Math.max(Math.abs(d-t),1),o=(h-n)/Math.max(Math.abs(h-n),1),0!==r&&0!==o?(f.isWalkableAt(d,h+o)&&p.push([d,h+o]),f.isWalkableAt(d+r,h)&&p.push([d+r,h]),(f.isWalkableAt(d,h+o)||f.isWalkableAt(d+r,h))&&p.push([d+r,h+o]),!f.isWalkableAt(d-r,h)&&f.isWalkableAt(d,h+o)&&p.push([d-r,h+o]),!f.isWalkableAt(d,h-o)&&f.isWalkableAt(d+r,h)&&p.push([d+r,h-o])):0===r?f.isWalkableAt(d,h+o)&&(p.push([d,h+o]),f.isWalkableAt(d+1,h)||p.push([d+1,h+o]),f.isWalkableAt(d-1,h)||p.push([d-1,h+o])):f.isWalkableAt(d+r,h)&&(p.push([d+r,h]),f.isWalkableAt(d,h+1)||p.push([d+r,h+1]),f.isWalkableAt(d,h-1)||p.push([d+r,h-1]));else for(l=0,u=(a=f.getNeighbors(e,i.IfAtMostOneObstacle)).length;l<u;++l)s=a[l],p.push([s.x,s.y]);return p},e.exports=o},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=function(e,t){if(Array.isArray(e))return e;if(Symbol.iterator in Object(e))return function(e,t){var n=[],r=!0,i=!1,o=void 0;try{for(var a,s=e[Symbol.iterator]();!(r=(a=s.next()).done)&&(n.push(a.value),!t||n.length!==t);r=!0);}catch(e){i=!0,o=e}finally{try{!r&&s.return&&s.return()}finally{if(i)throw o}}return n}(e,t);throw new TypeError("Invalid attempt to destructure non-iterable instance")},i=n(2079),o=function e(t){var n=t||[],o=function(e){return e.command+" "+e.params.map((function(e){return t=6,function(e,t){for(var n=e.length;"0"===e.charAt(n-1);)n-=1;return"."===e.charAt(n-1)&&(n-=1),e.substr(0,n)}(e.toFixed(t));var t})).join(" ")},a=function(e,t){var n=e.command,r=e.params;switch(n){case"M":case"L":return[r[0],r[1]];case"H":return[r[0],t[1]];case"V":return[t[0],r[0]];case"Z":return null;case"C":return[r[4],r[5]];case"S":case"Q":return[r[2],r[3]];case"T":return[r[0],r[1]];case"A":return[r[5],r[6]]}},s=function(e,t,n){var r=e.params,o={V:function(e,t,n){var o=[{x:n[0],y:r[0]}],a=(0,i.matrixTransform)(o,t);return a[0].x===(0,i.matrixTransform)([{x:n[0],y:n[1]}],t)[0].x?{command:"V",params:[a[0].y]}:{command:"L",params:[a[0].x,a[0].y]}},H:function(e,t,n){var o=[{x:r[0],y:n[1]}],a=(0,i.matrixTransform)(o,t);return a[0].y===(0,i.matrixTransform)([{x:n[0],y:n[1]}],t)[0].y?{command:"H",params:[a[0].x]}:{command:"L",params:[a[0].x,a[0].y]}},A:function(e,t,n){var o=(0,i.transformEllipse)(r[0],r[1],r[2],t),a=r[4];t[0]*t[3]-t[1]*t[2]<0&&(a=a?"0":"1");var s=[{x:r[5],y:r[6]}],l=(0,i.matrixTransform)(s,t);return o.isDegenerate?{command:"L",params:[l[0].x,l[0].y]}:{command:"A",params:[o.rx,o.ry,o.ax,r[3],a,l[0].x,l[0].y]}},C:function(e,t,n){var o=[{x:r[0],y:r[1]},{x:r[2],y:r[3]},{x:r[4],y:r[5]}],a=(0,i.matrixTransform)(o,t);return{command:"C",params:[a[0].x,a[0].y,a[1].x,a[1].y,a[2].x,a[2].y]}},Z:function(e,t,n){return{command:"Z",params:[]}},default:function(e,t,n){var o=[{x:r[0],y:r[1]}],a=(0,i.matrixTransform)(o,t),s=e.params.slice(0,e.params.length);return s.splice(0,2,a[0].x,a[0].y),{command:e.command,params:s}}};return o[e.command]?o[e.command](e,t,n):o.default(e,t,n)},l=function(e,t){return function(n){var r="object"==typeof n?e.map((function(e){return n[e]})):arguments;return t.apply(null,r)}},u=function(t){return e((i=t,(o=(r=n).slice(0,r.length)).push(i),o));var r,i,o};return{moveto:l(["x","y"],(function(e,t){return u({command:"M",params:[e,t]})})),lineto:l(["x","y"],(function(e,t){return u({command:"L",params:[e,t]})})),hlineto:l(["x"],(function(e){return u({command:"H",params:[e]})})),vlineto:l(["y"],(function(e){return u({command:"V",params:[e]})})),closepath:function(){return u({command:"Z",params:[]})},curveto:l(["x1","y1","x2","y2","x","y"],(function(e,t,n,r,i,o){return u({command:"C",params:[e,t,n,r,i,o]})})),smoothcurveto:l(["x2","y2","x","y"],(function(e,t,n,r){return u({command:"S",params:[e,t,n,r]})})),qcurveto:l(["x1","y1","x","y"],(function(e,t,n,r){return u({command:"Q",params:[e,t,n,r]})})),smoothqcurveto:l(["x","y"],(function(e,t){return u({command:"T",params:[e,t]})})),arc:l(["rx","ry","xrot","largeArcFlag","sweepFlag","x","y"],(function(e,t,n,r,i,o,a){return u({command:"A",params:[e,t,n,r,i,o,a]})})),translate:l(["dx","dy"],(function(){var t=arguments.length<=0||void 0===arguments[0]?0:arguments[0],r=arguments.length<=1||void 0===arguments[1]?0:arguments[1];if(0===t&&0===r)return e(n);var i,o,l,u=(i=[0,0],o=[1,0,0,1,t,r],l=n.map((function(e){var t=s(e,o,i);return i=a(e,i),t})),{v:e(l)});return"object"==typeof u?u.v:void 0})),rotate:l(["angle","rx","ry"],(function(t){var r=arguments.length<=1||void 0===arguments[1]?0:arguments[1],i=arguments.length<=2||void 0===arguments[2]?0:arguments[2];if(0===t)return e(n);var o=function(){var o=void 0,l=void 0,u=n;0!==r&&0!==i&&(o=[0,0],l=[1,0,0,1,-r,-i],u=u.map((function(e){var t=s(e,l,o);return o=a(e,o),t})));var c=t*Math.PI/180,d=Math.cos(c),h=Math.sin(c);return o=[0,0],l=[d,h,-h,d,0,0],u=u.map((function(e){var t=s(e,l,o);return o=a(e,o),t})),0!==r&&0!==i&&(o=[0,0],l=[1,0,0,1,r,i],u=u.map((function(e){var t=s(e,l,o);return o=a(e,o),t}))),{v:e(u)}}();return"object"==typeof o?o.v:void 0})),scale:l(["sx","sy"],(function(){var t=arguments.length<=0||void 0===arguments[0]?1:arguments[0],r=arguments.length<=1||void 0===arguments[1]?t:arguments[1];return function(){if(1===t&&1===r)return e(n);var i,o,l,u=(i=[0,0],o=[t,0,0,r,0,0],l=n.map((function(e){var t=s(e,o,i);return i=a(e,i),t})),{v:e(l)});return"object"==typeof u?u.v:void 0}()})),shearX:l(["angle"],(function(){var t=arguments.length<=0||void 0===arguments[0]?0:arguments[0];if(0===t)return e(n);var r,i,o,l=(r=[0,0],i=[1,0,Math.tan(t*Math.PI/180),1,0,0],o=n.map((function(e){var t=s(e,i,r);return r=a(e,r),t})),{v:e(o)});return"object"==typeof l?l.v:void 0})),shearY:l(["angle"],(function(){var t=arguments.length<=0||void 0===arguments[0]?0:arguments[0];if(0===t)return e(n);var r,i,o,l=(r=[0,0],i=[1,Math.tan(t*Math.PI/180),0,1,0,0],o=n.map((function(e){var t=s(e,i,r);return r=a(e,r),t})),{v:e(o)});return"object"==typeof l?l.v:void 0})),print:function(){return n.map(o).join(" ")},toString:function(){return(void 0).print()},points:function(){var e=[],t=[0,0],r=!0,i=!1,o=void 0;try{for(var s,l=n[Symbol.iterator]();!(r=(s=l.next()).done);r=!0){var u=s.value,c=a(u,t);t=c,c&&e.push(c)}}catch(e){i=!0,o=e}finally{try{!r&&l.return&&l.return()}finally{if(i)throw o}}return e},instructions:function(){return n.slice(0,n.length)},connect:function(t){var i,o,a,s,l,u,c,d=this.points(),h=d[d.length-1],f=t.points()[0],p=void 0;return"Z"!==n[n.length-1].command?(p=t.instructions().slice(1),i=f,o=r(h,2),a=o[0],s=o[1],l=r(i,2),u=l[0],c=l[1],(a!==u||s!==c)&&p.unshift({command:"L",params:f})):p=t.instructions(),e(this.instructions().concat(p))}}};t.default=function(){return o()},e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=function(e,t){if(Array.isArray(e))return e;if(Symbol.iterator in Object(e))return function(e,t){var n=[],r=!0,i=!1,o=void 0;try{for(var a,s=e[Symbol.iterator]();!(r=(a=s.next()).done)&&(n.push(a.value),!t||n.length!==t);r=!0);}catch(e){i=!0,o=e}finally{try{!r&&s.return&&s.return()}finally{if(i)throw o}}return n}(e,t);throw new TypeError("Invalid attempt to destructure non-iterable instance")},i=function(e){return e*e},o=function(e,t){var n=r(e,2),o=n[0],a=n[1],s=r(t,2),l=s[0],u=s[1];return Math.sqrt(i(o-l)+i(a-u))};t.default={distPointToPoint:o,distPointToParabol:function(e,t){var n=o(e,t);return 0==n?1/0:i(n)/(2*Math.abs(e[1]-t[1]))},circumCenter:function(e,t,n){var r=(e[0]-n[0])*(t[1]-n[1])-(t[0]-n[0])*(e[1]-n[1]);return 0==r?[1/0,1/0]:[(((e[0]-n[0])*(e[0]+n[0])+(e[1]-n[1])*(e[1]+n[1]))/2*(t[1]-n[1])-((t[0]-n[0])*(t[0]+n[0])+(t[1]-n[1])*(t[1]+n[1]))/2*(e[1]-n[1]))/r,(((t[0]-n[0])*(t[0]+n[0])+(t[1]-n[1])*(t[1]+n[1]))/2*(e[0]-n[0])-((e[0]-n[0])*(e[0]+n[0])+(e[1]-n[1])*(e[1]+n[1]))/2*(t[0]-n[0]))/r]},parabolsCrossX:function(e,t,n){if(e[1]===t[1])return[(e[0]+t[0])/2,(e[0]+t[0])/2];var r=(e[1]*t[0]-e[0]*t[1]+e[0]*n-t[0]*n+Math.sqrt((e[0]*e[0]+e[1]*e[1]-2*e[0]*t[0]+t[0]*t[0]-2*e[1]*t[1]+t[1]*t[1])*(e[1]*t[1]-e[1]*n-t[1]*n+n*n)))/(e[1]-t[1]),i=(e[1]*t[0]-e[0]*t[1]+e[0]*n-t[0]*n-Math.sqrt((e[0]*e[0]+e[1]*e[1]-2*e[0]*t[0]+t[0]*t[0]-2*e[1]*t[1]+t[1]*t[1])*(e[1]*t[1]-e[1]*n-t[1]*n+n*n)))/(e[1]-t[1]);return r<i?[r,i]:[i,r]},doHalflinesCross:function(e,t){var n=arguments.length<=2||void 0===arguments[2]?1e-10:arguments[2],r=t.ps[0]-e.ps[0],i=t.ps[1]-e.ps[1];if(e.m==1/0)return e.hp*(t.m*r-i)<=n&&t.vec[0]*r<=n;if(t.m==1/0)return t.hp*(e.m*r-i)>=-n&&e.vec[0]*r>=-n;var o=t.vec[0]*e.vec[1]-t.vec[1]*e.vec[0];if(0===o)return!1;var a=(i*t.vec[0]-r*t.vec[1])/o,s=(i*e.vec[0]-r*e.vec[1])/o;return a>=-n&&s>=n||a>=n&&s>=-n},matrixTransform:function(e,t){return e.map((function(e){return{x:e.x*t[0]+e.y*t[2]+t[4],y:e.x*t[1]+e.y*t[3]+t[5]}}))},transformEllipse:function(e,t,n,r){var i=Math.PI/180,o=Math.cos(n*i),a=Math.sin(n*i),s=[e*(r[0]*o+r[2]*a),e*(r[1]*o+r[3]*a),t*(-r[0]*a+r[2]*o),t*(-r[1]*a+r[3]*o)],l=s[0]*s[0]+s[2]*s[2],u=s[1]*s[1]+s[3]*s[3],c=((s[0]-s[3])*(s[0]-s[3])+(s[2]+s[1])*(s[2]+s[1]))*((s[0]+s[3])*(s[0]+s[3])+(s[2]-s[1])*(s[2]-s[1])),d=(l+u)/2;if(c<1e-10*d)return{rx:Math.sqrt(d),ry:Math.sqrt(d),ax:0,isDegenerate:!1};var h=s[0]*s[1]+s[2]*s[3],f=d+(c=Math.sqrt(c))/2,p=d-c/2,g=void 0,v=void 0,m=void 0;return(g=Math.abs(h)<1e-10&&Math.abs(f-u)<1e-10?90:180*Math.atan(Math.abs(h)>Math.abs(f-u)?(f-l)/h:h/(f-u))/Math.PI)>=0?(v=Math.sqrt(f),m=Math.sqrt(p)):(g+=90,v=Math.sqrt(p),m=Math.sqrt(f)),{rx:v,ry:m,ax:g,isDegenerate:v<1e-10*m||m<1e-10*v}}},e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.DagreEngine=void 0;const r=n(152),i=n(2081),o=n(67),a=n(151);t.DagreEngine=class{constructor(e={}){this.options=e}redistribute(e){var t=new i.graphlib.Graph({multigraph:!0});t.setGraph(this.options.graph||{}),t.setDefaultEdgeLabel((function(){return{}}));const n={};o.forEach(e.getNodes(),e=>{t.setNode(e.getID(),{width:e.width,height:e.height})}),o.forEach(e.getLinks(),e=>{e.getSourcePort()&&e.getTargetPort()&&(n[e.getID()]=!0,t.setEdge({v:e.getSourcePort().getNode().getID(),w:e.getTargetPort().getNode().getID(),name:e.getID()}))}),i.layout(t),t.nodes().forEach(n=>{const r=t.node(n);e.getNode(n).setPosition(r.x-r.width/2,r.y-r.height/2)}),this.options.includeLinks&&t.edges().forEach(n=>{const i=t.edge(n),o=e.getLink(n.name),s=[o.getFirstPoint()];for(let e=1;e<i.points.length-2;e++)s.push(new r.PointModel({link:o,position:new a.Point(i.points[e].x,i.points[e].y)}));o.setPoints(s.concat(o.getLastPoint()))})}}},function(e,t,n){e.exports={graphlib:n(153),layout:n(2152),debug:n(2210),util:{time:n(113).time,notime:n(113).notime},version:n(2211)}},function(e,t,n){var r=n(2083);e.exports={Graph:r.Graph,json:n(2142),alg:n(2143),version:r.version}},function(e,t,n){e.exports={Graph:n(606),version:n(2141)}},function(e,t,n){var r=n(1138);e.exports=function(e){return r(e,4)}},function(e,t,n){var r=n(308),i=n(189);e.exports=function(e,t){return e&&r(t,i(t),e)}},function(e,t,n){var r=n(308),i=n(258);e.exports=function(e,t){return e&&r(t,i(t),e)}},function(e,t,n){var r=n(130),i=n(363),o=n(2088),a=Object.prototype.hasOwnProperty;e.exports=function(e){if(!r(e))return o(e);var t=i(e),n=[];for(var s in e)("constructor"!=s||!t&&a.call(e,s))&&n.push(s);return n}},function(e,t){e.exports=function(e){var t=[];if(null!=e)for(var n in Object(e))t.push(n);return t}},function(e,t,n){var r=n(308),i=n(496);e.exports=function(e,t){return r(e,i(e),t)}},function(e,t,n){var r=n(308),i=n(1142);e.exports=function(e,t){return r(e,i(e),t)}},function(e,t,n){var r=n(857),i=n(1142),o=n(258);e.exports=function(e){return r(e,o,i)}},function(e,t){var n=Object.prototype.hasOwnProperty;e.exports=function(e){var t=e.length,r=new e.constructor(t);return t&&"string"==typeof e[0]&&n.call(e,"index")&&(r.index=e.index,r.input=e.input),r}},function(e,t,n){var r=n(608),i=n(2094),o=n(2095),a=n(2096),s=n(1143);e.exports=function(e,t,n){var l=e.constructor;switch(t){case"[object ArrayBuffer]":return r(e);case"[object Boolean]":case"[object Date]":return new l(+e);case"[object DataView]":return i(e,n);case"[object Float32Array]":case"[object Float64Array]":case"[object Int8Array]":case"[object Int16Array]":case"[object Int32Array]":case"[object Uint8Array]":case"[object Uint8ClampedArray]":case"[object Uint16Array]":case"[object Uint32Array]":return s(e,n);case"[object Map]":return new l;case"[object Number]":case"[object String]":return new l(e);case"[object RegExp]":return o(e);case"[object Set]":return new l;case"[object Symbol]":return a(e)}}},function(e,t,n){var r=n(608);e.exports=function(e,t){var n=t?r(e.buffer):e.buffer;return new e.constructor(n,e.byteOffset,e.byteLength)}},function(e,t){var n=/\w*$/;e.exports=function(e){var t=new e.constructor(e.source,n.exec(e));return t.lastIndex=e.lastIndex,t}},function(e,t,n){var r=n(236),i=r?r.prototype:void 0,o=i?i.valueOf:void 0;e.exports=function(e){return o?Object(o.call(e)):{}}},function(e,t,n){var r=n(2098),i=n(362),o=n(498),a=o&&o.isMap,s=a?i(a):r;e.exports=s},function(e,t,n){var r=n(238),i=n(158);e.exports=function(e){return i(e)&&"[object Map]"==r(e)}},function(e,t,n){var r=n(2100),i=n(362),o=n(498),a=o&&o.isSet,s=a?i(a):r;e.exports=s},function(e,t,n){var r=n(238),i=n(158);e.exports=function(e){return i(e)&&"[object Set]"==r(e)}},function(e,t){e.exports=function(e){return function(t,n,r){for(var i=-1,o=Object(t),a=r(t),s=a.length;s--;){var l=a[e?s:++i];if(!1===n(o[l],l,o))break}return t}}},function(e,t,n){var r=n(174);e.exports=function(e,t){return function(n,i){if(null==n)return n;if(!r(n))return e(n,i);for(var o=n.length,a=t?o:-1,s=Object(n);(t?a--:++a<o)&&!1!==i(s[a],a,s););return n}}},function(e,t,n){var r=n(411);e.exports=function(e,t){var n=[];return r(e,(function(e,r,i){t(e,r,i)&&n.push(e)})),n}},function(e,t,n){var r=n(2105),i=n(2106),o=n(1151);e.exports=function(e){var t=i(e);return 1==t.length&&t[0][2]?o(t[0][0],t[0][1]):function(n){return n===e||r(n,e,t)}}},function(e,t,n){var r=n(356),i=n(491);e.exports=function(e,t,n,o){var a=n.length,s=a,l=!o;if(null==e)return!s;for(e=Object(e);a--;){var u=n[a];if(l&&u[2]?u[1]!==e[u[0]]:!(u[0]in e))return!1}for(;++a<s;){var c=(u=n[a])[0],d=e[c],h=u[1];if(l&&u[2]){if(void 0===d&&!(c in e))return!1}else{var f=new r;if(o)var p=o(d,h,c,e,t,f);if(!(void 0===p?i(h,d,3,o,f):p))return!1}}return!0}},function(e,t,n){var r=n(1150),i=n(189);e.exports=function(e){for(var t=i(e),n=t.length;n--;){var o=t[n],a=e[o];t[n]=[o,a,r(a)]}return t}},function(e,t,n){var r=n(491),i=n(2108),o=n(1153),a=n(613),s=n(1150),l=n(1151),u=n(309);e.exports=function(e,t){return a(e)&&s(t)?l(u(e),t):function(n){var a=i(n,e);return void 0===a&&a===t?o(n,e):r(t,a,3)}}},function(e,t,n){var r=n(612);e.exports=function(e,t,n){var i=null==e?void 0:r(e,t);return void 0===i?n:i}},function(e,t,n){var r=n(2110),i=/[^.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\\]|\\.)*?)\2)\]|(?=(?:\.|\[\])(?:\.|\[\]|$))/g,o=/\\(\\)?/g,a=r((function(e){var t=[];return 46===e.charCodeAt(0)&&t.push(""),e.replace(i,(function(e,n,r,i){t.push(r?i.replace(o,"$1"):n||e)})),t}));e.exports=a},function(e,t,n){var r=n(2111);e.exports=function(e){var t=r(e,(function(e){return 500===n.size&&n.clear(),e})),n=t.cache;return t}},function(e,t,n){var r=n(493);function i(e,t){if("function"!=typeof e||null!=t&&"function"!=typeof t)throw new TypeError("Expected a function");var n=function(){var r=arguments,i=t?t.apply(this,r):r[0],o=n.cache;if(o.has(i))return o.get(i);var a=e.apply(this,r);return n.cache=o.set(i,a)||o,a};return n.cache=new(i.Cache||r),n}i.Cache=r,e.exports=i},function(e,t,n){var r=n(236),i=n(413),o=n(81),a=n(259),s=r?r.prototype:void 0,l=s?s.toString:void 0;e.exports=function e(t){if("string"==typeof t)return t;if(o(t))return i(t,e)+"";if(a(t))return l?l.call(t):"";var n=t+"";return"0"==n&&1/t==-1/0?"-0":n}},function(e,t){e.exports=function(e,t){return null!=e&&t in Object(e)}},function(e,t,n){var r=n(1155),i=n(2115),o=n(613),a=n(309);e.exports=function(e){return o(e)?r(a(e)):i(e)}},function(e,t,n){var r=n(612);e.exports=function(e){return function(t){return r(t,e)}}},function(e,t){var n=Object.prototype.hasOwnProperty;e.exports=function(e,t){return null!=e&&n.call(e,t)}},function(e,t,n){var r=n(499),i=n(238),o=n(282),a=n(81),s=n(174),l=n(237),u=n(363),c=n(283),d=Object.prototype.hasOwnProperty;e.exports=function(e){if(null==e)return!0;if(s(e)&&(a(e)||"string"==typeof e||"function"==typeof e.splice||l(e)||c(e)||o(e)))return!e.length;var t=i(e);if("[object Map]"==t||"[object Set]"==t)return!e.size;if(u(e))return!r(e).length;for(var n in e)if(d.call(e,n))return!1;return!0}},function(e,t){e.exports=function(e,t,n,r){var i=-1,o=null==e?0:e.length;for(r&&o&&(n=e[++i]);++i<o;)n=t(n,e[i],i,e);return n}},function(e,t){e.exports=function(e,t,n,r,i){return i(e,(function(e,i,o){n=r?(r=!1,e):t(n,e,i,o)})),n}},function(e,t,n){var r=n(499),i=n(238),o=n(174),a=n(2121),s=n(2122);e.exports=function(e){if(null==e)return 0;if(o(e))return a(e)?s(e):e.length;var t=i(e);return"[object Map]"==t||"[object Set]"==t?e.size:r(e).length}},function(e,t,n){var r=n(208),i=n(81),o=n(158);e.exports=function(e){return"string"==typeof e||!i(e)&&o(e)&&"[object String]"==r(e)}},function(e,t,n){var r=n(2123),i=n(2124),o=n(2125);e.exports=function(e){return i(e)?o(e):r(e)}},function(e,t,n){var r=n(1155)("length");e.exports=r},function(e,t){var n=RegExp("[\\u200d\\ud800-\\udfff\\u0300-\\u036f\\ufe20-\\ufe2f\\u20d0-\\u20ff\\ufe0e\\ufe0f]");e.exports=function(e){return n.test(e)}},function(e,t){var n="[\\ud800-\\udfff]",r="[\\u0300-\\u036f\\ufe20-\\ufe2f\\u20d0-\\u20ff]",i="\\ud83c[\\udffb-\\udfff]",o="[^\\ud800-\\udfff]",a="(?:\\ud83c[\\udde6-\\uddff]){2}",s="[\\ud800-\\udbff][\\udc00-\\udfff]",l="(?:"+r+"|"+i+")"+"?",u="[\\ufe0e\\ufe0f]?"+l+("(?:\\u200d(?:"+[o,a,s].join("|")+")[\\ufe0e\\ufe0f]?"+l+")*"),c="(?:"+[o+r+"?",r,a,s,n].join("|")+")",d=RegExp(i+"(?="+i+")|"+c+u,"g");e.exports=function(e){for(var t=d.lastIndex=0;d.test(e);)++t;return t}},function(e,t,n){var r=n(607),i=n(1145),o=n(610),a=n(183),s=n(364),l=n(81),u=n(237),c=n(281),d=n(130),h=n(283);e.exports=function(e,t,n){var f=l(e),p=f||u(e)||h(e);if(t=a(t,4),null==n){var g=e&&e.constructor;n=p?f?new g:[]:d(e)&&c(g)?i(s(e)):{}}return(p?r:o)(e,(function(e,r,i){return t(n,e,r,i)})),n}},function(e,t,n){var r=n(614),i=n(414),o=n(2132),a=n(1164),s=i((function(e){return o(r(e,1,a,!0))}));e.exports=s},function(e,t,n){var r=n(236),i=n(282),o=n(81),a=r?r.isConcatSpreadable:void 0;e.exports=function(e){return o(e)||i(e)||!!(a&&e&&e[a])}},function(e,t){e.exports=function(e,t,n){switch(n.length){case 0:return e.call(t);case 1:return e.call(t,n[0]);case 2:return e.call(t,n[0],n[1]);case 3:return e.call(t,n[0],n[1],n[2])}return e.apply(t,n)}},function(e,t,n){var r=n(609),i=n(1139),o=n(220),a=i?function(e,t){return i(e,"toString",{configurable:!0,enumerable:!1,value:r(t),writable:!0})}:o;e.exports=a},function(e,t){var n=Date.now;e.exports=function(e){var t=0,r=0;return function(){var i=n(),o=16-(i-r);if(r=i,o>0){if(++t>=800)return arguments[0]}else t=0;return e.apply(void 0,arguments)}}},function(e,t,n){var r=n(853),i=n(2133),o=n(2137),a=n(854),s=n(2138),l=n(494);e.exports=function(e,t,n){var u=-1,c=i,d=e.length,h=!0,f=[],p=f;if(n)h=!1,c=o;else if(d>=200){var g=t?null:s(e);if(g)return l(g);h=!1,c=a,p=new r}else p=t?[]:f;e:for(;++u<d;){var v=e[u],m=t?t(v):v;if(v=n||0!==v?v:0,h&&m==m){for(var y=p.length;y--;)if(p[y]===m)continue e;t&&p.push(m),f.push(v)}else c(p,m,n)||(p!==f&&p.push(m),f.push(v))}return f}},function(e,t,n){var r=n(2134);e.exports=function(e,t){return!!(null==e?0:e.length)&&r(e,t,0)>-1}},function(e,t,n){var r=n(1163),i=n(2135),o=n(2136);e.exports=function(e,t,n){return t==t?o(e,t,n):r(e,i,n)}},function(e,t){e.exports=function(e){return e!=e}},function(e,t){e.exports=function(e,t,n){for(var r=n-1,i=e.length;++r<i;)if(e[r]===t)return r;return-1}},function(e,t){e.exports=function(e,t,n){for(var r=-1,i=null==e?0:e.length;++r<i;)if(n(t,e[r]))return!0;return!1}},function(e,t,n){var r=n(862),i=n(2139),o=n(494),a=r&&1/o(new r([,-0]))[1]==1/0?function(e){return new r(e)}:i;e.exports=a},function(e,t){e.exports=function(){}},function(e,t,n){var r=n(413);e.exports=function(e,t){return r(t,(function(t){return e[t]}))}},function(e,t){e.exports="2.1.8"},function(e,t,n){var r=n(135),i=n(606);function o(e){return r.map(e.nodes(),(function(t){var n=e.node(t),i=e.parent(t),o={v:t};return r.isUndefined(n)||(o.value=n),r.isUndefined(i)||(o.parent=i),o}))}function a(e){return r.map(e.edges(),(function(t){var n=e.edge(t),i={v:t.v,w:t.w};return r.isUndefined(t.name)||(i.name=t.name),r.isUndefined(n)||(i.value=n),i}))}e.exports={write:function(e){var t={options:{directed:e.isDirected(),multigraph:e.isMultigraph(),compound:e.isCompound()},nodes:o(e),edges:a(e)};r.isUndefined(e.graph())||(t.value=r.clone(e.graph()));return t},read:function(e){var t=new i(e.options).setGraph(e.value);return r.each(e.nodes,(function(e){t.setNode(e.v,e.value),e.parent&&t.setParent(e.v,e.parent)})),r.each(e.edges,(function(e){t.setEdge({v:e.v,w:e.w,name:e.name},e.value)})),t}}},function(e,t,n){e.exports={components:n(2144),dijkstra:n(1166),dijkstraAll:n(2145),findCycles:n(2146),floydWarshall:n(2147),isAcyclic:n(2148),postorder:n(2149),preorder:n(2150),prim:n(2151),tarjan:n(1168),topsort:n(1169)}},function(e,t,n){var r=n(135);e.exports=function(e){var t,n={},i=[];function o(i){r.has(n,i)||(n[i]=!0,t.push(i),r.each(e.successors(i),o),r.each(e.predecessors(i),o))}return r.each(e.nodes(),(function(e){t=[],o(e),t.length&&i.push(t)})),i}},function(e,t,n){var r=n(1166),i=n(135);e.exports=function(e,t,n){return i.transform(e.nodes(),(function(i,o){i[o]=r(e,o,t,n)}),{})}},function(e,t,n){var r=n(135),i=n(1168);e.exports=function(e){return r.filter(i(e),(function(t){return t.length>1||1===t.length&&e.hasEdge(t[0],t[0])}))}},function(e,t,n){var r=n(135);e.exports=function(e,t,n){return function(e,t,n){var r={},i=e.nodes();return i.forEach((function(e){r[e]={},r[e][e]={distance:0},i.forEach((function(t){e!==t&&(r[e][t]={distance:Number.POSITIVE_INFINITY})})),n(e).forEach((function(n){var i=n.v===e?n.w:n.v,o=t(n);r[e][i]={distance:o,predecessor:e}}))})),i.forEach((function(e){var t=r[e];i.forEach((function(n){var o=r[n];i.forEach((function(n){var r=o[e],i=t[n],a=o[n],s=r.distance+i.distance;s<a.distance&&(a.distance=s,a.predecessor=i.predecessor)}))}))})),r}(e,t||i,n||function(t){return e.outEdges(t)})};var i=r.constant(1)},function(e,t,n){var r=n(1169);e.exports=function(e){try{r(e)}catch(e){if(e instanceof r.CycleException)return!1;throw e}return!0}},function(e,t,n){var r=n(1170);e.exports=function(e,t){return r(e,t,"post")}},function(e,t,n){var r=n(1170);e.exports=function(e,t){return r(e,t,"pre")}},function(e,t,n){var r=n(135),i=n(606),o=n(1167);e.exports=function(e,t){var n,a=new i,s={},l=new o;function u(e){var r=e.v===n?e.w:e.v,i=l.priority(r);if(void 0!==i){var o=t(e);o<i&&(s[r]=n,l.decrease(r,o))}}if(0===e.nodeCount())return a;r.each(e.nodes(),(function(e){l.add(e,Number.POSITIVE_INFINITY),a.setNode(e)})),l.decrease(e.nodes()[0],0);var c=!1;for(;l.size()>0;){if(n=l.removeMin(),r.has(s,n))a.setEdge(n,s[n]);else{if(c)throw new Error("Input graph is not connected: "+e);c=!0}e.nodeEdges(n).forEach(u)}return a}},function(e,t,n){"use strict";var r=n(70),i=n(2189),o=n(2192),a=n(2193),s=n(113).normalizeRanks,l=n(2195),u=n(113).removeEmptyRanks,c=n(2196),d=n(2197),h=n(2198),f=n(2199),p=n(2208),g=n(113),v=n(153).Graph;e.exports=function(e,t){var n=t&&t.debugTiming?g.time:g.notime;n("layout",(function(){var t=n("  buildLayoutGraph",(function(){return function(e){var t=new v({multigraph:!0,compound:!0}),n=M(e.graph());return t.setGraph(r.merge({},y,w(n,m),r.pick(n,b))),r.forEach(e.nodes(),(function(n){var i=M(e.node(n));t.setNode(n,r.defaults(w(i,C),A)),t.setParent(n,e.parent(n))})),r.forEach(e.edges(),(function(n){var i=M(e.edge(n));t.setEdge(n,r.merge({},I,w(i,_),r.pick(i,x)))})),t}(e)}));n("  runLayout",(function(){!function(e,t){t("    makeSpaceForEdgeLabels",(function(){!function(e){var t=e.graph();t.ranksep/=2,r.forEach(e.edges(),(function(n){var r=e.edge(n);r.minlen*=2,"c"!==r.labelpos.toLowerCase()&&("TB"===t.rankdir||"BT"===t.rankdir?r.width+=r.labeloffset:r.height+=r.labeloffset)}))}(e)})),t("    removeSelfEdges",(function(){!function(e){r.forEach(e.edges(),(function(t){if(t.v===t.w){var n=e.node(t.v);n.selfEdges||(n.selfEdges=[]),n.selfEdges.push({e:t,label:e.edge(t)}),e.removeEdge(t)}}))}(e)})),t("    acyclic",(function(){i.run(e)})),t("    nestingGraph.run",(function(){c.run(e)})),t("    rank",(function(){a(g.asNonCompoundGraph(e))})),t("    injectEdgeLabelProxies",(function(){!function(e){r.forEach(e.edges(),(function(t){var n=e.edge(t);if(n.width&&n.height){var r=e.node(t.v),i={rank:(e.node(t.w).rank-r.rank)/2+r.rank,e:t};g.addDummyNode(e,"edge-proxy",i,"_ep")}}))}(e)})),t("    removeEmptyRanks",(function(){u(e)})),t("    nestingGraph.cleanup",(function(){c.cleanup(e)})),t("    normalizeRanks",(function(){s(e)})),t("    assignRankMinMax",(function(){!function(e){var t=0;r.forEach(e.nodes(),(function(n){var i=e.node(n);i.borderTop&&(i.minRank=e.node(i.borderTop).rank,i.maxRank=e.node(i.borderBottom).rank,t=r.max(t,i.maxRank))})),e.graph().maxRank=t}(e)})),t("    removeEdgeLabelProxies",(function(){!function(e){r.forEach(e.nodes(),(function(t){var n=e.node(t);"edge-proxy"===n.dummy&&(e.edge(n.e).labelRank=n.rank,e.removeNode(t))}))}(e)})),t("    normalize.run",(function(){o.run(e)})),t("    parentDummyChains",(function(){l(e)})),t("    addBorderSegments",(function(){d(e)})),t("    order",(function(){f(e)})),t("    insertSelfEdges",(function(){!function(e){var t=g.buildLayerMatrix(e);r.forEach(t,(function(t){var n=0;r.forEach(t,(function(t,i){var o=e.node(t);o.order=i+n,r.forEach(o.selfEdges,(function(t){g.addDummyNode(e,"selfedge",{width:t.label.width,height:t.label.height,rank:o.rank,order:i+ ++n,e:t.e,label:t.label},"_se")})),delete o.selfEdges}))}))}(e)})),t("    adjustCoordinateSystem",(function(){h.adjust(e)})),t("    position",(function(){p(e)})),t("    positionSelfEdges",(function(){!function(e){r.forEach(e.nodes(),(function(t){var n=e.node(t);if("selfedge"===n.dummy){var r=e.node(n.e.v),i=r.x+r.width/2,o=r.y,a=n.x-i,s=r.height/2;e.setEdge(n.e,n.label),e.removeNode(t),n.label.points=[{x:i+2*a/3,y:o-s},{x:i+5*a/6,y:o-s},{x:i+a,y:o},{x:i+5*a/6,y:o+s},{x:i+2*a/3,y:o+s}],n.label.x=n.x,n.label.y=n.y}}))}(e)})),t("    removeBorderNodes",(function(){!function(e){r.forEach(e.nodes(),(function(t){if(e.children(t).length){var n=e.node(t),i=e.node(n.borderTop),o=e.node(n.borderBottom),a=e.node(r.last(n.borderLeft)),s=e.node(r.last(n.borderRight));n.width=Math.abs(s.x-a.x),n.height=Math.abs(o.y-i.y),n.x=a.x+n.width/2,n.y=i.y+n.height/2}})),r.forEach(e.nodes(),(function(t){"border"===e.node(t).dummy&&e.removeNode(t)}))}(e)})),t("    normalize.undo",(function(){o.undo(e)})),t("    fixupEdgeLabelCoords",(function(){!function(e){r.forEach(e.edges(),(function(t){var n=e.edge(t);if(r.has(n,"x"))switch("l"!==n.labelpos&&"r"!==n.labelpos||(n.width-=n.labeloffset),n.labelpos){case"l":n.x-=n.width/2+n.labeloffset;break;case"r":n.x+=n.width/2+n.labeloffset}}))}(e)})),t("    undoCoordinateSystem",(function(){h.undo(e)})),t("    translateGraph",(function(){!function(e){var t=Number.POSITIVE_INFINITY,n=0,i=Number.POSITIVE_INFINITY,o=0,a=e.graph(),s=a.marginx||0,l=a.marginy||0;function u(e){var r=e.x,a=e.y,s=e.width,l=e.height;t=Math.min(t,r-s/2),n=Math.max(n,r+s/2),i=Math.min(i,a-l/2),o=Math.max(o,a+l/2)}r.forEach(e.nodes(),(function(t){u(e.node(t))})),r.forEach(e.edges(),(function(t){var n=e.edge(t);r.has(n,"x")&&u(n)})),t-=s,i-=l,r.forEach(e.nodes(),(function(n){var r=e.node(n);r.x-=t,r.y-=i})),r.forEach(e.edges(),(function(n){var o=e.edge(n);r.forEach(o.points,(function(e){e.x-=t,e.y-=i})),r.has(o,"x")&&(o.x-=t),r.has(o,"y")&&(o.y-=i)})),a.width=n-t+s,a.height=o-i+l}(e)})),t("    assignNodeIntersects",(function(){!function(e){r.forEach(e.edges(),(function(t){var n,r,i=e.edge(t),o=e.node(t.v),a=e.node(t.w);i.points?(n=i.points[0],r=i.points[i.points.length-1]):(i.points=[],n=a,r=o),i.points.unshift(g.intersectRect(o,n)),i.points.push(g.intersectRect(a,r))}))}(e)})),t("    reversePoints",(function(){!function(e){r.forEach(e.edges(),(function(t){var n=e.edge(t);n.reversed&&n.points.reverse()}))}(e)})),t("    acyclic.undo",(function(){i.undo(e)}))}(t,n)})),n("  updateInputGraph",(function(){!function(e,t){r.forEach(e.nodes(),(function(n){var r=e.node(n),i=t.node(n);r&&(r.x=i.x,r.y=i.y,t.children(n).length&&(r.width=i.width,r.height=i.height))})),r.forEach(e.edges(),(function(n){var i=e.edge(n),o=t.edge(n);i.points=o.points,r.has(o,"x")&&(i.x=o.x,i.y=o.y)})),e.graph().width=t.graph().width,e.graph().height=t.graph().height}(e,t)}))}))};var m=["nodesep","edgesep","ranksep","marginx","marginy"],y={ranksep:50,edgesep:20,nodesep:50,rankdir:"tb"},b=["acyclicer","ranker","rankdir","align"],C=["width","height"],A={width:0,height:0},_=["minlen","weight","width","height","labeloffset"],I={minlen:1,weight:1,width:0,height:0,labeloffset:10,labelpos:"r"},x=["labelpos"];function w(e,t){return r.mapValues(r.pick(e,t),Number)}function M(e){var t={};return r.forEach(e,(function(e,n){t[n.toLowerCase()]=e})),t}},function(e,t,n){var r=n(1138);e.exports=function(e){return r(e,5)}},function(e,t,n){var r=n(414),i=n(235),o=n(415),a=n(258),s=Object.prototype,l=s.hasOwnProperty,u=r((function(e,t){e=Object(e);var n=-1,r=t.length,u=r>2?t[2]:void 0;for(u&&o(t[0],t[1],u)&&(r=1);++n<r;)for(var c=t[n],d=a(c),h=-1,f=d.length;++h<f;){var p=d[h],g=e[p];(void 0===g||i(g,s[p])&&!l.call(e,p))&&(e[p]=c[p])}return e}));e.exports=u},function(e,t,n){var r=n(2156)(n(2157));e.exports=r},function(e,t,n){var r=n(183),i=n(174),o=n(189);e.exports=function(e){return function(t,n,a){var s=Object(t);if(!i(t)){var l=r(n,3);t=o(t),n=function(e){return l(s[e],e,s)}}var u=e(t,n,a);return u>-1?s[l?t[u]:u]:void 0}}},function(e,t,n){var r=n(1163),i=n(183),o=n(2158),a=Math.max;e.exports=function(e,t,n){var s=null==e?0:e.length;if(!s)return-1;var l=null==n?0:o(n);return l<0&&(l=a(s+l,0)),r(e,i(t,3),l)}},function(e,t,n){var r=n(1171);e.exports=function(e){var t=r(e),n=t%1;return t==t?n?t-n:t:0}},function(e,t,n){var r=n(130),i=n(259),o=/^\s+|\s+$/g,a=/^[-+]0x[0-9a-f]+$/i,s=/^0b[01]+$/i,l=/^0o[0-7]+$/i,u=parseInt;e.exports=function(e){if("number"==typeof e)return e;if(i(e))return NaN;if(r(e)){var t="function"==typeof e.valueOf?e.valueOf():e;e=r(t)?t+"":t}if("string"!=typeof e)return 0===e?e:+e;e=e.replace(o,"");var n=s.test(e);return n||l.test(e)?u(e.slice(2),n?2:8):a.test(e)?NaN:+e}},function(e,t,n){var r=n(611),i=n(1148),o=n(258);e.exports=function(e,t){return null==e?e:r(e,i(t),o)}},function(e,t){e.exports=function(e){var t=null==e?0:e.length;return t?e[t-1]:void 0}},function(e,t,n){var r=n(410),i=n(610),o=n(183);e.exports=function(e,t){var n={};return t=o(t,3),i(e,(function(e,i,o){r(n,i,t(e,i,o))})),n}},function(e,t,n){var r=n(615),i=n(2164),o=n(220);e.exports=function(e){return e&&e.length?r(e,o,i):void 0}},function(e,t){e.exports=function(e,t){return e>t}},function(e,t,n){var r=n(2166),i=n(2169)((function(e,t,n){r(e,t,n)}));e.exports=i},function(e,t,n){var r=n(356),i=n(1173),o=n(611),a=n(2167),s=n(130),l=n(258),u=n(1174);e.exports=function e(t,n,c,d,h){t!==n&&o(n,(function(o,l){if(h||(h=new r),s(o))a(t,n,l,c,e,d,h);else{var f=d?d(u(t,l),o,l+"",t,n,h):void 0;void 0===f&&(f=o),i(t,l,f)}}),l)}},function(e,t,n){var r=n(1173),i=n(1140),o=n(1143),a=n(1141),s=n(1144),l=n(282),u=n(81),c=n(1164),d=n(237),h=n(281),f=n(130),p=n(623),g=n(283),v=n(1174),m=n(2168);e.exports=function(e,t,n,y,b,C,A){var _=v(e,n),I=v(t,n),x=A.get(I);if(x)r(e,n,x);else{var w=C?C(_,I,n+"",e,t,A):void 0,M=void 0===w;if(M){var S=u(I),T=!S&&d(I),E=!S&&!T&&g(I);w=I,S||T||E?u(_)?w=_:c(_)?w=a(_):T?(M=!1,w=i(I,!0)):E?(M=!1,w=o(I,!0)):w=[]:p(I)||l(I)?(w=_,l(_)?w=m(_):f(_)&&!h(_)||(w=s(I))):M=!1}M&&(A.set(I,w),b(w,I,y,C,A),A.delete(I)),r(e,n,w)}}},function(e,t,n){var r=n(308),i=n(258);e.exports=function(e){return r(e,i(e))}},function(e,t,n){var r=n(414),i=n(415);e.exports=function(e){return r((function(t,n){var r=-1,o=n.length,a=o>1?n[o-1]:void 0,s=o>2?n[2]:void 0;for(a=e.length>3&&"function"==typeof a?(o--,a):void 0,s&&i(n[0],n[1],s)&&(a=o<3?void 0:a,o=1),t=Object(t);++r<o;){var l=n[r];l&&e(t,l,r,a)}return t}))}},function(e,t,n){var r=n(615),i=n(1175),o=n(220);e.exports=function(e){return e&&e.length?r(e,o,i):void 0}},function(e,t,n){var r=n(615),i=n(183),o=n(1175);e.exports=function(e,t){return e&&e.length?r(e,i(t,2),o):void 0}},function(e,t,n){var r=n(146);e.exports=function(){return r.Date.now()}},function(e,t,n){var r=n(2174),i=n(2177)((function(e,t){return null==e?{}:r(e,t)}));e.exports=i},function(e,t,n){var r=n(2175),i=n(1153);e.exports=function(e,t){return r(e,t,(function(t,n){return i(e,n)}))}},function(e,t,n){var r=n(612),i=n(2176),o=n(412);e.exports=function(e,t,n){for(var a=-1,s=t.length,l={};++a<s;){var u=t[a],c=r(e,u);n(c,u)&&i(l,o(u,e),c)}return l}},function(e,t,n){var r=n(409),i=n(412),o=n(361),a=n(130),s=n(309);e.exports=function(e,t,n,l){if(!a(e))return e;for(var u=-1,c=(t=i(t,e)).length,d=c-1,h=e;null!=h&&++u<c;){var f=s(t[u]),p=n;if(u!=d){var g=h[f];void 0===(p=l?l(g,f,h):void 0)&&(p=a(g)?g:o(t[u+1])?[]:{})}r(h,f,p),h=h[f]}return e}},function(e,t,n){var r=n(1172),i=n(1161),o=n(1162);e.exports=function(e){return o(i(e,void 0,r),e+"")}},function(e,t,n){var r=n(2179)();e.exports=r},function(e,t,n){var r=n(2180),i=n(415),o=n(1171);e.exports=function(e){return function(t,n,a){return a&&"number"!=typeof a&&i(t,n,a)&&(n=a=void 0),t=o(t),void 0===n?(n=t,t=0):n=o(n),a=void 0===a?t<n?1:-1:o(a),r(t,n,a,e)}}},function(e,t){var n=Math.ceil,r=Math.max;e.exports=function(e,t,i,o){for(var a=-1,s=r(n((t-e)/(i||1)),0),l=Array(s);s--;)l[o?s:++a]=e,e+=i;return l}},function(e,t,n){var r=n(614),i=n(2182),o=n(414),a=n(415),s=o((function(e,t){if(null==e)return[];var n=t.length;return n>1&&a(e,t[0],t[1])?t=[]:n>2&&a(t[0],t[1],t[2])&&(t=[t[0]]),i(e,r(t,1),[])}));e.exports=s},function(e,t,n){var r=n(413),i=n(183),o=n(1159),a=n(2183),s=n(362),l=n(2184),u=n(220);e.exports=function(e,t,n){var c=-1;t=r(t.length?t:[u],s(i));var d=o(e,(function(e,n,i){return{criteria:r(t,(function(t){return t(e)})),index:++c,value:e}}));return a(d,(function(e,t){return l(e,t,n)}))}},function(e,t){e.exports=function(e,t){var n=e.length;for(e.sort(t);n--;)e[n]=e[n].value;return e}},function(e,t,n){var r=n(2185);e.exports=function(e,t,n){for(var i=-1,o=e.criteria,a=t.criteria,s=o.length,l=n.length;++i<s;){var u=r(o[i],a[i]);if(u)return i>=l?u:u*("desc"==n[i]?-1:1)}return e.index-t.index}},function(e,t,n){var r=n(259);e.exports=function(e,t){if(e!==t){var n=void 0!==e,i=null===e,o=e==e,a=r(e),s=void 0!==t,l=null===t,u=t==t,c=r(t);if(!l&&!c&&!a&&e>t||a&&s&&u&&!l&&!c||i&&s&&u||!n&&u||!o)return 1;if(!i&&!a&&!c&&e<t||c&&n&&o&&!i&&!a||l&&n&&o||!s&&o||!u)return-1}return 0}},function(e,t,n){var r=n(1152),i=0;e.exports=function(e){var t=++i;return r(e)+t}},function(e,t,n){var r=n(409),i=n(2188);e.exports=function(e,t){return i(e||[],t||[],r)}},function(e,t){e.exports=function(e,t,n){for(var r=-1,i=e.length,o=t.length,a={};++r<i;){var s=r<o?t[r]:void 0;n(a,e[r],s)}return a}},function(e,t,n){"use strict";var r=n(70),i=n(2190);e.exports={run:function(e){var t="greedy"===e.graph().acyclicer?i(e,function(e){return function(t){return e.edge(t).weight}}(e)):function(e){var t=[],n={},i={};function o(a){r.has(i,a)||(i[a]=!0,n[a]=!0,r.forEach(e.outEdges(a),(function(e){r.has(n,e.w)?t.push(e):o(e.w)})),delete n[a])}return r.forEach(e.nodes(),o),t}(e);r.forEach(t,(function(t){var n=e.edge(t);e.removeEdge(t),n.forwardName=t.name,n.reversed=!0,e.setEdge(t.w,t.v,n,r.uniqueId("rev"))}))},undo:function(e){r.forEach(e.edges(),(function(t){var n=e.edge(t);if(n.reversed){e.removeEdge(t);var r=n.forwardName;delete n.reversed,delete n.forwardName,e.setEdge(t.w,t.v,n,r)}}))}}},function(e,t,n){var r=n(70),i=n(153).Graph,o=n(2191);e.exports=function(e,t){if(e.nodeCount()<=1)return[];var n=function(e,t){var n=new i,a=0,s=0;r.forEach(e.nodes(),(function(e){n.setNode(e,{v:e,in:0,out:0})})),r.forEach(e.edges(),(function(e){var r=n.edge(e.v,e.w)||0,i=t(e),o=r+i;n.setEdge(e.v,e.w,o),s=Math.max(s,n.node(e.v).out+=i),a=Math.max(a,n.node(e.w).in+=i)}));var u=r.range(s+a+3).map((function(){return new o})),c=a+1;return r.forEach(n.nodes(),(function(e){l(u,c,n.node(e))})),{graph:n,buckets:u,zeroIdx:c}}(e,t||a),u=function(e,t,n){var r,i=[],o=t[t.length-1],a=t[0];for(;e.nodeCount();){for(;r=a.dequeue();)s(e,t,n,r);for(;r=o.dequeue();)s(e,t,n,r);if(e.nodeCount())for(var l=t.length-2;l>0;--l)if(r=t[l].dequeue()){i=i.concat(s(e,t,n,r,!0));break}}return i}(n.graph,n.buckets,n.zeroIdx);return r.flatten(r.map(u,(function(t){return e.outEdges(t.v,t.w)})),!0)};var a=r.constant(1);function s(e,t,n,i,o){var a=o?[]:void 0;return r.forEach(e.inEdges(i.v),(function(r){var i=e.edge(r),s=e.node(r.v);o&&a.push({v:r.v,w:r.w}),s.out-=i,l(t,n,s)})),r.forEach(e.outEdges(i.v),(function(r){var i=e.edge(r),o=r.w,a=e.node(o);a.in-=i,l(t,n,a)})),e.removeNode(i.v),a}function l(e,t,n){n.out?n.in?e[n.out-n.in+t].enqueue(n):e[e.length-1].enqueue(n):e[0].enqueue(n)}},function(e,t){function n(){var e={};e._next=e._prev=e,this._sentinel=e}function r(e){e._prev._next=e._next,e._next._prev=e._prev,delete e._next,delete e._prev}function i(e,t){if("_next"!==e&&"_prev"!==e)return t}e.exports=n,n.prototype.dequeue=function(){var e=this._sentinel,t=e._prev;if(t!==e)return r(t),t},n.prototype.enqueue=function(e){var t=this._sentinel;e._prev&&e._next&&r(e),e._next=t._next,t._next._prev=e,t._next=e,e._prev=t},n.prototype.toString=function(){for(var e=[],t=this._sentinel,n=t._prev;n!==t;)e.push(JSON.stringify(n,i)),n=n._prev;return"["+e.join(", ")+"]"}},function(e,t,n){"use strict";var r=n(70),i=n(113);e.exports={run:function(e){e.graph().dummyChains=[],r.forEach(e.edges(),(function(t){!function(e,t){var n,r,o,a=t.v,s=e.node(a).rank,l=t.w,u=e.node(l).rank,c=t.name,d=e.edge(t),h=d.labelRank;if(u===s+1)return;for(e.removeEdge(t),o=0,++s;s<u;++o,++s)d.points=[],r={width:0,height:0,edgeLabel:d,edgeObj:t,rank:s},n=i.addDummyNode(e,"edge",r,"_d"),s===h&&(r.width=d.width,r.height=d.height,r.dummy="edge-label",r.labelpos=d.labelpos),e.setEdge(a,n,{weight:d.weight},c),0===o&&e.graph().dummyChains.push(n),a=n;e.setEdge(a,l,{weight:d.weight},c)}(e,t)}))},undo:function(e){r.forEach(e.graph().dummyChains,(function(t){var n,r=e.node(t),i=r.edgeLabel;for(e.setEdge(r.edgeObj,i);r.dummy;)n=e.successors(t)[0],e.removeNode(t),i.points.push({x:r.x,y:r.y}),"edge-label"===r.dummy&&(i.x=r.x,i.y=r.y,i.width=r.width,i.height=r.height),t=n,r=e.node(t)}))}}},function(e,t,n){"use strict";var r=n(416).longestPath,i=n(1176),o=n(2194);e.exports=function(e){switch(e.graph().ranker){case"network-simplex":s(e);break;case"tight-tree":!function(e){r(e),i(e)}(e);break;case"longest-path":a(e);break;default:s(e)}};var a=r;function s(e){o(e)}},function(e,t,n){"use strict";var r=n(70),i=n(1176),o=n(416).slack,a=n(416).longestPath,s=n(153).alg.preorder,l=n(153).alg.postorder,u=n(113).simplify;function c(e){e=u(e),a(e);var t,n=i(e);for(f(n),d(n,e);t=g(n);)m(n,e,t,v(n,e,t))}function d(e,t){var n=l(e,e.nodes());n=n.slice(0,n.length-1),r.forEach(n,(function(n){!function(e,t,n){var r=e.node(n).parent;e.edge(n,r).cutvalue=h(e,t,n)}(e,t,n)}))}function h(e,t,n){var i=e.node(n).parent,o=!0,a=t.edge(n,i),s=0;return a||(o=!1,a=t.edge(i,n)),s=a.weight,r.forEach(t.nodeEdges(n),(function(r){var a,l,u=r.v===n,c=u?r.w:r.v;if(c!==i){var d=u===o,h=t.edge(r).weight;if(s+=d?h:-h,a=n,l=c,e.hasEdge(a,l)){var f=e.edge(n,c).cutvalue;s+=d?-f:f}}})),s}function f(e,t){arguments.length<2&&(t=e.nodes()[0]),p(e,{},1,t)}function p(e,t,n,i,o){var a=n,s=e.node(i);return t[i]=!0,r.forEach(e.neighbors(i),(function(o){r.has(t,o)||(n=p(e,t,n,o,i))})),s.low=a,s.lim=n++,o?s.parent=o:delete s.parent,n}function g(e){return r.find(e.edges(),(function(t){return e.edge(t).cutvalue<0}))}function v(e,t,n){var i=n.v,a=n.w;t.hasEdge(i,a)||(i=n.w,a=n.v);var s=e.node(i),l=e.node(a),u=s,c=!1;s.lim>l.lim&&(u=l,c=!0);var d=r.filter(t.edges(),(function(t){return c===y(e,e.node(t.v),u)&&c!==y(e,e.node(t.w),u)}));return r.minBy(d,(function(e){return o(t,e)}))}function m(e,t,n,i){var o=n.v,a=n.w;e.removeEdge(o,a),e.setEdge(i.v,i.w,{}),f(e),d(e,t),function(e,t){var n=r.find(e.nodes(),(function(e){return!t.node(e).parent})),i=s(e,n);i=i.slice(1),r.forEach(i,(function(n){var r=e.node(n).parent,i=t.edge(n,r),o=!1;i||(i=t.edge(r,n),o=!0),t.node(n).rank=t.node(r).rank+(o?i.minlen:-i.minlen)}))}(e,t)}function y(e,t,n){return n.low<=t.lim&&t.lim<=n.lim}e.exports=c,c.initLowLimValues=f,c.initCutValues=d,c.calcCutValue=h,c.leaveEdge=g,c.enterEdge=v,c.exchangeEdges=m},function(e,t,n){var r=n(70);e.exports=function(e){var t=function(e){var t={},n=0;function i(o){var a=n;r.forEach(e.children(o),i),t[o]={low:a,lim:n++}}return r.forEach(e.children(),i),t}(e);r.forEach(e.graph().dummyChains,(function(n){for(var r=e.node(n),i=r.edgeObj,o=function(e,t,n,r){var i,o,a=[],s=[],l=Math.min(t[n].low,t[r].low),u=Math.max(t[n].lim,t[r].lim);i=n;do{i=e.parent(i),a.push(i)}while(i&&(t[i].low>l||u>t[i].lim));o=i,i=r;for(;(i=e.parent(i))!==o;)s.push(i);return{path:a.concat(s.reverse()),lca:o}}(e,t,i.v,i.w),a=o.path,s=o.lca,l=0,u=a[l],c=!0;n!==i.w;){if(r=e.node(n),c){for(;(u=a[l])!==s&&e.node(u).maxRank<r.rank;)l++;u===s&&(c=!1)}if(!c){for(;l<a.length-1&&e.node(u=a[l+1]).minRank<=r.rank;)l++;u=a[l]}e.setParent(n,u),n=e.successors(n)[0]}}))}},function(e,t,n){var r=n(70),i=n(113);e.exports={run:function(e){var t=i.addDummyNode(e,"root",{},"_root"),n=function(e){var t={};return r.forEach(e.children(),(function(n){!function n(i,o){var a=e.children(i);a&&a.length&&r.forEach(a,(function(e){n(e,o+1)}));t[i]=o}(n,1)})),t}(e),o=r.max(r.values(n))-1,a=2*o+1;e.graph().nestingRoot=t,r.forEach(e.edges(),(function(t){e.edge(t).minlen*=a}));var s=function(e){return r.reduce(e.edges(),(function(t,n){return t+e.edge(n).weight}),0)}(e)+1;r.forEach(e.children(),(function(l){!function e(t,n,o,a,s,l,u){var c=t.children(u);if(!c.length)return void(u!==n&&t.setEdge(n,u,{weight:0,minlen:o}));var d=i.addBorderNode(t,"_bt"),h=i.addBorderNode(t,"_bb"),f=t.node(u);t.setParent(d,u),f.borderTop=d,t.setParent(h,u),f.borderBottom=h,r.forEach(c,(function(r){e(t,n,o,a,s,l,r);var i=t.node(r),c=i.borderTop?i.borderTop:r,f=i.borderBottom?i.borderBottom:r,p=i.borderTop?a:2*a,g=c!==f?1:s-l[u]+1;t.setEdge(d,c,{weight:p,minlen:g,nestingEdge:!0}),t.setEdge(f,h,{weight:p,minlen:g,nestingEdge:!0})})),t.parent(u)||t.setEdge(n,d,{weight:0,minlen:s+l[u]})}(e,t,a,s,o,n,l)})),e.graph().nodeRankFactor=a},cleanup:function(e){var t=e.graph();e.removeNode(t.nestingRoot),delete t.nestingRoot,r.forEach(e.edges(),(function(t){e.edge(t).nestingEdge&&e.removeEdge(t)}))}}},function(e,t,n){var r=n(70),i=n(113);function o(e,t,n,r,o,a){var s={width:0,height:0,rank:a,borderType:t},l=o[t][a-1],u=i.addDummyNode(e,"border",s,n);o[t][a]=u,e.setParent(u,r),l&&e.setEdge(l,u,{weight:1})}e.exports=function(e){r.forEach(e.children(),(function t(n){var i=e.children(n),a=e.node(n);if(i.length&&r.forEach(i,t),r.has(a,"minRank")){a.borderLeft=[],a.borderRight=[];for(var s=a.minRank,l=a.maxRank+1;s<l;++s)o(e,"borderLeft","_bl",n,a,s),o(e,"borderRight","_br",n,a,s)}}))}},function(e,t,n){"use strict";var r=n(70);function i(e){r.forEach(e.nodes(),(function(t){o(e.node(t))})),r.forEach(e.edges(),(function(t){o(e.edge(t))}))}function o(e){var t=e.width;e.width=e.height,e.height=t}function a(e){e.y=-e.y}function s(e){var t=e.x;e.x=e.y,e.y=t}e.exports={adjust:function(e){var t=e.graph().rankdir.toLowerCase();"lr"!==t&&"rl"!==t||i(e)},undo:function(e){var t=e.graph().rankdir.toLowerCase();"bt"!==t&&"rl"!==t||function(e){r.forEach(e.nodes(),(function(t){a(e.node(t))})),r.forEach(e.edges(),(function(t){var n=e.edge(t);r.forEach(n.points,a),r.has(n,"y")&&a(n)}))}(e);"lr"!==t&&"rl"!==t||(!function(e){r.forEach(e.nodes(),(function(t){s(e.node(t))})),r.forEach(e.edges(),(function(t){var n=e.edge(t);r.forEach(n.points,s),r.has(n,"x")&&s(n)}))}(e),i(e))}}},function(e,t,n){"use strict";var r=n(70),i=n(2200),o=n(2201),a=n(2202),s=n(2206),l=n(2207),u=n(153).Graph,c=n(113);function d(e,t,n){return r.map(t,(function(t){return s(e,t,n)}))}function h(e,t){var n=new u;r.forEach(e,(function(e){var i=e.graph().root,o=a(e,i,n,t);r.forEach(o.vs,(function(t,n){e.node(t).order=n})),l(e,n,o.vs)}))}function f(e,t){r.forEach(t,(function(t){r.forEach(t,(function(t,n){e.node(t).order=n}))}))}e.exports=function(e){var t=c.maxRank(e),n=d(e,r.range(1,t+1),"inEdges"),a=d(e,r.range(t-1,-1,-1),"outEdges"),s=i(e);f(e,s);for(var l,u=Number.POSITIVE_INFINITY,p=0,g=0;g<4;++p,++g){h(p%2?n:a,p%4>=2),s=c.buildLayerMatrix(e);var v=o(e,s);v<u&&(g=0,l=r.cloneDeep(s),u=v)}f(e,l)}},function(e,t,n){"use strict";var r=n(70);e.exports=function(e){var t={},n=r.filter(e.nodes(),(function(t){return!e.children(t).length})),i=r.max(r.map(n,(function(t){return e.node(t).rank}))),o=r.map(r.range(i+1),(function(){return[]}));var a=r.sortBy(n,(function(t){return e.node(t).rank}));return r.forEach(a,(function n(i){if(r.has(t,i))return;t[i]=!0;var a=e.node(i);o[a.rank].push(i),r.forEach(e.successors(i),n)})),o}},function(e,t,n){"use strict";var r=n(70);function i(e,t,n){for(var i=r.zipObject(n,r.map(n,(function(e,t){return t}))),o=r.flatten(r.map(t,(function(t){return r.sortBy(r.map(e.outEdges(t),(function(t){return{pos:i[t.w],weight:e.edge(t).weight}})),"pos")})),!0),a=1;a<n.length;)a<<=1;var s=2*a-1;a-=1;var l=r.map(new Array(s),(function(){return 0})),u=0;return r.forEach(o.forEach((function(e){var t=e.pos+a;l[t]+=e.weight;for(var n=0;t>0;)t%2&&(n+=l[t+1]),l[t=t-1>>1]+=e.weight;u+=e.weight*n}))),u}e.exports=function(e,t){for(var n=0,r=1;r<t.length;++r)n+=i(e,t[r-1],t[r]);return n}},function(e,t,n){var r=n(70),i=n(2203),o=n(2204),a=n(2205);e.exports=function e(t,n,s,l){var u=t.children(n),c=t.node(n),d=c?c.borderLeft:void 0,h=c?c.borderRight:void 0,f={};d&&(u=r.filter(u,(function(e){return e!==d&&e!==h})));var p=i(t,u);r.forEach(p,(function(n){if(t.children(n.v).length){var i=e(t,n.v,s,l);f[n.v]=i,r.has(i,"barycenter")&&(o=n,a=i,r.isUndefined(o.barycenter)?(o.barycenter=a.barycenter,o.weight=a.weight):(o.barycenter=(o.barycenter*o.weight+a.barycenter*a.weight)/(o.weight+a.weight),o.weight+=a.weight))}var o,a}));var g=o(p,s);!function(e,t){r.forEach(e,(function(e){e.vs=r.flatten(e.vs.map((function(e){return t[e]?t[e].vs:e})),!0)}))}(g,f);var v=a(g,l);if(d&&(v.vs=r.flatten([d,v.vs,h],!0),t.predecessors(d).length)){var m=t.node(t.predecessors(d)[0]),y=t.node(t.predecessors(h)[0]);r.has(v,"barycenter")||(v.barycenter=0,v.weight=0),v.barycenter=(v.barycenter*v.weight+m.order+y.order)/(v.weight+2),v.weight+=2}return v}},function(e,t,n){var r=n(70);e.exports=function(e,t){return r.map(t,(function(t){var n=e.inEdges(t);if(n.length){var i=r.reduce(n,(function(t,n){var r=e.edge(n),i=e.node(n.v);return{sum:t.sum+r.weight*i.order,weight:t.weight+r.weight}}),{sum:0,weight:0});return{v:t,barycenter:i.sum/i.weight,weight:i.weight}}return{v:t}}))}},function(e,t,n){"use strict";var r=n(70);e.exports=function(e,t){var n={};return r.forEach(e,(function(e,t){var i=n[e.v]={indegree:0,in:[],out:[],vs:[e.v],i:t};r.isUndefined(e.barycenter)||(i.barycenter=e.barycenter,i.weight=e.weight)})),r.forEach(t.edges(),(function(e){var t=n[e.v],i=n[e.w];r.isUndefined(t)||r.isUndefined(i)||(i.indegree++,t.out.push(n[e.w]))})),function(e){var t=[];function n(e){return function(t){t.merged||(r.isUndefined(t.barycenter)||r.isUndefined(e.barycenter)||t.barycenter>=e.barycenter)&&function(e,t){var n=0,r=0;e.weight&&(n+=e.barycenter*e.weight,r+=e.weight);t.weight&&(n+=t.barycenter*t.weight,r+=t.weight);e.vs=t.vs.concat(e.vs),e.barycenter=n/r,e.weight=r,e.i=Math.min(t.i,e.i),t.merged=!0}(e,t)}}function i(t){return function(n){n.in.push(t),0==--n.indegree&&e.push(n)}}for(;e.length;){var o=e.pop();t.push(o),r.forEach(o.in.reverse(),n(o)),r.forEach(o.out,i(o))}return r.map(r.filter(t,(function(e){return!e.merged})),(function(e){return r.pick(e,["vs","i","barycenter","weight"])}))}(r.filter(n,(function(e){return!e.indegree})))}},function(e,t,n){var r=n(70),i=n(113);function o(e,t,n){for(var i;t.length&&(i=r.last(t)).i<=n;)t.pop(),e.push(i.vs),n++;return n}e.exports=function(e,t){var n=i.partition(e,(function(e){return r.has(e,"barycenter")})),a=n.lhs,s=r.sortBy(n.rhs,(function(e){return-e.i})),l=[],u=0,c=0,d=0;a.sort((h=!!t,function(e,t){return e.barycenter<t.barycenter?-1:e.barycenter>t.barycenter?1:h?t.i-e.i:e.i-t.i})),d=o(l,s,d),r.forEach(a,(function(e){d+=e.vs.length,l.push(e.vs),u+=e.barycenter*e.weight,c+=e.weight,d=o(l,s,d)}));var h;var f={vs:r.flatten(l,!0)};c&&(f.barycenter=u/c,f.weight=c);return f}},function(e,t,n){var r=n(70),i=n(153).Graph;e.exports=function(e,t,n){var o=function(e){var t;for(;e.hasNode(t=r.uniqueId("_root")););return t}(e),a=new i({compound:!0}).setGraph({root:o}).setDefaultNodeLabel((function(t){return e.node(t)}));return r.forEach(e.nodes(),(function(i){var s=e.node(i),l=e.parent(i);(s.rank===t||s.minRank<=t&&t<=s.maxRank)&&(a.setNode(i),a.setParent(i,l||o),r.forEach(e[n](i),(function(t){var n=t.v===i?t.w:t.v,o=a.edge(n,i),s=r.isUndefined(o)?0:o.weight;a.setEdge(n,i,{weight:e.edge(t).weight+s})})),r.has(s,"minRank")&&a.setNode(i,{borderLeft:s.borderLeft[t],borderRight:s.borderRight[t]}))})),a}},function(e,t,n){var r=n(70);e.exports=function(e,t,n){var i,o={};r.forEach(n,(function(n){for(var r,a,s=e.parent(n);s;){if((r=e.parent(s))?(a=o[r],o[r]=s):(a=i,i=s),a&&a!==s)return void t.setEdge(a,s);s=r}}))}},function(e,t,n){"use strict";var r=n(70),i=n(113),o=n(2209).positionX;e.exports=function(e){(function(e){var t=i.buildLayerMatrix(e),n=e.graph().ranksep,o=0;r.forEach(t,(function(t){var i=r.max(r.map(t,(function(t){return e.node(t).height})));r.forEach(t,(function(t){e.node(t).y=o+i/2})),o+=i+n}))})(e=i.asNonCompoundGraph(e)),r.forEach(o(e),(function(t,n){e.node(n).x=t}))}},function(e,t,n){"use strict";var r=n(70),i=n(153).Graph,o=n(113);function a(e,t){var n={};return r.reduce(t,(function(t,i){var o=0,a=0,s=t.length,u=r.last(i);return r.forEach(i,(function(t,c){var d=function(e,t){if(e.node(t).dummy)return r.find(e.predecessors(t),(function(t){return e.node(t).dummy}))}(e,t),h=d?e.node(d).order:s;(d||t===u)&&(r.forEach(i.slice(a,c+1),(function(t){r.forEach(e.predecessors(t),(function(r){var i=e.node(r),a=i.order;!(a<o||h<a)||i.dummy&&e.node(t).dummy||l(n,r,t)}))})),a=c+1,o=h)})),i})),n}function s(e,t){var n={};function i(t,i,o,a,s){var u;r.forEach(r.range(i,o),(function(i){u=t[i],e.node(u).dummy&&r.forEach(e.predecessors(u),(function(t){var r=e.node(t);r.dummy&&(r.order<a||r.order>s)&&l(n,t,u)}))}))}return r.reduce(t,(function(t,n){var o,a=-1,s=0;return r.forEach(n,(function(r,l){if("border"===e.node(r).dummy){var u=e.predecessors(r);u.length&&(o=e.node(u[0]).order,i(n,s,l,a,o),s=l,a=o)}i(n,s,n.length,o,t.length)})),n})),n}function l(e,t,n){if(t>n){var r=t;t=n,n=r}var i=e[t];i||(e[t]=i={}),i[n]=!0}function u(e,t,n){if(t>n){var i=t;t=n,n=i}return r.has(e[t],n)}function c(e,t,n,i){var o={},a={},s={};return r.forEach(t,(function(e){r.forEach(e,(function(e,t){o[e]=e,a[e]=e,s[e]=t}))})),r.forEach(t,(function(e){var t=-1;r.forEach(e,(function(e){var l=i(e);if(l.length)for(var c=((l=r.sortBy(l,(function(e){return s[e]}))).length-1)/2,d=Math.floor(c),h=Math.ceil(c);d<=h;++d){var f=l[d];a[e]===e&&t<s[f]&&!u(n,e,f)&&(a[f]=e,a[e]=o[e]=o[f],t=s[f])}}))})),{root:o,align:a}}function d(e,t,n,o,a){var s={},l=function(e,t,n,o){var a=new i,s=e.graph(),l=function(e,t,n){return function(i,o,a){var s,l=i.node(o),u=i.node(a),c=0;if(c+=l.width/2,r.has(l,"labelpos"))switch(l.labelpos.toLowerCase()){case"l":s=-l.width/2;break;case"r":s=l.width/2}if(s&&(c+=n?s:-s),s=0,c+=(l.dummy?t:e)/2,c+=(u.dummy?t:e)/2,c+=u.width/2,r.has(u,"labelpos"))switch(u.labelpos.toLowerCase()){case"l":s=u.width/2;break;case"r":s=-u.width/2}return s&&(c+=n?s:-s),s=0,c}}(s.nodesep,s.edgesep,o);return r.forEach(t,(function(t){var i;r.forEach(t,(function(t){var r=n[t];if(a.setNode(r),i){var o=n[i],s=a.edge(o,r);a.setEdge(o,r,Math.max(l(e,t,i),s||0))}i=t}))})),a}(e,t,n,a),u=a?"borderLeft":"borderRight";function c(e,t){for(var n=l.nodes(),r=n.pop(),i={};r;)i[r]?e(r):(i[r]=!0,n.push(r),n=n.concat(t(r))),r=n.pop()}return c((function(e){s[e]=l.inEdges(e).reduce((function(e,t){return Math.max(e,s[t.v]+l.edge(t))}),0)}),l.predecessors.bind(l)),c((function(t){var n=l.outEdges(t).reduce((function(e,t){return Math.min(e,s[t.w]-l.edge(t))}),Number.POSITIVE_INFINITY),r=e.node(t);n!==Number.POSITIVE_INFINITY&&r.borderType!==u&&(s[t]=Math.max(s[t],n))}),l.successors.bind(l)),r.forEach(o,(function(e){s[e]=s[n[e]]})),s}function h(e,t){return r.minBy(r.values(t),(function(t){var n=Number.NEGATIVE_INFINITY,i=Number.POSITIVE_INFINITY;return r.forIn(t,(function(t,r){var o=function(e,t){return e.node(t).width}(e,r)/2;n=Math.max(t+o,n),i=Math.min(t-o,i)})),n-i}))}function f(e,t){var n=r.values(t),i=r.min(n),o=r.max(n);r.forEach(["u","d"],(function(n){r.forEach(["l","r"],(function(a){var s,l=n+a,u=e[l];if(u!==t){var c=r.values(u);(s="l"===a?i-r.min(c):o-r.max(c))&&(e[l]=r.mapValues(u,(function(e){return e+s})))}}))}))}function p(e,t){return r.mapValues(e.ul,(function(n,i){if(t)return e[t.toLowerCase()][i];var o=r.sortBy(r.map(e,i));return(o[1]+o[2])/2}))}e.exports={positionX:function(e){var t,n=o.buildLayerMatrix(e),i=r.merge(a(e,n),s(e,n)),l={};r.forEach(["u","d"],(function(o){t="u"===o?n:r.values(n).reverse(),r.forEach(["l","r"],(function(n){"r"===n&&(t=r.map(t,(function(e){return r.values(e).reverse()})));var a=("u"===o?e.predecessors:e.successors).bind(e),s=c(e,t,i,a),u=d(e,t,s.root,s.align,"r"===n);"r"===n&&(u=r.mapValues(u,(function(e){return-e}))),l[o+n]=u}))}));var u=h(e,l);return f(l,u),p(l,e.graph().align)},findType1Conflicts:a,findType2Conflicts:s,addConflict:l,hasConflict:u,verticalAlignment:c,horizontalCompaction:d,alignCoordinates:f,findSmallestWidthAlignment:h,balance:p}},function(e,t,n){var r=n(70),i=n(113),o=n(153).Graph;e.exports={debugOrdering:function(e){var t=i.buildLayerMatrix(e),n=new o({compound:!0,multigraph:!0}).setGraph({});return r.forEach(e.nodes(),(function(t){n.setNode(t,{label:t}),n.setParent(t,"layer"+e.node(t).rank)})),r.forEach(e.edges(),(function(e){n.setEdge(e.v,e.w,{},e.name)})),r.forEach(t,(function(e,t){var i="layer"+t;n.setNode(i,{rank:"same"}),r.reduce(e,(function(e,t){return n.setEdge(e,t,{style:"invis"}),t}))})),n}}},function(e,t){e.exports="0.8.5"},function(e,t,n){var r,i,o=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),a=this&&this.__setModuleDefault||(Object.create?function(e,t){Object.defineProperty(e,"default",{enumerable:!0,value:t})}:function(e,t){e.default=t}),s=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)"default"!==n&&Object.hasOwnProperty.call(e,n)&&o(t,e,n);return a(t,e),t};r=[n,t,n(0)],void 0===(i=function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.BaseCanvasWidget=void 0,n=s(n);class r extends n.Component{render(){return n.createElement("div",{className:"graphView"},this.props.children)}}t.BaseCanvasWidget=r}.apply(t,r))||(e.exports=i)},function(e,t,n){var r,i,o=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};r=[n,t,n(151),n(1116),n(152),n(0)],void 0===(i=function(e,t,n,r,i,a){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.AdvancedLinkWidget=t.CustomNodeFactory=t.CustomNodeModel=t.CustomPortModel=t.AdvancedLinkFactory=t.AdvancedLinkModel=void 0,a=o(a);class s extends r.DefaultLinkModel{constructor(){super({type:"advanced",width:2}),this.isVisible=()=>this._visible,this._visible=!0}serialize(){if(this.getSourcePort()&&this.getTargetPort()){const e=this.getSourcePort().getParent(),t=this.getTargetPort().getParent();this._visible=e.isVisible()&&t.isVisible()}else this._visible=!0;return Object.assign(Object.assign({},super.serialize()),{visible:this._visible})}deserialize(e){super.deserialize(e),this._visible=e.data.visible}}t.AdvancedLinkModel=s;class l extends r.DefaultLinkFactory{constructor(){super("advanced")}generateModel(){return new s}generateReactWidget(e){return a.default.createElement(g,{link:e.model,diagramEngine:this.engine})}}t.AdvancedLinkFactory=l;class u extends r.DefaultPortModel{canLinkToPort(e){return!(e instanceof u)||!(!this.options.in&&!e.getOptions().in)}createLinkModel(e){return new s}}t.CustomPortModel=u;class c extends r.DefaultNodeModel{constructor(e,t,n=!0){super(e,t),this.setVisible=e=>{this._visible=e},this.isVisible=()=>this._visible,this._visible=n}serialize(){return Object.assign(Object.assign({},super.serialize()),{visible:this._visible})}deserialize(e){super.deserialize(e),this._visible=e.data.visible}}t.CustomNodeModel=c;class d extends r.DefaultNodeFactory{generateReactWidget(e){return a.default.createElement(h,{engine:this.engine,node:e.model})}}t.CustomNodeFactory=d;class h extends r.DefaultNodeWidget{constructor(){super(...arguments),this.generatePort=e=>a.default.createElement(f,{engine:this.props.engine,port:e,key:e.getID()})}render(){const e=super.render(),t=this.props.node;return a.default.createElement("div",{style:{display:t.isVisible()?"unset":"none"}},e)}}class f extends a.default.Component{render(){const e=a.default.createElement(i.PortWidget,{engine:this.props.engine,port:this.props.port},a.default.createElement("div",{className:"S_Port"})),t=a.default.createElement(i.PortWidget,{engine:this.props.engine,port:this.props.port},a.default.createElement("div",{className:"S_OutPort"})),n=a.default.createElement("div",{style:{padding:"0 5px",flexGrow:1}},this.props.port.getOptions().label);return a.default.createElement("div",{style:{display:"flex",marginTop:"1px",alignItems:"center"}},this.props.port.getOptions().in?e:n,this.props.port.getOptions().in?n:t)}}const p=e=>{const{point:t,previousPoint:n}=e,r=90+180*Math.atan2(t.getPosition().y-n.getPosition().y,t.getPosition().x-n.getPosition().x)/Math.PI;return a.default.createElement("g",{className:"arrow",transform:"translate("+t.getPosition().x+", "+t.getPosition().y+")"},a.default.createElement("g",{style:{transform:"rotate("+r+"deg)"}},a.default.createElement("g",{transform:"translate(0, -3)"},a.default.createElement("polygon",{points:"0,10 5,25 -5,25",fill:e.color,onMouseLeave:()=>{this.setState({selected:!1})},onMouseEnter:()=>{this.setState({selected:!0})},"data-id":t.getID(),"data-linkid":t.getLink().getID()}))))};class g extends r.DefaultLinkWidget{constructor(){super(...arguments),this.renderStatus=!1}generateArrow(e,t){return a.default.createElement(p,{key:e.getID(),point:e,previousPoint:t,colorSelected:this.props.link.getOptions().selectedColor,color:this.props.link.getOptions().color})}render(){const e=this.props.link.getPoints(),t=[];if(this.refPaths=[],null!==this.props.link.getSourcePort()&&null!==this.props.link.getTargetPort()){let t,r;const o=this.props.link.getSourcePort().getOptions().alignment,a=this.props.link.getTargetPort().getOptions().alignment,s=e[0].getY(),l=e[e.length-1].getY();let u;if("left"===o&&"left"===a?(t=e[0].getX()-50,r=e[e.length-1].getX()-50,u=Math.abs(s-l)<50?(s+l)/2+50:(s+l)/2):"left"===o&&"right"===a?(t=e[0].getX()-50,r=e[e.length-1].getX()+50,u=t>r?(s+l)/2:Math.abs(s-l)<50?(s+l)/2+50:(s+l)/2):"right"===o&&"left"===a?(t=e[0].getX()+50,r=e[e.length-1].getX()-50,u=t<r?(s+l)/2:Math.abs(s-l)<50?(s+l)/2+50:(s+l)/2):"right"===o&&"right"===a&&(t=e[0].getX()+50,r=e[e.length-1].getX()+50,u=Math.abs(s-l)<50?(s+l)/2+50:(s+l)/2),2===e.length){const e=new i.PointModel({link:this.props.link,position:new n.Point(t,s)}),o=new i.PointModel({link:this.props.link,position:new n.Point(r,l)});this.props.link.addPoint(e,1),this.props.link.addPoint(o,-1);const a=new i.PointModel({link:this.props.link,position:new n.Point(t,u)}),c=new i.PointModel({link:this.props.link,position:new n.Point(r,u)});this.props.link.addPoint(a,2),this.props.link.addPoint(c,-2)}else e.length>3&&(e[1].setPosition(new n.Point(t,s)),e[e.length-2].setPosition(new n.Point(r,l)),e[2].setPosition(new n.Point(t,u)),e[e.length-3].setPosition(new n.Point(r,u)),e.length>6&&e.splice(3,e.length-6))}for(let n=0;n<e.length-1;n++)t.push(this.generateLink(i.LinkWidget.generateLinePath(e[n],e[n+1]),{"data-linkid":this.props.link.getID(),"data-point":n,onMouseDown:e=>{}},n));null!==this.props.link.getTargetPort()?t.push(this.generateArrow(e[e.length-1],e[e.length-2])):t.push(this.generatePoint(e[e.length-1]));const r=this.props.link;return a.default.createElement("g",{style:{display:r.isVisible()?"unset":"none"},"data-default-link-test":this.props.link.getOptions().testName},t)}}t.AdvancedLinkWidget=g}.apply(t,r))||(e.exports=i)},function(e,t,n){var r,i,o=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),a=this&&this.__setModuleDefault||(Object.create?function(e,t){Object.defineProperty(e,"default",{enumerable:!0,value:t})}:function(e,t){e.default=t}),s=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)"default"!==n&&Object.hasOwnProperty.call(e,n)&&o(t,e,n);return a(t,e),t},l=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};r=[n,t,n(90),n(417),n(418),n(419),n(420),n(62),n(2215),n(2216),n(2217),n(2218),n(0),n(78),n(176),n(2219),n(2220)],void 0===(i=function(e,t,n,r,i,o,a,u,c,d,h,f,p,g,v,m,y){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),r=l(r),i=l(i),o=l(o),a=l(a),u=l(u),c=l(c),d=l(d),h=l(h),f=l(f),p=s(p),v=s(v),m=l(m),y=l(y);class b extends p.Component{constructor(e){super(e),this.on_msg=(e,t)=>{const{type:n,payload:r}=e;switch(n){case"Controller::update_signal":{const e=this.state.logMsg+"\n<b>"+(new Date).toLocaleString()+"</b> Done";this.setState(t=>Object.assign(Object.assign({},t),{logMsg:e})),this.props.computingState&&this.props.toggleComputing();break}case"Controller::notification_msg":this.updateLog(r)}},this.componentWillUnmount=()=>{this.props.model.stopListening(this.props.model,"msg:custom",this.on_msg)},this.scrollToBottom=()=>{this.notificationRef.current.scrollTop=this.notificationRef.current.scrollHeight},this.updateLog=e=>{const t=this.state.logMsg+"\n"+(""!==e.log?e.log+"\n":"")+"<b>"+(new Date).toLocaleString()+"</b> "+e.msg;this.setState(e=>Object.assign(Object.assign({},e),{logMsg:t}))},this.printToLog=e=>{const t=this.state.logMsg+"\n<b>"+(new Date).toLocaleString()+"</b> "+e;this.setState(e=>Object.assign(Object.assign({},e),{logMsg:t}))},this.runBtnHandle=()=>{this.props.computingState?this.printToLog("A driver is running, please wait!"):(this.printToLog("Start computing"),this.props.send_msg({action:"Controller::runSignal",payload:this.props.selectedVariable}))},this.clickRunButton=()=>{this.props.computingState||this.props.toggleComputing(),this.setState(e=>Object.assign(Object.assign({},this.state),{expandNotification:!1})),this.runBtnHandle()},this.closeAddVariableDialog=()=>{this.setState(Object.assign(Object.assign({},this.state),{addVariableDialog:!1}))},this.state={treeData:{},expandNotification:!1,addVariableDialog:!1,controllerList:{},logMsg:""},this.notificationRef=p.default.createRef(),e.model.listenTo(e.model,"msg:custom",this.on_msg)}componentDidUpdate(e,t){}static getDerivedStateFromProps(e,t){let n=!1;const r=Object.assign({},t.controllerList),i=Object.keys(t.controllerList),o=Object.keys(e.selectedVariable);return i.length<o.length?o.forEach(t=>{if(!i.includes(t)){n=!0;let i,o,a="-1";const s=/\[([^)]+)\]/.exec(t);s?(i=t.replace(s[0],""),a=s[1],o=e.variableData[i].value[parseInt(a)]):(i=t,o=e.variableData[i].value),r[t]={component:p.default.createElement(y.default,{variableName:t,key:t,value:o}),value:o}}}):i.forEach(e=>{o.includes(e)||(n=!0,delete r[e])}),n?{controllerList:r}:null}render(){const{classes:e}=this.props;return p.default.createElement("div",{className:e.mainPanel+" cosapp-widget-box",style:{background:"white"}},p.default.createElement(m.default,{open:this.state.addVariableDialog,closeHandle:this.closeAddVariableDialog}),p.default.createElement("div",{className:e.topPanel},Object.keys(this.state.controllerList).map((e,t)=>this.state.controllerList[e].component)),p.default.createElement("div",{className:e.bottom},p.default.createElement("div",{style:{textAlign:"center"}},p.default.createElement(a.default,{color:"primary","aria-label":"add",size:"small",style:{margin:"0 10px 0 10px"},onClick:()=>{this.setState(Object.assign(Object.assign({},this.state),{addVariableDialog:!0}))}},p.default.createElement(c.default,null)),p.default.createElement(a.default,{onClick:this.clickRunButton,color:this.props.computingState?"secondary":"primary","aria-label":"add",size:"small",style:{margin:"0 10px 0 10px"}},p.default.createElement(f.default,null)),p.default.createElement(a.default,{color:"primary","aria-label":"add",size:"small",style:{margin:"0 10px 0 10px"}},p.default.createElement(h.default,null))),p.default.createElement(r.default,{square:!1,expanded:this.state.expandNotification,onChange:e=>{this.setState(e=>Object.assign(Object.assign({},e),{expandNotification:!e.expandNotification}))}},p.default.createElement(o.default,{expandIcon:p.default.createElement(d.default,null),"aria-controls":"panel1a-content",classes:{root:e.summaryRoot,expanded:e.summaryExpanded,expandIcon:e.summaryExpandIcon,content:e.summaryContent}},p.default.createElement(u.default,{className:e.heading},"Log")),p.default.createElement(i.default,{ref:this.notificationRef,classes:{root:e.panelRoot}},p.default.createElement(u.default,{style:{whiteSpace:"pre-line"},variant:"body2",dangerouslySetInnerHTML:{__html:this.state.logMsg}})))))}}t.default=n.withStyles(e=>({mainPanel:{display:"flex",flexDirection:"column"},topPanel:{width:"100%",flex:"1 1 auto",overflowY:"auto",height:0,marginBottom:30},bottomPanel:{height:"auto",width:"100%"},summaryExpandIcon:{padding:0},summaryContent:{margin:"7px 0!important"},summaryRoot:{minHeight:36,margin:"16px 0 0 0",background:"#e0e0e0",color:"rgb(50,50,50)"},summaryExpanded:{minHeight:"24px!important"},panelRoot:{height:150,overflowY:"auto",marginBottom:10}}))(g.connect(e=>({computingState:e.dashboardState.computing,variableData:e.dashboardState.variableData,selectedVariable:e.dashboardState.selectedVariable}),e=>({toggleComputing:()=>e(v.dashboardToggleRun())}))(b))}.apply(t,r))||(e.exports=i)},function(e,t,n){"use strict";var r=n(36),i=n(46);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(0)),a=(0,r(n(66)).default)(o.createElement("path",{d:"M19 13h-6v6h-2v-6H5v-2h6V5h2v6h6v2z"}),"Add");t.default=a},function(e,t,n){"use strict";var r=n(36),i=n(46);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(0)),a=(0,r(n(66)).default)(o.createElement("path",{d:"M16.59 8.59L12 13.17 7.41 8.59 6 10l6 6 6-6z"}),"ExpandMore");t.default=a},function(e,t,n){"use strict";var r=n(36),i=n(46);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(0)),a=(0,r(n(66)).default)(o.createElement("path",{d:"M3 13h2v-2H3v2zm0 4h2v-2H3v2zm0-8h2V7H3v2zm4 4h14v-2H7v2zm0 4h14v-2H7v2zM7 7v2h14V7H7zm-4 6h2v-2H3v2zm0 4h2v-2H3v2zm0-8h2V7H3v2zm4 4h14v-2H7v2zm0 4h14v-2H7v2zM7 7v2h14V7H7z"}),"ListOutlined");t.default=a},function(e,t,n){"use strict";var r=n(36),i=n(46);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(0)),a=(0,r(n(66)).default)(o.createElement("path",{d:"M10 8.64L15.27 12 10 15.36V8.64M8 5v14l11-7L8 5z"}),"PlayArrowOutlined");t.default=a},function(e,t,n){var r,i,o=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),a=this&&this.__setModuleDefault||(Object.create?function(e,t){Object.defineProperty(e,"default",{enumerable:!0,value:t})}:function(e,t){e.default=t}),s=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)"default"!==n&&Object.hasOwnProperty.call(e,n)&&o(t,e,n);return a(t,e),t},l=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};r=[n,t,n(90),n(75),n(108),n(109),n(110),n(175),n(103),n(88),n(432),n(0),n(78),n(176)],void 0===(i=function(e,t,n,r,i,o,a,u,c,d,h,f,p,g){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),r=l(r),i=l(i),o=l(o),a=l(a),u=l(u),c=l(c),d=l(d),h=l(h),f=s(f),g=s(g);class v extends f.Component{constructor(e){super(e),this.handleOpen=()=>{this.setState({open:!0})},this.handleClose=()=>{this.setState({open:!1})},this.handleVariableChange=(e,t)=>{this.setState(Object.assign(Object.assign({},this.state),{selectedVar:t}))},this.handdleAddBtnClick=()=>{let e,t,n="-1";const r=/\[([^)]+)\]/.exec(this.state.selectedVar);r?(e=this.state.selectedVar.replace(r[0],""),n=r[1],t=this.props.variableData[e].value[parseInt(n)]):(e=this.state.selectedVar,t=this.props.variableData[this.state.selectedVar].value),this.props.addParameterController(this.state.selectedVar,t)},this.state={open:!1,selectedVar:""},this.systemList=[],Object.keys(this.props.variableData).forEach(e=>{if("number"==typeof this.props.variableData[e].value)this.systemList.push(e);else if(Array.isArray(this.props.variableData[e].value))for(let t=0;t<this.props.variableData[e].value.length;t++)this.systemList.push(e+"["+t+"]")})}componentDidUpdate(e,t){this.props.selectedVariable,e.selectedVariable}render(){const e=this.props.classes;return f.default.createElement(i.default,{keepMounted:!0,open:this.props.open,onClose:this.handleClose,"aria-labelledby":"form-dialog-title",fullWidth:!0,maxWidth:"sm"},f.default.createElement(u.default,null,"Add new controller"),f.default.createElement(a.default,{style:{overflowY:"unset"}},f.default.createElement(c.default,{className:e.formControl},f.default.createElement(h.default,{value:this.state.selectedVar,onChange:this.handleVariableChange,options:this.systemList,getOptionLabel:e=>e.split(".").slice(1).join("."),defaultValue:this.systemList[0],renderInput:e=>f.default.createElement(d.default,Object.assign({},e,{variant:"standard",label:"Select variable",placeholder:"",fullWidth:!0}))}))),f.default.createElement(o.default,null,f.default.createElement(r.default,{color:"primary",onClick:this.props.closeHandle},"close"),f.default.createElement(r.default,{color:"primary",onClick:this.handdleAddBtnClick},"add")))}}t.default=n.withStyles(e=>({formControl:{padding:e.spacing(1),minWidth:300,width:"100%"}}))(p.connect(e=>(e=>({variableData:e.dashboardState.variableData,selectedVariable:e.dashboardState.selectedVariable}))(e),e=>({addParameterController:(t,n)=>e(g.dashboardAddController(t,n)),removeParameterController:t=>e(g.dashboardRemoveController(t))}))(v))}.apply(t,r))||(e.exports=i)},function(e,t,n){var r,i,o=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),a=this&&this.__setModuleDefault||(Object.create?function(e,t){Object.defineProperty(e,"default",{enumerable:!0,value:t})}:function(e,t){e.default=t}),s=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)"default"!==n&&Object.hasOwnProperty.call(e,n)&&o(t,e,n);return a(t,e),t},l=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};r=[n,t,n(90),n(260),n(73),n(199),n(164),n(421),n(62),n(2221),n(0),n(78),n(248),n(176)],void 0===(i=function(e,t,n,r,i,o,a,u,c,d,h,f,p,g){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),r=l(r),i=l(i),o=l(o),a=l(a),u=l(u),c=l(c),d=l(d),h=s(h),g=s(g);class v extends h.Component{constructor(e){let t;if(super(e),this.handleSliderChange=(e,t)=>{const n=Number(t.toPrecision(3));this.setState({value:Math.abs(n)<1e-5?Number(n.toExponential()):n})},this.getMetaData=e=>{const t=e.split("."),n=t.length,r=t[n-1],i=t[n-2],o=t.slice(0,n-2).join("."),a=this.props.portMetaData[o][i][r],{desc:s,limits:l,valid_range:u,unit:c}=a;let d;return d=l||(u||null),{desc:s,range:d,unit:c}},this.handleInputChange=e=>{this.setState({value:""===e.target.value?0:Number(e.target.value)},()=>{this.props.updateHandle(this.props.variableName,this.state.value)})},this.handlePopoverOpen=e=>{const t=e.currentTarget;t&&this.setState(e=>Object.assign(Object.assign({},e),{anchorEl:t}))},this.handlePopoverClose=()=>{this.setState(e=>Object.assign(Object.assign({},e),{anchorEl:null}))},this.initialVal=Number(e.value.toPrecision(3)),this.props.variableName.endsWith("]")){const e=this.props.variableName.indexOf("[");t=this.props.variableName.slice(0,e)}else t=this.props.variableName;const{desc:n,range:r,unit:i}=this.getMetaData(t);this.state={value:this.initialVal,desc:n,range:r,unit:i,anchorEl:null},this._uuid=p.create_UUID()}render(){const{desc:e,range:t,unit:n}=this.state,s=h.default.createElement(c.default,{"aria-owns":Boolean(this.state.anchorEl)?this._uuid:void 0,"aria-haspopup":"true",onMouseEnter:e?this.handlePopoverOpen:null,onMouseLeave:e?this.handlePopoverClose:null},this.props.variableName);let l,f;t?(l={min:t[0],max:t[1],step:Math.abs(t[1]-t[0])/100},f={step:Math.abs(t[1]-t[0])/100,min:t[0],max:t[1],type:"number","aria-labelledby":"input-slider"}):(l={},f={type:"number","aria-labelledby":"input-slider"});const p=h.default.createElement(u.default,Object.assign({},l,{value:this.state.value,onChange:this.handleSliderChange,onChangeCommitted:()=>{this.props.updateHandle(this.props.variableName,this.state.value)},"aria-labelledby":"input-slider"})),g=h.default.createElement(o.default,{value:this.state.value,margin:"dense",onChange:e=>{this.setState({value:e.target.value})},inputProps:f,onBlur:this.handleInputChange}),v=this.props.classes;let m;return e&&(m=h.default.createElement(a.default,{id:this._uuid,className:v.popover,classes:{paper:v.paper},open:Boolean(this.state.anchorEl),anchorEl:this.state.anchorEl,anchorOrigin:{vertical:"bottom",horizontal:"left"},transformOrigin:{vertical:"top",horizontal:"left"},onClose:this.handlePopoverClose,disableRestoreFocus:!0},h.default.createElement(c.default,null,e," - unit : ",n||"undefined"))),h.default.createElement("div",{style:{padding:"10px 10px 10px 10px",borderBottom:"solid 0.5px black"}},e?m:h.default.createElement("div",null),t?s:h.default.createElement("div",null),h.default.createElement(r.default,{container:!0,spacing:2,alignItems:"center"},h.default.createElement(r.default,{item:!0,xs:7},t?p:s),h.default.createElement(r.default,{item:!0,xs:3},g),h.default.createElement(r.default,{item:!0,xs:2},h.default.createElement(i.default,{"aria-label":"delete",onClick:()=>{this.props.removeParameterController(this.props.variableName)}},h.default.createElement(d.default,{fontSize:"small"})))))}}t.default=n.withStyles(e=>({popover:{pointerEvents:"none"},paper:{padding:e.spacing(1)}}))(f.connect(e=>(e=>({portMetaData:e.dashboardState.portMetaData}))(e),e=>({removeParameterController:t=>e(g.dashboardRemoveController(t)),updateHandle:(t,n)=>e(g.dashboardUpdateController(t,n))}))(v))}.apply(t,r))||(e.exports=i)},function(e,t,n){"use strict";var r=n(36),i=n(46);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(0)),a=(0,r(n(66)).default)(o.createElement("path",{d:"M6 19c0 1.1.9 2 2 2h8c1.1 0 2-.9 2-2V7H6v12zM19 4h-3.5l-1-1h-5l-1 1H5v2h14V4z"}),"Delete");t.default=a},function(e,t,n){var r,i,o=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),a=this&&this.__setModuleDefault||(Object.create?function(e,t){Object.defineProperty(e,"default",{enumerable:!0,value:t})}:function(e,t){e.default=t}),s=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)"default"!==n&&Object.hasOwnProperty.call(e,n)&&o(t,e,n);return a(t,e),t},l=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};r=[n,t,n(90),n(368),n(75),n(108),n(109),n(110),n(175),n(103),n(247),n(76),n(310),n(311),n(136),n(312),n(154),n(88),n(2223),n(2224),n(2225),n(2226),n(2227),n(2228),n(2229),n(377),n(2230),n(2231),n(2232),n(2233),n(2234),n(2235),n(378),n(2236),n(432),n(2237),n(0),n(379),n(78),n(305)],void 0===(i=function(e,t,n,r,i,o,a,u,c,d,h,f,p,g,v,m,y,b,C,A,_,I,x,w,M,S,T,E,O,k,L,D,P,N,j,R,z,F,B){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.DataViewer=void 0,r=l(r),i=l(i),o=l(o),a=l(a),u=l(u),c=l(c),d=l(d),h=l(h),f=l(f),p=l(p),g=l(g),v=l(v),m=l(m),y=l(y),b=l(b),C=l(C),A=l(A),_=l(_),I=l(I),x=l(x),w=l(w),M=l(M),S=l(S),T=l(T),E=l(E),O=l(O),k=l(k),L=l(L),D=l(D),P=l(P),N=l(N),j=l(j),R=l(R),z=s(z),F=l(F);const H={Add:z.forwardRef((e,t)=>z.default.createElement(C.default,Object.assign({},e,{ref:t}))),Check:z.forwardRef((e,t)=>z.default.createElement(_.default,Object.assign({},e,{ref:t}))),Clear:z.forwardRef((e,t)=>z.default.createElement(w.default,Object.assign({},e,{ref:t}))),Delete:z.forwardRef((e,t)=>z.default.createElement(M.default,Object.assign({},e,{ref:t}))),DetailPanel:z.forwardRef((e,t)=>z.default.createElement(x.default,Object.assign({},e,{ref:t}))),Edit:z.forwardRef((e,t)=>z.default.createElement(S.default,Object.assign({},e,{ref:t}))),Export:z.forwardRef((e,t)=>z.default.createElement(L.default,Object.assign({},e,{ref:t}))),Filter:z.forwardRef((e,t)=>z.default.createElement(T.default,Object.assign({},e,{ref:t}))),FirstPage:z.forwardRef((e,t)=>z.default.createElement(E.default,Object.assign({},e,{ref:t}))),LastPage:z.forwardRef((e,t)=>z.default.createElement(O.default,Object.assign({},e,{ref:t}))),NextPage:z.forwardRef((e,t)=>z.default.createElement(x.default,Object.assign({},e,{ref:t}))),PreviousPage:z.forwardRef((e,t)=>z.default.createElement(I.default,Object.assign({},e,{ref:t}))),ResetSearch:z.forwardRef((e,t)=>z.default.createElement(w.default,Object.assign({},e,{ref:t}))),Search:z.forwardRef((e,t)=>z.default.createElement(D.default,Object.assign({},e,{ref:t}))),SortArrow:z.forwardRef((e,t)=>z.default.createElement(A.default,Object.assign({},e,{ref:t}))),ThirdStateCheck:z.forwardRef((e,t)=>z.default.createElement(k.default,Object.assign({},e,{ref:t}))),ViewColumn:z.forwardRef((e,t)=>z.default.createElement(N.default,Object.assign({},e,{ref:t})))};function U(e){return z.default.createElement(F.default,{handle:".draggable-dialog-title",cancel:'[class*="MuiDialogContent-root"]'},z.default.createElement(f.default,Object.assign({},e)))}class V extends z.Component{constructor(e){super(e),this.singlePortToTableData=(e,t,n=!1)=>{const r=[],i=this.props.portMetaData[e][t];for(const[o,a]of Object.entries(i))if("__class__"!==o){const i=`${e}.${t}.${o}`,s=this.props.computedResult[i][1],l=this.props.computedResult[i][0];let u;u=isNaN(s)?"string"==typeof s?s:"Expand to view":s;const{desc:c=null,distribution:d=null,invalid_comment:h=null,limits:f=null,out_of_limits_comment:p=null,unit:g=null,valid_range:v=null}=a;let m,y="";g&&(y=` (${g})`),m=n?`${t}.${o}${y}`:`${o}${y}`,r.push({name:m,value:s,displayValue:u,desc:c,dtype:l,distribution:d,invalid_comment:h,limits:f,out_of_limits_comment:p,unit:g,valid_range:v})}return r},this.computedResultToTableData=(e,t)=>{if("All ports"!==t)return this.singlePortToTableData(e,t,!1);{const t=[];return this.state.dictSelector[e].forEach(n=>{"All ports"!==n&&t.push(...this.singlePortToTableData(e,n,!0))}),t}},this.handleSystemSelectChange=(e,t)=>{const n=this.state.dictSelector[t][0];this.setState(e=>Object.assign(Object.assign({},e),{traceConfig:{systemName:t,portName:n}}))},this.handlePortChange=(e,t)=>{this.setState(e=>Object.assign(Object.assign({},e),{traceConfig:Object.assign(Object.assign({},e.traceConfig),{portName:t})}))},this.toggleSetting=()=>{this.setState(e=>Object.assign(Object.assign({},e),{openSetting:!e.openSetting}))},this.updateSetting=()=>{this.setState(e=>{const t=e.traceConfig.systemName,n=e.traceConfig.portName;let r;r="inwards"!==n&&"outwards"!==n&&"All ports"!==n?"- "+this.props.portMetaData[t][n].__class__:"";const i=`${t} - ${n} ${r}`,o=this.computedResultToTableData(t,n);return Object.assign(Object.assign({},e),{openSetting:!1,title:i,rowData:o})})},this.check_value_in_range=(e,t)=>{let n=!0;if(!t)return n;if(Array.isArray(e)){if(0===e.length)return n;if(isNaN(e[0]))return n;for(const r of e)if(t[0]>r||r>t[1]){n=!1;break}}else(t[0]>e||e>t[1])&&(n=!1);return n},this.check_validity_and_limit_value=(e,t,n)=>{let r=[0,0],i=[0,0],o=!0,a=!0;if(t)for(let e=0;e<t.length;e++){const n=t[e];r[e]="-inf"===n?-1/0:"inf"===n?1/0:parseFloat(n+"")}else r=null;if(n)for(let e=0;e<n.length;e++){const t=n[e];i[e]="-inf"===t?-1/0:"inf"===t?1/0:parseFloat(t+"")}else i=null;return o=this.check_value_in_range(e,r),a=this.check_value_in_range(e,i),{in_valid:o,in_limit:a}},this.initialState=e.initialState;let t={};for(const[n,r]of Object.entries(e.portMetaData)){const e=Object.keys(r);e.unshift("All ports"),t[n]=e}let n=Object.keys(e.portMetaData);0===n.length&&(n=["None"],t={None:{None:["None"]}});this.state={traceConfig:{systemName:n[0],portName:t[n[0]][0]},openSetting:!1,listSelector:n,dictSelector:t,title:"Data Viewer",rowData:[]}}componentDidMount(){if(this.initialState){const e=this.initialState.traceConfig;let t;t="inwards"!==e.portName&&"outwards"!==e.portName&&"All ports"!==e.portName?"- "+this.props.portMetaData[e.systemName][e.portName].__class__:"";const n=`${e.systemName} - ${e.portName} ${t}`,r=this.computedResultToTableData(e.systemName,e.portName);this.setState(t=>Object.assign(Object.assign({},t),{traceConfig:e,rowData:r,title:n}))}}componentDidUpdate(e,t){if(e.computedResult!==this.props.computedResult){const e=this.computedResultToTableData(this.state.traceConfig.systemName,this.state.traceConfig.portName);this.setState(t=>Object.assign(Object.assign({},t),{rowData:e}))}}render(){const e=this.props.classes;return z.default.createElement("div",{className:"cosapp-widget-box"},z.default.createElement(o.default,{open:this.state.openSetting,"aria-labelledby":"draggable-dialog-title",fullWidth:!0,maxWidth:"sm",PaperComponent:U},z.default.createElement(c.default,{style:{cursor:"move"},className:"draggable-dialog-title"},"Data viewer configuration"),z.default.createElement(u.default,null,z.default.createElement(h.default,{style:{flexDirection:"row"}},z.default.createElement(j.default,{className:e.formControlShort,value:this.state.traceConfig.systemName,onChange:this.handleSystemSelectChange,options:this.state.listSelector,getOptionLabel:e=>e.replace("chart_viewer.",""),defaultValue:this.state.listSelector[0],disableClearable:!0,renderInput:e=>z.default.createElement(b.default,Object.assign({},e,{variant:"standard",label:"Select system",placeholder:"",fullWidth:!0}))}),z.default.createElement(d.default,{className:e.formControlShort},z.default.createElement(j.default,{disableClearable:!0,value:this.state.traceConfig.portName,onChange:this.handlePortChange,options:this.state.dictSelector[this.state.traceConfig.systemName],renderInput:e=>z.default.createElement(b.default,Object.assign({},e,{variant:"standard",label:"Select port",placeholder:"",fullWidth:!0}))})))),z.default.createElement(a.default,null,z.default.createElement(i.default,{autoFocus:!0,onClick:this.toggleSetting,color:"primary"},"Close"),z.default.createElement(i.default,{autoFocus:!0,onClick:this.updateSetting,color:"primary"},"Save"))),z.default.createElement("div",{className:"DataViewerMain",style:{width:"100%",height:"calc(100% - 35px)",overflow:"auto"}},z.default.createElement(R.default,{style:{height:"100%",display:"flex",flexDirection:"column"},icons:H,title:this.state.title,options:{paging:!1,search:!1,rowStyle:e=>{let t="";const{value:n,limits:r,valid_range:i}=e,{in_valid:o,in_limit:a}=this.check_validity_and_limit_value(n,i,r);return a?o||(t="#f5f56c"):t="#db3737",{backgroundColor:t}}},columns:[{title:"Variable",field:"name"},{title:"Value",field:"displayValue"},{title:"Description",field:"desc"}],data:this.state.rowData,detailPanel:e=>z.default.createElement(r.default,{margin:1},z.default.createElement(p.default,{size:"small","aria-label":"purchases"},z.default.createElement(m.default,null,z.default.createElement(y.default,null,z.default.createElement(v.default,null," ",z.default.createElement("b",null,"Property")," "),z.default.createElement(v.default,null,z.default.createElement("b",null,"Value")," "))),z.default.createElement(g.default,null,Object.entries(e).filter(([e,t])=>"tableData"!==e&&"displayValue"!==e).map(([e,t])=>{let n;return n=["value","limits","valid_range"].includes(e)?JSON.stringify(t).replace(new RegExp(",","g"),", "):t||"None",z.default.createElement(y.default,{key:e},z.default.createElement(v.default,{component:"th",scope:"row"},e),z.default.createElement(v.default,{style:{wordBreak:"break-word"}},n))}))))})),z.default.createElement("div",{className:this.props.classes.bottomBarDiv},z.default.createElement(i.default,{onClick:this.toggleSetting,style:{color:"rgb(50,50,50)"}}," ",z.default.createElement(P.default,null))))}}t.DataViewer=V,t.default=B.connect(e=>(e=>({computedResult:e.dashboardState.computedResult,portMetaData:e.dashboardState.portMetaData}))(e),e=>({}),null,{forwardRef:!0})(n.withStyles(e=>({textSizeSmall:{},toolbarHeigt:{minHeight:36,background:"rgb(50, 50, 50)"},viewSelector:{minWidth:120,color:"rgb(250, 250, 250)"},bgSelector:{minWidth:60,color:"rgb(250, 250, 250)"},textColor:{color:"rgb(250, 250, 250)"},backGround:{color:"rgb(50, 50, 50)"},formControl:{padding:e.spacing(1),width:"90%"},formControlShort:{padding:e.spacing(1),width:"calc(50% - 16px)"},formControlTiny:{width:"22%"},selectEmpty:{marginTop:e.spacing(2)},textField:{marginLeft:e.spacing(1),marginRight:e.spacing(1),width:"90%"},bottomBarDiv:{width:"100%",height:"35px",background:"#e0e0e0",display:"flex",flexDirection:"row-reverse"}}))(z.forwardRef((e,t)=>z.default.createElement(V,Object.assign({},e,{ref:t})))))}.apply(t,r))||(e.exports=i)},function(e,t,n){"use strict";var r=n(36),i=n(46);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(0)),a=(0,r(n(66)).default)(o.createElement("path",{d:"M19 3H5c-1.11 0-2 .9-2 2v14c0 1.1.89 2 2 2h14c1.1 0 2-.9 2-2V5c0-1.1-.9-2-2-2zm-2 10h-4v4h-2v-4H7v-2h4V7h2v4h4v2z"}),"AddBox");t.default=a},function(e,t,n){"use strict";var r=n(36),i=n(46);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(0)),a=(0,r(n(66)).default)(o.createElement("path",{d:"M20 12l-1.41-1.41L13 16.17V4h-2v12.17l-5.58-5.59L4 12l8 8 8-8z"}),"ArrowDownward");t.default=a},function(e,t,n){"use strict";var r=n(36),i=n(46);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(0)),a=(0,r(n(66)).default)(o.createElement("path",{d:"M9 16.17L4.83 12l-1.42 1.41L9 19 21 7l-1.41-1.41z"}),"Check");t.default=a},function(e,t,n){"use strict";var r=n(36),i=n(46);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(0)),a=(0,r(n(66)).default)(o.createElement("path",{d:"M15.41 7.41L14 6l-6 6 6 6 1.41-1.41L10.83 12z"}),"ChevronLeft");t.default=a},function(e,t,n){"use strict";var r=n(36),i=n(46);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(0)),a=(0,r(n(66)).default)(o.createElement("path",{d:"M10 6L8.59 7.41 13.17 12l-4.58 4.59L10 18l6-6z"}),"ChevronRight");t.default=a},function(e,t,n){"use strict";var r=n(36),i=n(46);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(0)),a=(0,r(n(66)).default)(o.createElement("path",{d:"M19 6.41L17.59 5 12 10.59 6.41 5 5 6.41 10.59 12 5 17.59 6.41 19 12 13.41 17.59 19 19 17.59 13.41 12z"}),"Clear");t.default=a},function(e,t,n){"use strict";var r=n(36),i=n(46);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(0)),a=(0,r(n(66)).default)(o.createElement("path",{d:"M6 19c0 1.1.9 2 2 2h8c1.1 0 2-.9 2-2V7H6v12zM8 9h8v10H8V9zm7.5-5l-1-1h-5l-1 1H5v2h14V4z"}),"DeleteOutline");t.default=a},function(e,t,n){"use strict";var r=n(36),i=n(46);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(0)),a=(0,r(n(66)).default)(o.createElement("path",{d:"M10 18h4v-2h-4v2zM3 6v2h18V6H3zm3 7h12v-2H6v2z"}),"FilterList");t.default=a},function(e,t,n){"use strict";var r=n(36),i=n(46);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(0)),a=(0,r(n(66)).default)(o.createElement("path",{d:"M18.41 16.59L13.82 12l4.59-4.59L17 6l-6 6 6 6zM6 6h2v12H6z"}),"FirstPage");t.default=a},function(e,t,n){"use strict";var r=n(36),i=n(46);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(0)),a=(0,r(n(66)).default)(o.createElement("path",{d:"M5.59 7.41L10.18 12l-4.59 4.59L7 18l6-6-6-6zM16 6h2v12h-2z"}),"LastPage");t.default=a},function(e,t,n){"use strict";var r=n(36),i=n(46);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(0)),a=(0,r(n(66)).default)(o.createElement("path",{d:"M19 13H5v-2h14v2z"}),"Remove");t.default=a},function(e,t,n){"use strict";var r=n(36),i=n(46);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(0)),a=(0,r(n(66)).default)(o.createElement("path",{d:"M19 12v7H5v-7H3v7c0 1.1.9 2 2 2h14c1.1 0 2-.9 2-2v-7h-2zm-6 .67l2.59-2.58L17 11.5l-5 5-5-5 1.41-1.41L11 12.67V3h2z"}),"SaveAlt");t.default=a},function(e,t,n){"use strict";var r=n(36),i=n(46);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(0)),a=(0,r(n(66)).default)(o.createElement("path",{d:"M15.5 14h-.79l-.28-.27C15.41 12.59 16 11.11 16 9.5 16 5.91 13.09 3 9.5 3S3 5.91 3 9.5 5.91 16 9.5 16c1.61 0 3.09-.59 4.23-1.57l.27.28v.79l5 4.99L20.49 19l-4.99-5zm-6 0C7.01 14 5 11.99 5 9.5S7.01 5 9.5 5 14 7.01 14 9.5 11.99 14 9.5 14z"}),"Search");t.default=a},function(e,t,n){"use strict";var r=n(36),i=n(46);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(0)),a=(0,r(n(66)).default)(o.createElement("path",{d:"M10 18h5V5h-5v13zm-6 0h5V5H4v13zM16 5v13h5V5h-5z"}),"ViewColumn");t.default=a},function(e,t,n){"use strict";var r=n(36);Object.defineProperty(t,"__esModule",{value:!0});var i={MTable:!0};Object.defineProperty(t,"MTable",{enumerable:!0,get:function(){return u.default}}),t.default=void 0;var o=r(n(50));n(2238);var a=r(n(0)),s=n(2240),l=n(2300),u=r(n(2301)),c=r(n(314)),d=n(616);Object.keys(d).forEach((function(e){"default"!==e&&"__esModule"!==e&&(Object.prototype.hasOwnProperty.call(i,e)||Object.defineProperty(t,e,{enumerable:!0,get:function(){return d[e]}}))})),u.default.defaultProps=s.defaultProps,u.default.propTypes=l.propTypes;var h=(0,c.default)((function(e){return{paginationRoot:{width:"100%"},paginationToolbar:{padding:0,width:"100%"},paginationCaption:{display:"none"},paginationSelectRoot:{margin:0}}}),{withTheme:!0})((function(e){return a.default.createElement(u.default,(0,o.default)({},e,{ref:e.tableRef}))}));t.default=h},function(e,t,n){"use strict";Array.prototype.find||n(2239)},function(e,t,n){"use strict";Object.defineProperty(Array.prototype,"find",{value:function(e){if(null==this)throw new TypeError('"this" is null or not defined');var t=Object(this),n=t.length>>>0;if("function"!=typeof e)throw new TypeError("predicate must be a function");for(var r=arguments[1],i=0;i<n;){var o=t[i];if(e.call(r,o,i,t))return o;i++}}})},function(e,t,n){"use strict";var r=n(46),i=n(36);Object.defineProperty(t,"__esModule",{value:!0}),t.defaultProps=void 0;var o=i(n(50)),a=i(n(0)),s=i(n(184)),l=i(n(221)),u=i(n(76)),c=i(n(422)),d=r(n(616)),h=i(n(3)),f=n(618),p=function(e){return a.default.createElement("div",{style:{display:"table",width:"100%",height:"100%",backgroundColor:(0,f.fade)(e.theme.palette.background.paper,.7)}},a.default.createElement("div",{style:{display:"table-cell",width:"100%",height:"100%",verticalAlign:"middle",textAlign:"center"}},a.default.createElement(s.default,null)))};p.propTypes={theme:h.default.any};var g={actions:[],classes:{},columns:[],components:{Action:d.MTableAction,Actions:d.MTableActions,Body:d.MTableBody,Cell:d.MTableCell,Container:function(e){return a.default.createElement(u.default,(0,o.default)({elevation:2},e))},EditField:d.MTableEditField,EditRow:d.MTableEditRow,FilterRow:d.MTableFilterRow,Groupbar:d.MTableGroupbar,GroupRow:d.MTableGroupRow,Header:d.MTableHeader,OverlayLoading:p,Pagination:c.default,Row:d.MTableBodyRow,Toolbar:d.MTableToolbar},data:[],icons:{Add:a.default.forwardRef((function(e,t){return a.default.createElement(l.default,(0,o.default)({},e,{ref:t}),"add_box")})),Check:a.default.forwardRef((function(e,t){return a.default.createElement(l.default,(0,o.default)({},e,{ref:t}),"check")})),Clear:a.default.forwardRef((function(e,t){return a.default.createElement(l.default,(0,o.default)({},e,{ref:t}),"clear")})),Delete:a.default.forwardRef((function(e,t){return a.default.createElement(l.default,(0,o.default)({},e,{ref:t}),"delete_outline")})),DetailPanel:a.default.forwardRef((function(e,t){return a.default.createElement(l.default,(0,o.default)({},e,{ref:t}),"chevron_right")})),Edit:a.default.forwardRef((function(e,t){return a.default.createElement(l.default,(0,o.default)({},e,{ref:t}),"edit")})),Export:a.default.forwardRef((function(e,t){return a.default.createElement(l.default,(0,o.default)({},e,{ref:t}),"save_alt")})),Filter:a.default.forwardRef((function(e,t){return a.default.createElement(l.default,(0,o.default)({},e,{ref:t}),"filter_list")})),FirstPage:a.default.forwardRef((function(e,t){return a.default.createElement(l.default,(0,o.default)({},e,{ref:t}),"first_page")})),LastPage:a.default.forwardRef((function(e,t){return a.default.createElement(l.default,(0,o.default)({},e,{ref:t}),"last_page")})),NextPage:a.default.forwardRef((function(e,t){return a.default.createElement(l.default,(0,o.default)({},e,{ref:t}),"chevron_right")})),PreviousPage:a.default.forwardRef((function(e,t){return a.default.createElement(l.default,(0,o.default)({},e,{ref:t}),"chevron_left")})),ResetSearch:a.default.forwardRef((function(e,t){return a.default.createElement(l.default,(0,o.default)({},e,{ref:t}),"clear")})),Search:a.default.forwardRef((function(e,t){return a.default.createElement(l.default,(0,o.default)({},e,{ref:t}),"search")})),SortArrow:a.default.forwardRef((function(e,t){return a.default.createElement(l.default,(0,o.default)({},e,{ref:t}),"arrow_downward")})),ThirdStateCheck:a.default.forwardRef((function(e,t){return a.default.createElement(l.default,(0,o.default)({},e,{ref:t}),"remove")})),ViewColumn:a.default.forwardRef((function(e,t){return a.default.createElement(l.default,(0,o.default)({},e,{ref:t}),"view_column")}))},isLoading:!1,title:"Table Title",options:{actionsColumnIndex:0,addRowPosition:"last",columnsButton:!1,detailPanelType:"multiple",debounceInterval:200,doubleHorizontalScroll:!1,emptyRowsWhenPaging:!0,exportAllData:!1,exportButton:!1,exportDelimiter:",",filtering:!1,groupTitle:!1,header:!0,hideFilterIcons:!1,loadingType:"overlay",padding:"default",searchAutoFocus:!1,paging:!0,pageSize:5,pageSizeOptions:[5,10,20],paginationType:"normal",showEmptyDataSourceMessage:!0,showFirstLastPageButtons:!0,showSelectAllCheckbox:!0,search:!0,showTitle:!0,showTextRowsSelected:!0,tableLayout:"auto",toolbarButtonAlignment:"right",searchFieldAlignment:"right",searchFieldStyle:{},searchFieldVariant:"standard",selection:!1,selectionProps:{},sorting:!0,toolbar:!0,defaultExpanded:!1,detailPanelColumnAlignment:"left",thirdSortClick:!0,overflowY:"auto"},localization:{grouping:{groupedBy:"Grouped By:",placeholder:"Drag headers here to group by"},pagination:{labelDisplayedRows:"{from}-{to} of {count}",labelRowsPerPage:"Rows per page:",labelRowsSelect:"rows"},toolbar:{},header:{},body:{filterRow:{},editRow:{saveTooltip:"Save",cancelTooltip:"Cancel",deleteText:"Are you sure you want to delete this row?"},addTooltip:"Add",deleteTooltip:"Delete",editTooltip:"Edit"}},style:{}};t.defaultProps=g},function(e,t,n){"use strict";var r=n(46),i=n(36);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(114)),a=i(n(33)),s=i(n(34)),l=i(n(45)),u=i(n(43)),c=i(n(44)),d=r(n(0)),h=i(n(3)),f=i(n(221)),p=i(n(73)),g=i(n(185));function v(e){var t=function(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],(function(){}))),!0}catch(e){return!1}}();return function(){var n,r=(0,c.default)(e);if(t){var i=(0,c.default)(this).constructor;n=Reflect.construct(r,arguments,i)}else n=r.apply(this,arguments);return(0,u.default)(this,n)}}var m=function(e){(0,l.default)(n,e);var t=v(n);function n(){return(0,a.default)(this,n),t.apply(this,arguments)}return(0,s.default)(n,[{key:"render",value:function(){var e=this,t=this.props.action;if("function"==typeof t&&!(t=t(this.props.data)))return null;if(t.action&&!(t=t.action(this.props.data)))return null;if(t.hidden)return null;var n=t.disabled||this.props.disabled,r="string"==typeof t.icon?d.createElement(f.default,t.iconProps,t.icon):"function"==typeof t.icon?t.icon((0,o.default)({},t.iconProps,{disabled:n})):d.createElement(t.icon,null),i=d.createElement(p.default,{size:this.props.size,color:"inherit",disabled:n,onClick:function(n){t.onClick&&(t.onClick(n,e.props.data),n.stopPropagation())}},r);return t.tooltip?n?d.createElement(g.default,{title:t.tooltip},d.createElement("span",null,i)):d.createElement(g.default,{title:t.tooltip},i):i}}]),n}(d.Component);m.defaultProps={action:{},data:{}},m.propTypes={action:h.default.oneOfType([h.default.func,h.default.object]).isRequired,data:h.default.oneOfType([h.default.object,h.default.arrayOf(h.default.object)]),disabled:h.default.bool,size:h.default.string};var y=m;t.default=y},function(e,t,n){"use strict";var r=n(46),i=n(36);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(33)),a=i(n(34)),s=i(n(45)),l=i(n(43)),u=i(n(44)),c=r(n(0)),d=i(n(3));function h(e){var t=function(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],(function(){}))),!0}catch(e){return!1}}();return function(){var n,r=(0,u.default)(e);if(t){var i=(0,u.default)(this).constructor;n=Reflect.construct(r,arguments,i)}else n=r.apply(this,arguments);return(0,l.default)(this,n)}}var f=function(e){(0,s.default)(n,e);var t=h(n);function n(){return(0,o.default)(this,n),t.apply(this,arguments)}return(0,a.default)(n,[{key:"render",value:function(){var e=this;return this.props.actions?this.props.actions.map((function(t,n){return c.createElement(e.props.components.Action,{action:t,key:"action-"+n,data:e.props.data,size:e.props.size,disabled:e.props.disabled})})):null}}]),n}(c.Component);f.defaultProps={actions:[],data:{}},f.propTypes={components:d.default.object.isRequired,actions:d.default.array.isRequired,data:d.default.oneOfType([d.default.object,d.default.arrayOf(d.default.object)]),disabled:d.default.bool,size:d.default.string};var p=f;t.default=p},function(e,t,n){"use strict";var r=n(46),i=n(36);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(223)),a=i(n(114)),s=i(n(33)),l=i(n(34)),u=i(n(45)),c=i(n(43)),d=i(n(44)),h=i(n(311)),f=i(n(136)),p=i(n(154)),g=i(n(3)),v=r(n(0));function m(e){var t=function(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],(function(){}))),!0}catch(e){return!1}}();return function(){var n,r=(0,d.default)(e);if(t){var i=(0,d.default)(this).constructor;n=Reflect.construct(r,arguments,i)}else n=r.apply(this,arguments);return(0,c.default)(this,n)}}var y=function(e){(0,u.default)(n,e);var t=m(n);function n(){return(0,s.default)(this,n),t.apply(this,arguments)}return(0,l.default)(n,[{key:"renderEmpty",value:function(e,t){var r="default"===this.props.options.padding?49:36,i=(0,a.default)({},n.defaultProps.localization,this.props.localization);if(this.props.options.showEmptyDataSourceMessage&&0===t.length){var s=0;return this.props.options.selection&&s++,this.props.actions&&this.props.actions.filter((function(e){return"row"===e.position||"function"==typeof e})).length>0&&s++,this.props.hasDetailPanel&&s++,this.props.isTreeData&&s++,v.createElement(p.default,{style:{height:r*(this.props.options.paging&&this.props.options.emptyRowsWhenPaging?this.props.pageSize:1)},key:"empty-0"},v.createElement(f.default,{style:{paddingTop:0,paddingBottom:0,textAlign:"center"},colSpan:this.props.columns.reduce((function(e,t){return t.hidden?e:e+1}),s),key:"empty-"},i.emptyDataSourceMessage))}if(this.props.options.emptyRowsWhenPaging)return v.createElement(v.Fragment,null,(0,o.default)(Array(e)).map((function(e,t){return v.createElement(p.default,{style:{height:r},key:"empty-"+t})})),e>0&&v.createElement(p.default,{style:{height:1},key:"empty-last1"}))}},{key:"renderUngroupedRows",value:function(e){var t=this;return e.map((function(e,r){return e.tableData.editing?v.createElement(t.props.components.EditRow,{columns:t.props.columns.filter((function(e){return!e.hidden})),components:t.props.components,data:e,icons:t.props.icons,localization:(0,a.default)({},n.defaultProps.localization.editRow,t.props.localization.editRow,{dateTimePickerLocalization:t.props.localization.dateTimePickerLocalization}),key:r,mode:e.tableData.editing,options:t.props.options,isTreeData:t.props.isTreeData,detailPanel:t.props.detailPanel,onEditingCanceled:t.props.onEditingCanceled,onEditingApproved:t.props.onEditingApproved,getFieldValue:t.props.getFieldValue}):v.createElement(t.props.components.Row,{components:t.props.components,icons:t.props.icons,data:e,index:r,key:"row-"+e.tableData.id,level:0,options:t.props.options,localization:(0,a.default)({},n.defaultProps.localization.editRow,t.props.localization.editRow),onRowSelected:t.props.onRowSelected,actions:t.props.actions,columns:t.props.columns,getFieldValue:t.props.getFieldValue,detailPanel:t.props.detailPanel,path:[r+t.props.pageSize*t.props.currentPage],onToggleDetailPanel:t.props.onToggleDetailPanel,onRowClick:t.props.onRowClick,isTreeData:t.props.isTreeData,onTreeExpandChanged:t.props.onTreeExpandChanged,onEditingCanceled:t.props.onEditingCanceled,onEditingApproved:t.props.onEditingApproved,hasAnyEditingRow:t.props.hasAnyEditingRow,treeDataMaxLevel:t.props.treeDataMaxLevel})}))}},{key:"renderGroupedRows",value:function(e,t){var r=this;return t.map((function(t,i){return v.createElement(r.props.components.GroupRow,{actions:r.props.actions,key:null==t.value?""+i:t.value,columns:r.props.columns,components:r.props.components,detailPanel:r.props.detailPanel,getFieldValue:r.props.getFieldValue,groupData:t,groups:e,icons:r.props.icons,level:0,path:[i+r.props.pageSize*r.props.currentPage],onGroupExpandChanged:r.props.onGroupExpandChanged,onRowSelected:r.props.onRowSelected,onRowClick:r.props.onRowClick,onEditingCanceled:r.props.onEditingCanceled,onEditingApproved:r.props.onEditingApproved,onToggleDetailPanel:r.props.onToggleDetailPanel,onTreeExpandChanged:r.props.onTreeExpandChanged,options:r.props.options,isTreeData:r.props.isTreeData,hasAnyEditingRow:r.props.hasAnyEditingRow,localization:(0,a.default)({},n.defaultProps.localization.editRow,r.props.localization.editRow)})}))}},{key:"render",value:function(){var e=this.props.renderData,t=this.props.columns.filter((function(e){return e.tableData.groupOrder>-1})).sort((function(e,t){return e.tableData.groupOrder-t.tableData.groupOrder})),r=0;return this.props.options.paging&&(r=this.props.pageSize-e.length),v.createElement(h.default,null,this.props.options.filtering&&v.createElement(this.props.components.FilterRow,{columns:this.props.columns.filter((function(e){return!e.hidden})),icons:this.props.icons,hasActions:this.props.actions.filter((function(e){return"row"===e.position||"function"==typeof e})).length>0,actionsColumnIndex:this.props.options.actionsColumnIndex,onFilterChanged:this.props.onFilterChanged,selection:this.props.options.selection,localization:(0,a.default)({},n.defaultProps.localization.filterRow,this.props.localization.filterRow,{dateTimePickerLocalization:this.props.localization.dateTimePickerLocalization}),hasDetailPanel:!!this.props.detailPanel,isTreeData:this.props.isTreeData,filterCellStyle:this.props.options.filterCellStyle,filterRowStyle:this.props.options.filterRowStyle,hideFilterIcons:this.props.options.hideFilterIcons}),this.props.showAddRow&&"first"===this.props.options.addRowPosition&&v.createElement(this.props.components.EditRow,{columns:this.props.columns.filter((function(e){return!e.hidden})),data:this.props.initialFormData,components:this.props.components,icons:this.props.icons,key:"key-add-row",mode:"add",localization:(0,a.default)({},n.defaultProps.localization.editRow,this.props.localization.editRow),options:this.props.options,isTreeData:this.props.isTreeData,detailPanel:this.props.detailPanel,onEditingCanceled:this.props.onEditingCanceled,onEditingApproved:this.props.onEditingApproved,getFieldValue:this.props.getFieldValue}),t.length>0?this.renderGroupedRows(t,e):this.renderUngroupedRows(e),this.props.showAddRow&&"last"===this.props.options.addRowPosition&&v.createElement(this.props.components.EditRow,{columns:this.props.columns.filter((function(e){return!e.hidden})),data:this.props.initialFormData,components:this.props.components,icons:this.props.icons,key:"key-add-row",mode:"add",localization:(0,a.default)({},n.defaultProps.localization.editRow,this.props.localization.editRow),options:this.props.options,isTreeData:this.props.isTreeData,detailPanel:this.props.detailPanel,onEditingCanceled:this.props.onEditingCanceled,onEditingApproved:this.props.onEditingApproved,getFieldValue:this.props.getFieldValue}),this.renderEmpty(r,e))}}]),n}(v.Component);y.defaultProps={actions:[],currentPage:0,pageSize:5,renderData:[],selection:!1,localization:{emptyDataSourceMessage:"No records to display",filterRow:{},editRow:{}}},y.propTypes={actions:g.default.array,components:g.default.object.isRequired,columns:g.default.array.isRequired,currentPage:g.default.number,detailPanel:g.default.oneOfType([g.default.func,g.default.arrayOf(g.default.oneOfType([g.default.object,g.default.func]))]),getFieldValue:g.default.func.isRequired,hasAnyEditingRow:g.default.bool,hasDetailPanel:g.default.bool.isRequired,icons:g.default.object.isRequired,isTreeData:g.default.bool.isRequired,onRowSelected:g.default.func,options:g.default.object.isRequired,pageSize:g.default.number,renderData:g.default.array,initialFormData:g.default.object,selection:g.default.bool.isRequired,showAddRow:g.default.bool,treeDataMaxLevel:g.default.number,localization:g.default.object,onFilterChanged:g.default.func,onGroupExpandChanged:g.default.func,onToggleDetailPanel:g.default.func.isRequired,onTreeExpandChanged:g.default.func.isRequired,onRowClick:g.default.func,onEditingCanceled:g.default.func,onEditingApproved:g.default.func};var b=y;t.default=b},function(e,t,n){"use strict";var r=n(46),i=n(36);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(223)),a=i(n(98)),s=i(n(50)),l=i(n(114)),u=i(n(33)),c=i(n(34)),d=i(n(9)),h=i(n(45)),f=i(n(43)),p=i(n(44)),g=i(n(8)),v=i(n(200)),m=i(n(136)),y=i(n(154)),b=i(n(73)),C=i(n(221)),A=i(n(185)),_=i(n(3)),I=r(n(0)),x=r(n(423));function w(e){var t=function(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],(function(){}))),!0}catch(e){return!1}}();return function(){var n,r=(0,p.default)(e);if(t){var i=(0,p.default)(this).constructor;n=Reflect.construct(r,arguments,i)}else n=r.apply(this,arguments);return(0,f.default)(this,n)}}var M=function(e){(0,h.default)(n,e);var t=w(n);function n(){var e;(0,u.default)(this,n);for(var r=arguments.length,i=new Array(r),o=0;o<r;o++)i[o]=arguments[o];return e=t.call.apply(t,[this].concat(i)),(0,g.default)((0,d.default)(e),"rotateIconStyle",(function(e){return{transform:e?"rotate(90deg)":"none"}})),e}return(0,c.default)(n,[{key:"renderColumns",value:function(){var e=this,t=x.elementSize(this.props);return this.props.columns.filter((function(e){return!(e.hidden||e.tableData.groupOrder>-1)})).sort((function(e,t){return e.tableData.columnOrder-t.tableData.columnOrder})).map((function(n,r){var i=e.props.getFieldValue(e.props.data,n);return I.createElement(e.props.components.Cell,{size:t,icons:e.props.icons,columnDef:(0,l.default)({cellStyle:e.props.options.cellStyle},n),value:i,key:"cell-"+e.props.data.tableData.id+"-"+n.tableData.id,rowData:e.props.data})}))}},{key:"renderActions",value:function(){var e=x.elementSize(this.props),t=x.rowActions(this.props),n=t.length*x.baseIconSize(this.props);return I.createElement(m.default,{size:e,padding:"none",key:"key-actions-column",style:(0,l.default)({width:n,padding:"0px 5px",boxSizing:"border-box"},this.props.options.actionsCellStyle)},I.createElement("div",{style:{display:"flex"}},I.createElement(this.props.components.Actions,{data:this.props.data,actions:t,components:this.props.components,size:e,disabled:this.props.hasAnyEditingRow})))}},{key:"renderSelectionColumn",value:function(){var e=this,t=this.props.options.selectionProps||{};"function"==typeof t&&(t=t(this.props.data));var n=x.elementSize(this.props),r=x.selectionMaxWidth(this.props,this.props.treeDataMaxLevel),i="medium"===n?{marginLeft:9*this.props.level}:{padding:"4px",marginLeft:5+9*this.props.level};return I.createElement(m.default,{size:n,padding:"none",key:"key-selection-column",style:{width:r}},I.createElement(v.default,(0,s.default)({size:n,checked:!0===this.props.data.tableData.checked,onClick:function(e){return e.stopPropagation()},value:this.props.data.tableData.id.toString(),onChange:function(t){return e.props.onRowSelected(t,e.props.path,e.props.data)},style:i},t)))}},{key:"renderDetailPanelColumn",value:function(){var e=this,t=x.elementSize(this.props),n=function(e){var t=e.icon,n=e.iconProps;return"string"==typeof t?I.createElement(C.default,n,t):I.createElement(t,(0,l.default)({},n))};return"function"==typeof this.props.detailPanel?I.createElement(m.default,{size:t,padding:"none",key:"key-detail-panel-column",style:(0,l.default)({width:42,textAlign:"center"},this.props.options.detailPanelColumnStyle)},I.createElement(b.default,{size:t,style:(0,l.default)({transition:"all ease 200ms"},this.rotateIconStyle(this.props.data.tableData.showDetailPanel)),onClick:function(t){e.props.onToggleDetailPanel(e.props.path,e.props.detailPanel),t.stopPropagation()}},I.createElement(this.props.icons.DetailPanel,null))):I.createElement(m.default,{size:t,padding:"none",key:"key-detail-panel-column"},I.createElement("div",{style:(0,l.default)({width:42*this.props.detailPanel.length,textAlign:"center",display:"flex"},this.props.options.detailPanelColumnStyle)},this.props.detailPanel.map((function(r,i){"function"==typeof r&&(r=r(e.props.data));var o=(e.props.data.tableData.showDetailPanel||"").toString()===r.render.toString(),a=I.createElement(e.props.icons.DetailPanel,null),s=!0;return o?r.openIcon?(a=I.createElement(n,{icon:r.openIcon,iconProps:r.iconProps}),s=!1):r.icon&&(a=I.createElement(n,{icon:r.icon,iconProps:r.iconProps})):r.icon&&(a=I.createElement(n,{icon:r.icon,iconProps:r.iconProps}),s=!1),a=I.createElement(b.default,{size:t,key:"key-detail-panel-"+i,style:(0,l.default)({transition:"all ease 200ms"},e.rotateIconStyle(s&&o)),disabled:r.disabled,onClick:function(t){e.props.onToggleDetailPanel(e.props.path,r.render),t.stopPropagation()}},a),r.tooltip&&(a=I.createElement(A.default,{key:"key-detail-panel-"+i,title:r.tooltip},a)),a}))))}},{key:"renderTreeDataColumn",value:function(){var e=this,t=x.elementSize(this.props);return this.props.data.tableData.childRows&&this.props.data.tableData.childRows.length>0?I.createElement(m.default,{size:t,padding:"none",key:"key-tree-data-column",style:{width:48+9*(this.props.treeDataMaxLevel-2)}},I.createElement(b.default,{size:t,style:(0,l.default)({transition:"all ease 200ms",marginLeft:9*this.props.level},this.rotateIconStyle(this.props.data.tableData.isTreeExpanded)),onClick:function(t){e.props.onTreeExpandChanged(e.props.path,e.props.data),t.stopPropagation()}},I.createElement(this.props.icons.DetailPanel,null))):I.createElement(m.default,{padding:"none",key:"key-tree-data-column"})}},{key:"getStyle",value:function(e,t){var n={transition:"all ease 300ms"};return"function"==typeof this.props.options.rowStyle?n=(0,l.default)({},n,this.props.options.rowStyle(this.props.data,e,t)):this.props.options.rowStyle&&(n=(0,l.default)({},n,this.props.options.rowStyle)),this.props.onRowClick&&(n.cursor="pointer"),this.props.hasAnyEditingRow&&(n.opacity=.2),n}},{key:"render",value:function(){var e=this,t=x.elementSize(this.props),n=this.renderColumns();if(this.props.options.selection&&n.splice(0,0,this.renderSelectionColumn()),this.props.actions&&this.props.actions.filter((function(e){return"row"===e.position||"function"==typeof e})).length>0)if(-1===this.props.options.actionsColumnIndex)n.push(this.renderActions());else if(this.props.options.actionsColumnIndex>=0){var r=0;this.props.options.selection&&(r=1),n.splice(this.props.options.actionsColumnIndex+r,0,this.renderActions())}this.props.detailPanel&&("right"===this.props.options.detailPanelColumnAlignment?n.push(this.renderDetailPanelColumn()):n.splice(0,0,this.renderDetailPanelColumn())),this.props.isTreeData&&n.splice(0,0,this.renderTreeDataColumn()),this.props.columns.filter((function(e){return e.tableData.groupOrder>-1})).forEach((function(e){n.splice(0,0,I.createElement(m.default,{size:t,padding:"none",key:"key-group-cell"+e.tableData.id}))}));var i=this.props,l=(i.icons,i.data,i.columns,i.components,i.detailPanel),u=(i.getFieldValue,i.isTreeData,i.onRowClick),c=(i.onRowSelected,i.onTreeExpandChanged,i.onToggleDetailPanel),d=i.onEditingCanceled,h=i.onEditingApproved,f=(i.options,i.hasAnyEditingRow),p=i.treeDataMaxLevel,g=(i.localization,i.actions,(0,a.default)(i,["icons","data","columns","components","detailPanel","getFieldValue","isTreeData","onRowClick","onRowSelected","onTreeExpandChanged","onToggleDetailPanel","onEditingCanceled","onEditingApproved","options","hasAnyEditingRow","treeDataMaxLevel","localization","actions"]));return I.createElement(I.Fragment,null,I.createElement(y.default,(0,s.default)({selected:f},g,{hover:!!u,style:this.getStyle(this.props.index,this.props.level),onClick:function(t){u&&u(t,e.props.data,(function(t){var n=l;Array.isArray(n)&&("function"==typeof(n=n[t||0])&&(n=n(e.props.data)),n=n.render),c(e.props.path,n)}))}}),n),this.props.data.tableData&&this.props.data.tableData.showDetailPanel&&I.createElement(y.default,null,I.createElement(m.default,{size:t,colSpan:n.length,padding:"none"},this.props.data.tableData.showDetailPanel(this.props.data))),this.props.data.tableData.childRows&&this.props.data.tableData.isTreeExpanded&&this.props.data.tableData.childRows.map((function(t,n){return t.tableData.editing?I.createElement(e.props.components.EditRow,{columns:e.props.columns.filter((function(e){return!e.hidden})),components:e.props.components,data:t,icons:e.props.icons,localization:e.props.localization,getFieldValue:e.props.getFieldValue,key:n,mode:t.tableData.editing,options:e.props.options,isTreeData:e.props.isTreeData,detailPanel:e.props.detailPanel,onEditingCanceled:d,onEditingApproved:h}):I.createElement(e.props.components.Row,(0,s.default)({},e.props,{data:t,index:n,key:n,level:e.props.level+1,path:[].concat((0,o.default)(e.props.path),[n]),onEditingCanceled:d,onEditingApproved:h,hasAnyEditingRow:e.props.hasAnyEditingRow,treeDataMaxLevel:p}))})))}}]),n}(I.Component);t.default=M,M.defaultProps={actions:[],index:0,data:{},options:{},path:[]},M.propTypes={actions:_.default.array,icons:_.default.any.isRequired,index:_.default.number.isRequired,data:_.default.object.isRequired,detailPanel:_.default.oneOfType([_.default.func,_.default.arrayOf(_.default.oneOfType([_.default.object,_.default.func]))]),hasAnyEditingRow:_.default.bool,options:_.default.object.isRequired,onRowSelected:_.default.func,path:_.default.arrayOf(_.default.number),treeDataMaxLevel:_.default.number,getFieldValue:_.default.func.isRequired,columns:_.default.array,onToggleDetailPanel:_.default.func.isRequired,onRowClick:_.default.func,onEditingApproved:_.default.func,onEditingCanceled:_.default.func}},function(e,t,n){"use strict";var r=n(46),i=n(36);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(50)),a=i(n(114)),s=i(n(33)),l=i(n(34)),u=i(n(9)),c=i(n(45)),d=i(n(43)),h=i(n(44)),f=i(n(8)),p=i(n(121)),g=i(n(313)),v=i(n(62)),m=i(n(3)),y=r(n(0)),b=n(621);function C(e){var t=function(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],(function(){}))),!0}catch(e){return!1}}();return function(){var n,r=(0,h.default)(e);if(t){var i=(0,h.default)(this).constructor;n=Reflect.construct(r,arguments,i)}else n=r.apply(this,arguments);return(0,d.default)(this,n)}}var A=function(e){(0,c.default)(n,e);var t=C(n);function n(e){var r;return(0,s.default)(this,n),r=t.call(this,e),(0,f.default)((0,u.default)(r),"getItemStyle",(function(e,t){return(0,a.default)({userSelect:"none",margin:"0 ".concat(8,"px 0 0")},t)})),(0,f.default)((0,u.default)(r),"getListStyle",(function(e){return{background:"#0000000a",display:"flex",width:"100%",padding:8,overflow:"auto",border:"1px solid #ccc",borderStyle:"dashed"}})),r.state={},r}return(0,l.default)(n,[{key:"render",value:function(){var e=this;return y.createElement(p.default,{style:{padding:0,minHeight:"unset"}},y.createElement(b.Droppable,{droppableId:"groups",direction:"horizontal",placeholder:"Deneme"},(function(t,n){return y.createElement("div",{ref:t.innerRef,style:e.getListStyle(n.isDraggingOver)},e.props.groupColumns.length>0&&y.createElement(v.default,{variant:"caption",style:{padding:8}},e.props.localization.groupedBy),e.props.groupColumns.map((function(t,n){return y.createElement(b.Draggable,{key:t.tableData.id,draggableId:t.tableData.id.toString(),index:n},(function(r,i){return y.createElement("div",(0,o.default)({ref:r.innerRef},r.draggableProps,r.dragHandleProps,{style:e.getItemStyle(i.isDragging,r.draggableProps.style)}),y.createElement(g.default,(0,o.default)({},r.dragHandleProps,{onClick:function(){return e.props.onSortChanged(t)},label:y.createElement("div",{style:{display:"flex",alignItems:"center"}},y.createElement("div",{style:{float:"left"}},t.title),t.tableData.groupSort&&y.createElement(e.props.icons.SortArrow,{style:{transition:"300ms ease all",transform:"asc"===t.tableData.groupSort?"rotate(-180deg)":"none",fontSize:18}})),style:{boxShadow:"none",textTransform:"none"},onDelete:function(){return e.props.onGroupRemoved(t,n)}})))}))})),0===e.props.groupColumns.length&&y.createElement(v.default,{variant:"caption",style:{padding:8}},e.props.localization.placeholder),t.placeholder)})))}}]),n}(y.Component);A.defaultProps={},A.propTypes={localization:m.default.shape({groupedBy:m.default.string,placeholder:m.default.string})};var _=A;t.default=_},function(e,t,n){"use strict";var r=n(46),i=n(36);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(114)),a=i(n(223)),s=i(n(33)),l=i(n(34)),u=i(n(9)),c=i(n(45)),d=i(n(43)),h=i(n(44)),f=i(n(8)),p=i(n(136)),g=i(n(154)),v=i(n(73)),m=i(n(3)),y=r(n(0));function b(e){var t=function(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],(function(){}))),!0}catch(e){return!1}}();return function(){var n,r=(0,h.default)(e);if(t){var i=(0,h.default)(this).constructor;n=Reflect.construct(r,arguments,i)}else n=r.apply(this,arguments);return(0,d.default)(this,n)}}var C=function(e){(0,c.default)(n,e);var t=b(n);function n(){var e;(0,s.default)(this,n);for(var r=arguments.length,i=new Array(r),o=0;o<r;o++)i[o]=arguments[o];return e=t.call.apply(t,[this].concat(i)),(0,f.default)((0,u.default)(e),"rotateIconStyle",(function(e){return{transform:e?"rotate(90deg)":"none"}})),e}return(0,l.default)(n,[{key:"render",value:function(){var e=this,t=this.props.columns.filter((function(e){return!e.hidden})).length;this.props.options.selection&&t++,this.props.detailPanel&&t++,this.props.actions&&this.props.actions.length>0&&t++;var n,r=this.props.groups[this.props.level];this.props.groupData.isExpanded&&(n=this.props.groups.length>this.props.level+1?this.props.groupData.groups.map((function(t,n){return y.createElement(e.props.components.GroupRow,{actions:e.props.actions,key:t.value||""+n,columns:e.props.columns,components:e.props.components,detailPanel:e.props.detailPanel,getFieldValue:e.props.getFieldValue,groupData:t,groups:e.props.groups,icons:e.props.icons,level:e.props.level+1,path:[].concat((0,a.default)(e.props.path),[n]),onGroupExpandChanged:e.props.onGroupExpandChanged,onRowSelected:e.props.onRowSelected,onRowClick:e.props.onRowClick,onToggleDetailPanel:e.props.onToggleDetailPanel,onTreeExpandChanged:e.props.onTreeExpandChanged,onEditingCanceled:e.props.onEditingCanceled,onEditingApproved:e.props.onEditingApproved,options:e.props.options,hasAnyEditingRow:e.props.hasAnyEditingRow,isTreeData:e.props.isTreeData})})):this.props.groupData.data.map((function(t,n){return t.tableData.editing?y.createElement(e.props.components.EditRow,{columns:e.props.columns,components:e.props.components,data:t,icons:e.props.icons,path:[].concat((0,a.default)(e.props.path),[n]),localization:e.props.localization,key:n,mode:t.tableData.editing,options:e.props.options,isTreeData:e.props.isTreeData,detailPanel:e.props.detailPanel,onEditingCanceled:e.props.onEditingCanceled,onEditingApproved:e.props.onEditingApproved,getFieldValue:e.props.getFieldValue}):y.createElement(e.props.components.Row,{actions:e.props.actions,key:n,columns:e.props.columns,components:e.props.components,data:t,detailPanel:e.props.detailPanel,getFieldValue:e.props.getFieldValue,icons:e.props.icons,path:[].concat((0,a.default)(e.props.path),[n]),onRowSelected:e.props.onRowSelected,onRowClick:e.props.onRowClick,onToggleDetailPanel:e.props.onToggleDetailPanel,options:e.props.options,isTreeData:e.props.isTreeData,onTreeExpandChanged:e.props.onTreeExpandChanged,onEditingCanceled:e.props.onEditingCanceled,onEditingApproved:e.props.onEditingApproved,hasAnyEditingRow:e.props.hasAnyEditingRow})})));for(var i=[],s=0;s<this.props.level;s++)i.push(y.createElement(p.default,{padding:"checkbox",key:s}));var l=this.props.groupData.value;r.lookup&&(l=r.lookup[l]);var u=r.title;"function"==typeof this.props.options.groupTitle?u=this.props.options.groupTitle(this.props.groupData):"string"!=typeof u&&(u=y.cloneElement(u));var c=this.props.options.groupRowSeparator||": ";return y.createElement(y.Fragment,null,y.createElement(g.default,null,i,y.createElement(this.props.components.Cell,{colSpan:t,padding:"none",columnDef:r,value:l,icons:this.props.icons},y.createElement(v.default,{style:(0,o.default)({transition:"all ease 200ms"},this.rotateIconStyle(this.props.groupData.isExpanded)),onClick:function(t){e.props.onGroupExpandChanged(e.props.path)}},y.createElement(this.props.icons.DetailPanel,null)),y.createElement("b",null,u,c))),n)}}]),n}(y.Component);t.default=C,C.defaultProps={columns:[],groups:[],options:{},level:0},C.propTypes={actions:m.default.array,columns:m.default.arrayOf(m.default.object),components:m.default.object,detailPanel:m.default.oneOfType([m.default.func,m.default.arrayOf(m.default.object)]),getFieldValue:m.default.func,groupData:m.default.object,groups:m.default.arrayOf(m.default.object),hasAnyEditingRow:m.default.bool,icons:m.default.object,isTreeData:m.default.bool.isRequired,level:m.default.number,localization:m.default.object,onGroupExpandChanged:m.default.func,onRowSelected:m.default.func,onRowClick:m.default.func,onToggleDetailPanel:m.default.func.isRequired,onTreeExpandChanged:m.default.func.isRequired,onEditingCanceled:m.default.func,onEditingApproved:m.default.func,options:m.default.object,path:m.default.arrayOf(m.default.number)}},function(e,t,n){"use strict";var r=n(46),i=n(36);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(50)),a=i(n(98)),s=i(n(114)),l=i(n(33)),u=i(n(34)),c=i(n(9)),d=i(n(45)),h=i(n(43)),f=i(n(44)),p=i(n(8)),g=r(n(0)),v=i(n(136)),m=i(n(3));function y(e){var t=function(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],(function(){}))),!0}catch(e){return!1}}();return function(){var n,r=(0,f.default)(e);if(t){var i=(0,f.default)(this).constructor;n=Reflect.construct(r,arguments,i)}else n=r.apply(this,arguments);return(0,h.default)(this,n)}}var b=/^\d{4}-(0[1-9]|1[0-2])-([12]\d|0[1-9]|3[01])([T\s](([01]\d|2[0-3])\:[0-5]\d|24\:00)(\:[0-5]\d([\.,]\d+)?)?([zZ]|([\+-])([01]\d|2[0-3])\:?([0-5]\d)?)?)?$/,C=function(e){(0,d.default)(n,e);var t=y(n);function n(){var e;(0,l.default)(this,n);for(var r=arguments.length,i=new Array(r),o=0;o<r;o++)i[o]=arguments[o];return e=t.call.apply(t,[this].concat(i)),(0,p.default)((0,c.default)(e),"handleClickCell",(function(t){e.props.columnDef.disableClick&&t.stopPropagation()})),(0,p.default)((0,c.default)(e),"getStyle",(function(){var t={color:"inherit",width:e.props.columnDef.tableData.width,boxSizing:"border-box",fontSize:"inherit",fontFamily:"inherit",fontWeight:"inherit"};return t="function"==typeof e.props.columnDef.cellStyle?(0,s.default)({},t,e.props.columnDef.cellStyle(e.props.value,e.props.rowData)):(0,s.default)({},t,e.props.columnDef.cellStyle),e.props.columnDef.disableClick&&(t.cursor="default"),(0,s.default)({},e.props.style,t)})),e}return(0,u.default)(n,[{key:"getRenderValue",value:function(){var e=this.props.columnDef.dateSetting&&this.props.columnDef.dateSetting.locale?this.props.columnDef.dateSetting.locale:void 0;if(void 0!==this.props.columnDef.emptyValue&&(void 0===this.props.value||null===this.props.value))return this.getEmptyValue(this.props.columnDef.emptyValue);if(this.props.columnDef.render)return this.props.rowData?this.props.columnDef.render(this.props.rowData,"row"):this.props.columnDef.render(this.props.value,"group");if("boolean"===this.props.columnDef.type){var t={textAlign:"left",verticalAlign:"middle",width:48};return this.props.value?g.createElement(this.props.icons.Check,{style:t}):g.createElement(this.props.icons.ThirdStateCheck,{style:t})}return"date"===this.props.columnDef.type?this.props.value instanceof Date?this.props.value.toLocaleDateString():b.exec(this.props.value)?new Date(this.props.value).toLocaleDateString(e):this.props.value:"time"===this.props.columnDef.type?this.props.value instanceof Date?this.props.value.toLocaleTimeString():b.exec(this.props.value)?new Date(this.props.value).toLocaleTimeString(e):this.props.value:"datetime"===this.props.columnDef.type?this.props.value instanceof Date?this.props.value.toLocaleString():b.exec(this.props.value)?new Date(this.props.value).toLocaleString(e):this.props.value:"currency"===this.props.columnDef.type?this.getCurrencyValue(this.props.columnDef.currencySetting,this.props.value):"boolean"==typeof this.props.value?this.props.value.toString():this.props.value}},{key:"getEmptyValue",value:function(e){return"function"==typeof e?this.props.columnDef.emptyValue(this.props.rowData):e}},{key:"getCurrencyValue",value:function(e,t){return void 0!==e?new Intl.NumberFormat(void 0!==e.locale?e.locale:"en-US",{style:"currency",currency:void 0!==e.currencyCode?e.currencyCode:"USD",minimumFractionDigits:void 0!==e.minimumFractionDigits?e.minimumFractionDigits:2,maximumFractionDigits:void 0!==e.maximumFractionDigits?e.maximumFractionDigits:2}).format(void 0!==t?t:0):new Intl.NumberFormat("en-US",{style:"currency",currency:"USD"}).format(void 0!==t?t:0)}},{key:"render",value:function(){var e=this.props,t=(e.icons,e.columnDef),n=(e.rowData,(0,a.default)(e,["icons","columnDef","rowData"])),r=void 0!==t.align?t.align:-1!==["numeric","currency"].indexOf(this.props.columnDef.type)?"right":"left";return g.createElement(v.default,(0,o.default)({size:this.props.size},n,{style:this.getStyle(),align:r,onClick:this.handleClickCell}),this.props.children,this.getRenderValue())}}]),n}(g.Component);t.default=C,C.defaultProps={columnDef:{},value:void 0},C.propTypes={columnDef:m.default.object.isRequired,value:m.default.any,rowData:m.default.object}},function(e,t,n){"use strict";var r=n(46),i=n(36);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(50)),a=i(n(98)),s=i(n(114)),l=i(n(33)),u=i(n(34)),c=i(n(9)),d=i(n(45)),h=i(n(43)),f=i(n(44)),p=i(n(8)),g=i(n(136)),v=i(n(154)),m=i(n(62)),y=i(n(3)),b=r(n(0)),C=n(1177),A=r(n(423));function _(e){var t=function(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],(function(){}))),!0}catch(e){return!1}}();return function(){var n,r=(0,f.default)(e);if(t){var i=(0,f.default)(this).constructor;n=Reflect.construct(r,arguments,i)}else n=r.apply(this,arguments);return(0,h.default)(this,n)}}var I=function(e){(0,d.default)(n,e);var t=_(n);function n(e){var r;return(0,l.default)(this,n),r=t.call(this,e),(0,p.default)((0,c.default)(r),"cancelEdit",(function(e){27===e.keyCode&&r.props.onEditingCanceled(r.props.mode,r.props.data)})),r.state={data:e.data?JSON.parse(JSON.stringify(e.data)):r.createRowData()},r}return(0,u.default)(n,[{key:"createRowData",value:function(){return this.props.columns.filter((function(e){return(e.initialEditValue||0===e.initialEditValue)&&e.field})).reduce((function(e,t){return e[t.field]=t.initialEditValue,e}),{})}},{key:"renderColumns",value:function(){var e=this,t=A.elementSize(this.props);return this.props.columns.filter((function(e){return!(e.hidden||e.tableData.groupOrder>-1)})).sort((function(e,t){return e.tableData.columnOrder-t.tableData.columnOrder})).map((function(n,r){var i=void 0!==e.state.data[n.field]?e.state.data[n.field]:(0,C.byString)(e.state.data,n.field),o=function(t,n){var r={color:"inherit"};return r="function"==typeof t.cellStyle?(0,s.default)({},r,t.cellStyle(n,e.props.data)):(0,s.default)({},r,t.cellStyle),t.disableClick&&(r.cursor="default"),(0,s.default)({},r)};0===r&&e.props.level;var l=!1;if(void 0===n.editable&&(l=!0),"always"===n.editable&&(l=!0),"onAdd"===n.editable&&"add"===e.props.mode&&(l=!0),"onUpdate"===n.editable&&"update"===e.props.mode&&(l=!0),"function"==typeof n.editable&&(l=n.editable(n,e.props.data)),n.field&&l){var u=n.editComponent,c=(0,a.default)(n,["editComponent"]),d=u||e.props.components.EditField;return b.createElement(g.default,{size:t,key:n.tableData.id,align:-1!==["numeric"].indexOf(n.type)?"right":"left",style:o(n,i)},b.createElement(d,{key:n.tableData.id,columnDef:c,value:i,locale:e.props.localization.dateTimePickerLocalization,rowData:e.state.data,onChange:function(t){var r=(0,s.default)({},e.state.data);(0,C.setByString)(r,n.field,t),e.setState({data:r})},onRowDataChange:function(t){e.setState({data:t})}}))}var h=e.props.getFieldValue(e.state.data,n);return b.createElement(e.props.components.Cell,{size:t,icons:e.props.icons,columnDef:n,value:h,key:n.tableData.id,rowData:e.props.data,style:o(n,i)})}))}},{key:"renderActions",value:function(){var e=this,t=A.elementSize(this.props),r=(0,s.default)({},n.defaultProps.localization,this.props.localization),i=[{icon:this.props.icons.Check,tooltip:r.saveTooltip,onClick:function(){delete e.state.data.tableData,e.props.onEditingApproved(e.props.mode,e.state.data,e.props.data)}},{icon:this.props.icons.Clear,tooltip:r.cancelTooltip,onClick:function(){e.props.onEditingCanceled(e.props.mode,e.props.data)}}];return b.createElement(g.default,{size:t,padding:"none",key:"key-actions-column",style:(0,s.default)({width:42*i.length,padding:"0px 5px"},this.props.options.editCellStyle)},b.createElement("div",{style:{display:"flex"}},b.createElement(this.props.components.Actions,{data:this.props.data,actions:i,components:this.props.components,size:t})))}},{key:"getStyle",value:function(){return{borderBottom:"1px solid red"}}},{key:"render",value:function(){var e,t=A.elementSize(this.props),r=(0,s.default)({},n.defaultProps.localization,this.props.localization);if("add"===this.props.mode||"update"===this.props.mode)e=this.renderColumns();else{var i=this.props.columns.filter((function(e){return!(e.hidden||e.tableData.groupOrder>-1)})).length;e=[b.createElement(g.default,{size:t,padding:0===this.props.options.actionsColumnIndex?"none":void 0,key:"key-edit-cell",colSpan:i},b.createElement(m.default,{variant:"h6"},r.deleteText))]}if(this.props.options.selection&&e.splice(0,0,b.createElement(g.default,{padding:"none",key:"key-selection-cell"})),this.props.isTreeData&&e.splice(0,0,b.createElement(g.default,{padding:"none",key:"key-tree-data-cell"})),-1===this.props.options.actionsColumnIndex)e.push(this.renderActions());else if(this.props.options.actionsColumnIndex>=0){var l=0;this.props.options.selection&&(l=1),this.props.isTreeData&&(l=1,this.props.options.selection&&e.splice(1,1)),e.splice(this.props.options.actionsColumnIndex+l,0,this.renderActions())}if(this.props.detailPanel){var u="left"===this.props.options.detailPanelColumnAlignment?0:e.length;e.splice(u,0,b.createElement(g.default,{padding:"none",key:"key-detail-panel-cell"}))}this.props.columns.filter((function(e){return e.tableData.groupOrder>-1})).forEach((function(t){e.splice(0,0,b.createElement(g.default,{padding:"none",key:"key-group-cell"+t.tableData.id}))}));var c=this.props,d=(c.detailPanel,c.isTreeData,c.onRowClick,c.onRowSelected,c.onTreeExpandChanged,c.onToggleDetailPanel,c.onEditingApproved,c.onEditingCanceled,c.getFieldValue,c.components,c.icons,c.columns,c.localization,c.options,c.actions,(0,a.default)(c,["detailPanel","isTreeData","onRowClick","onRowSelected","onTreeExpandChanged","onToggleDetailPanel","onEditingApproved","onEditingCanceled","getFieldValue","components","icons","columns","localization","options","actions"]));return b.createElement(b.Fragment,null,b.createElement(v.default,(0,o.default)({onKeyDown:this.cancelEdit},d,{style:this.getStyle()}),e))}}]),n}(b.Component);t.default=I,I.defaultProps={actions:[],index:0,options:{},path:[],localization:{saveTooltip:"Save",cancelTooltip:"Cancel",deleteText:"Are you sure you want to delete this row?"}},I.propTypes={actions:y.default.array,icons:y.default.any.isRequired,index:y.default.number.isRequired,data:y.default.object,detailPanel:y.default.oneOfType([y.default.func,y.default.arrayOf(y.default.oneOfType([y.default.object,y.default.func]))]),options:y.default.object.isRequired,onRowSelected:y.default.func,path:y.default.arrayOf(y.default.number),columns:y.default.array,onRowClick:y.default.func,onEditingApproved:y.default.func,onEditingCanceled:y.default.func,localization:y.default.object,getFieldValue:y.default.func}},function(e,t,n){"use strict";var r=n(46),i=n(36);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(50)),a=i(n(98)),s=i(n(33)),l=i(n(34)),u=i(n(45)),c=i(n(43)),d=i(n(44)),h=r(n(0)),f=i(n(88)),p=i(n(200)),g=i(n(132)),v=i(n(120)),m=i(n(1178)),y=n(1192),b=i(n(3));function C(e){var t=function(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],(function(){}))),!0}catch(e){return!1}}();return function(){var n,r=(0,d.default)(e);if(t){var i=(0,d.default)(this).constructor;n=Reflect.construct(r,arguments,i)}else n=r.apply(this,arguments);return(0,c.default)(this,n)}}var A=function(e){(0,u.default)(n,e);var t=C(n);function n(){return(0,s.default)(this,n),t.apply(this,arguments)}return(0,l.default)(n,[{key:"getProps",value:function(){var e=this.props;e.columnDef,e.rowData,e.onRowDataChange;return(0,a.default)(e,["columnDef","rowData","onRowDataChange"])}},{key:"renderLookupField",value:function(){var e=this;return h.createElement(g.default,(0,o.default)({},this.getProps(),{value:void 0===this.props.value?"":this.props.value,onChange:function(t){return e.props.onChange(t.target.value)},style:{fontSize:13},SelectDisplayProps:{"aria-label":this.props.columnDef.title}}),Object.keys(this.props.columnDef.lookup).map((function(t){return h.createElement(v.default,{key:t,value:t},e.props.columnDef.lookup[t])})))}},{key:"renderBooleanField",value:function(){var e=this;return h.createElement(p.default,(0,o.default)({},this.getProps(),{value:String(this.props.value),checked:Boolean(this.props.value),onChange:function(t){return e.props.onChange(t.target.checked)},style:{paddingLeft:0,paddingTop:0,paddingBottom:0},inputProps:{"aria-label":this.props.columnDef.title}}))}},{key:"renderDateField",value:function(){return h.createElement(y.MuiPickersUtilsProvider,{utils:m.default,locale:this.props.dateTimePickerLocalization},h.createElement(y.DatePicker,(0,o.default)({},this.getProps(),{format:"dd.MM.yyyy",value:this.props.value||null,onChange:this.props.onChange,clearable:!0,InputProps:{style:{fontSize:13}},inputProps:{"aria-label":"".concat(this.props.columnDef.title,": press space to edit")}})))}},{key:"renderTimeField",value:function(){return h.createElement(y.MuiPickersUtilsProvider,{utils:m.default,locale:this.props.dateTimePickerLocalization},h.createElement(y.TimePicker,(0,o.default)({},this.getProps(),{format:"HH:mm:ss",value:this.props.value||null,onChange:this.props.onChange,clearable:!0,InputProps:{style:{fontSize:13},inputProps:{"aria-label":"".concat(this.props.columnDef.title,": press space to edit")}}})))}},{key:"renderDateTimeField",value:function(){return h.createElement(y.MuiPickersUtilsProvider,{utils:m.default,locale:this.props.dateTimePickerLocalization},h.createElement(y.DateTimePicker,(0,o.default)({},this.getProps(),{format:"dd.MM.yyyy HH:mm:ss",value:this.props.value||null,onChange:this.props.onChange,clearable:!0,InputProps:{style:{fontSize:13},inputProps:{"aria-label":"".concat(this.props.columnDef.title,": press space to edit")}}})))}},{key:"renderTextField",value:function(){var e=this;return h.createElement(f.default,(0,o.default)({},this.getProps(),{style:"numeric"===this.props.columnDef.type?{float:"right"}:{},type:"numeric"===this.props.columnDef.type?"number":"text",placeholder:this.props.columnDef.editPlaceholder||this.props.columnDef.title,value:void 0===this.props.value?"":this.props.value,onChange:function(t){return e.props.onChange(t.target.value)},InputProps:{style:{fontSize:13},inputProps:{"aria-label":this.props.columnDef.title}}}))}},{key:"renderCurrencyField",value:function(){var e=this;return h.createElement(f.default,(0,o.default)({},this.getProps(),{placeholder:this.props.columnDef.editPlaceholder||this.props.columnDef.title,value:void 0===this.props.value?"":this.props.value,onChange:function(t){return e.props.onChange(t.target.value)},inputProps:{style:{fontSize:13,textAlign:"right","aria-label":this.props.columnDef.title}}}))}},{key:"render",value:function(){return this.props.columnDef.lookup?this.renderLookupField():"boolean"===this.props.columnDef.type?this.renderBooleanField():"date"===this.props.columnDef.type?this.renderDateField():"time"===this.props.columnDef.type?this.renderTimeField():"datetime"===this.props.columnDef.type?this.renderDateTimeField():"currency"===this.props.columnDef.type?this.renderCurrencyField():this.renderTextField()}}]),n}(h.Component);A.propTypes={value:b.default.any,onChange:b.default.func.isRequired,columnDef:b.default.object.isRequired,dateTimePickerLocalization:b.default.object};var _=A;t.default=_},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){(0,i.default)(1,arguments);var t=(0,r.default)(e);return t.setMinutes(0,0,0),t};var r=o(n(47)),i=o(n(40));function o(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t,n){var i;n=n||{},i="string"==typeof r[e]?r[e]:1===t?r[e].one:r[e].other.replace("{{count}}",t);if(n.addSuffix)return n.comparison>0?"in "+i:i+" ago";return i};var r={lessThanXSeconds:{one:"less than a second",other:"less than {{count}} seconds"},xSeconds:{one:"1 second",other:"{{count}} seconds"},halfAMinute:"half a minute",lessThanXMinutes:{one:"less than a minute",other:"less than {{count}} minutes"},xMinutes:{one:"1 minute",other:"{{count}} minutes"},aboutXHours:{one:"about 1 hour",other:"about {{count}} hours"},xHours:{one:"1 hour",other:"{{count}} hours"},xDays:{one:"1 day",other:"{{count}} days"},aboutXWeeks:{one:"about 1 week",other:"about {{count}} weeks"},xWeeks:{one:"1 week",other:"{{count}} weeks"},aboutXMonths:{one:"about 1 month",other:"about {{count}} months"},xMonths:{one:"1 month",other:"{{count}} months"},aboutXYears:{one:"about 1 year",other:"about {{count}} years"},xYears:{one:"1 year",other:"{{count}} years"},overXYears:{one:"over 1 year",other:"over {{count}} years"},almostXYears:{one:"almost 1 year",other:"almost {{count}} years"}};e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var r,i=(r=n(2253))&&r.__esModule?r:{default:r};var o={date:(0,i.default)({formats:{full:"EEEE, MMMM do, y",long:"MMMM do, y",medium:"MMM d, y",short:"MM/dd/yyyy"},defaultWidth:"full"}),time:(0,i.default)({formats:{full:"h:mm:ss a zzzz",long:"h:mm:ss a z",medium:"h:mm:ss a",short:"h:mm a"},defaultWidth:"full"}),dateTime:(0,i.default)({formats:{full:"{{date}} 'at' {{time}}",long:"{{date}} 'at' {{time}}",medium:"{{date}}, {{time}}",short:"{{date}}, {{time}}"},defaultWidth:"full"})};t.default=o,e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){return function(t){var n=t||{},r=n.width?String(n.width):e.defaultWidth;return e.formats[r]||e.formats[e.defaultWidth]}},e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t,n,i){return r[e]};var r={lastWeek:"'last' eeee 'at' p",yesterday:"'yesterday at' p",today:"'today at' p",tomorrow:"'tomorrow at' p",nextWeek:"eeee 'at' p",other:"P"};e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var r,i=(r=n(2256))&&r.__esModule?r:{default:r};var o={ordinalNumber:function(e,t){var n=Number(e),r=n%100;if(r>20||r<10)switch(r%10){case 1:return n+"st";case 2:return n+"nd";case 3:return n+"rd"}return n+"th"},era:(0,i.default)({values:{narrow:["B","A"],abbreviated:["BC","AD"],wide:["Before Christ","Anno Domini"]},defaultWidth:"wide"}),quarter:(0,i.default)({values:{narrow:["1","2","3","4"],abbreviated:["Q1","Q2","Q3","Q4"],wide:["1st quarter","2nd quarter","3rd quarter","4th quarter"]},defaultWidth:"wide",argumentCallback:function(e){return Number(e)-1}}),month:(0,i.default)({values:{narrow:["J","F","M","A","M","J","J","A","S","O","N","D"],abbreviated:["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"],wide:["January","February","March","April","May","June","July","August","September","October","November","December"]},defaultWidth:"wide"}),day:(0,i.default)({values:{narrow:["S","M","T","W","T","F","S"],short:["Su","Mo","Tu","We","Th","Fr","Sa"],abbreviated:["Sun","Mon","Tue","Wed","Thu","Fri","Sat"],wide:["Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"]},defaultWidth:"wide"}),dayPeriod:(0,i.default)({values:{narrow:{am:"a",pm:"p",midnight:"mi",noon:"n",morning:"morning",afternoon:"afternoon",evening:"evening",night:"night"},abbreviated:{am:"AM",pm:"PM",midnight:"midnight",noon:"noon",morning:"morning",afternoon:"afternoon",evening:"evening",night:"night"},wide:{am:"a.m.",pm:"p.m.",midnight:"midnight",noon:"noon",morning:"morning",afternoon:"afternoon",evening:"evening",night:"night"}},defaultWidth:"wide",formattingValues:{narrow:{am:"a",pm:"p",midnight:"mi",noon:"n",morning:"in the morning",afternoon:"in the afternoon",evening:"in the evening",night:"at night"},abbreviated:{am:"AM",pm:"PM",midnight:"midnight",noon:"noon",morning:"in the morning",afternoon:"in the afternoon",evening:"in the evening",night:"at night"},wide:{am:"a.m.",pm:"p.m.",midnight:"midnight",noon:"noon",morning:"in the morning",afternoon:"in the afternoon",evening:"in the evening",night:"at night"}},defaultFormattingWidth:"wide"})};t.default=o,e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){return function(t,n){var r,i=n||{};if("formatting"===(i.context?String(i.context):"standalone")&&e.formattingValues){var o=e.defaultFormattingWidth||e.defaultWidth,a=i.width?String(i.width):o;r=e.formattingValues[a]||e.formattingValues[o]}else{var s=e.defaultWidth,l=i.width?String(i.width):e.defaultWidth;r=e.values[l]||e.values[s]}return r[e.argumentCallback?e.argumentCallback(t):t]}},e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var r=o(n(2258)),i=o(n(2259));function o(e){return e&&e.__esModule?e:{default:e}}var a={ordinalNumber:(0,r.default)({matchPattern:/^(\d+)(th|st|nd|rd)?/i,parsePattern:/\d+/i,valueCallback:function(e){return parseInt(e,10)}}),era:(0,i.default)({matchPatterns:{narrow:/^(b|a)/i,abbreviated:/^(b\.?\s?c\.?|b\.?\s?c\.?\s?e\.?|a\.?\s?d\.?|c\.?\s?e\.?)/i,wide:/^(before christ|before common era|anno domini|common era)/i},defaultMatchWidth:"wide",parsePatterns:{any:[/^b/i,/^(a|c)/i]},defaultParseWidth:"any"}),quarter:(0,i.default)({matchPatterns:{narrow:/^[1234]/i,abbreviated:/^q[1234]/i,wide:/^[1234](th|st|nd|rd)? quarter/i},defaultMatchWidth:"wide",parsePatterns:{any:[/1/i,/2/i,/3/i,/4/i]},defaultParseWidth:"any",valueCallback:function(e){return e+1}}),month:(0,i.default)({matchPatterns:{narrow:/^[jfmasond]/i,abbreviated:/^(jan|feb|mar|apr|may|jun|jul|aug|sep|oct|nov|dec)/i,wide:/^(january|february|march|april|may|june|july|august|september|october|november|december)/i},defaultMatchWidth:"wide",parsePatterns:{narrow:[/^j/i,/^f/i,/^m/i,/^a/i,/^m/i,/^j/i,/^j/i,/^a/i,/^s/i,/^o/i,/^n/i,/^d/i],any:[/^ja/i,/^f/i,/^mar/i,/^ap/i,/^may/i,/^jun/i,/^jul/i,/^au/i,/^s/i,/^o/i,/^n/i,/^d/i]},defaultParseWidth:"any"}),day:(0,i.default)({matchPatterns:{narrow:/^[smtwf]/i,short:/^(su|mo|tu|we|th|fr|sa)/i,abbreviated:/^(sun|mon|tue|wed|thu|fri|sat)/i,wide:/^(sunday|monday|tuesday|wednesday|thursday|friday|saturday)/i},defaultMatchWidth:"wide",parsePatterns:{narrow:[/^s/i,/^m/i,/^t/i,/^w/i,/^t/i,/^f/i,/^s/i],any:[/^su/i,/^m/i,/^tu/i,/^w/i,/^th/i,/^f/i,/^sa/i]},defaultParseWidth:"any"}),dayPeriod:(0,i.default)({matchPatterns:{narrow:/^(a|p|mi|n|(in the|at) (morning|afternoon|evening|night))/i,any:/^([ap]\.?\s?m\.?|midnight|noon|(in the|at) (morning|afternoon|evening|night))/i},defaultMatchWidth:"any",parsePatterns:{any:{am:/^a/i,pm:/^p/i,midnight:/^mi/i,noon:/^no/i,morning:/morning/i,afternoon:/afternoon/i,evening:/evening/i,night:/night/i}},defaultParseWidth:"any"})};t.default=a,e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){return function(t,n){var r=String(t),i=n||{},o=r.match(e.matchPattern);if(!o)return null;var a=o[0],s=r.match(e.parsePattern);if(!s)return null;var l=e.valueCallback?e.valueCallback(s[0]):s[0];return{value:l=i.valueCallback?i.valueCallback(l):l,rest:r.slice(a.length)}}},e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){return function(t,n){var r=String(t),i=n||{},o=i.width,a=o&&e.matchPatterns[o]||e.matchPatterns[e.defaultMatchWidth],s=r.match(a);if(!s)return null;var l,u=s[0],c=o&&e.parsePatterns[o]||e.parsePatterns[e.defaultParseWidth];return l="[object Array]"===Object.prototype.toString.call(c)?function(e,t){for(var n=0;n<e.length;n++)if(t(e[n]))return n}(c,(function(e){return e.test(u)})):function(e,t){for(var n in e)if(e.hasOwnProperty(n)&&t(e[n]))return n}(c,(function(e){return e.test(u)})),l=e.valueCallback?e.valueCallback(l):l,{value:l=i.valueCallback?i.valueCallback(l):l,rest:r.slice(u.length)}}},e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){(0,o.default)(2,arguments);var n=(0,i.default)(e).getTime(),a=(0,r.default)(t);return new Date(n+a)};var r=a(n(77)),i=a(n(47)),o=a(n(40));function a(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){if(null==e)throw new TypeError("assign requires that input parameter not be null or undefined");for(var n in t=t||{})t.hasOwnProperty(n)&&(e[n]=t[n]);return e},e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var r=c(n(617)),i=c(n(2263)),o=c(n(2264)),a=c(n(2265)),s=c(n(2267)),l=c(n(425)),u=c(n(424));function c(e){return e&&e.__esModule?e:{default:e}}var d=/^(1[0-2]|0?\d)/,h=/^(3[0-1]|[0-2]?\d)/,f=/^(36[0-6]|3[0-5]\d|[0-2]?\d?\d)/,p=/^(5[0-3]|[0-4]?\d)/,g=/^(2[0-3]|[0-1]?\d)/,v=/^(2[0-4]|[0-1]?\d)/,m=/^(1[0-1]|0?\d)/,y=/^(1[0-2]|0?\d)/,b=/^[0-5]?\d/,C=/^[0-5]?\d/,A=/^\d/,_=/^\d{1,2}/,I=/^\d{1,3}/,x=/^\d{1,4}/,w=/^-?\d+/,M=/^-?\d/,S=/^-?\d{1,2}/,T=/^-?\d{1,3}/,E=/^-?\d{1,4}/,O=/^([+-])(\d{2})(\d{2})?|Z/,k=/^([+-])(\d{2})(\d{2})|Z/,L=/^([+-])(\d{2})(\d{2})((\d{2}))?|Z/,D=/^([+-])(\d{2}):(\d{2})|Z/,P=/^([+-])(\d{2}):(\d{2})(:(\d{2}))?|Z/;function N(e,t,n){var r=t.match(e);if(!r)return null;var i=parseInt(r[0],10);return{value:n?n(i):i,rest:t.slice(r[0].length)}}function j(e,t){var n=t.match(e);return n?"Z"===n[0]?{value:0,rest:t.slice(1)}:{value:("+"===n[1]?1:-1)*(36e5*(n[2]?parseInt(n[2],10):0)+6e4*(n[3]?parseInt(n[3],10):0)+1e3*(n[5]?parseInt(n[5],10):0)),rest:t.slice(n[0].length)}:null}function R(e,t){return N(w,e,t)}function z(e,t,n){switch(e){case 1:return N(A,t,n);case 2:return N(_,t,n);case 3:return N(I,t,n);case 4:return N(x,t,n);default:return N(new RegExp("^\\d{1,"+e+"}"),t,n)}}function F(e,t,n){switch(e){case 1:return N(M,t,n);case 2:return N(S,t,n);case 3:return N(T,t,n);case 4:return N(E,t,n);default:return N(new RegExp("^-?\\d{1,"+e+"}"),t,n)}}function B(e){switch(e){case"morning":return 4;case"evening":return 17;case"pm":case"noon":case"afternoon":return 12;case"am":case"midnight":case"night":default:return 0}}function H(e,t){var n,r=t>0,i=r?t:1-t;if(i<=50)n=e||100;else{var o=i+50;n=e+100*Math.floor(o/100)-(e>=o%100?100:0)}return r?n:1-n}var U=[31,28,31,30,31,30,31,31,30,31,30,31],V=[31,29,31,30,31,30,31,31,30,31,30,31];function Y(e){return e%400==0||e%4==0&&e%100!=0}var W={G:{priority:140,parse:function(e,t,n,r){switch(t){case"G":case"GG":case"GGG":return n.era(e,{width:"abbreviated"})||n.era(e,{width:"narrow"});case"GGGGG":return n.era(e,{width:"narrow"});case"GGGG":default:return n.era(e,{width:"wide"})||n.era(e,{width:"abbreviated"})||n.era(e,{width:"narrow"})}},set:function(e,t,n,r){return t.era=n,e.setUTCFullYear(n,0,1),e.setUTCHours(0,0,0,0),e},incompatibleTokens:["R","u","t","T"]},y:{priority:130,parse:function(e,t,n,r){var i=function(e){return{year:e,isTwoDigitYear:"yy"===t}};switch(t){case"y":return z(4,e,i);case"yo":return n.ordinalNumber(e,{unit:"year",valueCallback:i});default:return z(t.length,e,i)}},validate:function(e,t,n){return t.isTwoDigitYear||t.year>0},set:function(e,t,n,r){var i=e.getUTCFullYear();if(n.isTwoDigitYear){var o=H(n.year,i);return e.setUTCFullYear(o,0,1),e.setUTCHours(0,0,0,0),e}var a="era"in t&&1!==t.era?1-n.year:n.year;return e.setUTCFullYear(a,0,1),e.setUTCHours(0,0,0,0),e},incompatibleTokens:["Y","R","u","w","I","i","e","c","t","T"]},Y:{priority:130,parse:function(e,t,n,r){var i=function(e){return{year:e,isTwoDigitYear:"YY"===t}};switch(t){case"Y":return z(4,e,i);case"Yo":return n.ordinalNumber(e,{unit:"year",valueCallback:i});default:return z(t.length,e,i)}},validate:function(e,t,n){return t.isTwoDigitYear||t.year>0},set:function(e,t,n,i){var o=(0,r.default)(e,i);if(n.isTwoDigitYear){var a=H(n.year,o);return e.setUTCFullYear(a,0,i.firstWeekContainsDate),e.setUTCHours(0,0,0,0),(0,u.default)(e,i)}var s="era"in t&&1!==t.era?1-n.year:n.year;return e.setUTCFullYear(s,0,i.firstWeekContainsDate),e.setUTCHours(0,0,0,0),(0,u.default)(e,i)},incompatibleTokens:["y","R","u","Q","q","M","L","I","d","D","i","t","T"]},R:{priority:130,parse:function(e,t,n,r){return F("R"===t?4:t.length,e)},set:function(e,t,n,r){var i=new Date(0);return i.setUTCFullYear(n,0,4),i.setUTCHours(0,0,0,0),(0,l.default)(i)},incompatibleTokens:["G","y","Y","u","Q","q","M","L","w","d","D","e","c","t","T"]},u:{priority:130,parse:function(e,t,n,r){return F("u"===t?4:t.length,e)},set:function(e,t,n,r){return e.setUTCFullYear(n,0,1),e.setUTCHours(0,0,0,0),e},incompatibleTokens:["G","y","Y","R","w","I","i","e","c","t","T"]},Q:{priority:120,parse:function(e,t,n,r){switch(t){case"Q":case"QQ":return z(t.length,e);case"Qo":return n.ordinalNumber(e,{unit:"quarter"});case"QQQ":return n.quarter(e,{width:"abbreviated",context:"formatting"})||n.quarter(e,{width:"narrow",context:"formatting"});case"QQQQQ":return n.quarter(e,{width:"narrow",context:"formatting"});case"QQQQ":default:return n.quarter(e,{width:"wide",context:"formatting"})||n.quarter(e,{width:"abbreviated",context:"formatting"})||n.quarter(e,{width:"narrow",context:"formatting"})}},validate:function(e,t,n){return t>=1&&t<=4},set:function(e,t,n,r){return e.setUTCMonth(3*(n-1),1),e.setUTCHours(0,0,0,0),e},incompatibleTokens:["Y","R","q","M","L","w","I","d","D","i","e","c","t","T"]},q:{priority:120,parse:function(e,t,n,r){switch(t){case"q":case"qq":return z(t.length,e);case"qo":return n.ordinalNumber(e,{unit:"quarter"});case"qqq":return n.quarter(e,{width:"abbreviated",context:"standalone"})||n.quarter(e,{width:"narrow",context:"standalone"});case"qqqqq":return n.quarter(e,{width:"narrow",context:"standalone"});case"qqqq":default:return n.quarter(e,{width:"wide",context:"standalone"})||n.quarter(e,{width:"abbreviated",context:"standalone"})||n.quarter(e,{width:"narrow",context:"standalone"})}},validate:function(e,t,n){return t>=1&&t<=4},set:function(e,t,n,r){return e.setUTCMonth(3*(n-1),1),e.setUTCHours(0,0,0,0),e},incompatibleTokens:["Y","R","Q","M","L","w","I","d","D","i","e","c","t","T"]},M:{priority:110,parse:function(e,t,n,r){var i=function(e){return e-1};switch(t){case"M":return N(d,e,i);case"MM":return z(2,e,i);case"Mo":return n.ordinalNumber(e,{unit:"month",valueCallback:i});case"MMM":return n.month(e,{width:"abbreviated",context:"formatting"})||n.month(e,{width:"narrow",context:"formatting"});case"MMMMM":return n.month(e,{width:"narrow",context:"formatting"});case"MMMM":default:return n.month(e,{width:"wide",context:"formatting"})||n.month(e,{width:"abbreviated",context:"formatting"})||n.month(e,{width:"narrow",context:"formatting"})}},validate:function(e,t,n){return t>=0&&t<=11},set:function(e,t,n,r){return e.setUTCMonth(n,1),e.setUTCHours(0,0,0,0),e},incompatibleTokens:["Y","R","q","Q","L","w","I","D","i","e","c","t","T"]},L:{priority:110,parse:function(e,t,n,r){var i=function(e){return e-1};switch(t){case"L":return N(d,e,i);case"LL":return z(2,e,i);case"Lo":return n.ordinalNumber(e,{unit:"month",valueCallback:i});case"LLL":return n.month(e,{width:"abbreviated",context:"standalone"})||n.month(e,{width:"narrow",context:"standalone"});case"LLLLL":return n.month(e,{width:"narrow",context:"standalone"});case"LLLL":default:return n.month(e,{width:"wide",context:"standalone"})||n.month(e,{width:"abbreviated",context:"standalone"})||n.month(e,{width:"narrow",context:"standalone"})}},validate:function(e,t,n){return t>=0&&t<=11},set:function(e,t,n,r){return e.setUTCMonth(n,1),e.setUTCHours(0,0,0,0),e},incompatibleTokens:["Y","R","q","Q","M","w","I","D","i","e","c","t","T"]},w:{priority:100,parse:function(e,t,n,r){switch(t){case"w":return N(p,e);case"wo":return n.ordinalNumber(e,{unit:"week"});default:return z(t.length,e)}},validate:function(e,t,n){return t>=1&&t<=53},set:function(e,t,n,r){return(0,u.default)((0,s.default)(e,n,r),r)},incompatibleTokens:["y","R","u","q","Q","M","L","I","d","D","i","t","T"]},I:{priority:100,parse:function(e,t,n,r){switch(t){case"I":return N(p,e);case"Io":return n.ordinalNumber(e,{unit:"week"});default:return z(t.length,e)}},validate:function(e,t,n){return t>=1&&t<=53},set:function(e,t,n,r){return(0,l.default)((0,a.default)(e,n,r),r)},incompatibleTokens:["y","Y","u","q","Q","M","L","w","d","D","e","c","t","T"]},d:{priority:90,subPriority:1,parse:function(e,t,n,r){switch(t){case"d":return N(h,e);case"do":return n.ordinalNumber(e,{unit:"date"});default:return z(t.length,e)}},validate:function(e,t,n){var r=Y(e.getUTCFullYear()),i=e.getUTCMonth();return r?t>=1&&t<=V[i]:t>=1&&t<=U[i]},set:function(e,t,n,r){return e.setUTCDate(n),e.setUTCHours(0,0,0,0),e},incompatibleTokens:["Y","R","q","Q","w","I","D","i","e","c","t","T"]},D:{priority:90,subPriority:1,parse:function(e,t,n,r){switch(t){case"D":case"DD":return N(f,e);case"Do":return n.ordinalNumber(e,{unit:"date"});default:return z(t.length,e)}},validate:function(e,t,n){return Y(e.getUTCFullYear())?t>=1&&t<=366:t>=1&&t<=365},set:function(e,t,n,r){return e.setUTCMonth(0,n),e.setUTCHours(0,0,0,0),e},incompatibleTokens:["Y","R","q","Q","M","L","w","I","d","E","i","e","c","t","T"]},E:{priority:90,parse:function(e,t,n,r){switch(t){case"E":case"EE":case"EEE":return n.day(e,{width:"abbreviated",context:"formatting"})||n.day(e,{width:"short",context:"formatting"})||n.day(e,{width:"narrow",context:"formatting"});case"EEEEE":return n.day(e,{width:"narrow",context:"formatting"});case"EEEEEE":return n.day(e,{width:"short",context:"formatting"})||n.day(e,{width:"narrow",context:"formatting"});case"EEEE":default:return n.day(e,{width:"wide",context:"formatting"})||n.day(e,{width:"abbreviated",context:"formatting"})||n.day(e,{width:"short",context:"formatting"})||n.day(e,{width:"narrow",context:"formatting"})}},validate:function(e,t,n){return t>=0&&t<=6},set:function(e,t,n,r){return(e=(0,i.default)(e,n,r)).setUTCHours(0,0,0,0),e},incompatibleTokens:["D","i","e","c","t","T"]},e:{priority:90,parse:function(e,t,n,r){var i=function(e){var t=7*Math.floor((e-1)/7);return(e+r.weekStartsOn+6)%7+t};switch(t){case"e":case"ee":return z(t.length,e,i);case"eo":return n.ordinalNumber(e,{unit:"day",valueCallback:i});case"eee":return n.day(e,{width:"abbreviated",context:"formatting"})||n.day(e,{width:"short",context:"formatting"})||n.day(e,{width:"narrow",context:"formatting"});case"eeeee":return n.day(e,{width:"narrow",context:"formatting"});case"eeeeee":return n.day(e,{width:"short",context:"formatting"})||n.day(e,{width:"narrow",context:"formatting"});case"eeee":default:return n.day(e,{width:"wide",context:"formatting"})||n.day(e,{width:"abbreviated",context:"formatting"})||n.day(e,{width:"short",context:"formatting"})||n.day(e,{width:"narrow",context:"formatting"})}},validate:function(e,t,n){return t>=0&&t<=6},set:function(e,t,n,r){return(e=(0,i.default)(e,n,r)).setUTCHours(0,0,0,0),e},incompatibleTokens:["y","R","u","q","Q","M","L","I","d","D","E","i","c","t","T"]},c:{priority:90,parse:function(e,t,n,r){var i=function(e){var t=7*Math.floor((e-1)/7);return(e+r.weekStartsOn+6)%7+t};switch(t){case"c":case"cc":return z(t.length,e,i);case"co":return n.ordinalNumber(e,{unit:"day",valueCallback:i});case"ccc":return n.day(e,{width:"abbreviated",context:"standalone"})||n.day(e,{width:"short",context:"standalone"})||n.day(e,{width:"narrow",context:"standalone"});case"ccccc":return n.day(e,{width:"narrow",context:"standalone"});case"cccccc":return n.day(e,{width:"short",context:"standalone"})||n.day(e,{width:"narrow",context:"standalone"});case"cccc":default:return n.day(e,{width:"wide",context:"standalone"})||n.day(e,{width:"abbreviated",context:"standalone"})||n.day(e,{width:"short",context:"standalone"})||n.day(e,{width:"narrow",context:"standalone"})}},validate:function(e,t,n){return t>=0&&t<=6},set:function(e,t,n,r){return(e=(0,i.default)(e,n,r)).setUTCHours(0,0,0,0),e},incompatibleTokens:["y","R","u","q","Q","M","L","I","d","D","E","i","e","t","T"]},i:{priority:90,parse:function(e,t,n,r){var i=function(e){return 0===e?7:e};switch(t){case"i":case"ii":return z(t.length,e);case"io":return n.ordinalNumber(e,{unit:"day"});case"iii":return n.day(e,{width:"abbreviated",context:"formatting",valueCallback:i})||n.day(e,{width:"short",context:"formatting",valueCallback:i})||n.day(e,{width:"narrow",context:"formatting",valueCallback:i});case"iiiii":return n.day(e,{width:"narrow",context:"formatting",valueCallback:i});case"iiiiii":return n.day(e,{width:"short",context:"formatting",valueCallback:i})||n.day(e,{width:"narrow",context:"formatting",valueCallback:i});case"iiii":default:return n.day(e,{width:"wide",context:"formatting",valueCallback:i})||n.day(e,{width:"abbreviated",context:"formatting",valueCallback:i})||n.day(e,{width:"short",context:"formatting",valueCallback:i})||n.day(e,{width:"narrow",context:"formatting",valueCallback:i})}},validate:function(e,t,n){return t>=1&&t<=7},set:function(e,t,n,r){return(e=(0,o.default)(e,n,r)).setUTCHours(0,0,0,0),e},incompatibleTokens:["y","Y","u","q","Q","M","L","w","d","D","E","e","c","t","T"]},a:{priority:80,parse:function(e,t,n,r){switch(t){case"a":case"aa":case"aaa":return n.dayPeriod(e,{width:"abbreviated",context:"formatting"})||n.dayPeriod(e,{width:"narrow",context:"formatting"});case"aaaaa":return n.dayPeriod(e,{width:"narrow",context:"formatting"});case"aaaa":default:return n.dayPeriod(e,{width:"wide",context:"formatting"})||n.dayPeriod(e,{width:"abbreviated",context:"formatting"})||n.dayPeriod(e,{width:"narrow",context:"formatting"})}},set:function(e,t,n,r){return e.setUTCHours(B(n),0,0,0),e},incompatibleTokens:["b","B","H","K","k","t","T"]},b:{priority:80,parse:function(e,t,n,r){switch(t){case"b":case"bb":case"bbb":return n.dayPeriod(e,{width:"abbreviated",context:"formatting"})||n.dayPeriod(e,{width:"narrow",context:"formatting"});case"bbbbb":return n.dayPeriod(e,{width:"narrow",context:"formatting"});case"bbbb":default:return n.dayPeriod(e,{width:"wide",context:"formatting"})||n.dayPeriod(e,{width:"abbreviated",context:"formatting"})||n.dayPeriod(e,{width:"narrow",context:"formatting"})}},set:function(e,t,n,r){return e.setUTCHours(B(n),0,0,0),e},incompatibleTokens:["a","B","H","K","k","t","T"]},B:{priority:80,parse:function(e,t,n,r){switch(t){case"B":case"BB":case"BBB":return n.dayPeriod(e,{width:"abbreviated",context:"formatting"})||n.dayPeriod(e,{width:"narrow",context:"formatting"});case"BBBBB":return n.dayPeriod(e,{width:"narrow",context:"formatting"});case"BBBB":default:return n.dayPeriod(e,{width:"wide",context:"formatting"})||n.dayPeriod(e,{width:"abbreviated",context:"formatting"})||n.dayPeriod(e,{width:"narrow",context:"formatting"})}},set:function(e,t,n,r){return e.setUTCHours(B(n),0,0,0),e},incompatibleTokens:["a","b","t","T"]},h:{priority:70,parse:function(e,t,n,r){switch(t){case"h":return N(y,e);case"ho":return n.ordinalNumber(e,{unit:"hour"});default:return z(t.length,e)}},validate:function(e,t,n){return t>=1&&t<=12},set:function(e,t,n,r){var i=e.getUTCHours()>=12;return i&&n<12?e.setUTCHours(n+12,0,0,0):i||12!==n?e.setUTCHours(n,0,0,0):e.setUTCHours(0,0,0,0),e},incompatibleTokens:["H","K","k","t","T"]},H:{priority:70,parse:function(e,t,n,r){switch(t){case"H":return N(g,e);case"Ho":return n.ordinalNumber(e,{unit:"hour"});default:return z(t.length,e)}},validate:function(e,t,n){return t>=0&&t<=23},set:function(e,t,n,r){return e.setUTCHours(n,0,0,0),e},incompatibleTokens:["a","b","h","K","k","t","T"]},K:{priority:70,parse:function(e,t,n,r){switch(t){case"K":return N(m,e);case"Ko":return n.ordinalNumber(e,{unit:"hour"});default:return z(t.length,e)}},validate:function(e,t,n){return t>=0&&t<=11},set:function(e,t,n,r){return e.getUTCHours()>=12&&n<12?e.setUTCHours(n+12,0,0,0):e.setUTCHours(n,0,0,0),e},incompatibleTokens:["a","b","h","H","k","t","T"]},k:{priority:70,parse:function(e,t,n,r){switch(t){case"k":return N(v,e);case"ko":return n.ordinalNumber(e,{unit:"hour"});default:return z(t.length,e)}},validate:function(e,t,n){return t>=1&&t<=24},set:function(e,t,n,r){var i=n<=24?n%24:n;return e.setUTCHours(i,0,0,0),e},incompatibleTokens:["a","b","h","H","K","t","T"]},m:{priority:60,parse:function(e,t,n,r){switch(t){case"m":return N(b,e);case"mo":return n.ordinalNumber(e,{unit:"minute"});default:return z(t.length,e)}},validate:function(e,t,n){return t>=0&&t<=59},set:function(e,t,n,r){return e.setUTCMinutes(n,0,0),e},incompatibleTokens:["t","T"]},s:{priority:50,parse:function(e,t,n,r){switch(t){case"s":return N(C,e);case"so":return n.ordinalNumber(e,{unit:"second"});default:return z(t.length,e)}},validate:function(e,t,n){return t>=0&&t<=59},set:function(e,t,n,r){return e.setUTCSeconds(n,0),e},incompatibleTokens:["t","T"]},S:{priority:30,parse:function(e,t,n,r){return z(t.length,e,(function(e){return Math.floor(e*Math.pow(10,3-t.length))}))},set:function(e,t,n,r){return e.setUTCMilliseconds(n),e},incompatibleTokens:["t","T"]},X:{priority:10,parse:function(e,t,n,r){switch(t){case"X":return j(O,e);case"XX":return j(k,e);case"XXXX":return j(L,e);case"XXXXX":return j(P,e);case"XXX":default:return j(D,e)}},set:function(e,t,n,r){return t.timestampIsSet?e:new Date(e.getTime()-n)},incompatibleTokens:["t","T","x"]},x:{priority:10,parse:function(e,t,n,r){switch(t){case"x":return j(O,e);case"xx":return j(k,e);case"xxxx":return j(L,e);case"xxxxx":return j(P,e);case"xxx":default:return j(D,e)}},set:function(e,t,n,r){return t.timestampIsSet?e:new Date(e.getTime()-n)},incompatibleTokens:["t","T","X"]},t:{priority:40,parse:function(e,t,n,r){return R(e)},set:function(e,t,n,r){return[new Date(1e3*n),{timestampIsSet:!0}]},incompatibleTokens:"*"},T:{priority:20,parse:function(e,t,n,r){return R(e)},set:function(e,t,n,r){return[new Date(n),{timestampIsSet:!0}]},incompatibleTokens:"*"}};t.default=W,e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t,n){(0,o.default)(2,arguments);var a=n||{},s=a.locale,l=s&&s.options&&s.options.weekStartsOn,u=null==l?0:(0,r.default)(l),c=null==a.weekStartsOn?u:(0,r.default)(a.weekStartsOn);if(!(c>=0&&c<=6))throw new RangeError("weekStartsOn must be between 0 and 6 inclusively");var d=(0,i.default)(e),h=(0,r.default)(t),f=d.getUTCDay(),p=h%7,g=(p+7)%7,v=(g<c?7:0)+h-f;return d.setUTCDate(d.getUTCDate()+v),d};var r=a(n(77)),i=a(n(47)),o=a(n(40));function a(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){(0,o.default)(2,arguments);var n=(0,r.default)(t);n%7==0&&(n-=7);var a=1,s=(0,i.default)(e),l=s.getUTCDay(),u=n%7,c=(u+7)%7,d=(c<a?7:0)+n-l;return s.setUTCDate(s.getUTCDate()+d),s};var r=a(n(77)),i=a(n(47)),o=a(n(40));function a(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){(0,a.default)(2,arguments);var n=(0,i.default)(e),s=(0,r.default)(t),l=(0,o.default)(n)-s;return n.setUTCDate(n.getUTCDate()-7*l),n};var r=s(n(77)),i=s(n(47)),o=s(n(1184)),a=s(n(40));function s(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){(0,o.default)(1,arguments);var t=(0,r.default)(e),n=new Date(0);n.setUTCFullYear(t,0,4),n.setUTCHours(0,0,0,0);var a=(0,i.default)(n);return a};var r=a(n(1185)),i=a(n(425)),o=a(n(40));function a(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t,n){(0,a.default)(2,arguments);var s=(0,i.default)(e),l=(0,r.default)(t),u=(0,o.default)(s,n)-l;return s.setUTCDate(s.getUTCDate()-7*u),s};var r=s(n(77)),i=s(n(47)),o=s(n(1186)),a=s(n(40));function s(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){(0,a.default)(1,arguments);var n=t||{},s=n.locale,l=s&&s.options&&s.options.firstWeekContainsDate,u=null==l?1:(0,r.default)(l),c=null==n.firstWeekContainsDate?u:(0,r.default)(n.firstWeekContainsDate),d=(0,i.default)(e,t),h=new Date(0);h.setUTCFullYear(d,0,c),h.setUTCHours(0,0,0,0);var f=(0,o.default)(h,t);return f};var r=s(n(77)),i=s(n(617)),o=s(n(424)),a=s(n(40));function s(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var r=c(n(2270)),i=c(n(2271)),o=c(n(1184)),a=c(n(1185)),s=c(n(1186)),l=c(n(617)),u=c(n(1187));function c(e){return e&&e.__esModule?e:{default:e}}var d="midnight",h="noon",f="morning",p="afternoon",g="evening",v="night";function m(e,t){var n=e>0?"-":"+",r=Math.abs(e),i=Math.floor(r/60),o=r%60;if(0===o)return n+String(i);var a=t||"";return n+String(i)+a+(0,u.default)(o,2)}function y(e,t){return e%60==0?(e>0?"-":"+")+(0,u.default)(Math.abs(e)/60,2):b(e,t)}function b(e,t){var n=t||"",r=e>0?"-":"+",i=Math.abs(e);return r+(0,u.default)(Math.floor(i/60),2)+n+(0,u.default)(i%60,2)}var C={G:function(e,t,n){var r=e.getUTCFullYear()>0?1:0;switch(t){case"G":case"GG":case"GGG":return n.era(r,{width:"abbreviated"});case"GGGGG":return n.era(r,{width:"narrow"});case"GGGG":default:return n.era(r,{width:"wide"})}},y:function(e,t,n){if("yo"===t){var i=e.getUTCFullYear(),o=i>0?i:1-i;return n.ordinalNumber(o,{unit:"year"})}return r.default.y(e,t)},Y:function(e,t,n,r){var i=(0,l.default)(e,r),o=i>0?i:1-i;if("YY"===t){var a=o%100;return(0,u.default)(a,2)}return"Yo"===t?n.ordinalNumber(o,{unit:"year"}):(0,u.default)(o,t.length)},R:function(e,t){var n=(0,a.default)(e);return(0,u.default)(n,t.length)},u:function(e,t){var n=e.getUTCFullYear();return(0,u.default)(n,t.length)},Q:function(e,t,n){var r=Math.ceil((e.getUTCMonth()+1)/3);switch(t){case"Q":return String(r);case"QQ":return(0,u.default)(r,2);case"Qo":return n.ordinalNumber(r,{unit:"quarter"});case"QQQ":return n.quarter(r,{width:"abbreviated",context:"formatting"});case"QQQQQ":return n.quarter(r,{width:"narrow",context:"formatting"});case"QQQQ":default:return n.quarter(r,{width:"wide",context:"formatting"})}},q:function(e,t,n){var r=Math.ceil((e.getUTCMonth()+1)/3);switch(t){case"q":return String(r);case"qq":return(0,u.default)(r,2);case"qo":return n.ordinalNumber(r,{unit:"quarter"});case"qqq":return n.quarter(r,{width:"abbreviated",context:"standalone"});case"qqqqq":return n.quarter(r,{width:"narrow",context:"standalone"});case"qqqq":default:return n.quarter(r,{width:"wide",context:"standalone"})}},M:function(e,t,n){var i=e.getUTCMonth();switch(t){case"M":case"MM":return r.default.M(e,t);case"Mo":return n.ordinalNumber(i+1,{unit:"month"});case"MMM":return n.month(i,{width:"abbreviated",context:"formatting"});case"MMMMM":return n.month(i,{width:"narrow",context:"formatting"});case"MMMM":default:return n.month(i,{width:"wide",context:"formatting"})}},L:function(e,t,n){var r=e.getUTCMonth();switch(t){case"L":return String(r+1);case"LL":return(0,u.default)(r+1,2);case"Lo":return n.ordinalNumber(r+1,{unit:"month"});case"LLL":return n.month(r,{width:"abbreviated",context:"standalone"});case"LLLLL":return n.month(r,{width:"narrow",context:"standalone"});case"LLLL":default:return n.month(r,{width:"wide",context:"standalone"})}},w:function(e,t,n,r){var i=(0,s.default)(e,r);return"wo"===t?n.ordinalNumber(i,{unit:"week"}):(0,u.default)(i,t.length)},I:function(e,t,n){var r=(0,o.default)(e);return"Io"===t?n.ordinalNumber(r,{unit:"week"}):(0,u.default)(r,t.length)},d:function(e,t,n){return"do"===t?n.ordinalNumber(e.getUTCDate(),{unit:"date"}):r.default.d(e,t)},D:function(e,t,n){var r=(0,i.default)(e);return"Do"===t?n.ordinalNumber(r,{unit:"dayOfYear"}):(0,u.default)(r,t.length)},E:function(e,t,n){var r=e.getUTCDay();switch(t){case"E":case"EE":case"EEE":return n.day(r,{width:"abbreviated",context:"formatting"});case"EEEEE":return n.day(r,{width:"narrow",context:"formatting"});case"EEEEEE":return n.day(r,{width:"short",context:"formatting"});case"EEEE":default:return n.day(r,{width:"wide",context:"formatting"})}},e:function(e,t,n,r){var i=e.getUTCDay(),o=(i-r.weekStartsOn+8)%7||7;switch(t){case"e":return String(o);case"ee":return(0,u.default)(o,2);case"eo":return n.ordinalNumber(o,{unit:"day"});case"eee":return n.day(i,{width:"abbreviated",context:"formatting"});case"eeeee":return n.day(i,{width:"narrow",context:"formatting"});case"eeeeee":return n.day(i,{width:"short",context:"formatting"});case"eeee":default:return n.day(i,{width:"wide",context:"formatting"})}},c:function(e,t,n,r){var i=e.getUTCDay(),o=(i-r.weekStartsOn+8)%7||7;switch(t){case"c":return String(o);case"cc":return(0,u.default)(o,t.length);case"co":return n.ordinalNumber(o,{unit:"day"});case"ccc":return n.day(i,{width:"abbreviated",context:"standalone"});case"ccccc":return n.day(i,{width:"narrow",context:"standalone"});case"cccccc":return n.day(i,{width:"short",context:"standalone"});case"cccc":default:return n.day(i,{width:"wide",context:"standalone"})}},i:function(e,t,n){var r=e.getUTCDay(),i=0===r?7:r;switch(t){case"i":return String(i);case"ii":return(0,u.default)(i,t.length);case"io":return n.ordinalNumber(i,{unit:"day"});case"iii":return n.day(r,{width:"abbreviated",context:"formatting"});case"iiiii":return n.day(r,{width:"narrow",context:"formatting"});case"iiiiii":return n.day(r,{width:"short",context:"formatting"});case"iiii":default:return n.day(r,{width:"wide",context:"formatting"})}},a:function(e,t,n){var r=e.getUTCHours()/12>=1?"pm":"am";switch(t){case"a":case"aa":return n.dayPeriod(r,{width:"abbreviated",context:"formatting"});case"aaa":return n.dayPeriod(r,{width:"abbreviated",context:"formatting"}).toLowerCase();case"aaaaa":return n.dayPeriod(r,{width:"narrow",context:"formatting"});case"aaaa":default:return n.dayPeriod(r,{width:"wide",context:"formatting"})}},b:function(e,t,n){var r,i=e.getUTCHours();switch(r=12===i?h:0===i?d:i/12>=1?"pm":"am",t){case"b":case"bb":return n.dayPeriod(r,{width:"abbreviated",context:"formatting"});case"bbb":return n.dayPeriod(r,{width:"abbreviated",context:"formatting"}).toLowerCase();case"bbbbb":return n.dayPeriod(r,{width:"narrow",context:"formatting"});case"bbbb":default:return n.dayPeriod(r,{width:"wide",context:"formatting"})}},B:function(e,t,n){var r,i=e.getUTCHours();switch(r=i>=17?g:i>=12?p:i>=4?f:v,t){case"B":case"BB":case"BBB":return n.dayPeriod(r,{width:"abbreviated",context:"formatting"});case"BBBBB":return n.dayPeriod(r,{width:"narrow",context:"formatting"});case"BBBB":default:return n.dayPeriod(r,{width:"wide",context:"formatting"})}},h:function(e,t,n){if("ho"===t){var i=e.getUTCHours()%12;return 0===i&&(i=12),n.ordinalNumber(i,{unit:"hour"})}return r.default.h(e,t)},H:function(e,t,n){return"Ho"===t?n.ordinalNumber(e.getUTCHours(),{unit:"hour"}):r.default.H(e,t)},K:function(e,t,n){var r=e.getUTCHours()%12;return"Ko"===t?n.ordinalNumber(r,{unit:"hour"}):(0,u.default)(r,t.length)},k:function(e,t,n){var r=e.getUTCHours();return 0===r&&(r=24),"ko"===t?n.ordinalNumber(r,{unit:"hour"}):(0,u.default)(r,t.length)},m:function(e,t,n){return"mo"===t?n.ordinalNumber(e.getUTCMinutes(),{unit:"minute"}):r.default.m(e,t)},s:function(e,t,n){return"so"===t?n.ordinalNumber(e.getUTCSeconds(),{unit:"second"}):r.default.s(e,t)},S:function(e,t){return r.default.S(e,t)},X:function(e,t,n,r){var i=(r._originalDate||e).getTimezoneOffset();if(0===i)return"Z";switch(t){case"X":return y(i);case"XXXX":case"XX":return b(i);case"XXXXX":case"XXX":default:return b(i,":")}},x:function(e,t,n,r){var i=(r._originalDate||e).getTimezoneOffset();switch(t){case"x":return y(i);case"xxxx":case"xx":return b(i);case"xxxxx":case"xxx":default:return b(i,":")}},O:function(e,t,n,r){var i=(r._originalDate||e).getTimezoneOffset();switch(t){case"O":case"OO":case"OOO":return"GMT"+m(i,":");case"OOOO":default:return"GMT"+b(i,":")}},z:function(e,t,n,r){var i=(r._originalDate||e).getTimezoneOffset();switch(t){case"z":case"zz":case"zzz":return"GMT"+m(i,":");case"zzzz":default:return"GMT"+b(i,":")}},t:function(e,t,n,r){var i=r._originalDate||e,o=Math.floor(i.getTime()/1e3);return(0,u.default)(o,t.length)},T:function(e,t,n,r){var i=(r._originalDate||e).getTime();return(0,u.default)(i,t.length)}};t.default=C,e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var r,i=(r=n(1187))&&r.__esModule?r:{default:r};var o={y:function(e,t){var n=e.getUTCFullYear(),r=n>0?n:1-n;return(0,i.default)("yy"===t?r%100:r,t.length)},M:function(e,t){var n=e.getUTCMonth();return"M"===t?String(n+1):(0,i.default)(n+1,2)},d:function(e,t){return(0,i.default)(e.getUTCDate(),t.length)},a:function(e,t){var n=e.getUTCHours()/12>=1?"pm":"am";switch(t){case"a":case"aa":return n.toUpperCase();case"aaa":return n;case"aaaaa":return n[0];case"aaaa":default:return"am"===n?"a.m.":"p.m."}},h:function(e,t){return(0,i.default)(e.getUTCHours()%12||12,t.length)},H:function(e,t){return(0,i.default)(e.getUTCHours(),t.length)},m:function(e,t){return(0,i.default)(e.getUTCMinutes(),t.length)},s:function(e,t){return(0,i.default)(e.getUTCSeconds(),t.length)},S:function(e,t){var n=t.length,r=e.getUTCMilliseconds(),o=Math.floor(r*Math.pow(10,n-3));return(0,i.default)(o,t.length)}};t.default=o,e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){(0,i.default)(1,arguments);var t=(0,r.default)(e),n=t.getTime();t.setUTCMonth(0,1),t.setUTCHours(0,0,0,0);var o=t.getTime(),a=n-o;return Math.floor(a/864e5)+1};var r=o(n(47)),i=o(n(40));function o(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){(0,i.default)(1,arguments);var t=(0,r.default)(e),n=t.getFullYear(),o=t.getMonth(),a=new Date(0);return a.setFullYear(n,o+1,0),a.setHours(0,0,0,0),a.getDate()};var r=o(n(47)),i=o(n(40));function o(e){return e&&e.__esModule?e:{default:e}}e.exports=t.default},function(e,t,n){"use strict";var r=n(46),i=n(36);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(634)),a=i(n(114)),s=i(n(33)),l=i(n(34)),u=i(n(9)),c=i(n(45)),d=i(n(43)),h=i(n(44)),f=i(n(8)),p=r(n(0)),g=i(n(3)),v=i(n(136)),m=i(n(154)),y=i(n(88)),b=i(n(103)),C=i(n(132)),A=i(n(199)),_=i(n(192)),I=i(n(120)),x=i(n(200)),w=i(n(426)),M=i(n(165)),S=(i(n(221)),i(n(185))),T=i(n(1178)),E=n(1192);function O(e){var t=function(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],(function(){}))),!0}catch(e){return!1}}();return function(){var n,r=(0,h.default)(e);if(t){var i=(0,h.default)(this).constructor;n=Reflect.construct(r,arguments,i)}else n=r.apply(this,arguments);return(0,d.default)(this,n)}}var k={PaperProps:{style:{maxHeight:224,width:250}}},L=function(e){(0,c.default)(n,e);var t=O(n);function n(){var e;(0,s.default)(this,n);for(var r=arguments.length,i=new Array(r),l=0;l<r;l++)i[l]=arguments[l];return e=t.call.apply(t,[this].concat(i)),(0,f.default)((0,u.default)(e),"getLocalizationData",(function(){return(0,a.default)({},n.defaultProps.localization,e.props.localization)})),(0,f.default)((0,u.default)(e),"getLocalizedFilterPlaceHolder",(function(t){return t.filterPlaceholder||e.getLocalizationData().filterPlaceHolder||""})),(0,f.default)((0,u.default)(e),"LookupFilter",(function(t){var n=t.columnDef,r=p.useState(n.tableData.filterValue||[]),i=(0,o.default)(r,2),a=i[0],s=i[1];return p.useEffect((function(){s(n.tableData.filterValue||[])}),[n.tableData.filterValue]),p.createElement(b.default,{style:{width:"100%"}},p.createElement(_.default,{htmlFor:"select-multiple-checkbox",style:{marginTop:-16}},e.getLocalizedFilterPlaceHolder(n)),p.createElement(C.default,{multiple:!0,value:a,onClose:function(){e.props.onFilterChanged(n.tableData.id,a)},onChange:function(e){s(e.target.value)},input:p.createElement(A.default,{id:"select-multiple-checkbox"}),renderValue:function(e){return e.map((function(e){return n.lookup[e]})).join(", ")},MenuProps:k,style:{marginTop:0}},Object.keys(n.lookup).map((function(e){return p.createElement(I.default,{key:e,value:e},p.createElement(x.default,{checked:a.indexOf(e.toString())>-1}),p.createElement(w.default,{primary:n.lookup[e]}))}))))})),(0,f.default)((0,u.default)(e),"renderFilterComponent",(function(t){return p.createElement(t.filterComponent,{columnDef:t,onFilterChanged:e.props.onFilterChanged})})),(0,f.default)((0,u.default)(e),"renderBooleanFilter",(function(t){return p.createElement(x.default,{indeterminate:void 0===t.tableData.filterValue,checked:"checked"===t.tableData.filterValue,onChange:function(){var n;void 0===t.tableData.filterValue?n="checked":"checked"===t.tableData.filterValue&&(n="unchecked"),e.props.onFilterChanged(t.tableData.id,n)}})})),(0,f.default)((0,u.default)(e),"renderDefaultFilter",(function(t){var n=e.getLocalizationData(),r=e.props.icons.Filter;return p.createElement(y.default,{style:"numeric"===t.type?{float:"right"}:{},type:"numeric"===t.type?"number":"search",value:t.tableData.filterValue||"",placeholder:e.getLocalizedFilterPlaceHolder(t),onChange:function(n){e.props.onFilterChanged(t.tableData.id,n.target.value)},inputProps:{"aria-label":"filter data by ".concat(t.title)},InputProps:e.props.hideFilterIcons||t.hideFilterIcon?void 0:{startAdornment:p.createElement(M.default,{position:"start"},p.createElement(S.default,{title:n.filterTooltip},p.createElement(r,null)))}})})),(0,f.default)((0,u.default)(e),"renderDateTypeFilter",(function(t){var n={value:t.tableData.filterValue||null,onChange:function(n){return e.props.onFilterChanged(t.tableData.id,n)},placeholder:e.getLocalizedFilterPlaceHolder(t),clearable:!0},r=null;return"date"===t.type?r=p.createElement(E.DatePicker,n):"datetime"===t.type?r=p.createElement(E.DateTimePicker,n):"time"===t.type&&(r=p.createElement(E.TimePicker,n)),p.createElement(E.MuiPickersUtilsProvider,{utils:T.default,locale:e.props.localization.dateTimePickerLocalization},r)})),e}return(0,l.default)(n,[{key:"getComponentForColumn",value:function(e){return!1===e.filtering?null:e.field||e.customFilterAndSearch?e.filterComponent?this.renderFilterComponent(e):e.lookup?p.createElement(this.LookupFilter,{columnDef:e}):"boolean"===e.type?this.renderBooleanFilter(e):["date","datetime","time"].includes(e.type)?this.renderDateTypeFilter(e):this.renderDefaultFilter(e):void 0}},{key:"render",value:function(){var e=this,t=this.props.columns.filter((function(e){return!(e.hidden||e.tableData.groupOrder>-1)})).sort((function(e,t){return e.tableData.columnOrder-t.tableData.columnOrder})).map((function(t){return p.createElement(v.default,{key:t.tableData.id,style:(0,a.default)({},e.props.filterCellStyle,t.filterCellStyle)},e.getComponentForColumn(t))}));if(this.props.selection&&t.splice(0,0,p.createElement(v.default,{padding:"none",key:"key-selection-column"})),this.props.hasActions)if(-1===this.props.actionsColumnIndex)t.push(p.createElement(v.default,{key:"key-action-column"}));else{var n=0;this.props.selection&&(n=1),t.splice(this.props.actionsColumnIndex+n,0,p.createElement(v.default,{key:"key-action-column"}))}return this.props.hasDetailPanel&&t.splice(0,0,p.createElement(v.default,{padding:"none",key:"key-detail-panel-column"})),this.props.isTreeData>0&&t.splice(0,0,p.createElement(v.default,{padding:"none",key:"key-tree-data-filter"})),this.props.columns.filter((function(e){return e.tableData.groupOrder>-1})).forEach((function(e){t.splice(0,0,p.createElement(v.default,{padding:"checkbox",key:"key-group-filter"+e.tableData.id}))})),p.createElement(m.default,{style:(0,a.default)({height:10},this.props.filterRowStyle)},t)}}]),n}(p.Component);L.defaultProps={columns:[],selection:!1,hasActions:!1,localization:{filterTooltip:"Filter"},hideFilterIcons:!1},L.propTypes={columns:g.default.array.isRequired,hasDetailPanel:g.default.bool.isRequired,isTreeData:g.default.bool.isRequired,onFilterChanged:g.default.func.isRequired,filterCellStyle:g.default.object,filterRowStyle:g.default.object,selection:g.default.bool.isRequired,actionsColumnIndex:g.default.number,hasActions:g.default.bool,localization:g.default.object,hideFilterIcons:g.default.bool};var D=L;t.default=D},function(e,t,n){"use strict";var r=n(46),i=n(36);Object.defineProperty(t,"__esModule",{value:!0}),t.default=t.styles=t.MTableHeader=void 0;var o=i(n(114)),a=i(n(50)),s=i(n(33)),l=i(n(34)),u=i(n(45)),c=i(n(43)),d=i(n(44)),h=r(n(0)),f=i(n(3)),p=i(n(312)),g=i(n(154)),v=i(n(136)),m=i(n(427)),y=i(n(200)),b=i(n(314)),C=n(621),A=n(90),_=r(n(423));function I(e){var t=function(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],(function(){}))),!0}catch(e){return!1}}();return function(){var n,r=(0,d.default)(e);if(t){var i=(0,d.default)(this).constructor;n=Reflect.construct(r,arguments,i)}else n=r.apply(this,arguments);return(0,c.default)(this,n)}}var x=function(e){(0,u.default)(n,e);var t=I(n);function n(){return(0,s.default)(this,n),t.apply(this,arguments)}return(0,l.default)(n,[{key:"renderHeader",value:function(){var e=this,t="default"===this.props.options.padding?"medium":"small";return this.props.columns.filter((function(e){return!(e.hidden||e.tableData.groupOrder>-1)})).sort((function(e,t){return e.tableData.columnOrder-t.tableData.columnOrder})).map((function(n,r){var i=n.title;e.props.draggable&&(i=h.createElement(C.Draggable,{key:n.tableData.id,draggableId:n.tableData.id.toString(),index:r},(function(e,t){return h.createElement("div",(0,a.default)({ref:e.innerRef},e.draggableProps,e.dragHandleProps),n.title)}))),!1!==n.sorting&&e.props.sorting&&(i=h.createElement(m.default,{IconComponent:e.props.icons.SortArrow,active:e.props.orderBy===n.tableData.id,direction:e.props.orderDirection||"asc",onClick:function(){var t=n.tableData.id!==e.props.orderBy?"asc":"asc"===e.props.orderDirection?"desc":"desc"===e.props.orderDirection&&e.props.thirdSortClick?"":"desc"!==e.props.orderDirection||e.props.thirdSortClick?""===e.props.orderDirection?"asc":"desc":"asc";e.props.onOrderChange(n.tableData.id,t)}},i)),n.tooltip&&(i=h.createElement(A.Tooltip,{title:n.tooltip},h.createElement("span",null,i)));var s=void 0!==n.align?n.align:-1!==["numeric","currency"].indexOf(n.type)?"right":"left";return h.createElement(v.default,{key:n.tableData.id,align:s,className:e.props.classes.header,style:(0,o.default)({},e.props.headerStyle,n.headerStyle,{boxSizing:"border-box",width:n.tableData.width}),size:t},i)}))}},{key:"renderActionsHeader",value:function(){var e=(0,o.default)({},n.defaultProps.localization,this.props.localization),t=_.actionsColumnWidth(this.props);return h.createElement(v.default,{key:"key-actions-column",padding:"checkbox",className:this.props.classes.header,style:(0,o.default)({},this.props.headerStyle,{width:t,textAlign:"center",boxSizing:"border-box"})},h.createElement(m.default,{hideSortIcon:!0,disabled:!0},e.actions))}},{key:"renderSelectionHeader",value:function(){var e=this,t=_.selectionMaxWidth(this.props,this.props.treeDataMaxLevel);return h.createElement(v.default,{padding:"none",key:"key-selection-column",className:this.props.classes.header,style:(0,o.default)({},this.props.headerStyle,{width:t})},this.props.showSelectAllCheckbox&&h.createElement(y.default,{indeterminate:this.props.selectedCount>0&&this.props.selectedCount<this.props.dataCount,checked:this.props.dataCount>0&&this.props.selectedCount===this.props.dataCount,onChange:function(t,n){return e.props.onAllSelected&&e.props.onAllSelected(n)}}))}},{key:"renderDetailPanelColumnCell",value:function(){return h.createElement(v.default,{padding:"none",key:"key-detail-panel-column",className:this.props.classes.header,style:(0,o.default)({},this.props.headerStyle)})}},{key:"render",value:function(){var e=this,t=this.renderHeader();if(this.props.hasSelection&&t.splice(0,0,this.renderSelectionHeader()),this.props.showActionsColumn)if(this.props.actionsHeaderIndex>=0){var n=0;this.props.hasSelection&&(n=1),t.splice(this.props.actionsHeaderIndex+n,0,this.renderActionsHeader())}else-1===this.props.actionsHeaderIndex&&t.push(this.renderActionsHeader());return this.props.hasDetailPanel&&("right"===this.props.detailPanelColumnAlignment?t.push(this.renderDetailPanelColumnCell()):t.splice(0,0,this.renderDetailPanelColumnCell())),this.props.isTreeData>0&&t.splice(0,0,h.createElement(v.default,{padding:"none",key:"key-tree-data-header",className:this.props.classes.header,style:(0,o.default)({},this.props.headerStyle)})),this.props.columns.filter((function(e){return e.tableData.groupOrder>-1})).forEach((function(n){t.splice(0,0,h.createElement(v.default,{padding:"checkbox",key:"key-group-header"+n.tableData.id,className:e.props.classes.header}))})),h.createElement(p.default,null,h.createElement(g.default,null,t))}}]),n}(h.Component);t.MTableHeader=x,x.defaultProps={dataCount:0,hasSelection:!1,headerStyle:{},selectedCount:0,sorting:!0,localization:{actions:"Actions"},orderBy:void 0,orderDirection:"asc",actionsHeaderIndex:0,detailPanelColumnAlignment:"left",draggable:!0,thirdSortClick:!0},x.propTypes={columns:f.default.array.isRequired,dataCount:f.default.number,hasDetailPanel:f.default.bool.isRequired,detailPanelColumnAlignment:f.default.string,hasSelection:f.default.bool,headerStyle:f.default.object,localization:f.default.object,selectedCount:f.default.number,sorting:f.default.bool,onAllSelected:f.default.func,onOrderChange:f.default.func,orderBy:f.default.number,orderDirection:f.default.string,actionsHeaderIndex:f.default.number,showActionsColumn:f.default.bool,showSelectAllCheckbox:f.default.bool,draggable:f.default.bool,thirdSortClick:f.default.bool,tooltip:f.default.string};var w=function(e){return{header:{position:"sticky",top:0,zIndex:10,backgroundColor:e.palette.background.paper}}};t.styles=w;var M=(0,b.default)(w)(x);t.default=M},function(e,t,n){"use strict";var r=n(36);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var i=(0,r(n(2276)).default)();t.default=i},function(e,t,n){"use strict";var r=n(36);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;r(n(8));var i=r(n(98)),o=n(31),a=r(n(2277)),s=r(n(2278)),l=r(n(2279)),u=r(n(2288)),c=r(n(2289)),d=r(n(2290)),h=r(n(2291)),f=r(n(2292)),p=r(n(2293));var g=function(){for(var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},t=e.breakpoints,n=void 0===t?{}:t,r=e.mixins,g=void 0===r?{}:r,v=e.palette,m=void 0===v?{}:v,y=e.spacing,b=e.typography,C=void 0===b?{}:b,A=(0,i.default)(e,["breakpoints","mixins","palette","spacing","typography"]),_=(0,l.default)(m),I=(0,a.default)(n),x=(0,h.default)(y),w=(0,o.deepmerge)({breakpoints:I,direction:"ltr",mixins:(0,s.default)(I,x,g),overrides:{},palette:_,props:{},shadows:c.default,typography:(0,u.default)(_,C),spacing:x,shape:d.default,transitions:f.default,zIndex:p.default},A),M=arguments.length,S=new Array(M>1?M-1:0),T=1;T<M;T++)S[T-1]=arguments[T];return w=S.reduce((function(e,t){return(0,o.deepmerge)(e,t)}),w)};t.default=g},function(e,t,n){"use strict";var r=n(36);Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){var t=e.values,n=void 0===t?{xs:0,sm:600,md:960,lg:1280,xl:1920}:t,r=e.unit,s=void 0===r?"px":r,l=e.step,u=void 0===l?5:l,c=(0,o.default)(e,["values","unit","step"]);function d(e){var t="number"==typeof n[e]?n[e]:e;return"@media (min-width:".concat(t).concat(s,")")}function h(e,t){var r=a.indexOf(t);return r===a.length-1?d(e):"@media (min-width:".concat("number"==typeof n[e]?n[e]:e).concat(s,") and ")+"(max-width:".concat((-1!==r&&"number"==typeof n[a[r+1]]?n[a[r+1]]:t)-u/100).concat(s,")")}return(0,i.default)({keys:a,values:n,up:d,down:function(e){var t=a.indexOf(e)+1,r=n[a[t]];return t===a.length?d("xs"):"@media (max-width:".concat(("number"==typeof r&&t>0?r:e)-u/100).concat(s,")")},between:h,only:function(e){return h(e,e)},width:function(e){return n[e]}},c)},t.keys=void 0;var i=r(n(50)),o=r(n(98)),a=["xs","sm","md","lg","xl"];t.keys=a},function(e,t,n){"use strict";var r=n(36);Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t,n){var r;return(0,o.default)({gutters:function(){var n=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};return(0,o.default)({paddingLeft:t(2),paddingRight:t(2)},n,(0,i.default)({},e.up("sm"),(0,o.default)({paddingLeft:t(3),paddingRight:t(3)},n[e.up("sm")])))},toolbar:(r={minHeight:56},(0,i.default)(r,"".concat(e.up("xs")," and (orientation: landscape)"),{minHeight:48}),(0,i.default)(r,e.up("sm"),{minHeight:64}),r)},n)};var i=r(n(8)),o=r(n(50))},function(e,t,n){"use strict";var r=n(36);Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){var t=e.primary,n=void 0===t?{light:u.default[300],main:u.default[500],dark:u.default[700]}:t,r=e.secondary,b=void 0===r?{light:c.default.A200,main:c.default.A400,dark:c.default.A700}:r,C=e.error,A=void 0===C?{light:d.default[300],main:d.default[500],dark:d.default[700]}:C,_=e.warning,I=void 0===_?{light:h.default[300],main:h.default[500],dark:h.default[700]}:_,x=e.info,w=void 0===x?{light:f.default[300],main:f.default[500],dark:f.default[700]}:x,M=e.success,S=void 0===M?{light:p.default[300],main:p.default[500],dark:p.default[700]}:M,T=e.type,E=void 0===T?"light":T,O=e.contrastThreshold,k=void 0===O?3:O,L=e.tonalOffset,D=void 0===L?.2:L,P=(0,o.default)(e,["primary","secondary","error","warning","info","success","type","contrastThreshold","tonalOffset"]);function N(e){return(0,g.getContrastRatio)(e,m.text.primary)>=k?m.text.primary:v.text.primary}var j=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:500,n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:300,r=arguments.length>3&&void 0!==arguments[3]?arguments[3]:700;if(!(e=(0,i.default)({},e)).main&&e[t]&&(e.main=e[t]),!e.main)throw new Error((0,a.formatMuiErrorMessage)(4,t));if("string"!=typeof e.main)throw new Error(_formatMuiErrorMessage(5,JSON.stringify(e.main)));return y(e,"light",n,D),y(e,"dark",r,D),e.contrastText||(e.contrastText=N(e.main)),e},R={dark:m,light:v};0;return(0,a.deepmerge)((0,i.default)({common:s.default,type:E,primary:j(n),secondary:j(b,"A400","A200","A700"),error:j(A),warning:j(I),info:j(w),success:j(S),grey:l.default,contrastThreshold:k,getContrastText:N,augmentColor:j,tonalOffset:D},R[E]),P)},t.dark=t.light=void 0;var i=r(n(50)),o=r(n(98)),a=n(31),s=r(n(2280)),l=r(n(2281)),u=r(n(2282)),c=r(n(2283)),d=r(n(2284)),h=r(n(2285)),f=r(n(2286)),p=r(n(2287)),g=n(618),v={text:{primary:"rgba(0, 0, 0, 0.87)",secondary:"rgba(0, 0, 0, 0.54)",disabled:"rgba(0, 0, 0, 0.38)",hint:"rgba(0, 0, 0, 0.38)"},divider:"rgba(0, 0, 0, 0.12)",background:{paper:s.default.white,default:l.default[50]},action:{active:"rgba(0, 0, 0, 0.54)",hover:"rgba(0, 0, 0, 0.04)",hoverOpacity:.04,selected:"rgba(0, 0, 0, 0.08)",selectedOpacity:.08,disabled:"rgba(0, 0, 0, 0.26)",disabledBackground:"rgba(0, 0, 0, 0.12)",disabledOpacity:.38,focus:"rgba(0, 0, 0, 0.12)",focusOpacity:.12,activatedOpacity:.12}};t.light=v;var m={text:{primary:s.default.white,secondary:"rgba(255, 255, 255, 0.7)",disabled:"rgba(255, 255, 255, 0.5)",hint:"rgba(255, 255, 255, 0.5)",icon:"rgba(255, 255, 255, 0.5)"},divider:"rgba(255, 255, 255, 0.12)",background:{paper:l.default[800],default:"#303030"},action:{active:s.default.white,hover:"rgba(255, 255, 255, 0.08)",hoverOpacity:.08,selected:"rgba(255, 255, 255, 0.16)",selectedOpacity:.16,disabled:"rgba(255, 255, 255, 0.3)",disabledBackground:"rgba(255, 255, 255, 0.12)",disabledOpacity:.38,focus:"rgba(255, 255, 255, 0.12)",focusOpacity:.12,activatedOpacity:.24}};function y(e,t,n,r){var i=r.light||r,o=r.dark||1.5*r;e[t]||(e.hasOwnProperty(n)?e[t]=e[n]:"light"===t?e.light=(0,g.lighten)(e.main,i):"dark"===t&&(e.dark=(0,g.darken)(e.main,o)))}t.dark=m},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var r={black:"#000",white:"#fff"};t.default=r},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var r={50:"#fafafa",100:"#f5f5f5",200:"#eeeeee",300:"#e0e0e0",400:"#bdbdbd",500:"#9e9e9e",600:"#757575",700:"#616161",800:"#424242",900:"#212121",A100:"#d5d5d5",A200:"#aaaaaa",A400:"#303030",A700:"#616161"};t.default=r},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var r={50:"#e8eaf6",100:"#c5cae9",200:"#9fa8da",300:"#7986cb",400:"#5c6bc0",500:"#3f51b5",600:"#3949ab",700:"#303f9f",800:"#283593",900:"#1a237e",A100:"#8c9eff",A200:"#536dfe",A400:"#3d5afe",A700:"#304ffe"};t.default=r},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var r={50:"#fce4ec",100:"#f8bbd0",200:"#f48fb1",300:"#f06292",400:"#ec407a",500:"#e91e63",600:"#d81b60",700:"#c2185b",800:"#ad1457",900:"#880e4f",A100:"#ff80ab",A200:"#ff4081",A400:"#f50057",A700:"#c51162"};t.default=r},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var r={50:"#ffebee",100:"#ffcdd2",200:"#ef9a9a",300:"#e57373",400:"#ef5350",500:"#f44336",600:"#e53935",700:"#d32f2f",800:"#c62828",900:"#b71c1c",A100:"#ff8a80",A200:"#ff5252",A400:"#ff1744",A700:"#d50000"};t.default=r},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var r={50:"#fff3e0",100:"#ffe0b2",200:"#ffcc80",300:"#ffb74d",400:"#ffa726",500:"#ff9800",600:"#fb8c00",700:"#f57c00",800:"#ef6c00",900:"#e65100",A100:"#ffd180",A200:"#ffab40",A400:"#ff9100",A700:"#ff6d00"};t.default=r},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var r={50:"#e3f2fd",100:"#bbdefb",200:"#90caf9",300:"#64b5f6",400:"#42a5f5",500:"#2196f3",600:"#1e88e5",700:"#1976d2",800:"#1565c0",900:"#0d47a1",A100:"#82b1ff",A200:"#448aff",A400:"#2979ff",A700:"#2962ff"};t.default=r},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var r={50:"#e8f5e9",100:"#c8e6c9",200:"#a5d6a7",300:"#81c784",400:"#66bb6a",500:"#4caf50",600:"#43a047",700:"#388e3c",800:"#2e7d32",900:"#1b5e20",A100:"#b9f6ca",A200:"#69f0ae",A400:"#00e676",A700:"#00c853"};t.default=r},function(e,t,n){"use strict";var r=n(36);Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){var n="function"==typeof t?t(e):t,r=n.fontFamily,u=void 0===r?'"Roboto", "Helvetica", "Arial", sans-serif':r,c=n.fontSize,d=void 0===c?14:c,h=n.fontWeightLight,f=void 0===h?300:h,p=n.fontWeightRegular,g=void 0===p?400:p,v=n.fontWeightMedium,m=void 0===v?500:v,y=n.fontWeightBold,b=void 0===y?700:y,C=n.htmlFontSize,A=void 0===C?16:C,_=n.allVariants,I=n.pxToRem,x=(0,o.default)(n,["fontFamily","fontSize","fontWeightLight","fontWeightRegular","fontWeightMedium","fontWeightBold","htmlFontSize","allVariants","pxToRem"]);0;var w=d/14,M=I||function(e){return"".concat(e/A*w,"rem")},S=function(e,t,n,r,o){return(0,i.default)({fontFamily:u,fontWeight:e,fontSize:M(t),lineHeight:n},'"Roboto", "Helvetica", "Arial", sans-serif'===u?{letterSpacing:"".concat(s(r/t),"em")}:{},o,_)},T={h1:S(f,96,1.167,-1.5),h2:S(f,60,1.2,-.5),h3:S(g,48,1.167,0),h4:S(g,34,1.235,.25),h5:S(g,24,1.334,0),h6:S(m,20,1.6,.15),subtitle1:S(g,16,1.75,.15),subtitle2:S(m,14,1.57,.1),body1:S(g,16,1.5,.15),body2:S(g,14,1.43,.15),button:S(m,14,1.75,.4,l),caption:S(g,12,1.66,.4),overline:S(g,12,2.66,1,l)};return(0,a.deepmerge)((0,i.default)({htmlFontSize:A,pxToRem:M,round:s,fontFamily:u,fontSize:d,fontWeightLight:f,fontWeightRegular:g,fontWeightMedium:m,fontWeightBold:b},T),x,{clone:!1})};var i=r(n(50)),o=r(n(98)),a=n(31);function s(e){return Math.round(1e5*e)/1e5}var l={textTransform:"uppercase"}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;function r(){return["".concat(arguments.length<=0?void 0:arguments[0],"px ").concat(arguments.length<=1?void 0:arguments[1],"px ").concat(arguments.length<=2?void 0:arguments[2],"px ").concat(arguments.length<=3?void 0:arguments[3],"px rgba(0,0,0,").concat(.2,")"),"".concat(arguments.length<=4?void 0:arguments[4],"px ").concat(arguments.length<=5?void 0:arguments[5],"px ").concat(arguments.length<=6?void 0:arguments[6],"px ").concat(arguments.length<=7?void 0:arguments[7],"px rgba(0,0,0,").concat(.14,")"),"".concat(arguments.length<=8?void 0:arguments[8],"px ").concat(arguments.length<=9?void 0:arguments[9],"px ").concat(arguments.length<=10?void 0:arguments[10],"px ").concat(arguments.length<=11?void 0:arguments[11],"px rgba(0,0,0,").concat(.12,")")].join(",")}var i=["none",r(0,2,1,-1,0,1,1,0,0,1,3,0),r(0,3,1,-2,0,2,2,0,0,1,5,0),r(0,3,3,-2,0,3,4,0,0,1,8,0),r(0,2,4,-1,0,4,5,0,0,1,10,0),r(0,3,5,-1,0,5,8,0,0,1,14,0),r(0,3,5,-1,0,6,10,0,0,1,18,0),r(0,4,5,-2,0,7,10,1,0,2,16,1),r(0,5,5,-3,0,8,10,1,0,3,14,2),r(0,5,6,-3,0,9,12,1,0,3,16,2),r(0,6,6,-3,0,10,14,1,0,4,18,3),r(0,6,7,-4,0,11,15,1,0,4,20,3),r(0,7,8,-4,0,12,17,2,0,5,22,4),r(0,7,8,-4,0,13,19,2,0,5,24,4),r(0,7,9,-4,0,14,21,2,0,5,26,4),r(0,8,9,-5,0,15,22,2,0,6,28,5),r(0,8,10,-5,0,16,24,2,0,6,30,5),r(0,8,11,-5,0,17,26,2,0,6,32,5),r(0,9,11,-5,0,18,28,2,0,7,34,6),r(0,9,12,-6,0,19,29,2,0,7,36,6),r(0,10,13,-6,0,20,31,3,0,8,38,7),r(0,10,13,-6,0,21,33,3,0,8,40,7),r(0,10,14,-6,0,22,35,3,0,8,42,7),r(0,11,14,-7,0,23,36,3,0,9,44,8),r(0,11,15,-7,0,24,38,3,0,9,46,8)];t.default=i},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var r={borderRadius:4};t.default=r},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:8;if(e.mui)return e;var t=(0,r.createUnarySpacing)({spacing:e}),n=function(){for(var e=arguments.length,n=new Array(e),r=0;r<e;r++)n[r]=arguments[r];return 0===n.length?t(1):1===n.length?t(n[0]):n.map((function(e){if("string"==typeof e)return e;var n=t(e);return"number"==typeof n?"".concat(n,"px"):n})).join(" ")};return Object.defineProperty(n,"unit",{get:function(){return e}}),n.mui=!0,n};var r=n(619)},function(e,t,n){"use strict";var r=n(36);Object.defineProperty(t,"__esModule",{value:!0}),t.default=t.duration=t.easing=void 0;var i=r(n(98)),o={easeInOut:"cubic-bezier(0.4, 0, 0.2, 1)",easeOut:"cubic-bezier(0.0, 0, 0.2, 1)",easeIn:"cubic-bezier(0.4, 0, 1, 1)",sharp:"cubic-bezier(0.4, 0, 0.6, 1)"};t.easing=o;var a={shortest:150,shorter:200,short:250,standard:300,complex:375,enteringScreen:225,leavingScreen:195};function s(e){return"".concat(Math.round(e),"ms")}t.duration=a;var l={easing:o,duration:a,create:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:["all"],t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=t.duration,r=void 0===n?a.standard:n,l=t.easing,u=void 0===l?o.easeInOut:l,c=t.delay,d=void 0===c?0:c;(0,i.default)(t,["duration","easing","delay"]);return(Array.isArray(e)?e:[e]).map((function(e){return"".concat(e," ").concat("string"==typeof r?r:s(r)," ").concat(u," ").concat("string"==typeof d?d:s(d))})).join(",")},getAutoHeightDuration:function(e){if(!e)return 0;var t=e/36;return Math.round(10*(4+15*Math.pow(t,.25)+t/5))}};t.default=l},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var r={mobileStepper:1e3,speedDial:1050,appBar:1100,drawer:1200,modal:1300,snackbar:1400,tooltip:1500};t.default=r},function(e,t,n){"use strict";var r=n(46),i=n(36);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(114)),a=i(n(33)),s=i(n(34)),l=i(n(9)),u=i(n(45)),c=i(n(43)),d=i(n(44)),h=i(n(8)),f=i(n(73)),p=i(n(314)),g=i(n(185)),v=i(n(62)),m=i(n(3)),y=r(n(0));function b(e){var t=function(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],(function(){}))),!0}catch(e){return!1}}();return function(){var n,r=(0,d.default)(e);if(t){var i=(0,d.default)(this).constructor;n=Reflect.construct(r,arguments,i)}else n=r.apply(this,arguments);return(0,c.default)(this,n)}}var C=function(e){(0,u.default)(n,e);var t=b(n);function n(){var e;(0,a.default)(this,n);for(var r=arguments.length,i=new Array(r),o=0;o<r;o++)i[o]=arguments[o];return e=t.call.apply(t,[this].concat(i)),(0,h.default)((0,l.default)(e),"handleFirstPageButtonClick",(function(t){e.props.onChangePage(t,0)})),(0,h.default)((0,l.default)(e),"handleBackButtonClick",(function(t){e.props.onChangePage(t,e.props.page-1)})),(0,h.default)((0,l.default)(e),"handleNextButtonClick",(function(t){e.props.onChangePage(t,e.props.page+1)})),(0,h.default)((0,l.default)(e),"handleLastPageButtonClick",(function(t){e.props.onChangePage(t,Math.max(0,Math.ceil(e.props.count/e.props.rowsPerPage)-1))})),e}return(0,s.default)(n,[{key:"render",value:function(){var e=this.props,t=e.classes,r=e.count,i=e.page,a=e.rowsPerPage,s=e.theme,l=e.showFirstLastPageButtons,u=(0,o.default)({},n.defaultProps.localization,this.props.localization);return y.createElement("div",{className:t.root},l&&y.createElement(g.default,{title:u.firstTooltip},y.createElement("span",null,y.createElement(f.default,{onClick:this.handleFirstPageButtonClick,disabled:0===i,"aria-label":u.firstAriaLabel},"rtl"===s.direction?y.createElement(this.props.icons.LastPage,null):y.createElement(this.props.icons.FirstPage,null)))),y.createElement(g.default,{title:u.previousTooltip},y.createElement("span",null,y.createElement(f.default,{onClick:this.handleBackButtonClick,disabled:0===i,"aria-label":u.previousAriaLabel},"rtl"===s.direction?y.createElement(this.props.icons.NextPage,null):y.createElement(this.props.icons.PreviousPage,null)))),y.createElement(v.default,{variant:"caption",style:{flex:1,textAlign:"center",alignSelf:"center",flexBasis:"inherit"}},u.labelDisplayedRows.replace("{from}",0===this.props.count?0:this.props.page*this.props.rowsPerPage+1).replace("{to}",Math.min((this.props.page+1)*this.props.rowsPerPage,this.props.count)).replace("{count}",this.props.count)),y.createElement(g.default,{title:u.nextTooltip},y.createElement("span",null,y.createElement(f.default,{onClick:this.handleNextButtonClick,disabled:i>=Math.ceil(r/a)-1,"aria-label":u.nextAriaLabel},"rtl"===s.direction?y.createElement(this.props.icons.PreviousPage,null):y.createElement(this.props.icons.NextPage,null)))),l&&y.createElement(g.default,{title:u.lastTooltip},y.createElement("span",null,y.createElement(f.default,{onClick:this.handleLastPageButtonClick,disabled:i>=Math.ceil(r/a)-1,"aria-label":u.lastAriaLabel},"rtl"===s.direction?y.createElement(this.props.icons.FirstPage,null):y.createElement(this.props.icons.LastPage,null)))))}}]),n}(y.Component);C.propTypes={onChangePage:m.default.func,page:m.default.number,count:m.default.number,rowsPerPage:m.default.number,classes:m.default.object,localization:m.default.object,theme:m.default.any,showFirstLastPageButtons:m.default.bool},C.defaultProps={showFirstLastPageButtons:!0,localization:{firstTooltip:"First Page",previousTooltip:"Previous Page",nextTooltip:"Next Page",lastTooltip:"Last Page",labelDisplayedRows:"{from}-{to} of {count}",labelRowsPerPage:"Rows per page:"}};var A=(0,p.default)((function(e){return{root:{flexShrink:0,color:e.palette.text.secondary,display:"flex"}}}),{withTheme:!0})(C);t.default=A},function(e,t,n){"use strict";var r=n(46),i=n(36);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(114)),a=i(n(33)),s=i(n(34)),l=i(n(9)),u=i(n(45)),c=i(n(43)),d=i(n(44)),h=i(n(8)),f=i(n(73)),p=i(n(314)),g=i(n(185)),v=i(n(428)),m=i(n(75)),y=i(n(3)),b=r(n(0));function C(e){var t=function(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],(function(){}))),!0}catch(e){return!1}}();return function(){var n,r=(0,d.default)(e);if(t){var i=(0,d.default)(this).constructor;n=Reflect.construct(r,arguments,i)}else n=r.apply(this,arguments);return(0,c.default)(this,n)}}var A=function(e){(0,u.default)(n,e);var t=C(n);function n(){var e;(0,a.default)(this,n);for(var r=arguments.length,i=new Array(r),o=0;o<r;o++)i[o]=arguments[o];return e=t.call.apply(t,[this].concat(i)),(0,h.default)((0,l.default)(e),"handleFirstPageButtonClick",(function(t){e.props.onChangePage(t,0)})),(0,h.default)((0,l.default)(e),"handleBackButtonClick",(function(t){e.props.onChangePage(t,e.props.page-1)})),(0,h.default)((0,l.default)(e),"handleNextButtonClick",(function(t){e.props.onChangePage(t,e.props.page+1)})),(0,h.default)((0,l.default)(e),"handleNumberButtonClick",(function(t){return function(n){e.props.onChangePage(n,t)}})),(0,h.default)((0,l.default)(e),"handleLastPageButtonClick",(function(t){e.props.onChangePage(t,Math.max(0,Math.ceil(e.props.count/e.props.rowsPerPage)-1))})),e}return(0,s.default)(n,[{key:"renderPagesButton",value:function(e,t){for(var n=[],r=e;r<=t;r++){var i=r===this.props.page?"contained":"text";n.push(b.createElement(m.default,{size:"small",style:{boxShadow:"none",maxWidth:"30px",maxHeight:"30px",minWidth:"30px",minHeight:"30px"},disabled:r===this.props.page,variant:i,onClick:this.handleNumberButtonClick(r),key:r},r+1))}return b.createElement("span",null,n)}},{key:"render",value:function(){var e=this.props,t=e.classes,r=e.count,i=e.page,a=e.rowsPerPage,s=e.theme,l=e.showFirstLastPageButtons,u=(0,o.default)({},n.defaultProps.localization,this.props.localization),c=Math.ceil(r/a)-1,d=Math.max(i-1,0),h=Math.min(c,i+1);return b.createElement("div",{className:t.root},l&&b.createElement(g.default,{title:u.firstTooltip},b.createElement("span",null,b.createElement(f.default,{onClick:this.handleFirstPageButtonClick,disabled:0===i,"aria-label":u.firstAriaLabel},"rtl"===s.direction?b.createElement(this.props.icons.LastPage,null):b.createElement(this.props.icons.FirstPage,null)))),b.createElement(g.default,{title:u.previousTooltip},b.createElement("span",null,b.createElement(f.default,{onClick:this.handleBackButtonClick,disabled:0===i,"aria-label":u.previousAriaLabel},b.createElement(this.props.icons.PreviousPage,null)))),b.createElement(v.default,{smDown:!0},this.renderPagesButton(d,h)),b.createElement(g.default,{title:u.nextTooltip},b.createElement("span",null,b.createElement(f.default,{onClick:this.handleNextButtonClick,disabled:i>=c,"aria-label":u.nextAriaLabel},b.createElement(this.props.icons.NextPage,null)))),l&&b.createElement(g.default,{title:u.lastTooltip},b.createElement("span",null,b.createElement(f.default,{onClick:this.handleLastPageButtonClick,disabled:i>=Math.ceil(r/a)-1,"aria-label":u.lastAriaLabel},"rtl"===s.direction?b.createElement(this.props.icons.FirstPage,null):b.createElement(this.props.icons.LastPage,null)))))}}]),n}(b.Component);A.propTypes={onChangePage:y.default.func,page:y.default.number,count:y.default.number,rowsPerPage:y.default.number,classes:y.default.object,localization:y.default.object,theme:y.default.any,showFirstLastPageButtons:y.default.bool},A.defaultProps={showFirstLastPageButtons:!0,localization:{firstTooltip:"First Page",previousTooltip:"Previous Page",nextTooltip:"Next Page",lastTooltip:"Last Page",labelDisplayedRows:"{from}-{to} of {count}",labelRowsPerPage:"Rows per page:"}};var _=(0,p.default)((function(e){return{root:{flexShrink:0,color:e.palette.text.secondary,marginLeft:e.spacing(2.5)}}}),{withTheme:!0})(A);t.default=_},function(e,t,n){"use strict";var r=n(46),i=n(36);Object.defineProperty(t,"__esModule",{value:!0}),t.default=t.styles=t.MTableToolbar=void 0;var o=i(n(114)),a=i(n(33)),s=i(n(34)),l=i(n(9)),u=i(n(45)),c=i(n(43)),d=i(n(44)),h=i(n(8)),f=i(n(200)),p=(i(n(429)),i(n(73))),g=i(n(165)),v=i(n(288)),m=i(n(120)),y=i(n(88)),b=i(n(121)),C=i(n(185)),A=i(n(62)),_=i(n(314)),I=n(618),x=i(n(11)),w=n(2297),M=r(n(3)),S=r(n(0));function T(e){var t=function(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],(function(){}))),!0}catch(e){return!1}}();return function(){var n,r=(0,d.default)(e);if(t){var i=(0,d.default)(this).constructor;n=Reflect.construct(r,arguments,i)}else n=r.apply(this,arguments);return(0,c.default)(this,n)}}var E=function(e){(0,u.default)(n,e);var t=T(n);function n(e){var r;return(0,a.default)(this,n),r=t.call(this,e),(0,h.default)((0,l.default)(r),"onSearchChange",(function(e){r.props.dataManager.changeSearchText(e),r.setState({searchText:e},r.props.onSearchChanged(e))})),(0,h.default)((0,l.default)(r),"defaultExportCsv",(function(){var e=r.props.columns.filter((function(e){return!e.hidden&&e.field&&!1!==e.export})).sort((function(e,t){return e.tableData.columnOrder>t.tableData.columnOrder?1:-1})),t=(r.props.exportAllData?r.props.data:r.props.renderData).map((function(t){return e.map((function(e){return r.props.getFieldValue(t,e)}))}));new w.CsvBuilder((r.props.exportFileName||r.props.title||"data")+".csv").setDelimeter(r.props.exportDelimiter).setColumns(e.map((function(e){return e.title}))).addRows(t).exportFile()})),(0,h.default)((0,l.default)(r),"exportCsv",(function(){r.props.exportCsv?r.props.exportCsv(r.props.columns,r.props.data):r.defaultExportCsv(),r.setState({exportButtonAnchorEl:null})})),r.state={columnsButtonAnchorEl:null,exportButtonAnchorEl:null,searchText:e.searchText},r}return(0,s.default)(n,[{key:"renderSearch",value:function(){var e=this,t=(0,o.default)({},n.defaultProps.localization,this.props.localization);return this.props.search?S.createElement(y.default,{autoFocus:this.props.searchAutoFocus,className:"left"===this.props.searchFieldAlignment&&!1===this.props.showTitle?null:this.props.classes.searchField,value:this.state.searchText,onChange:function(t){return e.onSearchChange(t.target.value)},placeholder:t.searchPlaceholder,variant:this.props.searchFieldVariant,InputProps:{startAdornment:S.createElement(g.default,{position:"start"},S.createElement(C.default,{title:t.searchTooltip},S.createElement(this.props.icons.Search,{fontSize:"small"}))),endAdornment:S.createElement(g.default,{position:"end"},S.createElement(p.default,{disabled:!this.state.searchText,onClick:function(){return e.onSearchChange("")}},S.createElement(this.props.icons.ResetSearch,{fontSize:"small"}))),style:this.props.searchFieldStyle,inputProps:{"aria-label":"Search"}}}):null}},{key:"renderDefaultActions",value:function(){var e=this,t=(0,o.default)({},n.defaultProps.localization,this.props.localization),r=this.props.classes;return S.createElement("div",null,this.props.columnsButton&&S.createElement("span",null,S.createElement(C.default,{title:t.showColumnsTitle},S.createElement(p.default,{color:"inherit",onClick:function(t){return e.setState({columnsButtonAnchorEl:t.currentTarget})},"aria-label":t.showColumnsAriaLabel},S.createElement(this.props.icons.ViewColumn,null))),S.createElement(v.default,{anchorEl:this.state.columnsButtonAnchorEl,open:Boolean(this.state.columnsButtonAnchorEl),onClose:function(){return e.setState({columnsButtonAnchorEl:null})}},S.createElement(m.default,{key:"text",disabled:!0,style:{opacity:1,fontWeight:600,fontSize:12}},t.addRemoveColumns),this.props.columns.map((function(t){return S.createElement("li",{key:t.tableData.id},S.createElement(m.default,{className:r.formControlLabel,component:"label",htmlFor:"column-toggle-".concat(t.tableData.id),disabled:!1===t.removable},S.createElement(f.default,{checked:!t.hidden,id:"column-toggle-".concat(t.tableData.id),onChange:function(){return e.props.onColumnsChanged(t,!t.hidden)}}),S.createElement("span",null,t.title)))})))),this.props.exportButton&&S.createElement("span",null,S.createElement(C.default,{title:t.exportTitle},S.createElement(p.default,{color:"inherit",onClick:function(t){return e.setState({exportButtonAnchorEl:t.currentTarget})},"aria-label":t.exportAriaLabel},S.createElement(this.props.icons.Export,null))),S.createElement(v.default,{anchorEl:this.state.exportButtonAnchorEl,open:Boolean(this.state.exportButtonAnchorEl),onClose:function(){return e.setState({exportButtonAnchorEl:null})}},S.createElement(m.default,{key:"export-csv",onClick:this.exportCsv},t.exportName))),S.createElement("span",null,S.createElement(this.props.components.Actions,{actions:this.props.actions&&this.props.actions.filter((function(e){return"toolbar"===e.position})),components:this.props.components})))}},{key:"renderSelectedActions",value:function(){return S.createElement(S.Fragment,null,S.createElement(this.props.components.Actions,{actions:this.props.actions.filter((function(e){return"toolbarOnSelect"===e.position})),data:this.props.selectedRows,components:this.props.components}))}},{key:"renderActions",value:function(){var e=this.props.classes;return S.createElement("div",{className:e.actions},S.createElement("div",null,this.props.selectedRows&&this.props.selectedRows.length>0?this.renderSelectedActions():this.renderDefaultActions()))}},{key:"renderToolbarTitle",value:function(e){var t=this.props.classes,n="string"==typeof e?S.createElement(A.default,{variant:"h6",style:{whiteSpace:"nowrap",overflow:"hidden",textOverflow:"ellipsis"}},e):e;return S.createElement("div",{className:t.title},n)}},{key:"render",value:function(){var e=this.props.classes,t=(0,o.default)({},n.defaultProps.localization,this.props.localization),r=this.props.showTextRowsSelected&&this.props.selectedRows&&this.props.selectedRows.length>0?"function"==typeof t.nRowsSelected?t.nRowsSelected(this.props.selectedRows.length):t.nRowsSelected.replace("{0}",this.props.selectedRows.length):this.props.showTitle?this.props.title:null;return S.createElement(b.default,{className:(0,x.default)(e.root,(0,h.default)({},e.highlight,this.props.showTextRowsSelected&&this.props.selectedRows&&this.props.selectedRows.length>0))},r&&this.renderToolbarTitle(r),"left"===this.props.searchFieldAlignment&&this.renderSearch(),"left"===this.props.toolbarButtonAlignment&&this.renderActions(),S.createElement("div",{className:e.spacer}),"right"===this.props.searchFieldAlignment&&this.renderSearch(),"right"===this.props.toolbarButtonAlignment&&this.renderActions())}}]),n}(S.Component);t.MTableToolbar=E,E.defaultProps={actions:[],columns:[],columnsButton:!1,localization:{addRemoveColumns:"Add or remove columns",nRowsSelected:"{0} row(s) selected",showColumnsTitle:"Show Columns",showColumnsAriaLabel:"Show Columns",exportTitle:"Export",exportAriaLabel:"Export",exportName:"Export as CSV",searchTooltip:"Search",searchPlaceholder:"Search"},search:!0,showTitle:!0,searchText:"",showTextRowsSelected:!0,toolbarButtonAlignment:"right",searchAutoFocus:!1,searchFieldAlignment:"right",searchFieldVariant:"standard",selectedRows:[],title:"No Title!"},E.propTypes={actions:M.default.array,columns:M.default.array,columnsButton:M.default.bool,components:M.default.object.isRequired,getFieldValue:M.default.func.isRequired,localization:M.default.object.isRequired,onColumnsChanged:M.default.func.isRequired,dataManager:M.default.object.isRequired,searchText:M.default.string,onSearchChanged:M.default.func.isRequired,search:M.default.bool.isRequired,searchFieldStyle:M.default.object,searchFieldVariant:M.default.string,selectedRows:M.default.array,title:M.default.oneOfType([M.default.element,M.default.string]),showTitle:M.default.bool.isRequired,showTextRowsSelected:M.default.bool.isRequired,toolbarButtonAlignment:M.default.string.isRequired,searchFieldAlignment:M.default.string.isRequired,renderData:M.default.array,data:M.default.array,exportAllData:M.default.bool,exportButton:M.default.bool,exportDelimiter:M.default.string,exportFileName:M.default.string,exportCsv:M.default.func,classes:M.default.object,searchAutoFocus:M.default.bool};var O=function(e){return{root:{paddingRight:e.spacing(1)},highlight:"light"===e.palette.type?{color:e.palette.secondary.main,backgroundColor:(0,I.lighten)(e.palette.secondary.light,.85)}:{color:e.palette.text.primary,backgroundColor:e.palette.secondary.dark},spacer:{flex:"1 1 10%"},actions:{color:e.palette.text.secondary},title:{overflow:"hidden"},searchField:{minWidth:150,paddingLeft:e.spacing(2)},formControlLabel:{paddingLeft:e.spacing(1),paddingRight:e.spacing(1)}}};t.styles=O;var k=(0,_.default)(O)(E);t.default=k},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(2298);t.CsvBuilder=r.default},function(e,t,n){"use strict";var r,i=this&&this.__extends||(r=function(e,t){return(r=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(e,t)},function(e,t){function n(){this.constructor=e}r(e,t),e.prototype=null===t?Object.create(t):(n.prototype=t.prototype,new n)}),o=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};Object.defineProperty(t,"__esModule",{value:!0});var a=function(e){function t(t){var n=e.call(this)||this;return n._FileName="",n._Delimeter=",",n._Columns=[],n._RowData=[],n._FileName=t,n}return i(t,e),t.prototype.setColumns=function(e){return this._Columns=e,this},t.prototype.setDelimeter=function(e){return this._Delimeter=e,this},t.prototype.addRow=function(e){return this._RowData.push(e),this},t.prototype.addRows=function(e){return this._RowData=this._RowData.concat(e),this},t.prototype.escapeCell=function(e){return"string"==typeof e?'"'+e.replace(/\"/g,'""')+'"':e},t.prototype.getRowData=function(e){return e.map(this.escapeCell).join(this._Delimeter)},t.prototype.exportFile=function(){var t=this,n=[];this._Columns&&this._Columns.length>0&&n.push(this.getRowData(this._Columns)),this._RowData.forEach((function(e){n.push(t.getRowData(e))}));var r=n.join("\r\n");e.prototype.exportFile.call(this,"csv",this._FileName,r)},t}(o(n(2299)).default);t.default=a},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=function(){function e(){}return e.prototype.exportFile=function(e,t,n){if(window.navigator.msSaveOrOpenBlob){var r=new Blob([n]);window.navigator.msSaveOrOpenBlob(r,t)}else{var i="data:text/"+e+";charset=utf-8,\ufeff"+encodeURIComponent(n),o=document.createElement("a");o.setAttribute("href",i),o.setAttribute("download",t),document.body.appendChild(o),o.click()}},e}();t.default=r},function(e,t,n){"use strict";var r=n(36);Object.defineProperty(t,"__esModule",{value:!0}),t.propTypes=void 0;var i=r(n(3)),o=i.default.shape({current:i.default.element}),a=i.default.shape({classes:i.default.object,innerRef:o}),s={actions:i.default.arrayOf(i.default.oneOfType([i.default.func,i.default.shape({icon:i.default.oneOfType([i.default.element,i.default.func,i.default.string,o]).isRequired,isFreeAction:i.default.bool,position:i.default.oneOf(["auto","toolbar","toolbarOnSelect","row"]),tooltip:i.default.string,onClick:i.default.func.isRequired,iconProps:i.default.object,disabled:i.default.bool,hidden:i.default.bool})])),columns:i.default.arrayOf(i.default.shape({cellStyle:i.default.oneOfType([i.default.object,i.default.func]),currencySetting:i.default.shape({locale:i.default.string,currencyCode:i.default.string,minimumFractionDigits:i.default.number,maximumFractionDigits:i.default.number}),customFilterAndSearch:i.default.func,customSort:i.default.func,defaultFilter:i.default.any,defaultSort:i.default.oneOf(["asc","desc"]),editComponent:i.default.oneOfType([i.default.element,i.default.func]),emptyValue:i.default.oneOfType([i.default.string,i.default.node,i.default.func]),export:i.default.bool,field:i.default.string,filtering:i.default.bool,filterCellStyle:i.default.object,filterPlaceholder:i.default.string,filterComponent:i.default.oneOfType([i.default.element,i.default.func]),grouping:i.default.bool,headerStyle:i.default.object,hidden:i.default.bool,hideFilterIcon:i.default.bool,initialEditValue:i.default.any,lookup:i.default.object,editable:i.default.oneOfType([i.default.func,i.default.oneOf(["always","onUpdate","onAdd","never"])]),removable:i.default.bool,render:i.default.func,searchable:i.default.bool,sorting:i.default.bool,title:i.default.oneOfType([i.default.element,i.default.string]),type:i.default.oneOf(["string","boolean","numeric","date","datetime","time","currency"])})).isRequired,components:i.default.shape({Action:i.default.oneOfType([i.default.element,i.default.func,a]),Actions:i.default.oneOfType([i.default.element,i.default.func,a]),Body:i.default.oneOfType([i.default.element,i.default.func,a]),Cell:i.default.oneOfType([i.default.element,i.default.func,a]),Container:i.default.oneOfType([i.default.element,i.default.func,a]),EditField:i.default.oneOfType([i.default.element,i.default.func,a]),EditRow:i.default.oneOfType([i.default.element,i.default.func,a]),FilterRow:i.default.oneOfType([i.default.element,i.default.func,a]),Groupbar:i.default.oneOfType([i.default.element,i.default.func,a]),GroupRow:i.default.oneOfType([i.default.element,i.default.func,a]),Header:i.default.oneOfType([i.default.element,i.default.func,a]),OverlayLoading:i.default.oneOfType([i.default.element,i.default.func,a]),Pagination:i.default.oneOfType([i.default.element,i.default.func,a]),Row:i.default.oneOfType([i.default.element,i.default.func,a]),Toolbar:i.default.oneOfType([i.default.element,i.default.func,a])}),data:i.default.oneOfType([i.default.arrayOf(i.default.object),i.default.func]).isRequired,editable:i.default.shape({isEditable:i.default.func,isDeletable:i.default.func,onRowAdd:i.default.func,onRowUpdate:i.default.func,onRowDelete:i.default.func,onRowAddCancelled:i.default.func,onRowUpdateCancelled:i.default.func,isEditHidden:i.default.func,isDeleteHidden:i.default.func}),detailPanel:i.default.oneOfType([i.default.func,i.default.arrayOf(i.default.oneOfType([i.default.func,i.default.shape({disabled:i.default.bool,icon:i.default.oneOfType([i.default.element,i.default.func,i.default.string,o]),openIcon:i.default.oneOfType([i.default.element,i.default.func,i.default.string,o]),tooltip:i.default.string,render:i.default.func.isRequired})]))]),icons:i.default.shape({Add:i.default.oneOfType([i.default.element,i.default.func,o]),Check:i.default.oneOfType([i.default.element,i.default.func,o]),Clear:i.default.oneOfType([i.default.element,i.default.func,o]),Delete:i.default.oneOfType([i.default.element,i.default.func,o]),DetailPanel:i.default.oneOfType([i.default.element,i.default.func,o]),Edit:i.default.oneOfType([i.default.element,i.default.func,o]),Export:i.default.oneOfType([i.default.element,i.default.func,o]),Filter:i.default.oneOfType([i.default.element,i.default.func,o]),FirstPage:i.default.oneOfType([i.default.element,i.default.func,o]),LastPage:i.default.oneOfType([i.default.element,i.default.func,o]),NextPage:i.default.oneOfType([i.default.element,i.default.func,o]),PreviousPage:i.default.oneOfType([i.default.element,i.default.func,o]),Refresh:i.default.oneOfType([i.default.element,i.default.func,o]),ResetSearch:i.default.oneOfType([i.default.element,i.default.func,o]),Search:i.default.oneOfType([i.default.element,i.default.func,o]),SortArrow:i.default.oneOfType([i.default.element,i.default.func,o]),ThirdStateCheck:i.default.oneOfType([i.default.element,i.default.func,o]),ViewColumn:i.default.oneOfType([i.default.element,i.default.func,o])}),isLoading:i.default.bool,title:i.default.oneOfType([i.default.element,i.default.string]),options:i.default.shape({actionsCellStyle:i.default.object,editCellStyle:i.default.object,detailPanelColumnStyle:i.default.object,actionsColumnIndex:i.default.number,addRowPosition:i.default.oneOf(["first","last"]),columnsButton:i.default.bool,defaultExpanded:i.default.bool|i.default.func,debounceInterval:i.default.number,detailPanelType:i.default.oneOf(["single","multiple"]),doubleHorizontalScroll:i.default.bool,emptyRowsWhenPaging:i.default.bool,exportAllData:i.default.bool,exportButton:i.default.bool,exportDelimiter:i.default.string,exportFileName:i.default.string,exportCsv:i.default.func,filtering:i.default.bool,filterCellStyle:i.default.object,filterRowStyle:i.default.object,header:i.default.bool,headerStyle:i.default.object,hideFilterIcons:i.default.bool,initialPage:i.default.number,maxBodyHeight:i.default.oneOfType([i.default.number,i.default.string]),minBodyHeight:i.default.oneOfType([i.default.number,i.default.string]),loadingType:i.default.oneOf(["overlay","linear"]),overflowY:i.default.oneOf(["visible","hidden","scroll","auto","initial","inherit"]),padding:i.default.oneOf(["default","dense"]),paging:i.default.bool,pageSize:i.default.number,pageSizeOptions:i.default.arrayOf(i.default.oneOfType([i.default.number,i.default.shape({value:i.default.number,label:i.default.string})])),paginationType:i.default.oneOf(["normal","stepped"]),rowStyle:i.default.oneOfType([i.default.object,i.default.func]),search:i.default.bool,searchText:i.default.string,toolbarButtonAlignment:i.default.oneOf(["left","right"]),searchFieldAlignment:i.default.oneOf(["left","right"]),searchFieldStyle:i.default.object,searchAutoFocus:i.default.bool,searchFieldVariant:i.default.oneOf(["standard","filled","outlined"]),selection:i.default.bool,selectionProps:i.default.oneOfType([i.default.object,i.default.func]),showEmptyDataSourceMessage:i.default.bool,showFirstLastPageButtons:i.default.bool,showSelectAllCheckbox:i.default.bool,showTitle:i.default.bool,showTextRowsSelected:i.default.bool,sorting:i.default.bool,toolbar:i.default.bool,thirdSortClick:i.default.bool}),localization:i.default.shape({grouping:i.default.shape({groupedBy:i.default.string,placeholder:i.default.string}),pagination:i.default.object,toolbar:i.default.object,header:i.default.object,body:i.default.object}),initialFormData:i.default.object,onSearchChange:i.default.func,onFilterChange:i.default.func,onColumnDragged:i.default.func,onGroupRemoved:i.default.func,onSelectionChange:i.default.func,onChangeRowsPerPage:i.default.func,onChangePage:i.default.func,onChangeColumnHidden:i.default.func,onOrderChange:i.default.func,onRowClick:i.default.func,onTreeExpandChange:i.default.func,onQueryChange:i.default.func,tableRef:i.default.any,style:i.default.object,page:i.default.number,totalCount:i.default.number};t.propTypes=s},function(e,t,n){"use strict";var r=n(46),i=n(36);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(50)),a=i(n(223)),s=i(n(114)),l=i(n(33)),u=i(n(34)),c=i(n(9)),d=i(n(45)),h=i(n(43)),f=i(n(44)),p=i(n(8)),g=i(n(310)),v=i(n(430)),m=i(n(154)),y=i(n(316)),b=i(n(2302)),C=r(n(0)),A=n(616),_=n(621),I=i(n(2303)),x=n(2304),w=i(n(2305)),M=n(90),S=r(n(423));function T(e){var t=function(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],(function(){}))),!0}catch(e){return!1}}();return function(){var n,r=(0,f.default)(e);if(t){var i=(0,f.default)(this).constructor;n=Reflect.construct(r,arguments,i)}else n=r.apply(this,arguments);return(0,h.default)(this,n)}}var E=function(e){(0,d.default)(n,e);var t=T(n);function n(e){var r;(0,l.default)(this,n),r=t.call(this,e),(0,p.default)((0,c.default)(r),"dataManager",new I.default),(0,p.default)((0,c.default)(r),"isRemoteData",(function(e){return!Array.isArray((e||r.props).data)})),(0,p.default)((0,c.default)(r),"isOutsidePageNumbers",(function(e){return void 0!==e.page&&void 0!==e.totalCount})),(0,p.default)((0,c.default)(r),"onAllSelected",(function(e){r.dataManager.changeAllSelected(e),r.setState(r.dataManager.getRenderState(),(function(){return r.onSelectionChange()}))})),(0,p.default)((0,c.default)(r),"onChangeColumnHidden",(function(e,t){r.dataManager.changeColumnHidden(e,t),r.setState(r.dataManager.getRenderState(),(function(){r.props.onChangeColumnHidden&&r.props.onChangeColumnHidden(e,t)}))})),(0,p.default)((0,c.default)(r),"onChangeGroupOrder",(function(e){r.dataManager.changeGroupOrder(e.tableData.id),r.setState(r.dataManager.getRenderState())})),(0,p.default)((0,c.default)(r),"onChangeOrder",(function(e,t){var n=""===t?-1:e;if(r.dataManager.changeOrder(n,t),r.isRemoteData()){var i=(0,s.default)({},r.state.query);i.page=0,i.orderBy=r.state.columns.find((function(e){return e.tableData.id===n})),i.orderDirection=t,r.onQueryChange(i,(function(){r.props.onOrderChange&&r.props.onOrderChange(n,t)}))}else r.setState(r.dataManager.getRenderState(),(function(){r.props.onOrderChange&&r.props.onOrderChange(n,t)}))})),(0,p.default)((0,c.default)(r),"onChangePage",(function(e,t){if(r.isRemoteData()){var n=(0,s.default)({},r.state.query);n.page=t,r.onQueryChange(n,(function(){r.props.onChangePage&&r.props.onChangePage(t)}))}else r.isOutsidePageNumbers(r.props)||r.dataManager.changeCurrentPage(t),r.setState(r.dataManager.getRenderState(),(function(){r.props.onChangePage&&r.props.onChangePage(t)}))})),(0,p.default)((0,c.default)(r),"onChangeRowsPerPage",(function(e){var t=e.target.value;if(r.dataManager.changePageSize(t),r.props.onChangePage&&r.props.onChangePage(0),r.isRemoteData()){var n=(0,s.default)({},r.state.query);n.pageSize=e.target.value,n.page=0,r.onQueryChange(n,(function(){r.props.onChangeRowsPerPage&&r.props.onChangeRowsPerPage(t)}))}else r.dataManager.changeCurrentPage(0),r.setState(r.dataManager.getRenderState(),(function(){r.props.onChangeRowsPerPage&&r.props.onChangeRowsPerPage(t)}))})),(0,p.default)((0,c.default)(r),"onDragEnd",(function(e){e&&e.source&&e.destination&&(r.dataManager.changeByDrag(e),r.setState(r.dataManager.getRenderState(),(function(){r.props.onColumnDragged&&"headers"===e.destination.droppableId&&"headers"===e.source.droppableId&&r.props.onColumnDragged(e.source.index,e.destination.index)})))})),(0,p.default)((0,c.default)(r),"onGroupExpandChanged",(function(e){r.dataManager.changeGroupExpand(e),r.setState(r.dataManager.getRenderState())})),(0,p.default)((0,c.default)(r),"onGroupRemoved",(function(e,t){var n={combine:null,destination:{droppableId:"headers",index:0},draggableId:e.tableData.id,mode:"FLUID",reason:"DROP",source:{index:t,droppableId:"groups"},type:"DEFAULT"};r.dataManager.changeByDrag(n),r.setState(r.dataManager.getRenderState(),(function(){r.props.onGroupRemoved&&r.props.onGroupRemoved(e,t)}))})),(0,p.default)((0,c.default)(r),"onEditingApproved",(function(e,t,n){"add"===e&&r.props.editable&&r.props.editable.onRowAdd?r.setState({isLoading:!0},(function(){r.props.editable.onRowAdd(t).then((function(e){r.setState({isLoading:!1,showAddRow:!1},(function(){r.isRemoteData()&&r.onQueryChange(r.state.query)}))})).catch((function(e){r.setState({isLoading:!1})}))})):"update"===e&&r.props.editable&&r.props.editable.onRowUpdate?r.setState({isLoading:!0},(function(){r.props.editable.onRowUpdate(t,n).then((function(e){r.dataManager.changeRowEditing(n),r.setState((0,s.default)({isLoading:!1},r.dataManager.getRenderState()),(function(){r.isRemoteData()&&r.onQueryChange(r.state.query)}))})).catch((function(e){r.setState({isLoading:!1})}))})):"delete"===e&&r.props.editable&&r.props.editable.onRowDelete&&r.setState({isLoading:!0},(function(){r.props.editable.onRowDelete(n).then((function(e){r.dataManager.changeRowEditing(n),r.setState((0,s.default)({isLoading:!1},r.dataManager.getRenderState()),(function(){r.isRemoteData()&&r.onQueryChange(r.state.query)}))})).catch((function(e){r.setState({isLoading:!1})}))}))})),(0,p.default)((0,c.default)(r),"onEditingCanceled",(function(e,t){"add"===e?(r.props.editable.onRowAddCancelled&&r.props.editable.onRowAddCancelled(),r.setState({showAddRow:!1})):"update"===e?(r.props.editable.onRowUpdateCancelled&&r.props.editable.onRowUpdateCancelled(),r.dataManager.changeRowEditing(t),r.setState(r.dataManager.getRenderState())):"delete"===e&&(r.dataManager.changeRowEditing(t),r.setState(r.dataManager.getRenderState()))})),(0,p.default)((0,c.default)(r),"onQueryChange",(function(e,t){e=(0,s.default)({},r.state.query,e),r.setState({isLoading:!0},(function(){r.props.data(e).then((function(n){e.totalCount=n.totalCount,e.page=n.page,r.dataManager.setData(n.data),r.setState((0,s.default)({isLoading:!1},r.dataManager.getRenderState(),{query:e}),(function(){t&&t()}))}))}))})),(0,p.default)((0,c.default)(r),"onRowSelected",(function(e,t,n){r.dataManager.changeRowSelected(e.target.checked,t),r.setState(r.dataManager.getRenderState(),(function(){return r.onSelectionChange(n)}))})),(0,p.default)((0,c.default)(r),"onSelectionChange",(function(e){if(r.props.onSelectionChange){var t=[];!function e(n){n.forEach((function(n){n.tableData.checked&&t.push(n),n.tableData.childRows&&e(n.tableData.childRows)}))}(r.state.originalData),r.props.onSelectionChange(t,e)}})),(0,p.default)((0,c.default)(r),"onSearchChangeDebounce",(0,x.debounce)((function(e){if(r.isRemoteData()){var t=(0,s.default)({},r.state.query);t.page=0,t.search=e,r.onQueryChange(t)}else r.setState(r.dataManager.getRenderState(),(function(){r.props.onSearchChange&&r.props.onSearchChange(e)}))}),r.props.options.debounceInterval)),(0,p.default)((0,c.default)(r),"onFilterChange",(function(e,t){r.dataManager.changeFilterValue(e,t),r.setState({},r.onFilterChangeDebounce)})),(0,p.default)((0,c.default)(r),"onFilterChangeDebounce",(0,x.debounce)((function(){if(r.isRemoteData()){var e=(0,s.default)({},r.state.query);e.page=0,e.filters=r.state.columns.filter((function(e){return e.tableData.filterValue})).map((function(e){return{column:e,operator:"=",value:e.tableData.filterValue}})),r.onQueryChange(e)}else r.setState(r.dataManager.getRenderState(),(function(){if(r.props.onFilterChange){var e=r.state.columns.filter((function(e){return e.tableData.filterValue})).map((function(e){return{column:e,operator:"=",value:e.tableData.filterValue}}));r.props.onFilterChange(e)}}))}),r.props.options.debounceInterval)),(0,p.default)((0,c.default)(r),"onTreeExpandChanged",(function(e,t){r.dataManager.changeTreeExpand(e),r.setState(r.dataManager.getRenderState(),(function(){r.props.onTreeExpandChange&&r.props.onTreeExpandChange(t,t.tableData.isTreeExpanded)}))})),(0,p.default)((0,c.default)(r),"onToggleDetailPanel",(function(e,t){r.dataManager.changeDetailPanelVisibility(e,t),r.setState(r.dataManager.getRenderState())})),(0,p.default)((0,c.default)(r),"renderTable",(function(e){return C.createElement(g.default,{style:{tableLayout:e.options.fixedColumns&&(e.options.fixedColumns.left||e.options.fixedColumns.right)?"fixed":e.options.tableLayout}},e.options.header&&C.createElement(e.components.Header,{actions:e.actions,localization:(0,s.default)({},n.defaultProps.localization.header,r.props.localization.header),columns:r.state.columns,hasSelection:e.options.selection,headerStyle:e.options.headerStyle,icons:e.icons,selectedCount:r.state.selectedCount,dataCount:e.parentChildData?r.state.treefiedDataLength:r.state.columns.filter((function(e){return e.tableData.groupOrder>-1})).length>0?r.state.groupedDataLength:r.state.data.length,hasDetailPanel:!!e.detailPanel,detailPanelColumnAlignment:e.options.detailPanelColumnAlignment,showActionsColumn:e.actions&&e.actions.filter((function(e){return"row"===e.position||"function"==typeof e})).length>0,showSelectAllCheckbox:e.options.showSelectAllCheckbox,orderBy:r.state.orderBy,orderDirection:r.state.orderDirection,onAllSelected:r.onAllSelected,onOrderChange:r.onChangeOrder,actionsHeaderIndex:e.options.actionsColumnIndex,sorting:e.options.sorting,grouping:e.options.grouping,isTreeData:void 0!==r.props.parentChildData,draggable:e.options.draggable,thirdSortClick:e.options.thirdSortClick,treeDataMaxLevel:r.state.treeDataMaxLevel,options:e.options}),C.createElement(e.components.Body,{actions:e.actions,components:e.components,icons:e.icons,renderData:r.state.renderData,currentPage:r.state.currentPage,initialFormData:e.initialFormData,pageSize:r.state.pageSize,columns:r.state.columns,detailPanel:e.detailPanel,options:e.options,getFieldValue:r.dataManager.getFieldValue,isTreeData:void 0!==r.props.parentChildData,onFilterChanged:r.onFilterChange,onRowSelected:r.onRowSelected,onToggleDetailPanel:r.onToggleDetailPanel,onGroupExpandChanged:r.onGroupExpandChanged,onTreeExpandChanged:r.onTreeExpandChanged,onEditingCanceled:r.onEditingCanceled,onEditingApproved:r.onEditingApproved,localization:(0,s.default)({},n.defaultProps.localization.body,r.props.localization.body),onRowClick:r.props.onRowClick,showAddRow:r.state.showAddRow,hasAnyEditingRow:!(!r.state.lastEditingRow&&!r.state.showAddRow),hasDetailPanel:!!e.detailPanel,treeDataMaxLevel:r.state.treeDataMaxLevel}))})),(0,p.default)((0,c.default)(r),"getColumnsWidth",(function(e,t){var n=[],i=S.actionsColumnWidth(e);if(i>0&&(t>0&&e.options.actionsColumnIndex>=0&&e.options.actionsColumnIndex<t||t<0&&e.options.actionsColumnIndex<0&&e.options.actionsColumnIndex>=t)&&n.push(i+"px"),e.options.selection){var o=S.selectionMaxWidth(e,r.state.treeDataMaxLevel);n.push(o+"px")}for(var a=0;a<Math.abs(t)&&a<e.columns.length;a++){var s=e.columns[a>=0?a:e.columns.length-1-a];s.tableData&&("number"==typeof s.tableData.width?n.push(s.tableData.width+"px"):n.push(s.tableData.width))}return"calc("+n.join(" + ")+")"}));var i=r.getProps(e);r.setDataManagerFields(i,!0);var o=r.dataManager.getRenderState();return r.state=(0,s.default)({data:[]},o,{query:{filters:o.columns.filter((function(e){return e.tableData.filterValue})).map((function(e){return{column:e,operator:"=",value:e.tableData.filterValue}})),orderBy:o.columns.find((function(e){return e.tableData.id===o.orderBy})),orderDirection:o.orderDirection,page:0,pageSize:i.options.pageSize,search:o.searchText,totalCount:0},showAddRow:!1,width:0}),r.tableContainerDiv=C.createRef(),r}return(0,u.default)(n,[{key:"componentDidMount",value:function(){var e=this;this.setState((0,s.default)({},this.dataManager.getRenderState(),{width:this.tableContainerDiv.current.scrollWidth}),(function(){e.isRemoteData()&&e.onQueryChange(e.state.query)}))}},{key:"setDataManagerFields",value:function(e,t){var n=-1,r="";e&&!1!==e.options.sorting&&(r=(n=e.columns.findIndex((function(e){return e.defaultSort&&!1!==e.sorting})))>-1?e.columns[n].defaultSort:""),this.dataManager.setColumns(e.columns),this.dataManager.setDefaultExpanded(e.options.defaultExpanded),this.dataManager.changeRowEditing(),this.isRemoteData(e)?(this.dataManager.changeApplySearch(!1),this.dataManager.changeApplyFilters(!1),this.dataManager.changeApplySort(!1)):(this.dataManager.changeApplySearch(!0),this.dataManager.changeApplyFilters(!0),this.dataManager.changeApplySort(!0),this.dataManager.setData(e.data)),(t||n!==this.dataManager.orderBy&&r!==this.dataManager.orderDirection)&&this.dataManager.changeOrder(n,r),t&&this.dataManager.changeSearchText(e.options.searchText||""),t&&this.dataManager.changeCurrentPage(e.options.initialPage?e.options.initialPage:0),(t||this.isRemoteData())&&this.dataManager.changePageSize(e.options.pageSize),this.dataManager.changePaging(e.options.paging),t&&this.dataManager.changeParentFunc(e.parentChildData),this.dataManager.changeDetailPanelType(e.options.detailPanelType)}},{key:"cleanColumns",value:function(e){return e.map((function(e){var t=(0,s.default)({},e);return delete t.tableData,t}))}},{key:"componentDidUpdate",value:function(e){var t=this.cleanColumns(e.columns),n=this.cleanColumns(this.props.columns),r=!(0,w.default)(t,n);if(r=r||!(0,w.default)(e.options,this.props.options),this.isRemoteData()||(r=r||!(0,w.default)(e.data,this.props.data)),r){var i=this.getProps(this.props);this.setDataManagerFields(i),this.setState(this.dataManager.getRenderState())}var o=this.isRemoteData()?this.state.query.totalCount:this.state.data.length,a=this.isRemoteData()?this.state.query.page:this.state.currentPage,s=this.isRemoteData()?this.state.query.pageSize:this.state.pageSize;o<=s*a&&0!==a&&this.onChangePage(null,Math.max(0,Math.ceil(o/s)-1))}},{key:"getProps",value:function(e){var t=this,r=(0,s.default)({},e||this.props);r.components=(0,s.default)({},n.defaultProps.components,r.components),r.icons=(0,s.default)({},n.defaultProps.icons,r.icons),r.options=(0,s.default)({},n.defaultProps.options,r.options);var i=(0,s.default)({},n.defaultProps.localization.body,r.localization.body);return r.actions=(0,a.default)(r.actions||[]),r.options.selection?r.actions=r.actions.filter((function(e){return e})).map((function(e){return"auto"===e.position||!1===e.isFreeAction||void 0===e.position&&void 0===e.isFreeAction?"function"==typeof e?{action:e,position:"toolbarOnSelect"}:(0,s.default)({},e,{position:"toolbarOnSelect"}):e.isFreeAction?"function"==typeof e?{action:e,position:"toolbar"}:(0,s.default)({},e,{position:"toolbar"}):e})):r.actions=r.actions.filter((function(e){return e})).map((function(e){return"auto"===e.position||!1===e.isFreeAction||void 0===e.position&&void 0===e.isFreeAction?"function"==typeof e?{action:e,position:"row"}:(0,s.default)({},e,{position:"row"}):e.isFreeAction?"function"==typeof e?{action:e,position:"toolbar"}:(0,s.default)({},e,{position:"toolbar"}):e})),r.editable&&(r.editable.onRowAdd&&r.actions.push({icon:r.icons.Add,tooltip:i.addTooltip,position:"toolbar",disabled:!!this.dataManager.lastEditingRow,onClick:function(){t.dataManager.changeRowEditing(),t.setState((0,s.default)({},t.dataManager.getRenderState(),{showAddRow:!t.state.showAddRow}))}}),r.editable.onRowUpdate&&r.actions.push((function(e){return{icon:r.icons.Edit,tooltip:r.editable.editTooltip?r.editable.editTooltip(e):i.editTooltip,disabled:r.editable.isEditable&&!r.editable.isEditable(e),hidden:r.editable.isEditHidden&&r.editable.isEditHidden(e),onClick:function(e,n){t.dataManager.changeRowEditing(n,"update"),t.setState((0,s.default)({},t.dataManager.getRenderState(),{showAddRow:!1}))}}})),r.editable.onRowDelete&&r.actions.push((function(e){return{icon:r.icons.Delete,tooltip:r.editable.deleteTooltip?r.editable.deleteTooltip(e):i.deleteTooltip,disabled:r.editable.isDeletable&&!r.editable.isDeletable(e),hidden:r.editable.isDeleteHidden&&r.editable.isDeleteHidden(e),onClick:function(e,n){t.dataManager.changeRowEditing(n,"delete"),t.setState((0,s.default)({},t.dataManager.getRenderState(),{showAddRow:!1}))}}}))),r}},{key:"renderFooter",value:function(){var e=this.getProps();if(e.options.paging){var t=(0,s.default)({},n.defaultProps.localization.pagination,this.props.localization.pagination),r=this.isOutsidePageNumbers(e),i=r?Math.min(e.page,Math.floor(e.totalCount/this.state.pageSize)):this.state.currentPage,a=r?e.totalCount:this.state.data.length;return C.createElement(g.default,null,C.createElement(v.default,{style:{display:"grid"}},C.createElement(m.default,null,C.createElement(e.components.Pagination,{classes:{root:e.classes.paginationRoot,toolbar:e.classes.paginationToolbar,caption:e.classes.paginationCaption,selectRoot:e.classes.paginationSelectRoot},style:{float:"rtl"===e.theme.direction?"":"right",overflowX:"auto"},colSpan:3,count:this.isRemoteData()?this.state.query.totalCount:a,icons:e.icons,rowsPerPage:this.state.pageSize,rowsPerPageOptions:e.options.pageSizeOptions,SelectProps:{renderValue:function(e){return C.createElement("div",{style:{padding:"0px 5px"}},e+" "+t.labelRowsSelect+" ")}},page:this.isRemoteData()?this.state.query.page:i,onChangePage:this.onChangePage,onChangeRowsPerPage:this.onChangeRowsPerPage,ActionsComponent:function(n){return"normal"===e.options.paginationType?C.createElement(A.MTablePagination,(0,o.default)({},n,{icons:e.icons,localization:t,showFirstLastPageButtons:e.options.showFirstLastPageButtons})):C.createElement(A.MTableSteppedPagination,(0,o.default)({},n,{icons:e.icons,localization:t,showFirstLastPageButtons:e.options.showFirstLastPageButtons}))},labelDisplayedRows:function(e){return t.labelDisplayedRows.replace("{from}",e.from).replace("{to}",e.to).replace("{count}",e.count)},labelRowsPerPage:t.labelRowsPerPage}))))}}},{key:"render",value:function(){var e=this,t=this.getProps();return C.createElement(_.DragDropContext,{onDragEnd:this.onDragEnd,nonce:t.options.cspNonce},C.createElement(t.components.Container,{style:(0,s.default)({position:"relative"},t.style)},t.options.toolbar&&C.createElement(t.components.Toolbar,{actions:t.actions,components:t.components,selectedRows:this.state.selectedCount>0?this.state.originalData.filter((function(e){return e.tableData.checked})):[],columns:this.state.columns,columnsButton:t.options.columnsButton,icons:t.icons,exportAllData:t.options.exportAllData,exportButton:t.options.exportButton,exportDelimiter:t.options.exportDelimiter,exportFileName:t.options.exportFileName,exportCsv:t.options.exportCsv,getFieldValue:this.dataManager.getFieldValue,data:this.state.data,renderData:this.state.renderData,search:t.options.search,showTitle:t.options.showTitle,showTextRowsSelected:t.options.showTextRowsSelected,toolbarButtonAlignment:t.options.toolbarButtonAlignment,searchFieldAlignment:t.options.searchFieldAlignment,searchAutoFocus:t.options.searchAutoFocus,searchFieldStyle:t.options.searchFieldStyle,searchFieldVariant:t.options.searchFieldVariant,title:t.title,searchText:this.dataManager.searchText,onSearchChanged:this.onSearchChangeDebounce,dataManager:this.dataManager,onColumnsChanged:this.onChangeColumnHidden,localization:(0,s.default)({},n.defaultProps.localization.toolbar,this.props.localization.toolbar)}),t.options.grouping&&C.createElement(t.components.Groupbar,{icons:t.icons,localization:(0,s.default)({},n.defaultProps.localization.grouping,t.localization.grouping),groupColumns:this.state.columns.filter((function(e){return e.tableData.groupOrder>-1})).sort((function(e,t){return e.tableData.groupOrder-t.tableData.groupOrder})),onSortChanged:this.onChangeGroupOrder,onGroupRemoved:this.onGroupRemoved}),C.createElement(O,{double:t.options.doubleHorizontalScroll},C.createElement(_.Droppable,{droppableId:"headers",direction:"horizontal"},(function(n,r){var i=e.renderTable(t);return C.createElement("div",{ref:n.innerRef},C.createElement("div",{ref:e.tableContainerDiv,style:{maxHeight:t.options.maxBodyHeight,minHeight:t.options.minBodyHeight,overflowY:t.options.overflowY}},e.state.width&&t.options.fixedColumns&&t.options.fixedColumns.right?C.createElement("div",{style:{width:e.getColumnsWidth(t,-1*t.options.fixedColumns.right),position:"absolute",top:0,right:0,boxShadow:"-2px 0px 15px rgba(125,147,178,.25)",overflowX:"hidden",zIndex:11}},C.createElement("div",{style:{width:e.state.width,background:"white",transform:"translateX(calc(".concat(e.getColumnsWidth(t,-1*t.options.fixedColumns.right)," - 100%))")}},i)):null,C.createElement("div",null,i),e.state.width&&t.options.fixedColumns&&t.options.fixedColumns.left?C.createElement("div",{style:{width:e.getColumnsWidth(t,t.options.fixedColumns.left),position:"absolute",top:0,left:0,boxShadow:"2px 0px 15px rgba(125,147,178,.25)",overflowX:"hidden",zIndex:11}},C.createElement("div",{style:{width:e.state.width,background:"white"}},i)):null),n.placeholder)}))),(this.state.isLoading||t.isLoading)&&"linear"===t.options.loadingType&&C.createElement("div",{style:{position:"relative",width:"100%"}},C.createElement("div",{style:{position:"absolute",top:0,left:0,height:"100%",width:"100%"}},C.createElement(y.default,null))),this.renderFooter(),(this.state.isLoading||t.isLoading)&&"overlay"===t.options.loadingType&&C.createElement("div",{style:{position:"absolute",top:0,left:0,height:"100%",width:"100%",zIndex:11}},C.createElement(t.components.OverlayLoading,{theme:t.theme}))))}}]),n}(C.Component);t.default=E;var O=(0,M.withStyles)((function(){return{horizontalScrollContainer:{"& ::-webkit-scrollbar":{"-webkit-appearance":"none"},"& ::-webkit-scrollbar:horizontal":{height:8},"& ::-webkit-scrollbar-thumb":{borderRadius:4,border:"2px solid white",backgroundColor:"rgba(0, 0, 0, .3)"}}}}))((function(e){var t=e.double,n=e.children,r=e.classes;return t?C.createElement(b.default,null,n):C.createElement("div",{className:r.horizontalScrollContainer,style:{overflowX:"auto",position:"relative"}},n)}))},function(e,t,n){var r;r=function(e){return function(e){var t={};function n(r){if(t[r])return t[r].exports;var i=t[r]={exports:{},id:r,loaded:!1};return e[r].call(i.exports,i,i.exports,n),i.loaded=!0,i.exports}return n.m=e,n.c=t,n.p="",n(0)}([function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r,i=function(){function e(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}return function(t,n,r){return n&&e(t.prototype,n),r&&e(t,r),t}}(),o=n(1),a=(r=o)&&r.__esModule?r:{default:r},s=function(e){function t(e){!function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,t);var n=function(e,t){if(!e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return!t||"object"!=typeof t&&"function"!=typeof t?e:t}(this,(t.__proto__||Object.getPrototypeOf(t)).call(this,e));return n.state={width:"auto"},n.boundCalculateWidth=n.calculateWidth.bind(n),n}return function(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function, not "+typeof t);e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,enumerable:!1,writable:!0,configurable:!0}}),t&&(Object.setPrototypeOf?Object.setPrototypeOf(e,t):e.__proto__=t)}(t,e),i(t,[{key:"componentDidMount",value:function(){var e=this.refs.outerDiv,t=this.refs.childrenWrapper;this.calculateWidth(),window.addEventListener("resize",this.boundCalculateWidth),e.onscroll=function(){t.scrollLeft=e.scrollLeft},t.onscroll=function(){e.scrollLeft=t.scrollLeft}}},{key:"componentWillUnmount",value:function(){window.removeEventListener("resize",this.boundCalculateWidth)}},{key:"componentDidUpdate",value:function(){this.calculateWidth()}},{key:"calculateWidth",value:function(){var e=this.getChildWrapperWidth();null==e&&(e="auto"),e!==this.state.width&&this.setState({width:e})}},{key:"getChildWrapperWidth",value:function(){var e=null;return this.refs.childrenWrapper&&this.refs.childrenWrapper.scrollWidth&&(e=this.refs.childrenWrapper.scrollWidth+"px"),e}},{key:"render",value:function(){var e={paddingTop:"1px",width:this.state.width};return a.default.createElement("div",null,a.default.createElement("div",{ref:"outerDiv",style:{overflowX:"auto",overflowY:"hidden"}},a.default.createElement("div",{ref:"innerDiv",style:e}," ")),a.default.createElement("div",{ref:"childrenWrapper",style:{overflow:"auto",overflowY:"hidden"}},this.props.children))}}]),t}(a.default.Component);t.default=s,e.exports=t.default},function(t,n){t.exports=e}])},e.exports=r(n(0))},function(e,t,n){"use strict";var r=n(36);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var i=r(n(114)),o=r(n(223)),a=r(n(33)),s=r(n(34)),l=r(n(8)),u=r(n(637)),c=n(1177),d=function(){function e(){var t=this;(0,a.default)(this,e),(0,l.default)(this,"applyFilters",!1),(0,l.default)(this,"applySearch",!1),(0,l.default)(this,"applySort",!1),(0,l.default)(this,"currentPage",0),(0,l.default)(this,"detailPanelType","multiple"),(0,l.default)(this,"lastDetailPanelRow",void 0),(0,l.default)(this,"lastEditingRow",void 0),(0,l.default)(this,"orderBy",-1),(0,l.default)(this,"orderDirection",""),(0,l.default)(this,"pageSize",5),(0,l.default)(this,"paging",!0),(0,l.default)(this,"parentFunc",null),(0,l.default)(this,"searchText",""),(0,l.default)(this,"selectedCount",0),(0,l.default)(this,"treefiedDataLength",0),(0,l.default)(this,"treeDataMaxLevel",0),(0,l.default)(this,"groupedDataLength",0),(0,l.default)(this,"defaultExpanded",!1),(0,l.default)(this,"data",[]),(0,l.default)(this,"columns",[]),(0,l.default)(this,"filteredData",[]),(0,l.default)(this,"searchedData",[]),(0,l.default)(this,"groupedData",[]),(0,l.default)(this,"treefiedData",[]),(0,l.default)(this,"sortedData",[]),(0,l.default)(this,"pagedData",[]),(0,l.default)(this,"renderData",[]),(0,l.default)(this,"filtered",!1),(0,l.default)(this,"searched",!1),(0,l.default)(this,"grouped",!1),(0,l.default)(this,"treefied",!1),(0,l.default)(this,"sorted",!1),(0,l.default)(this,"paged",!1),(0,l.default)(this,"rootGroupsIndex",{}),(0,l.default)(this,"expandTreeForNodes",(function(e){e.forEach((function(e){for(var n=e;t.parentFunc(n,t.data);){var r=t.parentFunc(n,t.data);r&&(r.tableData.isTreeExpanded=!0),n=r}}))})),(0,l.default)(this,"findDataByPath",(function(e,n){if(t.isDataType("tree"))return n.reduce((function(e,t){return e&&e.tableData&&e.tableData.childRows&&e.tableData.childRows[t]}),{tableData:{childRows:e}});var r={groups:e};return n.reduce((function(e,t){return e.groups.length>0?e.groups[t]:e.data?e.data[t]:void 0}),r)})),(0,l.default)(this,"getFieldValue",(function(e,t){var n=!(arguments.length>2&&void 0!==arguments[2])||arguments[2],r=void 0!==e[t.field]?e[t.field]:(0,c.byString)(e,t.field);return t.lookup&&n&&(r=t.lookup[r]),r})),(0,l.default)(this,"getRenderState",(function(){return!1===t.filtered&&t.filterData(),!1===t.searched&&t.searchData(),!1===t.grouped&&t.isDataType("group")&&t.groupData(),!1===t.treefied&&t.isDataType("tree")&&t.treefyData(),!1===t.sorted&&t.sortData(),!1===t.paged&&t.pageData(),{columns:t.columns,currentPage:t.currentPage,data:t.sortedData,lastEditingRow:t.lastEditingRow,orderBy:t.orderBy,orderDirection:t.orderDirection,originalData:t.data,pageSize:t.pageSize,renderData:t.pagedData,searchText:t.searchText,selectedCount:t.selectedCount,treefiedDataLength:t.treefiedDataLength,treeDataMaxLevel:t.treeDataMaxLevel,groupedDataLength:t.groupedDataLength}})),(0,l.default)(this,"filterData",(function(){t.searched=t.grouped=t.treefied=t.sorted=t.paged=!1,t.filteredData=(0,o.default)(t.data),t.applyFilters&&t.columns.filter((function(e){return e.tableData.filterValue})).forEach((function(e){var n=e.lookup,r=e.type,i=e.tableData;e.customFilterAndSearch?t.filteredData=t.filteredData.filter((function(t){return!!e.customFilterAndSearch(i.filterValue,t,e)})):n?t.filteredData=t.filteredData.filter((function(n){var r=t.getFieldValue(n,e,!1);return!i.filterValue||0===i.filterValue.length||i.filterValue.indexOf(null!=r&&r.toString())>-1})):"numeric"===r?t.filteredData=t.filteredData.filter((function(n){return t.getFieldValue(n,e)+""===i.filterValue})):"boolean"===r&&i.filterValue?t.filteredData=t.filteredData.filter((function(n){var r=t.getFieldValue(n,e);return r&&"checked"===i.filterValue||!r&&"unchecked"===i.filterValue})):["date","datetime"].includes(r)?t.filteredData=t.filteredData.filter((function(n){var o=t.getFieldValue(n,e),a=o?new Date(o):null;if(a&&"Invalid Date"!==a.toString()){var s=i.filterValue,l="",c="";return"date"===r?(l=(0,u.default)(a,"MM/dd/yyyy"),c=(0,u.default)(s,"MM/dd/yyyy")):"datetime"===r&&(l=(0,u.default)(a,"MM/dd/yyyy - HH:mm"),c=(0,u.default)(s,"MM/dd/yyyy - HH:mm")),l===c}return!0})):t.filteredData="time"===r?t.filteredData.filter((function(n){var r=t.getFieldValue(n,e)||null;if(r){var o=i.filterValue;return r===(0,u.default)(o,"HH:mm")}return!0})):t.filteredData.filter((function(n){var r=t.getFieldValue(n,e);return r&&r.toString().toUpperCase().includes(i.filterValue.toUpperCase())}))})),t.filtered=!0})),(0,l.default)(this,"searchData",(function(){t.grouped=t.treefied=t.sorted=t.paged=!1,t.searchedData=(0,o.default)(t.filteredData),t.searchText&&t.applySearch&&(t.searchedData=t.searchedData.filter((function(e){return t.columns.filter((function(e){return void 0===e.searchable?!e.hidden:e.searchable})).some((function(n){if(n.customFilterAndSearch)return!!n.customFilterAndSearch(t.searchText,e,n);if(n.field){var r=t.getFieldValue(e,n);if(r)return r.toString().toUpperCase().includes(t.searchText.toUpperCase())}}))}))),t.searched=!0}))}return(0,s.default)(e,[{key:"setData",value:function(e){var t=this;this.selectedCount=0,this.data=e.map((function(e,n){return e.tableData=(0,i.default)({},e.tableData,{id:n}),e.tableData.checked&&t.selectedCount++,e})),this.filtered=!1}},{key:"setColumns",value:function(e){var t=e.filter((function(e){return void 0===e.width&&!e.hidden})),n=["0px"];this.columns=e.map((function(e,t){return e.tableData=(0,i.default)({columnOrder:t,filterValue:e.defaultFilter,groupOrder:e.defaultGroupOrder,groupSort:e.defaultGroupSort||"asc",width:e.width},e.tableData,{id:t}),void 0!==e.width&&("number"==typeof e.width?n.push(e.width+"px"):n.push(e.width)),e})),n="("+n.join(" + ")+")",t.forEach((function(e){e.tableData.width="calc((100% - ".concat(n,") / ").concat(t.length,")")}))}},{key:"setDefaultExpanded",value:function(e){this.defaultExpanded=e}},{key:"changeApplySearch",value:function(e){this.applySearch=e,this.searched=!1}},{key:"changeApplyFilters",value:function(e){this.applyFilters=e,this.filtered=!1}},{key:"changeApplySort",value:function(e){this.applySort=e,this.sorted=!1}},{key:"changePaging",value:function(e){this.paging=e,this.paged=!1}},{key:"changeCurrentPage",value:function(e){this.currentPage=e,this.paged=!1}},{key:"changePageSize",value:function(e){this.pageSize=e,this.paged=!1}},{key:"changeParentFunc",value:function(e){this.parentFunc=e}},{key:"changeFilterValue",value:function(e,t){this.columns[e].tableData.filterValue=t,this.filtered=!1}},{key:"changeRowSelected",value:function(e,t){var n=this,r=this.findDataByPath(this.sortedData,t);r.tableData.checked=e,this.selectedCount=this.selectedCount+(e?1:-1);!function t(r){r.tableData.childRows&&r.tableData.childRows.forEach((function(r){r.tableData.checked!==e&&(r.tableData.checked=e,n.selectedCount=n.selectedCount+(e?1:-1)),t(r)}))}(r),this.filtered=!1}},{key:"changeDetailPanelVisibility",value:function(e,t){var n=this.findDataByPath(this.sortedData,e);(n.tableData.showDetailPanel||"").toString()===t.toString()?n.tableData.showDetailPanel=void 0:n.tableData.showDetailPanel=t,"single"===this.detailPanelType&&this.lastDetailPanelRow&&this.lastDetailPanelRow!=n&&(this.lastDetailPanelRow.tableData.showDetailPanel=void 0),this.lastDetailPanelRow=n}},{key:"changeGroupExpand",value:function(e){var t=this.findDataByPath(this.sortedData,e);t.isExpanded=!t.isExpanded}},{key:"changeSearchText",value:function(e){this.searchText=e,this.searched=!1,this.currentPage=0}},{key:"changeRowEditing",value:function(e,t){e?(e.tableData.editing=t,this.lastEditingRow&&this.lastEditingRow!=e&&(this.lastEditingRow.tableData.editing=void 0),this.lastEditingRow=t?e:void 0):this.lastEditingRow&&(this.lastEditingRow.tableData.editing=void 0,this.lastEditingRow=void 0)}},{key:"changeAllSelected",value:function(e){var t=0;if(this.isDataType("group")){!function n(r){r.forEach((function(r){r.groups.length>0?n(r.groups):r.data.forEach((function(n){n.tableData.checked=!n.tableData.disabled&&e,t++}))}))}(this.groupedData)}else this.searchedData.map((function(t){return t.tableData.checked=!t.tableData.disabled&&e,t})),t=this.searchedData.length;this.selectedCount=e?t:0}},{key:"changeOrder",value:function(e,t){this.orderBy=e,this.orderDirection=t,this.currentPage=0,this.sorted=!1}},{key:"changeGroupOrder",value:function(e){var t=this.columns.find((function(t){return t.tableData.id===e}));"asc"===t.tableData.groupSort?t.tableData.groupSort="desc":t.tableData.groupSort="asc",this.sorted=!1}},{key:"changeColumnHidden",value:function(e,t){e.hidden=t}},{key:"changeTreeExpand",value:function(e){var t=this.findDataByPath(this.sortedData,e);t.tableData.isTreeExpanded=!t.tableData.isTreeExpanded}},{key:"changeDetailPanelType",value:function(e){this.detailPanelType=e}},{key:"changeByDrag",value:function(e){var t=0,n=this.columns.filter((function(e){return e.tableData.groupOrder>-1})).sort((function(e,t){return e.tableData.groupOrder-t.tableData.groupOrder}));if("groups"===e.destination.droppableId&&"groups"===e.source.droppableId){t=Math.min(e.destination.index,e.source.index);var r=Math.max(e.destination.index,e.source.index);if(n=n.slice(t,r+1),e.destination.index<e.source.index){var i=n.pop();n.unshift(i)}else{var o=n.shift();n.push(o)}}else if("groups"===e.destination.droppableId&&"headers"===e.source.droppableId){var a=this.columns.find((function(t){return t.tableData.id==e.draggableId}));if(!1===a.grouping||!a.field)return;n.splice(e.destination.index,0,a)}else{if("headers"!==e.destination.droppableId||"groups"!==e.source.droppableId){if("headers"===e.destination.droppableId&&"headers"===e.source.droppableId){t=Math.min(e.destination.index,e.source.index);for(var s=Math.max(e.destination.index,e.source.index),l=this.columns.sort((function(e,t){return e.tableData.columnOrder-t.tableData.columnOrder})).filter((function(e){return void 0===e.tableData.groupOrder})),u=0,c=0,d=0;d<l.length&&c<=t;d++)l[d].hidden?u++:c++;for(var h=t+u,f=h,p=0;p<s-t&&f<l.length;f++)l[f].hidden||p++;var g=l.slice(h,f+1);if(e.destination.index<e.source.index){var v=g.pop();g.unshift(v)}else{var m=g.shift();g.push(m)}for(var y=0;y<g.length;y++)g[y].tableData.columnOrder=h+y;return}return}this.columns.find((function(t){return t.tableData.id==e.draggableId})).tableData.groupOrder=void 0,n.splice(e.source.index,1)}for(var b=0;b<n.length;b++)n[b].tableData.groupOrder=t+b;this.sorted=this.grouped=!1}},{key:"findGroupByGroupPath",value:function(e,t){var n={groups:e,groupsIndex:this.rootGroupsIndex};return t.reduce((function(e,t){if(e)return void 0!==e.groupsIndex[t]?e.groups[e.groupsIndex[t]]:void 0}),n)}},{key:"isDataType",value:function(e){var t="normal";return this.parentFunc?t="tree":this.columns.find((function(e){return e.tableData.groupOrder>-1}))&&(t="group"),e===t}},{key:"sort",value:function(e,t,n){if("numeric"===n)return e-t;if(e!==t){if(!e)return-1;if(!t)return 1}return e<t?-1:e>t?1:0}},{key:"sortList",value:function(e){var t=this,n=this.columns.find((function(e){return e.tableData.id===t.orderBy}));return n.customSort?"desc"===this.orderDirection?e.sort((function(e,t){return n.customSort(t,e,"row")})):e.sort((function(e,t){return n.customSort(e,t,"row")})):e.sort("desc"===this.orderDirection?function(e,r){return t.sort(t.getFieldValue(r,n),t.getFieldValue(e,n),n.type)}:function(e,r){return t.sort(t.getFieldValue(e,n),t.getFieldValue(r,n),n.type)})}},{key:"groupData",value:function(){var e=this;this.sorted=this.paged=!1,this.groupedDataLength=0;var t=(0,o.default)(this.searchedData),n=this.columns.filter((function(e){return e.tableData.groupOrder>-1})).sort((function(e,t){return e.tableData.groupOrder-t.tableData.groupOrder})),r=t.reduce((function(t,r){var i=t;return(i=n.reduce((function(t,n){var i,a=r[n.field]||(0,c.byString)(r,n.field);if(void 0!==t.groupsIndex[a]&&(i=t.groups[t.groupsIndex[a]]),!i){var s=[].concat((0,o.default)(t.path||[]),[a]);i={value:a,groups:[],groupsIndex:{},data:[],isExpanded:(e.findGroupByGroupPath(e.groupedData,s)||{isExpanded:"boolean"==typeof e.defaultExpanded&&e.defaultExpanded}).isExpanded,path:s},t.groups.push(i),t.groupsIndex[a]=t.groups.length-1}return i}),i)).data.push(r),e.groupedDataLength++,t}),{groups:[],groupsIndex:{}});this.groupedData=r.groups,this.grouped=!0,this.rootGroupsIndex=r.groupsIndex}},{key:"treefyData",value:function(){var e=this;this.sorted=this.paged=!1,this.data.forEach((function(e){return e.tableData.childRows=null})),this.treefiedData=[],this.treefiedDataLength=0,this.treeDataMaxLevel=0,(this.searchText||this.columns.some((function(e){return e.tableData.filterValue})))&&(this.data.forEach((function(e){e.tableData.isTreeExpanded=!1})),this.expandTreeForNodes(this.searchedData));this.data.forEach((function(t){!function t(n){n.tableData.markedForTreeRemove=!1;var r=e.parentFunc(n,e.data);r?(r.tableData.childRows=r.tableData.childRows||[],r.tableData.childRows.includes(n)||(r.tableData.childRows.push(n),e.treefiedDataLength++),t(r),n.tableData.path=[].concat((0,o.default)(r.tableData.path),[r.tableData.childRows.length-1]),e.treeDataMaxLevel=Math.max(e.treeDataMaxLevel,n.tableData.path.length)):e.treefiedData.includes(n)||(e.treefiedData.push(n),e.treefiedDataLength++,n.tableData.path=[e.treefiedData.length-1])}(t)}));this.data.forEach((function(t){if(!e.searchText&&!e.columns.some((function(e){return e.tableData.filterValue}))&&void 0===t.tableData.isTreeExpanded){var n="boolean"==typeof e.defaultExpanded?e.defaultExpanded:e.defaultExpanded(t);t.tableData.isTreeExpanded=n}!t.tableData.isTreeExpanded&&e.searchedData.indexOf(t)<0&&function(t){var n=e.treefiedData;t.tableData.path.forEach((function(e){n.tableData&&n.tableData.childRows&&(n=n.tableData.childRows),n=n[e]})),n.tableData.markedForTreeRemove=!0}(t)})),this.data.forEach((function(t){e.searchedData.indexOf(t)>-1&&function e(t){t.tableData.childRows&&t.tableData.childRows.forEach((function(t){e(t)})),t.tableData.markedForTreeRemove=!1}(t)}));!function e(t){for(var n=t.length-1;n>=0;n--){var r=t[n];r.tableData.childRows&&e(r.tableData.childRows),r.tableData.markedForTreeRemove&&t.splice(n,1)}}(this.treefiedData),this.treefied=!0}},{key:"sortData",value:function(){var e=this;if(this.paged=!1,this.isDataType("group")){this.sortedData=(0,o.default)(this.groupedData);var t=this.columns.filter((function(e){return e.tableData.groupOrder>-1})).sort((function(e,t){return e.tableData.groupOrder-t.tableData.groupOrder})),n=function(t,n){return n.customSort?t.sort("desc"===n.tableData.groupSort?function(e,t){return n.customSort(t.value,e.value,"group")}:function(e,t){return n.customSort(e.value,t.value,"group")}):t.sort("desc"===n.tableData.groupSort?function(t,r){return e.sort(r.value,t.value,n.type)}:function(t,r){return e.sort(t.value,r.value,n.type)})};this.sortedData=n(this.sortedData,t[0]);!function r(i,o){i.forEach((function(i){if(i.groups.length>0){var a=t[o];i.groups=n(i.groups,a),r(i.groups,o+1)}else e.orderBy>=0&&e.orderDirection&&(i.data=e.sortList(i.data))}))}(this.sortedData,1)}else if(this.isDataType("tree")){if(this.sortedData=(0,o.default)(this.treefiedData),-1!=this.orderBy){this.sortedData=this.sortList(this.sortedData);!function t(n){n.forEach((function(n){n.tableData.childRows&&(n.tableData.childRows=e.sortList(n.tableData.childRows),t(n.tableData.childRows))}))}(this.sortedData)}}else this.isDataType("normal")&&(this.sortedData=(0,o.default)(this.searchedData),-1!=this.orderBy&&this.applySort&&(this.sortedData=this.sortList(this.sortedData)));this.sorted=!0}},{key:"pageData",value:function(){if(this.pagedData=(0,o.default)(this.sortedData),this.paging){var e=this.currentPage*this.pageSize,t=e+this.pageSize;this.pagedData=this.pagedData.slice(e,t)}this.paged=!0}}]),e}();t.default=d},function(e,t){function n(e,t,n){var r,i,o,a,s;function l(){var u=Date.now()-a;u<t&&u>=0?r=setTimeout(l,t-u):(r=null,n||(s=e.apply(o,i),o=i=null))}null==t&&(t=100);var u=function(){o=this,i=arguments,a=Date.now();var u=n&&!r;return r||(r=setTimeout(l,t)),u&&(s=e.apply(o,i),o=i=null),s};return u.clear=function(){r&&(clearTimeout(r),r=null)},u.flush=function(){r&&(s=e.apply(o,i),o=i=null,clearTimeout(r),r=null)},u}n.debounce=n,e.exports=n},function(e,t,n){"use strict";var r=Array.isArray,i=Object.keys,o=Object.prototype.hasOwnProperty;e.exports=function e(t,n){if(t===n)return!0;if(t&&n&&"object"==typeof t&&"object"==typeof n){var a,s,l,u=r(t),c=r(n);if(u&&c){if((s=t.length)!=n.length)return!1;for(a=s;0!=a--;)if(!e(t[a],n[a]))return!1;return!0}if(u!=c)return!1;var d=t instanceof Date,h=n instanceof Date;if(d!=h)return!1;if(d&&h)return t.getTime()==n.getTime();var f=t instanceof RegExp,p=n instanceof RegExp;if(f!=p)return!1;if(f&&p)return t.toString()==n.toString();var g=i(t);if((s=g.length)!==i(n).length)return!1;for(a=s;0!=a--;)if(!o.call(n,g[a]))return!1;for(a=s;0!=a--;)if(!e(t[l=g[a]],n[l]))return!1;return!0}return t!=t&&n!=n}},function(e,t,n){var r,i,o=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),a=this&&this.__setModuleDefault||(Object.create?function(e,t){Object.defineProperty(e,"default",{enumerable:!0,value:t})}:function(e,t){e.default=t}),s=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)"default"!==n&&Object.hasOwnProperty.call(e,n)&&o(t,e,n);return a(t,e),t},l=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};r=[n,t,n(90),n(0),n(78),n(176),n(2307)],void 0===(i=function(e,t,n,r,i,o,a){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),r=s(r),o=s(o),a=l(a);class u extends r.Component{constructor(e){super(e),this.on_msg=(e,t)=>{const{type:n,payload:r}=e;switch(n){case"GeometryView::geo_data":{const e=r.threejs_data,n=r.binary_position,i=r.time_step;for(const r in n){const i=n[r],o=(i[1]-i[0]+1)/2;for(let n=0;n<o;n++){const o=new Float64Array(t[i[0]+2*n].buffer),a=new Uint16Array(t[i[0]+2*n+1].buffer),s=new Float32Array(o.length);s.set(o),e[r][n].vertices=s,e[r][n].faces=Array.from(a)}}this.buffer_data[i]=e,"remaining"in r&&0==r.remaining&&this.computedUpdate();break}case"GeometryView::update_signal":this.computedUpdate()}},this.componentDidMount=()=>{this.props.send_msg({action:"GeometryView::requestInitialGeometry"})},this.componentWillUnmount=()=>{this.props.model.stopListening(this.props.model,"msg:custom",this.on_msg)},this.updateScene=()=>{},this.computedUpdate=()=>{const e=[];for(let t=0;t<Object.keys(this.buffer_data).length;t++)e.push("Time step "+t.toString());this.props.add3DData(e),this.setState(e=>Object.assign(Object.assign({},e),{threeData:Object.assign({},this.buffer_data)}),()=>{this.updateRender(),this.time_step=0,this.buffer_data={}})},this.updateRender=()=>{this.setState(e=>Object.assign(Object.assign({},e),{updateSignal:e.updateSignal+1}))},e.model.listenTo(e.model,"msg:custom",this.on_msg);const t={},n=[];for(let e=0;e<Object.keys(t).length;e++)n.push("Time step "+e.toString());this.props.add3DData(n),this.state={threeData:t,updateSignal:0},this.divRef=r.default.createRef(),this.time_step=0,this.buffer_data={}}render(){return r.default.createElement(a.default,{model:this.props.model,send_msg:this.props.send_msg,displayStatus:!0,threeData:this.state.threeData,updateSignal:this.state.updateSignal})}}t.default=n.withStyles(()=>({root:{height:"100%",flexGrow:1},paper:{background:"aliceblue",height:"100%"}}))(i.connect(e=>({}),e=>({add3DData:t=>e(o.dashboardAdd3DData(t))}))(u))}.apply(t,r))||(e.exports=i)},function(e,t,n){var r,i,o=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),a=this&&this.__setModuleDefault||(Object.create?function(e,t){Object.defineProperty(e,"default",{enumerable:!0,value:t})}:function(e,t){e.default=t}),s=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)"default"!==n&&Object.hasOwnProperty.call(e,n)&&o(t,e,n);return a(t,e),t},l=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};r=[n,t,n(90),n(75),n(103),n(120),n(132),n(121),n(2308),n(2309),n(0),n(78),n(20),n(2310),n(2347),n(638),n(332),n(248),n(176)],void 0===(i=function(e,t,n,r,i,o,a,u,c,d,h,f,p,g,v,m,y,b,C){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.Plot3DView=void 0,r=l(r),i=l(i),o=l(o),a=l(a),u=l(u),c=l(c),d=l(d),h=s(h),p=s(p),C=s(C);class A extends h.Component{constructor(e){super(e),this.divRef=h.default.createRef(),this.requestID=null,this.generateScene=()=>{this.sceneSetup(),this.startAnimationLoop(),this.resizeCanvasToDisplaySize()},this.resizeCanvasToDisplaySize=()=>{null!==this.divRef.current&&(this.renderer.setSize(this.divRef.current.clientWidth,this.divRef.current.clientHeight,!1),this.camera.aspect=this.divRef.current.clientWidth/this.divRef.current.clientHeight,this.camera.updateProjectionMatrix())},this.handleWindowResize=()=>{clearTimeout(this.resizeTimeout),this.resizeTimeout=setTimeout(()=>{this.forceUpdate()},500)},this.sceneSetup=()=>{if(null!==this.divRef.current){this.camera=new p.PerspectiveCamera(90,2,.1,1e3),this.camera.position.set(8,8,8),this.camera.up.set(0,0,1),this.scene=new p.Scene;const e=40,t=40;this.gridHelper=new p.GridHelper(e,t,8947848,8947848),this.gridHelper.geometry.rotateX(Math.PI/2),this.scene.add(this.gridHelper),this.addSceneAxe(new p.Vector3(1,0,0),65280),this.addSceneAxe(new p.Vector3(0,1,0),16711680),this.addSceneAxe(new p.Vector3(0,0,1),16776960);const n=[];n[0]=new p.AmbientLight(4210752),n[1]=new p.PointLight(16777215,1,0),this.scene.add(n[0]),this.camera.add(n[1]),this.scene.add(this.camera),this.renderer=new p.WebGLRenderer({alpha:!0,antialias:!1}),this.renderer.setPixelRatio(window.devicePixelRatio),this.renderer.setClearColor(0,0),this.renderer.setSize(500,500,!1),this.divRef.current.appendChild(this.renderer.domElement);const r=new g.OrbitControls(this.camera,this.renderer.domElement);r.rotateSpeed=1,r.zoomSpeed=1.2,r.panSpeed=.8,r.target.set(this.scene.position.x,this.scene.position.y,this.scene.position.z),this.controls=r}},this.addSceneAxe=(e,t)=>{const n=new p.Vector3(0,0,0),r=new p.ArrowHelper(e,n,20,t,.4,.2);this.scene.add(r);const i=[n.x,n.y,n.z,20*e.x,20*e.y,20*e.z],o=new p.Color(t),a=[o.r,o.g,o.b,o.r,o.g,o.b],s=new m.LineGeometry;s.setPositions(i),s.setColors(a);const l=new y.LineMaterial({linewidth:1.5,vertexColors:!0});l.resolution.set(800,600);const u=new v.Line2(s,l);this.sceneAxe.push(r,u),this.scene.add(u)},this.updateGeo=(e,t,n=!1)=>{if(this.computedScene.hasOwnProperty(e)&&!n){for(const t in this.computedScene[e])if(this.computedScene[e].hasOwnProperty(t)){const n=this.computedScene[e][t];this.shapeGroup.add(n)}this.scene.add(this.shapeGroup)}else{this.computedScene[e]={};for(const n in t)if(t.hasOwnProperty(n)){const{facesGroup:r,newRefLength:i}=A.dataProcessing(n,t[n]);this.refLength=Math.max(i,this.refLength),this.computedScene[e][n]=r,this.shapeGroup.add(r)}this.scene.add(this.shapeGroup)}},this.updateCamera=e=>{let t=0;t=0==this.refLength?8:2*this.refLength,0==e?this.camera.position.set(t,t,t):10==e?this.camera.position.set(t,0,0):20==e?this.camera.position.set(0,t,0):30==e&&this.camera.position.set(0,0,t)},this.updateUpDirection=e=>{window.cancelAnimationFrame(this.requestID),this.controls.dispose(),10==e?this.camera.up.set(1,0,0):20==e?this.camera.up.set(0,1,0):30==e&&this.camera.up.set(0,0,1),this.controls=new g.OrbitControls(this.camera,this.renderer.domElement),this.controls.rotateSpeed=1,this.controls.zoomSpeed=1.2,this.controls.panSpeed=.8,this.controls.target.set(this.scene.position.x,this.scene.position.y,this.scene.position.z),this.startAnimationLoop()},this.startAnimationLoop=()=>{this.requestID=window.requestAnimationFrame(this.startAnimationLoop),this.controls.update(),this.renderer.clearDepth(),this.renderer.render(this.scene,this.camera)},this.cleanScene=()=>{this.scene.remove(this.shapeGroup),this.shapeGroup=new p.Group},this.getDataHandle=(e,t=!1)=>{let n;if(this.cleanScene(),this.props.threeData.hasOwnProperty(e)){if(n=this.props.threeData[e],this.updateGeo(e,n,t),!this.sceneScaled){this.camera.lookAt(this.scene.position),this.camera.position.set(2*this.refLength,2*this.refLength,2*this.refLength),this.camera.far=40*this.refLength,this.gridHelper.scale.multiplyScalar(this.refLength/5);for(let e=0;e<this.sceneAxe.length;e++)this.sceneAxe[e].scale.multiplyScalar(this.refLength/5);this.sceneScaled=!0}this.renderer.render(this.scene,this.camera)}},this.handleNextBtn=()=>{let e;this.setState(t=>(e=t.selectedTime+1,e<this.props.timeStepList.length?Object.assign(Object.assign({},t),{selectedTime:e}):e===this.props.timeStepList.length&&1!==e?Object.assign(Object.assign({},t),{selectedTime:1}):t),()=>{0!==this.state.selectedTime&&this.getDataHandle(this.state.selectedTime-1)})},this.handlePreviousBtn=()=>{let e;this.setState(t=>(e=t.selectedTime-1,e>0?Object.assign(Object.assign({},t),{selectedTime:e}):0===e?Object.assign(Object.assign({},t),{selectedTime:this.props.timeStepList.length-1}):t),()=>{0!==this.state.selectedTime&&this.getDataHandle(this.state.selectedTime-1)})},this.handlePlayBtn=()=>{1!==this.props.timeStepList.length&&(this.state.playAnimation?(clearInterval(this.animationInverval),this.setState(Object.assign(Object.assign({},this.state),{playAnimation:!1}))):this.animationInverval=setInterval(()=>{let e;this.setState(t=>(e=t.selectedTime+1,e<this.props.timeStepList.length?Object.assign(Object.assign({},t),{selectedTime:e,playAnimation:!0}):e===this.props.timeStepList.length&&1!==e?(e=1,Object.assign(Object.assign({},t),{selectedTime:1,playAnimation:!0})):(e=t.selectedTime,Object.assign(Object.assign({},t),{playAnimation:!0})))),0!==e&&this.getDataHandle(e-1)},40))},this.geometry=new p.BufferGeometry,this.geometry.setDrawRange(0,3e4),this.refLength=0,this.sceneAxe=[],this.shapeGroup=new p.Group,this.sceneScaled=!1,this.computedScene={},this.progressData={time_step:-1,data:{}},this.state={selectedView:0,selectedUpDir:30,bgColor:"radial-gradient(#efeded, #8f9091)",selectedTime:0,playAnimation:!1},this.resizeTimeout=null}componentDidMount(){this.generateScene(),window.addEventListener("resize",this.handleWindowResize),this.props.timeStepList.length>1&&(this.getDataHandle(0),this.setState(Object.assign(Object.assign({},this.state),{selectedTime:1})))}componentDidUpdate(e,t){this.props.displayStatus&&this.resizeCanvasToDisplaySize(),this.state.selectedView!==t.selectedView&&this.updateCamera(this.state.selectedView),this.state.selectedUpDir!==t.selectedUpDir&&this.updateUpDirection(this.state.selectedUpDir),this.props.updateSignal!==e.updateSignal&&(this.computedScene={},0!==this.state.selectedTime&&this.getDataHandle(this.state.selectedTime-1,!0)),this.props.timeStepList.length>1&&1==e.timeStepList.length&&(this.getDataHandle(0),this.setState(Object.assign(Object.assign({},this.state),{selectedTime:1})))}componentWillUnmount(){window.removeEventListener("resize",this.handleWindowResize),window.cancelAnimationFrame(this.requestID),this.controls.dispose()}render(){const{classes:e}=this.props;return h.default.createElement("div",{className:"cosapp-widget-box"},h.default.createElement("div",{ref:this.divRef,style:{width:"100%",height:"calc(100% - 36px)",background:this.state.bgColor}}),h.default.createElement(u.default,{variant:"dense",classes:{dense:e.toolbarHeigt}},h.default.createElement(i.default,{style:{marginRight:10}},h.default.createElement(a.default,{value:this.state.selectedView,classes:{icon:e.textColor,select:e.viewSelector},onChange:e=>{this.setState(Object.assign(Object.assign({},this.state),{selectedView:e.target.value}))}},h.default.createElement(o.default,{value:0},"3D"),h.default.createElement(o.default,{value:10},"Orientation X"),h.default.createElement(o.default,{value:20},"Orientation Y"),h.default.createElement(o.default,{value:30},"Orientation Z"))),h.default.createElement(i.default,{style:{marginRight:10}},h.default.createElement(a.default,{value:this.state.selectedUpDir,classes:{icon:e.textColor,select:e.viewSelector},onChange:e=>{this.setState(Object.assign(Object.assign({},this.state),{selectedUpDir:e.target.value}))}},h.default.createElement(o.default,{value:10},"X Direction"),h.default.createElement(o.default,{value:20},"Y Direction"),h.default.createElement(o.default,{value:30},"Z Direction"))),h.default.createElement(i.default,{style:{marginRight:10}},h.default.createElement(a.default,{value:this.state.selectedTime,classes:{icon:e.textColor,select:e.viewSelector},onChange:e=>{const t=e.target.value;0!==t&&this.getDataHandle(t-1),this.setState(Object.assign(Object.assign({},this.state),{selectedTime:t}))}},this.props.timeStepList.map((e,t)=>h.default.createElement(o.default,{key:t,value:t},e)))),h.default.createElement(r.default,{onClick:this.handlePlayBtn,classes:{root:e.textColor}},this.state.playAnimation?h.default.createElement(c.default,null):h.default.createElement(d.default,null))))}}t.Plot3DView=A,A.dataProcessing=(e,t)=>{const n=new p.Group;let r=0;if(e.includes("edge"))for(let e=0;e<t.length;e++){const r=t[e],i=new p.LineBasicMaterial({linewidth:10,color:"black"}),o=new p.BufferGeometry,a=new Float32Array(r.vertices.length);for(let e=0;e<r.vertices.length;e++)a[e]=r.vertices[e];o.setAttribute("position",new p.BufferAttribute(a,3,!1)),o.setIndex(r.faces);const s=new p.Line(o,i);s.quaternion.set(r.quat[0],r.quat[1],r.quat[2],r.quat[3]),s.position.set(r.pos[0],r.pos[1],r.pos[2]),n.add(s)}else if(e.includes("misc"))for(const e in t){const i=t[e];"points"===e?i.forEach(e=>{const t=e.position;let i,o;i=e.hasOwnProperty("radius")?e.radius:.1,o=e.hasOwnProperty("color")?e.color:"yellow";const a=new p.SphereBufferGeometry(i),s=new p.MeshBasicMaterial({color:o}),l=new p.Mesh(a,s);l.position.set(t[0],t[1],t[2]),n.add(l);const u=1.5*i;r=Math.max(r,u)}):"vectors"===e&&i.forEach(e=>{let t;t=e.hasOwnProperty("color")?b.colorNameToHex(e.color):3735794;const i=new p.Vector3(e.position[0],e.position[1],e.position[2]),o=new p.Vector3(e.direction[0],e.direction[1],e.direction[2]),a=o.length();o.normalize();const s=new p.ArrowHelper(o,i,a,t,.4,.2);n.add(s);const l=1.5*a;r=Math.max(r,l)})}else for(let e=0;e<t.length;e++){const i=t[e];let o;o=i.transparent?new p.MeshPhongMaterial({color:i.color,side:p.DoubleSide,transparent:i.transparent,opacity:.33,wireframe:!1,flatShading:!0}):new p.MeshPhongMaterial({color:i.color,side:p.DoubleSide,wireframe:!1,flatShading:!0});const a=new p.BufferGeometry;a.setAttribute("position",new p.BufferAttribute(i.vertices,3,!1)),a.setIndex(i.faces),a.computeVertexNormals(),a.computeBoundingBox(),a.computeBoundingSphere();const s=a.boundingBox,l=new p.Vector3;s.getSize(l);const u=Math.max(l.x,l.y,l.z);r=Math.max(r,u);const c=new p.Mesh(a,o);c.quaternion.set(i.quat[0],i.quat[1],i.quat[2],i.quat[3]),c.position.set(i.pos[0],i.pos[1],i.pos[2]),n.add(c)}return{facesGroup:n,newRefLength:r}},t.default=n.withStyles(()=>({textSizeSmall:{},toolbarHeigt:{minHeight:36,background:"rgb(224 224 224)"},viewSelector:{minWidth:120,color:"rgb(50, 50, 50)"},bgSelector:{minWidth:60,color:"rgb(50, 50, 50)"},textColor:{color:"rgb(50, 50, 50)"},backGround:{color:"rgb(50, 50, 50)"}}))(f.connect(e=>(e=>({timeStepList:e.dashboardState.timeStepList}))(e),e=>({add3DData:t=>e(C.dashboardAdd3DData(t))}))(A))}.apply(t,r))||(e.exports=i)},function(e,t,n){"use strict";var r=n(36),i=n(46);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(0)),a=(0,r(n(66)).default)(o.createElement("path",{d:"M6 19h4V5H6v14zm8-14v14h4V5h-4z"}),"Pause");t.default=a},function(e,t,n){"use strict";var r=n(36),i=n(46);Object.defineProperty(t,"__esModule",{value:!0}),t.default=void 0;var o=i(n(0)),a=(0,r(n(66)).default)(o.createElement("path",{d:"M8 5v14l11-7z"}),"PlayArrow");t.default=a},function(e,t,n){"use strict";n.r(t),n.d(t,"OrbitControls",(function(){return i})),n.d(t,"MapControls",(function(){return o}));var r=n(20),i=function(e,t){var n,i,o,a,s;void 0===t&&console.warn('THREE.OrbitControls: The second parameter "domElement" is now mandatory.'),t===document&&console.error('THREE.OrbitControls: "document" should not be used as the target "domElement". Please use "renderer.domElement" instead.'),this.object=e,this.domElement=t,this.enabled=!0,this.target=new r.Vector3,this.minDistance=0,this.maxDistance=1/0,this.minZoom=0,this.maxZoom=1/0,this.minPolarAngle=0,this.maxPolarAngle=Math.PI,this.minAzimuthAngle=-1/0,this.maxAzimuthAngle=1/0,this.enableDamping=!1,this.dampingFactor=.05,this.enableZoom=!0,this.zoomSpeed=1,this.enableRotate=!0,this.rotateSpeed=1,this.enablePan=!0,this.panSpeed=1,this.screenSpacePanning=!1,this.keyPanSpeed=7,this.autoRotate=!1,this.autoRotateSpeed=2,this.enableKeys=!0,this.keys={LEFT:37,UP:38,RIGHT:39,BOTTOM:40},this.mouseButtons={LEFT:r.MOUSE.ROTATE,MIDDLE:r.MOUSE.DOLLY,RIGHT:r.MOUSE.PAN},this.touches={ONE:r.TOUCH.ROTATE,TWO:r.TOUCH.DOLLY_PAN},this.target0=this.target.clone(),this.position0=this.object.position.clone(),this.zoom0=this.object.zoom,this.getPolarAngle=function(){return g.phi},this.getAzimuthalAngle=function(){return g.theta},this.saveState=function(){l.target0.copy(l.target),l.position0.copy(l.object.position),l.zoom0=l.object.zoom},this.reset=function(){l.target.copy(l.target0),l.object.position.copy(l.position0),l.object.zoom=l.zoom0,l.object.updateProjectionMatrix(),l.dispatchEvent(u),l.update(),f=h.NONE},this.update=(n=new r.Vector3,i=(new r.Quaternion).setFromUnitVectors(e.up,new r.Vector3(0,1,0)),o=i.clone().inverse(),a=new r.Vector3,s=new r.Quaternion,function(){var e=l.object.position;return n.copy(e).sub(l.target),n.applyQuaternion(i),g.setFromVector3(n),l.autoRotate&&f===h.NONE&&O(2*Math.PI/60/60*l.autoRotateSpeed),l.enableDamping?(g.theta+=v.theta*l.dampingFactor,g.phi+=v.phi*l.dampingFactor):(g.theta+=v.theta,g.phi+=v.phi),g.theta=Math.max(l.minAzimuthAngle,Math.min(l.maxAzimuthAngle,g.theta)),g.phi=Math.max(l.minPolarAngle,Math.min(l.maxPolarAngle,g.phi)),g.makeSafe(),g.radius*=m,g.radius=Math.max(l.minDistance,Math.min(l.maxDistance,g.radius)),!0===l.enableDamping?l.target.addScaledVector(y,l.dampingFactor):l.target.add(y),n.setFromSpherical(g),n.applyQuaternion(o),e.copy(l.target).add(n),l.object.lookAt(l.target),!0===l.enableDamping?(v.theta*=1-l.dampingFactor,v.phi*=1-l.dampingFactor,y.multiplyScalar(1-l.dampingFactor)):(v.set(0,0,0),y.set(0,0,0)),m=1,!!(b||a.distanceToSquared(l.object.position)>p||8*(1-s.dot(l.object.quaternion))>p)&&(l.dispatchEvent(u),a.copy(l.object.position),s.copy(l.object.quaternion),b=!1,!0)}),this.dispose=function(){l.domElement.removeEventListener("contextmenu",ee,!1),l.domElement.removeEventListener("mousedown",G,!1),l.domElement.removeEventListener("wheel",Z,!1),l.domElement.removeEventListener("touchstart",K,!1),l.domElement.removeEventListener("touchend",$,!1),l.domElement.removeEventListener("touchmove",J,!1),document.removeEventListener("mousemove",X,!1),document.removeEventListener("mouseup",q,!1),l.domElement.removeEventListener("keydown",Q,!1)};var l=this,u={type:"change"},c={type:"start"},d={type:"end"},h={NONE:-1,ROTATE:0,DOLLY:1,PAN:2,TOUCH_ROTATE:3,TOUCH_PAN:4,TOUCH_DOLLY_PAN:5,TOUCH_DOLLY_ROTATE:6},f=h.NONE,p=1e-6,g=new r.Spherical,v=new r.Spherical,m=1,y=new r.Vector3,b=!1,C=new r.Vector2,A=new r.Vector2,_=new r.Vector2,I=new r.Vector2,x=new r.Vector2,w=new r.Vector2,M=new r.Vector2,S=new r.Vector2,T=new r.Vector2;function E(){return Math.pow(.95,l.zoomSpeed)}function O(e){v.theta-=e}function k(e){v.phi-=e}var L,D=(L=new r.Vector3,function(e,t){L.setFromMatrixColumn(t,0),L.multiplyScalar(-e),y.add(L)}),P=function(){var e=new r.Vector3;return function(t,n){!0===l.screenSpacePanning?e.setFromMatrixColumn(n,1):(e.setFromMatrixColumn(n,0),e.crossVectors(l.object.up,e)),e.multiplyScalar(t),y.add(e)}}(),N=function(){var e=new r.Vector3;return function(t,n){var r=l.domElement;if(l.object.isPerspectiveCamera){var i=l.object.position;e.copy(i).sub(l.target);var o=e.length();o*=Math.tan(l.object.fov/2*Math.PI/180),D(2*t*o/r.clientHeight,l.object.matrix),P(2*n*o/r.clientHeight,l.object.matrix)}else l.object.isOrthographicCamera?(D(t*(l.object.right-l.object.left)/l.object.zoom/r.clientWidth,l.object.matrix),P(n*(l.object.top-l.object.bottom)/l.object.zoom/r.clientHeight,l.object.matrix)):(console.warn("WARNING: OrbitControls.js encountered an unknown camera type - pan disabled."),l.enablePan=!1)}}();function j(e){l.object.isPerspectiveCamera?m/=e:l.object.isOrthographicCamera?(l.object.zoom=Math.max(l.minZoom,Math.min(l.maxZoom,l.object.zoom*e)),l.object.updateProjectionMatrix(),b=!0):(console.warn("WARNING: OrbitControls.js encountered an unknown camera type - dolly/zoom disabled."),l.enableZoom=!1)}function R(e){l.object.isPerspectiveCamera?m*=e:l.object.isOrthographicCamera?(l.object.zoom=Math.max(l.minZoom,Math.min(l.maxZoom,l.object.zoom/e)),l.object.updateProjectionMatrix(),b=!0):(console.warn("WARNING: OrbitControls.js encountered an unknown camera type - dolly/zoom disabled."),l.enableZoom=!1)}function z(e){C.set(e.clientX,e.clientY)}function F(e){I.set(e.clientX,e.clientY)}function B(e){if(1==e.touches.length)C.set(e.touches[0].pageX,e.touches[0].pageY);else{var t=.5*(e.touches[0].pageX+e.touches[1].pageX),n=.5*(e.touches[0].pageY+e.touches[1].pageY);C.set(t,n)}}function H(e){if(1==e.touches.length)I.set(e.touches[0].pageX,e.touches[0].pageY);else{var t=.5*(e.touches[0].pageX+e.touches[1].pageX),n=.5*(e.touches[0].pageY+e.touches[1].pageY);I.set(t,n)}}function U(e){var t=e.touches[0].pageX-e.touches[1].pageX,n=e.touches[0].pageY-e.touches[1].pageY,r=Math.sqrt(t*t+n*n);M.set(0,r)}function V(e){if(1==e.touches.length)A.set(e.touches[0].pageX,e.touches[0].pageY);else{var t=.5*(e.touches[0].pageX+e.touches[1].pageX),n=.5*(e.touches[0].pageY+e.touches[1].pageY);A.set(t,n)}_.subVectors(A,C).multiplyScalar(l.rotateSpeed);var r=l.domElement;O(2*Math.PI*_.x/r.clientHeight),k(2*Math.PI*_.y/r.clientHeight),C.copy(A)}function Y(e){if(1==e.touches.length)x.set(e.touches[0].pageX,e.touches[0].pageY);else{var t=.5*(e.touches[0].pageX+e.touches[1].pageX),n=.5*(e.touches[0].pageY+e.touches[1].pageY);x.set(t,n)}w.subVectors(x,I).multiplyScalar(l.panSpeed),N(w.x,w.y),I.copy(x)}function W(e){var t=e.touches[0].pageX-e.touches[1].pageX,n=e.touches[0].pageY-e.touches[1].pageY,r=Math.sqrt(t*t+n*n);S.set(0,r),T.set(0,Math.pow(S.y/M.y,l.zoomSpeed)),j(T.y),M.copy(S)}function G(e){if(!1!==l.enabled){var t;switch(e.preventDefault(),l.domElement.focus?l.domElement.focus():window.focus(),e.button){case 0:t=l.mouseButtons.LEFT;break;case 1:t=l.mouseButtons.MIDDLE;break;case 2:t=l.mouseButtons.RIGHT;break;default:t=-1}switch(t){case r.MOUSE.DOLLY:if(!1===l.enableZoom)return;!function(e){M.set(e.clientX,e.clientY)}(e),f=h.DOLLY;break;case r.MOUSE.ROTATE:if(e.ctrlKey||e.metaKey||e.shiftKey){if(!1===l.enablePan)return;F(e),f=h.PAN}else{if(!1===l.enableRotate)return;z(e),f=h.ROTATE}break;case r.MOUSE.PAN:if(e.ctrlKey||e.metaKey||e.shiftKey){if(!1===l.enableRotate)return;z(e),f=h.ROTATE}else{if(!1===l.enablePan)return;F(e),f=h.PAN}break;default:f=h.NONE}f!==h.NONE&&(document.addEventListener("mousemove",X,!1),document.addEventListener("mouseup",q,!1),l.dispatchEvent(c))}}function X(e){if(!1!==l.enabled)switch(e.preventDefault(),f){case h.ROTATE:if(!1===l.enableRotate)return;!function(e){A.set(e.clientX,e.clientY),_.subVectors(A,C).multiplyScalar(l.rotateSpeed);var t=l.domElement;O(2*Math.PI*_.x/t.clientHeight),k(2*Math.PI*_.y/t.clientHeight),C.copy(A),l.update()}(e);break;case h.DOLLY:if(!1===l.enableZoom)return;!function(e){S.set(e.clientX,e.clientY),T.subVectors(S,M),T.y>0?j(E()):T.y<0&&R(E()),M.copy(S),l.update()}(e);break;case h.PAN:if(!1===l.enablePan)return;!function(e){x.set(e.clientX,e.clientY),w.subVectors(x,I).multiplyScalar(l.panSpeed),N(w.x,w.y),I.copy(x),l.update()}(e)}}function q(e){!1!==l.enabled&&(document.removeEventListener("mousemove",X,!1),document.removeEventListener("mouseup",q,!1),l.dispatchEvent(d),f=h.NONE)}function Z(e){!1===l.enabled||!1===l.enableZoom||f!==h.NONE&&f!==h.ROTATE||(e.preventDefault(),e.stopPropagation(),l.dispatchEvent(c),function(e){e.deltaY<0?R(E()):e.deltaY>0&&j(E()),l.update()}(e),l.dispatchEvent(d))}function Q(e){!1!==l.enabled&&!1!==l.enableKeys&&!1!==l.enablePan&&function(e){var t=!1;switch(e.keyCode){case l.keys.UP:N(0,l.keyPanSpeed),t=!0;break;case l.keys.BOTTOM:N(0,-l.keyPanSpeed),t=!0;break;case l.keys.LEFT:N(l.keyPanSpeed,0),t=!0;break;case l.keys.RIGHT:N(-l.keyPanSpeed,0),t=!0}t&&(e.preventDefault(),l.update())}(e)}function K(e){if(!1!==l.enabled){switch(e.preventDefault(),e.touches.length){case 1:switch(l.touches.ONE){case r.TOUCH.ROTATE:if(!1===l.enableRotate)return;B(e),f=h.TOUCH_ROTATE;break;case r.TOUCH.PAN:if(!1===l.enablePan)return;H(e),f=h.TOUCH_PAN;break;default:f=h.NONE}break;case 2:switch(l.touches.TWO){case r.TOUCH.DOLLY_PAN:if(!1===l.enableZoom&&!1===l.enablePan)return;!function(e){l.enableZoom&&U(e),l.enablePan&&H(e)}(e),f=h.TOUCH_DOLLY_PAN;break;case r.TOUCH.DOLLY_ROTATE:if(!1===l.enableZoom&&!1===l.enableRotate)return;!function(e){l.enableZoom&&U(e),l.enableRotate&&B(e)}(e),f=h.TOUCH_DOLLY_ROTATE;break;default:f=h.NONE}break;default:f=h.NONE}f!==h.NONE&&l.dispatchEvent(c)}}function J(e){if(!1!==l.enabled)switch(e.preventDefault(),e.stopPropagation(),f){case h.TOUCH_ROTATE:if(!1===l.enableRotate)return;V(e),l.update();break;case h.TOUCH_PAN:if(!1===l.enablePan)return;Y(e),l.update();break;case h.TOUCH_DOLLY_PAN:if(!1===l.enableZoom&&!1===l.enablePan)return;!function(e){l.enableZoom&&W(e),l.enablePan&&Y(e)}(e),l.update();break;case h.TOUCH_DOLLY_ROTATE:if(!1===l.enableZoom&&!1===l.enableRotate)return;!function(e){l.enableZoom&&W(e),l.enableRotate&&V(e)}(e),l.update();break;default:f=h.NONE}}function $(e){!1!==l.enabled&&(l.dispatchEvent(d),f=h.NONE)}function ee(e){!1!==l.enabled&&e.preventDefault()}l.domElement.addEventListener("contextmenu",ee,!1),l.domElement.addEventListener("mousedown",G,!1),l.domElement.addEventListener("wheel",Z,!1),l.domElement.addEventListener("touchstart",K,!1),l.domElement.addEventListener("touchend",$,!1),l.domElement.addEventListener("touchmove",J,!1),l.domElement.addEventListener("keydown",Q,!1),-1===l.domElement.tabIndex&&(l.domElement.tabIndex=0),this.update()};i.prototype=Object.create(r.EventDispatcher.prototype),i.prototype.constructor=i;var o=function(e,t){i.call(this,e,t),this.mouseButtons.LEFT=r.MOUSE.PAN,this.mouseButtons.RIGHT=r.MOUSE.ROTATE,this.touches.ONE=r.TOUCH.PAN,this.touches.TWO=r.TOUCH.DOLLY_ROTATE};o.prototype=Object.create(r.EventDispatcher.prototype),o.prototype.constructor=o},function(e,t,n){var r,i,o=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),a=this&&this.__setModuleDefault||(Object.create?function(e,t){Object.defineProperty(e,"default",{enumerable:!0,value:t})}:function(e,t){e.default=t}),s=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)"default"!==n&&Object.hasOwnProperty.call(e,n)&&o(t,e,n);return a(t,e),t},l=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};r=[n,t,n(0),n(90),n(78),n(519),n(75),n(2312),n(103),n(120),n(132),n(248),n(2313)],void 0===(i=function(e,t,n,r,i,o,a,u,c,d,h,f){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.StructureElement=void 0,n=s(n),a=l(a),c=l(c),d=l(d),h=l(h);class p extends n.Component{constructor(e){let t;super(e),this.on_msg=(e,t)=>{const{type:n,payload:r}=e;switch(n){case"StructureView::structureData":{const e=JSON.parse(JSON.stringify({nodes:r.nodes,edges:r.edges})),{groups:t,title:n}=r,i=new u.DataSet(r.nodes),o=new u.DataSet(r.edges),a={nodes:i,edges:o};this._network=this.generateNetwork(a,this.state.options,!1),this.setState(n=>Object.assign(Object.assign({},n),{nodes:i,edges:o,systems:t,rawData:e}));break}case"StructureView::importError":this._divRef.current.innerHTML="Structure widget requires CoSApp >= 0.11.5"}},this.measure=e=>{try{this._network.fit({animation:!0})}catch(e){}},this.bestFit=()=>{const e=this._network,t=this.state.nodes;if(!t||!e)return;e.moveTo({scale:1}),e.stopSimulation();const n={top:1/0,left:1/0,right:-1/0,bottom:-1/0};t.getIds().forEach(t=>{const r=e.getBoundingBox(t);r.top<n.top&&(n.top=r.top),r.left<n.left&&(n.left=r.left),r.right>n.right&&(n.right=r.right),r.bottom>n.bottom&&(n.bottom=r.bottom)});const r=e.canvas.body.container.clientWidth,i=e.canvas.body.container.clientHeight,o=r/(n.right-n.left),a=i/(n.bottom-n.top);let s=o;s*(n.bottom-n.top)>i&&(s=a),s>1&&(s*=.9),e.moveTo({scale:s,position:{x:(n.right+n.left)/2,y:(n.bottom+n.top)/2}})},this.generateNetwork=(e,t,n)=>{this._network&&this._network.destroy();const r=document.getElementById(this.state.divID),i=new u.Network(r,e,t);return n||(i.on("doubleClick",e=>{1===e.nodes.length&&!1===i.isCluster(e.nodes[0])&&this.clusterFromSelection(e.nodes[0])}),i.on("selectNode",e=>{1===e.nodes.length&&!0===i.isCluster(e.nodes[0])&&i.openCluster(e.nodes[0])}),i.on("stabilized",e=>{i.fit({animation:!0})})),i},this.clusterSystem=(e,t,n,r)=>{const i={joinCondition:function(t){return t.group===e},processProperties:function(e,n,i){const o=r.get({filter:function(e){return e.label===t}});return e.title=o[0].title,e},clusterNodeProperties:{id:"cluster:"+e,borderWidth:3,shape:"database",label:t,group:n,allowSingleNodeCluster:!0}};this._network.cluster(i)},this.clusterBySystem=()=>{const e={nodes:this.state.nodes,edges:this.state.edges},t=this.state.options;this._network=this.generateNetwork(e,t,!1);for(let e=0;e<this.state.systems.length;e++){const t=this.state.systems[e],n=t.split("."),r=n.pop();0!==n.length&&this.clusterSystem(t,r,n.join("."),this.state.nodes)}},this.setHierarchicalLayout=()=>{const e=new u.DataView(this.state.nodes,{filter:e=>void 0!==e.label&&e.label.length>0,fields:["id","label","title","group","shape","level"]}),t=[];this.state.rawData.edges.forEach(e=>{if(void 0!==e.hidden&&e.hidden){let n=JSON.parse(JSON.stringify(e));delete n.hidden,t.push(n)}});const n={nodes:e,edges:new u.DataSet(t)};this._network=this.generateNetwork(n,{layout:{hierarchical:{enabled:!0}},physics:!1},!0)},this.clusterFromSelection=e=>{const t=this.state.nodes.get(e),n=this.state.systems,r=[];for(let e=n.length-1;e>=0;e--)(n[e].startsWith(t.group)||this._network.isCluster("cluster:"+n[e]))&&r.unshift(n[e]);const i={nodes:this.state.nodes,edges:this.state.edges};this._network.setData(i);for(let e=0;e<r.length;e++){let t=r[e],n=t.split("."),i=n.pop();this.clusterSystem(t,i,n.join("."),this.state.nodes)}},this.resetAllClusters=()=>{const e={nodes:this.state.nodes,edges:this.state.edges};this._network=this.generateNetwork(e,this.state.options,!1)},e.model.listenTo(e.model,"msg:custom",this.on_msg),this._network=null,e.initialState?(t=e.initialState.traceConfig,t.bgColor||(t.bgColor=0)):t={bgColor:0},this.state={divID:f.create_UUID(),network:null,traceConfig:t,rawData:{nodes:[],edges:[]},nodes:null,edges:null,systems:[],options:{layout:{hierarchical:!1,randomSeed:8},nodes:{shape:"circle"}}}}componentDidUpdate(e,t){}componentDidMount(){this.props.send_msg({action:"StructureView::getData"}),this._resizeHandler=o.debounce(this.measure,500),window.addEventListener("resize",this._resizeHandler)}componentWillUnmount(){this.props.model.stopListening(this.props.model,"msg:custom",this.on_msg),window.removeEventListener("resize",this._resizeHandler)}render(){return n.default.createElement("div",{className:"cosapp-widget-box"},n.default.createElement("div",{className:1===this.state.traceConfig.bgColor?"pbsView":null,id:this.state.divID,style:{height:"calc(100% - 30px)"}}),n.default.createElement("div",{style:{height:"30px",display:"flex",background:"#e0e0e0"}},n.default.createElement(a.default,{onClick:this.clusterBySystem},"Cluster view"),n.default.createElement(a.default,{onClick:this.setHierarchicalLayout},"Hierarchical view"),n.default.createElement(a.default,{onClick:()=>{this.props.send_msg({action:"StructureView::getData"})}},"Refresh"),n.default.createElement(c.default,{style:{marginRight:10,marginLeft:10}},n.default.createElement(h.default,{value:this.state.traceConfig.bgColor,onChange:e=>{this.setState(t=>Object.assign(Object.assign({},t),{traceConfig:Object.assign(Object.assign({},t.traceConfig),{bgColor:e.target.value})}))}},n.default.createElement(d.default,{value:1},"Dark"),n.default.createElement(d.default,{value:0},"Light")))))}}t.StructureElement=p,t.default=i.connect(e=>({}),e=>({}),null,{forwardRef:!0})(r.withStyles(e=>({textColor:{color:"rgb(250, 250, 250)",background:"#525354",margin:"0px 5px"}}))(n.forwardRef((e,t)=>n.default.createElement(p,Object.assign({},e,{ref:t})))))}.apply(t,r))||(e.exports=i)},function(e,t,n){"use strict";
/**
 * vis.js
 * https://github.com/almende/vis
 *
 * A dynamic, browser-based visualization library.
 *
 * @version 4.21.0
 * @date    2017-10-12
 *
 * @license
 * Copyright (C) 2011-2017 Almende B.V, http://almende.com
 *
 * Vis.js is dual licensed under both
 *
 * * The Apache 2.0 License
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * and
 *
 * * The MIT License
 *   http://opensource.org/licenses/MIT
 *
 * Vis.js may be distributed under either license.
 */var r;r=function(){return function(e){var t={};function n(r){if(t[r])return t[r].exports;var i=t[r]={i:r,l:!1,exports:{}};return e[r].call(i.exports,i,i.exports,n),i.l=!0,i.exports}return n.m=e,n.c=t,n.d=function(e,t,r){n.o(e,t)||Object.defineProperty(e,t,{configurable:!1,enumerable:!0,get:r})},n.n=function(e){var t=e&&e.__esModule?function(){return e.default}:function(){return e};return n.d(t,"a",t),t},n.o=function(e,t){return Object.prototype.hasOwnProperty.call(e,t)},n.p="",n(n.s=123)}([function(e,t,n){t.__esModule=!0,t.default=function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}},function(e,t,n){t.__esModule=!0;var r,i=n(169),o=(r=i)&&r.__esModule?r:{default:r};t.default=function(){function e(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),(0,o.default)(e,r.key,r)}}return function(t,n,r){return n&&e(t.prototype,n),r&&e(t,r),t}}()},function(e,t,n){var r=s(n(77)),i=s(n(29)),o=s(n(8)),a=s(n(6));function s(e){return e&&e.__esModule?e:{default:e}}var l=n(9),u=n(157);function c(e,t,n,r){var i=!1;!0===r&&(i=null===t[n]&&void 0!==e[n]),i?delete e[n]:e[n]=t[n]}t.isNumber=function(e){return e instanceof Number||"number"==typeof e},t.recursiveDOMDelete=function(e){if(e)for(;!0===e.hasChildNodes();)t.recursiveDOMDelete(e.firstChild),e.removeChild(e.firstChild)},t.giveRange=function(e,t,n,r){if(t==e)return.5;var i=1/(t-e);return Math.max(0,(r-e)*i)},t.isString=function(e){return e instanceof String||"string"==typeof e},t.isDate=function(e){if(e instanceof Date)return!0;if(t.isString(e)){if(d.exec(e))return!0;if(!isNaN(Date.parse(e)))return!0}return!1},t.randomUUID=function(){return u.v4()},t.assignAllKeys=function(e,t){for(var n in e)e.hasOwnProperty(n)&&"object"!==(0,a.default)(e[n])&&(e[n]=t)},t.fillIfDefined=function(e,n){var r=arguments.length>2&&void 0!==arguments[2]&&arguments[2];for(var i in e)void 0!==n[i]&&(null===n[i]||"object"!==(0,a.default)(n[i])?c(e,n,i,r):"object"===(0,a.default)(e[i])&&t.fillIfDefined(e[i],n[i],r))},t.extend=function(e,t){for(var n=1;n<arguments.length;n++){var r=arguments[n];for(var i in r)r.hasOwnProperty(i)&&(e[i]=r[i])}return e},t.selectiveExtend=function(e,t,n){if(!Array.isArray(e))throw new Error("Array with property names expected as first argument");for(var r=2;r<arguments.length;r++)for(var i=arguments[r],o=0;o<e.length;o++){var a=e[o];i&&i.hasOwnProperty(a)&&(t[a]=i[a])}return t},t.selectiveDeepExtend=function(e,n,r){var i=arguments.length>3&&void 0!==arguments[3]&&arguments[3];if(Array.isArray(r))throw new TypeError("Arrays are not supported by deepExtend");for(var o=0;o<e.length;o++){var a=e[o];if(r.hasOwnProperty(a))if(r[a]&&r[a].constructor===Object)void 0===n[a]&&(n[a]={}),n[a].constructor===Object?t.deepExtend(n[a],r[a],!1,i):c(n,r,a,i);else{if(Array.isArray(r[a]))throw new TypeError("Arrays are not supported by deepExtend");c(n,r,a,i)}}return n},t.selectiveNotDeepExtend=function(e,n,r){var i=arguments.length>3&&void 0!==arguments[3]&&arguments[3];if(Array.isArray(r))throw new TypeError("Arrays are not supported by deepExtend");for(var o in r)if(r.hasOwnProperty(o)&&-1===e.indexOf(o))if(r[o]&&r[o].constructor===Object)void 0===n[o]&&(n[o]={}),n[o].constructor===Object?t.deepExtend(n[o],r[o]):c(n,r,o,i);else if(Array.isArray(r[o])){n[o]=[];for(var a=0;a<r[o].length;a++)n[o].push(r[o][a])}else c(n,r,o,i);return n},t.deepExtend=function(e,n){var r=arguments.length>2&&void 0!==arguments[2]&&arguments[2],i=arguments.length>3&&void 0!==arguments[3]&&arguments[3];for(var o in n)if(n.hasOwnProperty(o)||!0===r)if(n[o]&&n[o].constructor===Object)void 0===e[o]&&(e[o]={}),e[o].constructor===Object?t.deepExtend(e[o],n[o],r):c(e,n,o,i);else if(Array.isArray(n[o])){e[o]=[];for(var a=0;a<n[o].length;a++)e[o].push(n[o][a])}else c(e,n,o,i);return e},t.equalArray=function(e,t){if(e.length!=t.length)return!1;for(var n=0,r=e.length;n<r;n++)if(e[n]!=t[n])return!1;return!0},t.convert=function(e,n){var r;if(void 0!==e){if(null===e)return null;if(!n)return e;if("string"!=typeof n&&!(n instanceof String))throw new Error("Type must be a string");switch(n){case"boolean":case"Boolean":return Boolean(e);case"number":case"Number":return t.isString(e)&&!isNaN(Date.parse(e))?l(e).valueOf():Number(e.valueOf());case"string":case"String":return String(e);case"Date":if(t.isNumber(e))return new Date(e);if(e instanceof Date)return new Date(e.valueOf());if(l.isMoment(e))return new Date(e.valueOf());if(t.isString(e))return(r=d.exec(e))?new Date(Number(r[1])):l(new Date(e)).toDate();throw new Error("Cannot convert object of type "+t.getType(e)+" to type Date");case"Moment":if(t.isNumber(e))return l(e);if(e instanceof Date)return l(e.valueOf());if(l.isMoment(e))return l(e);if(t.isString(e))return r=d.exec(e),l(r?Number(r[1]):e);throw new Error("Cannot convert object of type "+t.getType(e)+" to type Date");case"ISODate":if(t.isNumber(e))return new Date(e);if(e instanceof Date)return e.toISOString();if(l.isMoment(e))return e.toDate().toISOString();if(t.isString(e))return(r=d.exec(e))?new Date(Number(r[1])).toISOString():l(e).format();throw new Error("Cannot convert object of type "+t.getType(e)+" to type ISODate");case"ASPDate":if(t.isNumber(e))return"/Date("+e+")/";if(e instanceof Date)return"/Date("+e.valueOf()+")/";if(t.isString(e))return"/Date("+((r=d.exec(e))?new Date(Number(r[1])).valueOf():new Date(e).valueOf())+")/";throw new Error("Cannot convert object of type "+t.getType(e)+" to type ASPDate");default:throw new Error('Unknown type "'+n+'"')}}};var d=/^\/?Date\((\-?\d+)/i;t.getType=function(e){var t=void 0===e?"undefined":(0,a.default)(e);return"object"==t?null===e?"null":e instanceof Boolean?"Boolean":e instanceof Number?"Number":e instanceof String?"String":Array.isArray(e)?"Array":e instanceof Date?"Date":"Object":"number"==t?"Number":"boolean"==t?"Boolean":"string"==t?"String":void 0===t?"undefined":t},t.copyAndExtendArray=function(e,t){for(var n=[],r=0;r<e.length;r++)n.push(e[r]);return n.push(t),n},t.copyArray=function(e){for(var t=[],n=0;n<e.length;n++)t.push(e[n]);return t},t.getAbsoluteLeft=function(e){return e.getBoundingClientRect().left},t.getAbsoluteRight=function(e){return e.getBoundingClientRect().right},t.getAbsoluteTop=function(e){return e.getBoundingClientRect().top},t.addClassName=function(e,t){var n=e.className.split(" "),r=t.split(" ");n=n.concat(r.filter((function(e){return n.indexOf(e)<0}))),e.className=n.join(" ")},t.removeClassName=function(e,t){var n=e.className.split(" "),r=t.split(" ");n=n.filter((function(e){return r.indexOf(e)<0})),e.className=n.join(" ")},t.forEach=function(e,t){var n,r;if(Array.isArray(e))for(n=0,r=e.length;n<r;n++)t(e[n],n,e);else for(n in e)e.hasOwnProperty(n)&&t(e[n],n,e)},t.toArray=function(e){var t=[];for(var n in e)e.hasOwnProperty(n)&&t.push(e[n]);return t},t.updateProperty=function(e,t,n){return e[t]!==n&&(e[t]=n,!0)},t.throttle=function(e){var t=!1;return function(){t||(t=!0,requestAnimationFrame((function(){t=!1,e()})))}},t.addEventListener=function(e,t,n,r){e.addEventListener?(void 0===r&&(r=!1),"mousewheel"===t&&navigator.userAgent.indexOf("Firefox")>=0&&(t="DOMMouseScroll"),e.addEventListener(t,n,r)):e.attachEvent("on"+t,n)},t.removeEventListener=function(e,t,n,r){e.removeEventListener?(void 0===r&&(r=!1),"mousewheel"===t&&navigator.userAgent.indexOf("Firefox")>=0&&(t="DOMMouseScroll"),e.removeEventListener(t,n,r)):e.detachEvent("on"+t,n)},t.preventDefault=function(e){e||(e=window.event),e.preventDefault?e.preventDefault():e.returnValue=!1},t.getTarget=function(e){var t;return e||(e=window.event),e.target?t=e.target:e.srcElement&&(t=e.srcElement),null!=t.nodeType&&3==t.nodeType&&(t=t.parentNode),t},t.hasParent=function(e,t){for(var n=e;n;){if(n===t)return!0;n=n.parentNode}return!1},t.option={},t.option.asBoolean=function(e,t){return"function"==typeof e&&(e=e()),null!=e?0!=e:t||null},t.option.asNumber=function(e,t){return"function"==typeof e&&(e=e()),null!=e?Number(e)||t||null:t||null},t.option.asString=function(e,t){return"function"==typeof e&&(e=e()),null!=e?String(e):t||null},t.option.asSize=function(e,n){return"function"==typeof e&&(e=e()),t.isString(e)?e:t.isNumber(e)?e+"px":n||null},t.option.asElement=function(e,t){return"function"==typeof e&&(e=e()),e||t||null},t.hexToRGB=function(e){e=e.replace(/^#?([a-f\d])([a-f\d])([a-f\d])$/i,(function(e,t,n,r){return t+t+n+n+r+r}));var t=/^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(e);return t?{r:parseInt(t[1],16),g:parseInt(t[2],16),b:parseInt(t[3],16)}:null},t.overrideOpacity=function(e,n){var r;return-1!=e.indexOf("rgba")?e:-1!=e.indexOf("rgb")?"rgba("+(r=e.substr(e.indexOf("(")+1).replace(")","").split(","))[0]+","+r[1]+","+r[2]+","+n+")":null==(r=t.hexToRGB(e))?e:"rgba("+r.r+","+r.g+","+r.b+","+n+")"},t.RGBToHex=function(e,t,n){return"#"+((1<<24)+(e<<16)+(t<<8)+n).toString(16).slice(1)},t.parseColor=function(e){var n;if(!0===t.isString(e)){if(!0===t.isValidRGB(e)){var r=e.substr(4).substr(0,e.length-5).split(",").map((function(e){return parseInt(e)}));e=t.RGBToHex(r[0],r[1],r[2])}if(!0===t.isValidHex(e)){var i=t.hexToHSV(e),o={h:i.h,s:.8*i.s,v:Math.min(1,1.02*i.v)},a={h:i.h,s:Math.min(1,1.25*i.s),v:.8*i.v},s=t.HSVToHex(a.h,a.s,a.v),l=t.HSVToHex(o.h,o.s,o.v);n={background:e,border:s,highlight:{background:l,border:s},hover:{background:l,border:s}}}else n={background:e,border:e,highlight:{background:e,border:e},hover:{background:e,border:e}}}else(n={}).background=e.background||void 0,n.border=e.border||void 0,t.isString(e.highlight)?n.highlight={border:e.highlight,background:e.highlight}:(n.highlight={},n.highlight.background=e.highlight&&e.highlight.background||void 0,n.highlight.border=e.highlight&&e.highlight.border||void 0),t.isString(e.hover)?n.hover={border:e.hover,background:e.hover}:(n.hover={},n.hover.background=e.hover&&e.hover.background||void 0,n.hover.border=e.hover&&e.hover.border||void 0);return n},t.RGBToHSV=function(e,t,n){e/=255,t/=255,n/=255;var r=Math.min(e,Math.min(t,n)),i=Math.max(e,Math.max(t,n));return r==i?{h:0,s:0,v:r}:{h:60*((e==r?3:n==r?1:5)-(e==r?t-n:n==r?e-t:n-e)/(i-r))/360,s:(i-r)/i,v:i}};var h=function(e){var t={};return e.split(";").forEach((function(e){if(""!=e.trim()){var n=e.split(":"),r=n[0].trim(),i=n[1].trim();t[r]=i}})),t},f=function(e){return(0,o.default)(e).map((function(t){return t+": "+e[t]})).join("; ")};t.addCssText=function(e,n){var r=h(e.style.cssText),i=h(n),o=t.extend(r,i);e.style.cssText=f(o)},t.removeCssText=function(e,t){var n=h(e.style.cssText),r=h(t);for(var i in r)r.hasOwnProperty(i)&&delete n[i];e.style.cssText=f(n)},t.HSVToRGB=function(e,t,n){var r,i,o,a=Math.floor(6*e),s=6*e-a,l=n*(1-t),u=n*(1-s*t),c=n*(1-(1-s)*t);switch(a%6){case 0:r=n,i=c,o=l;break;case 1:r=u,i=n,o=l;break;case 2:r=l,i=n,o=c;break;case 3:r=l,i=u,o=n;break;case 4:r=c,i=l,o=n;break;case 5:r=n,i=l,o=u}return{r:Math.floor(255*r),g:Math.floor(255*i),b:Math.floor(255*o)}},t.HSVToHex=function(e,n,r){var i=t.HSVToRGB(e,n,r);return t.RGBToHex(i.r,i.g,i.b)},t.hexToHSV=function(e){var n=t.hexToRGB(e);return t.RGBToHSV(n.r,n.g,n.b)},t.isValidHex=function(e){return/(^#[0-9A-F]{6}$)|(^#[0-9A-F]{3}$)/i.test(e)},t.isValidRGB=function(e){return e=e.replace(" ",""),/rgb\((\d{1,3}),(\d{1,3}),(\d{1,3})\)/i.test(e)},t.isValidRGBA=function(e){return e=e.replace(" ",""),/rgba\((\d{1,3}),(\d{1,3}),(\d{1,3}),(.{1,3})\)/i.test(e)},t.selectiveBridgeObject=function(e,n){if(null!==n&&"object"===(void 0===n?"undefined":(0,a.default)(n))){for(var r=(0,i.default)(n),o=0;o<e.length;o++)n.hasOwnProperty(e[o])&&"object"==(0,a.default)(n[e[o]])&&(r[e[o]]=t.bridgeObject(n[e[o]]));return r}return null},t.bridgeObject=function(e){if(null!==e&&"object"===(void 0===e?"undefined":(0,a.default)(e))){var n=(0,i.default)(e);if(e instanceof Element)n=e;else for(var r in n=(0,i.default)(e),e)e.hasOwnProperty(r)&&"object"==(0,a.default)(e[r])&&(n[r]=t.bridgeObject(e[r]));return n}return null},t.insertSort=function(e,t){for(var n=0;n<e.length;n++){for(var r=e[n],i=n;i>0&&t(r,e[i-1])<0;i--)e[i]=e[i-1];e[i]=r}return e},t.mergeOptions=function(e,t,n){var r=arguments.length>3&&void 0!==arguments[3]?arguments[3]:{},o=function(e){return null!=e},s=function(e){return null!==e&&"object"===(void 0===e?"undefined":(0,a.default)(e))},l=function(e){for(var t in e)if(e.hasOwnProperty(t))return!1;return!0};if(!s(e))throw new Error("Parameter mergeTarget must be an object");if(!s(t))throw new Error("Parameter options must be an object");if(!o(n))throw new Error("Parameter option must have a value");if(!s(r))throw new Error("Parameter globalOptions must be an object");var u=function(e,t,n){s(e[n])||(e[n]={});var r=t[n],i=e[n];for(var o in r)r.hasOwnProperty(o)&&(i[o]=r[o])},c=t[n],d=s(r)&&!l(r),h=d?r[n]:void 0,f=h?h.enabled:void 0;if(void 0!==c){if("boolean"==typeof c)return s(e[n])||(e[n]={}),void(e[n].enabled=c);if(null===c&&!s(e[n])){if(!o(h))return;e[n]=(0,i.default)(h)}if(s(c)){var p=!0;void 0!==c.enabled?p=c.enabled:void 0!==f&&(p=h.enabled),u(e,t,n),e[n].enabled=p}}},t.binarySearchCustom=function(e,t,n,r){for(var i=0,o=0,a=e.length-1;o<=a&&i<1e4;){var s=Math.floor((o+a)/2),l=e[s],u=t(void 0===r?l[n]:l[n][r]);if(0==u)return s;-1==u?o=s+1:a=s-1,i++}return-1},t.binarySearchValue=function(e,t,n,r,i){var o,a,s,l,u=0,c=0,d=e.length-1;for(i=null!=i?i:function(e,t){return e==t?0:e<t?-1:1};c<=d&&u<1e4;){if(l=Math.floor(.5*(d+c)),o=e[Math.max(0,l-1)][n],a=e[l][n],s=e[Math.min(e.length-1,l+1)][n],0==i(a,t))return l;if(i(o,t)<0&&i(a,t)>0)return"before"==r?Math.max(0,l-1):l;if(i(a,t)<0&&i(s,t)>0)return"before"==r?l:Math.min(e.length-1,l+1);i(a,t)<0?c=l+1:d=l-1,u++}return-1},t.easingFunctions={linear:function(e){return e},easeInQuad:function(e){return e*e},easeOutQuad:function(e){return e*(2-e)},easeInOutQuad:function(e){return e<.5?2*e*e:(4-2*e)*e-1},easeInCubic:function(e){return e*e*e},easeOutCubic:function(e){return--e*e*e+1},easeInOutCubic:function(e){return e<.5?4*e*e*e:(e-1)*(2*e-2)*(2*e-2)+1},easeInQuart:function(e){return e*e*e*e},easeOutQuart:function(e){return 1- --e*e*e*e},easeInOutQuart:function(e){return e<.5?8*e*e*e*e:1-8*--e*e*e*e},easeInQuint:function(e){return e*e*e*e*e},easeOutQuint:function(e){return 1+--e*e*e*e*e},easeInOutQuint:function(e){return e<.5?16*e*e*e*e*e:1+16*--e*e*e*e*e}},t.getScrollBarWidth=function(){var e=document.createElement("p");e.style.width="100%",e.style.height="200px";var t=document.createElement("div");t.style.position="absolute",t.style.top="0px",t.style.left="0px",t.style.visibility="hidden",t.style.width="200px",t.style.height="150px",t.style.overflow="hidden",t.appendChild(e),document.body.appendChild(t);var n=e.offsetWidth;t.style.overflow="scroll";var r=e.offsetWidth;return n==r&&(r=t.clientWidth),document.body.removeChild(t),n-r},t.topMost=function(e,t){var n=void 0;Array.isArray(t)||(t=[t]);var i=!0,o=!1,a=void 0;try{for(var s,l=(0,r.default)(e);!(i=(s=l.next()).done);i=!0){var u=s.value;if(u){n=u[t[0]];for(var c=1;c<t.length;c++)n&&(n=n[t[c]]);if(void 0!==n)break}}}catch(e){o=!0,a=e}finally{try{!i&&l.return&&l.return()}finally{if(o)throw a}}return n}},function(e,t,n){e.exports={default:n(194),__esModule:!0}},function(e,t,n){t.__esModule=!0;var r,i=n(6),o=(r=i)&&r.__esModule?r:{default:r};t.default=function(e,t){if(!e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return!t||"object"!==(void 0===t?"undefined":(0,o.default)(t))&&"function"!=typeof t?e:t}},function(e,t,n){t.__esModule=!0;var r=a(n(196)),i=a(n(29)),o=a(n(6));function a(e){return e&&e.__esModule?e:{default:e}}t.default=function(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function, not "+(void 0===t?"undefined":(0,o.default)(t)));e.prototype=(0,i.default)(t&&t.prototype,{constructor:{value:e,enumerable:!1,writable:!0,configurable:!0}}),t&&(r.default?(0,r.default)(e,t):e.__proto__=t)}},function(e,t,n){t.__esModule=!0;var r=a(n(142)),i=a(n(144)),o="function"==typeof i.default&&"symbol"==typeof r.default?function(e){return typeof e}:function(e){return e&&"function"==typeof i.default&&e.constructor===i.default&&e!==i.default.prototype?"symbol":typeof e};function a(e){return e&&e.__esModule?e:{default:e}}t.default="function"==typeof i.default&&"symbol"===o(r.default)?function(e){return void 0===e?"undefined":o(e)}:function(e){return e&&"function"==typeof i.default&&e.constructor===i.default&&e!==i.default.prototype?"symbol":void 0===e?"undefined":o(e)}},function(e,t){var n=e.exports={version:"2.5.1"};"number"==typeof __e&&(__e=n)},function(e,t,n){e.exports={default:n(140),__esModule:!0}},function(e,t,n){e.exports="undefined"!=typeof window&&window.moment||n(154)},function(e,t,n){if("undefined"!=typeof window){var r=n(175),i=window.Hammer||n(176);e.exports=r(i,{preventDefault:"mouse"})}else e.exports=function(){return{on:e=function(){},off:e,destroy:e,emit:e,get:function(t){return{set:e}}};var e}},function(e,t,n){var r=a(n(19)),i=a(n(6)),o=a(n(8));function a(e){return e&&e.__esModule?e:{default:e}}var s=n(2),l=n(43);function u(e,t){if(e&&!Array.isArray(e)&&(t=e,e=null),this._options=t||{},this._data={},this.length=0,this._fieldId=this._options.fieldId||"id",this._type={},this._options.type)for(var n=(0,o.default)(this._options.type),r=0,i=n.length;r<i;r++){var a=n[r],s=this._options.type[a];this._type[a]="Date"==s||"ISODate"==s||"ASPDate"==s?"Date":s}this._subscribers={},e&&this.add(e),this.setOptions(t)}u.prototype.setOptions=function(e){e&&void 0!==e.queue&&(!1===e.queue?this._queue&&(this._queue.destroy(),delete this._queue):(this._queue||(this._queue=l.extend(this,{replace:["add","update","remove"]})),"object"===(0,i.default)(e.queue)&&this._queue.setOptions(e.queue)))},u.prototype.on=function(e,t){var n=this._subscribers[e];n||(n=[],this._subscribers[e]=n),n.push({callback:t})},u.prototype.off=function(e,t){var n=this._subscribers[e];n&&(this._subscribers[e]=n.filter((function(e){return e.callback!=t})))},u.prototype._trigger=function(e,t,n){if("*"==e)throw new Error("Cannot trigger event *");var r=[];e in this._subscribers&&(r=r.concat(this._subscribers[e])),"*"in this._subscribers&&(r=r.concat(this._subscribers["*"]));for(var i=0,o=r.length;i<o;i++){var a=r[i];a.callback&&a.callback(e,t,n||null)}},u.prototype.add=function(e,t){var n,r=[];if(Array.isArray(e))for(var o=0,a=e.length;o<a;o++)n=this._addItem(e[o]),r.push(n);else{if(!e||"object"!==(void 0===e?"undefined":(0,i.default)(e)))throw new Error("Unknown dataType");n=this._addItem(e),r.push(n)}return r.length&&this._trigger("add",{items:r},t),r},u.prototype.update=function(e,t){var n=[],r=[],o=[],a=[],l=this,u=l._fieldId,c=function(e){var t=e[u];if(l._data[t]){var i=s.extend({},l._data[t]);t=l._updateItem(e),r.push(t),a.push(e),o.push(i)}else t=l._addItem(e),n.push(t)};if(Array.isArray(e))for(var d=0,h=e.length;d<h;d++)e[d]&&"object"===(0,i.default)(e[d])?c(e[d]):console.warn("Ignoring input item, which is not an object at index "+d);else{if(!e||"object"!==(void 0===e?"undefined":(0,i.default)(e)))throw new Error("Unknown dataType");c(e)}if(n.length&&this._trigger("add",{items:n},t),r.length){var f={items:r,oldData:o,data:a};this._trigger("update",f,t)}return n.concat(r)},u.prototype.get=function(e){var t,n,r,i,a=this,l=s.getType(arguments[0]);if("String"==l||"Number"==l?(t=arguments[0],r=arguments[1]):"Array"==l?(n=arguments[0],r=arguments[1]):r=arguments[0],r&&r.returnType){var u=["Array","Object"];i=-1==u.indexOf(r.returnType)?"Array":r.returnType}else i="Array";var c,d,h,f,p,g=r&&r.type||this._options.type,v=r&&r.filter,m=[];if(null!=t)(c=a._getItem(t,g))&&v&&!v(c)&&(c=null);else if(null!=n)for(f=0,p=n.length;f<p;f++)c=a._getItem(n[f],g),v&&!v(c)||m.push(c);else for(f=0,p=(d=(0,o.default)(this._data)).length;f<p;f++)h=d[f],c=a._getItem(h,g),v&&!v(c)||m.push(c);if(r&&r.order&&null==t&&this._sort(m,r.order),r&&r.fields){var y=r.fields;if(null!=t)c=this._filterFields(c,y);else for(f=0,p=m.length;f<p;f++)m[f]=this._filterFields(m[f],y)}if("Object"==i){var b,C={};for(f=0,p=m.length;f<p;f++)C[(b=m[f]).id]=b;return C}return null!=t?c:m},u.prototype.getIds=function(e){var t,n,r,i,a,s=this._data,l=e&&e.filter,u=e&&e.order,c=e&&e.type||this._options.type,d=(0,o.default)(s),h=[];if(l)if(u){for(a=[],t=0,n=d.length;t<n;t++)r=d[t],l(i=this._getItem(r,c))&&a.push(i);for(this._sort(a,u),t=0,n=a.length;t<n;t++)h.push(a[t][this._fieldId])}else for(t=0,n=d.length;t<n;t++)r=d[t],l(i=this._getItem(r,c))&&h.push(i[this._fieldId]);else if(u){for(a=[],t=0,n=d.length;t<n;t++)r=d[t],a.push(s[r]);for(this._sort(a,u),t=0,n=a.length;t<n;t++)h.push(a[t][this._fieldId])}else for(t=0,n=d.length;t<n;t++)i=s[r=d[t]],h.push(i[this._fieldId]);return h},u.prototype.getDataSet=function(){return this},u.prototype.forEach=function(e,t){var n,r,i,a,s=t&&t.filter,l=t&&t.type||this._options.type,u=this._data,c=(0,o.default)(u);if(t&&t.order){var d=this.get(t);for(n=0,r=d.length;n<r;n++)e(i=d[n],a=i[this._fieldId])}else for(n=0,r=c.length;n<r;n++)a=c[n],i=this._getItem(a,l),s&&!s(i)||e(i,a)},u.prototype.map=function(e,t){var n,r,i,a,s=t&&t.filter,l=t&&t.type||this._options.type,u=[],c=this._data,d=(0,o.default)(c);for(n=0,r=d.length;n<r;n++)i=d[n],a=this._getItem(i,l),s&&!s(a)||u.push(e(a,i));return t&&t.order&&this._sort(u,t.order),u},u.prototype._filterFields=function(e,t){if(!e)return e;var n,r,i={},a=(0,o.default)(e),s=a.length;if(Array.isArray(t))for(n=0;n<s;n++)r=a[n],-1!=t.indexOf(r)&&(i[r]=e[r]);else for(n=0;n<s;n++)r=a[n],t.hasOwnProperty(r)&&(i[t[r]]=e[r]);return i},u.prototype._sort=function(e,t){if(s.isString(t)){var n=t;e.sort((function(e,t){var r=e[n],i=t[n];return r>i?1:r<i?-1:0}))}else{if("function"!=typeof t)throw new TypeError("Order must be a function or a string");e.sort(t)}},u.prototype.remove=function(e,t){var n,r,i,o,a,s=[],l=[];for(r=0,i=(n=Array.isArray(e)?e:[e]).length;r<i;r++)(a=this._remove(n[r]))&&null!=(o=a[this._fieldId])&&(s.push(o),l.push(a));return s.length&&this._trigger("remove",{items:s,oldData:l},t),s},u.prototype._remove=function(e){var t,n;return s.isNumber(e)||s.isString(e)?n=e:e&&"object"===(void 0===e?"undefined":(0,i.default)(e))&&(n=e[this._fieldId]),void 0!==n&&this._data[n]?(t=this._data[n],delete this._data[n],this.length--,t):null},u.prototype.clear=function(e){var t,n,r=(0,o.default)(this._data),i=[];for(t=0,n=r.length;t<n;t++)i.push(this._data[r[t]]);return this._data={},this.length=0,this._trigger("remove",{items:r,oldData:i},e),r},u.prototype.max=function(e){var t,n,r=this._data,i=(0,o.default)(r),a=null,s=null;for(t=0,n=i.length;t<n;t++){var l=r[i[t]],u=l[e];null!=u&&(!a||u>s)&&(a=l,s=u)}return a},u.prototype.min=function(e){var t,n,r=this._data,i=(0,o.default)(r),a=null,s=null;for(t=0,n=i.length;t<n;t++){var l=r[i[t]],u=l[e];null!=u&&(!a||u<s)&&(a=l,s=u)}return a},u.prototype.distinct=function(e){var t,n,r,i=this._data,a=(0,o.default)(i),l=[],u=this._options.type&&this._options.type[e]||null,c=0;for(t=0,r=a.length;t<r;t++){var d=i[a[t]][e],h=!1;for(n=0;n<c;n++)if(l[n]==d){h=!0;break}h||void 0===d||(l[c]=d,c++)}if(u)for(t=0,r=l.length;t<r;t++)l[t]=s.convert(l[t],u);return l},u.prototype._addItem=function(e){var t=e[this._fieldId];if(null!=t){if(this._data[t])throw new Error("Cannot add item: item with id "+t+" already exists")}else t=s.randomUUID(),e[this._fieldId]=t;var n,r,i={},a=(0,o.default)(e);for(n=0,r=a.length;n<r;n++){var l=a[n],u=this._type[l];i[l]=s.convert(e[l],u)}return this._data[t]=i,this.length++,t},u.prototype._getItem=function(e,t){var n,r,i,a,l=this._data[e];if(!l)return null;var u={},c=(0,o.default)(l);if(t)for(i=0,a=c.length;i<a;i++)r=l[n=c[i]],u[n]=s.convert(r,t[n]);else for(i=0,a=c.length;i<a;i++)r=l[n=c[i]],u[n]=r;return u[this._fieldId]||(u[this._fieldId]=l.id),u},u.prototype._updateItem=function(e){var t=e[this._fieldId];if(null==t)throw new Error("Cannot update item: item has no id (item: "+(0,r.default)(e)+")");var n=this._data[t];if(!n)throw new Error("Cannot update item: no item with id "+t+" found");for(var i=(0,o.default)(e),a=0,l=i.length;a<l;a++){var u=i[a],c=this._type[u];n[u]=s.convert(e[u],c)}return t},e.exports=u},function(e,t,n){var r,i=n(8),o=(r=i)&&r.__esModule?r:{default:r},a=n(2),s=n(11);function l(e,t){this._data=null,this._ids={},this.length=0,this._options=t||{},this._fieldId="id",this._subscribers={};var n=this;this.listener=function(){n._onEvent.apply(n,arguments)},this.setData(e)}l.prototype.setData=function(e){var t,n,r,i,o;if(this._data){for(this._data.off&&this._data.off("*",this.listener),o=[],r=0,i=(t=this._data.getIds({filter:this._options&&this._options.filter})).length;r<i;r++)o.push(this._data._data[t[r]]);this._ids={},this.length=0,this._trigger("remove",{items:t,oldData:o})}if(this._data=e,this._data){for(this._fieldId=this._options.fieldId||this._data&&this._data.options&&this._data.options.fieldId||"id",r=0,i=(t=this._data.getIds({filter:this._options&&this._options.filter})).length;r<i;r++)n=t[r],this._ids[n]=!0;this.length=t.length,this._trigger("add",{items:t}),this._data.on&&this._data.on("*",this.listener)}},l.prototype.refresh=function(){var e,t,n,r=this._data.getIds({filter:this._options&&this._options.filter}),i=(0,o.default)(this._ids),a={},s=[],l=[],u=[];for(t=0,n=r.length;t<n;t++)a[e=r[t]]=!0,this._ids[e]||(s.push(e),this._ids[e]=!0);for(t=0,n=i.length;t<n;t++)a[e=i[t]]||(l.push(e),u.push(this._data._data[e]),delete this._ids[e]);this.length+=s.length-l.length,s.length&&this._trigger("add",{items:s}),l.length&&this._trigger("remove",{items:l,oldData:u})},l.prototype.get=function(e){var t,n,r,i=this,o=a.getType(arguments[0]);"String"==o||"Number"==o||"Array"==o?(t=arguments[0],n=arguments[1],r=arguments[2]):(n=arguments[0],r=arguments[1]);var s=a.extend({},this._options,n);this._options.filter&&n&&n.filter&&(s.filter=function(e){return i._options.filter(e)&&n.filter(e)});var l=[];return null!=t&&l.push(t),l.push(s),l.push(r),this._data&&this._data.get.apply(this._data,l)},l.prototype.getIds=function(e){var t;if(this._data){var n,r=this._options.filter;n=e&&e.filter?r?function(t){return r(t)&&e.filter(t)}:e.filter:r,t=this._data.getIds({filter:n,order:e&&e.order})}else t=[];return t},l.prototype.map=function(e,t){var n=[];if(this._data){var r,i=this._options.filter;r=t&&t.filter?i?function(e){return i(e)&&t.filter(e)}:t.filter:i,n=this._data.map(e,{filter:r,order:t&&t.order})}else n=[];return n},l.prototype.getDataSet=function(){for(var e=this;e instanceof l;)e=e._data;return e||null},l.prototype._onEvent=function(e,t,n){var r,i,o,a=t&&t.items,s=[],l=[],u=[],c=[],d=[],h=[];if(a&&this._data){switch(e){case"add":for(r=0,i=a.length;r<i;r++)o=a[r],this.get(o)&&(this._ids[o]=!0,s.push(o));break;case"update":for(r=0,i=a.length;r<i;r++)o=a[r],this.get(o)?this._ids[o]?(l.push(o),d.push(t.data[r]),c.push(t.oldData[r])):(this._ids[o]=!0,s.push(o)):this._ids[o]&&(delete this._ids[o],u.push(o),h.push(t.oldData[r]));break;case"remove":for(r=0,i=a.length;r<i;r++)o=a[r],this._ids[o]&&(delete this._ids[o],u.push(o),h.push(t.oldData[r]))}this.length+=s.length-u.length,s.length&&this._trigger("add",{items:s},n),l.length&&this._trigger("update",{items:l,oldData:c,data:d},n),u.length&&this._trigger("remove",{items:u,oldData:h},n)}},l.prototype.on=s.prototype.on,l.prototype.off=s.prototype.off,l.prototype._trigger=s.prototype._trigger,l.prototype.subscribe=l.prototype.on,l.prototype.unsubscribe=l.prototype.off,e.exports=l},function(e,t,n){var r=n(57)("wks"),i=n(40),o=n(18).Symbol,a="function"==typeof o;(e.exports=function(e){return r[e]||(r[e]=a&&o[e]||(a?o:i)("Symbol."+e))}).store=r},function(e,t,n){t.prepareElements=function(e){for(var t in e)e.hasOwnProperty(t)&&(e[t].redundant=e[t].used,e[t].used=[])},t.cleanupElements=function(e){for(var t in e)if(e.hasOwnProperty(t)&&e[t].redundant){for(var n=0;n<e[t].redundant.length;n++)e[t].redundant[n].parentNode.removeChild(e[t].redundant[n]);e[t].redundant=[]}},t.resetElements=function(e){t.prepareElements(e),t.cleanupElements(e),t.prepareElements(e)},t.getSVGElement=function(e,t,n){var r;return t.hasOwnProperty(e)?t[e].redundant.length>0?(r=t[e].redundant[0],t[e].redundant.shift()):(r=document.createElementNS("http://www.w3.org/2000/svg",e),n.appendChild(r)):(r=document.createElementNS("http://www.w3.org/2000/svg",e),t[e]={used:[],redundant:[]},n.appendChild(r)),t[e].used.push(r),r},t.getDOMElement=function(e,t,n,r){var i;return t.hasOwnProperty(e)?t[e].redundant.length>0?(i=t[e].redundant[0],t[e].redundant.shift()):(i=document.createElement(e),void 0!==r?n.insertBefore(i,r):n.appendChild(i)):(i=document.createElement(e),t[e]={used:[],redundant:[]},void 0!==r?n.insertBefore(i,r):n.appendChild(i)),t[e].used.push(i),i},t.drawPoint=function(e,n,r,i,o,a){var s;if("circle"==r.style?((s=t.getSVGElement("circle",i,o)).setAttributeNS(null,"cx",e),s.setAttributeNS(null,"cy",n),s.setAttributeNS(null,"r",.5*r.size)):((s=t.getSVGElement("rect",i,o)).setAttributeNS(null,"x",e-.5*r.size),s.setAttributeNS(null,"y",n-.5*r.size),s.setAttributeNS(null,"width",r.size),s.setAttributeNS(null,"height",r.size)),void 0!==r.styles&&s.setAttributeNS(null,"style",r.styles),s.setAttributeNS(null,"class",r.className+" vis-point"),a){var l=t.getSVGElement("text",i,o);a.xOffset&&(e+=a.xOffset),a.yOffset&&(n+=a.yOffset),a.content&&(l.textContent=a.content),a.className&&l.setAttributeNS(null,"class",a.className+" vis-label"),l.setAttributeNS(null,"x",e),l.setAttributeNS(null,"y",n)}return s},t.drawBar=function(e,n,r,i,o,a,s,l){if(0!=i){i<0&&(n-=i*=-1);var u=t.getSVGElement("rect",a,s);u.setAttributeNS(null,"x",e-.5*r),u.setAttributeNS(null,"y",n),u.setAttributeNS(null,"width",r),u.setAttributeNS(null,"height",i),u.setAttributeNS(null,"class",o),l&&u.setAttributeNS(null,"style",l)}}},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0}),t.printStyle=void 0;var r=l(n(19)),i=l(n(6)),o=l(n(8)),a=l(n(0)),s=l(n(1));function l(e){return e&&e.__esModule?e:{default:e}}var u=n(2),c=!1,d=void 0,h="background: #FFeeee; color: #dd0000",f=function(){function e(){(0,a.default)(this,e)}return(0,s.default)(e,null,[{key:"validate",value:function(t,n,r){c=!1,d=n;var i=n;return void 0!==r&&(i=n[r]),e.parse(t,i,[]),c}},{key:"parse",value:function(t,n,r){for(var i in t)t.hasOwnProperty(i)&&e.check(i,t,n,r)}},{key:"check",value:function(t,n,r,i){if(void 0!==r[t]||void 0!==r.__any__){var o=t,a=!0;void 0===r[t]&&void 0!==r.__any__&&(o="__any__",a="object"===e.getType(n[t]));var s=r[o];a&&void 0!==s.__type__&&(s=s.__type__),e.checkFields(t,n,r,o,s,i)}else e.getSuggestion(t,r,i)}},{key:"checkFields",value:function(t,n,r,i,a,s){var l=function(n){console.log("%c"+n+e.printLocation(s,t),h)},d=e.getType(n[t]),f=a[d];void 0!==f?"array"===e.getType(f)&&-1===f.indexOf(n[t])?(l('Invalid option detected in "'+t+'". Allowed values are:'+e.print(f)+' not "'+n[t]+'". '),c=!0):"object"===d&&"__any__"!==i&&(s=u.copyAndExtendArray(s,t),e.parse(n[t],r[i],s)):void 0===a.any&&(l('Invalid type received for "'+t+'". Expected: '+e.print((0,o.default)(a))+". Received ["+d+'] "'+n[t]+'"'),c=!0)}},{key:"getType",value:function(e){var t=void 0===e?"undefined":(0,i.default)(e);return"object"===t?null===e?"null":e instanceof Boolean?"boolean":e instanceof Number?"number":e instanceof String?"string":Array.isArray(e)?"array":e instanceof Date?"date":void 0!==e.nodeType?"dom":!0===e._isAMomentObject?"moment":"object":"number"===t?"number":"boolean"===t?"boolean":"string"===t?"string":void 0===t?"undefined":t}},{key:"getSuggestion",value:function(t,n,r){var i=e.findInOptions(t,n,r,!1),a=e.findInOptions(t,d,[],!0),s=void 0;s=void 0!==i.indexMatch?" in "+e.printLocation(i.path,t,"")+'Perhaps it was incomplete? Did you mean: "'+i.indexMatch+'"?\n\n':a.distance<=4&&i.distance>a.distance?" in "+e.printLocation(i.path,t,"")+"Perhaps it was misplaced? Matching option found at: "+e.printLocation(a.path,a.closestMatch,""):i.distance<=8?'. Did you mean "'+i.closestMatch+'"?'+e.printLocation(i.path,t):". Did you mean one of these: "+e.print((0,o.default)(n))+e.printLocation(r,t),console.log('%cUnknown option detected: "'+t+'"'+s,h),c=!0}},{key:"findInOptions",value:function(t,n,r){var i=arguments.length>3&&void 0!==arguments[3]&&arguments[3],o=1e9,a="",s=[],l=t.toLowerCase(),c=void 0;for(var d in n){var h=void 0;if(void 0!==n[d].__type__&&!0===i){var f=e.findInOptions(t,n[d],u.copyAndExtendArray(r,d));o>f.distance&&(a=f.closestMatch,s=f.path,o=f.distance,c=f.indexMatch)}else-1!==d.toLowerCase().indexOf(l)&&(c=d),o>(h=e.levenshteinDistance(t,d))&&(a=d,s=u.copyArray(r),o=h)}return{closestMatch:a,path:s,distance:o,indexMatch:c}}},{key:"printLocation",value:function(e,t){for(var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:"Problem value found at: \n",r="\n\n"+n+"options = {\n",i=0;i<e.length;i++){for(var o=0;o<i+1;o++)r+="  ";r+=e[i]+": {\n"}for(var a=0;a<e.length+1;a++)r+="  ";r+=t+"\n";for(var s=0;s<e.length+1;s++){for(var l=0;l<e.length-s;l++)r+="  ";r+="}\n"}return r+"\n\n"}},{key:"print",value:function(e){return(0,r.default)(e).replace(/(\")|(\[)|(\])|(,"__type__")/g,"").replace(/(\,)/g,", ")}},{key:"levenshteinDistance",value:function(e,t){if(0===e.length)return t.length;if(0===t.length)return e.length;var n,r,i=[];for(n=0;n<=t.length;n++)i[n]=[n];for(r=0;r<=e.length;r++)i[0][r]=r;for(n=1;n<=t.length;n++)for(r=1;r<=e.length;r++)t.charAt(n-1)==e.charAt(r-1)?i[n][r]=i[n-1][r-1]:i[n][r]=Math.min(i[n-1][r-1]+1,Math.min(i[n][r-1]+1,i[n-1][r]+1));return i[t.length][e.length]}}]),e}();t.default=f,t.printStyle=h},function(e,t,n){var r=n(2);function i(e,t){this.options=null,this.props=null}i.prototype.setOptions=function(e){e&&r.extend(this.options,e)},i.prototype.redraw=function(){return!1},i.prototype.destroy=function(){},i.prototype._isResized=function(){var e=this.props._previousWidth!==this.props.width||this.props._previousHeight!==this.props.height;return this.props._previousWidth=this.props.width,this.props._previousHeight=this.props.height,e},e.exports=i},function(e,t,n){var r=n(18),i=n(7),o=n(80),a=n(26),s=function(e,t,n){var l,u,c,d=e&s.F,h=e&s.G,f=e&s.S,p=e&s.P,g=e&s.B,v=e&s.W,m=h?i:i[t]||(i[t]={}),y=m.prototype,b=h?r:f?r[t]:(r[t]||{}).prototype;for(l in h&&(n=t),n)(u=!d&&b&&void 0!==b[l])&&l in m||(c=u?b[l]:n[l],m[l]=h&&"function"!=typeof b[l]?n[l]:g&&u?o(c,r):v&&b[l]==c?function(e){var t=function(t,n,r){if(this instanceof e){switch(arguments.length){case 0:return new e;case 1:return new e(t);case 2:return new e(t,n)}return new e(t,n,r)}return e.apply(this,arguments)};return t.prototype=e.prototype,t}(c):p&&"function"==typeof c?o(Function.call,c):c,p&&((m.virtual||(m.virtual={}))[l]=c,e&s.R&&y&&!y[l]&&a(y,l,c)))};s.F=1,s.G=2,s.S=4,s.P=8,s.B=16,s.W=32,s.U=64,s.R=128,e.exports=s},function(e,t){var n=e.exports="undefined"!=typeof window&&window.Math==Math?window:"undefined"!=typeof self&&self.Math==Math?self:Function("return this")();"number"==typeof __g&&(__g=n)},function(e,t,n){e.exports={default:n(160),__esModule:!0}},function(e,t,n){var r=n(27),i=n(81),o=n(53),a=Object.defineProperty;t.f=n(21)?Object.defineProperty:function(e,t,n){if(r(e),t=o(t,!0),r(n),i)try{return a(e,t,n)}catch(e){}if("get"in n||"set"in n)throw TypeError("Accessors not supported!");return"value"in n&&(e[t]=n.value),e}},function(e,t,n){e.exports=!n(28)((function(){return 7!=Object.defineProperty({},"a",{get:function(){return 7}}).a}))},function(e,t){var n={}.hasOwnProperty;e.exports=function(e,t){return n.call(e,t)}},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=a(n(6)),i=a(n(0)),o=a(n(1));function a(e){return e&&e.__esModule?e:{default:e}}var s=function(){function e(t,n,r){(0,i.default)(this,e),this.body=n,this.labelModule=r,this.setOptions(t),this.top=void 0,this.left=void 0,this.height=void 0,this.width=void 0,this.radius=void 0,this.margin=void 0,this.refreshNeeded=!0,this.boundingBox={top:0,left:0,right:0,bottom:0}}return(0,o.default)(e,[{key:"setOptions",value:function(e){this.options=e}},{key:"_setMargins",value:function(e){this.margin={},this.options.margin&&("object"==(0,r.default)(this.options.margin)?(this.margin.top=this.options.margin.top,this.margin.right=this.options.margin.right,this.margin.bottom=this.options.margin.bottom,this.margin.left=this.options.margin.left):(this.margin.top=this.options.margin,this.margin.right=this.options.margin,this.margin.bottom=this.options.margin,this.margin.left=this.options.margin)),e.adjustSizes(this.margin)}},{key:"_distanceToBorder",value:function(e,t){var n=this.options.borderWidth;return this.resize(e),Math.min(Math.abs(this.width/2/Math.cos(t)),Math.abs(this.height/2/Math.sin(t)))+n}},{key:"enableShadow",value:function(e,t){t.shadow&&(e.shadowColor=t.shadowColor,e.shadowBlur=t.shadowSize,e.shadowOffsetX=t.shadowX,e.shadowOffsetY=t.shadowY)}},{key:"disableShadow",value:function(e,t){t.shadow&&(e.shadowColor="rgba(0,0,0,0)",e.shadowBlur=0,e.shadowOffsetX=0,e.shadowOffsetY=0)}},{key:"enableBorderDashes",value:function(e,t){if(!1!==t.borderDashes)if(void 0!==e.setLineDash){var n=t.borderDashes;!0===n&&(n=[5,15]),e.setLineDash(n)}else console.warn("setLineDash is not supported in this browser. The dashed borders cannot be used."),this.options.shapeProperties.borderDashes=!1,t.borderDashes=!1}},{key:"disableBorderDashes",value:function(e,t){!1!==t.borderDashes&&(void 0!==e.setLineDash?e.setLineDash([0]):(console.warn("setLineDash is not supported in this browser. The dashed borders cannot be used."),this.options.shapeProperties.borderDashes=!1,t.borderDashes=!1))}},{key:"needsRefresh",value:function(e,t){return!0===this.refreshNeeded?(this.refreshNeeded=!1,!0):void 0===this.width||this.labelModule.differentState(e,t)}},{key:"initContextForDraw",value:function(e,t){var n=t.borderWidth/this.body.view.scale;e.lineWidth=Math.min(this.width,n),e.strokeStyle=t.borderColor,e.fillStyle=t.color}},{key:"performStroke",value:function(e,t){var n=t.borderWidth/this.body.view.scale;e.save(),n>0&&(this.enableBorderDashes(e,t),e.stroke(),this.disableBorderDashes(e,t)),e.restore()}},{key:"performFill",value:function(e,t){this.enableShadow(e,t),e.fill(),this.disableShadow(e,t),this.performStroke(e,t)}},{key:"_addBoundingBoxMargin",value:function(e){this.boundingBox.left-=e,this.boundingBox.top-=e,this.boundingBox.bottom+=e,this.boundingBox.right+=e}},{key:"_updateBoundingBox",value:function(e,t,n,r,i){void 0!==n&&this.resize(n,r,i),this.left=e-this.width/2,this.top=t-this.height/2,this.boundingBox.left=this.left,this.boundingBox.top=this.top,this.boundingBox.bottom=this.top+this.height,this.boundingBox.right=this.left+this.width}},{key:"updateBoundingBox",value:function(e,t,n,r,i){this._updateBoundingBox(e,t,n,r,i)}},{key:"getDimensionsFromLabel",value:function(e,t,n){this.textSize=this.labelModule.getTextSize(e,t,n);var r=this.textSize.width,i=this.textSize.height;return 0===r&&(r=14,i=14),{width:r,height:i}}}]),e}();t.default=s},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=l(n(3)),i=l(n(0)),o=l(n(1)),a=l(n(4)),s=l(n(5));function l(e){return e&&e.__esModule?e:{default:e}}var u=function(e){function t(e,n,o){return(0,i.default)(this,t),(0,a.default)(this,(t.__proto__||(0,r.default)(t)).call(this,e,n,o))}return(0,s.default)(t,e),(0,o.default)(t,[{key:"resize",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:this.selected,n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:this.hover,r=arguments.length>3&&void 0!==arguments[3]?arguments[3]:{size:this.options.size};if(this.needsRefresh(t,n)){this.labelModule.getTextSize(e,t,n);var i=2*r.size;this.width=i,this.height=i,this.radius=.5*this.width}}},{key:"_drawShape",value:function(e,t,n,r,i,o,a,s){if(this.resize(e,o,a,s),this.left=r-this.width/2,this.top=i-this.height/2,this.initContextForDraw(e,s),e[t](r,i,s.size),this.performFill(e,s),void 0!==this.options.label){this.labelModule.calculateLabelSize(e,o,a,r,i,"hanging");var l=i+.5*this.height+.5*this.labelModule.size.height;this.labelModule.draw(e,r,l,o,a,"hanging")}this.updateBoundingBox(r,i)}},{key:"updateBoundingBox",value:function(e,t){this.boundingBox.top=t-this.options.size,this.boundingBox.left=e-this.options.size,this.boundingBox.right=e+this.options.size,this.boundingBox.bottom=t+this.options.size,void 0!==this.options.label&&this.labelModule.size.width>0&&(this.boundingBox.left=Math.min(this.boundingBox.left,this.labelModule.size.left),this.boundingBox.right=Math.max(this.boundingBox.right,this.labelModule.size.left+this.labelModule.size.width),this.boundingBox.bottom=Math.max(this.boundingBox.bottom,this.boundingBox.bottom+this.labelModule.size.height))}}]),t}(l(n(23)).default);t.default=u},function(e,t,n){var r=n(78),i=n(51);e.exports=function(e){return r(i(e))}},function(e,t,n){var r=n(20),i=n(39);e.exports=n(21)?function(e,t,n){return r.f(e,t,i(1,n))}:function(e,t,n){return e[t]=n,e}},function(e,t,n){var r=n(32);e.exports=function(e){if(!r(e))throw TypeError(e+" is not an object!");return e}},function(e,t){e.exports=function(e){try{return!!e()}catch(e){return!0}}},function(e,t,n){e.exports={default:n(138),__esModule:!0}},function(e,t,n){t.__esModule=!0;var r=o(n(188)),i=o(n(77));function o(e){return e&&e.__esModule?e:{default:e}}t.default=function(e,t){if(Array.isArray(e))return e;if((0,r.default)(Object(e)))return function(e,t){var n=[],r=!0,o=!1,a=void 0;try{for(var s,l=(0,i.default)(e);!(r=(s=l.next()).done)&&(n.push(s.value),!t||n.length!==t);r=!0);}catch(e){o=!0,a=e}finally{try{!r&&l.return&&l.return()}finally{if(o)throw a}}return n}(e,t);throw new TypeError("Invalid attempt to destructure non-iterable instance")}},function(e,t){e.exports={}},function(e,t){e.exports=function(e){return"object"==typeof e?null!==e:"function"==typeof e}},function(e,t,n){var r=n(84),i=n(58);e.exports=Object.keys||function(e){return r(e,i)}},function(e,t,n){function r(e,t,n){this.x=void 0!==e?e:0,this.y=void 0!==t?t:0,this.z=void 0!==n?n:0}r.subtract=function(e,t){var n=new r;return n.x=e.x-t.x,n.y=e.y-t.y,n.z=e.z-t.z,n},r.add=function(e,t){var n=new r;return n.x=e.x+t.x,n.y=e.y+t.y,n.z=e.z+t.z,n},r.avg=function(e,t){return new r((e.x+t.x)/2,(e.y+t.y)/2,(e.z+t.z)/2)},r.crossProduct=function(e,t){var n=new r;return n.x=e.y*t.z-e.z*t.y,n.y=e.z*t.x-e.x*t.z,n.z=e.x*t.y-e.y*t.x,n},r.prototype.length=function(){return Math.sqrt(this.x*this.x+this.y*this.y+this.z*this.z)},e.exports=r},function(e,t,n){var r,i,o;i=[],void 0===(o="function"==typeof(r=function(){return function(e){var t,n=e&&e.preventDefault||!1,r=e&&e.container||window,i={},o={keydown:{},keyup:{}},a={};for(t=97;t<=122;t++)a[String.fromCharCode(t)]={code:t-97+65,shift:!1};for(t=65;t<=90;t++)a[String.fromCharCode(t)]={code:t,shift:!0};for(t=0;t<=9;t++)a[""+t]={code:48+t,shift:!1};for(t=1;t<=12;t++)a["F"+t]={code:111+t,shift:!1};for(t=0;t<=9;t++)a["num"+t]={code:96+t,shift:!1};a["num*"]={code:106,shift:!1},a["num+"]={code:107,shift:!1},a["num-"]={code:109,shift:!1},a["num/"]={code:111,shift:!1},a["num."]={code:110,shift:!1},a.left={code:37,shift:!1},a.up={code:38,shift:!1},a.right={code:39,shift:!1},a.down={code:40,shift:!1},a.space={code:32,shift:!1},a.enter={code:13,shift:!1},a.shift={code:16,shift:void 0},a.esc={code:27,shift:!1},a.backspace={code:8,shift:!1},a.tab={code:9,shift:!1},a.ctrl={code:17,shift:!1},a.alt={code:18,shift:!1},a.delete={code:46,shift:!1},a.pageup={code:33,shift:!1},a.pagedown={code:34,shift:!1},a["="]={code:187,shift:!1},a["-"]={code:189,shift:!1},a["]"]={code:221,shift:!1},a["["]={code:219,shift:!1};var s=function(e){u(e,"keydown")},l=function(e){u(e,"keyup")},u=function(e,t){if(void 0!==o[t][e.keyCode]){for(var r=o[t][e.keyCode],i=0;i<r.length;i++)(void 0===r[i].shift||1==r[i].shift&&1==e.shiftKey||0==r[i].shift&&0==e.shiftKey)&&r[i].fn(e);1==n&&e.preventDefault()}};return i.bind=function(e,t,n){if(void 0===n&&(n="keydown"),void 0===a[e])throw new Error("unsupported key: "+e);void 0===o[n][a[e].code]&&(o[n][a[e].code]=[]),o[n][a[e].code].push({fn:t,shift:a[e].shift})},i.bindAll=function(e,t){for(var n in void 0===t&&(t="keydown"),a)a.hasOwnProperty(n)&&i.bind(n,e,t)},i.getKey=function(e){for(var t in a)if(a.hasOwnProperty(t)){if(1==e.shiftKey&&1==a[t].shift&&e.keyCode==a[t].code)return t;if(0==e.shiftKey&&0==a[t].shift&&e.keyCode==a[t].code)return t;if(e.keyCode==a[t].code&&"shift"==t)return t}return"unknown key, currently not supported"},i.unbind=function(e,t,n){if(void 0===n&&(n="keydown"),void 0===a[e])throw new Error("unsupported key: "+e);if(void 0!==t){var r=[],i=o[n][a[e].code];if(void 0!==i)for(var s=0;s<i.length;s++)i[s].fn==t&&i[s].shift==a[e].shift||r.push(o[n][a[e].code][s]);o[n][a[e].code]=r}else o[n][a[e].code]=[]},i.reset=function(){o={keydown:{},keyup:{}}},i.destroy=function(){o={keydown:{},keyup:{}},r.removeEventListener("keydown",s,!0),r.removeEventListener("keyup",l,!0)},r.addEventListener("keydown",s,!0),r.addEventListener("keyup",l,!0),i}})?r.apply(t,i):r)||(e.exports=o)},function(e,t,n){t.convertHiddenOptions=function(e,n,r){if(r&&!Array.isArray(r))return t.convertHiddenOptions(e,n,[r]);if(n.hiddenDates=[],r&&1==Array.isArray(r)){for(var i=0;i<r.length;i++)if(void 0===r[i].repeat){var o={};o.start=e(r[i].start).toDate().valueOf(),o.end=e(r[i].end).toDate().valueOf(),n.hiddenDates.push(o)}n.hiddenDates.sort((function(e,t){return e.start-t.start}))}},t.updateHiddenDates=function(e,n,r){if(r&&!Array.isArray(r))return t.updateHiddenDates(e,n,[r]);if(r&&void 0!==n.domProps.centerContainer.width){t.convertHiddenOptions(e,n,r);for(var i=e(n.range.start),o=e(n.range.end),a=(n.range.end-n.range.start)/n.domProps.centerContainer.width,s=0;s<r.length;s++)if(void 0!==r[s].repeat){var l=e(r[s].start),u=e(r[s].end);if("Invalid Date"==l._d)throw new Error("Supplied start date is not valid: "+r[s].start);if("Invalid Date"==u._d)throw new Error("Supplied end date is not valid: "+r[s].end);if(u-l>=4*a){var c=0,d=o.clone();switch(r[s].repeat){case"daily":l.day()!=u.day()&&(c=1),l.dayOfYear(i.dayOfYear()),l.year(i.year()),l.subtract(7,"days"),u.dayOfYear(i.dayOfYear()),u.year(i.year()),u.subtract(7-c,"days"),d.add(1,"weeks");break;case"weekly":var h=u.diff(l,"days"),f=l.day();l.date(i.date()),l.month(i.month()),l.year(i.year()),u=l.clone(),l.day(f),u.day(f),u.add(h,"days"),l.subtract(1,"weeks"),u.subtract(1,"weeks"),d.add(1,"weeks");break;case"monthly":l.month()!=u.month()&&(c=1),l.month(i.month()),l.year(i.year()),l.subtract(1,"months"),u.month(i.month()),u.year(i.year()),u.subtract(1,"months"),u.add(c,"months"),d.add(1,"months");break;case"yearly":l.year()!=u.year()&&(c=1),l.year(i.year()),l.subtract(1,"years"),u.year(i.year()),u.subtract(1,"years"),u.add(c,"years"),d.add(1,"years");break;default:return void console.log("Wrong repeat format, allowed are: daily, weekly, monthly, yearly. Given:",r[s].repeat)}for(;l<d;)switch(n.hiddenDates.push({start:l.valueOf(),end:u.valueOf()}),r[s].repeat){case"daily":l.add(1,"days"),u.add(1,"days");break;case"weekly":l.add(1,"weeks"),u.add(1,"weeks");break;case"monthly":l.add(1,"months"),u.add(1,"months");break;case"yearly":l.add(1,"y"),u.add(1,"y");break;default:return void console.log("Wrong repeat format, allowed are: daily, weekly, monthly, yearly. Given:",r[s].repeat)}n.hiddenDates.push({start:l.valueOf(),end:u.valueOf()})}}t.removeDuplicates(n);var p=t.isHidden(n.range.start,n.hiddenDates),g=t.isHidden(n.range.end,n.hiddenDates),v=n.range.start,m=n.range.end;1==p.hidden&&(v=1==n.range.startToFront?p.startDate-1:p.endDate+1),1==g.hidden&&(m=1==n.range.endToFront?g.startDate-1:g.endDate+1),1!=p.hidden&&1!=g.hidden||n.range._applyRange(v,m)}},t.removeDuplicates=function(e){for(var t=e.hiddenDates,n=[],r=0;r<t.length;r++)for(var i=0;i<t.length;i++)r!=i&&1!=t[i].remove&&1!=t[r].remove&&(t[i].start>=t[r].start&&t[i].end<=t[r].end?t[i].remove=!0:t[i].start>=t[r].start&&t[i].start<=t[r].end?(t[r].end=t[i].end,t[i].remove=!0):t[i].end>=t[r].start&&t[i].end<=t[r].end&&(t[r].start=t[i].start,t[i].remove=!0));for(r=0;r<t.length;r++)!0!==t[r].remove&&n.push(t[r]);e.hiddenDates=n,e.hiddenDates.sort((function(e,t){return e.start-t.start}))},t.printDates=function(e){for(var t=0;t<e.length;t++)console.log(t,new Date(e[t].start),new Date(e[t].end),e[t].start,e[t].end,e[t].remove)},t.stepOverHiddenDates=function(e,t,n){for(var r=!1,i=t.current.valueOf(),o=0;o<t.hiddenDates.length;o++){var a=t.hiddenDates[o].start,s=t.hiddenDates[o].end;if(i>=a&&i<s){r=!0;break}}if(1==r&&i<t._end.valueOf()&&i!=n){var l=e(n),u=e(s);l.year()!=u.year()?t.switchedYear=!0:l.month()!=u.month()?t.switchedMonth=!0:l.dayOfYear()!=u.dayOfYear()&&(t.switchedDay=!0),t.current=u}},t.toScreen=function(e,n,r){var i;if(0==e.body.hiddenDates.length)return i=e.range.conversion(r),(n.valueOf()-i.offset)*i.scale;var o=t.isHidden(n,e.body.hiddenDates);1==o.hidden&&(n=o.startDate);var a=t.getHiddenDurationBetween(e.body.hiddenDates,e.range.start,e.range.end);if(n<e.range.start){i=e.range.conversion(r,a);var s=t.getHiddenDurationBeforeStart(e.body.hiddenDates,n,i.offset);return n=e.options.moment(n).toDate().valueOf(),n+=s,-(i.offset-n.valueOf())*i.scale}if(n>e.range.end){var l={start:e.range.start,end:n};return n=t.correctTimeForHidden(e.options.moment,e.body.hiddenDates,l,n),i=e.range.conversion(r,a),(n.valueOf()-i.offset)*i.scale}return n=t.correctTimeForHidden(e.options.moment,e.body.hiddenDates,e.range,n),i=e.range.conversion(r,a),(n.valueOf()-i.offset)*i.scale},t.toTime=function(e,n,r){if(0==e.body.hiddenDates.length){var i=e.range.conversion(r);return new Date(n/i.scale+i.offset)}var o=t.getHiddenDurationBetween(e.body.hiddenDates,e.range.start,e.range.end),a=(e.range.end-e.range.start-o)*n/r,s=t.getAccumulatedHiddenDuration(e.body.hiddenDates,e.range,a);return new Date(s+a+e.range.start)},t.getHiddenDurationBetween=function(e,t,n){for(var r=0,i=0;i<e.length;i++){var o=e[i].start,a=e[i].end;o>=t&&a<n&&(r+=a-o)}return r},t.getHiddenDurationBeforeStart=function(e,t,n){for(var r=0,i=0;i<e.length;i++){var o=e[i].start,a=e[i].end;o>=t&&a<=n&&(r+=a-o)}return r},t.correctTimeForHidden=function(e,n,r,i){return i=e(i).toDate().valueOf(),i-=t.getHiddenDurationBefore(e,n,r,i)},t.getHiddenDurationBefore=function(e,t,n,r){var i=0;r=e(r).toDate().valueOf();for(var o=0;o<t.length;o++){var a=t[o].start,s=t[o].end;a>=n.start&&s<n.end&&r>=s&&(i+=s-a)}return i},t.getAccumulatedHiddenDuration=function(e,t,n){for(var r=0,i=0,o=t.start,a=0;a<e.length;a++){var s=e[a].start,l=e[a].end;if(s>=t.start&&l<t.end){if(i+=s-o,o=l,i>=n)break;r+=l-s}}return r},t.snapAwayFromHidden=function(e,n,r,i){var o=t.isHidden(n,e);return 1==o.hidden?r<0?1==i?o.startDate-(o.endDate-n)-1:o.startDate-1:1==i?o.endDate+(n-o.startDate)+1:o.endDate+1:n},t.isHidden=function(e,t){for(var n=0;n<t.length;n++){var r=t[n].start,i=t[n].end;if(e>=r&&e<i)return{hidden:!0,startDate:r,endDate:i}}return{hidden:!1,startDate:r,endDate:i}}},function(e,t,n){t.onTouch=function(e,t){t.inputHandler=function(e){e.isFirst&&t(e)},e.on("hammer.input",t.inputHandler)},t.onRelease=function(e,t){return t.inputHandler=function(e){e.isFinal&&t(e)},e.on("hammer.input",t.inputHandler)},t.offTouch=function(e,t){e.off("hammer.input",t.inputHandler)},t.offRelease=t.offTouch,t.disablePreventDefaultVertically=function(e){return e.getTouchAction=function(){return["pan-y"]},e}},function(e,t,n){var r=o(n(6)),i=o(n(8));function o(e){return e&&e.__esModule?e:{default:e}}var a=n(10),s=n(2),l=n(9);function u(e,t,n){this.id=null,this.parent=null,this.data=e,this.dom=null,this.conversion=t||{},this.options=n||{},this.selected=!1,this.displayed=!1,this.groupShowing=!0,this.dirty=!0,this.top=null,this.right=null,this.left=null,this.width=null,this.height=null,this.editable=null,this._updateEditStatus()}u.prototype.stack=!0,u.prototype.select=function(){this.selected=!0,this.dirty=!0,this.displayed&&this.redraw()},u.prototype.unselect=function(){this.selected=!1,this.dirty=!0,this.displayed&&this.redraw()},u.prototype.setData=function(e){null!=e.group&&this.data.group!=e.group&&null!=this.parent&&this.parent.itemSet._moveToGroup(this,e.group),this.parent&&(this.parent.stackDirty=!0),null!=e.subgroup&&this.data.subgroup!=e.subgroup&&null!=this.parent&&this.parent.changeSubgroup(this,this.data.subgroup,e.subgroup),this.data=e,this._updateEditStatus(),this.dirty=!0,this.displayed&&this.redraw()},u.prototype.setParent=function(e){this.displayed?(this.hide(),this.parent=e,this.parent&&this.show()):this.parent=e},u.prototype.isVisible=function(e){return!1},u.prototype.show=function(){return!1},u.prototype.hide=function(){return!1},u.prototype.redraw=function(){},u.prototype.repositionX=function(){},u.prototype.repositionY=function(){},u.prototype._repaintDragCenter=function(){if(this.selected&&this.options.editable.updateTime&&!this.dom.dragCenter){var e=this,t=document.createElement("div");t.className="vis-drag-center",t.dragCenterItem=this;var n=new a(t);n.on("tap",(function(t){e.parent.itemSet.body.emitter.emit("click",{event:t,item:e.id})})),n.on("doubletap",(function(t){t.stopPropagation(),e.parent.itemSet._onUpdateItem(e),e.parent.itemSet.body.emitter.emit("doubleClick",{event:t,item:e.id})})),this.dom.box?this.dom.dragLeft?this.dom.box.insertBefore(t,this.dom.dragLeft):this.dom.box.appendChild(t):this.dom.point&&this.dom.point.appendChild(t),this.dom.dragCenter=t}else!this.selected&&this.dom.dragCenter&&(this.dom.dragCenter.parentNode&&this.dom.dragCenter.parentNode.removeChild(this.dom.dragCenter),this.dom.dragCenter=null)},u.prototype._repaintDeleteButton=function(e){var t=(this.options.editable.overrideItems||null==this.editable)&&this.options.editable.remove||!this.options.editable.overrideItems&&null!=this.editable&&this.editable.remove;if(this.selected&&t&&!this.dom.deleteButton){var n=this,r=document.createElement("div");this.options.rtl?r.className="vis-delete-rtl":r.className="vis-delete",r.title="Delete this item",new a(r).on("tap",(function(e){e.stopPropagation(),n.parent.removeFromDataSet(n)})),e.appendChild(r),this.dom.deleteButton=r}else!this.selected&&this.dom.deleteButton&&(this.dom.deleteButton.parentNode&&this.dom.deleteButton.parentNode.removeChild(this.dom.deleteButton),this.dom.deleteButton=null)},u.prototype._repaintOnItemUpdateTimeTooltip=function(e){if(this.options.tooltipOnItemUpdateTime){var t=(this.options.editable.updateTime||!0===this.data.editable)&&!1!==this.data.editable;if(this.selected&&t&&!this.dom.onItemUpdateTimeTooltip){var n=document.createElement("div");n.className="vis-onUpdateTime-tooltip",e.appendChild(n),this.dom.onItemUpdateTimeTooltip=n}else!this.selected&&this.dom.onItemUpdateTimeTooltip&&(this.dom.onItemUpdateTimeTooltip.parentNode&&this.dom.onItemUpdateTimeTooltip.parentNode.removeChild(this.dom.onItemUpdateTimeTooltip),this.dom.onItemUpdateTimeTooltip=null);if(this.dom.onItemUpdateTimeTooltip){this.dom.onItemUpdateTimeTooltip.style.visibility=this.parent.itemSet.touchParams.itemIsDragging?"visible":"hidden",this.options.rtl?this.dom.onItemUpdateTimeTooltip.style.right=this.dom.content.style.right:this.dom.onItemUpdateTimeTooltip.style.left=this.dom.content.style.left;var r,i=this.parent.itemSet.body.domProps.scrollTop;("top"==this.options.orientation.item?this.top:this.parent.height-this.top-this.height)+this.parent.top-50<-i?(this.dom.onItemUpdateTimeTooltip.style.bottom="",this.dom.onItemUpdateTimeTooltip.style.top=this.height+2+"px"):(this.dom.onItemUpdateTimeTooltip.style.top="",this.dom.onItemUpdateTimeTooltip.style.bottom=this.height+2+"px"),this.options.tooltipOnItemUpdateTime&&this.options.tooltipOnItemUpdateTime.template?r=this.options.tooltipOnItemUpdateTime.template.bind(this)(this.data):(r="start: "+l(this.data.start).format("MM/DD/YYYY hh:mm"),this.data.end&&(r+="<br> end: "+l(this.data.end).format("MM/DD/YYYY hh:mm"))),this.dom.onItemUpdateTimeTooltip.innerHTML=r}}},u.prototype._updateContents=function(e){var t,n,r,i,o=this.parent.itemSet.itemsData.get(this.id),a=this.dom.box||this.dom.point,s=a.getElementsByClassName("vis-item-visible-frame")[0];if(r=this.options.visibleFrameTemplate?(i=this.options.visibleFrameTemplate.bind(this))(o,a):"",s)if(r instanceof Object&&!(r instanceof Element))i(o,s);else if(this._contentToString(this.itemVisibleFrameContent)!==this._contentToString(r)){if(r instanceof Element)s.innerHTML="",s.appendChild(r);else if(null!=r)s.innerHTML=r;else if("background"!=this.data.type||void 0!==this.data.content)throw new Error('Property "content" missing in item '+this.id);this.itemVisibleFrameContent=r}if((t=this.options.template?(n=this.options.template.bind(this))(o,e,this.data):this.data.content)instanceof Object&&!(t instanceof Element))n(o,e);else if(this._contentToString(this.content)!==this._contentToString(t)){if(t instanceof Element)e.innerHTML="",e.appendChild(t);else if(null!=t)e.innerHTML=t;else if("background"!=this.data.type||void 0!==this.data.content)throw new Error('Property "content" missing in item '+this.id);this.content=t}},u.prototype._updateDataAttributes=function(e){if(this.options.dataAttributes&&this.options.dataAttributes.length>0){var t=[];if(Array.isArray(this.options.dataAttributes))t=this.options.dataAttributes;else{if("all"!=this.options.dataAttributes)return;t=(0,i.default)(this.data)}for(var n=0;n<t.length;n++){var r=t[n],o=this.data[r];null!=o?e.setAttribute("data-"+r,o):e.removeAttribute("data-"+r)}}},u.prototype._updateStyle=function(e){this.style&&(s.removeCssText(e,this.style),this.style=null),this.data.style&&(s.addCssText(e,this.data.style),this.style=this.data.style)},u.prototype._contentToString=function(e){return"string"==typeof e?e:e&&"outerHTML"in e?e.outerHTML:e},u.prototype._updateEditStatus=function(){this.options&&("boolean"==typeof this.options.editable?this.editable={updateTime:this.options.editable,updateGroup:this.options.editable,remove:this.options.editable}:"object"===(0,r.default)(this.options.editable)&&(this.editable={},s.selectiveExtend(["updateTime","updateGroup","remove"],this.editable,this.options.editable))),this.options&&this.options.editable&&!0===this.options.editable.overrideItems||this.data&&("boolean"==typeof this.data.editable?this.editable={updateTime:this.data.editable,updateGroup:this.data.editable,remove:this.data.editable}:"object"===(0,r.default)(this.data.editable)&&(this.editable={},s.selectiveExtend(["updateTime","updateGroup","remove"],this.editable,this.data.editable)))},u.prototype.getWidthLeft=function(){return 0},u.prototype.getWidthRight=function(){return 0},u.prototype.getTitle=function(){return this.data.title},e.exports=u},function(e,t){e.exports=function(e,t){return{enumerable:!(1&e),configurable:!(2&e),writable:!(4&e),value:t}}},function(e,t){var n=0,r=Math.random();e.exports=function(e){return"Symbol(".concat(void 0===e?"":e,")_",(++n+r).toString(36))}},function(e,t,n){var r=n(51);e.exports=function(e){return Object(r(e))}},function(e,t){t.f={}.propertyIsEnumerable},function(e,t,n){function r(e){this.delay=null,this.max=1/0,this._queue=[],this._timeout=null,this._extended=null,this.setOptions(e)}r.prototype.setOptions=function(e){e&&void 0!==e.delay&&(this.delay=e.delay),e&&void 0!==e.max&&(this.max=e.max),this._flushIfNeeded()},r.extend=function(e,t){var n=new r(t);if(void 0!==e.flush)throw new Error("Target object already has a property flush");e.flush=function(){n.flush()};var i=[{name:"flush",original:void 0}];if(t&&t.replace)for(var o=0;o<t.replace.length;o++){var a=t.replace[o];i.push({name:a,original:e[a]}),n.replace(e,a)}return n._extended={object:e,methods:i},n},r.prototype.destroy=function(){if(this.flush(),this._extended){for(var e=this._extended.object,t=this._extended.methods,n=0;n<t.length;n++){var r=t[n];r.original?e[r.name]=r.original:delete e[r.name]}this._extended=null}},r.prototype.replace=function(e,t){var n=this,r=e[t];if(!r)throw new Error("Method "+t+" undefined");e[t]=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];n.queue({args:e,fn:r,context:this})}},r.prototype.queue=function(e){"function"==typeof e?this._queue.push({fn:e}):this._queue.push(e),this._flushIfNeeded()},r.prototype._flushIfNeeded=function(){if(this._queue.length>this.max&&this.flush(),clearTimeout(this._timeout),this.queue.length>0&&"number"==typeof this.delay){var e=this;this._timeout=setTimeout((function(){e.flush()}),this.delay)}},r.prototype.flush=function(){for(;this._queue.length>0;){var e=this._queue.shift();e.fn.apply(e.context||e.fn,e.args||[])}},e.exports=r},function(e,t){function n(e){if(e)return function(e){for(var t in n.prototype)e[t]=n.prototype[t];return e}(e)}e.exports=n,n.prototype.on=n.prototype.addEventListener=function(e,t){return this._callbacks=this._callbacks||{},(this._callbacks[e]=this._callbacks[e]||[]).push(t),this},n.prototype.once=function(e,t){var n=this;function r(){n.off(e,r),t.apply(this,arguments)}return this._callbacks=this._callbacks||{},r.fn=t,this.on(e,r),this},n.prototype.off=n.prototype.removeListener=n.prototype.removeAllListeners=n.prototype.removeEventListener=function(e,t){if(this._callbacks=this._callbacks||{},0==arguments.length)return this._callbacks={},this;var n,r=this._callbacks[e];if(!r)return this;if(1==arguments.length)return delete this._callbacks[e],this;for(var i=0;i<r.length;i++)if((n=r[i])===t||n.fn===t){r.splice(i,1);break}return this},n.prototype.emit=function(e){this._callbacks=this._callbacks||{};var t=[].slice.call(arguments,1),n=this._callbacks[e];if(n)for(var r=0,i=(n=n.slice(0)).length;r<i;++r)n[r].apply(this,t);return this},n.prototype.listeners=function(e){return this._callbacks=this._callbacks||{},this._callbacks[e]||[]},n.prototype.hasListeners=function(e){return!!this.listeners(e).length}},function(e,t,n){var r,i=n(6),o=(r=i)&&r.__esModule?r:{default:r},a=n(2),s=n(16),l=n(66),u=n(36),c=n(9);function d(e,t){this.dom={foreground:null,lines:[],majorTexts:[],minorTexts:[],redundant:{lines:[],majorTexts:[],minorTexts:[]}},this.props={range:{start:0,end:0,minimumStep:0},lineTop:0},this.defaultOptions={orientation:{axis:"bottom"},showMinorLabels:!0,showMajorLabels:!0,maxMinorChars:7,format:l.FORMAT,moment:c,timeAxis:null},this.options=a.extend({},this.defaultOptions),this.body=e,this._create(),this.setOptions(t)}d.prototype=new s,d.prototype.setOptions=function(e){e&&(a.selectiveExtend(["showMinorLabels","showMajorLabels","maxMinorChars","hiddenDates","timeAxis","moment","rtl"],this.options,e),a.selectiveDeepExtend(["format"],this.options,e),"orientation"in e&&("string"==typeof e.orientation?this.options.orientation.axis=e.orientation:"object"===(0,o.default)(e.orientation)&&"axis"in e.orientation&&(this.options.orientation.axis=e.orientation.axis)),"locale"in e&&("function"==typeof c.locale?c.locale(e.locale):c.lang(e.locale)))},d.prototype._create=function(){this.dom.foreground=document.createElement("div"),this.dom.background=document.createElement("div"),this.dom.foreground.className="vis-time-axis vis-foreground",this.dom.background.className="vis-time-axis vis-background"},d.prototype.destroy=function(){this.dom.foreground.parentNode&&this.dom.foreground.parentNode.removeChild(this.dom.foreground),this.dom.background.parentNode&&this.dom.background.parentNode.removeChild(this.dom.background),this.body=null},d.prototype.redraw=function(){var e=this.props,t=this.dom.foreground,n=this.dom.background,r="top"==this.options.orientation.axis?this.body.dom.top:this.body.dom.bottom,i=t.parentNode!==r;this._calculateCharSize();var o=this.options.showMinorLabels&&"none"!==this.options.orientation.axis,a=this.options.showMajorLabels&&"none"!==this.options.orientation.axis;e.minorLabelHeight=o?e.minorCharHeight:0,e.majorLabelHeight=a?e.majorCharHeight:0,e.height=e.minorLabelHeight+e.majorLabelHeight,e.width=t.offsetWidth,e.minorLineHeight=this.body.domProps.root.height-e.majorLabelHeight-("top"==this.options.orientation.axis?this.body.domProps.bottom.height:this.body.domProps.top.height),e.minorLineWidth=1,e.majorLineHeight=e.minorLineHeight+e.majorLabelHeight,e.majorLineWidth=1;var s=t.nextSibling,l=n.nextSibling;return t.parentNode&&t.parentNode.removeChild(t),n.parentNode&&n.parentNode.removeChild(n),t.style.height=this.props.height+"px",this._repaintLabels(),s?r.insertBefore(t,s):r.appendChild(t),l?this.body.dom.backgroundVertical.insertBefore(n,l):this.body.dom.backgroundVertical.appendChild(n),this._isResized()||i},d.prototype._repaintLabels=function(){var e=this.options.orientation.axis,t=a.convert(this.body.range.start,"Number"),n=a.convert(this.body.range.end,"Number"),r=this.body.util.toTime((this.props.minorCharWidth||10)*this.options.maxMinorChars).valueOf(),i=r-u.getHiddenDurationBefore(this.options.moment,this.body.hiddenDates,this.body.range,r);i-=this.body.util.toTime(0).valueOf();var o=new l(new Date(t),new Date(n),i,this.body.hiddenDates,this.options);o.setMoment(this.options.moment),this.options.format&&o.setFormat(this.options.format),this.options.timeAxis&&o.setScale(this.options.timeAxis),this.step=o;var s,c,d,f,p,g=this.dom;g.redundant.lines=g.lines,g.redundant.majorTexts=g.majorTexts,g.redundant.minorTexts=g.minorTexts,g.lines=[],g.majorTexts=[],g.minorTexts=[];var v,m,y,b,C=0,A=void 0,_=0;for(o.start(),s=o.getCurrent(),d=this.body.util.toScreen(s);o.hasNext()&&_<1e3;){switch(_++,f=o.isMajor(),b=o.getClassName(),y=o.getLabelMinor(),c=d,o.next(),s=o.getCurrent(),o.isMajor(),v=C,C=(d=this.body.util.toScreen(s))-c,o.scale){case"week":p=!0;break;default:p=C>=.4*v}if(this.options.showMinorLabels&&p){var I=this._repaintMinorText(c,y,e,b);I.style.width=C+"px"}f&&this.options.showMajorLabels?(c>0&&(null==A&&(A=c),I=this._repaintMajorText(c,o.getLabelMajor(),e,b)),m=this._repaintMajorLine(c,C,e,b)):p?m=this._repaintMinorLine(c,C,e,b):m&&(m.style.width=parseInt(m.style.width)+C+"px")}if(1e3!==_||h||(console.warn("Something is wrong with the Timeline scale. Limited drawing of grid lines to 1000 lines."),h=!0),this.options.showMajorLabels){var x=this.body.util.toTime(0),w=o.getLabelMajor(x),M=w.length*(this.props.majorCharWidth||10)+10;(null==A||M<A)&&this._repaintMajorText(0,w,e,b)}a.forEach(this.dom.redundant,(function(e){for(;e.length;){var t=e.pop();t&&t.parentNode&&t.parentNode.removeChild(t)}}))},d.prototype._repaintMinorText=function(e,t,n,r){var i=this.dom.redundant.minorTexts.shift();if(!i){var o=document.createTextNode("");(i=document.createElement("div")).appendChild(o),this.dom.foreground.appendChild(i)}return this.dom.minorTexts.push(i),i.innerHTML=t,i.style.top="top"==n?this.props.majorLabelHeight+"px":"0",this.options.rtl?(i.style.left="",i.style.right=e+"px"):i.style.left=e+"px",i.className="vis-text vis-minor "+r,i},d.prototype._repaintMajorText=function(e,t,n,r){var i=this.dom.redundant.majorTexts.shift();if(!i){var o=document.createElement("div");(i=document.createElement("div")).appendChild(o),this.dom.foreground.appendChild(i)}return i.childNodes[0].innerHTML=t,i.className="vis-text vis-major "+r,i.style.top="top"==n?"0":this.props.minorLabelHeight+"px",this.options.rtl?(i.style.left="",i.style.right=e+"px"):i.style.left=e+"px",this.dom.majorTexts.push(i),i},d.prototype._repaintMinorLine=function(e,t,n,r){var i=this.dom.redundant.lines.shift();i||(i=document.createElement("div"),this.dom.background.appendChild(i)),this.dom.lines.push(i);var o=this.props;return i.style.top="top"==n?o.majorLabelHeight+"px":this.body.domProps.top.height+"px",i.style.height=o.minorLineHeight+"px",this.options.rtl?(i.style.left="",i.style.right=e-o.minorLineWidth/2+"px",i.className="vis-grid vis-vertical-rtl vis-minor "+r):(i.style.left=e-o.minorLineWidth/2+"px",i.className="vis-grid vis-vertical vis-minor "+r),i.style.width=t+"px",i},d.prototype._repaintMajorLine=function(e,t,n,r){var i=this.dom.redundant.lines.shift();i||(i=document.createElement("div"),this.dom.background.appendChild(i)),this.dom.lines.push(i);var o=this.props;return i.style.top="top"==n?"0":this.body.domProps.top.height+"px",this.options.rtl?(i.style.left="",i.style.right=e-o.majorLineWidth/2+"px",i.className="vis-grid vis-vertical-rtl vis-major "+r):(i.style.left=e-o.majorLineWidth/2+"px",i.className="vis-grid vis-vertical vis-major "+r),i.style.height=o.majorLineHeight+"px",i.style.width=t+"px",i},d.prototype._calculateCharSize=function(){this.dom.measureCharMinor||(this.dom.measureCharMinor=document.createElement("DIV"),this.dom.measureCharMinor.className="vis-text vis-minor vis-measure",this.dom.measureCharMinor.style.position="absolute",this.dom.measureCharMinor.appendChild(document.createTextNode("0")),this.dom.foreground.appendChild(this.dom.measureCharMinor)),this.props.minorCharHeight=this.dom.measureCharMinor.clientHeight,this.props.minorCharWidth=this.dom.measureCharMinor.clientWidth,this.dom.measureCharMajor||(this.dom.measureCharMajor=document.createElement("DIV"),this.dom.measureCharMajor.className="vis-text vis-major vis-measure",this.dom.measureCharMajor.style.position="absolute",this.dom.measureCharMajor.appendChild(document.createTextNode("0")),this.dom.foreground.appendChild(this.dom.measureCharMajor)),this.props.majorCharHeight=this.dom.measureCharMajor.clientHeight,this.props.majorCharWidth=this.dom.measureCharMajor.clientWidth};var h=!1;e.exports=d},function(e,t,n){var r=n(10),i=n(2),o=n(16),a=n(9),s=n(98);function l(e,t){this.body=e,this.defaultOptions={moment:a,locales:s,locale:"en",id:void 0,title:void 0},this.options=i.extend({},this.defaultOptions),t&&t.time?this.customTime=t.time:this.customTime=new Date,this.eventParams={},this.setOptions(t),this._create()}l.prototype=new o,l.prototype.setOptions=function(e){e&&i.selectiveExtend(["moment","locale","locales","id"],this.options,e)},l.prototype._create=function(){var e=document.createElement("div");e["custom-time"]=this,e.className="vis-custom-time "+(this.options.id||""),e.style.position="absolute",e.style.top="0px",e.style.height="100%",this.bar=e;var t=document.createElement("div");function n(e){this.body.range._onMouseWheel(e)}t.style.position="relative",t.style.top="0px",t.style.left="-10px",t.style.height="100%",t.style.width="20px",t.addEventListener?(t.addEventListener("mousewheel",n.bind(this),!1),t.addEventListener("DOMMouseScroll",n.bind(this),!1)):t.attachEvent("onmousewheel",n.bind(this)),e.appendChild(t),this.hammer=new r(t),this.hammer.on("panstart",this._onDragStart.bind(this)),this.hammer.on("panmove",this._onDrag.bind(this)),this.hammer.on("panend",this._onDragEnd.bind(this)),this.hammer.get("pan").set({threshold:5,direction:r.DIRECTION_HORIZONTAL})},l.prototype.destroy=function(){this.hide(),this.hammer.destroy(),this.hammer=null,this.body=null},l.prototype.redraw=function(){var e=this.body.dom.backgroundVertical;this.bar.parentNode!=e&&(this.bar.parentNode&&this.bar.parentNode.removeChild(this.bar),e.appendChild(this.bar));var t=this.body.util.toScreen(this.customTime),n=this.options.locales[this.options.locale];n||(this.warned||(console.log("WARNING: options.locales['"+this.options.locale+"'] not found. See http://visjs.org/docs/timeline/#Localization"),this.warned=!0),n=this.options.locales.en);var r=this.options.title;return void 0===r?r=(r=n.time+": "+this.options.moment(this.customTime).format("dddd, MMMM Do YYYY, H:mm:ss")).charAt(0).toUpperCase()+r.substring(1):"function"==typeof r&&(r=r.call(this.customTime)),this.bar.style.left=t+"px",this.bar.title=r,!1},l.prototype.hide=function(){this.bar.parentNode&&this.bar.parentNode.removeChild(this.bar)},l.prototype.setCustomTime=function(e){this.customTime=i.convert(e,"Date"),this.redraw()},l.prototype.getCustomTime=function(){return new Date(this.customTime.valueOf())},l.prototype.setCustomTitle=function(e){this.options.title=e},l.prototype._onDragStart=function(e){this.eventParams.dragging=!0,this.eventParams.customTime=this.customTime,e.stopPropagation()},l.prototype._onDrag=function(e){if(this.eventParams.dragging){var t=this.body.util.toScreen(this.eventParams.customTime)+e.deltaX,n=this.body.util.toTime(t);this.setCustomTime(n),this.body.emitter.emit("timechange",{id:this.options.id,time:new Date(this.customTime.valueOf()),event:e}),e.stopPropagation()}},l.prototype._onDragEnd=function(e){this.eventParams.dragging&&(this.body.emitter.emit("timechanged",{id:this.options.id,time:new Date(this.customTime.valueOf()),event:e}),e.stopPropagation())},l.customTimeFromTarget=function(e){for(var t=e.target;t;){if(t.hasOwnProperty("custom-time"))return t["custom-time"];t=t.parentNode}return null},e.exports=l},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=o(n(0)),i=o(n(1));function o(e){return e&&e.__esModule?e:{default:e}}var a=n(2),s=n(117).default,l=n(48).default,u=n(193).default,c=n(200).default,d=n(201).default,h=n(202).default,f=n(203).default,p=n(204).default,g=n(205).default,v=n(206).default,m=n(207).default,y=n(208).default,b=n(209).default,C=n(210).default,A=n(211).default,_=n(212).default,I=n(213).default,x=n(15).printStyle,w=function(){function e(t,n,i,o,l,u){(0,r.default)(this,e),this.options=a.bridgeObject(l),this.globalOptions=l,this.defaultOptions=u,this.body=n,this.edges=[],this.id=void 0,this.imagelist=i,this.grouplist=o,this.x=void 0,this.y=void 0,this.baseSize=this.options.size,this.baseFontSize=this.options.font.size,this.predefinedPosition=!1,this.selected=!1,this.hover=!1,this.labelModule=new s(this.body,this.options,!1),this.setOptions(t)}return(0,i.default)(e,[{key:"attachEdge",value:function(e){-1===this.edges.indexOf(e)&&this.edges.push(e)}},{key:"detachEdge",value:function(e){var t=this.edges.indexOf(e);-1!=t&&this.edges.splice(t,1)}},{key:"setOptions",value:function(t){var n=this.options.shape;if(t){if(void 0!==t.id&&(this.id=t.id),void 0===this.id)throw new Error("Node must have an id");e.checkMass(t,this.id),void 0!==t.x&&(null===t.x?(this.x=void 0,this.predefinedPosition=!1):(this.x=parseInt(t.x),this.predefinedPosition=!0)),void 0!==t.y&&(null===t.y?(this.y=void 0,this.predefinedPosition=!1):(this.y=parseInt(t.y),this.predefinedPosition=!0)),void 0!==t.size&&(this.baseSize=t.size),void 0!==t.value&&(t.value=parseFloat(t.value)),e.parseOptions(this.options,t,!0,this.globalOptions,this.grouplist);var r=[t,this.options,this.defaultOptions];return this.chooser=l.choosify("node",r),this._load_images(),this.updateLabelModule(t),this.updateShape(n),void 0!==t.hidden||void 0!==t.physics}}},{key:"_load_images",value:function(){if("circularImage"===this.options.shape||"image"===this.options.shape){if(void 0===this.options.image)throw new Error("Option image must be defined for node type '"+this.options.shape+"'");if(void 0===this.imagelist)throw new Error("Internal Error: No images provided");if("string"==typeof this.options.image)this.imageObj=this.imagelist.load(this.options.image,this.options.brokenImage,this.id);else{if(void 0===this.options.image.unselected)throw new Error("No unselected image provided");this.imageObj=this.imagelist.load(this.options.image.unselected,this.options.brokenImage,this.id),void 0!==this.options.image.selected?this.imageObjAlt=this.imagelist.load(this.options.image.selected,this.options.brokenImage,this.id):this.imageObjAlt=void 0}}}},{key:"getFormattingValues",value:function(){var e={color:this.options.color.background,borderWidth:this.options.borderWidth,borderColor:this.options.color.border,size:this.options.size,borderDashes:this.options.shapeProperties.borderDashes,borderRadius:this.options.shapeProperties.borderRadius,shadow:this.options.shadow.enabled,shadowColor:this.options.shadow.color,shadowSize:this.options.shadow.size,shadowX:this.options.shadow.x,shadowY:this.options.shadow.y};return this.selected||this.hover?!0===this.chooser?this.selected?(e.borderWidth*=2,e.color=this.options.color.highlight.background,e.borderColor=this.options.color.highlight.border,e.shadow=this.options.shadow.enabled):this.hover&&(e.color=this.options.color.hover.background,e.borderColor=this.options.color.hover.border,e.shadow=this.options.shadow.enabled):"function"==typeof this.chooser&&(this.chooser(e,this.options.id,this.selected,this.hover),!1===e.shadow&&(e.shadowColor===this.options.shadow.color&&e.shadowSize===this.options.shadow.size&&e.shadowX===this.options.shadow.x&&e.shadowY===this.options.shadow.y||(e.shadow=!0))):e.shadow=this.options.shadow.enabled,e}},{key:"updateLabelModule",value:function(t){void 0!==this.options.label&&null!==this.options.label||(this.options.label=""),e.updateGroupOptions(this.options,t,this.grouplist);var n=this.grouplist.get(this.options.group,!1),r=[t,this.options,n,this.globalOptions,this.defaultOptions];this.labelModule.update(this.options,r),void 0!==this.labelModule.baseSize&&(this.baseFontSize=this.labelModule.baseSize)}},{key:"updateShape",value:function(e){if(e===this.options.shape&&this.shape)this.shape.setOptions(this.options,this.imageObj,this.imageObjAlt);else switch(this.options.shape){case"box":this.shape=new u(this.options,this.body,this.labelModule);break;case"circle":this.shape=new c(this.options,this.body,this.labelModule);break;case"circularImage":this.shape=new d(this.options,this.body,this.labelModule,this.imageObj,this.imageObjAlt);break;case"database":this.shape=new h(this.options,this.body,this.labelModule);break;case"diamond":this.shape=new f(this.options,this.body,this.labelModule);break;case"dot":this.shape=new p(this.options,this.body,this.labelModule);break;case"ellipse":this.shape=new g(this.options,this.body,this.labelModule);break;case"icon":this.shape=new v(this.options,this.body,this.labelModule);break;case"image":this.shape=new m(this.options,this.body,this.labelModule,this.imageObj,this.imageObjAlt);break;case"square":this.shape=new y(this.options,this.body,this.labelModule);break;case"hexagon":this.shape=new b(this.options,this.body,this.labelModule);break;case"star":this.shape=new C(this.options,this.body,this.labelModule);break;case"text":this.shape=new A(this.options,this.body,this.labelModule);break;case"triangle":this.shape=new _(this.options,this.body,this.labelModule);break;case"triangleDown":this.shape=new I(this.options,this.body,this.labelModule);break;default:this.shape=new g(this.options,this.body,this.labelModule)}this.needsRefresh()}},{key:"select",value:function(){this.selected=!0,this.needsRefresh()}},{key:"unselect",value:function(){this.selected=!1,this.needsRefresh()}},{key:"needsRefresh",value:function(){this.shape.refreshNeeded=!0}},{key:"getTitle",value:function(){return this.options.title}},{key:"distanceToBorder",value:function(e,t){return this.shape.distanceToBorder(e,t)}},{key:"isFixed",value:function(){return this.options.fixed.x&&this.options.fixed.y}},{key:"isSelected",value:function(){return this.selected}},{key:"getValue",value:function(){return this.options.value}},{key:"getLabelSize",value:function(){return this.labelModule.size()}},{key:"setValueRange",value:function(e,t,n){if(void 0!==this.options.value){var r=this.options.scaling.customScalingFunction(e,t,n,this.options.value),i=this.options.scaling.max-this.options.scaling.min;if(!0===this.options.scaling.label.enabled){var o=this.options.scaling.label.max-this.options.scaling.label.min;this.options.font.size=this.options.scaling.label.min+r*o}this.options.size=this.options.scaling.min+r*i}else this.options.size=this.baseSize,this.options.font.size=this.baseFontSize;this.updateLabelModule()}},{key:"draw",value:function(e){var t=this.getFormattingValues();this.shape.draw(e,this.x,this.y,this.selected,this.hover,t)}},{key:"updateBoundingBox",value:function(e){this.shape.updateBoundingBox(this.x,this.y,e)}},{key:"resize",value:function(e){var t=this.getFormattingValues();this.shape.resize(e,this.selected,this.hover,t)}},{key:"getItemsOnPoint",value:function(e){var t=[];return this.labelModule.visible()&&l.pointInRect(this.labelModule.getSize(),e)&&t.push({nodeId:this.id,labelId:0}),l.pointInRect(this.shape.boundingBox,e)&&t.push({nodeId:this.id}),t}},{key:"isOverlappingWith",value:function(e){return this.shape.left<e.right&&this.shape.left+this.shape.width>e.left&&this.shape.top<e.bottom&&this.shape.top+this.shape.height>e.top}},{key:"isBoundingBoxOverlappingWith",value:function(e){return this.shape.boundingBox.left<e.right&&this.shape.boundingBox.right>e.left&&this.shape.boundingBox.top<e.bottom&&this.shape.boundingBox.bottom>e.top}}],[{key:"updateGroupOptions",value:function(e,t,n){if(void 0!==n){var r=e.group;if(void 0!==t&&void 0!==t.group&&r!==t.group)throw new Error("updateGroupOptions: group values in options don't match.");if("number"==typeof r||"string"==typeof r&&""!=r){var i=n.get(r);a.selectiveNotDeepExtend(["font"],e,i),e.color=a.parseColor(e.color)}}}},{key:"parseOptions",value:function(t,n){var r=arguments.length>2&&void 0!==arguments[2]&&arguments[2],i=arguments.length>3&&void 0!==arguments[3]?arguments[3]:{},o=arguments[4],s=["color","fixed","shadow"];if(a.selectiveNotDeepExtend(s,t,n,r),e.checkMass(n),a.mergeOptions(t,n,"shadow",i),void 0!==n.color&&null!==n.color){var l=a.parseColor(n.color);a.fillIfDefined(t.color,l)}else!0===r&&null===n.color&&(t.color=a.bridgeObject(i.color));void 0!==n.fixed&&null!==n.fixed&&("boolean"==typeof n.fixed?(t.fixed.x=n.fixed,t.fixed.y=n.fixed):(void 0!==n.fixed.x&&"boolean"==typeof n.fixed.x&&(t.fixed.x=n.fixed.x),void 0!==n.fixed.y&&"boolean"==typeof n.fixed.y&&(t.fixed.y=n.fixed.y))),!0===r&&null===n.font&&(t.font=a.bridgeObject(i.font)),e.updateGroupOptions(t,n,o),void 0!==n.scaling&&a.mergeOptions(t.scaling,n.scaling,"label",i.scaling)}},{key:"checkMass",value:function(e,t){if(void 0!==e.mass&&e.mass<=0){var n="";void 0!==t&&(n=" in node id: "+t),console.log("%cNegative or zero mass disallowed"+n+", setting mass to 1.",x),e.mass=1}}}]),e}();t.default=w},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=a(n(6)),i=a(n(0)),o=a(n(1));function a(e){return e&&e.__esModule?e:{default:e}}var s=n(2),l=function(){function e(){(0,i.default)(this,e)}return(0,o.default)(e,null,[{key:"choosify",value:function(e,t){var n=["node","edge","label"],i=!0,o=s.topMost(t,"chosen");if("boolean"==typeof o)i=o;else if("object"===(void 0===o?"undefined":(0,r.default)(o))){if(-1===n.indexOf(e))throw new Error("choosify: subOption '"+e+"' should be one of '"+n.join("', '")+"'");var a=s.topMost(t,["chosen",e]);"boolean"!=typeof a&&"function"!=typeof a||(i=a)}return i}},{key:"pointInRect",value:function(e,t,n){if(e.width<=0||e.height<=0)return!1;if(void 0!==n){var r={x:t.x-n.x,y:t.y-n.y};if(0!==n.angle){var i=-n.angle;t={x:Math.cos(i)*r.x-Math.sin(i)*r.y,y:Math.sin(i)*r.x+Math.cos(i)*r.y}}else t=r}var o=e.x+e.width,a=e.y+e.width;return e.left<t.x&&o>t.x&&e.top<t.y&&a>t.y}},{key:"isValidLabel",value:function(e){return"string"==typeof e&&""!==e}}]),e}();t.default=l},function(e,t,n){n(125);for(var r=n(18),i=n(26),o=n(31),a=n(13)("toStringTag"),s="CSSRuleList,CSSStyleDeclaration,CSSValueList,ClientRectList,DOMRectList,DOMStringList,DOMTokenList,DataTransferItemList,FileList,HTMLAllCollection,HTMLCollection,HTMLFormElement,HTMLSelectElement,MediaList,MimeTypeArray,NamedNodeMap,NodeList,PaintRequestList,Plugin,PluginArray,SVGLengthList,SVGNumberList,SVGPathSegList,SVGPointList,SVGStringList,SVGTransformList,SourceBufferList,StyleSheetList,TextTrackCueList,TextTrackList,TouchList".split(","),l=0;l<s.length;l++){var u=s[l],c=r[u],d=c&&c.prototype;d&&!d[a]&&i(d,a,u),o[u]=o.Array}},function(e,t){var n={}.toString;e.exports=function(e){return n.call(e).slice(8,-1)}},function(e,t){e.exports=function(e){if(null==e)throw TypeError("Can't call method on  "+e);return e}},function(e,t){e.exports=!0},function(e,t,n){var r=n(32);e.exports=function(e,t){if(!r(e))return e;var n,i;if(t&&"function"==typeof(n=e.toString)&&!r(i=n.call(e)))return i;if("function"==typeof(n=e.valueOf)&&!r(i=n.call(e)))return i;if(!t&&"function"==typeof(n=e.toString)&&!r(i=n.call(e)))return i;throw TypeError("Can't convert object to primitive value")}},function(e,t,n){var r=n(27),i=n(130),o=n(58),a=n(56)("IE_PROTO"),s=function(){},l=function(){var e,t=n(82)("iframe"),r=o.length;for(t.style.display="none",n(134).appendChild(t),t.src="javascript:",(e=t.contentWindow.document).open(),e.write("<script>document.F=Object<\/script>"),e.close(),l=e.F;r--;)delete l.prototype[o[r]];return l()};e.exports=Object.create||function(e,t){var n;return null!==e?(s.prototype=r(e),n=new s,s.prototype=null,n[a]=e):n=l(),void 0===t?n:i(n,t)}},function(e,t){var n=Math.ceil,r=Math.floor;e.exports=function(e){return isNaN(e=+e)?0:(e>0?r:n)(e)}},function(e,t,n){var r=n(57)("keys"),i=n(40);e.exports=function(e){return r[e]||(r[e]=i(e))}},function(e,t,n){var r=n(18),i=r["__core-js_shared__"]||(r["__core-js_shared__"]={});e.exports=function(e){return i[e]||(i[e]={})}},function(e,t){e.exports="constructor,hasOwnProperty,isPrototypeOf,propertyIsEnumerable,toLocaleString,toString,valueOf".split(",")},function(e,t,n){var r=n(20).f,i=n(22),o=n(13)("toStringTag");e.exports=function(e,t,n){e&&!i(e=n?e:e.prototype,o)&&r(e,o,{configurable:!0,value:t})}},function(e,t,n){var r=n(135)(!0);n(79)(String,"String",(function(e){this._t=String(e),this._i=0}),(function(){var e,t=this._t,n=this._i;return n>=t.length?{value:void 0,done:!0}:(e=r(t,n),this._i+=e.length,{value:e,done:!1})}))},function(e,t,n){t.f=n(13)},function(e,t,n){var r=n(18),i=n(7),o=n(52),a=n(61),s=n(20).f;e.exports=function(e){var t=i.Symbol||(i.Symbol=o?{}:r.Symbol||{});"_"==e.charAt(0)||e in t||s(t,e,{value:a.f(e)})}},function(e,t){t.f=Object.getOwnPropertySymbols},function(e,t,n){var r=a(n(8)),i=a(n(19)),o=a(n(6));function a(e){return e&&e.__esModule?e:{default:e}}var s=n(2),l=n(9),u=n(16),c=n(36);function d(e,t){var n=l().hours(0).minutes(0).seconds(0).milliseconds(0),r=n.clone().add(-3,"days").valueOf(),i=n.clone().add(3,"days").valueOf();this.millisecondsPerPixelCache=void 0,void 0===t?(this.start=r,this.end=i):(this.start=t.start||r,this.end=t.end||i),this.rolling=!1,this.body=e,this.deltaDifference=0,this.scaleOffset=0,this.startToFront=!1,this.endToFront=!0,this.defaultOptions={rtl:!1,start:null,end:null,moment:l,direction:"horizontal",moveable:!0,zoomable:!0,min:null,max:null,zoomMin:10,zoomMax:31536e10,rollingMode:{follow:!1,offset:.5}},this.options=s.extend({},this.defaultOptions),this.props={touch:{}},this.animationTimer=null,this.body.emitter.on("panstart",this._onDragStart.bind(this)),this.body.emitter.on("panmove",this._onDrag.bind(this)),this.body.emitter.on("panend",this._onDragEnd.bind(this)),this.body.emitter.on("mousewheel",this._onMouseWheel.bind(this)),this.body.emitter.on("touch",this._onTouch.bind(this)),this.body.emitter.on("pinch",this._onPinch.bind(this)),this.body.dom.rollingModeBtn.addEventListener("click",this.startRolling.bind(this)),this.setOptions(t)}function h(e){if("horizontal"!=e&&"vertical"!=e)throw new TypeError('Unknown direction "'+e+'". Choose "horizontal" or "vertical".')}d.prototype=new u,d.prototype.setOptions=function(e){e&&(s.selectiveExtend(["animation","direction","min","max","zoomMin","zoomMax","moveable","zoomable","moment","activate","hiddenDates","zoomKey","rtl","showCurrentTime","rollingMode","horizontalScroll"],this.options,e),e.rollingMode&&e.rollingMode.follow&&this.startRolling(),("start"in e||"end"in e)&&this.setRange(e.start,e.end))},d.prototype.startRolling=function(){var e=this;!function t(){e.stopRolling(),e.rolling=!0;var n=e.end-e.start,r=s.convert(new Date,"Date").valueOf(),i=r-n*e.options.rollingMode.offset,o=r+n*(1-e.options.rollingMode.offset);e.setRange(i,o,{animation:!1}),(n=1/e.conversion(e.body.domProps.center.width).scale/10)<30&&(n=30),n>1e3&&(n=1e3),e.body.dom.rollingModeBtn.style.visibility="hidden",e.currentTimeTimer=setTimeout(t,n)}()},d.prototype.stopRolling=function(){void 0!==this.currentTimeTimer&&(clearTimeout(this.currentTimeTimer),this.rolling=!1,this.body.dom.rollingModeBtn.style.visibility="visible")},d.prototype.setRange=function(e,t,n,a,l){n||(n={}),!0!==n.byUser&&(n.byUser=!1);var u=this,d=null!=e?s.convert(e,"Date").valueOf():null,h=null!=t?s.convert(t,"Date").valueOf():null;if(this._cancelAnimation(),this.millisecondsPerPixelCache=void 0,n.animation){var f=this.start,p=this.end,g="object"===(0,o.default)(n.animation)&&"duration"in n.animation?n.animation.duration:500,v="object"===(0,o.default)(n.animation)&&"easingFunction"in n.animation?n.animation.easingFunction:"easeInOutQuad",m=s.easingFunctions[v];if(!m)throw new Error("Unknown easing function "+(0,i.default)(v)+". Choose from: "+(0,r.default)(s.easingFunctions).join(", "));var y=(new Date).valueOf(),b=!1;return function e(){if(!u.props.touch.dragging){var t=(new Date).valueOf()-y,r=m(t/g),i=t>g,o=i||null===d?d:f+(d-f)*r,s=i||null===h?h:p+(h-p)*r;C=u._applyRange(o,s),c.updateHiddenDates(u.options.moment,u.body,u.options.hiddenDates),b=b||C;var v={start:new Date(u.start),end:new Date(u.end),byUser:n.byUser,event:n.event};if(l&&l(r,C,i),C&&u.body.emitter.emit("rangechange",v),i){if(b&&(u.body.emitter.emit("rangechanged",v),a))return a()}else u.animationTimer=setTimeout(e,20)}}()}var C=this._applyRange(d,h);if(c.updateHiddenDates(this.options.moment,this.body,this.options.hiddenDates),C){var A={start:new Date(this.start),end:new Date(this.end),byUser:n.byUser,event:n.event};if(this.body.emitter.emit("rangechange",A),clearTimeout(u.timeoutID),u.timeoutID=setTimeout((function(){u.body.emitter.emit("rangechanged",A)}),200),a)return a()}},d.prototype.getMillisecondsPerPixel=function(){return void 0===this.millisecondsPerPixelCache&&(this.millisecondsPerPixelCache=(this.end-this.start)/this.body.dom.center.clientWidth),this.millisecondsPerPixelCache},d.prototype._cancelAnimation=function(){this.animationTimer&&(clearTimeout(this.animationTimer),this.animationTimer=null)},d.prototype._applyRange=function(e,t){var n,r=null!=e?s.convert(e,"Date").valueOf():this.start,i=null!=t?s.convert(t,"Date").valueOf():this.end,o=null!=this.options.max?s.convert(this.options.max,"Date").valueOf():null,a=null!=this.options.min?s.convert(this.options.min,"Date").valueOf():null;if(isNaN(r)||null===r)throw new Error('Invalid start "'+e+'"');if(isNaN(i)||null===i)throw new Error('Invalid end "'+t+'"');if(i<r&&(i=r),null!==a&&r<a&&(r+=n=a-r,i+=n,null!=o&&i>o&&(i=o)),null!==o&&i>o&&(r-=n=i-o,i-=n,null!=a&&r<a&&(r=a)),null!==this.options.zoomMin){var l=parseFloat(this.options.zoomMin);l<0&&(l=0),i-r<l&&(this.end-this.start===l&&r>=this.start-.5&&i<=this.end?(r=this.start,i=this.end):(r-=(n=l-(i-r))/2,i+=n/2))}if(null!==this.options.zoomMax){var u=parseFloat(this.options.zoomMax);u<0&&(u=0),i-r>u&&(this.end-this.start===u&&r<this.start&&i>this.end?(r=this.start,i=this.end):(r+=(n=i-r-u)/2,i-=n/2))}var c=this.start!=r||this.end!=i;return r>=this.start&&r<=this.end||i>=this.start&&i<=this.end||this.start>=r&&this.start<=i||this.end>=r&&this.end<=i||this.body.emitter.emit("checkRangedItems"),this.start=r,this.end=i,c},d.prototype.getRange=function(){return{start:this.start,end:this.end}},d.prototype.conversion=function(e,t){return d.conversion(this.start,this.end,e,t)},d.conversion=function(e,t,n,r){return void 0===r&&(r=0),0!=n&&t-e!=0?{offset:e,scale:n/(t-e-r)}:{offset:0,scale:1}},d.prototype._onDragStart=function(e){this.deltaDifference=0,this.previousDelta=0,this.options.moveable&&this._isInsideRange(e)&&this.props.touch.allowDragging&&(this.stopRolling(),this.props.touch.start=this.start,this.props.touch.end=this.end,this.props.touch.dragging=!0,this.body.dom.root&&(this.body.dom.root.style.cursor="move"))},d.prototype._onDrag=function(e){if(e&&this.props.touch.dragging&&this.options.moveable&&this.props.touch.allowDragging){var t=this.options.direction;h(t);var n="horizontal"==t?e.deltaX:e.deltaY;n-=this.deltaDifference;var r=this.props.touch.end-this.props.touch.start;r-=c.getHiddenDurationBetween(this.body.hiddenDates,this.start,this.end);var i,o="horizontal"==t?this.body.domProps.center.width:this.body.domProps.center.height;i=this.options.rtl?n/o*r:-n/o*r;var a=this.props.touch.start+i,s=this.props.touch.end+i,l=c.snapAwayFromHidden(this.body.hiddenDates,a,this.previousDelta-n,!0),u=c.snapAwayFromHidden(this.body.hiddenDates,s,this.previousDelta-n,!0);if(l!=a||u!=s)return this.deltaDifference+=n,this.props.touch.start=l,this.props.touch.end=u,void this._onDrag(e);this.previousDelta=n,this._applyRange(a,s);var d=new Date(this.start),f=new Date(this.end);this.body.emitter.emit("rangechange",{start:d,end:f,byUser:!0,event:e}),this.body.emitter.emit("panmove")}},d.prototype._onDragEnd=function(e){this.props.touch.dragging&&this.options.moveable&&this.props.touch.allowDragging&&(this.props.touch.dragging=!1,this.body.dom.root&&(this.body.dom.root.style.cursor="auto"),this.body.emitter.emit("rangechanged",{start:new Date(this.start),end:new Date(this.end),byUser:!0,event:e}))},d.prototype._onMouseWheel=function(e){var t=0;if(e.wheelDelta?t=e.wheelDelta/120:e.detail&&(t=-e.detail/3),!(this.options.zoomKey&&!e[this.options.zoomKey]&&this.options.zoomable||!this.options.zoomable&&this.options.moveable)&&this.options.zoomable&&this.options.moveable&&this._isInsideRange(e)&&t){var n,r;if(n=t<0?1-t/5:1/(1+t/5),this.rolling)r=this.start+(this.end-this.start)*this.options.rollingMode.offset;else{var i=this.getPointer({x:e.clientX,y:e.clientY},this.body.dom.center);r=this._pointerToDate(i)}this.zoom(n,r,t,e),e.preventDefault()}},d.prototype._onTouch=function(e){this.props.touch.start=this.start,this.props.touch.end=this.end,this.props.touch.allowDragging=!0,this.props.touch.center=null,this.scaleOffset=0,this.deltaDifference=0,s.preventDefault(e)},d.prototype._onPinch=function(e){if(this.options.zoomable&&this.options.moveable){s.preventDefault(e),this.props.touch.allowDragging=!1,this.props.touch.center||(this.props.touch.center=this.getPointer(e.center,this.body.dom.center)),this.stopRolling();var t=1/(e.scale+this.scaleOffset),n=this._pointerToDate(this.props.touch.center),r=c.getHiddenDurationBetween(this.body.hiddenDates,this.start,this.end),i=c.getHiddenDurationBefore(this.options.moment,this.body.hiddenDates,this,n),o=r-i,a=n-i+(this.props.touch.start-(n-i))*t,l=n+o+(this.props.touch.end-(n+o))*t;this.startToFront=1-t<=0,this.endToFront=t-1<=0;var u=c.snapAwayFromHidden(this.body.hiddenDates,a,1-t,!0),d=c.snapAwayFromHidden(this.body.hiddenDates,l,t-1,!0);u==a&&d==l||(this.props.touch.start=u,this.props.touch.end=d,this.scaleOffset=1-e.scale,a=u,l=d);var h={animation:!1,byUser:!0,event:e};this.setRange(a,l,h),this.startToFront=!1,this.endToFront=!0}},d.prototype._isInsideRange=function(e){var t,n=e.center?e.center.x:e.clientX;t=this.options.rtl?n-s.getAbsoluteLeft(this.body.dom.centerContainer):s.getAbsoluteRight(this.body.dom.centerContainer)-n;var r=this.body.util.toTime(t);return r>=this.start&&r<=this.end},d.prototype._pointerToDate=function(e){var t,n=this.options.direction;if(h(n),"horizontal"==n)return this.body.util.toTime(e.x).valueOf();var r=this.body.domProps.center.height;return t=this.conversion(r),e.y/t.scale+t.offset},d.prototype.getPointer=function(e,t){return this.options.rtl?{x:s.getAbsoluteRight(t)-e.x,y:e.y-s.getAbsoluteTop(t)}:{x:e.x-s.getAbsoluteLeft(t),y:e.y-s.getAbsoluteTop(t)}},d.prototype.zoom=function(e,t,n,r){null==t&&(t=(this.start+this.end)/2);var i=c.getHiddenDurationBetween(this.body.hiddenDates,this.start,this.end),o=c.getHiddenDurationBefore(this.options.moment,this.body.hiddenDates,this,t),a=i-o,s=t-o+(this.start-(t-o))*e,l=t+a+(this.end-(t+a))*e;this.startToFront=!(n>0),this.endToFront=!(-n>0);var u=c.snapAwayFromHidden(this.body.hiddenDates,s,n,!0),d=c.snapAwayFromHidden(this.body.hiddenDates,l,-n,!0);u==s&&d==l||(s=u,l=d);var h={animation:!1,byUser:!0,event:r};this.setRange(s,l,h),this.startToFront=!1,this.endToFront=!0},d.prototype.move=function(e){var t=this.end-this.start,n=this.start+t*e,r=this.end+t*e;this.start=n,this.end=r},d.prototype.moveTo=function(e){var t=(this.start+this.end)/2-e,n=this.start-t,r=this.end-t;this.setRange(n,r,{animation:!1,byUser:!0,event:null})},e.exports=d},function(e,t,n){var r=o(n(19)),i=o(n(6));function o(e){return e&&e.__esModule?e:{default:e}}var a=n(44),s=n(10),l=n(37),u=n(2),c=n(45),d=n(97),h=n(36),f=n(46);function p(){}a(p.prototype),p.prototype._create=function(e){this.dom={},this.dom.container=e,this.dom.root=document.createElement("div"),this.dom.background=document.createElement("div"),this.dom.backgroundVertical=document.createElement("div"),this.dom.backgroundHorizontal=document.createElement("div"),this.dom.centerContainer=document.createElement("div"),this.dom.leftContainer=document.createElement("div"),this.dom.rightContainer=document.createElement("div"),this.dom.center=document.createElement("div"),this.dom.left=document.createElement("div"),this.dom.right=document.createElement("div"),this.dom.top=document.createElement("div"),this.dom.bottom=document.createElement("div"),this.dom.shadowTop=document.createElement("div"),this.dom.shadowBottom=document.createElement("div"),this.dom.shadowTopLeft=document.createElement("div"),this.dom.shadowBottomLeft=document.createElement("div"),this.dom.shadowTopRight=document.createElement("div"),this.dom.shadowBottomRight=document.createElement("div"),this.dom.rollingModeBtn=document.createElement("div"),this.dom.root.className="vis-timeline",this.dom.background.className="vis-panel vis-background",this.dom.backgroundVertical.className="vis-panel vis-background vis-vertical",this.dom.backgroundHorizontal.className="vis-panel vis-background vis-horizontal",this.dom.centerContainer.className="vis-panel vis-center",this.dom.leftContainer.className="vis-panel vis-left",this.dom.rightContainer.className="vis-panel vis-right",this.dom.top.className="vis-panel vis-top",this.dom.bottom.className="vis-panel vis-bottom",this.dom.left.className="vis-content",this.dom.center.className="vis-content",this.dom.right.className="vis-content",this.dom.shadowTop.className="vis-shadow vis-top",this.dom.shadowBottom.className="vis-shadow vis-bottom",this.dom.shadowTopLeft.className="vis-shadow vis-top",this.dom.shadowBottomLeft.className="vis-shadow vis-bottom",this.dom.shadowTopRight.className="vis-shadow vis-top",this.dom.shadowBottomRight.className="vis-shadow vis-bottom",this.dom.rollingModeBtn.className="vis-rolling-mode-btn",this.dom.root.appendChild(this.dom.background),this.dom.root.appendChild(this.dom.backgroundVertical),this.dom.root.appendChild(this.dom.backgroundHorizontal),this.dom.root.appendChild(this.dom.centerContainer),this.dom.root.appendChild(this.dom.leftContainer),this.dom.root.appendChild(this.dom.rightContainer),this.dom.root.appendChild(this.dom.top),this.dom.root.appendChild(this.dom.bottom),this.dom.root.appendChild(this.dom.bottom),this.dom.root.appendChild(this.dom.rollingModeBtn),this.dom.centerContainer.appendChild(this.dom.center),this.dom.leftContainer.appendChild(this.dom.left),this.dom.rightContainer.appendChild(this.dom.right),this.dom.centerContainer.appendChild(this.dom.shadowTop),this.dom.centerContainer.appendChild(this.dom.shadowBottom),this.dom.leftContainer.appendChild(this.dom.shadowTopLeft),this.dom.leftContainer.appendChild(this.dom.shadowBottomLeft),this.dom.rightContainer.appendChild(this.dom.shadowTopRight),this.dom.rightContainer.appendChild(this.dom.shadowBottomRight),this.props={root:{},background:{},centerContainer:{},leftContainer:{},rightContainer:{},center:{},left:{},right:{},top:{},bottom:{},border:{},scrollTop:0,scrollTopMin:0},this.on("rangechange",function(){!0===this.initialDrawDone&&this._redraw()}.bind(this)),this.on("rangechanged",function(){this.initialRangeChangeDone||(this.initialRangeChangeDone=!0)}.bind(this)),this.on("touch",this._onTouch.bind(this)),this.on("panmove",this._onDrag.bind(this));var t=this;this._origRedraw=this._redraw.bind(this),this._redraw=u.throttle(this._origRedraw),this.on("_change",(function(e){t.itemSet&&t.itemSet.initialItemSetDrawn&&e&&1==e.queue?t._redraw():t._origRedraw()})),this.hammer=new s(this.dom.root);var n=this.hammer.get("pinch").set({enable:!0});function r(e){this.isActive()&&this.emit("mousewheel",e);var t=0,n=0;if("detail"in e&&(n=-1*e.detail),"wheelDelta"in e&&(n=e.wheelDelta),"wheelDeltaY"in e&&(n=e.wheelDeltaY),"wheelDeltaX"in e&&(t=-1*e.wheelDeltaX),"axis"in e&&e.axis===e.HORIZONTAL_AXIS&&(t=-1*n,n=0),"deltaY"in e&&(n=-1*e.deltaY),"deltaX"in e&&(t=e.deltaX),this.options.zoomKey&&!e[this.options.zoomKey])if(e.preventDefault(),this.options.verticalScroll&&Math.abs(n)>=Math.abs(t)){var r=this.props.scrollTop+n;this.isActive()&&(this._setScrollTop(r),this._redraw(),this.emit("scroll",e))}else if(this.options.horizontalScroll){var i=(Math.abs(t)>=Math.abs(n)?t:n)/120*(this.range.end-this.range.start)/20,o=this.range.start+i,a=this.range.end+i,s={animation:!1,byUser:!0,event:e};this.range.setRange(o,a,s)}}function i(e){if(t.options.verticalScroll&&(e.preventDefault(),t.isActive())){var n=-e.target.scrollTop;t._setScrollTop(n),t._redraw(),t.emit("scrollSide",e)}}l.disablePreventDefaultVertically(n),this.hammer.get("pan").set({threshold:5,direction:s.DIRECTION_HORIZONTAL}),this.listeners={},["tap","doubletap","press","pinch","pan","panstart","panmove","panend"].forEach((function(e){var n=function(n){t.isActive()&&t.emit(e,n)};t.hammer.on(e,n),t.listeners[e]=n})),l.onTouch(this.hammer,function(e){t.emit("touch",e)}.bind(this)),l.onRelease(this.hammer,function(e){t.emit("release",e)}.bind(this)),this.dom.centerContainer.addEventListener?(this.dom.centerContainer.addEventListener("mousewheel",r.bind(this),!1),this.dom.centerContainer.addEventListener("DOMMouseScroll",r.bind(this),!1)):this.dom.centerContainer.attachEvent("onmousewheel",r.bind(this)),this.dom.left.parentNode.addEventListener("scroll",i.bind(this)),this.dom.right.parentNode.addEventListener("scroll",i.bind(this));var o=!1;if(this.dom.center.addEventListener("dragover",function(e){if(e.preventDefault&&e.preventDefault(),!(!e.target.className.indexOf("vis")>-1||o))return e.dataTransfer.dropEffect="move",o=!0,!1}.bind(this),!1),this.dom.center.addEventListener("drop",function(e){e.preventDefault&&e.preventDefault(),e.stopPropagation&&e.stopPropagation();try{var n=JSON.parse(e.dataTransfer.getData("text"));if(!n||!n.content)return}catch(e){return!1}return o=!1,e.center={x:e.clientX,y:e.clientY},"item"!==n.target?t.itemSet._onAddItem(e):t.itemSet._onDropObjectOnItem(e),t.emit("drop",t.getEventProperties(e)),!1}.bind(this),!1),this.customTimes=[],this.touch={},this.redrawCount=0,this.initialDrawDone=!1,this.initialRangeChangeDone=!1,!e)throw new Error("No container provided");e.appendChild(this.dom.root)},p.prototype.setOptions=function(e){if(e){if(u.selectiveExtend(["width","height","minHeight","maxHeight","autoResize","start","end","clickToUse","dataAttributes","hiddenDates","locale","locales","moment","rtl","zoomKey","horizontalScroll","verticalScroll"],this.options,e),this.dom.rollingModeBtn.style.visibility="hidden",this.options.rtl&&(this.dom.container.style.direction="rtl",this.dom.backgroundVertical.className="vis-panel vis-background vis-vertical-rtl"),this.options.verticalScroll&&(this.options.rtl?this.dom.rightContainer.className="vis-panel vis-right vis-vertical-scroll":this.dom.leftContainer.className="vis-panel vis-left vis-vertical-scroll"),"object"!==(0,i.default)(this.options.orientation)&&(this.options.orientation={item:void 0,axis:void 0}),"orientation"in e&&("string"==typeof e.orientation?this.options.orientation={item:e.orientation,axis:e.orientation}:"object"===(0,i.default)(e.orientation)&&("item"in e.orientation&&(this.options.orientation.item=e.orientation.item),"axis"in e.orientation&&(this.options.orientation.axis=e.orientation.axis))),"both"===this.options.orientation.axis){if(!this.timeAxis2){var t=this.timeAxis2=new c(this.body);t.setOptions=function(e){var n=e?u.extend({},e):{};n.orientation="top",c.prototype.setOptions.call(t,n)},this.components.push(t)}}else if(this.timeAxis2){var n=this.components.indexOf(this.timeAxis2);-1!==n&&this.components.splice(n,1),this.timeAxis2.destroy(),this.timeAxis2=null}if("function"==typeof e.drawPoints&&(e.drawPoints={onRender:e.drawPoints}),"hiddenDates"in this.options&&h.convertHiddenOptions(this.options.moment,this.body,this.options.hiddenDates),"clickToUse"in e&&(e.clickToUse?this.activator||(this.activator=new d(this.dom.root)):this.activator&&(this.activator.destroy(),delete this.activator)),"showCustomTime"in e)throw new Error("Option `showCustomTime` is deprecated. Create a custom time bar via timeline.addCustomTime(time [, id])");this._initAutoResize()}if(this.components.forEach((function(t){return t.setOptions(e)})),"configure"in e){this.configurator||(this.configurator=this._createConfigurator()),this.configurator.setOptions(e.configure);var r=u.deepExtend({},this.options);this.components.forEach((function(e){u.deepExtend(r,e.options)})),this.configurator.setModuleOptions({global:r})}this._redraw()},p.prototype.isActive=function(){return!this.activator||this.activator.active},p.prototype.destroy=function(){for(var e in this.setItems(null),this.setGroups(null),this.off(),this._stopAutoResize(),this.dom.root.parentNode&&this.dom.root.parentNode.removeChild(this.dom.root),this.dom=null,this.activator&&(this.activator.destroy(),delete this.activator),this.listeners)this.listeners.hasOwnProperty(e)&&delete this.listeners[e];this.listeners=null,this.hammer=null,this.components.forEach((function(e){return e.destroy()})),this.body=null},p.prototype.setCustomTime=function(e,t){var n=this.customTimes.filter((function(e){return t===e.options.id}));if(0===n.length)throw new Error("No custom time bar found with id "+(0,r.default)(t));n.length>0&&n[0].setCustomTime(e)},p.prototype.getCustomTime=function(e){var t=this.customTimes.filter((function(t){return t.options.id===e}));if(0===t.length)throw new Error("No custom time bar found with id "+(0,r.default)(e));return t[0].getCustomTime()},p.prototype.setCustomTimeTitle=function(e,t){var n=this.customTimes.filter((function(e){return e.options.id===t}));if(0===n.length)throw new Error("No custom time bar found with id "+(0,r.default)(t));if(n.length>0)return n[0].setCustomTitle(e)},p.prototype.getEventProperties=function(e){return{event:e}},p.prototype.addCustomTime=function(e,t){var n=void 0!==e?u.convert(e,"Date").valueOf():new Date;if(this.customTimes.some((function(e){return e.options.id===t})))throw new Error("A custom time with id "+(0,r.default)(t)+" already exists");var i=new f(this.body,u.extend({},this.options,{time:n,id:t}));return this.customTimes.push(i),this.components.push(i),this._redraw(),t},p.prototype.removeCustomTime=function(e){var t=this.customTimes.filter((function(t){return t.options.id===e}));if(0===t.length)throw new Error("No custom time bar found with id "+(0,r.default)(e));t.forEach(function(e){this.customTimes.splice(this.customTimes.indexOf(e),1),this.components.splice(this.components.indexOf(e),1),e.destroy()}.bind(this))},p.prototype.getVisibleItems=function(){return this.itemSet&&this.itemSet.getVisibleItems()||[]},p.prototype.fit=function(e,t){var n=this.getDataRange();if(null!==n.min||null!==n.max){var r=n.max-n.min,i=new Date(n.min.valueOf()-.01*r),o=new Date(n.max.valueOf()+.01*r),a=!e||void 0===e.animation||e.animation;this.range.setRange(i,o,{animation:a},t)}},p.prototype.getDataRange=function(){throw new Error("Cannot invoke abstract method getDataRange")},p.prototype.setWindow=function(e,t,n,r){var i,o;"function"==typeof arguments[2]&&(r=arguments[2],n={}),1==arguments.length?(i=void 0===(o=arguments[0]).animation||o.animation,this.range.setRange(o.start,o.end,{animation:i})):2==arguments.length&&"function"==typeof arguments[1]?(r=arguments[1],i=void 0===(o=arguments[0]).animation||o.animation,this.range.setRange(o.start,o.end,{animation:i},r)):(i=!n||void 0===n.animation||n.animation,this.range.setRange(e,t,{animation:i},r))},p.prototype.moveTo=function(e,t,n){"function"==typeof arguments[1]&&(n=arguments[1],t={});var r=this.range.end-this.range.start,i=u.convert(e,"Date").valueOf(),o=i-r/2,a=i+r/2,s=!t||void 0===t.animation||t.animation;this.range.setRange(o,a,{animation:s},n)},p.prototype.getWindow=function(){var e=this.range.getRange();return{start:new Date(e.start),end:new Date(e.end)}},p.prototype.zoomIn=function(e,t,n){if(!(!e||e<0||e>1)){"function"==typeof arguments[1]&&(n=arguments[1],t={});var r=this.getWindow(),i=r.start.valueOf(),o=r.end.valueOf(),a=o-i,s=a/(1+e),l=(a-s)/2,u=i+l,c=o-l;this.setWindow(u,c,t,n)}},p.prototype.zoomOut=function(e,t,n){if(!(!e||e<0||e>1)){"function"==typeof arguments[1]&&(n=arguments[1],t={});var r=this.getWindow(),i=r.start.valueOf(),o=r.end.valueOf(),a=o-i,s=i-a*e/2,l=o+a*e/2;this.setWindow(s,l,t,n)}},p.prototype.redraw=function(){this._redraw()},p.prototype._redraw=function(){this.redrawCount++;var e=!1,t=this.options,n=this.props,r=this.dom;if(r&&r.container&&0!=r.root.offsetWidth){h.updateHiddenDates(this.options.moment,this.body,this.options.hiddenDates),"top"==t.orientation?(u.addClassName(r.root,"vis-top"),u.removeClassName(r.root,"vis-bottom")):(u.removeClassName(r.root,"vis-top"),u.addClassName(r.root,"vis-bottom")),r.root.style.maxHeight=u.option.asSize(t.maxHeight,""),r.root.style.minHeight=u.option.asSize(t.minHeight,""),r.root.style.width=u.option.asSize(t.width,""),n.border.left=(r.centerContainer.offsetWidth-r.centerContainer.clientWidth)/2,n.border.right=n.border.left,n.border.top=(r.centerContainer.offsetHeight-r.centerContainer.clientHeight)/2,n.border.bottom=n.border.top,n.borderRootHeight=r.root.offsetHeight-r.root.clientHeight,n.borderRootWidth=r.root.offsetWidth-r.root.clientWidth,0===r.centerContainer.clientHeight&&(n.border.left=n.border.top,n.border.right=n.border.left),0===r.root.clientHeight&&(n.borderRootWidth=n.borderRootHeight),n.center.height=r.center.offsetHeight,n.left.height=r.left.offsetHeight,n.right.height=r.right.offsetHeight,n.top.height=r.top.clientHeight||-n.border.top,n.bottom.height=r.bottom.clientHeight||-n.border.bottom;var i=Math.max(n.left.height,n.center.height,n.right.height),o=n.top.height+i+n.bottom.height+n.borderRootHeight+n.border.top+n.border.bottom;r.root.style.height=u.option.asSize(t.height,o+"px"),n.root.height=r.root.offsetHeight,n.background.height=n.root.height-n.borderRootHeight;var a=n.root.height-n.top.height-n.bottom.height-n.borderRootHeight;n.centerContainer.height=a,n.leftContainer.height=a,n.rightContainer.height=n.leftContainer.height,n.root.width=r.root.offsetWidth,n.background.width=n.root.width-n.borderRootWidth,this.initialDrawDone||(n.scrollbarWidth=u.getScrollBarWidth()),t.verticalScroll?t.rtl?(n.left.width=r.leftContainer.clientWidth||-n.border.left,n.right.width=r.rightContainer.clientWidth+n.scrollbarWidth||-n.border.right):(n.left.width=r.leftContainer.clientWidth+n.scrollbarWidth||-n.border.left,n.right.width=r.rightContainer.clientWidth||-n.border.right):(n.left.width=r.leftContainer.clientWidth||-n.border.left,n.right.width=r.rightContainer.clientWidth||-n.border.right),this._setDOM();var l=this._updateScrollTop();"top"!=t.orientation.item&&(l+=Math.max(n.centerContainer.height-n.center.height-n.border.top-n.border.bottom,0)),r.center.style.top=l+"px";var c=0==n.scrollTop?"hidden":"",d=n.scrollTop==n.scrollTopMin?"hidden":"";r.shadowTop.style.visibility=c,r.shadowBottom.style.visibility=d,r.shadowTopLeft.style.visibility=c,r.shadowBottomLeft.style.visibility=d,r.shadowTopRight.style.visibility=c,r.shadowBottomRight.style.visibility=d,t.verticalScroll&&(r.rightContainer.className="vis-panel vis-right vis-vertical-scroll",r.leftContainer.className="vis-panel vis-left vis-vertical-scroll",r.shadowTopRight.style.visibility="hidden",r.shadowBottomRight.style.visibility="hidden",r.shadowTopLeft.style.visibility="hidden",r.shadowBottomLeft.style.visibility="hidden",r.left.style.top="0px",r.right.style.top="0px"),(!t.verticalScroll||n.center.height<n.centerContainer.height)&&(r.left.style.top=l+"px",r.right.style.top=l+"px",r.rightContainer.className=r.rightContainer.className.replace(new RegExp("(?:^|\\s)vis-vertical-scroll(?:\\s|$)")," "),r.leftContainer.className=r.leftContainer.className.replace(new RegExp("(?:^|\\s)vis-vertical-scroll(?:\\s|$)")," "),n.left.width=r.leftContainer.clientWidth||-n.border.left,n.right.width=r.rightContainer.clientWidth||-n.border.right,this._setDOM());var f=n.center.height>n.centerContainer.height;if(this.hammer.get("pan").set({direction:f?s.DIRECTION_ALL:s.DIRECTION_HORIZONTAL}),this.components.forEach((function(t){e=t.redraw()||e})),e){if(this.redrawCount<5)return void this.body.emitter.emit("_change");console.log("WARNING: infinite loop in redraw?")}else this.redrawCount=0;this.body.emitter.emit("changed")}},p.prototype._setDOM=function(){var e=this.props,t=this.dom;e.leftContainer.width=e.left.width,e.rightContainer.width=e.right.width;var n=e.root.width-e.left.width-e.right.width-e.borderRootWidth;e.center.width=n,e.centerContainer.width=n,e.top.width=n,e.bottom.width=n,t.background.style.height=e.background.height+"px",t.backgroundVertical.style.height=e.background.height+"px",t.backgroundHorizontal.style.height=e.centerContainer.height+"px",t.centerContainer.style.height=e.centerContainer.height+"px",t.leftContainer.style.height=e.leftContainer.height+"px",t.rightContainer.style.height=e.rightContainer.height+"px",t.background.style.width=e.background.width+"px",t.backgroundVertical.style.width=e.centerContainer.width+"px",t.backgroundHorizontal.style.width=e.background.width+"px",t.centerContainer.style.width=e.center.width+"px",t.top.style.width=e.top.width+"px",t.bottom.style.width=e.bottom.width+"px",t.background.style.left="0",t.background.style.top="0",t.backgroundVertical.style.left=e.left.width+e.border.left+"px",t.backgroundVertical.style.top="0",t.backgroundHorizontal.style.left="0",t.backgroundHorizontal.style.top=e.top.height+"px",t.centerContainer.style.left=e.left.width+"px",t.centerContainer.style.top=e.top.height+"px",t.leftContainer.style.left="0",t.leftContainer.style.top=e.top.height+"px",t.rightContainer.style.left=e.left.width+e.center.width+"px",t.rightContainer.style.top=e.top.height+"px",t.top.style.left=e.left.width+"px",t.top.style.top="0",t.bottom.style.left=e.left.width+"px",t.bottom.style.top=e.top.height+e.centerContainer.height+"px",t.center.style.left="0",t.left.style.left="0",t.right.style.left="0"},p.prototype.repaint=function(){throw new Error("Function repaint is deprecated. Use redraw instead.")},p.prototype.setCurrentTime=function(e){if(!this.currentTime)throw new Error("Option showCurrentTime must be true");this.currentTime.setCurrentTime(e)},p.prototype.getCurrentTime=function(){if(!this.currentTime)throw new Error("Option showCurrentTime must be true");return this.currentTime.getCurrentTime()},p.prototype._toTime=function(e){return h.toTime(this,e,this.props.center.width)},p.prototype._toGlobalTime=function(e){return h.toTime(this,e,this.props.root.width)},p.prototype._toScreen=function(e){return h.toScreen(this,e,this.props.center.width)},p.prototype._toGlobalScreen=function(e){return h.toScreen(this,e,this.props.root.width)},p.prototype._initAutoResize=function(){1==this.options.autoResize?this._startAutoResize():this._stopAutoResize()},p.prototype._startAutoResize=function(){var e=this;this._stopAutoResize(),this._onResize=function(){1==e.options.autoResize?e.dom.root&&(e.dom.root.offsetWidth==e.props.lastWidth&&e.dom.root.offsetHeight==e.props.lastHeight||(e.props.lastWidth=e.dom.root.offsetWidth,e.props.lastHeight=e.dom.root.offsetHeight,e.props.scrollbarWidth=u.getScrollBarWidth(),e.body.emitter.emit("_change"))):e._stopAutoResize()},u.addEventListener(window,"resize",this._onResize),e.dom.root&&(e.props.lastWidth=e.dom.root.offsetWidth,e.props.lastHeight=e.dom.root.offsetHeight),this.watchTimer=setInterval(this._onResize,1e3)},p.prototype._stopAutoResize=function(){this.watchTimer&&(clearInterval(this.watchTimer),this.watchTimer=void 0),this._onResize&&(u.removeEventListener(window,"resize",this._onResize),this._onResize=null)},p.prototype._onTouch=function(e){this.touch.allowDragging=!0,this.touch.initialScrollTop=this.props.scrollTop},p.prototype._onPinch=function(e){this.touch.allowDragging=!1},p.prototype._onDrag=function(e){if(e&&this.touch.allowDragging){var t=e.deltaY,n=this._getScrollTop(),r=this._setScrollTop(this.touch.initialScrollTop+t);this.options.verticalScroll&&(this.dom.left.parentNode.scrollTop=-this.props.scrollTop,this.dom.right.parentNode.scrollTop=-this.props.scrollTop),r!=n&&this.emit("verticalDrag")}},p.prototype._setScrollTop=function(e){return this.props.scrollTop=e,this._updateScrollTop(),this.props.scrollTop},p.prototype._updateScrollTop=function(){var e=Math.min(this.props.centerContainer.height-this.props.center.height,0);return e!=this.props.scrollTopMin&&("top"!=this.options.orientation.item&&(this.props.scrollTop+=e-this.props.scrollTopMin),this.props.scrollTopMin=e),this.props.scrollTop>0&&(this.props.scrollTop=0),this.props.scrollTop<e&&(this.props.scrollTop=e),this.options.verticalScroll&&(this.dom.left.parentNode.scrollTop=-this.props.scrollTop,this.dom.right.parentNode.scrollTop=-this.props.scrollTop),this.props.scrollTop},p.prototype._getScrollTop=function(){return this.props.scrollTop},p.prototype._createConfigurator=function(){throw new Error("Cannot invoke abstract method _createConfigurator")},e.exports=p},function(e,t,n){var r=n(9),i=n(36),o=n(2);function a(e,t,n,i,o){this.moment=r,this.current=this.moment(),this._start=this.moment(),this._end=this.moment(),this.autoScale=!0,this.scale="day",this.step=1,this.setRange(e,t,n),this.switchedDay=!1,this.switchedMonth=!1,this.switchedYear=!1,Array.isArray(i)?this.hiddenDates=i:this.hiddenDates=null!=i?[i]:[],this.format=a.FORMAT,this.options=o||{}}a.FORMAT={minorLabels:{millisecond:"SSS",second:"s",minute:"HH:mm",hour:"HH:mm",weekday:"ddd D",day:"D",week:"w",month:"MMM",year:"YYYY"},majorLabels:{millisecond:"HH:mm:ss",second:"D MMMM HH:mm",minute:"ddd D MMMM",hour:"ddd D MMMM",weekday:"MMMM YYYY",day:"MMMM YYYY",week:"MMMM YYYY",month:"YYYY",year:""}},a.prototype.setMoment=function(e){this.moment=e,this.current=this.moment(this.current.valueOf()),this._start=this.moment(this._start.valueOf()),this._end=this.moment(this._end.valueOf())},a.prototype.setFormat=function(e){var t=o.deepExtend({},a.FORMAT);this.format=o.deepExtend(t,e)},a.prototype.setRange=function(e,t,n){if(!(e instanceof Date&&t instanceof Date))throw"No legal start or end date in method setRange";this._start=null!=e?this.moment(e.valueOf()):new Date,this._end=null!=t?this.moment(t.valueOf()):new Date,this.autoScale&&this.setMinimumStep(n)},a.prototype.start=function(){this.current=this._start.clone(),this.roundToMinor()},a.prototype.roundToMinor=function(){switch("week"==this.scale&&this.current.weekday(0),this.scale){case"year":this.current.year(this.step*Math.floor(this.current.year()/this.step)),this.current.month(0);case"month":this.current.date(1);case"week":case"day":case"weekday":this.current.hours(0);case"hour":this.current.minutes(0);case"minute":this.current.seconds(0);case"second":this.current.milliseconds(0)}if(1!=this.step)switch(this.scale){case"millisecond":this.current.subtract(this.current.milliseconds()%this.step,"milliseconds");break;case"second":this.current.subtract(this.current.seconds()%this.step,"seconds");break;case"minute":this.current.subtract(this.current.minutes()%this.step,"minutes");break;case"hour":this.current.subtract(this.current.hours()%this.step,"hours");break;case"weekday":case"day":this.current.subtract((this.current.date()-1)%this.step,"day");break;case"week":this.current.subtract(this.current.week()%this.step,"week");break;case"month":this.current.subtract(this.current.month()%this.step,"month");break;case"year":this.current.subtract(this.current.year()%this.step,"year")}},a.prototype.hasNext=function(){return this.current.valueOf()<=this._end.valueOf()},a.prototype.next=function(){var e=this.current.valueOf();switch(this.scale){case"millisecond":this.current.add(this.step,"millisecond");break;case"second":this.current.add(this.step,"second");break;case"minute":this.current.add(this.step,"minute");break;case"hour":this.current.add(this.step,"hour"),this.current.month()<6?this.current.subtract(this.current.hours()%this.step,"hour"):this.current.hours()%this.step!=0&&this.current.add(this.step-this.current.hours()%this.step,"hour");break;case"weekday":case"day":this.current.add(this.step,"day");break;case"week":if(0!==this.current.weekday())this.current.weekday(0),this.current.add(this.step,"week");else if(!1===this.options.showMajorLabels)this.current.add(this.step,"week");else{var t=this.current.clone();t.add(1,"week"),t.isSame(this.current,"month")?this.current.add(this.step,"week"):(this.current.add(this.step,"week"),this.current.date(1))}break;case"month":this.current.add(this.step,"month");break;case"year":this.current.add(this.step,"year")}if(1!=this.step)switch(this.scale){case"millisecond":this.current.milliseconds()>0&&this.current.milliseconds()<this.step&&this.current.milliseconds(0);break;case"second":this.current.seconds()>0&&this.current.seconds()<this.step&&this.current.seconds(0);break;case"minute":this.current.minutes()>0&&this.current.minutes()<this.step&&this.current.minutes(0);break;case"hour":this.current.hours()>0&&this.current.hours()<this.step&&this.current.hours(0);break;case"weekday":case"day":this.current.date()<this.step+1&&this.current.date(1);break;case"week":this.current.week()<this.step&&this.current.week(1);break;case"month":this.current.month()<this.step&&this.current.month(0)}this.current.valueOf()==e&&(this.current=this._end.clone()),this.switchedDay=!1,this.switchedMonth=!1,this.switchedYear=!1,i.stepOverHiddenDates(this.moment,this,e)},a.prototype.getCurrent=function(){return this.current},a.prototype.setScale=function(e){e&&"string"==typeof e.scale&&(this.scale=e.scale,this.step=e.step>0?e.step:1,this.autoScale=!1)},a.prototype.setAutoScale=function(e){this.autoScale=e},a.prototype.setMinimumStep=function(e){null!=e&&(31104e9>e&&(this.scale="year",this.step=1e3),15552e9>e&&(this.scale="year",this.step=500),31104e8>e&&(this.scale="year",this.step=100),15552e8>e&&(this.scale="year",this.step=50),31104e7>e&&(this.scale="year",this.step=10),15552e7>e&&(this.scale="year",this.step=5),31104e6>e&&(this.scale="year",this.step=1),7776e6>e&&(this.scale="month",this.step=3),2592e6>e&&(this.scale="month",this.step=1),432e6>e&&(this.scale="day",this.step=5),1728e5>e&&(this.scale="day",this.step=2),864e5>e&&(this.scale="day",this.step=1),432e5>e&&(this.scale="weekday",this.step=1),144e5>e&&(this.scale="hour",this.step=4),36e5>e&&(this.scale="hour",this.step=1),9e5>e&&(this.scale="minute",this.step=15),6e5>e&&(this.scale="minute",this.step=10),3e5>e&&(this.scale="minute",this.step=5),6e4>e&&(this.scale="minute",this.step=1),15e3>e&&(this.scale="second",this.step=15),1e4>e&&(this.scale="second",this.step=10),5e3>e&&(this.scale="second",this.step=5),1e3>e&&(this.scale="second",this.step=1),200>e&&(this.scale="millisecond",this.step=200),100>e&&(this.scale="millisecond",this.step=100),50>e&&(this.scale="millisecond",this.step=50),10>e&&(this.scale="millisecond",this.step=10),5>e&&(this.scale="millisecond",this.step=5),1>e&&(this.scale="millisecond",this.step=1))},a.snap=function(e,t,n){var i=r(e);if("year"==t){var o=i.year()+Math.round(i.month()/12);i.year(Math.round(o/n)*n),i.month(0),i.date(0),i.hours(0),i.minutes(0),i.seconds(0),i.milliseconds(0)}else if("month"==t)i.date()>15?(i.date(1),i.add(1,"month")):i.date(1),i.hours(0),i.minutes(0),i.seconds(0),i.milliseconds(0);else if("week"==t)i.weekday()>2?(i.weekday(0),i.add(1,"week")):i.weekday(0),i.hours(0),i.minutes(0),i.seconds(0),i.milliseconds(0);else if("day"==t){switch(n){case 5:case 2:i.hours(24*Math.round(i.hours()/24));break;default:i.hours(12*Math.round(i.hours()/12))}i.minutes(0),i.seconds(0),i.milliseconds(0)}else if("weekday"==t){switch(n){case 5:case 2:i.hours(12*Math.round(i.hours()/12));break;default:i.hours(6*Math.round(i.hours()/6))}i.minutes(0),i.seconds(0),i.milliseconds(0)}else if("hour"==t){switch(n){case 4:i.minutes(60*Math.round(i.minutes()/60));break;default:i.minutes(30*Math.round(i.minutes()/30))}i.seconds(0),i.milliseconds(0)}else if("minute"==t){switch(n){case 15:case 10:i.minutes(5*Math.round(i.minutes()/5)),i.seconds(0);break;case 5:i.seconds(60*Math.round(i.seconds()/60));break;default:i.seconds(30*Math.round(i.seconds()/30))}i.milliseconds(0)}else if("second"==t)switch(n){case 15:case 10:i.seconds(5*Math.round(i.seconds()/5)),i.milliseconds(0);break;case 5:i.milliseconds(1e3*Math.round(i.milliseconds()/1e3));break;default:i.milliseconds(500*Math.round(i.milliseconds()/500))}else if("millisecond"==t){var a=n>5?n/2:1;i.milliseconds(Math.round(i.milliseconds()/a)*a)}return i},a.prototype.isMajor=function(){if(1==this.switchedYear)switch(this.scale){case"year":case"month":case"week":case"weekday":case"day":case"hour":case"minute":case"second":case"millisecond":return!0;default:return!1}else if(1==this.switchedMonth)switch(this.scale){case"week":case"weekday":case"day":case"hour":case"minute":case"second":case"millisecond":return!0;default:return!1}else if(1==this.switchedDay)switch(this.scale){case"millisecond":case"second":case"minute":case"hour":return!0;default:return!1}var e=this.moment(this.current);switch(this.scale){case"millisecond":return 0==e.milliseconds();case"second":return 0==e.seconds();case"minute":return 0==e.hours()&&0==e.minutes();case"hour":return 0==e.hours();case"weekday":case"day":case"week":return 1==e.date();case"month":return 0==e.month();case"year":default:return!1}},a.prototype.getLabelMinor=function(e){if(null==e&&(e=this.current),e instanceof Date&&(e=this.moment(e)),"function"==typeof this.format.minorLabels)return this.format.minorLabels(e,this.scale,this.step);var t=this.format.minorLabels[this.scale];switch(this.scale){case"week":if(this.isMajor()&&0!==e.weekday())return"";default:return t&&t.length>0?this.moment(e).format(t):""}},a.prototype.getLabelMajor=function(e){if(null==e&&(e=this.current),e instanceof Date&&(e=this.moment(e)),"function"==typeof this.format.majorLabels)return this.format.majorLabels(e,this.scale,this.step);var t=this.format.majorLabels[this.scale];return t&&t.length>0?this.moment(e).format(t):""},a.prototype.getClassName=function(){var e=this.moment,t=this.moment(this.current),n=t.locale?t.locale("en"):t.lang("en"),r=this.step,i=[];function o(e){return e/r%2==0?" vis-even":" vis-odd"}function a(t){return t.isSame(new Date,"day")?" vis-today":t.isSame(e().add(1,"day"),"day")?" vis-tomorrow":t.isSame(e().add(-1,"day"),"day")?" vis-yesterday":""}function s(e){return e.isSame(new Date,"week")?" vis-current-week":""}function l(e){return e.isSame(new Date,"month")?" vis-current-month":""}switch(this.scale){case"millisecond":i.push(a(n)),i.push(o(n.milliseconds()));break;case"second":i.push(a(n)),i.push(o(n.seconds()));break;case"minute":i.push(a(n)),i.push(o(n.minutes()));break;case"hour":i.push("vis-h"+n.hours()+(4==this.step?"-h"+(n.hours()+4):"")),i.push(a(n)),i.push(o(n.hours()));break;case"weekday":i.push("vis-"+n.format("dddd").toLowerCase()),i.push(a(n)),i.push(s(n)),i.push(o(n.date()));break;case"day":i.push("vis-day"+n.date()),i.push("vis-"+n.format("MMMM").toLowerCase()),i.push(a(n)),i.push(l(n)),i.push(this.step<=2?a(n):""),i.push(this.step<=2?"vis-"+n.format("dddd").toLowerCase():""),i.push(o(n.date()-1));break;case"week":i.push("vis-week"+n.format("w")),i.push(s(n)),i.push(o(n.week()));break;case"month":i.push("vis-"+n.format("MMMM").toLowerCase()),i.push(l(n)),i.push(o(n.month()));break;case"year":i.push("vis-year"+n.year()),i.push(n.isSame(new Date,"year")?" vis-current-year":""),i.push(o(n.year()))}return i.filter(String).join(" ")},e.exports=a},function(e,t,n){var r=n(2),i=n(16),o=n(9),a=n(98);function s(e,t){this.body=e,this.defaultOptions={rtl:!1,showCurrentTime:!0,moment:o,locales:a,locale:"en"},this.options=r.extend({},this.defaultOptions),this.offset=0,this._create(),this.setOptions(t)}s.prototype=new i,s.prototype._create=function(){var e=document.createElement("div");e.className="vis-current-time",e.style.position="absolute",e.style.top="0px",e.style.height="100%",this.bar=e},s.prototype.destroy=function(){this.options.showCurrentTime=!1,this.redraw(),this.body=null},s.prototype.setOptions=function(e){e&&r.selectiveExtend(["rtl","showCurrentTime","moment","locale","locales"],this.options,e)},s.prototype.redraw=function(){if(this.options.showCurrentTime){var e=this.body.dom.backgroundVertical;this.bar.parentNode!=e&&(this.bar.parentNode&&this.bar.parentNode.removeChild(this.bar),e.appendChild(this.bar),this.start());var t=this.options.moment((new Date).valueOf()+this.offset),n=this.body.util.toScreen(t),r=this.options.locales[this.options.locale];r||(this.warned||(console.log("WARNING: options.locales['"+this.options.locale+"'] not found. See http://visjs.org/docs/timeline/#Localization"),this.warned=!0),r=this.options.locales.en);var i=r.current+" "+r.time+": "+t.format("dddd, MMMM Do YYYY, H:mm:ss");i=i.charAt(0).toUpperCase()+i.substring(1),this.options.rtl?this.bar.style.right=n+"px":this.bar.style.left=n+"px",this.bar.title=i}else this.bar.parentNode&&this.bar.parentNode.removeChild(this.bar),this.stop();return!1},s.prototype.start=function(){var e=this;!function t(){e.stop();var n=1/e.body.range.conversion(e.body.domProps.center.width).scale/10;n<30&&(n=30),n>1e3&&(n=1e3),e.redraw(),e.body.emitter.emit("currentTimeTick"),e.currentTimeTimer=setTimeout(t,n)}()},s.prototype.stop=function(){void 0!==this.currentTimeTimer&&(clearTimeout(this.currentTimeTimer),delete this.currentTimeTimer)},s.prototype.setCurrentTime=function(e){var t=r.convert(e,"Date").valueOf(),n=(new Date).valueOf();this.offset=t-n,this.redraw()},s.prototype.getCurrentTime=function(){return new Date((new Date).valueOf()+this.offset)},e.exports=s},function(e,t,n){var r,i=n(8),o=(r=i)&&r.__esModule?r:{default:r},a=n(2),s=n(100);function l(e,t,n){if(this.groupId=e,this.subgroups={},this.subgroupStack={},this.subgroupStackAll=!1,this.doInnerStack=!1,this.subgroupIndex=0,this.subgroupOrderer=t&&t.subgroupOrder,this.itemSet=n,this.isVisible=null,this.stackDirty=!0,t&&t.nestedGroups&&(this.nestedGroups=t.nestedGroups,0==t.showNested?this.showNested=!1:this.showNested=!0),t&&t.subgroupStack)if("boolean"==typeof t.subgroupStack)this.doInnerStack=t.subgroupStack,this.subgroupStackAll=t.subgroupStack;else for(var r in t.subgroupStack)this.subgroupStack[r]=t.subgroupStack[r],this.doInnerStack=this.doInnerStack||t.subgroupStack[r];this.nestedInGroup=null,this.dom={},this.props={label:{width:0,height:0}},this.className=null,this.items={},this.visibleItems=[],this.itemsInRange=[],this.orderedItems={byStart:[],byEnd:[]},this.checkRangedItems=!1;var i=this;this.itemSet.body.emitter.on("checkRangedItems",(function(){i.checkRangedItems=!0})),this._create(),this.setData(t)}l.prototype._create=function(){var e=document.createElement("div");this.itemSet.options.groupEditable.order?e.className="vis-label draggable":e.className="vis-label",this.dom.label=e;var t=document.createElement("div");t.className="vis-inner",e.appendChild(t),this.dom.inner=t;var n=document.createElement("div");n.className="vis-group",n["timeline-group"]=this,this.dom.foreground=n,this.dom.background=document.createElement("div"),this.dom.background.className="vis-group",this.dom.axis=document.createElement("div"),this.dom.axis.className="vis-group",this.dom.marker=document.createElement("div"),this.dom.marker.style.visibility="hidden",this.dom.marker.style.position="absolute",this.dom.marker.innerHTML="",this.dom.background.appendChild(this.dom.marker)},l.prototype.setData=function(e){var t,n;if((t=this.itemSet.options&&this.itemSet.options.groupTemplate?(n=this.itemSet.options.groupTemplate.bind(this))(e,this.dom.inner):e&&e.content)instanceof Element){for(this.dom.inner.appendChild(t);this.dom.inner.firstChild;)this.dom.inner.removeChild(this.dom.inner.firstChild);this.dom.inner.appendChild(t)}else t instanceof Object?n(e,this.dom.inner):this.dom.inner.innerHTML=null!=t?t:this.groupId||"";if(this.dom.label.title=e&&e.title||"",this.dom.inner.firstChild?a.removeClassName(this.dom.inner,"vis-hidden"):a.addClassName(this.dom.inner,"vis-hidden"),e&&e.nestedGroups){this.nestedGroups&&this.nestedGroups==e.nestedGroups||(this.nestedGroups=e.nestedGroups),void 0===e.showNested&&void 0!==this.showNested||(0==e.showNested?this.showNested=!1:this.showNested=!0),a.addClassName(this.dom.label,"vis-nesting-group");var r=this.itemSet.options.rtl?"collapsed-rtl":"collapsed";this.showNested?(a.removeClassName(this.dom.label,r),a.addClassName(this.dom.label,"expanded")):(a.removeClassName(this.dom.label,"expanded"),a.addClassName(this.dom.label,r))}else this.nestedGroups&&(this.nestedGroups=null,r=this.itemSet.options.rtl?"collapsed-rtl":"collapsed",a.removeClassName(this.dom.label,r),a.removeClassName(this.dom.label,"expanded"),a.removeClassName(this.dom.label,"vis-nesting-group"));e&&e.nestedInGroup&&(a.addClassName(this.dom.label,"vis-nested-group"),this.itemSet.options&&this.itemSet.options.rtl?this.dom.inner.style.paddingRight="30px":this.dom.inner.style.paddingLeft="30px");var i=e&&e.className||null;i!=this.className&&(this.className&&(a.removeClassName(this.dom.label,this.className),a.removeClassName(this.dom.foreground,this.className),a.removeClassName(this.dom.background,this.className),a.removeClassName(this.dom.axis,this.className)),a.addClassName(this.dom.label,i),a.addClassName(this.dom.foreground,i),a.addClassName(this.dom.background,i),a.addClassName(this.dom.axis,i),this.className=i),this.style&&(a.removeCssText(this.dom.label,this.style),this.style=null),e&&e.style&&(a.addCssText(this.dom.label,e.style),this.style=e.style)},l.prototype.getLabelWidth=function(){return this.props.label.width},l.prototype._didMarkerHeightChange=function(){var e=this.dom.marker.clientHeight;if(e!=this.lastMarkerHeight){this.lastMarkerHeight=e;var t={},n=0;if(a.forEach(this.items,(function(e,r){e.dirty=!0,e.displayed&&(t[r]=e.redraw(!0),n=t[r].length)})),n>0)for(var r=0;r<n;r++)a.forEach(t,(function(e){e[r]()}));return!0}},l.prototype._calculateGroupSizeAndPosition=function(){var e=this.dom.foreground.offsetTop,t=this.dom.foreground.offsetLeft,n=this.dom.foreground.offsetWidth;this.top=e,this.right=t,this.width=n},l.prototype._redrawItems=function(e,t,n,r){if(e||this.stackDirty||this.isVisible&&!t){var i={},o=null;if("function"==typeof this.itemSet.options.order){var l=this,u={},c=0;if(a.forEach(this.items,(function(e,t){e.displayed||(u[t]=e.redraw(!0),c=u[t].length,l.visibleItems.push(e))})),c>0)for(var d=0;d<c;d++)a.forEach(u,(function(e){e[d]()}));if(a.forEach(this.items,(function(e){e.repositionX(!1)})),this.doInnerStack&&this.itemSet.options.stackSubgroups){for(o in this.subgroups)i[o]=this.subgroups[o].items.slice().sort((function(e,t){return l.itemSet.options.order(e.data,t.data)}));s.stackSubgroupsWithInnerStack(i,n,this.subgroups)}else{var h=this.orderedItems.byStart.slice().sort((function(e,t){return l.itemSet.options.order(e.data,t.data)}));s.stack(h,n,!0)}this.visibleItems=this._updateItemsInRange(this.orderedItems,this.visibleItems,r)}else if(this.visibleItems=this._updateItemsInRange(this.orderedItems,this.visibleItems,r),this.itemSet.options.stack)if(this.doInnerStack&&this.itemSet.options.stackSubgroups){for(o in this.subgroups)i[o]=this.subgroups[o].items;s.stackSubgroupsWithInnerStack(i,n,this.subgroups)}else s.stack(this.visibleItems,n,!0);else s.nostack(this.visibleItems,n,this.subgroups,this.itemSet.options.stackSubgroups);this.stackDirty=!1}},l.prototype._didResize=function(e,t){e=a.updateProperty(this,"height",t)||e;var n=this.dom.inner.clientWidth,r=this.dom.inner.clientHeight;return e=a.updateProperty(this.props.label,"width",n)||e,e=a.updateProperty(this.props.label,"height",r)||e},l.prototype._applyGroupHeight=function(e){this.dom.background.style.height=e+"px",this.dom.foreground.style.height=e+"px",this.dom.label.style.height=e+"px"},l.prototype._updateItemsVerticalPosition=function(e){for(var t=0,n=this.visibleItems.length;t<n;t++){var r=this.visibleItems[t];r.repositionY(e),this.isVisible||"__background__"==this.groupId||r.displayed&&r.hide()}},l.prototype.redraw=function(e,t,n,r){var i,o,a=!1,s=this.isVisible,l=[function(){n=this._didMarkerHeightChange.bind(this)}.bind(this),this._updateSubGroupHeights.bind(this,t),this._calculateGroupSizeAndPosition.bind(this),function(){this.isVisible=this._isGroupVisible.bind(this)(e,t)}.bind(this),function(){this._redrawItems.bind(this)(n,s,t,e)}.bind(this),this._updateSubgroupsSizes.bind(this),function(){i=this._calculateHeight.bind(this)(t)}.bind(this),this._calculateGroupSizeAndPosition.bind(this),function(){a=this._didResize.bind(this)(a,i)}.bind(this),function(){this._applyGroupHeight.bind(this)(i)}.bind(this),function(){this._updateItemsVerticalPosition.bind(this)(t)}.bind(this),function(){return!this.isVisible&&this.height&&(a=!1),a}];return r?l:(l.forEach((function(e){o=e()})),o)},l.prototype._updateSubGroupHeights=function(e){if((0,o.default)(this.subgroups).length>0){var t=this;this.resetSubgroups(),a.forEach(this.visibleItems,(function(n){void 0!==n.data.subgroup&&(t.subgroups[n.data.subgroup].height=Math.max(t.subgroups[n.data.subgroup].height,n.height+e.item.vertical),t.subgroups[n.data.subgroup].visible=!0)}))}},l.prototype._isGroupVisible=function(e,t){return this.top<=e.body.domProps.centerContainer.height-e.body.domProps.scrollTop+t.axis&&this.top+this.height+t.axis>=-e.body.domProps.scrollTop},l.prototype._calculateHeight=function(e){var t,n=this.visibleItems;if(n.length>0){var r=n[0].top,i=n[0].top+n[0].height;if(a.forEach(n,(function(e){r=Math.min(r,e.top),i=Math.max(i,e.top+e.height)})),r>e.axis){var o=r-e.axis;i-=o,a.forEach(n,(function(e){e.top-=o}))}t=i+e.item.vertical/2}else t=0;return t=Math.max(t,this.props.label.height)},l.prototype.show=function(){this.dom.label.parentNode||this.itemSet.dom.labelSet.appendChild(this.dom.label),this.dom.foreground.parentNode||this.itemSet.dom.foreground.appendChild(this.dom.foreground),this.dom.background.parentNode||this.itemSet.dom.background.appendChild(this.dom.background),this.dom.axis.parentNode||this.itemSet.dom.axis.appendChild(this.dom.axis)},l.prototype.hide=function(){var e=this.dom.label;e.parentNode&&e.parentNode.removeChild(e);var t=this.dom.foreground;t.parentNode&&t.parentNode.removeChild(t);var n=this.dom.background;n.parentNode&&n.parentNode.removeChild(n);var r=this.dom.axis;r.parentNode&&r.parentNode.removeChild(r)},l.prototype.add=function(e){if(this.items[e.id]=e,e.setParent(this),this.stackDirty=!0,void 0!==e.data.subgroup&&(this._addToSubgroup(e),this.orderSubgroups()),-1==this.visibleItems.indexOf(e)){var t=this.itemSet.body.range;this._checkIfVisible(e,this.visibleItems,t)}},l.prototype._addToSubgroup=function(e,t){null!=(t=t||e.data.subgroup)&&void 0===this.subgroups[t]&&(this.subgroups[t]={height:0,top:0,start:e.data.start,end:e.data.end||e.data.start,visible:!1,index:this.subgroupIndex,items:[],stack:this.subgroupStackAll||this.subgroupStack[t]||!1},this.subgroupIndex++),new Date(e.data.start)<new Date(this.subgroups[t].start)&&(this.subgroups[t].start=e.data.start);var n=e.data.end||e.data.start;new Date(n)>new Date(this.subgroups[t].end)&&(this.subgroups[t].end=n),this.subgroups[t].items.push(e)},l.prototype._updateSubgroupsSizes=function(){var e=this;if(e.subgroups)for(var t in e.subgroups){var n=e.subgroups[t].items[0].data.end||e.subgroups[t].items[0].data.start,r=e.subgroups[t].items[0].data.start,i=n-1;e.subgroups[t].items.forEach((function(e){new Date(e.data.start)<new Date(r)&&(r=e.data.start);var t=e.data.end||e.data.start;new Date(t)>new Date(i)&&(i=t)})),e.subgroups[t].start=r,e.subgroups[t].end=new Date(i-1)}},l.prototype.orderSubgroups=function(){if(void 0!==this.subgroupOrderer){var e,t=[];if("string"==typeof this.subgroupOrderer){for(e in this.subgroups)t.push({subgroup:e,sortField:this.subgroups[e].items[0].data[this.subgroupOrderer]});t.sort((function(e,t){return e.sortField-t.sortField}))}else if("function"==typeof this.subgroupOrderer){for(e in this.subgroups)t.push(this.subgroups[e].items[0].data);t.sort(this.subgroupOrderer)}if(t.length>0)for(var n=0;n<t.length;n++)this.subgroups[t[n].subgroup].index=n}},l.prototype.resetSubgroups=function(){for(var e in this.subgroups)this.subgroups.hasOwnProperty(e)&&(this.subgroups[e].visible=!1,this.subgroups[e].height=0)},l.prototype.remove=function(e){delete this.items[e.id],e.setParent(null),this.stackDirty=!0;var t=this.visibleItems.indexOf(e);-1!=t&&this.visibleItems.splice(t,1),void 0!==e.data.subgroup&&(this._removeFromSubgroup(e),this.orderSubgroups())},l.prototype._removeFromSubgroup=function(e,t){if(null!=(t=t||e.data.subgroup)){var n=this.subgroups[t];if(n){var r=n.items.indexOf(e);r>=0&&(n.items.splice(r,1),n.items.length?this._updateSubgroupsSizes():delete this.subgroups[t])}}},l.prototype.removeFromDataSet=function(e){this.itemSet.removeItem(e.id)},l.prototype.order=function(){for(var e=a.toArray(this.items),t=[],n=[],r=0;r<e.length;r++)void 0!==e[r].data.end&&n.push(e[r]),t.push(e[r]);this.orderedItems={byStart:t,byEnd:n},s.orderByStart(this.orderedItems.byStart),s.orderByEnd(this.orderedItems.byEnd)},l.prototype._updateItemsInRange=function(e,t,n){var r=[],i={},o=(n.end-n.start)/4,s=n.start-o,l=n.end+o,u=function(e){return e<s?-1:e<=l?0:1};if(t.length>0)for(var c=0;c<t.length;c++)this._checkIfVisibleWithReference(t[c],r,i,n);var d=a.binarySearchCustom(e.byStart,u,"data","start");if(this._traceVisible(d,e.byStart,r,i,(function(e){return e.data.start<s||e.data.start>l})),1==this.checkRangedItems)for(this.checkRangedItems=!1,c=0;c<e.byEnd.length;c++)this._checkIfVisibleWithReference(e.byEnd[c],r,i,n);else{var h=a.binarySearchCustom(e.byEnd,u,"data","end");this._traceVisible(h,e.byEnd,r,i,(function(e){return e.data.end<s||e.data.end>l}))}var f={},p=0;for(c=0;c<r.length;c++){var g=r[c];g.displayed||(f[c]=g.redraw(!0),p=f[c].length)}if(p>0)for(var v=0;v<p;v++)a.forEach(f,(function(e){e[v]()}));for(c=0;c<r.length;c++)r[c].repositionX();return r},l.prototype._traceVisible=function(e,t,n,r,i){if(-1!=e){var o,a;for(o=e;o>=0&&!i(a=t[o]);o--)void 0===r[a.id]&&(r[a.id]=!0,n.push(a));for(o=e+1;o<t.length&&!i(a=t[o]);o++)void 0===r[a.id]&&(r[a.id]=!0,n.push(a))}},l.prototype._checkIfVisible=function(e,t,n){e.isVisible(n)?(e.displayed||e.show(),e.repositionX(),t.push(e)):e.displayed&&e.hide()},l.prototype._checkIfVisibleWithReference=function(e,t,n,r){e.isVisible(r)?void 0===n[e.id]&&(n[e.id]=!0,t.push(e)):e.displayed&&e.hide()},l.prototype.changeSubgroup=function(e,t,n){this._removeFromSubgroup(e,t),this._addToSubgroup(e,n),this.orderSubgroups()},e.exports=l},function(e,t,n){var r,i=n(29),o=(r=i)&&r.__esModule?r:{default:r},a=n(68);function s(e,t,n){a.call(this,e,t,n),this.width=0,this.height=0,this.top=0,this.left=0}s.prototype=(0,o.default)(a.prototype),s.prototype.redraw=function(e,t,n){this.visibleItems=this._updateItemsInRange(this.orderedItems,this.visibleItems,e),this.width=this.dom.background.offsetWidth,this.dom.background.style.height="0";for(var r=0,i=this.visibleItems.length;r<i;r++)this.visibleItems[r].repositionY(t);return!1},s.prototype.show=function(){this.dom.background.parentNode||this.itemSet.dom.background.appendChild(this.dom.background)},e.exports=s},function(e,t,n){var r=n(38);function i(e,t,n){if(this.props={content:{width:0}},this.overflow=!1,this.options=n,e){if(null==e.start)throw new Error('Property "start" missing in item '+e.id);if(null==e.end)throw new Error('Property "end" missing in item '+e.id)}r.call(this,e,t,n)}i.prototype=new r(null,null,null),i.prototype.baseClassName="vis-item vis-range",i.prototype.isVisible=function(e){return this.data.start<e.end&&this.data.end>e.start},i.prototype._createDomElement=function(){this.dom||(this.dom={},this.dom.box=document.createElement("div"),this.dom.frame=document.createElement("div"),this.dom.frame.className="vis-item-overflow",this.dom.box.appendChild(this.dom.frame),this.dom.visibleFrame=document.createElement("div"),this.dom.visibleFrame.className="vis-item-visible-frame",this.dom.box.appendChild(this.dom.visibleFrame),this.dom.content=document.createElement("div"),this.dom.content.className="vis-item-content",this.dom.frame.appendChild(this.dom.content),this.dom.box["timeline-item"]=this,this.dirty=!0)},i.prototype._appendDomElement=function(){if(!this.parent)throw new Error("Cannot redraw item: no parent attached");if(!this.dom.box.parentNode){var e=this.parent.dom.foreground;if(!e)throw new Error("Cannot redraw item: parent has no foreground container element");e.appendChild(this.dom.box)}this.displayed=!0},i.prototype._updateDirtyDomComponents=function(){if(this.dirty){this._updateContents(this.dom.content),this._updateDataAttributes(this.dom.box),this._updateStyle(this.dom.box);var e=this.editable.updateTime||this.editable.updateGroup,t=(this.data.className?" "+this.data.className:"")+(this.selected?" vis-selected":"")+(e?" vis-editable":" vis-readonly");this.dom.box.className=this.baseClassName+t,this.dom.content.style.maxWidth="none"}},i.prototype._getDomComponentsSizes=function(){return this.overflow="hidden"!==window.getComputedStyle(this.dom.frame).overflow,{content:{width:this.dom.content.offsetWidth},box:{height:this.dom.box.offsetHeight}}},i.prototype._updateDomComponentsSizes=function(e){this.props.content.width=e.content.width,this.height=e.box.height,this.dom.content.style.maxWidth="",this.dirty=!1},i.prototype._repaintDomAdditionals=function(){this._repaintOnItemUpdateTimeTooltip(this.dom.box),this._repaintDeleteButton(this.dom.box),this._repaintDragCenter(),this._repaintDragLeft(),this._repaintDragRight()},i.prototype.redraw=function(e){var t,n,r=[this._createDomElement.bind(this),this._appendDomElement.bind(this),this._updateDirtyDomComponents.bind(this),function(){this.dirty&&(t=this._getDomComponentsSizes.bind(this)())}.bind(this),function(){this.dirty&&this._updateDomComponentsSizes.bind(this)(t)}.bind(this),this._repaintDomAdditionals.bind(this)];return e?r:(r.forEach((function(e){n=e()})),n)},i.prototype.show=function(){this.displayed||this.redraw()},i.prototype.hide=function(){if(this.displayed){var e=this.dom.box;e.parentNode&&e.parentNode.removeChild(e),this.displayed=!1}},i.prototype.repositionX=function(e){var t,n,r=this.parent.width,i=this.conversion.toScreen(this.data.start),o=this.conversion.toScreen(this.data.end),a=void 0===this.data.align?this.options.align:this.data.align;!1===this.data.limitSize||void 0!==e&&!0!==e||(i<-r&&(i=-r),o>2*r&&(o=2*r));var s=Math.max(o-i+.5,1);switch(this.overflow?(this.options.rtl?this.right=i:this.left=i,this.width=s+this.props.content.width,n=this.props.content.width):(this.options.rtl?this.right=i:this.left=i,this.width=s,n=Math.min(o-i,this.props.content.width)),this.options.rtl?this.dom.box.style.right=this.right+"px":this.dom.box.style.left=this.left+"px",this.dom.box.style.width=s+"px",a){case"left":this.options.rtl?this.dom.content.style.right="0":this.dom.content.style.left="0";break;case"right":this.options.rtl?this.dom.content.style.right=Math.max(s-n,0)+"px":this.dom.content.style.left=Math.max(s-n,0)+"px";break;case"center":this.options.rtl?this.dom.content.style.right=Math.max((s-n)/2,0)+"px":this.dom.content.style.left=Math.max((s-n)/2,0)+"px";break;default:t=this.overflow?o>0?Math.max(-i,0):-n:i<0?-i:0,this.options.rtl?this.dom.content.style.right=t+"px":(this.dom.content.style.left=t+"px",this.dom.content.style.width="calc(100% - "+t+"px)")}},i.prototype.repositionY=function(){var e=this.options.orientation.item,t=this.dom.box;t.style.top="top"==e?this.top+"px":this.parent.height-this.top-this.height+"px"},i.prototype._repaintDragLeft=function(){if((this.selected||this.options.itemsAlwaysDraggable.range)&&this.options.editable.updateTime&&!this.dom.dragLeft){var e=document.createElement("div");e.className="vis-drag-left",e.dragLeftItem=this,this.dom.box.appendChild(e),this.dom.dragLeft=e}else this.selected||this.options.itemsAlwaysDraggable.range||!this.dom.dragLeft||(this.dom.dragLeft.parentNode&&this.dom.dragLeft.parentNode.removeChild(this.dom.dragLeft),this.dom.dragLeft=null)},i.prototype._repaintDragRight=function(){if((this.selected||this.options.itemsAlwaysDraggable.range)&&this.options.editable.updateTime&&!this.dom.dragRight){var e=document.createElement("div");e.className="vis-drag-right",e.dragRightItem=this,this.dom.box.appendChild(e),this.dom.dragRight=e}else this.selected||this.options.itemsAlwaysDraggable.range||!this.dom.dragRight||(this.dom.dragRight.parentNode&&this.dom.dragRight.parentNode.removeChild(this.dom.dragRight),this.dom.dragRight=null)},e.exports=i},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=s(n(19)),i=s(n(6)),o=s(n(0)),a=s(n(1));function s(e){return e&&e.__esModule?e:{default:e}}var l=n(2),u=n(179).default,c=function(){function e(t,n,r){var i=arguments.length>3&&void 0!==arguments[3]?arguments[3]:1;(0,o.default)(this,e),this.parent=t,this.changedOptions=[],this.container=n,this.allowCreation=!1,this.options={},this.initialized=!1,this.popupCounter=0,this.defaultOptions={enabled:!1,filter:!0,container:void 0,showButton:!0},l.extend(this.options,this.defaultOptions),this.configureOptions=r,this.moduleOptions={},this.domElements=[],this.popupDiv={},this.popupLimit=5,this.popupHistory={},this.colorPicker=new u(i),this.wrapper=void 0}return(0,a.default)(e,[{key:"setOptions",value:function(e){if(void 0!==e){this.popupHistory={},this._removePopup();var t=!0;"string"==typeof e?this.options.filter=e:e instanceof Array?this.options.filter=e.join():"object"===(void 0===e?"undefined":(0,i.default)(e))?(void 0!==e.container&&(this.options.container=e.container),void 0!==e.filter&&(this.options.filter=e.filter),void 0!==e.showButton&&(this.options.showButton=e.showButton),void 0!==e.enabled&&(t=e.enabled)):"boolean"==typeof e?(this.options.filter=!0,t=e):"function"==typeof e&&(this.options.filter=e,t=!0),!1===this.options.filter&&(t=!1),this.options.enabled=t}this._clean()}},{key:"setModuleOptions",value:function(e){this.moduleOptions=e,!0===this.options.enabled&&(this._clean(),void 0!==this.options.container&&(this.container=this.options.container),this._create())}},{key:"_create",value:function(){var e=this;this._clean(),this.changedOptions=[];var t=this.options.filter,n=0,r=!1;for(var i in this.configureOptions)this.configureOptions.hasOwnProperty(i)&&(this.allowCreation=!1,r=!1,"function"==typeof t?r=(r=t(i,[]))||this._handleObject(this.configureOptions[i],[i],!0):!0!==t&&-1===t.indexOf(i)||(r=!0),!1!==r&&(this.allowCreation=!0,n>0&&this._makeItem([]),this._makeHeader(i),this._handleObject(this.configureOptions[i],[i])),n++);if(!0===this.options.showButton){var o=document.createElement("div");o.className="vis-configuration vis-config-button",o.innerHTML="generate options",o.onclick=function(){e._printOptions()},o.onmouseover=function(){o.className="vis-configuration vis-config-button hover"},o.onmouseout=function(){o.className="vis-configuration vis-config-button"},this.optionsContainer=document.createElement("div"),this.optionsContainer.className="vis-configuration vis-config-option-container",this.domElements.push(this.optionsContainer),this.domElements.push(o)}this._push()}},{key:"_push",value:function(){this.wrapper=document.createElement("div"),this.wrapper.className="vis-configuration-wrapper",this.container.appendChild(this.wrapper);for(var e=0;e<this.domElements.length;e++)this.wrapper.appendChild(this.domElements[e]);this._showPopupIfNeeded()}},{key:"_clean",value:function(){for(var e=0;e<this.domElements.length;e++)this.wrapper.removeChild(this.domElements[e]);void 0!==this.wrapper&&(this.container.removeChild(this.wrapper),this.wrapper=void 0),this.domElements=[],this._removePopup()}},{key:"_getValue",value:function(e){for(var t=this.moduleOptions,n=0;n<e.length;n++){if(void 0===t[e[n]]){t=void 0;break}t=t[e[n]]}return t}},{key:"_makeItem",value:function(e){if(!0===this.allowCreation){var t=document.createElement("div");t.className="vis-configuration vis-config-item vis-config-s"+e.length;for(var n=arguments.length,r=Array(n>1?n-1:0),i=1;i<n;i++)r[i-1]=arguments[i];return r.forEach((function(e){t.appendChild(e)})),this.domElements.push(t),this.domElements.length}return 0}},{key:"_makeHeader",value:function(e){var t=document.createElement("div");t.className="vis-configuration vis-config-header",t.innerHTML=e,this._makeItem([],t)}},{key:"_makeLabel",value:function(e,t){var n=arguments.length>2&&void 0!==arguments[2]&&arguments[2],r=document.createElement("div");return r.className="vis-configuration vis-config-label vis-config-s"+t.length,r.innerHTML=!0===n?"<i><b>"+e+":</b></i>":e+":",r}},{key:"_makeDropdown",value:function(e,t,n){var r=document.createElement("select");r.className="vis-configuration vis-config-select";var i=0;void 0!==t&&-1!==e.indexOf(t)&&(i=e.indexOf(t));for(var o=0;o<e.length;o++){var a=document.createElement("option");a.value=e[o],o===i&&(a.selected="selected"),a.innerHTML=e[o],r.appendChild(a)}var s=this;r.onchange=function(){s._update(this.value,n)};var l=this._makeLabel(n[n.length-1],n);this._makeItem(n,l,r)}},{key:"_makeRange",value:function(e,t,n){var r=e[0],i=e[1],o=e[2],a=e[3],s=document.createElement("input");s.className="vis-configuration vis-config-range";try{s.type="range",s.min=i,s.max=o}catch(e){}s.step=a;var l="",u=0;void 0!==t?(t<0&&1.2*t<i?(s.min=Math.ceil(1.2*t),u=s.min,l="range increased"):t/1.2<i&&(s.min=Math.ceil(t/1.2),u=s.min,l="range increased"),1.2*t>o&&1!==o&&(s.max=Math.ceil(1.2*t),u=s.max,l="range increased"),s.value=t):s.value=r;var c=document.createElement("input");c.className="vis-configuration vis-config-rangeinput",c.value=s.value;var d=this;s.onchange=function(){c.value=this.value,d._update(Number(this.value),n)},s.oninput=function(){c.value=this.value};var h=this._makeLabel(n[n.length-1],n),f=this._makeItem(n,h,s,c);""!==l&&this.popupHistory[f]!==u&&(this.popupHistory[f]=u,this._setupPopup(l,f))}},{key:"_setupPopup",value:function(e,t){var n=this;if(!0===this.initialized&&!0===this.allowCreation&&this.popupCounter<this.popupLimit){var r=document.createElement("div");r.id="vis-configuration-popup",r.className="vis-configuration-popup",r.innerHTML=e,r.onclick=function(){n._removePopup()},this.popupCounter+=1,this.popupDiv={html:r,index:t}}}},{key:"_removePopup",value:function(){void 0!==this.popupDiv.html&&(this.popupDiv.html.parentNode.removeChild(this.popupDiv.html),clearTimeout(this.popupDiv.hideTimeout),clearTimeout(this.popupDiv.deleteTimeout),this.popupDiv={})}},{key:"_showPopupIfNeeded",value:function(){var e=this;if(void 0!==this.popupDiv.html){var t=this.domElements[this.popupDiv.index].getBoundingClientRect();this.popupDiv.html.style.left=t.left+"px",this.popupDiv.html.style.top=t.top-30+"px",document.body.appendChild(this.popupDiv.html),this.popupDiv.hideTimeout=setTimeout((function(){e.popupDiv.html.style.opacity=0}),1500),this.popupDiv.deleteTimeout=setTimeout((function(){e._removePopup()}),1800)}}},{key:"_makeCheckbox",value:function(e,t,n){var r=document.createElement("input");r.type="checkbox",r.className="vis-configuration vis-config-checkbox",r.checked=e,void 0!==t&&(r.checked=t,t!==e&&("object"===(void 0===e?"undefined":(0,i.default)(e))?t!==e.enabled&&this.changedOptions.push({path:n,value:t}):this.changedOptions.push({path:n,value:t})));var o=this;r.onchange=function(){o._update(this.checked,n)};var a=this._makeLabel(n[n.length-1],n);this._makeItem(n,a,r)}},{key:"_makeTextInput",value:function(e,t,n){var r=document.createElement("input");r.type="text",r.className="vis-configuration vis-config-text",r.value=t,t!==e&&this.changedOptions.push({path:n,value:t});var i=this;r.onchange=function(){i._update(this.value,n)};var o=this._makeLabel(n[n.length-1],n);this._makeItem(n,o,r)}},{key:"_makeColorField",value:function(e,t,n){var r=this,i=e[1],o=document.createElement("div");"none"!==(t=void 0===t?i:t)?(o.className="vis-configuration vis-config-colorBlock",o.style.backgroundColor=t):o.className="vis-configuration vis-config-colorBlock none",t=void 0===t?i:t,o.onclick=function(){r._showColorPicker(t,o,n)};var a=this._makeLabel(n[n.length-1],n);this._makeItem(n,a,o)}},{key:"_showColorPicker",value:function(e,t,n){var r=this;t.onclick=function(){},this.colorPicker.insertTo(t),this.colorPicker.show(),this.colorPicker.setColor(e),this.colorPicker.setUpdateCallback((function(e){var i="rgba("+e.r+","+e.g+","+e.b+","+e.a+")";t.style.backgroundColor=i,r._update(i,n)})),this.colorPicker.setCloseCallback((function(){t.onclick=function(){r._showColorPicker(e,t,n)}}))}},{key:"_handleObject",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:[],n=arguments.length>2&&void 0!==arguments[2]&&arguments[2],r=!1,i=this.options.filter,o=!1;for(var a in e)if(e.hasOwnProperty(a)){r=!0;var s=e[a],u=l.copyAndExtendArray(t,a);if("function"==typeof i&&!1===(r=i(a,t))&&!(s instanceof Array)&&"string"!=typeof s&&"boolean"!=typeof s&&s instanceof Object&&(this.allowCreation=!1,r=this._handleObject(s,u,!0),this.allowCreation=!1===n),!1!==r){o=!0;var c=this._getValue(u);if(s instanceof Array)this._handleArray(s,c,u);else if("string"==typeof s)this._makeTextInput(s,c,u);else if("boolean"==typeof s)this._makeCheckbox(s,c,u);else if(s instanceof Object){var d=!0;if(-1!==t.indexOf("physics")&&this.moduleOptions.physics.solver!==a&&(d=!1),!0===d)if(void 0!==s.enabled){var h=l.copyAndExtendArray(u,"enabled"),f=this._getValue(h);if(!0===f){var p=this._makeLabel(a,u,!0);this._makeItem(u,p),o=this._handleObject(s,u)||o}else this._makeCheckbox(s,f,u)}else{var g=this._makeLabel(a,u,!0);this._makeItem(u,g),o=this._handleObject(s,u)||o}}else console.error("dont know how to handle",s,a,u)}}return o}},{key:"_handleArray",value:function(e,t,n){"string"==typeof e[0]&&"color"===e[0]?(this._makeColorField(e,t,n),e[1]!==t&&this.changedOptions.push({path:n,value:t})):"string"==typeof e[0]?(this._makeDropdown(e,t,n),e[0]!==t&&this.changedOptions.push({path:n,value:t})):"number"==typeof e[0]&&(this._makeRange(e,t,n),e[0]!==t&&this.changedOptions.push({path:n,value:Number(t)}))}},{key:"_update",value:function(e,t){var n=this._constructOptions(e,t);this.parent.body&&this.parent.body.emitter&&this.parent.body.emitter.emit&&this.parent.body.emitter.emit("configChange",n),this.initialized=!0,this.parent.setOptions(n)}},{key:"_constructOptions",value:function(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},r=n;e="false"!==(e="true"===e||e)&&e;for(var i=0;i<t.length;i++)"global"!==t[i]&&(void 0===r[t[i]]&&(r[t[i]]={}),i!==t.length-1?r=r[t[i]]:r[t[i]]=e);return n}},{key:"_printOptions",value:function(){var e=this.getOptions();this.optionsContainer.innerHTML="<pre>var options = "+(0,r.default)(e,null,2)+"</pre>"}},{key:"getOptions",value:function(){for(var e={},t=0;t<this.changedOptions.length;t++)this._constructOptions(this.changedOptions[t].value,this.changedOptions[t].path,e);return e}}]),e}();t.default=c},function(e,t,n){var r,i=n(6),o=(r=i)&&r.__esModule?r:{default:r},a=n(14);function s(e,t){}function l(e,t){return{style:(t=void 0===t?{}:t).style||e.options.drawPoints.style,styles:t.styles||e.options.drawPoints.styles,size:t.size||e.options.drawPoints.size,className:t.className||e.className}}s.draw=function(e,t,n,r){r=r||0;for(var i=function(e,t){var n=void 0;return e.options&&e.options.drawPoints&&e.options.drawPoints.onRender&&"function"==typeof e.options.drawPoints.onRender&&(n=e.options.drawPoints.onRender),t.group.options&&t.group.options.drawPoints&&t.group.options.drawPoints.onRender&&"function"==typeof t.group.options.drawPoints.onRender&&(n=t.group.options.drawPoints.onRender),n}(n,t),s=0;s<e.length;s++)if(i){var u=i(e[s],t);!0!==u&&"object"!==(void 0===u?"undefined":(0,o.default)(u))||a.drawPoint(e[s].screen_x+r,e[s].screen_y,l(t,u),n.svgElements,n.svg,e[s].label)}else a.drawPoint(e[s].screen_x+r,e[s].screen_y,l(t),n.svgElements,n.svg,e[s].label)},s.drawIcon=function(e,t,n,r,i,o){var s=.5*i,u=a.getSVGElement("rect",o.svgElements,o.svg);u.setAttributeNS(null,"x",t),u.setAttributeNS(null,"y",n-s),u.setAttributeNS(null,"width",r),u.setAttributeNS(null,"height",2*s),u.setAttributeNS(null,"class","vis-outline"),a.drawPoint(t+.5*r,n,l(e),o.svgElements,o.svg)},e.exports=s},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=l(n(3)),i=l(n(0)),o=l(n(1)),a=l(n(4)),s=l(n(5));function l(e){return e&&e.__esModule?e:{default:e}}var u=function(e){function t(e,n,o){(0,i.default)(this,t);var s=(0,a.default)(this,(t.__proto__||(0,r.default)(t)).call(this,e,n,o));return s.labelOffset=0,s.selected=!1,s}return(0,s.default)(t,e),(0,o.default)(t,[{key:"setOptions",value:function(e,t,n){this.options=e,void 0===t&&void 0===n||this.setImages(t,n)}},{key:"setImages",value:function(e,t){t&&this.selected?(this.imageObj=t,this.imageObjAlt=e):(this.imageObj=e,this.imageObjAlt=t)}},{key:"switchImages",value:function(e){var t=e&&!this.selected||!e&&this.selected;if(this.selected=e,void 0!==this.imageObjAlt&&t){var n=this.imageObj;this.imageObj=this.imageObjAlt,this.imageObjAlt=n}}},{key:"_resizeImage",value:function(){var e,t;if(!1===this.options.shapeProperties.useImageSize){var n=1,r=1;this.imageObj.width&&this.imageObj.height&&(this.imageObj.width>this.imageObj.height?n=this.imageObj.width/this.imageObj.height:r=this.imageObj.height/this.imageObj.width),e=2*this.options.size*n,t=2*this.options.size*r}else e=this.imageObj.width,t=this.imageObj.height;this.width=e,this.height=t,this.radius=.5*this.width}},{key:"_drawRawCircle",value:function(e,t,n,r){this.initContextForDraw(e,r),e.circle(t,n,r.size),this.performFill(e,r)}},{key:"_drawImageAtPosition",value:function(e,t){if(0!=this.imageObj.width){e.globalAlpha=1,this.enableShadow(e,t);var n=1;!0===this.options.shapeProperties.interpolation&&(n=this.imageObj.width/this.width/this.body.view.scale),this.imageObj.drawImageAtPosition(e,n,this.left,this.top,this.width,this.height),this.disableShadow(e,t)}}},{key:"_drawImageLabel",value:function(e,t,n,r,i){var o,a=0;if(void 0!==this.height){a=.5*this.height;var s=this.labelModule.getTextSize(e,r,i);s.lineCount>=1&&(a+=s.height/2)}o=n+a,this.options.label&&(this.labelOffset=a),this.labelModule.draw(e,t,o,r,i,"hanging")}}]),t}(l(n(23)).default);t.default=u},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=l(n(19)),i=l(n(6)),o=l(n(29)),a=l(n(0)),s=l(n(1));function l(e){return e&&e.__esModule?e:{default:e}}var u=n(2),c=n(117).default,d=n(48).default,h=n(215).default,f=n(217).default,p=n(218).default,g=n(219).default,v=function(){function e(t,n,r,i){if((0,a.default)(this,e),void 0===n)throw new Error("No body provided");this.options=u.bridgeObject(r),this.globalOptions=r,this.defaultOptions=i,this.body=n,this.id=void 0,this.fromId=void 0,this.toId=void 0,this.selected=!1,this.hover=!1,this.labelDirty=!0,this.baseWidth=this.options.width,this.baseFontSize=this.options.font.size,this.from=void 0,this.to=void 0,this.edgeType=void 0,this.connected=!1,this.labelModule=new c(this.body,this.options,!0),this.setOptions(t)}return(0,s.default)(e,[{key:"setOptions",value:function(t){if(t){e.parseOptions(this.options,t,!0,this.globalOptions),void 0!==t.id&&(this.id=t.id),void 0!==t.from&&(this.fromId=t.from),void 0!==t.to&&(this.toId=t.to),void 0!==t.title&&(this.title=t.title),void 0!==t.value&&(t.value=parseFloat(t.value));var n=[t,this.options,this.defaultOptions];this.chooser=d.choosify("edge",n),this.updateLabelModule(t);var r=this.updateEdgeType();return this._setInteractionWidths(),this.connect(),void 0===t.hidden&&void 0===t.physics||(r=!0),r}}},{key:"getFormattingValues",value:function(){var e=!0===this.options.arrows.to||!0===this.options.arrows.to.enabled,t=!0===this.options.arrows.from||!0===this.options.arrows.from.enabled,n=!0===this.options.arrows.middle||!0===this.options.arrows.middle.enabled,r=this.options.color.inherit,i={toArrow:e,toArrowScale:this.options.arrows.to.scaleFactor,toArrowType:this.options.arrows.to.type,middleArrow:n,middleArrowScale:this.options.arrows.middle.scaleFactor,middleArrowType:this.options.arrows.middle.type,fromArrow:t,fromArrowScale:this.options.arrows.from.scaleFactor,fromArrowType:this.options.arrows.from.type,arrowStrikethrough:this.options.arrowStrikethrough,color:r?void 0:this.options.color.color,inheritsColor:r,opacity:this.options.color.opacity,hidden:this.options.hidden,length:this.options.length,shadow:this.options.shadow.enabled,shadowColor:this.options.shadow.color,shadowSize:this.options.shadow.size,shadowX:this.options.shadow.x,shadowY:this.options.shadow.y,dashes:this.options.dashes,width:this.options.width};if(this.selected||this.hover)if(!0===this.chooser){if(this.selected){var o=this.options.selectionWidth;"function"==typeof o?i.width=o(i.width):"number"==typeof o&&(i.width+=o),i.width=Math.max(i.width,.3/this.body.view.scale),i.color=this.options.color.highlight,i.shadow=this.options.shadow.enabled}else if(this.hover){var a=this.options.hoverWidth;"function"==typeof a?i.width=a(i.width):"number"==typeof a&&(i.width+=a),i.width=Math.max(i.width,.3/this.body.view.scale),i.color=this.options.color.hover,i.shadow=this.options.shadow.enabled}}else"function"==typeof this.chooser&&(this.chooser(i,this.options.id,this.selected,this.hover),void 0!==i.color&&(i.inheritsColor=!1),!1===i.shadow&&(i.shadowColor===this.options.shadow.color&&i.shadowSize===this.options.shadow.size&&i.shadowX===this.options.shadow.x&&i.shadowY===this.options.shadow.y||(i.shadow=!0)));else i.shadow=this.options.shadow.enabled,i.width=Math.max(i.width,.3/this.body.view.scale);return i}},{key:"updateLabelModule",value:function(e){var t=[e,this.options,this.globalOptions,this.defaultOptions];this.labelModule.update(this.options,t),void 0!==this.labelModule.baseSize&&(this.baseFontSize=this.labelModule.baseSize)}},{key:"updateEdgeType",value:function(){var e=this.options.smooth,t=!1,n=!0;return void 0!==this.edgeType&&((this.edgeType instanceof f&&!0===e.enabled&&"dynamic"===e.type||this.edgeType instanceof h&&!0===e.enabled&&"cubicBezier"===e.type||this.edgeType instanceof p&&!0===e.enabled&&"dynamic"!==e.type&&"cubicBezier"!==e.type||this.edgeType instanceof g&&!1===e.type.enabled)&&(n=!1),!0===n&&(t=this.cleanup())),!0===n?!0===e.enabled?"dynamic"===e.type?(t=!0,this.edgeType=new f(this.options,this.body,this.labelModule)):"cubicBezier"===e.type?this.edgeType=new h(this.options,this.body,this.labelModule):this.edgeType=new p(this.options,this.body,this.labelModule):this.edgeType=new g(this.options,this.body,this.labelModule):this.edgeType.setOptions(this.options),t}},{key:"connect",value:function(){this.disconnect(),this.from=this.body.nodes[this.fromId]||void 0,this.to=this.body.nodes[this.toId]||void 0,this.connected=void 0!==this.from&&void 0!==this.to,!0===this.connected?(this.from.attachEdge(this),this.to.attachEdge(this)):(this.from&&this.from.detachEdge(this),this.to&&this.to.detachEdge(this)),this.edgeType.connect()}},{key:"disconnect",value:function(){this.from&&(this.from.detachEdge(this),this.from=void 0),this.to&&(this.to.detachEdge(this),this.to=void 0),this.connected=!1}},{key:"getTitle",value:function(){return this.title}},{key:"isSelected",value:function(){return this.selected}},{key:"getValue",value:function(){return this.options.value}},{key:"setValueRange",value:function(e,t,n){if(void 0!==this.options.value){var r=this.options.scaling.customScalingFunction(e,t,n,this.options.value),i=this.options.scaling.max-this.options.scaling.min;if(!0===this.options.scaling.label.enabled){var o=this.options.scaling.label.max-this.options.scaling.label.min;this.options.font.size=this.options.scaling.label.min+r*o}this.options.width=this.options.scaling.min+r*i}else this.options.width=this.baseWidth,this.options.font.size=this.baseFontSize;this._setInteractionWidths(),this.updateLabelModule()}},{key:"_setInteractionWidths",value:function(){"function"==typeof this.options.hoverWidth?this.edgeType.hoverWidth=this.options.hoverWidth(this.options.width):this.edgeType.hoverWidth=this.options.hoverWidth+this.options.width,"function"==typeof this.options.selectionWidth?this.edgeType.selectionWidth=this.options.selectionWidth(this.options.width):this.edgeType.selectionWidth=this.options.selectionWidth+this.options.width}},{key:"draw",value:function(e){var t=this.getFormattingValues();if(!t.hidden){var n=this.edgeType.getViaNode(),r={};this.edgeType.fromPoint=this.edgeType.from,this.edgeType.toPoint=this.edgeType.to,t.fromArrow&&(r.from=this.edgeType.getArrowData(e,"from",n,this.selected,this.hover,t),!1===t.arrowStrikethrough&&(this.edgeType.fromPoint=r.from.core)),t.toArrow&&(r.to=this.edgeType.getArrowData(e,"to",n,this.selected,this.hover,t),!1===t.arrowStrikethrough&&(this.edgeType.toPoint=r.to.core)),t.middleArrow&&(r.middle=this.edgeType.getArrowData(e,"middle",n,this.selected,this.hover,t)),this.edgeType.drawLine(e,t,this.selected,this.hover,n),this.drawArrows(e,r,t),this.drawLabel(e,n)}}},{key:"drawArrows",value:function(e,t,n){n.fromArrow&&this.edgeType.drawArrowHead(e,n,this.selected,this.hover,t.from),n.middleArrow&&this.edgeType.drawArrowHead(e,n,this.selected,this.hover,t.middle),n.toArrow&&this.edgeType.drawArrowHead(e,n,this.selected,this.hover,t.to)}},{key:"drawLabel",value:function(e,t){if(void 0!==this.options.label){var n=this.from,r=this.to;if(this.labelModule.differentState(this.selected,this.hover)&&this.labelModule.getTextSize(e,this.selected,this.hover),n.id!=r.id){this.labelModule.pointToSelf=!1;var i=this.edgeType.getPoint(.5,t);e.save();var o=this._getRotation(e);0!=o.angle&&(e.translate(o.x,o.y),e.rotate(o.angle)),this.labelModule.draw(e,i.x,i.y,this.selected,this.hover),e.restore()}else{var a,s;this.labelModule.pointToSelf=!0;var l=this.options.selfReferenceSize;n.shape.width>n.shape.height?(a=n.x+.5*n.shape.width,s=n.y-l):(a=n.x+l,s=n.y-.5*n.shape.height),i=this._pointOnCircle(a,s,l,.125),this.labelModule.draw(e,i.x,i.y,this.selected,this.hover)}}}},{key:"getItemsOnPoint",value:function(e){var t=[];if(this.labelModule.visible()){var n=this._getRotation();d.pointInRect(this.labelModule.getSize(),e,n)&&t.push({edgeId:this.id,labelId:0})}var r={left:e.x,top:e.y};return this.isOverlappingWith(r)&&t.push({edgeId:this.id}),t}},{key:"isOverlappingWith",value:function(e){if(this.connected){var t=this.from.x,n=this.from.y,r=this.to.x,i=this.to.y,o=e.left,a=e.top;return this.edgeType.getDistanceToEdge(t,n,r,i,o,a)<10}return!1}},{key:"_getRotation",value:function(e){var t=this.edgeType.getViaNode(),n=this.edgeType.getPoint(.5,t);void 0!==e&&this.labelModule.calculateLabelSize(e,this.selected,this.hover,n.x,n.y);var r={x:n.x,y:this.labelModule.size.yLine,angle:0};if(!this.labelModule.visible())return r;if("horizontal"===this.options.font.align)return r;var i=this.from.y-this.to.y,o=this.from.x-this.to.x,a=Math.atan2(i,o);return(a<-1&&o<0||a>0&&o<0)&&(a+=Math.PI),r.angle=a,r}},{key:"_pointOnCircle",value:function(e,t,n,r){var i=2*r*Math.PI;return{x:e+n*Math.cos(i),y:t-n*Math.sin(i)}}},{key:"select",value:function(){this.selected=!0}},{key:"unselect",value:function(){this.selected=!1}},{key:"cleanup",value:function(){return this.edgeType.cleanup()}},{key:"remove",value:function(){this.cleanup(),this.disconnect(),delete this.body.edges[this.id]}},{key:"endPointsValid",value:function(){return void 0!==this.body.nodes[this.fromId]&&void 0!==this.body.nodes[this.toId]}}],[{key:"parseOptions",value:function(e,t){var n=arguments.length>2&&void 0!==arguments[2]&&arguments[2],a=arguments.length>3&&void 0!==arguments[3]?arguments[3]:{},s=arguments.length>4&&void 0!==arguments[4]&&arguments[4],l=["arrowStrikethrough","id","from","hidden","hoverWidth","labelHighlightBold","length","line","opacity","physics","scaling","selectionWidth","selfReferenceSize","to","title","value","width","font","chosen","widthConstraint"];if(u.selectiveDeepExtend(l,e,t,n),d.isValidLabel(t.label)?e.label=t.label:e.label=void 0,u.mergeOptions(e,t,"smooth",a),u.mergeOptions(e,t,"shadow",a),void 0!==t.dashes&&null!==t.dashes?e.dashes=t.dashes:!0===n&&null===t.dashes&&(e.dashes=(0,o.default)(a.dashes)),void 0!==t.scaling&&null!==t.scaling?(void 0!==t.scaling.min&&(e.scaling.min=t.scaling.min),void 0!==t.scaling.max&&(e.scaling.max=t.scaling.max),u.mergeOptions(e.scaling,t.scaling,"label",a.scaling)):!0===n&&null===t.scaling&&(e.scaling=(0,o.default)(a.scaling)),void 0!==t.arrows&&null!==t.arrows)if("string"==typeof t.arrows){var c=t.arrows.toLowerCase();e.arrows.to.enabled=-1!=c.indexOf("to"),e.arrows.middle.enabled=-1!=c.indexOf("middle"),e.arrows.from.enabled=-1!=c.indexOf("from")}else{if("object"!==(0,i.default)(t.arrows))throw new Error("The arrow newOptions can only be an object or a string. Refer to the documentation. You used:"+(0,r.default)(t.arrows));u.mergeOptions(e.arrows,t.arrows,"to",a.arrows),u.mergeOptions(e.arrows,t.arrows,"middle",a.arrows),u.mergeOptions(e.arrows,t.arrows,"from",a.arrows)}else!0===n&&null===t.arrows&&(e.arrows=(0,o.default)(a.arrows));if(void 0!==t.color&&null!==t.color){var h=t.color,f=e.color;if(s)u.deepExtend(f,a.color,!1,n);else for(var p in f)f.hasOwnProperty(p)&&delete f[p];if(u.isString(f))f.color=f,f.highlight=f,f.hover=f,f.inherit=!1,void 0===h.opacity&&(f.opacity=1);else{var g=!1;void 0!==h.color&&(f.color=h.color,g=!0),void 0!==h.highlight&&(f.highlight=h.highlight,g=!0),void 0!==h.hover&&(f.hover=h.hover,g=!0),void 0!==h.inherit&&(f.inherit=h.inherit),void 0!==h.opacity&&(f.opacity=Math.min(1,Math.max(0,h.opacity))),!0===g?f.inherit=!1:void 0===f.inherit&&(f.inherit="from")}}else!0===n&&null===t.color&&(e.color=u.bridgeObject(a.color));!0===n&&null===t.font&&(e.font=u.bridgeObject(a.font))}}]),e}();t.default=v},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=l(n(3)),i=l(n(0)),o=l(n(1)),a=l(n(4)),s=l(n(5));function l(e){return e&&e.__esModule?e:{default:e}}var u=function(e){function t(e,n,o){return(0,i.default)(this,t),(0,a.default)(this,(t.__proto__||(0,r.default)(t)).call(this,e,n,o))}return(0,s.default)(t,e),(0,o.default)(t,[{key:"_findBorderPositionBezier",value:function(e,t){var n,r,i,o=arguments.length>2&&void 0!==arguments[2]?arguments[2]:this._getViaCoordinates(),a=10,s=0,l=0,u=1,c=.2,d=this.to,h=!1;for(e.id===this.from.id&&(d=this.from,h=!0);l<=u&&s<a;){var f=.5*(l+u);if(n=this.getPoint(f,o),r=Math.atan2(d.y-n.y,d.x-n.x),i=d.distanceToBorder(t,r)-Math.sqrt(Math.pow(n.x-d.x,2)+Math.pow(n.y-d.y,2)),Math.abs(i)<c)break;i<0?!1===h?l=f:u=f:!1===h?u=f:l=f,s++}return n.t=f,n}},{key:"_getDistanceToBezierEdge",value:function(e,t,n,r,i,o,a){var s=1e9,l=void 0,u=void 0,c=void 0,d=void 0,h=void 0,f=e,p=t;for(u=1;u<10;u++)c=.1*u,d=Math.pow(1-c,2)*e+2*c*(1-c)*a.x+Math.pow(c,2)*n,h=Math.pow(1-c,2)*t+2*c*(1-c)*a.y+Math.pow(c,2)*r,u>0&&(s=(l=this._getDistanceToLine(f,p,d,h,i,o))<s?l:s),f=d,p=h;return s}},{key:"_bezierCurve",value:function(e,t,n,r){var i=void 0!==n&&void 0!==n.x,o=void 0!==r&&void 0!==r.x;e.beginPath(),e.moveTo(this.fromPoint.x,this.fromPoint.y),i&&o?e.bezierCurveTo(n.x,n.y,r.x,r.y,this.toPoint.x,this.toPoint.y):i?e.quadraticCurveTo(n.x,n.y,this.toPoint.x,this.toPoint.y):e.lineTo(this.toPoint.x,this.toPoint.y),this.enableShadow(e,t),e.stroke(),this.disableShadow(e,t)}},{key:"getViaNode",value:function(){return this._getViaCoordinates()}}]),t}(l(n(118)).default);t.default=u},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=o(n(0)),i=o(n(1));function o(e){return e&&e.__esModule?e:{default:e}}var a=n(2),s=function(){function e(){(0,r.default)(this,e)}return(0,i.default)(e,null,[{key:"getRange",value:function(e){var t,n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:[],r=1e9,i=-1e9,o=1e9,a=-1e9;if(n.length>0)for(var s=0;s<n.length;s++)o>(t=e[n[s]]).shape.boundingBox.left&&(o=t.shape.boundingBox.left),a<t.shape.boundingBox.right&&(a=t.shape.boundingBox.right),r>t.shape.boundingBox.top&&(r=t.shape.boundingBox.top),i<t.shape.boundingBox.bottom&&(i=t.shape.boundingBox.bottom);return 1e9===o&&-1e9===a&&1e9===r&&-1e9===i&&(r=0,i=0,o=0,a=0),{minX:o,maxX:a,minY:r,maxY:i}}},{key:"getRangeCore",value:function(e){var t,n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:[],r=1e9,i=-1e9,o=1e9,a=-1e9;if(n.length>0)for(var s=0;s<n.length;s++)o>(t=e[n[s]]).x&&(o=t.x),a<t.x&&(a=t.x),r>t.y&&(r=t.y),i<t.y&&(i=t.y);return 1e9===o&&-1e9===a&&1e9===r&&-1e9===i&&(r=0,i=0,o=0,a=0),{minX:o,maxX:a,minY:r,maxY:i}}},{key:"findCenter",value:function(e){return{x:.5*(e.maxX+e.minX),y:.5*(e.maxY+e.minY)}}},{key:"cloneOptions",value:function(e,t){var n={};return void 0===t||"node"===t?(a.deepExtend(n,e.options,!0),n.x=e.x,n.y=e.y,n.amountOfConnections=e.edges.length):a.deepExtend(n,e.options,!0),n}}]),e}();t.default=s},function(e,t,n){e.exports={default:n(124),__esModule:!0}},function(e,t,n){var r=n(50);e.exports=Object("z").propertyIsEnumerable(0)?Object:function(e){return"String"==r(e)?e.split(""):Object(e)}},function(e,t,n){var r=n(52),i=n(17),o=n(83),a=n(26),s=n(22),l=n(31),u=n(129),c=n(59),d=n(85),h=n(13)("iterator"),f=!([].keys&&"next"in[].keys()),p=function(){return this};e.exports=function(e,t,n,g,v,m,y){u(n,t,g);var b,C,A,_=function(e){if(!f&&e in M)return M[e];switch(e){case"keys":case"values":return function(){return new n(this,e)}}return function(){return new n(this,e)}},I=t+" Iterator",x="values"==v,w=!1,M=e.prototype,S=M[h]||M["@@iterator"]||v&&M[v],T=S||_(v),E=v?x?_("entries"):T:void 0,O="Array"==t&&M.entries||S;if(O&&(A=d(O.call(new e)))!==Object.prototype&&A.next&&(c(A,I,!0),r||s(A,h)||a(A,h,p)),x&&S&&"values"!==S.name&&(w=!0,T=function(){return S.call(this)}),r&&!y||!f&&!w&&M[h]||a(M,h,T),l[t]=T,l[I]=p,v)if(b={values:x?T:_("values"),keys:m?T:_("keys"),entries:E},y)for(C in b)C in M||o(M,C,b[C]);else i(i.P+i.F*(f||w),t,b);return b}},function(e,t,n){var r=n(128);e.exports=function(e,t,n){if(r(e),void 0===t)return e;switch(n){case 1:return function(n){return e.call(t,n)};case 2:return function(n,r){return e.call(t,n,r)};case 3:return function(n,r,i){return e.call(t,n,r,i)}}return function(){return e.apply(t,arguments)}}},function(e,t,n){e.exports=!n(21)&&!n(28)((function(){return 7!=Object.defineProperty(n(82)("div"),"a",{get:function(){return 7}}).a}))},function(e,t,n){var r=n(32),i=n(18).document,o=r(i)&&r(i.createElement);e.exports=function(e){return o?i.createElement(e):{}}},function(e,t,n){e.exports=n(26)},function(e,t,n){var r=n(22),i=n(25),o=n(131)(!1),a=n(56)("IE_PROTO");e.exports=function(e,t){var n,s=i(e),l=0,u=[];for(n in s)n!=a&&r(s,n)&&u.push(n);for(;t.length>l;)r(s,n=t[l++])&&(~o(u,n)||u.push(n));return u}},function(e,t,n){var r=n(22),i=n(41),o=n(56)("IE_PROTO"),a=Object.prototype;e.exports=Object.getPrototypeOf||function(e){return e=i(e),r(e,o)?e[o]:"function"==typeof e.constructor&&e instanceof e.constructor?e.constructor.prototype:e instanceof Object?a:null}},function(e,t,n){var r=n(50),i=n(13)("toStringTag"),o="Arguments"==r(function(){return arguments}());e.exports=function(e){var t,n,a;return void 0===e?"Undefined":null===e?"Null":"string"==typeof(n=function(e,t){try{return e[t]}catch(e){}}(t=Object(e),i))?n:o?r(t):"Object"==(a=r(t))&&"function"==typeof t.callee?"Arguments":a}},function(e,t,n){var r=n(17),i=n(7),o=n(28);e.exports=function(e,t){var n=(i.Object||{})[e]||Object[e],a={};a[e]=t(n),r(r.S+r.F*o((function(){n(1)})),"Object",a)}},function(e,t,n){var r=n(84),i=n(58).concat("length","prototype");t.f=Object.getOwnPropertyNames||function(e){return r(e,i)}},function(e,t,n){var r=n(42),i=n(39),o=n(25),a=n(53),s=n(22),l=n(81),u=Object.getOwnPropertyDescriptor;t.f=n(21)?u:function(e,t){if(e=o(e),t=a(t,!0),l)try{return u(e,t)}catch(e){}if(s(e,t))return i(!r.f.call(e,t),e[t])}},function(e,t,n){e.exports={default:n(162),__esModule:!0}},function(e,t,n){e.exports=function(e,t){this.x=void 0!==e?e:0,this.y=void 0!==t?t:0}},function(e,t,n){var r=n(2);function i(e,t){if(void 0===e)throw new Error("No container element defined");if(this.container=e,this.visible=!t||null==t.visible||t.visible,this.visible){this.frame=document.createElement("DIV"),this.frame.style.width="100%",this.frame.style.position="relative",this.container.appendChild(this.frame),this.frame.prev=document.createElement("INPUT"),this.frame.prev.type="BUTTON",this.frame.prev.value="Prev",this.frame.appendChild(this.frame.prev),this.frame.play=document.createElement("INPUT"),this.frame.play.type="BUTTON",this.frame.play.value="Play",this.frame.appendChild(this.frame.play),this.frame.next=document.createElement("INPUT"),this.frame.next.type="BUTTON",this.frame.next.value="Next",this.frame.appendChild(this.frame.next),this.frame.bar=document.createElement("INPUT"),this.frame.bar.type="BUTTON",this.frame.bar.style.position="absolute",this.frame.bar.style.border="1px solid red",this.frame.bar.style.width="100px",this.frame.bar.style.height="6px",this.frame.bar.style.borderRadius="2px",this.frame.bar.style.MozBorderRadius="2px",this.frame.bar.style.border="1px solid #7F7F7F",this.frame.bar.style.backgroundColor="#E5E5E5",this.frame.appendChild(this.frame.bar),this.frame.slide=document.createElement("INPUT"),this.frame.slide.type="BUTTON",this.frame.slide.style.margin="0px",this.frame.slide.value=" ",this.frame.slide.style.position="relative",this.frame.slide.style.left="-100px",this.frame.appendChild(this.frame.slide);var n=this;this.frame.slide.onmousedown=function(e){n._onMouseDown(e)},this.frame.prev.onclick=function(e){n.prev(e)},this.frame.play.onclick=function(e){n.togglePlay(e)},this.frame.next.onclick=function(e){n.next(e)}}this.onChangeCallback=void 0,this.values=[],this.index=void 0,this.playTimeout=void 0,this.playInterval=1e3,this.playLoop=!0}i.prototype.prev=function(){var e=this.getIndex();e>0&&(e--,this.setIndex(e))},i.prototype.next=function(){var e=this.getIndex();e<this.values.length-1&&(e++,this.setIndex(e))},i.prototype.playNext=function(){var e=new Date,t=this.getIndex();t<this.values.length-1?(t++,this.setIndex(t)):this.playLoop&&(t=0,this.setIndex(t));var n=new Date-e,r=Math.max(this.playInterval-n,0),i=this;this.playTimeout=setTimeout((function(){i.playNext()}),r)},i.prototype.togglePlay=function(){void 0===this.playTimeout?this.play():this.stop()},i.prototype.play=function(){this.playTimeout||(this.playNext(),this.frame&&(this.frame.play.value="Stop"))},i.prototype.stop=function(){clearInterval(this.playTimeout),this.playTimeout=void 0,this.frame&&(this.frame.play.value="Play")},i.prototype.setOnChangeCallback=function(e){this.onChangeCallback=e},i.prototype.setPlayInterval=function(e){this.playInterval=e},i.prototype.getPlayInterval=function(){return this.playInterval},i.prototype.setPlayLoop=function(e){this.playLoop=e},i.prototype.onChange=function(){void 0!==this.onChangeCallback&&this.onChangeCallback()},i.prototype.redraw=function(){if(this.frame){this.frame.bar.style.top=this.frame.clientHeight/2-this.frame.bar.offsetHeight/2+"px",this.frame.bar.style.width=this.frame.clientWidth-this.frame.prev.clientWidth-this.frame.play.clientWidth-this.frame.next.clientWidth-30+"px";var e=this.indexToLeft(this.index);this.frame.slide.style.left=e+"px"}},i.prototype.setValues=function(e){this.values=e,this.values.length>0?this.setIndex(0):this.index=void 0},i.prototype.setIndex=function(e){if(!(e<this.values.length))throw new Error("Index out of range");this.index=e,this.redraw(),this.onChange()},i.prototype.getIndex=function(){return this.index},i.prototype.get=function(){return this.values[this.index]},i.prototype._onMouseDown=function(e){if(e.which?1===e.which:1===e.button){this.startClientX=e.clientX,this.startSlideX=parseFloat(this.frame.slide.style.left),this.frame.style.cursor="move";var t=this;this.onmousemove=function(e){t._onMouseMove(e)},this.onmouseup=function(e){t._onMouseUp(e)},r.addEventListener(document,"mousemove",this.onmousemove),r.addEventListener(document,"mouseup",this.onmouseup),r.preventDefault(e)}},i.prototype.leftToIndex=function(e){var t=parseFloat(this.frame.bar.style.width)-this.frame.slide.clientWidth-10,n=e-3,r=Math.round(n/t*(this.values.length-1));return r<0&&(r=0),r>this.values.length-1&&(r=this.values.length-1),r},i.prototype.indexToLeft=function(e){var t=parseFloat(this.frame.bar.style.width)-this.frame.slide.clientWidth-10;return e/(this.values.length-1)*t+3},i.prototype._onMouseMove=function(e){var t=e.clientX-this.startClientX,n=this.startSlideX+t,i=this.leftToIndex(n);this.setIndex(i),r.preventDefault()},i.prototype._onMouseUp=function(e){this.frame.style.cursor="auto",r.removeEventListener(document,"mousemove",this.onmousemove),r.removeEventListener(document,"mouseup",this.onmouseup),r.preventDefault()},e.exports=i},function(e,t,n){function r(e,t,n,r){this._start=0,this._end=0,this._step=1,this.prettyStep=!0,this.precision=5,this._current=0,this.setRange(e,t,n,r)}r.prototype.isNumeric=function(e){return!isNaN(parseFloat(e))&&isFinite(e)},r.prototype.setRange=function(e,t,n,r){if(!this.isNumeric(e))throw new Error("Parameter 'start' is not numeric; value: "+e);if(!this.isNumeric(t))throw new Error("Parameter 'end' is not numeric; value: "+e);if(!this.isNumeric(n))throw new Error("Parameter 'step' is not numeric; value: "+e);this._start=e||0,this._end=t||0,this.setStep(n,r)},r.prototype.setStep=function(e,t){void 0===e||e<=0||(void 0!==t&&(this.prettyStep=t),!0===this.prettyStep?this._step=r.calculatePrettyStep(e):this._step=e)},r.calculatePrettyStep=function(e){var t=function(e){return Math.log(e)/Math.LN10},n=Math.pow(10,Math.round(t(e))),r=2*Math.pow(10,Math.round(t(e/2))),i=5*Math.pow(10,Math.round(t(e/5))),o=n;return Math.abs(r-e)<=Math.abs(o-e)&&(o=r),Math.abs(i-e)<=Math.abs(o-e)&&(o=i),o<=0&&(o=1),o},r.prototype.getCurrent=function(){return parseFloat(this._current.toPrecision(this.precision))},r.prototype.getStep=function(){return this._step},r.prototype.start=function(e){void 0===e&&(e=!1),this._current=this._start-this._start%this._step,e&&this.getCurrent()<this._start&&this.next()},r.prototype.next=function(){this._current+=this._step},r.prototype.end=function(){return this._current>this._end},e.exports=r},function(e,t,n){var r,i=n(6),o=(r=i)&&r.__esModule?r:{default:r},a=n(2),s=n(95),l=n(34),u={BAR:0,BARCOLOR:1,BARSIZE:2,DOT:3,DOTLINE:4,DOTCOLOR:5,DOTSIZE:6,GRID:7,LINE:8,SURFACE:9},c={dot:u.DOT,"dot-line":u.DOTLINE,"dot-color":u.DOTCOLOR,"dot-size":u.DOTSIZE,line:u.LINE,grid:u.GRID,surface:u.SURFACE,bar:u.BAR,"bar-color":u.BARCOLOR,"bar-size":u.BARSIZE},d=["width","height","filterLabel","legendLabel","xLabel","yLabel","zLabel","xValueLabel","yValueLabel","zValueLabel","showXAxis","showYAxis","showZAxis","showGrid","showPerspective","showShadow","keepAspectRatio","verticalRatio","dotSizeRatio","dotSizeMinFraction","dotSizeMaxFraction","showAnimationControls","animationInterval","animationPreload","animationAutoStart","axisColor","gridColor","xCenter","yCenter"],h=["xBarWidth","yBarWidth","valueMin","valueMax","xMin","xMax","xStep","yMin","yMax","yStep","zMin","zMax","zStep"],f=void 0;function p(e){for(var t in e)if(e.hasOwnProperty(t))return!1;return!0}function g(e,t){return void 0===e||""===e?t:e+(void 0===(n=t)||""===n||"string"!=typeof n?n:n.charAt(0).toUpperCase()+n.slice(1));var n}function v(e,t,n,r){for(var i,o=0;o<n.length;++o)t[g(r,i=n[o])]=e[i]}function m(e,t,n,r){for(var i,o=0;o<n.length;++o)void 0!==e[i=n[o]]&&(t[g(r,i)]=e[i])}function y(e,t){void 0!==e.backgroundColor&&function(e,t){var n="white",r="gray",i=1;if("string"==typeof e)n=e,r="none",i=0;else{if("object"!==(void 0===e?"undefined":(0,o.default)(e)))throw new Error("Unsupported type of backgroundColor");void 0!==e.fill&&(n=e.fill),void 0!==e.stroke&&(r=e.stroke),void 0!==e.strokeWidth&&(i=e.strokeWidth)}t.frame.style.backgroundColor=n,t.frame.style.borderColor=r,t.frame.style.borderWidth=i+"px",t.frame.style.borderStyle="solid"}(e.backgroundColor,t),function(e,t){void 0!==e&&(void 0===t.dataColor&&(t.dataColor={}),"string"==typeof e?(t.dataColor.fill=e,t.dataColor.stroke=e):(e.fill&&(t.dataColor.fill=e.fill),e.stroke&&(t.dataColor.stroke=e.stroke),void 0!==e.strokeWidth&&(t.dataColor.strokeWidth=e.strokeWidth)))}(e.dataColor,t),function(e,t){if(void 0!==e){var n,r;if("string"==typeof e){if(-1===(n=void 0===(r=c[e])?-1:r))throw new Error("Style '"+e+"' is invalid")}else{if(!function(e){var t=!1;for(var n in u)if(u[n]===e){t=!0;break}return t}(e))throw new Error("Style '"+e+"' is invalid");n=e}t.style=n}}(e.style,t),function(e,t){if(void 0===e){if(void 0===f.showLegend){var n=t.style===u.DOTCOLOR||t.style===u.DOTSIZE;t.showLegend=n}}else t.showLegend=e}(e.showLegend,t),b(e.cameraPosition,t),void 0!==e.tooltip&&(t.showTooltip=e.tooltip),null!=e.onclick&&(t.onclick_callback=e.onclick),void 0!==e.tooltipStyle&&a.selectiveDeepExtend(["tooltipStyle"],t,e)}function b(e,t){var n=e;void 0!==n&&(void 0===t.camera&&(t.camera=new s),t.camera.setArmRotation(n.horizontal,n.vertical),t.camera.setArmLength(n.distance))}e.exports.STYLE=u,e.exports.setDefaults=function(e,t){if(void 0===e||p(e))throw new Error("No DEFAULTS passed");if(void 0===t)throw new Error("No dst passed");f=e,v(e,t,d),v(e,t,h,"default"),y(e,t),t.margin=10,t.showGrayBottom=!1,t.showTooltip=!1,t.onclick_callback=null,t.eye=new l(0,0,-1)},e.exports.setOptions=function(e,t){if(void 0!==e){if(void 0===t)throw new Error("No dst passed");if(void 0===f||p(f))throw new Error("DEFAULTS not set for module Settings");m(e,t,d),m(e,t,h,"default"),y(e,t)}},e.exports.setCameraPosition=b},function(e,t,n){var r,i=n(165),o=(r=i)&&r.__esModule?r:{default:r},a=n(34);function s(){this.armLocation=new a,this.armRotation={},this.armRotation.horizontal=0,this.armRotation.vertical=0,this.armLength=1.7,this.cameraOffset=new a,this.offsetMultiplier=.6,this.cameraLocation=new a,this.cameraRotation=new a(.5*Math.PI,0,0),this.calculateCameraOrientation()}s.prototype.setOffset=function(e,t){var n=Math.abs,r=o.default,i=this.offsetMultiplier,a=this.armLength*i;n(e)>a&&(e=r(e)*a),n(t)>a&&(t=r(t)*a),this.cameraOffset.x=e,this.cameraOffset.y=t,this.calculateCameraOrientation()},s.prototype.getOffset=function(){return this.cameraOffset},s.prototype.setArmLocation=function(e,t,n){this.armLocation.x=e,this.armLocation.y=t,this.armLocation.z=n,this.calculateCameraOrientation()},s.prototype.setArmRotation=function(e,t){void 0!==e&&(this.armRotation.horizontal=e),void 0!==t&&(this.armRotation.vertical=t,this.armRotation.vertical<0&&(this.armRotation.vertical=0),this.armRotation.vertical>.5*Math.PI&&(this.armRotation.vertical=.5*Math.PI)),void 0===e&&void 0===t||this.calculateCameraOrientation()},s.prototype.getArmRotation=function(){var e={};return e.horizontal=this.armRotation.horizontal,e.vertical=this.armRotation.vertical,e},s.prototype.setArmLength=function(e){void 0!==e&&(this.armLength=e,this.armLength<.71&&(this.armLength=.71),this.armLength>5&&(this.armLength=5),this.setOffset(this.cameraOffset.x,this.cameraOffset.y),this.calculateCameraOrientation())},s.prototype.getArmLength=function(){return this.armLength},s.prototype.getCameraLocation=function(){return this.cameraLocation},s.prototype.getCameraRotation=function(){return this.cameraRotation},s.prototype.calculateCameraOrientation=function(){this.cameraLocation.x=this.armLocation.x-this.armLength*Math.sin(this.armRotation.horizontal)*Math.cos(this.armRotation.vertical),this.cameraLocation.y=this.armLocation.y-this.armLength*Math.cos(this.armRotation.horizontal)*Math.cos(this.armRotation.vertical),this.cameraLocation.z=this.armLocation.z+this.armLength*Math.sin(this.armRotation.vertical),this.cameraRotation.x=Math.PI/2-this.armRotation.vertical,this.cameraRotation.y=0,this.cameraRotation.z=-this.armRotation.horizontal;var e=this.cameraRotation.x,t=this.cameraRotation.z,n=this.cameraOffset.x,r=this.cameraOffset.y,i=Math.sin,o=Math.cos;this.cameraLocation.x=this.cameraLocation.x+n*o(t)+r*-i(t)*o(e),this.cameraLocation.y=this.cameraLocation.y+n*i(t)+r*o(t)*o(e),this.cameraLocation.z=this.cameraLocation.z+r*i(e)},e.exports=s},function(e,t,n){var r=n(12);function i(e,t,n){this.dataGroup=e,this.column=t,this.graph=n,this.index=void 0,this.value=void 0,this.values=e.getDistinctValues(this.column),this.values.length>0&&this.selectValue(0),this.dataPoints=[],this.loaded=!1,this.onLoadCallback=void 0,n.animationPreload?(this.loaded=!1,this.loadInBackground()):this.loaded=!0}i.prototype.isLoaded=function(){return this.loaded},i.prototype.getLoadedProgress=function(){for(var e=this.values.length,t=0;this.dataPoints[t];)t++;return Math.round(t/e*100)},i.prototype.getLabel=function(){return this.graph.filterLabel},i.prototype.getColumn=function(){return this.column},i.prototype.getSelectedValue=function(){if(void 0!==this.index)return this.values[this.index]},i.prototype.getValues=function(){return this.values},i.prototype.getValue=function(e){if(e>=this.values.length)throw new Error("Index out of range");return this.values[e]},i.prototype._getDataPoints=function(e){if(void 0===e&&(e=this.index),void 0===e)return[];var t;if(this.dataPoints[e])t=this.dataPoints[e];else{var n={};n.column=this.column,n.value=this.values[e];var i=new r(this.dataGroup.getDataSet(),{filter:function(e){return e[n.column]==n.value}}).get();t=this.dataGroup._getDataPoints(i),this.dataPoints[e]=t}return t},i.prototype.setOnLoadCallback=function(e){this.onLoadCallback=e},i.prototype.selectValue=function(e){if(e>=this.values.length)throw new Error("Index out of range");this.index=e,this.value=this.values[e]},i.prototype.loadInBackground=function(e){void 0===e&&(e=0);var t=this.graph.frame;if(e<this.values.length){void 0===t.progress&&(t.progress=document.createElement("DIV"),t.progress.style.position="absolute",t.progress.style.color="gray",t.appendChild(t.progress));var n=this.getLoadedProgress();t.progress.innerHTML="Loading animation... "+n+"%",t.progress.style.bottom="60px",t.progress.style.left="10px";var r=this;setTimeout((function(){r.loadInBackground(e+1)}),10),this.loaded=!1}else this.loaded=!0,void 0!==t.progress&&(t.removeChild(t.progress),t.progress=void 0),this.onLoadCallback&&this.onLoadCallback()},e.exports=i},function(e,t,n){var r=n(35),i=n(44),o=n(10),a=n(2);function s(e){this.active=!1,this.dom={container:e},this.dom.overlay=document.createElement("div"),this.dom.overlay.className="vis-overlay",this.dom.container.appendChild(this.dom.overlay),this.hammer=o(this.dom.overlay),this.hammer.on("tap",this._onTapOverlay.bind(this));var t=this;["tap","doubletap","press","pinch","pan","panstart","panmove","panend"].forEach((function(e){t.hammer.on(e,(function(e){e.stopPropagation()}))})),document&&document.body&&(this.onClick=function(n){(function(e,t){for(;e;){if(e===t)return!0;e=e.parentNode}return!1})(n.target,e)||t.deactivate()},document.body.addEventListener("click",this.onClick)),void 0!==this.keycharm&&this.keycharm.destroy(),this.keycharm=r(),this.escListener=this.deactivate.bind(this)}i(s.prototype),s.current=null,s.prototype.destroy=function(){this.deactivate(),this.dom.overlay.parentNode.removeChild(this.dom.overlay),this.onClick&&document.body.removeEventListener("click",this.onClick),this.hammer.destroy(),this.hammer=null},s.prototype.activate=function(){s.current&&s.current.deactivate(),s.current=this,this.active=!0,this.dom.overlay.style.display="none",a.addClassName(this.dom.container,"vis-active"),this.emit("change"),this.emit("activate"),this.keycharm.bind("esc",this.escListener)},s.prototype.deactivate=function(){this.active=!1,this.dom.overlay.style.display="",a.removeClassName(this.dom.container,"vis-active"),this.keycharm.unbind("esc",this.escListener),this.emit("change"),this.emit("deactivate")},s.prototype._onTapOverlay=function(e){this.activate(),e.stopPropagation()},e.exports=s},function(e,t,n){t.en={current:"current",time:"time"},t.en_EN=t.en,t.en_US=t.en,t.it={current:"attuale",time:"tempo"},t.it_IT=t.it,t.it_CH=t.it,t.nl={current:"huidige",time:"tijd"},t.nl_NL=t.nl,t.nl_BE=t.nl,t.de={current:"Aktuelle",time:"Zeit"},t.de_DE=t.de,t.fr={current:"actuel",time:"heure"},t.fr_FR=t.fr,t.fr_CA=t.fr,t.fr_BE=t.fr,t.es={current:"corriente",time:"hora"},t.es_ES=t.es},function(e,t,n){var r=o(n(29)),i=o(n(6));function o(e){return e&&e.__esModule?e:{default:e}}var a=n(10),s=n(2),l=n(11),u=n(12),c=n(66),d=n(16),h=n(68),f=n(69),p=n(101),g=n(102),v=n(70),m=n(103),y=n(104).default,b="__ungrouped__",C="__background__";function A(e,t){this.body=e,this.defaultOptions={type:null,orientation:{item:"bottom"},align:"auto",stack:!0,stackSubgroups:!0,groupOrderSwap:function(e,t,n){var r=t.order;t.order=e.order,e.order=r},groupOrder:"order",selectable:!0,multiselect:!1,itemsAlwaysDraggable:{item:!1,range:!1},editable:{updateTime:!1,updateGroup:!1,add:!1,remove:!1,overrideItems:!1},groupEditable:{order:!1,add:!1,remove:!1},snap:c.snap,onDropObjectOnItem:function(e,t,n){n(t)},onAdd:function(e,t){t(e)},onUpdate:function(e,t){t(e)},onMove:function(e,t){t(e)},onRemove:function(e,t){t(e)},onMoving:function(e,t){t(e)},onAddGroup:function(e,t){t(e)},onMoveGroup:function(e,t){t(e)},onRemoveGroup:function(e,t){t(e)},margin:{item:{horizontal:10,vertical:10},axis:20},showTooltips:!0,tooltip:{followMouse:!1,overflowMethod:"flip"},tooltipOnItemUpdateTime:!1},this.options=s.extend({},this.defaultOptions),this.options.rtl=t.rtl,this.itemOptions={type:{start:"Date",end:"Date"}},this.conversion={toScreen:e.util.toScreen,toTime:e.util.toTime},this.dom={},this.props={},this.hammer=null;var n=this;this.itemsData=null,this.groupsData=null,this.itemListeners={add:function(e,t,r){n._onAdd(t.items)},update:function(e,t,r){n._onUpdate(t.items)},remove:function(e,t,r){n._onRemove(t.items)}},this.groupListeners={add:function(e,t,r){if(n._onAddGroups(t.items),n.groupsData&&n.groupsData.length>0){var i=n.groupsData.getDataSet();i.get().forEach((function(e){if(e.nestedGroups){0!=e.showNested&&(e.showNested=!0);var t=[];e.nestedGroups.forEach((function(n){var r=i.get(n);r&&(r.nestedInGroup=e.id,0==e.showNested&&(r.visible=!1),t=t.concat(r))})),i.update(t,r)}}))}},update:function(e,t,r){n._onUpdateGroups(t.items)},remove:function(e,t,r){n._onRemoveGroups(t.items)}},this.items={},this.groups={},this.groupIds=[],this.selection=[],this.popup=null,this.touchParams={},this.groupTouchParams={},this._create(),this.setOptions(t)}A.prototype=new d,A.types={background:m,box:p,range:v,point:g},A.prototype._create=function(){var e=document.createElement("div");e.className="vis-itemset",e["timeline-itemset"]=this,this.dom.frame=e;var t=document.createElement("div");t.className="vis-background",e.appendChild(t),this.dom.background=t;var n=document.createElement("div");n.className="vis-foreground",e.appendChild(n),this.dom.foreground=n;var r=document.createElement("div");r.className="vis-axis",this.dom.axis=r;var i=document.createElement("div");i.className="vis-labelset",this.dom.labelSet=i,this._updateUngrouped();var o=new f(C,null,this);o.show(),this.groups[C]=o,this.hammer=new a(this.body.dom.centerContainer),this.hammer.on("hammer.input",function(e){e.isFirst&&this._onTouch(e)}.bind(this)),this.hammer.on("panstart",this._onDragStart.bind(this)),this.hammer.on("panmove",this._onDrag.bind(this)),this.hammer.on("panend",this._onDragEnd.bind(this)),this.hammer.get("pan").set({threshold:5,direction:a.DIRECTION_HORIZONTAL}),this.hammer.on("tap",this._onSelectItem.bind(this)),this.hammer.on("press",this._onMultiSelectItem.bind(this)),this.hammer.on("doubletap",this._onAddItem.bind(this)),this.options.rtl?this.groupHammer=new a(this.body.dom.rightContainer):this.groupHammer=new a(this.body.dom.leftContainer),this.groupHammer.on("tap",this._onGroupClick.bind(this)),this.groupHammer.on("panstart",this._onGroupDragStart.bind(this)),this.groupHammer.on("panmove",this._onGroupDrag.bind(this)),this.groupHammer.on("panend",this._onGroupDragEnd.bind(this)),this.groupHammer.get("pan").set({threshold:5,direction:a.DIRECTION_VERTICAL}),this.body.dom.centerContainer.addEventListener("mouseover",this._onMouseOver.bind(this)),this.body.dom.centerContainer.addEventListener("mouseout",this._onMouseOut.bind(this)),this.body.dom.centerContainer.addEventListener("mousemove",this._onMouseMove.bind(this)),this.body.dom.centerContainer.addEventListener("contextmenu",this._onDragEnd.bind(this)),this.body.dom.centerContainer.addEventListener("mousewheel",this._onMouseWheel.bind(this)),this.show()},A.prototype.setOptions=function(e){if(e){s.selectiveExtend(["type","rtl","align","order","stack","stackSubgroups","selectable","multiselect","multiselectPerGroup","groupOrder","dataAttributes","template","groupTemplate","visibleFrameTemplate","hide","snap","groupOrderSwap","showTooltips","tooltip","tooltipOnItemUpdateTime"],this.options,e),"itemsAlwaysDraggable"in e&&("boolean"==typeof e.itemsAlwaysDraggable?(this.options.itemsAlwaysDraggable.item=e.itemsAlwaysDraggable,this.options.itemsAlwaysDraggable.range=!1):"object"===(0,i.default)(e.itemsAlwaysDraggable)&&(s.selectiveExtend(["item","range"],this.options.itemsAlwaysDraggable,e.itemsAlwaysDraggable),this.options.itemsAlwaysDraggable.item||(this.options.itemsAlwaysDraggable.range=!1))),"orientation"in e&&("string"==typeof e.orientation?this.options.orientation.item="top"===e.orientation?"top":"bottom":"object"===(0,i.default)(e.orientation)&&"item"in e.orientation&&(this.options.orientation.item=e.orientation.item)),"margin"in e&&("number"==typeof e.margin?(this.options.margin.axis=e.margin,this.options.margin.item.horizontal=e.margin,this.options.margin.item.vertical=e.margin):"object"===(0,i.default)(e.margin)&&(s.selectiveExtend(["axis"],this.options.margin,e.margin),"item"in e.margin&&("number"==typeof e.margin.item?(this.options.margin.item.horizontal=e.margin.item,this.options.margin.item.vertical=e.margin.item):"object"===(0,i.default)(e.margin.item)&&s.selectiveExtend(["horizontal","vertical"],this.options.margin.item,e.margin.item)))),"editable"in e&&("boolean"==typeof e.editable?(this.options.editable.updateTime=e.editable,this.options.editable.updateGroup=e.editable,this.options.editable.add=e.editable,this.options.editable.remove=e.editable,this.options.editable.overrideItems=!1):"object"===(0,i.default)(e.editable)&&s.selectiveExtend(["updateTime","updateGroup","add","remove","overrideItems"],this.options.editable,e.editable)),"groupEditable"in e&&("boolean"==typeof e.groupEditable?(this.options.groupEditable.order=e.groupEditable,this.options.groupEditable.add=e.groupEditable,this.options.groupEditable.remove=e.groupEditable):"object"===(0,i.default)(e.groupEditable)&&s.selectiveExtend(["order","add","remove"],this.options.groupEditable,e.groupEditable));var t=function(t){var n=e[t];if(n){if(!(n instanceof Function))throw new Error("option "+t+" must be a function "+t+"(item, callback)");this.options[t]=n}}.bind(this);["onDropObjectOnItem","onAdd","onUpdate","onRemove","onMove","onMoving","onAddGroup","onMoveGroup","onRemoveGroup"].forEach(t),this.markDirty()}},A.prototype.markDirty=function(e){this.groupIds=[],e&&e.refreshItems&&s.forEach(this.items,(function(e){e.dirty=!0,e.displayed&&e.redraw()}))},A.prototype.destroy=function(){this.hide(),this.setItems(null),this.setGroups(null),this.hammer=null,this.body=null,this.conversion=null},A.prototype.hide=function(){this.dom.frame.parentNode&&this.dom.frame.parentNode.removeChild(this.dom.frame),this.dom.axis.parentNode&&this.dom.axis.parentNode.removeChild(this.dom.axis),this.dom.labelSet.parentNode&&this.dom.labelSet.parentNode.removeChild(this.dom.labelSet)},A.prototype.show=function(){this.dom.frame.parentNode||this.body.dom.center.appendChild(this.dom.frame),this.dom.axis.parentNode||this.body.dom.backgroundVertical.appendChild(this.dom.axis),this.dom.labelSet.parentNode||(this.options.rtl?this.body.dom.right.appendChild(this.dom.labelSet):this.body.dom.left.appendChild(this.dom.labelSet))},A.prototype.setSelection=function(e){var t,n,r,i;for(null==e&&(e=[]),Array.isArray(e)||(e=[e]),t=0,n=this.selection.length;t<n;t++)r=this.selection[t],(i=this.items[r])&&i.unselect();for(this.selection=[],t=0,n=e.length;t<n;t++)r=e[t],(i=this.items[r])&&(this.selection.push(r),i.select())},A.prototype.getSelection=function(){return this.selection.concat([])},A.prototype.getVisibleItems=function(){var e,t,n=this.body.range.getRange();this.options.rtl?(e=this.body.util.toScreen(n.start),t=this.body.util.toScreen(n.end)):(t=this.body.util.toScreen(n.start),e=this.body.util.toScreen(n.end));var r=[];for(var i in this.groups)if(this.groups.hasOwnProperty(i))for(var o=this.groups[i],a=o.isVisible?o.visibleItems:[],s=0;s<a.length;s++){var l=a[s];this.options.rtl?l.right<t&&l.right+l.width>e&&r.push(l.id):l.left<e&&l.left+l.width>t&&r.push(l.id)}return r},A.prototype._deselect=function(e){for(var t=this.selection,n=0,r=t.length;n<r;n++)if(t[n]==e){t.splice(n,1);break}},A.prototype.redraw=function(){var e=this.options.margin,t=this.body.range,n=s.option.asSize,r=this.options,i=r.orientation.item,o=!1,a=this.dom.frame;this.props.top=this.body.domProps.top.height+this.body.domProps.border.top,this.options.rtl?this.props.right=this.body.domProps.right.width+this.body.domProps.border.right:this.props.left=this.body.domProps.left.width+this.body.domProps.border.left,a.className="vis-itemset",o=this._orderGroups()||o;var l=t.end-t.start,u=l!=this.lastVisibleInterval||this.props.width!=this.props.lastWidth,c=t.start!=this.lastRangeStart,d=r.stack!=this.lastStack,h=r.stackSubgroups!=this.lastStackSubgroups,f=u||c||d||h;this.lastVisibleInterval=l,this.lastRangeStart=t.start,this.lastStack=r.stack,this.lastStackSubgroups=r.stackSubgroups,this.props.lastWidth=this.props.width;var p=this._firstGroup(),g={item:e.item,axis:e.axis},v={item:e.item,axis:e.item.vertical/2},m=0,y=e.axis+e.item.vertical;this.groups[C].redraw(t,v,f);var b={},A=0;if(s.forEach(this.groups,(function(e,n){if(n!==C){var r=e==p?g:v;b[n]=e.redraw(t,r,f,!0),A=b[n].length}})),A>0){for(var _={},I=0;I<A;I++)s.forEach(b,(function(e,t){_[t]=e[I]()}));s.forEach(this.groups,(function(e,t){if(t!==C){var n=_[t];o=n||o,m+=e.height}})),m=Math.max(m,y)}return m=Math.max(m,y),a.style.height=n(m),this.props.width=a.offsetWidth,this.props.height=m,this.dom.axis.style.top=n("top"==i?this.body.domProps.top.height+this.body.domProps.border.top:this.body.domProps.top.height+this.body.domProps.centerContainer.height),this.options.rtl?this.dom.axis.style.right="0":this.dom.axis.style.left="0",this.initialItemSetDrawn=!0,o=this._isResized()||o},A.prototype._firstGroup=function(){var e="top"==this.options.orientation.item?0:this.groupIds.length-1,t=this.groupIds[e];return this.groups[t]||this.groups[b]||null},A.prototype._updateUngrouped=function(){var e,t,n=this.groups[b];if(this.groupsData){if(n)for(t in n.hide(),delete this.groups[b],this.items)if(this.items.hasOwnProperty(t)){(e=this.items[t]).parent&&e.parent.remove(e);var r=this._getGroupId(e.data),i=this.groups[r];i&&i.add(e)||e.hide()}}else if(!n){for(t in n=new h(null,null,this),this.groups[b]=n,this.items)this.items.hasOwnProperty(t)&&(e=this.items[t],n.add(e));n.show()}},A.prototype.getLabelSet=function(){return this.dom.labelSet},A.prototype.setItems=function(e){var t,n=this,r=this.itemsData;if(e){if(!(e instanceof l||e instanceof u))throw new TypeError("Data must be an instance of DataSet or DataView");this.itemsData=e}else this.itemsData=null;if(r&&(s.forEach(this.itemListeners,(function(e,t){r.off(t,e)})),t=r.getIds(),this._onRemove(t)),this.itemsData){var i=this.id;s.forEach(this.itemListeners,(function(e,t){n.itemsData.on(t,e,i)})),t=this.itemsData.getIds(),this._onAdd(t),this._updateUngrouped()}this.body.emitter.emit("_change",{queue:!0})},A.prototype.getItems=function(){return this.itemsData},A.prototype.setGroups=function(e){var t,n=this;if(this.groupsData&&(s.forEach(this.groupListeners,(function(e,t){n.groupsData.off(t,e)})),t=this.groupsData.getIds(),this.groupsData=null,this._onRemoveGroups(t)),e){if(!(e instanceof l||e instanceof u))throw new TypeError("Data must be an instance of DataSet or DataView");this.groupsData=e}else this.groupsData=null;if(this.groupsData){var r=this.groupsData;this.groupsData instanceof u&&(r=this.groupsData.getDataSet()),r.get().forEach((function(e){e.nestedGroups&&e.nestedGroups.forEach((function(t){var n=r.get(t);n.nestedInGroup=e.id,0==e.showNested&&(n.visible=!1),r.update(n)}))}));var i=this.id;s.forEach(this.groupListeners,(function(e,t){n.groupsData.on(t,e,i)})),t=this.groupsData.getIds(),this._onAddGroups(t)}this._updateUngrouped(),this._order(),this.body.emitter.emit("_change",{queue:!0})},A.prototype.getGroups=function(){return this.groupsData},A.prototype.removeItem=function(e){var t=this.itemsData.get(e),n=this.itemsData.getDataSet();t&&this.options.onRemove(t,(function(t){t&&n.remove(e)}))},A.prototype._getType=function(e){return e.type||this.options.type||(e.end?"range":"box")},A.prototype._getGroupId=function(e){return"background"==this._getType(e)&&null==e.group?C:this.groupsData?e.group:b},A.prototype._onUpdate=function(e){var t=this;e.forEach(function(e){var n,r=t.itemsData.get(e,t.itemOptions),i=t.items[e],o=r?t._getType(r):null,a=A.types[o];if(i&&(a&&i instanceof a?t._updateItem(i,r):(n=i.selected,t._removeItem(i),i=null)),!i&&r){if(!a)throw"rangeoverflow"==o?new TypeError('Item type "rangeoverflow" is deprecated. Use css styling instead: .vis-item.vis-range .vis-item-content {overflow: visible;}'):new TypeError('Unknown item type "'+o+'"');(i=new a(r,t.conversion,t.options)).id=e,t._addItem(i),n&&(this.selection.push(e),i.select())}}.bind(this)),this._order(),this.body.emitter.emit("_change",{queue:!0})},A.prototype._onAdd=A.prototype._onUpdate,A.prototype._onRemove=function(e){var t=0,n=this;e.forEach((function(e){var r=n.items[e];r&&(t++,n._removeItem(r))})),t&&(this._order(),this.body.emitter.emit("_change",{queue:!0}))},A.prototype._order=function(){s.forEach(this.groups,(function(e){e.order()}))},A.prototype._onUpdateGroups=function(e){this._onAddGroups(e)},A.prototype._onAddGroups=function(e){var t=this;e.forEach((function(e){var n=t.groupsData.get(e),i=t.groups[e];if(i)i.setData(n);else{if(e==b||e==C)throw new Error("Illegal group id. "+e+" is a reserved id.");var o=(0,r.default)(t.options);for(var a in s.extend(o,{height:null}),i=new h(e,n,t),t.groups[e]=i,t.items)if(t.items.hasOwnProperty(a)){var l=t.items[a];l.data.group==e&&i.add(l)}i.order(),i.show()}})),this.body.emitter.emit("_change",{queue:!0})},A.prototype._onRemoveGroups=function(e){var t=this.groups;e.forEach((function(e){var n=t[e];n&&(n.hide(),delete t[e])})),this.markDirty(),this.body.emitter.emit("_change",{queue:!0})},A.prototype._orderGroups=function(){if(this.groupsData){var e=this.groupsData.getIds({order:this.options.groupOrder});e=this._orderNestedGroups(e);var t=!s.equalArray(e,this.groupIds);if(t){var n=this.groups;e.forEach((function(e){n[e].hide()})),e.forEach((function(e){n[e].show()})),this.groupIds=e}return t}return!1},A.prototype._orderNestedGroups=function(e){var t=[];return e.forEach((function(e){var n=this.groupsData.get(e);if(n.nestedInGroup||t.push(e),n.nestedGroups){var r=this.groupsData.get({filter:function(t){return t.nestedInGroup==e},order:this.options.groupOrder}).map((function(e){return e.id}));t=t.concat(r)}}),this),t},A.prototype._addItem=function(e){this.items[e.id]=e;var t=this._getGroupId(e.data),n=this.groups[t];n?n&&n.data&&n.data.showNested&&(e.groupShowing=!0):e.groupShowing=!1,n&&n.add(e)},A.prototype._updateItem=function(e,t){e.setData(t);var n=this._getGroupId(e.data),r=this.groups[n];r?r&&r.data&&r.data.showNested&&(e.groupShowing=!0):e.groupShowing=!1},A.prototype._removeItem=function(e){e.hide(),delete this.items[e.id];var t=this.selection.indexOf(e.id);-1!=t&&this.selection.splice(t,1),e.parent&&e.parent.remove(e)},A.prototype._constructByEndArray=function(e){for(var t=[],n=0;n<e.length;n++)e[n]instanceof v&&t.push(e[n]);return t},A.prototype._onTouch=function(e){this.touchParams.item=this.itemFromTarget(e),this.touchParams.dragLeftItem=e.target.dragLeftItem||!1,this.touchParams.dragRightItem=e.target.dragRightItem||!1,this.touchParams.itemProps=null},A.prototype._getGroupIndex=function(e){for(var t=0;t<this.groupIds.length;t++)if(e==this.groupIds[t])return t},A.prototype._onDragStart=function(e){if(!this.touchParams.itemIsDragging){var t,n=this.touchParams.item||null,r=this;if(n&&(n.selected||this.options.itemsAlwaysDraggable.item)){if(this.options.editable.overrideItems&&!this.options.editable.updateTime&&!this.options.editable.updateGroup)return;if(null!=n.editable&&!n.editable.updateTime&&!n.editable.updateGroup&&!this.options.editable.overrideItems)return;var i=this.touchParams.dragLeftItem,o=this.touchParams.dragRightItem;if(this.touchParams.itemIsDragging=!0,this.touchParams.selectedItem=n,i)t={item:i,initialX:e.center.x,dragLeft:!0,data:this._cloneItemData(n.data)},this.touchParams.itemProps=[t];else if(o)t={item:o,initialX:e.center.x,dragRight:!0,data:this._cloneItemData(n.data)},this.touchParams.itemProps=[t];else if(this.options.editable.add&&(e.srcEvent.ctrlKey||e.srcEvent.metaKey))this._onDragStartAddItem(e);else{this.groupIds.length<1&&this.redraw();var a=this._getGroupIndex(n.data.group),s=this.options.itemsAlwaysDraggable.item&&!n.selected?[n.id]:this.getSelection();this.touchParams.itemProps=s.map(function(t){var n=r.items[t],i=r._getGroupIndex(n.data.group);return{item:n,initialX:e.center.x,groupOffset:a-i,data:this._cloneItemData(n.data)}}.bind(this))}e.stopPropagation()}else this.options.editable.add&&(e.srcEvent.ctrlKey||e.srcEvent.metaKey)&&this._onDragStartAddItem(e)}},A.prototype._onDragStartAddItem=function(e){var t,n,r=this.options.snap||null;this.options.rtl?n=(t=s.getAbsoluteRight(this.dom.frame))-e.center.x+10:(t=s.getAbsoluteLeft(this.dom.frame),n=e.center.x-t-10);var i=this.body.util.toTime(n),o=this.body.util.getScale(),a=this.body.util.getStep(),l=r?r(i,o,a):i,u={type:"range",start:l,end:l,content:"new item"},c=s.randomUUID();u[this.itemsData._fieldId]=c;var d=this.groupFromTarget(e);d&&(u.group=d.groupId);var h=new v(u,this.conversion,this.options);h.id=c,h.data=this._cloneItemData(u),this._addItem(h),this.touchParams.selectedItem=h;var f={item:h,initialX:e.center.x,data:h.data};this.options.rtl?f.dragLeft=!0:f.dragRight=!0,this.touchParams.itemProps=[f],e.stopPropagation()},A.prototype._onDrag=function(e){if(this.touchParams.itemProps){e.stopPropagation();var t,n=this,r=this.options.snap||null;t=this.options.rtl?this.body.dom.root.offsetLeft+this.body.domProps.right.width:this.body.dom.root.offsetLeft+this.body.domProps.left.width;var i=this.body.util.getScale(),o=this.body.util.getStep(),a=this.touchParams.selectedItem,l=(this.options.editable.overrideItems||null==a.editable)&&this.options.editable.updateGroup||!this.options.editable.overrideItems&&null!=a.editable&&a.editable.updateGroup,u=null;if(l&&a&&null!=a.data.group){var c=n.groupFromTarget(e);c&&(u=this._getGroupIndex(c.groupId))}this.touchParams.itemProps.forEach(function(c){var d,h,f,p,g,v=n.body.util.toTime(e.center.x-t),m=n.body.util.toTime(c.initialX-t);d=this.options.rtl?-(v-m):v-m;var y=this._cloneItemData(c.item.data);if(null==c.item.editable||c.item.editable.updateTime||c.item.editable.updateGroup||n.options.editable.overrideItems){if((this.options.editable.overrideItems||null==a.editable)&&this.options.editable.updateTime||!this.options.editable.overrideItems&&null!=a.editable&&a.editable.updateTime)if(c.dragLeft)this.options.rtl?null!=y.end&&(f=s.convert(c.data.end,"Date"),g=new Date(f.valueOf()+d),y.end=r?r(g,i,o):g):null!=y.start&&(h=s.convert(c.data.start,"Date"),p=new Date(h.valueOf()+d),y.start=r?r(p,i,o):p);else if(c.dragRight)this.options.rtl?null!=y.start&&(h=s.convert(c.data.start,"Date"),p=new Date(h.valueOf()+d),y.start=r?r(p,i,o):p):null!=y.end&&(f=s.convert(c.data.end,"Date"),g=new Date(f.valueOf()+d),y.end=r?r(g,i,o):g);else if(null!=y.start)if(h=s.convert(c.data.start,"Date").valueOf(),p=new Date(h+d),null!=y.end){var b=(f=s.convert(c.data.end,"Date")).valueOf()-h.valueOf();y.start=r?r(p,i,o):p,y.end=new Date(y.start.valueOf()+b)}else y.start=r?r(p,i,o):p;if(l&&!c.dragLeft&&!c.dragRight&&null!=u&&null!=y.group){var C=u-c.groupOffset;C=Math.max(0,C),C=Math.min(n.groupIds.length-1,C),y.group=n.groupIds[C]}y=this._cloneItemData(y),n.options.onMoving(y,function(e){e&&c.item.setData(this._cloneItemData(e,"Date"))}.bind(this))}}.bind(this)),this.body.emitter.emit("_change")}},A.prototype._moveToGroup=function(e,t){var n=this.groups[t];if(n&&n.groupId!=e.data.group){var r=e.parent;r.remove(e),r.order(),e.data.group=n.groupId,n.add(e),n.order()}},A.prototype._onDragEnd=function(e){if(this.touchParams.itemIsDragging=!1,this.touchParams.itemProps){e.stopPropagation();var t=this,n=this.itemsData.getDataSet(),r=this.touchParams.itemProps;this.touchParams.itemProps=null,r.forEach(function(e){var r=e.item.id;if(null!=t.itemsData.get(r,t.itemOptions)){var i=this._cloneItemData(e.item.data);t.options.onMove(i,(function(i){i?(i[n._fieldId]=r,n.update(i)):(e.item.setData(e.data),t.body.emitter.emit("_change"))}))}else t.options.onAdd(e.item.data,(function(n){t._removeItem(e.item),n&&t.itemsData.getDataSet().add(n),t.body.emitter.emit("_change")}))}.bind(this))}},A.prototype._onGroupClick=function(e){var t=this.groupFromTarget(e);if(t&&t.nestedGroups){var n=this.groupsData.getDataSet(),r=n.get(t.groupId);null==r.showNested&&(r.showNested=!0),r.showNested=!r.showNested;var i=n.get(t.nestedGroups).map((function(e){return e.visible=r.showNested,e}));if(n.update(i.concat(r)),r.showNested)s.removeClassName(t.dom.label,"collapsed"),s.addClassName(t.dom.label,"expanded");else{s.removeClassName(t.dom.label,"expanded");var o=this.options.rtl?"collapsed-rtl":"collapsed";s.addClassName(t.dom.label,o)}}},A.prototype._onGroupDragStart=function(e){this.options.groupEditable.order&&(this.groupTouchParams.group=this.groupFromTarget(e),this.groupTouchParams.group&&(e.stopPropagation(),this.groupTouchParams.originalOrder=this.groupsData.getIds({order:this.options.groupOrder})))},A.prototype._onGroupDrag=function(e){if(this.options.groupEditable.order&&this.groupTouchParams.group){e.stopPropagation();var t=this.groupsData;this.groupsData instanceof u&&(t=this.groupsData.getDataSet());var n=this.groupFromTarget(e);if(n&&n.height!=this.groupTouchParams.group.height){var r=n.top<this.groupTouchParams.group.top,i=e.center?e.center.y:e.clientY,o=s.getAbsoluteTop(n.dom.foreground),a=this.groupTouchParams.group.height;if(r){if(o+a<i)return}else if(o+n.height-a>i)return}if(n&&n!=this.groupTouchParams.group){var l=t.get(n.groupId),c=t.get(this.groupTouchParams.group.groupId);c&&l&&(this.options.groupOrderSwap(c,l,t),t.update(c),t.update(l));var d=t.getIds({order:this.options.groupOrder});if(!s.equalArray(d,this.groupTouchParams.originalOrder))for(var h=this.groupTouchParams.originalOrder,f=this.groupTouchParams.group.groupId,p=Math.min(h.length,d.length),g=0,v=0,m=0;g<p;){for(;g+v<p&&g+m<p&&d[g+v]==h[g+m];)g++;if(g+v>=p)break;if(d[g+v]==f)v=1;else if(h[g+m]==f)m=1;else{var y=d.indexOf(h[g+m]),b=t.get(d[g+v]),C=t.get(h[g+m]);this.options.groupOrderSwap(b,C,t),t.update(b),t.update(C);var A=d[g+v];d[g+v]=h[g+m],d[y]=A,g++}}}}},A.prototype._onGroupDragEnd=function(e){if(this.options.groupEditable.order&&this.groupTouchParams.group){e.stopPropagation();var t=this,n=t.groupTouchParams.group.groupId,r=t.groupsData.getDataSet(),i=s.extend({},r.get(n));t.options.onMoveGroup(i,(function(e){if(e)e[r._fieldId]=n,r.update(e);else{var i=r.getIds({order:t.options.groupOrder});if(!s.equalArray(i,t.groupTouchParams.originalOrder))for(var o=t.groupTouchParams.originalOrder,a=Math.min(o.length,i.length),l=0;l<a;){for(;l<a&&i[l]==o[l];)l++;if(l>=a)break;var u=i.indexOf(o[l]),c=r.get(i[l]),d=r.get(o[l]);t.options.groupOrderSwap(c,d,r),r.update(c),r.update(d);var h=i[l];i[l]=o[l],i[u]=h,l++}}})),t.body.emitter.emit("groupDragged",{groupId:n})}},A.prototype._onSelectItem=function(e){if(this.options.selectable){var t=e.srcEvent&&(e.srcEvent.ctrlKey||e.srcEvent.metaKey),n=e.srcEvent&&e.srcEvent.shiftKey;if(t||n)this._onMultiSelectItem(e);else{var r=this.getSelection(),i=this.itemFromTarget(e),o=i?[i.id]:[];this.setSelection(o);var a=this.getSelection();(a.length>0||r.length>0)&&this.body.emitter.emit("select",{items:a,event:e})}}},A.prototype._onMouseOver=function(e){var t=this.itemFromTarget(e);if(t&&t!==this.itemFromRelatedTarget(e)){var n=t.getTitle();if(this.options.showTooltips&&n){null==this.popup&&(this.popup=new y(this.body.dom.root,this.options.tooltip.overflowMethod||"flip")),this.popup.setText(n);var r=this.body.dom.centerContainer;this.popup.setPosition(e.clientX-s.getAbsoluteLeft(r)+r.offsetLeft,e.clientY-s.getAbsoluteTop(r)+r.offsetTop),this.popup.show()}else null!=this.popup&&this.popup.hide();this.body.emitter.emit("itemover",{item:t.id,event:e})}},A.prototype._onMouseOut=function(e){var t=this.itemFromTarget(e);t&&t!==this.itemFromRelatedTarget(e)&&(null!=this.popup&&this.popup.hide(),this.body.emitter.emit("itemout",{item:t.id,event:e}))},A.prototype._onMouseMove=function(e){if(this.itemFromTarget(e)&&this.options.showTooltips&&this.options.tooltip.followMouse&&this.popup&&!this.popup.hidden){var t=this.body.dom.centerContainer;this.popup.setPosition(e.clientX-s.getAbsoluteLeft(t)+t.offsetLeft,e.clientY-s.getAbsoluteTop(t)+t.offsetTop),this.popup.show()}},A.prototype._onMouseWheel=function(e){this.touchParams.itemIsDragging&&this._onDragEnd(e)},A.prototype._onUpdateItem=function(e){if(this.options.selectable&&this.options.editable.add){var t=this;if(e){var n=t.itemsData.get(e.id);this.options.onUpdate(n,(function(e){e&&t.itemsData.getDataSet().update(e)}))}}},A.prototype._onDropObjectOnItem=function(e){var t=this.itemFromTarget(e),n=JSON.parse(e.dataTransfer.getData("text"));this.options.onDropObjectOnItem(n,t)},A.prototype._onAddItem=function(e){if(this.options.selectable&&this.options.editable.add){var t,n,r=this,i=this.options.snap||null;this.options.rtl?n=(t=s.getAbsoluteRight(this.dom.frame))-e.center.x:(t=s.getAbsoluteLeft(this.dom.frame),n=e.center.x-t);var o,a,l=this.body.util.toTime(n),u=this.body.util.getScale(),c=this.body.util.getStep();"drop"==e.type?((a=JSON.parse(e.dataTransfer.getData("text"))).content=a.content?a.content:"new item",a.start=a.start?a.start:i?i(l,u,c):l,a.type=a.type||"box",a[this.itemsData._fieldId]=a.id||s.randomUUID(),"range"!=a.type||a.end||(o=this.body.util.toTime(n+this.props.width/5),a.end=i?i(o,u,c):o)):((a={start:i?i(l,u,c):l,content:"new item"})[this.itemsData._fieldId]=s.randomUUID(),"range"===this.options.type&&(o=this.body.util.toTime(n+this.props.width/5),a.end=i?i(o,u,c):o));var d=this.groupFromTarget(e);d&&(a.group=d.groupId),a=this._cloneItemData(a),this.options.onAdd(a,(function(t){t&&(r.itemsData.getDataSet().add(t),"drop"==e.type&&r.setSelection([t.id]))}))}},A.prototype._onMultiSelectItem=function(e){if(this.options.selectable){var t=this.itemFromTarget(e);if(t){var n=this.options.multiselect?this.getSelection():[];if(e.srcEvent&&e.srcEvent.shiftKey&&this.options.multiselect){var r=this.itemsData.get(t.id).group,i=void 0;this.options.multiselectPerGroup&&n.length>0&&(i=this.itemsData.get(n[0]).group),this.options.multiselectPerGroup&&null!=i&&i!=r||n.push(t.id);var o=A._getItemRange(this.itemsData.get(n,this.itemOptions));if(!this.options.multiselectPerGroup||i==r)for(var a in n=[],this.items)if(this.items.hasOwnProperty(a)){var s=this.items[a],l=s.data.start,u=void 0!==s.data.end?s.data.end:l;!(l>=o.min&&u<=o.max)||this.options.multiselectPerGroup&&i!=this.itemsData.get(s.id).group||s instanceof m||n.push(s.id)}}else{var c=n.indexOf(t.id);-1==c?n.push(t.id):n.splice(c,1)}this.setSelection(n),this.body.emitter.emit("select",{items:this.getSelection(),event:e})}}},A._getItemRange=function(e){var t=null,n=null;return e.forEach((function(e){(null==n||e.start<n)&&(n=e.start),null!=e.end?(null==t||e.end>t)&&(t=e.end):(null==t||e.start>t)&&(t=e.start)})),{min:n,max:t}},A.prototype.itemFromElement=function(e){for(var t=e;t;){if(t.hasOwnProperty("timeline-item"))return t["timeline-item"];t=t.parentNode}return null},A.prototype.itemFromTarget=function(e){return this.itemFromElement(e.target)},A.prototype.itemFromRelatedTarget=function(e){return this.itemFromElement(e.relatedTarget)},A.prototype.groupFromTarget=function(e){var t=e.center?e.center.y:e.clientY,n=this.groupIds;n.length<=0&&this.groupsData&&(n=this.groupsData.getIds({order:this.options.groupOrder}));for(var r=0;r<n.length;r++){var i=n[r],o=this.groups[i],a=o.dom.foreground,l=s.getAbsoluteTop(a);if(t>l&&t<l+a.offsetHeight)return o;if("top"===this.options.orientation.item){if(r===this.groupIds.length-1&&t>l)return o}else if(0===r&&t<l+a.offset)return o}return null},A.itemSetFromTarget=function(e){for(var t=e.target;t;){if(t.hasOwnProperty("timeline-itemset"))return t["timeline-itemset"];t=t.parentNode}return null},A.prototype._cloneItemData=function(e,t){var n=s.extend({},e);return t||(t=this.itemsData.getDataSet()._options.type),null!=n.start&&(n.start=s.convert(n.start,t&&t.start||"Date")),null!=n.end&&(n.end=s.convert(n.end,t&&t.end||"Date")),n},e.exports=A},function(e,t,n){t.orderByStart=function(e){e.sort((function(e,t){return e.data.start-t.data.start}))},t.orderByEnd=function(e){e.sort((function(e,t){return("end"in e.data?e.data.end:e.data.start)-("end"in t.data?t.data.end:t.data.start)}))},t.stack=function(e,n,r){if(r)for(var i=0;i<e.length;i++)e[i].top=null;for(i=0;i<e.length;i++){var o=e[i];if(o.stack&&null===o.top){o.top=n.axis;do{for(var a=null,s=0,l=e.length;s<l;s++){var u=e[s];if(null!==u.top&&u!==o&&u.stack&&t.collision(o,u,n.item,u.options.rtl)){a=u;break}}null!=a&&(o.top=a.top+a.height+n.item.vertical)}while(a)}}},t.substack=function(e,n,r){for(var i=0;i<e.length;i++)e[i].top=null;var o=r.height;for(i=0;i<e.length;i++){var a=e[i];if(a.stack&&null===a.top){a.top=a.baseTop;do{for(var s=null,l=0,u=e.length;l<u;l++){var c=e[l];if(null!==c.top&&c!==a&&t.collision(a,c,n.item,c.options.rtl)){s=c;break}}null!=s&&(a.top=s.top+s.height+n.item.vertical),a.top+a.height>o&&(o=a.top+a.height)}while(s)}}r.height=o-r.top+.5*n.item.vertical},t.nostack=function(e,n,r,i){for(var o=0;o<e.length;o++)if(null==e[o].data.subgroup)e[o].top=n.item.vertical;else if(void 0!==e[o].data.subgroup&&i){var a=0;for(var s in r)r.hasOwnProperty(s)&&1==r[s].visible&&r[s].index<r[e[o].data.subgroup].index&&(a+=r[s].height,r[e[o].data.subgroup].top=a);e[o].top=a+.5*n.item.vertical}i||t.stackSubgroups(e,n,r)},t.stackSubgroups=function(e,n,r){for(var i in r)if(r.hasOwnProperty(i)){r[i].top=0;do{var o=null;for(var a in r)if(null!==r[a].top&&a!==i&&r[i].index>r[a].index&&t.collisionByTimes(r[i],r[a])){o=r[a];break}null!=o&&(r[i].top=o.top+o.height)}while(o)}for(var s=0;s<e.length;s++)void 0!==e[s].data.subgroup&&(e[s].top=r[e[s].data.subgroup].top+.5*n.item.vertical)},t.stackSubgroupsWithInnerStack=function(e,n,r){var i=!1,o=[];for(var a in r)r[a].hasOwnProperty("index")?o[r[a].index]=a:o.push(a);for(var s=0;s<o.length;s++)if(a=o[s],r.hasOwnProperty(a)){for(var l in i=i||r[a].stack,r[a].top=0,r)r[l].visible&&r[a].index>r[l].index&&(r[a].top+=r[l].height);for(var u=e[a],c=0;c<u.length;c++)void 0!==u[c].data.subgroup&&(u[c].top=r[u[c].data.subgroup].top+.5*n.item.vertical,r[a].stack&&(u[c].baseTop=u[c].top));i&&r[a].stack&&t.substack(e[a],n,r[a])}},t.collision=function(e,t,n,r){return r?e.right-n.horizontal+.001<t.right+t.width&&e.right+e.width+n.horizontal-.001>t.right&&e.top-n.vertical+.001<t.top+t.height&&e.top+e.height+n.vertical-.001>t.top:e.left-n.horizontal+.001<t.left+t.width&&e.left+e.width+n.horizontal-.001>t.left&&e.top-n.vertical+.001<t.top+t.height&&e.top+e.height+n.vertical-.001>t.top},t.collisionByTimes=function(e,t){return e.start<=t.start&&e.end>=t.start&&e.top<t.top+t.height&&e.top+e.height>t.top||t.start<=e.start&&t.end>=e.start&&t.top<e.top+e.height&&t.top+t.height>e.top}},function(e,t,n){var r=n(38);function i(e,t,n){if(this.props={dot:{width:0,height:0},line:{width:0,height:0}},this.options=n,e&&null==e.start)throw new Error('Property "start" missing in item '+e);r.call(this,e,t,n)}i.prototype=new r(null,null,null),i.prototype.isVisible=function(e){var t=this.options.align,n=this.width*e.getMillisecondsPerPixel();return"right"==t?this.data.start.getTime()>e.start&&this.data.start.getTime()-n<e.end:"left"==t?this.data.start.getTime()+n>e.start&&this.data.start.getTime()<e.end:this.data.start.getTime()+n/2>e.start&&this.data.start.getTime()-n/2<e.end},i.prototype._createDomElement=function(){this.dom||(this.dom={},this.dom.box=document.createElement("DIV"),this.dom.content=document.createElement("DIV"),this.dom.content.className="vis-item-content",this.dom.box.appendChild(this.dom.content),this.dom.line=document.createElement("DIV"),this.dom.line.className="vis-line",this.dom.dot=document.createElement("DIV"),this.dom.dot.className="vis-dot",this.dom.box["timeline-item"]=this,this.dirty=!0)},i.prototype._appendDomElement=function(){if(!this.parent)throw new Error("Cannot redraw item: no parent attached");if(!this.dom.box.parentNode){var e=this.parent.dom.foreground;if(!e)throw new Error("Cannot redraw item: parent has no foreground container element");e.appendChild(this.dom.box)}if(!this.dom.line.parentNode){var t=this.parent.dom.background;if(!t)throw new Error("Cannot redraw item: parent has no background container element");t.appendChild(this.dom.line)}if(!this.dom.dot.parentNode){var n=this.parent.dom.axis;if(!t)throw new Error("Cannot redraw item: parent has no axis container element");n.appendChild(this.dom.dot)}this.displayed=!0},i.prototype._updateDirtyDomComponents=function(){if(this.dirty){this._updateContents(this.dom.content),this._updateDataAttributes(this.dom.box),this._updateStyle(this.dom.box);var e=this.editable.updateTime||this.editable.updateGroup,t=(this.data.className?" "+this.data.className:"")+(this.selected?" vis-selected":"")+(e?" vis-editable":" vis-readonly");this.dom.box.className="vis-item vis-box"+t,this.dom.line.className="vis-item vis-line"+t,this.dom.dot.className="vis-item vis-dot"+t}},i.prototype._getDomComponentsSizes=function(){return{previous:{right:this.dom.box.style.right,left:this.dom.box.style.left},dot:{height:this.dom.dot.offsetHeight,width:this.dom.dot.offsetWidth},line:{width:this.dom.line.offsetWidth},box:{width:this.dom.box.offsetWidth,height:this.dom.box.offsetHeight}}},i.prototype._updateDomComponentsSizes=function(e){this.options.rtl?this.dom.box.style.right="0px":this.dom.box.style.left="0px",this.props.dot.height=e.dot.height,this.props.dot.width=e.dot.width,this.props.line.width=e.line.width,this.width=e.box.width,this.height=e.box.height,this.options.rtl?this.dom.box.style.right=e.previous.right:this.dom.box.style.left=e.previous.left,this.dirty=!1},i.prototype._repaintDomAdditionals=function(){this._repaintOnItemUpdateTimeTooltip(this.dom.box),this._repaintDragCenter(),this._repaintDeleteButton(this.dom.box)},i.prototype.redraw=function(e){var t,n,r=[this._createDomElement.bind(this),this._appendDomElement.bind(this),this._updateDirtyDomComponents.bind(this),function(){this.dirty&&(t=this._getDomComponentsSizes())}.bind(this),function(){this.dirty&&this._updateDomComponentsSizes.bind(this)(t)}.bind(this),this._repaintDomAdditionals.bind(this)];return e?r:(r.forEach((function(e){n=e()})),n)},i.prototype.show=function(){this.displayed||this.redraw()},i.prototype.hide=function(){if(this.displayed){var e=this.dom;e.box.parentNode&&e.box.parentNode.removeChild(e.box),e.line.parentNode&&e.line.parentNode.removeChild(e.line),e.dot.parentNode&&e.dot.parentNode.removeChild(e.dot),this.displayed=!1}},i.prototype.repositionX=function(){var e=this.conversion.toScreen(this.data.start),t=this.options.align;"right"==t?this.options.rtl?(this.right=e-this.width,this.dom.box.style.right=this.right+"px",this.dom.line.style.right=e-this.props.line.width+"px",this.dom.dot.style.right=e-this.props.line.width/2-this.props.dot.width/2+"px"):(this.left=e-this.width,this.dom.box.style.left=this.left+"px",this.dom.line.style.left=e-this.props.line.width+"px",this.dom.dot.style.left=e-this.props.line.width/2-this.props.dot.width/2+"px"):"left"==t?this.options.rtl?(this.right=e,this.dom.box.style.right=this.right+"px",this.dom.line.style.right=e+"px",this.dom.dot.style.right=e+this.props.line.width/2-this.props.dot.width/2+"px"):(this.left=e,this.dom.box.style.left=this.left+"px",this.dom.line.style.left=e+"px",this.dom.dot.style.left=e+this.props.line.width/2-this.props.dot.width/2+"px"):this.options.rtl?(this.right=e-this.width/2,this.dom.box.style.right=this.right+"px",this.dom.line.style.right=e-this.props.line.width+"px",this.dom.dot.style.right=e-this.props.dot.width/2+"px"):(this.left=e-this.width/2,this.dom.box.style.left=this.left+"px",this.dom.line.style.left=e-this.props.line.width/2+"px",this.dom.dot.style.left=e-this.props.dot.width/2+"px")},i.prototype.repositionY=function(){var e=this.options.orientation.item,t=this.dom.box,n=this.dom.line,r=this.dom.dot;if("top"==e)t.style.top=(this.top||0)+"px",n.style.top="0",n.style.height=this.parent.top+this.top+1+"px",n.style.bottom="";else{var i=this.parent.itemSet.props.height,o=i-this.parent.top-this.parent.height+this.top;t.style.top=(this.parent.height-this.top-this.height||0)+"px",n.style.top=i-o+"px",n.style.bottom="0"}r.style.top=-this.props.dot.height/2+"px"},i.prototype.getWidthLeft=function(){return this.width/2},i.prototype.getWidthRight=function(){return this.width/2},e.exports=i},function(e,t,n){var r=n(38);function i(e,t,n){if(this.props={dot:{top:0,width:0,height:0},content:{height:0,marginLeft:0,marginRight:0}},this.options=n,e&&null==e.start)throw new Error('Property "start" missing in item '+e);r.call(this,e,t,n)}i.prototype=new r(null,null,null),i.prototype.isVisible=function(e){var t=this.width*e.getMillisecondsPerPixel();return this.data.start.getTime()+t>e.start&&this.data.start<e.end},i.prototype._createDomElement=function(){this.dom||(this.dom={},this.dom.point=document.createElement("div"),this.dom.content=document.createElement("div"),this.dom.content.className="vis-item-content",this.dom.point.appendChild(this.dom.content),this.dom.dot=document.createElement("div"),this.dom.point.appendChild(this.dom.dot),this.dom.point["timeline-item"]=this,this.dirty=!0)},i.prototype._appendDomElement=function(){if(!this.parent)throw new Error("Cannot redraw item: no parent attached");if(!this.dom.point.parentNode){var e=this.parent.dom.foreground;if(!e)throw new Error("Cannot redraw item: parent has no foreground container element");e.appendChild(this.dom.point)}this.displayed=!0},i.prototype._updateDirtyDomComponents=function(){if(this.dirty){this._updateContents(this.dom.content),this._updateDataAttributes(this.dom.point),this._updateStyle(this.dom.point);var e=this.editable.updateTime||this.editable.updateGroup,t=(this.data.className?" "+this.data.className:"")+(this.selected?" vis-selected":"")+(e?" vis-editable":" vis-readonly");this.dom.point.className="vis-item vis-point"+t,this.dom.dot.className="vis-item vis-dot"+t}},i.prototype._getDomComponentsSizes=function(){return{dot:{width:this.dom.dot.offsetWidth,height:this.dom.dot.offsetHeight},content:{width:this.dom.content.offsetWidth,height:this.dom.content.offsetHeight},point:{width:this.dom.point.offsetWidth,height:this.dom.point.offsetHeight}}},i.prototype._updateDomComponentsSizes=function(e){this.props.dot.width=e.dot.width,this.props.dot.height=e.dot.height,this.props.content.height=e.content.height,this.options.rtl?this.dom.content.style.marginRight=2*this.props.dot.width+"px":this.dom.content.style.marginLeft=2*this.props.dot.width+"px",this.width=e.point.width,this.height=e.point.height,this.dom.dot.style.top=(this.height-this.props.dot.height)/2+"px",this.options.rtl?this.dom.dot.style.right=this.props.dot.width/2+"px":this.dom.dot.style.left=this.props.dot.width/2+"px",this.dirty=!1},i.prototype._repaintDomAdditionals=function(){this._repaintOnItemUpdateTimeTooltip(this.dom.point),this._repaintDragCenter(),this._repaintDeleteButton(this.dom.point)},i.prototype.redraw=function(e){var t,n,r=[this._createDomElement.bind(this),this._appendDomElement.bind(this),this._updateDirtyDomComponents.bind(this),function(){this.dirty&&(t=this._getDomComponentsSizes())}.bind(this),function(){this.dirty&&this._updateDomComponentsSizes.bind(this)(t)}.bind(this),this._repaintDomAdditionals.bind(this)];return e?r:(r.forEach((function(e){n=e()})),n)},i.prototype.show=function(){this.displayed||this.redraw()},i.prototype.hide=function(){this.displayed&&(this.dom.point.parentNode&&this.dom.point.parentNode.removeChild(this.dom.point),this.displayed=!1)},i.prototype.repositionX=function(){var e=this.conversion.toScreen(this.data.start);this.options.rtl?(this.right=e-this.props.dot.width,this.dom.point.style.right=this.right+"px"):(this.left=e-this.props.dot.width,this.dom.point.style.left=this.left+"px")},i.prototype.repositionY=function(){var e=this.options.orientation.item,t=this.dom.point;t.style.top="top"==e?this.top+"px":this.parent.height-this.top-this.height+"px"},i.prototype.getWidthLeft=function(){return this.props.dot.width},i.prototype.getWidthRight=function(){return this.props.dot.width},e.exports=i},function(e,t,n){var r=n(38),i=n(69),o=n(70);function a(e,t,n){if(this.props={content:{width:0}},this.overflow=!1,e){if(null==e.start)throw new Error('Property "start" missing in item '+e.id);if(null==e.end)throw new Error('Property "end" missing in item '+e.id)}r.call(this,e,t,n)}a.prototype=new r(null,null,null),a.prototype.baseClassName="vis-item vis-background",a.prototype.stack=!1,a.prototype.isVisible=function(e){return this.data.start<e.end&&this.data.end>e.start},a.prototype._createDomElement=function(){this.dom||(this.dom={},this.dom.box=document.createElement("div"),this.dom.frame=document.createElement("div"),this.dom.frame.className="vis-item-overflow",this.dom.box.appendChild(this.dom.frame),this.dom.content=document.createElement("div"),this.dom.content.className="vis-item-content",this.dom.frame.appendChild(this.dom.content),this.dirty=!0)},a.prototype._appendDomElement=function(){if(!this.parent)throw new Error("Cannot redraw item: no parent attached");if(!this.dom.box.parentNode){var e=this.parent.dom.background;if(!e)throw new Error("Cannot redraw item: parent has no background container element");e.appendChild(this.dom.box)}this.displayed=!0},a.prototype._updateDirtyDomComponents=function(){if(this.dirty){this._updateContents(this.dom.content),this._updateDataAttributes(this.dom.content),this._updateStyle(this.dom.box);var e=(this.data.className?" "+this.data.className:"")+(this.selected?" vis-selected":"");this.dom.box.className=this.baseClassName+e}},a.prototype._getDomComponentsSizes=function(){return this.overflow="hidden"!==window.getComputedStyle(this.dom.content).overflow,{content:{width:this.dom.content.offsetWidth}}},a.prototype._updateDomComponentsSizes=function(e){this.props.content.width=e.content.width,this.height=0,this.dirty=!1},a.prototype._repaintDomAdditionals=function(){},a.prototype.redraw=function(e){var t,n,r=[this._createDomElement.bind(this),this._appendDomElement.bind(this),this._updateDirtyDomComponents.bind(this),function(){this.dirty&&(t=this._getDomComponentsSizes.bind(this)())}.bind(this),function(){this.dirty&&this._updateDomComponentsSizes.bind(this)(t)}.bind(this),this._repaintDomAdditionals.bind(this)];return e?r:(r.forEach((function(e){n=e()})),n)},a.prototype.show=o.prototype.show,a.prototype.hide=o.prototype.hide,a.prototype.repositionX=o.prototype.repositionX,a.prototype.repositionY=function(e){var t,n=this.options.orientation.item;if(void 0!==this.data.subgroup){var r=this.data.subgroup;this.dom.box.style.height=this.parent.subgroups[r].height+"px",this.dom.box.style.top="top"==n?this.parent.top+this.parent.subgroups[r].top+"px":this.parent.top+this.parent.height-this.parent.subgroups[r].top-this.parent.subgroups[r].height+"px",this.dom.box.style.bottom=""}else this.parent instanceof i?(t=Math.max(this.parent.height,this.parent.itemSet.body.domProps.center.height,this.parent.itemSet.body.domProps.centerContainer.height),this.dom.box.style.bottom="bottom"==n?"0":"",this.dom.box.style.top="top"==n?"0":""):(t=this.parent.height,this.dom.box.style.top=this.parent.top+"px",this.dom.box.style.bottom="");this.dom.box.style.height=t+"px"},e.exports=a},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=o(n(0)),i=o(n(1));function o(e){return e&&e.__esModule?e:{default:e}}var a=function(){function e(t,n){(0,r.default)(this,e),this.container=t,this.overflowMethod=n||"cap",this.x=0,this.y=0,this.padding=5,this.hidden=!1,this.frame=document.createElement("div"),this.frame.className="vis-tooltip",this.container.appendChild(this.frame)}return(0,i.default)(e,[{key:"setPosition",value:function(e,t){this.x=parseInt(e),this.y=parseInt(t)}},{key:"setText",value:function(e){e instanceof Element?(this.frame.innerHTML="",this.frame.appendChild(e)):this.frame.innerHTML=e}},{key:"show",value:function(e){if(void 0===e&&(e=!0),!0===e){var t=this.frame.clientHeight,n=this.frame.clientWidth,r=this.frame.parentNode.clientHeight,i=this.frame.parentNode.clientWidth,o=0,a=0;if("flip"==this.overflowMethod){var s=!1,l=!0;this.y-t<this.padding&&(l=!1),this.x+n>i-this.padding&&(s=!0),o=s?this.x-n:this.x,a=l?this.y-t:this.y}else(a=this.y-t)+t+this.padding>r&&(a=r-t-this.padding),a<this.padding&&(a=this.padding),(o=this.x)+n+this.padding>i&&(o=i-n-this.padding),o<this.padding&&(o=this.padding);this.frame.style.left=o+"px",this.frame.style.top=a+"px",this.frame.style.visibility="visible",this.hidden=!1}else this.hide()}},{key:"hide",value:function(){this.hidden=!0,this.frame.style.left="0",this.frame.style.top="0",this.frame.style.visibility="hidden"}},{key:"destroy",value:function(){this.frame.parentNode.removeChild(this.frame)}}]),e}();t.default=a},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r="string",i="boolean",o="number",a="object",s={configure:{enabled:{boolean:i},filter:{boolean:i,function:"function"},container:{dom:"dom"},__type__:{object:a,boolean:i,function:"function"}},align:{string:r},rtl:{boolean:i,undefined:"undefined"},rollingMode:{follow:{boolean:i},offset:{number:o,undefined:"undefined"},__type__:{object:a}},verticalScroll:{boolean:i,undefined:"undefined"},horizontalScroll:{boolean:i,undefined:"undefined"},autoResize:{boolean:i},throttleRedraw:{number:o},clickToUse:{boolean:i},dataAttributes:{string:r,array:"array"},editable:{add:{boolean:i,undefined:"undefined"},remove:{boolean:i,undefined:"undefined"},updateGroup:{boolean:i,undefined:"undefined"},updateTime:{boolean:i,undefined:"undefined"},overrideItems:{boolean:i,undefined:"undefined"},__type__:{boolean:i,object:a}},end:{number:o,date:"date",string:r,moment:"moment"},format:{minorLabels:{millisecond:{string:r,undefined:"undefined"},second:{string:r,undefined:"undefined"},minute:{string:r,undefined:"undefined"},hour:{string:r,undefined:"undefined"},weekday:{string:r,undefined:"undefined"},day:{string:r,undefined:"undefined"},week:{string:r,undefined:"undefined"},month:{string:r,undefined:"undefined"},year:{string:r,undefined:"undefined"},__type__:{object:a,function:"function"}},majorLabels:{millisecond:{string:r,undefined:"undefined"},second:{string:r,undefined:"undefined"},minute:{string:r,undefined:"undefined"},hour:{string:r,undefined:"undefined"},weekday:{string:r,undefined:"undefined"},day:{string:r,undefined:"undefined"},week:{string:r,undefined:"undefined"},month:{string:r,undefined:"undefined"},year:{string:r,undefined:"undefined"},__type__:{object:a,function:"function"}},__type__:{object:a}},moment:{function:"function"},groupOrder:{string:r,function:"function"},groupEditable:{add:{boolean:i,undefined:"undefined"},remove:{boolean:i,undefined:"undefined"},order:{boolean:i,undefined:"undefined"},__type__:{boolean:i,object:a}},groupOrderSwap:{function:"function"},height:{string:r,number:o},hiddenDates:{start:{date:"date",number:o,string:r,moment:"moment"},end:{date:"date",number:o,string:r,moment:"moment"},repeat:{string:r},__type__:{object:a,array:"array"}},itemsAlwaysDraggable:{item:{boolean:i,undefined:"undefined"},range:{boolean:i,undefined:"undefined"},__type__:{boolean:i,object:a}},limitSize:{boolean:i},locale:{string:r},locales:{__any__:{any:"any"},__type__:{object:a}},margin:{axis:{number:o},item:{horizontal:{number:o,undefined:"undefined"},vertical:{number:o,undefined:"undefined"},__type__:{object:a,number:o}},__type__:{object:a,number:o}},max:{date:"date",number:o,string:r,moment:"moment"},maxHeight:{number:o,string:r},maxMinorChars:{number:o},min:{date:"date",number:o,string:r,moment:"moment"},minHeight:{number:o,string:r},moveable:{boolean:i},multiselect:{boolean:i},multiselectPerGroup:{boolean:i},onAdd:{function:"function"},onDropObjectOnItem:{function:"function"},onUpdate:{function:"function"},onMove:{function:"function"},onMoving:{function:"function"},onRemove:{function:"function"},onAddGroup:{function:"function"},onMoveGroup:{function:"function"},onRemoveGroup:{function:"function"},onInitialDrawComplete:{function:"function"},order:{function:"function"},orientation:{axis:{string:r,undefined:"undefined"},item:{string:r,undefined:"undefined"},__type__:{string:r,object:a}},selectable:{boolean:i},showCurrentTime:{boolean:i},showMajorLabels:{boolean:i},showMinorLabels:{boolean:i},stack:{boolean:i},stackSubgroups:{boolean:i},snap:{function:"function",null:"null"},start:{date:"date",number:o,string:r,moment:"moment"},template:{function:"function"},groupTemplate:{function:"function"},visibleFrameTemplate:{string:r,function:"function"},showTooltips:{boolean:i},tooltip:{followMouse:{boolean:i},overflowMethod:{string:["cap","flip"]},__type__:{object:a}},tooltipOnItemUpdateTime:{template:{function:"function"},__type__:{boolean:i,object:a}},timeAxis:{scale:{string:r,undefined:"undefined"},step:{number:o,undefined:"undefined"},__type__:{object:a}},type:{string:r},width:{string:r,number:o},zoomable:{boolean:i},zoomKey:{string:["ctrlKey","altKey","metaKey",""]},zoomMax:{number:o},zoomMin:{number:o},__type__:{object:a}};t.allOptions=s,t.configureOptions={global:{align:["center","left","right"],direction:!1,autoResize:!0,clickToUse:!1,editable:{add:!1,remove:!1,updateGroup:!1,updateTime:!1},end:"",format:{minorLabels:{millisecond:"SSS",second:"s",minute:"HH:mm",hour:"HH:mm",weekday:"ddd D",day:"D",week:"w",month:"MMM",year:"YYYY"},majorLabels:{millisecond:"HH:mm:ss",second:"D MMMM HH:mm",minute:"ddd D MMMM",hour:"ddd D MMMM",weekday:"MMMM YYYY",day:"MMMM YYYY",week:"MMMM YYYY",month:"YYYY",year:""}},groupsDraggable:!1,height:"",locale:"",margin:{axis:[20,0,100,1],item:{horizontal:[10,0,100,1],vertical:[10,0,100,1]}},max:"",maxHeight:"",maxMinorChars:[7,0,20,1],min:"",minHeight:"",moveable:!1,multiselect:!1,multiselectPerGroup:!1,orientation:{axis:["both","bottom","top"],item:["bottom","top"]},selectable:!0,showCurrentTime:!1,showMajorLabels:!0,showMinorLabels:!0,stack:!0,stackSubgroups:!0,start:"",showTooltips:!0,tooltip:{followMouse:!1,overflowMethod:"flip"},tooltipOnItemUpdateTime:!1,type:["box","point","range","background"],width:"100%",zoomable:!0,zoomKey:["ctrlKey","altKey","metaKey",""],zoomMax:[31536e10,10,31536e10,1],zoomMin:[10,10,31536e10,1]}}},function(e,t,n){var r,i=n(6),o=(r=i)&&r.__esModule?r:{default:r},a=n(2),s=n(14),l=n(11),u=n(12),c=n(16),d=n(107),h=n(109),f=n(112),p=n(110),g=n(111),v=n(72);function m(e,t){this.id=a.randomUUID(),this.body=e,this.defaultOptions={yAxisOrientation:"left",defaultGroup:"default",sort:!0,sampling:!0,stack:!1,graphHeight:"400px",shaded:{enabled:!1,orientation:"bottom"},style:"line",barChart:{width:50,sideBySide:!1,align:"center"},interpolation:{enabled:!0,parametrization:"centripetal",alpha:.5},drawPoints:{enabled:!0,size:6,style:"square"},dataAxis:{},legend:{},groups:{visibility:{}}},this.options=a.extend({},this.defaultOptions),this.dom={},this.props={},this.hammer=null,this.groups={},this.abortedGraphUpdate=!1,this.updateSVGheight=!1,this.updateSVGheightOnResize=!1,this.forceGraphUpdate=!0;var n=this;this.itemsData=null,this.groupsData=null,this.itemListeners={add:function(e,t,r){n._onAdd(t.items)},update:function(e,t,r){n._onUpdate(t.items)},remove:function(e,t,r){n._onRemove(t.items)}},this.groupListeners={add:function(e,t,r){n._onAddGroups(t.items)},update:function(e,t,r){n._onUpdateGroups(t.items)},remove:function(e,t,r){n._onRemoveGroups(t.items)}},this.items={},this.selection=[],this.lastStart=this.body.range.start,this.touchParams={},this.svgElements={},this.setOptions(t),this.groupsUsingDefaultStyles=[0],this.body.emitter.on("rangechanged",(function(){n.lastStart=n.body.range.start,n.svg.style.left=a.option.asSize(-n.props.width),n.forceGraphUpdate=!0,n.redraw.call(n)})),this._create(),this.framework={svg:this.svg,svgElements:this.svgElements,options:this.options,groups:this.groups}}m.prototype=new c,m.prototype._create=function(){var e=document.createElement("div");e.className="vis-line-graph",this.dom.frame=e,this.svg=document.createElementNS("http://www.w3.org/2000/svg","svg"),this.svg.style.position="relative",this.svg.style.height=(""+this.options.graphHeight).replace("px","")+"px",this.svg.style.display="block",e.appendChild(this.svg),this.options.dataAxis.orientation="left",this.yAxisLeft=new d(this.body,this.options.dataAxis,this.svg,this.options.groups),this.options.dataAxis.orientation="right",this.yAxisRight=new d(this.body,this.options.dataAxis,this.svg,this.options.groups),delete this.options.dataAxis.orientation,this.legendLeft=new f(this.body,this.options.legend,"left",this.options.groups),this.legendRight=new f(this.body,this.options.legend,"right",this.options.groups),this.show()},m.prototype.setOptions=function(e){e&&(void 0===e.graphHeight&&void 0!==e.height?(this.updateSVGheight=!0,this.updateSVGheightOnResize=!0):void 0!==this.body.domProps.centerContainer.height&&void 0!==e.graphHeight&&parseInt((e.graphHeight+"").replace("px",""))<this.body.domProps.centerContainer.height&&(this.updateSVGheight=!0),a.selectiveDeepExtend(["sampling","defaultGroup","stack","height","graphHeight","yAxisOrientation","style","barChart","dataAxis","sort","groups"],this.options,e),a.mergeOptions(this.options,e,"interpolation"),a.mergeOptions(this.options,e,"drawPoints"),a.mergeOptions(this.options,e,"shaded"),a.mergeOptions(this.options,e,"legend"),e.interpolation&&"object"==(0,o.default)(e.interpolation)&&e.interpolation.parametrization&&("uniform"==e.interpolation.parametrization?this.options.interpolation.alpha=0:"chordal"==e.interpolation.parametrization?this.options.interpolation.alpha=1:(this.options.interpolation.parametrization="centripetal",this.options.interpolation.alpha=.5)),this.yAxisLeft&&void 0!==e.dataAxis&&(this.yAxisLeft.setOptions(this.options.dataAxis),this.yAxisRight.setOptions(this.options.dataAxis)),this.legendLeft&&void 0!==e.legend&&(this.legendLeft.setOptions(this.options.legend),this.legendRight.setOptions(this.options.legend)),this.groups.hasOwnProperty("__ungrouped__")&&this.groups.__ungrouped__.setOptions(e)),this.dom.frame&&(this.forceGraphUpdate=!0,this.body.emitter.emit("_change",{queue:!0}))},m.prototype.hide=function(){this.dom.frame.parentNode&&this.dom.frame.parentNode.removeChild(this.dom.frame)},m.prototype.show=function(){this.dom.frame.parentNode||this.body.dom.center.appendChild(this.dom.frame)},m.prototype.setItems=function(e){var t,n=this,r=this.itemsData;if(e){if(!(e instanceof l||e instanceof u))throw new TypeError("Data must be an instance of DataSet or DataView");this.itemsData=e}else this.itemsData=null;if(r&&(a.forEach(this.itemListeners,(function(e,t){r.off(t,e)})),t=r.getIds(),this._onRemove(t)),this.itemsData){var i=this.id;a.forEach(this.itemListeners,(function(e,t){n.itemsData.on(t,e,i)})),t=this.itemsData.getIds(),this._onAdd(t)}},m.prototype.setGroups=function(e){var t,n=this;if(this.groupsData){a.forEach(this.groupListeners,(function(e,t){n.groupsData.off(t,e)})),t=this.groupsData.getIds(),this.groupsData=null;for(var r=0;r<t.length;r++)this._removeGroup(t[r])}if(e){if(!(e instanceof l||e instanceof u))throw new TypeError("Data must be an instance of DataSet or DataView");this.groupsData=e}else this.groupsData=null;if(this.groupsData){var i=this.id;a.forEach(this.groupListeners,(function(e,t){n.groupsData.on(t,e,i)})),t=this.groupsData.getIds(),this._onAddGroups(t)}},m.prototype._onUpdate=function(e){this._updateAllGroupData(e)},m.prototype._onAdd=function(e){this._onUpdate(e)},m.prototype._onRemove=function(e){this._onUpdate(e)},m.prototype._onUpdateGroups=function(e){this._updateAllGroupData(null,e)},m.prototype._onAddGroups=function(e){this._onUpdateGroups(e)},m.prototype._onRemoveGroups=function(e){for(var t=0;t<e.length;t++)this._removeGroup(e[t]);this.forceGraphUpdate=!0,this.body.emitter.emit("_change",{queue:!0})},m.prototype._removeGroup=function(e){this.groups.hasOwnProperty(e)&&("right"==this.groups[e].options.yAxisOrientation?(this.yAxisRight.removeGroup(e),this.legendRight.removeGroup(e),this.legendRight.redraw()):(this.yAxisLeft.removeGroup(e),this.legendLeft.removeGroup(e),this.legendLeft.redraw()),delete this.groups[e])},m.prototype._updateGroup=function(e,t){this.groups.hasOwnProperty(t)?(this.groups[t].update(e),"right"==this.groups[t].options.yAxisOrientation?(this.yAxisRight.updateGroup(t,this.groups[t]),this.legendRight.updateGroup(t,this.groups[t]),this.yAxisLeft.removeGroup(t),this.legendLeft.removeGroup(t)):(this.yAxisLeft.updateGroup(t,this.groups[t]),this.legendLeft.updateGroup(t,this.groups[t]),this.yAxisRight.removeGroup(t),this.legendRight.removeGroup(t))):(this.groups[t]=new h(e,t,this.options,this.groupsUsingDefaultStyles),"right"==this.groups[t].options.yAxisOrientation?(this.yAxisRight.addGroup(t,this.groups[t]),this.legendRight.addGroup(t,this.groups[t])):(this.yAxisLeft.addGroup(t,this.groups[t]),this.legendLeft.addGroup(t,this.groups[t]))),this.legendLeft.redraw(),this.legendRight.redraw()},m.prototype._updateAllGroupData=function(e,t){if(null!=this.itemsData){var n={},r=this.itemsData.get(),i=this.itemsData._fieldId,o={};e&&e.map((function(e){o[e]=e}));for(var s={},l=0;l<r.length;l++){var u=r[l],c=u.group;null==c&&(c="__ungrouped__"),s.hasOwnProperty(c)?s[c]++:s[c]=1}var d={};if(!t&&e)for(c in this.groups)if(this.groups.hasOwnProperty(c)){var h=(v=this.groups[c]).getItems();n[c]=h.filter((function(e){return d[e[i]]=e[i],e[i]!==o[e[i]]}));var f=s[c];s[c]-=n[c].length,n[c].length<f&&(n[c][f-1]={})}for(l=0;l<r.length;l++)if(null==(c=(u=r[l]).group)&&(c="__ungrouped__"),t||!e||u[i]===o[u[i]]||!d.hasOwnProperty(u[i])){n.hasOwnProperty(c)||(n[c]=new Array(s[c]));var p=a.bridgeObject(u);p.x=a.convert(u.x,"Date"),p.end=a.convert(u.end,"Date"),p.orginalY=u.y,p.y=Number(u.y),p[i]=u[i];var g=n[c].length-s[c]--;n[c][g]=p}for(c in this.groups)this.groups.hasOwnProperty(c)&&(n.hasOwnProperty(c)||(n[c]=new Array(0)));for(c in n)if(n.hasOwnProperty(c))if(0==n[c].length)this.groups.hasOwnProperty(c)&&this._removeGroup(c);else{var v=void 0;null!=this.groupsData&&(v=this.groupsData.get(c)),null==v&&(v={id:c,content:this.options.defaultGroup+c}),this._updateGroup(v,c),this.groups[c].setItems(n[c])}this.forceGraphUpdate=!0,this.body.emitter.emit("_change",{queue:!0})}},m.prototype.redraw=function(){var e=!1;this.props.width=this.dom.frame.offsetWidth,this.props.height=this.body.domProps.centerContainer.height-this.body.domProps.border.top-this.body.domProps.border.bottom,e=this._isResized()||e;var t=this.body.range.end-this.body.range.start,n=t!=this.lastVisibleInterval;if(this.lastVisibleInterval=t,1==e&&(this.svg.style.width=a.option.asSize(3*this.props.width),this.svg.style.left=a.option.asSize(-this.props.width),-1==(this.options.height+"").indexOf("%")&&1!=this.updateSVGheightOnResize||(this.updateSVGheight=!0)),1==this.updateSVGheight?(this.options.graphHeight!=this.props.height+"px"&&(this.options.graphHeight=this.props.height+"px",this.svg.style.height=this.props.height+"px"),this.updateSVGheight=!1):this.svg.style.height=(""+this.options.graphHeight).replace("px","")+"px",1==e||1==n||1==this.abortedGraphUpdate||1==this.forceGraphUpdate)e=this._updateGraph()||e,this.forceGraphUpdate=!1;else if(0!=this.lastStart){var r=this.body.range.start-this.lastStart,i=this.body.range.end-this.body.range.start;if(0!=this.props.width){var o=r*(this.props.width/i);this.svg.style.left=-this.props.width-o+"px"}}return this.legendLeft.redraw(),this.legendRight.redraw(),e},m.prototype._getSortedGroupIds=function(){var e=[];for(var t in this.groups)if(this.groups.hasOwnProperty(t)){var n=this.groups[t];1!=n.visible||void 0!==this.options.groups.visibility[t]&&1!=this.options.groups.visibility[t]||e.push({id:t,zIndex:n.options.zIndex})}a.insertSort(e,(function(e,t){var n=e.zIndex,r=t.zIndex;return void 0===n&&(n=0),void 0===r&&(r=0),n==r?0:n<r?-1:1}));for(var r=new Array(e.length),i=0;i<e.length;i++)r[i]=e[i].id;return r},m.prototype._updateGraph=function(){if(s.prepareElements(this.svgElements),0!=this.props.width&&null!=this.itemsData){var e,t,n={},r=this.body.util.toGlobalTime(-this.body.domProps.root.width),i=this.body.util.toGlobalTime(2*this.body.domProps.root.width),o=this._getSortedGroupIds();if(o.length>0){var a={};for(this._getRelevantData(o,a,r,i),this._applySampling(o,a),t=0;t<o.length;t++)this._convertXcoordinates(a[o[t]]);if(this._getYRanges(o,a,n),1==this._updateYAxis(o,n))return s.cleanupElements(this.svgElements),this.abortedGraphUpdate=!0,!0;this.abortedGraphUpdate=!1;var l=void 0;for(t=0;t<o.length;t++)e=this.groups[o[t]],!0===this.options.stack&&"line"===this.options.style&&(null!=e.options.excludeFromStacking&&e.options.excludeFromStacking||(null!=l&&(this._stack(a[e.id],a[l.id]),1==e.options.shaded.enabled&&"group"!==e.options.shaded.orientation&&("top"==e.options.shaded.orientation&&"group"!==l.options.shaded.orientation?(l.options.shaded.orientation="group",l.options.shaded.groupId=e.id):(e.options.shaded.orientation="group",e.options.shaded.groupId=l.id))),l=e)),this._convertYcoordinates(a[o[t]],e);var u={};for(t=0;t<o.length;t++)if("line"===(e=this.groups[o[t]]).options.style&&1==e.options.shaded.enabled){var c=a[o[t]];if(null==c||0==c.length)continue;if(u.hasOwnProperty(o[t])||(u[o[t]]=g.calcPath(c,e)),"group"===e.options.shaded.orientation){var d=e.options.shaded.groupId;if(-1===o.indexOf(d)){console.log(e.id+": Unknown shading group target given:"+d);continue}u.hasOwnProperty(d)||(u[d]=g.calcPath(a[d],this.groups[d])),g.drawShading(u[o[t]],e,u[d],this.framework)}else g.drawShading(u[o[t]],e,void 0,this.framework)}for(p.draw(o,a,this.framework),t=0;t<o.length;t++)if(e=this.groups[o[t]],a[o[t]].length>0)switch(e.options.style){case"line":u.hasOwnProperty(o[t])||(u[o[t]]=g.calcPath(a[o[t]],e)),g.draw(u[o[t]],e,this.framework);case"point":case"points":"point"!=e.options.style&&"points"!=e.options.style&&1!=e.options.drawPoints.enabled||v.draw(a[o[t]],e,this.framework)}}}return s.cleanupElements(this.svgElements),!1},m.prototype._stack=function(e,t){var n,r,i,o,a;n=0;for(var s=0;s<e.length;s++){o=void 0,a=void 0;for(var l=n;l<t.length;l++){if(t[l].x===e[s].x){o=t[l],a=t[l],n=l;break}if(t[l].x>e[s].x){a=t[l],o=0==l?a:t[l-1],n=l;break}}void 0===a&&(o=t[t.length-1],a=t[t.length-1]),r=a.x-o.x,i=a.y-o.y,e[s].y=0==r?e[s].orginalY+a.y:e[s].orginalY+i/r*(e[s].x-o.x)+o.y}},m.prototype._getRelevantData=function(e,t,n,r){var i,o,s,l;if(e.length>0)for(o=0;o<e.length;o++){var u=(i=this.groups[e[o]]).getItems();if(1==i.options.sort){var c=function(e,t){return e.getTime()==t.getTime()?0:e<t?-1:1},d=Math.max(0,a.binarySearchValue(u,n,"x","before",c)),h=Math.min(u.length,a.binarySearchValue(u,r,"x","after",c)+1);h<=0&&(h=u.length);var f=new Array(h-d);for(s=d;s<h;s++)l=i.itemsData[s],f[s-d]=l;t[e[o]]=f}else t[e[o]]=i.itemsData}},m.prototype._applySampling=function(e,t){if(e.length>0)for(var n=0;n<e.length;n++)if(1==this.groups[e[n]].options.sampling){var r=t[e[n]];if(r.length>0){var i,o=r.length,a=o/(this.body.util.toGlobalScreen(r[r.length-1].x)-this.body.util.toGlobalScreen(r[0].x));i=Math.min(Math.ceil(.2*o),Math.max(1,Math.round(a)));for(var s=new Array(o),l=0;l<o;l+=i)s[Math.round(l/i)]=r[l];t[e[n]]=s.splice(0,Math.round(o/i))}}},m.prototype._getYRanges=function(e,t,n){var r,i,o,a,s=[],l=[];if(e.length>0){for(o=0;o<e.length;o++)r=t[e[o]],a=this.groups[e[o]].options,r.length>0&&(i=this.groups[e[o]],!0===a.stack&&"bar"===a.style?"left"===a.yAxisOrientation?s=s.concat(r):l=l.concat(r):n[e[o]]=i.getYRange(r,e[o]));p.getStackedYRange(s,n,e,"__barStackLeft","left"),p.getStackedYRange(l,n,e,"__barStackRight","right")}},m.prototype._updateYAxis=function(e,t){var n,r,i=!1,o=!1,a=!1,s=1e9,l=1e9,u=-1e9,c=-1e9;if(e.length>0){for(var d=0;d<e.length;d++){var h=this.groups[e[d]];h&&"right"!=h.options.yAxisOrientation?(o=!0,s=1e9,u=-1e9):h&&h.options.yAxisOrientation&&(a=!0,l=1e9,c=-1e9)}for(d=0;d<e.length;d++)t.hasOwnProperty(e[d])&&!0!==t[e[d]].ignore&&(n=t[e[d]].min,r=t[e[d]].max,"right"!=t[e[d]].yAxisOrientation?(o=!0,s=s>n?n:s,u=u<r?r:u):(a=!0,l=l>n?n:l,c=c<r?r:c));1==o&&this.yAxisLeft.setRange(s,u),1==a&&this.yAxisRight.setRange(l,c)}i=this._toggleAxisVisiblity(o,this.yAxisLeft)||i,i=this._toggleAxisVisiblity(a,this.yAxisRight)||i,1==a&&1==o?(this.yAxisLeft.drawIcons=!0,this.yAxisRight.drawIcons=!0):(this.yAxisLeft.drawIcons=!1,this.yAxisRight.drawIcons=!1),this.yAxisRight.master=!o,this.yAxisRight.masterAxis=this.yAxisLeft,0==this.yAxisRight.master?(this.yAxisLeft.lineOffset=1==a?this.yAxisRight.width:0,i=this.yAxisLeft.redraw()||i,i=this.yAxisRight.redraw()||i):i=this.yAxisRight.redraw()||i;var f=["__barStackLeft","__barStackRight","__lineStackLeft","__lineStackRight"];for(d=0;d<f.length;d++)-1!=e.indexOf(f[d])&&e.splice(e.indexOf(f[d]),1);return i},m.prototype._toggleAxisVisiblity=function(e,t){var n=!1;return 0==e?t.dom.frame.parentNode&&0==t.hidden&&(t.hide(),n=!0):t.dom.frame.parentNode||1!=t.hidden||(t.show(),n=!0),n},m.prototype._convertXcoordinates=function(e){for(var t=this.body.util.toScreen,n=0;n<e.length;n++)e[n].screen_x=t(e[n].x)+this.props.width,e[n].screen_y=e[n].y,null!=e[n].end?e[n].screen_end=t(e[n].end)+this.props.width:e[n].screen_end=void 0},m.prototype._convertYcoordinates=function(e,t){var n=this.yAxisLeft,r=Number(this.svg.style.height.replace("px",""));"right"==t.options.yAxisOrientation&&(n=this.yAxisRight);for(var i=0;i<e.length;i++)e[i].screen_y=Math.round(n.convertValue(e[i].y));t.setZeroPosition(Math.min(r,n.convertValue(0)))},e.exports=m},function(e,t,n){var r,i=n(8),o=(r=i)&&r.__esModule?r:{default:r},a=n(2),s=n(14),l=n(16),u=n(108);function c(e,t,n,r){this.id=a.randomUUID(),this.body=e,this.defaultOptions={orientation:"left",showMinorLabels:!0,showMajorLabels:!0,icons:!1,majorLinesOffset:7,minorLinesOffset:4,labelOffsetX:10,labelOffsetY:2,iconWidth:20,width:"40px",visible:!0,alignZeros:!0,left:{range:{min:void 0,max:void 0},format:function(e){return""+parseFloat(e.toPrecision(3))},title:{text:void 0,style:void 0}},right:{range:{min:void 0,max:void 0},format:function(e){return""+parseFloat(e.toPrecision(3))},title:{text:void 0,style:void 0}}},this.linegraphOptions=r,this.linegraphSVG=n,this.props={},this.DOMelements={lines:{},labels:{},title:{}},this.dom={},this.scale=void 0,this.range={start:0,end:0},this.options=a.extend({},this.defaultOptions),this.conversionFactor=1,this.setOptions(t),this.width=Number((""+this.options.width).replace("px","")),this.minWidth=this.width,this.height=this.linegraphSVG.getBoundingClientRect().height,this.hidden=!1,this.stepPixels=25,this.zeroCrossing=-1,this.amountOfSteps=-1,this.lineOffset=0,this.master=!0,this.masterAxis=null,this.svgElements={},this.iconsRemoved=!1,this.groups={},this.amountOfGroups=0,this._create(),this.framework={svg:this.svg,svgElements:this.svgElements,options:this.options,groups:this.groups};var i=this;this.body.emitter.on("verticalDrag",(function(){i.dom.lineContainer.style.top=i.body.domProps.scrollTop+"px"}))}c.prototype=new l,c.prototype.addGroup=function(e,t){this.groups.hasOwnProperty(e)||(this.groups[e]=t),this.amountOfGroups+=1},c.prototype.updateGroup=function(e,t){this.groups.hasOwnProperty(e)||(this.amountOfGroups+=1),this.groups[e]=t},c.prototype.removeGroup=function(e){this.groups.hasOwnProperty(e)&&(delete this.groups[e],this.amountOfGroups-=1)},c.prototype.setOptions=function(e){if(e){var t=!1;this.options.orientation!=e.orientation&&void 0!==e.orientation&&(t=!0),a.selectiveDeepExtend(["orientation","showMinorLabels","showMajorLabels","icons","majorLinesOffset","minorLinesOffset","labelOffsetX","labelOffsetY","iconWidth","width","visible","left","right","alignZeros"],this.options,e),this.minWidth=Number((""+this.options.width).replace("px","")),!0===t&&this.dom.frame&&(this.hide(),this.show())}},c.prototype._create=function(){this.dom.frame=document.createElement("div"),this.dom.frame.style.width=this.options.width,this.dom.frame.style.height=this.height,this.dom.lineContainer=document.createElement("div"),this.dom.lineContainer.style.width="100%",this.dom.lineContainer.style.height=this.height,this.dom.lineContainer.style.position="relative",this.svg=document.createElementNS("http://www.w3.org/2000/svg","svg"),this.svg.style.position="absolute",this.svg.style.top="0px",this.svg.style.height="100%",this.svg.style.width="100%",this.svg.style.display="block",this.dom.frame.appendChild(this.svg)},c.prototype._redrawGroupIcons=function(){var e;s.prepareElements(this.svgElements);var t=this.options.iconWidth,n=11.5;e="left"===this.options.orientation?4:this.width-t-4;var r=(0,o.default)(this.groups);r.sort((function(e,t){return e<t?-1:1}));for(var i=0;i<r.length;i++){var a=r[i];!0!==this.groups[a].visible||void 0!==this.linegraphOptions.visibility[a]&&!0!==this.linegraphOptions.visibility[a]||(this.groups[a].getLegend(t,15,this.framework,e,n),n+=19)}s.cleanupElements(this.svgElements),this.iconsRemoved=!1},c.prototype._cleanupIcons=function(){!1===this.iconsRemoved&&(s.prepareElements(this.svgElements),s.cleanupElements(this.svgElements),this.iconsRemoved=!0)},c.prototype.show=function(){this.hidden=!1,this.dom.frame.parentNode||("left"===this.options.orientation?this.body.dom.left.appendChild(this.dom.frame):this.body.dom.right.appendChild(this.dom.frame)),this.dom.lineContainer.parentNode||this.body.dom.backgroundHorizontal.appendChild(this.dom.lineContainer)},c.prototype.hide=function(){this.hidden=!0,this.dom.frame.parentNode&&this.dom.frame.parentNode.removeChild(this.dom.frame),this.dom.lineContainer.parentNode&&this.dom.lineContainer.parentNode.removeChild(this.dom.lineContainer)},c.prototype.setRange=function(e,t){this.range.start=e,this.range.end=t},c.prototype.redraw=function(){var e=!1,t=0;for(var n in this.dom.lineContainer.style.top=this.body.domProps.scrollTop+"px",this.groups)this.groups.hasOwnProperty(n)&&(!0!==this.groups[n].visible||void 0!==this.linegraphOptions.visibility[n]&&!0!==this.linegraphOptions.visibility[n]||t++);if(0===this.amountOfGroups||0===t)this.hide();else{this.show(),this.height=Number(this.linegraphSVG.style.height.replace("px","")),this.dom.lineContainer.style.height=this.height+"px",this.width=!0===this.options.visible?Number((""+this.options.width).replace("px","")):0;var r=this.props,i=this.dom.frame;i.className="vis-data-axis",this._calculateCharSize();var o=this.options.orientation,a=this.options.showMinorLabels,s=this.options.showMajorLabels;r.minorLabelHeight=a?r.minorCharHeight:0,r.majorLabelHeight=s?r.majorCharHeight:0,r.minorLineWidth=this.body.dom.backgroundHorizontal.offsetWidth-this.lineOffset-this.width+2*this.options.minorLinesOffset,r.minorLineHeight=1,r.majorLineWidth=this.body.dom.backgroundHorizontal.offsetWidth-this.lineOffset-this.width+2*this.options.majorLinesOffset,r.majorLineHeight=1,"left"===o?(i.style.top="0",i.style.left="0",i.style.bottom="",i.style.width=this.width+"px",i.style.height=this.height+"px",this.props.width=this.body.domProps.left.width,this.props.height=this.body.domProps.left.height):(i.style.top="",i.style.bottom="0",i.style.left="0",i.style.width=this.width+"px",i.style.height=this.height+"px",this.props.width=this.body.domProps.right.width,this.props.height=this.body.domProps.right.height),e=this._redrawLabels(),e=this._isResized()||e,!0===this.options.icons?this._redrawGroupIcons():this._cleanupIcons(),this._redrawTitle(o)}return e},c.prototype._redrawLabels=function(){var e=this,t=!1;s.prepareElements(this.DOMelements.lines),s.prepareElements(this.DOMelements.labels);var n=this.options.orientation,r=null!=this.options[n].range?this.options[n].range:{},i=!0;null!=r.max&&(this.range.end=r.max,i=!1);var o=!0;null!=r.min&&(this.range.start=r.min,o=!1),this.scale=new u(this.range.start,this.range.end,o,i,this.dom.frame.offsetHeight,this.props.majorCharHeight,this.options.alignZeros,this.options[n].format),!1===this.master&&null!=this.masterAxis&&this.scale.followScale(this.masterAxis.scale),this.maxLabelSize=0,this.scale.getLines().forEach((function(t){var r=t.y,i=t.major;e.options.showMinorLabels&&!1===i&&e._redrawLabel(r-2,t.val,n,"vis-y-axis vis-minor",e.props.minorCharHeight),i&&r>=0&&e._redrawLabel(r-2,t.val,n,"vis-y-axis vis-major",e.props.majorCharHeight),!0===e.master&&(i?e._redrawLine(r,n,"vis-grid vis-horizontal vis-major",e.options.majorLinesOffset,e.props.majorLineWidth):e._redrawLine(r,n,"vis-grid vis-horizontal vis-minor",e.options.minorLinesOffset,e.props.minorLineWidth))}));var a=0;void 0!==this.options[n].title&&void 0!==this.options[n].title.text&&(a=this.props.titleCharHeight);var l=!0===this.options.icons?Math.max(this.options.iconWidth,a)+this.options.labelOffsetX+15:a+this.options.labelOffsetX+15;return this.maxLabelSize>this.width-l&&!0===this.options.visible?(this.width=this.maxLabelSize+l,this.options.width=this.width+"px",s.cleanupElements(this.DOMelements.lines),s.cleanupElements(this.DOMelements.labels),this.redraw(),t=!0):this.maxLabelSize<this.width-l&&!0===this.options.visible&&this.width>this.minWidth?(this.width=Math.max(this.minWidth,this.maxLabelSize+l),this.options.width=this.width+"px",s.cleanupElements(this.DOMelements.lines),s.cleanupElements(this.DOMelements.labels),this.redraw(),t=!0):(s.cleanupElements(this.DOMelements.lines),s.cleanupElements(this.DOMelements.labels),t=!1),t},c.prototype.convertValue=function(e){return this.scale.convertValue(e)},c.prototype.screenToValue=function(e){return this.scale.screenToValue(e)},c.prototype._redrawLabel=function(e,t,n,r,i){var o=s.getDOMElement("div",this.DOMelements.labels,this.dom.frame);o.className=r,o.innerHTML=t,"left"===n?(o.style.left="-"+this.options.labelOffsetX+"px",o.style.textAlign="right"):(o.style.right="-"+this.options.labelOffsetX+"px",o.style.textAlign="left"),o.style.top=e-.5*i+this.options.labelOffsetY+"px",t+="";var a=Math.max(this.props.majorCharWidth,this.props.minorCharWidth);this.maxLabelSize<t.length*a&&(this.maxLabelSize=t.length*a)},c.prototype._redrawLine=function(e,t,n,r,i){if(!0===this.master){var o=s.getDOMElement("div",this.DOMelements.lines,this.dom.lineContainer);o.className=n,o.innerHTML="","left"===t?o.style.left=this.width-r+"px":o.style.right=this.width-r+"px",o.style.width=i+"px",o.style.top=e+"px"}},c.prototype._redrawTitle=function(e){if(s.prepareElements(this.DOMelements.title),void 0!==this.options[e].title&&void 0!==this.options[e].title.text){var t=s.getDOMElement("div",this.DOMelements.title,this.dom.frame);t.className="vis-y-axis vis-title vis-"+e,t.innerHTML=this.options[e].title.text,void 0!==this.options[e].title.style&&a.addCssText(t,this.options[e].title.style),"left"===e?t.style.left=this.props.titleCharHeight+"px":t.style.right=this.props.titleCharHeight+"px",t.style.width=this.height+"px"}s.cleanupElements(this.DOMelements.title)},c.prototype._calculateCharSize=function(){if(!("minorCharHeight"in this.props)){var e=document.createTextNode("0"),t=document.createElement("div");t.className="vis-y-axis vis-minor vis-measure",t.appendChild(e),this.dom.frame.appendChild(t),this.props.minorCharHeight=t.clientHeight,this.props.minorCharWidth=t.clientWidth,this.dom.frame.removeChild(t)}if(!("majorCharHeight"in this.props)){var n=document.createTextNode("0"),r=document.createElement("div");r.className="vis-y-axis vis-major vis-measure",r.appendChild(n),this.dom.frame.appendChild(r),this.props.majorCharHeight=r.clientHeight,this.props.majorCharWidth=r.clientWidth,this.dom.frame.removeChild(r)}if(!("titleCharHeight"in this.props)){var i=document.createTextNode("0"),o=document.createElement("div");o.className="vis-y-axis vis-title vis-measure",o.appendChild(i),this.dom.frame.appendChild(o),this.props.titleCharHeight=o.clientHeight,this.props.titleCharWidth=o.clientWidth,this.dom.frame.removeChild(o)}},e.exports=c},function(e,t,n){function r(e,t,n,r,i,o){var a=arguments.length>6&&void 0!==arguments[6]&&arguments[6],s=arguments.length>7&&void 0!==arguments[7]&&arguments[7];if(this.majorSteps=[1,2,5,10],this.minorSteps=[.25,.5,1,2],this.customLines=null,this.containerHeight=i,this.majorCharHeight=o,this._start=e,this._end=t,this.scale=1,this.minorStepIdx=-1,this.magnitudefactor=1,this.determineScale(),this.zeroAlign=a,this.autoScaleStart=n,this.autoScaleEnd=r,this.formattingFunction=s,n||r){var l=this,u=function(e){var t=e-e%(l.magnitudefactor*l.minorSteps[l.minorStepIdx]);return e%(l.magnitudefactor*l.minorSteps[l.minorStepIdx])>l.magnitudefactor*l.minorSteps[l.minorStepIdx]*.5?t+l.magnitudefactor*l.minorSteps[l.minorStepIdx]:t};n&&(this._start-=2*this.magnitudefactor*this.minorSteps[this.minorStepIdx],this._start=u(this._start)),r&&(this._end+=this.magnitudefactor*this.minorSteps[this.minorStepIdx],this._end=u(this._end)),this.determineScale()}}r.prototype.setCharHeight=function(e){this.majorCharHeight=e},r.prototype.setHeight=function(e){this.containerHeight=e},r.prototype.determineScale=function(){var e=this._end-this._start;this.scale=this.containerHeight/e;var t=this.majorCharHeight/this.scale,n=e>0?Math.round(Math.log(e)/Math.LN10):0;this.minorStepIdx=-1,this.magnitudefactor=Math.pow(10,n);var r=0;n<0&&(r=n);for(var i=!1,o=r;Math.abs(o)<=Math.abs(n);o++){this.magnitudefactor=Math.pow(10,o);for(var a=0;a<this.minorSteps.length;a++)if(this.magnitudefactor*this.minorSteps[a]>=t){i=!0,this.minorStepIdx=a;break}if(!0===i)break}},r.prototype.is_major=function(e){return e%(this.magnitudefactor*this.majorSteps[this.minorStepIdx])==0},r.prototype.getStep=function(){return this.magnitudefactor*this.minorSteps[this.minorStepIdx]},r.prototype.getFirstMajor=function(){var e=this.magnitudefactor*this.majorSteps[this.minorStepIdx];return this.convertValue(this._start+(e-this._start%e)%e)},r.prototype.formatValue=function(e){var t=e.toPrecision(5);return"function"==typeof this.formattingFunction&&(t=this.formattingFunction(e)),"number"==typeof t?""+t:"string"==typeof t?t:e.toPrecision(5)},r.prototype.getLines=function(){for(var e=[],t=this.getStep(),n=(t-this._start%t)%t,r=this._start+n;this._end-r>1e-5;r+=t)r!=this._start&&e.push({major:this.is_major(r),y:this.convertValue(r),val:this.formatValue(r)});return e},r.prototype.followScale=function(e){var t=this.minorStepIdx,n=this._start,r=this._end,i=this,o=function(){i.magnitudefactor*=2},a=function(){i.magnitudefactor/=2};e.minorStepIdx<=1&&this.minorStepIdx<=1||e.minorStepIdx>1&&this.minorStepIdx>1||(e.minorStepIdx<this.minorStepIdx?(this.minorStepIdx=1,2==t||o(),o()):(this.minorStepIdx=2,1==t||a(),a()));for(var s=e.convertValue(0),l=e.getStep()*e.scale,u=!1,c=0;!u&&c++<5;){this.scale=l/(this.minorSteps[this.minorStepIdx]*this.magnitudefactor);var d=this.containerHeight/this.scale;this._start=n,this._end=this._start+d;var h=this._end*this.scale,f=this.magnitudefactor*this.majorSteps[this.minorStepIdx],p=this.getFirstMajor()-e.getFirstMajor();if(this.zeroAlign){var g=s-h;this._end+=g/this.scale,this._start=this._end-d}else this.autoScaleStart?(this._start-=p/this.scale,this._end=this._start+d):(this._start+=f-p/this.scale,this._end=this._start+d);if(!this.autoScaleEnd&&this._end>r+1e-5)a(),u=!1;else{if(!this.autoScaleStart&&this._start<n-1e-5){if(!(this.zeroAlign&&n>=0)){a(),u=!1;continue}console.warn("Can't adhere to given 'min' range, due to zeroalign")}this.autoScaleStart&&this.autoScaleEnd&&d<r-n?(o(),u=!1):u=!0}}},r.prototype.convertValue=function(e){return this.containerHeight-(e-this._start)*this.scale},r.prototype.screenToValue=function(e){return(this.containerHeight-e)/this.scale+this._start},e.exports=r},function(e,t,n){var r,i=n(6),o=(r=i)&&r.__esModule?r:{default:r},a=n(2),s=n(110),l=n(111),u=n(72);function c(e,t,n,r){this.id=t,this.options=a.selectiveBridgeObject(["sampling","style","sort","yAxisOrientation","barChart","drawPoints","shaded","interpolation","zIndex","excludeFromStacking","excludeFromLegend"],n),this.usingDefaultStyle=void 0===e.className,this.groupsUsingDefaultStyles=r,this.zeroPosition=0,this.update(e),1==this.usingDefaultStyle&&(this.groupsUsingDefaultStyles[0]+=1),this.itemsData=[],this.visible=void 0===e.visible||e.visible}c.prototype.setItems=function(e){null!=e?(this.itemsData=e,1==this.options.sort&&a.insertSort(this.itemsData,(function(e,t){return e.x>t.x?1:-1}))):this.itemsData=[]},c.prototype.getItems=function(){return this.itemsData},c.prototype.setZeroPosition=function(e){this.zeroPosition=e},c.prototype.setOptions=function(e){void 0!==e&&(a.selectiveDeepExtend(["sampling","style","sort","yAxisOrientation","barChart","zIndex","excludeFromStacking","excludeFromLegend"],this.options,e),"function"==typeof e.drawPoints&&(e.drawPoints={onRender:e.drawPoints}),a.mergeOptions(this.options,e,"interpolation"),a.mergeOptions(this.options,e,"drawPoints"),a.mergeOptions(this.options,e,"shaded"),e.interpolation&&"object"==(0,o.default)(e.interpolation)&&e.interpolation.parametrization&&("uniform"==e.interpolation.parametrization?this.options.interpolation.alpha=0:"chordal"==e.interpolation.parametrization?this.options.interpolation.alpha=1:(this.options.interpolation.parametrization="centripetal",this.options.interpolation.alpha=.5)))},c.prototype.update=function(e){this.group=e,this.content=e.content||"graph",this.className=e.className||this.className||"vis-graph-group"+this.groupsUsingDefaultStyles[0]%10,this.visible=void 0===e.visible||e.visible,this.style=e.style,this.setOptions(e.options)},c.prototype.getLegend=function(e,t,n,r,i){switch(null!=n&&null!=n||(n={svg:document.createElementNS("http://www.w3.org/2000/svg","svg"),svgElements:{},options:this.options,groups:[this]}),null!=r&&null!=r||(r=0),null!=i&&null!=i||(i=.5*t),this.options.style){case"line":l.drawIcon(this,r,i,e,t,n);break;case"points":case"point":u.drawIcon(this,r,i,e,t,n);break;case"bar":s.drawIcon(this,r,i,e,t,n)}return{icon:n.svg,label:this.content,orientation:this.options.yAxisOrientation}},c.prototype.getYRange=function(e){for(var t=e[0].y,n=e[0].y,r=0;r<e.length;r++)t=t>e[r].y?e[r].y:t,n=n<e[r].y?e[r].y:n;return{min:t,max:n,yAxisOrientation:this.options.yAxisOrientation}},e.exports=c},function(e,t,n){var r=n(14),i=n(72);function o(e,t){}o.drawIcon=function(e,t,n,i,o,a){var s=.5*o,l=r.getSVGElement("rect",a.svgElements,a.svg);l.setAttributeNS(null,"x",t),l.setAttributeNS(null,"y",n-s),l.setAttributeNS(null,"width",i),l.setAttributeNS(null,"height",2*s),l.setAttributeNS(null,"class","vis-outline");var u=Math.round(.3*i),c=e.options.barChart.width/u,d=Math.round(.4*o),h=Math.round(.75*o),f=Math.round((i-2*u)/3);if(r.drawBar(t+.5*u+f,n+s-d-1,u,d,e.className+" vis-bar",a.svgElements,a.svg,e.style),r.drawBar(t+1.5*u+f+2,n+s-h-1,u,h,e.className+" vis-bar",a.svgElements,a.svg,e.style),1==e.options.drawPoints.enabled){var p={style:e.options.drawPoints.style,styles:e.options.drawPoints.styles,size:e.options.drawPoints.size/c,className:e.className};r.drawPoint(t+.5*u+f,n+s-d-1,p,a.svgElements,a.svg),r.drawPoint(t+1.5*u+f+2,n+s-h-1,p,a.svgElements,a.svg)}},o.draw=function(e,t,n){var a,s,l,u,c,d,h=[],f={},p=0;for(c=0;c<e.length;c++)if("bar"===(u=n.groups[e[c]]).options.style&&!0===u.visible&&(void 0===n.options.groups.visibility[e[c]]||!0===n.options.groups.visibility[e[c]]))for(d=0;d<t[e[c]].length;d++)h.push({screen_x:t[e[c]][d].screen_x,screen_end:t[e[c]][d].screen_end,screen_y:t[e[c]][d].screen_y,x:t[e[c]][d].x,end:t[e[c]][d].end,y:t[e[c]][d].y,groupId:e[c],label:t[e[c]][d].label}),p+=1;if(0!==p)for(h.sort((function(e,t){return e.screen_x===t.screen_x?e.groupId<t.groupId?-1:1:e.screen_x-t.screen_x})),o._getDataIntersections(f,h),c=0;c<h.length;c++){var g=null!=(u=n.groups[h[c].groupId]).options.barChart.minWidth?u.options.barChart.minWidth:.1*u.options.barChart.width,v=0;if(void 0===f[s=h[c].screen_x])c+1<h.length&&(a=Math.abs(h[c+1].screen_x-s)),l=o._getSafeDrawData(a,u,g);else{var m=c+(f[s].amount-f[s].resolved);m<h.length&&(a=Math.abs(h[m].screen_x-s)),l=o._getSafeDrawData(a,u,g),f[s].resolved+=1,!0===u.options.stack&&!0!==u.options.excludeFromStacking?h[c].screen_y<u.zeroPosition?(v=f[s].accumulatedNegative,f[s].accumulatedNegative+=u.zeroPosition-h[c].screen_y):(v=f[s].accumulatedPositive,f[s].accumulatedPositive+=u.zeroPosition-h[c].screen_y):!0===u.options.barChart.sideBySide&&(l.width=l.width/f[s].amount,l.offset+=f[s].resolved*l.width-.5*l.width*(f[s].amount+1))}var y=l.width,b=h[c].screen_x;if(null!=h[c].screen_end?b+=.5*(y=h[c].screen_end-h[c].screen_x):b+=l.offset,r.drawBar(b,h[c].screen_y-v,y,u.zeroPosition-h[c].screen_y,u.className+" vis-bar",n.svgElements,n.svg,u.style),!0===u.options.drawPoints.enabled){var C={screen_x:h[c].screen_x,screen_y:h[c].screen_y-v,x:h[c].x,y:h[c].y,groupId:h[c].groupId,label:h[c].label};i.draw([C],u,n,l.offset)}}},o._getDataIntersections=function(e,t){for(var n,r=0;r<t.length;r++)r+1<t.length&&(n=Math.abs(t[r+1].screen_x-t[r].screen_x)),r>0&&(n=Math.min(n,Math.abs(t[r-1].screen_x-t[r].screen_x))),0===n&&(void 0===e[t[r].screen_x]&&(e[t[r].screen_x]={amount:0,resolved:0,accumulatedPositive:0,accumulatedNegative:0}),e[t[r].screen_x].amount+=1)},o._getSafeDrawData=function(e,t,n){var r,i;return e<t.options.barChart.width&&e>0?(r=e<n?n:e,i=0,"left"===t.options.barChart.align?i-=.5*e:"right"===t.options.barChart.align&&(i+=.5*e)):(r=t.options.barChart.width,i=0,"left"===t.options.barChart.align?i-=.5*t.options.barChart.width:"right"===t.options.barChart.align&&(i+=.5*t.options.barChart.width)),{width:r,offset:i}},o.getStackedYRange=function(e,t,n,r,i){if(e.length>0){e.sort((function(e,t){return e.screen_x===t.screen_x?e.groupId<t.groupId?-1:1:e.screen_x-t.screen_x}));var a={};o._getDataIntersections(a,e),t[r]=o._getStackedYRange(a,e),t[r].yAxisOrientation=i,n.push(r)}},o._getStackedYRange=function(e,t){for(var n,r=t[0].screen_y,i=t[0].screen_y,o=0;o<t.length;o++)void 0===e[n=t[o].screen_x]?(r=r>t[o].screen_y?t[o].screen_y:r,i=i<t[o].screen_y?t[o].screen_y:i):t[o].screen_y<0?e[n].accumulatedNegative+=t[o].screen_y:e[n].accumulatedPositive+=t[o].screen_y;for(var a in e)e.hasOwnProperty(a)&&(r=(r=r>e[a].accumulatedNegative?e[a].accumulatedNegative:r)>e[a].accumulatedPositive?e[a].accumulatedPositive:r,i=(i=i<e[a].accumulatedNegative?e[a].accumulatedNegative:i)<e[a].accumulatedPositive?e[a].accumulatedPositive:i);return{min:r,max:i}},e.exports=o},function(e,t,n){var r=n(14);function i(e,t){}i.calcPath=function(e,t){if(null!=e&&e.length>0)return 1==t.options.interpolation.enabled?i._catmullRom(e,t):i._linear(e)},i.drawIcon=function(e,t,n,i,o,a){var s,l,u=.5*o,c=r.getSVGElement("rect",a.svgElements,a.svg);if(c.setAttributeNS(null,"x",t),c.setAttributeNS(null,"y",n-u),c.setAttributeNS(null,"width",i),c.setAttributeNS(null,"height",2*u),c.setAttributeNS(null,"class","vis-outline"),(s=r.getSVGElement("path",a.svgElements,a.svg)).setAttributeNS(null,"class",e.className),void 0!==e.style&&s.setAttributeNS(null,"style",e.style),s.setAttributeNS(null,"d","M"+t+","+n+" L"+(t+i)+","+n),1==e.options.shaded.enabled&&(l=r.getSVGElement("path",a.svgElements,a.svg),"top"==e.options.shaded.orientation?l.setAttributeNS(null,"d","M"+t+", "+(n-u)+"L"+t+","+n+" L"+(t+i)+","+n+" L"+(t+i)+","+(n-u)):l.setAttributeNS(null,"d","M"+t+","+n+" L"+t+","+(n+u)+" L"+(t+i)+","+(n+u)+"L"+(t+i)+","+n),l.setAttributeNS(null,"class",e.className+" vis-icon-fill"),void 0!==e.options.shaded.style&&""!==e.options.shaded.style&&l.setAttributeNS(null,"style",e.options.shaded.style)),1==e.options.drawPoints.enabled){var d={style:e.options.drawPoints.style,styles:e.options.drawPoints.styles,size:e.options.drawPoints.size,className:e.className};r.drawPoint(t+.5*i,n,d,a.svgElements,a.svg)}},i.drawShading=function(e,t,n,i){if(1==t.options.shaded.enabled){var o,a=Number(i.svg.style.height.replace("px","")),s=r.getSVGElement("path",i.svgElements,i.svg),l="L";1==t.options.interpolation.enabled&&(l="C");var u=0;u="top"==t.options.shaded.orientation?0:"bottom"==t.options.shaded.orientation?a:Math.min(Math.max(0,t.zeroPosition),a),o="group"==t.options.shaded.orientation&&null!=n&&null!=n?"M"+e[0][0]+","+e[0][1]+" "+this.serializePath(e,l,!1)+" L"+n[n.length-1][0]+","+n[n.length-1][1]+" "+this.serializePath(n,l,!0)+n[0][0]+","+n[0][1]+" Z":"M"+e[0][0]+","+e[0][1]+" "+this.serializePath(e,l,!1)+" V"+u+" H"+e[0][0]+" Z",s.setAttributeNS(null,"class",t.className+" vis-fill"),void 0!==t.options.shaded.style&&s.setAttributeNS(null,"style",t.options.shaded.style),s.setAttributeNS(null,"d",o)}},i.draw=function(e,t,n){if(null!=e&&null!=e){var i=r.getSVGElement("path",n.svgElements,n.svg);i.setAttributeNS(null,"class",t.className),void 0!==t.style&&i.setAttributeNS(null,"style",t.style);var o="L";1==t.options.interpolation.enabled&&(o="C"),i.setAttributeNS(null,"d","M"+e[0][0]+","+e[0][1]+" "+this.serializePath(e,o,!1))}},i.serializePath=function(e,t,n){if(e.length<2)return"";var r,i=t;if(n)for(r=e.length-2;r>0;r--)i+=e[r][0]+","+e[r][1]+" ";else for(r=1;r<e.length;r++)i+=e[r][0]+","+e[r][1]+" ";return i},i._catmullRomUniform=function(e){var t,n,r,i,o,a,s=[];s.push([Math.round(e[0].screen_x),Math.round(e[0].screen_y)]);for(var l=e.length,u=0;u<l-1;u++)t=0==u?e[0]:e[u-1],n=e[u],r=e[u+1],i=u+2<l?e[u+2]:r,o={screen_x:(-t.screen_x+6*n.screen_x+r.screen_x)*(1/6),screen_y:(-t.screen_y+6*n.screen_y+r.screen_y)*(1/6)},a={screen_x:(n.screen_x+6*r.screen_x-i.screen_x)*(1/6),screen_y:(n.screen_y+6*r.screen_y-i.screen_y)*(1/6)},s.push([o.screen_x,o.screen_y]),s.push([a.screen_x,a.screen_y]),s.push([r.screen_x,r.screen_y]);return s},i._catmullRom=function(e,t){var n=t.options.interpolation.alpha;if(0==n||void 0===n)return this._catmullRomUniform(e);var r,i,o,a,s,l,u,c,d,h,f,p,g,v,m,y,b,C,A,_=[];_.push([Math.round(e[0].screen_x),Math.round(e[0].screen_y)]);for(var I=e.length,x=0;x<I-1;x++)r=0==x?e[0]:e[x-1],i=e[x],o=e[x+1],a=x+2<I?e[x+2]:o,u=Math.sqrt(Math.pow(r.screen_x-i.screen_x,2)+Math.pow(r.screen_y-i.screen_y,2)),c=Math.sqrt(Math.pow(i.screen_x-o.screen_x,2)+Math.pow(i.screen_y-o.screen_y,2)),d=Math.sqrt(Math.pow(o.screen_x-a.screen_x,2)+Math.pow(o.screen_y-a.screen_y,2)),v=Math.pow(d,n),y=Math.pow(d,2*n),m=Math.pow(c,n),b=Math.pow(c,2*n),A=Math.pow(u,n),h=2*(C=Math.pow(u,2*n))+3*A*m+b,f=2*y+3*v*m+b,(p=3*A*(A+m))>0&&(p=1/p),(g=3*v*(v+m))>0&&(g=1/g),s={screen_x:(-b*r.screen_x+h*i.screen_x+C*o.screen_x)*p,screen_y:(-b*r.screen_y+h*i.screen_y+C*o.screen_y)*p},l={screen_x:(y*i.screen_x+f*o.screen_x-b*a.screen_x)*g,screen_y:(y*i.screen_y+f*o.screen_y-b*a.screen_y)*g},0==s.screen_x&&0==s.screen_y&&(s=i),0==l.screen_x&&0==l.screen_y&&(l=o),_.push([s.screen_x,s.screen_y]),_.push([l.screen_x,l.screen_y]),_.push([o.screen_x,o.screen_y]);return _},i._linear=function(e){for(var t=[],n=0;n<e.length;n++)t.push([e[n].screen_x,e[n].screen_y]);return t},e.exports=i},function(e,t,n){var r,i=n(8),o=(r=i)&&r.__esModule?r:{default:r},a=n(2),s=n(14),l=n(16);function u(e,t,n,r){this.body=e,this.defaultOptions={enabled:!1,icons:!0,iconSize:20,iconSpacing:6,left:{visible:!0,position:"top-left"},right:{visible:!0,position:"top-right"}},this.side=n,this.options=a.extend({},this.defaultOptions),this.linegraphOptions=r,this.svgElements={},this.dom={},this.groups={},this.amountOfGroups=0,this._create(),this.framework={svg:this.svg,svgElements:this.svgElements,options:this.options,groups:this.groups},this.setOptions(t)}u.prototype=new l,u.prototype.clear=function(){this.groups={},this.amountOfGroups=0},u.prototype.addGroup=function(e,t){1!=t.options.excludeFromLegend&&(this.groups.hasOwnProperty(e)||(this.groups[e]=t),this.amountOfGroups+=1)},u.prototype.updateGroup=function(e,t){this.groups[e]=t},u.prototype.removeGroup=function(e){this.groups.hasOwnProperty(e)&&(delete this.groups[e],this.amountOfGroups-=1)},u.prototype._create=function(){this.dom.frame=document.createElement("div"),this.dom.frame.className="vis-legend",this.dom.frame.style.position="absolute",this.dom.frame.style.top="10px",this.dom.frame.style.display="block",this.dom.textArea=document.createElement("div"),this.dom.textArea.className="vis-legend-text",this.dom.textArea.style.position="relative",this.dom.textArea.style.top="0px",this.svg=document.createElementNS("http://www.w3.org/2000/svg","svg"),this.svg.style.position="absolute",this.svg.style.top="0px",this.svg.style.width=this.options.iconSize+5+"px",this.svg.style.height="100%",this.dom.frame.appendChild(this.svg),this.dom.frame.appendChild(this.dom.textArea)},u.prototype.hide=function(){this.dom.frame.parentNode&&this.dom.frame.parentNode.removeChild(this.dom.frame)},u.prototype.show=function(){this.dom.frame.parentNode||this.body.dom.center.appendChild(this.dom.frame)},u.prototype.setOptions=function(e){a.selectiveDeepExtend(["enabled","orientation","icons","left","right"],this.options,e)},u.prototype.redraw=function(){var e=0,t=(0,o.default)(this.groups);t.sort((function(e,t){return e<t?-1:1}));for(var n=0;n<t.length;n++){var r=t[n];1!=this.groups[r].visible||void 0!==this.linegraphOptions.visibility[r]&&1!=this.linegraphOptions.visibility[r]||e++}if(0==this.options[this.side].visible||0==this.amountOfGroups||0==this.options.enabled||0==e)this.hide();else{if(this.show(),"top-left"==this.options[this.side].position||"bottom-left"==this.options[this.side].position?(this.dom.frame.style.left="4px",this.dom.frame.style.textAlign="left",this.dom.textArea.style.textAlign="left",this.dom.textArea.style.left=this.options.iconSize+15+"px",this.dom.textArea.style.right="",this.svg.style.left="0px",this.svg.style.right=""):(this.dom.frame.style.right="4px",this.dom.frame.style.textAlign="right",this.dom.textArea.style.textAlign="right",this.dom.textArea.style.right=this.options.iconSize+15+"px",this.dom.textArea.style.left="",this.svg.style.right="0px",this.svg.style.left=""),"top-left"==this.options[this.side].position||"top-right"==this.options[this.side].position)this.dom.frame.style.top=4-Number(this.body.dom.center.style.top.replace("px",""))+"px",this.dom.frame.style.bottom="";else{var i=this.body.domProps.center.height-this.body.domProps.centerContainer.height;this.dom.frame.style.bottom=4+i+Number(this.body.dom.center.style.top.replace("px",""))+"px",this.dom.frame.style.top=""}0==this.options.icons?(this.dom.frame.style.width=this.dom.textArea.offsetWidth+10+"px",this.dom.textArea.style.right="",this.dom.textArea.style.left="",this.svg.style.width="0px"):(this.dom.frame.style.width=this.options.iconSize+15+this.dom.textArea.offsetWidth+10+"px",this.drawLegendIcons());var a="";for(n=0;n<t.length;n++)r=t[n],1!=this.groups[r].visible||void 0!==this.linegraphOptions.visibility[r]&&1!=this.linegraphOptions.visibility[r]||(a+=this.groups[r].content+"<br />");this.dom.textArea.innerHTML=a,this.dom.textArea.style.lineHeight=.75*this.options.iconSize+this.options.iconSpacing+"px"}},u.prototype.drawLegendIcons=function(){if(this.dom.frame.parentNode){var e=(0,o.default)(this.groups);e.sort((function(e,t){return e<t?-1:1})),s.resetElements(this.svgElements);var t=window.getComputedStyle(this.dom.frame).paddingTop,n=Number(t.replace("px","")),r=n,i=this.options.iconSize,a=.75*this.options.iconSize,l=n+.5*a+3;this.svg.style.width=i+5+n+"px";for(var u=0;u<e.length;u++){var c=e[u];1!=this.groups[c].visible||void 0!==this.linegraphOptions.visibility[c]&&1!=this.linegraphOptions.visibility[c]||(this.groups[c].getLegend(i,a,this.framework,r,l),l+=a+this.options.iconSpacing)}}},e.exports=u},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r="string",i="boolean",o="number",a="object",s={configure:{enabled:{boolean:i},filter:{boolean:i,function:"function"},container:{dom:"dom"},__type__:{object:a,boolean:i,function:"function"}},yAxisOrientation:{string:["left","right"]},defaultGroup:{string:r},sort:{boolean:i},sampling:{boolean:i},stack:{boolean:i},graphHeight:{string:r,number:o},shaded:{enabled:{boolean:i},orientation:{string:["bottom","top","zero","group"]},groupId:{object:a},__type__:{boolean:i,object:a}},style:{string:["line","bar","points"]},barChart:{width:{number:o},minWidth:{number:o},sideBySide:{boolean:i},align:{string:["left","center","right"]},__type__:{object:a}},interpolation:{enabled:{boolean:i},parametrization:{string:["centripetal","chordal","uniform"]},alpha:{number:o},__type__:{object:a,boolean:i}},drawPoints:{enabled:{boolean:i},onRender:{function:"function"},size:{number:o},style:{string:["square","circle"]},__type__:{object:a,boolean:i,function:"function"}},dataAxis:{showMinorLabels:{boolean:i},showMajorLabels:{boolean:i},icons:{boolean:i},width:{string:r,number:o},visible:{boolean:i},alignZeros:{boolean:i},left:{range:{min:{number:o,undefined:"undefined"},max:{number:o,undefined:"undefined"},__type__:{object:a}},format:{function:"function"},title:{text:{string:r,number:o,undefined:"undefined"},style:{string:r,undefined:"undefined"},__type__:{object:a}},__type__:{object:a}},right:{range:{min:{number:o,undefined:"undefined"},max:{number:o,undefined:"undefined"},__type__:{object:a}},format:{function:"function"},title:{text:{string:r,number:o,undefined:"undefined"},style:{string:r,undefined:"undefined"},__type__:{object:a}},__type__:{object:a}},__type__:{object:a}},legend:{enabled:{boolean:i},icons:{boolean:i},left:{visible:{boolean:i},position:{string:["top-right","bottom-right","top-left","bottom-left"]},__type__:{object:a}},right:{visible:{boolean:i},position:{string:["top-right","bottom-right","top-left","bottom-left"]},__type__:{object:a}},__type__:{object:a,boolean:i}},groups:{visibility:{any:"any"},__type__:{object:a}},autoResize:{boolean:i},throttleRedraw:{number:o},clickToUse:{boolean:i},end:{number:o,date:"date",string:r,moment:"moment"},format:{minorLabels:{millisecond:{string:r,undefined:"undefined"},second:{string:r,undefined:"undefined"},minute:{string:r,undefined:"undefined"},hour:{string:r,undefined:"undefined"},weekday:{string:r,undefined:"undefined"},day:{string:r,undefined:"undefined"},month:{string:r,undefined:"undefined"},year:{string:r,undefined:"undefined"},__type__:{object:a}},majorLabels:{millisecond:{string:r,undefined:"undefined"},second:{string:r,undefined:"undefined"},minute:{string:r,undefined:"undefined"},hour:{string:r,undefined:"undefined"},weekday:{string:r,undefined:"undefined"},day:{string:r,undefined:"undefined"},month:{string:r,undefined:"undefined"},year:{string:r,undefined:"undefined"},__type__:{object:a}},__type__:{object:a}},moment:{function:"function"},height:{string:r,number:o},hiddenDates:{start:{date:"date",number:o,string:r,moment:"moment"},end:{date:"date",number:o,string:r,moment:"moment"},repeat:{string:r},__type__:{object:a,array:"array"}},locale:{string:r},locales:{__any__:{any:"any"},__type__:{object:a}},max:{date:"date",number:o,string:r,moment:"moment"},maxHeight:{number:o,string:r},maxMinorChars:{number:o},min:{date:"date",number:o,string:r,moment:"moment"},minHeight:{number:o,string:r},moveable:{boolean:i},multiselect:{boolean:i},orientation:{string:r},showCurrentTime:{boolean:i},showMajorLabels:{boolean:i},showMinorLabels:{boolean:i},start:{date:"date",number:o,string:r,moment:"moment"},timeAxis:{scale:{string:r,undefined:"undefined"},step:{number:o,undefined:"undefined"},__type__:{object:a}},width:{string:r,number:o},zoomable:{boolean:i},zoomKey:{string:["ctrlKey","altKey","metaKey",""]},zoomMax:{number:o},zoomMin:{number:o},zIndex:{number:o},__type__:{object:a}};t.allOptions=s,t.configureOptions={global:{sort:!0,sampling:!0,stack:!1,shaded:{enabled:!1,orientation:["zero","top","bottom","group"]},style:["line","bar","points"],barChart:{width:[50,5,100,5],minWidth:[50,5,100,5],sideBySide:!1,align:["left","center","right"]},interpolation:{enabled:!0,parametrization:["centripetal","chordal","uniform"]},drawPoints:{enabled:!0,size:[6,2,30,1],style:["square","circle"]},dataAxis:{showMinorLabels:!0,showMajorLabels:!0,icons:!1,width:[40,0,200,1],visible:!0,alignZeros:!0,left:{title:{text:"",style:""}},right:{title:{text:"",style:""}}},legend:{enabled:!1,icons:!0,left:{visible:!0,position:["top-right","bottom-right","top-left","bottom-left"]},right:{visible:!0,position:["top-right","bottom-right","top-left","bottom-left"]}},autoResize:!0,clickToUse:!1,end:"",format:{minorLabels:{millisecond:"SSS",second:"s",minute:"HH:mm",hour:"HH:mm",weekday:"ddd D",day:"D",month:"MMM",year:"YYYY"},majorLabels:{millisecond:"HH:mm:ss",second:"D MMMM HH:mm",minute:"ddd D MMMM",hour:"ddd D MMMM",weekday:"MMMM YYYY",day:"MMMM YYYY",month:"YYYY",year:""}},height:"",locale:"",max:"",maxHeight:"",maxMinorChars:[7,0,20,1],min:"",minHeight:"",moveable:!0,orientation:["both","bottom","top"],showCurrentTime:!1,showMajorLabels:!0,showMinorLabels:!0,start:"",width:"100%",zoomable:!0,zoomKey:["ctrlKey","altKey","metaKey",""],zoomMax:[31536e10,10,31536e10,1],zoomMin:[10,10,31536e10,1],zIndex:0}}},function(e,t,n){var r,i=n(29),o=(r=i)&&r.__esModule?r:{default:r};function a(e){return p=e,function(){var e={};if(g=0,v=p.charAt(0),T(),"strict"===m&&(e.strict=!0,T()),"graph"!==m&&"digraph"!==m||(e.type=m,T()),y===d&&(e.id=m,T()),"{"!=m)throw P("Angle bracket { expected");if(T(),E(e),"}"!=m)throw P("Angle bracket } expected");if(T(),""!==m)throw P("End of file expected");return T(),delete e.node,delete e.edge,delete e.graph,e}()}var s={fontsize:"font.size",fontcolor:"font.color",labelfontcolor:"font.color",fontname:"font.face",color:["color.border","color.background"],fillcolor:"color.background",tooltip:"title",labeltooltip:"title"},l=(0,o.default)(s);l.color="color.color",l.style="dashes";var u=0,c=1,d=2,h=3,f={"{":!0,"}":!0,"[":!0,"]":!0,";":!0,"=":!0,",":!0,"->":!0,"--":!0},p="",g=0,v="",m="",y=u;function b(){g++,v=p.charAt(g)}function C(){return p.charAt(g+1)}var A=/[a-zA-Z_0-9.:#]/;function _(e){return A.test(e)}function I(e,t){if(e||(e={}),t)for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n]);return e}function x(e,t,n){for(var r=t.split("."),i=e;r.length;){var o=r.shift();r.length?(i[o]||(i[o]={}),i=i[o]):i[o]=n}}function w(e,t){for(var n,r,i=null,o=[e],a=e;a.parent;)o.push(a.parent),a=a.parent;if(a.nodes)for(n=0,r=a.nodes.length;n<r;n++)if(t.id===a.nodes[n].id){i=a.nodes[n];break}for(i||(i={id:t.id},e.node&&(i.attr=I(i.attr,e.node))),n=o.length-1;n>=0;n--){var s=o[n];s.nodes||(s.nodes=[]),-1===s.nodes.indexOf(i)&&s.nodes.push(i)}t.attr&&(i.attr=I(i.attr,t.attr))}function M(e,t){if(e.edges||(e.edges=[]),e.edges.push(t),e.edge){var n=I({},e.edge);t.attr=I(n,t.attr)}}function S(e,t,n,r,i){var o={from:t,to:n,type:r};return e.edge&&(o.attr=I({},e.edge)),o.attr=I(o.attr||{},i),o}function T(){for(y=u,m="";" "===v||"\t"===v||"\n"===v||"\r"===v;)b();do{var e=!1;if("#"===v){for(var t=g-1;" "===p.charAt(t)||"\t"===p.charAt(t);)t--;if("\n"===p.charAt(t)||""===p.charAt(t)){for(;""!=v&&"\n"!=v;)b();e=!0}}if("/"===v&&"/"===C()){for(;""!=v&&"\n"!=v;)b();e=!0}if("/"===v&&"*"===C()){for(;""!=v;){if("*"===v&&"/"===C()){b(),b();break}b()}e=!0}for(;" "===v||"\t"===v||"\n"===v||"\r"===v;)b()}while(e);if(""!==v){var n=v+C();if(f[n])return y=c,m=n,b(),void b();if(f[v])return y=c,m=v,void b();if(_(v)||"-"===v){for(m+=v,b();_(v);)m+=v,b();return"false"===m?m=!1:"true"===m?m=!0:isNaN(Number(m))||(m=Number(m)),void(y=d)}if('"'===v){for(b();""!=v&&('"'!=v||'"'===v&&'"'===C());)'"'===v?(m+=v,b()):"\\"===v&&"n"===C()?(m+="\n",b()):m+=v,b();if('"'!=v)throw P('End of string " expected');return b(),void(y=d)}for(y=h;""!=v;)m+=v,b();throw new SyntaxError('Syntax error in part "'+N(m,30)+'"')}y=c}function E(e){for(;""!==m&&"}"!=m;)O(e),";"===m&&T()}function O(e){var t=k(e);if(t)L(e,t);else if(!function(e){return"node"===m?(T(),e.node=D(),"node"):"edge"===m?(T(),e.edge=D(),"edge"):"graph"===m?(T(),e.graph=D(),"graph"):null}(e)){if(y!=d)throw P("Identifier expected");var n=m;if(T(),"="===m){if(T(),y!=d)throw P("Identifier expected");e[n]=m,T()}else!function(e,t){var n={id:t},r=D();r&&(n.attr=r),w(e,n),L(e,t)}(e,n)}}function k(e){var t=null;if("subgraph"===m&&((t={}).type="subgraph",T(),y===d&&(t.id=m,T())),"{"===m){if(T(),t||(t={}),t.parent=e,t.node=e.node,t.edge=e.edge,t.graph=e.graph,E(t),"}"!=m)throw P("Angle bracket } expected");T(),delete t.node,delete t.edge,delete t.graph,delete t.parent,e.subgraphs||(e.subgraphs=[]),e.subgraphs.push(t)}return t}function L(e,t){for(;"->"===m||"--"===m;){var n,r=m;T();var i=k(e);if(i)n=i;else{if(y!=d)throw P("Identifier or subgraph expected");w(e,{id:n=m}),T()}M(e,S(e,t,n,r,D())),t=n}}function D(){for(var e=null,t={dashed:!0,solid:!1,dotted:[1,5]};"["===m;){for(T(),e={};""!==m&&"]"!=m;){if(y!=d)throw P("Attribute name expected");var n=m;if(T(),"="!=m)throw P("Equal sign = expected");if(T(),y!=d)throw P("Attribute value expected");var r=m;"style"===n&&(r=t[r]),x(e,n,r),T(),","==m&&T()}if("]"!=m)throw P("Bracket ] expected");T()}return e}function P(e){return new SyntaxError(e+', got "'+N(m,30)+'" (char '+g+")")}function N(e,t){return e.length<=t?e:e.substr(0,27)+"..."}function j(e,t,n){for(var r=t.split("."),i=r.pop(),o=e,a=0;a<r.length;a++){var s=r[a];s in o||(o[s]={}),o=o[s]}return o[i]=n,e}function R(e,t){var n={};for(var r in e)if(e.hasOwnProperty(r)){var i=t[r];Array.isArray(i)?i.forEach((function(t){j(n,t,e[r])})):j(n,"string"==typeof i?i:r,e[r])}return n}t.parseDOT=a,t.DOTToGraph=function(e){var t=a(e),n={nodes:[],edges:[],options:{}};if(t.nodes&&t.nodes.forEach((function(e){var t={id:e.id,label:String(e.label||e.id)};I(t,R(e.attr,s)),t.image&&(t.shape="image"),n.nodes.push(t)})),t.edges){var r=function(e){var t={from:e.from,to:e.to};return I(t,R(e.attr,l)),t.arrows="->"===e.type?"to":void 0,t};t.edges.forEach((function(e){var t,i,o,a,s;t=e.from instanceof Object?e.from.nodes:{id:e.from},i=e.to instanceof Object?e.to.nodes:{id:e.to},e.from instanceof Object&&e.from.edges&&e.from.edges.forEach((function(e){var t=r(e);n.edges.push(t)})),o=t,a=i,s=function(t,i){var o=S(n,t.id,i.id,e.type,e.attr),a=r(o);n.edges.push(a)},Array.isArray(o)?o.forEach((function(e){Array.isArray(a)?a.forEach((function(t){s(e,t)})):s(e,a)})):Array.isArray(a)?a.forEach((function(e){s(o,e)})):s(o,a),e.to instanceof Object&&e.to.edges&&e.to.edges.forEach((function(e){var t=r(e);n.edges.push(t)}))}))}return t.attr&&(n.options=t.attr),n}},function(e,t,n){t.parseGephi=function(e,t){var n=[],r=[],i={edges:{inheritColor:!1},nodes:{fixed:!1,parseColor:!1}};void 0!==t&&(void 0!==t.fixed&&(i.nodes.fixed=t.fixed),void 0!==t.parseColor&&(i.nodes.parseColor=t.parseColor),void 0!==t.inheritColor&&(i.edges.inheritColor=t.inheritColor));for(var o=e.edges,a=e.nodes,s=0;s<o.length;s++){var l={},u=o[s];l.id=u.id,l.from=u.source,l.to=u.target,l.attributes=u.attributes,l.label=u.label,l.title=void 0!==u.attributes?u.attributes.title:void 0,"Directed"===u.type&&(l.arrows="to"),u.color&&!1===i.inheritColor&&(l.color=u.color),n.push(l)}for(var c=0;c<a.length;c++){var d={},h=a[c];d.id=h.id,d.attributes=h.attributes,d.x=h.x,d.y=h.y,d.label=h.label,d.title=void 0!==h.attributes?h.attributes.title:h.title,!0===i.nodes.parseColor?d.color=h.color:d.color=void 0!==h.color?{background:h.color,border:h.color,highlight:{background:h.color,border:h.color},hover:{background:h.color,border:h.color}}:void 0,d.size=h.size,d.fixed=i.nodes.fixed&&void 0!==h.x&&void 0!==h.y,r.push(d)}return{nodes:r,edges:n}}},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=a(n(0)),i=a(n(1)),o=a(n(185));function a(e){return e&&e.__esModule?e:{default:e}}var s=function(){function e(t){(0,r.default)(this,e),this.images={},this.imageBroken={},this.callback=t}return(0,i.default)(e,[{key:"_tryloadBrokenUrl",value:function(e,t,n){void 0!==e&&void 0!==n&&(void 0!==t?(n.onerror=function(){console.error("Could not load brokenImage:",t)},n.image.src=t):console.warn("No broken url image defined"))}},{key:"_redrawWithImage",value:function(e){this.callback&&this.callback(e)}},{key:"load",value:function(e,t){var n=this,r=this.images[e];if(r)return r;var i=new o.default;return this.images[e]=i,i.image.onload=function(){n._fixImageCoordinates(i.image),i.init(),n._redrawWithImage(i)},i.image.onerror=function(){console.error("Could not load image:",e),n._tryloadBrokenUrl(e,t,i)},i.image.src=e,i}},{key:"_fixImageCoordinates",value:function(e){0===e.width&&(document.body.appendChild(e),e.width=e.offsetWidth,e.height=e.offsetHeight,document.body.removeChild(e))}}]),e}();t.default=s},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=s(n(30)),i=s(n(6)),o=s(n(0)),a=s(n(1));function s(e){return e&&e.__esModule?e:{default:e}}var l=n(2),u=n(48).default,c=n(191).default,d=["bold","ital","boldital","mono"],h=function(){function e(t,n){var r=arguments.length>2&&void 0!==arguments[2]&&arguments[2];(0,o.default)(this,e),this.body=t,this.pointToSelf=!1,this.baseSize=void 0,this.fontOptions={},this.setOptions(n),this.size={top:0,left:0,width:0,height:0,yLine:0},this.isEdgeLabel=r}return(0,a.default)(e,[{key:"setOptions",value:function(e){if(this.elementOptions=e,this.initFontOptions(e.font),u.isValidLabel(e.label)?this.labelDirty=!0:e.label="",void 0!==e.font&&null!==e.font)if("string"==typeof e.font)this.baseSize=this.fontOptions.size;else if("object"===(0,i.default)(e.font)){var t=e.font.size;void 0!==t&&(this.baseSize=t)}}},{key:"initFontOptions",value:function(t){var n=this;l.forEach(d,(function(e){n.fontOptions[e]={}})),e.parseFontString(this.fontOptions,t)?this.fontOptions.vadjust=0:l.forEach(t,(function(e,t){null!=e&&"object"!==(void 0===e?"undefined":(0,i.default)(e))&&(n.fontOptions[t]=e)}))}},{key:"constrain",value:function(e){var t={constrainWidth:!1,maxWdt:-1,minWdt:-1,constrainHeight:!1,minHgt:-1,valign:"middle"},n=l.topMost(e,"widthConstraint");if("number"==typeof n)t.maxWdt=Number(n),t.minWdt=Number(n);else if("object"===(void 0===n?"undefined":(0,i.default)(n))){var r=l.topMost(e,["widthConstraint","maximum"]);"number"==typeof r&&(t.maxWdt=Number(r));var o=l.topMost(e,["widthConstraint","minimum"]);"number"==typeof o&&(t.minWdt=Number(o))}var a=l.topMost(e,"heightConstraint");if("number"==typeof a)t.minHgt=Number(a);else if("object"===(void 0===a?"undefined":(0,i.default)(a))){var s=l.topMost(e,["heightConstraint","minimum"]);"number"==typeof s&&(t.minHgt=Number(s));var u=l.topMost(e,["heightConstraint","valign"]);"string"==typeof u&&("top"!==u&&"bottom"!==u||(t.valign=u))}return t}},{key:"update",value:function(e,t){this.setOptions(e,!0),this.propagateFonts(t),l.deepExtend(this.fontOptions,this.constrain(t)),this.fontOptions.chooser=u.choosify("label",t)}},{key:"adjustSizes",value:function(e){var t=e?e.right+e.left:0;this.fontOptions.constrainWidth&&(this.fontOptions.maxWdt-=t,this.fontOptions.minWdt-=t);var n=e?e.top+e.bottom:0;this.fontOptions.constrainHeight&&(this.fontOptions.minHgt-=n)}},{key:"addFontOptionsToPile",value:function(e,t){for(var n=0;n<t.length;++n)this.addFontToPile(e,t[n])}},{key:"addFontToPile",value:function(e,t){if(void 0!==t&&void 0!==t.font&&null!==t.font){var n=t.font;e.push(n)}}},{key:"getBasicOptions",value:function(t){for(var n={},r=0;r<t.length;++r){var i=t[r],o={};e.parseFontString(o,i)&&(i=o),l.forEach(i,(function(e,t){void 0!==e&&(n.hasOwnProperty(t)||(-1!==d.indexOf(t)?n[t]={}:n[t]=e))}))}return n}},{key:"getFontOption",value:function(t,n,r){for(var i=void 0,o=0;o<t.length;++o){var a=t[o];if(a.hasOwnProperty(n)){if(null==(i=a[n]))continue;var s={};if(e.parseFontString(s,i)&&(i=s),i.hasOwnProperty(r))return i[r]}}if(this.fontOptions.hasOwnProperty(r))return this.fontOptions[r];throw new Error("Did not find value for multi-font for property: '"+r+"'")}},{key:"getFontOptions",value:function(e,t){for(var n={},r=["color","size","face","mod","vadjust"],i=0;i<r.length;++i){var o=r[i];n[o]=this.getFontOption(e,t,o)}return n}},{key:"propagateFonts",value:function(e){var t=this,n=[];this.addFontOptionsToPile(n,e),this.fontOptions=this.getBasicOptions(n);for(var r=function(e){var r=d[e],i=t.fontOptions[r],o=t.getFontOptions(n,r);l.forEach(o,(function(e,t){i[t]=e})),i.size=Number(i.size),i.vadjust=Number(i.vadjust)},i=0;i<d.length;++i)r(i)}},{key:"draw",value:function(e,t,n,r,i){var o=arguments.length>5&&void 0!==arguments[5]?arguments[5]:"middle";if(void 0!==this.elementOptions.label){var a=this.fontOptions.size*this.body.view.scale;this.elementOptions.label&&a<this.elementOptions.scaling.label.drawThreshold-1||(a>=this.elementOptions.scaling.label.maxVisible&&(a=Number(this.elementOptions.scaling.label.maxVisible)/this.body.view.scale),this.calculateLabelSize(e,r,i,t,n,o),this._drawBackground(e),this._drawText(e,t,this.size.yLine,o,a))}}},{key:"_drawBackground",value:function(e){if(void 0!==this.fontOptions.background&&"none"!==this.fontOptions.background){e.fillStyle=this.fontOptions.background;var t=this.getSize();e.fillRect(t.left,t.top,t.width,t.height)}}},{key:"_drawText",value:function(e,t,n){var i=arguments.length>3&&void 0!==arguments[3]?arguments[3]:"middle",o=arguments[4],a=this._setAlignment(e,t,n,i),s=(0,r.default)(a,2);t=s[0],n=s[1],e.textAlign="left",t-=this.size.width/2,this.fontOptions.valign&&this.size.height>this.size.labelHeight&&("top"===this.fontOptions.valign&&(n-=(this.size.height-this.size.labelHeight)/2),"bottom"===this.fontOptions.valign&&(n+=(this.size.height-this.size.labelHeight)/2));for(var l=0;l<this.lineCount;l++){var u=this.lines[l];if(u&&u.blocks){var c=0;this.isEdgeLabel||"center"===this.fontOptions.align?c+=(this.size.width-u.width)/2:"right"===this.fontOptions.align&&(c+=this.size.width-u.width);for(var d=0;d<u.blocks.length;d++){var h=u.blocks[d];e.font=h.font;var f=this._getColor(h.color,o,h.strokeColor),p=(0,r.default)(f,2),g=p[0],v=p[1];h.strokeWidth>0&&(e.lineWidth=h.strokeWidth,e.strokeStyle=v,e.lineJoin="round"),e.fillStyle=g,h.strokeWidth>0&&e.strokeText(h.text,t+c,n+h.vadjust),e.fillText(h.text,t+c,n+h.vadjust),c+=h.width}n+=u.height}}}},{key:"_setAlignment",value:function(e,t,n,r){return this.isEdgeLabel&&"horizontal"!==this.fontOptions.align&&!1===this.pointToSelf?(t=0,n=0,"top"===this.fontOptions.align?(e.textBaseline="alphabetic",n-=4):"bottom"===this.fontOptions.align?(e.textBaseline="hanging",n+=4):e.textBaseline="middle"):e.textBaseline=r,[t,n]}},{key:"_getColor",value:function(e,t,n){var r=e||"#000000",i=n||"#ffffff";if(t<=this.elementOptions.scaling.label.drawThreshold){var o=Math.max(0,Math.min(1,1-(this.elementOptions.scaling.label.drawThreshold-t)));r=l.overrideOpacity(r,o),i=l.overrideOpacity(i,o)}return[r,i]}},{key:"getTextSize",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]&&arguments[1],n=arguments.length>2&&void 0!==arguments[2]&&arguments[2];return this._processLabel(e,t,n),{width:this.size.width,height:this.size.height,lineCount:this.lineCount}}},{key:"getSize",value:function(){var e=this.size.left,t=this.size.top-1;if(this.isEdgeLabel){var n=.5*-this.size.width;switch(this.fontOptions.align){case"middle":e=n,t=.5*-this.size.height;break;case"top":e=n,t=-(this.size.height+2);break;case"bottom":e=n,t=2}}return{left:e,top:t,width:this.size.width,height:this.size.height}}},{key:"calculateLabelSize",value:function(e,t,n){var r=arguments.length>3&&void 0!==arguments[3]?arguments[3]:0,i=arguments.length>4&&void 0!==arguments[4]?arguments[4]:0,o=arguments.length>5&&void 0!==arguments[5]?arguments[5]:"middle";this._processLabel(e,t,n),this.size.left=r-.5*this.size.width,this.size.top=i-.5*this.size.height,this.size.yLine=i+.5*(1-this.lineCount)*this.fontOptions.size,"hanging"===o&&(this.size.top+=.5*this.fontOptions.size,this.size.top+=4,this.size.yLine+=4)}},{key:"getFormattingValues",value:function(e,t,n,r){var i=function(e,t,n){return"normal"===t?"mod"===n?"":e[n]:void 0!==e[t][n]?e[t][n]:e[n]},o={color:i(this.fontOptions,r,"color"),size:i(this.fontOptions,r,"size"),face:i(this.fontOptions,r,"face"),mod:i(this.fontOptions,r,"mod"),vadjust:i(this.fontOptions,r,"vadjust"),strokeWidth:this.fontOptions.strokeWidth,strokeColor:this.fontOptions.strokeColor};(t||n)&&("normal"===r&&!0===this.fontOptions.chooser&&this.elementOptions.labelHighlightBold?o.mod="bold":"function"==typeof this.fontOptions.chooser&&this.fontOptions.chooser(o,this.elementOptions.id,t,n));var a="";return void 0!==o.mod&&""!==o.mod&&(a+=o.mod+" "),a+=o.size+"px "+o.face,e.font=a.replace(/"/g,""),o.font=e.font,o.height=o.size,o}},{key:"differentState",value:function(e,t){return e!==this.selectedState||t!==this.hoverState}},{key:"_processLabelText",value:function(e,t,n,r){return new c(e,this,t,n).process(r)}},{key:"_processLabel",value:function(e,t,n){if(!1!==this.labelDirty||this.differentState(t,n)){var r=this._processLabelText(e,t,n,this.elementOptions.label);this.fontOptions.minWdt>0&&r.width<this.fontOptions.minWdt&&(r.width=this.fontOptions.minWdt),this.size.labelHeight=r.height,this.fontOptions.minHgt>0&&r.height<this.fontOptions.minHgt&&(r.height=this.fontOptions.minHgt),this.lines=r.lines,this.lineCount=r.lines.length,this.size.width=r.width,this.size.height=r.height,this.selectedState=t,this.hoverState=n,this.labelDirty=!1}}},{key:"visible",value:function(){return 0!==this.size.width&&0!==this.size.height&&void 0!==this.elementOptions.label&&!(this.fontOptions.size*this.body.view.scale<this.elementOptions.scaling.label.drawThreshold-1)}}],[{key:"parseFontString",value:function(e,t){if(!t||"string"!=typeof t)return!1;var n=t.split(" ");return e.size=n[0].replace("px",""),e.face=n[1],e.color=n[2],!0}}]),e}();t.default=h},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=a(n(30)),i=a(n(0)),o=a(n(1));function a(e){return e&&e.__esModule?e:{default:e}}var s=n(2),l=n(119).default,u=function(){function e(t,n,r){(0,i.default)(this,e),this.body=n,this.labelModule=r,this.options={},this.setOptions(t),this.colorDirty=!0,this.color={},this.selectionWidth=2,this.hoverWidth=1.5,this.fromPoint=this.from,this.toPoint=this.to}return(0,o.default)(e,[{key:"connect",value:function(){this.from=this.body.nodes[this.options.from],this.to=this.body.nodes[this.options.to]}},{key:"cleanup",value:function(){return!1}},{key:"setOptions",value:function(e){this.options=e,this.from=this.body.nodes[this.options.from],this.to=this.body.nodes[this.options.to],this.id=this.options.id}},{key:"drawLine",value:function(e,t,n,r,i){e.strokeStyle=this.getColor(e,t,n,r),e.lineWidth=t.width,!1!==t.dashes?this._drawDashedLine(e,t,i):this._drawLine(e,t,i)}},{key:"_drawLine",value:function(e,t,n,i,o){if(this.from!=this.to)this._line(e,t,n,i,o);else{var a=this._getCircleData(e),s=(0,r.default)(a,3),l=s[0],u=s[1],c=s[2];this._circle(e,t,l,u,c)}}},{key:"_drawDashedLine",value:function(e,t,n,i,o){e.lineCap="round";var a=[5,5];if(!0===Array.isArray(t.dashes)&&(a=t.dashes),void 0!==e.setLineDash){if(e.save(),e.setLineDash(a),e.lineDashOffset=0,this.from!=this.to)this._line(e,t,n);else{var s=this._getCircleData(e),l=(0,r.default)(s,3),u=l[0],c=l[1],d=l[2];this._circle(e,t,u,c,d)}e.setLineDash([0]),e.lineDashOffset=0,e.restore()}else{if(this.from!=this.to)e.dashedLine(this.from.x,this.from.y,this.to.x,this.to.y,a);else{var h=this._getCircleData(e),f=(0,r.default)(h,3),p=f[0],g=f[1],v=f[2];this._circle(e,t,p,g,v)}this.enableShadow(e,t),e.stroke(),this.disableShadow(e,t)}}},{key:"findBorderPosition",value:function(e,t,n){return this.from!=this.to?this._findBorderPosition(e,t,n):this._findBorderPositionCircle(e,t,n)}},{key:"findBorderPositions",value:function(e){var t={},n={};if(this.from!=this.to)t=this._findBorderPosition(this.from,e),n=this._findBorderPosition(this.to,e);else{var i=this._getCircleData(e).slice(0,2),o=(0,r.default)(i,2),a=o[0],s=o[1];t=this._findBorderPositionCircle(this.from,e,{x:a,y:s,low:.25,high:.6,direction:-1}),n=this._findBorderPositionCircle(this.from,e,{x:a,y:s,low:.6,high:.8,direction:1})}return{from:t,to:n}}},{key:"_getCircleData",value:function(e){var t=void 0,n=void 0,r=this.from,i=this.options.selfReferenceSize;return void 0!==e&&void 0===r.shape.width&&r.shape.resize(e),r.shape.width>r.shape.height?(t=r.x+.5*r.shape.width,n=r.y-i):(t=r.x+i,n=r.y-.5*r.shape.height),[t,n,i]}},{key:"_pointOnCircle",value:function(e,t,n,r){var i=2*r*Math.PI;return{x:e+n*Math.cos(i),y:t-n*Math.sin(i)}}},{key:"_findBorderPositionCircle",value:function(e,t,n){for(var r=n.x,i=n.y,o=n.low,a=n.high,s=n.direction,l=0,u=this.options.selfReferenceSize,c=void 0,d=void 0,h=void 0,f=.5*(o+a);o<=a&&l<10&&(f=.5*(o+a),c=this._pointOnCircle(r,i,u,f),d=Math.atan2(e.y-c.y,e.x-c.x),h=e.distanceToBorder(t,d)-Math.sqrt(Math.pow(c.x-e.x,2)+Math.pow(c.y-e.y,2)),!(Math.abs(h)<.05));)h>0?s>0?o=f:a=f:s>0?a=f:o=f,l++;return c.t=f,c}},{key:"getLineWidth",value:function(e,t){return!0===e?Math.max(this.selectionWidth,.3/this.body.view.scale):!0===t?Math.max(this.hoverWidth,.3/this.body.view.scale):Math.max(this.options.width,.3/this.body.view.scale)}},{key:"getColor",value:function(e,t,n,r){if(!1!==t.inheritsColor){if("both"===t.inheritsColor&&this.from.id!==this.to.id){var i=e.createLinearGradient(this.from.x,this.from.y,this.to.x,this.to.y),o=void 0,a=void 0;return o=this.from.options.color.highlight.border,a=this.to.options.color.highlight.border,!1===this.from.selected&&!1===this.to.selected?(o=s.overrideOpacity(this.from.options.color.border,t.opacity),a=s.overrideOpacity(this.to.options.color.border,t.opacity)):!0===this.from.selected&&!1===this.to.selected?a=this.to.options.color.border:!1===this.from.selected&&!0===this.to.selected&&(o=this.from.options.color.border),i.addColorStop(0,o),i.addColorStop(1,a),i}return"to"===t.inheritsColor?s.overrideOpacity(this.to.options.color.border,t.opacity):s.overrideOpacity(this.from.options.color.border,t.opacity)}return s.overrideOpacity(t.color,t.opacity)}},{key:"_circle",value:function(e,t,n,r,i){this.enableShadow(e,t),e.beginPath(),e.arc(n,r,i,0,2*Math.PI,!1),e.stroke(),this.disableShadow(e,t)}},{key:"getDistanceToEdge",value:function(e,t,n,i,o,a,s,l){var u=0;if(this.from!=this.to)u=this._getDistanceToEdge(e,t,n,i,o,a,s);else{var c=this._getCircleData(void 0),d=(0,r.default)(c,3),h=d[0],f=d[1],p=d[2],g=h-o,v=f-a;u=Math.abs(Math.sqrt(g*g+v*v)-p)}return u}},{key:"_getDistanceToLine",value:function(e,t,n,r,i,o){var a=n-e,s=r-t,l=((i-e)*a+(o-t)*s)/(a*a+s*s);l>1?l=1:l<0&&(l=0);var u=e+l*a-i,c=t+l*s-o;return Math.sqrt(u*u+c*c)}},{key:"getArrowData",value:function(e,t,n,i,o,a){var s=void 0,l=void 0,u=void 0,c=void 0,d=void 0,h=void 0,f=void 0,p=a.width;if("from"===t?(u=this.from,c=this.to,d=.1,h=a.fromArrowScale,f=a.fromArrowType):"to"===t?(u=this.to,c=this.from,d=-.1,h=a.toArrowScale,f=a.toArrowType):(u=this.to,c=this.from,h=a.middleArrowScale,f=a.middleArrowType),u!=c)if("middle"!==t)if(!0===this.options.smooth.enabled){l=this.findBorderPosition(u,e,{via:n});var g=this.getPoint(Math.max(0,Math.min(1,l.t+d)),n);s=Math.atan2(l.y-g.y,l.x-g.x)}else s=Math.atan2(u.y-c.y,u.x-c.x),l=this.findBorderPosition(u,e);else s=Math.atan2(u.y-c.y,u.x-c.x),l=this.getPoint(.5,n);else{var v=this._getCircleData(e),m=(0,r.default)(v,3),y=m[0],b=m[1],C=m[2];"from"===t?s=-2*(l=this.findBorderPosition(this.from,e,{x:y,y:b,low:.25,high:.6,direction:-1})).t*Math.PI+1.5*Math.PI+.1*Math.PI:"to"===t?s=-2*(l=this.findBorderPosition(this.from,e,{x:y,y:b,low:.6,high:1,direction:1})).t*Math.PI+1.5*Math.PI-1.1*Math.PI:(l=this._pointOnCircle(y,b,C,.175),s=3.9269908169872414)}"middle"===t&&h<0&&(p*=-1);var A=15*h+3*p;return{point:l,core:{x:l.x-.9*A*Math.cos(s),y:l.y-.9*A*Math.sin(s)},angle:s,length:A,type:f}}},{key:"drawArrowHead",value:function(e,t,n,r,i){e.strokeStyle=this.getColor(e,t,n,r),e.fillStyle=e.strokeStyle,e.lineWidth=t.width,l.draw(e,i),this.enableShadow(e,t),e.fill(),this.disableShadow(e,t)}},{key:"enableShadow",value:function(e,t){!0===t.shadow&&(e.shadowColor=t.shadowColor,e.shadowBlur=t.shadowSize,e.shadowOffsetX=t.shadowX,e.shadowOffsetY=t.shadowY)}},{key:"disableShadow",value:function(e,t){!0===t.shadow&&(e.shadowColor="rgba(0,0,0,0)",e.shadowBlur=0,e.shadowOffsetX=0,e.shadowOffsetY=0)}}]),e}();t.default=u},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=l(n(3)),i=l(n(4)),o=l(n(5)),a=l(n(0)),s=l(n(1));function l(e){return e&&e.__esModule?e:{default:e}}var u=function(){function e(){(0,a.default)(this,e)}return(0,s.default)(e,null,[{key:"transform",value:function(e,t){e instanceof Array||(e=[e]);for(var n=t.point.x,r=t.point.y,i=t.angle,o=t.length,a=0;a<e.length;++a){var s=e[a],l=s.x*Math.cos(i)-s.y*Math.sin(i),u=s.x*Math.sin(i)+s.y*Math.cos(i);s.x=n+o*l,s.y=r+o*u}}},{key:"drawPath",value:function(e,t){e.beginPath(),e.moveTo(t[0].x,t[0].y);for(var n=1;n<t.length;++n)e.lineTo(t[n].x,t[n].y);e.closePath()}}]),e}(),c=function(e){function t(){return(0,a.default)(this,t),(0,i.default)(this,(t.__proto__||(0,r.default)(t)).apply(this,arguments))}return(0,o.default)(t,e),(0,s.default)(t,null,[{key:"draw",value:function(e,t){var n=[{x:0,y:0},{x:-1,y:.3},{x:-.9,y:0},{x:-1,y:-.3}];u.transform(n,t),u.drawPath(e,n)}}]),t}(u),d=function(){function e(){(0,a.default)(this,e)}return(0,s.default)(e,null,[{key:"draw",value:function(e,t){var n={x:-.4,y:0};u.transform(n,t),e.circle(n.x,n.y,.4*t.length)}}]),e}(),h=function(){function e(){(0,a.default)(this,e)}return(0,s.default)(e,null,[{key:"draw",value:function(e,t){var n=[{x:0,y:.5},{x:0,y:-.5},{x:-.15,y:-.5},{x:-.15,y:.5}];u.transform(n,t),u.drawPath(e,n)}}]),e}(),f=function(){function e(){(0,a.default)(this,e)}return(0,s.default)(e,null,[{key:"draw",value:function(e,t){var n;switch(t.type&&(n=t.type.toLowerCase()),n){case"circle":d.draw(e,t);break;case"bar":h.draw(e,t);break;case"arrow":default:c.draw(e,t)}}}]),e}();t.default=f},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=o(n(0)),i=o(n(1));function o(e){return e&&e.__esModule?e:{default:e}}var a=function(){function e(t,n,i){(0,r.default)(this,e),this.body=t,this.physicsBody=n,this.barnesHutTree,this.setOptions(i),this.randomSeed=5}return(0,i.default)(e,[{key:"setOptions",value:function(e){this.options=e,this.thetaInversed=1/this.options.theta,this.overlapAvoidanceFactor=1-Math.max(0,Math.min(1,this.options.avoidOverlap))}},{key:"seededRandom",value:function(){var e=1e4*Math.sin(this.randomSeed++);return e-Math.floor(e)}},{key:"solve",value:function(){if(0!==this.options.gravitationalConstant&&this.physicsBody.physicsNodeIndices.length>0){var e=void 0,t=this.body.nodes,n=this.physicsBody.physicsNodeIndices,r=n.length,i=this._formBarnesHutTree(t,n);this.barnesHutTree=i;for(var o=0;o<r;o++)(e=t[n[o]]).options.mass>0&&this._getForceContributions(i.root,e)}}},{key:"_getForceContributions",value:function(e,t){this._getForceContribution(e.children.NW,t),this._getForceContribution(e.children.NE,t),this._getForceContribution(e.children.SW,t),this._getForceContribution(e.children.SE,t)}},{key:"_getForceContribution",value:function(e,t){var n,r,i;e.childrenCount>0&&(n=e.centerOfMass.x-t.x,r=e.centerOfMass.y-t.y,(i=Math.sqrt(n*n+r*r))*e.calcSize>this.thetaInversed?this._calculateForces(i,n,r,t,e):4===e.childrenCount?this._getForceContributions(e,t):e.children.data.id!=t.id&&this._calculateForces(i,n,r,t,e))}},{key:"_calculateForces",value:function(e,t,n,r,i){0===e&&(t=e=.1),this.overlapAvoidanceFactor<1&&r.shape.radius&&(e=Math.max(.1+this.overlapAvoidanceFactor*r.shape.radius,e-r.shape.radius));var o=this.options.gravitationalConstant*i.mass*r.options.mass/Math.pow(e,3),a=t*o,s=n*o;this.physicsBody.forces[r.id].x+=a,this.physicsBody.forces[r.id].y+=s}},{key:"_formBarnesHutTree",value:function(e,t){for(var n=void 0,r=t.length,i=e[t[0]].x,o=e[t[0]].y,a=e[t[0]].x,s=e[t[0]].y,l=1;l<r;l++){var u=e[t[l]],c=u.x,d=u.y;u.options.mass>0&&(c<i&&(i=c),c>a&&(a=c),d<o&&(o=d),d>s&&(s=d))}var h=Math.abs(a-i)-Math.abs(s-o);h>0?(o-=.5*h,s+=.5*h):(i+=.5*h,a-=.5*h);var f=Math.max(1e-5,Math.abs(a-i)),p=.5*f,g=.5*(i+a),v=.5*(o+s),m={root:{centerOfMass:{x:0,y:0},mass:0,range:{minX:g-p,maxX:g+p,minY:v-p,maxY:v+p},size:f,calcSize:1/f,children:{data:null},maxWidth:0,level:0,childrenCount:4}};this._splitBranch(m.root);for(var y=0;y<r;y++)(n=e[t[y]]).options.mass>0&&this._placeInTree(m.root,n);return m}},{key:"_updateBranchMass",value:function(e,t){var n=e.centerOfMass,r=e.mass+t.options.mass,i=1/r;n.x=n.x*e.mass+t.x*t.options.mass,n.x*=i,n.y=n.y*e.mass+t.y*t.options.mass,n.y*=i,e.mass=r;var o=Math.max(Math.max(t.height,t.radius),t.width);e.maxWidth=e.maxWidth<o?o:e.maxWidth}},{key:"_placeInTree",value:function(e,t,n){1==n&&void 0!==n||this._updateBranchMass(e,t);var r=e.children.NW.range,i=void 0;i=r.maxX>t.x?r.maxY>t.y?"NW":"SW":r.maxY>t.y?"NE":"SE",this._placeInRegion(e,t,i)}},{key:"_placeInRegion",value:function(e,t,n){var r=e.children[n];switch(r.childrenCount){case 0:r.children.data=t,r.childrenCount=1,this._updateBranchMass(r,t);break;case 1:r.children.data.x===t.x&&r.children.data.y===t.y?(t.x+=this.seededRandom(),t.y+=this.seededRandom()):(this._splitBranch(r),this._placeInTree(r,t));break;case 4:this._placeInTree(r,t)}}},{key:"_splitBranch",value:function(e){var t=null;1===e.childrenCount&&(t=e.children.data,e.mass=0,e.centerOfMass.x=0,e.centerOfMass.y=0),e.childrenCount=4,e.children.data=null,this._insertRegion(e,"NW"),this._insertRegion(e,"NE"),this._insertRegion(e,"SW"),this._insertRegion(e,"SE"),null!=t&&this._placeInTree(e,t)}},{key:"_insertRegion",value:function(e,t){var n=void 0,r=void 0,i=void 0,o=void 0,a=.5*e.size;switch(t){case"NW":n=e.range.minX,r=e.range.minX+a,i=e.range.minY,o=e.range.minY+a;break;case"NE":n=e.range.minX+a,r=e.range.maxX,i=e.range.minY,o=e.range.minY+a;break;case"SW":n=e.range.minX,r=e.range.minX+a,i=e.range.minY+a,o=e.range.maxY;break;case"SE":n=e.range.minX+a,r=e.range.maxX,i=e.range.minY+a,o=e.range.maxY}e.children[t]={centerOfMass:{x:0,y:0},mass:0,range:{minX:n,maxX:r,minY:i,maxY:o},size:.5*e.size,calcSize:2*e.calcSize,children:{data:null},maxWidth:0,level:e.level+1,childrenCount:0}}},{key:"_debug",value:function(e,t){void 0!==this.barnesHutTree&&(e.lineWidth=1,this._drawBranch(this.barnesHutTree.root,e,t))}},{key:"_drawBranch",value:function(e,t,n){void 0===n&&(n="#FF0000"),4===e.childrenCount&&(this._drawBranch(e.children.NW,t),this._drawBranch(e.children.NE,t),this._drawBranch(e.children.SE,t),this._drawBranch(e.children.SW,t)),t.strokeStyle=n,t.beginPath(),t.moveTo(e.range.minX,e.range.minY),t.lineTo(e.range.maxX,e.range.minY),t.stroke(),t.beginPath(),t.moveTo(e.range.maxX,e.range.minY),t.lineTo(e.range.maxX,e.range.maxY),t.stroke(),t.beginPath(),t.moveTo(e.range.maxX,e.range.maxY),t.lineTo(e.range.minX,e.range.maxY),t.stroke(),t.beginPath(),t.moveTo(e.range.minX,e.range.maxY),t.lineTo(e.range.minX,e.range.minY),t.stroke()}}]),e}();t.default=a},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=o(n(0)),i=o(n(1));function o(e){return e&&e.__esModule?e:{default:e}}var a=function(){function e(t,n,i){(0,r.default)(this,e),this.body=t,this.physicsBody=n,this.setOptions(i)}return(0,i.default)(e,[{key:"setOptions",value:function(e){this.options=e}},{key:"solve",value:function(){for(var e=void 0,t=void 0,n=void 0,r=void 0,i=this.body.nodes,o=this.physicsBody.physicsNodeIndices,a=this.physicsBody.forces,s=0;s<o.length;s++)e=-(r=i[o[s]]).x,t=-r.y,n=Math.sqrt(e*e+t*t),this._calculateForces(n,e,t,a,r)}},{key:"_calculateForces",value:function(e,t,n,r,i){var o=0===e?0:this.options.centralGravity/e;r[i.id].x=t*o,r[i.id].y=n*o}}]),e}();t.default=a},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r="string",i="boolean",o="number",a="object",s=["arrow","circle","bar"],l={configure:{enabled:{boolean:i},filter:{boolean:i,string:r,array:"array",function:"function"},container:{dom:"dom"},showButton:{boolean:i},__type__:{object:a,boolean:i,string:r,array:"array",function:"function"}},edges:{arrows:{to:{enabled:{boolean:i},scaleFactor:{number:o},type:{string:s},__type__:{object:a,boolean:i}},middle:{enabled:{boolean:i},scaleFactor:{number:o},type:{string:s},__type__:{object:a,boolean:i}},from:{enabled:{boolean:i},scaleFactor:{number:o},type:{string:s},__type__:{object:a,boolean:i}},__type__:{string:["from","to","middle"],object:a}},arrowStrikethrough:{boolean:i},chosen:{label:{boolean:i,function:"function"},edge:{boolean:i,function:"function"},__type__:{object:a,boolean:i}},color:{color:{string:r},highlight:{string:r},hover:{string:r},inherit:{string:["from","to","both"],boolean:i},opacity:{number:o},__type__:{object:a,string:r}},dashes:{boolean:i,array:"array"},font:{color:{string:r},size:{number:o},face:{string:r},background:{string:r},strokeWidth:{number:o},strokeColor:{string:r},align:{string:["horizontal","top","middle","bottom"]},vadjust:{number:o},multi:{boolean:i,string:r},bold:{color:{string:r},size:{number:o},face:{string:r},mod:{string:r},vadjust:{number:o},__type__:{object:a,string:r}},boldital:{color:{string:r},size:{number:o},face:{string:r},mod:{string:r},vadjust:{number:o},__type__:{object:a,string:r}},ital:{color:{string:r},size:{number:o},face:{string:r},mod:{string:r},vadjust:{number:o},__type__:{object:a,string:r}},mono:{color:{string:r},size:{number:o},face:{string:r},mod:{string:r},vadjust:{number:o},__type__:{object:a,string:r}},__type__:{object:a,string:r}},hidden:{boolean:i},hoverWidth:{function:"function",number:o},label:{string:r,undefined:"undefined"},labelHighlightBold:{boolean:i},length:{number:o,undefined:"undefined"},physics:{boolean:i},scaling:{min:{number:o},max:{number:o},label:{enabled:{boolean:i},min:{number:o},max:{number:o},maxVisible:{number:o},drawThreshold:{number:o},__type__:{object:a,boolean:i}},customScalingFunction:{function:"function"},__type__:{object:a}},selectionWidth:{function:"function",number:o},selfReferenceSize:{number:o},shadow:{enabled:{boolean:i},color:{string:r},size:{number:o},x:{number:o},y:{number:o},__type__:{object:a,boolean:i}},smooth:{enabled:{boolean:i},type:{string:["dynamic","continuous","discrete","diagonalCross","straightCross","horizontal","vertical","curvedCW","curvedCCW","cubicBezier"]},roundness:{number:o},forceDirection:{string:["horizontal","vertical","none"],boolean:i},__type__:{object:a,boolean:i}},title:{string:r,undefined:"undefined"},width:{number:o},widthConstraint:{maximum:{number:o},__type__:{object:a,boolean:i,number:o}},value:{number:o,undefined:"undefined"},__type__:{object:a}},groups:{useDefaultGroups:{boolean:i},__any__:"get from nodes, will be overwritten below",__type__:{object:a}},interaction:{dragNodes:{boolean:i},dragView:{boolean:i},hideEdgesOnDrag:{boolean:i},hideNodesOnDrag:{boolean:i},hover:{boolean:i},keyboard:{enabled:{boolean:i},speed:{x:{number:o},y:{number:o},zoom:{number:o},__type__:{object:a}},bindToWindow:{boolean:i},__type__:{object:a,boolean:i}},multiselect:{boolean:i},navigationButtons:{boolean:i},selectable:{boolean:i},selectConnectedEdges:{boolean:i},hoverConnectedEdges:{boolean:i},tooltipDelay:{number:o},zoomView:{boolean:i},__type__:{object:a}},layout:{randomSeed:{undefined:"undefined",number:o},improvedLayout:{boolean:i},hierarchical:{enabled:{boolean:i},levelSeparation:{number:o},nodeSpacing:{number:o},treeSpacing:{number:o},blockShifting:{boolean:i},edgeMinimization:{boolean:i},parentCentralization:{boolean:i},direction:{string:["UD","DU","LR","RL"]},sortMethod:{string:["hubsize","directed"]},__type__:{object:a,boolean:i}},__type__:{object:a}},manipulation:{enabled:{boolean:i},initiallyActive:{boolean:i},addNode:{boolean:i,function:"function"},addEdge:{boolean:i,function:"function"},editNode:{function:"function"},editEdge:{editWithoutDrag:{function:"function"},__type__:{object:a,boolean:i,function:"function"}},deleteNode:{boolean:i,function:"function"},deleteEdge:{boolean:i,function:"function"},controlNodeStyle:"get from nodes, will be overwritten below",__type__:{object:a,boolean:i}},nodes:{borderWidth:{number:o},borderWidthSelected:{number:o,undefined:"undefined"},brokenImage:{string:r,undefined:"undefined"},chosen:{label:{boolean:i,function:"function"},node:{boolean:i,function:"function"},__type__:{object:a,boolean:i}},color:{border:{string:r},background:{string:r},highlight:{border:{string:r},background:{string:r},__type__:{object:a,string:r}},hover:{border:{string:r},background:{string:r},__type__:{object:a,string:r}},__type__:{object:a,string:r}},fixed:{x:{boolean:i},y:{boolean:i},__type__:{object:a,boolean:i}},font:{align:{string:r},color:{string:r},size:{number:o},face:{string:r},background:{string:r},strokeWidth:{number:o},strokeColor:{string:r},vadjust:{number:o},multi:{boolean:i,string:r},bold:{color:{string:r},size:{number:o},face:{string:r},mod:{string:r},vadjust:{number:o},__type__:{object:a,string:r}},boldital:{color:{string:r},size:{number:o},face:{string:r},mod:{string:r},vadjust:{number:o},__type__:{object:a,string:r}},ital:{color:{string:r},size:{number:o},face:{string:r},mod:{string:r},vadjust:{number:o},__type__:{object:a,string:r}},mono:{color:{string:r},size:{number:o},face:{string:r},mod:{string:r},vadjust:{number:o},__type__:{object:a,string:r}},__type__:{object:a,string:r}},group:{string:r,number:o,undefined:"undefined"},heightConstraint:{minimum:{number:o},valign:{string:r},__type__:{object:a,boolean:i,number:o}},hidden:{boolean:i},icon:{face:{string:r},code:{string:r},size:{number:o},color:{string:r},__type__:{object:a}},id:{string:r,number:o},image:{selected:{string:r,undefined:"undefined"},unselected:{string:r,undefined:"undefined"},__type__:{object:a,string:r}},label:{string:r,undefined:"undefined"},labelHighlightBold:{boolean:i},level:{number:o,undefined:"undefined"},margin:{top:{number:o},right:{number:o},bottom:{number:o},left:{number:o},__type__:{object:a,number:o}},mass:{number:o},physics:{boolean:i},scaling:{min:{number:o},max:{number:o},label:{enabled:{boolean:i},min:{number:o},max:{number:o},maxVisible:{number:o},drawThreshold:{number:o},__type__:{object:a,boolean:i}},customScalingFunction:{function:"function"},__type__:{object:a}},shadow:{enabled:{boolean:i},color:{string:r},size:{number:o},x:{number:o},y:{number:o},__type__:{object:a,boolean:i}},shape:{string:["ellipse","circle","database","box","text","image","circularImage","diamond","dot","star","triangle","triangleDown","square","icon","hexagon"]},shapeProperties:{borderDashes:{boolean:i,array:"array"},borderRadius:{number:o},interpolation:{boolean:i},useImageSize:{boolean:i},useBorderWithImage:{boolean:i},__type__:{object:a}},size:{number:o},title:{string:r,dom:"dom",undefined:"undefined"},value:{number:o,undefined:"undefined"},widthConstraint:{minimum:{number:o},maximum:{number:o},__type__:{object:a,boolean:i,number:o}},x:{number:o},y:{number:o},__type__:{object:a}},physics:{enabled:{boolean:i},barnesHut:{gravitationalConstant:{number:o},centralGravity:{number:o},springLength:{number:o},springConstant:{number:o},damping:{number:o},avoidOverlap:{number:o},__type__:{object:a}},forceAtlas2Based:{gravitationalConstant:{number:o},centralGravity:{number:o},springLength:{number:o},springConstant:{number:o},damping:{number:o},avoidOverlap:{number:o},__type__:{object:a}},repulsion:{centralGravity:{number:o},springLength:{number:o},springConstant:{number:o},nodeDistance:{number:o},damping:{number:o},__type__:{object:a}},hierarchicalRepulsion:{centralGravity:{number:o},springLength:{number:o},springConstant:{number:o},nodeDistance:{number:o},damping:{number:o},__type__:{object:a}},maxVelocity:{number:o},minVelocity:{number:o},solver:{string:["barnesHut","repulsion","hierarchicalRepulsion","forceAtlas2Based"]},stabilization:{enabled:{boolean:i},iterations:{number:o},updateInterval:{number:o},onlyDynamicEdges:{boolean:i},fit:{boolean:i},__type__:{object:a,boolean:i}},timestep:{number:o},adaptiveTimestep:{boolean:i},__type__:{object:a,boolean:i}},autoResize:{boolean:i},clickToUse:{boolean:i},locale:{string:r},locales:{__any__:{any:"any"},__type__:{object:a}},height:{string:r},width:{string:r},__type__:{object:a}};l.groups.__any__=l.nodes,l.manipulation.controlNodeStyle=l.nodes,t.allOptions=l,t.configureOptions={nodes:{borderWidth:[1,0,10,1],borderWidthSelected:[2,0,10,1],color:{border:["color","#2B7CE9"],background:["color","#97C2FC"],highlight:{border:["color","#2B7CE9"],background:["color","#D2E5FF"]},hover:{border:["color","#2B7CE9"],background:["color","#D2E5FF"]}},fixed:{x:!1,y:!1},font:{color:["color","#343434"],size:[14,0,100,1],face:["arial","verdana","tahoma"],background:["color","none"],strokeWidth:[0,0,50,1],strokeColor:["color","#ffffff"]},hidden:!1,labelHighlightBold:!0,physics:!0,scaling:{min:[10,0,200,1],max:[30,0,200,1],label:{enabled:!1,min:[14,0,200,1],max:[30,0,200,1],maxVisible:[30,0,200,1],drawThreshold:[5,0,20,1]}},shadow:{enabled:!1,color:"rgba(0,0,0,0.5)",size:[10,0,20,1],x:[5,-30,30,1],y:[5,-30,30,1]},shape:["ellipse","box","circle","database","diamond","dot","square","star","text","triangle","triangleDown","hexagon"],shapeProperties:{borderDashes:!1,borderRadius:[6,0,20,1],interpolation:!0,useImageSize:!1},size:[25,0,200,1]},edges:{arrows:{to:{enabled:!1,scaleFactor:[1,0,3,.05],type:"arrow"},middle:{enabled:!1,scaleFactor:[1,0,3,.05],type:"arrow"},from:{enabled:!1,scaleFactor:[1,0,3,.05],type:"arrow"}},arrowStrikethrough:!0,color:{color:["color","#848484"],highlight:["color","#848484"],hover:["color","#848484"],inherit:["from","to","both",!0,!1],opacity:[1,0,1,.05]},dashes:!1,font:{color:["color","#343434"],size:[14,0,100,1],face:["arial","verdana","tahoma"],background:["color","none"],strokeWidth:[2,0,50,1],strokeColor:["color","#ffffff"],align:["horizontal","top","middle","bottom"]},hidden:!1,hoverWidth:[1.5,0,5,.1],labelHighlightBold:!0,physics:!0,scaling:{min:[1,0,100,1],max:[15,0,100,1],label:{enabled:!0,min:[14,0,200,1],max:[30,0,200,1],maxVisible:[30,0,200,1],drawThreshold:[5,0,20,1]}},selectionWidth:[1.5,0,5,.1],selfReferenceSize:[20,0,200,1],shadow:{enabled:!1,color:"rgba(0,0,0,0.5)",size:[10,0,20,1],x:[5,-30,30,1],y:[5,-30,30,1]},smooth:{enabled:!0,type:["dynamic","continuous","discrete","diagonalCross","straightCross","horizontal","vertical","curvedCW","curvedCCW","cubicBezier"],forceDirection:["horizontal","vertical","none"],roundness:[.5,0,1,.05]},width:[1,0,30,1]},layout:{hierarchical:{enabled:!1,levelSeparation:[150,20,500,5],nodeSpacing:[100,20,500,5],treeSpacing:[200,20,500,5],blockShifting:!0,edgeMinimization:!0,parentCentralization:!0,direction:["UD","DU","LR","RL"],sortMethod:["hubsize","directed"]}},interaction:{dragNodes:!0,dragView:!0,hideEdgesOnDrag:!1,hideNodesOnDrag:!1,hover:!1,keyboard:{enabled:!1,speed:{x:[10,0,40,1],y:[10,0,40,1],zoom:[.02,0,.1,.005]},bindToWindow:!0},multiselect:!1,navigationButtons:!1,selectable:!0,selectConnectedEdges:!0,hoverConnectedEdges:!0,tooltipDelay:[300,0,1e3,25],zoomView:!0},manipulation:{enabled:!1,initiallyActive:!1},physics:{enabled:!0,barnesHut:{gravitationalConstant:[-2e3,-3e4,0,50],centralGravity:[.3,0,10,.05],springLength:[95,0,500,5],springConstant:[.04,0,1.2,.005],damping:[.09,0,1,.01],avoidOverlap:[0,0,1,.01]},forceAtlas2Based:{gravitationalConstant:[-50,-500,0,1],centralGravity:[.01,0,1,.005],springLength:[95,0,500,5],springConstant:[.08,0,1.2,.005],damping:[.4,0,1,.01],avoidOverlap:[0,0,1,.01]},repulsion:{centralGravity:[.2,0,10,.05],springLength:[200,0,500,5],springConstant:[.05,0,1.2,.005],nodeDistance:[100,0,500,5],damping:[.09,0,1,.01]},hierarchicalRepulsion:{centralGravity:[.2,0,10,.05],springLength:[100,0,500,5],springConstant:[.01,0,1.2,.005],nodeDistance:[120,0,500,5],damping:[.09,0,1,.01]},maxVelocity:[50,0,150,1],minVelocity:[.1,.01,.5,.01],solver:["barnesHut","forceAtlas2Based","repulsion","hierarchicalRepulsion"],timestep:[.5,.01,1,.01]}}},function(e,t,n){var r=n(2);r.extend(t,n(159)),r.extend(t,n(177)),r.extend(t,n(181))},function(e,t,n){n(49),n(60),e.exports=n(136)},function(e,t,n){var r=n(126),i=n(127),o=n(31),a=n(25);e.exports=n(79)(Array,"Array",(function(e,t){this._t=a(e),this._i=0,this._k=t}),(function(){var e=this._t,t=this._k,n=this._i++;return!e||n>=e.length?(this._t=void 0,i(1)):i(0,"keys"==t?n:"values"==t?e[n]:[n,e[n]])}),"values"),o.Arguments=o.Array,r("keys"),r("values"),r("entries")},function(e,t){e.exports=function(){}},function(e,t){e.exports=function(e,t){return{value:t,done:!!e}}},function(e,t){e.exports=function(e){if("function"!=typeof e)throw TypeError(e+" is not a function!");return e}},function(e,t,n){var r=n(54),i=n(39),o=n(59),a={};n(26)(a,n(13)("iterator"),(function(){return this})),e.exports=function(e,t,n){e.prototype=r(a,{next:i(1,n)}),o(e,t+" Iterator")}},function(e,t,n){var r=n(20),i=n(27),o=n(33);e.exports=n(21)?Object.defineProperties:function(e,t){i(e);for(var n,a=o(t),s=a.length,l=0;s>l;)r.f(e,n=a[l++],t[n]);return e}},function(e,t,n){var r=n(25),i=n(132),o=n(133);e.exports=function(e){return function(t,n,a){var s,l=r(t),u=i(l.length),c=o(a,u);if(e&&n!=n){for(;u>c;)if((s=l[c++])!=s)return!0}else for(;u>c;c++)if((e||c in l)&&l[c]===n)return e||c||0;return!e&&-1}}},function(e,t,n){var r=n(55),i=Math.min;e.exports=function(e){return e>0?i(r(e),9007199254740991):0}},function(e,t,n){var r=n(55),i=Math.max,o=Math.min;e.exports=function(e,t){return(e=r(e))<0?i(e+t,0):o(e,t)}},function(e,t,n){var r=n(18).document;e.exports=r&&r.documentElement},function(e,t,n){var r=n(55),i=n(51);e.exports=function(e){return function(t,n){var o,a,s=String(i(t)),l=r(n),u=s.length;return l<0||l>=u?e?"":void 0:(o=s.charCodeAt(l))<55296||o>56319||l+1===u||(a=s.charCodeAt(l+1))<56320||a>57343?e?s.charAt(l):o:e?s.slice(l,l+2):a-56320+(o-55296<<10)+65536}}},function(e,t,n){var r=n(27),i=n(137);e.exports=n(7).getIterator=function(e){var t=i(e);if("function"!=typeof t)throw TypeError(e+" is not iterable!");return r(t.call(e))}},function(e,t,n){var r=n(86),i=n(13)("iterator"),o=n(31);e.exports=n(7).getIteratorMethod=function(e){if(null!=e)return e[i]||e["@@iterator"]||o[r(e)]}},function(e,t,n){n(139);var r=n(7).Object;e.exports=function(e,t){return r.create(e,t)}},function(e,t,n){var r=n(17);r(r.S,"Object",{create:n(54)})},function(e,t,n){n(141),e.exports=n(7).Object.keys},function(e,t,n){var r=n(41),i=n(33);n(87)("keys",(function(){return function(e){return i(r(e))}}))},function(e,t,n){e.exports={default:n(143),__esModule:!0}},function(e,t,n){n(60),n(49),e.exports=n(61).f("iterator")},function(e,t,n){e.exports={default:n(145),__esModule:!0}},function(e,t,n){n(146),n(151),n(152),n(153),e.exports=n(7).Symbol},function(e,t,n){var r=n(18),i=n(22),o=n(21),a=n(17),s=n(83),l=n(147).KEY,u=n(28),c=n(57),d=n(59),h=n(40),f=n(13),p=n(61),g=n(62),v=n(148),m=n(149),y=n(27),b=n(25),C=n(53),A=n(39),_=n(54),I=n(150),x=n(89),w=n(20),M=n(33),S=x.f,T=w.f,E=I.f,O=r.Symbol,k=r.JSON,L=k&&k.stringify,D=f("_hidden"),P=f("toPrimitive"),N={}.propertyIsEnumerable,j=c("symbol-registry"),R=c("symbols"),z=c("op-symbols"),F=Object.prototype,B="function"==typeof O,H=r.QObject,U=!H||!H.prototype||!H.prototype.findChild,V=o&&u((function(){return 7!=_(T({},"a",{get:function(){return T(this,"a",{value:7}).a}})).a}))?function(e,t,n){var r=S(F,t);r&&delete F[t],T(e,t,n),r&&e!==F&&T(F,t,r)}:T,Y=function(e){var t=R[e]=_(O.prototype);return t._k=e,t},W=B&&"symbol"==typeof O.iterator?function(e){return"symbol"==typeof e}:function(e){return e instanceof O},G=function(e,t,n){return e===F&&G(z,t,n),y(e),t=C(t,!0),y(n),i(R,t)?(n.enumerable?(i(e,D)&&e[D][t]&&(e[D][t]=!1),n=_(n,{enumerable:A(0,!1)})):(i(e,D)||T(e,D,A(1,{})),e[D][t]=!0),V(e,t,n)):T(e,t,n)},X=function(e,t){y(e);for(var n,r=v(t=b(t)),i=0,o=r.length;o>i;)G(e,n=r[i++],t[n]);return e},q=function(e){var t=N.call(this,e=C(e,!0));return!(this===F&&i(R,e)&&!i(z,e))&&(!(t||!i(this,e)||!i(R,e)||i(this,D)&&this[D][e])||t)},Z=function(e,t){if(e=b(e),t=C(t,!0),e!==F||!i(R,t)||i(z,t)){var n=S(e,t);return!n||!i(R,t)||i(e,D)&&e[D][t]||(n.enumerable=!0),n}},Q=function(e){for(var t,n=E(b(e)),r=[],o=0;n.length>o;)i(R,t=n[o++])||t==D||t==l||r.push(t);return r},K=function(e){for(var t,n=e===F,r=E(n?z:b(e)),o=[],a=0;r.length>a;)!i(R,t=r[a++])||n&&!i(F,t)||o.push(R[t]);return o};B||(s((O=function(){if(this instanceof O)throw TypeError("Symbol is not a constructor!");var e=h(arguments.length>0?arguments[0]:void 0),t=function(n){this===F&&t.call(z,n),i(this,D)&&i(this[D],e)&&(this[D][e]=!1),V(this,e,A(1,n))};return o&&U&&V(F,e,{configurable:!0,set:t}),Y(e)}).prototype,"toString",(function(){return this._k})),x.f=Z,w.f=G,n(88).f=I.f=Q,n(42).f=q,n(63).f=K,o&&!n(52)&&s(F,"propertyIsEnumerable",q,!0),p.f=function(e){return Y(f(e))}),a(a.G+a.W+a.F*!B,{Symbol:O});for(var J="hasInstance,isConcatSpreadable,iterator,match,replace,search,species,split,toPrimitive,toStringTag,unscopables".split(","),$=0;J.length>$;)f(J[$++]);for(var ee=M(f.store),te=0;ee.length>te;)g(ee[te++]);a(a.S+a.F*!B,"Symbol",{for:function(e){return i(j,e+="")?j[e]:j[e]=O(e)},keyFor:function(e){if(!W(e))throw TypeError(e+" is not a symbol!");for(var t in j)if(j[t]===e)return t},useSetter:function(){U=!0},useSimple:function(){U=!1}}),a(a.S+a.F*!B,"Object",{create:function(e,t){return void 0===t?_(e):X(_(e),t)},defineProperty:G,defineProperties:X,getOwnPropertyDescriptor:Z,getOwnPropertyNames:Q,getOwnPropertySymbols:K}),k&&a(a.S+a.F*(!B||u((function(){var e=O();return"[null]"!=L([e])||"{}"!=L({a:e})||"{}"!=L(Object(e))}))),"JSON",{stringify:function(e){if(void 0!==e&&!W(e)){for(var t,n,r=[e],i=1;arguments.length>i;)r.push(arguments[i++]);return"function"==typeof(t=r[1])&&(n=t),!n&&m(t)||(t=function(e,t){if(n&&(t=n.call(this,e,t)),!W(t))return t}),r[1]=t,L.apply(k,r)}}}),O.prototype[P]||n(26)(O.prototype,P,O.prototype.valueOf),d(O,"Symbol"),d(Math,"Math",!0),d(r.JSON,"JSON",!0)},function(e,t,n){var r=n(40)("meta"),i=n(32),o=n(22),a=n(20).f,s=0,l=Object.isExtensible||function(){return!0},u=!n(28)((function(){return l(Object.preventExtensions({}))})),c=function(e){a(e,r,{value:{i:"O"+ ++s,w:{}}})},d=e.exports={KEY:r,NEED:!1,fastKey:function(e,t){if(!i(e))return"symbol"==typeof e?e:("string"==typeof e?"S":"P")+e;if(!o(e,r)){if(!l(e))return"F";if(!t)return"E";c(e)}return e[r].i},getWeak:function(e,t){if(!o(e,r)){if(!l(e))return!0;if(!t)return!1;c(e)}return e[r].w},onFreeze:function(e){return u&&d.NEED&&l(e)&&!o(e,r)&&c(e),e}}},function(e,t,n){var r=n(33),i=n(63),o=n(42);e.exports=function(e){var t=r(e),n=i.f;if(n)for(var a,s=n(e),l=o.f,u=0;s.length>u;)l.call(e,a=s[u++])&&t.push(a);return t}},function(e,t,n){var r=n(50);e.exports=Array.isArray||function(e){return"Array"==r(e)}},function(e,t,n){var r=n(25),i=n(88).f,o={}.toString,a="object"==typeof window&&window&&Object.getOwnPropertyNames?Object.getOwnPropertyNames(window):[];e.exports.f=function(e){return a&&"[object Window]"==o.call(e)?function(e){try{return i(e)}catch(e){return a.slice()}}(e):i(r(e))}},function(e,t){},function(e,t,n){n(62)("asyncIterator")},function(e,t,n){n(62)("observable")},function(e,t,n){(function(e){e.exports=function(){var t,n;function r(){return t.apply(null,arguments)}function i(e){return e instanceof Array||"[object Array]"===Object.prototype.toString.call(e)}function o(e){return null!=e&&"[object Object]"===Object.prototype.toString.call(e)}function a(e){return void 0===e}function s(e){return"number"==typeof e||"[object Number]"===Object.prototype.toString.call(e)}function l(e){return e instanceof Date||"[object Date]"===Object.prototype.toString.call(e)}function u(e,t){var n,r=[];for(n=0;n<e.length;++n)r.push(t(e[n],n));return r}function c(e,t){return Object.prototype.hasOwnProperty.call(e,t)}function d(e,t){for(var n in t)c(t,n)&&(e[n]=t[n]);return c(t,"toString")&&(e.toString=t.toString),c(t,"valueOf")&&(e.valueOf=t.valueOf),e}function h(e,t,n,r){return _t(e,t,n,r,!0).utc()}function f(e){return null==e._pf&&(e._pf={empty:!1,unusedTokens:[],unusedInput:[],overflow:-2,charsLeftOver:0,nullInput:!1,invalidMonth:null,invalidFormat:!1,userInvalidated:!1,iso:!1,parsedDateParts:[],meridiem:null,rfc2822:!1,weekdayMismatch:!1}),e._pf}function p(e){if(null==e._isValid){var t=f(e),r=n.call(t.parsedDateParts,(function(e){return null!=e})),i=!isNaN(e._d.getTime())&&t.overflow<0&&!t.empty&&!t.invalidMonth&&!t.invalidWeekday&&!t.weekdayMismatch&&!t.nullInput&&!t.invalidFormat&&!t.userInvalidated&&(!t.meridiem||t.meridiem&&r);if(e._strict&&(i=i&&0===t.charsLeftOver&&0===t.unusedTokens.length&&void 0===t.bigHour),null!=Object.isFrozen&&Object.isFrozen(e))return i;e._isValid=i}return e._isValid}function g(e){var t=h(NaN);return null!=e?d(f(t),e):f(t).userInvalidated=!0,t}n=Array.prototype.some?Array.prototype.some:function(e){for(var t=Object(this),n=t.length>>>0,r=0;r<n;r++)if(r in t&&e.call(this,t[r],r,t))return!0;return!1};var v=r.momentProperties=[];function m(e,t){var n,r,i;if(a(t._isAMomentObject)||(e._isAMomentObject=t._isAMomentObject),a(t._i)||(e._i=t._i),a(t._f)||(e._f=t._f),a(t._l)||(e._l=t._l),a(t._strict)||(e._strict=t._strict),a(t._tzm)||(e._tzm=t._tzm),a(t._isUTC)||(e._isUTC=t._isUTC),a(t._offset)||(e._offset=t._offset),a(t._pf)||(e._pf=f(t)),a(t._locale)||(e._locale=t._locale),v.length>0)for(n=0;n<v.length;n++)a(i=t[r=v[n]])||(e[r]=i);return e}var y=!1;function b(e){m(this,e),this._d=new Date(null!=e._d?e._d.getTime():NaN),this.isValid()||(this._d=new Date(NaN)),!1===y&&(y=!0,r.updateOffset(this),y=!1)}function C(e){return e instanceof b||null!=e&&null!=e._isAMomentObject}function A(e){return e<0?Math.ceil(e)||0:Math.floor(e)}function _(e){var t=+e,n=0;return 0!==t&&isFinite(t)&&(n=A(t)),n}function I(e,t,n){var r,i=Math.min(e.length,t.length),o=Math.abs(e.length-t.length),a=0;for(r=0;r<i;r++)(n&&e[r]!==t[r]||!n&&_(e[r])!==_(t[r]))&&a++;return a+o}function x(e){!1===r.suppressDeprecationWarnings&&"undefined"!=typeof console&&console.warn&&console.warn("Deprecation warning: "+e)}function w(e,t){var n=!0;return d((function(){if(null!=r.deprecationHandler&&r.deprecationHandler(null,e),n){for(var i,o=[],a=0;a<arguments.length;a++){if(i="","object"==typeof arguments[a]){for(var s in i+="\n["+a+"] ",arguments[0])i+=s+": "+arguments[0][s]+", ";i=i.slice(0,-2)}else i=arguments[a];o.push(i)}x(e+"\nArguments: "+Array.prototype.slice.call(o).join("")+"\n"+(new Error).stack),n=!1}return t.apply(this,arguments)}),t)}var M,S={};function T(e,t){null!=r.deprecationHandler&&r.deprecationHandler(e,t),S[e]||(x(t),S[e]=!0)}function E(e){return e instanceof Function||"[object Function]"===Object.prototype.toString.call(e)}function O(e,t){var n,r=d({},e);for(n in t)c(t,n)&&(o(e[n])&&o(t[n])?(r[n]={},d(r[n],e[n]),d(r[n],t[n])):null!=t[n]?r[n]=t[n]:delete r[n]);for(n in e)c(e,n)&&!c(t,n)&&o(e[n])&&(r[n]=d({},r[n]));return r}function k(e){null!=e&&this.set(e)}r.suppressDeprecationWarnings=!1,r.deprecationHandler=null,M=Object.keys?Object.keys:function(e){var t,n=[];for(t in e)c(e,t)&&n.push(t);return n};var L={};function D(e,t){var n=e.toLowerCase();L[n]=L[n+"s"]=L[t]=e}function P(e){return"string"==typeof e?L[e]||L[e.toLowerCase()]:void 0}function N(e){var t,n,r={};for(n in e)c(e,n)&&(t=P(n))&&(r[t]=e[n]);return r}var j={};function R(e,t){j[e]=t}function z(e,t,n){var r=""+Math.abs(e),i=t-r.length;return(e>=0?n?"+":"":"-")+Math.pow(10,Math.max(0,i)).toString().substr(1)+r}var F=/(\[[^\[]*\])|(\\)?([Hh]mm(ss)?|Mo|MM?M?M?|Do|DDDo|DD?D?D?|ddd?d?|do?|w[o|w]?|W[o|W]?|Qo?|YYYYYY|YYYYY|YYYY|YY|gg(ggg?)?|GG(GGG?)?|e|E|a|A|hh?|HH?|kk?|mm?|ss?|S{1,9}|x|X|zz?|ZZ?|.)/g,B=/(\[[^\[]*\])|(\\)?(LTS|LT|LL?L?L?|l{1,4})/g,H={},U={};function V(e,t,n,r){var i=r;"string"==typeof r&&(i=function(){return this[r]()}),e&&(U[e]=i),t&&(U[t[0]]=function(){return z(i.apply(this,arguments),t[1],t[2])}),n&&(U[n]=function(){return this.localeData().ordinal(i.apply(this,arguments),e)})}function Y(e,t){return e.isValid()?(t=W(t,e.localeData()),H[t]=H[t]||function(e){var t,n,r,i=e.match(F);for(t=0,n=i.length;t<n;t++)U[i[t]]?i[t]=U[i[t]]:i[t]=(r=i[t]).match(/\[[\s\S]/)?r.replace(/^\[|\]$/g,""):r.replace(/\\/g,"");return function(t){var r,o="";for(r=0;r<n;r++)o+=E(i[r])?i[r].call(t,e):i[r];return o}}(t),H[t](e)):e.localeData().invalidDate()}function W(e,t){var n=5;function r(e){return t.longDateFormat(e)||e}for(B.lastIndex=0;n>=0&&B.test(e);)e=e.replace(B,r),B.lastIndex=0,n-=1;return e}var G=/\d/,X=/\d\d/,q=/\d{3}/,Z=/\d{4}/,Q=/[+-]?\d{6}/,K=/\d\d?/,J=/\d\d\d\d?/,$=/\d\d\d\d\d\d?/,ee=/\d{1,3}/,te=/\d{1,4}/,ne=/[+-]?\d{1,6}/,re=/\d+/,ie=/[+-]?\d+/,oe=/Z|[+-]\d\d:?\d\d/gi,ae=/Z|[+-]\d\d(?::?\d\d)?/gi,se=/[0-9]*['a-z\u00A0-\u05FF\u0700-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]+|[\u0600-\u06FF\/]+(\s*?[\u0600-\u06FF]+){1,2}/i,le={};function ue(e,t,n){le[e]=E(t)?t:function(e,r){return e&&n?n:t}}function ce(e,t){return c(le,e)?le[e](t._strict,t._locale):new RegExp(de(e.replace("\\","").replace(/\\(\[)|\\(\])|\[([^\]\[]*)\]|\\(.)/g,(function(e,t,n,r,i){return t||n||r||i}))))}function de(e){return e.replace(/[-\/\\^$*+?.()|[\]{}]/g,"\\$&")}var he={};function fe(e,t){var n,r=t;for("string"==typeof e&&(e=[e]),s(t)&&(r=function(e,n){n[t]=_(e)}),n=0;n<e.length;n++)he[e[n]]=r}function pe(e,t){fe(e,(function(e,n,r,i){r._w=r._w||{},t(e,r._w,r,i)}))}function ge(e,t,n){null!=t&&c(he,e)&&he[e](t,n._a,n,e)}function ve(e){return me(e)?366:365}function me(e){return e%4==0&&e%100!=0||e%400==0}V("Y",0,0,(function(){var e=this.year();return e<=9999?""+e:"+"+e})),V(0,["YY",2],0,(function(){return this.year()%100})),V(0,["YYYY",4],0,"year"),V(0,["YYYYY",5],0,"year"),V(0,["YYYYYY",6,!0],0,"year"),D("year","y"),R("year",1),ue("Y",ie),ue("YY",K,X),ue("YYYY",te,Z),ue("YYYYY",ne,Q),ue("YYYYYY",ne,Q),fe(["YYYYY","YYYYYY"],0),fe("YYYY",(function(e,t){t[0]=2===e.length?r.parseTwoDigitYear(e):_(e)})),fe("YY",(function(e,t){t[0]=r.parseTwoDigitYear(e)})),fe("Y",(function(e,t){t[0]=parseInt(e,10)})),r.parseTwoDigitYear=function(e){return _(e)+(_(e)>68?1900:2e3)};var ye,be=Ce("FullYear",!0);function Ce(e,t){return function(n){return null!=n?(_e(this,e,n),r.updateOffset(this,t),this):Ae(this,e)}}function Ae(e,t){return e.isValid()?e._d["get"+(e._isUTC?"UTC":"")+t]():NaN}function _e(e,t,n){e.isValid()&&!isNaN(n)&&("FullYear"===t&&me(e.year())?e._d["set"+(e._isUTC?"UTC":"")+t](n,e.month(),Ie(n,e.month())):e._d["set"+(e._isUTC?"UTC":"")+t](n))}function Ie(e,t){if(isNaN(e)||isNaN(t))return NaN;var n,r=(t%(n=12)+n)%n;return e+=(t-r)/12,1===r?me(e)?29:28:31-r%7%2}ye=Array.prototype.indexOf?Array.prototype.indexOf:function(e){var t;for(t=0;t<this.length;++t)if(this[t]===e)return t;return-1},V("M",["MM",2],"Mo",(function(){return this.month()+1})),V("MMM",0,0,(function(e){return this.localeData().monthsShort(this,e)})),V("MMMM",0,0,(function(e){return this.localeData().months(this,e)})),D("month","M"),R("month",8),ue("M",K),ue("MM",K,X),ue("MMM",(function(e,t){return t.monthsShortRegex(e)})),ue("MMMM",(function(e,t){return t.monthsRegex(e)})),fe(["M","MM"],(function(e,t){t[1]=_(e)-1})),fe(["MMM","MMMM"],(function(e,t,n,r){var i=n._locale.monthsParse(e,r,n._strict);null!=i?t[1]=i:f(n).invalidMonth=e}));var xe=/D[oD]?(\[[^\[\]]*\]|\s)+MMMM?/,we="January_February_March_April_May_June_July_August_September_October_November_December".split("_"),Me="Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_");function Se(e,t,n){var r,i,o,a=e.toLocaleLowerCase();if(!this._monthsParse)for(this._monthsParse=[],this._longMonthsParse=[],this._shortMonthsParse=[],r=0;r<12;++r)o=h([2e3,r]),this._shortMonthsParse[r]=this.monthsShort(o,"").toLocaleLowerCase(),this._longMonthsParse[r]=this.months(o,"").toLocaleLowerCase();return n?"MMM"===t?-1!==(i=ye.call(this._shortMonthsParse,a))?i:null:-1!==(i=ye.call(this._longMonthsParse,a))?i:null:"MMM"===t?-1!==(i=ye.call(this._shortMonthsParse,a))||-1!==(i=ye.call(this._longMonthsParse,a))?i:null:-1!==(i=ye.call(this._longMonthsParse,a))||-1!==(i=ye.call(this._shortMonthsParse,a))?i:null}function Te(e,t){var n;if(!e.isValid())return e;if("string"==typeof t)if(/^\d+$/.test(t))t=_(t);else if(!s(t=e.localeData().monthsParse(t)))return e;return n=Math.min(e.date(),Ie(e.year(),t)),e._d["set"+(e._isUTC?"UTC":"")+"Month"](t,n),e}function Ee(e){return null!=e?(Te(this,e),r.updateOffset(this,!0),this):Ae(this,"Month")}var Oe=se,ke=se;function Le(){function e(e,t){return t.length-e.length}var t,n,r=[],i=[],o=[];for(t=0;t<12;t++)n=h([2e3,t]),r.push(this.monthsShort(n,"")),i.push(this.months(n,"")),o.push(this.months(n,"")),o.push(this.monthsShort(n,""));for(r.sort(e),i.sort(e),o.sort(e),t=0;t<12;t++)r[t]=de(r[t]),i[t]=de(i[t]);for(t=0;t<24;t++)o[t]=de(o[t]);this._monthsRegex=new RegExp("^("+o.join("|")+")","i"),this._monthsShortRegex=this._monthsRegex,this._monthsStrictRegex=new RegExp("^("+i.join("|")+")","i"),this._monthsShortStrictRegex=new RegExp("^("+r.join("|")+")","i")}function De(e,t,n,r,i,o,a){var s=new Date(e,t,n,r,i,o,a);return e<100&&e>=0&&isFinite(s.getFullYear())&&s.setFullYear(e),s}function Pe(e){var t=new Date(Date.UTC.apply(null,arguments));return e<100&&e>=0&&isFinite(t.getUTCFullYear())&&t.setUTCFullYear(e),t}function Ne(e,t,n){var r=7+t-n;return-(7+Pe(e,0,r).getUTCDay()-t)%7+r-1}function je(e,t,n,r,i){var o,a,s=1+7*(t-1)+(7+n-r)%7+Ne(e,r,i);return s<=0?a=ve(o=e-1)+s:s>ve(e)?(o=e+1,a=s-ve(e)):(o=e,a=s),{year:o,dayOfYear:a}}function Re(e,t,n){var r,i,o=Ne(e.year(),t,n),a=Math.floor((e.dayOfYear()-o-1)/7)+1;return a<1?r=a+ze(i=e.year()-1,t,n):a>ze(e.year(),t,n)?(r=a-ze(e.year(),t,n),i=e.year()+1):(i=e.year(),r=a),{week:r,year:i}}function ze(e,t,n){var r=Ne(e,t,n),i=Ne(e+1,t,n);return(ve(e)-r+i)/7}V("w",["ww",2],"wo","week"),V("W",["WW",2],"Wo","isoWeek"),D("week","w"),D("isoWeek","W"),R("week",5),R("isoWeek",5),ue("w",K),ue("ww",K,X),ue("W",K),ue("WW",K,X),pe(["w","ww","W","WW"],(function(e,t,n,r){t[r.substr(0,1)]=_(e)})),V("d",0,"do","day"),V("dd",0,0,(function(e){return this.localeData().weekdaysMin(this,e)})),V("ddd",0,0,(function(e){return this.localeData().weekdaysShort(this,e)})),V("dddd",0,0,(function(e){return this.localeData().weekdays(this,e)})),V("e",0,0,"weekday"),V("E",0,0,"isoWeekday"),D("day","d"),D("weekday","e"),D("isoWeekday","E"),R("day",11),R("weekday",11),R("isoWeekday",11),ue("d",K),ue("e",K),ue("E",K),ue("dd",(function(e,t){return t.weekdaysMinRegex(e)})),ue("ddd",(function(e,t){return t.weekdaysShortRegex(e)})),ue("dddd",(function(e,t){return t.weekdaysRegex(e)})),pe(["dd","ddd","dddd"],(function(e,t,n,r){var i=n._locale.weekdaysParse(e,r,n._strict);null!=i?t.d=i:f(n).invalidWeekday=e})),pe(["d","e","E"],(function(e,t,n,r){t[r]=_(e)}));var Fe="Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),Be="Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),He="Su_Mo_Tu_We_Th_Fr_Sa".split("_");function Ue(e,t,n){var r,i,o,a=e.toLocaleLowerCase();if(!this._weekdaysParse)for(this._weekdaysParse=[],this._shortWeekdaysParse=[],this._minWeekdaysParse=[],r=0;r<7;++r)o=h([2e3,1]).day(r),this._minWeekdaysParse[r]=this.weekdaysMin(o,"").toLocaleLowerCase(),this._shortWeekdaysParse[r]=this.weekdaysShort(o,"").toLocaleLowerCase(),this._weekdaysParse[r]=this.weekdays(o,"").toLocaleLowerCase();return n?"dddd"===t?-1!==(i=ye.call(this._weekdaysParse,a))?i:null:"ddd"===t?-1!==(i=ye.call(this._shortWeekdaysParse,a))?i:null:-1!==(i=ye.call(this._minWeekdaysParse,a))?i:null:"dddd"===t?-1!==(i=ye.call(this._weekdaysParse,a))||-1!==(i=ye.call(this._shortWeekdaysParse,a))||-1!==(i=ye.call(this._minWeekdaysParse,a))?i:null:"ddd"===t?-1!==(i=ye.call(this._shortWeekdaysParse,a))||-1!==(i=ye.call(this._weekdaysParse,a))||-1!==(i=ye.call(this._minWeekdaysParse,a))?i:null:-1!==(i=ye.call(this._minWeekdaysParse,a))||-1!==(i=ye.call(this._weekdaysParse,a))||-1!==(i=ye.call(this._shortWeekdaysParse,a))?i:null}var Ve=se,Ye=se,We=se;function Ge(){function e(e,t){return t.length-e.length}var t,n,r,i,o,a=[],s=[],l=[],u=[];for(t=0;t<7;t++)n=h([2e3,1]).day(t),r=this.weekdaysMin(n,""),i=this.weekdaysShort(n,""),o=this.weekdays(n,""),a.push(r),s.push(i),l.push(o),u.push(r),u.push(i),u.push(o);for(a.sort(e),s.sort(e),l.sort(e),u.sort(e),t=0;t<7;t++)s[t]=de(s[t]),l[t]=de(l[t]),u[t]=de(u[t]);this._weekdaysRegex=new RegExp("^("+u.join("|")+")","i"),this._weekdaysShortRegex=this._weekdaysRegex,this._weekdaysMinRegex=this._weekdaysRegex,this._weekdaysStrictRegex=new RegExp("^("+l.join("|")+")","i"),this._weekdaysShortStrictRegex=new RegExp("^("+s.join("|")+")","i"),this._weekdaysMinStrictRegex=new RegExp("^("+a.join("|")+")","i")}function Xe(){return this.hours()%12||12}function qe(e,t){V(e,0,0,(function(){return this.localeData().meridiem(this.hours(),this.minutes(),t)}))}function Ze(e,t){return t._meridiemParse}V("H",["HH",2],0,"hour"),V("h",["hh",2],0,Xe),V("k",["kk",2],0,(function(){return this.hours()||24})),V("hmm",0,0,(function(){return""+Xe.apply(this)+z(this.minutes(),2)})),V("hmmss",0,0,(function(){return""+Xe.apply(this)+z(this.minutes(),2)+z(this.seconds(),2)})),V("Hmm",0,0,(function(){return""+this.hours()+z(this.minutes(),2)})),V("Hmmss",0,0,(function(){return""+this.hours()+z(this.minutes(),2)+z(this.seconds(),2)})),qe("a",!0),qe("A",!1),D("hour","h"),R("hour",13),ue("a",Ze),ue("A",Ze),ue("H",K),ue("h",K),ue("k",K),ue("HH",K,X),ue("hh",K,X),ue("kk",K,X),ue("hmm",J),ue("hmmss",$),ue("Hmm",J),ue("Hmmss",$),fe(["H","HH"],3),fe(["k","kk"],(function(e,t,n){var r=_(e);t[3]=24===r?0:r})),fe(["a","A"],(function(e,t,n){n._isPm=n._locale.isPM(e),n._meridiem=e})),fe(["h","hh"],(function(e,t,n){t[3]=_(e),f(n).bigHour=!0})),fe("hmm",(function(e,t,n){var r=e.length-2;t[3]=_(e.substr(0,r)),t[4]=_(e.substr(r)),f(n).bigHour=!0})),fe("hmmss",(function(e,t,n){var r=e.length-4,i=e.length-2;t[3]=_(e.substr(0,r)),t[4]=_(e.substr(r,2)),t[5]=_(e.substr(i)),f(n).bigHour=!0})),fe("Hmm",(function(e,t,n){var r=e.length-2;t[3]=_(e.substr(0,r)),t[4]=_(e.substr(r))})),fe("Hmmss",(function(e,t,n){var r=e.length-4,i=e.length-2;t[3]=_(e.substr(0,r)),t[4]=_(e.substr(r,2)),t[5]=_(e.substr(i))}));var Qe,Ke=Ce("Hours",!0),Je={calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},longDateFormat:{LTS:"h:mm:ss A",LT:"h:mm A",L:"MM/DD/YYYY",LL:"MMMM D, YYYY",LLL:"MMMM D, YYYY h:mm A",LLLL:"dddd, MMMM D, YYYY h:mm A"},invalidDate:"Invalid date",ordinal:"%d",dayOfMonthOrdinalParse:/\d{1,2}/,relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},months:we,monthsShort:Me,week:{dow:0,doy:6},weekdays:Fe,weekdaysMin:He,weekdaysShort:Be,meridiemParse:/[ap]\.?m?\.?/i},$e={},et={};function tt(e){return e?e.toLowerCase().replace("_","-"):e}function nt(t){var n=null;if(!$e[t]&&void 0!==e&&e&&e.exports)try{n=Qe._abbr,function(){var e=new Error('Cannot find module "./locale"');throw e.code="MODULE_NOT_FOUND",e}(),rt(n)}catch(e){}return $e[t]}function rt(e,t){var n;return e&&(n=a(t)?ot(e):it(e,t))&&(Qe=n),Qe._abbr}function it(e,t){if(null!==t){var n=Je;if(t.abbr=e,null!=$e[e])T("defineLocaleOverride","use moment.updateLocale(localeName, config) to change an existing locale. moment.defineLocale(localeName, config) should only be used for creating a new locale See http://momentjs.com/guides/#/warnings/define-locale/ for more info."),n=$e[e]._config;else if(null!=t.parentLocale){if(null==$e[t.parentLocale])return et[t.parentLocale]||(et[t.parentLocale]=[]),et[t.parentLocale].push({name:e,config:t}),null;n=$e[t.parentLocale]._config}return $e[e]=new k(O(n,t)),et[e]&&et[e].forEach((function(e){it(e.name,e.config)})),rt(e),$e[e]}return delete $e[e],null}function ot(e){var t;if(e&&e._locale&&e._locale._abbr&&(e=e._locale._abbr),!e)return Qe;if(!i(e)){if(t=nt(e))return t;e=[e]}return function(e){for(var t,n,r,i,o=0;o<e.length;){for(t=(i=tt(e[o]).split("-")).length,n=(n=tt(e[o+1]))?n.split("-"):null;t>0;){if(r=nt(i.slice(0,t).join("-")))return r;if(n&&n.length>=t&&I(i,n,!0)>=t-1)break;t--}o++}return null}(e)}function at(e){var t,n=e._a;return n&&-2===f(e).overflow&&(t=n[1]<0||n[1]>11?1:n[2]<1||n[2]>Ie(n[0],n[1])?2:n[3]<0||n[3]>24||24===n[3]&&(0!==n[4]||0!==n[5]||0!==n[6])?3:n[4]<0||n[4]>59?4:n[5]<0||n[5]>59?5:n[6]<0||n[6]>999?6:-1,f(e)._overflowDayOfYear&&(t<0||t>2)&&(t=2),f(e)._overflowWeeks&&-1===t&&(t=7),f(e)._overflowWeekday&&-1===t&&(t=8),f(e).overflow=t),e}function st(e,t,n){return null!=e?e:null!=t?t:n}function lt(e){var t,n,i,o,a=[];if(!e._d){for(i=function(e){var t=new Date(r.now());return e._useUTC?[t.getUTCFullYear(),t.getUTCMonth(),t.getUTCDate()]:[t.getFullYear(),t.getMonth(),t.getDate()]}(e),e._w&&null==e._a[2]&&null==e._a[1]&&function(e){var t,n,r,i,o,a,s,l;if(null!=(t=e._w).GG||null!=t.W||null!=t.E)o=1,a=4,n=st(t.GG,e._a[0],Re(It(),1,4).year),r=st(t.W,1),((i=st(t.E,1))<1||i>7)&&(l=!0);else{o=e._locale._week.dow,a=e._locale._week.doy;var u=Re(It(),o,a);n=st(t.gg,e._a[0],u.year),r=st(t.w,u.week),null!=t.d?((i=t.d)<0||i>6)&&(l=!0):null!=t.e?(i=t.e+o,(t.e<0||t.e>6)&&(l=!0)):i=o}r<1||r>ze(n,o,a)?f(e)._overflowWeeks=!0:null!=l?f(e)._overflowWeekday=!0:(s=je(n,r,i,o,a),e._a[0]=s.year,e._dayOfYear=s.dayOfYear)}(e),null!=e._dayOfYear&&(o=st(e._a[0],i[0]),(e._dayOfYear>ve(o)||0===e._dayOfYear)&&(f(e)._overflowDayOfYear=!0),n=Pe(o,0,e._dayOfYear),e._a[1]=n.getUTCMonth(),e._a[2]=n.getUTCDate()),t=0;t<3&&null==e._a[t];++t)e._a[t]=a[t]=i[t];for(;t<7;t++)e._a[t]=a[t]=null==e._a[t]?2===t?1:0:e._a[t];24===e._a[3]&&0===e._a[4]&&0===e._a[5]&&0===e._a[6]&&(e._nextDay=!0,e._a[3]=0),e._d=(e._useUTC?Pe:De).apply(null,a),null!=e._tzm&&e._d.setUTCMinutes(e._d.getUTCMinutes()-e._tzm),e._nextDay&&(e._a[3]=24),e._w&&void 0!==e._w.d&&e._w.d!==e._d.getDay()&&(f(e).weekdayMismatch=!0)}}var ut=/^\s*((?:[+-]\d{6}|\d{4})-(?:\d\d-\d\d|W\d\d-\d|W\d\d|\d\d\d|\d\d))(?:(T| )(\d\d(?::\d\d(?::\d\d(?:[.,]\d+)?)?)?)([\+\-]\d\d(?::?\d\d)?|\s*Z)?)?$/,ct=/^\s*((?:[+-]\d{6}|\d{4})(?:\d\d\d\d|W\d\d\d|W\d\d|\d\d\d|\d\d))(?:(T| )(\d\d(?:\d\d(?:\d\d(?:[.,]\d+)?)?)?)([\+\-]\d\d(?::?\d\d)?|\s*Z)?)?$/,dt=/Z|[+-]\d\d(?::?\d\d)?/,ht=[["YYYYYY-MM-DD",/[+-]\d{6}-\d\d-\d\d/],["YYYY-MM-DD",/\d{4}-\d\d-\d\d/],["GGGG-[W]WW-E",/\d{4}-W\d\d-\d/],["GGGG-[W]WW",/\d{4}-W\d\d/,!1],["YYYY-DDD",/\d{4}-\d{3}/],["YYYY-MM",/\d{4}-\d\d/,!1],["YYYYYYMMDD",/[+-]\d{10}/],["YYYYMMDD",/\d{8}/],["GGGG[W]WWE",/\d{4}W\d{3}/],["GGGG[W]WW",/\d{4}W\d{2}/,!1],["YYYYDDD",/\d{7}/]],ft=[["HH:mm:ss.SSSS",/\d\d:\d\d:\d\d\.\d+/],["HH:mm:ss,SSSS",/\d\d:\d\d:\d\d,\d+/],["HH:mm:ss",/\d\d:\d\d:\d\d/],["HH:mm",/\d\d:\d\d/],["HHmmss.SSSS",/\d\d\d\d\d\d\.\d+/],["HHmmss,SSSS",/\d\d\d\d\d\d,\d+/],["HHmmss",/\d\d\d\d\d\d/],["HHmm",/\d\d\d\d/],["HH",/\d\d/]],pt=/^\/?Date\((\-?\d+)/i;function gt(e){var t,n,r,i,o,a,s=e._i,l=ut.exec(s)||ct.exec(s);if(l){for(f(e).iso=!0,t=0,n=ht.length;t<n;t++)if(ht[t][1].exec(l[1])){i=ht[t][0],r=!1!==ht[t][2];break}if(null==i)return void(e._isValid=!1);if(l[3]){for(t=0,n=ft.length;t<n;t++)if(ft[t][1].exec(l[3])){o=(l[2]||" ")+ft[t][0];break}if(null==o)return void(e._isValid=!1)}if(!r&&null!=o)return void(e._isValid=!1);if(l[4]){if(!dt.exec(l[4]))return void(e._isValid=!1);a="Z"}e._f=i+(o||"")+(a||""),Ct(e)}else e._isValid=!1}var vt=/^(?:(Mon|Tue|Wed|Thu|Fri|Sat|Sun),?\s)?(\d{1,2})\s(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)\s(\d{2,4})\s(\d\d):(\d\d)(?::(\d\d))?\s(?:(UT|GMT|[ECMP][SD]T)|([Zz])|([+-]\d{4}))$/;function mt(e){var t=parseInt(e,10);return t<=49?2e3+t:t<=999?1900+t:t}var yt={UT:0,GMT:0,EDT:-240,EST:-300,CDT:-300,CST:-360,MDT:-360,MST:-420,PDT:-420,PST:-480};function bt(e){var t,n,r,i,o,a,s,l=vt.exec(e._i.replace(/\([^)]*\)|[\n\t]/g," ").replace(/(\s\s+)/g," ").trim());if(l){var u=(t=l[4],n=l[3],r=l[2],i=l[5],o=l[6],a=l[7],s=[mt(t),Me.indexOf(n),parseInt(r,10),parseInt(i,10),parseInt(o,10)],a&&s.push(parseInt(a,10)),s);if(!function(e,t,n){return!e||Be.indexOf(e)===new Date(t[0],t[1],t[2]).getDay()||(f(n).weekdayMismatch=!0,n._isValid=!1,!1)}(l[1],u,e))return;e._a=u,e._tzm=function(e,t,n){if(e)return yt[e];if(t)return 0;var r=parseInt(n,10),i=r%100;return(r-i)/100*60+i}(l[8],l[9],l[10]),e._d=Pe.apply(null,e._a),e._d.setUTCMinutes(e._d.getUTCMinutes()-e._tzm),f(e).rfc2822=!0}else e._isValid=!1}function Ct(e){if(e._f!==r.ISO_8601)if(e._f!==r.RFC_2822){e._a=[],f(e).empty=!0;var t,n,i,o,a,s=""+e._i,l=s.length,u=0;for(i=W(e._f,e._locale).match(F)||[],t=0;t<i.length;t++)o=i[t],(n=(s.match(ce(o,e))||[])[0])&&((a=s.substr(0,s.indexOf(n))).length>0&&f(e).unusedInput.push(a),s=s.slice(s.indexOf(n)+n.length),u+=n.length),U[o]?(n?f(e).empty=!1:f(e).unusedTokens.push(o),ge(o,n,e)):e._strict&&!n&&f(e).unusedTokens.push(o);f(e).charsLeftOver=l-u,s.length>0&&f(e).unusedInput.push(s),e._a[3]<=12&&!0===f(e).bigHour&&e._a[3]>0&&(f(e).bigHour=void 0),f(e).parsedDateParts=e._a.slice(0),f(e).meridiem=e._meridiem,e._a[3]=function(e,t,n){var r;return null==n?t:null!=e.meridiemHour?e.meridiemHour(t,n):null!=e.isPM?((r=e.isPM(n))&&t<12&&(t+=12),r||12!==t||(t=0),t):t}(e._locale,e._a[3],e._meridiem),lt(e),at(e)}else bt(e);else gt(e)}function At(e){var t=e._i,n=e._f;return e._locale=e._locale||ot(e._l),null===t||void 0===n&&""===t?g({nullInput:!0}):("string"==typeof t&&(e._i=t=e._locale.preparse(t)),C(t)?new b(at(t)):(l(t)?e._d=t:i(n)?function(e){var t,n,r,i,o;if(0===e._f.length)return f(e).invalidFormat=!0,void(e._d=new Date(NaN));for(i=0;i<e._f.length;i++)o=0,t=m({},e),null!=e._useUTC&&(t._useUTC=e._useUTC),t._f=e._f[i],Ct(t),p(t)&&(o+=f(t).charsLeftOver,o+=10*f(t).unusedTokens.length,f(t).score=o,(null==r||o<r)&&(r=o,n=t));d(e,n||t)}(e):n?Ct(e):function(e){var t=e._i;a(t)?e._d=new Date(r.now()):l(t)?e._d=new Date(t.valueOf()):"string"==typeof t?function(e){var t=pt.exec(e._i);null===t?(gt(e),!1===e._isValid&&(delete e._isValid,bt(e),!1===e._isValid&&(delete e._isValid,r.createFromInputFallback(e)))):e._d=new Date(+t[1])}(e):i(t)?(e._a=u(t.slice(0),(function(e){return parseInt(e,10)})),lt(e)):o(t)?function(e){if(!e._d){var t=N(e._i);e._a=u([t.year,t.month,t.day||t.date,t.hour,t.minute,t.second,t.millisecond],(function(e){return e&&parseInt(e,10)})),lt(e)}}(e):s(t)?e._d=new Date(t):r.createFromInputFallback(e)}(e),p(e)||(e._d=null),e))}function _t(e,t,n,r,a){var s,l={};return!0!==n&&!1!==n||(r=n,n=void 0),(o(e)&&function(e){if(Object.getOwnPropertyNames)return 0===Object.getOwnPropertyNames(e).length;var t;for(t in e)if(e.hasOwnProperty(t))return!1;return!0}(e)||i(e)&&0===e.length)&&(e=void 0),l._isAMomentObject=!0,l._useUTC=l._isUTC=a,l._l=n,l._i=e,l._f=t,l._strict=r,(s=new b(at(At(l))))._nextDay&&(s.add(1,"d"),s._nextDay=void 0),s}function It(e,t,n,r){return _t(e,t,n,r,!1)}r.createFromInputFallback=w("value provided is not in a recognized RFC2822 or ISO format. moment construction falls back to js Date(), which is not reliable across all browsers and versions. Non RFC2822/ISO date formats are discouraged and will be removed in an upcoming major release. Please refer to http://momentjs.com/guides/#/warnings/js-date/ for more info.",(function(e){e._d=new Date(e._i+(e._useUTC?" UTC":""))})),r.ISO_8601=function(){},r.RFC_2822=function(){};var xt=w("moment().min is deprecated, use moment.max instead. http://momentjs.com/guides/#/warnings/min-max/",(function(){var e=It.apply(null,arguments);return this.isValid()&&e.isValid()?e<this?this:e:g()})),wt=w("moment().max is deprecated, use moment.min instead. http://momentjs.com/guides/#/warnings/min-max/",(function(){var e=It.apply(null,arguments);return this.isValid()&&e.isValid()?e>this?this:e:g()}));function Mt(e,t){var n,r;if(1===t.length&&i(t[0])&&(t=t[0]),!t.length)return It();for(n=t[0],r=1;r<t.length;++r)t[r].isValid()&&!t[r][e](n)||(n=t[r]);return n}var St=["year","quarter","month","week","day","hour","minute","second","millisecond"];function Tt(e){var t=N(e),n=t.year||0,r=t.quarter||0,i=t.month||0,o=t.week||0,a=t.day||0,s=t.hour||0,l=t.minute||0,u=t.second||0,c=t.millisecond||0;this._isValid=function(e){for(var t in e)if(-1===ye.call(St,t)||null!=e[t]&&isNaN(e[t]))return!1;for(var n=!1,r=0;r<St.length;++r)if(e[St[r]]){if(n)return!1;parseFloat(e[St[r]])!==_(e[St[r]])&&(n=!0)}return!0}(t),this._milliseconds=+c+1e3*u+6e4*l+1e3*s*60*60,this._days=+a+7*o,this._months=+i+3*r+12*n,this._data={},this._locale=ot(),this._bubble()}function Et(e){return e instanceof Tt}function Ot(e){return e<0?-1*Math.round(-1*e):Math.round(e)}function kt(e,t){V(e,0,0,(function(){var e=this.utcOffset(),n="+";return e<0&&(e=-e,n="-"),n+z(~~(e/60),2)+t+z(~~e%60,2)}))}kt("Z",":"),kt("ZZ",""),ue("Z",ae),ue("ZZ",ae),fe(["Z","ZZ"],(function(e,t,n){n._useUTC=!0,n._tzm=Dt(ae,e)}));var Lt=/([\+\-]|\d\d)/gi;function Dt(e,t){var n=(t||"").match(e);if(null===n)return null;var r=((n[n.length-1]||[])+"").match(Lt)||["-",0,0],i=60*r[1]+_(r[2]);return 0===i?0:"+"===r[0]?i:-i}function Pt(e,t){var n,i;return t._isUTC?(n=t.clone(),i=(C(e)||l(e)?e.valueOf():It(e).valueOf())-n.valueOf(),n._d.setTime(n._d.valueOf()+i),r.updateOffset(n,!1),n):It(e).local()}function Nt(e){return 15*-Math.round(e._d.getTimezoneOffset()/15)}function jt(){return!!this.isValid()&&this._isUTC&&0===this._offset}r.updateOffset=function(){};var Rt=/^(\-|\+)?(?:(\d*)[. ])?(\d+)\:(\d+)(?:\:(\d+)(\.\d*)?)?$/,zt=/^(-|\+)?P(?:([-+]?[0-9,.]*)Y)?(?:([-+]?[0-9,.]*)M)?(?:([-+]?[0-9,.]*)W)?(?:([-+]?[0-9,.]*)D)?(?:T(?:([-+]?[0-9,.]*)H)?(?:([-+]?[0-9,.]*)M)?(?:([-+]?[0-9,.]*)S)?)?$/;function Ft(e,t){var n,r,i,o,a,l,u=e,d=null;return Et(e)?u={ms:e._milliseconds,d:e._days,M:e._months}:s(e)?(u={},t?u[t]=e:u.milliseconds=e):(d=Rt.exec(e))?(n="-"===d[1]?-1:1,u={y:0,d:_(d[2])*n,h:_(d[3])*n,m:_(d[4])*n,s:_(d[5])*n,ms:_(Ot(1e3*d[6]))*n}):(d=zt.exec(e))?(n="-"===d[1]?-1:(d[1],1),u={y:Bt(d[2],n),M:Bt(d[3],n),w:Bt(d[4],n),d:Bt(d[5],n),h:Bt(d[6],n),m:Bt(d[7],n),s:Bt(d[8],n)}):null==u?u={}:"object"==typeof u&&("from"in u||"to"in u)&&(o=It(u.from),a=It(u.to),i=o.isValid()&&a.isValid()?(a=Pt(a,o),o.isBefore(a)?l=Ht(o,a):((l=Ht(a,o)).milliseconds=-l.milliseconds,l.months=-l.months),l):{milliseconds:0,months:0},(u={}).ms=i.milliseconds,u.M=i.months),r=new Tt(u),Et(e)&&c(e,"_locale")&&(r._locale=e._locale),r}function Bt(e,t){var n=e&&parseFloat(e.replace(",","."));return(isNaN(n)?0:n)*t}function Ht(e,t){var n={milliseconds:0,months:0};return n.months=t.month()-e.month()+12*(t.year()-e.year()),e.clone().add(n.months,"M").isAfter(t)&&--n.months,n.milliseconds=+t-+e.clone().add(n.months,"M"),n}function Ut(e,t){return function(n,r){var i;return null===r||isNaN(+r)||(T(t,"moment()."+t+"(period, number) is deprecated. Please use moment()."+t+"(number, period). See http://momentjs.com/guides/#/warnings/add-inverted-param/ for more info."),i=n,n=r,r=i),Vt(this,Ft(n="string"==typeof n?+n:n,r),e),this}}function Vt(e,t,n,i){var o=t._milliseconds,a=Ot(t._days),s=Ot(t._months);e.isValid()&&(i=null==i||i,s&&Te(e,Ae(e,"Month")+s*n),a&&_e(e,"Date",Ae(e,"Date")+a*n),o&&e._d.setTime(e._d.valueOf()+o*n),i&&r.updateOffset(e,a||s))}Ft.fn=Tt.prototype,Ft.invalid=function(){return Ft(NaN)};var Yt=Ut(1,"add"),Wt=Ut(-1,"subtract");function Gt(e,t){var n=12*(t.year()-e.year())+(t.month()-e.month()),r=e.clone().add(n,"months");return-(n+(t-r<0?(t-r)/(r-e.clone().add(n-1,"months")):(t-r)/(e.clone().add(n+1,"months")-r)))||0}function Xt(e){var t;return void 0===e?this._locale._abbr:(null!=(t=ot(e))&&(this._locale=t),this)}r.defaultFormat="YYYY-MM-DDTHH:mm:ssZ",r.defaultFormatUtc="YYYY-MM-DDTHH:mm:ss[Z]";var qt=w("moment().lang() is deprecated. Instead, use moment().localeData() to get the language configuration. Use moment().locale() to change languages.",(function(e){return void 0===e?this.localeData():this.locale(e)}));function Zt(){return this._locale}function Qt(e,t){V(0,[e,e.length],0,t)}function Kt(e,t,n,r,i){var o;return null==e?Re(this,r,i).year:(t>(o=ze(e,r,i))&&(t=o),Jt.call(this,e,t,n,r,i))}function Jt(e,t,n,r,i){var o=je(e,t,n,r,i),a=Pe(o.year,0,o.dayOfYear);return this.year(a.getUTCFullYear()),this.month(a.getUTCMonth()),this.date(a.getUTCDate()),this}V(0,["gg",2],0,(function(){return this.weekYear()%100})),V(0,["GG",2],0,(function(){return this.isoWeekYear()%100})),Qt("gggg","weekYear"),Qt("ggggg","weekYear"),Qt("GGGG","isoWeekYear"),Qt("GGGGG","isoWeekYear"),D("weekYear","gg"),D("isoWeekYear","GG"),R("weekYear",1),R("isoWeekYear",1),ue("G",ie),ue("g",ie),ue("GG",K,X),ue("gg",K,X),ue("GGGG",te,Z),ue("gggg",te,Z),ue("GGGGG",ne,Q),ue("ggggg",ne,Q),pe(["gggg","ggggg","GGGG","GGGGG"],(function(e,t,n,r){t[r.substr(0,2)]=_(e)})),pe(["gg","GG"],(function(e,t,n,i){t[i]=r.parseTwoDigitYear(e)})),V("Q",0,"Qo","quarter"),D("quarter","Q"),R("quarter",7),ue("Q",G),fe("Q",(function(e,t){t[1]=3*(_(e)-1)})),V("D",["DD",2],"Do","date"),D("date","D"),R("date",9),ue("D",K),ue("DD",K,X),ue("Do",(function(e,t){return e?t._dayOfMonthOrdinalParse||t._ordinalParse:t._dayOfMonthOrdinalParseLenient})),fe(["D","DD"],2),fe("Do",(function(e,t){t[2]=_(e.match(K)[0])}));var $t=Ce("Date",!0);V("DDD",["DDDD",3],"DDDo","dayOfYear"),D("dayOfYear","DDD"),R("dayOfYear",4),ue("DDD",ee),ue("DDDD",q),fe(["DDD","DDDD"],(function(e,t,n){n._dayOfYear=_(e)})),V("m",["mm",2],0,"minute"),D("minute","m"),R("minute",14),ue("m",K),ue("mm",K,X),fe(["m","mm"],4);var en=Ce("Minutes",!1);V("s",["ss",2],0,"second"),D("second","s"),R("second",15),ue("s",K),ue("ss",K,X),fe(["s","ss"],5);var tn,nn=Ce("Seconds",!1);for(V("S",0,0,(function(){return~~(this.millisecond()/100)})),V(0,["SS",2],0,(function(){return~~(this.millisecond()/10)})),V(0,["SSS",3],0,"millisecond"),V(0,["SSSS",4],0,(function(){return 10*this.millisecond()})),V(0,["SSSSS",5],0,(function(){return 100*this.millisecond()})),V(0,["SSSSSS",6],0,(function(){return 1e3*this.millisecond()})),V(0,["SSSSSSS",7],0,(function(){return 1e4*this.millisecond()})),V(0,["SSSSSSSS",8],0,(function(){return 1e5*this.millisecond()})),V(0,["SSSSSSSSS",9],0,(function(){return 1e6*this.millisecond()})),D("millisecond","ms"),R("millisecond",16),ue("S",ee,G),ue("SS",ee,X),ue("SSS",ee,q),tn="SSSS";tn.length<=9;tn+="S")ue(tn,re);function rn(e,t){t[6]=_(1e3*("0."+e))}for(tn="S";tn.length<=9;tn+="S")fe(tn,rn);var on=Ce("Milliseconds",!1);V("z",0,0,"zoneAbbr"),V("zz",0,0,"zoneName");var an=b.prototype;function sn(e){return e}an.add=Yt,an.calendar=function(e,t){var n=e||It(),i=Pt(n,this).startOf("day"),o=r.calendarFormat(this,i)||"sameElse",a=t&&(E(t[o])?t[o].call(this,n):t[o]);return this.format(a||this.localeData().calendar(o,this,It(n)))},an.clone=function(){return new b(this)},an.diff=function(e,t,n){var r,i,o;if(!this.isValid())return NaN;if(!(r=Pt(e,this)).isValid())return NaN;switch(i=6e4*(r.utcOffset()-this.utcOffset()),t=P(t)){case"year":o=Gt(this,r)/12;break;case"month":o=Gt(this,r);break;case"quarter":o=Gt(this,r)/3;break;case"second":o=(this-r)/1e3;break;case"minute":o=(this-r)/6e4;break;case"hour":o=(this-r)/36e5;break;case"day":o=(this-r-i)/864e5;break;case"week":o=(this-r-i)/6048e5;break;default:o=this-r}return n?o:A(o)},an.endOf=function(e){return void 0===(e=P(e))||"millisecond"===e?this:("date"===e&&(e="day"),this.startOf(e).add(1,"isoWeek"===e?"week":e).subtract(1,"ms"))},an.format=function(e){e||(e=this.isUtc()?r.defaultFormatUtc:r.defaultFormat);var t=Y(this,e);return this.localeData().postformat(t)},an.from=function(e,t){return this.isValid()&&(C(e)&&e.isValid()||It(e).isValid())?Ft({to:this,from:e}).locale(this.locale()).humanize(!t):this.localeData().invalidDate()},an.fromNow=function(e){return this.from(It(),e)},an.to=function(e,t){return this.isValid()&&(C(e)&&e.isValid()||It(e).isValid())?Ft({from:this,to:e}).locale(this.locale()).humanize(!t):this.localeData().invalidDate()},an.toNow=function(e){return this.to(It(),e)},an.get=function(e){return E(this[e=P(e)])?this[e]():this},an.invalidAt=function(){return f(this).overflow},an.isAfter=function(e,t){var n=C(e)?e:It(e);return!(!this.isValid()||!n.isValid())&&("millisecond"===(t=P(a(t)?"millisecond":t))?this.valueOf()>n.valueOf():n.valueOf()<this.clone().startOf(t).valueOf())},an.isBefore=function(e,t){var n=C(e)?e:It(e);return!(!this.isValid()||!n.isValid())&&("millisecond"===(t=P(a(t)?"millisecond":t))?this.valueOf()<n.valueOf():this.clone().endOf(t).valueOf()<n.valueOf())},an.isBetween=function(e,t,n,r){return("("===(r=r||"()")[0]?this.isAfter(e,n):!this.isBefore(e,n))&&(")"===r[1]?this.isBefore(t,n):!this.isAfter(t,n))},an.isSame=function(e,t){var n,r=C(e)?e:It(e);return!(!this.isValid()||!r.isValid())&&("millisecond"===(t=P(t||"millisecond"))?this.valueOf()===r.valueOf():(n=r.valueOf(),this.clone().startOf(t).valueOf()<=n&&n<=this.clone().endOf(t).valueOf()))},an.isSameOrAfter=function(e,t){return this.isSame(e,t)||this.isAfter(e,t)},an.isSameOrBefore=function(e,t){return this.isSame(e,t)||this.isBefore(e,t)},an.isValid=function(){return p(this)},an.lang=qt,an.locale=Xt,an.localeData=Zt,an.max=wt,an.min=xt,an.parsingFlags=function(){return d({},f(this))},an.set=function(e,t){if("object"==typeof e)for(var n=function(e){var t=[];for(var n in e)t.push({unit:n,priority:j[n]});return t.sort((function(e,t){return e.priority-t.priority})),t}(e=N(e)),r=0;r<n.length;r++)this[n[r].unit](e[n[r].unit]);else if(E(this[e=P(e)]))return this[e](t);return this},an.startOf=function(e){switch(e=P(e)){case"year":this.month(0);case"quarter":case"month":this.date(1);case"week":case"isoWeek":case"day":case"date":this.hours(0);case"hour":this.minutes(0);case"minute":this.seconds(0);case"second":this.milliseconds(0)}return"week"===e&&this.weekday(0),"isoWeek"===e&&this.isoWeekday(1),"quarter"===e&&this.month(3*Math.floor(this.month()/3)),this},an.subtract=Wt,an.toArray=function(){var e=this;return[e.year(),e.month(),e.date(),e.hour(),e.minute(),e.second(),e.millisecond()]},an.toObject=function(){var e=this;return{years:e.year(),months:e.month(),date:e.date(),hours:e.hours(),minutes:e.minutes(),seconds:e.seconds(),milliseconds:e.milliseconds()}},an.toDate=function(){return new Date(this.valueOf())},an.toISOString=function(){if(!this.isValid())return null;var e=this.clone().utc();return e.year()<0||e.year()>9999?Y(e,"YYYYYY-MM-DD[T]HH:mm:ss.SSS[Z]"):E(Date.prototype.toISOString)?this.toDate().toISOString():Y(e,"YYYY-MM-DD[T]HH:mm:ss.SSS[Z]")},an.inspect=function(){if(!this.isValid())return"moment.invalid(/* "+this._i+" */)";var e="moment",t="";this.isLocal()||(e=0===this.utcOffset()?"moment.utc":"moment.parseZone",t="Z");var n="["+e+'("]',r=0<=this.year()&&this.year()<=9999?"YYYY":"YYYYYY",i=t+'[")]';return this.format(n+r+"-MM-DD[T]HH:mm:ss.SSS"+i)},an.toJSON=function(){return this.isValid()?this.toISOString():null},an.toString=function(){return this.clone().locale("en").format("ddd MMM DD YYYY HH:mm:ss [GMT]ZZ")},an.unix=function(){return Math.floor(this.valueOf()/1e3)},an.valueOf=function(){return this._d.valueOf()-6e4*(this._offset||0)},an.creationData=function(){return{input:this._i,format:this._f,locale:this._locale,isUTC:this._isUTC,strict:this._strict}},an.year=be,an.isLeapYear=function(){return me(this.year())},an.weekYear=function(e){return Kt.call(this,e,this.week(),this.weekday(),this.localeData()._week.dow,this.localeData()._week.doy)},an.isoWeekYear=function(e){return Kt.call(this,e,this.isoWeek(),this.isoWeekday(),1,4)},an.quarter=an.quarters=function(e){return null==e?Math.ceil((this.month()+1)/3):this.month(3*(e-1)+this.month()%3)},an.month=Ee,an.daysInMonth=function(){return Ie(this.year(),this.month())},an.week=an.weeks=function(e){var t=this.localeData().week(this);return null==e?t:this.add(7*(e-t),"d")},an.isoWeek=an.isoWeeks=function(e){var t=Re(this,1,4).week;return null==e?t:this.add(7*(e-t),"d")},an.weeksInYear=function(){var e=this.localeData()._week;return ze(this.year(),e.dow,e.doy)},an.isoWeeksInYear=function(){return ze(this.year(),1,4)},an.date=$t,an.day=an.days=function(e){if(!this.isValid())return null!=e?this:NaN;var t=this._isUTC?this._d.getUTCDay():this._d.getDay();return null!=e?(e=function(e,t){return"string"!=typeof e?e:isNaN(e)?"number"==typeof(e=t.weekdaysParse(e))?e:null:parseInt(e,10)}(e,this.localeData()),this.add(e-t,"d")):t},an.weekday=function(e){if(!this.isValid())return null!=e?this:NaN;var t=(this.day()+7-this.localeData()._week.dow)%7;return null==e?t:this.add(e-t,"d")},an.isoWeekday=function(e){if(!this.isValid())return null!=e?this:NaN;if(null!=e){var t=function(e,t){return"string"==typeof e?t.weekdaysParse(e)%7||7:isNaN(e)?null:e}(e,this.localeData());return this.day(this.day()%7?t:t-7)}return this.day()||7},an.dayOfYear=function(e){var t=Math.round((this.clone().startOf("day")-this.clone().startOf("year"))/864e5)+1;return null==e?t:this.add(e-t,"d")},an.hour=an.hours=Ke,an.minute=an.minutes=en,an.second=an.seconds=nn,an.millisecond=an.milliseconds=on,an.utcOffset=function(e,t,n){var i,o=this._offset||0;if(!this.isValid())return null!=e?this:NaN;if(null!=e){if("string"==typeof e){if(null===(e=Dt(ae,e)))return this}else Math.abs(e)<16&&!n&&(e*=60);return!this._isUTC&&t&&(i=Nt(this)),this._offset=e,this._isUTC=!0,null!=i&&this.add(i,"m"),o!==e&&(!t||this._changeInProgress?Vt(this,Ft(e-o,"m"),1,!1):this._changeInProgress||(this._changeInProgress=!0,r.updateOffset(this,!0),this._changeInProgress=null)),this}return this._isUTC?o:Nt(this)},an.utc=function(e){return this.utcOffset(0,e)},an.local=function(e){return this._isUTC&&(this.utcOffset(0,e),this._isUTC=!1,e&&this.subtract(Nt(this),"m")),this},an.parseZone=function(){if(null!=this._tzm)this.utcOffset(this._tzm,!1,!0);else if("string"==typeof this._i){var e=Dt(oe,this._i);null!=e?this.utcOffset(e):this.utcOffset(0,!0)}return this},an.hasAlignedHourOffset=function(e){return!!this.isValid()&&(e=e?It(e).utcOffset():0,(this.utcOffset()-e)%60==0)},an.isDST=function(){return this.utcOffset()>this.clone().month(0).utcOffset()||this.utcOffset()>this.clone().month(5).utcOffset()},an.isLocal=function(){return!!this.isValid()&&!this._isUTC},an.isUtcOffset=function(){return!!this.isValid()&&this._isUTC},an.isUtc=jt,an.isUTC=jt,an.zoneAbbr=function(){return this._isUTC?"UTC":""},an.zoneName=function(){return this._isUTC?"Coordinated Universal Time":""},an.dates=w("dates accessor is deprecated. Use date instead.",$t),an.months=w("months accessor is deprecated. Use month instead",Ee),an.years=w("years accessor is deprecated. Use year instead",be),an.zone=w("moment().zone is deprecated, use moment().utcOffset instead. http://momentjs.com/guides/#/warnings/zone/",(function(e,t){return null!=e?("string"!=typeof e&&(e=-e),this.utcOffset(e,t),this):-this.utcOffset()})),an.isDSTShifted=w("isDSTShifted is deprecated. See http://momentjs.com/guides/#/warnings/dst-shifted/ for more information",(function(){if(!a(this._isDSTShifted))return this._isDSTShifted;var e={};if(m(e,this),(e=At(e))._a){var t=e._isUTC?h(e._a):It(e._a);this._isDSTShifted=this.isValid()&&I(e._a,t.toArray())>0}else this._isDSTShifted=!1;return this._isDSTShifted}));var ln=k.prototype;function un(e,t,n,r){var i=ot(),o=h().set(r,t);return i[n](o,e)}function cn(e,t,n){if(s(e)&&(t=e,e=void 0),e=e||"",null!=t)return un(e,t,n,"month");var r,i=[];for(r=0;r<12;r++)i[r]=un(e,r,n,"month");return i}function dn(e,t,n,r){"boolean"==typeof e?(s(t)&&(n=t,t=void 0),t=t||""):(n=t=e,e=!1,s(t)&&(n=t,t=void 0),t=t||"");var i,o=ot(),a=e?o._week.dow:0;if(null!=n)return un(t,(n+a)%7,r,"day");var l=[];for(i=0;i<7;i++)l[i]=un(t,(i+a)%7,r,"day");return l}ln.calendar=function(e,t,n){var r=this._calendar[e]||this._calendar.sameElse;return E(r)?r.call(t,n):r},ln.longDateFormat=function(e){var t=this._longDateFormat[e],n=this._longDateFormat[e.toUpperCase()];return t||!n?t:(this._longDateFormat[e]=n.replace(/MMMM|MM|DD|dddd/g,(function(e){return e.slice(1)})),this._longDateFormat[e])},ln.invalidDate=function(){return this._invalidDate},ln.ordinal=function(e){return this._ordinal.replace("%d",e)},ln.preparse=sn,ln.postformat=sn,ln.relativeTime=function(e,t,n,r){var i=this._relativeTime[n];return E(i)?i(e,t,n,r):i.replace(/%d/i,e)},ln.pastFuture=function(e,t){var n=this._relativeTime[e>0?"future":"past"];return E(n)?n(t):n.replace(/%s/i,t)},ln.set=function(e){var t,n;for(n in e)E(t=e[n])?this[n]=t:this["_"+n]=t;this._config=e,this._dayOfMonthOrdinalParseLenient=new RegExp((this._dayOfMonthOrdinalParse.source||this._ordinalParse.source)+"|"+/\d{1,2}/.source)},ln.months=function(e,t){return e?i(this._months)?this._months[e.month()]:this._months[(this._months.isFormat||xe).test(t)?"format":"standalone"][e.month()]:i(this._months)?this._months:this._months.standalone},ln.monthsShort=function(e,t){return e?i(this._monthsShort)?this._monthsShort[e.month()]:this._monthsShort[xe.test(t)?"format":"standalone"][e.month()]:i(this._monthsShort)?this._monthsShort:this._monthsShort.standalone},ln.monthsParse=function(e,t,n){var r,i,o;if(this._monthsParseExact)return Se.call(this,e,t,n);for(this._monthsParse||(this._monthsParse=[],this._longMonthsParse=[],this._shortMonthsParse=[]),r=0;r<12;r++){if(i=h([2e3,r]),n&&!this._longMonthsParse[r]&&(this._longMonthsParse[r]=new RegExp("^"+this.months(i,"").replace(".","")+"$","i"),this._shortMonthsParse[r]=new RegExp("^"+this.monthsShort(i,"").replace(".","")+"$","i")),n||this._monthsParse[r]||(o="^"+this.months(i,"")+"|^"+this.monthsShort(i,""),this._monthsParse[r]=new RegExp(o.replace(".",""),"i")),n&&"MMMM"===t&&this._longMonthsParse[r].test(e))return r;if(n&&"MMM"===t&&this._shortMonthsParse[r].test(e))return r;if(!n&&this._monthsParse[r].test(e))return r}},ln.monthsRegex=function(e){return this._monthsParseExact?(c(this,"_monthsRegex")||Le.call(this),e?this._monthsStrictRegex:this._monthsRegex):(c(this,"_monthsRegex")||(this._monthsRegex=ke),this._monthsStrictRegex&&e?this._monthsStrictRegex:this._monthsRegex)},ln.monthsShortRegex=function(e){return this._monthsParseExact?(c(this,"_monthsRegex")||Le.call(this),e?this._monthsShortStrictRegex:this._monthsShortRegex):(c(this,"_monthsShortRegex")||(this._monthsShortRegex=Oe),this._monthsShortStrictRegex&&e?this._monthsShortStrictRegex:this._monthsShortRegex)},ln.week=function(e){return Re(e,this._week.dow,this._week.doy).week},ln.firstDayOfYear=function(){return this._week.doy},ln.firstDayOfWeek=function(){return this._week.dow},ln.weekdays=function(e,t){return e?i(this._weekdays)?this._weekdays[e.day()]:this._weekdays[this._weekdays.isFormat.test(t)?"format":"standalone"][e.day()]:i(this._weekdays)?this._weekdays:this._weekdays.standalone},ln.weekdaysMin=function(e){return e?this._weekdaysMin[e.day()]:this._weekdaysMin},ln.weekdaysShort=function(e){return e?this._weekdaysShort[e.day()]:this._weekdaysShort},ln.weekdaysParse=function(e,t,n){var r,i,o;if(this._weekdaysParseExact)return Ue.call(this,e,t,n);for(this._weekdaysParse||(this._weekdaysParse=[],this._minWeekdaysParse=[],this._shortWeekdaysParse=[],this._fullWeekdaysParse=[]),r=0;r<7;r++){if(i=h([2e3,1]).day(r),n&&!this._fullWeekdaysParse[r]&&(this._fullWeekdaysParse[r]=new RegExp("^"+this.weekdays(i,"").replace(".",".?")+"$","i"),this._shortWeekdaysParse[r]=new RegExp("^"+this.weekdaysShort(i,"").replace(".",".?")+"$","i"),this._minWeekdaysParse[r]=new RegExp("^"+this.weekdaysMin(i,"").replace(".",".?")+"$","i")),this._weekdaysParse[r]||(o="^"+this.weekdays(i,"")+"|^"+this.weekdaysShort(i,"")+"|^"+this.weekdaysMin(i,""),this._weekdaysParse[r]=new RegExp(o.replace(".",""),"i")),n&&"dddd"===t&&this._fullWeekdaysParse[r].test(e))return r;if(n&&"ddd"===t&&this._shortWeekdaysParse[r].test(e))return r;if(n&&"dd"===t&&this._minWeekdaysParse[r].test(e))return r;if(!n&&this._weekdaysParse[r].test(e))return r}},ln.weekdaysRegex=function(e){return this._weekdaysParseExact?(c(this,"_weekdaysRegex")||Ge.call(this),e?this._weekdaysStrictRegex:this._weekdaysRegex):(c(this,"_weekdaysRegex")||(this._weekdaysRegex=Ve),this._weekdaysStrictRegex&&e?this._weekdaysStrictRegex:this._weekdaysRegex)},ln.weekdaysShortRegex=function(e){return this._weekdaysParseExact?(c(this,"_weekdaysRegex")||Ge.call(this),e?this._weekdaysShortStrictRegex:this._weekdaysShortRegex):(c(this,"_weekdaysShortRegex")||(this._weekdaysShortRegex=Ye),this._weekdaysShortStrictRegex&&e?this._weekdaysShortStrictRegex:this._weekdaysShortRegex)},ln.weekdaysMinRegex=function(e){return this._weekdaysParseExact?(c(this,"_weekdaysRegex")||Ge.call(this),e?this._weekdaysMinStrictRegex:this._weekdaysMinRegex):(c(this,"_weekdaysMinRegex")||(this._weekdaysMinRegex=We),this._weekdaysMinStrictRegex&&e?this._weekdaysMinStrictRegex:this._weekdaysMinRegex)},ln.isPM=function(e){return"p"===(e+"").toLowerCase().charAt(0)},ln.meridiem=function(e,t,n){return e>11?n?"pm":"PM":n?"am":"AM"},rt("en",{dayOfMonthOrdinalParse:/\d{1,2}(th|st|nd|rd)/,ordinal:function(e){var t=e%10;return e+(1===_(e%100/10)?"th":1===t?"st":2===t?"nd":3===t?"rd":"th")}}),r.lang=w("moment.lang is deprecated. Use moment.locale instead.",rt),r.langData=w("moment.langData is deprecated. Use moment.localeData instead.",ot);var hn=Math.abs;function fn(e,t,n,r){var i=Ft(t,n);return e._milliseconds+=r*i._milliseconds,e._days+=r*i._days,e._months+=r*i._months,e._bubble()}function pn(e){return e<0?Math.floor(e):Math.ceil(e)}function gn(e){return 4800*e/146097}function vn(e){return 146097*e/4800}function mn(e){return function(){return this.as(e)}}var yn=mn("ms"),bn=mn("s"),Cn=mn("m"),An=mn("h"),_n=mn("d"),In=mn("w"),xn=mn("M"),wn=mn("y");function Mn(e){return function(){return this.isValid()?this._data[e]:NaN}}var Sn=Mn("milliseconds"),Tn=Mn("seconds"),En=Mn("minutes"),On=Mn("hours"),kn=Mn("days"),Ln=Mn("months"),Dn=Mn("years"),Pn=Math.round,Nn={ss:44,s:45,m:45,h:22,d:26,M:11};function jn(e,t,n,r,i){return i.relativeTime(t||1,!!n,e,r)}var Rn=Math.abs;function zn(e){return(e>0)-(e<0)||+e}function Fn(){if(!this.isValid())return this.localeData().invalidDate();var e,t,n=Rn(this._milliseconds)/1e3,r=Rn(this._days),i=Rn(this._months);e=A(n/60),t=A(e/60),n%=60,e%=60;var o=A(i/12),a=i%=12,s=r,l=t,u=e,c=n?n.toFixed(3).replace(/\.?0+$/,""):"",d=this.asSeconds();if(!d)return"P0D";var h=d<0?"-":"",f=zn(this._months)!==zn(d)?"-":"",p=zn(this._days)!==zn(d)?"-":"",g=zn(this._milliseconds)!==zn(d)?"-":"";return h+"P"+(o?f+o+"Y":"")+(a?f+a+"M":"")+(s?p+s+"D":"")+(l||u||c?"T":"")+(l?g+l+"H":"")+(u?g+u+"M":"")+(c?g+c+"S":"")}var Bn=Tt.prototype;return Bn.isValid=function(){return this._isValid},Bn.abs=function(){var e=this._data;return this._milliseconds=hn(this._milliseconds),this._days=hn(this._days),this._months=hn(this._months),e.milliseconds=hn(e.milliseconds),e.seconds=hn(e.seconds),e.minutes=hn(e.minutes),e.hours=hn(e.hours),e.months=hn(e.months),e.years=hn(e.years),this},Bn.add=function(e,t){return fn(this,e,t,1)},Bn.subtract=function(e,t){return fn(this,e,t,-1)},Bn.as=function(e){if(!this.isValid())return NaN;var t,n,r=this._milliseconds;if("month"===(e=P(e))||"year"===e)return t=this._days+r/864e5,n=this._months+gn(t),"month"===e?n:n/12;switch(t=this._days+Math.round(vn(this._months)),e){case"week":return t/7+r/6048e5;case"day":return t+r/864e5;case"hour":return 24*t+r/36e5;case"minute":return 1440*t+r/6e4;case"second":return 86400*t+r/1e3;case"millisecond":return Math.floor(864e5*t)+r;default:throw new Error("Unknown unit "+e)}},Bn.asMilliseconds=yn,Bn.asSeconds=bn,Bn.asMinutes=Cn,Bn.asHours=An,Bn.asDays=_n,Bn.asWeeks=In,Bn.asMonths=xn,Bn.asYears=wn,Bn.valueOf=function(){return this.isValid()?this._milliseconds+864e5*this._days+this._months%12*2592e6+31536e6*_(this._months/12):NaN},Bn._bubble=function(){var e,t,n,r,i,o=this._milliseconds,a=this._days,s=this._months,l=this._data;return o>=0&&a>=0&&s>=0||o<=0&&a<=0&&s<=0||(o+=864e5*pn(vn(s)+a),a=0,s=0),l.milliseconds=o%1e3,e=A(o/1e3),l.seconds=e%60,t=A(e/60),l.minutes=t%60,n=A(t/60),l.hours=n%24,a+=A(n/24),i=A(gn(a)),s+=i,a-=pn(vn(i)),r=A(s/12),s%=12,l.days=a,l.months=s,l.years=r,this},Bn.clone=function(){return Ft(this)},Bn.get=function(e){return e=P(e),this.isValid()?this[e+"s"]():NaN},Bn.milliseconds=Sn,Bn.seconds=Tn,Bn.minutes=En,Bn.hours=On,Bn.days=kn,Bn.weeks=function(){return A(this.days()/7)},Bn.months=Ln,Bn.years=Dn,Bn.humanize=function(e){if(!this.isValid())return this.localeData().invalidDate();var t=this.localeData(),n=function(e,t,n){var r=Ft(e).abs(),i=Pn(r.as("s")),o=Pn(r.as("m")),a=Pn(r.as("h")),s=Pn(r.as("d")),l=Pn(r.as("M")),u=Pn(r.as("y")),c=i<=Nn.ss&&["s",i]||i<Nn.s&&["ss",i]||o<=1&&["m"]||o<Nn.m&&["mm",o]||a<=1&&["h"]||a<Nn.h&&["hh",a]||s<=1&&["d"]||s<Nn.d&&["dd",s]||l<=1&&["M"]||l<Nn.M&&["MM",l]||u<=1&&["y"]||["yy",u];return c[2]=t,c[3]=+e>0,c[4]=n,jn.apply(null,c)}(this,!e,t);return e&&(n=t.pastFuture(+this,n)),t.postformat(n)},Bn.toISOString=Fn,Bn.toString=Fn,Bn.toJSON=Fn,Bn.locale=Xt,Bn.localeData=Zt,Bn.toIsoString=w("toIsoString() is deprecated. Please use toISOString() instead (notice the capitals)",Fn),Bn.lang=qt,V("X",0,0,"unix"),V("x",0,0,"valueOf"),ue("x",ie),ue("X",/[+-]?\d+(\.\d{1,3})?/),fe("X",(function(e,t,n){n._d=new Date(1e3*parseFloat(e,10))})),fe("x",(function(e,t,n){n._d=new Date(_(e))})),r.version="2.19.1",t=It,r.fn=an,r.min=function(){var e=[].slice.call(arguments,0);return Mt("isBefore",e)},r.max=function(){var e=[].slice.call(arguments,0);return Mt("isAfter",e)},r.now=function(){return Date.now?Date.now():+new Date},r.utc=h,r.unix=function(e){return It(1e3*e)},r.months=function(e,t){return cn(e,t,"months")},r.isDate=l,r.locale=rt,r.invalid=g,r.duration=Ft,r.isMoment=C,r.weekdays=function(e,t,n){return dn(e,t,n,"weekdays")},r.parseZone=function(){return It.apply(null,arguments).parseZone()},r.localeData=ot,r.isDuration=Et,r.monthsShort=function(e,t){return cn(e,t,"monthsShort")},r.weekdaysMin=function(e,t,n){return dn(e,t,n,"weekdaysMin")},r.defineLocale=it,r.updateLocale=function(e,t){if(null!=t){var n,r=Je;null!=$e[e]&&(r=$e[e]._config),t=O(r,t),(n=new k(t)).parentLocale=$e[e],$e[e]=n,rt(e)}else null!=$e[e]&&(null!=$e[e].parentLocale?$e[e]=$e[e].parentLocale:null!=$e[e]&&delete $e[e]);return $e[e]},r.locales=function(){return M($e)},r.weekdaysShort=function(e,t,n){return dn(e,t,n,"weekdaysShort")},r.normalizeUnits=P,r.relativeTimeRounding=function(e){return void 0===e?Pn:"function"==typeof e&&(Pn=e,!0)},r.relativeTimeThreshold=function(e,t){return void 0!==Nn[e]&&(void 0===t?Nn[e]:(Nn[e]=t,"s"===e&&(Nn.ss=t-1),!0))},r.calendarFormat=function(e,t){var n=e.diff(t,"days",!0);return n<-6?"sameElse":n<-1?"lastWeek":n<0?"lastDay":n<1?"sameDay":n<2?"nextDay":n<7?"nextWeek":"sameElse"},r.prototype=an,r}()}).call(t,n(155)(e))},function(e,t){e.exports=function(e){return e.webpackPolyfill||(e.deprecate=function(){},e.paths=[],e.children||(e.children=[]),Object.defineProperty(e,"loaded",{enumerable:!0,get:function(){return e.l}}),Object.defineProperty(e,"id",{enumerable:!0,get:function(){return e.i}}),e.webpackPolyfill=1),e}},function(e,t){function n(e){throw new Error("Cannot find module '"+e+"'.")}n.keys=function(){return[]},n.resolve=n,e.exports=n,n.id=156},function(e,t,n){(function(t){var n,r="undefined"!=typeof window?window:void 0!==t?t:null;if(r&&r.crypto&&crypto.getRandomValues){var i=new Uint8Array(16);n=function(){return crypto.getRandomValues(i),i}}if(!n){var o=new Array(16);n=function(){for(var e,t=0;t<16;t++)0==(3&t)&&(e=4294967296*Math.random()),o[t]=e>>>((3&t)<<3)&255;return o}}for(var a=[],s={},l=0;l<256;l++)a[l]=(l+256).toString(16).substr(1),s[a[l]]=l;function u(e,t){var n=t||0,r=a;return r[e[n++]]+r[e[n++]]+r[e[n++]]+r[e[n++]]+"-"+r[e[n++]]+r[e[n++]]+"-"+r[e[n++]]+r[e[n++]]+"-"+r[e[n++]]+r[e[n++]]+"-"+r[e[n++]]+r[e[n++]]+r[e[n++]]+r[e[n++]]+r[e[n++]]+r[e[n++]]}var c=n(),d=[1|c[0],c[1],c[2],c[3],c[4],c[5]],h=16383&(c[6]<<8|c[7]),f=0,p=0;function g(e,t,r){var i=t&&r||0;"string"==typeof e&&(t="binary"==e?new Array(16):null,e=null);var o=(e=e||{}).random||(e.rng||n)();if(o[6]=15&o[6]|64,o[8]=63&o[8]|128,t)for(var a=0;a<16;a++)t[i+a]=o[a];return t||u(o)}var v=g;v.v1=function(e,t,n){var r=t&&n||0,i=t||[],o=void 0!==(e=e||{}).clockseq?e.clockseq:h,a=void 0!==e.msecs?e.msecs:(new Date).getTime(),s=void 0!==e.nsecs?e.nsecs:p+1,l=a-f+(s-p)/1e4;if(l<0&&void 0===e.clockseq&&(o=o+1&16383),(l<0||a>f)&&void 0===e.nsecs&&(s=0),s>=1e4)throw new Error("uuid.v1(): Can't create more than 10M uuids/sec");f=a,p=s,h=o;var c=(1e4*(268435455&(a+=122192928e5))+s)%4294967296;i[r++]=c>>>24&255,i[r++]=c>>>16&255,i[r++]=c>>>8&255,i[r++]=255&c;var g=a/4294967296*1e4&268435455;i[r++]=g>>>8&255,i[r++]=255&g,i[r++]=g>>>24&15|16,i[r++]=g>>>16&255,i[r++]=o>>>8|128,i[r++]=255&o;for(var v=e.node||d,m=0;m<6;m++)i[r+m]=v[m];return t||u(i)},v.v4=g,v.parse=function(e,t,n){var r=t&&n||0,i=0;for(t=t||[],e.toLowerCase().replace(/[0-9a-f]{2}/g,(function(e){i<16&&(t[r+i++]=s[e])}));i<16;)t[r+i++]=0;return t},v.unparse=u,e.exports=v}).call(t,n(158))},function(e,t){var n;n=function(){return this}();try{n=n||Function("return this")()||(0,eval)("this")}catch(e){"object"==typeof window&&(n=window)}e.exports=n},function(e,t,n){t.util=n(2),t.DOMutil=n(14),t.DataSet=n(11),t.DataView=n(12),t.Queue=n(43),t.Graph3d=n(161),t.graph3d={Camera:n(95),Filter:n(96),Point2d:n(91),Point3d:n(34),Slider:n(92),StepNumber:n(93)},t.moment=n(9),t.Hammer=n(10),t.keycharm=n(35)},function(e,t,n){var r=n(7),i=r.JSON||(r.JSON={stringify:JSON.stringify});e.exports=function(e){return i.stringify.apply(i,arguments)}},function(e,t,n){var r,i=n(90),o=(r=i)&&r.__esModule?r:{default:r},a=n(44),s=n(2),l=n(34),u=n(91),c=n(92),d=n(93),h=n(94),f=n(15).default,p=n(15).printStyle,g=n(172).allOptions,v=n(173);function m(e,t,n){if(!(this instanceof m))throw new SyntaxError("Constructor must be called with the new operator");this.containerElement=e,this.dataGroup=new v,this.dataPoints=null,this.create(),h.setDefaults(m.DEFAULTS,this),this.colX=void 0,this.colY=void 0,this.colZ=void 0,this.colValue=void 0,this.setOptions(n),this.setData(t)}function y(e){return"clientX"in e?e.clientX:e.targetTouches[0]&&e.targetTouches[0].clientX||0}function b(e){return"clientY"in e?e.clientY:e.targetTouches[0]&&e.targetTouches[0].clientY||0}m.STYLE=h.STYLE,m.DEFAULTS={width:"400px",height:"400px",filterLabel:"time",legendLabel:"value",xLabel:"x",yLabel:"y",zLabel:"z",xValueLabel:function(e){return e},yValueLabel:function(e){return e},zValueLabel:function(e){return e},showXAxis:!0,showYAxis:!0,showZAxis:!0,showGrid:!0,showPerspective:!0,showShadow:!1,keepAspectRatio:!0,verticalRatio:.5,dotSizeRatio:.02,dotSizeMinFraction:.5,dotSizeMaxFraction:2.5,showAnimationControls:void 0,animationInterval:1e3,animationPreload:!1,animationAutoStart:void 0,axisColor:"#4D4D4D",gridColor:"#D3D3D3",xCenter:"55%",yCenter:"50%",style:m.STYLE.DOT,tooltip:!1,tooltipStyle:{content:{padding:"10px",border:"1px solid #4d4d4d",color:"#1a1a1a",background:"rgba(255,255,255,0.7)",borderRadius:"2px",boxShadow:"5px 5px 10px rgba(128,128,128,0.5)"},line:{height:"40px",width:"0",borderLeft:"1px solid #4d4d4d"},dot:{height:"0",width:"0",border:"5px solid #4d4d4d",borderRadius:"5px"}},dataColor:{fill:"#7DC1FF",stroke:"#3267D2",strokeWidth:1},cameraPosition:{horizontal:1,vertical:.5,distance:1.7},showLegend:void 0,backgroundColor:void 0,xBarWidth:void 0,yBarWidth:void 0,valueMin:void 0,valueMax:void 0,xMin:void 0,xMax:void 0,xStep:void 0,yMin:void 0,yMax:void 0,yStep:void 0,zMin:void 0,zMax:void 0,zStep:void 0},a(m.prototype),m.prototype._setScale=function(){this.scale=new l(1/this.xRange.range(),1/this.yRange.range(),1/this.zRange.range()),this.keepAspectRatio&&(this.scale.x<this.scale.y?this.scale.y=this.scale.x:this.scale.x=this.scale.y),this.scale.z*=this.verticalRatio,void 0!==this.valueRange&&(this.scale.value=1/this.valueRange.range());var e=this.xRange.center()*this.scale.x,t=this.yRange.center()*this.scale.y,n=this.zRange.center()*this.scale.z;this.camera.setArmLocation(e,t,n)},m.prototype._convert3Dto2D=function(e){var t=this._convertPointToTranslation(e);return this._convertTranslationToScreen(t)},m.prototype._convertPointToTranslation=function(e){var t=this.camera.getCameraLocation(),n=this.camera.getCameraRotation(),r=e.x*this.scale.x,i=e.y*this.scale.y,o=e.z*this.scale.z,a=t.x,s=t.y,u=t.z,c=Math.sin(n.x),d=Math.cos(n.x),h=Math.sin(n.y),f=Math.cos(n.y),p=Math.sin(n.z),g=Math.cos(n.z);return new l(f*(p*(i-s)+g*(r-a))-h*(o-u),c*(f*(o-u)+h*(p*(i-s)+g*(r-a)))+d*(g*(i-s)-p*(r-a)),d*(f*(o-u)+h*(p*(i-s)+g*(r-a)))-c*(g*(i-s)-p*(r-a)))},m.prototype._convertTranslationToScreen=function(e){var t,n,r=this.eye.x,i=this.eye.y,o=this.eye.z,a=e.x,s=e.y,l=e.z;return this.showPerspective?(t=o/l*(a-r),n=o/l*(s-i)):(t=a*(-o/this.camera.getArmLength()),n=s*(-o/this.camera.getArmLength())),new u(this.currentXCenter+t*this.frame.canvas.clientWidth,this.currentYCenter-n*this.frame.canvas.clientWidth)},m.prototype._calcTranslations=function(e){for(var t=0;t<e.length;t++){var n=e[t];n.trans=this._convertPointToTranslation(n.point),n.screen=this._convertTranslationToScreen(n.trans);var r=this._convertPointToTranslation(n.bottom);n.dist=this.showPerspective?r.length():-r.z}e.sort((function(e,t){return t.dist-e.dist}))},m.prototype._initializeRanges=function(){var e=this.dataGroup;this.xRange=e.xRange,this.yRange=e.yRange,this.zRange=e.zRange,this.valueRange=e.valueRange,this.xStep=e.xStep,this.yStep=e.yStep,this.zStep=e.zStep,this.xBarWidth=e.xBarWidth,this.yBarWidth=e.yBarWidth,this.colX=e.colX,this.colY=e.colY,this.colZ=e.colZ,this.colValue=e.colValue,this._setScale()},m.prototype.getDataPoints=function(e){for(var t=[],n=0;n<e.length;n++){var r=new l;r.x=e[n][this.colX]||0,r.y=e[n][this.colY]||0,r.z=e[n][this.colZ]||0,r.data=e[n],void 0!==this.colValue&&(r.value=e[n][this.colValue]||0);var i={};i.point=r,i.bottom=new l(r.x,r.y,this.zRange.min),i.trans=void 0,i.screen=void 0,t.push(i)}return t},m.prototype._getDataPoints=function(e){var t,n,r,i,o=[];if(this.style===m.STYLE.GRID||this.style===m.STYLE.SURFACE){var a=this.dataGroup.getDistinctValues(this.colX,e),s=this.dataGroup.getDistinctValues(this.colY,e);o=this.getDataPoints(e);var l=[];for(r=0;r<o.length;r++){i=o[r];var u=a.indexOf(i.point.x),c=s.indexOf(i.point.y);void 0===l[u]&&(l[u]=[]),l[u][c]=i}for(t=0;t<l.length;t++)for(n=0;n<l[t].length;n++)l[t][n]&&(l[t][n].pointRight=t<l.length-1?l[t+1][n]:void 0,l[t][n].pointTop=n<l[t].length-1?l[t][n+1]:void 0,l[t][n].pointCross=t<l.length-1&&n<l[t].length-1?l[t+1][n+1]:void 0)}else if(this._checkValueField(e),o=this.getDataPoints(e),this.style===m.STYLE.LINE)for(r=0;r<o.length;r++)r>0&&(o[r-1].pointNext=o[r]);return o},m.prototype.create=function(){for(;this.containerElement.hasChildNodes();)this.containerElement.removeChild(this.containerElement.firstChild);this.frame=document.createElement("div"),this.frame.style.position="relative",this.frame.style.overflow="hidden",this.frame.canvas=document.createElement("canvas"),this.frame.canvas.style.position="relative",this.frame.appendChild(this.frame.canvas);var e=document.createElement("DIV");e.style.color="red",e.style.fontWeight="bold",e.style.padding="10px",e.innerHTML="Error: your browser does not support HTML canvas",this.frame.canvas.appendChild(e),this.frame.filter=document.createElement("div"),this.frame.filter.style.position="absolute",this.frame.filter.style.bottom="0px",this.frame.filter.style.left="0px",this.frame.filter.style.width="100%",this.frame.appendChild(this.frame.filter);var t=this;s.addEventListener(this.frame.canvas,"mousedown",(function(e){t._onMouseDown(e)})),s.addEventListener(this.frame.canvas,"touchstart",(function(e){t._onTouchStart(e)})),s.addEventListener(this.frame.canvas,"mousewheel",(function(e){t._onWheel(e)})),s.addEventListener(this.frame.canvas,"mousemove",(function(e){t._onTooltip(e)})),s.addEventListener(this.frame.canvas,"click",(function(e){t._onClick(e)})),this.containerElement.appendChild(this.frame)},m.prototype._setSize=function(e,t){this.frame.style.width=e,this.frame.style.height=t,this._resizeCanvas()},m.prototype._resizeCanvas=function(){this.frame.canvas.style.width="100%",this.frame.canvas.style.height="100%",this.frame.canvas.width=this.frame.canvas.clientWidth,this.frame.canvas.height=this.frame.canvas.clientHeight,this.frame.filter.style.width=this.frame.canvas.clientWidth-20+"px"},m.prototype.animationStart=function(){if(this.animationAutoStart&&this.dataGroup.dataFilter){if(!this.frame.filter||!this.frame.filter.slider)throw new Error("No animation available");this.frame.filter.slider.play()}},m.prototype.animationStop=function(){this.frame.filter&&this.frame.filter.slider&&this.frame.filter.slider.stop()},m.prototype._resizeCenter=function(){"%"===this.xCenter.charAt(this.xCenter.length-1)?this.currentXCenter=parseFloat(this.xCenter)/100*this.frame.canvas.clientWidth:this.currentXCenter=parseFloat(this.xCenter),"%"===this.yCenter.charAt(this.yCenter.length-1)?this.currentYCenter=parseFloat(this.yCenter)/100*(this.frame.canvas.clientHeight-this.frame.filter.clientHeight):this.currentYCenter=parseFloat(this.yCenter)},m.prototype.getCameraPosition=function(){var e=this.camera.getArmRotation();return e.distance=this.camera.getArmLength(),e},m.prototype._readData=function(e){this.dataPoints=this.dataGroup.initializeData(this,e,this.style),this._initializeRanges(),this._redrawFilter()},m.prototype.setData=function(e){null!=e&&(this._readData(e),this.redraw(),this.animationStart())},m.prototype.setOptions=function(e){void 0!==e&&(!0===f.validate(e,g)&&console.log("%cErrors have been found in the supplied options object.",p),this.animationStop(),h.setOptions(e,this),this.setPointDrawingMethod(),this._setSize(this.width,this.height),this.setData(this.dataGroup.getDataTable()),this.animationStart())},m.prototype.setPointDrawingMethod=function(){var e=void 0;switch(this.style){case m.STYLE.BAR:e=m.prototype._redrawBarGraphPoint;break;case m.STYLE.BARCOLOR:e=m.prototype._redrawBarColorGraphPoint;break;case m.STYLE.BARSIZE:e=m.prototype._redrawBarSizeGraphPoint;break;case m.STYLE.DOT:e=m.prototype._redrawDotGraphPoint;break;case m.STYLE.DOTLINE:e=m.prototype._redrawDotLineGraphPoint;break;case m.STYLE.DOTCOLOR:e=m.prototype._redrawDotColorGraphPoint;break;case m.STYLE.DOTSIZE:e=m.prototype._redrawDotSizeGraphPoint;break;case m.STYLE.SURFACE:e=m.prototype._redrawSurfaceGraphPoint;break;case m.STYLE.GRID:e=m.prototype._redrawGridGraphPoint;break;case m.STYLE.LINE:e=m.prototype._redrawLineGraphPoint;break;default:throw new Error("Can not determine point drawing method for graph style '"+this.style+"'")}this._pointDrawingMethod=e},m.prototype.redraw=function(){if(void 0===this.dataPoints)throw new Error("Graph data not initialized");this._resizeCanvas(),this._resizeCenter(),this._redrawSlider(),this._redrawClear(),this._redrawAxis(),this._redrawDataGraph(),this._redrawInfo(),this._redrawLegend()},m.prototype._getContext=function(){var e=this.frame.canvas.getContext("2d");return e.lineJoin="round",e.lineCap="round",e},m.prototype._redrawClear=function(){var e=this.frame.canvas;e.getContext("2d").clearRect(0,0,e.width,e.height)},m.prototype._dotSize=function(){return this.frame.clientWidth*this.dotSizeRatio},m.prototype._getLegendWidth=function(){return this.style===m.STYLE.DOTSIZE?this._dotSize()*this.dotSizeMaxFraction:this.style===m.STYLE.BARSIZE?this.xBarWidth:20},m.prototype._redrawLegend=function(){if(!0===this.showLegend&&this.style!==m.STYLE.LINE&&this.style!==m.STYLE.BARSIZE){var e=this.style===m.STYLE.BARSIZE||this.style===m.STYLE.DOTSIZE,t=this.style===m.STYLE.DOTSIZE||this.style===m.STYLE.DOTCOLOR||this.style===m.STYLE.BARCOLOR,n=Math.max(.25*this.frame.clientHeight,100),r=this.margin,i=this._getLegendWidth(),o=this.frame.clientWidth-this.margin,a=o-i,s=r+n,l=this._getContext();if(l.lineWidth=1,l.font="14px arial",!1===e){var c,h=n;for(c=0;c<h;c++){var f=(c-0)/(h-0)*240,p=this._hsv2rgb(f,1,1);l.strokeStyle=p,l.beginPath(),l.moveTo(a,r+c),l.lineTo(o,r+c),l.stroke()}l.strokeStyle=this.axisColor,l.strokeRect(a,r,i,n)}else{var g;this.style===m.STYLE.DOTSIZE?g=i*(this.dotSizeMinFraction/this.dotSizeMaxFraction):(this.style,m.STYLE.BARSIZE),l.strokeStyle=this.axisColor,l.fillStyle=this.dataColor.fill,l.beginPath(),l.moveTo(a,r),l.lineTo(o,r),l.lineTo(a+g,s),l.lineTo(a,s),l.closePath(),l.fill(),l.stroke()}var v,y,b=t?this.valueRange.min:this.zRange.min,C=t?this.valueRange.max:this.zRange.max,A=new d(b,C,(C-b)/5,!0);for(A.start(!0);!A.end();)c=s-(A.getCurrent()-b)/(C-b)*n,v=new u(a-5,c),y=new u(a,c),this._line(l,v,y),l.textAlign="right",l.textBaseline="middle",l.fillStyle=this.axisColor,l.fillText(A.getCurrent(),a-10,c),A.next();l.textAlign="right",l.textBaseline="top";var _=this.legendLabel;l.fillText(_,o,s+this.margin)}},m.prototype._redrawFilter=function(){var e=this.dataGroup.dataFilter,t=this.frame.filter;if(t.innerHTML="",e){var n={visible:this.showAnimationControls},r=new c(t,n);t.slider=r,t.style.padding="10px",r.setValues(e.values),r.setPlayInterval(this.animationInterval);var i=this;r.setOnChangeCallback((function(){var e=i.dataGroup.dataFilter,t=r.getIndex();e.selectValue(t),i.dataPoints=e._getDataPoints(),i.redraw()}))}else t.slider=void 0},m.prototype._redrawSlider=function(){void 0!==this.frame.filter.slider&&this.frame.filter.slider.redraw()},m.prototype._redrawInfo=function(){var e=this.dataGroup.getInfo();if(void 0!==e){var t=this._getContext();t.font="14px arial",t.lineStyle="gray",t.fillStyle="gray",t.textAlign="left",t.textBaseline="top";var n=this.margin,r=this.margin;t.fillText(e,n,r)}},m.prototype._line=function(e,t,n,r){void 0!==r&&(e.strokeStyle=r),e.beginPath(),e.moveTo(t.x,t.y),e.lineTo(n.x,n.y),e.stroke()},m.prototype.drawAxisLabelX=function(e,t,n,r,i){void 0===i&&(i=0);var o=this._convert3Dto2D(t);Math.cos(2*r)>0?(e.textAlign="center",e.textBaseline="top",o.y+=i):Math.sin(2*r)<0?(e.textAlign="right",e.textBaseline="middle"):(e.textAlign="left",e.textBaseline="middle"),e.fillStyle=this.axisColor,e.fillText(n,o.x,o.y)},m.prototype.drawAxisLabelY=function(e,t,n,r,i){void 0===i&&(i=0);var o=this._convert3Dto2D(t);Math.cos(2*r)<0?(e.textAlign="center",e.textBaseline="top",o.y+=i):Math.sin(2*r)>0?(e.textAlign="right",e.textBaseline="middle"):(e.textAlign="left",e.textBaseline="middle"),e.fillStyle=this.axisColor,e.fillText(n,o.x,o.y)},m.prototype.drawAxisLabelZ=function(e,t,n,r){void 0===r&&(r=0);var i=this._convert3Dto2D(t);e.textAlign="right",e.textBaseline="middle",e.fillStyle=this.axisColor,e.fillText(n,i.x-r,i.y)},m.prototype._line3d=function(e,t,n,r){var i=this._convert3Dto2D(t),o=this._convert3Dto2D(n);this._line(e,i,o,r)},m.prototype._redrawAxis=function(){var e,t,n,r,i,o,a,s,c,h,f=this._getContext();f.font=24/this.camera.getArmLength()+"px arial";var p,g,v,m=.025/this.scale.x,y=.025/this.scale.y,b=5/this.camera.getArmLength(),C=this.camera.getArmRotation().horizontal,A=new u(Math.cos(C),Math.sin(C)),_=this.xRange,I=this.yRange,x=this.zRange;for(f.lineWidth=1,r=void 0===this.defaultXStep,(n=new d(_.min,_.max,this.xStep,r)).start(!0);!n.end();){var w=n.getCurrent();if(this.showGrid?(e=new l(w,I.min,x.min),t=new l(w,I.max,x.min),this._line3d(f,e,t,this.gridColor)):this.showXAxis&&(e=new l(w,I.min,x.min),t=new l(w,I.min+m,x.min),this._line3d(f,e,t,this.axisColor),e=new l(w,I.max,x.min),t=new l(w,I.max-m,x.min),this._line3d(f,e,t,this.axisColor)),this.showXAxis){a=A.x>0?I.min:I.max,p=new l(w,a,x.min);var M="  "+this.xValueLabel(w)+"  ";this.drawAxisLabelX(f,p,M,C,b)}n.next()}for(f.lineWidth=1,r=void 0===this.defaultYStep,(n=new d(I.min,I.max,this.yStep,r)).start(!0);!n.end();){var S=n.getCurrent();if(this.showGrid?(e=new l(_.min,S,x.min),t=new l(_.max,S,x.min),this._line3d(f,e,t,this.gridColor)):this.showYAxis&&(e=new l(_.min,S,x.min),t=new l(_.min+y,S,x.min),this._line3d(f,e,t,this.axisColor),e=new l(_.max,S,x.min),t=new l(_.max-y,S,x.min),this._line3d(f,e,t,this.axisColor)),this.showYAxis){o=A.y>0?_.min:_.max,p=new l(o,S,x.min);var T="  "+this.yValueLabel(S)+"  ";this.drawAxisLabelY(f,p,T,C,b)}n.next()}if(this.showZAxis){for(f.lineWidth=1,r=void 0===this.defaultZStep,(n=new d(x.min,x.max,this.zStep,r)).start(!0),o=A.x>0?_.min:_.max,a=A.y<0?I.min:I.max;!n.end();){var E=n.getCurrent(),O=new l(o,a,E),k=this._convert3Dto2D(O);t=new u(k.x-b,k.y),this._line(f,k,t,this.axisColor);var L=this.zValueLabel(E)+" ";this.drawAxisLabelZ(f,O,L,5),n.next()}f.lineWidth=1,e=new l(o,a,x.min),t=new l(o,a,x.max),this._line3d(f,e,t,this.axisColor)}this.showXAxis&&(f.lineWidth=1,g=new l(_.min,I.min,x.min),v=new l(_.max,I.min,x.min),this._line3d(f,g,v,this.axisColor),g=new l(_.min,I.max,x.min),v=new l(_.max,I.max,x.min),this._line3d(f,g,v,this.axisColor)),this.showYAxis&&(f.lineWidth=1,e=new l(_.min,I.min,x.min),t=new l(_.min,I.max,x.min),this._line3d(f,e,t,this.axisColor),e=new l(_.max,I.min,x.min),t=new l(_.max,I.max,x.min),this._line3d(f,e,t,this.axisColor));var D=this.xLabel;D.length>0&&this.showXAxis&&(h=.1/this.scale.y,o=(_.max+3*_.min)/4,a=A.x>0?I.min-h:I.max+h,i=new l(o,a,x.min),this.drawAxisLabelX(f,i,D,C));var P=this.yLabel;P.length>0&&this.showYAxis&&(c=.1/this.scale.x,o=A.y>0?_.min-c:_.max+c,a=(I.max+3*I.min)/4,i=new l(o,a,x.min),this.drawAxisLabelY(f,i,P,C));var N=this.zLabel;N.length>0&&this.showZAxis&&(o=A.x>0?_.min:_.max,a=A.y<0?I.min:I.max,s=(x.max+3*x.min)/4,i=new l(o,a,s),this.drawAxisLabelZ(f,i,N,30))},m.prototype._hsv2rgb=function(e,t,n){var r,i,o,a,s,l;switch(a=n*t,s=Math.floor(e/60),l=a*(1-Math.abs(e/60%2-1)),s){case 0:r=a,i=l,o=0;break;case 1:r=l,i=a,o=0;break;case 2:r=0,i=a,o=l;break;case 3:r=0,i=l,o=a;break;case 4:r=l,i=0,o=a;break;case 5:r=a,i=0,o=l;break;default:r=0,i=0,o=0}return"RGB("+parseInt(255*r)+","+parseInt(255*i)+","+parseInt(255*o)+")"},m.prototype._getStrokeWidth=function(e){return void 0!==e?this.showPerspective?1/-e.trans.z*this.dataColor.strokeWidth:-this.eye.z/this.camera.getArmLength()*this.dataColor.strokeWidth:this.dataColor.strokeWidth},m.prototype._redrawBar=function(e,t,n,r,i,o){var a,s=this,u=t.point,c=this.zRange.min,d=[{point:new l(u.x-n,u.y-r,u.z)},{point:new l(u.x+n,u.y-r,u.z)},{point:new l(u.x+n,u.y+r,u.z)},{point:new l(u.x-n,u.y+r,u.z)}],h=[{point:new l(u.x-n,u.y-r,c)},{point:new l(u.x+n,u.y-r,c)},{point:new l(u.x+n,u.y+r,c)},{point:new l(u.x-n,u.y+r,c)}];d.forEach((function(e){e.screen=s._convert3Dto2D(e.point)})),h.forEach((function(e){e.screen=s._convert3Dto2D(e.point)}));var f=[{corners:d,center:l.avg(h[0].point,h[2].point)},{corners:[d[0],d[1],h[1],h[0]],center:l.avg(h[1].point,h[0].point)},{corners:[d[1],d[2],h[2],h[1]],center:l.avg(h[2].point,h[1].point)},{corners:[d[2],d[3],h[3],h[2]],center:l.avg(h[3].point,h[2].point)},{corners:[d[3],d[0],h[0],h[3]],center:l.avg(h[0].point,h[3].point)}];t.surfaces=f;for(var p=0;p<f.length;p++){a=f[p];var g=this._convertPointToTranslation(a.center);a.dist=this.showPerspective?g.length():-g.z}f.sort((function(e,t){var n=t.dist-e.dist;return n||(e.corners===d?1:t.corners===d?-1:0)})),e.lineWidth=this._getStrokeWidth(t),e.strokeStyle=o,e.fillStyle=i;for(var v=2;v<f.length;v++)a=f[v],this._polygon(e,a.corners)},m.prototype._polygon=function(e,t,n,r){if(!(t.length<2)){void 0!==n&&(e.fillStyle=n),void 0!==r&&(e.strokeStyle=r),e.beginPath(),e.moveTo(t[0].screen.x,t[0].screen.y);for(var i=1;i<t.length;++i){var o=t[i];e.lineTo(o.screen.x,o.screen.y)}e.closePath(),e.fill(),e.stroke()}},m.prototype._drawCircle=function(e,t,n,r,i){var o=this._calcRadius(t,i);e.lineWidth=this._getStrokeWidth(t),e.strokeStyle=r,e.fillStyle=n,e.beginPath(),e.arc(t.screen.x,t.screen.y,o,0,2*Math.PI,!0),e.fill(),e.stroke()},m.prototype._getColorsRegular=function(e){var t=240*(1-(e.point.z-this.zRange.min)*this.scale.z/this.verticalRatio);return{fill:this._hsv2rgb(t,1,1),border:this._hsv2rgb(t,1,.8)}},m.prototype._getColorsColor=function(e){var t,n;if("string"==typeof e.point.value)t=e.point.value,n=e.point.value;else{var r=240*(1-(e.point.value-this.valueRange.min)*this.scale.value);t=this._hsv2rgb(r,1,1),n=this._hsv2rgb(r,1,.8)}return{fill:t,border:n}},m.prototype._getColorsSize=function(){return{fill:this.dataColor.fill,border:this.dataColor.stroke}},m.prototype._calcRadius=function(e,t){var n;return void 0===t&&(t=this._dotSize()),(n=this.showPerspective?t/-e.trans.z:t*(-this.eye.z/this.camera.getArmLength()))<0&&(n=0),n},m.prototype._redrawBarGraphPoint=function(e,t){var n=this.xBarWidth/2,r=this.yBarWidth/2,i=this._getColorsRegular(t);this._redrawBar(e,t,n,r,i.fill,i.border)},m.prototype._redrawBarColorGraphPoint=function(e,t){var n=this.xBarWidth/2,r=this.yBarWidth/2,i=this._getColorsColor(t);this._redrawBar(e,t,n,r,i.fill,i.border)},m.prototype._redrawBarSizeGraphPoint=function(e,t){var n=(t.point.value-this.valueRange.min)/this.valueRange.range(),r=this.xBarWidth/2*(.8*n+.2),i=this.yBarWidth/2*(.8*n+.2),o=this._getColorsSize();this._redrawBar(e,t,r,i,o.fill,o.border)},m.prototype._redrawDotGraphPoint=function(e,t){var n=this._getColorsRegular(t);this._drawCircle(e,t,n.fill,n.border)},m.prototype._redrawDotLineGraphPoint=function(e,t){var n=this._convert3Dto2D(t.bottom);e.lineWidth=1,this._line(e,n,t.screen,this.gridColor),this._redrawDotGraphPoint(e,t)},m.prototype._redrawDotColorGraphPoint=function(e,t){var n=this._getColorsColor(t);this._drawCircle(e,t,n.fill,n.border)},m.prototype._redrawDotSizeGraphPoint=function(e,t){var n=this._dotSize(),r=(t.point.value-this.valueRange.min)/this.valueRange.range(),i=n*this.dotSizeMinFraction,o=i+(n*this.dotSizeMaxFraction-i)*r,a=this._getColorsSize();this._drawCircle(e,t,a.fill,a.border,o)},m.prototype._redrawSurfaceGraphPoint=function(e,t){var n=t.pointRight,r=t.pointTop,i=t.pointCross;if(void 0!==t&&void 0!==n&&void 0!==r&&void 0!==i){var o,a,s=!0;if(this.showGrayBottom||this.showShadow){var u=l.subtract(i.trans,t.trans),c=l.subtract(r.trans,n.trans),d=l.crossProduct(u,c),h=d.length();s=d.z>0}if(s){var f,p=240*(1-((t.point.z+n.point.z+r.point.z+i.point.z)/4-this.zRange.min)*this.scale.z/this.verticalRatio);this.showShadow?(f=Math.min(1+d.x/h/2,1),a=o=this._hsv2rgb(p,1,f)):(f=1,o=this._hsv2rgb(p,1,f),a=this.axisColor)}else o="gray",a=this.axisColor;e.lineWidth=this._getStrokeWidth(t);var g=[t,n,i,r];this._polygon(e,g,o,a)}},m.prototype._drawGridLine=function(e,t,n){if(void 0!==t&&void 0!==n){var r=240*(1-((t.point.z+n.point.z)/2-this.zRange.min)*this.scale.z/this.verticalRatio);e.lineWidth=2*this._getStrokeWidth(t),e.strokeStyle=this._hsv2rgb(r,1,1),this._line(e,t.screen,n.screen)}},m.prototype._redrawGridGraphPoint=function(e,t){this._drawGridLine(e,t,t.pointRight),this._drawGridLine(e,t,t.pointTop)},m.prototype._redrawLineGraphPoint=function(e,t){void 0!==t.pointNext&&(e.lineWidth=this._getStrokeWidth(t),e.strokeStyle=this.dataColor.stroke,this._line(e,t.screen,t.pointNext.screen))},m.prototype._redrawDataGraph=function(){var e,t=this._getContext();if(!(void 0===this.dataPoints||this.dataPoints.length<=0))for(this._calcTranslations(this.dataPoints),e=0;e<this.dataPoints.length;e++){var n=this.dataPoints[e];this._pointDrawingMethod.call(this,t,n)}},m.prototype._storeMousePosition=function(e){this.startMouseX=y(e),this.startMouseY=b(e),this._startCameraOffset=this.camera.getOffset()},m.prototype._onMouseDown=function(e){if(e=e||window.event,this.leftButtonDown&&this._onMouseUp(e),this.leftButtonDown=e.which?1===e.which:1===e.button,this.leftButtonDown||this.touchDown){this._storeMousePosition(e),this.startStart=new Date(this.start),this.startEnd=new Date(this.end),this.startArmRotation=this.camera.getArmRotation(),this.frame.style.cursor="move";var t=this;this.onmousemove=function(e){t._onMouseMove(e)},this.onmouseup=function(e){t._onMouseUp(e)},s.addEventListener(document,"mousemove",t.onmousemove),s.addEventListener(document,"mouseup",t.onmouseup),s.preventDefault(e)}},m.prototype._onMouseMove=function(e){this.moving=!0,e=e||window.event;var t=parseFloat(y(e))-this.startMouseX,n=parseFloat(b(e))-this.startMouseY;if(e&&!0===e.ctrlKey){var r=.5*this.frame.clientWidth,i=.5*this.frame.clientHeight,o=(this._startCameraOffset.x||0)-t/r*this.camera.armLength*.8,a=(this._startCameraOffset.y||0)+n/i*this.camera.armLength*.8;this.camera.setOffset(o,a),this._storeMousePosition(e)}else{var l=this.startArmRotation.horizontal+t/200,u=this.startArmRotation.vertical+n/200,c=Math.sin(4/360*2*Math.PI);Math.abs(Math.sin(l))<c&&(l=Math.round(l/Math.PI)*Math.PI-.001),Math.abs(Math.cos(l))<c&&(l=(Math.round(l/Math.PI-.5)+.5)*Math.PI-.001),Math.abs(Math.sin(u))<c&&(u=Math.round(u/Math.PI)*Math.PI),Math.abs(Math.cos(u))<c&&(u=(Math.round(u/Math.PI-.5)+.5)*Math.PI),this.camera.setArmRotation(l,u)}this.redraw();var d=this.getCameraPosition();this.emit("cameraPositionChange",d),s.preventDefault(e)},m.prototype._onMouseUp=function(e){this.frame.style.cursor="auto",this.leftButtonDown=!1,s.removeEventListener(document,"mousemove",this.onmousemove),s.removeEventListener(document,"mouseup",this.onmouseup),s.preventDefault(e)},m.prototype._onClick=function(e){if(this.onclick_callback){if(this.moving)this.moving=!1;else{var t=this.frame.getBoundingClientRect(),n=y(e)-t.left,r=b(e)-t.top,i=this._dataPointFromXY(n,r);i&&this.onclick_callback(i.point.data)}s.preventDefault(e)}},m.prototype._onTooltip=function(e){var t=this.frame.getBoundingClientRect(),n=y(e)-t.left,r=b(e)-t.top;if(this.showTooltip)if(this.tooltipTimeout&&clearTimeout(this.tooltipTimeout),this.leftButtonDown)this._hideTooltip();else if(this.tooltip&&this.tooltip.dataPoint){var i=this._dataPointFromXY(n,r);i!==this.tooltip.dataPoint&&(i?this._showTooltip(i):this._hideTooltip())}else{var o=this;this.tooltipTimeout=setTimeout((function(){o.tooltipTimeout=null;var e=o._dataPointFromXY(n,r);e&&o._showTooltip(e)}),300)}},m.prototype._onTouchStart=function(e){this.touchDown=!0;var t=this;this.ontouchmove=function(e){t._onTouchMove(e)},this.ontouchend=function(e){t._onTouchEnd(e)},s.addEventListener(document,"touchmove",t.ontouchmove),s.addEventListener(document,"touchend",t.ontouchend),this._onMouseDown(e)},m.prototype._onTouchMove=function(e){this._onMouseMove(e)},m.prototype._onTouchEnd=function(e){this.touchDown=!1,s.removeEventListener(document,"touchmove",this.ontouchmove),s.removeEventListener(document,"touchend",this.ontouchend),this._onMouseUp(e)},m.prototype._onWheel=function(e){e||(e=window.event);var t=0;if(e.wheelDelta?t=e.wheelDelta/120:e.detail&&(t=-e.detail/3),t){var n=this.camera.getArmLength()*(1-t/10);this.camera.setArmLength(n),this.redraw(),this._hideTooltip()}var r=this.getCameraPosition();this.emit("cameraPositionChange",r),s.preventDefault(e)},m.prototype._insideTriangle=function(e,t){var n=t[0],r=t[1],i=t[2];function o(e){return e>0?1:e<0?-1:0}var a=o((r.x-n.x)*(e.y-n.y)-(r.y-n.y)*(e.x-n.x)),s=o((i.x-r.x)*(e.y-r.y)-(i.y-r.y)*(e.x-r.x)),l=o((n.x-i.x)*(e.y-i.y)-(n.y-i.y)*(e.x-i.x));return!(0!=a&&0!=s&&a!=s||0!=s&&0!=l&&s!=l||0!=a&&0!=l&&a!=l)},m.prototype._dataPointFromXY=function(e,t){var n,r=null,i=null,o=null,a=new u(e,t);if(this.style===m.STYLE.BAR||this.style===m.STYLE.BARCOLOR||this.style===m.STYLE.BARSIZE)for(n=this.dataPoints.length-1;n>=0;n--){var s=(r=this.dataPoints[n]).surfaces;if(s)for(var l=s.length-1;l>=0;l--){var c=s[l].corners,d=[c[0].screen,c[1].screen,c[2].screen],h=[c[2].screen,c[3].screen,c[0].screen];if(this._insideTriangle(a,d)||this._insideTriangle(a,h))return r}}else for(n=0;n<this.dataPoints.length;n++){var f=(r=this.dataPoints[n]).screen;if(f){var p=Math.abs(e-f.x),g=Math.abs(t-f.y),v=Math.sqrt(p*p+g*g);(null===o||v<o)&&v<100&&(o=v,i=r)}}return i},m.prototype.hasBars=function(e){return e==m.STYLE.BAR||e==m.STYLE.BARCOLOR||e==m.STYLE.BARSIZE},m.prototype._showTooltip=function(e){var t,n,r;this.tooltip?(t=this.tooltip.dom.content,n=this.tooltip.dom.line,r=this.tooltip.dom.dot):(t=document.createElement("div"),(0,o.default)(t.style,{},this.tooltipStyle.content),t.style.position="absolute",n=document.createElement("div"),(0,o.default)(n.style,{},this.tooltipStyle.line),n.style.position="absolute",r=document.createElement("div"),(0,o.default)(r.style,{},this.tooltipStyle.dot),r.style.position="absolute",this.tooltip={dataPoint:null,dom:{content:t,line:n,dot:r}}),this._hideTooltip(),this.tooltip.dataPoint=e,"function"==typeof this.showTooltip?t.innerHTML=this.showTooltip(e.point):t.innerHTML="<table><tr><td>"+this.xLabel+":</td><td>"+e.point.x+"</td></tr><tr><td>"+this.yLabel+":</td><td>"+e.point.y+"</td></tr><tr><td>"+this.zLabel+":</td><td>"+e.point.z+"</td></tr></table>",t.style.left="0",t.style.top="0",this.frame.appendChild(t),this.frame.appendChild(n),this.frame.appendChild(r);var i=t.offsetWidth,a=t.offsetHeight,s=n.offsetHeight,l=r.offsetWidth,u=r.offsetHeight,c=e.screen.x-i/2;c=Math.min(Math.max(c,10),this.frame.clientWidth-10-i),n.style.left=e.screen.x+"px",n.style.top=e.screen.y-s+"px",t.style.left=c+"px",t.style.top=e.screen.y-s-a+"px",r.style.left=e.screen.x-l/2+"px",r.style.top=e.screen.y-u/2+"px"},m.prototype._hideTooltip=function(){if(this.tooltip)for(var e in this.tooltip.dataPoint=null,this.tooltip.dom)if(this.tooltip.dom.hasOwnProperty(e)){var t=this.tooltip.dom[e];t&&t.parentNode&&t.parentNode.removeChild(t)}},m.prototype.setCameraPosition=function(e){h.setCameraPosition(e,this),this.redraw()},m.prototype.setSize=function(e,t){this._setSize(e,t),this.redraw()},e.exports=m},function(e,t,n){n(163),e.exports=n(7).Object.assign},function(e,t,n){var r=n(17);r(r.S+r.F,"Object",{assign:n(164)})},function(e,t,n){var r=n(33),i=n(63),o=n(42),a=n(41),s=n(78),l=Object.assign;e.exports=!l||n(28)((function(){var e={},t={},n=Symbol(),r="abcdefghijklmnopqrst";return e[n]=7,r.split("").forEach((function(e){t[e]=e})),7!=l({},e)[n]||Object.keys(l({},t)).join("")!=r}))?function(e,t){for(var n=a(e),l=arguments.length,u=1,c=i.f,d=o.f;l>u;)for(var h,f=s(arguments[u++]),p=c?r(f).concat(c(f)):r(f),g=p.length,v=0;g>v;)d.call(f,h=p[v++])&&(n[h]=f[h]);return n}:l},function(e,t,n){e.exports={default:n(166),__esModule:!0}},function(e,t,n){n(167),e.exports=n(7).Math.sign},function(e,t,n){var r=n(17);r(r.S,"Math",{sign:n(168)})},function(e,t){e.exports=Math.sign||function(e){return 0==(e=+e)||e!=e?e:e<0?-1:1}},function(e,t,n){e.exports={default:n(170),__esModule:!0}},function(e,t,n){n(171);var r=n(7).Object;e.exports=function(e,t,n){return r.defineProperty(e,t,n)}},function(e,t,n){var r=n(17);r(r.S+r.F*!n(21),"Object",{defineProperty:n(20).f})},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r="string",i="boolean",o="number",a={fill:{string:r},stroke:{string:r},strokeWidth:{number:o},__type__:{string:r,object:"object",undefined:"undefined"}},s={animationAutoStart:{boolean:i,undefined:"undefined"},animationInterval:{number:o},animationPreload:{boolean:i},axisColor:{string:r},backgroundColor:a,xBarWidth:{number:o,undefined:"undefined"},yBarWidth:{number:o,undefined:"undefined"},cameraPosition:{distance:{number:o},horizontal:{number:o},vertical:{number:o},__type__:{object:"object"}},xCenter:{string:r},yCenter:{string:r},dataColor:a,dotSizeMinFraction:{number:o},dotSizeMaxFraction:{number:o},dotSizeRatio:{number:o},filterLabel:{string:r},gridColor:{string:r},onclick:{function:"function"},keepAspectRatio:{boolean:i},xLabel:{string:r},yLabel:{string:r},zLabel:{string:r},legendLabel:{string:r},xMin:{number:o,undefined:"undefined"},yMin:{number:o,undefined:"undefined"},zMin:{number:o,undefined:"undefined"},xMax:{number:o,undefined:"undefined"},yMax:{number:o,undefined:"undefined"},zMax:{number:o,undefined:"undefined"},showAnimationControls:{boolean:i,undefined:"undefined"},showGrid:{boolean:i},showLegend:{boolean:i,undefined:"undefined"},showPerspective:{boolean:i},showShadow:{boolean:i},showXAxis:{boolean:i},showYAxis:{boolean:i},showZAxis:{boolean:i},xStep:{number:o,undefined:"undefined"},yStep:{number:o,undefined:"undefined"},zStep:{number:o,undefined:"undefined"},style:{number:o,string:["bar","bar-color","bar-size","dot","dot-line","dot-color","dot-size","line","grid","surface"]},tooltip:{boolean:i,function:"function"},tooltipStyle:{content:{color:{string:r},background:{string:r},border:{string:r},borderRadius:{string:r},boxShadow:{string:r},padding:{string:r},__type__:{object:"object"}},line:{borderLeft:{string:r},height:{string:r},width:{string:r},__type__:{object:"object"}},dot:{border:{string:r},borderRadius:{string:r},height:{string:r},width:{string:r},__type__:{object:"object"}},__type__:{object:"object"}},xValueLabel:{function:"function"},yValueLabel:{function:"function"},zValueLabel:{function:"function"},valueMax:{number:o,undefined:"undefined"},valueMin:{number:o,undefined:"undefined"},verticalRatio:{number:o},height:{string:r},width:{string:r},__type__:{object:"object"}};t.allOptions=s},function(e,t,n){var r=n(11),i=n(12),o=n(174),a=n(96),s=n(94),l=n(34);function u(){this.dataTable=null}u.prototype.initializeData=function(e,t,n){if(void 0!==t){var o;if(Array.isArray(t)&&(t=new r(t)),!(t instanceof r||t instanceof i))throw new Error("Array, DataSet, or DataView expected");if(0!=(o=t.get()).length){this.style=n,this.dataSet&&this.dataSet.off("*",this._onChange),this.dataSet=t,this.dataTable=o;var s=this;this._onChange=function(){e.setData(s.dataSet)},this.dataSet.on("*",this._onChange),this.colX="x",this.colY="y",this.colZ="z";var l=e.hasBars(n);if(l&&(void 0!==e.defaultXBarWidth?this.xBarWidth=e.defaultXBarWidth:this.xBarWidth=this.getSmallestDifference(o,this.colX)||1,void 0!==e.defaultYBarWidth?this.yBarWidth=e.defaultYBarWidth:this.yBarWidth=this.getSmallestDifference(o,this.colY)||1),this._initializeRange(o,this.colX,e,l),this._initializeRange(o,this.colY,e,l),this._initializeRange(o,this.colZ,e,!1),o[0].hasOwnProperty("style")){this.colValue="style";var u=this.getColumnRange(o,this.colValue);this._setRangeDefaults(u,e.defaultValueMin,e.defaultValueMax),this.valueRange=u}return this.getDataTable()[0].hasOwnProperty("filter")&&void 0===this.dataFilter&&(this.dataFilter=new a(this,"filter",e),this.dataFilter.setOnLoadCallback((function(){e.redraw()}))),this.dataFilter?this.dataFilter._getDataPoints():this._getDataPoints(this.getDataTable())}}},u.prototype._collectRangeSettings=function(e,t){if(-1==["x","y","z"].indexOf(e))throw new Error("Column '"+e+"' invalid");var n=e.toUpperCase();return{barWidth:this[e+"BarWidth"],min:t["default"+n+"Min"],max:t["default"+n+"Max"],step:t["default"+n+"Step"],range_label:e+"Range",step_label:e+"Step"}},u.prototype._initializeRange=function(e,t,n,r){var i=this._collectRangeSettings(t,n),o=this.getColumnRange(e,t);r&&"z"!=t&&o.expand(i.barWidth/2),this._setRangeDefaults(o,i.min,i.max),this[i.range_label]=o,this[i.step_label]=void 0!==i.step?i.step:o.range()/5},u.prototype.getDistinctValues=function(e,t){void 0===t&&(t=this.dataTable);for(var n=[],r=0;r<t.length;r++){var i=t[r][e]||0;-1===n.indexOf(i)&&n.push(i)}return n.sort((function(e,t){return e-t}))},u.prototype.getSmallestDifference=function(e,t){for(var n=this.getDistinctValues(e,t),r=null,i=1;i<n.length;i++){var o=n[i]-n[i-1];(null==r||r>o)&&(r=o)}return r},u.prototype.getColumnRange=function(e,t){for(var n=new o,r=0;r<e.length;r++){var i=e[r][t];n.adjust(i)}return n},u.prototype.getNumberOfRows=function(){return this.dataTable.length},u.prototype._setRangeDefaults=function(e,t,n){void 0!==t&&(e.min=t),void 0!==n&&(e.max=n),e.max<=e.min&&(e.max=e.min+1)},u.prototype.getDataTable=function(){return this.dataTable},u.prototype.getDataSet=function(){return this.dataSet},u.prototype.getDataPoints=function(e){for(var t=[],n=0;n<e.length;n++){var r=new l;r.x=e[n][this.colX]||0,r.y=e[n][this.colY]||0,r.z=e[n][this.colZ]||0,r.data=e[n],void 0!==this.colValue&&(r.value=e[n][this.colValue]||0);var i={};i.point=r,i.bottom=new l(r.x,r.y,this.zRange.min),i.trans=void 0,i.screen=void 0,t.push(i)}return t},u.prototype.initDataAsMatrix=function(e){var t,n,r,i,o=this.getDistinctValues(this.colX,e),a=this.getDistinctValues(this.colY,e),s=this.getDataPoints(e),l=[];for(r=0;r<s.length;r++){i=s[r];var u=o.indexOf(i.point.x),c=a.indexOf(i.point.y);void 0===l[u]&&(l[u]=[]),l[u][c]=i}for(t=0;t<l.length;t++)for(n=0;n<l[t].length;n++)l[t][n]&&(l[t][n].pointRight=t<l.length-1?l[t+1][n]:void 0,l[t][n].pointTop=n<l[t].length-1?l[t][n+1]:void 0,l[t][n].pointCross=t<l.length-1&&n<l[t].length-1?l[t+1][n+1]:void 0);return s},u.prototype.getInfo=function(){var e=this.dataFilter;if(e)return e.getLabel()+": "+e.getSelectedValue()},u.prototype.reload=function(){this.dataTable&&this.setData(this.dataTable)},u.prototype._getDataPoints=function(e){var t=[];if(this.style===s.STYLE.GRID||this.style===s.STYLE.SURFACE)t=this.initDataAsMatrix(e);else if(this._checkValueField(e),t=this.getDataPoints(e),this.style===s.STYLE.LINE)for(var n=0;n<t.length;n++)n>0&&(t[n-1].pointNext=t[n]);return t},u.prototype._checkValueField=function(e){if(this.style===s.STYLE.BARCOLOR||this.style===s.STYLE.BARSIZE||this.style===s.STYLE.DOTCOLOR||this.style===s.STYLE.DOTSIZE){if(void 0===this.colValue)throw new Error("Expected data to have  field 'style'  for graph style '"+this.style+"'");if(void 0===e[0][this.colValue])throw new Error("Expected data to have  field '"+this.colValue+"'  for graph style '"+this.style+"'")}},e.exports=u},function(e,t,n){function r(){this.min=void 0,this.max=void 0}r.prototype.adjust=function(e){void 0!==e&&((void 0===this.min||this.min>e)&&(this.min=e),(void 0===this.max||this.max<e)&&(this.max=e))},r.prototype.combine=function(e){this.add(e.min),this.add(e.max)},r.prototype.expand=function(e){if(void 0!==e){var t=this.min-e,n=this.max+e;if(t>n)throw new Error("Passed expansion value makes range invalid");this.min=t,this.max=n}},r.prototype.range=function(){return this.max-this.min},r.prototype.center=function(){return(this.min+this.max)/2},e.exports=r},function(e,t,n){var r,i,o;i=[],void 0===(o="function"==typeof(r=function(){var e=null;return function t(n,r){var i=r||{preventDefault:!1};if(n.Manager){var o=n,a=function(e,n){var r=Object.create(i);return n&&o.assign(r,n),t(new o(e,r),r)};return o.assign(a,o),a.Manager=function(e,n){var r=Object.create(i);return n&&o.assign(r,n),t(new o.Manager(e,r),r)},a}var s=Object.create(n),l=n.element;function u(e){return e.match(/[^ ]+/g)}function c(t){if("hammer.input"!==t.type){if(t.srcEvent._handled||(t.srcEvent._handled={}),t.srcEvent._handled[t.type])return;t.srcEvent._handled[t.type]=!0}var n=!1;t.stopPropagation=function(){n=!0};var r=t.srcEvent.stopPropagation.bind(t.srcEvent);"function"==typeof r&&(t.srcEvent.stopPropagation=function(){r(),t.stopPropagation()}),t.firstTarget=e;for(var i=e;i&&!n;){var o=i.hammer;if(o)for(var a,s=0;s<o.length;s++)if(a=o[s]._handlers[t.type])for(var l=0;l<a.length&&!n;l++)a[l](t);i=i.parentNode}}return l.hammer||(l.hammer=[]),l.hammer.push(s),n.on("hammer.input",(function(t){!0!==i.preventDefault&&i.preventDefault!==t.pointerType||t.preventDefault(),t.isFirst&&(e=t.target)})),s._handlers={},s.on=function(e,t){return u(e).forEach((function(e){var r=s._handlers[e];r||(s._handlers[e]=r=[],n.on(e,c)),r.push(t)})),s},s.off=function(e,t){return u(e).forEach((function(e){var r=s._handlers[e];r&&((r=t?r.filter((function(e){return e!==t})):[]).length>0?s._handlers[e]=r:(n.off(e,c),delete s._handlers[e]))})),s},s.emit=function(t,r){e=r.target,n.emit(t,r)},s.destroy=function(){var e=n.element.hammer,t=e.indexOf(s);-1!==t&&e.splice(t,1),e.length||delete n.element.hammer,s._handlers={},n.destroy()},s}})?r.apply(t,i):r)||(e.exports=o)},function(e,t,n){var r;
/*! Hammer.JS - v2.0.7 - 2016-04-22
 * http://hammerjs.github.io/
 *
 * Copyright (c) 2016 Jorik Tangelder;
 * Licensed under the MIT license */!function(i,o,a,s){var l,u=["","webkit","Moz","MS","ms","o"],c=o.createElement("div"),d=Math.round,h=Math.abs,f=Date.now;function p(e,t,n){return setTimeout(A(e,n),t)}function g(e,t,n){return!!Array.isArray(e)&&(v(e,n[t],n),!0)}function v(e,t,n){var r;if(e)if(e.forEach)e.forEach(t,n);else if(void 0!==e.length)for(r=0;r<e.length;)t.call(n,e[r],r,e),r++;else for(r in e)e.hasOwnProperty(r)&&t.call(n,e[r],r,e)}function m(e,t,n){var r="DEPRECATED METHOD: "+t+"\n"+n+" AT \n";return function(){var t=new Error("get-stack-trace"),n=t&&t.stack?t.stack.replace(/^[^\(]+?[\n$]/gm,"").replace(/^\s+at\s+/gm,"").replace(/^Object.<anonymous>\s*\(/gm,"{anonymous}()@"):"Unknown Stack Trace",o=i.console&&(i.console.warn||i.console.log);return o&&o.call(i.console,r,n),e.apply(this,arguments)}}l="function"!=typeof Object.assign?function(e){if(null==e)throw new TypeError("Cannot convert undefined or null to object");for(var t=Object(e),n=1;n<arguments.length;n++){var r=arguments[n];if(null!=r)for(var i in r)r.hasOwnProperty(i)&&(t[i]=r[i])}return t}:Object.assign;var y=m((function(e,t,n){for(var r=Object.keys(t),i=0;i<r.length;)(!n||n&&void 0===e[r[i]])&&(e[r[i]]=t[r[i]]),i++;return e}),"extend","Use `assign`."),b=m((function(e,t){return y(e,t,!0)}),"merge","Use `assign`.");function C(e,t,n){var r,i=t.prototype;(r=e.prototype=Object.create(i)).constructor=e,r._super=i,n&&l(r,n)}function A(e,t){return function(){return e.apply(t,arguments)}}function _(e,t){return"function"==typeof e?e.apply(t&&t[0]||void 0,t):e}function I(e,t){return void 0===e?t:e}function x(e,t,n){v(T(t),(function(t){e.addEventListener(t,n,!1)}))}function w(e,t,n){v(T(t),(function(t){e.removeEventListener(t,n,!1)}))}function M(e,t){for(;e;){if(e==t)return!0;e=e.parentNode}return!1}function S(e,t){return e.indexOf(t)>-1}function T(e){return e.trim().split(/\s+/g)}function E(e,t,n){if(e.indexOf&&!n)return e.indexOf(t);for(var r=0;r<e.length;){if(n&&e[r][n]==t||!n&&e[r]===t)return r;r++}return-1}function O(e){return Array.prototype.slice.call(e,0)}function k(e,t,n){for(var r=[],i=[],o=0;o<e.length;){var a=t?e[o][t]:e[o];E(i,a)<0&&r.push(e[o]),i[o]=a,o++}return n&&(r=t?r.sort((function(e,n){return e[t]>n[t]})):r.sort()),r}function L(e,t){for(var n,r,i=t[0].toUpperCase()+t.slice(1),o=0;o<u.length;){if((r=(n=u[o])?n+i:t)in e)return r;o++}}var D=1;function P(e){var t=e.ownerDocument||e;return t.defaultView||t.parentWindow||i}var N="ontouchstart"in i,j=void 0!==L(i,"PointerEvent"),R=N&&/mobile|tablet|ip(ad|hone|od)|android/i.test(navigator.userAgent),z=["x","y"],F=["clientX","clientY"];function B(e,t){var n=this;this.manager=e,this.callback=t,this.element=e.element,this.target=e.options.inputTarget,this.domHandler=function(t){_(e.options.enable,[e])&&n.handler(t)},this.init()}function H(e,t,n){var r=n.pointers.length,i=n.changedPointers.length,o=1&t&&r-i==0,a=12&t&&r-i==0;n.isFirst=!!o,n.isFinal=!!a,o&&(e.session={}),n.eventType=t,function(e,t){var n=e.session,r=t.pointers,i=r.length;n.firstInput||(n.firstInput=U(t)),i>1&&!n.firstMultiple?n.firstMultiple=U(t):1===i&&(n.firstMultiple=!1);var o=n.firstInput,a=n.firstMultiple,s=a?a.center:o.center,l=t.center=V(r);t.timeStamp=f(),t.deltaTime=t.timeStamp-o.timeStamp,t.angle=X(s,l),t.distance=G(s,l),function(e,t){var n=t.center,r=e.offsetDelta||{},i=e.prevDelta||{},o=e.prevInput||{};1!==t.eventType&&4!==o.eventType||(i=e.prevDelta={x:o.deltaX||0,y:o.deltaY||0},r=e.offsetDelta={x:n.x,y:n.y}),t.deltaX=i.x+(n.x-r.x),t.deltaY=i.y+(n.y-r.y)}(n,t),t.offsetDirection=W(t.deltaX,t.deltaY);var u,c,d=Y(t.deltaTime,t.deltaX,t.deltaY);t.overallVelocityX=d.x,t.overallVelocityY=d.y,t.overallVelocity=h(d.x)>h(d.y)?d.x:d.y,t.scale=a?(u=a.pointers,G((c=r)[0],c[1],F)/G(u[0],u[1],F)):1,t.rotation=a?function(e,t){return X(t[1],t[0],F)+X(e[1],e[0],F)}(a.pointers,r):0,t.maxPointers=n.prevInput?t.pointers.length>n.prevInput.maxPointers?t.pointers.length:n.prevInput.maxPointers:t.pointers.length,function(e,t){var n,r,i,o,a=e.lastInterval||t,s=t.timeStamp-a.timeStamp;if(8!=t.eventType&&(s>25||void 0===a.velocity)){var l=t.deltaX-a.deltaX,u=t.deltaY-a.deltaY,c=Y(s,l,u);r=c.x,i=c.y,n=h(c.x)>h(c.y)?c.x:c.y,o=W(l,u),e.lastInterval=t}else n=a.velocity,r=a.velocityX,i=a.velocityY,o=a.direction;t.velocity=n,t.velocityX=r,t.velocityY=i,t.direction=o}(n,t);var p=e.element;M(t.srcEvent.target,p)&&(p=t.srcEvent.target),t.target=p}(e,n),e.emit("hammer.input",n),e.recognize(n),e.session.prevInput=n}function U(e){for(var t=[],n=0;n<e.pointers.length;)t[n]={clientX:d(e.pointers[n].clientX),clientY:d(e.pointers[n].clientY)},n++;return{timeStamp:f(),pointers:t,center:V(t),deltaX:e.deltaX,deltaY:e.deltaY}}function V(e){var t=e.length;if(1===t)return{x:d(e[0].clientX),y:d(e[0].clientY)};for(var n=0,r=0,i=0;i<t;)n+=e[i].clientX,r+=e[i].clientY,i++;return{x:d(n/t),y:d(r/t)}}function Y(e,t,n){return{x:t/e||0,y:n/e||0}}function W(e,t){return e===t?1:h(e)>=h(t)?e<0?2:4:t<0?8:16}function G(e,t,n){n||(n=z);var r=t[n[0]]-e[n[0]],i=t[n[1]]-e[n[1]];return Math.sqrt(r*r+i*i)}function X(e,t,n){n||(n=z);var r=t[n[0]]-e[n[0]],i=t[n[1]]-e[n[1]];return 180*Math.atan2(i,r)/Math.PI}B.prototype={handler:function(){},init:function(){this.evEl&&x(this.element,this.evEl,this.domHandler),this.evTarget&&x(this.target,this.evTarget,this.domHandler),this.evWin&&x(P(this.element),this.evWin,this.domHandler)},destroy:function(){this.evEl&&w(this.element,this.evEl,this.domHandler),this.evTarget&&w(this.target,this.evTarget,this.domHandler),this.evWin&&w(P(this.element),this.evWin,this.domHandler)}};var q={mousedown:1,mousemove:2,mouseup:4};function Z(){this.evEl="mousedown",this.evWin="mousemove mouseup",this.pressed=!1,B.apply(this,arguments)}C(Z,B,{handler:function(e){var t=q[e.type];1&t&&0===e.button&&(this.pressed=!0),2&t&&1!==e.which&&(t=4),this.pressed&&(4&t&&(this.pressed=!1),this.callback(this.manager,t,{pointers:[e],changedPointers:[e],pointerType:"mouse",srcEvent:e}))}});var Q={pointerdown:1,pointermove:2,pointerup:4,pointercancel:8,pointerout:8},K={2:"touch",3:"pen",4:"mouse",5:"kinect"},J="pointerdown",$="pointermove pointerup pointercancel";function ee(){this.evEl=J,this.evWin=$,B.apply(this,arguments),this.store=this.manager.session.pointerEvents=[]}i.MSPointerEvent&&!i.PointerEvent&&(J="MSPointerDown",$="MSPointerMove MSPointerUp MSPointerCancel"),C(ee,B,{handler:function(e){var t=this.store,n=!1,r=e.type.toLowerCase().replace("ms",""),i=Q[r],o=K[e.pointerType]||e.pointerType,a="touch"==o,s=E(t,e.pointerId,"pointerId");1&i&&(0===e.button||a)?s<0&&(t.push(e),s=t.length-1):12&i&&(n=!0),s<0||(t[s]=e,this.callback(this.manager,i,{pointers:t,changedPointers:[e],pointerType:o,srcEvent:e}),n&&t.splice(s,1))}});var te={touchstart:1,touchmove:2,touchend:4,touchcancel:8};function ne(){this.evTarget="touchstart",this.evWin="touchstart touchmove touchend touchcancel",this.started=!1,B.apply(this,arguments)}function re(e,t){var n=O(e.touches),r=O(e.changedTouches);return 12&t&&(n=k(n.concat(r),"identifier",!0)),[n,r]}C(ne,B,{handler:function(e){var t=te[e.type];if(1===t&&(this.started=!0),this.started){var n=re.call(this,e,t);12&t&&n[0].length-n[1].length==0&&(this.started=!1),this.callback(this.manager,t,{pointers:n[0],changedPointers:n[1],pointerType:"touch",srcEvent:e})}}});var ie={touchstart:1,touchmove:2,touchend:4,touchcancel:8};function oe(){this.evTarget="touchstart touchmove touchend touchcancel",this.targetIds={},B.apply(this,arguments)}function ae(e,t){var n=O(e.touches),r=this.targetIds;if(3&t&&1===n.length)return r[n[0].identifier]=!0,[n,n];var i,o,a=O(e.changedTouches),s=[],l=this.target;if(o=n.filter((function(e){return M(e.target,l)})),1===t)for(i=0;i<o.length;)r[o[i].identifier]=!0,i++;for(i=0;i<a.length;)r[a[i].identifier]&&s.push(a[i]),12&t&&delete r[a[i].identifier],i++;return s.length?[k(o.concat(s),"identifier",!0),s]:void 0}function se(){B.apply(this,arguments);var e=A(this.handler,this);this.touch=new oe(this.manager,e),this.mouse=new Z(this.manager,e),this.primaryTouch=null,this.lastTouches=[]}function le(e,t){1&e?(this.primaryTouch=t.changedPointers[0].identifier,ue.call(this,t)):12&e&&ue.call(this,t)}function ue(e){var t=e.changedPointers[0];if(t.identifier===this.primaryTouch){var n={x:t.clientX,y:t.clientY};this.lastTouches.push(n);var r=this.lastTouches;setTimeout((function(){var e=r.indexOf(n);e>-1&&r.splice(e,1)}),2500)}}function ce(e){for(var t=e.srcEvent.clientX,n=e.srcEvent.clientY,r=0;r<this.lastTouches.length;r++){var i=this.lastTouches[r],o=Math.abs(t-i.x),a=Math.abs(n-i.y);if(o<=25&&a<=25)return!0}return!1}C(oe,B,{handler:function(e){var t=ie[e.type],n=ae.call(this,e,t);n&&this.callback(this.manager,t,{pointers:n[0],changedPointers:n[1],pointerType:"touch",srcEvent:e})}}),C(se,B,{handler:function(e,t,n){var r="touch"==n.pointerType,i="mouse"==n.pointerType;if(!(i&&n.sourceCapabilities&&n.sourceCapabilities.firesTouchEvents)){if(r)le.call(this,t,n);else if(i&&ce.call(this,n))return;this.callback(e,t,n)}},destroy:function(){this.touch.destroy(),this.mouse.destroy()}});var de=L(c.style,"touchAction"),he=void 0!==de,fe=function(){if(!he)return!1;var e={},t=i.CSS&&i.CSS.supports;return["auto","manipulation","pan-y","pan-x","pan-x pan-y","none"].forEach((function(n){e[n]=!t||i.CSS.supports("touch-action",n)})),e}();function pe(e,t){this.manager=e,this.set(t)}function ge(e){this.options=l({},this.defaults,e||{}),this.id=D++,this.manager=null,this.options.enable=I(this.options.enable,!0),this.state=1,this.simultaneous={},this.requireFail=[]}function ve(e){return 16&e?"cancel":8&e?"end":4&e?"move":2&e?"start":""}function me(e){return 16==e?"down":8==e?"up":2==e?"left":4==e?"right":""}function ye(e,t){var n=t.manager;return n?n.get(e):e}function be(){ge.apply(this,arguments)}function Ce(){be.apply(this,arguments),this.pX=null,this.pY=null}function Ae(){be.apply(this,arguments)}function _e(){ge.apply(this,arguments),this._timer=null,this._input=null}function Ie(){be.apply(this,arguments)}function xe(){be.apply(this,arguments)}function we(){ge.apply(this,arguments),this.pTime=!1,this.pCenter=!1,this._timer=null,this._input=null,this.count=0}function Me(e,t){return(t=t||{}).recognizers=I(t.recognizers,Me.defaults.preset),new Se(e,t)}function Se(e,t){var n;this.options=l({},Me.defaults,t||{}),this.options.inputTarget=this.options.inputTarget||e,this.handlers={},this.session={},this.recognizers=[],this.oldCssProps={},this.element=e,this.input=new((n=this).options.inputClass||(j?ee:R?oe:N?se:Z))(n,H),this.touchAction=new pe(this,this.options.touchAction),Te(this,!0),v(this.options.recognizers,(function(e){var t=this.add(new e[0](e[1]));e[2]&&t.recognizeWith(e[2]),e[3]&&t.requireFailure(e[3])}),this)}function Te(e,t){var n,r=e.element;r.style&&(v(e.options.cssProps,(function(i,o){n=L(r.style,o),t?(e.oldCssProps[n]=r.style[n],r.style[n]=i):r.style[n]=e.oldCssProps[n]||""})),t||(e.oldCssProps={}))}pe.prototype={set:function(e){"compute"==e&&(e=this.compute()),he&&this.manager.element.style&&fe[e]&&(this.manager.element.style[de]=e),this.actions=e.toLowerCase().trim()},update:function(){this.set(this.manager.options.touchAction)},compute:function(){var e=[];return v(this.manager.recognizers,(function(t){_(t.options.enable,[t])&&(e=e.concat(t.getTouchAction()))})),function(e){if(S(e,"none"))return"none";var t=S(e,"pan-x"),n=S(e,"pan-y");return t&&n?"none":t||n?t?"pan-x":"pan-y":S(e,"manipulation")?"manipulation":"auto"}(e.join(" "))},preventDefaults:function(e){var t=e.srcEvent,n=e.offsetDirection;if(this.manager.session.prevented)t.preventDefault();else{var r=this.actions,i=S(r,"none")&&!fe.none,o=S(r,"pan-y")&&!fe["pan-y"],a=S(r,"pan-x")&&!fe["pan-x"];if(i){var s=1===e.pointers.length,l=e.distance<2,u=e.deltaTime<250;if(s&&l&&u)return}if(!a||!o)return i||o&&6&n||a&&24&n?this.preventSrc(t):void 0}},preventSrc:function(e){this.manager.session.prevented=!0,e.preventDefault()}},ge.prototype={defaults:{},set:function(e){return l(this.options,e),this.manager&&this.manager.touchAction.update(),this},recognizeWith:function(e){if(g(e,"recognizeWith",this))return this;var t=this.simultaneous;return t[(e=ye(e,this)).id]||(t[e.id]=e,e.recognizeWith(this)),this},dropRecognizeWith:function(e){return g(e,"dropRecognizeWith",this)||(e=ye(e,this),delete this.simultaneous[e.id]),this},requireFailure:function(e){if(g(e,"requireFailure",this))return this;var t=this.requireFail;return-1===E(t,e=ye(e,this))&&(t.push(e),e.requireFailure(this)),this},dropRequireFailure:function(e){if(g(e,"dropRequireFailure",this))return this;e=ye(e,this);var t=E(this.requireFail,e);return t>-1&&this.requireFail.splice(t,1),this},hasRequireFailures:function(){return this.requireFail.length>0},canRecognizeWith:function(e){return!!this.simultaneous[e.id]},emit:function(e){var t=this,n=this.state;function r(n){t.manager.emit(n,e)}n<8&&r(t.options.event+ve(n)),r(t.options.event),e.additionalEvent&&r(e.additionalEvent),n>=8&&r(t.options.event+ve(n))},tryEmit:function(e){if(this.canEmit())return this.emit(e);this.state=32},canEmit:function(){for(var e=0;e<this.requireFail.length;){if(!(33&this.requireFail[e].state))return!1;e++}return!0},recognize:function(e){var t=l({},e);if(!_(this.options.enable,[this,t]))return this.reset(),void(this.state=32);56&this.state&&(this.state=1),this.state=this.process(t),30&this.state&&this.tryEmit(t)},process:function(e){},getTouchAction:function(){},reset:function(){}},C(be,ge,{defaults:{pointers:1},attrTest:function(e){var t=this.options.pointers;return 0===t||e.pointers.length===t},process:function(e){var t=this.state,n=e.eventType,r=6&t,i=this.attrTest(e);return r&&(8&n||!i)?16|t:r||i?4&n?8|t:2&t?4|t:2:32}}),C(Ce,be,{defaults:{event:"pan",threshold:10,pointers:1,direction:30},getTouchAction:function(){var e=this.options.direction,t=[];return 6&e&&t.push("pan-y"),24&e&&t.push("pan-x"),t},directionTest:function(e){var t=this.options,n=!0,r=e.distance,i=e.direction,o=e.deltaX,a=e.deltaY;return i&t.direction||(6&t.direction?(i=0===o?1:o<0?2:4,n=o!=this.pX,r=Math.abs(e.deltaX)):(i=0===a?1:a<0?8:16,n=a!=this.pY,r=Math.abs(e.deltaY))),e.direction=i,n&&r>t.threshold&&i&t.direction},attrTest:function(e){return be.prototype.attrTest.call(this,e)&&(2&this.state||!(2&this.state)&&this.directionTest(e))},emit:function(e){this.pX=e.deltaX,this.pY=e.deltaY;var t=me(e.direction);t&&(e.additionalEvent=this.options.event+t),this._super.emit.call(this,e)}}),C(Ae,be,{defaults:{event:"pinch",threshold:0,pointers:2},getTouchAction:function(){return["none"]},attrTest:function(e){return this._super.attrTest.call(this,e)&&(Math.abs(e.scale-1)>this.options.threshold||2&this.state)},emit:function(e){if(1!==e.scale){var t=e.scale<1?"in":"out";e.additionalEvent=this.options.event+t}this._super.emit.call(this,e)}}),C(_e,ge,{defaults:{event:"press",pointers:1,time:251,threshold:9},getTouchAction:function(){return["auto"]},process:function(e){var t=this.options,n=e.pointers.length===t.pointers,r=e.distance<t.threshold,i=e.deltaTime>t.time;if(this._input=e,!r||!n||12&e.eventType&&!i)this.reset();else if(1&e.eventType)this.reset(),this._timer=p((function(){this.state=8,this.tryEmit()}),t.time,this);else if(4&e.eventType)return 8;return 32},reset:function(){clearTimeout(this._timer)},emit:function(e){8===this.state&&(e&&4&e.eventType?this.manager.emit(this.options.event+"up",e):(this._input.timeStamp=f(),this.manager.emit(this.options.event,this._input)))}}),C(Ie,be,{defaults:{event:"rotate",threshold:0,pointers:2},getTouchAction:function(){return["none"]},attrTest:function(e){return this._super.attrTest.call(this,e)&&(Math.abs(e.rotation)>this.options.threshold||2&this.state)}}),C(xe,be,{defaults:{event:"swipe",threshold:10,velocity:.3,direction:30,pointers:1},getTouchAction:function(){return Ce.prototype.getTouchAction.call(this)},attrTest:function(e){var t,n=this.options.direction;return 30&n?t=e.overallVelocity:6&n?t=e.overallVelocityX:24&n&&(t=e.overallVelocityY),this._super.attrTest.call(this,e)&&n&e.offsetDirection&&e.distance>this.options.threshold&&e.maxPointers==this.options.pointers&&h(t)>this.options.velocity&&4&e.eventType},emit:function(e){var t=me(e.offsetDirection);t&&this.manager.emit(this.options.event+t,e),this.manager.emit(this.options.event,e)}}),C(we,ge,{defaults:{event:"tap",pointers:1,taps:1,interval:300,time:250,threshold:9,posThreshold:10},getTouchAction:function(){return["manipulation"]},process:function(e){var t=this.options,n=e.pointers.length===t.pointers,r=e.distance<t.threshold,i=e.deltaTime<t.time;if(this.reset(),1&e.eventType&&0===this.count)return this.failTimeout();if(r&&i&&n){if(4!=e.eventType)return this.failTimeout();var o=!this.pTime||e.timeStamp-this.pTime<t.interval,a=!this.pCenter||G(this.pCenter,e.center)<t.posThreshold;if(this.pTime=e.timeStamp,this.pCenter=e.center,a&&o?this.count+=1:this.count=1,this._input=e,0==this.count%t.taps)return this.hasRequireFailures()?(this._timer=p((function(){this.state=8,this.tryEmit()}),t.interval,this),2):8}return 32},failTimeout:function(){return this._timer=p((function(){this.state=32}),this.options.interval,this),32},reset:function(){clearTimeout(this._timer)},emit:function(){8==this.state&&(this._input.tapCount=this.count,this.manager.emit(this.options.event,this._input))}}),Me.VERSION="2.0.7",Me.defaults={domEvents:!1,touchAction:"compute",enable:!0,inputTarget:null,inputClass:null,preset:[[Ie,{enable:!1}],[Ae,{enable:!1},["rotate"]],[xe,{direction:6}],[Ce,{direction:6},["swipe"]],[we],[we,{event:"doubletap",taps:2},["tap"]],[_e]],cssProps:{userSelect:"none",touchSelect:"none",touchCallout:"none",contentZooming:"none",userDrag:"none",tapHighlightColor:"rgba(0,0,0,0)"}},Se.prototype={set:function(e){return l(this.options,e),e.touchAction&&this.touchAction.update(),e.inputTarget&&(this.input.destroy(),this.input.target=e.inputTarget,this.input.init()),this},stop:function(e){this.session.stopped=e?2:1},recognize:function(e){var t=this.session;if(!t.stopped){var n;this.touchAction.preventDefaults(e);var r=this.recognizers,i=t.curRecognizer;(!i||i&&8&i.state)&&(i=t.curRecognizer=null);for(var o=0;o<r.length;)n=r[o],2===t.stopped||i&&n!=i&&!n.canRecognizeWith(i)?n.reset():n.recognize(e),!i&&14&n.state&&(i=t.curRecognizer=n),o++}},get:function(e){if(e instanceof ge)return e;for(var t=this.recognizers,n=0;n<t.length;n++)if(t[n].options.event==e)return t[n];return null},add:function(e){if(g(e,"add",this))return this;var t=this.get(e.options.event);return t&&this.remove(t),this.recognizers.push(e),e.manager=this,this.touchAction.update(),e},remove:function(e){if(g(e,"remove",this))return this;if(e=this.get(e)){var t=this.recognizers,n=E(t,e);-1!==n&&(t.splice(n,1),this.touchAction.update())}return this},on:function(e,t){if(void 0!==e&&void 0!==t){var n=this.handlers;return v(T(e),(function(e){n[e]=n[e]||[],n[e].push(t)})),this}},off:function(e,t){if(void 0!==e){var n=this.handlers;return v(T(e),(function(e){t?n[e]&&n[e].splice(E(n[e],t),1):delete n[e]})),this}},emit:function(e,t){this.options.domEvents&&function(e,t){var n=o.createEvent("Event");n.initEvent(e,!0,!0),n.gesture=t,t.target.dispatchEvent(n)}(e,t);var n=this.handlers[e]&&this.handlers[e].slice();if(n&&n.length){t.type=e,t.preventDefault=function(){t.srcEvent.preventDefault()};for(var r=0;r<n.length;)n[r](t),r++}},destroy:function(){this.element&&Te(this,!1),this.handlers={},this.session={},this.input.destroy(),this.element=null}},l(Me,{INPUT_START:1,INPUT_MOVE:2,INPUT_END:4,INPUT_CANCEL:8,STATE_POSSIBLE:1,STATE_BEGAN:2,STATE_CHANGED:4,STATE_ENDED:8,STATE_RECOGNIZED:8,STATE_CANCELLED:16,STATE_FAILED:32,DIRECTION_NONE:1,DIRECTION_LEFT:2,DIRECTION_RIGHT:4,DIRECTION_UP:8,DIRECTION_DOWN:16,DIRECTION_HORIZONTAL:6,DIRECTION_VERTICAL:24,DIRECTION_ALL:30,Manager:Se,Input:B,TouchAction:pe,TouchInput:oe,MouseInput:Z,PointerEventInput:ee,TouchMouseInput:se,SingleTouchInput:ne,Recognizer:ge,AttrRecognizer:be,Tap:we,Pan:Ce,Swipe:xe,Pinch:Ae,Rotate:Ie,Press:_e,on:x,off:w,each:v,merge:b,extend:y,assign:l,inherit:C,bindFn:A,prefixed:L}),(void 0!==i?i:"undefined"!=typeof self?self:{}).Hammer=Me,void 0===(r=function(){return Me}.call(t,n,t,e))||(e.exports=r)}(window,document)},function(e,t,n){t.util=n(2),t.DOMutil=n(14),t.DataSet=n(11),t.DataView=n(12),t.Queue=n(43),t.Timeline=n(178),t.Graph2d=n(180),t.timeline={Core:n(65),DateUtil:n(36),Range:n(64),stack:n(100),TimeStep:n(66),components:{items:{Item:n(38),BackgroundItem:n(103),BoxItem:n(101),PointItem:n(102),RangeItem:n(70)},BackgroundGroup:n(69),Component:n(16),CurrentTime:n(67),CustomTime:n(46),DataAxis:n(107),DataScale:n(108),GraphGroup:n(109),Group:n(68),ItemSet:n(99),Legend:n(112),LineGraph:n(106),TimeAxis:n(45)}},t.moment=n(9),t.Hammer=n(10),t.keycharm=n(35)},function(e,t,n){var r=n(9),i=n(2),o=n(11),a=n(12),s=n(64),l=n(65),u=n(45),c=n(67),d=n(46),h=n(99),f=n(15).printStyle,p=n(105).allOptions,g=n(105).configureOptions,v=n(71).default,m=n(15).default;function y(e,t,n,l){if(!(this instanceof y))throw new SyntaxError("Constructor must be called with the new operator");if(!(Array.isArray(n)||n instanceof o||n instanceof a)&&n instanceof Object){var d=l;l=n,n=d}l&&l.throttleRedraw&&console.warn('Timeline option "throttleRedraw" is DEPRICATED and no longer supported. It will be removed in the next MAJOR release.');var f=this;if(this.defaultOptions={start:null,end:null,autoResize:!0,orientation:{axis:"bottom",item:"bottom"},moment:r,width:null,height:null,maxHeight:null,minHeight:null},this.options=i.deepExtend({},this.defaultOptions),this._create(e),!l||l&&void 0===l.rtl){this.dom.root.style.visibility="hidden";for(var p,g=this.dom.root;!p&&g;)p=window.getComputedStyle(g,null).direction,g=g.parentElement;this.options.rtl=p&&"rtl"==p.toLowerCase()}else this.options.rtl=l.rtl;this.options.rollingMode=l&&l.rollingMode,this.options.onInitialDrawComplete=l&&l.onInitialDrawComplete,this.components=[],this.body={dom:this.dom,domProps:this.props,emitter:{on:this.on.bind(this),off:this.off.bind(this),emit:this.emit.bind(this)},hiddenDates:[],util:{getScale:function(){return f.timeAxis.step.scale},getStep:function(){return f.timeAxis.step.step},toScreen:f._toScreen.bind(f),toGlobalScreen:f._toGlobalScreen.bind(f),toTime:f._toTime.bind(f),toGlobalTime:f._toGlobalTime.bind(f)}},this.range=new s(this.body,this.options),this.components.push(this.range),this.body.range=this.range,this.timeAxis=new u(this.body,this.options),this.timeAxis2=null,this.components.push(this.timeAxis),this.currentTime=new c(this.body,this.options),this.components.push(this.currentTime),this.itemSet=new h(this.body,this.options),this.components.push(this.itemSet),this.itemsData=null,this.groupsData=null,this.dom.root.onclick=function(e){f.emit("click",f.getEventProperties(e))},this.dom.root.ondblclick=function(e){f.emit("doubleClick",f.getEventProperties(e))},this.dom.root.oncontextmenu=function(e){f.emit("contextmenu",f.getEventProperties(e))},this.dom.root.onmouseover=function(e){f.emit("mouseOver",f.getEventProperties(e))},window.PointerEvent?(this.dom.root.onpointerdown=function(e){f.emit("mouseDown",f.getEventProperties(e))},this.dom.root.onpointermove=function(e){f.emit("mouseMove",f.getEventProperties(e))},this.dom.root.onpointerup=function(e){f.emit("mouseUp",f.getEventProperties(e))}):(this.dom.root.onmousemove=function(e){f.emit("mouseMove",f.getEventProperties(e))},this.dom.root.onmousedown=function(e){f.emit("mouseDown",f.getEventProperties(e))},this.dom.root.onmouseup=function(e){f.emit("mouseUp",f.getEventProperties(e))}),this.initialFitDone=!1,this.on("changed",(function(){if(null!=this.itemsData&&!this.options.rollingMode){if(!f.initialFitDone)if(f.initialFitDone=!0,null!=f.options.start||null!=f.options.end){if(null==f.options.start||null==f.options.end)var e=f.getItemRange();var t=null!=f.options.start?f.options.start:e.min,n=null!=f.options.end?f.options.end:e.max;f.setWindow(t,n,{animation:!1})}else f.fit({animation:!1});!f.initialDrawDone&&f.initialRangeChangeDone&&(f.initialDrawDone=!0,f.dom.root.style.visibility="visible",f.options.onInitialDrawComplete&&setTimeout((function(){return f.options.onInitialDrawComplete()}),0))}})),l&&this.setOptions(l),n&&this.setGroups(n),t&&this.setItems(t),this._redraw()}function b(e){return i.convert(e.data.start,"Date").valueOf()}function C(e){var t=null!=e.data.end?e.data.end:e.data.start;return i.convert(t,"Date").valueOf()}function A(e,t){var n=e.props.leftContainer.height,r=e.props.left.height,i=t.parent,o=i.top,a=!0,s=e.timeAxis.options.orientation.axis,l=function(){return"bottom"==s?i.height-t.top-t.height:t.top},u=-1*e._getScrollTop(),c=o+l(),d=t.height;return c<u?o+n<=o+l()+d&&(o+=l()-e.itemSet.options.margin.item.vertical):c+d>u+n?o+=l()+d-n+e.itemSet.options.margin.item.vertical:a=!1,{shouldScroll:a,scrollOffset:o=Math.min(o,r-n),itemTop:c}}y.prototype=new l,y.prototype._createConfigurator=function(){return new v(this,this.dom.container,g)},y.prototype.redraw=function(){this.itemSet&&this.itemSet.markDirty({refreshItems:!0}),this._redraw()},y.prototype.setOptions=function(e){if(!0===m.validate(e,p)&&console.log("%cErrors have been found in the supplied options object.",f),l.prototype.setOptions.call(this,e),"type"in e&&e.type!==this.options.type){this.options.type=e.type;var t=this.itemsData;if(t){var n=this.getSelection();this.setItems(null),this.setItems(t),this.setSelection(n)}}},y.prototype.setItems=function(e){var t;t=e?e instanceof o||e instanceof a?e:new o(e,{type:{start:"Date",end:"Date"}}):null,this.itemsData=t,this.itemSet&&this.itemSet.setItems(t)},y.prototype.setGroups=function(e){var t;if(e){var n=function(e){return!1!==e.visible};t=e instanceof o||e instanceof a?new a(e,{filter:n}):new o(e.filter(n))}else t=null;this.groupsData=t,this.itemSet.setGroups(t)},y.prototype.setData=function(e){e&&e.groups&&this.setGroups(e.groups),e&&e.items&&this.setItems(e.items)},y.prototype.setSelection=function(e,t){this.itemSet&&this.itemSet.setSelection(e),t&&t.focus&&this.focus(e,t)},y.prototype.getSelection=function(){return this.itemSet&&this.itemSet.getSelection()||[]},y.prototype.focus=function(e,t){if(this.itemsData&&null!=e){var n=Array.isArray(e)?e:[e],r=this.itemsData.getDataSet().get(n,{type:{start:"Date",end:"Date"}}),i=null,o=null;if(r.forEach((function(e){var t=e.start.valueOf(),n="end"in e?e.end.valueOf():e.start.valueOf();(null===i||t<i)&&(i=t),(null===o||n>o)&&(o=n)})),null!==i&&null!==o){var a=this,s=this.itemSet.items[n[0]],l=-1*this._getScrollTop(),u=null,c=function(){var e=A(a,s);e.shouldScroll&&e.itemTop!=u.itemTop&&(a._setScrollTop(-e.scrollOffset),a._redraw())},d=(i+o)/2,h=Math.max(this.range.end-this.range.start,1.1*(o-i)),f=!t||void 0===t.animation||t.animation;f||(u={shouldScroll:!1,scrollOffset:-1,itemTop:-1}),this.range.setRange(d-h/2,d+h/2,{animation:f},(function(){c(),setTimeout(c,100)}),(function(e,t,n){var r=A(a,s);if(u||(u=r),u.itemTop!=r.itemTop||u.shouldScroll){u.itemTop!=r.itemTop&&r.shouldScroll&&(u=r,l=-1*a._getScrollTop());var i=l,o=u.scrollOffset,c=n?o:i+(o-i)*e;a._setScrollTop(-c),t||a._redraw()}}))}}},y.prototype.fit=function(e,t){var n,r=!e||void 0===e.animation||e.animation,i=this.itemsData&&this.itemsData.getDataSet();1===i.length&&void 0===i.get()[0].end?(n=this.getDataRange(),this.moveTo(n.min.valueOf(),{animation:r},t)):(n=this.getItemRange(),this.range.setRange(n.min,n.max,{animation:r},t))},y.prototype.getItemRange=function(){var e=this.getDataRange(),t=null!==e.min?e.min.valueOf():null,n=null!==e.max?e.max.valueOf():null,r=null,o=null;if(null!=t&&null!=n){var a=n-t;a<=0&&(a=10);var s=a/this.props.center.width,l={},u=0;if(i.forEach(this.itemSet.items,(function(e,t){e.groupShowing&&(l[t]=e.redraw(!0),u=l[t].length)})),u>0)for(var c=0;c<u;c++)i.forEach(l,(function(e){e[c]()}));if(i.forEach(this.itemSet.items,function(e){var i,a,l=b(e),u=C(e);this.options.rtl?(i=l-(e.getWidthRight()+10)*s,a=u+(e.getWidthLeft()+10)*s):(i=l-(e.getWidthLeft()+10)*s,a=u+(e.getWidthRight()+10)*s),i<t&&(t=i,r=e),a>n&&(n=a,o=e)}.bind(this)),r&&o){var d=r.getWidthLeft()+10,h=o.getWidthRight()+10,f=this.props.center.width-d-h;f>0&&(this.options.rtl?(t=b(r)-h*a/f,n=C(o)+d*a/f):(t=b(r)-d*a/f,n=C(o)+h*a/f))}}return{min:null!=t?new Date(t):null,max:null!=n?new Date(n):null}},y.prototype.getDataRange=function(){var e=null,t=null,n=this.itemsData&&this.itemsData.getDataSet();return n&&n.forEach((function(n){var r=i.convert(n.start,"Date").valueOf(),o=i.convert(null!=n.end?n.end:n.start,"Date").valueOf();(null===e||r<e)&&(e=r),(null===t||o>t)&&(t=o)})),{min:null!=e?new Date(e):null,max:null!=t?new Date(t):null}},y.prototype.getEventProperties=function(e){var t,n=e.center?e.center.x:e.clientX,r=e.center?e.center.y:e.clientY;t=this.options.rtl?i.getAbsoluteRight(this.dom.centerContainer)-n:n-i.getAbsoluteLeft(this.dom.centerContainer);var o=r-i.getAbsoluteTop(this.dom.centerContainer),a=this.itemSet.itemFromTarget(e),s=this.itemSet.groupFromTarget(e),l=d.customTimeFromTarget(e),u=this.itemSet.options.snap||null,c=this.body.util.getScale(),h=this.body.util.getStep(),f=this._toTime(t),p=u?u(f,c,h):f,g=i.getTarget(e),v=null;return null!=a?v="item":null!=l?v="custom-time":i.hasParent(g,this.timeAxis.dom.foreground)||this.timeAxis2&&i.hasParent(g,this.timeAxis2.dom.foreground)?v="axis":i.hasParent(g,this.itemSet.dom.labelSet)?v="group-label":i.hasParent(g,this.currentTime.bar)?v="current-time":i.hasParent(g,this.dom.center)&&(v="background"),{event:e,item:a?a.id:null,group:s?s.groupId:null,what:v,pageX:e.srcEvent?e.srcEvent.pageX:e.pageX,pageY:e.srcEvent?e.srcEvent.pageY:e.pageY,x:t,y:o,time:f,snappedTime:p}},y.prototype.toggleRollingMode=function(){this.range.rolling?this.range.stopRolling():(null==this.options.rollingMode&&this.setOptions(this.options),this.range.startRolling())},e.exports=y},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=a(n(19)),i=a(n(0)),o=a(n(1));function a(e){return e&&e.__esModule?e:{default:e}}var s=n(10),l=n(37),u=n(2),c=function(){function e(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:1;(0,i.default)(this,e),this.pixelRatio=t,this.generated=!1,this.centerCoordinates={x:144.5,y:144.5},this.r=289*.49,this.color={r:255,g:255,b:255,a:1},this.hueCircle=void 0,this.initialColor={r:255,g:255,b:255,a:1},this.previousColor=void 0,this.applied=!1,this.updateCallback=function(){},this.closeCallback=function(){},this._create()}return(0,o.default)(e,[{key:"insertTo",value:function(e){void 0!==this.hammer&&(this.hammer.destroy(),this.hammer=void 0),this.container=e,this.container.appendChild(this.frame),this._bindHammer(),this._setSize()}},{key:"setUpdateCallback",value:function(e){if("function"!=typeof e)throw new Error("Function attempted to set as colorPicker update callback is not a function.");this.updateCallback=e}},{key:"setCloseCallback",value:function(e){if("function"!=typeof e)throw new Error("Function attempted to set as colorPicker closing callback is not a function.");this.closeCallback=e}},{key:"_isColorString",value:function(e){if("string"==typeof e)return{black:"#000000",navy:"#000080",darkblue:"#00008B",mediumblue:"#0000CD",blue:"#0000FF",darkgreen:"#006400",green:"#008000",teal:"#008080",darkcyan:"#008B8B",deepskyblue:"#00BFFF",darkturquoise:"#00CED1",mediumspringgreen:"#00FA9A",lime:"#00FF00",springgreen:"#00FF7F",aqua:"#00FFFF",cyan:"#00FFFF",midnightblue:"#191970",dodgerblue:"#1E90FF",lightseagreen:"#20B2AA",forestgreen:"#228B22",seagreen:"#2E8B57",darkslategray:"#2F4F4F",limegreen:"#32CD32",mediumseagreen:"#3CB371",turquoise:"#40E0D0",royalblue:"#4169E1",steelblue:"#4682B4",darkslateblue:"#483D8B",mediumturquoise:"#48D1CC",indigo:"#4B0082",darkolivegreen:"#556B2F",cadetblue:"#5F9EA0",cornflowerblue:"#6495ED",mediumaquamarine:"#66CDAA",dimgray:"#696969",slateblue:"#6A5ACD",olivedrab:"#6B8E23",slategray:"#708090",lightslategray:"#778899",mediumslateblue:"#7B68EE",lawngreen:"#7CFC00",chartreuse:"#7FFF00",aquamarine:"#7FFFD4",maroon:"#800000",purple:"#800080",olive:"#808000",gray:"#808080",skyblue:"#87CEEB",lightskyblue:"#87CEFA",blueviolet:"#8A2BE2",darkred:"#8B0000",darkmagenta:"#8B008B",saddlebrown:"#8B4513",darkseagreen:"#8FBC8F",lightgreen:"#90EE90",mediumpurple:"#9370D8",darkviolet:"#9400D3",palegreen:"#98FB98",darkorchid:"#9932CC",yellowgreen:"#9ACD32",sienna:"#A0522D",brown:"#A52A2A",darkgray:"#A9A9A9",lightblue:"#ADD8E6",greenyellow:"#ADFF2F",paleturquoise:"#AFEEEE",lightsteelblue:"#B0C4DE",powderblue:"#B0E0E6",firebrick:"#B22222",darkgoldenrod:"#B8860B",mediumorchid:"#BA55D3",rosybrown:"#BC8F8F",darkkhaki:"#BDB76B",silver:"#C0C0C0",mediumvioletred:"#C71585",indianred:"#CD5C5C",peru:"#CD853F",chocolate:"#D2691E",tan:"#D2B48C",lightgrey:"#D3D3D3",palevioletred:"#D87093",thistle:"#D8BFD8",orchid:"#DA70D6",goldenrod:"#DAA520",crimson:"#DC143C",gainsboro:"#DCDCDC",plum:"#DDA0DD",burlywood:"#DEB887",lightcyan:"#E0FFFF",lavender:"#E6E6FA",darksalmon:"#E9967A",violet:"#EE82EE",palegoldenrod:"#EEE8AA",lightcoral:"#F08080",khaki:"#F0E68C",aliceblue:"#F0F8FF",honeydew:"#F0FFF0",azure:"#F0FFFF",sandybrown:"#F4A460",wheat:"#F5DEB3",beige:"#F5F5DC",whitesmoke:"#F5F5F5",mintcream:"#F5FFFA",ghostwhite:"#F8F8FF",salmon:"#FA8072",antiquewhite:"#FAEBD7",linen:"#FAF0E6",lightgoldenrodyellow:"#FAFAD2",oldlace:"#FDF5E6",red:"#FF0000",fuchsia:"#FF00FF",magenta:"#FF00FF",deeppink:"#FF1493",orangered:"#FF4500",tomato:"#FF6347",hotpink:"#FF69B4",coral:"#FF7F50",darkorange:"#FF8C00",lightsalmon:"#FFA07A",orange:"#FFA500",lightpink:"#FFB6C1",pink:"#FFC0CB",gold:"#FFD700",peachpuff:"#FFDAB9",navajowhite:"#FFDEAD",moccasin:"#FFE4B5",bisque:"#FFE4C4",mistyrose:"#FFE4E1",blanchedalmond:"#FFEBCD",papayawhip:"#FFEFD5",lavenderblush:"#FFF0F5",seashell:"#FFF5EE",cornsilk:"#FFF8DC",lemonchiffon:"#FFFACD",floralwhite:"#FFFAF0",snow:"#FFFAFA",yellow:"#FFFF00",lightyellow:"#FFFFE0",ivory:"#FFFFF0",white:"#FFFFFF"}[e]}},{key:"setColor",value:function(e){var t=!(arguments.length>1&&void 0!==arguments[1])||arguments[1];if("none"!==e){var n=void 0,i=this._isColorString(e);if(void 0!==i&&(e=i),!0===u.isString(e)){if(!0===u.isValidRGB(e)){var o=e.substr(4).substr(0,e.length-5).split(",");n={r:o[0],g:o[1],b:o[2],a:1}}else if(!0===u.isValidRGBA(e)){var a=e.substr(5).substr(0,e.length-6).split(",");n={r:a[0],g:a[1],b:a[2],a:a[3]}}else if(!0===u.isValidHex(e)){var s=u.hexToRGB(e);n={r:s.r,g:s.g,b:s.b,a:1}}}else if(e instanceof Object&&void 0!==e.r&&void 0!==e.g&&void 0!==e.b){var l=void 0!==e.a?e.a:"1.0";n={r:e.r,g:e.g,b:e.b,a:l}}if(void 0===n)throw new Error("Unknown color passed to the colorPicker. Supported are strings: rgb, hex, rgba. Object: rgb ({r:r,g:g,b:b,[a:a]}). Supplied: "+(0,r.default)(e));this._setColor(n,t)}}},{key:"show",value:function(){void 0!==this.closeCallback&&(this.closeCallback(),this.closeCallback=void 0),this.applied=!1,this.frame.style.display="block",this._generateHueCircle()}},{key:"_hide",value:function(){var e=this,t=!(arguments.length>0&&void 0!==arguments[0])||arguments[0];!0===t&&(this.previousColor=u.extend({},this.color)),!0===this.applied&&this.updateCallback(this.initialColor),this.frame.style.display="none",setTimeout((function(){void 0!==e.closeCallback&&(e.closeCallback(),e.closeCallback=void 0)}),0)}},{key:"_save",value:function(){this.updateCallback(this.color),this.applied=!1,this._hide()}},{key:"_apply",value:function(){this.applied=!0,this.updateCallback(this.color),this._updatePicker(this.color)}},{key:"_loadLast",value:function(){void 0!==this.previousColor?this.setColor(this.previousColor,!1):alert("There is no last color to load...")}},{key:"_setColor",value:function(e){var t=!(arguments.length>1&&void 0!==arguments[1])||arguments[1];!0===t&&(this.initialColor=u.extend({},e)),this.color=e;var n=u.RGBToHSV(e.r,e.g,e.b),r=2*Math.PI,i=this.r*n.s,o=this.centerCoordinates.x+i*Math.sin(r*n.h),a=this.centerCoordinates.y+i*Math.cos(r*n.h);this.colorPickerSelector.style.left=o-.5*this.colorPickerSelector.clientWidth+"px",this.colorPickerSelector.style.top=a-.5*this.colorPickerSelector.clientHeight+"px",this._updatePicker(e)}},{key:"_setOpacity",value:function(e){this.color.a=e/100,this._updatePicker(this.color)}},{key:"_setBrightness",value:function(e){var t=u.RGBToHSV(this.color.r,this.color.g,this.color.b);t.v=e/100;var n=u.HSVToRGB(t.h,t.s,t.v);n.a=this.color.a,this.color=n,this._updatePicker()}},{key:"_updatePicker",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:this.color,t=u.RGBToHSV(e.r,e.g,e.b),n=this.colorPickerCanvas.getContext("2d");void 0===this.pixelRation&&(this.pixelRatio=(window.devicePixelRatio||1)/(n.webkitBackingStorePixelRatio||n.mozBackingStorePixelRatio||n.msBackingStorePixelRatio||n.oBackingStorePixelRatio||n.backingStorePixelRatio||1)),n.setTransform(this.pixelRatio,0,0,this.pixelRatio,0,0);var r=this.colorPickerCanvas.clientWidth,i=this.colorPickerCanvas.clientHeight;n.clearRect(0,0,r,i),n.putImageData(this.hueCircle,0,0),n.fillStyle="rgba(0,0,0,"+(1-t.v)+")",n.circle(this.centerCoordinates.x,this.centerCoordinates.y,this.r),n.fill(),this.brightnessRange.value=100*t.v,this.opacityRange.value=100*e.a,this.initialColorDiv.style.backgroundColor="rgba("+this.initialColor.r+","+this.initialColor.g+","+this.initialColor.b+","+this.initialColor.a+")",this.newColorDiv.style.backgroundColor="rgba("+this.color.r+","+this.color.g+","+this.color.b+","+this.color.a+")"}},{key:"_setSize",value:function(){this.colorPickerCanvas.style.width="100%",this.colorPickerCanvas.style.height="100%",this.colorPickerCanvas.width=289*this.pixelRatio,this.colorPickerCanvas.height=289*this.pixelRatio}},{key:"_create",value:function(){if(this.frame=document.createElement("div"),this.frame.className="vis-color-picker",this.colorPickerDiv=document.createElement("div"),this.colorPickerSelector=document.createElement("div"),this.colorPickerSelector.className="vis-selector",this.colorPickerDiv.appendChild(this.colorPickerSelector),this.colorPickerCanvas=document.createElement("canvas"),this.colorPickerDiv.appendChild(this.colorPickerCanvas),this.colorPickerCanvas.getContext){var e=this.colorPickerCanvas.getContext("2d");this.pixelRatio=(window.devicePixelRatio||1)/(e.webkitBackingStorePixelRatio||e.mozBackingStorePixelRatio||e.msBackingStorePixelRatio||e.oBackingStorePixelRatio||e.backingStorePixelRatio||1),this.colorPickerCanvas.getContext("2d").setTransform(this.pixelRatio,0,0,this.pixelRatio,0,0)}else{var t=document.createElement("DIV");t.style.color="red",t.style.fontWeight="bold",t.style.padding="10px",t.innerHTML="Error: your browser does not support HTML canvas",this.colorPickerCanvas.appendChild(t)}this.colorPickerDiv.className="vis-color",this.opacityDiv=document.createElement("div"),this.opacityDiv.className="vis-opacity",this.brightnessDiv=document.createElement("div"),this.brightnessDiv.className="vis-brightness",this.arrowDiv=document.createElement("div"),this.arrowDiv.className="vis-arrow",this.opacityRange=document.createElement("input");try{this.opacityRange.type="range",this.opacityRange.min="0",this.opacityRange.max="100"}catch(e){}this.opacityRange.value="100",this.opacityRange.className="vis-range",this.brightnessRange=document.createElement("input");try{this.brightnessRange.type="range",this.brightnessRange.min="0",this.brightnessRange.max="100"}catch(e){}this.brightnessRange.value="100",this.brightnessRange.className="vis-range",this.opacityDiv.appendChild(this.opacityRange),this.brightnessDiv.appendChild(this.brightnessRange);var n=this;this.opacityRange.onchange=function(){n._setOpacity(this.value)},this.opacityRange.oninput=function(){n._setOpacity(this.value)},this.brightnessRange.onchange=function(){n._setBrightness(this.value)},this.brightnessRange.oninput=function(){n._setBrightness(this.value)},this.brightnessLabel=document.createElement("div"),this.brightnessLabel.className="vis-label vis-brightness",this.brightnessLabel.innerHTML="brightness:",this.opacityLabel=document.createElement("div"),this.opacityLabel.className="vis-label vis-opacity",this.opacityLabel.innerHTML="opacity:",this.newColorDiv=document.createElement("div"),this.newColorDiv.className="vis-new-color",this.newColorDiv.innerHTML="new",this.initialColorDiv=document.createElement("div"),this.initialColorDiv.className="vis-initial-color",this.initialColorDiv.innerHTML="initial",this.cancelButton=document.createElement("div"),this.cancelButton.className="vis-button vis-cancel",this.cancelButton.innerHTML="cancel",this.cancelButton.onclick=this._hide.bind(this,!1),this.applyButton=document.createElement("div"),this.applyButton.className="vis-button vis-apply",this.applyButton.innerHTML="apply",this.applyButton.onclick=this._apply.bind(this),this.saveButton=document.createElement("div"),this.saveButton.className="vis-button vis-save",this.saveButton.innerHTML="save",this.saveButton.onclick=this._save.bind(this),this.loadButton=document.createElement("div"),this.loadButton.className="vis-button vis-load",this.loadButton.innerHTML="load last",this.loadButton.onclick=this._loadLast.bind(this),this.frame.appendChild(this.colorPickerDiv),this.frame.appendChild(this.arrowDiv),this.frame.appendChild(this.brightnessLabel),this.frame.appendChild(this.brightnessDiv),this.frame.appendChild(this.opacityLabel),this.frame.appendChild(this.opacityDiv),this.frame.appendChild(this.newColorDiv),this.frame.appendChild(this.initialColorDiv),this.frame.appendChild(this.cancelButton),this.frame.appendChild(this.applyButton),this.frame.appendChild(this.saveButton),this.frame.appendChild(this.loadButton)}},{key:"_bindHammer",value:function(){var e=this;this.drag={},this.pinch={},this.hammer=new s(this.colorPickerCanvas),this.hammer.get("pinch").set({enable:!0}),l.onTouch(this.hammer,(function(t){e._moveSelector(t)})),this.hammer.on("tap",(function(t){e._moveSelector(t)})),this.hammer.on("panstart",(function(t){e._moveSelector(t)})),this.hammer.on("panmove",(function(t){e._moveSelector(t)})),this.hammer.on("panend",(function(t){e._moveSelector(t)}))}},{key:"_generateHueCircle",value:function(){if(!1===this.generated){var e=this.colorPickerCanvas.getContext("2d");void 0===this.pixelRation&&(this.pixelRatio=(window.devicePixelRatio||1)/(e.webkitBackingStorePixelRatio||e.mozBackingStorePixelRatio||e.msBackingStorePixelRatio||e.oBackingStorePixelRatio||e.backingStorePixelRatio||1)),e.setTransform(this.pixelRatio,0,0,this.pixelRatio,0,0);var t=this.colorPickerCanvas.clientWidth,n=this.colorPickerCanvas.clientHeight;e.clearRect(0,0,t,n);var r=void 0,i=void 0,o=void 0,a=void 0;this.centerCoordinates={x:.5*t,y:.5*n},this.r=.49*t;var s=2*Math.PI/360,l=1/this.r,c=void 0;for(o=0;o<360;o++)for(a=0;a<this.r;a++)r=this.centerCoordinates.x+a*Math.sin(s*o),i=this.centerCoordinates.y+a*Math.cos(s*o),c=u.HSVToRGB(o*(1/360),a*l,1),e.fillStyle="rgb("+c.r+","+c.g+","+c.b+")",e.fillRect(r-.5,i-.5,2,2);e.strokeStyle="rgba(0,0,0,1)",e.circle(this.centerCoordinates.x,this.centerCoordinates.y,this.r),e.stroke(),this.hueCircle=e.getImageData(0,0,t,n)}this.generated=!0}},{key:"_moveSelector",value:function(e){var t=this.colorPickerDiv.getBoundingClientRect(),n=e.center.x-t.left,r=e.center.y-t.top,i=.5*this.colorPickerDiv.clientHeight,o=.5*this.colorPickerDiv.clientWidth,a=n-o,s=r-i,l=Math.atan2(a,s),c=.98*Math.min(Math.sqrt(a*a+s*s),o),d=Math.cos(l)*c+i,h=Math.sin(l)*c+o;this.colorPickerSelector.style.top=d-.5*this.colorPickerSelector.clientHeight+"px",this.colorPickerSelector.style.left=h-.5*this.colorPickerSelector.clientWidth+"px";var f=l/(2*Math.PI);f=f<0?f+1:f;var p=c/this.r,g=u.RGBToHSV(this.color.r,this.color.g,this.color.b);g.h=f,g.s=p;var v=u.HSVToRGB(g.h,g.s,g.v);v.a=this.color.a,this.color=v,this.initialColorDiv.style.backgroundColor="rgba("+this.initialColor.r+","+this.initialColor.g+","+this.initialColor.b+","+this.initialColor.a+")",this.newColorDiv.style.backgroundColor="rgba("+this.color.r+","+this.color.g+","+this.color.b+","+this.color.a+")"}}]),e}();t.default=c},function(e,t,n){var r=n(9),i=n(2),o=n(11),a=n(12),s=n(64),l=n(65),u=n(45),c=n(67),d=n(46),h=n(106),f=n(15).printStyle,p=n(113).allOptions,g=n(113).configureOptions,v=n(71).default,m=n(15).default;function y(e,t,n,l){if(!(Array.isArray(n)||n instanceof o||n instanceof a)&&n instanceof Object){var d=l;l=n,n=d}l&&l.throttleRedraw&&console.warn('Graph2d option "throttleRedraw" is DEPRICATED and no longer supported. It will be removed in the next MAJOR release.');var f=this;this.defaultOptions={start:null,end:null,autoResize:!0,orientation:{axis:"bottom",item:"bottom"},moment:r,width:null,height:null,maxHeight:null,minHeight:null},this.options=i.deepExtend({},this.defaultOptions),this._create(e),this.components=[],this.body={dom:this.dom,domProps:this.props,emitter:{on:this.on.bind(this),off:this.off.bind(this),emit:this.emit.bind(this)},hiddenDates:[],util:{toScreen:f._toScreen.bind(f),toGlobalScreen:f._toGlobalScreen.bind(f),toTime:f._toTime.bind(f),toGlobalTime:f._toGlobalTime.bind(f)}},this.range=new s(this.body),this.components.push(this.range),this.body.range=this.range,this.timeAxis=new u(this.body),this.components.push(this.timeAxis),this.currentTime=new c(this.body),this.components.push(this.currentTime),this.linegraph=new h(this.body),this.components.push(this.linegraph),this.itemsData=null,this.groupsData=null,this.on("tap",(function(e){f.emit("click",f.getEventProperties(e))})),this.on("doubletap",(function(e){f.emit("doubleClick",f.getEventProperties(e))})),this.dom.root.oncontextmenu=function(e){f.emit("contextmenu",f.getEventProperties(e))},l&&this.setOptions(l),n&&this.setGroups(n),t&&this.setItems(t),this._redraw()}y.prototype=new l,y.prototype.setOptions=function(e){!0===m.validate(e,p)&&console.log("%cErrors have been found in the supplied options object.",f),l.prototype.setOptions.call(this,e)},y.prototype.setItems=function(e){var t,n=null==this.itemsData;if(t=e?e instanceof o||e instanceof a?e:new o(e,{type:{start:"Date",end:"Date"}}):null,this.itemsData=t,this.linegraph&&this.linegraph.setItems(t),n)if(null!=this.options.start||null!=this.options.end){var r=null!=this.options.start?this.options.start:null,i=null!=this.options.end?this.options.end:null;this.setWindow(r,i,{animation:!1})}else this.fit({animation:!1})},y.prototype.setGroups=function(e){var t;t=e?e instanceof o||e instanceof a?e:new o(e):null,this.groupsData=t,this.linegraph.setGroups(t)},y.prototype.getLegend=function(e,t,n){return void 0===t&&(t=15),void 0===n&&(n=15),void 0!==this.linegraph.groups[e]?this.linegraph.groups[e].getLegend(t,n):"cannot find group:'"+e+"'"},y.prototype.isGroupVisible=function(e){return void 0!==this.linegraph.groups[e]&&this.linegraph.groups[e].visible&&(void 0===this.linegraph.options.groups.visibility[e]||1==this.linegraph.options.groups.visibility[e])},y.prototype.getDataRange=function(){var e=null,t=null;for(var n in this.linegraph.groups)if(this.linegraph.groups.hasOwnProperty(n)&&1==this.linegraph.groups[n].visible)for(var r=0;r<this.linegraph.groups[n].itemsData.length;r++){var o=this.linegraph.groups[n].itemsData[r],a=i.convert(o.x,"Date").valueOf();e=null==e||e>a?a:e,t=null==t||t<a?a:t}return{min:null!=e?new Date(e):null,max:null!=t?new Date(t):null}},y.prototype.getEventProperties=function(e){var t=e.center?e.center.x:e.clientX,n=e.center?e.center.y:e.clientY,r=t-i.getAbsoluteLeft(this.dom.centerContainer),o=n-i.getAbsoluteTop(this.dom.centerContainer),a=this._toTime(r),s=d.customTimeFromTarget(e),l=i.getTarget(e),u=null;i.hasParent(l,this.timeAxis.dom.foreground)||this.timeAxis2&&i.hasParent(l,this.timeAxis2.dom.foreground)?u="axis":i.hasParent(l,this.linegraph.yAxisLeft.dom.frame)||i.hasParent(l,this.linegraph.yAxisRight.dom.frame)?u="data-axis":i.hasParent(l,this.linegraph.legendLeft.dom.frame)||i.hasParent(l,this.linegraph.legendRight.dom.frame)?u="legend":null!=s?u="custom-time":i.hasParent(l,this.currentTime.bar)?u="current-time":i.hasParent(l,this.dom.center)&&(u="background");var c=[],h=this.linegraph.yAxisLeft,f=this.linegraph.yAxisRight;return!h.hidden&&this.itemsData.length>0&&c.push(h.screenToValue(o)),!f.hidden&&this.itemsData.length>0&&c.push(f.screenToValue(o)),{event:e,what:u,pageX:e.srcEvent?e.srcEvent.pageX:e.pageX,pageY:e.srcEvent?e.srcEvent.pageY:e.pageY,x:r,y:o,time:a,value:c}},y.prototype._createConfigurator=function(){return new v(this,this.dom.container,g)},e.exports=y},function(e,t,n){t.util=n(2),t.DOMutil=n(14),t.DataSet=n(11),t.DataView=n(12),t.Queue=n(43),t.Network=n(182),t.network={Images:n(116),dotparser:n(114),gephiParser:n(115),allOptions:n(122)},t.network.convertDot=function(e){return t.network.dotparser.DOTToGraph(e)},t.network.convertGephi=function(e,n){return t.network.gephiParser.parseGephi(e,n)},t.moment=n(9),t.Hammer=n(10),t.keycharm=n(35)},function(e,t,n){n(183);var r=n(44),i=n(2),o=n(114),a=n(115),s=n(97),l=n(184),u=n(116).default,c=n(186).default,d=n(187).default,h=n(214).default,f=n(220).default,p=n(227).default,g=n(229).default,v=n(230).default,m=n(231).default,y=n(232).default,b=n(234).default,C=n(235).default,A=n(237).default,_=n(71).default,I=n(15).default,x=n(15).printStyle,w=n(122),M=w.allOptions,S=w.configureOptions,T=n(238).default;function E(e,t,n){var r=this;if(!(this instanceof E))throw new SyntaxError("Constructor must be called with the new operator");this.options={},this.defaultOptions={locale:"en",locales:l,clickToUse:!1},i.extend(this.options,this.defaultOptions),this.body={container:e,nodes:{},nodeIndices:[],edges:{},edgeIndices:[],emitter:{on:this.on.bind(this),off:this.off.bind(this),emit:this.emit.bind(this),once:this.once.bind(this)},eventListeners:{onTap:function(){},onTouch:function(){},onDoubleTap:function(){},onHold:function(){},onDragStart:function(){},onDrag:function(){},onDragEnd:function(){},onMouseWheel:function(){},onPinch:function(){},onMouseMove:function(){},onRelease:function(){},onContext:function(){}},data:{nodes:null,edges:null},functions:{createNode:function(){},createEdge:function(){},getPointer:function(){}},modules:{},view:{scale:1,translation:{x:0,y:0}}},this.bindEventListeners(),this.images=new u((function(){return r.body.emitter.emit("_requestRedraw")})),this.groups=new c,this.canvas=new v(this.body),this.selectionHandler=new b(this.body,this.canvas),this.interactionHandler=new y(this.body,this.canvas,this.selectionHandler),this.view=new m(this.body,this.canvas),this.renderer=new g(this.body,this.canvas),this.physics=new f(this.body),this.layoutEngine=new C(this.body),this.clustering=new p(this.body),this.manipulation=new A(this.body,this.canvas,this.selectionHandler),this.nodesHandler=new d(this.body,this.images,this.groups,this.layoutEngine),this.edgesHandler=new h(this.body,this.images,this.groups),this.body.modules.kamadaKawai=new T(this.body,150,.05),this.body.modules.clustering=this.clustering,this.canvas._create(),this.setOptions(n),this.setData(t)}r(E.prototype),E.prototype.setOptions=function(e){var t=this;if(void 0!==e){if(!0===I.validate(e,M)&&console.log("%cErrors have been found in the supplied options object.",x),i.selectiveDeepExtend(["locale","locales","clickToUse"],this.options,e),e=this.layoutEngine.setOptions(e.layout,e),this.canvas.setOptions(e),this.groups.setOptions(e.groups),this.nodesHandler.setOptions(e.nodes),this.edgesHandler.setOptions(e.edges),this.physics.setOptions(e.physics),this.manipulation.setOptions(e.manipulation,e,this.options),this.interactionHandler.setOptions(e.interaction),this.renderer.setOptions(e.interaction),this.selectionHandler.setOptions(e.interaction),void 0!==e.groups&&this.body.emitter.emit("refreshNodes"),"configure"in e&&(this.configurator||(this.configurator=new _(this,this.body.container,S,this.canvas.pixelRatio)),this.configurator.setOptions(e.configure)),this.configurator&&!0===this.configurator.options.enabled){var n={nodes:{},edges:{},layout:{},interaction:{},manipulation:{},physics:{},global:{}};i.deepExtend(n.nodes,this.nodesHandler.options),i.deepExtend(n.edges,this.edgesHandler.options),i.deepExtend(n.layout,this.layoutEngine.options),i.deepExtend(n.interaction,this.selectionHandler.options),i.deepExtend(n.interaction,this.renderer.options),i.deepExtend(n.interaction,this.interactionHandler.options),i.deepExtend(n.manipulation,this.manipulation.options),i.deepExtend(n.physics,this.physics.options),i.deepExtend(n.global,this.canvas.options),i.deepExtend(n.global,this.options),this.configurator.setModuleOptions(n)}void 0!==e.clickToUse?!0===e.clickToUse?void 0===this.activator&&(this.activator=new s(this.canvas.frame),this.activator.on("change",(function(){t.body.emitter.emit("activate")}))):(void 0!==this.activator&&(this.activator.destroy(),delete this.activator),this.body.emitter.emit("activate")):this.body.emitter.emit("activate"),this.canvas.setSize(),this.body.emitter.emit("startSimulation")}},E.prototype._updateVisibleIndices=function(){var e=this.body.nodes,t=this.body.edges;for(var n in this.body.nodeIndices=[],this.body.edgeIndices=[],e)e.hasOwnProperty(n)&&(this.clustering._isClusteredNode(n)||!1!==e[n].options.hidden||this.body.nodeIndices.push(e[n].id));for(var r in t)if(t.hasOwnProperty(r)){var i=t[r],o=e[i.fromId],a=e[i.toId],s=void 0!==o&&void 0!==a;!this.clustering._isClusteredEdge(r)&&!1===i.options.hidden&&s&&!1===o.options.hidden&&!1===a.options.hidden&&this.body.edgeIndices.push(i.id)}},E.prototype.bindEventListeners=function(){var e=this;this.body.emitter.on("_dataChanged",(function(){e.edgesHandler._updateState(),e.body.emitter.emit("_dataUpdated")})),this.body.emitter.on("_dataUpdated",(function(){e.clustering._updateState(),e._updateVisibleIndices(),e._updateValueRange(e.body.nodes),e._updateValueRange(e.body.edges),e.body.emitter.emit("startSimulation"),e.body.emitter.emit("_requestRedraw")}))},E.prototype.setData=function(e){if(this.body.emitter.emit("resetPhysics"),this.body.emitter.emit("_resetData"),this.selectionHandler.unselectAll(),e&&e.dot&&(e.nodes||e.edges))throw new SyntaxError('Data must contain either parameter "dot" or  parameter pair "nodes" and "edges", but not both.');if(this.setOptions(e&&e.options),e&&e.dot){console.log("The dot property has been deprecated. Please use the static convertDot method to convert DOT into vis.network format and use the normal data format with nodes and edges. This converter is used like this: var data = vis.network.convertDot(dotString);");var t=o.DOTToGraph(e.dot);this.setData(t)}else if(e&&e.gephi){console.log("The gephi property has been deprecated. Please use the static convertGephi method to convert gephi into vis.network format and use the normal data format with nodes and edges. This converter is used like this: var data = vis.network.convertGephi(gephiJson);");var n=a.parseGephi(e.gephi);this.setData(n)}else this.nodesHandler.setData(e&&e.nodes,!0),this.edgesHandler.setData(e&&e.edges,!0),this.body.emitter.emit("_dataChanged"),this.body.emitter.emit("_dataLoaded"),this.body.emitter.emit("initPhysics")},E.prototype.destroy=function(){for(var e in this.body.emitter.emit("destroy"),this.body.emitter.off(),this.off(),delete this.groups,delete this.canvas,delete this.selectionHandler,delete this.interactionHandler,delete this.view,delete this.renderer,delete this.physics,delete this.layoutEngine,delete this.clustering,delete this.manipulation,delete this.nodesHandler,delete this.edgesHandler,delete this.configurator,delete this.images,this.body.nodes)this.body.nodes.hasOwnProperty(e)&&delete this.body.nodes[e];for(var t in this.body.edges)this.body.edges.hasOwnProperty(t)&&delete this.body.edges[t];i.recursiveDOMDelete(this.body.container)},E.prototype._updateValueRange=function(e){var t,n=void 0,r=void 0,i=0;for(t in e)if(e.hasOwnProperty(t)){var o=e[t].getValue();void 0!==o&&(n=void 0===n?o:Math.min(o,n),r=void 0===r?o:Math.max(o,r),i+=o)}if(void 0!==n&&void 0!==r)for(t in e)e.hasOwnProperty(t)&&e[t].setValueRange(n,r,i)},E.prototype.isActive=function(){return!this.activator||this.activator.active},E.prototype.setSize=function(){return this.canvas.setSize.apply(this.canvas,arguments)},E.prototype.canvasToDOM=function(){return this.canvas.canvasToDOM.apply(this.canvas,arguments)},E.prototype.DOMtoCanvas=function(){return this.canvas.DOMtoCanvas.apply(this.canvas,arguments)},E.prototype.findNode=function(){return this.clustering.findNode.apply(this.clustering,arguments)},E.prototype.isCluster=function(){return this.clustering.isCluster.apply(this.clustering,arguments)},E.prototype.openCluster=function(){return this.clustering.openCluster.apply(this.clustering,arguments)},E.prototype.cluster=function(){return this.clustering.cluster.apply(this.clustering,arguments)},E.prototype.getNodesInCluster=function(){return this.clustering.getNodesInCluster.apply(this.clustering,arguments)},E.prototype.clusterByConnection=function(){return this.clustering.clusterByConnection.apply(this.clustering,arguments)},E.prototype.clusterByHubsize=function(){return this.clustering.clusterByHubsize.apply(this.clustering,arguments)},E.prototype.clusterOutliers=function(){return this.clustering.clusterOutliers.apply(this.clustering,arguments)},E.prototype.getSeed=function(){return this.layoutEngine.getSeed.apply(this.layoutEngine,arguments)},E.prototype.enableEditMode=function(){return this.manipulation.enableEditMode.apply(this.manipulation,arguments)},E.prototype.disableEditMode=function(){return this.manipulation.disableEditMode.apply(this.manipulation,arguments)},E.prototype.addNodeMode=function(){return this.manipulation.addNodeMode.apply(this.manipulation,arguments)},E.prototype.editNode=function(){return this.manipulation.editNode.apply(this.manipulation,arguments)},E.prototype.editNodeMode=function(){return console.log("Deprecated: Please use editNode instead of editNodeMode."),this.manipulation.editNode.apply(this.manipulation,arguments)},E.prototype.addEdgeMode=function(){return this.manipulation.addEdgeMode.apply(this.manipulation,arguments)},E.prototype.editEdgeMode=function(){return this.manipulation.editEdgeMode.apply(this.manipulation,arguments)},E.prototype.deleteSelected=function(){return this.manipulation.deleteSelected.apply(this.manipulation,arguments)},E.prototype.getPositions=function(){return this.nodesHandler.getPositions.apply(this.nodesHandler,arguments)},E.prototype.storePositions=function(){return this.nodesHandler.storePositions.apply(this.nodesHandler,arguments)},E.prototype.moveNode=function(){return this.nodesHandler.moveNode.apply(this.nodesHandler,arguments)},E.prototype.getBoundingBox=function(){return this.nodesHandler.getBoundingBox.apply(this.nodesHandler,arguments)},E.prototype.getConnectedNodes=function(e){return void 0!==this.body.nodes[e]?this.nodesHandler.getConnectedNodes.apply(this.nodesHandler,arguments):this.edgesHandler.getConnectedNodes.apply(this.edgesHandler,arguments)},E.prototype.getConnectedEdges=function(){return this.nodesHandler.getConnectedEdges.apply(this.nodesHandler,arguments)},E.prototype.startSimulation=function(){return this.physics.startSimulation.apply(this.physics,arguments)},E.prototype.stopSimulation=function(){return this.physics.stopSimulation.apply(this.physics,arguments)},E.prototype.stabilize=function(){return this.physics.stabilize.apply(this.physics,arguments)},E.prototype.getSelection=function(){return this.selectionHandler.getSelection.apply(this.selectionHandler,arguments)},E.prototype.setSelection=function(){return this.selectionHandler.setSelection.apply(this.selectionHandler,arguments)},E.prototype.getSelectedNodes=function(){return this.selectionHandler.getSelectedNodes.apply(this.selectionHandler,arguments)},E.prototype.getSelectedEdges=function(){return this.selectionHandler.getSelectedEdges.apply(this.selectionHandler,arguments)},E.prototype.getNodeAt=function(){var e=this.selectionHandler.getNodeAt.apply(this.selectionHandler,arguments);return void 0!==e&&void 0!==e.id?e.id:e},E.prototype.getEdgeAt=function(){var e=this.selectionHandler.getEdgeAt.apply(this.selectionHandler,arguments);return void 0!==e&&void 0!==e.id?e.id:e},E.prototype.selectNodes=function(){return this.selectionHandler.selectNodes.apply(this.selectionHandler,arguments)},E.prototype.selectEdges=function(){return this.selectionHandler.selectEdges.apply(this.selectionHandler,arguments)},E.prototype.unselectAll=function(){this.selectionHandler.unselectAll.apply(this.selectionHandler,arguments),this.redraw()},E.prototype.redraw=function(){return this.renderer.redraw.apply(this.renderer,arguments)},E.prototype.getScale=function(){return this.view.getScale.apply(this.view,arguments)},E.prototype.getViewPosition=function(){return this.view.getViewPosition.apply(this.view,arguments)},E.prototype.fit=function(){return this.view.fit.apply(this.view,arguments)},E.prototype.moveTo=function(){return this.view.moveTo.apply(this.view,arguments)},E.prototype.focus=function(){return this.view.focus.apply(this.view,arguments)},E.prototype.releaseNode=function(){return this.view.releaseNode.apply(this.view,arguments)},E.prototype.getOptionsFromConfigurator=function(){var e={};return this.configurator&&(e=this.configurator.getOptions.apply(this.configurator)),e},e.exports=E},function(e,t,n){"undefined"!=typeof CanvasRenderingContext2D&&(CanvasRenderingContext2D.prototype.circle=function(e,t,n){this.beginPath(),this.arc(e,t,n,0,2*Math.PI,!1),this.closePath()},CanvasRenderingContext2D.prototype.square=function(e,t,n){this.beginPath(),this.rect(e-n,t-n,2*n,2*n),this.closePath()},CanvasRenderingContext2D.prototype.triangle=function(e,t,n){this.beginPath(),t+=.275*(n*=1.15);var r=2*n,i=r/2,o=Math.sqrt(3)/6*r,a=Math.sqrt(r*r-i*i);this.moveTo(e,t-(a-o)),this.lineTo(e+i,t+o),this.lineTo(e-i,t+o),this.lineTo(e,t-(a-o)),this.closePath()},CanvasRenderingContext2D.prototype.triangleDown=function(e,t,n){this.beginPath(),t-=.275*(n*=1.15);var r=2*n,i=r/2,o=Math.sqrt(3)/6*r,a=Math.sqrt(r*r-i*i);this.moveTo(e,t+(a-o)),this.lineTo(e+i,t-o),this.lineTo(e-i,t-o),this.lineTo(e,t+(a-o)),this.closePath()},CanvasRenderingContext2D.prototype.star=function(e,t,n){this.beginPath(),t+=.1*(n*=.82);for(var r=0;r<10;r++){var i=r%2==0?1.3*n:.5*n;this.lineTo(e+i*Math.sin(2*r*Math.PI/10),t-i*Math.cos(2*r*Math.PI/10))}this.closePath()},CanvasRenderingContext2D.prototype.diamond=function(e,t,n){this.beginPath(),this.lineTo(e,t+n),this.lineTo(e+n,t),this.lineTo(e,t-n),this.lineTo(e-n,t),this.closePath()},CanvasRenderingContext2D.prototype.roundRect=function(e,t,n,r,i){var o=Math.PI/180;n-2*i<0&&(i=n/2),r-2*i<0&&(i=r/2),this.beginPath(),this.moveTo(e+i,t),this.lineTo(e+n-i,t),this.arc(e+n-i,t+i,i,270*o,360*o,!1),this.lineTo(e+n,t+r-i),this.arc(e+n-i,t+r-i,i,0,90*o,!1),this.lineTo(e+i,t+r),this.arc(e+i,t+r-i,i,90*o,180*o,!1),this.lineTo(e,t+i),this.arc(e+i,t+i,i,180*o,270*o,!1),this.closePath()},CanvasRenderingContext2D.prototype.ellipse_vis=function(e,t,n,r){var i=n/2*.5522848,o=r/2*.5522848,a=e+n,s=t+r,l=e+n/2,u=t+r/2;this.beginPath(),this.moveTo(e,u),this.bezierCurveTo(e,u-o,l-i,t,l,t),this.bezierCurveTo(l+i,t,a,u-o,a,u),this.bezierCurveTo(a,u+o,l+i,s,l,s),this.bezierCurveTo(l-i,s,e,u+o,e,u),this.closePath()},CanvasRenderingContext2D.prototype.database=function(e,t,n,r){var i=r*(1/3),o=n/2*.5522848,a=i/2*.5522848,s=e+n,l=t+i,u=e+n/2,c=t+i/2,d=t+(r-i/2),h=t+r;this.beginPath(),this.moveTo(s,c),this.bezierCurveTo(s,c+a,u+o,l,u,l),this.bezierCurveTo(u-o,l,e,c+a,e,c),this.bezierCurveTo(e,c-a,u-o,t,u,t),this.bezierCurveTo(u+o,t,s,c-a,s,c),this.lineTo(s,d),this.bezierCurveTo(s,d+a,u+o,h,u,h),this.bezierCurveTo(u-o,h,e,d+a,e,d),this.lineTo(e,c)},CanvasRenderingContext2D.prototype.dashedLine=function(e,t,n,r,i){this.beginPath(),this.moveTo(e,t);for(var o=i.length,a=n-e,s=r-t,l=s/a,u=Math.sqrt(a*a+s*s),c=0,d=!0,h=0,f=i[0];u>=.1;)(f=i[c++%o])>u&&(f=u),h=Math.sqrt(f*f/(1+l*l)),e+=h=a<0?-h:h,t+=l*h,!0===d?this.lineTo(e,t):this.moveTo(e,t),u-=f,d=!d},CanvasRenderingContext2D.prototype.hexagon=function(e,t,n){this.beginPath();var r=2*Math.PI/6;this.moveTo(e+n,t);for(var i=1;i<6;i++)this.lineTo(e+n*Math.cos(r*i),t+n*Math.sin(r*i));this.closePath()})},function(e,t,n){t.en={edit:"Edit",del:"Delete selected",back:"Back",addNode:"Add Node",addEdge:"Add Edge",editNode:"Edit Node",editEdge:"Edit Edge",addDescription:"Click in an empty space to place a new node.",edgeDescription:"Click on a node and drag the edge to another node to connect them.",editEdgeDescription:"Click on the control points and drag them to a node to connect to it.",createEdgeError:"Cannot link edges to a cluster.",deleteClusterError:"Clusters cannot be deleted.",editClusterError:"Clusters cannot be edited."},t.en_EN=t.en,t.en_US=t.en,t.de={edit:"Editieren",del:"Lösche Auswahl",back:"Zurück",addNode:"Knoten hinzufügen",addEdge:"Kante hinzufügen",editNode:"Knoten editieren",editEdge:"Kante editieren",addDescription:"Klicke auf eine freie Stelle, um einen neuen Knoten zu plazieren.",edgeDescription:"Klicke auf einen Knoten und ziehe die Kante zu einem anderen Knoten, um diese zu verbinden.",editEdgeDescription:"Klicke auf die Verbindungspunkte und ziehe diese auf einen Knoten, um sie zu verbinden.",createEdgeError:"Es ist nicht möglich, Kanten mit Clustern zu verbinden.",deleteClusterError:"Cluster können nicht gelöscht werden.",editClusterError:"Cluster können nicht editiert werden."},t.de_DE=t.de,t.es={edit:"Editar",del:"Eliminar selección",back:"Átras",addNode:"Añadir nodo",addEdge:"Añadir arista",editNode:"Editar nodo",editEdge:"Editar arista",addDescription:"Haga clic en un lugar vacío para colocar un nuevo nodo.",edgeDescription:"Haga clic en un nodo y arrastre la arista hacia otro nodo para conectarlos.",editEdgeDescription:"Haga clic en un punto de control y arrastrelo a un nodo para conectarlo.",createEdgeError:"No se puede conectar una arista a un grupo.",deleteClusterError:"No es posible eliminar grupos.",editClusterError:"No es posible editar grupos."},t.es_ES=t.es,t.it={edit:"Modifica",del:"Cancella la selezione",back:"Indietro",addNode:"Aggiungi un nodo",addEdge:"Aggiungi un vertice",editNode:"Modifica il nodo",editEdge:"Modifica il vertice",addDescription:"Clicca per aggiungere un nuovo nodo",edgeDescription:"Clicca su un nodo e trascinalo ad un altro nodo per connetterli.",editEdgeDescription:"Clicca sui Punti di controllo e trascinali ad un nodo per connetterli.",createEdgeError:"Non si possono collegare vertici ad un cluster",deleteClusterError:"I cluster non possono essere cancellati",editClusterError:"I clusters non possono essere modificati."},t.it_IT=t.it,t.nl={edit:"Wijzigen",del:"Selectie verwijderen",back:"Terug",addNode:"Node toevoegen",addEdge:"Link toevoegen",editNode:"Node wijzigen",editEdge:"Link wijzigen",addDescription:"Klik op een leeg gebied om een nieuwe node te maken.",edgeDescription:"Klik op een node en sleep de link naar een andere node om ze te verbinden.",editEdgeDescription:"Klik op de verbindingspunten en sleep ze naar een node om daarmee te verbinden.",createEdgeError:"Kan geen link maken naar een cluster.",deleteClusterError:"Clusters kunnen niet worden verwijderd.",editClusterError:"Clusters kunnen niet worden aangepast."},t.nl_NL=t.nl,t.nl_BE=t.nl,t["pt-br"]={edit:"Editar",del:"Remover selecionado",back:"Voltar",addNode:"Adicionar nó",addEdge:"Adicionar aresta",editNode:"Editar nó",editEdge:"Editar aresta",addDescription:"Clique em um espaço em branco para adicionar um novo nó",edgeDescription:"Clique em um nó e arraste a aresta até outro nó para conectá-los",editEdgeDescription:"Clique nos pontos de controle e os arraste para um nó para conectá-los",createEdgeError:"Não foi possível linkar arestas a um cluster.",deleteClusterError:"Clusters não puderam ser removidos.",editClusterError:"Clusters não puderam ser editados."},t["pt-BR"]=t["pt-br"],t.pt_BR=t["pt-br"],t.pt_br=t["pt-br"],t.ru={edit:"Редактировать",del:"Удалить выбранное",back:"Назад",addNode:"Добавить узел",addEdge:"Добавить ребро",editNode:"Редактировать узел",editEdge:"Редактировать ребро",addDescription:"Кликните в свободное место, чтобы добавить новый узел.",edgeDescription:"Кликните на узел и протяните ребро к другому узлу, чтобы соединить их.",editEdgeDescription:"Кликните на контрольные точки и перетащите их в узел, чтобы подключиться к нему.",createEdgeError:"Невозможно соединить ребра в кластер.",deleteClusterError:"Кластеры не могут быть удалены",editClusterError:"Кластеры недоступны для редактирования."},t.ru_RU=t.ru,t.cn={edit:"编辑",del:"删除选定",back:"返回",addNode:"添加节点",addEdge:"添加连接线",editNode:"编辑节点",editEdge:"编辑连接线",addDescription:"单击空白处放置新节点。",edgeDescription:"单击某个节点并将该连接线拖动到另一个节点以连接它们。",editEdgeDescription:"单击控制节点并将它们拖到节点上连接。",createEdgeError:"无法将连接线连接到群集。",deleteClusterError:"无法删除群集。",editClusterError:"无法编辑群集。"},t.zh_CN=t.cn},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=o(n(0)),i=o(n(1));function o(e){return e&&e.__esModule?e:{default:e}}var a=function(){function e(){(0,r.default)(this,e),this.NUM_ITERATIONS=4,this.image=new Image,this.canvas=document.createElement("canvas")}return(0,i.default)(e,[{key:"init",value:function(){if(!this.initialized()){this.src=this.image.src;var e=this.image.width,t=this.image.height;this.width=e,this.height=t;var n=Math.floor(t/2),r=Math.floor(t/4),i=Math.floor(t/8),o=Math.floor(t/16),a=Math.floor(e/2),s=Math.floor(e/4),l=Math.floor(e/8),u=Math.floor(e/16);this.canvas.width=3*s,this.canvas.height=n,this.coordinates=[[0,0,a,n],[a,0,s,r],[a,r,l,i],[5*l,r,u,o]],this._fillMipMap()}}},{key:"initialized",value:function(){return void 0!==this.coordinates}},{key:"_fillMipMap",value:function(){var e=this.canvas.getContext("2d"),t=this.coordinates[0];e.drawImage(this.image,t[0],t[1],t[2],t[3]);for(var n=1;n<this.NUM_ITERATIONS;n++){var r=this.coordinates[n-1],i=this.coordinates[n];e.drawImage(this.canvas,r[0],r[1],r[2],r[3],i[0],i[1],i[2],i[3])}}},{key:"drawImageAtPosition",value:function(e,t,n,r,i,o){if(this.initialized())if(t>2){t*=.5;for(var a=0;t>2&&a<this.NUM_ITERATIONS;)t*=.5,a+=1;a>=this.NUM_ITERATIONS&&(a=this.NUM_ITERATIONS-1);var s=this.coordinates[a];e.drawImage(this.canvas,s[0],s[1],s[2],s[3],n,r,i,o)}else e.drawImage(this.image,n,r,i,o)}}]),e}();t.default=a},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=o(n(0)),i=o(n(1));function o(e){return e&&e.__esModule?e:{default:e}}var a=n(2),s=function(){function e(){(0,r.default)(this,e),this.clear(),this.defaultIndex=0,this.groupsArray=[],this.groupIndex=0,this.defaultGroups=[{border:"#2B7CE9",background:"#97C2FC",highlight:{border:"#2B7CE9",background:"#D2E5FF"},hover:{border:"#2B7CE9",background:"#D2E5FF"}},{border:"#FFA500",background:"#FFFF00",highlight:{border:"#FFA500",background:"#FFFFA3"},hover:{border:"#FFA500",background:"#FFFFA3"}},{border:"#FA0A10",background:"#FB7E81",highlight:{border:"#FA0A10",background:"#FFAFB1"},hover:{border:"#FA0A10",background:"#FFAFB1"}},{border:"#41A906",background:"#7BE141",highlight:{border:"#41A906",background:"#A1EC76"},hover:{border:"#41A906",background:"#A1EC76"}},{border:"#E129F0",background:"#EB7DF4",highlight:{border:"#E129F0",background:"#F0B3F5"},hover:{border:"#E129F0",background:"#F0B3F5"}},{border:"#7C29F0",background:"#AD85E4",highlight:{border:"#7C29F0",background:"#D3BDF0"},hover:{border:"#7C29F0",background:"#D3BDF0"}},{border:"#C37F00",background:"#FFA807",highlight:{border:"#C37F00",background:"#FFCA66"},hover:{border:"#C37F00",background:"#FFCA66"}},{border:"#4220FB",background:"#6E6EFD",highlight:{border:"#4220FB",background:"#9B9BFD"},hover:{border:"#4220FB",background:"#9B9BFD"}},{border:"#FD5A77",background:"#FFC0CB",highlight:{border:"#FD5A77",background:"#FFD1D9"},hover:{border:"#FD5A77",background:"#FFD1D9"}},{border:"#4AD63A",background:"#C2FABC",highlight:{border:"#4AD63A",background:"#E6FFE3"},hover:{border:"#4AD63A",background:"#E6FFE3"}},{border:"#990000",background:"#EE0000",highlight:{border:"#BB0000",background:"#FF3333"},hover:{border:"#BB0000",background:"#FF3333"}},{border:"#FF6000",background:"#FF6000",highlight:{border:"#FF6000",background:"#FF6000"},hover:{border:"#FF6000",background:"#FF6000"}},{border:"#97C2FC",background:"#2B7CE9",highlight:{border:"#D2E5FF",background:"#2B7CE9"},hover:{border:"#D2E5FF",background:"#2B7CE9"}},{border:"#399605",background:"#255C03",highlight:{border:"#399605",background:"#255C03"},hover:{border:"#399605",background:"#255C03"}},{border:"#B70054",background:"#FF007E",highlight:{border:"#B70054",background:"#FF007E"},hover:{border:"#B70054",background:"#FF007E"}},{border:"#AD85E4",background:"#7C29F0",highlight:{border:"#D3BDF0",background:"#7C29F0"},hover:{border:"#D3BDF0",background:"#7C29F0"}},{border:"#4557FA",background:"#000EA1",highlight:{border:"#6E6EFD",background:"#000EA1"},hover:{border:"#6E6EFD",background:"#000EA1"}},{border:"#FFC0CB",background:"#FD5A77",highlight:{border:"#FFD1D9",background:"#FD5A77"},hover:{border:"#FFD1D9",background:"#FD5A77"}},{border:"#C2FABC",background:"#74D66A",highlight:{border:"#E6FFE3",background:"#74D66A"},hover:{border:"#E6FFE3",background:"#74D66A"}},{border:"#EE0000",background:"#990000",highlight:{border:"#FF3333",background:"#BB0000"},hover:{border:"#FF3333",background:"#BB0000"}}],this.options={},this.defaultOptions={useDefaultGroups:!0},a.extend(this.options,this.defaultOptions)}return(0,i.default)(e,[{key:"setOptions",value:function(e){var t=["useDefaultGroups"];if(void 0!==e)for(var n in e)if(e.hasOwnProperty(n)&&-1===t.indexOf(n)){var r=e[n];this.add(n,r)}}},{key:"clear",value:function(){this.groups={},this.groupsArray=[]}},{key:"get",value:function(e){var t=!(arguments.length>1&&void 0!==arguments[1])||arguments[1],n=this.groups[e];if(void 0===n&&t)if(!1===this.options.useDefaultGroups&&this.groupsArray.length>0){var r=this.groupIndex%this.groupsArray.length;this.groupIndex++,(n={}).color=this.groups[this.groupsArray[r]],this.groups[e]=n}else{var i=this.defaultIndex%this.defaultGroups.length;this.defaultIndex++,(n={}).color=this.defaultGroups[i],this.groups[e]=n}return n}},{key:"add",value:function(e,t){return this.groups[e]=t,this.groupsArray.push(e),t}}]),e}();t.default=s},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=o(n(0)),i=o(n(1));function o(e){return e&&e.__esModule?e:{default:e}}var a=n(2),s=n(11),l=n(12),u=n(47).default,c=function(){function e(t,n,i,o){var s=this;if((0,r.default)(this,e),this.body=t,this.images=n,this.groups=i,this.layoutEngine=o,this.body.functions.createNode=this.create.bind(this),this.nodesListeners={add:function(e,t){s.add(t.items)},update:function(e,t){s.update(t.items,t.data,t.oldData)},remove:function(e,t){s.remove(t.items)}},this.defaultOptions={borderWidth:1,borderWidthSelected:2,brokenImage:void 0,color:{border:"#2B7CE9",background:"#97C2FC",highlight:{border:"#2B7CE9",background:"#D2E5FF"},hover:{border:"#2B7CE9",background:"#D2E5FF"}},fixed:{x:!1,y:!1},font:{color:"#343434",size:14,face:"arial",background:"none",strokeWidth:0,strokeColor:"#ffffff",align:"center",vadjust:0,multi:!1,bold:{mod:"bold"},boldital:{mod:"bold italic"},ital:{mod:"italic"},mono:{mod:"",size:15,face:"monospace",vadjust:2}},group:void 0,hidden:!1,icon:{face:"FontAwesome",code:void 0,size:50,color:"#2B7CE9"},image:void 0,label:void 0,labelHighlightBold:!0,level:void 0,margin:{top:5,right:5,bottom:5,left:5},mass:1,physics:!0,scaling:{min:10,max:30,label:{enabled:!1,min:14,max:30,maxVisible:30,drawThreshold:5},customScalingFunction:function(e,t,n,r){if(t===e)return.5;var i=1/(t-e);return Math.max(0,(r-e)*i)}},shadow:{enabled:!1,color:"rgba(0,0,0,0.5)",size:10,x:5,y:5},shape:"ellipse",shapeProperties:{borderDashes:!1,borderRadius:6,interpolation:!0,useImageSize:!1,useBorderWithImage:!1},size:25,title:void 0,value:void 0,x:void 0,y:void 0},this.defaultOptions.mass<=0)throw"Internal error: mass in defaultOptions of NodesHandler may not be zero or negative";this.options=a.bridgeObject(this.defaultOptions),this.bindEventListeners()}return(0,i.default)(e,[{key:"bindEventListeners",value:function(){var e=this;this.body.emitter.on("refreshNodes",this.refresh.bind(this)),this.body.emitter.on("refresh",this.refresh.bind(this)),this.body.emitter.on("destroy",(function(){a.forEach(e.nodesListeners,(function(t,n){e.body.data.nodes&&e.body.data.nodes.off(n,t)})),delete e.body.functions.createNode,delete e.nodesListeners.add,delete e.nodesListeners.update,delete e.nodesListeners.remove,delete e.nodesListeners}))}},{key:"setOptions",value:function(e){if(void 0!==e){if(u.parseOptions(this.options,e),void 0!==e.shape)for(var t in this.body.nodes)this.body.nodes.hasOwnProperty(t)&&this.body.nodes[t].updateShape();if(void 0!==e.font)for(var n in this.body.nodes)this.body.nodes.hasOwnProperty(n)&&(this.body.nodes[n].updateLabelModule(),this.body.nodes[n].needsRefresh());if(void 0!==e.size)for(var r in this.body.nodes)this.body.nodes.hasOwnProperty(r)&&this.body.nodes[r].needsRefresh();void 0===e.hidden&&void 0===e.physics||this.body.emitter.emit("_dataChanged")}}},{key:"setData",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]&&arguments[1],n=this.body.data.nodes;if(e instanceof s||e instanceof l)this.body.data.nodes=e;else if(Array.isArray(e))this.body.data.nodes=new s,this.body.data.nodes.add(e);else{if(e)throw new TypeError("Array or DataSet expected");this.body.data.nodes=new s}if(n&&a.forEach(this.nodesListeners,(function(e,t){n.off(t,e)})),this.body.nodes={},this.body.data.nodes){var r=this;a.forEach(this.nodesListeners,(function(e,t){r.body.data.nodes.on(t,e)}));var i=this.body.data.nodes.getIds();this.add(i,!0)}!1===t&&this.body.emitter.emit("_dataChanged")}},{key:"add",value:function(e){for(var t=arguments.length>1&&void 0!==arguments[1]&&arguments[1],n=void 0,r=[],i=0;i<e.length;i++){n=e[i];var o=this.body.data.nodes.get(n),a=this.create(o);r.push(a),this.body.nodes[n]=a}this.layoutEngine.positionInitially(r),!1===t&&this.body.emitter.emit("_dataChanged")}},{key:"update",value:function(e,t,n){for(var r=this.body.nodes,i=!1,o=0;o<e.length;o++){var a=e[o],s=r[a],l=t[o];void 0!==s?s.setOptions(l)&&(i=!0):(i=!0,s=this.create(l),r[a]=s)}i||void 0===n||(i=t.some((function(e,t){var r=n[t];return r&&r.level!==e.level}))),!0===i?this.body.emitter.emit("_dataChanged"):this.body.emitter.emit("_dataUpdated")}},{key:"remove",value:function(e){for(var t=this.body.nodes,n=0;n<e.length;n++)delete t[e[n]];this.body.emitter.emit("_dataChanged")}},{key:"create",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:u;return new t(e,this.body,this.images,this.groups,this.options,this.defaultOptions)}},{key:"refresh",value:function(){var e=this,t=arguments.length>0&&void 0!==arguments[0]&&arguments[0];a.forEach(this.body.nodes,(function(n,r){var i=e.body.data.nodes.get(r);void 0!==i&&(!0===t&&n.setOptions({x:null,y:null}),n.setOptions({fixed:!1}),n.setOptions(i))}))}},{key:"getPositions",value:function(e){var t={};if(void 0!==e){if(!0===Array.isArray(e)){for(var n=0;n<e.length;n++)if(void 0!==this.body.nodes[e[n]]){var r=this.body.nodes[e[n]];t[e[n]]={x:Math.round(r.x),y:Math.round(r.y)}}}else if(void 0!==this.body.nodes[e]){var i=this.body.nodes[e];t[e]={x:Math.round(i.x),y:Math.round(i.y)}}}else for(var o=0;o<this.body.nodeIndices.length;o++){var a=this.body.nodes[this.body.nodeIndices[o]];t[this.body.nodeIndices[o]]={x:Math.round(a.x),y:Math.round(a.y)}}return t}},{key:"storePositions",value:function(){var e=[],t=this.body.data.nodes.getDataSet();for(var n in t._data)if(t._data.hasOwnProperty(n)){var r=this.body.nodes[n];t._data[n].x==Math.round(r.x)&&t._data[n].y==Math.round(r.y)||e.push({id:r.id,x:Math.round(r.x),y:Math.round(r.y)})}t.update(e)}},{key:"getBoundingBox",value:function(e){if(void 0!==this.body.nodes[e])return this.body.nodes[e].shape.boundingBox}},{key:"getConnectedNodes",value:function(e,t){var n=[];if(void 0!==this.body.nodes[e])for(var r=this.body.nodes[e],i={},o=0;o<r.edges.length;o++){var a=r.edges[o];"to"!==t&&a.toId==r.id?void 0===i[a.fromId]&&(n.push(a.fromId),i[a.fromId]=!0):"from"!==t&&a.fromId==r.id&&void 0===i[a.toId]&&(n.push(a.toId),i[a.toId]=!0)}return n}},{key:"getConnectedEdges",value:function(e){var t=[];if(void 0!==this.body.nodes[e])for(var n=this.body.nodes[e],r=0;r<n.edges.length;r++)t.push(n.edges[r].id);else console.log("NodeId provided for getConnectedEdges does not exist. Provided: ",e);return t}},{key:"moveNode",value:function(e,t,n){var r=this;void 0!==this.body.nodes[e]?(this.body.nodes[e].x=Number(t),this.body.nodes[e].y=Number(n),setTimeout((function(){r.body.emitter.emit("startSimulation")}),0)):console.log("Node id supplied to moveNode does not exist. Provided: ",e)}}]),e}();t.default=c},function(e,t,n){e.exports={default:n(189),__esModule:!0}},function(e,t,n){n(49),n(60),e.exports=n(190)},function(e,t,n){var r=n(86),i=n(13)("iterator"),o=n(31);e.exports=n(7).isIterable=function(e){var t=Object(e);return void 0!==t[i]||"@@iterator"in t||o.hasOwnProperty(r(t))}},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=o(n(0)),i=o(n(1));function o(e){return e&&e.__esModule?e:{default:e}}var a=n(192).default,s=n(48).default,l=function(){function e(t,n,i,o){var s=this;(0,r.default)(this,e),this.ctx=t,this.parent=n,this.lines=new a((function(e,n){if(void 0===e)return 0;var r=s.parent.getFormattingValues(t,i,o,n),a=0;return""!==e&&(a=s.ctx.measureText(e).width),{width:a,values:r}}))}return(0,i.default)(e,[{key:"process",value:function(e){if(!s.isValidLabel(e))return this.lines.finalize();var t=this.parent.fontOptions;e=(e=e.replace(/\r\n/g,"\n")).replace(/\r/g,"\n");var n=String(e).split("\n"),r=n.length;if(t.multi)for(var i=0;i<r;i++){var o=this.splitBlocks(n[i],t.multi);if(void 0!==o)if(0!==o.length){if(t.maxWdt>0)for(var a=0;a<o.length;a++){var l=o[a].mod,u=o[a].text;this.splitStringIntoLines(u,l,!0)}else for(var c=0;c<o.length;c++){var d=o[c].mod,h=o[c].text;this.lines.append(h,d)}this.lines.newLine()}else this.lines.newLine("")}else if(t.maxWdt>0)for(var f=0;f<r;f++)this.splitStringIntoLines(n[f]);else for(var p=0;p<r;p++)this.lines.newLine(n[p]);return this.lines.finalize()}},{key:"decodeMarkupSystem",value:function(e){var t="none";return"markdown"===e||"md"===e?t="markdown":!0!==e&&"html"!==e||(t="html"),t}},{key:"splitHtmlBlocks",value:function(e){for(var t=[],n={bold:!1,ital:!1,mono:!1,spacing:!1,position:0,buffer:"",modStack:[],mod:function(){return 0===this.modStack.length?"normal":this.modStack[0]},modName:function(){return 0===this.modStack.length?"normal":"mono"===this.modStack[0]?"mono":n.bold&&n.ital?"boldital":n.bold?"bold":n.ital?"ital":void 0},emitBlock:function(){this.spacing&&(this.add(" "),this.spacing=!1),this.buffer.length>0&&(t.push({text:this.buffer,mod:this.modName()}),this.buffer="")},add:function(e){" "===e&&(n.spacing=!0),n.spacing&&(this.buffer+=" ",this.spacing=!1)," "!=e&&(this.buffer+=e)}};n.position<e.length;){var r=e.charAt(n.position);/[ \t]/.test(r)?n.mono?n.add(r):n.spacing=!0:/</.test(r)?n.mono||n.bold||!/<b>/.test(e.substr(n.position,3))?n.mono||n.ital||!/<i>/.test(e.substr(n.position,3))?!n.mono&&/<code>/.test(e.substr(n.position,6))?(n.emitBlock(),n.mono=!0,n.modStack.unshift("mono"),n.position+=5):!n.mono&&"bold"===n.mod()&&/<\/b>/.test(e.substr(n.position,4))?(n.emitBlock(),n.bold=!1,n.modStack.shift(),n.position+=3):!n.mono&&"ital"===n.mod()&&/<\/i>/.test(e.substr(n.position,4))?(n.emitBlock(),n.ital=!1,n.modStack.shift(),n.position+=3):"mono"===n.mod()&&/<\/code>/.test(e.substr(n.position,7))?(n.emitBlock(),n.mono=!1,n.modStack.shift(),n.position+=6):n.add(r):(n.emitBlock(),n.ital=!0,n.modStack.unshift("ital"),n.position+=2):(n.emitBlock(),n.bold=!0,n.modStack.unshift("bold"),n.position+=2):/&/.test(r)?/&lt;/.test(e.substr(n.position,4))?(n.add("<"),n.position+=3):/&amp;/.test(e.substr(n.position,5))?(n.add("&"),n.position+=4):n.add("&"):n.add(r),n.position++}return n.emitBlock(),t}},{key:"splitMarkdownBlocks",value:function(e){for(var t=[],n={bold:!1,ital:!1,mono:!1,beginable:!0,spacing:!1,position:0,buffer:"",modStack:[],mod:function(){return 0===this.modStack.length?"normal":this.modStack[0]},modName:function(){return 0===this.modStack.length?"normal":"mono"===this.modStack[0]?"mono":n.bold&&n.ital?"boldital":n.bold?"bold":n.ital?"ital":void 0},emitBlock:function(){this.spacing&&(this.add(" "),this.spacing=!1),this.buffer.length>0&&(t.push({text:this.buffer,mod:this.modName()}),this.buffer="")},add:function(e){" "===e&&(n.spacing=!0),n.spacing&&(this.buffer+=" ",this.spacing=!1)," "!=e&&(this.buffer+=e)}};n.position<e.length;){var r=e.charAt(n.position);/[ \t]/.test(r)?(n.mono?n.add(r):n.spacing=!0,n.beginable=!0):/\\/.test(r)?n.position<e.length+1&&(n.position++,r=e.charAt(n.position),/ \t/.test(r)?n.spacing=!0:(n.add(r),n.beginable=!1)):n.mono||n.bold||!n.beginable&&!n.spacing||!/\*/.test(r)?n.mono||n.ital||!n.beginable&&!n.spacing||!/\_/.test(r)?!n.mono&&(n.beginable||n.spacing)&&/`/.test(r)?(n.emitBlock(),n.mono=!0,n.modStack.unshift("mono")):!n.mono&&"bold"===n.mod()&&/\*/.test(r)?n.position===e.length-1||/[.,_` \t\n]/.test(e.charAt(n.position+1))?(n.emitBlock(),n.bold=!1,n.modStack.shift()):n.add(r):!n.mono&&"ital"===n.mod()&&/\_/.test(r)?n.position===e.length-1||/[.,*` \t\n]/.test(e.charAt(n.position+1))?(n.emitBlock(),n.ital=!1,n.modStack.shift()):n.add(r):n.mono&&"mono"===n.mod()&&/`/.test(r)?n.position===e.length-1||/[.,*_ \t\n]/.test(e.charAt(n.position+1))?(n.emitBlock(),n.mono=!1,n.modStack.shift()):n.add(r):(n.add(r),n.beginable=!1):(n.emitBlock(),n.ital=!0,n.modStack.unshift("ital")):(n.emitBlock(),n.bold=!0,n.modStack.unshift("bold")),n.position++}return n.emitBlock(),t}},{key:"splitBlocks",value:function(e,t){var n=this.decodeMarkupSystem(t);return"none"===n?[{text:e,mod:"normal"}]:"markdown"===n?this.splitMarkdownBlocks(e):"html"===n?this.splitHtmlBlocks(e):void 0}},{key:"overMaxWidth",value:function(e){var t=this.ctx.measureText(e).width;return this.lines.curWidth()+t>this.parent.fontOptions.maxWdt}},{key:"getLongestFit",value:function(e){for(var t="",n=0;n<e.length;){var r=t+(""===t?"":" ")+e[n];if(this.overMaxWidth(r))break;t=r,n++}return n}},{key:"getLongestFitWord",value:function(e){for(var t=0;t<e.length&&!this.overMaxWidth(e.slice(0,t));)t++;return t}},{key:"splitStringIntoLines",value:function(e){for(var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:"normal",n=arguments.length>2&&void 0!==arguments[2]&&arguments[2],r=(e=(e=e.replace(/^( +)/g,"$1\r")).replace(/([^\r][^ ]*)( +)/g,"$1\r$2\r")).split("\r");r.length>0;){var i=this.getLongestFit(r);if(0===i){var o=r[0],a=this.getLongestFitWord(o);this.lines.newLine(o.slice(0,a),t),r[0]=o.slice(a)}else{var s=i;" "===r[i-1]?i--:" "===r[s]&&s++;var l=r.slice(0,i).join("");i==r.length&&n?this.lines.append(l,t):this.lines.newLine(l,t),r=r.slice(s)}}}}]),e}();t.default=l},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=a(n(90)),i=a(n(0)),o=a(n(1));function a(e){return e&&e.__esModule?e:{default:e}}var s=function(){function e(t){(0,i.default)(this,e),this.measureText=t,this.current=0,this.width=0,this.height=0,this.lines=[]}return(0,o.default)(e,[{key:"_add",value:function(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:"normal";void 0===this.lines[e]&&(this.lines[e]={width:0,height:0,blocks:[]});var i=t;void 0!==t&&""!==t||(i=" ");var o=this.measureText(i,n),a=(0,r.default)({},o.values);a.text=t,a.width=o.width,a.mod=n,void 0!==t&&""!==t||(a.width=0),this.lines[e].blocks.push(a),this.lines[e].width+=a.width}},{key:"curWidth",value:function(){var e=this.lines[this.current];return void 0===e?0:e.width}},{key:"append",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:"normal";this._add(this.current,e,t)}},{key:"newLine",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:"normal";this._add(this.current,e,t),this.current++}},{key:"determineLineHeights",value:function(){for(var e=0;e<this.lines.length;e++){var t=this.lines[e],n=0;if(void 0!==t.blocks)for(var r=0;r<t.blocks.length;r++){var i=t.blocks[r];n<i.height&&(n=i.height)}t.height=n}}},{key:"determineLabelSize",value:function(){for(var e=0,t=0,n=0;n<this.lines.length;n++){var r=this.lines[n];r.width>e&&(e=r.width),t+=r.height}this.width=e,this.height=t}},{key:"removeEmptyBlocks",value:function(){for(var e=[],t=0;t<this.lines.length;t++){var n=this.lines[t];if(0!==n.blocks.length&&(t!==this.lines.length-1||0!==n.width)){var i={};(0,r.default)(i,n),i.blocks=[];for(var o=void 0,a=[],s=0;s<n.blocks.length;s++){var l=n.blocks[s];0!==l.width?a.push(l):void 0===o&&(o=l)}0===a.length&&void 0!==o&&a.push(o),i.blocks=a,e.push(i)}}return e}},{key:"finalize",value:function(){this.determineLineHeights(),this.determineLabelSize();var e=this.removeEmptyBlocks();return{width:this.width,height:this.height,lines:e}}}]),e}();t.default=s},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=l(n(3)),i=l(n(0)),o=l(n(1)),a=l(n(4)),s=l(n(5));function l(e){return e&&e.__esModule?e:{default:e}}var u=function(e){function t(e,n,o){(0,i.default)(this,t);var s=(0,a.default)(this,(t.__proto__||(0,r.default)(t)).call(this,e,n,o));return s._setMargins(o),s}return(0,s.default)(t,e),(0,o.default)(t,[{key:"resize",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:this.selected,n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:this.hover;if(this.needsRefresh(t,n)){var r=this.getDimensionsFromLabel(e,t,n);this.width=r.width+this.margin.right+this.margin.left,this.height=r.height+this.margin.top+this.margin.bottom,this.radius=this.width/2}}},{key:"draw",value:function(e,t,n,r,i,o){this.resize(e,r,i),this.left=t-this.width/2,this.top=n-this.height/2,this.initContextForDraw(e,o),e.roundRect(this.left,this.top,this.width,this.height,o.borderRadius),this.performFill(e,o),this.updateBoundingBox(t,n,e,r,i),this.labelModule.draw(e,this.left+this.textSize.width/2+this.margin.left,this.top+this.textSize.height/2+this.margin.top,r,i)}},{key:"updateBoundingBox",value:function(e,t,n,r,i){this._updateBoundingBox(e,t,n,r,i);var o=this.options.shapeProperties.borderRadius;this._addBoundingBoxMargin(o)}},{key:"distanceToBorder",value:function(e,t){this.resize(e);var n=this.options.borderWidth;return Math.min(Math.abs(this.width/2/Math.cos(t)),Math.abs(this.height/2/Math.sin(t)))+n}}]),t}(l(n(23)).default);t.default=u},function(e,t,n){n(195),e.exports=n(7).Object.getPrototypeOf},function(e,t,n){var r=n(41),i=n(85);n(87)("getPrototypeOf",(function(){return function(e){return i(r(e))}}))},function(e,t,n){e.exports={default:n(197),__esModule:!0}},function(e,t,n){n(198),e.exports=n(7).Object.setPrototypeOf},function(e,t,n){var r=n(17);r(r.S,"Object",{setPrototypeOf:n(199).set})},function(e,t,n){var r=n(32),i=n(27),o=function(e,t){if(i(e),!r(t)&&null!==t)throw TypeError(t+": can't set as prototype!")};e.exports={set:Object.setPrototypeOf||("__proto__"in{}?function(e,t,r){try{(r=n(80)(Function.call,n(89).f(Object.prototype,"__proto__").set,2))(e,[]),t=!(e instanceof Array)}catch(e){t=!0}return function(e,n){return o(e,n),t?e.__proto__=n:r(e,n),e}}({},!1):void 0),check:o}},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=l(n(3)),i=l(n(0)),o=l(n(1)),a=l(n(4)),s=l(n(5));function l(e){return e&&e.__esModule?e:{default:e}}var u=function(e){function t(e,n,o){(0,i.default)(this,t);var s=(0,a.default)(this,(t.__proto__||(0,r.default)(t)).call(this,e,n,o));return s._setMargins(o),s}return(0,s.default)(t,e),(0,o.default)(t,[{key:"resize",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:this.selected,n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:this.hover;if(this.needsRefresh(t,n)){var r=this.getDimensionsFromLabel(e,t,n),i=Math.max(r.width+this.margin.right+this.margin.left,r.height+this.margin.top+this.margin.bottom);this.options.size=i/2,this.width=i,this.height=i,this.radius=this.width/2}}},{key:"draw",value:function(e,t,n,r,i,o){this.resize(e,r,i),this.left=t-this.width/2,this.top=n-this.height/2,this._drawRawCircle(e,t,n,o),this.updateBoundingBox(t,n),this.labelModule.draw(e,this.left+this.textSize.width/2+this.margin.left,n,r,i)}},{key:"updateBoundingBox",value:function(e,t){this.boundingBox.top=t-this.options.size,this.boundingBox.left=e-this.options.size,this.boundingBox.right=e+this.options.size,this.boundingBox.bottom=t+this.options.size}},{key:"distanceToBorder",value:function(e,t){return this.resize(e),.5*this.width}}]),t}(l(n(73)).default);t.default=u},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=l(n(3)),i=l(n(0)),o=l(n(1)),a=l(n(4)),s=l(n(5));function l(e){return e&&e.__esModule?e:{default:e}}var u=function(e){function t(e,n,o,s,l){(0,i.default)(this,t);var u=(0,a.default)(this,(t.__proto__||(0,r.default)(t)).call(this,e,n,o));return u.setImages(s,l),u}return(0,s.default)(t,e),(0,o.default)(t,[{key:"resize",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:this.selected,n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:this.hover,r=void 0===this.imageObj.src||void 0===this.imageObj.width||void 0===this.imageObj.height;if(r){var i=2*this.options.size;return this.width=i,this.height=i,void(this.radius=.5*this.width)}this.needsRefresh(t,n)&&this._resizeImage()}},{key:"draw",value:function(e,t,n,r,i,o){this.switchImages(r),this.resize(),this.left=t-this.width/2,this.top=n-this.height/2,this._drawRawCircle(e,t,n,o),e.save(),e.clip(),this._drawImageAtPosition(e,o),e.restore(),this._drawImageLabel(e,t,n,r,i),this.updateBoundingBox(t,n)}},{key:"updateBoundingBox",value:function(e,t){this.boundingBox.top=t-this.options.size,this.boundingBox.left=e-this.options.size,this.boundingBox.right=e+this.options.size,this.boundingBox.bottom=t+this.options.size,this.boundingBox.left=Math.min(this.boundingBox.left,this.labelModule.size.left),this.boundingBox.right=Math.max(this.boundingBox.right,this.labelModule.size.left+this.labelModule.size.width),this.boundingBox.bottom=Math.max(this.boundingBox.bottom,this.boundingBox.bottom+this.labelOffset)}},{key:"distanceToBorder",value:function(e,t){return this.resize(e),.5*this.width}}]),t}(l(n(73)).default);t.default=u},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=l(n(3)),i=l(n(0)),o=l(n(1)),a=l(n(4)),s=l(n(5));function l(e){return e&&e.__esModule?e:{default:e}}var u=function(e){function t(e,n,o){(0,i.default)(this,t);var s=(0,a.default)(this,(t.__proto__||(0,r.default)(t)).call(this,e,n,o));return s._setMargins(o),s}return(0,s.default)(t,e),(0,o.default)(t,[{key:"resize",value:function(e,t,n){if(this.needsRefresh(t,n)){var r=this.getDimensionsFromLabel(e,t,n).width+this.margin.right+this.margin.left;this.width=r,this.height=r,this.radius=this.width/2}}},{key:"draw",value:function(e,t,n,r,i,o){this.resize(e,r,i),this.left=t-this.width/2,this.top=n-this.height/2,this.initContextForDraw(e,o),e.database(t-this.width/2,n-this.height/2,this.width,this.height),this.performFill(e,o),this.updateBoundingBox(t,n,e,r,i),this.labelModule.draw(e,this.left+this.textSize.width/2+this.margin.left,this.top+this.textSize.height/2+this.margin.top,r,i)}},{key:"distanceToBorder",value:function(e,t){return this._distanceToBorder(e,t)}}]),t}(l(n(23)).default);t.default=u},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=l(n(3)),i=l(n(0)),o=l(n(1)),a=l(n(4)),s=l(n(5));function l(e){return e&&e.__esModule?e:{default:e}}var u=function(e){function t(e,n,o){return(0,i.default)(this,t),(0,a.default)(this,(t.__proto__||(0,r.default)(t)).call(this,e,n,o))}return(0,s.default)(t,e),(0,o.default)(t,[{key:"draw",value:function(e,t,n,r,i,o){this._drawShape(e,"diamond",4,t,n,r,i,o)}},{key:"distanceToBorder",value:function(e,t){return this._distanceToBorder(e,t)}}]),t}(l(n(24)).default);t.default=u},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=l(n(3)),i=l(n(0)),o=l(n(1)),a=l(n(4)),s=l(n(5));function l(e){return e&&e.__esModule?e:{default:e}}var u=function(e){function t(e,n,o){return(0,i.default)(this,t),(0,a.default)(this,(t.__proto__||(0,r.default)(t)).call(this,e,n,o))}return(0,s.default)(t,e),(0,o.default)(t,[{key:"draw",value:function(e,t,n,r,i,o){this._drawShape(e,"circle",2,t,n,r,i,o)}},{key:"distanceToBorder",value:function(e,t){return this.resize(e),this.options.size}}]),t}(l(n(24)).default);t.default=u},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=l(n(3)),i=l(n(0)),o=l(n(1)),a=l(n(4)),s=l(n(5));function l(e){return e&&e.__esModule?e:{default:e}}var u=function(e){function t(e,n,o){return(0,i.default)(this,t),(0,a.default)(this,(t.__proto__||(0,r.default)(t)).call(this,e,n,o))}return(0,s.default)(t,e),(0,o.default)(t,[{key:"resize",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:this.selected,n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:this.hover;if(this.needsRefresh(t,n)){var r=this.getDimensionsFromLabel(e,t,n);this.height=2*r.height,this.width=r.width+r.height,this.radius=.5*this.width}}},{key:"draw",value:function(e,t,n,r,i,o){this.resize(e,r,i),this.left=t-.5*this.width,this.top=n-.5*this.height,this.initContextForDraw(e,o),e.ellipse_vis(this.left,this.top,this.width,this.height),this.performFill(e,o),this.updateBoundingBox(t,n,e,r,i),this.labelModule.draw(e,t,n,r,i)}},{key:"distanceToBorder",value:function(e,t){this.resize(e);var n=.5*this.width,r=.5*this.height,i=Math.sin(t)*n,o=Math.cos(t)*r;return n*r/Math.sqrt(i*i+o*o)}}]),t}(l(n(23)).default);t.default=u},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=l(n(3)),i=l(n(0)),o=l(n(1)),a=l(n(4)),s=l(n(5));function l(e){return e&&e.__esModule?e:{default:e}}var u=function(e){function t(e,n,o){(0,i.default)(this,t);var s=(0,a.default)(this,(t.__proto__||(0,r.default)(t)).call(this,e,n,o));return s._setMargins(o),s}return(0,s.default)(t,e),(0,o.default)(t,[{key:"resize",value:function(e,t,n){this.needsRefresh(t,n)&&(this.iconSize={width:Number(this.options.icon.size),height:Number(this.options.icon.size)},this.width=this.iconSize.width+this.margin.right+this.margin.left,this.height=this.iconSize.height+this.margin.top+this.margin.bottom,this.radius=.5*this.width)}},{key:"draw",value:function(e,t,n,r,i,o){this.resize(e,r,i),this.options.icon.size=this.options.icon.size||50,this.left=t-this.width/2,this.top=n-this.height/2,this._icon(e,t,n,r,i,o),void 0!==this.options.label&&this.labelModule.draw(e,this.left+this.iconSize.width/2+this.margin.left,n+this.height/2+5,r),this.updateBoundingBox(t,n)}},{key:"updateBoundingBox",value:function(e,t){this.boundingBox.top=t-.5*this.options.icon.size,this.boundingBox.left=e-.5*this.options.icon.size,this.boundingBox.right=e+.5*this.options.icon.size,this.boundingBox.bottom=t+.5*this.options.icon.size,void 0!==this.options.label&&this.labelModule.size.width>0&&(this.boundingBox.left=Math.min(this.boundingBox.left,this.labelModule.size.left),this.boundingBox.right=Math.max(this.boundingBox.right,this.labelModule.size.left+this.labelModule.size.width),this.boundingBox.bottom=Math.max(this.boundingBox.bottom,this.boundingBox.bottom+this.labelModule.size.height+5))}},{key:"_icon",value:function(e,t,n,r,i,o){var a=Number(this.options.icon.size);void 0!==this.options.icon.code?(e.font=(r?"bold ":"")+a+"px "+this.options.icon.face,e.fillStyle=this.options.icon.color||"black",e.textAlign="center",e.textBaseline="middle",this.enableShadow(e,o),e.fillText(this.options.icon.code,t,n),this.disableShadow(e,o)):console.error("When using the icon shape, you need to define the code in the icon options object. This can be done per node or globally.")}},{key:"distanceToBorder",value:function(e,t){return this._distanceToBorder(e,t)}}]),t}(l(n(23)).default);t.default=u},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=l(n(3)),i=l(n(0)),o=l(n(1)),a=l(n(4)),s=l(n(5));function l(e){return e&&e.__esModule?e:{default:e}}var u=function(e){function t(e,n,o,s,l){(0,i.default)(this,t);var u=(0,a.default)(this,(t.__proto__||(0,r.default)(t)).call(this,e,n,o));return u.setImages(s,l),u}return(0,s.default)(t,e),(0,o.default)(t,[{key:"resize",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:this.selected,n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:this.hover,r=void 0===this.imageObj.src||void 0===this.imageObj.width||void 0===this.imageObj.height;if(r){var i=2*this.options.size;return this.width=i,void(this.height=i)}this.needsRefresh(t,n)&&this._resizeImage()}},{key:"draw",value:function(e,t,n,r,i,o){if(this.switchImages(r),this.resize(),this.left=t-this.width/2,this.top=n-this.height/2,!0===this.options.shapeProperties.useBorderWithImage){var a=this.options.borderWidth,s=this.options.borderWidthSelected||2*this.options.borderWidth,l=(r?s:a)/this.body.view.scale;e.lineWidth=Math.min(this.width,l),e.beginPath(),e.strokeStyle=r?this.options.color.highlight.border:i?this.options.color.hover.border:this.options.color.border,e.fillStyle=r?this.options.color.highlight.background:i?this.options.color.hover.background:this.options.color.background,e.rect(this.left-.5*e.lineWidth,this.top-.5*e.lineWidth,this.width+e.lineWidth,this.height+e.lineWidth),e.fill(),this.performStroke(e,o),e.closePath()}this._drawImageAtPosition(e,o),this._drawImageLabel(e,t,n,r,i),this.updateBoundingBox(t,n)}},{key:"updateBoundingBox",value:function(e,t){this.resize(),this._updateBoundingBox(e,t),void 0!==this.options.label&&this.labelModule.size.width>0&&(this.boundingBox.left=Math.min(this.boundingBox.left,this.labelModule.size.left),this.boundingBox.right=Math.max(this.boundingBox.right,this.labelModule.size.left+this.labelModule.size.width),this.boundingBox.bottom=Math.max(this.boundingBox.bottom,this.boundingBox.bottom+this.labelOffset))}},{key:"distanceToBorder",value:function(e,t){return this._distanceToBorder(e,t)}}]),t}(l(n(73)).default);t.default=u},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=l(n(3)),i=l(n(0)),o=l(n(1)),a=l(n(4)),s=l(n(5));function l(e){return e&&e.__esModule?e:{default:e}}var u=function(e){function t(e,n,o){return(0,i.default)(this,t),(0,a.default)(this,(t.__proto__||(0,r.default)(t)).call(this,e,n,o))}return(0,s.default)(t,e),(0,o.default)(t,[{key:"draw",value:function(e,t,n,r,i,o){this._drawShape(e,"square",2,t,n,r,i,o)}},{key:"distanceToBorder",value:function(e,t){return this._distanceToBorder(e,t)}}]),t}(l(n(24)).default);t.default=u},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=l(n(3)),i=l(n(0)),o=l(n(1)),a=l(n(4)),s=l(n(5));function l(e){return e&&e.__esModule?e:{default:e}}var u=function(e){function t(e,n,o){return(0,i.default)(this,t),(0,a.default)(this,(t.__proto__||(0,r.default)(t)).call(this,e,n,o))}return(0,s.default)(t,e),(0,o.default)(t,[{key:"draw",value:function(e,t,n,r,i,o){this._drawShape(e,"hexagon",4,t,n,r,i,o)}},{key:"distanceToBorder",value:function(e,t){return this._distanceToBorder(e,t)}}]),t}(l(n(24)).default);t.default=u},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=l(n(3)),i=l(n(0)),o=l(n(1)),a=l(n(4)),s=l(n(5));function l(e){return e&&e.__esModule?e:{default:e}}var u=function(e){function t(e,n,o){return(0,i.default)(this,t),(0,a.default)(this,(t.__proto__||(0,r.default)(t)).call(this,e,n,o))}return(0,s.default)(t,e),(0,o.default)(t,[{key:"draw",value:function(e,t,n,r,i,o){this._drawShape(e,"star",4,t,n,r,i,o)}},{key:"distanceToBorder",value:function(e,t){return this._distanceToBorder(e,t)}}]),t}(l(n(24)).default);t.default=u},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=l(n(3)),i=l(n(0)),o=l(n(1)),a=l(n(4)),s=l(n(5));function l(e){return e&&e.__esModule?e:{default:e}}var u=function(e){function t(e,n,o){(0,i.default)(this,t);var s=(0,a.default)(this,(t.__proto__||(0,r.default)(t)).call(this,e,n,o));return s._setMargins(o),s}return(0,s.default)(t,e),(0,o.default)(t,[{key:"resize",value:function(e,t,n){this.needsRefresh(t,n)&&(this.textSize=this.labelModule.getTextSize(e,t,n),this.width=this.textSize.width+this.margin.right+this.margin.left,this.height=this.textSize.height+this.margin.top+this.margin.bottom,this.radius=.5*this.width)}},{key:"draw",value:function(e,t,n,r,i,o){this.resize(e,r,i),this.left=t-this.width/2,this.top=n-this.height/2,this.enableShadow(e,o),this.labelModule.draw(e,this.left+this.textSize.width/2+this.margin.left,this.top+this.textSize.height/2+this.margin.top,r,i),this.disableShadow(e,o),this.updateBoundingBox(t,n,e,r,i)}},{key:"distanceToBorder",value:function(e,t){return this._distanceToBorder(e,t)}}]),t}(l(n(23)).default);t.default=u},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=l(n(3)),i=l(n(0)),o=l(n(1)),a=l(n(4)),s=l(n(5));function l(e){return e&&e.__esModule?e:{default:e}}var u=function(e){function t(e,n,o){return(0,i.default)(this,t),(0,a.default)(this,(t.__proto__||(0,r.default)(t)).call(this,e,n,o))}return(0,s.default)(t,e),(0,o.default)(t,[{key:"draw",value:function(e,t,n,r,i,o){this._drawShape(e,"triangle",3,t,n,r,i,o)}},{key:"distanceToBorder",value:function(e,t){return this._distanceToBorder(e,t)}}]),t}(l(n(24)).default);t.default=u},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=l(n(3)),i=l(n(0)),o=l(n(1)),a=l(n(4)),s=l(n(5));function l(e){return e&&e.__esModule?e:{default:e}}var u=function(e){function t(e,n,o){return(0,i.default)(this,t),(0,a.default)(this,(t.__proto__||(0,r.default)(t)).call(this,e,n,o))}return(0,s.default)(t,e),(0,o.default)(t,[{key:"draw",value:function(e,t,n,r,i,o){this._drawShape(e,"triangleDown",3,t,n,r,i,o)}},{key:"distanceToBorder",value:function(e,t){return this._distanceToBorder(e,t)}}]),t}(l(n(24)).default);t.default=u},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=o(n(0)),i=o(n(1));function o(e){return e&&e.__esModule?e:{default:e}}var a=n(2),s=n(11),l=n(12),u=n(74).default,c=function(){function e(t,n,i){var o=this;(0,r.default)(this,e),this.body=t,this.images=n,this.groups=i,this.body.functions.createEdge=this.create.bind(this),this.edgesListeners={add:function(e,t){o.add(t.items)},update:function(e,t){o.update(t.items)},remove:function(e,t){o.remove(t.items)}},this.options={},this.defaultOptions={arrows:{to:{enabled:!1,scaleFactor:1,type:"arrow"},middle:{enabled:!1,scaleFactor:1,type:"arrow"},from:{enabled:!1,scaleFactor:1,type:"arrow"}},arrowStrikethrough:!0,color:{color:"#848484",highlight:"#848484",hover:"#848484",inherit:"from",opacity:1},dashes:!1,font:{color:"#343434",size:14,face:"arial",background:"none",strokeWidth:2,strokeColor:"#ffffff",align:"horizontal",multi:!1,vadjust:0,bold:{mod:"bold"},boldital:{mod:"bold italic"},ital:{mod:"italic"},mono:{mod:"",size:15,face:"courier new",vadjust:2}},hidden:!1,hoverWidth:1.5,label:void 0,labelHighlightBold:!0,length:void 0,physics:!0,scaling:{min:1,max:15,label:{enabled:!0,min:14,max:30,maxVisible:30,drawThreshold:5},customScalingFunction:function(e,t,n,r){if(t===e)return.5;var i=1/(t-e);return Math.max(0,(r-e)*i)}},selectionWidth:1.5,selfReferenceSize:20,shadow:{enabled:!1,color:"rgba(0,0,0,0.5)",size:10,x:5,y:5},smooth:{enabled:!0,type:"dynamic",forceDirection:"none",roundness:.5},title:void 0,width:1,value:void 0},a.deepExtend(this.options,this.defaultOptions),this.bindEventListeners()}return(0,i.default)(e,[{key:"bindEventListeners",value:function(){var e=this;this.body.emitter.on("_forceDisableDynamicCurves",(function(t){var n=!(arguments.length>1&&void 0!==arguments[1])||arguments[1];"dynamic"===t&&(t="continuous");var r=!1;for(var i in e.body.edges)if(e.body.edges.hasOwnProperty(i)){var o=e.body.edges[i],a=e.body.data.edges._data[i];if(void 0!==a){var s=a.smooth;void 0!==s&&!0===s.enabled&&"dynamic"===s.type&&(void 0===t?o.setOptions({smooth:!1}):o.setOptions({smooth:{type:t}}),r=!0)}}!0===n&&!0===r&&e.body.emitter.emit("_dataChanged")})),this.body.emitter.on("_dataUpdated",(function(){e.reconnectEdges()})),this.body.emitter.on("refreshEdges",this.refresh.bind(this)),this.body.emitter.on("refresh",this.refresh.bind(this)),this.body.emitter.on("destroy",(function(){a.forEach(e.edgesListeners,(function(t,n){e.body.data.edges&&e.body.data.edges.off(n,t)})),delete e.body.functions.createEdge,delete e.edgesListeners.add,delete e.edgesListeners.update,delete e.edgesListeners.remove,delete e.edgesListeners}))}},{key:"setOptions",value:function(e){if(void 0!==e){u.parseOptions(this.options,e,!0,this.defaultOptions,!0);var t=!1;if(void 0!==e.smooth)for(var n in this.body.edges)this.body.edges.hasOwnProperty(n)&&(t=this.body.edges[n].updateEdgeType()||t);if(void 0!==e.font)for(var r in this.body.edges)this.body.edges.hasOwnProperty(r)&&this.body.edges[r].updateLabelModule();void 0===e.hidden&&void 0===e.physics&&!0!==t||this.body.emitter.emit("_dataChanged")}}},{key:"setData",value:function(e){var t=this,n=arguments.length>1&&void 0!==arguments[1]&&arguments[1],r=this.body.data.edges;if(e instanceof s||e instanceof l)this.body.data.edges=e;else if(Array.isArray(e))this.body.data.edges=new s,this.body.data.edges.add(e);else{if(e)throw new TypeError("Array or DataSet expected");this.body.data.edges=new s}if(r&&a.forEach(this.edgesListeners,(function(e,t){r.off(t,e)})),this.body.edges={},this.body.data.edges){a.forEach(this.edgesListeners,(function(e,n){t.body.data.edges.on(n,e)}));var i=this.body.data.edges.getIds();this.add(i,!0)}this.body.emitter.emit("_adjustEdgesForHierarchicalLayout"),!1===n&&this.body.emitter.emit("_dataChanged")}},{key:"add",value:function(e){for(var t=arguments.length>1&&void 0!==arguments[1]&&arguments[1],n=this.body.edges,r=this.body.data.edges,i=0;i<e.length;i++){var o=e[i],a=n[o];a&&a.disconnect();var s=r.get(o,{showInternalIds:!0});n[o]=this.create(s)}this.body.emitter.emit("_adjustEdgesForHierarchicalLayout"),!1===t&&this.body.emitter.emit("_dataChanged")}},{key:"update",value:function(e){for(var t=this.body.edges,n=this.body.data.edges,r=!1,i=0;i<e.length;i++){var o=e[i],a=n.get(o),s=t[o];void 0!==s?(s.disconnect(),r=s.setOptions(a)||r,s.connect()):(this.body.edges[o]=this.create(a),r=!0)}!0===r?(this.body.emitter.emit("_adjustEdgesForHierarchicalLayout"),this.body.emitter.emit("_dataChanged")):this.body.emitter.emit("_dataUpdated")}},{key:"remove",value:function(e){var t=!(arguments.length>1&&void 0!==arguments[1])||arguments[1];if(0!==e.length){var n=this.body.edges;a.forEach(e,(function(e){var t=n[e];void 0!==t&&t.remove()})),t&&this.body.emitter.emit("_dataChanged")}}},{key:"refresh",value:function(){var e=this;a.forEach(this.body.edges,(function(t,n){var r=e.body.data.edges._data[n];void 0!==r&&t.setOptions(r)}))}},{key:"create",value:function(e){return new u(e,this.body,this.options,this.defaultOptions)}},{key:"reconnectEdges",value:function(){var e,t=this.body.nodes,n=this.body.edges;for(e in t)t.hasOwnProperty(e)&&(t[e].edges=[]);for(e in n)if(n.hasOwnProperty(e)){var r=n[e];r.from=null,r.to=null,r.connect()}}},{key:"getConnectedNodes",value:function(e){var t=[];if(void 0!==this.body.edges[e]){var n=this.body.edges[e];void 0!==n.fromId&&t.push(n.fromId),void 0!==n.toId&&t.push(n.toId)}return t}},{key:"_updateState",value:function(){this._addMissingEdges(),this._removeInvalidEdges()}},{key:"_removeInvalidEdges",value:function(){var e=this,t=[];a.forEach(this.body.edges,(function(n,r){var i=e.body.nodes[n.toId],o=e.body.nodes[n.fromId];void 0!==i&&!0===i.isCluster||void 0!==o&&!0===o.isCluster||void 0!==i&&void 0!==o||t.push(r)})),this.remove(t,!1)}},{key:"_addMissingEdges",value:function(){var e=this.body.edges,t=this.body.data.edges,n=[];t.forEach((function(t,r){void 0===e[r]&&n.push(r)})),this.add(n,!0)}}]),e}();t.default=c},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=u(n(30)),i=u(n(3)),o=u(n(0)),a=u(n(1)),s=u(n(4)),l=u(n(5));function u(e){return e&&e.__esModule?e:{default:e}}var c=function(e){function t(e,n,r){return(0,o.default)(this,t),(0,s.default)(this,(t.__proto__||(0,i.default)(t)).call(this,e,n,r))}return(0,l.default)(t,e),(0,a.default)(t,[{key:"_line",value:function(e,t,n){var r=n[0],i=n[1];this._bezierCurve(e,t,r,i)}},{key:"_getViaCoordinates",value:function(){var e=this.from.x-this.to.x,t=this.from.y-this.to.y,n=void 0,r=void 0,i=void 0,o=void 0,a=this.options.smooth.roundness;return(Math.abs(e)>Math.abs(t)||!0===this.options.smooth.forceDirection||"horizontal"===this.options.smooth.forceDirection)&&"vertical"!==this.options.smooth.forceDirection?(r=this.from.y,o=this.to.y,n=this.from.x-a*e,i=this.to.x+a*e):(r=this.from.y-a*t,o=this.to.y+a*t,n=this.from.x,i=this.to.x),[{x:n,y:r},{x:i,y:o}]}},{key:"getViaNode",value:function(){return this._getViaCoordinates()}},{key:"_findBorderPosition",value:function(e,t){return this._findBorderPositionBezier(e,t)}},{key:"_getDistanceToEdge",value:function(e,t,n,i,o,a){var s=arguments.length>6&&void 0!==arguments[6]?arguments[6]:this._getViaCoordinates(),l=(0,r.default)(s,2),u=l[0],c=l[1];return this._getDistanceToBezierEdge(e,t,n,i,o,a,u,c)}},{key:"getPoint",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:this._getViaCoordinates(),n=(0,r.default)(t,2),i=n[0],o=n[1],a=e,s=[];s[0]=Math.pow(1-a,3),s[1]=3*a*Math.pow(1-a,2),s[2]=3*Math.pow(a,2)*(1-a),s[3]=Math.pow(a,3);var l=s[0]*this.fromPoint.x+s[1]*i.x+s[2]*o.x+s[3]*this.toPoint.x,u=s[0]*this.fromPoint.y+s[1]*i.y+s[2]*o.y+s[3]*this.toPoint.y;return{x:l,y:u}}}]),t}(u(n(216)).default);t.default=c},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=l(n(3)),i=l(n(0)),o=l(n(1)),a=l(n(4)),s=l(n(5));function l(e){return e&&e.__esModule?e:{default:e}}var u=function(e){function t(e,n,o){return(0,i.default)(this,t),(0,a.default)(this,(t.__proto__||(0,r.default)(t)).call(this,e,n,o))}return(0,s.default)(t,e),(0,o.default)(t,[{key:"_getDistanceToBezierEdge",value:function(e,t,n,r,i,o,a,s){var l=1e9,u=void 0,c=void 0,d=void 0,h=void 0,f=void 0,p=e,g=t,v=[0,0,0,0];for(c=1;c<10;c++)d=.1*c,v[0]=Math.pow(1-d,3),v[1]=3*d*Math.pow(1-d,2),v[2]=3*Math.pow(d,2)*(1-d),v[3]=Math.pow(d,3),h=v[0]*e+v[1]*a.x+v[2]*s.x+v[3]*n,f=v[0]*t+v[1]*a.y+v[2]*s.y+v[3]*r,c>0&&(l=(u=this._getDistanceToLine(p,g,h,f,i,o))<l?u:l),p=h,g=f;return l}}]),t}(l(n(75)).default);t.default=u},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=u(n(30)),i=u(n(3)),o=u(n(0)),a=u(n(1)),s=u(n(4)),l=u(n(5));function u(e){return e&&e.__esModule?e:{default:e}}var c=function(e){function t(e,n,r){(0,o.default)(this,t);var a=(0,s.default)(this,(t.__proto__||(0,i.default)(t)).call(this,e,n,r));return a._boundFunction=function(){a.positionBezierNode()},a.body.emitter.on("_repositionBezierNodes",a._boundFunction),a}return(0,l.default)(t,e),(0,a.default)(t,[{key:"setOptions",value:function(e){var t=!1;this.options.physics!==e.physics&&(t=!0),this.options=e,this.id=this.options.id,this.from=this.body.nodes[this.options.from],this.to=this.body.nodes[this.options.to],this.setupSupportNode(),this.connect(),!0===t&&(this.via.setOptions({physics:this.options.physics}),this.positionBezierNode())}},{key:"connect",value:function(){this.from=this.body.nodes[this.options.from],this.to=this.body.nodes[this.options.to],void 0===this.from||void 0===this.to||!1===this.options.physics||this.from.id===this.to.id?this.via.setOptions({physics:!1}):this.via.setOptions({physics:!0})}},{key:"cleanup",value:function(){return this.body.emitter.off("_repositionBezierNodes",this._boundFunction),void 0!==this.via&&(delete this.body.nodes[this.via.id],this.via=void 0,!0)}},{key:"setupSupportNode",value:function(){if(void 0===this.via){var e="edgeId:"+this.id,t=this.body.functions.createNode({id:e,shape:"circle",physics:!0,hidden:!0});this.body.nodes[e]=t,this.via=t,this.via.parentEdgeId=this.id,this.positionBezierNode()}}},{key:"positionBezierNode",value:function(){void 0!==this.via&&void 0!==this.from&&void 0!==this.to?(this.via.x=.5*(this.from.x+this.to.x),this.via.y=.5*(this.from.y+this.to.y)):void 0!==this.via&&(this.via.x=0,this.via.y=0)}},{key:"_line",value:function(e,t,n){this._bezierCurve(e,t,n)}},{key:"getViaNode",value:function(){return this.via}},{key:"getPoint",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:this.via,n=e,i=void 0,o=void 0;if(this.from===this.to){var a=this._getCircleData(this.from),s=(0,r.default)(a,3),l=s[0],u=s[1],c=s[2],d=2*Math.PI*(1-n);i=l+c*Math.sin(d),o=u+c-c*(1-Math.cos(d))}else i=Math.pow(1-n,2)*this.fromPoint.x+2*n*(1-n)*t.x+Math.pow(n,2)*this.toPoint.x,o=Math.pow(1-n,2)*this.fromPoint.y+2*n*(1-n)*t.y+Math.pow(n,2)*this.toPoint.y;return{x:i,y:o}}},{key:"_findBorderPosition",value:function(e,t){return this._findBorderPositionBezier(e,t,this.via)}},{key:"_getDistanceToEdge",value:function(e,t,n,r,i,o){return this._getDistanceToBezierEdge(e,t,n,r,i,o,this.via)}}]),t}(u(n(75)).default);t.default=c},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=l(n(3)),i=l(n(0)),o=l(n(1)),a=l(n(4)),s=l(n(5));function l(e){return e&&e.__esModule?e:{default:e}}var u=function(e){function t(e,n,o){return(0,i.default)(this,t),(0,a.default)(this,(t.__proto__||(0,r.default)(t)).call(this,e,n,o))}return(0,s.default)(t,e),(0,o.default)(t,[{key:"_line",value:function(e,t,n){this._bezierCurve(e,t,n)}},{key:"getViaNode",value:function(){return this._getViaCoordinates()}},{key:"_getViaCoordinates",value:function(){var e=void 0,t=void 0,n=this.options.smooth.roundness,r=this.options.smooth.type,i=Math.abs(this.from.x-this.to.x),o=Math.abs(this.from.y-this.to.y);if("discrete"===r||"diagonalCross"===r){var a=void 0,s=void 0;a=s=i<=o?n*o:n*i,this.from.x>this.to.x&&(a=-a),this.from.y>=this.to.y&&(s=-s),e=this.from.x+a,t=this.from.y+s,"discrete"===r&&(i<=o?e=i<n*o?this.from.x:e:t=o<n*i?this.from.y:t)}else if("straightCross"===r){var l=(1-n)*i,u=(1-n)*o;i<=o?(l=0,this.from.y<this.to.y&&(u=-u)):(this.from.x<this.to.x&&(l=-l),u=0),e=this.to.x+l,t=this.to.y+u}else if("horizontal"===r){var c=(1-n)*i;this.from.x<this.to.x&&(c=-c),e=this.to.x+c,t=this.from.y}else if("vertical"===r){var d=(1-n)*o;this.from.y<this.to.y&&(d=-d),e=this.from.x,t=this.to.y+d}else if("curvedCW"===r){i=this.to.x-this.from.x,o=this.from.y-this.to.y;var h=Math.sqrt(i*i+o*o),f=Math.PI,p=(Math.atan2(o,i)+(.5*n+.5)*f)%(2*f);e=this.from.x+(.5*n+.5)*h*Math.sin(p),t=this.from.y+(.5*n+.5)*h*Math.cos(p)}else if("curvedCCW"===r){i=this.to.x-this.from.x,o=this.from.y-this.to.y;var g=Math.sqrt(i*i+o*o),v=Math.PI,m=(Math.atan2(o,i)+(.5*-n+.5)*v)%(2*v);e=this.from.x+(.5*n+.5)*g*Math.sin(m),t=this.from.y+(.5*n+.5)*g*Math.cos(m)}else{var y=void 0,b=void 0;y=b=i<=o?n*o:n*i,this.from.x>this.to.x&&(y=-y),this.from.y>=this.to.y&&(b=-b),e=this.from.x+y,t=this.from.y+b,i<=o?e=this.from.x<=this.to.x?this.to.x<e?this.to.x:e:this.to.x>e?this.to.x:e:t=this.from.y>=this.to.y?this.to.y>t?this.to.y:t:this.to.y<t?this.to.y:t}return{x:e,y:t}}},{key:"_findBorderPosition",value:function(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};return this._findBorderPositionBezier(e,t,n.via)}},{key:"_getDistanceToEdge",value:function(e,t,n,r,i,o){var a=arguments.length>6&&void 0!==arguments[6]?arguments[6]:this._getViaCoordinates();return this._getDistanceToBezierEdge(e,t,n,r,i,o,a)}},{key:"getPoint",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:this._getViaCoordinates(),n=e,r=Math.pow(1-n,2)*this.fromPoint.x+2*n*(1-n)*t.x+Math.pow(n,2)*this.toPoint.x,i=Math.pow(1-n,2)*this.fromPoint.y+2*n*(1-n)*t.y+Math.pow(n,2)*this.toPoint.y;return{x:r,y:i}}}]),t}(l(n(75)).default);t.default=u},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=l(n(3)),i=l(n(0)),o=l(n(1)),a=l(n(4)),s=l(n(5));function l(e){return e&&e.__esModule?e:{default:e}}var u=function(e){function t(e,n,o){return(0,i.default)(this,t),(0,a.default)(this,(t.__proto__||(0,r.default)(t)).call(this,e,n,o))}return(0,s.default)(t,e),(0,o.default)(t,[{key:"_line",value:function(e,t){e.beginPath(),e.moveTo(this.fromPoint.x,this.fromPoint.y),e.lineTo(this.toPoint.x,this.toPoint.y),this.enableShadow(e,t),e.stroke(),this.disableShadow(e,t)}},{key:"getViaNode",value:function(){}},{key:"getPoint",value:function(e){return{x:(1-e)*this.fromPoint.x+e*this.toPoint.x,y:(1-e)*this.fromPoint.y+e*this.toPoint.y}}},{key:"_findBorderPosition",value:function(e,t){var n=this.to,r=this.from;e.id===this.from.id&&(n=this.from,r=this.to);var i=Math.atan2(n.y-r.y,n.x-r.x),o=n.x-r.x,a=n.y-r.y,s=Math.sqrt(o*o+a*a),l=(s-e.distanceToBorder(t,i))/s,u={};return u.x=(1-l)*r.x+l*n.x,u.y=(1-l)*r.y+l*n.y,u}},{key:"_getDistanceToEdge",value:function(e,t,n,r,i,o){return this._getDistanceToLine(e,t,n,r,i,o)}}]),t}(l(n(118)).default);t.default=u},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=a(n(8)),i=a(n(0)),o=a(n(1));function a(e){return e&&e.__esModule?e:{default:e}}var s=n(120).default,l=n(221).default,u=n(222).default,c=n(223).default,d=n(224).default,h=n(121).default,f=n(225).default,p=n(226).default,g=n(2),v=n(119).default,m=function(){function e(t){(0,i.default)(this,e),this.body=t,this.physicsBody={physicsNodeIndices:[],physicsEdgeIndices:[],forces:{},velocities:{}},this.physicsEnabled=!0,this.simulationInterval=1e3/60,this.requiresTimeout=!0,this.previousStates={},this.referenceState={},this.freezeCache={},this.renderTimer=void 0,this.adaptiveTimestep=!1,this.adaptiveTimestepEnabled=!1,this.adaptiveCounter=0,this.adaptiveInterval=3,this.stabilized=!1,this.startedStabilization=!1,this.stabilizationIterations=0,this.ready=!1,this.options={},this.defaultOptions={enabled:!0,barnesHut:{theta:.5,gravitationalConstant:-2e3,centralGravity:.3,springLength:95,springConstant:.04,damping:.09,avoidOverlap:0},forceAtlas2Based:{theta:.5,gravitationalConstant:-50,centralGravity:.01,springConstant:.08,springLength:100,damping:.4,avoidOverlap:0},repulsion:{centralGravity:.2,springLength:200,springConstant:.05,nodeDistance:100,damping:.09,avoidOverlap:0},hierarchicalRepulsion:{centralGravity:0,springLength:100,springConstant:.01,nodeDistance:120,damping:.09},maxVelocity:50,minVelocity:.75,solver:"barnesHut",stabilization:{enabled:!0,iterations:1e3,updateInterval:50,onlyDynamicEdges:!1,fit:!0},timestep:.5,adaptiveTimestep:!0},g.extend(this.options,this.defaultOptions),this.timestep=.5,this.layoutFailed=!1,this.bindEventListeners()}return(0,o.default)(e,[{key:"bindEventListeners",value:function(){var e=this;this.body.emitter.on("initPhysics",(function(){e.initPhysics()})),this.body.emitter.on("_layoutFailed",(function(){e.layoutFailed=!0})),this.body.emitter.on("resetPhysics",(function(){e.stopSimulation(),e.ready=!1})),this.body.emitter.on("disablePhysics",(function(){e.physicsEnabled=!1,e.stopSimulation()})),this.body.emitter.on("restorePhysics",(function(){e.setOptions(e.options),!0===e.ready&&e.startSimulation()})),this.body.emitter.on("startSimulation",(function(){!0===e.ready&&e.startSimulation()})),this.body.emitter.on("stopSimulation",(function(){e.stopSimulation()})),this.body.emitter.on("destroy",(function(){e.stopSimulation(!1),e.body.emitter.off()})),this.body.emitter.on("_dataChanged",(function(){e.updatePhysicsData()}))}},{key:"setOptions",value:function(e){void 0!==e&&(!1===e?(this.options.enabled=!1,this.physicsEnabled=!1,this.stopSimulation()):!0===e?(this.options.enabled=!0,this.physicsEnabled=!0,this.startSimulation()):(this.physicsEnabled=!0,g.selectiveNotDeepExtend(["stabilization"],this.options,e),g.mergeOptions(this.options,e,"stabilization"),void 0===e.enabled&&(this.options.enabled=!0),!1===this.options.enabled&&(this.physicsEnabled=!1,this.stopSimulation()),this.timestep=this.options.timestep)),this.init()}},{key:"init",value:function(){var e;"forceAtlas2Based"===this.options.solver?(e=this.options.forceAtlas2Based,this.nodesSolver=new f(this.body,this.physicsBody,e),this.edgesSolver=new c(this.body,this.physicsBody,e),this.gravitySolver=new p(this.body,this.physicsBody,e)):"repulsion"===this.options.solver?(e=this.options.repulsion,this.nodesSolver=new l(this.body,this.physicsBody,e),this.edgesSolver=new c(this.body,this.physicsBody,e),this.gravitySolver=new h(this.body,this.physicsBody,e)):"hierarchicalRepulsion"===this.options.solver?(e=this.options.hierarchicalRepulsion,this.nodesSolver=new u(this.body,this.physicsBody,e),this.edgesSolver=new d(this.body,this.physicsBody,e),this.gravitySolver=new h(this.body,this.physicsBody,e)):(e=this.options.barnesHut,this.nodesSolver=new s(this.body,this.physicsBody,e),this.edgesSolver=new c(this.body,this.physicsBody,e),this.gravitySolver=new h(this.body,this.physicsBody,e)),this.modelOptions=e}},{key:"initPhysics",value:function(){!0===this.physicsEnabled&&!0===this.options.enabled?!0===this.options.stabilization.enabled?this.stabilize():(this.stabilized=!1,this.ready=!0,this.body.emitter.emit("fit",{},this.layoutFailed),this.startSimulation()):(this.ready=!0,this.body.emitter.emit("fit"))}},{key:"startSimulation",value:function(){!0===this.physicsEnabled&&!0===this.options.enabled?(this.stabilized=!1,this.adaptiveTimestep=!1,this.body.emitter.emit("_resizeNodes"),void 0===this.viewFunction&&(this.viewFunction=this.simulationStep.bind(this),this.body.emitter.on("initRedraw",this.viewFunction),this.body.emitter.emit("_startRendering"))):this.body.emitter.emit("_redraw")}},{key:"stopSimulation",value:function(){var e=!(arguments.length>0&&void 0!==arguments[0])||arguments[0];this.stabilized=!0,!0===e&&this._emitStabilized(),void 0!==this.viewFunction&&(this.body.emitter.off("initRedraw",this.viewFunction),this.viewFunction=void 0,!0===e&&this.body.emitter.emit("_stopRendering"))}},{key:"simulationStep",value:function(){var e=Date.now();this.physicsTick(),(Date.now()-e<.4*this.simulationInterval||!0===this.runDoubleSpeed)&&!1===this.stabilized&&(this.physicsTick(),this.runDoubleSpeed=!0),!0===this.stabilized&&this.stopSimulation()}},{key:"_emitStabilized",value:function(){var e=this,t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:this.stabilizationIterations;(this.stabilizationIterations>1||!0===this.startedStabilization)&&setTimeout((function(){e.body.emitter.emit("stabilized",{iterations:t}),e.startedStabilization=!1,e.stabilizationIterations=0}),0)}},{key:"physicsStep",value:function(){this.gravitySolver.solve(),this.nodesSolver.solve(),this.edgesSolver.solve(),this.moveNodes()}},{key:"adjustTimeStep",value:function(){!0===this._evaluateStepQuality()?this.timestep=1.2*this.timestep:this.timestep/1.2<this.options.timestep?this.timestep=this.options.timestep:(this.adaptiveCounter=-1,this.timestep=Math.max(this.options.timestep,this.timestep/1.2))}},{key:"physicsTick",value:function(){this._startStabilizing(),!0!==this.stabilized&&(!0===this.adaptiveTimestep&&!0===this.adaptiveTimestepEnabled?(this.adaptiveCounter%this.adaptiveInterval==0?(this.timestep=2*this.timestep,this.physicsStep(),this.revert(),this.timestep=.5*this.timestep,this.physicsStep(),this.physicsStep(),this.adjustTimeStep()):this.physicsStep(),this.adaptiveCounter+=1):(this.timestep=this.options.timestep,this.physicsStep()),!0===this.stabilized&&this.revert(),this.stabilizationIterations++)}},{key:"updatePhysicsData",value:function(){this.physicsBody.forces={},this.physicsBody.physicsNodeIndices=[],this.physicsBody.physicsEdgeIndices=[];var e=this.body.nodes,t=this.body.edges;for(var n in e)e.hasOwnProperty(n)&&!0===e[n].options.physics&&this.physicsBody.physicsNodeIndices.push(e[n].id);for(var r in t)t.hasOwnProperty(r)&&!0===t[r].options.physics&&this.physicsBody.physicsEdgeIndices.push(t[r].id);for(var i=0;i<this.physicsBody.physicsNodeIndices.length;i++){var o=this.physicsBody.physicsNodeIndices[i];this.physicsBody.forces[o]={x:0,y:0},void 0===this.physicsBody.velocities[o]&&(this.physicsBody.velocities[o]={x:0,y:0})}for(var a in this.physicsBody.velocities)void 0===e[a]&&delete this.physicsBody.velocities[a]}},{key:"revert",value:function(){var e=(0,r.default)(this.previousStates),t=this.body.nodes,n=this.physicsBody.velocities;this.referenceState={};for(var i=0;i<e.length;i++){var o=e[i];void 0!==t[o]?!0===t[o].options.physics&&(this.referenceState[o]={positions:{x:t[o].x,y:t[o].y}},n[o].x=this.previousStates[o].vx,n[o].y=this.previousStates[o].vy,t[o].x=this.previousStates[o].x,t[o].y=this.previousStates[o].y):delete this.previousStates[o]}}},{key:"_evaluateStepQuality",value:function(){var e=void 0,t=void 0,n=this.body.nodes,r=this.referenceState;for(var i in this.referenceState)if(this.referenceState.hasOwnProperty(i)&&void 0!==n[i]&&(e=n[i].x-r[i].positions.x,t=n[i].y-r[i].positions.y,Math.sqrt(Math.pow(e,2)+Math.pow(t,2))>.3))return!1;return!0}},{key:"moveNodes",value:function(){for(var e=this.physicsBody.physicsNodeIndices,t=0,n=0,r=0;r<e.length;r++){var i=e[r],o=this._performStep(i);t=Math.max(t,o),n+=o}this.adaptiveTimestepEnabled=n/e.length<5,this.stabilized=t<this.options.minVelocity}},{key:"calculateComponentVelocity",value:function(e,t,n){e+=(t-this.modelOptions.damping*e)/n*this.timestep;var r=this.options.maxVelocity||1e9;return Math.abs(e)>r&&(e=e>0?r:-r),e}},{key:"_performStep",value:function(e){var t=this.body.nodes[e],n=this.physicsBody.forces[e],r=this.physicsBody.velocities[e];return this.previousStates[e]={x:t.x,y:t.y,vx:r.x,vy:r.y},!1===t.options.fixed.x?(r.x=this.calculateComponentVelocity(r.x,n.x,t.options.mass),t.x+=r.x*this.timestep):(n.x=0,r.x=0),!1===t.options.fixed.y?(r.y=this.calculateComponentVelocity(r.y,n.y,t.options.mass),t.y+=r.y*this.timestep):(n.y=0,r.y=0),Math.sqrt(Math.pow(r.x,2)+Math.pow(r.y,2))}},{key:"_freezeNodes",value:function(){var e=this.body.nodes;for(var t in e)if(e.hasOwnProperty(t)&&e[t].x&&e[t].y){var n=e[t].options.fixed;this.freezeCache[t]={x:n.x,y:n.y},n.x=!0,n.y=!0}}},{key:"_restoreFrozenNodes",value:function(){var e=this.body.nodes;for(var t in e)e.hasOwnProperty(t)&&void 0!==this.freezeCache[t]&&(e[t].options.fixed.x=this.freezeCache[t].x,e[t].options.fixed.y=this.freezeCache[t].y);this.freezeCache={}}},{key:"stabilize",value:function(){var e=this,t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:this.options.stabilization.iterations;"number"!=typeof t&&(t=this.options.stabilization.iterations,console.log("The stabilize method needs a numeric amount of iterations. Switching to default: ",t)),0!==this.physicsBody.physicsNodeIndices.length?(this.adaptiveTimestep=this.options.adaptiveTimestep,this.body.emitter.emit("_resizeNodes"),this.stopSimulation(),this.stabilized=!1,this.body.emitter.emit("_blockRedraw"),this.targetIterations=t,!0===this.options.stabilization.onlyDynamicEdges&&this._freezeNodes(),this.stabilizationIterations=0,setTimeout((function(){return e._stabilizationBatch()}),0)):this.ready=!0}},{key:"_startStabilizing",value:function(){return!0!==this.startedStabilization&&(this.body.emitter.emit("startStabilizing"),this.startedStabilization=!0,!0)}},{key:"_stabilizationBatch",value:function(){var e=this,t=function(){return!1===e.stabilized&&e.stabilizationIterations<e.targetIterations},n=function(){e.body.emitter.emit("stabilizationProgress",{iterations:e.stabilizationIterations,total:e.targetIterations})};this._startStabilizing()&&n();for(var r=0;t()&&r<this.options.stabilization.updateInterval;)this.physicsTick(),r++;n(),t()?setTimeout(this._stabilizationBatch.bind(this),0):this._finalizeStabilization()}},{key:"_finalizeStabilization",value:function(){this.body.emitter.emit("_allowRedraw"),!0===this.options.stabilization.fit&&this.body.emitter.emit("fit"),!0===this.options.stabilization.onlyDynamicEdges&&this._restoreFrozenNodes(),this.body.emitter.emit("stabilizationIterationsDone"),this.body.emitter.emit("_requestRedraw"),!0===this.stabilized?this._emitStabilized():this.startSimulation(),this.ready=!0}},{key:"_drawForces",value:function(e){for(var t=0;t<this.physicsBody.physicsNodeIndices.length;t++){var n=this.physicsBody.physicsNodeIndices[t],r=this.body.nodes[n],i=this.physicsBody.forces[n],o=Math.sqrt(Math.pow(i.x,2)+Math.pow(i.x,2)),a=Math.min(Math.max(5,o),15),s=3*a,l=g.HSVToHex((180-180*Math.min(1,Math.max(0,.03*o)))/360,1,1),u={x:r.x+20*i.x,y:r.y+20*i.y};e.lineWidth=a,e.strokeStyle=l,e.beginPath(),e.moveTo(r.x,r.y),e.lineTo(u.x,u.y),e.stroke();var c=Math.atan2(i.y,i.x);e.fillStyle=l,v.draw(e,{type:"arrow",point:u,angle:c,length:s}),e.fill()}}}]),e}();t.default=m},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=o(n(0)),i=o(n(1));function o(e){return e&&e.__esModule?e:{default:e}}var a=function(){function e(t,n,i){(0,r.default)(this,e),this.body=t,this.physicsBody=n,this.setOptions(i)}return(0,i.default)(e,[{key:"setOptions",value:function(e){this.options=e}},{key:"solve",value:function(){for(var e,t,n,r,i,o,a,s,l=this.body.nodes,u=this.physicsBody.physicsNodeIndices,c=this.physicsBody.forces,d=this.options.nodeDistance,h=-2/3/d,f=0;f<u.length-1;f++){a=l[u[f]];for(var p=f+1;p<u.length;p++)e=(s=l[u[p]]).x-a.x,t=s.y-a.y,0===(n=Math.sqrt(e*e+t*t))&&(e=n=.1*Math.random()),n<2*d&&(o=n<.5*d?1:h*n+4/3,r=e*(o/=n),i=t*o,c[a.id].x-=r,c[a.id].y-=i,c[s.id].x+=r,c[s.id].y+=i)}}}]),e}();t.default=a},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=o(n(0)),i=o(n(1));function o(e){return e&&e.__esModule?e:{default:e}}var a=function(){function e(t,n,i){(0,r.default)(this,e),this.body=t,this.physicsBody=n,this.setOptions(i)}return(0,i.default)(e,[{key:"setOptions",value:function(e){this.options=e}},{key:"solve",value:function(){var e,t,n,r,i,o,a,s,l,u,c=this.body.nodes,d=this.physicsBody.physicsNodeIndices,h=this.physicsBody.forces,f=this.options.nodeDistance;for(l=0;l<d.length-1;l++)for(a=c[d[l]],u=l+1;u<d.length;u++)s=c[d[u]],a.level===s.level&&(e=s.x-a.x,t=s.y-a.y,o=(n=Math.sqrt(e*e+t*t))<f?-Math.pow(.05*n,2)+Math.pow(.05*f,2):0,0===n?n=.01:o/=n,r=e*o,i=t*o,h[a.id].x-=r,h[a.id].y-=i,h[s.id].x+=r,h[s.id].y+=i)}}]),e}();t.default=a},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=o(n(0)),i=o(n(1));function o(e){return e&&e.__esModule?e:{default:e}}var a=function(){function e(t,n,i){(0,r.default)(this,e),this.body=t,this.physicsBody=n,this.setOptions(i)}return(0,i.default)(e,[{key:"setOptions",value:function(e){this.options=e}},{key:"solve",value:function(){for(var e=void 0,t=void 0,n=this.physicsBody.physicsEdgeIndices,r=this.body.edges,i=void 0,o=void 0,a=void 0,s=0;s<n.length;s++)!0===(t=r[n[s]]).connected&&t.toId!==t.fromId&&void 0!==this.body.nodes[t.toId]&&void 0!==this.body.nodes[t.fromId]&&(void 0!==t.edgeType.via?(e=void 0===t.options.length?this.options.springLength:t.options.length,i=t.to,o=t.edgeType.via,a=t.from,this._calculateSpringForce(i,o,.5*e),this._calculateSpringForce(o,a,.5*e)):(e=void 0===t.options.length?1.5*this.options.springLength:t.options.length,this._calculateSpringForce(t.from,t.to,e)))}},{key:"_calculateSpringForce",value:function(e,t,n){var r=e.x-t.x,i=e.y-t.y,o=Math.max(Math.sqrt(r*r+i*i),.01),a=this.options.springConstant*(n-o)/o,s=r*a,l=i*a;void 0!==this.physicsBody.forces[e.id]&&(this.physicsBody.forces[e.id].x+=s,this.physicsBody.forces[e.id].y+=l),void 0!==this.physicsBody.forces[t.id]&&(this.physicsBody.forces[t.id].x-=s,this.physicsBody.forces[t.id].y-=l)}}]),e}();t.default=a},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=o(n(0)),i=o(n(1));function o(e){return e&&e.__esModule?e:{default:e}}var a=function(){function e(t,n,i){(0,r.default)(this,e),this.body=t,this.physicsBody=n,this.setOptions(i)}return(0,i.default)(e,[{key:"setOptions",value:function(e){this.options=e}},{key:"solve",value:function(){for(var e,t,n,r,i,o,a,s,l,u,c=this.body.edges,d=this.physicsBody.physicsEdgeIndices,h=this.physicsBody.physicsNodeIndices,f=this.physicsBody.forces,p=0;p<h.length;p++){var g=h[p];f[g].springFx=0,f[g].springFy=0}for(var v=0;v<d.length;v++)!0===(t=c[d[v]]).connected&&(e=void 0===t.options.length?this.options.springLength:t.options.length,n=t.from.x-t.to.x,r=t.from.y-t.to.y,s=0===(s=Math.sqrt(n*n+r*r))?.01:s,i=n*(a=this.options.springConstant*(e-s)/s),o=r*a,t.to.level!=t.from.level?(void 0!==f[t.toId]&&(f[t.toId].springFx-=i,f[t.toId].springFy-=o),void 0!==f[t.fromId]&&(f[t.fromId].springFx+=i,f[t.fromId].springFy+=o)):(void 0!==f[t.toId]&&(f[t.toId].x-=.5*i,f[t.toId].y-=.5*o),void 0!==f[t.fromId]&&(f[t.fromId].x+=.5*i,f[t.fromId].y+=.5*o)));a=1;for(var m=0;m<h.length;m++){var y=h[m];l=Math.min(a,Math.max(-a,f[y].springFx)),u=Math.min(a,Math.max(-a,f[y].springFy)),f[y].x+=l,f[y].y+=u}for(var b=0,C=0,A=0;A<h.length;A++){var _=h[A];b+=f[_].x,C+=f[_].y}for(var I=b/h.length,x=C/h.length,w=0;w<h.length;w++){var M=h[w];f[M].x-=I,f[M].y-=x}}}]),e}();t.default=a},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=l(n(3)),i=l(n(0)),o=l(n(1)),a=l(n(4)),s=l(n(5));function l(e){return e&&e.__esModule?e:{default:e}}var u=function(e){function t(e,n,o){return(0,i.default)(this,t),(0,a.default)(this,(t.__proto__||(0,r.default)(t)).call(this,e,n,o))}return(0,s.default)(t,e),(0,o.default)(t,[{key:"_calculateForces",value:function(e,t,n,r,i){0===e&&(t=e=.1*Math.random()),this.overlapAvoidanceFactor<1&&r.shape.radius&&(e=Math.max(.1+this.overlapAvoidanceFactor*r.shape.radius,e-r.shape.radius));var o=r.edges.length+1,a=this.options.gravitationalConstant*i.mass*r.options.mass*o/Math.pow(e,2),s=t*a,l=n*a;this.physicsBody.forces[r.id].x+=s,this.physicsBody.forces[r.id].y+=l}}]),t}(l(n(120)).default);t.default=u},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=l(n(3)),i=l(n(0)),o=l(n(1)),a=l(n(4)),s=l(n(5));function l(e){return e&&e.__esModule?e:{default:e}}var u=function(e){function t(e,n,o){return(0,i.default)(this,t),(0,a.default)(this,(t.__proto__||(0,r.default)(t)).call(this,e,n,o))}return(0,s.default)(t,e),(0,o.default)(t,[{key:"_calculateForces",value:function(e,t,n,r,i){if(e>0){var o=i.edges.length+1,a=this.options.centralGravity*o*i.options.mass;r[i.id].x=t*a,r[i.id].y=n*a}}}]),t}(l(n(121)).default);t.default=u},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=s(n(8)),i=s(n(6)),o=s(n(0)),a=s(n(1));function s(e){return e&&e.__esModule?e:{default:e}}var l=n(2),u=n(76).default,c=n(228).default,d=n(74).default,h=n(47).default,f=function(){function e(t){var n=this;(0,o.default)(this,e),this.body=t,this.clusteredNodes={},this.clusteredEdges={},this.options={},this.defaultOptions={},l.extend(this.options,this.defaultOptions),this.body.emitter.on("_resetData",(function(){n.clusteredNodes={},n.clusteredEdges={}}))}return(0,a.default)(e,[{key:"clusterByHubsize",value:function(e,t){void 0===e?e=this._getHubSize():"object"===(void 0===e?"undefined":(0,i.default)(e))&&(t=this._checkOptions(e),e=this._getHubSize());for(var n=[],r=0;r<this.body.nodeIndices.length;r++){var o=this.body.nodes[this.body.nodeIndices[r]];o.edges.length>=e&&n.push(o.id)}for(var a=0;a<n.length;a++)this.clusterByConnection(n[a],t,!0);this.body.emitter.emit("_dataChanged")}},{key:"cluster",value:function(){var e=this,t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},n=!(arguments.length>1&&void 0!==arguments[1])||arguments[1];if(void 0===t.joinCondition)throw new Error("Cannot call clusterByNodeData without a joinCondition function in the options.");t=this._checkOptions(t);var r={},i={};l.forEach(this.body.nodes,(function(n,o){var a=u.cloneOptions(n);!0===t.joinCondition(a)&&(r[o]=n,l.forEach(n.edges,(function(t){void 0===e.clusteredEdges[t.id]&&(i[t.id]=t)})))})),this._cluster(r,i,t,n)}},{key:"clusterByEdgeCount",value:function(e,t){var n=this,i=!(arguments.length>2&&void 0!==arguments[2])||arguments[2];t=this._checkOptions(t);for(var o=[],a={},s=void 0,l=void 0,c=void 0,d=function(i){var d={},h={},g=n.body.nodeIndices[i],v=n.body.nodes[g];if(void 0===a[g]){c=0,l=[];for(var m=0;m<v.edges.length;m++)s=v.edges[m],void 0===n.clusteredEdges[s.id]&&(s.toId!==s.fromId&&c++,l.push(s));if(c===e){f=function(e){if(void 0===t.joinCondition||null===t.joinCondition)return!0;var n=u.cloneOptions(e);return t.joinCondition(n)};for(var y=!0,b=0;b<l.length;b++){s=l[b];var C=n._getConnectedId(s,g);if(!f(v)){y=!1;break}h[s.id]=s,d[g]=v,d[C]=n.body.nodes[C],a[g]=!0}if((0,r.default)(d).length>0&&(0,r.default)(h).length>0&&!0===y)if(void 0!==(p=function(){for(var e=0;e<o.length;++e)for(var t in d)if(void 0!==o[e].nodes[t])return o[e]}())){for(var A in d)void 0===p.nodes[A]&&(p.nodes[A]=d[A]);for(var _ in h)void 0===p.edges[_]&&(p.edges[_]=h[_])}else o.push({nodes:d,edges:h})}}},h=0;h<this.body.nodeIndices.length;h++){var f,p;d(h)}for(h=0;h<o.length;h++)this._cluster(o[h].nodes,o[h].edges,t,!1);!0===i&&this.body.emitter.emit("_dataChanged")}},{key:"clusterOutliers",value:function(e){var t=!(arguments.length>1&&void 0!==arguments[1])||arguments[1];this.clusterByEdgeCount(1,e,t)}},{key:"clusterBridges",value:function(e){var t=!(arguments.length>1&&void 0!==arguments[1])||arguments[1];this.clusterByEdgeCount(2,e,t)}},{key:"clusterByConnection",value:function(e,t){var n=!(arguments.length>2&&void 0!==arguments[2])||arguments[2];if(void 0===e)throw new Error("No nodeId supplied to clusterByConnection!");if(void 0===this.body.nodes[e])throw new Error("The nodeId given to clusterByConnection does not exist!");var i=this.body.nodes[e];void 0===(t=this._checkOptions(t,i)).clusterNodeProperties.x&&(t.clusterNodeProperties.x=i.x),void 0===t.clusterNodeProperties.y&&(t.clusterNodeProperties.y=i.y),void 0===t.clusterNodeProperties.fixed&&(t.clusterNodeProperties.fixed={},t.clusterNodeProperties.fixed.x=i.options.fixed.x,t.clusterNodeProperties.fixed.y=i.options.fixed.y);var o={},a={},s=i.id,l=u.cloneOptions(i);o[s]=i;for(var c=0;c<i.edges.length;c++){var d=i.edges[c];if(void 0===this.clusteredEdges[d.id]){var h=this._getConnectedId(d,s);if(void 0===this.clusteredNodes[h])if(h!==s)if(void 0===t.joinCondition)a[d.id]=d,o[h]=this.body.nodes[h];else{var f=u.cloneOptions(this.body.nodes[h]);!0===t.joinCondition(l,f)&&(a[d.id]=d,o[h]=this.body.nodes[h])}else a[d.id]=d}}var p=(0,r.default)(o).map((function(e){return o[e].id}));for(g in o)if(o.hasOwnProperty(g))for(var g=o[g],v=0;v<g.edges.length;v++){var m=g.edges[v];p.indexOf(this._getConnectedId(m,g.id))>-1&&(a[m.id]=m)}this._cluster(o,a,t,n)}},{key:"_createClusterEdges",value:function(e,t,n,i){for(var o=void 0,a=void 0,s=void 0,l=void 0,u=void 0,c=void 0,d=(0,r.default)(e),h=[],f=0;f<d.length;f++){s=e[a=d[f]];for(var p=0;p<s.edges.length;p++)o=s.edges[p],void 0===this.clusteredEdges[o.id]&&(o.toId==o.fromId?t[o.id]=o:o.toId==a?(l=n.id,c=u=o.fromId):(l=o.toId,u=n.id,c=l),void 0===e[c]&&h.push({edge:o,fromId:u,toId:l}))}for(var g=[],v=function(e){for(var t=0;t<g.length;t++){var n=g[t],r=e.fromId===n.fromId&&e.toId===n.toId,i=e.fromId===n.toId&&e.toId===n.fromId;if(r||i)return n}return null},m=0;m<h.length;m++){var y=h[m],b=y.edge,C=v(y);null===C?(C=this._createClusteredEdge(y.fromId,y.toId,b,i),g.push(C)):C.clusteringEdgeReplacingIds.push(b.id),this.body.edges[b.id].edgeReplacedById=C.id,this._backupEdgeOptions(b),b.setOptions({physics:!1})}}},{key:"_checkOptions",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};return void 0===e.clusterEdgeProperties&&(e.clusterEdgeProperties={}),void 0===e.clusterNodeProperties&&(e.clusterNodeProperties={}),e}},{key:"_cluster",value:function(e,t,n){var i=!(arguments.length>3&&void 0!==arguments[3])||arguments[3],o=[];for(var a in e)e.hasOwnProperty(a)&&void 0!==this.clusteredNodes[a]&&o.push(a);for(var s=0;s<o.length;++s)delete e[o[s]];if(0!=(0,r.default)(e).length&&(1!=(0,r.default)(e).length||1==n.clusterNodeProperties.allowSingleNodeCluster)){var d=l.deepExtend({},n.clusterNodeProperties);if(void 0!==n.processProperties){var h=[];for(var f in e)if(e.hasOwnProperty(f)){var p=u.cloneOptions(e[f]);h.push(p)}var g=[];for(var v in t)if(t.hasOwnProperty(v)&&"clusterEdge:"!==v.substr(0,12)){var m=u.cloneOptions(t[v],"edge");g.push(m)}if(!(d=n.processProperties(d,h,g)))throw new Error("The processProperties function does not return properties!")}void 0===d.id&&(d.id="cluster:"+l.randomUUID());var y=d.id;void 0===d.label&&(d.label="cluster");var b=void 0;void 0===d.x&&(b=this._getClusterPosition(e),d.x=b.x),void 0===d.y&&(void 0===b&&(b=this._getClusterPosition(e)),d.y=b.y),d.id=y;var C=this.body.functions.createNode(d,c);C.containedNodes=e,C.containedEdges=t,C.clusterEdgeProperties=n.clusterEdgeProperties,this.body.nodes[d.id]=C,this._clusterEdges(e,t,d,n.clusterEdgeProperties),d.id=void 0,!0===i&&this.body.emitter.emit("_dataChanged")}}},{key:"_backupEdgeOptions",value:function(e){void 0===this.clusteredEdges[e.id]&&(this.clusteredEdges[e.id]={physics:e.options.physics})}},{key:"_restoreEdge",value:function(e){var t=this.clusteredEdges[e.id];void 0!==t&&(e.setOptions({physics:t.physics}),delete this.clusteredEdges[e.id])}},{key:"isCluster",value:function(e){return void 0!==this.body.nodes[e]?!0===this.body.nodes[e].isCluster:(console.log("Node does not exist."),!1)}},{key:"_getClusterPosition",value:function(e){for(var t=(0,r.default)(e),n=e[t[0]].x,i=e[t[0]].x,o=e[t[0]].y,a=e[t[0]].y,s=void 0,l=1;l<t.length;l++)n=(s=e[t[l]]).x<n?s.x:n,i=s.x>i?s.x:i,o=s.y<o?s.y:o,a=s.y>a?s.y:a;return{x:.5*(n+i),y:.5*(o+a)}}},{key:"openCluster",value:function(e,t){var n=!(arguments.length>2&&void 0!==arguments[2])||arguments[2];if(void 0===e)throw new Error("No clusterNodeId supplied to openCluster.");var r=this.body.nodes[e];if(void 0===r)throw new Error("The clusterNodeId supplied to openCluster does not exist.");if(!0!==r.isCluster||void 0===r.containedNodes||void 0===r.containedEdges)throw new Error("The node:"+e+" is not a valid cluster.");var i=this.findNode(e),o=i.indexOf(e)-1;if(o>=0){var a=i[o],s=this.body.nodes[a];return s._openChildCluster(e),delete this.body.nodes[e],void(!0===n&&this.body.emitter.emit("_dataChanged"))}var u=r.containedNodes,c=r.containedEdges;if(void 0!==t&&void 0!==t.releaseFunction&&"function"==typeof t.releaseFunction){var d={},h={x:r.x,y:r.y};for(var f in u)if(u.hasOwnProperty(f)){var p=this.body.nodes[f];d[f]={x:p.x,y:p.y}}var g=t.releaseFunction(h,d);for(var v in u)if(u.hasOwnProperty(v)){var m=this.body.nodes[v];void 0!==g[v]&&(m.x=void 0===g[v].x?r.x:g[v].x,m.y=void 0===g[v].y?r.y:g[v].y)}}else l.forEach(u,(function(e){!1===e.options.fixed.x&&(e.x=r.x),!1===e.options.fixed.y&&(e.y=r.y)}));for(var y in u)if(u.hasOwnProperty(y)){var b=this.body.nodes[y];b.vx=r.vx,b.vy=r.vy,b.setOptions({physics:!0}),delete this.clusteredNodes[y]}for(var C=[],A=0;A<r.edges.length;A++)C.push(r.edges[A]);for(var _=0;_<C.length;_++){for(var I=C[_],x=this._getConnectedId(I,e),w=this.clusteredNodes[x],M=0;M<I.clusteringEdgeReplacingIds.length;M++){var S=I.clusteringEdgeReplacingIds[M],T=this.body.edges[S];if(void 0!==T)if(void 0!==w){var E=this.body.nodes[w.clusterId];E.containedEdges[T.id]=T,delete c[T.id];var O=T.fromId,k=T.toId;T.toId==x?k=w.clusterId:O=w.clusterId,this._createClusteredEdge(O,k,T,E.clusterEdgeProperties,{hidden:!1,physics:!0})}else this._restoreEdge(T)}I.remove()}for(var L in c)c.hasOwnProperty(L)&&this._restoreEdge(c[L]);delete this.body.nodes[e],!0===n&&this.body.emitter.emit("_dataChanged")}},{key:"getNodesInCluster",value:function(e){var t=[];if(!0===this.isCluster(e)){var n=this.body.nodes[e].containedNodes;for(var r in n)n.hasOwnProperty(r)&&t.push(this.body.nodes[r].id)}return t}},{key:"findNode",value:function(e){for(var t=[],n=0,r=void 0;void 0!==this.clusteredNodes[e]&&n<100;){if(void 0===(r=this.body.nodes[e]))return[];t.push(r.id),e=this.clusteredNodes[e].clusterId,n++}return void 0===(r=this.body.nodes[e])?[]:(t.push(r.id),t.reverse(),t)}},{key:"updateClusteredNode",value:function(e,t){if(void 0===e)throw new Error("No clusteredNodeId supplied to updateClusteredNode.");if(void 0===t)throw new Error("No newOptions supplied to updateClusteredNode.");if(void 0===this.body.nodes[e])throw new Error("The clusteredNodeId supplied to updateClusteredNode does not exist.");this.body.nodes[e].setOptions(t),this.body.emitter.emit("_dataChanged")}},{key:"updateEdge",value:function(e,t){if(void 0===e)throw new Error("No startEdgeId supplied to updateEdge.");if(void 0===t)throw new Error("No newOptions supplied to updateEdge.");if(void 0===this.body.edges[e])throw new Error("The startEdgeId supplied to updateEdge does not exist.");for(var n=this.getClusteredEdges(e),r=0;r<n.length;r++)this.body.edges[n[r]].setOptions(t);this.body.emitter.emit("_dataChanged")}},{key:"getClusteredEdges",value:function(e){for(var t=[],n=0;void 0!==e&&void 0!==this.body.edges[e]&&n<100;)t.push(this.body.edges[e].id),e=this.body.edges[e].edgeReplacedById,n++;return t.reverse(),t}},{key:"getBaseEdge",value:function(e){return this.getBaseEdges(e)[0]}},{key:"getBaseEdges",value:function(e){for(var t=[e],n=[],r=[],i=0;t.length>0&&i<100;){var o=t.pop();if(void 0!==o){var a=this.body.edges[o];if(void 0!==a){i++;var s=a.clusteringEdgeReplacingIds;if(void 0===s)r.push(o);else for(var l=0;l<s.length;++l){var u=s[l];-1===t.indexOf(s)&&-1===n.indexOf(s)&&t.push(u)}n.push(o)}}}return r}},{key:"_getConnectedId",value:function(e,t){return e.toId!=t?e.toId:(e.fromId,e.fromId)}},{key:"_getHubSize",value:function(){for(var e=0,t=0,n=0,r=0,i=0;i<this.body.nodeIndices.length;i++){var o=this.body.nodes[this.body.nodeIndices[i]];o.edges.length>r&&(r=o.edges.length),e+=o.edges.length,t+=Math.pow(o.edges.length,2),n+=1}e/=n;var a=(t/=n)-Math.pow(e,2),s=Math.sqrt(a),l=Math.floor(e+2*s);return l>r&&(l=r),l}},{key:"_createClusteredEdge",value:function(e,t,n,r,i){var o=u.cloneOptions(n,"edge");l.deepExtend(o,r),o.from=e,o.to=t,o.id="clusterEdge:"+l.randomUUID(),void 0!==i&&l.deepExtend(o,i);var a=this.body.functions.createEdge(o);return a.clusteringEdgeReplacingIds=[n.id],a.connect(),this.body.edges[a.id]=a,a}},{key:"_clusterEdges",value:function(e,t,n,r){if(t instanceof d){var i=t,o={};o[i.id]=i,t=o}if(e instanceof h){var a=e,s={};s[a.id]=a,e=s}if(null==n)throw new Error("_clusterEdges: parameter clusterNode required");for(var l in void 0===r&&(r=n.clusterEdgeProperties),this._createClusterEdges(e,t,n,r),t)if(t.hasOwnProperty(l)&&void 0!==this.body.edges[l]){var u=this.body.edges[l];this._backupEdgeOptions(u),u.setOptions({physics:!1})}for(var c in e)e.hasOwnProperty(c)&&(this.clusteredNodes[c]={clusterId:n.id,node:this.body.nodes[c]},this.body.nodes[c].setOptions({physics:!1}))}},{key:"_getClusterNodeForNode",value:function(e){if(void 0!==e){var t=this.clusteredNodes[e];if(void 0!==t){var n=t.clusterId;if(void 0!==n)return this.body.nodes[n]}}}},{key:"_filter",value:function(e,t){var n=[];return l.forEach(e,(function(e){t(e)&&n.push(e)})),n}},{key:"_updateState",value:function(){var e=this,t=void 0,n=[],i=[],o=function(t){l.forEach(e.body.nodes,(function(e){!0===e.isCluster&&t(e)}))};for(t in this.clusteredNodes)this.clusteredNodes.hasOwnProperty(t)&&void 0===this.body.nodes[t]&&n.push(t);o((function(e){for(var t=0;t<n.length;t++)delete e.containedNodes[n[t]]}));for(var a=0;a<n.length;a++)delete this.clusteredNodes[n[a]];l.forEach(this.clusteredEdges,(function(t){var n=e.body.edges[t];void 0!==n&&n.endPointsValid()||i.push(t)})),o((function(e){l.forEach(e.containedEdges,(function(e,t){e.endPointsValid()||-1!==i.indexOf(t)||i.push(t)}))})),l.forEach(this.body.edges,(function(t,n){var r=!0,o=t.clusteringEdgeReplacingIds;if(void 0!==o){var a=0;l.forEach(o,(function(t){var n=e.body.edges[t];void 0!==n&&n.endPointsValid()&&(a+=1)})),r=a>0}t.endPointsValid()&&r||i.push(n)})),o((function(t){l.forEach(i,(function(n){delete t.containedEdges[n],l.forEach(t.edges,(function(r,o){r.id!==n?r.clusteringEdgeReplacingIds=e._filter(r.clusteringEdgeReplacingIds,(function(e){return-1===i.indexOf(e)})):t.edges[o]=null})),t.edges=e._filter(t.edges,(function(e){return null!==e}))}))})),l.forEach(i,(function(t){delete e.clusteredEdges[t]})),l.forEach(i,(function(t){delete e.body.edges[t]}));var s=(0,r.default)(this.body.edges);l.forEach(s,(function(t){var n=e.body.edges[t],r=e._isClusteredNode(n.fromId)||e._isClusteredNode(n.toId);if(r!==e._isClusteredEdge(n.id)){if(!r)throw new Error("remove edge from clustering not implemented!");var i=e._getClusterNodeForNode(n.fromId);void 0!==i&&e._clusterEdges(e.body.nodes[n.fromId],n,i);var o=e._getClusterNodeForNode(n.toId);void 0!==o&&e._clusterEdges(e.body.nodes[n.toId],n,o)}}));for(var u=!1,c=!0,d=function(){var t=[];o((function(e){var n=(0,r.default)(e.containedNodes).length,i=!0===e.options.allowSingleNodeCluster;(i&&n<1||!i&&n<2)&&t.push(e.id)}));for(var n=0;n<t.length;++n)e.openCluster(t[n],{},!1);c=t.length>0,u=u||c};c;)d();u&&this._updateState()}},{key:"_isClusteredNode",value:function(e){return void 0!==this.clusteredNodes[e]}},{key:"_isClusteredEdge",value:function(e){return void 0!==this.clusteredEdges[e]}}]),e}();t.default=f},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=l(n(3)),i=l(n(0)),o=l(n(1)),a=l(n(4)),s=l(n(5));function l(e){return e&&e.__esModule?e:{default:e}}var u=n(2),c=function(e){function t(e,n,o,s,l,u){(0,i.default)(this,t);var c=(0,a.default)(this,(t.__proto__||(0,r.default)(t)).call(this,e,n,o,s,l,u));return c.isCluster=!0,c.containedNodes={},c.containedEdges={},c}return(0,s.default)(t,e),(0,o.default)(t,[{key:"_openChildCluster",value:function(e){var t=this,n=this.body.nodes[e];if(void 0===this.containedNodes[e])throw new Error("node with id: "+e+" not in current cluster");if(!n.isCluster)throw new Error("node with id: "+e+" is not a cluster");delete this.containedNodes[e],u.forEach(n.edges,(function(e){delete t.containedEdges[e.id]})),u.forEach(n.containedNodes,(function(e,n){t.containedNodes[n]=e})),n.containedNodes={},u.forEach(n.containedEdges,(function(e,n){t.containedEdges[n]=e})),n.containedEdges={},u.forEach(n.edges,(function(e){u.forEach(t.edges,(function(n){var r=n.clusteringEdgeReplacingIds.indexOf(e.id);-1!==r&&(u.forEach(e.clusteringEdgeReplacingIds,(function(e){n.clusteringEdgeReplacingIds.push(e),t.body.edges[e].edgeReplacedById=n.id})),n.clusteringEdgeReplacingIds.splice(r,1))}))})),n.edges=[]}}]),t}(n(47).default);t.default=c},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=o(n(0)),i=o(n(1));function o(e){return e&&e.__esModule?e:{default:e}}var a=n(2),s=function(){function e(t,n){var i;(0,r.default)(this,e),void 0!==window&&(i=window.requestAnimationFrame||window.mozRequestAnimationFrame||window.webkitRequestAnimationFrame||window.msRequestAnimationFrame),window.requestAnimationFrame=void 0===i?function(e){e()}:i,this.body=t,this.canvas=n,this.redrawRequested=!1,this.renderTimer=void 0,this.requiresTimeout=!0,this.renderingActive=!1,this.renderRequests=0,this.allowRedraw=!0,this.dragging=!1,this.options={},this.defaultOptions={hideEdgesOnDrag:!1,hideNodesOnDrag:!1},a.extend(this.options,this.defaultOptions),this._determineBrowserMethod(),this.bindEventListeners()}return(0,i.default)(e,[{key:"bindEventListeners",value:function(){var e=this;this.body.emitter.on("dragStart",(function(){e.dragging=!0})),this.body.emitter.on("dragEnd",(function(){e.dragging=!1})),this.body.emitter.on("_resizeNodes",(function(){e._resizeNodes()})),this.body.emitter.on("_redraw",(function(){!1===e.renderingActive&&e._redraw()})),this.body.emitter.on("_blockRedraw",(function(){e.allowRedraw=!1})),this.body.emitter.on("_allowRedraw",(function(){e.allowRedraw=!0,e.redrawRequested=!1})),this.body.emitter.on("_requestRedraw",this._requestRedraw.bind(this)),this.body.emitter.on("_startRendering",(function(){e.renderRequests+=1,e.renderingActive=!0,e._startRendering()})),this.body.emitter.on("_stopRendering",(function(){e.renderRequests-=1,e.renderingActive=e.renderRequests>0,e.renderTimer=void 0})),this.body.emitter.on("destroy",(function(){e.renderRequests=0,e.allowRedraw=!1,e.renderingActive=!1,!0===e.requiresTimeout?clearTimeout(e.renderTimer):window.cancelAnimationFrame(e.renderTimer),e.body.emitter.off()}))}},{key:"setOptions",value:function(e){void 0!==e&&a.selectiveDeepExtend(["hideEdgesOnDrag","hideNodesOnDrag"],this.options,e)}},{key:"_requestNextFrame",value:function(e,t){if("undefined"!=typeof window){var n=void 0,r=window;return!0===this.requiresTimeout?n=r.setTimeout(e,t):r.requestAnimationFrame&&(n=r.requestAnimationFrame(e)),n}}},{key:"_startRendering",value:function(){!0===this.renderingActive&&void 0===this.renderTimer&&(this.renderTimer=this._requestNextFrame(this._renderStep.bind(this),this.simulationInterval))}},{key:"_renderStep",value:function(){!0===this.renderingActive&&(this.renderTimer=void 0,!0===this.requiresTimeout&&this._startRendering(),this._redraw(),!1===this.requiresTimeout&&this._startRendering())}},{key:"redraw",value:function(){this.body.emitter.emit("setSize"),this._redraw()}},{key:"_requestRedraw",value:function(){var e=this;!0!==this.redrawRequested&&!1===this.renderingActive&&!0===this.allowRedraw&&(this.redrawRequested=!0,this._requestNextFrame((function(){e._redraw(!1)}),0))}},{key:"_redraw",value:function(){var e=arguments.length>0&&void 0!==arguments[0]&&arguments[0];if(!0===this.allowRedraw){this.body.emitter.emit("initRedraw"),this.redrawRequested=!1,0!==this.canvas.frame.canvas.width&&0!==this.canvas.frame.canvas.height||this.canvas.setSize(),this.canvas.setTransform();var t=this.canvas.getContext(),n=this.canvas.frame.canvas.clientWidth,r=this.canvas.frame.canvas.clientHeight;if(t.clearRect(0,0,n,r),0===this.canvas.frame.clientWidth)return;t.save(),t.translate(this.body.view.translation.x,this.body.view.translation.y),t.scale(this.body.view.scale,this.body.view.scale),t.beginPath(),this.body.emitter.emit("beforeDrawing",t),t.closePath(),!1===e&&(!1===this.dragging||!0===this.dragging&&!1===this.options.hideEdgesOnDrag)&&this._drawEdges(t),(!1===this.dragging||!0===this.dragging&&!1===this.options.hideNodesOnDrag)&&this._drawNodes(t,e),t.beginPath(),this.body.emitter.emit("afterDrawing",t),t.closePath(),t.restore(),!0===e&&t.clearRect(0,0,n,r)}}},{key:"_resizeNodes",value:function(){this.canvas.setTransform();var e=this.canvas.getContext();e.save(),e.translate(this.body.view.translation.x,this.body.view.translation.y),e.scale(this.body.view.scale,this.body.view.scale);var t=this.body.nodes,n=void 0;for(var r in t)t.hasOwnProperty(r)&&((n=t[r]).resize(e),n.updateBoundingBox(e,n.selected));e.restore()}},{key:"_drawNodes",value:function(e){for(var t=arguments.length>1&&void 0!==arguments[1]&&arguments[1],n=this.body.nodes,r=this.body.nodeIndices,i=void 0,o=[],a=20,s=this.canvas.DOMtoCanvas({x:-a,y:-a}),l=this.canvas.DOMtoCanvas({x:this.canvas.frame.canvas.clientWidth+a,y:this.canvas.frame.canvas.clientHeight+a}),u={top:s.y,left:s.x,bottom:l.y,right:l.x},c=0;c<r.length;c++)(i=n[r[c]]).isSelected()?o.push(r[c]):!0===t||!0===i.isBoundingBoxOverlappingWith(u)?i.draw(e):i.updateBoundingBox(e,i.selected);for(var d=0;d<o.length;d++)(i=n[o[d]]).draw(e)}},{key:"_drawEdges",value:function(e){for(var t=this.body.edges,n=this.body.edgeIndices,r=void 0,i=0;i<n.length;i++)!0===(r=t[n[i]]).connected&&r.draw(e)}},{key:"_determineBrowserMethod",value:function(){if("undefined"!=typeof window){var e=navigator.userAgent.toLowerCase();this.requiresTimeout=!1,(-1!=e.indexOf("msie 9.0")||-1!=e.indexOf("safari")&&e.indexOf("chrome")<=-1)&&(this.requiresTimeout=!0)}else this.requiresTimeout=!0}}]),e}();t.default=s},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=o(n(0)),i=o(n(1));function o(e){return e&&e.__esModule?e:{default:e}}var a=n(10),s=n(37),l=n(2),u=function(){function e(t){(0,r.default)(this,e),this.body=t,this.pixelRatio=1,this.resizeTimer=void 0,this.resizeFunction=this._onResize.bind(this),this.cameraState={},this.initialized=!1,this.canvasViewCenter={},this.options={},this.defaultOptions={autoResize:!0,height:"100%",width:"100%"},l.extend(this.options,this.defaultOptions),this.bindEventListeners()}return(0,i.default)(e,[{key:"bindEventListeners",value:function(){var e=this;this.body.emitter.once("resize",(function(t){0!==t.width&&(e.body.view.translation.x=.5*t.width),0!==t.height&&(e.body.view.translation.y=.5*t.height)})),this.body.emitter.on("setSize",this.setSize.bind(this)),this.body.emitter.on("destroy",(function(){e.hammerFrame.destroy(),e.hammer.destroy(),e._cleanUp()}))}},{key:"setOptions",value:function(e){var t=this;void 0!==e&&l.selectiveDeepExtend(["width","height","autoResize"],this.options,e),!0===this.options.autoResize&&(this._cleanUp(),this.resizeTimer=setInterval((function(){!0===t.setSize()&&t.body.emitter.emit("_requestRedraw")}),1e3),this.resizeFunction=this._onResize.bind(this),l.addEventListener(window,"resize",this.resizeFunction))}},{key:"_cleanUp",value:function(){void 0!==this.resizeTimer&&clearInterval(this.resizeTimer),l.removeEventListener(window,"resize",this.resizeFunction),this.resizeFunction=void 0}},{key:"_onResize",value:function(){this.setSize(),this.body.emitter.emit("_redraw")}},{key:"_getCameraState",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:this.pixelRatio;!0===this.initialized&&(this.cameraState.previousWidth=this.frame.canvas.width/e,this.cameraState.previousHeight=this.frame.canvas.height/e,this.cameraState.scale=this.body.view.scale,this.cameraState.position=this.DOMtoCanvas({x:.5*this.frame.canvas.width/e,y:.5*this.frame.canvas.height/e}))}},{key:"_setCameraState",value:function(){if(void 0!==this.cameraState.scale&&0!==this.frame.canvas.clientWidth&&0!==this.frame.canvas.clientHeight&&0!==this.pixelRatio&&this.cameraState.previousWidth>0){var e=this.frame.canvas.width/this.pixelRatio/this.cameraState.previousWidth,t=this.frame.canvas.height/this.pixelRatio/this.cameraState.previousHeight,n=this.cameraState.scale;1!=e&&1!=t?n=.5*this.cameraState.scale*(e+t):1!=e?n=this.cameraState.scale*e:1!=t&&(n=this.cameraState.scale*t),this.body.view.scale=n;var r=this.DOMtoCanvas({x:.5*this.frame.canvas.clientWidth,y:.5*this.frame.canvas.clientHeight}),i={x:r.x-this.cameraState.position.x,y:r.y-this.cameraState.position.y};this.body.view.translation.x+=i.x*this.body.view.scale,this.body.view.translation.y+=i.y*this.body.view.scale}}},{key:"_prepareValue",value:function(e){if("number"==typeof e)return e+"px";if("string"==typeof e){if(-1!==e.indexOf("%")||-1!==e.indexOf("px"))return e;if(-1===e.indexOf("%"))return e+"px"}throw new Error("Could not use the value supplied for width or height:"+e)}},{key:"_create",value:function(){for(;this.body.container.hasChildNodes();)this.body.container.removeChild(this.body.container.firstChild);if(this.frame=document.createElement("div"),this.frame.className="vis-network",this.frame.style.position="relative",this.frame.style.overflow="hidden",this.frame.tabIndex=900,this.frame.canvas=document.createElement("canvas"),this.frame.canvas.style.position="relative",this.frame.appendChild(this.frame.canvas),this.frame.canvas.getContext)this._setPixelRatio(),this.setTransform();else{var e=document.createElement("DIV");e.style.color="red",e.style.fontWeight="bold",e.style.padding="10px",e.innerHTML="Error: your browser does not support HTML canvas",this.frame.canvas.appendChild(e)}this.body.container.appendChild(this.frame),this.body.view.scale=1,this.body.view.translation={x:.5*this.frame.canvas.clientWidth,y:.5*this.frame.canvas.clientHeight},this._bindHammer()}},{key:"_bindHammer",value:function(){var e=this;void 0!==this.hammer&&this.hammer.destroy(),this.drag={},this.pinch={},this.hammer=new a(this.frame.canvas),this.hammer.get("pinch").set({enable:!0}),this.hammer.get("pan").set({threshold:5,direction:a.DIRECTION_ALL}),s.onTouch(this.hammer,(function(t){e.body.eventListeners.onTouch(t)})),this.hammer.on("tap",(function(t){e.body.eventListeners.onTap(t)})),this.hammer.on("doubletap",(function(t){e.body.eventListeners.onDoubleTap(t)})),this.hammer.on("press",(function(t){e.body.eventListeners.onHold(t)})),this.hammer.on("panstart",(function(t){e.body.eventListeners.onDragStart(t)})),this.hammer.on("panmove",(function(t){e.body.eventListeners.onDrag(t)})),this.hammer.on("panend",(function(t){e.body.eventListeners.onDragEnd(t)})),this.hammer.on("pinch",(function(t){e.body.eventListeners.onPinch(t)})),this.frame.canvas.addEventListener("mousewheel",(function(t){e.body.eventListeners.onMouseWheel(t)})),this.frame.canvas.addEventListener("DOMMouseScroll",(function(t){e.body.eventListeners.onMouseWheel(t)})),this.frame.canvas.addEventListener("mousemove",(function(t){e.body.eventListeners.onMouseMove(t)})),this.frame.canvas.addEventListener("contextmenu",(function(t){e.body.eventListeners.onContext(t)})),this.hammerFrame=new a(this.frame),s.onRelease(this.hammerFrame,(function(t){e.body.eventListeners.onRelease(t)}))}},{key:"setSize",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:this.options.width,t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:this.options.height;e=this._prepareValue(e),t=this._prepareValue(t);var n=!1,r=this.frame.canvas.width,i=this.frame.canvas.height,o=this.pixelRatio;if(this._setPixelRatio(),e!=this.options.width||t!=this.options.height||this.frame.style.width!=e||this.frame.style.height!=t)this._getCameraState(o),this.frame.style.width=e,this.frame.style.height=t,this.frame.canvas.style.width="100%",this.frame.canvas.style.height="100%",this.frame.canvas.width=Math.round(this.frame.canvas.clientWidth*this.pixelRatio),this.frame.canvas.height=Math.round(this.frame.canvas.clientHeight*this.pixelRatio),this.options.width=e,this.options.height=t,this.canvasViewCenter={x:.5*this.frame.clientWidth,y:.5*this.frame.clientHeight},n=!0;else{var a=Math.round(this.frame.canvas.clientWidth*this.pixelRatio),s=Math.round(this.frame.canvas.clientHeight*this.pixelRatio);this.frame.canvas.width===a&&this.frame.canvas.height===s||this._getCameraState(o),this.frame.canvas.width!==a&&(this.frame.canvas.width=a,n=!0),this.frame.canvas.height!==s&&(this.frame.canvas.height=s,n=!0)}return!0===n&&(this.body.emitter.emit("resize",{width:Math.round(this.frame.canvas.width/this.pixelRatio),height:Math.round(this.frame.canvas.height/this.pixelRatio),oldWidth:Math.round(r/this.pixelRatio),oldHeight:Math.round(i/this.pixelRatio)}),this._setCameraState()),this.initialized=!0,n}},{key:"getContext",value:function(){return this.frame.canvas.getContext("2d")}},{key:"_determinePixelRatio",value:function(){var e=this.getContext();if(void 0===e)throw new Error("Could not get canvax context");var t=1;return"undefined"!=typeof window&&(t=window.devicePixelRatio||1),t/(e.webkitBackingStorePixelRatio||e.mozBackingStorePixelRatio||e.msBackingStorePixelRatio||e.oBackingStorePixelRatio||e.backingStorePixelRatio||1)}},{key:"_setPixelRatio",value:function(){this.pixelRatio=this._determinePixelRatio()}},{key:"setTransform",value:function(){var e=this.getContext();if(void 0===e)throw new Error("Could not get canvax context");e.setTransform(this.pixelRatio,0,0,this.pixelRatio,0,0)}},{key:"_XconvertDOMtoCanvas",value:function(e){return(e-this.body.view.translation.x)/this.body.view.scale}},{key:"_XconvertCanvasToDOM",value:function(e){return e*this.body.view.scale+this.body.view.translation.x}},{key:"_YconvertDOMtoCanvas",value:function(e){return(e-this.body.view.translation.y)/this.body.view.scale}},{key:"_YconvertCanvasToDOM",value:function(e){return e*this.body.view.scale+this.body.view.translation.y}},{key:"canvasToDOM",value:function(e){return{x:this._XconvertCanvasToDOM(e.x),y:this._YconvertCanvasToDOM(e.y)}}},{key:"DOMtoCanvas",value:function(e){return{x:this._XconvertDOMtoCanvas(e.x),y:this._YconvertDOMtoCanvas(e.y)}}}]),e}();t.default=u},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=o(n(0)),i=o(n(1));function o(e){return e&&e.__esModule?e:{default:e}}var a=n(2),s=n(76).default,l=function(){function e(t,n){var i=this;(0,r.default)(this,e),this.body=t,this.canvas=n,this.animationSpeed=1/this.renderRefreshRate,this.animationEasingFunction="easeInOutQuint",this.easingTime=0,this.sourceScale=0,this.targetScale=0,this.sourceTranslation=0,this.targetTranslation=0,this.lockedOnNodeId=void 0,this.lockedOnNodeOffset=void 0,this.touchTime=0,this.viewFunction=void 0,this.body.emitter.on("fit",this.fit.bind(this)),this.body.emitter.on("animationFinished",(function(){i.body.emitter.emit("_stopRendering")})),this.body.emitter.on("unlockNode",this.releaseNode.bind(this))}return(0,i.default)(e,[{key:"setOptions",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};this.options=e}},{key:"fit",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{nodes:[]},t=arguments.length>1&&void 0!==arguments[1]&&arguments[1],n=void 0,r=void 0;if(void 0!==e.nodes&&0!==e.nodes.length||(e.nodes=this.body.nodeIndices),!0===t){var i=0;for(var o in this.body.nodes)if(this.body.nodes.hasOwnProperty(o)){var a=this.body.nodes[o];!0===a.predefinedPosition&&(i+=1)}if(i>.5*this.body.nodeIndices.length)return void this.fit(e,!1);n=s.getRange(this.body.nodes,e.nodes);var l=this.body.nodeIndices.length;r=12.662/(l+7.4147)+.0964822;var u=Math.min(this.canvas.frame.canvas.clientWidth/600,this.canvas.frame.canvas.clientHeight/600);r*=u}else{this.body.emitter.emit("_resizeNodes"),n=s.getRange(this.body.nodes,e.nodes);var c=1.1*Math.abs(n.maxX-n.minX),d=1.1*Math.abs(n.maxY-n.minY),h=this.canvas.frame.canvas.clientWidth/c,f=this.canvas.frame.canvas.clientHeight/d;r=h<=f?h:f}(r>1||0===r)&&(r=1);var p=s.findCenter(n),g={position:p,scale:r,animation:e.animation};this.moveTo(g)}},{key:"focus",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};if(void 0!==this.body.nodes[e]){var n={x:this.body.nodes[e].x,y:this.body.nodes[e].y};t.position=n,t.lockedOnNode=e,this.moveTo(t)}else console.log("Node: "+e+" cannot be found.")}},{key:"moveTo",value:function(e){void 0!==e?(void 0===e.offset&&(e.offset={x:0,y:0}),void 0===e.offset.x&&(e.offset.x=0),void 0===e.offset.y&&(e.offset.y=0),void 0===e.scale&&(e.scale=this.body.view.scale),void 0===e.position&&(e.position=this.getViewPosition()),void 0===e.animation&&(e.animation={duration:0}),!1===e.animation&&(e.animation={duration:0}),!0===e.animation&&(e.animation={}),void 0===e.animation.duration&&(e.animation.duration=1e3),void 0===e.animation.easingFunction&&(e.animation.easingFunction="easeInOutQuad"),this.animateView(e)):e={}}},{key:"animateView",value:function(e){if(void 0!==e){this.animationEasingFunction=e.animation.easingFunction,this.releaseNode(),!0===e.locked&&(this.lockedOnNodeId=e.lockedOnNode,this.lockedOnNodeOffset=e.offset),0!=this.easingTime&&this._transitionRedraw(!0),this.sourceScale=this.body.view.scale,this.sourceTranslation=this.body.view.translation,this.targetScale=e.scale,this.body.view.scale=this.targetScale;var t=this.canvas.DOMtoCanvas({x:.5*this.canvas.frame.canvas.clientWidth,y:.5*this.canvas.frame.canvas.clientHeight}),n=t.x-e.position.x,r=t.y-e.position.y;this.targetTranslation={x:this.sourceTranslation.x+n*this.targetScale+e.offset.x,y:this.sourceTranslation.y+r*this.targetScale+e.offset.y},0===e.animation.duration?null!=this.lockedOnNodeId?(this.viewFunction=this._lockedRedraw.bind(this),this.body.emitter.on("initRedraw",this.viewFunction)):(this.body.view.scale=this.targetScale,this.body.view.translation=this.targetTranslation,this.body.emitter.emit("_requestRedraw")):(this.animationSpeed=1/(60*e.animation.duration*.001)||1/60,this.animationEasingFunction=e.animation.easingFunction,this.viewFunction=this._transitionRedraw.bind(this),this.body.emitter.on("initRedraw",this.viewFunction),this.body.emitter.emit("_startRendering"))}}},{key:"_lockedRedraw",value:function(){var e=this.body.nodes[this.lockedOnNodeId].x,t=this.body.nodes[this.lockedOnNodeId].y,n=this.canvas.DOMtoCanvas({x:.5*this.canvas.frame.canvas.clientWidth,y:.5*this.canvas.frame.canvas.clientHeight}),r=n.x-e,i=n.y-t,o=this.body.view.translation,a={x:o.x+r*this.body.view.scale+this.lockedOnNodeOffset.x,y:o.y+i*this.body.view.scale+this.lockedOnNodeOffset.y};this.body.view.translation=a}},{key:"releaseNode",value:function(){void 0!==this.lockedOnNodeId&&void 0!==this.viewFunction&&(this.body.emitter.off("initRedraw",this.viewFunction),this.lockedOnNodeId=void 0,this.lockedOnNodeOffset=void 0)}},{key:"_transitionRedraw",value:function(){var e=arguments.length>0&&void 0!==arguments[0]&&arguments[0];this.easingTime+=this.animationSpeed,this.easingTime=!0===e?1:this.easingTime;var t=a.easingFunctions[this.animationEasingFunction](this.easingTime);this.body.view.scale=this.sourceScale+(this.targetScale-this.sourceScale)*t,this.body.view.translation={x:this.sourceTranslation.x+(this.targetTranslation.x-this.sourceTranslation.x)*t,y:this.sourceTranslation.y+(this.targetTranslation.y-this.sourceTranslation.y)*t},this.easingTime>=1&&(this.body.emitter.off("initRedraw",this.viewFunction),this.easingTime=0,null!=this.lockedOnNodeId&&(this.viewFunction=this._lockedRedraw.bind(this),this.body.emitter.on("initRedraw",this.viewFunction)),this.body.emitter.emit("animationFinished"))}},{key:"getScale",value:function(){return this.body.view.scale}},{key:"getViewPosition",value:function(){return this.canvas.DOMtoCanvas({x:.5*this.canvas.frame.canvas.clientWidth,y:.5*this.canvas.frame.canvas.clientHeight})}}]),e}();t.default=l},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=o(n(0)),i=o(n(1));function o(e){return e&&e.__esModule?e:{default:e}}var a=n(2),s=n(233).default,l=n(104).default,u=function(){function e(t,n,i){(0,r.default)(this,e),this.body=t,this.canvas=n,this.selectionHandler=i,this.navigationHandler=new s(t,n),this.body.eventListeners.onTap=this.onTap.bind(this),this.body.eventListeners.onTouch=this.onTouch.bind(this),this.body.eventListeners.onDoubleTap=this.onDoubleTap.bind(this),this.body.eventListeners.onHold=this.onHold.bind(this),this.body.eventListeners.onDragStart=this.onDragStart.bind(this),this.body.eventListeners.onDrag=this.onDrag.bind(this),this.body.eventListeners.onDragEnd=this.onDragEnd.bind(this),this.body.eventListeners.onMouseWheel=this.onMouseWheel.bind(this),this.body.eventListeners.onPinch=this.onPinch.bind(this),this.body.eventListeners.onMouseMove=this.onMouseMove.bind(this),this.body.eventListeners.onRelease=this.onRelease.bind(this),this.body.eventListeners.onContext=this.onContext.bind(this),this.touchTime=0,this.drag={},this.pinch={},this.popup=void 0,this.popupObj=void 0,this.popupTimer=void 0,this.body.functions.getPointer=this.getPointer.bind(this),this.options={},this.defaultOptions={dragNodes:!0,dragView:!0,hover:!1,keyboard:{enabled:!1,speed:{x:10,y:10,zoom:.02},bindToWindow:!0},navigationButtons:!1,tooltipDelay:300,zoomView:!0},a.extend(this.options,this.defaultOptions),this.bindEventListeners()}return(0,i.default)(e,[{key:"bindEventListeners",value:function(){var e=this;this.body.emitter.on("destroy",(function(){clearTimeout(e.popupTimer),delete e.body.functions.getPointer}))}},{key:"setOptions",value:function(e){void 0!==e&&(a.selectiveNotDeepExtend(["hideEdgesOnDrag","hideNodesOnDrag","keyboard","multiselect","selectable","selectConnectedEdges"],this.options,e),a.mergeOptions(this.options,e,"keyboard"),e.tooltip&&(a.extend(this.options.tooltip,e.tooltip),e.tooltip.color&&(this.options.tooltip.color=a.parseColor(e.tooltip.color)))),this.navigationHandler.setOptions(this.options)}},{key:"getPointer",value:function(e){return{x:e.x-a.getAbsoluteLeft(this.canvas.frame.canvas),y:e.y-a.getAbsoluteTop(this.canvas.frame.canvas)}}},{key:"onTouch",value:function(e){(new Date).valueOf()-this.touchTime>50&&(this.drag.pointer=this.getPointer(e.center),this.drag.pinched=!1,this.pinch.scale=this.body.view.scale,this.touchTime=(new Date).valueOf())}},{key:"onTap",value:function(e){var t=this.getPointer(e.center),n=this.selectionHandler.options.multiselect&&(e.changedPointers[0].ctrlKey||e.changedPointers[0].metaKey);this.checkSelectionChanges(t,e,n),this.selectionHandler._generateClickEvent("click",e,t)}},{key:"onDoubleTap",value:function(e){var t=this.getPointer(e.center);this.selectionHandler._generateClickEvent("doubleClick",e,t)}},{key:"onHold",value:function(e){var t=this.getPointer(e.center),n=this.selectionHandler.options.multiselect;this.checkSelectionChanges(t,e,n),this.selectionHandler._generateClickEvent("click",e,t),this.selectionHandler._generateClickEvent("hold",e,t)}},{key:"onRelease",value:function(e){if((new Date).valueOf()-this.touchTime>10){var t=this.getPointer(e.center);this.selectionHandler._generateClickEvent("release",e,t),this.touchTime=(new Date).valueOf()}}},{key:"onContext",value:function(e){var t=this.getPointer({x:e.clientX,y:e.clientY});this.selectionHandler._generateClickEvent("oncontext",e,t)}},{key:"checkSelectionChanges",value:function(e,t){var n=arguments.length>2&&void 0!==arguments[2]&&arguments[2],r=this.selectionHandler.getSelection(),i=!1;i=!0===n?this.selectionHandler.selectAdditionalOnPoint(e):this.selectionHandler.selectOnPoint(e);var o=this.selectionHandler.getSelection(),a=this._determineDifference(r,o),s=this._determineDifference(o,r);a.edges.length>0&&(this.selectionHandler._generateClickEvent("deselectEdge",t,e,r),i=!0),a.nodes.length>0&&(this.selectionHandler._generateClickEvent("deselectNode",t,e,r),i=!0),s.nodes.length>0&&(this.selectionHandler._generateClickEvent("selectNode",t,e),i=!0),s.edges.length>0&&(this.selectionHandler._generateClickEvent("selectEdge",t,e),i=!0),!0===i&&this.selectionHandler._generateClickEvent("select",t,e)}},{key:"_determineDifference",value:function(e,t){var n=function(e,t){for(var n=[],r=0;r<e.length;r++){var i=e[r];-1===t.indexOf(i)&&n.push(i)}return n};return{nodes:n(e.nodes,t.nodes),edges:n(e.edges,t.edges)}}},{key:"onDragStart",value:function(e){void 0===this.drag.pointer&&this.onTouch(e);var t=this.selectionHandler.getNodeAt(this.drag.pointer);if(this.drag.dragging=!0,this.drag.selection=[],this.drag.translation=a.extend({},this.body.view.translation),this.drag.nodeId=void 0,void 0!==t&&!0===this.options.dragNodes){this.drag.nodeId=t.id,!1===t.isSelected()&&(this.selectionHandler.unselectAll(),this.selectionHandler.selectObject(t)),this.selectionHandler._generateClickEvent("dragStart",e,this.drag.pointer);var n=this.selectionHandler.selectionObj.nodes;for(var r in n)if(n.hasOwnProperty(r)){var i=n[r],o={id:i.id,node:i,x:i.x,y:i.y,xFixed:i.options.fixed.x,yFixed:i.options.fixed.y};i.options.fixed.x=!0,i.options.fixed.y=!0,this.drag.selection.push(o)}}else this.selectionHandler._generateClickEvent("dragStart",e,this.drag.pointer,void 0,!0)}},{key:"onDrag",value:function(e){var t=this;if(!0!==this.drag.pinched){this.body.emitter.emit("unlockNode");var n=this.getPointer(e.center),r=this.drag.selection;if(r&&r.length&&!0===this.options.dragNodes){this.selectionHandler._generateClickEvent("dragging",e,n);var i=n.x-this.drag.pointer.x,o=n.y-this.drag.pointer.y;r.forEach((function(e){var n=e.node;!1===e.xFixed&&(n.x=t.canvas._XconvertDOMtoCanvas(t.canvas._XconvertCanvasToDOM(e.x)+i)),!1===e.yFixed&&(n.y=t.canvas._YconvertDOMtoCanvas(t.canvas._YconvertCanvasToDOM(e.y)+o))})),this.body.emitter.emit("startSimulation")}else if(!0===this.options.dragView){if(this.selectionHandler._generateClickEvent("dragging",e,n,void 0,!0),void 0===this.drag.pointer)return void this.onDragStart(e);var a=n.x-this.drag.pointer.x,s=n.y-this.drag.pointer.y;this.body.view.translation={x:this.drag.translation.x+a,y:this.drag.translation.y+s},this.body.emitter.emit("_requestRedraw")}}}},{key:"onDragEnd",value:function(e){this.drag.dragging=!1;var t=this.drag.selection;t&&t.length?(t.forEach((function(e){e.node.options.fixed.x=e.xFixed,e.node.options.fixed.y=e.yFixed})),this.selectionHandler._generateClickEvent("dragEnd",e,this.getPointer(e.center)),this.body.emitter.emit("startSimulation")):(this.selectionHandler._generateClickEvent("dragEnd",e,this.getPointer(e.center),void 0,!0),this.body.emitter.emit("_requestRedraw"))}},{key:"onPinch",value:function(e){var t=this.getPointer(e.center);this.drag.pinched=!0,void 0===this.pinch.scale&&(this.pinch.scale=1);var n=this.pinch.scale*e.scale;this.zoom(n,t)}},{key:"zoom",value:function(e,t){if(!0===this.options.zoomView){var n=this.body.view.scale;e<1e-5&&(e=1e-5),e>10&&(e=10);var r=void 0;void 0!==this.drag&&!0===this.drag.dragging&&(r=this.canvas.DOMtoCanvas(this.drag.pointer));var i=this.body.view.translation,o=e/n,a=(1-o)*t.x+i.x*o,s=(1-o)*t.y+i.y*o;if(this.body.view.scale=e,this.body.view.translation={x:a,y:s},null!=r){var l=this.canvas.canvasToDOM(r);this.drag.pointer.x=l.x,this.drag.pointer.y=l.y}this.body.emitter.emit("_requestRedraw"),n<e?this.body.emitter.emit("zoom",{direction:"+",scale:this.body.view.scale,pointer:t}):this.body.emitter.emit("zoom",{direction:"-",scale:this.body.view.scale,pointer:t})}}},{key:"onMouseWheel",value:function(e){if(!0===this.options.zoomView){var t=0;if(e.wheelDelta?t=e.wheelDelta/120:e.detail&&(t=-e.detail/3),0!==t){var n=this.body.view.scale,r=t/10;t<0&&(r/=1-r),n*=1+r;var i=this.getPointer({x:e.clientX,y:e.clientY});this.zoom(n,i)}e.preventDefault()}}},{key:"onMouseMove",value:function(e){var t=this,n=this.getPointer({x:e.clientX,y:e.clientY}),r=!1;void 0!==this.popup&&(!1===this.popup.hidden&&this._checkHidePopup(n),!1===this.popup.hidden&&(r=!0,this.popup.setPosition(n.x+3,n.y-5),this.popup.show())),!1===this.options.keyboard.bindToWindow&&!0===this.options.keyboard.enabled&&this.canvas.frame.focus(),!1===r&&(void 0!==this.popupTimer&&(clearInterval(this.popupTimer),this.popupTimer=void 0),this.drag.dragging||(this.popupTimer=setTimeout((function(){return t._checkShowPopup(n)}),this.options.tooltipDelay))),!0===this.options.hover&&this.selectionHandler.hoverObject(e,n)}},{key:"_checkShowPopup",value:function(e){var t=this.canvas._XconvertDOMtoCanvas(e.x),n=this.canvas._YconvertDOMtoCanvas(e.y),r={left:t,top:n,right:t,bottom:n},i=void 0===this.popupObj?void 0:this.popupObj.id,o=!1,a="node";if(void 0===this.popupObj){for(var s=this.body.nodeIndices,u=this.body.nodes,c=void 0,d=[],h=0;h<s.length;h++)!0===(c=u[s[h]]).isOverlappingWith(r)&&void 0!==c.getTitle()&&d.push(s[h]);d.length>0&&(this.popupObj=u[d[d.length-1]],o=!0)}if(void 0===this.popupObj&&!1===o){for(var f=this.body.edgeIndices,p=this.body.edges,g=void 0,v=[],m=0;m<f.length;m++)!0===(g=p[f[m]]).isOverlappingWith(r)&&!0===g.connected&&void 0!==g.getTitle()&&v.push(f[m]);v.length>0&&(this.popupObj=p[v[v.length-1]],a="edge")}void 0!==this.popupObj?this.popupObj.id!==i&&(void 0===this.popup&&(this.popup=new l(this.canvas.frame)),this.popup.popupTargetType=a,this.popup.popupTargetId=this.popupObj.id,this.popup.setPosition(e.x+3,e.y-5),this.popup.setText(this.popupObj.getTitle()),this.popup.show(),this.body.emitter.emit("showPopup",this.popupObj.id)):void 0!==this.popup&&(this.popup.hide(),this.body.emitter.emit("hidePopup"))}},{key:"_checkHidePopup",value:function(e){var t=this.selectionHandler._pointerToPositionObject(e),n=!1;if("node"===this.popup.popupTargetType){if(void 0!==this.body.nodes[this.popup.popupTargetId]&&!0===(n=this.body.nodes[this.popup.popupTargetId].isOverlappingWith(t))){var r=this.selectionHandler.getNodeAt(e);n=void 0!==r&&r.id===this.popup.popupTargetId}}else void 0===this.selectionHandler.getNodeAt(e)&&void 0!==this.body.edges[this.popup.popupTargetId]&&(n=this.body.edges[this.popup.popupTargetId].isOverlappingWith(t));!1===n&&(this.popupObj=void 0,this.popup.hide(),this.body.emitter.emit("hidePopup"))}}]),e}();t.default=u},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=o(n(0)),i=o(n(1));function o(e){return e&&e.__esModule?e:{default:e}}var a=n(10),s=n(37),l=n(35),u=function(){function e(t,n){var i=this;(0,r.default)(this,e),this.body=t,this.canvas=n,this.iconsCreated=!1,this.navigationHammers=[],this.boundFunctions={},this.touchTime=0,this.activated=!1,this.body.emitter.on("activate",(function(){i.activated=!0,i.configureKeyboardBindings()})),this.body.emitter.on("deactivate",(function(){i.activated=!1,i.configureKeyboardBindings()})),this.body.emitter.on("destroy",(function(){void 0!==i.keycharm&&i.keycharm.destroy()})),this.options={}}return(0,i.default)(e,[{key:"setOptions",value:function(e){void 0!==e&&(this.options=e,this.create())}},{key:"create",value:function(){!0===this.options.navigationButtons?!1===this.iconsCreated&&this.loadNavigationElements():!0===this.iconsCreated&&this.cleanNavigation(),this.configureKeyboardBindings()}},{key:"cleanNavigation",value:function(){if(0!=this.navigationHammers.length){for(var e=0;e<this.navigationHammers.length;e++)this.navigationHammers[e].destroy();this.navigationHammers=[]}this.navigationDOM&&this.navigationDOM.wrapper&&this.navigationDOM.wrapper.parentNode&&this.navigationDOM.wrapper.parentNode.removeChild(this.navigationDOM.wrapper),this.iconsCreated=!1}},{key:"loadNavigationElements",value:function(){var e=this;this.cleanNavigation(),this.navigationDOM={};var t=["up","down","left","right","zoomIn","zoomOut","zoomExtends"],n=["_moveUp","_moveDown","_moveLeft","_moveRight","_zoomIn","_zoomOut","_fit"];this.navigationDOM.wrapper=document.createElement("div"),this.navigationDOM.wrapper.className="vis-navigation",this.canvas.frame.appendChild(this.navigationDOM.wrapper);for(var r=0;r<t.length;r++){this.navigationDOM[t[r]]=document.createElement("div"),this.navigationDOM[t[r]].className="vis-button vis-"+t[r],this.navigationDOM.wrapper.appendChild(this.navigationDOM[t[r]]);var i=new a(this.navigationDOM[t[r]]);"_fit"===n[r]?s.onTouch(i,this._fit.bind(this)):s.onTouch(i,this.bindToRedraw.bind(this,n[r])),this.navigationHammers.push(i)}var o=new a(this.canvas.frame);s.onRelease(o,(function(){e._stopMovement()})),this.navigationHammers.push(o),this.iconsCreated=!0}},{key:"bindToRedraw",value:function(e){void 0===this.boundFunctions[e]&&(this.boundFunctions[e]=this[e].bind(this),this.body.emitter.on("initRedraw",this.boundFunctions[e]),this.body.emitter.emit("_startRendering"))}},{key:"unbindFromRedraw",value:function(e){void 0!==this.boundFunctions[e]&&(this.body.emitter.off("initRedraw",this.boundFunctions[e]),this.body.emitter.emit("_stopRendering"),delete this.boundFunctions[e])}},{key:"_fit",value:function(){(new Date).valueOf()-this.touchTime>700&&(this.body.emitter.emit("fit",{duration:700}),this.touchTime=(new Date).valueOf())}},{key:"_stopMovement",value:function(){for(var e in this.boundFunctions)this.boundFunctions.hasOwnProperty(e)&&(this.body.emitter.off("initRedraw",this.boundFunctions[e]),this.body.emitter.emit("_stopRendering"));this.boundFunctions={}}},{key:"_moveUp",value:function(){this.body.view.translation.y+=this.options.keyboard.speed.y}},{key:"_moveDown",value:function(){this.body.view.translation.y-=this.options.keyboard.speed.y}},{key:"_moveLeft",value:function(){this.body.view.translation.x+=this.options.keyboard.speed.x}},{key:"_moveRight",value:function(){this.body.view.translation.x-=this.options.keyboard.speed.x}},{key:"_zoomIn",value:function(){var e=this.body.view.scale,t=this.body.view.scale*(1+this.options.keyboard.speed.zoom),n=this.body.view.translation,r=t/e,i=(1-r)*this.canvas.canvasViewCenter.x+n.x*r,o=(1-r)*this.canvas.canvasViewCenter.y+n.y*r;this.body.view.scale=t,this.body.view.translation={x:i,y:o},this.body.emitter.emit("zoom",{direction:"+",scale:this.body.view.scale,pointer:null})}},{key:"_zoomOut",value:function(){var e=this.body.view.scale,t=this.body.view.scale/(1+this.options.keyboard.speed.zoom),n=this.body.view.translation,r=t/e,i=(1-r)*this.canvas.canvasViewCenter.x+n.x*r,o=(1-r)*this.canvas.canvasViewCenter.y+n.y*r;this.body.view.scale=t,this.body.view.translation={x:i,y:o},this.body.emitter.emit("zoom",{direction:"-",scale:this.body.view.scale,pointer:null})}},{key:"configureKeyboardBindings",value:function(){var e=this;void 0!==this.keycharm&&this.keycharm.destroy(),!0===this.options.keyboard.enabled&&(!0===this.options.keyboard.bindToWindow?this.keycharm=l({container:window,preventDefault:!0}):this.keycharm=l({container:this.canvas.frame,preventDefault:!0}),this.keycharm.reset(),!0===this.activated&&(this.keycharm.bind("up",(function(){e.bindToRedraw("_moveUp")}),"keydown"),this.keycharm.bind("down",(function(){e.bindToRedraw("_moveDown")}),"keydown"),this.keycharm.bind("left",(function(){e.bindToRedraw("_moveLeft")}),"keydown"),this.keycharm.bind("right",(function(){e.bindToRedraw("_moveRight")}),"keydown"),this.keycharm.bind("=",(function(){e.bindToRedraw("_zoomIn")}),"keydown"),this.keycharm.bind("num+",(function(){e.bindToRedraw("_zoomIn")}),"keydown"),this.keycharm.bind("num-",(function(){e.bindToRedraw("_zoomOut")}),"keydown"),this.keycharm.bind("-",(function(){e.bindToRedraw("_zoomOut")}),"keydown"),this.keycharm.bind("[",(function(){e.bindToRedraw("_zoomOut")}),"keydown"),this.keycharm.bind("]",(function(){e.bindToRedraw("_zoomIn")}),"keydown"),this.keycharm.bind("pageup",(function(){e.bindToRedraw("_zoomIn")}),"keydown"),this.keycharm.bind("pagedown",(function(){e.bindToRedraw("_zoomOut")}),"keydown"),this.keycharm.bind("up",(function(){e.unbindFromRedraw("_moveUp")}),"keyup"),this.keycharm.bind("down",(function(){e.unbindFromRedraw("_moveDown")}),"keyup"),this.keycharm.bind("left",(function(){e.unbindFromRedraw("_moveLeft")}),"keyup"),this.keycharm.bind("right",(function(){e.unbindFromRedraw("_moveRight")}),"keyup"),this.keycharm.bind("=",(function(){e.unbindFromRedraw("_zoomIn")}),"keyup"),this.keycharm.bind("num+",(function(){e.unbindFromRedraw("_zoomIn")}),"keyup"),this.keycharm.bind("num-",(function(){e.unbindFromRedraw("_zoomOut")}),"keyup"),this.keycharm.bind("-",(function(){e.unbindFromRedraw("_zoomOut")}),"keyup"),this.keycharm.bind("[",(function(){e.unbindFromRedraw("_zoomOut")}),"keyup"),this.keycharm.bind("]",(function(){e.unbindFromRedraw("_zoomIn")}),"keyup"),this.keycharm.bind("pageup",(function(){e.unbindFromRedraw("_zoomIn")}),"keyup"),this.keycharm.bind("pagedown",(function(){e.unbindFromRedraw("_zoomOut")}),"keyup")))}}]),e}();t.default=u},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=o(n(0)),i=o(n(1));function o(e){return e&&e.__esModule?e:{default:e}}var a=n(47).default,s=n(74).default,l=n(2),u=function(){function e(t,n){var i=this;(0,r.default)(this,e),this.body=t,this.canvas=n,this.selectionObj={nodes:[],edges:[]},this.hoverObj={nodes:{},edges:{}},this.options={},this.defaultOptions={multiselect:!1,selectable:!0,selectConnectedEdges:!0,hoverConnectedEdges:!0},l.extend(this.options,this.defaultOptions),this.body.emitter.on("_dataChanged",(function(){i.updateSelection()}))}return(0,i.default)(e,[{key:"setOptions",value:function(e){void 0!==e&&l.selectiveDeepExtend(["multiselect","hoverConnectedEdges","selectable","selectConnectedEdges"],this.options,e)}},{key:"selectOnPoint",value:function(e){var t=!1;if(!0===this.options.selectable){var n=this.getNodeAt(e)||this.getEdgeAt(e);this.unselectAll(),void 0!==n&&(t=this.selectObject(n)),this.body.emitter.emit("_requestRedraw")}return t}},{key:"selectAdditionalOnPoint",value:function(e){var t=!1;if(!0===this.options.selectable){var n=this.getNodeAt(e)||this.getEdgeAt(e);void 0!==n&&(t=!0,!0===n.isSelected()?this.deselectObject(n):this.selectObject(n),this.body.emitter.emit("_requestRedraw"))}return t}},{key:"_initBaseEvent",value:function(e,t){var n={};return n.pointer={DOM:{x:t.x,y:t.y},canvas:this.canvas.DOMtoCanvas(t)},n.event=e,n}},{key:"_generateClickEvent",value:function(e,t,n,r){var i=arguments.length>4&&void 0!==arguments[4]&&arguments[4],o=this._initBaseEvent(t,n);if(!0===i)o.nodes=[],o.edges=[];else{var a=this.getSelection();o.nodes=a.nodes,o.edges=a.edges}void 0!==r&&(o.previousSelection=r),"click"==e&&(o.items=this.getClickedItems(n)),this.body.emitter.emit(e,o)}},{key:"selectObject",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:this.options.selectConnectedEdges;return void 0!==e&&(e instanceof a&&!0===t&&this._selectConnectedEdges(e),e.select(),this._addToSelection(e),!0)}},{key:"deselectObject",value:function(e){!0===e.isSelected()&&(e.selected=!1,this._removeFromSelection(e))}},{key:"_getAllNodesOverlappingWith",value:function(e){for(var t=[],n=this.body.nodes,r=0;r<this.body.nodeIndices.length;r++){var i=this.body.nodeIndices[r];n[i].isOverlappingWith(e)&&t.push(i)}return t}},{key:"_pointerToPositionObject",value:function(e){var t=this.canvas.DOMtoCanvas(e);return{left:t.x-1,top:t.y+1,right:t.x+1,bottom:t.y-1}}},{key:"getNodeAt",value:function(e){var t=!(arguments.length>1&&void 0!==arguments[1])||arguments[1],n=this._pointerToPositionObject(e),r=this._getAllNodesOverlappingWith(n);return r.length>0?!0===t?this.body.nodes[r[r.length-1]]:r[r.length-1]:void 0}},{key:"_getEdgesOverlappingWith",value:function(e,t){for(var n=this.body.edges,r=0;r<this.body.edgeIndices.length;r++){var i=this.body.edgeIndices[r];n[i].isOverlappingWith(e)&&t.push(i)}}},{key:"_getAllEdgesOverlappingWith",value:function(e){var t=[];return this._getEdgesOverlappingWith(e,t),t}},{key:"getEdgeAt",value:function(e){for(var t=!(arguments.length>1&&void 0!==arguments[1])||arguments[1],n=this.canvas.DOMtoCanvas(e),r=10,i=null,o=this.body.edges,a=0;a<this.body.edgeIndices.length;a++){var s=this.body.edgeIndices[a],l=o[s];if(l.connected){var u=l.from.x,c=l.from.y,d=l.to.x,h=l.to.y,f=l.edgeType.getDistanceToEdge(u,c,d,h,n.x,n.y);f<r&&(i=s,r=f)}}return null!==i?!0===t?this.body.edges[i]:i:void 0}},{key:"_addToSelection",value:function(e){e instanceof a?this.selectionObj.nodes[e.id]=e:this.selectionObj.edges[e.id]=e}},{key:"_addToHover",value:function(e){e instanceof a?this.hoverObj.nodes[e.id]=e:this.hoverObj.edges[e.id]=e}},{key:"_removeFromSelection",value:function(e){e instanceof a?(delete this.selectionObj.nodes[e.id],this._unselectConnectedEdges(e)):delete this.selectionObj.edges[e.id]}},{key:"unselectAll",value:function(){for(var e in this.selectionObj.nodes)this.selectionObj.nodes.hasOwnProperty(e)&&this.selectionObj.nodes[e].unselect();for(var t in this.selectionObj.edges)this.selectionObj.edges.hasOwnProperty(t)&&this.selectionObj.edges[t].unselect();this.selectionObj={nodes:{},edges:{}}}},{key:"_getSelectedNodeCount",value:function(){var e=0;for(var t in this.selectionObj.nodes)this.selectionObj.nodes.hasOwnProperty(t)&&(e+=1);return e}},{key:"_getSelectedNode",value:function(){for(var e in this.selectionObj.nodes)if(this.selectionObj.nodes.hasOwnProperty(e))return this.selectionObj.nodes[e]}},{key:"_getSelectedEdge",value:function(){for(var e in this.selectionObj.edges)if(this.selectionObj.edges.hasOwnProperty(e))return this.selectionObj.edges[e]}},{key:"_getSelectedEdgeCount",value:function(){var e=0;for(var t in this.selectionObj.edges)this.selectionObj.edges.hasOwnProperty(t)&&(e+=1);return e}},{key:"_getSelectedObjectCount",value:function(){var e=0;for(var t in this.selectionObj.nodes)this.selectionObj.nodes.hasOwnProperty(t)&&(e+=1);for(var n in this.selectionObj.edges)this.selectionObj.edges.hasOwnProperty(n)&&(e+=1);return e}},{key:"_selectionIsEmpty",value:function(){for(var e in this.selectionObj.nodes)if(this.selectionObj.nodes.hasOwnProperty(e))return!1;for(var t in this.selectionObj.edges)if(this.selectionObj.edges.hasOwnProperty(t))return!1;return!0}},{key:"_clusterInSelection",value:function(){for(var e in this.selectionObj.nodes)if(this.selectionObj.nodes.hasOwnProperty(e)&&this.selectionObj.nodes[e].clusterSize>1)return!0;return!1}},{key:"_selectConnectedEdges",value:function(e){for(var t=0;t<e.edges.length;t++){var n=e.edges[t];n.select(),this._addToSelection(n)}}},{key:"_hoverConnectedEdges",value:function(e){for(var t=0;t<e.edges.length;t++){var n=e.edges[t];n.hover=!0,this._addToHover(n)}}},{key:"_unselectConnectedEdges",value:function(e){for(var t=0;t<e.edges.length;t++){var n=e.edges[t];n.unselect(),this._removeFromSelection(n)}}},{key:"emitBlurEvent",value:function(e,t,n){var r=this._initBaseEvent(e,t);!0===n.hover&&(n.hover=!1,n instanceof a?(r.node=n.id,this.body.emitter.emit("blurNode",r)):(r.edge=n.id,this.body.emitter.emit("blurEdge",r)))}},{key:"emitHoverEvent",value:function(e,t,n){var r=this._initBaseEvent(e,t),i=!1;return!1===n.hover&&(n.hover=!0,this._addToHover(n),i=!0,n instanceof a?(r.node=n.id,this.body.emitter.emit("hoverNode",r)):(r.edge=n.id,this.body.emitter.emit("hoverEdge",r))),i}},{key:"hoverObject",value:function(e,t){var n=this.getNodeAt(t);void 0===n&&(n=this.getEdgeAt(t));var r=!1;for(var i in this.hoverObj.nodes)this.hoverObj.nodes.hasOwnProperty(i)&&(void 0===n||n instanceof a&&n.id!=i||n instanceof s)&&(this.emitBlurEvent(e,t,this.hoverObj.nodes[i]),delete this.hoverObj.nodes[i],r=!0);for(var o in this.hoverObj.edges)this.hoverObj.edges.hasOwnProperty(o)&&(!0===r?(this.hoverObj.edges[o].hover=!1,delete this.hoverObj.edges[o]):(void 0===n||n instanceof s&&n.id!=o||n instanceof a&&!n.hover)&&(this.emitBlurEvent(e,t,this.hoverObj.edges[o]),delete this.hoverObj.edges[o],r=!0));void 0!==n&&(r=r||this.emitHoverEvent(e,t,n),n instanceof a&&!0===this.options.hoverConnectedEdges&&this._hoverConnectedEdges(n)),!0===r&&this.body.emitter.emit("_requestRedraw")}},{key:"getSelection",value:function(){return{nodes:this.getSelectedNodes(),edges:this.getSelectedEdges()}}},{key:"getSelectedNodes",value:function(){var e=[];if(!0===this.options.selectable)for(var t in this.selectionObj.nodes)this.selectionObj.nodes.hasOwnProperty(t)&&e.push(this.selectionObj.nodes[t].id);return e}},{key:"getSelectedEdges",value:function(){var e=[];if(!0===this.options.selectable)for(var t in this.selectionObj.edges)this.selectionObj.edges.hasOwnProperty(t)&&e.push(this.selectionObj.edges[t].id);return e}},{key:"setSelection",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=void 0,r=void 0;if(!e||!e.nodes&&!e.edges)throw"Selection must be an object with nodes and/or edges properties";if((t.unselectAll||void 0===t.unselectAll)&&this.unselectAll(),e.nodes)for(n=0;n<e.nodes.length;n++){r=e.nodes[n];var i=this.body.nodes[r];if(!i)throw new RangeError('Node with id "'+r+'" not found');this.selectObject(i,t.highlightEdges)}if(e.edges)for(n=0;n<e.edges.length;n++){r=e.edges[n];var o=this.body.edges[r];if(!o)throw new RangeError('Edge with id "'+r+'" not found');this.selectObject(o)}this.body.emitter.emit("_requestRedraw")}},{key:"selectNodes",value:function(e){var t=!(arguments.length>1&&void 0!==arguments[1])||arguments[1];if(!e||void 0===e.length)throw"Selection must be an array with ids";this.setSelection({nodes:e},{highlightEdges:t})}},{key:"selectEdges",value:function(e){if(!e||void 0===e.length)throw"Selection must be an array with ids";this.setSelection({edges:e})}},{key:"updateSelection",value:function(){for(var e in this.selectionObj.nodes)this.selectionObj.nodes.hasOwnProperty(e)&&(this.body.nodes.hasOwnProperty(e)||delete this.selectionObj.nodes[e]);for(var t in this.selectionObj.edges)this.selectionObj.edges.hasOwnProperty(t)&&(this.body.edges.hasOwnProperty(t)||delete this.selectionObj.edges[t])}},{key:"getClickedItems",value:function(e){for(var t=this.canvas.DOMtoCanvas(e),n=[],r=this.body.nodeIndices,i=this.body.nodes,o=r.length-1;o>=0;o--){var a=i[r[o]].getItemsOnPoint(t);n.push.apply(n,a)}for(var s=this.body.edgeIndices,l=this.body.edges,u=s.length-1;u>=0;u--){var c=l[s[u]].getItemsOnPoint(t);n.push.apply(n,c)}return n}}]),e}();t.default=u},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=l(n(30)),i=l(n(6)),o=l(n(8)),a=l(n(0)),s=l(n(1));function l(e){return e&&e.__esModule?e:{default:e}}var u=n(2),c=n(76).default,d=n(236),h=d.HorizontalStrategy,f=d.VerticalStrategy,p=function(){function e(){(0,a.default)(this,e),this.childrenReference={},this.parentReference={},this.trees={},this.distributionOrdering={},this.levels={},this.distributionIndex={},this.isTree=!1,this.treeIndex=-1}return(0,s.default)(e,[{key:"addRelation",value:function(e,t){void 0===this.childrenReference[e]&&(this.childrenReference[e]=[]),this.childrenReference[e].push(t),void 0===this.parentReference[t]&&(this.parentReference[t]=[]),this.parentReference[t].push(e)}},{key:"checkIfTree",value:function(){for(var e in this.parentReference)if(this.parentReference[e].length>1)return void(this.isTree=!1);this.isTree=!0}},{key:"numTrees",value:function(){return this.treeIndex+1}},{key:"setTreeIndex",value:function(e,t){void 0!==t&&void 0===this.trees[e.id]&&(this.trees[e.id]=t,this.treeIndex=Math.max(t,this.treeIndex))}},{key:"ensureLevel",value:function(e){void 0===this.levels[e]&&(this.levels[e]=0)}},{key:"getMaxLevel",value:function(e){var t=this,n={};return function e(r){if(void 0!==n[r])return n[r];var i=t.levels[r];if(t.childrenReference[r]){var o=t.childrenReference[r];if(o.length>0)for(var a=0;a<o.length;a++)i=Math.max(i,e(o[a]))}return n[r]=i,i}(e)}},{key:"levelDownstream",value:function(e,t){void 0===this.levels[t.id]&&(void 0===this.levels[e.id]&&(this.levels[e.id]=0),this.levels[t.id]=this.levels[e.id]+1)}},{key:"setMinLevelToZero",value:function(e){var t=1e9;for(var n in e)e.hasOwnProperty(n)&&void 0!==this.levels[n]&&(t=Math.min(this.levels[n],t));for(var r in e)e.hasOwnProperty(r)&&void 0!==this.levels[r]&&(this.levels[r]-=t)}},{key:"getTreeSize",value:function(e,t){var n=1e9,r=-1e9,i=1e9,o=-1e9;for(var a in this.trees)if(this.trees.hasOwnProperty(a)&&this.trees[a]===t){var s=e[a];n=Math.min(s.x,n),r=Math.max(s.x,r),i=Math.min(s.y,i),o=Math.max(s.y,o)}return{min_x:n,max_x:r,min_y:i,max_y:o}}},{key:"hasSameParent",value:function(e,t){var n=this.parentReference[e.id],r=this.parentReference[t.id];if(void 0===n||void 0===r)return!1;for(var i=0;i<n.length;i++)for(var o=0;o<r.length;o++)if(n[i]==r[o])return!0;return!1}},{key:"inSameSubNetwork",value:function(e,t){return this.trees[e.id]===this.trees[t.id]}},{key:"getLevels",value:function(){return(0,o.default)(this.distributionOrdering)}},{key:"addToOrdering",value:function(e,t){void 0===this.distributionOrdering[t]&&(this.distributionOrdering[t]=[]);var n=!1,r=this.distributionOrdering[t];for(var i in r)if(r[i]===e){n=!0;break}n||(this.distributionOrdering[t].push(e),this.distributionIndex[e.id]=this.distributionOrdering[t].length-1)}}]),e}(),g=function(){function e(t){(0,a.default)(this,e),this.body=t,this.initialRandomSeed=Math.round(1e6*Math.random()),this.randomSeed=this.initialRandomSeed,this.setPhysics=!1,this.options={},this.optionsBackup={physics:{}},this.defaultOptions={randomSeed:void 0,improvedLayout:!0,hierarchical:{enabled:!1,levelSeparation:150,nodeSpacing:100,treeSpacing:200,blockShifting:!0,edgeMinimization:!0,parentCentralization:!0,direction:"UD",sortMethod:"hubsize"}},u.extend(this.options,this.defaultOptions),this.bindEventListeners()}return(0,s.default)(e,[{key:"bindEventListeners",value:function(){var e=this;this.body.emitter.on("_dataChanged",(function(){e.setupHierarchicalLayout()})),this.body.emitter.on("_dataLoaded",(function(){e.layoutNetwork()})),this.body.emitter.on("_resetHierarchicalLayout",(function(){e.setupHierarchicalLayout()})),this.body.emitter.on("_adjustEdgesForHierarchicalLayout",(function(){if(!0===e.options.hierarchical.enabled){var t=e.direction.curveType();e.body.emitter.emit("_forceDisableDynamicCurves",t,!1)}}))}},{key:"setOptions",value:function(e,t){if(void 0!==e){var n=this.options.hierarchical,r=n.enabled;if(u.selectiveDeepExtend(["randomSeed","improvedLayout"],this.options,e),u.mergeOptions(this.options,e,"hierarchical"),void 0!==e.randomSeed&&(this.initialRandomSeed=e.randomSeed),!0===n.enabled)return!0===r&&this.body.emitter.emit("refresh",!0),"RL"===n.direction||"DU"===n.direction?n.levelSeparation>0&&(n.levelSeparation*=-1):n.levelSeparation<0&&(n.levelSeparation*=-1),this.setDirectionStrategy(),this.body.emitter.emit("_resetHierarchicalLayout"),this.adaptAllOptionsForHierarchicalLayout(t);if(!0===r)return this.body.emitter.emit("refresh"),u.deepExtend(t,this.optionsBackup)}return t}},{key:"adaptAllOptionsForHierarchicalLayout",value:function(e){if(!0===this.options.hierarchical.enabled){var t=this.optionsBackup.physics;void 0===e.physics||!0===e.physics?(e.physics={enabled:void 0===t.enabled||t.enabled,solver:"hierarchicalRepulsion"},t.enabled=void 0===t.enabled||t.enabled,t.solver=t.solver||"barnesHut"):"object"===(0,i.default)(e.physics)?(t.enabled=void 0===e.physics.enabled||e.physics.enabled,t.solver=e.physics.solver||"barnesHut",e.physics.solver="hierarchicalRepulsion"):!1!==e.physics&&(t.solver="barnesHut",e.physics={solver:"hierarchicalRepulsion"});var n=this.direction.curveType();if(void 0===e.edges)this.optionsBackup.edges={smooth:{enabled:!0,type:"dynamic"}},e.edges={smooth:!1};else if(void 0===e.edges.smooth)this.optionsBackup.edges={smooth:{enabled:!0,type:"dynamic"}},e.edges.smooth=!1;else if("boolean"==typeof e.edges.smooth)this.optionsBackup.edges={smooth:e.edges.smooth},e.edges.smooth={enabled:e.edges.smooth,type:n};else{var r=e.edges.smooth;void 0!==r.type&&"dynamic"!==r.type&&(n=r.type),this.optionsBackup.edges={smooth:void 0===r.enabled||r.enabled,type:void 0===r.type?"dynamic":r.type,roundness:void 0===r.roundness?.5:r.roundness,forceDirection:void 0!==r.forceDirection&&r.forceDirection},e.edges.smooth={enabled:void 0===r.enabled||r.enabled,type:n,roundness:void 0===r.roundness?.5:r.roundness,forceDirection:void 0!==r.forceDirection&&r.forceDirection}}this.body.emitter.emit("_forceDisableDynamicCurves",n)}return e}},{key:"seededRandom",value:function(){var e=1e4*Math.sin(this.randomSeed++);return e-Math.floor(e)}},{key:"positionInitially",value:function(e){if(!0!==this.options.hierarchical.enabled){this.randomSeed=this.initialRandomSeed;for(var t=e.length+50,n=0;n<e.length;n++){var r=e[n],i=2*Math.PI*this.seededRandom();void 0===r.x&&(r.x=t*Math.cos(i)),void 0===r.y&&(r.y=t*Math.sin(i))}}}},{key:"layoutNetwork",value:function(){if(!0!==this.options.hierarchical.enabled&&!0===this.options.improvedLayout){for(var e=this.body.nodeIndices,t=0,n=0;n<e.length;n++)!0===this.body.nodes[e[n]].predefinedPosition&&(t+=1);if(t<.5*e.length){var r=0,i={clusterNodeProperties:{shape:"ellipse",label:"",group:"",font:{multi:!1}},clusterEdgeProperties:{label:"",font:{multi:!1},smooth:{enabled:!1}}};if(e.length>150){for(var o=e.length;e.length>150&&r<=10;){r+=1;var a=e.length;if(r%3==0?this.body.modules.clustering.clusterBridges(i):this.body.modules.clustering.clusterOutliers(i),a==e.length&&r%3!=0)return this._declusterAll(),this.body.emitter.emit("_layoutFailed"),void console.info("This network could not be positioned by this version of the improved layout algorithm. Please disable improvedLayout for better performance.")}this.body.modules.kamadaKawai.setOptions({springLength:Math.max(150,2*o)})}r>10&&console.info("The clustering didn't succeed within the amount of interations allowed, progressing with partial result."),this.body.modules.kamadaKawai.solve(e,this.body.edgeIndices,!0),this._shiftToCenter();for(var s=0;s<e.length;s++){var l=this.body.nodes[e[s]];!1===l.predefinedPosition&&(l.x+=70*(.5-this.seededRandom()),l.y+=70*(.5-this.seededRandom()))}this._declusterAll(),this.body.emitter.emit("_repositionBezierNodes")}}}},{key:"_shiftToCenter",value:function(){for(var e=c.getRangeCore(this.body.nodes,this.body.nodeIndices),t=c.findCenter(e),n=0;n<this.body.nodeIndices.length;n++){var r=this.body.nodes[this.body.nodeIndices[n]];r.x-=t.x,r.y-=t.y}}},{key:"_declusterAll",value:function(){for(var e=!0;!0===e;){e=!1;for(var t=0;t<this.body.nodeIndices.length;t++)!0===this.body.nodes[this.body.nodeIndices[t]].isCluster&&(e=!0,this.body.modules.clustering.openCluster(this.body.nodeIndices[t],{},!1));!0===e&&this.body.emitter.emit("_dataChanged")}}},{key:"getSeed",value:function(){return this.initialRandomSeed}},{key:"setupHierarchicalLayout",value:function(){if(!0===this.options.hierarchical.enabled&&this.body.nodeIndices.length>0){var e=void 0,t=void 0,n=!1,r=!1;for(t in this.lastNodeOnLevel={},this.hierarchical=new p,this.body.nodes)this.body.nodes.hasOwnProperty(t)&&(void 0!==(e=this.body.nodes[t]).options.level?(n=!0,this.hierarchical.levels[t]=e.options.level):r=!0);if(!0===r&&!0===n)throw new Error("To use the hierarchical layout, nodes require either no predefined levels or levels have to be defined for all nodes.");if(!0===r){var i=this.options.hierarchical.sortMethod;"hubsize"===i?this._determineLevelsByHubsize():"directed"===i?this._determineLevelsDirected():"custom"===i&&this._determineLevelsCustomCallback()}for(var o in this.body.nodes)this.body.nodes.hasOwnProperty(o)&&this.hierarchical.ensureLevel(o);var a=this._getDistribution();this._generateMap(),this._placeNodesByHierarchy(a),this._condenseHierarchy(),this._shiftToCenter()}}},{key:"_condenseHierarchy",value:function(){var e=this,t=!1,n={},i=function(t,n){var r=e.hierarchical.trees;for(var i in r)r.hasOwnProperty(i)&&r[i]===t&&e.direction.shift(i,n)},o=function(){for(var t=[],n=0;n<e.hierarchical.numTrees();n++)t.push(e.direction.getTreeSize(n));return t},a=function t(n,r){if(!r[n.id]&&(r[n.id]=!0,e.hierarchical.childrenReference[n.id])){var i=e.hierarchical.childrenReference[n.id];if(i.length>0)for(var o=0;o<i.length;o++)t(e.body.nodes[i[o]],r)}},s=function(t){var n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:1e9,i=1e9,o=1e9,a=1e9,s=-1e9;for(var l in t)if(t.hasOwnProperty(l)){var u=e.body.nodes[l],c=e.hierarchical.levels[u.id],d=e.direction.getPosition(u),h=e._getSpaceAroundNode(u,t),f=(0,r.default)(h,2),p=f[0],g=f[1];i=Math.min(p,i),o=Math.min(g,o),c<=n&&(a=Math.min(d,a),s=Math.max(d,s))}return[a,s,i,o]},l=function(t,n){var r=e.hierarchical.getMaxLevel(t.id),i=e.hierarchical.getMaxLevel(n.id);return Math.min(r,i)},u=function(t,n,r){for(var i=e.hierarchical,o=0;o<n.length;o++){var a=n[o],s=i.distributionOrdering[a];if(s.length>1)for(var l=0;l<s.length-1;l++){var u=s[l],c=s[l+1];i.hasSameParent(u,c)&&i.inSameSubNetwork(u,c)&&t(u,c,r)}}},c=function(n,r){var i=arguments.length>2&&void 0!==arguments[2]&&arguments[2],o=e.direction.getPosition(n),u=e.direction.getPosition(r),c=Math.abs(u-o),d=e.options.hierarchical.nodeSpacing;if(c>d){var h={},f={};a(n,h),a(r,f);var p=l(n,r),g=s(h,p),v=s(f,p),m=g[1],y=v[0],b=v[2],C=Math.abs(m-y);if(C>d){var A=m-y+d;A<-b+d&&(A=-b+d),A<0&&(e._shiftBlock(r.id,A),t=!0,!0===i&&e._centerParent(r))}}},d=function(i,o){for(var l=o.id,u=o.edges,c=e.hierarchical.levels[o.id],d=e.options.hierarchical.levelSeparation*e.options.hierarchical.levelSeparation,h={},f=[],p=0;p<u.length;p++){var g=u[p];if(g.toId!=g.fromId){var v=g.toId==l?g.from:g.to;h[u[p].id]=v,e.hierarchical.levels[v.id]<c&&f.push(g)}}var m=function(t,n){for(var r=0,i=0;i<n.length;i++)if(void 0!==h[n[i].id]){var o=e.direction.getPosition(h[n[i].id])-t;r+=o/Math.sqrt(o*o+d)}return r},y=function(t,n){for(var r=0,i=0;i<n.length;i++)if(void 0!==h[n[i].id]){var o=e.direction.getPosition(h[n[i].id])-t;r-=d*Math.pow(o*o+d,-1.5)}return r},b=function(t,n){for(var r=e.direction.getPosition(o),i={},a=0;a<t;a++){var s=m(r,n),l=y(r,n);if(void 0!==i[r-=Math.max(-40,Math.min(40,Math.round(s/l)))])break;i[r]=a}return r},C=b(i,f);!function(r){var i=e.direction.getPosition(o);if(void 0===n[o.id]){var l={};a(o,l),n[o.id]=l}var u=s(n[o.id]),c=u[2],d=u[3],h=r-i,f=0;h>0?f=Math.min(h,d-e.options.hierarchical.nodeSpacing):h<0&&(f=-Math.min(-h,c-e.options.hierarchical.nodeSpacing)),0!=f&&(e._shiftBlock(o.id,f),t=!0)}(C),function(n){var i=e.direction.getPosition(o),a=e._getSpaceAroundNode(o),s=(0,r.default)(a,2),l=s[0],u=s[1],c=n-i,d=i;c>0?d=Math.min(i+(u-e.options.hierarchical.nodeSpacing),n):c<0&&(d=Math.max(i-(l-e.options.hierarchical.nodeSpacing),n)),d!==i&&(e.direction.setPosition(o,d),t=!0)}(C=b(i,u))};!0===this.options.hierarchical.blockShifting&&(function(n){var r=e.hierarchical.getLevels();r=r.reverse();for(var i=0;i<n&&(t=!1,u(c,r,!0),!0===t);i++);}(5),function(){for(var t in e.body.nodes)e.body.nodes.hasOwnProperty(t)&&e._centerParent(e.body.nodes[t])}()),!0===this.options.hierarchical.edgeMinimization&&function(n){var r=e.hierarchical.getLevels();r=r.reverse();for(var i=0;i<n;i++){t=!1;for(var o=0;o<r.length;o++)for(var a=r[o],s=e.hierarchical.distributionOrdering[a],l=0;l<s.length;l++)d(1e3,s[l]);if(!0!==t)break}}(20),!0===this.options.hierarchical.parentCentralization&&function(){var t=e.hierarchical.getLevels();t=t.reverse();for(var n=0;n<t.length;n++)for(var r=t[n],i=e.hierarchical.distributionOrdering[r],o=0;o<i.length;o++)e._centerParent(i[o])}(),function(){for(var t=o(),n=0,r=0;r<t.length-1;r++)n+=t[r].max-t[r+1].min+e.options.hierarchical.treeSpacing,i(r+1,n)}()}},{key:"_getSpaceAroundNode",value:function(e,t){var n=!0;void 0===t&&(n=!1);var r=this.hierarchical.levels[e.id];if(void 0!==r){var i=this.hierarchical.distributionIndex[e.id],o=this.direction.getPosition(e),a=this.hierarchical.distributionOrdering[r],s=1e9,l=1e9;if(0!==i){var u=a[i-1];(!0===n&&void 0===t[u.id]||!1===n)&&(s=o-this.direction.getPosition(u))}if(i!=a.length-1){var c=a[i+1];if(!0===n&&void 0===t[c.id]||!1===n){var d=this.direction.getPosition(c);l=Math.min(l,d-o)}}return[s,l]}return[0,0]}},{key:"_centerParent",value:function(e){if(this.hierarchical.parentReference[e.id])for(var t=this.hierarchical.parentReference[e.id],n=0;n<t.length;n++){var i=t[n],o=this.body.nodes[i],a=this.hierarchical.childrenReference[i];if(void 0!==a){var s=this._getCenterPosition(a),l=this.direction.getPosition(o),u=this._getSpaceAroundNode(o),c=(0,r.default)(u,2),d=c[0],h=c[1],f=l-s;(f<0&&Math.abs(f)<h-this.options.hierarchical.nodeSpacing||f>0&&Math.abs(f)<d-this.options.hierarchical.nodeSpacing)&&this.direction.setPosition(o,s)}}}},{key:"_placeNodesByHierarchy",value:function(e){for(var t in this.positionedNodes={},e)if(e.hasOwnProperty(t)){var n=(0,o.default)(e[t]);n=this._indexArrayToNodes(n),this.direction.sort(n);for(var r=0,i=0;i<n.length;i++){var a=n[i];if(void 0===this.positionedNodes[a.id]){var s=this.options.hierarchical.nodeSpacing,l=s*r;r>0&&(l=this.direction.getPosition(n[i-1])+s),this.direction.setPosition(a,l,t),this._validatePositionAndContinue(a,t,l),r++}}}}},{key:"_placeBranchNodes",value:function(e,t){var n=this.hierarchical.childrenReference[e];if(void 0!==n){for(var r=[],i=0;i<n.length;i++)r.push(this.body.nodes[n[i]]);this.direction.sort(r);for(var o=0;o<r.length;o++){var a=r[o],s=this.hierarchical.levels[a.id];if(!(s>t&&void 0===this.positionedNodes[a.id]))return;var l=this.options.hierarchical.nodeSpacing,u=void 0;u=0===o?this.direction.getPosition(this.body.nodes[e]):this.direction.getPosition(r[o-1])+l,this.direction.setPosition(a,u,s),this._validatePositionAndContinue(a,s,u)}var c=this._getCenterPosition(r);this.direction.setPosition(this.body.nodes[e],c,t)}}},{key:"_validatePositionAndContinue",value:function(e,t,n){if(this.hierarchical.isTree){if(void 0!==this.lastNodeOnLevel[t]){var r=this.direction.getPosition(this.body.nodes[this.lastNodeOnLevel[t]]);if(n-r<this.options.hierarchical.nodeSpacing){var i=r+this.options.hierarchical.nodeSpacing-n,o=this._findCommonParent(this.lastNodeOnLevel[t],e.id);this._shiftBlock(o.withChild,i)}}this.lastNodeOnLevel[t]=e.id,this.positionedNodes[e.id]=!0,this._placeBranchNodes(e.id,t)}}},{key:"_indexArrayToNodes",value:function(e){for(var t=[],n=0;n<e.length;n++)t.push(this.body.nodes[e[n]]);return t}},{key:"_getDistribution",value:function(){var e={},t=void 0,n=void 0;for(t in this.body.nodes)if(this.body.nodes.hasOwnProperty(t)){n=this.body.nodes[t];var r=void 0===this.hierarchical.levels[t]?0:this.hierarchical.levels[t];this.direction.fix(n,r),void 0===e[r]&&(e[r]={}),e[r][t]=n}return e}},{key:"_getActiveEdges",value:function(e){var t=this,n=[];return u.forEach(e.edges,(function(e){-1!==t.body.edgeIndices.indexOf(e.id)&&n.push(e)})),n}},{key:"_getHubSizes",value:function(){var e=this,t={},n=this.body.nodeIndices;u.forEach(n,(function(n){var r=e.body.nodes[n],i=e._getActiveEdges(r).length;t[i]=!0}));var r=[];return u.forEach(t,(function(e){r.push(Number(e))})),r.sort((function(e,t){return t-e})),r}},{key:"_determineLevelsByHubsize",value:function(){for(var e=this,t=function(t,n){e.hierarchical.levelDownstream(t,n)},n=this._getHubSizes(),r=function(r){var i=n[r];if(0===i)return"break";u.forEach(e.body.nodeIndices,(function(n){var r=e.body.nodes[n];i===e._getActiveEdges(r).length&&e._crawlNetwork(t,n)}))},i=0;i<n.length&&"break"!==r(i);++i);}},{key:"_determineLevelsCustomCallback",value:function(){var e=this;this._crawlNetwork((function(t,n,r){var i=e.hierarchical.levels[t.id];void 0===i&&(i=e.hierarchical.levels[t.id]=1e5);var o=(c.cloneOptions(t,"node"),c.cloneOptions(n,"node"),void c.cloneOptions(r,"edge"));e.hierarchical.levels[n.id]=i+o})),this.hierarchical.setMinLevelToZero(this.body.nodes)}},{key:"_determineLevelsDirected",value:function(){var e=this;this._crawlNetwork((function(t,n,r){var i=e.hierarchical.levels[t.id],o=e.hierarchical.levels[n.id];(function(t){return u.forEach(e.body.edges,(function(e){if(e.toId===t.fromId&&e.fromId===t.toId)return!0})),!1})(r)&&void 0!==i&&void 0!==o||(void 0===i&&(i=e.hierarchical.levels[t.id]=1e4),r.toId==n.id?e.hierarchical.levels[n.id]=i+1:e.hierarchical.levels[n.id]=i-1)})),this.hierarchical.setMinLevelToZero(this.body.nodes)}},{key:"_generateMap",value:function(){var e=this;this._crawlNetwork((function(t,n){e.hierarchical.levels[n.id]>e.hierarchical.levels[t.id]&&e.hierarchical.addRelation(t.id,n.id)})),this.hierarchical.checkIfTree()}},{key:"_crawlNetwork",value:function(){var e=this,t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:function(){},n=arguments[1],r={},i=function n(i,o){if(void 0===r[i.id]){e.hierarchical.setTreeIndex(i,o),r[i.id]=!0;for(var a=void 0,s=e._getActiveEdges(i),l=0;l<s.length;l++){var u=s[l];!0===u.connected&&(a=u.toId==i.id?u.from:u.to,i.id!=a.id&&(t(i,a,u),n(a,o)))}}};if(void 0===n)for(var o=0,a=0;a<this.body.nodeIndices.length;a++){var s=this.body.nodeIndices[a];if(void 0===r[s]){var l=this.body.nodes[s];i(l,o),o+=1}}else{var u=this.body.nodes[n];if(void 0===u)return void console.error("Node not found:",n);i(u)}}},{key:"_shiftBlock",value:function(e,t){var n=this,r={};!function e(i){if(!r[i]){r[i]=!0,n.direction.shift(i,t);var o=n.hierarchical.childrenReference[i];if(void 0!==o)for(var a=0;a<o.length;a++)e(o[a])}}(e)}},{key:"_findCommonParent",value:function(e,t){var n=this,r={};return function e(t,r){var i=n.hierarchical.parentReference[r];if(void 0!==i)for(var o=0;o<i.length;o++){var a=i[o];t[a]=!0,e(t,a)}}(r,e),function e(t,r){var i=n.hierarchical.parentReference[r];if(void 0!==i)for(var o=0;o<i.length;o++){var a=i[o];if(void 0!==t[a])return{foundParent:a,withChild:r};var s=e(t,a);if(null!==s.foundParent)return s}return{foundParent:null,withChild:r}}(r,t)}},{key:"setDirectionStrategy",value:function(){var e="UD"===this.options.hierarchical.direction||"DU"===this.options.hierarchical.direction;this.direction=e?new f(this):new h(this)}},{key:"_getCenterPosition",value:function(e){for(var t=1e9,n=-1e9,r=0;r<e.length;r++){var i=void 0;if(void 0!==e[r].id)i=e[r];else{var o=e[r];i=this.body.nodes[o]}var a=this.direction.getPosition(i);t=Math.min(t,a),n=Math.max(n,a)}return.5*(t+n)}}]),e}();t.default=g},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0}),t.VerticalStrategy=t.HorizontalStrategy=void 0;var r=l(n(3)),i=l(n(4)),o=l(n(5)),a=l(n(0)),s=l(n(1));function l(e){return e&&e.__esModule?e:{default:e}}var u=function(){function e(){(0,a.default)(this,e)}return(0,s.default)(e,[{key:"abstract",value:function(){throw new Error("Can't instantiate abstract class!")}},{key:"fake_use",value:function(){}},{key:"curveType",value:function(){return this.abstract()}},{key:"getPosition",value:function(e){return this.fake_use(e),this.abstract()}},{key:"setPosition",value:function(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:void 0;this.fake_use(e,t,n),this.abstract()}},{key:"getTreeSize",value:function(e){return this.fake_use(e),this.abstract()}},{key:"sort",value:function(e){this.fake_use(e),this.abstract()}},{key:"fix",value:function(e,t){this.fake_use(e,t),this.abstract()}},{key:"shift",value:function(e,t){this.fake_use(e,t),this.abstract()}}]),e}(),c=function(e){function t(e){(0,a.default)(this,t);var n=(0,i.default)(this,(t.__proto__||(0,r.default)(t)).call(this));return n.layout=e,n}return(0,o.default)(t,e),(0,s.default)(t,[{key:"curveType",value:function(){return"horizontal"}},{key:"getPosition",value:function(e){return e.x}},{key:"setPosition",value:function(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:void 0;void 0!==n&&this.layout.hierarchical.addToOrdering(e,n),e.x=t}},{key:"getTreeSize",value:function(e){var t=this.layout.hierarchical.getTreeSize(this.layout.body.nodes,e);return{min:t.min_x,max:t.max_x}}},{key:"sort",value:function(e){e.sort((function(e,t){return void 0===e.x||void 0===t.x?0:e.x-t.x}))}},{key:"fix",value:function(e,t){e.y=this.layout.options.hierarchical.levelSeparation*t,e.options.fixed.y=!0}},{key:"shift",value:function(e,t){this.layout.body.nodes[e].x+=t}}]),t}(u),d=function(e){function t(e){(0,a.default)(this,t);var n=(0,i.default)(this,(t.__proto__||(0,r.default)(t)).call(this));return n.layout=e,n}return(0,o.default)(t,e),(0,s.default)(t,[{key:"curveType",value:function(){return"vertical"}},{key:"getPosition",value:function(e){return e.y}},{key:"setPosition",value:function(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:void 0;void 0!==n&&this.layout.hierarchical.addToOrdering(e,n),e.y=t}},{key:"getTreeSize",value:function(e){var t=this.layout.hierarchical.getTreeSize(this.layout.body.nodes,e);return{min:t.min_y,max:t.max_y}}},{key:"sort",value:function(e){e.sort((function(e,t){return void 0===e.y||void 0===t.y?0:e.y-t.y}))}},{key:"fix",value:function(e,t){e.x=this.layout.options.hierarchical.levelSeparation*t,e.options.fixed.x=!0}},{key:"shift",value:function(e,t){this.layout.body.nodes[e].y+=t}}]),t}(u);t.HorizontalStrategy=d,t.VerticalStrategy=c},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=l(n(8)),i=l(n(19)),o=l(n(6)),a=l(n(0)),s=l(n(1));function l(e){return e&&e.__esModule?e:{default:e}}var u=n(2),c=n(10),d=n(37),h=function(){function e(t,n,r){var i=this;(0,a.default)(this,e),this.body=t,this.canvas=n,this.selectionHandler=r,this.editMode=!1,this.manipulationDiv=void 0,this.editModeDiv=void 0,this.closeDiv=void 0,this.manipulationHammers=[],this.temporaryUIFunctions={},this.temporaryEventFunctions=[],this.touchTime=0,this.temporaryIds={nodes:[],edges:[]},this.guiEnabled=!1,this.inMode=!1,this.selectedControlNode=void 0,this.options={},this.defaultOptions={enabled:!1,initiallyActive:!1,addNode:!0,addEdge:!0,editNode:void 0,editEdge:!0,deleteNode:!0,deleteEdge:!0,controlNodeStyle:{shape:"dot",size:6,color:{background:"#ff0000",border:"#3c3c3c",highlight:{background:"#07f968",border:"#3c3c3c"}},borderWidth:2,borderWidthSelected:2}},u.extend(this.options,this.defaultOptions),this.body.emitter.on("destroy",(function(){i._clean()})),this.body.emitter.on("_dataChanged",this._restore.bind(this)),this.body.emitter.on("_resetData",this._restore.bind(this))}return(0,s.default)(e,[{key:"_restore",value:function(){!1!==this.inMode&&(!0===this.options.initiallyActive?this.enableEditMode():this.disableEditMode())}},{key:"setOptions",value:function(e,t,n){void 0!==t&&(void 0!==t.locale?this.options.locale=t.locale:this.options.locale=n.locale,void 0!==t.locales?this.options.locales=t.locales:this.options.locales=n.locales),void 0!==e&&("boolean"==typeof e?this.options.enabled=e:(this.options.enabled=!0,u.deepExtend(this.options,e)),!0===this.options.initiallyActive&&(this.editMode=!0),this._setup())}},{key:"toggleEditMode",value:function(){!0===this.editMode?this.disableEditMode():this.enableEditMode()}},{key:"enableEditMode",value:function(){this.editMode=!0,this._clean(),!0===this.guiEnabled&&(this.manipulationDiv.style.display="block",this.closeDiv.style.display="block",this.editModeDiv.style.display="none",this.showManipulatorToolbar())}},{key:"disableEditMode",value:function(){this.editMode=!1,this._clean(),!0===this.guiEnabled&&(this.manipulationDiv.style.display="none",this.closeDiv.style.display="none",this.editModeDiv.style.display="block",this._createEditButton())}},{key:"showManipulatorToolbar",value:function(){if(this._clean(),this.manipulationDOM={},!0===this.guiEnabled){this.editMode=!0,this.manipulationDiv.style.display="block",this.closeDiv.style.display="block";var e=this.selectionHandler._getSelectedNodeCount(),t=this.selectionHandler._getSelectedEdgeCount(),n=e+t,r=this.options.locales[this.options.locale],i=!1;!1!==this.options.addNode&&(this._createAddNodeButton(r),i=!0),!1!==this.options.addEdge&&(!0===i?this._createSeperator(1):i=!0,this._createAddEdgeButton(r)),1===e&&"function"==typeof this.options.editNode?(!0===i?this._createSeperator(2):i=!0,this._createEditNodeButton(r)):1===t&&0===e&&!1!==this.options.editEdge&&(!0===i?this._createSeperator(3):i=!0,this._createEditEdgeButton(r)),0!==n&&(e>0&&!1!==this.options.deleteNode||0===e&&!1!==this.options.deleteEdge)&&(!0===i&&this._createSeperator(4),this._createDeleteButton(r)),this._bindHammerToDiv(this.closeDiv,this.toggleEditMode.bind(this)),this._temporaryBindEvent("select",this.showManipulatorToolbar.bind(this))}this.body.emitter.emit("_redraw")}},{key:"addNodeMode",value:function(){if(!0!==this.editMode&&this.enableEditMode(),this._clean(),this.inMode="addNode",!0===this.guiEnabled){var e=this.options.locales[this.options.locale];this.manipulationDOM={},this._createBackButton(e),this._createSeperator(),this._createDescription(e.addDescription||this.options.locales.en.addDescription),this._bindHammerToDiv(this.closeDiv,this.toggleEditMode.bind(this))}this._temporaryBindEvent("click",this._performAddNode.bind(this))}},{key:"editNode",value:function(){var e=this;!0!==this.editMode&&this.enableEditMode(),this._clean();var t=this.selectionHandler._getSelectedNode();if(void 0!==t){if(this.inMode="editNode","function"!=typeof this.options.editNode)throw new Error("No function has been configured to handle the editing of nodes.");if(!0!==t.isCluster){var n=u.deepExtend({},t.options,!1);if(n.x=t.x,n.y=t.y,2!==this.options.editNode.length)throw new Error("The function for edit does not support two arguments (data, callback)");this.options.editNode(n,(function(t){null!=t&&"editNode"===e.inMode&&e.body.data.nodes.getDataSet().update(t),e.showManipulatorToolbar()}))}else alert(this.options.locales[this.options.locale].editClusterError||this.options.locales.en.editClusterError)}else this.showManipulatorToolbar()}},{key:"addEdgeMode",value:function(){if(!0!==this.editMode&&this.enableEditMode(),this._clean(),this.inMode="addEdge",!0===this.guiEnabled){var e=this.options.locales[this.options.locale];this.manipulationDOM={},this._createBackButton(e),this._createSeperator(),this._createDescription(e.edgeDescription||this.options.locales.en.edgeDescription),this._bindHammerToDiv(this.closeDiv,this.toggleEditMode.bind(this))}this._temporaryBindUI("onTouch",this._handleConnect.bind(this)),this._temporaryBindUI("onDragEnd",this._finishConnect.bind(this)),this._temporaryBindUI("onDrag",this._dragControlNode.bind(this)),this._temporaryBindUI("onRelease",this._finishConnect.bind(this)),this._temporaryBindUI("onDragStart",this._dragStartEdge.bind(this)),this._temporaryBindUI("onHold",(function(){}))}},{key:"editEdgeMode",value:function(){if(!0!==this.editMode&&this.enableEditMode(),this._clean(),this.inMode="editEdge","object"!==(0,o.default)(this.options.editEdge)||"function"!=typeof this.options.editEdge.editWithoutDrag||(this.edgeBeingEditedId=this.selectionHandler.getSelectedEdges()[0],void 0===this.edgeBeingEditedId)){if(!0===this.guiEnabled){var e=this.options.locales[this.options.locale];this.manipulationDOM={},this._createBackButton(e),this._createSeperator(),this._createDescription(e.editEdgeDescription||this.options.locales.en.editEdgeDescription),this._bindHammerToDiv(this.closeDiv,this.toggleEditMode.bind(this))}if(this.edgeBeingEditedId=this.selectionHandler.getSelectedEdges()[0],void 0!==this.edgeBeingEditedId){var t=this.body.edges[this.edgeBeingEditedId],n=this._getNewTargetNode(t.from.x,t.from.y),r=this._getNewTargetNode(t.to.x,t.to.y);this.temporaryIds.nodes.push(n.id),this.temporaryIds.nodes.push(r.id),this.body.nodes[n.id]=n,this.body.nodeIndices.push(n.id),this.body.nodes[r.id]=r,this.body.nodeIndices.push(r.id),this._temporaryBindUI("onTouch",this._controlNodeTouch.bind(this)),this._temporaryBindUI("onTap",(function(){})),this._temporaryBindUI("onHold",(function(){})),this._temporaryBindUI("onDragStart",this._controlNodeDragStart.bind(this)),this._temporaryBindUI("onDrag",this._controlNodeDrag.bind(this)),this._temporaryBindUI("onDragEnd",this._controlNodeDragEnd.bind(this)),this._temporaryBindUI("onMouseMove",(function(){})),this._temporaryBindEvent("beforeDrawing",(function(e){var i=t.edgeType.findBorderPositions(e);!1===n.selected&&(n.x=i.from.x,n.y=i.from.y),!1===r.selected&&(r.x=i.to.x,r.y=i.to.y)})),this.body.emitter.emit("_redraw")}else this.showManipulatorToolbar()}else{var i=this.body.edges[this.edgeBeingEditedId];this._performEditEdge(i.from,i.to)}}},{key:"deleteSelected",value:function(){var e=this;!0!==this.editMode&&this.enableEditMode(),this._clean(),this.inMode="delete";var t=this.selectionHandler.getSelectedNodes(),n=this.selectionHandler.getSelectedEdges(),r=void 0;if(t.length>0){for(var i=0;i<t.length;i++)if(!0===this.body.nodes[t[i]].isCluster)return void alert(this.options.locales[this.options.locale].deleteClusterError||this.options.locales.en.deleteClusterError);"function"==typeof this.options.deleteNode&&(r=this.options.deleteNode)}else n.length>0&&"function"==typeof this.options.deleteEdge&&(r=this.options.deleteEdge);if("function"==typeof r){var o={nodes:t,edges:n};if(2!==r.length)throw new Error("The function for delete does not support two arguments (data, callback)");r(o,(function(t){null!=t&&"delete"===e.inMode?(e.body.data.edges.getDataSet().remove(t.edges),e.body.data.nodes.getDataSet().remove(t.nodes),e.body.emitter.emit("startSimulation"),e.showManipulatorToolbar()):(e.body.emitter.emit("startSimulation"),e.showManipulatorToolbar())}))}else this.body.data.edges.getDataSet().remove(n),this.body.data.nodes.getDataSet().remove(t),this.body.emitter.emit("startSimulation"),this.showManipulatorToolbar()}},{key:"_setup",value:function(){!0===this.options.enabled?(this.guiEnabled=!0,this._createWrappers(),!1===this.editMode?this._createEditButton():this.showManipulatorToolbar()):(this._removeManipulationDOM(),this.guiEnabled=!1)}},{key:"_createWrappers",value:function(){void 0===this.manipulationDiv&&(this.manipulationDiv=document.createElement("div"),this.manipulationDiv.className="vis-manipulation",!0===this.editMode?this.manipulationDiv.style.display="block":this.manipulationDiv.style.display="none",this.canvas.frame.appendChild(this.manipulationDiv)),void 0===this.editModeDiv&&(this.editModeDiv=document.createElement("div"),this.editModeDiv.className="vis-edit-mode",!0===this.editMode?this.editModeDiv.style.display="none":this.editModeDiv.style.display="block",this.canvas.frame.appendChild(this.editModeDiv)),void 0===this.closeDiv&&(this.closeDiv=document.createElement("div"),this.closeDiv.className="vis-close",this.closeDiv.style.display=this.manipulationDiv.style.display,this.canvas.frame.appendChild(this.closeDiv))}},{key:"_getNewTargetNode",value:function(e,t){var n=u.deepExtend({},this.options.controlNodeStyle);n.id="targetNode"+u.randomUUID(),n.hidden=!1,n.physics=!1,n.x=e,n.y=t;var r=this.body.functions.createNode(n);return r.shape.boundingBox={left:e,right:e,top:t,bottom:t},r}},{key:"_createEditButton",value:function(){this._clean(),this.manipulationDOM={},u.recursiveDOMDelete(this.editModeDiv);var e=this.options.locales[this.options.locale],t=this._createButton("editMode","vis-button vis-edit vis-edit-mode",e.edit||this.options.locales.en.edit);this.editModeDiv.appendChild(t),this._bindHammerToDiv(t,this.toggleEditMode.bind(this))}},{key:"_clean",value:function(){this.inMode=!1,!0===this.guiEnabled&&(u.recursiveDOMDelete(this.editModeDiv),u.recursiveDOMDelete(this.manipulationDiv),this._cleanManipulatorHammers()),this._cleanupTemporaryNodesAndEdges(),this._unbindTemporaryUIs(),this._unbindTemporaryEvents(),this.body.emitter.emit("restorePhysics")}},{key:"_cleanManipulatorHammers",value:function(){if(0!=this.manipulationHammers.length){for(var e=0;e<this.manipulationHammers.length;e++)this.manipulationHammers[e].destroy();this.manipulationHammers=[]}}},{key:"_removeManipulationDOM",value:function(){this._clean(),u.recursiveDOMDelete(this.manipulationDiv),u.recursiveDOMDelete(this.editModeDiv),u.recursiveDOMDelete(this.closeDiv),this.manipulationDiv&&this.canvas.frame.removeChild(this.manipulationDiv),this.editModeDiv&&this.canvas.frame.removeChild(this.editModeDiv),this.closeDiv&&this.canvas.frame.removeChild(this.closeDiv),this.manipulationDiv=void 0,this.editModeDiv=void 0,this.closeDiv=void 0}},{key:"_createSeperator",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:1;this.manipulationDOM["seperatorLineDiv"+e]=document.createElement("div"),this.manipulationDOM["seperatorLineDiv"+e].className="vis-separator-line",this.manipulationDiv.appendChild(this.manipulationDOM["seperatorLineDiv"+e])}},{key:"_createAddNodeButton",value:function(e){var t=this._createButton("addNode","vis-button vis-add",e.addNode||this.options.locales.en.addNode);this.manipulationDiv.appendChild(t),this._bindHammerToDiv(t,this.addNodeMode.bind(this))}},{key:"_createAddEdgeButton",value:function(e){var t=this._createButton("addEdge","vis-button vis-connect",e.addEdge||this.options.locales.en.addEdge);this.manipulationDiv.appendChild(t),this._bindHammerToDiv(t,this.addEdgeMode.bind(this))}},{key:"_createEditNodeButton",value:function(e){var t=this._createButton("editNode","vis-button vis-edit",e.editNode||this.options.locales.en.editNode);this.manipulationDiv.appendChild(t),this._bindHammerToDiv(t,this.editNode.bind(this))}},{key:"_createEditEdgeButton",value:function(e){var t=this._createButton("editEdge","vis-button vis-edit",e.editEdge||this.options.locales.en.editEdge);this.manipulationDiv.appendChild(t),this._bindHammerToDiv(t,this.editEdgeMode.bind(this))}},{key:"_createDeleteButton",value:function(e){var t;t=this.options.rtl?"vis-button vis-delete-rtl":"vis-button vis-delete";var n=this._createButton("delete",t,e.del||this.options.locales.en.del);this.manipulationDiv.appendChild(n),this._bindHammerToDiv(n,this.deleteSelected.bind(this))}},{key:"_createBackButton",value:function(e){var t=this._createButton("back","vis-button vis-back",e.back||this.options.locales.en.back);this.manipulationDiv.appendChild(t),this._bindHammerToDiv(t,this.showManipulatorToolbar.bind(this))}},{key:"_createButton",value:function(e,t,n){var r=arguments.length>3&&void 0!==arguments[3]?arguments[3]:"vis-label";return this.manipulationDOM[e+"Div"]=document.createElement("div"),this.manipulationDOM[e+"Div"].className=t,this.manipulationDOM[e+"Label"]=document.createElement("div"),this.manipulationDOM[e+"Label"].className=r,this.manipulationDOM[e+"Label"].innerHTML=n,this.manipulationDOM[e+"Div"].appendChild(this.manipulationDOM[e+"Label"]),this.manipulationDOM[e+"Div"]}},{key:"_createDescription",value:function(e){this.manipulationDiv.appendChild(this._createButton("description","vis-button vis-none",e))}},{key:"_temporaryBindEvent",value:function(e,t){this.temporaryEventFunctions.push({event:e,boundFunction:t}),this.body.emitter.on(e,t)}},{key:"_temporaryBindUI",value:function(e,t){if(void 0===this.body.eventListeners[e])throw new Error("This UI function does not exist. Typo? You tried: "+e+" possible are: "+(0,i.default)((0,r.default)(this.body.eventListeners)));this.temporaryUIFunctions[e]=this.body.eventListeners[e],this.body.eventListeners[e]=t}},{key:"_unbindTemporaryUIs",value:function(){for(var e in this.temporaryUIFunctions)this.temporaryUIFunctions.hasOwnProperty(e)&&(this.body.eventListeners[e]=this.temporaryUIFunctions[e],delete this.temporaryUIFunctions[e]);this.temporaryUIFunctions={}}},{key:"_unbindTemporaryEvents",value:function(){for(var e=0;e<this.temporaryEventFunctions.length;e++){var t=this.temporaryEventFunctions[e].event,n=this.temporaryEventFunctions[e].boundFunction;this.body.emitter.off(t,n)}this.temporaryEventFunctions=[]}},{key:"_bindHammerToDiv",value:function(e,t){var n=new c(e,{});d.onTouch(n,t),this.manipulationHammers.push(n)}},{key:"_cleanupTemporaryNodesAndEdges",value:function(){for(var e=0;e<this.temporaryIds.edges.length;e++){this.body.edges[this.temporaryIds.edges[e]].disconnect(),delete this.body.edges[this.temporaryIds.edges[e]];var t=this.body.edgeIndices.indexOf(this.temporaryIds.edges[e]);-1!==t&&this.body.edgeIndices.splice(t,1)}for(var n=0;n<this.temporaryIds.nodes.length;n++){delete this.body.nodes[this.temporaryIds.nodes[n]];var r=this.body.nodeIndices.indexOf(this.temporaryIds.nodes[n]);-1!==r&&this.body.nodeIndices.splice(r,1)}this.temporaryIds={nodes:[],edges:[]}}},{key:"_controlNodeTouch",value:function(e){this.selectionHandler.unselectAll(),this.lastTouch=this.body.functions.getPointer(e.center),this.lastTouch.translation=u.extend({},this.body.view.translation)}},{key:"_controlNodeDragStart",value:function(e){var t=this.lastTouch,n=this.selectionHandler._pointerToPositionObject(t),r=this.body.nodes[this.temporaryIds.nodes[0]],i=this.body.nodes[this.temporaryIds.nodes[1]],o=this.body.edges[this.edgeBeingEditedId];this.selectedControlNode=void 0;var a=r.isOverlappingWith(n),s=i.isOverlappingWith(n);!0===a?(this.selectedControlNode=r,o.edgeType.from=r):!0===s&&(this.selectedControlNode=i,o.edgeType.to=i),void 0!==this.selectedControlNode&&this.selectionHandler.selectObject(this.selectedControlNode),this.body.emitter.emit("_redraw")}},{key:"_controlNodeDrag",value:function(e){this.body.emitter.emit("disablePhysics");var t=this.body.functions.getPointer(e.center),n=this.canvas.DOMtoCanvas(t);if(void 0!==this.selectedControlNode)this.selectedControlNode.x=n.x,this.selectedControlNode.y=n.y;else{var r=t.x-this.lastTouch.x,i=t.y-this.lastTouch.y;this.body.view.translation={x:this.lastTouch.translation.x+r,y:this.lastTouch.translation.y+i}}this.body.emitter.emit("_redraw")}},{key:"_controlNodeDragEnd",value:function(e){var t=this.body.functions.getPointer(e.center),n=this.selectionHandler._pointerToPositionObject(t),r=this.body.edges[this.edgeBeingEditedId];if(void 0!==this.selectedControlNode){this.selectionHandler.unselectAll();for(var i=this.selectionHandler._getAllNodesOverlappingWith(n),o=void 0,a=i.length-1;a>=0;a--)if(i[a]!==this.selectedControlNode.id){o=this.body.nodes[i[a]];break}if(void 0!==o&&void 0!==this.selectedControlNode)if(!0===o.isCluster)alert(this.options.locales[this.options.locale].createEdgeError||this.options.locales.en.createEdgeError);else{var s=this.body.nodes[this.temporaryIds.nodes[0]];this.selectedControlNode.id===s.id?this._performEditEdge(o.id,r.to.id):this._performEditEdge(r.from.id,o.id)}else r.updateEdgeType(),this.body.emitter.emit("restorePhysics");this.body.emitter.emit("_redraw")}}},{key:"_handleConnect",value:function(e){if((new Date).valueOf()-this.touchTime>100){this.lastTouch=this.body.functions.getPointer(e.center),this.lastTouch.translation=u.extend({},this.body.view.translation);var t=this.lastTouch,n=this.selectionHandler.getNodeAt(t);if(void 0!==n)if(!0===n.isCluster)alert(this.options.locales[this.options.locale].createEdgeError||this.options.locales.en.createEdgeError);else{var r=this._getNewTargetNode(n.x,n.y);this.body.nodes[r.id]=r,this.body.nodeIndices.push(r.id);var i=this.body.functions.createEdge({id:"connectionEdge"+u.randomUUID(),from:n.id,to:r.id,physics:!1,smooth:{enabled:!0,type:"continuous",roundness:.5}});this.body.edges[i.id]=i,this.body.edgeIndices.push(i.id),this.temporaryIds.nodes.push(r.id),this.temporaryIds.edges.push(i.id)}this.touchTime=(new Date).valueOf()}}},{key:"_dragControlNode",value:function(e){var t=this.body.functions.getPointer(e.center);if(void 0!==this.temporaryIds.nodes[0]){var n=this.body.nodes[this.temporaryIds.nodes[0]];n.x=this.canvas._XconvertDOMtoCanvas(t.x),n.y=this.canvas._YconvertDOMtoCanvas(t.y),this.body.emitter.emit("_redraw")}else{var r=t.x-this.lastTouch.x,i=t.y-this.lastTouch.y;this.body.view.translation={x:this.lastTouch.translation.x+r,y:this.lastTouch.translation.y+i}}}},{key:"_finishConnect",value:function(e){var t=this.body.functions.getPointer(e.center),n=this.selectionHandler._pointerToPositionObject(t),r=void 0;void 0!==this.temporaryIds.edges[0]&&(r=this.body.edges[this.temporaryIds.edges[0]].fromId);for(var i=this.selectionHandler._getAllNodesOverlappingWith(n),o=void 0,a=i.length-1;a>=0;a--)if(-1===this.temporaryIds.nodes.indexOf(i[a])){o=this.body.nodes[i[a]];break}this._cleanupTemporaryNodesAndEdges(),void 0!==o&&(!0===o.isCluster?alert(this.options.locales[this.options.locale].createEdgeError||this.options.locales.en.createEdgeError):void 0!==this.body.nodes[r]&&void 0!==this.body.nodes[o.id]&&this._performAddEdge(r,o.id)),this.body.emitter.emit("_redraw")}},{key:"_dragStartEdge",value:function(e){var t=this.lastTouch;this.selectionHandler._generateClickEvent("dragStart",e,t,void 0,!0)}},{key:"_performAddNode",value:function(e){var t=this,n={id:u.randomUUID(),x:e.pointer.canvas.x,y:e.pointer.canvas.y,label:"new"};if("function"==typeof this.options.addNode){if(2!==this.options.addNode.length)throw this.showManipulatorToolbar(),new Error("The function for add does not support two arguments (data,callback)");this.options.addNode(n,(function(e){null!=e&&"addNode"===t.inMode&&(t.body.data.nodes.getDataSet().add(e),t.showManipulatorToolbar())}))}else this.body.data.nodes.getDataSet().add(n),this.showManipulatorToolbar()}},{key:"_performAddEdge",value:function(e,t){var n=this,r={from:e,to:t};if("function"==typeof this.options.addEdge){if(2!==this.options.addEdge.length)throw new Error("The function for connect does not support two arguments (data,callback)");this.options.addEdge(r,(function(e){null!=e&&"addEdge"===n.inMode&&(n.body.data.edges.getDataSet().add(e),n.selectionHandler.unselectAll(),n.showManipulatorToolbar())}))}else this.body.data.edges.getDataSet().add(r),this.selectionHandler.unselectAll(),this.showManipulatorToolbar()}},{key:"_performEditEdge",value:function(e,t){var n=this,r={id:this.edgeBeingEditedId,from:e,to:t,label:this.body.data.edges._data[this.edgeBeingEditedId].label},i=this.options.editEdge;if("object"===(void 0===i?"undefined":(0,o.default)(i))&&(i=i.editWithoutDrag),"function"==typeof i){if(2!==i.length)throw new Error("The function for edit does not support two arguments (data, callback)");i(r,(function(e){null==e||"editEdge"!==n.inMode?(n.body.edges[r.id].updateEdgeType(),n.body.emitter.emit("_redraw"),n.showManipulatorToolbar()):(n.body.data.edges.getDataSet().update(e),n.selectionHandler.unselectAll(),n.showManipulatorToolbar())}))}else this.body.data.edges.getDataSet().update(r),this.selectionHandler.unselectAll(),this.showManipulatorToolbar()}}]),e}();t.default=h},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=s(n(30)),i=s(n(0)),o=s(n(1)),a=s(n(239));function s(e){return e&&e.__esModule?e:{default:e}}var l=function(){function e(t,n,r){(0,i.default)(this,e),this.body=t,this.springLength=n,this.springConstant=r,this.distanceSolver=new a.default}return(0,o.default)(e,[{key:"setOptions",value:function(e){e&&(e.springLength&&(this.springLength=e.springLength),e.springConstant&&(this.springConstant=e.springConstant))}},{key:"solve",value:function(e,t){var n=arguments.length>2&&void 0!==arguments[2]&&arguments[2],i=this.distanceSolver.getDistances(this.body,e,t);this._createL_matrix(i),this._createK_matrix(i),this._createE_matrix();for(var o=.01,a=1,s=0,l=Math.max(1e3,Math.min(10*this.body.nodeIndices.length,6e3)),u=5,c=1e9,d=0,h=0,f=0,p=0,g=0;c>o&&s<l;){s+=1;var v=this._getHighestEnergyNode(n),m=(0,r.default)(v,4);for(d=m[0],c=m[1],h=m[2],f=m[3],p=c,g=0;p>a&&g<u;){g+=1,this._moveNode(d,h,f);var y=this._getEnergy(d),b=(0,r.default)(y,3);p=b[0],h=b[1],f=b[2]}}}},{key:"_getHighestEnergyNode",value:function(e){for(var t=this.body.nodeIndices,n=this.body.nodes,i=0,o=t[0],a=0,s=0,l=0;l<t.length;l++){var u=t[l];if(!1===n[u].predefinedPosition||!0===n[u].isCluster&&!0===e||!0===n[u].options.fixed.x||!0===n[u].options.fixed.y){var c=this._getEnergy(u),d=(0,r.default)(c,3),h=d[0],f=d[1],p=d[2];i<h&&(i=h,o=u,a=f,s=p)}}return[o,i,a,s]}},{key:"_getEnergy",value:function(e){var t=(0,r.default)(this.E_sums[e],2),n=t[0],i=t[1];return[Math.sqrt(Math.pow(n,2)+Math.pow(i,2)),n,i]}},{key:"_moveNode",value:function(e,t,n){for(var r=this.body.nodeIndices,i=this.body.nodes,o=0,a=0,s=0,l=i[e].x,u=i[e].y,c=this.K_matrix[e],d=this.L_matrix[e],h=0;h<r.length;h++){var f=r[h];if(f!==e){var p=i[f].x,g=i[f].y,v=c[f],m=d[f],y=1/Math.pow(Math.pow(l-p,2)+Math.pow(u-g,2),1.5);o+=v*(1-m*Math.pow(u-g,2)*y),a+=v*(m*(l-p)*(u-g)*y),s+=v*(1-m*Math.pow(l-p,2)*y)}}var b=(t/o+n/a)/(a/o-s/a),C=-(a*b+t)/o;i[e].x+=C,i[e].y+=b,this._updateE_matrix(e)}},{key:"_createL_matrix",value:function(e){var t=this.body.nodeIndices,n=this.springLength;this.L_matrix=[];for(var r=0;r<t.length;r++){this.L_matrix[t[r]]={};for(var i=0;i<t.length;i++)this.L_matrix[t[r]][t[i]]=n*e[t[r]][t[i]]}}},{key:"_createK_matrix",value:function(e){var t=this.body.nodeIndices,n=this.springConstant;this.K_matrix=[];for(var r=0;r<t.length;r++){this.K_matrix[t[r]]={};for(var i=0;i<t.length;i++)this.K_matrix[t[r]][t[i]]=n*Math.pow(e[t[r]][t[i]],-2)}}},{key:"_createE_matrix",value:function(){var e=this.body.nodeIndices,t=this.body.nodes;this.E_matrix={},this.E_sums={};for(var n=0;n<e.length;n++)this.E_matrix[e[n]]=[];for(var r=0;r<e.length;r++){for(var i=e[r],o=t[i].x,a=t[i].y,s=0,l=0,u=r;u<e.length;u++){var c=e[u];if(c!==i){var d=t[c].x,h=t[c].y,f=1/Math.sqrt(Math.pow(o-d,2)+Math.pow(a-h,2));this.E_matrix[i][u]=[this.K_matrix[i][c]*(o-d-this.L_matrix[i][c]*(o-d)*f),this.K_matrix[i][c]*(a-h-this.L_matrix[i][c]*(a-h)*f)],this.E_matrix[c][r]=this.E_matrix[i][u],s+=this.E_matrix[i][u][0],l+=this.E_matrix[i][u][1]}}this.E_sums[i]=[s,l]}}},{key:"_updateE_matrix",value:function(e){for(var t=this.body.nodeIndices,n=this.body.nodes,r=this.E_matrix[e],i=this.K_matrix[e],o=this.L_matrix[e],a=n[e].x,s=n[e].y,l=0,u=0,c=0;c<t.length;c++){var d=t[c];if(d!==e){var h=r[c],f=h[0],p=h[1],g=n[d].x,v=n[d].y,m=1/Math.sqrt(Math.pow(a-g,2)+Math.pow(s-v,2)),y=i[d]*(a-g-o[d]*(a-g)*m),b=i[d]*(s-v-o[d]*(s-v)*m);r[c]=[y,b],l+=y,u+=b;var C=this.E_sums[d];C[0]+=y-f,C[1]+=b-p}}this.E_sums[e]=[l,u]}}]),e}();t.default=l},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=o(n(0)),i=o(n(1));function o(e){return e&&e.__esModule?e:{default:e}}var a=function(){function e(){(0,r.default)(this,e)}return(0,i.default)(e,[{key:"getDistances",value:function(e,t,n){for(var r={},i=e.edges,o=0;o<t.length;o++){var a={};r[t[o]]=a;for(var s=0;s<t.length;s++)a[t[s]]=o==s?0:1e9}for(var l=0;l<n.length;l++){var u=i[n[l]];!0===u.connected&&void 0!==r[u.fromId]&&void 0!==r[u.toId]&&(r[u.fromId][u.toId]=1,r[u.toId][u.fromId]=1)}for(var c=t.length,d=0;d<c;d++)for(var h=t[d],f=r[h],p=0;p<c-1;p++)for(var g=t[p],v=r[g],m=p+1;m<c;m++){var y=t[m],b=r[y],C=Math.min(v[y],v[h]+f[y]);v[y]=C,b[g]=C}return r}}]),e}();t.default=a}])},e.exports=r()},function(e,t,n){var r=n(2314);"string"==typeof r&&(r=[[e.i,r,""]]);var i={insert:"head",singleton:!1};n(147)(r,i);r.locals&&(e.exports=r.locals)},function(e,t,n){t=e.exports=n(68)(!1);var r=n(1096),i=r(n(2315)),o=r(n(2316)),a=r(n(2317)),s=r(n(2318)),l=r(n(2319)),u=r(n(2320)),c=r(n(2321)),d=r(n(2322)),h=r(n(2323)),f=r(n(2324)),p=r(n(2325)),g=r(n(2326)),v=r(n(2327));t.push([e.i,'.vis .overlay{position:absolute;top:0;left:0;width:100%;height:100%;z-index:10}.vis-active{box-shadow:0 0 10px #86d5f8}.vis [class*=span]{min-height:0;width:auto}div.vis-configuration{position:relative;display:block;float:left;font-size:12px}div.vis-configuration-wrapper{display:block;width:700px}div.vis-configuration-wrapper::after{clear:both;content:"";display:block}div.vis-configuration.vis-config-option-container{display:block;width:495px;background-color:#fff;border:2px solid #f7f8fa;border-radius:4px;margin-top:20px;left:10px;padding-left:5px}div.vis-configuration.vis-config-button{display:block;width:495px;height:25px;vertical-align:middle;line-height:25px;background-color:#f7f8fa;border:2px solid #ceced0;border-radius:4px;margin-top:20px;left:10px;padding-left:5px;cursor:pointer;margin-bottom:30px}div.vis-configuration.vis-config-button.hover{background-color:#4588e6;border:2px solid #214373;color:#fff}div.vis-configuration.vis-config-item{display:block;float:left;width:495px;height:25px;vertical-align:middle;line-height:25px}div.vis-configuration.vis-config-item.vis-config-s2{left:10px;background-color:#f7f8fa;padding-left:5px;border-radius:3px}div.vis-configuration.vis-config-item.vis-config-s3{left:20px;background-color:#e4e9f0;padding-left:5px;border-radius:3px}div.vis-configuration.vis-config-item.vis-config-s4{left:30px;background-color:#cfd8e6;padding-left:5px;border-radius:3px}div.vis-configuration.vis-config-header{font-size:18px;font-weight:700}div.vis-configuration.vis-config-label{width:120px;height:25px;line-height:25px}div.vis-configuration.vis-config-label.vis-config-s3{width:110px}div.vis-configuration.vis-config-label.vis-config-s4{width:100px}div.vis-configuration.vis-config-colorBlock{top:1px;width:30px;height:19px;border:1px solid #444;border-radius:2px;padding:0;margin:0;cursor:pointer}input.vis-configuration.vis-config-checkbox{left:-5px}input.vis-configuration.vis-config-rangeinput{position:relative;top:-5px;width:60px;padding:1px;margin:0;pointer-events:none}input.vis-configuration.vis-config-range{-webkit-appearance:none;border:0 solid #fff;background-color:rgba(0,0,0,0);width:300px;height:20px}input.vis-configuration.vis-config-range::-webkit-slider-runnable-track{width:300px;height:5px;background:#dedede;background:-moz-linear-gradient(top,#dedede 0,#c8c8c8 99%);background:-webkit-gradient(linear,left top,left bottom,color-stop(0,#dedede),color-stop(99%,#c8c8c8));background:-webkit-linear-gradient(top,#dedede 0,#c8c8c8 99%);background:-o-linear-gradient(top,#dedede 0,#c8c8c8 99%);background:-ms-linear-gradient(top,#dedede 0,#c8c8c8 99%);background:linear-gradient(to bottom,#dedede 0,#c8c8c8 99%);border:1px solid #999;box-shadow:#aaa 0 0 3px 0;border-radius:3px}input.vis-configuration.vis-config-range::-webkit-slider-thumb{-webkit-appearance:none;border:1px solid #14334b;height:17px;width:17px;border-radius:50%;background:#3876c2;background:-moz-linear-gradient(top,#3876c2 0,#385380 100%);background:-webkit-gradient(linear,left top,left bottom,color-stop(0,#3876c2),color-stop(100%,#385380));background:-webkit-linear-gradient(top,#3876c2 0,#385380 100%);background:-o-linear-gradient(top,#3876c2 0,#385380 100%);background:-ms-linear-gradient(top,#3876c2 0,#385380 100%);background:linear-gradient(to bottom,#3876c2 0,#385380 100%);box-shadow:#111927 0 0 1px 0;margin-top:-7px}input.vis-configuration.vis-config-range:focus{outline:0}input.vis-configuration.vis-config-range:focus::-webkit-slider-runnable-track{background:#9d9d9d;background:-moz-linear-gradient(top,#9d9d9d 0,#c8c8c8 99%);background:-webkit-gradient(linear,left top,left bottom,color-stop(0,#9d9d9d),color-stop(99%,#c8c8c8));background:-webkit-linear-gradient(top,#9d9d9d 0,#c8c8c8 99%);background:-o-linear-gradient(top,#9d9d9d 0,#c8c8c8 99%);background:-ms-linear-gradient(top,#9d9d9d 0,#c8c8c8 99%);background:linear-gradient(to bottom,#9d9d9d 0,#c8c8c8 99%)}input.vis-configuration.vis-config-range::-moz-range-track{width:300px;height:10px;background:#dedede;background:-moz-linear-gradient(top,#dedede 0,#c8c8c8 99%);background:-webkit-gradient(linear,left top,left bottom,color-stop(0,#dedede),color-stop(99%,#c8c8c8));background:-webkit-linear-gradient(top,#dedede 0,#c8c8c8 99%);background:-o-linear-gradient(top,#dedede 0,#c8c8c8 99%);background:-ms-linear-gradient(top,#dedede 0,#c8c8c8 99%);background:linear-gradient(to bottom,#dedede 0,#c8c8c8 99%);border:1px solid #999;box-shadow:#aaa 0 0 3px 0;border-radius:3px}input.vis-configuration.vis-config-range::-moz-range-thumb{border:none;height:16px;width:16px;border-radius:50%;background:#385380}input.vis-configuration.vis-config-range:-moz-focusring{outline:1px solid #fff;outline-offset:-1px}input.vis-configuration.vis-config-range::-ms-track{width:300px;height:5px;background:0 0;border-color:transparent;border-width:6px 0;color:transparent}input.vis-configuration.vis-config-range::-ms-fill-lower{background:#777;border-radius:10px}input.vis-configuration.vis-config-range::-ms-fill-upper{background:#ddd;border-radius:10px}input.vis-configuration.vis-config-range::-ms-thumb{border:none;height:16px;width:16px;border-radius:50%;background:#385380}input.vis-configuration.vis-config-range:focus::-ms-fill-lower{background:#888}input.vis-configuration.vis-config-range:focus::-ms-fill-upper{background:#ccc}.vis-configuration-popup{position:absolute;background:rgba(57,76,89,.85);border:2px solid #f2faff;line-height:30px;height:30px;width:150px;text-align:center;color:#fff;font-size:14px;border-radius:4px;-webkit-transition:opacity .3s ease-in-out;-moz-transition:opacity .3s ease-in-out;transition:opacity .3s ease-in-out}.vis-configuration-popup:after,.vis-configuration-popup:before{left:100%;top:50%;border:solid transparent;content:" ";height:0;width:0;position:absolute;pointer-events:none}.vis-configuration-popup:after{border-color:rgba(136,183,213,0);border-left-color:rgba(57,76,89,.85);border-width:8px;margin-top:-8px}.vis-configuration-popup:before{border-color:rgba(194,225,245,0);border-left-color:#f2faff;border-width:12px;margin-top:-12px}div.vis-tooltip{position:absolute;visibility:hidden;padding:5px;white-space:nowrap;font-family:verdana;font-size:14px;color:#000;background-color:#f5f4ed;-moz-border-radius:3px;-webkit-border-radius:3px;border-radius:3px;border:1px solid #808074;box-shadow:3px 3px 10px rgba(0,0,0,.2);pointer-events:none;z-index:5}div.vis-color-picker{position:absolute;top:0;left:30px;margin-top:-140px;margin-left:30px;width:310px;height:444px;z-index:1;padding:10px;border-radius:15px;background-color:#fff;display:none;box-shadow:rgba(0,0,0,.5) 0 0 10px 0}div.vis-color-picker div.vis-arrow{position:absolute;top:147px;left:5px}div.vis-color-picker div.vis-arrow::after,div.vis-color-picker div.vis-arrow::before{right:100%;top:50%;border:solid transparent;content:" ";height:0;width:0;position:absolute;pointer-events:none}div.vis-color-picker div.vis-arrow:after{border-color:rgba(255,255,255,0);border-right-color:#fff;border-width:30px;margin-top:-30px}div.vis-color-picker div.vis-color{position:absolute;width:289px;height:289px;cursor:pointer}div.vis-color-picker div.vis-brightness{position:absolute;top:313px}div.vis-color-picker div.vis-opacity{position:absolute;top:350px}div.vis-color-picker div.vis-selector{position:absolute;top:137px;left:137px;width:15px;height:15px;border-radius:15px;border:1px solid #fff;background:#4c4c4c;background:-moz-linear-gradient(top,#4c4c4c 0,#595959 12%,#666 25%,#474747 39%,#2c2c2c 50%,#000 51%,#111 60%,#2b2b2b 76%,#1c1c1c 91%,#131313 100%);background:-webkit-gradient(linear,left top,left bottom,color-stop(0,#4c4c4c),color-stop(12%,#595959),color-stop(25%,#666),color-stop(39%,#474747),color-stop(50%,#2c2c2c),color-stop(51%,#000),color-stop(60%,#111),color-stop(76%,#2b2b2b),color-stop(91%,#1c1c1c),color-stop(100%,#131313));background:-webkit-linear-gradient(top,#4c4c4c 0,#595959 12%,#666 25%,#474747 39%,#2c2c2c 50%,#000 51%,#111 60%,#2b2b2b 76%,#1c1c1c 91%,#131313 100%);background:-o-linear-gradient(top,#4c4c4c 0,#595959 12%,#666 25%,#474747 39%,#2c2c2c 50%,#000 51%,#111 60%,#2b2b2b 76%,#1c1c1c 91%,#131313 100%);background:-ms-linear-gradient(top,#4c4c4c 0,#595959 12%,#666 25%,#474747 39%,#2c2c2c 50%,#000 51%,#111 60%,#2b2b2b 76%,#1c1c1c 91%,#131313 100%);background:linear-gradient(to bottom,#4c4c4c 0,#595959 12%,#666 25%,#474747 39%,#2c2c2c 50%,#000 51%,#111 60%,#2b2b2b 76%,#1c1c1c 91%,#131313 100%)}div.vis-color-picker div.vis-new-color{position:absolute;width:140px;height:20px;border:1px solid rgba(0,0,0,.1);border-radius:5px;top:380px;left:159px;text-align:right;padding-right:2px;font-size:10px;color:rgba(0,0,0,.4);vertical-align:middle;line-height:20px}div.vis-color-picker div.vis-initial-color{position:absolute;width:140px;height:20px;border:1px solid rgba(0,0,0,.1);border-radius:5px;top:380px;left:10px;text-align:left;padding-left:2px;font-size:10px;color:rgba(0,0,0,.4);vertical-align:middle;line-height:20px}div.vis-color-picker div.vis-label{position:absolute;width:300px;left:10px}div.vis-color-picker div.vis-label.vis-brightness{top:300px}div.vis-color-picker div.vis-label.vis-opacity{top:338px}div.vis-color-picker div.vis-button{position:absolute;width:68px;height:25px;border-radius:10px;vertical-align:middle;text-align:center;line-height:25px;top:410px;border:2px solid #d9d9d9;background-color:#f7f7f7;cursor:pointer}div.vis-color-picker div.vis-button.vis-cancel{left:5px}div.vis-color-picker div.vis-button.vis-load{left:82px}div.vis-color-picker div.vis-button.vis-apply{left:159px}div.vis-color-picker div.vis-button.vis-save{left:236px}div.vis-color-picker input.vis-range{width:290px;height:20px}div.vis-network div.vis-manipulation{box-sizing:content-box;border-width:0;border-bottom:1px;border-style:solid;border-color:#d6d9d8;background:#fff;background:-moz-linear-gradient(top,#fff 0,#fcfcfc 48%,#fafafa 50%,#fcfcfc 100%);background:-webkit-gradient(linear,left top,left bottom,color-stop(0,#fff),color-stop(48%,#fcfcfc),color-stop(50%,#fafafa),color-stop(100%,#fcfcfc));background:-webkit-linear-gradient(top,#fff 0,#fcfcfc 48%,#fafafa 50%,#fcfcfc 100%);background:-o-linear-gradient(top,#fff 0,#fcfcfc 48%,#fafafa 50%,#fcfcfc 100%);background:-ms-linear-gradient(top,#fff 0,#fcfcfc 48%,#fafafa 50%,#fcfcfc 100%);background:linear-gradient(to bottom,#fff 0,#fcfcfc 48%,#fafafa 50%,#fcfcfc 100%);padding-top:4px;position:absolute;left:0;top:0;width:100%;height:28px}div.vis-network div.vis-edit-mode{position:absolute;left:0;top:5px;height:30px}div.vis-network div.vis-close{position:absolute;right:0;top:0;width:30px;height:30px;background-position:20px 3px;background-repeat:no-repeat;background-image:url('+i+");cursor:pointer;-webkit-touch-callout:none;-webkit-user-select:none;-khtml-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none}div.vis-network div.vis-close:hover{opacity:.6}div.vis-network div.vis-edit-mode div.vis-button,div.vis-network div.vis-manipulation div.vis-button{float:left;font-family:verdana;font-size:12px;-moz-border-radius:15px;border-radius:15px;display:inline-block;background-position:0 0;background-repeat:no-repeat;height:24px;margin-left:10px;cursor:pointer;padding:0 8px 0 8px;-webkit-touch-callout:none;-webkit-user-select:none;-khtml-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none}div.vis-network div.vis-manipulation div.vis-button:hover{box-shadow:1px 1px 8px rgba(0,0,0,.2)}div.vis-network div.vis-manipulation div.vis-button:active{box-shadow:1px 1px 8px rgba(0,0,0,.5)}div.vis-network div.vis-manipulation div.vis-button.vis-back{background-image:url("+o+")}div.vis-network div.vis-manipulation div.vis-button.vis-none:hover{box-shadow:1px 1px 8px transparent;cursor:default}div.vis-network div.vis-manipulation div.vis-button.vis-none:active{box-shadow:1px 1px 8px transparent}div.vis-network div.vis-manipulation div.vis-button.vis-none{padding:0}div.vis-network div.vis-manipulation div.notification{margin:2px;font-weight:700}div.vis-network div.vis-manipulation div.vis-button.vis-add{background-image:url("+a+")}div.vis-network div.vis-edit-mode div.vis-button.vis-edit,div.vis-network div.vis-manipulation div.vis-button.vis-edit{background-image:url("+s+")}div.vis-network div.vis-edit-mode div.vis-button.vis-edit.vis-edit-mode{background-color:#fcfcfc;border:1px solid #ccc}div.vis-network div.vis-manipulation div.vis-button.vis-connect{background-image:url("+l+")}div.vis-network div.vis-manipulation div.vis-button.vis-delete{background-image:url("+u+")}div.vis-network div.vis-edit-mode div.vis-label,div.vis-network div.vis-manipulation div.vis-label{margin:0 0 0 23px;line-height:25px}div.vis-network div.vis-manipulation div.vis-separator-line{float:left;display:inline-block;width:1px;height:21px;background-color:#bdbdbd;margin:0 7px 0 15px}div.vis-network div.vis-navigation div.vis-button{width:34px;height:34px;-moz-border-radius:17px;border-radius:17px;position:absolute;display:inline-block;background-position:2px 2px;background-repeat:no-repeat;cursor:pointer;-webkit-touch-callout:none;-webkit-user-select:none;-khtml-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none}div.vis-network div.vis-navigation div.vis-button:hover{box-shadow:0 0 3px 3px rgba(56,207,21,.3)}div.vis-network div.vis-navigation div.vis-button:active{box-shadow:0 0 1px 3px rgba(56,207,21,.95)}div.vis-network div.vis-navigation div.vis-button.vis-up{background-image:url("+c+");bottom:50px;left:55px}div.vis-network div.vis-navigation div.vis-button.vis-down{background-image:url("+d+");bottom:10px;left:55px}div.vis-network div.vis-navigation div.vis-button.vis-left{background-image:url("+h+");bottom:10px;left:15px}div.vis-network div.vis-navigation div.vis-button.vis-right{background-image:url("+f+");bottom:10px;left:95px}div.vis-network div.vis-navigation div.vis-button.vis-zoomIn{background-image:url("+p+");bottom:10px;right:15px}div.vis-network div.vis-navigation div.vis-button.vis-zoomOut{background-image:url("+g+");bottom:10px;right:55px}div.vis-network div.vis-navigation div.vis-button.vis-zoomExtends{background-image:url("+v+');bottom:50px;right:15px}.vis-current-time{background-color:#ff7f6e;width:2px;z-index:1;pointer-events:none}.vis-rolling-mode-btn{height:40px;width:40px;position:absolute;top:7px;right:20px;border-radius:50%;font-size:28px;cursor:pointer;opacity:.8;color:#fff;font-weight:700;text-align:center;background:#3876c2}.vis-rolling-mode-btn:before{content:"\\26F6"}.vis-rolling-mode-btn:hover{opacity:1}.vis-custom-time{background-color:#6e94ff;width:2px;cursor:move;z-index:1}.vis-panel.vis-background.vis-horizontal .vis-grid.vis-horizontal{position:absolute;width:100%;height:0;border-bottom:1px solid}.vis-panel.vis-background.vis-horizontal .vis-grid.vis-minor{border-color:#e5e5e5}.vis-panel.vis-background.vis-horizontal .vis-grid.vis-major{border-color:#bfbfbf}.vis-data-axis .vis-y-axis.vis-major{width:100%;position:absolute;color:#4d4d4d;white-space:nowrap}.vis-data-axis .vis-y-axis.vis-major.vis-measure{padding:0;margin:0;border:0;visibility:hidden;width:auto}.vis-data-axis .vis-y-axis.vis-minor{position:absolute;width:100%;color:#bebebe;white-space:nowrap}.vis-data-axis .vis-y-axis.vis-minor.vis-measure{padding:0;margin:0;border:0;visibility:hidden;width:auto}.vis-data-axis .vis-y-axis.vis-title{position:absolute;color:#4d4d4d;white-space:nowrap;bottom:20px;text-align:center}.vis-data-axis .vis-y-axis.vis-title.vis-measure{padding:0;margin:0;visibility:hidden;width:auto}.vis-data-axis .vis-y-axis.vis-title.vis-left{bottom:0;-webkit-transform-origin:left top;-moz-transform-origin:left top;-ms-transform-origin:left top;-o-transform-origin:left top;transform-origin:left bottom;-webkit-transform:rotate(-90deg);-moz-transform:rotate(-90deg);-ms-transform:rotate(-90deg);-o-transform:rotate(-90deg);transform:rotate(-90deg)}.vis-data-axis .vis-y-axis.vis-title.vis-right{bottom:0;-webkit-transform-origin:right bottom;-moz-transform-origin:right bottom;-ms-transform-origin:right bottom;-o-transform-origin:right bottom;transform-origin:right bottom;-webkit-transform:rotate(90deg);-moz-transform:rotate(90deg);-ms-transform:rotate(90deg);-o-transform:rotate(90deg);transform:rotate(90deg)}.vis-legend{background-color:rgba(247,252,255,.65);padding:5px;border:1px solid #b3b3b3;box-shadow:2px 2px 10px rgba(154,154,154,.55)}.vis-legend-text{white-space:nowrap;display:inline-block}.vis-item{position:absolute;color:#1a1a1a;border-color:#97b0f8;border-width:1px;background-color:#d5ddf6;display:inline-block;z-index:1}.vis-item.vis-selected{border-color:#ffc200;background-color:#fff785;z-index:2}.vis-editable.vis-selected{cursor:move}.vis-item.vis-point.vis-selected{background-color:#fff785}.vis-item.vis-box{text-align:center;border-style:solid;border-radius:2px}.vis-item.vis-point{background:0 0}.vis-item.vis-dot{position:absolute;padding:0;border-width:4px;border-style:solid;border-radius:4px}.vis-item.vis-range{border-style:solid;border-radius:2px;box-sizing:border-box}.vis-item.vis-background{border:none;background-color:rgba(213,221,246,.4);box-sizing:border-box;padding:0;margin:0}.vis-item .vis-item-overflow{position:relative;width:100%;height:100%;padding:0;margin:0;overflow:hidden}.vis-item-visible-frame{white-space:nowrap}.vis-item.vis-range .vis-item-content{position:relative;display:inline-block}.vis-item.vis-background .vis-item-content{position:absolute;display:inline-block}.vis-item.vis-line{padding:0;position:absolute;width:0;border-left-width:1px;border-left-style:solid}.vis-item .vis-item-content{white-space:nowrap;box-sizing:border-box;padding:5px}.vis-item .vis-onUpdateTime-tooltip{position:absolute;background:#4f81bd;color:#fff;width:200px;text-align:center;white-space:nowrap;padding:5px;border-radius:1px;transition:.4s;-o-transition:.4s;-moz-transition:.4s;-webkit-transition:.4s}.vis-item .vis-delete,.vis-item .vis-delete-rtl{position:absolute;top:0;width:24px;height:24px;box-sizing:border-box;padding:0 5px;cursor:pointer;-webkit-transition:background .2s linear;-moz-transition:background .2s linear;-ms-transition:background .2s linear;-o-transition:background .2s linear;transition:background .2s linear}.vis-item .vis-delete{right:-24px}.vis-item .vis-delete-rtl{left:-24px}.vis-item .vis-delete-rtl:after,.vis-item .vis-delete:after{content:"\\00D7";color:red;font-family:arial,sans-serif;font-size:22px;font-weight:700;-webkit-transition:color .2s linear;-moz-transition:color .2s linear;-ms-transition:color .2s linear;-o-transition:color .2s linear;transition:color .2s linear}.vis-item .vis-delete-rtl:hover,.vis-item .vis-delete:hover{background:red}.vis-item .vis-delete-rtl:hover:after,.vis-item .vis-delete:hover:after{color:#fff}.vis-item .vis-drag-center{position:absolute;width:100%;height:100%;top:0;left:0;cursor:move}.vis-item.vis-range .vis-drag-left{position:absolute;width:24px;max-width:20%;min-width:2px;height:100%;top:0;left:-4px;cursor:w-resize}.vis-item.vis-range .vis-drag-right{position:absolute;width:24px;max-width:20%;min-width:2px;height:100%;top:0;right:-4px;cursor:e-resize}.vis-range.vis-item.vis-readonly .vis-drag-left,.vis-range.vis-item.vis-readonly .vis-drag-right{cursor:auto}.vis-itemset{position:relative;padding:0;margin:0;box-sizing:border-box}.vis-itemset .vis-background,.vis-itemset .vis-foreground{position:absolute;width:100%;height:100%;overflow:visible}.vis-axis{position:absolute;width:100%;height:0;left:0;z-index:1}.vis-foreground .vis-group{position:relative;box-sizing:border-box;border-bottom:1px solid #bfbfbf}.vis-foreground .vis-group:last-child{border-bottom:none}.vis-nesting-group{cursor:pointer}.vis-nested-group{background:#f5f5f5}.vis-label.vis-nesting-group.expanded:before{content:"\\25BC"}.vis-label.vis-nesting-group.collapsed-rtl:before{content:"\\25C0"}.vis-label.vis-nesting-group.collapsed:before{content:"\\25B6"}.vis-overlay{position:absolute;top:0;left:0;width:100%;height:100%;z-index:10}.vis-labelset{position:relative;overflow:hidden;box-sizing:border-box}.vis-labelset .vis-label{position:relative;left:0;top:0;width:100%;color:#4d4d4d;box-sizing:border-box}.vis-labelset .vis-label{border-bottom:1px solid #bfbfbf}.vis-labelset .vis-label.draggable{cursor:pointer}.vis-labelset .vis-label:last-child{border-bottom:none}.vis-labelset .vis-label .vis-inner{display:inline-block;padding:5px}.vis-labelset .vis-label .vis-inner.vis-hidden{padding:0}.vis-panel{position:absolute;padding:0;margin:0;box-sizing:border-box}.vis-panel.vis-bottom,.vis-panel.vis-center,.vis-panel.vis-left,.vis-panel.vis-right,.vis-panel.vis-top{border:1px #bfbfbf}.vis-panel.vis-center,.vis-panel.vis-left,.vis-panel.vis-right{border-top-style:solid;border-bottom-style:solid;overflow:hidden}.vis-left.vis-panel.vis-vertical-scroll,.vis-right.vis-panel.vis-vertical-scroll{height:100%;overflow-x:hidden;overflow-y:scroll}.vis-left.vis-panel.vis-vertical-scroll{direction:rtl}.vis-left.vis-panel.vis-vertical-scroll .vis-content{direction:ltr}.vis-right.vis-panel.vis-vertical-scroll{direction:ltr}.vis-right.vis-panel.vis-vertical-scroll .vis-content{direction:rtl}.vis-panel.vis-bottom,.vis-panel.vis-center,.vis-panel.vis-top{border-left-style:solid;border-right-style:solid}.vis-background{overflow:hidden}.vis-panel>.vis-content{position:relative}.vis-panel .vis-shadow{position:absolute;width:100%;height:1px;box-shadow:0 0 10px rgba(0,0,0,.8)}.vis-panel .vis-shadow.vis-top{top:-1px;left:0}.vis-panel .vis-shadow.vis-bottom{bottom:-1px;left:0}.vis-graph-group0{fill:#4f81bd;fill-opacity:0;stroke-width:2px;stroke:#4f81bd}.vis-graph-group1{fill:#f79646;fill-opacity:0;stroke-width:2px;stroke:#f79646}.vis-graph-group2{fill:#8c51cf;fill-opacity:0;stroke-width:2px;stroke:#8c51cf}.vis-graph-group3{fill:#75c841;fill-opacity:0;stroke-width:2px;stroke:#75c841}.vis-graph-group4{fill:#ff0100;fill-opacity:0;stroke-width:2px;stroke:#ff0100}.vis-graph-group5{fill:#37d8e6;fill-opacity:0;stroke-width:2px;stroke:#37d8e6}.vis-graph-group6{fill:#042662;fill-opacity:0;stroke-width:2px;stroke:#042662}.vis-graph-group7{fill:#00ff26;fill-opacity:0;stroke-width:2px;stroke:#00ff26}.vis-graph-group8{fill:#f0f;fill-opacity:0;stroke-width:2px;stroke:#f0f}.vis-graph-group9{fill:#8f3938;fill-opacity:0;stroke-width:2px;stroke:#8f3938}.vis-timeline .vis-fill{fill-opacity:.1;stroke:none}.vis-timeline .vis-bar{fill-opacity:.5;stroke-width:1px}.vis-timeline .vis-point{stroke-width:2px;fill-opacity:1}.vis-timeline .vis-legend-background{stroke-width:1px;fill-opacity:.9;fill:#fff;stroke:#c2c2c2}.vis-timeline .vis-outline{stroke-width:1px;fill-opacity:1;fill:#fff;stroke:#e5e5e5}.vis-timeline .vis-icon-fill{fill-opacity:.3;stroke:none}.vis-time-axis{position:relative;overflow:hidden}.vis-time-axis.vis-foreground{top:0;left:0;width:100%}.vis-time-axis.vis-background{position:absolute;top:0;left:0;width:100%;height:100%}.vis-time-axis .vis-text{position:absolute;color:#4d4d4d;padding:3px;overflow:hidden;box-sizing:border-box;white-space:nowrap}.vis-time-axis .vis-text.vis-measure{position:absolute;padding-left:0;padding-right:0;margin-left:0;margin-right:0;visibility:hidden}.vis-time-axis .vis-grid.vis-vertical{position:absolute;border-left:1px solid}.vis-time-axis .vis-grid.vis-vertical-rtl{position:absolute;border-right:1px solid}.vis-time-axis .vis-grid.vis-minor{border-color:#e5e5e5}.vis-time-axis .vis-grid.vis-major{border-color:#bfbfbf}.vis-timeline{position:relative;border:1px solid #bfbfbf;overflow:hidden;padding:0;margin:0;box-sizing:border-box}',""])},function(e,t){e.exports="data:image/png;base64, "},function(e,t){e.exports="data:image/png;base64, "},function(e,t){e.exports="data:image/png;base64, "},function(e,t){e.exports="data:image/png;base64, "},function(e,t){e.exports="data:image/png;base64, "},function(e,t){e.exports="data:image/png;base64, "},function(e,t){e.exports="data:image/png;base64, "},function(e,t){e.exports="data:image/png;base64, "},function(e,t){e.exports="data:image/png;base64, "},function(e,t){e.exports="data:image/png;base64, "},function(e,t){e.exports="data:image/png;base64, "},function(e,t){e.exports="data:image/png;base64, "},function(e,t){e.exports="data:image/png;base64, "},function(e,t,n){var r,i,o=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),a=this&&this.__setModuleDefault||(Object.create?function(e,t){Object.defineProperty(e,"default",{enumerable:!0,value:t})}:function(e,t){e.default=t}),s=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)"default"!==n&&Object.hasOwnProperty.call(e,n)&&o(t,e,n);return a(t,e),t},l=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};r=[n,t,n(0),n(90),n(78),n(88),n(2329),n(103),n(622),n(432),n(440),n(48)],void 0===(i=function(e,t,n,r,i,o,a,u,c,d,h,f){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.SystemInfoElement=void 0,n=s(n),o=l(o),u=l(u),d=l(d),(h=l(h)).default.setOptions({gfm:!0,sanitize:!1});class p extends n.Component{constructor(e){super(e),this.on_msg=(e,t)=>{const{type:n,payload:r}=e;switch(n){case"SystemInfoComponent::infoData":if(this._initialized)return;{this._rawContent=r;const e=Object.keys(r)[0];this.handleSystemChange(null,e),this._initialized=!0}break;case"SystemInfoComponent::updateData":this._renderedContent={},this._rawContent=r,this.handleSystemChange(null,this.state.selectedSystem)}},this.handleSystemChange=(e,t)=>{if(t in this._renderedContent){const{htmlString:e,renderMath:n}=this._renderedContent[t];this.setState(n=>Object.assign(Object.assign({},n),{selectedSystem:t,selectedContent:e}),()=>{n&&this._mathjax.typeset(this._divRef.current)})}else{const e=this._rawContent[t],r=c.removeMath(e);(n=r.text,new Promise((e,t)=>{h.default(n,(n,r)=>{n?t(n):e(r)})})).then(e=>{const n=c.replaceMath(e,r.math),i=r.math.length>0;this._renderedContent[t]={htmlString:n,renderMath:i},this.setState(e=>Object.assign(Object.assign({},e),{selectedSystem:t,selectedContent:n}),()=>{i&&this._mathjax.typeset(this._divRef.current)})})}var n},e.model.listenTo(e.model,"msg:custom",this.on_msg),this._divRef=n.default.createRef();let t;t=this.props.systemConfig.enableEdit?f.PageConfig.getOption("fullMathjaxUrl"):"https://cdnjs.cloudflare.com/ajax/libs/mathjax/2.7.7/MathJax.js",this._mathjax=new a.MathJaxTypesetter({url:t,config:"TeX-AMS-MML_HTMLorMML-full,Safe"}),this._rawContent={},this._renderedContent={},this._initialized=!1,this.state={traceConfig:{},allSystem:[],selectedSystem:"",selectedContent:""},e.send_msg({action:"SystemInfoComponent::getData"})}componentDidUpdate(e,t){}componentDidMount(){}componentWillUnmount(){this.props.model.stopListening(this.props.model,"msg:custom",this.on_msg)}render(){return n.default.createElement("div",{className:"cosapp-widget-box"},n.default.createElement("div",{className:"cosapp-info-widget",ref:this._divRef,style:{height:"calc(100% - 30px)",paddingLeft:25,paddingRight:25,overflow:"auto"},dangerouslySetInnerHTML:{__html:this.state.selectedContent}}),n.default.createElement("div",{style:{height:"30px",display:"flex",background:"#e0e0e0"}},n.default.createElement(u.default,{className:this.props.classes.formControl},n.default.createElement(d.default,{disableClearable:!0,value:this.state.selectedSystem,onChange:this.handleSystemChange,options:Object.keys(this._rawContent),size:"small",renderInput:e=>n.default.createElement(o.default,Object.assign({},e,{size:"small",variant:"standard",placeholder:"",fullWidth:!0,InputLabelProps:{shrink:!1}}))}))))}}t.SystemInfoElement=p,t.default=i.connect(e=>({systemConfig:e.systemConfig}),e=>({}),null,{forwardRef:!0})(r.withStyles(e=>({textColor:{color:"rgb(250, 250, 250)",background:"#525354",margin:"0px 5px"},formControl:{paddingLeft:"5%",width:"90%"}}))(n.forwardRef((e,t)=>n.default.createElement(p,Object.assign({},e,{ref:t})))))}.apply(t,r))||(e.exports=i)},function(e,t,n){"use strict";n.r(t),n.d(t,"MathJaxTypesetter",(function(){return i}));var r=n(17);class i{constructor(e){this._initPromise=new r.PromiseDelegate,this._initialized=!1,this._url=e.url,this._config=e.config}typeset(e){this._initialized||this._init(),this._initPromise.promise.then(()=>{MathJax.Hub.Queue(["Typeset",MathJax.Hub,e]);try{MathJax.Hub.Queue(["Require",MathJax.Ajax,"[MathJax]/extensions/TeX/AMSmath.js"],()=>{MathJax.InputJax.TeX.resetEquationNumbers()})}catch(e){console.error("Error queueing resetEquationNumbers:",e)}})}_init(){const e=document.getElementsByTagName("head")[0],t=document.createElement("script");t.type="text/javascript",t.src=`${this._url}?config=${this._config}&amp;delayStartupUntil=configured`,t.charset="utf-8",e.appendChild(t),t.addEventListener("load",()=>{this._onLoad()}),this._initialized=!0}_onLoad(){MathJax.Hub.Config({tex2jax:{inlineMath:[["$","$"],["\\(","\\)"]],displayMath:[["$$","$$"],["\\[","\\]"]],processEscapes:!0,processEnvironments:!0},displayAlign:"center",CommonHTML:{linebreaks:{automatic:!0}},"HTML-CSS":{availableFonts:[],imageFont:null,preferredFont:null,webFont:"STIX-Web",styles:{".MathJax_Display":{margin:0}},linebreaks:{automatic:!0}},skipStartupTypeset:!0,messageStyle:"none"}),MathJax.Hub.Configured(),this._initPromise.resolve(void 0)}}},function(e,t,n){var r,i,o=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),a=this&&this.__setModuleDefault||(Object.create?function(e,t){Object.defineProperty(e,"default",{enumerable:!0,value:t})}:function(e,t){e.default=t}),s=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)"default"!==n&&Object.hasOwnProperty.call(e,n)&&o(t,e,n);return a(t,e),t},l=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};r=[n,t,n(90),n(75),n(108),n(109),n(110),n(175),n(103),n(192),n(120),n(76),n(132),n(88),n(377),n(378),n(0),n(520),n(379),n(78),n(248),n(582),n(583),n(584),n(585),n(400),n(586),n(587),n(588),n(589),n(2331),n(590),n(305)],void 0===(i=function(e,t,n,r,i,o,a,u,c,d,h,f,p,g,v,m,y,b,C,A,_){"use strict";function I(e){return y.default.createElement(C.default,{handle:".draggable-dialog-title",cancel:'[class*="MuiDialogContent-root"]'},y.default.createElement(f.default,Object.assign({},e)))}Object.defineProperty(t,"__esModule",{value:!0}),t.DocumentViewer=void 0,r=l(r),i=l(i),o=l(o),a=l(a),u=l(u),c=l(c),d=l(d),h=l(h),f=l(f),p=l(p),g=l(g),v=l(v),m=l(m),y=s(y),C=l(C);function x(e){e.somethingSelected()?e.indentSelection("add"):e.replaceSelection(e.getOption("indentWithTabs")?"\t":Array(e.getOption("indentUnit")+1).join(" "),"end","+input")}class w extends y.Component{constructor(e){super(e),this.documentSourceChange=e=>{const t=e.target.value;this.setState(e=>Object.assign(Object.assign({},e),{traceConfig:Object.assign(Object.assign({},e.traceConfig),{documentSource:t})}))},this.toggleSetting=()=>{this.setState(e=>Object.assign(Object.assign({},e),{openSetting:!e.openSetting}))},this.toggleEdit=()=>{this.setState(e=>Object.assign(Object.assign({},e),{openEdit:!e.openEdit}))},this.handleWidgetNameChange=e=>{const t=e.target.value;this.setState(e=>Object.assign(Object.assign({},e),{traceConfig:Object.assign(Object.assign({},e.traceConfig),{widgetName:t})}))},this.updateSetting=()=>{const e=this.state.traceConfig.documentSource;this.props.send_msg({action:"DocumentViewer::generateDocument",payload:{source:e,title:this.divID}}),this.setState(e=>Object.assign(Object.assign({},e),{openSetting:!1}))},this.handleSelectedWidgetChange=e=>{const t=e.target.value;this.setState(e=>Object.assign(Object.assign({},e),{selectedWidget:t}))},this.updateWidgetSelect=()=>{let e;e=this.state.selectedWidget in this.props.availableDocument?this.props.availableDocument[this.state.selectedWidget].documentSource:"",this.setState(t=>Object.assign(Object.assign({},t),{traceConfig:Object.assign(Object.assign({},t.traceConfig),{documentSource:e}),openEdit:!1}),()=>{this.props.send_msg({action:"DocumentViewer::generateDocument",payload:{source:e,title:this.divID}})})},this.initialState=e.initialState,this.divID="document-viewer-"+_.create_UUID();let t="",n="";for(const[e,r]of Object.entries(this.props.availableDocument))r.id===this.props.id&&(t=e,n=r.widgetName);this.state={traceConfig:{documentSource:"",widgetName:n},openSetting:!1,openEdit:!1,selectedWidget:t}}componentDidMount(){if(this.initialState){let{documentSource:e=""}=this.initialState.traceConfig;0===e.length&&(e="\n".repeat(10)),this.setState(t=>Object.assign(Object.assign({},t),{traceConfig:Object.assign(Object.assign({},t.traceConfig),{documentSource:e})}),()=>{this.props.send_msg({action:"DocumentViewer::generateDocument",payload:{source:e,title:this.divID}})})}}componentDidUpdate(e,t){}render(){this.props.classes;return y.default.createElement("div",{className:"cosapp-widget-box"},y.default.createElement(i.default,{open:this.state.openSetting,"aria-labelledby":"draggable-dialog-title",fullWidth:!0,maxWidth:"md",PaperComponent:I},y.default.createElement(u.default,{style:{cursor:"move"},className:"draggable-dialog-title"},"Document viewer configuration"),y.default.createElement(a.default,{style:{height:"50vh"}},y.default.createElement("div",{style:{height:"calc(100% - 60px)",overflowY:"auto"}},y.default.createElement(b.Controlled,{value:this.state.traceConfig.documentSource,options:{mode:"markdown",lineNumbers:!0,lineWrapping:!0,smartIndent:!0,indentWithTabs:!1,tabSize:4,indentUnit:4,foldGutter:!0,gutters:["CodeMirror-linenumbers","CodeMirror-foldgutter"],autoCloseTags:!0,matchBrackets:!0,autoCloseBrackets:!0,extraKeys:{Tab:x}},onBeforeChange:(e,t,n)=>{this.setState(e=>Object.assign(Object.assign({},e),{traceConfig:Object.assign(Object.assign({},e.traceConfig),{documentSource:n})}))},onChange:(e,t,n)=>{}})),y.default.createElement(c.default,{className:this.props.classes.formControl},y.default.createElement(g.default,{value:this.state.traceConfig.widgetName,onChange:this.handleWidgetNameChange,label:"Document name"}))),y.default.createElement(o.default,null,y.default.createElement(r.default,{autoFocus:!0,onClick:this.toggleSetting,color:"primary"},"Close"),y.default.createElement(r.default,{autoFocus:!0,onClick:this.updateSetting,color:"primary"},"Save"))),y.default.createElement(i.default,{open:this.state.openEdit,"aria-labelledby":"draggable-dialog-title",fullWidth:!0,maxWidth:"sm",PaperComponent:I},y.default.createElement(u.default,{style:{cursor:"move"},className:"draggable-dialog-title"},"Document viewer selector"),y.default.createElement(a.default,null,y.default.createElement(c.default,{style:{width:"100%"}},y.default.createElement(d.default,{id:"doct-type-select-id"},"Select document"),y.default.createElement(p.default,{labelId:"doc-type-select-id",value:this.state.selectedWidget,onChange:this.handleSelectedWidgetChange},Object.entries(this.props.availableDocument).map(([e,t])=>y.default.createElement(h.default,{key:e,value:e},t.widgetName))))),y.default.createElement(o.default,null,y.default.createElement(r.default,{autoFocus:!0,onClick:this.toggleEdit,color:"primary"},"Close"),y.default.createElement(r.default,{autoFocus:!0,onClick:this.updateWidgetSelect,color:"primary"},"Select"))),y.default.createElement("div",{id:this.divID,className:"DocumentViewerMain",style:{width:"100%",height:"calc(100% - 35px)",overflow:"auto",boxSizing:"border-box",padding:"10px",background:"white"}}),y.default.createElement("div",{className:this.props.classes.bottomBarDiv},y.default.createElement(r.default,{onClick:this.toggleEdit,style:{color:"rgb(50,50,50)"}}," ",y.default.createElement(m.default,null)),this.props.systemConfig.enableEdit?y.default.createElement(r.default,{onClick:this.toggleSetting,style:{color:"rgb(50,50,50)"}}," ",y.default.createElement(v.default,null)):y.default.createElement("div",null)))}}t.DocumentViewer=w,t.default=A.connect(e=>(e=>({systemConfig:e.systemConfig}))(e),e=>({}),null,{forwardRef:!0})(n.withStyles(e=>({textSizeSmall:{},toolbarHeigt:{minHeight:36,background:"rgb(50, 50, 50)"},viewSelector:{minWidth:120,color:"rgb(250, 250, 250)"},bgSelector:{minWidth:60,color:"rgb(250, 250, 250)"},textColor:{color:"rgb(250, 250, 250)"},backGround:{color:"rgb(50, 50, 50)"},formControl:{padding:"10px 5% 0px 5%;",width:"90%"},formControlShort:{padding:e.spacing(1),width:"calc(50% - 16px)"},formControlTiny:{width:"22%"},selectEmpty:{marginTop:e.spacing(2)},textField:{marginLeft:e.spacing(1),marginRight:e.spacing(1),width:"90%"},bottomBarDiv:{width:"100%",height:"35px",background:"#e0e0e0",display:"flex",flexDirection:"row-reverse"}}))(y.forwardRef((e,t)=>y.default.createElement(w,Object.assign({},e,{ref:t})))))}.apply(t,r))||(e.exports=i)},function(e,t,n){!function(e){"use strict";e.defineMode("markdown",(function(t,n){var r=e.getMode(t,"text/html"),i="null"==r.name;void 0===n.highlightFormatting&&(n.highlightFormatting=!1),void 0===n.maxBlockquoteDepth&&(n.maxBlockquoteDepth=0),void 0===n.taskLists&&(n.taskLists=!1),void 0===n.strikethrough&&(n.strikethrough=!1),void 0===n.emoji&&(n.emoji=!1),void 0===n.fencedCodeBlockHighlighting&&(n.fencedCodeBlockHighlighting=!0),void 0===n.fencedCodeBlockDefaultMode&&(n.fencedCodeBlockDefaultMode="text/plain"),void 0===n.xml&&(n.xml=!0),void 0===n.tokenTypeOverrides&&(n.tokenTypeOverrides={});var o={header:"header",code:"comment",quote:"quote",list1:"variable-2",list2:"variable-3",list3:"keyword",hr:"hr",image:"image",imageAltText:"image-alt-text",imageMarker:"image-marker",formatting:"formatting",linkInline:"link",linkEmail:"link",linkText:"link",linkHref:"string",em:"em",strong:"strong",strikethrough:"strikethrough",emoji:"builtin"};for(var a in o)o.hasOwnProperty(a)&&n.tokenTypeOverrides[a]&&(o[a]=n.tokenTypeOverrides[a]);var s=/^([*\-_])(?:\s*\1){2,}\s*$/,l=/^(?:[*\-+]|^[0-9]+([.)]))\s+/,u=/^\[(x| )\](?=\s)/i,c=n.allowAtxHeaderWithoutSpace?/^(#+)/:/^(#+)(?: |$)/,d=/^ {0,3}(?:\={1,}|-{2,})\s*$/,h=/^[^#!\[\]*_\\<>` "'(~:]+/,f=/^(~~~+|```+)[ \t]*([\w\/+#-]*)[^\n`]*$/,p=/^\s*\[[^\]]+?\]:.*$/,g=/[!"#$%&'()*+,\-.\/:;<=>?@\[\\\]^_`{|}~\xA1\xA7\xAB\xB6\xB7\xBB\xBF\u037E\u0387\u055A-\u055F\u0589\u058A\u05BE\u05C0\u05C3\u05C6\u05F3\u05F4\u0609\u060A\u060C\u060D\u061B\u061E\u061F\u066A-\u066D\u06D4\u0700-\u070D\u07F7-\u07F9\u0830-\u083E\u085E\u0964\u0965\u0970\u0AF0\u0DF4\u0E4F\u0E5A\u0E5B\u0F04-\u0F12\u0F14\u0F3A-\u0F3D\u0F85\u0FD0-\u0FD4\u0FD9\u0FDA\u104A-\u104F\u10FB\u1360-\u1368\u1400\u166D\u166E\u169B\u169C\u16EB-\u16ED\u1735\u1736\u17D4-\u17D6\u17D8-\u17DA\u1800-\u180A\u1944\u1945\u1A1E\u1A1F\u1AA0-\u1AA6\u1AA8-\u1AAD\u1B5A-\u1B60\u1BFC-\u1BFF\u1C3B-\u1C3F\u1C7E\u1C7F\u1CC0-\u1CC7\u1CD3\u2010-\u2027\u2030-\u2043\u2045-\u2051\u2053-\u205E\u207D\u207E\u208D\u208E\u2308-\u230B\u2329\u232A\u2768-\u2775\u27C5\u27C6\u27E6-\u27EF\u2983-\u2998\u29D8-\u29DB\u29FC\u29FD\u2CF9-\u2CFC\u2CFE\u2CFF\u2D70\u2E00-\u2E2E\u2E30-\u2E42\u3001-\u3003\u3008-\u3011\u3014-\u301F\u3030\u303D\u30A0\u30FB\uA4FE\uA4FF\uA60D-\uA60F\uA673\uA67E\uA6F2-\uA6F7\uA874-\uA877\uA8CE\uA8CF\uA8F8-\uA8FA\uA8FC\uA92E\uA92F\uA95F\uA9C1-\uA9CD\uA9DE\uA9DF\uAA5C-\uAA5F\uAADE\uAADF\uAAF0\uAAF1\uABEB\uFD3E\uFD3F\uFE10-\uFE19\uFE30-\uFE52\uFE54-\uFE61\uFE63\uFE68\uFE6A\uFE6B\uFF01-\uFF03\uFF05-\uFF0A\uFF0C-\uFF0F\uFF1A\uFF1B\uFF1F\uFF20\uFF3B-\uFF3D\uFF3F\uFF5B\uFF5D\uFF5F-\uFF65]|\uD800[\uDD00-\uDD02\uDF9F\uDFD0]|\uD801\uDD6F|\uD802[\uDC57\uDD1F\uDD3F\uDE50-\uDE58\uDE7F\uDEF0-\uDEF6\uDF39-\uDF3F\uDF99-\uDF9C]|\uD804[\uDC47-\uDC4D\uDCBB\uDCBC\uDCBE-\uDCC1\uDD40-\uDD43\uDD74\uDD75\uDDC5-\uDDC9\uDDCD\uDDDB\uDDDD-\uDDDF\uDE38-\uDE3D\uDEA9]|\uD805[\uDCC6\uDDC1-\uDDD7\uDE41-\uDE43\uDF3C-\uDF3E]|\uD809[\uDC70-\uDC74]|\uD81A[\uDE6E\uDE6F\uDEF5\uDF37-\uDF3B\uDF44]|\uD82F\uDC9F|\uD836[\uDE87-\uDE8B]/;function v(e,t,n){return t.f=t.inline=n,n(e,t)}function m(e,t,n){return t.f=t.block=n,n(e,t)}function y(t){if(t.linkTitle=!1,t.linkHref=!1,t.linkText=!1,t.em=!1,t.strong=!1,t.strikethrough=!1,t.quote=0,t.indentedCode=!1,t.f==C){var n=i;if(!n){var o=e.innerMode(r,t.htmlState);n="xml"==o.mode.name&&null===o.state.tagStart&&!o.state.context&&o.state.tokenize.isInText}n&&(t.f=x,t.block=b,t.htmlState=null)}return t.trailingSpace=0,t.trailingSpaceNewLine=!1,t.prevLine=t.thisLine,t.thisLine={stream:null},null}function b(r,i){var a,h=r.column()===i.indentation,g=!(a=i.prevLine.stream)||!/\S/.test(a.string),m=i.indentedCode,y=i.prevLine.hr,b=!1!==i.list,C=(i.listStack[i.listStack.length-1]||0)+3;i.indentedCode=!1;var I=i.indentation;if(null===i.indentationDiff&&(i.indentationDiff=i.indentation,b)){for(i.list=null;I<i.listStack[i.listStack.length-1];)i.listStack.pop(),i.listStack.length?i.indentation=i.listStack[i.listStack.length-1]:i.list=!1;!1!==i.list&&(i.indentationDiff=I-i.listStack[i.listStack.length-1])}var x=!(g||y||i.prevLine.header||b&&m||i.prevLine.fencedCodeEnd),w=(!1===i.list||y||g)&&i.indentation<=C&&r.match(s),M=null;if(i.indentationDiff>=4&&(m||i.prevLine.fencedCodeEnd||i.prevLine.header||g))return r.skipToEnd(),i.indentedCode=!0,o.code;if(r.eatSpace())return null;if(h&&i.indentation<=C&&(M=r.match(c))&&M[1].length<=6)return i.quote=0,i.header=M[1].length,i.thisLine.header=!0,n.highlightFormatting&&(i.formatting="header"),i.f=i.inline,_(i);if(i.indentation<=C&&r.eat(">"))return i.quote=h?1:i.quote+1,n.highlightFormatting&&(i.formatting="quote"),r.eatSpace(),_(i);if(!w&&!i.setext&&h&&i.indentation<=C&&(M=r.match(l))){var S=M[1]?"ol":"ul";return i.indentation=I+r.current().length,i.list=!0,i.quote=0,i.listStack.push(i.indentation),i.em=!1,i.strong=!1,i.code=!1,i.strikethrough=!1,n.taskLists&&r.match(u,!1)&&(i.taskList=!0),i.f=i.inline,n.highlightFormatting&&(i.formatting=["list","list-"+S]),_(i)}return h&&i.indentation<=C&&(M=r.match(f,!0))?(i.quote=0,i.fencedEndRE=new RegExp(M[1]+"+ *$"),i.localMode=n.fencedCodeBlockHighlighting&&function(n){if(e.findModeByName){var r=e.findModeByName(n);r&&(n=r.mime||r.mimes[0])}var i=e.getMode(t,n);return"null"==i.name?null:i}(M[2]||n.fencedCodeBlockDefaultMode),i.localMode&&(i.localState=e.startState(i.localMode)),i.f=i.block=A,n.highlightFormatting&&(i.formatting="code-block"),i.code=-1,_(i)):i.setext||!(x&&b||i.quote||!1!==i.list||i.code||w||p.test(r.string))&&(M=r.lookAhead(1))&&(M=M.match(d))?(i.setext?(i.header=i.setext,i.setext=0,r.skipToEnd(),n.highlightFormatting&&(i.formatting="header")):(i.header="="==M[0].charAt(0)?1:2,i.setext=i.header),i.thisLine.header=!0,i.f=i.inline,_(i)):w?(r.skipToEnd(),i.hr=!0,i.thisLine.hr=!0,o.hr):"["===r.peek()?v(r,i,T):v(r,i,i.inline)}function C(t,n){var o=r.token(t,n.htmlState);if(!i){var a=e.innerMode(r,n.htmlState);("xml"==a.mode.name&&null===a.state.tagStart&&!a.state.context&&a.state.tokenize.isInText||n.md_inside&&t.current().indexOf(">")>-1)&&(n.f=x,n.block=b,n.htmlState=null)}return o}function A(e,t){var r,i=t.listStack[t.listStack.length-1]||0,a=t.indentation<i,s=i+3;return t.fencedEndRE&&t.indentation<=s&&(a||e.match(t.fencedEndRE))?(n.highlightFormatting&&(t.formatting="code-block"),a||(r=_(t)),t.localMode=t.localState=null,t.block=b,t.f=x,t.fencedEndRE=null,t.code=0,t.thisLine.fencedCodeEnd=!0,a?m(e,t,t.block):r):t.localMode?t.localMode.token(e,t.localState):(e.skipToEnd(),o.code)}function _(e){var t=[];if(e.formatting){t.push(o.formatting),"string"==typeof e.formatting&&(e.formatting=[e.formatting]);for(var r=0;r<e.formatting.length;r++)t.push(o.formatting+"-"+e.formatting[r]),"header"===e.formatting[r]&&t.push(o.formatting+"-"+e.formatting[r]+"-"+e.header),"quote"===e.formatting[r]&&(!n.maxBlockquoteDepth||n.maxBlockquoteDepth>=e.quote?t.push(o.formatting+"-"+e.formatting[r]+"-"+e.quote):t.push("error"))}if(e.taskOpen)return t.push("meta"),t.length?t.join(" "):null;if(e.taskClosed)return t.push("property"),t.length?t.join(" "):null;if(e.linkHref?t.push(o.linkHref,"url"):(e.strong&&t.push(o.strong),e.em&&t.push(o.em),e.strikethrough&&t.push(o.strikethrough),e.emoji&&t.push(o.emoji),e.linkText&&t.push(o.linkText),e.code&&t.push(o.code),e.image&&t.push(o.image),e.imageAltText&&t.push(o.imageAltText,"link"),e.imageMarker&&t.push(o.imageMarker)),e.header&&t.push(o.header,o.header+"-"+e.header),e.quote&&(t.push(o.quote),!n.maxBlockquoteDepth||n.maxBlockquoteDepth>=e.quote?t.push(o.quote+"-"+e.quote):t.push(o.quote+"-"+n.maxBlockquoteDepth)),!1!==e.list){var i=(e.listStack.length-1)%3;i?1===i?t.push(o.list2):t.push(o.list3):t.push(o.list1)}return e.trailingSpaceNewLine?t.push("trailing-space-new-line"):e.trailingSpace&&t.push("trailing-space-"+(e.trailingSpace%2?"a":"b")),t.length?t.join(" "):null}function I(e,t){if(e.match(h,!0))return _(t)}function x(t,i){var a=i.text(t,i);if(void 0!==a)return a;if(i.list)return i.list=null,_(i);if(i.taskList)return" "===t.match(u,!0)[1]?i.taskOpen=!0:i.taskClosed=!0,n.highlightFormatting&&(i.formatting="task"),i.taskList=!1,_(i);if(i.taskOpen=!1,i.taskClosed=!1,i.header&&t.match(/^#+$/,!0))return n.highlightFormatting&&(i.formatting="header"),_(i);var s=t.next();if(i.linkTitle){i.linkTitle=!1;var l=s;"("===s&&(l=")");var c="^\\s*(?:[^"+(l=(l+"").replace(/([.?*+^\[\]\\(){}|-])/g,"\\$1"))+"\\\\]+|\\\\\\\\|\\\\.)"+l;if(t.match(new RegExp(c),!0))return o.linkHref}if("`"===s){var d=i.formatting;n.highlightFormatting&&(i.formatting="code"),t.eatWhile("`");var h=t.current().length;if(0!=i.code||i.quote&&1!=h){if(h==i.code){var f=_(i);return i.code=0,f}return i.formatting=d,_(i)}return i.code=h,_(i)}if(i.code)return _(i);if("\\"===s&&(t.next(),n.highlightFormatting)){var p=_(i),v=o.formatting+"-escape";return p?p+" "+v:v}if("!"===s&&t.match(/\[[^\]]*\] ?(?:\(|\[)/,!1))return i.imageMarker=!0,i.image=!0,n.highlightFormatting&&(i.formatting="image"),_(i);if("["===s&&i.imageMarker&&t.match(/[^\]]*\](\(.*?\)| ?\[.*?\])/,!1))return i.imageMarker=!1,i.imageAltText=!0,n.highlightFormatting&&(i.formatting="image"),_(i);if("]"===s&&i.imageAltText){n.highlightFormatting&&(i.formatting="image");var p=_(i);return i.imageAltText=!1,i.image=!1,i.inline=i.f=M,p}if("["===s&&!i.image)return i.linkText&&t.match(/^.*?\]/)||(i.linkText=!0,n.highlightFormatting&&(i.formatting="link")),_(i);if("]"===s&&i.linkText){n.highlightFormatting&&(i.formatting="link");var p=_(i);return i.linkText=!1,i.inline=i.f=t.match(/\(.*?\)| ?\[.*?\]/,!1)?M:x,p}if("<"===s&&t.match(/^(https?|ftps?):\/\/(?:[^\\>]|\\.)+>/,!1))return i.f=i.inline=w,n.highlightFormatting&&(i.formatting="link"),(p=_(i))?p+=" ":p="",p+o.linkInline;if("<"===s&&t.match(/^[^> \\]+@(?:[^\\>]|\\.)+>/,!1))return i.f=i.inline=w,n.highlightFormatting&&(i.formatting="link"),(p=_(i))?p+=" ":p="",p+o.linkEmail;if(n.xml&&"<"===s&&t.match(/^(!--|\?|!\[CDATA\[|[a-z][a-z0-9-]*(?:\s+[a-z_:.\-]+(?:\s*=\s*[^>]+)?)*\s*(?:>|$))/i,!1)){var y=t.string.indexOf(">",t.pos);if(-1!=y){var b=t.string.substring(t.start,y);/markdown\s*=\s*('|"){0,1}1('|"){0,1}/.test(b)&&(i.md_inside=!0)}return t.backUp(1),i.htmlState=e.startState(r),m(t,i,C)}if(n.xml&&"<"===s&&t.match(/^\/\w*?>/))return i.md_inside=!1,"tag";if("*"===s||"_"===s){for(var A=1,I=1==t.pos?" ":t.string.charAt(t.pos-2);A<3&&t.eat(s);)A++;var S=t.peek()||" ",T=!/\s/.test(S)&&(!g.test(S)||/\s/.test(I)||g.test(I)),E=!/\s/.test(I)&&(!g.test(I)||/\s/.test(S)||g.test(S)),O=null,k=null;if(A%2&&(i.em||!T||"*"!==s&&E&&!g.test(I)?i.em!=s||!E||"*"!==s&&T&&!g.test(S)||(O=!1):O=!0),A>1&&(i.strong||!T||"*"!==s&&E&&!g.test(I)?i.strong!=s||!E||"*"!==s&&T&&!g.test(S)||(k=!1):k=!0),null!=k||null!=O)return n.highlightFormatting&&(i.formatting=null==O?"strong":null==k?"em":"strong em"),!0===O&&(i.em=s),!0===k&&(i.strong=s),f=_(i),!1===O&&(i.em=!1),!1===k&&(i.strong=!1),f}else if(" "===s&&(t.eat("*")||t.eat("_"))){if(" "===t.peek())return _(i);t.backUp(1)}if(n.strikethrough)if("~"===s&&t.eatWhile(s)){if(i.strikethrough)return n.highlightFormatting&&(i.formatting="strikethrough"),f=_(i),i.strikethrough=!1,f;if(t.match(/^[^\s]/,!1))return i.strikethrough=!0,n.highlightFormatting&&(i.formatting="strikethrough"),_(i)}else if(" "===s&&t.match("~~",!0)){if(" "===t.peek())return _(i);t.backUp(2)}if(n.emoji&&":"===s&&t.match(/^(?:[a-z_\d+][a-z_\d+-]*|\-[a-z_\d+][a-z_\d+-]*):/)){i.emoji=!0,n.highlightFormatting&&(i.formatting="emoji");var L=_(i);return i.emoji=!1,L}return" "===s&&(t.match(/^ +$/,!1)?i.trailingSpace++:i.trailingSpace&&(i.trailingSpaceNewLine=!0)),_(i)}function w(e,t){if(">"===e.next()){t.f=t.inline=x,n.highlightFormatting&&(t.formatting="link");var r=_(t);return r?r+=" ":r="",r+o.linkInline}return e.match(/^[^>]+/,!0),o.linkInline}function M(e,t){if(e.eatSpace())return null;var r,i=e.next();return"("===i||"["===i?(t.f=t.inline=(r="("===i?")":"]",function(e,t){if(e.next()===r){t.f=t.inline=x,n.highlightFormatting&&(t.formatting="link-string");var i=_(t);return t.linkHref=!1,i}return e.match(S[r]),t.linkHref=!0,_(t)}),n.highlightFormatting&&(t.formatting="link-string"),t.linkHref=!0,_(t)):"error"}var S={")":/^(?:[^\\\(\)]|\\.|\((?:[^\\\(\)]|\\.)*\))*?(?=\))/,"]":/^(?:[^\\\[\]]|\\.|\[(?:[^\\\[\]]|\\.)*\])*?(?=\])/};function T(e,t){return e.match(/^([^\]\\]|\\.)*\]:/,!1)?(t.f=E,e.next(),n.highlightFormatting&&(t.formatting="link"),t.linkText=!0,_(t)):v(e,t,x)}function E(e,t){if(e.match("]:",!0)){t.f=t.inline=O,n.highlightFormatting&&(t.formatting="link");var r=_(t);return t.linkText=!1,r}return e.match(/^([^\]\\]|\\.)+/,!0),o.linkText}function O(e,t){return e.eatSpace()?null:(e.match(/^[^\s]+/,!0),void 0===e.peek()?t.linkTitle=!0:e.match(/^(?:\s+(?:"(?:[^"\\]|\\\\|\\.)+"|'(?:[^'\\]|\\\\|\\.)+'|\((?:[^)\\]|\\.)+\)))?/,!0),t.f=t.inline=x,o.linkHref+" url")}var k={startState:function(){return{f:b,prevLine:{stream:null},thisLine:{stream:null},block:b,htmlState:null,indentation:0,inline:x,text:I,formatting:!1,linkText:!1,linkHref:!1,linkTitle:!1,code:0,em:!1,strong:!1,header:0,setext:0,hr:!1,taskList:!1,list:!1,listStack:[],quote:0,trailingSpace:0,trailingSpaceNewLine:!1,strikethrough:!1,emoji:!1,fencedEndRE:null}},copyState:function(t){return{f:t.f,prevLine:t.prevLine,thisLine:t.thisLine,block:t.block,htmlState:t.htmlState&&e.copyState(r,t.htmlState),indentation:t.indentation,localMode:t.localMode,localState:t.localMode?e.copyState(t.localMode,t.localState):null,inline:t.inline,text:t.text,formatting:!1,linkText:t.linkText,linkTitle:t.linkTitle,linkHref:t.linkHref,code:t.code,em:t.em,strong:t.strong,strikethrough:t.strikethrough,emoji:t.emoji,header:t.header,setext:t.setext,hr:t.hr,taskList:t.taskList,list:t.list,listStack:t.listStack.slice(0),quote:t.quote,indentedCode:t.indentedCode,trailingSpace:t.trailingSpace,trailingSpaceNewLine:t.trailingSpaceNewLine,md_inside:t.md_inside,fencedEndRE:t.fencedEndRE}},token:function(e,t){if(t.formatting=!1,e!=t.thisLine.stream){if(t.header=0,t.hr=!1,e.match(/^\s*$/,!0))return y(t),null;if(t.prevLine=t.thisLine,t.thisLine={stream:e},t.taskList=!1,t.trailingSpace=0,t.trailingSpaceNewLine=!1,!t.localState&&(t.f=t.block,t.f!=C)){var n=e.match(/^\s*/,!0)[0].replace(/\t/g,"    ").length;if(t.indentation=n,t.indentationDiff=null,n>0)return null}}return t.f(e,t)},innerMode:function(e){return e.block==C?{state:e.htmlState,mode:r}:e.localState?{state:e.localState,mode:e.localMode}:{state:e,mode:k}},indent:function(t,n,i){return t.block==C&&r.indent?r.indent(t.htmlState,n,i):t.localState&&t.localMode.indent?t.localMode.indent(t.localState,n,i):e.Pass},blankLine:y,getType:_,blockCommentStart:"\x3c!--",blockCommentEnd:"--\x3e",closeBrackets:"()[]{}''\"\"``",fold:"markdown"};return k}),"xml"),e.defineMIME("text/markdown","markdown"),e.defineMIME("text/x-markdown","markdown")}(n(122),n(2332),n(2333))},function(e,t,n){!function(e){"use strict";var t={autoSelfClosers:{area:!0,base:!0,br:!0,col:!0,command:!0,embed:!0,frame:!0,hr:!0,img:!0,input:!0,keygen:!0,link:!0,meta:!0,param:!0,source:!0,track:!0,wbr:!0,menuitem:!0},implicitlyClosed:{dd:!0,li:!0,optgroup:!0,option:!0,p:!0,rp:!0,rt:!0,tbody:!0,td:!0,tfoot:!0,th:!0,tr:!0},contextGrabbers:{dd:{dd:!0,dt:!0},dt:{dd:!0,dt:!0},li:{li:!0},option:{option:!0,optgroup:!0},optgroup:{optgroup:!0},p:{address:!0,article:!0,aside:!0,blockquote:!0,dir:!0,div:!0,dl:!0,fieldset:!0,footer:!0,form:!0,h1:!0,h2:!0,h3:!0,h4:!0,h5:!0,h6:!0,header:!0,hgroup:!0,hr:!0,menu:!0,nav:!0,ol:!0,p:!0,pre:!0,section:!0,table:!0,ul:!0},rp:{rp:!0,rt:!0},rt:{rp:!0,rt:!0},tbody:{tbody:!0,tfoot:!0},td:{td:!0,th:!0},tfoot:{tbody:!0},th:{td:!0,th:!0},thead:{tbody:!0,tfoot:!0},tr:{tr:!0}},doNotIndent:{pre:!0},allowUnquoted:!0,allowMissing:!0,caseFold:!0},n={autoSelfClosers:{},implicitlyClosed:{},contextGrabbers:{},doNotIndent:{},allowUnquoted:!1,allowMissing:!1,allowMissingTagName:!1,caseFold:!1};e.defineMode("xml",(function(r,i){var o,a,s=r.indentUnit,l={},u=i.htmlMode?t:n;for(var c in u)l[c]=u[c];for(var c in i)l[c]=i[c];function d(e,t){function n(n){return t.tokenize=n,n(e,t)}var r=e.next();return"<"==r?e.eat("!")?e.eat("[")?e.match("CDATA[")?n(f("atom","]]>")):null:e.match("--")?n(f("comment","--\x3e")):e.match("DOCTYPE",!0,!0)?(e.eatWhile(/[\w\._\-]/),n(function e(t){return function(n,r){for(var i;null!=(i=n.next());){if("<"==i)return r.tokenize=e(t+1),r.tokenize(n,r);if(">"==i){if(1==t){r.tokenize=d;break}return r.tokenize=e(t-1),r.tokenize(n,r)}}return"meta"}}(1))):null:e.eat("?")?(e.eatWhile(/[\w\._\-]/),t.tokenize=f("meta","?>"),"meta"):(o=e.eat("/")?"closeTag":"openTag",t.tokenize=h,"tag bracket"):"&"==r?(e.eat("#")?e.eat("x")?e.eatWhile(/[a-fA-F\d]/)&&e.eat(";"):e.eatWhile(/[\d]/)&&e.eat(";"):e.eatWhile(/[\w\.\-:]/)&&e.eat(";"))?"atom":"error":(e.eatWhile(/[^&<]/),null)}function h(e,t){var n,r,i=e.next();if(">"==i||"/"==i&&e.eat(">"))return t.tokenize=d,o=">"==i?"endTag":"selfcloseTag","tag bracket";if("="==i)return o="equals",null;if("<"==i){t.tokenize=d,t.state=m,t.tagName=t.tagStart=null;var a=t.tokenize(e,t);return a?a+" tag error":"tag error"}return/[\'\"]/.test(i)?(t.tokenize=(n=i,(r=function(e,t){for(;!e.eol();)if(e.next()==n){t.tokenize=h;break}return"string"}).isInAttribute=!0,r),t.stringStartCol=e.column(),t.tokenize(e,t)):(e.match(/^[^\s\u00a0=<>\"\']*[^\s\u00a0=<>\"\'\/]/),"word")}function f(e,t){return function(n,r){for(;!n.eol();){if(n.match(t)){r.tokenize=d;break}n.next()}return e}}function p(e,t,n){this.prev=e.context,this.tagName=t||"",this.indent=e.indented,this.startOfLine=n,(l.doNotIndent.hasOwnProperty(t)||e.context&&e.context.noIndent)&&(this.noIndent=!0)}function g(e){e.context&&(e.context=e.context.prev)}function v(e,t){for(var n;;){if(!e.context)return;if(n=e.context.tagName,!l.contextGrabbers.hasOwnProperty(n)||!l.contextGrabbers[n].hasOwnProperty(t))return;g(e)}}function m(e,t,n){return"openTag"==e?(n.tagStart=t.column(),y):"closeTag"==e?b:m}function y(e,t,n){return"word"==e?(n.tagName=t.current(),a="tag",_):l.allowMissingTagName&&"endTag"==e?(a="tag bracket",_(e,0,n)):(a="error",y)}function b(e,t,n){if("word"==e){var r=t.current();return n.context&&n.context.tagName!=r&&l.implicitlyClosed.hasOwnProperty(n.context.tagName)&&g(n),n.context&&n.context.tagName==r||!1===l.matchClosing?(a="tag",C):(a="tag error",A)}return l.allowMissingTagName&&"endTag"==e?(a="tag bracket",C(e,0,n)):(a="error",A)}function C(e,t,n){return"endTag"!=e?(a="error",C):(g(n),m)}function A(e,t,n){return a="error",C(e,0,n)}function _(e,t,n){if("word"==e)return a="attribute",I;if("endTag"==e||"selfcloseTag"==e){var r=n.tagName,i=n.tagStart;return n.tagName=n.tagStart=null,"selfcloseTag"==e||l.autoSelfClosers.hasOwnProperty(r)?v(n,r):(v(n,r),n.context=new p(n,r,i==n.indented)),m}return a="error",_}function I(e,t,n){return"equals"==e?x:(l.allowMissing||(a="error"),_(e,0,n))}function x(e,t,n){return"string"==e?w:"word"==e&&l.allowUnquoted?(a="string",_):(a="error",_(e,0,n))}function w(e,t,n){return"string"==e?w:_(e,0,n)}return d.isInText=!0,{startState:function(e){var t={tokenize:d,state:m,indented:e||0,tagName:null,tagStart:null,context:null};return null!=e&&(t.baseIndent=e),t},token:function(e,t){if(!t.tagName&&e.sol()&&(t.indented=e.indentation()),e.eatSpace())return null;o=null;var n=t.tokenize(e,t);return(n||o)&&"comment"!=n&&(a=null,t.state=t.state(o||n,e,t),a&&(n="error"==a?n+" error":a)),n},indent:function(t,n,r){var i=t.context;if(t.tokenize.isInAttribute)return t.tagStart==t.indented?t.stringStartCol+1:t.indented+s;if(i&&i.noIndent)return e.Pass;if(t.tokenize!=h&&t.tokenize!=d)return r?r.match(/^(\s*)/)[0].length:0;if(t.tagName)return!1!==l.multilineTagIndentPastTag?t.tagStart+t.tagName.length+2:t.tagStart+s*(l.multilineTagIndentFactor||1);if(l.alignCDATA&&/<!\[CDATA\[/.test(n))return 0;var o=n&&/^<(\/)?([\w_:\.-]*)/.exec(n);if(o&&o[1])for(;i;){if(i.tagName==o[2]){i=i.prev;break}if(!l.implicitlyClosed.hasOwnProperty(i.tagName))break;i=i.prev}else if(o)for(;i;){var a=l.contextGrabbers[i.tagName];if(!a||!a.hasOwnProperty(o[2]))break;i=i.prev}for(;i&&i.prev&&!i.startOfLine;)i=i.prev;return i?i.indent+s:t.baseIndent||0},electricInput:/<\/[\s\w:]+>$/,blockCommentStart:"\x3c!--",blockCommentEnd:"--\x3e",configuration:l.htmlMode?"html":"xml",helperType:l.htmlMode?"html":"xml",skipAttribute:function(e){e.state==x&&(e.state=_)},xmlCurrentTag:function(e){return e.tagName?{name:e.tagName,close:"closeTag"==e.type}:null},xmlCurrentContext:function(e){for(var t=[],n=e.context;n;n=n.prev)t.push(n.tagName);return t.reverse()}}})),e.defineMIME("text/xml","xml"),e.defineMIME("application/xml","xml"),e.mimeModes.hasOwnProperty("text/html")||e.defineMIME("text/html",{name:"xml",htmlMode:!0})}(n(122))},function(e,t,n){!function(e){"use strict";e.modeInfo=[{name:"APL",mime:"text/apl",mode:"apl",ext:["dyalog","apl"]},{name:"PGP",mimes:["application/pgp","application/pgp-encrypted","application/pgp-keys","application/pgp-signature"],mode:"asciiarmor",ext:["asc","pgp","sig"]},{name:"ASN.1",mime:"text/x-ttcn-asn",mode:"asn.1",ext:["asn","asn1"]},{name:"Asterisk",mime:"text/x-asterisk",mode:"asterisk",file:/^extensions\.conf$/i},{name:"Brainfuck",mime:"text/x-brainfuck",mode:"brainfuck",ext:["b","bf"]},{name:"C",mime:"text/x-csrc",mode:"clike",ext:["c","h","ino"]},{name:"C++",mime:"text/x-c++src",mode:"clike",ext:["cpp","c++","cc","cxx","hpp","h++","hh","hxx"],alias:["cpp"]},{name:"Cobol",mime:"text/x-cobol",mode:"cobol",ext:["cob","cpy"]},{name:"C#",mime:"text/x-csharp",mode:"clike",ext:["cs"],alias:["csharp","cs"]},{name:"Clojure",mime:"text/x-clojure",mode:"clojure",ext:["clj","cljc","cljx"]},{name:"ClojureScript",mime:"text/x-clojurescript",mode:"clojure",ext:["cljs"]},{name:"Closure Stylesheets (GSS)",mime:"text/x-gss",mode:"css",ext:["gss"]},{name:"CMake",mime:"text/x-cmake",mode:"cmake",ext:["cmake","cmake.in"],file:/^CMakeLists\.txt$/},{name:"CoffeeScript",mimes:["application/vnd.coffeescript","text/coffeescript","text/x-coffeescript"],mode:"coffeescript",ext:["coffee"],alias:["coffee","coffee-script"]},{name:"Common Lisp",mime:"text/x-common-lisp",mode:"commonlisp",ext:["cl","lisp","el"],alias:["lisp"]},{name:"Cypher",mime:"application/x-cypher-query",mode:"cypher",ext:["cyp","cypher"]},{name:"Cython",mime:"text/x-cython",mode:"python",ext:["pyx","pxd","pxi"]},{name:"Crystal",mime:"text/x-crystal",mode:"crystal",ext:["cr"]},{name:"CSS",mime:"text/css",mode:"css",ext:["css"]},{name:"CQL",mime:"text/x-cassandra",mode:"sql",ext:["cql"]},{name:"D",mime:"text/x-d",mode:"d",ext:["d"]},{name:"Dart",mimes:["application/dart","text/x-dart"],mode:"dart",ext:["dart"]},{name:"diff",mime:"text/x-diff",mode:"diff",ext:["diff","patch"]},{name:"Django",mime:"text/x-django",mode:"django"},{name:"Dockerfile",mime:"text/x-dockerfile",mode:"dockerfile",file:/^Dockerfile$/},{name:"DTD",mime:"application/xml-dtd",mode:"dtd",ext:["dtd"]},{name:"Dylan",mime:"text/x-dylan",mode:"dylan",ext:["dylan","dyl","intr"]},{name:"EBNF",mime:"text/x-ebnf",mode:"ebnf"},{name:"ECL",mime:"text/x-ecl",mode:"ecl",ext:["ecl"]},{name:"edn",mime:"application/edn",mode:"clojure",ext:["edn"]},{name:"Eiffel",mime:"text/x-eiffel",mode:"eiffel",ext:["e"]},{name:"Elm",mime:"text/x-elm",mode:"elm",ext:["elm"]},{name:"Embedded JavaScript",mime:"application/x-ejs",mode:"htmlembedded",ext:["ejs"]},{name:"Embedded Ruby",mime:"application/x-erb",mode:"htmlembedded",ext:["erb"]},{name:"Erlang",mime:"text/x-erlang",mode:"erlang",ext:["erl"]},{name:"Esper",mime:"text/x-esper",mode:"sql"},{name:"Factor",mime:"text/x-factor",mode:"factor",ext:["factor"]},{name:"FCL",mime:"text/x-fcl",mode:"fcl"},{name:"Forth",mime:"text/x-forth",mode:"forth",ext:["forth","fth","4th"]},{name:"Fortran",mime:"text/x-fortran",mode:"fortran",ext:["f","for","f77","f90","f95"]},{name:"F#",mime:"text/x-fsharp",mode:"mllike",ext:["fs"],alias:["fsharp"]},{name:"Gas",mime:"text/x-gas",mode:"gas",ext:["s"]},{name:"Gherkin",mime:"text/x-feature",mode:"gherkin",ext:["feature"]},{name:"GitHub Flavored Markdown",mime:"text/x-gfm",mode:"gfm",file:/^(readme|contributing|history)\.md$/i},{name:"Go",mime:"text/x-go",mode:"go",ext:["go"]},{name:"Groovy",mime:"text/x-groovy",mode:"groovy",ext:["groovy","gradle"],file:/^Jenkinsfile$/},{name:"HAML",mime:"text/x-haml",mode:"haml",ext:["haml"]},{name:"Haskell",mime:"text/x-haskell",mode:"haskell",ext:["hs"]},{name:"Haskell (Literate)",mime:"text/x-literate-haskell",mode:"haskell-literate",ext:["lhs"]},{name:"Haxe",mime:"text/x-haxe",mode:"haxe",ext:["hx"]},{name:"HXML",mime:"text/x-hxml",mode:"haxe",ext:["hxml"]},{name:"ASP.NET",mime:"application/x-aspx",mode:"htmlembedded",ext:["aspx"],alias:["asp","aspx"]},{name:"HTML",mime:"text/html",mode:"htmlmixed",ext:["html","htm","handlebars","hbs"],alias:["xhtml"]},{name:"HTTP",mime:"message/http",mode:"http"},{name:"IDL",mime:"text/x-idl",mode:"idl",ext:["pro"]},{name:"Pug",mime:"text/x-pug",mode:"pug",ext:["jade","pug"],alias:["jade"]},{name:"Java",mime:"text/x-java",mode:"clike",ext:["java"]},{name:"Java Server Pages",mime:"application/x-jsp",mode:"htmlembedded",ext:["jsp"],alias:["jsp"]},{name:"JavaScript",mimes:["text/javascript","text/ecmascript","application/javascript","application/x-javascript","application/ecmascript"],mode:"javascript",ext:["js"],alias:["ecmascript","js","node"]},{name:"JSON",mimes:["application/json","application/x-json"],mode:"javascript",ext:["json","map"],alias:["json5"]},{name:"JSON-LD",mime:"application/ld+json",mode:"javascript",ext:["jsonld"],alias:["jsonld"]},{name:"JSX",mime:"text/jsx",mode:"jsx",ext:["jsx"]},{name:"Jinja2",mime:"text/jinja2",mode:"jinja2",ext:["j2","jinja","jinja2"]},{name:"Julia",mime:"text/x-julia",mode:"julia",ext:["jl"]},{name:"Kotlin",mime:"text/x-kotlin",mode:"clike",ext:["kt"]},{name:"LESS",mime:"text/x-less",mode:"css",ext:["less"]},{name:"LiveScript",mime:"text/x-livescript",mode:"livescript",ext:["ls"],alias:["ls"]},{name:"Lua",mime:"text/x-lua",mode:"lua",ext:["lua"]},{name:"Markdown",mime:"text/x-markdown",mode:"markdown",ext:["markdown","md","mkd"]},{name:"mIRC",mime:"text/mirc",mode:"mirc"},{name:"MariaDB SQL",mime:"text/x-mariadb",mode:"sql"},{name:"Mathematica",mime:"text/x-mathematica",mode:"mathematica",ext:["m","nb","wl","wls"]},{name:"Modelica",mime:"text/x-modelica",mode:"modelica",ext:["mo"]},{name:"MUMPS",mime:"text/x-mumps",mode:"mumps",ext:["mps"]},{name:"MS SQL",mime:"text/x-mssql",mode:"sql"},{name:"mbox",mime:"application/mbox",mode:"mbox",ext:["mbox"]},{name:"MySQL",mime:"text/x-mysql",mode:"sql"},{name:"Nginx",mime:"text/x-nginx-conf",mode:"nginx",file:/nginx.*\.conf$/i},{name:"NSIS",mime:"text/x-nsis",mode:"nsis",ext:["nsh","nsi"]},{name:"NTriples",mimes:["application/n-triples","application/n-quads","text/n-triples"],mode:"ntriples",ext:["nt","nq"]},{name:"Objective-C",mime:"text/x-objectivec",mode:"clike",ext:["m"],alias:["objective-c","objc"]},{name:"Objective-C++",mime:"text/x-objectivec++",mode:"clike",ext:["mm"],alias:["objective-c++","objc++"]},{name:"OCaml",mime:"text/x-ocaml",mode:"mllike",ext:["ml","mli","mll","mly"]},{name:"Octave",mime:"text/x-octave",mode:"octave",ext:["m"]},{name:"Oz",mime:"text/x-oz",mode:"oz",ext:["oz"]},{name:"Pascal",mime:"text/x-pascal",mode:"pascal",ext:["p","pas"]},{name:"PEG.js",mime:"null",mode:"pegjs",ext:["jsonld"]},{name:"Perl",mime:"text/x-perl",mode:"perl",ext:["pl","pm"]},{name:"PHP",mimes:["text/x-php","application/x-httpd-php","application/x-httpd-php-open"],mode:"php",ext:["php","php3","php4","php5","php7","phtml"]},{name:"Pig",mime:"text/x-pig",mode:"pig",ext:["pig"]},{name:"Plain Text",mime:"text/plain",mode:"null",ext:["txt","text","conf","def","list","log"]},{name:"PLSQL",mime:"text/x-plsql",mode:"sql",ext:["pls"]},{name:"PostgreSQL",mime:"text/x-pgsql",mode:"sql"},{name:"PowerShell",mime:"application/x-powershell",mode:"powershell",ext:["ps1","psd1","psm1"]},{name:"Properties files",mime:"text/x-properties",mode:"properties",ext:["properties","ini","in"],alias:["ini","properties"]},{name:"ProtoBuf",mime:"text/x-protobuf",mode:"protobuf",ext:["proto"]},{name:"Python",mime:"text/x-python",mode:"python",ext:["BUILD","bzl","py","pyw"],file:/^(BUCK|BUILD)$/},{name:"Puppet",mime:"text/x-puppet",mode:"puppet",ext:["pp"]},{name:"Q",mime:"text/x-q",mode:"q",ext:["q"]},{name:"R",mime:"text/x-rsrc",mode:"r",ext:["r","R"],alias:["rscript"]},{name:"reStructuredText",mime:"text/x-rst",mode:"rst",ext:["rst"],alias:["rst"]},{name:"RPM Changes",mime:"text/x-rpm-changes",mode:"rpm"},{name:"RPM Spec",mime:"text/x-rpm-spec",mode:"rpm",ext:["spec"]},{name:"Ruby",mime:"text/x-ruby",mode:"ruby",ext:["rb"],alias:["jruby","macruby","rake","rb","rbx"]},{name:"Rust",mime:"text/x-rustsrc",mode:"rust",ext:["rs"]},{name:"SAS",mime:"text/x-sas",mode:"sas",ext:["sas"]},{name:"Sass",mime:"text/x-sass",mode:"sass",ext:["sass"]},{name:"Scala",mime:"text/x-scala",mode:"clike",ext:["scala"]},{name:"Scheme",mime:"text/x-scheme",mode:"scheme",ext:["scm","ss"]},{name:"SCSS",mime:"text/x-scss",mode:"css",ext:["scss"]},{name:"Shell",mimes:["text/x-sh","application/x-sh"],mode:"shell",ext:["sh","ksh","bash"],alias:["bash","sh","zsh"],file:/^PKGBUILD$/},{name:"Sieve",mime:"application/sieve",mode:"sieve",ext:["siv","sieve"]},{name:"Slim",mimes:["text/x-slim","application/x-slim"],mode:"slim",ext:["slim"]},{name:"Smalltalk",mime:"text/x-stsrc",mode:"smalltalk",ext:["st"]},{name:"Smarty",mime:"text/x-smarty",mode:"smarty",ext:["tpl"]},{name:"Solr",mime:"text/x-solr",mode:"solr"},{name:"SML",mime:"text/x-sml",mode:"mllike",ext:["sml","sig","fun","smackspec"]},{name:"Soy",mime:"text/x-soy",mode:"soy",ext:["soy"],alias:["closure template"]},{name:"SPARQL",mime:"application/sparql-query",mode:"sparql",ext:["rq","sparql"],alias:["sparul"]},{name:"Spreadsheet",mime:"text/x-spreadsheet",mode:"spreadsheet",alias:["excel","formula"]},{name:"SQL",mime:"text/x-sql",mode:"sql",ext:["sql"]},{name:"SQLite",mime:"text/x-sqlite",mode:"sql"},{name:"Squirrel",mime:"text/x-squirrel",mode:"clike",ext:["nut"]},{name:"Stylus",mime:"text/x-styl",mode:"stylus",ext:["styl"]},{name:"Swift",mime:"text/x-swift",mode:"swift",ext:["swift"]},{name:"sTeX",mime:"text/x-stex",mode:"stex"},{name:"LaTeX",mime:"text/x-latex",mode:"stex",ext:["text","ltx","tex"],alias:["tex"]},{name:"SystemVerilog",mime:"text/x-systemverilog",mode:"verilog",ext:["v","sv","svh"]},{name:"Tcl",mime:"text/x-tcl",mode:"tcl",ext:["tcl"]},{name:"Textile",mime:"text/x-textile",mode:"textile",ext:["textile"]},{name:"TiddlyWiki",mime:"text/x-tiddlywiki",mode:"tiddlywiki"},{name:"Tiki wiki",mime:"text/tiki",mode:"tiki"},{name:"TOML",mime:"text/x-toml",mode:"toml",ext:["toml"]},{name:"Tornado",mime:"text/x-tornado",mode:"tornado"},{name:"troff",mime:"text/troff",mode:"troff",ext:["1","2","3","4","5","6","7","8","9"]},{name:"TTCN",mime:"text/x-ttcn",mode:"ttcn",ext:["ttcn","ttcn3","ttcnpp"]},{name:"TTCN_CFG",mime:"text/x-ttcn-cfg",mode:"ttcn-cfg",ext:["cfg"]},{name:"Turtle",mime:"text/turtle",mode:"turtle",ext:["ttl"]},{name:"TypeScript",mime:"application/typescript",mode:"javascript",ext:["ts"],alias:["ts"]},{name:"TypeScript-JSX",mime:"text/typescript-jsx",mode:"jsx",ext:["tsx"],alias:["tsx"]},{name:"Twig",mime:"text/x-twig",mode:"twig"},{name:"Web IDL",mime:"text/x-webidl",mode:"webidl",ext:["webidl"]},{name:"VB.NET",mime:"text/x-vb",mode:"vb",ext:["vb"]},{name:"VBScript",mime:"text/vbscript",mode:"vbscript",ext:["vbs"]},{name:"Velocity",mime:"text/velocity",mode:"velocity",ext:["vtl"]},{name:"Verilog",mime:"text/x-verilog",mode:"verilog",ext:["v"]},{name:"VHDL",mime:"text/x-vhdl",mode:"vhdl",ext:["vhd","vhdl"]},{name:"Vue.js Component",mimes:["script/x-vue","text/x-vue"],mode:"vue",ext:["vue"]},{name:"XML",mimes:["application/xml","text/xml"],mode:"xml",ext:["xml","xsl","xsd","svg"],alias:["rss","wsdl","xsd"]},{name:"XQuery",mime:"application/xquery",mode:"xquery",ext:["xy","xquery"]},{name:"Yacas",mime:"text/x-yacas",mode:"yacas",ext:["ys"]},{name:"YAML",mimes:["text/x-yaml","text/yaml"],mode:"yaml",ext:["yaml","yml"],alias:["yml"]},{name:"Z80",mime:"text/x-z80",mode:"z80",ext:["z80"]},{name:"mscgen",mime:"text/x-mscgen",mode:"mscgen",ext:["mscgen","mscin","msc"]},{name:"xu",mime:"text/x-xu",mode:"mscgen",ext:["xu"]},{name:"msgenny",mime:"text/x-msgenny",mode:"mscgen",ext:["msgenny"]},{name:"WebAssembly",mime:"text/webassembly",mode:"wast",ext:["wat","wast"]}];for(var t=0;t<e.modeInfo.length;t++){var n=e.modeInfo[t];n.mimes&&(n.mime=n.mimes[0])}e.findModeByMIME=function(t){t=t.toLowerCase();for(var n=0;n<e.modeInfo.length;n++){var r=e.modeInfo[n];if(r.mime==t)return r;if(r.mimes)for(var i=0;i<r.mimes.length;i++)if(r.mimes[i]==t)return r}return/\+xml$/.test(t)?e.findModeByMIME("application/xml"):/\+json$/.test(t)?e.findModeByMIME("application/json"):void 0},e.findModeByExtension=function(t){t=t.toLowerCase();for(var n=0;n<e.modeInfo.length;n++){var r=e.modeInfo[n];if(r.ext)for(var i=0;i<r.ext.length;i++)if(r.ext[i]==t)return r}},e.findModeByFileName=function(t){for(var n=0;n<e.modeInfo.length;n++){var r=e.modeInfo[n];if(r.file&&r.file.test(t))return r}var i=t.lastIndexOf("."),o=i>-1&&t.substring(i+1,t.length);if(o)return e.findModeByExtension(o)},e.findModeByName=function(t){t=t.toLowerCase();for(var n=0;n<e.modeInfo.length;n++){var r=e.modeInfo[n];if(r.name.toLowerCase()==t)return r;if(r.alias)for(var i=0;i<r.alias.length;i++)if(r.alias[i].toLowerCase()==t)return r}}}(n(122))},function(e,t,n){var r,i,o=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),a=this&&this.__setModuleDefault||(Object.create?function(e,t){Object.defineProperty(e,"default",{enumerable:!0,value:t})}:function(e,t){e.default=t}),s=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)"default"!==n&&Object.hasOwnProperty.call(e,n)&&o(t,e,n);return a(t,e),t},l=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};r=[n,t,n(90),n(75),n(108),n(109),n(110),n(175),n(103),n(192),n(120),n(76),n(132),n(88),n(377),n(378),n(0),n(520),n(379),n(78),n(248),n(582),n(583),n(584),n(585),n(400),n(586),n(587),n(588),n(589),n(1095),n(590),n(305)],void 0===(i=function(e,t,n,r,i,o,a,u,c,d,h,f,p,g,v,m,y,b,C,A,_){"use strict";function I(e){return y.default.createElement(C.default,{handle:".draggable-dialog-title",cancel:'[class*="MuiDialogContent-root"]'},y.default.createElement(f.default,Object.assign({},e)))}Object.defineProperty(t,"__esModule",{value:!0}),t.WidgetViewer=void 0,r=l(r),i=l(i),o=l(o),a=l(a),u=l(u),c=l(c),d=l(d),h=l(h),f=l(f),p=l(p),g=l(g),v=l(v),m=l(m),y=s(y),C=l(C);function x(e){e.somethingSelected()?e.indentSelection("add"):e.replaceSelection(e.getOption("indentWithTabs")?"\t":Array(e.getOption("indentUnit")+1).join(" "),"end","+input")}class w extends y.Component{constructor(e){super(e),this.pythonCodeChange=e=>{const t=e.target.value;this.setState(e=>Object.assign(Object.assign({},e),{traceConfig:Object.assign(Object.assign({},e.traceConfig),{pythonCode:t})}))},this.toggleSetting=()=>{this.setState(e=>Object.assign(Object.assign({},e),{openSetting:!e.openSetting}))},this.toggleEdit=()=>{this.setState(e=>Object.assign(Object.assign({},e),{openEdit:!e.openEdit}))},this.handleWidgetNameChange=e=>{const t=e.target.value;this.setState(e=>Object.assign(Object.assign({},e),{traceConfig:Object.assign(Object.assign({},e.traceConfig),{widgetName:t})}))},this.updateSetting=()=>{const e=this.state.traceConfig.pythonCode;this.props.send_msg({action:"WidgetViewer::executeCode",payload:{code:e,title:this.divID}}),this.setState(e=>Object.assign(Object.assign({},e),{openSetting:!1}))},this.handleSelectedWidgetChange=e=>{const t=e.target.value;this.setState(e=>Object.assign(Object.assign({},e),{selectedWidget:t}))},this.updateWidgetSelect=()=>{let e;e=this.state.selectedWidget in this.props.availableWidget?this.props.availableWidget[this.state.selectedWidget].pythonCode:"",this.setState(t=>Object.assign(Object.assign({},t),{traceConfig:Object.assign(Object.assign({},t.traceConfig),{pythonCode:e}),openEdit:!1}),()=>{this.props.send_msg({action:"WidgetViewer::executeCode",payload:{code:e,title:this.divID}})})},this.initialState=e.initialState,this.divID="widget-viewer-"+_.create_UUID();let t="",n="";for(const[e,r]of Object.entries(this.props.availableWidget))r.id===this.props.id&&(t=e,n=r.widgetName);this.state={traceConfig:{pythonCode:"# Define widget generation function here\n\nimport ipywidgets as widgets\nfrom cosapp.systems import System\ndef generate_widget(sys : System)-> widgets.Widget:\n    pass\n",widgetName:n},openSetting:!1,openEdit:!1,selectedWidget:t}}componentDidMount(){if(this.initialState){const{pythonCode:e=""}=this.initialState.traceConfig;this.setState(t=>Object.assign(Object.assign({},t),{traceConfig:Object.assign(Object.assign({},t.traceConfig),{pythonCode:e})}),()=>{this.props.send_msg({action:"WidgetViewer::executeCode",payload:{code:e,title:this.divID}})})}}componentDidUpdate(e,t){}render(){this.props.classes;return y.default.createElement("div",{className:"cosapp-widget-box"},y.default.createElement(i.default,{open:this.state.openSetting,"aria-labelledby":"draggable-dialog-title",fullWidth:!0,maxWidth:"md",PaperComponent:I},y.default.createElement(u.default,{style:{cursor:"move"},className:"draggable-dialog-title"},"Widget viewer configuration"),y.default.createElement(a.default,{style:{height:"50vh"}},y.default.createElement("div",{style:{height:"calc(100% - 60px)",overflowY:"auto"}},y.default.createElement(b.Controlled,{value:this.state.traceConfig.pythonCode,options:{mode:"python",lineNumbers:!0,lineWrapping:!0,smartIndent:!0,indentWithTabs:!1,tabSize:4,indentUnit:4,foldGutter:!0,gutters:["CodeMirror-linenumbers","CodeMirror-foldgutter"],autoCloseTags:!0,matchBrackets:!0,autoCloseBrackets:!0,extraKeys:{Tab:x}},onBeforeChange:(e,t,n)=>{this.setState(e=>Object.assign(Object.assign({},e),{traceConfig:Object.assign(Object.assign({},e.traceConfig),{pythonCode:n})}))},onChange:(e,t,n)=>{}})),y.default.createElement(c.default,{className:this.props.classes.formControl},y.default.createElement(g.default,{value:this.state.traceConfig.widgetName,onChange:this.handleWidgetNameChange,label:"Widget name"}))),y.default.createElement(o.default,null,y.default.createElement(r.default,{autoFocus:!0,onClick:this.toggleSetting,color:"primary"},"Close"),y.default.createElement(r.default,{autoFocus:!0,onClick:this.updateSetting,color:"primary"},"Save"))),y.default.createElement(i.default,{open:this.state.openEdit,"aria-labelledby":"draggable-dialog-title",fullWidth:!0,maxWidth:"sm",PaperComponent:I},y.default.createElement(u.default,{style:{cursor:"move"},className:"draggable-dialog-title"},"Widget viewer selector"),y.default.createElement(a.default,null,y.default.createElement(c.default,{style:{width:"100%"}},y.default.createElement(d.default,{id:"plot-type-select-id"},"Select widget"),y.default.createElement(p.default,{labelId:"plot-type-select-id",value:this.state.selectedWidget,onChange:this.handleSelectedWidgetChange},Object.entries(this.props.availableWidget).map(([e,t])=>y.default.createElement(h.default,{key:e,value:e},t.widgetName))))),y.default.createElement(o.default,null,y.default.createElement(r.default,{autoFocus:!0,onClick:this.toggleEdit,color:"primary"},"Close"),y.default.createElement(r.default,{autoFocus:!0,onClick:this.updateWidgetSelect,color:"primary"},"Select"))),y.default.createElement("div",{id:this.divID,className:"WidgetViewerMain",style:{width:"100%",height:"calc(100% - 35px)",overflow:"auto",boxSizing:"border-box",padding:"10px",background:"white"}}),y.default.createElement("div",{className:this.props.classes.bottomBarDiv},y.default.createElement(r.default,{onClick:this.toggleEdit,style:{color:"rgb(50,50,50)"}}," ",y.default.createElement(m.default,null)),this.props.systemConfig.enableEdit?y.default.createElement(r.default,{onClick:this.toggleSetting,style:{color:"rgb(50,50,50)"}}," ",y.default.createElement(v.default,null)):y.default.createElement("div",null)))}}t.WidgetViewer=w,t.default=A.connect(e=>(e=>({systemConfig:e.systemConfig}))(e),e=>({}),null,{forwardRef:!0})(n.withStyles(e=>({textSizeSmall:{},toolbarHeigt:{minHeight:36,background:"rgb(50, 50, 50)"},viewSelector:{minWidth:120,color:"rgb(250, 250, 250)"},bgSelector:{minWidth:60,color:"rgb(250, 250, 250)"},textColor:{color:"rgb(250, 250, 250)"},backGround:{color:"rgb(50, 50, 50)"},formControl:{padding:"10px 5% 0px 5%;",width:"90%"},formControlShort:{padding:e.spacing(1),width:"calc(50% - 16px)"},formControlTiny:{width:"22%"},selectEmpty:{marginTop:e.spacing(2)},textField:{marginLeft:e.spacing(1),marginRight:e.spacing(1),width:"90%"},bottomBarDiv:{width:"100%",height:"35px",background:"#e0e0e0",display:"flex",flexDirection:"row-reverse"}}))(y.forwardRef((e,t)=>y.default.createElement(w,Object.assign({},e,{ref:t})))))}.apply(t,r))||(e.exports=i)},function(e,t,n){var r,i,o=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),a=this&&this.__setModuleDefault||(Object.create?function(e,t){Object.defineProperty(e,"default",{enumerable:!0,value:t})}:function(e,t){e.default=t}),s=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)"default"!==n&&Object.hasOwnProperty.call(e,n)&&o(t,e,n);return a(t,e),t};r=[n,t,n(90),n(0),n(78),n(176)],void 0===(i=function(e,t,n,r,i,o){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.ElementWrapper=void 0,r=s(r),o=s(o);class a extends r.Component{constructor(e){super(e),this.on_msg=(e,t)=>{const{type:n,payload:r}=e;n.includes("::update_signal")&&this.computedUpdate(r)},this.componentWillUnmount=()=>{this.props.model.stopListening(this.props.model,"msg:custom",this.on_msg)},this.computedUpdate=e=>{const t=e.computed_data,n=e.recorder_data,r=e.driver_data;this.props.updateComputedResult({variable:t,recorder:n,driver:r})},e.model.listenTo(e.model,"msg:custom",this.on_msg)}render(){return r.default.createElement(this.props.CompClass,{ref:null,id:0,initialState:null,send_msg:this.props.send_msg,model:this.props.model})}}t.ElementWrapper=a,t.default=n.withStyles(e=>({}))(i.connect(e=>(e=>({computingState:e.dashboardState.computing,selectedVariable:e.dashboardState.selectedVariable,systemConfig:e.systemConfig}))(e),e=>({updateComputedResult:t=>e(o.dashboardUpdateComputedResult(t))}))(a))}.apply(t,r))||(e.exports=i)},function(e,t,n){"use strict";function r(e){return function(t){var n=t.dispatch,r=t.getState;return function(t){return function(i){return"function"==typeof i?i(n,r,e):t(i)}}}}n.r(t);var i=r();i.withExtraArgument=r,t.default=i},function(e){e.exports=JSON.parse('{"name":"cosapp_lab","version":"0.15.3","description":"Toolbox for managing and deploying CoSApp powered dashboards.","keywords":["jupyter","jupyterlab","jupyterlab-extension","widgets"],"files":["lib/**/*.{d.ts,eot,gif,html,jpg,js,js.map,json,png,svg,woff2,ttf}","style/**/*.{css,eot,gif,html,jpg,json,png,svg,woff2,ttf}"],"main":"lib/index.js","homepage":"https://gitlab.com/cosapp/cosapp_lab","bugs":{"url":"https://gitlab.com/cosapp/cosapp_lab/-/issues"},"license":"Proprietary License","author":{"name":"CoSApp Team","email":"duc-trung.le@safrangroup.com"},"scripts":{"build":"tsc  && webpack --env MODE=prod && jlpm run build:labextension","build:labextension":"jupyter labextension build .","build:labextension:dev":"jupyter labextension build --development True .","build:app":"tsc  && webpack --env TARGET=app","build:lab":"tsc  && webpack --env TARGET=lab && jlpm build:labextension:dev","build:all":"tsc  && webpack && jlpm build:labextension:dev","build:webpack":"webpack --env TARGET=lab && jlpm build:labextension:dev","build:docs":"sphinx-build -b html -d ./docs/_build/doctrees ./docs ./docs/_build","clean:all":"rimraf buildjs/* && rimraf lib/* && rimraf cosapp_lab/lab_static/*  && rimraf cosapp_lab/app_static/static/dist/* &&  copyfiles ./package.json ./buildjs","clean:lib":"rimraf buildjs/* && rimraf lib/* && rimraf cosapp_lab/lab_static/*","clean:labextension":"rimraf cosapp_lab/lab_static/static/lib_index_* && rimraf cosapp_lab/lab_static/static/remoteEntry*","eslint":"eslint . --ext .ts,.tsx --fix","eslint:check":"eslint . --ext .ts,.tsx","install:extension":"jupyter labextension develop --overwrite .","copy:extension":"python ./cosapp_lab/utils/copy_extension.py","watch":"run-p watch:src watch:labextension","watch:src":"tsc -w","watch:lab":"jupyter labextension watch .","test":"jest src --silent "},"dependencies":{"@antv/g6":"^3.5.7","@emotion/core":"^10.0.27","@emotion/styled":"^10.0.27","@jupyter-widgets/base":"^4.0.0","@jupyter-widgets/controls":"^3.0.0","@jupyter-widgets/html-manager":"^0.20.0","@jupyterlab/application":"^3.0.0","@jupyterlab/apputils":"^3.0.0","@jupyterlab/coreutils":"^5.0.0","@jupyterlab/mathjax2":"^3.0.7","@jupyterlab/notebook":"^3.0.0","@jupyterlab/services":"^6.0.0","@lumino/widgets":"^1.3.0","@material-ui/core":"^4.9.3","@material-ui/icons":"^4.5.1","@material-ui/lab":"^4.0.0-alpha.33","@projectstorm/react-diagrams":"^6.2.0","@types/enzyme-adapter-react-16":"^1.0.6","@typescript-eslint/eslint-plugin":"^4.14.2","@typescript-eslint/parser":"^4.14.2","closest":"^0.0.1","codemirror":"^5.59.4","dagre":"^0.8.4","flexlayout-react":"^0.5.4","global":"^4.4.0","ify-loader":"^1.1.0","lodash":"^4.17.15","material-table":"1.62.0","mathjs":"^6.2.5","ml-matrix":"^6.5.0","pathfinding":"^0.4.18","paths-js":"^0.4.10","plotly.js":"^1.57.1","react-codemirror2":"^7.2.1","react-dnd":"^7.3.0","react-dnd-html5-backend":"^7.4.4","react-draggable":"^4.4.3","react-plotly.js":"^2.5.0","react-redux":"^7.1.3","react-scripts":"3.2.0","react-select":"^3.0.8","react-sortable-tree":"2.7.1","react-table":"^7.6.0","redux":"^4.0.4","redux-thunk":"^2.3.0","resize-observer-polyfill":"^1.5.1","three":"^0.117.0","vis":"^4.21.0-EOL"},"resolutions":{"lodash":"4.17.15","react-sortable-tree":"2.7.1"},"devDependencies":{"@babel/core":"7","@babel/preset-env":"7","@jupyterlab/builder":"^3.0.0","@testing-library/react":"^9.4.1","@types/enzyme":"^3.10.5","@types/jest":"24","@types/node":"^10.11.6","@types/react":"16.9.11","@types/react-dom":"16.9.4","@types/react-redux":"^7.1.5","@types/react-sortable-tree":"^0.3.6","@wojtekmaj/enzyme-adapter-react-17":"^0.4.1","copyfiles":"^2.4.0","enzyme":"^3.11.0","eslint":"^7.5.0","eslint-config-prettier":"^6.10.1","eslint-plugin-prettier":"^3.1.2","fs-extra":"^7.0.0","jest":"24","jest-canvas-mock":"^2.3.1","mkdirp":"^0.5.1","npm-run-all":"^4.1.5","prettier":"^1.19.0","prettier-eslint":"^12.0.0","re-resizable":"^6.3.2","redux-mock-store":"^1.5.4","rimraf":"^2.6.2","source-map-loader":"^0.2.4","ts-jest":"24","ts-loader":"^5.2.1","typescript":"^3.8.3","webpack-cli":"^4.5.0"},"sideEffects":["style/*.css"],"jupyterlab":{"extension":true,"outputDir":"cosapp_lab/lab_static","sharedPackages":{"@jupyter-widgets/base":{"bundled":false,"singleton":true},"@jupyterlab/mathjax2":{"bundled":false,"singleton":true},"react":{"bundled":false,"singleton":true},"codemirror":{"bundled":false,"singleton":true}},"discovery":{"server":{"base":{"name":"cosapp_lab"},"managers":["conda"]}}}}')},function(e,t,n){(e.exports=n(68)(!1)).push([e.i,".SysExp {\n  width: 100%;\n  height: 100%;\n  display: \"flex\";\n}\n\n.cosapp-explorer > div {\n  height: 100%;\n}\n\n.MuiPaper-root.MuiDrawer-paper.MuiDrawer-paperAnchorLeft.MuiDrawer-paperAnchorDockedLeft {\n  /* Only for jupyter lab */\n  position: absolute !important;\n}\n\n.MainWindow {\n  height: calc(100% - 48px);\n  margin-top: 48px;\n  flex: 1 1 auto;\n}\n\n.hideDiv {\n  display: none !important;\n}\n\n.hideByPos {\n  position: absolute !important;\n  top: -9999px !important;\n  right: -9999px !important;\n}\n\n.showDiv {\n  width: 100%;\n  height: 100%;\n  padding: 8px 8px 0px 8px;\n}\n\ncanvas {\n  width: 100%;\n  height: 100%;\n}\n\n.cosapp-geometry-viewer > div.lm-Widget {\n  height: 100%;\n}\n\n.graphView {\n  height: 100%;\n\n  background-color: rgb(60, 60, 60) !important;\n  background-size: 50px 50px;\n  display: flex;\n  background-image: linear-gradient(\n      0deg,\n      transparent 24%,\n      rgba(255, 255, 255, 0.05) 25%,\n      rgba(255, 255, 255, 0.05) 26%,\n      transparent 27%,\n      transparent 74%,\n      rgba(255, 255, 255, 0.05) 75%,\n      rgba(255, 255, 255, 0.05) 76%,\n      transparent 77%,\n      transparent\n    ),\n    linear-gradient(\n      90deg,\n      transparent 24%,\n      rgba(255, 255, 255, 0.05) 25%,\n      rgba(255, 255, 255, 0.05) 26%,\n      transparent 27%,\n      transparent 74%,\n      rgba(255, 255, 255, 0.05) 75%,\n      rgba(255, 255, 255, 0.05) 76%,\n      transparent 77%,\n      transparent\n    );\n}\n\n.graphView > * {\n  height: 100%;\n  min-height: 100%;\n  width: 100%;\n}\n\n.pbsView {\n  background-color: rgb(60, 60, 60) !important;\n  background-size: 50px 50px;\n  background-image: linear-gradient(\n      0deg,\n      transparent 24%,\n      rgba(255, 255, 255, 0.05) 25%,\n      rgba(255, 255, 255, 0.05) 26%,\n      transparent 27%,\n      transparent 74%,\n      rgba(255, 255, 255, 0.05) 75%,\n      rgba(255, 255, 255, 0.05) 76%,\n      transparent 77%,\n      transparent\n    ),\n    linear-gradient(\n      90deg,\n      transparent 24%,\n      rgba(255, 255, 255, 0.05) 25%,\n      rgba(255, 255, 255, 0.05) 26%,\n      transparent 27%,\n      transparent 74%,\n      rgba(255, 255, 255, 0.05) 75%,\n      rgba(255, 255, 255, 0.05) 76%,\n      transparent 77%,\n      transparent\n    );\n}\n\n.S_Port {\n  width: 15px;\n  height: 15px;\n  background: rgba(192, 255, 0, 0.5);\n}\n\n.S_Port:hover {\n  background: rgba(192, 255, 0, 1);\n}\n\n.S_OutPort {\n  width: 15px;\n  height: 15px;\n  background: rgba(251, 44, 44, 0.5);\n}\n\n.S_OutPort:hover {\n  background: rgba(251, 44, 44, 1);\n}\n\n.chartviewer__flexlayout__layout {\n  height: calc(100% - 36px) !important;\n}\n\n\ndiv.outer__flexlayout__tabset-selected {\n  background-color: #3f51b5 !important;\n}\n\n\ndiv.inner__flexlayout__tabset-selected {\n  background-color: #cce8c4 !important;\n}\ndiv.inner__flexlayout__tabset {\n  background-color: rgb(250, 250, 250) ;\n}\n\ndiv.inner__flexlayout__tabset > div.flexlayout__tabset_tabbar_outer_top{\n  margin-left: 5px;\n  margin-right: 5px;\n  box-shadow: 0 3px 6px rgba(0,0,0,0.16), 0 3px 6px rgba(0,0,0,0.23);\n  border-width: 1px 0px 0px 0px;\n  border-style: solid;\n  border-color: #e2e2e2;\n}\n\ndiv.inner__flexlayout__tab {\n  background: rgb(250, 250, 250);\n}\n\n.MuiBackdrop-root {\n  background-color: rgb(0 0 0 / 8%)!important;\n}\n\n.flexlayout__splitter {\n  background-color: rgb(250, 250, 250)!important;\n}\n\n\n@media (hover: hover) {\n  .flexlayout__splitter:hover {\n      background-color: rgb(175, 175, 175)!important;\n  }\n}\n\n\ndiv.DataViewerMain  div[class^='Component-horizontalScrollContainer']  {\n  height: 2000px;\n}\n\ndiv.react-codemirror2 {\n  height: 100%!important;\n}\ndiv.react-codemirror2>div.CodeMirror {\n  height: 100%!important;\n}\n\ndiv.react-codemirror2 .cm-tab {\n   background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAADAAAAAMCAYAAAAkuj5RAAAAAXNSR0IArs4c6QAAAGFJREFUSMft1LsRQFAQheHPowAKoACx3IgEKtaEHujDjORSgWTH/ZOdnZOcM/sgk/kFFWY0qV8foQwS4MKBCS3qR6ixBJvElOobYAtivseIE120FaowJPN75GMu8j/LfMwNjh4HUpwg4LUAAAAASUVORK5CYII=);\n   background-position: right;\n   background-repeat: no-repeat;\n}\n\n\n\n.cosapp-widget-box {\n  height: calc(100% - 10px);\n  width: calc(100% - 10px);\n  margin: 0px 5px 10px 5px;\n  box-shadow:0 3px 6px rgba(0,0,0,0.16), 0 3px 6px rgba(0,0,0,0.23);\n  border-width: 0px 0px 0px 0px;\n  border-style: solid;\n  border-color: #e2e2e2;\n  border-radius: 0px;\n  outline-width: 0px;\n  outline-style: solid;\n  outline-color: #e2e2e2;\n}\n\n.cosapp-info-widget  tbody tr:nth-child(even){\n  background: #ecebeb;\n}\n\n.cosapp-info-widget  table{\n  border-spacing: 0px;\n}\n\n.cosapp-info-widget  code{\n  background-color: #eeeeee;\n  padding: 1px 5px;\n}\n\n.cosapp-info-widget  thead th{\n  border-bottom:1px solid #bdbdbd;\n}\n.cosapp-info-widget strong {\n  font-weight: bold!important;\n}\n.cosapp-info-widget table {\n  font-size: 12px;\n}\n.cosapp-info-widget td {\n    vertical-align: middle;\n    padding: 0.5em 0.5em;\n    line-height: normal;\n    white-space: normal;\n    max-width: none;\n    border: none;\n}\n",""])},function(e,t,n){var r,i,o=this&&this.__createBinding||(Object.create?function(e,t,n,r){void 0===r&&(r=n),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[n]}})}:function(e,t,n,r){void 0===r&&(r=n),e[r]=t[n]}),a=this&&this.__setModuleDefault||(Object.create?function(e,t){Object.defineProperty(e,"default",{enumerable:!0,value:t})}:function(e,t){e.default=t}),s=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var n in e)"default"!==n&&Object.hasOwnProperty.call(e,n)&&o(t,e,n);return a(t,e),t},l=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};r=[n,t,n(262),n(0),n(22),n(78),n(118),n(1188),n(901),n(620),n(2340),n(1190)],void 0===(i=function(e,t,n,r,i,o,a,u,c,d){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.SysExplorerAppView=void 0,r=s(r),i=l(i),c=l(c);class h extends n.VBoxView{constructor(){super(...arguments),this.class_name="cosapp-chart-viewer"}initialize(e){super.initialize(e)}getStore(){const e=Object.assign({},d.initialState),t=this.model.get("system_data"),n=u.createInitialStore(e,t),r=this.model.get("initial_store");for(const e in r)r.hasOwnProperty(e)&&n.hasOwnProperty(e)&&(n[e]=r[e]);return n.systemConfig.enableEdit=!1,n}setupReact(e,t){const n=a.createStore(d.rootReducer,this.getStore(),u.getEnhancers());return r.createElement(o.Provider,{store:n},r.createElement(c.default,{send_msg:t,model:e}))}render(){const e=this.setupReact(this.model,this.send.bind(this));i.default.render(e,document.getElementById("main"))}}t.SysExplorerAppView=h}.apply(t,r))||(e.exports=i)},function(e,t,n){var r=n(2341);"string"==typeof r&&(r=[[e.i,r,""]]);var i={insert:"head",singleton:!1};n(147)(r,i);r.locals&&(e.exports=r.locals)},function(e,t,n){(e.exports=n(68)(!1)).push([e.i,"/* BASICS */\n\n.CodeMirror {\n  /* Set height, width, borders, and global font properties here */\n  font-family: monospace;\n  height: 300px;\n  color: black;\n  direction: ltr;\n}\n\n/* PADDING */\n\n.CodeMirror-lines {\n  padding: 4px 0; /* Vertical padding around content */\n}\n.CodeMirror pre.CodeMirror-line,\n.CodeMirror pre.CodeMirror-line-like {\n  padding: 0 4px; /* Horizontal padding of content */\n}\n\n.CodeMirror-scrollbar-filler, .CodeMirror-gutter-filler {\n  background-color: white; /* The little square between H and V scrollbars */\n}\n\n/* GUTTER */\n\n.CodeMirror-gutters {\n  border-right: 1px solid #ddd;\n  background-color: #f7f7f7;\n  white-space: nowrap;\n}\n.CodeMirror-linenumbers {}\n.CodeMirror-linenumber {\n  padding: 0 3px 0 5px;\n  min-width: 20px;\n  text-align: right;\n  color: #999;\n  white-space: nowrap;\n}\n\n.CodeMirror-guttermarker { color: black; }\n.CodeMirror-guttermarker-subtle { color: #999; }\n\n/* CURSOR */\n\n.CodeMirror-cursor {\n  border-left: 1px solid black;\n  border-right: none;\n  width: 0;\n}\n/* Shown when moving in bi-directional text */\n.CodeMirror div.CodeMirror-secondarycursor {\n  border-left: 1px solid silver;\n}\n.cm-fat-cursor .CodeMirror-cursor {\n  width: auto;\n  border: 0 !important;\n  background: #7e7;\n}\n.cm-fat-cursor div.CodeMirror-cursors {\n  z-index: 1;\n}\n.cm-fat-cursor-mark {\n  background-color: rgba(20, 255, 20, 0.5);\n  -webkit-animation: blink 1.06s steps(1) infinite;\n  -moz-animation: blink 1.06s steps(1) infinite;\n  animation: blink 1.06s steps(1) infinite;\n}\n.cm-animate-fat-cursor {\n  width: auto;\n  border: 0;\n  -webkit-animation: blink 1.06s steps(1) infinite;\n  -moz-animation: blink 1.06s steps(1) infinite;\n  animation: blink 1.06s steps(1) infinite;\n  background-color: #7e7;\n}\n@-moz-keyframes blink {\n  0% {}\n  50% { background-color: transparent; }\n  100% {}\n}\n@-webkit-keyframes blink {\n  0% {}\n  50% { background-color: transparent; }\n  100% {}\n}\n@keyframes blink {\n  0% {}\n  50% { background-color: transparent; }\n  100% {}\n}\n\n/* Can style cursor different in overwrite (non-insert) mode */\n.CodeMirror-overwrite .CodeMirror-cursor {}\n\n.cm-tab { display: inline-block; text-decoration: inherit; }\n\n.CodeMirror-rulers {\n  position: absolute;\n  left: 0; right: 0; top: -50px; bottom: 0;\n  overflow: hidden;\n}\n.CodeMirror-ruler {\n  border-left: 1px solid #ccc;\n  top: 0; bottom: 0;\n  position: absolute;\n}\n\n/* DEFAULT THEME */\n\n.cm-s-default .cm-header {color: blue;}\n.cm-s-default .cm-quote {color: #090;}\n.cm-negative {color: #d44;}\n.cm-positive {color: #292;}\n.cm-header, .cm-strong {font-weight: bold;}\n.cm-em {font-style: italic;}\n.cm-link {text-decoration: underline;}\n.cm-strikethrough {text-decoration: line-through;}\n\n.cm-s-default .cm-keyword {color: #708;}\n.cm-s-default .cm-atom {color: #219;}\n.cm-s-default .cm-number {color: #164;}\n.cm-s-default .cm-def {color: #00f;}\n.cm-s-default .cm-variable,\n.cm-s-default .cm-punctuation,\n.cm-s-default .cm-property,\n.cm-s-default .cm-operator {}\n.cm-s-default .cm-variable-2 {color: #05a;}\n.cm-s-default .cm-variable-3, .cm-s-default .cm-type {color: #085;}\n.cm-s-default .cm-comment {color: #a50;}\n.cm-s-default .cm-string {color: #a11;}\n.cm-s-default .cm-string-2 {color: #f50;}\n.cm-s-default .cm-meta {color: #555;}\n.cm-s-default .cm-qualifier {color: #555;}\n.cm-s-default .cm-builtin {color: #30a;}\n.cm-s-default .cm-bracket {color: #997;}\n.cm-s-default .cm-tag {color: #170;}\n.cm-s-default .cm-attribute {color: #00c;}\n.cm-s-default .cm-hr {color: #999;}\n.cm-s-default .cm-link {color: #00c;}\n\n.cm-s-default .cm-error {color: #f00;}\n.cm-invalidchar {color: #f00;}\n\n.CodeMirror-composing { border-bottom: 2px solid; }\n\n/* Default styles for common addons */\n\ndiv.CodeMirror span.CodeMirror-matchingbracket {color: #0b0;}\ndiv.CodeMirror span.CodeMirror-nonmatchingbracket {color: #a22;}\n.CodeMirror-matchingtag { background: rgba(255, 150, 0, .3); }\n.CodeMirror-activeline-background {background: #e8f2ff;}\n\n/* STOP */\n\n/* The rest of this file contains styles related to the mechanics of\n   the editor. You probably shouldn't touch them. */\n\n.CodeMirror {\n  position: relative;\n  overflow: hidden;\n  background: white;\n}\n\n.CodeMirror-scroll {\n  overflow: scroll !important; /* Things will break if this is overridden */\n  /* 50px is the magic margin used to hide the element's real scrollbars */\n  /* See overflow: hidden in .CodeMirror */\n  margin-bottom: -50px; margin-right: -50px;\n  padding-bottom: 50px;\n  height: 100%;\n  outline: none; /* Prevent dragging from highlighting the element */\n  position: relative;\n}\n.CodeMirror-sizer {\n  position: relative;\n  border-right: 50px solid transparent;\n}\n\n/* The fake, visible scrollbars. Used to force redraw during scrolling\n   before actual scrolling happens, thus preventing shaking and\n   flickering artifacts. */\n.CodeMirror-vscrollbar, .CodeMirror-hscrollbar, .CodeMirror-scrollbar-filler, .CodeMirror-gutter-filler {\n  position: absolute;\n  z-index: 6;\n  display: none;\n  outline: none;\n}\n.CodeMirror-vscrollbar {\n  right: 0; top: 0;\n  overflow-x: hidden;\n  overflow-y: scroll;\n}\n.CodeMirror-hscrollbar {\n  bottom: 0; left: 0;\n  overflow-y: hidden;\n  overflow-x: scroll;\n}\n.CodeMirror-scrollbar-filler {\n  right: 0; bottom: 0;\n}\n.CodeMirror-gutter-filler {\n  left: 0; bottom: 0;\n}\n\n.CodeMirror-gutters {\n  position: absolute; left: 0; top: 0;\n  min-height: 100%;\n  z-index: 3;\n}\n.CodeMirror-gutter {\n  white-space: normal;\n  height: 100%;\n  display: inline-block;\n  vertical-align: top;\n  margin-bottom: -50px;\n}\n.CodeMirror-gutter-wrapper {\n  position: absolute;\n  z-index: 4;\n  background: none !important;\n  border: none !important;\n}\n.CodeMirror-gutter-background {\n  position: absolute;\n  top: 0; bottom: 0;\n  z-index: 4;\n}\n.CodeMirror-gutter-elt {\n  position: absolute;\n  cursor: default;\n  z-index: 4;\n}\n.CodeMirror-gutter-wrapper ::selection { background-color: transparent }\n.CodeMirror-gutter-wrapper ::-moz-selection { background-color: transparent }\n\n.CodeMirror-lines {\n  cursor: text;\n  min-height: 1px; /* prevents collapsing before first draw */\n}\n.CodeMirror pre.CodeMirror-line,\n.CodeMirror pre.CodeMirror-line-like {\n  /* Reset some styles that the rest of the page might have set */\n  -moz-border-radius: 0; -webkit-border-radius: 0; border-radius: 0;\n  border-width: 0;\n  background: transparent;\n  font-family: inherit;\n  font-size: inherit;\n  margin: 0;\n  white-space: pre;\n  word-wrap: normal;\n  line-height: inherit;\n  color: inherit;\n  z-index: 2;\n  position: relative;\n  overflow: visible;\n  -webkit-tap-highlight-color: transparent;\n  -webkit-font-variant-ligatures: contextual;\n  font-variant-ligatures: contextual;\n}\n.CodeMirror-wrap pre.CodeMirror-line,\n.CodeMirror-wrap pre.CodeMirror-line-like {\n  word-wrap: break-word;\n  white-space: pre-wrap;\n  word-break: normal;\n}\n\n.CodeMirror-linebackground {\n  position: absolute;\n  left: 0; right: 0; top: 0; bottom: 0;\n  z-index: 0;\n}\n\n.CodeMirror-linewidget {\n  position: relative;\n  z-index: 2;\n  padding: 0.1px; /* Force widget margins to stay inside of the container */\n}\n\n.CodeMirror-widget {}\n\n.CodeMirror-rtl pre { direction: rtl; }\n\n.CodeMirror-code {\n  outline: none;\n}\n\n/* Force content-box sizing for the elements where we expect it */\n.CodeMirror-scroll,\n.CodeMirror-sizer,\n.CodeMirror-gutter,\n.CodeMirror-gutters,\n.CodeMirror-linenumber {\n  -moz-box-sizing: content-box;\n  box-sizing: content-box;\n}\n\n.CodeMirror-measure {\n  position: absolute;\n  width: 100%;\n  height: 0;\n  overflow: hidden;\n  visibility: hidden;\n}\n\n.CodeMirror-cursor {\n  position: absolute;\n  pointer-events: none;\n}\n.CodeMirror-measure pre { position: static; }\n\ndiv.CodeMirror-cursors {\n  visibility: hidden;\n  position: relative;\n  z-index: 3;\n}\ndiv.CodeMirror-dragcursors {\n  visibility: visible;\n}\n\n.CodeMirror-focused div.CodeMirror-cursors {\n  visibility: visible;\n}\n\n.CodeMirror-selected { background: #d9d9d9; }\n.CodeMirror-focused .CodeMirror-selected { background: #d7d4f0; }\n.CodeMirror-crosshair { cursor: crosshair; }\n.CodeMirror-line::selection, .CodeMirror-line > span::selection, .CodeMirror-line > span > span::selection { background: #d7d4f0; }\n.CodeMirror-line::-moz-selection, .CodeMirror-line > span::-moz-selection, .CodeMirror-line > span > span::-moz-selection { background: #d7d4f0; }\n\n.cm-searching {\n  background-color: #ffa;\n  background-color: rgba(255, 255, 0, .4);\n}\n\n/* Used to force a border model for a node */\n.cm-force-border { padding-right: .1px; }\n\n@media print {\n  /* Hide the cursor when printing */\n  .CodeMirror div.CodeMirror-cursors {\n    visibility: hidden;\n  }\n}\n\n/* See issue #2901 */\n.cm-tab-wrap-hack:after { content: ''; }\n\n/* Help users use markselection to safely style text background */\nspan.CodeMirror-selectedtext { background: none; }\n",""])},function(e,t,n){var r,i;r=[n,t,n(262),n(17),n(18),n(1189),n(2343)],void 0===(i=function(e,t,n,r,i,o){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.IpyWidgetRenderView=t.IpyWidgetRenderModel=void 0;class a extends n.VBoxModel{defaults(){return Object.assign(Object.assign({},super.defaults()),{_model_name:a.model_name,_model_module:a.model_module,_model_module_version:a.model_module_version,_view_name:a.view_name,_view_module:a.view_module,_view_module_version:a.view_module_version,title:""})}initialize(e,t){super.initialize(e,t),this.widget_manager.display_model(void 0,this,{})}}t.IpyWidgetRenderModel=a,a.model_name="IpyWidgetRenderModel",a.model_module=o.MODULE_NAME,a.model_module_version=o.MODULE_VERSION,a.view_name="IpyWidgetRenderView",a.view_module=o.MODULE_NAME,a.view_module_version=o.MODULE_VERSION;class s extends n.VBoxView{constructor(){super(...arguments),this.class_name="cosapp-ipywidget-render"}initialize(e){super.initialize(e)}render(){super.render();const e=this.pWidget,t=this.model.get("title");e.id=r.UUID.uuid4();const n=document.getElementById(t);n.innerHTML="",i.Widget.attach(e,n)}}t.IpyWidgetRenderView=s}.apply(t,r))||(e.exports=i)},function(e,t,n){var r=n(2344);"string"==typeof r&&(r=[[e.i,r,""]]);var i={insert:"head",singleton:!1};n(147)(r,i);r.locals&&(e.exports=r.locals)},function(e,t,n){(e.exports=n(68)(!1)).push([e.i,".cosapp-sidecar {\n  background: var(--jp-layout-color0);\n  overflow-y: auto;\n}\n\n.jp-SideBar.jp-mod-right > .lm-TabBar-tabCloseIcon {\n  padding: 4px 0px 4px 4px;\n  background-size: 16px;\n  height: 16px;\n  width: 16px;\n  background-image: var(--jp-icon-close);\n  background-position: center;\n  background-repeat: no-repeat;\n  align-self: center;\n}\n\n.jp-SideBar.jp-mod-right > .lm-TabBar-tabCloseIcon:hover {\n  background-image: var(--jp-icon-close-circle);\n}\n",""])},function(e,t,n){var r;void 0===(r=function(e,t){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.remoteLoader=void 0;function n(e,t){const n=t.replace("^","");let r=e,i="index",o=e.indexOf("/");return-1!==o&&"@"==e[0]&&(o=e.indexOf("/",o+1)),-1!==o&&(i=e.substr(o+1),r=e.substr(0,o)),`https://cdn.jsdelivr.net/npm/${r}@${n}/dist/${i}`}t.remoteLoader=function(e,t){const r=window.requirejs,i={paths:{[e]:n(e,t)}};return r.config(i),new Promise((t,n)=>{r([e],t,n)})}}.apply(t,[n,t]))||(e.exports=r)},function(e,t,n){"use strict";n.r(t);var r={};n.r(r),n.d(r,"OutputModel",(function(){return O})),n.d(r,"OutputView",(function(){return k}));var i,o,a,s=n(262),l=n(35),u=(i=function(e,t){return(i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(e,t)},function(e,t){function n(){this.constructor=e}i(e,t),e.prototype=null===t?Object.create(t):(n.prototype=t.prototype,new n)}),c=function(){return(c=Object.assign||function(e){for(var t,n=1,r=arguments.length;n<r;n++)for(var i in t=arguments[n])Object.prototype.hasOwnProperty.call(t,i)&&(e[i]=t[i]);return e}).apply(this,arguments)},d=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return u(t,e),t.prototype.defaults=function(){return c(c({},e.prototype.defaults.call(this)),{_model_name:"OutputModel",_view_name:"OutputView",_model_module:"@jupyter-widgets/output",_view_module:"@jupyter-widgets/output",_model_module_version:"1.0.0",_view_module_version:"1.0.0"})},t}(l.DOMWidgetModel),h=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return u(t,e),t}(l.DOMWidgetView),f=n(18),p=n(6),g=n(16),v=n(119),m=n(1202),y=n(864),b=n(17);class C{constructor(e={}){this.clearNext=!1,this._trusted=!1,this._isDisposed=!1,this._stateChanged=new g.Signal(this),this._changed=new g.Signal(this),this._trusted=!!e.trusted,this.contentFactory=e.contentFactory||C.defaultContentFactory,this.list=new m.a,e.values&&Object(p.each)(e.values,e=>{this._add(e)}),this.list.changed.connect(this._onListChanged,this)}get stateChanged(){return this._stateChanged}get changed(){return this._changed}get length(){return this.list?this.list.length:0}get trusted(){return this._trusted}set trusted(e){if(e===this._trusted)return;const t=this._trusted=e;for(let e=0;e<this.list.length;e++){let n=this.list.get(e);const r=n.toJSON();n.dispose(),n=this._createItem({value:r,trusted:t}),this.list.set(e,n)}}get isDisposed(){return this._isDisposed}dispose(){this.isDisposed||(this._isDisposed=!0,this.list.dispose(),g.Signal.clearData(this))}get(e){return this.list.get(e)}set(e,t){t=b.JSONExt.deepCopy(t),o.normalize(t);const n=this._createItem({value:t,trusted:this._trusted});this.list.set(e,n)}add(e){return this.clearNext&&(this.clear(),this.clearNext=!1),this._add(e)}clear(e=!1){this._lastStream="",e?this.clearNext=!0:(Object(p.each)(this.list,e=>{e.dispose()}),this.list.clear())}fromJSON(e){this.clear(),Object(p.each)(e,e=>{this._add(e)})}toJSON(){return Object(p.toArray)(Object(p.map)(this.list,e=>e.toJSON()))}_add(e){const t=this._trusted;if(e=b.JSONExt.deepCopy(e),o.normalize(e),v.e(e)&&this._lastStream&&e.name===this._lastName&&this.shouldCombine({value:e,lastModel:this.list.get(this.length-1)})){this._lastStream+=e.text,this._lastStream=o.removeOverwrittenChars(this._lastStream),e.text=this._lastStream;const n=this._createItem({value:e,trusted:t}),r=this.length-1;return this.list.get(r).dispose(),this.list.set(r,n),r}v.e(e)&&(e.text=o.removeOverwrittenChars(e.text));const n=this._createItem({value:e,trusted:t});return v.e(e)?(this._lastStream=e.text,this._lastName=e.name):this._lastStream="",this.list.push(n)}shouldCombine(e){return!0}_createItem(e){const t=this.contentFactory.createOutputModel(e);return t.changed.connect(this._onGenericChange,this),t}_onListChanged(e,t){this._changed.emit(t)}_onGenericChange(){this._stateChanged.emit(void 0)}}!function(e){class t{createOutputModel(e){return new y.a(e)}}e.ContentFactory=t,e.defaultContentFactory=new t}(C||(C={})),(a=o||(o={})).normalize=function(e){v.e(e)&&Array.isArray(e.text)&&(e.text=e.text.join("\n"))},a.removeOverwrittenChars=function(e){return function(e){for(e=e.replace(/\r+\n/gm,"\n");e.search(/\r[^$]/g)>-1;){const t=e.match(/^(.*)\r+/m)[1];let n=e.match(/\r+(.*)$/m)[1];n+=t.slice(n.length,t.length),e=e.replace(/\r+.*$/m,"\r").replace(/^.*\r/m,n)}return e}(function(e){let t=e;do{t=(e=t).replace(/[^\n]\x08/gm,"")}while(t.length<e.length);return e}(e))};var A=n(327),_=n(102),I=n(84);class x extends f.Widget{constructor(e){super(),this.outputLengthChanged=new g.Signal(this),this._onIOPub=e=>{const t=this.model,n=e.header.msg_type;let r;const i=(e.content.transient||{}).display_id;let o;switch(n){case"execute_result":case"display_data":case"stream":case"error":r=Object.assign(Object.assign({},e.content),{output_type:n}),t.add(r);break;case"clear_output":{const n=e.content.wait;t.clear(n);break}case"update_display_data":if(r=Object.assign(Object.assign({},e.content),{output_type:"display_data"}),o=this._displayIdMap.get(i),o)for(const e of o)t.set(e,r)}i&&"display_data"===n&&(o=this._displayIdMap.get(i)||[],o.push(t.length-1),this._displayIdMap.set(i,o))},this._onExecuteReply=e=>{const t=this.model,n=e.content;if("ok"!==n.status)return;const r=n&&n.payload;if(!r||!r.length)return;const i=r.filter(e=>"page"===e.source);if(!i.length)return;const o={output_type:"display_data",data:JSON.parse(JSON.stringify(i[0])).data,metadata:{}};t.add(o)},this._minHeightTimeout=null,this._displayIdMap=new Map;const t=this.model=e.model;this.addClass("jp-OutputArea"),this.rendermime=e.rendermime,this.contentFactory=e.contentFactory||x.defaultContentFactory,this.layout=new f.PanelLayout;for(let e=0;e<t.length;e++){const n=t.get(e);this._insertOutput(e,n)}t.changed.connect(this.onModelChanged,this),t.stateChanged.connect(this.onStateChanged,this)}get widgets(){return this.layout.widgets}get future(){return this._future}set future(e){if(this.model.isDisposed)throw Error("Model is disposed");this._future!==e&&(this._future&&this._future.dispose(),this._future=e,this.model.clear(),this.widgets.length&&(this._clear(),this.outputLengthChanged.emit(this.model.length)),e.onIOPub=this._onIOPub,e.onReply=this._onExecuteReply,e.onStdin=t=>{I.KernelMessage.isInputRequestMsg(t)&&this.onInputRequest(t,e)})}dispose(){this._future&&(this._future.dispose(),this._future=null),this._displayIdMap.clear(),super.dispose()}onModelChanged(e,t){switch(t.type){case"add":this._insertOutput(t.newIndex,t.newValues[0]),this.outputLengthChanged.emit(this.model.length);break;case"remove":if(this.widgets.length){if(0===this.model.length)this._clear();else{const e=t.oldIndex;for(let n=0;n<t.oldValues.length&&e<this.widgets.length;++n){const t=this.widgets[e];t.parent=null,t.dispose()}this._moveDisplayIdIndices(e,t.oldValues.length),this._preventHeightChangeJitter()}this.outputLengthChanged.emit(this.model.length)}break;case"set":this._setOutput(t.newIndex,t.newValues[0]),this.outputLengthChanged.emit(this.model.length)}}_moveDisplayIdIndices(e,t){this._displayIdMap.forEach(n=>{const r=e+t;for(let i=n.length-1;i>=0;--i){const o=n[i];o>=e&&o<r?n.splice(i,1):o>=r&&(n[i]-=t)}})}onStateChanged(e){for(let e=0;e<this.model.length;e++)this._setOutput(e,this.model.get(e));this.outputLengthChanged.emit(this.model.length)}_clear(){if(!this.widgets.length)return;const e=this.widgets.length;for(let t=0;t<e;t++){const e=this.widgets[0];e.parent=null,e.dispose()}this._displayIdMap.clear(),this._preventHeightChangeJitter()}_preventHeightChangeJitter(){const e=this.node.getBoundingClientRect();this.node.style.minHeight=e.height+"px",this._minHeightTimeout&&window.clearTimeout(this._minHeightTimeout),this._minHeightTimeout=window.setTimeout(()=>{this.isDisposed||(this.node.style.minHeight="")},50)}onInputRequest(e,t){const n=this.contentFactory,r=e.content.prompt,i=e.content.password,o=new f.Panel;o.addClass("jp-OutputArea-child"),o.addClass("jp-OutputArea-stdin-item");const a=n.createOutputPrompt();a.addClass("jp-OutputArea-prompt"),o.addWidget(a);const s=n.createStdin({prompt:r,password:i,future:t});s.addClass("jp-OutputArea-output"),o.addWidget(s);this.layout.addWidget(o),s.value.then(e=>{this.model.add({output_type:"stream",name:"stdin",text:e+"\n"}),o.dispose()})}_setOutput(e,t){const n=this.layout,r=n.widgets[e],i=r.widgets?r.widgets[1]:r,o=this.rendermime.preferredMimeType(t.data,t.trusted?"any":"ensure");i.renderModel&&S.currentPreferredMimetype.get(i)===o&&x.isIsolated(o,t.metadata)===i instanceof S.IsolatedRenderer?i.renderModel(t):(n.widgets[e].dispose(),this._insertOutput(e,t))}_insertOutput(e,t){let n=this.createOutputItem(t);n?n.toggleClass("jp-OutputArea-executeResult",null!==t.executionCount):n=new f.Widget;this.layout.insertWidget(e,n)}createOutputItem(e){const t=this.createRenderedMimetype(e);if(!t)return null;const n=new f.Panel;n.addClass("jp-OutputArea-child");const r=this.contentFactory.createOutputPrompt();return r.executionCount=e.executionCount,r.addClass("jp-OutputArea-prompt"),n.addWidget(r),t.addClass("jp-OutputArea-output"),n.addWidget(t),n}createRenderedMimetype(e){const t=this.rendermime.preferredMimeType(e.data,e.trusted?"any":"ensure");if(!t)return null;let n=this.rendermime.createRenderer(t);return!0===x.isIsolated(t,e.metadata)&&(n=new S.IsolatedRenderer(n)),S.currentPreferredMimetype.set(n,t),n.renderModel(e).catch(e=>{const t=document.createElement("pre");t.textContent="Javascript Error: "+e.message,n.node.appendChild(t),n.node.className="lm-Widget jp-RenderedText",n.node.setAttribute("data-mime-type","application/vnd.jupyter.stderr")}),n}}!function(e){e.execute=async function(e,t,n,r){var i;let o=!0;r&&Array.isArray(r.tags)&&-1!==r.tags.indexOf("raises-exception")&&(o=!1);const a={code:e,stop_on_error:o},s=null===(i=n.session)||void 0===i?void 0:i.kernel;if(!s)throw new Error("Session has no kernel.");const l=s.requestExecute(a,!1,r);return t.future=l,l.done},e.isIsolated=function(e,t){const n=t[e];return n&&void 0!==n.isolated?!!n.isolated:!!t.isolated};class t{createOutputPrompt(){return new w}createStdin(e){return new M(e)}}e.ContentFactory=t,e.defaultContentFactory=new t}(x||(x={}));class w extends f.Widget{constructor(){super(),this._executionCount=null,this.addClass("jp-OutputPrompt")}get executionCount(){return this._executionCount}set executionCount(e){this._executionCount=e,this.node.textContent=null===e?"":`[${e}]:`}}class M extends f.Widget{constructor(e){super({node:S.createInputWidgetNode(e.prompt,e.password)}),this._promise=new b.PromiseDelegate,this.addClass("jp-Stdin"),this._input=this.node.getElementsByTagName("input")[0],this._input.focus(),this._future=e.future,this._value=e.prompt+" "}get value(){return this._promise.promise.then(()=>this._value)}handleEvent(e){const t=this._input;"keydown"===e.type&&13===e.keyCode&&(this._future.sendInputReply({status:"ok",value:t.value}),"password"===t.type?this._value+=Array(t.value.length+1).join("·"):this._value+=t.value,this._promise.resolve(void 0))}onAfterAttach(e){this._input.addEventListener("keydown",this),this.update()}onUpdateRequest(e){this._input.focus()}onBeforeDetach(e){this._input.removeEventListener("keydown",this)}}var S;!function(e){e.createInputWidgetNode=function(e,t){const n=document.createElement("div"),r=document.createElement("pre");r.className="jp-Stdin-prompt",r.textContent=e;const i=document.createElement("input");return i.className="jp-Stdin-input",t&&(i.type="password"),n.appendChild(r),r.appendChild(i),n};class t extends f.Widget{constructor(e){super({node:document.createElement("iframe")}),this.addClass("jp-mod-isolated"),this._wrapped=e;const t=this.node;t.frameBorder="0",t.scrolling="auto",t.addEventListener("load",()=>{t.contentDocument.open(),t.contentDocument.write(this._wrapped.node.innerHTML),t.contentDocument.close();const e=t.contentDocument.body;t.style.height=e.scrollHeight+"px",t.heightChangeObserver=new A.default(()=>{t.style.height=e.scrollHeight+"px"}),t.heightChangeObserver.observe(e)})}renderModel(e){return this._wrapped.renderModel(e).then(()=>{const e=this.node.contentWindow;e&&e.location.reload()})}}e.IsolatedRenderer=t,e.currentPreferredMimetype=new _.AttachedProperty({name:"preferredMimetype",create:e=>""})}(S||(S={}));var T=n(71),E=n.n(T);n(1430);class O extends d{defaults(){return Object.assign(Object.assign({},super.defaults()),{msg_id:""})}initialize(e,t){super.initialize(e,t),this._outputs=new C({values:e.outputs,trusted:!0})}get outputs(){return this._outputs}}class k extends h{_createElement(e){return this.pWidget=new f.Panel,this.pWidget.node}_setElement(e){if(this.el||e!==this.pWidget.node)throw new Error("Cannot reset the DOM element.");this.el=this.pWidget.node,this.$el=E()(this.pWidget.node)}render(){const e=this.model.widget_manager.renderMime;this._outputView=new x({rendermime:e,model:this.model.outputs}),this.pWidget.insertWidget(0,this._outputView),this.pWidget.addClass("jupyter-widgets"),this.pWidget.addClass("widget-output"),this.update()}}var L=n(865),D=n(866);class P extends f.Widget{constructor(e,t){super(),this.mimeType=e.mimeType,this._manager=t}async renderModel(e){const t=e.data[this.mimeType],n=this._manager.get_model(t.model_id);if(!n)return this.node.textContent="Error creating widget: could not find model",this.addClass("jupyter-widgets"),Promise.resolve();try{let e=await n;await this._manager.display_model(null,e,{el:this.node})}catch(e){console.log("Error displaying widget"),console.log(e),this.node.textContent="Error displaying widget",this.addClass("jupyter-widgets")}}}class N extends l.ManagerBase{constructor(e){super(),this.loader=e&&e.loader,this.renderMime=new L.a({initialFactories:D.f}),this.renderMime.addFactory({safe:!1,mimeTypes:["application/vnd.jupyter.widget-view+json"],createRenderer:e=>new P(e,this)},0)}display_view(e,t,n){return Promise.resolve(t).then(e=>(f.Widget.attach(e.pWidget,n.el),e.on("remove",()=>{console.log("View removed",e)}),e))}_get_comm_info(){return Promise.resolve({})}_create_comm(e,t,n,r,i){return Promise.resolve({on_close:()=>{},on_msg:()=>{},close:()=>{}})}loadClass(e,t,n){return new Promise((e,i)=>{"@jupyter-widgets/base"===t?e(l):"@jupyter-widgets/controls"===t?e(s):"@jupyter-widgets/output"===t?e(r):void 0!==this.loader?e(this.loader(t,n)):i(`Could not load module ${t}@${n}`)}).then(r=>r[e]?r[e]:Promise.reject(`Class ${e} not found in module ${t}@${n}`))}}n(1504),n(1505),n(1516);n.p=window.__jupyter_widgets_assets_path__||n.p;var j=n(1522),R=n(900).v2.state,z=n(900).v2.view;let F=new j;F.compile(R),F.compile(z);let B="https://unpkg.com/";const H=document.getElementsByTagName("script");Array.prototype.forEach.call(H,e=>{B=e.getAttribute("data-jupyter-widgets-cdn")||B});let U=function(e){return new Promise((t,n)=>{let r=window.requirejs;void 0===r?n("Requirejs is needed, please ensure it is loaded on the page."):r(e,t,n)})};function V(e,t){return U([""+e]).catch(n=>{let r=n.requireModules&&n.requireModules[0];if(r){console.log(`Falling back to ${B} for ${e}@${t}`);let n=window.requirejs;if(void 0===n)throw new Error("Requirejs is needed, please ensure it is loaded on the page.");const i={paths:{}};return i.paths[e]=function(e,t){let n=e,r="index",i=e.indexOf("/");return-1!=i&&"@"==e[0]&&(i=e.indexOf("/",i+1)),-1!=i&&(r=e.substr(i+1),n=e.substr(0,i)),`${B}${n}@${t}/dist/${r}`}(e,t),n.undef(r),n.config(i),U([""+e])}})}n.d(t,"version",(function(){return Y})),n.d(t,"generateEmbedScript",(function(){return W})),n.d(t,"HTMLManager",(function(){return N})),n.d(t,"requireLoader",(function(){return V}));const Y=n(1560).version;function W(e,t){return`<img src=${t} class="jupyter-widget">\n<script type="application/vnd.jupyter.widgets-state+json">${JSON.stringify(e)}<\/script>`}},function(e,t,n){"use strict";n.r(t);var r,i,o=n(20),a=n(230),s=n(332),l=function(e,t){o.Mesh.call(this),this.type="LineSegments2",this.geometry=void 0!==e?e:new a.a,this.material=void 0!==t?t:new s.LineMaterial({color:16777215*Math.random()})};l.prototype=Object.assign(Object.create(o.Mesh.prototype),{constructor:l,isLineSegments2:!0,computeLineDistances:(r=new o.Vector3,i=new o.Vector3,function(){for(var e=this.geometry,t=e.attributes.instanceStart,n=e.attributes.instanceEnd,a=new Float32Array(2*t.data.count),s=0,l=0,u=t.data.count;s<u;s++,l+=2)r.fromBufferAttribute(t,s),i.fromBufferAttribute(n,s),a[l]=0===l?0:a[l-1],a[l+1]=a[l]+r.distanceTo(i);var c=new o.InstancedInterleavedBuffer(a,2,1);return e.setAttribute("instanceDistanceStart",new o.InterleavedBufferAttribute(c,1,0)),e.setAttribute("instanceDistanceEnd",new o.InterleavedBufferAttribute(c,1,1)),this}),raycast:function(){var e=new o.Vector4,t=new o.Vector4,n=new o.Vector4,r=new o.Vector3,i=new o.Matrix4,a=new o.Line3,s=new o.Vector3;return function(l,u){null===l.camera&&console.error('LineSegments2: "Raycaster.camera" needs to be set in order to raycast against LineSegments2.');var c=l.ray,d=l.camera,h=d.projectionMatrix,f=this.geometry,p=this.material,g=p.resolution,v=p.linewidth,m=f.attributes.instanceStart,y=f.attributes.instanceEnd;c.at(1,n),n.w=1,n.applyMatrix4(d.matrixWorldInverse),n.applyMatrix4(h),n.multiplyScalar(1/n.w),n.x*=g.x/2,n.y*=g.y/2,n.z=0,r.copy(n);var b=this.matrixWorld;i.multiplyMatrices(d.matrixWorldInverse,b);for(var C=0,A=m.count;C<A;C++){e.fromBufferAttribute(m,C),t.fromBufferAttribute(y,C),e.w=1,t.w=1,e.applyMatrix4(i),t.applyMatrix4(i),e.applyMatrix4(h),t.applyMatrix4(h),e.multiplyScalar(1/e.w),t.multiplyScalar(1/t.w);var _=e.z<-1&&t.z<-1,I=e.z>1&&t.z>1;if(!_&&!I){e.x*=g.x/2,e.y*=g.y/2,t.x*=g.x/2,t.y*=g.y/2,a.start.copy(e),a.start.z=0,a.end.copy(t),a.end.z=0;var x=a.closestPointToPointParameter(r,!0);a.at(x,s);var w=o.MathUtils.lerp(e.z,t.z,x),M=w>=-1&&w<=1,S=r.distanceTo(s)<.5*v;if(M&&S){a.start.fromBufferAttribute(m,C),a.end.fromBufferAttribute(y,C),a.start.applyMatrix4(b),a.end.applyMatrix4(b);var T=new o.Vector3,E=new o.Vector3;c.distanceSqToSegment(a.start,a.end,E,T),u.push({point:E,pointOnLine:T,distance:c.origin.distanceTo(E),object:this,face:null,faceIndex:C,uv:null,uv2:null})}}}}}()});var u=n(638);n.d(t,"Line2",(function(){return c}));var c=function(e,t){l.call(this),this.type="Line2",this.geometry=void 0!==e?e:new u.LineGeometry,this.material=void 0!==t?t:new s.LineMaterial({color:16777215*Math.random()})};c.prototype=Object.assign(Object.create(l.prototype),{constructor:c,isLine2:!0})},function(e,t,n){"use strict";n.d(t,"a",(function(){return a}));var r=n(0),i=n(1203),o=(n(1206),n(640),n(1207),n(1209),n(23),n(128));const a=e=>r.createElement(i.Button,Object.assign({},e,{className:Object(o.a)(e.className,e.minimal?"minimal":"","jp-Button")}))},function(e,t,n){"use strict";function r(e){return e}n.d(t,"a",(function(){return r}))},function(e,t,n){"use strict";n.d(t,"a",(function(){return h}));var r=n(1),i=n(5),o=n(0),a=n.n(o),s=n(4),l=(n(3),n(91)),u=n.n(l),c=n(654);function d(e,t){var n={};return Object.keys(e).forEach((function(r){-1===t.indexOf(r)&&(n[r]=e[r])})),n}function h(e){return function(t){var n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},o=n.name,l=Object(i.a)(n,["name"]);var h,f=o,p="function"==typeof t?function(e){return{root:function(n){return t(Object(r.a)({theme:e},n))}}}:{root:t},g=Object(c.a)(p,Object(r.a)({Component:e,name:o||e.displayName,classNamePrefix:f},l));t.filterProps&&(h=t.filterProps,delete t.filterProps),t.propTypes&&(t.propTypes,delete t.propTypes);var v=a.a.forwardRef((function(t,n){var o=t.children,l=t.className,u=t.clone,c=t.component,f=Object(i.a)(t,["children","className","clone","component"]),p=g(t),v=Object(s.a)(p.root,l),m=f;if(h&&(m=d(m,h)),u)return a.a.cloneElement(o,Object(r.a)({className:Object(s.a)(o.props.className,v)},m));if("function"==typeof o)return o(Object(r.a)({className:v},m));var y=c||e;return a.a.createElement(y,Object(r.a)({ref:n,className:v},m),o)}));return u()(v,e),v}}},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=n.n(o),s=(n(3),n(91)),l=n.n(s),u=n(654),c=n(653),d=n(277);t.a=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};return function(n){var o=t.defaultTheme,s=t.withTheme,h=void 0!==s&&s,f=t.name,p=Object(i.a)(t,["defaultTheme","withTheme","name"]);var g=f,v=Object(u.a)(e,Object(r.a)({defaultTheme:o,Component:n,name:f||n.displayName,classNamePrefix:g},p)),m=a.a.forwardRef((function(e,t){e.classes;var s,l=e.innerRef,u=Object(i.a)(e,["classes","innerRef"]),p=v(Object(r.a)({},n.defaultProps,e)),g=u;return("string"==typeof f||h)&&(s=Object(d.a)()||o,f&&(g=Object(c.a)({theme:s,name:f,props:u})),h&&!g.theme&&(g.theme=s)),a.a.createElement(n,Object(r.a)({ref:l||t,classes:p},g))}));return l()(m,n),m}}},function(e,t,n){"use strict";var r=n(1),i=n(49),o=n(5),a=n(0),s=n(4),l=(n(3),n(341)),u=n(7),c=n(65),d=n(79),h=n(52),f=n(37),p=a.forwardRef((function(e,t){var n=e.children,u=e.classes,p=e.className,g=e.collapsedHeight,v=void 0===g?"0px":g,m=e.component,y=void 0===m?"div":m,b=e.disableStrictModeCompat,C=void 0!==b&&b,A=e.in,_=e.onEnter,I=e.onEntered,x=e.onEntering,w=e.onExit,M=e.onExited,S=e.onExiting,T=e.style,E=e.timeout,O=void 0===E?c.b.standard:E,k=e.TransitionComponent,L=void 0===k?l.a:k,D=Object(o.a)(e,["children","classes","className","collapsedHeight","component","disableStrictModeCompat","in","onEnter","onEntered","onEntering","onExit","onExited","onExiting","style","timeout","TransitionComponent"]),P=Object(h.a)(),N=a.useRef(),j=a.useRef(null),R=a.useRef(),z="number"==typeof v?"".concat(v,"px"):v;a.useEffect((function(){return function(){clearTimeout(N.current)}}),[]);var F=P.unstable_strictMode&&!C,B=a.useRef(null),H=Object(f.a)(t,F?B:void 0),U=function(e){return function(t,n){if(e){var r=F?[B.current,t]:[t,n],o=Object(i.a)(r,2),a=o[0],s=o[1];void 0===s?e(a):e(a,s)}}},V=U((function(e,t){e.style.height=z,_&&_(e,t)})),Y=U((function(e,t){var n=j.current?j.current.clientHeight:0,r=Object(d.a)({style:T,timeout:O},{mode:"enter"}).duration;if("auto"===O){var i=P.transitions.getAutoHeightDuration(n);e.style.transitionDuration="".concat(i,"ms"),R.current=i}else e.style.transitionDuration="string"==typeof r?r:"".concat(r,"ms");e.style.height="".concat(n,"px"),x&&x(e,t)})),W=U((function(e,t){e.style.height="auto",I&&I(e,t)})),G=U((function(e){var t=j.current?j.current.clientHeight:0;e.style.height="".concat(t,"px"),w&&w(e)})),X=U(M),q=U((function(e){var t=j.current?j.current.clientHeight:0,n=Object(d.a)({style:T,timeout:O},{mode:"exit"}).duration;if("auto"===O){var r=P.transitions.getAutoHeightDuration(t);e.style.transitionDuration="".concat(r,"ms"),R.current=r}else e.style.transitionDuration="string"==typeof n?n:"".concat(n,"ms");e.style.height=z,S&&S(e)}));return a.createElement(L,Object(r.a)({in:A,onEnter:V,onEntered:W,onEntering:Y,onExit:G,onExited:X,onExiting:q,addEndListener:function(e,t){var n=F?e:t;"auto"===O&&(N.current=setTimeout(n,R.current||0))},nodeRef:F?B:void 0,timeout:"auto"===O?null:O},D),(function(e,t){return a.createElement(y,Object(r.a)({className:Object(s.a)(u.container,p,{entered:u.entered,exited:!A&&"0px"===z&&u.hidden}[e]),style:Object(r.a)({minHeight:z},T),ref:H},t),a.createElement("div",{className:u.wrapper,ref:j},a.createElement("div",{className:u.wrapperInner},n)))}))}));p.muiSupportAuto=!0,t.a=Object(u.a)((function(e){return{container:{height:0,overflow:"hidden",transition:e.transitions.create("height")},entered:{height:"auto",overflow:"visible"},hidden:{visibility:"hidden"},wrapper:{display:"flex"},wrapperInner:{width:"100%"}}}),{name:"MuiCollapse"})(p)},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(4)),s=n(7),l=n(1310),u=o.forwardRef((function(e,t){var n=e.children,s=e.classes,u=e.className,c=e.invisible,d=void 0!==c&&c,h=e.open,f=e.transitionDuration,p=e.TransitionComponent,g=void 0===p?l.a:p,v=Object(i.a)(e,["children","classes","className","invisible","open","transitionDuration","TransitionComponent"]);return o.createElement(g,Object(r.a)({in:h,timeout:f},v),o.createElement("div",{className:Object(a.a)(s.root,u,d&&s.invisible),"aria-hidden":!0,ref:t},n))}));t.a=Object(s.a)({root:{zIndex:-1,position:"fixed",display:"flex",alignItems:"center",justifyContent:"center",right:0,bottom:0,top:0,left:0,backgroundColor:"rgba(0, 0, 0, 0.5)",WebkitTapHighlightColor:"transparent"},invisible:{backgroundColor:"transparent"}},{name:"MuiBackdrop"})(u)},function(e,t,n){"use strict";var r=n(0),i=n(22),o=(n(3),n(87)),a=n(37);var s="undefined"!=typeof window?r.useLayoutEffect:r.useEffect,l=r.forwardRef((function(e,t){var n=e.children,l=e.container,u=e.disablePortal,c=void 0!==u&&u,d=e.onRendered,h=r.useState(null),f=h[0],p=h[1],g=Object(a.a)(r.isValidElement(n)?n.ref:null,t);return s((function(){c||p(function(e){return e="function"==typeof e?e():e,i.findDOMNode(e)}(l)||document.body)}),[l,c]),s((function(){if(f&&!c)return Object(o.a)(t,f),function(){Object(o.a)(t,null)}}),[t,f,c]),s((function(){d&&(f||c)&&d()}),[d,f,c]),c?r.isValidElement(n)?r.cloneElement(n,{ref:g}):n:f?i.createPortal(n,f):f}));t.a=l},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(4)),s=n(1311),l=n(7),u=o.forwardRef((function(e,t){var n=e.disableUnderline,l=e.classes,u=e.fullWidth,c=void 0!==u&&u,d=e.inputComponent,h=void 0===d?"input":d,f=e.multiline,p=void 0!==f&&f,g=e.type,v=void 0===g?"text":g,m=Object(i.a)(e,["disableUnderline","classes","fullWidth","inputComponent","multiline","type"]);return o.createElement(s.a,Object(r.a)({classes:Object(r.a)({},l,{root:Object(a.a)(l.root,!n&&l.underline),underline:null}),fullWidth:c,inputComponent:h,multiline:p,ref:t,type:v},m))}));u.muiName="Input",t.a=Object(l.a)((function(e){var t="light"===e.palette.type,n=t?"rgba(0, 0, 0, 0.42)":"rgba(255, 255, 255, 0.7)",r=t?"rgba(0, 0, 0, 0.09)":"rgba(255, 255, 255, 0.09)";return{root:{position:"relative",backgroundColor:r,borderTopLeftRadius:e.shape.borderRadius,borderTopRightRadius:e.shape.borderRadius,transition:e.transitions.create("background-color",{duration:e.transitions.duration.shorter,easing:e.transitions.easing.easeOut}),"&:hover":{backgroundColor:t?"rgba(0, 0, 0, 0.13)":"rgba(255, 255, 255, 0.13)","@media (hover: none)":{backgroundColor:r}},"&$focused":{backgroundColor:t?"rgba(0, 0, 0, 0.09)":"rgba(255, 255, 255, 0.09)"},"&$disabled":{backgroundColor:t?"rgba(0, 0, 0, 0.12)":"rgba(255, 255, 255, 0.12)"}},colorSecondary:{"&$underline:after":{borderBottomColor:e.palette.secondary.main}},underline:{"&:after":{borderBottom:"2px solid ".concat(e.palette.primary.main),left:0,bottom:0,content:'""',position:"absolute",right:0,transform:"scaleX(0)",transition:e.transitions.create("transform",{duration:e.transitions.duration.shorter,easing:e.transitions.easing.easeOut}),pointerEvents:"none"},"&$focused:after":{transform:"scaleX(1)"},"&$error:after":{borderBottomColor:e.palette.error.main,transform:"scaleX(1)"},"&:before":{borderBottom:"1px solid ".concat(n),left:0,bottom:0,content:'"\\00a0"',position:"absolute",right:0,transition:e.transitions.create("border-bottom-color",{duration:e.transitions.duration.shorter}),pointerEvents:"none"},"&:hover:before":{borderBottom:"1px solid ".concat(e.palette.text.primary)},"&$disabled:before":{borderBottomStyle:"dotted"}},focused:{},disabled:{},adornedStart:{paddingLeft:12},adornedEnd:{paddingRight:12},error:{},marginDense:{},multiline:{padding:"27px 12px 10px","&$marginDense":{paddingTop:23,paddingBottom:6}},input:{padding:"27px 12px 10px","&:-webkit-autofill":{WebkitBoxShadow:"light"===e.palette.type?null:"0 0 0 100px #266798 inset",WebkitTextFillColor:"light"===e.palette.type?null:"#fff",caretColor:"light"===e.palette.type?null:"#fff",borderTopLeftRadius:"inherit",borderTopRightRadius:"inherit"}},inputMarginDense:{paddingTop:23,paddingBottom:6},inputHiddenLabel:{paddingTop:18,paddingBottom:19,"&$inputMarginDense":{paddingTop:10,paddingBottom:11}},inputMultiline:{padding:0},inputAdornedStart:{paddingLeft:0},inputAdornedEnd:{paddingRight:0}}}),{name:"MuiFilledInput"})(u)},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(105)),s=n(37);function l(e,t){return parseInt(e[t],10)||0}var u="undefined"!=typeof window?o.useLayoutEffect:o.useEffect,c={visibility:"hidden",position:"absolute",overflow:"hidden",height:0,top:0,left:0,transform:"translateZ(0)"},d=o.forwardRef((function(e,t){var n=e.onChange,d=e.rows,h=e.rowsMax,f=e.rowsMin,p=void 0===f?1:f,g=e.style,v=e.value,m=Object(i.a)(e,["onChange","rows","rowsMax","rowsMin","style","value"]),y=d||p,b=o.useRef(null!=v).current,C=o.useRef(null),A=Object(s.a)(t,C),_=o.useRef(null),I=o.useRef(0),x=o.useState({}),w=x[0],M=x[1],S=o.useCallback((function(){var t=C.current,n=window.getComputedStyle(t),r=_.current;r.style.width=n.width,r.value=t.value||e.placeholder||"x","\n"===r.value.slice(-1)&&(r.value+=" ");var i=n["box-sizing"],o=l(n,"padding-bottom")+l(n,"padding-top"),a=l(n,"border-bottom-width")+l(n,"border-top-width"),s=r.scrollHeight-o;r.value="x";var u=r.scrollHeight-o,c=s;y&&(c=Math.max(Number(y)*u,c)),h&&(c=Math.min(Number(h)*u,c));var d=(c=Math.max(c,u))+("border-box"===i?o+a:0),f=Math.abs(c-s)<=1;M((function(e){return I.current<20&&(d>0&&Math.abs((e.outerHeightStyle||0)-d)>1||e.overflow!==f)?(I.current+=1,{overflow:f,outerHeightStyle:d}):e}))}),[h,y,e.placeholder]);o.useEffect((function(){var e=Object(a.a)((function(){I.current=0,S()}));return window.addEventListener("resize",e),function(){e.clear(),window.removeEventListener("resize",e)}}),[S]),u((function(){S()})),o.useEffect((function(){I.current=0}),[v]);return o.createElement(o.Fragment,null,o.createElement("textarea",Object(r.a)({value:v,onChange:function(e){I.current=0,b||S(),n&&n(e)},ref:A,rows:y,style:Object(r.a)({height:w.outerHeightStyle,overflow:w.overflow?"hidden":null},g)},m)),o.createElement("textarea",{"aria-hidden":!0,className:e.className,readOnly:!0,ref:_,tabIndex:-1,style:Object(r.a)({},c,g)}))}));t.a=d},function(e,t,n){"use strict";var r=n(5),i=n(1),o=n(0),a=(n(3),n(4)),s=n(142),l=n(106),u=n(7),c=o.forwardRef((function(e,t){var n=e.children,u=e.classes,c=e.className,d=e.component,h=void 0===d?"p":d,f=(e.disabled,e.error,e.filled,e.focused,e.margin,e.required,e.variant,Object(r.a)(e,["children","classes","className","component","disabled","error","filled","focused","margin","required","variant"])),p=Object(l.a)(),g=Object(s.a)({props:e,muiFormControl:p,states:["variant","margin","disabled","error","filled","focused","required"]});return o.createElement(h,Object(i.a)({className:Object(a.a)(u.root,("filled"===g.variant||"outlined"===g.variant)&&u.contained,c,g.disabled&&u.disabled,g.error&&u.error,g.filled&&u.filled,g.focused&&u.focused,g.required&&u.required,"dense"===g.margin&&u.marginDense),ref:t},f)," "===n?o.createElement("span",{dangerouslySetInnerHTML:{__html:"&#8203;"}}):n)}));t.a=Object(u.a)((function(e){return{root:Object(i.a)({color:e.palette.text.secondary},e.typography.caption,{textAlign:"left",marginTop:3,margin:0,"&$disabled":{color:e.palette.text.disabled},"&$error":{color:e.palette.error.main}}),error:{},disabled:{},marginDense:{marginTop:4},contained:{marginLeft:14,marginRight:14},focused:{},filled:{},required:{}}}),{name:"MuiFormHelperText"})(c)},function(e,t,n){"use strict";n.d(t,"a",(function(){return s}));var r=n(1),i=n(0),o=n(277),a=n(653);function s(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=Object(o.a)(),s=Object(a.a)({theme:n,name:"MuiUseMediaQuery",props:{}});var l="function"==typeof e?e(n):e;l=l.replace(/^@media( ?)/m,"");var u="undefined"!=typeof window&&void 0!==window.matchMedia,c=Object(r.a)({},s,t),d=c.defaultMatches,h=void 0!==d&&d,f=c.matchMedia,p=void 0===f?u?window.matchMedia:null:f,g=c.noSsr,v=void 0!==g&&g,m=c.ssrMatchMedia,y=void 0===m?null:m,b=i.useState((function(){return v&&u?p(l).matches:y?y(l).matches:h})),C=b[0],A=b[1];return i.useEffect((function(){var e=!0;if(u){var t=p(l),n=function(){e&&A(t.matches)};return n(),t.addListener(n),function(){e=!1,t.removeListener(n)}}}),[l,p,u]),C}},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(4)),s=n(7),l=n(143),u=o.forwardRef((function(e,t){var n=e.children,s=e.classes,u=e.className,c=e.component,d=void 0===c?"ul":c,h=e.dense,f=void 0!==h&&h,p=e.disablePadding,g=void 0!==p&&p,v=e.subheader,m=Object(i.a)(e,["children","classes","className","component","dense","disablePadding","subheader"]),y=o.useMemo((function(){return{dense:f}}),[f]);return o.createElement(l.a.Provider,{value:y},o.createElement(d,Object(r.a)({className:Object(a.a)(s.root,u,f&&s.dense,!g&&s.padding,v&&s.subheader),ref:t},m),v,n))}));t.a=Object(s.a)({root:{listStyle:"none",margin:0,padding:0,position:"relative"},padding:{paddingTop:8,paddingBottom:8},dense:{},subheader:{paddingTop:0}},{name:"MuiList"})(u)},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(4)),s=n(7),l=n(231),u=n(141),c=n(37),d=n(143),h=n(22),f="undefined"==typeof window?o.useEffect:o.useLayoutEffect,p=o.forwardRef((function(e,t){var n=e.alignItems,s=void 0===n?"center":n,p=e.autoFocus,g=void 0!==p&&p,v=e.button,m=void 0!==v&&v,y=e.children,b=e.classes,C=e.className,A=e.component,_=e.ContainerComponent,I=void 0===_?"li":_,x=e.ContainerProps,w=(x=void 0===x?{}:x).className,M=Object(i.a)(x,["className"]),S=e.dense,T=void 0!==S&&S,E=e.disabled,O=void 0!==E&&E,k=e.disableGutters,L=void 0!==k&&k,D=e.divider,P=void 0!==D&&D,N=e.focusVisibleClassName,j=e.selected,R=void 0!==j&&j,z=Object(i.a)(e,["alignItems","autoFocus","button","children","classes","className","component","ContainerComponent","ContainerProps","dense","disabled","disableGutters","divider","focusVisibleClassName","selected"]),F=o.useContext(d.a),B={dense:T||F.dense||!1,alignItems:s},H=o.useRef(null);f((function(){g&&H.current&&H.current.focus()}),[g]);var U=o.Children.toArray(y),V=U.length&&Object(u.a)(U[U.length-1],["ListItemSecondaryAction"]),Y=o.useCallback((function(e){H.current=h.findDOMNode(e)}),[]),W=Object(c.a)(Y,t),G=Object(r.a)({className:Object(a.a)(b.root,C,B.dense&&b.dense,!L&&b.gutters,P&&b.divider,O&&b.disabled,m&&b.button,"center"!==s&&b.alignItemsFlexStart,V&&b.secondaryAction,R&&b.selected),disabled:O},z),X=A||"li";return m&&(G.component=A||"div",G.focusVisibleClassName=Object(a.a)(b.focusVisible,N),X=l.a),V?(X=G.component||A?X:"div","li"===I&&("li"===X?X="div":"li"===G.component&&(G.component="div")),o.createElement(d.a.Provider,{value:B},o.createElement(I,Object(r.a)({className:Object(a.a)(b.container,w),ref:W},M),o.createElement(X,G,U),U.pop()))):o.createElement(d.a.Provider,{value:B},o.createElement(X,Object(r.a)({ref:W},G),U))}));t.a=Object(s.a)((function(e){return{root:{display:"flex",justifyContent:"flex-start",alignItems:"center",position:"relative",textDecoration:"none",width:"100%",boxSizing:"border-box",textAlign:"left",paddingTop:8,paddingBottom:8,"&$focusVisible":{backgroundColor:e.palette.action.selected},"&$selected, &$selected:hover":{backgroundColor:e.palette.action.selected},"&$disabled":{opacity:.5}},container:{position:"relative"},focusVisible:{},dense:{paddingTop:4,paddingBottom:4},alignItemsFlexStart:{alignItems:"flex-start"},disabled:{},divider:{borderBottom:"1px solid ".concat(e.palette.divider),backgroundClip:"padding-box"},gutters:{paddingLeft:16,paddingRight:16},button:{transition:e.transitions.create("background-color",{duration:e.transitions.duration.shortest}),"&:hover":{textDecoration:"none",backgroundColor:e.palette.action.hover,"@media (hover: none)":{backgroundColor:"transparent"}}},secondaryAction:{paddingRight:48},selected:{}}}),{name:"MuiListItem"})(p)},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(116),n(3),n(22)),s=n(59),l=n(2359),u=n(453),c=n(37);function d(e,t,n){return e===t?e.firstChild:t&&t.nextElementSibling?t.nextElementSibling:n?null:e.firstChild}function h(e,t,n){return e===t?n?e.firstChild:e.lastChild:t&&t.previousElementSibling?t.previousElementSibling:n?null:e.lastChild}function f(e,t){if(void 0===t)return!0;var n=e.innerText;return void 0===n&&(n=e.textContent),0!==(n=n.trim().toLowerCase()).length&&(t.repeating?n[0]===t.keys[0]:0===n.indexOf(t.keys.join("")))}function p(e,t,n,r,i,o){for(var a=!1,s=i(e,t,!!t&&n);s;){if(s===e.firstChild){if(a)return;a=!0}var l=!r&&(s.disabled||"true"===s.getAttribute("aria-disabled"));if(s.hasAttribute("tabindex")&&f(s,o)&&!l)return void s.focus();s=i(e,s,n)}}var g="undefined"==typeof window?o.useEffect:o.useLayoutEffect,v=o.forwardRef((function(e,t){var n=e.actions,v=e.autoFocus,m=void 0!==v&&v,y=e.autoFocusItem,b=void 0!==y&&y,C=e.children,A=e.className,_=e.disabledItemsFocusable,I=void 0!==_&&_,x=e.disableListWrap,w=void 0!==x&&x,M=e.onKeyDown,S=e.variant,T=void 0===S?"selectedMenu":S,E=Object(i.a)(e,["actions","autoFocus","autoFocusItem","children","className","disabledItemsFocusable","disableListWrap","onKeyDown","variant"]),O=o.useRef(null),k=o.useRef({keys:[],repeating:!0,previousKeyMatched:!0,lastTime:null});g((function(){m&&O.current.focus()}),[m]),o.useImperativeHandle(n,(function(){return{adjustStyleForScrollbar:function(e,t){var n=!O.current.style.width;if(e.clientHeight<O.current.clientHeight&&n){var r="".concat(Object(u.a)(!0),"px");O.current.style["rtl"===t.direction?"paddingLeft":"paddingRight"]=r,O.current.style.width="calc(100% + ".concat(r,")")}return O.current}}}),[]);var L=o.useCallback((function(e){O.current=a.findDOMNode(e)}),[]),D=Object(c.a)(L,t),P=-1;o.Children.forEach(C,(function(e,t){o.isValidElement(e)&&(e.props.disabled||("selectedMenu"===T&&e.props.selected||-1===P)&&(P=t))}));var N=o.Children.map(C,(function(e,t){if(t===P){var n={};return b&&(n.autoFocus=!0),void 0===e.props.tabIndex&&"selectedMenu"===T&&(n.tabIndex=0),o.cloneElement(e,n)}return e}));return o.createElement(l.a,Object(r.a)({role:"menu",ref:D,className:A,onKeyDown:function(e){var t=O.current,n=e.key,r=Object(s.a)(t).activeElement;if("ArrowDown"===n)e.preventDefault(),p(t,r,w,I,d);else if("ArrowUp"===n)e.preventDefault(),p(t,r,w,I,h);else if("Home"===n)e.preventDefault(),p(t,null,w,I,d);else if("End"===n)e.preventDefault(),p(t,null,w,I,h);else if(1===n.length){var i=k.current,o=n.toLowerCase(),a=performance.now();i.keys.length>0&&(a-i.lastTime>500?(i.keys=[],i.repeating=!0,i.previousKeyMatched=!0):i.repeating&&o!==i.keys[0]&&(i.repeating=!1)),i.lastTime=a,i.keys.push(o);var l=r&&!i.repeating&&f(r,i);i.previousKeyMatched&&(l||p(t,r,!1,I,d,i))?e.preventDefault():i.previousKeyMatched=!1}M&&M(e)},tabIndex:m?0:-1},E),N)}));t.a=v},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(1287)),s=n(277),l=n(2354),u=n(107),c=n(87),d=n(37);function h(e){return"function"==typeof e?e():e}var f="undefined"!=typeof window?o.useLayoutEffect:o.useEffect,p={},g=o.forwardRef((function(e,t){var n=e.anchorEl,g=e.children,v=e.container,m=e.disablePortal,y=void 0!==m&&m,b=e.keepMounted,C=void 0!==b&&b,A=e.modifiers,_=e.open,I=e.placement,x=void 0===I?"bottom":I,w=e.popperOptions,M=void 0===w?p:w,S=e.popperRef,T=e.style,E=e.transition,O=void 0!==E&&E,k=Object(i.a)(e,["anchorEl","children","container","disablePortal","keepMounted","modifiers","open","placement","popperOptions","popperRef","style","transition"]),L=o.useRef(null),D=Object(d.a)(L,t),P=o.useRef(null),N=Object(d.a)(P,S),j=o.useRef(N);f((function(){j.current=N}),[N]),o.useImperativeHandle(S,(function(){return P.current}),[]);var R=o.useState(!0),z=R[0],F=R[1],B=function(e,t){if("ltr"===(t&&t.direction||"ltr"))return e;switch(e){case"bottom-end":return"bottom-start";case"bottom-start":return"bottom-end";case"top-end":return"top-start";case"top-start":return"top-end";default:return e}}(x,Object(s.a)()),H=o.useState(B),U=H[0],V=H[1];o.useEffect((function(){P.current&&P.current.update()}));var Y=o.useCallback((function(){if(L.current&&n&&_){P.current&&(P.current.destroy(),j.current(null));var e=function(e){V(e.placement)},t=(h(n),new a.a(h(n),L.current,Object(r.a)({placement:B},M,{modifiers:Object(r.a)({},y?{}:{preventOverflow:{boundariesElement:"window"}},A,M.modifiers),onCreate:Object(u.a)(e,M.onCreate),onUpdate:Object(u.a)(e,M.onUpdate)})));j.current(t)}}),[n,y,A,_,B,M]),W=o.useCallback((function(e){Object(c.a)(D,e),Y()}),[D,Y]),G=function(){P.current&&(P.current.destroy(),j.current(null))};if(o.useEffect((function(){return function(){G()}}),[]),o.useEffect((function(){_||O||G()}),[_,O]),!C&&!_&&(!O||z))return null;var X={placement:U};return O&&(X.TransitionProps={in:_,onEnter:function(){F(!1)},onExited:function(){F(!0),G()}}),o.createElement(l.a,{disablePortal:y,container:v},o.createElement("div",Object(r.a)({ref:W,role:"tooltip"},k,{style:Object(r.a)({position:"fixed",top:0,left:0,display:_||!C||O?null:"none"},T)}),"function"==typeof g?g(X):g))}));t.a=g},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(4)),s=n(1311),l=n(21),u=n(7),c=n(52),d=n(15),h=o.forwardRef((function(e,t){e.children;var n=e.classes,s=e.className,u=e.label,h=e.labelWidth,f=e.notched,p=e.style,g=Object(i.a)(e,["children","classes","className","label","labelWidth","notched","style"]),v="rtl"===Object(c.a)().direction?"right":"left";if(void 0!==u)return o.createElement("fieldset",Object(r.a)({"aria-hidden":!0,className:Object(a.a)(n.root,s),ref:t,style:p},g),o.createElement("legend",{className:Object(a.a)(n.legendLabelled,f&&n.legendNotched)},u?o.createElement("span",null,u):o.createElement("span",{dangerouslySetInnerHTML:{__html:"&#8203;"}})));var m=h>0?.75*h+8:.01;return o.createElement("fieldset",Object(r.a)({"aria-hidden":!0,style:Object(r.a)(Object(l.a)({},"padding".concat(Object(d.a)(v)),8),p),className:Object(a.a)(n.root,s),ref:t},g),o.createElement("legend",{className:n.legend,style:{width:f?m:.01}},o.createElement("span",{dangerouslySetInnerHTML:{__html:"&#8203;"}})))})),f=Object(u.a)((function(e){return{root:{position:"absolute",bottom:0,right:0,top:-5,left:0,margin:0,padding:"0 8px",pointerEvents:"none",borderRadius:"inherit",borderStyle:"solid",borderWidth:1,overflow:"hidden"},legend:{textAlign:"left",padding:0,lineHeight:"11px",transition:e.transitions.create("width",{duration:150,easing:e.transitions.easing.easeOut})},legendLabelled:{display:"block",width:"auto",textAlign:"left",padding:0,height:11,fontSize:"0.75em",visibility:"hidden",maxWidth:.01,transition:e.transitions.create("max-width",{duration:50,easing:e.transitions.easing.easeOut}),"& > span":{paddingLeft:5,paddingRight:5,display:"inline-block"}},legendNotched:{maxWidth:1e3,transition:e.transitions.create("max-width",{duration:100,easing:e.transitions.easing.easeOut,delay:50})}}}),{name:"PrivateNotchedOutline"})(h),p=o.forwardRef((function(e,t){var n=e.classes,l=e.fullWidth,u=void 0!==l&&l,c=e.inputComponent,d=void 0===c?"input":c,h=e.label,p=e.labelWidth,g=void 0===p?0:p,v=e.multiline,m=void 0!==v&&v,y=e.notched,b=e.type,C=void 0===b?"text":b,A=Object(i.a)(e,["classes","fullWidth","inputComponent","label","labelWidth","multiline","notched","type"]);return o.createElement(s.a,Object(r.a)({renderSuffix:function(e){return o.createElement(f,{className:n.notchedOutline,label:h,labelWidth:g,notched:void 0!==y?y:Boolean(e.startAdornment||e.filled||e.focused)})},classes:Object(r.a)({},n,{root:Object(a.a)(n.root,n.underline),notchedOutline:null}),fullWidth:u,inputComponent:d,multiline:m,ref:t,type:C},A))}));p.muiName="Input";t.a=Object(u.a)((function(e){var t="light"===e.palette.type?"rgba(0, 0, 0, 0.23)":"rgba(255, 255, 255, 0.23)";return{root:{position:"relative",borderRadius:e.shape.borderRadius,"&:hover $notchedOutline":{borderColor:e.palette.text.primary},"@media (hover: none)":{"&:hover $notchedOutline":{borderColor:t}},"&$focused $notchedOutline":{borderColor:e.palette.primary.main,borderWidth:2},"&$error $notchedOutline":{borderColor:e.palette.error.main},"&$disabled $notchedOutline":{borderColor:e.palette.action.disabled}},colorSecondary:{"&$focused $notchedOutline":{borderColor:e.palette.secondary.main}},focused:{},disabled:{},adornedStart:{paddingLeft:14},adornedEnd:{paddingRight:14},error:{},marginDense:{},multiline:{padding:"18.5px 14px","&$marginDense":{paddingTop:10.5,paddingBottom:10.5}},notchedOutline:{borderColor:t},input:{padding:"18.5px 14px","&:-webkit-autofill":{WebkitBoxShadow:"light"===e.palette.type?null:"0 0 0 100px #266798 inset",WebkitTextFillColor:"light"===e.palette.type?null:"#fff",caretColor:"light"===e.palette.type?null:"#fff",borderRadius:"inherit"}},inputMarginDense:{paddingTop:10.5,paddingBottom:10.5},inputMultiline:{padding:0},inputAdornedStart:{paddingLeft:0},inputAdornedEnd:{paddingRight:0}}}),{name:"MuiOutlinedInput"})(p)},,,,,,,,,,function(e,t,n){"use strict";n.d(t,"a",(function(){return i}));var r=n(1);function i(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},t=e.baseClasses,n=e.newClasses;e.Component;if(!n)return t;var i=Object(r.a)({},t);return Object.keys(n).forEach((function(e){n[e]&&(i[e]="".concat(t[e]," ").concat(n[e]))})),i}},function(e,t,n){"use strict";n.d(t,"a",(function(){return d}));var r=n(1),i=n(99),o=n(80),a=n(0),s=n.n(a),l=n(89),u=n(318),c=n(1308),d=function(){function e(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};Object(i.a)(this,e),this.options=t}return Object(o.a)(e,[{key:"collect",value:function(e){var t=new Map;this.sheetsRegistry=new l.b;var n=Object(c.a)();return s.a.createElement(u.b,Object(r.a)({sheetsManager:t,serverGenerateClassName:n,sheetsRegistry:this.sheetsRegistry},this.options),e)}},{key:"toString",value:function(){return this.sheetsRegistry?this.sheetsRegistry.toString():""}},{key:"getStyleElement",value:function(e){return s.a.createElement("style",Object(r.a)({id:"jss-server-side",key:"jss-server-side",dangerouslySetInnerHTML:{__html:this.toString()}},e))}}]),e}()},function(e,t,n){"use strict";var r=n(1),i=n(0),o=n.n(i),a=(n(3),n(448)),s=n(277),l=n(449);t.a=function(e){var t=e.children,n=e.theme,i=Object(s.a)(),u=o.a.useMemo((function(){var e=null===i?n:function(e,t){return"function"==typeof t?t(e):Object(r.a)({},e,t)}(i,n);return null!=e&&(e[l.a]=null!==i),e}),[n,i]);return o.a.createElement(a.a.Provider,{value:u},t)}},function(e,t,n){"use strict";var r=n(0),i=n(22),o=(n(3),n(59)),a=n(37);t.a=function(e){var t=e.children,n=e.disableAutoFocus,s=void 0!==n&&n,l=e.disableEnforceFocus,u=void 0!==l&&l,c=e.disableRestoreFocus,d=void 0!==c&&c,h=e.getDoc,f=e.isEnabled,p=e.open,g=r.useRef(),v=r.useRef(null),m=r.useRef(null),y=r.useRef(),b=r.useRef(null),C=r.useCallback((function(e){b.current=i.findDOMNode(e)}),[]),A=Object(a.a)(t.ref,C),_=r.useRef();return r.useEffect((function(){_.current=p}),[p]),!_.current&&p&&"undefined"!=typeof window&&(y.current=h().activeElement),r.useEffect((function(){if(p){var e=Object(o.a)(b.current);s||!b.current||b.current.contains(e.activeElement)||(b.current.hasAttribute("tabIndex")||b.current.setAttribute("tabIndex",-1),b.current.focus());var t=function(){null!==b.current&&(e.hasFocus()&&!u&&f()&&!g.current?b.current&&!b.current.contains(e.activeElement)&&b.current.focus():g.current=!1)},n=function(t){!u&&f()&&9===t.keyCode&&e.activeElement===b.current&&(g.current=!0,t.shiftKey?m.current.focus():v.current.focus())};e.addEventListener("focus",t,!0),e.addEventListener("keydown",n,!0);var r=setInterval((function(){t()}),50);return function(){clearInterval(r),e.removeEventListener("focus",t,!0),e.removeEventListener("keydown",n,!0),d||(y.current&&y.current.focus&&y.current.focus(),y.current=null)}}}),[s,u,d,f,p]),r.createElement(r.Fragment,null,r.createElement("div",{tabIndex:0,ref:v,"data-test":"sentinelStart"}),r.cloneElement(t,{ref:A}),r.createElement("div",{tabIndex:0,ref:m,"data-test":"sentinelEnd"}))}},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(4)),s=n(7),l=n(15),u=o.forwardRef((function(e,t){var n=e.classes,s=e.className,u=e.color,c=void 0===u?"default":u,d=e.component,h=void 0===d?"li":d,f=e.disableGutters,p=void 0!==f&&f,g=e.disableSticky,v=void 0!==g&&g,m=e.inset,y=void 0!==m&&m,b=Object(i.a)(e,["classes","className","color","component","disableGutters","disableSticky","inset"]);return o.createElement(h,Object(r.a)({className:Object(a.a)(n.root,s,"default"!==c&&n["color".concat(Object(l.a)(c))],y&&n.inset,!v&&n.sticky,!p&&n.gutters),ref:t},b))}));t.a=Object(s.a)((function(e){return{root:{boxSizing:"border-box",lineHeight:"48px",listStyle:"none",color:e.palette.text.secondary,fontFamily:e.typography.fontFamily,fontWeight:e.typography.fontWeightMedium,fontSize:e.typography.pxToRem(14)},colorPrimary:{color:e.palette.primary.main},colorInherit:{color:"inherit"},gutters:{paddingLeft:16,paddingRight:16},inset:{paddingLeft:72},sticky:{position:"sticky",top:0,zIndex:1,backgroundColor:"inherit"}}}),{name:"MuiListSubheader"})(u)},function(e,t,n){"use strict";var r=n(5),i=n(21),o=n(1),a=n(0),s=(n(3),n(4)),l=n(7),u=n(231),c=n(15),d=a.forwardRef((function(e,t){var n=e.classes,i=e.className,l=e.disabled,d=void 0!==l&&l,h=e.disableFocusRipple,f=void 0!==h&&h,p=e.fullWidth,g=e.icon,v=e.indicator,m=e.label,y=e.onChange,b=e.onClick,C=e.onFocus,A=e.selected,_=e.selectionFollowsFocus,I=e.textColor,x=void 0===I?"inherit":I,w=e.value,M=e.wrapped,S=void 0!==M&&M,T=Object(r.a)(e,["classes","className","disabled","disableFocusRipple","fullWidth","icon","indicator","label","onChange","onClick","onFocus","selected","selectionFollowsFocus","textColor","value","wrapped"]);return a.createElement(u.a,Object(o.a)({focusRipple:!f,className:Object(s.a)(n.root,n["textColor".concat(Object(c.a)(x))],i,d&&n.disabled,A&&n.selected,m&&g&&n.labelIcon,p&&n.fullWidth,S&&n.wrapped),ref:t,role:"tab","aria-selected":A,disabled:d,onClick:function(e){y&&y(e,w),b&&b(e)},onFocus:function(e){_&&!A&&y&&y(e,w),C&&C(e)},tabIndex:A?0:-1},T),a.createElement("span",{className:n.wrapper},g,m),v)}));t.a=Object(l.a)((function(e){var t;return{root:Object(o.a)({},e.typography.button,(t={maxWidth:264,minWidth:72,position:"relative",boxSizing:"border-box",minHeight:48,flexShrink:0,padding:"6px 12px"},Object(i.a)(t,e.breakpoints.up("sm"),{padding:"6px 24px"}),Object(i.a)(t,"overflow","hidden"),Object(i.a)(t,"whiteSpace","normal"),Object(i.a)(t,"textAlign","center"),Object(i.a)(t,e.breakpoints.up("sm"),{minWidth:160}),t)),labelIcon:{minHeight:72,paddingTop:9,"& $wrapper > *:first-child":{marginBottom:6}},textColorInherit:{color:"inherit",opacity:.7,"&$selected":{opacity:1},"&$disabled":{opacity:.5}},textColorPrimary:{color:e.palette.text.secondary,"&$selected":{color:e.palette.primary.main},"&$disabled":{color:e.palette.text.disabled}},textColorSecondary:{color:e.palette.text.secondary,"&$selected":{color:e.palette.secondary.main},"&$disabled":{color:e.palette.text.disabled}},selected:{},disabled:{},fullWidth:{flexShrink:1,flexGrow:1,flexBasis:0,maxWidth:"none"},wrapped:{fontSize:e.typography.pxToRem(12),lineHeight:1.5},wrapper:{display:"inline-flex",alignItems:"center",justifyContent:"center",width:"100%",flexDirection:"column"}}}),{name:"MuiTab"})(d)},function(e,t,n){"use strict";var r=n(1),i=n(5),o=n(0),a=(n(3),n(4)),s=n(338),l=n(337),u=n(7),c=n(231),d=o.createElement(s.a,{fontSize:"small"}),h=o.createElement(l.a,{fontSize:"small"}),f=o.forwardRef((function(e,t){var n=e.classes,s=e.className,l=e.direction,u=e.orientation,f=e.disabled,p=Object(i.a)(e,["classes","className","direction","orientation","disabled"]);return o.createElement(c.a,Object(r.a)({component:"div",className:Object(a.a)(n.root,s,f&&n.disabled,"vertical"===u&&n.vertical),ref:t,role:null,tabIndex:null},p),"left"===l?d:h)}));t.a=Object(u.a)({root:{width:40,flexShrink:0,opacity:.8,"&$disabled":{opacity:0}},vertical:{width:"100%",height:40,"& svg":{transform:"rotate(90deg)"}},disabled:{}},{name:"MuiTabScrollButton"})(f)},function(e,t,n){"use strict";(function(e){n.d(t,"a",(function(){return l}));var r=n(0),i=n(652);function o(e,t){return function(e){if(Array.isArray(e))return e}(e)||function(e,t){if(!(Symbol.iterator in Object(e))&&"[object Arguments]"!==Object.prototype.toString.call(e))return;var n=[],r=!0,i=!1,o=void 0;try{for(var a,s=e[Symbol.iterator]();!(r=(a=s.next()).done)&&(n.push(a.value),!t||n.length!==t);r=!0);}catch(e){i=!0,o=e}finally{try{r||null==s.return||s.return()}finally{if(i)throw o}}return n}(e,t)||function(){throw new TypeError("Invalid attempt to destructure non-iterable instance")}()}function a(e,t){if(null==e)return{};var n,r,i=function(e,t){if(null==e)return{};var n,r,i={},o=Object.keys(e);for(r=0;r<o.length;r++)n=o[r],t.indexOf(n)>=0||(i[n]=e[n]);return i}(e,t);if(Object.getOwnPropertySymbols){var o=Object.getOwnPropertySymbols(e);for(r=0;r<o.length;r++)n=o[r],t.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(e,n)&&(i[n]=e[n])}return i}var s=0,l=Object(r.memo)((function(e){var t=e.children,n=o(function(e){if("manager"in e){return[{dragDropManager:e.manager},!1]}var t=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:c(),n=arguments.length>2?arguments[2]:void 0,r=arguments.length>3?arguments[3]:void 0,o=t;o[u]||(o[u]=Object(i.b)(e,t,n,r));return o[u]}(e.backend,e.context,e.options,e.debugMode),n=!e.context;return[t,n]}(a(e,["children"])),2),l=n[0],d=n[1];return r.useEffect((function(){return d&&s++,function(){d&&(0===--s&&(c()[u]=null))}}),[]),r.createElement(i.a.Provider,{value:l},t)}));l.displayName="DndProvider";var u=Symbol.for("__REACT_DND_CONTEXT_INSTANCE__");function c(){return void 0!==e?e:window}}).call(this,n(61))},function(e,t,n){"use strict";var r,i=n(1),o=n(5),a=n(21),s=n(0),l=(n(116),n(3),n(4)),u=n(105),c=n(202);function d(){if(r)return r;var e=document.createElement("div"),t=document.createElement("div");return t.style.width="10px",t.style.height="1px",e.appendChild(t),e.dir="rtl",e.style.fontSize="14px",e.style.width="4px",e.style.height="1px",e.style.position="absolute",e.style.top="-1000px",e.style.overflow="scroll",document.body.appendChild(e),r="reverse",e.scrollLeft>0?r="default":(e.scrollLeft=1,0===e.scrollLeft&&(r="negative")),document.body.removeChild(e),r}function h(e,t){var n=e.scrollLeft;if("rtl"!==t)return n;switch(d()){case"negative":return e.scrollWidth-e.clientWidth+n;case"reverse":return e.scrollWidth-e.clientWidth-n;default:return n}}function f(e){return(1+Math.sin(Math.PI*e-Math.PI/2))/2}var p={width:99,height:99,position:"absolute",top:-9999,overflow:"scroll"};function g(e){var t=e.onChange,n=Object(o.a)(e,["onChange"]),r=s.useRef(),a=s.useRef(null),l=function(){r.current=a.current.offsetHeight-a.current.clientHeight};return s.useEffect((function(){var e=Object(u.a)((function(){var e=r.current;l(),e!==r.current&&t(r.current)}));return window.addEventListener("resize",e),function(){e.clear(),window.removeEventListener("resize",e)}}),[t]),s.useEffect((function(){l(),t(r.current)}),[t]),s.createElement("div",Object(i.a)({style:p,ref:a},n))}var v=n(7),m=n(15),y=s.forwardRef((function(e,t){var n=e.classes,r=e.className,a=e.color,u=e.orientation,c=Object(o.a)(e,["classes","className","color","orientation"]);return s.createElement("span",Object(i.a)({className:Object(l.a)(n.root,n["color".concat(Object(m.a)(a))],r,"vertical"===u&&n.vertical),ref:t},c))})),b=Object(v.a)((function(e){return{root:{position:"absolute",height:2,bottom:0,width:"100%",transition:e.transitions.create()},colorPrimary:{backgroundColor:e.palette.primary.main},colorSecondary:{backgroundColor:e.palette.secondary.main},vertical:{height:"100%",width:2,right:0}}}),{name:"PrivateTabIndicator"})(y),C=n(2379),A=n(53),_=n(52),I=s.forwardRef((function(e,t){var n=e["aria-label"],r=e["aria-labelledby"],p=e.action,v=e.centered,m=void 0!==v&&v,y=e.children,I=e.classes,x=e.className,w=e.component,M=void 0===w?"div":w,S=e.indicatorColor,T=void 0===S?"secondary":S,E=e.onChange,O=e.orientation,k=void 0===O?"horizontal":O,L=e.ScrollButtonComponent,D=void 0===L?C.a:L,P=e.scrollButtons,N=void 0===P?"auto":P,j=e.selectionFollowsFocus,R=e.TabIndicatorProps,z=void 0===R?{}:R,F=e.TabScrollButtonProps,B=e.textColor,H=void 0===B?"inherit":B,U=e.value,V=e.variant,Y=void 0===V?"standard":V,W=Object(o.a)(e,["aria-label","aria-labelledby","action","centered","children","classes","className","component","indicatorColor","onChange","orientation","ScrollButtonComponent","scrollButtons","selectionFollowsFocus","TabIndicatorProps","TabScrollButtonProps","textColor","value","variant"]),G=Object(_.a)(),X="scrollable"===Y,q="rtl"===G.direction,Z="vertical"===k,Q=Z?"scrollTop":"scrollLeft",K=Z?"top":"left",J=Z?"bottom":"right",$=Z?"clientHeight":"clientWidth",ee=Z?"height":"width";var te=s.useState(!1),ne=te[0],re=te[1],ie=s.useState({}),oe=ie[0],ae=ie[1],se=s.useState({start:!1,end:!1}),le=se[0],ue=se[1],ce=s.useState({overflow:"hidden",marginBottom:null}),de=ce[0],he=ce[1],fe=new Map,pe=s.useRef(null),ge=s.useRef(null),ve=function(){var e,t,n=pe.current;if(n){var r=n.getBoundingClientRect();e={clientWidth:n.clientWidth,scrollLeft:n.scrollLeft,scrollTop:n.scrollTop,scrollLeftNormalized:h(n,G.direction),scrollWidth:n.scrollWidth,top:r.top,bottom:r.bottom,left:r.left,right:r.right}}if(n&&!1!==U){var i=ge.current.children;if(i.length>0){var o=i[fe.get(U)];0,t=o?o.getBoundingClientRect():null}}return{tabsMeta:e,tabMeta:t}},me=Object(A.a)((function(){var e,t=ve(),n=t.tabsMeta,r=t.tabMeta,i=0;if(r&&n)if(Z)i=r.top-n.top+n.scrollTop;else{var o=q?n.scrollLeftNormalized+n.clientWidth-n.scrollWidth:n.scrollLeft;i=r.left-n.left+o}var s=(e={},Object(a.a)(e,K,i),Object(a.a)(e,ee,r?r[ee]:0),e);if(isNaN(oe[K])||isNaN(oe[ee]))ae(s);else{var l=Math.abs(oe[K]-s[K]),u=Math.abs(oe[ee]-s[ee]);(l>=1||u>=1)&&ae(s)}})),ye=function(e){!function(e,t,n){var r=arguments.length>3&&void 0!==arguments[3]?arguments[3]:{},i=arguments.length>4&&void 0!==arguments[4]?arguments[4]:function(){},o=r.ease,a=void 0===o?f:o,s=r.duration,l=void 0===s?300:s,u=null,c=t[e],d=!1,h=function(){d=!0},p=function r(o){if(d)i(new Error("Animation cancelled"));else{null===u&&(u=o);var s=Math.min(1,(o-u)/l);t[e]=a(s)*(n-c)+c,s>=1?requestAnimationFrame((function(){i(null)})):requestAnimationFrame(r)}};c===n?i(new Error("Element already at target position")):requestAnimationFrame(p)}(Q,pe.current,e)},be=function(e){var t=pe.current[Q];Z?t+=e:(t+=e*(q?-1:1),t*=q&&"reverse"===d()?-1:1),ye(t)},Ce=function(){be(-pe.current[$])},Ae=function(){be(pe.current[$])},_e=s.useCallback((function(e){he({overflow:null,marginBottom:-e})}),[]),Ie=Object(A.a)((function(){var e=ve(),t=e.tabsMeta,n=e.tabMeta;if(n&&t)if(n[K]<t[K]){var r=t[Q]+(n[K]-t[K]);ye(r)}else if(n[J]>t[J]){var i=t[Q]+(n[J]-t[J]);ye(i)}})),xe=Object(A.a)((function(){if(X&&"off"!==N){var e,t,n=pe.current,r=n.scrollTop,i=n.scrollHeight,o=n.clientHeight,a=n.scrollWidth,s=n.clientWidth;if(Z)e=r>1,t=r<i-o-1;else{var l=h(pe.current,G.direction);e=q?l<a-s-1:l>1,t=q?l>1:l<a-s-1}e===le.start&&t===le.end||ue({start:e,end:t})}}));s.useEffect((function(){var e=Object(u.a)((function(){me(),xe()})),t=Object(c.a)(pe.current);return t.addEventListener("resize",e),function(){e.clear(),t.removeEventListener("resize",e)}}),[me,xe]);var we=s.useCallback(Object(u.a)((function(){xe()})));s.useEffect((function(){return function(){we.clear()}}),[we]),s.useEffect((function(){re(!0)}),[]),s.useEffect((function(){me(),xe()})),s.useEffect((function(){Ie()}),[Ie,oe]),s.useImperativeHandle(p,(function(){return{updateIndicator:me,updateScrollButtons:xe}}),[me,xe]);var Me=s.createElement(b,Object(i.a)({className:I.indicator,orientation:k,color:T},z,{style:Object(i.a)({},oe,z.style)})),Se=0,Te=s.Children.map(y,(function(e){if(!s.isValidElement(e))return null;var t=void 0===e.props.value?Se:e.props.value;fe.set(t,Se);var n=t===U;return Se+=1,s.cloneElement(e,{fullWidth:"fullWidth"===Y,indicator:n&&!ne&&Me,selected:n,selectionFollowsFocus:j,onChange:E,textColor:H,value:t})})),Ee=function(){var e={};e.scrollbarSizeListener=X?s.createElement(g,{className:I.scrollable,onChange:_e}):null;var t=le.start||le.end,n=X&&("auto"===N&&t||"desktop"===N||"on"===N);return e.scrollButtonStart=n?s.createElement(D,Object(i.a)({orientation:k,direction:q?"right":"left",onClick:Ce,disabled:!le.start,className:Object(l.a)(I.scrollButtons,"on"!==N&&I.scrollButtonsDesktop)},F)):null,e.scrollButtonEnd=n?s.createElement(D,Object(i.a)({orientation:k,direction:q?"left":"right",onClick:Ae,disabled:!le.end,className:Object(l.a)(I.scrollButtons,"on"!==N&&I.scrollButtonsDesktop)},F)):null,e}();return s.createElement(M,Object(i.a)({className:Object(l.a)(I.root,x,Z&&I.vertical),ref:t},W),Ee.scrollButtonStart,Ee.scrollbarSizeListener,s.createElement("div",{className:Object(l.a)(I.scroller,X?I.scrollable:I.fixed),style:de,ref:pe,onScroll:we},s.createElement("div",{"aria-label":n,"aria-labelledby":r,className:Object(l.a)(I.flexContainer,Z&&I.flexContainerVertical,m&&!X&&I.centered),onKeyDown:function(e){var t=e.target;if("tab"===t.getAttribute("role")){var n=null,r="vertical"!==k?"ArrowLeft":"ArrowUp",i="vertical"!==k?"ArrowRight":"ArrowDown";switch("vertical"!==k&&"rtl"===G.direction&&(r="ArrowRight",i="ArrowLeft"),e.key){case r:n=t.previousElementSibling||ge.current.lastChild;break;case i:n=t.nextElementSibling||ge.current.firstChild;break;case"Home":n=ge.current.firstChild;break;case"End":n=ge.current.lastChild}null!==n&&(n.focus(),e.preventDefault())}},ref:ge,role:"tablist"},Te),ne&&Me),Ee.scrollButtonEnd)}));t.a=Object(v.a)((function(e){return{root:{overflow:"hidden",minHeight:48,WebkitOverflowScrolling:"touch",display:"flex"},vertical:{flexDirection:"column"},flexContainer:{display:"flex"},flexContainerVertical:{flexDirection:"column"},centered:{justifyContent:"center"},scroller:{position:"relative",display:"inline-block",flex:"1 1 auto",whiteSpace:"nowrap"},fixed:{overflowX:"hidden",width:"100%"},scrollable:{overflowX:"scroll",scrollbarWidth:"none","&::-webkit-scrollbar":{display:"none"}},scrollButtons:{},scrollButtonsDesktop:Object(a.a)({},e.breakpoints.down("xs"),{display:"none"}),indicator:{}}}),{name:"MuiTabs"})(I)},function(e,t,n){"use strict";var r=n(92),i=n(1),o=n(226),a=n(101),s=(n(3),n(0)),l=n.n(s),u=n(275);function c(e,t){var n=Object.create(null);return e&&s.Children.map(e,(function(e){return e})).forEach((function(e){n[e.key]=function(e){return t&&Object(s.isValidElement)(e)?t(e):e}(e)})),n}function d(e,t,n){return null!=n[t]?n[t]:e.props[t]}function h(e,t,n){var r=c(e.children),i=function(e,t){function n(n){return n in t?t[n]:e[n]}e=e||{},t=t||{};var r,i=Object.create(null),o=[];for(var a in e)a in t?o.length&&(i[a]=o,o=[]):o.push(a);var s={};for(var l in t){if(i[l])for(r=0;r<i[l].length;r++){var u=i[l][r];s[i[l][r]]=n(u)}s[l]=n(l)}for(r=0;r<o.length;r++)s[o[r]]=n(o[r]);return s}(t,r);return Object.keys(i).forEach((function(o){var a=i[o];if(Object(s.isValidElement)(a)){var l=o in t,u=o in r,c=t[o],h=Object(s.isValidElement)(c)&&!c.props.in;!u||l&&!h?u||!l||h?u&&l&&Object(s.isValidElement)(c)&&(i[o]=Object(s.cloneElement)(a,{onExited:n.bind(null,a),in:c.props.in,exit:d(a,"exit",e),enter:d(a,"enter",e)})):i[o]=Object(s.cloneElement)(a,{in:!1}):i[o]=Object(s.cloneElement)(a,{onExited:n.bind(null,a),in:!0,exit:d(a,"exit",e),enter:d(a,"enter",e)})}})),i}var f=Object.values||function(e){return Object.keys(e).map((function(t){return e[t]}))},p=function(e){function t(t,n){var r,i=(r=e.call(this,t,n)||this).handleExited.bind(Object(o.a)(r));return r.state={contextValue:{isMounting:!0},handleExited:i,firstRender:!0},r}Object(a.a)(t,e);var n=t.prototype;return n.componentDidMount=function(){this.mounted=!0,this.setState({contextValue:{isMounting:!1}})},n.componentWillUnmount=function(){this.mounted=!1},t.getDerivedStateFromProps=function(e,t){var n,r,i=t.children,o=t.handleExited;return{children:t.firstRender?(n=e,r=o,c(n.children,(function(e){return Object(s.cloneElement)(e,{onExited:r.bind(null,e),in:!0,appear:d(e,"appear",n),enter:d(e,"enter",n),exit:d(e,"exit",n)})}))):h(e,i,o),firstRender:!1}},n.handleExited=function(e,t){var n=c(this.props.children);e.key in n||(e.props.onExited&&e.props.onExited(t),this.mounted&&this.setState((function(t){var n=Object(i.a)({},t.children);return delete n[e.key],{children:n}})))},n.render=function(){var e=this.props,t=e.component,n=e.childFactory,i=Object(r.a)(e,["component","childFactory"]),o=this.state.contextValue,a=f(this.state.children).map(n);return delete i.appear,delete i.enter,delete i.exit,null===t?l.a.createElement(u.a.Provider,{value:o},a):l.a.createElement(u.a.Provider,{value:o},l.a.createElement(t,i,a))},t}(l.a.Component);p.propTypes={},p.defaultProps={component:"div",childFactory:function(e){return e}};t.a=p}])}));